package RDF::SKOS::STW;

use base 'RDF::SKOS::Redland';

=head1 NAME

RDF::SKOS::STW - SKOS Representation of STW

=head1 SYNOPSIS

   use RDF::SKOS::STW;
   my $stw = new RDF::SKOS::STW;
   # API like RDF::SKOS

=head1 DESCRIPTION

@@@@

=head1 INTERFACE

=head2 Constructor

The construct does not expect any parameters.

=cut

our $model;
sub _initialize {
    use RDF::Redland;
    my $storage = new RDF::Redland::Storage ("hashes", "test", "hash-type='memory'");
    $model   = new RDF::Redland::Model ($storage, "");

    my $parser  = new RDF::Redland::Parser (undef, "application/rdf+xml")
	or die "Failed to find parser\n";
    my $uri = new RDF::Redland::URI ("file:data/stw.rdf");

    $/ = undef;
    my $rdf = <DATA>;
    $parser->parse_string_into_model ($rdf, $uri, $model);
}

sub new {
    my $class = shift;

    _initialize unless $model;
    my $skos = new RDF::SKOS::Redland ($model);
    return bless $skos, $class; # rebless it
}

=pod

=head2 Methods

See L<RDF::SKOS>.

=head1 AUTHOR

Robert Barta, C<< <drrho at cpan.org> >>

=head1 BUGS

Please report any bugs or feature requests to C<bug-rdf-skos at rt.cpan.org>, or through
the web interface at L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=RDF-SKOS>.  I will be notified, and then you'll
automatically be notified of progress on your bug as I make changes.

=head1 COPYRIGHT & LICENSE

Copyright 2009 Robert Barta, all rights reserved.

This program is free software; you can redistribute it and/or modify it under the same terms as Perl
itself.

=cut

our $VERSION = '0.01';

"against all odds";


__DATA__
<?xml version="1.0" encoding="utf-8"?>
<rdf:RDF xmlns:cc="http://creativecommons.org/ns#" xmlns:dcterms="http://purl.org/dc/terms/" xmlns:owl="http://www.w3.org/2002/07/owl#" xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#" xmlns:rdfs="http://www.w3.org/2000/01/rdf-schema#" xmlns:skos="http://www.w3.org/2004/02/skos/core#" xmlns:stw="http://zbw.eu/stw/" xmlns:xsd="http://www.w3.org/2001/XMLSchema#" xmlns:zbwext="http://zbw.eu/namespaces/skos-extensions/" xml:base="http://zbw.eu/stw/">
  <rdf:Description rdf:about="descriptor/17782-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72180">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.16</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16207-5">
    <skos:broader rdf:resource="thsys/73376"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16207-5">
    <skos:broader rdf:resource="descriptor/15654-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14066-0">
    <skos:prefLabel xml:lang="en">Coconut</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14066-0">
    <skos:prefLabel xml:lang="de">Kokosnuss</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70324">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71097">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.07.02.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18847-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13553-1">
    <skos:altLabel xml:lang="de">Transportschifffahrt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13553-1">
    <skos:altLabel xml:lang="de">Transportschiffahrt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13553-1">
    <skos:altLabel xml:lang="de">Schiffsfrachtverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13553-1">
    <skos:altLabel xml:lang="de">Frachtschiffahrt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14438-3">
    <skos:narrower rdf:resource="descriptor/14439-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14438-3">
    <skos:narrower rdf:resource="descriptor/14436-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14144-6">
    <skos:related rdf:resource="descriptor/19346-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14144-6">
    <skos:related rdf:resource="descriptor/19345-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14144-6">
    <skos:related rdf:resource="descriptor/12953-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14144-6">
    <skos:related rdf:resource="descriptor/18219-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11664-2">
    <skos:related rdf:resource="descriptor/13742-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11664-2">
    <skos:related rdf:resource="descriptor/19321-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19068-6">
    <skos:prefLabel xml:lang="en">Stock</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19068-6">
    <skos:prefLabel xml:lang="de">Bestand</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18764-5">
    <skos:related rdf:resource="descriptor/13960-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18764-5">
    <skos:related rdf:resource="descriptor/15609-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19514-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71078">
    <rdfs:label xml:lang="de">Öffentliches Vermögen und öffentliche Schulden</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18851-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10533-1">
    <skos:prefLabel xml:lang="en">Key sector</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10533-1">
    <skos:prefLabel xml:lang="de">Schlüsselsektor</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11892-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18431-4">
    <skos:related rdf:resource="descriptor/14626-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21094-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id94">
    <rdf:value xml:lang="de">Für gemeinsame kulturelle Identitäten, Mentalitäten, Konventionen u.ä. von Menschen eines breiteren Kulturkreises</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18337-3">
    <skos:broader rdf:resource="thsys/73351"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18337-3">
    <skos:broader rdf:resource="thsys/71019"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18337-3">
    <skos:broader rdf:resource="thsys/70238"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11392-2">
    <zbwext:useInsteadNote rdf:nodeID="id48"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11304-4">
    <skos:prefLabel xml:lang="en">Employers' organization</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11304-4">
    <skos:prefLabel xml:lang="de">Arbeitgeberverband</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18449-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73342">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12372-3">
    <skos:scopeNote xml:lang="de">Nur für die bilanzielle und steuerrechtliche Gewinnermittlung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12246-1">
    <skos:narrower rdf:resource="descriptor/10824-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12246-1">
    <skos:narrower rdf:resource="descriptor/10693-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12246-1">
    <skos:narrower rdf:resource="descriptor/12236-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70854">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11434-5">
    <skos:related rdf:resource="descriptor/11436-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11434-5">
    <skos:related rdf:resource="descriptor/18149-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11434-5">
    <skos:related rdf:resource="descriptor/11426-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19779-1">
    <skos:altLabel xml:lang="de">Redetechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19779-1">
    <skos:altLabel xml:lang="de">Moderationstechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18182-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16743-2">
    <skos:related rdf:resource="descriptor/16740-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16743-2">
    <skos:related rdf:resource="descriptor/10252-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16743-2">
    <skos:related rdf:resource="descriptor/16738-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16743-2">
    <skos:related rdf:resource="descriptor/16737-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18970-2">
    <skos:related rdf:resource="descriptor/19762-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18970-2">
    <skos:related rdf:resource="descriptor/19788-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18970-2">
    <skos:related rdf:resource="descriptor/16146-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18970-2">
    <skos:related rdf:resource="descriptor/18911-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18970-2">
    <skos:related rdf:resource="descriptor/18968-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19509-2">
    <skos:related rdf:resource="descriptor/11328-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19509-2">
    <skos:related rdf:resource="descriptor/11201-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19509-2">
    <skos:related rdf:resource="descriptor/16140-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19509-2">
    <skos:related rdf:resource="descriptor/16438-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19509-2">
    <skos:related rdf:resource="descriptor/11187-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19509-2">
    <skos:related rdf:resource="descriptor/12609-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19509-2">
    <skos:related rdf:resource="descriptor/11305-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13456-6">
    <skos:broader rdf:resource="thsys/70953"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13456-6">
    <skos:broader rdf:resource="thsys/70854"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13456-6">
    <skos:broader rdf:resource="descriptor/12180-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13456-6">
    <skos:broader rdf:resource="descriptor/12668-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18620-1">
    <skos:broader rdf:resource="thsys/70940"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18620-1">
    <skos:broader rdf:resource="thsys/70927"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18620-1">
    <skos:broader rdf:resource="thsys/70394"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18620-1">
    <skos:broader rdf:resource="thsys/70386"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18620-1">
    <skos:broader rdf:resource="descriptor/12670-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10895-2">
    <skos:related rdf:resource="descriptor/10913-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10895-2">
    <skos:related rdf:resource="descriptor/10956-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10895-2">
    <skos:related rdf:resource="descriptor/19565-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10895-2">
    <skos:related rdf:resource="descriptor/10964-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10895-2">
    <skos:related rdf:resource="descriptor/10963-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10895-2">
    <skos:related rdf:resource="descriptor/10942-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10895-2">
    <skos:related rdf:resource="descriptor/10949-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10895-2">
    <skos:related rdf:resource="descriptor/10948-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10895-2">
    <skos:related rdf:resource="descriptor/10891-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10895-2">
    <skos:related rdf:resource="descriptor/10944-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11670-0">
    <skos:broader rdf:resource="thsys/71074"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11670-0">
    <skos:broader rdf:resource="descriptor/11657-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15011-6">
    <skos:prefLabel xml:lang="en">Recycling product</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15011-6">
    <skos:prefLabel xml:lang="de">Sekundärrohstoff</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16525-5">
    <skos:narrower rdf:resource="descriptor/13310-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12607-2">
    <skos:narrower rdf:resource="descriptor/12625-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16460-0">
    <skos:broader rdf:resource="thsys/73363"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16460-0">
    <skos:broader rdf:resource="thsys/70865"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16460-0">
    <skos:broader rdf:resource="descriptor/19720-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10745-0">
    <skos:related rdf:resource="descriptor/10722-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10745-0">
    <skos:related rdf:resource="descriptor/10772-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10745-0">
    <skos:related rdf:resource="descriptor/10771-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10745-0">
    <skos:related rdf:resource="descriptor/10770-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10745-0">
    <skos:related rdf:resource="descriptor/10746-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15700-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12276-6">
    <skos:related rdf:resource="descriptor/12876-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12276-6">
    <skos:related rdf:resource="descriptor/12372-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12276-6">
    <skos:related rdf:resource="descriptor/12270-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12424-3">
    <skos:broader rdf:resource="thsys/71043"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12424-3">
    <skos:broader rdf:resource="descriptor/10011-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12424-3">
    <skos:broader rdf:resource="descriptor/12408-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18541-4">
    <skos:altLabel xml:lang="de">Marktzutritt in Auslandsmärkte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18541-4">
    <skos:altLabel xml:lang="de">Markteintritt in Auslandsmärkte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18541-4">
    <skos:altLabel xml:lang="de">Internationale Markterschließungsstrategie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18541-4">
    <skos:altLabel xml:lang="de">Erschließungsstrategie von Auslandsmärkten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11191-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17791-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10298-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70346">
    <skos:prefLabel xml:lang="de">P.07.02  Metallprodukte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12914-0">
    <skos:altLabel xml:lang="de">Großanlagenbau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18090-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10411-1">
    <skos:broader rdf:resource="thsys/70406"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10411-1">
    <skos:broader rdf:resource="thsys/70209"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24663-4">
    <skos:related rdf:resource="descriptor/16429-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24663-4">
    <skos:related rdf:resource="descriptor/19756-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24663-4">
    <skos:related rdf:resource="descriptor/18142-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24663-4">
    <skos:related rdf:resource="descriptor/21306-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16229-2">
    <skos:prefLabel xml:lang="en">Utopia</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16229-2">
    <skos:prefLabel xml:lang="de">Utopie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19335-2">
    <skos:narrower rdf:resource="descriptor/10761-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19335-2">
    <skos:narrower rdf:resource="descriptor/10751-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19335-2">
    <skos:narrower rdf:resource="descriptor/10736-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19335-2">
    <skos:narrower rdf:resource="descriptor/10735-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18092-0">
    <skos:prefLabel xml:lang="en">Inventory control system</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18092-0">
    <skos:prefLabel xml:lang="de">Warenwirtschaftssystem</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15762-3">
    <skos:prefLabel xml:lang="en">Environment agency</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15762-3">
    <skos:prefLabel xml:lang="de">Umweltbehörde</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10754-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19805-5">
    <skos:broader rdf:resource="thsys/73340"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19805-5">
    <skos:broader rdf:resource="descriptor/12612-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15728-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19358-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18871-4">
    <skos:prefLabel xml:lang="en">Study of economics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18871-4">
    <skos:prefLabel xml:lang="de">Wirtschaftsstudium</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18276-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11088-5">
    <skos:narrower rdf:resource="descriptor/11073-4"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id229">
    <rdfs:seeAlso rdf:resource="descriptor/16510-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14318-6">
    <skos:prefLabel xml:lang="en">Transport container</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14318-6">
    <skos:prefLabel xml:lang="de">Container</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16048-6">
    <skos:prefLabel xml:lang="en">EU economic law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16048-6">
    <skos:prefLabel xml:lang="de">EU-Wirtschaftsrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18030-1">
    <skos:broader rdf:resource="thsys/73345"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18030-1">
    <skos:broader rdf:resource="descriptor/15679-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20314-0">
    <skos:prefLabel xml:lang="en">Product information</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20314-0">
    <skos:prefLabel xml:lang="de">Produktinformation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12722-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15063-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13082-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14021-1">
    <skos:prefLabel xml:lang="en">Construction material</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14021-1">
    <skos:prefLabel xml:lang="de">Baustoff</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11521-3">
    <skos:related rdf:resource="descriptor/21101-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11222-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16541-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11679-3">
    <skos:related rdf:resource="descriptor/11678-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11679-3">
    <skos:related rdf:resource="descriptor/12876-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11679-3">
    <skos:related rdf:resource="descriptor/12880-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11679-3">
    <skos:related rdf:resource="descriptor/12863-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11679-3">
    <skos:related rdf:resource="descriptor/12848-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19612-2">
    <skos:altLabel xml:lang="de">Ländergröße</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15977-3">
    <skos:narrower rdf:resource="descriptor/18790-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18137-4">
    <skos:broader rdf:resource="thsys/71024"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18137-4">
    <skos:broader rdf:resource="descriptor/18313-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10958-4">
    <skos:narrower rdf:resource="descriptor/10961-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10958-4">
    <skos:narrower rdf:resource="descriptor/10957-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18374-4">
    <skos:prefLabel xml:lang="en">Coal policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18374-4">
    <skos:prefLabel xml:lang="de">Kohlepolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19040-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12738-1">
    <skos:related rdf:resource="descriptor/12732-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12738-1">
    <skos:related rdf:resource="descriptor/12730-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13066-5">
    <skos:altLabel xml:lang="de">Werftindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13066-5">
    <skos:altLabel xml:lang="de">Schiffsbau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12436-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10001-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12963-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11066-1">
    <skos:broader rdf:resource="thsys/70039"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11066-1">
    <skos:broader rdf:resource="descriptor/10032-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16745-5">
    <skos:narrower rdf:resource="descriptor/10166-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16745-5">
    <skos:narrower rdf:resource="descriptor/19732-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16745-5">
    <skos:narrower rdf:resource="descriptor/16743-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16745-5">
    <skos:narrower rdf:resource="descriptor/16742-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16745-5">
    <skos:narrower rdf:resource="descriptor/11174-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16745-5">
    <skos:narrower rdf:resource="descriptor/10054-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16745-5">
    <skos:narrower rdf:resource="descriptor/16741-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16745-5">
    <skos:narrower rdf:resource="descriptor/16740-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16745-5">
    <skos:narrower rdf:resource="descriptor/10252-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16745-5">
    <skos:narrower rdf:resource="descriptor/10250-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16745-5">
    <skos:narrower rdf:resource="descriptor/15704-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16745-5">
    <skos:narrower rdf:resource="descriptor/16738-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16745-5">
    <skos:narrower rdf:resource="descriptor/10041-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16745-5">
    <skos:narrower rdf:resource="descriptor/10247-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16745-5">
    <skos:narrower rdf:resource="descriptor/11161-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16745-5">
    <skos:narrower rdf:resource="descriptor/16712-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16745-5">
    <skos:narrower rdf:resource="descriptor/19206-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16745-5">
    <skos:narrower rdf:resource="descriptor/16737-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#topConceptOf">
    <rdfs:isDefinedBy rdf:resource="http://www.w3.org/2004/02/skos/core"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19732-6">
    <skos:altLabel xml:lang="de">Universalökonomie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16308-6">
    <skos:related rdf:resource="descriptor/16421-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16308-6">
    <skos:related rdf:resource="descriptor/16226-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16308-6">
    <skos:related rdf:resource="descriptor/16512-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16308-6">
    <skos:related rdf:resource="descriptor/19773-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16308-6">
    <skos:related rdf:resource="descriptor/16297-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17639-6">
    <skos:prefLabel xml:lang="en">United Arab Emirates</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17639-6">
    <skos:prefLabel xml:lang="de">Vereinigte Arabische Emirate</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13075-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13341-2">
    <skos:prefLabel xml:lang="en">Archive</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13341-2">
    <skos:prefLabel xml:lang="de">Archiv</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18589-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12313-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18860-2">
    <skos:related rdf:resource="descriptor/15861-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18860-2">
    <skos:related rdf:resource="descriptor/15666-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18860-2">
    <skos:related rdf:resource="descriptor/19601-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18860-2">
    <skos:related rdf:resource="descriptor/16759-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18860-2">
    <skos:related rdf:resource="descriptor/11407-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18860-2">
    <skos:related rdf:resource="descriptor/11354-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18860-2">
    <skos:related rdf:resource="descriptor/11394-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18860-2">
    <skos:related rdf:resource="descriptor/11400-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18860-2">
    <skos:related rdf:resource="descriptor/11355-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18860-2">
    <skos:related rdf:resource="descriptor/18856-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14204-0">
    <skos:narrower rdf:resource="descriptor/14190-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73387">
    <skos:prefLabel xml:lang="en">N.02.01  Economic history</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73387">
    <skos:prefLabel xml:lang="de">N.02.01  Wirtschaftsgeschichte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19124-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12901-2">
    <skos:related rdf:resource="descriptor/12903-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12901-2">
    <skos:related rdf:resource="descriptor/19856-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12901-2">
    <skos:related rdf:resource="descriptor/12882-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14436-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18788-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15909-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19801-6">
    <skos:prefLabel xml:lang="en">User interface</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19801-6">
    <skos:prefLabel xml:lang="de">Benutzerschnittstelle</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16602-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19417-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10137-2">
    <skos:broader rdf:resource="thsys/70455"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10137-2">
    <skos:broader rdf:resource="descriptor/10236-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70321">
    <rdfs:label xml:lang="de">Gesellschaft und sozialer Wandel</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16648-3">
    <skos:broader rdf:resource="thsys/73366"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16648-3">
    <skos:broader rdf:resource="thsys/70866"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16648-3">
    <skos:broader rdf:resource="thsys/70828"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16648-3">
    <skos:broader rdf:resource="descriptor/16441-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11324-5">
    <skos:prefLabel xml:lang="en">Wage drift</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11324-5">
    <skos:prefLabel xml:lang="de">Lohndrift</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14989-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14301-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18856-0">
    <skos:related rdf:resource="descriptor/11378-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18856-0">
    <skos:related rdf:resource="descriptor/11375-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18856-0">
    <skos:related rdf:resource="descriptor/11367-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18856-0">
    <skos:related rdf:resource="descriptor/18860-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18856-0">
    <skos:related rdf:resource="descriptor/11288-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18856-0">
    <skos:related rdf:resource="descriptor/11355-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11532-5">
    <skos:prefLabel xml:lang="en">Tax court</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11532-5">
    <skos:prefLabel xml:lang="de">Finanzgericht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14622-3">
    <skos:prefLabel xml:lang="en">Household cleaning agent</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14622-3">
    <skos:prefLabel xml:lang="de">Pflegemittel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18892-3">
    <skos:altLabel xml:lang="de">Prophylaxe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18892-3">
    <skos:altLabel xml:lang="de">Krankheitsprävention</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18892-3">
    <skos:altLabel xml:lang="de">Gesundheitsbewegung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12004-4">
    <skos:prefLabel xml:lang="en">Infrastructure policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12004-4">
    <skos:prefLabel xml:lang="de">Infrastrukturpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70406">
    <skos:broader rdf:resource="thsys/70021"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12046-2">
    <skos:altLabel xml:lang="de">Unternehmensgeschichte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19144-2">
    <skos:prefLabel xml:lang="en">Wide-area networks</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19144-2">
    <skos:prefLabel xml:lang="de">WAN</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12963-1">
    <skos:broader rdf:resource="thsys/71018"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12963-1">
    <skos:broader rdf:resource="descriptor/12964-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18773-4">
    <skos:prefLabel xml:lang="en">European science and technology policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18773-4">
    <skos:prefLabel xml:lang="de">Europäische Forschungs- und Technologiepolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11651-4">
    <skos:related rdf:resource="descriptor/11682-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11651-4">
    <skos:related rdf:resource="descriptor/16029-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11651-4">
    <skos:related rdf:resource="descriptor/16028-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19803-2">
    <skos:prefLabel xml:lang="en">Object-oriented programming</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19803-2">
    <skos:prefLabel xml:lang="de">Objektorientierte Programmierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19615-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11421-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19152-3">
    <skos:prefLabel xml:lang="en">Lean management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19152-3">
    <skos:prefLabel xml:lang="de">Lean Management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12268-5">
    <skos:broader rdf:resource="thsys/70900"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12268-5">
    <skos:broader rdf:resource="descriptor/12282-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18601-5">
    <skos:related rdf:resource="descriptor/18603-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18601-5">
    <skos:related rdf:resource="descriptor/18602-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18601-5">
    <skos:related rdf:resource="descriptor/13536-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18601-5">
    <skos:related rdf:resource="descriptor/13535-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18601-5">
    <skos:related rdf:resource="descriptor/13533-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15023-6">
    <skos:broader rdf:resource="thsys/72183"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15023-6">
    <skos:broader rdf:resource="descriptor/15119-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13555-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19728-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70040">
    <rdfs:label xml:lang="de">Konditionen- und Preispolitik</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10339-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13649-2">
    <skos:broader rdf:resource="thsys/70922"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13649-2">
    <skos:broader rdf:resource="thsys/70135"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13649-2">
    <skos:broader rdf:resource="descriptor/13667-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13649-2">
    <skos:broader rdf:resource="descriptor/18170-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18363-2">
    <skos:altLabel xml:lang="de">Sonnenenergieförderung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id297">
    <rdfs:seeAlso rdf:resource="descriptor/13418-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11427-2">
    <skos:narrower rdf:resource="descriptor/11519-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11427-2">
    <skos:narrower rdf:resource="descriptor/11432-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71014">
    <skos:broader rdf:resource="thsys/71016"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14444-1">
    <skos:prefLabel xml:lang="en">Bicycle and motorcycle</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14444-1">
    <skos:prefLabel xml:lang="de">Zweirad</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10830-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16792-3">
    <skos:narrower rdf:resource="descriptor/18179-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12053-5">
    <skos:related rdf:resource="descriptor/15075-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12053-5">
    <skos:related rdf:resource="descriptor/12073-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12053-5">
    <skos:related rdf:resource="descriptor/12088-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12053-5">
    <skos:related rdf:resource="descriptor/12069-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12053-5">
    <skos:related rdf:resource="descriptor/24856-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12053-5">
    <skos:related rdf:resource="descriptor/12884-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12053-5">
    <skos:related rdf:resource="descriptor/12470-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12053-5">
    <skos:related rdf:resource="descriptor/12216-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12053-5">
    <skos:related rdf:resource="descriptor/19069-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12053-5">
    <skos:related rdf:resource="descriptor/12039-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12053-5">
    <skos:related rdf:resource="descriptor/19108-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12053-5">
    <skos:related rdf:resource="descriptor/12164-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12053-5">
    <skos:related rdf:resource="descriptor/19834-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14705-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70020">
    <rdfs:label xml:lang="en">Business cycle</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70020">
    <rdfs:label xml:lang="de">Konjunktur</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16290-6">
    <skos:narrower rdf:resource="descriptor/11127-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16290-6">
    <skos:narrower rdf:resource="descriptor/16306-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16290-6">
    <skos:narrower rdf:resource="descriptor/16283-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16290-6">
    <skos:narrower rdf:resource="descriptor/16268-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15429-6">
    <skos:prefLabel xml:lang="en">Systems theory</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15429-6">
    <skos:prefLabel xml:lang="de">Systemtheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14121-4">
    <skos:altLabel xml:lang="de">Hühnerei</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14121-4">
    <skos:altLabel xml:lang="de">Geflügelei</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70485">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12087-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19203-5">
    <skos:scopeNote xml:lang="de">Anwendung biologischer Erkenntnisse und Verfahren in der ökonomischen Theorie.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70358">
    <skos:prefLabel xml:lang="de">P.07  Metalle</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16692-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19117-5">
    <skos:broader rdf:resource="thsys/71036"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19117-5">
    <skos:broader rdf:resource="thsys/70471"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19117-5">
    <skos:broader rdf:resource="descriptor/12713-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19117-5">
    <skos:broader rdf:resource="descriptor/18002-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13979-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17433-5">
    <skos:altLabel xml:lang="de">Nederland</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17433-5">
    <skos:altLabel xml:lang="de">Königreich der Niederlande</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17433-5">
    <skos:altLabel xml:lang="de">Holland</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11590-5">
    <skos:related rdf:resource="descriptor/11591-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11590-5">
    <skos:related rdf:resource="descriptor/19378-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12670-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18240-4">
    <skos:prefLabel xml:lang="en">Titanium</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18240-4">
    <skos:prefLabel xml:lang="de">Titan</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14776-4">
    <skos:narrower rdf:resource="descriptor/14779-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14776-4">
    <skos:narrower rdf:resource="descriptor/14778-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14776-4">
    <skos:narrower rdf:resource="descriptor/14777-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14776-4">
    <skos:narrower rdf:resource="descriptor/14775-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12640-4">
    <skos:narrower rdf:resource="descriptor/12573-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12640-4">
    <skos:narrower rdf:resource="descriptor/11579-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12640-4">
    <skos:narrower rdf:resource="descriptor/20503-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12640-4">
    <skos:narrower rdf:resource="descriptor/15473-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12640-4">
    <skos:narrower rdf:resource="descriptor/16767-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12640-4">
    <skos:narrower rdf:resource="descriptor/12668-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12640-4">
    <skos:narrower rdf:resource="descriptor/12656-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12640-4">
    <skos:narrower rdf:resource="descriptor/12651-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18854-4">
    <skos:related rdf:resource="descriptor/18123-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18854-4">
    <skos:related rdf:resource="descriptor/18852-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19550-5">
    <skos:broader rdf:resource="thsys/70307"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19550-5">
    <skos:broader rdf:resource="thsys/70091"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16325-6">
    <skos:broader rdf:resource="thsys/73363"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16325-6">
    <skos:broader rdf:resource="thsys/70865"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16325-6">
    <skos:broader rdf:resource="descriptor/13412-5"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id166">
    <rdfs:seeAlso rdf:resource="descriptor/13004-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11721-2">
    <skos:narrower rdf:resource="descriptor/11724-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="../stw">
    <cc:license rdf:resource="http://creativecommons.org/licenses/by-nc-sa/3.0/de/"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19707-5">
    <skos:altLabel xml:lang="de">Sprachenregelung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19707-5">
    <skos:altLabel xml:lang="de">Sprachenrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19707-5">
    <skos:altLabel xml:lang="de">Sprachenpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19707-5">
    <skos:altLabel xml:lang="de">Sprachengesetzgebung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11636-0">
    <skos:prefLabel xml:lang="en">Tax burden</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11636-0">
    <skos:prefLabel xml:lang="de">Steuerbelastung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70957">
    <skos:narrower rdf:resource="descriptor/19390-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70957">
    <skos:narrower rdf:resource="descriptor/13236-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70957">
    <skos:narrower rdf:resource="descriptor/13190-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70957">
    <skos:narrower rdf:resource="descriptor/18995-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70957">
    <skos:narrower rdf:resource="descriptor/13472-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70957">
    <skos:narrower rdf:resource="descriptor/13193-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70957">
    <skos:narrower rdf:resource="descriptor/19393-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70957">
    <skos:narrower rdf:resource="descriptor/19182-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70957">
    <skos:narrower rdf:resource="descriptor/13221-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70957">
    <skos:narrower rdf:resource="descriptor/19184-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70957">
    <skos:narrower rdf:resource="descriptor/13224-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70957">
    <skos:narrower rdf:resource="descriptor/19185-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70957">
    <skos:narrower rdf:resource="descriptor/19192-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70957">
    <skos:narrower rdf:resource="descriptor/19367-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70957">
    <skos:narrower rdf:resource="descriptor/13215-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70957">
    <skos:narrower rdf:resource="descriptor/13214-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70957">
    <skos:narrower rdf:resource="descriptor/18547-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70957">
    <skos:narrower rdf:resource="descriptor/19181-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70957">
    <skos:narrower rdf:resource="descriptor/19388-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70957">
    <skos:narrower rdf:resource="descriptor/19385-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70957">
    <skos:narrower rdf:resource="descriptor/19397-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70957">
    <skos:narrower rdf:resource="descriptor/13199-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70957">
    <skos:narrower rdf:resource="descriptor/13198-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70957">
    <skos:narrower rdf:resource="descriptor/13194-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70957">
    <skos:narrower rdf:resource="descriptor/19675-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70957">
    <skos:narrower rdf:resource="descriptor/13192-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70957">
    <skos:narrower rdf:resource="descriptor/13191-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12307-0">
    <skos:related rdf:resource="descriptor/12882-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12307-0">
    <skos:related rdf:resource="descriptor/11589-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12307-0">
    <skos:related rdf:resource="descriptor/12848-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18224-2">
    <skos:prefLabel xml:lang="en">Czech Republic</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18224-2">
    <skos:prefLabel xml:lang="de">Tschechische Republik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10323-5">
    <skos:prefLabel xml:lang="en">Multiproduct production</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10323-5">
    <skos:prefLabel xml:lang="de">Mehrproduktproduktion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15968-4">
    <skos:scopeNote xml:lang="de">Literatur über die Geschichte des Rechts allgemein oder des Rechtssystems in einem Land bzw. über die Geschichte eines Rechtsgebiets.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14292-5">
    <skos:related rdf:resource="descriptor/14642-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16729-3">
    <skos:broader rdf:resource="thsys/73393"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16729-3">
    <skos:broader rdf:resource="descriptor/16732-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18889-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11675-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18089-3">
    <skos:scopeNote xml:lang="de">Nur für Eisenbahnverkehr.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19186-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10618-0">
    <skos:broader rdf:resource="thsys/73366"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10618-0">
    <skos:broader rdf:resource="thsys/71097"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10618-0">
    <skos:broader rdf:resource="descriptor/16439-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14440-2">
    <zbwext:useInsteadNote rdf:nodeID="id251"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17919-0">
    <skos:prefLabel xml:lang="en">Chile</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17919-0">
    <skos:prefLabel xml:lang="de">Chile</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19146-5">
    <skos:related rdf:resource="descriptor/18070-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19146-5">
    <skos:related rdf:resource="descriptor/19151-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19146-5">
    <skos:related rdf:resource="descriptor/19601-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19146-5">
    <skos:related rdf:resource="descriptor/10459-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19146-5">
    <skos:related rdf:resource="descriptor/20468-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16301-6">
    <skos:related rdf:resource="descriptor/16335-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16301-6">
    <skos:related rdf:resource="descriptor/11347-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16301-6">
    <skos:related rdf:resource="descriptor/15977-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16301-6">
    <skos:related rdf:resource="descriptor/15972-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16301-6">
    <skos:related rdf:resource="descriptor/19715-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16301-6">
    <skos:related rdf:resource="descriptor/12879-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11990-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#narrower">
    <rdfs:subPropertyOf rdf:resource="http://www.w3.org/2004/02/skos/core#narrowerTransitive"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13320-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13355-5">
    <skos:narrower rdf:resource="descriptor/18788-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14057-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16344-2">
    <skos:altLabel xml:lang="de">Fraktion des Parlaments</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18290-3">
    <skos:prefLabel xml:lang="en">Liquid cristal display</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18290-3">
    <skos:prefLabel xml:lang="de">Flüssigkristallanzeige</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73358">
    <rdfs:label xml:lang="de">Soziale Gruppen</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11159-1">
    <skos:altLabel xml:lang="en">Psychological contracts</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11159-1">
    <skos:altLabel xml:lang="en">Psychological contract</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11159-1">
    <skos:altLabel xml:lang="en">Implicit contract theory</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11159-1">
    <skos:altLabel xml:lang="de">Psychologischer Vertrag</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11159-1">
    <skos:altLabel xml:lang="de">Impliziter Vertrag</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11159-1">
    <skos:altLabel xml:lang="de">Implizite-Kontrakte-Theorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70021">
    <skos:prefLabel xml:lang="de">V.04  Konjunktur, Wachstum und Wirtschaftsstruktur</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15988-5">
    <skos:narrower rdf:resource="descriptor/18272-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15988-5">
    <skos:narrower rdf:resource="descriptor/18961-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19607-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14513-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14127-6">
    <skos:broader rdf:resource="thsys/70467"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14127-6">
    <skos:broader rdf:resource="descriptor/14129-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10880-1">
    <skos:scopeNote xml:lang="de">Dem allgemeinen Handelsbrauch oder rechtlichen Festlegungen entsprechende Liefer- und Zahlungsmodalitäten im Außenhandel.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18649-5">
    <skos:narrower rdf:resource="descriptor/18650-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16445-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11825-4">
    <skos:altLabel xml:lang="x-other">Economie sociale</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17978-5">
    <skos:broader rdf:resource="thsys/70307"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17201-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13724-0">
    <skos:related rdf:resource="descriptor/19538-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15941-3">
    <skos:related rdf:resource="descriptor/15672-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15941-3">
    <skos:related rdf:resource="descriptor/19525-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15941-3">
    <skos:related rdf:resource="descriptor/19788-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15941-3">
    <skos:related rdf:resource="descriptor/15939-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15941-3">
    <skos:related rdf:resource="descriptor/10173-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15941-3">
    <skos:related rdf:resource="descriptor/15938-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15941-3">
    <skos:related rdf:resource="descriptor/14724-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15941-3">
    <skos:related rdf:resource="descriptor/15061-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15941-3">
    <skos:related rdf:resource="descriptor/15708-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15941-3">
    <skos:related rdf:resource="descriptor/19638-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12865-1">
    <skos:broader rdf:resource="thsys/71040"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12865-1">
    <skos:broader rdf:resource="thsys/70814"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12865-1">
    <skos:broader rdf:resource="thsys/70087"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12865-1">
    <skos:broader rdf:resource="descriptor/12072-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10029-5">
    <skos:related rdf:resource="descriptor/10966-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10029-5">
    <skos:related rdf:resource="descriptor/10913-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10029-5">
    <skos:related rdf:resource="descriptor/10968-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10029-5">
    <skos:related rdf:resource="descriptor/18850-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10833-3">
    <skos:editorialNote xml:lang="de">Nur zur Postkoordination verwenden.</skos:editorialNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10948-0">
    <skos:altLabel xml:lang="en">Contestable market</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10948-0">
    <skos:altLabel xml:lang="de">Potenzielle Konkurrenz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10948-0">
    <skos:altLabel xml:lang="de">Potentieller Wettbewerb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10948-0">
    <skos:altLabel xml:lang="de">Potentielle Konkurrenz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10948-0">
    <skos:altLabel xml:lang="de">Latenter Wettbewerb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18413-6">
    <skos:related rdf:resource="descriptor/14384-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18413-6">
    <skos:related rdf:resource="descriptor/13160-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18413-6">
    <skos:related rdf:resource="descriptor/18443-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18413-6">
    <skos:related rdf:resource="descriptor/18562-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10763-5">
    <skos:altLabel xml:lang="de">Dollar</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18901-0">
    <skos:related rdf:resource="descriptor/13374-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18901-0">
    <skos:related rdf:resource="descriptor/14731-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18901-0">
    <skos:related rdf:resource="descriptor/14169-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14317-1">
    <skos:related rdf:resource="descriptor/13576-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14317-1">
    <skos:related rdf:resource="descriptor/13066-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14317-1">
    <skos:related rdf:resource="descriptor/18394-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14317-1">
    <skos:related rdf:resource="descriptor/18393-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16290-6">
    <skos:related rdf:resource="descriptor/15720-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16290-6">
    <skos:related rdf:resource="descriptor/16286-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16290-6">
    <skos:related rdf:resource="descriptor/19683-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11736-3">
    <skos:altLabel xml:lang="de">Funktionale Einkommensverteilung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19503-0">
    <skos:altLabel xml:lang="en">Trade union rights</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19503-0">
    <skos:altLabel xml:lang="de">Gewerkschaftsrechte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11348-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19273-5">
    <skos:altLabel xml:lang="de">Ländlicher Kapitalmarkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11688-2">
    <skos:prefLabel xml:lang="en">Optimal taxation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11688-2">
    <skos:prefLabel xml:lang="de">Optimale Besteuerung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15266-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24708-1">
    <skos:editorialNote xml:lang="de">Der Begriff darf nur verwendet werden, wenn mindestens 2 der genannten Branchen gemeinsam behandelt werden. Ansonsten muß der spezielle Deskriptor vergeben werden.</skos:editorialNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19009-1">
    <skos:related rdf:resource="descriptor/15713-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10986-6">
    <skos:broader rdf:resource="thsys/71102"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10986-6">
    <skos:broader rdf:resource="descriptor/11068-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10986-6">
    <skos:broader rdf:resource="descriptor/16411-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12283-2">
    <skos:broader rdf:resource="thsys/70876"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12283-2">
    <skos:broader rdf:resource="thsys/70212"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16252-0">
    <skos:related rdf:resource="descriptor/19725-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16252-0">
    <skos:related rdf:resource="descriptor/16684-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16252-0">
    <skos:related rdf:resource="descriptor/18815-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16252-0">
    <skos:related rdf:resource="descriptor/12085-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16252-0">
    <skos:related rdf:resource="descriptor/15825-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16252-0">
    <skos:related rdf:resource="descriptor/15823-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16252-0">
    <skos:related rdf:resource="descriptor/11289-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11590-5">
    <skos:prefLabel xml:lang="en">Municipal budgeting</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11590-5">
    <skos:prefLabel xml:lang="de">Kommunale Finanzplanung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10501-0">
    <skos:broader rdf:resource="thsys/70575"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10501-0">
    <skos:broader rdf:resource="thsys/70091"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11616-6">
    <skos:prefLabel xml:lang="en">State-Owned Assets</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11616-6">
    <skos:prefLabel xml:lang="de">Öffentliches Vermögen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12294-4">
    <skos:related rdf:resource="descriptor/10015-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12294-4">
    <skos:related rdf:resource="descriptor/12699-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11411-3">
    <skos:related rdf:resource="descriptor/11436-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11411-3">
    <skos:related rdf:resource="descriptor/11426-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11411-3">
    <skos:related rdf:resource="descriptor/11419-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11411-3">
    <skos:related rdf:resource="descriptor/11444-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11411-3">
    <skos:related rdf:resource="descriptor/11443-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18266-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12648-2">
    <skos:related rdf:resource="descriptor/12649-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12648-2">
    <skos:related rdf:resource="descriptor/10027-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12648-2">
    <skos:related rdf:resource="descriptor/12922-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12648-2">
    <skos:related rdf:resource="descriptor/12670-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12648-2">
    <skos:related rdf:resource="descriptor/12640-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12725-3">
    <skos:prefLabel xml:lang="en">Manufacturing system</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12725-3">
    <skos:prefLabel xml:lang="de">Fertigungssystem</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16680-0">
    <skos:related rdf:resource="descriptor/19611-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16680-0">
    <skos:related rdf:resource="descriptor/15720-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16680-0">
    <skos:related rdf:resource="descriptor/18932-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16680-0">
    <skos:related rdf:resource="descriptor/16215-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16680-0">
    <skos:related rdf:resource="descriptor/13355-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16680-0">
    <skos:related rdf:resource="descriptor/18924-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16680-0">
    <skos:related rdf:resource="descriptor/15694-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16680-0">
    <skos:related rdf:resource="descriptor/15654-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16680-0">
    <skos:related rdf:resource="descriptor/16679-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16680-0">
    <skos:related rdf:resource="descriptor/16556-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13128-2">
    <skos:altLabel xml:lang="en">Garment industry</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13128-2">
    <skos:altLabel xml:lang="en">Apparel industry</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13128-2">
    <skos:altLabel xml:lang="de">Konfektionsindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16774-5">
    <skos:prefLabel xml:lang="en">Database</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16774-5">
    <skos:prefLabel xml:lang="de">Datenbank</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13473-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12254-2">
    <skos:related rdf:resource="descriptor/11203-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12254-2">
    <skos:related rdf:resource="descriptor/18760-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12254-2">
    <skos:related rdf:resource="descriptor/18759-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12254-2">
    <skos:related rdf:resource="descriptor/12274-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14945-0">
    <skos:related rdf:resource="descriptor/14088-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13632-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12433-2">
    <skos:prefLabel xml:lang="en">Group valuation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12433-2">
    <skos:prefLabel xml:lang="de">Sammelbewertung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19337-5">
    <skos:related rdf:resource="descriptor/10722-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19337-5">
    <skos:related rdf:resource="descriptor/19665-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19337-5">
    <skos:related rdf:resource="descriptor/10746-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19337-5">
    <skos:related rdf:resource="descriptor/10781-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19337-5">
    <skos:related rdf:resource="descriptor/10779-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19337-5">
    <skos:related rdf:resource="descriptor/18730-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19337-5">
    <skos:related rdf:resource="descriptor/13688-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19569-5">
    <skos:scopeNote xml:lang="de">Nur für statistische Literatur; nicht für Bootstrapping in der Computer- und Software-Technik und nicht für die Steuerung von gesellschaftlichen Prozessen im Sinne des US-amerikanischen Bootstrapping.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71086">
    <rdfs:label xml:lang="en">Development theory</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71086">
    <rdfs:label xml:lang="de">Entwicklungstheorie</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71084">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19248-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10473-0">
    <skos:prefLabel xml:lang="en">Choice of technology</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10473-0">
    <skos:prefLabel xml:lang="de">Technologiewahl</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19728-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13860-6">
    <skos:prefLabel xml:lang="en">Housing research</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13860-6">
    <skos:prefLabel xml:lang="de">Wohnungswissenschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10000-1">
    <skos:prefLabel xml:lang="en">Supply</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10000-1">
    <skos:prefLabel xml:lang="de">Angebot</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18071-1">
    <skos:prefLabel xml:lang="en">Environmental restoration</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18071-1">
    <skos:prefLabel xml:lang="de">Umweltsanierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73338">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71038">
    <rdfs:label xml:lang="de">IS-Komponenten</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17500-2">
    <skos:prefLabel xml:lang="en">Tajikistan</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17500-2">
    <skos:prefLabel xml:lang="de">Tadschikistan</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11160-2">
    <skos:broader rdf:resource="thsys/70274"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11160-2">
    <skos:broader rdf:resource="descriptor/10975-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11160-2">
    <skos:broader rdf:resource="descriptor/11161-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12720-6">
    <skos:broader rdf:resource="thsys/70310"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12720-6">
    <skos:broader rdf:resource="descriptor/12725-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15755-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18075-0">
    <skos:prefLabel xml:lang="en">CD-ROM</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18075-0">
    <skos:prefLabel xml:lang="de">CD-ROM</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15010-1">
    <skos:related rdf:resource="descriptor/18623-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15010-1">
    <skos:related rdf:resource="descriptor/14640-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15010-1">
    <skos:related rdf:resource="descriptor/14603-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15010-1">
    <skos:related rdf:resource="descriptor/15728-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12361-1">
    <skos:broader rdf:resource="thsys/73391"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12361-1">
    <skos:broader rdf:resource="thsys/71043"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12361-1">
    <skos:broader rdf:resource="descriptor/12326-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12361-1">
    <skos:broader rdf:resource="descriptor/19142-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11318-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10672-1">
    <skos:altLabel xml:lang="de">Monetaristische Zahlungsbilanztheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19464-5">
    <skos:broader rdf:resource="thsys/71065"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19464-5">
    <skos:broader rdf:resource="descriptor/19460-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19464-5">
    <skos:broader rdf:resource="descriptor/12913-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10494-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13224-6">
    <skos:broader rdf:resource="thsys/70957"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13224-6">
    <skos:broader rdf:resource="descriptor/13472-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17783-6">
    <skos:prefLabel xml:lang="en">Haiti</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17783-6">
    <skos:prefLabel xml:lang="de">Haiti</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19334-4">
    <skos:altLabel xml:lang="de">Währungsrealignment</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24856-0">
    <skos:related rdf:resource="descriptor/12079-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24856-0">
    <skos:related rdf:resource="descriptor/18106-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24856-0">
    <skos:related rdf:resource="descriptor/12053-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11051-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14331-0">
    <skos:broader rdf:resource="thsys/70549"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14331-0">
    <skos:broader rdf:resource="descriptor/12705-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14331-0">
    <skos:broader rdf:resource="descriptor/14336-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15770-4">
    <skos:narrower rdf:resource="descriptor/15746-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15770-4">
    <skos:narrower rdf:resource="descriptor/19632-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15770-4">
    <skos:narrower rdf:resource="descriptor/19481-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15770-4">
    <skos:narrower rdf:resource="descriptor/15737-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15770-4">
    <skos:narrower rdf:resource="descriptor/15734-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15770-4">
    <skos:narrower rdf:resource="descriptor/12021-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70169">
    <skos:broader rdf:resource="thsys/70341"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19338-3">
    <skos:scopeNote xml:lang="de">Für die verschiedenen Formen des Lebensmittelhandels BENUTZE die entsprechenden Deskriptoren.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19338-3">
    <skos:scopeNote xml:lang="de">Nur für den Nahrungsmittelmarkt im ökonomischen Sinne.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18317-2">
    <skos:altLabel xml:lang="de">Landwirtschaftliche Biotechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18317-2">
    <skos:altLabel xml:lang="de">Agrarbiotechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16687-0">
    <skos:broader rdf:resource="thsys/73355"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16687-0">
    <skos:broader rdf:resource="descriptor/16564-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14043-5">
    <skos:related rdf:resource="descriptor/12921-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14043-5">
    <skos:related rdf:resource="descriptor/13222-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18060-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14060-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16265-5">
    <skos:related rdf:resource="descriptor/12837-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16265-5">
    <skos:related rdf:resource="descriptor/15858-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16265-5">
    <skos:related rdf:resource="descriptor/16269-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16265-5">
    <skos:related rdf:resource="descriptor/12775-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16265-5">
    <skos:related rdf:resource="descriptor/13661-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16265-5">
    <skos:related rdf:resource="descriptor/15798-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13527-2">
    <skos:prefLabel xml:lang="en">Rail track</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13527-2">
    <skos:prefLabel xml:lang="de">Schienenstrecke</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18338-1">
    <skos:related rdf:resource="descriptor/12961-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18338-1">
    <skos:related rdf:resource="descriptor/18311-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18338-1">
    <skos:related rdf:resource="descriptor/18988-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18338-1">
    <skos:related rdf:resource="descriptor/18336-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18338-1">
    <skos:related rdf:resource="descriptor/11801-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12325-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18207-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18484-4">
    <skos:altLabel xml:lang="de">Industriearbeiterinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14418-2">
    <skos:narrower rdf:resource="descriptor/18246-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14418-2">
    <skos:narrower rdf:resource="descriptor/18297-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14418-2">
    <skos:narrower rdf:resource="descriptor/14353-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14450-6">
    <skos:altLabel xml:lang="de">U-Boot</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18496-4">
    <skos:broader rdf:resource="thsys/71066"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18496-4">
    <skos:broader rdf:resource="thsys/70889"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18496-4">
    <skos:broader rdf:resource="descriptor/12029-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19293-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10926-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10994-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14954-6">
    <skos:prefLabel xml:lang="en">Cheese</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14954-6">
    <skos:prefLabel xml:lang="de">Käse</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19247-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19842-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19132-2">
    <skos:prefLabel xml:lang="en">Advertising restriction</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19132-2">
    <skos:prefLabel xml:lang="de">Werbebeschränkung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18830-4">
    <skos:related rdf:resource="descriptor/16272-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18830-4">
    <skos:related rdf:resource="descriptor/12047-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18830-4">
    <skos:related rdf:resource="descriptor/19680-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18830-4">
    <skos:related rdf:resource="descriptor/16409-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18830-4">
    <skos:related rdf:resource="descriptor/16587-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18830-4">
    <skos:related rdf:resource="descriptor/11037-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19129-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15740-6">
    <skos:broader rdf:resource="thsys/70239"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15740-6">
    <skos:broader rdf:resource="descriptor/15737-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71071">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11589-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11957-1">
    <skos:related rdf:resource="descriptor/19613-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11957-1">
    <skos:related rdf:resource="descriptor/14814-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13137-1">
    <skos:altLabel xml:lang="de">Lederwarenindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13137-1">
    <skos:altLabel xml:lang="de">Ledertaschenindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13137-1">
    <skos:altLabel xml:lang="de">Lederbekleidungsindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13137-1">
    <skos:altLabel xml:lang="de">Kofferindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19570-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18523-6">
    <skos:altLabel xml:lang="de">PC-Handel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19619-2">
    <skos:broader rdf:resource="thsys/73353"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21042-2">
    <skos:altLabel xml:lang="de">Osttimor</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21042-2">
    <skos:altLabel xml:lang="de">Ost-Timor</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21042-2">
    <skos:altLabel xml:lang="de">Demokratische Republik Timor-Leste</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21042-2">
    <skos:altLabel xml:lang="de">Demokratische Republik Osttimor</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15917-0">
    <skos:prefLabel xml:lang="en">Inuit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15917-0">
    <skos:prefLabel xml:lang="de">Inuit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11444-2">
    <skos:prefLabel xml:lang="en">Currency school</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11444-2">
    <skos:prefLabel xml:lang="de">Currency-Theorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10054-6">
    <skos:broader rdf:resource="thsys/70141"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10054-6">
    <skos:broader rdf:resource="descriptor/16745-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13871-1">
    <skos:broader rdf:resource="thsys/71055"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13871-1">
    <skos:broader rdf:resource="thsys/70958"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13871-1">
    <skos:broader rdf:resource="descriptor/18535-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15124-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19344-1">
    <skos:related rdf:resource="descriptor/19343-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19344-1">
    <skos:related rdf:resource="descriptor/12950-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19344-1">
    <skos:related rdf:resource="descriptor/14142-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71061">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.13.01.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15954-1">
    <skos:prefLabel xml:lang="en">International migration</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15954-1">
    <skos:prefLabel xml:lang="de">Internationale Wanderung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13472-1">
    <skos:narrower rdf:resource="descriptor/13224-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13472-1">
    <skos:narrower rdf:resource="descriptor/13215-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12216-3">
    <skos:related rdf:resource="descriptor/12226-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12216-3">
    <skos:related rdf:resource="descriptor/12898-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12216-3">
    <skos:related rdf:resource="descriptor/12218-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12216-3">
    <skos:related rdf:resource="descriptor/12458-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12216-3">
    <skos:related rdf:resource="descriptor/12357-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12216-3">
    <skos:related rdf:resource="descriptor/12053-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19250-3">
    <skos:prefLabel xml:lang="en">Economies of scope</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19250-3">
    <skos:prefLabel xml:lang="de">Verbundvorteil</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19841-1">
    <skos:narrower rdf:resource="descriptor/15979-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19841-1">
    <skos:narrower rdf:resource="descriptor/15962-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19841-1">
    <skos:narrower rdf:resource="descriptor/13410-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17980-4">
    <skos:narrower rdf:resource="descriptor/17433-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17980-4">
    <skos:narrower rdf:resource="descriptor/17198-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17980-4">
    <skos:narrower rdf:resource="descriptor/17201-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16727-0">
    <skos:narrower rdf:resource="descriptor/15064-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16727-0">
    <skos:narrower rdf:resource="descriptor/20022-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16727-0">
    <skos:narrower rdf:resource="descriptor/19989-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16727-0">
    <skos:narrower rdf:resource="descriptor/19219-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16727-0">
    <skos:narrower rdf:resource="descriptor/16726-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16727-0">
    <skos:narrower rdf:resource="descriptor/19218-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16727-0">
    <skos:narrower rdf:resource="descriptor/19743-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16727-0">
    <skos:narrower rdf:resource="descriptor/19070-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16727-0">
    <skos:narrower rdf:resource="descriptor/19742-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12244-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18852-1">
    <skos:scopeNote xml:lang="de">Organisatorische Gestaltung und Aufbau von Gewerkschaften.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18368-6">
    <skos:related rdf:resource="descriptor/12605-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18368-6">
    <skos:related rdf:resource="descriptor/15757-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18368-6">
    <skos:related rdf:resource="descriptor/14035-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18368-6">
    <skos:related rdf:resource="descriptor/16001-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16387-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10891-3">
    <skos:prefLabel xml:lang="en">Industrial economics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10891-3">
    <skos:prefLabel xml:lang="de">Industrieökonomik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16603-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13627-5">
    <skos:broader rdf:resource="thsys/70940"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13627-5">
    <skos:broader rdf:resource="thsys/70934"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13627-5">
    <skos:broader rdf:resource="descriptor/13553-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12748-5">
    <skos:prefLabel xml:lang="en">Work scheduling</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12748-5">
    <skos:prefLabel xml:lang="de">Arbeitsplanung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19016-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10513-0">
    <skos:prefLabel xml:lang="en">Economic development</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10513-0">
    <skos:prefLabel xml:lang="de">Entwicklung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73354">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18429-5">
    <skos:scopeNote xml:lang="de">Für alle Verfahrenstechniken, die in der Chemischen Industrie angewendet werden.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13464-0">
    <skos:related rdf:resource="descriptor/16081-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13464-0">
    <skos:related rdf:resource="descriptor/15977-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13464-0">
    <skos:related rdf:resource="descriptor/12191-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13464-0">
    <skos:related rdf:resource="descriptor/12787-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13464-0">
    <skos:related rdf:resource="descriptor/13210-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13464-0">
    <skos:related rdf:resource="descriptor/19819-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13464-0">
    <skos:related rdf:resource="descriptor/16053-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13464-0">
    <skos:related rdf:resource="descriptor/16050-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13464-0">
    <skos:related rdf:resource="descriptor/19080-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13464-0">
    <skos:related rdf:resource="descriptor/12033-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13135-5">
    <skos:related rdf:resource="descriptor/14630-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13135-5">
    <skos:related rdf:resource="descriptor/14829-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13135-5">
    <skos:related rdf:resource="descriptor/14127-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18027-4">
    <skos:prefLabel xml:lang="en">Broadcast programme</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18027-4">
    <skos:prefLabel xml:lang="de">Rundfunkprogramm</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19508-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18079-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14385-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13291-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12722-2">
    <skos:related rdf:resource="descriptor/12718-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14354-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14306-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10005-5">
    <skos:narrower rdf:resource="descriptor/11124-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10005-5">
    <skos:narrower rdf:resource="descriptor/12026-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10005-5">
    <skos:narrower rdf:resource="descriptor/11044-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18130-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13053-0">
    <skos:broader rdf:resource="thsys/70997"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13053-0">
    <skos:broader rdf:resource="descriptor/13051-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18383-3">
    <skos:broader rdf:resource="thsys/71009"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18383-3">
    <skos:broader rdf:resource="descriptor/12041-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11422-5">
    <skos:related rdf:resource="descriptor/11460-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11422-5">
    <skos:related rdf:resource="descriptor/11449-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18149-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15126-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10854-2">
    <skos:broader rdf:resource="thsys/71091"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10854-2">
    <skos:broader rdf:resource="thsys/70951"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10854-2">
    <skos:broader rdf:resource="descriptor/25742-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18396-1">
    <skos:related rdf:resource="descriptor/13079-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18396-1">
    <skos:related rdf:resource="descriptor/18252-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18396-1">
    <skos:related rdf:resource="descriptor/13078-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18396-1">
    <skos:related rdf:resource="descriptor/13076-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15610-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12744-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70871">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18165-6">
    <skos:prefLabel xml:lang="en">Environmental management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18165-6">
    <skos:prefLabel xml:lang="de">Umweltmanagement</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15839-1">
    <zbwext:useInsteadNote rdf:nodeID="id192"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19477-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19731-1">
    <skos:broader rdf:resource="thsys/73394"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19731-1">
    <skos:broader rdf:resource="descriptor/16719-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18752-5">
    <skos:related rdf:resource="descriptor/13840-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17201-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11888-1">
    <skos:broader rdf:resource="thsys/70572"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11888-1">
    <skos:broader rdf:resource="thsys/70058"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11888-1">
    <skos:broader rdf:resource="descriptor/15709-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19147-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18235-4">
    <skos:related rdf:resource="descriptor/18024-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13198-2">
    <skos:prefLabel xml:lang="en">Chemicals trade</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13198-2">
    <skos:prefLabel xml:lang="de">Chemikalienhandel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14997-2">
    <skos:broader rdf:resource="thsys/73298"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14997-2">
    <skos:broader rdf:resource="descriptor/18222-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12845-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10324-3">
    <skos:broader rdf:resource="thsys/70310"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10324-3">
    <skos:broader rdf:resource="descriptor/12725-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14490-1">
    <skos:prefLabel xml:lang="en">Telecommunications equipment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14490-1">
    <skos:prefLabel xml:lang="de">Nachrichtentechnisches Gerät</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18257-1">
    <skos:broader rdf:resource="thsys/70476"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18257-1">
    <skos:broader rdf:resource="descriptor/14710-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18257-1">
    <skos:broader rdf:resource="descriptor/14151-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10481-1">
    <skos:broader rdf:resource="thsys/73391"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10481-1">
    <skos:broader rdf:resource="thsys/71109"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10481-1">
    <skos:broader rdf:resource="descriptor/16053-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10115-5">
    <skos:related rdf:resource="descriptor/11129-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10115-5">
    <skos:related rdf:resource="descriptor/12354-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10115-5">
    <skos:related rdf:resource="descriptor/10287-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16329-5">
    <skos:narrower rdf:resource="descriptor/19694-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16329-5">
    <skos:narrower rdf:resource="descriptor/16330-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16329-5">
    <skos:narrower rdf:resource="descriptor/16327-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16329-5">
    <skos:narrower rdf:resource="descriptor/16324-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14502-6">
    <skos:related rdf:resource="descriptor/14554-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14502-6">
    <skos:related rdf:resource="descriptor/13075-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11036-3">
    <skos:prefLabel xml:lang="en">Worker self-management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11036-3">
    <skos:prefLabel xml:lang="de">Arbeiterselbstverwaltung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20817-2">
    <skos:broader rdf:resource="thsys/71091"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20817-2">
    <skos:broader rdf:resource="thsys/70091"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20817-2">
    <skos:broader rdf:resource="descriptor/10587-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12738-1">
    <skos:altLabel xml:lang="de">PC-Werkzeugmaschine</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12738-1">
    <skos:altLabel xml:lang="de">DNC-Werkzeugmaschine</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12738-1">
    <skos:altLabel xml:lang="de">CNC-Werkzeugmaschine</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12738-1">
    <skos:altLabel xml:lang="de">AC-Werkzeugmaschine</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21020-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12777-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10587-6">
    <skos:narrower rdf:resource="descriptor/20817-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18250-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71081">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.09.03</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13302-5">
    <skos:broader rdf:resource="thsys/70888"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13302-5">
    <skos:broader rdf:resource="thsys/70855"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13302-5">
    <skos:broader rdf:resource="descriptor/13305-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13302-5">
    <skos:broader rdf:resource="descriptor/11825-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16487-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10062-0">
    <skos:narrower rdf:resource="descriptor/10113-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10062-0">
    <skos:narrower rdf:resource="descriptor/10956-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10062-0">
    <skos:narrower rdf:resource="descriptor/11090-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10062-0">
    <skos:narrower rdf:resource="descriptor/10144-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10062-0">
    <skos:narrower rdf:resource="descriptor/18310-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10062-0">
    <skos:narrower rdf:resource="descriptor/10249-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10062-0">
    <skos:narrower rdf:resource="descriptor/10181-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10062-0">
    <skos:narrower rdf:resource="descriptor/10891-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10062-0">
    <skos:narrower rdf:resource="descriptor/10076-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10062-0">
    <skos:narrower rdf:resource="descriptor/10236-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10062-0">
    <skos:narrower rdf:resource="descriptor/11715-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10062-0">
    <skos:narrower rdf:resource="descriptor/10525-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10062-0">
    <skos:narrower rdf:resource="descriptor/10629-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10062-0">
    <skos:narrower rdf:resource="descriptor/11219-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12071-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14957-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11833-5">
    <skos:related rdf:resource="descriptor/11834-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11833-5">
    <skos:related rdf:resource="descriptor/13569-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11833-5">
    <skos:related rdf:resource="descriptor/18187-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11833-5">
    <skos:related rdf:resource="descriptor/11832-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11833-5">
    <skos:related rdf:resource="descriptor/13588-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11833-5">
    <skos:related rdf:resource="descriptor/13264-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11833-5">
    <skos:related rdf:resource="descriptor/13510-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11833-5">
    <skos:related rdf:resource="descriptor/13639-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16250-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18558-1">
    <skos:prefLabel xml:lang="en">Bar code</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18558-1">
    <skos:prefLabel xml:lang="de">Barcode</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13727-1">
    <skos:prefLabel xml:lang="en">Action de jouissance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13727-1">
    <skos:prefLabel xml:lang="de">Genussschein</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12192-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19139-2">
    <skos:prefLabel xml:lang="en">Sales management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19139-2">
    <skos:prefLabel xml:lang="de">Verkauf</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15055-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19034-2">
    <skos:prefLabel xml:lang="en">Standardization</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19034-2">
    <skos:prefLabel xml:lang="de">Normung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70951">
    <skos:broader rdf:resource="thsys/70952"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12016-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70811">
    <skos:broader rdf:resource="thsys/70082"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18903-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16008-4">
    <skos:broader rdf:resource="thsys/73378"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16008-4">
    <skos:broader rdf:resource="thsys/73361"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16008-4">
    <skos:broader rdf:resource="descriptor/16017-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10043-4">
    <skos:prefLabel xml:lang="en">Small open economy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10043-4">
    <skos:prefLabel xml:lang="de">Kleines-offenes-Land</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12138-4">
    <skos:altLabel xml:lang="de">Organisatorische Zentralisierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70934">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12255-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12313-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70306">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.05.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10912-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18744-4">
    <skos:altLabel xml:lang="de">Vertrieb von Versicherungen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18744-4">
    <skos:altLabel xml:lang="de">Versicherungsvertrieb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10963-4">
    <skos:narrower rdf:resource="descriptor/10962-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10963-4">
    <skos:narrower rdf:resource="descriptor/10894-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10963-4">
    <skos:narrower rdf:resource="descriptor/10958-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10963-4">
    <skos:narrower rdf:resource="descriptor/18011-5"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id77">
    <rdfs:seeAlso rdf:resource="descriptor/19081-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19831-4">
    <skos:broader rdf:resource="thsys/70209"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19831-4">
    <skos:broader rdf:resource="descriptor/15767-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70082">
    <skos:prefLabel xml:lang="de">V.12  Umwelt- und Ressourcenökonomik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13649-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70832">
    <skos:narrower rdf:resource="descriptor/13411-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70832">
    <skos:narrower rdf:resource="descriptor/19515-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70832">
    <skos:narrower rdf:resource="descriptor/16160-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70832">
    <skos:narrower rdf:resource="descriptor/18913-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70832">
    <skos:narrower rdf:resource="descriptor/16547-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70832">
    <skos:narrower rdf:resource="descriptor/13418-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70832">
    <skos:narrower rdf:resource="descriptor/19512-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70832">
    <skos:narrower rdf:resource="descriptor/16539-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70832">
    <skos:narrower rdf:resource="descriptor/18739-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70832">
    <skos:narrower rdf:resource="descriptor/18180-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70832">
    <skos:narrower rdf:resource="descriptor/18105-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70832">
    <skos:narrower rdf:resource="descriptor/16548-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70832">
    <skos:narrower rdf:resource="descriptor/19514-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70832">
    <skos:narrower rdf:resource="descriptor/19551-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70832">
    <skos:narrower rdf:resource="descriptor/13292-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70832">
    <skos:narrower rdf:resource="descriptor/19513-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70832">
    <skos:narrower rdf:resource="descriptor/13416-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70832">
    <skos:narrower rdf:resource="descriptor/13415-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70832">
    <skos:narrower rdf:resource="descriptor/16531-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70832">
    <skos:narrower rdf:resource="descriptor/12591-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70832">
    <skos:narrower rdf:resource="descriptor/13406-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70832">
    <skos:narrower rdf:resource="descriptor/16527-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70832">
    <skos:narrower rdf:resource="descriptor/16526-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17739-2">
    <skos:prefLabel xml:lang="en">Sao Tome and Principe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17739-2">
    <skos:prefLabel xml:lang="de">Sao Tome und Principe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14816-4">
    <skos:altLabel xml:lang="de">Vervielfältigungsprodukt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14816-4">
    <skos:altLabel xml:lang="de">Photokopie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14816-4">
    <skos:altLabel xml:lang="de">Fotokopie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10804-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18809-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11574-3">
    <skos:broader rdf:resource="thsys/71080"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11574-3">
    <skos:broader rdf:resource="thsys/70888"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11574-3">
    <skos:broader rdf:resource="thsys/70884"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11574-3">
    <skos:broader rdf:resource="descriptor/11570-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11574-3">
    <skos:broader rdf:resource="descriptor/18754-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11574-3">
    <skos:broader rdf:resource="descriptor/18755-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19175-5">
    <skos:related rdf:resource="descriptor/17687-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19175-5">
    <skos:related rdf:resource="descriptor/17672-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12303-1">
    <skos:broader rdf:resource="thsys/71034"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11738-6">
    <skos:broader rdf:resource="thsys/71114"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11738-6">
    <skos:broader rdf:resource="thsys/70269"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11738-6">
    <skos:broader rdf:resource="descriptor/15045-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11738-6">
    <skos:broader rdf:resource="descriptor/11753-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15686-0">
    <skos:scopeNote xml:lang="de">Befasst sich u.a. mit der Entwicklung der Wissenschaft oder einzelner Disziplinen, ihren Inhalten und ihrer Vermittlung bzw. mit dem Wissenschaftssystem (Einrichtungen, Scientific Community) einzelner Länder/Regionen. Zunehmend werden quantitative Analysen unternommen (Scientometrie).</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18112-6">
    <skos:prefLabel xml:lang="en">Telework</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18112-6">
    <skos:prefLabel xml:lang="de">Telearbeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#narrower">
    <rdfs:label xml:lang="en">has narrower</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20002-5">
    <skos:broader rdf:resource="thsys/70952"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20002-5">
    <skos:broader rdf:resource="descriptor/10838-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19779-1">
    <skos:broader rdf:resource="thsys/70580"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17590-3">
    <skos:prefLabel xml:lang="en">Thailand</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17590-3">
    <skos:prefLabel xml:lang="de">Thailand</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10944-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14215-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18218-4">
    <skos:related rdf:resource="descriptor/12941-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18218-4">
    <skos:related rdf:resource="descriptor/14135-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19171-6">
    <skos:related rdf:resource="descriptor/19172-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19477-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11211-4">
    <skos:prefLabel xml:lang="en">Labour market segmentation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11211-4">
    <skos:prefLabel xml:lang="de">Arbeitsmarktsegmentierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#narrowMatch">
    <rdfs:subPropertyOf rdf:resource="http://www.w3.org/2004/02/skos/core#narrower"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#narrowMatch">
    <rdfs:subPropertyOf rdf:resource="http://www.w3.org/2004/02/skos/core#mappingRelation"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18248-2">
    <skos:related rdf:resource="descriptor/13627-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18967-5">
    <skos:scopeNote xml:lang="de">Landschaftliche Neugestaltung eines durch wirtschaftliche oder militärische Nutzung zerstörten Geländes.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13229-3">
    <skos:prefLabel xml:lang="en">Photographic shop</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13229-3">
    <skos:prefLabel xml:lang="de">Fotohandel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13985-0">
    <skos:altLabel xml:lang="en">Clerical occupations</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13985-0">
    <skos:altLabel xml:lang="de">Sekretärinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13985-0">
    <skos:altLabel xml:lang="de">Sekretäre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13985-0">
    <skos:altLabel xml:lang="de">Schreibkräfte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13985-0">
    <skos:altLabel xml:lang="de">Sachbearbeiterinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13985-0">
    <skos:altLabel xml:lang="de">Sachbearbeiter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13985-0">
    <skos:altLabel xml:lang="de">Datentypistinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13985-0">
    <skos:altLabel xml:lang="de">Datentypisten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13985-0">
    <skos:altLabel xml:lang="de">Bürokaufleute</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13985-0">
    <skos:altLabel xml:lang="de">Bürofachkräfte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13985-0">
    <skos:altLabel xml:lang="de">Büroangestellte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11237-0">
    <skos:broader rdf:resource="thsys/73380"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11237-0">
    <skos:broader rdf:resource="thsys/71061"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11237-0">
    <skos:broader rdf:resource="thsys/70868"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11237-0">
    <skos:broader rdf:resource="descriptor/13413-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19427-4">
    <skos:scopeNote xml:lang="de">Zerlegung des Rohöls</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11441-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18741-3">
    <skos:altLabel xml:lang="de">Versicherungsökonomie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18741-3">
    <skos:altLabel xml:lang="de">Ökonomische Theorie der Versicherungsmärkte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70301">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11206-4">
    <skos:related rdf:resource="descriptor/10658-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11206-4">
    <skos:related rdf:resource="descriptor/10614-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11071-1">
    <skos:altLabel xml:lang="en">Law of value</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11071-1">
    <skos:altLabel xml:lang="en">Labor theory of value</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11071-1">
    <skos:altLabel xml:lang="de">Wertgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11071-1">
    <skos:altLabel xml:lang="de">Ricardianische Wertlehre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11071-1">
    <skos:altLabel xml:lang="de">Ricardianische Arbeitswertlehre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11071-1">
    <skos:altLabel xml:lang="de">Marxistische Werttheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11071-1">
    <skos:altLabel xml:lang="de">Marxistische Wertlehre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11071-1">
    <skos:altLabel xml:lang="de">Arbeitswerttheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11071-1">
    <skos:altLabel xml:lang="de">Arbeitswert</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13706-2">
    <skos:broader rdf:resource="thsys/70903"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13706-2">
    <skos:broader rdf:resource="descriptor/13283-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12883-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17553-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11969-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17704-0">
    <skos:broader rdf:resource="thsys/70560"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17704-0">
    <skos:broader rdf:resource="descriptor/17724-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15470-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19842-6">
    <skos:scopeNote xml:lang="de">Für die Vollstreckung von Forderungen aller Art.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18958-6">
    <skos:broader rdf:resource="thsys/73371"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18958-6">
    <skos:broader rdf:resource="thsys/70818"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18958-6">
    <skos:broader rdf:resource="thsys/70239"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18958-6">
    <skos:broader rdf:resource="descriptor/15752-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11407-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14483-5">
    <zbwext:useInsteadNote rdf:nodeID="id160"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18655-3">
    <zbwext:useInsteadNote rdf:nodeID="id210"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18324-5">
    <skos:broader rdf:resource="thsys/71024"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18324-5">
    <skos:broader rdf:resource="thsys/71019"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18324-5">
    <skos:broader rdf:resource="descriptor/18315-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18324-5">
    <skos:broader rdf:resource="descriptor/12961-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18324-5">
    <skos:broader rdf:resource="descriptor/13452-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17829-1">
    <skos:altLabel xml:lang="en">United States of America</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17829-1">
    <skos:altLabel xml:lang="de">Vereinigte Staaten von Amerika</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17829-1">
    <skos:altLabel xml:lang="de">Vereinigte Staaten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18475-5">
    <skos:broader rdf:resource="thsys/70891"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18475-5">
    <skos:broader rdf:resource="descriptor/18737-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14488-2">
    <skos:related rdf:resource="descriptor/14486-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14434-4">
    <skos:broader rdf:resource="thsys/70258"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14434-4">
    <skos:broader rdf:resource="descriptor/14441-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24704-2">
    <skos:related rdf:resource="descriptor/19155-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18017-0">
    <skos:narrower rdf:resource="descriptor/18662-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18017-0">
    <skos:narrower rdf:resource="descriptor/18767-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18017-0">
    <skos:narrower rdf:resource="descriptor/20341-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18837-4">
    <skos:broader rdf:resource="thsys/71064"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18837-4">
    <skos:broader rdf:resource="thsys/70856"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18837-4">
    <skos:broader rdf:resource="descriptor/19448-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11912-2">
    <skos:broader rdf:resource="thsys/71070"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11912-2">
    <skos:broader rdf:resource="descriptor/10032-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11912-2">
    <skos:broader rdf:resource="descriptor/11951-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19357-6">
    <skos:related rdf:resource="descriptor/10833-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11353-5">
    <skos:related rdf:resource="descriptor/19253-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11353-5">
    <skos:related rdf:resource="descriptor/15656-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11353-5">
    <skos:related rdf:resource="descriptor/11367-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11353-5">
    <skos:related rdf:resource="descriptor/11397-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11353-5">
    <skos:related rdf:resource="descriptor/11354-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11353-5">
    <skos:related rdf:resource="descriptor/11394-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11353-5">
    <skos:related rdf:resource="descriptor/11400-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11353-5">
    <skos:related rdf:resource="descriptor/19916-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19300-0">
    <skos:prefLabel xml:lang="en">Rice price</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19300-0">
    <skos:prefLabel xml:lang="de">Reispreis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18399-2">
    <skos:related rdf:resource="descriptor/12905-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15704-3">
    <skos:broader rdf:resource="thsys/73387"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15704-3">
    <skos:broader rdf:resource="thsys/70141"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15704-3">
    <skos:broader rdf:resource="thsys/70109"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15704-3">
    <skos:broader rdf:resource="descriptor/15706-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15704-3">
    <skos:broader rdf:resource="descriptor/16745-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19777-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18328-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13117-0">
    <skos:altLabel xml:lang="de">Zellstoff- und Papierindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13117-0">
    <skos:altLabel xml:lang="de">Pappeindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13117-0">
    <skos:altLabel xml:lang="de">Papiermühle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13117-0">
    <skos:altLabel xml:lang="de">Papierindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13117-0">
    <skos:altLabel xml:lang="de">Papier- und Zellstoffindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13117-0">
    <skos:altLabel xml:lang="de">Kartonindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id372">
    <rdf:value xml:lang="de">Für langfristige qualitative Entwicklungsprozesse</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id390">
    <rdfs:seeAlso rdf:resource="descriptor/19042-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14083-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12882-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20002-5">
    <skos:prefLabel xml:lang="en">Parallel imports</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20002-5">
    <skos:prefLabel xml:lang="de">Parallelimport</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11671-5">
    <skos:prefLabel xml:lang="en">Luxury tax</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11671-5">
    <skos:prefLabel xml:lang="de">Luxussteuer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12387-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18829-3">
    <skos:related rdf:resource="descriptor/11559-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18829-3">
    <skos:related rdf:resource="descriptor/11318-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18829-3">
    <skos:related rdf:resource="descriptor/11185-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15026-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70179">
    <skos:prefLabel xml:lang="de">V.07  Außenwirtschaft und internationale Wirtschaftsbeziehungen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19570-6">
    <skos:prefLabel xml:lang="en">Structural break</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19570-6">
    <skos:prefLabel xml:lang="de">Strukturbruch</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20002-5">
    <skos:related rdf:resource="descriptor/12778-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20002-5">
    <skos:related rdf:resource="descriptor/18066-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20002-5">
    <skos:related rdf:resource="descriptor/12800-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20002-5">
    <skos:related rdf:resource="descriptor/10876-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20002-5">
    <skos:related rdf:resource="descriptor/10872-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20002-5">
    <skos:related rdf:resource="descriptor/16053-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19022-2">
    <skos:narrower rdf:resource="descriptor/14135-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18000-3">
    <skos:related rdf:resource="descriptor/18649-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18000-3">
    <skos:related rdf:resource="descriptor/19431-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18000-3">
    <skos:related rdf:resource="descriptor/18647-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18000-3">
    <skos:related rdf:resource="descriptor/18655-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19365-0">
    <zbwext:useInsteadNote rdf:nodeID="id166"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12470-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14916-0">
    <skos:prefLabel xml:lang="en">Furs</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14916-0">
    <skos:prefLabel xml:lang="de">Pelzwaren</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11350-4">
    <skos:broader rdf:resource="thsys/70852"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11350-4">
    <skos:broader rdf:resource="thsys/70235"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11350-4">
    <skos:broader rdf:resource="descriptor/10984-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12170-1">
    <skos:scopeNote xml:lang="de">Messung des Informationswertes z.B. von Börsen- oder Firmeninformationen, von statistischen Erhebungen, usw.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70879">
    <skos:broader rdf:resource="thsys/w"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19111-3">
    <skos:prefLabel xml:lang="en">Job</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19111-3">
    <skos:prefLabel xml:lang="de">Arbeitsplatz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15718-6">
    <skos:related rdf:resource="descriptor/16597-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15718-6">
    <skos:related rdf:resource="descriptor/15721-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15718-6">
    <skos:related rdf:resource="descriptor/15719-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15718-6">
    <skos:related rdf:resource="descriptor/15717-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15718-6">
    <skos:related rdf:resource="descriptor/16654-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15718-6">
    <skos:related rdf:resource="descriptor/16623-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15718-6">
    <skos:related rdf:resource="descriptor/16674-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15718-6">
    <skos:related rdf:resource="descriptor/15698-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15718-6">
    <skos:related rdf:resource="descriptor/16645-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15718-6">
    <skos:related rdf:resource="descriptor/16663-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15718-6">
    <skos:related rdf:resource="descriptor/19772-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15718-6">
    <skos:related rdf:resource="descriptor/11073-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15718-6">
    <skos:related rdf:resource="descriptor/15713-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15718-6">
    <skos:related rdf:resource="descriptor/11400-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15718-6">
    <skos:related rdf:resource="descriptor/10488-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11579-0">
    <skos:broader rdf:resource="thsys/70859"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11579-0">
    <skos:broader rdf:resource="thsys/70144"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11579-0">
    <skos:broader rdf:resource="descriptor/12640-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10774-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14334-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16042-4">
    <skos:prefLabel xml:lang="en">Private law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16042-4">
    <skos:prefLabel xml:lang="de">Privatrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13156-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18897-0">
    <skos:broader rdf:resource="thsys/73346"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18897-0">
    <skos:broader rdf:resource="descriptor/16670-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18791-2">
    <skos:broader rdf:resource="thsys/70875"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18791-2">
    <skos:broader rdf:resource="descriptor/12881-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18791-2">
    <skos:broader rdf:resource="descriptor/13389-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19396-3">
    <skos:related rdf:resource="descriptor/18103-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19396-3">
    <skos:related rdf:resource="descriptor/18425-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19396-3">
    <skos:related rdf:resource="descriptor/18424-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19396-3">
    <skos:related rdf:resource="descriptor/18523-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12188-3">
    <skos:broader rdf:resource="thsys/70087"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12188-3">
    <skos:broader rdf:resource="descriptor/12186-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19276-6">
    <skos:broader rdf:resource="thsys/71108"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19276-6">
    <skos:broader rdf:resource="descriptor/13726-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19276-6">
    <skos:broader rdf:resource="descriptor/11441-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10891-3">
    <skos:broader rdf:resource="thsys/70101"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10891-3">
    <skos:broader rdf:resource="descriptor/10062-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16842-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16066-4">
    <skos:broader rdf:resource="thsys/73366"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16066-4">
    <skos:broader rdf:resource="thsys/73332"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16066-4">
    <skos:broader rdf:resource="thsys/71084"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16066-4">
    <skos:broader rdf:resource="descriptor/16184-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16066-4">
    <skos:broader rdf:resource="descriptor/19350-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18839-0">
    <skos:related rdf:resource="descriptor/18841-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73360">
    <skos:broader rdf:resource="thsys/70160"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15810-4">
    <skos:broader rdf:resource="thsys/73358"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19037-3">
    <skos:editorialNote xml:lang="de">Möglichst spezifischer indexieren.</skos:editorialNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19641-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16551-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19568-0">
    <skos:narrower rdf:resource="descriptor/19569-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14722-6">
    <skos:narrower rdf:resource="descriptor/14725-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71035">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14122-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12818-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11294-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19724-5">
    <skos:broader rdf:resource="thsys/73363"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19724-5">
    <skos:broader rdf:resource="thsys/70865"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19724-5">
    <skos:broader rdf:resource="descriptor/12670-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17754-6">
    <skos:broader rdf:resource="thsys/70085"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17754-6">
    <skos:broader rdf:resource="descriptor/17750-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17754-6">
    <skos:broader rdf:resource="descriptor/17978-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16828-1">
    <skos:scopeNote xml:lang="de">Durch Parlamentsbeschluss aufgelöst am 31.12.1992; Nachfolgestaaten: Slowakei und Tschechische Republik.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18281-4">
    <skos:broader rdf:resource="thsys/71015"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18281-4">
    <skos:broader rdf:resource="descriptor/12977-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13515-2">
    <skos:prefLabel xml:lang="en">Transport mode</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13515-2">
    <skos:prefLabel xml:lang="de">Verkehrsträger</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18365-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18520-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18767-6">
    <skos:altLabel xml:lang="de">Dokument-Management-System</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18767-6">
    <skos:altLabel xml:lang="de">Dokumentenverwaltungssystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18767-6">
    <skos:altLabel xml:lang="de">Dokumentenmanagementsystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18767-6">
    <skos:altLabel xml:lang="de">Dokumentenmanagement-System</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18767-6">
    <skos:altLabel xml:lang="de">Dokumenten-Management-System</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18767-6">
    <skos:altLabel xml:lang="de">Dokumentenmanagement System</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18767-6">
    <skos:altLabel xml:lang="de">DMS (Dokumentenmanagement-System)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16146-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12915-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18590-5">
    <skos:prefLabel xml:lang="en">Ship finance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18590-5">
    <skos:prefLabel xml:lang="de">Schifffahrtsfinanzierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15690-2">
    <skos:narrower rdf:resource="descriptor/15700-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15690-2">
    <skos:narrower rdf:resource="descriptor/15706-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15690-2">
    <skos:narrower rdf:resource="descriptor/15699-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15690-2">
    <skos:narrower rdf:resource="descriptor/13511-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15690-2">
    <skos:narrower rdf:resource="descriptor/10465-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15690-2">
    <skos:narrower rdf:resource="descriptor/15695-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15690-2">
    <skos:narrower rdf:resource="descriptor/15723-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15690-2">
    <skos:narrower rdf:resource="descriptor/15968-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15690-2">
    <skos:narrower rdf:resource="descriptor/15696-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15690-2">
    <skos:narrower rdf:resource="descriptor/15694-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15690-2">
    <skos:narrower rdf:resource="descriptor/15689-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11156-0">
    <skos:broader rdf:resource="thsys/70274"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11156-0">
    <skos:broader rdf:resource="thsys/70180"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11156-0">
    <skos:broader rdf:resource="descriptor/19213-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16312-1">
    <skos:broader rdf:resource="thsys/73382"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16312-1">
    <skos:broader rdf:resource="descriptor/16305-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19120-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20104-4">
    <skos:altLabel xml:lang="de">Taylor Regel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19323-2">
    <skos:scopeNote xml:lang="de">Nationale und internationale Bestimmungen zur Regelung des Aufenthaltsrechts sowie des Rechts zur Aufnahme und Ausübung einer beruflichen und gewerblichen Tätigkeit.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18260-5">
    <skos:altLabel xml:lang="en">Biotech industry</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12964-6">
    <skos:altLabel xml:lang="de">Seefischerei</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11879-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70051">
    <skos:broader rdf:resource="thsys/70052"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10514-5">
    <skos:altLabel xml:lang="de">Wirtschaftliche Unterentwicklung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16134-6">
    <skos:broader rdf:resource="thsys/73360"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16134-6">
    <skos:broader rdf:resource="thsys/70843"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16134-6">
    <skos:broader rdf:resource="thsys/70176"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16134-6">
    <skos:broader rdf:resource="descriptor/16019-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11464-3">
    <skos:broader rdf:resource="thsys/70306"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14377-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14635-1">
    <skos:broader rdf:resource="thsys/70014"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14635-1">
    <skos:broader rdf:resource="descriptor/14599-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15315-0">
    <skos:prefLabel xml:lang="en">Nonparametric statistics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15315-0">
    <skos:prefLabel xml:lang="de">Nichtparametrisches Verfahren</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19585-0">
    <skos:broader rdf:resource="thsys/72181"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19585-0">
    <skos:broader rdf:resource="thsys/71019"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19585-0">
    <skos:broader rdf:resource="descriptor/15076-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18421-0">
    <skos:related rdf:resource="descriptor/18585-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18421-0">
    <skos:related rdf:resource="descriptor/18422-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18421-0">
    <skos:related rdf:resource="descriptor/13051-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18421-0">
    <skos:related rdf:resource="descriptor/14433-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18421-0">
    <skos:related rdf:resource="descriptor/18423-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18631-3">
    <skos:related rdf:resource="descriptor/11876-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18631-3">
    <skos:related rdf:resource="descriptor/19463-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18631-3">
    <skos:related rdf:resource="descriptor/11864-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11454-6">
    <skos:altLabel xml:lang="de">Vorsichtskasse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11454-6">
    <skos:altLabel xml:lang="de">Transaktionskasse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11454-6">
    <skos:altLabel xml:lang="de">Spekulationskasse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11454-6">
    <skos:altLabel xml:lang="de">Kassenmittelansatz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11454-6">
    <skos:altLabel xml:lang="de">Kassenhaltungstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11454-6">
    <skos:altLabel xml:lang="de">Cambridge-Gleichung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10565-2">
    <skos:broader rdf:resource="thsys/71098"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10565-2">
    <skos:broader rdf:resource="thsys/71085"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10565-2">
    <skos:broader rdf:resource="descriptor/18481-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18298-1">
    <skos:related rdf:resource="descriptor/18590-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18298-1">
    <skos:related rdf:resource="descriptor/14455-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12786-4">
    <skos:altLabel xml:lang="en">Service management</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12786-4">
    <skos:altLabel xml:lang="de">Verkaufsgespräch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12786-4">
    <skos:altLabel xml:lang="de">Nachkaufmarketing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12786-4">
    <skos:altLabel xml:lang="de">Lieferservice</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12786-4">
    <skos:altLabel xml:lang="de">Kundenservice</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12786-4">
    <skos:altLabel xml:lang="de">Kundenbetreuung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12786-4">
    <skos:altLabel xml:lang="de">Kundenberatung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11409-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18570-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16698-2">
    <skos:broader rdf:resource="thsys/73357"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16698-2">
    <skos:broader rdf:resource="thsys/70814"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16698-2">
    <skos:broader rdf:resource="descriptor/16564-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13809-6">
    <skos:broader rdf:resource="thsys/70891"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13809-6">
    <skos:broader rdf:resource="descriptor/13816-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10077-1">
    <skos:related rdf:resource="descriptor/10156-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10077-1">
    <skos:related rdf:resource="descriptor/10148-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10077-1">
    <skos:related rdf:resource="descriptor/10221-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10077-1">
    <skos:related rdf:resource="descriptor/10085-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10077-1">
    <skos:related rdf:resource="descriptor/10179-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10077-1">
    <skos:related rdf:resource="descriptor/10210-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10077-1">
    <skos:related rdf:resource="descriptor/10073-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24707-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#broaderTransitive">
    <owl:inverseOf rdf:resource="http://www.w3.org/2004/02/skos/core#narrowerTransitive"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#note">
    <skos:definition xml:lang="en">A general note, for any purpose.</skos:definition>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14269-0">
    <skos:related rdf:resource="descriptor/14273-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14269-0">
    <skos:related rdf:resource="descriptor/13034-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14269-0">
    <skos:related rdf:resource="descriptor/14382-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14269-0">
    <skos:related rdf:resource="descriptor/14266-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13120-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73351">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70128">
    <skos:prefLabel xml:lang="de">G.05  Australien und südpazifischer Inselraum</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21082-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15635-3">
    <skos:prefLabel xml:lang="en">Biotechnology</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15635-3">
    <skos:prefLabel xml:lang="de">Biotechnologie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#mappingRelation">
    <rdfs:label xml:lang="en">mapping relation</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15019-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14831-1">
    <skos:altLabel xml:lang="de">Täschnerwaren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14831-1">
    <skos:altLabel xml:lang="de">Sattlerwaren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14831-1">
    <skos:altLabel xml:lang="de">Ledertasche</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14831-1">
    <skos:altLabel xml:lang="de">Koffer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14831-1">
    <skos:altLabel xml:lang="de">Handtasche</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18415-2">
    <skos:related rdf:resource="descriptor/18398-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18415-2">
    <skos:related rdf:resource="descriptor/14343-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18415-2">
    <skos:related rdf:resource="descriptor/12930-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70811">
    <skos:narrower rdf:resource="descriptor/18378-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70811">
    <skos:narrower rdf:resource="descriptor/18379-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70811">
    <skos:narrower rdf:resource="descriptor/18377-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70811">
    <skos:narrower rdf:resource="descriptor/12922-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70811">
    <skos:narrower rdf:resource="descriptor/15753-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70811">
    <skos:narrower rdf:resource="descriptor/19490-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70811">
    <skos:narrower rdf:resource="descriptor/11675-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70811">
    <skos:narrower rdf:resource="descriptor/15754-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70811">
    <skos:narrower rdf:resource="descriptor/18382-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70811">
    <skos:narrower rdf:resource="descriptor/15752-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70811">
    <skos:narrower rdf:resource="descriptor/18995-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70811">
    <skos:narrower rdf:resource="descriptor/10593-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70811">
    <skos:narrower rdf:resource="descriptor/14047-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70811">
    <skos:narrower rdf:resource="descriptor/18381-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70811">
    <skos:narrower rdf:resource="descriptor/15745-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70811">
    <skos:narrower rdf:resource="descriptor/15741-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70811">
    <skos:narrower rdf:resource="descriptor/19489-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70811">
    <skos:narrower rdf:resource="descriptor/18343-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70811">
    <skos:narrower rdf:resource="descriptor/18387-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70811">
    <skos:narrower rdf:resource="descriptor/15751-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70811">
    <skos:narrower rdf:resource="descriptor/18286-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70811">
    <skos:narrower rdf:resource="descriptor/18372-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70811">
    <skos:narrower rdf:resource="descriptor/10545-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70811">
    <skos:narrower rdf:resource="descriptor/18352-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70811">
    <skos:narrower rdf:resource="descriptor/19488-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15067-0">
    <skos:broader rdf:resource="thsys/71035"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15067-0">
    <skos:broader rdf:resource="descriptor/15060-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15067-0">
    <skos:broader rdf:resource="descriptor/15483-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72181">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13479-1">
    <skos:related rdf:resource="descriptor/13492-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13479-1">
    <skos:related rdf:resource="descriptor/13475-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13479-1">
    <skos:related rdf:resource="descriptor/19141-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13479-1">
    <skos:related rdf:resource="descriptor/18527-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13479-1">
    <skos:related rdf:resource="descriptor/14335-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70562">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">B.01.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18298-1">
    <skos:broader rdf:resource="thsys/70876"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18298-1">
    <skos:broader rdf:resource="descriptor/12283-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19360-3">
    <skos:related rdf:resource="descriptor/19839-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19360-3">
    <skos:related rdf:resource="descriptor/10815-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19360-3">
    <skos:related rdf:resource="descriptor/10813-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19360-3">
    <skos:related rdf:resource="descriptor/10786-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19360-3">
    <skos:related rdf:resource="descriptor/10613-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19360-3">
    <skos:related rdf:resource="descriptor/11268-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15768-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14028-1">
    <skos:broader rdf:resource="thsys/70201"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16268-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17911-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16619-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12357-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18737-1">
    <skos:narrower rdf:resource="descriptor/18475-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19854-6">
    <skos:altLabel xml:lang="de">Visumpflicht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19854-6">
    <skos:altLabel xml:lang="de">Sichtvermerk</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19854-6">
    <skos:altLabel xml:lang="de">Reiseverkehrsvorschriften</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19854-6">
    <skos:altLabel xml:lang="de">Reisepass</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19854-6">
    <skos:altLabel xml:lang="de">Reisepaß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19854-6">
    <skos:altLabel xml:lang="de">Reiseformalitäten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19854-6">
    <skos:altLabel xml:lang="de">Passrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19854-6">
    <skos:altLabel xml:lang="de">Pass</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19854-6">
    <skos:altLabel xml:lang="de">Paßrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19854-6">
    <skos:altLabel xml:lang="de">Paß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19854-6">
    <skos:altLabel xml:lang="de">Einreisebestimmungen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19854-6">
    <skos:altLabel xml:lang="de">Einreisebedingungen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19854-6">
    <skos:altLabel xml:lang="de">Ausreisebestimmungen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19854-6">
    <skos:altLabel xml:lang="de">Ausreisebedingungen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19420-4">
    <skos:altLabel xml:lang="en">Lump-sum tax</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19420-4">
    <skos:altLabel xml:lang="de">Pauschsteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19420-4">
    <skos:altLabel xml:lang="de">Pauschalsteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19420-4">
    <skos:altLabel xml:lang="de">Einheitssteuer (Kopfsteuer)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19797-6">
    <skos:related rdf:resource="descriptor/19798-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19797-6">
    <skos:related rdf:resource="descriptor/10455-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12458-0">
    <skos:related rdf:resource="descriptor/12337-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12458-0">
    <skos:related rdf:resource="descriptor/12325-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12458-0">
    <skos:related rdf:resource="descriptor/12217-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12458-0">
    <skos:related rdf:resource="descriptor/12216-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12458-0">
    <skos:related rdf:resource="descriptor/12357-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15979-6">
    <skos:related rdf:resource="descriptor/11217-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15979-6">
    <skos:related rdf:resource="descriptor/19763-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15979-6">
    <skos:related rdf:resource="descriptor/16198-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15979-6">
    <skos:related rdf:resource="descriptor/16504-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15979-6">
    <skos:related rdf:resource="descriptor/19511-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15979-6">
    <skos:related rdf:resource="descriptor/19813-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15979-6">
    <skos:related rdf:resource="descriptor/13995-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15979-6">
    <skos:related rdf:resource="descriptor/19704-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15979-6">
    <skos:related rdf:resource="descriptor/16195-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15979-6">
    <skos:related rdf:resource="descriptor/15962-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16476-6">
    <skos:narrower rdf:resource="descriptor/18992-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16709-2">
    <skos:broader rdf:resource="thsys/73394"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16709-2">
    <skos:broader rdf:resource="thsys/70120"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16709-2">
    <skos:broader rdf:resource="descriptor/15671-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16709-2">
    <skos:broader rdf:resource="descriptor/16719-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20018-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12867-4">
    <skos:altLabel xml:lang="de">Mitunternehmerschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12867-4">
    <skos:altLabel xml:lang="de">Mitunternehmerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70934">
    <skos:prefLabel xml:lang="en">W.12.01.03.02  Cargo shipping</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70934">
    <skos:prefLabel xml:lang="de">W.12.01.03.02  Frachtschifffahrt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10524-2">
    <skos:broader rdf:resource="thsys/73368"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10524-2">
    <skos:broader rdf:resource="thsys/71085"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10524-2">
    <skos:broader rdf:resource="descriptor/19163-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10524-2">
    <skos:broader rdf:resource="descriptor/10560-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15905-0">
    <skos:related rdf:resource="descriptor/19760-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19682-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70141">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14107-5">
    <skos:prefLabel xml:lang="en">Pome</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14107-5">
    <skos:prefLabel xml:lang="de">Kernobst</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10829-1">
    <zbwext:useInsteadNote rdf:nodeID="id157"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#broader">
    <owl:inverseOf rdf:resource="http://www.w3.org/2004/02/skos/core#narrower"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71119">
    <skos:broader rdf:resource="thsys/70180"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11667-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18200-2">
    <skos:related rdf:resource="descriptor/18189-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18200-2">
    <skos:related rdf:resource="descriptor/19365-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11063-0">
    <skos:scopeNote xml:lang="de">Vergleichende Studien verschiedener Wirtschafts- und/oder politischer Systeme.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11160-2">
    <skos:prefLabel xml:lang="en">New institutional economics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11160-2">
    <skos:prefLabel xml:lang="de">Neue Institutionenökonomik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16349-6">
    <skos:prefLabel xml:lang="en">European election</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16349-6">
    <skos:prefLabel xml:lang="de">Europawahl</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18550-3">
    <skos:narrower rdf:resource="descriptor/18515-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10145-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14712-2">
    <skos:prefLabel xml:lang="en">Polystyrene</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14712-2">
    <skos:prefLabel xml:lang="de">Polystyrol</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13083-5">
    <skos:broader rdf:resource="thsys/70995"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13083-5">
    <skos:broader rdf:resource="descriptor/12993-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13440-0">
    <skos:altLabel xml:lang="de">Landwirtschaftliches Familienunternehmen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13440-0">
    <skos:altLabel xml:lang="de">Landwirtschaftlicher Familienhaushalt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13440-0">
    <skos:altLabel xml:lang="de">Bauernhof</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13440-0">
    <skos:altLabel xml:lang="de">Bäuerlicher Familienbetrieb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13440-0">
    <skos:altLabel xml:lang="de">Agrarwirtschaftliches Familienunternehmen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13440-0">
    <skos:altLabel xml:lang="de">Agrarwirtschaftlicher Familienhaushalt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19806-3">
    <skos:related rdf:resource="descriptor/19809-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19806-3">
    <skos:related rdf:resource="descriptor/12733-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19806-3">
    <skos:related rdf:resource="descriptor/18170-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19050-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18335-0">
    <skos:prefLabel xml:lang="en">Food habit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18335-0">
    <skos:prefLabel xml:lang="de">Ernährungsgewohnheit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19811-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24622-4">
    <skos:broader rdf:resource="thsys/70180"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24622-4">
    <skos:broader rdf:resource="descriptor/15982-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19846-5">
    <skos:altLabel xml:lang="de">Konsumentenboykott</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19846-5">
    <skos:altLabel xml:lang="de">Käuferstreik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10835-6">
    <skos:altLabel xml:lang="de">Außenwirtschaftsstatistik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12069-4">
    <skos:related rdf:resource="descriptor/18115-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12069-4">
    <skos:related rdf:resource="descriptor/12072-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12069-4">
    <skos:related rdf:resource="descriptor/12057-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12069-4">
    <skos:related rdf:resource="descriptor/12046-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12069-4">
    <skos:related rdf:resource="descriptor/12039-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12069-4">
    <skos:related rdf:resource="descriptor/12053-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12847-3">
    <skos:altLabel xml:lang="de">Zeitungsbeilage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12847-3">
    <skos:altLabel xml:lang="de">Zeitungsanzeige</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12847-3">
    <skos:altLabel xml:lang="de">Zeitschriftenwerbung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12847-3">
    <skos:altLabel xml:lang="de">Werbeanzeige</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12847-3">
    <skos:altLabel xml:lang="de">Inserat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12847-3">
    <skos:altLabel xml:lang="de">Anzeigenwerbung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12847-3">
    <skos:altLabel xml:lang="de">Annonce</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12847-3">
    <skos:altLabel xml:lang="de">Adressbuchwerbung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12847-3">
    <skos:altLabel xml:lang="de">Adreßbuchwerbung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12287-1">
    <skos:altLabel xml:lang="de">Kapitalsammelstelle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12287-1">
    <skos:altLabel xml:lang="de">Institutioneller Investor</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14368-5">
    <skos:prefLabel xml:lang="en">Mining machinery</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14368-5">
    <skos:prefLabel xml:lang="de">Bergwerksmaschine</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10710-5">
    <skos:broader rdf:resource="thsys/71096"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10710-5">
    <skos:broader rdf:resource="thsys/71081"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10710-5">
    <skos:broader rdf:resource="thsys/71074"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10710-5">
    <skos:broader rdf:resource="descriptor/11545-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10710-5">
    <skos:broader rdf:resource="descriptor/10683-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10847-6">
    <skos:prefLabel xml:lang="en">Export processing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10847-6">
    <skos:prefLabel xml:lang="de">Veredelungsverkehr</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18077-3">
    <skos:related rdf:resource="descriptor/14958-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14056-3">
    <skos:prefLabel xml:lang="en">Groundnut</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14056-3">
    <skos:prefLabel xml:lang="de">Erdnuss</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13185-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19696-5">
    <skos:broader rdf:resource="thsys/73378"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19696-5">
    <skos:broader rdf:resource="descriptor/10968-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14078-0">
    <skos:broader rdf:resource="thsys/70341"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14078-0">
    <skos:broader rdf:resource="descriptor/14070-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18486-0">
    <skos:prefLabel xml:lang="en">Manufacturing employment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18486-0">
    <skos:prefLabel xml:lang="de">Industrielle Beschäftigung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16435-6">
    <skos:broader rdf:resource="thsys/73366"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16435-6">
    <skos:broader rdf:resource="descriptor/16273-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18340-0">
    <skos:prefLabel xml:lang="en">Tropical forest</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18340-0">
    <skos:prefLabel xml:lang="de">Tropischer Regenwald</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16370-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19652-4">
    <skos:broader rdf:resource="thsys/70893"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19652-4">
    <skos:broader rdf:resource="descriptor/18682-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19353-0">
    <skos:prefLabel xml:lang="en">Euro</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19353-0">
    <skos:prefLabel xml:lang="de">Euro</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18932-3">
    <skos:broader rdf:resource="thsys/70825"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18932-3">
    <skos:broader rdf:resource="thsys/70013"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18932-3">
    <skos:broader rdf:resource="descriptor/10300-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12254-2">
    <skos:narrower rdf:resource="descriptor/19057-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12254-2">
    <skos:narrower rdf:resource="descriptor/18369-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12254-2">
    <skos:narrower rdf:resource="descriptor/12030-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12254-2">
    <skos:narrower rdf:resource="descriptor/12243-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12254-2">
    <skos:narrower rdf:resource="descriptor/19066-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12254-2">
    <skos:narrower rdf:resource="descriptor/18758-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11792-0">
    <skos:prefLabel xml:lang="en">Sectoral crisis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11792-0">
    <skos:prefLabel xml:lang="de">Branchenkrise</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19829-5">
    <skos:related rdf:resource="descriptor/11572-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19829-5">
    <skos:related rdf:resource="descriptor/16552-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19829-5">
    <skos:related rdf:resource="descriptor/10973-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19829-5">
    <skos:related rdf:resource="descriptor/19446-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19829-5">
    <skos:related rdf:resource="descriptor/11820-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19829-5">
    <skos:related rdf:resource="descriptor/11817-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11800-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10342-1">
    <skos:related rdf:resource="descriptor/10031-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10342-1">
    <skos:related rdf:resource="descriptor/10343-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10342-1">
    <skos:related rdf:resource="descriptor/15075-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10342-1">
    <skos:related rdf:resource="descriptor/10025-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10342-1">
    <skos:related rdf:resource="descriptor/15715-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10342-1">
    <skos:related rdf:resource="descriptor/10513-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10342-1">
    <skos:related rdf:resource="descriptor/11270-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15018-6">
    <skos:broader rdf:resource="thsys/70238"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15018-6">
    <skos:broader rdf:resource="thsys/70161"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15018-6">
    <skos:broader rdf:resource="descriptor/15009-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13113-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10229-4">
    <skos:related rdf:resource="descriptor/10327-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10229-4">
    <skos:related rdf:resource="descriptor/11464-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10229-4">
    <skos:related rdf:resource="descriptor/10241-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10229-4">
    <skos:related rdf:resource="descriptor/10237-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10917-4">
    <skos:prefLabel xml:lang="en">Non-discrimination principle (Competition law)</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10917-4">
    <skos:prefLabel xml:lang="de">Diskriminierungsverbot (Wettbewerbsrecht)</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11981-4">
    <skos:related rdf:resource="descriptor/18031-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11981-4">
    <skos:related rdf:resource="descriptor/11973-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11981-4">
    <skos:related rdf:resource="descriptor/19446-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11981-4">
    <skos:related rdf:resource="descriptor/18489-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11981-4">
    <skos:related rdf:resource="descriptor/18748-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11981-4">
    <skos:related rdf:resource="descriptor/11817-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11981-4">
    <skos:related rdf:resource="descriptor/11940-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11981-4">
    <skos:related rdf:resource="descriptor/11905-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11981-4">
    <skos:related rdf:resource="descriptor/11904-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18252-4">
    <skos:narrower rdf:resource="descriptor/14469-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19363-4">
    <skos:related rdf:resource="descriptor/10512-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19363-4">
    <skos:related rdf:resource="descriptor/11972-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19363-4">
    <skos:related rdf:resource="descriptor/11983-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19363-4">
    <skos:related rdf:resource="descriptor/11988-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19363-4">
    <skos:related rdf:resource="descriptor/10510-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19363-4">
    <skos:related rdf:resource="descriptor/11943-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18500-4">
    <skos:broader rdf:resource="thsys/70969"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18500-4">
    <skos:broader rdf:resource="descriptor/13177-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13592-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17618-0">
    <skos:prefLabel xml:lang="en">Bahrain</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17618-0">
    <skos:prefLabel xml:lang="de">Bahrain</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id54">
    <rdfs:seeAlso rdf:resource="descriptor/15107-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10699-2">
    <skos:related rdf:resource="descriptor/10706-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10699-2">
    <skos:related rdf:resource="descriptor/10838-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12049-3">
    <skos:broader rdf:resource="thsys/71034"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12049-3">
    <skos:broader rdf:resource="descriptor/12072-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15555-1">
    <skos:narrower rdf:resource="descriptor/15437-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13065-0">
    <skos:prefLabel xml:lang="en">Road vehicle construction</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13065-0">
    <skos:prefLabel xml:lang="de">Straßenfahrzeugbau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18519-4">
    <skos:related rdf:resource="descriptor/19295-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18519-4">
    <skos:related rdf:resource="descriptor/19294-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18519-4">
    <skos:related rdf:resource="descriptor/18518-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18519-4">
    <skos:related rdf:resource="descriptor/18192-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18519-4">
    <skos:related rdf:resource="descriptor/18191-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18519-4">
    <skos:related rdf:resource="descriptor/14064-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16563-4">
    <skos:related rdf:resource="descriptor/18859-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16563-4">
    <skos:related rdf:resource="descriptor/15429-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16563-4">
    <skos:related rdf:resource="descriptor/14015-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16563-4">
    <skos:related rdf:resource="descriptor/16723-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16563-4">
    <skos:related rdf:resource="descriptor/15790-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16563-4">
    <skos:related rdf:resource="descriptor/19635-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16563-4">
    <skos:related rdf:resource="descriptor/15644-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17570-2">
    <skos:altLabel xml:lang="en">Republic of Korea</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17570-2">
    <skos:altLabel xml:lang="de">Republik Korea</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17570-2">
    <skos:altLabel xml:lang="de">Korea (Republik)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11231-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70847">
    <skos:prefLabel xml:lang="en">W.24  Education system</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70847">
    <skos:prefLabel xml:lang="de">W.24  Bildungswesen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18119-6">
    <skos:prefLabel xml:lang="en">Affiliate corporation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18119-6">
    <skos:prefLabel xml:lang="de">Tochtergesellschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16729-3">
    <skos:scopeNote xml:lang="de">Problematisierung des Verhältnisses von Theorie und Praxis in der Wissenschaft. Realitätsnähe theoretischer Annahmen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13117-0">
    <skos:broader rdf:resource="thsys/70990"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13117-0">
    <skos:broader rdf:resource="descriptor/12993-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10320-4">
    <skos:altLabel xml:lang="en">By-product</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10320-4">
    <skos:altLabel xml:lang="de">Verbundene Produktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10320-4">
    <skos:altLabel xml:lang="de">Nebenprodukt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10320-4">
    <skos:altLabel xml:lang="de">Kumulative Produktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18728-2">
    <skos:related rdf:resource="descriptor/13742-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18728-2">
    <skos:related rdf:resource="descriptor/18676-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18728-2">
    <skos:related rdf:resource="descriptor/18730-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18728-2">
    <skos:related rdf:resource="descriptor/13687-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15669-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19301-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12030-3">
    <skos:broader rdf:resource="thsys/71066"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12030-3">
    <skos:broader rdf:resource="thsys/70889"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12030-3">
    <skos:broader rdf:resource="thsys/70883"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12030-3">
    <skos:broader rdf:resource="descriptor/12254-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12030-3">
    <skos:broader rdf:resource="descriptor/18747-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18134-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71114">
    <skos:narrower rdf:resource="descriptor/21082-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71114">
    <skos:narrower rdf:resource="descriptor/11765-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71114">
    <skos:narrower rdf:resource="descriptor/11752-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71114">
    <skos:narrower rdf:resource="descriptor/11751-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71114">
    <skos:narrower rdf:resource="descriptor/19234-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71114">
    <skos:narrower rdf:resource="descriptor/11872-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71114">
    <skos:narrower rdf:resource="descriptor/10099-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71114">
    <skos:narrower rdf:resource="descriptor/11742-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71114">
    <skos:narrower rdf:resource="descriptor/19233-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71114">
    <skos:narrower rdf:resource="descriptor/11739-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71114">
    <skos:narrower rdf:resource="descriptor/11738-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71114">
    <skos:narrower rdf:resource="descriptor/11736-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71114">
    <skos:narrower rdf:resource="descriptor/11735-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71114">
    <skos:narrower rdf:resource="descriptor/11753-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71114">
    <skos:narrower rdf:resource="descriptor/11713-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16012-6">
    <skos:prefLabel xml:lang="en">Radio law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16012-6">
    <skos:prefLabel xml:lang="de">Rundfunkrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13699-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13067-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18753-3">
    <skos:prefLabel xml:lang="en">Urban housing construction</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18753-3">
    <skos:prefLabel xml:lang="de">Städtischer Wohnungsbau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15609-4">
    <skos:prefLabel xml:lang="en">Computer science</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15609-4">
    <skos:prefLabel xml:lang="de">Informatik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18216-1">
    <skos:prefLabel xml:lang="en">Food substitute</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18216-1">
    <skos:prefLabel xml:lang="de">Nahrungsmittelimitat</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12589-6">
    <skos:narrower rdf:resource="descriptor/11366-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15026-0">
    <skos:related rdf:resource="descriptor/15372-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15026-0">
    <skos:related rdf:resource="descriptor/10035-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20314-0">
    <skos:scopeNote xml:lang="de">Für Texte, die über Produkte informieren, BENUTZE Publikationsart Marktinformation.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14602-2">
    <skos:altLabel xml:lang="de">Verdünnungsmittel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14602-2">
    <skos:altLabel xml:lang="de">Trockenmittel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14602-2">
    <skos:altLabel xml:lang="de">Lösemittel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14602-2">
    <skos:altLabel xml:lang="de">Laborchemikalie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14602-2">
    <skos:altLabel xml:lang="de">Kautschukhilfsmittel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14602-2">
    <skos:altLabel xml:lang="de">Kältehilfsmittel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14602-2">
    <skos:altLabel xml:lang="de">Holzschutzmittel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14602-2">
    <skos:altLabel xml:lang="de">Frostschutzmittel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14602-2">
    <skos:altLabel xml:lang="de">Flammschutzmittel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14602-2">
    <skos:altLabel xml:lang="de">Feuerlöschmittel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14602-2">
    <skos:altLabel xml:lang="de">Feinchemikalie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14602-2">
    <skos:altLabel xml:lang="de">Elektronikchemikalie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14602-2">
    <skos:altLabel xml:lang="de">Chemisches Hilfsmittel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14602-2">
    <skos:altLabel xml:lang="de">Betonschutzmittel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14602-2">
    <skos:altLabel xml:lang="de">Bautenschutzmittel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14602-2">
    <skos:altLabel xml:lang="de">Antioxidantien</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14602-2">
    <skos:altLabel xml:lang="de">Additiv</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10030-6">
    <skos:related rdf:resource="descriptor/21061-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10030-6">
    <skos:related rdf:resource="descriptor/10016-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10030-6">
    <skos:related rdf:resource="descriptor/13301-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13202-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18428-0">
    <skos:broader rdf:resource="thsys/70996"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18428-0">
    <skos:broader rdf:resource="descriptor/24708-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18428-0">
    <skos:broader rdf:resource="descriptor/13076-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19035-0">
    <skos:broader rdf:resource="thsys/70582"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19603-3">
    <skos:prefLabel xml:lang="en">Higher education statistics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19603-3">
    <skos:prefLabel xml:lang="de">Hochschulstatistik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19131-4">
    <skos:related rdf:resource="descriptor/19132-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13161-4">
    <skos:broader rdf:resource="thsys/70986"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13161-4">
    <skos:broader rdf:resource="descriptor/13160-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70897">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70993">
    <skos:broader rdf:resource="thsys/70994"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13537-6">
    <skos:narrower rdf:resource="descriptor/18596-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15755-0">
    <skos:broader rdf:resource="thsys/73351"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15755-0">
    <skos:broader rdf:resource="thsys/70238"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15755-0">
    <skos:broader rdf:resource="descriptor/15763-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16480-1">
    <skos:broader rdf:resource="thsys/73370"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16480-1">
    <skos:broader rdf:resource="thsys/70833"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16480-1">
    <skos:broader rdf:resource="thsys/70324"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16480-1">
    <skos:broader rdf:resource="descriptor/16545-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15711-6">
    <skos:narrower rdf:resource="descriptor/16389-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15711-6">
    <skos:narrower rdf:resource="descriptor/16629-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15711-6">
    <skos:narrower rdf:resource="descriptor/16391-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15711-6">
    <skos:narrower rdf:resource="descriptor/16628-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15711-6">
    <skos:narrower rdf:resource="descriptor/16382-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15711-6">
    <skos:narrower rdf:resource="descriptor/16387-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14261-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70416">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19583-4">
    <skos:altLabel xml:lang="de">Eulersche Gleichung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19583-4">
    <skos:altLabel xml:lang="de">Eulersche Formel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19583-4">
    <skos:altLabel xml:lang="de">Eulergleichungsmodell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71003">
    <skos:broader rdf:resource="thsys/71004"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18832-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10773-2">
    <skos:prefLabel xml:lang="en">Exchange rate</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10773-2">
    <skos:prefLabel xml:lang="de">Wechselkurs</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15009-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13793-2">
    <skos:broader rdf:resource="thsys/70114"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13793-2">
    <skos:broader rdf:resource="descriptor/13807-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16384-4">
    <skos:altLabel xml:lang="de">Verbandsdemokratie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16384-4">
    <skos:altLabel xml:lang="de">Ständestaat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16384-4">
    <skos:altLabel xml:lang="de">Ständegesellschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16384-4">
    <skos:altLabel xml:lang="de">Neokorporatismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16384-4">
    <skos:altLabel xml:lang="de">Korporativismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13032-1">
    <skos:prefLabel xml:lang="en">Iron and steel industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13032-1">
    <skos:prefLabel xml:lang="de">Hüttenindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18980-6">
    <skos:broader rdf:resource="thsys/71028"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18980-6">
    <skos:broader rdf:resource="descriptor/19231-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12643-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12697-3">
    <zbwext:useInsteadNote rdf:nodeID="id366"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12697-3">
    <zbwext:useInsteadNote rdf:nodeID="id365"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70936">
    <skos:narrower rdf:resource="descriptor/13298-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70936">
    <skos:narrower rdf:resource="descriptor/18594-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70936">
    <skos:narrower rdf:resource="descriptor/18593-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70936">
    <skos:narrower rdf:resource="descriptor/18592-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70936">
    <skos:narrower rdf:resource="descriptor/13546-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70936">
    <skos:narrower rdf:resource="descriptor/13576-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70936">
    <skos:narrower rdf:resource="descriptor/18591-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70936">
    <skos:narrower rdf:resource="descriptor/13545-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70936">
    <skos:narrower rdf:resource="descriptor/18590-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70936">
    <skos:narrower rdf:resource="descriptor/13983-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70936">
    <skos:narrower rdf:resource="descriptor/13256-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70936">
    <skos:narrower rdf:resource="descriptor/13252-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70936">
    <skos:narrower rdf:resource="descriptor/18589-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70936">
    <skos:narrower rdf:resource="descriptor/14317-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70936">
    <skos:narrower rdf:resource="descriptor/18588-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70936">
    <skos:narrower rdf:resource="descriptor/18587-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70936">
    <skos:narrower rdf:resource="descriptor/13530-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70936">
    <skos:narrower rdf:resource="thsys/70933"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70936">
    <skos:narrower rdf:resource="thsys/70935"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70936">
    <skos:narrower rdf:resource="thsys/70934"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12354-5">
    <skos:altLabel xml:lang="de">Anlagevermögen des Betriebs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10693-0">
    <skos:related rdf:resource="descriptor/10866-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10693-0">
    <skos:related rdf:resource="descriptor/10836-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10693-0">
    <skos:related rdf:resource="descriptor/12236-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10693-0">
    <skos:related rdf:resource="descriptor/10858-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10693-0">
    <skos:related rdf:resource="descriptor/10854-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12357-6">
    <skos:prefLabel xml:lang="en">Balance sheet</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12357-6">
    <skos:prefLabel xml:lang="de">Bilanz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14293-3">
    <skos:prefLabel xml:lang="en">Aluminium</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14293-3">
    <skos:prefLabel xml:lang="de">Aluminium</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13029-4">
    <skos:altLabel xml:lang="de">Warmwalzwerk</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13029-4">
    <skos:altLabel xml:lang="de">Walzwerk</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13029-4">
    <skos:altLabel xml:lang="de">Stahlerzeugung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13029-4">
    <skos:altLabel xml:lang="de">Rohstahlerzeugung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13029-4">
    <skos:altLabel xml:lang="de">Roheisenerzeugung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13029-4">
    <skos:altLabel xml:lang="de">Eisenschaffende Industrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13029-4">
    <skos:altLabel xml:lang="de">Eisenindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13029-4">
    <skos:altLabel xml:lang="de">Eisen- und Stahlindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18009-6">
    <skos:altLabel xml:lang="en">Alcohol fuel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18009-6">
    <skos:altLabel xml:lang="de">Methanolkraftstoff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18009-6">
    <skos:altLabel xml:lang="de">Ethanolkraftstoff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18009-6">
    <skos:altLabel xml:lang="de">Biosprit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18009-6">
    <skos:altLabel xml:lang="de">Biodiesel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18009-6">
    <skos:altLabel xml:lang="de">Äthanolkraftstoff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18009-6">
    <skos:altLabel xml:lang="de">Alkoholischer Kraftstoff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73386">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11754-1">
    <skos:narrower rdf:resource="descriptor/10180-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11754-1">
    <skos:narrower rdf:resource="descriptor/12022-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71031">
    <rdfs:label xml:lang="en">Central Africa</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71031">
    <rdfs:label xml:lang="de">Zentralafrika</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14381-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18075-0">
    <skos:broader rdf:resource="thsys/73305"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18075-0">
    <skos:broader rdf:resource="descriptor/14514-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14381-6">
    <skos:prefLabel xml:lang="en">Machinery</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14381-6">
    <skos:prefLabel xml:lang="de">Maschine</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73299">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11529-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12980-1">
    <skos:broader rdf:resource="thsys/71009"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12980-1">
    <skos:broader rdf:resource="descriptor/12922-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12980-1">
    <skos:broader rdf:resource="descriptor/12913-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10897-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18767-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18619-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19270-4">
    <skos:prefLabel xml:lang="en">Financial market regulation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19270-4">
    <skos:prefLabel xml:lang="de">Börsenregulierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10489-6">
    <skos:broader rdf:resource="thsys/70182"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10489-6">
    <skos:broader rdf:resource="descriptor/15708-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11401-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14626-2">
    <skos:narrower rdf:resource="descriptor/18266-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19396-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19154-6">
    <skos:related rdf:resource="descriptor/12903-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19154-6">
    <skos:related rdf:resource="descriptor/12090-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19154-6">
    <skos:related rdf:resource="descriptor/20261-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19154-6">
    <skos:related rdf:resource="descriptor/19102-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19154-6">
    <skos:related rdf:resource="descriptor/12185-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19154-6">
    <skos:related rdf:resource="descriptor/12120-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19154-6">
    <skos:related rdf:resource="descriptor/19522-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19154-6">
    <skos:related rdf:resource="descriptor/10192-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19154-6">
    <skos:related rdf:resource="descriptor/19156-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19154-6">
    <skos:related rdf:resource="descriptor/12175-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19154-6">
    <skos:related rdf:resource="descriptor/13714-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19154-6">
    <skos:related rdf:resource="descriptor/11153-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10475-3">
    <skos:broader rdf:resource="thsys/71109"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10475-3">
    <skos:broader rdf:resource="descriptor/10466-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17977-0">
    <skos:broader rdf:resource="thsys/70307"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17977-0">
    <skos:broader rdf:resource="descriptor/10492-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11417-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10064-3">
    <skos:prefLabel xml:lang="en">Global model</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10064-3">
    <skos:prefLabel xml:lang="de">Weltmodell</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10542-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18961-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13236-6">
    <skos:broader rdf:resource="thsys/70957"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13236-6">
    <skos:broader rdf:resource="descriptor/13224-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70046">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18329-2">
    <skos:narrower rdf:resource="descriptor/19455-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15668-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11649-5">
    <skos:altLabel xml:lang="de">Umsatzausgleichsteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11917-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20077-2">
    <skos:prefLabel xml:lang="en">Social contract</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20077-2">
    <skos:prefLabel xml:lang="de">Sozialvertrag</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11446-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73301">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70372">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17580-6">
    <skos:altLabel xml:lang="de">Königreich Kambodscha</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17580-6">
    <skos:altLabel xml:lang="de">Khmer-Republik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17580-6">
    <skos:altLabel xml:lang="de">Kamputschea</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18850-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12985-5">
    <skos:broader rdf:resource="thsys/71007"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12985-5">
    <skos:broader rdf:resource="descriptor/12980-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16081-1">
    <skos:narrower rdf:resource="descriptor/11848-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16081-1">
    <skos:narrower rdf:resource="descriptor/16023-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16081-1">
    <skos:narrower rdf:resource="descriptor/11417-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16081-1">
    <skos:narrower rdf:resource="descriptor/13731-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16081-1">
    <skos:narrower rdf:resource="descriptor/16102-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16081-1">
    <skos:narrower rdf:resource="descriptor/10621-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16081-1">
    <skos:narrower rdf:resource="descriptor/16052-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16081-1">
    <skos:narrower rdf:resource="descriptor/18311-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16081-1">
    <skos:narrower rdf:resource="descriptor/18312-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16081-1">
    <skos:narrower rdf:resource="descriptor/15993-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16081-1">
    <skos:narrower rdf:resource="descriptor/19818-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16081-1">
    <skos:narrower rdf:resource="descriptor/18352-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16081-1">
    <skos:narrower rdf:resource="descriptor/13693-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16081-1">
    <skos:narrower rdf:resource="descriptor/10606-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16081-1">
    <skos:narrower rdf:resource="descriptor/13426-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10160-0">
    <skos:related rdf:resource="descriptor/10112-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18698-6">
    <skos:scopeNote xml:lang="de">Politische Maßnahmen des Staates und der Zentralbank zur Beeinflussung der Kreditkonditionen und der Kreditvergabe.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18607-0">
    <skos:broader rdf:resource="thsys/70932"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18607-0">
    <skos:broader rdf:resource="descriptor/13570-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13644-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11640-2">
    <skos:narrower rdf:resource="descriptor/12558-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11640-2">
    <skos:narrower rdf:resource="descriptor/12849-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11640-2">
    <skos:narrower rdf:resource="descriptor/16111-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11640-2">
    <skos:narrower rdf:resource="descriptor/12874-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11640-2">
    <skos:narrower rdf:resource="descriptor/16104-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11640-2">
    <skos:narrower rdf:resource="descriptor/10829-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11640-2">
    <skos:narrower rdf:resource="descriptor/18831-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11640-2">
    <skos:narrower rdf:resource="descriptor/16092-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11640-2">
    <skos:narrower rdf:resource="descriptor/18302-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11640-2">
    <skos:narrower rdf:resource="descriptor/10827-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12433-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16024-6">
    <skos:prefLabel xml:lang="en">Military law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16024-6">
    <skos:prefLabel xml:lang="de">Wehrrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10176-6">
    <skos:related rdf:resource="descriptor/18301-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10176-6">
    <skos:related rdf:resource="descriptor/18144-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10176-6">
    <skos:related rdf:resource="descriptor/19205-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10176-6">
    <skos:related rdf:resource="descriptor/10173-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10176-6">
    <skos:related rdf:resource="descriptor/11272-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10176-6">
    <skos:related rdf:resource="descriptor/19521-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11904-1">
    <skos:broader rdf:resource="thsys/71040"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11904-1">
    <skos:broader rdf:resource="thsys/71034"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11904-1">
    <skos:broader rdf:resource="descriptor/11915-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15668-2">
    <skos:broader rdf:resource="thsys/70819"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15668-2">
    <skos:broader rdf:resource="thsys/70581"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15668-2">
    <skos:broader rdf:resource="descriptor/15675-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12119-1">
    <skos:broader rdf:resource="thsys/73385"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12119-1">
    <skos:broader rdf:resource="thsys/73358"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12119-1">
    <skos:broader rdf:resource="thsys/71050"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12119-1">
    <skos:broader rdf:resource="descriptor/12621-1"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id208">
    <rdf:value xml:lang="de">Für rechtliche Regelungen bezüglich Rundfunk und Fernsehen</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70878">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18181-1">
    <skos:broader rdf:resource="thsys/71093"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18181-1">
    <skos:broader rdf:resource="descriptor/10626-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18181-1">
    <skos:broader rdf:resource="descriptor/16421-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17709-4">
    <skos:altLabel xml:lang="de">Republik Liberia</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17709-4">
    <skos:altLabel xml:lang="de">Liberien</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11305-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18216-1">
    <skos:broader rdf:resource="thsys/70074"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18216-1">
    <skos:broader rdf:resource="descriptor/14611-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18216-1">
    <skos:broader rdf:resource="descriptor/14926-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11676-2">
    <skos:prefLabel xml:lang="en">Capital levy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11676-2">
    <skos:prefLabel xml:lang="de">Substanzsteuer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14337-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11154-4">
    <skos:prefLabel xml:lang="en">Bureaucracy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11154-4">
    <skos:prefLabel xml:lang="de">Bürokratie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19607-2">
    <skos:related rdf:resource="descriptor/11068-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19607-2">
    <skos:related rdf:resource="descriptor/11063-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19607-2">
    <skos:related rdf:resource="descriptor/15976-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19607-2">
    <skos:related rdf:resource="descriptor/15972-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19607-2">
    <skos:related rdf:resource="descriptor/15970-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19607-2">
    <skos:related rdf:resource="descriptor/15966-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19607-2">
    <skos:related rdf:resource="descriptor/16292-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19607-2">
    <skos:related rdf:resource="descriptor/19822-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19607-2">
    <skos:related rdf:resource="descriptor/16258-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19607-2">
    <skos:related rdf:resource="descriptor/19841-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19607-2">
    <skos:related rdf:resource="descriptor/16602-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16042-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70057">
    <skos:broader rdf:resource="thsys/70049"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19603-3">
    <skos:altLabel xml:lang="de">Universitätsstatistik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70904">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11613-5">
    <skos:altLabel xml:lang="en">Fiscal debt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11613-5">
    <skos:altLabel xml:lang="de">Staatsverschuldung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11613-5">
    <skos:altLabel xml:lang="de">Staatsschulden</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11613-5">
    <skos:altLabel xml:lang="de">Schuldenpolitik des Staates</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11613-5">
    <skos:altLabel xml:lang="de">Öffentliches Darlehen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11613-5">
    <skos:altLabel xml:lang="de">Öffentlicher Kredit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11613-5">
    <skos:altLabel xml:lang="de">Öffentliche Verschuldung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11613-5">
    <skos:altLabel xml:lang="de">Öffentliche Schuldenpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70554">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12884-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19350-6">
    <skos:narrower rdf:resource="descriptor/16066-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13281-1">
    <skos:broader rdf:resource="thsys/70905"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16104-1">
    <skos:altLabel xml:lang="de">KStG</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16104-1">
    <skos:altLabel xml:lang="de">Körperschaftsteuergesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16104-1">
    <skos:altLabel xml:lang="de">Körperschaftssteuerrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11210-6">
    <skos:related rdf:resource="descriptor/11790-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70985">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19287-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10374-2">
    <skos:related rdf:resource="descriptor/19587-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10374-2">
    <skos:related rdf:resource="descriptor/10373-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18979-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10499-3">
    <skos:related rdf:resource="descriptor/16603-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10499-3">
    <skos:related rdf:resource="descriptor/10568-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19511-1">
    <skos:scopeNote xml:lang="de">Staatliche und private Fürsorgemaßnahmen für Straftäter/innen und ihre Angehörigen während und nach der Strafverbüßung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19784-1">
    <skos:narrower rdf:resource="descriptor/13591-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19784-1">
    <skos:narrower rdf:resource="descriptor/18678-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19784-1">
    <skos:narrower rdf:resource="descriptor/11194-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19784-1">
    <skos:narrower rdf:resource="descriptor/19074-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19784-1">
    <skos:narrower rdf:resource="descriptor/11187-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10565-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10052-3">
    <skos:related rdf:resource="descriptor/10220-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13024-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18291-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11676-2">
    <skos:narrower rdf:resource="descriptor/11682-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11676-2">
    <skos:narrower rdf:resource="descriptor/11659-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11676-2">
    <skos:narrower rdf:resource="descriptor/11651-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19451-0">
    <skos:related rdf:resource="descriptor/18490-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19451-0">
    <skos:related rdf:resource="descriptor/11940-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19451-0">
    <skos:related rdf:resource="descriptor/18060-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19451-0">
    <skos:related rdf:resource="descriptor/10822-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18652-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15336-6">
    <skos:broader rdf:resource="thsys/72394"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15336-6">
    <skos:broader rdf:resource="thsys/72186"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15336-6">
    <skos:broader rdf:resource="descriptor/15340-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15336-6">
    <skos:broader rdf:resource="descriptor/15405-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18984-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://zbw.eu/namespaces/skos-extensions/indexedItem">
    <rdfs:isDefinedBy rdf:resource="../namespaces/skos-extensions/skos-extensions.rdf"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13306-4">
    <skos:prefLabel xml:lang="en">Catering business</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13306-4">
    <skos:prefLabel xml:lang="de">Gastronomie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13888-5">
    <skos:related rdf:resource="descriptor/13959-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13888-5">
    <skos:related rdf:resource="descriptor/12996-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19144-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14961-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19571-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19817-5">
    <skos:prefLabel xml:lang="en">European environmental law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19817-5">
    <skos:prefLabel xml:lang="de">EU-Umweltrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18616-6">
    <skos:prefLabel xml:lang="en">Freight transport policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18616-6">
    <skos:prefLabel xml:lang="de">Güterverkehrspolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11378-3">
    <skos:broader rdf:resource="thsys/70844"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11378-3">
    <skos:broader rdf:resource="descriptor/15909-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18689-0">
    <skos:altLabel xml:lang="en">Stock Index derivatives</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18689-0">
    <skos:altLabel xml:lang="en">Index futures</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18689-0">
    <skos:altLabel xml:lang="de">SMI-Futures</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18689-0">
    <skos:altLabel xml:lang="de">Indexterminmarkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18689-0">
    <skos:altLabel xml:lang="de">Index-Terminmarkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18689-0">
    <skos:altLabel xml:lang="de">Indexoption</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18689-0">
    <skos:altLabel xml:lang="de">DAX-Futures</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18689-0">
    <skos:altLabel xml:lang="de">Aktienindex-Termingeschäft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16429-1">
    <skos:related rdf:resource="descriptor/16623-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16429-1">
    <skos:related rdf:resource="descriptor/15998-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16429-1">
    <skos:related rdf:resource="descriptor/19768-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16429-1">
    <skos:related rdf:resource="descriptor/24663-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16429-1">
    <skos:related rdf:resource="descriptor/18142-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16429-1">
    <skos:related rdf:resource="descriptor/16678-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16429-1">
    <skos:related rdf:resource="descriptor/15713-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16429-1">
    <skos:related rdf:resource="descriptor/11400-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11406-3">
    <skos:broader rdf:resource="thsys/70843"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11406-3">
    <skos:broader rdf:resource="thsys/70176"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11406-3">
    <skos:broader rdf:resource="descriptor/18884-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10736-1">
    <skos:prefLabel xml:lang="en">Flexible exchange rate</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10736-1">
    <skos:prefLabel xml:lang="de">Flexibler Wechselkurs</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11164-1">
    <skos:prefLabel xml:lang="en">Power (Social Sciences)</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11164-1">
    <skos:prefLabel xml:lang="de">Macht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19663-6">
    <skos:broader rdf:resource="thsys/73362"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19663-6">
    <skos:broader rdf:resource="thsys/71092"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19663-6">
    <skos:broader rdf:resource="descriptor/16495-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10851-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14686-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10586-1">
    <skos:broader rdf:resource="thsys/71097"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10586-1">
    <skos:broader rdf:resource="thsys/71084"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10586-1">
    <skos:broader rdf:resource="descriptor/10625-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18627-1">
    <skos:altLabel xml:lang="de">Gemeinsame Verkehrspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14014-5">
    <skos:related rdf:resource="descriptor/14015-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14014-5">
    <skos:related rdf:resource="descriptor/15674-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14014-5">
    <skos:related rdf:resource="descriptor/16543-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14014-5">
    <skos:related rdf:resource="descriptor/18906-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14014-5">
    <skos:related rdf:resource="descriptor/14012-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13475-2">
    <skos:altLabel xml:lang="en">Sales personnel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13475-2">
    <skos:altLabel xml:lang="de">Verkaufsfahrerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13475-2">
    <skos:altLabel xml:lang="de">Verkaufsfahrer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13475-2">
    <skos:altLabel xml:lang="de">Verkaufsberufe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13475-2">
    <skos:altLabel xml:lang="de">Verkäuferinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13475-2">
    <skos:altLabel xml:lang="de">Verkäufer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13475-2">
    <skos:altLabel xml:lang="de">Fachverkäuferinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13475-2">
    <skos:altLabel xml:lang="de">Fachverkäufer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70998">
    <skos:broader rdf:resource="thsys/70999"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12521-5">
    <skos:narrower rdf:resource="descriptor/12520-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12521-5">
    <skos:narrower rdf:resource="descriptor/12513-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14488-2">
    <zbwext:useInsteadNote rdf:nodeID="id193"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16724-6">
    <skos:prefLabel xml:lang="en">Structuralism</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16724-6">
    <skos:prefLabel xml:lang="de">Strukturalismus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19297-5">
    <skos:prefLabel xml:lang="en">Rice market</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19297-5">
    <skos:prefLabel xml:lang="de">Reismarkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12424-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16202-1">
    <skos:broader rdf:resource="thsys/73390"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16202-1">
    <skos:broader rdf:resource="thsys/73334"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16202-1">
    <skos:broader rdf:resource="descriptor/16198-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18179-2">
    <skos:altLabel xml:lang="en">OPAC (Online Public Access Catalogue)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18179-2">
    <skos:altLabel xml:lang="de">Online-Publikumskatalog</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18179-2">
    <skos:altLabel xml:lang="de">Online-Benutzerkatalog</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18843-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15872-3">
    <skos:narrower rdf:resource="descriptor/15902-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15872-3">
    <skos:narrower rdf:resource="descriptor/11987-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15872-3">
    <skos:narrower rdf:resource="descriptor/19611-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15872-3">
    <skos:narrower rdf:resource="descriptor/11957-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15872-3">
    <skos:narrower rdf:resource="descriptor/15874-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16010-3">
    <skos:broader rdf:resource="thsys/73360"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16010-3">
    <skos:broader rdf:resource="thsys/73296"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16010-3">
    <skos:broader rdf:resource="thsys/70825"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16010-3">
    <skos:broader rdf:resource="descriptor/13660-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19784-1">
    <zbwext:useInsteadNote rdf:nodeID="id384"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19784-1">
    <zbwext:useInsteadNote rdf:nodeID="id383"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14450-6">
    <skos:prefLabel xml:lang="en">Warship</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14450-6">
    <skos:prefLabel xml:lang="de">Kriegsschiff</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12855-4">
    <skos:related rdf:resource="descriptor/11709-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14586-2">
    <skos:related rdf:resource="descriptor/13097-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14586-2">
    <skos:related rdf:resource="descriptor/13235-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14586-2">
    <skos:related rdf:resource="descriptor/15675-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14586-2">
    <skos:related rdf:resource="descriptor/14581-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14586-2">
    <skos:related rdf:resource="descriptor/14029-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14586-2">
    <skos:related rdf:resource="descriptor/14446-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18435-3">
    <skos:prefLabel xml:lang="en">Textile processing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18435-3">
    <skos:prefLabel xml:lang="de">Textiltechnik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70498">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19660-5">
    <skos:broader rdf:resource="thsys/73342"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19660-5">
    <skos:broader rdf:resource="thsys/71036"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12980-1">
    <skos:altLabel xml:lang="en">Minerals industries</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12980-1">
    <skos:altLabel xml:lang="de">Montanindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12980-1">
    <skos:altLabel xml:lang="de">Mineralbergbau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18118-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15971-1">
    <skos:broader rdf:resource="thsys/70154"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14464-2">
    <skos:broader rdf:resource="thsys/70135"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14464-2">
    <skos:broader rdf:resource="descriptor/14473-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13210-3">
    <skos:prefLabel xml:lang="en">Trade</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13210-3">
    <skos:prefLabel xml:lang="de">Handel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14971-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16277-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16361-2">
    <skos:prefLabel xml:lang="en">Communist party</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16361-2">
    <skos:prefLabel xml:lang="de">Kommunistische Partei</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18314-1">
    <skos:prefLabel xml:lang="en">Malnutrition</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18314-1">
    <skos:prefLabel xml:lang="de">Unterernährung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10537-0">
    <skos:altLabel xml:lang="de">Stufentheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10537-0">
    <skos:altLabel xml:lang="de">Rostowsche Stufentheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10537-0">
    <skos:altLabel xml:lang="de">Entwicklungsstadium</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10537-0">
    <skos:altLabel xml:lang="de">Entwicklungsphase</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11965-2">
    <skos:broader rdf:resource="thsys/72181"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11965-2">
    <skos:broader rdf:resource="thsys/71064"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11965-2">
    <skos:broader rdf:resource="descriptor/19072-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10037-6">
    <skos:broader rdf:resource="thsys/70878"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10037-6">
    <skos:broader rdf:resource="thsys/70175"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10037-6">
    <skos:broader rdf:resource="descriptor/10032-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10037-6">
    <skos:broader rdf:resource="descriptor/16554-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12088-0">
    <skos:related rdf:resource="descriptor/12087-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12088-0">
    <skos:related rdf:resource="descriptor/15483-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12088-0">
    <skos:related rdf:resource="descriptor/12085-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12088-0">
    <skos:related rdf:resource="descriptor/12884-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12088-0">
    <skos:related rdf:resource="descriptor/12053-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18717-0">
    <skos:narrower rdf:resource="descriptor/18743-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18717-0">
    <skos:narrower rdf:resource="descriptor/18531-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18717-0">
    <skos:narrower rdf:resource="descriptor/18716-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19033-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11230-0">
    <skos:altLabel xml:lang="en">Dual labour market theory</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11230-0">
    <skos:altLabel xml:lang="en">Dual labor market hypothesis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11230-0">
    <skos:altLabel xml:lang="en">Dual labor market</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11230-0">
    <skos:altLabel xml:lang="de">Duale Arbeitsmarkttheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11663-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15775-1">
    <skos:broader rdf:resource="thsys/70239"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13643-0">
    <skos:prefLabel xml:lang="en">Package tourism</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13643-0">
    <skos:prefLabel xml:lang="de">Pauschaltourismus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10981-2">
    <skos:altLabel xml:lang="de">Sektorpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10981-2">
    <skos:altLabel xml:lang="de">Sektorale Wirtschaftspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10981-2">
    <skos:altLabel xml:lang="de">Sektorale Strukturpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10981-2">
    <skos:altLabel xml:lang="de">Branchenpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18545-3">
    <skos:related rdf:resource="descriptor/13488-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18545-3">
    <skos:related rdf:resource="descriptor/13200-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10872-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13306-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17743-4">
    <skos:prefLabel xml:lang="en">Central Africa</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17743-4">
    <skos:prefLabel xml:lang="de">Zentralafrika</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16023-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19681-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15935-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19503-0">
    <skos:related rdf:resource="descriptor/16038-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19503-0">
    <skos:related rdf:resource="descriptor/16484-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19503-0">
    <skos:related rdf:resource="descriptor/11349-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19503-0">
    <skos:related rdf:resource="descriptor/18123-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19503-0">
    <skos:related rdf:resource="descriptor/11313-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#narrower">
    <rdf:type rdf:resource="http://www.w3.org/1999/02/22-rdf-syntax-ns#Property"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#narrower">
    <rdf:type rdf:resource="http://www.w3.org/2002/07/owl#ObjectProperty"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#broaderTransitive">
    <rdfs:label xml:lang="en">broader transitive</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13469-4">
    <skos:altLabel xml:lang="de">Verbundgruppe im Handel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13469-4">
    <skos:altLabel xml:lang="de">Ladengemeinschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13469-4">
    <skos:altLabel xml:lang="de">Kooperation im Handel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13469-4">
    <skos:altLabel xml:lang="de">Handelsverbundsystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13469-4">
    <skos:altLabel xml:lang="de">Handelsverbundgruppe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13469-4">
    <skos:altLabel xml:lang="de">Gemeinschaftswarenhaus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11158-3">
    <skos:broader rdf:resource="thsys/73395"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11158-3">
    <skos:broader rdf:resource="thsys/70562"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13269-5">
    <skos:scopeNote xml:lang="de">Datennetz in einem begrenzten geographischen Gebiet, meist im Besitz und Gebrauch einer einzelnen Organisation, z.B. eines Unternehmens oder einer Verwaltung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11563-1">
    <skos:broader rdf:resource="thsys/70967"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11563-1">
    <skos:broader rdf:resource="thsys/70899"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11563-1">
    <skos:broader rdf:resource="thsys/70888"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11563-1">
    <skos:broader rdf:resource="descriptor/11845-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11563-1">
    <skos:broader rdf:resource="descriptor/18752-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11563-1">
    <skos:broader rdf:resource="descriptor/11571-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15874-6">
    <skos:prefLabel xml:lang="en">Geomorphology</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15874-6">
    <skos:prefLabel xml:lang="de">Geomorphologie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12919-4">
    <skos:broader rdf:resource="thsys/70989"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12919-4">
    <skos:broader rdf:resource="descriptor/12913-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10412-6">
    <skos:altLabel xml:lang="de">Stabilisierungskrise</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19787-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10904-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11804-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19111-3">
    <skos:related rdf:resource="descriptor/11239-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19111-3">
    <skos:related rdf:resource="descriptor/18140-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19111-3">
    <skos:related rdf:resource="descriptor/11246-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19111-3">
    <skos:related rdf:resource="descriptor/11266-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19111-3">
    <skos:related rdf:resource="descriptor/12607-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19111-3">
    <skos:related rdf:resource="descriptor/11303-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13565-1">
    <skos:related rdf:resource="descriptor/13299-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13565-1">
    <skos:related rdf:resource="descriptor/16070-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13565-1">
    <skos:related rdf:resource="descriptor/18616-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13565-1">
    <skos:related rdf:resource="descriptor/18615-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13565-1">
    <skos:related rdf:resource="descriptor/13621-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18848-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14643-2">
    <skos:prefLabel xml:lang="en">Magnesite</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14643-2">
    <skos:prefLabel xml:lang="de">Magnesit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10610-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19837-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19956-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18032-4">
    <skos:altLabel xml:lang="de">Verkauf des Betriebs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18032-4">
    <skos:altLabel xml:lang="de">Unternehmensverkauf</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18032-4">
    <skos:altLabel xml:lang="de">Betriebsveräußerung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18032-4">
    <skos:altLabel xml:lang="de">Betriebsübertragung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18032-4">
    <skos:altLabel xml:lang="de">Betriebsüberlassung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14267-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18560-0">
    <skos:altLabel xml:lang="en">Meals on wheels</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18560-0">
    <skos:altLabel xml:lang="de">Partyservice</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18560-0">
    <skos:altLabel xml:lang="de">Essen auf Rädern</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18560-0">
    <skos:altLabel xml:lang="de">Caterer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12296-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70931">
    <skos:broader rdf:resource="thsys/70932"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70905">
    <skos:narrower rdf:resource="descriptor/18670-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70905">
    <skos:narrower rdf:resource="descriptor/13281-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70905">
    <skos:narrower rdf:resource="descriptor/18672-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70905">
    <skos:narrower rdf:resource="descriptor/13801-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70905">
    <skos:narrower rdf:resource="descriptor/13731-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70905">
    <skos:narrower rdf:resource="descriptor/12287-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70905">
    <skos:narrower rdf:resource="descriptor/18673-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70905">
    <skos:narrower rdf:resource="descriptor/18671-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70905">
    <skos:narrower rdf:resource="descriptor/13723-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70905">
    <skos:narrower rdf:resource="descriptor/11414-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70905">
    <skos:narrower rdf:resource="descriptor/11441-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70905">
    <skos:narrower rdf:resource="descriptor/12243-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70905">
    <skos:narrower rdf:resource="descriptor/18085-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70905">
    <skos:narrower rdf:resource="descriptor/18668-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70905">
    <skos:narrower rdf:resource="descriptor/18703-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70905">
    <skos:narrower rdf:resource="descriptor/13694-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70905">
    <skos:narrower rdf:resource="descriptor/13693-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70905">
    <skos:narrower rdf:resource="descriptor/18709-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70905">
    <skos:narrower rdf:resource="descriptor/13691-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70905">
    <skos:narrower rdf:resource="descriptor/13773-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70905">
    <skos:narrower rdf:resource="descriptor/13689-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70905">
    <skos:narrower rdf:resource="descriptor/18708-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70905">
    <skos:narrower rdf:resource="descriptor/18707-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70905">
    <skos:narrower rdf:resource="descriptor/13688-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70905">
    <skos:narrower rdf:resource="descriptor/13687-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70905">
    <skos:narrower rdf:resource="descriptor/13965-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70905">
    <skos:narrower rdf:resource="descriptor/13283-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70905">
    <skos:narrower rdf:resource="thsys/70900"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70905">
    <skos:narrower rdf:resource="thsys/70903"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70905">
    <skos:narrower rdf:resource="thsys/70904"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10500-2">
    <skos:narrower rdf:resource="descriptor/18459-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17707-1">
    <skos:prefLabel xml:lang="en">Cape Verde</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17707-1">
    <skos:prefLabel xml:lang="de">Kap Verde</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12287-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18317-2">
    <skos:related rdf:resource="descriptor/12956-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18317-2">
    <skos:related rdf:resource="descriptor/12951-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18317-2">
    <skos:related rdf:resource="descriptor/13450-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11710-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70278">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14368-5">
    <skos:altLabel xml:lang="de">Bergbaumaschine</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14368-5">
    <skos:altLabel xml:lang="de">Bergbauausrüstung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10251-4">
    <skos:broader rdf:resource="thsys/70234"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10251-4">
    <skos:broader rdf:resource="descriptor/10249-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10251-4">
    <skos:broader rdf:resource="descriptor/10247-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16454-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12079-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13957-5">
    <skos:related rdf:resource="descriptor/13960-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13957-5">
    <skos:related rdf:resource="descriptor/13871-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13957-5">
    <skos:related rdf:resource="descriptor/18787-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13957-5">
    <skos:related rdf:resource="descriptor/13953-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13957-5">
    <skos:related rdf:resource="descriptor/13376-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13957-5">
    <skos:related rdf:resource="descriptor/18492-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18281-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17758-5">
    <skos:prefLabel xml:lang="en">America</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17758-5">
    <skos:prefLabel xml:lang="de">Amerika</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18135-1">
    <skos:broader rdf:resource="thsys/70361"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18135-1">
    <skos:broader rdf:resource="descriptor/18012-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18161-0">
    <skos:altLabel xml:lang="en">Statistical services</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18161-0">
    <skos:altLabel xml:lang="en">Bureau of statistics</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18161-0">
    <skos:altLabel xml:lang="de">Statistische Behörde</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20810-2">
    <skos:broader rdf:resource="thsys/71092"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20810-2">
    <skos:broader rdf:resource="descriptor/19354-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13816-2">
    <skos:altLabel xml:lang="de">Hausratversicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13816-2">
    <skos:altLabel xml:lang="de">Glasversicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13816-2">
    <skos:altLabel xml:lang="de">Einbruchdiebstahlversicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14969-0">
    <skos:broader rdf:resource="thsys/73301"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14969-0">
    <skos:broader rdf:resource="descriptor/14129-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14969-0">
    <skos:broader rdf:resource="descriptor/14931-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14969-0">
    <skos:broader rdf:resource="descriptor/14161-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13387-6">
    <skos:altLabel xml:lang="de">Service-Rechenzentrum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15480-6">
    <skos:broader rdf:resource="thsys/73103"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15480-6">
    <skos:broader rdf:resource="descriptor/15467-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11819-6">
    <skos:altLabel xml:lang="de">Handwerksförderung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11819-6">
    <skos:altLabel xml:lang="de">Förderung des Handwerks</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10251-4">
    <skos:narrower rdf:resource="descriptor/11929-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18582-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12648-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13473-6">
    <skos:prefLabel xml:lang="en">Self-service</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13473-6">
    <skos:prefLabel xml:lang="de">Selbstbedienung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18605-4">
    <skos:broader rdf:resource="thsys/73389"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18605-4">
    <skos:broader rdf:resource="thsys/73332"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18605-4">
    <skos:broader rdf:resource="thsys/70932"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18605-4">
    <skos:broader rdf:resource="descriptor/16170-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18605-4">
    <skos:broader rdf:resource="descriptor/18567-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18574-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19134-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16017-3">
    <skos:narrower rdf:resource="descriptor/15983-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16017-3">
    <skos:narrower rdf:resource="descriptor/19703-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16017-3">
    <skos:narrower rdf:resource="descriptor/19687-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16017-3">
    <skos:narrower rdf:resource="descriptor/16309-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16017-3">
    <skos:narrower rdf:resource="descriptor/16008-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16017-3">
    <skos:narrower rdf:resource="descriptor/15998-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16017-3">
    <skos:narrower rdf:resource="descriptor/19649-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70477">
    <skos:narrower rdf:resource="descriptor/14512-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70477">
    <skos:narrower rdf:resource="descriptor/14511-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70477">
    <skos:narrower rdf:resource="descriptor/14561-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70477">
    <skos:narrower rdf:resource="descriptor/14510-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70477">
    <skos:narrower rdf:resource="descriptor/14509-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70477">
    <skos:narrower rdf:resource="descriptor/14554-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70477">
    <skos:narrower rdf:resource="descriptor/14507-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70477">
    <skos:narrower rdf:resource="descriptor/14504-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70477">
    <skos:narrower rdf:resource="descriptor/14502-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14545-2">
    <skos:related rdf:resource="descriptor/14561-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17181-6">
    <skos:prefLabel xml:lang="en">Hungary</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17181-6">
    <skos:prefLabel xml:lang="de">Ungarn</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19773-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16325-6">
    <skos:altLabel xml:lang="de">Luftstreitkräfte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18655-3">
    <skos:narrower rdf:resource="descriptor/13666-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18655-3">
    <skos:narrower rdf:resource="descriptor/18771-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18655-3">
    <skos:narrower rdf:resource="descriptor/18659-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18655-3">
    <skos:narrower rdf:resource="descriptor/18660-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18655-3">
    <skos:narrower rdf:resource="descriptor/18712-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18655-3">
    <skos:narrower rdf:resource="descriptor/18763-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15666-6">
    <skos:altLabel xml:lang="de">Soziologie der Erziehung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15666-6">
    <skos:altLabel xml:lang="de">Pädagogische Soziologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15666-6">
    <skos:altLabel xml:lang="de">Erziehungswissenschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15666-6">
    <skos:altLabel xml:lang="de">Erziehungssoziologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15666-6">
    <skos:altLabel xml:lang="de">Bildungstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15994-3">
    <skos:broader rdf:resource="thsys/73361"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15994-3">
    <skos:broader rdf:resource="descriptor/15998-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16160-5">
    <skos:prefLabel xml:lang="en">Social security legislation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16160-5">
    <skos:prefLabel xml:lang="de">Sozialversicherungsrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19138-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11741-3">
    <skos:related rdf:resource="descriptor/12574-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11741-3">
    <skos:related rdf:resource="descriptor/12594-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11741-3">
    <skos:related rdf:resource="descriptor/15839-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11741-3">
    <skos:related rdf:resource="descriptor/11037-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16927-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12363-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13308-0">
    <skos:broader rdf:resource="thsys/70944"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13308-0">
    <skos:broader rdf:resource="descriptor/13307-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19566-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11220-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14633-5">
    <skos:prefLabel xml:lang="en">Wax</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14633-5">
    <skos:prefLabel xml:lang="de">Wachs</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14962-0">
    <skos:prefLabel xml:lang="en">Animal fat</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14962-0">
    <skos:prefLabel xml:lang="de">Tierfett</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12721-4">
    <skos:broader rdf:resource="thsys/71005"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12721-4">
    <skos:broader rdf:resource="descriptor/19231-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14082-2">
    <zbwext:useInsteadNote rdf:nodeID="id324"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18024-3">
    <skos:narrower rdf:resource="descriptor/18231-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18024-3">
    <skos:narrower rdf:resource="descriptor/14189-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18024-3">
    <skos:narrower rdf:resource="descriptor/14186-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18024-3">
    <skos:narrower rdf:resource="descriptor/14180-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18024-3">
    <skos:narrower rdf:resource="descriptor/18203-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18024-3">
    <skos:narrower rdf:resource="descriptor/14171-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70575">
    <skos:broader rdf:resource="thsys/w"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14407-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10056-2">
    <skos:altLabel xml:lang="de">Rationale Erwartungsbildung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10056-2">
    <skos:altLabel xml:lang="de">Rationale Erwartungen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10056-2">
    <skos:altLabel xml:lang="de">Ratex</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10613-3">
    <skos:prefLabel xml:lang="en">International labour mobility</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10613-3">
    <skos:prefLabel xml:lang="de">Internationale Arbeitsmobilität</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18823-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71070">
    <rdfs:label xml:lang="en">Regional economics</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71070">
    <rdfs:label xml:lang="de">Regionalökonomik</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13426-1">
    <skos:altLabel xml:lang="de">Landwirtschaftsrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13426-1">
    <skos:altLabel xml:lang="de">Flurbereinigungsgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13426-1">
    <skos:altLabel xml:lang="de">Agrargesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19423-5">
    <skos:broader rdf:resource="thsys/71074"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19423-5">
    <skos:broader rdf:resource="descriptor/19422-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19239-5">
    <skos:prefLabel xml:lang="en">Real business cycle</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19239-5">
    <skos:prefLabel xml:lang="de">Real Business Cycle</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16545-6">
    <skos:narrower rdf:resource="descriptor/18754-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16545-6">
    <skos:narrower rdf:resource="descriptor/16542-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16545-6">
    <skos:narrower rdf:resource="descriptor/16540-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16545-6">
    <skos:narrower rdf:resource="descriptor/16480-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16545-6">
    <skos:narrower rdf:resource="descriptor/19509-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16545-6">
    <skos:narrower rdf:resource="descriptor/18142-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16545-6">
    <skos:narrower rdf:resource="descriptor/16477-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16545-6">
    <skos:narrower rdf:resource="descriptor/18909-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16545-6">
    <skos:narrower rdf:resource="descriptor/16476-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16545-6">
    <skos:narrower rdf:resource="descriptor/15911-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16545-6">
    <skos:narrower rdf:resource="descriptor/12592-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16545-6">
    <skos:narrower rdf:resource="descriptor/16529-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16545-6">
    <skos:narrower rdf:resource="descriptor/16533-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16545-6">
    <skos:narrower rdf:resource="descriptor/16475-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10265-0">
    <skos:broader rdf:resource="thsys/71111"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10265-0">
    <skos:broader rdf:resource="descriptor/10272-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14270-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13716-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17705-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13362-1">
    <skos:broader rdf:resource="thsys/70819"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12357-6">
    <skos:broader rdf:resource="thsys/71043"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12357-6">
    <skos:broader rdf:resource="descriptor/12381-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19084-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10627-6">
    <skos:related rdf:resource="descriptor/10628-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10627-6">
    <skos:related rdf:resource="descriptor/10771-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10627-6">
    <skos:related rdf:resource="descriptor/10969-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10627-6">
    <skos:related rdf:resource="descriptor/19351-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10627-6">
    <skos:related rdf:resource="descriptor/10612-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16050-5">
    <skos:prefLabel xml:lang="en">Corporate law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16050-5">
    <skos:prefLabel xml:lang="de">Gesellschaftsrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13004-6">
    <skos:related rdf:resource="descriptor/13142-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13004-6">
    <skos:related rdf:resource="descriptor/12919-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13004-6">
    <skos:related rdf:resource="descriptor/19366-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13004-6">
    <skos:related rdf:resource="descriptor/19365-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13004-6">
    <skos:related rdf:resource="descriptor/14151-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13004-6">
    <skos:related rdf:resource="descriptor/14817-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13004-6">
    <skos:related rdf:resource="descriptor/12996-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16329-5">
    <skos:related rdf:resource="descriptor/16233-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16329-5">
    <skos:related rdf:resource="descriptor/16305-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16329-5">
    <skos:related rdf:resource="descriptor/16306-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16329-5">
    <skos:related rdf:resource="descriptor/13989-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16329-5">
    <skos:related rdf:resource="descriptor/13413-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16329-5">
    <skos:related rdf:resource="descriptor/20855-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16329-5">
    <skos:related rdf:resource="descriptor/20842-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16329-5">
    <skos:related rdf:resource="descriptor/20826-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16329-5">
    <skos:related rdf:resource="descriptor/16315-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14042-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10981-2">
    <skos:broader rdf:resource="thsys/73368"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10981-2">
    <skos:broader rdf:resource="thsys/71110"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10981-2">
    <skos:broader rdf:resource="thsys/70575"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10981-2">
    <skos:broader rdf:resource="descriptor/10987-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17610-2">
    <skos:prefLabel xml:lang="en">Pakistan</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17610-2">
    <skos:prefLabel xml:lang="de">Pakistan</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19449-1">
    <skos:broader rdf:resource="thsys/73351"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19449-1">
    <skos:broader rdf:resource="thsys/71068"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19449-1">
    <skos:broader rdf:resource="thsys/70239"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19449-1">
    <skos:broader rdf:resource="descriptor/19483-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19449-1">
    <skos:broader rdf:resource="descriptor/11900-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13441-5">
    <skos:altLabel xml:lang="en">Crop insurance</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13441-5">
    <skos:altLabel xml:lang="de">Viehversicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13441-5">
    <skos:altLabel xml:lang="de">Ernteversicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13441-5">
    <skos:altLabel xml:lang="de">Agrarversicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12087-2">
    <skos:altLabel xml:lang="de">Unternehmensplanspiel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12087-2">
    <skos:altLabel xml:lang="de">Management-Planspiel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14163-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16771-4">
    <skos:prefLabel xml:lang="en">Library science</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16771-4">
    <skos:prefLabel xml:lang="de">Bibliothekswissenschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13437-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12490-4">
    <skos:prefLabel xml:lang="en">Direct costs</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12490-4">
    <skos:prefLabel xml:lang="de">Einzelkosten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17584-5">
    <skos:prefLabel xml:lang="en">Malaysia</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17584-5">
    <skos:prefLabel xml:lang="de">Malaysia</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13848-3">
    <skos:prefLabel xml:lang="en">Housing investment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13848-3">
    <skos:prefLabel xml:lang="de">Wohnungsbauinvestition</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18443-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12277-4">
    <skos:prefLabel xml:lang="en">Creditor protection</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12277-4">
    <skos:prefLabel xml:lang="de">Gläubigerschutz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11824-6">
    <skos:broader rdf:resource="thsys/70963"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11824-6">
    <skos:broader rdf:resource="descriptor/11782-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19725-3">
    <skos:related rdf:resource="descriptor/18366-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19725-3">
    <skos:related rdf:resource="descriptor/16024-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19725-3">
    <skos:related rdf:resource="descriptor/16458-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19725-3">
    <skos:related rdf:resource="descriptor/19720-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19725-3">
    <skos:related rdf:resource="descriptor/19724-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19725-3">
    <skos:related rdf:resource="descriptor/16252-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19725-3">
    <skos:related rdf:resource="descriptor/13412-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19725-3">
    <skos:related rdf:resource="descriptor/15415-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19725-3">
    <skos:related rdf:resource="descriptor/16470-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19725-3">
    <skos:related rdf:resource="descriptor/16204-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19745-4">
    <skos:scopeNote xml:lang="de">Für die Formalkatalogisierung BENUTZE Publikationsart Interview.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19745-4">
    <skos:scopeNote xml:lang="de">Nur für Methode, Technik und Auswertung von Befragungen zur Erhebung von Daten.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16411-6">
    <skos:broader rdf:resource="thsys/73383"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16411-6">
    <skos:broader rdf:resource="descriptor/16405-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18615-1">
    <skos:narrower rdf:resource="descriptor/24787-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18615-1">
    <skos:narrower rdf:resource="descriptor/18582-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18615-1">
    <skos:narrower rdf:resource="descriptor/18621-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13681-6">
    <skos:narrower rdf:resource="descriptor/13679-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13681-6">
    <skos:narrower rdf:resource="descriptor/13656-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16989-5">
    <skos:prefLabel xml:lang="en">Sweden</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16989-5">
    <skos:prefLabel xml:lang="de">Schweden</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70307">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19683-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16670-3">
    <skos:broader rdf:resource="thsys/73346"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16670-3">
    <skos:broader rdf:resource="thsys/70573"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11564-6">
    <skos:prefLabel xml:lang="en">Family benefits</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11564-6">
    <skos:prefLabel xml:lang="de">Familienleistungsausgleich</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13321-1">
    <skos:broader rdf:resource="thsys/70958"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18956-3">
    <skos:altLabel xml:lang="de">Sportverein</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18956-3">
    <skos:altLabel xml:lang="de">Sportveranstalter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19683-0">
    <skos:related rdf:resource="descriptor/10343-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19683-0">
    <skos:related rdf:resource="descriptor/19285-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19683-0">
    <skos:related rdf:resource="descriptor/19755-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19683-0">
    <skos:related rdf:resource="descriptor/16295-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19683-0">
    <skos:related rdf:resource="descriptor/15720-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19683-0">
    <skos:related rdf:resource="descriptor/19769-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19683-0">
    <skos:related rdf:resource="descriptor/16290-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19683-0">
    <skos:related rdf:resource="descriptor/16286-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19683-0">
    <skos:related rdf:resource="descriptor/19668-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19683-0">
    <skos:related rdf:resource="descriptor/16521-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19683-0">
    <skos:related rdf:resource="descriptor/10816-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19683-0">
    <skos:related rdf:resource="descriptor/16510-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19683-0">
    <skos:related rdf:resource="descriptor/19198-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13250-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12356-1">
    <skos:broader rdf:resource="thsys/71043"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12356-1">
    <skos:broader rdf:resource="thsys/70184"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12356-1">
    <skos:broader rdf:resource="descriptor/10024-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19589-6">
    <skos:broader rdf:resource="thsys/72181"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19589-6">
    <skos:broader rdf:resource="thsys/71115"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19589-6">
    <skos:broader rdf:resource="descriptor/15076-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16957-4">
    <skos:prefLabel xml:lang="en">Appenzell</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16957-4">
    <skos:prefLabel xml:lang="de">Appenzell</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11268-3">
    <skos:altLabel xml:lang="de">Gastarbeiterinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11268-3">
    <skos:altLabel xml:lang="de">Gastarbeiter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11268-3">
    <skos:altLabel xml:lang="de">Fremdarbeiterinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11268-3">
    <skos:altLabel xml:lang="de">Fremdarbeiter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11268-3">
    <skos:altLabel xml:lang="de">Ausländerbeschäftigung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11268-3">
    <skos:altLabel xml:lang="de">Arbeitsimmigrantinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11268-3">
    <skos:altLabel xml:lang="de">Arbeitsimmigranten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19140-3">
    <skos:narrower rdf:resource="descriptor/12812-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18024-3">
    <skos:altLabel xml:lang="en">Tidal power</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18024-3">
    <skos:altLabel xml:lang="en">Renewable energy technology</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18024-3">
    <skos:altLabel xml:lang="de">Regenerierbarer Energieträger</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18024-3">
    <skos:altLabel xml:lang="de">Regenerierbarer Energierohstoff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18024-3">
    <skos:altLabel xml:lang="de">Regenerativer Energieträger</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18024-3">
    <skos:altLabel xml:lang="de">Gezeitenenergie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18024-3">
    <skos:altLabel xml:lang="de">Erneuerbarer Energieträger</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18024-3">
    <skos:altLabel xml:lang="de">Erneuerbare Energietechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18024-3">
    <skos:altLabel xml:lang="de">Erneuerbare Energiequelle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18024-3">
    <skos:altLabel xml:lang="de">Erneuerbare Energie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18024-3">
    <skos:altLabel xml:lang="de">Alternative Energie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12927-5">
    <skos:narrower rdf:resource="descriptor/13253-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13414-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12407-3">
    <skos:prefLabel xml:lang="en">Acquisition costs</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12407-3">
    <skos:prefLabel xml:lang="de">Anschaffungskosten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12501-4">
    <skos:altLabel xml:lang="en">Staff costs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11642-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12964-6">
    <skos:prefLabel xml:lang="en">Fishery</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12964-6">
    <skos:prefLabel xml:lang="de">Fischerei</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19483-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10980-4">
    <skos:related rdf:resource="descriptor/19686-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10980-4">
    <skos:related rdf:resource="descriptor/11054-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10746-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10028-0">
    <zbwext:useInsteadNote rdf:nodeID="id363"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13541-1">
    <skos:prefLabel xml:lang="en">Canal</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13541-1">
    <skos:prefLabel xml:lang="de">Kanal</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14969-0">
    <skos:narrower rdf:resource="descriptor/14967-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18551-1">
    <skos:broader rdf:resource="thsys/70960"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18551-1">
    <skos:broader rdf:resource="descriptor/13473-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18551-1">
    <skos:broader rdf:resource="descriptor/13218-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13154-1">
    <skos:broader rdf:resource="thsys/70986"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13154-1">
    <skos:broader rdf:resource="descriptor/13160-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13154-1">
    <skos:broader rdf:resource="descriptor/19383-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10898-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13716-6">
    <skos:prefLabel xml:lang="en">Stockbrokers</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13716-6">
    <skos:prefLabel xml:lang="de">Börsenmakler</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11661-1">
    <skos:related rdf:resource="descriptor/11648-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10948-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13483-3">
    <skos:prefLabel xml:lang="en">Specialized retailer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13483-3">
    <skos:prefLabel xml:lang="de">Facheinzelhandel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17015-2">
    <skos:prefLabel xml:lang="en">San Marino</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17015-2">
    <skos:prefLabel xml:lang="de">San Marino</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id226">
    <rdf:value xml:lang="de">für die Industrie</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12648-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19916-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15467-5">
    <skos:prefLabel xml:lang="en">Decision under risk</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15467-5">
    <skos:prefLabel xml:lang="de">Entscheidung bei Risiko</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13693-6">
    <skos:narrower rdf:resource="descriptor/16072-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13693-6">
    <skos:narrower rdf:resource="descriptor/16061-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13693-6">
    <skos:narrower rdf:resource="descriptor/18703-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13693-6">
    <skos:narrower rdf:resource="descriptor/13694-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13693-6">
    <skos:narrower rdf:resource="descriptor/18722-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18682-0">
    <skos:narrower rdf:resource="descriptor/19652-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18351-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12027-6">
    <skos:prefLabel xml:lang="en">Property management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12027-6">
    <skos:prefLabel xml:lang="de">Immobilienverwaltung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19028-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15981-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70151">
    <rdfs:label xml:lang="de">Naturwissenschaften und Technik</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13264-1">
    <skos:related rdf:resource="descriptor/11834-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13264-1">
    <skos:related rdf:resource="descriptor/18625-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13264-1">
    <skos:related rdf:resource="descriptor/13569-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13264-1">
    <skos:related rdf:resource="descriptor/11833-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13264-1">
    <skos:related rdf:resource="descriptor/13588-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13264-1">
    <skos:related rdf:resource="descriptor/13979-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13264-1">
    <skos:related rdf:resource="descriptor/13510-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13264-1">
    <skos:related rdf:resource="descriptor/12927-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14355-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19205-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12930-2">
    <skos:broader rdf:resource="thsys/70985"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12930-2">
    <skos:broader rdf:resource="descriptor/12993-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13699-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15759-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15811-2">
    <skos:narrower rdf:resource="descriptor/18924-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15811-2">
    <skos:narrower rdf:resource="descriptor/12103-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15811-2">
    <skos:narrower rdf:resource="descriptor/12617-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15979-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18782-3">
    <skos:related rdf:resource="descriptor/10458-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18782-3">
    <skos:related rdf:resource="descriptor/11372-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18782-3">
    <skos:related rdf:resource="descriptor/11373-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19365-0">
    <skos:broader rdf:resource="thsys/71019"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19365-0">
    <skos:broader rdf:resource="descriptor/12961-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10717-5">
    <skos:related rdf:resource="descriptor/19313-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10797-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14794-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18632-1">
    <skos:broader rdf:resource="thsys/71069"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18632-1">
    <skos:broader rdf:resource="thsys/70925"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18632-1">
    <skos:broader rdf:resource="descriptor/11833-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18632-1">
    <skos:broader rdf:resource="descriptor/16552-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11551-1">
    <skos:scopeNote xml:lang="de">Kosten, die den Unternehmen oder den Bürger/innen aufgrund politischer oder administrativer Maßnahmen entstehen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19094-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19140-3">
    <skos:related rdf:resource="descriptor/12778-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19140-3">
    <skos:related rdf:resource="descriptor/12510-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19140-3">
    <skos:related rdf:resource="descriptor/10940-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19140-3">
    <skos:related rdf:resource="descriptor/19139-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19140-3">
    <skos:related rdf:resource="descriptor/12766-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19140-3">
    <skos:related rdf:resource="descriptor/18007-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19390-1">
    <skos:related rdf:resource="descriptor/18405-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19390-1">
    <skos:related rdf:resource="descriptor/13029-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19390-1">
    <skos:related rdf:resource="descriptor/13236-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19390-1">
    <skos:related rdf:resource="descriptor/14285-2"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id99">
    <rdfs:seeAlso rdf:resource="descriptor/13691-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11142-4">
    <skos:altLabel xml:lang="en">Soviet-type enterprise</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11142-4">
    <skos:altLabel xml:lang="de">Volkseigener Betrieb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11142-4">
    <skos:altLabel xml:lang="de">VEB</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11142-4">
    <skos:altLabel xml:lang="de">Staatsunternehmen im Sozialismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17723-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13273-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10977-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18523-6">
    <skos:prefLabel xml:lang="en">Computer trade</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18523-6">
    <skos:prefLabel xml:lang="de">Computerhandel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70073">
    <skos:broader rdf:resource="thsys/70174"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13127-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13155-6">
    <skos:related rdf:resource="descriptor/12955-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13155-6">
    <skos:related rdf:resource="descriptor/14964-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13155-6">
    <skos:related rdf:resource="descriptor/19340-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13155-6">
    <skos:related rdf:resource="descriptor/13940-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16039-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19343-3">
    <skos:related rdf:resource="descriptor/19344-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19343-3">
    <skos:related rdf:resource="descriptor/12950-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19343-3">
    <skos:related rdf:resource="descriptor/14142-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19343-3">
    <skos:related rdf:resource="descriptor/12946-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16502-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14449-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13621-3">
    <skos:altLabel xml:lang="de">Warenverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13621-3">
    <skos:altLabel xml:lang="de">Warentransport</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13621-3">
    <skos:altLabel xml:lang="de">Gütertransport</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13621-3">
    <skos:altLabel xml:lang="de">Güterbeförderung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13621-3">
    <skos:altLabel xml:lang="de">Frachtverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13621-3">
    <skos:altLabel xml:lang="de">Frachttransport</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16150-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12612-2">
    <skos:altLabel xml:lang="en">Human-factors engineering</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12612-2">
    <skos:altLabel xml:lang="en">Human factors engineering</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12612-2">
    <skos:altLabel xml:lang="de">Arbeitsphysiologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12612-2">
    <skos:altLabel xml:lang="de">Arbeitsmedizin</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70938">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.12.01.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12046-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19600-2">
    <skos:broader rdf:resource="thsys/70017"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15491-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18595-2">
    <skos:prefLabel xml:lang="en">Container shipping</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18595-2">
    <skos:prefLabel xml:lang="de">Containerschifffahrt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13231-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13780-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19226-0">
    <skos:related rdf:resource="descriptor/10206-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19226-0">
    <skos:related rdf:resource="descriptor/10205-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14964-3">
    <skos:altLabel xml:lang="de">Wurst</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14964-3">
    <skos:altLabel xml:lang="de">Verarbeitetes Fleisch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14964-3">
    <skos:altLabel xml:lang="de">Trockenfleisch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14964-3">
    <skos:altLabel xml:lang="de">Speck</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14964-3">
    <skos:altLabel xml:lang="de">Schinken</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14964-3">
    <skos:altLabel xml:lang="de">Räucherfleisch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14964-3">
    <skos:altLabel xml:lang="de">Pökelfleisch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14964-3">
    <skos:altLabel xml:lang="de">Fleischpastete</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13097-1">
    <skos:broader rdf:resource="thsys/71004"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13097-1">
    <skos:broader rdf:resource="descriptor/12993-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14049-0">
    <skos:broader rdf:resource="thsys/70461"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14049-0">
    <skos:broader rdf:resource="descriptor/13427-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16112-2">
    <skos:broader rdf:resource="thsys/73337"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16112-2">
    <skos:broader rdf:resource="thsys/73334"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16112-2">
    <skos:broader rdf:resource="thsys/71077"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16112-2">
    <skos:broader rdf:resource="thsys/71073"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16112-2">
    <skos:broader rdf:resource="descriptor/16195-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19745-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11366-3">
    <skos:prefLabel xml:lang="en">Employer-provided training</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11366-3">
    <skos:prefLabel xml:lang="de">Betriebliche Bildungsarbeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18430-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12699-6">
    <skos:altLabel xml:lang="de">Wartung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12699-6">
    <skos:altLabel xml:lang="de">Reparatur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12699-6">
    <skos:altLabel xml:lang="de">Instandsetzung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12699-6">
    <skos:altLabel xml:lang="de">Anlagenerhaltung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19532-0">
    <skos:altLabel xml:lang="en">Housing tenure choice</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19532-0">
    <skos:altLabel xml:lang="de">Wohnungsbesitzwahl</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19532-0">
    <skos:altLabel xml:lang="de">Besitzwahltheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11642-5">
    <skos:prefLabel xml:lang="en">Tax system</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11642-5">
    <skos:prefLabel xml:lang="de">Steuersystem</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19711-0">
    <skos:prefLabel xml:lang="en">International conference</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19711-0">
    <skos:prefLabel xml:lang="de">Internationale Konferenz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17688-0">
    <skos:altLabel xml:lang="de">Republik Südafrika</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18143-2">
    <skos:broader rdf:resource="thsys/70843"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18143-2">
    <skos:broader rdf:resource="thsys/70244"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18143-2">
    <skos:broader rdf:resource="descriptor/11361-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11987-6">
    <skos:prefLabel xml:lang="en">Urban geography</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11987-6">
    <skos:prefLabel xml:lang="de">Stadtgeographie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16402-0">
    <skos:prefLabel xml:lang="en">Social democracy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16402-0">
    <skos:prefLabel xml:lang="de">Sozialdemokratie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15728-3">
    <skos:prefLabel xml:lang="en">Exhaust emission technology</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15728-3">
    <skos:prefLabel xml:lang="de">Abgastechnik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17982-0">
    <skos:altLabel xml:lang="de">EFTA-Länder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11530-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11648-0">
    <skos:broader rdf:resource="thsys/71076"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11648-0">
    <skos:broader rdf:resource="thsys/71074"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11648-0">
    <skos:broader rdf:resource="descriptor/11547-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14177-5">
    <skos:related rdf:resource="descriptor/12977-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10598-1">
    <skos:broader rdf:resource="thsys/73366"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10598-1">
    <skos:broader rdf:resource="thsys/71084"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10598-1">
    <skos:broader rdf:resource="descriptor/10452-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10598-1">
    <skos:broader rdf:resource="descriptor/10555-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73385">
    <rdfs:label xml:lang="de">Arbeits- und Organisationspsychologie</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13282-6">
    <skos:prefLabel xml:lang="en">Insurance agents</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13282-6">
    <skos:prefLabel xml:lang="de">Versicherungsvermittlung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20077-2">
    <skos:altLabel xml:lang="de">Gesellschaftsvertrag</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70814">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18879-2">
    <skos:related rdf:resource="descriptor/11379-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18879-2">
    <skos:related rdf:resource="descriptor/11378-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18879-2">
    <skos:related rdf:resource="descriptor/18874-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15571-3">
    <skos:prefLabel xml:lang="en">Inventory model</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15571-3">
    <skos:prefLabel xml:lang="de">Lagerhaltungsmodell</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19411-5">
    <skos:broader rdf:resource="thsys/71076"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19411-5">
    <skos:broader rdf:resource="descriptor/11696-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18170-6">
    <skos:narrower rdf:resource="descriptor/13653-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18170-6">
    <skos:narrower rdf:resource="descriptor/15410-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18170-6">
    <skos:narrower rdf:resource="descriptor/13649-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18206-4">
    <skos:broader rdf:resource="thsys/70265"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18206-4">
    <skos:broader rdf:resource="descriptor/12829-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18206-4">
    <skos:broader rdf:resource="descriptor/18211-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70889">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19718-0">
    <skos:broader rdf:resource="thsys/73366"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19718-0">
    <skos:broader rdf:resource="descriptor/16421-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11313-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10302-6">
    <skos:altLabel xml:lang="en">Status good</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10302-6">
    <skos:altLabel xml:lang="en">Premium product</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10302-6">
    <skos:altLabel xml:lang="de">Spitzenprodukt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10302-6">
    <skos:altLabel xml:lang="de">Hochpreisprodukt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10615-6">
    <skos:prefLabel xml:lang="en">International factor mobility</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10615-6">
    <skos:prefLabel xml:lang="de">Internationale Faktormobilität</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16654-1">
    <skos:scopeNote xml:lang="de">Ausschluss Einzelner oder sozialer Gruppen von der aktiven Teilnahme am gesellschaftlichen Leben.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19311-2">
    <skos:scopeNote xml:lang="de">Das Ausmaß der Widerspiegelung von Wechselkursänderungen in den Import- und Exportpreisen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10348-3">
    <skos:altLabel xml:lang="de">Krisenzyklus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19679-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19382-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19328-6">
    <skos:narrower rdf:resource="descriptor/11573-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19328-6">
    <skos:narrower rdf:resource="descriptor/19723-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19328-6">
    <skos:narrower rdf:resource="descriptor/10603-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13746-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14643-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10562-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13370-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10002-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19735-0">
    <skos:related rdf:resource="descriptor/15640-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17255-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18059-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16623-5">
    <skos:broader rdf:resource="thsys/73395"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16623-5">
    <skos:broader rdf:resource="thsys/73377"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16623-5">
    <skos:broader rdf:resource="descriptor/19768-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13608-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14710-6">
    <skos:scopeNote xml:lang="de">Nur die synthetisch erzeugten Polymere der chemischen Industrie.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15491-1">
    <skos:related rdf:resource="descriptor/15554-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12022-2">
    <skos:prefLabel xml:lang="en">Land rent</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12022-2">
    <skos:prefLabel xml:lang="de">Bodenrente</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12697-3">
    <skos:scopeNote xml:lang="de">Anpassung einer Volkswirtschaft / einer Branche an veränderte ökonomische Rahmenbedingungen; auch die staatlichen Maßnahmen zur Förderung dieses Prozesses.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15768-5">
    <skos:narrower rdf:resource="descriptor/18628-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15768-5">
    <skos:narrower rdf:resource="descriptor/18364-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15768-5">
    <skos:narrower rdf:resource="descriptor/19483-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15768-5">
    <skos:narrower rdf:resource="descriptor/15761-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15768-5">
    <skos:narrower rdf:resource="descriptor/15760-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15768-5">
    <skos:narrower rdf:resource="descriptor/15738-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15768-5">
    <skos:narrower rdf:resource="descriptor/19727-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12435-5">
    <skos:broader rdf:resource="thsys/71043"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12435-5">
    <skos:broader rdf:resource="descriptor/12408-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11651-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19082-5">
    <skos:altLabel xml:lang="de">Verlustübernahme</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19082-5">
    <skos:altLabel xml:lang="de">Gewinnabführung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10944-1">
    <skos:related rdf:resource="descriptor/10895-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10944-1">
    <skos:related rdf:resource="descriptor/10948-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16485-5">
    <skos:broader rdf:resource="thsys/73392"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16485-5">
    <skos:broader rdf:resource="thsys/70814"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16485-5">
    <skos:broader rdf:resource="thsys/70292"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16485-5">
    <skos:broader rdf:resource="descriptor/18976-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70160">
    <skos:broader rdf:resource="thsys/70154"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14985-2">
    <skos:narrower rdf:resource="descriptor/14981-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14985-2">
    <skos:narrower rdf:resource="descriptor/14980-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17950-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19789-5">
    <skos:prefLabel xml:lang="en">Diabetes</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19789-5">
    <skos:prefLabel xml:lang="de">Diabetes</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11353-5">
    <skos:narrower rdf:resource="descriptor/16275-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11353-5">
    <skos:narrower rdf:resource="descriptor/18863-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11353-5">
    <skos:narrower rdf:resource="descriptor/11352-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16404-3">
    <skos:prefLabel xml:lang="en">Radicalism</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16404-3">
    <skos:prefLabel xml:lang="de">Radikalismus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19545-5">
    <skos:narrower rdf:resource="descriptor/19546-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71032">
    <skos:broader rdf:resource="thsys/70148"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19691-1">
    <skos:prefLabel xml:lang="en">Local self-government</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19691-1">
    <skos:prefLabel xml:lang="de">Kommunale Selbstverwaltung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70839">
    <skos:broader rdf:resource="thsys/w"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19847-3">
    <skos:broader rdf:resource="thsys/73360"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11963-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13427-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19270-4">
    <skos:altLabel xml:lang="de">Regulierung der Börse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19270-4">
    <skos:altLabel xml:lang="de">Einschuss (Börse)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19270-4">
    <skos:altLabel xml:lang="de">Einschuß (Börse)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14005-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14617-3">
    <skos:related rdf:resource="descriptor/14803-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14617-3">
    <skos:related rdf:resource="descriptor/14802-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12451-0">
    <skos:broader rdf:resource="thsys/70186"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12451-0">
    <skos:broader rdf:resource="descriptor/12452-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19227-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70141">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11535-6">
    <skos:related rdf:resource="descriptor/11540-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11535-6">
    <skos:related rdf:resource="descriptor/11536-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11535-6">
    <skos:related rdf:resource="descriptor/11729-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11535-6">
    <skos:related rdf:resource="descriptor/11525-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11535-6">
    <skos:related rdf:resource="descriptor/11532-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11535-6">
    <skos:related rdf:resource="descriptor/19380-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72180">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18614-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14037-0">
    <skos:prefLabel xml:lang="en">Coal power station</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14037-0">
    <skos:prefLabel xml:lang="de">Kohlekraftwerk</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19645-1">
    <skos:broader rdf:resource="thsys/70182"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19645-1">
    <skos:broader rdf:resource="descriptor/15916-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24782-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15684-4">
    <skos:related rdf:resource="descriptor/10032-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15684-4">
    <skos:related rdf:resource="descriptor/18871-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15684-4">
    <skos:related rdf:resource="descriptor/11360-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15684-4">
    <skos:related rdf:resource="descriptor/12602-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13509-4">
    <skos:prefLabel xml:lang="en">Transit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13509-4">
    <skos:prefLabel xml:lang="de">Transit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10015-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17569-1">
    <skos:broader rdf:resource="thsys/70147"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17569-1">
    <skos:broader rdf:resource="descriptor/19166-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19616-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16410-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18993-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19595-4">
    <skos:broader rdf:resource="thsys/73104"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19595-4">
    <skos:broader rdf:resource="descriptor/15446-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18356-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73350">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13763-4">
    <skos:altLabel xml:lang="en">Funding</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12171-6">
    <skos:altLabel xml:lang="de">Mitarbeiterinformation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12171-6">
    <skos:altLabel xml:lang="de">Interne Information</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12171-6">
    <skos:altLabel xml:lang="de">Innerbetriebliche Information</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12171-6">
    <skos:altLabel xml:lang="de">Inhouse-Kommunikation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12171-6">
    <skos:altLabel xml:lang="de">Betriebsinterne Kommunikation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12171-6">
    <skos:altLabel xml:lang="de">Betriebliche Kommunikationsstruktur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12171-6">
    <skos:altLabel xml:lang="de">Betriebliche Informationsstruktur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11765-3">
    <skos:broader rdf:resource="thsys/71114"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11765-3">
    <skos:broader rdf:resource="descriptor/10249-5"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id375">
    <rdfs:seeAlso rdf:resource="descriptor/10905-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70368">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19447-5">
    <skos:narrower rdf:resource="descriptor/11999-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19447-5">
    <skos:narrower rdf:resource="descriptor/18967-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19447-5">
    <skos:narrower rdf:resource="descriptor/11890-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17678-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18054-1">
    <skos:prefLabel xml:lang="en">Mediterranean region</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18054-1">
    <skos:prefLabel xml:lang="de">Mittelmeerraum</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13492-2">
    <skos:narrower rdf:resource="descriptor/18552-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16028-5">
    <skos:prefLabel xml:lang="en">Inheritance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16028-5">
    <skos:prefLabel xml:lang="de">Erbe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70131">
    <skos:prefLabel xml:lang="en">G.01.04  Southern Europe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70131">
    <skos:prefLabel xml:lang="de">G.01.04  Südeuropa</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13629-1">
    <skos:narrower rdf:resource="descriptor/18612-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11490-2">
    <skos:prefLabel xml:lang="en">Interest rate policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11490-2">
    <skos:prefLabel xml:lang="de">Zinspolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10958-4">
    <skos:related rdf:resource="descriptor/10913-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10958-4">
    <skos:related rdf:resource="descriptor/16051-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70580">
    <skos:narrower rdf:resource="descriptor/19779-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70580">
    <skos:narrower rdf:resource="descriptor/14003-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70580">
    <skos:narrower rdf:resource="descriptor/19778-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70580">
    <skos:narrower rdf:resource="descriptor/15676-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70580">
    <skos:narrower rdf:resource="descriptor/19777-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70580">
    <skos:narrower rdf:resource="descriptor/19776-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70580">
    <skos:narrower rdf:resource="descriptor/19775-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70580">
    <skos:narrower rdf:resource="descriptor/15657-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70580">
    <skos:narrower rdf:resource="descriptor/19608-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70580">
    <skos:narrower rdf:resource="descriptor/15642-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19214-0">
    <skos:related rdf:resource="descriptor/10060-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14195-3">
    <skos:broader rdf:resource="thsys/70418"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14195-3">
    <skos:broader rdf:resource="descriptor/14198-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11721-2">
    <skos:prefLabel xml:lang="en">Government size</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11721-2">
    <skos:prefLabel xml:lang="de">Staatsquote</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10508-0">
    <skos:broader rdf:resource="thsys/70307"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10508-0">
    <skos:broader rdf:resource="thsys/70091"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13392-6">
    <skos:scopeNote xml:lang="de">Gesellschaften, welche geschäftsmäßig jene Verwertungsrechte wahrnehmen, die sich für Publizist/innen und Künstler/innen aus dem Urheberrecht ergeben.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18688-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10668-6">
    <skos:broader rdf:resource="thsys/71099"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10668-6">
    <skos:broader rdf:resource="descriptor/10676-0"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id147">
    <rdf:value xml:lang="de">Für die Verbreitung von Informationen</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18630-5">
    <skos:related rdf:resource="descriptor/13590-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18630-5">
    <skos:related rdf:resource="descriptor/13179-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18630-5">
    <skos:related rdf:resource="descriptor/13514-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18630-5">
    <skos:related rdf:resource="descriptor/18501-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18338-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17994-0">
    <skos:broader rdf:resource="thsys/70307"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17994-0">
    <skos:broader rdf:resource="descriptor/11123-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70562">
    <skos:narrower rdf:resource="descriptor/12138-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70562">
    <skos:narrower rdf:resource="descriptor/19155-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70562">
    <skos:narrower rdf:resource="descriptor/12160-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70562">
    <skos:narrower rdf:resource="descriptor/19562-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70562">
    <skos:narrower rdf:resource="descriptor/12116-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70562">
    <skos:narrower rdf:resource="descriptor/15429-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70562">
    <skos:narrower rdf:resource="descriptor/12094-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70562">
    <skos:narrower rdf:resource="descriptor/12155-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70562">
    <skos:narrower rdf:resource="descriptor/12154-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70562">
    <skos:narrower rdf:resource="descriptor/12153-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70562">
    <skos:narrower rdf:resource="descriptor/12105-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70562">
    <skos:narrower rdf:resource="descriptor/24847-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70562">
    <skos:narrower rdf:resource="descriptor/12112-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70562">
    <skos:narrower rdf:resource="descriptor/12111-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70562">
    <skos:narrower rdf:resource="descriptor/12129-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70562">
    <skos:narrower rdf:resource="descriptor/12106-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70562">
    <skos:narrower rdf:resource="descriptor/12091-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70562">
    <skos:narrower rdf:resource="descriptor/20977-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70562">
    <skos:narrower rdf:resource="descriptor/12146-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70562">
    <skos:narrower rdf:resource="descriptor/18867-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70562">
    <skos:narrower rdf:resource="descriptor/20948-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70562">
    <skos:narrower rdf:resource="descriptor/11158-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70562">
    <skos:narrower rdf:resource="descriptor/24704-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70562">
    <skos:narrower rdf:resource="descriptor/12120-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70562">
    <skos:narrower rdf:resource="descriptor/12757-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70562">
    <skos:narrower rdf:resource="descriptor/12137-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70562">
    <skos:narrower rdf:resource="descriptor/24661-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70562">
    <skos:narrower rdf:resource="descriptor/12042-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70562">
    <skos:narrower rdf:resource="descriptor/12136-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70562">
    <skos:narrower rdf:resource="descriptor/12617-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70562">
    <skos:narrower rdf:resource="descriptor/12621-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70562">
    <skos:narrower rdf:resource="descriptor/12135-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70562">
    <skos:narrower rdf:resource="descriptor/12134-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19301-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16300-1">
    <skos:prefLabel xml:lang="en">Nation state</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16300-1">
    <skos:prefLabel xml:lang="de">Nationalstaat</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18556-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15940-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13469-4">
    <skos:related rdf:resource="descriptor/10940-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13469-4">
    <skos:related rdf:resource="descriptor/18527-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18749-1">
    <skos:related rdf:resource="descriptor/13831-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18749-1">
    <skos:related rdf:resource="descriptor/12932-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18749-1">
    <skos:related rdf:resource="descriptor/13840-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18749-1">
    <skos:related rdf:resource="descriptor/12014-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11823-1">
    <skos:altLabel xml:lang="de">Binnenhandelspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12724-5">
    <skos:broader rdf:resource="thsys/70310"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12724-5">
    <skos:broader rdf:resource="descriptor/12725-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12882-1">
    <skos:prefLabel xml:lang="en">Audit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12882-1">
    <skos:prefLabel xml:lang="de">Revision</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20068-3">
    <skos:related rdf:resource="descriptor/13387-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20068-3">
    <skos:related rdf:resource="descriptor/18771-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://zbw.eu/namespaces/skos-extensions/useInsteadNote">
    <rdfs:subPropertyOf rdf:resource="http://www.w3.org/2004/02/skos/core#note"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14044-3">
    <skos:prefLabel xml:lang="en">Oil and fat</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14044-3">
    <skos:prefLabel xml:lang="de">Öl und Fett</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71069">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.11.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14989-1">
    <skos:prefLabel xml:lang="en">Feed</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14989-1">
    <skos:prefLabel xml:lang="de">Futtermittel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13808-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16797-0">
    <skos:altLabel xml:lang="de">Wörterbuch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16797-0">
    <skos:altLabel xml:lang="de">Lexikon</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16797-0">
    <skos:altLabel xml:lang="de">Enzyklopädie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16797-0">
    <skos:altLabel xml:lang="de">Biographie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71041">
    <rdfs:label xml:lang="de">Revision und Controlling</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#Concept">
    <rdf:type rdf:resource="http://www.w3.org/2002/07/owl#Class"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17624-5">
    <skos:scopeNote xml:lang="de">Verwendet für die gesamte Literatur über Jemen, auch aus der Zeit vor der staatlichen Wiedervereinigung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13020-1">
    <skos:prefLabel xml:lang="en">Processing of non-metallic minerals</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13020-1">
    <skos:prefLabel xml:lang="de">Verarbeitung von Steinen und Erden</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16158-6">
    <skos:broader rdf:resource="thsys/73335"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16158-6">
    <skos:broader rdf:resource="thsys/70835"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16158-6">
    <skos:broader rdf:resource="descriptor/16159-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11525-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12735-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13694-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13199-0">
    <skos:broader rdf:resource="thsys/70957"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13199-0">
    <skos:broader rdf:resource="descriptor/13200-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16370-1">
    <skos:altLabel xml:lang="de">Parteikongress</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16370-1">
    <skos:altLabel xml:lang="de">Parteikongreß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10730-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18334-2">
    <skos:related rdf:resource="descriptor/13428-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18334-2">
    <skos:related rdf:resource="descriptor/18892-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18334-2">
    <skos:related rdf:resource="descriptor/10495-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18334-2">
    <skos:related rdf:resource="descriptor/11801-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10005-5">
    <skos:prefLabel xml:lang="en">Property</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10005-5">
    <skos:prefLabel xml:lang="de">Eigentum</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14561-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13441-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13546-5">
    <skos:related rdf:resource="descriptor/13513-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13546-5">
    <skos:related rdf:resource="descriptor/13610-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13546-5">
    <skos:related rdf:resource="descriptor/18592-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13546-5">
    <skos:related rdf:resource="descriptor/14455-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13546-5">
    <skos:related rdf:resource="descriptor/13252-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13546-5">
    <skos:related rdf:resource="descriptor/13600-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18170-6">
    <skos:broader rdf:resource="thsys/73339"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18170-6">
    <skos:broader rdf:resource="descriptor/15611-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18170-6">
    <skos:broader rdf:resource="descriptor/15606-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18938-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19224-4">
    <skos:narrower rdf:resource="descriptor/19266-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15437-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13799-4">
    <skos:related rdf:resource="descriptor/12302-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13799-4">
    <skos:related rdf:resource="descriptor/24859-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73306">
    <skos:narrower rdf:resource="descriptor/14779-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73306">
    <skos:narrower rdf:resource="descriptor/14778-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73306">
    <skos:narrower rdf:resource="descriptor/14777-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73306">
    <skos:narrower rdf:resource="descriptor/14776-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73306">
    <skos:narrower rdf:resource="descriptor/14775-6"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id240">
    <rdf:value xml:lang="de">Für Staatsunternehmen sozialistischer Länder</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18880-3">
    <skos:scopeNote xml:lang="de">Für Wirtschaftshochschulen und einzelne Fachbereiche.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70962">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11721-2">
    <skos:broader rdf:resource="thsys/71083"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11721-2">
    <skos:broader rdf:resource="thsys/70858"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11721-2">
    <skos:broader rdf:resource="descriptor/15075-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73350">
    <skos:narrower rdf:resource="descriptor/19784-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73350">
    <skos:narrower rdf:resource="descriptor/15900-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73350">
    <skos:narrower rdf:resource="descriptor/19442-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73350">
    <skos:narrower rdf:resource="descriptor/19529-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73350">
    <skos:narrower rdf:resource="descriptor/15669-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73350">
    <skos:narrower rdf:resource="descriptor/15628-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73350">
    <skos:narrower rdf:resource="descriptor/15033-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73350">
    <skos:narrower rdf:resource="descriptor/14174-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19135-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18455-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19070-5">
    <skos:altLabel xml:lang="de">Empirisches Verfahren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19070-5">
    <skos:altLabel xml:lang="de">Empirische Untersuchung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13120-4">
    <skos:broader rdf:resource="thsys/70991"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13120-4">
    <skos:broader rdf:resource="descriptor/13106-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10733-0">
    <skos:prefLabel xml:lang="en">ECU</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10733-0">
    <skos:prefLabel xml:lang="de">ECU</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11261-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70397">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11479-4">
    <skos:related rdf:resource="descriptor/11487-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11479-4">
    <skos:related rdf:resource="descriptor/11484-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11479-4">
    <skos:related rdf:resource="descriptor/11494-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11479-4">
    <skos:related rdf:resource="descriptor/11448-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16757-5">
    <skos:scopeNote xml:lang="de">Für die in Unternehmen, Verwaltungen etc. vorhandene und eingesetzte DV-technische Ausstattung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19549-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12765-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20306-6">
    <skos:related rdf:resource="descriptor/19564-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20306-6">
    <skos:related rdf:resource="descriptor/12809-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20306-6">
    <skos:related rdf:resource="descriptor/19128-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16389-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70843">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.24.03</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14822-2">
    <skos:broader rdf:resource="thsys/70372"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15924-3">
    <skos:altLabel xml:lang="de">Jugend</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10151-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70416">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10572-5">
    <skos:broader rdf:resource="thsys/71090"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10572-5">
    <skos:broader rdf:resource="thsys/71084"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10572-5">
    <skos:broader rdf:resource="thsys/70140"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10572-5">
    <skos:broader rdf:resource="descriptor/18121-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18397-6">
    <skos:altLabel xml:lang="en">Business service</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18397-6">
    <skos:altLabel xml:lang="de">Unternehmensorientierte Dienstleistung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18397-6">
    <skos:altLabel xml:lang="de">Unternehmensinterne Dienstleistung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18397-6">
    <skos:altLabel xml:lang="de">Unternehmensbezogene Dienstleistung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18397-6">
    <skos:altLabel xml:lang="de">Dienstleistung für Unternehmen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18663-4">
    <skos:related rdf:resource="descriptor/18643-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18663-4">
    <skos:related rdf:resource="descriptor/18653-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18663-4">
    <skos:related rdf:resource="descriptor/13278-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18663-4">
    <skos:related rdf:resource="descriptor/13682-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16960-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15125-5">
    <skos:related rdf:resource="descriptor/15023-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16741-6">
    <skos:related rdf:resource="descriptor/10986-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16741-6">
    <skos:related rdf:resource="descriptor/15965-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16741-6">
    <skos:related rdf:resource="descriptor/15704-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16741-6">
    <skos:related rdf:resource="descriptor/10041-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11448-1">
    <skos:narrower rdf:resource="descriptor/11446-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19068-6">
    <skos:narrower rdf:resource="descriptor/13830-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19068-6">
    <skos:narrower rdf:resource="descriptor/12562-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19068-6">
    <skos:narrower rdf:resource="descriptor/18951-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14052-4">
    <skos:altLabel xml:lang="de">Wasserentsalzungsanlage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14052-4">
    <skos:altLabel xml:lang="de">Wasseraufbereitungstechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14052-4">
    <skos:altLabel xml:lang="de">Meerwasserentsalzungsanlage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14052-4">
    <skos:altLabel xml:lang="de">Entsalzungsanlage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70201">
    <skos:narrower rdf:resource="descriptor/18277-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70201">
    <skos:narrower rdf:resource="descriptor/12649-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70201">
    <skos:narrower rdf:resource="descriptor/12841-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70201">
    <skos:narrower rdf:resource="descriptor/18278-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70201">
    <skos:narrower rdf:resource="descriptor/10027-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70201">
    <skos:narrower rdf:resource="descriptor/18279-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70201">
    <skos:narrower rdf:resource="descriptor/18559-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70201">
    <skos:narrower rdf:resource="descriptor/18280-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70201">
    <skos:narrower rdf:resource="descriptor/14586-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70201">
    <skos:narrower rdf:resource="descriptor/14581-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70201">
    <skos:narrower rdf:resource="descriptor/12613-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70201">
    <skos:narrower rdf:resource="descriptor/14048-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70201">
    <skos:narrower rdf:resource="descriptor/14593-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70201">
    <skos:narrower rdf:resource="descriptor/14047-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70201">
    <skos:narrower rdf:resource="descriptor/18455-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70201">
    <skos:narrower rdf:resource="descriptor/10016-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70201">
    <skos:narrower rdf:resource="descriptor/14961-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70201">
    <skos:narrower rdf:resource="descriptor/18077-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70201">
    <skos:narrower rdf:resource="descriptor/14044-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70201">
    <skos:narrower rdf:resource="descriptor/18300-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70201">
    <skos:narrower rdf:resource="descriptor/14573-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70201">
    <skos:narrower rdf:resource="descriptor/14043-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70201">
    <skos:narrower rdf:resource="descriptor/18252-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70201">
    <skos:narrower rdf:resource="descriptor/14042-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70201">
    <skos:narrower rdf:resource="descriptor/18292-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70201">
    <skos:narrower rdf:resource="descriptor/14041-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70201">
    <skos:narrower rdf:resource="descriptor/14040-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70201">
    <skos:narrower rdf:resource="descriptor/18276-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70201">
    <skos:narrower rdf:resource="descriptor/14151-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70201">
    <skos:narrower rdf:resource="descriptor/10007-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70201">
    <skos:narrower rdf:resource="descriptor/14404-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70201">
    <skos:narrower rdf:resource="descriptor/14033-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70201">
    <skos:narrower rdf:resource="descriptor/14032-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70201">
    <skos:narrower rdf:resource="descriptor/18613-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70201">
    <skos:narrower rdf:resource="descriptor/18562-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70201">
    <skos:narrower rdf:resource="descriptor/14030-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70201">
    <skos:narrower rdf:resource="descriptor/14029-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70201">
    <skos:narrower rdf:resource="descriptor/14251-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70201">
    <skos:narrower rdf:resource="descriptor/14028-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70201">
    <skos:narrower rdf:resource="descriptor/14849-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70201">
    <skos:narrower rdf:resource="descriptor/14724-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70201">
    <skos:narrower rdf:resource="descriptor/14026-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70201">
    <skos:narrower rdf:resource="descriptor/14723-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70201">
    <skos:narrower rdf:resource="descriptor/24664-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70201">
    <skos:narrower rdf:resource="descriptor/14025-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70201">
    <skos:narrower rdf:resource="descriptor/18275-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70201">
    <skos:narrower rdf:resource="descriptor/14024-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70201">
    <skos:narrower rdf:resource="descriptor/14023-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70201">
    <skos:narrower rdf:resource="descriptor/14022-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70201">
    <skos:narrower rdf:resource="descriptor/14021-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70201">
    <skos:narrower rdf:resource="descriptor/14020-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70201">
    <skos:narrower rdf:resource="descriptor/14019-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10542-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15102-3">
    <skos:prefLabel xml:lang="en">Income statistics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15102-3">
    <skos:prefLabel xml:lang="de">Einkommensstatistik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70029">
    <skos:narrower rdf:resource="descriptor/20206-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70029">
    <skos:narrower rdf:resource="descriptor/20277-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70029">
    <skos:narrower rdf:resource="descriptor/12761-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70029">
    <skos:narrower rdf:resource="descriptor/12714-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70029">
    <skos:narrower rdf:resource="descriptor/12759-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70029">
    <skos:narrower rdf:resource="descriptor/12758-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70029">
    <skos:narrower rdf:resource="descriptor/12755-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70029">
    <skos:narrower rdf:resource="descriptor/12754-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70029">
    <skos:narrower rdf:resource="descriptor/12643-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70029">
    <skos:narrower rdf:resource="descriptor/12760-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70029">
    <skos:narrower rdf:resource="descriptor/12757-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70029">
    <skos:narrower rdf:resource="descriptor/12750-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70029">
    <skos:narrower rdf:resource="descriptor/12749-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70029">
    <skos:narrower rdf:resource="descriptor/19074-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70029">
    <skos:narrower rdf:resource="descriptor/12765-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70029">
    <skos:narrower rdf:resource="descriptor/12748-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70029">
    <skos:narrower rdf:resource="descriptor/12747-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19066-3">
    <skos:narrower rdf:resource="descriptor/13248-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18550-3">
    <skos:related rdf:resource="descriptor/13862-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18550-3">
    <skos:related rdf:resource="descriptor/13218-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18550-3">
    <skos:related rdf:resource="descriptor/18548-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18343-1">
    <skos:broader rdf:resource="thsys/73351"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18343-1">
    <skos:broader rdf:resource="thsys/71018"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18343-1">
    <skos:broader rdf:resource="thsys/70811"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18343-1">
    <skos:broader rdf:resource="descriptor/15745-3"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id126">
    <rdf:value xml:lang="de">Für die natürliche Kautschukgewinnung</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12880-5">
    <skos:related rdf:resource="descriptor/11679-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12880-5">
    <skos:related rdf:resource="descriptor/11709-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12880-5">
    <skos:related rdf:resource="descriptor/12876-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12880-5">
    <skos:related rdf:resource="descriptor/16104-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12880-5">
    <skos:related rdf:resource="descriptor/11536-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16090-0">
    <skos:prefLabel xml:lang="en">EU tax law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16090-0">
    <skos:prefLabel xml:lang="de">EU-Steuerrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18175-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13499-2">
    <skos:related rdf:resource="descriptor/13501-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15074-3">
    <skos:prefLabel xml:lang="en">Environmental statistics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15074-3">
    <skos:prefLabel xml:lang="de">Umweltstatistik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15938-6">
    <skos:prefLabel xml:lang="en">Family</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15938-6">
    <skos:prefLabel xml:lang="de">Familie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20018-4">
    <skos:scopeNote xml:lang="de">Breites Spektrum nicht öffentlich gehandelter Unternehmensbeteiligungen, die der Finanzierung in entscheidenden Entwicklungsphasen der Unternehmen dienen; ohne ausreichende Sicherheiten, ggf. unter Einflussnahme auf das Management und mit der Absicht, zur Realisierung eines dem Risiko entsprechenden Gewinns die Beteiligung wieder zu veräußern</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13744-1">
    <skos:altLabel xml:lang="de">Wertpapierterminhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13744-1">
    <skos:altLabel xml:lang="de">Wertpapier-Futures</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14027-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15272-6">
    <skos:broader rdf:resource="thsys/72186"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15272-6">
    <skos:broader rdf:resource="descriptor/15258-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70029">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">B.06.04</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19673-3">
    <skos:related rdf:resource="descriptor/12438-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19673-3">
    <skos:related rdf:resource="descriptor/12427-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19673-3">
    <skos:related rdf:resource="descriptor/12423-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70198">
    <rdfs:label xml:lang="de">Geographische Begriffe</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73300">
    <skos:prefLabel xml:lang="de">P.20.05  Getränke</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19834-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17994-0">
    <skos:prefLabel xml:lang="en">CMEA countries</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17994-0">
    <skos:prefLabel xml:lang="de">RGW-Staaten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18716-2">
    <skos:broader rdf:resource="thsys/70900"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18716-2">
    <skos:broader rdf:resource="descriptor/18717-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13775-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19387-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19174-0">
    <skos:prefLabel xml:lang="en">CARICOM countries</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19174-0">
    <skos:prefLabel xml:lang="de">CARICOM-Staaten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70505">
    <skos:prefLabel xml:lang="en">G.01.02  Northern Europe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70505">
    <skos:prefLabel xml:lang="de">G.01.02  Nordeuropa</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71052">
    <skos:prefLabel xml:lang="de">V.13.06.03  Führungskräfte und akademische Berufe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13410-2">
    <skos:related rdf:resource="descriptor/18793-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13410-2">
    <skos:related rdf:resource="descriptor/15971-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13410-2">
    <skos:related rdf:resource="descriptor/15970-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13410-2">
    <skos:related rdf:resource="descriptor/13995-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13410-2">
    <skos:related rdf:resource="descriptor/13389-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13410-2">
    <skos:related rdf:resource="descriptor/15966-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13410-2">
    <skos:related rdf:resource="descriptor/15962-2"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id128">
    <rdf:value xml:lang="de">Für Binnenhäfen</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19171-6">
    <skos:prefLabel xml:lang="en">APEC Countries</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19171-6">
    <skos:prefLabel xml:lang="de">APEC-Staaten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18725-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13397-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18384-1">
    <skos:broader rdf:resource="thsys/71009"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18384-1">
    <skos:broader rdf:resource="thsys/70878"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18384-1">
    <skos:broader rdf:resource="descriptor/10436-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19615-3">
    <skos:prefLabel xml:lang="en">Earthquake</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19615-3">
    <skos:prefLabel xml:lang="de">Erdbeben</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18891-5">
    <skos:related rdf:resource="descriptor/13371-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18891-5">
    <skos:related rdf:resource="descriptor/16479-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18891-5">
    <skos:related rdf:resource="descriptor/16635-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14337-2">
    <skos:broader rdf:resource="thsys/70549"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14337-2">
    <skos:broader rdf:resource="descriptor/14336-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16373-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12376-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19060-1">
    <skos:broader rdf:resource="thsys/70960"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19060-1">
    <skos:broader rdf:resource="descriptor/13219-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19060-1">
    <skos:broader rdf:resource="descriptor/13209-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18555-0">
    <skos:altLabel xml:lang="de">Preispolitik des Einzelhandels</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70406">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14643-2">
    <skos:related rdf:resource="descriptor/14300-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73365">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14317-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12307-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18940-4">
    <skos:related rdf:resource="descriptor/14514-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18940-4">
    <skos:related rdf:resource="descriptor/18767-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18940-4">
    <skos:related rdf:resource="descriptor/20341-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17947-2">
    <skos:broader rdf:resource="thsys/70128"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17947-2">
    <skos:broader rdf:resource="descriptor/17948-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10392-0">
    <skos:related rdf:resource="descriptor/11494-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10030-6">
    <skos:prefLabel xml:lang="en">Goods</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10030-6">
    <skos:prefLabel xml:lang="de">Güter</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#semanticRelation">
    <rdfs:domain rdf:resource="http://www.w3.org/2004/02/skos/core#Concept"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12665-2">
    <zbwext:useInsteadNote rdf:nodeID="id195"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12246-1">
    <skos:altLabel xml:lang="en">Guarantee</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18134-3">
    <skos:prefLabel xml:lang="en">Saxony</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18134-3">
    <skos:prefLabel xml:lang="de">Sachsen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10696-1">
    <skos:altLabel xml:lang="de">Freier Welthandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17599-6">
    <skos:prefLabel xml:lang="en">Myanmar</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17599-6">
    <skos:prefLabel xml:lang="de">Myanmar</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id14">
    <rdf:value xml:lang="de">Für den Bau von Industrie- und Verwaltungsgebäuden</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11535-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14396-0">
    <skos:prefLabel xml:lang="en">Industrial truck</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14396-0">
    <skos:prefLabel xml:lang="de">Flurförderzeug</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13894-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13361-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71073">
    <skos:narrower rdf:resource="descriptor/16122-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71073">
    <skos:narrower rdf:resource="descriptor/11710-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71073">
    <skos:narrower rdf:resource="descriptor/16087-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71073">
    <skos:narrower rdf:resource="descriptor/11640-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71073">
    <skos:narrower rdf:resource="descriptor/16112-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71073">
    <skos:narrower rdf:resource="descriptor/16111-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71073">
    <skos:narrower rdf:resource="descriptor/10830-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71073">
    <skos:narrower rdf:resource="descriptor/16110-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71073">
    <skos:narrower rdf:resource="descriptor/16104-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71073">
    <skos:narrower rdf:resource="descriptor/10829-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71073">
    <skos:narrower rdf:resource="descriptor/16100-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71073">
    <skos:narrower rdf:resource="descriptor/18831-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71073">
    <skos:narrower rdf:resource="descriptor/11525-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71073">
    <skos:narrower rdf:resource="descriptor/11532-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71073">
    <skos:narrower rdf:resource="descriptor/16090-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71073">
    <skos:narrower rdf:resource="descriptor/16092-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71073">
    <skos:narrower rdf:resource="descriptor/10828-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71073">
    <skos:narrower rdf:resource="descriptor/11698-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71073">
    <skos:narrower rdf:resource="descriptor/10827-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19553-6">
    <skos:related rdf:resource="descriptor/16619-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19553-6">
    <skos:related rdf:resource="descriptor/10622-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19553-6">
    <skos:related rdf:resource="descriptor/19550-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19553-6">
    <skos:related rdf:resource="descriptor/10488-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15910-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19066-3">
    <skos:broader rdf:resource="thsys/70943"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19066-3">
    <skos:broader rdf:resource="thsys/70883"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19066-3">
    <skos:broader rdf:resource="descriptor/12254-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10928-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14436-0">
    <skos:prefLabel xml:lang="en">Truck</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14436-0">
    <skos:prefLabel xml:lang="de">Lastkraftwagen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12175-5">
    <skos:prefLabel xml:lang="en">Supervisory board</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12175-5">
    <skos:prefLabel xml:lang="de">Aufsichtsrat</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18629-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13709-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12473-4">
    <skos:altLabel xml:lang="de">Kosten- und Erlös-Informationssystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12473-4">
    <skos:altLabel xml:lang="de">EDV-gestützte Kostenrechnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12473-4">
    <skos:altLabel xml:lang="de">Computergestützte Kostenrechnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11818-1">
    <skos:prefLabel xml:lang="en">Crafts board</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11818-1">
    <skos:prefLabel xml:lang="de">Handwerkskammer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18682-0">
    <skos:related rdf:resource="descriptor/13743-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18682-0">
    <skos:related rdf:resource="descriptor/16080-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18682-0">
    <skos:related rdf:resource="descriptor/13742-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18682-0">
    <skos:related rdf:resource="descriptor/13769-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18682-0">
    <skos:related rdf:resource="descriptor/12265-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18682-0">
    <skos:related rdf:resource="descriptor/12299-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18682-0">
    <skos:related rdf:resource="descriptor/13713-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13401-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16588-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16480-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10220-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11339-6">
    <skos:narrower rdf:resource="descriptor/11348-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11339-6">
    <skos:narrower rdf:resource="descriptor/11340-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16290-6">
    <skos:prefLabel xml:lang="en">Political change</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16290-6">
    <skos:prefLabel xml:lang="de">Politischer Wandel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19470-3">
    <skos:related rdf:resource="descriptor/11627-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19470-3">
    <skos:related rdf:resource="descriptor/18345-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17787-5">
    <skos:altLabel xml:lang="de">Republik Kuba</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13071-5">
    <skos:related rdf:resource="descriptor/14750-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18873-0">
    <skos:altLabel xml:lang="de">Universitätsbibliothek</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20897-6">
    <skos:altLabel xml:lang="en">Hub-and-spoke transportation system</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20897-6">
    <skos:altLabel xml:lang="en">Hub-and-spoke transportation network</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20897-6">
    <skos:altLabel xml:lang="de">Hub-and-Spoke-Verkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20897-6">
    <skos:altLabel xml:lang="de">Hub-and-Spoke-Netz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20897-6">
    <skos:altLabel xml:lang="de">Hub-and-Spoke Verkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20897-6">
    <skos:altLabel xml:lang="de">Hub-and-Spoke Netz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70902">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15682-1">
    <skos:prefLabel xml:lang="en">Weltanschauung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15682-1">
    <skos:prefLabel xml:lang="de">Weltanschauung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10227-1">
    <skos:altLabel xml:lang="en">Competitive-equilibrium model</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10227-1">
    <skos:altLabel xml:lang="en">Competitive equilibrium model</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10227-1">
    <skos:altLabel xml:lang="de">Vollständiger Wettbewerb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10227-1">
    <skos:altLabel xml:lang="de">Vollständige Konkurrenz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10227-1">
    <skos:altLabel xml:lang="de">Konkurrenzgleichgewicht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18364-0">
    <skos:related rdf:resource="descriptor/15775-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18364-0">
    <skos:related rdf:resource="descriptor/18362-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12198-0">
    <skos:broader rdf:resource="thsys/70087"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12198-0">
    <skos:broader rdf:resource="descriptor/12181-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15713-2">
    <skos:altLabel xml:lang="de">Soziale Benachteiligung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11164-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11295-0">
    <skos:broader rdf:resource="thsys/73385"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11295-0">
    <skos:broader rdf:resource="thsys/71058"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70209">
    <skos:narrower rdf:resource="descriptor/15778-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70209">
    <skos:narrower rdf:resource="descriptor/10111-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70209">
    <skos:narrower rdf:resource="descriptor/15074-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70209">
    <skos:narrower rdf:resource="descriptor/15773-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70209">
    <skos:narrower rdf:resource="descriptor/15767-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70209">
    <skos:narrower rdf:resource="descriptor/19633-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70209">
    <skos:narrower rdf:resource="descriptor/15760-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70209">
    <skos:narrower rdf:resource="descriptor/15759-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70209">
    <skos:narrower rdf:resource="descriptor/19442-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70209">
    <skos:narrower rdf:resource="descriptor/18072-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70209">
    <skos:narrower rdf:resource="descriptor/18325-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70209">
    <skos:narrower rdf:resource="descriptor/10097-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70209">
    <skos:narrower rdf:resource="descriptor/10411-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70209">
    <skos:narrower rdf:resource="descriptor/19831-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70209">
    <skos:narrower rdf:resource="descriptor/10092-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70209">
    <skos:narrower rdf:resource="descriptor/15776-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70209">
    <skos:narrower rdf:resource="descriptor/18957-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70209">
    <skos:narrower rdf:resource="thsys/70238"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70209">
    <skos:narrower rdf:resource="thsys/70239"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16156-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18241-2">
    <skos:prefLabel xml:lang="en">Container terminal</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18241-2">
    <skos:prefLabel xml:lang="de">Containerhafen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73298">
    <skos:broader rdf:resource="thsys/70498"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13557-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15472-5">
    <skos:altLabel xml:lang="en">Group decision making</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15472-5">
    <skos:altLabel xml:lang="en">Committee decision making</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15472-5">
    <skos:altLabel xml:lang="de">Multipersonale Entscheidung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15472-5">
    <skos:altLabel xml:lang="de">Kollektiventscheidung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15472-5">
    <skos:altLabel xml:lang="de">Kollektive Entscheidung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15472-5">
    <skos:altLabel xml:lang="de">Interpersonale Entscheidung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19495-1">
    <skos:broader rdf:resource="thsys/71062"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19495-1">
    <skos:broader rdf:resource="thsys/71060"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19495-1">
    <skos:broader rdf:resource="descriptor/11219-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16159-4">
    <skos:narrower rdf:resource="descriptor/16160-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16159-4">
    <skos:narrower rdf:resource="descriptor/16158-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16159-4">
    <skos:narrower rdf:resource="descriptor/16156-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16159-4">
    <skos:narrower rdf:resource="descriptor/16155-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16159-4">
    <skos:narrower rdf:resource="descriptor/16143-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16159-4">
    <skos:narrower rdf:resource="descriptor/16481-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16159-4">
    <skos:narrower rdf:resource="descriptor/18904-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19550-5">
    <skos:related rdf:resource="descriptor/19553-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19550-5">
    <skos:related rdf:resource="descriptor/10492-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16605-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id363">
    <rdfs:seeAlso rdf:resource="descriptor/12320-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14829-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10822-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12846-5">
    <skos:related rdf:resource="descriptor/12845-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12846-5">
    <skos:related rdf:resource="descriptor/12837-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12846-5">
    <skos:related rdf:resource="descriptor/12829-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12846-5">
    <skos:related rdf:resource="descriptor/15858-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12846-5">
    <skos:related rdf:resource="descriptor/12844-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12846-5">
    <skos:related rdf:resource="descriptor/12818-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12846-5">
    <skos:related rdf:resource="descriptor/19747-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14440-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70561">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11965-2">
    <skos:altLabel xml:lang="de">Städtestatistik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18636-0">
    <skos:altLabel xml:lang="de">Umweltverträglicher Tourismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18636-0">
    <skos:altLabel xml:lang="de">Umweltschonender Tourismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18636-0">
    <skos:altLabel xml:lang="de">Sanfter Tourismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18636-0">
    <skos:altLabel xml:lang="de">Ökotourismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70562">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19328-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18787-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15494-2">
    <skos:altLabel xml:lang="de">Vollständige Enumeration</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15494-2">
    <skos:altLabel xml:lang="de">Vollenumeration</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16716-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14281-3">
    <skos:prefLabel xml:lang="en">Ferro-alloy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14281-3">
    <skos:prefLabel xml:lang="de">Ferrolegierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14070-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19226-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13052-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10434-3">
    <skos:broader rdf:resource="thsys/70406"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10434-3">
    <skos:broader rdf:resource="descriptor/10249-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11976-4">
    <skos:broader rdf:resource="thsys/71064"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12747-0">
    <skos:prefLabel xml:lang="en">Production scheduling</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12747-0">
    <skos:prefLabel xml:lang="de">Ablaufplanung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12033-4">
    <skos:broader rdf:resource="thsys/73392"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12033-4">
    <skos:broader rdf:resource="thsys/70040"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12033-4">
    <skos:broader rdf:resource="descriptor/16039-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14916-0">
    <skos:broader rdf:resource="thsys/70031"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14916-0">
    <skos:broader rdf:resource="descriptor/14129-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10329-0">
    <skos:broader rdf:resource="thsys/70406"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10329-0">
    <skos:broader rdf:resource="thsys/70233"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10329-0">
    <skos:broader rdf:resource="descriptor/10256-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10227-1">
    <skos:broader rdf:resource="thsys/71119"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10227-1">
    <skos:broader rdf:resource="thsys/70242"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10227-1">
    <skos:broader rdf:resource="descriptor/10956-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10227-1">
    <skos:broader rdf:resource="descriptor/10905-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71029">
    <rdfs:label xml:lang="en">Plant production</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71029">
    <rdfs:label xml:lang="de">Pflanzenbau</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16247-0">
    <skos:related rdf:resource="descriptor/16295-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16247-0">
    <skos:related rdf:resource="descriptor/16286-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16247-0">
    <skos:related rdf:resource="descriptor/11047-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16247-0">
    <skos:related rdf:resource="descriptor/16249-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16397-2">
    <skos:altLabel xml:lang="de">Anarchismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15664-3">
    <skos:broader rdf:resource="thsys/70151"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15664-3">
    <skos:broader rdf:resource="descriptor/15685-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15858-4">
    <skos:prefLabel xml:lang="en">Advertising psychology</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15858-4">
    <skos:prefLabel xml:lang="de">Werbepsychologie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10250-6">
    <skos:related rdf:resource="descriptor/11473-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10250-6">
    <skos:related rdf:resource="descriptor/19229-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10250-6">
    <skos:related rdf:resource="descriptor/10249-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10250-6">
    <skos:related rdf:resource="descriptor/10377-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15107-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21315-0">
    <skos:scopeNote xml:lang="de">Werbung im Internet: Z.B. Bannerwerbung und Werbe-Mails.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id103">
    <rdf:value xml:lang="de">Für Bundesstaaten</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70171">
    <skos:prefLabel xml:lang="de">V.06.01  Wirtschaftssysteme</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16674-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10673-6">
    <skos:related rdf:resource="descriptor/10771-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18200-2">
    <skos:broader rdf:resource="thsys/73332"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18200-2">
    <skos:broader rdf:resource="thsys/71010"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18200-2">
    <skos:broader rdf:resource="descriptor/10593-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14759-4">
    <skos:broader rdf:resource="thsys/73305"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14759-4">
    <skos:broader rdf:resource="descriptor/14752-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13255-2">
    <skos:broader rdf:resource="thsys/70941"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13255-2">
    <skos:broader rdf:resource="descriptor/13510-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19687-6">
    <skos:prefLabel xml:lang="en">Citizenship</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19687-6">
    <skos:prefLabel xml:lang="de">Staatsangehörigkeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12212-4">
    <skos:prefLabel xml:lang="en">Portfolio management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12212-4">
    <skos:prefLabel xml:lang="de">Portfolio-Management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11036-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12812-1">
    <skos:broader rdf:resource="thsys/70459"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12812-1">
    <skos:broader rdf:resource="descriptor/19140-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19299-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14759-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12518-1">
    <skos:prefLabel xml:lang="en">Differential costing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12518-1">
    <skos:prefLabel xml:lang="de">Grenzplankostenrechnung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11845-5">
    <skos:altLabel xml:lang="de">Wohnungsbausubvention</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11845-5">
    <skos:altLabel xml:lang="de">Wohnungsbaurecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11845-5">
    <skos:altLabel xml:lang="de">Wohnungsbaugesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11845-5">
    <skos:altLabel xml:lang="de">Wohnungsbauförderungsgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11845-5">
    <skos:altLabel xml:lang="de">Wohnungsbauförderung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11845-5">
    <skos:altLabel xml:lang="de">Wohnbauförderung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11845-5">
    <skos:altLabel xml:lang="de">Öffentliches Baudarlehen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11845-5">
    <skos:altLabel xml:lang="de">Neubauförderung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18287-6">
    <skos:prefLabel xml:lang="en">Storage tank</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18287-6">
    <skos:prefLabel xml:lang="de">Tanklager</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11090-4">
    <skos:altLabel xml:lang="de">Wirtschaftstheorie des Sozialismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11090-4">
    <skos:altLabel xml:lang="de">Politische Ökonomie des Marxismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11090-4">
    <skos:altLabel xml:lang="de">Marxistische Wirtschaftstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11090-4">
    <skos:altLabel xml:lang="de">Marxistische Politische Ökonomie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70086">
    <skos:narrower rdf:resource="descriptor/17804-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70086">
    <skos:narrower rdf:resource="descriptor/17801-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70086">
    <skos:narrower rdf:resource="descriptor/17800-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70086">
    <skos:narrower rdf:resource="descriptor/17799-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70086">
    <skos:narrower rdf:resource="descriptor/17793-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70086">
    <skos:narrower rdf:resource="descriptor/17792-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70086">
    <skos:narrower rdf:resource="descriptor/17791-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70086">
    <skos:narrower rdf:resource="descriptor/17790-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70086">
    <skos:narrower rdf:resource="descriptor/17787-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70086">
    <skos:narrower rdf:resource="descriptor/17785-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70086">
    <skos:narrower rdf:resource="descriptor/17784-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70086">
    <skos:narrower rdf:resource="descriptor/17783-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70086">
    <skos:narrower rdf:resource="descriptor/17782-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70086">
    <skos:narrower rdf:resource="descriptor/17781-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70086">
    <skos:narrower rdf:resource="descriptor/17780-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70086">
    <skos:narrower rdf:resource="descriptor/17779-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70086">
    <skos:narrower rdf:resource="descriptor/17778-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70086">
    <skos:narrower rdf:resource="descriptor/17777-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70086">
    <skos:narrower rdf:resource="descriptor/17773-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70086">
    <skos:narrower rdf:resource="descriptor/17772-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70086">
    <skos:narrower rdf:resource="descriptor/17771-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70086">
    <skos:narrower rdf:resource="descriptor/17770-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19191-0">
    <skos:broader rdf:resource="thsys/71028"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19191-0">
    <skos:broader rdf:resource="descriptor/12041-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19191-0">
    <skos:broader rdf:resource="descriptor/18318-0"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id307">
    <rdf:value xml:lang="de">Für städtische Wirtschaftsentwicklung</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18087-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70175">
    <skos:broader rdf:resource="thsys/v"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11111-1">
    <skos:narrower rdf:resource="descriptor/11079-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11111-1">
    <skos:narrower rdf:resource="descriptor/11071-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11704-2">
    <skos:narrower rdf:resource="descriptor/11679-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11704-2">
    <skos:narrower rdf:resource="descriptor/10404-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11704-2">
    <skos:narrower rdf:resource="descriptor/19404-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11704-2">
    <skos:narrower rdf:resource="descriptor/19567-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11704-2">
    <skos:narrower rdf:resource="descriptor/19408-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11704-2">
    <skos:narrower rdf:resource="descriptor/21014-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11704-2">
    <skos:narrower rdf:resource="descriptor/11706-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11704-2">
    <skos:narrower rdf:resource="descriptor/11703-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11704-2">
    <skos:narrower rdf:resource="descriptor/21011-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11704-2">
    <skos:narrower rdf:resource="descriptor/11702-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11704-2">
    <skos:narrower rdf:resource="descriptor/11660-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11704-2">
    <skos:narrower rdf:resource="descriptor/19413-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11911-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11751-0">
    <skos:narrower rdf:resource="descriptor/19234-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12882-1">
    <skos:narrower rdf:resource="descriptor/12880-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12882-1">
    <skos:narrower rdf:resource="descriptor/12891-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12882-1">
    <skos:narrower rdf:resource="descriptor/12883-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12529-3">
    <skos:related rdf:resource="descriptor/10931-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12529-3">
    <skos:related rdf:resource="descriptor/18555-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13658-1">
    <skos:altLabel xml:lang="de">Medien</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13658-1">
    <skos:altLabel xml:lang="de">Massenmedien</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13658-1">
    <skos:altLabel xml:lang="de">Audiovisuelle Medien</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19488-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15979-6">
    <skos:narrower rdf:resource="descriptor/16304-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15298-2">
    <skos:altLabel xml:lang="en">Test statistics</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15298-2">
    <skos:altLabel xml:lang="en">Test for equality</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15298-2">
    <skos:altLabel xml:lang="en">Power function</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15298-2">
    <skos:altLabel xml:lang="en">Dominance test</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15298-2">
    <skos:altLabel xml:lang="de">Testtheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15298-2">
    <skos:altLabel xml:lang="de">Teststärke</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15298-2">
    <skos:altLabel xml:lang="de">Test (Statistik)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15298-2">
    <skos:altLabel xml:lang="de">Test auf Gleichheit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15298-2">
    <skos:altLabel xml:lang="de">Signifikanzniveau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15298-2">
    <skos:altLabel xml:lang="de">Parametertest</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15298-2">
    <skos:altLabel xml:lang="de">Macht eines Tests</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15298-2">
    <skos:altLabel xml:lang="de">Hypothesentest</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15298-2">
    <skos:altLabel xml:lang="de">Homogenitätstest</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15298-2">
    <skos:altLabel xml:lang="de">Gütefunktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15298-2">
    <skos:altLabel xml:lang="de">Fehler zweiter Art</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15298-2">
    <skos:altLabel xml:lang="de">Fehler erster Art</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15298-2">
    <skos:altLabel xml:lang="de">Durbin-Watson-Test</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15298-2">
    <skos:altLabel xml:lang="de">Differenzentest</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15621-0">
    <skos:narrower rdf:resource="descriptor/19804-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15621-0">
    <skos:narrower rdf:resource="descriptor/18020-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15621-0">
    <skos:narrower rdf:resource="descriptor/24784-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15621-0">
    <skos:narrower rdf:resource="descriptor/18139-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10595-0">
    <skos:broader rdf:resource="thsys/71085"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10595-0">
    <skos:broader rdf:resource="descriptor/10555-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71062">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14217-5">
    <skos:related rdf:resource="descriptor/14218-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10298-6">
    <skos:altLabel xml:lang="de">Verbraucherinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10298-6">
    <skos:altLabel xml:lang="de">Nachfragerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10298-6">
    <skos:altLabel xml:lang="de">Nachfrager</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10298-6">
    <skos:altLabel xml:lang="de">Konsumentinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10298-6">
    <skos:altLabel xml:lang="de">Konsumenten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10298-6">
    <skos:altLabel xml:lang="de">Konsument</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16976-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18376-0">
    <zbwext:useInsteadNote rdf:nodeID="id84"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12702-1">
    <skos:related rdf:resource="descriptor/12805-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12702-1">
    <skos:related rdf:resource="descriptor/10459-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12702-1">
    <skos:related rdf:resource="descriptor/10443-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12702-1">
    <skos:related rdf:resource="descriptor/18495-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12702-1">
    <skos:related rdf:resource="descriptor/18492-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19083-3">
    <skos:broader rdf:resource="thsys/70035"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19083-3">
    <skos:broader rdf:resource="descriptor/16815-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13020-1">
    <skos:scopeNote xml:lang="de">Verarbeitung von Mineralien, die weder zu den Metallen noch zu den Energierohstoffen zählen</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18320-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15762-3">
    <skos:related rdf:resource="descriptor/15769-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15762-3">
    <skos:related rdf:resource="descriptor/15768-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15762-3">
    <skos:related rdf:resource="descriptor/19483-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15762-3">
    <skos:related rdf:resource="descriptor/19473-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16708-4">
    <skos:scopeNote xml:lang="de">Im weiteren Sinne die Lehre vom Wesen und den Voraussetzungen von Erkenntnis.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19032-6">
    <skos:editorialNote xml:lang="de">Sparsam verwenden.</skos:editorialNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10658-2">
    <skos:altLabel xml:lang="en">Intra-regional trade</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10658-2">
    <skos:altLabel xml:lang="de">Binnenhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11064-5">
    <skos:broader rdf:resource="thsys/70171"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11064-5">
    <skos:broader rdf:resource="descriptor/11068-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13398-1">
    <skos:altLabel xml:lang="de">Privathaushalt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13398-1">
    <skos:altLabel xml:lang="de">Haushalt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18328-4">
    <skos:related rdf:resource="descriptor/12032-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18328-4">
    <skos:related rdf:resource="descriptor/12023-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18328-4">
    <skos:related rdf:resource="descriptor/11921-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18328-4">
    <skos:related rdf:resource="descriptor/11859-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18067-6">
    <skos:altLabel xml:lang="de">Schadenersatzhaftung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18067-6">
    <skos:altLabel xml:lang="de">Haftungsrisiko</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18067-6">
    <skos:altLabel xml:lang="de">Haftungsrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18067-6">
    <skos:altLabel xml:lang="de">Haftpflicht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13161-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18332-6">
    <skos:narrower rdf:resource="descriptor/11806-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13523-3">
    <skos:broader rdf:resource="thsys/70941"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13523-3">
    <skos:broader rdf:resource="descriptor/12013-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13299-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12805-5">
    <skos:altLabel xml:lang="en">Industrial design</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12805-5">
    <skos:altLabel xml:lang="de">Produktform</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12805-5">
    <skos:altLabel xml:lang="de">Produktfarbe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12805-5">
    <skos:altLabel xml:lang="de">Industriedesign</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13034-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17938-3">
    <skos:prefLabel xml:lang="en">Kiribati</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17938-3">
    <skos:prefLabel xml:lang="de">Kiribati</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19477-3">
    <skos:broader rdf:resource="thsys/71064"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19477-3">
    <skos:broader rdf:resource="thsys/70182"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19477-3">
    <skos:broader rdf:resource="descriptor/11894-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17695-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10276-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11823-1">
    <skos:narrower rdf:resource="descriptor/18545-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14603-0">
    <skos:editorialNote xml:lang="de">Möglichst spezifischer indexieren.</skos:editorialNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15103-1">
    <skos:broader rdf:resource="thsys/72181"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15103-1">
    <skos:broader rdf:resource="thsys/71005"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15103-1">
    <skos:broader rdf:resource="descriptor/15076-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18942-0">
    <skos:broader rdf:resource="thsys/73345"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18942-0">
    <skos:broader rdf:resource="thsys/70823"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18942-0">
    <skos:broader rdf:resource="descriptor/16754-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18226-5">
    <skos:altLabel xml:lang="x-other">Slovenija</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18226-5">
    <skos:altLabel xml:lang="de">Republik Slowenien</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12824-1">
    <skos:broader rdf:resource="thsys/70384"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12824-1">
    <skos:broader rdf:resource="descriptor/12846-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12824-1">
    <skos:broader rdf:resource="descriptor/12780-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19737-3">
    <skos:altLabel xml:lang="de">Monastery</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19737-3">
    <skos:altLabel xml:lang="de">Kloster</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11206-4">
    <skos:altLabel xml:lang="en">Regional division of labor</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11206-4">
    <skos:altLabel xml:lang="de">Regionale Spezialisierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11206-4">
    <skos:altLabel xml:lang="de">Räumliche Arbeitsteilung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11206-4">
    <skos:altLabel xml:lang="de">Interregionale Arbeitsteilung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70882">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18745-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11741-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19444-4">
    <skos:altLabel xml:lang="de">Verstädterungseffekt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19444-4">
    <skos:altLabel xml:lang="de">Urbanisierungsvorteil</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19444-4">
    <skos:altLabel xml:lang="de">Urbanisierungsnachteil</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19444-4">
    <skos:altLabel xml:lang="de">Urbanisationseffekt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10255-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73372">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15714-0">
    <skos:broader rdf:resource="thsys/73386"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15714-0">
    <skos:broader rdf:resource="thsys/73358"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13325-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19218-6">
    <skos:prefLabel xml:lang="en">Marginal analysis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19218-6">
    <skos:prefLabel xml:lang="de">Marginalanalyse</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11669-6">
    <skos:prefLabel xml:lang="en">Wage tax</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11669-6">
    <skos:prefLabel xml:lang="de">Lohnsteuer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14230-6">
    <skos:prefLabel xml:lang="en">Mica</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14230-6">
    <skos:prefLabel xml:lang="de">Glimmer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19216-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15969-2">
    <skos:related rdf:resource="descriptor/15976-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15969-2">
    <skos:related rdf:resource="descriptor/15966-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15969-2">
    <skos:related rdf:resource="descriptor/19768-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11562-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12198-0">
    <skos:related rdf:resource="descriptor/12184-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14931-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id36">
    <rdf:value xml:lang="de">Für Auslandsaufträge des produzierenden Gewerbes</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11526-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70040">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71045">
    <skos:broader rdf:resource="thsys/70183"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19065-5">
    <skos:prefLabel xml:lang="en">Land development</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19065-5">
    <skos:prefLabel xml:lang="de">Grundstückserschließung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70416">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">P.13</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13736-0">
    <skos:altLabel xml:lang="en">Covered bond</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13085-1">
    <skos:narrower rdf:resource="descriptor/13090-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18704-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18008-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12436-3">
    <skos:related rdf:resource="descriptor/12435-5"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id113">
    <rdf:value xml:lang="de">Für betriebswirtschaftliche Ansätze der Investitionstheorie</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16476-6">
    <skos:related rdf:resource="descriptor/16479-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16476-6">
    <skos:related rdf:resource="descriptor/20409-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16476-6">
    <skos:related rdf:resource="descriptor/16668-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16476-6">
    <skos:related rdf:resource="descriptor/14723-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71100">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17709-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18001-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10327-4">
    <skos:altLabel xml:lang="en">Aggregate production function</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10327-4">
    <skos:altLabel xml:lang="de">Produktionsfunktion der Gesamtwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10327-4">
    <skos:altLabel xml:lang="de">Makroökonomische Produktionsfunktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10327-4">
    <skos:altLabel xml:lang="de">Gesamtwirtschaftliche Produktionsfunktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10327-4">
    <skos:altLabel xml:lang="de">Aggregierte Produktionsfunktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19478-1">
    <skos:prefLabel xml:lang="en">Population structure of communities</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19478-1">
    <skos:prefLabel xml:lang="de">Städtische Bevölkerungsstruktur</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13136-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15266-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15709-0">
    <skos:editorialNote xml:lang="de">Möglichst spezifischer indexieren.</skos:editorialNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13244-0">
    <skos:related rdf:resource="descriptor/14436-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13244-0">
    <skos:related rdf:resource="descriptor/13558-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13244-0">
    <skos:related rdf:resource="descriptor/18582-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18663-4">
    <skos:scopeNote xml:lang="de">Gesamtheit aller Einrichtungen und Anlagen im Bereich der Telekommunikation.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16525-5">
    <skos:broader rdf:resource="thsys/70834"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16525-5">
    <skos:broader rdf:resource="descriptor/16475-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19336-0">
    <skos:prefLabel xml:lang="en">European Economic and Monetary Union</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19336-0">
    <skos:prefLabel xml:lang="de">Europäische Wirtschafts- und Währungsunion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16245-4">
    <skos:narrower rdf:resource="descriptor/16247-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16547-2">
    <skos:altLabel xml:lang="de">Sozialabgabe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16547-2">
    <skos:altLabel xml:lang="de">Arbeitgeberbeitrag</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13446-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12276-6">
    <skos:prefLabel xml:lang="en">Payout policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12276-6">
    <skos:prefLabel xml:lang="de">Gewinnverwendung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13704-6">
    <skos:related rdf:resource="descriptor/18681-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10805-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18630-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13514-4">
    <skos:broader rdf:resource="thsys/70925"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13514-4">
    <skos:broader rdf:resource="descriptor/12013-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18924-2">
    <zbwext:useInsteadNote rdf:nodeID="id182"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14488-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19098-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13137-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14981-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11388-0">
    <skos:altLabel xml:lang="de">Berufsförderung Behinderter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11388-0">
    <skos:altLabel xml:lang="de">Berufliche Behindertenrehabilitation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11388-0">
    <skos:altLabel xml:lang="de">Arbeitsförderung Behinderter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13064-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13283-4">
    <skos:prefLabel xml:lang="en">Bank</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13283-4">
    <skos:prefLabel xml:lang="de">Bank</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16648-3">
    <skos:related rdf:resource="descriptor/16515-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16648-3">
    <skos:related rdf:resource="descriptor/15654-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16648-3">
    <skos:related rdf:resource="descriptor/19712-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16648-3">
    <skos:related rdf:resource="descriptor/16489-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19681-4">
    <skos:broader rdf:resource="thsys/73382"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19681-4">
    <skos:broader rdf:resource="descriptor/16305-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19681-4">
    <skos:broader rdf:resource="descriptor/16278-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11793-5">
    <skos:related rdf:resource="descriptor/10414-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11793-5">
    <skos:related rdf:resource="descriptor/11788-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11793-5">
    <skos:related rdf:resource="descriptor/10533-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11793-5">
    <skos:related rdf:resource="descriptor/24782-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11793-5">
    <skos:related rdf:resource="descriptor/19248-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18932-3">
    <skos:altLabel xml:lang="de">Zuschauerverhalten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18932-3">
    <skos:altLabel xml:lang="de">Mediennutzung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18932-3">
    <skos:altLabel xml:lang="de">Leseverhalten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18932-3">
    <skos:altLabel xml:lang="de">Hörfunknutzung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18932-3">
    <skos:altLabel xml:lang="de">Fernsehnutzung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16306-3">
    <skos:broader rdf:resource="thsys/73380"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16306-3">
    <skos:broader rdf:resource="thsys/73375"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16306-3">
    <skos:broader rdf:resource="descriptor/16290-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12209-0">
    <skos:broader rdf:resource="thsys/71044"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12209-0">
    <skos:broader rdf:resource="descriptor/12041-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19841-1">
    <skos:broader rdf:resource="thsys/70862"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19841-1">
    <skos:broader rdf:resource="thsys/70154"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15038-0">
    <skos:narrower rdf:resource="descriptor/15051-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13352-4">
    <skos:altLabel xml:lang="en">Motion picture industry</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13352-4">
    <skos:altLabel xml:lang="de">Videoverleih</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13352-4">
    <skos:altLabel xml:lang="de">Videoindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13352-4">
    <skos:altLabel xml:lang="de">Videofilmverleih</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13352-4">
    <skos:altLabel xml:lang="de">Videofilmproduktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13352-4">
    <skos:altLabel xml:lang="de">Kinofilmproduktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13352-4">
    <skos:altLabel xml:lang="de">Filmverleih</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13352-4">
    <skos:altLabel xml:lang="de">Filmproduzent</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13352-4">
    <skos:altLabel xml:lang="de">Filmproduktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13352-4">
    <skos:altLabel xml:lang="de">Filmindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13352-4">
    <skos:altLabel xml:lang="de">Filmgesellschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13352-4">
    <skos:altLabel xml:lang="de">Filmbranche</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70651">
    <skos:broader rdf:resource="thsys/70538"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15266-1">
    <skos:prefLabel xml:lang="en">Method of moments</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15266-1">
    <skos:prefLabel xml:lang="de">Momentenmethode</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19762-4">
    <skos:prefLabel xml:lang="en">Fathers</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19762-4">
    <skos:prefLabel xml:lang="de">Väter</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73385">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17595-0">
    <skos:broader rdf:resource="thsys/71032"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17595-0">
    <skos:broader rdf:resource="descriptor/17553-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11651-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12573-0">
    <skos:broader rdf:resource="thsys/70244"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12573-0">
    <skos:broader rdf:resource="descriptor/12567-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12573-0">
    <skos:broader rdf:resource="descriptor/12640-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12796-1">
    <skos:related rdf:resource="descriptor/19153-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12796-1">
    <skos:related rdf:resource="descriptor/12062-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12796-1">
    <skos:related rdf:resource="descriptor/12474-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12796-1">
    <skos:related rdf:resource="descriptor/12452-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12796-1">
    <skos:related rdf:resource="descriptor/12520-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12796-1">
    <skos:related rdf:resource="descriptor/12446-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70855">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14731-5">
    <skos:broader rdf:resource="thsys/70199"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14731-5">
    <skos:broader rdf:resource="descriptor/14728-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18908-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12013-3">
    <skos:narrower rdf:resource="descriptor/13514-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12013-3">
    <skos:narrower rdf:resource="descriptor/18090-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12013-3">
    <skos:narrower rdf:resource="descriptor/13533-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12013-3">
    <skos:narrower rdf:resource="descriptor/18617-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12013-3">
    <skos:narrower rdf:resource="descriptor/13516-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12013-3">
    <skos:narrower rdf:resource="descriptor/19610-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12013-3">
    <skos:narrower rdf:resource="descriptor/13523-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18632-1">
    <skos:altLabel xml:lang="en">Regional traffic management</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18632-1">
    <skos:altLabel xml:lang="de">Regionaler Verkehrsplan</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18632-1">
    <skos:altLabel xml:lang="de">Regionale Verkehrsplanung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15021-3">
    <skos:altLabel xml:lang="de">Delphi-Methode</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15021-3">
    <skos:altLabel xml:lang="de">Delphi-Befragung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18676-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18178-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11197-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17735-3">
    <skos:broader rdf:resource="thsys/71031"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17735-3">
    <skos:broader rdf:resource="descriptor/17743-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15429-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11691-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15362-5">
    <skos:prefLabel xml:lang="en">Heteroscedasticity</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15362-5">
    <skos:prefLabel xml:lang="de">Heteroskedastizität</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10446-3">
    <skos:related rdf:resource="descriptor/19244-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19519-6">
    <skos:prefLabel xml:lang="en">Business history</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19519-6">
    <skos:prefLabel xml:lang="de">Business History</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12961-5">
    <skos:altLabel xml:lang="de">Waldwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16509-3">
    <skos:broader rdf:resource="thsys/73373"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16509-3">
    <skos:broader rdf:resource="descriptor/16273-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15656-2">
    <skos:prefLabel xml:lang="en">Learning</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15656-2">
    <skos:prefLabel xml:lang="de">Lernen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19614-5">
    <skos:prefLabel xml:lang="en">Ocean utilization</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19614-5">
    <skos:prefLabel xml:lang="de">Meeresnutzung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12981-6">
    <skos:broader rdf:resource="thsys/71009"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12981-6">
    <skos:broader rdf:resource="descriptor/12990-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14317-1">
    <skos:prefLabel xml:lang="en">Offshore equipment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14317-1">
    <skos:prefLabel xml:lang="de">Offshore-Technik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19854-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10307-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70244">
    <rdfs:label xml:lang="de">Personalführung, -planung und -entwicklung</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18942-0">
    <skos:narrower rdf:resource="descriptor/19128-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14295-6">
    <skos:broader rdf:resource="thsys/70134"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14295-6">
    <skos:broader rdf:resource="descriptor/14274-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14295-6">
    <skos:broader rdf:resource="descriptor/14302-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17988-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13294-6">
    <skos:broader rdf:resource="thsys/70891"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13294-6">
    <skos:broader rdf:resource="descriptor/13828-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14599-0">
    <skos:prefLabel xml:lang="en">Chemical</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14599-0">
    <skos:prefLabel xml:lang="de">Chemikalie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11848-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73305">
    <rdfs:label xml:lang="de">Büromaschinen und Datenverarbeitungsgeräte</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16977-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70851">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10961-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18157-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19606-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18428-0">
    <skos:related rdf:resource="descriptor/18649-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18428-0">
    <skos:related rdf:resource="descriptor/13278-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18428-0">
    <skos:related rdf:resource="descriptor/14490-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18428-0">
    <skos:related rdf:resource="descriptor/13682-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18719-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70416">
    <rdfs:label xml:lang="de">Holzprodukte</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18818-1">
    <skos:broader rdf:resource="thsys/73365"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18818-1">
    <skos:broader rdf:resource="thsys/71098"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18818-1">
    <skos:broader rdf:resource="thsys/70982"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18818-1">
    <skos:broader rdf:resource="descriptor/10692-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18818-1">
    <skos:broader rdf:resource="descriptor/16445-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18139-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12917-1">
    <skos:altLabel xml:lang="en">Small-scale manufacturing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14712-2">
    <skos:altLabel xml:lang="de">PS (Polystyrol)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70140">
    <skos:prefLabel xml:lang="de">B.00  Betriebswirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19468-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id320">
    <rdfs:seeAlso rdf:resource="descriptor/13630-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19482-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12183-6">
    <skos:related rdf:resource="descriptor/12187-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12183-6">
    <skos:related rdf:resource="descriptor/12182-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12183-6">
    <skos:related rdf:resource="descriptor/19729-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18906-4">
    <skos:broader rdf:resource="thsys/70834"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18906-4">
    <skos:broader rdf:resource="descriptor/12047-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16719-6">
    <skos:prefLabel xml:lang="en">Philosophy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16719-6">
    <skos:prefLabel xml:lang="de">Philosophie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13712-0">
    <skos:broader rdf:resource="thsys/72184"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13712-0">
    <skos:broader rdf:resource="thsys/71106"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13712-0">
    <skos:broader rdf:resource="descriptor/15082-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18270-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13217-3">
    <skos:related rdf:resource="descriptor/19549-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13217-3">
    <skos:related rdf:resource="descriptor/13051-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13217-3">
    <skos:related rdf:resource="descriptor/13052-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13217-3">
    <skos:related rdf:resource="descriptor/14433-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13217-3">
    <skos:related rdf:resource="descriptor/13248-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73342">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.10.07</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19301-5">
    <skos:scopeNote xml:lang="de">Nur für die Gesamtheit des Teesektors.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11510-1">
    <skos:prefLabel xml:lang="en">Wage indexation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11510-1">
    <skos:prefLabel xml:lang="de">Lohnindexierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19593-1">
    <skos:broader rdf:resource="thsys/73104"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19593-1">
    <skos:broader rdf:resource="descriptor/15458-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14180-2">
    <skos:altLabel xml:lang="de">Erdwärme</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18951-6">
    <skos:broader rdf:resource="thsys/70822"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18951-6">
    <skos:broader rdf:resource="descriptor/19068-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13635-6">
    <skos:altLabel xml:lang="en">Farm tourism</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13635-6">
    <skos:altLabel xml:lang="en">Agrotourism</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13635-6">
    <skos:altLabel xml:lang="de">Urlaub auf dem Bauernhof</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13635-6">
    <skos:altLabel xml:lang="de">Tourismus auf dem Lande</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13635-6">
    <skos:altLabel xml:lang="de">Landtourismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13635-6">
    <skos:altLabel xml:lang="de">Ländlicher Tourismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13635-6">
    <skos:altLabel xml:lang="de">Bauernhoftourismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14012-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11422-5">
    <skos:altLabel xml:lang="en">Money demand</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11422-5">
    <skos:altLabel xml:lang="de">Geldnachfragefunktion der Gesamtwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71079">
    <skos:broader rdf:resource="thsys/70144"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16193-4">
    <skos:altLabel xml:lang="de">Schmiergeld</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16193-4">
    <skos:altLabel xml:lang="de">Bribery</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16193-4">
    <skos:altLabel xml:lang="de">Bestechung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16193-4">
    <skos:altLabel xml:lang="de">Bestechlichkeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16193-4">
    <skos:altLabel xml:lang="de">Ämterkauf</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11844-0">
    <skos:prefLabel xml:lang="en">Urban construction policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11844-0">
    <skos:prefLabel xml:lang="de">Städtebaupolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11912-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13560-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13561-2">
    <skos:prefLabel xml:lang="en">Road financing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13561-2">
    <skos:prefLabel xml:lang="de">Straßenbaufinanzierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16989-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14364-6">
    <skos:prefLabel xml:lang="en">Wind power plant</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14364-6">
    <skos:prefLabel xml:lang="de">Windkraftanlage</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19664-4">
    <skos:narrower rdf:resource="descriptor/19337-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19664-4">
    <skos:narrower rdf:resource="descriptor/18730-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19664-4">
    <skos:narrower rdf:resource="descriptor/13688-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11064-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18317-2">
    <skos:prefLabel xml:lang="en">Agricultural biotechnology</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18317-2">
    <skos:prefLabel xml:lang="de">Agrarbiotechnologie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18971-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12960-0">
    <skos:altLabel xml:lang="de">Zierpflanzenanbau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12960-0">
    <skos:altLabel xml:lang="de">Blumenzucht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24665-0">
    <skos:prefLabel xml:lang="en">Direct banking</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24665-0">
    <skos:prefLabel xml:lang="de">Direktbank</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11269-1">
    <skos:prefLabel xml:lang="en">Disabled workers</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11269-1">
    <skos:prefLabel xml:lang="de">Behinderte Arbeitskräfte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18479-4">
    <skos:prefLabel xml:lang="en">Small- and medium-scale industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18479-4">
    <skos:prefLabel xml:lang="de">Mittelständische Industrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13225-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14805-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12047-0">
    <skos:prefLabel xml:lang="en">Nonprofit organization</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12047-0">
    <skos:prefLabel xml:lang="de">Nonprofit-Organisation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19419-3">
    <skos:broader rdf:resource="thsys/71074"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19419-3">
    <skos:broader rdf:resource="thsys/71019"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19419-3">
    <skos:broader rdf:resource="descriptor/18338-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10882-4">
    <skos:prefLabel xml:lang="en">Foreign trade institution</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10882-4">
    <skos:prefLabel xml:lang="de">Staatliche Außenhandelsorganisation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id277">
    <rdf:value xml:lang="de">Für die rechtsprechende Gewalt</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16353-1">
    <skos:altLabel xml:lang="en">Voters' behaviour</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16353-1">
    <skos:altLabel xml:lang="de">Wahlsoziologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16353-1">
    <skos:altLabel xml:lang="de">Wahlforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16353-1">
    <skos:altLabel xml:lang="de">Wählerverhalten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16353-1">
    <skos:altLabel xml:lang="de">Wählersoziologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16353-1">
    <skos:altLabel xml:lang="de">Wählerpsychologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18766-1">
    <skos:prefLabel xml:lang="en">Computer programming</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18766-1">
    <skos:prefLabel xml:lang="de">Programmierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13029-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13620-5">
    <skos:narrower rdf:resource="descriptor/13537-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13620-5">
    <skos:narrower rdf:resource="descriptor/18622-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10038-4">
    <skos:prefLabel xml:lang="en">Expectations hypothesis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10038-4">
    <skos:prefLabel xml:lang="de">Erwartungstheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11570-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18911-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18842-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13630-2">
    <skos:related rdf:resource="descriptor/14440-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13630-2">
    <skos:related rdf:resource="descriptor/14437-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11151-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11495-6">
    <skos:narrower rdf:resource="descriptor/11447-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19314-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11430-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10950-6">
    <skos:broader rdf:resource="thsys/70242"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10950-6">
    <skos:broader rdf:resource="descriptor/10891-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13163-0">
    <skos:prefLabel xml:lang="en">Confectionery industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13163-0">
    <skos:prefLabel xml:lang="de">Süßwarenindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18022-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19670-2">
    <skos:broader rdf:resource="thsys/70234"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18664-2">
    <skos:narrower rdf:resource="descriptor/19145-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18535-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18150-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11551-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10267-3">
    <skos:altLabel xml:lang="de">Staatsverbrauch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10267-3">
    <skos:altLabel xml:lang="de">Staatskonsum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10267-3">
    <skos:altLabel xml:lang="de">Öffentlicher Verbrauch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19261-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19156-2">
    <skos:broader rdf:resource="thsys/70087"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16633-2">
    <skos:narrower rdf:resource="descriptor/15764-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12085-6">
    <skos:editorialNote xml:lang="de">Sparsam und möglichst spezifischer indexieren.</skos:editorialNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16504-6">
    <skos:prefLabel xml:lang="en">Impunity</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16504-6">
    <skos:prefLabel xml:lang="de">Straffreiheit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10545-1">
    <skos:scopeNote xml:lang="de">Marktausgleichslager für Rohstoffe und Agrarprodukte zur Exportpreisstabilisierung in Entwicklungsländern.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14516-2">
    <skos:altLabel xml:lang="de">Tonbandgerät</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14516-2">
    <skos:altLabel xml:lang="de">Rundfunkgerät</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14516-2">
    <skos:altLabel xml:lang="de">Radiogerät</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14516-2">
    <skos:altLabel xml:lang="de">Plattenspieler</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14516-2">
    <skos:altLabel xml:lang="de">Mikrophon</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14516-2">
    <skos:altLabel xml:lang="de">Lautsprecher</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14516-2">
    <skos:altLabel xml:lang="de">Kopfhörer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14516-2">
    <skos:altLabel xml:lang="de">Kassettenrecorder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14516-2">
    <skos:altLabel xml:lang="de">Hifi-Gerät</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14516-2">
    <skos:altLabel xml:lang="de">CD-Spieler</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12049-3">
    <skos:narrower rdf:resource="descriptor/12908-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12049-3">
    <skos:narrower rdf:resource="descriptor/13701-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19155-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10161-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71099">
    <skos:broader rdf:resource="thsys/71100"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16521-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73385">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13360-5">
    <skos:altLabel xml:lang="de">Rundfunksender</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13360-5">
    <skos:altLabel xml:lang="de">Rundfunk- und Fernsehanstalt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13360-5">
    <skos:altLabel xml:lang="de">Radiosender</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13360-5">
    <skos:altLabel xml:lang="de">Öffentlich-rechtliche Rundfunkanstalt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13360-5">
    <skos:altLabel xml:lang="de">Hörfunksender</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72394">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id98">
    <rdfs:seeAlso rdf:resource="descriptor/12711-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18668-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14251-5">
    <skos:broader rdf:resource="thsys/70201"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14251-5">
    <skos:broader rdf:resource="descriptor/14021-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10658-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13641-4">
    <skos:related rdf:resource="descriptor/19854-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10040-3">
    <skos:narrower rdf:resource="descriptor/10184-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10040-3">
    <skos:narrower rdf:resource="descriptor/24857-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10040-3">
    <skos:narrower rdf:resource="descriptor/19209-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10040-3">
    <skos:narrower rdf:resource="descriptor/19208-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15734-1">
    <skos:broader rdf:resource="thsys/73351"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15734-1">
    <skos:broader rdf:resource="thsys/71011"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15734-1">
    <skos:broader rdf:resource="thsys/70817"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15734-1">
    <skos:broader rdf:resource="thsys/70239"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15734-1">
    <skos:broader rdf:resource="descriptor/18379-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15734-1">
    <skos:broader rdf:resource="descriptor/15770-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19452-5">
    <skos:broader rdf:resource="thsys/71067"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19452-5">
    <skos:broader rdf:resource="thsys/71064"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19452-5">
    <skos:broader rdf:resource="descriptor/11932-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19452-5">
    <skos:broader rdf:resource="descriptor/11990-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12230-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14197-6">
    <skos:related rdf:resource="descriptor/17985-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14197-6">
    <skos:related rdf:resource="descriptor/14193-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14197-6">
    <skos:related rdf:resource="descriptor/14044-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14197-6">
    <skos:related rdf:resource="descriptor/12920-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14197-6">
    <skos:related rdf:resource="descriptor/14218-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14197-6">
    <skos:related rdf:resource="descriptor/19182-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14197-6">
    <skos:related rdf:resource="descriptor/13221-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14197-6">
    <skos:related rdf:resource="descriptor/19387-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14197-6">
    <skos:related rdf:resource="descriptor/19181-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14197-6">
    <skos:related rdf:resource="descriptor/18373-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14197-6">
    <skos:related rdf:resource="descriptor/19388-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16430-2">
    <skos:broader rdf:resource="thsys/73394"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16430-2">
    <skos:broader rdf:resource="thsys/73383"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16430-2">
    <skos:broader rdf:resource="thsys/73377"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16430-2">
    <skos:broader rdf:resource="descriptor/16709-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24783-1">
    <skos:altLabel xml:lang="en">NPM (New Public Management)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24783-1">
    <skos:altLabel xml:lang="de">Wirkungsorientierte Verwaltungsführung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24783-1">
    <skos:altLabel xml:lang="de">Neues Steuerungsmodell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14383-2">
    <skos:related rdf:resource="descriptor/18095-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14383-2">
    <skos:related rdf:resource="descriptor/14392-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14383-2">
    <skos:related rdf:resource="descriptor/14846-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14383-2">
    <skos:related rdf:resource="descriptor/14829-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11817-3">
    <skos:broader rdf:resource="thsys/71110"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11817-3">
    <skos:broader rdf:resource="thsys/70038"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11817-3">
    <skos:broader rdf:resource="descriptor/10981-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17179-0">
    <skos:related rdf:resource="descriptor/17016-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17179-0">
    <skos:related rdf:resource="descriptor/19077-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17179-0">
    <skos:related rdf:resource="descriptor/18054-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19229-1">
    <skos:prefLabel xml:lang="en">New classical macroeconomics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19229-1">
    <skos:prefLabel xml:lang="de">Neue klassische Makroökonomik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18496-4">
    <skos:altLabel xml:lang="de">Haus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71011">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16525-5">
    <skos:scopeNote xml:lang="de">Ambulante und stationäre Betreuung alter Menschen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13981-1">
    <skos:related rdf:resource="descriptor/18644-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13981-1">
    <skos:related rdf:resource="descriptor/13270-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13981-1">
    <skos:related rdf:resource="descriptor/11559-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16192-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10213-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18914-5">
    <skos:broader rdf:resource="thsys/73335"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18914-5">
    <skos:broader rdf:resource="thsys/70833"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18914-5">
    <skos:broader rdf:resource="thsys/70324"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18914-5">
    <skos:broader rdf:resource="descriptor/16160-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16031-2">
    <skos:prefLabel xml:lang="en">Cooperative legislation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16031-2">
    <skos:prefLabel xml:lang="de">Genossenschaftsrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13163-0">
    <skos:related rdf:resource="descriptor/13237-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13163-0">
    <skos:related rdf:resource="descriptor/14996-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13163-0">
    <skos:related rdf:resource="descriptor/18213-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13163-0">
    <skos:related rdf:resource="descriptor/19296-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13163-0">
    <skos:related rdf:resource="descriptor/14065-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13163-0">
    <skos:related rdf:resource="descriptor/14942-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11438-4">
    <skos:prefLabel xml:lang="en">Silver standard</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11438-4">
    <skos:prefLabel xml:lang="de">Silberwährung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18187-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15728-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11969-1">
    <skos:altLabel xml:lang="de">Verslummung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16704-5">
    <skos:scopeNote xml:lang="de">Geistesbewegung des 17. bis 18. Jahrhunderts (ca. 1680-1790)</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18705-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13075-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19498-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11580-1">
    <skos:broader rdf:resource="thsys/73390"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11580-1">
    <skos:broader rdf:resource="thsys/70859"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11580-1">
    <skos:broader rdf:resource="thsys/70144"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11580-1">
    <skos:broader rdf:resource="descriptor/16007-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11675-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10510-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13274-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12513-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73348">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.10.04</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10110-1">
    <skos:broader rdf:resource="thsys/71113"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10110-1">
    <skos:broader rdf:resource="thsys/70057"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10110-1">
    <skos:broader rdf:resource="descriptor/16623-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15013-2">
    <skos:prefLabel xml:lang="en">Radioactive waste</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15013-2">
    <skos:prefLabel xml:lang="de">Radioaktiver Abfall</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12523-1">
    <skos:narrower rdf:resource="descriptor/12518-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16792-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14327-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15611-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70209">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.12.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19168-2">
    <skos:prefLabel xml:lang="en">Marshall Islands</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19168-2">
    <skos:prefLabel xml:lang="de">Marshallinseln</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11065-3">
    <skos:altLabel xml:lang="de">Theorie der Verfügungsrechte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11065-3">
    <skos:altLabel xml:lang="de">Theorie der Handlungsrechte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11065-3">
    <skos:altLabel xml:lang="de">Theorie der Eigentumsrechte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11065-3">
    <skos:altLabel xml:lang="de">Property-Rights-Theorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11065-3">
    <skos:altLabel xml:lang="de">Ökonomische Theorie der Eigentumsrechte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11065-3">
    <skos:altLabel xml:lang="de">Ökonomik der Eigentumsrechte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11346-2">
    <skos:broader rdf:resource="thsys/70278"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11346-2">
    <skos:broader rdf:resource="descriptor/16440-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11346-2">
    <skos:broader rdf:resource="descriptor/11345-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18479-4">
    <skos:related rdf:resource="descriptor/12917-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10329-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19090-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19344-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18031-6">
    <skos:related rdf:resource="descriptor/12070-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18031-6">
    <skos:related rdf:resource="descriptor/10472-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18031-6">
    <skos:related rdf:resource="descriptor/11981-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18031-6">
    <skos:related rdf:resource="descriptor/11940-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13066-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20176-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18099-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16355-4">
    <skos:broader rdf:resource="thsys/73376"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13667-0">
    <skos:altLabel xml:lang="de">Telephonausrüstung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13667-0">
    <skos:altLabel xml:lang="de">Telephon</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13667-0">
    <skos:altLabel xml:lang="de">Telefonzelle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13667-0">
    <skos:altLabel xml:lang="de">Telefonverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13667-0">
    <skos:altLabel xml:lang="de">Telefonausrüstung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13667-0">
    <skos:altLabel xml:lang="de">Telefonapparat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13667-0">
    <skos:altLabel xml:lang="de">Öffentliches Telefon</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13667-0">
    <skos:altLabel xml:lang="de">Fernsprechausrüstung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13667-0">
    <skos:altLabel xml:lang="de">Anrufbeantworter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19372-3">
    <skos:broader rdf:resource="thsys/71061"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19372-3">
    <skos:broader rdf:resource="thsys/70833"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19372-3">
    <skos:broader rdf:resource="thsys/70324"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19372-3">
    <skos:broader rdf:resource="thsys/70144"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19372-3">
    <skos:broader rdf:resource="thsys/70052"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19372-3">
    <skos:broader rdf:resource="descriptor/11722-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12571-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10030-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11434-5">
    <skos:altLabel xml:lang="en">Coin</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11434-5">
    <skos:altLabel xml:lang="de">Scheidemünze</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11434-5">
    <skos:altLabel xml:lang="de">Numismatik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11434-5">
    <skos:altLabel xml:lang="de">Münzprägestätte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11434-5">
    <skos:altLabel xml:lang="de">Münzkunde</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11434-5">
    <skos:altLabel xml:lang="de">Münze</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19522-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13324-2">
    <skos:narrower rdf:resource="descriptor/18965-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13324-2">
    <skos:narrower rdf:resource="descriptor/13190-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13324-2">
    <skos:narrower rdf:resource="descriptor/18963-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13324-2">
    <skos:narrower rdf:resource="descriptor/18962-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13324-2">
    <skos:narrower rdf:resource="descriptor/13191-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19690-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11968-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13006-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14384-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19495-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19648-2">
    <skos:broader rdf:resource="thsys/73356"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19648-2">
    <skos:broader rdf:resource="thsys/70572"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19648-2">
    <skos:broader rdf:resource="descriptor/19639-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13409-1">
    <skos:narrower rdf:resource="descriptor/13424-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13409-1">
    <skos:narrower rdf:resource="descriptor/13420-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13409-1">
    <skos:narrower rdf:resource="descriptor/13414-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13409-1">
    <skos:narrower rdf:resource="descriptor/11852-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10909-3">
    <skos:altLabel xml:lang="en">Competitive pricing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10909-3">
    <skos:altLabel xml:lang="de">Wettbewerbspreis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10909-3">
    <skos:altLabel xml:lang="de">Preiskonkurrenz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10909-3">
    <skos:altLabel xml:lang="de">Konkurrenzpreis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71038">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15922-0">
    <skos:related rdf:resource="descriptor/15917-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18897-0">
    <skos:narrower rdf:resource="descriptor/19793-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18897-0">
    <skos:narrower rdf:resource="descriptor/19790-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18897-0">
    <skos:narrower rdf:resource="descriptor/18898-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18243-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18343-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18593-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18741-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12399-4">
    <skos:broader rdf:resource="thsys/71043"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12399-4">
    <skos:broader rdf:resource="descriptor/12356-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11986-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15738-0">
    <skos:altLabel xml:lang="de">Internationaler Umweltschutz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15738-0">
    <skos:altLabel xml:lang="de">Internationale Umweltschutzpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15738-0">
    <skos:altLabel xml:lang="de">Globale Umweltpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13615-5">
    <skos:related rdf:resource="descriptor/18622-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13615-5">
    <skos:related rdf:resource="descriptor/18621-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13615-5">
    <skos:related rdf:resource="descriptor/14318-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11154-4">
    <skos:related rdf:resource="descriptor/10469-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11154-4">
    <skos:related rdf:resource="descriptor/11559-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11154-4">
    <skos:related rdf:resource="descriptor/13413-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11154-4">
    <skos:related rdf:resource="descriptor/11158-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11154-4">
    <skos:related rdf:resource="descriptor/11551-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11154-4">
    <skos:related rdf:resource="descriptor/11038-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18200-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10690-6">
    <skos:prefLabel xml:lang="en">Dumping</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10690-6">
    <skos:prefLabel xml:lang="de">Dumping</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16267-1">
    <skos:prefLabel xml:lang="en">Civil war</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16267-1">
    <skos:prefLabel xml:lang="de">Bürgerkrieg</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12891-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17609-1">
    <skos:altLabel xml:lang="de">Königreich Nepal</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17621-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18867-2">
    <skos:prefLabel xml:lang="en">Learning organization</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18867-2">
    <skos:prefLabel xml:lang="de">Lernende Organisation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id368">
    <rdf:value xml:lang="de">Für das Studium der Wirtschaftswissenschaft</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10558-6">
    <skos:altLabel xml:lang="de">Entwicklungsinstitution</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10558-6">
    <skos:altLabel xml:lang="de">Entwicklungshilfeorganisation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14230-6">
    <skos:broader rdf:resource="thsys/70348"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14230-6">
    <skos:broader rdf:resource="descriptor/14236-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18650-6">
    <skos:scopeNote xml:lang="de">Betreiber und Anbieter von Fernsprechnetzen und -diensten</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id376">
    <rdf:value xml:lang="de">für die Struktur einer einzelnen Region</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20888-0">
    <skos:prefLabel xml:lang="en">Green certificates</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20888-0">
    <skos:prefLabel xml:lang="de">Grüne Zertifikate</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18878-4">
    <skos:broader rdf:resource="thsys/70844"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15977-3">
    <skos:prefLabel xml:lang="en">Arbitration</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15977-3">
    <skos:prefLabel xml:lang="de">Schiedsgerichtsbarkeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13525-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11054-1">
    <skos:broader rdf:resource="thsys/70858"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11054-1">
    <skos:broader rdf:resource="thsys/70274"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11054-1">
    <skos:broader rdf:resource="descriptor/10201-5"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id300">
    <rdf:value xml:lang="de">Für Fachliteratur, die sich an bestimmte Fachdisziplinen oder Berufsgruppen wendet,</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17502-5">
    <skos:prefLabel xml:lang="en">Turkmenistan</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17502-5">
    <skos:prefLabel xml:lang="de">Turkmenistan</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12618-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18453-1">
    <skos:broader rdf:resource="thsys/71005"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18453-1">
    <skos:broader rdf:resource="thsys/70238"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18453-1">
    <skos:broader rdf:resource="descriptor/15763-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17778-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13747-2">
    <skos:related rdf:resource="descriptor/18718-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10605-2">
    <skos:broader rdf:resource="thsys/70179"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70871">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11907-2">
    <skos:altLabel xml:lang="de">Regionale Verteilungswirkung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11907-2">
    <skos:altLabel xml:lang="de">Raumwirkung von Maßnahmen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11907-2">
    <skos:altLabel xml:lang="de">Raumwirksamkeit von Maßnahmen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11907-2">
    <skos:altLabel xml:lang="de">Raumwirksame Maßnahme</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11907-2">
    <skos:altLabel xml:lang="de">Räumliche Wirkung von Maßnahmen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11907-2">
    <skos:altLabel xml:lang="de">Räumliche Inzidenz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10968-1">
    <skos:prefLabel xml:lang="en">Lobbying</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10968-1">
    <skos:prefLabel xml:lang="de">Interessenpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19509-2">
    <skos:narrower rdf:resource="descriptor/19325-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15267-6">
    <skos:prefLabel xml:lang="en">Monte Carlo method</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15267-6">
    <skos:prefLabel xml:lang="de">Monte-Carlo-Methode</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13076-2">
    <skos:altLabel xml:lang="de">Elektrotechnische Industrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14218-3">
    <skos:prefLabel xml:lang="en">Petrochemical</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14218-3">
    <skos:prefLabel xml:lang="de">Mineralölprodukt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10010-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18125-4">
    <skos:broader rdf:resource="thsys/70851"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18125-4">
    <skos:broader rdf:resource="thsys/70235"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18125-4">
    <skos:broader rdf:resource="descriptor/11168-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18125-4">
    <skos:broader rdf:resource="descriptor/11219-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71095">
    <skos:broader rdf:resource="thsys/70179"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16680-0">
    <skos:altLabel xml:lang="de">Musiksoziologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16680-0">
    <skos:altLabel xml:lang="de">Literatursoziologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16680-0">
    <skos:altLabel xml:lang="de">Kunstsoziologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16680-0">
    <skos:altLabel xml:lang="de">Kulturtheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16342-6">
    <skos:scopeNote xml:lang="de">Politische Ordnung mit Beteiligung des Parlaments am politischen Entscheidungsprozess.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10099-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14485-1">
    <skos:altLabel xml:lang="de">Kraftfahrzeugelektronik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14485-1">
    <skos:altLabel xml:lang="de">Kraftfahrzeugbeleuchtung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14485-1">
    <skos:altLabel xml:lang="de">Kfz-Elektronik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14485-1">
    <skos:altLabel xml:lang="de">Kfz-Elektrik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14485-1">
    <skos:altLabel xml:lang="de">Autoelektronik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12273-5">
    <skos:related rdf:resource="descriptor/12259-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12273-5">
    <skos:related rdf:resource="descriptor/12385-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12273-5">
    <skos:related rdf:resource="descriptor/12244-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12273-5">
    <skos:related rdf:resource="descriptor/12284-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12273-5">
    <skos:related rdf:resource="descriptor/13727-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12273-5">
    <skos:related rdf:resource="descriptor/12274-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12273-5">
    <skos:related rdf:resource="descriptor/19080-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12273-5">
    <skos:related rdf:resource="descriptor/18722-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11753-3">
    <zbwext:useInsteadNote rdf:nodeID="id73"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14737-0">
    <skos:broader rdf:resource="thsys/70536"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14737-0">
    <skos:broader rdf:resource="descriptor/14734-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13080-4">
    <skos:broader rdf:resource="thsys/70995"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13080-4">
    <skos:broader rdf:resource="thsys/70964"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13080-4">
    <skos:broader rdf:resource="descriptor/12910-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11783-1">
    <skos:broader rdf:resource="thsys/71110"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11783-1">
    <skos:broader rdf:resource="descriptor/11782-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11800-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13683-2">
    <skos:related rdf:resource="descriptor/18661-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13683-2">
    <skos:related rdf:resource="descriptor/19808-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16961-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19355-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16496-0">
    <skos:related rdf:resource="descriptor/16495-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16496-0">
    <skos:related rdf:resource="descriptor/16815-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16496-0">
    <skos:related rdf:resource="descriptor/18772-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16512-0">
    <skos:broader rdf:resource="thsys/73373"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16512-0">
    <skos:broader rdf:resource="descriptor/16509-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11905-6">
    <skos:broader rdf:resource="thsys/71067"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11905-6">
    <skos:broader rdf:resource="thsys/71034"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11905-6">
    <skos:broader rdf:resource="descriptor/11913-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15706-6">
    <skos:narrower rdf:resource="descriptor/15704-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15706-6">
    <skos:narrower rdf:resource="descriptor/15703-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15706-6">
    <skos:narrower rdf:resource="descriptor/10500-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15706-6">
    <skos:narrower rdf:resource="descriptor/13462-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15706-6">
    <skos:narrower rdf:resource="descriptor/18838-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15706-6">
    <skos:narrower rdf:resource="descriptor/11416-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15706-6">
    <skos:narrower rdf:resource="descriptor/12046-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15706-6">
    <skos:narrower rdf:resource="descriptor/11533-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15706-6">
    <skos:narrower rdf:resource="descriptor/19519-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15706-6">
    <skos:narrower rdf:resource="descriptor/13691-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15706-6">
    <skos:narrower rdf:resource="descriptor/13444-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14734-6">
    <zbwext:useInsteadNote rdf:nodeID="id173"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19793-0">
    <skos:altLabel xml:lang="de">Schistosomiasis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19793-0">
    <skos:altLabel xml:lang="de">Malaria</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19793-0">
    <skos:altLabel xml:lang="de">Bilharziose</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17746-5">
    <skos:prefLabel xml:lang="en">Algeria</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17746-5">
    <skos:prefLabel xml:lang="de">Algerien</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21061-5">
    <skos:prefLabel xml:lang="en">Asset</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21061-5">
    <skos:prefLabel xml:lang="de">Vermögensgegenstand</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14537-1">
    <skos:broader rdf:resource="thsys/70423"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70960">
    <skos:narrower rdf:resource="descriptor/19306-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70960">
    <skos:narrower rdf:resource="descriptor/19349-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70960">
    <skos:narrower rdf:resource="descriptor/19303-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70960">
    <skos:narrower rdf:resource="descriptor/13234-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70960">
    <skos:narrower rdf:resource="descriptor/19347-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70960">
    <skos:narrower rdf:resource="descriptor/19301-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70960">
    <skos:narrower rdf:resource="descriptor/18521-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70960">
    <skos:narrower rdf:resource="descriptor/13238-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70960">
    <skos:narrower rdf:resource="descriptor/13237-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70960">
    <skos:narrower rdf:resource="descriptor/19346-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70960">
    <skos:narrower rdf:resource="descriptor/19345-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70960">
    <skos:narrower rdf:resource="descriptor/18551-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70960">
    <skos:narrower rdf:resource="descriptor/19344-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70960">
    <skos:narrower rdf:resource="descriptor/19343-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70960">
    <skos:narrower rdf:resource="descriptor/19300-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70960">
    <skos:narrower rdf:resource="descriptor/19297-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70960">
    <skos:narrower rdf:resource="descriptor/19342-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70960">
    <skos:narrower rdf:resource="descriptor/19338-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70960">
    <skos:narrower rdf:resource="descriptor/19394-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70960">
    <skos:narrower rdf:resource="descriptor/19677-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70960">
    <skos:narrower rdf:resource="descriptor/19183-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70960">
    <skos:narrower rdf:resource="descriptor/19189-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70960">
    <skos:narrower rdf:resource="descriptor/19395-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70960">
    <skos:narrower rdf:resource="descriptor/13219-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70960">
    <skos:narrower rdf:resource="descriptor/19060-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70960">
    <skos:narrower rdf:resource="descriptor/13218-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70960">
    <skos:narrower rdf:resource="descriptor/18520-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70960">
    <skos:narrower rdf:resource="descriptor/13188-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70960">
    <skos:narrower rdf:resource="descriptor/19366-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70960">
    <skos:narrower rdf:resource="descriptor/19296-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70960">
    <skos:narrower rdf:resource="descriptor/19294-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70960">
    <skos:narrower rdf:resource="descriptor/18519-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70960">
    <skos:narrower rdf:resource="descriptor/19293-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70960">
    <skos:narrower rdf:resource="descriptor/19188-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70960">
    <skos:narrower rdf:resource="descriptor/18517-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70960">
    <skos:narrower rdf:resource="descriptor/13207-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70960">
    <skos:narrower rdf:resource="descriptor/19341-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70960">
    <skos:narrower rdf:resource="descriptor/19292-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70960">
    <skos:narrower rdf:resource="descriptor/13225-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70960">
    <skos:narrower rdf:resource="descriptor/19340-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70960">
    <skos:narrower rdf:resource="descriptor/19382-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70960">
    <skos:narrower rdf:resource="descriptor/13205-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70960">
    <skos:narrower rdf:resource="descriptor/18516-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70960">
    <skos:narrower rdf:resource="descriptor/19290-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70960">
    <skos:narrower rdf:resource="descriptor/19674-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70960">
    <skos:narrower rdf:resource="descriptor/13431-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18818-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14011-4">
    <skos:related rdf:resource="descriptor/15664-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71100">
    <skos:prefLabel xml:lang="en">V.07.01  International economics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71100">
    <skos:prefLabel xml:lang="de">V.07.01  Außenwirtschaftstheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12184-4">
    <skos:altLabel xml:lang="de">GmbH und Stille Gesellschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18195-4">
    <skos:prefLabel xml:lang="en">Tropical commodity</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18195-4">
    <skos:prefLabel xml:lang="de">Tropisches Produkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11934-6">
    <skos:related rdf:resource="descriptor/13590-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11934-6">
    <skos:related rdf:resource="descriptor/11834-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10548-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12304-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12788-0">
    <skos:related rdf:resource="descriptor/10964-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12788-0">
    <skos:related rdf:resource="descriptor/21408-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13007-0">
    <skos:related rdf:resource="descriptor/14824-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13007-0">
    <skos:related rdf:resource="descriptor/13053-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13309-5">
    <skos:broader rdf:resource="thsys/70944"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13309-5">
    <skos:broader rdf:resource="thsys/70245"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13309-5">
    <skos:broader rdf:resource="descriptor/13306-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20497-1">
    <skos:related rdf:resource="descriptor/15836-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20497-1">
    <skos:related rdf:resource="descriptor/16566-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20497-1">
    <skos:related rdf:resource="descriptor/21306-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11521-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19202-0">
    <skos:scopeNote xml:lang="de">Zeitliche/dynamische Konsistenz bzw. Inkonsistenz von beabsichtigten Handlungen vor allem wirtschaftspolitischer Entscheidungsträger.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13731-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14353-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13773-1">
    <skos:related rdf:resource="descriptor/13693-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15940-5">
    <skos:prefLabel xml:lang="en">Family statistics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15940-5">
    <skos:prefLabel xml:lang="de">Familienstatistik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15078-2">
    <skos:prefLabel xml:lang="en">Price index for external trade</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15078-2">
    <skos:prefLabel xml:lang="de">Außenhandelspreisindex</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73380">
    <rdfs:label xml:lang="de">Regierung und öffentliche Verwaltung</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17255-3">
    <skos:prefLabel xml:lang="en">France</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17255-3">
    <skos:prefLabel xml:lang="de">Frankreich</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14537-1">
    <skos:narrower rdf:resource="descriptor/18251-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14537-1">
    <skos:narrower rdf:resource="descriptor/14527-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17642-3">
    <skos:related rdf:resource="descriptor/19718-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17642-3">
    <skos:related rdf:resource="descriptor/17992-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17642-3">
    <skos:related rdf:resource="descriptor/17977-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16023-1">
    <skos:related rdf:resource="descriptor/12979-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16023-1">
    <skos:related rdf:resource="descriptor/18379-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16023-1">
    <skos:related rdf:resource="descriptor/14188-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16023-1">
    <skos:related rdf:resource="descriptor/18261-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16023-1">
    <skos:related rdf:resource="descriptor/13329-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16023-1">
    <skos:related rdf:resource="descriptor/15730-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11354-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13002-3">
    <skos:related rdf:resource="descriptor/14728-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13002-3">
    <skos:related rdf:resource="descriptor/19785-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13002-3">
    <skos:related rdf:resource="descriptor/13193-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13002-3">
    <skos:related rdf:resource="descriptor/18259-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13002-3">
    <skos:related rdf:resource="descriptor/19675-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13407-5">
    <skos:altLabel xml:lang="de">Vertretung fremder Staaten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13407-5">
    <skos:altLabel xml:lang="de">Konsulat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13407-5">
    <skos:altLabel xml:lang="de">Diplomatische Vertretung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13407-5">
    <skos:altLabel xml:lang="de">Botschaft im Ausland</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13407-5">
    <skos:altLabel xml:lang="de">Auswärtiger Dienst</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73336">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11931-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70875">
    <skos:prefLabel xml:lang="de">W.21.01  Beratungsdienstleistungen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19186-0">
    <skos:related rdf:resource="descriptor/19062-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19186-0">
    <skos:related rdf:resource="descriptor/13058-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19186-0">
    <skos:related rdf:resource="descriptor/14381-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13385-3">
    <skos:related rdf:resource="descriptor/12613-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13385-3">
    <skos:related rdf:resource="descriptor/16328-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12917-1">
    <skos:narrower rdf:resource="descriptor/18404-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12917-1">
    <skos:narrower rdf:resource="descriptor/18389-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19052-0">
    <skos:related rdf:resource="descriptor/14435-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19052-0">
    <skos:related rdf:resource="descriptor/14434-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12994-4">
    <skos:broader rdf:resource="thsys/70994"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12994-4">
    <skos:broader rdf:resource="descriptor/12996-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13164-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11818-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13142-1">
    <skos:related rdf:resource="descriptor/19391-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13142-1">
    <skos:related rdf:resource="descriptor/13232-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13142-1">
    <skos:related rdf:resource="descriptor/14833-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13142-1">
    <skos:related rdf:resource="descriptor/13137-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13142-1">
    <skos:related rdf:resource="descriptor/13904-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13142-1">
    <skos:related rdf:resource="descriptor/13004-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13142-1">
    <skos:related rdf:resource="descriptor/13128-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12750-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16173-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18207-2">
    <skos:prefLabel xml:lang="en">Company magazine</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18207-2">
    <skos:prefLabel xml:lang="de">Betriebszeitschrift</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16426-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19333-6">
    <skos:related rdf:resource="descriptor/10781-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18020-4">
    <skos:related rdf:resource="descriptor/18004-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19133-0">
    <skos:broader rdf:resource="thsys/73384"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19133-0">
    <skos:broader rdf:resource="thsys/70292"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11590-5">
    <skos:broader rdf:resource="thsys/73367"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11590-5">
    <skos:broader rdf:resource="thsys/71081"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11590-5">
    <skos:broader rdf:resource="thsys/71064"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11590-5">
    <skos:broader rdf:resource="descriptor/11595-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13425-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10348-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14940-3">
    <skos:narrower rdf:resource="descriptor/14942-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14940-3">
    <skos:narrower rdf:resource="descriptor/14941-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18963-6">
    <skos:related rdf:resource="descriptor/18230-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16768-0">
    <skos:altLabel xml:lang="de">Bibliographienkunde</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18943-5">
    <skos:related rdf:resource="descriptor/19475-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18943-5">
    <skos:related rdf:resource="descriptor/10142-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18943-5">
    <skos:related rdf:resource="descriptor/10141-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18943-5">
    <skos:related rdf:resource="descriptor/13345-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18943-5">
    <skos:related rdf:resource="descriptor/16754-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11914-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15662-0">
    <skos:related rdf:resource="descriptor/15759-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15662-0">
    <skos:related rdf:resource="descriptor/15664-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15662-0">
    <skos:related rdf:resource="descriptor/15746-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15662-0">
    <skos:related rdf:resource="descriptor/15633-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19765-5">
    <skos:broader rdf:resource="thsys/73356"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19401-1">
    <skos:broader rdf:resource="thsys/71077"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18944-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11909-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13420-6">
    <skos:narrower rdf:resource="descriptor/16318-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13420-6">
    <skos:narrower rdf:resource="descriptor/15974-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13420-6">
    <skos:narrower rdf:resource="descriptor/16300-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13420-6">
    <skos:narrower rdf:resource="descriptor/19713-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13420-6">
    <skos:narrower rdf:resource="descriptor/19228-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13420-6">
    <skos:narrower rdf:resource="descriptor/19709-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18633-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19676-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14122-2">
    <skos:narrower rdf:resource="descriptor/18219-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14122-2">
    <skos:narrower rdf:resource="descriptor/19021-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14122-2">
    <skos:narrower rdf:resource="descriptor/18220-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14122-2">
    <skos:narrower rdf:resource="descriptor/18218-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13409-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11740-5">
    <skos:related rdf:resource="descriptor/12057-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11740-5">
    <skos:related rdf:resource="descriptor/19761-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11740-5">
    <skos:related rdf:resource="descriptor/11739-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11740-5">
    <skos:related rdf:resource="descriptor/10282-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10028-0">
    <skos:prefLabel xml:lang="en">Value added</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10028-0">
    <skos:prefLabel xml:lang="de">Wertschöpfung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13400-5">
    <skos:prefLabel xml:lang="en">Environmental organization</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13400-5">
    <skos:prefLabel xml:lang="de">Umweltorganisation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14261-2">
    <skos:altLabel xml:lang="de">Weißblech</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14261-2">
    <skos:altLabel xml:lang="de">Stahlblech</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14261-2">
    <skos:altLabel xml:lang="de">Feinblech</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14261-2">
    <skos:altLabel xml:lang="de">Eisenblech</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71063">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17990-1">
    <skos:related rdf:resource="descriptor/17931-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17990-1">
    <skos:related rdf:resource="descriptor/17791-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17990-1">
    <skos:related rdf:resource="descriptor/17989-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17990-1">
    <skos:related rdf:resource="descriptor/19170-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70561">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">G.02.03</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70397">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18815-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10038-4">
    <skos:altLabel xml:lang="de">Sichere Erwartung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10038-4">
    <skos:altLabel xml:lang="de">Erwartungshypothese</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10038-4">
    <skos:altLabel xml:lang="de">Erwartungsbildung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12807-1">
    <skos:prefLabel xml:lang="en">Product life cycle</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12807-1">
    <skos:prefLabel xml:lang="de">Produktlebenszyklus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10269-6">
    <skos:prefLabel xml:lang="en">National income</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10269-6">
    <skos:prefLabel xml:lang="de">Volkseinkommen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19501-4">
    <skos:broader rdf:resource="thsys/70278"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12849-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13355-5">
    <skos:altLabel xml:lang="de">Unterhaltungskünstlerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13355-5">
    <skos:altLabel xml:lang="de">Unterhaltungskünstler</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13355-5">
    <skos:altLabel xml:lang="de">Schauspielerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13355-5">
    <skos:altLabel xml:lang="de">Schauspieler</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13355-5">
    <skos:altLabel xml:lang="de">Sängerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13355-5">
    <skos:altLabel xml:lang="de">Sänger</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13355-5">
    <skos:altLabel xml:lang="de">Pop-Sängerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13355-5">
    <skos:altLabel xml:lang="de">Pop-Sänger</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13355-5">
    <skos:altLabel xml:lang="de">Musikerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13355-5">
    <skos:altLabel xml:lang="de">Musiker</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13355-5">
    <skos:altLabel xml:lang="de">Kunstmalerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13355-5">
    <skos:altLabel xml:lang="de">Kunstmaler</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13355-5">
    <skos:altLabel xml:lang="de">Künstlerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13355-5">
    <skos:altLabel xml:lang="de">Künstler</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13355-5">
    <skos:altLabel xml:lang="de">Kreative Berufe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13355-5">
    <skos:altLabel xml:lang="de">Komponistinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13355-5">
    <skos:altLabel xml:lang="de">Komponisten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13355-5">
    <skos:altLabel xml:lang="de">Graphikerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13355-5">
    <skos:altLabel xml:lang="de">Graphiker</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13355-5">
    <skos:altLabel xml:lang="de">Grafikerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13355-5">
    <skos:altLabel xml:lang="de">Grafiker</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13355-5">
    <skos:altLabel xml:lang="de">Freischaffende Künstlerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13355-5">
    <skos:altLabel xml:lang="de">Freischaffende Künstler</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13355-5">
    <skos:altLabel xml:lang="de">Bühnenkünstlerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13355-5">
    <skos:altLabel xml:lang="de">Bühnenkünstler</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13355-5">
    <skos:altLabel xml:lang="de">Bildende Künstlerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13355-5">
    <skos:altLabel xml:lang="de">Bildende Künstler</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13852-5">
    <skos:related rdf:resource="descriptor/12031-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13852-5">
    <skos:related rdf:resource="descriptor/12905-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13852-5">
    <skos:related rdf:resource="descriptor/13853-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13852-5">
    <skos:related rdf:resource="descriptor/18506-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14793-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18727-4">
    <skos:related rdf:resource="descriptor/11441-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11735-5">
    <skos:broader rdf:resource="thsys/71114"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#hiddenLabel">
    <rdfs:isDefinedBy rdf:resource="http://www.w3.org/2004/02/skos/core"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19661-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71091">
    <skos:narrower rdf:resource="descriptor/10847-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71091">
    <skos:narrower rdf:resource="descriptor/13509-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71091">
    <skos:narrower rdf:resource="descriptor/10675-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71091">
    <skos:narrower rdf:resource="descriptor/11125-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71091">
    <skos:narrower rdf:resource="descriptor/10882-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71091">
    <skos:narrower rdf:resource="descriptor/19357-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71091">
    <skos:narrower rdf:resource="descriptor/10877-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71091">
    <skos:narrower rdf:resource="descriptor/10843-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71091">
    <skos:narrower rdf:resource="descriptor/18126-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71091">
    <skos:narrower rdf:resource="descriptor/10656-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71091">
    <skos:narrower rdf:resource="descriptor/19356-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71091">
    <skos:narrower rdf:resource="descriptor/10838-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71091">
    <skos:narrower rdf:resource="descriptor/19316-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71091">
    <skos:narrower rdf:resource="descriptor/18437-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71091">
    <skos:narrower rdf:resource="descriptor/20817-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71091">
    <skos:narrower rdf:resource="descriptor/10865-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71091">
    <skos:narrower rdf:resource="descriptor/10836-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71091">
    <skos:narrower rdf:resource="descriptor/10604-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71091">
    <skos:narrower rdf:resource="descriptor/18544-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71091">
    <skos:narrower rdf:resource="descriptor/10852-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71091">
    <skos:narrower rdf:resource="descriptor/19355-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71091">
    <skos:narrower rdf:resource="descriptor/15078-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71091">
    <skos:narrower rdf:resource="descriptor/10834-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71091">
    <skos:narrower rdf:resource="descriptor/10686-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71091">
    <skos:narrower rdf:resource="descriptor/18534-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71091">
    <skos:narrower rdf:resource="descriptor/10854-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71091">
    <skos:narrower rdf:resource="descriptor/10833-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71091">
    <skos:narrower rdf:resource="descriptor/18008-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#notation">
    <rdfs:comment xml:lang="en">By convention, skos:notation is used with a typed literal in the
      object position of the triple.</rdfs:comment>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70244">
    <skos:narrower rdf:resource="descriptor/19655-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70244">
    <skos:narrower rdf:resource="descriptor/12587-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70244">
    <skos:narrower rdf:resource="descriptor/11281-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70244">
    <skos:narrower rdf:resource="descriptor/12575-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70244">
    <skos:narrower rdf:resource="descriptor/12155-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70244">
    <skos:narrower rdf:resource="descriptor/19112-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70244">
    <skos:narrower rdf:resource="descriptor/12565-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70244">
    <skos:narrower rdf:resource="descriptor/19007-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70244">
    <skos:narrower rdf:resource="descriptor/12585-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70244">
    <skos:narrower rdf:resource="descriptor/12589-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70244">
    <skos:narrower rdf:resource="descriptor/12563-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70244">
    <skos:narrower rdf:resource="descriptor/19001-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70244">
    <skos:narrower rdf:resource="descriptor/12574-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70244">
    <skos:narrower rdf:resource="descriptor/12562-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70244">
    <skos:narrower rdf:resource="descriptor/12573-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70244">
    <skos:narrower rdf:resource="descriptor/12572-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70244">
    <skos:narrower rdf:resource="descriptor/12571-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70244">
    <skos:narrower rdf:resource="descriptor/19009-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70244">
    <skos:narrower rdf:resource="descriptor/19008-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70244">
    <skos:narrower rdf:resource="descriptor/15839-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70244">
    <skos:narrower rdf:resource="descriptor/19002-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70244">
    <skos:narrower rdf:resource="descriptor/11052-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70244">
    <skos:narrower rdf:resource="descriptor/11203-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70244">
    <skos:narrower rdf:resource="descriptor/10261-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70244">
    <skos:narrower rdf:resource="descriptor/11191-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70244">
    <skos:narrower rdf:resource="descriptor/18070-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70244">
    <skos:narrower rdf:resource="descriptor/12580-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70244">
    <skos:narrower rdf:resource="descriptor/12588-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70244">
    <skos:narrower rdf:resource="descriptor/12314-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70244">
    <skos:narrower rdf:resource="descriptor/15825-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70244">
    <skos:narrower rdf:resource="descriptor/12570-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70244">
    <skos:narrower rdf:resource="descriptor/18143-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70244">
    <skos:narrower rdf:resource="descriptor/21306-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70244">
    <skos:narrower rdf:resource="descriptor/19005-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70244">
    <skos:narrower rdf:resource="descriptor/18207-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70244">
    <skos:narrower rdf:resource="descriptor/12577-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70244">
    <skos:narrower rdf:resource="descriptor/15842-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70244">
    <skos:narrower rdf:resource="descriptor/18885-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70244">
    <skos:narrower rdf:resource="descriptor/11365-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70244">
    <skos:narrower rdf:resource="descriptor/11288-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70244">
    <skos:narrower rdf:resource="descriptor/19656-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70244">
    <skos:narrower rdf:resource="descriptor/11384-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70244">
    <skos:narrower rdf:resource="descriptor/11358-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70244">
    <skos:narrower rdf:resource="descriptor/12569-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70244">
    <skos:narrower rdf:resource="descriptor/15837-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70244">
    <skos:narrower rdf:resource="descriptor/25881-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70244">
    <skos:narrower rdf:resource="descriptor/18140-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70244">
    <skos:narrower rdf:resource="descriptor/11184-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70244">
    <skos:narrower rdf:resource="descriptor/11306-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70244">
    <skos:narrower rdf:resource="descriptor/11303-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70244">
    <skos:narrower rdf:resource="descriptor/19003-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13383-0">
    <skos:prefLabel xml:lang="en">Opinion polls</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13383-0">
    <skos:prefLabel xml:lang="de">Meinungsforschung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15801-5">
    <skos:prefLabel xml:lang="en">Motivation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15801-5">
    <skos:prefLabel xml:lang="de">Motivation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18371-3">
    <skos:prefLabel xml:lang="en">Natural gas policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18371-3">
    <skos:prefLabel xml:lang="de">Erdgaspolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12783-3">
    <skos:prefLabel xml:lang="en">Trade fair</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12783-3">
    <skos:prefLabel xml:lang="de">Messe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18985-3">
    <skos:related rdf:resource="descriptor/18984-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18985-3">
    <skos:related rdf:resource="descriptor/12024-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18985-3">
    <skos:related rdf:resource="descriptor/19054-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11105-3">
    <skos:altLabel xml:lang="en">Transitional society</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11105-3">
    <skos:altLabel xml:lang="de">Übergangsgesellschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15714-0">
    <skos:related rdf:resource="descriptor/15717-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15714-0">
    <skos:related rdf:resource="descriptor/16615-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15714-0">
    <skos:related rdf:resource="descriptor/15712-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15714-0">
    <skos:related rdf:resource="descriptor/16650-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15714-0">
    <skos:related rdf:resource="descriptor/15659-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15714-0">
    <skos:related rdf:resource="descriptor/10984-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15714-0">
    <skos:related rdf:resource="descriptor/15472-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15714-0">
    <skos:related rdf:resource="descriptor/12119-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15714-0">
    <skos:related rdf:resource="descriptor/15909-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18280-6">
    <skos:broader rdf:resource="thsys/73344"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18280-6">
    <skos:broader rdf:resource="thsys/70201"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18664-2">
    <zbwext:useInsteadNote rdf:nodeID="id340"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11411-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18900-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10455-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70239">
    <skos:broader rdf:resource="thsys/70209"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71006">
    <skos:narrower rdf:resource="descriptor/14317-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71006">
    <skos:narrower rdf:resource="descriptor/18394-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71006">
    <skos:narrower rdf:resource="descriptor/19182-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71006">
    <skos:narrower rdf:resource="descriptor/19387-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71006">
    <skos:narrower rdf:resource="descriptor/19181-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71006">
    <skos:narrower rdf:resource="descriptor/18372-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71006">
    <skos:narrower rdf:resource="descriptor/12984-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71006">
    <skos:narrower rdf:resource="descriptor/18393-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71006">
    <skos:narrower rdf:resource="descriptor/19047-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71006">
    <skos:narrower rdf:resource="descriptor/19386-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71006">
    <skos:narrower rdf:resource="descriptor/19385-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71006">
    <skos:narrower rdf:resource="descriptor/12983-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18124-6">
    <skos:prefLabel xml:lang="en">Long-term unemployment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18124-6">
    <skos:prefLabel xml:lang="de">Langzeitarbeitslosigkeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24663-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id138">
    <rdfs:seeAlso rdf:resource="descriptor/18502-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18789-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13001-5">
    <skos:broader rdf:resource="thsys/70992"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13001-5">
    <skos:broader rdf:resource="descriptor/12920-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13001-5">
    <skos:broader rdf:resource="descriptor/12995-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19678-0">
    <skos:related rdf:resource="descriptor/18649-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19678-0">
    <skos:related rdf:resource="descriptor/12010-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19678-0">
    <skos:related rdf:resource="descriptor/19431-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19678-0">
    <skos:related rdf:resource="descriptor/24662-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19678-0">
    <skos:related rdf:resource="descriptor/12975-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12064-0">
    <skos:altLabel xml:lang="de">Privatunternehmen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12064-0">
    <skos:altLabel xml:lang="de">Privatsektor</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12064-0">
    <skos:altLabel xml:lang="de">Privates Unternehmen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12064-0">
    <skos:altLabel xml:lang="de">Privater Sektor</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13421-4">
    <skos:scopeNote xml:lang="de">Mitgliedstaaten eines Bundesstaates.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70538">
    <skos:narrower rdf:resource="descriptor/18234-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70538">
    <skos:narrower rdf:resource="descriptor/14184-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70538">
    <skos:narrower rdf:resource="descriptor/14215-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70538">
    <skos:narrower rdf:resource="descriptor/14603-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70538">
    <skos:narrower rdf:resource="descriptor/18235-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70538">
    <skos:narrower rdf:resource="descriptor/14177-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70538">
    <skos:narrower rdf:resource="descriptor/14176-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70538">
    <skos:narrower rdf:resource="descriptor/14175-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70538">
    <skos:narrower rdf:resource="descriptor/14174-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70538">
    <skos:narrower rdf:resource="descriptor/14173-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70538">
    <skos:narrower rdf:resource="thsys/70651"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70538">
    <skos:narrower rdf:resource="thsys/70649"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70538">
    <skos:narrower rdf:resource="thsys/70650"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70538">
    <skos:narrower rdf:resource="thsys/70648"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14792-6">
    <skos:related rdf:resource="descriptor/14788-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16112-2">
    <skos:narrower rdf:resource="descriptor/10830-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16112-2">
    <skos:narrower rdf:resource="descriptor/10845-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15437-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19618-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13979-2">
    <skos:related rdf:resource="descriptor/13264-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13979-2">
    <skos:related rdf:resource="descriptor/13510-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13979-2">
    <skos:related rdf:resource="descriptor/11559-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12949-2">
    <skos:related rdf:resource="descriptor/14074-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12949-2">
    <skos:related rdf:resource="descriptor/14073-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12949-2">
    <skos:related rdf:resource="descriptor/13434-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12949-2">
    <skos:related rdf:resource="descriptor/13452-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#memberList">
    <rdf:type rdf:resource="http://www.w3.org/1999/02/22-rdf-syntax-ns#Property"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#memberList">
    <rdf:type rdf:resource="http://www.w3.org/2002/07/owl#FunctionalProperty"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#memberList">
    <rdf:type rdf:resource="http://www.w3.org/2002/07/owl#ObjectProperty"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10831-0">
    <skos:related rdf:resource="descriptor/11709-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10831-0">
    <skos:related rdf:resource="descriptor/10830-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10831-0">
    <skos:related rdf:resource="descriptor/10817-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10831-0">
    <skos:related rdf:resource="descriptor/10829-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10831-0">
    <skos:related rdf:resource="descriptor/12446-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10831-0">
    <skos:related rdf:resource="descriptor/12872-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10831-0">
    <skos:related rdf:resource="descriptor/10826-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10831-0">
    <skos:related rdf:resource="descriptor/10827-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10256-1">
    <skos:broader rdf:resource="thsys/70233"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10256-1">
    <skos:broader rdf:resource="descriptor/10249-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19832-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id373">
    <rdfs:seeAlso rdf:resource="descriptor/10414-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20977-1">
    <skos:scopeNote xml:lang="de">Bildung von organisatorischen Einheiten nach dem Prinzip sich ergänzender Grundbausteine, die nach Bedarf unterschiedlich kombiniert werden können.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71101">
    <rdfs:label xml:lang="en">Centrally planned economy</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71101">
    <rdfs:label xml:lang="de">Planwirtschaft</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12770-5">
    <skos:altLabel xml:lang="de">Strategische Marketingplanung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12770-5">
    <skos:altLabel xml:lang="de">Marketingstrategie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12770-5">
    <skos:altLabel xml:lang="de">Marketingplanung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12770-5">
    <skos:altLabel xml:lang="de">Marketing-Mix</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12770-5">
    <skos:altLabel xml:lang="de">Marketingkonzeption</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12770-5">
    <skos:altLabel xml:lang="de">Kommunikationsstrategie (Marketing)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12770-5">
    <skos:altLabel xml:lang="de">Kommunikationspolitik (Marketing)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19718-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11681-2">
    <skos:broader rdf:resource="thsys/71074"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11681-2">
    <skos:broader rdf:resource="descriptor/11661-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11637-5">
    <skos:broader rdf:resource="thsys/71077"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11637-5">
    <skos:broader rdf:resource="descriptor/11526-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71069">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14070-2">
    <skos:prefLabel xml:lang="en">Oil Seed Plant</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14070-2">
    <skos:prefLabel xml:lang="de">Ölpflanze</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14631-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12251-1">
    <skos:editorialNote xml:lang="de">Möglichst spezifischer indexieren.</skos:editorialNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10558-6">
    <skos:related rdf:resource="descriptor/11886-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10558-6">
    <skos:related rdf:resource="descriptor/10589-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10558-6">
    <skos:related rdf:resource="descriptor/13395-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10558-6">
    <skos:related rdf:resource="descriptor/10618-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10558-6">
    <skos:related rdf:resource="descriptor/16439-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10558-6">
    <skos:related rdf:resource="descriptor/10560-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10558-6">
    <skos:related rdf:resource="descriptor/10555-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10558-6">
    <skos:related rdf:resource="descriptor/10552-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10558-6">
    <skos:related rdf:resource="descriptor/10550-1"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id21">
    <rdfs:seeAlso rdf:resource="descriptor/12501-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14674-5">
    <skos:prefLabel xml:lang="en">Epoxyde</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14674-5">
    <skos:prefLabel xml:lang="de">Epoxid</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18266-0">
    <skos:prefLabel xml:lang="en">Insecticide</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18266-0">
    <skos:prefLabel xml:lang="de">Insektizid</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18438-4">
    <skos:related rdf:resource="descriptor/18382-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70259">
    <skos:prefLabel xml:lang="en">V.05.05  Credit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70259">
    <skos:prefLabel xml:lang="de">V.05.05  Kredit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18521-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10085-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11968-3">
    <skos:broader rdf:resource="thsys/71064"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11968-3">
    <skos:broader rdf:resource="descriptor/11970-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15010-1">
    <skos:broader rdf:resource="thsys/70238"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15010-1">
    <skos:broader rdf:resource="thsys/70161"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15010-1">
    <skos:broader rdf:resource="descriptor/15743-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15010-1">
    <skos:broader rdf:resource="descriptor/15009-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17937-5">
    <skos:prefLabel xml:lang="en">Fiji</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17937-5">
    <skos:prefLabel xml:lang="de">Fidschi</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17496-2">
    <skos:prefLabel xml:lang="en">Kazakhstan</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17496-2">
    <skos:prefLabel xml:lang="de">Kasachstan</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19707-5">
    <skos:related rdf:resource="descriptor/15676-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19707-5">
    <skos:related rdf:resource="descriptor/19775-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21072-0">
    <skos:prefLabel xml:lang="en">Insurance coverage</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21072-0">
    <skos:prefLabel xml:lang="de">Versicherungsschutz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10619-5">
    <skos:related rdf:resource="descriptor/10626-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10619-5">
    <skos:related rdf:resource="descriptor/10625-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10619-5">
    <skos:related rdf:resource="descriptor/10624-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10619-5">
    <skos:related rdf:resource="descriptor/18064-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10619-5">
    <skos:related rdf:resource="descriptor/10586-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10619-5">
    <skos:related rdf:resource="descriptor/10618-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10619-5">
    <skos:related rdf:resource="descriptor/10617-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10619-5">
    <skos:related rdf:resource="descriptor/10682-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18360-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16783-4">
    <skos:altLabel xml:lang="de">Bibliotheksunterricht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16783-4">
    <skos:altLabel xml:lang="de">Benutzerverhalten in Bibliotheken</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16783-4">
    <skos:altLabel xml:lang="de">Benutzerschulung in Bibliotheken</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13701-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12230-2">
    <skos:prefLabel xml:lang="en">Security analysis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12230-2">
    <skos:prefLabel xml:lang="de">Wertpapieranalyse</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19860-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14302-0">
    <skos:broader rdf:resource="thsys/70134"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14302-0">
    <skos:broader rdf:resource="descriptor/14266-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10126-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16374-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19626-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10462-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10470-6">
    <skos:related rdf:resource="descriptor/10473-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10470-6">
    <skos:related rdf:resource="descriptor/10472-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10470-6">
    <skos:related rdf:resource="descriptor/10471-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10470-6">
    <skos:related rdf:resource="descriptor/10466-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10470-6">
    <skos:related rdf:resource="descriptor/13962-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10470-6">
    <skos:related rdf:resource="descriptor/16695-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10470-6">
    <skos:related rdf:resource="descriptor/10465-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10470-6">
    <skos:related rdf:resource="descriptor/16696-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10470-6">
    <skos:related rdf:resource="descriptor/15664-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10470-6">
    <skos:related rdf:resource="descriptor/19795-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10470-6">
    <skos:related rdf:resource="descriptor/10437-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15392-3">
    <skos:broader rdf:resource="thsys/72183"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15392-3">
    <skos:broader rdf:resource="descriptor/15396-2"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id125">
    <rdf:value xml:lang="de">Für menschliche Grundrechte im internationalen Recht</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70415">
    <rdfs:label xml:lang="de">Wasserfahrzeuge</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12391-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11658-4">
    <skos:prefLabel xml:lang="en">Real estate transfer tax</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11658-4">
    <skos:prefLabel xml:lang="de">Grunderwerbsteuer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17632-6">
    <skos:broader rdf:resource="thsys/70253"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17632-6">
    <skos:broader rdf:resource="descriptor/17608-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17632-6">
    <skos:broader rdf:resource="descriptor/17978-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12794-5">
    <skos:prefLabel xml:lang="en">Pricing behaviour of firms</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12794-5">
    <skos:prefLabel xml:lang="de">Betriebliche Preispolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11237-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10106-6">
    <skos:altLabel xml:lang="en">Social choice function</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10106-6">
    <skos:altLabel xml:lang="de">Soziale Präferenzordnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10106-6">
    <skos:altLabel xml:lang="de">Soziale Präferenzen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10106-6">
    <skos:altLabel xml:lang="de">Kollektive Wohlfahrtsfunktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10106-6">
    <skos:altLabel xml:lang="de">Kollektive Präferenzordnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10106-6">
    <skos:altLabel xml:lang="de">Kollektive Nutzenfunktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10106-6">
    <skos:altLabel xml:lang="de">Gesellschaftliche Wohlfahrtsfunktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10106-6">
    <skos:altLabel xml:lang="de">Gesellschaftliche Präferenzordnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15635-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15498-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16717-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13824-3">
    <skos:altLabel xml:lang="de">Versicherungstarif</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13824-3">
    <skos:altLabel xml:lang="de">Versicherungsbeitrag</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13824-3">
    <skos:altLabel xml:lang="de">Prämie für Versicherungen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13824-3">
    <skos:altLabel xml:lang="de">Beitrag für Versicherungen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12164-3">
    <skos:broader rdf:resource="thsys/70383"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70856">
    <skos:narrower rdf:resource="descriptor/18837-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70856">
    <skos:narrower rdf:resource="descriptor/18097-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70856">
    <skos:narrower rdf:resource="descriptor/18836-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70856">
    <skos:narrower rdf:resource="descriptor/12047-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70856">
    <skos:narrower rdf:resource="descriptor/18835-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70856">
    <skos:narrower rdf:resource="descriptor/18834-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70856">
    <skos:narrower rdf:resource="descriptor/18833-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70856">
    <skos:narrower rdf:resource="descriptor/18830-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70856">
    <skos:narrower rdf:resource="descriptor/18831-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70856">
    <skos:narrower rdf:resource="thsys/70855"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70856">
    <skos:narrower rdf:resource="thsys/70853"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13571-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11433-0">
    <skos:related rdf:resource="descriptor/11459-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11433-0">
    <skos:related rdf:resource="descriptor/18698-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11433-0">
    <skos:related rdf:resource="descriptor/13776-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11433-0">
    <skos:related rdf:resource="descriptor/12251-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15716-3">
    <skos:prefLabel xml:lang="en">Social mobility</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15716-3">
    <skos:prefLabel xml:lang="de">Soziale Mobilität</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11582-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16191-1">
    <skos:broader rdf:resource="thsys/73334"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16191-1">
    <skos:broader rdf:resource="descriptor/16201-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18350-4">
    <skos:related rdf:resource="descriptor/12975-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18350-4">
    <skos:related rdf:resource="descriptor/14176-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18350-4">
    <skos:related rdf:resource="descriptor/19487-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18350-4">
    <skos:related rdf:resource="descriptor/11810-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18350-4">
    <skos:related rdf:resource="descriptor/18996-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18350-4">
    <skos:related rdf:resource="descriptor/18989-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18350-4">
    <skos:related rdf:resource="descriptor/18348-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18350-4">
    <skos:related rdf:resource="descriptor/14175-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17578-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17586-1">
    <skos:broader rdf:resource="thsys/70327"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17586-1">
    <skos:broader rdf:resource="descriptor/17589-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17586-1">
    <skos:broader rdf:resource="descriptor/17979-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70876">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13312-2">
    <skos:related rdf:resource="descriptor/13849-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13312-2">
    <skos:related rdf:resource="descriptor/13829-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13312-2">
    <skos:related rdf:resource="descriptor/18180-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13312-2">
    <skos:related rdf:resource="descriptor/18907-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13312-2">
    <skos:related rdf:resource="descriptor/14007-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13312-2">
    <skos:related rdf:resource="descriptor/18864-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18140-1">
    <skos:prefLabel xml:lang="en">Job search</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18140-1">
    <skos:prefLabel xml:lang="de">Arbeitsuche</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12046-2">
    <skos:scopeNote xml:lang="de">Geschichte einzelner Unternehmen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13671-2">
    <skos:prefLabel xml:lang="en">Office automation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13671-2">
    <skos:prefLabel xml:lang="de">Bürokommunikation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19470-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10178-2">
    <skos:broader rdf:resource="thsys/71112"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10178-2">
    <skos:broader rdf:resource="thsys/70493"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10178-2">
    <skos:broader rdf:resource="descriptor/10301-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12665-2">
    <skos:broader rdf:resource="thsys/73384"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12665-2">
    <skos:broader rdf:resource="thsys/70368"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12665-2">
    <skos:broader rdf:resource="descriptor/12036-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11939-3">
    <skos:prefLabel xml:lang="en">Gemeinschaftsaufgabe Regionale Wirtschaftsstruktur</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11939-3">
    <skos:prefLabel xml:lang="de">Gemeinschaftsaufgabe Regionale Wirtschaftsstruktur</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12592-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14803-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#broader">
    <rdfs:subPropertyOf rdf:resource="http://www.w3.org/2004/02/skos/core#broaderTransitive"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19340-2">
    <skos:related rdf:resource="descriptor/19347-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19340-2">
    <skos:related rdf:resource="descriptor/12955-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19340-2">
    <skos:related rdf:resource="descriptor/13155-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19340-2">
    <skos:related rdf:resource="descriptor/13940-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19340-2">
    <skos:related rdf:resource="descriptor/14122-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12284-0">
    <skos:prefLabel xml:lang="en">Capital increase</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12284-0">
    <skos:prefLabel xml:lang="de">Kapitalerhöhung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18345-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14033-1">
    <skos:prefLabel xml:lang="en">DIY-product</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14033-1">
    <skos:prefLabel xml:lang="de">Heimwerkerartikel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13006-2">
    <skos:prefLabel xml:lang="en">Plastic products industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13006-2">
    <skos:prefLabel xml:lang="de">Kunststoffverarbeitung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18979-5">
    <skos:broader rdf:resource="thsys/71020"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18979-5">
    <skos:broader rdf:resource="thsys/70867"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18979-5">
    <skos:broader rdf:resource="descriptor/18810-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20241-1">
    <skos:prefLabel xml:lang="en">Virtual enterprise</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20241-1">
    <skos:prefLabel xml:lang="de">Virtuelles Unternehmen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19751-2">
    <skos:prefLabel xml:lang="en">Self-organization</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19751-2">
    <skos:prefLabel xml:lang="de">Selbstorganisation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13731-3">
    <skos:related rdf:resource="descriptor/16080-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13731-3">
    <skos:related rdf:resource="descriptor/13742-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13731-3">
    <skos:related rdf:resource="descriptor/19321-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13731-3">
    <skos:related rdf:resource="descriptor/16050-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13731-3">
    <skos:related rdf:resource="descriptor/13723-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13731-3">
    <skos:related rdf:resource="descriptor/13693-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14735-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15775-1">
    <skos:altLabel xml:lang="en">Environmental impact assessment</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15775-1">
    <skos:altLabel xml:lang="de">UVP (Umweltverträglichkeitsprüfung)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15775-1">
    <skos:altLabel xml:lang="de">Umweltverträglichkeitsprüfung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15767-0">
    <skos:scopeNote xml:lang="de">Abwägung von (sozialen) Kosten und Nutzen der Umweltnutzung. Zielkonflikte zwischen Wirtschaftswachstum und Umweltbelastung. Analyse der Allokationswirkungen verschiedener umweltpolitischer Instrumente.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18382-5">
    <skos:narrower rdf:resource="descriptor/19181-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16288-0">
    <skos:related rdf:resource="descriptor/16440-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16288-0">
    <skos:related rdf:resource="descriptor/19684-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16288-0">
    <skos:related rdf:resource="descriptor/15719-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16288-0">
    <skos:related rdf:resource="descriptor/16463-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16288-0">
    <skos:related rdf:resource="descriptor/18064-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16288-0">
    <skos:related rdf:resource="descriptor/16286-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16288-0">
    <skos:related rdf:resource="descriptor/16280-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16288-0">
    <skos:related rdf:resource="descriptor/16395-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16288-0">
    <skos:related rdf:resource="descriptor/19715-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16288-0">
    <skos:related rdf:resource="descriptor/16435-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16288-0">
    <skos:related rdf:resource="descriptor/19851-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16288-0">
    <skos:related rdf:resource="descriptor/15878-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16288-0">
    <skos:related rdf:resource="descriptor/16497-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16288-0">
    <skos:related rdf:resource="descriptor/16204-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10612-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18393-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70834">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13207-6">
    <skos:altLabel xml:lang="de">Getränkegroßhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13207-6">
    <skos:altLabel xml:lang="de">Getränkeeinzelhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13207-6">
    <skos:altLabel xml:lang="de">Getränkeabholmarkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71114">
    <rdfs:label xml:lang="de">Einkommens- und Vermögensverteilung</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18461-2">
    <skos:prefLabel xml:lang="en">Industrial research</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18461-2">
    <skos:prefLabel xml:lang="de">Industrielle Forschung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10523-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19195-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16950-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13207-6">
    <skos:broader rdf:resource="thsys/70960"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13207-6">
    <skos:broader rdf:resource="descriptor/13219-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15637-6">
    <skos:broader rdf:resource="thsys/73388"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15637-6">
    <skos:broader rdf:resource="descriptor/15671-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11197-0">
    <skos:related rdf:resource="descriptor/18993-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11197-0">
    <skos:related rdf:resource="descriptor/12910-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18652-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14646-3">
    <skos:narrower rdf:resource="descriptor/14045-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14096-5">
    <skos:prefLabel xml:lang="en">Rye</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14096-5">
    <skos:prefLabel xml:lang="de">Roggen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13766-5">
    <skos:prefLabel xml:lang="en">Swap</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13766-5">
    <skos:prefLabel xml:lang="de">Swap</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12298-3">
    <skos:broader rdf:resource="thsys/70036"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12298-3">
    <skos:broader rdf:resource="descriptor/12296-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19169-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15519-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70995">
    <rdfs:label xml:lang="de">Feinmechanische und Optische Industrie</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15966-1">
    <skos:related rdf:resource="descriptor/15976-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15966-1">
    <skos:related rdf:resource="descriptor/15971-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15966-1">
    <skos:related rdf:resource="descriptor/15970-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15966-1">
    <skos:related rdf:resource="descriptor/15969-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15966-1">
    <skos:related rdf:resource="descriptor/19607-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15966-1">
    <skos:related rdf:resource="descriptor/15968-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15966-1">
    <skos:related rdf:resource="descriptor/13995-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15966-1">
    <skos:related rdf:resource="descriptor/13389-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15966-1">
    <skos:related rdf:resource="descriptor/19841-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15966-1">
    <skos:related rdf:resource="descriptor/16338-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15966-1">
    <skos:related rdf:resource="descriptor/13410-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13873-4">
    <skos:altLabel xml:lang="de">Fischerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13873-4">
    <skos:altLabel xml:lang="de">Fischer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71023">
    <skos:broader rdf:resource="thsys/71028"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15927-4">
    <skos:prefLabel xml:lang="en">Men</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15927-4">
    <skos:prefLabel xml:lang="de">Männer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13983-4">
    <skos:broader rdf:resource="thsys/71055"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13983-4">
    <skos:broader rdf:resource="thsys/70936"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13983-4">
    <skos:broader rdf:resource="descriptor/13979-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12022-2">
    <skos:altLabel xml:lang="en">Ground rent</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12022-2">
    <skos:altLabel xml:lang="de">Grundrente</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18538-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16808-0">
    <skos:narrower rdf:resource="descriptor/16806-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16808-0">
    <skos:narrower rdf:resource="descriptor/16805-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16335-3">
    <skos:scopeNote xml:lang="de">Kontrolle der öffentlichen Verwaltung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12312-0">
    <skos:altLabel xml:lang="de">Kurzfristige Erfolgsrechnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12312-0">
    <skos:altLabel xml:lang="de">Betriebsergebnisrechnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12057-4">
    <skos:broader rdf:resource="thsys/71034"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12057-4">
    <skos:broader rdf:resource="thsys/70049"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18622-4">
    <skos:prefLabel xml:lang="en">Container handling</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18622-4">
    <skos:prefLabel xml:lang="de">Containerumschlag</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19730-3">
    <skos:prefLabel xml:lang="en">Constructivism</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19730-3">
    <skos:prefLabel xml:lang="de">Konstruktivismus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10195-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12577-6">
    <skos:related rdf:resource="descriptor/12108-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12577-6">
    <skos:related rdf:resource="descriptor/12585-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12577-6">
    <skos:related rdf:resource="descriptor/15839-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12577-6">
    <skos:related rdf:resource="descriptor/11052-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12577-6">
    <skos:related rdf:resource="descriptor/15835-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12577-6">
    <skos:related rdf:resource="descriptor/11192-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12577-6">
    <skos:related rdf:resource="descriptor/15837-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12577-6">
    <skos:related rdf:resource="descriptor/12617-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12577-6">
    <skos:related rdf:resource="descriptor/11314-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12577-6">
    <skos:related rdf:resource="descriptor/11305-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70992">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.06.01.06.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18805-3">
    <skos:narrower rdf:resource="descriptor/18372-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19306-2">
    <skos:broader rdf:resource="thsys/71029"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19306-2">
    <skos:broader rdf:resource="thsys/70960"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19306-2">
    <skos:broader rdf:resource="descriptor/13431-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11233-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70459">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15966-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#narrower">
    <rdfs:isDefinedBy rdf:resource="http://www.w3.org/2004/02/skos/core"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14703-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15088-6">
    <skos:broader rdf:resource="thsys/72184"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15088-6">
    <skos:broader rdf:resource="thsys/70814"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15088-6">
    <skos:broader rdf:resource="thsys/70504"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15088-6">
    <skos:broader rdf:resource="descriptor/15073-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15088-6">
    <skos:broader rdf:resource="descriptor/15093-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12981-6">
    <skos:prefLabel xml:lang="en">Lignite mining</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12981-6">
    <skos:prefLabel xml:lang="de">Braunkohlenbergbau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16051-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13072-3">
    <skos:broader rdf:resource="thsys/73297"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13072-3">
    <skos:broader rdf:resource="thsys/70799"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13072-3">
    <skos:broader rdf:resource="descriptor/18424-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19707-5">
    <skos:scopeNote xml:lang="de">Alle politischen Maßnahmen, die an der Sprache von Bevölkerungsgruppen ansetzen</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19453-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19801-6">
    <skos:related rdf:resource="descriptor/19805-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19801-6">
    <skos:related rdf:resource="descriptor/15416-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19801-6">
    <skos:related rdf:resource="descriptor/19651-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19801-6">
    <skos:related rdf:resource="descriptor/14765-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15947-5">
    <skos:broader rdf:resource="thsys/73347"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15947-5">
    <skos:broader rdf:resource="thsys/70573"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15947-5">
    <skos:broader rdf:resource="thsys/70182"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70328">
    <skos:prefLabel xml:lang="de">P.14  Pappe, Papier und Papierprodukte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70927">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.12.05.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13513-6">
    <skos:scopeNote xml:lang="de">Zu den Verkehrsmitteln zählen auch Nachrichtenverkehrsmittel, Zugtiere, Pipelines, usw.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12801-6">
    <skos:scopeNote xml:lang="de">Für generische Arzneimittel BENUTZE: Generika.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18676-2">
    <skos:scopeNote xml:lang="de">Vorwiegend für Wertpapierbörsen, aber auch für Devisen- und Warenbörsen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12747-0">
    <skos:broader rdf:resource="thsys/71035"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12747-0">
    <skos:broader rdf:resource="thsys/70029"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12747-0">
    <skos:broader rdf:resource="descriptor/12758-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18374-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13688-6">
    <skos:related rdf:resource="descriptor/19337-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13688-6">
    <skos:related rdf:resource="descriptor/18730-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13688-6">
    <skos:related rdf:resource="descriptor/13692-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13688-6">
    <skos:related rdf:resource="descriptor/13746-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71101">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://zbw.eu/namespaces/skos-extensions/indexedItem">
    <skos:definition xml:lang="en">The concept is used as an index term or subject 
    for the resource.</skos:definition>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19314-3">
    <skos:broader rdf:resource="thsys/71098"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19314-3">
    <skos:broader rdf:resource="descriptor/10686-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18610-4">
    <skos:prefLabel xml:lang="en">Public transport policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18610-4">
    <skos:prefLabel xml:lang="de">Nahverkehrspolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11239-3">
    <skos:broader rdf:resource="thsys/71061"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11239-3">
    <skos:broader rdf:resource="thsys/70038"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11239-3">
    <skos:broader rdf:resource="descriptor/10987-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12165-1">
    <skos:related rdf:resource="descriptor/12172-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12165-1">
    <skos:related rdf:resource="descriptor/18825-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12165-1">
    <skos:related rdf:resource="descriptor/20921-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12165-1">
    <skos:related rdf:resource="descriptor/12171-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12165-1">
    <skos:related rdf:resource="descriptor/18017-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12165-1">
    <skos:related rdf:resource="descriptor/12039-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70859">
    <skos:narrower rdf:resource="descriptor/18825-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70859">
    <skos:narrower rdf:resource="descriptor/11580-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70859">
    <skos:narrower rdf:resource="descriptor/11589-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70859">
    <skos:narrower rdf:resource="descriptor/18823-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70859">
    <skos:narrower rdf:resource="descriptor/11579-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70859">
    <skos:narrower rdf:resource="descriptor/18835-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70859">
    <skos:narrower rdf:resource="descriptor/18834-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70859">
    <skos:narrower rdf:resource="descriptor/18833-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70859">
    <skos:narrower rdf:resource="descriptor/11576-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13273-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19420-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19081-0">
    <skos:broader rdf:resource="thsys/71046"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19081-0">
    <skos:broader rdf:resource="descriptor/19080-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19417-0">
    <skos:prefLabel xml:lang="en">Securities transaction tax</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19417-0">
    <skos:prefLabel xml:lang="de">Börsenumsatzsteuer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14479-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19217-1">
    <skos:altLabel xml:lang="de">Intangible Güter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19217-1">
    <skos:altLabel xml:lang="de">Immaterielles Gut</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19217-1">
    <skos:altLabel xml:lang="de">Immaterielle Güter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19189-1">
    <skos:broader rdf:resource="thsys/71029"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19189-1">
    <skos:broader rdf:resource="thsys/70960"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19189-1">
    <skos:broader rdf:resource="descriptor/19188-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10629-2">
    <skos:related rdf:resource="descriptor/19356-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10629-2">
    <skos:related rdf:resource="descriptor/10682-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10629-2">
    <skos:related rdf:resource="descriptor/10605-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10629-2">
    <skos:related rdf:resource="descriptor/10686-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15064-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10183-2">
    <skos:prefLabel xml:lang="en">Optimal consumption</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10183-2">
    <skos:prefLabel xml:lang="de">Optimaler Konsum</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12188-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18941-2">
    <skos:altLabel xml:lang="de">Online-Informationsbank</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18941-2">
    <skos:altLabel xml:lang="de">Informations-Datenbank</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18941-2">
    <skos:altLabel xml:lang="de">Informationsbank</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15628-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19568-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14953-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18411-3">
    <skos:altLabel xml:lang="de">Kautschukmaschinenbau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18411-3">
    <skos:altLabel xml:lang="de">Gummimaschinenbau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11340-0">
    <skos:related rdf:resource="descriptor/11348-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10912-0">
    <skos:altLabel xml:lang="de">Vertikale Integration</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18557-3">
    <skos:broader rdf:resource="thsys/70239"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18557-3">
    <skos:broader rdf:resource="descriptor/19564-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14461-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id255">
    <rdf:value xml:lang="de">Für die Postdienste allgemein</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73372">
    <skos:narrower rdf:resource="descriptor/10987-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73372">
    <skos:narrower rdf:resource="descriptor/10439-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73372">
    <skos:narrower rdf:resource="descriptor/11779-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73372">
    <skos:narrower rdf:resource="descriptor/18976-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73372">
    <skos:narrower rdf:resource="descriptor/11704-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73372">
    <skos:narrower rdf:resource="descriptor/10978-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73372">
    <skos:narrower rdf:resource="descriptor/10976-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73372">
    <skos:narrower rdf:resource="descriptor/10387-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73372">
    <skos:narrower rdf:resource="descriptor/10438-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73372">
    <skos:narrower rdf:resource="descriptor/10970-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73372">
    <skos:narrower rdf:resource="descriptor/19251-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73372">
    <skos:narrower rdf:resource="descriptor/10382-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73372">
    <skos:narrower rdf:resource="descriptor/11728-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73372">
    <skos:narrower rdf:resource="descriptor/11235-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16440-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14397-5">
    <skos:broader rdf:resource="thsys/70549"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14397-5">
    <skos:broader rdf:resource="descriptor/12693-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70467">
    <skos:prefLabel xml:lang="de">P.01.02  Tierische Produkte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18905-6">
    <skos:related rdf:resource="descriptor/16017-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18905-6">
    <skos:related rdf:resource="descriptor/16545-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18905-6">
    <skos:related rdf:resource="descriptor/11711-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18905-6">
    <skos:related rdf:resource="descriptor/16401-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19642-0">
    <skos:altLabel xml:lang="en">Non-white population</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19642-0">
    <skos:altLabel xml:lang="en">Coloured people</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19642-0">
    <skos:altLabel xml:lang="de">Farbige</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73393">
    <skos:narrower rdf:resource="descriptor/16732-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73393">
    <skos:narrower rdf:resource="descriptor/15686-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73393">
    <skos:narrower rdf:resource="descriptor/19010-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73393">
    <skos:narrower rdf:resource="descriptor/16727-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73393">
    <skos:narrower rdf:resource="descriptor/15685-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73393">
    <skos:narrower rdf:resource="descriptor/19253-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73393">
    <skos:narrower rdf:resource="descriptor/16730-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73393">
    <skos:narrower rdf:resource="descriptor/16729-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73393">
    <skos:narrower rdf:resource="descriptor/15429-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73393">
    <skos:narrower rdf:resource="descriptor/19746-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73393">
    <skos:narrower rdf:resource="descriptor/19749-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73393">
    <skos:narrower rdf:resource="descriptor/16720-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73393">
    <skos:narrower rdf:resource="descriptor/20022-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73393">
    <skos:narrower rdf:resource="descriptor/19989-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73393">
    <skos:narrower rdf:resource="descriptor/19770-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73393">
    <skos:narrower rdf:resource="descriptor/19219-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73393">
    <skos:narrower rdf:resource="descriptor/16726-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73393">
    <skos:narrower rdf:resource="descriptor/16716-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73393">
    <skos:narrower rdf:resource="descriptor/19744-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73393">
    <skos:narrower rdf:resource="descriptor/19743-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73393">
    <skos:narrower rdf:resource="descriptor/10436-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73393">
    <skos:narrower rdf:resource="descriptor/19741-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73393">
    <skos:narrower rdf:resource="descriptor/19204-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73393">
    <skos:narrower rdf:resource="descriptor/19070-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73393">
    <skos:narrower rdf:resource="descriptor/16745-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73393">
    <skos:narrower rdf:resource="descriptor/19203-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73393">
    <skos:narrower rdf:resource="descriptor/19745-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73393">
    <skos:narrower rdf:resource="descriptor/19742-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19358-4">
    <skos:scopeNote xml:lang="de">Problematik des Schuldendienstes (Zinsen, Tilgung) für Auslandsschulden und Lösungsalternativen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17668-6">
    <skos:altLabel xml:lang="de">Republik Mauritius</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18996-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10718-3">
    <skos:broader rdf:resource="thsys/71095"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10718-3">
    <skos:broader rdf:resource="descriptor/10722-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15961-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19434-0">
    <skos:broader rdf:resource="thsys/73391"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19434-0">
    <skos:broader rdf:resource="thsys/71071"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19434-0">
    <skos:broader rdf:resource="descriptor/15981-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19434-0">
    <skos:broader rdf:resource="descriptor/16051-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12955-0">
    <skos:narrower rdf:resource="descriptor/12956-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12955-0">
    <skos:narrower rdf:resource="descriptor/12972-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12955-0">
    <skos:narrower rdf:resource="descriptor/12953-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12955-0">
    <skos:narrower rdf:resource="descriptor/12952-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12955-0">
    <skos:narrower rdf:resource="descriptor/12950-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12955-0">
    <skos:narrower rdf:resource="descriptor/12971-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12955-0">
    <skos:narrower rdf:resource="descriptor/12946-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12955-0">
    <skos:narrower rdf:resource="descriptor/12941-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12955-0">
    <skos:narrower rdf:resource="descriptor/12969-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10590-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18747-5">
    <skos:altLabel xml:lang="de">Immobilienökonomie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18747-5">
    <skos:altLabel xml:lang="de">Immobiliengesellschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18747-5">
    <skos:altLabel xml:lang="de">Grundstückswirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18747-5">
    <skos:altLabel xml:lang="de">Grundstücks- und Wohnungswirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12304-6">
    <skos:prefLabel xml:lang="en">Takeover</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12304-6">
    <skos:prefLabel xml:lang="de">Übernahme</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19483-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11309-1">
    <skos:broader rdf:resource="thsys/70052"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18375-2">
    <skos:broader rdf:resource="thsys/71009"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18375-2">
    <skos:broader rdf:resource="thsys/70979"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18375-2">
    <skos:broader rdf:resource="descriptor/15752-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18375-2">
    <skos:broader rdf:resource="descriptor/11820-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18978-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13775-4">
    <skos:broader rdf:resource="thsys/73392"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13775-4">
    <skos:broader rdf:resource="thsys/71045"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13775-4">
    <skos:broader rdf:resource="thsys/70899"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13775-4">
    <skos:broader rdf:resource="descriptor/12252-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10761-2">
    <skos:prefLabel xml:lang="en">Crawling peg</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10761-2">
    <skos:prefLabel xml:lang="de">Stufenflexibilität</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19774-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12737-3">
    <skos:related rdf:resource="descriptor/15611-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12737-3">
    <skos:related rdf:resource="descriptor/19660-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12470-3">
    <skos:related rdf:resource="descriptor/12208-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12470-3">
    <skos:related rdf:resource="descriptor/12514-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12470-3">
    <skos:related rdf:resource="descriptor/12053-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13643-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17161-5">
    <skos:broader rdf:resource="thsys/70131"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17161-5">
    <skos:broader rdf:resource="descriptor/17016-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17161-5">
    <skos:broader rdf:resource="descriptor/17991-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17161-5">
    <skos:broader rdf:resource="descriptor/17983-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16326-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14169-4">
    <skos:prefLabel xml:lang="en">Animal</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14169-4">
    <skos:prefLabel xml:lang="de">Tier</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12913-2">
    <skos:related rdf:resource="descriptor/15103-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12913-2">
    <skos:related rdf:resource="descriptor/16692-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12913-2">
    <skos:related rdf:resource="descriptor/10007-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12913-2">
    <skos:related rdf:resource="descriptor/11820-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12913-2">
    <skos:related rdf:resource="descriptor/15019-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12913-2">
    <skos:related rdf:resource="descriptor/18488-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12913-2">
    <skos:related rdf:resource="descriptor/15703-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12913-2">
    <skos:related rdf:resource="descriptor/12051-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12913-2">
    <skos:related rdf:resource="descriptor/10500-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11438-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11807-6">
    <skos:altLabel xml:lang="de">Fischwirtschaftspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19840-3">
    <skos:related rdf:resource="descriptor/18314-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19840-3">
    <skos:related rdf:resource="descriptor/15889-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19840-3">
    <skos:related rdf:resource="descriptor/10583-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19840-3">
    <skos:related rdf:resource="descriptor/10495-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19840-3">
    <skos:related rdf:resource="descriptor/10488-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14204-0">
    <skos:related rdf:resource="descriptor/12988-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18872-2">
    <skos:broader rdf:resource="thsys/70844"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18872-2">
    <skos:broader rdf:resource="thsys/70140"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18872-2">
    <skos:broader rdf:resource="descriptor/18871-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11551-1">
    <skos:altLabel xml:lang="en">Taking</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11551-1">
    <skos:altLabel xml:lang="de">Kosten der Bürokratieüberwälzung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11551-1">
    <skos:altLabel xml:lang="de">Folgekosten gesetzlicher Regelungen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11551-1">
    <skos:altLabel xml:lang="de">Bürokratiefolgekosten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10902-3">
    <skos:prefLabel xml:lang="en">Market share</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10902-3">
    <skos:prefLabel xml:lang="de">Marktanteil</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10511-4">
    <skos:related rdf:resource="descriptor/16566-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10511-4">
    <skos:related rdf:resource="descriptor/10506-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10511-4">
    <skos:related rdf:resource="descriptor/18313-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10511-4">
    <skos:related rdf:resource="descriptor/10501-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10511-4">
    <skos:related rdf:resource="descriptor/10498-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10511-4">
    <skos:related rdf:resource="descriptor/10522-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10511-4">
    <skos:related rdf:resource="descriptor/19286-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10511-4">
    <skos:related rdf:resource="descriptor/13425-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18836-6">
    <skos:broader rdf:resource="thsys/70856"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18836-6">
    <skos:broader rdf:resource="thsys/70834"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18836-6">
    <skos:broader rdf:resource="descriptor/18835-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19474-2">
    <skos:prefLabel xml:lang="en">Urban poverty</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19474-2">
    <skos:prefLabel xml:lang="de">Städtische Armut</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71083">
    <skos:broader rdf:resource="thsys/70144"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18007-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19052-0">
    <skos:broader rdf:resource="thsys/70997"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19052-0">
    <skos:broader rdf:resource="descriptor/13051-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19816-0">
    <skos:broader rdf:resource="thsys/73334"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19816-0">
    <skos:broader rdf:resource="descriptor/15981-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19816-0">
    <skos:broader rdf:resource="descriptor/16198-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10779-4">
    <skos:broader rdf:resource="thsys/71103"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10779-4">
    <skos:broader rdf:resource="thsys/71088"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10779-4">
    <skos:broader rdf:resource="descriptor/10807-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19146-5">
    <skos:broader rdf:resource="thsys/70485"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13410-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10185-5">
    <skos:prefLabel xml:lang="en">Factor demand</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10185-5">
    <skos:prefLabel xml:lang="de">Faktornachfrage</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18316-4">
    <skos:broader rdf:resource="thsys/71029"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18316-4">
    <skos:broader rdf:resource="descriptor/12949-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70967">
    <skos:narrower rdf:resource="descriptor/11845-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70967">
    <skos:narrower rdf:resource="descriptor/13848-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70967">
    <skos:narrower rdf:resource="descriptor/18503-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70967">
    <skos:narrower rdf:resource="descriptor/18502-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70967">
    <skos:narrower rdf:resource="descriptor/12932-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70967">
    <skos:narrower rdf:resource="descriptor/18753-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70967">
    <skos:narrower rdf:resource="descriptor/11843-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70967">
    <skos:narrower rdf:resource="descriptor/11563-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70967">
    <skos:narrower rdf:resource="descriptor/12014-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70901">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16002-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18722-0">
    <skos:scopeNote xml:lang="de">Grundsätze über eine ausreichende Eigenkapitalausstattung und Zahlungsbereitschaft.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71111">
    <skos:prefLabel xml:lang="en">V.03.06  National accounts</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71111">
    <skos:prefLabel xml:lang="de">V.03.06  Volkswirtschaftliche Gesamtrechnung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14596-6">
    <skos:related rdf:resource="descriptor/14731-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14596-6">
    <skos:related rdf:resource="descriptor/13188-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14596-6">
    <skos:related rdf:resource="descriptor/14030-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14596-6">
    <skos:related rdf:resource="descriptor/18320-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14596-6">
    <skos:related rdf:resource="descriptor/19339-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10843-0">
    <skos:related rdf:resource="descriptor/12810-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10843-0">
    <skos:related rdf:resource="descriptor/18437-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15683-6">
    <skos:altLabel xml:lang="en">Snowboarding</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15683-6">
    <skos:altLabel xml:lang="de">Skisport</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15683-6">
    <skos:altLabel xml:lang="de">Schisport</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13177-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14268-2">
    <skos:related rdf:resource="descriptor/14378-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14268-2">
    <skos:related rdf:resource="descriptor/13028-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16287-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19204-3">
    <skos:prefLabel xml:lang="en">Evolutionary economics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19204-3">
    <skos:prefLabel xml:lang="de">Evolutionsökonomik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12065-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18156-0">
    <skos:related rdf:resource="descriptor/10179-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18156-0">
    <skos:related rdf:resource="descriptor/19376-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18156-0">
    <skos:related rdf:resource="descriptor/19374-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18156-0">
    <skos:related rdf:resource="descriptor/10381-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70816">
    <skos:broader rdf:resource="thsys/70818"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18398-4">
    <skos:broader rdf:resource="thsys/70985"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18398-4">
    <skos:broader rdf:resource="descriptor/10470-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12936-4">
    <skos:related rdf:resource="descriptor/15869-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14744-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14188-0">
    <skos:related rdf:resource="descriptor/12979-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14188-0">
    <skos:related rdf:resource="descriptor/18378-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14188-0">
    <skos:related rdf:resource="descriptor/16023-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14188-0">
    <skos:related rdf:resource="descriptor/18990-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14188-0">
    <skos:related rdf:resource="descriptor/14189-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14188-0">
    <skos:related rdf:resource="descriptor/13179-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14188-0">
    <skos:related rdf:resource="descriptor/19624-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14188-0">
    <skos:related rdf:resource="descriptor/13256-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14188-0">
    <skos:related rdf:resource="descriptor/15733-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14188-0">
    <skos:related rdf:resource="descriptor/10490-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14983-6">
    <skos:prefLabel xml:lang="en">Spirits</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14983-6">
    <skos:prefLabel xml:lang="de">Spirituosen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17745-0">
    <skos:altLabel xml:lang="de">Arabische Republik Ägypten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10065-1">
    <skos:related rdf:resource="descriptor/16727-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11335-0">
    <skos:related rdf:resource="descriptor/18173-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11346-2">
    <skos:related rdf:resource="descriptor/16150-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11346-2">
    <skos:related rdf:resource="descriptor/16484-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11346-2">
    <skos:related rdf:resource="descriptor/11350-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11346-2">
    <skos:related rdf:resource="descriptor/11337-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11346-2">
    <skos:related rdf:resource="descriptor/11342-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11346-2">
    <skos:related rdf:resource="descriptor/20455-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11346-2">
    <skos:related rdf:resource="descriptor/18169-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11346-2">
    <skos:related rdf:resource="descriptor/11339-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11346-2">
    <skos:related rdf:resource="descriptor/11314-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19437-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13465-5">
    <skos:narrower rdf:resource="descriptor/15080-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13465-5">
    <skos:narrower rdf:resource="descriptor/15077-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13465-5">
    <skos:narrower rdf:resource="descriptor/10835-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13136-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13420-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10524-2">
    <skos:narrower rdf:resource="descriptor/10541-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10524-2">
    <skos:narrower rdf:resource="descriptor/18325-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10524-2">
    <skos:narrower rdf:resource="descriptor/10505-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10524-2">
    <skos:narrower rdf:resource="descriptor/10575-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10524-2">
    <skos:narrower rdf:resource="descriptor/18481-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10524-2">
    <skos:narrower rdf:resource="descriptor/10570-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10524-2">
    <skos:narrower rdf:resource="descriptor/10517-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10524-2">
    <skos:narrower rdf:resource="descriptor/10562-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10524-2">
    <skos:narrower rdf:resource="descriptor/10534-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14509-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13841-3">
    <skos:altLabel xml:lang="en">Housing expenditures</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70095">
    <skos:broader rdf:resource="thsys/n"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70852">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70968">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18344-6">
    <skos:altLabel xml:lang="en">Driftnet fishing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18344-6">
    <skos:altLabel xml:lang="de">Treibnetztechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18344-6">
    <skos:altLabel xml:lang="de">Fischfangtechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18805-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12014-1">
    <skos:altLabel xml:lang="de">Baulandausweisung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19558-3">
    <skos:scopeNote xml:lang="de">Verfahren zur Bewertung der Güter Umwelt, Naturparks, Erholungsgebiete, usw.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11895-4">
    <skos:broader rdf:resource="thsys/70058"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11895-4">
    <skos:broader rdf:resource="descriptor/10513-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12575-3">
    <skos:prefLabel xml:lang="en">Redundancy programme</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12575-3">
    <skos:prefLabel xml:lang="de">Sozialplan</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17632-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id248">
    <rdf:value xml:lang="de">Für die organisierte Verfolgung spezieller Interessen durch Verbände u.ä.</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11215-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16052-1">
    <skos:prefLabel xml:lang="en">Trading law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16052-1">
    <skos:prefLabel xml:lang="de">Gewerberecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18509-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10978-5">
    <skos:related rdf:resource="descriptor/11024-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10978-5">
    <skos:related rdf:resource="descriptor/10217-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10978-5">
    <skos:related rdf:resource="descriptor/10397-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10978-5">
    <skos:related rdf:resource="descriptor/10960-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10978-5">
    <skos:related rdf:resource="descriptor/10213-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10978-5">
    <skos:related rdf:resource="descriptor/10976-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10978-5">
    <skos:related rdf:resource="descriptor/10201-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10978-5">
    <skos:related rdf:resource="descriptor/12475-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10978-5">
    <skos:related rdf:resource="descriptor/11494-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10978-5">
    <skos:related rdf:resource="descriptor/11543-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10978-5">
    <skos:related rdf:resource="descriptor/10380-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10978-5">
    <skos:related rdf:resource="descriptor/12794-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11147-1">
    <skos:prefLabel xml:lang="en">Agroindustrial complex</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11147-1">
    <skos:prefLabel xml:lang="de">Agrar-Industrie-Komplex</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13023-2">
    <skos:prefLabel xml:lang="en">Cement industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13023-2">
    <skos:prefLabel xml:lang="de">Zementindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19384-3">
    <skos:prefLabel xml:lang="en">Voucher</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19384-3">
    <skos:prefLabel xml:lang="de">Voucher</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10217-4">
    <skos:broader rdf:resource="thsys/70178"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10217-4">
    <skos:broader rdf:resource="descriptor/10144-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13777-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11560-0">
    <skos:prefLabel xml:lang="en">Military budget</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11560-0">
    <skos:prefLabel xml:lang="de">Verteidigungshaushalt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16970-5">
    <skos:broader rdf:resource="thsys/70003"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16970-5">
    <skos:broader rdf:resource="descriptor/16972-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18332-6">
    <skos:broader rdf:resource="thsys/73362"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18332-6">
    <skos:broader rdf:resource="thsys/71020"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18332-6">
    <skos:broader rdf:resource="descriptor/11801-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11487-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70270">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10460-2">
    <skos:narrower rdf:resource="descriptor/10125-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10460-2">
    <skos:narrower rdf:resource="descriptor/10440-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73369">
    <skos:narrower rdf:resource="descriptor/16487-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73369">
    <skos:narrower rdf:resource="descriptor/19707-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73369">
    <skos:narrower rdf:resource="descriptor/16483-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73369">
    <skos:narrower rdf:resource="descriptor/11404-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73369">
    <skos:narrower rdf:resource="descriptor/16517-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73369">
    <skos:narrower rdf:resource="descriptor/16516-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73369">
    <skos:narrower rdf:resource="descriptor/16515-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73369">
    <skos:narrower rdf:resource="descriptor/19706-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73369">
    <skos:narrower rdf:resource="descriptor/11402-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73369">
    <skos:narrower rdf:resource="descriptor/16514-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73369">
    <skos:narrower rdf:resource="descriptor/11399-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73369">
    <skos:narrower rdf:resource="descriptor/11398-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73369">
    <skos:narrower rdf:resource="descriptor/11397-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73369">
    <skos:narrower rdf:resource="descriptor/18945-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73369">
    <skos:narrower rdf:resource="descriptor/11392-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10949-5">
    <skos:scopeNote xml:lang="de">Alle Anbieter, die aktuell oder potenziell in Wettbewerb zueinander stehen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19784-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12243-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19773-6">
    <skos:related rdf:resource="descriptor/19752-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19773-6">
    <skos:related rdf:resource="descriptor/16308-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19773-6">
    <skos:related rdf:resource="descriptor/10595-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19773-6">
    <skos:related rdf:resource="descriptor/19680-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19773-6">
    <skos:related rdf:resource="descriptor/16587-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19773-6">
    <skos:related rdf:resource="descriptor/19521-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19773-6">
    <skos:related rdf:resource="descriptor/16266-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19773-6">
    <skos:related rdf:resource="descriptor/11243-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19773-6">
    <skos:related rdf:resource="descriptor/18073-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12725-3">
    <skos:altLabel xml:lang="de">Produktionstyp</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12725-3">
    <skos:altLabel xml:lang="de">Produktionssystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12725-3">
    <skos:altLabel xml:lang="de">Fertigungstyp</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14381-6">
    <skos:narrower rdf:resource="descriptor/14393-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14381-6">
    <skos:narrower rdf:resource="descriptor/14343-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14381-6">
    <skos:narrower rdf:resource="descriptor/14392-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14381-6">
    <skos:narrower rdf:resource="descriptor/14385-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14381-6">
    <skos:narrower rdf:resource="descriptor/14384-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14381-6">
    <skos:narrower rdf:resource="descriptor/14383-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14381-6">
    <skos:narrower rdf:resource="descriptor/14380-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14381-6">
    <skos:narrower rdf:resource="descriptor/14379-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14381-6">
    <skos:narrower rdf:resource="descriptor/14355-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14381-6">
    <skos:narrower rdf:resource="descriptor/14378-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14381-6">
    <skos:narrower rdf:resource="descriptor/14371-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14381-6">
    <skos:narrower rdf:resource="descriptor/14368-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14381-6">
    <skos:narrower rdf:resource="descriptor/14366-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10303-4">
    <zbwext:useInsteadNote rdf:nodeID="id9"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19664-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70816">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.32.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18548-4">
    <skos:broader rdf:resource="thsys/71065"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18548-4">
    <skos:broader rdf:resource="thsys/70947"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18548-4">
    <skos:broader rdf:resource="descriptor/19460-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18548-4">
    <skos:broader rdf:resource="descriptor/13200-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17994-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20404-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14435-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70997">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#hiddenLabel">
    <skos:definition xml:lang="en">A lexical label for a resource that should be hidden when
      generating visual displays of the resource, but should still be accessible to free text search
      operations.</skos:definition>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71067">
    <skos:narrower rdf:resource="descriptor/13862-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71067">
    <skos:narrower rdf:resource="descriptor/12005-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71067">
    <skos:narrower rdf:resource="descriptor/11934-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71067">
    <skos:narrower rdf:resource="descriptor/18160-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71067">
    <skos:narrower rdf:resource="descriptor/11915-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71067">
    <skos:narrower rdf:resource="descriptor/11932-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71067">
    <skos:narrower rdf:resource="descriptor/19291-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71067">
    <skos:narrower rdf:resource="descriptor/19448-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71067">
    <skos:narrower rdf:resource="descriptor/11914-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71067">
    <skos:narrower rdf:resource="descriptor/11913-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71067">
    <skos:narrower rdf:resource="descriptor/19452-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71067">
    <skos:narrower rdf:resource="descriptor/11897-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71067">
    <skos:narrower rdf:resource="descriptor/11849-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71067">
    <skos:narrower rdf:resource="descriptor/20321-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71067">
    <skos:narrower rdf:resource="descriptor/18816-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71067">
    <skos:narrower rdf:resource="descriptor/18799-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71067">
    <skos:narrower rdf:resource="descriptor/19364-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71067">
    <skos:narrower rdf:resource="descriptor/19451-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71067">
    <skos:narrower rdf:resource="descriptor/18490-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71067">
    <skos:narrower rdf:resource="descriptor/11906-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71067">
    <skos:narrower rdf:resource="descriptor/11863-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71067">
    <skos:narrower rdf:resource="descriptor/18546-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71067">
    <skos:narrower rdf:resource="descriptor/11905-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13729-4">
    <skos:related rdf:resource="descriptor/12265-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13729-4">
    <skos:related rdf:resource="descriptor/18677-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13729-4">
    <skos:related rdf:resource="descriptor/13716-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13729-4">
    <skos:related rdf:resource="descriptor/18736-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14729-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15050-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19519-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19269-3">
    <skos:related rdf:resource="descriptor/19271-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14383-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12786-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18623-2">
    <skos:altLabel xml:lang="en">Transport ecology</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18623-2">
    <skos:altLabel xml:lang="de">Umweltbelastung durch Verkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17933-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15731-0">
    <skos:narrower rdf:resource="descriptor/13327-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13337-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id62">
    <rdfs:seeAlso rdf:resource="descriptor/13306-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18032-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19902-0">
    <skos:altLabel xml:lang="de">Vollzeitarbeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19902-0">
    <skos:altLabel xml:lang="de">Normalarbeitsverhältnis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19902-0">
    <skos:altLabel xml:lang="de">Beschäftigungsverhältnis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19854-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24704-2">
    <skos:prefLabel xml:lang="en">Business process</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24704-2">
    <skos:prefLabel xml:lang="de">Geschäftsprozess</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15463-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19229-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11708-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10642-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73393">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12208-2">
    <skos:narrower rdf:resource="descriptor/12203-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10940-2">
    <skos:altLabel xml:lang="en">Exclusive trading</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10940-2">
    <skos:altLabel xml:lang="de">Vertragshändlersystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10940-2">
    <skos:altLabel xml:lang="de">Selektiver Vertrieb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10940-2">
    <skos:altLabel xml:lang="de">Ausschließlichkeitsvertrag</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10940-2">
    <skos:altLabel xml:lang="de">Alleinvertrieb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10940-2">
    <skos:altLabel xml:lang="de">Alleinbezugsbindung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10948-0">
    <skos:related rdf:resource="descriptor/10895-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10948-0">
    <skos:related rdf:resource="descriptor/19432-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10948-0">
    <skos:related rdf:resource="descriptor/10949-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10948-0">
    <skos:related rdf:resource="descriptor/10928-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10948-0">
    <skos:related rdf:resource="descriptor/10944-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12306-2">
    <skos:broader rdf:resource="thsys/70005"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14940-3">
    <skos:prefLabel xml:lang="en">Bakery product</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14940-3">
    <skos:prefLabel xml:lang="de">Backwaren</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16786-5">
    <skos:broader rdf:resource="thsys/70823"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16786-5">
    <skos:broader rdf:resource="descriptor/15473-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19573-0">
    <skos:related rdf:resource="descriptor/15328-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18111-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12914-0">
    <zbwext:useInsteadNote rdf:nodeID="id14"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18915-3">
    <skos:related rdf:resource="descriptor/13214-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18915-3">
    <skos:related rdf:resource="descriptor/14033-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17947-2">
    <skos:prefLabel xml:lang="en">Samoa</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17947-2">
    <skos:prefLabel xml:lang="de">Samoa</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14182-5">
    <skos:broader rdf:resource="thsys/70650"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14182-5">
    <skos:broader rdf:resource="thsys/70227"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14182-5">
    <skos:broader rdf:resource="descriptor/14175-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19159-3">
    <skos:broader rdf:resource="thsys/73392"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19159-3">
    <skos:broader rdf:resource="thsys/71034"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19159-3">
    <skos:broader rdf:resource="descriptor/16036-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12285-5">
    <skos:prefLabel xml:lang="en">Return to capital</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12285-5">
    <skos:prefLabel xml:lang="de">Kapitalertrag</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18375-2">
    <skos:narrower rdf:resource="descriptor/18374-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18375-2">
    <skos:narrower rdf:resource="descriptor/18373-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18375-2">
    <skos:narrower rdf:resource="descriptor/18371-3"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id318">
    <rdf:value xml:lang="de">Für die distributive Wirkung von Steuern</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16007-6">
    <skos:related rdf:resource="descriptor/12189-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13683-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15467-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18758-0">
    <skos:scopeNote xml:lang="de">Leasing von Hardware und Software</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15640-3">
    <skos:narrower rdf:resource="descriptor/19737-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15640-3">
    <skos:narrower rdf:resource="descriptor/19740-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15640-3">
    <skos:narrower rdf:resource="descriptor/19736-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15640-3">
    <skos:narrower rdf:resource="descriptor/16401-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15640-3">
    <skos:narrower rdf:resource="descriptor/19739-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13620-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11372-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18663-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18839-0">
    <skos:prefLabel xml:lang="en">Cooperative management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18839-0">
    <skos:prefLabel xml:lang="de">Genossenschaftsmanagement</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17700-1">
    <skos:prefLabel xml:lang="en">Cote d'Ivoire</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17700-1">
    <skos:prefLabel xml:lang="de">Elfenbeinküste</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18922-6">
    <skos:narrower rdf:resource="descriptor/18099-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14984-4">
    <skos:broader rdf:resource="thsys/73300"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14984-4">
    <skos:broader rdf:resource="descriptor/14971-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15765-4">
    <skos:prefLabel xml:lang="en">Ecology</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15765-4">
    <skos:prefLabel xml:lang="de">Umweltforschung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13823-5">
    <skos:prefLabel xml:lang="en">Actuarial theory</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13823-5">
    <skos:prefLabel xml:lang="de">Versicherungstechnik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19790-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16783-4">
    <skos:prefLabel xml:lang="en">Library use</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16783-4">
    <skos:prefLabel xml:lang="de">Bibliotheksbenutzung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12655-5">
    <skos:prefLabel xml:lang="en">Bill of materials</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12655-5">
    <skos:prefLabel xml:lang="de">Stückliste</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19551-3">
    <skos:prefLabel xml:lang="en">Managed care</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19551-3">
    <skos:prefLabel xml:lang="de">Managed Care</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15458-6">
    <skos:broader rdf:resource="thsys/73104"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15458-6">
    <skos:broader rdf:resource="thsys/71035"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15458-6">
    <skos:broader rdf:resource="descriptor/15045-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19454-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#topConceptOf">
    <rdfs:subPropertyOf rdf:resource="http://www.w3.org/2004/02/skos/core#inScheme"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16080-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16295-3">
    <skos:related rdf:resource="descriptor/10343-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16295-3">
    <skos:related rdf:resource="descriptor/19285-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16295-3">
    <skos:related rdf:resource="descriptor/11127-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16295-3">
    <skos:related rdf:resource="descriptor/16332-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16295-3">
    <skos:related rdf:resource="descriptor/13420-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16295-3">
    <skos:related rdf:resource="descriptor/11034-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16295-3">
    <skos:related rdf:resource="descriptor/16286-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16295-3">
    <skos:related rdf:resource="descriptor/19683-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16295-3">
    <skos:related rdf:resource="descriptor/19668-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16295-3">
    <skos:related rdf:resource="descriptor/15889-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16295-3">
    <skos:related rdf:resource="descriptor/16521-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16295-3">
    <skos:related rdf:resource="descriptor/10816-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16295-3">
    <skos:related rdf:resource="descriptor/16003-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16295-3">
    <skos:related rdf:resource="descriptor/16470-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16295-3">
    <skos:related rdf:resource="descriptor/16524-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16295-3">
    <skos:related rdf:resource="descriptor/16510-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16295-3">
    <skos:related rdf:resource="descriptor/16247-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12292-1">
    <skos:broader rdf:resource="thsys/71090"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12292-1">
    <skos:broader rdf:resource="thsys/71089"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12292-1">
    <skos:broader rdf:resource="descriptor/18681-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12292-1">
    <skos:broader rdf:resource="descriptor/10811-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12292-1">
    <skos:broader rdf:resource="descriptor/10823-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19035-0">
    <skos:altLabel xml:lang="de">Vorgabe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19035-0">
    <skos:altLabel xml:lang="de">Plan</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15050-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15264-5">
    <skos:broader rdf:resource="thsys/72186"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15264-5">
    <skos:broader rdf:resource="descriptor/15272-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19210-1">
    <skos:broader rdf:resource="thsys/71118"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19210-1">
    <skos:broader rdf:resource="thsys/70057"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19210-1">
    <skos:broader rdf:resource="descriptor/10164-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18756-4">
    <skos:broader rdf:resource="thsys/70884"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18756-4">
    <skos:broader rdf:resource="descriptor/13837-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70968">
    <skos:narrower rdf:resource="descriptor/13903-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70968">
    <skos:narrower rdf:resource="descriptor/13186-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70968">
    <skos:narrower rdf:resource="descriptor/13185-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70968">
    <skos:narrower rdf:resource="descriptor/13077-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70968">
    <skos:narrower rdf:resource="descriptor/13181-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70968">
    <skos:narrower rdf:resource="descriptor/13180-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16136-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19703-6">
    <skos:prefLabel xml:lang="en">Constitutional reform</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19703-6">
    <skos:prefLabel xml:lang="de">Verfassungsreform</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18383-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13862-2">
    <skos:prefLabel xml:lang="en">Residential location</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13862-2">
    <skos:prefLabel xml:lang="de">Wohnstandort</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18219-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11397-6">
    <skos:editorialNote xml:lang="de">Möglichst spezifischer indexieren.</skos:editorialNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16516-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70386">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18738-6">
    <skos:prefLabel xml:lang="en">Private health insurance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18738-6">
    <skos:prefLabel xml:lang="de">Private Krankenversicherung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19164-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10606-0">
    <skos:scopeNote xml:lang="de">Vgl. auch Außenhandelsbeschränkung plus Unterbegriffe.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13620-5">
    <skos:altLabel xml:lang="de">Stauerei</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13620-5">
    <skos:altLabel xml:lang="de">Frachtumschlag</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13381-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14583-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16016-5">
    <skos:altLabel xml:lang="de">Verfassungstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16016-5">
    <skos:altLabel xml:lang="de">Verfassungssystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16016-5">
    <skos:altLabel xml:lang="de">Staatsverfassung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16016-5">
    <skos:altLabel xml:lang="de">Staatliche Verfassung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16016-5">
    <skos:altLabel xml:lang="de">Konstitutionalismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16016-5">
    <skos:altLabel xml:lang="de">Grundgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16782-6">
    <skos:broader rdf:resource="thsys/70823"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16782-6">
    <skos:broader rdf:resource="thsys/70383"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16782-6">
    <skos:broader rdf:resource="descriptor/16760-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10693-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71110">
    <rdfs:label xml:lang="en">Economic structure</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71110">
    <rdfs:label xml:lang="de">Wirtschaftsstruktur</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13727-1">
    <skos:broader rdf:resource="thsys/71107"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13727-1">
    <skos:broader rdf:resource="thsys/71046"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13727-1">
    <skos:broader rdf:resource="descriptor/12299-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18674-6">
    <skos:narrower rdf:resource="descriptor/24707-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70368">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">B.05.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15105-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18806-1">
    <skos:related rdf:resource="descriptor/19446-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18806-1">
    <skos:related rdf:resource="descriptor/19378-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18806-1">
    <skos:related rdf:resource="descriptor/11553-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18806-1">
    <skos:related rdf:resource="descriptor/11552-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19038-1">
    <skos:broader rdf:resource="thsys/70582"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16708-4">
    <skos:broader rdf:resource="thsys/73394"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16708-4">
    <skos:broader rdf:resource="descriptor/16719-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11180-3">
    <skos:prefLabel xml:lang="en">Labour</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11180-3">
    <skos:prefLabel xml:lang="de">Arbeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19577-6">
    <skos:altLabel xml:lang="en">Wavelets</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19577-6">
    <skos:altLabel xml:lang="en">Random coefficient model</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19577-6">
    <skos:altLabel xml:lang="en">Kalman filter model</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10000-1">
    <skos:broader rdf:resource="thsys/70175"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10000-1">
    <skos:broader rdf:resource="thsys/70140"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10000-1">
    <skos:broader rdf:resource="descriptor/10012-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12570-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16749-4">
    <skos:altLabel xml:lang="de">Neomalthusianismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16749-4">
    <skos:altLabel xml:lang="de">Malthusianismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14614-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15722-1">
    <skos:prefLabel xml:lang="en">Social behaviour</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15722-1">
    <skos:prefLabel xml:lang="de">Soziales Verhalten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18667-3">
    <skos:prefLabel xml:lang="en">Financial engineering</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18667-3">
    <skos:prefLabel xml:lang="de">Financial Engineering</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11203-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11063-0">
    <skos:broader rdf:resource="thsys/73382"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11063-0">
    <skos:broader rdf:resource="thsys/70039"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12212-4">
    <skos:broader rdf:resource="thsys/70897"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12212-4">
    <skos:broader rdf:resource="thsys/70485"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12212-4">
    <skos:broader rdf:resource="thsys/70187"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10772-4">
    <skos:altLabel xml:lang="de">Währungskonkurrenz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10772-4">
    <skos:altLabel xml:lang="de">Parallelwährung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10772-4">
    <skos:altLabel xml:lang="de">Konkurrenz der Währungen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10772-4">
    <skos:altLabel xml:lang="de">Entnationalisierung des Geldes</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10772-4">
    <skos:altLabel xml:lang="de">Doppelwährung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10772-4">
    <skos:altLabel xml:lang="de">Bimetallismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10715-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11400-1">
    <skos:prefLabel xml:lang="en">Educational opportunities</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11400-1">
    <skos:prefLabel xml:lang="de">Bildungschancen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11568-5">
    <skos:narrower rdf:resource="descriptor/11573-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11568-5">
    <skos:narrower rdf:resource="descriptor/19384-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11568-5">
    <skos:narrower rdf:resource="descriptor/11572-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11568-5">
    <skos:narrower rdf:resource="descriptor/11570-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12713-3">
    <skos:altLabel xml:lang="en">TQM</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12713-3">
    <skos:altLabel xml:lang="en">Total quality management</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12713-3">
    <skos:altLabel xml:lang="en">Six Sigma</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12713-3">
    <skos:altLabel xml:lang="en">Quality assurance</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12713-3">
    <skos:altLabel xml:lang="de">Qualitätswirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12713-3">
    <skos:altLabel xml:lang="de">Qualitätswesen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12713-3">
    <skos:altLabel xml:lang="de">Qualitätsüberwachung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12713-3">
    <skos:altLabel xml:lang="de">Qualitätssicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12713-3">
    <skos:altLabel xml:lang="de">Qualitätskontrolle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12713-3">
    <skos:altLabel xml:lang="de">Gütesicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12713-3">
    <skos:altLabel xml:lang="de">Ausschusskontrolle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12713-3">
    <skos:altLabel xml:lang="de">Ausschußkontrolle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16245-4">
    <skos:prefLabel xml:lang="en">Authoritarianism</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16245-4">
    <skos:prefLabel xml:lang="de">Autoritäres System</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19840-3">
    <skos:broader rdf:resource="thsys/73347"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19840-3">
    <skos:broader rdf:resource="thsys/70504"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19840-3">
    <skos:broader rdf:resource="thsys/70091"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15466-0">
    <skos:narrower rdf:resource="descriptor/16277-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15466-0">
    <skos:narrower rdf:resource="descriptor/15476-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15466-0">
    <skos:narrower rdf:resource="descriptor/15475-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15466-0">
    <skos:narrower rdf:resource="descriptor/15472-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15466-0">
    <skos:narrower rdf:resource="descriptor/15468-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15466-0">
    <skos:narrower rdf:resource="descriptor/15467-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15466-0">
    <skos:narrower rdf:resource="descriptor/20756-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10627-6">
    <skos:prefLabel xml:lang="en">Economic union</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10627-6">
    <skos:prefLabel xml:lang="de">Wirtschaftsunion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id356">
    <rdfs:seeAlso rdf:resource="descriptor/14548-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70969">
    <skos:broader rdf:resource="thsys/70970"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13151-0">
    <skos:broader rdf:resource="thsys/73295"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13151-0">
    <skos:broader rdf:resource="descriptor/13157-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19332-1">
    <skos:broader rdf:resource="thsys/71095"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19332-1">
    <skos:broader rdf:resource="descriptor/10764-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16896-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19795-3">
    <skos:prefLabel xml:lang="en">Engineering</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19795-3">
    <skos:prefLabel xml:lang="de">Ingenieurwissenschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14327-5">
    <skos:prefLabel xml:lang="en">Catalyst</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14327-5">
    <skos:prefLabel xml:lang="de">Katalysator</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15267-6">
    <skos:altLabel xml:lang="de">Monte-Carlo-Simulation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15267-6">
    <skos:altLabel xml:lang="de">Monte-Carlo Methode</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15267-6">
    <skos:altLabel xml:lang="de">Monte Carlo Methode</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id168">
    <rdf:value xml:lang="de">Für Literatur zur rechtlichen Stellung der Gemeindevertretung</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13075-4">
    <skos:broader rdf:resource="thsys/70996"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13075-4">
    <skos:broader rdf:resource="descriptor/13076-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10494-6">
    <skos:prefLabel xml:lang="en">Development indicator</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10494-6">
    <skos:prefLabel xml:lang="de">Entwicklungsindikator</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11821-5">
    <skos:related rdf:resource="descriptor/11820-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11821-5">
    <skos:related rdf:resource="descriptor/18478-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19284-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11718-5">
    <skos:prefLabel xml:lang="en">Public Good</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11718-5">
    <skos:prefLabel xml:lang="de">Öffentliches Gut</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16628-2">
    <skos:related rdf:resource="descriptor/19756-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16628-2">
    <skos:related rdf:resource="descriptor/18142-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16628-2">
    <skos:related rdf:resource="descriptor/16678-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16628-2">
    <skos:related rdf:resource="descriptor/15920-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16338-4">
    <skos:broader rdf:resource="thsys/73379"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15716-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11611-2">
    <skos:narrower rdf:resource="descriptor/11609-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18980-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16713-4">
    <skos:broader rdf:resource="thsys/73394"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16713-4">
    <skos:broader rdf:resource="descriptor/16719-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14130-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19804-0">
    <skos:altLabel xml:lang="de">Agenten-Technologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17497-0">
    <skos:prefLabel xml:lang="en">Kyrgyzstan</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17497-0">
    <skos:prefLabel xml:lang="de">Kirgisistan</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18190-0">
    <skos:prefLabel xml:lang="en">Cotton production</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18190-0">
    <skos:prefLabel xml:lang="de">Baumwollanbau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16072-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15715-5">
    <skos:broader rdf:resource="thsys/73395"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15715-5">
    <skos:broader rdf:resource="thsys/70504"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19152-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12592-3">
    <skos:related rdf:resource="descriptor/11341-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12592-3">
    <skos:related rdf:resource="descriptor/12502-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12592-3">
    <skos:related rdf:resource="descriptor/11558-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12592-3">
    <skos:related rdf:resource="descriptor/15839-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12592-3">
    <skos:related rdf:resource="descriptor/11052-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12592-3">
    <skos:related rdf:resource="descriptor/13309-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12592-3">
    <skos:related rdf:resource="descriptor/15837-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10501-0">
    <skos:prefLabel xml:lang="en">Informal sector</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10501-0">
    <skos:prefLabel xml:lang="de">Informeller Sektor</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16740-1">
    <skos:broader rdf:resource="thsys/70141"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16740-1">
    <skos:broader rdf:resource="descriptor/16739-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16740-1">
    <skos:broader rdf:resource="descriptor/16745-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16797-0">
    <skos:narrower rdf:resource="descriptor/16768-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13496-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14192-2">
    <skos:prefLabel xml:lang="en">Mining product</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14192-2">
    <skos:prefLabel xml:lang="de">Bergbauprodukt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70244">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11957-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12298-3">
    <skos:prefLabel xml:lang="en">Hidden Reserves</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12298-3">
    <skos:prefLabel xml:lang="de">Stille Reserve</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17724-1">
    <skos:altLabel xml:lang="de">Westafrikanische Staaten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17724-1">
    <skos:altLabel xml:lang="de">Westafrikanische Länder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18351-2">
    <skos:prefLabel xml:lang="en">Energy statistics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18351-2">
    <skos:prefLabel xml:lang="de">Energiestatistik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18162-5">
    <skos:related rdf:resource="descriptor/14266-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18216-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10667-1">
    <skos:broader rdf:resource="thsys/71099"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10667-1">
    <skos:broader rdf:resource="thsys/71095"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10667-1">
    <skos:broader rdf:resource="thsys/70355"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10667-1">
    <skos:broader rdf:resource="descriptor/11506-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73356">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11734-0">
    <skos:related rdf:resource="descriptor/11613-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11734-0">
    <skos:related rdf:resource="descriptor/11556-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11734-0">
    <skos:related rdf:resource="descriptor/11554-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70039">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.06</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10223-2">
    <skos:prefLabel xml:lang="en">Incomplete market</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10223-2">
    <skos:prefLabel xml:lang="de">Unvollkommener Markt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10683-3">
    <skos:related rdf:resource="descriptor/19310-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10145-3">
    <skos:related rdf:resource="descriptor/10144-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20810-2">
    <skos:altLabel xml:lang="en">EU-Stability and Growth Pact</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20810-2">
    <skos:altLabel xml:lang="de">Stabilitätspakt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20810-2">
    <skos:altLabel xml:lang="de">Stabilitätskriterien</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13536-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13374-1">
    <skos:altLabel xml:lang="en">Veterinarians</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13374-1">
    <skos:altLabel xml:lang="de">Veterinärmedizin</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13374-1">
    <skos:altLabel xml:lang="de">Veterinärinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13374-1">
    <skos:altLabel xml:lang="de">Veterinäre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13374-1">
    <skos:altLabel xml:lang="de">Tiermedizinerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13374-1">
    <skos:altLabel xml:lang="de">Tiermediziner</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13374-1">
    <skos:altLabel xml:lang="de">Tiermedizin</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13374-1">
    <skos:altLabel xml:lang="de">Tierklinik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13374-1">
    <skos:altLabel xml:lang="de">Tierhygiene</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13374-1">
    <skos:altLabel xml:lang="de">Tierheilkunde</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13374-1">
    <skos:altLabel xml:lang="de">Tierarztpraxis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13374-1">
    <skos:altLabel xml:lang="de">Tierärztinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13374-1">
    <skos:altLabel xml:lang="de">Tierärzte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13374-1">
    <skos:altLabel xml:lang="de">Tierarzt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13374-1">
    <skos:altLabel xml:lang="de">Fleischbeschau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10272-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18620-1">
    <skos:altLabel xml:lang="de">Logistikdienstleistung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11266-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17570-2">
    <skos:related rdf:resource="descriptor/17569-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19510-3">
    <skos:altLabel xml:lang="de">Soziale Sachleistung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19510-3">
    <skos:altLabel xml:lang="de">Sachleistungstransfer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19510-3">
    <skos:altLabel xml:lang="de">Sachleistung in der Sozialhilfe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19510-3">
    <skos:altLabel xml:lang="de">Naturalleistung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17999-4">
    <skos:broader rdf:resource="thsys/70307"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10236-0">
    <skos:prefLabel xml:lang="en">Monetary theory</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10236-0">
    <skos:prefLabel xml:lang="de">Geldtheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18204-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12818-3">
    <skos:broader rdf:resource="thsys/73384"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12818-3">
    <skos:broader rdf:resource="thsys/70874"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12818-3">
    <skos:broader rdf:resource="thsys/70384"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12818-3">
    <skos:broader rdf:resource="descriptor/12770-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12309-3">
    <skos:broader rdf:resource="thsys/70037"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12309-3">
    <skos:broader rdf:resource="descriptor/12215-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16245-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10513-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16122-6">
    <skos:prefLabel xml:lang="en">Customs law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16122-6">
    <skos:prefLabel xml:lang="de">Zollrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16324-1">
    <skos:related rdf:resource="descriptor/13421-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16324-1">
    <skos:related rdf:resource="descriptor/18808-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16324-1">
    <skos:related rdf:resource="descriptor/13414-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16324-1">
    <skos:related rdf:resource="descriptor/18809-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16324-1">
    <skos:related rdf:resource="descriptor/16551-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16324-1">
    <skos:related rdf:resource="descriptor/16314-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12689-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15119-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10313-1">
    <skos:narrower rdf:resource="descriptor/10122-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10313-1">
    <skos:narrower rdf:resource="descriptor/18328-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10313-1">
    <skos:narrower rdf:resource="descriptor/10304-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13460-1">
    <skos:broader rdf:resource="thsys/70963"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13460-1">
    <skos:broader rdf:resource="descriptor/10032-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14127-6">
    <skos:prefLabel xml:lang="en">Hides and skins</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14127-6">
    <skos:prefLabel xml:lang="de">Häute und Felle</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18881-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13028-6">
    <skos:broader rdf:resource="thsys/71003"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13028-6">
    <skos:broader rdf:resource="descriptor/13034-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11560-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id235">
    <rdfs:seeAlso rdf:resource="descriptor/19553-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13342-0">
    <skos:prefLabel xml:lang="en">Driving school</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13342-0">
    <skos:prefLabel xml:lang="de">Fahrschule</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16421-3">
    <skos:narrower rdf:resource="descriptor/19719-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16421-3">
    <skos:narrower rdf:resource="descriptor/19718-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16421-3">
    <skos:narrower rdf:resource="descriptor/18181-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16079-2">
    <skos:altLabel xml:lang="de">Wechselgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11298-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13699-1">
    <skos:prefLabel xml:lang="en">Equity mutual funds</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13699-1">
    <skos:prefLabel xml:lang="de">Aktienfonds</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13047-2">
    <skos:prefLabel xml:lang="en">Vehicle construction</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13047-2">
    <skos:prefLabel xml:lang="de">Fahrzeugbau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70342">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">P.20.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14393-6">
    <skos:altLabel xml:lang="de">Schleifmaschine</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14393-6">
    <skos:altLabel xml:lang="de">Presse (Werkzeugmaschine)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13723-2">
    <skos:related rdf:resource="descriptor/12210-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13723-2">
    <skos:related rdf:resource="descriptor/13731-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13723-2">
    <skos:related rdf:resource="descriptor/19321-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13723-2">
    <skos:related rdf:resource="descriptor/10121-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13723-2">
    <skos:related rdf:resource="descriptor/12287-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13723-2">
    <skos:related rdf:resource="descriptor/18671-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13723-2">
    <skos:related rdf:resource="descriptor/11414-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13723-2">
    <skos:related rdf:resource="descriptor/25742-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13723-2">
    <skos:related rdf:resource="descriptor/18085-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19708-3">
    <skos:related rdf:resource="descriptor/16440-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19708-3">
    <skos:related rdf:resource="descriptor/16278-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19708-3">
    <skos:related rdf:resource="descriptor/19715-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19708-3">
    <skos:related rdf:resource="descriptor/16184-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19708-3">
    <skos:related rdf:resource="descriptor/16455-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19708-3">
    <skos:related rdf:resource="descriptor/16439-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19708-3">
    <skos:related rdf:resource="descriptor/19711-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19708-3">
    <skos:related rdf:resource="descriptor/20826-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19708-3">
    <skos:related rdf:resource="descriptor/16497-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12747-0">
    <skos:related rdf:resource="descriptor/12644-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12747-0">
    <skos:related rdf:resource="descriptor/12755-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12747-0">
    <skos:related rdf:resource="descriptor/12760-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12747-0">
    <skos:related rdf:resource="descriptor/12765-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12747-0">
    <skos:related rdf:resource="descriptor/10002-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12747-0">
    <skos:related rdf:resource="descriptor/12134-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16978-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11802-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15746-1">
    <skos:narrower rdf:resource="descriptor/19627-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15746-1">
    <skos:narrower rdf:resource="descriptor/18967-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15746-1">
    <skos:narrower rdf:resource="descriptor/19629-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15746-1">
    <skos:narrower rdf:resource="descriptor/19628-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15746-1">
    <skos:narrower rdf:resource="descriptor/19625-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15746-1">
    <skos:narrower rdf:resource="descriptor/18336-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16973-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18907-2">
    <skos:altLabel xml:lang="en">Long term care</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18907-2">
    <skos:altLabel xml:lang="de">Krankenpflegedienst</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18907-2">
    <skos:altLabel xml:lang="de">Hauspflege</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18907-2">
    <skos:altLabel xml:lang="de">Häuslicher Pflegedienst</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18907-2">
    <skos:altLabel xml:lang="de">Ambulanter Pflegedienst</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18907-2">
    <skos:altLabel xml:lang="de">Ambulante Pflege</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18565-4">
    <skos:altLabel xml:lang="de">Speisewagenwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18565-4">
    <skos:altLabel xml:lang="de">Raststätte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70892">
    <rdfs:label xml:lang="de">Versicherungswesen</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15448-2">
    <skos:altLabel xml:lang="en">Strategic equilibrium</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15448-2">
    <skos:altLabel xml:lang="en">Nash solution</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15448-2">
    <skos:altLabel xml:lang="en">Cournot-Nash-equilibrium</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15448-2">
    <skos:altLabel xml:lang="en">Cournot-Nash equilibrium</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15448-2">
    <skos:altLabel xml:lang="de">Strategisches Gleichgewicht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15448-2">
    <skos:altLabel xml:lang="de">Nash-Verhandlungslösung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15448-2">
    <skos:altLabel xml:lang="de">Nash-Lösung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15448-2">
    <skos:altLabel xml:lang="de">Nash Gleichgewicht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15448-2">
    <skos:altLabel xml:lang="de">Cournot-Nash-Gleichgewicht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17176-6">
    <skos:altLabel xml:lang="en">Socialist Federal Republic of Yugoslavia</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17176-6">
    <skos:altLabel xml:lang="de">Sozialistische Föderative Republik Jugoslawien</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17176-6">
    <skos:altLabel xml:lang="de">Jugoslavija (bis 1990)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11558-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13533-0">
    <skos:altLabel xml:lang="de">Seehafen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10016-0">
    <skos:broader rdf:resource="thsys/70550"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10016-0">
    <skos:broader rdf:resource="thsys/70233"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10016-0">
    <skos:broader rdf:resource="thsys/70201"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10016-0">
    <skos:broader rdf:resource="thsys/70030"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73348">
    <skos:broader rdf:resource="thsys/70151"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14888-0">
    <skos:altLabel xml:lang="de">Webtextilien</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14888-0">
    <skos:altLabel xml:lang="de">Webereiprodukt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14888-0">
    <skos:altLabel xml:lang="de">Velours</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14888-0">
    <skos:altLabel xml:lang="de">Tuch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14888-0">
    <skos:altLabel xml:lang="de">Samt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14888-0">
    <skos:altLabel xml:lang="de">Plüsch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14888-0">
    <skos:altLabel xml:lang="de">Kunstleder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14888-0">
    <skos:altLabel xml:lang="de">Gummiertes Gewebe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14888-0">
    <skos:altLabel xml:lang="de">Gewebe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14888-0">
    <skos:altLabel xml:lang="de">Beschichtetes Gewebe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14888-0">
    <skos:altLabel xml:lang="de">Beschichtete Textilien</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13820-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11556-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12247-6">
    <skos:altLabel xml:lang="de">Industrieschuldverschreibung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12247-6">
    <skos:altLabel xml:lang="de">Industrieanleihe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12247-6">
    <skos:altLabel xml:lang="de">Anleihe der Industrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18896-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13312-2">
    <skos:narrower rdf:resource="descriptor/13310-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19447-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#historyNote">
    <rdf:type rdf:resource="http://www.w3.org/1999/02/22-rdf-syntax-ns#Property"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#historyNote">
    <rdf:type rdf:resource="http://www.w3.org/2002/07/owl#AnnotationProperty"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12540-1">
    <skos:broader rdf:resource="thsys/70855"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12540-1">
    <skos:broader rdf:resource="thsys/70578"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12540-1">
    <skos:broader rdf:resource="descriptor/10011-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13085-1">
    <skos:related rdf:resource="descriptor/19858-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13085-1">
    <skos:related rdf:resource="descriptor/14548-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18231-5">
    <skos:related rdf:resource="descriptor/14364-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18231-5">
    <skos:related rdf:resource="descriptor/15901-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14948-1">
    <skos:broader rdf:resource="thsys/70342"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14948-1">
    <skos:broader rdf:resource="descriptor/14931-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16438-0">
    <skos:related rdf:resource="descriptor/10626-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16438-0">
    <skos:related rdf:resource="descriptor/20449-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16438-0">
    <skos:related rdf:resource="descriptor/10618-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16438-0">
    <skos:related rdf:resource="descriptor/19509-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18218-4">
    <skos:altLabel xml:lang="en">Chicken meat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18218-4">
    <skos:altLabel xml:lang="de">Hühnerfleisch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14664-1">
    <skos:prefLabel xml:lang="en">Alkanes</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14664-1">
    <skos:prefLabel xml:lang="de">Alkane</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13434-2">
    <skos:broader rdf:resource="thsys/71028"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13434-2">
    <skos:broader rdf:resource="descriptor/18980-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10850-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12697-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10757-0">
    <skos:prefLabel xml:lang="en">Swiss Franc</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10757-0">
    <skos:prefLabel xml:lang="de">Schweizer Franken</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11970-2">
    <zbwext:useInsteadNote rdf:nodeID="id304"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73393">
    <rdfs:label xml:lang="en">Philosophy of science</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73393">
    <rdfs:label xml:lang="de">Wissenschaftstheorie</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12309-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70301">
    <skos:prefLabel xml:lang="de">B.05  Materialwirtschaft und Logistik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13160-6">
    <skos:related rdf:resource="descriptor/18991-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13160-6">
    <skos:related rdf:resource="descriptor/18413-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13160-6">
    <skos:related rdf:resource="descriptor/14384-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13160-6">
    <skos:related rdf:resource="descriptor/19338-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13160-6">
    <skos:related rdf:resource="descriptor/14931-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13160-6">
    <skos:related rdf:resource="descriptor/18443-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13160-6">
    <skos:related rdf:resource="descriptor/16004-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13160-6">
    <skos:related rdf:resource="descriptor/13219-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13160-6">
    <skos:related rdf:resource="descriptor/18442-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13160-6">
    <skos:related rdf:resource="descriptor/18222-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13160-6">
    <skos:related rdf:resource="descriptor/13306-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13160-6">
    <skos:related rdf:resource="descriptor/13949-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13160-6">
    <skos:related rdf:resource="descriptor/18320-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19668-3">
    <skos:broader rdf:resource="thsys/73366"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19668-3">
    <skos:broader rdf:resource="thsys/70039"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11752-5">
    <skos:altLabel xml:lang="de">Verteilungskampf</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14909-4">
    <skos:prefLabel xml:lang="en">Children's wear</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14909-4">
    <skos:prefLabel xml:lang="de">Kinderbekleidung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12385-1">
    <skos:related rdf:resource="descriptor/12401-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12385-1">
    <skos:related rdf:resource="descriptor/12297-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12385-1">
    <skos:related rdf:resource="descriptor/12273-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12385-1">
    <skos:related rdf:resource="descriptor/12361-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12385-1">
    <skos:related rdf:resource="descriptor/12406-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13109-6">
    <skos:prefLabel xml:lang="en">Carpentry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13109-6">
    <skos:prefLabel xml:lang="de">Holzhandwerk</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16664-5">
    <skos:altLabel xml:lang="en">Social position</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16664-5">
    <skos:altLabel xml:lang="de">Sozioökonomischer Status</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16664-5">
    <skos:altLabel xml:lang="de">Soziale Stellung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16664-5">
    <skos:altLabel xml:lang="de">Soziale Position</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id232">
    <rdf:value xml:lang="de">Für die Analyse chaotischer Systeme</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18422-5">
    <skos:broader rdf:resource="thsys/71056"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18422-5">
    <skos:broader rdf:resource="thsys/70997"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18422-5">
    <skos:broader rdf:resource="descriptor/18400-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14762-1">
    <skos:broader rdf:resource="thsys/73305"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14762-1">
    <skos:broader rdf:resource="thsys/71038"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14762-1">
    <skos:broader rdf:resource="descriptor/14751-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16312-1">
    <skos:prefLabel xml:lang="en">Centralized state</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16312-1">
    <skos:prefLabel xml:lang="de">Zentralstaat</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71020">
    <skos:broader rdf:resource="thsys/71028"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11430-6">
    <skos:related rdf:resource="descriptor/11438-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11430-6">
    <skos:related rdf:resource="descriptor/14298-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11430-6">
    <skos:related rdf:resource="descriptor/11444-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11430-6">
    <skos:related rdf:resource="descriptor/19260-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11430-6">
    <skos:related rdf:resource="descriptor/11443-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19478-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18187-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13995-4">
    <skos:narrower rdf:resource="descriptor/16301-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12700-5">
    <skos:prefLabel xml:lang="en">Capacity utilization</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12700-5">
    <skos:prefLabel xml:lang="de">Kapazitätsauslastung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19166-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15488-4">
    <skos:prefLabel xml:lang="en">Branch-and-Bound algorithm</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15488-4">
    <skos:prefLabel xml:lang="de">Branch-and-Bound-Verfahren</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19666-0">
    <skos:broader rdf:resource="thsys/70234"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12591-5">
    <skos:altLabel xml:lang="de">Unterstützungskasse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12591-5">
    <skos:altLabel xml:lang="de">Recht der betrieblichen Altersversorgung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12591-5">
    <skos:altLabel xml:lang="de">Betriebsrentenrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12591-5">
    <skos:altLabel xml:lang="de">Betriebsrente</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12591-5">
    <skos:altLabel xml:lang="de">Betriebliche Alterssicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16441-4">
    <skos:narrower rdf:resource="descriptor/10597-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16441-4">
    <skos:narrower rdf:resource="descriptor/10623-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16441-4">
    <skos:narrower rdf:resource="descriptor/10587-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16441-4">
    <skos:narrower rdf:resource="descriptor/16648-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16441-4">
    <skos:narrower rdf:resource="descriptor/19708-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16441-4">
    <skos:narrower rdf:resource="descriptor/10617-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16441-4">
    <skos:narrower rdf:resource="descriptor/19711-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71101">
    <skos:narrower rdf:resource="descriptor/11033-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71101">
    <skos:narrower rdf:resource="descriptor/11146-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71101">
    <skos:narrower rdf:resource="descriptor/11030-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71101">
    <skos:narrower rdf:resource="descriptor/10985-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71101">
    <skos:narrower rdf:resource="descriptor/11125-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71101">
    <skos:narrower rdf:resource="descriptor/11124-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71101">
    <skos:narrower rdf:resource="descriptor/11142-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71101">
    <skos:narrower rdf:resource="descriptor/11149-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71101">
    <skos:narrower rdf:resource="descriptor/11090-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71101">
    <skos:narrower rdf:resource="descriptor/11143-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71101">
    <skos:narrower rdf:resource="descriptor/11141-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71101">
    <skos:narrower rdf:resource="descriptor/11024-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71101">
    <skos:narrower rdf:resource="descriptor/11151-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71101">
    <skos:narrower rdf:resource="descriptor/11060-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71101">
    <skos:narrower rdf:resource="descriptor/11034-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71101">
    <skos:narrower rdf:resource="descriptor/11061-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71101">
    <skos:narrower rdf:resource="descriptor/11020-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71101">
    <skos:narrower rdf:resource="descriptor/11073-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71101">
    <skos:narrower rdf:resource="descriptor/11088-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71101">
    <skos:narrower rdf:resource="descriptor/11048-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71101">
    <skos:narrower rdf:resource="descriptor/11051-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71101">
    <skos:narrower rdf:resource="descriptor/11047-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71101">
    <skos:narrower rdf:resource="descriptor/11134-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71101">
    <skos:narrower rdf:resource="descriptor/11044-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71101">
    <skos:narrower rdf:resource="descriptor/11001-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71101">
    <skos:narrower rdf:resource="descriptor/11129-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71101">
    <skos:narrower rdf:resource="descriptor/10996-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71101">
    <skos:narrower rdf:resource="descriptor/11147-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19433-2">
    <skos:related rdf:resource="descriptor/19533-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70394">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19491-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70071">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11685-1">
    <skos:related rdf:resource="descriptor/10112-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11685-1">
    <skos:related rdf:resource="descriptor/11688-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13010-4">
    <skos:prefLabel xml:lang="en">Fine ceramics industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13010-4">
    <skos:prefLabel xml:lang="de">Feinkeramische Industrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18681-2">
    <skos:narrower rdf:resource="descriptor/12292-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11834-3">
    <skos:prefLabel xml:lang="en">Transportation rate</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11834-3">
    <skos:prefLabel xml:lang="de">Verkehrstarif</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12069-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72182">
    <skos:narrower rdf:resource="descriptor/15025-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72182">
    <skos:narrower rdf:resource="descriptor/15211-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72182">
    <skos:narrower rdf:resource="descriptor/19581-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72182">
    <skos:narrower rdf:resource="descriptor/15064-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72182">
    <skos:narrower rdf:resource="descriptor/15814-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72182">
    <skos:narrower rdf:resource="descriptor/15063-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72182">
    <skos:narrower rdf:resource="descriptor/15062-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72182">
    <skos:narrower rdf:resource="descriptor/19568-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72182">
    <skos:narrower rdf:resource="descriptor/19150-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72182">
    <skos:narrower rdf:resource="descriptor/15372-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72182">
    <skos:narrower rdf:resource="descriptor/19070-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72182">
    <skos:narrower rdf:resource="descriptor/19569-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72182">
    <skos:narrower rdf:resource="descriptor/10229-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://zbw.eu/namespaces/skos-extensions/Descriptor">
    <rdfs:label xml:lang="en">Descriptor</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14456-1">
    <skos:prefLabel xml:lang="en">Ship's equipment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14456-1">
    <skos:prefLabel xml:lang="de">Schiffsausrüstung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11740-5">
    <skos:broader rdf:resource="thsys/71115"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11740-5">
    <skos:broader rdf:resource="descriptor/10006-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16639-4">
    <skos:altLabel xml:lang="de">Lebenshaltung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18067-6">
    <skos:related rdf:resource="descriptor/19733-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18067-6">
    <skos:related rdf:resource="descriptor/13810-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13145-2">
    <skos:broader rdf:resource="thsys/70987"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13145-2">
    <skos:broader rdf:resource="descriptor/18094-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15956-4">
    <skos:prefLabel xml:lang="en">Return migration</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15956-4">
    <skos:prefLabel xml:lang="de">Rückwanderung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13475-2">
    <skos:related rdf:resource="descriptor/19139-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13475-2">
    <skos:related rdf:resource="descriptor/12820-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13475-2">
    <skos:related rdf:resource="descriptor/13479-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13475-2">
    <skos:related rdf:resource="descriptor/13466-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13475-2">
    <skos:related rdf:resource="descriptor/13200-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16943-1">
    <skos:broader rdf:resource="thsys/70438"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16943-1">
    <skos:broader rdf:resource="descriptor/16950-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19104-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10748-1">
    <skos:prefLabel xml:lang="en">Yen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10748-1">
    <skos:prefLabel xml:lang="de">Yen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15264-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14610-3">
    <skos:prefLabel xml:lang="en">Plastics additive</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14610-3">
    <skos:prefLabel xml:lang="de">Kunststoffhilfsmittel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18073-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12564-1">
    <skos:scopeNote xml:lang="de">Einsatz von EDV im Personalwesen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19292-1">
    <skos:broader rdf:resource="thsys/71029"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19292-1">
    <skos:broader rdf:resource="thsys/70986"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19292-1">
    <skos:broader rdf:resource="thsys/70960"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19292-1">
    <skos:broader rdf:resource="descriptor/19338-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10421-5">
    <skos:broader rdf:resource="thsys/70406"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10421-5">
    <skos:broader rdf:resource="descriptor/10434-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70554">
    <rdfs:label xml:lang="en">Central Europe</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70554">
    <rdfs:label xml:lang="de">Mitteleuropa</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14933-0">
    <skos:altLabel xml:lang="de">Gefrierkost</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18683-5">
    <skos:scopeNote xml:lang="de">Computergestützte Hedging-Strategien, mit Hilfe von mathematischen Programmen große Aktienportfolios gegen Kursverluste zu schützen, was durch Bildung sog. idealer Portfolios erreicht werden soll.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16976-0">
    <skos:prefLabel xml:lang="en">Ticino</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16976-0">
    <skos:prefLabel xml:lang="de">Tessin</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15491-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13989-6">
    <skos:narrower rdf:resource="descriptor/13984-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73394">
    <rdfs:label xml:lang="en">Philosophy</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73394">
    <rdfs:label xml:lang="de">Philosophie</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17580-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19576-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18234-6">
    <skos:altLabel xml:lang="de">Industrielle Abwärme</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18234-6">
    <skos:altLabel xml:lang="de">Abwärme</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12612-2">
    <skos:related rdf:resource="descriptor/15416-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12612-2">
    <skos:related rdf:resource="descriptor/12615-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12612-2">
    <skos:related rdf:resource="descriptor/12617-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12612-2">
    <skos:related rdf:resource="descriptor/12607-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15932-4">
    <skos:prefLabel xml:lang="en">Ethnic discrimination</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15932-4">
    <skos:prefLabel xml:lang="de">Ethnische Diskriminierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18338-1">
    <skos:prefLabel xml:lang="en">Forest policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18338-1">
    <skos:prefLabel xml:lang="de">Forstpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18094-3">
    <skos:prefLabel xml:lang="en">Textile and clothing industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18094-3">
    <skos:prefLabel xml:lang="de">Textil- und Bekleidungsindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18531-0">
    <skos:broader rdf:resource="thsys/70956"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18531-0">
    <skos:broader rdf:resource="descriptor/18717-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15989-3">
    <skos:broader rdf:resource="thsys/73360"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15989-3">
    <skos:broader rdf:resource="descriptor/16019-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18392-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12927-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11701-1">
    <skos:altLabel xml:lang="de">Zonenrandförderungsgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11701-1">
    <skos:altLabel xml:lang="de">Zonenrandförderung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11701-1">
    <skos:altLabel xml:lang="de">Steuerliche Standortpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11701-1">
    <skos:altLabel xml:lang="de">Regionales Steuergefälle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11701-1">
    <skos:altLabel xml:lang="de">Regionale Steuerdifferenzierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11701-1">
    <skos:altLabel xml:lang="de">Gesetz zur Förderung des Zonenrandgebietes</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11701-1">
    <skos:altLabel xml:lang="de">Gesetz zur Förderung der Berliner Wirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11701-1">
    <skos:altLabel xml:lang="de">Fördergebietsgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11701-1">
    <skos:altLabel xml:lang="de">Berlinförderungsgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11701-1">
    <skos:altLabel xml:lang="de">Berlinförderung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12502-2">
    <skos:broader rdf:resource="thsys/70245"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12502-2">
    <skos:broader rdf:resource="descriptor/12501-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12502-2">
    <skos:broader rdf:resource="descriptor/19566-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10508-0">
    <skos:altLabel xml:lang="en">SIC (Semi-Industrialized Countries)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10508-0">
    <skos:altLabel xml:lang="en">Semi-industrialized countries</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10508-0">
    <skos:altLabel xml:lang="en">NIC (Newly Industrializing Countries)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10508-0">
    <skos:altLabel xml:lang="de">Halbindustrialisierte Länder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18707-3">
    <skos:narrower rdf:resource="descriptor/18708-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18707-3">
    <skos:narrower rdf:resource="descriptor/13687-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13253-6">
    <skos:broader rdf:resource="thsys/70924"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13253-6">
    <skos:broader rdf:resource="descriptor/12927-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19376-2">
    <skos:broader rdf:resource="thsys/71083"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13479-1">
    <skos:broader rdf:resource="thsys/70956"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13940-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18648-0">
    <skos:broader rdf:resource="thsys/70922"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18648-0">
    <skos:broader rdf:resource="thsys/70900"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18648-0">
    <skos:broader rdf:resource="descriptor/18718-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19546-3">
    <skos:broader rdf:resource="thsys/73362"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19546-3">
    <skos:broader rdf:resource="thsys/71020"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19546-3">
    <skos:broader rdf:resource="descriptor/19545-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19546-3">
    <skos:broader rdf:resource="descriptor/11806-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19276-6">
    <skos:prefLabel xml:lang="en">Certificate of deposit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19276-6">
    <skos:prefLabel xml:lang="de">Depositenzertifikat</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16155-5">
    <skos:broader rdf:resource="thsys/73335"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16155-5">
    <skos:broader rdf:resource="thsys/70835"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16155-5">
    <skos:broader rdf:resource="thsys/70833"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16155-5">
    <skos:broader rdf:resource="thsys/70324"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16155-5">
    <skos:broader rdf:resource="descriptor/16159-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19686-1">
    <skos:scopeNote xml:lang="de">Das Verhältnis zwischen Staat und gesellschaftlichen Gruppen ist bestimmt durch Laissez-Faire oder staatliches Eingreifen in direkter oder indirekter Weise. Die Palette der Möglichkeiten reicht von staatlicher Öffentlichkeitsarbeit über finanzielle Anreize bis hin zu rechtlichen Ge- oder Verboten.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17940-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12794-5">
    <skos:related rdf:resource="descriptor/19127-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12794-5">
    <skos:related rdf:resource="descriptor/10978-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12794-5">
    <skos:related rdf:resource="descriptor/12808-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12794-5">
    <skos:related rdf:resource="descriptor/10909-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12794-5">
    <skos:related rdf:resource="descriptor/10216-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12794-5">
    <skos:related rdf:resource="descriptor/10213-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12794-5">
    <skos:related rdf:resource="descriptor/12531-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12794-5">
    <skos:related rdf:resource="descriptor/10198-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15658-5">
    <skos:related rdf:resource="descriptor/19529-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15658-5">
    <skos:related rdf:resource="descriptor/15790-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15658-5">
    <skos:related rdf:resource="descriptor/19785-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15658-5">
    <skos:related rdf:resource="descriptor/15664-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15658-5">
    <skos:related rdf:resource="descriptor/14042-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15658-5">
    <skos:related rdf:resource="descriptor/16670-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15658-5">
    <skos:related rdf:resource="descriptor/13371-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15658-5">
    <skos:related rdf:resource="descriptor/18892-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15658-5">
    <skos:related rdf:resource="descriptor/16635-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15658-5">
    <skos:related rdf:resource="descriptor/18777-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15658-5">
    <skos:related rdf:resource="descriptor/19811-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15658-5">
    <skos:related rdf:resource="descriptor/15633-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15658-5">
    <skos:related rdf:resource="descriptor/16697-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11046-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13310-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15787-1">
    <skos:narrower rdf:resource="descriptor/15823-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11999-6">
    <skos:related rdf:resource="descriptor/11983-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11999-6">
    <skos:related rdf:resource="descriptor/11969-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11999-6">
    <skos:related rdf:resource="descriptor/11998-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11999-6">
    <skos:related rdf:resource="descriptor/11890-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11999-6">
    <skos:related rdf:resource="descriptor/11901-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12741-5">
    <skos:broader rdf:resource="thsys/70270"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12741-5">
    <skos:broader rdf:resource="descriptor/12744-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70361">
    <skos:prefLabel xml:lang="en">G.01.01.01  Germany</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70361">
    <skos:prefLabel xml:lang="de">G.01.01.01  Deutschland</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13671-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10282-0">
    <skos:prefLabel xml:lang="en">Income hypothesis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10282-0">
    <skos:prefLabel xml:lang="de">Einkommenshypothese</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15460-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14741-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10432-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12602-5">
    <skos:broader rdf:resource="thsys/71050"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12602-5">
    <skos:broader rdf:resource="descriptor/15685-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19118-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15446-6">
    <skos:narrower rdf:resource="descriptor/19595-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18426-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11194-6">
    <skos:altLabel xml:lang="en">Leisure time</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11194-6">
    <skos:altLabel xml:lang="de">Erholung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11194-6">
    <skos:altLabel xml:lang="de">Eigenzeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10278-5">
    <zbwext:useInsteadNote rdf:nodeID="id114"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19285-5">
    <skos:scopeNote xml:lang="de">Instabilität von Volkswirtschaften, Märkten oder Wirtschaftsordnungen sowie systembedingte Knappheiten oder Mangelerscheinungen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13514-4">
    <skos:related rdf:resource="descriptor/13590-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13514-4">
    <skos:related rdf:resource="descriptor/18630-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11753-3">
    <skos:scopeNote xml:lang="de">Nur zur Messung von Verteilung und Konzentration des Einkommens bzw. des Vermögens.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19858-5">
    <skos:altLabel xml:lang="en">Gun-control law</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19858-5">
    <skos:altLabel xml:lang="de">WaffG</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19858-5">
    <skos:altLabel xml:lang="de">Waffenschein</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19858-5">
    <skos:altLabel xml:lang="de">Waffengesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19858-5">
    <skos:altLabel xml:lang="de">Waffenbesitzkarte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18719-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70037">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18760-6">
    <skos:altLabel xml:lang="de">Leasinggeschäft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19476-5">
    <skos:prefLabel xml:lang="en">City portrait</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19476-5">
    <skos:prefLabel xml:lang="de">Stadtbeschreibung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10249-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13588-3">
    <skos:related rdf:resource="descriptor/11833-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13588-3">
    <skos:related rdf:resource="descriptor/13264-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13588-3">
    <skos:related rdf:resource="descriptor/13594-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13588-3">
    <skos:related rdf:resource="descriptor/15520-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13588-3">
    <skos:related rdf:resource="descriptor/15519-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13588-3">
    <skos:related rdf:resource="descriptor/15518-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15715-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14087-6">
    <skos:prefLabel xml:lang="en">Barley</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14087-6">
    <skos:prefLabel xml:lang="de">Gerste</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18928-1">
    <skos:broader rdf:resource="thsys/70825"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18928-1">
    <skos:broader rdf:resource="descriptor/10032-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11702-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18703-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20442-5">
    <skos:scopeNote xml:lang="de">Effiziente Reaktion der Anbieter auf die Konsumentennachfrage durch Zusammenarbeit zwischen allen Beteiligten der logistischen Kette</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14683-4">
    <skos:broader rdf:resource="thsys/70556"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14683-4">
    <skos:broader rdf:resource="descriptor/14656-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19508-4">
    <skos:prefLabel xml:lang="en">Workfare</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19508-4">
    <skos:prefLabel xml:lang="de">Workfare</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14920-2">
    <skos:broader rdf:resource="thsys/70031"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14920-2">
    <skos:broader rdf:resource="descriptor/14586-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14920-2">
    <skos:broader rdf:resource="descriptor/14915-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11241-2">
    <skos:broader rdf:resource="thsys/71080"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11241-2">
    <skos:broader rdf:resource="thsys/71061"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11241-2">
    <skos:broader rdf:resource="descriptor/11572-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11241-2">
    <skos:broader rdf:resource="descriptor/11235-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73395">
    <rdfs:label xml:lang="de">Soziale Schichtung</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13493-0">
    <skos:broader rdf:resource="thsys/70947"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13493-0">
    <skos:broader rdf:resource="descriptor/13200-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18727-4">
    <skos:broader rdf:resource="thsys/71107"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18727-4">
    <skos:broader rdf:resource="thsys/70899"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18727-4">
    <skos:broader rdf:resource="descriptor/13769-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13177-3">
    <skos:narrower rdf:resource="descriptor/13179-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13177-3">
    <skos:narrower rdf:resource="descriptor/18500-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18992-6">
    <skos:broader rdf:resource="thsys/73295"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18992-6">
    <skos:broader rdf:resource="descriptor/10981-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18992-6">
    <skos:broader rdf:resource="descriptor/16476-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11311-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70845">
    <skos:broader rdf:resource="thsys/70847"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16297-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16483-2">
    <skos:broader rdf:resource="thsys/73369"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16483-2">
    <skos:broader rdf:resource="thsys/70819"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16483-2">
    <skos:broader rdf:resource="descriptor/16273-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18759-5">
    <skos:related rdf:resource="descriptor/18760-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18759-5">
    <skos:related rdf:resource="descriptor/12254-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10427-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14106-0">
    <skos:prefLabel xml:lang="en">Leguminosae</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14106-0">
    <skos:prefLabel xml:lang="de">Hülsenfrucht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16122-6">
    <skos:broader rdf:resource="thsys/73337"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16122-6">
    <skos:broader rdf:resource="thsys/71096"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16122-6">
    <skos:broader rdf:resource="thsys/71073"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16122-6">
    <skos:broader rdf:resource="descriptor/11525-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18795-1">
    <skos:broader rdf:resource="thsys/70873"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18795-1">
    <skos:broader rdf:resource="thsys/70459"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70996">
    <skos:prefLabel xml:lang="en">W.06.01.04  Electrical industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70996">
    <skos:prefLabel xml:lang="de">W.06.01.04  Elektroindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15963-0">
    <skos:related rdf:resource="descriptor/16198-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15963-0">
    <skos:related rdf:resource="descriptor/16504-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15963-0">
    <skos:related rdf:resource="descriptor/18011-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15686-0">
    <skos:prefLabel xml:lang="en">Science studies</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15686-0">
    <skos:prefLabel xml:lang="de">Wissenschaftsforschung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16968-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11679-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15807-0">
    <skos:related rdf:resource="descriptor/15810-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15807-0">
    <skos:related rdf:resource="descriptor/15722-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15807-0">
    <skos:related rdf:resource="descriptor/15806-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15807-0">
    <skos:related rdf:resource="descriptor/19758-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16611-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19668-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10383-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15964-5">
    <skos:related rdf:resource="descriptor/19860-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15964-5">
    <skos:related rdf:resource="descriptor/15965-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15964-5">
    <skos:related rdf:resource="descriptor/15659-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15964-5">
    <skos:related rdf:resource="descriptor/16713-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15964-5">
    <skos:related rdf:resource="descriptor/15998-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15964-5">
    <skos:related rdf:resource="descriptor/19768-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15964-5">
    <skos:related rdf:resource="descriptor/16426-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11516-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19609-5">
    <skos:scopeNote xml:lang="de">Nichtparametrisches Verfahren zur Bestimmung der relativen Effizienz von Unternehmen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19729-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70865">
    <rdfs:label xml:lang="en">Armed forces</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70865">
    <rdfs:label xml:lang="de">Militär</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13338-5">
    <skos:broader rdf:resource="thsys/70876"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18551-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19494-3">
    <skos:related rdf:resource="descriptor/11278-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19494-3">
    <skos:related rdf:resource="descriptor/11183-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19494-3">
    <skos:related rdf:resource="descriptor/11182-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10659-0">
    <skos:broader rdf:resource="thsys/71099"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10659-0">
    <skos:broader rdf:resource="descriptor/10678-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10659-0">
    <skos:broader rdf:resource="descriptor/10773-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13264-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16090-0">
    <skos:altLabel xml:lang="de">EU-Steuerrichtlinie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16090-0">
    <skos:altLabel xml:lang="de">EU-Steuerharmonisierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16090-0">
    <skos:altLabel xml:lang="de">EG-Steuerrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16090-0">
    <skos:altLabel xml:lang="de">EG-Steuerharmonisierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18441-1">
    <skos:altLabel xml:lang="de">Lebensmittelrecht der EU</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18441-1">
    <skos:altLabel xml:lang="de">EU-Bedarfsgegenständerecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70903">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18633-6">
    <skos:broader rdf:resource="thsys/71064"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18633-6">
    <skos:broader rdf:resource="thsys/70925"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18633-6">
    <skos:broader rdf:resource="descriptor/11833-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18633-6">
    <skos:broader rdf:resource="descriptor/16549-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18794-3">
    <skos:broader rdf:resource="thsys/70951"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18794-3">
    <skos:broader rdf:resource="thsys/70873"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18794-3">
    <skos:broader rdf:resource="descriptor/12783-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13588-3">
    <skos:broader rdf:resource="thsys/70928"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13588-3">
    <skos:broader rdf:resource="descriptor/10032-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13588-3">
    <skos:broader rdf:resource="descriptor/13573-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19348-0">
    <skos:related rdf:resource="descriptor/12958-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19348-0">
    <skos:related rdf:resource="descriptor/14985-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13003-1">
    <skos:scopeNote xml:lang="de">Umfasst im wesentlichen die Stufen des nuklearen Brennstoffkreislaufs: Förderung und Aufbereitung von Uranerz, Anreicherung und Brennelementfertigung sowie Wiederaufarbeitung und Lagerung gebrauchter Brennstäbe.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13026-3">
    <skos:prefLabel xml:lang="en">Precious metal industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13026-3">
    <skos:prefLabel xml:lang="de">Edelmetallindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18087-0">
    <skos:related rdf:resource="descriptor/18693-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18087-0">
    <skos:related rdf:resource="descriptor/10135-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18087-0">
    <skos:related rdf:resource="descriptor/10803-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12920-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73333">
    <skos:prefLabel xml:lang="en">N.05.08  International law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73333">
    <skos:prefLabel xml:lang="de">N.05.08  Internationales Recht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12184-4">
    <skos:prefLabel xml:lang="en">Limited liability company with a dormant partner</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12184-4">
    <skos:prefLabel xml:lang="de">GmbH &amp;amp; Still</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19360-3">
    <skos:altLabel xml:lang="de">Migrantenüberweisung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19360-3">
    <skos:altLabel xml:lang="de">Heimatüberweisung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19360-3">
    <skos:altLabel xml:lang="de">Heimattransfer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19360-3">
    <skos:altLabel xml:lang="de">Gastarbeitertransfers</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19360-3">
    <skos:altLabel xml:lang="de">Gastarbeitertransfer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12501-4">
    <skos:related rdf:resource="descriptor/11341-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12501-4">
    <skos:related rdf:resource="descriptor/11241-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12501-4">
    <skos:related rdf:resource="descriptor/19566-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13713-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17801-2">
    <skos:prefLabel xml:lang="en">St. Vincent and the Grenadines</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17801-2">
    <skos:prefLabel xml:lang="de">St. Vincent und die Grenadinen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15859-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18291-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71053">
    <skos:broader rdf:resource="thsys/71055"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71096">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14122-2">
    <skos:prefLabel xml:lang="en">Meat</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14122-2">
    <skos:prefLabel xml:lang="de">Fleisch</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13349-0">
    <skos:broader rdf:resource="thsys/71052"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13349-0">
    <skos:broader rdf:resource="thsys/70879"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13349-0">
    <skos:broader rdf:resource="descriptor/11356-6"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id324">
    <rdfs:seeAlso rdf:resource="descriptor/14997-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19298-3">
    <skos:prefLabel xml:lang="en">Rice policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19298-3">
    <skos:prefLabel xml:lang="de">Reispolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13600-4">
    <skos:related rdf:resource="descriptor/13513-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13600-4">
    <skos:related rdf:resource="descriptor/13610-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13600-4">
    <skos:related rdf:resource="descriptor/15518-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13600-4">
    <skos:related rdf:resource="descriptor/13546-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13600-4">
    <skos:related rdf:resource="descriptor/12689-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18193-1">
    <skos:prefLabel xml:lang="en">Cocoa production</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18193-1">
    <skos:prefLabel xml:lang="de">Kakaoanbau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14734-6">
    <skos:broader rdf:resource="thsys/70536"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14734-6">
    <skos:broader rdf:resource="descriptor/14599-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71023">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.01.04</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16333-0">
    <skos:scopeNote xml:lang="de">Oberster staatlicher Repräsentant</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19622-6">
    <skos:narrower rdf:resource="descriptor/15901-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12727-6">
    <skos:prefLabel xml:lang="en">Series production</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12727-6">
    <skos:prefLabel xml:lang="de">Serienfertigung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12441-3">
    <skos:related rdf:resource="descriptor/12408-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11143-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18940-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19500-6">
    <skos:prefLabel xml:lang="en">Non-cash employee benefits</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19500-6">
    <skos:prefLabel xml:lang="de">Geldwerter Vorteil</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13573-2">
    <skos:narrower rdf:resource="descriptor/13571-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13573-2">
    <skos:narrower rdf:resource="descriptor/13588-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13573-2">
    <skos:narrower rdf:resource="descriptor/18623-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13573-2">
    <skos:narrower rdf:resource="descriptor/13511-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13573-2">
    <skos:narrower rdf:resource="descriptor/13594-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13573-2">
    <skos:narrower rdf:resource="descriptor/18418-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13573-2">
    <skos:narrower rdf:resource="descriptor/18423-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10064-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15740-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12958-1">
    <skos:broader rdf:resource="thsys/71026"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12958-1">
    <skos:broader rdf:resource="descriptor/12947-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12605-6">
    <skos:altLabel xml:lang="de">Strahlenschutzverordnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12605-6">
    <skos:altLabel xml:lang="de">Strahlenschutzrichtlinie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18062-2">
    <skos:related rdf:resource="descriptor/16389-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18062-2">
    <skos:related rdf:resource="descriptor/19688-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18062-2">
    <skos:related rdf:resource="descriptor/19684-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18062-2">
    <skos:related rdf:resource="descriptor/19705-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18062-2">
    <skos:related rdf:resource="descriptor/16300-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18062-2">
    <skos:related rdf:resource="descriptor/16416-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18062-2">
    <skos:related rdf:resource="descriptor/16264-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18062-2">
    <skos:related rdf:resource="descriptor/16299-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18062-2">
    <skos:related rdf:resource="descriptor/15797-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18062-2">
    <skos:related rdf:resource="descriptor/15918-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71093">
    <skos:prefLabel xml:lang="en">V.07.04  Economic integration</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71093">
    <skos:prefLabel xml:lang="de">V.07.04  Wirtschaftsintegration</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18843-2">
    <skos:broader rdf:resource="thsys/70854"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18843-2">
    <skos:broader rdf:resource="descriptor/12180-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13729-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19203-5">
    <skos:broader rdf:resource="thsys/73393"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19203-5">
    <skos:broader rdf:resource="thsys/70141"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19203-5">
    <skos:broader rdf:resource="descriptor/10032-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19469-2">
    <skos:broader rdf:resource="thsys/71064"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19469-2">
    <skos:broader rdf:resource="descriptor/11973-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11286-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13085-1">
    <skos:prefLabel xml:lang="en">Metal goods industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13085-1">
    <skos:prefLabel xml:lang="de">EBM-Industrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18012-3">
    <skos:broader rdf:resource="thsys/70361"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18012-3">
    <skos:broader rdf:resource="descriptor/17991-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18012-3">
    <skos:broader rdf:resource="descriptor/16825-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18012-3">
    <skos:broader rdf:resource="descriptor/19178-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18012-3">
    <skos:broader rdf:resource="descriptor/19177-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18012-3">
    <skos:broader rdf:resource="descriptor/17983-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11282-2">
    <skos:prefLabel xml:lang="en">Unskilled workers</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11282-2">
    <skos:prefLabel xml:lang="de">Ungelernte Arbeitskräfte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18407-1">
    <skos:related rdf:resource="descriptor/14368-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18407-1">
    <skos:related rdf:resource="descriptor/18390-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18407-1">
    <skos:related rdf:resource="descriptor/12980-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12056-6">
    <skos:related rdf:resource="descriptor/16063-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12056-6">
    <skos:related rdf:resource="descriptor/12452-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12056-6">
    <skos:related rdf:resource="descriptor/12446-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12056-6">
    <skos:related rdf:resource="descriptor/10897-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14255-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id380">
    <rdfs:seeAlso rdf:resource="descriptor/19210-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19126-4">
    <skos:broader rdf:resource="thsys/70040"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19126-4">
    <skos:broader rdf:resource="descriptor/12788-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70539">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11829-3">
    <skos:prefLabel xml:lang="en">Transport investment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11829-3">
    <skos:prefLabel xml:lang="de">Verkehrsinvestition</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19808-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73340">
    <skos:broader rdf:resource="thsys/73342"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13655-0">
    <skos:scopeNote xml:lang="de">Für Veranstalter und Programme BENUTZE die entsprechenden Deskriptoren.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19356-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19685-3">
    <skos:prefLabel xml:lang="en">Capital City</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19685-3">
    <skos:prefLabel xml:lang="de">Hauptstadt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15972-6">
    <skos:altLabel xml:lang="de">Vorsorglicher Rechtsschutz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15972-6">
    <skos:altLabel xml:lang="de">Vorläufiger Rechtsschutz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15972-6">
    <skos:altLabel xml:lang="de">Einstweilige Verfügung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15972-6">
    <skos:altLabel xml:lang="de">Einstweilige Anordnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18019-3">
    <skos:narrower rdf:resource="descriptor/17948-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18019-3">
    <skos:narrower rdf:resource="descriptor/19172-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11356-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11790-4">
    <skos:prefLabel xml:lang="en">Economic change</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11790-4">
    <skos:prefLabel xml:lang="de">Strukturwandel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17772-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14683-4">
    <skos:narrower rdf:resource="descriptor/14692-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14683-4">
    <skos:narrower rdf:resource="descriptor/18079-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14683-4">
    <skos:narrower rdf:resource="descriptor/14673-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14683-4">
    <skos:narrower rdf:resource="descriptor/14662-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14683-4">
    <skos:narrower rdf:resource="descriptor/14664-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21041-4">
    <skos:prefLabel xml:lang="en">Systems selling strategy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21041-4">
    <skos:prefLabel xml:lang="de">Systemgeschäft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14307-4">
    <skos:prefLabel xml:lang="en">Mercury</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14307-4">
    <skos:prefLabel xml:lang="de">Quecksilber</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10314-6">
    <skos:related rdf:resource="descriptor/18160-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10314-6">
    <skos:related rdf:resource="descriptor/10326-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10314-6">
    <skos:related rdf:resource="descriptor/15709-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13028-6">
    <skos:prefLabel xml:lang="en">Foundry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13028-6">
    <skos:prefLabel xml:lang="de">Gießereiindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20206-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10076-3">
    <skos:narrower rdf:resource="descriptor/10085-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10076-3">
    <skos:narrower rdf:resource="descriptor/11927-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10076-3">
    <skos:narrower rdf:resource="descriptor/10081-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10076-3">
    <skos:narrower rdf:resource="descriptor/10077-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10076-3">
    <skos:narrower rdf:resource="descriptor/10075-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10076-3">
    <skos:narrower rdf:resource="descriptor/10073-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10076-3">
    <skos:narrower rdf:resource="descriptor/10070-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12469-2">
    <skos:prefLabel xml:lang="en">ABC analysis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12469-2">
    <skos:prefLabel xml:lang="de">ABC-Analyse</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19550-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10697-6">
    <skos:prefLabel xml:lang="en">Green money</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10697-6">
    <skos:prefLabel xml:lang="de">Grenzausgleich</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12703-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11030-1">
    <skos:prefLabel xml:lang="en">National economic plan</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11030-1">
    <skos:prefLabel xml:lang="de">Volkswirtschaftsplan</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10179-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11659-2">
    <skos:broader rdf:resource="thsys/71074"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11659-2">
    <skos:broader rdf:resource="thsys/71066"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11659-2">
    <skos:broader rdf:resource="thsys/70889"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11659-2">
    <skos:broader rdf:resource="thsys/70184"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11659-2">
    <skos:broader rdf:resource="descriptor/11676-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14181-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16726-2">
    <skos:related rdf:resource="descriptor/16729-3"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id332">
    <rdfs:seeAlso rdf:resource="descriptor/12955-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10382-3">
    <skos:narrower rdf:resource="descriptor/11490-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10382-3">
    <skos:narrower rdf:resource="descriptor/21101-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10382-3">
    <skos:narrower rdf:resource="descriptor/11521-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10382-3">
    <skos:narrower rdf:resource="descriptor/10721-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10382-3">
    <skos:narrower rdf:resource="descriptor/11487-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10382-3">
    <skos:narrower rdf:resource="descriptor/11484-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10382-3">
    <skos:narrower rdf:resource="descriptor/11494-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10382-3">
    <skos:narrower rdf:resource="descriptor/19547-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10382-3">
    <skos:narrower rdf:resource="descriptor/19262-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10382-3">
    <skos:narrower rdf:resource="descriptor/11479-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18936-2">
    <skos:altLabel xml:lang="en">TV commercial</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18936-2">
    <skos:altLabel xml:lang="en">Television commercial</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18936-2">
    <skos:altLabel xml:lang="de">Werbefernsehen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18936-2">
    <skos:altLabel xml:lang="de">TV-Werbung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18936-2">
    <skos:altLabel xml:lang="de">TV-Werbespot</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18936-2">
    <skos:altLabel xml:lang="de">Fernseh-Werbespot</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12603-3">
    <skos:prefLabel xml:lang="en">Humanization of work</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12603-3">
    <skos:prefLabel xml:lang="de">Humanisierung der Arbeitswelt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14135-0">
    <skos:prefLabel xml:lang="en">Poultry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14135-0">
    <skos:prefLabel xml:lang="de">Geflügel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18979-5">
    <skos:prefLabel xml:lang="en">Agricultural administration</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18979-5">
    <skos:prefLabel xml:lang="de">Landwirtschaftsverwaltung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12307-0">
    <skos:broader rdf:resource="thsys/70005"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11817-3">
    <skos:altLabel xml:lang="de">Gewerbeförderung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11817-3">
    <skos:altLabel xml:lang="de">Förderung von Gewerbebetrieben</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id163">
    <rdfs:seeAlso rdf:resource="descriptor/11683-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12916-3">
    <skos:related rdf:resource="descriptor/13058-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12916-3">
    <skos:related rdf:resource="descriptor/18100-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12916-3">
    <skos:related rdf:resource="descriptor/12278-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12916-3">
    <skos:related rdf:resource="descriptor/12914-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10018-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10676-0">
    <skos:scopeNote xml:lang="de">Die monetäre Außenwirtschaftstheorie oder Währungstheorie beinhaltet die theoretischen Grundlagen der internationalen Währungsordnung und des internationalen Kapitalverkehrs sowie die Zahlungsbilanztheorie.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11516-3">
    <skos:broader rdf:resource="thsys/70355"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11516-3">
    <skos:broader rdf:resource="descriptor/11507-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18501-2">
    <skos:broader rdf:resource="thsys/70969"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18501-2">
    <skos:broader rdf:resource="descriptor/13176-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11894-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18799-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16792-3">
    <skos:broader rdf:resource="thsys/70822"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11213-0">
    <skos:broader rdf:resource="thsys/71060"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11213-0">
    <skos:broader rdf:resource="descriptor/11182-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11754-1">
    <skos:related rdf:resource="descriptor/11765-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11754-1">
    <skos:related rdf:resource="descriptor/18155-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11754-1">
    <skos:related rdf:resource="descriptor/10217-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11754-1">
    <skos:related rdf:resource="descriptor/10186-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16209-1">
    <skos:broader rdf:resource="thsys/70095"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16209-1">
    <skos:broader rdf:resource="descriptor/16563-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16509-3">
    <skos:related rdf:resource="descriptor/16332-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16509-3">
    <skos:related rdf:resource="descriptor/15970-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16509-3">
    <skos:related rdf:resource="descriptor/16283-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16509-3">
    <skos:related rdf:resource="descriptor/16551-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16509-3">
    <skos:related rdf:resource="descriptor/16510-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10697-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19392-4">
    <skos:prefLabel xml:lang="en">Private school</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19392-4">
    <skos:prefLabel xml:lang="de">Privatschule</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19158-5">
    <skos:broader rdf:resource="thsys/71043"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19158-5">
    <skos:broader rdf:resource="thsys/71034"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19158-5">
    <skos:broader rdf:resource="descriptor/12337-5"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id352">
    <rdfs:seeAlso rdf:resource="descriptor/19670-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16463-1">
    <skos:altLabel xml:lang="de">Sicherheitspartnerschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16463-1">
    <skos:altLabel xml:lang="de">Internationale Sicherheitsstrategie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16463-1">
    <skos:altLabel xml:lang="de">Bündnisstrategie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13703-1">
    <skos:related rdf:resource="descriptor/18590-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13703-1">
    <skos:related rdf:resource="descriptor/12257-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13703-1">
    <skos:related rdf:resource="descriptor/13736-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13703-1">
    <skos:related rdf:resource="descriptor/13775-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17641-5">
    <skos:altLabel xml:lang="x-other">Kypros</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17641-5">
    <skos:altLabel xml:lang="x-other">Kybriaki Dimokratia</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17641-5">
    <skos:altLabel xml:lang="x-other">Kibris Cumhuriyeti</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17641-5">
    <skos:altLabel xml:lang="de">Republik Zypern</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16229-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19619-2">
    <skos:prefLabel xml:lang="en">Volcano</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19619-2">
    <skos:prefLabel xml:lang="de">Vulkan</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15675-5">
    <skos:broader rdf:resource="thsys/70819"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15675-5">
    <skos:broader rdf:resource="thsys/70581"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18884-2">
    <skos:prefLabel xml:lang="en">Education finance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18884-2">
    <skos:prefLabel xml:lang="de">Bildungsfinanzierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19642-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15902-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19316-6">
    <skos:broader rdf:resource="thsys/71091"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19316-6">
    <skos:broader rdf:resource="descriptor/10683-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16090-0">
    <skos:broader rdf:resource="thsys/73331"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16090-0">
    <skos:broader rdf:resource="thsys/71073"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16090-0">
    <skos:broader rdf:resource="descriptor/10829-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16090-0">
    <skos:broader rdf:resource="descriptor/16182-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14902-4">
    <skos:prefLabel xml:lang="en">Women's wear</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14902-4">
    <skos:prefLabel xml:lang="de">Damenbekleidung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11901-0">
    <skos:altLabel xml:lang="de">Sanierung von Gebäuden</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11901-0">
    <skos:altLabel xml:lang="de">Gebäudesanierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11901-0">
    <skos:altLabel xml:lang="de">Fassadensanierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11901-0">
    <skos:altLabel xml:lang="de">Fassadenreinigung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14298-0">
    <skos:prefLabel xml:lang="en">Gold</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14298-0">
    <skos:prefLabel xml:lang="de">Gold</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15176-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10615-6">
    <skos:narrower rdf:resource="descriptor/10811-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10615-6">
    <skos:narrower rdf:resource="descriptor/10613-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13529-5">
    <skos:narrower rdf:resource="descriptor/13541-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10945-6">
    <skos:related rdf:resource="descriptor/10466-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10945-6">
    <skos:related rdf:resource="descriptor/19251-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10945-6">
    <skos:related rdf:resource="descriptor/19548-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10945-6">
    <skos:related rdf:resource="descriptor/18082-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10945-6">
    <skos:related rdf:resource="descriptor/10459-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70074">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">P.12.07</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16562-6">
    <skos:broader rdf:resource="thsys/70180"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16562-6">
    <skos:broader rdf:resource="thsys/70141"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16562-6">
    <skos:broader rdf:resource="descriptor/10032-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13079-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14157-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10194-4">
    <skos:broader rdf:resource="thsys/71117"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10194-4">
    <skos:broader rdf:resource="descriptor/10141-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18942-0">
    <skos:related rdf:resource="descriptor/20314-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18615-1">
    <skos:broader rdf:resource="thsys/70940"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18615-1">
    <skos:broader rdf:resource="descriptor/13264-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#OrderedCollection">
    <skos:definition xml:lang="en">An ordered collection of concepts, where both the grouping and
      the ordering are meaningful.</skos:definition>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70941">
    <skos:prefLabel xml:lang="en">W.12.01.01  Rail transport</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70941">
    <skos:prefLabel xml:lang="de">W.12.01.01  Schienenverkehr</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#narrowerTransitive">
    <skos:definition>skos:narrowerTransitive is a transitive superproperty of skos:broader. By
      convention, skos:narrowerTransitive is not intended to be used in assertions, but provides a
      mechanism whereby the transitive closure of skos:narrower can be queried.</skos:definition>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19702-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16541-0">
    <zbwext:useInsteadNote rdf:nodeID="id283"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17983-5">
    <skos:prefLabel xml:lang="en">EU countries</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17983-5">
    <skos:prefLabel xml:lang="de">EU-Staaten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14014-5">
    <skos:altLabel xml:lang="de">Sozialpädagoginnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14014-5">
    <skos:altLabel xml:lang="de">Sozialpädagogen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14014-5">
    <skos:altLabel xml:lang="de">Sozialhelfer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14014-5">
    <skos:altLabel xml:lang="de">Sozialfürsorgerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14014-5">
    <skos:altLabel xml:lang="de">Sozialfürsorger</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14014-5">
    <skos:altLabel xml:lang="de">Sozialarbeiterinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14014-5">
    <skos:altLabel xml:lang="de">Sozialarbeiter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14014-5">
    <skos:altLabel xml:lang="de">Kindergärtnerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14014-5">
    <skos:altLabel xml:lang="de">Kindergärtner</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14014-5">
    <skos:altLabel xml:lang="de">Jugendpfleger</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14014-5">
    <skos:altLabel xml:lang="de">Jugendhelfer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18599-1">
    <skos:broader rdf:resource="thsys/71055"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18599-1">
    <skos:broader rdf:resource="thsys/70933"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18599-1">
    <skos:broader rdf:resource="descriptor/13979-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10022-5">
    <skos:narrower rdf:resource="descriptor/18157-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10088-3">
    <skos:related rdf:resource="descriptor/10109-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10088-3">
    <skos:related rdf:resource="descriptor/10106-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10088-3">
    <skos:related rdf:resource="descriptor/10089-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10347-5">
    <skos:broader rdf:resource="thsys/70020"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10347-5">
    <skos:broader rdf:resource="descriptor/10249-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11370-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12774-4">
    <skos:narrower rdf:resource="descriptor/19149-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12774-4">
    <skos:narrower rdf:resource="descriptor/13485-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12774-4">
    <skos:narrower rdf:resource="descriptor/10851-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19460-6">
    <skos:broader rdf:resource="thsys/71065"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19460-6">
    <skos:broader rdf:resource="descriptor/10025-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18992-6">
    <skos:narrower rdf:resource="descriptor/11646-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12543-2">
    <skos:prefLabel xml:lang="en">Depreciation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12543-2">
    <skos:prefLabel xml:lang="de">Abschreibung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16509-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15733-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10704-0">
    <skos:prefLabel xml:lang="en">Most favoured nation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10704-0">
    <skos:prefLabel xml:lang="de">Meistbegünstigung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18314-1">
    <zbwext:useInsteadNote rdf:nodeID="id338"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19507-6">
    <skos:prefLabel xml:lang="en">Welfare recipients</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19507-6">
    <skos:prefLabel xml:lang="de">Sozialhilfeempfänger</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16615-4">
    <skos:broader rdf:resource="thsys/70321"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13537-6">
    <skos:prefLabel xml:lang="en">Cargo handling in port</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13537-6">
    <skos:prefLabel xml:lang="de">Hafenumschlag</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19368-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18740-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19547-1">
    <skos:related rdf:resource="descriptor/19263-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19547-1">
    <skos:related rdf:resource="descriptor/11495-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19547-1">
    <skos:related rdf:resource="descriptor/11494-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19547-1">
    <skos:related rdf:resource="descriptor/19262-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18210-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13769-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14633-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18325-3">
    <skos:prefLabel xml:lang="en">Sustainable development</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18325-3">
    <skos:prefLabel xml:lang="de">Nachhaltige Entwicklung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18937-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14195-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70897">
    <skos:broader rdf:resource="thsys/70900"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10621-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12619-2">
    <skos:prefLabel xml:lang="en">Learning process</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12619-2">
    <skos:prefLabel xml:lang="de">Lernprozess</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18092-0">
    <skos:altLabel xml:lang="en">Distribution information system</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18092-0">
    <skos:altLabel xml:lang="de">WWS (Warenwirtschaftssystem)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19337-5">
    <skos:prefLabel xml:lang="en">Currency crisis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19337-5">
    <skos:prefLabel xml:lang="de">Währungskrise</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16027-0">
    <skos:prefLabel xml:lang="en">Marriage law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16027-0">
    <skos:prefLabel xml:lang="de">Eherecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19184-4">
    <skos:broader rdf:resource="thsys/71007"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19184-4">
    <skos:broader rdf:resource="thsys/70957"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19184-4">
    <skos:broader rdf:resource="descriptor/18995-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11658-4">
    <skos:broader rdf:resource="thsys/71074"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11658-4">
    <skos:broader rdf:resource="thsys/71066"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11658-4">
    <skos:broader rdf:resource="descriptor/11681-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11020-4">
    <skos:prefLabel xml:lang="en">Planning agency</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11020-4">
    <skos:prefLabel xml:lang="de">Planungsbehörde</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13470-5">
    <skos:prefLabel xml:lang="en">Market fair</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13470-5">
    <skos:prefLabel xml:lang="de">Marktveranstaltung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11547-6">
    <skos:altLabel xml:lang="de">Besteuerung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14337-2">
    <skos:prefLabel xml:lang="en">Air conditioner</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14337-2">
    <skos:prefLabel xml:lang="de">Klimaanlage</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11679-3">
    <skos:altLabel xml:lang="de">Unternehmensteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13530-6">
    <skos:broader rdf:resource="thsys/70936"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13530-6">
    <skos:broader rdf:resource="descriptor/13256-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19537-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12401-1">
    <skos:broader rdf:resource="thsys/71043"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12401-1">
    <skos:broader rdf:resource="descriptor/10019-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12401-1">
    <skos:broader rdf:resource="descriptor/12274-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17176-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14034-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16014-2">
    <skos:prefLabel xml:lang="en">Liability of the state</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16014-2">
    <skos:prefLabel xml:lang="de">Staatshaftung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10543-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16283-3">
    <skos:related rdf:resource="descriptor/16336-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16283-3">
    <skos:related rdf:resource="descriptor/19703-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16283-3">
    <skos:related rdf:resource="descriptor/11127-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16283-3">
    <skos:related rdf:resource="descriptor/18068-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16283-3">
    <skos:related rdf:resource="descriptor/16509-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18270-2">
    <skos:altLabel xml:lang="en">MIMD (Multiple Instruction Multiple Data)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18270-2">
    <skos:altLabel xml:lang="de">Multiprozessorrechner</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13055-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id201">
    <rdf:value xml:lang="de">Für Effekte wirtschaftspolitischer Maßnahmen</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13584-4">
    <skos:altLabel xml:lang="en">Modal choice in transportation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13584-4">
    <skos:altLabel xml:lang="en">Choice of mode</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13584-4">
    <skos:altLabel xml:lang="de">Transportmittelwahl</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13584-4">
    <skos:altLabel xml:lang="de">Reisemittelwahl</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11407-1">
    <skos:scopeNote xml:lang="de">Bezieht sich auf die wirtschaftlichen Aspekte des Bildungsverhaltens und die gesamtwirtschaftliche Auswirkung von Struktur und Entwicklung des Bildungswesens.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16539-1">
    <skos:prefLabel xml:lang="en">Pension finance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16539-1">
    <skos:prefLabel xml:lang="de">Rentenfinanzierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14906-3">
    <skos:altLabel xml:lang="de">Herrenwäsche</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14906-3">
    <skos:altLabel xml:lang="de">Herrenoberbekleidung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19306-2">
    <skos:related rdf:resource="descriptor/19307-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19306-2">
    <skos:related rdf:resource="descriptor/18316-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19306-2">
    <skos:related rdf:resource="descriptor/14084-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19306-2">
    <skos:related rdf:resource="descriptor/13165-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15773-5">
    <skos:prefLabel xml:lang="en">Environmental costs</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15773-5">
    <skos:prefLabel xml:lang="de">Umweltschutzkosten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10344-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11321-4">
    <skos:broader rdf:resource="thsys/71113"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11321-4">
    <skos:broader rdf:resource="thsys/70278"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11321-4">
    <skos:broader rdf:resource="thsys/70245"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11321-4">
    <skos:broader rdf:resource="descriptor/11341-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11321-4">
    <skos:broader rdf:resource="descriptor/11322-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11321-4">
    <skos:broader rdf:resource="descriptor/11769-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16995-3">
    <skos:altLabel xml:lang="en">Danmark</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16995-3">
    <skos:altLabel xml:lang="de">Königreich Dänemark</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12570-6">
    <skos:broader rdf:resource="thsys/73385"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12570-6">
    <skos:broader rdf:resource="thsys/70244"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12570-6">
    <skos:broader rdf:resource="descriptor/12571-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71028">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18986-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71004">
    <skos:broader rdf:resource="thsys/71005"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18943-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10973-1">
    <skos:broader rdf:resource="thsys/70038"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10973-1">
    <skos:broader rdf:resource="descriptor/10981-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14728-1">
    <skos:related rdf:resource="descriptor/13002-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14728-1">
    <skos:related rdf:resource="descriptor/19785-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14728-1">
    <skos:related rdf:resource="descriptor/13193-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14728-1">
    <skos:related rdf:resource="descriptor/14042-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14728-1">
    <skos:related rdf:resource="descriptor/14068-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14728-1">
    <skos:related rdf:resource="descriptor/16670-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14728-1">
    <skos:related rdf:resource="descriptor/14734-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14728-1">
    <skos:related rdf:resource="descriptor/13371-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14728-1">
    <skos:related rdf:resource="descriptor/14724-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14728-1">
    <skos:related rdf:resource="descriptor/14026-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14728-1">
    <skos:related rdf:resource="descriptor/14723-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14728-1">
    <skos:related rdf:resource="descriptor/19539-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14728-1">
    <skos:related rdf:resource="descriptor/18259-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14728-1">
    <skos:related rdf:resource="descriptor/19675-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19410-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19198-0">
    <skos:scopeNote xml:lang="de">Auch für Glaubwürdigkeit wirtschaftspolitischer Ankündigungen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11849-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71010">
    <skos:broader rdf:resource="thsys/w"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19037-3">
    <skos:broader rdf:resource="thsys/72186"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15971-1">
    <skos:altLabel xml:lang="de">Rechtsentscheidung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15971-1">
    <skos:altLabel xml:lang="de">Rechtsentscheid</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15971-1">
    <skos:altLabel xml:lang="de">Präzedenzfall</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15971-1">
    <skos:altLabel xml:lang="de">Gerichtsentscheidung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15971-1">
    <skos:altLabel xml:lang="de">Gerichtsentscheid</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14936-1">
    <skos:broader rdf:resource="thsys/70498"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14936-1">
    <skos:broader rdf:resource="descriptor/14931-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19764-0">
    <skos:altLabel xml:lang="de">Gesetz zum Schutz der Beschäftigten vor sexueller Belästigung am Arbeitsplatz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70337">
    <skos:narrower rdf:resource="descriptor/15005-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70337">
    <skos:narrower rdf:resource="descriptor/14422-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70337">
    <skos:narrower rdf:resource="descriptor/14413-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70337">
    <skos:narrower rdf:resource="descriptor/18223-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70337">
    <skos:narrower rdf:resource="descriptor/14450-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70337">
    <skos:narrower rdf:resource="descriptor/14600-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70337">
    <skos:narrower rdf:resource="descriptor/15001-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12592-3">
    <skos:prefLabel xml:lang="en">Fringe benefits</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12592-3">
    <skos:prefLabel xml:lang="de">Betriebliche Sozialleistungen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11361-6">
    <skos:prefLabel xml:lang="en">Vocational training</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11361-6">
    <skos:prefLabel xml:lang="de">Berufsausbildung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10597-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14164-0">
    <skos:related rdf:resource="descriptor/12963-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17753-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16771-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13571-6">
    <skos:prefLabel xml:lang="en">Transport sociology</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13571-6">
    <skos:prefLabel xml:lang="de">Verkehrssoziologie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19174-0">
    <skos:altLabel xml:lang="de">CARICOM-Länder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16300-1">
    <skos:related rdf:resource="descriptor/16416-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16300-1">
    <skos:related rdf:resource="descriptor/16264-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16300-1">
    <skos:related rdf:resource="descriptor/18062-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16300-1">
    <skos:related rdf:resource="descriptor/16299-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16300-1">
    <skos:related rdf:resource="descriptor/15797-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11068-4">
    <skos:broader rdf:resource="thsys/70039"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15918-5">
    <skos:related rdf:resource="descriptor/16615-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15918-5">
    <skos:related rdf:resource="descriptor/16421-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15918-5">
    <skos:related rdf:resource="descriptor/15928-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15918-5">
    <skos:related rdf:resource="descriptor/16264-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15918-5">
    <skos:related rdf:resource="descriptor/18062-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15918-5">
    <skos:related rdf:resource="descriptor/16299-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15918-5">
    <skos:related rdf:resource="descriptor/16512-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15918-5">
    <skos:related rdf:resource="descriptor/16661-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15918-5">
    <skos:related rdf:resource="descriptor/19639-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15918-5">
    <skos:related rdf:resource="descriptor/16556-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15918-5">
    <skos:related rdf:resource="descriptor/15931-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15918-5">
    <skos:related rdf:resource="descriptor/15907-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18165-6">
    <skos:related rdf:resource="descriptor/15771-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18165-6">
    <skos:related rdf:resource="descriptor/15770-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18165-6">
    <skos:related rdf:resource="descriptor/15768-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18165-6">
    <skos:related rdf:resource="descriptor/15767-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18165-6">
    <skos:related rdf:resource="descriptor/19633-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18165-6">
    <skos:related rdf:resource="descriptor/18186-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18165-6">
    <skos:related rdf:resource="descriptor/20125-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18165-6">
    <skos:related rdf:resource="descriptor/18453-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18165-6">
    <skos:related rdf:resource="descriptor/12879-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71074">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11526-0">
    <skos:related rdf:resource="descriptor/16212-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11526-0">
    <skos:related rdf:resource="descriptor/10975-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11526-0">
    <skos:related rdf:resource="descriptor/11540-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11526-0">
    <skos:related rdf:resource="descriptor/11525-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11526-0">
    <skos:related rdf:resource="descriptor/11728-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13361-3">
    <skos:broader rdf:resource="thsys/70822"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13361-3">
    <skos:broader rdf:resource="descriptor/13350-1"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id257">
    <rdf:value xml:lang="de">Für Fernmeldeunternehmen allein</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13967-2">
    <skos:broader rdf:resource="thsys/71055"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13967-2">
    <skos:broader rdf:resource="thsys/70876"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13967-2">
    <skos:broader rdf:resource="descriptor/18535-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11735-5">
    <skos:related rdf:resource="descriptor/21082-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11735-5">
    <skos:related rdf:resource="descriptor/11765-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11735-5">
    <skos:related rdf:resource="descriptor/11779-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11735-5">
    <skos:related rdf:resource="descriptor/11752-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11735-5">
    <skos:related rdf:resource="descriptor/11751-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11735-5">
    <skos:related rdf:resource="descriptor/11692-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11735-5">
    <skos:related rdf:resource="descriptor/11770-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11735-5">
    <skos:related rdf:resource="descriptor/11753-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11735-5">
    <skos:related rdf:resource="descriptor/11713-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16030-4">
    <skos:related rdf:resource="descriptor/19774-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16030-4">
    <skos:related rdf:resource="descriptor/16477-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16030-4">
    <skos:related rdf:resource="descriptor/15938-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16030-4">
    <skos:related rdf:resource="descriptor/16029-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15860-3">
    <skos:narrower rdf:resource="descriptor/10056-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15860-3">
    <skos:narrower rdf:resource="descriptor/19216-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15163-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12253-4">
    <skos:prefLabel xml:lang="en">Credit rating</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12253-4">
    <skos:prefLabel xml:lang="de">Kreditwürdigkeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10725-6">
    <skos:altLabel xml:lang="en">Capital control</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10725-6">
    <skos:altLabel xml:lang="de">Kapitalverkehrskontrolle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10725-6">
    <skos:altLabel xml:lang="de">Kapitalverkehrsbeschränkung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10725-6">
    <skos:altLabel xml:lang="de">Kapitalexportverbot</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10725-6">
    <skos:altLabel xml:lang="de">Kapitalausfuhrverbot</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24707-3">
    <skos:related rdf:resource="descriptor/10021-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24707-3">
    <skos:related rdf:resource="descriptor/19196-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12302-3">
    <skos:narrower rdf:resource="descriptor/19379-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12302-3">
    <skos:narrower rdf:resource="descriptor/25621-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12302-3">
    <skos:narrower rdf:resource="descriptor/13692-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16201-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13011-2">
    <skos:broader rdf:resource="thsys/71008"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13011-2">
    <skos:broader rdf:resource="descriptor/13020-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18157-5">
    <skos:broader rdf:resource="thsys/71089"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18157-5">
    <skos:broader rdf:resource="thsys/71084"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18157-5">
    <skos:broader rdf:resource="descriptor/10022-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18157-5">
    <skos:broader rdf:resource="descriptor/19358-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19381-2">
    <skos:related rdf:resource="descriptor/19416-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11893-1">
    <skos:altLabel xml:lang="de">Wochenendpendlerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11893-1">
    <skos:altLabel xml:lang="de">Wochenendpendler</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11893-1">
    <skos:altLabel xml:lang="de">Tagespendlerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11893-1">
    <skos:altLabel xml:lang="de">Tagespendler</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11893-1">
    <skos:altLabel xml:lang="de">Schülerpendlerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11893-1">
    <skos:altLabel xml:lang="de">Schülerpendler</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11893-1">
    <skos:altLabel xml:lang="de">Pendlerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11893-1">
    <skos:altLabel xml:lang="de">Pendler</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11893-1">
    <skos:altLabel xml:lang="de">Pendelwanderung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11893-1">
    <skos:altLabel xml:lang="de">Pendelströme</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11893-1">
    <skos:altLabel xml:lang="de">Grenzpendlerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11893-1">
    <skos:altLabel xml:lang="de">Grenzpendler</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18413-6">
    <skos:broader rdf:resource="thsys/71002"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18413-6">
    <skos:broader rdf:resource="descriptor/13058-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10963-4">
    <skos:related rdf:resource="descriptor/10956-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10963-4">
    <skos:related rdf:resource="descriptor/10964-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10963-4">
    <skos:related rdf:resource="descriptor/10942-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10963-4">
    <skos:related rdf:resource="descriptor/10895-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10963-4">
    <skos:related rdf:resource="descriptor/10949-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10963-4">
    <skos:related rdf:resource="descriptor/10891-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14784-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16084-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13336-2">
    <skos:broader rdf:resource="thsys/70818"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13336-2">
    <skos:broader rdf:resource="descriptor/15727-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18832-0">
    <skos:broader rdf:resource="thsys/73386"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18832-0">
    <skos:broader rdf:resource="thsys/70853"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18832-0">
    <skos:broader rdf:resource="thsys/70180"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18832-0">
    <skos:broader rdf:resource="descriptor/15722-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18832-0">
    <skos:broader rdf:resource="descriptor/16709-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16295-3">
    <skos:narrower rdf:resource="descriptor/19379-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19734-2">
    <skos:altLabel xml:lang="en">Philosophy of economics</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19734-2">
    <skos:altLabel xml:lang="de">Philosophie der Ökonomik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13680-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15772-0">
    <skos:broader rdf:resource="thsys/70239"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15772-0">
    <skos:broader rdf:resource="descriptor/19711-0"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id348">
    <rdf:value xml:lang="de">Für verhaltenswissenschaftlich/verhaltenspsychologisch ausgerichtete ökonomische Analysen</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18762-2">
    <skos:narrower rdf:resource="descriptor/13390-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18762-2">
    <skos:narrower rdf:resource="descriptor/13346-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18762-2">
    <skos:narrower rdf:resource="descriptor/18424-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18762-2">
    <skos:narrower rdf:resource="descriptor/13379-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18762-2">
    <skos:narrower rdf:resource="descriptor/18763-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19053-5">
    <skos:related rdf:resource="descriptor/19063-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19053-5">
    <skos:related rdf:resource="descriptor/14042-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19053-5">
    <skos:related rdf:resource="descriptor/15908-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70838">
    <skos:prefLabel xml:lang="en">W.25.01  Occupational health</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70838">
    <skos:prefLabel xml:lang="de">W.25.01  Betriebliche Gesundheitspolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19680-6">
    <skos:related rdf:resource="descriptor/16544-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19680-6">
    <skos:related rdf:resource="descriptor/19752-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19680-6">
    <skos:related rdf:resource="descriptor/18830-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19680-6">
    <skos:related rdf:resource="descriptor/11825-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19680-6">
    <skos:related rdf:resource="descriptor/19773-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19680-6">
    <skos:related rdf:resource="descriptor/16587-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19680-6">
    <skos:related rdf:resource="descriptor/18073-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11326-1">
    <skos:altLabel xml:lang="en">Wage Share</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11326-1">
    <skos:altLabel xml:lang="de">Gesamtwirtschaftliche Lohnquote</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71009">
    <skos:prefLabel xml:lang="en">W.05.01  Mining</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71009">
    <skos:prefLabel xml:lang="de">W.05.01  Bergbau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id377">
    <rdfs:seeAlso rdf:resource="descriptor/12041-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10567-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11558-1">
    <skos:related rdf:resource="descriptor/16545-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11558-1">
    <skos:related rdf:resource="descriptor/16542-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11558-1">
    <skos:related rdf:resource="descriptor/12592-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11521-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18612-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14608-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14015-3">
    <skos:prefLabel xml:lang="en">Social scientists</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14015-3">
    <skos:prefLabel xml:lang="de">Sozialwissenschaftler</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12206-6">
    <skos:altLabel xml:lang="de">Finanzierungslehre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13995-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10241-0">
    <skos:altLabel xml:lang="en">Aggregate supply function</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10241-0">
    <skos:altLabel xml:lang="de">Makroökonomische Angebotsfunktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10241-0">
    <skos:altLabel xml:lang="de">Angebotsfunktion der Gesamtwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10241-0">
    <skos:altLabel xml:lang="de">Angebot der Gesamtwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19451-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17609-1">
    <skos:broader rdf:resource="thsys/70561"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17609-1">
    <skos:broader rdf:resource="descriptor/19167-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19518-1">
    <skos:related rdf:resource="descriptor/18903-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19518-1">
    <skos:related rdf:resource="descriptor/13372-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16957-4">
    <skos:altLabel xml:lang="de">Appenzell Innerrhoden</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16957-4">
    <skos:altLabel xml:lang="de">Appenzell Außerrhoden</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11771-1">
    <skos:narrower rdf:resource="descriptor/10985-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11771-1">
    <skos:narrower rdf:resource="descriptor/19235-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19623-4">
    <skos:prefLabel xml:lang="en">Air</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19623-4">
    <skos:prefLabel xml:lang="de">Luft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13283-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13516-0">
    <skos:prefLabel xml:lang="en">Airport</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13516-0">
    <skos:prefLabel xml:lang="de">Flughafen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19416-2">
    <skos:broader rdf:resource="thsys/71075"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19416-2">
    <skos:broader rdf:resource="descriptor/20189-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19416-2">
    <skos:broader rdf:resource="descriptor/11547-6"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id11">
    <rdf:value xml:lang="de">Für ambulante Pflege</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16541-0">
    <skos:altLabel xml:lang="de">Soziale Rehabilitation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16541-0">
    <skos:altLabel xml:lang="de">Schwerbehindertenrehabilitation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16541-0">
    <skos:altLabel xml:lang="de">Rehabilitationsmaßnahme</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16541-0">
    <skos:altLabel xml:lang="de">Medizinische Rehabilitation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16541-0">
    <skos:altLabel xml:lang="de">Behindertenrehabilitation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10124-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19941-4">
    <skos:scopeNote xml:lang="de">Aufsicht bzw. Kontrolle der EU über die Subventionspolitik der Mitgliedstaaten zur Verhinderung missbräuchlicher Beihilfen und Subventionen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19804-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19305-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70866">
    <skos:narrower rdf:resource="descriptor/11125-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70866">
    <skos:narrower rdf:resource="descriptor/10882-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70866">
    <skos:narrower rdf:resource="descriptor/16648-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70866">
    <skos:narrower rdf:resource="descriptor/10603-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70866">
    <skos:narrower rdf:resource="descriptor/13407-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70866">
    <skos:narrower rdf:resource="descriptor/16489-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11956-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18955-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10628-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11702-6">
    <skos:scopeNote xml:lang="de">Selektive Ausnahmen von der allgemeinen Steuerpflicht.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18655-3">
    <skos:broader rdf:resource="thsys/70922"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18655-3">
    <skos:broader rdf:resource="descriptor/13278-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15742-2">
    <skos:broader rdf:resource="thsys/73351"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15742-2">
    <skos:broader rdf:resource="thsys/70239"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15742-2">
    <skos:broader rdf:resource="descriptor/15737-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18844-0">
    <skos:prefLabel xml:lang="en">Tenants' association</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18844-0">
    <skos:prefLabel xml:lang="de">Mieterorganisation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11966-0">
    <skos:broader rdf:resource="thsys/71064"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11966-0">
    <skos:broader rdf:resource="descriptor/11970-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12736-5">
    <skos:narrower rdf:resource="descriptor/12737-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14439-1">
    <skos:prefLabel xml:lang="en">Bus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14439-1">
    <skos:prefLabel xml:lang="de">Omnibus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18882-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21072-0">
    <skos:scopeNote xml:lang="de">Literatur über den Umfang des Versicherungsschutzes und über den Anteil der gegen ein Risiko versicherten bzw. nicht versicherten Bevölkerung. Es wird mit der jeweiligen Versicherungssparte postkoordiniert.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14428-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10984-3">
    <skos:prefLabel xml:lang="en">Interest group</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10984-3">
    <skos:prefLabel xml:lang="de">Interessenvertretung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17727-2">
    <skos:prefLabel xml:lang="en">Angola</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17727-2">
    <skos:prefLabel xml:lang="de">Angola</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18214-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17626-1">
    <skos:broader rdf:resource="thsys/70253"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17626-1">
    <skos:broader rdf:resource="descriptor/17608-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17626-1">
    <skos:broader rdf:resource="descriptor/17978-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19136-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18531-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12898-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13340-4">
    <skos:related rdf:resource="descriptor/13560-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13340-4">
    <skos:related rdf:resource="descriptor/13330-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70199">
    <rdfs:label xml:lang="de">Pharmazeutische Produkte</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73355">
    <skos:prefLabel xml:lang="de">N.06.04.01  Wirtschafts- und Industriesoziologie, Siedlungssoziologie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18170-6">
    <skos:scopeNote xml:lang="de">Konstruktion und Rekonstruktion von Bildern aus Daten</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19823-3">
    <skos:broader rdf:resource="thsys/73392"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19823-3">
    <skos:broader rdf:resource="descriptor/16035-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14310-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12897-2">
    <skos:altLabel xml:lang="de">Grundsätze ordnungsmäßiger Abschlussprüfung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12897-2">
    <skos:altLabel xml:lang="de">Grundsätze ordnungsmäßiger Abschlußprüfung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10112-4">
    <skos:broader rdf:resource="thsys/71111"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10112-4">
    <skos:broader rdf:resource="thsys/70057"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10112-4">
    <skos:broader rdf:resource="descriptor/10113-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15993-5">
    <skos:altLabel xml:lang="de">Kohleverstromungsgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15993-5">
    <skos:altLabel xml:lang="de">EnWG (Energiewirtschaftsgesetz)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15993-5">
    <skos:altLabel xml:lang="de">Energiewirtschaftsrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15993-5">
    <skos:altLabel xml:lang="de">Energiewirtschaftsgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15993-5">
    <skos:altLabel xml:lang="de">Energiesicherungsgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15993-5">
    <skos:altLabel xml:lang="de">Energiegesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15993-5">
    <skos:altLabel xml:lang="de">Energieeinsparungsgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16521-6">
    <skos:prefLabel xml:lang="en">National security</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16521-6">
    <skos:prefLabel xml:lang="de">Nationale Sicherheit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13281-1">
    <skos:prefLabel xml:lang="en">Pawnshop</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13281-1">
    <skos:prefLabel xml:lang="de">Leihhaus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15880-4">
    <skos:broader rdf:resource="thsys/73352"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10465-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18006-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14723-4">
    <skos:scopeNote xml:lang="de">für Sucht, Drogenpolitik und drogenökonomische Fragen BENUTZE die entsprechenden Deskriptoren.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14723-4">
    <skos:scopeNote xml:lang="de">Natürliche, halbsynthetische oder künstlich hergestellte Stoffe, die zu einer Veränderung des Bewusstseinszustands führen können.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16564-2">
    <skos:narrower rdf:resource="descriptor/19766-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16564-2">
    <skos:narrower rdf:resource="descriptor/16687-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16564-2">
    <skos:narrower rdf:resource="descriptor/13571-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16564-2">
    <skos:narrower rdf:resource="descriptor/16695-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16564-2">
    <skos:narrower rdf:resource="descriptor/11992-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16564-2">
    <skos:narrower rdf:resource="descriptor/13861-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16564-2">
    <skos:narrower rdf:resource="descriptor/16688-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16564-2">
    <skos:narrower rdf:resource="descriptor/12110-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16564-2">
    <skos:narrower rdf:resource="descriptor/16684-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16564-2">
    <skos:narrower rdf:resource="descriptor/16680-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16564-2">
    <skos:narrower rdf:resource="descriptor/16700-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16564-2">
    <skos:narrower rdf:resource="descriptor/16692-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16564-2">
    <skos:narrower rdf:resource="descriptor/16683-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16564-2">
    <skos:narrower rdf:resource="descriptor/16698-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16564-2">
    <skos:narrower rdf:resource="descriptor/13445-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10717-5">
    <skos:broader rdf:resource="thsys/71095"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10717-5">
    <skos:broader rdf:resource="descriptor/10722-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70932">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20002-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11262-1">
    <skos:altLabel xml:lang="de">Werktätige</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11262-1">
    <skos:altLabel xml:lang="de">Personal</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11262-1">
    <skos:altLabel xml:lang="de">Mitarbeiter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11262-1">
    <skos:altLabel xml:lang="de">Lohnempfänger</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11262-1">
    <skos:altLabel xml:lang="de">Lohnabhängige</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11262-1">
    <skos:altLabel xml:lang="de">Erwerbstätige</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11262-1">
    <skos:altLabel xml:lang="de">Betriebsangehörige</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11262-1">
    <skos:altLabel xml:lang="de">Beschäftigte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11262-1">
    <skos:altLabel xml:lang="de">Besatzung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11262-1">
    <skos:altLabel xml:lang="de">Belegschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11262-1">
    <skos:altLabel xml:lang="de">Arbeitskräftestruktur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11262-1">
    <skos:altLabel xml:lang="de">Arbeitnehmer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15628-0">
    <skos:altLabel xml:lang="de">Nukleartechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15628-0">
    <skos:altLabel xml:lang="de">Nuklearforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15628-0">
    <skos:altLabel xml:lang="de">Kerntechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15628-0">
    <skos:altLabel xml:lang="de">Kernphysik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15628-0">
    <skos:altLabel xml:lang="de">Kernenergieforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15628-0">
    <skos:altLabel xml:lang="de">Atomtechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15628-0">
    <skos:altLabel xml:lang="de">Atomforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15628-0">
    <skos:altLabel xml:lang="de">Atomenergieforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13678-2">
    <skos:prefLabel xml:lang="en">Telephone network</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13678-2">
    <skos:prefLabel xml:lang="de">Telefonnetz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19524-6">
    <skos:broader rdf:resource="thsys/70049"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71119">
    <skos:prefLabel xml:lang="de">V.02.04  Markttheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18078-1">
    <skos:prefLabel xml:lang="en">Engineering plastics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18078-1">
    <skos:prefLabel xml:lang="de">Technischer Kunststoff</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12204-3">
    <skos:broader rdf:resource="thsys/70897"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12204-3">
    <skos:broader rdf:resource="thsys/70187"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12204-3">
    <skos:broader rdf:resource="descriptor/12210-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13962-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18841-6">
    <skos:scopeNote xml:lang="de">Beinhaltet vor allem rechts- und sozialwissenschaftliche sowie ökonomische Aspekte.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16143-5">
    <skos:altLabel xml:lang="en">Short-term disability benefit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16143-5">
    <skos:altLabel xml:lang="de">Lohngarantie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16143-5">
    <skos:altLabel xml:lang="de">Lohnfortzahlungsgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16143-5">
    <skos:altLabel xml:lang="de">Gesetz über die Fortzahlung des Arbeitsentgelts im Krankheitsfalle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16143-5">
    <skos:altLabel xml:lang="de">Garantierter Jahreslohn</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14053-2">
    <skos:related rdf:resource="descriptor/14189-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14053-2">
    <skos:related rdf:resource="descriptor/13179-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14053-2">
    <skos:related rdf:resource="descriptor/18380-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11258-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18222-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13024-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14236-1">
    <skos:related rdf:resource="descriptor/14255-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14236-1">
    <skos:related rdf:resource="descriptor/18388-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14236-1">
    <skos:related rdf:resource="descriptor/13019-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14236-1">
    <skos:related rdf:resource="descriptor/14601-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14236-1">
    <skos:related rdf:resource="descriptor/14192-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14236-1">
    <skos:related rdf:resource="descriptor/12980-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14236-1">
    <skos:related rdf:resource="descriptor/14021-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70536">
    <skos:narrower rdf:resource="descriptor/14741-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70536">
    <skos:narrower rdf:resource="descriptor/14739-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70536">
    <skos:narrower rdf:resource="descriptor/14738-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70536">
    <skos:narrower rdf:resource="descriptor/14622-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70536">
    <skos:narrower rdf:resource="descriptor/14737-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70536">
    <skos:narrower rdf:resource="descriptor/14736-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70536">
    <skos:narrower rdf:resource="descriptor/14735-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70536">
    <skos:narrower rdf:resource="descriptor/14734-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70536">
    <skos:narrower rdf:resource="descriptor/18263-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70536">
    <skos:narrower rdf:resource="descriptor/14733-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70536">
    <skos:narrower rdf:resource="descriptor/14732-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70536">
    <skos:narrower rdf:resource="descriptor/18264-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70536">
    <skos:narrower rdf:resource="descriptor/18265-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70536">
    <skos:narrower rdf:resource="descriptor/18282-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14525-1">
    <skos:prefLabel xml:lang="en">Video product</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14525-1">
    <skos:prefLabel xml:lang="de">Videoprodukt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70927">
    <skos:broader rdf:resource="thsys/70928"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19412-3">
    <skos:related rdf:resource="descriptor/19236-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11629-4">
    <skos:narrower rdf:resource="descriptor/13249-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19374-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13315-3">
    <skos:altLabel xml:lang="en">Funeral director</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13315-3">
    <skos:altLabel xml:lang="de">Leichenbestatter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13315-3">
    <skos:altLabel xml:lang="de">Krematorium</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13315-3">
    <skos:altLabel xml:lang="de">Grabverkauf</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13315-3">
    <skos:altLabel xml:lang="de">Friedhof</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13315-3">
    <skos:altLabel xml:lang="de">Beerdigungsinstitut</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11302-1">
    <skos:related rdf:resource="descriptor/13992-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11302-1">
    <skos:related rdf:resource="descriptor/18159-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11302-1">
    <skos:related rdf:resource="descriptor/18083-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11302-1">
    <skos:related rdf:resource="descriptor/12085-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11302-1">
    <skos:related rdf:resource="descriptor/20948-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11302-1">
    <skos:related rdf:resource="descriptor/11274-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11302-1">
    <skos:related rdf:resource="descriptor/11320-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11302-1">
    <skos:related rdf:resource="descriptor/15825-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11302-1">
    <skos:related rdf:resource="descriptor/15823-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11302-1">
    <skos:related rdf:resource="descriptor/12570-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11302-1">
    <skos:related rdf:resource="descriptor/18143-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11302-1">
    <skos:related rdf:resource="descriptor/16658-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13333-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19081-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18323-0">
    <skos:prefLabel xml:lang="en">Financing of rural development</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18323-0">
    <skos:prefLabel xml:lang="de">Ländliche Entwicklungsfinanzierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10781-3">
    <skos:related rdf:resource="descriptor/13743-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10781-3">
    <skos:related rdf:resource="descriptor/10821-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10781-3">
    <skos:related rdf:resource="descriptor/10773-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10781-3">
    <skos:related rdf:resource="descriptor/19337-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10781-3">
    <skos:related rdf:resource="descriptor/19333-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10781-3">
    <skos:related rdf:resource="descriptor/10779-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10781-3">
    <skos:related rdf:resource="descriptor/13757-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10781-3">
    <skos:related rdf:resource="descriptor/10776-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15908-1">
    <skos:related rdf:resource="descriptor/15679-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15908-1">
    <skos:related rdf:resource="descriptor/16155-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15908-1">
    <skos:related rdf:resource="descriptor/18180-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15908-1">
    <skos:related rdf:resource="descriptor/16670-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15908-1">
    <skos:related rdf:resource="descriptor/16529-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15908-1">
    <skos:related rdf:resource="descriptor/19053-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19450-2">
    <skos:broader rdf:resource="thsys/71068"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19450-2">
    <skos:broader rdf:resource="descriptor/11869-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13202-2">
    <skos:broader rdf:resource="thsys/70959"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13202-2">
    <skos:broader rdf:resource="descriptor/18522-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11504-3">
    <zbwext:useInsteadNote rdf:nodeID="id140"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12284-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17497-0">
    <skos:broader rdf:resource="thsys/71032"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17497-0">
    <skos:broader rdf:resource="descriptor/17595-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17497-0">
    <skos:broader rdf:resource="descriptor/19078-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21094-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16300-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18664-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19626-5">
    <skos:broader rdf:resource="thsys/73351"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19626-5">
    <skos:broader rdf:resource="descriptor/19630-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17783-6">
    <skos:altLabel xml:lang="de">Republik Haiti</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18245-1">
    <skos:prefLabel xml:lang="en">Mobile robot</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18245-1">
    <skos:prefLabel xml:lang="de">Mobiler Roboter</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18498-0">
    <skos:related rdf:resource="descriptor/18604-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18498-0">
    <skos:related rdf:resource="descriptor/13516-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10223-2">
    <skos:broader rdf:resource="thsys/71119"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10223-2">
    <skos:broader rdf:resource="thsys/70242"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10223-2">
    <skos:broader rdf:resource="descriptor/10905-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18536-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19712-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21082-4">
    <skos:prefLabel xml:lang="en">Distributional effect</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21082-4">
    <skos:prefLabel xml:lang="de">Verteilungswirkung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70292">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12196-4">
    <skos:related rdf:resource="descriptor/10911-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12196-4">
    <skos:related rdf:resource="descriptor/12868-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12196-4">
    <skos:related rdf:resource="descriptor/16076-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12196-4">
    <skos:related rdf:resource="descriptor/12304-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12196-4">
    <skos:related rdf:resource="descriptor/11696-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12196-4">
    <skos:related rdf:resource="descriptor/12094-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12196-4">
    <skos:related rdf:resource="descriptor/12194-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12196-4">
    <skos:related rdf:resource="descriptor/18032-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19488-5">
    <skos:scopeNote xml:lang="de">Natürliche Ressourcen im Gemeineigentum.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13608-2">
    <skos:prefLabel xml:lang="en">Transport Management Science</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13608-2">
    <skos:prefLabel xml:lang="de">Verkehrsbetriebslehre</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14169-4">
    <skos:broader rdf:resource="thsys/73351"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14169-4">
    <skos:broader rdf:resource="thsys/70403"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13647-6">
    <skos:scopeNote xml:lang="de">Gesamtheit der Rechtsnormen zum Post- und Fernmeldewesen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12292-1">
    <skos:scopeNote xml:lang="de">Wertpapierkäufe im Ausland zu Zwecken der Kapitalanlage ohne Absicht der Einflussnahme auf den Geschäftsbetrieb.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12779-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15765-4">
    <skos:scopeNote xml:lang="de">Interdisziplinärer Wissenschaftsbereich zur Analyse der Wechselbeziehungen zwischen Lebewesen und ihrer Umwelt bzw. von Ökosystemen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19220-5">
    <skos:altLabel xml:lang="en">Wants</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19220-5">
    <skos:altLabel xml:lang="de">Bedürfnislehre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19220-5">
    <skos:altLabel xml:lang="de">Bedürfnis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19079-1">
    <skos:broader rdf:resource="thsys/70183"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19079-1">
    <skos:broader rdf:resource="descriptor/12251-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18676-2">
    <skos:broader rdf:resource="thsys/73390"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18676-2">
    <skos:broader rdf:resource="thsys/70893"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18676-2">
    <skos:broader rdf:resource="thsys/70187"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18676-2">
    <skos:broader rdf:resource="descriptor/13731-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14015-3">
    <skos:broader rdf:resource="thsys/71052"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14015-3">
    <skos:broader rdf:resource="descriptor/13349-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14522-0">
    <skos:broader rdf:resource="thsys/70373"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14522-0">
    <skos:broader rdf:resource="descriptor/18103-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19814-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10505-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12564-1">
    <skos:related rdf:resource="descriptor/19001-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12564-1">
    <skos:related rdf:resource="descriptor/19008-3"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id188">
    <rdfs:seeAlso rdf:resource="descriptor/18983-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11972-5">
    <skos:altLabel xml:lang="en">Megacity</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11972-5">
    <skos:altLabel xml:lang="de">Städteballung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11972-5">
    <skos:altLabel xml:lang="de">Megastadt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13260-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13420-6">
    <skos:altLabel xml:lang="de">Staatsfunktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13420-6">
    <skos:altLabel xml:lang="de">Staatliche Rolle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13420-6">
    <skos:altLabel xml:lang="de">Staatliche Ordnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13420-6">
    <skos:altLabel xml:lang="de">Rolle des Staates</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19084-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19372-3">
    <skos:related rdf:resource="descriptor/19508-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19372-3">
    <skos:related rdf:resource="descriptor/11578-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19372-3">
    <skos:related rdf:resource="descriptor/11552-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19372-3">
    <skos:related rdf:resource="descriptor/11243-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12177-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16455-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71060">
    <skos:prefLabel xml:lang="en">V.13.01.03  Unemployment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71060">
    <skos:prefLabel xml:lang="de">V.13.01.03  Arbeitslosigkeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19384-3">
    <skos:broader rdf:resource="thsys/71080"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19384-3">
    <skos:broader rdf:resource="descriptor/11568-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10183-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71096">
    <rdfs:label xml:lang="en">Tariffs</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71096">
    <rdfs:label xml:lang="de">Zoll</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18995-0">
    <skos:broader rdf:resource="thsys/71010"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18995-0">
    <skos:broader rdf:resource="thsys/70957"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18995-0">
    <skos:broader rdf:resource="thsys/70811"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18995-0">
    <skos:broader rdf:resource="descriptor/10012-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71015">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10627-6">
    <skos:scopeNote xml:lang="de">Gemeinsamer Markt plus Koordinierung der Wirtschaftspolitik.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19376-2">
    <skos:prefLabel xml:lang="en">Generational accounting</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19376-2">
    <skos:prefLabel xml:lang="de">Intergenerative Belastungsrechnung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10147-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15063-1">
    <skos:broader rdf:resource="thsys/72182"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15063-1">
    <skos:broader rdf:resource="descriptor/15064-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16472-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18078-1">
    <skos:broader rdf:resource="thsys/70476"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18078-1">
    <skos:broader rdf:resource="descriptor/14702-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14188-0">
    <skos:prefLabel xml:lang="en">Water</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14188-0">
    <skos:prefLabel xml:lang="de">Wasser</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70134">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10208-5">
    <skos:related rdf:resource="descriptor/20522-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10208-5">
    <skos:related rdf:resource="descriptor/10894-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10208-5">
    <skos:related rdf:resource="descriptor/19223-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10208-5">
    <skos:related rdf:resource="descriptor/24662-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15019-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18970-2">
    <skos:altLabel xml:lang="en">Motherhood</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18970-2">
    <skos:altLabel xml:lang="en">Maternity</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18970-2">
    <skos:altLabel xml:lang="de">Mutterschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18970-2">
    <skos:altLabel xml:lang="de">Mutter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11187-3">
    <skos:broader rdf:resource="thsys/71116"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11187-3">
    <skos:broader rdf:resource="thsys/71059"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11187-3">
    <skos:broader rdf:resource="thsys/71050"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11187-3">
    <skos:broader rdf:resource="descriptor/19784-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16786-5">
    <skos:prefLabel xml:lang="en">Information retrieval</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16786-5">
    <skos:prefLabel xml:lang="de">Informationsretrieval</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70997">
    <skos:narrower rdf:resource="descriptor/13069-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70997">
    <skos:narrower rdf:resource="descriptor/13065-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70997">
    <skos:narrower rdf:resource="descriptor/13060-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70997">
    <skos:narrower rdf:resource="descriptor/13055-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70997">
    <skos:narrower rdf:resource="descriptor/13053-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70997">
    <skos:narrower rdf:resource="descriptor/18422-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70997">
    <skos:narrower rdf:resource="descriptor/18421-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70997">
    <skos:narrower rdf:resource="descriptor/19549-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70997">
    <skos:narrower rdf:resource="descriptor/13051-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70997">
    <skos:narrower rdf:resource="descriptor/18423-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70997">
    <skos:narrower rdf:resource="descriptor/19052-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70997">
    <skos:narrower rdf:resource="descriptor/13046-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13655-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15712-4">
    <skos:narrower rdf:resource="descriptor/16699-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15712-4">
    <skos:narrower rdf:resource="descriptor/15931-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15712-4">
    <skos:narrower rdf:resource="descriptor/12577-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15712-4">
    <skos:narrower rdf:resource="descriptor/11314-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12796-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16304-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12543-2">
    <skos:altLabel xml:lang="de">Abschreibungsverfahren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12543-2">
    <skos:altLabel xml:lang="de">Abschreibungsplan</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12543-2">
    <skos:altLabel xml:lang="de">Abschreibungsmethode</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12737-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11191-5">
    <skos:altLabel xml:lang="en">Worker dislocation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11191-5">
    <skos:altLabel xml:lang="en">Layoff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11191-5">
    <skos:altLabel xml:lang="en">Employee dismissal</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11191-5">
    <skos:altLabel xml:lang="de">Personalfreisetzung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11191-5">
    <skos:altLabel xml:lang="de">Freisetzung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11191-5">
    <skos:altLabel xml:lang="de">Entlassung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11191-5">
    <skos:altLabel xml:lang="de">Betriebsbedingte Kündigung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11191-5">
    <skos:altLabel xml:lang="de">Arbeitsvertragsbeendigung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11191-5">
    <skos:altLabel xml:lang="de">Arbeitsplatzverlust</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14070-2">
    <skos:related rdf:resource="descriptor/14961-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14070-2">
    <skos:related rdf:resource="descriptor/14055-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11235-4">
    <skos:related rdf:resource="descriptor/11233-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11235-4">
    <skos:related rdf:resource="descriptor/16011-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11235-4">
    <skos:related rdf:resource="descriptor/11239-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11235-4">
    <skos:related rdf:resource="descriptor/11238-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11235-4">
    <skos:related rdf:resource="descriptor/11392-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11235-4">
    <skos:related rdf:resource="descriptor/11389-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11235-4">
    <skos:related rdf:resource="descriptor/11237-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11235-4">
    <skos:related rdf:resource="descriptor/11185-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11235-4">
    <skos:related rdf:resource="descriptor/11246-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11235-4">
    <skos:related rdf:resource="descriptor/11219-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11235-4">
    <skos:related rdf:resource="descriptor/19767-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11235-4">
    <skos:related rdf:resource="descriptor/19493-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11235-4">
    <skos:related rdf:resource="descriptor/11183-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11235-4">
    <skos:related rdf:resource="descriptor/11182-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11235-4">
    <skos:related rdf:resource="descriptor/10996-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11928-1">
    <skos:narrower rdf:resource="descriptor/11932-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11928-1">
    <skos:narrower rdf:resource="descriptor/10219-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11928-1">
    <skos:narrower rdf:resource="descriptor/11927-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11928-1">
    <skos:narrower rdf:resource="descriptor/20321-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13974-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15914-6">
    <skos:scopeNote xml:lang="de">Zusammensetzung der Bevölkerung nach den Merkmalen Geschlecht, Beruf, Familienstand, Konfession, Rasse, Nationalität usw.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70328">
    <skos:narrower rdf:resource="descriptor/14810-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70328">
    <skos:narrower rdf:resource="descriptor/14808-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70328">
    <skos:narrower rdf:resource="descriptor/14809-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70328">
    <skos:narrower rdf:resource="descriptor/14807-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70328">
    <skos:narrower rdf:resource="descriptor/14805-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70328">
    <skos:narrower rdf:resource="descriptor/14804-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70328">
    <skos:narrower rdf:resource="descriptor/14803-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70328">
    <skos:narrower rdf:resource="descriptor/14802-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70328">
    <skos:narrower rdf:resource="descriptor/14798-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70328">
    <skos:narrower rdf:resource="descriptor/18205-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70328">
    <skos:narrower rdf:resource="descriptor/14796-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11305-2">
    <skos:scopeNote xml:lang="de">Allgemeine physische, soziale und unternehmensbezogene Faktoren, welche die Arbeitsumgebung beeinflussen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19265-4">
    <skos:related rdf:resource="descriptor/12230-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19265-4">
    <skos:related rdf:resource="descriptor/12295-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19265-4">
    <skos:related rdf:resource="descriptor/12272-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10785-2">
    <skos:broader rdf:resource="thsys/71089"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10785-2">
    <skos:broader rdf:resource="thsys/70114"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10785-2">
    <skos:broader rdf:resource="descriptor/13807-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10785-2">
    <skos:broader rdf:resource="descriptor/10811-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19367-3">
    <skos:broader rdf:resource="thsys/71019"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19367-3">
    <skos:broader rdf:resource="thsys/70991"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19367-3">
    <skos:broader rdf:resource="thsys/70957"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19367-3">
    <skos:broader rdf:resource="descriptor/18995-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12284-0">
    <skos:broader rdf:resource="thsys/71046"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12284-0">
    <skos:broader rdf:resource="thsys/70036"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19290-5">
    <skos:related rdf:resource="descriptor/14055-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19290-5">
    <skos:related rdf:resource="descriptor/18190-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18075-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19916-3">
    <skos:prefLabel xml:lang="en">Educational attainment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19916-3">
    <skos:prefLabel xml:lang="de">Bildungsabschluss</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10876-6">
    <skos:broader rdf:resource="thsys/70951"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10876-6">
    <skos:broader rdf:resource="thsys/70041"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10876-6">
    <skos:broader rdf:resource="descriptor/12766-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19359-2">
    <skos:broader rdf:resource="thsys/73366"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19359-2">
    <skos:broader rdf:resource="thsys/73332"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19359-2">
    <skos:broader rdf:resource="thsys/71089"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19359-2">
    <skos:broader rdf:resource="descriptor/16184-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19359-2">
    <skos:broader rdf:resource="descriptor/18694-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19359-2">
    <skos:broader rdf:resource="descriptor/19358-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18553-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10602-1">
    <skos:prefLabel xml:lang="en">Foreign operations</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10602-1">
    <skos:prefLabel xml:lang="de">Auslandsgeschäft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16046-3">
    <skos:related rdf:resource="descriptor/18704-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16046-3">
    <skos:related rdf:resource="descriptor/10774-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12585-0">
    <skos:altLabel xml:lang="en">Managerial grid</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12585-0">
    <skos:altLabel xml:lang="en">Management grid</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12585-0">
    <skos:altLabel xml:lang="de">Personalbetreuung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12585-0">
    <skos:altLabel xml:lang="de">Mitarbeiterführung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12585-0">
    <skos:altLabel xml:lang="de">Managementtechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12585-0">
    <skos:altLabel xml:lang="de">Management-by-Prinzip</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12585-0">
    <skos:altLabel xml:lang="de">Führungstechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12585-0">
    <skos:altLabel xml:lang="de">Führungsprinzip</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12585-0">
    <skos:altLabel xml:lang="de">Führungskonzeption</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70932">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19328-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13446-2">
    <skos:broader rdf:resource="thsys/73348"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13446-2">
    <skos:broader rdf:resource="thsys/71028"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13446-2">
    <skos:broader rdf:resource="descriptor/15685-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12064-0">
    <skos:narrower rdf:resource="descriptor/13811-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12064-0">
    <skos:narrower rdf:resource="descriptor/19392-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12064-0">
    <skos:narrower rdf:resource="descriptor/18005-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14798-1">
    <skos:altLabel xml:lang="de">Toilettenpapier</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14798-1">
    <skos:altLabel xml:lang="de">Tampon</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14798-1">
    <skos:altLabel xml:lang="de">Papierwindel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14798-1">
    <skos:altLabel xml:lang="de">Papiertuch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14798-1">
    <skos:altLabel xml:lang="de">Papiertaschentuch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14798-1">
    <skos:altLabel xml:lang="de">Papierserviette</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14798-1">
    <skos:altLabel xml:lang="de">Mehrzwecktuch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14798-1">
    <skos:altLabel xml:lang="de">Hygienepapierprodukt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13557-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10590-3">
    <skos:related rdf:resource="descriptor/12154-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10590-3">
    <skos:related rdf:resource="descriptor/11556-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10590-3">
    <skos:related rdf:resource="descriptor/11531-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10590-3">
    <skos:related rdf:resource="descriptor/11530-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10590-3">
    <skos:related rdf:resource="descriptor/11529-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10590-3">
    <skos:related rdf:resource="descriptor/12208-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10590-3">
    <skos:related rdf:resource="descriptor/10561-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10590-3">
    <skos:related rdf:resource="descriptor/10548-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19327-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18935-4">
    <skos:related rdf:resource="descriptor/18929-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18935-4">
    <skos:related rdf:resource="descriptor/18936-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18935-4">
    <skos:related rdf:resource="descriptor/18934-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18935-4">
    <skos:related rdf:resource="descriptor/13655-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19800-1">
    <skos:related rdf:resource="descriptor/15605-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19800-1">
    <skos:related rdf:resource="descriptor/15599-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19800-1">
    <skos:related rdf:resource="descriptor/19247-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19039-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13287-3">
    <skos:altLabel xml:lang="en">Credit union</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13287-3">
    <skos:altLabel xml:lang="de">Volksbank</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13287-3">
    <skos:altLabel xml:lang="de">Raiffeisenkasse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13287-3">
    <skos:altLabel xml:lang="de">Genossenschaftsbank</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14633-5">
    <skos:related rdf:resource="descriptor/14129-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14633-5">
    <skos:related rdf:resource="descriptor/14656-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14633-5">
    <skos:related rdf:resource="descriptor/14044-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14633-5">
    <skos:related rdf:resource="descriptor/14131-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16757-5">
    <skos:prefLabel xml:lang="en">Information technology</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16757-5">
    <skos:prefLabel xml:lang="de">Informationstechnik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15778-2">
    <skos:related rdf:resource="descriptor/15768-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15757-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12981-6">
    <skos:altLabel xml:lang="de">Pechkohlenbergbau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12981-6">
    <skos:altLabel xml:lang="de">Braunkohlenindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18730-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14630-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70354">
    <rdfs:label xml:lang="en">Africa South of Sahara</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70354">
    <rdfs:label xml:lang="de">Afrika südlich der Sahara</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11441-1">
    <skos:prefLabel xml:lang="en">Financial innovation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11441-1">
    <skos:prefLabel xml:lang="de">Finanzinnovation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18947-4">
    <skos:altLabel xml:lang="de">Telekommunikationselektronikerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18947-4">
    <skos:altLabel xml:lang="de">Telekommunikationselektroniker</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18947-4">
    <skos:altLabel xml:lang="de">Telegraphistinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18947-4">
    <skos:altLabel xml:lang="de">Telegraphisten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18947-4">
    <skos:altLabel xml:lang="de">Telegrafistinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18947-4">
    <skos:altLabel xml:lang="de">Telegrafisten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18947-4">
    <skos:altLabel xml:lang="de">Telefonistinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18947-4">
    <skos:altLabel xml:lang="de">Telefonisten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18947-4">
    <skos:altLabel xml:lang="de">Funkerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18947-4">
    <skos:altLabel xml:lang="de">Funker</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18947-4">
    <skos:altLabel xml:lang="de">Fernmeldetechnische Berufe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18947-4">
    <skos:altLabel xml:lang="de">Fernmeldetechnikerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18947-4">
    <skos:altLabel xml:lang="de">Fernmeldetechniker</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18947-4">
    <skos:altLabel xml:lang="de">Fernmeldeberufe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10575-6">
    <skos:broader rdf:resource="thsys/71065"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10575-6">
    <skos:broader rdf:resource="thsys/70091"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10575-6">
    <skos:broader rdf:resource="descriptor/11895-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10575-6">
    <skos:broader rdf:resource="descriptor/10524-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11090-4">
    <skos:broader rdf:resource="thsys/71101"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11090-4">
    <skos:broader rdf:resource="thsys/70141"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11090-4">
    <skos:broader rdf:resource="descriptor/10062-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13181-5">
    <skos:altLabel xml:lang="de">Wärmeschutzinstallation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13181-5">
    <skos:altLabel xml:lang="de">Schallschutzinstallation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13181-5">
    <skos:altLabel xml:lang="de">Installationshandwerk</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13181-5">
    <skos:altLabel xml:lang="de">Gebäudeisolierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13181-5">
    <skos:altLabel xml:lang="de">Gebäudedämmung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13181-5">
    <skos:altLabel xml:lang="de">Abdämmungsgewerbe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14251-5">
    <skos:altLabel xml:lang="en">Refractory material</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14251-5">
    <skos:altLabel xml:lang="de">Feuerfester Stoff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14251-5">
    <skos:altLabel xml:lang="de">Feuerfeste Keramik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13966-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18636-0">
    <skos:related rdf:resource="descriptor/18640-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18636-0">
    <skos:related rdf:resource="descriptor/13635-6"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id379">
    <rdf:value xml:lang="de">Für den Bestand an Wohnraum</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19324-0">
    <skos:altLabel xml:lang="de">Handelskrieg</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19324-0">
    <skos:altLabel xml:lang="de">Außenhandelskonflikt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18877-6">
    <skos:broader rdf:resource="thsys/70844"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18877-6">
    <skos:broader rdf:resource="thsys/70176"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18877-6">
    <skos:broader rdf:resource="descriptor/11402-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18877-6">
    <skos:broader rdf:resource="descriptor/11398-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15124-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13307-2">
    <skos:altLabel xml:lang="en">Catering and hotels</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13307-2">
    <skos:altLabel xml:lang="de">Hotel- und Gaststättengewerbe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19323-2">
    <skos:related rdf:resource="descriptor/16081-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19323-2">
    <skos:related rdf:resource="descriptor/16017-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19323-2">
    <skos:related rdf:resource="descriptor/19687-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19323-2">
    <skos:related rdf:resource="descriptor/15994-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19323-2">
    <skos:related rdf:resource="descriptor/19649-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19323-2">
    <skos:related rdf:resource="descriptor/10917-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19323-2">
    <skos:related rdf:resource="descriptor/18001-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19323-2">
    <skos:related rdf:resource="descriptor/15989-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15775-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12777-5">
    <skos:narrower rdf:resource="descriptor/12782-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14293-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19520-0">
    <skos:altLabel xml:lang="de">Handelshaus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19520-0">
    <skos:altLabel xml:lang="de">Handelsgesellschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10160-0">
    <skos:altLabel xml:lang="en">Utility measurement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10160-0">
    <skos:altLabel xml:lang="de">Ordinaler Nutzen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10160-0">
    <skos:altLabel xml:lang="de">Nutzenmaßstab</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10160-0">
    <skos:altLabel xml:lang="de">Kardinaler Nutzen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10160-0">
    <skos:altLabel xml:lang="de">Interdependenter Nutzen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10160-0">
    <skos:altLabel xml:lang="de">Indirekter Nutzen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10160-0">
    <skos:altLabel xml:lang="de">Additiver Nutzen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18089-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13239-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15711-6">
    <skos:related rdf:resource="descriptor/20261-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15711-6">
    <skos:related rdf:resource="descriptor/16595-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15711-6">
    <skos:related rdf:resource="descriptor/16396-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15711-6">
    <skos:related rdf:resource="descriptor/16226-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15711-6">
    <skos:related rdf:resource="descriptor/16630-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15711-6">
    <skos:related rdf:resource="descriptor/10984-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15711-6">
    <skos:related rdf:resource="descriptor/16266-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#related">
    <rdfs:comment xml:lang="en">skos:related is disjoint with skos:broaderTransitive</rdfs:comment>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13285-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11664-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18166-4">
    <skos:altLabel xml:lang="de">Sonntagsarbeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18166-4">
    <skos:altLabel xml:lang="de">Samstagsarbeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18166-4">
    <skos:altLabel xml:lang="de">Feiertagsarbeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10772-4">
    <skos:related rdf:resource="descriptor/18127-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10772-4">
    <skos:related rdf:resource="descriptor/10754-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10772-4">
    <skos:related rdf:resource="descriptor/10746-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10772-4">
    <skos:related rdf:resource="descriptor/10745-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10772-4">
    <skos:related rdf:resource="descriptor/11494-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10772-4">
    <skos:related rdf:resource="descriptor/10236-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10772-4">
    <skos:related rdf:resource="descriptor/19261-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10772-4">
    <skos:related rdf:resource="descriptor/19260-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11134-3">
    <skos:broader rdf:resource="thsys/71101"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11134-3">
    <skos:broader rdf:resource="descriptor/10911-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18955-5">
    <skos:broader rdf:resource="thsys/70819"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18955-5">
    <skos:broader rdf:resource="descriptor/10032-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70245">
    <rdfs:label xml:lang="de">Vergütungssysteme und betriebliche Sozialleistungen</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11869-5">
    <skos:altLabel xml:lang="de">Raumordnerisches Leitbild</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11869-5">
    <skos:altLabel xml:lang="de">Räumliche Ordnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11869-5">
    <skos:altLabel xml:lang="de">Ordnung im Raum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11869-5">
    <skos:altLabel xml:lang="de">Leitbild des Raumes</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16767-2">
    <skos:related rdf:resource="descriptor/16788-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16767-2">
    <skos:related rdf:resource="descriptor/15473-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16767-2">
    <skos:related rdf:resource="descriptor/13197-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16767-2">
    <skos:related rdf:resource="descriptor/18951-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16767-2">
    <skos:related rdf:resource="descriptor/18950-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13849-1">
    <skos:broader rdf:resource="thsys/70888"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16019-6">
    <skos:broader rdf:resource="thsys/73380"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16019-6">
    <skos:broader rdf:resource="thsys/73360"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16019-6">
    <skos:broader rdf:resource="thsys/70871"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16019-6">
    <skos:broader rdf:resource="descriptor/16007-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12738-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16309-4">
    <skos:altLabel xml:lang="en">State autonomy</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16309-4">
    <skos:altLabel xml:lang="de">Staatliche Souveränität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16309-4">
    <skos:altLabel xml:lang="de">Staatliche Autonomie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16309-4">
    <skos:altLabel xml:lang="de">Innere Souveränität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16309-4">
    <skos:altLabel xml:lang="de">Äußere Souveränität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19437-1">
    <skos:prefLabel xml:lang="en">Declining industrial area</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19437-1">
    <skos:prefLabel xml:lang="de">Alternde Industrieregion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14798-1">
    <skos:related rdf:resource="descriptor/14042-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12371-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13159-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70925">
    <skos:prefLabel xml:lang="de">W.12.06  Verkehrsgeographie und regionale Verkehrspolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10042-6">
    <skos:broader rdf:resource="thsys/71100"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10042-6">
    <skos:broader rdf:resource="thsys/70234"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10042-6">
    <skos:broader rdf:resource="descriptor/10065-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18445-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20104-4">
    <skos:broader rdf:resource="thsys/70305"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20104-4">
    <skos:broader rdf:resource="descriptor/11490-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18899-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19206-6">
    <skos:broader rdf:resource="thsys/71083"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19206-6">
    <skos:broader rdf:resource="thsys/70141"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19206-6">
    <skos:broader rdf:resource="descriptor/11526-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19206-6">
    <skos:broader rdf:resource="descriptor/16745-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18292-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11844-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73340">
    <skos:prefLabel xml:lang="de">N.10.07.02  Programmierung und Software</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18514-0">
    <skos:altLabel xml:lang="de">Gebrauchtwagenhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12625-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10945-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11986-1">
    <skos:related rdf:resource="descriptor/13860-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11986-1">
    <skos:related rdf:resource="descriptor/11990-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11986-1">
    <skos:related rdf:resource="descriptor/19473-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11986-1">
    <skos:related rdf:resource="descriptor/11988-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11986-1">
    <skos:related rdf:resource="descriptor/15695-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11986-1">
    <skos:related rdf:resource="descriptor/11970-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11986-1">
    <skos:related rdf:resource="descriptor/11951-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11986-1">
    <skos:related rdf:resource="descriptor/16205-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18848-6">
    <skos:altLabel xml:lang="en">Insider-outsider approach</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18848-6">
    <skos:altLabel xml:lang="de">Insider-Outsider Theorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19729-2">
    <skos:altLabel xml:lang="de">Haftungsbeschränkung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19729-2">
    <skos:altLabel xml:lang="de">Durchgriffshaftung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13445-4">
    <skos:related rdf:resource="descriptor/16688-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13445-4">
    <skos:related rdf:resource="descriptor/10503-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13445-4">
    <skos:related rdf:resource="descriptor/19461-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13445-4">
    <skos:related rdf:resource="descriptor/13454-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13445-4">
    <skos:related rdf:resource="descriptor/13451-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13445-4">
    <skos:related rdf:resource="descriptor/13425-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16142-0">
    <skos:altLabel xml:lang="en">Wrongful-discharge law</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16142-0">
    <skos:altLabel xml:lang="en">Protection against dismissal</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16142-0">
    <skos:altLabel xml:lang="de">Kündigungsschutzrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16142-0">
    <skos:altLabel xml:lang="de">Kündigungsschutzgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16142-0">
    <skos:altLabel xml:lang="de">Kündigungsfrist</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16142-0">
    <skos:altLabel xml:lang="de">KSchG (Kündigungsschutzgesetz)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16142-0">
    <skos:altLabel xml:lang="de">Arbeitsrechtlicher Kündigungsschutz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14504-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16588-2">
    <skos:broader rdf:resource="thsys/70321"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70328">
    <rdfs:label xml:lang="de">Pappe, Papier und Papierprodukte</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19803-2">
    <skos:altLabel xml:lang="de">Objektorientierte Systementwicklung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19803-2">
    <skos:altLabel xml:lang="de">Objektorientierte Systemanalyse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19803-2">
    <skos:altLabel xml:lang="de">Objektorientierte Software-Entwicklung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14715-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19784-1">
    <skos:broader rdf:resource="thsys/73350"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19178-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18153-6">
    <skos:prefLabel xml:lang="en">Industrial service</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18153-6">
    <skos:prefLabel xml:lang="de">Industrielle Dienstleistung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16026-2">
    <skos:prefLabel xml:lang="en">Civil code</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16026-2">
    <skos:prefLabel xml:lang="de">Bürgerliches Recht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16549-5">
    <skos:altLabel xml:lang="en">Local policy</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16549-5">
    <skos:altLabel xml:lang="de">Stadtpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16549-5">
    <skos:altLabel xml:lang="de">Lokalpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16549-5">
    <skos:altLabel xml:lang="de">Gemeindepolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19708-3">
    <skos:prefLabel xml:lang="en">International cooperation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19708-3">
    <skos:prefLabel xml:lang="de">Internationale Zusammenarbeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13545-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10517-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13337-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13390-3">
    <skos:prefLabel xml:lang="en">Software industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13390-3">
    <skos:prefLabel xml:lang="de">Softwareindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13745-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19342-5">
    <skos:prefLabel xml:lang="en">Fruit market</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19342-5">
    <skos:prefLabel xml:lang="de">Obstmarkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11979-5">
    <skos:narrower rdf:resource="descriptor/11972-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10823-6">
    <skos:broader rdf:resource="thsys/71090"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10823-6">
    <skos:broader rdf:resource="thsys/71044"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10823-6">
    <skos:broader rdf:resource="descriptor/10008-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10823-6">
    <skos:broader rdf:resource="descriptor/10811-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10823-6">
    <skos:broader rdf:resource="descriptor/10605-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14703-3">
    <skos:prefLabel xml:lang="en">Synthetic resin</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14703-3">
    <skos:prefLabel xml:lang="de">Kunstharz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11817-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18002-6">
    <skos:narrower rdf:resource="descriptor/18023-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18002-6">
    <skos:narrower rdf:resource="descriptor/19117-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13715-1">
    <skos:related rdf:resource="descriptor/13743-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13715-1">
    <skos:related rdf:resource="descriptor/13742-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13715-1">
    <skos:related rdf:resource="descriptor/12299-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13715-1">
    <skos:related rdf:resource="descriptor/10773-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13715-1">
    <skos:related rdf:resource="descriptor/18679-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13715-1">
    <skos:related rdf:resource="descriptor/19274-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13715-1">
    <skos:related rdf:resource="descriptor/19554-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13715-1">
    <skos:related rdf:resource="descriptor/19669-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13715-1">
    <skos:related rdf:resource="descriptor/19671-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13715-1">
    <skos:related rdf:resource="descriptor/18730-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13715-1">
    <skos:related rdf:resource="descriptor/13279-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13715-1">
    <skos:related rdf:resource="descriptor/10001-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18278-0">
    <skos:related rdf:resource="descriptor/10016-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18278-0">
    <skos:related rdf:resource="descriptor/13459-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16030-4">
    <skos:broader rdf:resource="thsys/73392"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16030-4">
    <skos:broader rdf:resource="thsys/70814"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16030-4">
    <skos:broader rdf:resource="descriptor/16026-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16055-2">
    <skos:altLabel xml:lang="de">HandwO</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16055-2">
    <skos:altLabel xml:lang="de">Handwerksordnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15396-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18538-0">
    <skos:prefLabel xml:lang="en">Export industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18538-0">
    <skos:prefLabel xml:lang="de">Exportindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10436-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11559-6">
    <skos:narrower rdf:resource="descriptor/11289-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13368-3">
    <skos:broader rdf:resource="thsys/73296"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13368-3">
    <skos:broader rdf:resource="descriptor/18449-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18764-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15708-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17743-4">
    <skos:altLabel xml:lang="de">Zentralafrikanische Staaten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17743-4">
    <skos:altLabel xml:lang="de">Zentralafrikanische Länder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17743-4">
    <skos:altLabel xml:lang="de">Äquatorialafrika</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10150-3">
    <skos:broader rdf:resource="thsys/73386"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10150-3">
    <skos:broader rdf:resource="thsys/70180"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10150-3">
    <skos:broader rdf:resource="descriptor/15722-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10150-3">
    <skos:broader rdf:resource="descriptor/16709-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11668-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13645-3">
    <skos:prefLabel xml:lang="en">Vacation behaviour</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13645-3">
    <skos:prefLabel xml:lang="de">Urlaubsverhalten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15473-3">
    <skos:broader rdf:resource="thsys/70383"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15473-3">
    <skos:broader rdf:resource="descriptor/13345-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15473-3">
    <skos:broader rdf:resource="descriptor/12640-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11888-1">
    <skos:prefLabel xml:lang="en">Internal migration</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11888-1">
    <skos:prefLabel xml:lang="de">Binnenwanderung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13470-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70873">
    <skos:broader rdf:resource="thsys/70876"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70477">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13074-6">
    <skos:broader rdf:resource="thsys/70799"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13074-6">
    <skos:broader rdf:resource="descriptor/18424-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12111-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10176-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12918-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12304-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70811">
    <rdfs:label xml:lang="en">Resource economics</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70811">
    <rdfs:label xml:lang="de">Ressourcenökonomik</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13667-0">
    <skos:narrower rdf:resource="descriptor/18081-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13667-0">
    <skos:narrower rdf:resource="descriptor/13649-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10414-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11350-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13297-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14266-6">
    <skos:broader rdf:resource="thsys/70358"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14266-6">
    <skos:broader rdf:resource="descriptor/14236-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14266-6">
    <skos:broader rdf:resource="descriptor/14601-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70113">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70459">
    <skos:narrower rdf:resource="descriptor/12778-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70459">
    <skos:narrower rdf:resource="descriptor/12510-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70459">
    <skos:narrower rdf:resource="descriptor/10940-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70459">
    <skos:narrower rdf:resource="descriptor/19140-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70459">
    <skos:narrower rdf:resource="descriptor/13475-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70459">
    <skos:narrower rdf:resource="descriptor/19139-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70459">
    <skos:narrower rdf:resource="descriptor/18552-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70459">
    <skos:narrower rdf:resource="descriptor/19138-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70459">
    <skos:narrower rdf:resource="descriptor/19556-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70459">
    <skos:narrower rdf:resource="descriptor/19136-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70459">
    <skos:narrower rdf:resource="descriptor/21283-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70459">
    <skos:narrower rdf:resource="descriptor/18795-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70459">
    <skos:narrower rdf:resource="descriptor/12783-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70459">
    <skos:narrower rdf:resource="descriptor/18082-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70459">
    <skos:narrower rdf:resource="descriptor/12782-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70459">
    <skos:narrower rdf:resource="descriptor/19137-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70459">
    <skos:narrower rdf:resource="descriptor/12781-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70459">
    <skos:narrower rdf:resource="descriptor/18549-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70459">
    <skos:narrower rdf:resource="descriptor/19544-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70459">
    <skos:narrower rdf:resource="descriptor/12780-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70459">
    <skos:narrower rdf:resource="descriptor/19120-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70459">
    <skos:narrower rdf:resource="descriptor/19541-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70459">
    <skos:narrower rdf:resource="descriptor/12812-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70459">
    <skos:narrower rdf:resource="descriptor/12777-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70459">
    <skos:narrower rdf:resource="descriptor/18007-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15103-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14119-5">
    <skos:broader rdf:resource="thsys/70169"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14119-5">
    <skos:broader rdf:resource="descriptor/14117-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20967-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19716-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13434-2">
    <skos:prefLabel xml:lang="en">Crop yield</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13434-2">
    <skos:prefLabel xml:lang="de">Ernte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16966-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18425-6">
    <skos:prefLabel xml:lang="en">Electronics policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18425-6">
    <skos:prefLabel xml:lang="de">Elektronikpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16679-6">
    <skos:broader rdf:resource="thsys/70829"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16679-6">
    <skos:broader rdf:resource="thsys/70013"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16679-6">
    <skos:broader rdf:resource="descriptor/16554-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14895-3">
    <skos:narrower rdf:resource="descriptor/14897-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12209-0">
    <skos:related rdf:resource="descriptor/12487-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12209-0">
    <skos:related rdf:resource="descriptor/12212-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12209-0">
    <skos:related rdf:resource="descriptor/10008-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12209-0">
    <skos:related rdf:resource="descriptor/10294-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12209-0">
    <skos:related rdf:resource="descriptor/12206-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12209-0">
    <skos:related rdf:resource="descriptor/12224-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12209-0">
    <skos:related rdf:resource="descriptor/12215-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13410-2">
    <skos:broader rdf:resource="thsys/70862"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13410-2">
    <skos:broader rdf:resource="thsys/70154"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13410-2">
    <skos:broader rdf:resource="descriptor/19841-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13641-4">
    <skos:prefLabel xml:lang="en">International tourism</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13641-4">
    <skos:prefLabel xml:lang="de">Internationaler Tourismus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16318-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13721-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19196-4">
    <skos:scopeNote xml:lang="de">Scharfer (spekulativer) Preisanstieg.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18680-4">
    <skos:related rdf:resource="descriptor/11415-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12380-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70799">
    <rdfs:label xml:lang="en">Electronics industry</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70799">
    <rdfs:label xml:lang="de">Elektronikindustrie</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71103">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12977-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11233-1">
    <skos:related rdf:resource="descriptor/11230-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11233-1">
    <skos:related rdf:resource="descriptor/11235-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11233-1">
    <skos:related rdf:resource="descriptor/11243-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12504-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24783-1">
    <skos:scopeNote xml:lang="de">Beschreibt die neue, auch ökonomisch definierte Rolle und das entsprechende Funktionsverständnis von Staat und Verwaltung. Charakterisiert eine Bewegung zur Modernisierung von Verwaltungen und zur Wiedergewinnung von Handlungsmöglichkeiten des Staates mit dem Ziel einer Verbesserung von Effektivität, Effizienz und Kostenwirtschaftlichkeit.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17500-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12785-6">
    <skos:broader rdf:resource="thsys/70040"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12785-6">
    <skos:broader rdf:resource="descriptor/12770-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12116-0">
    <skos:altLabel xml:lang="en">Scientific management</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12116-0">
    <skos:altLabel xml:lang="de">Wissenschaftliche Betriebsführung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12116-0">
    <skos:altLabel xml:lang="de">Wissenschaftliche Arbeitsorganisation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12116-0">
    <skos:altLabel xml:lang="de">Taylor-System</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12116-0">
    <skos:altLabel xml:lang="de">Taylorische Arbeitsteilung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10395-1">
    <skos:altLabel xml:lang="en">Tinbergen-assignment</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10395-1">
    <skos:altLabel xml:lang="en">Mundell-assignment</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10395-1">
    <skos:altLabel xml:lang="en">Cambridge-assignment</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10395-1">
    <skos:altLabel xml:lang="en">Assignment problem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10395-1">
    <skos:altLabel xml:lang="de">Wirtschaftspolitische Rollenverteilung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10395-1">
    <skos:altLabel xml:lang="de">Wirtschaftspolitische Mittelwahl</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10395-1">
    <skos:altLabel xml:lang="de">Rollenverteilung der Wirtschaftspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19078-3">
    <skos:broader rdf:resource="thsys/70307"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19302-3">
    <skos:broader rdf:resource="thsys/71029"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19302-3">
    <skos:broader rdf:resource="descriptor/18198-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18413-6">
    <skos:prefLabel xml:lang="en">Food machinery industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18413-6">
    <skos:prefLabel xml:lang="de">Nahrungsmittelmaschinenbau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10106-6">
    <skos:prefLabel xml:lang="en">Social welfare function</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10106-6">
    <skos:prefLabel xml:lang="de">Soziale Wohlfahrtsfunktion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16395-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16522-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70429">
    <rdfs:label xml:lang="en">Southern Africa</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70429">
    <rdfs:label xml:lang="de">Südliches Afrika</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10029-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10852-6">
    <skos:related rdf:resource="descriptor/10882-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10852-6">
    <skos:related rdf:resource="descriptor/10880-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10852-6">
    <skos:related rdf:resource="descriptor/10604-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10852-6">
    <skos:related rdf:resource="descriptor/10606-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10852-6">
    <skos:related rdf:resource="descriptor/10835-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10852-6">
    <skos:related rdf:resource="descriptor/10686-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10852-6">
    <skos:related rdf:resource="descriptor/10859-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13974-5">
    <skos:altLabel xml:lang="en">Servants</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13974-5">
    <skos:altLabel xml:lang="de">Kindermädchen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13974-5">
    <skos:altLabel xml:lang="de">Hauswirtschaftsberufe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13974-5">
    <skos:altLabel xml:lang="de">Hauswirtschaftliche Berufe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13974-5">
    <skos:altLabel xml:lang="de">Haushaltspersonal</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13974-5">
    <skos:altLabel xml:lang="de">Hausangestellte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13974-5">
    <skos:altLabel xml:lang="de">Dienstmädchen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13974-5">
    <skos:altLabel xml:lang="de">Dienstboten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13974-5">
    <skos:altLabel xml:lang="de">Diener</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19321-6">
    <skos:prefLabel xml:lang="en">Financial liberalization</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19321-6">
    <skos:prefLabel xml:lang="de">Kapitalmarktliberalisierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18525-2">
    <skos:related rdf:resource="descriptor/18065-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10041-1">
    <skos:altLabel xml:lang="de">Politische Ökonomie der Klassiker</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10041-1">
    <skos:altLabel xml:lang="de">Klassische Schule</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71001">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.06.01.01.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20449-5">
    <skos:prefLabel xml:lang="en">International works council</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20449-5">
    <skos:prefLabel xml:lang="de">Internationaler Betriebsrat</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12800-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16970-5">
    <skos:prefLabel xml:lang="en">St. Gallen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16970-5">
    <skos:prefLabel xml:lang="de">Sankt Gallen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19251-1">
    <skos:altLabel xml:lang="de">Innovationsförderung des Staates</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14336-4">
    <skos:narrower rdf:resource="descriptor/14052-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14336-4">
    <skos:narrower rdf:resource="descriptor/14345-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14336-4">
    <skos:narrower rdf:resource="descriptor/21110-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14336-4">
    <skos:narrower rdf:resource="descriptor/14341-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14336-4">
    <skos:narrower rdf:resource="descriptor/14339-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14336-4">
    <skos:narrower rdf:resource="descriptor/14381-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14336-4">
    <skos:narrower rdf:resource="descriptor/12681-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14336-4">
    <skos:narrower rdf:resource="descriptor/14337-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14336-4">
    <skos:narrower rdf:resource="descriptor/12736-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14336-4">
    <skos:narrower rdf:resource="descriptor/14334-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14336-4">
    <skos:narrower rdf:resource="descriptor/12693-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14336-4">
    <skos:narrower rdf:resource="descriptor/14331-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14336-4">
    <skos:narrower rdf:resource="descriptor/14335-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14336-4">
    <skos:narrower rdf:resource="descriptor/14329-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16524-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16522-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16014-2">
    <skos:scopeNote xml:lang="de">Haftung des Staates für Schäden, die durch Verletzung der Amtspflicht entstanden sind</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13398-1">
    <skos:prefLabel xml:lang="en">Household</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13398-1">
    <skos:prefLabel xml:lang="de">Privater Haushalt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70926">
    <rdfs:label xml:lang="en">Transport policy</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70926">
    <rdfs:label xml:lang="de">Verkehrspolitik</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11379-1">
    <skos:narrower rdf:resource="descriptor/18871-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10806-6">
    <skos:related rdf:resource="descriptor/10817-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10806-6">
    <skos:related rdf:resource="descriptor/10807-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10806-6">
    <skos:related rdf:resource="descriptor/10805-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10806-6">
    <skos:related rdf:resource="descriptor/18540-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19191-0">
    <skos:related rdf:resource="descriptor/18313-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19152-3">
    <skos:broader rdf:resource="thsys/70485"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19152-3">
    <skos:broader rdf:resource="descriptor/12085-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12920-5">
    <skos:narrower rdf:resource="descriptor/13001-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12920-5">
    <skos:narrower rdf:resource="descriptor/12984-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12920-5">
    <skos:narrower rdf:resource="descriptor/19047-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13160-6">
    <skos:editorialNote xml:lang="de">Möglichst spezifischer indexieren</skos:editorialNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18084-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15640-3">
    <skos:related rdf:resource="descriptor/17017-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15640-3">
    <skos:related rdf:resource="descriptor/15678-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15640-3">
    <skos:related rdf:resource="descriptor/13393-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15640-3">
    <skos:related rdf:resource="descriptor/19735-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14469-6">
    <skos:broader rdf:resource="thsys/70135"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14469-6">
    <skos:broader rdf:resource="descriptor/18252-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14469-6">
    <skos:broader rdf:resource="descriptor/14473-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18011-5">
    <skos:related rdf:resource="descriptor/10567-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18011-5">
    <skos:related rdf:resource="descriptor/10986-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18011-5">
    <skos:related rdf:resource="descriptor/10894-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18011-5">
    <skos:related rdf:resource="descriptor/15963-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18011-5">
    <skos:related rdf:resource="descriptor/10377-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11350-4">
    <skos:prefLabel xml:lang="en">Social partners</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11350-4">
    <skos:prefLabel xml:lang="de">Sozialpartner</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13301-0">
    <skos:related rdf:resource="descriptor/13612-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13301-0">
    <skos:related rdf:resource="descriptor/10016-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13301-0">
    <skos:related rdf:resource="descriptor/18126-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13301-0">
    <skos:related rdf:resource="descriptor/10030-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13301-0">
    <skos:related rdf:resource="descriptor/18013-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13301-0">
    <skos:related rdf:resource="descriptor/16601-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12656-3">
    <skos:related rdf:resource="descriptor/12755-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12656-3">
    <skos:related rdf:resource="descriptor/18104-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12656-3">
    <skos:related rdf:resource="descriptor/15571-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12656-3">
    <skos:related rdf:resource="descriptor/19118-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12656-3">
    <skos:related rdf:resource="descriptor/12677-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12656-3">
    <skos:related rdf:resource="descriptor/12651-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12656-3">
    <skos:related rdf:resource="descriptor/12035-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19029-2">
    <skos:broader rdf:resource="thsys/70582"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19398-6">
    <skos:altLabel xml:lang="de">Staatlicher Grundbesitz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19398-6">
    <skos:altLabel xml:lang="de">Staatliche Immobilien</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19398-6">
    <skos:altLabel xml:lang="de">Öffentliches Gebäude</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13535-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70144">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.09</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12825-6">
    <skos:prefLabel xml:lang="en">Radio advertising</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12825-6">
    <skos:prefLabel xml:lang="de">Rundfunkwerbung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14973-2">
    <skos:related rdf:resource="descriptor/19018-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14973-2">
    <skos:related rdf:resource="descriptor/14928-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14973-2">
    <skos:related rdf:resource="descriptor/13151-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16540-2">
    <skos:related rdf:resource="descriptor/18914-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16540-2">
    <skos:related rdf:resource="descriptor/16539-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16540-2">
    <skos:related rdf:resource="descriptor/13416-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16540-2">
    <skos:related rdf:resource="descriptor/16475-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11034-0">
    <skos:related rdf:resource="descriptor/16295-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11034-0">
    <skos:related rdf:resource="descriptor/11051-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13351-6">
    <skos:broader rdf:resource="thsys/70822"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13351-6">
    <skos:broader rdf:resource="descriptor/19475-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71029">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.01.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11661-1">
    <skos:broader rdf:resource="thsys/71076"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11661-1">
    <skos:broader rdf:resource="thsys/71074"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11661-1">
    <skos:broader rdf:resource="descriptor/11547-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16120-3">
    <skos:related rdf:resource="descriptor/12903-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70956">
    <skos:prefLabel xml:lang="en">W.10.02  Commercial management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70956">
    <skos:prefLabel xml:lang="de">W.10.02  Handelsbetriebslehre</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16402-0">
    <skos:related rdf:resource="descriptor/11067-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16402-0">
    <skos:related rdf:resource="descriptor/11060-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16402-0">
    <skos:related rdf:resource="descriptor/16373-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18632-1">
    <skos:related rdf:resource="descriptor/18633-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18632-1">
    <skos:related rdf:resource="descriptor/11876-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18632-1">
    <skos:related rdf:resource="descriptor/18610-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71061">
    <rdfs:label xml:lang="en">Labour market policy</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71061">
    <rdfs:label xml:lang="de">Arbeitsmarktpolitik</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15086-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73376">
    <skos:narrower rdf:resource="descriptor/19702-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73376">
    <skos:narrower rdf:resource="descriptor/16353-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73376">
    <skos:narrower rdf:resource="descriptor/16357-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73376">
    <skos:narrower rdf:resource="descriptor/15983-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73376">
    <skos:narrower rdf:resource="descriptor/19701-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73376">
    <skos:narrower rdf:resource="descriptor/19699-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73376">
    <skos:narrower rdf:resource="descriptor/16355-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73376">
    <skos:narrower rdf:resource="descriptor/19700-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73376">
    <skos:narrower rdf:resource="descriptor/16354-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73376">
    <skos:narrower rdf:resource="descriptor/16269-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73376">
    <skos:narrower rdf:resource="descriptor/16311-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73376">
    <skos:narrower rdf:resource="descriptor/19698-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73376">
    <skos:narrower rdf:resource="descriptor/16287-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73376">
    <skos:narrower rdf:resource="descriptor/16207-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73376">
    <skos:narrower rdf:resource="descriptor/16275-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73376">
    <skos:narrower rdf:resource="descriptor/19975-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73376">
    <skos:narrower rdf:resource="descriptor/16226-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73376">
    <skos:narrower rdf:resource="descriptor/16272-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73376">
    <skos:narrower rdf:resource="descriptor/13661-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73376">
    <skos:narrower rdf:resource="descriptor/13662-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73376">
    <skos:narrower rdf:resource="descriptor/16265-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73376">
    <skos:narrower rdf:resource="descriptor/16193-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73376">
    <skos:narrower rdf:resource="descriptor/15798-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73376">
    <skos:narrower rdf:resource="descriptor/19697-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73376">
    <skos:narrower rdf:resource="descriptor/15961-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73376">
    <skos:narrower rdf:resource="descriptor/20855-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73376">
    <skos:narrower rdf:resource="descriptor/20842-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73376">
    <skos:narrower rdf:resource="descriptor/16633-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73376">
    <skos:narrower rdf:resource="descriptor/16349-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73376">
    <skos:narrower rdf:resource="descriptor/16266-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73376">
    <skos:narrower rdf:resource="descriptor/16301-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71016">
    <rdfs:label xml:lang="en">Power industry</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71016">
    <rdfs:label xml:lang="de">Energiewirtschaft</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16397-2">
    <skos:prefLabel xml:lang="en">Anarchy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16397-2">
    <skos:prefLabel xml:lang="de">Anarchie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11596-0">
    <skos:altLabel xml:lang="en">Budget</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11596-0">
    <skos:altLabel xml:lang="de">Staatshaushalt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11596-0">
    <skos:altLabel xml:lang="de">Staatsbudget</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11596-0">
    <skos:altLabel xml:lang="de">Haushaltspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11596-0">
    <skos:altLabel xml:lang="de">Haushaltsplan der Öffentlichen Hand</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11596-0">
    <skos:altLabel xml:lang="de">Haushalt des Staates</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11596-0">
    <skos:altLabel xml:lang="de">Etat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11596-0">
    <skos:altLabel xml:lang="de">Bundeshaushaltsplan</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11596-0">
    <skos:altLabel xml:lang="de">Bundeshaushalt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11596-0">
    <skos:altLabel xml:lang="de">Budgetpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18749-1">
    <skos:broader rdf:resource="thsys/71066"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18749-1">
    <skos:broader rdf:resource="thsys/70888"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18749-1">
    <skos:broader rdf:resource="descriptor/12029-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15740-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71059">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71092">
    <rdfs:label xml:lang="en">European integration</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71092">
    <rdfs:label xml:lang="de">Europäische Integration</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11598-3">
    <skos:related rdf:resource="descriptor/12492-0"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id266">
    <rdfs:seeAlso rdf:resource="descriptor/18843-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19336-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12914-0">
    <skos:scopeNote xml:lang="de">Bau von Produktionsanlagen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19098-4">
    <skos:related rdf:resource="descriptor/19097-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19098-4">
    <skos:related rdf:resource="descriptor/10011-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11149-4">
    <skos:altLabel xml:lang="x-other">Ejido</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11149-4">
    <skos:altLabel xml:lang="de">Volkseigenes Gut</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11149-4">
    <skos:altLabel xml:lang="de">VEG (Volkseigenes Gut)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11149-4">
    <skos:altLabel xml:lang="de">Sowchos</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11149-4">
    <skos:altLabel xml:lang="de">LPG (Landwirtschaftliche Produktionsgenossenschaft)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11149-4">
    <skos:altLabel xml:lang="de">Landwirtschaftlicher Kollektivbetrieb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11149-4">
    <skos:altLabel xml:lang="de">Landwirtschaftliche Produktionsgenossenschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11149-4">
    <skos:altLabel xml:lang="de">Kolchos</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11149-4">
    <skos:altLabel xml:lang="de">Kibbuz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11322-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19585-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18209-5">
    <skos:prefLabel xml:lang="en">Journal</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18209-5">
    <skos:prefLabel xml:lang="de">Fachzeitschrift</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13546-5">
    <skos:narrower rdf:resource="descriptor/18342-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16048-6">
    <skos:related rdf:resource="descriptor/18108-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16048-6">
    <skos:related rdf:resource="descriptor/18365-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16048-6">
    <skos:related rdf:resource="descriptor/16090-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16048-6">
    <skos:related rdf:resource="descriptor/10734-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16048-6">
    <skos:related rdf:resource="descriptor/19336-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16048-6">
    <skos:related rdf:resource="descriptor/18109-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16048-6">
    <skos:related rdf:resource="descriptor/12331-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19622-6">
    <skos:related rdf:resource="descriptor/15755-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19622-6">
    <skos:related rdf:resource="descriptor/15888-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14245-0">
    <skos:broader rdf:resource="thsys/70348"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14245-0">
    <skos:broader rdf:resource="descriptor/14236-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13259-1">
    <skos:related rdf:resource="descriptor/12693-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19074-4">
    <skos:prefLabel xml:lang="en">Operating time</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19074-4">
    <skos:prefLabel xml:lang="de">Betriebszeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71105">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18422-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10687-2">
    <skos:related rdf:resource="descriptor/18181-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10687-2">
    <skos:related rdf:resource="descriptor/16084-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19654-0">
    <skos:broader rdf:resource="thsys/71036"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19654-0">
    <skos:broader rdf:resource="thsys/70880"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19654-0">
    <skos:broader rdf:resource="descriptor/15598-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11783-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11983-0">
    <skos:broader rdf:resource="thsys/71064"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11983-0">
    <skos:broader rdf:resource="descriptor/19468-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18540-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14735-4">
    <skos:related rdf:resource="descriptor/18263-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13793-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10162-3">
    <skos:related rdf:resource="descriptor/19210-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10162-3">
    <skos:related rdf:resource="descriptor/10181-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12027-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11711-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12020-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12040-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71082">
    <rdfs:label xml:lang="en">Fiscal policy</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71082">
    <rdfs:label xml:lang="de">Finanzpolitik</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70870">
    <rdfs:label xml:lang="de">Gebietskörperschaften</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10140-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12867-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11264-4">
    <skos:related rdf:resource="descriptor/15107-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11264-4">
    <skos:related rdf:resource="descriptor/11297-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11264-4">
    <skos:related rdf:resource="descriptor/18168-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11264-4">
    <skos:related rdf:resource="descriptor/11182-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11183-4">
    <skos:prefLabel xml:lang="en">Labour market</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11183-4">
    <skos:prefLabel xml:lang="de">Arbeitsmarkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15897-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19492-0">
    <skos:altLabel xml:lang="en">Interregional labor mobility</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19492-0">
    <skos:altLabel xml:lang="de">Interregionale Arbeitsmobilität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15656-2">
    <skos:broader rdf:resource="thsys/70847"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15656-2">
    <skos:broader rdf:resource="thsys/70143"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15656-2">
    <skos:broader rdf:resource="descriptor/19601-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15298-2">
    <skos:narrower rdf:resource="descriptor/19575-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15752-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70013">
    <rdfs:label xml:lang="de">Spezielle Soziologien und sozialwissenschaftliche Teilbereiche</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12182-1">
    <skos:prefLabel xml:lang="en">Limited liability company</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12182-1">
    <skos:prefLabel xml:lang="de">GmbH</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14421-6">
    <skos:related rdf:resource="descriptor/14423-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14421-6">
    <skos:related rdf:resource="descriptor/13246-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14421-6">
    <skos:related rdf:resource="descriptor/13575-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14421-6">
    <skos:related rdf:resource="descriptor/13057-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14421-6">
    <skos:related rdf:resource="descriptor/18418-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10928-6">
    <skos:prefLabel xml:lang="en">Market entry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10928-6">
    <skos:prefLabel xml:lang="de">Markteintritt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15835-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15115-1">
    <skos:broader rdf:resource="thsys/72183"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70850">
    <skos:broader rdf:resource="thsys/70853"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10387-0">
    <skos:related rdf:resource="descriptor/11490-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10387-0">
    <skos:related rdf:resource="descriptor/10439-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10387-0">
    <skos:related rdf:resource="descriptor/10404-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10387-0">
    <skos:related rdf:resource="descriptor/10353-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10387-0">
    <skos:related rdf:resource="descriptor/10347-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10387-0">
    <skos:related rdf:resource="descriptor/10339-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10387-0">
    <skos:related rdf:resource="descriptor/10338-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10387-0">
    <skos:related rdf:resource="descriptor/11494-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10387-0">
    <skos:related rdf:resource="descriptor/10382-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10387-0">
    <skos:related rdf:resource="descriptor/10380-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10387-0">
    <skos:related rdf:resource="descriptor/10381-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10387-0">
    <skos:related rdf:resource="descriptor/10377-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14131-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10013-6">
    <skos:altLabel xml:lang="de">Nachfragestruktur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10013-6">
    <skos:altLabel xml:lang="de">Bedarf</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18694-0">
    <skos:narrower rdf:resource="descriptor/19359-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18694-0">
    <skos:narrower rdf:resource="descriptor/18695-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13624-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12667-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10411-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13268-0">
    <skos:prefLabel xml:lang="en">Radio communication</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13268-0">
    <skos:prefLabel xml:lang="de">Funktechnik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12069-4">
    <skos:prefLabel xml:lang="en">Firm performance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12069-4">
    <skos:prefLabel xml:lang="de">Unternehmensentwicklung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18144-0">
    <skos:prefLabel xml:lang="en">Housework</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18144-0">
    <skos:prefLabel xml:lang="de">Hausarbeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11771-1">
    <skos:related rdf:resource="descriptor/16019-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11771-1">
    <skos:related rdf:resource="descriptor/11779-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11771-1">
    <skos:related rdf:resource="descriptor/11040-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16712-6">
    <skos:broader rdf:resource="thsys/70141"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16712-6">
    <skos:broader rdf:resource="descriptor/16745-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19391-6">
    <skos:broader rdf:resource="thsys/70959"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19391-6">
    <skos:broader rdf:resource="descriptor/10012-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14839-6">
    <skos:altLabel xml:lang="de">Zelt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14839-6">
    <skos:altLabel xml:lang="de">Segel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14839-6">
    <skos:altLabel xml:lang="de">Plane</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19600-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18395-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19499-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15662-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id291">
    <rdfs:seeAlso rdf:resource="descriptor/11211-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13846-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16551-4">
    <skos:prefLabel xml:lang="en">State policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16551-4">
    <skos:prefLabel xml:lang="de">Landespolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19673-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18013-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10162-3">
    <skos:altLabel xml:lang="en">Stated preferences</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10162-3">
    <skos:altLabel xml:lang="de">Offenbarte Präferenz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10162-3">
    <skos:altLabel xml:lang="de">Bekundete Präferenzen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10798-0">
    <skos:altLabel xml:lang="de">Kapitalbesitz im Ausland</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10798-0">
    <skos:altLabel xml:lang="de">Kapitalanlage im Ausland</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19424-3">
    <skos:related rdf:resource="descriptor/13824-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19424-3">
    <skos:related rdf:resource="descriptor/13828-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19424-3">
    <skos:related rdf:resource="descriptor/11678-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16645-2">
    <skos:broader rdf:resource="thsys/73395"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16645-2">
    <skos:broader rdf:resource="descriptor/16639-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14197-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71014">
    <rdfs:label xml:lang="de">Regenerative Energiewirtschaft</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19648-2">
    <skos:prefLabel xml:lang="en">Ethnic Germans</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19648-2">
    <skos:prefLabel xml:lang="de">Aussiedler</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16512-0">
    <skos:related rdf:resource="descriptor/16615-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16512-0">
    <skos:related rdf:resource="descriptor/16308-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16512-0">
    <skos:related rdf:resource="descriptor/16421-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16512-0">
    <skos:related rdf:resource="descriptor/16513-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16512-0">
    <skos:related rdf:resource="descriptor/16264-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16512-0">
    <skos:related rdf:resource="descriptor/16299-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16512-0">
    <skos:related rdf:resource="descriptor/16661-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16512-0">
    <skos:related rdf:resource="descriptor/16486-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16512-0">
    <skos:related rdf:resource="descriptor/16515-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16512-0">
    <skos:related rdf:resource="descriptor/15918-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16512-0">
    <skos:related rdf:resource="descriptor/15931-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16512-0">
    <skos:related rdf:resource="descriptor/15713-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15762-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19225-2">
    <skos:prefLabel xml:lang="en">Relative prices</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19225-2">
    <skos:prefLabel xml:lang="de">Relativer Preis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12441-3">
    <skos:narrower rdf:resource="descriptor/12543-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18157-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19654-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18754-1">
    <skos:prefLabel xml:lang="en">Public housing policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18754-1">
    <skos:prefLabel xml:lang="de">Soziale Wohnungspolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15494-2">
    <skos:broader rdf:resource="thsys/73103"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15494-2">
    <skos:broader rdf:resource="thsys/71035"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15494-2">
    <skos:broader rdf:resource="descriptor/15469-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20409-3">
    <skos:altLabel xml:lang="x-other">Narcoéconomie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20409-3">
    <skos:altLabel xml:lang="x-other">Narcoeconomie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20409-3">
    <skos:altLabel xml:lang="de">Drogenökonomie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id196">
    <rdf:value xml:lang="de">Für das militärische Nachschubwesen</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17013-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19436-3">
    <skos:related rdf:resource="descriptor/11596-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11452-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13818-5">
    <skos:broader rdf:resource="thsys/70892"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13818-5">
    <skos:broader rdf:resource="thsys/70867"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13818-5">
    <skos:broader rdf:resource="descriptor/18810-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20402-3">
    <skos:prefLabel xml:lang="en">Network</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20402-3">
    <skos:prefLabel xml:lang="de">Netzwerk</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16454-2">
    <skos:broader rdf:resource="thsys/73365"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13660-0">
    <skos:prefLabel xml:lang="en">Media law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13660-0">
    <skos:prefLabel xml:lang="de">Medienrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17485-0">
    <skos:prefLabel xml:lang="en">Ukraine</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17485-0">
    <skos:prefLabel xml:lang="de">Ukraine</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20485-1">
    <skos:prefLabel xml:lang="en">False self-employment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20485-1">
    <skos:prefLabel xml:lang="de">Scheinselbstständigkeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70899">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11822-3">
    <skos:related rdf:resource="descriptor/11820-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11822-3">
    <skos:related rdf:resource="descriptor/11823-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11822-3">
    <skos:related rdf:resource="descriptor/16052-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10895-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14122-2">
    <skos:broader rdf:resource="thsys/73302"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14122-2">
    <skos:broader rdf:resource="thsys/70467"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14122-2">
    <skos:broader rdf:resource="descriptor/14129-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14122-2">
    <skos:broader rdf:resource="descriptor/14931-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19789-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13577-1">
    <skos:broader rdf:resource="thsys/70943"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13577-1">
    <skos:broader rdf:resource="descriptor/10470-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70996">
    <rdfs:label xml:lang="en">Electrical industry</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70996">
    <rdfs:label xml:lang="de">Elektroindustrie</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13023-2">
    <skos:broader rdf:resource="thsys/71008"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13023-2">
    <skos:broader rdf:resource="descriptor/13020-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10109-0">
    <skos:altLabel xml:lang="de">Arrow-Unmöglichkeitstheorem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18028-2">
    <skos:prefLabel xml:lang="en">Special interest magazine</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18028-2">
    <skos:prefLabel xml:lang="de">Spezialzeitschrift</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12801-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core">
    <ns0:contributor xmlns:ns0="http://purl.org/dc/elements/1.1/">Participants in W3C's Semantic Web Deployment Working Group.</ns0:contributor>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core">
    <ns0:contributor xmlns:ns0="http://purl.org/dc/elements/1.1/">Nikki Rogers</ns0:contributor>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core">
    <ns0:contributor xmlns:ns0="http://purl.org/dc/elements/1.1/">Dave Beckett</ns0:contributor>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12214-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12533-5">
    <skos:altLabel xml:lang="de">Kostenträger</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13060-3">
    <skos:altLabel xml:lang="de">Nutzfahrzeugbau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14192-2">
    <skos:narrower rdf:resource="descriptor/14193-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14192-2">
    <skos:narrower rdf:resource="descriptor/14200-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14192-2">
    <skos:narrower rdf:resource="descriptor/14638-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14192-2">
    <skos:narrower rdf:resource="descriptor/14198-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14192-2">
    <skos:narrower rdf:resource="descriptor/14197-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14192-2">
    <skos:narrower rdf:resource="descriptor/14196-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12872-4">
    <skos:prefLabel xml:lang="en">Flags of convenience</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12872-4">
    <skos:prefLabel xml:lang="de">Flaggenwechsel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70005">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70342">
    <rdfs:label xml:lang="en">Processed food and vegetables</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70342">
    <rdfs:label xml:lang="de">Verarbeitetes Obst und Gemüse</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12955-0">
    <skos:broader rdf:resource="thsys/71025"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12955-0">
    <skos:broader rdf:resource="descriptor/12945-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19009-1">
    <skos:altLabel xml:lang="de">Intrige</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13306-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15922-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14632-0">
    <skos:altLabel xml:lang="en">Hazardous substances for the environment</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14632-0">
    <skos:altLabel xml:lang="de">Umwelttoxischer Stoff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14632-0">
    <skos:altLabel xml:lang="de">Umweltgefährdender Stoff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19856-2">
    <skos:related rdf:resource="descriptor/12901-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15712-4">
    <skos:altLabel xml:lang="en">Social interaction</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15712-4">
    <skos:altLabel xml:lang="de">Zwischenmenschliche Beziehungen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15712-4">
    <skos:altLabel xml:lang="de">Soziale Kontakte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15712-4">
    <skos:altLabel xml:lang="de">Soziale Interaktionsforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15712-4">
    <skos:altLabel xml:lang="de">Soziale Interaktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15712-4">
    <skos:altLabel xml:lang="de">Interpersonelle Beziehungen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15712-4">
    <skos:altLabel xml:lang="de">Interpersonale Beziehungen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19741-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73359">
    <rdfs:label xml:lang="de">Verhalten und soziale Beziehungen</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13587-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70160">
    <skos:narrower rdf:resource="descriptor/19842-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70160">
    <skos:narrower rdf:resource="descriptor/16041-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70160">
    <skos:narrower rdf:resource="descriptor/15981-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70160">
    <skos:narrower rdf:resource="descriptor/18905-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70160">
    <skos:narrower rdf:resource="descriptor/19852-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70160">
    <skos:narrower rdf:resource="descriptor/16007-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70160">
    <skos:narrower rdf:resource="descriptor/19323-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70160">
    <skos:narrower rdf:resource="descriptor/18809-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70160">
    <skos:narrower rdf:resource="descriptor/16002-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70160">
    <skos:narrower rdf:resource="descriptor/15961-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70160">
    <skos:narrower rdf:resource="descriptor/19849-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70160">
    <skos:narrower rdf:resource="thsys/73360"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70160">
    <skos:narrower rdf:resource="thsys/73361"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70160">
    <skos:narrower rdf:resource="thsys/73334"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12969-3">
    <skos:prefLabel xml:lang="en">Aquaculture</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12969-3">
    <skos:prefLabel xml:lang="de">Aquakultur</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24661-1">
    <skos:altLabel xml:lang="de">Veränderungsmanagement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18369-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11184-2">
    <skos:altLabel xml:lang="en">Occupational mobility</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11184-2">
    <skos:altLabel xml:lang="en">Labour turnover</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11184-2">
    <skos:altLabel xml:lang="en">Labor turnover</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11184-2">
    <skos:altLabel xml:lang="en">Labor mobility</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11184-2">
    <skos:altLabel xml:lang="en">Job turnover</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11184-2">
    <skos:altLabel xml:lang="de">Versetzung im Betrieb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11184-2">
    <skos:altLabel xml:lang="de">Stellenwechsel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11184-2">
    <skos:altLabel xml:lang="de">Fluktuation am Arbeitsmarkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11184-2">
    <skos:altLabel xml:lang="de">Betriebswechsel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11184-2">
    <skos:altLabel xml:lang="de">Berufswechsel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11184-2">
    <skos:altLabel xml:lang="de">Berufsmobilität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11184-2">
    <skos:altLabel xml:lang="de">Arbeitsplatzwechsel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11184-2">
    <skos:altLabel xml:lang="de">Arbeitsplatzdynamik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11184-2">
    <skos:altLabel xml:lang="de">Arbeitsmarktfluktuation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11184-2">
    <skos:altLabel xml:lang="de">Arbeitsmarktbewegung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17782-1">
    <skos:altLabel xml:lang="de">Republik Guatemala</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18063-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13502-4">
    <skos:scopeNote xml:lang="de">Das Recht, ausländischen Verkehrsunternehmen die Teilnahme am Inlandverkehr zu verweigern.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13152-5">
    <skos:prefLabel xml:lang="en">Spirit industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13152-5">
    <skos:prefLabel xml:lang="de">Spirituosenindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70348">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">P.06</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19429-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10252-2">
    <skos:altLabel xml:lang="de">Neoklassische Theorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70020">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13665-4">
    <skos:scopeNote xml:lang="de">Für Veranstalter und Programme BENUTZE die entsprechenden Deskriptoren.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10094-1">
    <skos:related rdf:resource="descriptor/12535-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10094-1">
    <skos:related rdf:resource="descriptor/10097-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10094-1">
    <skos:related rdf:resource="descriptor/11065-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10858-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14149-3">
    <skos:broader rdf:resource="thsys/70465"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19832-2">
    <skos:prefLabel xml:lang="en">Groupware</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19832-2">
    <skos:prefLabel xml:lang="de">Groupware</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20506-5">
    <skos:related rdf:resource="descriptor/15396-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12025-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19770-5">
    <skos:prefLabel xml:lang="en">Nobel prize</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19770-5">
    <skos:prefLabel xml:lang="de">Nobelpreis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19506-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12297-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15825-5">
    <skos:altLabel xml:lang="en">Trait theory of leadership</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15825-5">
    <skos:altLabel xml:lang="en">Personality approach</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15825-5">
    <skos:altLabel xml:lang="de">Personalführungsgrundsätze</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15825-5">
    <skos:altLabel xml:lang="de">Führungsverhalten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15825-5">
    <skos:altLabel xml:lang="de">Führungsstil</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15825-5">
    <skos:altLabel xml:lang="de">Führungsforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18987-6">
    <skos:broader rdf:resource="thsys/71019"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18987-6">
    <skos:broader rdf:resource="descriptor/12065-5"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id141">
    <rdfs:seeAlso rdf:resource="descriptor/18481-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13436-5">
    <skos:prefLabel xml:lang="en">Farmers</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13436-5">
    <skos:prefLabel xml:lang="de">Landwirte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10842-2">
    <skos:broader rdf:resource="thsys/70179"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10842-2">
    <skos:broader rdf:resource="thsys/70101"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10842-2">
    <skos:broader rdf:resource="descriptor/10895-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11253-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18755-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11305-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18924-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15054-2">
    <skos:narrower rdf:resource="descriptor/18171-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18374-4">
    <skos:related rdf:resource="descriptor/19192-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18374-4">
    <skos:related rdf:resource="descriptor/12990-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18374-4">
    <skos:related rdf:resource="descriptor/14037-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18374-4">
    <skos:related rdf:resource="descriptor/14200-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12923-6">
    <skos:broader rdf:resource="thsys/70963"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16699-0">
    <skos:altLabel xml:lang="en">Intergenerational aspects</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16699-0">
    <skos:altLabel xml:lang="de">Generationsbeziehungen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16699-0">
    <skos:altLabel xml:lang="de">Generationenvertrag</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16699-0">
    <skos:altLabel xml:lang="de">Generationenverhältnis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16699-0">
    <skos:altLabel xml:lang="de">Generationenproblem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16699-0">
    <skos:altLabel xml:lang="de">Generationenkonflikt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18156-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70305">
    <skos:broader rdf:resource="thsys/70055"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10225-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14919-1">
    <skos:broader rdf:resource="thsys/70031"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14919-1">
    <skos:broader rdf:resource="descriptor/14915-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17739-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13601-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11660-3">
    <skos:related rdf:resource="descriptor/13398-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11660-3">
    <skos:related rdf:resource="descriptor/11669-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11660-3">
    <skos:related rdf:resource="descriptor/18948-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11660-3">
    <skos:related rdf:resource="descriptor/11564-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15475-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12295-2">
    <skos:altLabel xml:lang="de">Profitrate</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12295-2">
    <skos:altLabel xml:lang="de">Kapitalrentabilität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19097-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10438-2">
    <skos:related rdf:resource="descriptor/10439-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10438-2">
    <skos:related rdf:resource="descriptor/11572-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10438-2">
    <skos:related rdf:resource="descriptor/16102-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10438-2">
    <skos:related rdf:resource="descriptor/10008-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10438-2">
    <skos:related rdf:resource="descriptor/12224-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13459-0">
    <skos:broader rdf:resource="thsys/70956"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13459-0">
    <skos:broader rdf:resource="descriptor/13460-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13459-0">
    <skos:broader rdf:resource="descriptor/12041-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70889">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19446-0">
    <skos:narrower rdf:resource="descriptor/11940-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15983-1">
    <skos:broader rdf:resource="thsys/73376"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15983-1">
    <skos:broader rdf:resource="thsys/73361"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15983-1">
    <skos:broader rdf:resource="descriptor/16017-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15117-4">
    <skos:scopeNote xml:lang="de">Abweichung eines Schätzwertes von dem zu schätzenden Parameter; Fehlentscheidungen bei statistischen Tests; Schätz- und Messgenauigkeiten.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14854-3">
    <skos:prefLabel xml:lang="en">Man-made fibre</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14854-3">
    <skos:prefLabel xml:lang="de">Chemiefaser</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="http://zbw.eu/namespaces/skos-extensions/indexedItem">
    <rdfs:subPropertyOf rdf:resource="http://www.w3.org/2000/01/rdf-schema#seeAlso"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13063-4">
    <skos:related rdf:resource="descriptor/13255-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13063-4">
    <skos:related rdf:resource="descriptor/14428-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13063-4">
    <skos:related rdf:resource="descriptor/13574-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18929-6">
    <skos:related rdf:resource="descriptor/13352-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18929-6">
    <skos:related rdf:resource="descriptor/18934-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18929-6">
    <skos:related rdf:resource="descriptor/18935-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13374-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16079-2">
    <skos:broader rdf:resource="thsys/73391"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16079-2">
    <skos:broader rdf:resource="thsys/71107"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16079-2">
    <skos:broader rdf:resource="descriptor/16080-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15954-1">
    <skos:broader rdf:resource="thsys/70572"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15954-1">
    <skos:broader rdf:resource="descriptor/15709-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24623-2">
    <skos:prefLabel xml:lang="en">Reservation wage</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24623-2">
    <skos:prefLabel xml:lang="de">Anspruchslohn</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12012-5">
    <skos:broader rdf:resource="thsys/71063"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12012-5">
    <skos:broader rdf:resource="descriptor/11716-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70403">
    <skos:prefLabel xml:lang="de">P.01.03  Tiere</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16421-3">
    <skos:altLabel xml:lang="de">Regionaler Partikularismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16421-3">
    <skos:altLabel xml:lang="de">Regionale Kooperation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16421-3">
    <skos:altLabel xml:lang="de">Regionale Integration</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16421-3">
    <skos:altLabel xml:lang="de">Regionale Identität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16421-3">
    <skos:altLabel xml:lang="de">Partikularismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16421-3">
    <skos:altLabel xml:lang="de">Pan-Ideen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10111-6">
    <skos:prefLabel xml:lang="en">Polluter-pays principle</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10111-6">
    <skos:prefLabel xml:lang="de">Verursacherprinzip</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13128-2">
    <skos:related rdf:resource="descriptor/13142-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13128-2">
    <skos:related rdf:resource="descriptor/18295-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13128-2">
    <skos:related rdf:resource="descriptor/13137-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13128-2">
    <skos:related rdf:resource="descriptor/14899-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18549-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13809-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17465-6">
    <skos:broader rdf:resource="thsys/70174"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17465-6">
    <skos:broader rdf:resource="descriptor/19077-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17465-6">
    <skos:broader rdf:resource="descriptor/19078-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12971-2">
    <skos:broader rdf:resource="thsys/71025"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12971-2">
    <skos:broader rdf:resource="descriptor/12955-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13181-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70227">
    <skos:narrower rdf:resource="descriptor/18231-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70227">
    <skos:narrower rdf:resource="descriptor/14189-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70227">
    <skos:narrower rdf:resource="descriptor/18163-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70227">
    <skos:narrower rdf:resource="descriptor/14186-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70227">
    <skos:narrower rdf:resource="descriptor/18024-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70227">
    <skos:narrower rdf:resource="descriptor/13003-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70227">
    <skos:narrower rdf:resource="descriptor/15628-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70227">
    <skos:narrower rdf:resource="descriptor/16001-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70227">
    <skos:narrower rdf:resource="descriptor/14182-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70227">
    <skos:narrower rdf:resource="descriptor/14180-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70227">
    <skos:narrower rdf:resource="descriptor/18235-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70227">
    <skos:narrower rdf:resource="descriptor/12975-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70227">
    <skos:narrower rdf:resource="descriptor/18354-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70227">
    <skos:narrower rdf:resource="descriptor/14176-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70227">
    <skos:narrower rdf:resource="descriptor/19487-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70227">
    <skos:narrower rdf:resource="descriptor/18351-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70227">
    <skos:narrower rdf:resource="descriptor/11813-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70227">
    <skos:narrower rdf:resource="descriptor/15993-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70227">
    <skos:narrower rdf:resource="descriptor/18174-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70227">
    <skos:narrower rdf:resource="descriptor/18350-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70227">
    <skos:narrower rdf:resource="descriptor/18996-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70227">
    <skos:narrower rdf:resource="descriptor/18349-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70227">
    <skos:narrower rdf:resource="descriptor/18989-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70227">
    <skos:narrower rdf:resource="descriptor/18348-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70227">
    <skos:narrower rdf:resource="descriptor/18347-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70227">
    <skos:narrower rdf:resource="descriptor/14175-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70227">
    <skos:narrower rdf:resource="descriptor/19838-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70227">
    <skos:narrower rdf:resource="thsys/70539"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13197-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19849-6">
    <skos:altLabel xml:lang="de">Rechtsverfolgungskosten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19849-6">
    <skos:altLabel xml:lang="de">Prozesskosten (Gericht)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19849-6">
    <skos:altLabel xml:lang="de">Prozeßkosten (Gericht)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19849-6">
    <skos:altLabel xml:lang="de">Gerichtliche Prozeßkosten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18878-4">
    <skos:prefLabel xml:lang="en">Subject of field</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18878-4">
    <skos:prefLabel xml:lang="de">Studienfach</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19727-6">
    <skos:related rdf:resource="descriptor/19817-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19727-6">
    <skos:related rdf:resource="descriptor/18370-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19727-6">
    <skos:related rdf:resource="descriptor/18961-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19218-6">
    <skos:altLabel xml:lang="de">Zurechnungstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19218-6">
    <skos:altLabel xml:lang="de">Grenzproduktivität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16724-6">
    <skos:altLabel xml:lang="en">Structuralist-evolutionary theory</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16724-6">
    <skos:altLabel xml:lang="en">Structuralist view</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16724-6">
    <skos:altLabel xml:lang="en">Structuralist theory</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16724-6">
    <skos:altLabel xml:lang="en">Structuralist perspective</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16724-6">
    <skos:altLabel xml:lang="en">Structuralist assessment</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16724-6">
    <skos:altLabel xml:lang="de">Strukturalistischer Ansatz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12533-5">
    <skos:narrower rdf:resource="descriptor/12531-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12228-3">
    <skos:narrower rdf:resource="descriptor/13746-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11264-4">
    <skos:broader rdf:resource="thsys/72181"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11264-4">
    <skos:broader rdf:resource="thsys/70051"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11264-4">
    <skos:broader rdf:resource="descriptor/15076-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11264-4">
    <skos:broader rdf:resource="descriptor/18887-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19499-0">
    <skos:prefLabel xml:lang="en">Women's wages</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19499-0">
    <skos:prefLabel xml:lang="de">Frauenarbeitslohn</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16017-3">
    <skos:broader rdf:resource="thsys/73381"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16017-3">
    <skos:broader rdf:resource="thsys/73361"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16017-3">
    <skos:broader rdf:resource="descriptor/16007-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19308-5">
    <skos:related rdf:resource="descriptor/10567-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19308-5">
    <skos:related rdf:resource="descriptor/19113-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19308-5">
    <skos:related rdf:resource="descriptor/10622-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19308-5">
    <skos:related rdf:resource="descriptor/10711-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19308-5">
    <skos:related rdf:resource="descriptor/18001-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14014-5">
    <skos:prefLabel xml:lang="en">Caring professions</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14014-5">
    <skos:prefLabel xml:lang="de">Sozialberufe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73368">
    <rdfs:label xml:lang="de">Struktur- und Entwicklungspolitik</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71098">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13485-6">
    <skos:related rdf:resource="descriptor/19754-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13485-6">
    <skos:related rdf:resource="descriptor/19141-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13485-6">
    <skos:related rdf:resource="descriptor/18532-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20277-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10406-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11530-2">
    <skos:related rdf:resource="descriptor/10590-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11530-2">
    <skos:related rdf:resource="descriptor/11531-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11530-2">
    <skos:related rdf:resource="descriptor/11529-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19094-5">
    <skos:altLabel xml:lang="de">Sachanlagevermögen des Betriebs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13003-1">
    <skos:prefLabel xml:lang="en">Nuclear industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13003-1">
    <skos:prefLabel xml:lang="de">Nuklearindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id386">
    <rdfs:seeAlso rdf:resource="descriptor/10184-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19733-4">
    <skos:prefLabel xml:lang="en">Responsibility</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19733-4">
    <skos:prefLabel xml:lang="de">Verantwortung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70037">
    <skos:broader rdf:resource="thsys/b"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10625-3">
    <skos:broader rdf:resource="thsys/71097"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10625-3">
    <skos:broader rdf:resource="descriptor/16436-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19393-2">
    <skos:related rdf:resource="descriptor/13153-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19393-2">
    <skos:related rdf:resource="descriptor/14044-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19393-2">
    <skos:related rdf:resource="descriptor/19338-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10307-3">
    <skos:altLabel xml:lang="de">Produktionsfunktion (Cobb-Douglas)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10307-3">
    <skos:altLabel xml:lang="de">CD-Produktionsfunktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19190-2">
    <skos:broader rdf:resource="thsys/71029"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19190-2">
    <skos:broader rdf:resource="thsys/71020"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19190-2">
    <skos:broader rdf:resource="descriptor/19187-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15935-5">
    <skos:broader rdf:resource="thsys/70142"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15935-5">
    <skos:broader rdf:resource="descriptor/15918-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11157-5">
    <skos:altLabel xml:lang="de">Gesundheitsökonomie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16680-0">
    <skos:broader rdf:resource="thsys/70013"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16680-0">
    <skos:broader rdf:resource="descriptor/16564-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19257-3">
    <skos:prefLabel xml:lang="en">Cash-in-Advance Model</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19257-3">
    <skos:prefLabel xml:lang="de">Cash-in-Advance-Modell</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19353-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13034-4">
    <skos:prefLabel xml:lang="en">Metal industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13034-4">
    <skos:prefLabel xml:lang="de">Metallindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13451-2">
    <skos:altLabel xml:lang="de">Agrarsystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19941-4">
    <skos:related rdf:resource="descriptor/11572-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19941-4">
    <skos:related rdf:resource="descriptor/19404-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19941-4">
    <skos:related rdf:resource="descriptor/11702-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19941-4">
    <skos:related rdf:resource="descriptor/11541-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14329-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19042-3">
    <skos:prefLabel xml:lang="en">Forecast</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19042-3">
    <skos:prefLabel xml:lang="de">Prognose</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13533-0">
    <skos:broader rdf:resource="thsys/70933"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13533-0">
    <skos:broader rdf:resource="descriptor/12013-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11857-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19847-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13268-0">
    <skos:broader rdf:resource="thsys/70922"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13268-0">
    <skos:broader rdf:resource="descriptor/13682-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12026-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10768-2">
    <skos:related rdf:resource="descriptor/10754-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10768-2">
    <skos:related rdf:resource="descriptor/10733-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70930">
    <skos:narrower rdf:resource="descriptor/13624-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70930">
    <skos:narrower rdf:resource="descriptor/13622-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70930">
    <skos:narrower rdf:resource="descriptor/18570-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70930">
    <skos:narrower rdf:resource="descriptor/13615-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70930">
    <skos:narrower rdf:resource="descriptor/18622-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70930">
    <skos:narrower rdf:resource="descriptor/18621-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70930">
    <skos:narrower rdf:resource="descriptor/18595-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71052">
    <skos:broader rdf:resource="thsys/71057"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18024-3">
    <skos:broader rdf:resource="thsys/73351"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18024-3">
    <skos:broader rdf:resource="thsys/70651"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18024-3">
    <skos:broader rdf:resource="thsys/70227"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18024-3">
    <skos:broader rdf:resource="descriptor/14184-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18024-3">
    <skos:broader rdf:resource="descriptor/18286-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19790-6">
    <skos:broader rdf:resource="thsys/73346"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19790-6">
    <skos:broader rdf:resource="descriptor/18897-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16308-6">
    <skos:broader rdf:resource="thsys/70321"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71103">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17626-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11539-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19537-4">
    <skos:related rdf:resource="descriptor/12272-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19537-4">
    <skos:related rdf:resource="descriptor/12212-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19537-4">
    <skos:related rdf:resource="descriptor/10293-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11230-0">
    <skos:prefLabel xml:lang="en">Dual labour market</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11230-0">
    <skos:prefLabel xml:lang="de">Dualer Arbeitsmarkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15651-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13862-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70943">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13385-3">
    <skos:broader rdf:resource="thsys/70876"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18995-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id201">
    <rdfs:seeAlso rdf:resource="descriptor/10991-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18815-0">
    <skos:scopeNote xml:lang="de">Die Streitkräfte werden als Wirtschaftsbetrieb aufgefaßt; im Mittelpunkt steht die sinnvolle Verwendung der knappen Mittel des Verteidigungshaushalts.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70989">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.06.01.09</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19370-0">
    <skos:broader rdf:resource="thsys/70144"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19370-0">
    <skos:broader rdf:resource="descriptor/11540-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11680-4">
    <skos:broader rdf:resource="thsys/71074"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11680-4">
    <skos:broader rdf:resource="descriptor/11661-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15937-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11681-2">
    <skos:prefLabel xml:lang="en">Transaction tax</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11681-2">
    <skos:prefLabel xml:lang="de">Verkehrsteuer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12207-4">
    <skos:altLabel xml:lang="de">Zinsrechnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12207-4">
    <skos:altLabel xml:lang="de">Zinseszinsrechnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12207-4">
    <skos:altLabel xml:lang="de">Rentenrechnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12207-4">
    <skos:altLabel xml:lang="de">Annuitätenrechnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13077-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id270">
    <rdfs:seeAlso rdf:resource="descriptor/14469-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18726-6">
    <skos:altLabel xml:lang="en">Stock voting right</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18726-6">
    <skos:altLabel xml:lang="de">Aktienstimmrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13501-6">
    <skos:narrower rdf:resource="descriptor/18609-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13501-6">
    <skos:narrower rdf:resource="descriptor/18614-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12987-1">
    <skos:altLabel xml:lang="en">Deepsea mining</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12987-1">
    <skos:altLabel xml:lang="de">Tiefseebergbau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16182-2">
    <skos:narrower rdf:resource="descriptor/16048-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16182-2">
    <skos:narrower rdf:resource="descriptor/18365-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16182-2">
    <skos:narrower rdf:resource="descriptor/19817-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16182-2">
    <skos:narrower rdf:resource="descriptor/16090-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16182-2">
    <skos:narrower rdf:resource="descriptor/18904-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16182-2">
    <skos:narrower rdf:resource="descriptor/18441-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16182-2">
    <skos:narrower rdf:resource="descriptor/12331-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16182-2">
    <skos:narrower rdf:resource="descriptor/18961-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70581">
    <skos:broader rdf:resource="thsys/70060"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10180-1">
    <skos:broader rdf:resource="thsys/70814"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10180-1">
    <skos:broader rdf:resource="thsys/70493"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10180-1">
    <skos:broader rdf:resource="descriptor/11754-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10180-1">
    <skos:broader rdf:resource="descriptor/10177-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10511-4">
    <skos:broader rdf:resource="thsys/70091"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10511-4">
    <skos:broader rdf:resource="descriptor/11068-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20553-3">
    <skos:broader rdf:resource="thsys/71085"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17773-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13755-3">
    <skos:related rdf:resource="descriptor/18731-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19850-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11267-5">
    <skos:broader rdf:resource="thsys/70843"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11267-5">
    <skos:broader rdf:resource="thsys/70176"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11267-5">
    <skos:broader rdf:resource="thsys/70051"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11267-5">
    <skos:broader rdf:resource="descriptor/11183-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16963-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17199-1">
    <skos:prefLabel xml:lang="en">Western Europe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17199-1">
    <skos:prefLabel xml:lang="de">Westeuropa</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15902-6">
    <skos:broader rdf:resource="thsys/73354"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15902-6">
    <skos:broader rdf:resource="descriptor/15872-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10256-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18679-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18664-2">
    <skos:prefLabel xml:lang="en">Corporate network</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18664-2">
    <skos:prefLabel xml:lang="de">Unternehmensnetzwerk</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15911-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13724-0">
    <skos:altLabel xml:lang="en">OTC-market</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13724-0">
    <skos:altLabel xml:lang="en">OTC market</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13724-0">
    <skos:altLabel xml:lang="de">Telefonhandel mit Effekten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13724-0">
    <skos:altLabel xml:lang="de">OTC-Wertpapierhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13724-0">
    <skos:altLabel xml:lang="de">OTC-Effektenhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13724-0">
    <skos:altLabel xml:lang="de">Freiverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13724-0">
    <skos:altLabel xml:lang="de">Außerbörslicher Verkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13724-0">
    <skos:altLabel xml:lang="de">Außerbörslicher Handel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19400-3">
    <skos:prefLabel xml:lang="en">Laffer curve</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19400-3">
    <skos:prefLabel xml:lang="de">Laffer-Kurve</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19362-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13502-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17967-3">
    <skos:broader rdf:resource="thsys/70198"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17967-3">
    <skos:broader rdf:resource="descriptor/17954-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10693-0">
    <skos:broader rdf:resource="thsys/73392"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10693-0">
    <skos:broader rdf:resource="thsys/71098"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10693-0">
    <skos:broader rdf:resource="thsys/70951"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10693-0">
    <skos:broader rdf:resource="descriptor/12246-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10693-0">
    <skos:broader rdf:resource="descriptor/10684-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17672-1">
    <skos:prefLabel xml:lang="en">East Africa</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17672-1">
    <skos:prefLabel xml:lang="de">Ostafrika</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id137">
    <rdfs:seeAlso rdf:resource="descriptor/11224-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18932-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13964-1">
    <skos:altLabel xml:lang="de">Wirtschaftsingenieurinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13964-1">
    <skos:altLabel xml:lang="de">Vertriebsingenieurinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13964-1">
    <skos:altLabel xml:lang="de">Vertriebsingenieure</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13964-1">
    <skos:altLabel xml:lang="de">Verkaufsingenieurinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13964-1">
    <skos:altLabel xml:lang="de">Verkaufsingenieure</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13964-1">
    <skos:altLabel xml:lang="de">Ingenieurökonominnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13964-1">
    <skos:altLabel xml:lang="de">Ingenieurökonomen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13964-1">
    <skos:altLabel xml:lang="de">Betriebsingenieurinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13964-1">
    <skos:altLabel xml:lang="de">Betriebsingenieure</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11829-3">
    <skos:broader rdf:resource="thsys/70928"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11829-3">
    <skos:broader rdf:resource="descriptor/11556-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14906-3">
    <skos:broader rdf:resource="thsys/70031"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14906-3">
    <skos:broader rdf:resource="descriptor/14899-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71055">
    <skos:broader rdf:resource="thsys/71057"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19222-1">
    <skos:prefLabel xml:lang="en">Price stickiness</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19222-1">
    <skos:prefLabel xml:lang="de">Preisrigidität</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12298-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11339-6">
    <skos:related rdf:resource="descriptor/11752-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11339-6">
    <skos:related rdf:resource="descriptor/11347-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11339-6">
    <skos:related rdf:resource="descriptor/11346-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11339-6">
    <skos:related rdf:resource="descriptor/19247-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11339-6">
    <skos:related rdf:resource="descriptor/19844-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11339-6">
    <skos:related rdf:resource="descriptor/11185-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11339-6">
    <skos:related rdf:resource="descriptor/11307-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18028-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18591-3">
    <skos:altLabel xml:lang="de">Schiffahrtsrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18591-3">
    <skos:altLabel xml:lang="de">Bundeswasserstraßengesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18591-3">
    <skos:altLabel xml:lang="de">Binnenschifffahrtsordnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18591-3">
    <skos:altLabel xml:lang="de">Binnenschiffahrtsordnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18964-4">
    <skos:broader rdf:resource="thsys/70816"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18964-4">
    <skos:broader rdf:resource="thsys/70239"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18964-4">
    <skos:broader rdf:resource="descriptor/15774-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73103">
    <skos:broader rdf:resource="thsys/v"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19856-2">
    <skos:broader rdf:resource="thsys/73392"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12228-3">
    <skos:prefLabel xml:lang="en">Corporate liquidity</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12228-3">
    <skos:prefLabel xml:lang="de">Betriebliche Liquidität</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id80">
    <rdfs:seeAlso rdf:resource="descriptor/14995-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19741-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12779-1">
    <skos:broader rdf:resource="thsys/70963"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12779-1">
    <skos:broader rdf:resource="thsys/70873"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12779-1">
    <skos:broader rdf:resource="descriptor/13470-5"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id68">
    <rdf:value xml:lang="de">Für Deutschland bis 1945</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10593-4">
    <skos:altLabel xml:lang="en">Common fund for commodities</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10593-4">
    <skos:altLabel xml:lang="de">Rohstoffonds</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10593-4">
    <skos:altLabel xml:lang="de">Rohstofffonds</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10593-4">
    <skos:altLabel xml:lang="de">Kompensatorische Rohstoffinanzierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10593-4">
    <skos:altLabel xml:lang="de">Kompensatorische Rohstofffinanzierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10593-4">
    <skos:altLabel xml:lang="de">Internationales Rohstoffabkommen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10593-4">
    <skos:altLabel xml:lang="de">Internationale Rohstoffpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10593-4">
    <skos:altLabel xml:lang="de">Gemeinsamer Rohstoffonds</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10593-4">
    <skos:altLabel xml:lang="de">Gemeinsamer Rohstofffonds</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12441-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71019">
    <skos:broader rdf:resource="thsys/w"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15733-3">
    <skos:scopeNote xml:lang="de">Verunreinigung von Grundwasser, Binnengewässern und Wasserstraßen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15648-1">
    <skos:prefLabel xml:lang="en">Judaism</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15648-1">
    <skos:prefLabel xml:lang="de">Judentum</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18609-3">
    <skos:broader rdf:resource="thsys/70939"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18609-3">
    <skos:broader rdf:resource="descriptor/13632-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18609-3">
    <skos:broader rdf:resource="descriptor/13501-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19449-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18183-4">
    <skos:broader rdf:resource="thsys/71111"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18183-4">
    <skos:broader rdf:resource="thsys/70239"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18183-4">
    <skos:broader rdf:resource="descriptor/10265-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13643-0">
    <skos:related rdf:resource="descriptor/18637-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16205-2">
    <skos:broader rdf:resource="thsys/71064"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16205-2">
    <skos:broader rdf:resource="thsys/70095"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16205-2">
    <skos:broader rdf:resource="descriptor/16563-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13660-0">
    <skos:broader rdf:resource="thsys/73360"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13660-0">
    <skos:broader rdf:resource="thsys/70825"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13660-0">
    <skos:broader rdf:resource="descriptor/16019-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12987-1">
    <skos:related rdf:resource="descriptor/16187-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12987-1">
    <skos:related rdf:resource="descriptor/19618-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12987-1">
    <skos:related rdf:resource="descriptor/15887-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#prefLabel">
    <rdfs:comment xml:lang="en">skos:prefLabel, skos:altLabel and skos:hiddenLabel are pairwise
      disjoint properties.</rdfs:comment>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#prefLabel">
    <rdfs:comment xml:lang="en">The range of skos:prefLabel is the class of RDF plain literals.</rdfs:comment>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#prefLabel">
    <rdfs:comment xml:lang="en">No two concepts in the same concept scheme may have the same value
      for skos:prefLabel in a given language.</rdfs:comment>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12984-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10831-0">
    <skos:altLabel xml:lang="de">Steuerparadies</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10831-0">
    <skos:altLabel xml:lang="de">Niedrigsteuerland</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/a">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12403-4">
    <skos:broader rdf:resource="thsys/71042"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12403-4">
    <skos:broader rdf:resource="descriptor/12308-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13285-0">
    <skos:broader rdf:resource="thsys/70902"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13285-0">
    <skos:broader rdf:resource="descriptor/13291-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70538">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70834">
    <skos:narrower rdf:resource="descriptor/19830-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70834">
    <skos:narrower rdf:resource="descriptor/13405-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70834">
    <skos:narrower rdf:resource="descriptor/19511-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70834">
    <skos:narrower rdf:resource="descriptor/16543-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70834">
    <skos:narrower rdf:resource="descriptor/18906-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70834">
    <skos:narrower rdf:resource="descriptor/14014-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70834">
    <skos:narrower rdf:resource="descriptor/18836-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70834">
    <skos:narrower rdf:resource="descriptor/18908-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70834">
    <skos:narrower rdf:resource="descriptor/18907-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70834">
    <skos:narrower rdf:resource="descriptor/14007-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70834">
    <skos:narrower rdf:resource="descriptor/19715-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70834">
    <skos:narrower rdf:resource="descriptor/18855-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70834">
    <skos:narrower rdf:resource="descriptor/18864-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70834">
    <skos:narrower rdf:resource="descriptor/16535-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70834">
    <skos:narrower rdf:resource="descriptor/13312-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70834">
    <skos:narrower rdf:resource="descriptor/19773-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70834">
    <skos:narrower rdf:resource="descriptor/16525-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70834">
    <skos:narrower rdf:resource="descriptor/13310-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18293-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11709-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10141-4">
    <skos:narrower rdf:resource="descriptor/10225-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10141-4">
    <skos:narrower rdf:resource="descriptor/10222-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10141-4">
    <skos:narrower rdf:resource="descriptor/10147-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10141-4">
    <skos:narrower rdf:resource="descriptor/12170-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10141-4">
    <skos:narrower rdf:resource="descriptor/12168-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10141-4">
    <skos:narrower rdf:resource="descriptor/10194-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18939-3">
    <skos:related rdf:resource="descriptor/12825-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18939-3">
    <skos:related rdf:resource="descriptor/16012-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18939-3">
    <skos:related rdf:resource="descriptor/16517-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18939-3">
    <skos:related rdf:resource="descriptor/13360-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18939-3">
    <skos:related rdf:resource="descriptor/18928-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18939-3">
    <skos:related rdf:resource="descriptor/18922-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13852-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12609-5">
    <skos:related rdf:resource="descriptor/12613-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12609-5">
    <skos:related rdf:resource="descriptor/19509-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12609-5">
    <skos:related rdf:resource="descriptor/15737-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12609-5">
    <skos:related rdf:resource="descriptor/11814-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12609-5">
    <skos:related rdf:resource="descriptor/12615-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12609-5">
    <skos:related rdf:resource="descriptor/12618-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12609-5">
    <skos:related rdf:resource="descriptor/12610-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12609-5">
    <skos:related rdf:resource="descriptor/11305-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10159-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18342-3">
    <skos:prefLabel xml:lang="en">Fishery fleet</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18342-3">
    <skos:prefLabel xml:lang="de">Fischereiflotte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70026">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">G.04</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19033-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10776-3">
    <skos:altLabel xml:lang="de">Valuta</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10776-3">
    <skos:altLabel xml:lang="de">Sorten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10776-3">
    <skos:altLabel xml:lang="de">Auslandswährung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19165-1">
    <skos:narrower rdf:resource="descriptor/17567-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19165-1">
    <skos:narrower rdf:resource="descriptor/17565-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12949-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11034-0">
    <skos:prefLabel xml:lang="en">Rationing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11034-0">
    <skos:prefLabel xml:lang="de">Rationierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16344-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11580-1">
    <skos:narrower rdf:resource="descriptor/18365-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13434-2">
    <skos:altLabel xml:lang="de">Bodenertrag</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11024-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18975-6">
    <skos:narrower rdf:resource="descriptor/13780-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11875-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13452-0">
    <skos:altLabel xml:lang="de">Pflanzenanbausystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13452-0">
    <skos:altLabel xml:lang="de">Fruchtwechselwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13452-0">
    <skos:altLabel xml:lang="de">Fruchtfolge</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18655-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15946-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70824">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17675-2">
    <skos:altLabel xml:lang="de">Republik Seychellen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10558-6">
    <skos:broader rdf:resource="thsys/73378"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10558-6">
    <skos:broader rdf:resource="thsys/73366"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10558-6">
    <skos:broader rdf:resource="thsys/71085"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10558-6">
    <skos:broader rdf:resource="thsys/71084"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10558-6">
    <skos:broader rdf:resource="thsys/70853"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10558-6">
    <skos:broader rdf:resource="descriptor/12091-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10558-6">
    <skos:broader rdf:resource="descriptor/10984-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19700-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19856-2">
    <zbwext:useInsteadNote rdf:nodeID="id315"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19856-2">
    <zbwext:useInsteadNote rdf:nodeID="id314"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15790-5">
    <skos:broader rdf:resource="thsys/70017"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15790-5">
    <skos:broader rdf:resource="descriptor/15685-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15969-2">
    <skos:narrower rdf:resource="descriptor/15965-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11701-1">
    <skos:broader rdf:resource="thsys/71075"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11701-1">
    <skos:broader rdf:resource="thsys/71069"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11701-1">
    <skos:broader rdf:resource="descriptor/11702-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11701-1">
    <skos:broader rdf:resource="descriptor/19291-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11701-1">
    <skos:broader rdf:resource="descriptor/16552-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15774-3">
    <skos:narrower rdf:resource="descriptor/18964-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18214-5">
    <skos:broader rdf:resource="thsys/70498"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18214-5">
    <skos:broader rdf:resource="descriptor/14931-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19412-3">
    <skos:prefLabel xml:lang="en">Direct consumption tax</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19412-3">
    <skos:prefLabel xml:lang="de">Ausgabensteuer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19415-4">
    <skos:altLabel xml:lang="de">Einheitssteuer (Flat tax)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73386">
    <skos:narrower rdf:resource="descriptor/15811-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73386">
    <skos:narrower rdf:resource="descriptor/15719-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73386">
    <skos:narrower rdf:resource="descriptor/15714-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73386">
    <skos:narrower rdf:resource="descriptor/19772-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73386">
    <skos:narrower rdf:resource="descriptor/13383-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73386">
    <skos:narrower rdf:resource="descriptor/16265-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73386">
    <skos:narrower rdf:resource="descriptor/11164-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73386">
    <skos:narrower rdf:resource="descriptor/18924-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73386">
    <skos:narrower rdf:resource="descriptor/12103-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73386">
    <skos:narrower rdf:resource="descriptor/15798-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73386">
    <skos:narrower rdf:resource="descriptor/15835-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73386">
    <skos:narrower rdf:resource="descriptor/15797-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73386">
    <skos:narrower rdf:resource="descriptor/18832-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73386">
    <skos:narrower rdf:resource="descriptor/10150-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19506-1">
    <skos:related rdf:resource="descriptor/18910-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19506-1">
    <skos:related rdf:resource="descriptor/10622-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19506-1">
    <skos:related rdf:resource="descriptor/11328-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19506-1">
    <skos:related rdf:resource="descriptor/15069-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19506-1">
    <skos:related rdf:resource="descriptor/10498-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19506-1">
    <skos:related rdf:resource="descriptor/10488-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19443-6">
    <skos:prefLabel xml:lang="en">Localization economies</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19443-6">
    <skos:prefLabel xml:lang="de">Lokalisationseffekt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18566-2">
    <skos:broader rdf:resource="thsys/70943"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12865-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10181-6">
    <skos:related rdf:resource="descriptor/10162-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10181-6">
    <skos:related rdf:resource="descriptor/10161-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10181-6">
    <skos:related rdf:resource="descriptor/10300-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10181-6">
    <skos:related rdf:resource="descriptor/10276-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10181-6">
    <skos:related rdf:resource="descriptor/10171-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10181-6">
    <skos:related rdf:resource="descriptor/19540-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24785-4">
    <skos:altLabel xml:lang="de">Verbraucherzufriedenheit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24785-4">
    <skos:altLabel xml:lang="de">Käuferzufriedenheit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16579-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15757-3">
    <skos:altLabel xml:lang="de">Radioaktive Strahlung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15757-3">
    <skos:altLabel xml:lang="de">Kontamination durch Strahlen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15757-3">
    <skos:altLabel xml:lang="de">Kernenergieschaden</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11640-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16417-1">
    <skos:altLabel xml:lang="en">Non-Aligned movement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16417-1">
    <skos:altLabel xml:lang="en">Nonaligned countries</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16417-1">
    <skos:altLabel xml:lang="en">Movement of nonaligned countries</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16417-1">
    <skos:altLabel xml:lang="de">Neutrale Staaten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16417-1">
    <skos:altLabel xml:lang="de">Dauernde Neutralität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16417-1">
    <skos:altLabel xml:lang="de">Blockfreiheit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16417-1">
    <skos:altLabel xml:lang="de">Blockfreienbewegung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16417-1">
    <skos:altLabel xml:lang="de">Blockfreie Staaten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16417-1">
    <skos:altLabel xml:lang="de">Bewegung Blockfreier Staaten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12919-4">
    <skos:narrower rdf:resource="descriptor/13006-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12919-4">
    <skos:narrower rdf:resource="descriptor/18411-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12919-4">
    <skos:narrower rdf:resource="descriptor/13000-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21020-5">
    <skos:scopeNote xml:lang="de">Aufbau und Verwaltung integrierter Logistikketten.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16658-0">
    <skos:prefLabel xml:lang="en">Elite</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16658-0">
    <skos:prefLabel xml:lang="de">Elite</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12033-4">
    <skos:altLabel xml:lang="de">AGB-Gesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12033-4">
    <skos:altLabel xml:lang="de">AGB (Allgemeine Geschäftsbedingungen)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id262">
    <rdfs:seeAlso rdf:resource="descriptor/10177-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17607-5">
    <skos:prefLabel xml:lang="en">Maldives</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17607-5">
    <skos:prefLabel xml:lang="de">Malediven</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10120-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12394-0">
    <skos:altLabel xml:lang="en">Social reporting</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12394-0">
    <skos:altLabel xml:lang="en">Social balance sheet</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12394-0">
    <skos:altLabel xml:lang="en">Social audit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12394-0">
    <skos:altLabel xml:lang="en">Corporate social accounting</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12394-0">
    <skos:altLabel xml:lang="de">Sozialbezogene Rechnungslegung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12394-0">
    <skos:altLabel xml:lang="de">Gesellschaftsbezogene Rechnungslegung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70888">
    <skos:broader rdf:resource="thsys/70889"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id178">
    <rdf:value xml:lang="de">Für Bürgerkriege</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18860-2">
    <skos:broader rdf:resource="thsys/70847"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18860-2">
    <skos:broader rdf:resource="thsys/70176"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18860-2">
    <skos:broader rdf:resource="thsys/70143"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18860-2">
    <skos:broader rdf:resource="thsys/70013"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18860-2">
    <skos:broader rdf:resource="descriptor/18859-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70929">
    <skos:broader rdf:resource="thsys/70943"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19068-6">
    <skos:related rdf:resource="descriptor/19605-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19068-6">
    <skos:related rdf:resource="descriptor/12380-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12069-4">
    <skos:narrower rdf:resource="descriptor/12071-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17618-0">
    <skos:altLabel xml:lang="de">Staat Bahrain</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17618-0">
    <skos:altLabel xml:lang="de">Bahrein</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17738-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11030-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19027-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13334-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15124-0">
    <skos:prefLabel xml:lang="en">Sample census</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15124-0">
    <skos:prefLabel xml:lang="de">Mikrozensus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11501-2">
    <skos:related rdf:resource="descriptor/11507-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16187-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19198-0">
    <skos:related rdf:resource="descriptor/19202-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19198-0">
    <skos:related rdf:resource="descriptor/10991-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19198-0">
    <skos:related rdf:resource="descriptor/10987-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19198-0">
    <skos:related rdf:resource="descriptor/19755-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19198-0">
    <skos:related rdf:resource="descriptor/19758-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19198-0">
    <skos:related rdf:resource="descriptor/19683-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19198-0">
    <skos:related rdf:resource="descriptor/10378-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19198-0">
    <skos:related rdf:resource="descriptor/18138-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13124-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12920-5">
    <skos:related rdf:resource="descriptor/19182-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12920-5">
    <skos:related rdf:resource="descriptor/13221-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12920-5">
    <skos:related rdf:resource="descriptor/19181-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12920-5">
    <skos:related rdf:resource="descriptor/18373-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12920-5">
    <skos:related rdf:resource="descriptor/14197-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19539-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73391">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13283-4">
    <skos:related rdf:resource="descriptor/10774-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13283-4">
    <skos:related rdf:resource="descriptor/11414-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13283-4">
    <skos:related rdf:resource="descriptor/13693-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13283-4">
    <skos:related rdf:resource="descriptor/13691-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13283-4">
    <skos:related rdf:resource="descriptor/13690-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13283-4">
    <skos:related rdf:resource="descriptor/13689-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13283-4">
    <skos:related rdf:resource="descriptor/18707-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13283-4">
    <skos:related rdf:resource="descriptor/13686-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13283-4">
    <skos:related rdf:resource="descriptor/13965-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10266-5">
    <skos:broader rdf:resource="thsys/71111"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10266-5">
    <skos:broader rdf:resource="descriptor/15064-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10282-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18002-6">
    <skos:broader rdf:resource="thsys/71036"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18002-6">
    <skos:broader rdf:resource="thsys/70471"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18002-6">
    <skos:broader rdf:resource="descriptor/18175-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18002-6">
    <skos:broader rdf:resource="descriptor/12730-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15744-5">
    <skos:related rdf:resource="descriptor/18623-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15744-5">
    <skos:related rdf:resource="descriptor/16187-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15744-5">
    <skos:related rdf:resource="descriptor/18594-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15744-5">
    <skos:related rdf:resource="descriptor/19631-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15744-5">
    <skos:related rdf:resource="descriptor/15887-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73336">
    <skos:prefLabel xml:lang="en">N.05.06  Labour law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73336">
    <skos:prefLabel xml:lang="de">N.05.06  Arbeitsrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11990-3">
    <skos:prefLabel xml:lang="en">Urban economics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11990-3">
    <skos:prefLabel xml:lang="de">Stadtökonomik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10866-2">
    <skos:altLabel xml:lang="de">Exportversicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10866-2">
    <skos:altLabel xml:lang="de">Ausfuhrkreditversicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18743-6">
    <skos:broader rdf:resource="thsys/70892"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18743-6">
    <skos:broader rdf:resource="descriptor/18717-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20485-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15373-0">
    <skos:related rdf:resource="descriptor/15076-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15373-0">
    <skos:related rdf:resource="descriptor/10037-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15373-0">
    <skos:related rdf:resource="descriptor/15705-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10253-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19288-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16588-2">
    <skos:altLabel xml:lang="en">Civil society</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16588-2">
    <skos:altLabel xml:lang="de">Zivilgesellschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16588-2">
    <skos:altLabel xml:lang="de">Bürgerliche Gesellschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16588-2">
    <skos:altLabel xml:lang="de">Bürgergesellschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13279-2">
    <skos:narrower rdf:resource="descriptor/18682-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13279-2">
    <skos:narrower rdf:resource="descriptor/10884-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13279-2">
    <skos:narrower rdf:resource="descriptor/18691-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13279-2">
    <skos:narrower rdf:resource="descriptor/18731-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13069-6">
    <skos:prefLabel xml:lang="en">Bicycle and motorcycle industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13069-6">
    <skos:prefLabel xml:lang="de">Zweiradindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12793-0">
    <skos:related rdf:resource="descriptor/20206-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12793-0">
    <skos:related rdf:resource="descriptor/12818-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12793-0">
    <skos:related rdf:resource="descriptor/10951-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12793-0">
    <skos:related rdf:resource="descriptor/10690-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12793-0">
    <skos:related rdf:resource="descriptor/13480-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14472-3">
    <skos:broader rdf:resource="thsys/70254"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14472-3">
    <skos:broader rdf:resource="descriptor/18103-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19559-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13321-1">
    <skos:related rdf:resource="descriptor/14846-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13321-1">
    <skos:related rdf:resource="descriptor/13935-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13321-1">
    <skos:related rdf:resource="descriptor/13330-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13321-1">
    <skos:related rdf:resource="descriptor/14899-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70373">
    <skos:prefLabel xml:lang="en">P.10.02  Audiovisual product</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70373">
    <skos:prefLabel xml:lang="de">P.10.02  Unterhaltungselektronik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70013">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14964-3">
    <skos:related rdf:resource="descriptor/13155-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14964-3">
    <skos:related rdf:resource="descriptor/13940-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14964-3">
    <skos:related rdf:resource="descriptor/14122-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11430-6">
    <skos:altLabel xml:lang="de">Goldstandard</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11430-6">
    <skos:altLabel xml:lang="de">Goldparität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11430-6">
    <skos:altLabel xml:lang="de">Goldkernwährung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10691-4">
    <skos:altLabel xml:lang="de">Handelsembargo</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10691-4">
    <skos:altLabel xml:lang="de">Exportverbot</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10691-4">
    <skos:altLabel xml:lang="de">Exportembargo</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10691-4">
    <skos:altLabel xml:lang="de">Ausfuhrverbot</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10691-4">
    <skos:altLabel xml:lang="de">Ausfuhrembargo</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16828-1">
    <skos:related rdf:resource="descriptor/18224-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16828-1">
    <skos:related rdf:resource="descriptor/18225-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12053-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19608-0">
    <skos:related rdf:resource="descriptor/19778-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19608-0">
    <skos:related rdf:resource="descriptor/16724-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19608-0">
    <skos:related rdf:resource="descriptor/13277-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19608-0">
    <skos:related rdf:resource="descriptor/15676-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19608-0">
    <skos:related rdf:resource="descriptor/19777-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19608-0">
    <skos:related rdf:resource="descriptor/19776-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18095-1">
    <skos:broader rdf:resource="thsys/71002"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18095-1">
    <skos:broader rdf:resource="descriptor/13058-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17684-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70960">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14583-1">
    <skos:altLabel xml:lang="de">Telespiel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14583-1">
    <skos:altLabel xml:lang="de">Computerspiel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14583-1">
    <skos:altLabel xml:lang="de">Bildschirmspiel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11040-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10602-1">
    <zbwext:useInsteadNote rdf:nodeID="id37"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10602-1">
    <zbwext:useInsteadNote rdf:nodeID="id36"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19593-1">
    <skos:prefLabel xml:lang="en">Normal form game</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19593-1">
    <skos:prefLabel xml:lang="de">Normalformspiel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18069-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14840-0">
    <skos:broader rdf:resource="thsys/70066"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10424-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70575">
    <rdfs:label xml:lang="de">Wirtschaftssektoren und spezielle Wirtschaftslehren</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18304-4">
    <skos:related rdf:resource="descriptor/14931-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18304-4">
    <skos:related rdf:resource="descriptor/18443-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10082-1">
    <skos:narrower rdf:resource="descriptor/11459-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13844-4">
    <skos:related rdf:resource="descriptor/18746-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13844-4">
    <skos:related rdf:resource="descriptor/18496-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13844-4">
    <skos:related rdf:resource="descriptor/11901-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13844-4">
    <skos:related rdf:resource="descriptor/13846-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13731-3">
    <skos:narrower rdf:resource="descriptor/18677-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13731-3">
    <skos:narrower rdf:resource="descriptor/18676-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13731-3">
    <skos:narrower rdf:resource="descriptor/18702-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18813-4">
    <skos:altLabel xml:lang="de">Privatfinanzierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18813-4">
    <skos:altLabel xml:lang="de">Konzessionsmodell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18813-4">
    <skos:altLabel xml:lang="de">Betreibermodell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13509-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18941-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13592-5">
    <skos:related rdf:resource="descriptor/13644-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13592-5">
    <skos:related rdf:resource="descriptor/13638-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13592-5">
    <skos:related rdf:resource="descriptor/11850-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17599-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18726-6">
    <skos:prefLabel xml:lang="en">Share voting right</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18726-6">
    <skos:prefLabel xml:lang="de">Stimmrecht (Gesellschaftsrecht)</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13478-3">
    <skos:prefLabel xml:lang="en">Itinerant selling</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13478-3">
    <skos:prefLabel xml:lang="de">Ambulanter Handel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19275-1">
    <skos:broader rdf:resource="thsys/71108"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19275-1">
    <skos:broader rdf:resource="descriptor/13726-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19275-1">
    <skos:broader rdf:resource="descriptor/11441-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16914-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14804-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19941-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13674-3">
    <skos:broader rdf:resource="thsys/70922"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13674-3">
    <skos:broader rdf:resource="descriptor/13682-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13450-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16326-4">
    <skos:prefLabel xml:lang="en">Navy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16326-4">
    <skos:prefLabel xml:lang="de">Marine</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13342-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70819">
    <skos:broader rdf:resource="thsys/w"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14432-1">
    <skos:broader rdf:resource="thsys/70258"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14432-1">
    <skos:broader rdf:resource="descriptor/14444-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18891-5">
    <skos:broader rdf:resource="thsys/73347"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18891-5">
    <skos:broader rdf:resource="thsys/70839"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18891-5">
    <skos:broader rdf:resource="thsys/70573"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13173-4">
    <skos:related rdf:resource="descriptor/19610-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13814-6">
    <skos:broader rdf:resource="thsys/70932"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13814-6">
    <skos:broader rdf:resource="thsys/70891"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13814-6">
    <skos:broader rdf:resource="descriptor/13297-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15658-5">
    <skos:altLabel xml:lang="de">Naturmedizin</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15658-5">
    <skos:altLabel xml:lang="de">Medizinische Forschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15658-5">
    <skos:altLabel xml:lang="de">Heilkunst</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19753-5">
    <skos:related rdf:resource="descriptor/16013-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19753-5">
    <skos:related rdf:resource="descriptor/11164-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19753-5">
    <skos:related rdf:resource="descriptor/16258-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19753-5">
    <skos:related rdf:resource="descriptor/11158-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19753-5">
    <skos:related rdf:resource="descriptor/15825-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19753-5">
    <skos:related rdf:resource="descriptor/15823-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19753-5">
    <skos:related rdf:resource="descriptor/16245-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#historyNote">
    <rdfs:label xml:lang="en">history note</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11318-0">
    <skos:narrower rdf:resource="descriptor/19512-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19178-6">
    <skos:altLabel xml:lang="de">G7-Staaten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19178-6">
    <skos:altLabel xml:lang="de">G7-Länder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19178-6">
    <skos:altLabel xml:lang="de">G-7-Länder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18074-2">
    <skos:prefLabel xml:lang="en">Computer-aided learning</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18074-2">
    <skos:prefLabel xml:lang="de">Computergestütztes Lernen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18852-1">
    <skos:prefLabel xml:lang="en">Trade union organization</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18852-1">
    <skos:prefLabel xml:lang="de">Gewerkschaftsorganisation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71007">
    <skos:prefLabel xml:lang="en">W.05.01.01  Metal mining</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71007">
    <skos:prefLabel xml:lang="de">W.05.01.01  Erzbergbau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19811-3">
    <skos:scopeNote xml:lang="de">Elektronische Erkennungsverfahren auf der Basis biologischer Merkmale wie Fingerabdruck, Iris, Stimme, usw.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14833-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18112-6">
    <skos:related rdf:resource="descriptor/20241-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18112-6">
    <skos:related rdf:resource="descriptor/13278-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16326-4">
    <skos:altLabel xml:lang="de">Seestreitkräfte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16326-4">
    <skos:altLabel xml:lang="de">Kriegsmarine</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16326-4">
    <skos:altLabel xml:lang="de">Kriegsflotte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13293-1">
    <skos:prefLabel xml:lang="en">Credit insurance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13293-1">
    <skos:prefLabel xml:lang="de">Kreditversicherung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71081">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13004-6">
    <zbwext:useInsteadNote rdf:nodeID="id126"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20842-3">
    <skos:broader rdf:resource="thsys/73376"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20842-3">
    <skos:broader rdf:resource="descriptor/20855-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18414-4">
    <skos:broader rdf:resource="thsys/71002"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18414-4">
    <skos:broader rdf:resource="descriptor/13058-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14803-6">
    <skos:altLabel xml:lang="de">Pappewaren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14803-6">
    <skos:altLabel xml:lang="de">Papierwaren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10917-4">
    <skos:related rdf:resource="descriptor/10964-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10917-4">
    <skos:related rdf:resource="descriptor/19323-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10917-4">
    <skos:related rdf:resource="descriptor/10704-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10917-4">
    <skos:related rdf:resource="descriptor/15713-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10917-4">
    <skos:related rdf:resource="descriptor/11189-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18358-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11366-3">
    <skos:narrower rdf:resource="descriptor/18885-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11366-3">
    <skos:narrower rdf:resource="descriptor/11365-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11366-3">
    <skos:narrower rdf:resource="descriptor/11228-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19756-6">
    <skos:narrower rdf:resource="descriptor/19764-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14622-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11399-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17652-0">
    <skos:narrower rdf:resource="descriptor/17743-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17652-0">
    <skos:narrower rdf:resource="descriptor/17724-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17652-0">
    <skos:narrower rdf:resource="descriptor/17687-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17652-0">
    <skos:narrower rdf:resource="descriptor/17672-1"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id175">
    <rdfs:seeAlso rdf:resource="descriptor/14440-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12984-0">
    <skos:altLabel xml:lang="de">Ölförderindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12984-0">
    <skos:altLabel xml:lang="de">Mineralölgewinnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12984-0">
    <skos:altLabel xml:lang="de">Mineralölförderung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12984-0">
    <skos:altLabel xml:lang="de">Mineralölförderindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12984-0">
    <skos:altLabel xml:lang="de">Erdölbohrung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18365-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15683-6">
    <skos:prefLabel xml:lang="en">Winter sports</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15683-6">
    <skos:prefLabel xml:lang="de">Wintersport</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id52">
    <rdfs:seeAlso rdf:resource="descriptor/10115-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70822">
    <skos:narrower rdf:resource="descriptor/13350-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70822">
    <skos:narrower rdf:resource="descriptor/18881-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70822">
    <skos:narrower rdf:resource="descriptor/13361-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70822">
    <skos:narrower rdf:resource="descriptor/16789-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70822">
    <skos:narrower rdf:resource="descriptor/18179-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70822">
    <skos:narrower rdf:resource="descriptor/13358-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70822">
    <skos:narrower rdf:resource="descriptor/16797-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70822">
    <skos:narrower rdf:resource="descriptor/16794-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70822">
    <skos:narrower rdf:resource="descriptor/16780-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70822">
    <skos:narrower rdf:resource="descriptor/18873-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70822">
    <skos:narrower rdf:resource="descriptor/16767-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70822">
    <skos:narrower rdf:resource="descriptor/16777-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70822">
    <skos:narrower rdf:resource="descriptor/18951-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70822">
    <skos:narrower rdf:resource="descriptor/16771-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70822">
    <skos:narrower rdf:resource="descriptor/16802-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70822">
    <skos:narrower rdf:resource="descriptor/18945-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70822">
    <skos:narrower rdf:resource="descriptor/16792-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70822">
    <skos:narrower rdf:resource="descriptor/16783-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70822">
    <skos:narrower rdf:resource="descriptor/16769-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70822">
    <skos:narrower rdf:resource="descriptor/18952-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70822">
    <skos:narrower rdf:resource="descriptor/13351-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70822">
    <skos:narrower rdf:resource="descriptor/16768-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70822">
    <skos:narrower rdf:resource="descriptor/18950-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70822">
    <skos:narrower rdf:resource="descriptor/18949-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19289-4">
    <skos:broader rdf:resource="thsys/70171"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19289-4">
    <skos:broader rdf:resource="descriptor/10012-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13595-6">
    <skos:prefLabel xml:lang="en">Integration effect of traffic</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13595-6">
    <skos:prefLabel xml:lang="de">Verkehrsintegration</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19554-4">
    <skos:related rdf:resource="descriptor/13742-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19554-4">
    <skos:related rdf:resource="descriptor/13715-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19554-4">
    <skos:related rdf:resource="descriptor/18731-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10349-1">
    <skos:prefLabel xml:lang="en">Inventory cycle</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10349-1">
    <skos:prefLabel xml:lang="de">Lagerzyklus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12153-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17743-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id101">
    <rdf:value xml:lang="de">Für Financial Reform im Sinne von Kapitalmarktreform</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19273-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16417-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19800-1">
    <skos:altLabel xml:lang="de">Virus (Datenverarbeitung)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19800-1">
    <skos:altLabel xml:lang="de">Viren-Scanner</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19800-1">
    <skos:altLabel xml:lang="de">Virenprogramm</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19800-1">
    <skos:altLabel xml:lang="de">PC-Virus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19800-1">
    <skos:altLabel xml:lang="de">Computervirus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19800-1">
    <skos:altLabel xml:lang="de">Antiviren-Programm</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12651-6">
    <skos:prefLabel xml:lang="en">Procurement planning</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12651-6">
    <skos:prefLabel xml:lang="de">Beschaffungsplanung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15013-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70060">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18896-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19399-4">
    <skos:broader rdf:resource="thsys/71078"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19399-4">
    <skos:broader rdf:resource="thsys/70038"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19399-4">
    <skos:broader rdf:resource="descriptor/19829-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19518-1">
    <skos:broader rdf:resource="thsys/70839"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19518-1">
    <skos:broader rdf:resource="thsys/70573"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19518-1">
    <skos:broader rdf:resource="descriptor/11555-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10598-1">
    <skos:related rdf:resource="descriptor/10472-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10598-1">
    <skos:related rdf:resource="descriptor/18789-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10598-1">
    <skos:related rdf:resource="descriptor/19723-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10598-1">
    <skos:related rdf:resource="descriptor/10576-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10598-1">
    <skos:related rdf:resource="descriptor/16524-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17801-2">
    <skos:broader rdf:resource="thsys/70086"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17801-2">
    <skos:broader rdf:resource="descriptor/17791-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19673-3">
    <skos:prefLabel xml:lang="en">Corporate value</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19673-3">
    <skos:prefLabel xml:lang="de">Unternehmenswert</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12913-2">
    <skos:narrower rdf:resource="descriptor/12979-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12913-2">
    <skos:narrower rdf:resource="descriptor/12993-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12913-2">
    <skos:narrower rdf:resource="descriptor/12920-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12913-2">
    <skos:narrower rdf:resource="descriptor/19464-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12913-2">
    <skos:narrower rdf:resource="descriptor/12919-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12913-2">
    <skos:narrower rdf:resource="descriptor/18153-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12913-2">
    <skos:narrower rdf:resource="descriptor/13019-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12913-2">
    <skos:narrower rdf:resource="descriptor/13004-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12913-2">
    <skos:narrower rdf:resource="descriptor/18538-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12913-2">
    <skos:narrower rdf:resource="descriptor/12975-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12913-2">
    <skos:narrower rdf:resource="descriptor/12980-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12913-2">
    <skos:narrower rdf:resource="descriptor/12904-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12913-2">
    <skos:narrower rdf:resource="descriptor/18320-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19443-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14545-2">
    <skos:prefLabel xml:lang="en">Cutlery</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14545-2">
    <skos:prefLabel xml:lang="de">Schneidwaren und Besteck</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19601-0">
    <skos:prefLabel xml:lang="en">Cognition</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19601-0">
    <skos:prefLabel xml:lang="de">Kognition</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14027-3">
    <skos:prefLabel xml:lang="en">One-way packaging</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14027-3">
    <skos:prefLabel xml:lang="de">Einwegverpackung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13395-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71112">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id54">
    <rdf:value xml:lang="de">Für Unternehmenszählungen der amtlichen Statistik</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11669-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16497-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18903-3">
    <skos:related rdf:resource="descriptor/19518-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18903-3">
    <skos:related rdf:resource="descriptor/13372-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18903-3">
    <skos:related rdf:resource="descriptor/11555-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19352-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14160-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11824-6">
    <skos:scopeNote xml:lang="de">Bezieht sich auf die Struktur des gesamten Distributionsbereichs, also von Einzelhandel, Großhandel und Versandhandel.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11688-2">
    <skos:related rdf:resource="descriptor/10103-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11688-2">
    <skos:related rdf:resource="descriptor/11685-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13632-5">
    <skos:altLabel xml:lang="de">Personentransport</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13632-5">
    <skos:altLabel xml:lang="de">Personenbeförderung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13632-5">
    <skos:altLabel xml:lang="de">Passagierverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13632-5">
    <skos:altLabel xml:lang="de">Fahrgastverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13632-5">
    <skos:altLabel xml:lang="de">Fahrgastbeförderung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71102">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24665-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18226-5">
    <skos:related rdf:resource="descriptor/17176-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16286-4">
    <skos:broader rdf:resource="thsys/73375"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18745-2">
    <skos:prefLabel xml:lang="en">Real estate price</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18745-2">
    <skos:prefLabel xml:lang="de">Immobilienpreis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19846-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11414-4">
    <skos:scopeNote xml:lang="de">Bezeichnet im allgemeinen Institutionen, deren Geschäftstätigkeit primär darauf gerichtet ist, zu einem Ausgleich zwischen Anlage- und Finanzbedarf beizutragen; vor allem Banken, Versicherungen, Investmentfonds, Leasing- und Factoringinstitute, Beteiligungsgesellschaften.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12495-1">
    <skos:broader rdf:resource="thsys/70578"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12495-1">
    <skos:broader rdf:resource="descriptor/10011-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71119">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14775-6">
    <skos:prefLabel xml:lang="en">Flat glass</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14775-6">
    <skos:prefLabel xml:lang="de">Flachglas</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16540-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70114">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12867-4">
    <skos:prefLabel xml:lang="en">Co-partners</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12867-4">
    <skos:prefLabel xml:lang="de">Mitunternehmer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16012-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10602-1">
    <skos:narrower rdf:resource="descriptor/10822-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12923-6">
    <skos:prefLabel xml:lang="en">Repair business</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12923-6">
    <skos:prefLabel xml:lang="de">Reparaturgewerbe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13650-3">
    <skos:prefLabel xml:lang="en">Book</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13650-3">
    <skos:prefLabel xml:lang="de">Buch</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70829">
    <rdfs:label xml:lang="en">Leisure services</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70829">
    <rdfs:label xml:lang="de">Freizeitwirtschaft</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10303-4">
    <skos:scopeNote xml:lang="de">Ansatz der Produktionstheorie nach T.C Koopmans, der als Modellvariable die Aktivitäten des Produktionsbereichs auffasst.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13096-3">
    <skos:broader rdf:resource="thsys/71004"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13096-3">
    <skos:broader rdf:resource="descriptor/12993-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15609-4">
    <skos:broader rdf:resource="thsys/73339"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15609-4">
    <skos:broader rdf:resource="thsys/71039"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15609-4">
    <skos:broader rdf:resource="thsys/70882"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15019-4">
    <skos:broader rdf:resource="thsys/70161"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15019-4">
    <skos:broader rdf:resource="descriptor/15009-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18855-2">
    <skos:broader rdf:resource="thsys/70850"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18855-2">
    <skos:broader rdf:resource="thsys/70834"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18855-2">
    <skos:broader rdf:resource="descriptor/16543-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25822-5">
    <skos:altLabel xml:lang="en">CSR (Corporate social responsibility)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25822-5">
    <skos:altLabel xml:lang="en">Corporate citizenship</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25822-5">
    <skos:altLabel xml:lang="de">Unternehmerische Sozialverantwortung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25822-5">
    <skos:altLabel xml:lang="de">Soziales Engagement von Unternehmen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25822-5">
    <skos:altLabel xml:lang="de">Soziale Verantwortung von Unternehmen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25822-5">
    <skos:altLabel xml:lang="de">Gesellschaftliches Engagement von Unternehmen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25822-5">
    <skos:altLabel xml:lang="de">Gesellschaftliche Verantwortung von Unternehmen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18326-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19134-5">
    <skos:altLabel xml:lang="de">Beschwerdemanagement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19180-5">
    <skos:related rdf:resource="descriptor/19167-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71060">
    <skos:narrower rdf:resource="descriptor/11216-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71060">
    <skos:narrower rdf:resource="descriptor/19495-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71060">
    <skos:narrower rdf:resource="descriptor/11210-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71060">
    <skos:narrower rdf:resource="descriptor/19471-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71060">
    <skos:narrower rdf:resource="descriptor/19439-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71060">
    <skos:narrower rdf:resource="descriptor/10255-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71060">
    <skos:narrower rdf:resource="descriptor/18150-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71060">
    <skos:narrower rdf:resource="descriptor/18124-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71060">
    <skos:narrower rdf:resource="descriptor/19461-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71060">
    <skos:narrower rdf:resource="descriptor/10339-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71060">
    <skos:narrower rdf:resource="descriptor/11200-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71060">
    <skos:narrower rdf:resource="descriptor/11213-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71060">
    <skos:narrower rdf:resource="descriptor/19498-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71060">
    <skos:narrower rdf:resource="descriptor/18140-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71060">
    <skos:narrower rdf:resource="descriptor/11182-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21110-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21042-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14705-6">
    <skos:prefLabel xml:lang="en">Polyethylene</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14705-6">
    <skos:prefLabel xml:lang="de">Polyethylen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13346-6">
    <skos:broader rdf:resource="thsys/70882"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13346-6">
    <skos:broader rdf:resource="thsys/70823"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13346-6">
    <skos:broader rdf:resource="descriptor/19475-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13346-6">
    <skos:broader rdf:resource="descriptor/13345-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13346-6">
    <skos:broader rdf:resource="descriptor/18762-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12533-5">
    <skos:broader rdf:resource="thsys/70578"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12533-5">
    <skos:broader rdf:resource="descriptor/12480-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71054">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14601-4">
    <skos:prefLabel xml:lang="en">Chemical element</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14601-4">
    <skos:prefLabel xml:lang="de">Chemisches Element</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19671-0">
    <skos:scopeNote xml:lang="de">Festsetzung der amtlichen Kurse für die an der Börse zugelassenen Wertpapiere.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13194-3">
    <skos:altLabel xml:lang="de">Baustoffgroßhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70327">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">G.02.04</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12230-2">
    <skos:broader rdf:resource="thsys/71107"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16335-3">
    <skos:related rdf:resource="descriptor/16019-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16335-3">
    <skos:related rdf:resource="descriptor/13413-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16335-3">
    <skos:related rdf:resource="descriptor/20842-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16335-3">
    <skos:related rdf:resource="descriptor/16301-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14496-3">
    <skos:related rdf:resource="descriptor/14186-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14496-3">
    <skos:related rdf:resource="descriptor/14407-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12056-6">
    <skos:narrower rdf:resource="descriptor/18119-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12056-6">
    <skos:narrower rdf:resource="descriptor/12055-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12056-6">
    <skos:narrower rdf:resource="descriptor/12050-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71091">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14895-3">
    <skos:altLabel xml:lang="de">Tischwäsche</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14895-3">
    <skos:altLabel xml:lang="de">Schlafsack</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14895-3">
    <skos:altLabel xml:lang="de">Möbelstoff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14895-3">
    <skos:altLabel xml:lang="de">Matratze</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14895-3">
    <skos:altLabel xml:lang="de">Haushaltswäsche</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14895-3">
    <skos:altLabel xml:lang="de">Handtuch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14895-3">
    <skos:altLabel xml:lang="de">Gardine</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14895-3">
    <skos:altLabel xml:lang="de">Dekorationsstoff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14895-3">
    <skos:altLabel xml:lang="de">Decke</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14895-3">
    <skos:altLabel xml:lang="de">Bettwäsche</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14895-3">
    <skos:altLabel xml:lang="de">Bettwaren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13543-4">
    <skos:prefLabel xml:lang="en">Coastal shipping</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13543-4">
    <skos:prefLabel xml:lang="de">Küstenschifffahrt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14150-4">
    <skos:broader rdf:resource="thsys/70465"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14150-4">
    <skos:broader rdf:resource="descriptor/14149-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16278-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18758-0">
    <skos:related rdf:resource="descriptor/15621-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18758-0">
    <skos:related rdf:resource="descriptor/16757-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18758-0">
    <skos:related rdf:resource="descriptor/14751-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19331-3">
    <skos:broader rdf:resource="thsys/71096"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19331-3">
    <skos:broader rdf:resource="thsys/71093"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19331-3">
    <skos:broader rdf:resource="descriptor/10713-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10387-0">
    <skos:narrower rdf:resource="descriptor/10392-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10387-0">
    <skos:narrower rdf:resource="descriptor/10383-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12793-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13274-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18117-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14946-5">
    <skos:altLabel xml:lang="de">Marmelade</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14946-5">
    <skos:altLabel xml:lang="de">Gelee</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18821-5">
    <skos:related rdf:resource="descriptor/12809-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18821-5">
    <skos:related rdf:resource="descriptor/13488-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18821-5">
    <skos:related rdf:resource="descriptor/18101-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19112-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17177-4">
    <skos:prefLabel xml:lang="en">Romania</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17177-4">
    <skos:prefLabel xml:lang="de">Rumänien</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14200-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19472-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16757-5">
    <skos:altLabel xml:lang="x-other">Informatique</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16757-5">
    <skos:altLabel xml:lang="en">Information and communication technologies</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16757-5">
    <skos:altLabel xml:lang="en">ICT (Information and communication technologies)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16757-5">
    <skos:altLabel xml:lang="de">Neue Informationstechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16757-5">
    <skos:altLabel xml:lang="de">Kommunikationstechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16757-5">
    <skos:altLabel xml:lang="de">IuK-Technik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16757-5">
    <skos:altLabel xml:lang="de">IT (Informationstechnik)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16757-5">
    <skos:altLabel xml:lang="de">Informationstechnologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16757-5">
    <skos:altLabel xml:lang="de">Informations- und Kommunikationstechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19742-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14090-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://zbw.eu/namespaces/skos-extensions/Thsys">
    <skos:definition xml:lang="en">The systematic part (subject categories) 
    of the STW.</skos:definition>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12257-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17744-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18235-4">
    <skos:prefLabel xml:lang="en">Fossil fuel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18235-4">
    <skos:prefLabel xml:lang="de">Fossiler Energieträger</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18182-6">
    <skos:scopeNote xml:lang="de">Aufgabe der Produktion von Rüstungsgütern zugunsten der Herstellung ziviler Produkte.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12774-4">
    <skos:altLabel xml:lang="en">Marketing research</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12774-4">
    <skos:altLabel xml:lang="de">Marktprognose</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12774-4">
    <skos:altLabel xml:lang="de">Marktbeobachtung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12774-4">
    <skos:altLabel xml:lang="de">Marktanalyse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12774-4">
    <skos:altLabel xml:lang="de">Konkurrenzanalyse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12774-4">
    <skos:altLabel xml:lang="de">Absatzmarktforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12774-4">
    <skos:altLabel xml:lang="de">Absatzforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15664-3">
    <skos:prefLabel xml:lang="en">Natural sciences</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15664-3">
    <skos:prefLabel xml:lang="de">Naturwissenschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15080-1">
    <skos:related rdf:resource="descriptor/13209-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14931-4">
    <skos:prefLabel xml:lang="en">Food</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14931-4">
    <skos:prefLabel xml:lang="de">Nahrungsmittel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11267-5">
    <skos:prefLabel xml:lang="en">Supply of apprenticeships</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11267-5">
    <skos:prefLabel xml:lang="de">Lehrstellenmarkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18188-1">
    <skos:related rdf:resource="descriptor/19266-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18188-1">
    <skos:related rdf:resource="descriptor/10678-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18188-1">
    <skos:related rdf:resource="descriptor/10668-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18188-1">
    <skos:related rdf:resource="descriptor/10001-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19605-6">
    <skos:related rdf:resource="descriptor/19068-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70893">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13149-1">
    <skos:related rdf:resource="descriptor/19349-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13149-1">
    <skos:related rdf:resource="descriptor/14130-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13149-1">
    <skos:related rdf:resource="descriptor/12952-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10227-1">
    <skos:prefLabel xml:lang="en">Perfect competition</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10227-1">
    <skos:prefLabel xml:lang="de">Vollkommener Wettbewerb</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10942-5">
    <skos:broader rdf:resource="thsys/73391"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10942-5">
    <skos:broader rdf:resource="thsys/70212"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11951-6">
    <skos:narrower rdf:resource="descriptor/11963-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11951-6">
    <skos:narrower rdf:resource="descriptor/11912-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11951-6">
    <skos:narrower rdf:resource="descriptor/11873-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17806-6">
    <skos:broader rdf:resource="thsys/70080"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17806-6">
    <skos:broader rdf:resource="descriptor/17758-5"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id333">
    <rdf:value xml:lang="de">Für Titandioxid</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16621-2">
    <skos:altLabel xml:lang="de">Gewalttätigkeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16621-2">
    <skos:altLabel xml:lang="de">Gewaltbereitschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16621-2">
    <skos:altLabel xml:lang="de">Aggressivität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17013-6">
    <skos:prefLabel xml:lang="en">Malta</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17013-6">
    <skos:prefLabel xml:lang="de">Malta</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15054-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11127-0">
    <skos:broader rdf:resource="thsys/73382"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11127-0">
    <skos:broader rdf:resource="thsys/73375"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11127-0">
    <skos:broader rdf:resource="thsys/70321"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11127-0">
    <skos:broader rdf:resource="thsys/70039"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11127-0">
    <skos:broader rdf:resource="descriptor/16290-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15745-3">
    <skos:broader rdf:resource="thsys/73351"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15745-3">
    <skos:broader rdf:resource="thsys/70811"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10816-3">
    <skos:related rdf:resource="descriptor/16295-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10816-3">
    <skos:related rdf:resource="descriptor/19683-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10816-3">
    <skos:related rdf:resource="descriptor/19668-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10816-3">
    <skos:related rdf:resource="descriptor/19235-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10816-3">
    <skos:related rdf:resource="descriptor/20536-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10816-3">
    <skos:related rdf:resource="descriptor/12253-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10816-3">
    <skos:related rdf:resource="descriptor/13777-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10816-3">
    <skos:related rdf:resource="descriptor/10293-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10816-3">
    <skos:related rdf:resource="descriptor/10824-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10816-3">
    <skos:related rdf:resource="descriptor/18542-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10816-3">
    <skos:related rdf:resource="descriptor/18694-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10816-3">
    <skos:related rdf:resource="descriptor/10797-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10816-3">
    <skos:related rdf:resource="descriptor/19315-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10816-3">
    <skos:related rdf:resource="descriptor/10858-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10816-3">
    <skos:related rdf:resource="descriptor/10854-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13188-5">
    <skos:related rdf:resource="descriptor/10884-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13188-5">
    <skos:related rdf:resource="descriptor/13238-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13188-5">
    <skos:related rdf:resource="descriptor/13472-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13188-5">
    <skos:related rdf:resource="descriptor/13219-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13188-5">
    <skos:related rdf:resource="descriptor/18320-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13188-5">
    <skos:related rdf:resource="descriptor/14596-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13188-5">
    <skos:related rdf:resource="descriptor/14054-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13188-5">
    <skos:related rdf:resource="descriptor/13431-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13188-5">
    <skos:related rdf:resource="descriptor/13429-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13188-5">
    <skos:related rdf:resource="descriptor/18008-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19320-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19433-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70943">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19339-1">
    <skos:prefLabel xml:lang="en">Agricultural soil</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19339-1">
    <skos:prefLabel xml:lang="de">Agrarboden</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19966-2">
    <skos:broader rdf:resource="thsys/71115"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19966-2">
    <skos:broader rdf:resource="thsys/70245"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19966-2">
    <skos:broader rdf:resource="descriptor/10006-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70463">
    <skos:broader rdf:resource="thsys/70014"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70082">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://creativecommons.org/ns#license">
    <rdf:type rdf:resource="http://www.w3.org/2002/07/owl#ObjectProperty"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10113-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10021-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11314-1">
    <skos:related rdf:resource="descriptor/12090-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11314-1">
    <skos:related rdf:resource="descriptor/16150-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11314-1">
    <skos:related rdf:resource="descriptor/11350-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11314-1">
    <skos:related rdf:resource="descriptor/10401-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11314-1">
    <skos:related rdf:resource="descriptor/18125-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11314-1">
    <skos:related rdf:resource="descriptor/11346-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11314-1">
    <skos:related rdf:resource="descriptor/16692-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11314-1">
    <skos:related rdf:resource="descriptor/12603-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11314-1">
    <skos:related rdf:resource="descriptor/18123-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11314-1">
    <skos:related rdf:resource="descriptor/12577-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11314-1">
    <skos:related rdf:resource="descriptor/11185-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11314-1">
    <skos:related rdf:resource="descriptor/19493-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10806-6">
    <skos:scopeNote xml:lang="de">Nur für Geschäftsbanken; nicht für internationale Finanzinstitutionen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16364-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15909-6">
    <skos:broader rdf:resource="thsys/70142"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10250-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71050">
    <skos:prefLabel xml:lang="en">B.04.04  Human engineering</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71050">
    <skos:prefLabel xml:lang="de">B.04.04  Arbeitswissenschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15399-3">
    <skos:broader rdf:resource="thsys/72394"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15399-3">
    <skos:broader rdf:resource="thsys/71035"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15399-3">
    <skos:broader rdf:resource="descriptor/15405-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15408-0">
    <skos:altLabel xml:lang="de">Logit-Regression</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19097-6">
    <skos:prefLabel xml:lang="en">Cost analysis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19097-6">
    <skos:prefLabel xml:lang="de">Kostenanalyse</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10298-6">
    <skos:editorialNote xml:lang="de">Möglichst spezifischer indexieren.</skos:editorialNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71018">
    <skos:narrower rdf:resource="descriptor/12967-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71018">
    <skos:narrower rdf:resource="descriptor/12966-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71018">
    <skos:narrower rdf:resource="descriptor/19383-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71018">
    <skos:narrower rdf:resource="descriptor/19382-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71018">
    <skos:narrower rdf:resource="descriptor/18344-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71018">
    <skos:narrower rdf:resource="descriptor/18343-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71018">
    <skos:narrower rdf:resource="descriptor/18312-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71018">
    <skos:narrower rdf:resource="descriptor/11807-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71018">
    <skos:narrower rdf:resource="descriptor/18342-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71018">
    <skos:narrower rdf:resource="descriptor/13873-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71018">
    <skos:narrower rdf:resource="descriptor/12964-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71018">
    <skos:narrower rdf:resource="descriptor/18341-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71018">
    <skos:narrower rdf:resource="descriptor/12963-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71018">
    <skos:narrower rdf:resource="descriptor/12969-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18876-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20982-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70052">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.13</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13179-6">
    <skos:broader rdf:resource="thsys/70969"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13179-6">
    <skos:broader rdf:resource="descriptor/13177-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15887-4">
    <skos:related rdf:resource="descriptor/15892-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15887-4">
    <skos:related rdf:resource="descriptor/15744-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15887-4">
    <skos:related rdf:resource="descriptor/14161-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15887-4">
    <skos:related rdf:resource="descriptor/19631-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15887-4">
    <skos:related rdf:resource="descriptor/19614-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15887-4">
    <skos:related rdf:resource="descriptor/15891-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15887-4">
    <skos:related rdf:resource="descriptor/12987-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15887-4">
    <skos:related rdf:resource="descriptor/15883-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15887-4">
    <skos:related rdf:resource="descriptor/19616-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15887-4">
    <skos:related rdf:resource="descriptor/12964-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15922-0">
    <skos:broader rdf:resource="thsys/70142"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15922-0">
    <skos:broader rdf:resource="descriptor/19643-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71001">
    <skos:narrower rdf:resource="descriptor/13039-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71001">
    <skos:narrower rdf:resource="descriptor/18406-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71001">
    <skos:narrower rdf:resource="descriptor/18405-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71001">
    <skos:narrower rdf:resource="descriptor/19390-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71001">
    <skos:narrower rdf:resource="descriptor/13029-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71001">
    <skos:narrower rdf:resource="descriptor/13064-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71001">
    <skos:narrower rdf:resource="descriptor/18404-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71001">
    <skos:narrower rdf:resource="descriptor/18403-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71001">
    <skos:narrower rdf:resource="descriptor/13027-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15468-3">
    <skos:scopeNote xml:lang="de">Entscheidungssituation ohne Kenntnis von Wahrscheinlichkeiten.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18382-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11318-0">
    <skos:broader rdf:resource="thsys/71081"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11318-0">
    <skos:broader rdf:resource="thsys/70872"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11318-0">
    <skos:broader rdf:resource="thsys/70278"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11318-0">
    <skos:broader rdf:resource="descriptor/11554-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11318-0">
    <skos:broader rdf:resource="descriptor/11322-2"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id276">
    <rdfs:seeAlso rdf:resource="descriptor/19841-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15108-5">
    <skos:broader rdf:resource="thsys/72181"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15108-5">
    <skos:broader rdf:resource="thsys/71112"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15108-5">
    <skos:broader rdf:resource="descriptor/15076-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17993-2">
    <skos:broader rdf:resource="thsys/70307"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17993-2">
    <skos:broader rdf:resource="descriptor/17999-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17993-2">
    <skos:broader rdf:resource="descriptor/10499-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13418-0">
    <skos:narrower rdf:resource="descriptor/18180-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13418-0">
    <skos:narrower rdf:resource="descriptor/13416-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13418-0">
    <skos:narrower rdf:resource="descriptor/13415-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13418-0">
    <skos:narrower rdf:resource="descriptor/13406-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10095-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19353-0">
    <skos:broader rdf:resource="thsys/71092"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19353-0">
    <skos:broader rdf:resource="descriptor/10764-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70321">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19034-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12998-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13352-4">
    <skos:broader rdf:resource="thsys/70828"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13352-4">
    <skos:broader rdf:resource="descriptor/18920-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12217-1">
    <skos:narrower rdf:resource="descriptor/18721-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16597-1">
    <skos:broader rdf:resource="thsys/70504"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16597-1">
    <skos:broader rdf:resource="thsys/70321"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24787-0">
    <skos:related rdf:resource="descriptor/18620-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19223-6">
    <skos:broader rdf:resource="thsys/70178"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19223-6">
    <skos:broader rdf:resource="descriptor/10217-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18362-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id364">
    <rdfs:seeAlso rdf:resource="descriptor/10928-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19483-1">
    <skos:altLabel xml:lang="en">Natural resource accounts</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19483-1">
    <skos:altLabel xml:lang="en">Environmental resource accounts</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19483-1">
    <skos:altLabel xml:lang="de">Umweltmodell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18628-6">
    <skos:prefLabel xml:lang="en">Sustainable transport policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18628-6">
    <skos:prefLabel xml:lang="de">Umweltverträgliche Verkehrspolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14993-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70987">
    <skos:narrower rdf:resource="descriptor/13149-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70987">
    <skos:narrower rdf:resource="descriptor/13148-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70987">
    <skos:narrower rdf:resource="descriptor/13146-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70987">
    <skos:narrower rdf:resource="descriptor/18435-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70987">
    <skos:narrower rdf:resource="descriptor/13145-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70987">
    <skos:narrower rdf:resource="descriptor/13141-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70987">
    <skos:narrower rdf:resource="descriptor/13935-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70987">
    <skos:narrower rdf:resource="descriptor/18094-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70987">
    <skos:narrower rdf:resource="descriptor/13144-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70987">
    <skos:narrower rdf:resource="descriptor/18295-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70987">
    <skos:narrower rdf:resource="descriptor/13131-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70987">
    <skos:narrower rdf:resource="descriptor/13130-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70987">
    <skos:narrower rdf:resource="descriptor/13128-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70987">
    <skos:narrower rdf:resource="descriptor/13127-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14545-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12716-4">
    <skos:prefLabel xml:lang="en">Make-to-order production</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12716-4">
    <skos:prefLabel xml:lang="de">Auftragsfertigung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17586-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15908-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11327-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13165-3">
    <skos:broader rdf:resource="thsys/70986"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13165-3">
    <skos:broader rdf:resource="descriptor/13160-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14846-2">
    <skos:related rdf:resource="descriptor/13321-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14846-2">
    <skos:related rdf:resource="descriptor/14392-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14846-2">
    <skos:related rdf:resource="descriptor/14840-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14846-2">
    <skos:related rdf:resource="descriptor/13145-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14846-2">
    <skos:related rdf:resource="descriptor/14630-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14846-2">
    <skos:related rdf:resource="descriptor/13141-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14846-2">
    <skos:related rdf:resource="descriptor/19061-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14846-2">
    <skos:related rdf:resource="descriptor/18436-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14846-2">
    <skos:related rdf:resource="descriptor/14383-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14846-2">
    <skos:related rdf:resource="descriptor/14849-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14846-2">
    <skos:related rdf:resource="descriptor/14899-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10758-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15554-3">
    <skos:altLabel xml:lang="de">Unendlichstufiges Entscheidungsproblem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15554-3">
    <skos:altLabel xml:lang="de">Dynamische Programmierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19694-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13348-2">
    <skos:related rdf:resource="descriptor/15790-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13348-2">
    <skos:related rdf:resource="descriptor/19013-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13348-2">
    <skos:related rdf:resource="descriptor/13371-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13348-2">
    <skos:related rdf:resource="descriptor/18890-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71064">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15267-6">
    <skos:broader rdf:resource="thsys/71035"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15267-6">
    <skos:broader rdf:resource="thsys/70269"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15267-6">
    <skos:broader rdf:resource="descriptor/15057-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15267-6">
    <skos:broader rdf:resource="descriptor/19591-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16184-5">
    <skos:prefLabel xml:lang="en">International agreement</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16184-5">
    <skos:prefLabel xml:lang="de">Internationales Abkommen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18219-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19562-5">
    <skos:broader rdf:resource="thsys/70562"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19562-5">
    <skos:broader rdf:resource="descriptor/12091-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17806-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15086-3">
    <skos:broader rdf:resource="thsys/72184"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15086-3">
    <skos:broader rdf:resource="thsys/71005"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15086-3">
    <skos:broader rdf:resource="descriptor/15093-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15086-3">
    <skos:broader rdf:resource="descriptor/15103-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15644-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15778-2">
    <skos:broader rdf:resource="thsys/70209"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15778-2">
    <skos:broader rdf:resource="descriptor/15767-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18184-2">
    <skos:broader rdf:resource="thsys/71041"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18184-2">
    <skos:broader rdf:resource="thsys/70485"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18184-2">
    <skos:broader rdf:resource="descriptor/19163-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18184-2">
    <skos:broader rdf:resource="descriptor/12884-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12142-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12910-1">
    <skos:prefLabel xml:lang="en">Trade (Craft)</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12910-1">
    <skos:prefLabel xml:lang="de">Handwerk</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14846-2">
    <skos:narrower rdf:resource="descriptor/14889-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14846-2">
    <skos:narrower rdf:resource="descriptor/14888-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14846-2">
    <skos:narrower rdf:resource="descriptor/14844-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14846-2">
    <skos:narrower rdf:resource="descriptor/14914-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14846-2">
    <skos:narrower rdf:resource="descriptor/14839-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14846-2">
    <skos:narrower rdf:resource="descriptor/14895-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14846-2">
    <skos:narrower rdf:resource="descriptor/14885-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11965-2">
    <skos:related rdf:resource="descriptor/19478-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11965-2">
    <skos:related rdf:resource="descriptor/19477-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11965-2">
    <skos:related rdf:resource="descriptor/11962-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11965-2">
    <skos:related rdf:resource="descriptor/11852-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/a">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18331-1">
    <skos:prefLabel xml:lang="en">EU agricultural price policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18331-1">
    <skos:prefLabel xml:lang="de">EU-Agrarpreispolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11293-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19002-1">
    <skos:scopeNote xml:lang="de">Nur für die Kontrolle von Mitarbeiterleistung im Unternehmen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18169-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12041-5">
    <skos:altLabel xml:lang="en">Business administration</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12041-5">
    <skos:altLabel xml:lang="de">Unternehmenstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12041-5">
    <skos:altLabel xml:lang="de">Spezielle Betriebswirtschaftslehre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12041-5">
    <skos:altLabel xml:lang="de">BWL</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12041-5">
    <skos:altLabel xml:lang="de">Betriebswirtschaftstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12041-5">
    <skos:altLabel xml:lang="de">Betriebswirtschaftliche Theorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12041-5">
    <skos:altLabel xml:lang="de">Allgemeine Betriebswirtschaftslehre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10510-6">
    <skos:scopeNote xml:lang="de">Beschreibung der städtischen Entwicklung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19563-3">
    <skos:broader rdf:resource="thsys/70175"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19563-3">
    <skos:broader rdf:resource="thsys/70140"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15675-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15066-2">
    <skos:related rdf:resource="descriptor/15243-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15066-2">
    <skos:related rdf:resource="descriptor/19034-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15066-2">
    <skos:related rdf:resource="descriptor/12647-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14020-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13964-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12056-6">
    <skos:broader rdf:resource="thsys/70087"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12056-6">
    <skos:broader rdf:resource="descriptor/10911-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18208-0">
    <skos:related rdf:resource="descriptor/15621-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18208-0">
    <skos:related rdf:resource="descriptor/14751-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70561">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18969-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70183">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">B.02.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10976-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11357-4">
    <skos:related rdf:resource="descriptor/11352-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19668-3">
    <skos:prefLabel xml:lang="en">Economic security</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19668-3">
    <skos:prefLabel xml:lang="de">Ökonomische Sicherheit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19265-4">
    <skos:broader rdf:resource="thsys/70455"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19265-4">
    <skos:broader rdf:resource="thsys/70187"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19265-4">
    <skos:broader rdf:resource="descriptor/10135-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16670-3">
    <skos:prefLabel xml:lang="en">Disease</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16670-3">
    <skos:prefLabel xml:lang="de">Krankheit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70878">
    <skos:prefLabel xml:lang="de">W.20.01  Forschungsbereiche</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15713-2">
    <skos:related rdf:resource="descriptor/15718-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15713-2">
    <skos:related rdf:resource="descriptor/16654-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15713-2">
    <skos:related rdf:resource="descriptor/16623-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15713-2">
    <skos:related rdf:resource="descriptor/16674-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15713-2">
    <skos:related rdf:resource="descriptor/16656-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15713-2">
    <skos:related rdf:resource="descriptor/19009-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15713-2">
    <skos:related rdf:resource="descriptor/16512-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15713-2">
    <skos:related rdf:resource="descriptor/16661-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15713-2">
    <skos:related rdf:resource="descriptor/19011-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15713-2">
    <skos:related rdf:resource="descriptor/16429-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15713-2">
    <skos:related rdf:resource="descriptor/10917-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15713-2">
    <skos:related rdf:resource="descriptor/11400-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15713-2">
    <skos:related rdf:resource="descriptor/16084-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15743-0">
    <skos:broader rdf:resource="thsys/73351"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15743-0">
    <skos:broader rdf:resource="thsys/70238"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15743-0">
    <skos:broader rdf:resource="descriptor/15763-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15397-0">
    <skos:broader rdf:resource="thsys/72394"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15397-0">
    <skos:broader rdf:resource="descriptor/15405-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16314-4">
    <skos:altLabel xml:lang="de">Föderalistischer Staat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16314-4">
    <skos:altLabel xml:lang="de">Bund-Länder-Kooperation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16314-4">
    <skos:altLabel xml:lang="de">Bundesstaat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16314-4">
    <skos:altLabel xml:lang="de">Bund Länder Kooperation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14958-5">
    <skos:altLabel xml:lang="en">Cooking fat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14958-5">
    <skos:altLabel xml:lang="de">Speiseöl</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14958-5">
    <skos:altLabel xml:lang="de">Nahrungsfett</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18721-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18684-3">
    <skos:broader rdf:resource="thsys/71105"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18684-3">
    <skos:broader rdf:resource="descriptor/11504-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18684-3">
    <skos:broader rdf:resource="descriptor/12234-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13466-3">
    <skos:scopeNote xml:lang="de">Umfasst alle Arten der Handelsvermittlung, sowohl selbständige Handelsvertreter/innen und Handelsmakler/innen als auch angestellte Mitarbeiter/innen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13218-1">
    <skos:narrower rdf:resource="descriptor/18551-1"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id250">
    <rdf:value xml:lang="de">Für die zahlenmäßige Entwicklung der Beschäftigung</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18343-1">
    <skos:related rdf:resource="descriptor/14161-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18343-1">
    <skos:related rdf:resource="descriptor/19626-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18343-1">
    <skos:related rdf:resource="descriptor/12964-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18343-1">
    <skos:related rdf:resource="descriptor/14158-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19141-1">
    <skos:related rdf:resource="descriptor/10298-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19141-1">
    <skos:related rdf:resource="descriptor/12665-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19141-1">
    <skos:related rdf:resource="descriptor/13479-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19141-1">
    <skos:related rdf:resource="descriptor/13485-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19141-1">
    <skos:related rdf:resource="descriptor/19133-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14631-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12932-5">
    <skos:narrower rdf:resource="descriptor/18753-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12932-5">
    <skos:narrower rdf:resource="descriptor/11843-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15651-5">
    <skos:related rdf:resource="descriptor/13001-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15651-5">
    <skos:related rdf:resource="descriptor/14200-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15651-5">
    <skos:related rdf:resource="descriptor/13032-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15651-5">
    <skos:related rdf:resource="descriptor/18353-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15651-5">
    <skos:related rdf:resource="descriptor/14212-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10811-6">
    <skos:broader rdf:resource="thsys/71089"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10811-6">
    <skos:broader rdf:resource="thsys/70486"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10811-6">
    <skos:broader rdf:resource="thsys/70179"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10811-6">
    <skos:broader rdf:resource="descriptor/10615-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13325-0">
    <skos:broader rdf:resource="thsys/70818"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13029-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13414-1">
    <skos:altLabel xml:lang="en">County</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13414-1">
    <skos:altLabel xml:lang="de">Verwaltungsbezirk</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13414-1">
    <skos:altLabel xml:lang="de">Regierungsprovinz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13414-1">
    <skos:altLabel xml:lang="de">Regierungsbezirk</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13414-1">
    <skos:altLabel xml:lang="de">Landkreis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13414-1">
    <skos:altLabel xml:lang="de">Kreis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13414-1">
    <skos:altLabel xml:lang="de">Distrikt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13414-1">
    <skos:altLabel xml:lang="de">Bezirk</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18975-6">
    <skos:related rdf:resource="descriptor/19563-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18975-6">
    <skos:related rdf:resource="descriptor/18908-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18975-6">
    <skos:related rdf:resource="descriptor/25621-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18975-6">
    <skos:related rdf:resource="descriptor/13398-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13546-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11626-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71059">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.13.04</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24855-2">
    <skos:prefLabel xml:lang="en">Overshooting</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24855-2">
    <skos:prefLabel xml:lang="de">Overshooting</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14404-6">
    <skos:narrower rdf:resource="descriptor/14408-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15054-2">
    <skos:prefLabel xml:lang="en">Nonlinear dynamics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15054-2">
    <skos:prefLabel xml:lang="de">Nichtlineare dynamische Systeme</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id199">
    <rdf:value xml:lang="de">Für Zulieferer</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12647-4">
    <skos:related rdf:resource="descriptor/18783-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12647-4">
    <skos:related rdf:resource="descriptor/12649-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12647-4">
    <skos:related rdf:resource="descriptor/15066-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12647-4">
    <skos:related rdf:resource="descriptor/19034-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13389-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18059-5">
    <skos:prefLabel xml:lang="en">Economic information</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18059-5">
    <skos:prefLabel xml:lang="de">Wirtschaftsinformation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19696-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10624-5">
    <skos:broader rdf:resource="thsys/70179"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19715-6">
    <skos:prefLabel xml:lang="en">Mediation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19715-6">
    <skos:prefLabel xml:lang="de">Mediation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12918-6">
    <skos:altLabel xml:lang="de">Verbrauchsgüterindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12918-6">
    <skos:altLabel xml:lang="de">Gebrauchsgüterindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11043-6">
    <skos:prefLabel xml:lang="en">Mixed economy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11043-6">
    <skos:prefLabel xml:lang="de">Gemischtes Wirtschaftssystem</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13712-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17938-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16146-6">
    <skos:related rdf:resource="descriptor/11284-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16146-6">
    <skos:related rdf:resource="descriptor/19788-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16146-6">
    <skos:related rdf:resource="descriptor/18970-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16146-6">
    <skos:related rdf:resource="descriptor/11240-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12232-5">
    <skos:broader rdf:resource="thsys/71106"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12232-5">
    <skos:broader rdf:resource="thsys/71046"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12232-5">
    <skos:broader rdf:resource="descriptor/12299-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16046-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71106">
    <rdfs:label xml:lang="en">Stock market</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71106">
    <rdfs:label xml:lang="de">Aktienmarkt</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18475-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11771-1">
    <skos:prefLabel xml:lang="en">Expropriation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11771-1">
    <skos:prefLabel xml:lang="de">Enteignung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14037-0">
    <skos:altLabel xml:lang="de">Kohlenkraftwerk</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14037-0">
    <skos:altLabel xml:lang="de">Kohlebefeuertes Kraftwerk</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70979">
    <rdfs:label xml:lang="en">Industrial policy</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70979">
    <rdfs:label xml:lang="de">Industriepolitik</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12108-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24782-3">
    <skos:related rdf:resource="descriptor/11793-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24782-3">
    <skos:related rdf:resource="descriptor/20241-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24782-3">
    <skos:related rdf:resource="descriptor/24708-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24782-3">
    <skos:related rdf:resource="descriptor/18659-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24782-3">
    <skos:related rdf:resource="descriptor/19544-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24782-3">
    <skos:related rdf:resource="descriptor/24665-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18358-2">
    <skos:related rdf:resource="descriptor/19463-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18373-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12759-0">
    <skos:broader rdf:resource="thsys/70550"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12759-0">
    <skos:broader rdf:resource="thsys/70029"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12759-0">
    <skos:broader rdf:resource="descriptor/12808-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10326-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13436-5">
    <skos:altLabel xml:lang="de">Winzerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13436-5">
    <skos:altLabel xml:lang="de">Winzer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13436-5">
    <skos:altLabel xml:lang="de">Weinbauern</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13436-5">
    <skos:altLabel xml:lang="de">Landwirtinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13436-5">
    <skos:altLabel xml:lang="de">Landwirt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13436-5">
    <skos:altLabel xml:lang="de">Bauern</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13436-5">
    <skos:altLabel xml:lang="de">Bäuerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12711-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13034-4">
    <skos:narrower rdf:resource="descriptor/13039-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13034-4">
    <skos:narrower rdf:resource="descriptor/13029-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13034-4">
    <skos:narrower rdf:resource="descriptor/13037-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13034-4">
    <skos:narrower rdf:resource="descriptor/18399-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13034-4">
    <skos:narrower rdf:resource="descriptor/13032-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13034-4">
    <skos:narrower rdf:resource="descriptor/13028-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13034-4">
    <skos:narrower rdf:resource="descriptor/13085-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73304">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">P.18.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11753-3">
    <skos:broader rdf:resource="thsys/71114"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19048-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15441-2">
    <skos:prefLabel xml:lang="en">Prisoner's dilemma</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15441-2">
    <skos:prefLabel xml:lang="de">Gefangenendilemma</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73383">
    <skos:narrower rdf:resource="descriptor/16424-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73383">
    <skos:narrower rdf:resource="descriptor/16229-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73383">
    <skos:narrower rdf:resource="descriptor/16233-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73383">
    <skos:narrower rdf:resource="descriptor/16402-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73383">
    <skos:narrower rdf:resource="descriptor/16404-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73383">
    <skos:narrower rdf:resource="descriptor/16430-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73383">
    <skos:narrower rdf:resource="descriptor/11170-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73383">
    <skos:narrower rdf:resource="descriptor/16416-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73383">
    <skos:narrower rdf:resource="descriptor/11088-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73383">
    <skos:narrower rdf:resource="descriptor/16411-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73383">
    <skos:narrower rdf:resource="descriptor/16410-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73383">
    <skos:narrower rdf:resource="descriptor/19680-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73383">
    <skos:narrower rdf:resource="descriptor/16409-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73383">
    <skos:narrower rdf:resource="descriptor/16406-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73383">
    <skos:narrower rdf:resource="descriptor/16405-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73383">
    <skos:narrower rdf:resource="descriptor/20855-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73383">
    <skos:narrower rdf:resource="descriptor/16249-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73383">
    <skos:narrower rdf:resource="descriptor/10525-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73383">
    <skos:narrower rdf:resource="descriptor/10515-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73383">
    <skos:narrower rdf:resource="descriptor/16400-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73383">
    <skos:narrower rdf:resource="descriptor/16397-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16011-1">
    <skos:related rdf:resource="descriptor/10406-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16011-1">
    <skos:related rdf:resource="descriptor/11239-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16011-1">
    <skos:related rdf:resource="descriptor/11235-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16011-1">
    <skos:related rdf:resource="descriptor/11180-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15922-0">
    <skos:prefLabel xml:lang="en">American Indians</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15922-0">
    <skos:prefLabel xml:lang="de">Indianer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11484-4">
    <skos:prefLabel xml:lang="en">Reserve requirement policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11484-4">
    <skos:prefLabel xml:lang="de">Mindestreservepolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11337-3">
    <skos:scopeNote xml:lang="de">Theorie kollektiver Arbeitsverträge und -verhandlungen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19685-3">
    <skos:broader rdf:resource="thsys/73381"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19685-3">
    <skos:broader rdf:resource="descriptor/11970-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11696-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17743-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71095">
    <rdfs:label xml:lang="en">Currency</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71095">
    <rdfs:label xml:lang="de">Währung</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11727-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11576-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11124-6">
    <skos:related rdf:resource="descriptor/10985-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11124-6">
    <skos:related rdf:resource="descriptor/11044-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11820-0">
    <skos:narrower rdf:resource="descriptor/18405-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11820-0">
    <skos:narrower rdf:resource="descriptor/18417-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11820-0">
    <skos:narrower rdf:resource="descriptor/18447-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11820-0">
    <skos:narrower rdf:resource="descriptor/18419-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11820-0">
    <skos:narrower rdf:resource="descriptor/19251-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11820-0">
    <skos:narrower rdf:resource="descriptor/18482-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11820-0">
    <skos:narrower rdf:resource="descriptor/18481-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11820-0">
    <skos:narrower rdf:resource="descriptor/18425-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11820-0">
    <skos:narrower rdf:resource="descriptor/18375-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10373-4">
    <skos:related rdf:resource="descriptor/10375-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10373-4">
    <skos:related rdf:resource="descriptor/10374-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71013">
    <skos:broader rdf:resource="thsys/71016"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10626-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10454-4">
    <skos:scopeNote xml:lang="de">Verfahren oder Vorgänge, bei denen manuelle Tätigkeiten durch nicht-manuelle ersetzt werden, aber keine automatischen Prozesse umfassen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15909-6">
    <skos:prefLabel xml:lang="en">Population</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15909-6">
    <skos:prefLabel xml:lang="de">Bevölkerung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15684-4">
    <skos:broader rdf:resource="thsys/70843"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15684-4">
    <skos:broader rdf:resource="thsys/70143"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15684-4">
    <skos:broader rdf:resource="descriptor/15666-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15108-5">
    <skos:related rdf:resource="descriptor/10276-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11784-6">
    <skos:narrower rdf:resource="descriptor/11793-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11784-6">
    <skos:narrower rdf:resource="descriptor/11788-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11784-6">
    <skos:narrower rdf:resource="descriptor/10533-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13407-5">
    <skos:prefLabel xml:lang="en">Diplomatic mission</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13407-5">
    <skos:prefLabel xml:lang="de">Auslandsvertretung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19352-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24664-2">
    <skos:broader rdf:resource="thsys/70201"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24664-2">
    <skos:broader rdf:resource="descriptor/10030-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70476">
    <skos:narrower rdf:resource="descriptor/18078-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70476">
    <skos:narrower rdf:resource="descriptor/18257-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70476">
    <skos:narrower rdf:resource="descriptor/14716-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70476">
    <skos:narrower rdf:resource="descriptor/14715-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70476">
    <skos:narrower rdf:resource="descriptor/14713-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70476">
    <skos:narrower rdf:resource="descriptor/14712-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70476">
    <skos:narrower rdf:resource="descriptor/14711-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70476">
    <skos:narrower rdf:resource="descriptor/14710-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70476">
    <skos:narrower rdf:resource="descriptor/14705-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70476">
    <skos:narrower rdf:resource="descriptor/18291-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70476">
    <skos:narrower rdf:resource="descriptor/18230-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70476">
    <skos:narrower rdf:resource="descriptor/14702-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70476">
    <skos:narrower rdf:resource="descriptor/14703-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70476">
    <skos:narrower rdf:resource="descriptor/14700-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70476">
    <skos:narrower rdf:resource="descriptor/14698-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70476">
    <skos:narrower rdf:resource="descriptor/14854-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70476">
    <skos:narrower rdf:resource="descriptor/14883-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12361-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16604-2">
    <skos:related rdf:resource="descriptor/19766-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16604-2">
    <skos:related rdf:resource="descriptor/16606-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16604-2">
    <skos:related rdf:resource="descriptor/13662-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16604-2">
    <skos:related rdf:resource="descriptor/10142-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16604-2">
    <skos:related rdf:resource="descriptor/15961-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16604-2">
    <skos:related rdf:resource="descriptor/18944-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16604-2">
    <skos:related rdf:resource="descriptor/13345-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16604-2">
    <skos:related rdf:resource="descriptor/16754-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16604-2">
    <skos:related rdf:resource="descriptor/16601-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15477-2">
    <skos:related rdf:resource="descriptor/19771-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12755-1">
    <skos:related rdf:resource="descriptor/12727-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12755-1">
    <skos:related rdf:resource="descriptor/12656-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12755-1">
    <skos:related rdf:resource="descriptor/12035-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12755-1">
    <skos:related rdf:resource="descriptor/12747-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70818">
    <skos:broader rdf:resource="thsys/w"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71032">
    <skos:narrower rdf:resource="descriptor/17595-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71032">
    <skos:narrower rdf:resource="descriptor/17503-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71032">
    <skos:narrower rdf:resource="descriptor/17502-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71032">
    <skos:narrower rdf:resource="descriptor/17500-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71032">
    <skos:narrower rdf:resource="descriptor/17593-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71032">
    <skos:narrower rdf:resource="descriptor/17497-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71032">
    <skos:narrower rdf:resource="descriptor/17496-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71032">
    <skos:narrower rdf:resource="descriptor/17472-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71032">
    <skos:narrower rdf:resource="descriptor/17491-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71032">
    <skos:narrower rdf:resource="descriptor/17464-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19261-5">
    <skos:related rdf:resource="descriptor/10772-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19261-5">
    <skos:related rdf:resource="descriptor/13691-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11372-1">
    <skos:prefLabel xml:lang="en">University teachers</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11372-1">
    <skos:prefLabel xml:lang="de">Hochschullehrer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19000-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13002-3">
    <skos:altLabel xml:lang="de">Arzneimittelindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19345-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13584-4">
    <skos:broader rdf:resource="thsys/70928"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10870-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71030">
    <skos:broader rdf:resource="thsys/70026"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12420-4">
    <skos:prefLabel xml:lang="en">Constant (Accounting valuation)</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12420-4">
    <skos:prefLabel xml:lang="de">Festwert</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12533-5">
    <skos:prefLabel xml:lang="en">Cost unit statement of income</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12533-5">
    <skos:prefLabel xml:lang="de">Kostenträgerrechnung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18256-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14548-3">
    <skos:narrower rdf:resource="descriptor/14545-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14548-3">
    <skos:narrower rdf:resource="descriptor/14549-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13188-5">
    <skos:altLabel xml:lang="de">Landwarenhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13188-5">
    <skos:altLabel xml:lang="de">Landhandelsgenossenschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13188-5">
    <skos:altLabel xml:lang="de">Agrarhandelsgenossenschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13188-5">
    <skos:altLabel xml:lang="de">Agrarhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13188-5">
    <skos:altLabel xml:lang="de">Agrargroßhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13188-5">
    <skos:altLabel xml:lang="de">Agrareinzelhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16273-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16966-3">
    <skos:broader rdf:resource="thsys/70003"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16966-3">
    <skos:broader rdf:resource="descriptor/16972-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17713-6">
    <skos:broader rdf:resource="thsys/70560"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17713-6">
    <skos:broader rdf:resource="descriptor/17724-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14130-3">
    <skos:prefLabel xml:lang="en">Wool</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14130-3">
    <skos:prefLabel xml:lang="de">Wolle</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18978-0">
    <skos:broader rdf:resource="thsys/71052"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18978-0">
    <skos:broader rdf:resource="descriptor/11284-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18978-0">
    <skos:broader rdf:resource="descriptor/11302-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14011-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11389-5">
    <skos:related rdf:resource="descriptor/11390-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11389-5">
    <skos:related rdf:resource="descriptor/12573-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11389-5">
    <skos:related rdf:resource="descriptor/11299-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11389-5">
    <skos:related rdf:resource="descriptor/19825-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11389-5">
    <skos:related rdf:resource="descriptor/11392-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11389-5">
    <skos:related rdf:resource="descriptor/11235-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11389-5">
    <skos:related rdf:resource="descriptor/11189-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15769-3">
    <skos:altLabel xml:lang="de">Umweltschutzrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15769-3">
    <skos:altLabel xml:lang="de">Umweltschutzgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15769-3">
    <skos:altLabel xml:lang="de">Umweltgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14725-0">
    <skos:altLabel xml:lang="de">Insulin</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10880-1">
    <skos:prefLabel xml:lang="en">International trade rules</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10880-1">
    <skos:prefLabel xml:lang="de">Internationale Handelsregeln</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13573-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71073">
    <skos:prefLabel xml:lang="en">V.09.08  Financial law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71073">
    <skos:prefLabel xml:lang="de">V.09.08  Finanzrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20855-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17017-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12064-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12765-5">
    <skos:related rdf:resource="descriptor/19784-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12765-5">
    <skos:related rdf:resource="descriptor/15051-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12765-5">
    <skos:related rdf:resource="descriptor/19074-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12765-5">
    <skos:related rdf:resource="descriptor/11187-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12765-5">
    <skos:related rdf:resource="descriptor/12747-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10498-5">
    <skos:related rdf:resource="descriptor/10511-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10498-5">
    <skos:related rdf:resource="descriptor/16638-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10498-5">
    <skos:related rdf:resource="descriptor/15998-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10498-5">
    <skos:related rdf:resource="descriptor/19506-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10498-5">
    <skos:related rdf:resource="descriptor/10494-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10498-5">
    <skos:related rdf:resource="descriptor/18894-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10498-5">
    <skos:related rdf:resource="descriptor/10488-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19560-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18459-3">
    <skos:prefLabel xml:lang="en">Rural industrialization</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18459-3">
    <skos:prefLabel xml:lang="de">Ländliche Industrialisierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11754-1">
    <skos:broader rdf:resource="thsys/71115"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11754-1">
    <skos:broader rdf:resource="thsys/70049"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13613-2">
    <skos:altLabel xml:lang="de">Tarifgemeinschaft im Verkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13613-2">
    <skos:altLabel xml:lang="de">ÖPNV-Verkehrsverbund</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19336-0">
    <skos:broader rdf:resource="thsys/73362"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19336-0">
    <skos:broader rdf:resource="thsys/71092"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19336-0">
    <skos:broader rdf:resource="descriptor/10627-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19336-0">
    <skos:broader rdf:resource="descriptor/10771-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19336-0">
    <skos:broader rdf:resource="descriptor/16495-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16471-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11734-0">
    <skos:altLabel xml:lang="de">Verdrängungseffekt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17922-4">
    <skos:altLabel xml:lang="de">Kooperative Republik Guyana</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17922-4">
    <skos:altLabel xml:lang="de">Guayana</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17922-4">
    <skos:altLabel xml:lang="de">Britisch-Guayana</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13645-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19557-5">
    <skos:prefLabel xml:lang="en">Project finance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19557-5">
    <skos:prefLabel xml:lang="de">Projektfinanzierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14157-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11801-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16713-4">
    <skos:prefLabel xml:lang="en">Humanism</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16713-4">
    <skos:prefLabel xml:lang="de">Humanismus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10575-6">
    <zbwext:useInsteadNote rdf:nodeID="id186"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12809-4">
    <skos:prefLabel xml:lang="en">Product quality</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12809-4">
    <skos:prefLabel xml:lang="de">Produktqualität</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15638-4">
    <skos:broader rdf:resource="thsys/73349"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15638-4">
    <skos:broader rdf:resource="descriptor/15664-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16432-5">
    <skos:narrower rdf:resource="descriptor/10993-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20468-1">
    <skos:related rdf:resource="descriptor/12158-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20468-1">
    <skos:related rdf:resource="descriptor/19122-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20468-1">
    <skos:related rdf:resource="descriptor/18070-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20468-1">
    <skos:related rdf:resource="descriptor/10459-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20468-1">
    <skos:related rdf:resource="descriptor/19146-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15719-4">
    <skos:altLabel xml:lang="de">Gesellschaftlicher Konflikt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18124-6">
    <skos:related rdf:resource="descriptor/19199-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19054-3">
    <skos:broader rdf:resource="thsys/70969"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19054-3">
    <skos:broader rdf:resource="descriptor/12905-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16124-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10770-1">
    <skos:altLabel xml:lang="de">Internationale Währungsreserven</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10770-1">
    <skos:altLabel xml:lang="de">Internationale Liquidität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10770-1">
    <skos:altLabel xml:lang="de">Devisenreserven</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10070-1">
    <skos:related rdf:resource="descriptor/10227-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10070-1">
    <skos:related rdf:resource="descriptor/15458-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10070-1">
    <skos:related rdf:resource="descriptor/10217-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10070-1">
    <skos:related rdf:resource="descriptor/10081-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10070-1">
    <skos:related rdf:resource="descriptor/10080-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10070-1">
    <skos:related rdf:resource="descriptor/10073-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71093">
    <rdfs:label xml:lang="en">Economic integration</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71093">
    <rdfs:label xml:lang="de">Wirtschaftsintegration</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11642-5">
    <skos:broader rdf:resource="thsys/71077"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14042-0">
    <skos:related rdf:resource="descriptor/19063-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14042-0">
    <skos:related rdf:resource="descriptor/14728-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14042-0">
    <skos:related rdf:resource="descriptor/15658-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14042-0">
    <skos:related rdf:resource="descriptor/14798-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14042-0">
    <skos:related rdf:resource="descriptor/14817-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14042-0">
    <skos:related rdf:resource="descriptor/13371-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14042-0">
    <skos:related rdf:resource="descriptor/14531-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14042-0">
    <skos:related rdf:resource="descriptor/19053-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14042-0">
    <skos:related rdf:resource="descriptor/14527-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12371-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15064-6">
    <zbwext:useInsteadNote rdf:nodeID="id313"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15969-2">
    <skos:broader rdf:resource="thsys/73394"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15969-2">
    <skos:broader rdf:resource="thsys/70154"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15969-2">
    <skos:broader rdf:resource="descriptor/16719-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17793-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19745-4">
    <skos:related rdf:resource="descriptor/15115-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19745-4">
    <skos:related rdf:resource="descriptor/15814-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19745-4">
    <skos:related rdf:resource="descriptor/13383-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19745-4">
    <skos:related rdf:resource="descriptor/15821-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19745-4">
    <skos:related rdf:resource="descriptor/12774-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14094-2">
    <skos:altLabel xml:lang="de">Manihot</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14094-2">
    <skos:altLabel xml:lang="de">Cassave</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19756-6">
    <skos:broader rdf:resource="thsys/73358"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19756-6">
    <skos:broader rdf:resource="descriptor/15713-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14196-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17943-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70258">
    <skos:broader rdf:resource="thsys/70257"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71063">
    <skos:broader rdf:resource="thsys/70058"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11297-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16648-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10007-1">
    <skos:related rdf:resource="descriptor/15086-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10007-1">
    <skos:related rdf:resource="descriptor/12721-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10007-1">
    <skos:related rdf:resource="descriptor/10656-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10007-1">
    <skos:related rdf:resource="descriptor/12913-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19975-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13291-5">
    <skos:narrower rdf:resource="descriptor/13272-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13291-5">
    <skos:narrower rdf:resource="descriptor/13702-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13291-5">
    <skos:narrower rdf:resource="descriptor/12281-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13291-5">
    <skos:narrower rdf:resource="descriptor/18086-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13291-5">
    <skos:narrower rdf:resource="descriptor/13703-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13291-5">
    <skos:narrower rdf:resource="descriptor/10550-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13291-5">
    <skos:narrower rdf:resource="descriptor/13285-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13291-5">
    <skos:narrower rdf:resource="descriptor/13284-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13291-5">
    <skos:narrower rdf:resource="descriptor/18540-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18890-0">
    <skos:prefLabel xml:lang="en">Health personnel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18890-0">
    <skos:prefLabel xml:lang="de">Gesundheitsberufe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15415-3">
    <skos:related rdf:resource="descriptor/15429-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15415-3">
    <skos:related rdf:resource="descriptor/15624-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15415-3">
    <skos:related rdf:resource="descriptor/15458-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15415-3">
    <skos:related rdf:resource="descriptor/19751-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15415-3">
    <skos:related rdf:resource="descriptor/15054-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15415-3">
    <skos:related rdf:resource="descriptor/19808-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15415-3">
    <skos:related rdf:resource="descriptor/19725-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15415-3">
    <skos:related rdf:resource="descriptor/10462-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15415-3">
    <skos:related rdf:resource="descriptor/15416-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15415-3">
    <skos:related rdf:resource="descriptor/15045-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15415-3">
    <skos:related rdf:resource="descriptor/15609-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15415-3">
    <skos:related rdf:resource="descriptor/15470-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15415-3">
    <skos:related rdf:resource="descriptor/15633-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13058-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11687-4">
    <skos:prefLabel xml:lang="en">Negative income tax</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11687-4">
    <skos:prefLabel xml:lang="de">Negative Einkommensteuer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19209-0">
    <skos:broader rdf:resource="thsys/71116"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19209-0">
    <skos:broader rdf:resource="thsys/70057"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19209-0">
    <skos:broader rdf:resource="descriptor/10040-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70925">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.12.06</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15798-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19809-4">
    <skos:prefLabel xml:lang="en">Virtual reality</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19809-4">
    <skos:prefLabel xml:lang="de">Virtuelle Realität</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70060">
    <skos:narrower rdf:resource="descriptor/15644-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70060">
    <skos:narrower rdf:resource="thsys/70580"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70060">
    <skos:narrower rdf:resource="thsys/70581"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70060">
    <skos:narrower rdf:resource="thsys/70143"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10499-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15883-5">
    <skos:prefLabel xml:lang="en">Coastal area</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15883-5">
    <skos:prefLabel xml:lang="de">Küstenregion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13179-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18831-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14151-2">
    <skos:broader rdf:resource="thsys/70201"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18585-5">
    <skos:altLabel xml:lang="de">Verkehrsleitsystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18208-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19084-1">
    <skos:altLabel xml:lang="de">Ostmitteleuropäische Staaten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19084-1">
    <skos:altLabel xml:lang="de">Ostmitteleuropäische Länder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19084-1">
    <skos:altLabel xml:lang="de">MOEL (Mittel-Osteuropäische Länder)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19084-1">
    <skos:altLabel xml:lang="de">MOE (Mittel-Osteuropäische Staaten)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19084-1">
    <skos:altLabel xml:lang="de">Mittel-Osteuropäische Staaten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19084-1">
    <skos:altLabel xml:lang="de">Mittelosteuropa</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19084-1">
    <skos:altLabel xml:lang="de">Mitteleuropa (Ost)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24664-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16995-3">
    <skos:prefLabel xml:lang="en">Denmark</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16995-3">
    <skos:prefLabel xml:lang="de">Dänemark</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10567-5">
    <skos:scopeNote xml:lang="de">Freiräume für wirtschaftliche Aktivitäten, in denen die landesüblichen Regulierungen ganz oder teilweise ausgesetzt sind.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18095-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70003">
    <skos:broader rdf:resource="thsys/70554"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18988-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18648-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13561-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16198-1">
    <skos:broader rdf:resource="thsys/73334"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16198-1">
    <skos:broader rdf:resource="descriptor/16007-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18548-4">
    <skos:related rdf:resource="descriptor/18550-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18548-4">
    <skos:related rdf:resource="descriptor/18546-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19012-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12703-6">
    <skos:altLabel xml:lang="de">Montagetechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12617-6">
    <skos:prefLabel xml:lang="en">Occupational psychology</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12617-6">
    <skos:prefLabel xml:lang="de">Arbeitspsychologie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19546-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71078">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11562-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16273-6">
    <skos:prefLabel xml:lang="en">Politics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16273-6">
    <skos:prefLabel xml:lang="de">Politik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71046">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">B.02.01.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10487-3">
    <skos:related rdf:resource="descriptor/10473-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10487-3">
    <skos:related rdf:resource="descriptor/10472-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10487-3">
    <skos:related rdf:resource="descriptor/18073-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19855-4">
    <skos:related rdf:resource="descriptor/16332-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19855-4">
    <skos:related rdf:resource="descriptor/16304-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19855-4">
    <skos:related rdf:resource="descriptor/16521-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19855-4">
    <skos:related rdf:resource="descriptor/16191-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19855-4">
    <skos:related rdf:resource="descriptor/19847-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18838-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12391-6">
    <skos:prefLabel xml:lang="en">Illusory earnings</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12391-6">
    <skos:prefLabel xml:lang="de">Scheingewinn</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10513-0">
    <skos:related rdf:resource="descriptor/10414-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10513-0">
    <skos:related rdf:resource="descriptor/10342-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10513-0">
    <skos:related rdf:resource="descriptor/10514-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10513-0">
    <skos:related rdf:resource="descriptor/18325-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10513-0">
    <skos:related rdf:resource="descriptor/10525-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10513-0">
    <skos:related rdf:resource="descriptor/10560-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10513-0">
    <skos:related rdf:resource="descriptor/10492-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10513-0">
    <skos:related rdf:resource="descriptor/10494-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10513-0">
    <skos:related rdf:resource="descriptor/19361-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19316-6">
    <skos:altLabel xml:lang="en">Rule of origin</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19316-6">
    <skos:altLabel xml:lang="en">Made in</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19316-6">
    <skos:altLabel xml:lang="en">Local content</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19316-6">
    <skos:altLabel xml:lang="de">Ursprungszeugnis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19316-6">
    <skos:altLabel xml:lang="de">Ursprungsregelung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19316-6">
    <skos:altLabel xml:lang="de">Ursprungsregeln</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19316-6">
    <skos:altLabel xml:lang="de">Ursprungserzeugnis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19316-6">
    <skos:altLabel xml:lang="de">Produktherkunft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19316-6">
    <skos:altLabel xml:lang="de">Identitätsnachweis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14065-2">
    <skos:broader rdf:resource="thsys/73298"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14065-2">
    <skos:broader rdf:resource="thsys/70341"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14065-2">
    <skos:broader rdf:resource="descriptor/18222-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16110-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71115">
    <skos:broader rdf:resource="thsys/70234"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13355-5">
    <skos:prefLabel xml:lang="en">Artists</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13355-5">
    <skos:prefLabel xml:lang="de">Künstlerische Berufe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20404-6">
    <skos:broader rdf:resource="thsys/70383"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20404-6">
    <skos:broader rdf:resource="descriptor/16782-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15624-1">
    <skos:scopeNote xml:lang="de">Mit Hilfe der Systemanalyse soll für ein gegebenes Aufgabengebiet ein funktionsfähiges Informationssystem erstellt werden.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72186">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11046-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17476-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18653-0">
    <skos:related rdf:resource="descriptor/18643-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18653-0">
    <skos:related rdf:resource="descriptor/18663-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18653-0">
    <skos:related rdf:resource="descriptor/18649-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18653-0">
    <skos:related rdf:resource="descriptor/18652-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18653-0">
    <skos:related rdf:resource="descriptor/18651-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18653-0">
    <skos:related rdf:resource="descriptor/13278-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18653-0">
    <skos:related rdf:resource="descriptor/16516-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13095-5">
    <skos:altLabel xml:lang="de">Edelsteinschleiferei</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13095-5">
    <skos:altLabel xml:lang="de">Edelsteinindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13095-5">
    <skos:altLabel xml:lang="de">Bijouterieindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12399-4">
    <skos:prefLabel xml:lang="en">Current assets</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12399-4">
    <skos:prefLabel xml:lang="de">Umlaufvermögen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13647-6">
    <skos:broader rdf:resource="thsys/73389"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13647-6">
    <skos:broader rdf:resource="thsys/70923"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13647-6">
    <skos:broader rdf:resource="descriptor/13569-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19820-2">
    <skos:scopeNote xml:lang="de">Rechtssystem, das in England entwickelt wurde und sich hauptsächlich auf Gewohnheitsrecht und vorausgegangene Entscheidungen höherer Gerichte stützt.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14600-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15061-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19481-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18454-6">
    <skos:narrower rdf:resource="descriptor/13071-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10488-1">
    <skos:prefLabel xml:lang="en">Poverty</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10488-1">
    <skos:prefLabel xml:lang="de">Armut</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16349-6">
    <skos:broader rdf:resource="thsys/73376"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16349-6">
    <skos:broader rdf:resource="thsys/73362"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16349-6">
    <skos:broader rdf:resource="descriptor/16354-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16349-6">
    <skos:broader rdf:resource="descriptor/16495-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11092-0">
    <skos:altLabel xml:lang="en">Surplus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11092-0">
    <skos:altLabel xml:lang="de">Mehrwertrate</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11092-0">
    <skos:altLabel xml:lang="de">Mehrwert</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11092-0">
    <skos:altLabel xml:lang="de">Ausbeutungsrate</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21011-6">
    <skos:prefLabel xml:lang="en">Tax increase</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21011-6">
    <skos:prefLabel xml:lang="de">Steuererhöhung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id24">
    <rdfs:seeAlso rdf:resource="descriptor/19492-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13220-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18702-6">
    <skos:altLabel xml:lang="de">Kapitalschutz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17713-6">
    <skos:prefLabel xml:lang="en">Mauritania</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17713-6">
    <skos:prefLabel xml:lang="de">Mauretanien</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14962-0">
    <skos:altLabel xml:lang="de">Tran</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14962-0">
    <skos:altLabel xml:lang="de">Tieröl</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14962-0">
    <skos:altLabel xml:lang="de">Talg</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14962-0">
    <skos:altLabel xml:lang="de">Schmalz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14962-0">
    <skos:altLabel xml:lang="de">Lebertran</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14962-0">
    <skos:altLabel xml:lang="de">Geflügelfett</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14962-0">
    <skos:altLabel xml:lang="de">Fischöl</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16187-6">
    <skos:prefLabel xml:lang="en">Maritime law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16187-6">
    <skos:prefLabel xml:lang="de">Seerecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19628-1">
    <skos:prefLabel xml:lang="en">Nature reserve</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19628-1">
    <skos:prefLabel xml:lang="de">Naturschutzgebiet</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14117-2">
    <skos:prefLabel xml:lang="en">Tropical fruit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14117-2">
    <skos:prefLabel xml:lang="de">Tropische Frucht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11294-2">
    <skos:altLabel xml:lang="de">Stellung im Beruf</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11294-2">
    <skos:altLabel xml:lang="de">Status im Beruf</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11294-2">
    <skos:altLabel xml:lang="de">Berufsprestige</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11294-2">
    <skos:altLabel xml:lang="de">Berufsposition</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11294-2">
    <skos:altLabel xml:lang="de">Beruflicher Status</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19303-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19356-1">
    <skos:broader rdf:resource="thsys/71091"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19356-1">
    <skos:broader rdf:resource="descriptor/10013-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14512-3">
    <skos:broader rdf:resource="thsys/70477"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14512-3">
    <skos:broader rdf:resource="descriptor/14502-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11384-1">
    <skos:broader rdf:resource="thsys/70843"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11384-1">
    <skos:broader rdf:resource="thsys/70244"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11384-1">
    <skos:broader rdf:resource="thsys/70176"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11384-1">
    <skos:broader rdf:resource="descriptor/11401-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11384-1">
    <skos:broader rdf:resource="descriptor/16134-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13047-2">
    <skos:related rdf:resource="descriptor/13513-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13047-2">
    <skos:related rdf:resource="descriptor/12925-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13047-2">
    <skos:related rdf:resource="descriptor/13058-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10830-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11290-3">
    <skos:related rdf:resource="descriptor/19112-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11290-3">
    <skos:related rdf:resource="descriptor/19496-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11290-3">
    <skos:related rdf:resource="descriptor/11295-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11290-3">
    <skos:related rdf:resource="descriptor/18850-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11290-3">
    <skos:related rdf:resource="descriptor/11298-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11290-3">
    <skos:related rdf:resource="descriptor/12618-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11290-3">
    <skos:related rdf:resource="descriptor/11293-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11290-3">
    <skos:related rdf:resource="descriptor/19825-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11290-3">
    <skos:related rdf:resource="descriptor/11252-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11290-3">
    <skos:related rdf:resource="descriptor/19505-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11290-3">
    <skos:related rdf:resource="descriptor/11361-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11290-3">
    <skos:related rdf:resource="descriptor/11303-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14496-3">
    <skos:prefLabel xml:lang="en">Solar cell</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14496-3">
    <skos:prefLabel xml:lang="de">Solarzelle</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19110-5">
    <skos:broader rdf:resource="thsys/71051"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19110-5">
    <skos:broader rdf:resource="descriptor/16354-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18775-0">
    <skos:scopeNote xml:lang="de">Auch für die Zusammenarbeit zwischen öffentlichen und privaten Trägern.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19416-2">
    <skos:scopeNote xml:lang="de">Zweckgebundene, zeitlich begrenzte Sondersteuern.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14496-3">
    <skos:broader rdf:resource="thsys/70254"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14496-3">
    <skos:broader rdf:resource="descriptor/18251-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13851-0">
    <skos:prefLabel xml:lang="en">Construction costs</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13851-0">
    <skos:prefLabel xml:lang="de">Baukosten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18349-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16666-1">
    <skos:scopeNote xml:lang="de">Nur im Sinne der Problematisierung des Alkoholkonsums.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19582-6">
    <skos:broader rdf:resource="thsys/72393"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19582-6">
    <skos:broader rdf:resource="descriptor/15376-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18499-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19080-2">
    <skos:related rdf:resource="descriptor/13464-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19080-2">
    <skos:related rdf:resource="descriptor/16050-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19080-2">
    <skos:related rdf:resource="descriptor/19043-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19080-2">
    <skos:related rdf:resource="descriptor/12273-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19080-2">
    <skos:related rdf:resource="descriptor/12356-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10019-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12194-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11044-4">
    <skos:prefLabel xml:lang="en">Common property</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11044-4">
    <skos:prefLabel xml:lang="de">Gesellschaftliches Eigentum</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11402-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19473-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11180-3">
    <skos:broader rdf:resource="thsys/70113"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11180-3">
    <skos:broader rdf:resource="thsys/70052"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11180-3">
    <skos:broader rdf:resource="descriptor/10326-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19440-5">
    <skos:prefLabel xml:lang="en">Regional wage structure</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19440-5">
    <skos:prefLabel xml:lang="de">Regionale Lohnstruktur</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12939-5">
    <skos:prefLabel xml:lang="en">Forage crops</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12939-5">
    <skos:prefLabel xml:lang="de">Futterbau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16205-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11490-2">
    <skos:related rdf:resource="descriptor/10137-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11490-2">
    <skos:related rdf:resource="descriptor/10135-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11490-2">
    <skos:related rdf:resource="descriptor/13763-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11490-2">
    <skos:related rdf:resource="descriptor/10387-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12299-1">
    <skos:narrower rdf:resource="descriptor/19264-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12299-1">
    <skos:narrower rdf:resource="descriptor/13767-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12299-1">
    <skos:narrower rdf:resource="descriptor/13727-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12299-1">
    <skos:narrower rdf:resource="descriptor/13726-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12299-1">
    <skos:narrower rdf:resource="descriptor/13721-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12299-1">
    <skos:narrower rdf:resource="descriptor/12232-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19310-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11356-6">
    <skos:broader rdf:resource="thsys/71052"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11356-6">
    <skos:broader rdf:resource="thsys/70844"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11356-6">
    <skos:broader rdf:resource="descriptor/11294-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11356-6">
    <skos:broader rdf:resource="descriptor/11355-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11416-0">
    <skos:prefLabel xml:lang="en">Monetary history</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11416-0">
    <skos:prefLabel xml:lang="de">Geldgeschichte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19362-6">
    <skos:broader rdf:resource="thsys/71065"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19362-6">
    <skos:broader rdf:resource="thsys/70504"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19362-6">
    <skos:broader rdf:resource="thsys/70091"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19362-6">
    <skos:broader rdf:resource="descriptor/10488-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19266-2">
    <skos:broader rdf:resource="thsys/70455"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19266-2">
    <skos:broader rdf:resource="descriptor/19224-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14664-1">
    <skos:altLabel xml:lang="de">Propan</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14664-1">
    <skos:altLabel xml:lang="de">Paraffin</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14664-1">
    <skos:altLabel xml:lang="de">Methan</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14664-1">
    <skos:altLabel xml:lang="de">Ethan</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14664-1">
    <skos:altLabel xml:lang="de">Butan</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14664-1">
    <skos:altLabel xml:lang="de">Äthan</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73335">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.05.07</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10560-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11340-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18908-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14809-1">
    <skos:altLabel xml:lang="de">Wachspapier</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14809-1">
    <skos:altLabel xml:lang="de">Technisches Papier</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14809-1">
    <skos:altLabel xml:lang="de">Sonderpapier</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14809-1">
    <skos:altLabel xml:lang="de">Pergamentpapier</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14809-1">
    <skos:altLabel xml:lang="de">Gummiertes Papier</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14809-1">
    <skos:altLabel xml:lang="de">Gestrichenes Papier</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14809-1">
    <skos:altLabel xml:lang="de">Beschichtetes Papier</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14809-1">
    <skos:altLabel xml:lang="de">Beschichtete Pappe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14100-5">
    <skos:broader rdf:resource="thsys/70169"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14100-5">
    <skos:broader rdf:resource="descriptor/14117-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19670-2">
    <skos:narrower rdf:resource="descriptor/18912-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73304">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16037-4">
    <skos:related rdf:resource="descriptor/13392-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16037-4">
    <skos:related rdf:resource="descriptor/13367-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16037-4">
    <skos:related rdf:resource="descriptor/20110-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16037-4">
    <skos:related rdf:resource="descriptor/10481-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16037-4">
    <skos:related rdf:resource="descriptor/13660-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16037-4">
    <skos:related rdf:resource="descriptor/15961-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16037-4">
    <skos:related rdf:resource="descriptor/16053-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16037-4">
    <skos:related rdf:resource="descriptor/18433-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19320-1">
    <skos:altLabel xml:lang="de">Vergeltungszoll</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19320-1">
    <skos:altLabel xml:lang="de">Retorsionszoll</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19474-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10576-4">
    <skos:altLabel xml:lang="de">Beratung (Landwirtschaft)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10576-4">
    <skos:altLabel xml:lang="de">Agrarwirtschaftsberatung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10576-4">
    <skos:altLabel xml:lang="de">Agrarberatung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13410-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18963-6">
    <skos:broader rdf:resource="thsys/70816"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18963-6">
    <skos:broader rdf:resource="descriptor/13324-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19342-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18328-4">
    <skos:broader rdf:resource="thsys/71066"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18328-4">
    <skos:broader rdf:resource="thsys/71023"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18328-4">
    <skos:broader rdf:resource="descriptor/10313-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14776-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16479-0">
    <skos:related rdf:resource="descriptor/16483-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16479-0">
    <skos:related rdf:resource="descriptor/16545-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16479-0">
    <skos:related rdf:resource="descriptor/18900-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16479-0">
    <skos:related rdf:resource="descriptor/16670-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16479-0">
    <skos:related rdf:resource="descriptor/13371-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16479-0">
    <skos:related rdf:resource="descriptor/18892-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16479-0">
    <skos:related rdf:resource="descriptor/18891-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16479-0">
    <skos:related rdf:resource="descriptor/18444-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16479-0">
    <skos:related rdf:resource="descriptor/11157-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16479-0">
    <skos:related rdf:resource="descriptor/16476-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16479-0">
    <skos:related rdf:resource="descriptor/18958-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70186">
    <skos:prefLabel xml:lang="en">B.03.01  Annual accounts</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70186">
    <skos:prefLabel xml:lang="de">B.03.01  Jahresabschluss</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18671-5">
    <skos:prefLabel xml:lang="en">Financial sector</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18671-5">
    <skos:prefLabel xml:lang="de">Finanzsektor</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14222-5">
    <skos:broader rdf:resource="thsys/70650"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14222-5">
    <skos:broader rdf:resource="thsys/70134"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14222-5">
    <skos:broader rdf:resource="descriptor/14302-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14222-5">
    <skos:broader rdf:resource="descriptor/14181-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18640-2">
    <skos:altLabel xml:lang="de">Fremdenverkehrsökologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14798-1">
    <skos:broader rdf:resource="thsys/70328"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14798-1">
    <skos:broader rdf:resource="descriptor/14803-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14798-1">
    <skos:broader rdf:resource="descriptor/14032-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13844-4">
    <skos:broader rdf:resource="thsys/70969"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13844-4">
    <skos:broader rdf:resource="descriptor/12699-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13844-4">
    <skos:broader rdf:resource="descriptor/12905-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10124-4">
    <skos:scopeNote xml:lang="de">Kapitalmobilität innerhalb einer Volkswirtschaft.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19623-4">
    <skos:altLabel xml:lang="de">Luftverflüssigung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19623-4">
    <skos:altLabel xml:lang="de">Flüssige Luft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73371">
    <skos:broader rdf:resource="thsys/73374"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70036">
    <rdfs:label xml:lang="de">Kapitalverwendung</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70847">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.24</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14211-3">
    <skos:prefLabel xml:lang="en">Peat</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14211-3">
    <skos:prefLabel xml:lang="de">Torf</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73373">
    <skos:narrower rdf:resource="descriptor/15979-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73373">
    <skos:narrower rdf:resource="descriptor/16332-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73373">
    <skos:narrower rdf:resource="descriptor/18068-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73373">
    <skos:narrower rdf:resource="descriptor/15970-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73373">
    <skos:narrower rdf:resource="descriptor/19705-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73373">
    <skos:narrower rdf:resource="descriptor/16513-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73373">
    <skos:narrower rdf:resource="descriptor/16328-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73373">
    <skos:narrower rdf:resource="descriptor/16283-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73373">
    <skos:narrower rdf:resource="descriptor/16304-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73373">
    <skos:narrower rdf:resource="descriptor/16512-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73373">
    <skos:narrower rdf:resource="descriptor/16486-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73373">
    <skos:narrower rdf:resource="descriptor/19704-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73373">
    <skos:narrower rdf:resource="descriptor/16510-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73373">
    <skos:narrower rdf:resource="descriptor/16509-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19463-0">
    <skos:broader rdf:resource="thsys/71065"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19463-0">
    <skos:broader rdf:resource="thsys/71063"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19463-0">
    <skos:broader rdf:resource="descriptor/11716-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11059-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70313">
    <skos:broader rdf:resource="thsys/p"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19421-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16429-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19435-5">
    <skos:prefLabel xml:lang="en">Quantity regulation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19435-5">
    <skos:prefLabel xml:lang="de">Quotenregulierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12962-3">
    <skos:prefLabel xml:lang="en">Hunting</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12962-3">
    <skos:prefLabel xml:lang="de">Jagd</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10013-6">
    <skos:prefLabel xml:lang="en">Demand</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10013-6">
    <skos:prefLabel xml:lang="de">Nachfrage</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14033-1">
    <skos:related rdf:resource="descriptor/14567-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14033-1">
    <skos:related rdf:resource="descriptor/13214-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14033-1">
    <skos:related rdf:resource="descriptor/18915-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14033-1">
    <skos:related rdf:resource="descriptor/14032-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14033-1">
    <skos:related rdf:resource="descriptor/14029-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14033-1">
    <skos:related rdf:resource="descriptor/14743-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14033-1">
    <skos:related rdf:resource="descriptor/14021-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12032-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11567-0">
    <skos:prefLabel xml:lang="en">Food subsidy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11567-0">
    <skos:prefLabel xml:lang="de">Nahrungsmittelsubvention</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16717-3">
    <skos:prefLabel xml:lang="en">Materialism</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16717-3">
    <skos:prefLabel xml:lang="de">Materialismus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10595-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12693-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18793-5">
    <skos:prefLabel xml:lang="en">Experts</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18793-5">
    <skos:prefLabel xml:lang="de">Sachverständige</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71104">
    <skos:narrower rdf:resource="descriptor/18693-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71104">
    <skos:narrower rdf:resource="descriptor/13744-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71104">
    <skos:narrower rdf:resource="descriptor/10221-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71104">
    <skos:narrower rdf:resource="descriptor/18691-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71104">
    <skos:narrower rdf:resource="descriptor/18438-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71104">
    <skos:narrower rdf:resource="descriptor/18690-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71104">
    <skos:narrower rdf:resource="descriptor/10210-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71104">
    <skos:narrower rdf:resource="descriptor/13735-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71104">
    <skos:narrower rdf:resource="descriptor/18689-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71104">
    <skos:narrower rdf:resource="descriptor/13722-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71104">
    <skos:narrower rdf:resource="descriptor/13720-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71104">
    <skos:narrower rdf:resource="descriptor/18688-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71104">
    <skos:narrower rdf:resource="descriptor/19280-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71104">
    <skos:narrower rdf:resource="descriptor/18687-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19357-6">
    <skos:altLabel xml:lang="de">Außenwirtschaft der Landesteile</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13970-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15637-6">
    <skos:prefLabel xml:lang="en">Buddhism</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15637-6">
    <skos:prefLabel xml:lang="de">Buddhismus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14437-5">
    <skos:related rdf:resource="descriptor/19059-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14437-5">
    <skos:related rdf:resource="descriptor/13630-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17928-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13529-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13580-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16754-4">
    <skos:related rdf:resource="descriptor/19253-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16754-4">
    <skos:related rdf:resource="descriptor/10326-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16754-4">
    <skos:related rdf:resource="descriptor/15798-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16754-4">
    <skos:related rdf:resource="descriptor/16760-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16754-4">
    <skos:related rdf:resource="descriptor/19475-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16754-4">
    <skos:related rdf:resource="descriptor/15598-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16754-4">
    <skos:related rdf:resource="descriptor/18943-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16754-4">
    <skos:related rdf:resource="descriptor/16604-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16754-4">
    <skos:related rdf:resource="descriptor/13345-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19116-0">
    <skos:altLabel xml:lang="en">Lean enterprise</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19116-0">
    <skos:altLabel xml:lang="de">Schlankes Unternehmen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19116-0">
    <skos:altLabel xml:lang="de">Schlanke Produktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70572">
    <skos:prefLabel xml:lang="de">N.03.02  Wanderungen und Mobilität der Bevölkerung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#editorialNote">
    <rdfs:subPropertyOf rdf:resource="http://www.w3.org/2004/02/skos/core#note"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19624-2">
    <skos:related rdf:resource="descriptor/13179-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19624-2">
    <skos:related rdf:resource="descriptor/14188-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19624-2">
    <skos:related rdf:resource="descriptor/15901-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19624-2">
    <skos:related rdf:resource="descriptor/19632-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71006">
    <skos:prefLabel xml:lang="de">W.05.01.02  Erdöl- und Erdgasförderung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19472-6">
    <skos:related rdf:resource="descriptor/19471-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19472-6">
    <skos:related rdf:resource="descriptor/11207-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19472-6">
    <skos:related rdf:resource="descriptor/19462-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19472-6">
    <skos:related rdf:resource="descriptor/19468-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19007-5">
    <skos:altLabel xml:lang="de">Personalwerbung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19007-5">
    <skos:altLabel xml:lang="de">Personalmarktforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#scopeNote">
    <rdfs:subPropertyOf rdf:resource="http://www.w3.org/2004/02/skos/core#note"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10882-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19233-3">
    <skos:prefLabel xml:lang="en">Rural income inequality</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19233-3">
    <skos:prefLabel xml:lang="de">Ländliche Einkommensverteilung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70346">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17627-6">
    <skos:broader rdf:resource="thsys/70253"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17627-6">
    <skos:broader rdf:resource="descriptor/17608-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17627-6">
    <skos:broader rdf:resource="descriptor/17978-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12090-6">
    <skos:related rdf:resource="descriptor/11069-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12090-6">
    <skos:related rdf:resource="descriptor/12089-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12090-6">
    <skos:related rdf:resource="descriptor/12108-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12090-6">
    <skos:related rdf:resource="descriptor/18116-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12090-6">
    <skos:related rdf:resource="descriptor/11310-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12090-6">
    <skos:related rdf:resource="descriptor/12085-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12090-6">
    <skos:related rdf:resource="descriptor/16050-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12090-6">
    <skos:related rdf:resource="descriptor/19154-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12090-6">
    <skos:related rdf:resource="descriptor/16136-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12090-6">
    <skos:related rdf:resource="descriptor/11314-1"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id282">
    <rdf:value xml:lang="de">Für theoretische Untersuchungen zur Wirtschaftsregulierung</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11973-3">
    <skos:related rdf:resource="descriptor/13862-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11973-3">
    <skos:related rdf:resource="descriptor/11981-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12106-3">
    <skos:narrower rdf:resource="descriptor/24847-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12106-3">
    <skos:narrower rdf:resource="descriptor/12112-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12106-3">
    <skos:narrower rdf:resource="descriptor/12110-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16031-2">
    <skos:altLabel xml:lang="de">Genossenschaftsgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13221-5">
    <skos:related rdf:resource="descriptor/12920-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13221-5">
    <skos:related rdf:resource="descriptor/14218-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13221-5">
    <skos:related rdf:resource="descriptor/19182-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13221-5">
    <skos:related rdf:resource="descriptor/19181-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13221-5">
    <skos:related rdf:resource="descriptor/14197-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71060">
    <skos:broader rdf:resource="thsys/70051"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11067-6">
    <skos:prefLabel xml:lang="en">Economic democracy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11067-6">
    <skos:prefLabel xml:lang="de">Wirtschaftsdemokratie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11829-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18230-0">
    <skos:related rdf:resource="descriptor/14822-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18230-0">
    <skos:related rdf:resource="descriptor/18963-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13612-4">
    <skos:altLabel xml:lang="de">Verkehrsversorgung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13612-4">
    <skos:altLabel xml:lang="de">Transportleistung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13612-4">
    <skos:altLabel xml:lang="de">Fahrleistung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13612-4">
    <skos:altLabel xml:lang="de">Beförderungsleistung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19208-2">
    <skos:prefLabel xml:lang="en">Allocative efficiency</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19208-2">
    <skos:prefLabel xml:lang="de">Allokationseffizienz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10895-2">
    <skos:broader rdf:resource="thsys/70101"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10895-2">
    <skos:broader rdf:resource="thsys/70041"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13296-2">
    <skos:broader rdf:resource="thsys/70892"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19601-0">
    <skos:related rdf:resource="descriptor/19253-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19601-0">
    <skos:related rdf:resource="descriptor/18859-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19601-0">
    <skos:related rdf:resource="descriptor/15676-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19601-0">
    <skos:related rdf:resource="descriptor/15787-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19601-0">
    <skos:related rdf:resource="descriptor/15786-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19601-0">
    <skos:related rdf:resource="descriptor/15611-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19601-0">
    <skos:related rdf:resource="descriptor/16708-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19601-0">
    <skos:related rdf:resource="descriptor/15466-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19601-0">
    <skos:related rdf:resource="descriptor/19599-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19601-0">
    <skos:related rdf:resource="descriptor/19146-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19601-0">
    <skos:related rdf:resource="descriptor/18860-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16150-1">
    <skos:prefLabel xml:lang="en">Collective agreement</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16150-1">
    <skos:prefLabel xml:lang="de">Tarifvertrag</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70227">
    <skos:prefLabel xml:lang="en">V.12.02  Energy economics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70227">
    <skos:prefLabel xml:lang="de">V.12.02  Energieökonomik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70209">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70902">
    <rdfs:label xml:lang="de">Spezialbanken</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71118">
    <skos:broader rdf:resource="thsys/70180"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10439-0">
    <skos:altLabel xml:lang="de">Wirtschaftswachstumspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10439-0">
    <skos:altLabel xml:lang="de">Wachstumsförderungsprogramm</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15446-6">
    <skos:prefLabel xml:lang="en">Cooperative game</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15446-6">
    <skos:prefLabel xml:lang="de">Kooperatives Spiel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13693-6">
    <skos:related rdf:resource="descriptor/16080-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13693-6">
    <skos:related rdf:resource="descriptor/11417-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13693-6">
    <skos:related rdf:resource="descriptor/13731-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13693-6">
    <skos:related rdf:resource="descriptor/13756-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13693-6">
    <skos:related rdf:resource="descriptor/13773-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13693-6">
    <skos:related rdf:resource="descriptor/13690-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13693-6">
    <skos:related rdf:resource="descriptor/18707-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13693-6">
    <skos:related rdf:resource="descriptor/13687-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13693-6">
    <skos:related rdf:resource="descriptor/13283-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70278">
    <skos:broader rdf:resource="thsys/70052"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11043-6">
    <skos:related rdf:resource="descriptor/11101-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11043-6">
    <skos:related rdf:resource="descriptor/11059-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11043-6">
    <skos:related rdf:resource="descriptor/18822-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11043-6">
    <skos:related rdf:resource="descriptor/11722-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11043-6">
    <skos:related rdf:resource="descriptor/11626-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11043-6">
    <skos:related rdf:resource="descriptor/11825-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19660-5">
    <skos:related rdf:resource="descriptor/12737-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19660-5">
    <skos:related rdf:resource="descriptor/10455-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19660-5">
    <skos:related rdf:resource="descriptor/10454-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19660-5">
    <skos:related rdf:resource="descriptor/16757-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19660-5">
    <skos:related rdf:resource="descriptor/12711-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19660-5">
    <skos:related rdf:resource="descriptor/15606-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19660-5">
    <skos:related rdf:resource="descriptor/13671-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16363-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#narrowerTransitive">
    <rdf:type rdf:resource="http://www.w3.org/1999/02/22-rdf-syntax-ns#Property"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#narrowerTransitive">
    <rdf:type rdf:resource="http://www.w3.org/2002/07/owl#TransitiveProperty"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#narrowerTransitive">
    <rdf:type rdf:resource="http://www.w3.org/2002/07/owl#ObjectProperty"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18383-3">
    <skos:prefLabel xml:lang="en">Mining management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18383-3">
    <skos:prefLabel xml:lang="de">Bergwirtschaftslehre</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20341-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15776-6">
    <skos:related rdf:resource="descriptor/15763-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15776-6">
    <skos:related rdf:resource="descriptor/20967-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15776-6">
    <skos:related rdf:resource="descriptor/20888-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15776-6">
    <skos:related rdf:resource="descriptor/20762-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15776-6">
    <skos:related rdf:resource="descriptor/20744-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70888">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.17.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19500-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14078-0">
    <skos:prefLabel xml:lang="en">Rape (Plant)</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14078-0">
    <skos:prefLabel xml:lang="de">Raps</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70888">
    <skos:narrower rdf:resource="descriptor/13832-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70888">
    <skos:narrower rdf:resource="descriptor/13860-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70888">
    <skos:narrower rdf:resource="descriptor/13865-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70888">
    <skos:narrower rdf:resource="descriptor/13305-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70888">
    <skos:narrower rdf:resource="descriptor/13867-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70888">
    <skos:narrower rdf:resource="descriptor/13849-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70888">
    <skos:narrower rdf:resource="descriptor/13831-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70888">
    <skos:narrower rdf:resource="descriptor/11848-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70888">
    <skos:narrower rdf:resource="descriptor/11847-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70888">
    <skos:narrower rdf:resource="descriptor/13845-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70888">
    <skos:narrower rdf:resource="descriptor/13830-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70888">
    <skos:narrower rdf:resource="descriptor/11845-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70888">
    <skos:narrower rdf:resource="descriptor/13848-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70888">
    <skos:narrower rdf:resource="descriptor/18503-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70888">
    <skos:narrower rdf:resource="descriptor/18502-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70888">
    <skos:narrower rdf:resource="descriptor/12932-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70888">
    <skos:narrower rdf:resource="descriptor/13829-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70888">
    <skos:narrower rdf:resource="descriptor/13862-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70888">
    <skos:narrower rdf:resource="descriptor/13841-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70888">
    <skos:narrower rdf:resource="descriptor/18749-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70888">
    <skos:narrower rdf:resource="descriptor/11574-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70888">
    <skos:narrower rdf:resource="descriptor/18752-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70888">
    <skos:narrower rdf:resource="descriptor/13840-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70888">
    <skos:narrower rdf:resource="descriptor/18751-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70888">
    <skos:narrower rdf:resource="descriptor/19532-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70888">
    <skos:narrower rdf:resource="descriptor/18753-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70888">
    <skos:narrower rdf:resource="descriptor/11994-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70888">
    <skos:narrower rdf:resource="descriptor/13861-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70888">
    <skos:narrower rdf:resource="descriptor/11843-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70888">
    <skos:narrower rdf:resource="descriptor/18754-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70888">
    <skos:narrower rdf:resource="descriptor/11969-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70888">
    <skos:narrower rdf:resource="descriptor/16671-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70888">
    <skos:narrower rdf:resource="descriptor/13302-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70888">
    <skos:narrower rdf:resource="descriptor/11563-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70888">
    <skos:narrower rdf:resource="descriptor/12014-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19487-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19164-3">
    <skos:related rdf:resource="descriptor/20972-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18820-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18722-0">
    <skos:related rdf:resource="descriptor/12273-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18722-0">
    <skos:related rdf:resource="descriptor/18721-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18722-0">
    <skos:related rdf:resource="descriptor/20733-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18722-0">
    <skos:related rdf:resource="descriptor/13746-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18722-0">
    <skos:related rdf:resource="descriptor/18719-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17920-1">
    <skos:altLabel xml:lang="de">Republik Ecuador</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17920-1">
    <skos:altLabel xml:lang="de">Ekuador</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13371-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13704-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13144-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16990-6">
    <skos:related rdf:resource="descriptor/19083-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18589-4">
    <skos:related rdf:resource="descriptor/13530-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16780-3">
    <skos:related rdf:resource="descriptor/16792-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13644-5">
    <skos:prefLabel xml:lang="en">Tourism</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13644-5">
    <skos:prefLabel xml:lang="de">Tourismus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13298-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18106-1">
    <skos:prefLabel xml:lang="en">Success factor</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18106-1">
    <skos:prefLabel xml:lang="de">Erfolgsfaktor</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12168-2">
    <skos:altLabel xml:lang="en">Information effectiveness</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12168-2">
    <skos:altLabel xml:lang="de">Effizienz von Informationen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14074-1">
    <skos:prefLabel xml:lang="en">Agricultural crop</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14074-1">
    <skos:prefLabel xml:lang="de">Pflanzenbauprodukt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12936-4">
    <skos:broader rdf:resource="thsys/71024"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12936-4">
    <skos:broader rdf:resource="descriptor/18313-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19727-6">
    <skos:prefLabel xml:lang="en">EU environmental policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19727-6">
    <skos:prefLabel xml:lang="de">EU-Umweltpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12917-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13284-2">
    <skos:broader rdf:resource="thsys/70902"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13284-2">
    <skos:broader rdf:resource="descriptor/13291-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13066-5">
    <skos:broader rdf:resource="thsys/70999"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13066-5">
    <skos:broader rdf:resource="descriptor/13047-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15677-1">
    <skos:narrower rdf:resource="descriptor/18435-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15677-1">
    <skos:narrower rdf:resource="descriptor/15651-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15677-1">
    <skos:narrower rdf:resource="descriptor/18434-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15677-1">
    <skos:narrower rdf:resource="descriptor/18429-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15677-1">
    <skos:narrower rdf:resource="descriptor/15635-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18777-3">
    <skos:related rdf:resource="descriptor/19785-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18777-3">
    <skos:related rdf:resource="descriptor/15658-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18777-3">
    <skos:related rdf:resource="descriptor/18430-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18777-3">
    <skos:related rdf:resource="descriptor/14722-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18777-3">
    <skos:related rdf:resource="descriptor/18260-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19768-6">
    <skos:altLabel xml:lang="en">Fairness</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19768-6">
    <skos:altLabel xml:lang="en">Equity</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19768-6">
    <skos:altLabel xml:lang="de">Vertikale Gerechtigkeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19768-6">
    <skos:altLabel xml:lang="de">Horizontale Gerechtigkeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16645-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13837-1">
    <skos:broader rdf:resource="thsys/70884"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13837-1">
    <skos:broader rdf:resource="thsys/70036"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13837-1">
    <skos:broader rdf:resource="descriptor/10213-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13837-1">
    <skos:broader rdf:resource="descriptor/12285-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11346-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15674-0">
    <skos:related rdf:resource="descriptor/16543-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15674-0">
    <skos:related rdf:resource="descriptor/14014-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14911-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15142-5">
    <skos:altLabel xml:lang="en">Graphical analysis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15142-5">
    <skos:altLabel xml:lang="de">Statistische Darstellungsmethode</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15142-5">
    <skos:altLabel xml:lang="de">Statistische Darstellung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15142-5">
    <skos:altLabel xml:lang="de">Graphische Datenanalyse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15142-5">
    <skos:altLabel xml:lang="de">Explorative Datenanalyse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15142-5">
    <skos:altLabel xml:lang="de">Beschreibende Statistik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11352-0">
    <skos:prefLabel xml:lang="en">General education</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11352-0">
    <skos:prefLabel xml:lang="de">Allgemeinbildung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11083-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13290-0">
    <skos:broader rdf:resource="thsys/70903"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13290-0">
    <skos:broader rdf:resource="descriptor/13283-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70174">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">G.01.03</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12195-6">
    <skos:prefLabel xml:lang="en">Choice of organizational form</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12195-6">
    <skos:prefLabel xml:lang="de">Rechtsformwahl</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71069">
    <rdfs:label xml:lang="en">Regional policy</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71069">
    <rdfs:label xml:lang="de">Regionalpolitik</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19446-0">
    <skos:prefLabel xml:lang="en">Local economic policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19446-0">
    <skos:prefLabel xml:lang="de">Kommunale Wirtschaftspolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11703-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18262-1">
    <skos:prefLabel xml:lang="en">Toiletries industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18262-1">
    <skos:prefLabel xml:lang="de">Körperpflegemittelindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15931-6">
    <skos:narrower rdf:resource="descriptor/15932-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11434-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18665-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18903-3">
    <skos:altLabel xml:lang="de">Pflegesatz im Krankenhaus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18903-3">
    <skos:altLabel xml:lang="de">Krankenhausfinanzierungsstruktur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18903-3">
    <skos:altLabel xml:lang="de">Krankenhausetat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19438-6">
    <skos:altLabel xml:lang="de">Regionale Verteilung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18264-4">
    <skos:prefLabel xml:lang="en">Deodorant</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18264-4">
    <skos:prefLabel xml:lang="de">Deodorant</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16639-4">
    <skos:prefLabel xml:lang="en">Standard of living</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16639-4">
    <skos:prefLabel xml:lang="de">Lebensstandard</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18962-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18209-5">
    <skos:broader rdf:resource="thsys/70265"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18209-5">
    <skos:broader rdf:resource="descriptor/13669-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11073-4">
    <skos:broader rdf:resource="thsys/71101"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11073-4">
    <skos:broader rdf:resource="thsys/70052"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11073-4">
    <skos:broader rdf:resource="descriptor/11090-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11073-4">
    <skos:broader rdf:resource="descriptor/11088-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11859-1">
    <skos:related rdf:resource="descriptor/11869-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11859-1">
    <skos:related rdf:resource="descriptor/19065-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11859-1">
    <skos:related rdf:resource="descriptor/18327-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11859-1">
    <skos:related rdf:resource="descriptor/11921-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11859-1">
    <skos:related rdf:resource="descriptor/18328-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11859-1">
    <skos:related rdf:resource="descriptor/19454-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11859-1">
    <skos:related rdf:resource="descriptor/12015-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11859-1">
    <skos:related rdf:resource="descriptor/12014-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14581-5">
    <skos:altLabel xml:lang="de">Stofftier</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14581-5">
    <skos:altLabel xml:lang="de">Spielzeugauto</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14581-5">
    <skos:altLabel xml:lang="de">Spielzeug</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14581-5">
    <skos:altLabel xml:lang="de">Spielkarten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14581-5">
    <skos:altLabel xml:lang="de">Schachspiel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14581-5">
    <skos:altLabel xml:lang="de">Schachcomputer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14581-5">
    <skos:altLabel xml:lang="de">Puppe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14581-5">
    <skos:altLabel xml:lang="de">Modelleisenbahn</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14581-5">
    <skos:altLabel xml:lang="de">Gesellschaftsspiel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14581-5">
    <skos:altLabel xml:lang="de">Elektronische Spielwaren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12643-5">
    <skos:prefLabel xml:lang="en">Just in time</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12643-5">
    <skos:prefLabel xml:lang="de">Just in Time</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10824-4">
    <skos:related rdf:resource="descriptor/10567-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10824-4">
    <skos:related rdf:resource="descriptor/10816-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10824-4">
    <skos:related rdf:resource="descriptor/10293-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10824-4">
    <skos:related rdf:resource="descriptor/10823-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10824-4">
    <skos:related rdf:resource="descriptor/19315-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17811-6">
    <skos:prefLabel xml:lang="en">Canada</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17811-6">
    <skos:prefLabel xml:lang="de">Kanada</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16031-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13743-3">
    <skos:narrower rdf:resource="descriptor/19554-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18656-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16579-3">
    <skos:prefLabel xml:lang="en">Theory of society</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16579-3">
    <skos:prefLabel xml:lang="de">Gesellschaftstheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17806-6">
    <skos:prefLabel xml:lang="en">North America</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17806-6">
    <skos:prefLabel xml:lang="de">Nordamerika</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10110-1">
    <skos:related rdf:resource="descriptor/21082-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10110-1">
    <skos:related rdf:resource="descriptor/11765-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10110-1">
    <skos:related rdf:resource="descriptor/11779-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10110-1">
    <skos:related rdf:resource="descriptor/11752-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10110-1">
    <skos:related rdf:resource="descriptor/11751-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10110-1">
    <skos:related rdf:resource="descriptor/11691-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10110-1">
    <skos:related rdf:resource="descriptor/10093-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10110-1">
    <skos:related rdf:resource="descriptor/10099-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10110-1">
    <skos:related rdf:resource="descriptor/11770-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19532-0">
    <skos:scopeNote xml:lang="de">Wahl zwischen Wohneigentum oder Mietverhältnis.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18564-6">
    <skos:prefLabel xml:lang="en">Conference sector</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18564-6">
    <skos:prefLabel xml:lang="de">Tagungswirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19371-5">
    <skos:related rdf:resource="descriptor/18149-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14272-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13823-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12160-4">
    <skos:related rdf:resource="descriptor/12511-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12160-4">
    <skos:related rdf:resource="descriptor/13993-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12160-4">
    <skos:related rdf:resource="descriptor/13413-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12160-4">
    <skos:related rdf:resource="descriptor/12042-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10142-2">
    <skos:altLabel xml:lang="de">Informationszugang</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10142-2">
    <skos:altLabel xml:lang="de">Informationsverteilung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10142-2">
    <skos:altLabel xml:lang="de">Informationstransfer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10142-2">
    <skos:altLabel xml:lang="de">Informationsfluss</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10142-2">
    <skos:altLabel xml:lang="de">Informationsfluß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10142-2">
    <skos:altLabel xml:lang="de">Informationsdiffusion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10142-2">
    <skos:altLabel xml:lang="de">Diffusionsprozess der Information</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10142-2">
    <skos:altLabel xml:lang="de">Diffusionsprozeß der Information</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10734-5">
    <skos:prefLabel xml:lang="en">European Monetary System</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10734-5">
    <skos:prefLabel xml:lang="de">Europäischer Währungsverbund</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#narrowMatch">
    <owl:inverseOf rdf:resource="http://www.w3.org/2004/02/skos/core#broadMatch"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10629-2">
    <skos:broader rdf:resource="thsys/71100"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10629-2">
    <skos:broader rdf:resource="descriptor/10062-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11149-4">
    <skos:prefLabel xml:lang="en">Collective farm</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11149-4">
    <skos:prefLabel xml:lang="de">Sozialistischer Landwirtschaftsbetrieb</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13006-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13306-4">
    <skos:narrower rdf:resource="descriptor/18565-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13306-4">
    <skos:narrower rdf:resource="descriptor/18006-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13306-4">
    <skos:narrower rdf:resource="descriptor/13309-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13306-4">
    <skos:narrower rdf:resource="descriptor/18560-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24847-1">
    <skos:prefLabel xml:lang="en">Organizational behaviour</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24847-1">
    <skos:prefLabel xml:lang="de">Organisationsverhalten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19561-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13620-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19201-2">
    <skos:altLabel xml:lang="de">Zufallsgewinn</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19201-2">
    <skos:altLabel xml:lang="de">Quasimonopolgewinn</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19201-2">
    <skos:altLabel xml:lang="de">Q-Gewinn</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19201-2">
    <skos:altLabel xml:lang="de">Marktlagengewinn</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12565-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10169-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15983-1">
    <skos:altLabel xml:lang="de">Wahlordnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15983-1">
    <skos:altLabel xml:lang="de">Passives Wahlrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15983-1">
    <skos:altLabel xml:lang="de">Bundeswahlordnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15983-1">
    <skos:altLabel xml:lang="de">Bundeswahlgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15983-1">
    <skos:altLabel xml:lang="de">Aktives Wahlrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12942-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17704-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14032-3">
    <skos:prefLabel xml:lang="en">Household goods</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14032-3">
    <skos:prefLabel xml:lang="de">Haushaltswaren</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13032-1">
    <skos:altLabel xml:lang="de">Hüttenkokerei</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13032-1">
    <skos:altLabel xml:lang="de">Hochofen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71043">
    <skos:prefLabel xml:lang="de">B.03.01.01  Buchführung und Bilanzierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73367">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19344-1">
    <skos:broader rdf:resource="thsys/71025"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19344-1">
    <skos:broader rdf:resource="thsys/70960"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19344-1">
    <skos:broader rdf:resource="descriptor/19674-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11543-0">
    <skos:related rdf:resource="descriptor/10978-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11543-0">
    <skos:related rdf:resource="descriptor/10213-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11543-0">
    <skos:related rdf:resource="descriptor/11544-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11543-0">
    <skos:related rdf:resource="descriptor/10889-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11543-0">
    <skos:related rdf:resource="descriptor/15730-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11543-0">
    <skos:related rdf:resource="descriptor/18957-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15883-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70175">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.00</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12171-6">
    <skos:narrower rdf:resource="descriptor/19008-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12171-6">
    <skos:narrower rdf:resource="descriptor/18207-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15328-5">
    <skos:broader rdf:resource="thsys/72186"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15328-5">
    <skos:broader rdf:resource="descriptor/15330-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70031">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14251-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11937-0">
    <skos:altLabel xml:lang="de">Europäische Regionalpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11937-0">
    <skos:altLabel xml:lang="de">EG-Regionalpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19327-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19582-6">
    <skos:related rdf:resource="descriptor/15117-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19582-6">
    <skos:related rdf:resource="descriptor/19576-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14382-4">
    <skos:broader rdf:resource="thsys/70318"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14382-4">
    <skos:broader rdf:resource="descriptor/14393-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19666-0">
    <skos:related rdf:resource="descriptor/10112-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19666-0">
    <skos:related rdf:resource="descriptor/10991-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19666-0">
    <skos:related rdf:resource="descriptor/10025-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70418">
    <skos:broader rdf:resource="thsys/p"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11673-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11349-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19743-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15444-3">
    <skos:broader rdf:resource="thsys/70582"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15904-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19108-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14633-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71002">
    <skos:broader rdf:resource="thsys/71004"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13235-1">
    <skos:broader rdf:resource="thsys/70959"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13235-1">
    <skos:broader rdf:resource="descriptor/18522-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14189-5">
    <skos:altLabel xml:lang="en">Hydroelectric power</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20536-3">
    <skos:broader rdf:resource="thsys/73384"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20536-3">
    <skos:broader rdf:resource="thsys/70041"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12312-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18954-0">
    <skos:prefLabel xml:lang="en">Sports marketing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18954-0">
    <skos:prefLabel xml:lang="de">Sportmarketing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13712-0">
    <skos:altLabel xml:lang="en">Stock market index</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13712-0">
    <skos:altLabel xml:lang="en">NEMAX-All-Share-Index</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13712-0">
    <skos:altLabel xml:lang="de">SMAX</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13712-0">
    <skos:altLabel xml:lang="de">SDAX</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13712-0">
    <skos:altLabel xml:lang="de">NEMAX 50</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13712-0">
    <skos:altLabel xml:lang="de">NEMAX</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13712-0">
    <skos:altLabel xml:lang="de">MDAX</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13712-0">
    <skos:altLabel xml:lang="de">Dow-Jones-Index</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13712-0">
    <skos:altLabel xml:lang="de">DAX 100</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13712-0">
    <skos:altLabel xml:lang="de">DAX</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13712-0">
    <skos:altLabel xml:lang="de">CDAX</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18013-1">
    <skos:altLabel xml:lang="en">Tertiary sector</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18013-1">
    <skos:altLabel xml:lang="de">Tertiärer Sektor</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18013-1">
    <skos:altLabel xml:lang="de">Dienstleistungsgewerbe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18013-1">
    <skos:altLabel xml:lang="de">Dienstleistungsbranche</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10489-6">
    <skos:related rdf:resource="descriptor/15943-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10489-6">
    <skos:related rdf:resource="descriptor/15939-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10489-6">
    <skos:related rdf:resource="descriptor/19645-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10489-6">
    <skos:related rdf:resource="descriptor/16749-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13274-5">
    <skos:narrower rdf:resource="descriptor/13273-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13274-5">
    <skos:narrower rdf:resource="descriptor/13272-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17792-5">
    <skos:altLabel xml:lang="de">Republik Nicaragua</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17792-5">
    <skos:altLabel xml:lang="de">Nikaragua</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11083-1">
    <skos:scopeNote xml:lang="de">Politischer, wirtschaftlicher und kultureller Imperialismus.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19605-6">
    <skos:broader rdf:resource="thsys/72183"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19605-6">
    <skos:broader rdf:resource="descriptor/15142-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10022-5">
    <skos:related rdf:resource="descriptor/10019-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10022-5">
    <skos:related rdf:resource="descriptor/18178-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10022-5">
    <skos:related rdf:resource="descriptor/12251-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10022-5">
    <skos:related rdf:resource="descriptor/11607-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10022-5">
    <skos:related rdf:resource="descriptor/19358-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10135-6">
    <skos:prefLabel xml:lang="en">Interest rate</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10135-6">
    <skos:prefLabel xml:lang="de">Zins</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10786-0">
    <skos:altLabel xml:lang="de">Kapitalverkehrsbilanz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19418-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16522-4">
    <skos:broader rdf:resource="thsys/73363"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16522-4">
    <skos:broader rdf:resource="thsys/70865"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12253-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11890-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24786-2">
    <skos:prefLabel xml:lang="en">Life annuity</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24786-2">
    <skos:prefLabel xml:lang="de">Leibrente</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18649-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10380-0">
    <skos:related rdf:resource="descriptor/10978-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10380-0">
    <skos:related rdf:resource="descriptor/10401-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10380-0">
    <skos:related rdf:resource="descriptor/11345-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10380-0">
    <skos:related rdf:resource="descriptor/10387-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10380-0">
    <skos:related rdf:resource="descriptor/11770-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19611-4">
    <skos:related rdf:resource="descriptor/15902-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19611-4">
    <skos:related rdf:resource="descriptor/16597-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19611-4">
    <skos:related rdf:resource="descriptor/11869-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19611-4">
    <skos:related rdf:resource="descriptor/16215-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19611-4">
    <skos:related rdf:resource="descriptor/16680-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19611-4">
    <skos:related rdf:resource="descriptor/18924-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19611-4">
    <skos:related rdf:resource="descriptor/15694-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19611-4">
    <skos:related rdf:resource="descriptor/15654-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19611-4">
    <skos:related rdf:resource="descriptor/16556-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15410-6">
    <skos:altLabel xml:lang="en">Optical character recognition</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15410-6">
    <skos:altLabel xml:lang="en">OCR (Optical Character Recognition)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15410-6">
    <skos:altLabel xml:lang="en">Ident-system</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15410-6">
    <skos:altLabel xml:lang="de">Identifikationstechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15410-6">
    <skos:altLabel xml:lang="de">Identifikationssystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17951-4">
    <skos:altLabel xml:lang="de">Republik Vanuatu</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17951-4">
    <skos:altLabel xml:lang="de">Neue Hebriden</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15470-2">
    <skos:altLabel xml:lang="en">Decision model</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15470-2">
    <skos:altLabel xml:lang="de">Präskriptive Entscheidungstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15470-2">
    <skos:altLabel xml:lang="de">Normative Entscheidungstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15470-2">
    <skos:altLabel xml:lang="de">Entscheidungsmodell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15470-2">
    <skos:altLabel xml:lang="de">Entscheidungslogik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15470-2">
    <skos:altLabel xml:lang="de">Entscheidungslehre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15470-2">
    <skos:altLabel xml:lang="de">Deskriptive Entscheidungstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17703-2">
    <skos:altLabel xml:lang="de">Republik Gambia</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13381-4">
    <skos:broader rdf:resource="thsys/70876"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14958-5">
    <skos:narrower rdf:resource="descriptor/14957-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14958-5">
    <skos:narrower rdf:resource="descriptor/14960-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18670-0">
    <skos:narrower rdf:resource="descriptor/19277-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18670-0">
    <skos:narrower rdf:resource="descriptor/18685-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11618-2">
    <skos:altLabel xml:lang="en">Treasury note</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11618-2">
    <skos:altLabel xml:lang="en">Treasury bond</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11618-2">
    <skos:altLabel xml:lang="en">Treasury bill</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11618-2">
    <skos:altLabel xml:lang="de">U-Schätze</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11618-2">
    <skos:altLabel xml:lang="de">Unverzinsliche Schatzanweisung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11618-2">
    <skos:altLabel xml:lang="de">Schatzwechsel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11618-2">
    <skos:altLabel xml:lang="de">Schatzobligation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11618-2">
    <skos:altLabel xml:lang="de">Schatzbrief</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11618-2">
    <skos:altLabel xml:lang="de">Schatzanweisung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11618-2">
    <skos:altLabel xml:lang="de">Bundesschatzbrief</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15763-1">
    <skos:related rdf:resource="descriptor/19670-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15763-1">
    <skos:related rdf:resource="descriptor/15074-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15763-1">
    <skos:related rdf:resource="descriptor/15770-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15763-1">
    <skos:related rdf:resource="descriptor/18071-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15763-1">
    <skos:related rdf:resource="descriptor/15768-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15763-1">
    <skos:related rdf:resource="descriptor/18186-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15763-1">
    <skos:related rdf:resource="descriptor/18740-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15763-1">
    <skos:related rdf:resource="descriptor/19016-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15763-1">
    <skos:related rdf:resource="descriptor/18030-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15763-1">
    <skos:related rdf:resource="descriptor/10058-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15763-1">
    <skos:related rdf:resource="descriptor/14626-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15763-1">
    <skos:related rdf:resource="descriptor/18266-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15763-1">
    <skos:related rdf:resource="descriptor/18444-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15763-1">
    <skos:related rdf:resource="descriptor/18613-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15763-1">
    <skos:related rdf:resource="descriptor/15776-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15763-1">
    <skos:related rdf:resource="descriptor/13325-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10387-0">
    <skos:altLabel xml:lang="de">Verstetigungspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10387-0">
    <skos:altLabel xml:lang="de">Stabilitätspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10387-0">
    <skos:altLabel xml:lang="de">Stabilisierungspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10387-0">
    <skos:altLabel xml:lang="de">Konjunktursteuerung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10387-0">
    <skos:altLabel xml:lang="de">Konjunkturstabilisierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10387-0">
    <skos:altLabel xml:lang="de">Konjunkturprogramm</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10387-0">
    <skos:altLabel xml:lang="de">Gesamtwirtschaftliche Stabilisierungspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14379-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20321-3">
    <skos:prefLabel xml:lang="en">New economic geography</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20321-3">
    <skos:prefLabel xml:lang="de">Neue ökonomische Geographie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11660-3">
    <skos:prefLabel xml:lang="en">Family taxation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11660-3">
    <skos:prefLabel xml:lang="de">Familienbesteuerung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18573-5">
    <skos:narrower rdf:resource="descriptor/18570-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19256-5">
    <skos:altLabel xml:lang="de">Buffer Stock Ansatz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19319-0">
    <skos:broader rdf:resource="thsys/71098"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19319-0">
    <skos:broader rdf:resource="descriptor/10686-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19617-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19725-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10517-6">
    <skos:prefLabel xml:lang="en">Balanced growth</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10517-6">
    <skos:prefLabel xml:lang="de">Gleichgewichtiges Wachstum</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10305-0">
    <skos:broader rdf:resource="thsys/70233"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10305-0">
    <skos:broader rdf:resource="descriptor/10327-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18288-4">
    <skos:prefLabel xml:lang="en">Genetic engineering law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18288-4">
    <skos:prefLabel xml:lang="de">Gentechnikrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16477-4">
    <skos:altLabel xml:lang="de">Familienfürsorge des Staates</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70929">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11686-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11723-5">
    <skos:prefLabel xml:lang="en">Free-rider problem</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11723-5">
    <skos:prefLabel xml:lang="de">Trittbrettfahrerverhalten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16717-3">
    <skos:altLabel xml:lang="de">Materialistische Dialektik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16717-3">
    <skos:altLabel xml:lang="de">Historischer Materialismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16717-3">
    <skos:altLabel xml:lang="de">Diamat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16717-3">
    <skos:altLabel xml:lang="de">Dialektischer Materialismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10821-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19538-2">
    <skos:related rdf:resource="descriptor/13742-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19538-2">
    <skos:related rdf:resource="descriptor/13724-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71094">
    <skos:narrower rdf:resource="descriptor/19335-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71094">
    <skos:narrower rdf:resource="descriptor/10821-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71094">
    <skos:narrower rdf:resource="descriptor/10773-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71094">
    <skos:narrower rdf:resource="descriptor/10771-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71094">
    <skos:narrower rdf:resource="descriptor/18127-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71094">
    <skos:narrower rdf:resource="descriptor/10770-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71094">
    <skos:narrower rdf:resource="descriptor/10769-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71094">
    <skos:narrower rdf:resource="descriptor/19337-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71094">
    <skos:narrower rdf:resource="descriptor/10768-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71094">
    <skos:narrower rdf:resource="descriptor/10764-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71094">
    <skos:narrower rdf:resource="descriptor/10761-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71094">
    <skos:narrower rdf:resource="descriptor/10758-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71094">
    <skos:narrower rdf:resource="descriptor/10754-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71094">
    <skos:narrower rdf:resource="descriptor/19334-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71094">
    <skos:narrower rdf:resource="descriptor/10751-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71094">
    <skos:narrower rdf:resource="descriptor/10750-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71094">
    <skos:narrower rdf:resource="descriptor/10746-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71094">
    <skos:narrower rdf:resource="descriptor/10745-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71094">
    <skos:narrower rdf:resource="descriptor/10736-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71094">
    <skos:narrower rdf:resource="descriptor/10735-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71094">
    <skos:narrower rdf:resource="descriptor/10734-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71094">
    <skos:narrower rdf:resource="descriptor/10733-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71094">
    <skos:narrower rdf:resource="descriptor/10728-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71094">
    <skos:narrower rdf:resource="descriptor/18695-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11090-4">
    <skos:narrower rdf:resource="descriptor/11141-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11090-4">
    <skos:narrower rdf:resource="descriptor/11073-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18856-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13146-0">
    <skos:altLabel xml:lang="de">Textilfärberei</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11327-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10195-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13935-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18347-0">
    <skos:scopeNote xml:lang="de">Tabellarische Darstellung des Aufkommens und der Verwendung aller Energieträger in einem Wirtschaftsraum pro Periode nach festgelegtem Schema.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12474-2">
    <skos:broader rdf:resource="thsys/70578"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18921-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10593-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15468-3">
    <skos:prefLabel xml:lang="en">Decision under uncertainty</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15468-3">
    <skos:prefLabel xml:lang="de">Entscheidung bei Unsicherheit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11602-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70883">
    <rdfs:label xml:lang="en">Leasing</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70883">
    <rdfs:label xml:lang="de">Leasing</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12018-0">
    <skos:broader rdf:resource="thsys/73368"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12018-0">
    <skos:broader rdf:resource="thsys/71066"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12018-0">
    <skos:broader rdf:resource="thsys/70889"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12018-0">
    <skos:broader rdf:resource="descriptor/11869-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12018-0">
    <skos:broader rdf:resource="descriptor/16273-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12274-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19778-3">
    <skos:altLabel xml:lang="de">Sprachnormung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19778-3">
    <skos:altLabel xml:lang="de">Fachsprache</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13176-5">
    <skos:altLabel xml:lang="de">Straßenunterhalt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13176-5">
    <skos:altLabel xml:lang="de">Straßeninstandsetzung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13176-5">
    <skos:altLabel xml:lang="de">Straßenbautechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13617-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13144-4">
    <skos:broader rdf:resource="thsys/70987"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13144-4">
    <skos:broader rdf:resource="descriptor/13131-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10615-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18082-3">
    <zbwext:useInsteadNote rdf:nodeID="id147"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10227-1">
    <skos:related rdf:resource="descriptor/10102-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10227-1">
    <skos:related rdf:resource="descriptor/10070-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18229-6">
    <skos:altLabel xml:lang="de">Computerschrott</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19670-2">
    <skos:related rdf:resource="descriptor/15779-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19670-2">
    <skos:related rdf:resource="descriptor/12403-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19670-2">
    <skos:related rdf:resource="descriptor/15763-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19670-2">
    <skos:related rdf:resource="descriptor/10058-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19670-2">
    <skos:related rdf:resource="descriptor/19194-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18540-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16210-2">
    <skos:scopeNote xml:lang="de">Umfasst die gesamte sozialistische Ideengeschichte marxistischer und nicht-marxistischer Provenienz.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18500-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11070-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16737-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18143-2">
    <skos:related rdf:resource="descriptor/18115-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18143-2">
    <skos:related rdf:resource="descriptor/12588-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18143-2">
    <skos:related rdf:resource="descriptor/11302-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18143-2">
    <skos:related rdf:resource="descriptor/18872-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18143-2">
    <skos:related rdf:resource="descriptor/19656-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16055-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14245-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19082-5">
    <skos:related rdf:resource="descriptor/12863-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14432-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18457-0">
    <skos:broader rdf:resource="thsys/71005"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18457-0">
    <skos:broader rdf:resource="descriptor/10337-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13983-4">
    <skos:altLabel xml:lang="de">Seemänner</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13983-4">
    <skos:altLabel xml:lang="de">Seeleute</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13983-4">
    <skos:altLabel xml:lang="de">Seefahrtsberufe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13983-4">
    <skos:altLabel xml:lang="de">Schiffsingenieurinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13983-4">
    <skos:altLabel xml:lang="de">Schiffsingenieure</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13983-4">
    <skos:altLabel xml:lang="de">Schiffsführerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13983-4">
    <skos:altLabel xml:lang="de">Schiffsführer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13983-4">
    <skos:altLabel xml:lang="de">Schiffer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13983-4">
    <skos:altLabel xml:lang="de">Schiffahrtsberufe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13983-4">
    <skos:altLabel xml:lang="de">Nautikerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13983-4">
    <skos:altLabel xml:lang="de">Nautiker</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13983-4">
    <skos:altLabel xml:lang="de">Matrosen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13983-4">
    <skos:altLabel xml:lang="de">Maritime Berufe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13983-4">
    <skos:altLabel xml:lang="de">Lotsen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13983-4">
    <skos:altLabel xml:lang="de">Kapitäninnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13983-4">
    <skos:altLabel xml:lang="de">Kapitäne</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13983-4">
    <skos:altLabel xml:lang="de">Hafenmeisterinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13983-4">
    <skos:altLabel xml:lang="de">Hafenmeister</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13983-4">
    <skos:altLabel xml:lang="de">Bootsleute</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13983-4">
    <skos:altLabel xml:lang="de">Binnenschifferinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13983-4">
    <skos:altLabel xml:lang="de">Binnenschiffer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16524-0">
    <skos:narrower rdf:resource="descriptor/18820-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12497-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17569-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id334">
    <rdf:value xml:lang="de">Für Tochtergesellschaften im Ausland</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19758-2">
    <skos:prefLabel xml:lang="en">Prestige</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19758-2">
    <skos:prefLabel xml:lang="de">Prestige</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19397-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14702-5">
    <skos:altLabel xml:lang="de">Plastik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12246-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19674-1">
    <skos:narrower rdf:resource="descriptor/19346-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19674-1">
    <skos:narrower rdf:resource="descriptor/19344-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19674-1">
    <skos:narrower rdf:resource="descriptor/19677-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19674-1">
    <skos:narrower rdf:resource="descriptor/19293-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14926-4">
    <skos:broader rdf:resource="thsys/70498"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14926-4">
    <skos:broader rdf:resource="descriptor/14931-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10032-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11203-3">
    <skos:altLabel xml:lang="de">Zeitarbeit als Leiharbeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11203-3">
    <skos:altLabel xml:lang="de">Personalverleih</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11203-3">
    <skos:altLabel xml:lang="de">Personalleasing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11203-3">
    <skos:altLabel xml:lang="de">Personal-Leasing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11203-3">
    <skos:altLabel xml:lang="de">Mittelbare Beschäftigung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11203-3">
    <skos:altLabel xml:lang="de">Mitarbeiter-Leasing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11203-3">
    <skos:altLabel xml:lang="de">Leiharbeitsrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11203-3">
    <skos:altLabel xml:lang="de">Gewerbliche Arbeitnehmerüberlassung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11203-3">
    <skos:altLabel xml:lang="de">Gesetz zur Regelung der gewerbsmäßigen Arbeitnehmerüberlassung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11203-3">
    <skos:altLabel xml:lang="de">Fremdfirmenarbeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11203-3">
    <skos:altLabel xml:lang="de">AÜG (Arbeitnehmerüberlassungsgesetz)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11203-3">
    <skos:altLabel xml:lang="de">Arbeitskräfteüberlassung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11203-3">
    <skos:altLabel xml:lang="de">Arbeitnehmerüberlassungsrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11203-3">
    <skos:altLabel xml:lang="de">Arbeitnehmerüberlassung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17749-6">
    <skos:altLabel xml:lang="de">Königreich Marokko</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71028">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18243-5">
    <skos:broader rdf:resource="thsys/70346"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18243-5">
    <skos:broader rdf:resource="descriptor/12649-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18243-5">
    <skos:broader rdf:resource="descriptor/14269-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16517-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16654-1">
    <skos:prefLabel xml:lang="en">Social isolation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16654-1">
    <skos:prefLabel xml:lang="de">Soziale Isolation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70182">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19549-4">
    <skos:altLabel xml:lang="en">Motor vehicle market</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19549-4">
    <skos:altLabel xml:lang="de">Kfz-Markt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19394-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10859-6">
    <skos:broader rdf:resource="thsys/70951"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10859-6">
    <skos:broader rdf:resource="descriptor/13459-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16513-5">
    <skos:broader rdf:resource="thsys/73373"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16513-5">
    <skos:broader rdf:resource="descriptor/16509-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15082-4">
    <skos:broader rdf:resource="thsys/72184"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15082-4">
    <skos:broader rdf:resource="descriptor/15149-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14999-5">
    <skos:prefLabel xml:lang="en">Cigar</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14999-5">
    <skos:prefLabel xml:lang="de">Zigarre</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10604-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14285-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13128-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12283-2">
    <skos:narrower rdf:resource="descriptor/18298-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13633-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18014-6">
    <skos:related rdf:resource="descriptor/12032-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16668-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14514-6">
    <skos:prefLabel xml:lang="en">Optical memory</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14514-6">
    <skos:prefLabel xml:lang="de">Optischer Speicher</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15797-5">
    <skos:related rdf:resource="descriptor/16300-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15797-5">
    <skos:related rdf:resource="descriptor/16416-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15797-5">
    <skos:related rdf:resource="descriptor/16264-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15797-5">
    <skos:related rdf:resource="descriptor/18062-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15797-5">
    <skos:related rdf:resource="descriptor/16299-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15797-5">
    <skos:related rdf:resource="descriptor/16215-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10652-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18097-4">
    <skos:prefLabel xml:lang="en">Donation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18097-4">
    <skos:prefLabel xml:lang="de">Spende</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15372-2">
    <skos:narrower rdf:resource="descriptor/15543-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10555-5">
    <zbwext:useInsteadNote rdf:nodeID="id86"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18795-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11564-6">
    <skos:scopeNote xml:lang="de">Alle einkommenswirksamen kinderbezogenen Maßnahmen der Familienpolitik.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12882-1">
    <skos:broader rdf:resource="thsys/71041"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17181-6">
    <skos:altLabel xml:lang="x-other">Magyarorszag</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17181-6">
    <skos:altLabel xml:lang="de">Ungarische Volksrepublik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17181-6">
    <skos:altLabel xml:lang="de">Republik Ungarn</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17689-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10138-0">
    <skos:related rdf:resource="descriptor/15458-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10138-0">
    <skos:related rdf:resource="descriptor/10076-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10138-0">
    <skos:related rdf:resource="descriptor/15470-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10138-0">
    <skos:related rdf:resource="descriptor/13322-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11163-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12655-5">
    <skos:broader rdf:resource="thsys/70368"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#exactMatch">
    <rdfs:isDefinedBy rdf:resource="http://www.w3.org/2004/02/skos/core"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18863-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19331-3">
    <skos:prefLabel xml:lang="en">Tariff harmonization</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19331-3">
    <skos:prefLabel xml:lang="de">Zollharmonisierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11309-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19460-6">
    <skos:prefLabel xml:lang="en">Rural economy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19460-6">
    <skos:prefLabel xml:lang="de">Ländliche Wirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12775-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12425-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14267-4">
    <skos:altLabel xml:lang="de">Hartmetall</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16635-5">
    <skos:prefLabel xml:lang="en">Health</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16635-5">
    <skos:prefLabel xml:lang="de">Gesundheit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17919-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15716-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70903">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.14.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19816-0">
    <skos:prefLabel xml:lang="en">Criminal procedure</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19816-0">
    <skos:prefLabel xml:lang="de">Strafverfahren</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14039-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18864-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13513-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13075-4">
    <skos:prefLabel xml:lang="en">Electric household appliances industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13075-4">
    <skos:prefLabel xml:lang="de">Elektrohaushaltsgeräteindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11124-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20789-2">
    <skos:prefLabel xml:lang="en">Spare parts</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20789-2">
    <skos:prefLabel xml:lang="de">Ersatzteil</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14463-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12196-4">
    <skos:broader rdf:resource="thsys/73391"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12196-4">
    <skos:broader rdf:resource="thsys/71040"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12196-4">
    <skos:broader rdf:resource="thsys/70087"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70930">
    <rdfs:label xml:lang="en">Combined transport</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70930">
    <rdfs:label xml:lang="de">Kombinierter Verkehr</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19044-6">
    <zbwext:useInsteadNote rdf:nodeID="id347"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24784-6">
    <skos:related rdf:resource="descriptor/16595-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24784-6">
    <skos:related rdf:resource="descriptor/18761-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20906-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11870-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12774-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19457-2">
    <skos:prefLabel xml:lang="en">Small town</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19457-2">
    <skos:prefLabel xml:lang="de">Kleinstadt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18493-3">
    <skos:broader rdf:resource="thsys/73351"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18493-3">
    <skos:broader rdf:resource="thsys/70970"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18493-3">
    <skos:broader rdf:resource="descriptor/15765-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11485-2">
    <skos:related rdf:resource="descriptor/11464-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11485-2">
    <skos:related rdf:resource="descriptor/19262-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16782-6">
    <skos:prefLabel xml:lang="en">Documentation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16782-6">
    <skos:prefLabel xml:lang="de">Dokumentation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19599-3">
    <skos:narrower rdf:resource="descriptor/19771-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12320-1">
    <skos:altLabel xml:lang="en">Value chain</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12320-1">
    <skos:altLabel xml:lang="en">Value added statement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12320-1">
    <skos:altLabel xml:lang="en">EVA (Economic value added)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12320-1">
    <skos:altLabel xml:lang="en">Economic value added</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12320-1">
    <skos:altLabel xml:lang="de">Wertschöpfungsstrategie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12320-1">
    <skos:altLabel xml:lang="de">Wertschöpfungsrechnung des Betriebs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12320-1">
    <skos:altLabel xml:lang="de">Wertschöpfungskette</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12320-1">
    <skos:altLabel xml:lang="de">Wertkettenanalyse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12320-1">
    <skos:altLabel xml:lang="de">Wertkette</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12320-1">
    <skos:altLabel xml:lang="de">Wertekette</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12320-1">
    <skos:altLabel xml:lang="de">Leistungskette</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12320-1">
    <skos:altLabel xml:lang="de">Betriebliche Wertschöpfungsrechnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19377-0">
    <skos:related rdf:resource="descriptor/11990-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19377-0">
    <skos:related rdf:resource="descriptor/11177-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16120-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10628-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17688-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12504-5">
    <skos:altLabel xml:lang="en">Cost of production</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12504-5">
    <skos:altLabel xml:lang="de">Herstellkosten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12504-5">
    <skos:altLabel xml:lang="de">Fertigungskosten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71059">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11679-3">
    <skos:broader rdf:resource="thsys/71075"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11679-3">
    <skos:broader rdf:resource="thsys/70046"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11679-3">
    <skos:broader rdf:resource="descriptor/11704-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10492-3">
    <skos:prefLabel xml:lang="en">Developing countries</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10492-3">
    <skos:prefLabel xml:lang="de">Entwicklungsländer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71084">
    <skos:broader rdf:resource="thsys/70091"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15868-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16435-6">
    <skos:narrower rdf:resource="descriptor/10619-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16435-6">
    <skos:narrower rdf:resource="descriptor/19705-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16435-6">
    <skos:narrower rdf:resource="descriptor/15738-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16435-6">
    <skos:narrower rdf:resource="descriptor/20826-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16435-6">
    <skos:narrower rdf:resource="descriptor/18772-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10475-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10035-3">
    <skos:prefLabel xml:lang="en">Economic dynamics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10035-3">
    <skos:prefLabel xml:lang="de">Dynamisches Modell</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18119-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11559-6">
    <skos:prefLabel xml:lang="en">Civil service</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11559-6">
    <skos:prefLabel xml:lang="de">Öffentlicher Dienst</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11394-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18360-1">
    <skos:related rdf:resource="descriptor/18163-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18360-1">
    <skos:related rdf:resource="descriptor/12974-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18360-1">
    <skos:related rdf:resource="descriptor/19838-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16344-2">
    <skos:related rdf:resource="descriptor/19694-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16344-2">
    <skos:related rdf:resource="descriptor/13397-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11551-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19421-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10537-0">
    <skos:related rdf:resource="descriptor/10500-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10537-0">
    <skos:related rdf:resource="descriptor/19242-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10537-0">
    <skos:related rdf:resource="descriptor/10494-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10537-0">
    <skos:related rdf:resource="descriptor/10521-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16277-5">
    <skos:related rdf:resource="descriptor/19857-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16277-5">
    <skos:related rdf:resource="descriptor/10975-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16277-5">
    <skos:related rdf:resource="descriptor/16287-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16277-5">
    <skos:related rdf:resource="descriptor/16273-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16277-5">
    <skos:related rdf:resource="descriptor/16226-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16277-5">
    <skos:related rdf:resource="descriptor/15472-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18088-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19637-0">
    <skos:altLabel xml:lang="de">Rentnerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19637-0">
    <skos:altLabel xml:lang="de">Rentenbezieher</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19637-0">
    <skos:altLabel xml:lang="de">Pensionärinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19637-0">
    <skos:altLabel xml:lang="de">Pensionäre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11848-6">
    <skos:prefLabel xml:lang="en">Housing legislation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11848-6">
    <skos:prefLabel xml:lang="de">Wohnungsrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15571-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18822-3">
    <skos:prefLabel xml:lang="en">Public-private partnership</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18822-3">
    <skos:prefLabel xml:lang="de">Public-Private Partnership</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70007">
    <skos:broader rdf:resource="thsys/n"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16635-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10843-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10713-6">
    <skos:prefLabel xml:lang="en">Tariff policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10713-6">
    <skos:prefLabel xml:lang="de">Zollpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15450-1">
    <skos:prefLabel xml:lang="en">Noncooperative game</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15450-1">
    <skos:prefLabel xml:lang="de">Nichtkooperatives Spiel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18534-1">
    <skos:altLabel xml:lang="de">Geschichte des Außenhandels</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18534-1">
    <skos:altLabel xml:lang="de">Außenwirtschaftsgeschichte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12572-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12912-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11389-5">
    <skos:altLabel xml:lang="en">Reemployment</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11389-5">
    <skos:altLabel xml:lang="de">Wiedereingliederung in den Beruf</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11389-5">
    <skos:altLabel xml:lang="de">Eingliederungsbeihilfe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11389-5">
    <skos:altLabel xml:lang="de">Eingliederung in den Beruf</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11389-5">
    <skos:altLabel xml:lang="de">Eingliederung in den Arbeitsmarkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11389-5">
    <skos:altLabel xml:lang="de">Beruflicher Wiedereinstieg</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11389-5">
    <skos:altLabel xml:lang="de">Berufliche Wiedereingliederung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70292">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13409-1">
    <skos:prefLabel xml:lang="en">Territorial authority</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13409-1">
    <skos:prefLabel xml:lang="de">Gebietskörperschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12015-6">
    <skos:broader rdf:resource="thsys/73351"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12015-6">
    <skos:broader rdf:resource="thsys/71066"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12015-6">
    <skos:broader rdf:resource="descriptor/10326-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12015-6">
    <skos:broader rdf:resource="descriptor/15745-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18347-0">
    <skos:broader rdf:resource="thsys/71016"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18347-0">
    <skos:broader rdf:resource="thsys/70227"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18347-0">
    <skos:broader rdf:resource="descriptor/18351-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18123-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15764-6">
    <skos:broader rdf:resource="thsys/70239"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15764-6">
    <skos:broader rdf:resource="thsys/70120"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15764-6">
    <skos:broader rdf:resource="descriptor/16633-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14619-6">
    <skos:altLabel xml:lang="de">Photopapier</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14619-6">
    <skos:altLabel xml:lang="de">Photographischer Film</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14619-6">
    <skos:altLabel xml:lang="de">Photochemikalie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14619-6">
    <skos:altLabel xml:lang="de">Fotopapier</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14619-6">
    <skos:altLabel xml:lang="de">Fotografischer Film</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14619-6">
    <skos:altLabel xml:lang="de">Film für Fotografie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14150-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14076-4">
    <skos:prefLabel xml:lang="en">Fungus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14076-4">
    <skos:prefLabel xml:lang="de">Pilz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70182">
    <rdfs:label xml:lang="en">Population dynamics</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70182">
    <rdfs:label xml:lang="de">Bevölkerungsentwicklung</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14040-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11055-6">
    <skos:broader rdf:resource="thsys/71102"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11055-6">
    <skos:broader rdf:resource="descriptor/11068-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14298-0">
    <skos:broader rdf:resource="thsys/70134"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14298-0">
    <skos:broader rdf:resource="descriptor/14296-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14989-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10942-5">
    <skos:prefLabel xml:lang="en">Restrictive business practices</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10942-5">
    <skos:prefLabel xml:lang="de">Wettbewerbsbeschränkung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19005-2">
    <skos:broader rdf:resource="thsys/70244"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19005-2">
    <skos:broader rdf:resource="thsys/70235"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19005-2">
    <skos:broader rdf:resource="thsys/70052"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11790-4">
    <skos:broader rdf:resource="thsys/71110"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11790-4">
    <skos:broader rdf:resource="thsys/70575"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13965-6">
    <skos:narrower rdf:resource="descriptor/18709-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13479-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18984-5">
    <skos:prefLabel xml:lang="en">Wetlands</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18984-5">
    <skos:prefLabel xml:lang="de">Feuchtgebiet</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13287-3">
    <skos:prefLabel xml:lang="en">Credit cooperative</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13287-3">
    <skos:prefLabel xml:lang="de">Kreditgenossenschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18257-1">
    <skos:related rdf:resource="descriptor/14698-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19028-4">
    <skos:broader rdf:resource="thsys/70582"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19176-3">
    <skos:prefLabel xml:lang="en">ECOWAS countries</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19176-3">
    <skos:prefLabel xml:lang="de">ECOWAS-Staaten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12589-6">
    <skos:prefLabel xml:lang="en">Employee training</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12589-6">
    <skos:prefLabel xml:lang="de">Personalentwicklung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18447-3">
    <skos:related rdf:resource="descriptor/18366-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18447-3">
    <skos:related rdf:resource="descriptor/18182-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18447-3">
    <skos:related rdf:resource="descriptor/12925-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18447-3">
    <skos:related rdf:resource="descriptor/18814-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18447-3">
    <skos:related rdf:resource="descriptor/16271-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18447-3">
    <skos:related rdf:resource="descriptor/11582-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18447-3">
    <skos:related rdf:resource="descriptor/19723-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70455">
    <rdfs:label xml:lang="en">Interest rate</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70455">
    <rdfs:label xml:lang="de">Zins</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19414-6">
    <skos:altLabel xml:lang="de">Exportbesteuerung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19414-6">
    <skos:altLabel xml:lang="de">Ausfuhrsteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19821-0">
    <skos:altLabel xml:lang="de">Genehmigungsverfahren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12280-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12996-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11839-0">
    <skos:prefLabel xml:lang="en">Protection of tenants</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11839-0">
    <skos:prefLabel xml:lang="de">Mieterschutz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17689-5">
    <skos:broader rdf:resource="thsys/70429"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17689-5">
    <skos:broader rdf:resource="descriptor/17687-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71011">
    <skos:narrower rdf:resource="descriptor/12979-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71011">
    <skos:narrower rdf:resource="descriptor/18378-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71011">
    <skos:narrower rdf:resource="descriptor/16023-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71011">
    <skos:narrower rdf:resource="descriptor/18990-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71011">
    <skos:narrower rdf:resource="descriptor/18379-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71011">
    <skos:narrower rdf:resource="descriptor/18377-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71011">
    <skos:narrower rdf:resource="descriptor/18380-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71011">
    <skos:narrower rdf:resource="descriptor/15734-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71011">
    <skos:narrower rdf:resource="descriptor/15733-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71011">
    <skos:narrower rdf:resource="descriptor/10490-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10093-3">
    <skos:related rdf:resource="descriptor/11765-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10093-3">
    <skos:related rdf:resource="descriptor/10110-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10093-3">
    <skos:related rdf:resource="descriptor/16623-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10093-3">
    <skos:related rdf:resource="descriptor/19768-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11498-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17607-5">
    <skos:altLabel xml:lang="de">Republik Malediven</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11305-2">
    <skos:broader rdf:resource="thsys/70235"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11305-2">
    <skos:broader rdf:resource="thsys/70113"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11305-2">
    <skos:broader rdf:resource="descriptor/15715-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13387-6">
    <skos:prefLabel xml:lang="en">Data-processing centre</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13387-6">
    <skos:prefLabel xml:lang="de">Rechenzentrum</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13347-4">
    <skos:related rdf:resource="descriptor/18649-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13347-4">
    <skos:related rdf:resource="descriptor/14004-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13347-4">
    <skos:related rdf:resource="descriptor/13660-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13347-4">
    <skos:related rdf:resource="descriptor/16516-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13347-4">
    <skos:related rdf:resource="descriptor/18928-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13347-4">
    <skos:related rdf:resource="descriptor/13658-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13347-4">
    <skos:related rdf:resource="descriptor/13352-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19853-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11641-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10209-3">
    <skos:altLabel xml:lang="de">Oligopson</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12896-4">
    <skos:prefLabel xml:lang="en">Audit certificate</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12896-4">
    <skos:prefLabel xml:lang="de">Bestätigungsvermerk</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19519-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11328-4">
    <skos:altLabel xml:lang="en">Living wage ordinances</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11328-4">
    <skos:altLabel xml:lang="de">Lohnnormierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11328-4">
    <skos:altLabel xml:lang="de">Lohnminimum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19129-5">
    <skos:broader rdf:resource="thsys/70550"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19129-5">
    <skos:broader rdf:resource="descriptor/12808-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18346-2">
    <skos:related rdf:resource="descriptor/14364-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18346-2">
    <skos:related rdf:resource="descriptor/14408-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18346-2">
    <skos:related rdf:resource="descriptor/14407-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18346-2">
    <skos:related rdf:resource="descriptor/18357-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18346-2">
    <skos:related rdf:resource="descriptor/12977-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15102-3">
    <skos:altLabel xml:lang="de">Einkommenstatistik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19263-1">
    <skos:broader rdf:resource="thsys/70305"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19263-1">
    <skos:broader rdf:resource="descriptor/19262-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12039-6">
    <skos:prefLabel xml:lang="en">Business statistics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12039-6">
    <skos:prefLabel xml:lang="de">Betriebsstatistik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15952-5">
    <skos:narrower rdf:resource="descriptor/19640-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15947-5">
    <skos:scopeNote xml:lang="de">Verbreitung von Krankheiten im allgemeinen oder einer bestimmten Krankheit. Zahlenmäßiges Verhältnis zwischen Kranken und Gesunden einer Bevölkerung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/p">
    <rdfs:label xml:lang="en">Commodities</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/p">
    <rdfs:label xml:lang="de">Produkte</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14436-0">
    <skos:related rdf:resource="descriptor/13244-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14436-0">
    <skos:related rdf:resource="descriptor/13055-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10109-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18671-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70445">
    <skos:prefLabel xml:lang="en">G.03.01.01  East Africa</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70445">
    <skos:prefLabel xml:lang="de">G.03.01.01  Ostafrika</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12807-1">
    <skos:related rdf:resource="descriptor/10015-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12807-1">
    <skos:related rdf:resource="descriptor/10928-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12807-1">
    <skos:related rdf:resource="descriptor/10459-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14029-6">
    <skos:prefLabel xml:lang="en">Leisure-time product</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14029-6">
    <skos:prefLabel xml:lang="de">Freizeitartikel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10077-1">
    <skos:prefLabel xml:lang="en">Intertemporal equilibrium model</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10077-1">
    <skos:prefLabel xml:lang="de">Intertemporales Gleichgewicht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19072-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15988-5">
    <skos:related rdf:resource="descriptor/15769-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15988-5">
    <skos:related rdf:resource="descriptor/15727-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15988-5">
    <skos:related rdf:resource="descriptor/13325-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15988-5">
    <skos:related rdf:resource="descriptor/18958-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15988-5">
    <skos:related rdf:resource="descriptor/15009-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70031">
    <rdfs:label xml:lang="en">Clothing</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70031">
    <rdfs:label xml:lang="de">Bekleidung</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19627-3">
    <skos:related rdf:resource="descriptor/12955-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19627-3">
    <skos:related rdf:resource="descriptor/14169-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14638-2">
    <skos:broader rdf:resource="thsys/70418"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14638-2">
    <skos:broader rdf:resource="descriptor/14205-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14638-2">
    <skos:broader rdf:resource="descriptor/14192-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17013-6">
    <skos:broader rdf:resource="thsys/70131"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17013-6">
    <skos:broader rdf:resource="descriptor/17016-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17013-6">
    <skos:broader rdf:resource="descriptor/17983-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13861-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16642-1">
    <skos:related rdf:resource="descriptor/16730-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16642-1">
    <skos:related rdf:resource="descriptor/19733-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16642-1">
    <skos:related rdf:resource="descriptor/16723-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16642-1">
    <skos:related rdf:resource="descriptor/16544-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16642-1">
    <skos:related rdf:resource="descriptor/15806-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16642-1">
    <skos:related rdf:resource="descriptor/16641-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16642-1">
    <skos:related rdf:resource="descriptor/19752-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16642-1">
    <skos:related rdf:resource="descriptor/16432-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16642-1">
    <skos:related rdf:resource="descriptor/16207-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16642-1">
    <skos:related rdf:resource="descriptor/16430-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16642-1">
    <skos:related rdf:resource="descriptor/19754-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16642-1">
    <skos:related rdf:resource="descriptor/16405-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16642-1">
    <skos:related rdf:resource="descriptor/20855-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16642-1">
    <skos:related rdf:resource="descriptor/16633-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18989-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70161">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">P.23</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18433-0">
    <skos:related rdf:resource="descriptor/16039-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18433-0">
    <skos:related rdf:resource="descriptor/16037-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18433-0">
    <skos:related rdf:resource="descriptor/18643-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18433-0">
    <skos:related rdf:resource="descriptor/16053-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18433-0">
    <skos:related rdf:resource="descriptor/18762-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18433-0">
    <skos:related rdf:resource="descriptor/15606-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18433-0">
    <skos:related rdf:resource="descriptor/15605-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18433-0">
    <skos:related rdf:resource="descriptor/15604-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18433-0">
    <skos:related rdf:resource="descriptor/16190-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70874">
    <skos:broader rdf:resource="thsys/70876"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16436-4">
    <skos:broader rdf:resource="thsys/73366"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16436-4">
    <skos:broader rdf:resource="descriptor/16292-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11729-0">
    <skos:broader rdf:resource="thsys/71082"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11729-0">
    <skos:broader rdf:resource="descriptor/10994-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11729-0">
    <skos:broader rdf:resource="descriptor/11728-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15872-3">
    <skos:broader rdf:resource="thsys/73354"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15872-3">
    <skos:broader rdf:resource="descriptor/15876-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12757-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19851-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12951-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10798-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19291-3">
    <skos:related rdf:resource="descriptor/10567-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19291-3">
    <skos:related rdf:resource="descriptor/12697-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19291-3">
    <skos:related rdf:resource="descriptor/18160-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19291-3">
    <skos:related rdf:resource="descriptor/11915-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19291-3">
    <skos:related rdf:resource="descriptor/11914-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19291-3">
    <skos:related rdf:resource="descriptor/11913-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19291-3">
    <skos:related rdf:resource="descriptor/16552-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19787-2">
    <skos:broader rdf:resource="thsys/73347"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19397-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12575-3">
    <skos:altLabel xml:lang="de">Sozialplanabfindung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17791-0">
    <skos:altLabel xml:lang="de">Zentralamerika</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17791-0">
    <skos:altLabel xml:lang="de">Mittelamerikanische Staaten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17791-0">
    <skos:altLabel xml:lang="de">Mittelamerikanische Länder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17553-2">
    <skos:altLabel xml:lang="de">Asiatische Staaten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17553-2">
    <skos:altLabel xml:lang="de">Asiatische Länder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11565-4">
    <skos:prefLabel xml:lang="en">War victims' relief</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11565-4">
    <skos:prefLabel xml:lang="de">Kriegsopferfürsorge</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15111-2">
    <skos:altLabel xml:lang="en">Data measurement process</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15111-2">
    <skos:altLabel xml:lang="de">Zufallszahlenauswahl</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15111-2">
    <skos:altLabel xml:lang="de">Erhebungstechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15111-2">
    <skos:altLabel xml:lang="de">Auswahlverfahren (Statistik)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15111-2">
    <skos:altLabel xml:lang="de">Auswahltechnik (Statistik)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18436-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13840-5">
    <skos:prefLabel xml:lang="en">Home ownership</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13840-5">
    <skos:prefLabel xml:lang="de">Wohneigentum</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19777-5">
    <skos:related rdf:resource="descriptor/16719-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19777-5">
    <skos:related rdf:resource="descriptor/19608-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13025-5">
    <skos:related rdf:resource="descriptor/14190-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13025-5">
    <skos:related rdf:resource="descriptor/14293-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14032-3">
    <skos:narrower rdf:resource="descriptor/14798-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14032-3">
    <skos:narrower rdf:resource="descriptor/14502-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10691-4">
    <skos:related rdf:resource="descriptor/10685-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13375-6">
    <skos:prefLabel xml:lang="en">Dentists</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13375-6">
    <skos:prefLabel xml:lang="de">Zahnärzte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16176-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13733-6">
    <skos:altLabel xml:lang="de">Vinkulierte Namensaktie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13733-6">
    <skos:altLabel xml:lang="de">Vinkulierte Aktie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19807-1">
    <skos:prefLabel xml:lang="en">Data mining</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19807-1">
    <skos:prefLabel xml:lang="de">Data Mining</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13731-3">
    <zbwext:useInsteadNote rdf:nodeID="id165"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18441-1">
    <skos:prefLabel xml:lang="en">EU food legislation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18441-1">
    <skos:prefLabel xml:lang="de">EU-Lebensmittelrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15728-3">
    <skos:broader rdf:resource="thsys/70818"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15728-3">
    <skos:broader rdf:resource="descriptor/15727-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12035-0">
    <skos:broader rdf:resource="thsys/70394"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16774-5">
    <skos:narrower rdf:resource="descriptor/18770-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16774-5">
    <skos:narrower rdf:resource="descriptor/18769-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16774-5">
    <skos:narrower rdf:resource="descriptor/18940-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12956-5">
    <skos:altLabel xml:lang="de">Viehzucht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15621-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id57">
    <rdf:value xml:lang="de">Für Wertpapierbörse</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13416-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11609-3">
    <skos:prefLabel xml:lang="en">Municipal bond</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11609-3">
    <skos:prefLabel xml:lang="de">Kommunalobligation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18189-6">
    <skos:broader rdf:resource="thsys/70465"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18189-6">
    <skos:broader rdf:resource="descriptor/14151-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18189-6">
    <skos:broader rdf:resource="descriptor/14149-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10787-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10237-5">
    <skos:altLabel xml:lang="de">Nachfragetheorie der Gesamtwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10237-5">
    <skos:altLabel xml:lang="de">Nachfragefunktion der Gesamtwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10237-5">
    <skos:altLabel xml:lang="de">Nachfrage der Gesamtwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10237-5">
    <skos:altLabel xml:lang="de">Makroökonomische Nachfragefunktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10337-1">
    <skos:broader rdf:resource="thsys/70575"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10337-1">
    <skos:broader rdf:resource="thsys/70020"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10337-1">
    <skos:broader rdf:resource="descriptor/10338-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10337-1">
    <skos:broader rdf:resource="descriptor/18010-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16550-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14510-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12282-4">
    <skos:prefLabel xml:lang="en">contribution of capital</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12282-4">
    <skos:prefLabel xml:lang="de">Kapitalbeteiligung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14461-1">
    <skos:prefLabel xml:lang="en">Special ship</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14461-1">
    <skos:prefLabel xml:lang="de">Spezialschiff</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18138-2">
    <skos:related rdf:resource="descriptor/19202-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18138-2">
    <skos:related rdf:resource="descriptor/10991-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18138-2">
    <skos:related rdf:resource="descriptor/10987-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18138-2">
    <skos:related rdf:resource="descriptor/11223-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18138-2">
    <skos:related rdf:resource="descriptor/12210-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18138-2">
    <skos:related rdf:resource="descriptor/19198-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18138-2">
    <skos:related rdf:resource="descriptor/10382-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71025">
    <skos:broader rdf:resource="thsys/71028"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13692-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10349-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12935-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16421-3">
    <skos:broader rdf:resource="thsys/73375"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16421-3">
    <skos:broader rdf:resource="thsys/73366"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12083-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10615-6">
    <skos:related rdf:resource="descriptor/10614-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10615-6">
    <skos:related rdf:resource="descriptor/10638-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16606-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11855-2">
    <skos:broader rdf:resource="thsys/70058"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11855-2">
    <skos:broader rdf:resource="descriptor/11857-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12110-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id157">
    <rdfs:seeAlso rdf:resource="descriptor/10827-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13570-1">
    <skos:broader rdf:resource="thsys/70943"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18490-2">
    <skos:altLabel xml:lang="de">Industrieller Standort</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71052">
    <rdfs:label xml:lang="de">Führungskräfte und akademische Berufe</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15982-3">
    <skos:related rdf:resource="descriptor/19215-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15982-3">
    <skos:related rdf:resource="descriptor/16039-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15982-3">
    <skos:related rdf:resource="descriptor/20077-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15982-3">
    <skos:related rdf:resource="descriptor/16184-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19614-5">
    <skos:altLabel xml:lang="de">Meereswirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17570-2">
    <skos:broader rdf:resource="thsys/70147"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17570-2">
    <skos:broader rdf:resource="descriptor/19166-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16774-5">
    <skos:altLabel xml:lang="en">Databank</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16774-5">
    <skos:altLabel xml:lang="de">Online-Datenbank</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16774-5">
    <skos:altLabel xml:lang="de">Elektronische Datenbank</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12741-5">
    <skos:related rdf:resource="descriptor/10327-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12741-5">
    <skos:related rdf:resource="descriptor/10187-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14958-5">
    <skos:prefLabel xml:lang="en">Edible fat</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14958-5">
    <skos:prefLabel xml:lang="de">Speisefett</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10517-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13564-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15686-0">
    <skos:narrower rdf:resource="descriptor/19746-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15686-0">
    <skos:narrower rdf:resource="descriptor/16745-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19071-3">
    <skos:altLabel xml:lang="en">Internationalization</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19071-3">
    <skos:altLabel xml:lang="en">Globalisation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19071-3">
    <skos:altLabel xml:lang="de">Transnationalisierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19071-3">
    <skos:altLabel xml:lang="de">Internationalisierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11198-5">
    <skos:broader rdf:resource="thsys/71005"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11198-5">
    <skos:broader rdf:resource="descriptor/11180-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18381-0">
    <skos:scopeNote xml:lang="de">Vor allem die Theorie der intertemporalen Allokation von Ressourcen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15778-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71054">
    <rdfs:label xml:lang="de">Kaufmännische, Organisations- und Verwaltungsberufe</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12024-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19578-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18220-3">
    <skos:altLabel xml:lang="de">Kalbfleisch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25621-1">
    <skos:prefLabel xml:lang="en">Personal bankruptcy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25621-1">
    <skos:prefLabel xml:lang="de">Privatinsolvenz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19548-6">
    <skos:related rdf:resource="descriptor/10466-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19548-6">
    <skos:related rdf:resource="descriptor/21041-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19548-6">
    <skos:related rdf:resource="descriptor/20277-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19548-6">
    <skos:related rdf:resource="descriptor/12262-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19548-6">
    <skos:related rdf:resource="descriptor/18070-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19548-6">
    <skos:related rdf:resource="descriptor/10945-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19548-6">
    <skos:related rdf:resource="descriptor/18082-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19548-6">
    <skos:related rdf:resource="descriptor/10459-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19548-6">
    <skos:related rdf:resource="descriptor/10475-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19548-6">
    <skos:related rdf:resource="descriptor/10443-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10882-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15715-5">
    <skos:related rdf:resource="descriptor/13849-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15715-5">
    <skos:related rdf:resource="descriptor/10342-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15715-5">
    <skos:related rdf:resource="descriptor/18887-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15715-5">
    <skos:related rdf:resource="descriptor/15073-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15715-5">
    <skos:related rdf:resource="descriptor/16639-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15715-5">
    <skos:related rdf:resource="descriptor/16638-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14423-2">
    <skos:altLabel xml:lang="en">Space satellite</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14423-2">
    <skos:altLabel xml:lang="de">Weltraumrakete</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14423-2">
    <skos:altLabel xml:lang="de">Satellit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id5">
    <rdf:value xml:lang="de">Für Abstimmungen im Rahmen von realen Wahlvorgängen</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11820-0">
    <skos:altLabel xml:lang="de">Industrielle Sektorpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11820-0">
    <skos:altLabel xml:lang="de">Industrieförderung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19237-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19749-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11464-3">
    <skos:prefLabel xml:lang="en">Monetary aggregate</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11464-3">
    <skos:prefLabel xml:lang="de">Monetäres Aggregat</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10697-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18889-6">
    <skos:altLabel xml:lang="de">Europäische Bildungspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18889-6">
    <skos:altLabel xml:lang="de">EU-Bildungsprogramm</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19070-5">
    <skos:prefLabel xml:lang="en">Empirical method</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19070-5">
    <skos:prefLabel xml:lang="de">Empirische Methode</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10745-0">
    <skos:altLabel xml:lang="en">Monetary agreement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10745-0">
    <skos:altLabel xml:lang="de">Währungsabkommen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10745-0">
    <skos:altLabel xml:lang="de">Internationale währungspolitische Zusammenarbeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11258-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18681-2">
    <skos:prefLabel xml:lang="en">Securities investment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18681-2">
    <skos:prefLabel xml:lang="de">Wertpapieranlage</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12533-5">
    <skos:related rdf:resource="descriptor/12490-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70415">
    <skos:prefLabel xml:lang="de">P.09.03  Wasserfahrzeuge</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18343-1">
    <skos:altLabel xml:lang="de">Überfischung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10672-1">
    <skos:related rdf:resource="descriptor/10677-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10672-1">
    <skos:related rdf:resource="descriptor/10671-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13795-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14327-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10964-2">
    <skos:related rdf:resource="descriptor/16081-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10964-2">
    <skos:related rdf:resource="descriptor/10963-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10964-2">
    <skos:related rdf:resource="descriptor/10942-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10964-2">
    <skos:related rdf:resource="descriptor/10895-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10964-2">
    <skos:related rdf:resource="descriptor/10951-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10964-2">
    <skos:related rdf:resource="descriptor/12788-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10964-2">
    <skos:related rdf:resource="descriptor/10917-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19723-0">
    <skos:prefLabel xml:lang="en">Military aid</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19723-0">
    <skos:prefLabel xml:lang="de">Militärhilfe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18101-4">
    <skos:related rdf:resource="descriptor/12821-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18101-4">
    <skos:related rdf:resource="descriptor/18835-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18101-4">
    <skos:related rdf:resource="descriptor/18821-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12158-5">
    <skos:broader rdf:resource="thsys/70485"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12158-5">
    <skos:broader rdf:resource="descriptor/19163-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19334-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70873">
    <skos:prefLabel xml:lang="en">W.21.03  Trade fair industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70873">
    <skos:prefLabel xml:lang="de">W.21.03  Messewirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18695-5">
    <skos:broader rdf:resource="thsys/71094"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18695-5">
    <skos:broader rdf:resource="thsys/71084"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18695-5">
    <skos:broader rdf:resource="descriptor/18694-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18695-5">
    <skos:broader rdf:resource="descriptor/19358-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18431-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12370-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16353-1">
    <skos:narrower rdf:resource="descriptor/19700-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15011-6">
    <skos:related rdf:resource="descriptor/18964-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15011-6">
    <skos:related rdf:resource="descriptor/13324-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18279-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10880-1">
    <skos:altLabel xml:lang="en">Trade document</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10880-1">
    <skos:altLabel xml:lang="en">Terms of shipment</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10880-1">
    <skos:altLabel xml:lang="en">Incoterms</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10880-1">
    <skos:altLabel xml:lang="en">Fob (Free on Board)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10880-1">
    <skos:altLabel xml:lang="en">Fas (Free alongside ship)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10880-1">
    <skos:altLabel xml:lang="en">Combi terms</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10880-1">
    <skos:altLabel xml:lang="en">Cif (Cost, Insurance, Freight)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10880-1">
    <skos:altLabel xml:lang="de">Technik des Außenhandels</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10880-1">
    <skos:altLabel xml:lang="de">Lieferklauseln im Außenhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10880-1">
    <skos:altLabel xml:lang="de">ICC-Bedingungen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10880-1">
    <skos:altLabel xml:lang="de">Handelsklauseln im Außenhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10880-1">
    <skos:altLabel xml:lang="de">Frachtklauseln im Außenhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10880-1">
    <skos:altLabel xml:lang="de">ECE-Bedingungen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10880-1">
    <skos:altLabel xml:lang="de">Dokumente gegen Zahlung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10880-1">
    <skos:altLabel xml:lang="de">Dokumente gegen Akzept</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10880-1">
    <skos:altLabel xml:lang="de">Außenhandelstechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10880-1">
    <skos:altLabel xml:lang="de">Außenhandelsdokument</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11559-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10635-0">
    <skos:broader rdf:resource="thsys/71100"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10635-0">
    <skos:broader rdf:resource="descriptor/10629-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12733-4">
    <skos:related rdf:resource="descriptor/19806-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15776-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13242-4">
    <skos:altLabel xml:lang="de">Schubschifffahrt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13242-4">
    <skos:altLabel xml:lang="de">Schubschiffahrt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13242-4">
    <skos:altLabel xml:lang="de">Partikulier</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13242-4">
    <skos:altLabel xml:lang="de">Kanalschifffahrt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13242-4">
    <skos:altLabel xml:lang="de">Kanalschiffahrt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13242-4">
    <skos:altLabel xml:lang="de">Flussschifffahrt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13242-4">
    <skos:altLabel xml:lang="de">Flußschiffahrt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13242-4">
    <skos:altLabel xml:lang="de">Binnenwasserstraßenverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13242-4">
    <skos:altLabel xml:lang="de">Binnenschiffahrt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71031">
    <skos:broader rdf:resource="thsys/70354"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11367-1">
    <skos:related rdf:resource="descriptor/18920-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11367-1">
    <skos:related rdf:resource="descriptor/18888-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11367-1">
    <skos:related rdf:resource="descriptor/11398-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11367-1">
    <skos:related rdf:resource="descriptor/11397-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11367-1">
    <skos:related rdf:resource="descriptor/11407-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11367-1">
    <skos:related rdf:resource="descriptor/11354-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11367-1">
    <skos:related rdf:resource="descriptor/11353-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11367-1">
    <skos:related rdf:resource="descriptor/11355-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11367-1">
    <skos:related rdf:resource="descriptor/18856-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12972-0">
    <skos:related rdf:resource="descriptor/14125-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11147-1">
    <skos:broader rdf:resource="thsys/71101"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11147-1">
    <skos:broader rdf:resource="thsys/71028"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11147-1">
    <skos:broader rdf:resource="descriptor/18320-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11464-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18281-4">
    <skos:altLabel xml:lang="en">Combined heat and power</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18281-4">
    <skos:altLabel xml:lang="en">CHP (Combined Heat and Power)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18281-4">
    <skos:altLabel xml:lang="de">WKK (Wärmekraftkopplung)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18281-4">
    <skos:altLabel xml:lang="de">Wärmekraftkopplung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18281-4">
    <skos:altLabel xml:lang="de">KWK (Kraft-Wärme-Kopplung)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18281-4">
    <skos:altLabel xml:lang="de">Kraftwärmekopplung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18281-4">
    <skos:altLabel xml:lang="de">Kraft-Wärme-Kopplung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18281-4">
    <skos:altLabel xml:lang="de">BHKW (Blockheizkraftwerk)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11769-2">
    <skos:narrower rdf:resource="descriptor/11321-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11769-2">
    <skos:narrower rdf:resource="descriptor/20368-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#notation">
    <skos:definition xml:lang="en">A notation, also known as classification code, is a string of
      characters such as "T58.5" or "303.4833" used to uniquely identify a concept within the scope
      of a given concept scheme.</skos:definition>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13090-1">
    <skos:broader rdf:resource="thsys/71003"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13090-1">
    <skos:broader rdf:resource="descriptor/13085-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18251-6">
    <skos:altLabel xml:lang="de">Optoelektronisches Bauelement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16315-2">
    <skos:broader rdf:resource="thsys/73381"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16315-2">
    <skos:broader rdf:resource="descriptor/16013-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70394">
    <skos:broader rdf:resource="thsys/70301"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70870">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19428-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20077-2">
    <skos:related rdf:resource="descriptor/15982-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20077-2">
    <skos:related rdf:resource="descriptor/11057-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20077-2">
    <skos:related rdf:resource="descriptor/10401-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10685-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12829-5">
    <skos:prefLabel xml:lang="en">Advertising media</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12829-5">
    <skos:prefLabel xml:lang="de">Werbeträger</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18996-5">
    <skos:broader rdf:resource="thsys/71016"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18996-5">
    <skos:broader rdf:resource="thsys/70227"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18996-5">
    <skos:broader rdf:resource="descriptor/10012-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10975-4">
    <skos:broader rdf:resource="thsys/70858"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10975-4">
    <skos:broader rdf:resource="thsys/70274"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10975-4">
    <skos:broader rdf:resource="descriptor/10032-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12296-0">
    <skos:altLabel xml:lang="de">Rücklagenpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12296-0">
    <skos:altLabel xml:lang="de">Reserven der Bilanz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12296-0">
    <skos:altLabel xml:lang="de">Offene Rücklage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12296-0">
    <skos:altLabel xml:lang="de">Gewinn- und Kapitalrücklagen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12296-0">
    <skos:altLabel xml:lang="de">Gesetzliche Rücklage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12296-0">
    <skos:altLabel xml:lang="de">Freie Rücklage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16007-6">
    <skos:prefLabel xml:lang="en">Public Law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16007-6">
    <skos:prefLabel xml:lang="de">Öffentliches Recht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11731-6">
    <skos:broader rdf:resource="thsys/71082"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11731-6">
    <skos:broader rdf:resource="thsys/71081"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11731-6">
    <skos:broader rdf:resource="descriptor/11728-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14724-2">
    <skos:related rdf:resource="descriptor/15672-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14724-2">
    <skos:related rdf:resource="descriptor/19788-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14724-2">
    <skos:related rdf:resource="descriptor/14728-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14724-2">
    <skos:related rdf:resource="descriptor/14817-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14724-2">
    <skos:related rdf:resource="descriptor/15941-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10401-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19776-0">
    <skos:prefLabel xml:lang="en">Rhetoric</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19776-0">
    <skos:prefLabel xml:lang="de">Rhetorik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70071">
    <rdfs:label xml:lang="de">Agrarprodukte</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12949-2">
    <skos:altLabel xml:lang="en">Plant breeding</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12949-2">
    <skos:altLabel xml:lang="de">Sonderkultur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12949-2">
    <skos:altLabel xml:lang="de">Pflanzenzüchtung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12949-2">
    <skos:altLabel xml:lang="de">Dauerkultur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71084">
    <rdfs:label xml:lang="en">Development aid</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71084">
    <rdfs:label xml:lang="de">Entwicklungshilfe</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16720-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19191-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11355-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11459-3">
    <skos:prefLabel xml:lang="en">Credit rationing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11459-3">
    <skos:prefLabel xml:lang="de">Kreditrationierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14627-0">
    <skos:prefLabel xml:lang="en">Sulphur</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14627-0">
    <skos:prefLabel xml:lang="de">Schwefel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17595-0">
    <skos:prefLabel xml:lang="en">Central Asia</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17595-0">
    <skos:prefLabel xml:lang="de">Zentralasien</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18687-4">
    <skos:broader rdf:resource="thsys/71106"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18687-4">
    <skos:broader rdf:resource="thsys/71104"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18687-4">
    <skos:broader rdf:resource="descriptor/13744-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18687-4">
    <skos:broader rdf:resource="descriptor/13735-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14798-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18939-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20176-0">
    <skos:scopeNote xml:lang="de">Es handelt sich um Güter, die aufgrund eines besonderen Vertrauens in bestimmte Qualitätsmerkmale erworben werden, welche aber kaum nachprüfbar sind, z.B. Bioprodukte.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12446-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19208-2">
    <skos:broader rdf:resource="thsys/70180"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19208-2">
    <skos:broader rdf:resource="thsys/70057"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19208-2">
    <skos:broader rdf:resource="descriptor/11039-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19208-2">
    <skos:broader rdf:resource="descriptor/10040-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70940">
    <rdfs:label xml:lang="en">Freight transport</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70940">
    <rdfs:label xml:lang="de">Güterverkehr</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20977-1">
    <skos:broader rdf:resource="thsys/70562"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20977-1">
    <skos:broader rdf:resource="descriptor/12111-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19033-4">
    <skos:prefLabel xml:lang="en">Measurement</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19033-4">
    <skos:prefLabel xml:lang="de">Messung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18230-0">
    <skos:broader rdf:resource="thsys/70476"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18230-0">
    <skos:broader rdf:resource="thsys/70161"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18230-0">
    <skos:broader rdf:resource="descriptor/14702-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18230-0">
    <skos:broader rdf:resource="descriptor/15009-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18798-2">
    <skos:related rdf:resource="descriptor/18797-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73294">
    <rdfs:label xml:lang="de">Genussmittelindustrie</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14353-4">
    <skos:scopeNote xml:lang="de">Auch für Raketenantriebssysteme.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20068-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20381-6">
    <skos:related rdf:resource="descriptor/12565-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20381-6">
    <skos:related rdf:resource="descriptor/12562-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20381-6">
    <skos:related rdf:resource="descriptor/12572-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20381-6">
    <skos:related rdf:resource="descriptor/11270-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11706-5">
    <skos:broader rdf:resource="thsys/71075"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11706-5">
    <skos:broader rdf:resource="descriptor/11704-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11706-5">
    <skos:broader rdf:resource="descriptor/11729-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18567-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18349-3">
    <skos:prefLabel xml:lang="en">Energy management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18349-3">
    <skos:prefLabel xml:lang="de">Energiemanagement</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19208-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70505">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10002-4">
    <skos:altLabel xml:lang="de">Kommissionierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10002-4">
    <skos:altLabel xml:lang="de">Auftragszusammenstellung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10527-3">
    <skos:related rdf:resource="descriptor/10675-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10527-3">
    <skos:related rdf:resource="descriptor/10636-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19426-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11294-2">
    <skos:prefLabel xml:lang="en">Occupational status</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11294-2">
    <skos:prefLabel xml:lang="de">Erwerbsstatus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12320-1">
    <skos:prefLabel xml:lang="en">Business value creation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12320-1">
    <skos:prefLabel xml:lang="de">Betriebliche Wertschöpfung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#inScheme">
    <rdfs:comment xml:lang="en">A concept may be a member of more than one concept scheme.</rdfs:comment>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19752-0">
    <skos:altLabel xml:lang="en">Social capital theory</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19752-0">
    <skos:altLabel xml:lang="de">Sozialkapital</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19752-0">
    <skos:altLabel xml:lang="de">Soziales Kapital</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18014-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14772-5">
    <skos:related rdf:resource="descriptor/18279-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14772-5">
    <skos:related rdf:resource="descriptor/18280-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14652-1">
    <skos:narrower rdf:resource="descriptor/14651-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70318">
    <skos:prefLabel xml:lang="de">P.08.02  Maschinen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71101">
    <skos:prefLabel xml:lang="en">V.06.01.02  Centrally planned economy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71101">
    <skos:prefLabel xml:lang="de">V.06.01.02  Planwirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24662-6">
    <skos:altLabel xml:lang="en">Essential facility doctrine</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24662-6">
    <skos:altLabel xml:lang="en">EFD (Essential facilities doctrine)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id88">
    <rdfs:seeAlso rdf:resource="descriptor/10559-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12270-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18603-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19055-1">
    <zbwext:useInsteadNote rdf:nodeID="id330"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16754-4">
    <skos:altLabel xml:lang="de">Informationsprozess</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16754-4">
    <skos:altLabel xml:lang="de">Informationsprozeß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19511-1">
    <skos:prefLabel xml:lang="en">Prisoners' welfare</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19511-1">
    <skos:prefLabel xml:lang="de">Straffälligenhilfe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18429-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18442-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18576-6">
    <skos:related rdf:resource="descriptor/13613-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18576-6">
    <skos:related rdf:resource="descriptor/11974-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18576-6">
    <skos:related rdf:resource="descriptor/11876-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18576-6">
    <skos:related rdf:resource="descriptor/13249-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18576-6">
    <skos:related rdf:resource="descriptor/18610-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15678-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14227-2">
    <skos:broader rdf:resource="thsys/70348"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14227-2">
    <skos:broader rdf:resource="descriptor/14237-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17679-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16650-2">
    <zbwext:useInsteadNote rdf:nodeID="id293"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11272-5">
    <skos:broader rdf:resource="thsys/70051"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12029-2">
    <skos:broader rdf:resource="thsys/71066"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12029-2">
    <skos:broader rdf:resource="thsys/70889"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13293-1">
    <skos:related rdf:resource="descriptor/12252-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13293-1">
    <skos:related rdf:resource="descriptor/13777-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15535-0">
    <skos:broader rdf:resource="thsys/71035"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15535-0">
    <skos:broader rdf:resource="descriptor/15055-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10268-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13409-1">
    <skos:related rdf:resource="descriptor/13413-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13409-1">
    <skos:related rdf:resource="descriptor/16322-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11996-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18688-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13488-0">
    <skos:broader rdf:resource="thsys/71050"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13488-0">
    <skos:broader rdf:resource="thsys/70947"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13488-0">
    <skos:broader rdf:resource="descriptor/11187-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18774-2">
    <skos:altLabel xml:lang="en">R&amp;amp;D subsidies</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18774-2">
    <skos:altLabel xml:lang="de">Forschungsfinanzhilfe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18774-2">
    <skos:altLabel xml:lang="de">Finanzhilfe für die Forschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14149-3">
    <skos:narrower rdf:resource="descriptor/18189-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14149-3">
    <skos:narrower rdf:resource="descriptor/14150-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13504-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18315-6">
    <skos:related rdf:resource="descriptor/18195-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18315-6">
    <skos:related rdf:resource="descriptor/15895-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18315-6">
    <skos:related rdf:resource="descriptor/18326-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19272-0">
    <skos:broader rdf:resource="thsys/70187"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19272-0">
    <skos:broader rdf:resource="thsys/70058"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19272-0">
    <skos:broader rdf:resource="descriptor/13723-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13653-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10470-6">
    <skos:altLabel xml:lang="de">Technik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17784-4">
    <skos:prefLabel xml:lang="en">Honduras</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17784-4">
    <skos:prefLabel xml:lang="de">Honduras</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18553-4">
    <skos:altLabel xml:lang="de">Warengruppenmanagement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18553-4">
    <skos:altLabel xml:lang="de">Regaloptimierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14144-6">
    <skos:prefLabel xml:lang="en">Pig</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14144-6">
    <skos:prefLabel xml:lang="de">Schwein</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11862-5">
    <skos:altLabel xml:lang="de">Ländliche Gemeinde</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11862-5">
    <skos:altLabel xml:lang="de">Landgemeinde</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11862-5">
    <skos:altLabel xml:lang="de">Dorfgemeinschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10894-4">
    <skos:broader rdf:resource="thsys/71071"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10894-4">
    <skos:broader rdf:resource="descriptor/10963-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10902-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19635-4">
    <skos:prefLabel xml:lang="en">Historiography</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19635-4">
    <skos:prefLabel xml:lang="de">Geschichtswissenschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17471-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16198-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73348">
    <skos:prefLabel xml:lang="de">N.10.04  Biowissenschaften</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16405-1">
    <skos:altLabel xml:lang="de">Politisch-ideologische Strömung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16405-1">
    <skos:altLabel xml:lang="de">Politische Strömung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16405-1">
    <skos:altLabel xml:lang="de">Politische Richtung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16405-1">
    <skos:altLabel xml:lang="de">Politische Ideologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16405-1">
    <skos:altLabel xml:lang="de">Politische Idee</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10144-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73369">
    <rdfs:label xml:lang="de">Kultur-, Bildungs- und Wissenschaftspolitik</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11111-1">
    <skos:related rdf:resource="descriptor/19734-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11111-1">
    <skos:related rdf:resource="descriptor/10217-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11111-1">
    <skos:related rdf:resource="descriptor/10161-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11111-1">
    <skos:related rdf:resource="descriptor/16709-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19838-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18244-3">
    <skos:prefLabel xml:lang="en">Spinning machinery</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18244-3">
    <skos:prefLabel xml:lang="de">Spinnmaschine</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10667-1">
    <skos:prefLabel xml:lang="en">International transmission of inflation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10667-1">
    <skos:prefLabel xml:lang="de">Inflationsübertragung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19313-5">
    <skos:altLabel xml:lang="de">Spazierstockeffekt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19313-5">
    <skos:altLabel xml:lang="de">J-Kurven-Effekt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13428-4">
    <skos:broader rdf:resource="thsys/71020"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13428-4">
    <skos:broader rdf:resource="thsys/70986"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17413-4">
    <skos:altLabel xml:lang="x-other">Eire</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17413-4">
    <skos:altLabel xml:lang="en">Irish Republic</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17413-4">
    <skos:altLabel xml:lang="de">Republik Irland</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18211-4">
    <skos:related rdf:resource="descriptor/18450-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18211-4">
    <skos:related rdf:resource="descriptor/18449-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18211-4">
    <skos:related rdf:resource="descriptor/16010-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18211-4">
    <skos:related rdf:resource="descriptor/18768-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18211-4">
    <skos:related rdf:resource="descriptor/13662-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13794-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15396-2">
    <skos:related rdf:resource="descriptor/19784-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15396-2">
    <skos:related rdf:resource="descriptor/15225-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15396-2">
    <skos:related rdf:resource="descriptor/15340-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15396-2">
    <skos:related rdf:resource="descriptor/15072-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15396-2">
    <skos:related rdf:resource="descriptor/20506-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15396-2">
    <skos:related rdf:resource="descriptor/10045-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15396-2">
    <skos:related rdf:resource="descriptor/18076-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17983-5">
    <skos:altLabel xml:lang="de">EU-Länder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17983-5">
    <skos:altLabel xml:lang="de">EG-Staaten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17983-5">
    <skos:altLabel xml:lang="de">EG-Länder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16066-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19169-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19322-4">
    <skos:broader rdf:resource="thsys/71098"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19322-4">
    <skos:broader rdf:resource="descriptor/10686-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70876">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70873">
    <skos:narrower rdf:resource="descriptor/18564-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70873">
    <skos:narrower rdf:resource="descriptor/18797-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70873">
    <skos:narrower rdf:resource="descriptor/18799-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70873">
    <skos:narrower rdf:resource="descriptor/18798-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70873">
    <skos:narrower rdf:resource="descriptor/18795-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70873">
    <skos:narrower rdf:resource="descriptor/12783-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70873">
    <skos:narrower rdf:resource="descriptor/12779-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70873">
    <skos:narrower rdf:resource="descriptor/18794-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16962-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10505-6">
    <skos:scopeNote xml:lang="de">Ausbau und geförderte Weiterentwicklung der Landwirtschaft, vor allem in Entwicklungsländern.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11765-3">
    <skos:related rdf:resource="descriptor/10110-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11765-3">
    <skos:related rdf:resource="descriptor/11751-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11765-3">
    <skos:related rdf:resource="descriptor/11754-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11765-3">
    <skos:related rdf:resource="descriptor/10093-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11765-3">
    <skos:related rdf:resource="descriptor/11092-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11765-3">
    <skos:related rdf:resource="descriptor/19218-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11765-3">
    <skos:related rdf:resource="descriptor/11336-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11765-3">
    <skos:related rdf:resource="descriptor/10126-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11765-3">
    <skos:related rdf:resource="descriptor/11735-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11765-3">
    <skos:related rdf:resource="descriptor/11071-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17738-4">
    <skos:altLabel xml:lang="de">Republik Ruanda</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12385-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18992-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18654-5">
    <skos:broader rdf:resource="thsys/70922"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18654-5">
    <skos:broader rdf:resource="descriptor/12780-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19834-5">
    <skos:related rdf:resource="descriptor/10192-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19834-5">
    <skos:related rdf:resource="descriptor/12053-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16658-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12480-0">
    <skos:related rdf:resource="descriptor/12742-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12480-0">
    <skos:related rdf:resource="descriptor/19096-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12480-0">
    <skos:related rdf:resource="descriptor/10011-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10769-0">
    <skos:related rdf:resource="descriptor/10771-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10769-0">
    <skos:related rdf:resource="descriptor/10764-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18362-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18579-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19291-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19388-2">
    <skos:broader rdf:resource="thsys/70992"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19388-2">
    <skos:broader rdf:resource="thsys/70957"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19388-2">
    <skos:broader rdf:resource="descriptor/10010-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15746-1">
    <skos:altLabel xml:lang="de">Naturschutzrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15746-1">
    <skos:altLabel xml:lang="de">Naturschutzgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15746-1">
    <skos:altLabel xml:lang="de">Bundesnaturschutzgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15746-1">
    <skos:altLabel xml:lang="de">Biotopschutz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10035-3">
    <skos:broader rdf:resource="thsys/70049"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10035-3">
    <skos:broader rdf:resource="descriptor/10065-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18019-3">
    <skos:altLabel xml:lang="en">Pacific basin</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18019-3">
    <skos:altLabel xml:lang="de">Pazifisches Becken</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18019-3">
    <skos:altLabel xml:lang="de">Pazifische Staaten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18019-3">
    <skos:altLabel xml:lang="de">Pazifische Länder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18019-3">
    <skos:altLabel xml:lang="de">Pazifikstaaten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18019-3">
    <skos:altLabel xml:lang="de">Pazifikregion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10559-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11060-6">
    <skos:broader rdf:resource="thsys/73396"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11060-6">
    <skos:broader rdf:resource="thsys/73382"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11060-6">
    <skos:broader rdf:resource="thsys/71101"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11060-6">
    <skos:broader rdf:resource="descriptor/16405-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11060-6">
    <skos:broader rdf:resource="descriptor/16602-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14028-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10375-0">
    <skos:broader rdf:resource="thsys/70020"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10375-0">
    <skos:broader rdf:resource="descriptor/10361-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19848-1">
    <skos:altLabel xml:lang="de">Unternehmensgeheimnis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19848-1">
    <skos:altLabel xml:lang="de">Betriebsgeheimnis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19848-1">
    <skos:altLabel xml:lang="de">Betriebliches Geheimnis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18292-6">
    <skos:broader rdf:resource="thsys/70201"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18292-6">
    <skos:broader rdf:resource="thsys/70143"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73340">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13365-2">
    <skos:altLabel xml:lang="de">Taschenbuchverlag</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19496-6">
    <skos:prefLabel xml:lang="en">Multiple job holding</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19496-6">
    <skos:prefLabel xml:lang="de">Nebentätigkeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19330-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16124-2">
    <skos:scopeNote xml:lang="de">Allgemeine Literatur über diverse Bestimmungen zum Schutz von Arbeitskräften.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18241-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16543-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16549-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19753-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10088-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19329-4">
    <skos:broader rdf:resource="thsys/73333"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19329-4">
    <skos:broader rdf:resource="thsys/71097"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19329-4">
    <skos:broader rdf:resource="descriptor/16032-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19329-4">
    <skos:broader rdf:resource="descriptor/13464-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17595-0">
    <skos:altLabel xml:lang="de">Zentralasiatische Staaten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17595-0">
    <skos:altLabel xml:lang="de">Zentralasiatische Länder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17595-0">
    <skos:altLabel xml:lang="de">Innerasien</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10166-2">
    <skos:prefLabel xml:lang="en">Utilitarianism</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10166-2">
    <skos:prefLabel xml:lang="de">Utilitarismus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18756-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11337-3">
    <skos:altLabel xml:lang="en">Collective bargaining theory</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11337-3">
    <skos:altLabel xml:lang="de">Tarifvertragstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11337-3">
    <skos:altLabel xml:lang="de">Kollektive Verhandlungstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11337-3">
    <skos:altLabel xml:lang="de">Kollektive Lohnverhandlungstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11337-3">
    <skos:altLabel xml:lang="de">Hicks-Lohntheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11337-3">
    <skos:altLabel xml:lang="de">Fluktuationskostenansatz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11337-3">
    <skos:altLabel xml:lang="de">Bargaining-Theorie des Lohnes</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11127-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13817-0">
    <skos:altLabel xml:lang="de">Montageversicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13817-0">
    <skos:altLabel xml:lang="de">Maschinenversicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13817-0">
    <skos:altLabel xml:lang="de">Elektronikversicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13817-0">
    <skos:altLabel xml:lang="de">EDV-Versicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13817-0">
    <skos:altLabel xml:lang="de">Computerversicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18816-5">
    <skos:prefLabel xml:lang="en">Military base</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18816-5">
    <skos:prefLabel xml:lang="de">Militärstandort</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12729-2">
    <skos:prefLabel xml:lang="en">Shopwork</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12729-2">
    <skos:prefLabel xml:lang="de">Werkstattfertigung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10515-3">
    <skos:broader rdf:resource="thsys/73383"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10515-3">
    <skos:broader rdf:resource="thsys/71086"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10515-3">
    <skos:broader rdf:resource="descriptor/10525-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18224-2">
    <skos:broader rdf:resource="thsys/70073"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18224-2">
    <skos:broader rdf:resource="descriptor/19085-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18224-2">
    <skos:broader rdf:resource="descriptor/19084-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18224-2">
    <skos:broader rdf:resource="descriptor/17983-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16066-4">
    <skos:prefLabel xml:lang="en">Lomé Agreement</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16066-4">
    <skos:prefLabel xml:lang="de">Cotonou-Abkommen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13776-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19148-1">
    <skos:altLabel xml:lang="en">Ratio delay study</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16079-2">
    <skos:related rdf:resource="descriptor/13767-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19694-2">
    <skos:altLabel xml:lang="de">Parteikoalition</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19694-2">
    <skos:altLabel xml:lang="de">Parteienkoalition</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19694-2">
    <skos:altLabel xml:lang="de">Koalitionsregierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19852-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10656-6">
    <skos:altLabel xml:lang="en">Trade in manufactures</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10656-6">
    <skos:altLabel xml:lang="en">Manufactured exports</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10656-6">
    <skos:altLabel xml:lang="en">Inter-industry trade</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10656-6">
    <skos:altLabel xml:lang="en">Industrial exports</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10656-6">
    <skos:altLabel xml:lang="de">Interindustrieller Außenhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10656-6">
    <skos:altLabel xml:lang="de">Industrieaußenhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10656-6">
    <skos:altLabel xml:lang="de">Außenhandel mit Industrieprodukten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14819-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10332-4">
    <skos:altLabel xml:lang="en">Increasing returns to scale</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10332-4">
    <skos:altLabel xml:lang="en">Economies of scale</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10332-4">
    <skos:altLabel xml:lang="en">Diseconomies of scale</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10332-4">
    <skos:altLabel xml:lang="en">Diminishing returns</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10332-4">
    <skos:altLabel xml:lang="de">Variabler Skalenertrag</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10332-4">
    <skos:altLabel xml:lang="de">Skalenelastizität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10332-4">
    <skos:altLabel xml:lang="de">Skaleneffekt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10332-4">
    <skos:altLabel xml:lang="de">Größenvorteil</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10332-4">
    <skos:altLabel xml:lang="de">Größenkostenersparnis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12988-6">
    <skos:scopeNote xml:lang="de">Nicht zu verwechseln mit Nichtmetall-Bergbau</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12249-2">
    <skos:related rdf:resource="descriptor/12244-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12249-2">
    <skos:related rdf:resource="descriptor/12271-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19160-4">
    <skos:altLabel xml:lang="en">Ecoaudit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19160-4">
    <skos:altLabel xml:lang="de">Umweltschutz-Audit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19160-4">
    <skos:altLabel xml:lang="de">Umweltprüfung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19160-4">
    <skos:altLabel xml:lang="de">Umwelt-Auditing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19160-4">
    <skos:altLabel xml:lang="de">Umwelt-Audit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11494-1">
    <skos:altLabel xml:lang="de">Preisniveaustabilisierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11494-1">
    <skos:altLabel xml:lang="de">Geldwertstabilisierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11494-1">
    <skos:altLabel xml:lang="de">Disinflation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11494-1">
    <skos:altLabel xml:lang="de">Antiinflationspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19604-1">
    <skos:related rdf:resource="descriptor/15107-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19604-1">
    <skos:related rdf:resource="descriptor/19579-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19604-1">
    <skos:related rdf:resource="descriptor/13465-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19604-1">
    <skos:related rdf:resource="descriptor/18013-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19604-1">
    <skos:related rdf:resource="descriptor/13689-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13298-5">
    <skos:prefLabel xml:lang="en">Marine insurance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13298-5">
    <skos:prefLabel xml:lang="de">Schiffsversicherung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16834-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70142">
    <skos:prefLabel xml:lang="de">N.03  Demographie und Bevölkerung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20402-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19249-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17705-5">
    <skos:altLabel xml:lang="de">Revolutionäre Volksrepublik Guinea</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17705-5">
    <skos:altLabel xml:lang="de">Republik Guinea</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16292-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10682-5">
    <skos:broader rdf:resource="thsys/73366"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10682-5">
    <skos:broader rdf:resource="thsys/71098"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10682-5">
    <skos:broader rdf:resource="descriptor/10987-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10656-6">
    <skos:broader rdf:resource="thsys/71091"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10656-6">
    <skos:broader rdf:resource="thsys/71005"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10656-6">
    <skos:broader rdf:resource="descriptor/10605-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11866-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12312-0">
    <skos:narrower rdf:resource="descriptor/12488-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12312-0">
    <skos:narrower rdf:resource="descriptor/12371-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12312-0">
    <skos:narrower rdf:resource="descriptor/12485-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17593-4">
    <skos:altLabel xml:lang="de">Mongolische Volksrepublik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19811-3">
    <skos:related rdf:resource="descriptor/15658-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19811-3">
    <skos:related rdf:resource="descriptor/15633-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12009-1">
    <skos:narrower rdf:resource="descriptor/19752-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14106-0">
    <skos:broader rdf:resource="thsys/70169"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14106-0">
    <skos:broader rdf:resource="descriptor/14105-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15733-3">
    <skos:broader rdf:resource="thsys/73351"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15733-3">
    <skos:broader rdf:resource="thsys/71011"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15733-3">
    <skos:broader rdf:resource="thsys/70817"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15733-3">
    <skos:broader rdf:resource="thsys/70238"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15733-3">
    <skos:broader rdf:resource="descriptor/15763-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11996-5">
    <skos:altLabel xml:lang="en">Intrametropolitan migration</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11996-5">
    <skos:altLabel xml:lang="de">Stadt-Peripherie-Wanderung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11996-5">
    <skos:altLabel xml:lang="de">Innerstädtische Wanderung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11996-5">
    <skos:altLabel xml:lang="de">Innerstädtische Mobilität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18536-4">
    <skos:prefLabel xml:lang="en">EU internal trade</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18536-4">
    <skos:prefLabel xml:lang="de">EU-Binnenhandel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16504-6">
    <skos:related rdf:resource="descriptor/15979-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16504-6">
    <skos:related rdf:resource="descriptor/19816-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16504-6">
    <skos:related rdf:resource="descriptor/16198-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16504-6">
    <skos:related rdf:resource="descriptor/19813-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16504-6">
    <skos:related rdf:resource="descriptor/15970-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16504-6">
    <skos:related rdf:resource="descriptor/15963-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12711-0">
    <skos:prefLabel xml:lang="en">Manufacturing technology</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12711-0">
    <skos:prefLabel xml:lang="de">Fertigungstechnik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11088-5">
    <skos:broader rdf:resource="thsys/73394"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11088-5">
    <skos:broader rdf:resource="thsys/73383"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11088-5">
    <skos:broader rdf:resource="thsys/71101"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11088-5">
    <skos:broader rdf:resource="descriptor/16405-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18257-1">
    <skos:altLabel xml:lang="de">Synthetischer Latex</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18257-1">
    <skos:altLabel xml:lang="de">Synthetischer Kautschuk</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18257-1">
    <skos:altLabel xml:lang="de">Chlorkautschuk</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id5">
    <rdfs:seeAlso rdf:resource="descriptor/19692-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19854-6">
    <skos:scopeNote xml:lang="de">Alle Regelungen, die beim Grenzübertritt von einem Staat in einen anderen gelten.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19578-4">
    <skos:prefLabel xml:lang="en">Health statistics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19578-4">
    <skos:prefLabel xml:lang="de">Gesundheitsstatistik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18751-0">
    <skos:prefLabel xml:lang="en">Company housing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18751-0">
    <skos:prefLabel xml:lang="de">Werkswohnung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12259-6">
    <skos:broader rdf:resource="thsys/71046"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12259-6">
    <skos:broader rdf:resource="descriptor/12276-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12259-6">
    <skos:broader rdf:resource="descriptor/25742-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12564-1">
    <skos:prefLabel xml:lang="en">Human resource information system</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12564-1">
    <skos:prefLabel xml:lang="de">Personal-Informationssystem</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19297-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10548-2">
    <skos:prefLabel xml:lang="en">Feasibility study</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10548-2">
    <skos:prefLabel xml:lang="de">Durchführbarkeitsstudie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10401-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18744-4">
    <skos:prefLabel xml:lang="en">Insurance marketing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18744-4">
    <skos:prefLabel xml:lang="de">Versicherungsmarketing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19579-2">
    <skos:prefLabel xml:lang="en">Craft statistics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19579-2">
    <skos:prefLabel xml:lang="de">Handwerksstatistik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19035-0">
    <skos:prefLabel xml:lang="en">Planning</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19035-0">
    <skos:prefLabel xml:lang="de">Planung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10443-2">
    <skos:related rdf:resource="descriptor/12702-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10443-2">
    <skos:related rdf:resource="descriptor/19548-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10443-2">
    <skos:related rdf:resource="descriptor/10459-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10443-2">
    <skos:related rdf:resource="descriptor/12711-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18391-4">
    <skos:related rdf:resource="descriptor/14298-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17503-3">
    <skos:scopeNote xml:lang="de">Erklärte sich 1991 unabhängig von der Sowjetunion und trat der GUS bei</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id357">
    <rdf:value xml:lang="de">Für die reine Umstellung/Neueinführung einer Währung innerhalb eines Landes oder eines Währungsgebietes</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id342">
    <rdfs:seeAlso rdf:resource="descriptor/19141-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16472-0">
    <skos:altLabel xml:lang="de">Militärische Besetzung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16472-0">
    <skos:altLabel xml:lang="de">Invasion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16472-0">
    <skos:altLabel xml:lang="de">Besatzungsstatus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16472-0">
    <skos:altLabel xml:lang="de">Besatzungspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16472-0">
    <skos:altLabel xml:lang="de">Annexion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71056">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.13.06.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18753-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71045">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10198-3">
    <skos:broader rdf:resource="thsys/70178"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10198-3">
    <skos:broader rdf:resource="descriptor/10217-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18780-0">
    <skos:broader rdf:resource="thsys/70878"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18780-0">
    <skos:broader rdf:resource="thsys/70175"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18780-0">
    <skos:broader rdf:resource="descriptor/13343-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18571-2">
    <skos:prefLabel xml:lang="en">Railway financing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18571-2">
    <skos:prefLabel xml:lang="de">Eisenbahnfinanzierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11352-0">
    <skos:broader rdf:resource="thsys/70845"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11352-0">
    <skos:broader rdf:resource="thsys/70176"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11352-0">
    <skos:broader rdf:resource="thsys/70143"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11352-0">
    <skos:broader rdf:resource="descriptor/11353-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18393-0">
    <skos:prefLabel xml:lang="en">Mineral oil production technology</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18393-0">
    <skos:prefLabel xml:lang="de">Erdölfördertechnik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17565-2">
    <skos:altLabel xml:lang="en">People's Republic of China</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17565-2">
    <skos:altLabel xml:lang="de">Volksrepublik China</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17565-2">
    <skos:altLabel xml:lang="de">Chinesische Volksrepublik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17565-2">
    <skos:altLabel xml:lang="de">China (Volksrepublik)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18885-0">
    <skos:prefLabel xml:lang="en">In-house training</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18885-0">
    <skos:prefLabel xml:lang="de">Betriebliche Weiterbildung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10930-5">
    <skos:broader rdf:resource="thsys/70212"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10930-5">
    <skos:broader rdf:resource="descriptor/10942-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10930-5">
    <skos:broader rdf:resource="descriptor/12794-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10635-0">
    <skos:narrower rdf:resource="descriptor/10636-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10635-0">
    <skos:narrower rdf:resource="descriptor/10631-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13029-4">
    <skos:broader rdf:resource="thsys/71001"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13029-4">
    <skos:broader rdf:resource="descriptor/13034-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70341">
    <skos:broader rdf:resource="thsys/70071"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10150-3">
    <skos:related rdf:resource="descriptor/19221-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10150-3">
    <skos:related rdf:resource="descriptor/19599-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10150-3">
    <skos:related rdf:resource="descriptor/18832-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10150-3">
    <skos:related rdf:resource="descriptor/16401-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15088-6">
    <skos:related rdf:resource="descriptor/15069-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15088-6">
    <skos:related rdf:resource="descriptor/11032-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16828-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18486-0">
    <skos:related rdf:resource="descriptor/18457-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18486-0">
    <skos:related rdf:resource="descriptor/18484-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18486-0">
    <skos:related rdf:resource="descriptor/11198-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18490-2">
    <skos:related rdf:resource="descriptor/18480-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18490-2">
    <skos:related rdf:resource="descriptor/19451-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18490-2">
    <skos:related rdf:resource="descriptor/11906-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18490-2">
    <skos:related rdf:resource="descriptor/11940-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18490-2">
    <skos:related rdf:resource="descriptor/11905-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18499-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10595-0">
    <skos:prefLabel xml:lang="en">Autonomy-compatible help</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10595-0">
    <skos:prefLabel xml:lang="de">Selbsthilfeförderung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19456-4">
    <skos:related rdf:resource="descriptor/11990-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19456-4">
    <skos:related rdf:resource="descriptor/18753-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19456-4">
    <skos:related rdf:resource="descriptor/11994-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19456-4">
    <skos:related rdf:resource="descriptor/19452-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19456-4">
    <skos:related rdf:resource="descriptor/19453-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18232-3">
    <skos:broader rdf:resource="thsys/70648"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18232-3">
    <skos:broader rdf:resource="descriptor/14188-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19175-5">
    <skos:scopeNote xml:lang="de">Mitgliedsländer des Common Market for Eastern and Southern Africa</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19072-1">
    <skos:scopeNote xml:lang="de">für statistische Übersichten BENUTZE zusätzlich die Publikationsart Statistik.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16016-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18627-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14305-1">
    <skos:prefLabel xml:lang="en">Nickel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14305-1">
    <skos:prefLabel xml:lang="de">Nickel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15408-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19345-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24858-3">
    <skos:broader rdf:resource="thsys/70046"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19737-3">
    <skos:prefLabel xml:lang="en">Holy order</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19737-3">
    <skos:prefLabel xml:lang="de">Religiöser Orden</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10693-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12212-4">
    <skos:narrower rdf:resource="descriptor/18683-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12212-4">
    <skos:narrower rdf:resource="descriptor/10210-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12212-4">
    <skos:narrower rdf:resource="descriptor/12210-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16007-6">
    <skos:narrower rdf:resource="descriptor/11417-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16007-6">
    <skos:narrower rdf:resource="descriptor/16019-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16007-6">
    <skos:narrower rdf:resource="descriptor/13569-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16007-6">
    <skos:narrower rdf:resource="descriptor/11580-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16007-6">
    <skos:narrower rdf:resource="descriptor/15981-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16007-6">
    <skos:narrower rdf:resource="descriptor/16198-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16007-6">
    <skos:narrower rdf:resource="descriptor/16017-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16007-6">
    <skos:narrower rdf:resource="descriptor/16159-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16007-6">
    <skos:narrower rdf:resource="descriptor/19323-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16007-6">
    <skos:narrower rdf:resource="descriptor/18809-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16007-6">
    <skos:narrower rdf:resource="descriptor/16003-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16007-6">
    <skos:narrower rdf:resource="descriptor/16002-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16007-6">
    <skos:narrower rdf:resource="descriptor/15961-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16007-6">
    <skos:narrower rdf:resource="descriptor/18311-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16007-6">
    <skos:narrower rdf:resource="descriptor/11525-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16007-6">
    <skos:narrower rdf:resource="descriptor/15993-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16007-6">
    <skos:narrower rdf:resource="descriptor/18352-0"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id57">
    <rdfs:seeAlso rdf:resource="descriptor/18682-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13046-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14305-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14849-3">
    <skos:altLabel xml:lang="de">Textilfaser</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14849-3">
    <skos:altLabel xml:lang="de">Spinnfaser</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18143-2">
    <skos:narrower rdf:resource="descriptor/12087-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18623-2">
    <skos:narrower rdf:resource="descriptor/15777-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18623-2">
    <skos:narrower rdf:resource="descriptor/18640-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15672-4">
    <skos:narrower rdf:resource="descriptor/20491-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13671-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70071">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10769-0">
    <skos:altLabel xml:lang="en">Currency block</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10769-0">
    <skos:altLabel xml:lang="de">Währungsgebiet</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10769-0">
    <skos:altLabel xml:lang="de">Währungsblock</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14190-6">
    <skos:prefLabel xml:lang="en">Bauxite</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14190-6">
    <skos:prefLabel xml:lang="de">Bauxit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18638-3">
    <skos:broader rdf:resource="thsys/71064"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18638-3">
    <skos:broader rdf:resource="thsys/70924"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18638-3">
    <skos:broader rdf:resource="descriptor/13644-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19624-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70979">
    <skos:prefLabel xml:lang="en">W.06.02  Industrial policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70979">
    <skos:prefLabel xml:lang="de">W.06.02  Industriepolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13440-0">
    <skos:broader rdf:resource="thsys/71024"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13440-0">
    <skos:broader rdf:resource="thsys/70814"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13440-0">
    <skos:broader rdf:resource="descriptor/18313-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13440-0">
    <skos:broader rdf:resource="descriptor/12865-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17779-4">
    <skos:broader rdf:resource="thsys/70086"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17779-4">
    <skos:broader rdf:resource="descriptor/17791-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18803-0">
    <skos:related rdf:resource="descriptor/13362-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18803-0">
    <skos:related rdf:resource="descriptor/18906-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18803-0">
    <skos:related rdf:resource="descriptor/11628-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18803-0">
    <skos:related rdf:resource="descriptor/13413-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18803-0">
    <skos:related rdf:resource="descriptor/19398-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18803-0">
    <skos:related rdf:resource="descriptor/13303-3"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id369">
    <rdfs:seeAlso rdf:resource="descriptor/18181-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11940-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14153-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10156-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16395-6">
    <skos:scopeNote xml:lang="de">Für politische Gewaltakte von Gruppen oder Einzelnen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12027-6">
    <skos:related rdf:resource="descriptor/18746-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71102">
    <skos:prefLabel xml:lang="en">V.06.01.01  Market economy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71102">
    <skos:prefLabel xml:lang="de">V.06.01.01  Marktwirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18679-3">
    <skos:prefLabel xml:lang="en">Volatility</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18679-3">
    <skos:prefLabel xml:lang="de">Volatilität</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13177-3">
    <skos:broader rdf:resource="thsys/70969"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13177-3">
    <skos:broader rdf:resource="descriptor/12905-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13570-1">
    <skos:prefLabel xml:lang="en">Traffic safety</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13570-1">
    <skos:prefLabel xml:lang="de">Verkehrssicherheit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11616-6">
    <skos:broader rdf:resource="thsys/71115"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11616-6">
    <skos:broader rdf:resource="thsys/71078"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11616-6">
    <skos:broader rdf:resource="descriptor/10024-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11616-6">
    <skos:broader rdf:resource="descriptor/11540-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10722-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19827-2">
    <skos:altLabel xml:lang="de">Funktionsfähigkeit der Märkte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19827-2">
    <skos:altLabel xml:lang="de">Effizienz des Marktes</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14934-5">
    <skos:altLabel xml:lang="de">Rübenzucker</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14934-5">
    <skos:altLabel xml:lang="de">Rohrzucker</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14934-5">
    <skos:altLabel xml:lang="de">Melasse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14934-5">
    <skos:altLabel xml:lang="de">Kandis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11495-6">
    <skos:broader rdf:resource="thsys/70355"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11495-6">
    <skos:broader rdf:resource="descriptor/10242-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11495-6">
    <skos:broader rdf:resource="descriptor/10038-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13809-6">
    <skos:altLabel xml:lang="de">Brandversicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18335-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14051-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18190-0">
    <skos:broader rdf:resource="thsys/71029"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18190-0">
    <skos:broader rdf:resource="descriptor/12933-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10552-4">
    <skos:related rdf:resource="descriptor/19399-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10552-4">
    <skos:related rdf:resource="descriptor/20553-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10552-4">
    <skos:related rdf:resource="descriptor/10568-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10552-4">
    <skos:related rdf:resource="descriptor/10558-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10552-4">
    <skos:related rdf:resource="descriptor/10555-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10552-4">
    <skos:related rdf:resource="descriptor/10550-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10552-4">
    <skos:related rdf:resource="descriptor/10797-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13329-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12628-1">
    <skos:altLabel xml:lang="de">Schichtdienst</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11533-3">
    <skos:broader rdf:resource="thsys/73387"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11533-3">
    <skos:broader rdf:resource="thsys/70144"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11533-3">
    <skos:broader rdf:resource="thsys/70109"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11533-3">
    <skos:broader rdf:resource="descriptor/15706-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11526-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15894-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16656-4">
    <skos:prefLabel xml:lang="en">Segregation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16656-4">
    <skos:prefLabel xml:lang="de">Segregation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id167">
    <rdf:value xml:lang="de">Für Versorgungsdienstleistungen der Kommunen</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12187-5">
    <skos:prefLabel xml:lang="en">Limited partnership</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12187-5">
    <skos:prefLabel xml:lang="de">Kommanditgesellschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14427-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16156-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14249-6">
    <skos:prefLabel xml:lang="en">Concrete</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14249-6">
    <skos:prefLabel xml:lang="de">Beton</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12842-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18949-0">
    <skos:narrower rdf:resource="descriptor/16794-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12249-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24624-0">
    <skos:broader rdf:resource="thsys/71034"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24624-0">
    <skos:broader rdf:resource="descriptor/12881-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18310-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17171-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12863-5">
    <skos:related rdf:resource="descriptor/11679-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12863-5">
    <skos:related rdf:resource="descriptor/18119-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12863-5">
    <skos:related rdf:resource="descriptor/12874-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12863-5">
    <skos:related rdf:resource="descriptor/10897-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12863-5">
    <skos:related rdf:resource="descriptor/19082-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12863-5">
    <skos:related rdf:resource="descriptor/10828-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70881">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70875">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10223-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12805-5">
    <skos:related rdf:resource="descriptor/14051-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12805-5">
    <skos:related rdf:resource="descriptor/12810-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12805-5">
    <skos:related rdf:resource="descriptor/18295-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12805-5">
    <skos:related rdf:resource="descriptor/18070-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12805-5">
    <skos:related rdf:resource="descriptor/12702-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12805-5">
    <skos:related rdf:resource="descriptor/18788-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12805-5">
    <skos:related rdf:resource="descriptor/19747-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13841-3">
    <skos:related rdf:resource="descriptor/13849-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13841-3">
    <skos:related rdf:resource="descriptor/13837-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13841-3">
    <skos:related rdf:resource="descriptor/13851-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15810-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17580-6">
    <skos:prefLabel xml:lang="en">Cambodia</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17580-6">
    <skos:prefLabel xml:lang="de">Kambodscha</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18984-5">
    <skos:related rdf:resource="descriptor/14211-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18984-5">
    <skos:related rdf:resource="descriptor/15883-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18984-5">
    <skos:related rdf:resource="descriptor/15868-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18984-5">
    <skos:related rdf:resource="descriptor/18985-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15621-0">
    <skos:prefLabel xml:lang="en">Software</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15621-0">
    <skos:prefLabel xml:lang="de">Software</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15611-3">
    <skos:prefLabel xml:lang="en">Artificial intelligence</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15611-3">
    <skos:prefLabel xml:lang="de">Künstliche Intelligenz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19778-3">
    <skos:broader rdf:resource="thsys/70580"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19850-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19758-2">
    <skos:altLabel xml:lang="en">Social prestige</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19758-2">
    <skos:altLabel xml:lang="de">Sozialprestige</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19758-2">
    <skos:altLabel xml:lang="de">Reputation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19758-2">
    <skos:altLabel xml:lang="de">Ansehen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19811-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id259">
    <rdf:value xml:lang="de">Für staatliche Preisfestsetzungen</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11350-4">
    <skos:related rdf:resource="descriptor/16484-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11350-4">
    <skos:related rdf:resource="descriptor/11349-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11350-4">
    <skos:related rdf:resource="descriptor/10401-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11350-4">
    <skos:related rdf:resource="descriptor/11346-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11350-4">
    <skos:related rdf:resource="descriptor/11345-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11350-4">
    <skos:related rdf:resource="descriptor/11314-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16374-0">
    <skos:prefLabel xml:lang="en">Socialist party</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16374-0">
    <skos:prefLabel xml:lang="de">Sozialistische Partei</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11688-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15709-0">
    <skos:narrower rdf:resource="descriptor/13867-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15709-0">
    <skos:narrower rdf:resource="descriptor/15958-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15709-0">
    <skos:narrower rdf:resource="descriptor/15954-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15709-0">
    <skos:narrower rdf:resource="descriptor/11888-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19535-1">
    <skos:prefLabel xml:lang="en">Share repurchase</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19535-1">
    <skos:prefLabel xml:lang="de">Aktienrückkauf</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12603-3">
    <skos:broader rdf:resource="thsys/71050"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19542-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13023-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11134-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15759-6">
    <skos:broader rdf:resource="thsys/73351"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15759-6">
    <skos:broader rdf:resource="thsys/70209"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11366-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19087-2">
    <skos:prefLabel xml:lang="en">Serbia-Montenegro</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19087-2">
    <skos:prefLabel xml:lang="de">Serbien-Montenegro</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11568-5">
    <skos:related rdf:resource="descriptor/11779-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11568-5">
    <skos:related rdf:resource="descriptor/19221-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11568-5">
    <skos:related rdf:resource="descriptor/19328-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11568-5">
    <skos:related rdf:resource="descriptor/11770-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13636-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12423-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70921">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19642-0">
    <skos:related rdf:resource="descriptor/15929-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19642-0">
    <skos:related rdf:resource="descriptor/16513-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19642-0">
    <skos:related rdf:resource="descriptor/15928-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19642-0">
    <skos:related rdf:resource="descriptor/15932-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19642-0">
    <skos:related rdf:resource="descriptor/15931-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11004-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18076-5">
    <skos:narrower rdf:resource="descriptor/19576-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12923-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17790-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12273-5">
    <skos:prefLabel xml:lang="en">Equity capital</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12273-5">
    <skos:prefLabel xml:lang="de">Eigenkapital</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11658-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70815">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.33</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10827-5">
    <skos:scopeNote xml:lang="de">Steuerrechtliche Vorschriften eines Staates für Steuerinländer/innen mit Auslandsbeziehungen und Steuerausländer/innen mit Inlandsbeziehungen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12368-1">
    <skos:prefLabel xml:lang="en">Accounts receivable</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12368-1">
    <skos:prefLabel xml:lang="de">Forderungen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19386-6">
    <skos:broader rdf:resource="thsys/71006"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19386-6">
    <skos:broader rdf:resource="descriptor/18388-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19386-6">
    <skos:broader rdf:resource="descriptor/11813-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17569-1">
    <skos:prefLabel xml:lang="en">North Korea</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17569-1">
    <skos:prefLabel xml:lang="de">Nordkorea</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11848-6">
    <skos:broader rdf:resource="thsys/73390"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11848-6">
    <skos:broader rdf:resource="thsys/70888"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11848-6">
    <skos:broader rdf:resource="descriptor/16081-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15494-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70998">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.06.01.03.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18361-6">
    <skos:related rdf:resource="descriptor/14483-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18361-6">
    <skos:related rdf:resource="descriptor/18354-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18361-6">
    <skos:related rdf:resource="descriptor/14174-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18361-6">
    <skos:related rdf:resource="descriptor/18359-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11067-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12983-2">
    <skos:altLabel xml:lang="de">Erdgasförderindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12983-2">
    <skos:altLabel xml:lang="de">Erdgasbohrung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14158-2">
    <skos:altLabel xml:lang="de">Tiefkühlfisch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14158-2">
    <skos:altLabel xml:lang="de">Frischfisch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70342">
    <skos:broader rdf:resource="thsys/70498"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11872-2">
    <skos:related rdf:resource="descriptor/19234-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11872-2">
    <skos:related rdf:resource="descriptor/19440-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11872-2">
    <skos:related rdf:resource="descriptor/11911-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14851-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18988-4">
    <skos:related rdf:resource="descriptor/18987-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18988-4">
    <skos:related rdf:resource="descriptor/18338-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12904-3">
    <skos:prefLabel xml:lang="en">Building materials industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12904-3">
    <skos:prefLabel xml:lang="de">Baustoffindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10641-5">
    <skos:related rdf:resource="descriptor/11932-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10641-5">
    <skos:related rdf:resource="descriptor/10842-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10641-5">
    <skos:related rdf:resource="descriptor/10614-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10641-5">
    <skos:related rdf:resource="descriptor/10637-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18834-3">
    <skos:related rdf:resource="descriptor/18823-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18834-3">
    <skos:related rdf:resource="descriptor/12047-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18834-3">
    <skos:related rdf:resource="descriptor/18833-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18834-3">
    <skos:related rdf:resource="descriptor/18717-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70556">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18014-6">
    <skos:prefLabel xml:lang="en">Land set-aside</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18014-6">
    <skos:prefLabel xml:lang="de">Flächenstilllegung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18138-2">
    <skos:broader rdf:resource="thsys/70897"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18138-2">
    <skos:broader rdf:resource="thsys/70049"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18138-2">
    <skos:broader rdf:resource="thsys/70038"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16354-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11991-1">
    <skos:altLabel xml:lang="en">Urban fringe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11991-1">
    <skos:altLabel xml:lang="de">Vorstadt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11991-1">
    <skos:altLabel xml:lang="de">Verstädterte Zone</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11991-1">
    <skos:altLabel xml:lang="de">Trabantenstadt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11991-1">
    <skos:altLabel xml:lang="de">Stadtrand</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11991-1">
    <skos:altLabel xml:lang="de">Satellitenstadt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11991-1">
    <skos:altLabel xml:lang="de">Peri-urbanes Gebiet</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11991-1">
    <skos:altLabel xml:lang="de">Großraum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18691-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20077-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19330-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12050-4">
    <skos:broader rdf:resource="thsys/70087"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12050-4">
    <skos:broader rdf:resource="descriptor/12056-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12171-6">
    <skos:related rdf:resource="descriptor/12165-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12171-6">
    <skos:related rdf:resource="descriptor/12164-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10711-3">
    <skos:related rdf:resource="descriptor/10567-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10711-3">
    <skos:related rdf:resource="descriptor/10847-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10711-3">
    <skos:related rdf:resource="descriptor/13509-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10711-3">
    <skos:related rdf:resource="descriptor/19308-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19185-2">
    <skos:related rdf:resource="descriptor/13033-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19185-2">
    <skos:related rdf:resource="descriptor/18392-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19185-2">
    <skos:related rdf:resource="descriptor/14299-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13756-1">
    <skos:altLabel xml:lang="en">Banking services</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13756-1">
    <skos:altLabel xml:lang="de">Bankprodukt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13756-1">
    <skos:altLabel xml:lang="de">Bankdienstleistung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16455-0">
    <skos:altLabel xml:lang="en">Peace keeping force</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16455-0">
    <skos:altLabel xml:lang="de">UN-Friedenstruppe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16455-0">
    <skos:altLabel xml:lang="de">Multinationale Streitkräfte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16455-0">
    <skos:altLabel xml:lang="de">Internationale Streitkräfte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16455-0">
    <skos:altLabel xml:lang="de">Friedenstruppe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16455-0">
    <skos:altLabel xml:lang="de">Friedenssicherungstruppe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19570-6">
    <skos:altLabel xml:lang="en">Structural change (Statistics)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20921-0">
    <skos:broader rdf:resource="thsys/73341"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20921-0">
    <skos:broader rdf:resource="thsys/70922"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20921-0">
    <skos:broader rdf:resource="thsys/70384"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20921-0">
    <skos:broader rdf:resource="thsys/70383"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20921-0">
    <skos:broader rdf:resource="descriptor/18659-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20921-0">
    <skos:broader rdf:resource="descriptor/19651-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18434-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15859-2">
    <skos:related rdf:resource="descriptor/12837-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15859-2">
    <skos:related rdf:resource="descriptor/12829-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15859-2">
    <skos:related rdf:resource="descriptor/12844-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15859-2">
    <skos:related rdf:resource="descriptor/10298-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15859-2">
    <skos:related rdf:resource="descriptor/15787-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15859-2">
    <skos:related rdf:resource="descriptor/10202-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15859-2">
    <skos:related rdf:resource="descriptor/19754-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15859-2">
    <skos:related rdf:resource="descriptor/10300-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18881-1">
    <skos:prefLabel xml:lang="en">Economic library</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18881-1">
    <skos:prefLabel xml:lang="de">Wirtschaftsbibliothek</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14611-1">
    <skos:broader rdf:resource="thsys/70074"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14611-1">
    <skos:broader rdf:resource="descriptor/14602-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11600-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18825-4">
    <skos:broader rdf:resource="thsys/70859"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18825-4">
    <skos:broader rdf:resource="descriptor/15598-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16741-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11064-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13045-6">
    <skos:broader rdf:resource="thsys/71002"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13045-6">
    <skos:broader rdf:resource="descriptor/13058-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19143-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13981-1">
    <skos:altLabel xml:lang="de">Postzustellerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13981-1">
    <skos:altLabel xml:lang="de">Postzusteller</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13981-1">
    <skos:altLabel xml:lang="de">Postverteilerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13981-1">
    <skos:altLabel xml:lang="de">Postverteiler</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13981-1">
    <skos:altLabel xml:lang="de">Postpersonal</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13981-1">
    <skos:altLabel xml:lang="de">Briefträgerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13981-1">
    <skos:altLabel xml:lang="de">Briefträger</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12565-6">
    <skos:narrower rdf:resource="descriptor/12563-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70536">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">P.12.05</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12712-5">
    <skos:prefLabel xml:lang="en">Production management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12712-5">
    <skos:prefLabel xml:lang="de">Produktionswirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19652-4">
    <skos:related rdf:resource="descriptor/13743-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19652-4">
    <skos:related rdf:resource="descriptor/16080-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19652-4">
    <skos:related rdf:resource="descriptor/13742-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19652-4">
    <skos:related rdf:resource="descriptor/12265-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71007">
    <rdfs:label xml:lang="en">Metal mining</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71007">
    <rdfs:label xml:lang="de">Erzbergbau</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19183-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70834">
    <rdfs:label xml:lang="de">Soziale Dienste</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10486-5">
    <skos:related rdf:resource="descriptor/15676-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10486-5">
    <skos:related rdf:resource="descriptor/18863-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10486-5">
    <skos:related rdf:resource="descriptor/11369-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10486-5">
    <skos:related rdf:resource="descriptor/11400-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19086-4">
    <skos:broader rdf:resource="thsys/70130"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19086-4">
    <skos:broader rdf:resource="descriptor/17179-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18901-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10062-0">
    <skos:prefLabel xml:lang="en">Economic theory</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10062-0">
    <skos:prefLabel xml:lang="de">Wirtschaftstheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14792-6">
    <skos:broader rdf:resource="thsys/70416"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11906-4">
    <skos:prefLabel xml:lang="en">Industrial geography</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11906-4">
    <skos:prefLabel xml:lang="de">Industriegeographie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14480-4">
    <skos:broader rdf:resource="thsys/70254"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14480-4">
    <skos:broader rdf:resource="descriptor/14472-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19614-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18151-3">
    <skos:broader rdf:resource="thsys/70180"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18151-3">
    <skos:broader rdf:resource="thsys/70049"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12718-0">
    <skos:altLabel xml:lang="de">Einzelproduktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12796-1">
    <skos:scopeNote xml:lang="de">Nur für innerbetriebliche Verrechnungspreise.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18429-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70361">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15346-3">
    <skos:prefLabel xml:lang="en">Tobit model</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15346-3">
    <skos:prefLabel xml:lang="de">Tobit-Modell</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13563-5">
    <skos:altLabel xml:lang="en">Road traffic</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10136-4">
    <skos:altLabel xml:lang="en">Term structure theory</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10136-4">
    <skos:altLabel xml:lang="en">Expectations hypothesis of the term structure</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10136-4">
    <skos:altLabel xml:lang="de">Erwartungshypothese der Zinsstruktur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11986-1">
    <skos:scopeNote xml:lang="de">Interdisziplinärer Forschungsbereich zur Analyse der Stadt und städtischer Phänomene.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18668-1">
    <skos:altLabel xml:lang="de">Kreditderivat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18668-1">
    <skos:altLabel xml:lang="de">Derivatives Finanzprodukt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18668-1">
    <skos:altLabel xml:lang="de">Derivatives Finanzinstrument</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13466-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12612-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18445-0">
    <skos:related rdf:resource="descriptor/13161-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18445-0">
    <skos:related rdf:resource="descriptor/15757-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18445-0">
    <skos:related rdf:resource="descriptor/19529-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18445-0">
    <skos:related rdf:resource="descriptor/19792-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18445-0">
    <skos:related rdf:resource="descriptor/14930-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13714-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18438-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15699-5">
    <skos:altLabel xml:lang="de">Urgeschichte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15699-5">
    <skos:altLabel xml:lang="de">Prähistorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15699-5">
    <skos:altLabel xml:lang="de">Frühgeschichte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19055-1">
    <skos:prefLabel xml:lang="en">Exploratory drilling</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19055-1">
    <skos:prefLabel xml:lang="de">Testbohrung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18584-0">
    <skos:broader rdf:resource="thsys/70939"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18584-0">
    <skos:broader rdf:resource="thsys/70937"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18584-0">
    <skos:broader rdf:resource="descriptor/13264-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18584-0">
    <skos:broader rdf:resource="descriptor/18578-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18584-0">
    <skos:broader rdf:resource="descriptor/13558-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17639-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14350-3">
    <skos:prefLabel xml:lang="en">Diesel engine</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14350-3">
    <skos:prefLabel xml:lang="de">Dieselmotor</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14255-4">
    <skos:altLabel xml:lang="de">Schmirgel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14255-4">
    <skos:altLabel xml:lang="de">Schleifstoff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14255-4">
    <skos:altLabel xml:lang="de">Polierstein</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10994-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71020">
    <skos:narrower rdf:resource="descriptor/19307-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71020">
    <skos:narrower rdf:resource="descriptor/19304-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71020">
    <skos:narrower rdf:resource="descriptor/19348-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71020">
    <skos:narrower rdf:resource="descriptor/19298-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71020">
    <skos:narrower rdf:resource="descriptor/13428-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71020">
    <skos:narrower rdf:resource="descriptor/19545-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71020">
    <skos:narrower rdf:resource="descriptor/19190-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71020">
    <skos:narrower rdf:resource="descriptor/18979-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71020">
    <skos:narrower rdf:resource="descriptor/18851-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71020">
    <skos:narrower rdf:resource="descriptor/19295-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71020">
    <skos:narrower rdf:resource="descriptor/10497-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71020">
    <skos:narrower rdf:resource="descriptor/10697-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71020">
    <skos:narrower rdf:resource="descriptor/19187-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71020">
    <skos:narrower rdf:resource="descriptor/11938-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71020">
    <skos:narrower rdf:resource="descriptor/18014-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71020">
    <skos:narrower rdf:resource="descriptor/19546-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71020">
    <skos:narrower rdf:resource="descriptor/18331-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71020">
    <skos:narrower rdf:resource="descriptor/18332-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71020">
    <skos:narrower rdf:resource="descriptor/11806-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71020">
    <skos:narrower rdf:resource="descriptor/18333-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71020">
    <skos:narrower rdf:resource="descriptor/19330-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71020">
    <skos:narrower rdf:resource="descriptor/11562-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71020">
    <skos:narrower rdf:resource="descriptor/11804-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71020">
    <skos:narrower rdf:resource="descriptor/11809-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71020">
    <skos:narrower rdf:resource="descriptor/18330-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71020">
    <skos:narrower rdf:resource="descriptor/11802-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71020">
    <skos:narrower rdf:resource="descriptor/11801-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71020">
    <skos:narrower rdf:resource="descriptor/11800-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11367-1">
    <skos:broader rdf:resource="thsys/70847"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11367-1">
    <skos:broader rdf:resource="thsys/70176"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11696-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18382-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18214-5">
    <skos:prefLabel xml:lang="en">Fast Food</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18214-5">
    <skos:prefLabel xml:lang="de">Fast Food</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19560-2">
    <skos:altLabel xml:lang="en">Self-regulation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19560-2">
    <skos:altLabel xml:lang="de">Selbstregulierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19560-2">
    <skos:altLabel xml:lang="de">Selbstregulation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19560-2">
    <skos:altLabel xml:lang="de">Freiwillige Selbstverpflichtung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13433-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14796-5">
    <skos:related rdf:resource="descriptor/14803-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14796-5">
    <skos:related rdf:resource="descriptor/13191-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18966-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17684-1">
    <skos:altLabel xml:lang="de">Südwestafrika</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17684-1">
    <skos:altLabel xml:lang="de">Republik Namibia</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18175-3">
    <skos:narrower rdf:resource="descriptor/18660-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18175-3">
    <skos:narrower rdf:resource="descriptor/19653-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18175-3">
    <skos:narrower rdf:resource="descriptor/18002-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16960-1">
    <skos:broader rdf:resource="thsys/70003"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16960-1">
    <skos:broader rdf:resource="descriptor/16972-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17993-2">
    <skos:prefLabel xml:lang="en">OECD countries</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17993-2">
    <skos:prefLabel xml:lang="de">OECD-Staaten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12760-1">
    <skos:narrower rdf:resource="descriptor/12643-5"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id129">
    <rdfs:seeAlso rdf:resource="descriptor/12357-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19662-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19564-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18753-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18603-1">
    <skos:prefLabel xml:lang="en">Port management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18603-1">
    <skos:prefLabel xml:lang="de">Hafenwirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18545-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10291-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19357-6">
    <skos:scopeNote xml:lang="de">Außenwirtschaftliche Beziehungen von Regionen bzw. Provinzen unterhalb der gesamtstaatlichen Ebene.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11421-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17972-3">
    <skos:prefLabel xml:lang="en">Black Sea</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17972-3">
    <skos:prefLabel xml:lang="de">Schwarzes Meer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id195">
    <rdf:value xml:lang="de">Für die Beziehungen zwischen Unternehmen und Endverbraucher-Kunden</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14839-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19405-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73297">
    <rdfs:label xml:lang="de">Büromaschinen- und Computerindustrie</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19704-4">
    <skos:scopeNote xml:lang="de">Politische Bestrebungen und Maßnahmen auf dem Gebiet des materiellen Strafrechts, des Strafprozessrechts, des Strafvollzugs und der Verbrechensverhütung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15766-2">
    <skos:related rdf:resource="descriptor/16201-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15766-2">
    <skos:related rdf:resource="descriptor/15769-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15766-2">
    <skos:related rdf:resource="descriptor/18186-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11603-1">
    <skos:prefLabel xml:lang="en">Local intergovernmental transfer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11603-1">
    <skos:prefLabel xml:lang="de">Kommunaler Finanzausgleich</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12783-3">
    <skos:broader rdf:resource="thsys/70873"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12783-3">
    <skos:broader rdf:resource="thsys/70459"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12783-3">
    <skos:broader rdf:resource="descriptor/13470-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18577-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18334-2">
    <skos:prefLabel xml:lang="en">Nutrition policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18334-2">
    <skos:prefLabel xml:lang="de">Ernährungspolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18496-4">
    <skos:prefLabel xml:lang="en">Building</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18496-4">
    <skos:prefLabel xml:lang="de">Gebäude</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11971-0">
    <skos:related rdf:resource="descriptor/11992-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11971-0">
    <skos:related rdf:resource="descriptor/11996-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11971-0">
    <skos:related rdf:resource="descriptor/19478-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11971-0">
    <skos:related rdf:resource="descriptor/19477-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11971-0">
    <skos:related rdf:resource="descriptor/13454-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14775-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20068-3">
    <skos:altLabel xml:lang="en">ISP (Internet Service Provider)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20068-3">
    <skos:altLabel xml:lang="en">Internet service provider</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20068-3">
    <skos:altLabel xml:lang="en">ASP (Application Service Provider)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20068-3">
    <skos:altLabel xml:lang="en">Application service provider</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20068-3">
    <skos:altLabel xml:lang="de">Diensteanbieter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16548-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13570-1">
    <skos:narrower rdf:resource="descriptor/18580-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13570-1">
    <skos:narrower rdf:resource="descriptor/18593-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13570-1">
    <skos:narrower rdf:resource="descriptor/18607-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16198-1">
    <skos:related rdf:resource="descriptor/15979-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16198-1">
    <skos:related rdf:resource="descriptor/19763-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16198-1">
    <skos:related rdf:resource="descriptor/16504-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16198-1">
    <skos:related rdf:resource="descriptor/19813-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16198-1">
    <skos:related rdf:resource="descriptor/16641-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16198-1">
    <skos:related rdf:resource="descriptor/15963-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16198-1">
    <skos:related rdf:resource="descriptor/16196-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16198-1">
    <skos:related rdf:resource="descriptor/19704-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16198-1">
    <skos:related rdf:resource="descriptor/16195-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13576-3">
    <skos:altLabel xml:lang="de">Schiffstechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13576-3">
    <skos:altLabel xml:lang="de">Schiffbautechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13576-3">
    <skos:altLabel xml:lang="de">Schiffahrtstechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13967-2">
    <skos:related rdf:resource="descriptor/13282-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13967-2">
    <skos:related rdf:resource="descriptor/12820-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13967-2">
    <skos:related rdf:resource="descriptor/13466-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10137-2">
    <skos:related rdf:resource="descriptor/11490-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10137-2">
    <skos:related rdf:resource="descriptor/10135-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10137-2">
    <skos:related rdf:resource="descriptor/10126-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10137-2">
    <skos:related rdf:resource="descriptor/10294-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12535-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14301-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19142-6">
    <skos:broader rdf:resource="thsys/71043"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17977-0">
    <skos:prefLabel xml:lang="en">ACP countries</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17977-0">
    <skos:prefLabel xml:lang="de">AKP-Staaten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14095-0">
    <skos:related rdf:resource="descriptor/19300-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14095-0">
    <skos:related rdf:resource="descriptor/19298-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14095-0">
    <skos:related rdf:resource="descriptor/19297-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14095-0">
    <skos:related rdf:resource="descriptor/18197-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10675-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13135-5">
    <skos:broader rdf:resource="thsys/70988"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13135-5">
    <skos:broader rdf:resource="descriptor/13136-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10005-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13993-1">
    <skos:scopeNote xml:lang="de">Nur für Verwaltungspersonal allgemein oder in der Privatwirtschaft.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18547-6">
    <skos:narrower rdf:resource="descriptor/18514-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11929-6">
    <skos:prefLabel xml:lang="en">Regional multiplier</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11929-6">
    <skos:prefLabel xml:lang="de">Regionaler Multiplikator</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13985-0">
    <skos:related rdf:resource="descriptor/18977-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13985-0">
    <skos:related rdf:resource="descriptor/12042-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19540-1">
    <skos:altLabel xml:lang="en">Soft budget constraint</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19540-1">
    <skos:altLabel xml:lang="de">Budgetgerade</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19540-1">
    <skos:altLabel xml:lang="de">Budgetbegrenzung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19540-1">
    <skos:altLabel xml:lang="de">Betriebliche Budgetbegrenzung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10205-4">
    <skos:prefLabel xml:lang="en">Monopoly</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10205-4">
    <skos:prefLabel xml:lang="de">Monopol</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14446-4">
    <skos:broader rdf:resource="thsys/70415"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14446-4">
    <skos:broader rdf:resource="descriptor/14463-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70174">
    <skos:narrower rdf:resource="descriptor/17465-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70174">
    <skos:narrower rdf:resource="descriptor/17485-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70174">
    <skos:narrower rdf:resource="descriptor/17461-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70174">
    <skos:narrower rdf:resource="descriptor/17534-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70174">
    <skos:narrower rdf:resource="descriptor/19077-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70174">
    <skos:narrower rdf:resource="thsys/70035"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70174">
    <skos:narrower rdf:resource="thsys/70073"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70174">
    <skos:narrower rdf:resource="thsys/70130"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12491-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18227-3">
    <skos:related rdf:resource="descriptor/17176-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70965">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.08</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19446-0">
    <skos:altLabel xml:lang="de">Wirtschaftsförderung der Kommunen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19446-0">
    <skos:altLabel xml:lang="de">Städtische Wirtschaftsförderung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19446-0">
    <skos:altLabel xml:lang="de">Kommunale Wirtschaftsförderung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19446-0">
    <skos:altLabel xml:lang="de">Gemeindewirtschaftspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10378-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13428-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19077-5">
    <skos:prefLabel xml:lang="en">Eastern Europe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19077-5">
    <skos:prefLabel xml:lang="de">Osteuropa</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17991-6">
    <skos:prefLabel xml:lang="en">NATO countries</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17991-6">
    <skos:prefLabel xml:lang="de">NATO-Staaten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19569-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18634-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15719-4">
    <skos:narrower rdf:resource="descriptor/11752-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15719-4">
    <skos:narrower rdf:resource="descriptor/11087-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15719-4">
    <skos:narrower rdf:resource="descriptor/15842-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15719-4">
    <skos:narrower rdf:resource="descriptor/11339-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70867">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73355">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11354-3">
    <skos:narrower rdf:resource="descriptor/10486-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20921-0">
    <skos:prefLabel xml:lang="en">Internet portal</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20921-0">
    <skos:prefLabel xml:lang="de">Internet-Portal</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13069-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id362">
    <rdf:value xml:lang="de">Für den Bau von Werkzeugmaschinen</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11864-1">
    <skos:prefLabel xml:lang="en">Flächenverkehr</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11864-1">
    <skos:prefLabel xml:lang="de">Flächenverkehr</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18314-1">
    <skos:broader rdf:resource="thsys/73356"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18314-1">
    <skos:broader rdf:resource="thsys/73347"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18314-1">
    <skos:broader rdf:resource="thsys/70091"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18314-1">
    <skos:broader rdf:resource="descriptor/18444-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10303-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18399-2">
    <skos:prefLabel xml:lang="en">Metal construction</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18399-2">
    <skos:prefLabel xml:lang="de">Metallbau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15700-4">
    <skos:scopeNote xml:lang="de">Allgemein der Zeitraum, der der Gegenwart unmittelbar vorausgeht</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15677-1">
    <skos:related rdf:resource="descriptor/12711-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13421-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11995-0">
    <skos:related rdf:resource="descriptor/19453-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11995-0">
    <skos:related rdf:resource="descriptor/19065-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11995-0">
    <skos:related rdf:resource="descriptor/12014-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70241">
    <rdfs:label xml:lang="en">Private law</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70241">
    <rdfs:label xml:lang="de">Privatrecht</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18864-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16081-1">
    <skos:related rdf:resource="descriptor/11069-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16081-1">
    <skos:related rdf:resource="descriptor/16202-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16081-1">
    <skos:related rdf:resource="descriptor/10964-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16081-1">
    <skos:related rdf:resource="descriptor/16080-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16081-1">
    <skos:related rdf:resource="descriptor/13825-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16081-1">
    <skos:related rdf:resource="descriptor/19213-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16081-1">
    <skos:related rdf:resource="descriptor/19323-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16081-1">
    <skos:related rdf:resource="descriptor/13464-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16081-1">
    <skos:related rdf:resource="descriptor/16053-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16081-1">
    <skos:related rdf:resource="descriptor/16050-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16081-1">
    <skos:related rdf:resource="descriptor/11525-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16081-1">
    <skos:related rdf:resource="descriptor/11185-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18876-1">
    <skos:altLabel xml:lang="de">HRG (Hochschulrahmengesetz)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18876-1">
    <skos:altLabel xml:lang="de">Hochschulverfassung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18876-1">
    <skos:altLabel xml:lang="de">Hochschulrahmengesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18876-1">
    <skos:altLabel xml:lang="de">Hochschulgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14844-6">
    <skos:narrower rdf:resource="descriptor/14837-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13034-4">
    <skos:altLabel xml:lang="de">Metallverarbeitende Industrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13034-4">
    <skos:altLabel xml:lang="de">Metallerzeugende Industrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14938-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17700-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12361-1">
    <skos:related rdf:resource="descriptor/12385-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12361-1">
    <skos:related rdf:resource="descriptor/19092-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12361-1">
    <skos:related rdf:resource="descriptor/12406-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11596-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12299-1">
    <skos:altLabel xml:lang="de">Effekten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18310-2">
    <skos:scopeNote xml:lang="de">Ökonomik der Branchen, Regionen oder Gruppen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19027-6">
    <skos:prefLabel xml:lang="en">Adjustment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19027-6">
    <skos:prefLabel xml:lang="de">Anpassung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19394-0">
    <skos:related rdf:resource="descriptor/18900-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19394-0">
    <skos:related rdf:resource="descriptor/18811-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19394-0">
    <skos:related rdf:resource="descriptor/18991-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19394-0">
    <skos:related rdf:resource="descriptor/19338-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19394-0">
    <skos:related rdf:resource="descriptor/14931-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19394-0">
    <skos:related rdf:resource="descriptor/16666-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12810-5">
    <skos:altLabel xml:lang="de">Variantenmanagement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12810-5">
    <skos:altLabel xml:lang="de">Produktvariation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12425-1">
    <skos:prefLabel xml:lang="en">Inflation accounting</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12425-1">
    <skos:prefLabel xml:lang="de">Inflationsbilanzierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11474-0">
    <skos:altLabel xml:lang="de">Real-Balance-Effekt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19099-2">
    <skos:scopeNote xml:lang="de">Kostenartenbezogene Aufzeichnung der Kosten.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14306-6">
    <skos:prefLabel xml:lang="en">Platinum</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14306-6">
    <skos:prefLabel xml:lang="de">Platinmetall</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17712-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10555-5">
    <skos:prefLabel xml:lang="en">Development aid</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10555-5">
    <skos:prefLabel xml:lang="de">Entwicklungshilfe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70921">
    <skos:prefLabel xml:lang="de">W.13.02  Rundfunk und Fernsehen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14796-5">
    <skos:prefLabel xml:lang="en">Waste paper</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14796-5">
    <skos:prefLabel xml:lang="de">Altpapier</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12275-1">
    <skos:narrower rdf:resource="descriptor/19201-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12275-1">
    <skos:narrower rdf:resource="descriptor/19095-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12275-1">
    <skos:narrower rdf:resource="descriptor/12270-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18652-2">
    <skos:related rdf:resource="descriptor/18653-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18652-2">
    <skos:related rdf:resource="descriptor/18649-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70095">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.04</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19193-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11033-2">
    <skos:broader rdf:resource="thsys/71101"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18079-6">
    <skos:altLabel xml:lang="de">Fluorchlorkohlenwasserstoffe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18079-6">
    <skos:altLabel xml:lang="de">Chlorfluorkohlenwasserstoffe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14804-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19131-4">
    <skos:prefLabel xml:lang="en">Deceptive advertising</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19131-4">
    <skos:prefLabel xml:lang="de">Irreführende Werbung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19086-4">
    <skos:related rdf:resource="descriptor/17176-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11201-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19826-4">
    <skos:prefLabel xml:lang="en">Investment climate</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19826-4">
    <skos:prefLabel xml:lang="de">Investitionsklima</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17610-2">
    <skos:broader rdf:resource="thsys/70561"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17610-2">
    <skos:broader rdf:resource="descriptor/19167-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16958-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19217-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71018">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15908-1">
    <skos:altLabel xml:lang="en">Handicapped people</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15908-1">
    <skos:altLabel xml:lang="en">Handicapped</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15908-1">
    <skos:altLabel xml:lang="de">Schwerbehinderte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15908-1">
    <skos:altLabel xml:lang="de">Psychisch Behinderte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15908-1">
    <skos:altLabel xml:lang="de">Körperbehinderte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12254-2">
    <skos:editorialNote xml:lang="de">Möglichst spezifischer indexieren.</skos:editorialNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17599-6">
    <skos:altLabel xml:lang="de">Burma</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17599-6">
    <skos:altLabel xml:lang="de">Birma</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13355-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10027-2">
    <skos:prefLabel xml:lang="en">Intermediate good</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10027-2">
    <skos:prefLabel xml:lang="de">Vorprodukt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16387-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14092-6">
    <skos:broader rdf:resource="thsys/70341"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14092-6">
    <skos:broader rdf:resource="descriptor/14074-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18712-3">
    <skos:prefLabel xml:lang="en">Electronic banking</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18712-3">
    <skos:prefLabel xml:lang="de">Electronic Banking</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13385-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70872">
    <skos:broader rdf:resource="thsys/w"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70337">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70580">
    <rdfs:label xml:lang="de">Sprache und Literatur</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19055-1">
    <skos:broader rdf:resource="thsys/70969"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19055-1">
    <skos:broader rdf:resource="descriptor/19054-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13270-6">
    <skos:scopeNote xml:lang="de">Nur für den Post- und Fernmeldesektor insgesamt.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71031">
    <skos:narrower rdf:resource="descriptor/17744-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71031">
    <skos:narrower rdf:resource="descriptor/17743-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71031">
    <skos:narrower rdf:resource="descriptor/17740-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71031">
    <skos:narrower rdf:resource="descriptor/17739-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71031">
    <skos:narrower rdf:resource="descriptor/17735-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71031">
    <skos:narrower rdf:resource="descriptor/17742-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71031">
    <skos:narrower rdf:resource="descriptor/17732-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71031">
    <skos:narrower rdf:resource="descriptor/17726-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71031">
    <skos:narrower rdf:resource="descriptor/17727-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19342-5">
    <skos:related rdf:resource="descriptor/13161-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19342-5">
    <skos:related rdf:resource="descriptor/12947-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19342-5">
    <skos:related rdf:resource="descriptor/14110-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19342-5">
    <skos:related rdf:resource="descriptor/19341-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19342-5">
    <skos:related rdf:resource="descriptor/13225-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11680-4">
    <skos:prefLabel xml:lang="en">Consumption tax</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11680-4">
    <skos:prefLabel xml:lang="de">Verbrauchsteuer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19604-1">
    <skos:broader rdf:resource="thsys/72181"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19604-1">
    <skos:broader rdf:resource="thsys/70958"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19604-1">
    <skos:broader rdf:resource="descriptor/15076-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18576-6">
    <skos:broader rdf:resource="thsys/70941"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18576-6">
    <skos:broader rdf:resource="thsys/70939"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18576-6">
    <skos:broader rdf:resource="thsys/70925"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18576-6">
    <skos:broader rdf:resource="descriptor/18574-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18621-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70650">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19641-2">
    <skos:scopeNote xml:lang="de">Wechselwirkungen zwischen Bevölkerungs- und Wirtschaftsentwicklung</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19796-1">
    <skos:prefLabel xml:lang="en">Unit of measurement</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19796-1">
    <skos:prefLabel xml:lang="de">Maßeinheit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10843-0">
    <skos:prefLabel xml:lang="en">Intra-industry trade</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10843-0">
    <skos:prefLabel xml:lang="de">Intraindustrieller Handel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13421-4">
    <skos:broader rdf:resource="thsys/73367"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13421-4">
    <skos:broader rdf:resource="thsys/70870"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13421-4">
    <skos:broader rdf:resource="descriptor/16314-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70958">
    <skos:prefLabel xml:lang="de">W.09  Dienstleistungen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18211-4">
    <skos:prefLabel xml:lang="en">Press product</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18211-4">
    <skos:prefLabel xml:lang="de">Presseprodukt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70884">
    <skos:narrower rdf:resource="descriptor/11574-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70884">
    <skos:narrower rdf:resource="descriptor/13304-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70884">
    <skos:narrower rdf:resource="descriptor/16033-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70884">
    <skos:narrower rdf:resource="descriptor/13839-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70884">
    <skos:narrower rdf:resource="descriptor/11840-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70884">
    <skos:narrower rdf:resource="descriptor/11836-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70884">
    <skos:narrower rdf:resource="descriptor/11839-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70884">
    <skos:narrower rdf:resource="descriptor/18844-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70884">
    <skos:narrower rdf:resource="descriptor/18755-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70884">
    <skos:narrower rdf:resource="descriptor/13837-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70884">
    <skos:narrower rdf:resource="descriptor/18756-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16036-6">
    <skos:altLabel xml:lang="de">Schenkung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16036-6">
    <skos:altLabel xml:lang="de">Schadensersatz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16036-6">
    <skos:altLabel xml:lang="de">Schadenersatz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16036-6">
    <skos:altLabel xml:lang="de">Mahnverfahren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16036-6">
    <skos:altLabel xml:lang="de">Mahnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20401-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10864-6">
    <skos:scopeNote xml:lang="de">Ausfuhrhandel, der nicht vom Hersteller der Produkte, sondern von speziellen Handelsfirmen organisiert wird sowie diese Handelsfirmen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19300-0">
    <skos:broader rdf:resource="thsys/71029"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19300-0">
    <skos:broader rdf:resource="thsys/70960"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19300-0">
    <skos:broader rdf:resource="descriptor/19293-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24705-0">
    <skos:altLabel xml:lang="en">Commercialisation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24705-0">
    <skos:altLabel xml:lang="de">Ökonomisierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19966-2">
    <skos:scopeNote xml:lang="de">Vergütung für Leistungen freier Berufe.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15823-2">
    <skos:related rdf:resource="descriptor/12585-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15823-2">
    <skos:related rdf:resource="descriptor/16252-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15823-2">
    <skos:related rdf:resource="descriptor/13988-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15823-2">
    <skos:related rdf:resource="descriptor/15825-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15823-2">
    <skos:related rdf:resource="descriptor/11302-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15823-2">
    <skos:related rdf:resource="descriptor/16658-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15823-2">
    <skos:related rdf:resource="descriptor/19753-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19522-3">
    <skos:broader rdf:resource="thsys/70187"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12217-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11902-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19769-4">
    <skos:altLabel xml:lang="de">Gesellschaftskrise</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19430-1">
    <skos:prefLabel xml:lang="en">Chicago Antitrust Theory</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19430-1">
    <skos:prefLabel xml:lang="de">Chicago-Wettbewerbstheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11752-5">
    <skos:prefLabel xml:lang="en">Distributional conflict</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11752-5">
    <skos:prefLabel xml:lang="de">Verteilungskonflikt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13406-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19521-5">
    <skos:related rdf:resource="descriptor/19752-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19521-5">
    <skos:related rdf:resource="descriptor/10176-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19521-5">
    <skos:related rdf:resource="descriptor/18144-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19521-5">
    <skos:related rdf:resource="descriptor/19773-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19521-5">
    <skos:related rdf:resource="descriptor/13988-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15737-2">
    <skos:narrower rdf:resource="descriptor/12605-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15737-2">
    <skos:narrower rdf:resource="descriptor/15742-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15737-2">
    <skos:narrower rdf:resource="descriptor/15740-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15963-0">
    <skos:prefLabel xml:lang="en">Legalization</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15963-0">
    <skos:prefLabel xml:lang="de">Legalisierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10113-2">
    <skos:related rdf:resource="descriptor/10166-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10113-2">
    <skos:related rdf:resource="descriptor/10975-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10113-2">
    <skos:related rdf:resource="descriptor/10424-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10113-2">
    <skos:related rdf:resource="descriptor/11529-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10113-2">
    <skos:related rdf:resource="descriptor/16739-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10113-2">
    <skos:related rdf:resource="descriptor/10076-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10940-2">
    <skos:prefLabel xml:lang="en">Exclusive dealing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10940-2">
    <skos:prefLabel xml:lang="de">Vertriebsbindung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70839">
    <skos:narrower rdf:resource="descriptor/13375-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70839">
    <skos:narrower rdf:resource="descriptor/13374-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70839">
    <skos:narrower rdf:resource="descriptor/21002-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70839">
    <skos:narrower rdf:resource="descriptor/16541-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70839">
    <skos:narrower rdf:resource="descriptor/13348-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70839">
    <skos:narrower rdf:resource="descriptor/18738-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70839">
    <skos:narrower rdf:resource="descriptor/18180-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70839">
    <skos:narrower rdf:resource="descriptor/15658-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70839">
    <skos:narrower rdf:resource="descriptor/19551-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70839">
    <skos:narrower rdf:resource="descriptor/13292-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70839">
    <skos:narrower rdf:resource="descriptor/19518-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70839">
    <skos:narrower rdf:resource="descriptor/18903-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70839">
    <skos:narrower rdf:resource="descriptor/13372-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70839">
    <skos:narrower rdf:resource="descriptor/19526-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70839">
    <skos:narrower rdf:resource="descriptor/13371-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70839">
    <skos:narrower rdf:resource="descriptor/18892-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70839">
    <skos:narrower rdf:resource="descriptor/18891-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70839">
    <skos:narrower rdf:resource="descriptor/19578-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70839">
    <skos:narrower rdf:resource="descriptor/16683-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70839">
    <skos:narrower rdf:resource="descriptor/18063-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70839">
    <skos:narrower rdf:resource="descriptor/16479-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70839">
    <skos:narrower rdf:resource="descriptor/11157-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70839">
    <skos:narrower rdf:resource="descriptor/11555-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70839">
    <skos:narrower rdf:resource="descriptor/19517-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70839">
    <skos:narrower rdf:resource="descriptor/18890-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70839">
    <skos:narrower rdf:resource="descriptor/16635-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70839">
    <skos:narrower rdf:resource="descriptor/13415-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70839">
    <skos:narrower rdf:resource="descriptor/18334-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70839">
    <skos:narrower rdf:resource="descriptor/15908-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70839">
    <skos:narrower rdf:resource="descriptor/18894-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70839">
    <skos:narrower rdf:resource="descriptor/19676-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70839">
    <skos:narrower rdf:resource="descriptor/13370-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70839">
    <skos:narrower rdf:resource="thsys/70838"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15609-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11741-3">
    <skos:prefLabel xml:lang="en">Achievement principle</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11741-3">
    <skos:prefLabel xml:lang="de">Leistungsprinzip</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16437-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13188-5">
    <zbwext:useInsteadNote rdf:nodeID="id185"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15396-2">
    <skos:broader rdf:resource="thsys/72183"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15396-2">
    <skos:broader rdf:resource="descriptor/15064-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14244-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18813-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18949-0">
    <skos:related rdf:resource="descriptor/16783-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13738-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19267-0">
    <skos:prefLabel xml:lang="en">Structure of interest rates</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19267-0">
    <skos:prefLabel xml:lang="de">Zinsstruktur</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19775-2">
    <skos:related rdf:resource="descriptor/14003-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19775-2">
    <skos:related rdf:resource="descriptor/19707-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19775-2">
    <skos:related rdf:resource="descriptor/19639-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19775-2">
    <skos:related rdf:resource="descriptor/15642-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12418-5">
    <skos:altLabel xml:lang="en">Equity accounting</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10506-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10013-6">
    <skos:broader rdf:resource="thsys/70175"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10013-6">
    <skos:broader rdf:resource="thsys/70140"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10013-6">
    <skos:broader rdf:resource="descriptor/10012-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12326-3">
    <skos:narrower rdf:resource="descriptor/12337-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12326-3">
    <skos:narrower rdf:resource="descriptor/12331-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12326-3">
    <skos:narrower rdf:resource="descriptor/18722-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12326-3">
    <skos:narrower rdf:resource="descriptor/12361-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70341">
    <rdfs:label xml:lang="de">Pflanzenbauprodukte</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19403-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71070">
    <skos:prefLabel xml:lang="en">V.11.01  Regional economics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71070">
    <skos:prefLabel xml:lang="de">V.11.01  Regionalökonomik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73382">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10899-1">
    <skos:altLabel xml:lang="de">Horizontal integration</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17980-4">
    <skos:broader rdf:resource="thsys/70307"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17980-4">
    <skos:broader rdf:resource="descriptor/17999-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19278-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13382-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11628-6">
    <skos:related rdf:resource="descriptor/10985-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11628-6">
    <skos:related rdf:resource="descriptor/12931-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11628-6">
    <skos:related rdf:resource="descriptor/11633-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11628-6">
    <skos:related rdf:resource="descriptor/11142-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11628-6">
    <skos:related rdf:resource="descriptor/10979-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11628-6">
    <skos:related rdf:resource="descriptor/12189-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11628-6">
    <skos:related rdf:resource="descriptor/11612-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11628-6">
    <skos:related rdf:resource="descriptor/18803-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10667-1">
    <skos:related rdf:resource="descriptor/10346-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10667-1">
    <skos:related rdf:resource="descriptor/10659-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10667-1">
    <skos:related rdf:resource="descriptor/10668-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10667-1">
    <skos:related rdf:resource="descriptor/10242-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10667-1">
    <skos:related rdf:resource="descriptor/19311-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15930-1">
    <skos:related rdf:resource="descriptor/15903-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15930-1">
    <skos:related rdf:resource="descriptor/15893-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15930-1">
    <skos:related rdf:resource="descriptor/15935-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15930-1">
    <skos:related rdf:resource="descriptor/19639-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15930-1">
    <skos:related rdf:resource="descriptor/18313-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15930-1">
    <skos:related rdf:resource="descriptor/13451-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12174-0">
    <skos:prefLabel xml:lang="en">Public Limited Company</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12174-0">
    <skos:prefLabel xml:lang="de">Aktiengesellschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18764-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10205-4">
    <skos:related rdf:resource="descriptor/10913-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10205-4">
    <skos:related rdf:resource="descriptor/10206-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10205-4">
    <skos:related rdf:resource="descriptor/19226-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10205-4">
    <skos:related rdf:resource="descriptor/10904-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18378-3">
    <skos:prefLabel xml:lang="en">Water supply</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18378-3">
    <skos:prefLabel xml:lang="de">Wasserversorgung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12558-3">
    <skos:broader rdf:resource="thsys/73337"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12558-3">
    <skos:broader rdf:resource="thsys/71043"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12558-3">
    <skos:broader rdf:resource="thsys/70046"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12558-3">
    <skos:broader rdf:resource="descriptor/11640-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12558-3">
    <skos:broader rdf:resource="descriptor/12543-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70865">
    <skos:prefLabel xml:lang="en">W.22.01.06  Armed forces</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70865">
    <skos:prefLabel xml:lang="de">W.22.01.06  Militär</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14947-3">
    <skos:prefLabel xml:lang="en">Preserved fruit and vegetables</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14947-3">
    <skos:prefLabel xml:lang="de">Konserviertes Obst und Gemüse</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18102-2">
    <skos:broader rdf:resource="thsys/71037"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18102-2">
    <skos:broader rdf:resource="thsys/70471"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18102-2">
    <skos:broader rdf:resource="descriptor/15600-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24623-2">
    <skos:scopeNote xml:lang="de">Der Anspruchslohn ist der Betrag, den ein angebotener Lohnsatz übertreffen muss, damit ein Wirtschaftssubjekt Arbeit aufnimmt (oder zusätzliche Arbeit leistet), statt weitere Sucharbeitslosigkeit oder Freizeit zu präferieren.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14583-1">
    <skos:broader rdf:resource="thsys/70373"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14583-1">
    <skos:broader rdf:resource="descriptor/14525-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14583-1">
    <skos:broader rdf:resource="descriptor/14581-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17638-1">
    <skos:related rdf:resource="descriptor/16815-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10587-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12501-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14932-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12039-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19551-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14177-5">
    <skos:altLabel xml:lang="de">Fernheizung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11348-5">
    <skos:broader rdf:resource="thsys/71051"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11348-5">
    <skos:broader rdf:resource="thsys/70235"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11348-5">
    <skos:broader rdf:resource="descriptor/11339-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11813-4">
    <skos:related rdf:resource="descriptor/14184-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11813-4">
    <skos:related rdf:resource="descriptor/18388-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11813-4">
    <skos:related rdf:resource="descriptor/11812-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71102">
    <skos:narrower rdf:resource="descriptor/10986-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71102">
    <skos:narrower rdf:resource="descriptor/11101-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71102">
    <skos:narrower rdf:resource="descriptor/11059-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71102">
    <skos:narrower rdf:resource="descriptor/10977-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71102">
    <skos:narrower rdf:resource="descriptor/11055-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71102">
    <skos:narrower rdf:resource="descriptor/10201-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71102">
    <skos:narrower rdf:resource="descriptor/11046-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19702-1">
    <skos:scopeNote xml:lang="de">Bedeutung der Verteidigung des Wahlkreises für die Kandidaten und ihre politischen Entscheidungen</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11216-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18422-5">
    <skos:altLabel xml:lang="de">Kraftfahrzeugschlosser</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18422-5">
    <skos:altLabel xml:lang="de">Kraftfahrzeugmechanikerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18422-5">
    <skos:altLabel xml:lang="de">Kraftfahrzeugmechaniker</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18422-5">
    <skos:altLabel xml:lang="de">Kraftfahrzeugindustriearbeiter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18422-5">
    <skos:altLabel xml:lang="de">Kraftfahrzeughandwerkerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18422-5">
    <skos:altLabel xml:lang="de">Kraftfahrzeughandwerker</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18422-5">
    <skos:altLabel xml:lang="de">Kfz-technische Berufe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18422-5">
    <skos:altLabel xml:lang="de">Kfz-Mechaniker</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18422-5">
    <skos:altLabel xml:lang="de">Kfz-Handwerkerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18422-5">
    <skos:altLabel xml:lang="de">Kfz-Handwerker</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18422-5">
    <skos:altLabel xml:lang="de">Autoschlosser</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18422-5">
    <skos:altLabel xml:lang="de">Automobilarbeiterinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18422-5">
    <skos:altLabel xml:lang="de">Automobilarbeiter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18001-1">
    <skos:broader rdf:resource="thsys/71090"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18001-1">
    <skos:broader rdf:resource="thsys/71034"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18001-1">
    <skos:broader rdf:resource="descriptor/18119-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18001-1">
    <skos:broader rdf:resource="descriptor/12062-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18001-1">
    <skos:broader rdf:resource="descriptor/11905-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18531-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24787-0">
    <skos:prefLabel xml:lang="en">Logistics provider</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24787-0">
    <skos:prefLabel xml:lang="de">Logistikunternehmen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15610-5">
    <skos:broader rdf:resource="thsys/73340"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18006-5">
    <skos:prefLabel xml:lang="en">Fast-food industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18006-5">
    <skos:prefLabel xml:lang="de">Schnellgastronomie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16119-2">
    <skos:broader rdf:resource="thsys/73337"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16119-2">
    <skos:broader rdf:resource="thsys/71076"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16119-2">
    <skos:broader rdf:resource="thsys/70397"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16119-2">
    <skos:broader rdf:resource="descriptor/16092-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12800-1">
    <skos:narrower rdf:resource="descriptor/12798-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11832-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14796-5">
    <skos:broader rdf:resource="thsys/70328"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14796-5">
    <skos:broader rdf:resource="thsys/70161"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14796-5">
    <skos:broader rdf:resource="descriptor/14802-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14796-5">
    <skos:broader rdf:resource="descriptor/15009-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15671-6">
    <skos:broader rdf:resource="thsys/73388"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15671-6">
    <skos:broader rdf:resource="thsys/70850"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15671-6">
    <skos:broader rdf:resource="descriptor/15682-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14396-0">
    <skos:altLabel xml:lang="de">Gabelstapler</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14396-0">
    <skos:altLabel xml:lang="de">FTS (Fahrerloses Transportsystem)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14396-0">
    <skos:altLabel xml:lang="de">Fahrerloses Transportsystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14396-0">
    <skos:altLabel xml:lang="de">Elektrokarren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14396-0">
    <skos:altLabel xml:lang="de">Automatisches Transportsystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18520-5">
    <skos:broader rdf:resource="thsys/70960"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18520-5">
    <skos:broader rdf:resource="descriptor/18551-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18520-5">
    <skos:broader rdf:resource="descriptor/13480-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71042">
    <skos:prefLabel xml:lang="de">B.03.01.02  Erfolgsrechnung und betriebliche Kennzahlen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11875-3">
    <skos:prefLabel xml:lang="en">Spatial structure</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11875-3">
    <skos:prefLabel xml:lang="de">Regionalstruktur</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15381-1">
    <skos:scopeNote xml:lang="de">Grundlage der Modellentwicklung in der Ökonometrie, der Wirtschaftsinformatik, etc.: Ausgestaltung der zu untersuchenden empirischen Problemstellung bzw. formale Beschreibung betrieblicher Prozesse, Objekte, usw.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73372">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70843">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#member">
    <skos:definition xml:lang="en">A member of a collection.</skos:definition>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16146-6">
    <skos:broader rdf:resource="thsys/73336"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16146-6">
    <skos:broader rdf:resource="descriptor/16477-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16146-6">
    <skos:broader rdf:resource="descriptor/16124-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10426-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19723-0">
    <skos:scopeNote xml:lang="de">Finanz- und Sachhilfen an ausländische Staaten zur Bestreitung von Rüstungsaufwendungen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13592-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10184-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12370-0">
    <skos:related rdf:resource="descriptor/16067-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12370-0">
    <skos:related rdf:resource="descriptor/12775-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10201-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13414-1">
    <skos:broader rdf:resource="thsys/73367"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13414-1">
    <skos:broader rdf:resource="thsys/70870"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13414-1">
    <skos:broader rdf:resource="thsys/70058"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13414-1">
    <skos:broader rdf:resource="descriptor/13409-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16721-5">
    <skos:altLabel xml:lang="de">Kritischer Rationalismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19222-1">
    <skos:narrower rdf:resource="descriptor/11335-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11495-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70397">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">B.08.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16542-5">
    <skos:related rdf:resource="descriptor/16318-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16542-5">
    <skos:related rdf:resource="descriptor/19769-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16542-5">
    <skos:related rdf:resource="descriptor/11558-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12770-5">
    <skos:prefLabel xml:lang="en">Marketing strategy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12770-5">
    <skos:prefLabel xml:lang="de">Marketingpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11341-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11328-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15696-4">
    <skos:prefLabel xml:lang="en">Middle Ages</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15696-4">
    <skos:prefLabel xml:lang="de">Mittelalter</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70313">
    <skos:prefLabel xml:lang="de">P.17  Leder, Lederwaren und Schuhe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17999-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12228-3">
    <skos:related rdf:resource="descriptor/12302-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12228-3">
    <skos:related rdf:resource="descriptor/24859-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12228-3">
    <skos:related rdf:resource="descriptor/12205-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12228-3">
    <skos:related rdf:resource="descriptor/19093-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12228-3">
    <skos:related rdf:resource="descriptor/12226-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12228-3">
    <skos:related rdf:resource="descriptor/12225-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12228-3">
    <skos:related rdf:resource="descriptor/11460-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12228-3">
    <skos:related rdf:resource="descriptor/18084-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12228-3">
    <skos:related rdf:resource="descriptor/12218-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10027-2">
    <skos:altLabel xml:lang="en">Intermediate product</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10027-2">
    <skos:altLabel xml:lang="de">Zwischenprodukt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10027-2">
    <skos:altLabel xml:lang="de">Vorleistung (VGR)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10027-2">
    <skos:altLabel xml:lang="de">Teilfabrikat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10027-2">
    <skos:altLabel xml:lang="de">Halbwaren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10027-2">
    <skos:altLabel xml:lang="de">Halbfertigerzeugnis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10027-2">
    <skos:altLabel xml:lang="de">Halbfabrikat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15930-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70257">
    <skos:broader rdf:resource="thsys/p"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10413-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16384-4">
    <skos:scopeNote xml:lang="de">Formen der politischen Ordnung nach berufsständischen bzw. verbandsorientierten Prinzipien</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17592-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18647-2">
    <skos:prefLabel xml:lang="en">Multimedia</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18647-2">
    <skos:prefLabel xml:lang="de">Multimedia</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12945-3">
    <skos:related rdf:resource="descriptor/11276-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12945-3">
    <skos:related rdf:resource="descriptor/14041-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12945-3">
    <skos:related rdf:resource="descriptor/10505-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12945-3">
    <skos:related rdf:resource="descriptor/10503-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12945-3">
    <skos:related rdf:resource="descriptor/12961-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12945-3">
    <skos:related rdf:resource="descriptor/12964-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12945-3">
    <skos:related rdf:resource="descriptor/18320-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12945-3">
    <skos:related rdf:resource="descriptor/13446-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12945-3">
    <skos:related rdf:resource="descriptor/13450-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12945-3">
    <skos:related rdf:resource="descriptor/13433-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12945-3">
    <skos:related rdf:resource="descriptor/13426-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12945-3">
    <skos:related rdf:resource="descriptor/14054-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12945-3">
    <skos:related rdf:resource="descriptor/11801-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12325-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19941-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10106-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14082-2">
    <skos:altLabel xml:lang="de">Rohtabak</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13047-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11855-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#OrderedCollection">
    <rdfs:comment xml:lang="en">Ordered collections can be used where you would like a set of concepts to be displayed in a specific order, and optionally under a 'node label'.</rdfs:comment>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11917-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13358-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13620-5">
    <skos:related rdf:resource="descriptor/18620-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13620-5">
    <skos:related rdf:resource="descriptor/19064-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13620-5">
    <skos:related rdf:resource="descriptor/13622-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13620-5">
    <skos:related rdf:resource="descriptor/18617-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18007-3">
    <skos:scopeNote xml:lang="de">Nur zu verwenden für die Kennzeichnung abgesetzter Mengen eines Produkts.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10876-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16759-1">
    <skos:broader rdf:resource="thsys/71117"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16759-1">
    <skos:broader rdf:resource="thsys/70823"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16759-1">
    <skos:broader rdf:resource="thsys/70013"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16759-1">
    <skos:broader rdf:resource="descriptor/18859-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16948-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70327">
    <rdfs:label xml:lang="en">Southeast Asia</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70327">
    <rdfs:label xml:lang="de">Südostasien</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14888-0">
    <skos:broader rdf:resource="thsys/70066"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14888-0">
    <skos:broader rdf:resource="descriptor/14846-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13037-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73381">
    <skos:prefLabel xml:lang="de">N.04.02.01  Staat und Nation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15444-3">
    <skos:prefLabel xml:lang="en">Coalition</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15444-3">
    <skos:prefLabel xml:lang="de">Koalition</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18178-4">
    <skos:prefLabel xml:lang="en">Credit repayment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18178-4">
    <skos:prefLabel xml:lang="de">Kredittilgung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15769-3">
    <skos:narrower rdf:resource="descriptor/16023-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15769-3">
    <skos:narrower rdf:resource="descriptor/19817-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14602-2">
    <skos:narrower rdf:resource="descriptor/18267-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14602-2">
    <skos:narrower rdf:resource="descriptor/14630-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14602-2">
    <skos:narrower rdf:resource="descriptor/14740-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14602-2">
    <skos:narrower rdf:resource="descriptor/14217-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14602-2">
    <skos:narrower rdf:resource="descriptor/14617-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14602-2">
    <skos:narrower rdf:resource="descriptor/14611-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14602-2">
    <skos:narrower rdf:resource="descriptor/14610-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14602-2">
    <skos:narrower rdf:resource="descriptor/14327-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18492-5">
    <skos:broader rdf:resource="thsys/70970"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18492-5">
    <skos:broader rdf:resource="thsys/70828"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18492-5">
    <skos:broader rdf:resource="thsys/70581"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18492-5">
    <skos:broader rdf:resource="descriptor/15655-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19467-6">
    <zbwext:useInsteadNote rdf:nodeID="id111"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19467-6">
    <zbwext:useInsteadNote rdf:nodeID="id110"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12718-0">
    <skos:broader rdf:resource="thsys/70310"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12718-0">
    <skos:broader rdf:resource="descriptor/12725-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16815-3">
    <skos:related rdf:resource="descriptor/17638-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16815-3">
    <skos:related rdf:resource="descriptor/17983-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16815-3">
    <skos:related rdf:resource="descriptor/16496-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17682-5">
    <skos:broader rdf:resource="thsys/70429"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17682-5">
    <skos:broader rdf:resource="descriptor/17687-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11703-4">
    <skos:related rdf:resource="descriptor/19331-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11703-4">
    <skos:related rdf:resource="descriptor/19567-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11703-4">
    <skos:related rdf:resource="descriptor/11706-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11703-4">
    <skos:related rdf:resource="descriptor/11636-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11703-4">
    <skos:related rdf:resource="descriptor/10829-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11703-4">
    <skos:related rdf:resource="descriptor/16090-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18547-6">
    <skos:prefLabel xml:lang="en">Second-hand market</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18547-6">
    <skos:prefLabel xml:lang="de">Gebrauchtmarkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15907-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10930-5">
    <skos:prefLabel xml:lang="en">Price fixing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10930-5">
    <skos:prefLabel xml:lang="de">Preisabsprache</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70942">
    <skos:prefLabel xml:lang="en">W.12.01  Transport mode</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70942">
    <skos:prefLabel xml:lang="de">W.12.01  Verkehrsträger</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11943-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11536-4">
    <skos:broader rdf:resource="thsys/73380"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11536-4">
    <skos:broader rdf:resource="thsys/70869"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11536-4">
    <skos:broader rdf:resource="thsys/70144"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11536-4">
    <skos:broader rdf:resource="descriptor/13413-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70095">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19520-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15928-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19675-6">
    <skos:prefLabel xml:lang="en">Pharmaceutical market</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19675-6">
    <skos:prefLabel xml:lang="de">Arzneimittelmarkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14231-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14593-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15089-4">
    <skos:related rdf:resource="descriptor/11327-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15089-4">
    <skos:related rdf:resource="descriptor/11325-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15089-4">
    <skos:related rdf:resource="descriptor/11322-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10619-5">
    <skos:narrower rdf:resource="descriptor/10969-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10619-5">
    <skos:narrower rdf:resource="descriptor/10745-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10619-5">
    <skos:narrower rdf:resource="descriptor/10616-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13510-5">
    <skos:narrower rdf:resource="descriptor/11881-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13510-5">
    <skos:narrower rdf:resource="descriptor/13563-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13510-5">
    <skos:narrower rdf:resource="descriptor/11974-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13510-5">
    <skos:narrower rdf:resource="descriptor/13259-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13510-5">
    <skos:narrower rdf:resource="descriptor/13508-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13510-5">
    <skos:narrower rdf:resource="descriptor/13256-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13510-5">
    <skos:narrower rdf:resource="descriptor/13255-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13510-5">
    <skos:narrower rdf:resource="descriptor/11876-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13510-5">
    <skos:narrower rdf:resource="descriptor/13507-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13510-5">
    <skos:narrower rdf:resource="descriptor/13632-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13510-5">
    <skos:narrower rdf:resource="descriptor/13246-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13510-5">
    <skos:narrower rdf:resource="descriptor/13504-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13510-5">
    <skos:narrower rdf:resource="descriptor/18631-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13510-5">
    <skos:narrower rdf:resource="descriptor/13621-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13510-5">
    <skos:narrower rdf:resource="descriptor/13501-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13510-5">
    <skos:narrower rdf:resource="descriptor/11864-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13510-5">
    <skos:narrower rdf:resource="descriptor/13499-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13510-5">
    <skos:narrower rdf:resource="descriptor/13498-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13510-5">
    <skos:narrower rdf:resource="descriptor/12689-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14652-1">
    <skos:broader rdf:resource="thsys/70463"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14652-1">
    <skos:broader rdf:resource="descriptor/14642-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19724-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10961-1">
    <skos:related rdf:resource="descriptor/10957-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13325-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13039-1">
    <skos:prefLabel xml:lang="en">Steel forming</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13039-1">
    <skos:prefLabel xml:lang="de">Stahlverformung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19003-6">
    <skos:prefLabel xml:lang="en">Redundancy pay</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19003-6">
    <skos:prefLabel xml:lang="de">Abfindung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10248-0">
    <skos:related rdf:resource="descriptor/10272-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10248-0">
    <skos:related rdf:resource="descriptor/11141-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10248-0">
    <skos:related rdf:resource="descriptor/11186-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18850-5">
    <skos:scopeNote xml:lang="de">Für einzelne Berufe und Berufsgruppen sowie als Standesverbände freier Berufe, z.B. von Ärzten und Rechtsanwälten</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15093-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14759-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15939-4">
    <skos:altLabel xml:lang="en">Population control</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15939-4">
    <skos:altLabel xml:lang="en">Fertility control</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15939-4">
    <skos:altLabel xml:lang="en">Family planning program</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15939-4">
    <skos:altLabel xml:lang="en">Birth control</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15939-4">
    <skos:altLabel xml:lang="de">Geburtenregelung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15939-4">
    <skos:altLabel xml:lang="de">Geburtenplanung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15939-4">
    <skos:altLabel xml:lang="de">Geburtenkontrolle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15939-4">
    <skos:altLabel xml:lang="de">Familienplanungsprogramm</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18623-2">
    <skos:related rdf:resource="descriptor/18628-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18623-2">
    <skos:related rdf:resource="descriptor/19449-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18623-2">
    <skos:related rdf:resource="descriptor/15744-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18623-2">
    <skos:related rdf:resource="descriptor/15743-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18623-2">
    <skos:related rdf:resource="descriptor/15010-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18449-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15554-3">
    <skos:prefLabel xml:lang="en">Dynamic programming</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15554-3">
    <skos:prefLabel xml:lang="de">Dynamische Optimierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19786-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10313-1">
    <skos:altLabel xml:lang="de">Faktorproportion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16287-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19660-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18580-1">
    <skos:altLabel xml:lang="de">Sicherheit im Straßenverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11070-3">
    <skos:prefLabel xml:lang="en">Working class</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11070-3">
    <skos:prefLabel xml:lang="de">Arbeiterklasse</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14441-0">
    <skos:broader rdf:resource="thsys/70258"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14441-0">
    <skos:broader rdf:resource="descriptor/13513-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11433-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14928-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15554-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71100">
    <skos:broader rdf:resource="thsys/70179"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10224-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70815">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18859-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16365-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11399-2">
    <skos:altLabel xml:lang="de">Bildungsurlaubsgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16274-4">
    <skos:related rdf:resource="descriptor/12881-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16274-4">
    <skos:related rdf:resource="descriptor/18793-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16274-4">
    <skos:related rdf:resource="descriptor/16273-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16274-4">
    <skos:related rdf:resource="descriptor/11170-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11563-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70086">
    <skos:broader rdf:resource="thsys/70026"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70313">
    <skos:narrower rdf:resource="descriptor/14834-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70313">
    <skos:narrower rdf:resource="descriptor/14833-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70313">
    <skos:narrower rdf:resource="descriptor/14831-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70313">
    <skos:narrower rdf:resource="descriptor/14829-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10754-6">
    <skos:broader rdf:resource="thsys/71094"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10754-6">
    <skos:broader rdf:resource="descriptor/10764-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10847-6">
    <skos:related rdf:resource="descriptor/13509-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10847-6">
    <skos:related rdf:resource="descriptor/10711-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18237-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11637-5">
    <skos:prefLabel xml:lang="en">Economics of taxation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11637-5">
    <skos:prefLabel xml:lang="de">Steuerwissenschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13137-1">
    <skos:broader rdf:resource="thsys/70988"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13137-1">
    <skos:broader rdf:resource="descriptor/13136-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73370">
    <skos:prefLabel xml:lang="de">N.04.03.04  Gesundheits- und Sozialpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19113-6">
    <skos:related rdf:resource="descriptor/19308-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19113-6">
    <skos:related rdf:resource="descriptor/12716-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18823-1">
    <skos:broader rdf:resource="thsys/70859"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18823-1">
    <skos:broader rdf:resource="descriptor/16212-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18823-1">
    <skos:broader rdf:resource="descriptor/12041-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16417-1">
    <skos:scopeNote xml:lang="de">Sowohl für Neutralität im Fall eines einzelnen Konfliktes als auch für dauernde Neutralität bzw. Blockfreiheit</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10332-4">
    <skos:related rdf:resource="descriptor/19250-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10332-4">
    <skos:related rdf:resource="descriptor/10460-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10332-4">
    <skos:related rdf:resource="descriptor/10327-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10332-4">
    <skos:related rdf:resource="descriptor/19249-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10332-4">
    <skos:related rdf:resource="descriptor/12619-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10332-4">
    <skos:related rdf:resource="descriptor/10307-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16808-0">
    <skos:prefLabel xml:lang="en">Polar region</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16808-0">
    <skos:prefLabel xml:lang="de">Polargebiet</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18206-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18520-5">
    <skos:prefLabel xml:lang="en">Food discounter</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18520-5">
    <skos:prefLabel xml:lang="de">Lebensmittel-Discounter</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18440-3">
    <skos:related rdf:resource="descriptor/13159-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18440-3">
    <skos:related rdf:resource="descriptor/12910-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18440-3">
    <skos:related rdf:resource="descriptor/14940-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18210-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11073-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19687-6">
    <skos:altLabel xml:lang="de">Staatsbürgerschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19687-6">
    <skos:altLabel xml:lang="de">Staatsangehörigkeitsrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19687-6">
    <skos:altLabel xml:lang="de">Staatenlosigkeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19687-6">
    <skos:altLabel xml:lang="de">Einbürgerung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19687-6">
    <skos:altLabel xml:lang="de">Ausbürgerung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10616-4">
    <skos:narrower rdf:resource="descriptor/19326-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10616-4">
    <skos:narrower rdf:resource="descriptor/10698-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12564-1">
    <skos:altLabel xml:lang="en">Personnel information system</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12564-1">
    <skos:altLabel xml:lang="de">Personalinformationssystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12564-1">
    <skos:altLabel xml:lang="de">Personaldatenverarbeitung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70961">
    <skos:broader rdf:resource="thsys/70962"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18737-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13071-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11233-1">
    <skos:altLabel xml:lang="en">Secondary labor market</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11233-1">
    <skos:altLabel xml:lang="de">Zweiter Arbeitsmarkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18381-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10054-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13974-5">
    <skos:prefLabel xml:lang="en">Domestic workers</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13974-5">
    <skos:prefLabel xml:lang="de">Hauspersonal</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16204-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14345-3">
    <skos:altLabel xml:lang="de">Kugellager</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15711-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11231-5">
    <skos:prefLabel xml:lang="en">Internal labour market</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11231-5">
    <skos:prefLabel xml:lang="de">Interner Arbeitsmarkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16327-2">
    <skos:altLabel xml:lang="de">Landesministerium</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16327-2">
    <skos:altLabel xml:lang="de">Bundesministerium</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10964-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18677-0">
    <skos:prefLabel xml:lang="en">Insider trading regulation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18677-0">
    <skos:prefLabel xml:lang="de">Insiderrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16455-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71099">
    <skos:narrower rdf:resource="descriptor/10680-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71099">
    <skos:narrower rdf:resource="descriptor/10677-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71099">
    <skos:narrower rdf:resource="descriptor/10410-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71099">
    <skos:narrower rdf:resource="descriptor/10678-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71099">
    <skos:narrower rdf:resource="descriptor/24855-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71099">
    <skos:narrower rdf:resource="descriptor/10673-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71099">
    <skos:narrower rdf:resource="descriptor/10672-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71099">
    <skos:narrower rdf:resource="descriptor/10671-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71099">
    <skos:narrower rdf:resource="descriptor/10676-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71099">
    <skos:narrower rdf:resource="descriptor/10659-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71099">
    <skos:narrower rdf:resource="descriptor/19313-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71099">
    <skos:narrower rdf:resource="descriptor/10668-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71099">
    <skos:narrower rdf:resource="descriptor/10667-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71099">
    <skos:narrower rdf:resource="descriptor/10642-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13249-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13601-2">
    <skos:related rdf:resource="descriptor/18616-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13601-2">
    <skos:related rdf:resource="descriptor/18615-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18222-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70406">
    <skos:narrower rdf:resource="descriptor/10414-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70406">
    <skos:narrower rdf:resource="descriptor/10359-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70406">
    <skos:narrower rdf:resource="descriptor/10434-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70406">
    <skos:narrower rdf:resource="descriptor/10439-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70406">
    <skos:narrower rdf:resource="descriptor/10433-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70406">
    <skos:narrower rdf:resource="descriptor/10541-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70406">
    <skos:narrower rdf:resource="descriptor/10432-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70406">
    <skos:narrower rdf:resource="descriptor/10466-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70406">
    <skos:narrower rdf:resource="descriptor/10428-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70406">
    <skos:narrower rdf:resource="descriptor/10427-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70406">
    <skos:narrower rdf:resource="descriptor/10413-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70406">
    <skos:narrower rdf:resource="descriptor/10412-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70406">
    <skos:narrower rdf:resource="descriptor/11943-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70406">
    <skos:narrower rdf:resource="descriptor/10426-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70406">
    <skos:narrower rdf:resource="descriptor/10329-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70406">
    <skos:narrower rdf:resource="descriptor/10424-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70406">
    <skos:narrower rdf:resource="descriptor/19244-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70406">
    <skos:narrower rdf:resource="descriptor/20982-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70406">
    <skos:narrower rdf:resource="descriptor/10421-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70406">
    <skos:narrower rdf:resource="descriptor/10350-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70406">
    <skos:narrower rdf:resource="descriptor/19243-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70406">
    <skos:narrower rdf:resource="descriptor/19251-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70406">
    <skos:narrower rdf:resource="descriptor/10527-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70406">
    <skos:narrower rdf:resource="descriptor/10418-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70406">
    <skos:narrower rdf:resource="descriptor/10411-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70406">
    <skos:narrower rdf:resource="descriptor/10517-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70406">
    <skos:narrower rdf:resource="descriptor/10437-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70406">
    <skos:narrower rdf:resource="descriptor/10415-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70406">
    <skos:narrower rdf:resource="descriptor/19242-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20967-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10619-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14384-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18891-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14486-6">
    <skos:related rdf:resource="descriptor/14488-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16324-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15075-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14981-3">
    <skos:altLabel xml:lang="de">Sekt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14981-3">
    <skos:altLabel xml:lang="de">Champagner</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70959">
    <skos:prefLabel xml:lang="en">W.10.01.03  Consumer goods trade</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70959">
    <skos:prefLabel xml:lang="de">W.10.01.03  Konsumgüterhandel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15606-3">
    <skos:scopeNote xml:lang="de">Für die konkrete Ausstattung mit DV-Technik BENUTZE Informationstechnik. Vgl. auch: Automation, Computergestütztes Verfahren, Mikroelektronik und Informationssystem.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15606-3">
    <skos:scopeNote xml:lang="de">Nur für allgemeine Literatur über Einsatz und Auswirkungen der DV.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11043-6">
    <skos:altLabel xml:lang="en">Hybrid economy</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11043-6">
    <skos:altLabel xml:lang="de">Dritter Weg</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18369-4">
    <skos:altLabel xml:lang="en">Cross-border leasing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18369-4">
    <skos:altLabel xml:lang="en">Cross border leasing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18369-4">
    <skos:altLabel xml:lang="de">Grenzüberschreitendes Leasing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18369-4">
    <skos:altLabel xml:lang="de">Export-Leasing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18155-2">
    <skos:prefLabel xml:lang="en">Rent seeking</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18155-2">
    <skos:prefLabel xml:lang="de">Rent Seeking</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15126-3">
    <skos:broader rdf:resource="thsys/72183"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15126-3">
    <skos:broader rdf:resource="descriptor/15064-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70530">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17016-0">
    <skos:prefLabel xml:lang="en">Southern Europe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17016-0">
    <skos:prefLabel xml:lang="de">Südeuropa</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18062-2">
    <skos:prefLabel xml:lang="en">National unity</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18062-2">
    <skos:prefLabel xml:lang="de">Nationale Einheit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13678-2">
    <skos:altLabel xml:lang="de">Fernsprechwahlnetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13678-2">
    <skos:altLabel xml:lang="de">Fernsprechnetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18336-5">
    <skos:prefLabel xml:lang="en">Reforestation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18336-5">
    <skos:prefLabel xml:lang="de">Aufforstung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16749-4">
    <skos:related rdf:resource="descriptor/19650-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16749-4">
    <skos:related rdf:resource="descriptor/10489-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16749-4">
    <skos:related rdf:resource="descriptor/19641-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16749-4">
    <skos:related rdf:resource="descriptor/15627-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70833">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.26.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13673-5">
    <skos:prefLabel xml:lang="en">Word processing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13673-5">
    <skos:prefLabel xml:lang="de">Textverarbeitung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18947-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14196-1">
    <skos:related rdf:resource="descriptor/12978-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14196-1">
    <skos:related rdf:resource="descriptor/19386-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14196-1">
    <skos:related rdf:resource="descriptor/18371-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14196-1">
    <skos:related rdf:resource="descriptor/19385-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14196-1">
    <skos:related rdf:resource="descriptor/12983-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19054-3">
    <skos:altLabel xml:lang="de">Sprenggewerbe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19054-3">
    <skos:altLabel xml:lang="de">Erdbewegungsarbeiten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19054-3">
    <skos:altLabel xml:lang="de">Abbruchgewerbe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11487-5">
    <skos:broader rdf:resource="thsys/70305"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11487-5">
    <skos:broader rdf:resource="descriptor/10382-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19166-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14496-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13010-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21315-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10719-1">
    <skos:related rdf:resource="descriptor/10722-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10719-1">
    <skos:related rdf:resource="descriptor/10750-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10510-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19587-3">
    <skos:narrower rdf:resource="descriptor/10373-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19121-0">
    <skos:altLabel xml:lang="de">Betriebliche Marketinglehre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10058-5">
    <skos:prefLabel xml:lang="en">Social costs</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10058-5">
    <skos:prefLabel xml:lang="de">Soziale Kosten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19692-6">
    <skos:related rdf:resource="descriptor/16357-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19692-6">
    <skos:related rdf:resource="descriptor/16354-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19692-6">
    <skos:related rdf:resource="descriptor/18726-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19692-6">
    <skos:related rdf:resource="descriptor/10107-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19692-6">
    <skos:related rdf:resource="descriptor/10089-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11685-1">
    <skos:broader rdf:resource="thsys/71076"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11685-1">
    <skos:broader rdf:resource="descriptor/11696-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12120-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10500-2">
    <skos:broader rdf:resource="thsys/73387"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10500-2">
    <skos:broader rdf:resource="thsys/71110"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10500-2">
    <skos:broader rdf:resource="thsys/71005"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10500-2">
    <skos:broader rdf:resource="thsys/70109"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10500-2">
    <skos:broader rdf:resource="thsys/70091"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10500-2">
    <skos:broader rdf:resource="descriptor/15706-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10500-2">
    <skos:broader rdf:resource="descriptor/10513-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10196-0">
    <skos:broader rdf:resource="thsys/71119"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10196-0">
    <skos:broader rdf:resource="thsys/70178"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10196-0">
    <skos:broader rdf:resource="descriptor/10201-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18749-1">
    <skos:altLabel xml:lang="de">Wohnhaus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18749-1">
    <skos:altLabel xml:lang="de">Wohngebäude</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13969-5">
    <skos:narrower rdf:resource="descriptor/13282-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10658-2">
    <skos:broader rdf:resource="thsys/70058"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15460-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11696-3">
    <skos:related rdf:resource="descriptor/10911-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11696-3">
    <skos:related rdf:resource="descriptor/12196-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11696-3">
    <skos:related rdf:resource="descriptor/11704-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11696-3">
    <skos:related rdf:resource="descriptor/12195-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11696-3">
    <skos:related rdf:resource="descriptor/11904-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19396-3">
    <skos:prefLabel xml:lang="en">Electronic equipment market</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19396-3">
    <skos:prefLabel xml:lang="de">Elektronikmarkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14420-1">
    <skos:prefLabel xml:lang="en">Helicopter</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14420-1">
    <skos:prefLabel xml:lang="de">Hubschrauber</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14814-1">
    <skos:broader rdf:resource="thsys/73354"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14814-1">
    <skos:broader rdf:resource="thsys/70265"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14814-1">
    <skos:broader rdf:resource="descriptor/14812-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14947-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11067-6">
    <skos:related rdf:resource="descriptor/16402-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11067-6">
    <skos:related rdf:resource="descriptor/16226-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11067-6">
    <skos:related rdf:resource="descriptor/18073-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19823-3">
    <skos:prefLabel xml:lang="en">Enjoyment of rights</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19823-3">
    <skos:prefLabel xml:lang="de">Nutzungsrechte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12215-5">
    <skos:prefLabel xml:lang="en">Financial management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12215-5">
    <skos:prefLabel xml:lang="de">Betriebliche Finanzwirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12420-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11555-0">
    <skos:altLabel xml:lang="en">Health care expenditure</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11555-0">
    <skos:altLabel xml:lang="de">Öffentliche Gesundheitsausgaben</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11555-0">
    <skos:altLabel xml:lang="de">Kostendämpfung im Gesundheitswesen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11555-0">
    <skos:altLabel xml:lang="de">Gesundheitsausgaben</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13989-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16959-0">
    <skos:prefLabel xml:lang="en">Berne (Canton)</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16959-0">
    <skos:prefLabel xml:lang="de">Bern (Kanton)</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70988">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10162-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71007">
    <skos:broader rdf:resource="thsys/71009"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19505-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19626-5">
    <skos:altLabel xml:lang="de">Fischweg</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19626-5">
    <skos:altLabel xml:lang="de">Fischtreppe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19626-5">
    <skos:altLabel xml:lang="de">Fischschonzeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19626-5">
    <skos:altLabel xml:lang="de">Fischpass</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19626-5">
    <skos:altLabel xml:lang="de">Fischpaß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19235-6">
    <skos:related rdf:resource="descriptor/10985-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19235-6">
    <skos:related rdf:resource="descriptor/19310-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19235-6">
    <skos:related rdf:resource="descriptor/16416-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19235-6">
    <skos:related rdf:resource="descriptor/16299-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19235-6">
    <skos:related rdf:resource="descriptor/10816-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19235-6">
    <skos:related rdf:resource="descriptor/10572-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19235-6">
    <skos:related rdf:resource="descriptor/10823-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19235-6">
    <skos:related rdf:resource="descriptor/10515-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19235-6">
    <skos:related rdf:resource="descriptor/19315-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18577-4">
    <skos:broader rdf:resource="thsys/70939"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18577-4">
    <skos:broader rdf:resource="thsys/70938"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18577-4">
    <skos:broader rdf:resource="descriptor/18578-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19013-3">
    <skos:altLabel xml:lang="en">Mental health</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19013-3">
    <skos:altLabel xml:lang="de">Seelische Störung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19013-3">
    <skos:altLabel xml:lang="de">Psychotherapie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19013-3">
    <skos:altLabel xml:lang="de">Psychopathologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19013-3">
    <skos:altLabel xml:lang="de">Psychische Krankheit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19013-3">
    <skos:altLabel xml:lang="de">Psychiatrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19013-3">
    <skos:altLabel xml:lang="de">Neurose</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18426-4">
    <skos:scopeNote xml:lang="de">Nur verwenden für Forschung und Entwicklung der Elektrotechnik selbst.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13194-3">
    <skos:narrower rdf:resource="descriptor/13214-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13320-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10893-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11880-3">
    <skos:broader rdf:resource="thsys/71068"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11880-3">
    <skos:broader rdf:resource="descriptor/11875-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14371-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12916-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18165-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12399-4">
    <skos:related rdf:resource="descriptor/12299-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19823-3">
    <skos:scopeNote xml:lang="de">Für die Regelungen zur Nutzung von geistigem Eigentum BENUTZE Immaterialgüterrechte oder spezifischere Deskriptoren.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19823-3">
    <skos:scopeNote xml:lang="de">Nur im Sinne des Bürgerlichen Rechts. Betreffen den Gebrauch einer Sache/eines Rechts oder die Nutzung von Erträgen einer Sache/eines Rechts.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14483-5">
    <skos:prefLabel xml:lang="en">Cable and wire</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14483-5">
    <skos:prefLabel xml:lang="de">Kabel und Draht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10575-6">
    <skos:prefLabel xml:lang="en">Rural development</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10575-6">
    <skos:prefLabel xml:lang="de">Ländliche Entwicklung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11965-2">
    <skos:scopeNote xml:lang="de">Statistiken mit Gemeindedaten.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17923-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14184-1">
    <skos:prefLabel xml:lang="en">Primary energy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14184-1">
    <skos:prefLabel xml:lang="de">Primärenergieträger</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13815-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16357-0">
    <skos:related rdf:resource="descriptor/15983-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16357-0">
    <skos:related rdf:resource="descriptor/19701-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16357-0">
    <skos:related rdf:resource="descriptor/16354-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16357-0">
    <skos:related rdf:resource="descriptor/19692-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12210-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19339-1">
    <skos:broader rdf:resource="thsys/71066"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19339-1">
    <skos:broader rdf:resource="thsys/71023"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19339-1">
    <skos:broader rdf:resource="descriptor/12015-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12094-5">
    <skos:scopeNote xml:lang="de">Für betriebliche Anpassungsmaßnahmen der Organisationsstruktur.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70384">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">B.07.03</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10668-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13667-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18258-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16226-1">
    <skos:prefLabel xml:lang="en">Participation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16226-1">
    <skos:prefLabel xml:lang="de">Partizipation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12711-0">
    <skos:related rdf:resource="descriptor/15677-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12711-0">
    <skos:related rdf:resource="descriptor/12705-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12711-0">
    <skos:related rdf:resource="descriptor/18082-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12711-0">
    <skos:related rdf:resource="descriptor/10464-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12711-0">
    <skos:related rdf:resource="descriptor/10443-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12711-0">
    <skos:related rdf:resource="descriptor/19660-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11436-1">
    <skos:broader rdf:resource="thsys/70055"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11436-1">
    <skos:broader rdf:resource="descriptor/10764-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10022-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18269-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13992-3">
    <skos:scopeNote xml:lang="de">Hier auch Literatur zur Entrepreneurship-Theorie.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19828-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19591-5">
    <skos:broader rdf:resource="thsys/70269"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19591-5">
    <skos:broader rdf:resource="descriptor/15045-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12030-3">
    <skos:prefLabel xml:lang="en">Real estate leasing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12030-3">
    <skos:prefLabel xml:lang="de">Immobilien-Leasing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16868-3">
    <skos:broader rdf:resource="thsys/70361"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16868-3">
    <skos:broader rdf:resource="descriptor/18012-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12571-4">
    <skos:broader rdf:resource="thsys/73385"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12571-4">
    <skos:broader rdf:resource="thsys/70244"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12571-4">
    <skos:broader rdf:resource="descriptor/12573-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13076-2">
    <skos:prefLabel xml:lang="en">Electrical industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13076-2">
    <skos:prefLabel xml:lang="de">Elektroindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70891">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18505-1">
    <skos:narrower rdf:resource="descriptor/13848-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19323-2">
    <skos:altLabel xml:lang="de">Niederlassungsfreiheit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19323-2">
    <skos:altLabel xml:lang="de">Niederlassungsabkommen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19498-2">
    <skos:related rdf:resource="descriptor/11284-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19498-2">
    <skos:related rdf:resource="descriptor/11273-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16727-0">
    <skos:broader rdf:resource="thsys/73393"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16727-0">
    <skos:broader rdf:resource="descriptor/15685-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11151-3">
    <skos:broader rdf:resource="thsys/71101"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11151-3">
    <skos:broader rdf:resource="thsys/71024"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11151-3">
    <skos:broader rdf:resource="descriptor/12945-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13394-2">
    <skos:related rdf:resource="descriptor/13424-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13394-2">
    <skos:related rdf:resource="descriptor/16550-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13394-2">
    <skos:related rdf:resource="descriptor/11852-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11460-4">
    <skos:broader rdf:resource="thsys/70055"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14885-6">
    <skos:broader rdf:resource="thsys/70066"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14885-6">
    <skos:broader rdf:resource="descriptor/14846-2"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id288">
    <rdf:value xml:lang="de">Für nationale und internationale seerechtliche Regelungen</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13992-3">
    <skos:related rdf:resource="descriptor/18115-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13992-3">
    <skos:related rdf:resource="descriptor/12070-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13992-3">
    <skos:related rdf:resource="descriptor/12072-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13992-3">
    <skos:related rdf:resource="descriptor/18083-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13992-3">
    <skos:related rdf:resource="descriptor/25743-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13992-3">
    <skos:related rdf:resource="descriptor/12064-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13992-3">
    <skos:related rdf:resource="descriptor/12085-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13992-3">
    <skos:related rdf:resource="descriptor/11046-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13992-3">
    <skos:related rdf:resource="descriptor/20948-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13992-3">
    <skos:related rdf:resource="descriptor/11302-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13992-3">
    <skos:related rdf:resource="descriptor/13871-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13992-3">
    <skos:related rdf:resource="descriptor/12178-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13992-3">
    <skos:related rdf:resource="descriptor/19161-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13992-3">
    <skos:related rdf:resource="descriptor/11304-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70028">
    <skos:broader rdf:resource="thsys/b"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12925-2">
    <skos:prefLabel xml:lang="en">Defence industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12925-2">
    <skos:prefLabel xml:lang="de">Rüstungsindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id84">
    <rdfs:seeAlso rdf:resource="descriptor/18072-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70373">
    <rdfs:label xml:lang="en">Audiovisual product</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70373">
    <rdfs:label xml:lang="de">Unterhaltungselektronik</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13965-6">
    <skos:prefLabel xml:lang="en">Banking personnel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13965-6">
    <skos:prefLabel xml:lang="de">Bankberufe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14379-0">
    <skos:related rdf:resource="descriptor/14822-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14379-0">
    <skos:related rdf:resource="descriptor/13006-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14379-0">
    <skos:related rdf:resource="descriptor/18411-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14379-0">
    <skos:related rdf:resource="descriptor/14702-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17978-5">
    <skos:narrower rdf:resource="descriptor/17753-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17978-5">
    <skos:narrower rdf:resource="descriptor/17754-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17978-5">
    <skos:narrower rdf:resource="descriptor/17636-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17978-5">
    <skos:narrower rdf:resource="descriptor/17677-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17978-5">
    <skos:narrower rdf:resource="descriptor/17634-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17978-5">
    <skos:narrower rdf:resource="descriptor/17632-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17978-5">
    <skos:narrower rdf:resource="descriptor/17749-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17978-5">
    <skos:narrower rdf:resource="descriptor/17748-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17978-5">
    <skos:narrower rdf:resource="descriptor/17630-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17978-5">
    <skos:narrower rdf:resource="descriptor/17628-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17978-5">
    <skos:narrower rdf:resource="descriptor/17627-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17978-5">
    <skos:narrower rdf:resource="descriptor/17626-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17978-5">
    <skos:narrower rdf:resource="descriptor/17624-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17978-5">
    <skos:narrower rdf:resource="descriptor/17621-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17978-5">
    <skos:narrower rdf:resource="descriptor/17618-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17978-5">
    <skos:narrower rdf:resource="descriptor/17616-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17978-5">
    <skos:narrower rdf:resource="descriptor/17746-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17978-5">
    <skos:narrower rdf:resource="descriptor/17745-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70822">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13219-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17628-4">
    <skos:broader rdf:resource="thsys/70253"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17628-4">
    <skos:broader rdf:resource="descriptor/17608-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17628-4">
    <skos:broader rdf:resource="descriptor/17978-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70292">
    <skos:prefLabel xml:lang="en">B.07.05  Consumer behaviour</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70292">
    <skos:prefLabel xml:lang="de">B.07.05  Konsumentenverhalten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13370-2">
    <skos:prefLabel xml:lang="en">Physicians</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13370-2">
    <skos:prefLabel xml:lang="de">Ärzte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13339-3">
    <skos:broader rdf:resource="thsys/70964"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13339-3">
    <skos:broader rdf:resource="thsys/70876"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13339-3">
    <skos:broader rdf:resource="descriptor/12910-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11299-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id186">
    <rdfs:seeAlso rdf:resource="descriptor/10505-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10074-0">
    <skos:prefLabel xml:lang="en">Fixed-price analysis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10074-0">
    <skos:prefLabel xml:lang="de">Festpreismodell</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13639-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19699-6">
    <skos:altLabel xml:lang="de">Wahlkampfkostenerstattung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18615-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13987-3">
    <skos:broader rdf:resource="thsys/73297"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13987-3">
    <skos:broader rdf:resource="thsys/71054"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13987-3">
    <skos:broader rdf:resource="thsys/70882"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13987-3">
    <skos:broader rdf:resource="descriptor/18535-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70884">
    <skos:broader rdf:resource="thsys/70889"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19716-4">
    <skos:prefLabel xml:lang="en">Foreign criticism</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19716-4">
    <skos:prefLabel xml:lang="de">Auslandskritik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18656-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17750-0">
    <skos:altLabel xml:lang="de">Nordafrikanische Staaten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17750-0">
    <skos:altLabel xml:lang="de">Nordafrikanische Länder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17750-0">
    <skos:altLabel xml:lang="de">Maghreb-Länder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17750-0">
    <skos:altLabel xml:lang="de">Maghreb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19661-3">
    <skos:prefLabel xml:lang="en">Computer-aided method</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19661-3">
    <skos:prefLabel xml:lang="de">Computergestütztes Verfahren</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18380-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16013-4">
    <skos:narrower rdf:resource="descriptor/16315-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70096">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13412-5">
    <skos:broader rdf:resource="thsys/73363"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13412-5">
    <skos:broader rdf:resource="thsys/70865"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18003-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10312-3">
    <skos:broader rdf:resource="thsys/70270"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10312-3">
    <skos:broader rdf:resource="thsys/70233"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10312-3">
    <skos:broader rdf:resource="descriptor/10327-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16382-1">
    <skos:altLabel xml:lang="de">Bürgerprotest</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13564-3">
    <skos:related rdf:resource="descriptor/13558-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20068-3">
    <skos:prefLabel xml:lang="en">Service provider</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20068-3">
    <skos:prefLabel xml:lang="de">Service Provider</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13378-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18699-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13534-5">
    <skos:broader rdf:resource="thsys/70933"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13534-5">
    <skos:broader rdf:resource="descriptor/13533-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73374">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10556-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12488-5">
    <skos:prefLabel xml:lang="en">Cost-of-sales accounting</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12488-5">
    <skos:prefLabel xml:lang="de">Umsatzkostenverfahren</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10183-2">
    <skos:broader rdf:resource="thsys/71112"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10183-2">
    <skos:broader rdf:resource="thsys/70493"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10183-2">
    <skos:broader rdf:resource="descriptor/10181-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19725-3">
    <skos:scopeNote xml:lang="de">Im weiteren Sinne interdisziplinäre Wissenschaft, die sich mit der Entwicklung des Militär- und Kriegswesens sowie seiner Stellung im politischen, gesellschaftlichen, wirtschaftlichen und technischen Gesamtbereich befasst. Im engeren Sinne die Lehre von der Kriegsführung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12649-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18556-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14341-4">
    <skos:related rdf:resource="descriptor/14408-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11667-3">
    <skos:broader rdf:resource="thsys/71074"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11667-3">
    <skos:broader rdf:resource="thsys/70937"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11667-3">
    <skos:broader rdf:resource="thsys/70926"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11667-3">
    <skos:broader rdf:resource="descriptor/11681-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11667-3">
    <skos:broader rdf:resource="descriptor/18579-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19037-3">
    <skos:altLabel xml:lang="de">Empirischer Test</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16531-3">
    <skos:broader rdf:resource="thsys/71050"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16531-3">
    <skos:broader rdf:resource="thsys/70838"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16531-3">
    <skos:broader rdf:resource="thsys/70832"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16531-3">
    <skos:broader rdf:resource="thsys/70573"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16531-3">
    <skos:broader rdf:resource="thsys/70458"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19116-0">
    <skos:prefLabel xml:lang="en">Lean production</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19116-0">
    <skos:prefLabel xml:lang="de">Lean Production</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13817-0">
    <skos:related rdf:resource="descriptor/18030-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13817-0">
    <skos:related rdf:resource="descriptor/12613-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13817-0">
    <skos:related rdf:resource="descriptor/12705-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13817-0">
    <skos:related rdf:resource="descriptor/16757-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11034-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18655-3">
    <skos:altLabel xml:lang="de">Videotext</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18655-3">
    <skos:altLabel xml:lang="de">Telematikdienst</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17177-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18522-1">
    <skos:prefLabel xml:lang="en">Consumer goods trade</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18522-1">
    <skos:prefLabel xml:lang="de">Konsumgüterhandel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18936-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18399-2">
    <skos:altLabel xml:lang="de">Leichtmetallbau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18628-6">
    <skos:related rdf:resource="descriptor/18623-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19745-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19516-5">
    <skos:related rdf:resource="descriptor/11222-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19516-5">
    <skos:related rdf:resource="descriptor/19916-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10143-0">
    <skos:altLabel xml:lang="en">Theory of job search</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10143-0">
    <skos:altLabel xml:lang="en">Labour market search theory</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10143-0">
    <skos:altLabel xml:lang="en">Labor market search theory</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18456-2">
    <skos:broader rdf:resource="thsys/70575"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12307-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19167-4">
    <skos:prefLabel xml:lang="en">South Asia</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19167-4">
    <skos:prefLabel xml:lang="de">Südasien</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13142-1">
    <skos:broader rdf:resource="thsys/70988"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13142-1">
    <skos:broader rdf:resource="descriptor/12993-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19711-0">
    <skos:related rdf:resource="descriptor/19708-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19711-0">
    <skos:related rdf:resource="descriptor/16439-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19711-0">
    <skos:related rdf:resource="descriptor/20826-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15672-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11457-0">
    <skos:altLabel xml:lang="en">Liquidity trap</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11457-0">
    <skos:altLabel xml:lang="de">Liquiditätsfalle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11457-0">
    <skos:altLabel xml:lang="de">Keynessche Liquiditätspräferenztheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11457-0">
    <skos:altLabel xml:lang="de">Keynesianische Liquiditätspräferenztheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11483-6">
    <skos:altLabel xml:lang="de">Sonderlombard</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11483-6">
    <skos:altLabel xml:lang="de">Lombardsatz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73370">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16292-2">
    <skos:narrower rdf:resource="descriptor/16436-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16292-2">
    <skos:narrower rdf:resource="descriptor/10469-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16292-2">
    <skos:narrower rdf:resource="descriptor/16305-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16292-2">
    <skos:narrower rdf:resource="descriptor/16366-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16292-2">
    <skos:narrower rdf:resource="descriptor/16342-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16292-2">
    <skos:narrower rdf:resource="descriptor/19682-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16292-2">
    <skos:narrower rdf:resource="descriptor/16384-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16292-2">
    <skos:narrower rdf:resource="descriptor/16246-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16292-2">
    <skos:narrower rdf:resource="descriptor/16245-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14060-5">
    <skos:altLabel xml:lang="en">Seasonings</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14060-5">
    <skos:altLabel xml:lang="de">Zimt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14060-5">
    <skos:altLabel xml:lang="de">Würzsoße</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14060-5">
    <skos:altLabel xml:lang="de">Würzmittel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14060-5">
    <skos:altLabel xml:lang="de">Vanille</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14060-5">
    <skos:altLabel xml:lang="de">Tomatenketschup</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14060-5">
    <skos:altLabel xml:lang="de">Tomatenketchup</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14060-5">
    <skos:altLabel xml:lang="de">Speisewürze</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14060-5">
    <skos:altLabel xml:lang="de">Senf</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14060-5">
    <skos:altLabel xml:lang="de">Pfeffer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14060-5">
    <skos:altLabel xml:lang="de">Muskat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14060-5">
    <skos:altLabel xml:lang="de">Mayonnaise</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14060-5">
    <skos:altLabel xml:lang="de">Marinade</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14060-5">
    <skos:altLabel xml:lang="de">Majonäse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14060-5">
    <skos:altLabel xml:lang="de">Ketschup</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14060-5">
    <skos:altLabel xml:lang="de">Ketchup</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14060-5">
    <skos:altLabel xml:lang="de">Kardamom</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14060-5">
    <skos:altLabel xml:lang="de">Ingwer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14060-5">
    <skos:altLabel xml:lang="de">Gewürzpflanze</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14060-5">
    <skos:altLabel xml:lang="de">Gewürznelke</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14060-5">
    <skos:altLabel xml:lang="de">Gewürzmischung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14060-5">
    <skos:altLabel xml:lang="de">Essig</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14060-5">
    <skos:altLabel xml:lang="de">Chutney</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14060-5">
    <skos:altLabel xml:lang="de">Brühwürfel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71097">
    <skos:narrower rdf:resource="descriptor/19326-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71097">
    <skos:narrower rdf:resource="descriptor/19327-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71097">
    <skos:narrower rdf:resource="descriptor/11573-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71097">
    <skos:narrower rdf:resource="descriptor/10619-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71097">
    <skos:narrower rdf:resource="descriptor/10625-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71097">
    <skos:narrower rdf:resource="descriptor/20110-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71097">
    <skos:narrower rdf:resource="descriptor/18064-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71097">
    <skos:narrower rdf:resource="descriptor/10586-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71097">
    <skos:narrower rdf:resource="descriptor/10621-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71097">
    <skos:narrower rdf:resource="descriptor/19329-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71097">
    <skos:narrower rdf:resource="descriptor/19328-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71097">
    <skos:narrower rdf:resource="descriptor/10969-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71097">
    <skos:narrower rdf:resource="descriptor/10618-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71097">
    <skos:narrower rdf:resource="descriptor/10603-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71097">
    <skos:narrower rdf:resource="descriptor/10745-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71097">
    <skos:narrower rdf:resource="descriptor/10452-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71097">
    <skos:narrower rdf:resource="descriptor/18790-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71097">
    <skos:narrower rdf:resource="descriptor/10616-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71097">
    <skos:narrower rdf:resource="descriptor/19324-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71097">
    <skos:narrower rdf:resource="descriptor/10698-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71097">
    <skos:narrower rdf:resource="descriptor/19958-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71097">
    <skos:narrower rdf:resource="descriptor/10566-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71097">
    <skos:narrower rdf:resource="descriptor/10610-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14354-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11929-6">
    <skos:broader rdf:resource="thsys/71070"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11929-6">
    <skos:broader rdf:resource="descriptor/10251-4"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id328">
    <rdfs:seeAlso rdf:resource="descriptor/18663-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11290-3">
    <skos:broader rdf:resource="thsys/71058"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14022-6">
    <skos:altLabel xml:lang="en">Floor covering</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16458-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14054-0">
    <zbwext:useInsteadNote rdf:nodeID="id8"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10438-2">
    <zbwext:useInsteadNote rdf:nodeID="id159"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13228-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70180">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10567-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19246-1">
    <skos:prefLabel xml:lang="en">Regional change</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19246-1">
    <skos:prefLabel xml:lang="de">Regionaler Strukturwandel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10000-1">
    <skos:scopeNote xml:lang="de">Nur im ökonomischen, nicht im juristischen Sinne.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13671-2">
    <skos:related rdf:resource="descriptor/13673-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13671-2">
    <skos:related rdf:resource="descriptor/13278-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13671-2">
    <skos:related rdf:resource="descriptor/13653-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13671-2">
    <skos:related rdf:resource="descriptor/18002-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13671-2">
    <skos:related rdf:resource="descriptor/18454-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13671-2">
    <skos:related rdf:resource="descriptor/14750-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13671-2">
    <skos:related rdf:resource="descriptor/18111-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13671-2">
    <skos:related rdf:resource="descriptor/19660-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17669-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12930-2">
    <skos:related rdf:resource="descriptor/18398-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12930-2">
    <skos:related rdf:resource="descriptor/18272-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12930-2">
    <skos:related rdf:resource="descriptor/18415-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12930-2">
    <skos:related rdf:resource="descriptor/14343-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12930-2">
    <skos:related rdf:resource="descriptor/14051-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12930-2">
    <skos:related rdf:resource="descriptor/13117-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12023-0">
    <skos:broader rdf:resource="thsys/71066"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12023-0">
    <skos:broader rdf:resource="thsys/70889"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12023-0">
    <skos:broader rdf:resource="descriptor/10021-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17978-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70052">
    <skos:prefLabel xml:lang="en">V.13  Labour</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70052">
    <skos:prefLabel xml:lang="de">V.13  Arbeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18218-4">
    <skos:broader rdf:resource="thsys/73302"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18218-4">
    <skos:broader rdf:resource="descriptor/14122-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10613-3">
    <skos:related rdf:resource="descriptor/11258-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10613-3">
    <skos:related rdf:resource="descriptor/15956-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10613-3">
    <skos:related rdf:resource="descriptor/19360-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10613-3">
    <skos:related rdf:resource="descriptor/19492-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10613-3">
    <skos:related rdf:resource="descriptor/18609-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10613-3">
    <skos:related rdf:resource="descriptor/19656-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10613-3">
    <skos:related rdf:resource="descriptor/11268-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18859-1">
    <skos:altLabel xml:lang="en">Human behaviour</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18859-1">
    <skos:altLabel xml:lang="en">Behavioural theory</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18859-1">
    <skos:altLabel xml:lang="en">Behavioural science</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18859-1">
    <skos:altLabel xml:lang="de">Verhaltenswissenschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18859-1">
    <skos:altLabel xml:lang="de">Verhaltenstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18859-1">
    <skos:altLabel xml:lang="de">Verhaltensforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18859-1">
    <skos:altLabel xml:lang="de">Verhaltensbiologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18859-1">
    <skos:altLabel xml:lang="de">Menschliches Verhalten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18859-1">
    <skos:altLabel xml:lang="de">Handlungstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18859-1">
    <skos:altLabel xml:lang="de">Behaviorismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18478-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id283">
    <rdfs:seeAlso rdf:resource="descriptor/11388-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18327-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14509-6">
    <skos:altLabel xml:lang="de">Tiefkühlgerät</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14509-6">
    <skos:altLabel xml:lang="de">Gefriergerät</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14212-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70832">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73332">
    <skos:broader rdf:resource="thsys/73333"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19799-2">
    <skos:related rdf:resource="descriptor/18783-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70905">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.14</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16076-1">
    <skos:related rdf:resource="descriptor/12868-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16076-1">
    <skos:related rdf:resource="descriptor/12196-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16076-1">
    <skos:related rdf:resource="descriptor/10897-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11215-3">
    <skos:broader rdf:resource="thsys/71059"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11215-3">
    <skos:broader rdf:resource="thsys/70924"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11215-3">
    <skos:broader rdf:resource="descriptor/11194-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11215-3">
    <skos:broader rdf:resource="descriptor/11187-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70046">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">B.08</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18890-0">
    <skos:narrower rdf:resource="descriptor/13375-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18890-0">
    <skos:narrower rdf:resource="descriptor/13370-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18890-0">
    <skos:narrower rdf:resource="descriptor/14005-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18495-6">
    <skos:altLabel xml:lang="de">Baustofftechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14301-2">
    <skos:prefLabel xml:lang="en">Manganese</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14301-2">
    <skos:prefLabel xml:lang="de">Mangan</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13633-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70269">
    <skos:prefLabel xml:lang="en">N.09  Mathematics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70269">
    <skos:prefLabel xml:lang="de">N.09  Mathematik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10977-0">
    <skos:broader rdf:resource="thsys/71102"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10977-0">
    <skos:broader rdf:resource="thsys/70141"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10977-0">
    <skos:broader rdf:resource="descriptor/10986-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70455">
    <skos:broader rdf:resource="thsys/70055"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10671-3">
    <skos:broader rdf:resource="thsys/71099"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10671-3">
    <skos:broader rdf:resource="descriptor/10678-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73389">
    <rdfs:label xml:lang="en">Traffic law</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73389">
    <rdfs:label xml:lang="de">Verkehrsrecht</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16072-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id231">
    <rdfs:seeAlso rdf:resource="descriptor/18969-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13418-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19286-3">
    <skos:prefLabel xml:lang="en">Autarchy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19286-3">
    <skos:prefLabel xml:lang="de">Autarkie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18991-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10247-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11547-6">
    <skos:related rdf:resource="descriptor/11637-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11547-6">
    <skos:related rdf:resource="descriptor/11694-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11547-6">
    <skos:related rdf:resource="descriptor/11642-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11547-6">
    <skos:related rdf:resource="descriptor/11640-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11547-6">
    <skos:related rdf:resource="descriptor/19405-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11547-6">
    <skos:related rdf:resource="descriptor/11635-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#note">
    <rdf:type rdf:resource="http://www.w3.org/1999/02/22-rdf-syntax-ns#Property"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#note">
    <rdf:type rdf:resource="http://www.w3.org/2002/07/owl#AnnotationProperty"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20189-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18630-5">
    <skos:broader rdf:resource="thsys/70926"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18630-5">
    <skos:broader rdf:resource="descriptor/11828-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71087">
    <skos:prefLabel xml:lang="en">V.07.08  Balance of payments</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71087">
    <skos:prefLabel xml:lang="de">V.07.08  Zahlungsbilanz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10908-5">
    <skos:related rdf:resource="descriptor/12713-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10908-5">
    <skos:related rdf:resource="descriptor/10909-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13304-1">
    <skos:related rdf:resource="descriptor/16033-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13304-1">
    <skos:related rdf:resource="descriptor/13839-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13304-1">
    <skos:related rdf:resource="descriptor/11840-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13304-1">
    <skos:related rdf:resource="descriptor/18755-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13304-1">
    <skos:related rdf:resource="descriptor/13837-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10787-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16782-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18337-3">
    <skos:prefLabel xml:lang="en">Deforestation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18337-3">
    <skos:prefLabel xml:lang="de">Abholzung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11448-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70928">
    <skos:narrower rdf:resource="descriptor/13590-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70928">
    <skos:narrower rdf:resource="descriptor/13580-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70928">
    <skos:narrower rdf:resource="descriptor/13587-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70928">
    <skos:narrower rdf:resource="descriptor/18625-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70928">
    <skos:narrower rdf:resource="descriptor/18187-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70928">
    <skos:narrower rdf:resource="descriptor/13588-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70928">
    <skos:narrower rdf:resource="descriptor/13584-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70928">
    <skos:narrower rdf:resource="descriptor/11829-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70928">
    <skos:narrower rdf:resource="descriptor/18624-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70928">
    <skos:narrower rdf:resource="descriptor/13591-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70928">
    <skos:narrower rdf:resource="descriptor/13610-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70928">
    <skos:narrower rdf:resource="thsys/70927"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70928">
    <skos:narrower rdf:resource="thsys/70926"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20189-5">
    <skos:prefLabel xml:lang="en">Earmarked tax</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20189-5">
    <skos:prefLabel xml:lang="de">Zweckgebundene Steuer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18880-3">
    <skos:related rdf:resource="descriptor/14017-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18880-3">
    <skos:related rdf:resource="descriptor/10032-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18880-3">
    <skos:related rdf:resource="descriptor/18871-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18880-3">
    <skos:related rdf:resource="descriptor/18780-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18880-3">
    <skos:related rdf:resource="descriptor/18881-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19174-0">
    <skos:broader rdf:resource="thsys/70307"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19174-0">
    <skos:broader rdf:resource="descriptor/10492-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15717-1">
    <skos:related rdf:resource="descriptor/16597-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15717-1">
    <skos:related rdf:resource="descriptor/16664-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15717-1">
    <skos:related rdf:resource="descriptor/15719-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15717-1">
    <skos:related rdf:resource="descriptor/15718-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15717-1">
    <skos:related rdf:resource="descriptor/15716-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15717-1">
    <skos:related rdf:resource="descriptor/15714-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15717-1">
    <skos:related rdf:resource="descriptor/19754-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15225-1">
    <skos:related rdf:resource="descriptor/15396-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15225-1">
    <skos:related rdf:resource="descriptor/15067-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15225-1">
    <skos:related rdf:resource="descriptor/19575-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15225-1">
    <skos:related rdf:resource="descriptor/15554-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12953-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10610-2">
    <skos:broader rdf:resource="thsys/71097"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10610-2">
    <skos:broader rdf:resource="descriptor/10573-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13953-6">
    <skos:altLabel xml:lang="de">Totengräber</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13953-6">
    <skos:altLabel xml:lang="de">Tiefbauberufe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13953-6">
    <skos:altLabel xml:lang="de">Straßenbauarbeiter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13953-6">
    <skos:altLabel xml:lang="de">Sprengmeister</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13953-6">
    <skos:altLabel xml:lang="de">Schleusenwärter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13953-6">
    <skos:altLabel xml:lang="de">Maurer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13953-6">
    <skos:altLabel xml:lang="de">Hochbauberufe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13953-6">
    <skos:altLabel xml:lang="de">Friedhofsarbeiter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13953-6">
    <skos:altLabel xml:lang="de">Bauhilfsberufe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13953-6">
    <skos:altLabel xml:lang="de">Bauarbeiter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18515-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73337">
    <skos:prefLabel xml:lang="de">N.05.05  Finanz- und Steuerrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11694-0">
    <skos:broader rdf:resource="thsys/71076"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11694-0">
    <skos:broader rdf:resource="descriptor/11637-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12376-2">
    <skos:narrower rdf:resource="descriptor/12423-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18538-0">
    <skos:related rdf:resource="descriptor/10656-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18538-0">
    <skos:related rdf:resource="descriptor/10864-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18538-0">
    <skos:related rdf:resource="descriptor/10836-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11536-4">
    <skos:altLabel xml:lang="en">Tax administration</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11536-4">
    <skos:altLabel xml:lang="de">Zollverwaltung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11536-4">
    <skos:altLabel xml:lang="de">Steuerverwaltung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11536-4">
    <skos:altLabel xml:lang="de">Schatzamt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11536-4">
    <skos:altLabel xml:lang="de">Finanzbehörde</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11536-4">
    <skos:altLabel xml:lang="de">Finanzamt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18969-1">
    <skos:broader rdf:resource="thsys/73357"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18969-1">
    <skos:broader rdf:resource="thsys/70814"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18969-1">
    <skos:broader rdf:resource="descriptor/16587-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14190-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12180-5">
    <skos:altLabel xml:lang="de">Genossenschaftswesen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12180-5">
    <skos:altLabel xml:lang="de">Eingetragene Genossenschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12702-1">
    <skos:narrower rdf:resource="descriptor/12733-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14211-3">
    <skos:related rdf:resource="descriptor/12940-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14211-3">
    <skos:related rdf:resource="descriptor/18984-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11290-3">
    <skos:prefLabel xml:lang="en">Occupation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11290-3">
    <skos:prefLabel xml:lang="de">Beruf</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14997-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18308-3">
    <skos:broader rdf:resource="thsys/70575"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18308-3">
    <skos:broader rdf:resource="thsys/70051"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18308-3">
    <skos:broader rdf:resource="descriptor/11782-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14043-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13060-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10349-1">
    <skos:altLabel xml:lang="en">Inventory growth cycle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10349-1">
    <skos:altLabel xml:lang="en">Inventory fluctuations</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12571-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13290-0">
    <skos:prefLabel xml:lang="en">Savings bank</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13290-0">
    <skos:prefLabel xml:lang="de">Sparkasse</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12331-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15258-0">
    <skos:prefLabel xml:lang="en">Statistical inference</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15258-0">
    <skos:prefLabel xml:lang="de">Inferenzstatistik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19706-0">
    <skos:altLabel xml:lang="de">Staat und Kirche</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19706-0">
    <skos:altLabel xml:lang="de">Religionspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19706-0">
    <skos:altLabel xml:lang="de">Kirche und Staat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19706-0">
    <skos:altLabel xml:lang="de">Antireligiöse Politik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11586-3">
    <skos:related rdf:resource="descriptor/11613-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11586-3">
    <skos:related rdf:resource="descriptor/11731-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10008-6">
    <skos:broader rdf:resource="thsys/71044"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10008-6">
    <skos:broader rdf:resource="thsys/70530"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19587-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id6">
    <rdf:value xml:lang="de">Für marktwirtschaftlich organisierte Wirtschaftssysteme</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18922-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19731-1">
    <skos:prefLabel xml:lang="en">Philosophy of religion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19731-1">
    <skos:prefLabel xml:lang="de">Religionsphilosophie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18569-3">
    <skos:related rdf:resource="descriptor/13255-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18569-3">
    <skos:related rdf:resource="descriptor/18572-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16692-0">
    <skos:related rdf:resource="descriptor/16687-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16692-0">
    <skos:related rdf:resource="descriptor/12110-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16692-0">
    <skos:related rdf:resource="descriptor/16603-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16692-0">
    <skos:related rdf:resource="descriptor/18484-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16692-0">
    <skos:related rdf:resource="descriptor/11198-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16692-0">
    <skos:related rdf:resource="descriptor/12913-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16692-0">
    <skos:related rdf:resource="descriptor/12602-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16692-0">
    <skos:related rdf:resource="descriptor/11309-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16692-0">
    <skos:related rdf:resource="descriptor/12617-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16692-0">
    <skos:related rdf:resource="descriptor/11314-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13501-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15376-1">
    <skos:prefLabel xml:lang="en">Econometric Model</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15376-1">
    <skos:prefLabel xml:lang="de">Ökonometrisches Modell</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70899">
    <skos:broader rdf:resource="thsys/70900"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11032-4">
    <skos:broader rdf:resource="thsys/72181"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11032-4">
    <skos:broader rdf:resource="thsys/70814"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11032-4">
    <skos:broader rdf:resource="descriptor/15108-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11032-4">
    <skos:broader rdf:resource="descriptor/15102-3"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id28">
    <rdfs:seeAlso rdf:resource="descriptor/13376-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10424-6">
    <skos:broader rdf:resource="thsys/70406"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10424-6">
    <skos:broader rdf:resource="descriptor/10434-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13109-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12250-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11779-6">
    <skos:narrower rdf:resource="descriptor/11778-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11779-6">
    <skos:narrower rdf:resource="descriptor/10401-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11779-6">
    <skos:narrower rdf:resource="descriptor/11770-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11779-6">
    <skos:narrower rdf:resource="descriptor/10380-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20888-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11378-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19626-5">
    <skos:scopeNote xml:lang="de">Erhaltung der Fischbestände</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10111-6">
    <skos:broader rdf:resource="thsys/71033"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10111-6">
    <skos:broader rdf:resource="thsys/70209"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10111-6">
    <skos:broader rdf:resource="thsys/70057"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10111-6">
    <skos:broader rdf:resource="descriptor/10094-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16271-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13387-6">
    <skos:broader rdf:resource="thsys/71037"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13387-6">
    <skos:broader rdf:resource="thsys/70882"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13387-6">
    <skos:broader rdf:resource="descriptor/13379-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13588-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16749-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18304-4">
    <skos:prefLabel xml:lang="en">Food packaging</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18304-4">
    <skos:prefLabel xml:lang="de">Lebensmittelverpackung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18059-5">
    <skos:narrower rdf:resource="descriptor/18770-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18059-5">
    <skos:narrower rdf:resource="descriptor/10851-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13684-0">
    <skos:narrower rdf:resource="descriptor/18656-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11394-5">
    <skos:broader rdf:resource="thsys/70878"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11394-5">
    <skos:broader rdf:resource="thsys/70847"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11394-5">
    <skos:broader rdf:resource="thsys/70176"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11394-5">
    <skos:broader rdf:resource="thsys/70143"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11394-5">
    <skos:broader rdf:resource="thsys/70013"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11394-5">
    <skos:broader rdf:resource="descriptor/16554-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15901-1">
    <skos:altLabel xml:lang="en">Hurricane</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15901-1">
    <skos:altLabel xml:lang="en">Blizzard</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15901-1">
    <skos:altLabel xml:lang="de">Wirbelsturm</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15901-1">
    <skos:altLabel xml:lang="de">Wind</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15901-1">
    <skos:altLabel xml:lang="de">Tornado</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15901-1">
    <skos:altLabel xml:lang="de">Taifun</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15901-1">
    <skos:altLabel xml:lang="de">Orkan</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15901-1">
    <skos:altLabel xml:lang="de">Hurrikan</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15718-6">
    <skos:prefLabel xml:lang="en">Social inequality</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15718-6">
    <skos:prefLabel xml:lang="de">Soziale Ungleichheit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12602-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18232-3">
    <skos:prefLabel xml:lang="en">Drinking water</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18232-3">
    <skos:prefLabel xml:lang="de">Trinkwasser</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10803-5">
    <skos:broader rdf:resource="thsys/70187"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10619-5">
    <skos:altLabel xml:lang="de">Internationale Wirtschaftspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11069-2">
    <skos:broader rdf:resource="thsys/73390"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11069-2">
    <skos:broader rdf:resource="thsys/70039"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19466-1">
    <skos:broader rdf:resource="thsys/70924"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19466-1">
    <skos:broader rdf:resource="thsys/70878"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19466-1">
    <skos:broader rdf:resource="thsys/70013"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19466-1">
    <skos:broader rdf:resource="descriptor/16554-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70187">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11574-3">
    <skos:prefLabel xml:lang="en">Housing benefit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11574-3">
    <skos:prefLabel xml:lang="de">Wohngeld</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10942-5">
    <zbwext:useInsteadNote rdf:nodeID="id364"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13515-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12424-3">
    <skos:scopeNote xml:lang="de">Nur für Bewertung gemäß Bilanzrecht.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13424-5">
    <skos:altLabel xml:lang="en">Municipal association</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13424-5">
    <skos:altLabel xml:lang="de">Kommunaler Zweckverband</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13424-5">
    <skos:altLabel xml:lang="de">Gemeindezweckverband</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70999">
    <skos:broader rdf:resource="thsys/71004"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11039-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14640-1">
    <skos:broader rdf:resource="thsys/70463"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14640-1">
    <skos:broader rdf:resource="descriptor/18254-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19169-0">
    <skos:broader rdf:resource="thsys/70128"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19169-0">
    <skos:broader rdf:resource="descriptor/17948-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13720-1">
    <skos:broader rdf:resource="thsys/71104"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13720-1">
    <skos:broader rdf:resource="thsys/71103"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13720-1">
    <skos:broader rdf:resource="thsys/71088"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13720-1">
    <skos:broader rdf:resource="descriptor/10221-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13720-1">
    <skos:broader rdf:resource="descriptor/10779-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13650-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19207-4">
    <skos:prefLabel xml:lang="en">Mathematical economics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19207-4">
    <skos:prefLabel xml:lang="de">Mathematische Ökonomie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18017-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12591-5">
    <skos:narrower rdf:resource="descriptor/12291-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12591-5">
    <skos:narrower rdf:resource="descriptor/18105-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18722-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19177-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70958">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19466-1">
    <skos:prefLabel xml:lang="en">Tourism research</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19466-1">
    <skos:prefLabel xml:lang="de">Tourismusforschung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16266-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70992">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14357-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24624-0">
    <skos:related rdf:resource="descriptor/12070-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24624-0">
    <skos:related rdf:resource="descriptor/12262-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13312-2">
    <skos:altLabel xml:lang="de">Wohnheim</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13312-2">
    <skos:altLabel xml:lang="de">Tagesheim</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13312-2">
    <skos:altLabel xml:lang="de">Pflegeheim</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13312-2">
    <skos:altLabel xml:lang="de">Freizeitheim</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13312-2">
    <skos:altLabel xml:lang="de">Ferienheim</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17804-3">
    <skos:prefLabel xml:lang="en">Trinidad and Tobago</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17804-3">
    <skos:prefLabel xml:lang="de">Trinidad und Tobago</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13767-3">
    <skos:altLabel xml:lang="en">Tratte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13767-3">
    <skos:altLabel xml:lang="de">Wechselkredit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13767-3">
    <skos:altLabel xml:lang="de">Wechseldiskontkredit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13767-3">
    <skos:altLabel xml:lang="de">Solawechsel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13767-3">
    <skos:altLabel xml:lang="de">Rediskontkredit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13767-3">
    <skos:altLabel xml:lang="de">Diskontkredit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13767-3">
    <skos:altLabel xml:lang="de">Akzeptkredit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19221-3">
    <skos:prefLabel xml:lang="en">Private transfer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19221-3">
    <skos:prefLabel xml:lang="de">Privater Transfer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15727-5">
    <skos:related rdf:resource="descriptor/15774-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15727-5">
    <skos:related rdf:resource="descriptor/12976-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15727-5">
    <skos:related rdf:resource="descriptor/15731-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15727-5">
    <skos:related rdf:resource="descriptor/13325-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15727-5">
    <skos:related rdf:resource="descriptor/15988-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15727-5">
    <skos:related rdf:resource="descriptor/24621-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15727-5">
    <skos:related rdf:resource="descriptor/15009-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12212-4">
    <skos:scopeNote xml:lang="de">Theorie von H. M. Markowitz zur optimalen Zusammenstellung von Wertpapierportefeuilles. Der Ansatz wurde inzwischen auf weitere Bereiche ausgedehnt und findet besonders im strategischen Management allgemeine Anwendung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12617-6">
    <skos:broader rdf:resource="thsys/73385"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12617-6">
    <skos:broader rdf:resource="thsys/71050"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12617-6">
    <skos:broader rdf:resource="thsys/70838"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12617-6">
    <skos:broader rdf:resource="thsys/70562"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12617-6">
    <skos:broader rdf:resource="descriptor/15836-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12617-6">
    <skos:broader rdf:resource="descriptor/15811-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16038-2">
    <skos:related rdf:resource="descriptor/10984-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16038-2">
    <skos:related rdf:resource="descriptor/10968-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16038-2">
    <skos:related rdf:resource="descriptor/19503-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16038-2">
    <skos:related rdf:resource="descriptor/16050-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16038-2">
    <skos:related rdf:resource="descriptor/16031-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16038-2">
    <skos:related rdf:resource="descriptor/18831-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16038-2">
    <skos:related rdf:resource="descriptor/16026-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17255-3">
    <skos:altLabel xml:lang="x-other">Republique Francaise</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17255-3">
    <skos:altLabel xml:lang="de">Französische Republik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13649-2">
    <skos:altLabel xml:lang="de">Videotelefon</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13649-2">
    <skos:altLabel xml:lang="de">Fernsehtelefon</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13649-2">
    <skos:altLabel xml:lang="de">Bildtelephon</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13649-2">
    <skos:altLabel xml:lang="de">Bildfernsprechen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#broader">
    <rdf:type rdf:resource="http://www.w3.org/1999/02/22-rdf-syntax-ns#Property"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#broader">
    <rdf:type rdf:resource="http://www.w3.org/2002/07/owl#ObjectProperty"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17962-6">
    <skos:broader rdf:resource="thsys/70198"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17962-6">
    <skos:broader rdf:resource="descriptor/17954-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/p">
    <skos:prefLabel xml:lang="en">P  Commodities</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/p">
    <skos:prefLabel xml:lang="de">P  Produkte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11484-4">
    <skos:altLabel xml:lang="de">Mindestreservesatz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11484-4">
    <skos:altLabel xml:lang="de">Bardepot</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18013-1">
    <skos:prefLabel xml:lang="en">Service industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18013-1">
    <skos:prefLabel xml:lang="de">Dienstleistungssektor</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13429-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14642-4">
    <skos:broader rdf:resource="thsys/70463"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14642-4">
    <skos:broader rdf:resource="descriptor/14599-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18194-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10353-3">
    <skos:prefLabel xml:lang="en">Political business cycle</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10353-3">
    <skos:prefLabel xml:lang="de">Politischer Konjunkturzyklus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12986-3">
    <skos:broader rdf:resource="thsys/71009"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12986-3">
    <skos:broader rdf:resource="descriptor/12980-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18732-4">
    <skos:related rdf:resource="descriptor/12265-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18732-4">
    <skos:related rdf:resource="descriptor/13785-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19430-1">
    <skos:broader rdf:resource="thsys/70242"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19430-1">
    <skos:broader rdf:resource="descriptor/10956-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19579-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15327-0">
    <skos:prefLabel xml:lang="en">ARMA model</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15327-0">
    <skos:prefLabel xml:lang="de">ARMA-Modell</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16633-2">
    <skos:prefLabel xml:lang="en">Social consciousness</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16633-2">
    <skos:prefLabel xml:lang="de">Gesellschaftliches Bewusstsein</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18254-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11066-1">
    <skos:altLabel xml:lang="en">Spontaneous order</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11066-1">
    <skos:altLabel xml:lang="de">Wirtschaftsordnungstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11066-1">
    <skos:altLabel xml:lang="de">Theorie der Wirtschaftsorganisation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11066-1">
    <skos:altLabel xml:lang="de">Theorie der Wirtschaftsordnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11066-1">
    <skos:altLabel xml:lang="de">Spontane Ordnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13352-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16970-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16140-4">
    <skos:altLabel xml:lang="de">Kinderarbeitsschutzgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16140-4">
    <skos:altLabel xml:lang="de">Kinderarbeitsschutz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16140-4">
    <skos:altLabel xml:lang="de">Jugendarbeitsschutzgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16140-4">
    <skos:altLabel xml:lang="de">Gesetz zum Schutze der arbeitenden Jugend</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14807-5">
    <skos:prefLabel xml:lang="en">Stationery</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14807-5">
    <skos:prefLabel xml:lang="de">Schreibwaren</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18150-5">
    <skos:prefLabel xml:lang="en">Natural rate of unemployment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18150-5">
    <skos:prefLabel xml:lang="de">Natürliche Arbeitslosigkeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24783-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17946-4">
    <skos:broader rdf:resource="thsys/70128"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17946-4">
    <skos:broader rdf:resource="descriptor/17948-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11600-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19758-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18889-6">
    <skos:related rdf:resource="descriptor/18773-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73336">
    <skos:broader rdf:resource="thsys/70154"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16515-1">
    <skos:related rdf:resource="descriptor/16487-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16515-1">
    <skos:related rdf:resource="descriptor/16483-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16515-1">
    <skos:related rdf:resource="descriptor/19749-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16515-1">
    <skos:related rdf:resource="descriptor/16512-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16515-1">
    <skos:related rdf:resource="descriptor/13662-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16515-1">
    <skos:related rdf:resource="descriptor/18920-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16515-1">
    <skos:related rdf:resource="descriptor/18923-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16515-1">
    <skos:related rdf:resource="descriptor/19781-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16515-1">
    <skos:related rdf:resource="descriptor/18922-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16515-1">
    <skos:related rdf:resource="descriptor/16648-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16515-1">
    <skos:related rdf:resource="descriptor/15654-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16515-1">
    <skos:related rdf:resource="descriptor/11397-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11400-1">
    <skos:altLabel xml:lang="en">Equal opportunities in education</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11400-1">
    <skos:altLabel xml:lang="de">Gleiche Bildungschancen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11400-1">
    <skos:altLabel xml:lang="de">Chancengleichheit in der Bildung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11400-1">
    <skos:altLabel xml:lang="de">Bildungszugang</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11400-1">
    <skos:altLabel xml:lang="de">Bildungsungleichheit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11400-1">
    <skos:altLabel xml:lang="de">Ausbildungschancen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="http://zbw.eu/namespaces/skos-extensions/">
    <dcterms:modified>2009-03-16</dcterms:modified>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16477-4">
    <skos:related rdf:resource="descriptor/18864-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16477-4">
    <skos:related rdf:resource="descriptor/16480-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16477-4">
    <skos:related rdf:resource="descriptor/18142-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16477-4">
    <skos:related rdf:resource="descriptor/16030-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16477-4">
    <skos:related rdf:resource="descriptor/15938-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16477-4">
    <skos:related rdf:resource="descriptor/15911-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16477-4">
    <skos:related rdf:resource="descriptor/11406-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16477-4">
    <skos:related rdf:resource="descriptor/16475-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16972-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10491-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15737-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13611-6">
    <skos:related rdf:resource="descriptor/13622-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71062">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.13.01.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12725-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16342-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73369">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14229-5">
    <skos:altLabel xml:lang="de">Anhydrit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14229-5">
    <skos:altLabel xml:lang="de">Alabaster</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19213-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16278-3">
    <skos:narrower rdf:resource="descriptor/10627-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16278-3">
    <skos:narrower rdf:resource="descriptor/18062-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16278-3">
    <skos:narrower rdf:resource="descriptor/19681-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16278-3">
    <skos:narrower rdf:resource="descriptor/16495-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12023-0">
    <skos:related rdf:resource="descriptor/12031-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12023-0">
    <skos:related rdf:resource="descriptor/12026-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12023-0">
    <skos:related rdf:resource="descriptor/12020-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12023-0">
    <skos:related rdf:resource="descriptor/12019-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12023-0">
    <skos:related rdf:resource="descriptor/11921-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12023-0">
    <skos:related rdf:resource="descriptor/18328-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12023-0">
    <skos:related rdf:resource="descriptor/19454-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15627-2">
    <skos:related rdf:resource="descriptor/15928-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15627-2">
    <skos:related rdf:resource="descriptor/16719-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15627-2">
    <skos:related rdf:resource="descriptor/15659-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15627-2">
    <skos:related rdf:resource="descriptor/16556-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15627-2">
    <skos:related rdf:resource="descriptor/15633-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15627-2">
    <skos:related rdf:resource="descriptor/16749-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11313-3">
    <skos:broader rdf:resource="thsys/73378"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11313-3">
    <skos:broader rdf:resource="thsys/70851"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11313-3">
    <skos:broader rdf:resource="thsys/70235"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11313-3">
    <skos:broader rdf:resource="descriptor/11350-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12357-6">
    <skos:altLabel xml:lang="de">Handelsbilanz des Betriebs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13386-1">
    <skos:altLabel xml:lang="de">Nachrichtenagentur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10912-0">
    <skos:prefLabel xml:lang="en">Vertical integration</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10912-0">
    <skos:prefLabel xml:lang="de">Vertikale Konzentration</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71109">
    <skos:prefLabel xml:lang="de">V.04.04  Forschung und Technik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15598-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="../stw">
    <rdf:type rdf:resource="http://www.w3.org/2004/02/skos/core#ConceptScheme"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14524-3">
    <skos:prefLabel xml:lang="en">Video cassette</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14524-3">
    <skos:prefLabel xml:lang="de">Videokassette</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18575-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17628-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18857-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20842-3">
    <skos:prefLabel xml:lang="en">Good governance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20842-3">
    <skos:prefLabel xml:lang="de">Good Governance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15646-5">
    <skos:broader rdf:resource="thsys/73388"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15646-5">
    <skos:broader rdf:resource="descriptor/15671-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70199">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16013-4">
    <skos:altLabel xml:lang="en">State power</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16013-4">
    <skos:altLabel xml:lang="de">Staatsmacht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16013-4">
    <skos:altLabel xml:lang="de">Staatshoheit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16013-4">
    <skos:altLabel xml:lang="de">Staatliches Gewaltmonopol</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16013-4">
    <skos:altLabel xml:lang="de">Staatliche Macht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16013-4">
    <skos:altLabel xml:lang="de">Staatliche Gewalt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16013-4">
    <skos:altLabel xml:lang="de">Staatliche Autorität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16013-4">
    <skos:altLabel xml:lang="de">Macht des Staates</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16013-4">
    <skos:altLabel xml:lang="de">Hoheitsrechte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16013-4">
    <skos:altLabel xml:lang="de">Hoheitsgewalt des Staates</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16013-4">
    <skos:altLabel xml:lang="de">Gewaltmonopol des Staates</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16013-4">
    <skos:altLabel xml:lang="de">Gebietshoheit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16013-4">
    <skos:altLabel xml:lang="de">Autorität des Staates</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70844">
    <skos:narrower rdf:resource="descriptor/18871-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70844">
    <skos:narrower rdf:resource="descriptor/18880-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70844">
    <skos:narrower rdf:resource="descriptor/11379-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70844">
    <skos:narrower rdf:resource="descriptor/11378-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70844">
    <skos:narrower rdf:resource="descriptor/18879-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70844">
    <skos:narrower rdf:resource="descriptor/18878-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70844">
    <skos:narrower rdf:resource="descriptor/19603-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70844">
    <skos:narrower rdf:resource="descriptor/18877-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70844">
    <skos:narrower rdf:resource="descriptor/18876-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70844">
    <skos:narrower rdf:resource="descriptor/11402-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70844">
    <skos:narrower rdf:resource="descriptor/11372-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70844">
    <skos:narrower rdf:resource="descriptor/18781-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70844">
    <skos:narrower rdf:resource="descriptor/18782-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70844">
    <skos:narrower rdf:resource="descriptor/18874-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70844">
    <skos:narrower rdf:resource="descriptor/11373-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70844">
    <skos:narrower rdf:resource="descriptor/18873-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70844">
    <skos:narrower rdf:resource="descriptor/11370-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70844">
    <skos:narrower rdf:resource="descriptor/18872-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70844">
    <skos:narrower rdf:resource="descriptor/11356-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14149-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10746-5">
    <skos:prefLabel xml:lang="en">International monetary system</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10746-5">
    <skos:prefLabel xml:lang="de">Internationales Währungssystem</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19207-4">
    <skos:altLabel xml:lang="de">Mathematische Wirtschaftswissenschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19207-4">
    <skos:altLabel xml:lang="de">Mathematische Wirtschaftstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13490-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70209">
    <skos:broader rdf:resource="thsys/70082"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71091">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.07.05</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10880-1">
    <skos:related rdf:resource="descriptor/16070-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10880-1">
    <skos:related rdf:resource="descriptor/10785-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10880-1">
    <skos:related rdf:resource="descriptor/10616-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10880-1">
    <skos:related rdf:resource="descriptor/19316-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10880-1">
    <skos:related rdf:resource="descriptor/10606-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10880-1">
    <skos:related rdf:resource="descriptor/10852-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10880-1">
    <skos:related rdf:resource="descriptor/10854-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10880-1">
    <skos:related rdf:resource="descriptor/10859-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14834-2">
    <skos:broader rdf:resource="thsys/70313"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14834-2">
    <skos:broader rdf:resource="descriptor/14586-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14834-2">
    <skos:broader rdf:resource="descriptor/14833-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18516-3">
    <skos:related rdf:resource="descriptor/19397-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18516-3">
    <skos:related rdf:resource="descriptor/12997-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18516-3">
    <skos:related rdf:resource="descriptor/13427-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19249-2">
    <skos:altLabel xml:lang="de">Optimale Unternehmensgröße</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19249-2">
    <skos:altLabel xml:lang="de">Betriebsgrößenoptimum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14802-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12573-0">
    <skos:altLabel xml:lang="de">Rekrutierungspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12573-0">
    <skos:altLabel xml:lang="de">Personalrekrutierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12573-0">
    <skos:altLabel xml:lang="de">Personaleinstellung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12573-0">
    <skos:altLabel xml:lang="de">Personalbereitstellung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12573-0">
    <skos:altLabel xml:lang="de">Personalanwerbung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12573-0">
    <skos:altLabel xml:lang="de">Einstellung von Personal</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73374">
    <skos:prefLabel xml:lang="de">N.04.03  Felder der Politik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19623-4">
    <skos:scopeNote xml:lang="de">Auch für Lufterzeugnisse als technische Gase</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11415-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13065-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11852-1">
    <skos:prefLabel xml:lang="en">Municipality</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11852-1">
    <skos:prefLabel xml:lang="de">Gemeinde</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11863-3">
    <skos:prefLabel xml:lang="en">Hinterland</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11863-3">
    <skos:prefLabel xml:lang="de">Hinterland</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19567-2">
    <skos:prefLabel xml:lang="en">Tax simplification</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19567-2">
    <skos:prefLabel xml:lang="de">Steuervereinfachung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15790-5">
    <skos:altLabel xml:lang="de">Theoretische Psychologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15790-5">
    <skos:altLabel xml:lang="de">Psychologischer Aspekt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15790-5">
    <skos:altLabel xml:lang="de">Psychologische Theorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15790-5">
    <skos:altLabel xml:lang="de">Psyche</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15790-5">
    <skos:altLabel xml:lang="de">Gestaltpsychologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20261-2">
    <skos:prefLabel xml:lang="en">Stakeholder</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20261-2">
    <skos:prefLabel xml:lang="de">Stakeholder</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18905-6">
    <skos:altLabel xml:lang="de">Subsidiarität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10692-2">
    <skos:broader rdf:resource="thsys/71098"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10692-2">
    <skos:broader rdf:resource="descriptor/10683-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18131-2">
    <skos:prefLabel xml:lang="en">East Germany</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18131-2">
    <skos:prefLabel xml:lang="de">Neue Bundesländer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18864-1">
    <skos:broader rdf:resource="thsys/70847"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18864-1">
    <skos:broader rdf:resource="thsys/70834"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18864-1">
    <skos:broader rdf:resource="thsys/70814"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19743-1">
    <skos:prefLabel xml:lang="en">Holistic approach</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19743-1">
    <skos:prefLabel xml:lang="de">Holismus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16514-3">
    <skos:prefLabel xml:lang="en">Film policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16514-3">
    <skos:prefLabel xml:lang="de">Filmpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15891-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18380-2">
    <skos:broader rdf:resource="thsys/71011"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10721-0">
    <skos:prefLabel xml:lang="en">Currency policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10721-0">
    <skos:prefLabel xml:lang="de">Währungspolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11164-1">
    <skos:narrower rdf:resource="descriptor/10029-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11164-1">
    <skos:narrower rdf:resource="descriptor/19606-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12881-3">
    <skos:related rdf:resource="descriptor/12889-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12881-3">
    <skos:related rdf:resource="descriptor/13389-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12881-3">
    <skos:related rdf:resource="descriptor/16274-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12881-3">
    <skos:related rdf:resource="descriptor/12214-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15992-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19314-3">
    <skos:related rdf:resource="descriptor/19325-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19314-3">
    <skos:related rdf:resource="descriptor/19320-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19314-3">
    <skos:related rdf:resource="descriptor/19324-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19314-3">
    <skos:related rdf:resource="descriptor/10690-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70254">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70471">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">B.06.03</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11066-1">
    <skos:narrower rdf:resource="descriptor/18828-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11066-1">
    <skos:narrower rdf:resource="descriptor/19287-1"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id234">
    <rdfs:seeAlso rdf:resource="descriptor/19550-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24782-3">
    <skos:scopeNote xml:lang="de">New Economy wird als Modell des neuen Wirtschaftens verstanden, bei dem die Vorteile des e-Commerce und des Internet (u.a. Vernetzung, Globalisierung, ideale Märkte) zu einer Beschleunigung, Rationalisierung und Effizienzsteigerung der ökonomischen Interaktionen führen. Hierin wird ein materieller Kern einer zukünftigen Wachstumsökonomie gesehen, der auch die zwischenzeitlich zu beobachtende Krise überstehen wird.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16046-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18639-1">
    <skos:related rdf:resource="descriptor/13645-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18639-1">
    <skos:related rdf:resource="descriptor/13639-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18639-1">
    <skos:related rdf:resource="descriptor/13638-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10508-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14039-3">
    <skos:narrower rdf:resource="descriptor/14364-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14039-3">
    <skos:narrower rdf:resource="descriptor/14053-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14039-3">
    <skos:narrower rdf:resource="descriptor/14407-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14039-3">
    <skos:narrower rdf:resource="descriptor/14037-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14039-3">
    <skos:narrower rdf:resource="descriptor/14035-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14039-3">
    <skos:narrower rdf:resource="descriptor/12977-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15790-5">
    <skos:related rdf:resource="descriptor/18859-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15790-5">
    <skos:related rdf:resource="descriptor/16563-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15790-5">
    <skos:related rdf:resource="descriptor/13348-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15790-5">
    <skos:related rdf:resource="descriptor/19013-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15790-5">
    <skos:related rdf:resource="descriptor/15658-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15790-5">
    <skos:related rdf:resource="descriptor/15644-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15790-5">
    <skos:related rdf:resource="descriptor/15633-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15790-5">
    <skos:related rdf:resource="descriptor/19747-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18402-4">
    <skos:broader rdf:resource="thsys/73349"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18402-4">
    <skos:broader rdf:resource="thsys/73343"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18402-4">
    <skos:broader rdf:resource="thsys/71003"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18402-4">
    <skos:broader rdf:resource="descriptor/19799-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18402-4">
    <skos:broader rdf:resource="descriptor/19795-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19053-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10769-0">
    <skos:prefLabel xml:lang="en">Currency area</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10769-0">
    <skos:prefLabel xml:lang="de">Währungsraum</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15893-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18967-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19612-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16404-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71086">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.08.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19452-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70055">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.05</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11384-1">
    <skos:related rdf:resource="descriptor/11361-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11692-4">
    <skos:broader rdf:resource="thsys/71076"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11692-4">
    <skos:broader rdf:resource="descriptor/21082-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11692-4">
    <skos:broader rdf:resource="descriptor/11696-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12939-5">
    <skos:narrower rdf:resource="descriptor/12957-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14307-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14124-5">
    <skos:related rdf:resource="descriptor/12946-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14124-5">
    <skos:related rdf:resource="descriptor/18306-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14124-5">
    <skos:related rdf:resource="descriptor/13158-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14124-5">
    <skos:related rdf:resource="descriptor/14955-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14124-5">
    <skos:related rdf:resource="descriptor/19677-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14124-5">
    <skos:related rdf:resource="descriptor/19545-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14124-5">
    <skos:related rdf:resource="descriptor/19183-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14124-5">
    <skos:related rdf:resource="descriptor/14971-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10125-2">
    <skos:related rdf:resource="descriptor/10123-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10125-2">
    <skos:related rdf:resource="descriptor/10120-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10125-2">
    <skos:related rdf:resource="descriptor/10440-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16641-3">
    <skos:prefLabel xml:lang="en">Social control</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16641-3">
    <skos:prefLabel xml:lang="de">Soziale Kontrolle</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19240-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14964-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18326-1">
    <skos:related rdf:resource="descriptor/18315-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18326-1">
    <skos:related rdf:resource="descriptor/12024-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18326-1">
    <skos:related rdf:resource="descriptor/10490-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71107">
    <skos:broader rdf:resource="thsys/70187"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18506-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13966-4">
    <skos:related rdf:resource="descriptor/12927-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13966-4">
    <skos:related rdf:resource="descriptor/13644-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13966-4">
    <skos:related rdf:resource="descriptor/13253-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13966-4">
    <skos:related rdf:resource="descriptor/18977-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13966-4">
    <skos:related rdf:resource="descriptor/13975-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11981-4">
    <skos:scopeNote xml:lang="de">Nur für kommunale Gewerbegebiete.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15951-0">
    <skos:prefLabel xml:lang="en">Emigration</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15951-0">
    <skos:prefLabel xml:lang="de">Auswanderung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13780-4">
    <skos:prefLabel xml:lang="en">Consumer credit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13780-4">
    <skos:prefLabel xml:lang="de">Verbraucherkredit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10121-3">
    <skos:prefLabel xml:lang="en">Capital shortage</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10121-3">
    <skos:prefLabel xml:lang="de">Kapitalmangel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24706-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15797-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12962-3">
    <skos:broader rdf:resource="thsys/71025"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12962-3">
    <skos:broader rdf:resource="thsys/71019"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15444-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#definition">
    <skos:definition xml:lang="en">A statement or formal explanation of the meaning of a concept.</skos:definition>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18924-2">
    <skos:related rdf:resource="descriptor/19611-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18924-2">
    <skos:related rdf:resource="descriptor/16680-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18924-2">
    <skos:related rdf:resource="descriptor/15694-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18924-2">
    <skos:related rdf:resource="descriptor/15654-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18924-2">
    <skos:related rdf:resource="descriptor/10876-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18924-2">
    <skos:related rdf:resource="descriptor/16556-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18448-1">
    <skos:altLabel xml:lang="en">Phonographic industry</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18448-1">
    <skos:altLabel xml:lang="de">Vervielfältigung von bespielten Tonträgern</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18448-1">
    <skos:altLabel xml:lang="de">Verlag von bespielten Tonträgern</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18448-1">
    <skos:altLabel xml:lang="de">Tonträgerverlag</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18448-1">
    <skos:altLabel xml:lang="de">Sinfonieorchester</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18448-1">
    <skos:altLabel xml:lang="de">Schallplattenverlag</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18448-1">
    <skos:altLabel xml:lang="de">Orchester</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18448-1">
    <skos:altLabel xml:lang="de">Opernhaus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18448-1">
    <skos:altLabel xml:lang="de">Musikverlag</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18448-1">
    <skos:altLabel xml:lang="de">Musikmanagement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18448-1">
    <skos:altLabel xml:lang="de">Musikinstrumentenhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18448-1">
    <skos:altLabel xml:lang="de">Musikinstrumentenbau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18448-1">
    <skos:altLabel xml:lang="de">Musikindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18448-1">
    <skos:altLabel xml:lang="de">Musikbranche</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18448-1">
    <skos:altLabel xml:lang="de">Musikalienverlag</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18448-1">
    <skos:altLabel xml:lang="de">Musikalienhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18448-1">
    <skos:altLabel xml:lang="de">Konzertveranstalter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18448-1">
    <skos:altLabel xml:lang="de">Konzerthaus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18448-1">
    <skos:altLabel xml:lang="de">Konzertaufführung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18448-1">
    <skos:altLabel xml:lang="de">Klavierbau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18448-1">
    <skos:altLabel xml:lang="de">Instrumentenbau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18448-1">
    <skos:altLabel xml:lang="de">Chor</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18448-1">
    <skos:altLabel xml:lang="de">CD-Verlag</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18448-1">
    <skos:altLabel xml:lang="de">Ballett</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18272-5">
    <skos:altLabel xml:lang="de">VerpackV</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18272-5">
    <skos:altLabel xml:lang="de">Verpackungsverordnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15103-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19636-2">
    <skos:prefLabel xml:lang="en">Guild</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19636-2">
    <skos:prefLabel xml:lang="de">Zunft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13528-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70853">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19210-1">
    <skos:narrower rdf:resource="descriptor/19558-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13309-5">
    <skos:altLabel xml:lang="de">Werkküche</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13309-5">
    <skos:altLabel xml:lang="de">Werkkantine</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13309-5">
    <skos:altLabel xml:lang="de">Personalverpflegung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13309-5">
    <skos:altLabel xml:lang="de">Mensa</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13309-5">
    <skos:altLabel xml:lang="de">Kantine</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13309-5">
    <skos:altLabel xml:lang="de">Gemeinschaftsverpflegung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13309-5">
    <skos:altLabel xml:lang="de">Betriebsverpflegung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13309-5">
    <skos:altLabel xml:lang="de">Betriebskantine</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13284-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18981-4">
    <skos:broader rdf:resource="thsys/71029"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18981-4">
    <skos:broader rdf:resource="descriptor/18198-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17502-5">
    <skos:altLabel xml:lang="de">Turkmenien</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73295">
    <skos:prefLabel xml:lang="en">W.06.01.12.01  Beverage industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73295">
    <skos:prefLabel xml:lang="de">W.06.01.12.01  Getränkeindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18915-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18456-2">
    <skos:related rdf:resource="descriptor/11782-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18456-2">
    <skos:related rdf:resource="descriptor/18455-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18456-2">
    <skos:related rdf:resource="descriptor/11784-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73346">
    <skos:prefLabel xml:lang="de">N.10.05.01  Gesundheitsrisiken und Krankheiten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16809-5">
    <skos:prefLabel xml:lang="en">World</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16809-5">
    <skos:prefLabel xml:lang="de">Welt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19271-2">
    <skos:prefLabel xml:lang="en">Capital asset pricing model</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19271-2">
    <skos:prefLabel xml:lang="de">Capital Asset Pricing Model</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11341-5">
    <skos:narrower rdf:resource="descriptor/12598-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11341-5">
    <skos:narrower rdf:resource="descriptor/12594-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11341-5">
    <skos:narrower rdf:resource="descriptor/11321-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11341-5">
    <skos:narrower rdf:resource="descriptor/19500-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11341-5">
    <skos:narrower rdf:resource="descriptor/11772-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70070">
    <skos:narrower rdf:resource="descriptor/14098-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70070">
    <skos:narrower rdf:resource="descriptor/14096-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70070">
    <skos:narrower rdf:resource="descriptor/14095-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70070">
    <skos:narrower rdf:resource="descriptor/14093-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70070">
    <skos:narrower rdf:resource="descriptor/14091-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70070">
    <skos:narrower rdf:resource="descriptor/14090-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70070">
    <skos:narrower rdf:resource="descriptor/14088-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70070">
    <skos:narrower rdf:resource="descriptor/14087-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16078-4">
    <skos:related rdf:resource="descriptor/19564-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16078-4">
    <skos:related rdf:resource="descriptor/18557-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16078-4">
    <skos:related rdf:resource="descriptor/18066-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16078-4">
    <skos:related rdf:resource="descriptor/10481-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16078-4">
    <skos:related rdf:resource="descriptor/19129-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16078-4">
    <skos:related rdf:resource="descriptor/12800-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16078-4">
    <skos:related rdf:resource="descriptor/19316-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16072-2">
    <skos:related rdf:resource="descriptor/13290-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19538-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18237-0">
    <skos:prefLabel xml:lang="en">Cadmium</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18237-0">
    <skos:prefLabel xml:lang="de">Cadmium</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11274-1">
    <skos:broader rdf:resource="thsys/70052"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11274-1">
    <skos:broader rdf:resource="descriptor/11262-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19061-6">
    <skos:broader rdf:resource="thsys/70959"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19061-6">
    <skos:broader rdf:resource="descriptor/18522-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19061-6">
    <skos:broader rdf:resource="descriptor/13209-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10565-2">
    <skos:prefLabel xml:lang="en">Export diversification</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10565-2">
    <skos:prefLabel xml:lang="de">Exportdiversifizierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19809-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15266-1">
    <skos:broader rdf:resource="thsys/72186"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15266-1">
    <skos:broader rdf:resource="descriptor/15272-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13297-0">
    <skos:broader rdf:resource="thsys/70891"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13297-0">
    <skos:broader rdf:resource="descriptor/13828-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15979-6">
    <skos:prefLabel xml:lang="en">Penal system</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15979-6">
    <skos:prefLabel xml:lang="de">Strafvollzug</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11156-0">
    <skos:altLabel xml:lang="en">Economic theory of crime</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11156-0">
    <skos:altLabel xml:lang="de">Ökonomische Theorie der Kriminalität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11156-0">
    <skos:altLabel xml:lang="de">Kriminalitätsökonomie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15798-3">
    <skos:related rdf:resource="descriptor/19779-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15798-3">
    <skos:related rdf:resource="descriptor/15676-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15798-3">
    <skos:related rdf:resource="descriptor/15722-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15798-3">
    <skos:related rdf:resource="descriptor/13270-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15798-3">
    <skos:related rdf:resource="descriptor/16265-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15798-3">
    <skos:related rdf:resource="descriptor/16761-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15798-3">
    <skos:related rdf:resource="descriptor/13658-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15798-3">
    <skos:related rdf:resource="descriptor/16754-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11241-2">
    <skos:related rdf:resource="descriptor/12501-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11241-2">
    <skos:related rdf:resource="descriptor/11236-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11241-2">
    <skos:related rdf:resource="descriptor/11246-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11241-2">
    <skos:related rdf:resource="descriptor/19566-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11241-2">
    <skos:related rdf:resource="descriptor/11243-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18238-5">
    <skos:broader rdf:resource="thsys/70134"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18238-5">
    <skos:broader rdf:resource="descriptor/14274-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18238-5">
    <skos:broader rdf:resource="descriptor/14302-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73393">
    <skos:prefLabel xml:lang="en">N.01.02  Philosophy of science</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73393">
    <skos:prefLabel xml:lang="de">N.01.02  Wissenschaftstheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11586-3">
    <skos:prefLabel xml:lang="en">Budget deficit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11586-3">
    <skos:prefLabel xml:lang="de">Haushaltsdefizit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14700-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17731-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71087">
    <skos:narrower rdf:resource="descriptor/10680-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71087">
    <skos:narrower rdf:resource="descriptor/19590-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71087">
    <skos:narrower rdf:resource="descriptor/10410-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71087">
    <skos:narrower rdf:resource="descriptor/10791-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71087">
    <skos:narrower rdf:resource="descriptor/10787-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71087">
    <skos:narrower rdf:resource="descriptor/10786-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71087">
    <skos:narrower rdf:resource="descriptor/10784-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71087">
    <skos:narrower rdf:resource="descriptor/10778-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14945-0">
    <skos:altLabel xml:lang="de">Müslimischung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14945-0">
    <skos:altLabel xml:lang="de">Getreideflocken</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10923-2">
    <skos:altLabel xml:lang="en">Collusion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10923-2">
    <skos:altLabel xml:lang="de">Syndikat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10923-2">
    <skos:altLabel xml:lang="de">Kartelltheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10923-2">
    <skos:altLabel xml:lang="de">Frühstückskartell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13691-3">
    <skos:scopeNote xml:lang="de">Geschichtliche Darstellungen des Bankwesens allgemein und in einzelnen Regionen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19177-1">
    <skos:broader rdf:resource="thsys/70307"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19177-1">
    <skos:broader rdf:resource="descriptor/17999-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19177-1">
    <skos:broader rdf:resource="descriptor/10499-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18375-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70538">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18120-0">
    <skos:narrower rdf:resource="descriptor/18524-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18120-0">
    <skos:narrower rdf:resource="descriptor/18479-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18120-0">
    <skos:narrower rdf:resource="descriptor/18093-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18120-0">
    <skos:narrower rdf:resource="descriptor/18993-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18120-0">
    <skos:narrower rdf:resource="descriptor/12917-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15111-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15616-0">
    <skos:related rdf:resource="descriptor/18270-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15616-0">
    <skos:related rdf:resource="descriptor/14751-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15616-0">
    <skos:related rdf:resource="descriptor/18139-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18308-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11309-1">
    <skos:prefLabel xml:lang="en">World of work</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11309-1">
    <skos:prefLabel xml:lang="de">Arbeitswelt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13940-1">
    <skos:altLabel xml:lang="de">Schlachterhandwerk</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13940-1">
    <skos:altLabel xml:lang="de">Schlachterei</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13940-1">
    <skos:altLabel xml:lang="de">Fleischwareneinzelhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13940-1">
    <skos:altLabel xml:lang="de">Fleischerei</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13966-4">
    <skos:broader rdf:resource="thsys/71055"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13966-4">
    <skos:broader rdf:resource="thsys/70924"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13966-4">
    <skos:broader rdf:resource="descriptor/13979-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19053-5">
    <skos:broader rdf:resource="thsys/70258"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19053-5">
    <skos:broader rdf:resource="descriptor/14441-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14545-2">
    <skos:broader rdf:resource="thsys/70346"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14545-2">
    <skos:broader rdf:resource="descriptor/14548-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#altLabel">
    <skos:definition xml:lang="en">An alternative lexical label for a resource.</skos:definition>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19551-3">
    <skos:altLabel xml:lang="en">HMO (Health Maintenance Organization)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19551-3">
    <skos:altLabel xml:lang="en">Health maintenance organization</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19551-3">
    <skos:altLabel xml:lang="en">Health maintenance organisation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19326-3">
    <skos:scopeNote xml:lang="de">Regeln des GATT und in seiner Nachfolge der WTO zur Regulierung der Außenhandelsbeziehungen unter den Mitgliedsländern.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18204-1">
    <skos:related rdf:resource="descriptor/14788-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70506">
    <skos:broader rdf:resource="thsys/p"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19196-4">
    <skos:prefLabel xml:lang="en">Bubbles</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19196-4">
    <skos:prefLabel xml:lang="de">Bubbles</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11389-5">
    <skos:prefLabel xml:lang="en">Labour force integration</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11389-5">
    <skos:prefLabel xml:lang="de">Berufliche Integration</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14251-5">
    <skos:prefLabel xml:lang="en">Fire resistant materials</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14251-5">
    <skos:prefLabel xml:lang="de">Feuerfestes Material</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16041-6">
    <skos:prefLabel xml:lang="en">Civil action</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16041-6">
    <skos:prefLabel xml:lang="de">Zivilprozess</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12589-6">
    <skos:related rdf:resource="descriptor/12574-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12589-6">
    <skos:related rdf:resource="descriptor/12572-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12589-6">
    <skos:related rdf:resource="descriptor/15839-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12589-6">
    <skos:related rdf:resource="descriptor/11052-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12589-6">
    <skos:related rdf:resource="descriptor/11399-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12589-6">
    <skos:related rdf:resource="descriptor/15837-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14371-2">
    <skos:broader rdf:resource="thsys/70318"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14371-2">
    <skos:broader rdf:resource="descriptor/14381-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15036-4">
    <skos:related rdf:resource="descriptor/15062-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15036-4">
    <skos:related rdf:resource="descriptor/15476-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15036-4">
    <skos:related rdf:resource="descriptor/16716-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15036-4">
    <skos:related rdf:resource="descriptor/15467-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15036-4">
    <skos:related rdf:resource="descriptor/18171-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16441-4">
    <skos:altLabel xml:lang="en">Foreign relations</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16441-4">
    <skos:altLabel xml:lang="de">Multilaterale Beziehungen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16441-4">
    <skos:altLabel xml:lang="de">Auswärtige Beziehungen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16441-4">
    <skos:altLabel xml:lang="de">Auslandsbeziehungen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16441-4">
    <skos:altLabel xml:lang="de">Außenpolitische Beziehungen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16441-4">
    <skos:altLabel xml:lang="de">Außenbeziehungen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19410-0">
    <skos:related rdf:resource="descriptor/21061-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19410-0">
    <skos:related rdf:resource="descriptor/21014-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19410-0">
    <skos:related rdf:resource="descriptor/19281-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19410-0">
    <skos:related rdf:resource="descriptor/11708-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19410-0">
    <skos:related rdf:resource="descriptor/24858-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19410-0">
    <skos:related rdf:resource="descriptor/11636-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19410-0">
    <skos:related rdf:resource="descriptor/19401-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19410-0">
    <skos:related rdf:resource="descriptor/12848-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19859-3">
    <skos:prefLabel xml:lang="en">Contract for work and services</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19859-3">
    <skos:prefLabel xml:lang="de">Werkvertrag</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19302-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18609-3">
    <skos:related rdf:resource="descriptor/11893-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18609-3">
    <skos:related rdf:resource="descriptor/10613-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13488-0">
    <skos:prefLabel xml:lang="en">Opening Hours</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13488-0">
    <skos:prefLabel xml:lang="de">Öffnungszeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13761-1">
    <skos:prefLabel xml:lang="en">Retail banking</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13761-1">
    <skos:prefLabel xml:lang="de">Retail Banking</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11939-3">
    <skos:broader rdf:resource="thsys/71069"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11939-3">
    <skos:broader rdf:resource="descriptor/16552-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11939-3">
    <skos:broader rdf:resource="descriptor/16314-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15771-2">
    <skos:broader rdf:resource="thsys/70815"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15771-2">
    <skos:broader rdf:resource="thsys/70239"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15771-2">
    <skos:broader rdf:resource="descriptor/10008-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11278-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14999-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70074">
    <rdfs:label xml:lang="de">Spezialchemikalien</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14510-0">
    <skos:prefLabel xml:lang="en">Microwave oven</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14510-0">
    <skos:prefLabel xml:lang="de">Mikrowellenherd</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13829-0">
    <skos:broader rdf:resource="thsys/70888"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13829-0">
    <skos:broader rdf:resource="descriptor/18749-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19802-4">
    <skos:related rdf:resource="descriptor/15610-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19046-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12303-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14902-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13755-3">
    <skos:narrower rdf:resource="descriptor/18712-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12294-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12618-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13130-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19848-1">
    <skos:related rdf:resource="descriptor/19824-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19848-1">
    <skos:related rdf:resource="descriptor/16067-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19848-1">
    <skos:related rdf:resource="descriptor/16191-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17104-3">
    <skos:prefLabel xml:lang="en">Portugal</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17104-3">
    <skos:prefLabel xml:lang="de">Portugal</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19744-6">
    <skos:related rdf:resource="descriptor/16727-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19744-6">
    <skos:related rdf:resource="descriptor/16554-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18766-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18950-1">
    <skos:altLabel xml:lang="de">Buchkonservierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18950-1">
    <skos:altLabel xml:lang="de">Bestandsrestaurierung in Bibliotheken</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18950-1">
    <skos:altLabel xml:lang="de">Bestandspflege in Bibliotheken</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70467">
    <rdfs:label xml:lang="de">Tierische Produkte</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11507-4">
    <skos:narrower rdf:resource="descriptor/11516-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16961-6">
    <skos:prefLabel xml:lang="en">Geneva</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16961-6">
    <skos:prefLabel xml:lang="de">Genf</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16745-5">
    <skos:related rdf:resource="descriptor/16732-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16745-5">
    <skos:related rdf:resource="descriptor/16727-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16745-5">
    <skos:related rdf:resource="descriptor/10032-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16745-5">
    <skos:related rdf:resource="descriptor/10062-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16745-5">
    <skos:related rdf:resource="descriptor/15706-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14984-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19730-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14665-6">
    <skos:related rdf:resource="descriptor/18009-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14665-6">
    <skos:related rdf:resource="descriptor/14972-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70855">
    <skos:prefLabel xml:lang="en">W.23.01  Social economy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70855">
    <skos:prefLabel xml:lang="de">W.23.01  Gemeinwirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15415-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11552-6">
    <skos:altLabel xml:lang="de">Öffentliche Infrastrukturinvestition</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70883">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70321">
    <skos:narrower rdf:resource="descriptor/11105-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70321">
    <skos:narrower rdf:resource="descriptor/11127-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70321">
    <skos:narrower rdf:resource="descriptor/20261-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70321">
    <skos:narrower rdf:resource="descriptor/16597-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70321">
    <skos:narrower rdf:resource="descriptor/16595-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70321">
    <skos:narrower rdf:resource="descriptor/15720-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70321">
    <skos:narrower rdf:resource="descriptor/15719-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70321">
    <skos:narrower rdf:resource="descriptor/19769-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70321">
    <skos:narrower rdf:resource="descriptor/16615-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70321">
    <skos:narrower rdf:resource="descriptor/19752-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70321">
    <skos:narrower rdf:resource="descriptor/16308-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70321">
    <skos:narrower rdf:resource="descriptor/19751-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70321">
    <skos:narrower rdf:resource="descriptor/16656-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70321">
    <skos:narrower rdf:resource="descriptor/12105-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70321">
    <skos:narrower rdf:resource="descriptor/12111-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70321">
    <skos:narrower rdf:resource="descriptor/12091-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70321">
    <skos:narrower rdf:resource="descriptor/18867-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70321">
    <skos:narrower rdf:resource="descriptor/24705-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70321">
    <skos:narrower rdf:resource="descriptor/16588-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70321">
    <skos:narrower rdf:resource="descriptor/16594-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70321">
    <skos:narrower rdf:resource="thsys/73395"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70321">
    <skos:narrower rdf:resource="thsys/73396"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19062-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11388-0">
    <skos:broader rdf:resource="thsys/71050"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11388-0">
    <skos:broader rdf:resource="thsys/70838"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11388-0">
    <skos:broader rdf:resource="thsys/70833"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11388-0">
    <skos:broader rdf:resource="thsys/70324"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11388-0">
    <skos:broader rdf:resource="descriptor/16541-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11388-0">
    <skos:broader rdf:resource="descriptor/11389-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10406-1">
    <skos:broader rdf:resource="thsys/70051"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10406-1">
    <skos:broader rdf:resource="descriptor/10993-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10406-1">
    <skos:broader rdf:resource="descriptor/11270-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19467-6">
    <skos:prefLabel xml:lang="en">Local development</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19467-6">
    <skos:prefLabel xml:lang="de">Gemeindeentwicklung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10877-4">
    <skos:broader rdf:resource="thsys/71091"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10877-4">
    <skos:broader rdf:resource="thsys/70951"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10877-4">
    <skos:broader rdf:resource="descriptor/11064-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10877-4">
    <skos:broader rdf:resource="descriptor/10605-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71044">
    <skos:narrower rdf:resource="descriptor/15515-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71044">
    <skos:narrower rdf:resource="descriptor/12294-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71044">
    <skos:narrower rdf:resource="descriptor/20301-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71044">
    <skos:narrower rdf:resource="descriptor/19150-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71044">
    <skos:narrower rdf:resource="descriptor/10015-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71044">
    <skos:narrower rdf:resource="descriptor/10293-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71044">
    <skos:narrower rdf:resource="descriptor/12208-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71044">
    <skos:narrower rdf:resource="descriptor/12278-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71044">
    <skos:narrower rdf:resource="descriptor/10008-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71044">
    <skos:narrower rdf:resource="descriptor/12207-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71044">
    <skos:narrower rdf:resource="descriptor/12203-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71044">
    <skos:narrower rdf:resource="descriptor/10823-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71044">
    <skos:narrower rdf:resource="descriptor/18107-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71044">
    <skos:narrower rdf:resource="descriptor/12209-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71044">
    <skos:narrower rdf:resource="descriptor/12224-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12302-3">
    <skos:altLabel xml:lang="de">Zahlungsverzug</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12302-3">
    <skos:altLabel xml:lang="de">Schuldnerverzug</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12302-3">
    <skos:altLabel xml:lang="de">Insolvenz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12681-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18787-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13333-1">
    <skos:related rdf:resource="descriptor/24621-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19758-2">
    <skos:related rdf:resource="descriptor/19755-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19758-2">
    <skos:related rdf:resource="descriptor/16664-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19758-2">
    <skos:related rdf:resource="descriptor/15807-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19758-2">
    <skos:related rdf:resource="descriptor/12806-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19758-2">
    <skos:related rdf:resource="descriptor/19198-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19758-2">
    <skos:related rdf:resource="descriptor/12075-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11999-6">
    <skos:prefLabel xml:lang="en">Urban regeneration</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11999-6">
    <skos:prefLabel xml:lang="de">Stadtsanierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14940-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10375-0">
    <skos:prefLabel xml:lang="en">Business survey</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10375-0">
    <skos:prefLabel xml:lang="de">Konjunkturumfrage</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19696-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11349-3">
    <skos:broader rdf:resource="thsys/73361"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11349-3">
    <skos:broader rdf:resource="thsys/71051"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11349-3">
    <skos:broader rdf:resource="thsys/70235"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11349-3">
    <skos:broader rdf:resource="descriptor/15998-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10122-1">
    <skos:broader rdf:resource="thsys/70486"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10122-1">
    <skos:broader rdf:resource="thsys/70233"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10122-1">
    <skos:broader rdf:resource="descriptor/10313-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19276-6">
    <skos:scopeNote xml:lang="de">Verbriefte Einlage von Nicht-Banken bei Banken, die gehandelt werden kann.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12492-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16363-5">
    <skos:related rdf:resource="descriptor/16411-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11142-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19260-0">
    <skos:altLabel xml:lang="de">Währungsinstitution</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19260-0">
    <skos:altLabel xml:lang="de">Währungsinstitut</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11667-3">
    <skos:altLabel xml:lang="de">Kfz-Steuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11667-3">
    <skos:altLabel xml:lang="de">Hubraumsteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12931-0">
    <skos:broader rdf:resource="thsys/71072"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12931-0">
    <skos:broader rdf:resource="thsys/71017"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12931-0">
    <skos:broader rdf:resource="thsys/70860"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10831-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18731-6">
    <skos:related rdf:resource="descriptor/19554-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18731-6">
    <skos:related rdf:resource="descriptor/13755-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15779-0">
    <skos:prefLabel xml:lang="en">Forest damage</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15779-0">
    <skos:prefLabel xml:lang="de">Waldschaden</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15611-3">
    <skos:altLabel xml:lang="en">AI (Artificial intelligence)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15611-3">
    <skos:altLabel xml:lang="de">KI (Künstliche Intelligenz)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12010-2">
    <skos:prefLabel xml:lang="en">Infrastructure network</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12010-2">
    <skos:prefLabel xml:lang="de">Netzinfrastruktur</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14750-1">
    <skos:broader rdf:resource="thsys/73305"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14750-1">
    <skos:broader rdf:resource="descriptor/14023-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10204-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13595-6">
    <skos:scopeNote xml:lang="de">Nur im Sinne einer räumlichen Integrationswirkung des Verkehrs.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14121-4">
    <skos:broader rdf:resource="thsys/70467"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14121-4">
    <skos:broader rdf:resource="descriptor/14129-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13339-3">
    <skos:related rdf:resource="descriptor/14404-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13339-3">
    <skos:related rdf:resource="descriptor/13330-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11663-4">
    <zbwext:useInsteadNote rdf:nodeID="id163"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11599-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11921-1">
    <skos:related rdf:resource="descriptor/12023-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11921-1">
    <skos:related rdf:resource="descriptor/12019-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11921-1">
    <skos:related rdf:resource="descriptor/18328-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11921-1">
    <skos:related rdf:resource="descriptor/11859-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11921-1">
    <skos:related rdf:resource="descriptor/19454-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11921-1">
    <skos:related rdf:resource="descriptor/12015-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17977-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17954-5">
    <skos:prefLabel xml:lang="en">Atlantic Ocean</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17954-5">
    <skos:prefLabel xml:lang="de">Atlantischer Ozean</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14223-3">
    <skos:altLabel xml:lang="de">Asbestprodukt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14223-3">
    <skos:altLabel xml:lang="de">Asbestfaser</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12842-6">
    <skos:broader rdf:resource="thsys/70828"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12842-6">
    <skos:broader rdf:resource="thsys/70384"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12842-6">
    <skos:broader rdf:resource="descriptor/12841-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12842-6">
    <skos:broader rdf:resource="descriptor/15661-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71051">
    <skos:prefLabel xml:lang="de">B.04.03  Arbeitsrecht und Arbeitsbeziehungen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19759-0">
    <skos:narrower rdf:resource="descriptor/19762-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19759-0">
    <skos:narrower rdf:resource="descriptor/18970-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19759-0">
    <skos:narrower rdf:resource="descriptor/18968-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14990-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16034-3">
    <skos:altLabel xml:lang="de">Produzentenhaftung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16409-0">
    <skos:prefLabel xml:lang="en">Collectivism</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16409-0">
    <skos:prefLabel xml:lang="de">Kollektivismus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14662-5">
    <skos:altLabel xml:lang="de">Propylen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14662-5">
    <skos:altLabel xml:lang="de">Propen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14662-5">
    <skos:altLabel xml:lang="de">Olefin</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14662-5">
    <skos:altLabel xml:lang="de">Ethylen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14662-5">
    <skos:altLabel xml:lang="de">Ethen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14662-5">
    <skos:altLabel xml:lang="de">Butylen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14662-5">
    <skos:altLabel xml:lang="de">Buten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14662-5">
    <skos:altLabel xml:lang="de">Äthylen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14662-5">
    <skos:altLabel xml:lang="de">Alpha-Olefin</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19467-6">
    <skos:altLabel xml:lang="de">Kommunale Entwicklung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19342-5">
    <skos:broader rdf:resource="thsys/71026"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19342-5">
    <skos:broader rdf:resource="thsys/70960"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19342-5">
    <skos:broader rdf:resource="descriptor/13431-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13715-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16463-1">
    <skos:scopeNote xml:lang="de">Nationale und internationale Sicherheitspolitik gegen Bedrohung durch Dritte einschl. vorbeugender Maßnahmen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13485-6">
    <skos:broader rdf:resource="thsys/73384"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13485-6">
    <skos:broader rdf:resource="thsys/70947"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13485-6">
    <skos:broader rdf:resource="thsys/70814"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13485-6">
    <skos:broader rdf:resource="thsys/70292"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13485-6">
    <skos:broader rdf:resource="descriptor/12774-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13485-6">
    <skos:broader rdf:resource="descriptor/10300-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12308-5">
    <skos:scopeNote xml:lang="de">Im Rechnungswesen das Ergebnis des betrieblichen Leistungsprozesses: Betriebsgewinn oder -verlust.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11402-4">
    <skos:prefLabel xml:lang="en">Higher education policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11402-4">
    <skos:prefLabel xml:lang="de">Hochschulpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11063-0">
    <skos:prefLabel xml:lang="en">Comparative systems</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11063-0">
    <skos:prefLabel xml:lang="de">Systemvergleich</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13315-3">
    <skos:broader rdf:resource="thsys/70958"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15968-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19008-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13727-1">
    <skos:altLabel xml:lang="de">Partizipationsschein</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13727-1">
    <skos:altLabel xml:lang="de">Genussrechtskapital</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13727-1">
    <skos:altLabel xml:lang="de">Genussrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13727-1">
    <skos:altLabel xml:lang="de">Genußschein</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13727-1">
    <skos:altLabel xml:lang="de">Genußrechtskapital</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13727-1">
    <skos:altLabel xml:lang="de">Genußrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12088-0">
    <skos:altLabel xml:lang="de">Unternehmungsplanung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12088-0">
    <skos:altLabel xml:lang="de">Betriebsplanung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12088-0">
    <skos:altLabel xml:lang="de">Betriebliche Planung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16190-3">
    <skos:prefLabel xml:lang="en">Computer crime</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16190-3">
    <skos:prefLabel xml:lang="de">Computerkriminalität</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70900">
    <skos:prefLabel xml:lang="en">W.14.03  Banking administration</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70900">
    <skos:prefLabel xml:lang="de">W.14.03  Bankbetriebslehre</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16645-2">
    <skos:altLabel xml:lang="de">Wohlstand</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16645-2">
    <skos:altLabel xml:lang="de">Luxus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15441-2">
    <skos:broader rdf:resource="thsys/73104"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15441-2">
    <skos:broader rdf:resource="descriptor/15458-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18685-1">
    <skos:prefLabel xml:lang="en">Asset-backed security</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18685-1">
    <skos:prefLabel xml:lang="de">Asset-backed security</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13071-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12722-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11650-6">
    <skos:prefLabel xml:lang="en">Income tax</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11650-6">
    <skos:prefLabel xml:lang="de">Einkommensteuer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16364-3">
    <skos:prefLabel xml:lang="en">Ecological Party</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16364-3">
    <skos:prefLabel xml:lang="de">Ökologische Partei</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15917-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16437-2">
    <skos:prefLabel xml:lang="en">Defence alliance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16437-2">
    <skos:prefLabel xml:lang="de">Bündnissystem</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13371-0">
    <skos:narrower rdf:resource="descriptor/19465-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13371-0">
    <skos:narrower rdf:resource="descriptor/13372-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13670-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11276-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18118-1">
    <skos:prefLabel xml:lang="en">Risk management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18118-1">
    <skos:prefLabel xml:lang="de">Risikomanagement</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18712-3">
    <skos:broader rdf:resource="thsys/70922"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18712-3">
    <skos:broader rdf:resource="thsys/70900"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18712-3">
    <skos:broader rdf:resource="descriptor/18655-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18712-3">
    <skos:broader rdf:resource="descriptor/13755-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18584-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18481-3">
    <skos:broader rdf:resource="thsys/73368"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18481-3">
    <skos:broader rdf:resource="thsys/71085"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18481-3">
    <skos:broader rdf:resource="thsys/70979"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18481-3">
    <skos:broader rdf:resource="descriptor/11820-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18481-3">
    <skos:broader rdf:resource="descriptor/10524-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13569-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10177-4">
    <skos:related rdf:resource="descriptor/13398-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10177-4">
    <skos:related rdf:resource="descriptor/18301-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10177-4">
    <skos:related rdf:resource="descriptor/10161-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16395-6">
    <zbwext:useInsteadNote rdf:nodeID="id253"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10373-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13221-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19800-1">
    <skos:prefLabel xml:lang="en">Computer virus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19800-1">
    <skos:prefLabel xml:lang="de">Computer-Virus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16112-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12356-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18076-5">
    <skos:scopeNote xml:lang="de">Feststellung der Richtung von Kausalität; unterstellt Beziehungen zwischen Zeitreihen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15965-3">
    <skos:scopeNote xml:lang="de">Das Recht, das aus der menschlichen Natur abgeleitet werden kann und das deshalb zeitlos gültig sowie mit der Vernunft vereinbar ist. Aus dem Naturrecht lassen sich vor allem die allgemeinen Menschen- und Grundrechte begründen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16318-3">
    <skos:altLabel xml:lang="de">Wohlfahrtsstaat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16318-3">
    <skos:altLabel xml:lang="de">Sozialstaatsprinzip</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16318-3">
    <skos:altLabel xml:lang="de">Soziale Demokratie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15515-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10415-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13253-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10905-4">
    <skos:altLabel xml:lang="en">Market thickness</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10905-4">
    <skos:altLabel xml:lang="de">Marktformenlehre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10905-4">
    <skos:altLabel xml:lang="de">Marktform</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15914-6">
    <skos:prefLabel xml:lang="en">Demographic structure</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15914-6">
    <skos:prefLabel xml:lang="de">Bevölkerungsstruktur</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70087">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">B.01.04</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18130-4">
    <skos:related rdf:resource="descriptor/16202-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18130-4">
    <skos:related rdf:resource="descriptor/19734-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18130-4">
    <skos:related rdf:resource="descriptor/16201-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18130-4">
    <skos:related rdf:resource="descriptor/18116-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18130-4">
    <skos:related rdf:resource="descriptor/16544-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18130-4">
    <skos:related rdf:resource="descriptor/16430-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18130-4">
    <skos:related rdf:resource="descriptor/10207-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18130-4">
    <skos:related rdf:resource="descriptor/20817-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18130-4">
    <skos:related rdf:resource="descriptor/11181-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13509-4">
    <skos:related rdf:resource="descriptor/10847-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13509-4">
    <skos:related rdf:resource="descriptor/13501-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13509-4">
    <skos:related rdf:resource="descriptor/15878-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13509-4">
    <skos:related rdf:resource="descriptor/10711-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20277-1">
    <skos:related rdf:resource="descriptor/12154-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20277-1">
    <skos:related rdf:resource="descriptor/12758-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20277-1">
    <skos:related rdf:resource="descriptor/19548-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20277-1">
    <skos:related rdf:resource="descriptor/12203-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14599-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13949-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18096-6">
    <skos:prefLabel xml:lang="en">Agriculture and forestry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18096-6">
    <skos:prefLabel xml:lang="de">Land- und Forstwirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18447-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id20">
    <rdfs:seeAlso rdf:resource="descriptor/12577-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70969">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.07.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18977-2">
    <skos:related rdf:resource="descriptor/13971-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18977-2">
    <skos:related rdf:resource="descriptor/13966-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18977-2">
    <skos:related rdf:resource="descriptor/13985-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12889-1">
    <skos:altLabel xml:lang="de">Steuerbevollmächtigte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12889-1">
    <skos:altLabel xml:lang="de">Steuerberatungsrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12889-1">
    <skos:altLabel xml:lang="de">Steuerberatungspraxis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12889-1">
    <skos:altLabel xml:lang="de">Steuerberatungsgesellschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12889-1">
    <skos:altLabel xml:lang="de">Steuerberatergebührenverordnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12889-1">
    <skos:altLabel xml:lang="de">Steuerberater</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24857-5">
    <skos:scopeNote xml:lang="de">Ein ökonomisches Allokationsverfahren ist pfadabhängig, wenn seine Geschichte eine dauernde Auswirkung auf spätere Allokationen hat.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18832-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14346-1">
    <skos:prefLabel xml:lang="en">Vending machine industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14346-1">
    <skos:prefLabel xml:lang="de">Automatenwirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18614-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11819-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12262-3">
    <skos:related rdf:resource="descriptor/12205-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12262-3">
    <skos:related rdf:resource="descriptor/20018-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12262-3">
    <skos:related rdf:resource="descriptor/19251-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12262-3">
    <skos:related rdf:resource="descriptor/19548-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12262-3">
    <skos:related rdf:resource="descriptor/24740-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12262-3">
    <skos:related rdf:resource="descriptor/18785-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12262-3">
    <skos:related rdf:resource="descriptor/18784-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12262-3">
    <skos:related rdf:resource="descriptor/24624-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12262-3">
    <skos:related rdf:resource="descriptor/12432-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13001-5">
    <skos:prefLabel xml:lang="en">Petrochemical industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13001-5">
    <skos:prefLabel xml:lang="de">Petrochemische Industrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16550-6">
    <skos:related rdf:resource="descriptor/13424-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16550-6">
    <skos:related rdf:resource="descriptor/13394-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16550-6">
    <skos:related rdf:resource="descriptor/19691-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16550-6">
    <skos:related rdf:resource="descriptor/11852-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19475-0">
    <skos:narrower rdf:resource="descriptor/13346-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19475-0">
    <skos:narrower rdf:resource="descriptor/13351-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19475-0">
    <skos:narrower rdf:resource="descriptor/13341-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70270">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18617-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70066">
    <skos:prefLabel xml:lang="en">P.18  Textiles</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70066">
    <skos:prefLabel xml:lang="de">P.18  Textilien</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19769-4">
    <skos:broader rdf:resource="thsys/70321"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12070-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18417-5">
    <skos:prefLabel xml:lang="en">Shipbuilding policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18417-5">
    <skos:prefLabel xml:lang="de">Schiffbaupolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19206-6">
    <skos:altLabel xml:lang="de">Finanztheoretische Dogmengeschichte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19206-6">
    <skos:altLabel xml:lang="de">Dogmengeschichte der Finanzwissenschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11609-3">
    <skos:broader rdf:resource="thsys/71105"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11609-3">
    <skos:broader rdf:resource="thsys/71078"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11609-3">
    <skos:broader rdf:resource="descriptor/11611-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11609-3">
    <skos:broader rdf:resource="descriptor/11610-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18522-1">
    <skos:broader rdf:resource="thsys/70959"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18522-1">
    <skos:broader rdf:resource="descriptor/13210-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16193-4">
    <skos:prefLabel xml:lang="en">Corruption</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16193-4">
    <skos:prefLabel xml:lang="de">Korruption</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15107-0">
    <skos:altLabel xml:lang="de">Unternehmenszählung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15107-0">
    <skos:altLabel xml:lang="de">Betriebszählung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15107-0">
    <skos:altLabel xml:lang="de">Betriebsstruktur-Statistik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15107-0">
    <skos:altLabel xml:lang="de">Arbeitsstättenzählung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10627-6">
    <skos:narrower rdf:resource="descriptor/19336-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10627-6">
    <skos:narrower rdf:resource="descriptor/10608-3"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id291">
    <rdf:value xml:lang="de">Für Segregation am Arbeitsmarkt</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16195-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11710-0">
    <skos:prefLabel xml:lang="en">Country of origin principle</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11710-0">
    <skos:prefLabel xml:lang="de">Ursprungslandprinzip</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19622-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19110-5">
    <skos:prefLabel xml:lang="en">Works council election</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19110-5">
    <skos:prefLabel xml:lang="de">Betriebswahl</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14485-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15066-2">
    <skos:prefLabel xml:lang="en">Statistical quality control</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15066-2">
    <skos:prefLabel xml:lang="de">Statistische Qualitätskontrolle</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19268-5">
    <skos:broader rdf:resource="thsys/70455"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19657-1">
    <skos:broader rdf:resource="thsys/73338"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19657-1">
    <skos:broader rdf:resource="thsys/70922"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19657-1">
    <skos:broader rdf:resource="descriptor/13658-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11843-2">
    <skos:prefLabel xml:lang="en">Social housing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11843-2">
    <skos:prefLabel xml:lang="de">Sozialer Wohnungsbau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19018-0">
    <skos:prefLabel xml:lang="en">Hop</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19018-0">
    <skos:prefLabel xml:lang="de">Hopfen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10884-0">
    <skos:broader rdf:resource="thsys/71010"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10884-0">
    <skos:broader rdf:resource="thsys/70893"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10884-0">
    <skos:broader rdf:resource="thsys/70187"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10884-0">
    <skos:broader rdf:resource="descriptor/13279-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14211-3">
    <skos:broader rdf:resource="thsys/70418"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14211-3">
    <skos:broader rdf:resource="descriptor/18235-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14296-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13545-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19664-4">
    <skos:prefLabel xml:lang="en">Financial crisis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19664-4">
    <skos:prefLabel xml:lang="de">Finanzmarktkrise</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19833-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19195-6">
    <skos:broader rdf:resource="thsys/70049"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15705-1">
    <skos:related rdf:resource="descriptor/15706-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15705-1">
    <skos:related rdf:resource="descriptor/15373-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15714-0">
    <skos:prefLabel xml:lang="en">Social group</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15714-0">
    <skos:prefLabel xml:lang="de">Soziale Gruppe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19221-3">
    <skos:narrower rdf:resource="descriptor/19360-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71057">
    <skos:prefLabel xml:lang="de">V.13.06  Berufsgruppen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18980-6">
    <skos:prefLabel xml:lang="en">Agricultural production</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18980-6">
    <skos:prefLabel xml:lang="de">Agrarproduktion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19511-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14328-3">
    <skos:related rdf:resource="descriptor/14438-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18138-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19119-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18761-4">
    <skos:related rdf:resource="descriptor/13390-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18761-4">
    <skos:related rdf:resource="descriptor/15621-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18761-4">
    <skos:related rdf:resource="descriptor/24784-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15057-3">
    <skos:narrower rdf:resource="descriptor/15267-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13570-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10491-5">
    <skos:prefLabel xml:lang="en">Brain drain</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10491-5">
    <skos:prefLabel xml:lang="de">Brain Drain</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13293-1">
    <skos:narrower rdf:resource="descriptor/10866-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73379">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id208">
    <rdfs:seeAlso rdf:resource="descriptor/16012-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16155-5">
    <skos:related rdf:resource="descriptor/16529-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16155-5">
    <skos:related rdf:resource="descriptor/15908-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16155-5">
    <skos:related rdf:resource="descriptor/16124-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16364-3">
    <skos:related rdf:resource="descriptor/13400-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15060-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11927-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19148-1">
    <skos:scopeNote xml:lang="de">Beobachtungstechnik bei Arbeitszeitstudien, bei der eine Häufigkeitsanalyse im Sinne der Stichprobentheorie durchgeführt wird.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13430-3">
    <skos:broader rdf:resource="thsys/71028"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13430-3">
    <skos:broader rdf:resource="thsys/70259"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13430-3">
    <skos:broader rdf:resource="descriptor/12251-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70936">
    <rdfs:label xml:lang="en">Shipping</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70936">
    <rdfs:label xml:lang="de">Schifffahrt</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13179-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11770-3">
    <skos:related rdf:resource="descriptor/10110-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11770-3">
    <skos:related rdf:resource="descriptor/18910-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11770-3">
    <skos:related rdf:resource="descriptor/10099-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11770-3">
    <skos:related rdf:resource="descriptor/11568-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11770-3">
    <skos:related rdf:resource="descriptor/11735-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11770-3">
    <skos:related rdf:resource="descriptor/10380-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15211-5">
    <skos:prefLabel xml:lang="en">Statistical distribution</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15211-5">
    <skos:prefLabel xml:lang="de">Statistische Verteilung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15398-5">
    <skos:broader rdf:resource="thsys/72394"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15398-5">
    <skos:broader rdf:resource="descriptor/15405-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11510-1">
    <skos:altLabel xml:lang="x-other">Scala mobile</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11510-1">
    <skos:altLabel xml:lang="en">Cost of living adjustment</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11510-1">
    <skos:altLabel xml:lang="de">Lohngleitklausel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11510-1">
    <skos:altLabel xml:lang="de">Indexlohn</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11510-1">
    <skos:altLabel xml:lang="de">Gleitender Lohn</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11510-1">
    <skos:altLabel xml:lang="de">Gleitende Lohnskala</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18111-1">
    <skos:broader rdf:resource="thsys/70052"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18111-1">
    <skos:broader rdf:resource="descriptor/11180-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17938-3">
    <skos:broader rdf:resource="thsys/70128"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17938-3">
    <skos:broader rdf:resource="descriptor/17948-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10661-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10490-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15076-6">
    <skos:related rdf:resource="descriptor/10272-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15076-6">
    <skos:related rdf:resource="descriptor/15064-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15076-6">
    <skos:related rdf:resource="descriptor/11962-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15076-6">
    <skos:related rdf:resource="descriptor/15373-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15076-6">
    <skos:related rdf:resource="descriptor/19584-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15076-6">
    <skos:related rdf:resource="descriptor/11534-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15076-6">
    <skos:related rdf:resource="descriptor/15061-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15076-6">
    <skos:related rdf:resource="descriptor/12039-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18741-3">
    <skos:broader rdf:resource="thsys/70892"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18741-3">
    <skos:broader rdf:resource="descriptor/10032-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18741-3">
    <skos:broader rdf:resource="descriptor/13821-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18923-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12192-5">
    <skos:related rdf:resource="descriptor/12867-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14339-5">
    <skos:altLabel xml:lang="en">Machinery tool</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14339-5">
    <skos:altLabel xml:lang="de">Ziehwerkzeug</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14339-5">
    <skos:altLabel xml:lang="de">Stanzwerkzeug</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14339-5">
    <skos:altLabel xml:lang="de">Gewindewerkzeug</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14339-5">
    <skos:altLabel xml:lang="de">Fräskopf</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12070-5">
    <skos:prefLabel xml:lang="en">Firm formation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12070-5">
    <skos:prefLabel xml:lang="de">Unternehmensgründung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12071-3">
    <skos:broader rdf:resource="thsys/71034"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12071-3">
    <skos:broader rdf:resource="thsys/70481"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12071-3">
    <skos:broader rdf:resource="descriptor/12069-4"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id386">
    <rdf:value xml:lang="de">Für ökonomische Analysen zur Zeiteinteilung</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12777-5">
    <skos:related rdf:resource="descriptor/10940-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12777-5">
    <skos:related rdf:resource="descriptor/12665-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12777-5">
    <skos:related rdf:resource="descriptor/18007-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11774-2">
    <skos:prefLabel xml:lang="en">Rate of profit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11774-2">
    <skos:prefLabel xml:lang="de">Gewinnquote</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="http://zbw.eu/namespaces/skos-extensions/Descriptor">
    <skos:definition xml:lang="en">The descriptors part of the STW.</skos:definition>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14273-2">
    <skos:broader rdf:resource="thsys/70358"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14273-2">
    <skos:broader rdf:resource="thsys/70161"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14273-2">
    <skos:broader rdf:resource="descriptor/14266-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14273-2">
    <skos:broader rdf:resource="descriptor/15009-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17607-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19553-6">
    <skos:altLabel xml:lang="en">Low-income households</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19553-6">
    <skos:altLabel xml:lang="de">Niedrigeinkommenshaushalt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19553-6">
    <skos:altLabel xml:lang="de">Haushalt mit niedrigem Einkommen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19553-6">
    <skos:altLabel xml:lang="de">Geringfügiges Einkommen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11153-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13738-3">
    <skos:prefLabel xml:lang="en">Bond market</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13738-3">
    <skos:prefLabel xml:lang="de">Rentenmarkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10691-4">
    <skos:broader rdf:resource="thsys/73332"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10691-4">
    <skos:broader rdf:resource="thsys/71098"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10691-4">
    <skos:broader rdf:resource="descriptor/18064-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10691-4">
    <skos:broader rdf:resource="descriptor/10692-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71060">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18335-0">
    <skos:related rdf:resource="descriptor/18314-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18335-0">
    <skos:related rdf:resource="descriptor/18301-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18335-0">
    <skos:related rdf:resource="descriptor/10495-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18335-0">
    <skos:related rdf:resource="descriptor/16668-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12111-3">
    <skos:prefLabel xml:lang="en">Organizational structure</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12111-3">
    <skos:prefLabel xml:lang="de">Organisationsstruktur</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20722-6">
    <skos:altLabel xml:lang="en">Earth Summit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20722-6">
    <skos:altLabel xml:lang="de">Regionale Agenda 21</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20722-6">
    <skos:altLabel xml:lang="de">Lokale Agenda 21</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10015-2">
    <skos:prefLabel xml:lang="en">Lifetime of capital equipment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10015-2">
    <skos:prefLabel xml:lang="de">Nutzungsdauer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17580-6">
    <skos:broader rdf:resource="thsys/70327"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17580-6">
    <skos:broader rdf:resource="descriptor/17589-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15910-0">
    <skos:prefLabel xml:lang="en">Population density</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15910-0">
    <skos:prefLabel xml:lang="de">Bevölkerungsdichte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10124-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15264-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14455-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19156-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70186">
    <skos:narrower rdf:resource="descriptor/16067-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70186">
    <skos:narrower rdf:resource="descriptor/12452-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70186">
    <skos:narrower rdf:resource="descriptor/12451-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70186">
    <skos:narrower rdf:resource="descriptor/12381-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70186">
    <skos:narrower rdf:resource="descriptor/12380-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70186">
    <skos:narrower rdf:resource="descriptor/12313-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70186">
    <skos:narrower rdf:resource="descriptor/12370-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70186">
    <skos:narrower rdf:resource="thsys/71043"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70186">
    <skos:narrower rdf:resource="thsys/71042"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71015">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.04.01.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19782-5">
    <skos:narrower rdf:resource="descriptor/18783-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19782-5">
    <skos:narrower rdf:resource="descriptor/15900-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19782-5">
    <skos:narrower rdf:resource="descriptor/18430-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19782-5">
    <skos:narrower rdf:resource="descriptor/18777-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19114-4">
    <skos:related rdf:resource="descriptor/21020-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19114-4">
    <skos:related rdf:resource="descriptor/19654-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19352-2">
    <skos:prefLabel xml:lang="en">EU enlargement</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19352-2">
    <skos:prefLabel xml:lang="de">EU-Erweiterung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12304-6">
    <skos:related rdf:resource="descriptor/12438-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12304-6">
    <skos:related rdf:resource="descriptor/12196-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12304-6">
    <skos:related rdf:resource="descriptor/19118-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12304-6">
    <skos:related rdf:resource="descriptor/18086-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12304-6">
    <skos:related rdf:resource="descriptor/10897-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12304-6">
    <skos:related rdf:resource="descriptor/18032-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19669-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16078-4">
    <skos:prefLabel xml:lang="en">Trademark law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16078-4">
    <skos:prefLabel xml:lang="de">Markenrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18414-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14450-6">
    <skos:related rdf:resource="descriptor/16326-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10115-5">
    <skos:altLabel xml:lang="de">Sachanlagevermögen der Gesamtwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10115-5">
    <skos:altLabel xml:lang="de">Realkapital</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10115-5">
    <skos:altLabel xml:lang="de">Kapitalstock</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10115-5">
    <skos:altLabel xml:lang="de">Gesamtwirtschaftliches Sachvermögen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10115-5">
    <skos:altLabel xml:lang="de">Anlagevermögen der Gesamtwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10115-5">
    <skos:altLabel xml:lang="de">Aggregiertes Anlagevermögen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19448-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13769-6">
    <skos:scopeNote xml:lang="de">An- und Verkauf, Verwahrung und Verwaltung sowie Emission von Effekten.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11707-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70438">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">G.01.01.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17745-0">
    <skos:broader rdf:resource="thsys/70085"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17745-0">
    <skos:broader rdf:resource="descriptor/17750-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17745-0">
    <skos:broader rdf:resource="descriptor/17978-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13117-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18770-3">
    <skos:altLabel xml:lang="en">Online business information</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18770-3">
    <skos:altLabel xml:lang="en">Business database</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18770-3">
    <skos:altLabel xml:lang="de">Finanzdatenbank</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19603-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14936-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15272-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13687-1">
    <skos:broader rdf:resource="thsys/70905"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13687-1">
    <skos:broader rdf:resource="thsys/70867"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13687-1">
    <skos:broader rdf:resource="descriptor/18707-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13687-1">
    <skos:broader rdf:resource="descriptor/18810-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17676-0">
    <skos:prefLabel xml:lang="en">Somalia</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17676-0">
    <skos:prefLabel xml:lang="de">Somalia</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70423">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12094-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15860-3">
    <zbwext:useInsteadNote rdf:nodeID="id271"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11817-3">
    <skos:narrower rdf:resource="descriptor/11820-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11817-3">
    <skos:narrower rdf:resource="descriptor/18507-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71089">
    <skos:broader rdf:resource="thsys/70179"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10199-1">
    <skos:related rdf:resource="descriptor/19542-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11635-2">
    <skos:related rdf:resource="descriptor/11641-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11635-2">
    <skos:related rdf:resource="descriptor/11707-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11635-2">
    <skos:related rdf:resource="descriptor/11547-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11635-2">
    <skos:related rdf:resource="descriptor/11545-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11635-2">
    <skos:related rdf:resource="descriptor/19400-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13807-3">
    <skos:broader rdf:resource="thsys/70114"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13807-3">
    <skos:broader rdf:resource="thsys/70037"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19839-2">
    <skos:scopeNote xml:lang="de">Jede Art Unterhaltspflicht zwischen Verwandten.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73357">
    <rdfs:label xml:lang="de">Lebensalter, Partnerschaft und Familie</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13813-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13369-1">
    <skos:prefLabel xml:lang="en">Newspaper publishing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13369-1">
    <skos:prefLabel xml:lang="de">Zeitungsverlag</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12341-0">
    <skos:prefLabel xml:lang="en">Account</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12341-0">
    <skos:prefLabel xml:lang="de">Konto</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15060-0">
    <skos:prefLabel xml:lang="en">Probability theory</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15060-0">
    <skos:prefLabel xml:lang="de">Wahrscheinlichkeitsrechnung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19384-3">
    <skos:related rdf:resource="descriptor/19284-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19384-3">
    <skos:related rdf:resource="descriptor/11570-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19384-3">
    <skos:related rdf:resource="descriptor/19510-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15917-0">
    <skos:related rdf:resource="descriptor/15922-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19716-4">
    <skos:scopeNote xml:lang="de">Wertende Aussagen aus Sicht der Interessen eines Staates über einen anderen. Der kritisierte Staat steht am Beginn des Indexats</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11537-2">
    <skos:prefLabel xml:lang="en">Local finance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11537-2">
    <skos:prefLabel xml:lang="de">Gemeindefinanzen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14055-5">
    <skos:related rdf:resource="descriptor/14070-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14055-5">
    <skos:related rdf:resource="descriptor/19290-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14055-5">
    <skos:related rdf:resource="descriptor/13127-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14055-5">
    <skos:related rdf:resource="descriptor/18190-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11186-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10993-2">
    <skos:prefLabel xml:lang="en">Target of economic policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10993-2">
    <skos:prefLabel xml:lang="de">Wirtschaftspolitisches Ziel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19412-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14543-6">
    <skos:altLabel xml:lang="de">Uhrwerk</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14543-6">
    <skos:altLabel xml:lang="de">Uhrenteil</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14543-6">
    <skos:altLabel xml:lang="de">Taschenuhr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14543-6">
    <skos:altLabel xml:lang="de">Digitaluhr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14543-6">
    <skos:altLabel xml:lang="de">Armbanduhr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20399-1">
    <skos:altLabel xml:lang="de">Flexibilisierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13610-1">
    <skos:broader rdf:resource="thsys/70928"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19368-1">
    <skos:related rdf:resource="descriptor/19463-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19368-1">
    <skos:related rdf:resource="descriptor/18323-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19368-1">
    <skos:related rdf:resource="descriptor/10575-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12079-1">
    <skos:broader rdf:resource="thsys/71036"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12079-1">
    <skos:broader rdf:resource="thsys/70485"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12079-1">
    <skos:broader rdf:resource="descriptor/12085-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12079-1">
    <skos:broader rdf:resource="descriptor/12165-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17198-3">
    <skos:altLabel xml:lang="x-other">Grand-Duche de Luxembourg</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17198-3">
    <skos:altLabel xml:lang="de">Großherzogtum Luxemburg</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73352">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.10.01.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14131-1">
    <skos:altLabel xml:lang="en">Beekeeping</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14131-1">
    <skos:altLabel xml:lang="de">Imkerei</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14131-1">
    <skos:altLabel xml:lang="de">Bienenzucht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14131-1">
    <skos:altLabel xml:lang="de">Apiculture</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70076">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16480-1">
    <skos:narrower rdf:resource="descriptor/16481-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16480-1">
    <skos:narrower rdf:resource="descriptor/16535-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16480-1">
    <skos:narrower rdf:resource="descriptor/16140-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16523-2">
    <skos:broader rdf:resource="thsys/73363"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16523-2">
    <skos:broader rdf:resource="thsys/70865"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15907-3">
    <skos:related rdf:resource="descriptor/16486-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15907-3">
    <skos:related rdf:resource="descriptor/19639-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15907-3">
    <skos:related rdf:resource="descriptor/15954-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15907-3">
    <skos:related rdf:resource="descriptor/19640-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15907-3">
    <skos:related rdf:resource="descriptor/15919-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15907-3">
    <skos:related rdf:resource="descriptor/15918-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15907-3">
    <skos:related rdf:resource="descriptor/15932-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15907-3">
    <skos:related rdf:resource="descriptor/15989-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14435-2">
    <skos:related rdf:resource="descriptor/14824-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14435-2">
    <skos:related rdf:resource="descriptor/13053-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14435-2">
    <skos:related rdf:resource="descriptor/14327-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14435-2">
    <skos:related rdf:resource="descriptor/19052-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13713-5">
    <skos:prefLabel xml:lang="en">Stock market</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13713-5">
    <skos:prefLabel xml:lang="de">Aktienmarkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11146-3">
    <skos:broader rdf:resource="thsys/71101"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11146-3">
    <skos:broader rdf:resource="descriptor/11143-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16297-6">
    <skos:broader rdf:resource="thsys/73375"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18029-0">
    <skos:broader rdf:resource="thsys/70485"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18029-0">
    <skos:broader rdf:resource="descriptor/19163-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18029-0">
    <skos:broader rdf:resource="descriptor/12085-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19652-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19596-2">
    <skos:related rdf:resource="descriptor/15460-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19596-2">
    <skos:related rdf:resource="descriptor/11160-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19596-2">
    <skos:related rdf:resource="descriptor/19204-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19596-2">
    <skos:related rdf:resource="descriptor/19203-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19596-2">
    <skos:related rdf:resource="descriptor/15633-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12208-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15057-3">
    <skos:prefLabel xml:lang="en">Simulation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15057-3">
    <skos:prefLabel xml:lang="de">Simulation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10656-6">
    <skos:prefLabel xml:lang="en">Manufactures trade</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10656-6">
    <skos:prefLabel xml:lang="de">Industriegüteraußenhandel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16732-0">
    <skos:altLabel xml:lang="de">Wissenschaftslehre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16732-0">
    <skos:altLabel xml:lang="de">Wissenschaftskunde</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16732-0">
    <skos:altLabel xml:lang="de">Analytische Wissenschaftstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14812-5">
    <skos:narrower rdf:resource="descriptor/18211-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14812-5">
    <skos:narrower rdf:resource="descriptor/14814-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14812-5">
    <skos:narrower rdf:resource="descriptor/13650-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12469-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10463-3">
    <skos:broader rdf:resource="thsys/71109"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10463-3">
    <skos:broader rdf:resource="thsys/70471"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10463-3">
    <skos:broader rdf:resource="descriptor/10466-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18082-3">
    <skos:scopeNote xml:lang="de">Nur für den Prozess der Ausbreitung von Neuerungen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10822-1">
    <skos:broader rdf:resource="thsys/71090"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10822-1">
    <skos:broader rdf:resource="descriptor/19231-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10822-1">
    <skos:broader rdf:resource="descriptor/10602-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18686-6">
    <skos:altLabel xml:lang="de">Euroanleihe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18686-6">
    <skos:altLabel xml:lang="de">ECU-Anleihe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14534-0">
    <skos:broader rdf:resource="thsys/70423"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14534-0">
    <skos:broader rdf:resource="descriptor/14539-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14534-0">
    <skos:broader rdf:resource="descriptor/14750-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14792-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15272-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10342-1">
    <skos:scopeNote xml:lang="de">Kurzfristige Entwicklung einer Volkswirtschaft.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11629-4">
    <skos:altLabel xml:lang="de">Öffentliches Verkehrsgewerbe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15468-3">
    <skos:altLabel xml:lang="de">Unsichere Entscheidungssituation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15468-3">
    <skos:altLabel xml:lang="de">Ungewisse Entscheidungssituation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15468-3">
    <skos:altLabel xml:lang="de">Savage-Niehans-Regel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15468-3">
    <skos:altLabel xml:lang="de">Pessimismus-Optimismus-Kriterium</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15468-3">
    <skos:altLabel xml:lang="de">Minimax-Risiko-Regel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15468-3">
    <skos:altLabel xml:lang="de">Mehrwertige Erwartung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15468-3">
    <skos:altLabel xml:lang="de">Laplace-Regel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15468-3">
    <skos:altLabel xml:lang="de">Kriterium des unzureichenden Grundes</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15468-3">
    <skos:altLabel xml:lang="de">Hurwicz-Kriterium</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15468-3">
    <skos:altLabel xml:lang="de">Entscheidungsregel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15468-3">
    <skos:altLabel xml:lang="de">Entscheidung unter Unsicherheit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15468-3">
    <skos:altLabel xml:lang="de">Entscheidung bei Ungewissheit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15468-3">
    <skos:altLabel xml:lang="de">Entscheidung bei Ungewißheit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13157-2">
    <skos:related rdf:resource="descriptor/12958-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13157-2">
    <skos:related rdf:resource="descriptor/13207-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13157-2">
    <skos:related rdf:resource="descriptor/14976-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70080">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19050-4">
    <skos:prefLabel xml:lang="en">Mortar</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19050-4">
    <skos:prefLabel xml:lang="de">Mörtel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18151-3">
    <skos:altLabel xml:lang="de">Wirtschaftlichkeitsprinzip</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18151-3">
    <skos:altLabel xml:lang="de">Wirtschaftliches Prinzip</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19966-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18294-2">
    <skos:prefLabel xml:lang="en">High-speed train</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18294-2">
    <skos:prefLabel xml:lang="de">Hochgeschwindigkeitszug</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13405-2">
    <skos:related rdf:resource="descriptor/19830-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13405-2">
    <skos:related rdf:resource="descriptor/16543-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13405-2">
    <skos:related rdf:resource="descriptor/18906-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70004">
    <rdfs:label xml:lang="en">Company valuation</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70004">
    <rdfs:label xml:lang="de">Unternehmensbewertung</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19720-6">
    <skos:scopeNote xml:lang="de">Politische Entscheidungen über Auf- oder Abrüstung, zwischen Berufsarmee und Wehrpflicht, Beteiligung der Armee an internationalen Einsätzen etc.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13128-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11299-6">
    <zbwext:useInsteadNote rdf:nodeID="id93"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16365-1">
    <skos:altLabel xml:lang="de">Staatliche Parteienfinanzierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16365-1">
    <skos:altLabel xml:lang="de">Parteispende</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16365-1">
    <skos:altLabel xml:lang="de">Parteifinanzierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18666-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70943">
    <skos:prefLabel xml:lang="de">W.12  Verkehr und Tourismus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18366-3">
    <skos:related rdf:resource="descriptor/18447-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18366-3">
    <skos:related rdf:resource="descriptor/12925-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18366-3">
    <skos:related rdf:resource="descriptor/15005-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18366-3">
    <skos:related rdf:resource="descriptor/18814-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18366-3">
    <skos:related rdf:resource="descriptor/19725-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18366-3">
    <skos:related rdf:resource="descriptor/16271-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18366-3">
    <skos:related rdf:resource="descriptor/13412-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18366-3">
    <skos:related rdf:resource="descriptor/13575-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13002-3">
    <skos:prefLabel xml:lang="en">Pharmaceutical industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13002-3">
    <skos:prefLabel xml:lang="de">Pharmazeutische Industrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13325-0">
    <skos:narrower rdf:resource="descriptor/13324-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13325-0">
    <skos:narrower rdf:resource="descriptor/13333-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13325-0">
    <skos:narrower rdf:resource="descriptor/18960-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13325-0">
    <skos:narrower rdf:resource="descriptor/13329-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15328-5">
    <skos:prefLabel xml:lang="en">Autocorrelation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15328-5">
    <skos:prefLabel xml:lang="de">Autokorrelation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11079-6">
    <skos:prefLabel xml:lang="en">Use-value</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11079-6">
    <skos:prefLabel xml:lang="de">Gebrauchswert</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11709-6">
    <skos:narrower rdf:resource="descriptor/10830-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13369-1">
    <skos:related rdf:resource="descriptor/13124-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13369-1">
    <skos:related rdf:resource="descriptor/13670-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71012">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15033-3">
    <skos:prefLabel xml:lang="en">Entropy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15033-3">
    <skos:prefLabel xml:lang="de">Entropie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11065-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16650-2">
    <skos:related rdf:resource="descriptor/15714-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16650-2">
    <skos:related rdf:resource="descriptor/10595-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16650-2">
    <skos:related rdf:resource="descriptor/16226-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16650-2">
    <skos:related rdf:resource="descriptor/10984-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16650-2">
    <skos:related rdf:resource="descriptor/10501-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18964-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11869-5">
    <skos:related rdf:resource="descriptor/13573-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11869-5">
    <skos:related rdf:resource="descriptor/19611-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11869-5">
    <skos:related rdf:resource="descriptor/11900-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11869-5">
    <skos:related rdf:resource="descriptor/11951-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11869-5">
    <skos:related rdf:resource="descriptor/11857-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11869-5">
    <skos:related rdf:resource="descriptor/11928-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11869-5">
    <skos:related rdf:resource="descriptor/11870-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11869-5">
    <skos:related rdf:resource="descriptor/11859-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18496-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16642-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12004-4">
    <skos:narrower rdf:resource="descriptor/11828-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10751-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10301-1">
    <skos:narrower rdf:resource="descriptor/10302-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10301-1">
    <skos:narrower rdf:resource="descriptor/10178-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10301-1">
    <skos:narrower rdf:resource="descriptor/10170-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11406-3">
    <skos:narrower rdf:resource="descriptor/18879-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15961-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19373-1">
    <skos:related rdf:resource="descriptor/13418-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19373-1">
    <skos:related rdf:resource="descriptor/13393-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19373-1">
    <skos:related rdf:resource="descriptor/18850-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14015-3">
    <skos:altLabel xml:lang="de">Soziologinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14015-3">
    <skos:altLabel xml:lang="de">Soziologen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14015-3">
    <skos:altLabel xml:lang="de">Sozialwissenschaftlerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14015-3">
    <skos:altLabel xml:lang="de">Sozialökonominnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14015-3">
    <skos:altLabel xml:lang="de">Sozialökonomen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14015-3">
    <skos:altLabel xml:lang="de">Politologinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14015-3">
    <skos:altLabel xml:lang="de">Politologen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14015-3">
    <skos:altLabel xml:lang="de">Kriminologinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14015-3">
    <skos:altLabel xml:lang="de">Kriminologen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14015-3">
    <skos:altLabel xml:lang="de">Ergonominnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14015-3">
    <skos:altLabel xml:lang="de">Ergonomen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19827-2">
    <skos:prefLabel xml:lang="en">Market efficiency</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19827-2">
    <skos:prefLabel xml:lang="de">Markteffizienz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17721-0">
    <skos:prefLabel xml:lang="en">Senegal</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17721-0">
    <skos:prefLabel xml:lang="de">Senegal</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14537-1">
    <skos:related rdf:resource="descriptor/13081-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14537-1">
    <skos:related rdf:resource="descriptor/13080-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14537-1">
    <skos:related rdf:resource="descriptor/14465-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14537-1">
    <skos:related rdf:resource="descriptor/14539-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70175">
    <skos:prefLabel xml:lang="en">V.00  Economy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70175">
    <skos:prefLabel xml:lang="de">V.00  Volkswirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13024-0">
    <skos:prefLabel xml:lang="en">Brick industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13024-0">
    <skos:prefLabel xml:lang="de">Ziegelindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14079-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19554-4">
    <skos:altLabel xml:lang="en">Noise trader</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11566-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15930-1">
    <skos:altLabel xml:lang="de">Nomadentum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70647">
    <skos:broader rdf:resource="thsys/70348"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12713-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73366">
    <rdfs:label xml:lang="de">Internationale Beziehungen und Außenpolitik</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19780-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18542-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17703-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19031-1">
    <skos:broader rdf:resource="thsys/70582"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16543-3">
    <skos:altLabel xml:lang="de">Soziale Dienstleistungen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16543-3">
    <skos:altLabel xml:lang="de">Soziale Beratung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16543-3">
    <skos:altLabel xml:lang="de">Sozialdienst</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16543-3">
    <skos:altLabel xml:lang="de">Sozialberatung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16543-3">
    <skos:altLabel xml:lang="de">Sozialarbeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13324-2">
    <skos:altLabel xml:lang="en">Waste recycling</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13324-2">
    <skos:altLabel xml:lang="de">Müllverwertung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13324-2">
    <skos:altLabel xml:lang="de">Kreislaufwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13324-2">
    <skos:altLabel xml:lang="de">Abfallverwertung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13324-2">
    <skos:altLabel xml:lang="de">Abfallrecycling</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13324-2">
    <skos:altLabel xml:lang="de">Abfallbörse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13056-1">
    <skos:related rdf:resource="descriptor/13578-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13056-1">
    <skos:related rdf:resource="descriptor/14423-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13056-1">
    <skos:related rdf:resource="descriptor/18419-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13056-1">
    <skos:related rdf:resource="descriptor/13507-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13056-1">
    <skos:related rdf:resource="descriptor/18418-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19403-4">
    <skos:broader rdf:resource="thsys/71076"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19403-4">
    <skos:broader rdf:resource="descriptor/19408-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19403-4">
    <skos:broader rdf:resource="descriptor/11636-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20553-3">
    <skos:altLabel xml:lang="en">Micro-financing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20553-3">
    <skos:altLabel xml:lang="en">Microcredit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20553-3">
    <skos:altLabel xml:lang="de">Mikrokreditprogramm</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20553-3">
    <skos:altLabel xml:lang="de">Mikrokredit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13766-5">
    <skos:related rdf:resource="descriptor/18157-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18349-3">
    <skos:related rdf:resource="descriptor/18458-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18349-3">
    <skos:related rdf:resource="descriptor/11811-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70582">
    <skos:broader rdf:resource="thsys/a"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70892">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11686-6">
    <skos:broader rdf:resource="thsys/71076"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11686-6">
    <skos:broader rdf:resource="descriptor/11684-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16030-4">
    <skos:prefLabel xml:lang="en">Family law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16030-4">
    <skos:prefLabel xml:lang="de">Familienrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14642-4">
    <skos:narrower rdf:resource="descriptor/18253-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14642-4">
    <skos:narrower rdf:resource="descriptor/18254-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14642-4">
    <skos:narrower rdf:resource="descriptor/14654-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14642-4">
    <skos:narrower rdf:resource="descriptor/14652-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14642-4">
    <skos:narrower rdf:resource="descriptor/14627-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14642-4">
    <skos:narrower rdf:resource="descriptor/14646-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14642-4">
    <skos:narrower rdf:resource="descriptor/14645-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14642-4">
    <skos:narrower rdf:resource="descriptor/14643-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14642-4">
    <skos:narrower rdf:resource="descriptor/14608-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14642-4">
    <skos:narrower rdf:resource="descriptor/18080-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14642-4">
    <skos:narrower rdf:resource="descriptor/14636-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14642-4">
    <skos:narrower rdf:resource="descriptor/18255-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19365-0">
    <skos:related rdf:resource="descriptor/18200-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19365-0">
    <skos:related rdf:resource="descriptor/18189-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19365-0">
    <skos:related rdf:resource="descriptor/13004-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18141-6">
    <skos:broader rdf:resource="thsys/70899"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18141-6">
    <skos:broader rdf:resource="descriptor/13756-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71083">
    <skos:narrower rdf:resource="descriptor/11723-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71083">
    <skos:narrower rdf:resource="descriptor/11721-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71083">
    <skos:narrower rdf:resource="descriptor/18156-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71083">
    <skos:narrower rdf:resource="descriptor/11718-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71083">
    <skos:narrower rdf:resource="descriptor/11717-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71083">
    <skos:narrower rdf:resource="descriptor/19377-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71083">
    <skos:narrower rdf:resource="descriptor/19212-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71083">
    <skos:narrower rdf:resource="descriptor/19376-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71083">
    <skos:narrower rdf:resource="descriptor/19371-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71083">
    <skos:narrower rdf:resource="descriptor/19374-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71083">
    <skos:narrower rdf:resource="descriptor/19206-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71083">
    <skos:narrower rdf:resource="descriptor/11526-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71083">
    <skos:narrower rdf:resource="descriptor/11715-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71083">
    <skos:narrower rdf:resource="descriptor/11711-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17464-1">
    <skos:scopeNote xml:lang="de">Erklärte sich 1990 unabhängig von der Sowjetunion und trat 1991 der GUS bei</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20756-3">
    <skos:broader rdf:resource="thsys/73103"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20756-3">
    <skos:broader rdf:resource="descriptor/15466-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11325-3">
    <skos:broader rdf:resource="thsys/70278"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11325-3">
    <skos:broader rdf:resource="descriptor/11519-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14095-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15745-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14318-6">
    <skos:scopeNote xml:lang="de">Auch für komplexe Transportsysteme mit Containern.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19497-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18976-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13165-3">
    <skos:prefLabel xml:lang="en">Sugar industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13165-3">
    <skos:prefLabel xml:lang="de">Zuckerindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13561-2">
    <skos:broader rdf:resource="thsys/70970"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13561-2">
    <skos:broader rdf:resource="thsys/70938"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13561-2">
    <skos:broader rdf:resource="descriptor/18624-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19588-1">
    <skos:prefLabel xml:lang="en">Commodity statistics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19588-1">
    <skos:prefLabel xml:lang="de">Rohstoffstatistik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19077-5">
    <skos:narrower rdf:resource="descriptor/17465-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19077-5">
    <skos:narrower rdf:resource="descriptor/17485-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19077-5">
    <skos:narrower rdf:resource="descriptor/17534-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16732-0">
    <skos:prefLabel xml:lang="en">Philosophy of science</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16732-0">
    <skos:prefLabel xml:lang="de">Wissenschaftstheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16252-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11448-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18139-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21082-4">
    <skos:scopeNote xml:lang="de">Verteilungswirkung von politischen oder ökonomischen Maßnahmen oder Ereignissen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11636-0">
    <skos:narrower rdf:resource="descriptor/19143-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11636-0">
    <skos:narrower rdf:resource="descriptor/19403-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11636-0">
    <skos:narrower rdf:resource="descriptor/19402-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11180-3">
    <skos:related rdf:resource="descriptor/16011-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11180-3">
    <skos:related rdf:resource="descriptor/11322-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11180-3">
    <skos:related rdf:resource="descriptor/11273-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11180-3">
    <skos:related rdf:resource="descriptor/11187-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11180-3">
    <skos:related rdf:resource="descriptor/12602-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11180-3">
    <skos:related rdf:resource="descriptor/11309-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11180-3">
    <skos:related rdf:resource="descriptor/11185-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11180-3">
    <skos:related rdf:resource="descriptor/10440-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11180-3">
    <skos:related rdf:resource="descriptor/11183-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11180-3">
    <skos:related rdf:resource="descriptor/11262-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11180-3">
    <skos:related rdf:resource="descriptor/10304-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11180-3">
    <skos:related rdf:resource="descriptor/11181-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11180-3">
    <skos:related rdf:resource="descriptor/11305-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11786-2">
    <skos:broader rdf:resource="thsys/71110"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18233-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19599-3">
    <skos:related rdf:resource="descriptor/12837-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19599-3">
    <skos:related rdf:resource="descriptor/15858-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19599-3">
    <skos:related rdf:resource="descriptor/15787-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19599-3">
    <skos:related rdf:resource="descriptor/19772-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19599-3">
    <skos:related rdf:resource="descriptor/19601-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19599-3">
    <skos:related rdf:resource="descriptor/10150-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15633-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13274-5">
    <skos:prefLabel xml:lang="en">Postal services</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13274-5">
    <skos:prefLabel xml:lang="de">Postdienst</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15001-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16328-0">
    <skos:broader rdf:resource="thsys/73373"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16328-0">
    <skos:broader rdf:resource="thsys/70862"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16328-0">
    <skos:broader rdf:resource="descriptor/13413-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14074-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17669-4">
    <skos:broader rdf:resource="thsys/70445"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17669-4">
    <skos:broader rdf:resource="descriptor/17672-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18854-4">
    <skos:broader rdf:resource="thsys/70851"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70120">
    <skos:prefLabel xml:lang="de">N.06.03  Soziales Verhalten und soziale Gruppen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18176-1">
    <skos:broader rdf:resource="thsys/70241"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18176-1">
    <skos:broader rdf:resource="descriptor/16042-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11201-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15714-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15057-3">
    <skos:related rdf:resource="descriptor/15067-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15057-3">
    <skos:related rdf:resource="descriptor/15483-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18761-4">
    <skos:altLabel xml:lang="de">Urheberrechtlicher Softwareschutz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18761-4">
    <skos:altLabel xml:lang="de">Softwareschutzrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18761-4">
    <skos:altLabel xml:lang="de">Softwarerecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18761-4">
    <skos:altLabel xml:lang="de">Schutz von Computerprogrammen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12920-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19415-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19158-5">
    <skos:altLabel xml:lang="de">Unternehmensfortführung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19158-5">
    <skos:altLabel xml:lang="de">Going-Concern-Prinzip</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71024">
    <skos:narrower rdf:resource="descriptor/21094-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71024">
    <skos:narrower rdf:resource="descriptor/11149-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71024">
    <skos:narrower rdf:resource="descriptor/11151-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71024">
    <skos:narrower rdf:resource="descriptor/18137-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71024">
    <skos:narrower rdf:resource="descriptor/13453-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71024">
    <skos:narrower rdf:resource="descriptor/18986-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71024">
    <skos:narrower rdf:resource="descriptor/18313-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71024">
    <skos:narrower rdf:resource="descriptor/18093-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71024">
    <skos:narrower rdf:resource="descriptor/13440-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71024">
    <skos:narrower rdf:resource="descriptor/13437-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71024">
    <skos:narrower rdf:resource="descriptor/19045-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71024">
    <skos:narrower rdf:resource="descriptor/18326-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71024">
    <skos:narrower rdf:resource="descriptor/12936-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71024">
    <skos:narrower rdf:resource="descriptor/13452-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71024">
    <skos:narrower rdf:resource="descriptor/18324-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71024">
    <skos:narrower rdf:resource="descriptor/13429-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#exactMatch">
    <rdfs:comment xml:lang="en">skos:exactMatch is disjoint with each of the properties
      skos:broadMatch and skos:relatedMatch.</rdfs:comment>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11494-1">
    <skos:prefLabel xml:lang="en">Anti-inflation policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11494-1">
    <skos:prefLabel xml:lang="de">Inflationsbekämpfung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19630-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70306">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14044-3">
    <skos:narrower rdf:resource="descriptor/14962-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14044-3">
    <skos:narrower rdf:resource="descriptor/14958-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14044-3">
    <skos:narrower rdf:resource="descriptor/14217-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14044-3">
    <skos:narrower rdf:resource="descriptor/14961-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14158-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19380-4">
    <skos:scopeNote xml:lang="de">Verfahren bei Aufstellung, Entscheidung, Vollzug und Kontrolle des öffentlichen Haushalts.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11146-3">
    <skos:scopeNote xml:lang="de">Nur für sozialistische Unternehmen: die Maßnahmen zur Gewährleistung des effektiven Einsatzes der materiellen und finanziellen Ressourcen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10590-3">
    <skos:prefLabel xml:lang="en">Project evaluation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10590-3">
    <skos:prefLabel xml:lang="de">Projektbewertung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18969-1">
    <skos:altLabel xml:lang="de">Wohngemeinschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18969-1">
    <skos:altLabel xml:lang="de">Nichteheliche Partnerschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18969-1">
    <skos:altLabel xml:lang="de">Nichteheliche Lebensgemeinschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18969-1">
    <skos:altLabel xml:lang="de">Lebenspartner</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18969-1">
    <skos:altLabel xml:lang="de">Eheähnliche Partnerschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18969-1">
    <skos:altLabel xml:lang="de">Eheähnliche Gemeinschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12962-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10706-3">
    <skos:narrower rdf:resource="descriptor/10685-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12251-1">
    <skos:prefLabel xml:lang="en">Credit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12251-1">
    <skos:prefLabel xml:lang="de">Kredit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13071-5">
    <skos:broader rdf:resource="thsys/73297"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13071-5">
    <skos:broader rdf:resource="descriptor/13072-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13071-5">
    <skos:broader rdf:resource="descriptor/18454-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13729-4">
    <skos:prefLabel xml:lang="en">Insider trading</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13729-4">
    <skos:prefLabel xml:lang="de">Insiderhandel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19770-5">
    <skos:related rdf:resource="descriptor/13349-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12029-2">
    <skos:narrower rdf:resource="descriptor/18749-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12029-2">
    <skos:narrower rdf:resource="descriptor/19398-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12029-2">
    <skos:narrower rdf:resource="descriptor/18329-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12029-2">
    <skos:narrower rdf:resource="descriptor/18122-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12029-2">
    <skos:narrower rdf:resource="descriptor/18496-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18144-0">
    <skos:altLabel xml:lang="de">Hausmänner</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18144-0">
    <skos:altLabel xml:lang="de">Hausmann</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18144-0">
    <skos:altLabel xml:lang="de">Hausfrauen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18144-0">
    <skos:altLabel xml:lang="de">Hausfrau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13722-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14883-3">
    <skos:related rdf:resource="descriptor/14810-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70234">
    <rdfs:label xml:lang="en">Macroeconomics</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70234">
    <rdfs:label xml:lang="de">Makroökonomik</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71085">
    <skos:prefLabel xml:lang="en">V.08.02  Development policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71085">
    <skos:prefLabel xml:lang="de">V.08.02  Entwicklungspolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11066-1">
    <skos:related rdf:resource="descriptor/11068-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11066-1">
    <skos:related rdf:resource="descriptor/11063-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11066-1">
    <skos:related rdf:resource="descriptor/11127-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11066-1">
    <skos:related rdf:resource="descriptor/11049-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11066-1">
    <skos:related rdf:resource="descriptor/10976-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11066-1">
    <skos:related rdf:resource="descriptor/19989-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11066-1">
    <skos:related rdf:resource="descriptor/11160-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11066-1">
    <skos:related rdf:resource="descriptor/11065-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70445">
    <rdfs:label xml:lang="en">East Africa</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70445">
    <rdfs:label xml:lang="de">Ostafrika</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16032-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13678-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13365-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15712-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70560">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10498-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17780-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15972-6">
    <skos:broader rdf:resource="thsys/70154"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18839-0">
    <skos:broader rdf:resource="thsys/70854"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18839-0">
    <skos:broader rdf:resource="descriptor/18834-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18981-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18318-0">
    <skos:altLabel xml:lang="de">Landwirtschaftliche Marktlehre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18318-0">
    <skos:altLabel xml:lang="de">Agrarökonomische Forschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18318-0">
    <skos:altLabel xml:lang="de">Agrarökonomie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12760-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10865-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13722-4">
    <skos:broader rdf:resource="thsys/71104"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13722-4">
    <skos:broader rdf:resource="descriptor/10221-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13722-4">
    <skos:broader rdf:resource="descriptor/18668-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12146-5">
    <skos:broader rdf:resource="thsys/70562"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12146-5">
    <skos:broader rdf:resource="descriptor/12111-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15912-3">
    <skos:altLabel xml:lang="de">Bevölkerungsvorausschätzung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15912-3">
    <skos:altLabel xml:lang="de">Bevölkerungsvorausberechnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15912-3">
    <skos:altLabel xml:lang="de">Bevölkerungstrend (Prognose)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12103-2">
    <skos:altLabel xml:lang="en">Contingency approach</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12103-2">
    <skos:altLabel xml:lang="de">Situativer Ansatz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19564-1">
    <skos:related rdf:resource="descriptor/20306-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19564-1">
    <skos:related rdf:resource="descriptor/12809-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19564-1">
    <skos:related rdf:resource="descriptor/19034-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19564-1">
    <skos:related rdf:resource="descriptor/16078-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19564-1">
    <skos:related rdf:resource="descriptor/19129-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19564-1">
    <skos:related rdf:resource="descriptor/12800-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19564-1">
    <skos:related rdf:resource="descriptor/19316-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16010-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18497-2">
    <skos:broader rdf:resource="thsys/70969"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18497-2">
    <skos:broader rdf:resource="descriptor/12905-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18497-2">
    <skos:broader rdf:resource="descriptor/10822-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18202-5">
    <skos:broader rdf:resource="thsys/70465"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18202-5">
    <skos:broader rdf:resource="descriptor/14150-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20136-5">
    <skos:broader rdf:resource="thsys/70958"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20136-5">
    <skos:broader rdf:resource="descriptor/13301-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12607-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10343-6">
    <skos:related rdf:resource="descriptor/10342-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10343-6">
    <skos:related rdf:resource="descriptor/19285-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10343-6">
    <skos:related rdf:resource="descriptor/10413-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10343-6">
    <skos:related rdf:resource="descriptor/10412-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10343-6">
    <skos:related rdf:resource="descriptor/16295-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10343-6">
    <skos:related rdf:resource="descriptor/19683-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10343-6">
    <skos:related rdf:resource="descriptor/10348-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10343-6">
    <skos:related rdf:resource="descriptor/10338-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18000-3">
    <skos:prefLabel xml:lang="en">Telecommunications network</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18000-3">
    <skos:prefLabel xml:lang="de">Telekommunikationsnetz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12901-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73305">
    <skos:narrower rdf:resource="descriptor/15621-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73305">
    <skos:narrower rdf:resource="descriptor/14762-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73305">
    <skos:narrower rdf:resource="descriptor/18004-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73305">
    <skos:narrower rdf:resource="descriptor/18270-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73305">
    <skos:narrower rdf:resource="descriptor/14514-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73305">
    <skos:narrower rdf:resource="descriptor/14614-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73305">
    <skos:narrower rdf:resource="descriptor/14759-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73305">
    <skos:narrower rdf:resource="descriptor/18269-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73305">
    <skos:narrower rdf:resource="descriptor/14755-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73305">
    <skos:narrower rdf:resource="descriptor/14749-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73305">
    <skos:narrower rdf:resource="descriptor/14752-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73305">
    <skos:narrower rdf:resource="descriptor/14751-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73305">
    <skos:narrower rdf:resource="descriptor/18271-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73305">
    <skos:narrower rdf:resource="descriptor/18075-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73305">
    <skos:narrower rdf:resource="descriptor/14750-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73305">
    <skos:narrower rdf:resource="descriptor/14765-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19721-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70337">
    <skos:broader rdf:resource="thsys/p"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16142-0">
    <skos:related rdf:resource="descriptor/11191-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18990-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14336-4">
    <skos:altLabel xml:lang="de">Zahnrad</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14336-4">
    <skos:altLabel xml:lang="de">Waage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14336-4">
    <skos:altLabel xml:lang="de">Verdichter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14336-4">
    <skos:altLabel xml:lang="de">Sprinkleranlage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14336-4">
    <skos:altLabel xml:lang="de">Kompressor</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14336-4">
    <skos:altLabel xml:lang="de">Industrieausrüstung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14336-4">
    <skos:altLabel xml:lang="de">Feuerlöschgerät</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14336-4">
    <skos:altLabel xml:lang="de">Fahrradkette</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70557">
    <skos:prefLabel xml:lang="en">G.01.05  Western Europe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70557">
    <skos:prefLabel xml:lang="de">G.01.05  Westeuropa</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15064-6">
    <skos:altLabel xml:lang="en">Statistic method</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15064-6">
    <skos:altLabel xml:lang="de">Statistisches Verfahren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15064-6">
    <skos:altLabel xml:lang="de">Statistische Analyse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15064-6">
    <skos:altLabel xml:lang="de">Statistikanwendung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15468-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15965-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18637-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18507-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14945-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12089-5">
    <skos:narrower rdf:resource="descriptor/19565-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12089-5">
    <skos:narrower rdf:resource="descriptor/12770-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12089-5">
    <skos:narrower rdf:resource="descriptor/12217-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12089-5">
    <skos:narrower rdf:resource="descriptor/12224-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13327-3">
    <skos:prefLabel xml:lang="en">Sewage treatment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13327-3">
    <skos:prefLabel xml:lang="de">Abwasserreinigung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12071-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12982-4">
    <skos:related rdf:resource="descriptor/14195-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19192-5">
    <skos:broader rdf:resource="thsys/71009"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19192-5">
    <skos:broader rdf:resource="thsys/70957"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19192-5">
    <skos:broader rdf:resource="descriptor/18996-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12488-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16802-5">
    <skos:altLabel xml:lang="en">Library cooperative</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16802-5">
    <skos:altLabel xml:lang="de">Verbundkatalog</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16802-5">
    <skos:altLabel xml:lang="de">Regionaler Bibliotheksverbund</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16802-5">
    <skos:altLabel xml:lang="de">Bibliotheksnetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20491-6">
    <skos:altLabel xml:lang="en">Lesbian</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20491-6">
    <skos:altLabel xml:lang="en">Gay men</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20491-6">
    <skos:altLabel xml:lang="de">Schwule</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20491-6">
    <skos:altLabel xml:lang="de">Lesbische Liebe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20491-6">
    <skos:altLabel xml:lang="de">Homosexuelle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20491-6">
    <skos:altLabel xml:lang="de">Gleichgeschlechtliche Liebe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14231-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19522-3">
    <skos:related rdf:resource="descriptor/12186-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19522-3">
    <skos:related rdf:resource="descriptor/12282-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19522-3">
    <skos:related rdf:resource="descriptor/24733-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19522-3">
    <skos:related rdf:resource="descriptor/19161-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19522-3">
    <skos:related rdf:resource="descriptor/19154-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19522-3">
    <skos:related rdf:resource="descriptor/13714-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19522-3">
    <skos:related rdf:resource="descriptor/19535-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19522-3">
    <skos:related rdf:resource="descriptor/11153-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12525-4">
    <skos:prefLabel xml:lang="en">Full costing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12525-4">
    <skos:prefLabel xml:lang="de">Vollkostenrechnung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11591-3">
    <skos:broader rdf:resource="thsys/73367"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11591-3">
    <skos:broader rdf:resource="thsys/71081"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11591-3">
    <skos:broader rdf:resource="thsys/71064"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11591-3">
    <skos:broader rdf:resource="descriptor/11596-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11591-3">
    <skos:broader rdf:resource="descriptor/11537-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18438-4">
    <skos:scopeNote xml:lang="de">Warentermingeschäft zur Ausschaltung von Preisrisiken bei Welthandelsrohstoffen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10618-0">
    <skos:altLabel xml:lang="en">International trade organization</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10618-0">
    <skos:altLabel xml:lang="de">Internationale Wirtschaftsinstitution</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10618-0">
    <skos:altLabel xml:lang="de">Internationale Handelsorganisation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13338-5">
    <skos:prefLabel xml:lang="en">Pest control</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13338-5">
    <skos:prefLabel xml:lang="de">Schädlingsbekämpfung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10911-2">
    <skos:prefLabel xml:lang="en">Business concentration</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10911-2">
    <skos:prefLabel xml:lang="de">Unternehmenszusammenschluss</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19164-3">
    <skos:altLabel xml:lang="de">Strategische Partnerschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15741-4">
    <skos:broader rdf:resource="thsys/73351"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15741-4">
    <skos:broader rdf:resource="thsys/70811"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15741-4">
    <skos:broader rdf:resource="thsys/70341"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15741-4">
    <skos:broader rdf:resource="descriptor/14074-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15741-4">
    <skos:broader rdf:resource="descriptor/18286-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16210-2">
    <skos:broader rdf:resource="thsys/70141"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16210-2">
    <skos:broader rdf:resource="descriptor/15723-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14437-5">
    <skos:altLabel xml:lang="de">Motorradzubehör</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14437-5">
    <skos:altLabel xml:lang="de">Motorisiertes Zweirad</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14437-5">
    <skos:altLabel xml:lang="de">Moped</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14437-5">
    <skos:altLabel xml:lang="de">Mofa</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14437-5">
    <skos:altLabel xml:lang="de">Kraftrad</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19428-2">
    <skos:altLabel xml:lang="en">Strategic network</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19428-2">
    <skos:altLabel xml:lang="en">Organization network</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19428-2">
    <skos:altLabel xml:lang="en">Network economy</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19428-2">
    <skos:altLabel xml:lang="de">Wirtschaftsnetzwerk</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19428-2">
    <skos:altLabel xml:lang="de">Netzwerk (Business network)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19428-2">
    <skos:altLabel xml:lang="de">Geschäftsbeziehung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18619-0">
    <skos:scopeNote xml:lang="de">Hersteller, Großhändler und Importeure von Tiefkühlwaren; Transporteure, Heimlieferanten, Kühlhäuser; Hersteller von Kühltechnik und Kältemitteln</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24661-1">
    <skos:scopeNote xml:lang="de">Gezieltes Management von Veränderungsprozessen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16314-4">
    <skos:prefLabel xml:lang="en">Federalism</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16314-4">
    <skos:prefLabel xml:lang="de">Föderalismus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10472-2">
    <skos:broader rdf:resource="thsys/71109"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10472-2">
    <skos:broader rdf:resource="thsys/71084"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10472-2">
    <skos:broader rdf:resource="thsys/70879"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10472-2">
    <skos:broader rdf:resource="thsys/70179"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10472-2">
    <skos:broader rdf:resource="descriptor/18869-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17978-5">
    <skos:related rdf:resource="descriptor/17750-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17978-5">
    <skos:related rdf:resource="descriptor/17608-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17978-5">
    <skos:related rdf:resource="descriptor/17988-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17978-5">
    <skos:related rdf:resource="descriptor/18177-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12719-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16761-0">
    <skos:prefLabel xml:lang="en">Communication science</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16761-0">
    <skos:prefLabel xml:lang="de">Kommunikationswissenschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18781-5">
    <skos:broader rdf:resource="thsys/70878"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18781-5">
    <skos:broader rdf:resource="thsys/70844"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18781-5">
    <skos:broader rdf:resource="descriptor/11373-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18781-5">
    <skos:broader rdf:resource="descriptor/13343-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70872">
    <skos:prefLabel xml:lang="en">W.22  Public Sector</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70872">
    <skos:prefLabel xml:lang="de">W.22  Öffentlicher Sektor</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13726-3">
    <skos:related rdf:resource="descriptor/18088-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13726-3">
    <skos:related rdf:resource="descriptor/11421-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18645-6">
    <skos:broader rdf:resource="thsys/70923"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12957-3">
    <skos:prefLabel xml:lang="en">Grazing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12957-3">
    <skos:prefLabel xml:lang="de">Weidewirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12775-2">
    <skos:prefLabel xml:lang="en">Public Relations</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12775-2">
    <skos:prefLabel xml:lang="de">Öffentlichkeitsarbeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id17">
    <rdf:value xml:lang="de">für Wirtschaftsanthropologie</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14232-2">
    <skos:altLabel xml:lang="de">Kreide</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14232-2">
    <skos:altLabel xml:lang="de">Kohlensaurer Kalk</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14232-2">
    <skos:altLabel xml:lang="de">Kalkstein</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14232-2">
    <skos:altLabel xml:lang="de">Calciumcarbonat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70554">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11940-4">
    <skos:prefLabel xml:lang="en">Regional industrial promotion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11940-4">
    <skos:prefLabel xml:lang="de">Gewerbeansiedlung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10753-1">
    <skos:prefLabel xml:lang="en">Sterling</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10753-1">
    <skos:prefLabel xml:lang="de">Pfund Sterling</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13360-5">
    <skos:prefLabel xml:lang="en">Radio station</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13360-5">
    <skos:prefLabel xml:lang="de">Rundfunkanstalt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18173-0">
    <skos:broader rdf:resource="thsys/71062"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18173-0">
    <skos:broader rdf:resource="thsys/70278"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18173-0">
    <skos:broader rdf:resource="descriptor/11336-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12905-1">
    <skos:related rdf:resource="descriptor/18399-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12905-1">
    <skos:related rdf:resource="descriptor/18747-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12905-1">
    <skos:related rdf:resource="descriptor/18495-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12905-1">
    <skos:related rdf:resource="descriptor/12904-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12905-1">
    <skos:related rdf:resource="descriptor/14021-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12905-1">
    <skos:related rdf:resource="descriptor/12717-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12905-1">
    <skos:related rdf:resource="descriptor/15101-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12905-1">
    <skos:related rdf:resource="descriptor/16176-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12905-1">
    <skos:related rdf:resource="descriptor/18507-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12905-1">
    <skos:related rdf:resource="descriptor/13045-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12905-1">
    <skos:related rdf:resource="descriptor/14366-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12905-1">
    <skos:related rdf:resource="descriptor/13852-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12905-1">
    <skos:related rdf:resource="descriptor/18491-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12905-1">
    <skos:related rdf:resource="descriptor/18504-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12905-1">
    <skos:related rdf:resource="descriptor/13953-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12905-1">
    <skos:related rdf:resource="descriptor/18492-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14808-3">
    <skos:related rdf:resource="descriptor/14895-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13444-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13534-5">
    <skos:prefLabel xml:lang="en">Port infrastructure</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13534-5">
    <skos:prefLabel xml:lang="de">Hafenanlagen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19336-0">
    <skos:altLabel xml:lang="en">European Monetary Union</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19336-0">
    <skos:altLabel xml:lang="en">European Currency Union</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19336-0">
    <skos:altLabel xml:lang="en">EMU (European Monetary Union)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19336-0">
    <skos:altLabel xml:lang="en">EEMU (European Economic and Monetary Union)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19336-0">
    <skos:altLabel xml:lang="en">EEMU</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19336-0">
    <skos:altLabel xml:lang="de">Maastricht-Vertrag</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19336-0">
    <skos:altLabel xml:lang="de">EWWU (Europäische Wirtschafts- und Währungsunion)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19336-0">
    <skos:altLabel xml:lang="de">EWWU</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19336-0">
    <skos:altLabel xml:lang="de">EWU (Europäische Währungsunion)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19336-0">
    <skos:altLabel xml:lang="de">Europäische Währungsunion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19336-0">
    <skos:altLabel xml:lang="de">Euroland</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10293-2">
    <skos:broader rdf:resource="thsys/71044"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10293-2">
    <skos:broader rdf:resource="thsys/70530"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19692-6">
    <skos:altLabel xml:lang="en">Parliament voting</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19692-6">
    <skos:altLabel xml:lang="de">Parlamentsabstimmung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73361">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19817-5">
    <skos:related rdf:resource="descriptor/19727-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19817-5">
    <skos:related rdf:resource="descriptor/18961-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11869-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19740-0">
    <skos:prefLabel xml:lang="en">Protestantism</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19740-0">
    <skos:prefLabel xml:lang="de">Protestantismus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14933-0">
    <skos:narrower rdf:resource="descriptor/14995-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11592-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70051">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.13.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16713-4">
    <skos:scopeNote xml:lang="de">Stellt das Bemühen um Humanität, Menschenwürde und freie Persönlichkeitsentfaltung in den Mittelpunkt.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12170-1">
    <skos:related rdf:resource="descriptor/16759-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12170-1">
    <skos:related rdf:resource="descriptor/10140-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12170-1">
    <skos:related rdf:resource="descriptor/12168-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12170-1">
    <skos:related rdf:resource="descriptor/15467-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13808-1">
    <skos:related rdf:resource="descriptor/18118-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13808-1">
    <skos:related rdf:resource="descriptor/13441-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13808-1">
    <skos:related rdf:resource="descriptor/19247-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18709-6">
    <skos:prefLabel xml:lang="en">Bank managers</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18709-6">
    <skos:prefLabel xml:lang="de">Bankmanager</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11183-4">
    <skos:narrower rdf:resource="descriptor/19472-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11183-4">
    <skos:narrower rdf:resource="descriptor/11207-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11183-4">
    <skos:narrower rdf:resource="descriptor/11267-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11183-4">
    <skos:narrower rdf:resource="descriptor/19462-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11183-4">
    <skos:narrower rdf:resource="descriptor/11266-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11183-4">
    <skos:narrower rdf:resource="descriptor/11211-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11183-4">
    <skos:narrower rdf:resource="descriptor/10169-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13284-2">
    <skos:prefLabel xml:lang="en">Savings and loan association</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13284-2">
    <skos:prefLabel xml:lang="de">Bausparkasse</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16670-3">
    <skos:related rdf:resource="descriptor/15679-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16670-3">
    <skos:related rdf:resource="descriptor/15949-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16670-3">
    <skos:related rdf:resource="descriptor/16541-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16670-3">
    <skos:related rdf:resource="descriptor/14728-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16670-3">
    <skos:related rdf:resource="descriptor/15947-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16670-3">
    <skos:related rdf:resource="descriptor/15658-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16670-3">
    <skos:related rdf:resource="descriptor/13292-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16670-3">
    <skos:related rdf:resource="descriptor/13371-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16670-3">
    <skos:related rdf:resource="descriptor/18444-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16670-3">
    <skos:related rdf:resource="descriptor/16479-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16670-3">
    <skos:related rdf:resource="descriptor/16635-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16670-3">
    <skos:related rdf:resource="descriptor/11192-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16670-3">
    <skos:related rdf:resource="descriptor/16531-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16670-3">
    <skos:related rdf:resource="descriptor/15908-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19153-1">
    <skos:broader rdf:resource="thsys/70485"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19153-1">
    <skos:broader rdf:resource="descriptor/12085-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10489-6">
    <skos:prefLabel xml:lang="en">Population growth</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10489-6">
    <skos:prefLabel xml:lang="de">Bevölkerungswachstum</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18707-3">
    <skos:prefLabel xml:lang="en">Banking policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18707-3">
    <skos:prefLabel xml:lang="de">Bankenpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11060-6">
    <skos:related rdf:resource="descriptor/11142-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11060-6">
    <skos:related rdf:resource="descriptor/11090-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11060-6">
    <skos:related rdf:resource="descriptor/11123-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11060-6">
    <skos:related rdf:resource="descriptor/16374-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11060-6">
    <skos:related rdf:resource="descriptor/16210-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11060-6">
    <skos:related rdf:resource="descriptor/16402-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11060-6">
    <skos:related rdf:resource="descriptor/16651-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11060-6">
    <skos:related rdf:resource="descriptor/11061-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11060-6">
    <skos:related rdf:resource="descriptor/11088-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11060-6">
    <skos:related rdf:resource="descriptor/11047-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11060-6">
    <skos:related rdf:resource="descriptor/16409-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11060-6">
    <skos:related rdf:resource="descriptor/16629-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11060-6">
    <skos:related rdf:resource="descriptor/11070-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11060-6">
    <skos:related rdf:resource="descriptor/16387-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19226-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19548-6">
    <skos:prefLabel xml:lang="en">Innovation management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19548-6">
    <skos:prefLabel xml:lang="de">Innovationsmanagement</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13679-0">
    <skos:prefLabel xml:lang="en">Optical fibre transmission</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13679-0">
    <skos:prefLabel xml:lang="de">Lichtleiterkommunikation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15055-0">
    <skos:prefLabel xml:lang="en">Mathematical programming</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15055-0">
    <skos:prefLabel xml:lang="de">Mathematische Optimierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12158-5">
    <skos:altLabel xml:lang="de">Strategische Geschäftseinheit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20381-6">
    <skos:broader rdf:resource="thsys/70113"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11801-4">
    <skos:narrower rdf:resource="descriptor/19307-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11801-4">
    <skos:narrower rdf:resource="descriptor/19348-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11801-4">
    <skos:narrower rdf:resource="descriptor/19295-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11801-4">
    <skos:narrower rdf:resource="descriptor/19187-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11801-4">
    <skos:narrower rdf:resource="descriptor/18332-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11801-4">
    <skos:narrower rdf:resource="descriptor/11562-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11801-4">
    <skos:narrower rdf:resource="descriptor/11804-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11801-4">
    <skos:narrower rdf:resource="descriptor/11809-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11801-4">
    <skos:narrower rdf:resource="descriptor/11800-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13325-0">
    <skos:prefLabel xml:lang="en">Waste management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13325-0">
    <skos:prefLabel xml:lang="de">Abfallwirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18122-3">
    <skos:broader rdf:resource="thsys/71066"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18122-3">
    <skos:broader rdf:resource="thsys/70889"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18122-3">
    <skos:broader rdf:resource="descriptor/12029-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14728-1">
    <skos:prefLabel xml:lang="en">Pharmaceutical</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14728-1">
    <skos:prefLabel xml:lang="de">Pharmazeutisches Produkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15477-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19724-5">
    <skos:related rdf:resource="descriptor/16460-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19724-5">
    <skos:related rdf:resource="descriptor/19725-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19724-5">
    <skos:related rdf:resource="descriptor/18815-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16076-1">
    <skos:altLabel xml:lang="de">Umwandlungsgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14480-4">
    <skos:altLabel xml:lang="en">Integrated circuit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14480-4">
    <skos:altLabel xml:lang="en">IC (Integrated Circuit)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14480-4">
    <skos:altLabel xml:lang="de">Transistor</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14480-4">
    <skos:altLabel xml:lang="de">Mikroprozessor</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14480-4">
    <skos:altLabel xml:lang="de">Mikroelektronisches Produkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14480-4">
    <skos:altLabel xml:lang="de">Integrierter Schaltkreis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14480-4">
    <skos:altLabel xml:lang="de">Integrierte Schaltung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14480-4">
    <skos:altLabel xml:lang="de">Diode</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14480-4">
    <skos:altLabel xml:lang="de">Chip (Mikroelektronik)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16523-2">
    <skos:prefLabel xml:lang="en">Compulsory nonmilitary service</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16523-2">
    <skos:prefLabel xml:lang="de">Zivildienst</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10714-4">
    <skos:related rdf:resource="descriptor/10704-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10714-4">
    <skos:related rdf:resource="descriptor/10612-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18314-1">
    <skos:altLabel xml:lang="en">Undernutrition</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18314-1">
    <skos:altLabel xml:lang="de">Hunger</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18314-1">
    <skos:altLabel xml:lang="de">Fehlernährung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71065">
    <skos:prefLabel xml:lang="en">V.11.05  Rural economy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71065">
    <skos:prefLabel xml:lang="de">V.11.05  Ländliche Wirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18341-5">
    <skos:related rdf:resource="descriptor/18332-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21082-4">
    <skos:altLabel xml:lang="en">Distributional incidence</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21082-4">
    <skos:altLabel xml:lang="en">Distributional implication</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21082-4">
    <skos:altLabel xml:lang="de">Verteilungseffekt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21082-4">
    <skos:altLabel xml:lang="de">Umverteilungseffekt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21082-4">
    <skos:altLabel xml:lang="de">Inzidenz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11388-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11483-6">
    <skos:prefLabel xml:lang="en">Lombard rate policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11483-6">
    <skos:prefLabel xml:lang="de">Lombardpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14087-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70476">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">P.12.03</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18121-5">
    <zbwext:useInsteadNote rdf:nodeID="id153"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18294-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18257-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13000-0">
    <skos:related rdf:resource="descriptor/14702-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13000-0">
    <skos:related rdf:resource="descriptor/12994-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19539-0">
    <skos:prefLabel xml:lang="en">Blood donation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19539-0">
    <skos:prefLabel xml:lang="de">Blutspende</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19022-2">
    <skos:prefLabel xml:lang="en">Bird</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19022-2">
    <skos:prefLabel xml:lang="de">Vogel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10822-1">
    <skos:prefLabel xml:lang="en">Foreign production</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10822-1">
    <skos:prefLabel xml:lang="de">Auslandsproduktion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10872-0">
    <skos:altLabel xml:lang="de">Indirekter Import</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10872-0">
    <skos:altLabel xml:lang="de">Importgroßhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10872-0">
    <skos:altLabel xml:lang="de">Importgeschäft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10872-0">
    <skos:altLabel xml:lang="de">Importeur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10872-0">
    <skos:altLabel xml:lang="de">Einfuhrhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10872-0">
    <skos:altLabel xml:lang="de">Einfuhrgeschäft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13688-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70096">
    <rdfs:label xml:lang="de">Philosophie, Wissenschaftstheorie und Religion</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16504-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73350">
    <skos:prefLabel xml:lang="en">N.10.02  Physics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73350">
    <skos:prefLabel xml:lang="de">N.10.02  Physik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14692-3">
    <skos:prefLabel xml:lang="en">Styrene</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14692-3">
    <skos:prefLabel xml:lang="de">Styrol</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10380-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21282-3">
    <skos:prefLabel xml:lang="en">Competitive advantage</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21282-3">
    <skos:prefLabel xml:lang="de">Wettbewerbsvorteil</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15399-3">
    <skos:altLabel xml:lang="de">Varimax-Methode</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15399-3">
    <skos:altLabel xml:lang="de">Faktorrotation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10181-6">
    <skos:narrower rdf:resource="descriptor/19200-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10181-6">
    <skos:narrower rdf:resource="descriptor/10183-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10181-6">
    <skos:narrower rdf:resource="descriptor/19552-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10181-6">
    <skos:narrower rdf:resource="descriptor/10182-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10181-6">
    <skos:narrower rdf:resource="descriptor/10172-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10181-6">
    <skos:narrower rdf:resource="descriptor/10278-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10181-6">
    <skos:narrower rdf:resource="descriptor/10282-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10181-6">
    <skos:narrower rdf:resource="descriptor/19197-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12303-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12137-6">
    <skos:altLabel xml:lang="de">Organisatorische Dezentralisierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71013">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.04.01.03</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18944-3">
    <skos:broader rdf:resource="thsys/70823"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18944-3">
    <skos:broader rdf:resource="descriptor/10981-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19164-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19400-3">
    <skos:broader rdf:resource="thsys/71076"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15744-5">
    <skos:prefLabel xml:lang="en">Marine pollution</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15744-5">
    <skos:prefLabel xml:lang="de">Meeresverschmutzung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12670-2">
    <skos:altLabel xml:lang="en">Business logistics</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12670-2">
    <skos:altLabel xml:lang="de">Marketinglogistik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12670-2">
    <skos:altLabel xml:lang="de">Fertigungslogistik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12670-2">
    <skos:altLabel xml:lang="de">Distributionslogistik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12670-2">
    <skos:altLabel xml:lang="de">Betriebslogistik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12670-2">
    <skos:altLabel xml:lang="de">Beschaffungslogistik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12670-2">
    <skos:altLabel xml:lang="de">Absatzlogistik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10124-4">
    <skos:related rdf:resource="descriptor/19266-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10124-4">
    <skos:related rdf:resource="descriptor/19272-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10124-4">
    <skos:related rdf:resource="descriptor/10811-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10621-4">
    <skos:related rdf:resource="descriptor/16059-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10621-4">
    <skos:related rdf:resource="descriptor/18567-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10621-4">
    <skos:related rdf:resource="descriptor/10829-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10621-4">
    <skos:related rdf:resource="descriptor/16185-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10621-4">
    <skos:related rdf:resource="descriptor/19329-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10621-4">
    <skos:related rdf:resource="descriptor/16184-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10621-4">
    <skos:related rdf:resource="descriptor/20733-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10621-4">
    <skos:related rdf:resource="descriptor/10606-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10621-4">
    <skos:related rdf:resource="descriptor/10827-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10631-1">
    <skos:scopeNote xml:lang="de">Wird im Rahmen der realen Außenhandelstheorie benutzt bei der wohlstandsökonomischen Analyse internationaler Tauschbeziehungen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16013-4">
    <skos:prefLabel xml:lang="en">Supreme power</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16013-4">
    <skos:prefLabel xml:lang="de">Staatsgewalt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15381-1">
    <skos:broader rdf:resource="thsys/72393"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15381-1">
    <skos:broader rdf:resource="thsys/71039"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71023">
    <skos:narrower rdf:resource="descriptor/18329-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71023">
    <skos:narrower rdf:resource="descriptor/12032-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71023">
    <skos:narrower rdf:resource="descriptor/19455-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71023">
    <skos:narrower rdf:resource="descriptor/18014-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71023">
    <skos:narrower rdf:resource="descriptor/18985-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71023">
    <skos:narrower rdf:resource="descriptor/12024-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71023">
    <skos:narrower rdf:resource="descriptor/18328-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71023">
    <skos:narrower rdf:resource="descriptor/12016-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71023">
    <skos:narrower rdf:resource="descriptor/19339-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14110-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13305-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13290-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10715-2">
    <skos:prefLabel xml:lang="en">Tariff schedule</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10715-2">
    <skos:prefLabel xml:lang="de">Zolltarif</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18736-3">
    <skos:broader rdf:resource="thsys/73334"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18736-3">
    <skos:broader rdf:resource="descriptor/16195-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#topConceptOf">
    <rdf:type rdf:resource="http://www.w3.org/1999/02/22-rdf-syntax-ns#Property"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#topConceptOf">
    <rdf:type rdf:resource="http://www.w3.org/2002/07/owl#ObjectProperty"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18241-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16051-3">
    <skos:altLabel xml:lang="de">Konzentrationsrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16051-3">
    <skos:altLabel xml:lang="de">GWB (Gesetz gegen Wettbewerbsbeschränkungen)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16051-3">
    <skos:altLabel xml:lang="de">Gesetz gegen Wettbewerbsbeschränkungen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16051-3">
    <skos:altLabel xml:lang="de">Antitrustgesetzgebung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19150-0">
    <skos:narrower rdf:resource="descriptor/15346-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19150-0">
    <skos:narrower rdf:resource="descriptor/15411-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19150-0">
    <skos:narrower rdf:resource="descriptor/15408-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19150-0">
    <skos:narrower rdf:resource="descriptor/15021-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18768-4">
    <skos:prefLabel xml:lang="en">Newspaper database</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18768-4">
    <skos:prefLabel xml:lang="de">Pressedatenbank</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19207-4">
    <skos:broader rdf:resource="thsys/70049"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17934-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14775-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18131-2">
    <skos:broader rdf:resource="thsys/70361"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18131-2">
    <skos:broader rdf:resource="descriptor/18012-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12270-4">
    <skos:prefLabel xml:lang="en">Accounting profit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12270-4">
    <skos:prefLabel xml:lang="de">Bilanzgewinn</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13844-4">
    <skos:altLabel xml:lang="de">Wohnungsinstandsetzung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13844-4">
    <skos:altLabel xml:lang="de">Instandsetzung von Gebäuden</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13844-4">
    <skos:altLabel xml:lang="de">Instandhaltung von Gebäuden</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13844-4">
    <skos:altLabel xml:lang="de">Hausinstandsetzung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13844-4">
    <skos:altLabel xml:lang="de">Gebäudeinstandsetzung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10508-0">
    <skos:prefLabel xml:lang="en">Newly industrializing countries</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10508-0">
    <skos:prefLabel xml:lang="de">Schwellenländer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18668-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19510-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18523-6">
    <skos:broader rdf:resource="thsys/70959"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18523-6">
    <skos:broader rdf:resource="descriptor/13202-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11916-1">
    <skos:prefLabel xml:lang="en">Economic area</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11916-1">
    <skos:prefLabel xml:lang="de">Wirtschaftsraum</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70131">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">G.01.04</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16366-6">
    <skos:altLabel xml:lang="de">Zweiparteiensystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16366-6">
    <skos:altLabel xml:lang="de">Parteienstruktur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16366-6">
    <skos:altLabel xml:lang="de">Mehrparteiensystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16366-6">
    <skos:altLabel xml:lang="de">Einparteiensystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18267-5">
    <skos:related rdf:resource="descriptor/14052-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18267-5">
    <skos:related rdf:resource="descriptor/19486-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18267-5">
    <skos:related rdf:resource="descriptor/13327-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18657-6">
    <skos:related rdf:resource="descriptor/18660-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19082-5">
    <skos:prefLabel xml:lang="en">Profit and loss transfer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19082-5">
    <skos:prefLabel xml:lang="de">Ergebnisabführung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13670-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15721-3">
    <skos:related rdf:resource="descriptor/15719-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15721-3">
    <skos:related rdf:resource="descriptor/15718-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15721-3">
    <skos:related rdf:resource="descriptor/16674-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12964-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19512-6">
    <skos:scopeNote xml:lang="de">Alterssicherung des öffentlichen Dienstes.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11064-5">
    <skos:narrower rdf:resource="descriptor/10877-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12021-4">
    <skos:altLabel xml:lang="de">Bodenreinhaltung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12021-4">
    <skos:altLabel xml:lang="de">Bodenkonservierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12021-4">
    <skos:altLabel xml:lang="de">Bodenbewahrung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10816-3">
    <skos:scopeNote xml:lang="de">Länderspezifisches Risiko bei der Gewährung von Auslandskrediten und bei Auslandsinvestitionen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73371">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.04.03.03</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70850">
    <rdfs:label xml:lang="de">Religionsgemeinschaften</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19093-0">
    <skos:prefLabel xml:lang="en">Liquid assets</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19093-0">
    <skos:prefLabel xml:lang="de">Liquide Mittel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18774-2">
    <skos:related rdf:resource="descriptor/18874-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18774-2">
    <skos:related rdf:resource="descriptor/18784-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16547-2">
    <skos:broader rdf:resource="thsys/70832"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16547-2">
    <skos:broader rdf:resource="thsys/70458"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16547-2">
    <skos:broader rdf:resource="descriptor/18913-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70481">
    <skos:broader rdf:resource="thsys/70180"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11970-2">
    <skos:narrower rdf:resource="descriptor/11973-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11970-2">
    <skos:narrower rdf:resource="descriptor/11968-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11970-2">
    <skos:narrower rdf:resource="descriptor/19457-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11970-2">
    <skos:narrower rdf:resource="descriptor/19685-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11970-2">
    <skos:narrower rdf:resource="descriptor/11966-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10865-4">
    <skos:altLabel xml:lang="de">Ausfuhrkredit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10957-6">
    <skos:altLabel xml:lang="en">Merger policy</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10199-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70899">
    <rdfs:label xml:lang="en">Banking</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70899">
    <rdfs:label xml:lang="de">Bankgeschäft</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11579-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19429-0">
    <skos:prefLabel xml:lang="en">Spatial competition</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19429-0">
    <skos:prefLabel xml:lang="de">Räumlicher Wettbewerb</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10024-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14255-4">
    <skos:related rdf:resource="descriptor/14567-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14255-4">
    <skos:related rdf:resource="descriptor/14236-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11496-4">
    <skos:broader rdf:resource="thsys/70355"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19573-0">
    <skos:broader rdf:resource="thsys/72393"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19573-0">
    <skos:broader rdf:resource="descriptor/15376-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11189-6">
    <skos:altLabel xml:lang="en">Labor market discrimination</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11189-6">
    <skos:altLabel xml:lang="en">Affirmative action</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11189-6">
    <skos:altLabel xml:lang="de">Diskriminierung im Arbeitsleben</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11189-6">
    <skos:altLabel xml:lang="de">Diskriminierung am Arbeitsplatz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11189-6">
    <skos:altLabel xml:lang="de">Diskriminierung am Arbeitsmarkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11189-6">
    <skos:altLabel xml:lang="de">Arbeitsmarktchancen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71037">
    <skos:broader rdf:resource="thsys/71039"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10872-0">
    <skos:broader rdf:resource="thsys/70952"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10872-0">
    <skos:broader rdf:resource="descriptor/10605-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10872-0">
    <skos:broader rdf:resource="descriptor/10852-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18378-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19712-5">
    <skos:related rdf:resource="descriptor/16648-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19712-5">
    <skos:related rdf:resource="descriptor/10452-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10196-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18028-2">
    <skos:narrower rdf:resource="descriptor/18212-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18028-2">
    <skos:narrower rdf:resource="descriptor/18210-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18028-2">
    <skos:narrower rdf:resource="descriptor/18208-0"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id56">
    <rdf:value xml:lang="de">Für den Bildungsstand von z.B. Personengruppen</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73361">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12844-2">
    <skos:related rdf:resource="descriptor/15859-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12844-2">
    <skos:related rdf:resource="descriptor/12846-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12844-2">
    <skos:related rdf:resource="descriptor/12845-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12844-2">
    <skos:related rdf:resource="descriptor/12837-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12844-2">
    <skos:related rdf:resource="descriptor/12829-5"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id264">
    <rdfs:seeAlso rdf:resource="descriptor/19231-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18738-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70318">
    <skos:narrower rdf:resource="descriptor/14393-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70318">
    <skos:narrower rdf:resource="descriptor/14343-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70318">
    <skos:narrower rdf:resource="descriptor/14328-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70318">
    <skos:narrower rdf:resource="descriptor/14392-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70318">
    <skos:narrower rdf:resource="descriptor/18244-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70318">
    <skos:narrower rdf:resource="descriptor/14385-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70318">
    <skos:narrower rdf:resource="descriptor/12738-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70318">
    <skos:narrower rdf:resource="descriptor/14384-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70318">
    <skos:narrower rdf:resource="descriptor/14383-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70318">
    <skos:narrower rdf:resource="descriptor/14382-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70318">
    <skos:narrower rdf:resource="descriptor/14381-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70318">
    <skos:narrower rdf:resource="descriptor/14380-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70318">
    <skos:narrower rdf:resource="descriptor/14379-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70318">
    <skos:narrower rdf:resource="descriptor/14377-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70318">
    <skos:narrower rdf:resource="descriptor/14378-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70318">
    <skos:narrower rdf:resource="descriptor/14371-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70318">
    <skos:narrower rdf:resource="descriptor/14368-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70318">
    <skos:narrower rdf:resource="descriptor/14366-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70318">
    <skos:narrower rdf:resource="descriptor/14365-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12416-2">
    <skos:broader rdf:resource="thsys/73337"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12416-2">
    <skos:broader rdf:resource="thsys/70184"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12416-2">
    <skos:broader rdf:resource="thsys/70004"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10806-6">
    <skos:broader rdf:resource="thsys/71090"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10806-6">
    <skos:broader rdf:resource="thsys/71089"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10806-6">
    <skos:broader rdf:resource="thsys/70903"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10806-6">
    <skos:broader rdf:resource="descriptor/12062-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10806-6">
    <skos:broader rdf:resource="descriptor/13283-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11373-6">
    <skos:prefLabel xml:lang="en">Higher education institution</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11373-6">
    <skos:prefLabel xml:lang="de">Hochschule</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16406-6">
    <skos:altLabel xml:lang="de">Individualisierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12226-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12234-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12949-2">
    <skos:narrower rdf:resource="descriptor/18316-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12949-2">
    <skos:narrower rdf:resource="descriptor/18299-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12949-2">
    <skos:narrower rdf:resource="descriptor/18193-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12949-2">
    <skos:narrower rdf:resource="descriptor/18192-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12949-2">
    <skos:narrower rdf:resource="descriptor/12940-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12949-2">
    <skos:narrower rdf:resource="descriptor/12939-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12949-2">
    <skos:narrower rdf:resource="descriptor/12933-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70321">
    <skos:broader rdf:resource="thsys/70012"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id45">
    <rdf:value xml:lang="de">Für das berufliche Bildungswesen allgemein</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10332-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11825-4">
    <skos:narrower rdf:resource="descriptor/12189-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11825-4">
    <skos:narrower rdf:resource="descriptor/11628-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11825-4">
    <skos:narrower rdf:resource="descriptor/12180-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11825-4">
    <skos:narrower rdf:resource="descriptor/13302-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15519-5">
    <skos:scopeNote xml:lang="de">Distributions- und Verteilungsproblem: Güter sind von bestimmten Orten zu bestimmten anderen Orten auf bestimmten Transportstrecken unter Beachtung von Nebenbedingungen so zu bewegen, daß vorgegebene Ziele erfüllt werden.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14440-2">
    <skos:related rdf:resource="descriptor/13630-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19333-6">
    <skos:prefLabel xml:lang="en">Tobin tax</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19333-6">
    <skos:prefLabel xml:lang="de">Tobinsteuer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10546-6">
    <skos:related rdf:resource="descriptor/12036-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10546-6">
    <skos:related rdf:resource="descriptor/18117-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10546-6">
    <skos:related rdf:resource="descriptor/18182-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10546-6">
    <skos:related rdf:resource="descriptor/12759-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10546-6">
    <skos:related rdf:resource="descriptor/12810-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70928">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11510-1">
    <skos:scopeNote xml:lang="de">Anpassung von Löhnen und Gehältern an die Entwicklung des Preisniveaus.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19563-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12809-4">
    <skos:related rdf:resource="descriptor/19564-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12809-4">
    <skos:related rdf:resource="descriptor/20176-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12809-4">
    <skos:related rdf:resource="descriptor/20306-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12809-4">
    <skos:related rdf:resource="descriptor/12808-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12809-4">
    <skos:related rdf:resource="descriptor/20314-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12809-4">
    <skos:related rdf:resource="descriptor/12806-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12809-4">
    <skos:related rdf:resource="descriptor/10016-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12809-4">
    <skos:related rdf:resource="descriptor/19034-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12809-4">
    <skos:related rdf:resource="descriptor/18821-5"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id202">
    <rdf:value xml:lang="de">Für Einstellung und Verhalten von Verbrauchern allein</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13644-5">
    <skos:narrower rdf:resource="descriptor/18638-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13644-5">
    <skos:narrower rdf:resource="descriptor/13643-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13644-5">
    <skos:narrower rdf:resource="descriptor/18636-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13644-5">
    <skos:narrower rdf:resource="descriptor/18587-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13644-5">
    <skos:narrower rdf:resource="descriptor/13641-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13644-5">
    <skos:narrower rdf:resource="descriptor/18612-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13644-5">
    <skos:narrower rdf:resource="descriptor/13636-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13644-5">
    <skos:narrower rdf:resource="descriptor/13635-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18833-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70135">
    <skos:narrower rdf:resource="descriptor/14408-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70135">
    <skos:narrower rdf:resource="descriptor/13667-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70135">
    <skos:narrower rdf:resource="descriptor/18249-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70135">
    <skos:narrower rdf:resource="descriptor/14466-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70135">
    <skos:narrower rdf:resource="descriptor/18296-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70135">
    <skos:narrower rdf:resource="descriptor/14490-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70135">
    <skos:narrower rdf:resource="descriptor/18081-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70135">
    <skos:narrower rdf:resource="descriptor/14465-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70135">
    <skos:narrower rdf:resource="descriptor/14488-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70135">
    <skos:narrower rdf:resource="descriptor/14486-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70135">
    <skos:narrower rdf:resource="descriptor/14485-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70135">
    <skos:narrower rdf:resource="descriptor/14483-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70135">
    <skos:narrower rdf:resource="descriptor/14479-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70135">
    <skos:narrower rdf:resource="descriptor/14473-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70135">
    <skos:narrower rdf:resource="descriptor/14469-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70135">
    <skos:narrower rdf:resource="descriptor/18250-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70135">
    <skos:narrower rdf:resource="descriptor/13649-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70135">
    <skos:narrower rdf:resource="descriptor/14464-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70135">
    <skos:narrower rdf:resource="thsys/73305"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70135">
    <skos:narrower rdf:resource="thsys/70477"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70135">
    <skos:narrower rdf:resource="thsys/70254"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70135">
    <skos:narrower rdf:resource="thsys/70373"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14054-0">
    <skos:related rdf:resource="descriptor/14129-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14054-0">
    <skos:related rdf:resource="descriptor/14931-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14054-0">
    <skos:related rdf:resource="descriptor/14161-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14054-0">
    <skos:related rdf:resource="descriptor/14041-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14054-0">
    <skos:related rdf:resource="descriptor/12945-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14054-0">
    <skos:related rdf:resource="descriptor/13188-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14054-0">
    <skos:related rdf:resource="descriptor/18222-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14054-0">
    <skos:related rdf:resource="descriptor/14149-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14054-0">
    <skos:related rdf:resource="descriptor/10495-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14054-0">
    <skos:related rdf:resource="descriptor/18320-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14054-0">
    <skos:related rdf:resource="descriptor/18980-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14054-0">
    <skos:related rdf:resource="descriptor/13431-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18161-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16476-6">
    <skos:broader rdf:resource="thsys/73370"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16476-6">
    <skos:broader rdf:resource="descriptor/16545-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11659-2">
    <skos:prefLabel xml:lang="en">Real property tax</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11659-2">
    <skos:prefLabel xml:lang="de">Grundsteuer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14160-1">
    <skos:prefLabel xml:lang="en">Marine fauna</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14160-1">
    <skos:prefLabel xml:lang="de">Meerestier</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16070-6">
    <skos:narrower rdf:resource="descriptor/16169-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13662-3">
    <skos:broader rdf:resource="thsys/73376"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13662-3">
    <skos:broader rdf:resource="thsys/73361"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13662-3">
    <skos:broader rdf:resource="thsys/73296"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13662-3">
    <skos:broader rdf:resource="thsys/70825"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13662-3">
    <skos:broader rdf:resource="descriptor/15998-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14805-2">
    <skos:altLabel xml:lang="de">Karton</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19621-1">
    <skos:altLabel xml:lang="de">Gletscher</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19621-1">
    <skos:altLabel xml:lang="de">Eisberg</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71113">
    <rdfs:label xml:lang="en">Distributive politics</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71113">
    <rdfs:label xml:lang="de">Verteilungspolitik</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18495-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13411-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18594-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11427-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24784-6">
    <skos:scopeNote xml:lang="de">Software-Lizenzmodell ohne Nutzungsbeschränkungen. Die freie Verfügbarkeit des Quellcodes garantiert die Nutzung und Weiterentwicklung der Software durch beliebige Anwender gemäß den Open-Source-Bedingungen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10785-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15679-4">
    <skos:broader rdf:resource="thsys/73346"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13256-0">
    <skos:related rdf:resource="descriptor/14188-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13256-0">
    <skos:related rdf:resource="descriptor/13576-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13256-0">
    <skos:related rdf:resource="descriptor/18591-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13256-0">
    <skos:related rdf:resource="descriptor/13545-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13256-0">
    <skos:related rdf:resource="descriptor/13983-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13256-0">
    <skos:related rdf:resource="descriptor/13066-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13256-0">
    <skos:related rdf:resource="descriptor/14455-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13256-0">
    <skos:related rdf:resource="descriptor/13252-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18685-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16504-6">
    <skos:altLabel xml:lang="en">Leniency programme</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16504-6">
    <skos:altLabel xml:lang="en">Leniency program</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16504-6">
    <skos:altLabel xml:lang="en">Abolition</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16504-6">
    <skos:altLabel xml:lang="de">Straferlass</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16504-6">
    <skos:altLabel xml:lang="de">Straferlaß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16504-6">
    <skos:altLabel xml:lang="de">Strafbefreiung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16504-6">
    <skos:altLabel xml:lang="de">Strafaussetzung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16504-6">
    <skos:altLabel xml:lang="de">Kronzeugenregelung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16504-6">
    <skos:altLabel xml:lang="de">Gnadenrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16504-6">
    <skos:altLabel xml:lang="de">Gnadenerweis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16504-6">
    <skos:altLabel xml:lang="de">Bewährungsstrafe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16504-6">
    <skos:altLabel xml:lang="de">Bewährungsfrist</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16504-6">
    <skos:altLabel xml:lang="de">Begnadigung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16504-6">
    <skos:altLabel xml:lang="de">Amnestie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19621-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13207-6">
    <skos:narrower rdf:resource="descriptor/13234-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13391-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12917-1">
    <skos:prefLabel xml:lang="en">Small-scale industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12917-1">
    <skos:prefLabel xml:lang="de">Kleinindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71052">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12810-5">
    <skos:broader rdf:resource="thsys/70550"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12810-5">
    <skos:broader rdf:resource="descriptor/12808-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16142-0">
    <zbwext:useInsteadNote rdf:nodeID="id184"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14634-3">
    <skos:broader rdf:resource="thsys/70463"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14634-3">
    <skos:broader rdf:resource="descriptor/18254-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14634-3">
    <skos:broader rdf:resource="descriptor/14601-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13219-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13580-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18805-3">
    <skos:scopeNote xml:lang="de">Verwaltungsrechtliche Erlaubnis zur Ausübung eines Gewerbes oder zum Betrieb gewerblicher Anlagen in gesetzlich geregelten Fällen, wo die Gewerbefreiheit ausgesetzt ist.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71063">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.11.07</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70171">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.06.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70337">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">P.22</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19137-6">
    <skos:broader rdf:resource="thsys/70899"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19137-6">
    <skos:broader rdf:resource="thsys/70459"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19137-6">
    <skos:broader rdf:resource="descriptor/13756-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14262-0">
    <skos:prefLabel xml:lang="en">Wire</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14262-0">
    <skos:prefLabel xml:lang="de">Draht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12766-3">
    <skos:altLabel xml:lang="de">Absatzwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12766-3">
    <skos:altLabel xml:lang="de">Absatzmarketing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12766-3">
    <skos:altLabel xml:lang="de">Absatzbereich</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16312-1">
    <skos:related rdf:resource="descriptor/16421-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16312-1">
    <skos:related rdf:resource="descriptor/16314-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16312-1">
    <skos:related rdf:resource="descriptor/16297-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15446-6">
    <skos:broader rdf:resource="thsys/73104"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15446-6">
    <skos:broader rdf:resource="descriptor/15458-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10006-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19201-2">
    <skos:prefLabel xml:lang="en">Windfall profit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19201-2">
    <skos:prefLabel xml:lang="de">Windfall Profit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19167-4">
    <skos:related rdf:resource="descriptor/17589-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19167-4">
    <skos:related rdf:resource="descriptor/19180-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11393-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10380-0">
    <skos:narrower rdf:resource="descriptor/10404-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16041-6">
    <skos:broader rdf:resource="thsys/70160"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16041-6">
    <skos:broader rdf:resource="descriptor/15981-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11682-0">
    <skos:prefLabel xml:lang="en">Property tax</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11682-0">
    <skos:prefLabel xml:lang="de">Vermögensteuer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16849-0">
    <skos:broader rdf:resource="thsys/70361"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16849-0">
    <skos:broader rdf:resource="descriptor/18012-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16361-2">
    <skos:broader rdf:resource="thsys/73378"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16361-2">
    <skos:broader rdf:resource="descriptor/13397-3"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id103">
    <rdfs:seeAlso rdf:resource="descriptor/16314-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14397-5">
    <skos:prefLabel xml:lang="en">Crane</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14397-5">
    <skos:prefLabel xml:lang="de">Kran</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24787-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id360">
    <rdfs:seeAlso rdf:resource="descriptor/11273-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20756-3">
    <skos:related rdf:resource="descriptor/15470-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14150-4">
    <skos:related rdf:resource="descriptor/15897-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14150-4">
    <skos:related rdf:resource="descriptor/12912-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14150-4">
    <skos:related rdf:resource="descriptor/14788-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14150-4">
    <skos:related rdf:resource="descriptor/19367-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14150-4">
    <skos:related rdf:resource="descriptor/18987-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14150-4">
    <skos:related rdf:resource="descriptor/14377-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10711-3">
    <skos:prefLabel xml:lang="en">Free trade zone</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10711-3">
    <skos:prefLabel xml:lang="de">Freie Produktionszone</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18539-5">
    <skos:related rdf:resource="descriptor/10821-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18539-5">
    <skos:related rdf:resource="descriptor/18700-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18539-5">
    <skos:related rdf:resource="descriptor/10803-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18539-5">
    <skos:related rdf:resource="descriptor/13720-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18539-5">
    <skos:related rdf:resource="descriptor/10858-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18539-5">
    <skos:related rdf:resource="descriptor/10854-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13472-1">
    <skos:related rdf:resource="descriptor/10884-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13472-1">
    <skos:related rdf:resource="descriptor/18690-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13472-1">
    <skos:related rdf:resource="descriptor/13188-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13472-1">
    <skos:related rdf:resource="descriptor/13194-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10956-1">
    <skos:narrower rdf:resource="descriptor/10227-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10956-1">
    <skos:narrower rdf:resource="descriptor/10224-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10956-1">
    <skos:narrower rdf:resource="descriptor/19429-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10956-1">
    <skos:narrower rdf:resource="descriptor/10948-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10956-1">
    <skos:narrower rdf:resource="descriptor/10944-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10956-1">
    <skos:narrower rdf:resource="descriptor/19430-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19161-2">
    <skos:altLabel xml:lang="de">Eigentümerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13421-4">
    <skos:related rdf:resource="descriptor/18808-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13421-4">
    <skos:related rdf:resource="descriptor/13414-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13421-4">
    <skos:related rdf:resource="descriptor/16324-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13421-4">
    <skos:related rdf:resource="descriptor/18809-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13421-4">
    <skos:related rdf:resource="descriptor/16551-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13421-4">
    <skos:related rdf:resource="descriptor/11668-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13421-4">
    <skos:related rdf:resource="descriptor/11592-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13421-4">
    <skos:related rdf:resource="descriptor/19370-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14131-1">
    <skos:broader rdf:resource="thsys/70403"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14131-1">
    <skos:broader rdf:resource="descriptor/14169-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24785-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14240-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18412-1">
    <skos:prefLabel xml:lang="en">Mechanical engineering</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18412-1">
    <skos:prefLabel xml:lang="de">Maschinenbautechnik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19835-3">
    <skos:broader rdf:resource="thsys/70383"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19835-3">
    <skos:broader rdf:resource="thsys/70041"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19835-3">
    <skos:broader rdf:resource="descriptor/12775-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11740-5">
    <skos:altLabel xml:lang="de">Lebenseinkommensanalyse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19083-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10981-2">
    <skos:narrower rdf:resource="descriptor/11833-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10981-2">
    <skos:narrower rdf:resource="descriptor/18653-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10981-2">
    <skos:narrower rdf:resource="descriptor/16552-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10981-2">
    <skos:narrower rdf:resource="descriptor/10973-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10981-2">
    <skos:narrower rdf:resource="descriptor/12004-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10981-2">
    <skos:narrower rdf:resource="descriptor/18944-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10981-2">
    <skos:narrower rdf:resource="descriptor/11823-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10981-2">
    <skos:narrower rdf:resource="descriptor/11817-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10981-2">
    <skos:narrower rdf:resource="descriptor/13639-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10981-2">
    <skos:narrower rdf:resource="descriptor/18338-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10981-2">
    <skos:narrower rdf:resource="descriptor/11807-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10981-2">
    <skos:narrower rdf:resource="descriptor/11810-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10981-2">
    <skos:narrower rdf:resource="descriptor/18707-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10981-2">
    <skos:narrower rdf:resource="descriptor/18992-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10981-2">
    <skos:narrower rdf:resource="descriptor/11801-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19067-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24703-4">
    <skos:prefLabel xml:lang="en">Single taxation (company / shareholder)</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24703-4">
    <skos:prefLabel xml:lang="de">Halbeinkünfteverfahren</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17706-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19060-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10201-5">
    <skos:related rdf:resource="descriptor/10978-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10201-5">
    <skos:related rdf:resource="descriptor/19524-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10201-5">
    <skos:related rdf:resource="descriptor/19153-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10201-5">
    <skos:related rdf:resource="descriptor/24857-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10201-5">
    <skos:related rdf:resource="descriptor/11055-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10201-5">
    <skos:related rdf:resource="descriptor/10905-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10201-5">
    <skos:related rdf:resource="descriptor/19827-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10201-5">
    <skos:related rdf:resource="descriptor/24705-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10201-5">
    <skos:related rdf:resource="descriptor/10076-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10201-5">
    <skos:related rdf:resource="descriptor/10040-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11371-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10650-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13816-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70041">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70939">
    <rdfs:label xml:lang="en">Passenger transport</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70939">
    <rdfs:label xml:lang="de">Personenverkehr</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18856-0">
    <skos:broader rdf:resource="thsys/70847"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18856-0">
    <skos:broader rdf:resource="descriptor/15909-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14379-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19301-5">
    <skos:broader rdf:resource="thsys/71029"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19301-5">
    <skos:broader rdf:resource="thsys/70960"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70017">
    <skos:prefLabel xml:lang="en">N.07  Psychology</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70017">
    <skos:prefLabel xml:lang="de">N.07  Psychologie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15873-1">
    <skos:broader rdf:resource="thsys/73354"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15873-1">
    <skos:broader rdf:resource="descriptor/15876-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70070">
    <skos:broader rdf:resource="thsys/70341"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19132-2">
    <skos:altLabel xml:lang="de">Werbeverbot</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19132-2">
    <skos:altLabel xml:lang="de">Werberecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13820-4">
    <skos:prefLabel xml:lang="en">Insurance fraud</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13820-4">
    <skos:prefLabel xml:lang="de">Versicherungsbetrug</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70536">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11786-2">
    <skos:prefLabel xml:lang="en">Inter-industry linkages</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11786-2">
    <skos:prefLabel xml:lang="de">Interindustrielle Verflechtung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11828-5">
    <skos:broader rdf:resource="thsys/71063"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11828-5">
    <skos:broader rdf:resource="thsys/70926"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11828-5">
    <skos:broader rdf:resource="descriptor/11833-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11828-5">
    <skos:broader rdf:resource="descriptor/12004-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13508-6">
    <skos:prefLabel xml:lang="en">Fast traffic</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13508-6">
    <skos:prefLabel xml:lang="de">Schnellverkehr</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70505">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10542-0">
    <skos:related rdf:resource="descriptor/10512-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10542-0">
    <skos:related rdf:resource="descriptor/11892-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10542-0">
    <skos:related rdf:resource="descriptor/10562-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10542-0">
    <skos:related rdf:resource="descriptor/11919-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12065-5">
    <skos:scopeNote xml:lang="de">Nur für Produktion in betriebswirtschaftlicher Sicht.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14145-4">
    <skos:prefLabel xml:lang="en">Livestock</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14145-4">
    <skos:prefLabel xml:lang="de">Vieh</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13508-6">
    <skos:related rdf:resource="descriptor/13555-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17624-5">
    <skos:prefLabel xml:lang="en">Yemen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17624-5">
    <skos:prefLabel xml:lang="de">Jemen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13647-6">
    <skos:narrower rdf:resource="descriptor/18643-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13647-6">
    <skos:narrower rdf:resource="descriptor/18642-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12372-3">
    <skos:related rdf:resource="descriptor/12276-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12372-3">
    <skos:related rdf:resource="descriptor/12275-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12372-3">
    <skos:related rdf:resource="descriptor/11652-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12372-3">
    <skos:related rdf:resource="descriptor/12312-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15879-3">
    <skos:altLabel xml:lang="de">Schären</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15879-3">
    <skos:altLabel xml:lang="de">Schäre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15879-3">
    <skos:altLabel xml:lang="de">Inselstaat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15879-3">
    <skos:altLabel xml:lang="de">Hallig</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16554-5">
    <skos:narrower rdf:resource="descriptor/11260-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16554-5">
    <skos:narrower rdf:resource="descriptor/10037-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16554-5">
    <skos:narrower rdf:resource="descriptor/19466-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16554-5">
    <skos:narrower rdf:resource="descriptor/13383-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16554-5">
    <skos:narrower rdf:resource="descriptor/12774-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16554-5">
    <skos:narrower rdf:resource="descriptor/19757-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16554-5">
    <skos:narrower rdf:resource="descriptor/16679-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16554-5">
    <skos:narrower rdf:resource="descriptor/16678-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16554-5">
    <skos:narrower rdf:resource="descriptor/19361-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16554-5">
    <skos:narrower rdf:resource="descriptor/11394-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16554-5">
    <skos:narrower rdf:resource="descriptor/11293-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16554-5">
    <skos:narrower rdf:resource="descriptor/19767-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70383">
    <skos:broader rdf:resource="thsys/70028"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13692-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12024-5">
    <skos:prefLabel xml:lang="en">Soil improvement</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12024-5">
    <skos:prefLabel xml:lang="de">Bodenverbesserung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15686-0">
    <skos:altLabel xml:lang="de">Wissenschaftswissenschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15686-0">
    <skos:altLabel xml:lang="de">Wissenschaftssoziologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15686-0">
    <skos:altLabel xml:lang="de">Wissenschaftsmetrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15686-0">
    <skos:altLabel xml:lang="de">Wissenschaftsentwicklung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15686-0">
    <skos:altLabel xml:lang="de">Szientometrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15686-0">
    <skos:altLabel xml:lang="de">Scientometrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14143-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16741-6">
    <skos:altLabel xml:lang="de">Physiokratismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16741-6">
    <skos:altLabel xml:lang="de">Physiokratische Schule</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16741-6">
    <skos:altLabel xml:lang="de">Physiokraten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13627-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19082-5">
    <skos:broader rdf:resource="thsys/71042"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19082-5">
    <skos:broader rdf:resource="thsys/70397"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19082-5">
    <skos:broader rdf:resource="thsys/70036"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19082-5">
    <skos:broader rdf:resource="descriptor/12276-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18072-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19646-6">
    <skos:prefLabel xml:lang="en">Suicide</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19646-6">
    <skos:prefLabel xml:lang="de">Selbstmord</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16252-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15088-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18103-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18297-3">
    <skos:altLabel xml:lang="de">Flugzeuginnenausstattung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10704-0">
    <skos:related rdf:resource="descriptor/10714-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10704-0">
    <skos:related rdf:resource="descriptor/19322-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10704-0">
    <skos:related rdf:resource="descriptor/10917-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11843-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12884-4">
    <skos:related rdf:resource="descriptor/12088-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12884-4">
    <skos:related rdf:resource="descriptor/12085-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12884-4">
    <skos:related rdf:resource="descriptor/18017-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12884-4">
    <skos:related rdf:resource="descriptor/12039-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12884-4">
    <skos:related rdf:resource="descriptor/12053-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15104-6">
    <skos:prefLabel xml:lang="en">Price statistics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15104-6">
    <skos:prefLabel xml:lang="de">Preisstatistik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15887-4">
    <skos:altLabel xml:lang="de">Weltmeer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15887-4">
    <skos:altLabel xml:lang="de">Ozean</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10243-3">
    <skos:narrower rdf:resource="descriptor/11959-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15648-1">
    <skos:altLabel xml:lang="de">Jüdische Religion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15648-1">
    <skos:altLabel xml:lang="de">Judaismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19781-0">
    <skos:related rdf:resource="descriptor/13357-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19781-0">
    <skos:related rdf:resource="descriptor/14040-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19781-0">
    <skos:related rdf:resource="descriptor/16515-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19781-0">
    <skos:related rdf:resource="descriptor/15694-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19781-0">
    <skos:related rdf:resource="descriptor/15654-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19781-0">
    <skos:related rdf:resource="descriptor/19634-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19781-0">
    <skos:related rdf:resource="descriptor/15689-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12187-5">
    <skos:altLabel xml:lang="de">KG</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12588-1">
    <skos:scopeNote xml:lang="de">Aus Sicht der Arbeitskräfte oder der Betriebe</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18784-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11627-1">
    <skos:narrower rdf:resource="descriptor/18345-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11627-1">
    <skos:narrower rdf:resource="descriptor/13333-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73300">
    <skos:broader rdf:resource="thsys/70498"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12436-3">
    <skos:altLabel xml:lang="de">Wiederherstellungswert</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12436-3">
    <skos:altLabel xml:lang="de">Reproduktionswert</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17179-0">
    <skos:altLabel xml:lang="de">Südosteuropäische Staaten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17179-0">
    <skos:altLabel xml:lang="de">Südosteuropäische Länder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17179-0">
    <skos:altLabel xml:lang="de">Balkanstaaten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17179-0">
    <skos:altLabel xml:lang="de">Balkanländer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73296">
    <skos:prefLabel xml:lang="en">W.06.01.08.01  Publishing industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73296">
    <skos:prefLabel xml:lang="de">W.06.01.08.01  Verlagsgewerbe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71056">
    <skos:narrower rdf:resource="descriptor/13964-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71056">
    <skos:narrower rdf:resource="descriptor/13935-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71056">
    <skos:narrower rdf:resource="descriptor/13962-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71056">
    <skos:narrower rdf:resource="descriptor/18400-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71056">
    <skos:narrower rdf:resource="descriptor/13904-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71056">
    <skos:narrower rdf:resource="descriptor/18422-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71056">
    <skos:narrower rdf:resource="descriptor/13960-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71056">
    <skos:narrower rdf:resource="descriptor/18484-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71056">
    <skos:narrower rdf:resource="descriptor/13894-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71056">
    <skos:narrower rdf:resource="descriptor/13949-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71056">
    <skos:narrower rdf:resource="descriptor/13890-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71056">
    <skos:narrower rdf:resource="descriptor/13959-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71056">
    <skos:narrower rdf:resource="descriptor/13888-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71056">
    <skos:narrower rdf:resource="descriptor/13953-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71056">
    <skos:narrower rdf:resource="descriptor/11358-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71056">
    <skos:narrower rdf:resource="descriptor/13957-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13849-1">
    <skos:narrower rdf:resource="descriptor/11994-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13849-1">
    <skos:narrower rdf:resource="descriptor/16671-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10206-2">
    <skos:broader rdf:resource="thsys/71119"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10206-2">
    <skos:broader rdf:resource="descriptor/10905-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17198-3">
    <skos:broader rdf:resource="thsys/70557"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17198-3">
    <skos:broader rdf:resource="descriptor/17199-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17198-3">
    <skos:broader rdf:resource="descriptor/17991-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17198-3">
    <skos:broader rdf:resource="descriptor/17983-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17198-3">
    <skos:broader rdf:resource="descriptor/17980-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18873-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19383-5">
    <skos:narrower rdf:resource="descriptor/13154-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19383-5">
    <skos:narrower rdf:resource="descriptor/13205-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19383-5">
    <skos:narrower rdf:resource="descriptor/12964-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18335-0">
    <skos:altLabel xml:lang="de">Eßverhalten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18335-0">
    <skos:altLabel xml:lang="de">Essverhalten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18335-0">
    <skos:altLabel xml:lang="de">Essgewohnheiten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18335-0">
    <skos:altLabel xml:lang="de">Eßgewohnheiten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18335-0">
    <skos:altLabel xml:lang="de">Ernährungsweise</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18335-0">
    <skos:altLabel xml:lang="de">Ernährungsverhalten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13481-0">
    <skos:related rdf:resource="descriptor/18556-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13481-0">
    <skos:related rdf:resource="descriptor/18546-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13481-0">
    <skos:related rdf:resource="descriptor/13200-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13481-0">
    <skos:related rdf:resource="descriptor/18013-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17758-5">
    <skos:narrower rdf:resource="descriptor/17931-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17758-5">
    <skos:narrower rdf:resource="descriptor/17806-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17758-5">
    <skos:narrower rdf:resource="descriptor/17791-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70245">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11189-6">
    <skos:broader rdf:resource="thsys/70051"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11189-6">
    <skos:broader rdf:resource="descriptor/15713-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16102-5">
    <skos:prefLabel xml:lang="en">Investment law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16102-5">
    <skos:prefLabel xml:lang="de">Investitionsrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16912-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19083-3">
    <skos:narrower rdf:resource="descriptor/17477-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19083-3">
    <skos:narrower rdf:resource="descriptor/17476-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19083-3">
    <skos:narrower rdf:resource="descriptor/17471-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10586-1">
    <skos:altLabel xml:lang="en">NIEO (New International Economic Order)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10586-1">
    <skos:altLabel xml:lang="en">New world economic order</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10586-1">
    <skos:altLabel xml:lang="de">NWWO (Neue Weltwirtschaftsordnung)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14919-1">
    <skos:altLabel xml:lang="de">Uniform</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14919-1">
    <skos:altLabel xml:lang="de">Spezialschutzbekleidung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14919-1">
    <skos:altLabel xml:lang="de">Schutzbekleidung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14919-1">
    <skos:altLabel xml:lang="de">Berufsbekleidung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14919-1">
    <skos:altLabel xml:lang="de">Arbeitsbekleidung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14636-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10045-0">
    <skos:altLabel xml:lang="en">Time lag</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10045-0">
    <skos:altLabel xml:lang="en">Outside lag</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10045-0">
    <skos:altLabel xml:lang="en">Lagged variable</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10045-0">
    <skos:altLabel xml:lang="en">Inside lag</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10045-0">
    <skos:altLabel xml:lang="de">Wirkungsverzögerung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10045-0">
    <skos:altLabel xml:lang="de">Verzögerte Variable</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10045-0">
    <skos:altLabel xml:lang="de">Reaktionsverzögerung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10045-0">
    <skos:altLabel xml:lang="de">Anpassungsverzögerung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14054-0">
    <skos:narrower rdf:resource="descriptor/14145-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14054-0">
    <skos:narrower rdf:resource="descriptor/14074-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16481-6">
    <skos:altLabel xml:lang="de">Kinderschutz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16481-6">
    <skos:altLabel xml:lang="de">Jugendschutzgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16481-6">
    <skos:altLabel xml:lang="de">Gesetz zum Schutze der Jugend in der Öffentlichkeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16458-1">
    <skos:broader rdf:resource="thsys/73363"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16458-1">
    <skos:broader rdf:resource="thsys/70865"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11888-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19129-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13825-1">
    <skos:narrower rdf:resource="descriptor/18994-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13825-1">
    <skos:narrower rdf:resource="descriptor/18734-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71005">
    <skos:narrower rdf:resource="descriptor/12993-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71005">
    <skos:narrower rdf:resource="descriptor/18480-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71005">
    <skos:narrower rdf:resource="descriptor/18479-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71005">
    <skos:narrower rdf:resource="descriptor/18459-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71005">
    <skos:narrower rdf:resource="descriptor/12917-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71005">
    <skos:narrower rdf:resource="descriptor/10843-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71005">
    <skos:narrower rdf:resource="descriptor/18737-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71005">
    <skos:narrower rdf:resource="descriptor/11821-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71005">
    <skos:narrower rdf:resource="descriptor/15103-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71005">
    <skos:narrower rdf:resource="descriptor/18490-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71005">
    <skos:narrower rdf:resource="descriptor/16692-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71005">
    <skos:narrower rdf:resource="descriptor/15086-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71005">
    <skos:narrower rdf:resource="descriptor/18488-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71005">
    <skos:narrower rdf:resource="descriptor/18487-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71005">
    <skos:narrower rdf:resource="descriptor/18478-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71005">
    <skos:narrower rdf:resource="descriptor/18453-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71005">
    <skos:narrower rdf:resource="descriptor/12721-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71005">
    <skos:narrower rdf:resource="descriptor/18467-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71005">
    <skos:narrower rdf:resource="descriptor/18461-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71005">
    <skos:narrower rdf:resource="descriptor/18458-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71005">
    <skos:narrower rdf:resource="descriptor/18153-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71005">
    <skos:narrower rdf:resource="descriptor/18486-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71005">
    <skos:narrower rdf:resource="descriptor/18457-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71005">
    <skos:narrower rdf:resource="descriptor/10656-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71005">
    <skos:narrower rdf:resource="descriptor/16603-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71005">
    <skos:narrower rdf:resource="descriptor/15703-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71005">
    <skos:narrower rdf:resource="descriptor/11906-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71005">
    <skos:narrower rdf:resource="descriptor/18489-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71005">
    <skos:narrower rdf:resource="descriptor/12051-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71005">
    <skos:narrower rdf:resource="descriptor/18484-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71005">
    <skos:narrower rdf:resource="descriptor/11198-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71005">
    <skos:narrower rdf:resource="descriptor/12913-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71005">
    <skos:narrower rdf:resource="descriptor/10500-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71005">
    <skos:narrower rdf:resource="descriptor/10464-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71005">
    <skos:narrower rdf:resource="descriptor/12908-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71005">
    <skos:narrower rdf:resource="thsys/70979"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71005">
    <skos:narrower rdf:resource="thsys/71004"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19457-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10075-5">
    <skos:prefLabel xml:lang="en">Stability of equilibrium</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10075-5">
    <skos:prefLabel xml:lang="de">Gleichgewichtsstabilität</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72186">
    <skos:narrower rdf:resource="descriptor/19575-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72186">
    <skos:narrower rdf:resource="descriptor/15243-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72186">
    <skos:narrower rdf:resource="descriptor/15298-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72186">
    <skos:narrower rdf:resource="descriptor/19037-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72186">
    <skos:narrower rdf:resource="descriptor/15272-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72186">
    <skos:narrower rdf:resource="descriptor/15340-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72186">
    <skos:narrower rdf:resource="descriptor/15315-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72186">
    <skos:narrower rdf:resource="descriptor/15336-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72186">
    <skos:narrower rdf:resource="descriptor/19148-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72186">
    <skos:narrower rdf:resource="descriptor/15266-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72186">
    <skos:narrower rdf:resource="descriptor/15265-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72186">
    <skos:narrower rdf:resource="descriptor/20506-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72186">
    <skos:narrower rdf:resource="descriptor/15264-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72186">
    <skos:narrower rdf:resource="descriptor/15365-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72186">
    <skos:narrower rdf:resource="descriptor/15258-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72186">
    <skos:narrower rdf:resource="descriptor/15362-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72186">
    <skos:narrower rdf:resource="descriptor/15251-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72186">
    <skos:narrower rdf:resource="descriptor/15328-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11826-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13219-6">
    <skos:related rdf:resource="descriptor/13238-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13219-6">
    <skos:related rdf:resource="descriptor/18991-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13219-6">
    <skos:related rdf:resource="descriptor/19338-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13219-6">
    <skos:related rdf:resource="descriptor/13160-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13219-6">
    <skos:related rdf:resource="descriptor/14931-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13219-6">
    <skos:related rdf:resource="descriptor/13188-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13219-6">
    <skos:related rdf:resource="descriptor/13306-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10597-3">
    <skos:altLabel xml:lang="en">South-south-relations</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10597-3">
    <skos:altLabel xml:lang="en">Economic cooperation among developing countries</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10597-3">
    <skos:altLabel xml:lang="en">ECDC (Economic Cooperation among Developing Countries)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10597-3">
    <skos:altLabel xml:lang="de">Süd-Süd-Handel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15753-4">
    <skos:broader rdf:resource="thsys/71010"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15753-4">
    <skos:broader rdf:resource="thsys/70811"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10555-5">
    <skos:narrower rdf:resource="descriptor/10598-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10555-5">
    <skos:narrower rdf:resource="descriptor/10595-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10555-5">
    <skos:narrower rdf:resource="descriptor/10589-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10555-5">
    <skos:narrower rdf:resource="descriptor/10573-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10555-5">
    <skos:narrower rdf:resource="descriptor/10557-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10555-5">
    <skos:narrower rdf:resource="descriptor/10543-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11633-6">
    <skos:narrower rdf:resource="descriptor/18642-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10911-2">
    <skos:narrower rdf:resource="descriptor/12863-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10911-2">
    <skos:narrower rdf:resource="descriptor/12056-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10911-2">
    <skos:narrower rdf:resource="descriptor/11134-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10911-2">
    <skos:narrower rdf:resource="descriptor/10923-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10911-2">
    <skos:narrower rdf:resource="descriptor/10897-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20077-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18975-6">
    <skos:altLabel xml:lang="de">Private Überschuldung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18975-6">
    <skos:altLabel xml:lang="de">Private Schulden</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13312-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19124-1">
    <skos:broader rdf:resource="thsys/73392"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19124-1">
    <skos:broader rdf:resource="thsys/70310"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19124-1">
    <skos:broader rdf:resource="thsys/70040"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19124-1">
    <skos:broader rdf:resource="descriptor/16036-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18705-0">
    <skos:broader rdf:resource="thsys/71072"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18705-0">
    <skos:broader rdf:resource="thsys/70903"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18705-0">
    <skos:broader rdf:resource="thsys/70860"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18705-0">
    <skos:broader rdf:resource="descriptor/11628-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18705-0">
    <skos:broader rdf:resource="descriptor/13283-4"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id307">
    <rdfs:seeAlso rdf:resource="descriptor/19468-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19825-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20553-3">
    <skos:related rdf:resource="descriptor/12251-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20553-3">
    <skos:related rdf:resource="descriptor/18673-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20553-3">
    <skos:related rdf:resource="descriptor/10552-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20553-3">
    <skos:related rdf:resource="descriptor/16533-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18395-3">
    <skos:related rdf:resource="descriptor/12918-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18395-3">
    <skos:related rdf:resource="descriptor/18522-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18395-3">
    <skos:related rdf:resource="descriptor/10301-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18395-3">
    <skos:related rdf:resource="descriptor/18100-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12396-3">
    <skos:broader rdf:resource="thsys/71043"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12396-3">
    <skos:broader rdf:resource="thsys/70397"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12396-3">
    <skos:broader rdf:resource="descriptor/12357-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10421-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/n">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14636-6">
    <skos:broader rdf:resource="thsys/70463"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14636-6">
    <skos:broader rdf:resource="descriptor/14642-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11580-1">
    <skos:altLabel xml:lang="de">VOL (Verdingungsordnung)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11580-1">
    <skos:altLabel xml:lang="de">VOB (Verdingungsordnung)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11580-1">
    <skos:altLabel xml:lang="de">Verordnung über die Preise bei öffentlichen Aufträgen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11580-1">
    <skos:altLabel xml:lang="de">Verdingungsordnung für Leistungen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11580-1">
    <skos:altLabel xml:lang="de">Verdingungsordnung für Bauleistungen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11580-1">
    <skos:altLabel xml:lang="de">Öffentliches Preisrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11580-1">
    <skos:altLabel xml:lang="de">Öffentliches Auftragsrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11580-1">
    <skos:altLabel xml:lang="de">LSP (Öffentliches Preisrecht)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11580-1">
    <skos:altLabel xml:lang="de">LSÖ (Öffentliches Preisrecht)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11580-1">
    <skos:altLabel xml:lang="de">Baupreis-VO</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17742-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71106">
    <skos:narrower rdf:resource="descriptor/12261-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71106">
    <skos:narrower rdf:resource="descriptor/13733-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71106">
    <skos:narrower rdf:resource="descriptor/19279-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71106">
    <skos:narrower rdf:resource="descriptor/18689-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71106">
    <skos:narrower rdf:resource="descriptor/12239-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71106">
    <skos:narrower rdf:resource="descriptor/13714-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71106">
    <skos:narrower rdf:resource="descriptor/19278-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71106">
    <skos:narrower rdf:resource="descriptor/16043-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71106">
    <skos:narrower rdf:resource="descriptor/18687-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71106">
    <skos:narrower rdf:resource="descriptor/13713-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71106">
    <skos:narrower rdf:resource="descriptor/13712-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71106">
    <skos:narrower rdf:resource="descriptor/12232-5"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id47">
    <rdf:value xml:lang="de">Für die berufliche Ausbildung</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11384-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70562">
    <skos:prefLabel xml:lang="en">B.01.02  Organization</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70562">
    <skos:prefLabel xml:lang="de">B.01.02  Organisation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12960-0">
    <skos:broader rdf:resource="thsys/71026"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12960-0">
    <skos:broader rdf:resource="descriptor/12940-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70013">
    <skos:broader rdf:resource="thsys/70012"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13510-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19606-4">
    <skos:related rdf:resource="descriptor/15463-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19606-4">
    <skos:related rdf:resource="descriptor/16440-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19708-3">
    <skos:altLabel xml:lang="de">Internationale Kooperation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19533-5">
    <zbwext:useInsteadNote rdf:nodeID="id242"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11365-5">
    <skos:narrower rdf:resource="descriptor/19543-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15642-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12572-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18932-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18695-5">
    <skos:scopeNote xml:lang="de">Auflagen des IWF oder der Weltbank bei der Gewährung von Krediten an Länder mit Zahlungsbilanzproblemen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14421-6">
    <skos:altLabel xml:lang="de">Luftschiff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19067-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18394-5">
    <skos:prefLabel xml:lang="en">Offshore industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18394-5">
    <skos:prefLabel xml:lang="de">Offshore-Industrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70046">
    <skos:prefLabel xml:lang="de">B.08  Betriebliches Steuerwesen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11452-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18614-3">
    <skos:altLabel xml:lang="de">Internationaler Güterverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70301">
    <skos:broader rdf:resource="thsys/b"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15860-3">
    <skos:related rdf:resource="descriptor/19734-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15860-3">
    <skos:related rdf:resource="descriptor/15861-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15860-3">
    <skos:related rdf:resource="descriptor/16721-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15860-3">
    <skos:related rdf:resource="descriptor/10975-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15860-3">
    <skos:related rdf:resource="descriptor/18151-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15860-3">
    <skos:related rdf:resource="descriptor/19533-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15860-3">
    <skos:related rdf:resource="descriptor/10252-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15860-3">
    <skos:related rdf:resource="descriptor/19219-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15860-3">
    <skos:related rdf:resource="descriptor/10300-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15860-3">
    <skos:related rdf:resource="descriptor/18832-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12732-6">
    <skos:prefLabel xml:lang="en">CAD-CAM</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12732-6">
    <skos:prefLabel xml:lang="de">CAD-CAM</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17721-0">
    <skos:broader rdf:resource="thsys/70560"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17721-0">
    <skos:broader rdf:resource="descriptor/17724-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71073">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.09.08</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13418-0">
    <skos:broader rdf:resource="thsys/70832"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13418-0">
    <skos:broader rdf:resource="thsys/70458"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13418-0">
    <skos:broader rdf:resource="descriptor/13828-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13418-0">
    <skos:broader rdf:resource="descriptor/16542-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13418-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19387-4">
    <skos:prefLabel xml:lang="en">Oil resources</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19387-4">
    <skos:prefLabel xml:lang="de">Erdölvorkommen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14146-2">
    <skos:prefLabel xml:lang="en">Goat</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14146-2">
    <skos:prefLabel xml:lang="de">Ziege</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13053-0">
    <skos:related rdf:resource="descriptor/13007-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13053-0">
    <skos:related rdf:resource="descriptor/14435-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19682-2">
    <skos:broader rdf:resource="thsys/73382"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19682-2">
    <skos:broader rdf:resource="descriptor/16292-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19755-1">
    <skos:prefLabel xml:lang="en">Confidence</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19755-1">
    <skos:prefLabel xml:lang="de">Vertrauen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19826-4">
    <skos:scopeNote xml:lang="de">Sowohl für inländische Investitionen als auch für Direktinvestitionen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13414-1">
    <skos:related rdf:resource="descriptor/13421-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13414-1">
    <skos:related rdf:resource="descriptor/18808-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13414-1">
    <skos:related rdf:resource="descriptor/16324-1"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id169">
    <rdf:value xml:lang="de">Für Untersuchungen über Massenmedien</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11595-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70881">
    <skos:broader rdf:resource="thsys/70882"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19498-2">
    <skos:altLabel xml:lang="de">Frauenerwerbslosigkeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18301-3">
    <skos:scopeNote xml:lang="de">Natur-, ingenieur-, sozial- und wirtschaftswissenschaftlich ausgerichtete Wissenschaft zu Ernährung und Haushaltsführung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18829-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12062-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13815-4">
    <skos:altLabel xml:lang="en">Catastrophe insurance</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13815-4">
    <skos:altLabel xml:lang="de">Wetterschadenversicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13815-4">
    <skos:altLabel xml:lang="de">Sturmversicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13815-4">
    <skos:altLabel xml:lang="de">Hagelversicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13815-4">
    <skos:altLabel xml:lang="de">Elementargefahrenversicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19724-5">
    <skos:scopeNote xml:lang="de">Versorgung der Truppe mit Verpflegung, Ausrüstung und Material aller Art.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13085-1">
    <skos:broader rdf:resource="thsys/71003"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13085-1">
    <skos:broader rdf:resource="descriptor/13034-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19669-1">
    <skos:altLabel xml:lang="en">Volume</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19669-1">
    <skos:altLabel xml:lang="en">Trade size</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19669-1">
    <skos:altLabel xml:lang="de">Handelsvolumen an der Börse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11038-6">
    <skos:broader rdf:resource="thsys/70274"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11038-6">
    <skos:broader rdf:resource="descriptor/10975-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13823-5">
    <skos:related rdf:resource="descriptor/13826-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13823-5">
    <skos:related rdf:resource="descriptor/13824-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13823-5">
    <skos:related rdf:resource="descriptor/13819-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13823-5">
    <skos:related rdf:resource="descriptor/21002-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13823-5">
    <skos:related rdf:resource="descriptor/16539-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18161-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12277-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19291-3">
    <skos:broader rdf:resource="thsys/73367"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19291-3">
    <skos:broader rdf:resource="thsys/71069"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19291-3">
    <skos:broader rdf:resource="thsys/71067"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19291-3">
    <skos:broader rdf:resource="thsys/70038"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19291-3">
    <skos:broader rdf:resource="descriptor/10987-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17940-2">
    <skos:broader rdf:resource="thsys/70128"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17940-2">
    <skos:broader rdf:resource="descriptor/17948-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11956-3">
    <skos:broader rdf:resource="thsys/71070"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11956-3">
    <skos:broader rdf:resource="descriptor/11873-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19255-0">
    <skos:broader rdf:resource="thsys/73387"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19255-0">
    <skos:broader rdf:resource="thsys/70109"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19255-0">
    <skos:broader rdf:resource="thsys/70055"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19255-0">
    <skos:broader rdf:resource="descriptor/10764-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19255-0">
    <skos:broader rdf:resource="descriptor/11416-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70481">
    <skos:narrower rdf:resource="descriptor/10114-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70481">
    <skos:narrower rdf:resource="descriptor/12071-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70481">
    <skos:narrower rdf:resource="descriptor/10190-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70481">
    <skos:narrower rdf:resource="descriptor/19249-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70481">
    <skos:narrower rdf:resource="descriptor/10187-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70481">
    <skos:narrower rdf:resource="descriptor/20881-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70481">
    <skos:narrower rdf:resource="descriptor/10186-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70481">
    <skos:narrower rdf:resource="descriptor/10185-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70481">
    <skos:narrower rdf:resource="descriptor/10192-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16955-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#broaderTransitive">
    <rdfs:isDefinedBy rdf:resource="http://www.w3.org/2004/02/skos/core"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15416-1">
    <skos:scopeNote xml:lang="de">Das Zusammenwirken von automatisierten und menschlichen/manuellen Tätigkeiten bei der Informationsverarbeitung. Teilautomatisierung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11633-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15077-4">
    <skos:related rdf:resource="descriptor/13200-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16782-6">
    <skos:altLabel xml:lang="de">Informations- und Dokumentationswesen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16782-6">
    <skos:altLabel xml:lang="de">Information und Dokumentation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16782-6">
    <skos:altLabel xml:lang="de">Dokumentationswesen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16782-6">
    <skos:altLabel xml:lang="de">Dokumentationslehre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15535-0">
    <skos:prefLabel xml:lang="en">Integer programming</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15535-0">
    <skos:prefLabel xml:lang="de">Ganzzahlige Optimierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16709-2">
    <skos:related rdf:resource="descriptor/11111-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16709-2">
    <skos:related rdf:resource="descriptor/15806-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16709-2">
    <skos:related rdf:resource="descriptor/16641-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16709-2">
    <skos:related rdf:resource="descriptor/19768-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12896-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16335-3">
    <skos:broader rdf:resource="thsys/73380"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16335-3">
    <skos:broader rdf:resource="thsys/70871"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10001-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19547-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16432-5">
    <skos:related rdf:resource="descriptor/16642-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16432-5">
    <skos:related rdf:resource="descriptor/16430-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16432-5">
    <skos:related rdf:resource="descriptor/16369-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11388-0">
    <skos:prefLabel xml:lang="en">Vocational rehabilitation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11388-0">
    <skos:prefLabel xml:lang="de">Berufliche Rehabilitation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18941-2">
    <skos:narrower rdf:resource="descriptor/18179-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16039-0">
    <skos:broader rdf:resource="thsys/73392"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16039-0">
    <skos:broader rdf:resource="descriptor/16036-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13334-6">
    <skos:altLabel xml:lang="de">Stadtentwässerung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13334-6">
    <skos:altLabel xml:lang="de">Kommunale Abwasserentsorgung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15416-1">
    <skos:altLabel xml:lang="en">Man-machine communication</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15416-1">
    <skos:altLabel xml:lang="de">Mensch-Maschine-Kommunikation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15416-1">
    <skos:altLabel xml:lang="de">Mensch-Computer-Interaktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15416-1">
    <skos:altLabel xml:lang="de">Mensch-Computer-Dialog</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11992-6">
    <skos:related rdf:resource="descriptor/11983-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11992-6">
    <skos:related rdf:resource="descriptor/19474-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11992-6">
    <skos:related rdf:resource="descriptor/11971-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11992-6">
    <skos:related rdf:resource="descriptor/11969-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11992-6">
    <skos:related rdf:resource="descriptor/16688-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11992-6">
    <skos:related rdf:resource="descriptor/16671-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11992-6">
    <skos:related rdf:resource="descriptor/19469-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18996-5">
    <skos:narrower rdf:resource="descriptor/19192-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18996-5">
    <skos:narrower rdf:resource="descriptor/19385-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13233-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14217-5">
    <skos:altLabel xml:lang="de">Schmieröl</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14217-5">
    <skos:altLabel xml:lang="de">Schmiermittel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14217-5">
    <skos:altLabel xml:lang="de">Schmierfett</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14217-5">
    <skos:altLabel xml:lang="de">Motorenöl</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14217-5">
    <skos:altLabel xml:lang="de">Isolieröl</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18277-2">
    <skos:narrower rdf:resource="descriptor/14808-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18277-2">
    <skos:narrower rdf:resource="descriptor/14043-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18277-2">
    <skos:narrower rdf:resource="descriptor/14895-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18277-2">
    <skos:narrower rdf:resource="descriptor/14022-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71015">
    <skos:broader rdf:resource="thsys/71016"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12178-6">
    <skos:altLabel xml:lang="de">Einzelkaufmann</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12178-6">
    <skos:altLabel xml:lang="de">Einzelfirma</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15689-1">
    <skos:related rdf:resource="descriptor/15699-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15689-1">
    <skos:related rdf:resource="descriptor/15696-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15689-1">
    <skos:related rdf:resource="descriptor/19781-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15689-1">
    <skos:related rdf:resource="descriptor/19634-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10282-0">
    <skos:broader rdf:resource="thsys/71112"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10282-0">
    <skos:broader rdf:resource="descriptor/10181-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10215-1">
    <skos:altLabel xml:lang="en">Skimming pricing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10215-1">
    <skos:altLabel xml:lang="en">Cream skimming</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10215-1">
    <skos:altLabel xml:lang="en">Basing-point system</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10215-1">
    <skos:altLabel xml:lang="en">Basing-point pricing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10215-1">
    <skos:altLabel xml:lang="de">Regionale Preispolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10215-1">
    <skos:altLabel xml:lang="de">Preisdiskriminierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10215-1">
    <skos:altLabel xml:lang="de">Nichtlineare Preisbildung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10215-1">
    <skos:altLabel xml:lang="de">Abschöpfungspreispolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14194-5">
    <skos:altLabel xml:lang="de">Pechkohle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14194-5">
    <skos:altLabel xml:lang="de">Braunkohlenbrikett</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13790-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10964-2">
    <skos:altLabel xml:lang="en">Fair-trade law</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id130">
    <rdfs:seeAlso rdf:resource="descriptor/13459-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19564-1">
    <skos:broader rdf:resource="thsys/70947"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18659-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18633-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14272-4">
    <skos:related rdf:resource="descriptor/13039-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10207-0">
    <skos:related rdf:resource="descriptor/18130-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10207-0">
    <skos:related rdf:resource="descriptor/13826-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10207-0">
    <skos:related rdf:resource="descriptor/18741-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10207-0">
    <skos:related rdf:resource="descriptor/13820-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10207-0">
    <skos:related rdf:resource="descriptor/10194-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10207-0">
    <skos:related rdf:resource="descriptor/19227-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11823-1">
    <skos:broader rdf:resource="thsys/71110"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11823-1">
    <skos:broader rdf:resource="thsys/70963"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11823-1">
    <skos:broader rdf:resource="descriptor/10981-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70477">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19727-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70144">
    <skos:narrower rdf:resource="descriptor/12511-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70144">
    <skos:narrower rdf:resource="descriptor/11580-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70144">
    <skos:narrower rdf:resource="descriptor/11541-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70144">
    <skos:narrower rdf:resource="descriptor/10590-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70144">
    <skos:narrower rdf:resource="descriptor/19373-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70144">
    <skos:narrower rdf:resource="descriptor/11722-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70144">
    <skos:narrower rdf:resource="descriptor/11540-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70144">
    <skos:narrower rdf:resource="descriptor/11539-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70144">
    <skos:narrower rdf:resource="descriptor/11579-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70144">
    <skos:narrower rdf:resource="descriptor/19372-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70144">
    <skos:narrower rdf:resource="descriptor/11531-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70144">
    <skos:narrower rdf:resource="descriptor/18149-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70144">
    <skos:narrower rdf:resource="descriptor/11582-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70144">
    <skos:narrower rdf:resource="descriptor/19370-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70144">
    <skos:narrower rdf:resource="descriptor/11530-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70144">
    <skos:narrower rdf:resource="descriptor/11529-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70144">
    <skos:narrower rdf:resource="descriptor/11537-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70144">
    <skos:narrower rdf:resource="descriptor/11536-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70144">
    <skos:narrower rdf:resource="descriptor/11535-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70144">
    <skos:narrower rdf:resource="descriptor/11534-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70144">
    <skos:narrower rdf:resource="descriptor/11533-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70144">
    <skos:narrower rdf:resource="descriptor/19941-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70144">
    <skos:narrower rdf:resource="descriptor/11576-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70144">
    <skos:narrower rdf:resource="thsys/71080"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70144">
    <skos:narrower rdf:resource="thsys/71083"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70144">
    <skos:narrower rdf:resource="thsys/71082"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70144">
    <skos:narrower rdf:resource="thsys/71073"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70144">
    <skos:narrower rdf:resource="thsys/71077"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70144">
    <skos:narrower rdf:resource="thsys/71072"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70144">
    <skos:narrower rdf:resource="thsys/71079"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70144">
    <skos:narrower rdf:resource="thsys/71078"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70144">
    <skos:narrower rdf:resource="thsys/71081"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17727-2">
    <skos:altLabel xml:lang="de">Volksrepublik Angola</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17727-2">
    <skos:altLabel xml:lang="de">Republik Angola</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17727-2">
    <skos:altLabel xml:lang="de">Portugiesisch-Westafrika</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17791-0">
    <skos:prefLabel xml:lang="en">Central America</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17791-0">
    <skos:prefLabel xml:lang="de">Mittelamerika</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id341">
    <rdf:value xml:lang="de">Für den Schumpeterschen / schöpferischen Unternehmer</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15372-2">
    <skos:broader rdf:resource="thsys/72182"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15372-2">
    <skos:broader rdf:resource="descriptor/15064-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18597-5">
    <skos:broader rdf:resource="thsys/70940"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18597-5">
    <skos:broader rdf:resource="thsys/70934"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18597-5">
    <skos:broader rdf:resource="descriptor/13553-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11628-6">
    <skos:narrower rdf:resource="descriptor/11629-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11628-6">
    <skos:narrower rdf:resource="descriptor/18705-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11628-6">
    <skos:narrower rdf:resource="descriptor/11627-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11628-6">
    <skos:narrower rdf:resource="descriptor/11626-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16661-4">
    <skos:broader rdf:resource="thsys/73358"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16661-4">
    <skos:broader rdf:resource="thsys/70142"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16661-4">
    <skos:broader rdf:resource="descriptor/15714-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16661-4">
    <skos:broader rdf:resource="descriptor/15914-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19812-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19211-6">
    <skos:scopeNote xml:lang="de">Veränderungen in den institutionellen Rahmenbedingungen des Wirtschaftsgeschehens, vor allem im Rechtssystem oder der Wirtschaftsverfassung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10691-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19515-0">
    <skos:broader rdf:resource="thsys/70832"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19515-0">
    <skos:broader rdf:resource="thsys/70458"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19515-0">
    <skos:broader rdf:resource="descriptor/13823-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19515-0">
    <skos:broader rdf:resource="descriptor/16539-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15767-0">
    <skos:altLabel xml:lang="en">Ecological economics</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15767-0">
    <skos:altLabel xml:lang="de">Umweltökonomie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15767-0">
    <skos:altLabel xml:lang="de">Umweltallokation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15767-0">
    <skos:altLabel xml:lang="de">Ökonomische Theorie der Umwelt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15767-0">
    <skos:altLabel xml:lang="de">Ökologisch orientierte Ökonomie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17984-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14716-1">
    <skos:prefLabel xml:lang="en">Foam material</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14716-1">
    <skos:prefLabel xml:lang="de">Schaumstoff</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12489-3">
    <skos:related rdf:resource="descriptor/12474-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12489-3">
    <skos:related rdf:resource="descriptor/12469-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#relatedMatch">
    <rdfs:isDefinedBy rdf:resource="http://www.w3.org/2004/02/skos/core"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14225-6">
    <skos:broader rdf:resource="thsys/70348"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14225-6">
    <skos:broader rdf:resource="descriptor/14227-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15476-4">
    <skos:prefLabel xml:lang="en">Multicriteria analysis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15476-4">
    <skos:prefLabel xml:lang="de">Multikriteria-Verfahren</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18143-2">
    <skos:altLabel xml:lang="en">Management training</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18143-2">
    <skos:altLabel xml:lang="de">Weiterbildung von Führungskräften</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18143-2">
    <skos:altLabel xml:lang="de">Trainee-Programm</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18143-2">
    <skos:altLabel xml:lang="de">Managerweiterbildung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18143-2">
    <skos:altLabel xml:lang="de">Managerschulung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18143-2">
    <skos:altLabel xml:lang="de">Managerausbildung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18143-2">
    <skos:altLabel xml:lang="de">Management-Weiterbildung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18143-2">
    <skos:altLabel xml:lang="de">Management-Ausbildung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18143-2">
    <skos:altLabel xml:lang="de">Kaderentwicklung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18143-2">
    <skos:altLabel xml:lang="de">Führungskräfteweiterbildung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18143-2">
    <skos:altLabel xml:lang="de">Führungskräfteentwicklung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18143-2">
    <skos:altLabel xml:lang="de">Förderung des Managementnachwuchses</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18143-2">
    <skos:altLabel xml:lang="de">Förderung des Führungsnachwuchses</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18143-2">
    <skos:altLabel xml:lang="de">Betriebliche Nachwuchspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18143-2">
    <skos:altLabel xml:lang="de">Ausbildung von Führungskräften</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18263-6">
    <skos:related rdf:resource="descriptor/14735-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11321-4">
    <skos:altLabel xml:lang="de">Investives Sparen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11321-4">
    <skos:altLabel xml:lang="de">Investiver Lohnanteil</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11321-4">
    <skos:altLabel xml:lang="de">Investive Lohnpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12777-5">
    <skos:prefLabel xml:lang="en">Marketing cooperative</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12777-5">
    <skos:prefLabel xml:lang="de">Absatzkooperation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11371-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11801-4">
    <skos:prefLabel xml:lang="en">Agricultural Policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11801-4">
    <skos:prefLabel xml:lang="de">Agrarpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18199-3">
    <skos:prefLabel xml:lang="en">Coca</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18199-3">
    <skos:prefLabel xml:lang="de">Koka</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16185-3">
    <skos:broader rdf:resource="thsys/73333"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16185-3">
    <skos:broader rdf:resource="descriptor/10481-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16185-3">
    <skos:broader rdf:resource="descriptor/16032-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11569-3">
    <skos:scopeNote xml:lang="de">Auch für entsprechende Programme und Transferzahlungen in anderen Ländern.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14740-4">
    <skos:prefLabel xml:lang="en">Surfactants</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14740-4">
    <skos:prefLabel xml:lang="de">Tenside</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10947-2">
    <skos:broader rdf:resource="thsys/70242"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10947-2">
    <skos:broader rdf:resource="descriptor/19565-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10947-2">
    <skos:broader rdf:resource="descriptor/10895-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11417-5">
    <skos:prefLabel xml:lang="en">Monetary law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11417-5">
    <skos:prefLabel xml:lang="de">Währungsrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11087-0">
    <skos:broader rdf:resource="thsys/73395"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11087-0">
    <skos:broader rdf:resource="descriptor/15719-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18814-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13304-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13161-4">
    <skos:altLabel xml:lang="de">Obstverarbeitende Industrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13161-4">
    <skos:altLabel xml:lang="de">Obstkonservenindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13161-4">
    <skos:altLabel xml:lang="de">Gemüseverwertungsindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13161-4">
    <skos:altLabel xml:lang="de">Gemüseverarbeitende Industrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13161-4">
    <skos:altLabel xml:lang="de">Gemüsekonservenindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13161-4">
    <skos:altLabel xml:lang="de">Fruchtkonservenindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19749-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14601-4">
    <skos:related rdf:resource="descriptor/14236-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14601-4">
    <skos:related rdf:resource="descriptor/14599-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12283-2">
    <skos:related rdf:resource="descriptor/12036-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12283-2">
    <skos:related rdf:resource="descriptor/20018-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12283-2">
    <skos:related rdf:resource="descriptor/12282-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12283-2">
    <skos:related rdf:resource="descriptor/12281-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12283-2">
    <skos:related rdf:resource="descriptor/12287-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12283-2">
    <skos:related rdf:resource="descriptor/12050-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12283-2">
    <skos:related rdf:resource="descriptor/11414-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18535-6">
    <skos:broader rdf:resource="thsys/71055"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18535-6">
    <skos:broader rdf:resource="thsys/70958"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15894-0">
    <skos:prefLabel xml:lang="en">Arid zone</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15894-0">
    <skos:prefLabel xml:lang="de">Trockenklima</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11713-1">
    <skos:scopeNote xml:lang="de">Verteilungswirkung öffentlicher Haushalte.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16426-0">
    <skos:altLabel xml:lang="de">Politische Freiheit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16980-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10381-5">
    <skos:narrower rdf:resource="descriptor/11727-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19239-5">
    <skos:altLabel xml:lang="en">RBC</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19239-5">
    <skos:altLabel xml:lang="de">Theorie realwirtschaftlich bedingter Konjunkturzyklen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19239-5">
    <skos:altLabel xml:lang="de">Reale-Konjunkturzyklus-Hypothese</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19239-5">
    <skos:altLabel xml:lang="de">Reale Konjunkturzyklushypothese</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10031-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19113-6">
    <skos:broader rdf:resource="thsys/70368"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19113-6">
    <skos:broader rdf:resource="descriptor/12667-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18861-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13691-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70928">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.12.05</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10926-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16017-3">
    <skos:prefLabel xml:lang="en">Constitutional law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16017-3">
    <skos:prefLabel xml:lang="de">Staatsrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12724-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12192-5">
    <skos:prefLabel xml:lang="en">Partnership</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12192-5">
    <skos:prefLabel xml:lang="de">Personengesellschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11487-5">
    <skos:related rdf:resource="descriptor/13738-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11487-5">
    <skos:related rdf:resource="descriptor/13762-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11487-5">
    <skos:related rdf:resource="descriptor/11611-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11487-5">
    <skos:related rdf:resource="descriptor/11479-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11487-5">
    <skos:related rdf:resource="descriptor/11421-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11400-1">
    <skos:related rdf:resource="descriptor/15718-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11400-1">
    <skos:related rdf:resource="descriptor/16623-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11400-1">
    <skos:related rdf:resource="descriptor/16429-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11400-1">
    <skos:related rdf:resource="descriptor/18863-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11400-1">
    <skos:related rdf:resource="descriptor/15713-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11400-1">
    <skos:related rdf:resource="descriptor/18860-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11400-1">
    <skos:related rdf:resource="descriptor/11397-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11400-1">
    <skos:related rdf:resource="descriptor/11394-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11400-1">
    <skos:related rdf:resource="descriptor/11353-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11400-1">
    <skos:related rdf:resource="descriptor/11189-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11400-1">
    <skos:related rdf:resource="descriptor/10486-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10870-4">
    <skos:broader rdf:resource="thsys/71045"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10870-4">
    <skos:broader rdf:resource="thsys/70951"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10870-4">
    <skos:broader rdf:resource="descriptor/10854-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14664-1">
    <skos:broader rdf:resource="thsys/70556"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14664-1">
    <skos:broader rdf:resource="descriptor/14683-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13815-4">
    <skos:broader rdf:resource="thsys/70891"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13815-4">
    <skos:broader rdf:resource="descriptor/13816-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16630-1">
    <skos:altLabel xml:lang="de">Jugendverband</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16630-1">
    <skos:altLabel xml:lang="de">Jugendbewegung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73362">
    <skos:narrower rdf:resource="descriptor/18627-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73362">
    <skos:narrower rdf:resource="descriptor/19727-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73362">
    <skos:narrower rdf:resource="descriptor/19445-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73362">
    <skos:narrower rdf:resource="descriptor/18909-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73362">
    <skos:narrower rdf:resource="descriptor/16349-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73362">
    <skos:narrower rdf:resource="descriptor/16496-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73362">
    <skos:narrower rdf:resource="descriptor/10734-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73362">
    <skos:narrower rdf:resource="descriptor/19336-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73362">
    <skos:narrower rdf:resource="descriptor/16495-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73362">
    <skos:narrower rdf:resource="descriptor/18773-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73362">
    <skos:narrower rdf:resource="descriptor/11937-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73362">
    <skos:narrower rdf:resource="descriptor/11944-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73362">
    <skos:narrower rdf:resource="descriptor/18772-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73362">
    <skos:narrower rdf:resource="descriptor/19546-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73362">
    <skos:narrower rdf:resource="descriptor/18341-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73362">
    <skos:narrower rdf:resource="descriptor/19352-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73362">
    <skos:narrower rdf:resource="descriptor/18370-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73362">
    <skos:narrower rdf:resource="descriptor/10608-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73362">
    <skos:narrower rdf:resource="descriptor/18536-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73362">
    <skos:narrower rdf:resource="descriptor/18889-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73362">
    <skos:narrower rdf:resource="descriptor/19956-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73362">
    <skos:narrower rdf:resource="descriptor/18882-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73362">
    <skos:narrower rdf:resource="descriptor/19663-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73362">
    <skos:narrower rdf:resource="descriptor/18331-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73362">
    <skos:narrower rdf:resource="descriptor/18332-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73362">
    <skos:narrower rdf:resource="descriptor/11806-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12765-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="r1238766808r28278r3">
    <rdf:rest rdf:resource="http://www.w3.org/1999/02/22-rdf-syntax-ns#nil"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10895-2">
    <skos:altLabel xml:lang="de">Wettbewerbsfreiheit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10895-2">
    <skos:altLabel xml:lang="de">Wettbewerbsfähigkeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10895-2">
    <skos:altLabel xml:lang="de">Konkurrenz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10895-2">
    <skos:altLabel xml:lang="de">Freier Wettbewerb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10612-5">
    <skos:altLabel xml:lang="en">FTA (Free trade area)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10612-5">
    <skos:altLabel xml:lang="en">Free trade association</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10612-5">
    <skos:altLabel xml:lang="en">Free trade agreement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70348">
    <skos:prefLabel xml:lang="de">P.06  Steine, Erden, Keramik und Glas</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13234-3">
    <skos:broader rdf:resource="thsys/70960"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13234-3">
    <skos:broader rdf:resource="descriptor/13207-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13416-4">
    <skos:broader rdf:resource="thsys/70832"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13416-4">
    <skos:broader rdf:resource="thsys/70458"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13416-4">
    <skos:broader rdf:resource="descriptor/13418-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13416-4">
    <skos:broader rdf:resource="descriptor/16527-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73366">
    <skos:narrower rdf:resource="descriptor/10619-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73366">
    <skos:narrower rdf:resource="descriptor/16436-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73366">
    <skos:narrower rdf:resource="descriptor/16440-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73366">
    <skos:narrower rdf:resource="descriptor/16389-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73366">
    <skos:narrower rdf:resource="descriptor/19480-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73366">
    <skos:narrower rdf:resource="descriptor/10598-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73366">
    <skos:narrower rdf:resource="descriptor/11049-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73366">
    <skos:narrower rdf:resource="descriptor/10597-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73366">
    <skos:narrower rdf:resource="descriptor/19714-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73366">
    <skos:narrower rdf:resource="descriptor/16422-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73366">
    <skos:narrower rdf:resource="descriptor/16421-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73366">
    <skos:narrower rdf:resource="descriptor/16278-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73366">
    <skos:narrower rdf:resource="descriptor/19719-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73366">
    <skos:narrower rdf:resource="descriptor/19718-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73366">
    <skos:narrower rdf:resource="descriptor/10623-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73366">
    <skos:narrower rdf:resource="descriptor/19668-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73366">
    <skos:narrower rdf:resource="descriptor/10587-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73366">
    <skos:narrower rdf:resource="descriptor/13395-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73366">
    <skos:narrower rdf:resource="descriptor/16417-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73366">
    <skos:narrower rdf:resource="descriptor/16264-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73366">
    <skos:narrower rdf:resource="descriptor/18062-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73366">
    <skos:narrower rdf:resource="descriptor/10583-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73366">
    <skos:narrower rdf:resource="descriptor/19713-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73366">
    <skos:narrower rdf:resource="descriptor/19631-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73366">
    <skos:narrower rdf:resource="descriptor/19715-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73366">
    <skos:narrower rdf:resource="descriptor/16215-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73366">
    <skos:narrower rdf:resource="descriptor/16648-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73366">
    <skos:narrower rdf:resource="descriptor/19359-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73366">
    <skos:narrower rdf:resource="descriptor/19717-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73366">
    <skos:narrower rdf:resource="descriptor/10574-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73366">
    <skos:narrower rdf:resource="descriptor/19228-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73366">
    <skos:narrower rdf:resource="descriptor/10573-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73366">
    <skos:narrower rdf:resource="descriptor/16184-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73366">
    <skos:narrower rdf:resource="descriptor/19708-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73366">
    <skos:narrower rdf:resource="descriptor/19712-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73366">
    <skos:narrower rdf:resource="descriptor/10618-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73366">
    <skos:narrower rdf:resource="descriptor/10603-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73366">
    <skos:narrower rdf:resource="descriptor/10617-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73366">
    <skos:narrower rdf:resource="descriptor/15738-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73366">
    <skos:narrower rdf:resource="descriptor/10452-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73366">
    <skos:narrower rdf:resource="descriptor/19509-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73366">
    <skos:narrower rdf:resource="descriptor/16435-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73366">
    <skos:narrower rdf:resource="descriptor/16439-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73366">
    <skos:narrower rdf:resource="descriptor/19711-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73366">
    <skos:narrower rdf:resource="descriptor/16438-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73366">
    <skos:narrower rdf:resource="descriptor/16441-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73366">
    <skos:narrower rdf:resource="descriptor/11083-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73366">
    <skos:narrower rdf:resource="descriptor/10698-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73366">
    <skos:narrower rdf:resource="descriptor/19709-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73366">
    <skos:narrower rdf:resource="descriptor/19071-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73366">
    <skos:narrower rdf:resource="descriptor/20826-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73366">
    <skos:narrower rdf:resource="descriptor/16502-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73366">
    <skos:narrower rdf:resource="descriptor/10558-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73366">
    <skos:narrower rdf:resource="descriptor/10557-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73366">
    <skos:narrower rdf:resource="descriptor/10555-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73366">
    <skos:narrower rdf:resource="descriptor/16493-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73366">
    <skos:narrower rdf:resource="descriptor/16066-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73366">
    <skos:narrower rdf:resource="descriptor/20733-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73366">
    <skos:narrower rdf:resource="descriptor/13407-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73366">
    <skos:narrower rdf:resource="descriptor/19716-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73366">
    <skos:narrower rdf:resource="descriptor/10682-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73366">
    <skos:narrower rdf:resource="descriptor/16489-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73366">
    <skos:narrower rdf:resource="descriptor/20722-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73366">
    <skos:narrower rdf:resource="thsys/73363"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73366">
    <skos:narrower rdf:resource="thsys/73365"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73366">
    <skos:narrower rdf:resource="thsys/73362"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73366">
    <skos:narrower rdf:resource="thsys/73364"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10140-6">
    <skos:broader rdf:resource="thsys/71117"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10140-6">
    <skos:broader rdf:resource="thsys/70823"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10140-6">
    <skos:broader rdf:resource="thsys/70383"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10140-6">
    <skos:broader rdf:resource="descriptor/10011-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10833-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19856-2">
    <skos:altLabel xml:lang="de">Vollmacht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19856-2">
    <skos:altLabel xml:lang="de">Vertretungsmacht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19856-2">
    <skos:altLabel xml:lang="de">Prokura</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19856-2">
    <skos:altLabel xml:lang="de">Handlungsvollmacht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19856-2">
    <skos:altLabel xml:lang="de">Gesetzliche Vertretung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18350-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70847">
    <skos:narrower rdf:resource="descriptor/11382-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70847">
    <skos:narrower rdf:resource="descriptor/15666-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70847">
    <skos:narrower rdf:resource="descriptor/11409-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70847">
    <skos:narrower rdf:resource="descriptor/15786-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70847">
    <skos:narrower rdf:resource="descriptor/15656-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70847">
    <skos:narrower rdf:resource="descriptor/11401-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70847">
    <skos:narrower rdf:resource="descriptor/11374-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70847">
    <skos:narrower rdf:resource="descriptor/18865-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70847">
    <skos:narrower rdf:resource="descriptor/18864-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70847">
    <skos:narrower rdf:resource="descriptor/11224-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70847">
    <skos:narrower rdf:resource="descriptor/11371-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70847">
    <skos:narrower rdf:resource="descriptor/13342-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70847">
    <skos:narrower rdf:resource="descriptor/11369-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70847">
    <skos:narrower rdf:resource="descriptor/18074-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70847">
    <skos:narrower rdf:resource="descriptor/11367-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70847">
    <skos:narrower rdf:resource="descriptor/18860-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70847">
    <skos:narrower rdf:resource="descriptor/11399-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70847">
    <skos:narrower rdf:resource="descriptor/18888-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70847">
    <skos:narrower rdf:resource="descriptor/11398-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70847">
    <skos:narrower rdf:resource="descriptor/11397-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70847">
    <skos:narrower rdf:resource="descriptor/11407-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70847">
    <skos:narrower rdf:resource="descriptor/11354-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70847">
    <skos:narrower rdf:resource="descriptor/11222-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70847">
    <skos:narrower rdf:resource="descriptor/11394-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70847">
    <skos:narrower rdf:resource="descriptor/18884-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70847">
    <skos:narrower rdf:resource="descriptor/19516-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70847">
    <skos:narrower rdf:resource="descriptor/18857-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70847">
    <skos:narrower rdf:resource="descriptor/11400-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70847">
    <skos:narrower rdf:resource="descriptor/19916-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70847">
    <skos:narrower rdf:resource="descriptor/11353-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70847">
    <skos:narrower rdf:resource="descriptor/10486-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70847">
    <skos:narrower rdf:resource="descriptor/11355-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70847">
    <skos:narrower rdf:resource="descriptor/18856-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70847">
    <skos:narrower rdf:resource="thsys/70843"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70847">
    <skos:narrower rdf:resource="thsys/70844"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70847">
    <skos:narrower rdf:resource="thsys/70845"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73392">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18489-1">
    <skos:broader rdf:resource="thsys/71005"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18489-1">
    <skos:broader rdf:resource="thsys/70058"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18489-1">
    <skos:broader rdf:resource="descriptor/11916-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17689-5">
    <skos:altLabel xml:lang="de">Königreich Swasiland</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70209">
    <skos:prefLabel xml:lang="en">V.12.01  Environmental economics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70209">
    <skos:prefLabel xml:lang="de">V.12.01  Umweltökonomik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19119-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18205-6">
    <skos:altLabel xml:lang="de">Zeitungspapier</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18205-6">
    <skos:altLabel xml:lang="de">Zeitungsdruckpapier</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18205-6">
    <skos:altLabel xml:lang="de">Schreibpapier</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13727-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17922-4">
    <skos:broader rdf:resource="thsys/71030"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17922-4">
    <skos:broader rdf:resource="descriptor/17931-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12867-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10179-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19291-3">
    <skos:narrower rdf:resource="descriptor/18031-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19291-3">
    <skos:narrower rdf:resource="descriptor/19448-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19291-3">
    <skos:narrower rdf:resource="descriptor/11701-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19291-3">
    <skos:narrower rdf:resource="descriptor/11940-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12078-3">
    <skos:related rdf:resource="descriptor/12303-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12078-3">
    <skos:related rdf:resource="descriptor/18118-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12078-3">
    <skos:related rdf:resource="descriptor/12166-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12078-3">
    <skos:related rdf:resource="descriptor/19247-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17011-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11209-5">
    <skos:related rdf:resource="descriptor/19496-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11209-5">
    <skos:related rdf:resource="descriptor/19640-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11209-5">
    <skos:related rdf:resource="descriptor/16052-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71056">
    <rdfs:label xml:lang="de">Technische und Fertigungsberufe</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17938-3">
    <skos:altLabel xml:lang="de">Gilbertinseln</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18300-5">
    <skos:scopeNote xml:lang="de">Bezeichnung für alle Produkte, die vorwiegend im Lebensmitteleinzelhandel neben dem einschlägigen Sortiment angeboten werden.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18791-2">
    <skos:related rdf:resource="descriptor/13391-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18791-2">
    <skos:related rdf:resource="descriptor/19159-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18791-2">
    <skos:related rdf:resource="descriptor/16058-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16250-4">
    <skos:broader rdf:resource="thsys/73396"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16250-4">
    <skos:broader rdf:resource="descriptor/16602-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70188">
    <skos:broader rdf:resource="thsys/70049"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12535-1">
    <skos:altLabel xml:lang="de">Kostenzurechnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12535-1">
    <skos:altLabel xml:lang="de">Kostenverteilung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19631-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17179-0">
    <skos:prefLabel xml:lang="en">Southeastern Europe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17179-0">
    <skos:prefLabel xml:lang="de">Südosteuropa</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14539-4">
    <skos:altLabel xml:lang="de">Phototechnisches Produkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14539-4">
    <skos:altLabel xml:lang="de">Photolaborausrüstung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14539-4">
    <skos:altLabel xml:lang="de">Fotolaborausrüstung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14539-4">
    <skos:altLabel xml:lang="de">Filmprojektor</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14539-4">
    <skos:altLabel xml:lang="de">Filmkamera</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14539-4">
    <skos:altLabel xml:lang="de">Blitzlichtgerät</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20744-3">
    <skos:broader rdf:resource="thsys/70239"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20744-3">
    <skos:broader rdf:resource="descriptor/15738-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19400-3">
    <skos:related rdf:resource="descriptor/19408-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19400-3">
    <skos:related rdf:resource="descriptor/11635-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17016-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13690-5">
    <skos:scopeNote xml:lang="de">Struktur der Kreditinstitute und ihre Beziehungen untereinander.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20789-2">
    <skos:related rdf:resource="descriptor/10027-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20789-2">
    <skos:related rdf:resource="descriptor/12699-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10746-5">
    <skos:broader rdf:resource="thsys/71094"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13474-4">
    <skos:broader rdf:resource="thsys/70956"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13474-4">
    <skos:broader rdf:resource="thsys/70550"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13474-4">
    <skos:broader rdf:resource="descriptor/12808-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14793-4">
    <skos:broader rdf:resource="thsys/70416"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14793-4">
    <skos:broader rdf:resource="descriptor/14788-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10486-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17597-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73354">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12036-5">
    <skos:altLabel xml:lang="de">Zwischenbetriebliche Zusammenarbeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12036-5">
    <skos:altLabel xml:lang="de">Zwischenbetriebliche Kooperation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12036-5">
    <skos:altLabel xml:lang="de">Zusammenarbeit von Unternehmen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12036-5">
    <skos:altLabel xml:lang="de">Kooperation von Betrieben</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12036-5">
    <skos:altLabel xml:lang="de">Betriebliche Zusammenarbeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12036-5">
    <skos:altLabel xml:lang="de">Betriebliche Kooperation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14804-4">
    <skos:broader rdf:resource="thsys/70333"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14804-4">
    <skos:broader rdf:resource="thsys/70328"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14804-4">
    <skos:broader rdf:resource="descriptor/14051-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14804-4">
    <skos:broader rdf:resource="descriptor/14803-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13417-2">
    <skos:related rdf:resource="descriptor/13418-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13417-2">
    <skos:related rdf:resource="descriptor/16159-4"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id343">
    <rdf:value xml:lang="de">Für statistische Erhebungen zur Struktur der Ausgaben privater Haushalte</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19517-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19111-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13867-6">
    <skos:altLabel xml:lang="en">Residential change</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13867-6">
    <skos:altLabel xml:lang="en">Move decision</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13867-6">
    <skos:altLabel xml:lang="de">Wohnortwechsel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13867-6">
    <skos:altLabel xml:lang="de">Wohnmobilität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13867-6">
    <skos:altLabel xml:lang="de">Umzugsverhalten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13867-6">
    <skos:altLabel xml:lang="de">Umzugsmotivation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13845-2">
    <skos:altLabel xml:lang="de">Wohnungsnot</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13845-2">
    <skos:altLabel xml:lang="de">Wohnungsmangel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13845-2">
    <skos:altLabel xml:lang="de">Wohnungsleerstand</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13845-2">
    <skos:altLabel xml:lang="de">Wohnungshalde</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13845-2">
    <skos:altLabel xml:lang="de">Wohnungsbedarf</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="../stw">
    <cc:attributionName xml:lang="en">German National Library of Economics (ZBW) / Leibniz Information Center for Economics</cc:attributionName>
  </rdf:Description>
  <rdf:Description rdf:about="../stw">
    <cc:attributionName xml:lang="de">ZBW - Deutsche Zentralbibliothek für Wirtschaftswissenschaften / Leibniz-Informationszentrum Wirtschaft</cc:attributionName>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12511-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19774-4">
    <skos:prefLabel xml:lang="en">Non-marital birth</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19774-4">
    <skos:prefLabel xml:lang="de">Nichteheliche Kinder</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19698-1">
    <skos:altLabel xml:lang="de">Präsidentenwahl</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11441-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14051-6">
    <skos:altLabel xml:lang="de">Produktverpackung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14051-6">
    <skos:altLabel xml:lang="de">Packstoff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19474-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18762-2">
    <skos:prefLabel xml:lang="en">Data processing industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18762-2">
    <skos:prefLabel xml:lang="de">Datenverarbeitungswirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15998-2">
    <skos:broader rdf:resource="thsys/73361"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15998-2">
    <skos:broader rdf:resource="descriptor/16017-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71059">
    <rdfs:label xml:lang="en">Working time</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71059">
    <rdfs:label xml:lang="de">Arbeitszeit</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11415-2">
    <skos:narrower rdf:resource="descriptor/11434-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11415-2">
    <skos:narrower rdf:resource="descriptor/10776-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11415-2">
    <skos:narrower rdf:resource="descriptor/11411-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19290-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18373-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13460-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12574-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19288-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13470-5">
    <skos:altLabel xml:lang="en">Road show</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13470-5">
    <skos:altLabel xml:lang="de">Marktwesen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11159-1">
    <skos:broader rdf:resource="thsys/71062"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11159-1">
    <skos:broader rdf:resource="descriptor/19215-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11159-1">
    <skos:broader rdf:resource="descriptor/11219-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12969-3">
    <skos:broader rdf:resource="thsys/71018"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12969-3">
    <skos:broader rdf:resource="descriptor/12955-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18522-1">
    <skos:related rdf:resource="descriptor/18395-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18522-1">
    <skos:related rdf:resource="descriptor/12918-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18522-1">
    <skos:related rdf:resource="descriptor/10301-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11916-1">
    <skos:related rdf:resource="descriptor/15902-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12935-6">
    <skos:broader rdf:resource="thsys/71026"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12935-6">
    <skos:broader rdf:resource="descriptor/12940-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10589-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20449-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70921">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.13.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16436-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11327-6">
    <skos:related rdf:resource="descriptor/11782-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11327-6">
    <skos:related rdf:resource="descriptor/10622-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11327-6">
    <skos:related rdf:resource="descriptor/15089-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11327-6">
    <skos:related rdf:resource="descriptor/11322-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11327-6">
    <skos:related rdf:resource="descriptor/11298-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18836-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18238-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19787-2">
    <skos:related rdf:resource="descriptor/18892-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13217-3">
    <skos:narrower rdf:resource="descriptor/19059-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13217-3">
    <skos:narrower rdf:resource="descriptor/18514-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12735-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14740-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12420-4">
    <skos:altLabel xml:lang="de">Festwertverfahren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12420-4">
    <skos:altLabel xml:lang="de">Festbewertung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13347-4">
    <skos:altLabel xml:lang="de">Medienindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13347-4">
    <skos:altLabel xml:lang="de">Kommunikationsindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18602-3">
    <skos:related rdf:resource="descriptor/18603-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18602-3">
    <skos:related rdf:resource="descriptor/18601-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18602-3">
    <skos:related rdf:resource="descriptor/13533-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18318-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20018-4">
    <skos:prefLabel xml:lang="en">Private equity</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20018-4">
    <skos:prefLabel xml:lang="de">Private Equity</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12775-2">
    <skos:narrower rdf:resource="descriptor/19835-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19782-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12094-5">
    <skos:altLabel xml:lang="en">Reengineering</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12094-5">
    <skos:altLabel xml:lang="en">Business reengineering</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12094-5">
    <skos:altLabel xml:lang="en">Business redesign</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12094-5">
    <skos:altLabel xml:lang="en">Business process reengineering</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12094-5">
    <skos:altLabel xml:lang="en">Business process redesign</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12094-5">
    <skos:altLabel xml:lang="de">Unternehmensrestrukturierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12094-5">
    <skos:altLabel xml:lang="de">Reorganisationsprozess</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12094-5">
    <skos:altLabel xml:lang="de">Reorganisationsprozeß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14017-6">
    <skos:related rdf:resource="descriptor/10032-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14017-6">
    <skos:related rdf:resource="descriptor/18871-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14017-6">
    <skos:related rdf:resource="descriptor/13964-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14017-6">
    <skos:related rdf:resource="descriptor/18880-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14017-6">
    <skos:related rdf:resource="descriptor/18780-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14118-0">
    <skos:prefLabel xml:lang="en">Ornamental plant</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14118-0">
    <skos:prefLabel xml:lang="de">Zierpflanze</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11487-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70964">
    <skos:narrower rdf:resource="descriptor/13141-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70964">
    <skos:narrower rdf:resource="descriptor/13339-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70964">
    <skos:narrower rdf:resource="descriptor/13903-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70964">
    <skos:narrower rdf:resource="descriptor/13080-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70964">
    <skos:narrower rdf:resource="descriptor/18927-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70964">
    <skos:narrower rdf:resource="descriptor/13052-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70964">
    <skos:narrower rdf:resource="descriptor/13185-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70964">
    <skos:narrower rdf:resource="descriptor/13109-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70964">
    <skos:narrower rdf:resource="descriptor/13316-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70964">
    <skos:narrower rdf:resource="descriptor/13940-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70964">
    <skos:narrower rdf:resource="descriptor/13077-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70964">
    <skos:narrower rdf:resource="descriptor/13181-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11811-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70828">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16353-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13981-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14957-0">
    <skos:broader rdf:resource="thsys/70498"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14957-0">
    <skos:broader rdf:resource="descriptor/14958-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14766-0">
    <skos:related rdf:resource="descriptor/13213-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14766-0">
    <skos:related rdf:resource="descriptor/14032-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14766-0">
    <skos:related rdf:resource="descriptor/13010-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14766-0">
    <skos:related rdf:resource="descriptor/14025-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13680-1">
    <skos:prefLabel xml:lang="en">ISDN</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13680-1">
    <skos:prefLabel xml:lang="de">ISDN</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16406-6">
    <skos:broader rdf:resource="thsys/73383"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19151-5">
    <skos:broader rdf:resource="thsys/70383"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18613-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15619-1">
    <skos:related rdf:resource="descriptor/18766-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12643-5">
    <skos:altLabel xml:lang="en">Just in sequence</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12643-5">
    <skos:altLabel xml:lang="en">JIT (Just in Time)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12643-5">
    <skos:altLabel xml:lang="de">Kanban</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19361-1">
    <skos:scopeNote xml:lang="de">Vorwiegend ökonomische, aber auch politische und soziologische Analysen zu entwicklungsbezogenen Fragestellungen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10249-5">
    <skos:altLabel xml:lang="de">Makrotheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10249-5">
    <skos:altLabel xml:lang="de">Makroökonomisches Modell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10249-5">
    <skos:altLabel xml:lang="de">Makroökonomie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10249-5">
    <skos:altLabel xml:lang="de">Makromodell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19175-5">
    <skos:prefLabel xml:lang="en">COMESA countries</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19175-5">
    <skos:prefLabel xml:lang="de">COMESA-Staaten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10527-3">
    <skos:prefLabel xml:lang="en">Immiserizing growth</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10527-3">
    <skos:prefLabel xml:lang="de">Immiserizing Growth</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71106">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19422-0">
    <skos:scopeNote xml:lang="de">Literatur zu einzelnen Verbrauchsteuern.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11206-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19811-3">
    <skos:prefLabel xml:lang="en">Biometry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19811-3">
    <skos:prefLabel xml:lang="de">Biometrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15743-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15917-0">
    <skos:broader rdf:resource="thsys/70142"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15917-0">
    <skos:broader rdf:resource="descriptor/19643-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12005-2">
    <skos:related rdf:resource="descriptor/18808-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12005-2">
    <skos:related rdf:resource="descriptor/13413-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12005-2">
    <skos:related rdf:resource="descriptor/16297-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12956-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12933-3">
    <skos:altLabel xml:lang="de">Marktfruchtbau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12933-3">
    <skos:altLabel xml:lang="de">Feldfruchtbau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13054-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19058-2">
    <skos:related rdf:resource="descriptor/14366-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11342-3">
    <skos:broader rdf:resource="thsys/70278"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11342-3">
    <skos:broader rdf:resource="descriptor/11345-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17636-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14378-2">
    <skos:related rdf:resource="descriptor/18402-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14378-2">
    <skos:related rdf:resource="descriptor/14268-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14378-2">
    <skos:related rdf:resource="descriptor/13028-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11774-2">
    <skos:related rdf:resource="descriptor/11326-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11774-2">
    <skos:related rdf:resource="descriptor/12275-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11774-2">
    <skos:related rdf:resource="descriptor/11736-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11320-6">
    <skos:altLabel xml:lang="de">Angestelltengehalt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12437-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12313-5">
    <skos:related rdf:resource="descriptor/12452-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12313-5">
    <skos:related rdf:resource="descriptor/24666-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12313-5">
    <skos:related rdf:resource="descriptor/12326-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15956-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15982-3">
    <skos:narrower rdf:resource="descriptor/19859-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15982-3">
    <skos:narrower rdf:resource="descriptor/19214-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15982-3">
    <skos:narrower rdf:resource="descriptor/16150-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15982-3">
    <skos:narrower rdf:resource="descriptor/18760-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15982-3">
    <skos:narrower rdf:resource="descriptor/18525-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15982-3">
    <skos:narrower rdf:resource="descriptor/16131-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15982-3">
    <skos:narrower rdf:resource="descriptor/24622-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18343-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11783-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11904-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14212-1">
    <skos:broader rdf:resource="thsys/70649"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14212-1">
    <skos:broader rdf:resource="descriptor/14218-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14212-1">
    <skos:broader rdf:resource="descriptor/14216-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14212-1">
    <skos:broader rdf:resource="descriptor/14664-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19469-2">
    <skos:related rdf:resource="descriptor/11992-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19469-2">
    <skos:related rdf:resource="descriptor/11983-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19469-2">
    <skos:related rdf:resource="descriptor/11969-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19469-2">
    <skos:related rdf:resource="descriptor/16656-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10056-2">
    <skos:related rdf:resource="descriptor/19202-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10056-2">
    <skos:related rdf:resource="descriptor/10225-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14744-3">
    <skos:prefLabel xml:lang="en">Pigment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14744-3">
    <skos:prefLabel xml:lang="de">Farbmittel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18444-2">
    <skos:altLabel xml:lang="en">Obesity</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18444-2">
    <skos:altLabel xml:lang="de">Übergewicht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18444-2">
    <skos:altLabel xml:lang="de">Gesundheitsgefährdung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10774-0">
    <skos:broader rdf:resource="thsys/70904"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10774-0">
    <skos:broader rdf:resource="thsys/70305"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10774-0">
    <skos:broader rdf:resource="descriptor/13413-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14514-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12299-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73363">
    <skos:narrower rdf:resource="descriptor/16523-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73363">
    <skos:narrower rdf:resource="descriptor/18366-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73363">
    <skos:narrower rdf:resource="descriptor/16024-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73363">
    <skos:narrower rdf:resource="descriptor/16522-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73363">
    <skos:narrower rdf:resource="descriptor/18817-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73363">
    <skos:narrower rdf:resource="descriptor/11560-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73363">
    <skos:narrower rdf:resource="descriptor/13998-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73363">
    <skos:narrower rdf:resource="descriptor/16460-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73363">
    <skos:narrower rdf:resource="descriptor/18814-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73363">
    <skos:narrower rdf:resource="descriptor/19725-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73363">
    <skos:narrower rdf:resource="descriptor/18816-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73363">
    <skos:narrower rdf:resource="descriptor/16684-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73363">
    <skos:narrower rdf:resource="descriptor/16458-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73363">
    <skos:narrower rdf:resource="descriptor/19720-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73363">
    <skos:narrower rdf:resource="descriptor/18815-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73363">
    <skos:narrower rdf:resource="descriptor/16271-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73363">
    <skos:narrower rdf:resource="descriptor/19724-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73363">
    <skos:narrower rdf:resource="descriptor/11582-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73363">
    <skos:narrower rdf:resource="descriptor/19723-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73363">
    <skos:narrower rdf:resource="descriptor/16252-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73363">
    <skos:narrower rdf:resource="descriptor/13412-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73363">
    <skos:narrower rdf:resource="descriptor/16326-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73363">
    <skos:narrower rdf:resource="descriptor/16325-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73363">
    <skos:narrower rdf:resource="descriptor/16455-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16805-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14408-5">
    <skos:prefLabel xml:lang="en">Heat pump</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14408-5">
    <skos:prefLabel xml:lang="de">Wärmepumpe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15365-6">
    <zbwext:useInsteadNote rdf:nodeID="id149"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24662-6">
    <skos:scopeNote xml:lang="de">Im Rahmen des Essential-Facility-Konzepts kann eine Wettbewerbsbehörde ein marktbeherrschendes Unternehmen verpflichten, strategisches Anlagevermögen mit Wettbewerbern auf nachgelagerten Märkten zu teilen. Beispiele: Verkehrsinfrastruktur oder Telekommunikationsnetze.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13224-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12591-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19671-0">
    <skos:broader rdf:resource="thsys/70893"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19671-0">
    <skos:broader rdf:resource="thsys/70187"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15330-4">
    <skos:prefLabel xml:lang="en">Correlation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15330-4">
    <skos:prefLabel xml:lang="de">Korrelation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18828-5">
    <skos:altLabel xml:lang="en">Theory of economic regulation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18828-5">
    <skos:altLabel xml:lang="de">Regulierungstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18828-5">
    <skos:altLabel xml:lang="de">Regulierungsökonomik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18828-5">
    <skos:altLabel xml:lang="de">Ökonomische Theorie der Regulierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18828-5">
    <skos:altLabel xml:lang="de">Ökonomische Regulierungstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14079-5">
    <skos:related rdf:resource="descriptor/12951-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12469-2">
    <skos:broader rdf:resource="thsys/70578"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12469-2">
    <skos:broader rdf:resource="descriptor/19108-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12631-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18947-4">
    <skos:related rdf:resource="descriptor/18649-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18947-4">
    <skos:related rdf:resource="descriptor/13962-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18947-4">
    <skos:related rdf:resource="descriptor/13890-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16743-2">
    <skos:altLabel xml:lang="de">Schwedische Schule</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18218-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13735-2">
    <skos:related rdf:resource="descriptor/10210-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12615-3">
    <skos:related rdf:resource="descriptor/12620-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12615-3">
    <skos:related rdf:resource="descriptor/12612-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12615-3">
    <skos:related rdf:resource="descriptor/12618-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12615-3">
    <skos:related rdf:resource="descriptor/11388-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12615-3">
    <skos:related rdf:resource="descriptor/11269-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12615-3">
    <skos:related rdf:resource="descriptor/12610-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12615-3">
    <skos:related rdf:resource="descriptor/12609-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12615-3">
    <skos:related rdf:resource="descriptor/12617-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12372-3">
    <skos:altLabel xml:lang="de">Steuerrechtliche Gewinnermittlung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12372-3">
    <skos:altLabel xml:lang="de">Steuerliche Gewinnermittlung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12372-3">
    <skos:altLabel xml:lang="de">Handelsrechtliche Gewinnermittlung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12372-3">
    <skos:altLabel xml:lang="de">Gewinnrealisierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16806-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11504-3">
    <skos:narrower rdf:resource="descriptor/11510-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11504-3">
    <skos:narrower rdf:resource="descriptor/18684-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14177-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13591-0">
    <skos:prefLabel xml:lang="en">Travel time</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13591-0">
    <skos:prefLabel xml:lang="de">Transportzeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16639-4">
    <skos:broader rdf:resource="thsys/73395"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16639-4">
    <skos:broader rdf:resource="thsys/70504"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18585-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11723-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18151-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11986-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13270-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19486-2">
    <skos:related rdf:resource="descriptor/18267-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19486-2">
    <skos:related rdf:resource="descriptor/14640-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13745-6">
    <skos:prefLabel xml:lang="en">Zero bond</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13745-6">
    <skos:prefLabel xml:lang="de">Zero Bond</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13324-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13820-4">
    <skos:broader rdf:resource="thsys/73334"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13820-4">
    <skos:broader rdf:resource="thsys/70892"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13820-4">
    <skos:broader rdf:resource="descriptor/18736-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13351-6">
    <skos:prefLabel xml:lang="en">Library</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13351-6">
    <skos:prefLabel xml:lang="de">Bibliothek</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13465-5">
    <skos:broader rdf:resource="thsys/72181"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13465-5">
    <skos:broader rdf:resource="thsys/70963"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13465-5">
    <skos:broader rdf:resource="descriptor/15076-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12705-2">
    <skos:related rdf:resource="descriptor/13817-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12705-2">
    <skos:related rdf:resource="descriptor/12065-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12705-2">
    <skos:related rdf:resource="descriptor/10015-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12705-2">
    <skos:related rdf:resource="descriptor/14336-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12705-2">
    <skos:related rdf:resource="descriptor/12699-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12705-2">
    <skos:related rdf:resource="descriptor/12711-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12705-2">
    <skos:related rdf:resource="descriptor/12750-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12705-2">
    <skos:related rdf:resource="descriptor/19074-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12705-2">
    <skos:related rdf:resource="descriptor/12914-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17988-2">
    <skos:broader rdf:resource="thsys/70307"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18421-0">
    <skos:broader rdf:resource="thsys/70997"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18421-0">
    <skos:broader rdf:resource="descriptor/10470-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13832-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16594-0">
    <zbwext:useInsteadNote rdf:nodeID="id104"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13117-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10110-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14646-3">
    <skos:altLabel xml:lang="de">Phosphorsäure</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14646-3">
    <skos:altLabel xml:lang="de">Phosphoroxid</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14646-3">
    <skos:altLabel xml:lang="de">Phosphid</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14646-3">
    <skos:altLabel xml:lang="de">Naturphosphat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19073-6">
    <skos:prefLabel xml:lang="en">Theory</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19073-6">
    <skos:prefLabel xml:lang="de">Theorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20368-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16052-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14673-0">
    <skos:broader rdf:resource="thsys/70556"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14673-0">
    <skos:broader rdf:resource="descriptor/14683-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12848-1">
    <skos:related rdf:resource="descriptor/11679-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12848-1">
    <skos:related rdf:resource="descriptor/11678-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12848-1">
    <skos:related rdf:resource="descriptor/12876-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12848-1">
    <skos:related rdf:resource="descriptor/19410-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12848-1">
    <skos:related rdf:resource="descriptor/12307-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13029-4">
    <skos:narrower rdf:resource="descriptor/13064-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13029-4">
    <skos:narrower rdf:resource="descriptor/18404-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13029-4">
    <skos:narrower rdf:resource="descriptor/18403-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13029-4">
    <skos:narrower rdf:resource="descriptor/13027-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13419-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11402-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19400-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19154-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10541-2">
    <skos:broader rdf:resource="thsys/71086"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10541-2">
    <skos:broader rdf:resource="thsys/70406"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10541-2">
    <skos:broader rdf:resource="descriptor/10414-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10541-2">
    <skos:broader rdf:resource="descriptor/10524-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19494-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16973-6">
    <skos:prefLabel xml:lang="en">Schwyz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16973-6">
    <skos:prefLabel xml:lang="de">Schwyz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70869">
    <rdfs:label xml:lang="de">Finanzverwaltung und öffentliche Finanzwirtschaft</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19229-1">
    <skos:altLabel xml:lang="en">NCM</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19229-1">
    <skos:altLabel xml:lang="en">Lucas supply function</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19229-1">
    <skos:altLabel xml:lang="en">Lucas critique</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19229-1">
    <skos:altLabel xml:lang="de">Neuklassische Makroökonomik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19229-1">
    <skos:altLabel xml:lang="de">Neue klassische Makroökonomie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13665-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19508-4">
    <skos:scopeNote xml:lang="de">Vorwiegend Maßnahmen in den USA zur Koppelung der Zahlung von Sozialhilfe an die Erbringung von Arbeitsleistungen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70988">
    <skos:prefLabel xml:lang="de">W.06.01.10  Leder- und Schuhindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11305-2">
    <skos:related rdf:resource="descriptor/12620-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11305-2">
    <skos:related rdf:resource="descriptor/19325-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11305-2">
    <skos:related rdf:resource="descriptor/16638-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11305-2">
    <skos:related rdf:resource="descriptor/19509-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11305-2">
    <skos:related rdf:resource="descriptor/12603-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11305-2">
    <skos:related rdf:resource="descriptor/12577-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11305-2">
    <skos:related rdf:resource="descriptor/15837-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11305-2">
    <skos:related rdf:resource="descriptor/12602-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11305-2">
    <skos:related rdf:resource="descriptor/11309-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11305-2">
    <skos:related rdf:resource="descriptor/12609-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11305-2">
    <skos:related rdf:resource="descriptor/11180-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12433-2">
    <skos:related rdf:resource="descriptor/12404-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70856">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.23</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18377-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11217-6">
    <skos:altLabel xml:lang="en">Forced labor</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11217-6">
    <skos:altLabel xml:lang="de">Arbeitszwang</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11217-6">
    <skos:altLabel xml:lang="de">Arbeitspflicht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19801-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14267-4">
    <skos:broader rdf:resource="thsys/70358"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18351-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10812-4">
    <skos:related rdf:resource="descriptor/18694-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10812-4">
    <skos:related rdf:resource="descriptor/10805-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10812-4">
    <skos:related rdf:resource="descriptor/10804-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10812-4">
    <skos:related rdf:resource="descriptor/10797-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11951-6">
    <skos:altLabel xml:lang="en">Spatial research</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11951-6">
    <skos:altLabel xml:lang="en">Regional research</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11951-6">
    <skos:altLabel xml:lang="en">Regional analysis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11951-6">
    <skos:altLabel xml:lang="de">Regionalwissenschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11951-6">
    <skos:altLabel xml:lang="de">Regionale Strukturforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11951-6">
    <skos:altLabel xml:lang="de">Regionalanalyse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11951-6">
    <skos:altLabel xml:lang="de">Regional- und Raumforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11951-6">
    <skos:altLabel xml:lang="de">Raumforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11951-6">
    <skos:altLabel xml:lang="de">Empirische Regionalforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16362-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19573-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16210-2">
    <skos:altLabel xml:lang="de">Utopischer Sozialismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16210-2">
    <skos:altLabel xml:lang="de">Syndikalismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16210-2">
    <skos:altLabel xml:lang="de">Staatssozialismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16210-2">
    <skos:altLabel xml:lang="de">Kathedersozialismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16210-2">
    <skos:altLabel xml:lang="de">Gildensozialismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16210-2">
    <skos:altLabel xml:lang="de">Frühsozialismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16210-2">
    <skos:altLabel xml:lang="de">Agrarsozialismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18023-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14032-3">
    <skos:altLabel xml:lang="de">Souvenirartikel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14032-3">
    <skos:altLabel xml:lang="de">Hausrat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14032-3">
    <skos:altLabel xml:lang="de">Geschenkartikel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14032-3">
    <skos:altLabel xml:lang="de">Andenkenartikel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11217-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18553-4">
    <skos:scopeNote xml:lang="de">Optimierung des Sortiments, auch durch Kooperation von Hersteller und Händler.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18596-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19225-2">
    <skos:broader rdf:resource="thsys/70355"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19225-2">
    <skos:broader rdf:resource="thsys/70178"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19225-2">
    <skos:broader rdf:resource="descriptor/10213-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10037-6">
    <skos:narrower rdf:resource="descriptor/10361-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10037-6">
    <skos:narrower rdf:resource="descriptor/19767-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13414-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14222-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16003-0">
    <skos:broader rdf:resource="thsys/73364"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16003-0">
    <skos:broader rdf:resource="thsys/73332"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16003-0">
    <skos:broader rdf:resource="descriptor/16007-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11685-1">
    <skos:prefLabel xml:lang="en">Excess burden</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11685-1">
    <skos:prefLabel xml:lang="de">Excess Burden</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10701-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19173-2">
    <skos:broader rdf:resource="thsys/70307"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19173-2">
    <skos:broader rdf:resource="descriptor/10492-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18694-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10229-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10714-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10439-0">
    <skos:broader rdf:resource="thsys/73372"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10439-0">
    <skos:broader rdf:resource="thsys/70406"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10439-0">
    <skos:broader rdf:resource="descriptor/10987-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70073">
    <skos:narrower rdf:resource="descriptor/17181-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70073">
    <skos:narrower rdf:resource="descriptor/16828-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70073">
    <skos:narrower rdf:resource="descriptor/18224-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70073">
    <skos:narrower rdf:resource="descriptor/18225-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70073">
    <skos:narrower rdf:resource="descriptor/16826-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70073">
    <skos:narrower rdf:resource="descriptor/19084-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15892-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10217-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18394-5">
    <skos:related rdf:resource="descriptor/14317-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18394-5">
    <skos:related rdf:resource="descriptor/19182-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18394-5">
    <skos:related rdf:resource="descriptor/12984-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18394-5">
    <skos:related rdf:resource="descriptor/12983-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19315-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12818-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18079-6">
    <skos:prefLabel xml:lang="en">CFC (Chloro Fluoro Carbons)</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18079-6">
    <skos:prefLabel xml:lang="de">FCKW</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19490-4">
    <skos:related rdf:resource="descriptor/10334-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19490-4">
    <skos:related rdf:resource="descriptor/19200-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19490-4">
    <skos:related rdf:resource="descriptor/15753-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19490-4">
    <skos:related rdf:resource="descriptor/18129-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19490-4">
    <skos:related rdf:resource="descriptor/10315-4"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id376">
    <rdfs:seeAlso rdf:resource="descriptor/11910-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17923-2">
    <skos:prefLabel xml:lang="en">Colombia</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17923-2">
    <skos:prefLabel xml:lang="de">Kolumbien</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11832-0">
    <skos:prefLabel xml:lang="en">Transport planning</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11832-0">
    <skos:prefLabel xml:lang="de">Verkehrsplanung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10814-0">
    <skos:broader rdf:resource="thsys/71089"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10814-0">
    <skos:broader rdf:resource="descriptor/10813-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18235-4">
    <skos:broader rdf:resource="thsys/70538"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18235-4">
    <skos:broader rdf:resource="thsys/70227"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18235-4">
    <skos:broader rdf:resource="descriptor/14184-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18235-4">
    <skos:broader rdf:resource="descriptor/15751-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16489-4">
    <skos:altLabel xml:lang="de">Nationales Interesse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16489-4">
    <skos:altLabel xml:lang="de">Außenpolitische Strategie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19122-5">
    <skos:related rdf:resource="descriptor/20468-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11434-5">
    <skos:broader rdf:resource="thsys/70114"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11434-5">
    <skos:broader rdf:resource="thsys/70055"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11434-5">
    <skos:broader rdf:resource="descriptor/11415-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13588-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15767-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19254-2">
    <skos:altLabel xml:lang="en">Menu costs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70101">
    <skos:prefLabel xml:lang="de">V.10  Industrieökonomik und Wettbewerb</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13729-4">
    <skos:altLabel xml:lang="en">Insider trade</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13729-4">
    <skos:altLabel xml:lang="en">Insider information</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13729-4">
    <skos:altLabel xml:lang="en">Inside information</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13729-4">
    <skos:altLabel xml:lang="de">Insidergeschäft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11490-2">
    <skos:narrower rdf:resource="descriptor/20104-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11490-2">
    <skos:narrower rdf:resource="descriptor/11483-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11490-2">
    <skos:narrower rdf:resource="descriptor/11478-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15918-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14750-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18405-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18481-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13302-5">
    <skos:related rdf:resource="descriptor/18754-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10947-2">
    <skos:altLabel xml:lang="de">Nachahmungswettbewerb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13845-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15695-6">
    <skos:altLabel xml:lang="en">Local history</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15695-6">
    <skos:altLabel xml:lang="de">Stadtteilgeschichte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15695-6">
    <skos:altLabel xml:lang="de">Gemeindegeschichte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19747-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15946-0">
    <skos:altLabel xml:lang="en">Infant mortality</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15946-0">
    <skos:altLabel xml:lang="de">Sterblichkeit (Kinder)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15946-0">
    <skos:altLabel xml:lang="de">Säuglingssterblichkeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19605-6">
    <skos:prefLabel xml:lang="en">Duration analysis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19605-6">
    <skos:prefLabel xml:lang="de">Statistische Bestandsanalyse</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11596-0">
    <skos:broader rdf:resource="thsys/71082"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11596-0">
    <skos:broader rdf:resource="thsys/71081"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11596-0">
    <skos:broader rdf:resource="descriptor/11540-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14204-0">
    <skos:altLabel xml:lang="de">Buntmetallerz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13368-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13277-6">
    <skos:scopeNote xml:lang="de">Verwendung des gesprochenen Wortes zur akustischen Datenein- und ausgabe.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19602-5">
    <skos:broader rdf:resource="thsys/72181"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19602-5">
    <skos:broader rdf:resource="thsys/70055"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19602-5">
    <skos:broader rdf:resource="descriptor/15076-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70986">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.06.01.12</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19594-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19048-5">
    <skos:altLabel xml:lang="de">Gneis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12299-1">
    <skos:related rdf:resource="descriptor/13743-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12299-1">
    <skos:related rdf:resource="descriptor/16080-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12299-1">
    <skos:related rdf:resource="descriptor/13742-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12299-1">
    <skos:related rdf:resource="descriptor/13769-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12299-1">
    <skos:related rdf:resource="descriptor/12265-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12299-1">
    <skos:related rdf:resource="descriptor/18682-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12299-1">
    <skos:related rdf:resource="descriptor/18681-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12299-1">
    <skos:related rdf:resource="descriptor/12230-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12299-1">
    <skos:related rdf:resource="descriptor/12399-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12299-1">
    <skos:related rdf:resource="descriptor/18670-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12299-1">
    <skos:related rdf:resource="descriptor/13715-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13218-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13170-3">
    <skos:related rdf:resource="descriptor/12703-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13170-3">
    <skos:related rdf:resource="descriptor/14028-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13170-3">
    <skos:related rdf:resource="descriptor/14020-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18880-3">
    <zbwext:useInsteadNote rdf:nodeID="id368"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15770-4">
    <skos:related rdf:resource="descriptor/12928-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15770-4">
    <skos:related rdf:resource="descriptor/15769-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15770-4">
    <skos:related rdf:resource="descriptor/15768-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15770-4">
    <skos:related rdf:resource="descriptor/19483-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15770-4">
    <skos:related rdf:resource="descriptor/15767-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15770-4">
    <skos:related rdf:resource="descriptor/18165-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15770-4">
    <skos:related rdf:resource="descriptor/15763-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15770-4">
    <skos:related rdf:resource="descriptor/15759-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15770-4">
    <skos:related rdf:resource="descriptor/16638-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15770-4">
    <skos:related rdf:resource="descriptor/20722-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19380-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12259-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16595-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15790-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18577-4">
    <skos:prefLabel xml:lang="en">Cycling</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18577-4">
    <skos:prefLabel xml:lang="de">Fahrradverkehr</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10978-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12135-3">
    <skos:broader rdf:resource="thsys/70562"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12135-3">
    <skos:broader rdf:resource="descriptor/12136-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18435-3">
    <skos:altLabel xml:lang="de">Webtechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18435-3">
    <skos:altLabel xml:lang="de">Nähtechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18435-3">
    <skos:altLabel xml:lang="de">Bekleidungstechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18271-0">
    <skos:prefLabel xml:lang="en">Payment card</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18271-0">
    <skos:prefLabel xml:lang="de">Chipkarte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12088-0">
    <skos:broader rdf:resource="thsys/70485"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18365-5">
    <skos:scopeNote xml:lang="de">Alle Richtlinien der Europäischen Union zur Vergabe öffentlicher Aufträge und zum öffentlichen Beschaffungswesen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10568-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14102-1">
    <skos:related rdf:resource="descriptor/12958-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14102-1">
    <skos:related rdf:resource="descriptor/14985-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10880-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14601-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18209-5">
    <skos:altLabel xml:lang="de">Wissenschaftliche Zeitschrift</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18209-5">
    <skos:altLabel xml:lang="de">Fachpresse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16287-2">
    <skos:prefLabel xml:lang="en">Formulation of political objectives</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16287-2">
    <skos:prefLabel xml:lang="de">Politische Willensbildung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16716-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11886-5">
    <skos:altLabel xml:lang="de">Regionale Wirtschaftsförderungsgesellschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11886-5">
    <skos:altLabel xml:lang="de">Regionale Entwicklungsinstitution</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73356">
    <skos:narrower rdf:resource="descriptor/18314-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73356">
    <skos:narrower rdf:resource="descriptor/16676-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73356">
    <skos:narrower rdf:resource="descriptor/19763-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73356">
    <skos:narrower rdf:resource="descriptor/19765-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73356">
    <skos:narrower rdf:resource="descriptor/19507-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73356">
    <skos:narrower rdf:resource="descriptor/15721-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73356">
    <skos:narrower rdf:resource="descriptor/16674-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73356">
    <skos:narrower rdf:resource="descriptor/19764-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73356">
    <skos:narrower rdf:resource="descriptor/19646-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73356">
    <skos:narrower rdf:resource="descriptor/18900-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73356">
    <skos:narrower rdf:resource="descriptor/16404-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73356">
    <skos:narrower rdf:resource="descriptor/16672-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73356">
    <skos:narrower rdf:resource="descriptor/16671-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73356">
    <skos:narrower rdf:resource="descriptor/19009-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73356">
    <skos:narrower rdf:resource="descriptor/16196-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73356">
    <skos:narrower rdf:resource="descriptor/16195-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73356">
    <skos:narrower rdf:resource="descriptor/19721-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73356">
    <skos:narrower rdf:resource="descriptor/16192-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73356">
    <skos:narrower rdf:resource="descriptor/16621-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73356">
    <skos:narrower rdf:resource="descriptor/15919-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73356">
    <skos:narrower rdf:resource="descriptor/19506-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73356">
    <skos:narrower rdf:resource="descriptor/16668-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73356">
    <skos:narrower rdf:resource="descriptor/15713-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73356">
    <skos:narrower rdf:resource="descriptor/15908-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73356">
    <skos:narrower rdf:resource="descriptor/19648-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73356">
    <skos:narrower rdf:resource="descriptor/10488-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73356">
    <skos:narrower rdf:resource="descriptor/16666-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70341">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19733-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18150-5">
    <skos:altLabel xml:lang="en">NRU (Natural rate of unemployment)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18150-5">
    <skos:altLabel xml:lang="en">Non-accelerating inflation rate of unemployment</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18150-5">
    <skos:altLabel xml:lang="de">Vollbeschäftigungsarbeitslosigkeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18150-5">
    <skos:altLabel xml:lang="de">Natürliche Arbeitslosenquote</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18150-5">
    <skos:altLabel xml:lang="de">NAIRU</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id372">
    <rdfs:seeAlso rdf:resource="descriptor/10513-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14985-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12075-2">
    <skos:altLabel xml:lang="en">Corporate image</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12075-2">
    <skos:altLabel xml:lang="de">Unternehmensimage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15733-3">
    <skos:altLabel xml:lang="de">Wasserverunreinigung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15733-3">
    <skos:altLabel xml:lang="de">Wasserverschmutzung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15733-3">
    <skos:altLabel xml:lang="de">Wassergefährdung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15733-3">
    <skos:altLabel xml:lang="de">Eutrophierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10676-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13141-3">
    <skos:broader rdf:resource="thsys/70987"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13141-3">
    <skos:broader rdf:resource="thsys/70964"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13141-3">
    <skos:broader rdf:resource="descriptor/12910-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10913-5">
    <skos:prefLabel xml:lang="en">Economic concentration</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10913-5">
    <skos:prefLabel xml:lang="de">Wirtschaftskonzentration</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13229-3">
    <skos:altLabel xml:lang="de">Photolabor</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13229-3">
    <skos:altLabel xml:lang="de">Photohandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13229-3">
    <skos:altLabel xml:lang="de">Photographenhandwerk</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13229-3">
    <skos:altLabel xml:lang="de">Fotolabor</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13229-3">
    <skos:altLabel xml:lang="de">Fotografenhandwerk</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13229-3">
    <skos:altLabel xml:lang="de">Fotogeschäft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13229-3">
    <skos:altLabel xml:lang="de">Fotofachhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13229-3">
    <skos:altLabel xml:lang="de">Fotofachgeschäft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19334-4">
    <skos:related rdf:resource="descriptor/10746-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18005-0">
    <skos:narrower rdf:resource="descriptor/18938-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17787-5">
    <skos:broader rdf:resource="thsys/70086"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17787-5">
    <skos:broader rdf:resource="descriptor/17791-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18864-1">
    <skos:scopeNote xml:lang="de">Alle Formen der kurz- oder langfristigen sowie der häuslichen, gemeinschaftlichen oder staatlichen Kinderbetreuung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14819-5">
    <skos:broader rdf:resource="thsys/70372"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14819-5">
    <skos:broader rdf:resource="descriptor/14822-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10602-1">
    <skos:altLabel xml:lang="de">Geschäftstätigkeit im Ausland</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10602-1">
    <skos:altLabel xml:lang="de">Auslandsauftrag</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16087-3">
    <skos:broader rdf:resource="thsys/73337"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16087-3">
    <skos:broader rdf:resource="thsys/71073"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16087-3">
    <skos:broader rdf:resource="descriptor/15971-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19823-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13903-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19629-6">
    <skos:broader rdf:resource="thsys/73351"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19629-6">
    <skos:broader rdf:resource="descriptor/15746-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14041-2">
    <skos:related rdf:resource="descriptor/12945-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14041-2">
    <skos:related rdf:resource="descriptor/14030-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14041-2">
    <skos:related rdf:resource="descriptor/14054-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70922">
    <rdfs:label xml:lang="en">Telecommunications</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70922">
    <rdfs:label xml:lang="de">Telekommunikation</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15974-2">
    <skos:broader rdf:resource="thsys/73381"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15974-2">
    <skos:broader rdf:resource="thsys/73361"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15974-2">
    <skos:broader rdf:resource="descriptor/13420-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19625-0">
    <skos:broader rdf:resource="thsys/73351"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19625-0">
    <skos:broader rdf:resource="thsys/73348"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19625-0">
    <skos:broader rdf:resource="thsys/70082"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19625-0">
    <skos:broader rdf:resource="descriptor/15746-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19625-0">
    <skos:broader rdf:resource="descriptor/18325-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73390">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.05.03</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11684-3">
    <skos:broader rdf:resource="thsys/71076"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11684-3">
    <skos:broader rdf:resource="descriptor/11694-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18718-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18691-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id195">
    <rdfs:seeAlso rdf:resource="descriptor/19133-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19214-0">
    <skos:broader rdf:resource="thsys/70274"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19214-0">
    <skos:broader rdf:resource="descriptor/19215-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19214-0">
    <skos:broader rdf:resource="descriptor/15982-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15678-6">
    <skos:altLabel xml:lang="de">Religionswissenschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14350-3">
    <skos:broader rdf:resource="thsys/70424"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14350-3">
    <skos:broader rdf:resource="descriptor/14357-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11358-2">
    <skos:broader rdf:resource="thsys/71056"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11358-2">
    <skos:broader rdf:resource="thsys/71055"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11358-2">
    <skos:broader rdf:resource="thsys/70843"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11358-2">
    <skos:broader rdf:resource="thsys/70244"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11358-2">
    <skos:broader rdf:resource="descriptor/11374-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19627-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10738-4">
    <skos:prefLabel xml:lang="en">French Franc</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10738-4">
    <skos:prefLabel xml:lang="de">Französischer Franc</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11939-3">
    <skos:related rdf:resource="descriptor/11910-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11939-3">
    <skos:related rdf:resource="descriptor/11938-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11825-4">
    <skos:related rdf:resource="descriptor/19680-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11825-4">
    <skos:related rdf:resource="descriptor/11044-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11825-4">
    <skos:related rdf:resource="descriptor/11043-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11825-4">
    <skos:related rdf:resource="descriptor/18073-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17675-2">
    <skos:prefLabel xml:lang="en">Seychelles</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17675-2">
    <skos:prefLabel xml:lang="de">Seychellen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11034-0">
    <skos:altLabel xml:lang="de">Zwangsbewirtschaftung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11034-0">
    <skos:altLabel xml:lang="de">Wirtschaftssicherstellungsgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11034-0">
    <skos:altLabel xml:lang="de">Sicherstellungsgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11034-0">
    <skos:altLabel xml:lang="de">Bundesleistungsgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11034-0">
    <skos:altLabel xml:lang="de">Bezugsscheinsystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11034-0">
    <skos:altLabel xml:lang="de">Bewirtschaftungsrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11034-0">
    <skos:altLabel xml:lang="de">Bewirtschaftung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14195-3">
    <skos:related rdf:resource="descriptor/14285-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14195-3">
    <skos:related rdf:resource="descriptor/12982-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14998-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16979-1">
    <skos:altLabel xml:lang="de">Vaud</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12908-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10410-3">
    <skos:broader rdf:resource="thsys/71099"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10410-3">
    <skos:broader rdf:resource="thsys/71087"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10410-3">
    <skos:broader rdf:resource="descriptor/10791-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10410-3">
    <skos:broader rdf:resource="descriptor/10993-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#definition">
    <rdfs:isDefinedBy rdf:resource="http://www.w3.org/2004/02/skos/core"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16158-6">
    <skos:related rdf:resource="descriptor/11569-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19271-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19808-6">
    <skos:related rdf:resource="descriptor/18661-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19808-6">
    <skos:related rdf:resource="descriptor/13683-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19808-6">
    <skos:related rdf:resource="descriptor/15415-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19808-6">
    <skos:related rdf:resource="descriptor/15611-3"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id302">
    <rdfs:seeAlso rdf:resource="descriptor/11494-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14702-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18950-1">
    <skos:prefLabel xml:lang="en">Preservation of library materials</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18950-1">
    <skos:prefLabel xml:lang="de">Bestandserhaltung in Bibliotheken</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10764-3">
    <skos:narrower rdf:resource="descriptor/10748-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10764-3">
    <skos:narrower rdf:resource="descriptor/10763-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10764-3">
    <skos:narrower rdf:resource="descriptor/10757-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10764-3">
    <skos:narrower rdf:resource="descriptor/19332-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10764-3">
    <skos:narrower rdf:resource="descriptor/10754-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10764-3">
    <skos:narrower rdf:resource="descriptor/10753-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10764-3">
    <skos:narrower rdf:resource="descriptor/11436-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10764-3">
    <skos:narrower rdf:resource="descriptor/10752-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10764-3">
    <skos:narrower rdf:resource="descriptor/19255-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10764-3">
    <skos:narrower rdf:resource="descriptor/10738-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10764-3">
    <skos:narrower rdf:resource="descriptor/19353-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10764-3">
    <skos:narrower rdf:resource="descriptor/10733-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10764-3">
    <skos:narrower rdf:resource="descriptor/10730-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18515-5">
    <skos:prefLabel xml:lang="en">Filling station and convenience store</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18515-5">
    <skos:prefLabel xml:lang="de">Tankstellen-Shop</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73368">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14596-6">
    <skos:prefLabel xml:lang="en">Agricultural chemical</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14596-6">
    <skos:prefLabel xml:lang="de">Agrochemikalie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13131-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id156">
    <rdfs:seeAlso rdf:resource="descriptor/16188-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11702-6">
    <skos:broader rdf:resource="thsys/71075"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11702-6">
    <skos:broader rdf:resource="thsys/70046"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11702-6">
    <skos:broader rdf:resource="descriptor/11704-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19501-4">
    <skos:prefLabel xml:lang="en">Earnings losses</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19501-4">
    <skos:prefLabel xml:lang="de">Verdienstausfall</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19802-4">
    <skos:prefLabel xml:lang="en">Data migration</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19802-4">
    <skos:prefLabel xml:lang="de">Datenmigration</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70066">
    <rdfs:label xml:lang="en">Textiles</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70066">
    <rdfs:label xml:lang="de">Textilien</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12785-6">
    <skos:related rdf:resource="descriptor/19555-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12785-6">
    <skos:related rdf:resource="descriptor/12665-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12785-6">
    <skos:related rdf:resource="descriptor/12786-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12785-6">
    <skos:related rdf:resource="descriptor/12787-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12785-6">
    <skos:related rdf:resource="descriptor/12033-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16387-5">
    <skos:prefLabel xml:lang="en">Labour movement</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16387-5">
    <skos:prefLabel xml:lang="de">Arbeiterbewegung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id285">
    <rdf:value xml:lang="de">Für Literatur über richtige Vortragstechniken und den Aufbau von Reden</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10243-3">
    <skos:altLabel xml:lang="en">Interindustry coefficient</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10243-3">
    <skos:altLabel xml:lang="en">Input-output table</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10243-3">
    <skos:altLabel xml:lang="de">Verflechtungsbilanz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10243-3">
    <skos:altLabel xml:lang="de">Leontief-System</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10243-3">
    <skos:altLabel xml:lang="de">Input-Output-Tabelle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10243-3">
    <skos:altLabel xml:lang="de">Input-Output-Rechnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10243-3">
    <skos:altLabel xml:lang="de">Input-Output-Modell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10243-3">
    <skos:altLabel xml:lang="de">Input-Output-Analyse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10243-3">
    <skos:altLabel xml:lang="de">Input-Output Analyse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10243-3">
    <skos:altLabel xml:lang="de">Inputkoeffizient</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18975-6">
    <skos:prefLabel xml:lang="en">Private debt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18975-6">
    <skos:prefLabel xml:lang="de">Private Verschuldung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id324">
    <rdf:value xml:lang="de">Für verarbeiteten Tabak</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12896-4">
    <skos:broader rdf:resource="thsys/71041"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12896-4">
    <skos:broader rdf:resource="descriptor/12903-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12408-1">
    <skos:prefLabel xml:lang="en">Accounting valuation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12408-1">
    <skos:prefLabel xml:lang="de">Bilanzielle Bewertung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19321-6">
    <skos:altLabel xml:lang="de">Liberalisierung des Finanzmarktes</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19321-6">
    <skos:altLabel xml:lang="de">Finanzmarktliberalisierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18264-4">
    <skos:broader rdf:resource="thsys/70536"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18264-4">
    <skos:broader rdf:resource="descriptor/14734-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15762-3">
    <skos:broader rdf:resource="thsys/73380"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15762-3">
    <skos:broader rdf:resource="thsys/70871"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15762-3">
    <skos:broader rdf:resource="thsys/70239"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15762-3">
    <skos:broader rdf:resource="descriptor/13413-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18891-5">
    <skos:narrower rdf:resource="descriptor/19527-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18891-5">
    <skos:narrower rdf:resource="descriptor/19551-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18891-5">
    <skos:narrower rdf:resource="descriptor/19539-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18891-5">
    <skos:narrower rdf:resource="descriptor/18894-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18891-5">
    <skos:narrower rdf:resource="descriptor/19786-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18981-4">
    <skos:prefLabel xml:lang="en">Maize production</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18981-4">
    <skos:prefLabel xml:lang="de">Maisanbau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19410-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10928-6">
    <skos:narrower rdf:resource="descriptor/18541-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14945-0">
    <skos:prefLabel xml:lang="en">Breakfast cereals</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14945-0">
    <skos:prefLabel xml:lang="de">Frühstückszerealien</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70962">
    <rdfs:label xml:lang="de">Handelssparten</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13756-1">
    <skos:related rdf:resource="descriptor/13693-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13756-1">
    <skos:related rdf:resource="descriptor/13686-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71013">
    <skos:narrower rdf:resource="descriptor/12992-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71013">
    <skos:narrower rdf:resource="descriptor/18368-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71013">
    <skos:narrower rdf:resource="descriptor/13003-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71013">
    <skos:narrower rdf:resource="descriptor/12976-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71013">
    <skos:narrower rdf:resource="descriptor/14035-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71013">
    <skos:narrower rdf:resource="descriptor/15628-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71013">
    <skos:narrower rdf:resource="descriptor/16001-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71013">
    <skos:narrower rdf:resource="descriptor/18367-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10721-0">
    <skos:related rdf:resource="descriptor/10774-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10721-0">
    <skos:related rdf:resource="descriptor/21101-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10721-0">
    <skos:related rdf:resource="descriptor/10764-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10721-0">
    <skos:related rdf:resource="descriptor/10676-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10854-2">
    <skos:narrower rdf:resource="descriptor/10870-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10854-2">
    <skos:narrower rdf:resource="descriptor/10865-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10854-2">
    <skos:narrower rdf:resource="descriptor/10850-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19190-2">
    <skos:related rdf:resource="descriptor/19189-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19190-2">
    <skos:related rdf:resource="descriptor/18981-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19190-2">
    <skos:related rdf:resource="descriptor/14093-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18437-6">
    <skos:prefLabel xml:lang="en">Intra-firm trade</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18437-6">
    <skos:prefLabel xml:lang="de">Firmeninterner Handel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12091-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16730-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18425-6">
    <skos:related rdf:resource="descriptor/10455-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18425-6">
    <skos:related rdf:resource="descriptor/16757-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18425-6">
    <skos:related rdf:resource="descriptor/15609-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18425-6">
    <skos:related rdf:resource="descriptor/19396-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18425-6">
    <skos:related rdf:resource="descriptor/18424-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16246-2">
    <skos:related rdf:resource="descriptor/15974-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16246-2">
    <skos:related rdf:resource="descriptor/16379-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16246-2">
    <skos:related rdf:resource="descriptor/16226-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16246-2">
    <skos:related rdf:resource="descriptor/16342-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16246-2">
    <skos:related rdf:resource="descriptor/16340-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16246-2">
    <skos:related rdf:resource="descriptor/11167-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16246-2">
    <skos:related rdf:resource="descriptor/16264-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16246-2">
    <skos:related rdf:resource="descriptor/16315-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16246-2">
    <skos:related rdf:resource="descriptor/16268-6"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id82">
    <rdf:value xml:lang="de">Für Produkte</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70989">
    <skos:prefLabel xml:lang="de">W.06.01.09  Kunststoff- und Gummiindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11452-3">
    <skos:broader rdf:resource="thsys/70306"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11452-3">
    <skos:broader rdf:resource="descriptor/10236-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13433-4">
    <skos:related rdf:resource="descriptor/15107-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13433-4">
    <skos:related rdf:resource="descriptor/19588-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13433-4">
    <skos:related rdf:resource="descriptor/12945-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13433-4">
    <skos:related rdf:resource="descriptor/19585-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11531-0">
    <skos:related rdf:resource="descriptor/19150-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11531-0">
    <skos:related rdf:resource="descriptor/10590-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11531-0">
    <skos:related rdf:resource="descriptor/11530-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11531-0">
    <skos:related rdf:resource="descriptor/11529-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10342-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18581-6">
    <skos:altLabel xml:lang="de">Verkehrsunfall im Straßenverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18581-6">
    <skos:altLabel xml:lang="de">Unfall im Straßenverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70070">
    <rdfs:label xml:lang="en">Grain</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70070">
    <rdfs:label xml:lang="de">Getreide</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18887-3">
    <skos:related rdf:resource="descriptor/15814-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18887-3">
    <skos:related rdf:resource="descriptor/15715-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19806-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18614-3">
    <skos:related rdf:resource="descriptor/10845-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18614-3">
    <skos:related rdf:resource="descriptor/18126-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18614-3">
    <skos:related rdf:resource="descriptor/13618-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18614-3">
    <skos:related rdf:resource="descriptor/10605-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11691-6">
    <skos:related rdf:resource="descriptor/10110-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11691-6">
    <skos:related rdf:resource="descriptor/11638-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11691-6">
    <skos:related rdf:resource="descriptor/16112-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11691-6">
    <skos:related rdf:resource="descriptor/11636-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11691-6">
    <skos:related rdf:resource="descriptor/16544-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19191-0">
    <skos:altLabel xml:lang="de">Agrarbetriebswirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19191-0">
    <skos:altLabel xml:lang="de">Agrarbetriebslehre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11285-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10804-3">
    <skos:narrower rdf:resource="descriptor/18686-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11724-3">
    <skos:scopeNote xml:lang="de">Von Adolph Wagner 1864 behauptete Gesetzmäßigkeit, dass der Anteil des öffentlichen Sektors am Sozialprodukt im Zeitverlauf steigt.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10828-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71077">
    <skos:broader rdf:resource="thsys/70144"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18749-1">
    <skos:prefLabel xml:lang="en">Residential real estate</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18749-1">
    <skos:prefLabel xml:lang="de">Wohnimmobilien</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18300-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19805-5">
    <skos:prefLabel xml:lang="en">Software ergonomics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19805-5">
    <skos:prefLabel xml:lang="de">Software-Ergonomie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71114">
    <skos:prefLabel xml:lang="de">V.03.04.01  Einkommens- und Vermögensverteilung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17943-3">
    <skos:prefLabel xml:lang="en">Papua New Guinea</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17943-3">
    <skos:prefLabel xml:lang="de">Papua-Neuguinea</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15327-0">
    <skos:related rdf:resource="descriptor/15328-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13829-0">
    <skos:editorialNote xml:lang="de">Möglichst spezifischer indexieren.</skos:editorialNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10542-0">
    <skos:broader rdf:resource="thsys/71086"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10542-0">
    <skos:broader rdf:resource="thsys/71070"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10542-0">
    <skos:broader rdf:resource="descriptor/10426-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10542-0">
    <skos:broader rdf:resource="descriptor/10525-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16692-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70014">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19720-6">
    <skos:related rdf:resource="descriptor/16024-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19720-6">
    <skos:related rdf:resource="descriptor/19725-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19720-6">
    <skos:related rdf:resource="descriptor/16458-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19720-6">
    <skos:related rdf:resource="descriptor/13412-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11052-5">
    <skos:altLabel xml:lang="de">Stimulation von Leistung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11052-5">
    <skos:altLabel xml:lang="de">Leistungsstimulation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/n">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16738-2">
    <skos:broader rdf:resource="thsys/70141"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16738-2">
    <skos:broader rdf:resource="descriptor/16739-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16738-2">
    <skos:broader rdf:resource="descriptor/16745-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16024-6">
    <skos:related rdf:resource="descriptor/16523-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16024-6">
    <skos:related rdf:resource="descriptor/16522-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16024-6">
    <skos:related rdf:resource="descriptor/19725-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16024-6">
    <skos:related rdf:resource="descriptor/19720-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16024-6">
    <skos:related rdf:resource="descriptor/13412-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16024-6">
    <skos:related rdf:resource="descriptor/16003-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11708-1">
    <skos:related rdf:resource="descriptor/11640-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11708-1">
    <skos:related rdf:resource="descriptor/11704-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11708-1">
    <skos:related rdf:resource="descriptor/19410-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15754-2">
    <skos:narrower rdf:resource="descriptor/18388-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15754-2">
    <skos:narrower rdf:resource="descriptor/11813-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18537-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13570-1">
    <skos:related rdf:resource="descriptor/13572-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13570-1">
    <skos:related rdf:resource="descriptor/13577-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13570-1">
    <skos:related rdf:resource="descriptor/13617-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70154">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15729-1">
    <skos:related rdf:resource="descriptor/15731-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15729-1">
    <skos:related rdf:resource="descriptor/15733-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15729-1">
    <skos:related rdf:resource="descriptor/13329-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15729-1">
    <skos:related rdf:resource="descriptor/15730-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25681-4">
    <skos:related rdf:resource="descriptor/16027-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25681-4">
    <skos:related rdf:resource="descriptor/15937-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15943-6">
    <skos:broader rdf:resource="thsys/70182"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15943-6">
    <skos:broader rdf:resource="descriptor/15941-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70423">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11191-5">
    <skos:related rdf:resource="descriptor/12575-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11191-5">
    <skos:related rdf:resource="descriptor/12572-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11191-5">
    <skos:related rdf:resource="descriptor/16142-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11191-5">
    <skos:related rdf:resource="descriptor/11182-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15475-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13278-4">
    <skos:altLabel xml:lang="en">Teleprocessing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13278-4">
    <skos:altLabel xml:lang="en">Data communication</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13278-4">
    <skos:altLabel xml:lang="de">Telematik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13278-4">
    <skos:altLabel xml:lang="de">Telekommunikationsdienst</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13278-4">
    <skos:altLabel xml:lang="de">Nachrichtenfernübermittlung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13278-4">
    <skos:altLabel xml:lang="de">Kommunikationsdienst</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13278-4">
    <skos:altLabel xml:lang="de">Fernmeldewesen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13278-4">
    <skos:altLabel xml:lang="de">DFÜ (Datenfernübertragung)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13278-4">
    <skos:altLabel xml:lang="de">Datenkommunikation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13278-4">
    <skos:altLabel xml:lang="de">Datenfernverarbeitung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13278-4">
    <skos:altLabel xml:lang="de">Datenfernübertragung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13278-4">
    <skos:altLabel xml:lang="de">Dateldienst</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18378-3">
    <skos:related rdf:resource="descriptor/12979-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18378-3">
    <skos:related rdf:resource="descriptor/18379-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18378-3">
    <skos:related rdf:resource="descriptor/14188-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12010-2">
    <skos:altLabel xml:lang="de">Bandinfrastruktur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16246-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10161-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11562-3">
    <skos:related rdf:resource="descriptor/18330-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16354-6">
    <skos:prefLabel xml:lang="en">Election</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16354-6">
    <skos:prefLabel xml:lang="de">Wahl</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12079-1">
    <skos:altLabel xml:lang="en">Executive information system</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12079-1">
    <skos:altLabel xml:lang="en">Decision support system</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12079-1">
    <skos:altLabel xml:lang="de">MUS (Managementunterstützungssystem)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12079-1">
    <skos:altLabel xml:lang="de">MIS (Management-Informationssystem)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12079-1">
    <skos:altLabel xml:lang="de">Managementunterstützungssystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12079-1">
    <skos:altLabel xml:lang="de">Führungsinformationssystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12079-1">
    <skos:altLabel xml:lang="de">Führungs-Informationssystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12079-1">
    <skos:altLabel xml:lang="de">FIS (Führungsinformationssystem)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12079-1">
    <skos:altLabel xml:lang="de">Entscheidungsunterstützungssystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12079-1">
    <skos:altLabel xml:lang="de">Computergestütztes Management</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13291-5">
    <skos:broader rdf:resource="thsys/70902"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13291-5">
    <skos:broader rdf:resource="descriptor/13283-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10088-3">
    <skos:broader rdf:resource="thsys/70274"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10088-3">
    <skos:broader rdf:resource="thsys/70057"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10088-3">
    <skos:broader rdf:resource="descriptor/10113-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11414-4">
    <skos:broader rdf:resource="thsys/70905"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11414-4">
    <skos:broader rdf:resource="thsys/70187"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18942-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70014">
    <rdfs:label xml:lang="de">Chemikalien</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12394-0">
    <skos:prefLabel xml:lang="en">Social accounting</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12394-0">
    <skos:prefLabel xml:lang="de">Sozialbilanz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14237-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11481-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11258-6">
    <skos:broader rdf:resource="thsys/70052"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11258-6">
    <skos:broader rdf:resource="descriptor/11262-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19247-6">
    <skos:broader rdf:resource="thsys/70140"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13974-5">
    <skos:related rdf:resource="descriptor/13398-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13974-5">
    <skos:related rdf:resource="descriptor/18301-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13974-5">
    <skos:related rdf:resource="descriptor/18144-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13974-5">
    <skos:related rdf:resource="descriptor/13975-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13974-5">
    <skos:related rdf:resource="descriptor/13949-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10287-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70872">
    <skos:narrower rdf:resource="descriptor/13993-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70872">
    <skos:narrower rdf:resource="descriptor/18829-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70872">
    <skos:narrower rdf:resource="descriptor/11722-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70872">
    <skos:narrower rdf:resource="descriptor/11559-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70872">
    <skos:narrower rdf:resource="descriptor/10458-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70872">
    <skos:narrower rdf:resource="descriptor/18803-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70872">
    <skos:narrower rdf:resource="descriptor/13303-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70872">
    <skos:narrower rdf:resource="descriptor/13358-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70872">
    <skos:narrower rdf:resource="descriptor/11318-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70872">
    <skos:narrower rdf:resource="descriptor/11289-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70872">
    <skos:narrower rdf:resource="thsys/70860"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70872">
    <skos:narrower rdf:resource="thsys/70871"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15026-0">
    <skos:prefLabel xml:lang="en">Analysis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15026-0">
    <skos:prefLabel xml:lang="de">Analysis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11813-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18493-3">
    <skos:prefLabel xml:lang="en">Ecological constructing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18493-3">
    <skos:prefLabel xml:lang="de">Bauökologie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19548-6">
    <skos:altLabel xml:lang="en">Venture management</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19548-6">
    <skos:altLabel xml:lang="en">Technology management</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19548-6">
    <skos:altLabel xml:lang="de">Technologiestrategie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19548-6">
    <skos:altLabel xml:lang="de">Technologieplanung (Betrieb)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19548-6">
    <skos:altLabel xml:lang="de">Technologieorientierte Unternehmensstrategie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19548-6">
    <skos:altLabel xml:lang="de">Technologiemanagement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19548-6">
    <skos:altLabel xml:lang="de">Strategisches Technologiemanagement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19548-6">
    <skos:altLabel xml:lang="de">Strategische Technologieplanung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19130-6">
    <skos:narrower rdf:resource="descriptor/17829-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19130-6">
    <skos:narrower rdf:resource="descriptor/17790-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19130-6">
    <skos:narrower rdf:resource="descriptor/17811-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19224-4">
    <skos:prefLabel xml:lang="en">Spatial price discrimination</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19224-4">
    <skos:prefLabel xml:lang="de">Regionale Preisstruktur</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12879-4">
    <skos:prefLabel xml:lang="en">Commissioner</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12879-4">
    <skos:prefLabel xml:lang="de">Betriebsbeauftragte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15472-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19857-0">
    <skos:scopeNote xml:lang="de">Das Zustandekommen eines Beschlusses wird in einem kollektiven Gremium durch eine ausschlaggebende Gegenstimme verhindert.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id158">
    <rdf:value xml:lang="de">Für wirtschafts- und sozialpolitischen Interventionismus</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16527-1">
    <skos:altLabel xml:lang="de">Rente (Alterssicherung)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16527-1">
    <skos:altLabel xml:lang="de">Altersvorsorge</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16527-1">
    <skos:altLabel xml:lang="de">Altersversorgung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16527-1">
    <skos:altLabel xml:lang="de">Altersruhegeld</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16527-1">
    <skos:altLabel xml:lang="de">Altersrente</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18649-5">
    <skos:related rdf:resource="descriptor/18643-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18649-5">
    <skos:related rdf:resource="descriptor/18653-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18649-5">
    <skos:related rdf:resource="descriptor/18000-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18649-5">
    <skos:related rdf:resource="descriptor/18428-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18649-5">
    <skos:related rdf:resource="descriptor/18652-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18649-5">
    <skos:related rdf:resource="descriptor/18651-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18649-5">
    <skos:related rdf:resource="descriptor/18947-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18649-5">
    <skos:related rdf:resource="descriptor/13278-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18649-5">
    <skos:related rdf:resource="descriptor/19678-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18649-5">
    <skos:related rdf:resource="descriptor/13682-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18649-5">
    <skos:related rdf:resource="descriptor/13347-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10806-6">
    <skos:altLabel xml:lang="en">International bank</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10806-6">
    <skos:altLabel xml:lang="en">Foreign bank</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10806-6">
    <skos:altLabel xml:lang="de">Multinationale Bank</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10806-6">
    <skos:altLabel xml:lang="de">Internationale Geschäftsbank</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10806-6">
    <skos:altLabel xml:lang="de">Bankmulti</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10806-6">
    <skos:altLabel xml:lang="de">Ausländische Bank</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19635-4">
    <skos:narrower rdf:resource="descriptor/19634-6"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id142">
    <rdf:value xml:lang="de">Für kommunale Industrie- und Gewerbegebiete</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16516-6">
    <skos:prefLabel xml:lang="en">Media policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16516-6">
    <skos:prefLabel xml:lang="de">Medienpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19767-1">
    <skos:prefLabel xml:lang="en">Labour market research</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19767-1">
    <skos:prefLabel xml:lang="de">Arbeitsmarktforschung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14610-3">
    <skos:altLabel xml:lang="de">Weichmacher für Kunststoff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14610-3">
    <skos:altLabel xml:lang="de">Stabilisator für Kunststoff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14610-3">
    <skos:altLabel xml:lang="de">Kunststoffstabilisator</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14610-3">
    <skos:altLabel xml:lang="de">Kunststoffhärter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14610-3">
    <skos:altLabel xml:lang="de">Kunststoffadditiv</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12669-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73377">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19442-1">
    <skos:altLabel xml:lang="en">Eco-thermodynamics</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18167-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70858">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.22.02.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19257-3">
    <skos:broader rdf:resource="thsys/70306"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19257-3">
    <skos:broader rdf:resource="descriptor/11449-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18992-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16552-2">
    <skos:prefLabel xml:lang="en">Regional policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16552-2">
    <skos:prefLabel xml:lang="de">Regionalpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14472-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18792-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16708-4">
    <skos:narrower rdf:resource="descriptor/16721-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16708-4">
    <skos:narrower rdf:resource="descriptor/16720-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16708-4">
    <skos:narrower rdf:resource="descriptor/19730-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19503-0">
    <skos:broader rdf:resource="thsys/73336"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19503-0">
    <skos:broader rdf:resource="thsys/70235"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19503-0">
    <skos:broader rdf:resource="descriptor/11185-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19199-5">
    <skos:prefLabel xml:lang="en">Hysteresis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19199-5">
    <skos:prefLabel xml:lang="de">Hysteresis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10301-1">
    <skos:altLabel xml:lang="de">Verbrauchsgut</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11999-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11170-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12736-5">
    <skos:broader rdf:resource="thsys/70549"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12736-5">
    <skos:broader rdf:resource="descriptor/14336-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17983-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70572">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14091-1">
    <skos:prefLabel xml:lang="en">Millet</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14091-1">
    <skos:prefLabel xml:lang="de">Hirse</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21002-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19645-1">
    <skos:scopeNote xml:lang="de">Bevölkerungswissenschaftliche Theorie über die Entwicklung der Geburten- und Sterbeziffern während der Phase des Überganges eines Landes von der Agrar- zur Industriegesellschaft</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19075-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12708-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19620-3">
    <skos:broader rdf:resource="thsys/73352"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19620-3">
    <skos:broader rdf:resource="descriptor/15889-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19620-3">
    <skos:broader rdf:resource="descriptor/15880-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11537-2">
    <skos:related rdf:resource="descriptor/11603-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11537-2">
    <skos:related rdf:resource="descriptor/19378-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11537-2">
    <skos:related rdf:resource="descriptor/19375-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73366">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.04.04</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16273-6">
    <skos:related rdf:resource="descriptor/15700-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16273-6">
    <skos:related rdf:resource="descriptor/16292-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16273-6">
    <skos:related rdf:resource="descriptor/16287-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16273-6">
    <skos:related rdf:resource="descriptor/16277-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16273-6">
    <skos:related rdf:resource="descriptor/16209-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16273-6">
    <skos:related rdf:resource="descriptor/13989-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16273-6">
    <skos:related rdf:resource="descriptor/16274-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70930">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12416-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16092-3">
    <skos:prefLabel xml:lang="en">Income tax law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16092-3">
    <skos:prefLabel xml:lang="de">Einkommensteuerrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10121-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19476-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id256">
    <rdfs:seeAlso rdf:resource="descriptor/13273-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19683-0">
    <skos:broader rdf:resource="thsys/73382"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10068-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18251-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12711-0">
    <skos:altLabel xml:lang="de">Produktionsverfahren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12711-0">
    <skos:altLabel xml:lang="de">Produktionstechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12711-0">
    <skos:altLabel xml:lang="de">Fertigungsverfahren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id200">
    <rdfs:seeAlso rdf:resource="descriptor/11341-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12588-1">
    <skos:related rdf:resource="descriptor/18143-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12588-1">
    <skos:related rdf:resource="descriptor/11299-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12588-1">
    <skos:related rdf:resource="descriptor/11295-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12588-1">
    <skos:related rdf:resource="descriptor/19656-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70493">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11728-2">
    <skos:altLabel xml:lang="de">Fiskalpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18195-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11804-5">
    <skos:prefLabel xml:lang="en">Agricultural structural policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11804-5">
    <skos:prefLabel xml:lang="de">Agrarstrukturpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14537-1">
    <skos:altLabel xml:lang="de">Topographisches Instrument</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14537-1">
    <skos:altLabel xml:lang="de">Optisches Messgerät</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14537-1">
    <skos:altLabel xml:lang="de">Optisches Meßgerät</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14537-1">
    <skos:altLabel xml:lang="de">Optisches Instrument</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14537-1">
    <skos:altLabel xml:lang="de">Mikroskop</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14537-1">
    <skos:altLabel xml:lang="de">Messgerät (Optik)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14537-1">
    <skos:altLabel xml:lang="de">Meßgerät (Optik)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14537-1">
    <skos:altLabel xml:lang="de">Lupe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14537-1">
    <skos:altLabel xml:lang="de">Geodätisches Instrument</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14537-1">
    <skos:altLabel xml:lang="de">Fernglas</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14537-1">
    <skos:altLabel xml:lang="de">Entfernungsmesser</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16441-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18240-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17413-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14040-4">
    <skos:broader rdf:resource="thsys/70581"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14040-4">
    <skos:broader rdf:resource="thsys/70201"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18969-1">
    <skos:narrower rdf:resource="descriptor/15938-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18969-1">
    <skos:narrower rdf:resource="descriptor/15937-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11652-2">
    <skos:altLabel xml:lang="de">Gewinnsteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11652-2">
    <skos:altLabel xml:lang="de">Gewinnbesteuerung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11652-2">
    <skos:altLabel xml:lang="de">Ertragsbesteuerung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11652-2">
    <skos:altLabel xml:lang="de">Ergebnissteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10931-3">
    <skos:altLabel xml:lang="de">Vertikale Preisbindung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10931-3">
    <skos:altLabel xml:lang="de">Unverbindliche Preisempfehlung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10931-3">
    <skos:altLabel xml:lang="de">Richtpreis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10931-3">
    <skos:altLabel xml:lang="de">Preisempfehlung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10931-3">
    <skos:altLabel xml:lang="de">Preisbindung der zweiten Hand</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10931-3">
    <skos:altLabel xml:lang="de">Empfohlener Preis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19722-2">
    <skos:prefLabel xml:lang="en">Consequences of war</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19722-2">
    <skos:prefLabel xml:lang="de">Kriegsfolgen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17642-3">
    <skos:narrower rdf:resource="descriptor/17750-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17642-3">
    <skos:narrower rdf:resource="descriptor/17986-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17642-3">
    <skos:narrower rdf:resource="descriptor/17984-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17642-3">
    <skos:narrower rdf:resource="descriptor/17652-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16185-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11742-1">
    <skos:prefLabel xml:lang="en">Lorenz curve</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11742-1">
    <skos:prefLabel xml:lang="de">Lorenz-Kurve</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15616-0">
    <skos:altLabel xml:lang="en">Time sharing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15616-0">
    <skos:altLabel xml:lang="en">Real-time-operation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15616-0">
    <skos:altLabel xml:lang="en">Real-time system</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15616-0">
    <skos:altLabel xml:lang="en">Interactive mode</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15616-0">
    <skos:altLabel xml:lang="en">Batch processing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15616-0">
    <skos:altLabel xml:lang="de">Stapelverarbeitung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15616-0">
    <skos:altLabel xml:lang="de">Online-Betrieb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15616-0">
    <skos:altLabel xml:lang="de">Offline-Betrieb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15616-0">
    <skos:altLabel xml:lang="de">Multitask-Betrieb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15616-0">
    <skos:altLabel xml:lang="de">Mehrbenutzersystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15616-0">
    <skos:altLabel xml:lang="de">Interaktiver Betrieb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15616-0">
    <skos:altLabel xml:lang="de">Echtzeitverarbeitung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15616-0">
    <skos:altLabel xml:lang="de">Echtzeitsystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15616-0">
    <skos:altLabel xml:lang="de">Dialogbetrieb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15616-0">
    <skos:altLabel xml:lang="de">Computerbetriebsablauf</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16198-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15659-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13762-6">
    <skos:prefLabel xml:lang="en">Repo transaction</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13762-6">
    <skos:prefLabel xml:lang="de">Pensionsgeschäft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70862">
    <skos:narrower rdf:resource="descriptor/13423-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70862">
    <skos:narrower rdf:resource="descriptor/13422-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70862">
    <skos:narrower rdf:resource="descriptor/15979-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70862">
    <skos:narrower rdf:resource="descriptor/16332-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70862">
    <skos:narrower rdf:resource="descriptor/13417-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70862">
    <skos:narrower rdf:resource="descriptor/15977-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70862">
    <skos:narrower rdf:resource="descriptor/16328-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70862">
    <skos:narrower rdf:resource="descriptor/16524-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70862">
    <skos:narrower rdf:resource="descriptor/15962-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70862">
    <skos:narrower rdf:resource="descriptor/19841-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70862">
    <skos:narrower rdf:resource="descriptor/16510-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70862">
    <skos:narrower rdf:resource="descriptor/11814-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70862">
    <skos:narrower rdf:resource="descriptor/13410-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70862">
    <skos:narrower rdf:resource="descriptor/11532-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70862">
    <skos:narrower rdf:resource="descriptor/18820-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70862">
    <skos:narrower rdf:resource="descriptor/11307-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24666-5">
    <skos:altLabel xml:lang="de">Fair-Value-Richtlinie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24666-5">
    <skos:altLabel xml:lang="de">EU-Fair-Value-Richtlinie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id100">
    <rdf:value xml:lang="de">für die Geschichte des Börsenwesens</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11642-5">
    <skos:altLabel xml:lang="de">Steuerverfassung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11642-5">
    <skos:altLabel xml:lang="de">Steuerstruktur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11642-5">
    <skos:altLabel xml:lang="de">Rationales Steuersystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11642-5">
    <skos:altLabel xml:lang="de">Historisches Steuersystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id66">
    <rdf:value xml:lang="de">Für Regelungen zum Schutz personenbezogener Daten</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70829">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11554-2">
    <skos:related rdf:resource="descriptor/11721-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11554-2">
    <skos:related rdf:resource="descriptor/11596-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11554-2">
    <skos:related rdf:resource="descriptor/11579-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11554-2">
    <skos:related rdf:resource="descriptor/11734-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70274">
    <skos:broader rdf:resource="thsys/70049"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10612-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18780-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id320">
    <rdf:value xml:lang="de">für den Pkw-Verkehr</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14346-1">
    <skos:related rdf:resource="descriptor/13058-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14346-1">
    <skos:related rdf:resource="descriptor/13354-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14346-1">
    <skos:related rdf:resource="descriptor/13200-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14346-1">
    <skos:related rdf:resource="descriptor/14335-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16035-1">
    <skos:broader rdf:resource="thsys/73392"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16035-1">
    <skos:broader rdf:resource="descriptor/16026-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12574-5">
    <skos:broader rdf:resource="thsys/73385"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12574-5">
    <skos:broader rdf:resource="thsys/70244"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11239-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20826-1">
    <skos:altLabel xml:lang="de">Weltordnungspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20826-1">
    <skos:altLabel xml:lang="de">Globale Steuerung (Global Governance)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24788-5">
    <skos:prefLabel xml:lang="en">Melanesia</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24788-5">
    <skos:prefLabel xml:lang="de">Melanesien</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19203-5">
    <skos:related rdf:resource="descriptor/19442-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19203-5">
    <skos:related rdf:resource="descriptor/19204-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19203-5">
    <skos:related rdf:resource="descriptor/19596-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19203-5">
    <skos:related rdf:resource="descriptor/15633-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13199-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12883-6">
    <skos:altLabel xml:lang="de">Rechnergestützte Prüfung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12883-6">
    <skos:altLabel xml:lang="de">EDV-gestützte Revision</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12883-6">
    <skos:altLabel xml:lang="de">EDV-gestützte Prüfung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19616-1">
    <skos:broader rdf:resource="thsys/73353"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12567-2">
    <skos:altLabel xml:lang="en">Human-resources management</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12567-2">
    <skos:altLabel xml:lang="en">Human resources management</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12567-2">
    <skos:altLabel xml:lang="de">Personalwirtschaftslehre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12567-2">
    <skos:altLabel xml:lang="de">Personalwesen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12567-2">
    <skos:altLabel xml:lang="de">Personalverwaltung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12567-2">
    <skos:altLabel xml:lang="de">Personalpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12567-2">
    <skos:altLabel xml:lang="de">Personalmanagement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12567-2">
    <skos:altLabel xml:lang="de">Personalarbeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12567-2">
    <skos:altLabel xml:lang="de">Personalabteilung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12567-2">
    <skos:altLabel xml:lang="de">HRM</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12567-2">
    <skos:altLabel xml:lang="de">Betriebliches Personalwesen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12567-2">
    <skos:altLabel xml:lang="de">Betriebliche Beschäftigungspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15765-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19810-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13367-5">
    <skos:altLabel xml:lang="de">Verlagswesen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16973-6">
    <skos:broader rdf:resource="thsys/70003"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16973-6">
    <skos:broader rdf:resource="descriptor/16972-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15880-4">
    <skos:narrower rdf:resource="descriptor/19622-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15880-4">
    <skos:narrower rdf:resource="descriptor/15894-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15880-4">
    <skos:narrower rdf:resource="descriptor/18069-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15880-4">
    <skos:narrower rdf:resource="descriptor/19620-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71091">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12868-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19016-4">
    <skos:broader rdf:resource="thsys/73347"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19016-4">
    <skos:broader rdf:resource="descriptor/15658-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11303-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15925-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19084-1">
    <skos:related rdf:resource="descriptor/19085-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19084-1">
    <skos:related rdf:resource="descriptor/19837-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19084-1">
    <skos:related rdf:resource="descriptor/19077-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19084-1">
    <skos:related rdf:resource="descriptor/16825-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19314-3">
    <skos:prefLabel xml:lang="en">Antidumping</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19314-3">
    <skos:prefLabel xml:lang="de">Antidumping</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19293-6">
    <skos:prefLabel xml:lang="en">Grain price</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19293-6">
    <skos:prefLabel xml:lang="de">Getreidepreis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19319-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16841-2">
    <skos:prefLabel xml:lang="en">Baden-Württemberg</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16841-2">
    <skos:prefLabel xml:lang="de">Baden-Württemberg</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14990-2">
    <skos:related rdf:resource="descriptor/12957-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14990-2">
    <skos:related rdf:resource="descriptor/12939-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18217-6">
    <skos:scopeNote xml:lang="de">Produkte sowohl auf Milch- wie auf Wasserbasis.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71006">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19598-5">
    <skos:prefLabel xml:lang="en">Analytic Hierarchy Process</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19598-5">
    <skos:prefLabel xml:lang="de">AHP-Verfahren</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id317">
    <rdf:value xml:lang="de">Für internationale steuerliche Doppelbelastungen</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70031">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11966-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19029-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19489-3">
    <skos:related rdf:resource="descriptor/19182-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19489-3">
    <skos:related rdf:resource="descriptor/19181-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10835-6">
    <skos:broader rdf:resource="thsys/72181"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10835-6">
    <skos:broader rdf:resource="thsys/70952"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10835-6">
    <skos:broader rdf:resource="descriptor/13465-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13457-4">
    <skos:scopeNote xml:lang="de">Nur für freiwillige Ketten als Kooperationsform des Handels.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19203-5">
    <skos:prefLabel xml:lang="en">Bioeconomics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19203-5">
    <skos:prefLabel xml:lang="de">Bioökonomik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14975-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19653-2">
    <skos:altLabel xml:lang="en">VOIP</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19653-2">
    <skos:altLabel xml:lang="en">Voice over internet protocol</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19653-2">
    <skos:altLabel xml:lang="en">Internet telephony</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19653-2">
    <skos:altLabel xml:lang="en">Computer telephone integration</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19653-2">
    <skos:altLabel xml:lang="de">Internettelefonie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19653-2">
    <skos:altLabel xml:lang="de">Internet-Telefonie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19653-2">
    <skos:altLabel xml:lang="de">Computer-Telefon-Integration</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19653-2">
    <skos:altLabel xml:lang="de">Computer-Telefonie-Integration</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19653-2">
    <skos:altLabel xml:lang="de">Computertelefonie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12019-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18352-0">
    <skos:scopeNote xml:lang="de">Recht zur Gewinnung von Bodenschätzen, auch im Festlandsockel und in den Küstengewässern.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13537-6">
    <skos:related rdf:resource="descriptor/13536-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13537-6">
    <skos:related rdf:resource="descriptor/18599-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13537-6">
    <skos:related rdf:resource="descriptor/13534-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13537-6">
    <skos:related rdf:resource="descriptor/13553-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11729-0">
    <skos:related rdf:resource="descriptor/11535-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70875">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.21.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12918-6">
    <skos:broader rdf:resource="thsys/71004"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12918-6">
    <skos:broader rdf:resource="descriptor/12993-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11209-5">
    <skos:broader rdf:resource="thsys/73334"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11209-5">
    <skos:broader rdf:resource="thsys/70052"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11209-5">
    <skos:broader rdf:resource="descriptor/16201-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11209-5">
    <skos:broader rdf:resource="descriptor/10018-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11209-5">
    <skos:broader rdf:resource="descriptor/11180-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16288-0">
    <skos:narrower rdf:resource="descriptor/16472-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16288-0">
    <skos:narrower rdf:resource="descriptor/16470-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16288-0">
    <skos:narrower rdf:resource="descriptor/16471-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16288-0">
    <skos:narrower rdf:resource="descriptor/19324-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11560-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12844-2">
    <skos:altLabel xml:lang="de">Werbestrategie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12844-2">
    <skos:altLabel xml:lang="de">Werbestil</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12844-2">
    <skos:altLabel xml:lang="de">Werbekonzeption</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12844-2">
    <skos:altLabel xml:lang="de">Werbekampagne</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12844-2">
    <skos:altLabel xml:lang="de">Werbefeldzug</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12844-2">
    <skos:altLabel xml:lang="de">Werbebotschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10298-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19697-3">
    <skos:altLabel xml:lang="de">Kreiswahl</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19697-3">
    <skos:altLabel xml:lang="de">Gemeindewahl</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15861-1">
    <skos:scopeNote xml:lang="de">Verhaltenswissenschaftlich bzw. psychologisch begründete Modifizierungen neoklassischer Verhaltensannahmen, die sich aus empirischen Beobachtungen ableiten.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12164-3">
    <skos:prefLabel xml:lang="en">Reporting</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12164-3">
    <skos:prefLabel xml:lang="de">Berichtswesen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14807-5">
    <skos:scopeNote xml:lang="de">Nur Papierschreibwaren.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13055-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19013-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17727-2">
    <skos:broader rdf:resource="thsys/71031"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17727-2">
    <skos:broader rdf:resource="descriptor/17743-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14593-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18977-2">
    <skos:altLabel xml:lang="de">Kaufmänner</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18977-2">
    <skos:altLabel xml:lang="de">Kaufmann</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18977-2">
    <skos:altLabel xml:lang="de">Kaufleute</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18977-2">
    <skos:altLabel xml:lang="de">Kauffrauen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18977-2">
    <skos:altLabel xml:lang="de">Industriekaufleute</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18977-2">
    <skos:altLabel xml:lang="de">Buchhalterinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18977-2">
    <skos:altLabel xml:lang="de">Buchhalter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70903">
    <skos:prefLabel xml:lang="en">W.14.02  Banking system</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70903">
    <skos:prefLabel xml:lang="de">W.14.02  Bankensystem</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16479-0">
    <skos:narrower rdf:resource="descriptor/11677-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16479-0">
    <skos:narrower rdf:resource="descriptor/16541-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16479-0">
    <skos:narrower rdf:resource="descriptor/18063-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16479-0">
    <skos:narrower rdf:resource="descriptor/18334-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16479-0">
    <skos:narrower rdf:resource="descriptor/12615-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18347-0">
    <skos:related rdf:resource="descriptor/18989-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16084-2">
    <skos:prefLabel xml:lang="en">Trade discrimination</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16084-2">
    <skos:prefLabel xml:lang="de">Außenhandelsdiskriminierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11335-0">
    <skos:prefLabel xml:lang="en">Wage rigidity</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11335-0">
    <skos:prefLabel xml:lang="de">Lohnrigidität</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id177">
    <rdfs:seeAlso rdf:resource="descriptor/13324-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15648-1">
    <skos:related rdf:resource="descriptor/16424-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15648-1">
    <skos:related rdf:resource="descriptor/15923-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15648-1">
    <skos:related rdf:resource="descriptor/16400-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18310-2">
    <skos:prefLabel xml:lang="en">Mesoeconomic analysis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18310-2">
    <skos:prefLabel xml:lang="de">Mesoökonomik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12081-0">
    <skos:prefLabel xml:lang="en">Middle management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12081-0">
    <skos:prefLabel xml:lang="de">Mittleres Management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15080-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19543-2">
    <skos:prefLabel xml:lang="en">German apprenticeship system</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19543-2">
    <skos:prefLabel xml:lang="de">Duale Berufsausbildung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13329-6">
    <skos:related rdf:resource="descriptor/12979-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13329-6">
    <skos:related rdf:resource="descriptor/16023-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13329-6">
    <skos:related rdf:resource="descriptor/18379-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13329-6">
    <skos:related rdf:resource="descriptor/15731-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13329-6">
    <skos:related rdf:resource="descriptor/15730-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13329-6">
    <skos:related rdf:resource="descriptor/15729-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15947-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14081-4">
    <skos:prefLabel xml:lang="en">Soybean</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14081-4">
    <skos:prefLabel xml:lang="de">Sojabohne</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id44">
    <rdf:value xml:lang="de">Für die Förderung des Wohnungsbaus</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11599-1">
    <skos:prefLabel xml:lang="en">Fiscal transfer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11599-1">
    <skos:prefLabel xml:lang="de">Finanzausgleich</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13067-3">
    <skos:broader rdf:resource="thsys/71002"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13067-3">
    <skos:broader rdf:resource="descriptor/13058-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13472-1">
    <skos:prefLabel xml:lang="en">Raw materials trade</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13472-1">
    <skos:prefLabel xml:lang="de">Rohstoffhandel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10822-1">
    <skos:related rdf:resource="descriptor/10614-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10822-1">
    <skos:related rdf:resource="descriptor/19451-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10822-1">
    <skos:related rdf:resource="descriptor/18001-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10822-1">
    <skos:related rdf:resource="descriptor/18794-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17724-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12392-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16989-5">
    <skos:altLabel xml:lang="en">Sverige</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16989-5">
    <skos:altLabel xml:lang="de">Königreich Schweden</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12119-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18350-4">
    <skos:narrower rdf:resource="descriptor/18349-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/p">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11722-0">
    <skos:altLabel xml:lang="de">Staatsaufgaben</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11722-0">
    <skos:altLabel xml:lang="de">Staatlicher Sektor</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11722-0">
    <skos:altLabel xml:lang="de">Öffentliche Hand</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11722-0">
    <skos:altLabel xml:lang="de">Öffentliche Aufgaben</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13253-6">
    <skos:prefLabel xml:lang="en">Travel agency</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13253-6">
    <skos:prefLabel xml:lang="de">Reisevermittler</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70372">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">P.16</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14837-3">
    <skos:altLabel xml:lang="de">Vlies</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14837-3">
    <skos:altLabel xml:lang="de">Spinnvlies</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14837-3">
    <skos:altLabel xml:lang="de">Einlagevlies</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70086">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11521-3">
    <skos:prefLabel xml:lang="en">Currency reform</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11521-3">
    <skos:prefLabel xml:lang="de">Währungsreform</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id80">
    <rdf:value xml:lang="de">Für Speiseeis</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13984-2">
    <skos:broader rdf:resource="thsys/73379"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13984-2">
    <skos:broader rdf:resource="thsys/71054"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13984-2">
    <skos:broader rdf:resource="descriptor/13989-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14673-0">
    <skos:prefLabel xml:lang="en">Dioxin</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14673-0">
    <skos:prefLabel xml:lang="de">Dioxin</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18454-6">
    <skos:broader rdf:resource="thsys/71004"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18454-6">
    <skos:broader rdf:resource="thsys/70958"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13157-2">
    <skos:prefLabel xml:lang="en">Beverage industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13157-2">
    <skos:prefLabel xml:lang="de">Getränkeindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18229-6">
    <skos:broader rdf:resource="thsys/70254"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18229-6">
    <skos:broader rdf:resource="thsys/70161"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18229-6">
    <skos:broader rdf:resource="descriptor/14273-2"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id76">
    <rdf:value xml:lang="de">Für Drogen als Heilmittel</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17983-5">
    <skos:related rdf:resource="descriptor/16495-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17983-5">
    <skos:related rdf:resource="descriptor/16815-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13079-3">
    <skos:altLabel xml:lang="de">Steuerungstechnische Industrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13079-3">
    <skos:altLabel xml:lang="de">Regeltechnische Industrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13079-3">
    <skos:altLabel xml:lang="de">Prüftechnische Industrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13079-3">
    <skos:altLabel xml:lang="de">Meßtechnische Industrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13079-3">
    <skos:altLabel xml:lang="de">Kontrolltechnische Industrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19740-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19484-6">
    <skos:related rdf:resource="descriptor/15903-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19484-6">
    <skos:related rdf:resource="descriptor/18337-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19635-4">
    <skos:altLabel xml:lang="de">Historiographie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19635-4">
    <skos:altLabel xml:lang="de">Geschichtsschreibung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18328-4">
    <skos:altLabel xml:lang="de">Spezielle Intensität (Landwirtschaft)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18328-4">
    <skos:altLabel xml:lang="de">Intensive Bodennutzung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18328-4">
    <skos:altLabel xml:lang="de">Extensive Bodennutzung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18328-4">
    <skos:altLabel xml:lang="de">Bodenbewirtschaftungsintensität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19097-6">
    <skos:altLabel xml:lang="de">Kostenvergleich</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19097-6">
    <skos:altLabel xml:lang="de">Kostenabweichungsanalyse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14834-2">
    <skos:altLabel xml:lang="de">Turnschuhe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14834-2">
    <skos:altLabel xml:lang="de">Freizeitschuhe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18422-5">
    <skos:prefLabel xml:lang="en">Automotive workers</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18422-5">
    <skos:prefLabel xml:lang="de">Kraftfahrzeugtechnische Berufe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12577-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20404-6">
    <skos:related rdf:resource="descriptor/18767-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20404-6">
    <skos:related rdf:resource="descriptor/12164-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19251-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15892-4">
    <skos:related rdf:resource="descriptor/15887-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15892-4">
    <skos:related rdf:resource="descriptor/13529-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15892-4">
    <skos:related rdf:resource="descriptor/12963-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15055-0">
    <skos:related rdf:resource="descriptor/10303-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14716-1">
    <skos:altLabel xml:lang="de">Weichschaum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14716-1">
    <skos:altLabel xml:lang="de">Schaumkunststoff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14716-1">
    <skos:altLabel xml:lang="de">Hartschaum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14215-2">
    <skos:prefLabel xml:lang="en">Fuel oil</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14215-2">
    <skos:prefLabel xml:lang="de">Heizöl</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10036-1">
    <skos:prefLabel xml:lang="en">Elasticity</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10036-1">
    <skos:prefLabel xml:lang="de">Elastizität</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id140">
    <rdfs:seeAlso rdf:resource="descriptor/16789-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14163-2">
    <skos:altLabel xml:lang="de">Tunfisch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14163-2">
    <skos:altLabel xml:lang="de">Thunfisch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14163-2">
    <skos:altLabel xml:lang="de">Seelachs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14163-2">
    <skos:altLabel xml:lang="de">Scholle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14163-2">
    <skos:altLabel xml:lang="de">Sardine</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14163-2">
    <skos:altLabel xml:lang="de">Makrele</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14163-2">
    <skos:altLabel xml:lang="de">Kabeljau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14163-2">
    <skos:altLabel xml:lang="de">Hering</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14163-2">
    <skos:altLabel xml:lang="de">Flunder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14163-2">
    <skos:altLabel xml:lang="de">Dorsch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19812-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70557">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">G.01.05</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19347-2">
    <skos:prefLabel xml:lang="en">Livestock market</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19347-2">
    <skos:prefLabel xml:lang="de">Viehmarkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10265-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13310-6">
    <skos:altLabel xml:lang="en">Nursing home</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13310-6">
    <skos:altLabel xml:lang="de">Seniorenheim</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13310-6">
    <skos:altLabel xml:lang="de">Altersheim</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13310-6">
    <skos:altLabel xml:lang="de">Altenwohnheim</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13310-6">
    <skos:altLabel xml:lang="de">Altenpflegeheim</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13669-3">
    <skos:related rdf:resource="descriptor/12847-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13669-3">
    <skos:related rdf:resource="descriptor/13368-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16485-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10329-0">
    <skos:altLabel xml:lang="de">Putty-Putty-Modell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10329-0">
    <skos:altLabel xml:lang="de">Clay-Clay-Modell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71093">
    <skos:narrower rdf:resource="descriptor/10628-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71093">
    <skos:narrower rdf:resource="descriptor/19331-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71093">
    <skos:narrower rdf:resource="descriptor/10627-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71093">
    <skos:narrower rdf:resource="descriptor/10626-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71093">
    <skos:narrower rdf:resource="descriptor/11703-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71093">
    <skos:narrower rdf:resource="descriptor/19323-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71093">
    <skos:narrower rdf:resource="descriptor/19309-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71093">
    <skos:narrower rdf:resource="descriptor/18181-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71093">
    <skos:narrower rdf:resource="descriptor/19351-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71093">
    <skos:narrower rdf:resource="descriptor/10612-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71093">
    <skos:narrower rdf:resource="descriptor/19350-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71093">
    <skos:narrower rdf:resource="thsys/71092"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18064-5">
    <skos:related rdf:resource="descriptor/10619-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18064-5">
    <skos:related rdf:resource="descriptor/16288-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18064-5">
    <skos:related rdf:resource="descriptor/19324-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18064-5">
    <skos:related rdf:resource="descriptor/10682-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12935-6">
    <skos:prefLabel xml:lang="en">Forest nursery</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12935-6">
    <skos:prefLabel xml:lang="de">Baumschule</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14149-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10984-3">
    <zbwext:useInsteadNote rdf:nodeID="id150"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15686-0">
    <skos:related rdf:resource="descriptor/16732-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15686-0">
    <skos:related rdf:resource="descriptor/15685-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10613-3">
    <skos:altLabel xml:lang="en">International labor mobility</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10613-3">
    <skos:altLabel xml:lang="de">Internationale Wanderung von Arbeitskräften</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10613-3">
    <skos:altLabel xml:lang="de">Internationale Arbeitskräftewanderung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10613-3">
    <skos:altLabel xml:lang="de">Arbeitsmigration</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19176-3">
    <skos:broader rdf:resource="thsys/70307"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19176-3">
    <skos:broader rdf:resource="descriptor/10492-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12778-3">
    <skos:altLabel xml:lang="en">Marketing channel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12778-3">
    <skos:altLabel xml:lang="de">Vertriebssystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12778-3">
    <skos:altLabel xml:lang="de">Vertriebskanal</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12778-3">
    <skos:altLabel xml:lang="de">Vertriebsform</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12778-3">
    <skos:altLabel xml:lang="de">Distributionsweg</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12778-3">
    <skos:altLabel xml:lang="de">Distributionssystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12778-3">
    <skos:altLabel xml:lang="de">Distributionskanal</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12778-3">
    <skos:altLabel xml:lang="de">Absatzwegepolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12778-3">
    <skos:altLabel xml:lang="de">Absatzweg</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12778-3">
    <skos:altLabel xml:lang="de">Absatzkanal</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13419-5">
    <skos:broader rdf:resource="thsys/73380"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13419-5">
    <skos:broader rdf:resource="thsys/70868"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13419-5">
    <skos:broader rdf:resource="descriptor/13413-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19627-3">
    <skos:scopeNote xml:lang="de">Schutz von Haus- und Nutztieren vor Quälerei und vor einer nicht-artgerechten Haltung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11771-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18284-5">
    <skos:prefLabel xml:lang="en">Eritrea</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18284-5">
    <skos:prefLabel xml:lang="de">Eritrea</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15408-0">
    <skos:broader rdf:resource="thsys/72393"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15408-0">
    <skos:broader rdf:resource="descriptor/19150-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18074-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14194-5">
    <skos:prefLabel xml:lang="en">Lignite</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14194-5">
    <skos:prefLabel xml:lang="de">Braunkohle</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19170-1">
    <skos:broader rdf:resource="thsys/70307"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19170-1">
    <skos:broader rdf:resource="descriptor/10492-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15773-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14632-0">
    <skos:related rdf:resource="descriptor/15017-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14632-0">
    <skos:related rdf:resource="descriptor/14600-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13381-4">
    <skos:related rdf:resource="descriptor/19055-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13381-4">
    <skos:related rdf:resource="descriptor/18789-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13381-4">
    <skos:related rdf:resource="descriptor/13960-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19379-3">
    <skos:prefLabel xml:lang="en">Sovereign insolvency</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19379-3">
    <skos:prefLabel xml:lang="de">Staatsbankrott</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11168-0">
    <skos:scopeNote xml:lang="de">Beschäftigt sich u.a. mit den Fragen: wie kommt es zur Bildung von Verbänden, welchen Anreiz bietet die Mitgliedschaft, welche Rolle spielen die Interessengruppen bei politischen und wirtschaftlichen Entscheidungen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10750-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19836-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10486-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19499-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10148-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16166-0">
    <skos:broader rdf:resource="thsys/73389"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16166-0">
    <skos:broader rdf:resource="thsys/70941"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16166-0">
    <skos:broader rdf:resource="descriptor/13569-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73103">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.18</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19663-6">
    <skos:prefLabel xml:lang="en">EU accession criteria</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19663-6">
    <skos:prefLabel xml:lang="de">EU-Beitrittskriterien</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16271-3">
    <skos:broader rdf:resource="thsys/73363"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16271-3">
    <skos:broader rdf:resource="thsys/70982"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16271-3">
    <skos:broader rdf:resource="thsys/70865"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17753-1">
    <skos:altLabel xml:lang="de">Spanische Sahara</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17753-1">
    <skos:altLabel xml:lang="de">Sahara (Demokratische Arabische Republik)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17753-1">
    <skos:altLabel xml:lang="de">Demokratische Arabische Republik Sahara</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15861-1">
    <skos:narrower rdf:resource="descriptor/10300-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15861-1">
    <skos:narrower rdf:resource="descriptor/18674-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10123-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13153-3">
    <skos:altLabel xml:lang="de">Ölmühlenindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13153-3">
    <skos:altLabel xml:lang="de">Margarineindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13153-3">
    <skos:altLabel xml:lang="de">Herstellung von pflanzlichen Ölen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13153-3">
    <skos:altLabel xml:lang="de">Herstellung von pflanzlichen Fetten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id311">
    <rdfs:seeAlso rdf:resource="descriptor/19581-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14192-2">
    <skos:broader rdf:resource="thsys/70418"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14446-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12850-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12136-1">
    <skos:altLabel xml:lang="de">Geschäftsbereich</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12136-1">
    <skos:altLabel xml:lang="de">Betrieblicher Funktionsbereich</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18832-0">
    <skos:prefLabel xml:lang="en">Self-interest</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18832-0">
    <skos:prefLabel xml:lang="de">Eigeninteresse</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70498">
    <skos:narrower rdf:resource="descriptor/14934-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70498">
    <skos:narrower rdf:resource="descriptor/14933-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70498">
    <skos:narrower rdf:resource="descriptor/14938-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70498">
    <skos:narrower rdf:resource="descriptor/14957-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70498">
    <skos:narrower rdf:resource="descriptor/14958-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70498">
    <skos:narrower rdf:resource="descriptor/14960-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70498">
    <skos:narrower rdf:resource="descriptor/14931-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70498">
    <skos:narrower rdf:resource="descriptor/14936-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70498">
    <skos:narrower rdf:resource="descriptor/14068-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70498">
    <skos:narrower rdf:resource="descriptor/14930-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70498">
    <skos:narrower rdf:resource="descriptor/14928-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70498">
    <skos:narrower rdf:resource="descriptor/14926-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70498">
    <skos:narrower rdf:resource="descriptor/14945-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70498">
    <skos:narrower rdf:resource="descriptor/14927-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70498">
    <skos:narrower rdf:resource="descriptor/18214-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70498">
    <skos:narrower rdf:resource="descriptor/14942-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70498">
    <skos:narrower rdf:resource="descriptor/14941-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70498">
    <skos:narrower rdf:resource="descriptor/14932-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70498">
    <skos:narrower rdf:resource="descriptor/14940-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70498">
    <skos:narrower rdf:resource="descriptor/14924-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70498">
    <skos:narrower rdf:resource="thsys/73302"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70498">
    <skos:narrower rdf:resource="thsys/73301"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70498">
    <skos:narrower rdf:resource="thsys/73300"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70498">
    <skos:narrower rdf:resource="thsys/73299"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70498">
    <skos:narrower rdf:resource="thsys/70342"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70498">
    <skos:narrower rdf:resource="thsys/73298"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70498">
    <skos:narrower rdf:resource="thsys/73303"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13845-2">
    <skos:broader rdf:resource="thsys/70888"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13845-2">
    <skos:broader rdf:resource="descriptor/12031-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24665-0">
    <skos:scopeNote xml:lang="de">Direktbanken sind Kreditinstitute ohne Filialen. Der Kontakt mit den Kunden/Kundinnen findet über das Internet oder ersatzweise über Call Center usw. statt.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18895-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11240-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13084-3">
    <skos:prefLabel xml:lang="en">Consumer electronics industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13084-3">
    <skos:prefLabel xml:lang="de">Unterhaltungselektronische Industrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19612-2">
    <skos:broader rdf:resource="thsys/73354"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70130">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#relatedMatch">
    <rdf:type rdf:resource="http://www.w3.org/1999/02/22-rdf-syntax-ns#Property"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#relatedMatch">
    <rdf:type rdf:resource="http://www.w3.org/2002/07/owl#SymmetricProperty"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#relatedMatch">
    <rdf:type rdf:resource="http://www.w3.org/2002/07/owl#ObjectProperty"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14611-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14427-1">
    <skos:broader rdf:resource="thsys/70258"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14427-1">
    <skos:broader rdf:resource="descriptor/14428-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19809-4">
    <skos:scopeNote xml:lang="de">Mensch-Maschine-System, dessen Techniken den Menschen unmittelbar in computer-generierte Welten integrieren, wobei gleichzeitig mehrere menschliche Sinne, z.B. Gesichts-, Hör- und Tastsinn, angesprochen werden können.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14081-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21002-0">
    <skos:altLabel xml:lang="en">Excess (Insurance)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21002-0">
    <skos:altLabel xml:lang="de">Zuzahlung (Versicherung)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21002-0">
    <skos:altLabel xml:lang="de">Selbstbehalt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21002-0">
    <skos:altLabel xml:lang="de">Eigenbehalt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15328-5">
    <skos:altLabel xml:lang="en">Serial correlation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15328-5">
    <skos:altLabel xml:lang="de">Autoregression</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18311-0">
    <skos:prefLabel xml:lang="en">Forestry legislation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18311-0">
    <skos:prefLabel xml:lang="de">Forstrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17964-2">
    <skos:prefLabel xml:lang="en">North Sea</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17964-2">
    <skos:prefLabel xml:lang="de">Nordsee</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14104-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18386-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18975-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12982-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11159-1">
    <skos:prefLabel xml:lang="en">Implicit contracts</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11159-1">
    <skos:prefLabel xml:lang="de">Implizite Kontrakte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10911-2">
    <skos:related rdf:resource="descriptor/12036-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10911-2">
    <skos:related rdf:resource="descriptor/12196-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10911-2">
    <skos:related rdf:resource="descriptor/18117-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10911-2">
    <skos:related rdf:resource="descriptor/11696-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10911-2">
    <skos:related rdf:resource="descriptor/10904-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10911-2">
    <skos:related rdf:resource="descriptor/12049-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10911-2">
    <skos:related rdf:resource="descriptor/12038-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14738-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10278-5">
    <skos:scopeNote xml:lang="de">Nur für makroökonomische Konsumfunktionen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15937-1">
    <skos:related rdf:resource="descriptor/25681-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15937-1">
    <skos:related rdf:resource="descriptor/19774-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15937-1">
    <skos:related rdf:resource="descriptor/16698-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15937-1">
    <skos:related rdf:resource="descriptor/10173-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15937-1">
    <skos:related rdf:resource="descriptor/15938-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15937-1">
    <skos:related rdf:resource="descriptor/16027-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12529-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12251-1">
    <skos:altLabel xml:lang="de">Darlehen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12251-1">
    <skos:altLabel xml:lang="de">Bankkredit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16342-6">
    <skos:broader rdf:resource="thsys/73379"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16342-6">
    <skos:broader rdf:resource="descriptor/16292-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18707-3">
    <skos:broader rdf:resource="thsys/71110"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18707-3">
    <skos:broader rdf:resource="thsys/70905"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18707-3">
    <skos:broader rdf:resource="descriptor/10981-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70060">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18205-6">
    <skos:broader rdf:resource="thsys/70328"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18205-6">
    <skos:broader rdf:resource="descriptor/14802-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19537-4">
    <skos:scopeNote xml:lang="de">Ausdruck für die Relation der Rendite des Marktportefeuilles und der Rendite einer Aktie. Beta indiziert das Marktrisiko einer Aktie bzw. eines Investitionsobjektes.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11661-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14047-4">
    <skos:prefLabel xml:lang="en">Raw materials</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14047-4">
    <skos:prefLabel xml:lang="de">Rohstoff</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15320-0">
    <skos:broader rdf:resource="thsys/72183"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15320-0">
    <skos:broader rdf:resource="descriptor/15142-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12380-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14014-5">
    <skos:narrower rdf:resource="descriptor/14007-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14014-5">
    <skos:narrower rdf:resource="descriptor/18890-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18079-6">
    <skos:broader rdf:resource="thsys/70556"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18079-6">
    <skos:broader rdf:resource="descriptor/14683-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15919-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10055-4">
    <zbwext:useInsteadNote rdf:nodeID="id260"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11278-0">
    <skos:altLabel xml:lang="de">Statistik der offenen Stellen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16333-0">
    <skos:broader rdf:resource="thsys/73381"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16333-0">
    <skos:broader rdf:resource="descriptor/19689-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19323-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12432-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18514-0">
    <skos:related rdf:resource="descriptor/14412-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70458">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71099">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19604-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14063-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11966-0">
    <skos:related rdf:resource="descriptor/11991-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11966-0">
    <skos:related rdf:resource="descriptor/11972-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11966-0">
    <skos:related rdf:resource="descriptor/11988-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11966-0">
    <skos:related rdf:resource="descriptor/19685-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11966-0">
    <skos:related rdf:resource="descriptor/11979-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10561-3">
    <skos:prefLabel xml:lang="en">Development project</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10561-3">
    <skos:prefLabel xml:lang="de">Entwicklungsprojekt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12435-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16476-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18752-5">
    <skos:prefLabel xml:lang="en">Homeownership promotion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18752-5">
    <skos:prefLabel xml:lang="de">Wohneigentumspolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10256-1">
    <skos:narrower rdf:resource="descriptor/10329-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10256-1">
    <skos:narrower rdf:resource="descriptor/10327-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10256-1">
    <skos:narrower rdf:resource="descriptor/10313-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10256-1">
    <skos:narrower rdf:resource="descriptor/10303-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14161-6">
    <skos:scopeNote xml:lang="de">Für alle Formen und Verarbeitungsstufen der Produkte von See- und Binnenfischerei sowie der Aquakultur.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70539">
    <skos:narrower rdf:resource="descriptor/18364-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70539">
    <skos:narrower rdf:resource="descriptor/18363-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70539">
    <skos:narrower rdf:resource="descriptor/18358-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70539">
    <skos:narrower rdf:resource="descriptor/18374-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70539">
    <skos:narrower rdf:resource="descriptor/18367-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70539">
    <skos:narrower rdf:resource="descriptor/18458-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70539">
    <skos:narrower rdf:resource="descriptor/20888-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70539">
    <skos:narrower rdf:resource="descriptor/18362-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70539">
    <skos:narrower rdf:resource="descriptor/18370-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70539">
    <skos:narrower rdf:resource="descriptor/18373-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70539">
    <skos:narrower rdf:resource="descriptor/18371-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70539">
    <skos:narrower rdf:resource="descriptor/11812-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70539">
    <skos:narrower rdf:resource="descriptor/18376-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70539">
    <skos:narrower rdf:resource="descriptor/11811-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70539">
    <skos:narrower rdf:resource="descriptor/18812-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70539">
    <skos:narrower rdf:resource="descriptor/11810-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70539">
    <skos:narrower rdf:resource="descriptor/18360-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70539">
    <skos:narrower rdf:resource="descriptor/18346-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11207-2">
    <zbwext:useInsteadNote rdf:nodeID="id280"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18408-6">
    <skos:related rdf:resource="descriptor/18429-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18408-6">
    <skos:related rdf:resource="descriptor/14331-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18408-6">
    <skos:related rdf:resource="descriptor/12996-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10754-6">
    <skos:altLabel xml:lang="de">Leitwährung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13755-3">
    <skos:altLabel xml:lang="en">Cash dispenser</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13755-3">
    <skos:altLabel xml:lang="en">ATM (Automatic Teller Machine)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13755-3">
    <skos:altLabel xml:lang="de">Geldausgabeautomat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13755-3">
    <skos:altLabel xml:lang="de">Bargeldautomat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13755-3">
    <skos:altLabel xml:lang="de">Bank-Informationssystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13755-3">
    <skos:altLabel xml:lang="de">Bankinformatik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13755-3">
    <skos:altLabel xml:lang="de">Automatisierung von Banken</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13755-3">
    <skos:altLabel xml:lang="de">Automatischer Bankschalter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73299">
    <skos:broader rdf:resource="thsys/70498"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19330-5">
    <skos:prefLabel xml:lang="en">Agricultural tariff</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19330-5">
    <skos:prefLabel xml:lang="de">Agrarzoll</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14076-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18571-2">
    <skos:related rdf:resource="descriptor/18575-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18571-2">
    <skos:related rdf:resource="descriptor/18572-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14045-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14465-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14812-5">
    <skos:altLabel xml:lang="de">Verlagsprodukt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14812-5">
    <skos:altLabel xml:lang="de">Printmedium</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14812-5">
    <skos:altLabel xml:lang="de">Kunstdruck</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14812-5">
    <skos:altLabel xml:lang="de">Grußkarte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14812-5">
    <skos:altLabel xml:lang="de">Drucksache</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14812-5">
    <skos:altLabel xml:lang="de">Druckmedium</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18660-3">
    <skos:related rdf:resource="descriptor/12036-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18660-3">
    <skos:related rdf:resource="descriptor/18657-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18660-3">
    <skos:related rdf:resource="descriptor/19544-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18660-3">
    <skos:related rdf:resource="descriptor/16802-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12438-6">
    <skos:altLabel xml:lang="en">Due Diligence (Mergers &amp;amp; Acquisitions)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70486">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10533-1">
    <skos:broader rdf:resource="thsys/71086"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10533-1">
    <skos:broader rdf:resource="descriptor/11784-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11779-6">
    <skos:broader rdf:resource="thsys/73372"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11779-6">
    <skos:broader rdf:resource="thsys/71113"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11779-6">
    <skos:broader rdf:resource="descriptor/10987-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13075-4">
    <skos:related rdf:resource="descriptor/14502-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71098">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17703-2">
    <skos:broader rdf:resource="thsys/70560"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17703-2">
    <skos:broader rdf:resource="descriptor/17724-1"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id86">
    <rdf:value xml:lang="de">Für Entwicklungshilfe von privaten Trägern</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14996-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13294-6">
    <skos:narrower rdf:resource="descriptor/18739-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13294-6">
    <skos:narrower rdf:resource="descriptor/18105-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14068-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10272-3">
    <skos:altLabel xml:lang="de">Volkswirtschaftliches Rechnungswesen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10272-3">
    <skos:altLabel xml:lang="de">Volkswirtschaftliches Kontensystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10272-3">
    <skos:altLabel xml:lang="de">Volkswirtschaftliches Bilanzsystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10272-3">
    <skos:altLabel xml:lang="de">VGR (Volkswirtschaftliche Gesamtrechnung)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10272-3">
    <skos:altLabel xml:lang="de">Nationale Buchführung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10272-3">
    <skos:altLabel xml:lang="de">Gesamtwirtschaftliches Rechnungswesen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70241">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18012-3">
    <skos:scopeNote xml:lang="de">Für die Bundesrepublik Deutschland ab 1949 sowie für zeitlich und räumlich die politischen Einheiten Deutschlands übergreifende Darstellungen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14215-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18270-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71109">
    <skos:narrower rdf:resource="descriptor/18869-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71109">
    <skos:narrower rdf:resource="descriptor/16487-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71109">
    <skos:narrower rdf:resource="descriptor/19253-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71109">
    <skos:narrower rdf:resource="descriptor/18031-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71109">
    <skos:narrower rdf:resource="descriptor/10473-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71109">
    <skos:narrower rdf:resource="descriptor/10472-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71109">
    <skos:narrower rdf:resource="descriptor/10470-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71109">
    <skos:narrower rdf:resource="descriptor/10469-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71109">
    <skos:narrower rdf:resource="descriptor/10466-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71109">
    <skos:narrower rdf:resource="descriptor/10465-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71109">
    <skos:narrower rdf:resource="descriptor/16696-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71109">
    <skos:narrower rdf:resource="descriptor/11931-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71109">
    <skos:narrower rdf:resource="descriptor/19252-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71109">
    <skos:narrower rdf:resource="descriptor/10463-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71109">
    <skos:narrower rdf:resource="descriptor/10481-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71109">
    <skos:narrower rdf:resource="descriptor/10478-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71109">
    <skos:narrower rdf:resource="descriptor/10458-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71109">
    <skos:narrower rdf:resource="descriptor/10455-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71109">
    <skos:narrower rdf:resource="descriptor/10454-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71109">
    <skos:narrower rdf:resource="descriptor/18065-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71109">
    <skos:narrower rdf:resource="descriptor/10125-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71109">
    <skos:narrower rdf:resource="descriptor/10452-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71109">
    <skos:narrower rdf:resource="descriptor/19251-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71109">
    <skos:narrower rdf:resource="descriptor/18082-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71109">
    <skos:narrower rdf:resource="descriptor/10459-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71109">
    <skos:narrower rdf:resource="descriptor/18461-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71109">
    <skos:narrower rdf:resource="descriptor/10464-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71109">
    <skos:narrower rdf:resource="descriptor/10450-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71109">
    <skos:narrower rdf:resource="descriptor/18785-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71109">
    <skos:narrower rdf:resource="descriptor/18775-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71109">
    <skos:narrower rdf:resource="descriptor/18784-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71109">
    <skos:narrower rdf:resource="descriptor/10437-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71109">
    <skos:narrower rdf:resource="descriptor/10436-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71109">
    <skos:narrower rdf:resource="descriptor/10475-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71109">
    <skos:narrower rdf:resource="descriptor/10446-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71109">
    <skos:narrower rdf:resource="descriptor/15635-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71109">
    <skos:narrower rdf:resource="descriptor/10443-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71109">
    <skos:narrower rdf:resource="descriptor/10487-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12508-4">
    <skos:altLabel xml:lang="de">Durchschnittskosten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11986-1">
    <skos:altLabel xml:lang="de">Urbanistik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11986-1">
    <skos:altLabel xml:lang="de">Empirische Stadtforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19062-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11451-5">
    <skos:related rdf:resource="descriptor/13767-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11451-5">
    <skos:related rdf:resource="descriptor/13804-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11451-5">
    <skos:related rdf:resource="descriptor/19093-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11451-5">
    <skos:related rdf:resource="descriptor/11460-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11451-5">
    <skos:related rdf:resource="descriptor/11448-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11451-5">
    <skos:related rdf:resource="descriptor/11415-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19796-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19560-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12168-2">
    <skos:prefLabel xml:lang="en">Informational efficiency</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12168-2">
    <skos:prefLabel xml:lang="de">Informationseffizienz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14480-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14930-6">
    <skos:narrower rdf:resource="descriptor/14947-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14930-6">
    <skos:narrower rdf:resource="descriptor/14967-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17476-1">
    <skos:altLabel xml:lang="de">Republik Lettland</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17476-1">
    <skos:altLabel xml:lang="de">Lettische SSR</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17476-1">
    <skos:altLabel xml:lang="de">Lettische Republik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70005">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19145-0">
    <skos:broader rdf:resource="thsys/71037"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19145-0">
    <skos:broader rdf:resource="descriptor/18664-2"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id308">
    <rdf:value xml:lang="de">für Stadtentwicklungsplanung</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14273-2">
    <skos:altLabel xml:lang="de">Altmetall</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15119-0">
    <skos:altLabel xml:lang="de">Laufende Stichprobe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13583-6">
    <skos:broader rdf:resource="thsys/70943"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12197-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18098-2">
    <skos:related rdf:resource="descriptor/18099-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18098-2">
    <skos:related rdf:resource="descriptor/18936-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18418-3">
    <skos:prefLabel xml:lang="en">Aircraft construction research</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18418-3">
    <skos:prefLabel xml:lang="de">Luftfahrtforschung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10178-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id240">
    <rdfs:seeAlso rdf:resource="descriptor/11142-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19172-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12897-2">
    <skos:related rdf:resource="descriptor/12903-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20261-2">
    <skos:altLabel xml:lang="en">Stakeholder value</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20261-2">
    <skos:altLabel xml:lang="en">Stakeholder theory</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20261-2">
    <skos:altLabel xml:lang="de">Anspruchspartner</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20261-2">
    <skos:altLabel xml:lang="de">Anspruchsgruppen-Konzept</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70373">
    <skos:narrower rdf:resource="descriptor/14583-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70373">
    <skos:narrower rdf:resource="descriptor/14525-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70373">
    <skos:narrower rdf:resource="descriptor/14524-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70373">
    <skos:narrower rdf:resource="descriptor/14522-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70373">
    <skos:narrower rdf:resource="descriptor/14515-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70373">
    <skos:narrower rdf:resource="descriptor/14513-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70373">
    <skos:narrower rdf:resource="descriptor/14518-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70373">
    <skos:narrower rdf:resource="descriptor/14516-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11071-1">
    <skos:related rdf:resource="descriptor/11765-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11071-1">
    <skos:related rdf:resource="descriptor/11073-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11071-1">
    <skos:related rdf:resource="descriptor/11079-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19322-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15958-0">
    <skos:broader rdf:resource="thsys/70572"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15958-0">
    <skos:broader rdf:resource="descriptor/15709-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19047-0">
    <skos:altLabel xml:lang="de">Mineralölförderservice</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19047-0">
    <skos:altLabel xml:lang="de">Erdgasförderservice</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18510-1">
    <skos:prefLabel xml:lang="en">Craft cooperative</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18510-1">
    <skos:prefLabel xml:lang="de">Handwerksgenossenschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16485-5">
    <skos:narrower rdf:resource="descriptor/11839-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70070">
    <skos:prefLabel xml:lang="en">P.01.01.01  Grain</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70070">
    <skos:prefLabel xml:lang="de">P.01.01.01  Getreide</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18696-3">
    <skos:related rdf:resource="descriptor/18698-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18696-3">
    <skos:related rdf:resource="descriptor/12251-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19790-6">
    <skos:altLabel xml:lang="en">Epidemiology</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19790-6">
    <skos:altLabel xml:lang="de">Seuche</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19790-6">
    <skos:altLabel xml:lang="de">Pest</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19790-6">
    <skos:altLabel xml:lang="de">Pandemie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19790-6">
    <skos:altLabel xml:lang="de">Epidemiologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19790-6">
    <skos:altLabel xml:lang="de">Endemie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19790-6">
    <skos:altLabel xml:lang="de">Cholera</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11352-0">
    <skos:related rdf:resource="descriptor/11357-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17503-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73103">
    <skos:prefLabel xml:lang="en">V.18  Decision theory</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73103">
    <skos:prefLabel xml:lang="de">V.18  Entscheidungstheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13594-1">
    <skos:related rdf:resource="descriptor/13588-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13594-1">
    <skos:related rdf:resource="descriptor/13595-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13594-1">
    <skos:related rdf:resource="descriptor/13593-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13594-1">
    <skos:related rdf:resource="descriptor/15709-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13594-1">
    <skos:related rdf:resource="descriptor/11863-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16280-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14024-2">
    <zbwext:useInsteadNote rdf:nodeID="id60"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71041">
    <skos:narrower rdf:resource="descriptor/12903-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71041">
    <skos:narrower rdf:resource="descriptor/16120-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71041">
    <skos:narrower rdf:resource="descriptor/18184-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71041">
    <skos:narrower rdf:resource="descriptor/12880-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71041">
    <skos:narrower rdf:resource="descriptor/19106-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71041">
    <skos:narrower rdf:resource="descriptor/12882-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71041">
    <skos:narrower rdf:resource="descriptor/19001-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71041">
    <skos:narrower rdf:resource="descriptor/12898-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71041">
    <skos:narrower rdf:resource="descriptor/12891-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71041">
    <skos:narrower rdf:resource="descriptor/12897-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71041">
    <skos:narrower rdf:resource="descriptor/19104-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71041">
    <skos:narrower rdf:resource="descriptor/12884-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71041">
    <skos:narrower rdf:resource="descriptor/12883-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71041">
    <skos:narrower rdf:resource="descriptor/12896-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18894-6">
    <skos:altLabel xml:lang="de">Medizinische Basisversorgung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id306">
    <rdfs:seeAlso rdf:resource="descriptor/11998-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15935-5">
    <skos:related rdf:resource="descriptor/15930-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18121-5">
    <skos:broader rdf:resource="thsys/71090"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18121-5">
    <skos:broader rdf:resource="thsys/70140"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18121-5">
    <skos:broader rdf:resource="descriptor/12036-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18121-5">
    <skos:broader rdf:resource="descriptor/10617-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16397-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18336-5">
    <skos:related rdf:resource="descriptor/15897-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18336-5">
    <skos:related rdf:resource="descriptor/18338-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18336-5">
    <skos:related rdf:resource="descriptor/12021-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10008-6">
    <skos:narrower rdf:resource="descriptor/15771-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10008-6">
    <skos:narrower rdf:resource="descriptor/12294-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10008-6">
    <skos:narrower rdf:resource="descriptor/11556-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10008-6">
    <skos:narrower rdf:resource="descriptor/10297-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10008-6">
    <skos:narrower rdf:resource="descriptor/10823-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10008-6">
    <skos:narrower rdf:resource="descriptor/18505-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10008-6">
    <skos:narrower rdf:resource="descriptor/10287-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17672-1">
    <skos:related rdf:resource="descriptor/19175-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16124-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15760-0">
    <skos:narrower rdf:resource="descriptor/18072-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15760-0">
    <skos:narrower rdf:resource="descriptor/18957-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19717-2">
    <skos:related rdf:resource="descriptor/10574-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19717-2">
    <skos:related rdf:resource="descriptor/19709-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19059-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73364">
    <skos:broader rdf:resource="thsys/73366"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11219-2">
    <skos:altLabel xml:lang="en">Labour economics</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11219-2">
    <skos:altLabel xml:lang="en">Labor market theory</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11219-2">
    <skos:altLabel xml:lang="en">Labor economics</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11219-2">
    <skos:altLabel xml:lang="de">Beschäftigungstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11219-2">
    <skos:altLabel xml:lang="de">Arbeitsökonomik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11219-2">
    <skos:altLabel xml:lang="de">Arbeitsökonomie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11219-2">
    <skos:altLabel xml:lang="de">Arbeitsmarktmodell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11219-2">
    <skos:altLabel xml:lang="de">Arbeitsmarktgleichgewicht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20277-1">
    <skos:altLabel xml:lang="en">Simultaneous engineering</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20277-1">
    <skos:altLabel xml:lang="en">Overlapping engineering</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20277-1">
    <skos:altLabel xml:lang="en">Concurrent engineering</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11875-3">
    <skos:narrower rdf:resource="descriptor/11880-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11875-3">
    <skos:narrower rdf:resource="descriptor/11910-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11875-3">
    <skos:narrower rdf:resource="descriptor/19224-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11875-3">
    <skos:narrower rdf:resource="descriptor/19440-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70424">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">P.08.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11790-4">
    <skos:altLabel xml:lang="en">Structural change</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11790-4">
    <skos:altLabel xml:lang="de">Wirtschaftsstrukturwandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11790-4">
    <skos:altLabel xml:lang="de">Sektoraler Wandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11790-4">
    <skos:altLabel xml:lang="de">Sektoraler Strukturwandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11790-4">
    <skos:altLabel xml:lang="de">Sektorale Strukturverschiebung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11790-4">
    <skos:altLabel xml:lang="de">Sectoral Shift</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11790-4">
    <skos:altLabel xml:lang="de">Intrasektoraler Strukturwandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11790-4">
    <skos:altLabel xml:lang="de">Intersektoraler Strukturwandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11790-4">
    <skos:altLabel xml:lang="de">Branchenstrukturwandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10647-0">
    <skos:prefLabel xml:lang="en">Effective protection</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10647-0">
    <skos:prefLabel xml:lang="de">Effektive Protektion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16042-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18516-3">
    <skos:broader rdf:resource="thsys/70960"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18516-3">
    <skos:broader rdf:resource="descriptor/13188-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18516-3">
    <skos:broader rdf:resource="descriptor/13198-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14998-0">
    <skos:related rdf:resource="descriptor/18167-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10310-0">
    <skos:prefLabel xml:lang="en">Single product production</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10310-0">
    <skos:prefLabel xml:lang="de">Einproduktproduktion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19743-1">
    <skos:broader rdf:resource="thsys/73393"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19743-1">
    <skos:broader rdf:resource="descriptor/16727-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15405-6">
    <skos:related rdf:resource="descriptor/15320-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70989">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10411-1">
    <skos:altLabel xml:lang="de">Wachstumsgrenze</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73377">
    <skos:narrower rdf:resource="descriptor/10993-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73377">
    <skos:narrower rdf:resource="descriptor/16623-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73377">
    <skos:narrower rdf:resource="descriptor/16432-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73377">
    <skos:narrower rdf:resource="descriptor/16430-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73377">
    <skos:narrower rdf:resource="descriptor/16429-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73377">
    <skos:narrower rdf:resource="descriptor/18830-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73377">
    <skos:narrower rdf:resource="descriptor/16427-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73377">
    <skos:narrower rdf:resource="descriptor/16426-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73377">
    <skos:narrower rdf:resource="descriptor/11400-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19008-3">
    <skos:related rdf:resource="descriptor/12564-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18961-3">
    <skos:broader rdf:resource="thsys/73331"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18961-3">
    <skos:broader rdf:resource="thsys/70818"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18961-3">
    <skos:broader rdf:resource="descriptor/16182-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18961-3">
    <skos:broader rdf:resource="descriptor/15988-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15730-2">
    <skos:altLabel xml:lang="de">Abwassergebühr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17184-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10242-5">
    <skos:prefLabel xml:lang="en">Theory of inflation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10242-5">
    <skos:prefLabel xml:lang="de">Inflationstheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12580-3">
    <skos:broader rdf:resource="thsys/73385"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12580-3">
    <skos:broader rdf:resource="thsys/70244"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12580-3">
    <skos:broader rdf:resource="descriptor/12585-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16335-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18820-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18736-3">
    <skos:related rdf:resource="descriptor/16201-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18736-3">
    <skos:related rdf:resource="descriptor/16112-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18736-3">
    <skos:related rdf:resource="descriptor/13729-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19571-4">
    <skos:prefLabel xml:lang="en">Martingale</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19571-4">
    <skos:prefLabel xml:lang="de">Martingale</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17596-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12941-4">
    <skos:prefLabel xml:lang="en">Aviculture</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12941-4">
    <skos:prefLabel xml:lang="de">Geflügelhaltung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11367-1">
    <skos:editorialNote xml:lang="de">Möglichst spezifischer indexieren.</skos:editorialNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15919-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70161">
    <skos:prefLabel xml:lang="de">P.23  Abfall und Sekundärrohstoffe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19264-6">
    <skos:prefLabel xml:lang="en">Floater</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19264-6">
    <skos:prefLabel xml:lang="de">Zinsvariables Papier</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10139-5">
    <skos:related rdf:resource="descriptor/15346-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10139-5">
    <skos:related rdf:resource="descriptor/15411-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10139-5">
    <skos:related rdf:resource="descriptor/15408-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13338-5">
    <skos:related rdf:resource="descriptor/14626-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18198-5">
    <skos:narrower rdf:resource="descriptor/19302-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18198-5">
    <skos:narrower rdf:resource="descriptor/18197-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18198-5">
    <skos:narrower rdf:resource="descriptor/18981-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11371-3">
    <skos:altLabel xml:lang="en">Distance training</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11371-3">
    <skos:altLabel xml:lang="en">Distance education</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11371-3">
    <skos:altLabel xml:lang="en">Correspondence course</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11371-3">
    <skos:altLabel xml:lang="en">Correspondance education</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11371-3">
    <skos:altLabel xml:lang="de">Fernuniversität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11371-3">
    <skos:altLabel xml:lang="de">Fernstudium</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11371-3">
    <skos:altLabel xml:lang="de">Fernstudien</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11371-3">
    <skos:altLabel xml:lang="de">Fernlehrgang</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70844">
    <skos:broader rdf:resource="thsys/70847"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15475-6">
    <skos:prefLabel xml:lang="en">Sequential Decision Making</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15475-6">
    <skos:prefLabel xml:lang="de">Mehrstufige Entscheidung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10112-4">
    <skos:altLabel xml:lang="en">Welfare measurement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10112-4">
    <skos:altLabel xml:lang="de">Wohlstandsmaß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10112-4">
    <skos:altLabel xml:lang="de">Wohlfahrtsmessung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10112-4">
    <skos:altLabel xml:lang="de">Wohlfahrtsmaß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18900-2">
    <skos:broader rdf:resource="thsys/73356"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18900-2">
    <skos:broader rdf:resource="thsys/73346"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18900-2">
    <skos:broader rdf:resource="thsys/73294"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18900-2">
    <skos:broader rdf:resource="descriptor/16668-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14803-6">
    <skos:narrower rdf:resource="descriptor/14808-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14803-6">
    <skos:narrower rdf:resource="descriptor/14807-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14803-6">
    <skos:narrower rdf:resource="descriptor/14804-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14803-6">
    <skos:narrower rdf:resource="descriptor/14798-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10870-4">
    <skos:prefLabel xml:lang="en">Forfaiting</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10870-4">
    <skos:prefLabel xml:lang="de">Forfaitierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14214-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73305">
    <skos:broader rdf:resource="thsys/70135"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10424-6">
    <skos:related rdf:resource="descriptor/10113-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10424-6">
    <skos:related rdf:resource="descriptor/10432-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10424-6">
    <skos:related rdf:resource="descriptor/10427-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10206-2">
    <skos:prefLabel xml:lang="en">Monopsony</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10206-2">
    <skos:prefLabel xml:lang="de">Monopson</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10735-3">
    <skos:related rdf:resource="descriptor/10736-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10735-3">
    <skos:related rdf:resource="descriptor/10728-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12172-4">
    <skos:altLabel xml:lang="de">Betriebsinformatik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17176-6">
    <skos:scopeNote xml:lang="de">Existierte bis Ende 1990. Nachfolgestaaten: Serbien-Montenegro, Bosnien-Herzegowina, Kroatien, Mazedonien, Slowenien.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70884">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.17.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18945-1">
    <skos:prefLabel xml:lang="en">Library policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18945-1">
    <skos:prefLabel xml:lang="de">Bibliothekspolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17972-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18603-1">
    <skos:broader rdf:resource="thsys/70933"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18603-1">
    <skos:broader rdf:resource="descriptor/13264-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21110-4">
    <skos:prefLabel xml:lang="en">Weapon</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21110-4">
    <skos:prefLabel xml:lang="de">Waffe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24706-5">
    <skos:related rdf:resource="descriptor/13801-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24706-5">
    <skos:related rdf:resource="descriptor/18271-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18077-3">
    <skos:broader rdf:resource="thsys/70201"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18077-3">
    <skos:broader rdf:resource="descriptor/14961-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16292-2">
    <skos:broader rdf:resource="thsys/73382"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14851-2">
    <skos:prefLabel xml:lang="en">Natural fibre</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14851-2">
    <skos:prefLabel xml:lang="de">Naturfaser</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16455-0">
    <skos:related rdf:resource="descriptor/16463-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16455-0">
    <skos:related rdf:resource="descriptor/19708-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16455-0">
    <skos:related rdf:resource="descriptor/16454-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16455-0">
    <skos:related rdf:resource="descriptor/16497-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16342-6">
    <skos:prefLabel xml:lang="en">Parliamentary system</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16342-6">
    <skos:prefLabel xml:lang="de">Parlamentarismus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18622-4">
    <skos:related rdf:resource="descriptor/13615-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12291-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14003-3">
    <skos:related rdf:resource="descriptor/19067-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14003-3">
    <skos:related rdf:resource="descriptor/19775-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14003-3">
    <skos:related rdf:resource="descriptor/15642-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71024">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.01.03</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14088-4">
    <skos:broader rdf:resource="thsys/70070"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14088-4">
    <skos:broader rdf:resource="descriptor/14074-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19090-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18672-3">
    <skos:prefLabel xml:lang="en">Credit card industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18672-3">
    <skos:prefLabel xml:lang="de">Kreditkartenorganisation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19466-1">
    <zbwext:useInsteadNote rdf:nodeID="id335"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10510-6">
    <skos:altLabel xml:lang="de">Städtische Entwicklung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18141-6">
    <skos:prefLabel xml:lang="en">Off-balance-sheet banking</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18141-6">
    <skos:prefLabel xml:lang="de">Außerbilanzgeschäft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19554-4">
    <skos:scopeNote xml:lang="de">Aktivitäten an Börsen, die durch Programmhandel und spekulative Attacken erzeugt werden.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14366-2">
    <skos:narrower rdf:resource="descriptor/14365-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16119-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11988-4">
    <skos:altLabel xml:lang="de">Optimale Stadtgröße</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11988-4">
    <skos:altLabel xml:lang="de">Gemeindegröße</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19751-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10891-3">
    <skos:altLabel xml:lang="en">Economics of industry</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10891-3">
    <skos:altLabel xml:lang="en">Economics of industrial organization</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10891-3">
    <skos:altLabel xml:lang="de">Marktökonomie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10891-3">
    <skos:altLabel xml:lang="de">Industrieökonomie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11105-3">
    <skos:related rdf:resource="descriptor/12697-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11105-3">
    <skos:related rdf:resource="descriptor/11127-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11105-3">
    <skos:related rdf:resource="descriptor/11049-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11105-3">
    <skos:related rdf:resource="descriptor/16588-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18513-2">
    <skos:prefLabel xml:lang="en">Commercial agents law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18513-2">
    <skos:prefLabel xml:lang="de">Handelsvertreterrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12845-0">
    <skos:broader rdf:resource="thsys/70874"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11589-4">
    <skos:prefLabel xml:lang="en">Public Accounting</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11589-4">
    <skos:prefLabel xml:lang="de">Öffentliches Rechnungswesen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15904-2">
    <skos:related rdf:resource="descriptor/19637-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15904-2">
    <skos:related rdf:resource="descriptor/18180-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15904-2">
    <skos:related rdf:resource="descriptor/19760-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15904-2">
    <skos:related rdf:resource="descriptor/16699-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15904-2">
    <skos:related rdf:resource="descriptor/16697-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15904-2">
    <skos:related rdf:resource="descriptor/19638-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15904-2">
    <skos:related rdf:resource="descriptor/16475-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11355-1">
    <skos:narrower rdf:resource="descriptor/11356-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15611-3">
    <skos:narrower rdf:resource="descriptor/13277-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15611-3">
    <skos:narrower rdf:resource="descriptor/19807-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15611-3">
    <skos:narrower rdf:resource="descriptor/18170-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19746-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70647">
    <skos:prefLabel xml:lang="de">P.06.01  Keramische Produkte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11313-3">
    <skos:prefLabel xml:lang="en">Trade union</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11313-3">
    <skos:prefLabel xml:lang="de">Gewerkschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15797-5">
    <skos:prefLabel xml:lang="en">Ethnopsychology</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15797-5">
    <skos:prefLabel xml:lang="de">Ethnopsychologie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15600-1">
    <skos:altLabel xml:lang="de">Dateneingabe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70839">
    <skos:prefLabel xml:lang="de">W.25  Gesundheits- und Veterinärwesen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16201-3">
    <skos:narrower rdf:resource="descriptor/11209-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16201-3">
    <skos:narrower rdf:resource="descriptor/18066-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16201-3">
    <skos:narrower rdf:resource="descriptor/16191-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16201-3">
    <skos:narrower rdf:resource="descriptor/19815-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16201-3">
    <skos:narrower rdf:resource="descriptor/16190-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19232-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24855-2">
    <skos:related rdf:resource="descriptor/18679-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24855-2">
    <skos:related rdf:resource="descriptor/10671-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18686-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10251-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10194-4">
    <skos:altLabel xml:lang="en">Private information</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10194-4">
    <skos:altLabel xml:lang="en">Islands model</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14170-5">
    <skos:broader rdf:resource="thsys/70403"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14170-5">
    <skos:broader rdf:resource="descriptor/14169-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24664-2">
    <zbwext:useInsteadNote rdf:nodeID="id71"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19438-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18304-4">
    <skos:broader rdf:resource="thsys/70333"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18304-4">
    <skos:broader rdf:resource="descriptor/14051-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16036-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13238-2">
    <skos:prefLabel xml:lang="en">Tobacco trade</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13238-2">
    <skos:prefLabel xml:lang="de">Tabakhandel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16215-6">
    <skos:prefLabel xml:lang="en">Country studies</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16215-6">
    <skos:prefLabel xml:lang="de">Länderforschung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12112-1">
    <skos:broader rdf:resource="thsys/70562"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12112-1">
    <skos:broader rdf:resource="thsys/70011"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12112-1">
    <skos:broader rdf:resource="descriptor/12106-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19609-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18455-4">
    <skos:broader rdf:resource="thsys/70201"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19573-0">
    <skos:prefLabel xml:lang="en">VAR model</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19573-0">
    <skos:prefLabel xml:lang="de">VAR-Modell</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11564-6">
    <skos:broader rdf:resource="thsys/71113"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11564-6">
    <skos:broader rdf:resource="thsys/71080"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11564-6">
    <skos:broader rdf:resource="thsys/70833"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11564-6">
    <skos:broader rdf:resource="thsys/70324"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11564-6">
    <skos:broader rdf:resource="descriptor/16477-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14817-2">
    <skos:related rdf:resource="descriptor/14833-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14817-2">
    <skos:related rdf:resource="descriptor/14042-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14817-2">
    <skos:related rdf:resource="descriptor/14151-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14817-2">
    <skos:related rdf:resource="descriptor/13004-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14817-2">
    <skos:related rdf:resource="descriptor/14724-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13811-5">
    <skos:altLabel xml:lang="de">VVaG</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13811-5">
    <skos:altLabel xml:lang="de">Vertragsversicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13811-5">
    <skos:altLabel xml:lang="de">Versicherungsverein auf Gegenseitigkeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13811-5">
    <skos:altLabel xml:lang="de">Private Versicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13811-5">
    <skos:altLabel xml:lang="de">Individualversicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14698-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16477-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19093-0">
    <skos:broader rdf:resource="thsys/71043"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19093-0">
    <skos:broader rdf:resource="descriptor/12399-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17804-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19322-4">
    <skos:prefLabel xml:lang="en">Open-door policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19322-4">
    <skos:prefLabel xml:lang="de">Politik der offenen Tür</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14969-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11975-6">
    <skos:broader rdf:resource="thsys/71064"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11975-6">
    <skos:broader rdf:resource="descriptor/11973-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19422-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70140">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12051-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17602-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10209-3">
    <skos:prefLabel xml:lang="en">Oligopoly</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10209-3">
    <skos:prefLabel xml:lang="de">Oligopol</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13980-3">
    <skos:related rdf:resource="descriptor/18607-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13980-3">
    <skos:related rdf:resource="descriptor/13247-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13980-3">
    <skos:related rdf:resource="descriptor/13246-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13980-3">
    <skos:related rdf:resource="descriptor/13975-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10704-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19253-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19128-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19657-1">
    <skos:prefLabel xml:lang="en">Interactive media</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19657-1">
    <skos:prefLabel xml:lang="de">Interaktive Medien</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70866">
    <skos:broader rdf:resource="thsys/70871"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18604-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17978-5">
    <skos:altLabel xml:lang="de">Arabische Länder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16030-4">
    <skos:narrower rdf:resource="descriptor/19839-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16030-4">
    <skos:narrower rdf:resource="descriptor/16027-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12501-4">
    <skos:narrower rdf:resource="descriptor/12502-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70959">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15690-2">
    <skos:broader rdf:resource="thsys/70007"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14023-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14023-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11752-5">
    <skos:broader rdf:resource="thsys/71114"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11752-5">
    <skos:broader rdf:resource="descriptor/15719-4"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id40">
    <rdfs:seeAlso rdf:resource="descriptor/10138-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15909-6">
    <skos:related rdf:resource="descriptor/15714-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15909-6">
    <skos:related rdf:resource="descriptor/15659-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15909-6">
    <skos:related rdf:resource="descriptor/15916-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15909-6">
    <skos:related rdf:resource="descriptor/15914-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15909-6">
    <skos:related rdf:resource="descriptor/15061-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15909-6">
    <skos:related rdf:resource="descriptor/15912-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15909-6">
    <skos:related rdf:resource="descriptor/15911-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15909-6">
    <skos:related rdf:resource="descriptor/15708-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15909-6">
    <skos:related rdf:resource="descriptor/15910-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19119-1">
    <skos:prefLabel xml:lang="en">Key-account management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19119-1">
    <skos:prefLabel xml:lang="de">Key-Account Management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15920-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13702-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70844">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19351-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71026">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11843-2">
    <skos:altLabel xml:lang="en">Public housing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11843-2">
    <skos:altLabel xml:lang="en">Low-income housing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11843-2">
    <skos:altLabel xml:lang="en">Low income housing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11843-2">
    <skos:altLabel xml:lang="de">Sozialwohnungsbau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11843-2">
    <skos:altLabel xml:lang="de">Sozialwohnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11843-2">
    <skos:altLabel xml:lang="de">Förderung des sozialen Wohnungsbaus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18698-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18363-2">
    <skos:prefLabel xml:lang="en">Solar energy policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18363-2">
    <skos:prefLabel xml:lang="de">Sonnenenergiepolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10907-0">
    <skos:prefLabel xml:lang="en">Buyer power</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10907-0">
    <skos:prefLabel xml:lang="de">Nachfragemacht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17791-0">
    <skos:narrower rdf:resource="descriptor/17804-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17791-0">
    <skos:narrower rdf:resource="descriptor/17801-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17791-0">
    <skos:narrower rdf:resource="descriptor/17800-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17791-0">
    <skos:narrower rdf:resource="descriptor/17799-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17791-0">
    <skos:narrower rdf:resource="descriptor/17793-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17791-0">
    <skos:narrower rdf:resource="descriptor/17792-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17791-0">
    <skos:narrower rdf:resource="descriptor/17790-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17791-0">
    <skos:narrower rdf:resource="descriptor/17787-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17791-0">
    <skos:narrower rdf:resource="descriptor/17785-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17791-0">
    <skos:narrower rdf:resource="descriptor/17784-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17791-0">
    <skos:narrower rdf:resource="descriptor/17783-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17791-0">
    <skos:narrower rdf:resource="descriptor/17782-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17791-0">
    <skos:narrower rdf:resource="descriptor/17781-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17791-0">
    <skos:narrower rdf:resource="descriptor/17780-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17791-0">
    <skos:narrower rdf:resource="descriptor/17779-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17791-0">
    <skos:narrower rdf:resource="descriptor/17778-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17791-0">
    <skos:narrower rdf:resource="descriptor/17777-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17791-0">
    <skos:narrower rdf:resource="descriptor/17773-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17791-0">
    <skos:narrower rdf:resource="descriptor/17772-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17791-0">
    <skos:narrower rdf:resource="descriptor/17771-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17791-0">
    <skos:narrower rdf:resource="descriptor/17770-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11203-3">
    <skos:scopeNote xml:lang="de">Gewerbliche Arbeitnehmerüberlassung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14299-5">
    <skos:related rdf:resource="descriptor/19185-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14299-5">
    <skos:related rdf:resource="descriptor/13033-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14299-5">
    <skos:related rdf:resource="descriptor/18392-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14082-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10537-0">
    <skos:prefLabel xml:lang="en">Stage of economic development</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10537-0">
    <skos:prefLabel xml:lang="de">Entwicklungsstufe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10776-3">
    <skos:prefLabel xml:lang="en">Foreign exchange</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10776-3">
    <skos:prefLabel xml:lang="de">Devisen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15598-4">
    <skos:related rdf:resource="descriptor/16757-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15598-4">
    <skos:related rdf:resource="descriptor/16754-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15598-4">
    <skos:related rdf:resource="descriptor/15606-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15483-0">
    <skos:prefLabel xml:lang="en">Operations research</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15483-0">
    <skos:prefLabel xml:lang="de">Operations Research</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11660-3">
    <skos:broader rdf:resource="thsys/71075"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11660-3">
    <skos:broader rdf:resource="thsys/70814"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11660-3">
    <skos:broader rdf:resource="descriptor/11704-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11660-3">
    <skos:broader rdf:resource="descriptor/16477-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13681-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18613-5">
    <skos:altLabel xml:lang="de">Gift</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18613-5">
    <skos:altLabel xml:lang="de">Gefahrstoff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18613-5">
    <skos:altLabel xml:lang="de">Gefahrenstoff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16421-3">
    <zbwext:useInsteadNote rdf:nodeID="id281"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14527-4">
    <skos:prefLabel xml:lang="en">Spectacles</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14527-4">
    <skos:prefLabel xml:lang="de">Augenoptisches Produkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73354">
    <rdfs:label xml:lang="de">Geowissenschaften und Umwelt</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14412-0">
    <skos:broader rdf:resource="thsys/70258"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14412-0">
    <skos:broader rdf:resource="descriptor/14433-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15640-3">
    <skos:broader rdf:resource="thsys/73388"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15640-3">
    <skos:broader rdf:resource="thsys/70850"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15640-3">
    <skos:broader rdf:resource="descriptor/15671-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11740-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18139-0">
    <skos:prefLabel xml:lang="en">Computer operating system</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18139-0">
    <skos:prefLabel xml:lang="de">Betriebssystem</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11880-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19343-3">
    <skos:broader rdf:resource="thsys/71025"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19343-3">
    <skos:broader rdf:resource="thsys/70960"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19343-3">
    <skos:broader rdf:resource="descriptor/19347-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19015-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17777-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14431-3">
    <skos:altLabel xml:lang="de">Elektromobil</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14431-3">
    <skos:altLabel xml:lang="de">Elektroauto</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16774-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70477">
    <skos:broader rdf:resource="thsys/70135"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10823-6">
    <skos:prefLabel xml:lang="en">Foreign direct investment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10823-6">
    <skos:prefLabel xml:lang="de">Direktinvestition</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11392-2">
    <skos:broader rdf:resource="thsys/73369"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11392-2">
    <skos:broader rdf:resource="thsys/70843"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11392-2">
    <skos:broader rdf:resource="thsys/70176"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11392-2">
    <skos:broader rdf:resource="descriptor/11397-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13131-6">
    <skos:related rdf:resource="descriptor/14895-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18256-3">
    <skos:scopeNote xml:lang="de">Sowohl der Abbau von Schwefel im Bergbau als auch die Gewinnung von Schwefel in der chemischen Grundstoffindustrie.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18186-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71111">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.03.06</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17954-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15764-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70937">
    <skos:broader rdf:resource="thsys/70938"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19706-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19315-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18605-4">
    <skos:prefLabel xml:lang="en">International air law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18605-4">
    <skos:prefLabel xml:lang="de">Internationales Luftrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19524-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13987-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14453-0">
    <skos:related rdf:resource="descriptor/14463-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11677-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19774-4">
    <skos:related rdf:resource="descriptor/19839-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19774-4">
    <skos:related rdf:resource="descriptor/18969-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19774-4">
    <skos:related rdf:resource="descriptor/16030-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19774-4">
    <skos:related rdf:resource="descriptor/15937-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19774-4">
    <skos:related rdf:resource="descriptor/18968-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12205-1">
    <skos:related rdf:resource="descriptor/12259-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12205-1">
    <skos:related rdf:resource="descriptor/12262-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12205-1">
    <skos:related rdf:resource="descriptor/19557-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12205-1">
    <skos:related rdf:resource="descriptor/20018-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12205-1">
    <skos:related rdf:resource="descriptor/18784-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12205-1">
    <skos:related rdf:resource="descriptor/12228-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12205-1">
    <skos:related rdf:resource="descriptor/12215-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12205-1">
    <skos:related rdf:resource="descriptor/10854-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11580-1">
    <skos:prefLabel xml:lang="en">Contract award regulation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11580-1">
    <skos:prefLabel xml:lang="de">Vergaberecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13249-4">
    <skos:prefLabel xml:lang="en">Public Transport</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13249-4">
    <skos:prefLabel xml:lang="de">Öffentlicher Personennahverkehr</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17962-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19245-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12180-5">
    <skos:broader rdf:resource="thsys/70854"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12180-5">
    <skos:broader rdf:resource="thsys/70087"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12180-5">
    <skos:broader rdf:resource="descriptor/12194-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12180-5">
    <skos:broader rdf:resource="descriptor/11825-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18063-0">
    <skos:related rdf:resource="descriptor/18913-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18063-0">
    <skos:related rdf:resource="descriptor/11555-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18063-0">
    <skos:related rdf:resource="descriptor/19517-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18063-0">
    <skos:related rdf:resource="descriptor/13415-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19730-3">
    <skos:scopeNote xml:lang="de">Verschiedene Strömungen, die die Subjektabhängigkeit aller Erkenntnis betonen, und z.T. vor allem in den Naturwissenschaften versuchen, einen begründeten und zirkelfreien Aufbau der Wissenschaft zu leisten.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70051">
    <skos:narrower rdf:resource="descriptor/10406-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70051">
    <skos:narrower rdf:resource="descriptor/19472-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70051">
    <skos:narrower rdf:resource="descriptor/18308-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70051">
    <skos:narrower rdf:resource="descriptor/11207-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70051">
    <skos:narrower rdf:resource="descriptor/19492-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70051">
    <skos:narrower rdf:resource="descriptor/11278-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70051">
    <skos:narrower rdf:resource="descriptor/11267-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70051">
    <skos:narrower rdf:resource="descriptor/19462-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70051">
    <skos:narrower rdf:resource="descriptor/10261-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70051">
    <skos:narrower rdf:resource="descriptor/11191-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70051">
    <skos:narrower rdf:resource="descriptor/11231-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70051">
    <skos:narrower rdf:resource="descriptor/10613-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70051">
    <skos:narrower rdf:resource="descriptor/11273-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70051">
    <skos:narrower rdf:resource="descriptor/11272-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70051">
    <skos:narrower rdf:resource="descriptor/11271-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70051">
    <skos:narrower rdf:resource="descriptor/10491-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70051">
    <skos:narrower rdf:resource="descriptor/19494-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70051">
    <skos:narrower rdf:resource="descriptor/11270-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70051">
    <skos:narrower rdf:resource="descriptor/18140-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70051">
    <skos:narrower rdf:resource="descriptor/11266-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70051">
    <skos:narrower rdf:resource="descriptor/11184-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70051">
    <skos:narrower rdf:resource="descriptor/11264-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70051">
    <skos:narrower rdf:resource="descriptor/18168-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70051">
    <skos:narrower rdf:resource="descriptor/19767-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70051">
    <skos:narrower rdf:resource="descriptor/19493-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70051">
    <skos:narrower rdf:resource="descriptor/11189-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70051">
    <skos:narrower rdf:resource="descriptor/11183-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70051">
    <skos:narrower rdf:resource="descriptor/10169-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70051">
    <skos:narrower rdf:resource="thsys/71060"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70051">
    <skos:narrower rdf:resource="thsys/71061"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70051">
    <skos:narrower rdf:resource="thsys/71062"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19657-1">
    <skos:related rdf:resource="descriptor/18647-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19657-1">
    <skos:related rdf:resource="descriptor/18659-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19657-1">
    <skos:related rdf:resource="descriptor/19544-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17199-1">
    <skos:broader rdf:resource="thsys/70557"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17199-1">
    <skos:broader rdf:resource="descriptor/16815-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10597-3">
    <skos:related rdf:resource="descriptor/10595-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10597-3">
    <skos:related rdf:resource="descriptor/10617-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10597-3">
    <skos:related rdf:resource="descriptor/10557-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19387-4">
    <skos:broader rdf:resource="thsys/71006"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19387-4">
    <skos:broader rdf:resource="thsys/70992"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19387-4">
    <skos:broader rdf:resource="descriptor/18388-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19387-4">
    <skos:broader rdf:resource="descriptor/11813-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10827-5">
    <zbwext:useInsteadNote rdf:nodeID="id33"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19047-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16683-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13826-6">
    <skos:prefLabel xml:lang="en">Actuarial risk</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13826-6">
    <skos:prefLabel xml:lang="de">Versicherungstechnisches Risiko</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15861-1">
    <skos:altLabel xml:lang="de">Verhaltensökonomie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15861-1">
    <skos:altLabel xml:lang="de">Ökonomische Verhaltenstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14985-2">
    <skos:broader rdf:resource="thsys/73300"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14985-2">
    <skos:broader rdf:resource="descriptor/14972-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19759-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14539-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10373-4">
    <skos:altLabel xml:lang="en">Leading indicator</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10373-4">
    <skos:altLabel xml:lang="en">Lagging indicator</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10373-4">
    <skos:altLabel xml:lang="de">Spätindikator</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10373-4">
    <skos:altLabel xml:lang="de">Konjunkturmessung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10373-4">
    <skos:altLabel xml:lang="de">Konjunkturdiagnose</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10373-4">
    <skos:altLabel xml:lang="de">Konjunkturbarometer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10373-4">
    <skos:altLabel xml:lang="de">Frühindikator</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71112">
    <skos:prefLabel xml:lang="de">V.03.05  Konsum und Sparen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10559-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16496-0">
    <skos:prefLabel xml:lang="en">European policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16496-0">
    <skos:prefLabel xml:lang="de">Europapolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14035-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14885-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18224-2">
    <skos:related rdf:resource="descriptor/16828-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10150-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16084-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17678-3">
    <skos:altLabel xml:lang="de">Vereinigte Republik Tansania</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19429-0">
    <skos:broader rdf:resource="thsys/71070"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19429-0">
    <skos:broader rdf:resource="thsys/70101"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19429-0">
    <skos:broader rdf:resource="descriptor/10956-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19429-0">
    <skos:broader rdf:resource="descriptor/10895-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19429-0">
    <skos:broader rdf:resource="descriptor/10905-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24706-5">
    <skos:scopeNote xml:lang="de">Die Kundenkarte ist ein Marketinginstrument zur Kundenbindung. Sie wird insbesondere im Handel und im Dienstleistungssektor eingesetzt. Mit der Kundenkarte können Verbraucher/innen bei angeschlossenen Partnern Bonuspunkte oder Rabatte sammeln, die später eingelöst werden.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15889-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13102-6">
    <skos:prefLabel xml:lang="en">Printing industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13102-6">
    <skos:prefLabel xml:lang="de">Druckindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71000">
    <skos:broader rdf:resource="thsys/71003"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15837-5">
    <skos:altLabel xml:lang="de">Mitarbeiterzufriedenheit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="r1238766808r28278r1">
    <owl:unionOf rdf:nodeID="r1238766808r28278r2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14139-6">
    <skos:broader rdf:resource="thsys/70403"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14139-6">
    <skos:broader rdf:resource="descriptor/14145-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13459-0">
    <skos:related rdf:resource="descriptor/18278-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13459-0">
    <skos:related rdf:resource="descriptor/12787-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18438-4">
    <skos:prefLabel xml:lang="en">Commodity hedging</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18438-4">
    <skos:prefLabel xml:lang="de">Rohstoff-Hedging</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15448-2">
    <skos:prefLabel xml:lang="en">Nash equilibrium</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15448-2">
    <skos:prefLabel xml:lang="de">Nash-Gleichgewicht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18616-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13342-0">
    <skos:altLabel xml:lang="de">Kraftfahrschule</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70087">
    <skos:prefLabel xml:lang="de">B.01.04  Rechtsformen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18492-5">
    <skos:related rdf:resource="descriptor/13860-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18492-5">
    <skos:related rdf:resource="descriptor/12702-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18492-5">
    <skos:related rdf:resource="descriptor/12905-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18492-5">
    <skos:related rdf:resource="descriptor/18491-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18492-5">
    <skos:related rdf:resource="descriptor/13376-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18492-5">
    <skos:related rdf:resource="descriptor/13957-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18023-5">
    <skos:altLabel xml:lang="de">Produktsteuerungssystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18023-5">
    <skos:altLabel xml:lang="de">Produktplanungssystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18023-5">
    <skos:altLabel xml:lang="de">Produktionssteuerungssystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18023-5">
    <skos:altLabel xml:lang="de">Produktionsplanungssystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18023-5">
    <skos:altLabel xml:lang="de">Leitstandsystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18023-5">
    <skos:altLabel xml:lang="de">Fertigungsplanungssystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18023-5">
    <skos:altLabel xml:lang="de">Fertigungslenkungssystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18023-5">
    <skos:altLabel xml:lang="de">Fertigungsleitsystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14777-2">
    <skos:altLabel xml:lang="en">Glass fibre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14777-2">
    <skos:altLabel xml:lang="de">Textile Glasfaser</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14777-2">
    <skos:altLabel xml:lang="de">Optische Glasfaser</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14777-2">
    <skos:altLabel xml:lang="de">Glaswolle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14777-2">
    <skos:altLabel xml:lang="de">Fiberglas</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18673-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11421-0">
    <skos:altLabel xml:lang="en">Federal funds rate</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11421-0">
    <skos:altLabel xml:lang="de">Tagesgeld</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11421-0">
    <skos:altLabel xml:lang="de">Interbankengeschäft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11421-0">
    <skos:altLabel xml:lang="de">Geldmarktkredit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11421-0">
    <skos:altLabel xml:lang="de">Geldmarktgeschäft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11421-0">
    <skos:altLabel xml:lang="de">Bankengeldmarkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11421-0">
    <skos:altLabel xml:lang="de">Bankenclearing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12992-1">
    <skos:altLabel xml:lang="de">Uranerzbergbau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11812-6">
    <skos:narrower rdf:resource="descriptor/18358-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11812-6">
    <skos:narrower rdf:resource="descriptor/18458-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11812-6">
    <skos:narrower rdf:resource="descriptor/18346-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17578-0">
    <skos:altLabel xml:lang="de">Republik Indonesien</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73361">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.05.01.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71036">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">B.09.03</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13209-2">
    <skos:altLabel xml:lang="en">Rack jobber</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13209-2">
    <skos:altLabel xml:lang="de">Großmarkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10570-2">
    <skos:related rdf:resource="descriptor/10614-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10570-2">
    <skos:related rdf:resource="descriptor/10838-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10570-2">
    <skos:related rdf:resource="descriptor/10565-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13308-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20987-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18514-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16471-2">
    <zbwext:useInsteadNote rdf:nodeID="id158"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13992-3">
    <skos:narrower rdf:resource="descriptor/19043-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18560-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14334-1">
    <skos:broader rdf:resource="thsys/70549"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14334-1">
    <skos:broader rdf:resource="descriptor/14336-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25621-1">
    <skos:broader rdf:resource="thsys/70814"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25621-1">
    <skos:broader rdf:resource="descriptor/12302-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16974-4">
    <skos:prefLabel xml:lang="en">Solothurn</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16974-4">
    <skos:prefLabel xml:lang="de">Solothurn</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15835-2">
    <skos:related rdf:resource="descriptor/12577-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15695-6">
    <skos:related rdf:resource="descriptor/11892-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15695-6">
    <skos:related rdf:resource="descriptor/11986-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15695-6">
    <skos:related rdf:resource="descriptor/10510-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15695-6">
    <skos:related rdf:resource="descriptor/19476-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19123-3">
    <skos:prefLabel xml:lang="en">Market saturation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19123-3">
    <skos:prefLabel xml:lang="de">Marktsättigung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19445-2">
    <skos:narrower rdf:resource="descriptor/11944-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19348-0">
    <skos:prefLabel xml:lang="en">Wine policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19348-0">
    <skos:prefLabel xml:lang="de">Weinpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11658-4">
    <skos:related rdf:resource="descriptor/11659-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11658-4">
    <skos:related rdf:resource="descriptor/12026-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16825-0">
    <skos:broader rdf:resource="thsys/70554"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16825-0">
    <skos:broader rdf:resource="descriptor/16815-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70035">
    <rdfs:label xml:lang="en">Northeast Europe</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70035">
    <rdfs:label xml:lang="de">Nordosteuropa</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15463-6">
    <skos:broader rdf:resource="thsys/73104"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15463-6">
    <skos:broader rdf:resource="descriptor/15458-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15405-6">
    <skos:broader rdf:resource="thsys/72394"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15405-6">
    <skos:broader rdf:resource="descriptor/15064-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16430-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11143-2">
    <skos:prefLabel xml:lang="en">Management in socialist countries</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11143-2">
    <skos:prefLabel xml:lang="de">Sozialistische Wirtschaftsführung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10845-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14204-0">
    <skos:broader rdf:resource="thsys/70418"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14204-0">
    <skos:broader rdf:resource="descriptor/14198-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16661-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19048-5">
    <skos:broader rdf:resource="thsys/70348"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19048-5">
    <skos:broader rdf:resource="descriptor/14237-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10012-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16160-5">
    <skos:narrower rdf:resource="descriptor/18914-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19469-2">
    <skos:prefLabel xml:lang="en">Ghetto</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19469-2">
    <skos:prefLabel xml:lang="de">Ghetto</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16512-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10798-0">
    <skos:broader rdf:resource="thsys/71089"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10798-0">
    <skos:broader rdf:resource="thsys/70183"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10798-0">
    <skos:broader rdf:resource="descriptor/10024-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10798-0">
    <skos:broader rdf:resource="descriptor/12280-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71003">
    <rdfs:label xml:lang="en">Metal industry</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71003">
    <rdfs:label xml:lang="de">Metallindustrie</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19179-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14934-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10963-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11348-5">
    <skos:altLabel xml:lang="de">Politischer Streik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11348-5">
    <skos:altLabel xml:lang="de">Generalstreik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10170-4">
    <skos:broader rdf:resource="thsys/71112"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10170-4">
    <skos:broader rdf:resource="thsys/70493"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10170-4">
    <skos:broader rdf:resource="descriptor/10301-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13555-4">
    <skos:broader rdf:resource="thsys/70937"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13555-4">
    <skos:broader rdf:resource="descriptor/13560-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13967-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13025-5">
    <skos:prefLabel xml:lang="en">Aluminium industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13025-5">
    <skos:prefLabel xml:lang="de">Aluminiumindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73362">
    <rdfs:label xml:lang="de">Europäische Integration und EU-Politik</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24784-6">
    <skos:altLabel xml:lang="de">Offener Quellcode</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24784-6">
    <skos:altLabel xml:lang="de">Freie Software</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18325-3">
    <skos:broader rdf:resource="thsys/71085"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18325-3">
    <skos:broader rdf:resource="thsys/70209"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18325-3">
    <skos:broader rdf:resource="descriptor/10524-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18975-6">
    <skos:broader rdf:resource="thsys/70814"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18975-6">
    <skos:broader rdf:resource="descriptor/10019-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19806-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13322-6">
    <zbwext:useInsteadNote rdf:nodeID="id30"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16475-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10143-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11048-3">
    <skos:broader rdf:resource="thsys/71101"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11048-3">
    <skos:broader rdf:resource="descriptor/11060-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11048-3">
    <skos:broader rdf:resource="descriptor/11043-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13305-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15117-4">
    <skos:broader rdf:resource="thsys/72183"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10913-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10627-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19317-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18366-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10005-5">
    <skos:related rdf:resource="descriptor/10024-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10005-5">
    <skos:related rdf:resource="descriptor/16645-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10005-5">
    <skos:related rdf:resource="descriptor/11040-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10005-5">
    <skos:related rdf:resource="descriptor/19161-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19212-4">
    <skos:prefLabel xml:lang="en">Leviathan model</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19212-4">
    <skos:prefLabel xml:lang="de">Leviathan-Modell</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12809-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73334">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.05.01.03</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19672-5">
    <skos:scopeNote xml:lang="de">Strategisches Potenzial eines Unternehmens in Form von Fertigkeiten, Fähigkeiten und Technologien, das als Grundlage zur Erzielung von Wettbewerbsvorteilen und Kundenzufriedenheit eingesetzt wird. Der Begriff ist nicht einheitlich abgegrenzt.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10624-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19627-3">
    <zbwext:useInsteadNote rdf:nodeID="id331"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19224-4">
    <skos:related rdf:resource="descriptor/11910-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19224-4">
    <skos:related rdf:resource="descriptor/19440-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19224-4">
    <skos:related rdf:resource="descriptor/10219-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19224-4">
    <skos:related rdf:resource="descriptor/10215-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10455-2">
    <skos:prefLabel xml:lang="en">Microelectronics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10455-2">
    <skos:prefLabel xml:lang="de">Mikroelektronik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17726-4">
    <skos:altLabel xml:lang="de">Spanisch-Guinea</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17726-4">
    <skos:altLabel xml:lang="de">Rio Muni und Fernando Poo</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17726-4">
    <skos:altLabel xml:lang="de">Republik Äquatorialguinea</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17726-4">
    <skos:altLabel xml:lang="de">Malabo</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19316-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12106-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19547-1">
    <skos:prefLabel xml:lang="en">Inflation targeting</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19547-1">
    <skos:prefLabel xml:lang="de">Inflation Targeting</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18622-4">
    <skos:broader rdf:resource="thsys/70930"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18622-4">
    <skos:broader rdf:resource="descriptor/13620-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19542-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10343-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19271-2">
    <skos:broader rdf:resource="thsys/70897"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19271-2">
    <skos:broader rdf:resource="thsys/70187"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19271-2">
    <skos:broader rdf:resource="descriptor/12210-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13379-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19142-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15891-6">
    <skos:broader rdf:resource="thsys/73354"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15891-6">
    <skos:broader rdf:resource="descriptor/15876-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17919-0">
    <skos:altLabel xml:lang="de">Republik Chile</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16155-5">
    <skos:prefLabel xml:lang="en">Disability legislation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16155-5">
    <skos:prefLabel xml:lang="de">Schwerbehindertenrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15340-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10314-6">
    <skos:prefLabel xml:lang="en">Factor mobility</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10314-6">
    <skos:prefLabel xml:lang="de">Faktormobilität</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18180-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15873-1">
    <skos:altLabel xml:lang="de">Tektonik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15873-1">
    <skos:altLabel xml:lang="de">Petrologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15873-1">
    <skos:altLabel xml:lang="de">Paläontologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15873-1">
    <skos:altLabel xml:lang="de">Lagerstättenkunde</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15873-1">
    <skos:altLabel xml:lang="de">Hydrologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15873-1">
    <skos:altLabel xml:lang="de">Erdschicht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15873-1">
    <skos:altLabel xml:lang="de">Erdgeschichte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15873-1">
    <skos:altLabel xml:lang="de">Bodenkunde</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15873-1">
    <skos:altLabel xml:lang="de">Bodenforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14211-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15872-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13405-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15066-2">
    <skos:altLabel xml:lang="de">Statistische Prozesskontrolle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15066-2">
    <skos:altLabel xml:lang="de">Statistische Prozeßkontrolle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18682-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12197-2">
    <skos:broader rdf:resource="thsys/70087"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12197-2">
    <skos:broader rdf:resource="descriptor/12194-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13414-1">
    <skos:scopeNote xml:lang="de">Nur für Gebietskörperschaften. Für alle Verwaltungseinheiten der mittleren Ebene, die keine Teilstaaten darstellen, z.B. die spanischen und italienischen Regionen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12832-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10072-4">
    <skos:prefLabel xml:lang="en">Core</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10072-4">
    <skos:prefLabel xml:lang="de">Core</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70445">
    <skos:broader rdf:resource="thsys/70354"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24788-5">
    <skos:broader rdf:resource="thsys/70128"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24788-5">
    <skos:broader rdf:resource="descriptor/17948-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18536-4">
    <skos:broader rdf:resource="thsys/73362"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18536-4">
    <skos:broader rdf:resource="thsys/70952"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18536-4">
    <skos:broader rdf:resource="descriptor/10605-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12038-1">
    <skos:related rdf:resource="descriptor/10911-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12038-1">
    <skos:related rdf:resource="descriptor/12071-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12038-1">
    <skos:related rdf:resource="descriptor/12708-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12038-1">
    <skos:related rdf:resource="descriptor/18120-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12038-1">
    <skos:related rdf:resource="descriptor/12049-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12038-1">
    <skos:related rdf:resource="descriptor/11783-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12038-1">
    <skos:related rdf:resource="descriptor/12224-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19678-0">
    <skos:scopeNote xml:lang="de">Sowohl für Literatur, die die gegebenen Bedingungen für den Zugang zu Infrastrukturnetzen diskutiert, als auch für netzökonomische Untersuchungen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13415-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13821-2">
    <skos:related rdf:resource="descriptor/13828-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19502-2">
    <skos:broader rdf:resource="thsys/70851"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19502-2">
    <skos:broader rdf:resource="thsys/70235"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15880-4">
    <skos:altLabel xml:lang="de">Klimazone</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14353-4">
    <skos:broader rdf:resource="thsys/70424"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14353-4">
    <skos:broader rdf:resource="descriptor/14355-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14353-4">
    <skos:broader rdf:resource="descriptor/14418-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12489-3">
    <skos:broader rdf:resource="thsys/70578"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12489-3">
    <skos:broader rdf:resource="descriptor/19096-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12489-3">
    <skos:broader rdf:resource="descriptor/19108-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18054-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15963-0">
    <skos:broader rdf:resource="thsys/73334"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15963-0">
    <skos:broader rdf:resource="descriptor/18068-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15963-0">
    <skos:broader rdf:resource="descriptor/19704-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13343-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17922-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19375-4">
    <skos:related rdf:resource="descriptor/11537-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19375-4">
    <skos:related rdf:resource="descriptor/19374-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19375-4">
    <skos:related rdf:resource="descriptor/11715-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13259-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10779-4">
    <skos:narrower rdf:resource="descriptor/13720-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18330-3">
    <skos:broader rdf:resource="thsys/71020"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18330-3">
    <skos:broader rdf:resource="descriptor/19310-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18330-3">
    <skos:broader rdf:resource="descriptor/11800-6"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id82">
    <rdfs:seeAlso rdf:resource="descriptor/14473-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15450-1">
    <skos:broader rdf:resource="thsys/73104"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15450-1">
    <skos:broader rdf:resource="descriptor/15458-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12014-1">
    <skos:related rdf:resource="descriptor/12932-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12014-1">
    <skos:related rdf:resource="descriptor/18749-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12014-1">
    <skos:related rdf:resource="descriptor/19065-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12014-1">
    <skos:related rdf:resource="descriptor/11859-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12014-1">
    <skos:related rdf:resource="descriptor/11995-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19002-1">
    <skos:related rdf:resource="descriptor/19001-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19002-1">
    <skos:related rdf:resource="descriptor/12574-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19002-1">
    <skos:related rdf:resource="descriptor/15839-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19002-1">
    <skos:related rdf:resource="descriptor/15837-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10730-6">
    <skos:prefLabel xml:lang="en">German Mark</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10730-6">
    <skos:prefLabel xml:lang="de">D-Mark</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11049-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13421-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14942-6">
    <skos:related rdf:resource="descriptor/13163-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14942-6">
    <skos:related rdf:resource="descriptor/13237-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14942-6">
    <skos:related rdf:resource="descriptor/14996-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14942-6">
    <skos:related rdf:resource="descriptor/18213-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11688-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16664-5">
    <skos:narrower rdf:resource="descriptor/11294-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14383-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14101-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12306-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71082">
    <skos:narrower rdf:resource="descriptor/19379-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71082">
    <skos:narrower rdf:resource="descriptor/18813-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71082">
    <skos:narrower rdf:resource="descriptor/11596-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71082">
    <skos:narrower rdf:resource="descriptor/19378-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71082">
    <skos:narrower rdf:resource="descriptor/11731-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71082">
    <skos:narrower rdf:resource="descriptor/11729-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71082">
    <skos:narrower rdf:resource="descriptor/11728-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71082">
    <skos:narrower rdf:resource="descriptor/11727-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71082">
    <skos:narrower rdf:resource="descriptor/11734-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71082">
    <skos:narrower rdf:resource="descriptor/10381-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14024-2">
    <skos:related rdf:resource="descriptor/13226-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14024-2">
    <skos:related rdf:resource="descriptor/14607-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11044-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12497-4">
    <skos:altLabel xml:lang="de">Kostenart</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18739-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13180-0">
    <skos:prefLabel xml:lang="en">Construction finishings and fittings</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13180-0">
    <skos:prefLabel xml:lang="de">Ausbaugewerbe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14225-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11637-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17804-3">
    <skos:broader rdf:resource="thsys/70086"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17804-3">
    <skos:broader rdf:resource="descriptor/17791-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12158-5">
    <skos:related rdf:resource="descriptor/18029-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12158-5">
    <skos:related rdf:resource="descriptor/12212-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12158-5">
    <skos:related rdf:resource="descriptor/20468-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19788-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11347-0">
    <skos:scopeNote xml:lang="de">Verfahren zur Ausgleichung der Interessen zwischen den Sozialpartnern bei Abschluss eines neuen Tarifvertrages durch staatliche Stellen oder durch Hinzuziehung einer neutralen Person.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19054-3">
    <skos:prefLabel xml:lang="en">Construction site preparation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19054-3">
    <skos:prefLabel xml:lang="de">Baustellenvorbereitung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24621-6">
    <skos:altLabel xml:lang="de">Müllentsorgung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24621-6">
    <skos:altLabel xml:lang="de">Entsorgung von Abfall</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24621-6">
    <skos:altLabel xml:lang="de">Abfallbeseitigung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14784-5">
    <skos:altLabel xml:lang="de">Holzbauelement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16212-5">
    <skos:altLabel xml:lang="en">Public administration science</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16212-5">
    <skos:altLabel xml:lang="de">Verwaltungstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16212-5">
    <skos:altLabel xml:lang="de">Verwaltungslehre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16212-5">
    <skos:altLabel xml:lang="de">Verwaltungskunde</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15005-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10556-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19296-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13987-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17726-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24856-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71034">
    <rdfs:label xml:lang="de">Unternehmensentwicklung, Betriebsgröße und Standortwahl</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18346-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18075-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10440-1">
    <skos:prefLabel xml:lang="en">Labour productivity</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10440-1">
    <skos:prefLabel xml:lang="de">Arbeitsproduktivität</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18567-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11734-0">
    <skos:prefLabel xml:lang="en">Crowding out</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11734-0">
    <skos:prefLabel xml:lang="de">Crowding out</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14189-5">
    <skos:broader rdf:resource="thsys/71015"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14189-5">
    <skos:broader rdf:resource="thsys/71014"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14189-5">
    <skos:broader rdf:resource="thsys/70651"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14189-5">
    <skos:broader rdf:resource="thsys/70648"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14189-5">
    <skos:broader rdf:resource="thsys/70227"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14189-5">
    <skos:broader rdf:resource="descriptor/18024-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18504-3">
    <skos:prefLabel xml:lang="en">Construction economics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18504-3">
    <skos:prefLabel xml:lang="de">Baubetriebslehre</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73391">
    <rdfs:label xml:lang="de">Wirtschaftsprivatrecht</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18892-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19427-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13867-6">
    <skos:broader rdf:resource="thsys/70888"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13867-6">
    <skos:broader rdf:resource="descriptor/15709-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13002-3">
    <skos:broader rdf:resource="thsys/70993"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13002-3">
    <skos:broader rdf:resource="descriptor/12996-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16745-5">
    <skos:scopeNote xml:lang="de">Literatur zur wirtschaftswissenschaftlichen Dogmengeschichte, aber auch zur Dogmengeschichte anderer Disziplinen oder zur Theoriegeschichte.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70858">
    <skos:broader rdf:resource="thsys/70860"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19941-4">
    <skos:altLabel xml:lang="de">Subventionskontrolle der EU</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19941-4">
    <skos:altLabel xml:lang="de">Beihilfenkontrolle der Europäischen Union</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19941-4">
    <skos:altLabel xml:lang="de">Beihilfenkontrolle der EU</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19941-4">
    <skos:altLabel xml:lang="de">Beihilfenaufsicht der Europäischen Union</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19941-4">
    <skos:altLabel xml:lang="de">Beihilfenaufsicht der EU</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19941-4">
    <skos:altLabel xml:lang="de">Beihilfekontrolle der EU</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19941-4">
    <skos:altLabel xml:lang="de">Beihilfeaufsicht der EU</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19626-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70238">
    <skos:narrower rdf:resource="descriptor/15779-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70238">
    <skos:narrower rdf:resource="descriptor/14632-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70238">
    <skos:narrower rdf:resource="descriptor/15763-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70238">
    <skos:narrower rdf:resource="descriptor/15757-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70238">
    <skos:narrower rdf:resource="descriptor/15017-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70238">
    <skos:narrower rdf:resource="descriptor/15755-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70238">
    <skos:narrower rdf:resource="descriptor/19486-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70238">
    <skos:narrower rdf:resource="descriptor/12976-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70238">
    <skos:narrower rdf:resource="descriptor/15744-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70238">
    <skos:narrower rdf:resource="descriptor/15743-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70238">
    <skos:narrower rdf:resource="descriptor/15739-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70238">
    <skos:narrower rdf:resource="descriptor/18069-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70238">
    <skos:narrower rdf:resource="descriptor/18453-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70238">
    <skos:narrower rdf:resource="descriptor/15018-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70238">
    <skos:narrower rdf:resource="descriptor/15735-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70238">
    <skos:narrower rdf:resource="descriptor/15733-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70238">
    <skos:narrower rdf:resource="descriptor/19485-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70238">
    <skos:narrower rdf:resource="descriptor/19484-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70238">
    <skos:narrower rdf:resource="descriptor/12016-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70238">
    <skos:narrower rdf:resource="descriptor/15729-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70238">
    <skos:narrower rdf:resource="descriptor/18337-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70238">
    <skos:narrower rdf:resource="descriptor/15010-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70238">
    <skos:narrower rdf:resource="descriptor/15009-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16638-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12047-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18265-2">
    <skos:prefLabel xml:lang="en">Bath additive</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18265-2">
    <skos:prefLabel xml:lang="de">Badezusatz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19350-6">
    <skos:broader rdf:resource="thsys/73332"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19350-6">
    <skos:broader rdf:resource="thsys/71093"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19350-6">
    <skos:broader rdf:resource="thsys/71092"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19350-6">
    <skos:broader rdf:resource="descriptor/10626-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19350-6">
    <skos:broader rdf:resource="descriptor/16188-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70144">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11533-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10573-3">
    <skos:broader rdf:resource="thsys/73366"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10573-3">
    <skos:broader rdf:resource="thsys/71084"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10573-3">
    <skos:broader rdf:resource="descriptor/10811-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10573-3">
    <skos:broader rdf:resource="descriptor/10555-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10573-3">
    <skos:broader rdf:resource="descriptor/10552-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18137-4">
    <skos:related rdf:resource="descriptor/15775-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18137-4">
    <skos:related rdf:resource="descriptor/19020-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18137-4">
    <skos:related rdf:resource="descriptor/18325-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18137-4">
    <skos:related rdf:resource="descriptor/14932-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18137-4">
    <skos:related rdf:resource="descriptor/18073-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10151-1">
    <skos:related rdf:resource="descriptor/15467-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10041-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17972-3">
    <skos:broader rdf:resource="thsys/70198"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17972-3">
    <skos:broader rdf:resource="descriptor/17954-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70962">
    <skos:broader rdf:resource="thsys/70963"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71096">
    <skos:narrower rdf:resource="descriptor/10649-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71096">
    <skos:narrower rdf:resource="descriptor/10715-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71096">
    <skos:narrower rdf:resource="descriptor/16122-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71096">
    <skos:narrower rdf:resource="descriptor/10714-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71096">
    <skos:narrower rdf:resource="descriptor/10713-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71096">
    <skos:narrower rdf:resource="descriptor/19331-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71096">
    <skos:narrower rdf:resource="descriptor/10710-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71096">
    <skos:narrower rdf:resource="descriptor/10845-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71096">
    <skos:narrower rdf:resource="descriptor/10644-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71096">
    <skos:narrower rdf:resource="descriptor/10695-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71096">
    <skos:narrower rdf:resource="descriptor/10711-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71096">
    <skos:narrower rdf:resource="descriptor/10564-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71096">
    <skos:narrower rdf:resource="descriptor/19330-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14090-3">
    <skos:broader rdf:resource="thsys/70070"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14090-3">
    <skos:broader rdf:resource="descriptor/14088-4"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id71">
    <rdf:value xml:lang="de">Für das Gut "Information"</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13100-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12640-4">
    <skos:altLabel xml:lang="de">Warenbeschaffung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12640-4">
    <skos:altLabel xml:lang="de">Materialbeschaffung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12640-4">
    <skos:altLabel xml:lang="de">Beschaffungswesen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12640-4">
    <skos:altLabel xml:lang="de">Beschaffungstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12640-4">
    <skos:altLabel xml:lang="de">Beschaffungslehre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12487-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11306-0">
    <skos:prefLabel xml:lang="en">Worker directors</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11306-0">
    <skos:prefLabel xml:lang="de">Arbeitsdirektoren</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12433-2">
    <skos:altLabel xml:lang="de">Lifo-Verfahren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12433-2">
    <skos:altLabel xml:lang="de">Last-in-First-out-Verfahren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12433-2">
    <skos:altLabel xml:lang="de">Kilo-Verfahren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12433-2">
    <skos:altLabel xml:lang="de">Kifo-Verfahren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12433-2">
    <skos:altLabel xml:lang="de">Hifo-Verfahren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12433-2">
    <skos:altLabel xml:lang="de">Gruppenbewertung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12433-2">
    <skos:altLabel xml:lang="de">First-in-First-out-Verfahren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12433-2">
    <skos:altLabel xml:lang="de">Fifo-Verfahren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12433-2">
    <skos:altLabel xml:lang="de">Durchschnittsbewertung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70865">
    <skos:broader rdf:resource="thsys/70871"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12210-1">
    <skos:broader rdf:resource="thsys/70897"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12210-1">
    <skos:broader rdf:resource="thsys/70187"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12210-1">
    <skos:broader rdf:resource="descriptor/12212-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18089-3">
    <skos:altLabel xml:lang="de">HGV (Hochgeschwindigkeitsverkehr)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18730-1">
    <skos:prefLabel xml:lang="en">Stock market crash</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18730-1">
    <skos:prefLabel xml:lang="de">Börsenkrise</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11652-2">
    <skos:prefLabel xml:lang="en">Taxation of earnings</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11652-2">
    <skos:prefLabel xml:lang="de">Ertragsteuer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14940-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18478-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19290-5">
    <skos:prefLabel xml:lang="en">Cotton market</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19290-5">
    <skos:prefLabel xml:lang="de">Baumwollmarkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19612-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10089-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71039">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19312-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10145-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18259-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15397-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10618-0">
    <skos:prefLabel xml:lang="en">International economic organization</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10618-0">
    <skos:prefLabel xml:lang="de">Internationale Wirtschaftsorganisation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13865-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16372-4">
    <skos:broader rdf:resource="thsys/73378"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16372-4">
    <skos:broader rdf:resource="descriptor/13397-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13649-2">
    <skos:prefLabel xml:lang="en">Video telephone</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13649-2">
    <skos:prefLabel xml:lang="de">Bildtelefon</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19559-1">
    <skos:altLabel xml:lang="de">Schulgeld</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19559-1">
    <skos:altLabel xml:lang="de">Schuletat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19032-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73386">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19619-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id51">
    <rdfs:seeAlso rdf:resource="descriptor/11913-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71111">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12847-3">
    <skos:prefLabel xml:lang="en">Print advertising</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12847-3">
    <skos:prefLabel xml:lang="de">Zeitungswerbung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12420-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16203-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15895-5">
    <skos:broader rdf:resource="thsys/73352"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15895-5">
    <skos:broader rdf:resource="thsys/70307"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15661-2">
    <skos:related rdf:resource="descriptor/18448-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15661-2">
    <skos:related rdf:resource="descriptor/14573-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15405-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73361">
    <skos:narrower rdf:resource="descriptor/15983-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73361">
    <skos:narrower rdf:resource="descriptor/19857-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73361">
    <skos:narrower rdf:resource="descriptor/19703-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73361">
    <skos:narrower rdf:resource="descriptor/13422-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73361">
    <skos:narrower rdf:resource="descriptor/16016-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73361">
    <skos:narrower rdf:resource="descriptor/11349-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73361">
    <skos:narrower rdf:resource="descriptor/16332-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73361">
    <skos:narrower rdf:resource="descriptor/16017-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73361">
    <skos:narrower rdf:resource="descriptor/16295-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73361">
    <skos:narrower rdf:resource="descriptor/16013-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73361">
    <skos:narrower rdf:resource="descriptor/19687-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73361">
    <skos:narrower rdf:resource="descriptor/15974-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73361">
    <skos:narrower rdf:resource="descriptor/16011-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73361">
    <skos:narrower rdf:resource="descriptor/16008-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73361">
    <skos:narrower rdf:resource="descriptor/13662-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73361">
    <skos:narrower rdf:resource="descriptor/16258-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73361">
    <skos:narrower rdf:resource="descriptor/19640-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73361">
    <skos:narrower rdf:resource="descriptor/15998-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73361">
    <skos:narrower rdf:resource="descriptor/15994-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73361">
    <skos:narrower rdf:resource="descriptor/19649-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73361">
    <skos:narrower rdf:resource="descriptor/19843-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18624-0">
    <skos:related rdf:resource="descriptor/18629-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18624-0">
    <skos:related rdf:resource="descriptor/11829-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/w">
    <rdfs:label xml:lang="en">Economic sectors</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/w">
    <rdfs:label xml:lang="de">Wirtschaftssektoren und spezielle Wirtschaftslehren</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14683-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10460-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19381-2">
    <skos:prefLabel xml:lang="en">War finance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19381-2">
    <skos:prefLabel xml:lang="de">Kriegsfinanzierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18761-4">
    <skos:broader rdf:resource="thsys/70881"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18761-4">
    <skos:broader rdf:resource="thsys/70241"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18761-4">
    <skos:broader rdf:resource="descriptor/18433-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10214-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15045-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11876-1">
    <skos:narrower rdf:resource="descriptor/18608-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18523-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14124-5">
    <skos:broader rdf:resource="thsys/73303"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14124-5">
    <skos:broader rdf:resource="thsys/70467"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14124-5">
    <skos:broader rdf:resource="descriptor/14129-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14124-5">
    <skos:broader rdf:resource="descriptor/14931-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71108">
    <skos:broader rdf:resource="thsys/70187"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19012-5">
    <skos:altLabel xml:lang="en">Unit labor costs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11345-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11020-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10095-6">
    <skos:broader rdf:resource="thsys/71118"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10095-6">
    <skos:broader rdf:resource="thsys/70057"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10095-6">
    <skos:broader rdf:resource="descriptor/10160-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18244-3">
    <skos:broader rdf:resource="thsys/70318"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18244-3">
    <skos:broader rdf:resource="descriptor/14392-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12800-1">
    <skos:broader rdf:resource="thsys/70550"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12800-1">
    <skos:broader rdf:resource="descriptor/10016-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13757-6">
    <skos:prefLabel xml:lang="en">Foreign exchange trading</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13757-6">
    <skos:prefLabel xml:lang="de">Devisenhandel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70960">
    <rdfs:label xml:lang="de">Lebensmittel- und Landhandel</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16949-3">
    <skos:prefLabel xml:lang="en">Upper Austria</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16949-3">
    <skos:prefLabel xml:lang="de">Oberösterreich</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16247-0">
    <skos:prefLabel xml:lang="en">Dictatorship</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16247-0">
    <skos:prefLabel xml:lang="de">Diktatur</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70991">
    <skos:narrower rdf:resource="descriptor/13120-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70991">
    <skos:narrower rdf:resource="descriptor/12912-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70991">
    <skos:narrower rdf:resource="descriptor/13113-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70991">
    <skos:narrower rdf:resource="descriptor/19367-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70991">
    <skos:narrower rdf:resource="descriptor/13110-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70991">
    <skos:narrower rdf:resource="descriptor/13109-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70991">
    <skos:narrower rdf:resource="descriptor/13894-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70991">
    <skos:narrower rdf:resource="descriptor/13106-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19468-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19752-0">
    <skos:prefLabel xml:lang="en">Social capital</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19752-0">
    <skos:prefLabel xml:lang="de">Social Capital</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70879">
    <skos:narrower rdf:resource="descriptor/13349-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70879">
    <skos:narrower rdf:resource="descriptor/18031-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70879">
    <skos:narrower rdf:resource="descriptor/10472-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70879">
    <skos:narrower rdf:resource="descriptor/12262-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70879">
    <skos:narrower rdf:resource="descriptor/10945-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70879">
    <skos:narrower rdf:resource="descriptor/10459-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70879">
    <skos:narrower rdf:resource="descriptor/18774-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70879">
    <skos:narrower rdf:resource="descriptor/19584-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70879">
    <skos:narrower rdf:resource="descriptor/18785-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70879">
    <skos:narrower rdf:resource="descriptor/18775-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70879">
    <skos:narrower rdf:resource="descriptor/13343-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70879">
    <skos:narrower rdf:resource="descriptor/18784-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70879">
    <skos:narrower rdf:resource="descriptor/10437-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70879">
    <skos:narrower rdf:resource="descriptor/10436-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70879">
    <skos:narrower rdf:resource="descriptor/18773-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70879">
    <skos:narrower rdf:resource="thsys/70878"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18613-5">
    <skos:broader rdf:resource="thsys/70201"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15055-0">
    <skos:altLabel xml:lang="en">Parametric linear programming</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15055-0">
    <skos:altLabel xml:lang="en">Linear programming</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15055-0">
    <skos:altLabel xml:lang="de">Simplexmethode</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15055-0">
    <skos:altLabel xml:lang="de">Programmierungsmodell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15055-0">
    <skos:altLabel xml:lang="de">Pivotelement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15055-0">
    <skos:altLabel xml:lang="de">Parametrische lineare Optimierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15055-0">
    <skos:altLabel xml:lang="de">Mathematische Programmierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15055-0">
    <skos:altLabel xml:lang="de">Lineare Programmierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15055-0">
    <skos:altLabel xml:lang="de">Lineare Optimierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19740-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10041-1">
    <skos:related rdf:resource="descriptor/10166-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10041-1">
    <skos:related rdf:resource="descriptor/16741-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10041-1">
    <skos:related rdf:resource="descriptor/10252-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10041-1">
    <skos:related rdf:resource="descriptor/10249-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18118-1">
    <skos:narrower rdf:resource="descriptor/18539-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18118-1">
    <skos:narrower rdf:resource="descriptor/19828-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18118-1">
    <skos:narrower rdf:resource="descriptor/18721-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18118-1">
    <skos:narrower rdf:resource="descriptor/12432-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18118-1">
    <skos:narrower rdf:resource="descriptor/10858-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70931">
    <skos:prefLabel xml:lang="en">W.12.01.04.01  Space travel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70931">
    <skos:prefLabel xml:lang="de">W.12.01.04.01  Raumfahrt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15405-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13351-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11913-0">
    <skos:related rdf:resource="descriptor/18160-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11913-0">
    <skos:related rdf:resource="descriptor/11915-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11913-0">
    <skos:related rdf:resource="descriptor/11932-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11913-0">
    <skos:related rdf:resource="descriptor/19291-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11913-0">
    <skos:related rdf:resource="descriptor/11914-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11965-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19635-4">
    <skos:broader rdf:resource="thsys/70007"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19635-4">
    <skos:broader rdf:resource="descriptor/15685-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#semanticRelation">
    <rdfs:range rdf:resource="http://www.w3.org/2004/02/skos/core#Concept"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13821-2">
    <skos:narrower rdf:resource="descriptor/18741-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10870-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10172-0">
    <skos:broader rdf:resource="thsys/71112"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10172-0">
    <skos:broader rdf:resource="thsys/70493"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10172-0">
    <skos:broader rdf:resource="descriptor/10181-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10172-0">
    <skos:broader rdf:resource="descriptor/10092-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14920-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13773-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10171-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19561-0">
    <skos:related rdf:resource="descriptor/12644-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19561-0">
    <skos:related rdf:resource="descriptor/10243-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19561-0">
    <skos:related rdf:resource="descriptor/15033-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19561-0">
    <skos:related rdf:resource="descriptor/18962-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13529-5">
    <skos:broader rdf:resource="thsys/70935"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13529-5">
    <skos:broader rdf:resource="descriptor/13514-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19196-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11186-5">
    <skos:narrower rdf:resource="descriptor/12116-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11186-5">
    <skos:narrower rdf:resource="descriptor/11206-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11186-5">
    <skos:narrower rdf:resource="descriptor/10614-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70337">
    <rdfs:label xml:lang="de">Rüstungsgüter</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13535-3">
    <skos:altLabel xml:lang="de">Hafenbautechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15793-6">
    <skos:broader rdf:resource="thsys/70292"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15793-6">
    <skos:broader rdf:resource="thsys/70017"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15793-6">
    <skos:broader rdf:resource="descriptor/19601-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12314-3">
    <skos:altLabel xml:lang="en">Human-resource accounting</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12314-3">
    <skos:altLabel xml:lang="de">Saarbrücker Formel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12314-3">
    <skos:altLabel xml:lang="de">Personalvermögensrechnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12314-3">
    <skos:altLabel xml:lang="de">Personalpotential</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12314-3">
    <skos:altLabel xml:lang="de">Personalbilanz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12314-3">
    <skos:altLabel xml:lang="de">Humanvermögensrechnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12314-3">
    <skos:altLabel xml:lang="de">Betriebliches Humanvermögen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12314-3">
    <skos:altLabel xml:lang="de">Betriebliches Humankapital</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18486-0">
    <skos:broader rdf:resource="thsys/71005"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18486-0">
    <skos:broader rdf:resource="descriptor/11270-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#example">
    <rdfs:subPropertyOf rdf:resource="http://www.w3.org/2004/02/skos/core#note"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17500-2">
    <skos:scopeNote xml:lang="de">Erklärte sich 1990 unabhängig von der Sowjetunion und trat 1991 der GUS bei</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10696-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15340-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10177-4">
    <skos:prefLabel xml:lang="en">Household economy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10177-4">
    <skos:prefLabel xml:lang="de">Haushaltsökonomik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18704-2">
    <skos:prefLabel xml:lang="en">Central bank independence</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18704-2">
    <skos:prefLabel xml:lang="de">Zentralbankautonomie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18855-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18673-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18277-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14938-4">
    <skos:prefLabel xml:lang="en">Pasta</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14938-4">
    <skos:prefLabel xml:lang="de">Teigwaren</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10223-2">
    <skos:related rdf:resource="descriptor/10224-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18369-4">
    <skos:related rdf:resource="descriptor/10854-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15398-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11271-0">
    <skos:prefLabel xml:lang="en">Labour force</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11271-0">
    <skos:prefLabel xml:lang="de">Erwerbspersonenpotenzial</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18123-1">
    <skos:related rdf:resource="descriptor/16484-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18123-1">
    <skos:related rdf:resource="descriptor/18854-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18123-1">
    <skos:related rdf:resource="descriptor/19503-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18123-1">
    <skos:related rdf:resource="descriptor/19502-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18123-1">
    <skos:related rdf:resource="descriptor/11313-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18123-1">
    <skos:related rdf:resource="descriptor/11314-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12271-2">
    <skos:broader rdf:resource="thsys/70036"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12271-2">
    <skos:broader rdf:resource="descriptor/12285-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12271-2">
    <skos:broader rdf:resource="descriptor/12276-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70245">
    <skos:prefLabel xml:lang="de">B.04.02  Vergütungssysteme und betriebliche Sozialleistungen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19209-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19072-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17702-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14146-2">
    <skos:broader rdf:resource="thsys/70403"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14146-2">
    <skos:broader rdf:resource="descriptor/14145-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14656-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14088-4">
    <skos:prefLabel xml:lang="en">Grain</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14088-4">
    <skos:prefLabel xml:lang="de">Getreide</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70014">
    <skos:prefLabel xml:lang="de">P.12  Chemikalien</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71051">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18707-3">
    <skos:altLabel xml:lang="de">Regulierung des Bankensektors</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18707-3">
    <skos:altLabel xml:lang="de">Bankpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18707-3">
    <skos:altLabel xml:lang="de">Bankenpolitik des Staates</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18861-0">
    <skos:related rdf:resource="descriptor/19760-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18861-0">
    <skos:related rdf:resource="descriptor/15924-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18861-0">
    <skos:related rdf:resource="descriptor/11369-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18861-0">
    <skos:related rdf:resource="descriptor/19759-0"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id350">
    <rdfs:seeAlso rdf:resource="descriptor/15463-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12703-6">
    <skos:prefLabel xml:lang="en">Assembling</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12703-6">
    <skos:prefLabel xml:lang="de">Montage</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18645-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14581-5">
    <skos:broader rdf:resource="thsys/70201"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19956-5">
    <skos:altLabel xml:lang="de">Europäischer Betriebsrat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13777-0">
    <skos:related rdf:resource="descriptor/10816-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13777-0">
    <skos:related rdf:resource="descriptor/12253-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13777-0">
    <skos:related rdf:resource="descriptor/13293-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13777-0">
    <skos:related rdf:resource="descriptor/12252-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13777-0">
    <skos:related rdf:resource="descriptor/13776-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13777-0">
    <skos:related rdf:resource="descriptor/12251-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13777-0">
    <skos:related rdf:resource="descriptor/18721-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13777-0">
    <skos:related rdf:resource="descriptor/20733-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13777-0">
    <skos:related rdf:resource="descriptor/19227-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18650-6">
    <skos:related rdf:resource="descriptor/13678-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18650-6">
    <skos:related rdf:resource="descriptor/13667-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16081-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13363-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70539">
    <skos:broader rdf:resource="thsys/70227"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18017-0">
    <skos:broader rdf:resource="thsys/73341"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18017-0">
    <skos:broader rdf:resource="thsys/71037"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19832-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71010">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.05</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16124-2">
    <skos:related rdf:resource="descriptor/16545-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16124-2">
    <skos:related rdf:resource="descriptor/19764-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16124-2">
    <skos:related rdf:resource="descriptor/16155-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16124-2">
    <skos:related rdf:resource="descriptor/12603-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19242-2">
    <skos:scopeNote xml:lang="de">Konvergenz von Wachstum, Entwicklung und Lebensverhältnissen zwischen Wirtschaftsräumen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13470-5">
    <skos:broader rdf:resource="thsys/70963"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10217-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15972-6">
    <skos:scopeNote xml:lang="de">Für den Schutz der gewerblich-geistigen Leistungen BENUTZE Gewerblicher Rechtsschutz und die engeren Deskriptoren</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15972-6">
    <skos:scopeNote xml:lang="de">Rechtsschutz ist der durch die Rechtsordnung gewährte Schutz der Rechtsgüter. Er muss im Konfliktfall nach einem vorgeschriebenen Verfahren geltend gemacht werden und kann auch vorläufig gewährt werden.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24856-0">
    <skos:scopeNote xml:lang="de">Prozess zur Identifizierung und Quantifizierung von Leistungsindikatoren, die eine Aussage über das Maß der Zielerreichung bezüglich Qualität, Zeit und Kosten ermöglichen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71118">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.02.05</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12313-5">
    <skos:scopeNote xml:lang="de">Die vom International Accounting Standards Committee verabschiedeten Richtlinien zur internationalen Harmonisierung der Rechnungslegung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10375-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13256-0">
    <skos:altLabel xml:lang="de">Wasserverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13256-0">
    <skos:altLabel xml:lang="de">Wasserstraßenverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13256-0">
    <skos:altLabel xml:lang="de">Seeverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13256-0">
    <skos:altLabel xml:lang="de">Seeschifffahrt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13256-0">
    <skos:altLabel xml:lang="de">Seeschiffahrt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13256-0">
    <skos:altLabel xml:lang="de">Schiffahrt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13256-0">
    <skos:altLabel xml:lang="de">Hochseeschifffahrt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13256-0">
    <skos:altLabel xml:lang="de">Hochseeschiffahrt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13256-0">
    <skos:altLabel xml:lang="de">Handelsschifffahrt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13256-0">
    <skos:altLabel xml:lang="de">Handelsschiffahrt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13120-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18275-6">
    <skos:broader rdf:resource="thsys/70201"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18275-6">
    <skos:broader rdf:resource="descriptor/14043-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18275-6">
    <skos:broader rdf:resource="descriptor/14023-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10968-1">
    <skos:related rdf:resource="descriptor/10029-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10968-1">
    <skos:related rdf:resource="descriptor/16038-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10968-1">
    <skos:related rdf:resource="descriptor/16287-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10968-1">
    <skos:related rdf:resource="descriptor/16430-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10968-1">
    <skos:related rdf:resource="descriptor/16379-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10968-1">
    <skos:related rdf:resource="descriptor/19975-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10968-1">
    <skos:related rdf:resource="descriptor/16226-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10968-1">
    <skos:related rdf:resource="descriptor/16384-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10968-1">
    <skos:related rdf:resource="descriptor/10984-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10968-1">
    <skos:related rdf:resource="descriptor/11161-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10968-1">
    <skos:related rdf:resource="descriptor/11168-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10968-1">
    <skos:related rdf:resource="descriptor/18832-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19359-2">
    <skos:altLabel xml:lang="de">Transfermoratorium</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19359-2">
    <skos:altLabel xml:lang="de">Stillhalteabkommen (Kreditabkommen)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19359-2">
    <skos:altLabel xml:lang="de">Moratorium (Kreditabkommen)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19359-2">
    <skos:altLabel xml:lang="de">Moratorium (Auslandsschulden)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11566-2">
    <skos:broader rdf:resource="thsys/71113"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11566-2">
    <skos:broader rdf:resource="thsys/71080"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11566-2">
    <skos:broader rdf:resource="thsys/70833"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11566-2">
    <skos:broader rdf:resource="thsys/70324"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11566-2">
    <skos:broader rdf:resource="descriptor/11778-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11566-2">
    <skos:broader rdf:resource="descriptor/11570-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71053">
    <skos:prefLabel xml:lang="en">V.13.06.02.02  Caring professions</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71053">
    <skos:prefLabel xml:lang="de">V.13.06.02.02  Sozialberufe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13450-4">
    <skos:altLabel xml:lang="en">Farm mechanization</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13450-4">
    <skos:altLabel xml:lang="en">Agro-technology</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13450-4">
    <skos:altLabel xml:lang="en">Agricultural technology</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13450-4">
    <skos:altLabel xml:lang="de">Mechanisierung der Landwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13450-4">
    <skos:altLabel xml:lang="de">Landwirtschaftstechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13450-4">
    <skos:altLabel xml:lang="de">Landtechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13450-4">
    <skos:altLabel xml:lang="de">Agrartechnologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13450-4">
    <skos:altLabel xml:lang="de">Agrarmechanisierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11406-3">
    <skos:prefLabel xml:lang="en">Financing of vocational education and training</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11406-3">
    <skos:prefLabel xml:lang="de">Ausbildungsfinanzierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18618-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13274-5">
    <skos:broader rdf:resource="thsys/70923"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13274-5">
    <skos:broader rdf:resource="descriptor/13270-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11786-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15142-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10797-2">
    <skos:altLabel xml:lang="de">Schuldnerland</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10797-2">
    <skos:altLabel xml:lang="de">Schulden im Ausland</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10797-2">
    <skos:altLabel xml:lang="de">Auslandsschulden</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14722-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15483-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10031-4">
    <skos:broader rdf:resource="thsys/70021"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19548-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13002-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19187-5">
    <skos:prefLabel xml:lang="en">Grain policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19187-5">
    <skos:prefLabel xml:lang="de">Getreidepolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10512-2">
    <skos:altLabel xml:lang="en">Urbanisation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10512-2">
    <skos:altLabel xml:lang="en">Urban sprawl</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10512-2">
    <skos:altLabel xml:lang="de">Verstädterungsprozess</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10512-2">
    <skos:altLabel xml:lang="de">Verstädterungsprozeß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10512-2">
    <skos:altLabel xml:lang="de">Verstädterung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10814-0">
    <skos:prefLabel xml:lang="en">Capital flight</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10814-0">
    <skos:prefLabel xml:lang="de">Kapitalflucht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18095-1">
    <skos:related rdf:resource="descriptor/18435-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18095-1">
    <skos:related rdf:resource="descriptor/14392-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18095-1">
    <skos:related rdf:resource="descriptor/18094-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18095-1">
    <skos:related rdf:resource="descriptor/14383-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11649-5">
    <skos:prefLabel xml:lang="en">Import turnover tax</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11649-5">
    <skos:prefLabel xml:lang="de">Einfuhrumsatzsteuer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14337-2">
    <skos:altLabel xml:lang="de">Ventilator</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14337-2">
    <skos:altLabel xml:lang="de">Trocknungsanlage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14337-2">
    <skos:altLabel xml:lang="de">Lüftungstechnische Anlage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14337-2">
    <skos:altLabel xml:lang="de">Lufttechnische Anlage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14337-2">
    <skos:altLabel xml:lang="de">Kühlanlage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14337-2">
    <skos:altLabel xml:lang="de">Kältetechnisches Produkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14337-2">
    <skos:altLabel xml:lang="de">Kälteanlage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14337-2">
    <skos:altLabel xml:lang="de">Eismaschine</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70811">
    <skos:prefLabel xml:lang="en">V.12.03  Resource economics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70811">
    <skos:prefLabel xml:lang="de">V.12.03  Ressourcenökonomik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id192">
    <rdf:value xml:lang="de">Für Instrumente zur Motivationsförderung der Mitarbeiter/innen</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19439-4">
    <skos:broader rdf:resource="thsys/71060"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19439-4">
    <skos:broader rdf:resource="thsys/70058"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19439-4">
    <skos:broader rdf:resource="descriptor/11182-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11370-5">
    <skos:prefLabel xml:lang="en">University of applied sciences</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11370-5">
    <skos:prefLabel xml:lang="de">Fachhochschule</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15722-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70498">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18709-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10338-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18519-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12070-5">
    <skos:altLabel xml:lang="en">Business start-up</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12070-5">
    <skos:altLabel xml:lang="de">Neuerrichtung des Betriebs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12070-5">
    <skos:altLabel xml:lang="de">Jungunternehmen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12070-5">
    <skos:altLabel xml:lang="de">Gründung des Betriebs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12070-5">
    <skos:altLabel xml:lang="de">Firmengründung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12070-5">
    <skos:altLabel xml:lang="de">Existenzgründung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12070-5">
    <skos:altLabel xml:lang="de">Betriebsgründung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10041-1">
    <skos:prefLabel xml:lang="en">Classical economics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10041-1">
    <skos:prefLabel xml:lang="de">Klassische Theorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11572-0">
    <skos:prefLabel xml:lang="en">Subsidy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11572-0">
    <skos:prefLabel xml:lang="de">Subvention</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10348-3">
    <skos:related rdf:resource="descriptor/10343-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10348-3">
    <skos:related rdf:resource="descriptor/10344-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10348-3">
    <skos:related rdf:resource="descriptor/11090-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10348-3">
    <skos:related rdf:resource="descriptor/11046-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13590-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11864-1">
    <skos:altLabel xml:lang="de">Verkehr in der Fläche</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18646-4">
    <skos:prefLabel xml:lang="en">Postal rate</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18646-4">
    <skos:prefLabel xml:lang="de">Postgebühr</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18643-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19727-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14252-3">
    <skos:altLabel xml:lang="de">Ziegelprodukt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14252-3">
    <skos:altLabel xml:lang="de">Steinzeugprodukt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14252-3">
    <skos:altLabel xml:lang="de">Mauerziegel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14252-3">
    <skos:altLabel xml:lang="de">Klinker</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14252-3">
    <skos:altLabel xml:lang="de">Grobsteinzeug</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14252-3">
    <skos:altLabel xml:lang="de">Dachziegel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18235-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11348-5">
    <skos:prefLabel xml:lang="en">Strike</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11348-5">
    <skos:prefLabel xml:lang="de">Streik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19174-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72393">
    <rdfs:label xml:lang="en">Econometrics</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72393">
    <rdfs:label xml:lang="de">Ökonometrie</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14371-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11269-1">
    <skos:broader rdf:resource="thsys/71050"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11269-1">
    <skos:broader rdf:resource="thsys/70838"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11269-1">
    <skos:broader rdf:resource="thsys/70052"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11269-1">
    <skos:broader rdf:resource="descriptor/15908-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11269-1">
    <skos:broader rdf:resource="descriptor/11262-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18582-4">
    <skos:related rdf:resource="descriptor/13244-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13832-4">
    <skos:prefLabel xml:lang="en">Second home</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13832-4">
    <skos:prefLabel xml:lang="de">Zweitwohnung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14309-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16552-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12990-5">
    <skos:prefLabel xml:lang="en">Coal mining</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12990-5">
    <skos:prefLabel xml:lang="de">Kohlenbergbau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18633-6">
    <skos:narrower rdf:resource="descriptor/11827-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18071-1">
    <skos:narrower rdf:resource="descriptor/18327-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18071-1">
    <skos:narrower rdf:resource="descriptor/13327-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10979-3">
    <skos:broader rdf:resource="thsys/70038"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19016-4">
    <skos:prefLabel xml:lang="en">Toxicology</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19016-4">
    <skos:prefLabel xml:lang="de">Toxikologie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18985-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70198">
    <skos:broader rdf:resource="thsys/g"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17748-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12216-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11246-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14189-5">
    <skos:related rdf:resource="descriptor/14053-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14189-5">
    <skos:related rdf:resource="descriptor/14188-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70486">
    <rdfs:label xml:lang="en">Capital</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70486">
    <rdfs:label xml:lang="de">Kapital</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id210">
    <rdf:value xml:lang="de">Für das Angebot von Informations-Datenbanken</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18668-1">
    <skos:broader rdf:resource="thsys/70905"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18668-1">
    <skos:broader rdf:resource="thsys/70187"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18668-1">
    <skos:broader rdf:resource="descriptor/11441-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14464-2">
    <skos:altLabel xml:lang="de">Wiederaufladbare Batterie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14464-2">
    <skos:altLabel xml:lang="de">Primärbatterie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14464-2">
    <skos:altLabel xml:lang="de">Akkumulator</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17732-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14040-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24783-1">
    <skos:broader rdf:resource="thsys/70871"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24783-1">
    <skos:broader rdf:resource="descriptor/13413-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72393">
    <skos:narrower rdf:resource="descriptor/19577-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72393">
    <skos:narrower rdf:resource="descriptor/19573-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72393">
    <skos:narrower rdf:resource="descriptor/15346-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72393">
    <skos:narrower rdf:resource="descriptor/15375-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72393">
    <skos:narrower rdf:resource="descriptor/15072-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72393">
    <skos:narrower rdf:resource="descriptor/15411-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72393">
    <skos:narrower rdf:resource="descriptor/19230-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72393">
    <skos:narrower rdf:resource="descriptor/15376-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72393">
    <skos:narrower rdf:resource="descriptor/10253-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72393">
    <skos:narrower rdf:resource="descriptor/15373-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72393">
    <skos:narrower rdf:resource="descriptor/19552-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72393">
    <skos:narrower rdf:resource="descriptor/15381-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72393">
    <skos:narrower rdf:resource="descriptor/10139-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72393">
    <skos:narrower rdf:resource="descriptor/15408-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72393">
    <skos:narrower rdf:resource="descriptor/10045-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72393">
    <skos:narrower rdf:resource="descriptor/19576-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72393">
    <skos:narrower rdf:resource="descriptor/18076-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72393">
    <skos:narrower rdf:resource="descriptor/19582-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72393">
    <skos:narrower rdf:resource="descriptor/15327-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72393">
    <skos:narrower rdf:resource="descriptor/19574-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13125-1">
    <skos:altLabel xml:lang="en">Cellulose industry</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15814-3">
    <skos:related rdf:resource="descriptor/16564-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15814-3">
    <skos:related rdf:resource="descriptor/18887-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15814-3">
    <skos:related rdf:resource="descriptor/16554-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15814-3">
    <skos:related rdf:resource="descriptor/19745-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16432-5">
    <skos:altLabel xml:lang="de">Politische Werte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16432-5">
    <skos:altLabel xml:lang="de">Politische Grundwerte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13673-5">
    <skos:related rdf:resource="descriptor/18110-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13673-5">
    <skos:related rdf:resource="descriptor/14750-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13673-5">
    <skos:related rdf:resource="descriptor/13671-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19029-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24858-3">
    <skos:related rdf:resource="descriptor/19410-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24858-3">
    <skos:related rdf:resource="descriptor/11636-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13665-4">
    <skos:broader rdf:resource="thsys/70921"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13665-4">
    <skos:broader rdf:resource="thsys/70824"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13665-4">
    <skos:broader rdf:resource="descriptor/13278-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13665-4">
    <skos:broader rdf:resource="descriptor/13658-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13665-4">
    <skos:broader rdf:resource="descriptor/13268-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19143-4">
    <skos:scopeNote xml:lang="de">Steuerdifferenz auf den Gewinn nach Handelsbilanz und nach Steuerbilanz.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20125-3">
    <skos:related rdf:resource="descriptor/18165-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20125-3">
    <skos:related rdf:resource="descriptor/19160-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20125-3">
    <skos:related rdf:resource="descriptor/10576-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17672-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17773-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12232-5">
    <skos:related rdf:resource="descriptor/12271-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12232-5">
    <skos:related rdf:resource="descriptor/13714-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12232-5">
    <skos:related rdf:resource="descriptor/19535-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12232-5">
    <skos:related rdf:resource="descriptor/16043-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12232-5">
    <skos:related rdf:resource="descriptor/13713-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12232-5">
    <skos:related rdf:resource="descriptor/13712-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12232-5">
    <skos:related rdf:resource="descriptor/12174-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12232-5">
    <skos:related rdf:resource="descriptor/13699-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18307-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21306-1">
    <skos:scopeNote xml:lang="de">Durch Einsatz von Diversity Management wird versucht, Vielfalt und Unterschiede (aufgrund von Geschlecht, ethnischer Zugehörigkeit, Behinderung, etc.) in der Belegschaft nutzbar zu machen und Chancengleichheit für alle Gruppen zu erzielen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20068-3">
    <skos:broader rdf:resource="thsys/70882"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20068-3">
    <skos:broader rdf:resource="descriptor/13379-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12188-3">
    <skos:related rdf:resource="descriptor/12187-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14502-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18373-6">
    <skos:prefLabel xml:lang="en">Oil policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18373-6">
    <skos:prefLabel xml:lang="de">Erdölpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12770-5">
    <skos:related rdf:resource="descriptor/19565-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12770-5">
    <skos:related rdf:resource="descriptor/12778-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12770-5">
    <skos:related rdf:resource="descriptor/12775-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12770-5">
    <skos:related rdf:resource="descriptor/10202-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12770-5">
    <skos:related rdf:resource="descriptor/10928-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12770-5">
    <skos:related rdf:resource="descriptor/19667-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19037-3">
    <skos:related rdf:resource="descriptor/15272-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70937">
    <skos:prefLabel xml:lang="en">W.12.01.02.01  Motor vehicle traffic</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70937">
    <skos:prefLabel xml:lang="de">W.12.01.02.01  Kraftfahrzeugverkehr</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id302">
    <rdf:value xml:lang="de">Für Maßnahmen zur Inflationsbekämpfung</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13478-3">
    <skos:altLabel xml:lang="de">Wochenmarkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13478-3">
    <skos:altLabel xml:lang="de">Weihnachtsmarkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13478-3">
    <skos:altLabel xml:lang="de">Wanderhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13478-3">
    <skos:altLabel xml:lang="de">Wandergewerbe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13478-3">
    <skos:altLabel xml:lang="de">Verkaufswagensystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13478-3">
    <skos:altLabel xml:lang="de">Straßenhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13478-3">
    <skos:altLabel xml:lang="de">Markthandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13478-3">
    <skos:altLabel xml:lang="de">Jahrmarkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13478-3">
    <skos:altLabel xml:lang="de">Fahrverkauf</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13478-3">
    <skos:altLabel xml:lang="de">Ambulantes Gewerbe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id139">
    <rdfs:seeAlso rdf:resource="descriptor/14728-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19392-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11285-3">
    <skos:related rdf:resource="descriptor/11320-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11285-3">
    <skos:related rdf:resource="descriptor/11289-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11285-3">
    <skos:related rdf:resource="descriptor/11286-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13501-6">
    <skos:related rdf:resource="descriptor/13509-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13501-6">
    <skos:related rdf:resource="descriptor/18567-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13501-6">
    <skos:related rdf:resource="descriptor/13499-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18121-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15535-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17177-4">
    <skos:related rdf:resource="descriptor/17480-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14777-2">
    <skos:broader rdf:resource="thsys/73306"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14777-2">
    <skos:broader rdf:resource="descriptor/14776-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14777-2">
    <skos:broader rdf:resource="descriptor/14849-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18825-4">
    <skos:prefLabel xml:lang="en">Administration Information System</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18825-4">
    <skos:prefLabel xml:lang="de">Verwaltungs-Informationssystem</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70461">
    <skos:broader rdf:resource="thsys/70014"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14021-1">
    <skos:broader rdf:resource="thsys/70201"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16036-6">
    <skos:narrower rdf:resource="descriptor/16039-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16036-6">
    <skos:narrower rdf:resource="descriptor/19159-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16036-6">
    <skos:narrower rdf:resource="descriptor/18067-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16036-6">
    <skos:narrower rdf:resource="descriptor/19124-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16036-6">
    <skos:narrower rdf:resource="descriptor/12246-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11715-4">
    <skos:broader rdf:resource="thsys/71083"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11715-4">
    <skos:broader rdf:resource="thsys/70858"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11715-4">
    <skos:broader rdf:resource="descriptor/10062-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11715-4">
    <skos:broader rdf:resource="descriptor/11526-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18353-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19600-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13534-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19856-2">
    <skos:scopeNote xml:lang="de">Handeln in fremdem Namen, das i.d.R. durch Vertretungsmacht gesichert ist.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19016-4">
    <skos:related rdf:resource="descriptor/15763-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19016-4">
    <skos:related rdf:resource="descriptor/15755-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19016-4">
    <skos:related rdf:resource="descriptor/14626-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19016-4">
    <skos:related rdf:resource="descriptor/19792-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19016-4">
    <skos:related rdf:resource="descriptor/18613-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19016-4">
    <skos:related rdf:resource="descriptor/14673-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15057-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12056-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70941">
    <skos:narrower rdf:resource="descriptor/13255-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70941">
    <skos:narrower rdf:resource="descriptor/13527-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70941">
    <skos:narrower rdf:resource="descriptor/18576-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70941">
    <skos:narrower rdf:resource="descriptor/18089-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70941">
    <skos:narrower rdf:resource="descriptor/13574-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70941">
    <skos:narrower rdf:resource="descriptor/13525-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70941">
    <skos:narrower rdf:resource="descriptor/16166-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70941">
    <skos:narrower rdf:resource="descriptor/18575-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70941">
    <skos:narrower rdf:resource="descriptor/18574-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70941">
    <skos:narrower rdf:resource="descriptor/18573-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70941">
    <skos:narrower rdf:resource="descriptor/18572-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70941">
    <skos:narrower rdf:resource="descriptor/18571-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70941">
    <skos:narrower rdf:resource="descriptor/18570-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70941">
    <skos:narrower rdf:resource="descriptor/18569-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70941">
    <skos:narrower rdf:resource="descriptor/18568-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70941">
    <skos:narrower rdf:resource="descriptor/13523-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18062-2">
    <skos:altLabel xml:lang="en">Unification</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18062-2">
    <skos:altLabel xml:lang="en">Reunification</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18062-2">
    <skos:altLabel xml:lang="de">Wiedervereinigung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18062-2">
    <skos:altLabel xml:lang="de">Gesamtdeutsche Frage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18062-2">
    <skos:altLabel xml:lang="de">Deutschlandfrage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18062-2">
    <skos:altLabel xml:lang="de">Deutsche Frage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16312-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18110-3">
    <skos:prefLabel xml:lang="en">Electronic publishing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18110-3">
    <skos:prefLabel xml:lang="de">Elektronisches Publizieren</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16209-1">
    <skos:related rdf:resource="descriptor/16214-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16209-1">
    <skos:related rdf:resource="descriptor/16212-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16209-1">
    <skos:related rdf:resource="descriptor/11063-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16209-1">
    <skos:related rdf:resource="descriptor/16564-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16209-1">
    <skos:related rdf:resource="descriptor/10975-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16209-1">
    <skos:related rdf:resource="descriptor/16292-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16209-1">
    <skos:related rdf:resource="descriptor/16273-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16209-1">
    <skos:related rdf:resource="descriptor/16215-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16209-1">
    <skos:related rdf:resource="descriptor/16205-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16209-1">
    <skos:related rdf:resource="descriptor/16441-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16209-1">
    <skos:related rdf:resource="descriptor/16405-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16209-1">
    <skos:related rdf:resource="descriptor/20855-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16209-1">
    <skos:related rdf:resource="descriptor/19635-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70342">
    <skos:prefLabel xml:lang="en">P.20.01  Processed food and vegetables</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70342">
    <skos:prefLabel xml:lang="de">P.20.01  Verarbeitetes Obst und Gemüse</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72393">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14932-2">
    <skos:altLabel xml:lang="de">Vollwertkost</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14932-2">
    <skos:altLabel xml:lang="de">Naturkost</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14932-2">
    <skos:altLabel xml:lang="de">Bioprodukt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14932-2">
    <skos:altLabel xml:lang="de">Bio-Nahrungsmittel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14932-2">
    <skos:altLabel xml:lang="de">Alternatives Nahrungsmittel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19498-2">
    <skos:broader rdf:resource="thsys/71060"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19498-2">
    <skos:broader rdf:resource="descriptor/11182-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18636-0">
    <skos:broader rdf:resource="thsys/70924"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18636-0">
    <skos:broader rdf:resource="descriptor/13644-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12025-3">
    <skos:altLabel xml:lang="de">Immobilienmaklerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12025-3">
    <skos:altLabel xml:lang="de">Immobilienhändlerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12025-3">
    <skos:altLabel xml:lang="de">Immobilienhändler</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12025-3">
    <skos:altLabel xml:lang="de">Häuserverwalterinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12025-3">
    <skos:altLabel xml:lang="de">Häuserverwalter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12025-3">
    <skos:altLabel xml:lang="de">Grundstücksmaklerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12025-3">
    <skos:altLabel xml:lang="de">Grundstücksmakler</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12025-3">
    <skos:altLabel xml:lang="de">Gebäudemaklerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12025-3">
    <skos:altLabel xml:lang="de">Gebäudemakler</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19456-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19066-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70258">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">P.09.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20277-1">
    <skos:scopeNote xml:lang="de">Gleichzeitige Aufstellung aller Teilpläne unter Berücksichtigung der gegenseitigen Interdependenzen. Vorwiegend in der Produktion und beim Projektmangement.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12898-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11687-4">
    <skos:broader rdf:resource="thsys/71076"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11687-4">
    <skos:broader rdf:resource="descriptor/11570-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11687-4">
    <skos:broader rdf:resource="descriptor/18910-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19125-6">
    <skos:broader rdf:resource="thsys/70101"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17931-3">
    <skos:prefLabel xml:lang="en">South America</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17931-3">
    <skos:prefLabel xml:lang="de">Südamerika</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10469-5">
    <skos:altLabel xml:lang="de">Technokratisches System</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10469-5">
    <skos:altLabel xml:lang="de">Expertenherrschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18971-0">
    <skos:prefLabel xml:lang="en">One-person household</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18971-0">
    <skos:prefLabel xml:lang="de">Alleinlebende</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13565-1">
    <skos:altLabel xml:lang="de">Werkverkehrsrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13565-1">
    <skos:altLabel xml:lang="de">Transportrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13565-1">
    <skos:altLabel xml:lang="de">Speditionsrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13565-1">
    <skos:altLabel xml:lang="de">KVO (Kraftverkehrsordnung)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13565-1">
    <skos:altLabel xml:lang="de">Kraftverkehrsordnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13565-1">
    <skos:altLabel xml:lang="de">Gütertransportrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13565-1">
    <skos:altLabel xml:lang="de">Güterkraftverkehrsgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13565-1">
    <skos:altLabel xml:lang="de">GüKG (Güterkraftverkehrsgesetz)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13565-1">
    <skos:altLabel xml:lang="de">Frachtvertrag</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13565-1">
    <skos:altLabel xml:lang="de">Frachtführer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13565-1">
    <skos:altLabel xml:lang="de">Frachtbrief</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13565-1">
    <skos:altLabel xml:lang="de">Allgemeine Deutsche Spediteur-Bedingungen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13565-1">
    <skos:altLabel xml:lang="de">ADSp (Allgemeine Deutsche Spediteur-Bedingungen)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11231-5">
    <skos:altLabel xml:lang="en">Internal labor market</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16033-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18936-2">
    <skos:broader rdf:resource="thsys/70824"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18936-2">
    <skos:broader rdf:resource="thsys/70384"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18936-2">
    <skos:broader rdf:resource="descriptor/12825-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11910-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13100-3">
    <skos:broader rdf:resource="thsys/70990"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13100-3">
    <skos:broader rdf:resource="descriptor/13117-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19794-5">
    <skos:prefLabel xml:lang="en">Electromagnetic field</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19794-5">
    <skos:prefLabel xml:lang="de">Elektromagnetisches Feld</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19672-5">
    <skos:prefLabel xml:lang="en">Core competence</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19672-5">
    <skos:prefLabel xml:lang="de">Kernkompetenz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19833-0">
    <skos:broader rdf:resource="thsys/71036"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19833-0">
    <skos:broader rdf:resource="descriptor/12165-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13451-2">
    <skos:scopeNote xml:lang="de">Gesamtheit der rechtlichen und sozialen Rahmenbedingungen, die für Landbevölkerung, Landwirtschaft und den ländlichen Raum maßgebend sind.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16329-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11322-2">
    <skos:prefLabel xml:lang="en">Wages</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11322-2">
    <skos:prefLabel xml:lang="de">Lohn</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14744-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10019-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10673-6">
    <skos:broader rdf:resource="thsys/71099"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10673-6">
    <skos:broader rdf:resource="descriptor/10769-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13975-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18707-3">
    <skos:related rdf:resource="descriptor/13693-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18707-3">
    <skos:related rdf:resource="descriptor/13690-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18707-3">
    <skos:related rdf:resource="descriptor/13283-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14029-6">
    <skos:broader rdf:resource="thsys/70201"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18076-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19856-2">
    <skos:prefLabel xml:lang="en">Proxy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19856-2">
    <skos:prefLabel xml:lang="de">Stellvertretung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13083-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18479-4">
    <skos:altLabel xml:lang="de">Industrieller Mittelstand</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15472-5">
    <skos:broader rdf:resource="thsys/73358"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15472-5">
    <skos:broader rdf:resource="thsys/73103"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15472-5">
    <skos:broader rdf:resource="thsys/71035"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15472-5">
    <skos:broader rdf:resource="thsys/70853"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15472-5">
    <skos:broader rdf:resource="descriptor/15466-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13594-1">
    <skos:broader rdf:resource="thsys/73354"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13594-1">
    <skos:broader rdf:resource="thsys/70925"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13594-1">
    <skos:broader rdf:resource="descriptor/15902-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13594-1">
    <skos:broader rdf:resource="descriptor/13573-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70184">
    <skos:broader rdf:resource="thsys/70046"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18920-3">
    <skos:altLabel xml:lang="de">Künstlerischer Sektor</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18920-3">
    <skos:altLabel xml:lang="de">Kulturwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18920-3">
    <skos:altLabel xml:lang="de">Kulturindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16190-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14809-1">
    <skos:prefLabel xml:lang="en">Special paper</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14809-1">
    <skos:prefLabel xml:lang="de">Spezialpapier</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19110-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12736-5">
    <skos:altLabel xml:lang="de">Manipulatorsystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19020-6">
    <skos:broader rdf:resource="thsys/70467"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19020-6">
    <skos:broader rdf:resource="descriptor/13427-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24733-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16076-1">
    <skos:broader rdf:resource="thsys/73391"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16076-1">
    <skos:broader rdf:resource="thsys/70087"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16076-1">
    <skos:broader rdf:resource="descriptor/16050-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13067-3">
    <skos:altLabel xml:lang="de">WZM-Industrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15894-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16278-3">
    <skos:prefLabel xml:lang="en">Political integration</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16278-3">
    <skos:prefLabel xml:lang="de">Politische Integration</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id351">
    <rdfs:seeAlso rdf:resource="descriptor/18628-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12021-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70651">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10195-2">
    <skos:broader rdf:resource="thsys/71119"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10195-2">
    <skos:broader rdf:resource="descriptor/10205-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12529-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19642-0">
    <skos:prefLabel xml:lang="en">Coloured persons</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19642-0">
    <skos:prefLabel xml:lang="de">Farbige Bevölkerung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10978-5">
    <skos:altLabel xml:lang="en">Price regulation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10978-5">
    <skos:altLabel xml:lang="de">Staatliche Preisfestsetzung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10978-5">
    <skos:altLabel xml:lang="de">Preissubventionierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10978-5">
    <skos:altLabel xml:lang="de">Preisstopp</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10978-5">
    <skos:altLabel xml:lang="de">Preisstabilisierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10978-5">
    <skos:altLabel xml:lang="de">Preisregulierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10978-5">
    <skos:altLabel xml:lang="de">Preisreform</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10978-5">
    <skos:altLabel xml:lang="de">Preispolitik des Staates</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10978-5">
    <skos:altLabel xml:lang="de">Preisliberalisierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10978-5">
    <skos:altLabel xml:lang="de">Öffentliche Preisbildung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10978-5">
    <skos:altLabel xml:lang="de">Mindestpreisregelung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18899-3">
    <skos:broader rdf:resource="thsys/73346"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18899-3">
    <skos:broader rdf:resource="descriptor/16670-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13785-1">
    <skos:prefLabel xml:lang="en">Underwriting services</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13785-1">
    <skos:prefLabel xml:lang="de">Emissionsgeschäft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19404-2">
    <skos:related rdf:resource="descriptor/11636-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19404-2">
    <skos:related rdf:resource="descriptor/11914-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19404-2">
    <skos:related rdf:resource="descriptor/11701-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19404-2">
    <skos:related rdf:resource="descriptor/19283-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19404-2">
    <skos:related rdf:resource="descriptor/19941-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19404-2">
    <skos:related rdf:resource="descriptor/10827-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10730-6">
    <skos:broader rdf:resource="thsys/71095"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10730-6">
    <skos:broader rdf:resource="descriptor/10764-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71056">
    <skos:prefLabel xml:lang="de">V.13.06.01  Technische und Fertigungsberufe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19420-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13964-1">
    <skos:related rdf:resource="descriptor/14017-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11929-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15646-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70258">
    <skos:narrower rdf:resource="descriptor/14444-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70258">
    <skos:narrower rdf:resource="descriptor/14442-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70258">
    <skos:narrower rdf:resource="descriptor/14441-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70258">
    <skos:narrower rdf:resource="descriptor/14428-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70258">
    <skos:narrower rdf:resource="descriptor/14440-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70258">
    <skos:narrower rdf:resource="descriptor/14439-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70258">
    <skos:narrower rdf:resource="descriptor/14438-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70258">
    <skos:narrower rdf:resource="descriptor/14437-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70258">
    <skos:narrower rdf:resource="descriptor/14427-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70258">
    <skos:narrower rdf:resource="descriptor/14436-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70258">
    <skos:narrower rdf:resource="descriptor/14435-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70258">
    <skos:narrower rdf:resource="descriptor/14434-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70258">
    <skos:narrower rdf:resource="descriptor/14433-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70258">
    <skos:narrower rdf:resource="descriptor/18294-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70258">
    <skos:narrower rdf:resource="descriptor/14412-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70258">
    <skos:narrower rdf:resource="descriptor/14432-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70258">
    <skos:narrower rdf:resource="descriptor/19053-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71077">
    <rdfs:label xml:lang="de">Steuern und Steuerwissenschaft</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15082-4">
    <skos:related rdf:resource="descriptor/15097-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19320-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19774-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10171-2">
    <skos:related rdf:resource="descriptor/19552-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10171-2">
    <skos:related rdf:resource="descriptor/10182-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10171-2">
    <skos:related rdf:resource="descriptor/10181-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10171-2">
    <skos:related rdf:resource="descriptor/19197-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19782-5">
    <skos:prefLabel xml:lang="en">Natural science research</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19782-5">
    <skos:prefLabel xml:lang="de">Naturwissenschaftliche Forschung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15470-2">
    <skos:related rdf:resource="descriptor/19581-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15470-2">
    <skos:related rdf:resource="descriptor/15458-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15470-2">
    <skos:related rdf:resource="descriptor/19150-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15470-2">
    <skos:related rdf:resource="descriptor/10164-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15470-2">
    <skos:related rdf:resource="descriptor/15483-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15470-2">
    <skos:related rdf:resource="descriptor/15415-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15470-2">
    <skos:related rdf:resource="descriptor/15498-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15470-2">
    <skos:related rdf:resource="descriptor/15466-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15470-2">
    <skos:related rdf:resource="descriptor/20756-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15470-2">
    <skos:related rdf:resource="descriptor/10138-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15814-3">
    <skos:altLabel xml:lang="de">Soziometrik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15814-3">
    <skos:altLabel xml:lang="de">Soziogramm</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10852-6">
    <skos:altLabel xml:lang="de">Exportwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10852-6">
    <skos:altLabel xml:lang="de">Außenhandelsgewerbe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11770-3">
    <skos:prefLabel xml:lang="en">Income redistribution</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11770-3">
    <skos:prefLabel xml:lang="de">Einkommensumverteilung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18934-6">
    <skos:related rdf:resource="descriptor/18929-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18934-6">
    <skos:related rdf:resource="descriptor/13655-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18934-6">
    <skos:related rdf:resource="descriptor/18935-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16001-4">
    <skos:altLabel xml:lang="de">Atomrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16001-4">
    <skos:altLabel xml:lang="de">Atomgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72394">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19157-0">
    <skos:scopeNote xml:lang="de">Rechtsform für freie Berufe, z.B. Gemeinschaftspraxen und Anwaltssozietäten</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16196-5">
    <skos:broader rdf:resource="thsys/73356"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16196-5">
    <skos:broader rdf:resource="thsys/73334"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16196-5">
    <skos:broader rdf:resource="descriptor/16563-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11416-0">
    <skos:broader rdf:resource="thsys/73387"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11416-0">
    <skos:broader rdf:resource="thsys/70109"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11416-0">
    <skos:broader rdf:resource="thsys/70055"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11416-0">
    <skos:broader rdf:resource="descriptor/15706-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16391-0">
    <skos:prefLabel xml:lang="en">Peace movement</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16391-0">
    <skos:prefLabel xml:lang="de">Friedensbewegung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16432-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10856-5">
    <skos:prefLabel xml:lang="en">Foreign trade agency</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10856-5">
    <skos:prefLabel xml:lang="de">Außenhandelsvertretung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71029">
    <skos:broader rdf:resource="thsys/71028"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13295-4">
    <skos:broader rdf:resource="thsys/70891"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13295-4">
    <skos:broader rdf:resource="descriptor/13297-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11201-0">
    <skos:prefLabel xml:lang="en">Child labour</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11201-0">
    <skos:prefLabel xml:lang="de">Kinderarbeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14366-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10504-1">
    <skos:related rdf:resource="descriptor/10512-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10504-1">
    <skos:related rdf:resource="descriptor/11993-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10504-1">
    <skos:related rdf:resource="descriptor/19477-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10504-1">
    <skos:related rdf:resource="descriptor/11902-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10504-1">
    <skos:related rdf:resource="descriptor/11969-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10504-1">
    <skos:related rdf:resource="descriptor/10503-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10504-1">
    <skos:related rdf:resource="descriptor/19362-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10504-1">
    <skos:related rdf:resource="descriptor/19461-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19095-3">
    <skos:prefLabel xml:lang="en">Capital gain</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19095-3">
    <skos:prefLabel xml:lang="de">Veräußerungsgewinn</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14153-5">
    <skos:broader rdf:resource="thsys/70465"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14153-5">
    <skos:broader rdf:resource="descriptor/14150-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13669-3">
    <skos:altLabel xml:lang="de">Publikumszeitschrift</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13669-3">
    <skos:altLabel xml:lang="de">Magazin (Zeitschrift)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13669-3">
    <skos:altLabel xml:lang="de">Illustrierte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14383-2">
    <skos:broader rdf:resource="thsys/70318"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14383-2">
    <skos:broader rdf:resource="descriptor/14381-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10635-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15211-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71035">
    <skos:narrower rdf:resource="descriptor/15067-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71035">
    <skos:narrower rdf:resource="descriptor/15060-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71035">
    <skos:narrower rdf:resource="descriptor/15520-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71035">
    <skos:narrower rdf:resource="descriptor/15519-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71035">
    <skos:narrower rdf:resource="descriptor/15518-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71035">
    <skos:narrower rdf:resource="descriptor/19073-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71035">
    <skos:narrower rdf:resource="descriptor/15429-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71035">
    <skos:narrower rdf:resource="descriptor/15225-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71035">
    <skos:narrower rdf:resource="descriptor/15458-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71035">
    <skos:narrower rdf:resource="descriptor/15057-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71035">
    <skos:narrower rdf:resource="descriptor/15515-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71035">
    <skos:narrower rdf:resource="descriptor/12761-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71035">
    <skos:narrower rdf:resource="descriptor/15340-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71035">
    <skos:narrower rdf:resource="descriptor/19150-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71035">
    <skos:narrower rdf:resource="descriptor/19149-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71035">
    <skos:narrower rdf:resource="descriptor/15125-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71035">
    <skos:narrower rdf:resource="descriptor/15483-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71035">
    <skos:narrower rdf:resource="descriptor/15543-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71035">
    <skos:narrower rdf:resource="descriptor/15051-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71035">
    <skos:narrower rdf:resource="descriptor/19148-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71035">
    <skos:narrower rdf:resource="descriptor/15267-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71035">
    <skos:narrower rdf:resource="descriptor/15055-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71035">
    <skos:narrower rdf:resource="descriptor/15220-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71035">
    <skos:narrower rdf:resource="descriptor/16716-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71035">
    <skos:narrower rdf:resource="descriptor/15119-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71035">
    <skos:narrower rdf:resource="descriptor/15571-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71035">
    <skos:narrower rdf:resource="descriptor/15415-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71035">
    <skos:narrower rdf:resource="descriptor/15555-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71035">
    <skos:narrower rdf:resource="descriptor/12103-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71035">
    <skos:narrower rdf:resource="descriptor/15498-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71035">
    <skos:narrower rdf:resource="descriptor/15472-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71035">
    <skos:narrower rdf:resource="descriptor/15038-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71035">
    <skos:narrower rdf:resource="descriptor/15535-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71035">
    <skos:narrower rdf:resource="descriptor/12207-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71035">
    <skos:narrower rdf:resource="descriptor/15399-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71035">
    <skos:narrower rdf:resource="descriptor/15494-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71035">
    <skos:narrower rdf:resource="descriptor/15470-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71035">
    <skos:narrower rdf:resource="descriptor/19147-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71035">
    <skos:narrower rdf:resource="descriptor/15469-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71035">
    <skos:narrower rdf:resource="descriptor/19070-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71035">
    <skos:narrower rdf:resource="descriptor/15554-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71035">
    <skos:narrower rdf:resource="descriptor/15493-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71035">
    <skos:narrower rdf:resource="descriptor/15491-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71035">
    <skos:narrower rdf:resource="descriptor/19609-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71035">
    <skos:narrower rdf:resource="descriptor/19542-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71035">
    <skos:narrower rdf:resource="descriptor/15488-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71035">
    <skos:narrower rdf:resource="descriptor/12747-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11845-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10434-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18097-4">
    <skos:broader rdf:resource="thsys/71045"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18097-4">
    <skos:broader rdf:resource="thsys/70856"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18097-4">
    <skos:broader rdf:resource="descriptor/25742-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10411-1">
    <skos:prefLabel xml:lang="en">Limits to growth</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10411-1">
    <skos:prefLabel xml:lang="de">Grenzen des Wachstums</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10854-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70188">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10229-4">
    <skos:altLabel xml:lang="de">Disaggregation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10229-4">
    <skos:altLabel xml:lang="de">Aggregationsverfahren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10229-4">
    <skos:altLabel xml:lang="de">Aggregationstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10996-3">
    <skos:prefLabel xml:lang="en">Employment planning</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10996-3">
    <skos:prefLabel xml:lang="de">Arbeitskräftelenkung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id380">
    <rdf:value xml:lang="de">Für Willingness-to-pay-Ansätze</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18543-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19677-2">
    <skos:related rdf:resource="descriptor/19545-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19677-2">
    <skos:related rdf:resource="descriptor/19183-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19677-2">
    <skos:related rdf:resource="descriptor/14124-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10711-3">
    <skos:narrower rdf:resource="descriptor/10695-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11525-2">
    <skos:broader rdf:resource="thsys/73337"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11525-2">
    <skos:broader rdf:resource="thsys/71073"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11525-2">
    <skos:broader rdf:resource="descriptor/16007-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15340-1">
    <skos:related rdf:resource="descriptor/15396-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15340-1">
    <skos:related rdf:resource="descriptor/15376-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15340-1">
    <skos:related rdf:resource="descriptor/15372-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15340-1">
    <skos:related rdf:resource="descriptor/15330-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15340-1">
    <skos:related rdf:resource="descriptor/15328-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17610-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10961-1">
    <skos:altLabel xml:lang="en">Unbundling</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10961-1">
    <skos:altLabel xml:lang="en">Ownership unbundling</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10961-1">
    <skos:altLabel xml:lang="en">Demerger</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10961-1">
    <skos:altLabel xml:lang="de">Entbündelung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16825-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11402-4">
    <skos:related rdf:resource="descriptor/16487-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11402-4">
    <skos:related rdf:resource="descriptor/18876-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11402-4">
    <skos:related rdf:resource="descriptor/18874-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11402-4">
    <skos:related rdf:resource="descriptor/11373-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11402-4">
    <skos:related rdf:resource="descriptor/10437-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18896-2">
    <skos:prefLabel xml:lang="en">Addictive buying</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18896-2">
    <skos:prefLabel xml:lang="de">Kaufsucht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10472-2">
    <skos:altLabel xml:lang="en">Transfer of technology</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10472-2">
    <skos:altLabel xml:lang="en">Technology gap</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10472-2">
    <skos:altLabel xml:lang="en">Technological gap</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10472-2">
    <skos:altLabel xml:lang="en">Import of technology</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10472-2">
    <skos:altLabel xml:lang="en">Export of technology</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10472-2">
    <skos:altLabel xml:lang="de">Technologische Lücke</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10472-2">
    <skos:altLabel xml:lang="de">Techniktransfer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10472-2">
    <skos:altLabel xml:lang="de">Lizenzexport</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12562-5">
    <skos:broader rdf:resource="thsys/70244"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12562-5">
    <skos:broader rdf:resource="descriptor/19068-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11915-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15072-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11526-0">
    <skos:narrower rdf:resource="descriptor/11637-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11526-0">
    <skos:narrower rdf:resource="descriptor/19206-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11526-0">
    <skos:narrower rdf:resource="descriptor/11715-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18278-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10807-4">
    <skos:prefLabel xml:lang="en">International financial market</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10807-4">
    <skos:prefLabel xml:lang="de">Internationaler Finanzmarkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12125-6">
    <skos:related rdf:resource="descriptor/12121-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12125-6">
    <skos:related rdf:resource="descriptor/12091-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12284-0">
    <skos:altLabel xml:lang="en">Seasoned equity offerings</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12284-0">
    <skos:altLabel xml:lang="de">Schütt-aus-hol-zurück-Verfahren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12284-0">
    <skos:altLabel xml:lang="de">SAHZ-Verfahren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12284-0">
    <skos:altLabel xml:lang="de">Ausschüttungs-Rückhol-Verfahren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id210">
    <rdfs:seeAlso rdf:resource="descriptor/18941-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10397-4">
    <skos:related rdf:resource="descriptor/10978-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10397-4">
    <skos:related rdf:resource="descriptor/11519-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10397-4">
    <skos:related rdf:resource="descriptor/15093-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10397-4">
    <skos:related rdf:resource="descriptor/11507-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10397-4">
    <skos:related rdf:resource="descriptor/11494-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10397-4">
    <skos:related rdf:resource="descriptor/11427-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14076-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10768-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13862-2">
    <skos:altLabel xml:lang="en">Theory of residential location</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13862-2">
    <skos:altLabel xml:lang="en">Residential-location theory</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13862-2">
    <skos:altLabel xml:lang="en">Residential location theory</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13862-2">
    <skos:altLabel xml:lang="en">Residential environment</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13862-2">
    <skos:altLabel xml:lang="en">Residence location</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13862-2">
    <skos:altLabel xml:lang="en">Living environment</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13862-2">
    <skos:altLabel xml:lang="de">Wohnumwelt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13862-2">
    <skos:altLabel xml:lang="de">Wohnumgebung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13862-2">
    <skos:altLabel xml:lang="de">Wohnumfeld</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13862-2">
    <skos:altLabel xml:lang="de">Wohnstandortwahl</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13862-2">
    <skos:altLabel xml:lang="de">Wohnstandorttheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13862-2">
    <skos:altLabel xml:lang="de">Wohnortwahl</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13862-2">
    <skos:altLabel xml:lang="de">Wohnlage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13862-2">
    <skos:altLabel xml:lang="de">Wohngebiet</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13862-2">
    <skos:altLabel xml:lang="de">Nahbereich der Wohnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13862-2">
    <skos:altLabel xml:lang="de">Nachbarschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21072-0">
    <skos:related rdf:resource="descriptor/13826-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21072-0">
    <skos:related rdf:resource="descriptor/13827-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21072-0">
    <skos:related rdf:resource="descriptor/18994-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16120-3">
    <skos:prefLabel xml:lang="en">Public accounting legislation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16120-3">
    <skos:prefLabel xml:lang="de">Wirtschaftsprüferrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#prefLabel">
    <rdfs:label xml:lang="en">preferred label</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70269">
    <rdfs:label xml:lang="en">Mathematics</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70269">
    <rdfs:label xml:lang="de">Mathematik</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12119-1">
    <skos:related rdf:resource="descriptor/15714-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12119-1">
    <skos:related rdf:resource="descriptor/19116-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12119-1">
    <skos:related rdf:resource="descriptor/12580-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12119-1">
    <skos:related rdf:resource="descriptor/12720-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13660-0">
    <skos:narrower rdf:resource="descriptor/16012-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13660-0">
    <skos:narrower rdf:resource="descriptor/16010-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16668-4">
    <skos:prefLabel xml:lang="en">Drug consumption</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16668-4">
    <skos:prefLabel xml:lang="de">Drogenkonsum</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10787-5">
    <skos:prefLabel xml:lang="en">Current account balance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10787-5">
    <skos:prefLabel xml:lang="de">Leistungsbilanz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15951-0">
    <skos:broader rdf:resource="thsys/70572"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15951-0">
    <skos:broader rdf:resource="descriptor/15954-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16043-2">
    <skos:related rdf:resource="descriptor/16080-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16043-2">
    <skos:related rdf:resource="descriptor/19535-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16043-2">
    <skos:related rdf:resource="descriptor/13713-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16043-2">
    <skos:related rdf:resource="descriptor/12174-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16043-2">
    <skos:related rdf:resource="descriptor/12232-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12393-2">
    <skos:related rdf:resource="descriptor/19106-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12393-2">
    <skos:related rdf:resource="descriptor/12365-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19447-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18856-0">
    <skos:altLabel xml:lang="de">Studienabbruch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18856-0">
    <skos:altLabel xml:lang="de">Schulabbrecherinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18856-0">
    <skos:altLabel xml:lang="de">Schulabbrecher</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18856-0">
    <skos:altLabel xml:lang="de">Ausbildungsabbruch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18856-0">
    <skos:altLabel xml:lang="de">Abbrecherinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18392-2">
    <skos:broader rdf:resource="thsys/71007"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18392-2">
    <skos:broader rdf:resource="descriptor/12988-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19125-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15968-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20321-3">
    <skos:related rdf:resource="descriptor/11897-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20321-3">
    <skos:related rdf:resource="descriptor/19312-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20321-3">
    <skos:related rdf:resource="descriptor/19364-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20321-3">
    <skos:related rdf:resource="descriptor/11919-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10123-6">
    <skos:related rdf:resource="descriptor/10460-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10123-6">
    <skos:related rdf:resource="descriptor/10125-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10123-6">
    <skos:related rdf:resource="descriptor/10120-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10323-5">
    <skos:related rdf:resource="descriptor/19250-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10323-5">
    <skos:related rdf:resource="descriptor/12055-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10323-5">
    <skos:related rdf:resource="descriptor/10303-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19117-5">
    <skos:prefLabel xml:lang="en">Computer-Aided Quality Assurance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19117-5">
    <skos:prefLabel xml:lang="de">CAQ</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15637-6">
    <skos:altLabel xml:lang="de">Lamaismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12234-1">
    <skos:editorialNote xml:lang="de">Möglichst spezifischer indexieren.</skos:editorialNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10882-4">
    <zbwext:useInsteadNote rdf:nodeID="id301"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18862-5">
    <skos:broader rdf:resource="thsys/70142"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18862-5">
    <skos:broader rdf:resource="descriptor/15925-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18290-3">
    <skos:altLabel xml:lang="en">LC-Display</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18290-3">
    <skos:altLabel xml:lang="en">LCD (Liquid Cristal Display)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18684-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10838-0">
    <skos:broader rdf:resource="thsys/71091"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10838-0">
    <skos:broader rdf:resource="descriptor/10833-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13761-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18524-4">
    <skos:prefLabel xml:lang="en">Small trader</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18524-4">
    <skos:prefLabel xml:lang="de">Mittelständischer Handel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70076">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18500-4">
    <skos:prefLabel xml:lang="en">Pipeline construction</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18500-4">
    <skos:prefLabel xml:lang="de">Rohrleitungsbau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20722-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12381-2">
    <skos:prefLabel xml:lang="en">Annual accounts</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12381-2">
    <skos:prefLabel xml:lang="de">Jahresabschluss</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16286-4">
    <skos:scopeNote xml:lang="de">Kollektive, meist gewaltsame Maßnahmen von Teilen der Bevölkerung, die auf eine Veränderung der bestehenden Staats- und Rechtsordnung abzielen</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20485-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19510-3">
    <skos:scopeNote xml:lang="de">Gewährung sozialer Leistungen in Naturalien statt in Geld.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12366-5">
    <skos:prefLabel xml:lang="en">Financial assets</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12366-5">
    <skos:prefLabel xml:lang="de">Betriebliches Finanzanlagevermögen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14673-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13622-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73348">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12188-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14479-3">
    <skos:altLabel xml:lang="de">Wechselstromgenerator</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14479-3">
    <skos:altLabel xml:lang="de">Stromerzeugungsaggregat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14479-3">
    <skos:altLabel xml:lang="de">Gleichstromgenerator</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10598-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10215-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12105-5">
    <skos:prefLabel xml:lang="en">Organizational change</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12105-5">
    <skos:prefLabel xml:lang="de">Organisatorischer Wandel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10728-0">
    <skos:broader rdf:resource="thsys/71094"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10728-0">
    <skos:broader rdf:resource="descriptor/10746-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19299-1">
    <skos:related rdf:resource="descriptor/10625-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19299-1">
    <skos:related rdf:resource="descriptor/10624-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19299-1">
    <skos:related rdf:resource="descriptor/16809-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19299-1">
    <skos:related rdf:resource="descriptor/18160-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19299-1">
    <skos:related rdf:resource="descriptor/10842-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19299-1">
    <skos:related rdf:resource="descriptor/10617-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19299-1">
    <skos:related rdf:resource="descriptor/19071-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19299-1">
    <skos:related rdf:resource="descriptor/16502-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11970-2">
    <skos:altLabel xml:lang="de">Stadttyp</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11970-2">
    <skos:altLabel xml:lang="de">Städtischer Raum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11970-2">
    <skos:altLabel xml:lang="de">Stadtgebiet</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11970-2">
    <skos:altLabel xml:lang="de">Stadtbegriff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18834-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16001-4">
    <skos:prefLabel xml:lang="en">Nuclear law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16001-4">
    <skos:prefLabel xml:lang="de">Kernenergierecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18073-4">
    <skos:related rdf:resource="descriptor/11067-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18073-4">
    <skos:related rdf:resource="descriptor/18137-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18073-4">
    <skos:related rdf:resource="descriptor/19680-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18073-4">
    <skos:related rdf:resource="descriptor/10501-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18073-4">
    <skos:related rdf:resource="descriptor/18673-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18073-4">
    <skos:related rdf:resource="descriptor/11825-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18073-4">
    <skos:related rdf:resource="descriptor/19773-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18073-4">
    <skos:related rdf:resource="descriptor/10487-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19400-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18628-6">
    <skos:broader rdf:resource="thsys/70926"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18628-6">
    <skos:broader rdf:resource="descriptor/11833-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18628-6">
    <skos:broader rdf:resource="descriptor/15768-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70403">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19410-0">
    <skos:prefLabel xml:lang="en">Tax base</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19410-0">
    <skos:prefLabel xml:lang="de">Steuerbemessung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71015">
    <rdfs:label xml:lang="en">Electric utility industry</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71015">
    <rdfs:label xml:lang="de">Elektrizitätswirtschaft</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73352">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14661-0">
    <skos:broader rdf:resource="thsys/70556"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14661-0">
    <skos:broader rdf:resource="descriptor/14656-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12725-3">
    <zbwext:useInsteadNote rdf:nodeID="id98"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11447-3">
    <skos:broader rdf:resource="thsys/70306"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11447-3">
    <skos:broader rdf:resource="descriptor/11495-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21282-3">
    <skos:related rdf:resource="descriptor/18106-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70957">
    <rdfs:label xml:lang="de">Rohstoff- und Altmaterialhandel</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10314-6">
    <skos:narrower rdf:resource="descriptor/10124-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10314-6">
    <skos:narrower rdf:resource="descriptor/10615-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10314-6">
    <skos:narrower rdf:resource="descriptor/11184-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18197-0">
    <skos:broader rdf:resource="thsys/71029"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18197-0">
    <skos:broader rdf:resource="descriptor/18198-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10960-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11174-5">
    <skos:prefLabel xml:lang="en">Radical economics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11174-5">
    <skos:prefLabel xml:lang="de">Radical Economics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18453-1">
    <skos:prefLabel xml:lang="en">Industrial pollution</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18453-1">
    <skos:prefLabel xml:lang="de">Industrielle Umweltbelastung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19056-6">
    <skos:prefLabel xml:lang="en">Stucco masons</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19056-6">
    <skos:prefLabel xml:lang="de">Stuckateurgewerbe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10778-6">
    <skos:prefLabel xml:lang="en">Reserve transactions account</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10778-6">
    <skos:prefLabel xml:lang="de">Devisenbilanz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18569-3">
    <skos:altLabel xml:lang="de">Schienenverkehrsberufe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18569-3">
    <skos:altLabel xml:lang="de">Lokomotivführerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18569-3">
    <skos:altLabel xml:lang="de">Lokomotivführer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18569-3">
    <skos:altLabel xml:lang="de">Eisenbahnerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18569-3">
    <skos:altLabel xml:lang="de">Eisenbahner</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18569-3">
    <skos:altLabel xml:lang="de">Eisenbahnarbeiterinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18569-3">
    <skos:altLabel xml:lang="de">Eisenbahnarbeiter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19044-6">
    <skos:prefLabel xml:lang="en">Comparison</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19044-6">
    <skos:prefLabel xml:lang="de">Vergleich</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16038-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13146-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19509-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10452-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16953-5">
    <skos:broader rdf:resource="thsys/70438"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16953-5">
    <skos:broader rdf:resource="descriptor/16950-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19408-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11189-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12030-3">
    <skos:related rdf:resource="descriptor/18750-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13807-3">
    <skos:prefLabel xml:lang="en">Payments</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13807-3">
    <skos:prefLabel xml:lang="de">Zahlungsverkehr</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10300-3">
    <skos:related rdf:resource="descriptor/15859-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10300-3">
    <skos:related rdf:resource="descriptor/12837-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10300-3">
    <skos:related rdf:resource="descriptor/15858-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10300-3">
    <skos:related rdf:resource="descriptor/19236-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10300-3">
    <skos:related rdf:resource="descriptor/10298-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10300-3">
    <skos:related rdf:resource="descriptor/15860-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10300-3">
    <skos:related rdf:resource="descriptor/15801-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10300-3">
    <skos:related rdf:resource="descriptor/12774-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10300-3">
    <skos:related rdf:resource="descriptor/19754-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10300-3">
    <skos:related rdf:resource="descriptor/10181-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10300-3">
    <skos:related rdf:resource="descriptor/16605-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10300-3">
    <skos:related rdf:resource="descriptor/18896-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10300-3">
    <skos:related rdf:resource="descriptor/19118-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10300-3">
    <skos:related rdf:resource="descriptor/18916-1"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id93">
    <rdf:value xml:lang="de">Für die Planung der beruflichen Laufbahn</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13415-6">
    <skos:altLabel xml:lang="de">Soziale Krankenversicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13415-6">
    <skos:altLabel xml:lang="de">Krankenversicherung des Staates</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13415-6">
    <skos:altLabel xml:lang="de">Krankenkasse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13415-6">
    <skos:altLabel xml:lang="de">Krankengeld</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13415-6">
    <skos:altLabel xml:lang="de">GKV</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15515-6">
    <skos:prefLabel xml:lang="en">Sensitivity analysis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15515-6">
    <skos:prefLabel xml:lang="de">Sensitivitätsanalyse</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10510-6">
    <zbwext:useInsteadNote rdf:nodeID="id306"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10510-6">
    <zbwext:useInsteadNote rdf:nodeID="id305"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13084-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14802-1">
    <skos:prefLabel xml:lang="en">Paper</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14802-1">
    <skos:prefLabel xml:lang="de">Papier</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19916-3">
    <skos:altLabel xml:lang="de">Bildungsabschluß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71099">
    <skos:prefLabel xml:lang="en">V.07.01.01  International monetary theory</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71099">
    <skos:prefLabel xml:lang="de">V.07.01.01  Monetäre Außenwirtschaftstheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13940-1">
    <skos:broader rdf:resource="thsys/70986"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13940-1">
    <skos:broader rdf:resource="thsys/70964"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13940-1">
    <skos:broader rdf:resource="descriptor/13160-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13940-1">
    <skos:broader rdf:resource="descriptor/12910-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70188">
    <skos:narrower rdf:resource="descriptor/10087-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70188">
    <skos:narrower rdf:resource="descriptor/10085-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70188">
    <skos:narrower rdf:resource="descriptor/11927-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70188">
    <skos:narrower rdf:resource="descriptor/10081-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70188">
    <skos:narrower rdf:resource="descriptor/10080-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70188">
    <skos:narrower rdf:resource="descriptor/10082-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70188">
    <skos:narrower rdf:resource="descriptor/10077-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70188">
    <skos:narrower rdf:resource="descriptor/10076-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70188">
    <skos:narrower rdf:resource="descriptor/10075-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70188">
    <skos:narrower rdf:resource="descriptor/10073-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70188">
    <skos:narrower rdf:resource="descriptor/10070-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19141-1">
    <skos:broader rdf:resource="thsys/70292"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19141-1">
    <skos:broader rdf:resource="descriptor/15909-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20409-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19414-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18289-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12628-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14001-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18524-4">
    <skos:broader rdf:resource="thsys/70963"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18524-4">
    <skos:broader rdf:resource="descriptor/18120-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18524-4">
    <skos:broader rdf:resource="descriptor/11824-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19818-3">
    <skos:scopeNote xml:lang="de">Rechtliche Regelungen für bestimmte, meist freie Berufe. In Deutschland gibt es kaum noch kodifiziertes Berufsrecht.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19442-1">
    <skos:related rdf:resource="descriptor/15767-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19442-1">
    <skos:related rdf:resource="descriptor/19203-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17682-5">
    <skos:prefLabel xml:lang="en">Lesotho</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17682-5">
    <skos:prefLabel xml:lang="de">Lesotho</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10893-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12520-0">
    <skos:related rdf:resource="descriptor/12796-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12520-0">
    <skos:related rdf:resource="descriptor/19153-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12520-0">
    <skos:related rdf:resource="descriptor/12535-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12520-0">
    <skos:related rdf:resource="descriptor/12474-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14433-6">
    <skos:broader rdf:resource="thsys/70258"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14433-6">
    <skos:broader rdf:resource="descriptor/14441-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14216-0">
    <skos:prefLabel xml:lang="en">Motor fuel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14216-0">
    <skos:prefLabel xml:lang="de">Kraftstoff</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70361">
    <skos:broader rdf:resource="thsys/70554"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10772-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19681-4">
    <skos:scopeNote xml:lang="de">Dauernder oder zeitlich begrenzter Zusammenschluss souveräner Staaten zu einem Staatenverband.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id89">
    <rdf:value xml:lang="de">Für theoretische Literatur</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18007-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18698-6">
    <skos:related rdf:resource="descriptor/18725-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18698-6">
    <skos:related rdf:resource="descriptor/18696-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18698-6">
    <skos:related rdf:resource="descriptor/11459-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18698-6">
    <skos:related rdf:resource="descriptor/11433-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18698-6">
    <skos:related rdf:resource="descriptor/12251-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18698-6">
    <skos:related rdf:resource="descriptor/10382-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20789-2">
    <skos:broader rdf:resource="thsys/70301"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20789-2">
    <skos:broader rdf:resource="descriptor/10016-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18709-6">
    <skos:broader rdf:resource="thsys/71052"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18709-6">
    <skos:broader rdf:resource="thsys/70905"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18709-6">
    <skos:broader rdf:resource="descriptor/11302-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18709-6">
    <skos:broader rdf:resource="descriptor/13965-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17687-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19808-6">
    <skos:prefLabel xml:lang="en">Neural networks</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19808-6">
    <skos:prefLabel xml:lang="de">Neuronale Netze</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14117-2">
    <skos:altLabel xml:lang="de">Subtropische Frucht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14117-2">
    <skos:altLabel xml:lang="de">Papaya</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14117-2">
    <skos:altLabel xml:lang="de">Mango</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14117-2">
    <skos:altLabel xml:lang="de">Kolanuss</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14117-2">
    <skos:altLabel xml:lang="de">Kolanuß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14117-2">
    <skos:altLabel xml:lang="de">Kiwi</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14117-2">
    <skos:altLabel xml:lang="de">Guave</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14117-2">
    <skos:altLabel xml:lang="de">Guaranà</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14117-2">
    <skos:altLabel xml:lang="de">Feige</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14117-2">
    <skos:altLabel xml:lang="de">Dattel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14117-2">
    <skos:altLabel xml:lang="de">Colanuss</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14117-2">
    <skos:altLabel xml:lang="de">Colanuß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14117-2">
    <skos:altLabel xml:lang="de">Avocado</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14117-2">
    <skos:altLabel xml:lang="de">Ananas</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18854-4">
    <skos:prefLabel xml:lang="en">Union reform</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18854-4">
    <skos:prefLabel xml:lang="de">Gewerkschaftsreform</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20842-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16421-3">
    <skos:prefLabel xml:lang="en">Regionalism</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16421-3">
    <skos:prefLabel xml:lang="de">Regionalismus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19100-1">
    <skos:prefLabel xml:lang="en">Activity-based costing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19100-1">
    <skos:prefLabel xml:lang="de">Prozesskostenrechnung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16328-0">
    <skos:altLabel xml:lang="de">Schutzpolizei</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16328-0">
    <skos:altLabel xml:lang="de">Kriminalpolizei</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16328-0">
    <skos:altLabel xml:lang="de">Bereitschaftspolizei</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10546-6">
    <skos:prefLabel xml:lang="en">Diversification</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10546-6">
    <skos:prefLabel xml:lang="de">Diversifikation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11087-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12207-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11599-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13078-5">
    <skos:prefLabel xml:lang="en">Precision mechanical parts industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13078-5">
    <skos:prefLabel xml:lang="de">Feinmechanische Industrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17654-3">
    <skos:prefLabel xml:lang="en">Ethiopia</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17654-3">
    <skos:prefLabel xml:lang="de">Äthiopien</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10684-1">
    <skos:related rdf:resource="descriptor/19319-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10684-1">
    <skos:related rdf:resource="descriptor/10866-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10684-1">
    <skos:related rdf:resource="descriptor/10565-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10684-1">
    <skos:related rdf:resource="descriptor/19317-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15662-0">
    <skos:prefLabel xml:lang="en">Nature</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15662-0">
    <skos:prefLabel xml:lang="de">Natur</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15073-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id209">
    <rdf:value xml:lang="de">Für Arzneimittel</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12418-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19508-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12781-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70017">
    <skos:narrower rdf:resource="descriptor/15836-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70017">
    <skos:narrower rdf:resource="descriptor/15793-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70017">
    <skos:narrower rdf:resource="descriptor/18859-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70017">
    <skos:narrower rdf:resource="descriptor/15672-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70017">
    <skos:narrower rdf:resource="descriptor/15790-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70017">
    <skos:narrower rdf:resource="descriptor/13348-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70017">
    <skos:narrower rdf:resource="descriptor/19013-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70017">
    <skos:narrower rdf:resource="descriptor/15787-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70017">
    <skos:narrower rdf:resource="descriptor/15801-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70017">
    <skos:narrower rdf:resource="descriptor/15786-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70017">
    <skos:narrower rdf:resource="descriptor/18070-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70017">
    <skos:narrower rdf:resource="descriptor/19601-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70017">
    <skos:narrower rdf:resource="descriptor/19600-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70017">
    <skos:narrower rdf:resource="descriptor/20491-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70017">
    <skos:narrower rdf:resource="descriptor/19599-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70017">
    <skos:narrower rdf:resource="descriptor/19771-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70017">
    <skos:narrower rdf:resource="thsys/73384"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70017">
    <skos:narrower rdf:resource="thsys/73385"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70017">
    <skos:narrower rdf:resource="thsys/73386"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11686-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13841-3">
    <skos:broader rdf:resource="thsys/70888"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13841-3">
    <skos:broader rdf:resource="descriptor/10011-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14442-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16043-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14434-4">
    <skos:prefLabel xml:lang="en">Vehicle trailer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14434-4">
    <skos:prefLabel xml:lang="de">Kraftfahrzeuganhänger</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19214-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70074">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18068-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10970-0">
    <skos:related rdf:resource="descriptor/11033-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10970-0">
    <skos:related rdf:resource="descriptor/11101-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10970-0">
    <skos:related rdf:resource="descriptor/19686-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10970-0">
    <skos:related rdf:resource="descriptor/10894-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10970-0">
    <skos:related rdf:resource="descriptor/15704-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10970-0">
    <skos:related rdf:resource="descriptor/10378-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18359-0">
    <skos:broader rdf:resource="thsys/71015"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18359-0">
    <skos:broader rdf:resource="descriptor/18353-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18359-0">
    <skos:broader rdf:resource="descriptor/18426-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18246-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18078-1">
    <skos:scopeNote xml:lang="de">Technischer Kunststoff, Spezialkunststoff, Hochleistungskunststoff und Thermoplaste sind zusammenfassende, nicht genau voneinander abgegrenzte Bezeichnungen für Kunststoffe mit besonderen Eigenschaften.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17932-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10595-0">
    <skos:related rdf:resource="descriptor/10597-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10595-0">
    <skos:related rdf:resource="descriptor/16650-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10595-0">
    <skos:related rdf:resource="descriptor/19773-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10595-0">
    <skos:related rdf:resource="descriptor/10523-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10595-0">
    <skos:related rdf:resource="descriptor/10557-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10595-0">
    <skos:related rdf:resource="descriptor/10515-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10595-0">
    <skos:related rdf:resource="descriptor/10534-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20442-5">
    <skos:altLabel xml:lang="en">ECR (Efficient Consumer Response)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18538-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10285-1">
    <skos:altLabel xml:lang="en">Savings ratio</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10285-1">
    <skos:altLabel xml:lang="en">Propensity to consume</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10285-1">
    <skos:altLabel xml:lang="de">Marginale Konsumquote</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10285-1">
    <skos:altLabel xml:lang="de">Konsumquote</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19610-6">
    <skos:broader rdf:resource="thsys/70943"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19610-6">
    <skos:broader rdf:resource="descriptor/12013-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11163-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15458-6">
    <skos:narrower rdf:resource="descriptor/15463-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15458-6">
    <skos:narrower rdf:resource="descriptor/19592-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15458-6">
    <skos:narrower rdf:resource="descriptor/19593-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15458-6">
    <skos:narrower rdf:resource="descriptor/15450-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15458-6">
    <skos:narrower rdf:resource="descriptor/15448-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15458-6">
    <skos:narrower rdf:resource="descriptor/15446-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15458-6">
    <skos:narrower rdf:resource="descriptor/15441-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15458-6">
    <skos:narrower rdf:resource="descriptor/19594-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15458-6">
    <skos:narrower rdf:resource="descriptor/19596-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15458-6">
    <skos:narrower rdf:resource="descriptor/15437-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11731-6">
    <skos:prefLabel xml:lang="en">Fiscal consolidation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11731-6">
    <skos:prefLabel xml:lang="de">Haushaltskonsolidierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10013-6">
    <zbwext:useInsteadNote rdf:nodeID="id224"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12023-0">
    <skos:prefLabel xml:lang="en">Land speculation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12023-0">
    <skos:prefLabel xml:lang="de">Bodenspekulation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19779-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10465-6">
    <skos:related rdf:resource="descriptor/13511-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10465-6">
    <skos:related rdf:resource="descriptor/10470-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10465-6">
    <skos:related rdf:resource="descriptor/10466-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10465-6">
    <skos:related rdf:resource="descriptor/15703-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15638-4">
    <skos:prefLabel xml:lang="en">Chemistry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15638-4">
    <skos:prefLabel xml:lang="de">Chemie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13334-6">
    <skos:broader rdf:resource="thsys/70817"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13334-6">
    <skos:broader rdf:resource="descriptor/13333-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13334-6">
    <skos:broader rdf:resource="descriptor/13329-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13347-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16315-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12475-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70056">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.14</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18109-2">
    <skos:related rdf:resource="descriptor/16048-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11303-6">
    <skos:prefLabel xml:lang="en">Occupational profile</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11303-6">
    <skos:prefLabel xml:lang="de">Anforderungsprofil</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15982-3">
    <skos:broader rdf:resource="thsys/73392"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14404-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11848-6">
    <skos:related rdf:resource="descriptor/13305-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11848-6">
    <skos:related rdf:resource="descriptor/11847-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11848-6">
    <skos:related rdf:resource="descriptor/13829-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11848-6">
    <skos:related rdf:resource="descriptor/11840-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11848-6">
    <skos:related rdf:resource="descriptor/16176-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20810-2">
    <skos:scopeNote xml:lang="de">1996 wurde in Dublin ein Stabilitätspakt für jene EU-Staaten geschlossen, die an der Europäischen Währungsunion mitwirken. Bei diesem Stabilitätspakt handelt es sich um eine Ergänzung zum Maastrichter Vertrag, der einen Grundstein der Europäischen Gemeinschaft darstellt.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11219-2">
    <skos:broader rdf:resource="thsys/71062"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11219-2">
    <skos:broader rdf:resource="descriptor/10062-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10021-0">
    <skos:related rdf:resource="descriptor/12280-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10021-0">
    <skos:related rdf:resource="descriptor/24707-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10021-0">
    <skos:related rdf:resource="descriptor/18674-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70113">
    <skos:broader rdf:resource="thsys/b"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11039-4">
    <skos:broader rdf:resource="thsys/70049"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12927-5">
    <skos:broader rdf:resource="thsys/70924"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12927-5">
    <skos:broader rdf:resource="descriptor/12907-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18754-1">
    <skos:narrower rdf:resource="descriptor/11574-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18754-1">
    <skos:narrower rdf:resource="descriptor/11843-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18762-2">
    <skos:related rdf:resource="descriptor/19475-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18762-2">
    <skos:related rdf:resource="descriptor/13987-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18762-2">
    <skos:related rdf:resource="descriptor/15606-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18762-2">
    <skos:related rdf:resource="descriptor/18433-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18518-6">
    <skos:scopeNote xml:lang="de">Nur für Kaffeeverarbeitung aller Art.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19355-3">
    <skos:prefLabel xml:lang="en">Foreign trade pattern</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19355-3">
    <skos:prefLabel xml:lang="de">Außenhandelsstruktur</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18100-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17623-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17771-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16749-4">
    <skos:broader rdf:resource="thsys/70142"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16749-4">
    <skos:broader rdf:resource="descriptor/15916-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12520-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19321-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11823-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19147-3">
    <skos:broader rdf:resource="thsys/73103"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19147-3">
    <skos:broader rdf:resource="thsys/71035"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19147-3">
    <skos:broader rdf:resource="descriptor/15470-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18135-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19296-0">
    <skos:broader rdf:resource="thsys/73294"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19296-0">
    <skos:broader rdf:resource="thsys/71029"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19296-0">
    <skos:broader rdf:resource="thsys/70960"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19296-0">
    <skos:broader rdf:resource="descriptor/19338-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19296-0">
    <skos:broader rdf:resource="descriptor/13431-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70878">
    <skos:broader rdf:resource="thsys/70879"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14174-4">
    <skos:related rdf:resource="descriptor/18163-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14174-4">
    <skos:related rdf:resource="descriptor/18361-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14174-4">
    <skos:related rdf:resource="descriptor/20888-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14174-4">
    <skos:related rdf:resource="descriptor/12974-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14174-4">
    <skos:related rdf:resource="descriptor/19838-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14174-4">
    <skos:related rdf:resource="descriptor/18359-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18884-2">
    <skos:broader rdf:resource="thsys/70847"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18884-2">
    <skos:broader rdf:resource="thsys/70176"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18884-2">
    <skos:broader rdf:resource="descriptor/25742-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15467-5">
    <skos:related rdf:resource="descriptor/10222-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15467-5">
    <skos:related rdf:resource="descriptor/18118-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15467-5">
    <skos:related rdf:resource="descriptor/12212-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15467-5">
    <skos:related rdf:resource="descriptor/12170-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15467-5">
    <skos:related rdf:resource="descriptor/15036-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15467-5">
    <skos:related rdf:resource="descriptor/10151-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15467-5">
    <skos:related rdf:resource="descriptor/15468-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10683-3">
    <skos:prefLabel xml:lang="en">Trade barrier</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10683-3">
    <skos:prefLabel xml:lang="de">Außenhandelsbeschränkung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10045-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19168-2">
    <skos:altLabel xml:lang="de">Republik Marshallinseln</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id152">
    <rdf:value xml:lang="de">Für die technische Zusammenarbeit zwischen Industrie- und Entwicklungsländern</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16134-6">
    <skos:narrower rdf:resource="descriptor/11384-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15104-6">
    <skos:related rdf:resource="descriptor/10213-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19021-4">
    <skos:related rdf:resource="descriptor/12952-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19021-4">
    <skos:related rdf:resource="descriptor/14143-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18860-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15468-3">
    <skos:broader rdf:resource="thsys/73103"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15468-3">
    <skos:broader rdf:resource="descriptor/15470-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15468-3">
    <skos:broader rdf:resource="descriptor/15466-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73297">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.06.01.04.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18915-3">
    <skos:altLabel xml:lang="en">Do it yourself</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14217-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70825">
    <rdfs:label xml:lang="en">Media industry</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70825">
    <rdfs:label xml:lang="de">Medienwirtschaft</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10145-3">
    <skos:altLabel xml:lang="de">Mikroökonomische Fundierung der Makroökonomik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10145-3">
    <skos:altLabel xml:lang="de">Mikro-Makro-Modell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10145-3">
    <skos:altLabel xml:lang="de">Mikrofundierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18555-0">
    <skos:prefLabel xml:lang="en">Retail pricing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18555-0">
    <skos:prefLabel xml:lang="de">Einzelhandelspreispolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70445">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73303">
    <skos:prefLabel xml:lang="de">P.20.02  Milch und Milchprodukte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70822">
    <skos:broader rdf:resource="thsys/70823"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18098-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12232-5">
    <skos:narrower rdf:resource="descriptor/12261-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12232-5">
    <skos:narrower rdf:resource="descriptor/13733-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12232-5">
    <skos:narrower rdf:resource="descriptor/19278-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17705-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15772-0">
    <skos:related rdf:resource="descriptor/19480-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15772-0">
    <skos:related rdf:resource="descriptor/15738-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15772-0">
    <skos:related rdf:resource="descriptor/20722-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15947-5">
    <skos:prefLabel xml:lang="en">Morbidity</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15947-5">
    <skos:prefLabel xml:lang="de">Morbidität</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10560-5">
    <skos:broader rdf:resource="thsys/73368"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10560-5">
    <skos:broader rdf:resource="thsys/71085"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10560-5">
    <skos:broader rdf:resource="descriptor/16273-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18929-6">
    <skos:prefLabel xml:lang="en">Television production industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18929-6">
    <skos:prefLabel xml:lang="de">Programmindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16326-4">
    <skos:related rdf:resource="descriptor/14450-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10019-1">
    <skos:broader rdf:resource="thsys/71111"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10019-1">
    <skos:broader rdf:resource="thsys/71045"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73359">
    <skos:broader rdf:resource="thsys/70120"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11668-1">
    <skos:related rdf:resource="descriptor/13421-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11668-1">
    <skos:related rdf:resource="descriptor/11653-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13221-5">
    <skos:prefLabel xml:lang="en">Petroleum trade</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13221-5">
    <skos:prefLabel xml:lang="de">Mineralölhandel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73367">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.04.03.07</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16496-0">
    <zbwext:useInsteadNote rdf:nodeID="id96"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12372-3">
    <skos:prefLabel xml:lang="en">Calculation of profits</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12372-3">
    <skos:prefLabel xml:lang="de">Gewinnermittlung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18112-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13120-4">
    <skos:related rdf:resource="descriptor/14794-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18178-4">
    <skos:broader rdf:resource="thsys/71045"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18178-4">
    <skos:broader rdf:resource="thsys/70899"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15677-1">
    <skos:broader rdf:resource="thsys/73345"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15677-1">
    <skos:broader rdf:resource="descriptor/10470-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73382">
    <rdfs:label xml:lang="en">Political system</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73382">
    <rdfs:label xml:lang="de">Politisches System</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18538-0">
    <skos:broader rdf:resource="thsys/70952"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18538-0">
    <skos:broader rdf:resource="descriptor/12913-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18538-0">
    <skos:broader rdf:resource="descriptor/10852-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20503-4">
    <skos:broader rdf:resource="thsys/70368"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20503-4">
    <skos:broader rdf:resource="descriptor/12640-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17016-0">
    <skos:altLabel xml:lang="de">Südeuropäische Staaten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17016-0">
    <skos:altLabel xml:lang="de">Südeuropäische Länder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19433-2">
    <skos:prefLabel xml:lang="en">Incentive regulation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19433-2">
    <skos:prefLabel xml:lang="de">Anreizregulierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16954-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70373">
    <skos:broader rdf:resource="thsys/70135"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12605-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10246-4">
    <skos:altLabel xml:lang="de">IS-LM-Modell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10246-4">
    <skos:altLabel xml:lang="de">Hicks-Diagramm</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13519-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10901-5">
    <zbwext:useInsteadNote rdf:nodeID="id171"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19617-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19529-3">
    <zbwext:useInsteadNote rdf:nodeID="id270"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72183">
    <skos:prefLabel xml:lang="de">V.16.03  Statistische Datengewinnung und deskriptive Statistik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14702-5">
    <skos:related rdf:resource="descriptor/14822-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14702-5">
    <skos:related rdf:resource="descriptor/14379-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14702-5">
    <skos:related rdf:resource="descriptor/14610-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14702-5">
    <skos:related rdf:resource="descriptor/13000-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14702-5">
    <skos:related rdf:resource="descriptor/14854-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14936-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#broadMatch">
    <rdfs:isDefinedBy rdf:resource="http://www.w3.org/2004/02/skos/core"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15082-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19284-0">
    <skos:scopeNote xml:lang="de">Ausgabe von Gutscheinen, die später in Anteile frei zu wählender Unternehmen einzutauschen sind.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14683-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18282-2">
    <skos:broader rdf:resource="thsys/70536"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18282-2">
    <skos:broader rdf:resource="descriptor/14734-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18282-2">
    <skos:broader rdf:resource="descriptor/14019-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#editorialNote">
    <rdfs:label xml:lang="en">editorial note</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id72">
    <rdfs:seeAlso rdf:resource="descriptor/12292-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16668-4">
    <skos:related rdf:resource="descriptor/16676-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16668-4">
    <skos:related rdf:resource="descriptor/19814-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16668-4">
    <skos:related rdf:resource="descriptor/16683-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16668-4">
    <skos:related rdf:resource="descriptor/18335-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16668-4">
    <skos:related rdf:resource="descriptor/20409-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16668-4">
    <skos:related rdf:resource="descriptor/16476-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16668-4">
    <skos:related rdf:resource="descriptor/14723-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12742-3">
    <skos:prefLabel xml:lang="en">Cost theory</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12742-3">
    <skos:prefLabel xml:lang="de">Kostentheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12184-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19173-2">
    <skos:related rdf:resource="descriptor/17791-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16664-5">
    <skos:prefLabel xml:lang="en">Social status</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16664-5">
    <skos:prefLabel xml:lang="de">Sozialer Status</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16726-2">
    <skos:narrower rdf:resource="descriptor/16730-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70987">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.06.01.11</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73381">
    <skos:narrower rdf:resource="descriptor/16016-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73381">
    <skos:narrower rdf:resource="descriptor/16233-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73381">
    <skos:narrower rdf:resource="descriptor/19690-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73381">
    <skos:narrower rdf:resource="descriptor/16017-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73381">
    <skos:narrower rdf:resource="descriptor/19689-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73381">
    <skos:narrower rdf:resource="descriptor/16333-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73381">
    <skos:narrower rdf:resource="descriptor/16295-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73381">
    <skos:narrower rdf:resource="descriptor/19688-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73381">
    <skos:narrower rdf:resource="descriptor/16013-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73381">
    <skos:narrower rdf:resource="descriptor/19684-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73381">
    <skos:narrower rdf:resource="descriptor/19687-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73381">
    <skos:narrower rdf:resource="descriptor/19686-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73381">
    <skos:narrower rdf:resource="descriptor/13420-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73381">
    <skos:narrower rdf:resource="descriptor/16318-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73381">
    <skos:narrower rdf:resource="descriptor/16309-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73381">
    <skos:narrower rdf:resource="descriptor/15974-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73381">
    <skos:narrower rdf:resource="descriptor/16300-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73381">
    <skos:narrower rdf:resource="descriptor/16264-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73381">
    <skos:narrower rdf:resource="descriptor/18062-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73381">
    <skos:narrower rdf:resource="descriptor/16299-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73381">
    <skos:narrower rdf:resource="descriptor/16258-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73381">
    <skos:narrower rdf:resource="descriptor/19685-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73381">
    <skos:narrower rdf:resource="descriptor/15878-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73381">
    <skos:narrower rdf:resource="descriptor/16315-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73381">
    <skos:narrower rdf:resource="descriptor/15797-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17992-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73337">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12183-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15654-6">
    <skos:narrower rdf:resource="descriptor/16207-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15654-6">
    <skos:narrower rdf:resource="descriptor/18295-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15654-6">
    <skos:narrower rdf:resource="descriptor/15655-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15483-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70096">
    <skos:narrower rdf:resource="descriptor/20077-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70096">
    <skos:narrower rdf:resource="thsys/73394"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70096">
    <skos:narrower rdf:resource="thsys/73388"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70096">
    <skos:narrower rdf:resource="thsys/73393"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10898-3">
    <skos:broader rdf:resource="thsys/70101"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10898-3">
    <skos:broader rdf:resource="descriptor/10904-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18752-5">
    <skos:altLabel xml:lang="de">Wohneigentumsförderung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18752-5">
    <skos:altLabel xml:lang="de">Förderung von Wohneigentum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18752-5">
    <skos:altLabel xml:lang="de">Bauherrenmodell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14743-5">
    <skos:related rdf:resource="descriptor/12998-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14743-5">
    <skos:related rdf:resource="descriptor/14033-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17740-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10987-4">
    <skos:editorialNote xml:lang="de">Möglichst spezifischer indexieren.</skos:editorialNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19094-5">
    <skos:narrower rdf:resource="descriptor/12705-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19685-3">
    <skos:related rdf:resource="descriptor/11966-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19685-3">
    <skos:related rdf:resource="descriptor/11979-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18316-4">
    <skos:prefLabel xml:lang="en">Sugarbeet and sugarcane production</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18316-4">
    <skos:prefLabel xml:lang="de">Zuckerpflanzenanbau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18965-2">
    <skos:related rdf:resource="descriptor/18272-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18965-2">
    <skos:related rdf:resource="descriptor/18307-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17674-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11863-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18450-0">
    <skos:broader rdf:resource="thsys/70959"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18450-0">
    <skos:broader rdf:resource="descriptor/18522-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70550">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73303">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16471-2">
    <skos:scopeNote xml:lang="de">Militärischer Eingriff in die Angelegenheiten eines anderen Staates.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11753-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11859-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70292">
    <skos:narrower rdf:resource="descriptor/12837-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70292">
    <skos:narrower rdf:resource="descriptor/15858-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70292">
    <skos:narrower rdf:resource="descriptor/15793-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70292">
    <skos:narrower rdf:resource="descriptor/16485-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70292">
    <skos:narrower rdf:resource="descriptor/15125-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70292">
    <skos:narrower rdf:resource="descriptor/15801-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70292">
    <skos:narrower rdf:resource="descriptor/13383-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70292">
    <skos:narrower rdf:resource="descriptor/15821-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70292">
    <skos:narrower rdf:resource="descriptor/16265-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70292">
    <skos:narrower rdf:resource="descriptor/15786-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70292">
    <skos:narrower rdf:resource="descriptor/24785-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70292">
    <skos:narrower rdf:resource="descriptor/24781-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70292">
    <skos:narrower rdf:resource="descriptor/13485-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70292">
    <skos:narrower rdf:resource="descriptor/19141-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70292">
    <skos:narrower rdf:resource="descriptor/10300-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70292">
    <skos:narrower rdf:resource="descriptor/19133-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70292">
    <skos:narrower rdf:resource="descriptor/19542-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19709-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13234-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12366-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18103-0">
    <skos:altLabel xml:lang="en">Data good</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18103-0">
    <skos:altLabel xml:lang="de">Industrieelektronik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16134-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19834-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10302-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19483-1">
    <skos:narrower rdf:resource="descriptor/19449-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17985-1">
    <skos:related rdf:resource="descriptor/18373-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17985-1">
    <skos:related rdf:resource="descriptor/14197-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71078">
    <skos:narrower rdf:resource="descriptor/10022-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71078">
    <skos:narrower rdf:resource="descriptor/11618-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71078">
    <skos:narrower rdf:resource="descriptor/11616-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71078">
    <skos:narrower rdf:resource="descriptor/11613-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71078">
    <skos:narrower rdf:resource="descriptor/19399-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71078">
    <skos:narrower rdf:resource="descriptor/11612-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71078">
    <skos:narrower rdf:resource="descriptor/19398-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71078">
    <skos:narrower rdf:resource="descriptor/11611-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71078">
    <skos:narrower rdf:resource="descriptor/11610-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71078">
    <skos:narrower rdf:resource="descriptor/11609-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71078">
    <skos:narrower rdf:resource="descriptor/11607-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25501-4">
    <skos:scopeNote xml:lang="de">Literatur mit Erscheinungsdatum vor 2007 vgl. unter "Industriegeschichte".</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25501-4">
    <skos:scopeNote xml:lang="de">Vorindustrieller Produktionsbetrieb des Merkantilismus.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10224-0">
    <skos:altLabel xml:lang="de">Unvollständiger Wettbewerb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10224-0">
    <skos:altLabel xml:lang="de">Unvollständige Konkurrenz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14164-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18305-2">
    <skos:related rdf:resource="descriptor/18306-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18305-2">
    <skos:related rdf:resource="descriptor/14976-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11637-5">
    <skos:scopeNote xml:lang="de">Gesamtheit der rechts-, wirtschafts- und sozialwissenschaftlichen Disziplinen, die sich mit der Besteuerung und ihren Wirkungen beschäftigen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18260-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17014-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12424-3">
    <zbwext:useInsteadNote rdf:nodeID="id133"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15958-0">
    <skos:related rdf:resource="descriptor/15956-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15958-0">
    <skos:related rdf:resource="descriptor/11888-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18430-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24624-0">
    <skos:scopeNote xml:lang="de">Business Angels sind i.d.R. ältere, erfahrene Unternehmer/Manager, die jungen Unternehmen Know-How und zumeist auch Kapital zur Verfügung stellen. Neben der Chance, finanziell am Erfolg eines vielversprechenden Unternehmens beteiligt zu sein, besteht der Reiz oft in der Möglichkeit, die eigenen Erfahrungen weiterzugeben und damit aussichtsreiche Gründer zum Erfolg zu führen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19305-4">
    <skos:broader rdf:resource="thsys/71029"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19305-4">
    <skos:broader rdf:resource="descriptor/19293-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19242-2">
    <skos:related rdf:resource="descriptor/10434-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19242-2">
    <skos:related rdf:resource="descriptor/11049-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19242-2">
    <skos:related rdf:resource="descriptor/11911-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19242-2">
    <skos:related rdf:resource="descriptor/19354-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19242-2">
    <skos:related rdf:resource="descriptor/10525-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19242-2">
    <skos:related rdf:resource="descriptor/10537-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19242-2">
    <skos:related rdf:resource="descriptor/10494-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12311-2">
    <skos:altLabel xml:lang="en">Revenues</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12311-2">
    <skos:altLabel xml:lang="de">Erlöse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12311-2">
    <skos:altLabel xml:lang="de">Erlös</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12475-0">
    <skos:broader rdf:resource="thsys/70855"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12475-0">
    <skos:broader rdf:resource="thsys/70578"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17793-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14192-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10162-3">
    <skos:broader rdf:resource="thsys/71118"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10162-3">
    <skos:broader rdf:resource="thsys/70493"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10162-3">
    <skos:broader rdf:resource="descriptor/10164-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19010-2">
    <skos:related rdf:resource="descriptor/15685-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19010-2">
    <skos:related rdf:resource="descriptor/11379-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19010-2">
    <skos:related rdf:resource="descriptor/19746-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19010-2">
    <skos:related rdf:resource="descriptor/12621-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19660-5">
    <skos:prefLabel xml:lang="en">Automation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19660-5">
    <skos:prefLabel xml:lang="de">Automation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11691-6">
    <skos:altLabel xml:lang="en">Tax discrimination</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11691-6">
    <skos:altLabel xml:lang="de">Steuerliche Ungleichbehandlung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11691-6">
    <skos:altLabel xml:lang="de">Steuerliche Benachteiligung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11691-6">
    <skos:altLabel xml:lang="de">Steuerdiskriminierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12363-4">
    <skos:altLabel xml:lang="en">Accounting date</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12363-4">
    <skos:altLabel xml:lang="de">Stichtagsprinzip der Bilanz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70135">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18403-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13821-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11963-6">
    <skos:broader rdf:resource="thsys/70058"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11963-6">
    <skos:broader rdf:resource="descriptor/11951-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18881-1">
    <skos:broader rdf:resource="thsys/70822"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18881-1">
    <skos:broader rdf:resource="descriptor/13361-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12372-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15887-4">
    <skos:broader rdf:resource="thsys/73353"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11886-5">
    <skos:prefLabel xml:lang="en">Regional development agency</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11886-5">
    <skos:prefLabel xml:lang="de">Regionale Entwicklungsgesellschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10350-2">
    <skos:broader rdf:resource="thsys/70406"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10350-2">
    <skos:broader rdf:resource="thsys/70020"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10350-2">
    <skos:broader rdf:resource="descriptor/10347-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71012">
    <skos:broader rdf:resource="thsys/71016"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15807-0">
    <skos:prefLabel xml:lang="en">Social role</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15807-0">
    <skos:prefLabel xml:lang="de">Soziale Rolle</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15889-0">
    <skos:narrower rdf:resource="descriptor/19624-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15889-0">
    <skos:narrower rdf:resource="descriptor/15901-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15889-0">
    <skos:narrower rdf:resource="descriptor/19615-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15889-0">
    <skos:narrower rdf:resource="descriptor/19620-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10271-5">
    <skos:broader rdf:resource="thsys/71111"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10271-5">
    <skos:broader rdf:resource="descriptor/10272-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19348-0">
    <skos:broader rdf:resource="thsys/71026"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19348-0">
    <skos:broader rdf:resource="thsys/71020"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19348-0">
    <skos:broader rdf:resource="descriptor/11801-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12849-6">
    <skos:altLabel xml:lang="de">Steuerrechtliches Anrechnungsverfahren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12849-6">
    <skos:altLabel xml:lang="de">Steuerliche Doppelbelastung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12849-6">
    <skos:altLabel xml:lang="de">Doppelbesteuerung (Anrechnungsverfahren)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12849-6">
    <skos:altLabel xml:lang="de">Doppelbelastung des Steuerrechts</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12849-6">
    <skos:altLabel xml:lang="de">Anrechnungsverfahren des Steuerrechts</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10112-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73381">
    <rdfs:label xml:lang="de">Staat und Nation</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10186-3">
    <skos:related rdf:resource="descriptor/11754-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10186-3">
    <skos:related rdf:resource="descriptor/10326-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10186-3">
    <skos:related rdf:resource="descriptor/10315-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10186-3">
    <skos:related rdf:resource="descriptor/10638-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10186-3">
    <skos:related rdf:resource="descriptor/10185-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19277-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17744-2">
    <skos:broader rdf:resource="thsys/71031"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17744-2">
    <skos:broader rdf:resource="descriptor/17743-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19658-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70860">
    <skos:prefLabel xml:lang="de">W.22.02  Öffentliche Wirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16651-0">
    <skos:prefLabel xml:lang="en">Solidarity</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16651-0">
    <skos:prefLabel xml:lang="de">Solidarität</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16422-1">
    <skos:prefLabel xml:lang="en">Separatism</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16422-1">
    <skos:prefLabel xml:lang="de">Separatismus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17923-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14462-6">
    <skos:prefLabel xml:lang="en">Tanker</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14462-6">
    <skos:prefLabel xml:lang="de">Tankschiff</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10040-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10421-5">
    <skos:altLabel xml:lang="de">Tobin-Johnson-Wachstumsmodell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10421-5">
    <skos:altLabel xml:lang="de">Monetäres Wachstumsmodell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10421-5">
    <skos:altLabel xml:lang="de">Keynes-Wicksell-Wachstumstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10421-5">
    <skos:altLabel xml:lang="de">Keynes-Wicksell-Modell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id2">
    <rdfs:seeAlso rdf:resource="descriptor/12761-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70871">
    <skos:narrower rdf:resource="descriptor/16212-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70871">
    <skos:narrower rdf:resource="descriptor/16336-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70871">
    <skos:narrower rdf:resource="descriptor/16019-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70871">
    <skos:narrower rdf:resource="descriptor/12511-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70871">
    <skos:narrower rdf:resource="descriptor/16335-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70871">
    <skos:narrower rdf:resource="descriptor/15762-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70871">
    <skos:narrower rdf:resource="descriptor/18161-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70871">
    <skos:narrower rdf:resource="descriptor/11020-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70871">
    <skos:narrower rdf:resource="descriptor/13413-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70871">
    <skos:narrower rdf:resource="descriptor/24783-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70871">
    <skos:narrower rdf:resource="thsys/70868"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70871">
    <skos:narrower rdf:resource="thsys/70862"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70871">
    <skos:narrower rdf:resource="thsys/70865"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70871">
    <skos:narrower rdf:resource="thsys/70870"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70871">
    <skos:narrower rdf:resource="thsys/70866"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70871">
    <skos:narrower rdf:resource="thsys/70869"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70871">
    <skos:narrower rdf:resource="thsys/70867"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13818-5">
    <skos:related rdf:resource="descriptor/13826-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13818-5">
    <skos:related rdf:resource="descriptor/13825-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13818-5">
    <skos:related rdf:resource="descriptor/13828-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13818-5">
    <skos:related rdf:resource="descriptor/16485-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18906-4">
    <skos:altLabel xml:lang="en">Welfare institution</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18906-4">
    <skos:altLabel xml:lang="de">Sozialstation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18906-4">
    <skos:altLabel xml:lang="de">Sozialpolitische Einrichtung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18906-4">
    <skos:altLabel xml:lang="de">Soziale Infrastruktur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73341">
    <skos:narrower rdf:resource="descriptor/19144-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73341">
    <skos:narrower rdf:resource="descriptor/15616-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73341">
    <skos:narrower rdf:resource="descriptor/18661-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73341">
    <skos:narrower rdf:resource="descriptor/18270-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73341">
    <skos:narrower rdf:resource="descriptor/18657-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73341">
    <skos:narrower rdf:resource="descriptor/13683-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73341">
    <skos:narrower rdf:resource="descriptor/18665-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73341">
    <skos:narrower rdf:resource="descriptor/13269-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73341">
    <skos:narrower rdf:resource="descriptor/20921-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73341">
    <skos:narrower rdf:resource="descriptor/18659-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73341">
    <skos:narrower rdf:resource="descriptor/18175-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73341">
    <skos:narrower rdf:resource="descriptor/16757-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73341">
    <skos:narrower rdf:resource="descriptor/15598-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73341">
    <skos:narrower rdf:resource="descriptor/18017-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73341">
    <skos:narrower rdf:resource="descriptor/19651-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73341">
    <skos:narrower rdf:resource="descriptor/19802-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73341">
    <skos:narrower rdf:resource="descriptor/19653-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73341">
    <skos:narrower rdf:resource="descriptor/20341-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73341">
    <skos:narrower rdf:resource="descriptor/15599-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73341">
    <skos:narrower rdf:resource="descriptor/14751-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73341">
    <skos:narrower rdf:resource="descriptor/18658-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73341">
    <skos:narrower rdf:resource="descriptor/19801-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10111-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18830-4">
    <skos:prefLabel xml:lang="en">Public welfare</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18830-4">
    <skos:prefLabel xml:lang="de">Gemeinwohl</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18527-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10123-6">
    <skos:broader rdf:resource="thsys/70486"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10123-6">
    <skos:broader rdf:resource="descriptor/10126-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15900-3">
    <skos:altLabel xml:lang="de">Raumforschung (Weltraum)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15900-3">
    <skos:altLabel xml:lang="de">Kosmologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15900-3">
    <skos:altLabel xml:lang="de">Astronomie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16605-0">
    <skos:broader rdf:resource="thsys/73396"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16605-0">
    <skos:broader rdf:resource="descriptor/16602-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16605-0">
    <skos:broader rdf:resource="descriptor/16588-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11638-3">
    <skos:broader rdf:resource="thsys/71077"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70134">
    <rdfs:label xml:lang="de">NE-Metalle</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12681-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11347-0">
    <skos:related rdf:resource="descriptor/15977-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11347-0">
    <skos:related rdf:resource="descriptor/19715-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11347-0">
    <skos:related rdf:resource="descriptor/16301-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11347-0">
    <skos:related rdf:resource="descriptor/11339-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11347-0">
    <skos:related rdf:resource="descriptor/11307-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15074-3">
    <skos:broader rdf:resource="thsys/72181"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15074-3">
    <skos:broader rdf:resource="thsys/70815"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15074-3">
    <skos:broader rdf:resource="thsys/70209"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15074-3">
    <skos:broader rdf:resource="descriptor/18887-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14043-5">
    <skos:prefLabel xml:lang="en">Furniture</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14043-5">
    <skos:prefLabel xml:lang="de">Möbel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11498-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12201-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14895-3">
    <skos:scopeNote xml:lang="de">Der Begriff umfasst sowohl Stoffe wie konfektionierte Textilien dieses Produktzweigs.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19114-4">
    <skos:broader rdf:resource="thsys/71036"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19114-4">
    <skos:broader rdf:resource="thsys/70394"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19114-4">
    <skos:broader rdf:resource="descriptor/12670-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19114-4">
    <skos:broader rdf:resource="descriptor/12165-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19518-1">
    <skos:prefLabel xml:lang="en">Hospital costs</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19518-1">
    <skos:prefLabel xml:lang="de">Krankenhauskosten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14422-4">
    <skos:broader rdf:resource="thsys/70337"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14422-4">
    <skos:broader rdf:resource="thsys/70256"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14422-4">
    <skos:broader rdf:resource="descriptor/14413-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14422-4">
    <skos:broader rdf:resource="descriptor/14421-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18806-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11805-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18137-4">
    <skos:prefLabel xml:lang="en">Organic farming</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18137-4">
    <skos:prefLabel xml:lang="de">Ökologischer Landbau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13851-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71063">
    <skos:prefLabel xml:lang="en">V.11.07  Public infrastructure</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71063">
    <skos:prefLabel xml:lang="de">V.11.07  Infrastruktur</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13513-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70386">
    <skos:narrower rdf:resource="descriptor/18620-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70386">
    <skos:narrower rdf:resource="descriptor/11934-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70386">
    <skos:narrower rdf:resource="descriptor/15518-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70386">
    <skos:narrower rdf:resource="descriptor/13600-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70386">
    <skos:narrower rdf:resource="descriptor/12689-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13693-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70101">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.10</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10440-1">
    <skos:related rdf:resource="descriptor/19012-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10440-1">
    <skos:related rdf:resource="descriptor/12594-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10440-1">
    <skos:related rdf:resource="descriptor/10125-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10440-1">
    <skos:related rdf:resource="descriptor/10304-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10440-1">
    <skos:related rdf:resource="descriptor/11180-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18002-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10608-3">
    <skos:broader rdf:resource="thsys/73362"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10608-3">
    <skos:broader rdf:resource="thsys/71092"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10608-3">
    <skos:broader rdf:resource="descriptor/10627-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10608-3">
    <skos:broader rdf:resource="descriptor/10012-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10608-3">
    <skos:broader rdf:resource="descriptor/16495-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70651">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70947">
    <skos:prefLabel xml:lang="en">W.10.05  Retail trade</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70947">
    <skos:prefLabel xml:lang="de">W.10.05  Einzelhandel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19262-3">
    <skos:narrower rdf:resource="descriptor/19263-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19262-3">
    <skos:narrower rdf:resource="descriptor/10397-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12628-1">
    <skos:related rdf:resource="descriptor/18166-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12628-1">
    <skos:related rdf:resource="descriptor/19504-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11187-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15718-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10103-5">
    <skos:related rdf:resource="descriptor/10650-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10103-5">
    <skos:related rdf:resource="descriptor/10102-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10103-5">
    <skos:related rdf:resource="descriptor/11688-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14336-4">
    <skos:broader rdf:resource="thsys/70549"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18411-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73378">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.04.02.04</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73297">
    <skos:narrower rdf:resource="descriptor/13073-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73297">
    <skos:narrower rdf:resource="descriptor/24708-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73297">
    <skos:narrower rdf:resource="descriptor/13987-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73297">
    <skos:narrower rdf:resource="descriptor/18433-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73297">
    <skos:narrower rdf:resource="descriptor/13072-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73297">
    <skos:narrower rdf:resource="descriptor/13071-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70415">
    <skos:narrower rdf:resource="descriptor/14463-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70415">
    <skos:narrower rdf:resource="descriptor/14462-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70415">
    <skos:narrower rdf:resource="descriptor/14461-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70415">
    <skos:narrower rdf:resource="descriptor/14456-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70415">
    <skos:narrower rdf:resource="descriptor/14455-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70415">
    <skos:narrower rdf:resource="descriptor/14317-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70415">
    <skos:narrower rdf:resource="descriptor/18248-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70415">
    <skos:narrower rdf:resource="descriptor/14450-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70415">
    <skos:narrower rdf:resource="descriptor/14449-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70415">
    <skos:narrower rdf:resource="descriptor/18247-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70415">
    <skos:narrower rdf:resource="descriptor/18242-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70415">
    <skos:narrower rdf:resource="descriptor/14446-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15903-4">
    <skos:related rdf:resource="descriptor/15894-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15903-4">
    <skos:related rdf:resource="descriptor/15893-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15903-4">
    <skos:related rdf:resource="descriptor/15930-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15903-4">
    <skos:related rdf:resource="descriptor/19620-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15903-4">
    <skos:related rdf:resource="descriptor/19484-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15903-4">
    <skos:related rdf:resource="descriptor/12016-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71030">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">G.04.03</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18072-6">
    <skos:altLabel xml:lang="en">Pollution taxation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18072-6">
    <skos:altLabel xml:lang="en">Carbon taxation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18072-6">
    <skos:altLabel xml:lang="en">Carbon tax</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18072-6">
    <skos:altLabel xml:lang="de">Umweltsteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18072-6">
    <skos:altLabel xml:lang="de">Öko-Steuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18072-6">
    <skos:altLabel xml:lang="de">Emissionssteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18072-6">
    <skos:altLabel xml:lang="de">CO2-Steuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70188">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21011-6">
    <skos:broader rdf:resource="thsys/71075"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21011-6">
    <skos:broader rdf:resource="descriptor/11704-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11362-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10780-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14936-1">
    <skos:prefLabel xml:lang="en">Foodstuffs</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14936-1">
    <skos:prefLabel xml:lang="de">Nährmittel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11418-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19174-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10675-2">
    <skos:broader rdf:resource="thsys/71100"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10675-2">
    <skos:broader rdf:resource="thsys/71091"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70179">
    <rdfs:label xml:lang="de">Außenwirtschaft und internationale Wirtschaftsbeziehungen</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10030-6">
    <skos:altLabel xml:lang="de">Wirtschaftsgüter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10030-6">
    <skos:altLabel xml:lang="de">Wirtschaftsgut</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10030-6">
    <skos:altLabel xml:lang="de">Gut</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18588-6">
    <skos:altLabel xml:lang="de">Linienschiffahrt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10876-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14728-1">
    <skos:narrower rdf:resource="descriptor/14731-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14728-1">
    <skos:narrower rdf:resource="descriptor/14730-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14728-1">
    <skos:narrower rdf:resource="descriptor/18258-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14728-1">
    <skos:narrower rdf:resource="descriptor/14729-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14728-1">
    <skos:narrower rdf:resource="descriptor/14722-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14026-5">
    <skos:broader rdf:resource="thsys/70201"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10556-3">
    <skos:broader rdf:resource="thsys/71084"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/w">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13714-3">
    <skos:narrower rdf:resource="descriptor/19279-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18379-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13624-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71026">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19087-2">
    <skos:related rdf:resource="descriptor/17176-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10947-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18333-4">
    <skos:prefLabel xml:lang="en">Land reform</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18333-4">
    <skos:prefLabel xml:lang="de">Bodenreform</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19584-2">
    <skos:prefLabel xml:lang="en">Science statistics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19584-2">
    <skos:prefLabel xml:lang="de">Forschungsstatistik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17609-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18671-5">
    <skos:related rdf:resource="descriptor/12287-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18671-5">
    <skos:related rdf:resource="descriptor/13723-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18671-5">
    <skos:related rdf:resource="descriptor/11414-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18671-5">
    <skos:related rdf:resource="descriptor/18085-4"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id184">
    <rdf:value xml:lang="de">Für Kündigungsschutz von Wohnungen</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16247-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13595-6">
    <skos:altLabel xml:lang="de">Regionale Integrationswirkung des Verkehrs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13595-6">
    <skos:altLabel xml:lang="de">Räumliche Integrationswirkung des Verkehrs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13595-6">
    <skos:altLabel xml:lang="de">Integrationswirkung des Verkehrs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18461-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13775-4">
    <skos:related rdf:resource="descriptor/12257-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13775-4">
    <skos:related rdf:resource="descriptor/13736-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13775-4">
    <skos:related rdf:resource="descriptor/18750-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13775-4">
    <skos:related rdf:resource="descriptor/13703-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13775-4">
    <skos:related rdf:resource="descriptor/12026-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14274-0">
    <skos:narrower rdf:resource="descriptor/14310-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14274-0">
    <skos:narrower rdf:resource="descriptor/14309-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14274-0">
    <skos:narrower rdf:resource="descriptor/14307-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14274-0">
    <skos:narrower rdf:resource="descriptor/14305-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14274-0">
    <skos:narrower rdf:resource="descriptor/18238-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14274-0">
    <skos:narrower rdf:resource="descriptor/14301-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14274-0">
    <skos:narrower rdf:resource="descriptor/14299-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14274-0">
    <skos:narrower rdf:resource="descriptor/14295-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14274-0">
    <skos:narrower rdf:resource="descriptor/18237-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14274-0">
    <skos:narrower rdf:resource="descriptor/14294-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17791-0">
    <skos:related rdf:resource="descriptor/17990-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17791-0">
    <skos:related rdf:resource="descriptor/17989-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17791-0">
    <skos:related rdf:resource="descriptor/19173-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12013-3">
    <skos:broader rdf:resource="thsys/71063"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12013-3">
    <skos:broader rdf:resource="thsys/70943"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12013-3">
    <skos:broader rdf:resource="descriptor/11716-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12569-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10822-1">
    <zbwext:useInsteadNote rdf:nodeID="id39"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10822-1">
    <zbwext:useInsteadNote rdf:nodeID="id38"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13527-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13445-4">
    <skos:scopeNote xml:lang="de">Analyse der sozialen Strukturverhältnisse ländlicher Gemeinden mit vorwiegend landwirtschaftlichen Lebensformen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13702-3">
    <skos:broader rdf:resource="thsys/71028"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13702-3">
    <skos:broader rdf:resource="thsys/70902"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13702-3">
    <skos:broader rdf:resource="descriptor/13291-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15327-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18180-3">
    <skos:related rdf:resource="descriptor/18907-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18180-3">
    <skos:related rdf:resource="descriptor/13292-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18180-3">
    <skos:related rdf:resource="descriptor/13312-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18180-3">
    <skos:related rdf:resource="descriptor/18972-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18180-3">
    <skos:related rdf:resource="descriptor/15908-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18180-3">
    <skos:related rdf:resource="descriptor/15904-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14359-6">
    <skos:broader rdf:resource="thsys/70424"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14359-6">
    <skos:broader rdf:resource="descriptor/14455-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14359-6">
    <skos:broader rdf:resource="descriptor/14355-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16406-6">
    <skos:prefLabel xml:lang="en">Individualism</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16406-6">
    <skos:prefLabel xml:lang="de">Individualismus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18117-3">
    <skos:related rdf:resource="descriptor/12073-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18117-3">
    <skos:related rdf:resource="descriptor/19250-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18117-3">
    <skos:related rdf:resource="descriptor/10911-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18117-3">
    <skos:related rdf:resource="descriptor/12083-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18117-3">
    <skos:related rdf:resource="descriptor/12295-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18117-3">
    <skos:related rdf:resource="descriptor/10463-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18117-3">
    <skos:related rdf:resource="descriptor/10320-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18117-3">
    <skos:related rdf:resource="descriptor/10459-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18117-3">
    <skos:related rdf:resource="descriptor/10546-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18117-3">
    <skos:related rdf:resource="descriptor/12617-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16039-0">
    <skos:related rdf:resource="descriptor/15982-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16039-0">
    <skos:related rdf:resource="descriptor/18433-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16039-0">
    <skos:related rdf:resource="descriptor/16176-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14516-2">
    <skos:prefLabel xml:lang="en">Audio product</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14516-2">
    <skos:prefLabel xml:lang="de">Audioprodukt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11401-6">
    <skos:prefLabel xml:lang="en">Curriculum</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11401-6">
    <skos:prefLabel xml:lang="de">Lehrplan</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10481-1">
    <skos:narrower rdf:resource="descriptor/16185-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#OrderedCollection">
    <rdf:type rdf:resource="http://www.w3.org/2002/07/owl#Class"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10803-5">
    <skos:prefLabel xml:lang="en">Hedging</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10803-5">
    <skos:prefLabel xml:lang="de">Hedging</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19737-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14334-1">
    <skos:altLabel xml:lang="de">Kupplung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19064-0">
    <skos:prefLabel xml:lang="en">Warehousing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19064-0">
    <skos:prefLabel xml:lang="de">Lagereigewerbe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13228-5">
    <skos:related rdf:resource="descriptor/13084-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13228-5">
    <skos:related rdf:resource="descriptor/14522-0"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id258">
    <rdfs:seeAlso rdf:resource="descriptor/10106-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15051-1">
    <skos:altLabel xml:lang="en">Program evaluation and review technique</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15051-1">
    <skos:altLabel xml:lang="en">Network flow</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15051-1">
    <skos:altLabel xml:lang="en">Network analysis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15051-1">
    <skos:altLabel xml:lang="en">Critical path method</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15051-1">
    <skos:altLabel xml:lang="en">CPM (Critical Path Method)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15051-1">
    <skos:altLabel xml:lang="de">Vorgangsknotennetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15051-1">
    <skos:altLabel xml:lang="de">PERT</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15051-1">
    <skos:altLabel xml:lang="de">Optimaler-Weg-Algorithmus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15051-1">
    <skos:altLabel xml:lang="de">Netzwerkflussproblem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15051-1">
    <skos:altLabel xml:lang="de">Netzwerkflußproblem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15051-1">
    <skos:altLabel xml:lang="de">Netzplan</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15051-1">
    <skos:altLabel xml:lang="de">Kürzeste-Entfernung-Algorithmus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15051-1">
    <skos:altLabel xml:lang="de">Flussproblem in Netzwerken</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15051-1">
    <skos:altLabel xml:lang="de">Flußproblem in Netzwerken</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15051-1">
    <skos:altLabel xml:lang="de">Ereignisknotennetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15051-1">
    <skos:altLabel xml:lang="de">Baumalgorithmus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19781-0">
    <skos:scopeNote xml:lang="de">Nur für historisches Erbe und historisch wertvolle Kulturgegenstände.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13440-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15742-2">
    <skos:prefLabel xml:lang="en">Air quality improvement</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15742-2">
    <skos:prefLabel xml:lang="de">Luftreinhaltung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16122-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15810-4">
    <skos:prefLabel xml:lang="en">Socialization</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15810-4">
    <skos:prefLabel xml:lang="de">Sozialisation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12408-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19219-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10745-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19311-2">
    <skos:prefLabel xml:lang="en">Exchange rate pass-through</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19311-2">
    <skos:prefLabel xml:lang="de">Exchange Rate Pass-Through</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71043">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15599-2">
    <skos:prefLabel xml:lang="en">Computer networks</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15599-2">
    <skos:prefLabel xml:lang="de">Computernetz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19776-0">
    <skos:broader rdf:resource="thsys/70580"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19776-0">
    <skos:broader rdf:resource="descriptor/15644-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11377-5">
    <skos:prefLabel xml:lang="en">School</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11377-5">
    <skos:prefLabel xml:lang="de">Schule</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18066-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19133-0">
    <skos:scopeNote xml:lang="de">Beziehungen zwischen Unternehmen und ihren Endverbraucher-Kunden.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13823-5">
    <skos:altLabel xml:lang="en">Actuarial mathematics</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13823-5">
    <skos:altLabel xml:lang="de">Versicherungsmathematik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13823-5">
    <skos:altLabel xml:lang="de">Berechnungsgrundlagen für Versicherungen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13823-5">
    <skos:altLabel xml:lang="de">Actuarial theory</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11495-6">
    <skos:prefLabel xml:lang="en">Inflation expectations</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11495-6">
    <skos:prefLabel xml:lang="de">Inflationserwartung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13661-5">
    <skos:related rdf:resource="descriptor/16269-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13661-5">
    <skos:related rdf:resource="descriptor/16287-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13661-5">
    <skos:related rdf:resource="descriptor/12775-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13661-5">
    <skos:related rdf:resource="descriptor/16272-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13661-5">
    <skos:related rdf:resource="descriptor/13662-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13661-5">
    <skos:related rdf:resource="descriptor/13383-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13661-5">
    <skos:related rdf:resource="descriptor/16265-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13661-5">
    <skos:related rdf:resource="descriptor/13658-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24623-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11694-0">
    <skos:altLabel xml:lang="de">Theorie der Besteuerung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11680-4">
    <skos:narrower rdf:resource="descriptor/19422-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11680-4">
    <skos:narrower rdf:resource="descriptor/19412-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70192">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10610-2">
    <skos:prefLabel xml:lang="en">European recovery program</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10610-2">
    <skos:prefLabel xml:lang="de">European Recovery Program</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18184-2">
    <skos:related rdf:resource="descriptor/18029-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18184-2">
    <skos:related rdf:resource="descriptor/12083-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19571-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70174">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70935">
    <rdfs:label xml:lang="en">Inland water transport</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70935">
    <rdfs:label xml:lang="de">Binnenschifffahrt</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19277-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18808-4">
    <skos:related rdf:resource="descriptor/12005-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18808-4">
    <skos:related rdf:resource="descriptor/13421-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18808-4">
    <skos:related rdf:resource="descriptor/13414-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18808-4">
    <skos:related rdf:resource="descriptor/16324-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18808-4">
    <skos:related rdf:resource="descriptor/16322-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73301">
    <rdfs:label xml:lang="de">Fisch und Fischprodukte</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19227-5">
    <skos:related rdf:resource="descriptor/13826-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19227-5">
    <skos:related rdf:resource="descriptor/18741-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19227-5">
    <skos:related rdf:resource="descriptor/10207-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19227-5">
    <skos:related rdf:resource="descriptor/13777-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19227-5">
    <skos:related rdf:resource="descriptor/10194-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18981-4">
    <skos:related rdf:resource="descriptor/19190-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18981-4">
    <skos:related rdf:resource="descriptor/19189-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18981-4">
    <skos:related rdf:resource="descriptor/14093-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13635-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18225-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10286-6">
    <skos:prefLabel xml:lang="en">Accelerator</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10286-6">
    <skos:prefLabel xml:lang="de">Akzelerator</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11210-6">
    <skos:broader rdf:resource="thsys/71110"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11210-6">
    <skos:broader rdf:resource="thsys/71060"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11210-6">
    <skos:broader rdf:resource="descriptor/11182-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16362-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13784-3">
    <skos:broader rdf:resource="thsys/70899"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13784-3">
    <skos:broader rdf:resource="descriptor/19282-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18741-3">
    <skos:related rdf:resource="descriptor/10207-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18741-3">
    <skos:related rdf:resource="descriptor/19227-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14095-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id347">
    <rdfs:seeAlso rdf:resource="descriptor/16058-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12693-4">
    <skos:broader rdf:resource="thsys/70549"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12693-4">
    <skos:broader rdf:resource="descriptor/14336-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13501-6">
    <skos:altLabel xml:lang="de">Internationaler Verkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12625-0">
    <skos:broader rdf:resource="thsys/71050"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12625-0">
    <skos:broader rdf:resource="descriptor/12607-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19363-4">
    <skos:altLabel xml:lang="en">Boom town</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11219-2">
    <skos:related rdf:resource="descriptor/12567-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11219-2">
    <skos:related rdf:resource="descriptor/11211-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11219-2">
    <skos:related rdf:resource="descriptor/11235-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11219-2">
    <skos:related rdf:resource="descriptor/19767-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11219-2">
    <skos:related rdf:resource="descriptor/11183-4"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id273">
    <rdfs:seeAlso rdf:resource="descriptor/13683-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10752-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11297-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15072-0">
    <skos:narrower rdf:resource="descriptor/15025-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15072-0">
    <skos:narrower rdf:resource="descriptor/15021-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71041">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14382-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10148-4">
    <skos:altLabel xml:lang="en">Value of time</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10148-4">
    <skos:altLabel xml:lang="de">Zeitwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10148-4">
    <skos:altLabel xml:lang="de">Zeitökonomie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10148-4">
    <skos:altLabel xml:lang="de">Zeitgewinnbewertung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10148-4">
    <skos:altLabel xml:lang="de">Zeitersparnisbewertung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10148-4">
    <skos:altLabel xml:lang="de">Zeitbewertung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19828-0">
    <skos:altLabel xml:lang="en">VaR (Value at Risk)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14240-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16306-3">
    <skos:prefLabel xml:lang="en">Change of government</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16306-3">
    <skos:prefLabel xml:lang="de">Regierungswechsel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18535-6">
    <skos:narrower rdf:resource="descriptor/13971-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18535-6">
    <skos:narrower rdf:resource="descriptor/13979-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18535-6">
    <skos:narrower rdf:resource="descriptor/18966-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18535-6">
    <skos:narrower rdf:resource="descriptor/13363-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18535-6">
    <skos:narrower rdf:resource="descriptor/14014-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18535-6">
    <skos:narrower rdf:resource="descriptor/13998-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18535-6">
    <skos:narrower rdf:resource="descriptor/13995-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18535-6">
    <skos:narrower rdf:resource="descriptor/14004-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18535-6">
    <skos:narrower rdf:resource="descriptor/13989-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18535-6">
    <skos:narrower rdf:resource="descriptor/13967-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18535-6">
    <skos:narrower rdf:resource="descriptor/11374-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18535-6">
    <skos:narrower rdf:resource="descriptor/13355-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18535-6">
    <skos:narrower rdf:resource="descriptor/18977-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18535-6">
    <skos:narrower rdf:resource="descriptor/14001-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18535-6">
    <skos:narrower rdf:resource="descriptor/13974-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18535-6">
    <skos:narrower rdf:resource="descriptor/14012-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18535-6">
    <skos:narrower rdf:resource="descriptor/13975-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18535-6">
    <skos:narrower rdf:resource="descriptor/13988-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18535-6">
    <skos:narrower rdf:resource="descriptor/11302-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18535-6">
    <skos:narrower rdf:resource="descriptor/13871-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18535-6">
    <skos:narrower rdf:resource="descriptor/13987-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18535-6">
    <skos:narrower rdf:resource="descriptor/13985-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18924-2">
    <skos:altLabel xml:lang="de">Kulturelle Identität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18220-3">
    <skos:related rdf:resource="descriptor/12950-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18220-3">
    <skos:related rdf:resource="descriptor/14142-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18370-5">
    <skos:prefLabel xml:lang="en">European Union Energy Policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18370-5">
    <skos:prefLabel xml:lang="de">EU-Energiepolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13488-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10894-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18298-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12807-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18524-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12594-6">
    <skos:scopeNote xml:lang="de">Für alle tarifbezogenen, übertariflichen und freiwilligen Vergütungsbestandteile, die an die Arbeitsleistung gekoppelt sind.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73306">
    <skos:broader rdf:resource="thsys/70348"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id261">
    <rdf:value xml:lang="de">Für private Geschäftsbanken allgemein</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70076">
    <rdfs:label xml:lang="en">Africa</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70076">
    <rdfs:label xml:lang="de">Afrika</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70429">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15772-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17927-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12207-4">
    <skos:narrower rdf:resource="descriptor/19536-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19694-2">
    <skos:related rdf:resource="descriptor/13397-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19694-2">
    <skos:related rdf:resource="descriptor/16344-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12617-6">
    <skos:scopeNote xml:lang="de">Arbeits-, Betriebs- und Organisationspsychologie sind nicht vollkommen deckungsgleich, haben ihren Schwerpunkt jedoch in der Analyse der Arbeit im Rahmen von Organisationen und werden deshalb hier zusammengefasst.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15989-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19513-4">
    <skos:prefLabel xml:lang="en">Funded pension system</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19513-4">
    <skos:prefLabel xml:lang="de">Kapitaldeckungsverfahren</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13285-0">
    <skos:related rdf:resource="descriptor/12236-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17627-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13164-5">
    <skos:related rdf:resource="descriptor/14997-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13164-5">
    <skos:related rdf:resource="descriptor/11677-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13164-5">
    <skos:related rdf:resource="descriptor/13238-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13164-5">
    <skos:related rdf:resource="descriptor/18196-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13164-5">
    <skos:related rdf:resource="descriptor/14082-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13164-5">
    <skos:related rdf:resource="descriptor/18900-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10986-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18244-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11278-0">
    <skos:related rdf:resource="descriptor/19494-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11043-6">
    <skos:broader rdf:resource="thsys/70171"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11043-6">
    <skos:broader rdf:resource="descriptor/11068-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16525-5">
    <skos:altLabel xml:lang="de">Altenzentrum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16525-5">
    <skos:altLabel xml:lang="de">Altentagesstätte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16525-5">
    <skos:altLabel xml:lang="de">Altenpflege</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15376-1">
    <skos:narrower rdf:resource="descriptor/19577-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15376-1">
    <skos:narrower rdf:resource="descriptor/19573-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15376-1">
    <skos:narrower rdf:resource="descriptor/15375-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15376-1">
    <skos:narrower rdf:resource="descriptor/10253-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15376-1">
    <skos:narrower rdf:resource="descriptor/10045-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15376-1">
    <skos:narrower rdf:resource="descriptor/19582-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15376-1">
    <skos:narrower rdf:resource="descriptor/15327-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15376-1">
    <skos:narrower rdf:resource="descriptor/19574-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13384-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15698-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10486-5">
    <skos:altLabel xml:lang="en">Literacy</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10486-5">
    <skos:altLabel xml:lang="de">Analphabetenquote</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10486-5">
    <skos:altLabel xml:lang="de">Alphabetisierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13308-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11613-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70927">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19170-1">
    <skos:prefLabel xml:lang="en">Andean Group</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19170-1">
    <skos:prefLabel xml:lang="de">Andenpakt-Staaten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19287-1">
    <skos:broader rdf:resource="thsys/70171"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19287-1">
    <skos:broader rdf:resource="descriptor/11068-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19287-1">
    <skos:broader rdf:resource="descriptor/11066-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10236-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14934-5">
    <skos:related rdf:resource="descriptor/14084-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14934-5">
    <skos:related rdf:resource="descriptor/13165-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14934-5">
    <skos:related rdf:resource="descriptor/14996-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14934-5">
    <skos:related rdf:resource="descriptor/18215-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14934-5">
    <skos:related rdf:resource="descriptor/14060-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10343-6">
    <skos:narrower rdf:resource="descriptor/19664-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10343-6">
    <skos:narrower rdf:resource="descriptor/11792-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11187-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14064-4">
    <skos:broader rdf:resource="thsys/73298"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14064-4">
    <skos:broader rdf:resource="thsys/70341"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14064-4">
    <skos:broader rdf:resource="descriptor/18222-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14269-0">
    <skos:prefLabel xml:lang="en">Metal product</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14269-0">
    <skos:prefLabel xml:lang="de">Metallprodukt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11696-3">
    <zbwext:useInsteadNote rdf:nodeID="id318"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18559-6">
    <skos:related rdf:resource="descriptor/18557-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18559-6">
    <skos:related rdf:resource="descriptor/15775-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18559-6">
    <skos:related rdf:resource="descriptor/18307-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18559-6">
    <skos:related rdf:resource="descriptor/18962-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17740-3">
    <skos:prefLabel xml:lang="en">Chad</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17740-3">
    <skos:prefLabel xml:lang="de">Tschad</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19024-5">
    <skos:altLabel xml:lang="de">Zwiebelgemüse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19024-5">
    <skos:altLabel xml:lang="de">Schnittlauch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19024-5">
    <skos:altLabel xml:lang="de">Porree</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19024-5">
    <skos:altLabel xml:lang="de">Lauch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19024-5">
    <skos:altLabel xml:lang="de">Knoblauch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12073-6">
    <skos:related rdf:resource="descriptor/11146-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12073-6">
    <skos:related rdf:resource="descriptor/10328-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12073-6">
    <skos:related rdf:resource="descriptor/18117-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12073-6">
    <skos:related rdf:resource="descriptor/12295-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12073-6">
    <skos:related rdf:resource="descriptor/10460-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12073-6">
    <skos:related rdf:resource="descriptor/18151-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12073-6">
    <skos:related rdf:resource="descriptor/12053-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11788-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18310-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73363">
    <skos:broader rdf:resource="thsys/73366"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16828-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73294">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.06.01.12.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70234">
    <skos:narrower rdf:resource="descriptor/10068-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70234">
    <skos:narrower rdf:resource="descriptor/10064-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70234">
    <skos:narrower rdf:resource="descriptor/19670-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70234">
    <skos:narrower rdf:resource="descriptor/10087-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70234">
    <skos:narrower rdf:resource="descriptor/19230-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70234">
    <skos:narrower rdf:resource="descriptor/10253-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70234">
    <skos:narrower rdf:resource="descriptor/10051-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70234">
    <skos:narrower rdf:resource="descriptor/19229-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70234">
    <skos:narrower rdf:resource="descriptor/10251-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70234">
    <skos:narrower rdf:resource="descriptor/10145-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70234">
    <skos:narrower rdf:resource="descriptor/10048-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70234">
    <skos:narrower rdf:resource="descriptor/19666-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70234">
    <skos:narrower rdf:resource="descriptor/10249-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70234">
    <skos:narrower rdf:resource="descriptor/10248-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70234">
    <skos:narrower rdf:resource="descriptor/10043-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70234">
    <skos:narrower rdf:resource="descriptor/10042-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70234">
    <skos:narrower rdf:resource="descriptor/10246-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70234">
    <skos:narrower rdf:resource="descriptor/10241-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70234">
    <skos:narrower rdf:resource="descriptor/11004-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70234">
    <skos:narrower rdf:resource="descriptor/10237-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70234">
    <skos:narrower rdf:resource="descriptor/10286-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70234">
    <skos:narrower rdf:resource="descriptor/10229-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70234">
    <skos:narrower rdf:resource="thsys/70530"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70234">
    <skos:narrower rdf:resource="thsys/71112"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70234">
    <skos:narrower rdf:resource="thsys/71111"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70234">
    <skos:narrower rdf:resource="thsys/70486"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70234">
    <skos:narrower rdf:resource="thsys/70233"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70234">
    <skos:narrower rdf:resource="thsys/71115"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11146-3">
    <skos:related rdf:resource="descriptor/12073-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11146-3">
    <skos:related rdf:resource="descriptor/12295-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11146-3">
    <skos:related rdf:resource="descriptor/12891-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19177-1">
    <skos:narrower rdf:resource="descriptor/17829-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19177-1">
    <skos:narrower rdf:resource="descriptor/17568-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19177-1">
    <skos:narrower rdf:resource="descriptor/17374-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19177-1">
    <skos:narrower rdf:resource="descriptor/17255-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19177-1">
    <skos:narrower rdf:resource="descriptor/18012-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19790-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14751-6">
    <skos:related rdf:resource="descriptor/15621-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14751-6">
    <skos:related rdf:resource="descriptor/15616-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14751-6">
    <skos:related rdf:resource="descriptor/16757-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14751-6">
    <skos:related rdf:resource="descriptor/15609-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14751-6">
    <skos:related rdf:resource="descriptor/15606-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14751-6">
    <skos:related rdf:resource="descriptor/18208-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14751-6">
    <skos:related rdf:resource="descriptor/15599-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14751-6">
    <skos:related rdf:resource="descriptor/18758-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14751-6">
    <skos:related rdf:resource="descriptor/16190-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14751-6">
    <skos:related rdf:resource="descriptor/13072-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14751-6">
    <skos:related rdf:resource="descriptor/18523-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14751-6">
    <skos:related rdf:resource="descriptor/14750-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15658-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19820-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19505-3">
    <skos:scopeNote xml:lang="de">Vorausberechnungen des künftigen Bedarfs oder der Ersatzraten für einzelne Berufsgruppen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12091-4">
    <zbwext:useInsteadNote rdf:nodeID="id245"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12091-4">
    <zbwext:useInsteadNote rdf:nodeID="id244"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19117-5">
    <skos:altLabel xml:lang="en">CIQ (Computer-Integrated Quality Assurance)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19117-5">
    <skos:altLabel xml:lang="de">Computerintegrierte Qualitätssicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19117-5">
    <skos:altLabel xml:lang="de">Computergestützte Qualitätssicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11414-4">
    <skos:altLabel xml:lang="en">Financial intermediary</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11414-4">
    <skos:altLabel xml:lang="de">Finanzmittler</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11414-4">
    <skos:altLabel xml:lang="de">Finanzintermediation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19072-1">
    <skos:altLabel xml:lang="de">Statistische Übersicht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19072-1">
    <skos:altLabel xml:lang="de">Statistische Daten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19652-4">
    <skos:prefLabel xml:lang="en">Nasdac</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19652-4">
    <skos:prefLabel xml:lang="de">Neuer Markt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16411-6">
    <skos:altLabel xml:lang="de">Politischer Liberalismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12215-5">
    <skos:narrower rdf:resource="descriptor/18539-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12215-5">
    <skos:narrower rdf:resource="descriptor/18084-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12215-5">
    <skos:narrower rdf:resource="descriptor/12309-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11377-5">
    <skos:altLabel xml:lang="de">Schulwesen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11377-5">
    <skos:altLabel xml:lang="de">Schulsystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id170">
    <rdf:value xml:lang="de">ABER: Für den gesamtwirtschaftlichen Verbrauch</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10816-3">
    <skos:prefLabel xml:lang="en">Country risk</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10816-3">
    <skos:prefLabel xml:lang="de">Länderrisiko</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12070-5">
    <skos:broader rdf:resource="thsys/71034"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19051-2">
    <skos:related rdf:resource="descriptor/13039-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19051-2">
    <skos:related rdf:resource="descriptor/13029-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19051-2">
    <skos:related rdf:resource="descriptor/14285-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70962">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17176-6">
    <skos:related rdf:resource="descriptor/11123-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17176-6">
    <skos:related rdf:resource="descriptor/18226-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17176-6">
    <skos:related rdf:resource="descriptor/19087-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17176-6">
    <skos:related rdf:resource="descriptor/18228-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17176-6">
    <skos:related rdf:resource="descriptor/18227-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17176-6">
    <skos:related rdf:resource="descriptor/19086-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18098-2">
    <skos:altLabel xml:lang="de">Schleichwerbung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18098-2">
    <skos:altLabel xml:lang="de">Produktplazierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18098-2">
    <skos:altLabel xml:lang="de">Produktplatzierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12111-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18299-6">
    <skos:related rdf:resource="descriptor/18521-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19505-3">
    <skos:broader rdf:resource="thsys/71058"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12651-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19460-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12971-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10787-5">
    <skos:altLabel xml:lang="de">Übertragungsbilanz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10787-5">
    <skos:altLabel xml:lang="de">Schenkungsbilanz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10787-5">
    <skos:altLabel xml:lang="de">Patentbilanz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10787-5">
    <skos:altLabel xml:lang="de">Lizenzbilanz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10787-5">
    <skos:altLabel xml:lang="de">Dienstleistungsbilanz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13379-5">
    <skos:prefLabel xml:lang="en">Computer services</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13379-5">
    <skos:prefLabel xml:lang="de">EDV-Dienstleistung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10827-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12121-0">
    <skos:related rdf:resource="descriptor/12105-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12121-0">
    <skos:related rdf:resource="descriptor/12125-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18350-4">
    <skos:altLabel xml:lang="de">Energiewirtschaftslehre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18350-4">
    <skos:altLabel xml:lang="de">Energieökonomie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17497-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11224-2">
    <skos:altLabel xml:lang="de">Personale Infrastruktur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11224-2">
    <skos:altLabel xml:lang="de">Menschliches Kapital</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11224-2">
    <skos:altLabel xml:lang="de">Immaterielles Kapital</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11224-2">
    <skos:altLabel xml:lang="de">Humankapitaltheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11224-2">
    <skos:altLabel xml:lang="de">Humankapitalkonzept</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11224-2">
    <skos:altLabel xml:lang="de">Arbeitsvermögen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11224-2">
    <skos:altLabel xml:lang="de">Arbeitskapital</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12609-5">
    <skos:altLabel xml:lang="en">Occupational health and safety</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12609-5">
    <skos:altLabel xml:lang="de">Unfallverhütung am Arbeitsplatz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12609-5">
    <skos:altLabel xml:lang="de">Unfallschutz im Betrieb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12609-5">
    <skos:altLabel xml:lang="de">Sicherheit am Arbeitsplatz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12609-5">
    <skos:altLabel xml:lang="de">Lärmschutz im Betrieb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12609-5">
    <skos:altLabel xml:lang="de">Betriebsgefahrenschutz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12609-5">
    <skos:altLabel xml:lang="de">Arbeitsstättenverordnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12609-5">
    <skos:altLabel xml:lang="de">Arbeitssicherheitsgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12609-5">
    <skos:altLabel xml:lang="de">Arbeitssicherheit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12609-5">
    <skos:altLabel xml:lang="de">Arbeitsschutzrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12609-5">
    <skos:altLabel xml:lang="de">Arbeitsschutzgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20176-0">
    <skos:prefLabel xml:lang="en">Credence goods</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20176-0">
    <skos:prefLabel xml:lang="de">Vertrauensgüter</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10557-1">
    <skos:prefLabel xml:lang="en">Development cooperation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10557-1">
    <skos:prefLabel xml:lang="de">Entwicklungskooperation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20485-1">
    <skos:broader rdf:resource="thsys/73336"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id133">
    <rdf:value xml:lang="de">Für Herstellkosten im Sinne der Kostenrechnung</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11824-6">
    <skos:related rdf:resource="descriptor/11823-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11824-6">
    <skos:related rdf:resource="descriptor/13460-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71004">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14527-4">
    <skos:broader rdf:resource="thsys/70423"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14527-4">
    <skos:broader rdf:resource="descriptor/14537-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13492-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19354-5">
    <skos:prefLabel xml:lang="en">Convergence criteria</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19354-5">
    <skos:prefLabel xml:lang="de">Konvergenzkriterien</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13322-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16777-6">
    <skos:prefLabel xml:lang="en">Indexing and retrieval language</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16777-6">
    <skos:prefLabel xml:lang="de">Dokumentationssprache</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12693-4">
    <skos:scopeNote xml:lang="de">Für finanzielle Fördermittel BENUTZE die entsprechenden Deskriptoren.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12693-4">
    <skos:scopeNote xml:lang="de">Nur für technische Fördereinrichtungen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70028">
    <skos:narrower rdf:resource="descriptor/20261-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70028">
    <skos:narrower rdf:resource="thsys/70087"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70028">
    <skos:narrower rdf:resource="thsys/70485"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70028">
    <skos:narrower rdf:resource="thsys/70562"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70028">
    <skos:narrower rdf:resource="thsys/71034"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70028">
    <skos:narrower rdf:resource="thsys/70383"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70028">
    <skos:narrower rdf:resource="thsys/71033"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70386">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16286-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12801-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73346">
    <skos:broader rdf:resource="thsys/73347"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19662-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10041-1">
    <skos:broader rdf:resource="thsys/70141"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10041-1">
    <skos:broader rdf:resource="descriptor/16745-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19294-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13829-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15372-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18739-4">
    <skos:broader rdf:resource="thsys/70891"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18739-4">
    <skos:broader rdf:resource="thsys/70832"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18739-4">
    <skos:broader rdf:resource="thsys/70458"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18739-4">
    <skos:broader rdf:resource="descriptor/13811-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18739-4">
    <skos:broader rdf:resource="descriptor/13294-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18739-4">
    <skos:broader rdf:resource="descriptor/16527-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71108">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.05.06.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12458-0">
    <skos:altLabel xml:lang="de">Theorie des Jahresabschlusses</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12458-0">
    <skos:altLabel xml:lang="de">Theorie der Rechnungslegung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12458-0">
    <skos:altLabel xml:lang="de">Rechnungslegungszweck</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12458-0">
    <skos:altLabel xml:lang="de">Rechnungslegungstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12458-0">
    <skos:altLabel xml:lang="de">Kontentheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12458-0">
    <skos:altLabel xml:lang="de">Buchführungstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12458-0">
    <skos:altLabel xml:lang="de">Bilanzauffassung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18008-1">
    <skos:broader rdf:resource="thsys/71091"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18008-1">
    <skos:broader rdf:resource="thsys/71028"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18008-1">
    <skos:broader rdf:resource="descriptor/10605-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11101-4">
    <skos:broader rdf:resource="thsys/71102"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11101-4">
    <skos:broader rdf:resource="descriptor/11046-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13179-6">
    <skos:prefLabel xml:lang="en">Hydraulic engineering</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13179-6">
    <skos:prefLabel xml:lang="de">Wasserbau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16802-5">
    <skos:prefLabel xml:lang="en">Library network</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16802-5">
    <skos:prefLabel xml:lang="de">Bibliotheksverbund</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14515-4">
    <skos:prefLabel xml:lang="en">Television set</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14515-4">
    <skos:prefLabel xml:lang="de">Fernsehgerät</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11919-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12985-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11833-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15362-5">
    <skos:related rdf:resource="descriptor/19574-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14725-0">
    <skos:prefLabel xml:lang="en">Hormone preparation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14725-0">
    <skos:prefLabel xml:lang="de">Hormonpräparat</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11652-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18773-4">
    <skos:broader rdf:resource="thsys/73362"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18773-4">
    <skos:broader rdf:resource="thsys/70879"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18773-4">
    <skos:broader rdf:resource="descriptor/10437-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18773-4">
    <skos:broader rdf:resource="descriptor/18772-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13000-0">
    <skos:prefLabel xml:lang="en">Plastics manufacturing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13000-0">
    <skos:prefLabel xml:lang="de">Kunststoffherstellung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18106-1">
    <skos:altLabel xml:lang="en">Profit determinants</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18106-1">
    <skos:altLabel xml:lang="de">Strategischer Erfolgsfaktor</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19515-0">
    <skos:related rdf:resource="descriptor/19513-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15655-4">
    <skos:altLabel xml:lang="en">Performing arts</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15655-4">
    <skos:altLabel xml:lang="de">Tanzkunst</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15655-4">
    <skos:altLabel xml:lang="de">Schauspielkunst</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15655-4">
    <skos:altLabel xml:lang="de">Pantomime</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15655-4">
    <skos:altLabel xml:lang="de">Malerei</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15655-4">
    <skos:altLabel xml:lang="de">Kunstepoche</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15655-4">
    <skos:altLabel xml:lang="de">Filmkunst</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15655-4">
    <skos:altLabel xml:lang="de">Darstellende Kunst</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15655-4">
    <skos:altLabel xml:lang="de">Bildhauerei</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15655-4">
    <skos:altLabel xml:lang="de">Bildende Kunst</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19566-4">
    <skos:altLabel xml:lang="de">Lohnkosten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18173-0">
    <skos:scopeNote xml:lang="de">Die Leistungsintensität der Arbeit hängt von der Lohnhöhe ab und beeinflusst das Produktionsergebnis.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13600-4">
    <skos:broader rdf:resource="thsys/70927"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13600-4">
    <skos:broader rdf:resource="thsys/70386"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12051-2">
    <skos:broader rdf:resource="thsys/71005"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12051-2">
    <skos:broader rdf:resource="descriptor/12041-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11927-3">
    <skos:altLabel xml:lang="en">General equilibrium regional model</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11927-3">
    <skos:altLabel xml:lang="de">Theorie des räumlichen Gleichgewichts</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11927-3">
    <skos:altLabel xml:lang="de">Regionale Gleichgewichtstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16719-6">
    <skos:broader rdf:resource="thsys/73394"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16719-6">
    <skos:broader rdf:resource="descriptor/15644-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12203-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70274">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19823-3">
    <skos:altLabel xml:lang="de">Nutzung (Bürgerliches Recht)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19823-3">
    <skos:altLabel xml:lang="de">Nutznießung (Bürgerliches Recht)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19823-3">
    <skos:altLabel xml:lang="de">Nießbrauch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19823-3">
    <skos:altLabel xml:lang="de">Dienstbarkeiten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19232-5">
    <skos:related rdf:resource="descriptor/11683-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19232-5">
    <skos:related rdf:resource="descriptor/10024-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19232-5">
    <skos:related rdf:resource="descriptor/19095-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19232-5">
    <skos:related rdf:resource="descriptor/12285-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10076-3">
    <skos:broader rdf:resource="thsys/70188"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10076-3">
    <skos:broader rdf:resource="descriptor/10062-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20491-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10771-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15111-2">
    <skos:narrower rdf:resource="descriptor/15243-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15062-3">
    <skos:related rdf:resource="descriptor/15298-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15062-3">
    <skos:related rdf:resource="descriptor/15272-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15062-3">
    <skos:related rdf:resource="descriptor/15315-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15062-3">
    <skos:related rdf:resource="descriptor/15036-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15768-5">
    <skos:prefLabel xml:lang="en">Environmental policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15768-5">
    <skos:prefLabel xml:lang="de">Umweltpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12681-4">
    <skos:altLabel xml:lang="de">Regalsystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12681-4">
    <skos:altLabel xml:lang="de">Palettierautomat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12681-4">
    <skos:altLabel xml:lang="de">Palette</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12681-4">
    <skos:altLabel xml:lang="de">Lagerausrüstung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12681-4">
    <skos:altLabel xml:lang="de">Hochregallager</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12681-4">
    <skos:altLabel xml:lang="de">Automatische Palettieranlage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11876-1">
    <skos:altLabel xml:lang="de">Regionaler Verkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11876-1">
    <skos:altLabel xml:lang="de">Kurzstreckenverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12881-3">
    <skos:broader rdf:resource="thsys/70875"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11185-0">
    <skos:related rdf:resource="descriptor/16081-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11185-0">
    <skos:related rdf:resource="descriptor/19491-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11185-0">
    <skos:related rdf:resource="descriptor/16159-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11185-0">
    <skos:related rdf:resource="descriptor/12567-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11185-0">
    <skos:related rdf:resource="descriptor/18829-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11185-0">
    <skos:related rdf:resource="descriptor/11310-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11185-0">
    <skos:related rdf:resource="descriptor/16052-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11185-0">
    <skos:related rdf:resource="descriptor/16134-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11185-0">
    <skos:related rdf:resource="descriptor/12602-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11185-0">
    <skos:related rdf:resource="descriptor/19844-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11185-0">
    <skos:related rdf:resource="descriptor/11235-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11185-0">
    <skos:related rdf:resource="descriptor/11262-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11185-0">
    <skos:related rdf:resource="descriptor/11339-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11185-0">
    <skos:related rdf:resource="descriptor/11307-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11185-0">
    <skos:related rdf:resource="descriptor/11314-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11185-0">
    <skos:related rdf:resource="descriptor/11180-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11185-0">
    <skos:related rdf:resource="descriptor/19003-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15679-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18198-5">
    <skos:prefLabel xml:lang="en">Grain production</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18198-5">
    <skos:prefLabel xml:lang="de">Getreideanbau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24621-6">
    <skos:narrower rdf:resource="descriptor/12976-6"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id388">
    <rdf:value xml:lang="de">Für den Anbau von Zuckerrüben und Zuckerrohr</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19673-3">
    <skos:altLabel xml:lang="en">Company value</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19673-3">
    <skos:altLabel xml:lang="de">Gesamtwert der Unternehmung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19017-2">
    <skos:broader rdf:resource="thsys/73304"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19017-2">
    <skos:broader rdf:resource="descriptor/14851-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19340-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10013-6">
    <skos:narrower rdf:resource="descriptor/19356-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10013-6">
    <skos:narrower rdf:resource="descriptor/10237-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10013-6">
    <skos:narrower rdf:resource="descriptor/10185-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12863-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15947-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70445">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12605-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73301">
    <skos:broader rdf:resource="thsys/70498"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15415-3">
    <skos:scopeNote xml:lang="de">Wissenschaft von den dynamischen Systemen, von deren selbständiger Steuerung und Regelung sowie der Informationsverarbeitung in Systemen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19291-3">
    <skos:altLabel xml:lang="de">Standortsicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18678-5">
    <skos:related rdf:resource="descriptor/19267-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18678-5">
    <skos:related rdf:resource="descriptor/18681-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18678-5">
    <skos:related rdf:resource="descriptor/10019-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18678-5">
    <skos:related rdf:resource="descriptor/18178-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13594-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15078-2">
    <skos:related rdf:resource="descriptor/10834-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12042-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12404-2">
    <skos:related rdf:resource="descriptor/12433-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12404-2">
    <skos:related rdf:resource="descriptor/12677-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12404-2">
    <skos:related rdf:resource="descriptor/19091-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19044-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14522-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14131-1">
    <skos:related rdf:resource="descriptor/14633-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14131-1">
    <skos:related rdf:resource="descriptor/14123-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18815-0">
    <skos:related rdf:resource="descriptor/18817-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18815-0">
    <skos:related rdf:resource="descriptor/11560-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18815-0">
    <skos:related rdf:resource="descriptor/19724-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18815-0">
    <skos:related rdf:resource="descriptor/16252-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14545-2">
    <skos:altLabel xml:lang="de">Tafelgerät</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14545-2">
    <skos:altLabel xml:lang="de">Schere</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14545-2">
    <skos:altLabel xml:lang="de">Messer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14545-2">
    <skos:altLabel xml:lang="de">Besteck</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17104-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16975-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13993-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18871-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18449-6">
    <skos:altLabel xml:lang="de">Pressewesen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11111-1">
    <skos:altLabel xml:lang="de">Wertlehre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11111-1">
    <skos:altLabel xml:lang="de">Tauschwert</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11111-1">
    <skos:altLabel xml:lang="de">Subjektive Wertlehre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12786-4">
    <skos:prefLabel xml:lang="en">Customer service</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12786-4">
    <skos:prefLabel xml:lang="de">Kundendienst</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16666-1">
    <skos:altLabel xml:lang="en">Alcohol abuse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16666-1">
    <skos:altLabel xml:lang="de">Trunksucht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16666-1">
    <skos:altLabel xml:lang="de">Trinksitten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16666-1">
    <skos:altLabel xml:lang="de">Trinkkultur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16666-1">
    <skos:altLabel xml:lang="de">Alkoholsucht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16666-1">
    <skos:altLabel xml:lang="de">Alkoholmissbrauch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16666-1">
    <skos:altLabel xml:lang="de">Alkoholmißbrauch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16666-1">
    <skos:altLabel xml:lang="de">Alkoholismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16666-1">
    <skos:altLabel xml:lang="de">Alkoholabhängigkeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10673-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19708-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10032-2">
    <skos:broader rdf:resource="thsys/70175"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10032-2">
    <skos:broader rdf:resource="thsys/70140"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10032-2">
    <skos:broader rdf:resource="thsys/70049"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10032-2">
    <skos:broader rdf:resource="descriptor/16563-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13656-5">
    <skos:prefLabel xml:lang="en">Cable television</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13656-5">
    <skos:prefLabel xml:lang="de">Kabelfernsehen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11046-0">
    <skos:altLabel xml:lang="de">Kapitalistische Gesellschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11046-0">
    <skos:altLabel xml:lang="de">Kapitalismustheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18984-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11185-0">
    <skos:narrower rdf:resource="descriptor/16150-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11185-0">
    <skos:narrower rdf:resource="descriptor/11347-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11185-0">
    <skos:narrower rdf:resource="descriptor/11191-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11185-0">
    <skos:narrower rdf:resource="descriptor/19812-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11185-0">
    <skos:narrower rdf:resource="descriptor/19503-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11185-0">
    <skos:narrower rdf:resource="descriptor/16136-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11185-0">
    <skos:narrower rdf:resource="descriptor/16131-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11185-0">
    <skos:narrower rdf:resource="descriptor/16124-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14130-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11962-1">
    <skos:altLabel xml:lang="en">Spatial statistics</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11962-1">
    <skos:altLabel xml:lang="de">Regionaldaten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20977-1">
    <skos:prefLabel xml:lang="en">Modularity</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20977-1">
    <skos:prefLabel xml:lang="de">Modularisierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id323">
    <rdfs:seeAlso rdf:resource="descriptor/10143-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71007">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18751-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14101-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15176-2">
    <skos:related rdf:resource="descriptor/15111-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15689-1">
    <skos:prefLabel xml:lang="en">Antiquity</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15689-1">
    <skos:prefLabel xml:lang="de">Altertum</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16633-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21061-5">
    <skos:altLabel xml:lang="de">Wirtschaftsgut (Steuerrecht)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21061-5">
    <skos:altLabel xml:lang="de">Geringwertige Wirtschaftsgüter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21061-5">
    <skos:altLabel xml:lang="de">Bewegliches Wirtschaftsgut</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70872">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12146-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12105-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13480-2">
    <skos:narrower rdf:resource="descriptor/13490-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13480-2">
    <skos:narrower rdf:resource="descriptor/18520-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10398-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13159-5">
    <skos:related rdf:resource="descriptor/14936-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13159-5">
    <skos:related rdf:resource="descriptor/18517-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13159-5">
    <skos:related rdf:resource="descriptor/18440-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19736-5">
    <skos:prefLabel xml:lang="en">Catholicism</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19736-5">
    <skos:prefLabel xml:lang="de">Katholizismus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16032-0">
    <skos:altLabel xml:lang="de">IPR (Internationales Privatrecht)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14144-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12562-5">
    <skos:altLabel xml:lang="de">Personalausstattung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16489-4">
    <skos:prefLabel xml:lang="en">Foreign policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16489-4">
    <skos:prefLabel xml:lang="de">Außenpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10236-0">
    <skos:narrower rdf:resource="descriptor/10137-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10236-0">
    <skos:narrower rdf:resource="descriptor/19259-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10236-0">
    <skos:narrower rdf:resource="descriptor/11473-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10236-0">
    <skos:narrower rdf:resource="descriptor/10250-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10236-0">
    <skos:narrower rdf:resource="descriptor/18696-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10236-0">
    <skos:narrower rdf:resource="descriptor/10242-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10236-0">
    <skos:narrower rdf:resource="descriptor/11452-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10236-0">
    <skos:narrower rdf:resource="descriptor/11449-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10236-0">
    <skos:narrower rdf:resource="descriptor/11444-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10236-0">
    <skos:narrower rdf:resource="descriptor/11443-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16458-1">
    <skos:prefLabel xml:lang="en">Military potential</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16458-1">
    <skos:prefLabel xml:lang="de">Militärpotenzial</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19236-4">
    <zbwext:useInsteadNote rdf:nodeID="id343"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71095">
    <skos:prefLabel xml:lang="en">V.07.03  Currency</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71095">
    <skos:prefLabel xml:lang="de">V.07.03  Währung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16695-1">
    <skos:prefLabel xml:lang="en">Sociology of technology</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16695-1">
    <skos:prefLabel xml:lang="de">Techniksoziologie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14428-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70855">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.23.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19351-4">
    <skos:related rdf:resource="descriptor/10628-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19351-4">
    <skos:related rdf:resource="descriptor/10627-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19351-4">
    <skos:related rdf:resource="descriptor/18181-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19351-4">
    <skos:related rdf:resource="descriptor/10612-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11754-1">
    <skos:scopeNote xml:lang="de">Ökonomischer Rentenbegriff.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17800-4">
    <skos:broader rdf:resource="thsys/70086"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17800-4">
    <skos:broader rdf:resource="descriptor/17791-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18960-5">
    <skos:broader rdf:resource="thsys/71033"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18960-5">
    <skos:broader rdf:resource="thsys/70818"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18960-5">
    <skos:broader rdf:resource="thsys/70301"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18960-5">
    <skos:broader rdf:resource="descriptor/18165-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18960-5">
    <skos:broader rdf:resource="descriptor/12648-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18960-5">
    <skos:broader rdf:resource="descriptor/13325-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70865">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18792-0">
    <skos:related rdf:resource="descriptor/19491-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18792-0">
    <skos:related rdf:resource="descriptor/11258-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18792-0">
    <skos:related rdf:resource="descriptor/11203-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18792-0">
    <skos:related rdf:resource="descriptor/19005-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18792-0">
    <skos:related rdf:resource="descriptor/11248-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19682-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11284-5">
    <skos:narrower rdf:resource="descriptor/18978-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11407-1">
    <skos:broader rdf:resource="thsys/70847"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11407-1">
    <skos:broader rdf:resource="thsys/70176"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11407-1">
    <skos:broader rdf:resource="descriptor/10032-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17829-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18172-2">
    <skos:altLabel xml:lang="de">Datenstruktur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18172-2">
    <skos:altLabel xml:lang="de">Datenformat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15571-3">
    <skos:related rdf:resource="descriptor/15554-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15571-3">
    <skos:related rdf:resource="descriptor/12677-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15571-3">
    <skos:related rdf:resource="descriptor/12656-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12903-5">
    <skos:broader rdf:resource="thsys/71041"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12903-5">
    <skos:broader rdf:resource="thsys/70876"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70923">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18800-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16828-1">
    <skos:prefLabel xml:lang="en">Czechoslovakia</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16828-1">
    <skos:prefLabel xml:lang="de">Tschechoslowakei</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14440-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11153-6">
    <skos:prefLabel xml:lang="en">Agency theory</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11153-6">
    <skos:prefLabel xml:lang="de">Agency Theory</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19022-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12783-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11844-0">
    <skos:narrower rdf:resource="descriptor/11999-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13384-5">
    <skos:broader rdf:resource="thsys/70875"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13384-5">
    <skos:broader rdf:resource="descriptor/12881-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12314-3">
    <skos:broader rdf:resource="thsys/71043"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12314-3">
    <skos:broader rdf:resource="thsys/70244"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18347-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13382-2">
    <skos:broader rdf:resource="thsys/73296"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13382-2">
    <skos:broader rdf:resource="thsys/71055"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13382-2">
    <skos:broader rdf:resource="thsys/70825"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13382-2">
    <skos:broader rdf:resource="descriptor/14004-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18506-6">
    <skos:prefLabel xml:lang="en">Building business cycle</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18506-6">
    <skos:prefLabel xml:lang="de">Baukonjunktur</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16305-5">
    <skos:broader rdf:resource="thsys/73382"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16305-5">
    <skos:broader rdf:resource="descriptor/16292-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19280-1">
    <skos:altLabel xml:lang="de">Black-Scholes Modell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12234-1">
    <skos:broader rdf:resource="thsys/71105"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12234-1">
    <skos:broader rdf:resource="thsys/71045"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12234-1">
    <skos:broader rdf:resource="descriptor/13721-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13498-4">
    <skos:prefLabel xml:lang="en">Charter traffic</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13498-4">
    <skos:prefLabel xml:lang="de">Charterverkehr</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13755-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70354">
    <skos:narrower rdf:resource="descriptor/17652-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70354">
    <skos:narrower rdf:resource="thsys/70445"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70354">
    <skos:narrower rdf:resource="thsys/70429"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70354">
    <skos:narrower rdf:resource="thsys/70560"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70354">
    <skos:narrower rdf:resource="thsys/71031"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11554-2">
    <skos:narrower rdf:resource="descriptor/11568-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11554-2">
    <skos:narrower rdf:resource="descriptor/10267-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11554-2">
    <skos:narrower rdf:resource="descriptor/11558-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11554-2">
    <skos:narrower rdf:resource="descriptor/11556-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11554-2">
    <skos:narrower rdf:resource="descriptor/11409-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11554-2">
    <skos:narrower rdf:resource="descriptor/11553-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11554-2">
    <skos:narrower rdf:resource="descriptor/11318-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11501-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19768-6">
    <skos:broader rdf:resource="thsys/73394"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19768-6">
    <skos:broader rdf:resource="thsys/70154"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19350-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13813-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11065-3">
    <skos:prefLabel xml:lang="en">Theory of Property Rights</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11065-3">
    <skos:prefLabel xml:lang="de">Eigentumsrechtstheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12631-5">
    <skos:related rdf:resource="descriptor/12620-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12631-5">
    <skos:related rdf:resource="descriptor/12574-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12631-5">
    <skos:related rdf:resource="descriptor/19011-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12631-5">
    <skos:related rdf:resource="descriptor/11187-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12631-5">
    <skos:related rdf:resource="descriptor/11303-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16721-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13574-0">
    <skos:prefLabel xml:lang="en">Railway engineering</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13574-0">
    <skos:prefLabel xml:lang="de">Eisenbahntechnik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11536-4">
    <skos:narrower rdf:resource="descriptor/16110-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18208-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19579-2">
    <skos:broader rdf:resource="thsys/72181"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19579-2">
    <skos:broader rdf:resource="thsys/70965"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19579-2">
    <skos:broader rdf:resource="descriptor/15076-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10816-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11574-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16767-2">
    <skos:altLabel xml:lang="de">Literaturbeschaffung in Bibliotheken</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16767-2">
    <skos:altLabel xml:lang="de">Bestandsaufbau in Bibliotheken</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16767-2">
    <skos:altLabel xml:lang="de">Beschaffungspolitik in Bibliotheken</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16767-2">
    <skos:altLabel xml:lang="de">Beschaffung in Bibliotheken</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16767-2">
    <skos:altLabel xml:lang="de">Akzession</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11803-0">
    <skos:scopeNote xml:lang="de">Kennzeichnet die Struktur der Landwirtschaft und des ländlichen Raumes z.B. hinsichtlich der Betriebsgrößen, der Bewirtschaftungsformen, der Bodennutzung und der Siedlungsformen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10864-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14245-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19475-0">
    <zbwext:useInsteadNote rdf:nodeID="id145"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14177-5">
    <skos:prefLabel xml:lang="en">District heating</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14177-5">
    <skos:prefLabel xml:lang="de">Fernwärme</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13686-3">
    <skos:broader rdf:resource="thsys/70900"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13686-3">
    <skos:broader rdf:resource="descriptor/12041-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19215-5">
    <skos:prefLabel xml:lang="en">Contract theory</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19215-5">
    <skos:prefLabel xml:lang="de">Vertragstheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15518-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18610-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13816-2">
    <skos:narrower rdf:resource="descriptor/13809-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13816-2">
    <skos:narrower rdf:resource="descriptor/13815-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14724-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73390">
    <skos:narrower rdf:resource="descriptor/16046-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73390">
    <skos:narrower rdf:resource="descriptor/11848-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73390">
    <skos:narrower rdf:resource="descriptor/11069-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73390">
    <skos:narrower rdf:resource="descriptor/16202-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73390">
    <skos:narrower rdf:resource="descriptor/16081-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73390">
    <skos:narrower rdf:resource="descriptor/16120-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73390">
    <skos:narrower rdf:resource="descriptor/16023-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73390">
    <skos:narrower rdf:resource="descriptor/11417-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73390">
    <skos:narrower rdf:resource="descriptor/11580-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73390">
    <skos:narrower rdf:resource="descriptor/16072-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73390">
    <skos:narrower rdf:resource="descriptor/16001-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73390">
    <skos:narrower rdf:resource="descriptor/13731-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73390">
    <skos:narrower rdf:resource="descriptor/16061-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73390">
    <skos:narrower rdf:resource="descriptor/16102-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73390">
    <skos:narrower rdf:resource="descriptor/10621-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73390">
    <skos:narrower rdf:resource="descriptor/16058-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73390">
    <skos:narrower rdf:resource="descriptor/18677-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73390">
    <skos:narrower rdf:resource="descriptor/16055-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73390">
    <skos:narrower rdf:resource="descriptor/16052-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73390">
    <skos:narrower rdf:resource="descriptor/18311-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73390">
    <skos:narrower rdf:resource="descriptor/18312-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73390">
    <skos:narrower rdf:resource="descriptor/18372-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73390">
    <skos:narrower rdf:resource="descriptor/15993-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73390">
    <skos:narrower rdf:resource="descriptor/19838-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73390">
    <skos:narrower rdf:resource="descriptor/13694-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73390">
    <skos:narrower rdf:resource="descriptor/10917-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73390">
    <skos:narrower rdf:resource="descriptor/10780-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73390">
    <skos:narrower rdf:resource="descriptor/18676-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73390">
    <skos:narrower rdf:resource="descriptor/19818-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73390">
    <skos:narrower rdf:resource="descriptor/18352-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73390">
    <skos:narrower rdf:resource="descriptor/13693-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73390">
    <skos:narrower rdf:resource="descriptor/19315-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73390">
    <skos:narrower rdf:resource="descriptor/10606-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73390">
    <skos:narrower rdf:resource="descriptor/18702-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73390">
    <skos:narrower rdf:resource="descriptor/13451-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73390">
    <skos:narrower rdf:resource="descriptor/13426-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70358">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id349">
    <rdf:value xml:lang="de">Für idealtypisches Verhalten des homo oeconomicus</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18850-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16247-0">
    <skos:broader rdf:resource="thsys/73382"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16247-0">
    <skos:broader rdf:resource="descriptor/16245-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11536-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14953-1">
    <skos:prefLabel xml:lang="en">Yogurt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14953-1">
    <skos:prefLabel xml:lang="de">Joghurt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18274-1">
    <skos:prefLabel xml:lang="en">Photovoltaic technology</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18274-1">
    <skos:prefLabel xml:lang="de">Photovoltaikanlage</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12331-3">
    <skos:related rdf:resource="descriptor/16048-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13496-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14175-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12749-3">
    <skos:prefLabel xml:lang="en">Turnaround time</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12749-3">
    <skos:prefLabel xml:lang="de">Durchlaufzeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14662-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14385-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18196-2">
    <skos:prefLabel xml:lang="en">Tobacco production</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18196-2">
    <skos:prefLabel xml:lang="de">Tabakanbau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11638-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11932-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16353-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11142-4">
    <skos:broader rdf:resource="thsys/71101"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11142-4">
    <skos:broader rdf:resource="thsys/71072"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11142-4">
    <skos:broader rdf:resource="thsys/70860"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11142-4">
    <skos:broader rdf:resource="descriptor/12072-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14989-1">
    <skos:narrower rdf:resource="descriptor/14990-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14989-1">
    <skos:narrower rdf:resource="descriptor/14993-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14989-1">
    <skos:narrower rdf:resource="descriptor/14991-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12404-2">
    <skos:scopeNote xml:lang="de">Ausweisung der Lagervorräte in der Bilanz</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19206-6">
    <skos:prefLabel xml:lang="en">Public finance traditions</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19206-6">
    <skos:prefLabel xml:lang="de">Finanzwissenschaftliche Dogmengeschichte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19547-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10829-1">
    <skos:narrower rdf:resource="descriptor/11710-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10829-1">
    <skos:narrower rdf:resource="descriptor/16090-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10829-1">
    <skos:narrower rdf:resource="descriptor/10828-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10829-1">
    <skos:narrower rdf:resource="descriptor/11698-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13511-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10025-6">
    <skos:altLabel xml:lang="de">Wirtschaft (Volkswirtschaft)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10025-6">
    <skos:altLabel xml:lang="de">Gesamtwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14906-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19183-6">
    <skos:broader rdf:resource="thsys/71025"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19183-6">
    <skos:broader rdf:resource="thsys/70960"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19183-6">
    <skos:broader rdf:resource="descriptor/13431-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13234-3">
    <skos:scopeNote xml:lang="de">Für den Handel mit Wein sowie mit Spirituosen</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12197-2">
    <skos:prefLabel xml:lang="en">Foundation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12197-2">
    <skos:prefLabel xml:lang="de">Stiftung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11691-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13063-4">
    <skos:broader rdf:resource="thsys/70999"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13063-4">
    <skos:broader rdf:resource="descriptor/13047-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11198-5">
    <skos:altLabel xml:lang="en">Industrial labor</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11198-5">
    <skos:altLabel xml:lang="de">Industrielle Arbeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18205-6">
    <skos:prefLabel xml:lang="en">Printing paper</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18205-6">
    <skos:prefLabel xml:lang="de">Druckpapier</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15837-5">
    <skos:related rdf:resource="descriptor/12620-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15837-5">
    <skos:related rdf:resource="descriptor/12585-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15837-5">
    <skos:related rdf:resource="descriptor/12589-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15837-5">
    <skos:related rdf:resource="descriptor/15839-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15837-5">
    <skos:related rdf:resource="descriptor/19002-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15837-5">
    <skos:related rdf:resource="descriptor/11052-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15837-5">
    <skos:related rdf:resource="descriptor/12603-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15837-5">
    <skos:related rdf:resource="descriptor/11192-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15837-5">
    <skos:related rdf:resource="descriptor/12577-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15837-5">
    <skos:related rdf:resource="descriptor/12592-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15837-5">
    <skos:related rdf:resource="descriptor/12617-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15837-5">
    <skos:related rdf:resource="descriptor/11305-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19516-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18171-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19324-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70549">
    <rdfs:label xml:lang="de">Maschine und Maschinenteil</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#hasTopConcept">
    <rdfs:label xml:lang="en">has top concept</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13666-2">
    <skos:altLabel xml:lang="de">Videokonferenz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13666-2">
    <skos:altLabel xml:lang="de">Fernsprechkonferenz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13666-2">
    <skos:altLabel xml:lang="de">Computerkonferenz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13666-2">
    <skos:altLabel xml:lang="de">Audiokonferenz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11810-3">
    <skos:altLabel xml:lang="de">Energiewirtschaftspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18302-1">
    <skos:altLabel xml:lang="de">Steuerrechtliche Bewertung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18302-1">
    <skos:altLabel xml:lang="de">Steuerbilanzrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12337-5">
    <skos:prefLabel xml:lang="en">GAAP (Generally Accepted Accounting Principles)</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12337-5">
    <skos:prefLabel xml:lang="de">Grundsätze ordnungsmäßiger Buchführung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16052-1">
    <skos:broader rdf:resource="thsys/73390"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16052-1">
    <skos:broader rdf:resource="descriptor/16081-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16052-1">
    <skos:broader rdf:resource="descriptor/16019-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18737-1">
    <skos:broader rdf:resource="thsys/71005"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18737-1">
    <skos:broader rdf:resource="thsys/70891"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18737-1">
    <skos:broader rdf:resource="descriptor/13808-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18293-4">
    <skos:altLabel xml:lang="en">Tile</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18293-4">
    <skos:altLabel xml:lang="de">Feinkeramische Fliese</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20289-1">
    <skos:altLabel xml:lang="x-other">Ecole de la Régulation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20289-1">
    <skos:altLabel xml:lang="de">Regulationstheorie (Entwicklungstheorie)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id74">
    <rdfs:seeAlso rdf:resource="descriptor/12849-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18982-2">
    <skos:related rdf:resource="descriptor/19462-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18982-2">
    <skos:related rdf:resource="descriptor/19460-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18982-2">
    <skos:related rdf:resource="descriptor/19233-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18982-2">
    <skos:related rdf:resource="descriptor/19362-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18982-2">
    <skos:related rdf:resource="descriptor/13454-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13682-4">
    <skos:narrower rdf:resource="descriptor/13681-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13682-4">
    <skos:narrower rdf:resource="descriptor/13268-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13682-4">
    <skos:narrower rdf:resource="descriptor/19658-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13682-4">
    <skos:narrower rdf:resource="descriptor/13674-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18020-4">
    <skos:prefLabel xml:lang="en">PC software</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18020-4">
    <skos:prefLabel xml:lang="de">PC-Software</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14525-1">
    <skos:altLabel xml:lang="en">Videorecorder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14525-1">
    <skos:altLabel xml:lang="en">Video cassette recorder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14525-1">
    <skos:altLabel xml:lang="en">VCR (Video Cassette Recorder)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14525-1">
    <skos:altLabel xml:lang="de">Videokamera</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14525-1">
    <skos:altLabel xml:lang="de">Videogerät</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14525-1">
    <skos:altLabel xml:lang="de">Kamerarecorder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14525-1">
    <skos:altLabel xml:lang="de">Camcorder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12436-3">
    <skos:broader rdf:resource="thsys/70004"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12436-3">
    <skos:broader rdf:resource="descriptor/19673-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13727-1">
    <skos:scopeNote xml:lang="de">Recht auf einen Anteil am Reingewinn.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10574-1">
    <skos:prefLabel xml:lang="en">Colonialism</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10574-1">
    <skos:prefLabel xml:lang="de">Kolonialismus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19471-1">
    <skos:prefLabel xml:lang="en">Urban unemployment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19471-1">
    <skos:prefLabel xml:lang="de">Städtische Arbeitslosigkeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19679-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13744-1">
    <skos:prefLabel xml:lang="en">Securities futures market</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13744-1">
    <skos:prefLabel xml:lang="de">Wertpapiertermingeschäft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14614-2">
    <skos:prefLabel xml:lang="en">Microform</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14614-2">
    <skos:prefLabel xml:lang="de">Mikroform</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16268-6">
    <skos:related rdf:resource="descriptor/16016-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16268-6">
    <skos:related rdf:resource="descriptor/16226-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16268-6">
    <skos:related rdf:resource="descriptor/20842-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16268-6">
    <skos:related rdf:resource="descriptor/16246-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10314-6">
    <skos:altLabel xml:lang="de">Faktorbewegung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11446-5">
    <skos:related rdf:resource="descriptor/11418-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19493-5">
    <skos:prefLabel xml:lang="en">Labour market flexibility</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19493-5">
    <skos:prefLabel xml:lang="de">Arbeitsmarktflexibilisierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19341-0">
    <skos:related rdf:resource="descriptor/13161-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19341-0">
    <skos:related rdf:resource="descriptor/19342-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19341-0">
    <skos:related rdf:resource="descriptor/12942-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19341-0">
    <skos:related rdf:resource="descriptor/14105-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19341-0">
    <skos:related rdf:resource="descriptor/13225-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10469-5">
    <skos:related rdf:resource="descriptor/16695-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10469-5">
    <skos:related rdf:resource="descriptor/18793-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10469-5">
    <skos:related rdf:resource="descriptor/11154-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16981-0">
    <skos:prefLabel xml:lang="en">Zug</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16981-0">
    <skos:prefLabel xml:lang="de">Zug (Kanton)</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17630-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12271-2">
    <skos:prefLabel xml:lang="en">Dividend</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12271-2">
    <skos:prefLabel xml:lang="de">Dividende</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19203-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id236">
    <rdfs:seeAlso rdf:resource="descriptor/19550-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18123-1">
    <skos:prefLabel xml:lang="en">Union behaviour</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18123-1">
    <skos:prefLabel xml:lang="de">Gewerkschaftspolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10361-4">
    <skos:narrower rdf:resource="descriptor/10375-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10361-4">
    <skos:narrower rdf:resource="descriptor/10374-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10361-4">
    <skos:narrower rdf:resource="descriptor/10373-4"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id243">
    <rdf:value xml:lang="de">Für den Mineralölmarkt</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19507-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14516-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11519-4">
    <skos:prefLabel xml:lang="en">Price level</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11519-4">
    <skos:prefLabel xml:lang="de">Preisniveau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18327-6">
    <skos:prefLabel xml:lang="en">Land reclamation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18327-6">
    <skos:prefLabel xml:lang="de">Bodensanierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12020-6">
    <skos:broader rdf:resource="thsys/73360"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12020-6">
    <skos:broader rdf:resource="thsys/71066"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12020-6">
    <skos:broader rdf:resource="descriptor/16019-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13213-4">
    <skos:related rdf:resource="descriptor/13214-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13213-4">
    <skos:related rdf:resource="descriptor/14032-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13213-4">
    <skos:related rdf:resource="descriptor/14779-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13213-4">
    <skos:related rdf:resource="descriptor/13011-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13213-4">
    <skos:related rdf:resource="descriptor/13010-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13213-4">
    <skos:related rdf:resource="descriptor/14766-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13213-4">
    <skos:related rdf:resource="descriptor/14548-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19436-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13161-4">
    <skos:prefLabel xml:lang="en">Processing of fruits and vegetables</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13161-4">
    <skos:prefLabel xml:lang="de">Verarbeitung von Obst und Gemüse</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19788-0">
    <skos:prefLabel xml:lang="en">Pregnancy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19788-0">
    <skos:prefLabel xml:lang="de">Schwangerschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70026">
    <rdfs:label xml:lang="en">America</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70026">
    <rdfs:label xml:lang="de">Amerika</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19797-6">
    <skos:prefLabel xml:lang="en">Nanotechnology</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19797-6">
    <skos:prefLabel xml:lang="de">Nanotechnologie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11803-0">
    <skos:related rdf:resource="descriptor/18313-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11803-0">
    <skos:related rdf:resource="descriptor/12032-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11803-0">
    <skos:related rdf:resource="descriptor/13451-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11803-0">
    <skos:related rdf:resource="descriptor/11804-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11803-0">
    <skos:related rdf:resource="descriptor/11809-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11803-0">
    <skos:related rdf:resource="descriptor/13443-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18841-6">
    <skos:broader rdf:resource="thsys/70854"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18841-6">
    <skos:broader rdf:resource="descriptor/16563-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11633-6">
    <skos:altLabel xml:lang="en">Fiscal monopoly</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11633-6">
    <skos:altLabel xml:lang="de">Zündwarenmonopol</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11633-6">
    <skos:altLabel xml:lang="de">Tabakmonopol</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11633-6">
    <skos:altLabel xml:lang="de">Steuermonopol</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11633-6">
    <skos:altLabel xml:lang="de">Staatsregal</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11633-6">
    <skos:altLabel xml:lang="de">Staatsmonopol</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11633-6">
    <skos:altLabel xml:lang="de">Finanzregal</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11633-6">
    <skos:altLabel xml:lang="de">Finanzmonopol</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11422-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19032-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15901-1">
    <skos:related rdf:resource="descriptor/18231-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15901-1">
    <skos:related rdf:resource="descriptor/19624-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11460-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17717-5">
    <skos:altLabel xml:lang="de">Bundesrepublik Nigeria</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18948-2">
    <skos:scopeNote xml:lang="de">Einkommen privater Haushalte.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10028-0">
    <skos:altLabel xml:lang="de">Nettoproduktionswert</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10028-0">
    <skos:altLabel xml:lang="de">Bruttowertschöpfung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21306-1">
    <skos:altLabel xml:lang="de">Diversitätsmanagement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18579-0">
    <skos:prefLabel xml:lang="en">Road transport policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18579-0">
    <skos:prefLabel xml:lang="de">Straßenverkehrspolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13831-6">
    <skos:prefLabel xml:lang="en">Housing statistics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13831-6">
    <skos:prefLabel xml:lang="de">Wohnungsstatistik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16943-1">
    <skos:prefLabel xml:lang="en">Burgenland</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16943-1">
    <skos:prefLabel xml:lang="de">Burgenland</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17717-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18756-4">
    <skos:altLabel xml:lang="de">Gewerberaummiete</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70179">
    <skos:broader rdf:resource="thsys/v"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11167-2">
    <skos:prefLabel xml:lang="en">Economic theory of democracy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11167-2">
    <skos:prefLabel xml:lang="de">Ökonomische Theorie der Demokratie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20977-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18695-5">
    <skos:related rdf:resource="descriptor/12697-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18695-5">
    <skos:related rdf:resource="descriptor/20842-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18695-5">
    <skos:related rdf:resource="descriptor/10556-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17592-6">
    <skos:altLabel xml:lang="de">Südvietnam</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17592-6">
    <skos:altLabel xml:lang="de">Sozialistische Republik Vietnam</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17592-6">
    <skos:altLabel xml:lang="de">Republik Vietnam</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17592-6">
    <skos:altLabel xml:lang="de">Nordvietnam</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17592-6">
    <skos:altLabel xml:lang="de">Demokratische Republik Vietnam</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16472-0">
    <skos:broader rdf:resource="thsys/73364"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16472-0">
    <skos:broader rdf:resource="descriptor/16288-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19007-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19815-2">
    <skos:scopeNote xml:lang="de">Einschleusen von Vermögenswerten aus schweren Straftaten und organisierter Kriminalität in den legalen Finanz- und Wirtschaftskreislauf.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70951">
    <skos:narrower rdf:resource="descriptor/18700-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70951">
    <skos:narrower rdf:resource="descriptor/18539-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70951">
    <skos:narrower rdf:resource="descriptor/10877-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70951">
    <skos:narrower rdf:resource="descriptor/10876-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70951">
    <skos:narrower rdf:resource="descriptor/18369-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70951">
    <skos:narrower rdf:resource="descriptor/18543-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70951">
    <skos:narrower rdf:resource="descriptor/18542-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70951">
    <skos:narrower rdf:resource="descriptor/18541-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70951">
    <skos:narrower rdf:resource="descriptor/10880-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70951">
    <skos:narrower rdf:resource="descriptor/10870-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70951">
    <skos:narrower rdf:resource="descriptor/10866-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70951">
    <skos:narrower rdf:resource="descriptor/10865-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70951">
    <skos:narrower rdf:resource="descriptor/10693-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70951">
    <skos:narrower rdf:resource="descriptor/18794-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70951">
    <skos:narrower rdf:resource="descriptor/10851-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70951">
    <skos:narrower rdf:resource="descriptor/10858-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70951">
    <skos:narrower rdf:resource="descriptor/10854-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70951">
    <skos:narrower rdf:resource="descriptor/10859-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70951">
    <skos:narrower rdf:resource="descriptor/10850-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18400-1">
    <skos:broader rdf:resource="thsys/71056"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18400-1">
    <skos:broader rdf:resource="thsys/71003"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12325-5">
    <skos:prefLabel xml:lang="en">Bookkeeping</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12325-5">
    <skos:prefLabel xml:lang="de">Buchführung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19565-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11850-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18216-1">
    <skos:altLabel xml:lang="de">Nahrungsmittelersatzstoff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18216-1">
    <skos:altLabel xml:lang="de">Lebensmittelersatzstoff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18216-1">
    <skos:altLabel xml:lang="de">Imitationsprodukt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18216-1">
    <skos:altLabel xml:lang="de">Imitat für Nahrungsmittel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18216-1">
    <skos:altLabel xml:lang="de">Ersatzstoff für Nahrungsmittel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17746-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14990-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10248-0">
    <skos:broader rdf:resource="thsys/71111"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10248-0">
    <skos:broader rdf:resource="thsys/70234"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10248-0">
    <skos:broader rdf:resource="descriptor/10249-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10589-2">
    <skos:scopeNote xml:lang="de">Entwicklungshilfe von Nichtregierungsorganisationen, z.B. von Kirchen, Stiftungen, usw.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16003-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11607-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15069-3">
    <skos:scopeNote xml:lang="de">Begriff der amtlichen Statistik: effektiver Aufwand, den private Haushalte im Rahmen ihres Einkommens bei ordnungsgemäßer Wirtschaftsführung aufbringen müssen; statistisch nicht direkt messbar.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18478-6">
    <skos:related rdf:resource="descriptor/11821-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18478-6">
    <skos:related rdf:resource="descriptor/10500-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18478-6">
    <skos:related rdf:resource="descriptor/18060-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14784-5">
    <skos:broader rdf:resource="thsys/70416"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14784-5">
    <skos:broader rdf:resource="descriptor/14788-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14784-5">
    <skos:broader rdf:resource="descriptor/14021-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15605-5">
    <skos:altLabel xml:lang="de">Verschlüsselung von Daten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15605-5">
    <skos:altLabel xml:lang="de">Sicherheit in der Informationstechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15605-5">
    <skos:altLabel xml:lang="de">Kryptoverfahren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15605-5">
    <skos:altLabel xml:lang="de">Kryptographie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15605-5">
    <skos:altLabel xml:lang="de">IT-Sicherheit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15605-5">
    <skos:altLabel xml:lang="de">Hacker</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15605-5">
    <skos:altLabel xml:lang="de">Elektronische Unterschrift</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15605-5">
    <skos:altLabel xml:lang="de">Elektronische Signatur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15605-5">
    <skos:altLabel xml:lang="de">Digitale Signatur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15605-5">
    <skos:altLabel xml:lang="de">Datenverschlüsselung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15605-5">
    <skos:altLabel xml:lang="de">Datensicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18979-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16566-5">
    <skos:related rdf:resource="descriptor/10032-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16566-5">
    <skos:related rdf:resource="descriptor/16687-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16566-5">
    <skos:related rdf:resource="descriptor/15836-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16566-5">
    <skos:related rdf:resource="descriptor/15706-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16566-5">
    <skos:related rdf:resource="descriptor/15861-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16566-5">
    <skos:related rdf:resource="descriptor/19643-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16566-5">
    <skos:related rdf:resource="descriptor/11064-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16566-5">
    <skos:related rdf:resource="descriptor/10511-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16566-5">
    <skos:related rdf:resource="descriptor/16562-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16566-5">
    <skos:related rdf:resource="descriptor/20497-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#scopeNote">
    <rdfs:label xml:lang="en">scope note</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11211-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id97">
    <rdf:value xml:lang="de">Für das Leasing von Kraftfahrzeugen</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10976-2">
    <skos:broader rdf:resource="thsys/73372"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10976-2">
    <skos:broader rdf:resource="thsys/70039"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10976-2">
    <skos:broader rdf:resource="thsys/70038"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10976-2">
    <skos:broader rdf:resource="descriptor/10987-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13615-5">
    <skos:altLabel xml:lang="de">Containertransport</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13615-5">
    <skos:altLabel xml:lang="de">Behälterverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70265">
    <skos:narrower rdf:resource="descriptor/13670-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70265">
    <skos:narrower rdf:resource="descriptor/13669-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70265">
    <skos:narrower rdf:resource="descriptor/18212-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70265">
    <skos:narrower rdf:resource="descriptor/18028-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70265">
    <skos:narrower rdf:resource="descriptor/14816-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70265">
    <skos:narrower rdf:resource="descriptor/18211-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70265">
    <skos:narrower rdf:resource="descriptor/14614-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70265">
    <skos:narrower rdf:resource="descriptor/14814-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70265">
    <skos:narrower rdf:resource="descriptor/18210-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70265">
    <skos:narrower rdf:resource="descriptor/14046-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70265">
    <skos:narrower rdf:resource="descriptor/18209-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70265">
    <skos:narrower rdf:resource="descriptor/14812-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70265">
    <skos:narrower rdf:resource="descriptor/18208-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70265">
    <skos:narrower rdf:resource="descriptor/13650-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70265">
    <skos:narrower rdf:resource="descriptor/18207-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70265">
    <skos:narrower rdf:resource="descriptor/18206-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10912-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12049-3">
    <skos:related rdf:resource="descriptor/10911-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12049-3">
    <skos:related rdf:resource="descriptor/12038-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10803-5">
    <skos:altLabel xml:lang="en">Currency hedging</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10803-5">
    <skos:altLabel xml:lang="de">Preissicherungsgeschäft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10803-5">
    <skos:altLabel xml:lang="de">Finanz-Hedging</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16629-0">
    <skos:prefLabel xml:lang="en">Trade union movement</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16629-0">
    <skos:prefLabel xml:lang="de">Gewerkschaftsbewegung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14607-6">
    <skos:prefLabel xml:lang="en">Adhesive</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14607-6">
    <skos:prefLabel xml:lang="de">Klebstoff und Dichtstoff</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11695-5">
    <skos:related rdf:resource="descriptor/19411-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11695-5">
    <skos:related rdf:resource="descriptor/11692-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10856-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70212">
    <skos:prefLabel xml:lang="de">V.10.01  Wirtschaftskonzentration und Wettbewerbsbeschränkung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10649-3">
    <skos:narrower rdf:resource="descriptor/10644-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10649-3">
    <skos:narrower rdf:resource="descriptor/10564-4"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id23">
    <rdfs:seeAlso rdf:resource="descriptor/10613-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12617-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18964-4">
    <skos:related rdf:resource="descriptor/15011-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18964-4">
    <skos:related rdf:resource="descriptor/13324-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18964-4">
    <skos:related rdf:resource="descriptor/24621-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13336-2">
    <skos:prefLabel xml:lang="en">Disposal site</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13336-2">
    <skos:prefLabel xml:lang="de">Mülldeponie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18181-1">
    <skos:scopeNote xml:lang="de">Regionalisierung der Außenhandelsbeziehungen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73341">
    <skos:broader rdf:resource="thsys/73342"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10693-0">
    <skos:altLabel xml:lang="de">Hermes-Deckung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10693-0">
    <skos:altLabel xml:lang="de">Exportrisikogarantie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10693-0">
    <skos:altLabel xml:lang="de">Exportbürgschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10693-0">
    <skos:altLabel xml:lang="de">Ausfuhrgewährleistung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10693-0">
    <skos:altLabel xml:lang="de">Ausfuhrdeckung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10693-0">
    <skos:altLabel xml:lang="de">Ausfuhrbürgschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12987-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19524-6">
    <skos:related rdf:resource="descriptor/24857-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19524-6">
    <skos:related rdf:resource="descriptor/10201-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19524-6">
    <skos:related rdf:resource="descriptor/19199-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12780-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19395-5">
    <skos:prefLabel xml:lang="en">Food inspection</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19395-5">
    <skos:prefLabel xml:lang="de">Lebensmittelüberwachung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19134-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19253-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17461-0">
    <skos:related rdf:resource="descriptor/19837-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17461-0">
    <skos:related rdf:resource="descriptor/11123-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17461-0">
    <skos:related rdf:resource="descriptor/17534-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17461-0">
    <skos:related rdf:resource="descriptor/19077-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17461-0">
    <skos:related rdf:resource="descriptor/19078-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13365-2">
    <skos:related rdf:resource="descriptor/15657-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13365-2">
    <skos:related rdf:resource="descriptor/13197-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13365-2">
    <skos:related rdf:resource="descriptor/13650-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19528-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18064-5">
    <skos:scopeNote xml:lang="de">Nur für zwischenstaatliche Sanktionen. Umfasst ökonomische, diplomatische und andere Maßnahmen gegenüber Staaten, die gegen internationales Recht verstoßen haben.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12308-5">
    <skos:altLabel xml:lang="de">Betriebserfolg</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13562-0">
    <skos:prefLabel xml:lang="en">Roads policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13562-0">
    <skos:prefLabel xml:lang="de">Straßenbaupolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18437-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19731-1">
    <skos:narrower rdf:resource="descriptor/16722-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13177-3">
    <skos:prefLabel xml:lang="en">Civil engineering</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13177-3">
    <skos:prefLabel xml:lang="de">Tiefbau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16724-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12780-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15373-0">
    <skos:broader rdf:resource="thsys/72393"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15373-0">
    <skos:broader rdf:resource="descriptor/15064-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15373-0">
    <skos:broader rdf:resource="descriptor/19207-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10661-6">
    <skos:altLabel xml:lang="en">Non-traded good</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10661-6">
    <skos:altLabel xml:lang="de">Lokales Gut</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13499-2">
    <skos:narrower rdf:resource="descriptor/13618-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71003">
    <skos:prefLabel xml:lang="en">W.06.01.01  Metal industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71003">
    <skos:prefLabel xml:lang="de">W.06.01.01  Metallindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19643-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12111-3">
    <skos:altLabel xml:lang="de">Spartenorganisation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12111-3">
    <skos:altLabel xml:lang="de">Organisationsform</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12111-3">
    <skos:altLabel xml:lang="de">Geschäftsbereichsorganisation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12111-3">
    <skos:altLabel xml:lang="de">Funktionsbereichsorganisation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12111-3">
    <skos:altLabel xml:lang="de">Funktionalorganisation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12111-3">
    <skos:altLabel xml:lang="de">Divisionsmanagement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12111-3">
    <skos:altLabel xml:lang="de">Divisionalorganisation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19339-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11833-5">
    <skos:narrower rdf:resource="descriptor/18629-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11833-5">
    <skos:narrower rdf:resource="descriptor/11828-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11833-5">
    <skos:narrower rdf:resource="descriptor/18628-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11833-5">
    <skos:narrower rdf:resource="descriptor/18579-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11833-5">
    <skos:narrower rdf:resource="descriptor/18633-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11833-5">
    <skos:narrower rdf:resource="descriptor/13545-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11833-5">
    <skos:narrower rdf:resource="descriptor/18632-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11833-5">
    <skos:narrower rdf:resource="descriptor/18610-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11833-5">
    <skos:narrower rdf:resource="descriptor/13519-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11833-5">
    <skos:narrower rdf:resource="descriptor/18616-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11833-5">
    <skos:narrower rdf:resource="descriptor/18627-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11833-5">
    <skos:narrower rdf:resource="descriptor/18575-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18562-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19354-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16975-2">
    <skos:broader rdf:resource="thsys/70003"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16975-2">
    <skos:broader rdf:resource="descriptor/16972-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70039">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11414-4">
    <skos:related rdf:resource="descriptor/13828-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11414-4">
    <skos:related rdf:resource="descriptor/12283-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11414-4">
    <skos:related rdf:resource="descriptor/12281-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11414-4">
    <skos:related rdf:resource="descriptor/12287-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11414-4">
    <skos:related rdf:resource="descriptor/18671-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11414-4">
    <skos:related rdf:resource="descriptor/13723-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11414-4">
    <skos:related rdf:resource="descriptor/18085-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11414-4">
    <skos:related rdf:resource="descriptor/13283-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10696-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10806-6">
    <skos:prefLabel xml:lang="en">Multinational bank</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10806-6">
    <skos:prefLabel xml:lang="de">Internationale Bank</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15766-2">
    <skos:broader rdf:resource="thsys/73334"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15766-2">
    <skos:broader rdf:resource="descriptor/16195-0"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id268">
    <rdf:value xml:lang="de">Für Produktionsgenossenschaft</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11610-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11059-5">
    <skos:broader rdf:resource="thsys/71102"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11059-5">
    <skos:broader rdf:resource="descriptor/11055-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13636-4">
    <skos:related rdf:resource="descriptor/13308-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71001">
    <skos:prefLabel xml:lang="en">W.06.01.01.01  Steel industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71001">
    <skos:prefLabel xml:lang="de">W.06.01.01.01  Stahlindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71050">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11345-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10557-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19164-3">
    <skos:broader rdf:resource="thsys/70140"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19164-3">
    <skos:broader rdf:resource="descriptor/19565-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19164-3">
    <skos:broader rdf:resource="descriptor/12036-5"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id329">
    <rdfs:seeAlso rdf:resource="descriptor/15599-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19704-4">
    <skos:related rdf:resource="descriptor/15979-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19704-4">
    <skos:related rdf:resource="descriptor/16198-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19704-4">
    <skos:related rdf:resource="descriptor/16196-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19704-4">
    <skos:related rdf:resource="descriptor/16195-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15914-6">
    <skos:altLabel xml:lang="de">Bevölkerungsaufbau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13528-0">
    <skos:broader rdf:resource="thsys/70935"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13528-0">
    <skos:broader rdf:resource="thsys/70933"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13528-0">
    <skos:broader rdf:resource="descriptor/13533-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19780-2">
    <skos:altLabel xml:lang="en">Sport fishery</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19780-2">
    <skos:altLabel xml:lang="de">Sportfischerei</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19780-2">
    <skos:altLabel xml:lang="de">Sportfischen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19780-2">
    <skos:altLabel xml:lang="de">Sportangeln</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19780-2">
    <skos:altLabel xml:lang="de">Freizeitangeln</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19780-2">
    <skos:altLabel xml:lang="de">Angelsport</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70301">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11817-3">
    <skos:related rdf:resource="descriptor/19829-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11817-3">
    <skos:related rdf:resource="descriptor/10973-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11817-3">
    <skos:related rdf:resource="descriptor/19446-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11817-3">
    <skos:related rdf:resource="descriptor/16052-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11817-3">
    <skos:related rdf:resource="descriptor/11981-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11817-3">
    <skos:related rdf:resource="descriptor/11814-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19338-3">
    <skos:narrower rdf:resource="descriptor/19296-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19338-3">
    <skos:narrower rdf:resource="descriptor/19294-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19338-3">
    <skos:narrower rdf:resource="descriptor/19292-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16435-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11792-0">
    <skos:narrower rdf:resource="descriptor/18730-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11792-0">
    <skos:narrower rdf:resource="descriptor/13688-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19316-6">
    <skos:scopeNote xml:lang="de">Kennzeichnung und Bescheinigung der Herkunft von Waren im Außenhandel.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14022-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14599-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15842-5">
    <skos:prefLabel xml:lang="en">Intra-company conflict</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15842-5">
    <skos:prefLabel xml:lang="de">Betrieblicher Konflikt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id298">
    <rdfs:seeAlso rdf:resource="descriptor/16615-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14539-4">
    <skos:narrower rdf:resource="descriptor/14534-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14539-4">
    <skos:narrower rdf:resource="descriptor/14538-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19479-6">
    <skos:broader rdf:resource="thsys/71064"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19479-6">
    <skos:broader rdf:resource="descriptor/11859-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17677-5">
    <skos:prefLabel xml:lang="en">Sudan</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17677-5">
    <skos:prefLabel xml:lang="de">Sudan</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18191-5">
    <skos:altLabel xml:lang="de">Internationales Kaffeeübereinkommen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18191-5">
    <skos:altLabel xml:lang="de">Internationales Kaffeeabkommen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70394">
    <rdfs:label xml:lang="en">Logistics</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70394">
    <rdfs:label xml:lang="de">Logistik</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19989-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25881-3">
    <skos:altLabel xml:lang="en">Work attitude</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25881-3">
    <skos:altLabel xml:lang="en">Organizational citizenship behaviour</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25881-3">
    <skos:altLabel xml:lang="en">Organisational citizenship behavior</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25881-3">
    <skos:altLabel xml:lang="en">OCB (Organizational citizenship behaviour)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25881-3">
    <skos:altLabel xml:lang="de">Arbeitseinstellung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19352-2">
    <skos:related rdf:resource="descriptor/18847-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19352-2">
    <skos:related rdf:resource="descriptor/20810-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19352-2">
    <skos:related rdf:resource="descriptor/19336-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19352-2">
    <skos:related rdf:resource="descriptor/19663-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19352-2">
    <skos:related rdf:resource="descriptor/19350-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12966-2">
    <skos:prefLabel xml:lang="en">Deep sea fishery</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12966-2">
    <skos:prefLabel xml:lang="de">Hochseefischerei</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18622-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18352-0">
    <skos:altLabel xml:lang="de">Explorationsrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18352-0">
    <skos:altLabel xml:lang="de">Bundesberggesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18352-0">
    <skos:altLabel xml:lang="de">Berggesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18352-0">
    <skos:altLabel xml:lang="de">Bergbaurecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18352-0">
    <skos:altLabel xml:lang="de">Bergbaukonzession</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18352-0">
    <skos:altLabel xml:lang="de">BBergG</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70858">
    <skos:prefLabel xml:lang="de">W.22.02.02  Finanztheorie und Public Choice</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19154-6">
    <skos:altLabel xml:lang="en">Corporate control</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19154-6">
    <skos:altLabel xml:lang="de">Unternehmensüberwachung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19154-6">
    <skos:altLabel xml:lang="de">Kontrolle des Managements</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19154-6">
    <skos:altLabel xml:lang="de">Externe Unternehmenskontrolle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19154-6">
    <skos:altLabel xml:lang="de">Eigentümerkontrolle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13860-6">
    <skos:related rdf:resource="descriptor/11847-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13860-6">
    <skos:related rdf:resource="descriptor/13829-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13860-6">
    <skos:related rdf:resource="descriptor/11986-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13860-6">
    <skos:related rdf:resource="descriptor/18508-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13860-6">
    <skos:related rdf:resource="descriptor/18493-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13860-6">
    <skos:related rdf:resource="descriptor/18491-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13860-6">
    <skos:related rdf:resource="descriptor/18492-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70819">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19196-4">
    <skos:broader rdf:resource="thsys/70897"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19196-4">
    <skos:broader rdf:resource="thsys/70187"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19196-4">
    <skos:broader rdf:resource="thsys/70049"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19196-4">
    <skos:broader rdf:resource="descriptor/10021-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19196-4">
    <skos:broader rdf:resource="descriptor/12210-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19656-3">
    <skos:prefLabel xml:lang="en">Expatriate experience</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19656-3">
    <skos:prefLabel xml:lang="de">Auslandsaufenthalt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11438-4">
    <skos:broader rdf:resource="thsys/70055"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11438-4">
    <skos:broader rdf:resource="descriptor/19255-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18537-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21094-4">
    <skos:altLabel xml:lang="de">Vertragslandwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14778-0">
    <skos:broader rdf:resource="thsys/73306"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14778-0">
    <skos:broader rdf:resource="thsys/70333"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14778-0">
    <skos:broader rdf:resource="descriptor/14051-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14778-0">
    <skos:broader rdf:resource="descriptor/14776-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10568-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13940-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71111">
    <rdfs:label xml:lang="en">National accounts</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71111">
    <rdfs:label xml:lang="de">Volkswirtschaftliche Gesamtrechnung</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14079-5">
    <skos:broader rdf:resource="thsys/70341"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14079-5">
    <skos:broader rdf:resource="descriptor/14074-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13293-1">
    <skos:broader rdf:resource="thsys/71045"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13293-1">
    <skos:broader rdf:resource="thsys/70891"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13293-1">
    <skos:broader rdf:resource="descriptor/13297-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14346-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15643-4">
    <skos:altLabel xml:lang="de">Soccer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12535-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16460-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13636-4">
    <skos:prefLabel xml:lang="en">Camping and youth hostel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13636-4">
    <skos:prefLabel xml:lang="de">Camping und Jugendherberge</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19058-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13096-3">
    <skos:altLabel xml:lang="de">Spielzeugindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13096-3">
    <skos:altLabel xml:lang="de">Christbaumschmuckindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id256">
    <rdf:value xml:lang="de">Für den Brief- und Paketverkehr</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19160-4">
    <skos:broader rdf:resource="thsys/71033"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19160-4">
    <skos:broader rdf:resource="descriptor/18165-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12788-0">
    <skos:narrower rdf:resource="descriptor/19126-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12788-0">
    <skos:narrower rdf:resource="descriptor/24706-5"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id19">
    <rdfs:seeAlso rdf:resource="descriptor/12574-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18003-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10229-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11459-3">
    <skos:altLabel xml:lang="en">Theory of credit rationing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11459-3">
    <skos:altLabel xml:lang="en">Nonprice credit rationing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11459-3">
    <skos:altLabel xml:lang="de">Theorie der Kreditrationierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12225-2">
    <skos:broader rdf:resource="thsys/70183"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18279-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11519-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16264-0">
    <skos:prefLabel xml:lang="en">Self-determination of peoples</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16264-0">
    <skos:prefLabel xml:lang="de">Nationale Selbstbestimmung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70536">
    <rdfs:label xml:lang="de">Waschmittel und Körperpflegemittel</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12780-2">
    <skos:altLabel xml:lang="de">Persönlicher Verkauf</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12780-2">
    <skos:altLabel xml:lang="de">Direktvertrieb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12780-2">
    <skos:altLabel xml:lang="de">Direktabsatz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12780-2">
    <skos:altLabel xml:lang="de">Dialogmarketing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14464-2">
    <skos:prefLabel xml:lang="en">Battery</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14464-2">
    <skos:prefLabel xml:lang="de">Batterie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19289-4">
    <skos:related rdf:resource="descriptor/10018-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16455-0">
    <skos:broader rdf:resource="thsys/73363"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16455-0">
    <skos:broader rdf:resource="thsys/70865"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16455-0">
    <skos:broader rdf:resource="descriptor/13412-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18081-5">
    <skos:altLabel xml:lang="en">Cellular telephone</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18081-5">
    <skos:altLabel xml:lang="de">Tragbares Telefon</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18081-5">
    <skos:altLabel xml:lang="de">Schnurloses Telefon</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18081-5">
    <skos:altLabel xml:lang="de">Ortsungebundenes Telefon</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18081-5">
    <skos:altLabel xml:lang="de">Handy-Telefon</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18081-5">
    <skos:altLabel xml:lang="de">Funktelefon</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18081-5">
    <skos:altLabel xml:lang="de">Drahtloses Telefon</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18081-5">
    <skos:altLabel xml:lang="de">Autotelefon</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13557-0">
    <skos:prefLabel xml:lang="en">Walking</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13557-0">
    <skos:prefLabel xml:lang="de">Fußgängerverkehr</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71094">
    <rdfs:label xml:lang="en">International monetary system</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71094">
    <rdfs:label xml:lang="de">Internationales Währungssystem</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14070-2">
    <skos:broader rdf:resource="thsys/70341"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14070-2">
    <skos:broader rdf:resource="descriptor/14073-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14026-5">
    <skos:related rdf:resource="descriptor/14741-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14026-5">
    <skos:related rdf:resource="descriptor/14738-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14026-5">
    <skos:related rdf:resource="descriptor/14728-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14026-5">
    <skos:related rdf:resource="descriptor/14734-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14026-5">
    <skos:related rdf:resource="descriptor/13199-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12302-3">
    <skos:prefLabel xml:lang="en">Insolvency</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12302-3">
    <skos:prefLabel xml:lang="de">Zahlungsunfähigkeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17954-5">
    <skos:narrower rdf:resource="descriptor/17972-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17954-5">
    <skos:narrower rdf:resource="descriptor/17967-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17954-5">
    <skos:narrower rdf:resource="descriptor/17964-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17954-5">
    <skos:narrower rdf:resource="descriptor/17962-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12218-6">
    <skos:broader rdf:resource="thsys/71042"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12218-6">
    <skos:broader rdf:resource="thsys/70037"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12218-6">
    <skos:broader rdf:resource="descriptor/12053-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13350-1">
    <skos:narrower rdf:resource="descriptor/13361-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13350-1">
    <skos:narrower rdf:resource="descriptor/18873-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12925-2">
    <skos:broader rdf:resource="thsys/70982"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12925-2">
    <skos:broader rdf:resource="descriptor/12993-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11331-1">
    <skos:prefLabel xml:lang="en">Union wage rate</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11331-1">
    <skos:prefLabel xml:lang="de">Tariflohn</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19233-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18132-0">
    <skos:broader rdf:resource="thsys/70361"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18132-0">
    <skos:broader rdf:resource="descriptor/18012-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71106">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18817-3">
    <skos:prefLabel xml:lang="en">Defence economics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18817-3">
    <skos:prefLabel xml:lang="de">Verteidigungsökonomik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12961-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16771-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11783-1">
    <skos:prefLabel xml:lang="en">Size distribution of firms</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11783-1">
    <skos:prefLabel xml:lang="de">Betriebsgrößenstruktur</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12947-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16204-4">
    <skos:related rdf:resource="descriptor/16288-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16204-4">
    <skos:related rdf:resource="descriptor/19725-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16204-4">
    <skos:related rdf:resource="descriptor/16684-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16204-4">
    <skos:related rdf:resource="descriptor/16470-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16204-4">
    <skos:related rdf:resource="descriptor/11163-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16204-4">
    <skos:related rdf:resource="descriptor/16497-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16204-4">
    <skos:related rdf:resource="descriptor/16391-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16204-4">
    <skos:related rdf:resource="descriptor/16427-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16204-4">
    <skos:related rdf:resource="descriptor/16445-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11721-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10564-4">
    <skos:altLabel xml:lang="de">Erziehungszollargument</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71088">
    <skos:narrower rdf:resource="descriptor/18701-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71088">
    <skos:narrower rdf:resource="descriptor/10821-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71088">
    <skos:narrower rdf:resource="descriptor/10773-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71088">
    <skos:narrower rdf:resource="descriptor/18699-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71088">
    <skos:narrower rdf:resource="descriptor/10770-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71088">
    <skos:narrower rdf:resource="descriptor/19333-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71088">
    <skos:narrower rdf:resource="descriptor/13766-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71088">
    <skos:narrower rdf:resource="descriptor/13720-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71088">
    <skos:narrower rdf:resource="descriptor/10781-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71088">
    <skos:narrower rdf:resource="descriptor/10780-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71088">
    <skos:narrower rdf:resource="descriptor/18688-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71088">
    <skos:narrower rdf:resource="descriptor/10779-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71088">
    <skos:narrower rdf:resource="descriptor/10719-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71088">
    <skos:narrower rdf:resource="descriptor/13757-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71088">
    <skos:narrower rdf:resource="descriptor/10776-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70943">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.12</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19639-3">
    <skos:broader rdf:resource="thsys/70572"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19639-3">
    <skos:broader rdf:resource="descriptor/15909-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18458-5">
    <skos:scopeNote xml:lang="de">Regelungen zwischen Industrie und Energiewirtschaft bzw. industrielle Eigenversorgung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19448-3">
    <skos:narrower rdf:resource="descriptor/18837-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13865-3">
    <skos:prefLabel xml:lang="en">Economics of housing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13865-3">
    <skos:prefLabel xml:lang="de">Wohnungswirtschaftslehre</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19317-4">
    <skos:broader rdf:resource="thsys/71098"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19317-4">
    <skos:broader rdf:resource="descriptor/18011-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19317-4">
    <skos:broader rdf:resource="descriptor/10686-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15025-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70087">
    <skos:broader rdf:resource="thsys/70028"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10960-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13377-2">
    <skos:related rdf:resource="descriptor/12253-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18630-5">
    <skos:prefLabel xml:lang="en">Policy for traffic routes</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18630-5">
    <skos:prefLabel xml:lang="de">Verkehrswegepolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18987-6">
    <skos:altLabel xml:lang="en">Wood-cutting</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18987-6">
    <skos:altLabel xml:lang="en">Wood cutting</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18987-6">
    <skos:altLabel xml:lang="en">Timber harvest</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18987-6">
    <skos:altLabel xml:lang="en">Forest harvest</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18987-6">
    <skos:altLabel xml:lang="de">Holzfällen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18987-6">
    <skos:altLabel xml:lang="de">Holzernte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11847-1">
    <skos:related rdf:resource="descriptor/13860-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11847-1">
    <skos:related rdf:resource="descriptor/13305-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11847-1">
    <skos:related rdf:resource="descriptor/11848-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11847-1">
    <skos:related rdf:resource="descriptor/13829-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11847-1">
    <skos:related rdf:resource="descriptor/10981-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19792-2">
    <skos:broader rdf:resource="thsys/73346"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19792-2">
    <skos:broader rdf:resource="descriptor/18444-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#Concept">
    <skos:definition xml:lang="en">An abstract idea or notion; a unit of thought.</skos:definition>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70242">
    <skos:narrower rdf:resource="descriptor/10956-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70242">
    <skos:narrower rdf:resource="descriptor/10227-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70242">
    <skos:narrower rdf:resource="descriptor/10951-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70242">
    <skos:narrower rdf:resource="descriptor/10224-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70242">
    <skos:narrower rdf:resource="descriptor/10223-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70242">
    <skos:narrower rdf:resource="descriptor/18828-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70242">
    <skos:narrower rdf:resource="descriptor/19432-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70242">
    <skos:narrower rdf:resource="descriptor/10950-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70242">
    <skos:narrower rdf:resource="descriptor/10949-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70242">
    <skos:narrower rdf:resource="descriptor/10948-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70242">
    <skos:narrower rdf:resource="descriptor/19431-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70242">
    <skos:narrower rdf:resource="descriptor/19226-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70242">
    <skos:narrower rdf:resource="descriptor/10945-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70242">
    <skos:narrower rdf:resource="descriptor/10947-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70242">
    <skos:narrower rdf:resource="descriptor/10944-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70242">
    <skos:narrower rdf:resource="descriptor/19430-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70242">
    <skos:narrower rdf:resource="descriptor/10889-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70988">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.06.01.10</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16524-0">
    <skos:related rdf:resource="descriptor/10598-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16524-0">
    <skos:related rdf:resource="descriptor/16295-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16524-0">
    <skos:related rdf:resource="descriptor/15889-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16524-0">
    <skos:related rdf:resource="descriptor/10583-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16524-0">
    <skos:related rdf:resource="descriptor/11566-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16524-0">
    <skos:related rdf:resource="descriptor/18912-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16524-0">
    <skos:related rdf:resource="descriptor/19722-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16524-0">
    <skos:related rdf:resource="descriptor/10603-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10988-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10718-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18734-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18255-5">
    <skos:altLabel xml:lang="de">Tonerde</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19798-4">
    <skos:prefLabel xml:lang="en">Microsystems</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19798-4">
    <skos:prefLabel xml:lang="de">Mikrosystemtechnik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13158-0">
    <zbwext:useInsteadNote rdf:nodeID="id218"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72180">
    <skos:narrower rdf:resource="thsys/72182"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72180">
    <skos:narrower rdf:resource="thsys/72183"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72180">
    <skos:narrower rdf:resource="thsys/72185"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72180">
    <skos:narrower rdf:resource="thsys/72186"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72180">
    <skos:narrower rdf:resource="thsys/72394"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72180">
    <skos:narrower rdf:resource="thsys/72181"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72180">
    <skos:narrower rdf:resource="thsys/72393"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15637-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10827-5">
    <skos:broader rdf:resource="thsys/73337"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10827-5">
    <skos:broader rdf:resource="thsys/71098"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10827-5">
    <skos:broader rdf:resource="thsys/71073"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10827-5">
    <skos:broader rdf:resource="descriptor/11640-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14422-4">
    <skos:altLabel xml:lang="de">Militärhubschrauber</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14422-4">
    <skos:altLabel xml:lang="de">Militärflugzeug</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14422-4">
    <skos:altLabel xml:lang="de">Kampfflugzeug</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14422-4">
    <skos:altLabel xml:lang="de">Aufklärungsflugzeug</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71052">
    <skos:narrower rdf:resource="descriptor/13375-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71052">
    <skos:narrower rdf:resource="descriptor/13349-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71052">
    <skos:narrower rdf:resource="descriptor/14017-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71052">
    <skos:narrower rdf:resource="descriptor/13964-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71052">
    <skos:narrower rdf:resource="descriptor/18978-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71052">
    <skos:narrower rdf:resource="descriptor/13992-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71052">
    <skos:narrower rdf:resource="descriptor/14015-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71052">
    <skos:narrower rdf:resource="descriptor/13348-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71052">
    <skos:narrower rdf:resource="descriptor/14011-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71052">
    <skos:narrower rdf:resource="descriptor/12867-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71052">
    <skos:narrower rdf:resource="descriptor/12081-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71052">
    <skos:narrower rdf:resource="descriptor/16252-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71052">
    <skos:narrower rdf:resource="descriptor/11374-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71052">
    <skos:narrower rdf:resource="descriptor/13960-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71052">
    <skos:narrower rdf:resource="descriptor/18764-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71052">
    <skos:narrower rdf:resource="descriptor/18488-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71052">
    <skos:narrower rdf:resource="descriptor/11372-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71052">
    <skos:narrower rdf:resource="descriptor/19043-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71052">
    <skos:narrower rdf:resource="descriptor/14012-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71052">
    <skos:narrower rdf:resource="descriptor/11302-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71052">
    <skos:narrower rdf:resource="descriptor/13959-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71052">
    <skos:narrower rdf:resource="descriptor/18709-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71052">
    <skos:narrower rdf:resource="descriptor/13370-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71052">
    <skos:narrower rdf:resource="descriptor/14005-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71052">
    <skos:narrower rdf:resource="descriptor/11356-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11666-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id222">
    <rdfs:seeAlso rdf:resource="descriptor/15839-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13998-5">
    <skos:broader rdf:resource="thsys/73363"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13998-5">
    <skos:broader rdf:resource="thsys/71055"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13998-5">
    <skos:broader rdf:resource="thsys/70865"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13998-5">
    <skos:broader rdf:resource="descriptor/18535-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21283-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70239">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12252-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16203-6">
    <skos:prefLabel xml:lang="en">Usury</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16203-6">
    <skos:prefLabel xml:lang="de">Wucher</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11294-2">
    <skos:broader rdf:resource="thsys/73355"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11294-2">
    <skos:broader rdf:resource="thsys/71058"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11294-2">
    <skos:broader rdf:resource="descriptor/16664-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10144-5">
    <skos:related rdf:resource="descriptor/10145-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18783-1">
    <skos:narrower rdf:resource="descriptor/18401-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18673-1">
    <skos:related rdf:resource="descriptor/20553-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18673-1">
    <skos:related rdf:resource="descriptor/13702-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18673-1">
    <skos:related rdf:resource="descriptor/19273-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18673-1">
    <skos:related rdf:resource="descriptor/18323-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18673-1">
    <skos:related rdf:resource="descriptor/13287-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18673-1">
    <skos:related rdf:resource="descriptor/18073-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18379-1">
    <skos:altLabel xml:lang="de">Wasserpfennig</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18379-1">
    <skos:altLabel xml:lang="de">Wasserentnahmeabgabe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70142">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14931-4">
    <skos:related rdf:resource="descriptor/14073-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14931-4">
    <skos:related rdf:resource="descriptor/18300-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14931-4">
    <skos:related rdf:resource="descriptor/13428-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14931-4">
    <skos:related rdf:resource="descriptor/11567-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14931-4">
    <skos:related rdf:resource="descriptor/14384-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14931-4">
    <skos:related rdf:resource="descriptor/19338-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14931-4">
    <skos:related rdf:resource="descriptor/19394-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14931-4">
    <skos:related rdf:resource="descriptor/10583-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14931-4">
    <skos:related rdf:resource="descriptor/13160-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14931-4">
    <skos:related rdf:resource="descriptor/19792-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14931-4">
    <skos:related rdf:resource="descriptor/14161-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14931-4">
    <skos:related rdf:resource="descriptor/18304-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14931-4">
    <skos:related rdf:resource="descriptor/18443-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14931-4">
    <skos:related rdf:resource="descriptor/16004-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14931-4">
    <skos:related rdf:resource="descriptor/13219-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14931-4">
    <skos:related rdf:resource="descriptor/18442-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14931-4">
    <skos:related rdf:resource="descriptor/14611-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14931-4">
    <skos:related rdf:resource="descriptor/18222-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14931-4">
    <skos:related rdf:resource="descriptor/13306-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14931-4">
    <skos:related rdf:resource="descriptor/10495-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14931-4">
    <skos:related rdf:resource="descriptor/14054-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25822-5">
    <skos:prefLabel xml:lang="en">Corporate social responsibility</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25822-5">
    <skos:prefLabel xml:lang="de">Corporate social responsibility</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70265">
    <skos:broader rdf:resource="thsys/p"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14602-2">
    <skos:prefLabel xml:lang="en">Specialty chemicals</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14602-2">
    <skos:prefLabel xml:lang="de">Spezialchemikalie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70142">
    <skos:narrower rdf:resource="descriptor/19643-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70142">
    <skos:narrower rdf:resource="descriptor/19478-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70142">
    <skos:narrower rdf:resource="descriptor/11971-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70142">
    <skos:narrower rdf:resource="descriptor/15929-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70142">
    <skos:narrower rdf:resource="descriptor/15935-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70142">
    <skos:narrower rdf:resource="descriptor/19637-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70142">
    <skos:narrower rdf:resource="descriptor/15928-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70142">
    <skos:narrower rdf:resource="descriptor/16661-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70142">
    <skos:narrower rdf:resource="descriptor/15927-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70142">
    <skos:narrower rdf:resource="descriptor/18862-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70142">
    <skos:narrower rdf:resource="descriptor/15926-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70142">
    <skos:narrower rdf:resource="descriptor/13454-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70142">
    <skos:narrower rdf:resource="descriptor/19721-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70142">
    <skos:narrower rdf:resource="descriptor/15925-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70142">
    <skos:narrower rdf:resource="descriptor/15924-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70142">
    <skos:narrower rdf:resource="descriptor/15923-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70142">
    <skos:narrower rdf:resource="descriptor/15917-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70142">
    <skos:narrower rdf:resource="descriptor/15922-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70142">
    <skos:narrower rdf:resource="descriptor/15945-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70142">
    <skos:narrower rdf:resource="descriptor/15920-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70142">
    <skos:narrower rdf:resource="descriptor/19642-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70142">
    <skos:narrower rdf:resource="descriptor/15938-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70142">
    <skos:narrower rdf:resource="descriptor/15918-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70142">
    <skos:narrower rdf:resource="descriptor/18861-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70142">
    <skos:narrower rdf:resource="descriptor/15916-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70142">
    <skos:narrower rdf:resource="descriptor/16749-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70142">
    <skos:narrower rdf:resource="descriptor/15914-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70142">
    <skos:narrower rdf:resource="descriptor/15061-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70142">
    <skos:narrower rdf:resource="descriptor/15912-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70142">
    <skos:narrower rdf:resource="descriptor/15911-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70142">
    <skos:narrower rdf:resource="descriptor/19641-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70142">
    <skos:narrower rdf:resource="descriptor/15910-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70142">
    <skos:narrower rdf:resource="descriptor/15909-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70142">
    <skos:narrower rdf:resource="descriptor/15908-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70142">
    <skos:narrower rdf:resource="descriptor/15907-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70142">
    <skos:narrower rdf:resource="descriptor/15905-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70142">
    <skos:narrower rdf:resource="descriptor/19638-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70142">
    <skos:narrower rdf:resource="descriptor/15904-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70142">
    <skos:narrower rdf:resource="descriptor/18971-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70142">
    <skos:narrower rdf:resource="descriptor/18968-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70142">
    <skos:narrower rdf:resource="thsys/70182"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70142">
    <skos:narrower rdf:resource="thsys/70572"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20506-5">
    <skos:scopeNote xml:lang="de">Mean Reversion ist eine Tendenz von bestimmten Zufallsvariablen (z.B. Zins) bei einem langfristigen Durchschnittswert zu bleiben oder nach einiger Zeit dorthin zurückzukehren. Diese Tendenz kann sich u.U. erst bei der Analyse sehr langfristiger Zeitreihen zeigen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18593-6">
    <skos:related rdf:resource="descriptor/13298-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18593-6">
    <skos:related rdf:resource="descriptor/18594-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18593-6">
    <skos:related rdf:resource="descriptor/18592-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18593-6">
    <skos:related rdf:resource="descriptor/14455-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10209-3">
    <skos:broader rdf:resource="thsys/71119"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10209-3">
    <skos:broader rdf:resource="descriptor/10905-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70109">
    <skos:narrower rdf:resource="descriptor/19636-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70109">
    <skos:narrower rdf:resource="descriptor/15706-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70109">
    <skos:narrower rdf:resource="descriptor/10772-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70109">
    <skos:narrower rdf:resource="descriptor/13511-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70109">
    <skos:narrower rdf:resource="descriptor/10465-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70109">
    <skos:narrower rdf:resource="descriptor/19405-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70109">
    <skos:narrower rdf:resource="descriptor/15723-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70109">
    <skos:narrower rdf:resource="descriptor/15698-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70109">
    <skos:narrower rdf:resource="descriptor/19255-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70109">
    <skos:narrower rdf:resource="descriptor/15704-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70109">
    <skos:narrower rdf:resource="descriptor/25501-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70109">
    <skos:narrower rdf:resource="descriptor/19288-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70109">
    <skos:narrower rdf:resource="descriptor/10574-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70109">
    <skos:narrower rdf:resource="descriptor/15703-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70109">
    <skos:narrower rdf:resource="descriptor/10500-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70109">
    <skos:narrower rdf:resource="descriptor/18537-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70109">
    <skos:narrower rdf:resource="descriptor/19520-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70109">
    <skos:narrower rdf:resource="descriptor/13462-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70109">
    <skos:narrower rdf:resource="descriptor/18838-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70109">
    <skos:narrower rdf:resource="descriptor/11416-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70109">
    <skos:narrower rdf:resource="descriptor/19261-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70109">
    <skos:narrower rdf:resource="descriptor/12046-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70109">
    <skos:narrower rdf:resource="descriptor/11533-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70109">
    <skos:narrower rdf:resource="descriptor/15705-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70109">
    <skos:narrower rdf:resource="descriptor/19519-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70109">
    <skos:narrower rdf:resource="descriptor/18729-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70109">
    <skos:narrower rdf:resource="descriptor/13691-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70109">
    <skos:narrower rdf:resource="descriptor/18534-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70109">
    <skos:narrower rdf:resource="descriptor/13444-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11991-1">
    <skos:broader rdf:resource="thsys/71064"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11991-1">
    <skos:broader rdf:resource="descriptor/11857-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10080-5">
    <skos:prefLabel xml:lang="en">Non-walrasian equilibrium</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10080-5">
    <skos:prefLabel xml:lang="de">Nicht-Walrasianisches Gleichgewicht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15998-2">
    <skos:prefLabel xml:lang="en">Civil right</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15998-2">
    <skos:prefLabel xml:lang="de">Grundrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19481-5">
    <skos:prefLabel xml:lang="en">Climate protection</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19481-5">
    <skos:prefLabel xml:lang="de">Klimaschutz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19529-3">
    <skos:broader rdf:resource="thsys/73350"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19529-3">
    <skos:broader rdf:resource="thsys/73347"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19529-3">
    <skos:broader rdf:resource="descriptor/15669-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13588-3">
    <skos:narrower rdf:resource="descriptor/13608-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13588-3">
    <skos:narrower rdf:resource="descriptor/13638-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15774-3">
    <skos:prefLabel xml:lang="en">Environmental engineering</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15774-3">
    <skos:prefLabel xml:lang="de">Umwelttechnik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11596-0">
    <skos:narrower rdf:resource="descriptor/11560-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11596-0">
    <skos:narrower rdf:resource="descriptor/11592-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11596-0">
    <skos:narrower rdf:resource="descriptor/11591-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11596-0">
    <skos:narrower rdf:resource="descriptor/11587-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11596-0">
    <skos:narrower rdf:resource="descriptor/11586-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19000-5">
    <skos:related rdf:resource="descriptor/17199-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19000-5">
    <skos:related rdf:resource="descriptor/17991-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19576-1">
    <skos:broader rdf:resource="thsys/72393"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19576-1">
    <skos:broader rdf:resource="descriptor/18076-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70403">
    <skos:narrower rdf:resource="descriptor/14146-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70403">
    <skos:narrower rdf:resource="descriptor/14170-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70403">
    <skos:narrower rdf:resource="descriptor/19023-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70403">
    <skos:narrower rdf:resource="descriptor/19022-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70403">
    <skos:narrower rdf:resource="descriptor/14145-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70403">
    <skos:narrower rdf:resource="descriptor/14169-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70403">
    <skos:narrower rdf:resource="descriptor/14164-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70403">
    <skos:narrower rdf:resource="descriptor/14163-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70403">
    <skos:narrower rdf:resource="descriptor/14144-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70403">
    <skos:narrower rdf:resource="descriptor/14143-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70403">
    <skos:narrower rdf:resource="descriptor/14142-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70403">
    <skos:narrower rdf:resource="descriptor/14141-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70403">
    <skos:narrower rdf:resource="descriptor/14139-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70403">
    <skos:narrower rdf:resource="descriptor/14160-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70403">
    <skos:narrower rdf:resource="descriptor/19019-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70403">
    <skos:narrower rdf:resource="descriptor/14135-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70403">
    <skos:narrower rdf:resource="descriptor/14158-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70403">
    <skos:narrower rdf:resource="descriptor/19015-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70403">
    <skos:narrower rdf:resource="descriptor/14131-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14145-4">
    <skos:related rdf:resource="descriptor/19347-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14145-4">
    <skos:related rdf:resource="descriptor/13940-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14483-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13813-1">
    <skos:broader rdf:resource="thsys/70937"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13813-1">
    <skos:broader rdf:resource="thsys/70891"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13813-1">
    <skos:broader rdf:resource="descriptor/13297-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10144-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73389">
    <skos:narrower rdf:resource="descriptor/13569-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73389">
    <skos:narrower rdf:resource="descriptor/18643-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73389">
    <skos:narrower rdf:resource="descriptor/16171-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73389">
    <skos:narrower rdf:resource="descriptor/16187-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73389">
    <skos:narrower rdf:resource="descriptor/16070-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73389">
    <skos:narrower rdf:resource="descriptor/18592-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73389">
    <skos:narrower rdf:resource="descriptor/18591-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73389">
    <skos:narrower rdf:resource="descriptor/13647-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73389">
    <skos:narrower rdf:resource="descriptor/16170-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73389">
    <skos:narrower rdf:resource="descriptor/16169-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73389">
    <skos:narrower rdf:resource="descriptor/13502-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73389">
    <skos:narrower rdf:resource="descriptor/18567-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73389">
    <skos:narrower rdf:resource="descriptor/18605-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73389">
    <skos:narrower rdf:resource="descriptor/18602-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73389">
    <skos:narrower rdf:resource="descriptor/13565-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73389">
    <skos:narrower rdf:resource="descriptor/16166-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18160-2">
    <skos:narrower rdf:resource="descriptor/19404-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18160-2">
    <skos:narrower rdf:resource="descriptor/19283-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13693-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19851-5">
    <skos:broader rdf:resource="thsys/73332"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19851-5">
    <skos:broader rdf:resource="descriptor/13410-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19802-4">
    <skos:scopeNote xml:lang="de">Einbindung vorhandener EDV-Systeme und Daten in neue Umgebungen. Systemumstellungen der DV.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73294">
    <skos:prefLabel xml:lang="de">W.06.01.12.02  Genussmittelindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71086">
    <skos:broader rdf:resource="thsys/70091"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19496-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19409-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17724-1">
    <skos:narrower rdf:resource="descriptor/17723-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17724-1">
    <skos:narrower rdf:resource="descriptor/17722-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17724-1">
    <skos:narrower rdf:resource="descriptor/17721-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17724-1">
    <skos:narrower rdf:resource="descriptor/17717-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17724-1">
    <skos:narrower rdf:resource="descriptor/17716-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17724-1">
    <skos:narrower rdf:resource="descriptor/17713-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17724-1">
    <skos:narrower rdf:resource="descriptor/17712-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17724-1">
    <skos:narrower rdf:resource="descriptor/17709-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17724-1">
    <skos:narrower rdf:resource="descriptor/17707-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17724-1">
    <skos:narrower rdf:resource="descriptor/17706-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17724-1">
    <skos:narrower rdf:resource="descriptor/17705-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17724-1">
    <skos:narrower rdf:resource="descriptor/17704-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17724-1">
    <skos:narrower rdf:resource="descriptor/17703-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17724-1">
    <skos:narrower rdf:resource="descriptor/17702-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17724-1">
    <skos:narrower rdf:resource="descriptor/17700-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17724-1">
    <skos:narrower rdf:resource="descriptor/17696-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17724-1">
    <skos:narrower rdf:resource="descriptor/17695-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18556-5">
    <skos:prefLabel xml:lang="en">Event marketing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18556-5">
    <skos:prefLabel xml:lang="de">Erlebnismarketing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15861-1">
    <skos:broader rdf:resource="thsys/70878"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15861-1">
    <skos:broader rdf:resource="thsys/70180"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15861-1">
    <skos:broader rdf:resource="descriptor/18859-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15861-1">
    <skos:broader rdf:resource="descriptor/10144-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19595-4">
    <skos:scopeNote xml:lang="de">Form eines eindeutigen Auszahlungsvektors in einem kooperativen Spiel.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13545-0">
    <skos:related rdf:resource="descriptor/18591-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13545-0">
    <skos:related rdf:resource="descriptor/13256-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13545-0">
    <skos:related rdf:resource="descriptor/18417-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13545-0">
    <skos:related rdf:resource="descriptor/13252-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19310-4">
    <skos:related rdf:resource="descriptor/11572-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19310-4">
    <skos:related rdf:resource="descriptor/19319-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19310-4">
    <skos:related rdf:resource="descriptor/19235-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19310-4">
    <skos:related rdf:resource="descriptor/15704-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19310-4">
    <skos:related rdf:resource="descriptor/19286-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19310-4">
    <skos:related rdf:resource="descriptor/10683-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12591-5">
    <skos:broader rdf:resource="thsys/70832"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12591-5">
    <skos:broader rdf:resource="thsys/70458"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12591-5">
    <skos:broader rdf:resource="thsys/70245"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12591-5">
    <skos:broader rdf:resource="descriptor/12592-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12591-5">
    <skos:broader rdf:resource="descriptor/16527-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16327-2">
    <skos:prefLabel xml:lang="en">Ministry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16327-2">
    <skos:prefLabel xml:lang="de">Ministerium</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10487-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18976-4">
    <skos:broader rdf:resource="thsys/73372"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18976-4">
    <skos:broader rdf:resource="thsys/71112"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18976-4">
    <skos:broader rdf:resource="thsys/70814"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18976-4">
    <skos:broader rdf:resource="descriptor/10987-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18572-0">
    <skos:altLabel xml:lang="de">Eisenbahnunternehmen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19412-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10348-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12079-1">
    <skos:prefLabel xml:lang="en">Management information system</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12079-1">
    <skos:prefLabel xml:lang="de">Management-Informationssystem</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13647-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13669-3">
    <skos:prefLabel xml:lang="en">Magazine</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13669-3">
    <skos:prefLabel xml:lang="de">Zeitschrift</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14645-5">
    <skos:broader rdf:resource="thsys/70463"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14645-5">
    <skos:broader rdf:resource="descriptor/14205-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14645-5">
    <skos:broader rdf:resource="descriptor/14642-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71071">
    <skos:broader rdf:resource="thsys/70101"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12983-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25621-1">
    <skos:altLabel xml:lang="en">Consumer bankruptcy</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25621-1">
    <skos:altLabel xml:lang="de">Verbraucherkonkurs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25621-1">
    <skos:altLabel xml:lang="de">Verbraucherinsolvenzverfahren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25621-1">
    <skos:altLabel xml:lang="de">Verbraucherinsolvenz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25621-1">
    <skos:altLabel xml:lang="de">Privatkonkurs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25621-1">
    <skos:altLabel xml:lang="de">Private Insolvenz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25621-1">
    <skos:altLabel xml:lang="de">Private Entschuldung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73339">
    <skos:narrower rdf:resource="descriptor/12172-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73339">
    <skos:narrower rdf:resource="descriptor/19809-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73339">
    <skos:narrower rdf:resource="descriptor/13277-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73339">
    <skos:narrower rdf:resource="descriptor/19808-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73339">
    <skos:narrower rdf:resource="descriptor/15410-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73339">
    <skos:narrower rdf:resource="descriptor/15416-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73339">
    <skos:narrower rdf:resource="descriptor/15611-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73339">
    <skos:narrower rdf:resource="descriptor/15609-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73339">
    <skos:narrower rdf:resource="descriptor/19807-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73339">
    <skos:narrower rdf:resource="descriptor/19806-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73339">
    <skos:narrower rdf:resource="descriptor/19811-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73339">
    <skos:narrower rdf:resource="descriptor/18170-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12046-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14266-6">
    <skos:related rdf:resource="descriptor/18280-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14266-6">
    <skos:related rdf:resource="descriptor/18162-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14266-6">
    <skos:related rdf:resource="descriptor/18402-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14266-6">
    <skos:related rdf:resource="descriptor/14269-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14266-6">
    <skos:related rdf:resource="descriptor/19184-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14266-6">
    <skos:related rdf:resource="descriptor/18401-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14266-6">
    <skos:related rdf:resource="descriptor/13034-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14266-6">
    <skos:related rdf:resource="descriptor/13224-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14266-6">
    <skos:related rdf:resource="descriptor/14382-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14266-6">
    <skos:related rdf:resource="descriptor/14313-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18977-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10021-0">
    <skos:editorialNote xml:lang="de">Möglichst spezifischer indexieren.</skos:editorialNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70935">
    <skos:narrower rdf:resource="descriptor/13541-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70935">
    <skos:narrower rdf:resource="descriptor/13529-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70935">
    <skos:narrower rdf:resource="descriptor/13242-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70935">
    <skos:narrower rdf:resource="descriptor/13528-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15945-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18745-2">
    <skos:altLabel xml:lang="en">Property price</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18745-2">
    <skos:altLabel xml:lang="en">House price</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18745-2">
    <skos:altLabel xml:lang="de">Hauspreis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13860-6">
    <skos:narrower rdf:resource="descriptor/13865-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13860-6">
    <skos:narrower rdf:resource="descriptor/13861-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10412-6">
    <skos:related rdf:resource="descriptor/10343-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10412-6">
    <skos:related rdf:resource="descriptor/10413-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19641-2">
    <skos:prefLabel xml:lang="en">Population economics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19641-2">
    <skos:prefLabel xml:lang="de">Bevölkerungsökonomie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73383">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11917-6">
    <skos:altLabel xml:lang="en">Distressed area</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11917-6">
    <skos:altLabel xml:lang="de">Wirtschaftsschwaches Gebiet</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11917-6">
    <skos:altLabel xml:lang="de">Unterentwickeltes Gebiet</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11917-6">
    <skos:altLabel xml:lang="de">Strukturschwache Region</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11917-6">
    <skos:altLabel xml:lang="de">Rückständiges Gebiet</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11917-6">
    <skos:altLabel xml:lang="de">Problemgebiet</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11917-6">
    <skos:altLabel xml:lang="de">Notstandsgebiet</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11917-6">
    <skos:altLabel xml:lang="de">Entwicklungsgebiet</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10470-6">
    <skos:narrower rdf:resource="descriptor/19799-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10470-6">
    <skos:narrower rdf:resource="descriptor/13578-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10470-6">
    <skos:narrower rdf:resource="descriptor/18366-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10470-6">
    <skos:narrower rdf:resource="descriptor/18398-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10470-6">
    <skos:narrower rdf:resource="descriptor/13577-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10470-6">
    <skos:narrower rdf:resource="descriptor/15677-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10470-6">
    <skos:narrower rdf:resource="descriptor/15774-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10470-6">
    <skos:narrower rdf:resource="descriptor/12613-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10470-6">
    <skos:narrower rdf:resource="descriptor/13576-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10470-6">
    <skos:narrower rdf:resource="descriptor/19797-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10470-6">
    <skos:narrower rdf:resource="descriptor/19798-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10470-6">
    <skos:narrower rdf:resource="descriptor/18412-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10470-6">
    <skos:narrower rdf:resource="descriptor/13575-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10470-6">
    <skos:narrower rdf:resource="descriptor/18443-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10470-6">
    <skos:narrower rdf:resource="descriptor/18421-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10470-6">
    <skos:narrower rdf:resource="descriptor/18344-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10470-6">
    <skos:narrower rdf:resource="descriptor/12711-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10470-6">
    <skos:narrower rdf:resource="descriptor/15727-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10470-6">
    <skos:narrower rdf:resource="descriptor/18353-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10470-6">
    <skos:narrower rdf:resource="descriptor/18426-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10470-6">
    <skos:narrower rdf:resource="descriptor/18390-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10470-6">
    <skos:narrower rdf:resource="descriptor/18495-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10470-6">
    <skos:narrower rdf:resource="descriptor/10487-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10470-6">
    <skos:narrower rdf:resource="descriptor/13450-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10830-2">
    <skos:related rdf:resource="descriptor/10831-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10830-2">
    <skos:related rdf:resource="descriptor/11638-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10830-2">
    <skos:related rdf:resource="descriptor/10817-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10830-2">
    <skos:related rdf:resource="descriptor/10814-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10830-2">
    <skos:related rdf:resource="descriptor/12446-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10830-2">
    <skos:related rdf:resource="descriptor/10826-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10830-2">
    <skos:related rdf:resource="descriptor/10827-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70333">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12524-6">
    <skos:prefLabel xml:lang="en">Marginal costing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12524-6">
    <skos:prefLabel xml:lang="de">Teilkostenrechnung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10672-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15891-6">
    <skos:prefLabel xml:lang="en">Oceanography</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15891-6">
    <skos:prefLabel xml:lang="de">Meeresforschung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12195-6">
    <skos:altLabel xml:lang="de">Wahl der Rechtsform</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12195-6">
    <skos:altLabel xml:lang="de">Unternehmensformwahl</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13824-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14433-6">
    <skos:altLabel xml:lang="de">Kraftwagen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14433-6">
    <skos:altLabel xml:lang="de">Kfz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14433-6">
    <skos:altLabel xml:lang="de">Automobil</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19613-0">
    <skos:prefLabel xml:lang="en">Aerial photograph</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19613-0">
    <skos:prefLabel xml:lang="de">Luftbild</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12234-1">
    <skos:altLabel xml:lang="de">Schuldverschreibung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12234-1">
    <skos:altLabel xml:lang="de">Obligation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12234-1">
    <skos:altLabel xml:lang="de">Inhaberschuldverschreibung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id285">
    <rdfs:seeAlso rdf:resource="descriptor/19779-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19380-4">
    <skos:broader rdf:resource="thsys/71081"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13541-1">
    <skos:broader rdf:resource="thsys/70935"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13541-1">
    <skos:broader rdf:resource="descriptor/13529-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10111-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11235-4">
    <skos:scopeNote xml:lang="de">Staatliche Maßnahmen, die die Beziehungen zwischen Angebot und Nachfrage auf dem Arbeitsmarkt zu beeinflussen suchen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19040-0">
    <skos:prefLabel xml:lang="en">Cooperation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19040-0">
    <skos:prefLabel xml:lang="de">Kooperation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73342">
    <skos:broader rdf:resource="thsys/70151"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19072-1">
    <skos:editorialNote xml:lang="de">Nur zur Postkoordination verwenden.</skos:editorialNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12458-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10706-3">
    <skos:altLabel xml:lang="en">Procedural protectionism</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10706-3">
    <skos:altLabel xml:lang="en">Non-tariff measures</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10706-3">
    <skos:altLabel xml:lang="de">Technische Handelshemmnisse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10706-3">
    <skos:altLabel xml:lang="de">Nicht-tarifäres Handelshemmnis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10706-3">
    <skos:altLabel xml:lang="de">Administrative Handelsbeschränkung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10706-3">
    <skos:altLabel xml:lang="de">Administrative Außenhandelsbeschränkung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70416">
    <skos:broader rdf:resource="thsys/p"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16981-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11397-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10699-2">
    <skos:narrower rdf:resource="descriptor/11649-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15993-5">
    <skos:narrower rdf:resource="descriptor/16001-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15993-5">
    <skos:narrower rdf:resource="descriptor/19838-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73379">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10970-0">
    <skos:prefLabel xml:lang="en">Government intervention</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10970-0">
    <skos:prefLabel xml:lang="de">Interventionismus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19114-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17758-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70310">
    <skos:narrower rdf:resource="descriptor/12729-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70310">
    <skos:narrower rdf:resource="descriptor/12727-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70310">
    <skos:narrower rdf:resource="descriptor/12703-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70310">
    <skos:narrower rdf:resource="descriptor/10324-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70310">
    <skos:narrower rdf:resource="descriptor/10323-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70310">
    <skos:narrower rdf:resource="descriptor/10454-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70310">
    <skos:narrower rdf:resource="descriptor/12724-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70310">
    <skos:narrower rdf:resource="descriptor/10320-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70310">
    <skos:narrower rdf:resource="descriptor/12722-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70310">
    <skos:narrower rdf:resource="descriptor/19124-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70310">
    <skos:narrower rdf:resource="descriptor/12719-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70310">
    <skos:narrower rdf:resource="descriptor/12711-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70310">
    <skos:narrower rdf:resource="descriptor/12725-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70310">
    <skos:narrower rdf:resource="descriptor/12720-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70310">
    <skos:narrower rdf:resource="descriptor/12718-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70310">
    <skos:narrower rdf:resource="descriptor/10310-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70310">
    <skos:narrower rdf:resource="descriptor/12717-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70310">
    <skos:narrower rdf:resource="descriptor/12716-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16087-3">
    <skos:related rdf:resource="descriptor/11640-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16087-3">
    <skos:related rdf:resource="descriptor/11532-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12026-1">
    <skos:broader rdf:resource="thsys/71066"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12026-1">
    <skos:broader rdf:resource="thsys/70889"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12026-1">
    <skos:broader rdf:resource="descriptor/10024-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12026-1">
    <skos:broader rdf:resource="descriptor/10005-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15416-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12620-3">
    <skos:prefLabel xml:lang="en">Work stress</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12620-3">
    <skos:prefLabel xml:lang="de">Stress</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15124-0">
    <skos:broader rdf:resource="thsys/72183"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15124-0">
    <skos:broader rdf:resource="thsys/70182"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15124-0">
    <skos:broader rdf:resource="descriptor/15115-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15124-0">
    <skos:broader rdf:resource="descriptor/15061-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18918-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15721-3">
    <skos:prefLabel xml:lang="en">Social problem</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15721-3">
    <skos:prefLabel xml:lang="de">Soziales Problem</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19643-5">
    <skos:prefLabel xml:lang="en">Indigenous people</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19643-5">
    <skos:prefLabel xml:lang="de">Ureinwohner</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18993-4">
    <skos:broader rdf:resource="thsys/71065"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18993-4">
    <skos:broader rdf:resource="descriptor/19460-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18993-4">
    <skos:broader rdf:resource="descriptor/18120-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18769-2">
    <skos:related rdf:resource="descriptor/18161-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11391-4">
    <skos:narrower rdf:resource="descriptor/18885-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18761-4">
    <skos:scopeNote xml:lang="de">Rechtliche Bestimmungen zum Schutz von Computerprogrammen aus allen Rechtsbereichen, vor allem Urheberrecht, Patentrecht und Wettbewerbsrecht.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14646-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16529-4">
    <skos:prefLabel xml:lang="en">Disability policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16529-4">
    <skos:prefLabel xml:lang="de">Behindertenpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70358">
    <rdfs:label xml:lang="de">Metalle</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18457-0">
    <skos:prefLabel xml:lang="en">Industrial business cycle</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18457-0">
    <skos:prefLabel xml:lang="de">Industriekonjunktur</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15739-5">
    <skos:narrower rdf:resource="descriptor/15777-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19432-4">
    <skos:scopeNote xml:lang="de">Beim Eintritt in einen Markt entstehen einmalige fixe Kosten, die bei Verlassen des Marktes nicht oder nur teilweise erstattet werden.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12524-6">
    <skos:broader rdf:resource="thsys/70578"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12524-6">
    <skos:broader rdf:resource="descriptor/12480-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13253-6">
    <skos:altLabel xml:lang="de">Reiseveranstalter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13253-6">
    <skos:altLabel xml:lang="de">Reisebüro</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13253-6">
    <skos:altLabel xml:lang="de">Fremdenverkehrsbüro</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18193-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18130-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16159-4">
    <skos:prefLabel xml:lang="en">Social legislation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16159-4">
    <skos:prefLabel xml:lang="de">Sozialrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19416-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10135-6">
    <skos:related rdf:resource="descriptor/10137-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10135-6">
    <skos:related rdf:resource="descriptor/18693-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10135-6">
    <skos:related rdf:resource="descriptor/18701-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10135-6">
    <skos:related rdf:resource="descriptor/19267-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10135-6">
    <skos:related rdf:resource="descriptor/18087-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10135-6">
    <skos:related rdf:resource="descriptor/11490-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10135-6">
    <skos:related rdf:resource="descriptor/18679-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10135-6">
    <skos:related rdf:resource="descriptor/10001-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10048-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18157-5">
    <skos:related rdf:resource="descriptor/13766-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18157-5">
    <skos:related rdf:resource="descriptor/18061-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10522-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13050-6">
    <skos:prefLabel xml:lang="en">Container construction</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13050-6">
    <skos:prefLabel xml:lang="de">Behälterbau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12051-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19601-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14999-5">
    <skos:broader rdf:resource="thsys/73298"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14999-5">
    <skos:broader rdf:resource="descriptor/14997-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16641-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id154">
    <rdfs:seeAlso rdf:resource="descriptor/10555-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13543-4">
    <skos:broader rdf:resource="thsys/70934"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13543-4">
    <skos:broader rdf:resource="descriptor/13256-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10966-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15911-5">
    <skos:related rdf:resource="descriptor/16486-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15911-5">
    <skos:related rdf:resource="descriptor/16477-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15911-5">
    <skos:related rdf:resource="descriptor/15708-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15911-5">
    <skos:related rdf:resource="descriptor/15909-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15911-5">
    <skos:related rdf:resource="descriptor/15989-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19454-1">
    <skos:prefLabel xml:lang="en">Land market</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19454-1">
    <skos:prefLabel xml:lang="de">Bodenmarkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11149-4">
    <skos:related rdf:resource="descriptor/11151-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11149-4">
    <skos:related rdf:resource="descriptor/13429-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12910-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10285-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19322-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73353">
    <skos:narrower rdf:resource="descriptor/15903-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73353">
    <skos:narrower rdf:resource="descriptor/15899-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73353">
    <skos:narrower rdf:resource="descriptor/19619-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73353">
    <skos:narrower rdf:resource="descriptor/15893-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73353">
    <skos:narrower rdf:resource="descriptor/15892-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73353">
    <skos:narrower rdf:resource="descriptor/19618-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73353">
    <skos:narrower rdf:resource="descriptor/19617-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73353">
    <skos:narrower rdf:resource="descriptor/15887-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73353">
    <skos:narrower rdf:resource="descriptor/15883-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73353">
    <skos:narrower rdf:resource="descriptor/19616-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73353">
    <skos:narrower rdf:resource="descriptor/15879-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73353">
    <skos:narrower rdf:resource="descriptor/15869-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73353">
    <skos:narrower rdf:resource="descriptor/15868-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73353">
    <skos:narrower rdf:resource="descriptor/18984-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19785-6">
    <skos:prefLabel xml:lang="en">Pharmaceutical research</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19785-6">
    <skos:prefLabel xml:lang="de">Pharmazeutische Forschung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15251-0">
    <skos:prefLabel xml:lang="en">Bias</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15251-0">
    <skos:prefLabel xml:lang="de">Bias</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10326-6">
    <skos:altLabel xml:lang="de">Produktionsmittel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12665-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18236-2">
    <skos:prefLabel xml:lang="en">Liquefied natural gas</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18236-2">
    <skos:prefLabel xml:lang="de">Flüssiggas</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13239-0">
    <skos:prefLabel xml:lang="en">Filling station</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13239-0">
    <skos:prefLabel xml:lang="de">Tankstelle</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18266-0">
    <skos:altLabel xml:lang="de">Insektenvertilgungsmittel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12004-4">
    <skos:related rdf:resource="descriptor/16552-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12004-4">
    <skos:related rdf:resource="descriptor/11900-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12004-4">
    <skos:related rdf:resource="descriptor/16549-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12004-4">
    <skos:related rdf:resource="descriptor/11552-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12004-4">
    <skos:related rdf:resource="descriptor/12001-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17580-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17951-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19396-3">
    <skos:broader rdf:resource="thsys/70959"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19396-3">
    <skos:broader rdf:resource="thsys/70799"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19396-3">
    <skos:broader rdf:resource="descriptor/10012-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11847-1">
    <skos:narrower rdf:resource="descriptor/11845-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11847-1">
    <skos:narrower rdf:resource="descriptor/18752-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11847-1">
    <skos:narrower rdf:resource="descriptor/18754-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11847-1">
    <skos:narrower rdf:resource="descriptor/18755-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12718-0">
    <zbwext:useInsteadNote rdf:nodeID="id79"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70307">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10182-4">
    <skos:related rdf:resource="descriptor/10183-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10182-4">
    <skos:related rdf:resource="descriptor/10013-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10182-4">
    <skos:related rdf:resource="descriptor/10180-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10182-4">
    <skos:related rdf:resource="descriptor/10178-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10182-4">
    <skos:related rdf:resource="descriptor/10278-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10182-4">
    <skos:related rdf:resource="descriptor/10171-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18017-0">
    <skos:scopeNote xml:lang="de">Konzeption, Koordinierung, Organisation und Administration des Informationssystems innerhalb einer Institution, meist eines Betriebs.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18222-6">
    <skos:narrower rdf:resource="descriptor/14083-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18222-6">
    <skos:narrower rdf:resource="descriptor/14997-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18222-6">
    <skos:narrower rdf:resource="descriptor/14996-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18222-6">
    <skos:narrower rdf:resource="descriptor/18213-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18222-6">
    <skos:narrower rdf:resource="descriptor/14065-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18222-6">
    <skos:narrower rdf:resource="descriptor/14064-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18222-6">
    <skos:narrower rdf:resource="descriptor/14060-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18222-6">
    <skos:narrower rdf:resource="descriptor/14972-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11710-0">
    <skos:broader rdf:resource="thsys/73337"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11710-0">
    <skos:broader rdf:resource="thsys/71073"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11710-0">
    <skos:broader rdf:resource="descriptor/10829-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11710-0">
    <skos:broader rdf:resource="descriptor/11684-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11571-2">
    <skos:altLabel xml:lang="de">Sparprämiengesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11571-2">
    <skos:altLabel xml:lang="de">Sparprämie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13852-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13652-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14980-5">
    <skos:broader rdf:resource="thsys/73300"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14980-5">
    <skos:broader rdf:resource="thsys/70342"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14980-5">
    <skos:broader rdf:resource="descriptor/14985-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73331">
    <skos:narrower rdf:resource="descriptor/19323-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73331">
    <skos:narrower rdf:resource="descriptor/16048-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73331">
    <skos:narrower rdf:resource="descriptor/18734-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73331">
    <skos:narrower rdf:resource="descriptor/18365-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73331">
    <skos:narrower rdf:resource="descriptor/19817-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73331">
    <skos:narrower rdf:resource="descriptor/16090-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73331">
    <skos:narrower rdf:resource="descriptor/18904-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73331">
    <skos:narrower rdf:resource="descriptor/16182-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73331">
    <skos:narrower rdf:resource="descriptor/18441-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73331">
    <skos:narrower rdf:resource="descriptor/12331-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73331">
    <skos:narrower rdf:resource="descriptor/18703-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73331">
    <skos:narrower rdf:resource="descriptor/18961-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73331">
    <skos:narrower rdf:resource="descriptor/10917-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70851">
    <skos:prefLabel xml:lang="de">W.23.02.02  Gewerkschaften</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14821-4">
    <skos:related rdf:resource="descriptor/14819-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70961">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70958">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.09</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16972-1">
    <skos:narrower rdf:resource="descriptor/16980-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16972-1">
    <skos:narrower rdf:resource="descriptor/16981-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16972-1">
    <skos:narrower rdf:resource="descriptor/16978-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16972-1">
    <skos:narrower rdf:resource="descriptor/16979-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16972-1">
    <skos:narrower rdf:resource="descriptor/16977-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16972-1">
    <skos:narrower rdf:resource="descriptor/16975-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16972-1">
    <skos:narrower rdf:resource="descriptor/16976-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16972-1">
    <skos:narrower rdf:resource="descriptor/16974-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16972-1">
    <skos:narrower rdf:resource="descriptor/16973-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16972-1">
    <skos:narrower rdf:resource="descriptor/16971-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16972-1">
    <skos:narrower rdf:resource="descriptor/16970-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16972-1">
    <skos:narrower rdf:resource="descriptor/16969-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16972-1">
    <skos:narrower rdf:resource="descriptor/16968-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16972-1">
    <skos:narrower rdf:resource="descriptor/16967-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16972-1">
    <skos:narrower rdf:resource="descriptor/16966-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16972-1">
    <skos:narrower rdf:resource="descriptor/16964-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16972-1">
    <skos:narrower rdf:resource="descriptor/16963-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16972-1">
    <skos:narrower rdf:resource="descriptor/16962-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16972-1">
    <skos:narrower rdf:resource="descriptor/16961-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16972-1">
    <skos:narrower rdf:resource="descriptor/16960-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16972-1">
    <skos:narrower rdf:resource="descriptor/16959-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16972-1">
    <skos:narrower rdf:resource="descriptor/16958-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16972-1">
    <skos:narrower rdf:resource="descriptor/16957-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16972-1">
    <skos:narrower rdf:resource="descriptor/16956-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19104-0">
    <skos:prefLabel xml:lang="en">Management control system</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19104-0">
    <skos:prefLabel xml:lang="de">Controlling-Informationssystem</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16201-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70815">
    <rdfs:label xml:lang="en">Environmental goods and services industry</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70815">
    <rdfs:label xml:lang="de">Umweltschutzsektor</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70160">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.05.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19093-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14777-2">
    <skos:prefLabel xml:lang="en">Glass fiber</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14777-2">
    <skos:prefLabel xml:lang="de">Glasfaser</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18751-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12030-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10082-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19830-6">
    <skos:scopeNote xml:lang="de">Private karitative Hilfen für Bedürftige.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13479-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17982-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18497-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13673-5">
    <skos:broader rdf:resource="thsys/73338"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13673-5">
    <skos:broader rdf:resource="thsys/71036"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13673-5">
    <skos:broader rdf:resource="descriptor/15606-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13235-1">
    <skos:altLabel xml:lang="de">Sportfachgeschäft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13235-1">
    <skos:altLabel xml:lang="de">Sportartikelgroßhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15776-6">
    <skos:altLabel xml:lang="en">Pollution permit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15776-6">
    <skos:altLabel xml:lang="en">Emission licence</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15776-6">
    <skos:altLabel xml:lang="en">Discharge permit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15776-6">
    <skos:altLabel xml:lang="de">Zertifikat zur Umweltverschmutzung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15776-6">
    <skos:altLabel xml:lang="de">Verschmutzungsrechte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15776-6">
    <skos:altLabel xml:lang="de">Umweltzertifikat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15776-6">
    <skos:altLabel xml:lang="de">Umweltlizenz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15776-6">
    <skos:altLabel xml:lang="de">Schadstoffzertifikat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15776-6">
    <skos:altLabel xml:lang="de">Handelbare Emissionsrechte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15776-6">
    <skos:altLabel xml:lang="de">Emissionszertifikat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15776-6">
    <skos:altLabel xml:lang="de">Emissionslizenz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15776-6">
    <skos:altLabel xml:lang="de">Blasenkonzept</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18885-0">
    <skos:altLabel xml:lang="de">Weiterbildung im Betrieb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18885-0">
    <skos:altLabel xml:lang="de">Betriebliche Fortbildung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19762-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18386-4">
    <skos:altLabel xml:lang="en">Minerals statistics</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18172-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11446-5">
    <skos:broader rdf:resource="thsys/70305"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11446-5">
    <skos:broader rdf:resource="descriptor/11448-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14947-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13052-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71110">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.04.03</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70988">
    <rdfs:label xml:lang="de">Leder- und Schuhindustrie</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id85">
    <rdfs:seeAlso rdf:resource="descriptor/10342-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11304-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12209-0">
    <skos:narrower rdf:resource="descriptor/20301-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12209-0">
    <skos:narrower rdf:resource="descriptor/12208-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16427-5">
    <skos:related rdf:resource="descriptor/19722-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16427-5">
    <skos:related rdf:resource="descriptor/16470-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16427-5">
    <skos:related rdf:resource="descriptor/16454-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16427-5">
    <skos:related rdf:resource="descriptor/16497-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16427-5">
    <skos:related rdf:resource="descriptor/16204-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16427-5">
    <skos:related rdf:resource="descriptor/16391-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16427-5">
    <skos:related rdf:resource="descriptor/16445-3"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id172">
    <rdf:value xml:lang="de">Für Fernkopiergeräte</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15619-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15700-4">
    <skos:prefLabel xml:lang="en">Contemporary history</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15700-4">
    <skos:prefLabel xml:lang="de">Zeitgeschichte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73359">
    <skos:narrower rdf:resource="descriptor/16440-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73359">
    <skos:narrower rdf:resource="descriptor/18859-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73359">
    <skos:narrower rdf:resource="descriptor/15722-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73359">
    <skos:narrower rdf:resource="descriptor/16654-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73359">
    <skos:narrower rdf:resource="descriptor/15712-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73359">
    <skos:narrower rdf:resource="descriptor/16651-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73359">
    <skos:narrower rdf:resource="descriptor/16226-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73359">
    <skos:narrower rdf:resource="descriptor/11163-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73359">
    <skos:narrower rdf:resource="descriptor/15798-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73359">
    <skos:narrower rdf:resource="descriptor/16587-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10553-2">
    <skos:broader rdf:resource="thsys/71085"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10553-2">
    <skos:broader rdf:resource="thsys/71084"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10553-2">
    <skos:broader rdf:resource="descriptor/10552-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11894-6">
    <skos:altLabel xml:lang="de">Räumliche Bevölkerungsentwicklung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14739-3">
    <skos:broader rdf:resource="thsys/70536"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14739-3">
    <skos:broader rdf:resource="descriptor/14741-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13193-5">
    <skos:altLabel xml:lang="de">Pharmagroßhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13193-5">
    <skos:altLabel xml:lang="de">Arzneimittelhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13193-5">
    <skos:altLabel xml:lang="de">Arzneimittelgroßhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18784-6">
    <skos:altLabel xml:lang="de">FuE-Finanzierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18784-6">
    <skos:altLabel xml:lang="de">Forschungs- und Entwicklungsfinanzierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18784-6">
    <skos:altLabel xml:lang="de">F&amp;amp;E-Finanzierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10324-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15243-6">
    <skos:altLabel xml:lang="en">Stratified sampling</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15243-6">
    <skos:altLabel xml:lang="en">Sampling theory</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15243-6">
    <skos:altLabel xml:lang="en">Sampling plan</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15243-6">
    <skos:altLabel xml:lang="en">Sample survey</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15243-6">
    <skos:altLabel xml:lang="en">Sample design</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15243-6">
    <skos:altLabel xml:lang="en">Random sampling</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15243-6">
    <skos:altLabel xml:lang="de">Zufallsstichprobenverfahren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15243-6">
    <skos:altLabel xml:lang="de">Zufallsstichprobe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15243-6">
    <skos:altLabel xml:lang="de">Zufallsauswahl</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15243-6">
    <skos:altLabel xml:lang="de">Stichprobenumfang</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15243-6">
    <skos:altLabel xml:lang="de">Stichprobentheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15243-6">
    <skos:altLabel xml:lang="de">Stichprobenplan</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15243-6">
    <skos:altLabel xml:lang="de">Stichprobenentnahme</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15243-6">
    <skos:altLabel xml:lang="de">Repräsentative Stichprobe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15243-6">
    <skos:altLabel xml:lang="de">Geschichtete Stichprobenauswahl</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10424-6">
    <skos:prefLabel xml:lang="en">Optimal growth</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10424-6">
    <skos:prefLabel xml:lang="de">Optimales Wachstum</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14909-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11836-6">
    <skos:broader rdf:resource="thsys/70884"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11836-6">
    <skos:broader rdf:resource="descriptor/10978-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11836-6">
    <skos:broader rdf:resource="descriptor/18755-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11203-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14633-5">
    <skos:altLabel xml:lang="de">Wachswaren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14633-5">
    <skos:altLabel xml:lang="de">Synthetisches Wachs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14633-5">
    <skos:altLabel xml:lang="de">Stearin</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14633-5">
    <skos:altLabel xml:lang="de">Naturwachs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14633-5">
    <skos:altLabel xml:lang="de">Kerze</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14633-5">
    <skos:altLabel xml:lang="de">Bienenwachs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15458-6">
    <skos:related rdf:resource="descriptor/15060-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15458-6">
    <skos:related rdf:resource="descriptor/10209-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15458-6">
    <skos:related rdf:resource="descriptor/15415-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15458-6">
    <skos:related rdf:resource="descriptor/11163-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15458-6">
    <skos:related rdf:resource="descriptor/11168-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15458-6">
    <skos:related rdf:resource="descriptor/15470-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15458-6">
    <skos:related rdf:resource="descriptor/10138-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15458-6">
    <skos:related rdf:resource="descriptor/10070-1"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id168">
    <rdfs:seeAlso rdf:resource="descriptor/15995-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15654-6">
    <skos:prefLabel xml:lang="en">Culture</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15654-6">
    <skos:prefLabel xml:lang="de">Kultur</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10074-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18095-1">
    <skos:prefLabel xml:lang="en">Textile machinery industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18095-1">
    <skos:prefLabel xml:lang="de">Textilmaschinenbau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19405-0">
    <skos:related rdf:resource="descriptor/11637-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19405-0">
    <skos:related rdf:resource="descriptor/11642-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19405-0">
    <skos:related rdf:resource="descriptor/11704-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19405-0">
    <skos:related rdf:resource="descriptor/11547-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11569-3">
    <skos:prefLabel xml:lang="en">Welfare payments</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11569-3">
    <skos:prefLabel xml:lang="de">Sozialhilfe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70858">
    <skos:narrower rdf:resource="descriptor/11724-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70858">
    <skos:narrower rdf:resource="descriptor/11057-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70858">
    <skos:narrower rdf:resource="descriptor/18828-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70858">
    <skos:narrower rdf:resource="descriptor/11721-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70858">
    <skos:narrower rdf:resource="descriptor/10975-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70858">
    <skos:narrower rdf:resource="descriptor/10353-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70858">
    <skos:narrower rdf:resource="descriptor/11167-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70858">
    <skos:narrower rdf:resource="descriptor/11718-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70858">
    <skos:narrower rdf:resource="descriptor/11054-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70858">
    <skos:narrower rdf:resource="descriptor/11526-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70858">
    <skos:narrower rdf:resource="descriptor/11715-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19837-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/n">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11164-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11215-3">
    <skos:altLabel xml:lang="en">Leave</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11215-3">
    <skos:altLabel xml:lang="en">Holiday</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11215-3">
    <skos:altLabel xml:lang="de">Urlaubsrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11215-3">
    <skos:altLabel xml:lang="de">Urlaubsanspruch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11215-3">
    <skos:altLabel xml:lang="de">Urlaubsabgeltung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11215-3">
    <skos:altLabel xml:lang="de">Ferien</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11215-3">
    <skos:altLabel xml:lang="de">Erholungsurlaub</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11215-3">
    <skos:altLabel xml:lang="de">Bundesurlaubsgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14534-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19158-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17990-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19011-0">
    <skos:altLabel xml:lang="en">Wage gap (Gender)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19011-0">
    <skos:altLabel xml:lang="en">Racial wage gap</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19011-0">
    <skos:altLabel xml:lang="en">Gender wage gap</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19011-0">
    <skos:altLabel xml:lang="en">Equal pay</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19011-0">
    <skos:altLabel xml:lang="de">Lohndiskriminierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12146-5">
    <skos:prefLabel xml:lang="en">Matrix-organization</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12146-5">
    <skos:prefLabel xml:lang="de">Matrixorganisation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18781-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12256-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14971-6">
    <skos:broader rdf:resource="thsys/73300"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14971-6">
    <skos:broader rdf:resource="descriptor/14976-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18964-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10465-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18417-5">
    <skos:broader rdf:resource="thsys/70999"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18417-5">
    <skos:broader rdf:resource="thsys/70979"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18417-5">
    <skos:broader rdf:resource="descriptor/11820-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14985-2">
    <skos:related rdf:resource="descriptor/14102-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14985-2">
    <skos:related rdf:resource="descriptor/19348-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14985-2">
    <skos:related rdf:resource="descriptor/13234-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14985-2">
    <skos:related rdf:resource="descriptor/12958-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13131-6">
    <skos:narrower rdf:resource="descriptor/13144-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10042-6">
    <skos:narrower rdf:resource="descriptor/10043-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19841-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12618-4">
    <skos:related rdf:resource="descriptor/12620-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12618-4">
    <skos:related rdf:resource="descriptor/12615-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12618-4">
    <skos:related rdf:resource="descriptor/11290-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12618-4">
    <skos:related rdf:resource="descriptor/12609-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14504-2">
    <skos:broader rdf:resource="thsys/70477"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14504-2">
    <skos:broader rdf:resource="descriptor/14502-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13255-2">
    <skos:prefLabel xml:lang="en">Rail transport</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13255-2">
    <skos:prefLabel xml:lang="de">Schienenverkehr</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13303-3">
    <skos:narrower rdf:resource="descriptor/10458-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13303-3">
    <skos:narrower rdf:resource="descriptor/13358-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13303-3">
    <skos:narrower rdf:resource="descriptor/19470-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19766-3">
    <skos:related rdf:resource="descriptor/18869-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19766-3">
    <skos:related rdf:resource="descriptor/19655-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19766-3">
    <skos:related rdf:resource="descriptor/19253-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19766-3">
    <skos:related rdf:resource="descriptor/16759-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19766-3">
    <skos:related rdf:resource="descriptor/10142-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19766-3">
    <skos:related rdf:resource="descriptor/16604-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13067-3">
    <skos:prefLabel xml:lang="en">Machine tool industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13067-3">
    <skos:prefLabel xml:lang="de">Werkzeugmaschinenbau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id284">
    <rdf:value xml:lang="de">Für die Anpassung einer Volkswirtschaft / einer Branche an veränderte ökonomische Rahmenbedingungen</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19596-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16226-1">
    <skos:broader rdf:resource="thsys/73376"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16226-1">
    <skos:broader rdf:resource="thsys/73359"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16226-1">
    <skos:broader rdf:resource="thsys/70853"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16688-5">
    <skos:related rdf:resource="descriptor/11992-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16688-5">
    <skos:related rdf:resource="descriptor/13861-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16688-5">
    <skos:related rdf:resource="descriptor/11879-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16688-5">
    <skos:related rdf:resource="descriptor/13445-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19195-6">
    <skos:related rdf:resource="descriptor/10239-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19195-6">
    <skos:related rdf:resource="descriptor/11919-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11087-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10913-5">
    <skos:related rdf:resource="descriptor/10029-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10913-5">
    <skos:related rdf:resource="descriptor/10942-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10913-5">
    <skos:related rdf:resource="descriptor/10895-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10913-5">
    <skos:related rdf:resource="descriptor/12036-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10913-5">
    <skos:related rdf:resource="descriptor/10960-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10913-5">
    <skos:related rdf:resource="descriptor/10205-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10913-5">
    <skos:related rdf:resource="descriptor/10904-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10913-5">
    <skos:related rdf:resource="descriptor/10958-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10913-5">
    <skos:related rdf:resource="descriptor/12282-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10913-5">
    <skos:related rdf:resource="descriptor/11786-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10913-5">
    <skos:related rdf:resource="descriptor/10891-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11906-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11375-2">
    <skos:prefLabel xml:lang="en">Pupils</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11375-2">
    <skos:prefLabel xml:lang="de">Schüler</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15752-6">
    <skos:related rdf:resource="descriptor/12922-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15752-6">
    <skos:related rdf:resource="descriptor/15753-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15752-6">
    <skos:related rdf:resource="descriptor/18995-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15752-6">
    <skos:related rdf:resource="descriptor/14047-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15752-6">
    <skos:related rdf:resource="descriptor/18381-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15752-6">
    <skos:related rdf:resource="descriptor/11810-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15752-6">
    <skos:related rdf:resource="descriptor/11801-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70858">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12670-2">
    <zbwext:useInsteadNote rdf:nodeID="id196"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11240-4">
    <skos:prefLabel xml:lang="en">Income replacement programme</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11240-4">
    <skos:prefLabel xml:lang="de">Lohnersatzleistungen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14308-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14194-5">
    <skos:broader rdf:resource="thsys/70418"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14194-5">
    <skos:broader rdf:resource="descriptor/14200-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11684-3">
    <skos:altLabel xml:lang="de">Steuerneutralität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11684-3">
    <skos:altLabel xml:lang="de">Neutralität der Besteuerung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11684-3">
    <skos:altLabel xml:lang="de">Grundsatz der Besteuerung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11684-3">
    <skos:altLabel xml:lang="de">Besteuerungsprinzip</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17599-6">
    <skos:broader rdf:resource="thsys/70327"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17599-6">
    <skos:broader rdf:resource="descriptor/17589-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18606-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70471">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19482-3">
    <skos:altLabel xml:lang="en">Stratospheric ozone</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19482-3">
    <skos:altLabel xml:lang="de">Ozonloch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14186-4">
    <skos:related rdf:resource="descriptor/18363-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14186-4">
    <skos:related rdf:resource="descriptor/14496-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14186-4">
    <skos:related rdf:resource="descriptor/14407-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17984-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11710-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18951-6">
    <skos:scopeNote xml:lang="de">Materialbestand in Bibliotheken, Archiven, usw.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13975-3">
    <skos:related rdf:resource="descriptor/13980-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13975-3">
    <skos:related rdf:resource="descriptor/13974-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13975-3">
    <skos:related rdf:resource="descriptor/13307-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13975-3">
    <skos:related rdf:resource="descriptor/13966-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13975-3">
    <skos:related rdf:resource="descriptor/13949-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13975-3">
    <skos:related rdf:resource="descriptor/18560-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19057-4">
    <skos:narrower rdf:resource="descriptor/19058-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19616-1">
    <skos:altLabel xml:lang="en">Continental shelf</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19616-1">
    <skos:altLabel xml:lang="de">Territorialgewässer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19616-1">
    <skos:altLabel xml:lang="de">Seegrenze</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19616-1">
    <skos:altLabel xml:lang="de">Kontinentalsockel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19616-1">
    <skos:altLabel xml:lang="de">Hoheitsgewässer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19616-1">
    <skos:altLabel xml:lang="de">Grenzsee</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19616-1">
    <skos:altLabel xml:lang="de">Festlandsockel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19616-1">
    <skos:altLabel xml:lang="de">Eigengewässer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19616-1">
    <skos:altLabel xml:lang="de">Dreimeilenzone</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10089-1">
    <skos:related rdf:resource="descriptor/10107-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10089-1">
    <skos:related rdf:resource="descriptor/10106-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10089-1">
    <skos:related rdf:resource="descriptor/11165-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10089-1">
    <skos:related rdf:resource="descriptor/15466-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10089-1">
    <skos:related rdf:resource="descriptor/10088-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10089-1">
    <skos:related rdf:resource="descriptor/19692-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11657-6">
    <skos:altLabel xml:lang="de">Gewerbesteuerrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11657-6">
    <skos:altLabel xml:lang="de">Gewerbesteuergesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11657-6">
    <skos:altLabel xml:lang="de">Gewerbekapitalsteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11657-6">
    <skos:altLabel xml:lang="de">Gewerbeertragsteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12224-4">
    <skos:altLabel xml:lang="de">Investitionspolitik des Betriebs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12224-4">
    <skos:altLabel xml:lang="de">Investitionsplanung des Betriebs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12224-4">
    <skos:altLabel xml:lang="de">Investitionsentscheidung des Betriebs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12224-4">
    <skos:altLabel xml:lang="de">Betriebliche Investitionswirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12224-4">
    <skos:altLabel xml:lang="de">Betriebliche Investitionsplanung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12224-4">
    <skos:altLabel xml:lang="de">Betriebliche Investitionsentscheidung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18119-6">
    <skos:related rdf:resource="descriptor/12863-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14189-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18250-1">
    <skos:broader rdf:resource="thsys/70135"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18250-1">
    <skos:broader rdf:resource="descriptor/14473-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11285-3">
    <skos:prefLabel xml:lang="en">White collar workers</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11285-3">
    <skos:prefLabel xml:lang="de">Angestellte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18129-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11657-6">
    <skos:prefLabel xml:lang="en">Business tax</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11657-6">
    <skos:prefLabel xml:lang="de">Gewerbesteuer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11938-5">
    <skos:related rdf:resource="descriptor/19632-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11938-5">
    <skos:related rdf:resource="descriptor/15883-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11938-5">
    <skos:related rdf:resource="descriptor/11939-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14996-4">
    <skos:altLabel xml:lang="de">Zuckerwaren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14996-4">
    <skos:altLabel xml:lang="de">Schokoladenprodukt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14996-4">
    <skos:altLabel xml:lang="de">Schokolade</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14996-4">
    <skos:altLabel xml:lang="de">Marzipan</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14996-4">
    <skos:altLabel xml:lang="de">Kunsthonig</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14996-4">
    <skos:altLabel xml:lang="de">Gummibärchen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15786-3">
    <skos:altLabel xml:lang="en">Learning theory</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15786-3">
    <skos:altLabel xml:lang="de">Lerntheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15786-3">
    <skos:altLabel xml:lang="de">Gedächtnistheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15786-3">
    <skos:altLabel xml:lang="de">Gedächtnispsychologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15786-3">
    <skos:altLabel xml:lang="de">Assoziationspsychologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13804-2">
    <skos:broader rdf:resource="thsys/70114"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13804-2">
    <skos:broader rdf:resource="descriptor/13793-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14255-4">
    <skos:broader rdf:resource="thsys/70348"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19989-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16389-1">
    <skos:related rdf:resource="descriptor/19688-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16389-1">
    <skos:related rdf:resource="descriptor/16422-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16389-1">
    <skos:related rdf:resource="descriptor/16396-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16389-1">
    <skos:related rdf:resource="descriptor/16416-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16389-1">
    <skos:related rdf:resource="descriptor/16264-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16389-1">
    <skos:related rdf:resource="descriptor/18062-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16389-1">
    <skos:related rdf:resource="descriptor/10574-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16389-1">
    <skos:related rdf:resource="descriptor/16426-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16389-1">
    <skos:related rdf:resource="descriptor/16267-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11688-2">
    <skos:altLabel xml:lang="en">Second-best taxation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11688-2">
    <skos:altLabel xml:lang="en">Optimal-taxation theory</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11688-2">
    <skos:altLabel xml:lang="de">Steueroptimum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11688-2">
    <skos:altLabel xml:lang="de">Ramsey-Regel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11688-2">
    <skos:altLabel xml:lang="de">Optimalsteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11688-2">
    <skos:altLabel xml:lang="de">Optimale Steuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71004">
    <rdfs:label xml:lang="de">Industriezweige</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19632-3">
    <skos:prefLabel xml:lang="en">Costal zone management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19632-3">
    <skos:prefLabel xml:lang="de">Küstenschutz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13301-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19555-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15658-5">
    <zbwext:useInsteadNote rdf:nodeID="id209"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19119-1">
    <skos:broader rdf:resource="thsys/70041"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19119-1">
    <skos:broader rdf:resource="descriptor/12111-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19119-1">
    <skos:broader rdf:resource="descriptor/12665-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19158-5">
    <skos:prefLabel xml:lang="en">Continuation of operations</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19158-5">
    <skos:prefLabel xml:lang="de">Betriebsfortführung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12363-4">
    <skos:related rdf:resource="descriptor/12337-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12363-4">
    <skos:related rdf:resource="descriptor/12357-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15941-3">
    <skos:broader rdf:resource="thsys/70814"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15941-3">
    <skos:broader rdf:resource="thsys/70182"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71012">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.04.01.04</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14765-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11573-5">
    <skos:scopeNote xml:lang="de">Staatliche Kriegs- und Opferentschädigungen an ehemalige Kriegsgegner und private internationale Organisationen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11331-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15741-4">
    <skos:altLabel xml:lang="de">Pflanzlicher Rohstoff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15741-4">
    <skos:altLabel xml:lang="de">Landwirtschaftlicher Rohstoff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15741-4">
    <skos:altLabel xml:lang="de">Agrarischer Rohstoff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15741-4">
    <skos:altLabel xml:lang="de">Agrarischer Industrierohstoff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16678-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11564-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14030-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19410-0">
    <skos:altLabel xml:lang="de">Steuerbemessungsgrundlage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19410-0">
    <skos:altLabel xml:lang="de">Bemessungsgrundlage für Steuern</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10698-4">
    <skos:prefLabel xml:lang="en">Trade agreement</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10698-4">
    <skos:prefLabel xml:lang="de">Handelsabkommen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11713-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70007">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16190-3">
    <skos:broader rdf:resource="thsys/73334"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16190-3">
    <skos:broader rdf:resource="descriptor/16201-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11168-0">
    <skos:narrower rdf:resource="descriptor/18125-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11168-0">
    <skos:narrower rdf:resource="descriptor/11177-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16786-5">
    <skos:scopeNote xml:lang="de">Nur für Literatur über Retrievalsysteme, Retrievaloperationen, Kosten der Recherche u.ä.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16696-6">
    <skos:altLabel xml:lang="de">Technologiefolgenabschätzung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16696-6">
    <skos:altLabel xml:lang="de">Technologiebewertung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16696-6">
    <skos:altLabel xml:lang="de">Technikfolgenabschätzung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16696-6">
    <skos:altLabel xml:lang="de">Technikfolgen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17598-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19471-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15672-4">
    <skos:broader rdf:resource="thsys/70017"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71058">
    <rdfs:label xml:lang="de">Berufe</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13229-3">
    <skos:related rdf:resource="descriptor/14539-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13229-3">
    <skos:related rdf:resource="descriptor/13082-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13229-3">
    <skos:related rdf:resource="descriptor/14046-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13229-3">
    <skos:related rdf:resource="descriptor/14619-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12191-0">
    <skos:prefLabel xml:lang="en">General partnership</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12191-0">
    <skos:prefLabel xml:lang="de">Offene Handelsgesellschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12883-6">
    <skos:broader rdf:resource="thsys/71041"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12883-6">
    <skos:broader rdf:resource="thsys/71036"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12883-6">
    <skos:broader rdf:resource="descriptor/12882-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12883-6">
    <skos:broader rdf:resource="descriptor/12165-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10836-4">
    <skos:altLabel xml:lang="de">Ausfuhr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70397">
    <skos:narrower rdf:resource="descriptor/16119-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70397">
    <skos:narrower rdf:resource="descriptor/12855-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70397">
    <skos:narrower rdf:resource="descriptor/12849-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70397">
    <skos:narrower rdf:resource="descriptor/12396-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70397">
    <skos:narrower rdf:resource="descriptor/12874-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70397">
    <skos:narrower rdf:resource="descriptor/16104-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70397">
    <skos:narrower rdf:resource="descriptor/11666-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70397">
    <skos:narrower rdf:resource="descriptor/24703-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70397">
    <skos:narrower rdf:resource="descriptor/12372-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70397">
    <skos:narrower rdf:resource="descriptor/11652-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70397">
    <skos:narrower rdf:resource="descriptor/19082-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70397">
    <skos:narrower rdf:resource="descriptor/16092-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70397">
    <skos:narrower rdf:resource="descriptor/11650-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70397">
    <skos:narrower rdf:resource="descriptor/18302-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70397">
    <skos:narrower rdf:resource="descriptor/12850-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14787-6">
    <skos:broader rdf:resource="thsys/70416"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14787-6">
    <skos:broader rdf:resource="descriptor/14043-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14787-6">
    <skos:broader rdf:resource="descriptor/14788-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11324-5">
    <skos:related rdf:resource="descriptor/11331-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12393-2">
    <skos:altLabel xml:lang="de">Verschmelzungsbilanz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12393-2">
    <skos:altLabel xml:lang="de">Vergleichsbilanz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12393-2">
    <skos:altLabel xml:lang="de">Umwandlungsbilanz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12393-2">
    <skos:altLabel xml:lang="de">Sanierungsbilanz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12393-2">
    <skos:altLabel xml:lang="de">Liquidationsbilanz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12393-2">
    <skos:altLabel xml:lang="de">Konkursbilanz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12393-2">
    <skos:altLabel xml:lang="de">Fusionsbilanz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12393-2">
    <skos:altLabel xml:lang="de">Auseinandersetzungsbilanz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12041-5">
    <skos:broader rdf:resource="thsys/70140"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12041-5">
    <skos:broader rdf:resource="descriptor/10032-2"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id215">
    <rdfs:seeAlso rdf:resource="descriptor/13079-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15115-1">
    <skos:related rdf:resource="descriptor/15111-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15115-1">
    <skos:related rdf:resource="descriptor/18161-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15115-1">
    <skos:related rdf:resource="descriptor/19745-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18357-4">
    <skos:prefLabel xml:lang="en">Local energy industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18357-4">
    <skos:prefLabel xml:lang="de">Kommunale Energiewirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17984-3">
    <skos:prefLabel xml:lang="en">English speaking Africa</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17984-3">
    <skos:prefLabel xml:lang="de">Anglophones Afrika</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18578-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11237-0">
    <skos:altLabel xml:lang="en">Labor administration</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11237-0">
    <skos:altLabel xml:lang="de">Arbeitsmarktverwaltung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11237-0">
    <skos:altLabel xml:lang="de">Arbeitsbehörde</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11237-0">
    <skos:altLabel xml:lang="de">Arbeitsamt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14683-4">
    <skos:prefLabel xml:lang="en">Hydrocarbons</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14683-4">
    <skos:prefLabel xml:lang="de">Kohlenwasserstoffe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70865">
    <skos:narrower rdf:resource="descriptor/16523-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70865">
    <skos:narrower rdf:resource="descriptor/18366-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70865">
    <skos:narrower rdf:resource="descriptor/16024-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70865">
    <skos:narrower rdf:resource="descriptor/16522-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70865">
    <skos:narrower rdf:resource="descriptor/18817-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70865">
    <skos:narrower rdf:resource="descriptor/11560-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70865">
    <skos:narrower rdf:resource="descriptor/13998-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70865">
    <skos:narrower rdf:resource="descriptor/16460-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70865">
    <skos:narrower rdf:resource="descriptor/18814-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70865">
    <skos:narrower rdf:resource="descriptor/19725-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70865">
    <skos:narrower rdf:resource="descriptor/18816-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70865">
    <skos:narrower rdf:resource="descriptor/16684-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70865">
    <skos:narrower rdf:resource="descriptor/16458-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70865">
    <skos:narrower rdf:resource="descriptor/19720-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70865">
    <skos:narrower rdf:resource="descriptor/18815-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70865">
    <skos:narrower rdf:resource="descriptor/16271-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70865">
    <skos:narrower rdf:resource="descriptor/19724-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70865">
    <skos:narrower rdf:resource="descriptor/11582-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70865">
    <skos:narrower rdf:resource="descriptor/19723-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70865">
    <skos:narrower rdf:resource="descriptor/16252-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70865">
    <skos:narrower rdf:resource="descriptor/13412-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70865">
    <skos:narrower rdf:resource="descriptor/16326-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70865">
    <skos:narrower rdf:resource="descriptor/16325-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70865">
    <skos:narrower rdf:resource="descriptor/16455-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18922-6">
    <skos:prefLabel xml:lang="en">Funding of the arts</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18922-6">
    <skos:prefLabel xml:lang="de">Kulturfinanzierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13237-4">
    <skos:prefLabel xml:lang="en">Confectionery trade</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13237-4">
    <skos:prefLabel xml:lang="de">Süßwarenhandel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19191-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15676-3">
    <skos:altLabel xml:lang="de">Natürliche Sprache</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70825">
    <skos:narrower rdf:resource="descriptor/14004-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70825">
    <skos:narrower rdf:resource="descriptor/18449-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70825">
    <skos:narrower rdf:resource="descriptor/16010-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70825">
    <skos:narrower rdf:resource="descriptor/13386-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70825">
    <skos:narrower rdf:resource="descriptor/13661-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70825">
    <skos:narrower rdf:resource="descriptor/13662-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70825">
    <skos:narrower rdf:resource="descriptor/13347-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70825">
    <skos:narrower rdf:resource="descriptor/18932-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70825">
    <skos:narrower rdf:resource="descriptor/13660-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70825">
    <skos:narrower rdf:resource="descriptor/18928-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70825">
    <skos:narrower rdf:resource="descriptor/18930-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70825">
    <skos:narrower rdf:resource="descriptor/16761-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70825">
    <skos:narrower rdf:resource="descriptor/13658-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70825">
    <skos:narrower rdf:resource="descriptor/13382-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70825">
    <skos:narrower rdf:resource="thsys/70824"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18851-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11320-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19596-2">
    <skos:altLabel xml:lang="de">Evolutorische Spieltheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73356">
    <rdfs:label xml:lang="de">Soziale Probleme</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12058-2">
    <skos:altLabel xml:lang="de">Unternehmensstilllegung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12058-2">
    <skos:altLabel xml:lang="de">Unternehmensstillegung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12058-2">
    <skos:altLabel xml:lang="de">Unternehmensliquidation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12058-2">
    <skos:altLabel xml:lang="de">Unternehmensauflösung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12058-2">
    <skos:altLabel xml:lang="de">Stilllegung des Betriebs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12058-2">
    <skos:altLabel xml:lang="de">Stillegung des Betriebs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12058-2">
    <skos:altLabel xml:lang="de">Betriebsstilllegung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12058-2">
    <skos:altLabel xml:lang="de">Betriebsstillegung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12058-2">
    <skos:altLabel xml:lang="de">Betriebsaufgabe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12976-6">
    <skos:prefLabel xml:lang="en">Disposal of radioactive wastes</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12976-6">
    <skos:prefLabel xml:lang="de">Nukleare Entsorgung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19641-2">
    <skos:related rdf:resource="descriptor/19650-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19641-2">
    <skos:related rdf:resource="descriptor/16749-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12354-5">
    <skos:broader rdf:resource="thsys/71043"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12354-5">
    <skos:broader rdf:resource="descriptor/12356-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13486-4">
    <skos:related rdf:resource="descriptor/12821-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13486-4">
    <skos:related rdf:resource="descriptor/18896-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13486-4">
    <skos:related rdf:resource="descriptor/18556-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13486-4">
    <skos:related rdf:resource="descriptor/13200-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15757-3">
    <skos:prefLabel xml:lang="en">Radiation effects</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15757-3">
    <skos:prefLabel xml:lang="de">Strahlenbelastung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18637-5">
    <skos:prefLabel xml:lang="en">Tourism law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18637-5">
    <skos:prefLabel xml:lang="de">Reiserecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17987-4">
    <skos:prefLabel xml:lang="en">SADC countries</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17987-4">
    <skos:prefLabel xml:lang="de">SADC-Staaten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10642-3">
    <skos:altLabel xml:lang="en">Marshall-Lerner condition</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10642-3">
    <skos:altLabel xml:lang="de">Robinson-Bedingung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10642-3">
    <skos:altLabel xml:lang="de">Marshall-Lerner-Stabilitätsbedingung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10642-3">
    <skos:altLabel xml:lang="de">Marshall-Lerner-Bedingung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12217-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73366">
    <skos:broader rdf:resource="thsys/70095"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14188-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16070-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19777-5">
    <skos:scopeNote xml:lang="de">Wissenschaft von den Zeichen (Buchstaben, Zahlen, Symbole, etc.).</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20018-4">
    <skos:related rdf:resource="descriptor/12205-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20018-4">
    <skos:related rdf:resource="descriptor/12262-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20018-4">
    <skos:related rdf:resource="descriptor/12283-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20018-4">
    <skos:related rdf:resource="descriptor/12282-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20018-4">
    <skos:related rdf:resource="descriptor/12050-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15019-4">
    <skos:prefLabel xml:lang="en">Industrial waste</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15019-4">
    <skos:prefLabel xml:lang="de">Industriemüll</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18314-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15806-2">
    <skos:related rdf:resource="descriptor/19733-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15806-2">
    <skos:related rdf:resource="descriptor/15810-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15806-2">
    <skos:related rdf:resource="descriptor/16642-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15806-2">
    <skos:related rdf:resource="descriptor/15807-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15806-2">
    <skos:related rdf:resource="descriptor/16641-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15806-2">
    <skos:related rdf:resource="descriptor/20855-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15806-2">
    <skos:related rdf:resource="descriptor/16709-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14958-5">
    <skos:broader rdf:resource="thsys/70498"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14958-5">
    <skos:broader rdf:resource="descriptor/14044-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14958-5">
    <skos:broader rdf:resource="descriptor/14931-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18552-6">
    <skos:altLabel xml:lang="en">TV shopping</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18552-6">
    <skos:altLabel xml:lang="en">Online shopping</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18552-6">
    <skos:altLabel xml:lang="en">Internet shopping</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70445">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">G.03.01.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14435-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14341-4">
    <skos:altLabel xml:lang="de">Flüssigkeitspumpe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18100-6">
    <skos:related rdf:resource="descriptor/18395-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18100-6">
    <skos:related rdf:resource="descriptor/12916-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18100-6">
    <skos:related rdf:resource="descriptor/12278-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73362">
    <skos:broader rdf:resource="thsys/73366"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19350-6">
    <skos:altLabel xml:lang="de">Assoziierungsabkommen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16962-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13444-6">
    <skos:prefLabel xml:lang="en">Agricultural history</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13444-6">
    <skos:prefLabel xml:lang="de">Agrargeschichte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24855-2">
    <skos:broader rdf:resource="thsys/71099"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24855-2">
    <skos:broader rdf:resource="descriptor/10773-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18517-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70003">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">G.01.01.03</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="http://zbw.eu/namespaces/skos-extensions/useInsteadNote">
    <rdfs:comment xml:lang="en">The note consists of a blank node. It contains 
    a rdfs:value literal which designates the aspect and a rdfs:seeAlso link to 
    the concept which should be used instead.</rdfs:comment>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18594-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14123-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12062-4">
    <skos:altLabel xml:lang="en">Multinational enterprise</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12062-4">
    <skos:altLabel xml:lang="en">Multinational corporation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12062-4">
    <skos:altLabel xml:lang="en">Multinational company</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12062-4">
    <skos:altLabel xml:lang="en">MNE (Multinational enterprise)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12062-4">
    <skos:altLabel xml:lang="en">MNC (Multinational company)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12062-4">
    <skos:altLabel xml:lang="en">Global firm</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12062-4">
    <skos:altLabel xml:lang="en">Global enterprise</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12062-4">
    <skos:altLabel xml:lang="de">Transnationales Unternehmen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12062-4">
    <skos:altLabel xml:lang="de">Multis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12062-4">
    <skos:altLabel xml:lang="de">Multi</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12062-4">
    <skos:altLabel xml:lang="de">Internationales Unternehmen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12062-4">
    <skos:altLabel xml:lang="de">Internationaler Konzern</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12062-4">
    <skos:altLabel xml:lang="de">Globales Unternehmen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18879-2">
    <skos:scopeNote xml:lang="de">Sowohl für die finanzielle Förderung des Studiums z.B. durch Stipendien als auch für die finanzielle Belastung durch das Studium z.B. durch Studiengebühren.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19934-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12922-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12018-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13766-5">
    <skos:narrower rdf:resource="descriptor/18701-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13766-5">
    <skos:narrower rdf:resource="descriptor/18699-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18276-4">
    <skos:broader rdf:resource="thsys/70201"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18276-4">
    <skos:broader rdf:resource="descriptor/14043-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20982-1">
    <skos:prefLabel xml:lang="en">Neo-Ricardian theory</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20982-1">
    <skos:prefLabel xml:lang="de">Neoricardianische Theorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19778-3">
    <skos:related rdf:resource="descriptor/15676-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19778-3">
    <skos:related rdf:resource="descriptor/19608-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19574-5">
    <skos:related rdf:resource="descriptor/15362-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13553-1">
    <skos:related rdf:resource="descriptor/13298-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13553-1">
    <skos:related rdf:resource="descriptor/13537-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13553-1">
    <skos:related rdf:resource="descriptor/14449-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13232-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16601-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11825-4">
    <skos:scopeNote xml:lang="de">Unmittelbar auf das Gemeinwohl ausgerichtete wirtschaftliche Tätigkeit, die sowohl im Rahmen eines pluralistischen als auch eines zentralgesteuerten Wirtschaftssystems ausgeführt werden kann.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18172-2">
    <skos:related rdf:resource="descriptor/18766-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18172-2">
    <skos:related rdf:resource="descriptor/16774-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10566-0">
    <skos:prefLabel xml:lang="en">Export earnings</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10566-0">
    <skos:prefLabel xml:lang="de">Exporterlös</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18539-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10016-0">
    <zbwext:useInsteadNote rdf:nodeID="id263"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25742-3">
    <skos:related rdf:resource="descriptor/12225-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25742-3">
    <skos:related rdf:resource="descriptor/13723-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25742-3">
    <skos:related rdf:resource="descriptor/18085-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10500-2">
    <skos:scopeNote xml:lang="de">Entstehung und Ausbau des industriellen Sektors.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18386-4">
    <skos:prefLabel xml:lang="en">Mining statistics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18386-4">
    <skos:prefLabel xml:lang="de">Bergbaustatistik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15858-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13591-0">
    <skos:related rdf:resource="descriptor/13584-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13591-0">
    <skos:related rdf:resource="descriptor/13612-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18638-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id374">
    <rdf:value xml:lang="de">für Darstellungen der Gesamtwirtschaft</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11518-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11357-4">
    <skos:broader rdf:resource="thsys/70845"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11357-4">
    <skos:broader rdf:resource="descriptor/11377-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13381-4">
    <skos:narrower rdf:resource="descriptor/18787-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19470-3">
    <skos:broader rdf:resource="thsys/71064"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19470-3">
    <skos:broader rdf:resource="descriptor/13303-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11048-3">
    <skos:related rdf:resource="descriptor/11101-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11048-3">
    <skos:related rdf:resource="descriptor/11055-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11048-3">
    <skos:related rdf:resource="descriptor/11036-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18580-1">
    <skos:prefLabel xml:lang="en">Road safety</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18580-1">
    <skos:prefLabel xml:lang="de">Straßenverkehrssicherheit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17984-3">
    <skos:broader rdf:resource="thsys/70307"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17984-3">
    <skos:broader rdf:resource="descriptor/17642-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11539-5">
    <skos:altLabel xml:lang="de">Subventionskontrolle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11539-5">
    <skos:altLabel xml:lang="de">Rechnungsprüfung der öffentlichen Finanzen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11539-5">
    <skos:altLabel xml:lang="de">Parlamentarische Finanzkontrolle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11539-5">
    <skos:altLabel xml:lang="de">Haushaltskontrolle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11539-5">
    <skos:altLabel xml:lang="de">Finanzkontrolle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11539-5">
    <skos:altLabel xml:lang="de">Budgetkontrolle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15243-6">
    <skos:prefLabel xml:lang="en">Sampling</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15243-6">
    <skos:prefLabel xml:lang="de">Stichprobenverfahren</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19280-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70256">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12782-5">
    <skos:broader rdf:resource="thsys/70958"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12782-5">
    <skos:broader rdf:resource="thsys/70956"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12782-5">
    <skos:broader rdf:resource="thsys/70459"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12782-5">
    <skos:broader rdf:resource="descriptor/10940-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12782-5">
    <skos:broader rdf:resource="descriptor/12777-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20321-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11434-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14350-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70498">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">P.20</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20022-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16396-4">
    <skos:broader rdf:resource="thsys/73375"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16396-4">
    <skos:broader rdf:resource="descriptor/16280-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19715-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18073-4">
    <skos:scopeNote xml:lang="de">Ansätze zu einer neuen Wirtschaftsordnung jenseits der arbeitsteiligen kapitalistischen Wirtschaft, auch unter Einschluss ökologischer und entwicklungsökonomischer Konzepte.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18085-4">
    <skos:narrower rdf:resource="descriptor/18672-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18085-4">
    <skos:narrower rdf:resource="descriptor/11441-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18085-4">
    <skos:narrower rdf:resource="descriptor/12243-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18913-0">
    <skos:altLabel xml:lang="de">Finanzierung der Sozialversicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11807-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20756-3">
    <skos:prefLabel xml:lang="en">Discrete choice</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20756-3">
    <skos:prefLabel xml:lang="de">Diskrete Entscheidung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14105-2">
    <skos:related rdf:resource="descriptor/13161-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14105-2">
    <skos:related rdf:resource="descriptor/14948-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14105-2">
    <skos:related rdf:resource="descriptor/19341-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14105-2">
    <skos:related rdf:resource="descriptor/12942-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14105-2">
    <skos:related rdf:resource="descriptor/13225-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19802-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13238-2">
    <skos:altLabel xml:lang="de">Tabakwarenhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16329-5">
    <skos:altLabel xml:lang="en">Federal government</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16329-5">
    <skos:altLabel xml:lang="de">Zentralregierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16329-5">
    <skos:altLabel xml:lang="de">Staatsregierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16329-5">
    <skos:altLabel xml:lang="de">Senatorinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16329-5">
    <skos:altLabel xml:lang="de">Senatoren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16329-5">
    <skos:altLabel xml:lang="de">Ministerrat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16329-5">
    <skos:altLabel xml:lang="de">Ministerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16329-5">
    <skos:altLabel xml:lang="de">Minister</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16329-5">
    <skos:altLabel xml:lang="de">Kabinett</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16329-5">
    <skos:altLabel xml:lang="de">Exekutive</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16329-5">
    <skos:altLabel xml:lang="de">Bundesregierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12073-6">
    <skos:prefLabel xml:lang="en">Economic effectiveness</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12073-6">
    <skos:prefLabel xml:lang="de">Wirtschaftlichkeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20381-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14229-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16268-6">
    <skos:broader rdf:resource="thsys/73375"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16268-6">
    <skos:broader rdf:resource="descriptor/16290-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12590-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10093-3">
    <skos:prefLabel xml:lang="en">Rawls's theory of justice</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10093-3">
    <skos:prefLabel xml:lang="de">Rawls-Gerechtigkeitstheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id364">
    <rdf:value xml:lang="de">Für Marktzutrittsbeschränkungen</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15706-6">
    <skos:broader rdf:resource="thsys/73387"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15706-6">
    <skos:broader rdf:resource="thsys/70109"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15706-6">
    <skos:broader rdf:resource="descriptor/15690-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19415-4">
    <skos:prefLabel xml:lang="en">Flat tax</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19415-4">
    <skos:prefLabel xml:lang="de">Flat Tax</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12425-1">
    <skos:broader rdf:resource="thsys/71043"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12425-1">
    <skos:broader rdf:resource="descriptor/12408-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13454-3">
    <skos:prefLabel xml:lang="en">Rural population</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13454-3">
    <skos:prefLabel xml:lang="de">Landbevölkerung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11494-1">
    <skos:related rdf:resource="descriptor/10772-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11494-1">
    <skos:related rdf:resource="descriptor/10978-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11494-1">
    <skos:related rdf:resource="descriptor/10397-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11494-1">
    <skos:related rdf:resource="descriptor/10392-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11494-1">
    <skos:related rdf:resource="descriptor/10387-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11494-1">
    <skos:related rdf:resource="descriptor/10242-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11494-1">
    <skos:related rdf:resource="descriptor/19547-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11494-1">
    <skos:related rdf:resource="descriptor/11506-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11494-1">
    <skos:related rdf:resource="descriptor/11504-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11494-1">
    <skos:related rdf:resource="descriptor/11479-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11494-1">
    <skos:related rdf:resource="descriptor/10381-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19356-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16587-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15971-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15600-1">
    <skos:prefLabel xml:lang="en">Data acquisition</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15600-1">
    <skos:prefLabel xml:lang="de">Datenerfassung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12261-5">
    <skos:broader rdf:resource="thsys/71106"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12261-5">
    <skos:broader rdf:resource="thsys/71046"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12261-5">
    <skos:broader rdf:resource="descriptor/12232-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19480-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15429-6">
    <skos:broader rdf:resource="thsys/73393"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15429-6">
    <skos:broader rdf:resource="thsys/71035"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15429-6">
    <skos:broader rdf:resource="thsys/70562"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15429-6">
    <skos:broader rdf:resource="thsys/70269"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15429-6">
    <skos:broader rdf:resource="thsys/70011"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15429-6">
    <skos:broader rdf:resource="descriptor/15685-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10797-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15088-6">
    <skos:prefLabel xml:lang="en">Consumer price index</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15088-6">
    <skos:prefLabel xml:lang="de">Lebenshaltungsindex</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70854">
    <skos:prefLabel xml:lang="de">W.23.01.01  Genossenschaftswesen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16532-1">
    <skos:broader rdf:resource="thsys/70458"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16532-1">
    <skos:broader rdf:resource="descriptor/13828-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12521-5">
    <skos:prefLabel xml:lang="en">Cost unit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12521-5">
    <skos:prefLabel xml:lang="de">Kostenstellenrechnung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10978-5">
    <skos:narrower rdf:resource="descriptor/18811-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10978-5">
    <skos:narrower rdf:resource="descriptor/11836-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10978-5">
    <skos:narrower rdf:resource="descriptor/18812-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10978-5">
    <skos:narrower rdf:resource="descriptor/11802-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10978-5">
    <skos:narrower rdf:resource="descriptor/10889-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10021-0">
    <skos:prefLabel xml:lang="en">speculation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10021-0">
    <skos:prefLabel xml:lang="de">Spekulation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18107-6">
    <skos:broader rdf:resource="thsys/71044"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10081-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19609-5">
    <skos:altLabel xml:lang="en">DEA (Data Envelopment Analysis)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19609-5">
    <skos:altLabel xml:lang="de">DE-Analyse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14380-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19523-1">
    <skos:broader rdf:resource="thsys/70530"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19523-1">
    <skos:broader rdf:resource="descriptor/10294-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15694-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16721-5">
    <skos:related rdf:resource="descriptor/15860-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16721-5">
    <skos:related rdf:resource="descriptor/16720-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16721-5">
    <skos:related rdf:resource="descriptor/16704-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12809-4">
    <skos:altLabel xml:lang="en">Quality</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12809-4">
    <skos:altLabel xml:lang="de">Qualität des Produkts</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11582-4">
    <skos:broader rdf:resource="thsys/73363"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11582-4">
    <skos:broader rdf:resource="thsys/70865"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11582-4">
    <skos:broader rdf:resource="thsys/70144"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11582-4">
    <skos:broader rdf:resource="descriptor/16460-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11582-4">
    <skos:broader rdf:resource="descriptor/11579-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11582-4">
    <skos:broader rdf:resource="descriptor/19724-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10361-4">
    <skos:scopeNote xml:lang="de">Nur Literatur über Konjunkturforschung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11258-6">
    <skos:related rdf:resource="descriptor/19491-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11258-6">
    <skos:related rdf:resource="descriptor/19241-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11258-6">
    <skos:related rdf:resource="descriptor/19492-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11258-6">
    <skos:related rdf:resource="descriptor/10613-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11258-6">
    <skos:related rdf:resource="descriptor/18792-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11258-6">
    <skos:related rdf:resource="descriptor/11268-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11258-6">
    <skos:related rdf:resource="descriptor/19902-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13647-6">
    <skos:related rdf:resource="descriptor/18644-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13647-6">
    <skos:related rdf:resource="descriptor/18646-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13647-6">
    <skos:related rdf:resource="descriptor/13270-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14119-5">
    <skos:prefLabel xml:lang="en">Citrus fruit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14119-5">
    <skos:prefLabel xml:lang="de">Zitrusfrucht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19395-5">
    <skos:broader rdf:resource="thsys/70986"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19395-5">
    <skos:broader rdf:resource="thsys/70960"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19395-5">
    <skos:broader rdf:resource="thsys/70867"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12904-3">
    <skos:related rdf:resource="descriptor/13019-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12904-3">
    <skos:related rdf:resource="descriptor/12905-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12904-3">
    <skos:related rdf:resource="descriptor/18495-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12904-3">
    <skos:related rdf:resource="descriptor/13194-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12904-3">
    <skos:related rdf:resource="descriptor/14021-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12904-3">
    <skos:related rdf:resource="descriptor/14366-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70822">
    <skos:prefLabel xml:lang="de">W.30.01  Bibliotheken</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12180-5">
    <skos:related rdf:resource="descriptor/11142-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12180-5">
    <skos:related rdf:resource="descriptor/18841-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12180-5">
    <skos:related rdf:resource="descriptor/18840-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12180-5">
    <skos:related rdf:resource="descriptor/16031-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12180-5">
    <skos:related rdf:resource="descriptor/18838-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15720-5">
    <skos:related rdf:resource="descriptor/11127-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15720-5">
    <skos:related rdf:resource="descriptor/15725-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15720-5">
    <skos:related rdf:resource="descriptor/15723-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15720-5">
    <skos:related rdf:resource="descriptor/15073-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15720-5">
    <skos:related rdf:resource="descriptor/19769-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15720-5">
    <skos:related rdf:resource="descriptor/16290-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15720-5">
    <skos:related rdf:resource="descriptor/19683-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15720-5">
    <skos:related rdf:resource="descriptor/16680-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15720-5">
    <skos:related rdf:resource="descriptor/16594-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15720-5">
    <skos:related rdf:resource="descriptor/10525-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70498">
    <skos:broader rdf:resource="thsys/p"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13480-2">
    <skos:related rdf:resource="descriptor/12793-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20972-4">
    <skos:editorialNote xml:lang="de">In jedem Fall zusätzliche Indexierung mit: Konglomerat.</skos:editorialNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19554-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10604-4">
    <skos:broader rdf:resource="thsys/71091"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10604-4">
    <skos:broader rdf:resource="thsys/70952"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10604-4">
    <skos:broader rdf:resource="descriptor/10012-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70882">
    <rdfs:label xml:lang="en">Data processing industry</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70882">
    <rdfs:label xml:lang="de">Datenverarbeitungswirtschaft</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17949-5">
    <skos:broader rdf:resource="thsys/70128"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17949-5">
    <skos:broader rdf:resource="descriptor/17948-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18918-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14083-0">
    <skos:related rdf:resource="descriptor/19301-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14083-0">
    <skos:related rdf:resource="descriptor/14074-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14083-0">
    <skos:related rdf:resource="descriptor/14068-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14083-0">
    <skos:related rdf:resource="descriptor/14971-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12296-0">
    <skos:broader rdf:resource="thsys/71046"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12296-0">
    <skos:broader rdf:resource="thsys/71043"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12296-0">
    <skos:broader rdf:resource="thsys/70036"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12296-0">
    <skos:broader rdf:resource="descriptor/12276-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12296-0">
    <skos:broader rdf:resource="descriptor/12273-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14122-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10007-1">
    <skos:narrower rdf:resource="descriptor/12278-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18069-2">
    <skos:prefLabel xml:lang="en">Climate change</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18069-2">
    <skos:prefLabel xml:lang="de">Klimaveränderung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12824-1">
    <skos:prefLabel xml:lang="en">Direct-to-consumer advertising</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12824-1">
    <skos:prefLabel xml:lang="de">Direktwerbung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11310-2">
    <skos:related rdf:resource="descriptor/12090-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11310-2">
    <skos:related rdf:resource="descriptor/12580-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11310-2">
    <skos:related rdf:resource="descriptor/16136-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11310-2">
    <skos:related rdf:resource="descriptor/11311-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11310-2">
    <skos:related rdf:resource="descriptor/11185-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11310-2">
    <skos:related rdf:resource="descriptor/11036-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14245-0">
    <skos:related rdf:resource="descriptor/14769-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14214-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16384-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10021-0">
    <skos:broader rdf:resource="thsys/70187"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10021-0">
    <skos:broader rdf:resource="thsys/70036"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10437-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10085-2">
    <skos:prefLabel xml:lang="en">Temporary equilibrium</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10085-2">
    <skos:prefLabel xml:lang="de">Temporäres Gleichgewicht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21101-5">
    <skos:broader rdf:resource="thsys/70305"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21101-5">
    <skos:broader rdf:resource="descriptor/10382-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11911-4">
    <skos:prefLabel xml:lang="en">Regional disparity</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11911-4">
    <skos:prefLabel xml:lang="de">Regionale Disparität</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12012-5">
    <skos:related rdf:resource="descriptor/10027-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10337-1">
    <skos:prefLabel xml:lang="en">Sectoral business cycle</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10337-1">
    <skos:prefLabel xml:lang="de">Branchenkonjunktur</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13587-5">
    <skos:broader rdf:resource="thsys/70942"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13587-5">
    <skos:broader rdf:resource="thsys/70928"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18307-5">
    <skos:prefLabel xml:lang="en">Environmentally friendly packaging</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18307-5">
    <skos:prefLabel xml:lang="de">Umweltorientierte Verpackung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15656-2">
    <skos:related rdf:resource="descriptor/11382-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15656-2">
    <skos:related rdf:resource="descriptor/15666-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15656-2">
    <skos:related rdf:resource="descriptor/15786-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15656-2">
    <skos:related rdf:resource="descriptor/11401-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15656-2">
    <skos:related rdf:resource="descriptor/18292-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15656-2">
    <skos:related rdf:resource="descriptor/11353-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19015-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19137-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11691-6">
    <skos:broader rdf:resource="thsys/71076"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11691-6">
    <skos:broader rdf:resource="descriptor/16623-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11691-6">
    <skos:broader rdf:resource="descriptor/11684-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19074-4">
    <skos:altLabel xml:lang="de">Produktionszeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19074-4">
    <skos:altLabel xml:lang="de">Maschinenlaufzeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19074-4">
    <skos:altLabel xml:lang="de">Fertigungszeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16411-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18018-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14787-6">
    <skos:prefLabel xml:lang="en">Wooden furniture</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14787-6">
    <skos:prefLabel xml:lang="de">Holzmöbel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16794-6">
    <skos:altLabel xml:lang="de">Internationaler Leihverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16794-6">
    <skos:altLabel xml:lang="de">Fernleihe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16794-6">
    <skos:altLabel xml:lang="de">Auswärtiger Leihverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18965-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#editorialNote">
    <skos:definition xml:lang="en">A note for an editor, translator or maintainer of the vocabulary.</skos:definition>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70904">
    <rdfs:label xml:lang="en">Central bank</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70904">
    <rdfs:label xml:lang="de">Zentralbank</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13292-3">
    <skos:broader rdf:resource="thsys/70891"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13292-3">
    <skos:broader rdf:resource="thsys/70839"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13292-3">
    <skos:broader rdf:resource="thsys/70832"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13292-3">
    <skos:broader rdf:resource="thsys/70573"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13292-3">
    <skos:broader rdf:resource="thsys/70458"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13292-3">
    <skos:broader rdf:resource="descriptor/13828-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13785-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16588-2">
    <skos:related rdf:resource="descriptor/16599-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16588-2">
    <skos:related rdf:resource="descriptor/11105-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16588-2">
    <skos:related rdf:resource="descriptor/16597-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16588-2">
    <skos:related rdf:resource="descriptor/16545-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16588-2">
    <skos:related rdf:resource="descriptor/15723-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16588-2">
    <skos:related rdf:resource="descriptor/16384-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16588-2">
    <skos:related rdf:resource="descriptor/16579-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16588-2">
    <skos:related rdf:resource="descriptor/16602-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14033-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10035-3">
    <skos:altLabel xml:lang="en">Dynamic economics</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10035-3">
    <skos:altLabel xml:lang="de">Dynamische Wirtschaftstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14899-2">
    <skos:related rdf:resource="descriptor/13321-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14899-2">
    <skos:related rdf:resource="descriptor/14846-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14899-2">
    <skos:related rdf:resource="descriptor/13141-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14899-2">
    <skos:related rdf:resource="descriptor/18436-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14899-2">
    <skos:related rdf:resource="descriptor/14833-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14899-2">
    <skos:related rdf:resource="descriptor/18295-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14899-2">
    <skos:related rdf:resource="descriptor/13128-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12217-1">
    <skos:broader rdf:resource="thsys/71043"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12217-1">
    <skos:broader rdf:resource="descriptor/12089-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17465-6">
    <skos:prefLabel xml:lang="en">Belarus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17465-6">
    <skos:prefLabel xml:lang="de">Weißrussland</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10850-3">
    <skos:broader rdf:resource="thsys/71045"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10850-3">
    <skos:broader rdf:resource="thsys/70951"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10850-3">
    <skos:broader rdf:resource="descriptor/10854-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12085-6">
    <skos:scopeNote xml:lang="de">Für Literatur über Führungspersonal BENUTZE Führungskräfte bzw. die spezifischeren Deskriptoren.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18552-6">
    <skos:broader rdf:resource="thsys/70947"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18552-6">
    <skos:broader rdf:resource="thsys/70459"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18552-6">
    <skos:broader rdf:resource="descriptor/13492-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18552-6">
    <skos:broader rdf:resource="descriptor/19544-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16379-4">
    <skos:related rdf:resource="descriptor/16226-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16379-4">
    <skos:related rdf:resource="descriptor/13662-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16379-4">
    <skos:related rdf:resource="descriptor/10984-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16379-4">
    <skos:related rdf:resource="descriptor/10968-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16379-4">
    <skos:related rdf:resource="descriptor/16406-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16379-4">
    <skos:related rdf:resource="descriptor/16246-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11650-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16288-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70930">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15869-6">
    <skos:altLabel xml:lang="de">Hochgebirge</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15869-6">
    <skos:altLabel xml:lang="de">Berggebiet</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19258-1">
    <skos:related rdf:resource="descriptor/19259-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19258-1">
    <skos:related rdf:resource="descriptor/11457-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19258-1">
    <skos:related rdf:resource="descriptor/11460-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14919-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14962-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11369-4">
    <skos:related rdf:resource="descriptor/18863-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11369-4">
    <skos:related rdf:resource="descriptor/18861-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11369-4">
    <skos:related rdf:resource="descriptor/11392-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11369-4">
    <skos:related rdf:resource="descriptor/10486-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70201">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13453-5">
    <skos:prefLabel xml:lang="en">Part-time farming</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13453-5">
    <skos:prefLabel xml:lang="de">Nebenerwerbslandwirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18811-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10718-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13165-3">
    <skos:altLabel xml:lang="en">Sugar manufacturing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15381-1">
    <skos:altLabel xml:lang="en">Statistical specification</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15381-1">
    <skos:altLabel xml:lang="en">Statistical identification</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15381-1">
    <skos:altLabel xml:lang="en">Specification error</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15381-1">
    <skos:altLabel xml:lang="en">Specification bias</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15381-1">
    <skos:altLabel xml:lang="en">Soft modelling</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15381-1">
    <skos:altLabel xml:lang="en">Misspecification</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15381-1">
    <skos:altLabel xml:lang="en">Econometric specification</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15381-1">
    <skos:altLabel xml:lang="en">Econometric modelling</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15381-1">
    <skos:altLabel xml:lang="de">Weiche Modellierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15381-1">
    <skos:altLabel xml:lang="de">Spezifikationsfehler</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15381-1">
    <skos:altLabel xml:lang="de">Ökonometrische Spezifikation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15381-1">
    <skos:altLabel xml:lang="de">Ökonometrische Modellierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15381-1">
    <skos:altLabel xml:lang="de">Modellspezifikation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15381-1">
    <skos:altLabel xml:lang="de">Modellierung (Wirtschaftsinformatik)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15381-1">
    <skos:altLabel xml:lang="de">Modellierung (Ökonometrie)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id235">
    <rdf:value xml:lang="de">Für Haushalte mit niedrigem Einkommen</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21110-4">
    <skos:scopeNote xml:lang="de">Alle Schusswaffen zum nicht-militärischen Gebrauch.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10472-2">
    <skos:related rdf:resource="descriptor/18031-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10472-2">
    <skos:related rdf:resource="descriptor/10470-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10472-2">
    <skos:related rdf:resource="descriptor/10598-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10472-2">
    <skos:related rdf:resource="descriptor/10572-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10472-2">
    <skos:related rdf:resource="descriptor/18121-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10472-2">
    <skos:related rdf:resource="descriptor/18082-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10472-2">
    <skos:related rdf:resource="descriptor/10823-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10472-2">
    <skos:related rdf:resource="descriptor/10487-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12667-5">
    <skos:scopeNote xml:lang="de">Entscheidung über die Gestaltung der Fertigungs- bzw. Leistungstiefe.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15801-5">
    <skos:narrower rdf:resource="descriptor/15839-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13970-6">
    <skos:prefLabel xml:lang="en">Sales workers</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13970-6">
    <skos:prefLabel xml:lang="de">Handelsberufe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19502-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14150-4">
    <skos:altLabel xml:lang="en">Pulpwood</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14150-4">
    <skos:altLabel xml:lang="en">Lumber</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14150-4">
    <skos:altLabel xml:lang="de">Rohholz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14150-4">
    <skos:altLabel xml:lang="de">Industrieholz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13315-3">
    <skos:related rdf:resource="descriptor/15949-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19020-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15476-4">
    <skos:altLabel xml:lang="en">Vectorial optimization problem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15476-4">
    <skos:altLabel xml:lang="en">Multiobjective decision making</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15476-4">
    <skos:altLabel xml:lang="en">Multicriteria decision making</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15476-4">
    <skos:altLabel xml:lang="en">MCDM (Multicriteria Decision Making)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15476-4">
    <skos:altLabel xml:lang="en">Goalprogramming</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15476-4">
    <skos:altLabel xml:lang="de">Zielprogrammierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15476-4">
    <skos:altLabel xml:lang="de">Vektormaximummodell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15476-4">
    <skos:altLabel xml:lang="de">Multikriterielles Entscheidungsverfahren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15476-4">
    <skos:altLabel xml:lang="de">Multikriterielles Entscheidungsmodell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15476-4">
    <skos:altLabel xml:lang="de">Multikriterielle Entscheidung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15476-4">
    <skos:altLabel xml:lang="de">Mehrfachzielsetzung bei Entscheidungsmodellen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15476-4">
    <skos:altLabel xml:lang="de">Mehrfachzielentscheidung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15476-4">
    <skos:altLabel xml:lang="de">Entscheidung bei mehrfacher Zielsetzung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19686-1">
    <skos:prefLabel xml:lang="en">Influence of government</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19686-1">
    <skos:prefLabel xml:lang="de">Staatliche Einflussnahme</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19089-5">
    <skos:related rdf:resource="descriptor/10008-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19089-5">
    <skos:related rdf:resource="descriptor/18107-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19089-5">
    <skos:related rdf:resource="descriptor/12677-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71075">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18769-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10623-0">
    <skos:prefLabel xml:lang="en">East West relations</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10623-0">
    <skos:prefLabel xml:lang="de">Ost-West-Beziehungen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12936-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19534-3">
    <zbwext:useInsteadNote rdf:nodeID="id67"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70991">
    <skos:prefLabel xml:lang="en">W.06.01.07  Timber industry and trade</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70991">
    <skos:prefLabel xml:lang="de">W.06.01.07  Holzwirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18020-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18407-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11127-0">
    <skos:altLabel xml:lang="de">Wirtschaftsordnungsreform</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11127-0">
    <skos:altLabel xml:lang="de">Transformationsprozess</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11127-0">
    <skos:altLabel xml:lang="de">Transformationsprozeß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11127-0">
    <skos:altLabel xml:lang="de">Transformationsökonomie (Theorie)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11127-0">
    <skos:altLabel xml:lang="de">Transformation des Wirtschaftssystems</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11127-0">
    <skos:altLabel xml:lang="de">Transformation der Wirtschaftsordnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11127-0">
    <skos:altLabel xml:lang="de">Transformation der Gesellschaftsordnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11127-0">
    <skos:altLabel xml:lang="de">Systemwechsel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11127-0">
    <skos:altLabel xml:lang="de">Systemwandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11127-0">
    <skos:altLabel xml:lang="de">Systemreform</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11127-0">
    <skos:altLabel xml:lang="de">Ordnungsreform</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10948-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10698-4">
    <skos:related rdf:resource="descriptor/19326-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10698-4">
    <skos:related rdf:resource="descriptor/20110-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10698-4">
    <skos:related rdf:resource="descriptor/10785-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10698-4">
    <skos:related rdf:resource="descriptor/19958-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10492-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13784-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18412-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13767-3">
    <skos:prefLabel xml:lang="en">Bill of exchange</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13767-3">
    <skos:prefLabel xml:lang="de">Wechsel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16493-6">
    <skos:prefLabel xml:lang="en">Diplomacy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16493-6">
    <skos:prefLabel xml:lang="de">Diplomatie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19383-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18029-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16192-6">
    <skos:altLabel xml:lang="de">Jugendstrafrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16192-6">
    <skos:altLabel xml:lang="de">Jugenddelinquenz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10471-4">
    <skos:altLabel xml:lang="de">Technologische Prognose</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10471-4">
    <skos:altLabel xml:lang="de">Technikprognose</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15077-4">
    <skos:altLabel xml:lang="de">Preisindex des Einzelhandels</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15077-4">
    <skos:altLabel xml:lang="de">Index der Einzelhandelspreise</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16166-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13422-2">
    <skos:altLabel xml:lang="de">Verfassungsgerichtsbarkeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17638-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18388-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12085-6">
    <skos:related rdf:resource="descriptor/13992-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12085-6">
    <skos:related rdf:resource="descriptor/12090-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12085-6">
    <skos:related rdf:resource="descriptor/12089-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12085-6">
    <skos:related rdf:resource="descriptor/12088-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12085-6">
    <skos:related rdf:resource="descriptor/12108-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12085-6">
    <skos:related rdf:resource="descriptor/18116-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12085-6">
    <skos:related rdf:resource="descriptor/11143-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12085-6">
    <skos:related rdf:resource="descriptor/15483-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12085-6">
    <skos:related rdf:resource="descriptor/16252-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12085-6">
    <skos:related rdf:resource="descriptor/12142-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12085-6">
    <skos:related rdf:resource="descriptor/12120-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12085-6">
    <skos:related rdf:resource="descriptor/11302-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12085-6">
    <skos:related rdf:resource="descriptor/12884-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14779-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18597-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12090-6">
    <skos:altLabel xml:lang="de">Unternehmenssatzung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12759-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18189-6">
    <skos:prefLabel xml:lang="en">Natural rubber</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18189-6">
    <skos:prefLabel xml:lang="de">Naturkautschuk</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13282-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13423-0">
    <skos:related rdf:resource="descriptor/16019-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14888-0">
    <skos:related rdf:resource="descriptor/13148-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11409-4">
    <skos:broader rdf:resource="thsys/71081"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11409-4">
    <skos:broader rdf:resource="thsys/70847"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11409-4">
    <skos:broader rdf:resource="thsys/70176"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11409-4">
    <skos:broader rdf:resource="descriptor/11554-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18390-6">
    <skos:prefLabel xml:lang="en">Mining engineering</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18390-6">
    <skos:prefLabel xml:lang="de">Bergtechnik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19440-5">
    <skos:related rdf:resource="descriptor/11207-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19440-5">
    <skos:related rdf:resource="descriptor/11910-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19440-5">
    <skos:related rdf:resource="descriptor/19224-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19440-5">
    <skos:related rdf:resource="descriptor/11872-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19440-5">
    <skos:related rdf:resource="descriptor/19011-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18177-6">
    <skos:broader rdf:resource="thsys/70903"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18177-6">
    <skos:broader rdf:resource="descriptor/13283-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12942-2">
    <skos:altLabel xml:lang="de">Gemüseanbau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16509-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14889-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11875-3">
    <skos:altLabel xml:lang="en">Spatial pattern</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11875-3">
    <skos:altLabel xml:lang="de">Regionale Versorgungsstruktur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11875-3">
    <skos:altLabel xml:lang="de">Raumstruktur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11875-3">
    <skos:altLabel xml:lang="de">Räumliche Struktur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id149">
    <rdf:value xml:lang="de">Für Varianzen</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10148-4">
    <skos:prefLabel xml:lang="en">Time in economics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10148-4">
    <skos:prefLabel xml:lang="de">Zeitökonomik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16037-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70969">
    <rdfs:label xml:lang="de">Bausparten</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10304-2">
    <skos:altLabel xml:lang="en">Labor intensity</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10304-2">
    <skos:altLabel xml:lang="de">Personalintensität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12207-4">
    <skos:prefLabel xml:lang="en">Mathematics of finance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12207-4">
    <skos:prefLabel xml:lang="de">Finanzmathematik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11664-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15657-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13034-4">
    <skos:related rdf:resource="descriptor/12925-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13034-4">
    <skos:related rdf:resource="descriptor/18402-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13034-4">
    <skos:related rdf:resource="descriptor/14269-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13034-4">
    <skos:related rdf:resource="descriptor/19184-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13034-4">
    <skos:related rdf:resource="descriptor/18401-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13034-4">
    <skos:related rdf:resource="descriptor/13224-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13034-4">
    <skos:related rdf:resource="descriptor/18400-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13034-4">
    <skos:related rdf:resource="descriptor/14382-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13034-4">
    <skos:related rdf:resource="descriptor/14266-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11754-1">
    <skos:altLabel xml:lang="de">Rente (Differentialrente)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11754-1">
    <skos:altLabel xml:lang="de">Quasirente</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11754-1">
    <skos:altLabel xml:lang="de">Grundrententheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11754-1">
    <skos:altLabel xml:lang="de">Differenzialrententheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11754-1">
    <skos:altLabel xml:lang="de">Differenzialrente</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11754-1">
    <skos:altLabel xml:lang="de">Differentialrententheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11754-1">
    <skos:altLabel xml:lang="de">Differentialrente</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14293-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10490-0">
    <skos:prefLabel xml:lang="en">Irrigation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10490-0">
    <skos:prefLabel xml:lang="de">Bewässerung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19464-5">
    <zbwext:useInsteadNote rdf:nodeID="id187"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12585-0">
    <skos:prefLabel xml:lang="en">Personnel management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12585-0">
    <skos:prefLabel xml:lang="de">Personalführung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14995-6">
    <skos:related rdf:resource="descriptor/14953-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13007-0">
    <skos:broader rdf:resource="thsys/70989"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13007-0">
    <skos:broader rdf:resource="descriptor/13004-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12521-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70925">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16033-5">
    <skos:broader rdf:resource="thsys/70884"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16033-5">
    <skos:broader rdf:resource="descriptor/12285-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14102-1">
    <skos:broader rdf:resource="thsys/70169"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14102-1">
    <skos:broader rdf:resource="descriptor/14110-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19504-5">
    <skos:prefLabel xml:lang="en">Night work</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19504-5">
    <skos:prefLabel xml:lang="de">Nachtarbeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11320-6">
    <skos:prefLabel xml:lang="en">Salary</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11320-6">
    <skos:prefLabel xml:lang="de">Gehalt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19123-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15900-3">
    <zbwext:useInsteadNote rdf:nodeID="id361"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14525-1">
    <skos:narrower rdf:resource="descriptor/14583-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14525-1">
    <skos:narrower rdf:resource="descriptor/14524-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14525-1">
    <skos:narrower rdf:resource="descriptor/14515-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18740-5">
    <skos:broader rdf:resource="thsys/70891"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18740-5">
    <skos:broader rdf:resource="descriptor/13810-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18869-5">
    <skos:narrower rdf:resource="descriptor/10472-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18869-5">
    <skos:narrower rdf:resource="descriptor/10491-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10418-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19476-5">
    <skos:related rdf:resource="descriptor/15695-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19476-5">
    <skos:related rdf:resource="descriptor/11987-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19476-5">
    <skos:related rdf:resource="descriptor/11970-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10110-1">
    <skos:narrower rdf:resource="descriptor/11741-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10110-1">
    <skos:narrower rdf:resource="descriptor/11037-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19003-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15473-3">
    <skos:narrower rdf:resource="descriptor/16786-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13188-5">
    <skos:broader rdf:resource="thsys/71028"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13188-5">
    <skos:broader rdf:resource="thsys/70960"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13188-5">
    <skos:broader rdf:resource="descriptor/13210-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10361-4">
    <skos:prefLabel xml:lang="en">Business cycle research</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10361-4">
    <skos:prefLabel xml:lang="de">Konjunkturforschung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19442-1">
    <skos:scopeNote xml:lang="de">Wärmelehre der Physik</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16092-3">
    <skos:narrower rdf:resource="descriptor/16119-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16092-3">
    <skos:narrower rdf:resource="descriptor/12850-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70080">
    <rdfs:label xml:lang="en">North America</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70080">
    <rdfs:label xml:lang="de">Nordamerika</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14516-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11032-4">
    <skos:altLabel xml:lang="de">Statistik des Haushaltsbudgets</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11032-4">
    <skos:altLabel xml:lang="de">Statistik des Familienbudgets</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11032-4">
    <skos:altLabel xml:lang="de">Familienbudget-Statistik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11032-4">
    <skos:altLabel xml:lang="de">Einkommens- und Verbrauchsstichprobe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19198-0">
    <skos:broader rdf:resource="thsys/70120"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19198-0">
    <skos:broader rdf:resource="thsys/70049"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19198-0">
    <skos:broader rdf:resource="thsys/70038"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12083-3">
    <skos:related rdf:resource="descriptor/18117-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12083-3">
    <skos:related rdf:resource="descriptor/18029-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12083-3">
    <skos:related rdf:resource="descriptor/18184-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12083-3">
    <skos:related rdf:resource="descriptor/12079-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12083-3">
    <skos:related rdf:resource="descriptor/18106-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12083-3">
    <skos:related rdf:resource="descriptor/10192-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18144-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18639-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18818-1">
    <skos:related rdf:resource="descriptor/19858-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18818-1">
    <skos:related rdf:resource="descriptor/18182-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18818-1">
    <skos:related rdf:resource="descriptor/19723-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10148-4">
    <skos:related rdf:resource="descriptor/19202-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10148-4">
    <skos:related rdf:resource="descriptor/19784-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10148-4">
    <skos:related rdf:resource="descriptor/10045-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10148-4">
    <skos:related rdf:resource="descriptor/10077-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10148-4">
    <skos:related rdf:resource="descriptor/19209-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10148-4">
    <skos:related rdf:resource="descriptor/18917-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11973-3">
    <skos:broader rdf:resource="thsys/71064"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11973-3">
    <skos:broader rdf:resource="descriptor/11970-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13704-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18773-4">
    <skos:altLabel xml:lang="de">EU-Technologiepolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18773-4">
    <skos:altLabel xml:lang="de">Europäischer Forschungsraum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18773-4">
    <skos:altLabel xml:lang="de">EU-Forschungspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17991-6">
    <skos:broader rdf:resource="thsys/70307"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17991-6">
    <skos:broader rdf:resource="descriptor/17999-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70901">
    <rdfs:label xml:lang="en">Investment fund</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70901">
    <rdfs:label xml:lang="de">Investmentfonds</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18552-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19812-1">
    <skos:prefLabel xml:lang="en">International labour legislation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19812-1">
    <skos:prefLabel xml:lang="de">Internationales Arbeitsrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73374">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19642-0">
    <skos:broader rdf:resource="thsys/73358"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19642-0">
    <skos:broader rdf:resource="thsys/70142"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19642-0">
    <skos:broader rdf:resource="descriptor/15918-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10136-4">
    <skos:prefLabel xml:lang="en">Theory of the term structure</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10136-4">
    <skos:prefLabel xml:lang="de">Zinsstrukturtheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13341-2">
    <skos:broader rdf:resource="thsys/70823"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13341-2">
    <skos:broader rdf:resource="descriptor/19475-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11441-1">
    <skos:broader rdf:resource="thsys/70905"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11441-1">
    <skos:broader rdf:resource="thsys/70187"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11441-1">
    <skos:broader rdf:resource="descriptor/18085-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12281-6">
    <skos:broader rdf:resource="thsys/70901"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12281-6">
    <skos:broader rdf:resource="descriptor/13291-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16550-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16046-3">
    <skos:broader rdf:resource="thsys/73390"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16046-3">
    <skos:broader rdf:resource="thsys/70904"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16046-3">
    <skos:broader rdf:resource="thsys/70305"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16046-3">
    <skos:broader rdf:resource="descriptor/11417-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16090-0">
    <skos:related rdf:resource="descriptor/11703-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16090-0">
    <skos:related rdf:resource="descriptor/16048-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16977-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id92">
    <rdf:value xml:lang="de">Für Erwerbstätigkeit im Sinne des Beschäftigungsvolumens</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11905-6">
    <zbwext:useInsteadNote rdf:nodeID="id51"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10736-1">
    <skos:broader rdf:resource="thsys/71094"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10736-1">
    <skos:broader rdf:resource="descriptor/19335-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13702-3">
    <skos:related rdf:resource="descriptor/19273-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13702-3">
    <skos:related rdf:resource="descriptor/18323-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13702-3">
    <skos:related rdf:resource="descriptor/13287-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13702-3">
    <skos:related rdf:resource="descriptor/18673-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13702-3">
    <skos:related rdf:resource="descriptor/13430-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17610-2">
    <skos:altLabel xml:lang="de">Islamische Republik Pakistan</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19260-0">
    <skos:related rdf:resource="descriptor/10772-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19260-0">
    <skos:related rdf:resource="descriptor/18127-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19260-0">
    <skos:related rdf:resource="descriptor/10764-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19260-0">
    <skos:related rdf:resource="descriptor/11430-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19260-0">
    <skos:related rdf:resource="descriptor/10382-3"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id110">
    <rdf:value xml:lang="de">Für die Wirtschaftsentwicklung von Gemeinden</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70463">
    <skos:narrower rdf:resource="descriptor/18253-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70463">
    <skos:narrower rdf:resource="descriptor/14634-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70463">
    <skos:narrower rdf:resource="descriptor/18254-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70463">
    <skos:narrower rdf:resource="descriptor/14654-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70463">
    <skos:narrower rdf:resource="descriptor/14653-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70463">
    <skos:narrower rdf:resource="descriptor/14652-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70463">
    <skos:narrower rdf:resource="descriptor/14651-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70463">
    <skos:narrower rdf:resource="descriptor/14627-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70463">
    <skos:narrower rdf:resource="descriptor/14205-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70463">
    <skos:narrower rdf:resource="descriptor/14646-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70463">
    <skos:narrower rdf:resource="descriptor/19486-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70463">
    <skos:narrower rdf:resource="descriptor/14645-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70463">
    <skos:narrower rdf:resource="descriptor/14643-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70463">
    <skos:narrower rdf:resource="descriptor/14608-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70463">
    <skos:narrower rdf:resource="descriptor/14640-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70463">
    <skos:narrower rdf:resource="descriptor/18080-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70463">
    <skos:narrower rdf:resource="descriptor/14642-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70463">
    <skos:narrower rdf:resource="descriptor/14636-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70463">
    <skos:narrower rdf:resource="descriptor/18255-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70463">
    <skos:narrower rdf:resource="descriptor/14292-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14087-6">
    <skos:broader rdf:resource="thsys/70070"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14087-6">
    <skos:broader rdf:resource="descriptor/14088-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19656-3">
    <zbwext:useInsteadNote rdf:nodeID="id35"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20241-1">
    <skos:related rdf:resource="descriptor/19809-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20241-1">
    <skos:related rdf:resource="descriptor/18552-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20241-1">
    <skos:related rdf:resource="descriptor/18654-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20241-1">
    <skos:related rdf:resource="descriptor/18112-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20241-1">
    <skos:related rdf:resource="descriptor/24782-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20241-1">
    <skos:related rdf:resource="descriptor/19544-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20241-1">
    <skos:related rdf:resource="descriptor/19541-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20241-1">
    <skos:related rdf:resource="descriptor/19428-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14583-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id245">
    <rdf:value xml:lang="de">für nicht-erwerbswirtschaftlich ausgerichtete Organisationen</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18610-4">
    <skos:broader rdf:resource="thsys/70939"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18610-4">
    <skos:broader rdf:resource="thsys/70926"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18610-4">
    <skos:broader rdf:resource="thsys/70925"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18610-4">
    <skos:broader rdf:resource="descriptor/11833-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19407-3">
    <skos:broader rdf:resource="thsys/71074"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19407-3">
    <skos:broader rdf:resource="descriptor/11547-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19195-6">
    <skos:prefLabel xml:lang="en">Bottleneck</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19195-6">
    <skos:prefLabel xml:lang="de">Bottleneck</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13766-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19138-4">
    <skos:altLabel xml:lang="de">Tauschring</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19138-4">
    <skos:altLabel xml:lang="de">Tauschkreis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19138-4">
    <skos:altLabel xml:lang="de">Tauschbörse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19268-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14752-4">
    <skos:related rdf:resource="descriptor/15605-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14752-4">
    <skos:related rdf:resource="descriptor/14513-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71114">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15473-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70936">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21082-4">
    <skos:narrower rdf:resource="descriptor/11692-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21082-4">
    <skos:narrower rdf:resource="descriptor/11907-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21082-4">
    <skos:narrower rdf:resource="descriptor/11713-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18359-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15078-2">
    <skos:broader rdf:resource="thsys/72184"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15078-2">
    <skos:broader rdf:resource="thsys/71091"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15078-2">
    <skos:broader rdf:resource="thsys/70952"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15078-2">
    <skos:broader rdf:resource="descriptor/15093-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15078-2">
    <skos:broader rdf:resource="descriptor/10835-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16771-4">
    <skos:related rdf:resource="descriptor/13351-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13794-0">
    <skos:prefLabel xml:lang="en">Custody services</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13794-0">
    <skos:prefLabel xml:lang="de">Depotgeschäft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19349-5">
    <skos:prefLabel xml:lang="en">Wool market</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19349-5">
    <skos:prefLabel xml:lang="de">Wollmarkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13249-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70650">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">P.04.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11805-3">
    <skos:related rdf:resource="descriptor/11800-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17480-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14204-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15135-2">
    <skos:related rdf:resource="descriptor/15124-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71040">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">B.08.03</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12244-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16606-5">
    <skos:prefLabel xml:lang="en">Postindustrial society</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16606-5">
    <skos:prefLabel xml:lang="de">Postindustrielle Gesellschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15643-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10501-0">
    <skos:related rdf:resource="descriptor/10512-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10501-0">
    <skos:related rdf:resource="descriptor/10511-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10501-0">
    <skos:related rdf:resource="descriptor/16650-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10501-0">
    <skos:related rdf:resource="descriptor/10018-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10501-0">
    <skos:related rdf:resource="descriptor/16672-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10501-0">
    <skos:related rdf:resource="descriptor/10522-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10501-0">
    <skos:related rdf:resource="descriptor/18073-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11661-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10082-1">
    <skos:prefLabel xml:lang="en">Theory of rationing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10082-1">
    <skos:prefLabel xml:lang="de">Mengenrationierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11049-1">
    <skos:scopeNote xml:lang="de">Konvergenz von politischen und/oder Wirtschaftssystemen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10450-5">
    <skos:prefLabel xml:lang="en">Basic research</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10450-5">
    <skos:prefLabel xml:lang="de">Grundlagenforschung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13387-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70549">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11123-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11252-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10208-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70901">
    <skos:narrower rdf:resource="descriptor/13709-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70901">
    <skos:narrower rdf:resource="descriptor/16061-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70901">
    <skos:narrower rdf:resource="descriptor/12281-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70901">
    <skos:narrower rdf:resource="descriptor/13704-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70901">
    <skos:narrower rdf:resource="descriptor/24740-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70901">
    <skos:narrower rdf:resource="descriptor/18088-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70901">
    <skos:narrower rdf:resource="descriptor/13699-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18962-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17989-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14947-3">
    <skos:altLabel xml:lang="de">Trockenobst</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14947-3">
    <skos:altLabel xml:lang="de">Trockengemüse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14947-3">
    <skos:altLabel xml:lang="de">Trockenfrucht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14947-3">
    <skos:altLabel xml:lang="de">Sauerkonserve</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14947-3">
    <skos:altLabel xml:lang="de">Obstkonserve</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14947-3">
    <skos:altLabel xml:lang="de">Konserviertes Gemüse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14947-3">
    <skos:altLabel xml:lang="de">Gemüsekonserve</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14947-3">
    <skos:altLabel xml:lang="de">Dosenobst</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14947-3">
    <skos:altLabel xml:lang="de">Dosengemüse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18861-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18683-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14397-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15372-2">
    <skos:altLabel xml:lang="en">Nonlinear model</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15372-2">
    <skos:altLabel xml:lang="en">Nonlinear econometrics</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15372-2">
    <skos:altLabel xml:lang="en">Nonlinear analysis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15372-2">
    <skos:altLabel xml:lang="de">Nichtlineare Statistik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15372-2">
    <skos:altLabel xml:lang="de">Nichtlineare Analyse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19771-3">
    <skos:broader rdf:resource="thsys/70017"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19771-3">
    <skos:broader rdf:resource="descriptor/19599-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12438-6">
    <skos:prefLabel xml:lang="en">Company valuation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12438-6">
    <skos:prefLabel xml:lang="de">Unternehmensbewertung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16960-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13185-4">
    <skos:altLabel xml:lang="de">Wasserinstallation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13185-4">
    <skos:altLabel xml:lang="de">SHK-Handwerk</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13185-4">
    <skos:altLabel xml:lang="de">Sanitärinstallation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13185-4">
    <skos:altLabel xml:lang="de">Sanitär-Heizungs-Klima-Handwerk</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13185-4">
    <skos:altLabel xml:lang="de">Lüftungsinstallation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13185-4">
    <skos:altLabel xml:lang="de">Klimatechnikinstallation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13185-4">
    <skos:altLabel xml:lang="de">Heizungsinstallation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13185-4">
    <skos:altLabel xml:lang="de">Gasinstallation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10973-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18683-5">
    <skos:prefLabel xml:lang="en">Portfolio insurance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18683-5">
    <skos:prefLabel xml:lang="de">Portfolio-Insurance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18132-0">
    <skos:prefLabel xml:lang="en">Brandenburg</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18132-0">
    <skos:prefLabel xml:lang="de">Brandenburg</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19023-0">
    <skos:prefLabel xml:lang="en">Whale</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19023-0">
    <skos:prefLabel xml:lang="de">Wal</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70183">
    <skos:narrower rdf:resource="descriptor/12262-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70183">
    <skos:narrower rdf:resource="descriptor/20018-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70183">
    <skos:narrower rdf:resource="descriptor/12244-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70183">
    <skos:narrower rdf:resource="descriptor/12225-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70183">
    <skos:narrower rdf:resource="descriptor/24733-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70183">
    <skos:narrower rdf:resource="descriptor/19079-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70183">
    <skos:narrower rdf:resource="descriptor/19540-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70183">
    <skos:narrower rdf:resource="descriptor/10798-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70183">
    <skos:narrower rdf:resource="thsys/71045"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70183">
    <skos:narrower rdf:resource="thsys/71046"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18224-2">
    <skos:altLabel xml:lang="x-other">Cesko</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18224-2">
    <skos:altLabel xml:lang="x-other">Ceska Republika</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18224-2">
    <skos:altLabel xml:lang="de">Tschechien</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18224-2">
    <skos:altLabel xml:lang="de">Tschechei</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11859-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14249-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71071">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.10.03</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15706-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11432-2">
    <skos:prefLabel xml:lang="en">Purchasing power</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11432-2">
    <skos:prefLabel xml:lang="de">Kaufkraft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11975-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id162">
    <rdf:value xml:lang="de">Für gesamtwirtschaftlichen Kapitalbedarf</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18493-3">
    <skos:altLabel xml:lang="de">Umweltverträgliches Bauen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18493-3">
    <skos:altLabel xml:lang="de">Umweltschonendes Bauen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18493-3">
    <skos:altLabel xml:lang="de">Umweltfreundliches Bauen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18493-3">
    <skos:altLabel xml:lang="de">Ökologisches Bauen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11311-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16330-6">
    <skos:broader rdf:resource="thsys/73380"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16330-6">
    <skos:broader rdf:resource="descriptor/16329-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14104-4">
    <skos:broader rdf:resource="thsys/70169"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14104-4">
    <skos:broader rdf:resource="descriptor/14074-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12570-6">
    <skos:narrower rdf:resource="descriptor/12569-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18983-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19250-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11407-1">
    <skos:narrower rdf:resource="descriptor/11222-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11407-1">
    <skos:narrower rdf:resource="descriptor/19516-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18935-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18880-3">
    <skos:broader rdf:resource="thsys/70844"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18880-3">
    <skos:broader rdf:resource="thsys/70175"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18880-3">
    <skos:broader rdf:resource="descriptor/11373-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16805-6">
    <skos:prefLabel xml:lang="en">Antarctica</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16805-6">
    <skos:prefLabel xml:lang="de">Antarktis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70274">
    <rdfs:label xml:lang="de">Neue Politische Ökonomie</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18888-1">
    <skos:broader rdf:resource="thsys/72181"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18888-1">
    <skos:broader rdf:resource="thsys/70847"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18888-1">
    <skos:broader rdf:resource="descriptor/18887-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10826-0">
    <skos:broader rdf:resource="thsys/71090"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10826-0">
    <skos:broader rdf:resource="thsys/71040"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12525-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13352-4">
    <skos:prefLabel xml:lang="en">Film industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13352-4">
    <skos:prefLabel xml:lang="de">Filmwirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14915-2">
    <skos:related rdf:resource="descriptor/14916-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13011-2">
    <skos:prefLabel xml:lang="en">Glass industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13011-2">
    <skos:prefLabel xml:lang="de">Glasindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10960-3">
    <skos:altLabel xml:lang="de">Preiskontrolle des Wettbewerbsrechts</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10960-3">
    <skos:altLabel xml:lang="de">Monopol-Preiskontrolle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#ConceptScheme">
    <skos:definition xml:lang="en">A set of concepts, optionally including statements about semantic
      relationships between those concepts.</skos:definition>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11957-1">
    <skos:prefLabel xml:lang="en">Cartography</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11957-1">
    <skos:prefLabel xml:lang="de">Kartographie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70923">
    <skos:broader rdf:resource="thsys/w"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19805-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10236-0">
    <skos:related rdf:resource="descriptor/10772-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10236-0">
    <skos:related rdf:resource="descriptor/10421-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10236-0">
    <skos:related rdf:resource="descriptor/10676-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10236-0">
    <skos:related rdf:resource="descriptor/10382-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10236-0">
    <skos:related rdf:resource="descriptor/11415-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10274-6">
    <skos:related rdf:resource="descriptor/19589-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18650-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18560-0">
    <zbwext:useInsteadNote rdf:nodeID="id62"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17621-4">
    <skos:altLabel xml:lang="de">Republik Irak</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13141-3">
    <skos:altLabel xml:lang="de">Schneiderhandwerk</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13141-3">
    <skos:altLabel xml:lang="de">Bekleidungshandwerk</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19214-0">
    <skos:prefLabel xml:lang="en">Incomplete contract</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19214-0">
    <skos:prefLabel xml:lang="de">Unvollständiger Vertrag</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70383">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70029">
    <skos:broader rdf:resource="thsys/70030"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11157-5">
    <skos:scopeNote xml:lang="de">Volkswirtschaftliche Untersuchungen des Gesundheitswesens.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11382-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19791-4">
    <skos:related rdf:resource="descriptor/12620-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24706-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14073-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71068">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21061-5">
    <skos:broader rdf:resource="thsys/70184"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21061-5">
    <skos:broader rdf:resource="descriptor/12356-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16187-6">
    <skos:scopeNote xml:lang="de">Gesamtheit der nationalen und internationalen Normen, die das Recht der See und der Seeschifffahrt betreffen. Das Seevölkerrecht regelt die Fragen des Meeresbergbaus, der Meeresverschmutzung, der Seenot, des Anlaufens fremder Häfen, der Abgrenzung von Küstengewässern etc.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14824-5">
    <skos:altLabel xml:lang="de">Luftschlauch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14824-5">
    <skos:altLabel xml:lang="de">Kraftfahrzeugreifen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14824-5">
    <skos:altLabel xml:lang="de">Kfz-Reifen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14824-5">
    <skos:altLabel xml:lang="de">Autoreifen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19091-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16717-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19787-2">
    <skos:prefLabel xml:lang="en">Hygiene</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19787-2">
    <skos:prefLabel xml:lang="de">Hygiene</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11270-2">
    <skos:related rdf:resource="descriptor/10342-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11270-2">
    <skos:related rdf:resource="descriptor/18308-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11270-2">
    <skos:related rdf:resource="descriptor/20381-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11270-2">
    <skos:related rdf:resource="descriptor/12700-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11270-2">
    <skos:related rdf:resource="descriptor/11266-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11270-2">
    <skos:related rdf:resource="descriptor/18168-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11270-2">
    <skos:related rdf:resource="descriptor/11183-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18505-1">
    <skos:prefLabel xml:lang="en">Construction investment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18505-1">
    <skos:prefLabel xml:lang="de">Bauinvestition</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13851-0">
    <skos:broader rdf:resource="thsys/70970"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13851-0">
    <skos:broader rdf:resource="descriptor/10011-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16336-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12120-2">
    <skos:prefLabel xml:lang="en">Governance structure</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12120-2">
    <skos:prefLabel xml:lang="de">Führungsorganisation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11778-1">
    <skos:narrower rdf:resource="descriptor/18752-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11778-1">
    <skos:narrower rdf:resource="descriptor/11682-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11778-1">
    <skos:narrower rdf:resource="descriptor/11777-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11778-1">
    <skos:narrower rdf:resource="descriptor/11571-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11778-1">
    <skos:narrower rdf:resource="descriptor/11566-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11778-1">
    <skos:narrower rdf:resource="descriptor/11772-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11778-1">
    <skos:narrower rdf:resource="descriptor/11769-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10286-6">
    <skos:related rdf:resource="descriptor/10251-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10286-6">
    <skos:related rdf:resource="descriptor/10291-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19857-0">
    <skos:prefLabel xml:lang="en">Veto</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19857-0">
    <skos:prefLabel xml:lang="de">Veto</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18772-6">
    <skos:narrower rdf:resource="descriptor/18627-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18772-6">
    <skos:narrower rdf:resource="descriptor/19727-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18772-6">
    <skos:narrower rdf:resource="descriptor/18909-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18772-6">
    <skos:narrower rdf:resource="descriptor/18773-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18772-6">
    <skos:narrower rdf:resource="descriptor/11937-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18772-6">
    <skos:narrower rdf:resource="descriptor/18341-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18772-6">
    <skos:narrower rdf:resource="descriptor/18370-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18772-6">
    <skos:narrower rdf:resource="descriptor/18889-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12216-3">
    <skos:broader rdf:resource="thsys/71043"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12216-3">
    <skos:broader rdf:resource="descriptor/19108-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16721-5">
    <skos:prefLabel xml:lang="en">Rationalism</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16721-5">
    <skos:prefLabel xml:lang="de">Rationalismus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10587-6">
    <skos:altLabel xml:lang="de">Nord-Süd-Handel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10587-6">
    <skos:altLabel xml:lang="de">Nord-Süd Handel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10587-6">
    <skos:altLabel xml:lang="de">Nord-Süd Beziehungen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73391">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.05.02.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19149-6">
    <skos:broader rdf:resource="thsys/72183"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19149-6">
    <skos:broader rdf:resource="thsys/71035"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19149-6">
    <skos:broader rdf:resource="descriptor/12774-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12280-1">
    <skos:altLabel xml:lang="de">Geldanlage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12280-1">
    <skos:altLabel xml:lang="de">Finanzinvestition</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12993-6">
    <skos:narrower rdf:resource="descriptor/12930-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12993-6">
    <skos:narrower rdf:resource="descriptor/13020-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12993-6">
    <skos:narrower rdf:resource="descriptor/13083-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12993-6">
    <skos:narrower rdf:resource="descriptor/18094-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12993-6">
    <skos:narrower rdf:resource="descriptor/13097-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12993-6">
    <skos:narrower rdf:resource="descriptor/13096-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12993-6">
    <skos:narrower rdf:resource="descriptor/13142-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12993-6">
    <skos:narrower rdf:resource="descriptor/13095-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12993-6">
    <skos:narrower rdf:resource="descriptor/12925-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12993-6">
    <skos:narrower rdf:resource="descriptor/13117-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12993-6">
    <skos:narrower rdf:resource="descriptor/13081-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12993-6">
    <skos:narrower rdf:resource="descriptor/13003-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12993-6">
    <skos:narrower rdf:resource="descriptor/13160-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12993-6">
    <skos:narrower rdf:resource="descriptor/12921-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12993-6">
    <skos:narrower rdf:resource="descriptor/13034-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12993-6">
    <skos:narrower rdf:resource="descriptor/18396-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12993-6">
    <skos:narrower rdf:resource="descriptor/13058-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12993-6">
    <skos:narrower rdf:resource="descriptor/13136-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12993-6">
    <skos:narrower rdf:resource="descriptor/13006-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12993-6">
    <skos:narrower rdf:resource="descriptor/12918-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12993-6">
    <skos:narrower rdf:resource="descriptor/24708-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12993-6">
    <skos:narrower rdf:resource="descriptor/12916-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12993-6">
    <skos:narrower rdf:resource="descriptor/10464-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12993-6">
    <skos:narrower rdf:resource="descriptor/13078-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12993-6">
    <skos:narrower rdf:resource="descriptor/13047-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12993-6">
    <skos:narrower rdf:resource="descriptor/13076-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12993-6">
    <skos:narrower rdf:resource="descriptor/12996-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10692-2">
    <skos:altLabel xml:lang="en">Voluntary export restraint</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10692-2">
    <skos:altLabel xml:lang="en">VER (Voluntary Export Restraint)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10692-2">
    <skos:altLabel xml:lang="de">Freiwillige Selbstbeschränkung des Exports</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10692-2">
    <skos:altLabel xml:lang="de">Exportrestriktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10692-2">
    <skos:altLabel xml:lang="de">Exportkontrolle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10692-2">
    <skos:altLabel xml:lang="de">Ausfuhrbeschränkung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19495-1">
    <skos:prefLabel xml:lang="en">Theory of unemployment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19495-1">
    <skos:prefLabel xml:lang="de">Theorie der Arbeitslosigkeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18076-5">
    <skos:related rdf:resource="descriptor/15396-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15060-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19431-6">
    <skos:related rdf:resource="descriptor/18000-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19431-6">
    <skos:related rdf:resource="descriptor/19678-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18880-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16155-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10606-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17464-1">
    <skos:prefLabel xml:lang="en">Armenia</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17464-1">
    <skos:prefLabel xml:lang="de">Armenien</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id175">
    <rdf:value xml:lang="de">Für Pkw</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25822-5">
    <skos:scopeNote xml:lang="de">Werte- und normengeleitete Wahrnehmung sozialer und ökologischer Verantwortung durch Unternehmen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13259-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15935-5">
    <skos:altLabel xml:lang="de">Zigeuner</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15935-5">
    <skos:altLabel xml:lang="de">Sinti</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19801-6">
    <skos:altLabel xml:lang="en">Man-machine-interface</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19801-6">
    <skos:altLabel xml:lang="en">Man-machine interface</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19801-6">
    <skos:altLabel xml:lang="en">GUI (Graphical User Interface)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19801-6">
    <skos:altLabel xml:lang="en">Graphical user interface</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19801-6">
    <skos:altLabel xml:lang="de">Mensch-Maschine-Schnittstelle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19801-6">
    <skos:altLabel xml:lang="de">Graphische Benutzeroberfläche</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19801-6">
    <skos:altLabel xml:lang="de">Benutzungsoberfläche</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19801-6">
    <skos:altLabel xml:lang="de">Benutzeroberfläche</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11373-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10121-3">
    <skos:related rdf:resource="descriptor/10239-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10121-3">
    <skos:related rdf:resource="descriptor/12225-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10121-3">
    <skos:related rdf:resource="descriptor/10120-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10121-3">
    <skos:related rdf:resource="descriptor/13723-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19427-4">
    <skos:broader rdf:resource="thsys/70992"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19427-4">
    <skos:broader rdf:resource="descriptor/13001-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19691-1">
    <skos:altLabel xml:lang="de">Stadtvertretung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19691-1">
    <skos:altLabel xml:lang="de">Stadtrat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19691-1">
    <skos:altLabel xml:lang="de">Örtliche Selbstverwaltung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19691-1">
    <skos:altLabel xml:lang="de">Lokale Selbstverwaltung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19691-1">
    <skos:altLabel xml:lang="de">Gemeindevertretung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19691-1">
    <skos:altLabel xml:lang="de">Gemeinderat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19691-1">
    <skos:altLabel xml:lang="de">Gemeindeparlament</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16333-0">
    <skos:altLabel xml:lang="en">President of state</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16333-0">
    <skos:altLabel xml:lang="de">Staatspräsident</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16333-0">
    <skos:altLabel xml:lang="de">Präsident</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16333-0">
    <skos:altLabel xml:lang="de">Monarch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16333-0">
    <skos:altLabel xml:lang="de">König</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16333-0">
    <skos:altLabel xml:lang="de">Kaiser</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16333-0">
    <skos:altLabel xml:lang="de">Bundespräsident</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12588-1">
    <skos:altLabel xml:lang="de">Laufbahnplanung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12588-1">
    <skos:altLabel xml:lang="de">Berufsplanung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12588-1">
    <skos:altLabel xml:lang="de">Berufliche Strategie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14807-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12669-1">
    <skos:prefLabel xml:lang="en">Supplier selection</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12669-1">
    <skos:prefLabel xml:lang="de">Lieferantenwahl</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12592-3">
    <skos:broader rdf:resource="thsys/70833"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12592-3">
    <skos:broader rdf:resource="thsys/70324"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12592-3">
    <skos:broader rdf:resource="thsys/70245"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12592-3">
    <skos:broader rdf:resource="descriptor/16545-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12592-3">
    <skos:broader rdf:resource="descriptor/12567-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11504-3">
    <skos:related rdf:resource="descriptor/15093-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11504-3">
    <skos:related rdf:resource="descriptor/11494-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71009">
    <skos:narrower rdf:resource="descriptor/18388-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71009">
    <skos:narrower rdf:resource="descriptor/12987-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71009">
    <skos:narrower rdf:resource="descriptor/19192-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71009">
    <skos:narrower rdf:resource="descriptor/12990-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71009">
    <skos:narrower rdf:resource="descriptor/18389-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71009">
    <skos:narrower rdf:resource="descriptor/12986-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71009">
    <skos:narrower rdf:resource="descriptor/13018-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71009">
    <skos:narrower rdf:resource="descriptor/18387-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71009">
    <skos:narrower rdf:resource="descriptor/12981-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71009">
    <skos:narrower rdf:resource="descriptor/18383-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71009">
    <skos:narrower rdf:resource="descriptor/18390-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71009">
    <skos:narrower rdf:resource="descriptor/18352-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71009">
    <skos:narrower rdf:resource="descriptor/18386-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71009">
    <skos:narrower rdf:resource="descriptor/18375-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71009">
    <skos:narrower rdf:resource="descriptor/18384-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71009">
    <skos:narrower rdf:resource="descriptor/13881-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71009">
    <skos:narrower rdf:resource="descriptor/12980-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71009">
    <skos:narrower rdf:resource="thsys/71006"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71009">
    <skos:narrower rdf:resource="thsys/71007"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19457-2">
    <skos:broader rdf:resource="thsys/71064"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19457-2">
    <skos:broader rdf:resource="descriptor/11970-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11153-6">
    <skos:related rdf:resource="descriptor/10190-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11153-6">
    <skos:related rdf:resource="descriptor/19533-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11153-6">
    <skos:related rdf:resource="descriptor/19522-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11153-6">
    <skos:related rdf:resource="descriptor/19154-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11153-6">
    <skos:related rdf:resource="descriptor/10194-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11153-6">
    <skos:related rdf:resource="descriptor/24622-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11200-2">
    <skos:related rdf:resource="descriptor/16700-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11200-2">
    <skos:related rdf:resource="descriptor/11275-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10300-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14432-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70969">
    <skos:narrower rdf:resource="descriptor/13179-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70969">
    <skos:narrower rdf:resource="descriptor/13177-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70969">
    <skos:narrower rdf:resource="descriptor/19055-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70969">
    <skos:narrower rdf:resource="descriptor/19056-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70969">
    <skos:narrower rdf:resource="descriptor/18501-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70969">
    <skos:narrower rdf:resource="descriptor/13176-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70969">
    <skos:narrower rdf:resource="descriptor/18500-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70969">
    <skos:narrower rdf:resource="descriptor/11578-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70969">
    <skos:narrower rdf:resource="descriptor/18499-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70969">
    <skos:narrower rdf:resource="descriptor/13109-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70969">
    <skos:narrower rdf:resource="descriptor/13173-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70969">
    <skos:narrower rdf:resource="descriptor/13535-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70969">
    <skos:narrower rdf:resource="descriptor/12915-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70969">
    <skos:narrower rdf:resource="descriptor/13844-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70969">
    <skos:narrower rdf:resource="descriptor/18498-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70969">
    <skos:narrower rdf:resource="descriptor/13170-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70969">
    <skos:narrower rdf:resource="descriptor/18985-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70969">
    <skos:narrower rdf:resource="descriptor/16769-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70969">
    <skos:narrower rdf:resource="descriptor/19054-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70969">
    <skos:narrower rdf:resource="descriptor/11901-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70969">
    <skos:narrower rdf:resource="descriptor/18497-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70969">
    <skos:narrower rdf:resource="descriptor/13846-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13277-6">
    <skos:altLabel xml:lang="en">Voice processing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13277-6">
    <skos:altLabel xml:lang="de">Sprachkommunikation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13277-6">
    <skos:altLabel xml:lang="de">Spracherkennung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13277-6">
    <skos:altLabel xml:lang="de">Spracheingabe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13277-6">
    <skos:altLabel xml:lang="de">Sprachausgabe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13277-6">
    <skos:altLabel xml:lang="de">Rechnergestützte Sprachverarbeitung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13277-6">
    <skos:altLabel xml:lang="de">Maschinelle Sprachverarbeitung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11527-5">
    <skos:related rdf:resource="descriptor/10164-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11527-5">
    <skos:related rdf:resource="descriptor/10179-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11527-5">
    <skos:related rdf:resource="descriptor/11529-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11527-5">
    <skos:related rdf:resource="descriptor/19536-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16004-5">
    <skos:scopeNote xml:lang="de">Das Lebensmittelrecht bezieht sich auch auf die sog. Bedarfsgegenstände, wie z.B. Bekleidung, Kosmetika, Spielwaren und Reinigungsmittel.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10686-4">
    <skos:narrower rdf:resource="descriptor/10713-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10686-4">
    <skos:narrower rdf:resource="descriptor/19319-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10686-4">
    <skos:narrower rdf:resource="descriptor/19320-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10686-4">
    <skos:narrower rdf:resource="descriptor/19322-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10686-4">
    <skos:narrower rdf:resource="descriptor/10570-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10686-4">
    <skos:narrower rdf:resource="descriptor/10690-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10686-4">
    <skos:narrower rdf:resource="descriptor/19317-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10686-4">
    <skos:narrower rdf:resource="descriptor/10684-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10686-4">
    <skos:narrower rdf:resource="descriptor/10683-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10686-4">
    <skos:narrower rdf:resource="descriptor/19314-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17616-4">
    <skos:altLabel xml:lang="en">OAPEC countries</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17616-4">
    <skos:altLabel xml:lang="en">Arab petroleum-exporting countries</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17616-4">
    <skos:altLabel xml:lang="en">Arab Gulf states</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17616-4">
    <skos:altLabel xml:lang="de">Piratenküste</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17616-4">
    <skos:altLabel xml:lang="de">OAPEC-Staaten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17616-4">
    <skos:altLabel xml:lang="de">OAPEC-Länder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17616-4">
    <skos:altLabel xml:lang="de">Golfregion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17616-4">
    <skos:altLabel xml:lang="de">Arabische Golfstaaten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13451-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17785-2">
    <skos:prefLabel xml:lang="en">Jamaica</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17785-2">
    <skos:prefLabel xml:lang="de">Jamaika</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11849-4">
    <skos:altLabel xml:lang="de">Regionaler Entleerungsprozess</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11849-4">
    <skos:altLabel xml:lang="de">Regionaler Entleerungsprozeß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11849-4">
    <skos:altLabel xml:lang="de">Regionale Zersiedlung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11849-4">
    <skos:altLabel xml:lang="de">Regionale Dekonzentration</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11849-4">
    <skos:altLabel xml:lang="de">Räumliche Dezentralisierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11849-4">
    <skos:altLabel xml:lang="de">Räumliche Dekonzentration</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11849-4">
    <skos:altLabel xml:lang="de">Entballung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11849-4">
    <skos:altLabel xml:lang="de">Disagglomeration</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70891">
    <skos:narrower rdf:resource="descriptor/13808-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70891">
    <skos:narrower rdf:resource="descriptor/13411-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70891">
    <skos:narrower rdf:resource="descriptor/18740-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70891">
    <skos:narrower rdf:resource="descriptor/13299-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70891">
    <skos:narrower rdf:resource="descriptor/13817-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70891">
    <skos:narrower rdf:resource="descriptor/18475-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70891">
    <skos:narrower rdf:resource="descriptor/13298-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70891">
    <skos:narrower rdf:resource="descriptor/13297-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70891">
    <skos:narrower rdf:resource="descriptor/13816-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70891">
    <skos:narrower rdf:resource="descriptor/13295-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70891">
    <skos:narrower rdf:resource="descriptor/18739-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70891">
    <skos:narrower rdf:resource="descriptor/18738-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70891">
    <skos:narrower rdf:resource="descriptor/18180-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70891">
    <skos:narrower rdf:resource="descriptor/18105-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70891">
    <skos:narrower rdf:resource="descriptor/19551-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70891">
    <skos:narrower rdf:resource="descriptor/13814-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70891">
    <skos:narrower rdf:resource="descriptor/24786-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70891">
    <skos:narrower rdf:resource="descriptor/13294-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70891">
    <skos:narrower rdf:resource="descriptor/13441-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70891">
    <skos:narrower rdf:resource="descriptor/13293-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70891">
    <skos:narrower rdf:resource="descriptor/13292-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70891">
    <skos:narrower rdf:resource="descriptor/13813-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70891">
    <skos:narrower rdf:resource="descriptor/18737-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70891">
    <skos:narrower rdf:resource="descriptor/13810-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70891">
    <skos:narrower rdf:resource="descriptor/13809-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70891">
    <skos:narrower rdf:resource="descriptor/10866-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70891">
    <skos:narrower rdf:resource="descriptor/13815-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15075-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13360-5">
    <skos:scopeNote xml:lang="de">Für Hörfunkanstalten sowie gemeinschaftliche Hörfunk- und Fernsehanstalten</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18507-4">
    <zbwext:useInsteadNote rdf:nodeID="id44"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id123">
    <rdf:value xml:lang="de">für die Staatsgewalt</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19472-6">
    <skos:scopeNote xml:lang="de">Theoretische und empirische Untersuchungen zur Problematik städtischer Arbeitsmärkte.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19804-0">
    <skos:broader rdf:resource="thsys/73340"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19804-0">
    <skos:broader rdf:resource="descriptor/15621-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15938-6">
    <skos:related rdf:resource="descriptor/13398-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15938-6">
    <skos:related rdf:resource="descriptor/15927-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15938-6">
    <skos:related rdf:resource="descriptor/15941-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15938-6">
    <skos:related rdf:resource="descriptor/15920-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15938-6">
    <skos:related rdf:resource="descriptor/12865-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15938-6">
    <skos:related rdf:resource="descriptor/15940-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15938-6">
    <skos:related rdf:resource="descriptor/16698-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15938-6">
    <skos:related rdf:resource="descriptor/16030-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15938-6">
    <skos:related rdf:resource="descriptor/16477-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15938-6">
    <skos:related rdf:resource="descriptor/18972-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15938-6">
    <skos:related rdf:resource="descriptor/10173-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15938-6">
    <skos:related rdf:resource="descriptor/15937-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15938-6">
    <skos:related rdf:resource="descriptor/18971-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17639-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19100-1">
    <skos:scopeNote xml:lang="de">Kostenrechnungsmodell, das die Kosten basierend auf dem Fertigungsablauf berechnet</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11347-0">
    <skos:altLabel xml:lang="en">Labor arbitration</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11347-0">
    <skos:altLabel xml:lang="en">Industrial conciliation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11347-0">
    <skos:altLabel xml:lang="de">Tarifliche Einigungsstelle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17534-6">
    <skos:broader rdf:resource="thsys/70174"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17534-6">
    <skos:broader rdf:resource="descriptor/19077-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17534-6">
    <skos:broader rdf:resource="descriptor/19078-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19595-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13202-2">
    <skos:altLabel xml:lang="de">Elektrogroßhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13202-2">
    <skos:altLabel xml:lang="de">Elektroeinzelhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13202-2">
    <skos:altLabel xml:lang="de">Beleuchtungshandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10036-1">
    <skos:narrower rdf:resource="descriptor/10334-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10036-1">
    <skos:narrower rdf:resource="descriptor/10055-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10036-1">
    <skos:narrower rdf:resource="descriptor/10171-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10036-1">
    <skos:narrower rdf:resource="descriptor/10652-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16517-4">
    <skos:altLabel xml:lang="de">Rundfunkmonopol</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16517-4">
    <skos:altLabel xml:lang="de">Rundfunk- und Fernsehpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16517-4">
    <skos:altLabel xml:lang="de">Radiopolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16517-4">
    <skos:altLabel xml:lang="de">Hörfunkpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16517-4">
    <skos:altLabel xml:lang="de">Fernsehpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70858">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14703-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18863-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13713-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17986-6">
    <skos:altLabel xml:lang="de">Französischsprachiges Afrika</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17986-6">
    <skos:altLabel xml:lang="de">Afrika (Frankophon)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11728-2">
    <skos:related rdf:resource="descriptor/11633-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11728-2">
    <skos:related rdf:resource="descriptor/11540-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11728-2">
    <skos:related rdf:resource="descriptor/11595-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11728-2">
    <skos:related rdf:resource="descriptor/11526-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11728-2">
    <skos:related rdf:resource="descriptor/11536-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11728-2">
    <skos:related rdf:resource="descriptor/11715-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11728-2">
    <skos:related rdf:resource="descriptor/11525-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11728-2">
    <skos:related rdf:resource="descriptor/11711-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70573">
    <skos:prefLabel xml:lang="en">V.14.03  Health</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70573">
    <skos:prefLabel xml:lang="de">V.14.03  Gesundheit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18592-1">
    <skos:prefLabel xml:lang="en">Ship register</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18592-1">
    <skos:prefLabel xml:lang="de">Schiffsregister</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16495-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19444-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16727-0">
    <skos:altLabel xml:lang="en">Economic methodology</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16727-0">
    <skos:altLabel xml:lang="en">Economic method</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16727-0">
    <skos:altLabel xml:lang="de">Wirtschaftswissenschaftliche Methode</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16727-0">
    <skos:altLabel xml:lang="de">Ökonomische Methode</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16727-0">
    <skos:altLabel xml:lang="de">Methodologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16727-0">
    <skos:altLabel xml:lang="de">Methodenlehre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16727-0">
    <skos:altLabel xml:lang="de">Methodenentwicklung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16727-0">
    <skos:altLabel xml:lang="de">Methode der wirtschaftswissenschaftlichen Forschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19348-0">
    <skos:altLabel xml:lang="de">Weinbaupolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70476">
    <skos:prefLabel xml:lang="de">P.12.03  Kunststoffe und Chemiefasern</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14636-6">
    <skos:prefLabel xml:lang="en">Ammonia</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14636-6">
    <skos:prefLabel xml:lang="de">Ammoniak</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14486-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10581-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70233">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19691-1">
    <skos:broader rdf:resource="thsys/73380"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19691-1">
    <skos:broader rdf:resource="descriptor/16308-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13279-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16372-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16740-1">
    <skos:related rdf:resource="descriptor/16743-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16740-1">
    <skos:related rdf:resource="descriptor/10252-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16740-1">
    <skos:related rdf:resource="descriptor/16738-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16740-1">
    <skos:related rdf:resource="descriptor/16737-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71040">
    <skos:narrower rdf:resource="descriptor/12868-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71040">
    <skos:narrower rdf:resource="descriptor/12196-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71040">
    <skos:narrower rdf:resource="descriptor/12796-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71040">
    <skos:narrower rdf:resource="descriptor/10831-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71040">
    <skos:narrower rdf:resource="descriptor/11703-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71040">
    <skos:narrower rdf:resource="descriptor/10830-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71040">
    <skos:narrower rdf:resource="descriptor/12195-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71040">
    <skos:narrower rdf:resource="descriptor/12194-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71040">
    <skos:narrower rdf:resource="descriptor/12863-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71040">
    <skos:narrower rdf:resource="descriptor/12867-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71040">
    <skos:narrower rdf:resource="descriptor/12446-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71040">
    <skos:narrower rdf:resource="descriptor/12872-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71040">
    <skos:narrower rdf:resource="descriptor/12865-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71040">
    <skos:narrower rdf:resource="descriptor/10828-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71040">
    <skos:narrower rdf:resource="descriptor/12177-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71040">
    <skos:narrower rdf:resource="descriptor/11904-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71040">
    <skos:narrower rdf:resource="descriptor/10826-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14274-0">
    <skos:related rdf:resource="descriptor/14285-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14274-0">
    <skos:related rdf:resource="descriptor/14296-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12939-5">
    <skos:broader rdf:resource="thsys/71029"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12939-5">
    <skos:broader rdf:resource="descriptor/12949-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11260-5">
    <zbwext:useInsteadNote rdf:nodeID="id386"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18855-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16454-2">
    <skos:altLabel xml:lang="de">Kollektive Sicherheit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18455-4">
    <skos:editorialNote xml:lang="de">nicht als Formschlagwort zu verwenden</skos:editorialNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15267-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13327-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13935-1">
    <skos:prefLabel xml:lang="en">Textile professions</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13935-1">
    <skos:prefLabel xml:lang="de">Textilberufe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12798-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15102-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70424">
    <rdfs:label xml:lang="de">Kraftmaschinen und Motoren</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10926-3">
    <skos:altLabel xml:lang="de">Marktzutrittsverhindernder Preis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10926-3">
    <skos:altLabel xml:lang="de">Markteintrittsverhindernder Preis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10926-3">
    <skos:altLabel xml:lang="de">Eintrittsverhindernder Preis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10140-6">
    <skos:related rdf:resource="descriptor/10060-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10140-6">
    <skos:related rdf:resource="descriptor/10147-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10140-6">
    <skos:related rdf:resource="descriptor/12170-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10140-6">
    <skos:related rdf:resource="descriptor/16759-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10140-6">
    <skos:related rdf:resource="descriptor/10141-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10140-6">
    <skos:related rdf:resource="descriptor/15473-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13244-0">
    <skos:broader rdf:resource="thsys/70940"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13244-0">
    <skos:broader rdf:resource="thsys/70937"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13244-0">
    <skos:broader rdf:resource="descriptor/13563-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13244-0">
    <skos:broader rdf:resource="descriptor/13621-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11545-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13890-4">
    <skos:prefLabel xml:lang="en">Electricians</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13890-4">
    <skos:prefLabel xml:lang="de">Elektroberufe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70874">
    <skos:narrower rdf:resource="descriptor/12846-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70874">
    <skos:narrower rdf:resource="descriptor/12845-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70874">
    <skos:narrower rdf:resource="descriptor/12844-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70874">
    <skos:narrower rdf:resource="descriptor/12841-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70874">
    <skos:narrower rdf:resource="descriptor/13971-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70874">
    <skos:narrower rdf:resource="descriptor/12818-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70874">
    <skos:narrower rdf:resource="descriptor/21315-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16739-0">
    <skos:related rdf:resource="descriptor/10113-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16739-0">
    <skos:related rdf:resource="descriptor/10252-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16739-0">
    <skos:related rdf:resource="descriptor/19218-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16739-0">
    <skos:related rdf:resource="descriptor/20881-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13488-0">
    <skos:altLabel xml:lang="en">Shopping hours</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13488-0">
    <skos:altLabel xml:lang="de">Ladenschlussgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13488-0">
    <skos:altLabel xml:lang="de">Ladenschluss</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13488-0">
    <skos:altLabel xml:lang="de">Ladenschlußgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13488-0">
    <skos:altLabel xml:lang="de">Ladenschluß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13488-0">
    <skos:altLabel xml:lang="de">Ladenöffnungszeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13488-0">
    <skos:altLabel xml:lang="de">Dienstleistungsabend</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11900-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16723-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11582-4">
    <skos:altLabel xml:lang="de">Verteidigungsgüterkauf</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11582-4">
    <skos:altLabel xml:lang="de">Rüstungsauftrag</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11582-4">
    <skos:altLabel xml:lang="de">Militärbeschaffungswesen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16486-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12294-4">
    <skos:prefLabel xml:lang="en">Replacement investment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12294-4">
    <skos:prefLabel xml:lang="de">Reinvestition</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12504-5">
    <skos:broader rdf:resource="thsys/70233"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12504-5">
    <skos:broader rdf:resource="thsys/70030"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12504-5">
    <skos:broader rdf:resource="descriptor/10011-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15340-1">
    <skos:altLabel xml:lang="en">Regression coefficient</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15340-1">
    <skos:altLabel xml:lang="en">Logistic regression</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15340-1">
    <skos:altLabel xml:lang="en">Linear regression model</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15340-1">
    <skos:altLabel xml:lang="de">Regressionsparameter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15340-1">
    <skos:altLabel xml:lang="de">Regressionskoeffizient</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15340-1">
    <skos:altLabel xml:lang="de">Regressionsfunktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15340-1">
    <skos:altLabel xml:lang="de">Regressionsanalyse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15340-1">
    <skos:altLabel xml:lang="de">Logistische Regression</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15340-1">
    <skos:altLabel xml:lang="de">Lineares Regressionsmodell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15340-1">
    <skos:altLabel xml:lang="de">Bestimmtheitsmaß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18204-1">
    <skos:broader rdf:resource="thsys/70465"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18204-1">
    <skos:broader rdf:resource="thsys/70161"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18204-1">
    <skos:broader rdf:resource="descriptor/14150-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18204-1">
    <skos:broader rdf:resource="descriptor/15009-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19495-1">
    <skos:scopeNote xml:lang="de">Wirtschaftstheoretische Untersuchungen zu Ursachen und Wirkungen von Arbeitslosigkeit.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19305-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13010-4">
    <skos:broader rdf:resource="thsys/71008"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13010-4">
    <skos:broader rdf:resource="descriptor/13014-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18005-0">
    <skos:altLabel xml:lang="de">Privatradio</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18005-0">
    <skos:altLabel xml:lang="de">Privatfunk</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18005-0">
    <skos:altLabel xml:lang="de">Privates Radio</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18005-0">
    <skos:altLabel xml:lang="de">Privater Hörfunk</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18005-0">
    <skos:altLabel xml:lang="de">Private Rundfunk- und Fernsehanstalt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18005-0">
    <skos:altLabel xml:lang="de">Kommerzieller Rundfunk</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18005-0">
    <skos:altLabel xml:lang="de">Kommerzielle Rundfunk- und Fernsehanstalt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24859-1">
    <zbwext:useInsteadNote rdf:nodeID="id380"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11532-5">
    <skos:broader rdf:resource="thsys/73337"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11532-5">
    <skos:broader rdf:resource="thsys/71073"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11532-5">
    <skos:broader rdf:resource="thsys/70862"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11532-5">
    <skos:broader rdf:resource="descriptor/13410-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15373-0">
    <skos:prefLabel xml:lang="en">Econometrics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15373-0">
    <skos:prefLabel xml:lang="de">Ökonometrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71067">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.11.03</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19402-6">
    <skos:altLabel xml:lang="en">Average tax</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70549">
    <skos:prefLabel xml:lang="de">P.08  Maschine und Maschinenteil</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11217-6">
    <skos:broader rdf:resource="thsys/70052"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11217-6">
    <skos:broader rdf:resource="descriptor/10996-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19633-1">
    <skos:altLabel xml:lang="de">Umweltinformationssystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19633-1">
    <skos:altLabel xml:lang="de">Umweltdatenbank</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19633-1">
    <skos:altLabel xml:lang="de">Öko-Informationssystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18176-1">
    <skos:narrower rdf:resource="descriptor/16037-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18176-1">
    <skos:narrower rdf:resource="descriptor/16053-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15633-0">
    <skos:related rdf:resource="descriptor/18859-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15633-0">
    <skos:related rdf:resource="descriptor/15765-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15633-0">
    <skos:related rdf:resource="descriptor/15790-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15633-0">
    <skos:related rdf:resource="descriptor/15662-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15633-0">
    <skos:related rdf:resource="descriptor/15658-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15633-0">
    <skos:related rdf:resource="descriptor/15415-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15633-0">
    <skos:related rdf:resource="descriptor/19204-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15633-0">
    <skos:related rdf:resource="descriptor/19596-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15633-0">
    <skos:related rdf:resource="descriptor/15635-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15633-0">
    <skos:related rdf:resource="descriptor/19203-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15633-0">
    <skos:related rdf:resource="descriptor/19811-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15633-0">
    <skos:related rdf:resource="descriptor/15627-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71016">
    <skos:prefLabel xml:lang="en">W.04.01  Power industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71016">
    <skos:prefLabel xml:lang="de">W.04.01  Energiewirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13345-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18700-3">
    <skos:prefLabel xml:lang="en">Foreign currency translation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18700-3">
    <skos:prefLabel xml:lang="de">Währungsumrechnung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17662-4">
    <skos:altLabel xml:lang="en">Comoro Islands</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17662-4">
    <skos:altLabel xml:lang="de">Islamische Bundesrepublik Komoren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70649">
    <skos:broader rdf:resource="thsys/70538"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70557">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19817-5">
    <skos:broader rdf:resource="thsys/73331"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19817-5">
    <skos:broader rdf:resource="descriptor/15769-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19817-5">
    <skos:broader rdf:resource="descriptor/16182-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16185-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id360">
    <rdf:value xml:lang="de">Für Untersuchungen zur Beteiligung von Frauen am Erwerbsleben</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12915-5">
    <skos:prefLabel xml:lang="en">Industrial constructing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12915-5">
    <skos:prefLabel xml:lang="de">Gewerbebau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18684-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19054-3">
    <skos:narrower rdf:resource="descriptor/19055-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13331-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19116-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15869-6">
    <skos:related rdf:resource="descriptor/19619-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15869-6">
    <skos:related rdf:resource="descriptor/12936-4"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id341">
    <rdfs:seeAlso rdf:resource="descriptor/25743-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19180-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12732-6">
    <skos:altLabel xml:lang="en">Computer-aided engineering</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12732-6">
    <skos:altLabel xml:lang="en">CAE (Computer-Aided Engineering)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73333">
    <skos:broader rdf:resource="thsys/70154"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13090-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15520-6">
    <skos:prefLabel xml:lang="en">Travelling-Salesman Problem</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15520-6">
    <skos:prefLabel xml:lang="de">Travelling-Salesman-Problem</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10503-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18070-3">
    <skos:broader rdf:resource="thsys/70485"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18070-3">
    <skos:broader rdf:resource="thsys/70244"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18070-3">
    <skos:broader rdf:resource="thsys/70017"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10808-2">
    <skos:broader rdf:resource="thsys/71089"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19115-2">
    <skos:altLabel xml:lang="de">Rechnergestützte Lagerverwaltung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19115-2">
    <skos:altLabel xml:lang="de">Rechnergestützte Lagersteuerung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10217-4">
    <skos:narrower rdf:resource="descriptor/10211-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10217-4">
    <skos:narrower rdf:resource="descriptor/10220-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10217-4">
    <skos:narrower rdf:resource="descriptor/10219-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10217-4">
    <skos:narrower rdf:resource="descriptor/19223-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10217-4">
    <skos:narrower rdf:resource="descriptor/10216-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10217-4">
    <skos:narrower rdf:resource="descriptor/10215-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10217-4">
    <skos:narrower rdf:resource="descriptor/10214-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10217-4">
    <skos:narrower rdf:resource="descriptor/10204-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10217-4">
    <skos:narrower rdf:resource="descriptor/10198-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19680-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15661-2">
    <skos:broader rdf:resource="thsys/70828"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15661-2">
    <skos:broader rdf:resource="thsys/70581"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15661-2">
    <skos:broader rdf:resource="descriptor/15655-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11675-4">
    <skos:broader rdf:resource="thsys/71074"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11675-4">
    <skos:broader rdf:resource="thsys/71010"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11675-4">
    <skos:broader rdf:resource="thsys/70811"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11675-4">
    <skos:broader rdf:resource="descriptor/11547-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11675-4">
    <skos:broader rdf:resource="descriptor/15752-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24707-3">
    <skos:broader rdf:resource="thsys/70187"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24707-3">
    <skos:broader rdf:resource="descriptor/18674-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19653-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15320-0">
    <skos:scopeNote xml:lang="de">Für Ranking-Listen BENUTZE Publikationsart "Rangliste".</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15320-0">
    <skos:scopeNote xml:lang="de">Einfachstes Skalierungsverfahren, bei dem durch Befragungen Rangordnungen hergestellt werden (z.B. Beliebtheit von Radiosendern oder Uni-Professoren). Zu verwenden für Literatur über Ranking allgemein oder für methodische Fragen zum Ranking.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11151-3">
    <skos:related rdf:resource="descriptor/11149-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11151-3">
    <skos:related rdf:resource="descriptor/13451-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12980-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24707-3">
    <skos:scopeNote xml:lang="de">Herdenverhalten wird auf Finanzmärkten beobachtet, wenn Akteure dazu neigen, die gleichen Entscheidungen zu treffen, die andere Marktteilnehmer bereits vor ihnen getroffen haben. Dieses Verhalten muss nicht notwendigerweise als irrational interpretiert werden.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70942">
    <skos:broader rdf:resource="thsys/70943"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16824-2">
    <skos:broader rdf:resource="thsys/70554"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16824-2">
    <skos:broader rdf:resource="descriptor/16825-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16824-2">
    <skos:broader rdf:resource="descriptor/17982-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11959-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17461-0">
    <skos:prefLabel xml:lang="en">USSR</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17461-0">
    <skos:prefLabel xml:lang="de">Sowjetunion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11294-2">
    <skos:narrower rdf:resource="descriptor/18083-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11294-2">
    <skos:narrower rdf:resource="descriptor/18509-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11294-2">
    <skos:narrower rdf:resource="descriptor/11302-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11294-2">
    <skos:narrower rdf:resource="descriptor/11289-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11294-2">
    <skos:narrower rdf:resource="descriptor/11288-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11294-2">
    <skos:narrower rdf:resource="descriptor/11286-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11294-2">
    <skos:narrower rdf:resource="descriptor/11285-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11294-2">
    <skos:narrower rdf:resource="descriptor/11356-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18621-6">
    <skos:prefLabel xml:lang="en">Container transport industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18621-6">
    <skos:prefLabel xml:lang="de">Container-Transportgewerbe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72182">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.16.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14626-2">
    <skos:broader rdf:resource="thsys/70461"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14626-2">
    <skos:broader rdf:resource="descriptor/14596-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12785-6">
    <skos:narrower rdf:resource="descriptor/19127-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70327">
    <skos:prefLabel xml:lang="en">G.02.04  Southeast Asia</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70327">
    <skos:prefLabel xml:lang="de">G.02.04  Südostasien</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71109">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18671-5">
    <skos:narrower rdf:resource="descriptor/18673-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18000-3">
    <skos:narrower rdf:resource="descriptor/13678-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18000-3">
    <skos:narrower rdf:resource="descriptor/13683-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18000-3">
    <skos:narrower rdf:resource="descriptor/13680-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18000-3">
    <skos:narrower rdf:resource="descriptor/15599-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13342-0">
    <skos:related rdf:resource="descriptor/13558-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13342-0">
    <skos:related rdf:resource="descriptor/14433-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14269-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12699-6">
    <skos:prefLabel xml:lang="en">Maintenance and repair</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12699-6">
    <skos:prefLabel xml:lang="de">Instandhaltung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id185">
    <rdf:value xml:lang="de">Für den Außenhandel mit Agrarprodukten</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18401-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13452-0">
    <skos:broader rdf:resource="thsys/71024"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13452-0">
    <skos:broader rdf:resource="descriptor/18313-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17464-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12693-4">
    <skos:related rdf:resource="descriptor/13513-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12693-4">
    <skos:related rdf:resource="descriptor/13259-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12693-4">
    <skos:related rdf:resource="descriptor/12644-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12693-4">
    <skos:related rdf:resource="descriptor/12681-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12693-4">
    <skos:related rdf:resource="descriptor/12736-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12693-4">
    <skos:related rdf:resource="descriptor/18409-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12693-4">
    <skos:related rdf:resource="descriptor/14318-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12693-4">
    <skos:related rdf:resource="descriptor/12689-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12693-4">
    <skos:related rdf:resource="descriptor/12730-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16563-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19739-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70876">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.21</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14581-5">
    <skos:narrower rdf:resource="descriptor/14583-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12105-5">
    <skos:narrower rdf:resource="descriptor/12094-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12105-5">
    <skos:narrower rdf:resource="descriptor/12589-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12105-5">
    <skos:narrower rdf:resource="descriptor/18867-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10852-6">
    <skos:narrower rdf:resource="descriptor/11125-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10852-6">
    <skos:narrower rdf:resource="descriptor/10872-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10852-6">
    <skos:narrower rdf:resource="descriptor/18538-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10852-6">
    <skos:narrower rdf:resource="descriptor/10864-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10852-6">
    <skos:narrower rdf:resource="descriptor/10856-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12177-1">
    <skos:prefLabel xml:lang="en">Corporate divestiture</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12177-1">
    <skos:prefLabel xml:lang="de">Betriebsaufspaltung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13459-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16484-0">
    <skos:broader rdf:resource="thsys/71051"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16484-0">
    <skos:broader rdf:resource="thsys/70278"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16484-0">
    <skos:broader rdf:resource="descriptor/10968-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70561">
    <rdfs:label xml:lang="en">South Asia</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70561">
    <rdfs:label xml:lang="de">Südasien</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12849-6">
    <skos:related rdf:resource="descriptor/11682-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12849-6">
    <skos:related rdf:resource="descriptor/16104-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12849-6">
    <skos:related rdf:resource="descriptor/16092-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12849-6">
    <skos:related rdf:resource="descriptor/10828-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12849-6">
    <skos:related rdf:resource="descriptor/12271-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13501-6">
    <skos:prefLabel xml:lang="en">International transport</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13501-6">
    <skos:prefLabel xml:lang="de">Grenzüberschreitender Verkehr</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14123-0">
    <skos:broader rdf:resource="thsys/70467"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14123-0">
    <skos:broader rdf:resource="descriptor/14129-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10986-6">
    <skos:related rdf:resource="descriptor/16741-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10986-6">
    <skos:related rdf:resource="descriptor/11055-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10986-6">
    <skos:related rdf:resource="descriptor/11046-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10986-6">
    <skos:related rdf:resource="descriptor/10696-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10986-6">
    <skos:related rdf:resource="descriptor/18011-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10986-6">
    <skos:related rdf:resource="descriptor/10377-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16740-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11220-3">
    <skos:broader rdf:resource="thsys/71062"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11220-3">
    <skos:broader rdf:resource="descriptor/11219-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15723-6">
    <skos:altLabel xml:lang="de">Gesellschaftsgeschichte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15723-6">
    <skos:altLabel xml:lang="de">Alltagsgeschichte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19655-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11610-4">
    <skos:broader rdf:resource="thsys/71078"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11610-4">
    <skos:broader rdf:resource="thsys/71064"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11610-4">
    <skos:broader rdf:resource="descriptor/11613-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11610-4">
    <skos:broader rdf:resource="descriptor/11537-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19074-4">
    <skos:narrower rdf:resource="descriptor/12714-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19074-4">
    <skos:narrower rdf:resource="descriptor/12749-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16212-5">
    <skos:broader rdf:resource="thsys/70871"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16212-5">
    <skos:broader rdf:resource="thsys/70095"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16212-5">
    <skos:broader rdf:resource="descriptor/16563-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18991-1">
    <skos:related rdf:resource="descriptor/18811-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18991-1">
    <skos:related rdf:resource="descriptor/19338-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18991-1">
    <skos:related rdf:resource="descriptor/19394-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18991-1">
    <skos:related rdf:resource="descriptor/13160-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18991-1">
    <skos:related rdf:resource="descriptor/13219-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18991-1">
    <skos:related rdf:resource="descriptor/19674-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19331-3">
    <skos:altLabel xml:lang="de">Zollvereinheitlichung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10466-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19591-5">
    <skos:altLabel xml:lang="de">Numerische Mathematik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19591-5">
    <skos:altLabel xml:lang="de">Numerik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18347-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20855-1">
    <skos:scopeNote xml:lang="de">Das Governance-Konzept befasst sich in Anlehnung an die Prozesse des Regierungshandelns mit der Ausgestaltung von Steuerungsstrukturen in politischen, gesellschaftlichen und ökonomischen Kontexten. Es soll über fest definierte Verhaltenskodices "gutes" d.h. gesellschaftlich verantwortliches und faires Regieren, Verwalten und/oder Management fördern. Das Konzept kann auf verschiedenen Handlungsebenen ansetzen und sowohl regional wie international orientiert sein. Akteure können Regierungen ebenso wie private Unternehmen und Einrichtungen der öffentlichen Hand sein. Der sozialwissenschaftliche Ansatz zur Untersuchung der genannten Steuerungsstrukturen wird häufig mit einer institutionellen Analyse gekoppelt.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11864-1">
    <skos:scopeNote xml:lang="de">Verkehr zur Verbindung relativ nahe beieinander liegender Orte.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19847-3">
    <skos:narrower rdf:resource="descriptor/16111-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19847-3">
    <skos:narrower rdf:resource="descriptor/19848-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19847-3">
    <skos:narrower rdf:resource="descriptor/13773-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14635-1">
    <skos:altLabel xml:lang="de">Zündhölzer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14635-1">
    <skos:altLabel xml:lang="de">Trinitrotoluol</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14635-1">
    <skos:altLabel xml:lang="de">TNT (Trinitrotoluol)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14635-1">
    <skos:altLabel xml:lang="de">Sprengstoff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14635-1">
    <skos:altLabel xml:lang="de">Pyrotechnisches Produkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14635-1">
    <skos:altLabel xml:lang="de">Nitroglyzerin</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14635-1">
    <skos:altLabel xml:lang="de">Feuerzeug</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14635-1">
    <skos:altLabel xml:lang="de">Feuerwerkskörper</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14635-1">
    <skos:altLabel xml:lang="de">Dynamit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18836-6">
    <skos:prefLabel xml:lang="en">Social marketing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18836-6">
    <skos:prefLabel xml:lang="de">Social Marketing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13694-4">
    <skos:prefLabel xml:lang="en">Deposit insurance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13694-4">
    <skos:prefLabel xml:lang="de">Einlagensicherung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11792-0">
    <skos:scopeNote xml:lang="de">Nur für strukturelle Krisen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18210-6">
    <skos:related rdf:resource="descriptor/15920-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13824-3">
    <skos:related rdf:resource="descriptor/19424-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13824-3">
    <skos:related rdf:resource="descriptor/13823-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13824-3">
    <skos:related rdf:resource="descriptor/19679-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13824-3">
    <skos:related rdf:resource="descriptor/18744-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13824-3">
    <skos:related rdf:resource="descriptor/16547-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13824-3">
    <skos:related rdf:resource="descriptor/21002-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18744-4">
    <skos:broader rdf:resource="thsys/70892"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18744-4">
    <skos:broader rdf:resource="descriptor/18101-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18647-2">
    <skos:related rdf:resource="descriptor/19809-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18647-2">
    <skos:related rdf:resource="descriptor/14522-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18647-2">
    <skos:related rdf:resource="descriptor/18000-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18647-2">
    <skos:related rdf:resource="descriptor/15416-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18647-2">
    <skos:related rdf:resource="descriptor/24708-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18647-2">
    <skos:related rdf:resource="descriptor/19657-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18647-2">
    <skos:related rdf:resource="descriptor/13674-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10433-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19385-1">
    <skos:broader rdf:resource="thsys/71016"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19385-1">
    <skos:broader rdf:resource="thsys/71006"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19385-1">
    <skos:broader rdf:resource="thsys/70957"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19385-1">
    <skos:broader rdf:resource="descriptor/18996-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10177-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10778-6">
    <skos:related rdf:resource="descriptor/10770-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10778-6">
    <skos:related rdf:resource="descriptor/10780-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18231-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10111-6">
    <skos:related rdf:resource="descriptor/15773-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10111-6">
    <skos:related rdf:resource="descriptor/15768-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10111-6">
    <skos:related rdf:resource="descriptor/15767-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10111-6">
    <skos:related rdf:resource="descriptor/10058-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15102-3">
    <skos:broader rdf:resource="thsys/72181"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15102-3">
    <skos:broader rdf:resource="thsys/71115"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15102-3">
    <skos:broader rdf:resource="descriptor/15076-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18623-2">
    <zbwext:useInsteadNote rdf:nodeID="id351"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18596-0">
    <skos:prefLabel xml:lang="en">Volume of seaborne cargoes handled</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18596-0">
    <skos:prefLabel xml:lang="de">Seegüterumschlag</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12311-2">
    <skos:broader rdf:resource="thsys/70005"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13219-6">
    <skos:prefLabel xml:lang="en">Food trade</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13219-6">
    <skos:prefLabel xml:lang="de">Lebensmittelhandel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12768-6">
    <skos:prefLabel xml:lang="en">Marketing information system</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12768-6">
    <skos:prefLabel xml:lang="de">Marketing-Informationssystem</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19448-3">
    <skos:prefLabel xml:lang="en">Place marketing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19448-3">
    <skos:prefLabel xml:lang="de">Standortmarketing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12047-0">
    <skos:narrower rdf:resource="descriptor/18906-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12047-0">
    <skos:narrower rdf:resource="descriptor/10984-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12047-0">
    <skos:narrower rdf:resource="descriptor/11825-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12078-3">
    <skos:broader rdf:resource="thsys/70485"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12078-3">
    <skos:broader rdf:resource="descriptor/12085-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11890-0">
    <skos:broader rdf:resource="thsys/71065"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11890-0">
    <skos:broader rdf:resource="descriptor/19447-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11890-0">
    <skos:broader rdf:resource="descriptor/19458-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12491-2">
    <skos:altLabel xml:lang="de">Nutzkosten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12491-2">
    <skos:altLabel xml:lang="de">Leerkosten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12491-2">
    <skos:altLabel xml:lang="de">Kapazitätskosten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12491-2">
    <skos:altLabel xml:lang="de">Fixkostenrechnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12491-2">
    <skos:altLabel xml:lang="de">Fixe Kosten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12491-2">
    <skos:altLabel xml:lang="de">Feste Kosten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14234-5">
    <skos:broader rdf:resource="thsys/70348"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14234-5">
    <skos:broader rdf:resource="descriptor/14237-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19503-0">
    <skos:prefLabel xml:lang="en">Trade union law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19503-0">
    <skos:prefLabel xml:lang="de">Gewerkschaftsrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18975-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11754-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10917-4">
    <skos:scopeNote xml:lang="de">ABER: Für Antidiskriminierungsregelungen wg. Geschlecht, Rasse, etc. BENUTZE: Gleichbehandlungsrecht.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10917-4">
    <skos:scopeNote xml:lang="de">Nur für Regelungen im Wettbewerbsrecht.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20906-3">
    <skos:related rdf:resource="descriptor/12931-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20906-3">
    <skos:related rdf:resource="descriptor/11572-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20906-3">
    <skos:related rdf:resource="descriptor/18913-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20906-3">
    <skos:related rdf:resource="descriptor/12010-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18179-2">
    <skos:prefLabel xml:lang="en">Online Public Access Catalogue</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18179-2">
    <skos:prefLabel xml:lang="de">Online-Katalog</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10102-0">
    <skos:prefLabel xml:lang="en">Pareto optimality</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10102-0">
    <skos:prefLabel xml:lang="de">Pareto-Optimum</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16422-1">
    <skos:broader rdf:resource="thsys/73375"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16422-1">
    <skos:broader rdf:resource="thsys/73366"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20206-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13127-4">
    <skos:related rdf:resource="descriptor/14055-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13127-4">
    <skos:related rdf:resource="descriptor/18190-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14106-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17608-3">
    <skos:broader rdf:resource="thsys/70253"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17608-3">
    <skos:broader rdf:resource="descriptor/17553-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11811-1">
    <skos:broader rdf:resource="thsys/71012"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11811-1">
    <skos:broader rdf:resource="thsys/70539"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11811-1">
    <skos:broader rdf:resource="descriptor/11810-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19843-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73365">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19497-4">
    <skos:broader rdf:resource="thsys/71061"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19497-4">
    <skos:broader rdf:resource="descriptor/11235-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18456-2">
    <skos:altLabel xml:lang="de">Wirtschaftszweigsystematik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17678-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19031-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24859-1">
    <skos:scopeNote xml:lang="de">Erfüllung von Zahlungsverpflichtungen aus Geschäften/Verträgen. Auch Erfüllung staatlicher Zahlungsverpflichtungen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11079-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20744-3">
    <skos:related rdf:resource="descriptor/19481-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20744-3">
    <skos:related rdf:resource="descriptor/20967-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20744-3">
    <skos:related rdf:resource="descriptor/15776-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12170-1">
    <skos:broader rdf:resource="thsys/71117"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12170-1">
    <skos:broader rdf:resource="thsys/70823"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12170-1">
    <skos:broader rdf:resource="descriptor/10141-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13511-3">
    <skos:prefLabel xml:lang="en">Transport history</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13511-3">
    <skos:prefLabel xml:lang="de">Verkehrsgeschichte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17589-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19078-3">
    <skos:related rdf:resource="descriptor/19837-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19078-3">
    <skos:related rdf:resource="descriptor/17461-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71037">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">B.09.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12168-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11068-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13386-1">
    <skos:broader rdf:resource="thsys/70825"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13386-1">
    <skos:broader rdf:resource="descriptor/13347-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10147-6">
    <skos:prefLabel xml:lang="en">Search theory</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10147-6">
    <skos:prefLabel xml:lang="de">Suchtheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19693-4">
    <skos:prefLabel xml:lang="en">Parliamentary allowance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19693-4">
    <skos:prefLabel xml:lang="de">Diäten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24859-1">
    <skos:broader rdf:resource="thsys/70037"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id119">
    <rdf:value xml:lang="de">Für betriebswirtschaftliche Ansätze der Produktionstheorie</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19156-2">
    <skos:prefLabel xml:lang="en">Advisory board</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19156-2">
    <skos:prefLabel xml:lang="de">Beirat</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13247-1">
    <skos:altLabel xml:lang="de">Luftverkehrsunternehmen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13247-1">
    <skos:altLabel xml:lang="de">Lufttransportgesellschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13247-1">
    <skos:altLabel xml:lang="de">Luftfahrtgesellschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13247-1">
    <skos:altLabel xml:lang="de">Fluggesellschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72181">
    <skos:broader rdf:resource="thsys/72180"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70012">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.06</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13336-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19845-0">
    <skos:altLabel xml:lang="de">Offenlegungspflicht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19338-3">
    <skos:broader rdf:resource="thsys/70986"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19338-3">
    <skos:broader rdf:resource="thsys/70960"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19338-3">
    <skos:broader rdf:resource="descriptor/10012-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10811-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10010-5">
    <skos:narrower rdf:resource="descriptor/19394-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10010-5">
    <skos:narrower rdf:resource="descriptor/10276-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10010-5">
    <skos:narrower rdf:resource="descriptor/19388-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10010-5">
    <skos:narrower rdf:resource="descriptor/18989-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10010-5">
    <skos:narrower rdf:resource="descriptor/16668-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11337-3">
    <skos:related rdf:resource="descriptor/16484-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11337-3">
    <skos:related rdf:resource="descriptor/11346-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11337-3">
    <skos:related rdf:resource="descriptor/11163-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11337-3">
    <skos:related rdf:resource="descriptor/18848-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11337-3">
    <skos:related rdf:resource="descriptor/11168-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11337-3">
    <skos:related rdf:resource="descriptor/19502-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10836-4">
    <skos:broader rdf:resource="thsys/71091"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10836-4">
    <skos:broader rdf:resource="descriptor/10833-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11182-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16961-6">
    <skos:altLabel xml:lang="x-other">Geneve</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16961-6">
    <skos:altLabel xml:lang="de">Genf (Stadt)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16961-6">
    <skos:altLabel xml:lang="de">Genf (Kanton)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12446-0">
    <skos:related rdf:resource="descriptor/12796-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12446-0">
    <skos:related rdf:resource="descriptor/10831-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12446-0">
    <skos:related rdf:resource="descriptor/10830-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12446-0">
    <skos:related rdf:resource="descriptor/12062-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12446-0">
    <skos:related rdf:resource="descriptor/12452-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12446-0">
    <skos:related rdf:resource="descriptor/12056-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12446-0">
    <skos:related rdf:resource="descriptor/12275-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18068-4">
    <skos:prefLabel xml:lang="en">Legal reform</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18068-4">
    <skos:prefLabel xml:lang="de">Rechtsreform</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18160-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15945-2">
    <skos:scopeNote xml:lang="de">Die Geschlechtszugehörigkeit und geschlechtsspezifische Unterschiede als thematischer Aspekt</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13213-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19121-0">
    <skos:prefLabel xml:lang="en">Marketing theory</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19121-0">
    <skos:prefLabel xml:lang="de">Marketingtheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18012-3">
    <skos:narrower rdf:resource="descriptor/18136-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18012-3">
    <skos:narrower rdf:resource="descriptor/19076-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18012-3">
    <skos:narrower rdf:resource="descriptor/16914-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18012-3">
    <skos:narrower rdf:resource="descriptor/18135-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18012-3">
    <skos:narrower rdf:resource="descriptor/18134-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18012-3">
    <skos:narrower rdf:resource="descriptor/16912-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18012-3">
    <skos:narrower rdf:resource="descriptor/16909-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18012-3">
    <skos:narrower rdf:resource="descriptor/16898-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18012-3">
    <skos:narrower rdf:resource="descriptor/19075-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18012-3">
    <skos:narrower rdf:resource="descriptor/16896-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18012-3">
    <skos:narrower rdf:resource="descriptor/18131-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18012-3">
    <skos:narrower rdf:resource="descriptor/18133-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18012-3">
    <skos:narrower rdf:resource="descriptor/16874-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18012-3">
    <skos:narrower rdf:resource="descriptor/16868-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18012-3">
    <skos:narrower rdf:resource="descriptor/16834-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18012-3">
    <skos:narrower rdf:resource="descriptor/16927-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18012-3">
    <skos:narrower rdf:resource="descriptor/16849-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18012-3">
    <skos:narrower rdf:resource="descriptor/18132-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18012-3">
    <skos:narrower rdf:resource="descriptor/16830-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18012-3">
    <skos:narrower rdf:resource="descriptor/16842-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18012-3">
    <skos:narrower rdf:resource="descriptor/16841-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18012-3">
    <skos:narrower rdf:resource="descriptor/16853-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18560-0">
    <skos:scopeNote xml:lang="de">Versorgung mit verzehrfertigen Speisen, die auf industrielle Weise vor- und zubereitet sind</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15793-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18274-1">
    <skos:broader rdf:resource="thsys/71015"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18274-1">
    <skos:broader rdf:resource="thsys/71014"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18274-1">
    <skos:broader rdf:resource="descriptor/14407-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15961-4">
    <skos:scopeNote xml:lang="de">Bedeutet allgemein das Recht auf Informationen bzw. sich Informationen zu beschaffen; gilt sowohl für öffentlich relevante Ereignisse und Vorgänge wie auch im Rahmen des Gesellschaftsrechts.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12705-2">
    <skos:altLabel xml:lang="de">Industrieanlage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12705-2">
    <skos:altLabel xml:lang="de">Fertigungsanlage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12705-2">
    <skos:altLabel xml:lang="de">Betriebsvorrichtung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12705-2">
    <skos:altLabel xml:lang="de">Betriebsmittel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12705-2">
    <skos:altLabel xml:lang="de">Betriebsanlage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12705-2">
    <skos:altLabel xml:lang="de">Anlagenwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12705-2">
    <skos:altLabel xml:lang="de">Anlagenmanagement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14905-5">
    <skos:broader rdf:resource="thsys/70031"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14905-5">
    <skos:broader rdf:resource="descriptor/14900-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16417-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17784-4">
    <skos:altLabel xml:lang="de">Republik Honduras</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10499-3">
    <skos:altLabel xml:lang="en">High-income countries</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10499-3">
    <skos:altLabel xml:lang="de">Industrieländer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11553-4">
    <skos:broader rdf:resource="thsys/71081"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11553-4">
    <skos:broader rdf:resource="thsys/71064"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11553-4">
    <skos:broader rdf:resource="descriptor/11554-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11553-4">
    <skos:broader rdf:resource="descriptor/11537-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16768-0">
    <skos:broader rdf:resource="thsys/70822"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16768-0">
    <skos:broader rdf:resource="descriptor/16797-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19564-1">
    <skos:scopeNote xml:lang="de">Information an der Ware, die über Qualitätseigenschaften Auskunft geben soll; teils freiwillig, teils aufgrund rechtlicher Normen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18314-1">
    <skos:related rdf:resource="descriptor/13428-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18314-1">
    <skos:related rdf:resource="descriptor/10583-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18314-1">
    <skos:related rdf:resource="descriptor/19840-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18314-1">
    <skos:related rdf:resource="descriptor/18335-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18314-1">
    <skos:related rdf:resource="descriptor/10495-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18314-1">
    <skos:related rdf:resource="descriptor/10488-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18287-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19115-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11036-3">
    <skos:related rdf:resource="descriptor/18843-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11036-3">
    <skos:related rdf:resource="descriptor/11310-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11036-3">
    <skos:related rdf:resource="descriptor/11048-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11036-3">
    <skos:related rdf:resource="descriptor/11044-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11036-3">
    <skos:related rdf:resource="descriptor/11769-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11290-3">
    <skos:altLabel xml:lang="de">Berufssystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11290-3">
    <skos:altLabel xml:lang="de">Berufskunde</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11290-3">
    <skos:altLabel xml:lang="de">Berufsgruppe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11290-3">
    <skos:altLabel xml:lang="de">Berufsfeld</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11290-3">
    <skos:altLabel xml:lang="de">Ausbildungsberuf</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11290-3">
    <skos:altLabel xml:lang="de">Anlernberuf</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18018-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11523-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14005-6">
    <skos:related rdf:resource="descriptor/13192-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70051">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19298-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12403-4">
    <skos:prefLabel xml:lang="en">Loss</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12403-4">
    <skos:prefLabel xml:lang="de">Verlust</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11346-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14100-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16061-0">
    <skos:prefLabel xml:lang="en">Regulation of investment funds</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16061-0">
    <skos:prefLabel xml:lang="de">Investmentrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16198-1">
    <skos:altLabel xml:lang="de">Strafprozessrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16198-1">
    <skos:altLabel xml:lang="de">Strafprozeßrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16198-1">
    <skos:altLabel xml:lang="de">Strafgesetzbuch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16198-1">
    <skos:altLabel xml:lang="de">Ordnungswidrigkeitenrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10452-1">
    <skos:prefLabel xml:lang="en">International technology cooperation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10452-1">
    <skos:prefLabel xml:lang="de">Internationale technische Zusammenarbeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16540-2">
    <skos:narrower rdf:resource="descriptor/18025-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14124-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19731-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15982-3">
    <skos:prefLabel xml:lang="en">Contract</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15982-3">
    <skos:prefLabel xml:lang="de">Vertrag</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16286-4">
    <skos:prefLabel xml:lang="en">Revolution</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16286-4">
    <skos:prefLabel xml:lang="de">Politische Unruhen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17658-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15887-4">
    <skos:prefLabel xml:lang="en">Ocean</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15887-4">
    <skos:prefLabel xml:lang="de">Meer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11833-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10353-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12523-1">
    <skos:broader rdf:resource="thsys/70578"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12523-1">
    <skos:broader rdf:resource="descriptor/12480-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11495-6">
    <skos:altLabel xml:lang="en">Expected inflation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11495-6">
    <skos:altLabel xml:lang="en">Anticipated inflation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11495-6">
    <skos:altLabel xml:lang="de">Inflationsantizipation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11495-6">
    <skos:altLabel xml:lang="de">Erwartete Inflation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11495-6">
    <skos:altLabel xml:lang="de">Antizipierte Inflation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16927-6">
    <skos:prefLabel xml:lang="en">German Democratic Republic</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16927-6">
    <skos:prefLabel xml:lang="de">DDR</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19064-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19447-5">
    <skos:related rdf:resource="descriptor/19437-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13744-1">
    <skos:related rdf:resource="descriptor/13769-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12589-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11638-3">
    <skos:altLabel xml:lang="de">Steuerpsychologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11638-3">
    <skos:altLabel xml:lang="de">Steuermentalität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11638-3">
    <skos:altLabel xml:lang="de">Steuerehrlichkeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11266-0">
    <skos:prefLabel xml:lang="en">Labour demand</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11266-0">
    <skos:prefLabel xml:lang="de">Arbeitsnachfrage</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10944-1">
    <skos:broader rdf:resource="thsys/70242"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10944-1">
    <skos:broader rdf:resource="descriptor/10956-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19279-0">
    <skos:broader rdf:resource="thsys/71106"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19279-0">
    <skos:broader rdf:resource="descriptor/13714-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17967-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15889-0">
    <skos:prefLabel xml:lang="en">Natural disaster</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15889-0">
    <skos:prefLabel xml:lang="de">Naturkatastrophe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70870">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12488-5">
    <skos:related rdf:resource="descriptor/12318-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12488-5">
    <skos:related rdf:resource="descriptor/12485-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70256">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">P.09.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10877-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13401-3">
    <skos:altLabel xml:lang="de">Verbraucherverband</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13401-3">
    <skos:altLabel xml:lang="de">Verbraucherberatung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10190-5">
    <skos:altLabel xml:lang="de">Theorie der Firma</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18253-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12737-3">
    <skos:broader rdf:resource="thsys/71036"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12737-3">
    <skos:broader rdf:resource="thsys/70549"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12737-3">
    <skos:broader rdf:resource="thsys/70471"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12737-3">
    <skos:broader rdf:resource="descriptor/12736-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14355-0">
    <skos:altLabel xml:lang="de">Turbine</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14355-0">
    <skos:altLabel xml:lang="de">Dampfmaschine</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15751-1">
    <skos:related rdf:resource="descriptor/18377-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15751-1">
    <skos:related rdf:resource="descriptor/15754-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15751-1">
    <skos:related rdf:resource="descriptor/10411-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15751-1">
    <skos:related rdf:resource="descriptor/18286-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18280-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16280-2">
    <skos:broader rdf:resource="thsys/73378"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13862-2">
    <skos:related rdf:resource="descriptor/13865-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13862-2">
    <skos:related rdf:resource="descriptor/19532-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13862-2">
    <skos:related rdf:resource="descriptor/11932-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13862-2">
    <skos:related rdf:resource="descriptor/11973-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13862-2">
    <skos:related rdf:resource="descriptor/19452-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13862-2">
    <skos:related rdf:resource="descriptor/13861-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13862-2">
    <skos:related rdf:resource="descriptor/18550-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13509-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11274-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11293-4">
    <skos:prefLabel xml:lang="en">Occupational research</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11293-4">
    <skos:prefLabel xml:lang="de">Berufsforschung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14525-1">
    <skos:broader rdf:resource="thsys/70373"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14525-1">
    <skos:broader rdf:resource="descriptor/14522-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10282-0">
    <skos:scopeNote xml:lang="de">Theoretische Annahmen über die Abhängigkeit des Konsums vom Einkommen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11807-6">
    <skos:prefLabel xml:lang="en">Fishery policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11807-6">
    <skos:prefLabel xml:lang="de">Fischereipolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11228-1">
    <skos:prefLabel xml:lang="en">In-service training</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11228-1">
    <skos:prefLabel xml:lang="de">Berufsbegleitende Ausbildung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16445-3">
    <skos:related rdf:resource="descriptor/16497-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16445-3">
    <skos:related rdf:resource="descriptor/16204-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16445-3">
    <skos:related rdf:resource="descriptor/16427-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15655-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18436-1">
    <skos:prefLabel xml:lang="en">Textile retailer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18436-1">
    <skos:prefLabel xml:lang="de">Textileinzelhandel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13852-5">
    <skos:broader rdf:resource="thsys/70970"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13852-5">
    <skos:broader rdf:resource="descriptor/10012-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13450-4">
    <skos:broader rdf:resource="thsys/71028"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13450-4">
    <skos:broader rdf:resource="descriptor/10470-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19618-4">
    <skos:broader rdf:resource="thsys/73353"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19618-4">
    <skos:broader rdf:resource="descriptor/15887-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19618-4">
    <skos:broader rdf:resource="descriptor/12015-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71062">
    <rdfs:label xml:lang="en">Labour market theory</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71062">
    <rdfs:label xml:lang="de">Arbeitsmarkttheorie</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16496-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19760-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18685-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70310">
    <rdfs:label xml:lang="de">Fertigungssysteme und Fertigungstechnik</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13210-3">
    <skos:narrower rdf:resource="descriptor/13238-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13210-3">
    <skos:narrower rdf:resource="descriptor/19063-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13210-3">
    <skos:narrower rdf:resource="descriptor/13472-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13210-3">
    <skos:narrower rdf:resource="descriptor/13221-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13210-3">
    <skos:narrower rdf:resource="descriptor/13219-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13210-3">
    <skos:narrower rdf:resource="descriptor/13188-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13210-3">
    <skos:narrower rdf:resource="descriptor/13217-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13210-3">
    <skos:narrower rdf:resource="descriptor/18522-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13210-3">
    <skos:narrower rdf:resource="descriptor/13209-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13210-3">
    <skos:narrower rdf:resource="descriptor/13200-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13210-3">
    <skos:narrower rdf:resource="descriptor/13198-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13210-3">
    <skos:narrower rdf:resource="descriptor/13194-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15606-3">
    <skos:altLabel xml:lang="de">Elektronische Datenverarbeitung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15606-3">
    <skos:altLabel xml:lang="de">EDV</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15606-3">
    <skos:altLabel xml:lang="de">Automatisierte Datenverarbeitung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15606-3">
    <skos:altLabel xml:lang="de">ADV</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15606-3">
    <skos:altLabel xml:lang="de">Administrative Datenverarbeitung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13633-3">
    <skos:related rdf:resource="descriptor/12013-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16363-5">
    <skos:prefLabel xml:lang="en">Liberal party</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16363-5">
    <skos:prefLabel xml:lang="de">Liberale Partei</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id332">
    <rdf:value xml:lang="de">Für die allgemeine Tier- bzw. Viehhaltung</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70985">
    <rdfs:label xml:lang="de">Verpackungswirtschaft</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73305">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">P.10.04</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18376-0">
    <skos:prefLabel xml:lang="en">Energy tax</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18376-0">
    <skos:prefLabel xml:lang="de">Energiesteuer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13287-3">
    <skos:broader rdf:resource="thsys/70903"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13287-3">
    <skos:broader rdf:resource="thsys/70854"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13287-3">
    <skos:broader rdf:resource="descriptor/12180-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13287-3">
    <skos:broader rdf:resource="descriptor/13283-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10210-4">
    <skos:altLabel xml:lang="en">Contingent-claims approach</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10210-4">
    <skos:altLabel xml:lang="de">Optionspreismodell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10210-4">
    <skos:altLabel xml:lang="de">Optionsbewertung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73372">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.04.03.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14911-3">
    <skos:prefLabel xml:lang="en">Headgear</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14911-3">
    <skos:prefLabel xml:lang="de">Kopfbedeckung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11473-2">
    <skos:altLabel xml:lang="de">Verkehrsgleichung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11473-2">
    <skos:altLabel xml:lang="de">Quantitätsgleichung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11473-2">
    <skos:altLabel xml:lang="de">Fishersche Quantitätsgleichung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19366-5">
    <skos:prefLabel xml:lang="en">Rubber market</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19366-5">
    <skos:prefLabel xml:lang="de">Kautschukmarkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13829-0">
    <zbwext:useInsteadNote rdf:nodeID="id379"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15606-3">
    <skos:narrower rdf:resource="descriptor/13673-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15606-3">
    <skos:narrower rdf:resource="descriptor/13277-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15606-3">
    <skos:narrower rdf:resource="descriptor/18170-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18640-2">
    <skos:prefLabel xml:lang="en">Ecological effects of tourism</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18640-2">
    <skos:prefLabel xml:lang="de">Tourismusökologie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19060-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15115-1">
    <skos:prefLabel xml:lang="en">Data collection</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15115-1">
    <skos:prefLabel xml:lang="de">Statistische Erhebung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10905-4">
    <skos:related rdf:resource="descriptor/10950-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10905-4">
    <skos:related rdf:resource="descriptor/10201-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10905-4">
    <skos:related rdf:resource="descriptor/10904-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10905-4">
    <skos:related rdf:resource="descriptor/19827-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10905-4">
    <skos:related rdf:resource="descriptor/10012-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10905-4">
    <skos:related rdf:resource="descriptor/13690-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13422-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19831-4">
    <skos:prefLabel xml:lang="en">Gemeinlastprinzip</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19831-4">
    <skos:prefLabel xml:lang="de">Gemeinlastprinzip</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12308-5">
    <skos:related rdf:resource="descriptor/12312-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20817-2">
    <skos:related rdf:resource="descriptor/18130-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20817-2">
    <skos:related rdf:resource="descriptor/10589-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20817-2">
    <skos:related rdf:resource="descriptor/19071-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12918-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10125-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18155-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12977-4">
    <skos:broader rdf:resource="thsys/71015"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12977-4">
    <skos:broader rdf:resource="descriptor/14039-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18371-3">
    <skos:related rdf:resource="descriptor/12978-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18371-3">
    <skos:related rdf:resource="descriptor/19385-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18371-3">
    <skos:related rdf:resource="descriptor/14196-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11567-0">
    <skos:broader rdf:resource="thsys/71080"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11567-0">
    <skos:broader rdf:resource="thsys/70986"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11567-0">
    <skos:broader rdf:resource="thsys/70833"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11567-0">
    <skos:broader rdf:resource="thsys/70324"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11567-0">
    <skos:broader rdf:resource="descriptor/11572-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11567-0">
    <skos:broader rdf:resource="descriptor/19510-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15690-2">
    <skos:related rdf:resource="descriptor/19784-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15690-2">
    <skos:related rdf:resource="descriptor/19635-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16001-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18662-6">
    <skos:prefLabel xml:lang="en">Network management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18662-6">
    <skos:prefLabel xml:lang="de">Netzwerkmanagement</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70843">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13560-4">
    <skos:related rdf:resource="descriptor/18579-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13560-4">
    <skos:related rdf:resource="descriptor/13563-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13560-4">
    <skos:related rdf:resource="descriptor/13176-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13560-4">
    <skos:related rdf:resource="descriptor/13340-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10985-1">
    <skos:related rdf:resource="descriptor/11124-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10985-1">
    <skos:related rdf:resource="descriptor/11142-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10985-1">
    <skos:related rdf:resource="descriptor/12064-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10985-1">
    <skos:related rdf:resource="descriptor/11061-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10985-1">
    <skos:related rdf:resource="descriptor/11628-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10985-1">
    <skos:related rdf:resource="descriptor/19235-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10985-1">
    <skos:related rdf:resource="descriptor/16409-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10985-1">
    <skos:related rdf:resource="descriptor/11044-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13513-6">
    <skos:broader rdf:resource="thsys/70943"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13513-6">
    <skos:broader rdf:resource="thsys/70257"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19764-0">
    <skos:broader rdf:resource="thsys/73356"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19764-0">
    <skos:broader rdf:resource="thsys/73336"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19764-0">
    <skos:broader rdf:resource="descriptor/19756-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11535-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12768-6">
    <skos:narrower rdf:resource="descriptor/18561-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12768-6">
    <skos:narrower rdf:resource="descriptor/19120-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16493-6">
    <skos:related rdf:resource="descriptor/16440-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16493-6">
    <skos:related rdf:resource="descriptor/13407-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16493-6">
    <skos:related rdf:resource="descriptor/16489-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21041-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10334-0">
    <skos:related rdf:resource="descriptor/19200-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10334-0">
    <skos:related rdf:resource="descriptor/19490-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10334-0">
    <skos:related rdf:resource="descriptor/10315-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10334-0">
    <skos:related rdf:resource="descriptor/10307-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10334-0">
    <skos:related rdf:resource="descriptor/10305-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14581-5">
    <skos:related rdf:resource="descriptor/14522-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14581-5">
    <skos:related rdf:resource="descriptor/14586-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14581-5">
    <skos:related rdf:resource="descriptor/13096-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14581-5">
    <skos:related rdf:resource="descriptor/13233-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14581-5">
    <skos:related rdf:resource="descriptor/15925-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14581-5">
    <skos:related rdf:resource="descriptor/14029-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14581-5">
    <skos:related rdf:resource="descriptor/14019-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18339-6">
    <skos:related rdf:resource="descriptor/12961-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18339-6">
    <skos:related rdf:resource="descriptor/13446-2"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="r1238766808r28278r1">
    <rdf:type rdf:resource="http://www.w3.org/2002/07/owl#Class"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15931-6">
    <skos:broader rdf:resource="thsys/73358"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15931-6">
    <skos:broader rdf:resource="descriptor/15712-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15938-6">
    <skos:editorialNote xml:lang="de">Möglichst spezifischer indexieren.</skos:editorialNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18492-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12392-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12668-3">
    <skos:altLabel xml:lang="de">Einkaufsverbund</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12668-3">
    <skos:altLabel xml:lang="de">Einkaufsverband</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12668-3">
    <skos:altLabel xml:lang="de">Einkaufsgemeinschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19349-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11850-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18129-3">
    <skos:related rdf:resource="descriptor/19200-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18129-3">
    <skos:related rdf:resource="descriptor/19490-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18129-3">
    <skos:related rdf:resource="descriptor/10315-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15917-0">
    <skos:altLabel xml:lang="de">Yuit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15917-0">
    <skos:altLabel xml:lang="de">Eskimos</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15917-0">
    <skos:altLabel xml:lang="de">Eskimo</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16639-4">
    <skos:related rdf:resource="descriptor/15715-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16639-4">
    <skos:related rdf:resource="descriptor/19754-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16639-4">
    <skos:related rdf:resource="descriptor/16638-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16639-4">
    <skos:related rdf:resource="descriptor/15069-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16639-4">
    <skos:related rdf:resource="descriptor/11432-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16639-4">
    <skos:related rdf:resource="descriptor/18948-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10492-3">
    <skos:broader rdf:resource="thsys/70307"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10492-3">
    <skos:broader rdf:resource="thsys/70091"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id152">
    <rdfs:seeAlso rdf:resource="descriptor/10598-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14755-5">
    <skos:prefLabel xml:lang="en">Mainframe computer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14755-5">
    <skos:prefLabel xml:lang="de">Großrechner</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10339-4">
    <skos:related rdf:resource="descriptor/10261-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10339-4">
    <skos:related rdf:resource="descriptor/10387-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10339-4">
    <skos:related rdf:resource="descriptor/10338-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13817-0">
    <skos:prefLabel xml:lang="en">Engineering insurance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13817-0">
    <skos:prefLabel xml:lang="de">Technische Versicherung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12005-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11956-3">
    <skos:prefLabel xml:lang="en">Gravity model</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11956-3">
    <skos:prefLabel xml:lang="de">Gravitationsmodell</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70824">
    <skos:broader rdf:resource="thsys/70825"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19598-5">
    <skos:altLabel xml:lang="en">Analytical hierarchy process</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19598-5">
    <skos:altLabel xml:lang="de">Analytischer Hierarchie-Prozess</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id182">
    <rdf:value xml:lang="de">Für gemeinsame Identitäten, Mentalitäten, Konventionen u.ä. von ethnischen Gruppen (Nation, Volk, Volksgruppe)</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12955-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18079-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18624-0">
    <skos:narrower rdf:resource="descriptor/13561-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18624-0">
    <skos:narrower rdf:resource="descriptor/18590-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18624-0">
    <skos:narrower rdf:resource="descriptor/18571-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15470-2">
    <skos:broader rdf:resource="thsys/73103"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15470-2">
    <skos:broader rdf:resource="thsys/71035"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15470-2">
    <skos:broader rdf:resource="descriptor/15685-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19237-2">
    <skos:broader rdf:resource="thsys/71111"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19237-2">
    <skos:broader rdf:resource="descriptor/10272-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20002-5">
    <skos:altLabel xml:lang="en">Grey market goods</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18280-6">
    <skos:prefLabel xml:lang="en">Superconductor</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18280-6">
    <skos:prefLabel xml:lang="de">Supraleiter</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id215">
    <rdf:value xml:lang="de">für die Branche</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11506-6">
    <skos:altLabel xml:lang="de">Preisniveauanstieg</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11506-6">
    <skos:altLabel xml:lang="de">Kaufkraftverlust des Geldes</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11506-6">
    <skos:altLabel xml:lang="de">Kaufkraftüberhang</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11506-6">
    <skos:altLabel xml:lang="de">Geldüberhang</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11506-6">
    <skos:altLabel xml:lang="de">Geldentwertung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10781-3">
    <skos:broader rdf:resource="thsys/71103"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10781-3">
    <skos:broader rdf:resource="thsys/71088"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10781-3">
    <skos:broader rdf:resource="descriptor/10021-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12515-0">
    <skos:prefLabel xml:lang="en">Direct costing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12515-0">
    <skos:prefLabel xml:lang="de">Direct Costing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19414-6">
    <skos:prefLabel xml:lang="en">Export tax</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19414-6">
    <skos:prefLabel xml:lang="de">Exportsteuer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14634-3">
    <skos:related rdf:resource="descriptor/18253-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14634-3">
    <skos:related rdf:resource="descriptor/14176-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16081-1">
    <skos:prefLabel xml:lang="en">Economic law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16081-1">
    <skos:prefLabel xml:lang="de">Wirtschaftsrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11704-2">
    <skos:prefLabel xml:lang="en">Tax policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11704-2">
    <skos:prefLabel xml:lang="de">Steuerpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12164-3">
    <skos:altLabel xml:lang="de">Internes Berichtswesen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12164-3">
    <skos:altLabel xml:lang="de">Betriebliche Berichterstattung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71068">
    <skos:narrower rdf:resource="descriptor/11903-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71068">
    <skos:narrower rdf:resource="descriptor/11880-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71068">
    <skos:narrower rdf:resource="descriptor/11879-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71068">
    <skos:narrower rdf:resource="descriptor/18967-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71068">
    <skos:narrower rdf:resource="descriptor/11875-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71068">
    <skos:narrower rdf:resource="descriptor/11900-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71068">
    <skos:narrower rdf:resource="descriptor/11870-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71068">
    <skos:narrower rdf:resource="descriptor/19450-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71068">
    <skos:narrower rdf:resource="descriptor/11869-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71068">
    <skos:narrower rdf:resource="descriptor/19449-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71068">
    <skos:narrower rdf:resource="descriptor/11866-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71068">
    <skos:narrower rdf:resource="descriptor/10562-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71068">
    <skos:narrower rdf:resource="descriptor/16266-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71068">
    <skos:narrower rdf:resource="descriptor/11859-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19596-2">
    <skos:broader rdf:resource="thsys/73104"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19596-2">
    <skos:broader rdf:resource="descriptor/15458-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15739-5">
    <skos:broader rdf:resource="thsys/70238"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15739-5">
    <skos:broader rdf:resource="descriptor/15763-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16026-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19051-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11055-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11233-1">
    <zbwext:useInsteadNote rdf:nodeID="id292"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19177-1">
    <skos:prefLabel xml:lang="en">G-5 countries</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19177-1">
    <skos:prefLabel xml:lang="de">G-5-Staaten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19774-4">
    <skos:altLabel xml:lang="en">Out-of-wedlock birth</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19774-4">
    <skos:altLabel xml:lang="en">Illegitimate birth</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19774-4">
    <skos:altLabel xml:lang="de">Unehelichkeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19774-4">
    <skos:altLabel xml:lang="de">Nichtehelichkeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19774-4">
    <skos:altLabel xml:lang="de">Nichteheliches Kind</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11507-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19601-0">
    <skos:broader rdf:resource="thsys/70017"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11007-3">
    <skos:altLabel xml:lang="en">Planification</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11007-3">
    <skos:altLabel xml:lang="en">Indirect planning</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19574-5">
    <skos:altLabel xml:lang="en">Generalized autoregressive conditional heteroscedasticity</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19574-5">
    <skos:altLabel xml:lang="en">GARCH model</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19574-5">
    <skos:altLabel xml:lang="en">Autoregressive conditional heteroscedasticity</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19574-5">
    <skos:altLabel xml:lang="de">GARCH-Modell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14534-0">
    <skos:altLabel xml:lang="de">Photokopiergerät</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14534-0">
    <skos:altLabel xml:lang="de">Fotokopiergerät</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14534-0">
    <skos:altLabel xml:lang="de">Farbkopiergerät</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10462-5">
    <zbwext:useInsteadNote rdf:nodeID="id215"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10462-5">
    <zbwext:useInsteadNote rdf:nodeID="id214"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id70">
    <rdfs:seeAlso rdf:resource="descriptor/12137-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19120-2">
    <skos:scopeNote xml:lang="de">Computergestützte Akquisitions- und Verkaufsmethoden; auch für Computereinsatz im Außendienst.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19239-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13755-3">
    <skos:broader rdf:resource="thsys/70900"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13755-3">
    <skos:broader rdf:resource="descriptor/19660-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12309-3">
    <skos:prefLabel xml:lang="en">Corporate Budgeting</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12309-3">
    <skos:prefLabel xml:lang="de">Betriebliche Budgetierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10094-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13746-4">
    <skos:related rdf:resource="descriptor/13763-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13746-4">
    <skos:related rdf:resource="descriptor/11484-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13746-4">
    <skos:related rdf:resource="descriptor/19665-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13746-4">
    <skos:related rdf:resource="descriptor/18722-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13746-4">
    <skos:related rdf:resource="descriptor/13692-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13746-4">
    <skos:related rdf:resource="descriptor/13688-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16033-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19024-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11320-6">
    <zbwext:useInsteadNote rdf:nodeID="id108"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19467-6">
    <skos:narrower rdf:resource="descriptor/10510-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19467-6">
    <skos:narrower rdf:resource="descriptor/19458-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12090-6">
    <skos:prefLabel xml:lang="en">Corporate constitution</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12090-6">
    <skos:prefLabel xml:lang="de">Unternehmensverfassung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70038">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16369-0">
    <skos:broader rdf:resource="thsys/73378"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16369-0">
    <skos:broader rdf:resource="descriptor/19696-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17677-5">
    <skos:broader rdf:resource="thsys/70445"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17677-5">
    <skos:broader rdf:resource="descriptor/17672-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17677-5">
    <skos:broader rdf:resource="descriptor/17978-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10615-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12724-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14462-6">
    <skos:related rdf:resource="descriptor/18598-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70148">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13561-2">
    <skos:related rdf:resource="descriptor/13562-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13561-2">
    <skos:related rdf:resource="descriptor/13176-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13561-2">
    <skos:related rdf:resource="descriptor/11667-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10015-2">
    <skos:related rdf:resource="descriptor/12294-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10015-2">
    <skos:related rdf:resource="descriptor/12807-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10015-2">
    <skos:related rdf:resource="descriptor/12705-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10015-2">
    <skos:related rdf:resource="descriptor/12208-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10015-2">
    <skos:related rdf:resource="descriptor/12278-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10015-2">
    <skos:related rdf:resource="descriptor/18107-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10015-2">
    <skos:related rdf:resource="descriptor/19094-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10015-2">
    <skos:related rdf:resource="descriptor/12543-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10272-3">
    <skos:broader rdf:resource="thsys/71111"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15764-6">
    <skos:related rdf:resource="descriptor/13400-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12808-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12984-0">
    <skos:prefLabel xml:lang="en">Petroleum production</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12984-0">
    <skos:prefLabel xml:lang="de">Erdölförderung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12110-5">
    <skos:broader rdf:resource="thsys/73355"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12110-5">
    <skos:broader rdf:resource="descriptor/16564-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12110-5">
    <skos:broader rdf:resource="descriptor/12106-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19706-0">
    <skos:broader rdf:resource="thsys/73369"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19706-0">
    <skos:broader rdf:resource="descriptor/16515-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10300-3">
    <skos:prefLabel xml:lang="en">Consumer behaviour</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10300-3">
    <skos:prefLabel xml:lang="de">Konsumentenverhalten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19975-1">
    <skos:scopeNote xml:lang="de">Begünstigung aufgrund verwandtschaftlicher, freundschaftlicher oder beruflicher/geschäftlicher Beziehungen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18838-2">
    <skos:scopeNote xml:lang="de">Geschichte der genossenschaftlichen Idee und des Genossenschaftswesens.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19182-1">
    <skos:altLabel xml:lang="de">Erdölmarkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12777-5">
    <skos:altLabel xml:lang="de">Kooperatives Marketing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12777-5">
    <skos:altLabel xml:lang="de">Gemeinschaftsmarketing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19471-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17772-4">
    <skos:broader rdf:resource="thsys/70086"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17772-4">
    <skos:broader rdf:resource="descriptor/17791-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15211-5">
    <skos:altLabel xml:lang="en">Student-T-model</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15211-5">
    <skos:altLabel xml:lang="en">Frequency distribution</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15211-5">
    <skos:altLabel xml:lang="en">Distribution function</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15211-5">
    <skos:altLabel xml:lang="en">Density function</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15211-5">
    <skos:altLabel xml:lang="en">Characteristic function</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15211-5">
    <skos:altLabel xml:lang="en">Binomial distribution</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15211-5">
    <skos:altLabel xml:lang="de">Weibull-Verteilung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15211-5">
    <skos:altLabel xml:lang="de">Wahrscheinlichkeitsverteilung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15211-5">
    <skos:altLabel xml:lang="de">Wahrscheinlichkeitsfunktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15211-5">
    <skos:altLabel xml:lang="de">Wahrscheinlichkeitsdichte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15211-5">
    <skos:altLabel xml:lang="de">Verteilungsfunktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15211-5">
    <skos:altLabel xml:lang="de">Verteilung (Statistik)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15211-5">
    <skos:altLabel xml:lang="de">T-Verteilung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15211-5">
    <skos:altLabel xml:lang="de">Testverteilung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15211-5">
    <skos:altLabel xml:lang="de">Student-Verteilung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15211-5">
    <skos:altLabel xml:lang="de">Standardnormalverteilung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15211-5">
    <skos:altLabel xml:lang="de">Poisson-Verteilung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15211-5">
    <skos:altLabel xml:lang="de">Normalverteilung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15211-5">
    <skos:altLabel xml:lang="de">Multivariate Distribution</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15211-5">
    <skos:altLabel xml:lang="de">Mehrdimensionale Verteilung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15211-5">
    <skos:altLabel xml:lang="de">Häufigkeitsverteilung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15211-5">
    <skos:altLabel xml:lang="de">Gauß-Verteilung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15211-5">
    <skos:altLabel xml:lang="de">Gamma-Verteilung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15211-5">
    <skos:altLabel xml:lang="de">F-Verteilung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15211-5">
    <skos:altLabel xml:lang="de">Dichtefunktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15211-5">
    <skos:altLabel xml:lang="de">Chi-Quadrat-Verteilung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15211-5">
    <skos:altLabel xml:lang="de">Charakteristische Funktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15211-5">
    <skos:altLabel xml:lang="de">Binomialverteilung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15211-5">
    <skos:altLabel xml:lang="de">Bernoulli-Verteilung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15717-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12755-1">
    <skos:altLabel xml:lang="en">Lot-sizing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12755-1">
    <skos:altLabel xml:lang="de">Produktionslos</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12755-1">
    <skos:altLabel xml:lang="de">Optimale Losgröße</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12755-1">
    <skos:altLabel xml:lang="de">Optimale Auftragsgröße</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12755-1">
    <skos:altLabel xml:lang="de">Losgrößenplanung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12755-1">
    <skos:altLabel xml:lang="de">Fertigungslos</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12755-1">
    <skos:altLabel xml:lang="de">Auftragsgröße</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18235-4">
    <skos:altLabel xml:lang="en">Non-renewable energy resource</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18235-4">
    <skos:altLabel xml:lang="de">Nicht-regenerierbarer Energierohstoff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18235-4">
    <skos:altLabel xml:lang="de">Fossiler Energierohstoff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18235-4">
    <skos:altLabel xml:lang="de">Fossiler Brennstoff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18235-4">
    <skos:altLabel xml:lang="de">Fossile Energie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18235-4">
    <skos:altLabel xml:lang="de">Erschöpfbarer Energieträger</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15475-6">
    <skos:related rdf:resource="descriptor/15220-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15475-6">
    <skos:related rdf:resource="descriptor/15554-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16449-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12905-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11653-0">
    <skos:altLabel xml:lang="de">Stadtsteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11653-0">
    <skos:altLabel xml:lang="de">Kommunalsteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11653-0">
    <skos:altLabel xml:lang="de">Kommunale Besteuerung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11653-0">
    <skos:altLabel xml:lang="de">Kommunalabgabe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11653-0">
    <skos:altLabel xml:lang="de">Gemeindeabgabe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16674-2">
    <skos:altLabel xml:lang="en">Social implications</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16674-2">
    <skos:altLabel xml:lang="en">Social impacts</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16674-2">
    <skos:altLabel xml:lang="de">Soziale Wirkungen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16674-2">
    <skos:altLabel xml:lang="de">Soziale Konsequenzen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16674-2">
    <skos:altLabel xml:lang="de">Soziale Implikationen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16674-2">
    <skos:altLabel xml:lang="de">Soziale Auswirkungen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16674-2">
    <skos:altLabel xml:lang="de">Gesellschaftliche Folgen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16826-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16552-2">
    <skos:altLabel xml:lang="de">Regionale Wirtschaftspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16552-2">
    <skos:altLabel xml:lang="de">Regionale Wirtschaftsförderung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16552-2">
    <skos:altLabel xml:lang="de">Regionale Strukturpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16552-2">
    <skos:altLabel xml:lang="de">Regionale Förderung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16552-2">
    <skos:altLabel xml:lang="de">Raumwirtschaftspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16552-2">
    <skos:altLabel xml:lang="de">Räumliche Wirtschaftspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16552-2">
    <skos:altLabel xml:lang="de">Räumliche Strukturpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16343-4">
    <skos:prefLabel xml:lang="en">Parliamentary committee</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16343-4">
    <skos:prefLabel xml:lang="de">Parlamentsausschuss</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11809-2">
    <skos:prefLabel xml:lang="en">Agrarian reform</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11809-2">
    <skos:prefLabel xml:lang="de">Agrarreform</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10421-5">
    <skos:prefLabel xml:lang="en">Monetary growth model</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10421-5">
    <skos:prefLabel xml:lang="de">Monetäre Wachstumstheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18519-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13423-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18071-1">
    <skos:altLabel xml:lang="en">Waste cleanup</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18071-1">
    <skos:altLabel xml:lang="de">Altlastensanierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11186-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12818-3">
    <skos:prefLabel xml:lang="en">Sales promotion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12818-3">
    <skos:prefLabel xml:lang="de">Verkaufsförderung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11668-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17433-5">
    <skos:prefLabel xml:lang="en">Netherlands</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17433-5">
    <skos:prefLabel xml:lang="de">Niederlande</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18482-1">
    <skos:broader rdf:resource="thsys/71080"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18482-1">
    <skos:broader rdf:resource="thsys/70979"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18482-1">
    <skos:broader rdf:resource="descriptor/11572-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18482-1">
    <skos:broader rdf:resource="descriptor/11820-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18455-4">
    <skos:related rdf:resource="descriptor/10016-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18455-4">
    <skos:related rdf:resource="descriptor/18456-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10002-4">
    <skos:prefLabel xml:lang="en">Order processing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10002-4">
    <skos:prefLabel xml:lang="de">Auftragsabwicklung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17658-2">
    <skos:broader rdf:resource="thsys/70445"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17658-2">
    <skos:broader rdf:resource="descriptor/17672-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10048-1">
    <skos:related rdf:resource="descriptor/10051-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14231-4">
    <skos:prefLabel xml:lang="en">Granite</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14231-4">
    <skos:prefLabel xml:lang="de">Granit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15738-0">
    <skos:prefLabel xml:lang="en">International environmental policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15738-0">
    <skos:prefLabel xml:lang="de">Internationale Umweltpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18080-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12607-2">
    <skos:altLabel xml:lang="en">Work structuring</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12607-2">
    <skos:altLabel xml:lang="de">Psychologische Arbeitsgestaltung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12607-2">
    <skos:altLabel xml:lang="de">Arbeitsstrukturierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12607-2">
    <skos:altLabel xml:lang="de">Arbeitsplatzgestaltung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12607-2">
    <skos:altLabel xml:lang="de">Arbeitsfeldgestaltung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15696-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18332-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10081-3">
    <skos:prefLabel xml:lang="en">Partial equilibrium</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10081-3">
    <skos:prefLabel xml:lang="de">Partielles Gleichgewicht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10763-5">
    <skos:prefLabel xml:lang="en">US Dollar</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10763-5">
    <skos:prefLabel xml:lang="de">US-Dollar</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10514-5">
    <skos:broader rdf:resource="thsys/70091"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12946-1">
    <zbwext:useInsteadNote rdf:nodeID="id219"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72184">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18294-2">
    <skos:altLabel xml:lang="x-other">Tren de alta Velocidad</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18294-2">
    <skos:altLabel xml:lang="x-other">Train a grande Vitesse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18294-2">
    <skos:altLabel xml:lang="x-other">TAV (Tren de alta Velocidad)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18294-2">
    <skos:altLabel xml:lang="de">TGV</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18294-2">
    <skos:altLabel xml:lang="de">Intercity-Express</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18294-2">
    <skos:altLabel xml:lang="de">ICE (Intercity-Express)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24665-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13437-3">
    <skos:prefLabel xml:lang="en">Agricultural tenancy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13437-3">
    <skos:prefLabel xml:lang="de">Landwirtschaftliche Pacht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15111-2">
    <skos:related rdf:resource="descriptor/15176-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15111-2">
    <skos:related rdf:resource="descriptor/15115-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70270">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">B.06.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11820-0">
    <skos:broader rdf:resource="thsys/71110"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11820-0">
    <skos:broader rdf:resource="thsys/70979"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11820-0">
    <skos:broader rdf:resource="descriptor/11817-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13401-3">
    <skos:broader rdf:resource="thsys/73378"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13401-3">
    <skos:broader rdf:resource="thsys/70853"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13401-3">
    <skos:broader rdf:resource="thsys/70814"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13401-3">
    <skos:broader rdf:resource="descriptor/10984-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19956-5">
    <skos:broader rdf:resource="thsys/73362"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19956-5">
    <skos:broader rdf:resource="thsys/71051"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19956-5">
    <skos:broader rdf:resource="thsys/70235"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19956-5">
    <skos:broader rdf:resource="descriptor/20449-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16761-0">
    <skos:related rdf:resource="descriptor/15798-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16761-0">
    <skos:related rdf:resource="descriptor/16760-2"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id382">
    <rdf:value xml:lang="de">für Zahlungsbedingungen</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13430-3">
    <skos:altLabel xml:lang="de">Landwirtschaftskredit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13480-2">
    <skos:prefLabel xml:lang="en">Cut-price shop</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13480-2">
    <skos:prefLabel xml:lang="de">Discounter</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13722-4">
    <skos:related rdf:resource="descriptor/18693-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13722-4">
    <skos:related rdf:resource="descriptor/18690-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18108-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18394-5">
    <skos:broader rdf:resource="thsys/71006"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18394-5">
    <skos:broader rdf:resource="descriptor/12987-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12535-1">
    <skos:broader rdf:resource="thsys/70578"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12535-1">
    <skos:broader rdf:resource="descriptor/12480-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13794-0">
    <skos:broader rdf:resource="thsys/70899"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13794-0">
    <skos:broader rdf:resource="descriptor/13769-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13193-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19431-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18137-4">
    <skos:altLabel xml:lang="en">Ecofarming</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18137-4">
    <skos:altLabel xml:lang="en">Eco-farming</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18137-4">
    <skos:altLabel xml:lang="en">Chemical-free farming</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18137-4">
    <skos:altLabel xml:lang="de">Umweltverträgliche Landwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18137-4">
    <skos:altLabel xml:lang="de">Ökologische Landwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18137-4">
    <skos:altLabel xml:lang="de">Ökolandbau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18137-4">
    <skos:altLabel xml:lang="de">Kontrolliert-biologischer Anbau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18137-4">
    <skos:altLabel xml:lang="de">Biologischer Anbau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14779-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14181-0">
    <skos:narrower rdf:resource="descriptor/14222-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14181-0">
    <skos:narrower rdf:resource="descriptor/14220-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71114">
    <skos:broader rdf:resource="thsys/71115"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11568-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18173-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12986-3">
    <skos:related rdf:resource="descriptor/14638-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12928-3">
    <skos:related rdf:resource="descriptor/15774-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12928-3">
    <skos:related rdf:resource="descriptor/15771-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12928-3">
    <skos:related rdf:resource="descriptor/15770-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12928-3">
    <skos:related rdf:resource="descriptor/15767-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12928-3">
    <skos:related rdf:resource="descriptor/15765-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12928-3">
    <skos:related rdf:resource="descriptor/18966-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12928-3">
    <skos:related rdf:resource="descriptor/13325-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10562-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11658-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13299-3">
    <skos:narrower rdf:resource="descriptor/13298-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12337-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16946-2">
    <skos:prefLabel xml:lang="en">Carinthia</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16946-2">
    <skos:prefLabel xml:lang="de">Kärnten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19838-4">
    <skos:related rdf:resource="descriptor/12974-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19838-4">
    <skos:related rdf:resource="descriptor/18360-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19838-4">
    <skos:related rdf:resource="descriptor/14174-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12401-1">
    <skos:related rdf:resource="descriptor/12297-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12401-1">
    <skos:related rdf:resource="descriptor/12385-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70384">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19433-2">
    <skos:broader rdf:resource="thsys/71071"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19433-2">
    <skos:broader rdf:resource="descriptor/20522-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19433-2">
    <skos:broader rdf:resource="descriptor/10894-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11495-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15571-3">
    <skos:altLabel xml:lang="en">Stock model</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15571-3">
    <skos:altLabel xml:lang="en">Replenishment systems</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15571-3">
    <skos:altLabel xml:lang="en">Probabilistic inventory model</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15571-3">
    <skos:altLabel xml:lang="en">Newsvendor model</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15571-3">
    <skos:altLabel xml:lang="en">Newsboy model</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15571-3">
    <skos:altLabel xml:lang="en">Multi-stage inventory system</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15571-3">
    <skos:altLabel xml:lang="en">Multi-echelon inventory system</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15571-3">
    <skos:altLabel xml:lang="en">Inventory control model</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15571-3">
    <skos:altLabel xml:lang="de">Stochastisches Lagerhaltungsmodell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15571-3">
    <skos:altLabel xml:lang="de">Mehrstufiges Lagerhaltungsmodell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15571-3">
    <skos:altLabel xml:lang="de">Einstufiges Lagerhaltungsmodell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15571-3">
    <skos:altLabel xml:lang="de">Deterministisches Lagerhaltungsmodell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18318-0">
    <skos:prefLabel xml:lang="en">Agricultural economics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18318-0">
    <skos:prefLabel xml:lang="de">Agrarökonomik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18240-4">
    <skos:broader rdf:resource="thsys/70134"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18240-4">
    <skos:broader rdf:resource="descriptor/14302-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11694-0">
    <skos:narrower rdf:resource="descriptor/11696-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11694-0">
    <skos:narrower rdf:resource="descriptor/11688-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11694-0">
    <skos:narrower rdf:resource="descriptor/11684-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10401-4">
    <skos:prefLabel xml:lang="en">Social pact</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10401-4">
    <skos:prefLabel xml:lang="de">Sozialpakt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12644-3">
    <skos:altLabel xml:lang="de">Materialfluß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15837-5">
    <skos:broader rdf:resource="thsys/73385"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15837-5">
    <skos:broader rdf:resource="thsys/70244"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18794-3">
    <skos:prefLabel xml:lang="en">Foreign fair</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18794-3">
    <skos:prefLabel xml:lang="de">Auslandsmesse</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18457-0">
    <skos:related rdf:resource="descriptor/12721-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18457-0">
    <skos:related rdf:resource="descriptor/18486-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17570-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12438-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18441-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11198-5">
    <skos:prefLabel xml:lang="en">Industrial labour</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11198-5">
    <skos:prefLabel xml:lang="de">Industriearbeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13681-6">
    <skos:broader rdf:resource="thsys/70922"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13681-6">
    <skos:broader rdf:resource="descriptor/13682-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12946-1">
    <skos:prefLabel xml:lang="en">Dairy farming</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12946-1">
    <skos:prefLabel xml:lang="de">Milchviehhaltung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11804-5">
    <skos:narrower rdf:resource="descriptor/11938-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11804-5">
    <skos:narrower rdf:resource="descriptor/18014-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25681-4">
    <skos:prefLabel xml:lang="en">Divorce</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25681-4">
    <skos:prefLabel xml:lang="de">Scheidung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11270-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19691-1">
    <skos:related rdf:resource="descriptor/19697-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19691-1">
    <skos:related rdf:resource="descriptor/16550-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19691-1">
    <skos:related rdf:resource="descriptor/11852-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18900-2">
    <skos:prefLabel xml:lang="en">Smoking</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18900-2">
    <skos:prefLabel xml:lang="de">Rauchen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18203-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17934-4">
    <skos:broader rdf:resource="thsys/71030"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17934-4">
    <skos:broader rdf:resource="descriptor/17931-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17934-4">
    <skos:broader rdf:resource="descriptor/19170-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13738-3">
    <skos:altLabel xml:lang="de">Anleihemarkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15448-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15036-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11372-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24623-2">
    <skos:broader rdf:resource="thsys/71062"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24623-2">
    <skos:broader rdf:resource="descriptor/11322-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19514-2">
    <skos:altLabel xml:lang="de">Wertschöpfungsteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19514-2">
    <skos:altLabel xml:lang="de">Wertschöpfungsbeitrag</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19514-2">
    <skos:altLabel xml:lang="de">Wertschöpfungsabgabe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19514-2">
    <skos:altLabel xml:lang="de">Maschinenbeitrag</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19514-2">
    <skos:altLabel xml:lang="de">Maschinenabgabe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19786-4">
    <skos:prefLabel xml:lang="en">Outpatient treatment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19786-4">
    <skos:prefLabel xml:lang="de">Ambulante Behandlung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16160-5">
    <skos:altLabel xml:lang="de">Reichsversicherungsordnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11236-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15117-4">
    <skos:prefLabel xml:lang="en">Error (Statistics)</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15117-4">
    <skos:prefLabel xml:lang="de">Statistischer Fehler</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10418-1">
    <skos:broader rdf:resource="thsys/70406"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10418-1">
    <skos:broader rdf:resource="descriptor/10434-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10418-1">
    <skos:broader rdf:resource="descriptor/10054-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13706-2">
    <skos:prefLabel xml:lang="en">Private bank</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13706-2">
    <skos:prefLabel xml:lang="de">Privatbank</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10583-0">
    <skos:prefLabel xml:lang="en">Food aid</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10583-0">
    <skos:prefLabel xml:lang="de">Nahrungsmittelhilfe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70465">
    <skos:narrower rdf:resource="descriptor/18201-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70465">
    <skos:narrower rdf:resource="descriptor/14157-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70465">
    <skos:narrower rdf:resource="descriptor/14076-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70465">
    <skos:narrower rdf:resource="descriptor/18189-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70465">
    <skos:narrower rdf:resource="descriptor/14154-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70465">
    <skos:narrower rdf:resource="descriptor/14153-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70465">
    <skos:narrower rdf:resource="descriptor/18204-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70465">
    <skos:narrower rdf:resource="descriptor/14150-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70465">
    <skos:narrower rdf:resource="descriptor/18202-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70465">
    <skos:narrower rdf:resource="descriptor/14149-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13874-2">
    <skos:altLabel xml:lang="de">Wildhegerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13874-2">
    <skos:altLabel xml:lang="de">Wildheger</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13874-2">
    <skos:altLabel xml:lang="de">Waldarbeiterinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13874-2">
    <skos:altLabel xml:lang="de">Waldarbeiter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13874-2">
    <skos:altLabel xml:lang="de">Jägerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13874-2">
    <skos:altLabel xml:lang="de">Jäger</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13874-2">
    <skos:altLabel xml:lang="de">Jagdberufe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13874-2">
    <skos:altLabel xml:lang="de">Holzfäller</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13874-2">
    <skos:altLabel xml:lang="de">Forstwirtschaftliche Arbeitskräfte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13874-2">
    <skos:altLabel xml:lang="de">Forstwirtinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13874-2">
    <skos:altLabel xml:lang="de">Forstwirte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13874-2">
    <skos:altLabel xml:lang="de">Försterinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13874-2">
    <skos:altLabel xml:lang="de">Förster</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13874-2">
    <skos:altLabel xml:lang="de">Forstarbeiter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12882-1">
    <skos:related rdf:resource="descriptor/12903-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12882-1">
    <skos:related rdf:resource="descriptor/12901-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12882-1">
    <skos:related rdf:resource="descriptor/12307-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14525-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70817">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19426-6">
    <skos:altLabel xml:lang="de">Erdölverarbeitung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16629-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14946-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18099-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11571-2">
    <skos:broader rdf:resource="thsys/71113"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11571-2">
    <skos:broader rdf:resource="thsys/71112"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11571-2">
    <skos:broader rdf:resource="descriptor/11778-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19153-1">
    <skos:related rdf:resource="descriptor/12796-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19153-1">
    <skos:related rdf:resource="descriptor/10201-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19153-1">
    <skos:related rdf:resource="descriptor/12535-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19153-1">
    <skos:related rdf:resource="descriptor/12474-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19153-1">
    <skos:related rdf:resource="descriptor/12520-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16759-1">
    <skos:scopeNote xml:lang="de">Verhalten bei der Informationsbeschaffung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18704-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71106">
    <skos:prefLabel xml:lang="en">V.05.06.02.01  Stock market</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71106">
    <skos:prefLabel xml:lang="de">V.05.06.02.01  Aktienmarkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15609-4">
    <skos:related rdf:resource="descriptor/15045-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15609-4">
    <skos:related rdf:resource="descriptor/16716-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15609-4">
    <skos:related rdf:resource="descriptor/15415-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15609-4">
    <skos:related rdf:resource="descriptor/16757-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15609-4">
    <skos:related rdf:resource="descriptor/18764-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15609-4">
    <skos:related rdf:resource="descriptor/18425-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15609-4">
    <skos:related rdf:resource="descriptor/15606-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15609-4">
    <skos:related rdf:resource="descriptor/14751-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15415-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14186-4">
    <skos:broader rdf:resource="thsys/71015"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14186-4">
    <skos:broader rdf:resource="thsys/71014"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14186-4">
    <skos:broader rdf:resource="thsys/70651"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14186-4">
    <skos:broader rdf:resource="thsys/70227"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14186-4">
    <skos:broader rdf:resource="descriptor/18024-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12247-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20826-1">
    <skos:scopeNote xml:lang="de">Versuch der politischen Gestaltung der Globalisierung durch Interessenausgleich zwischen nationalen Regierungen und zivilgesellschaftlichen Akteuren auf internationaler Ebene, z.B. im Rahmen von Weltkonferenzen. Gleichzeitig ein sozialwissenschaftlicher Ansatz zur Analyse des Phänomens.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19222-1">
    <skos:altLabel xml:lang="en">Sticky prices</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19270-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11309-1">
    <skos:related rdf:resource="descriptor/16692-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11309-1">
    <skos:related rdf:resource="descriptor/12603-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11309-1">
    <skos:related rdf:resource="descriptor/11305-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11309-1">
    <skos:related rdf:resource="descriptor/11180-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11667-3">
    <skos:related rdf:resource="descriptor/13561-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11667-3">
    <skos:related rdf:resource="descriptor/11673-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11667-3">
    <skos:related rdf:resource="descriptor/14433-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15912-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16547-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18965-2">
    <skos:broader rdf:resource="thsys/70816"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18965-2">
    <skos:broader rdf:resource="descriptor/13324-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18307-5">
    <skos:altLabel xml:lang="en">Sustainable packaging</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18307-5">
    <skos:altLabel xml:lang="de">Umweltverträgliche Verpackung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18307-5">
    <skos:altLabel xml:lang="de">Umweltfreundliche Verpackung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18307-5">
    <skos:altLabel xml:lang="de">Öko-Verpackung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12917-1">
    <skos:broader rdf:resource="thsys/71005"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12917-1">
    <skos:broader rdf:resource="descriptor/18120-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12917-1">
    <skos:broader rdf:resource="descriptor/11821-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16974-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13130-1">
    <skos:related rdf:resource="descriptor/18244-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13130-1">
    <skos:related rdf:resource="descriptor/14885-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17497-0">
    <skos:altLabel xml:lang="de">Kirgistan</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17497-0">
    <skos:altLabel xml:lang="de">Kirgisische Republik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17497-0">
    <skos:altLabel xml:lang="de">Kirgisien</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18782-3">
    <skos:broader rdf:resource="thsys/70878"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18782-3">
    <skos:broader rdf:resource="thsys/70844"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18782-3">
    <skos:broader rdf:resource="thsys/70176"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18782-3">
    <skos:broader rdf:resource="descriptor/10436-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12278-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14022-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19206-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12760-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15900-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13569-0">
    <skos:prefLabel xml:lang="en">Traffic law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13569-0">
    <skos:prefLabel xml:lang="de">Verkehrsrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10761-2">
    <skos:broader rdf:resource="thsys/71094"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10761-2">
    <skos:broader rdf:resource="descriptor/19335-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16258-2">
    <skos:prefLabel xml:lang="en">Legitimacy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16258-2">
    <skos:prefLabel xml:lang="de">Legitimität</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16283-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70879">
    <skos:prefLabel xml:lang="de">W.20  Forschung und Entwicklung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12825-6">
    <skos:related rdf:resource="descriptor/18939-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12825-6">
    <skos:related rdf:resource="descriptor/13360-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12825-6">
    <skos:related rdf:resource="descriptor/13665-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14897-6">
    <skos:prefLabel xml:lang="en">Carpet</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14897-6">
    <skos:prefLabel xml:lang="de">Teppich</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14955-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19143-4">
    <skos:related rdf:resource="descriptor/12270-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19461-4">
    <skos:prefLabel xml:lang="en">Rural unemployment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19461-4">
    <skos:prefLabel xml:lang="de">Ländliche Arbeitslosigkeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71041">
    <skos:prefLabel xml:lang="de">B.03.04  Revision und Controlling</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18224-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14930-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12758-2">
    <skos:prefLabel xml:lang="en">Production planning</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12758-2">
    <skos:prefLabel xml:lang="de">Produktionsplanung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70037">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19370-0">
    <skos:related rdf:resource="descriptor/13421-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19370-0">
    <skos:related rdf:resource="descriptor/11602-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12700-5">
    <skos:broader rdf:resource="thsys/70030"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12700-5">
    <skos:broader rdf:resource="thsys/70020"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13667-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20401-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18227-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70958">
    <rdfs:label xml:lang="de">Dienstleistungen</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11473-2">
    <skos:broader rdf:resource="thsys/70306"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11473-2">
    <skos:broader rdf:resource="descriptor/10236-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18371-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15949-1">
    <skos:related rdf:resource="descriptor/15679-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15949-1">
    <skos:related rdf:resource="descriptor/15947-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15949-1">
    <skos:related rdf:resource="descriptor/13294-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15949-1">
    <skos:related rdf:resource="descriptor/19760-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15949-1">
    <skos:related rdf:resource="descriptor/16670-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15949-1">
    <skos:related rdf:resource="descriptor/16683-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15949-1">
    <skos:related rdf:resource="descriptor/19645-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15949-1">
    <skos:related rdf:resource="descriptor/15708-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15949-1">
    <skos:related rdf:resource="descriptor/13315-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10602-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10249-5">
    <skos:prefLabel xml:lang="en">Macroeconomics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10249-5">
    <skos:prefLabel xml:lang="de">Makroökonomik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18662-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19638-5">
    <skos:related rdf:resource="descriptor/19637-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19638-5">
    <skos:related rdf:resource="descriptor/15943-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19638-5">
    <skos:related rdf:resource="descriptor/15941-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19638-5">
    <skos:related rdf:resource="descriptor/15904-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20522-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13682-4">
    <zbwext:useInsteadNote rdf:nodeID="id226"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13682-4">
    <zbwext:useInsteadNote rdf:nodeID="id225"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71051">
    <skos:narrower rdf:resource="descriptor/13992-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71051">
    <skos:narrower rdf:resource="descriptor/16150-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71051">
    <skos:narrower rdf:resource="descriptor/16484-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71051">
    <skos:narrower rdf:resource="descriptor/11349-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71051">
    <skos:narrower rdf:resource="descriptor/11348-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71051">
    <skos:narrower rdf:resource="descriptor/18159-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71051">
    <skos:narrower rdf:resource="descriptor/10401-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71051">
    <skos:narrower rdf:resource="descriptor/11347-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71051">
    <skos:narrower rdf:resource="descriptor/11310-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71051">
    <skos:narrower rdf:resource="descriptor/20449-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71051">
    <skos:narrower rdf:resource="descriptor/20455-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71051">
    <skos:narrower rdf:resource="descriptor/19956-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71051">
    <skos:narrower rdf:resource="descriptor/19110-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71051">
    <skos:narrower rdf:resource="descriptor/16136-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71051">
    <skos:narrower rdf:resource="descriptor/18169-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71051">
    <skos:narrower rdf:resource="descriptor/11311-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71051">
    <skos:narrower rdf:resource="descriptor/11340-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71051">
    <skos:narrower rdf:resource="descriptor/16131-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71051">
    <skos:narrower rdf:resource="descriptor/11185-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71051">
    <skos:narrower rdf:resource="descriptor/11339-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71051">
    <skos:narrower rdf:resource="descriptor/11306-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71051">
    <skos:narrower rdf:resource="descriptor/11314-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16532-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13329-6">
    <skos:altLabel xml:lang="de">Abwasserentsorgung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13329-6">
    <skos:altLabel xml:lang="de">Abwasserbeseitigung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73300">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10975-4">
    <skos:related rdf:resource="descriptor/10113-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10975-4">
    <skos:related rdf:resource="descriptor/16353-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10975-4">
    <skos:related rdf:resource="descriptor/18817-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10975-4">
    <skos:related rdf:resource="descriptor/10109-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10975-4">
    <skos:related rdf:resource="descriptor/15860-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10975-4">
    <skos:related rdf:resource="descriptor/16277-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10975-4">
    <skos:related rdf:resource="descriptor/16209-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10975-4">
    <skos:related rdf:resource="descriptor/12047-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10975-4">
    <skos:related rdf:resource="descriptor/10252-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10975-4">
    <skos:related rdf:resource="descriptor/11526-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15069-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14331-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18334-2">
    <skos:broader rdf:resource="thsys/73370"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18334-2">
    <skos:broader rdf:resource="thsys/70839"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18334-2">
    <skos:broader rdf:resource="thsys/70573"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18334-2">
    <skos:broader rdf:resource="descriptor/16479-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13574-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18674-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12668-3">
    <skos:narrower rdf:resource="descriptor/13456-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16265-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14220-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18226-5">
    <skos:broader rdf:resource="thsys/70130"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18226-5">
    <skos:broader rdf:resource="descriptor/17179-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18226-5">
    <skos:broader rdf:resource="descriptor/17983-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17790-2">
    <skos:broader rdf:resource="thsys/70086"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17790-2">
    <skos:broader rdf:resource="descriptor/19130-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17790-2">
    <skos:broader rdf:resource="descriptor/17791-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13682-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10829-1">
    <skos:scopeNote xml:lang="de">Internationale Rechtssätze aus dem allgemeinen Völkerrecht und den Doppelbesteuerungsabkommen, die Doppel- und Nichtbesteuerung vermeiden sollen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19360-3">
    <skos:broader rdf:resource="thsys/71089"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19360-3">
    <skos:broader rdf:resource="thsys/70052"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19360-3">
    <skos:broader rdf:resource="descriptor/19221-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19360-3">
    <skos:broader rdf:resource="descriptor/10785-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19843-4">
    <skos:related rdf:resource="descriptor/16303-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19843-4">
    <skos:related rdf:resource="descriptor/16486-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19843-4">
    <skos:related rdf:resource="descriptor/19639-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19843-4">
    <skos:related rdf:resource="descriptor/19640-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19843-4">
    <skos:related rdf:resource="descriptor/15919-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19843-4">
    <skos:related rdf:resource="descriptor/19649-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19843-4">
    <skos:related rdf:resource="descriptor/15989-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12921-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19259-6">
    <skos:broader rdf:resource="thsys/70306"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19259-6">
    <skos:broader rdf:resource="descriptor/10236-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14710-6">
    <skos:narrower rdf:resource="descriptor/18257-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14710-6">
    <skos:narrower rdf:resource="descriptor/14715-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14710-6">
    <skos:narrower rdf:resource="descriptor/14713-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14710-6">
    <skos:narrower rdf:resource="descriptor/14712-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14710-6">
    <skos:narrower rdf:resource="descriptor/14711-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14710-6">
    <skos:narrower rdf:resource="descriptor/14705-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14710-6">
    <skos:narrower rdf:resource="descriptor/14698-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10562-1">
    <skos:related rdf:resource="descriptor/11903-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10562-1">
    <skos:related rdf:resource="descriptor/10542-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10562-1">
    <skos:related rdf:resource="descriptor/11892-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10562-1">
    <skos:related rdf:resource="descriptor/11897-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10562-1">
    <skos:related rdf:resource="descriptor/11979-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10562-1">
    <skos:related rdf:resource="descriptor/11919-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13084-3">
    <skos:broader rdf:resource="thsys/70996"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13084-3">
    <skos:broader rdf:resource="descriptor/13076-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10020-2">
    <zbwext:useInsteadNote rdf:nodeID="id299"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16080-3">
    <skos:narrower rdf:resource="descriptor/16079-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16080-3">
    <skos:narrower rdf:resource="descriptor/16068-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19143-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14851-2">
    <skos:altLabel xml:lang="de">Tierische Faser</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14851-2">
    <skos:altLabel xml:lang="de">Sisal</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14851-2">
    <skos:altLabel xml:lang="de">Ramie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14851-2">
    <skos:altLabel xml:lang="de">Pflanzenfaser</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14851-2">
    <skos:altLabel xml:lang="de">Mohairwolle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14851-2">
    <skos:altLabel xml:lang="de">Kaschmirwolle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14851-2">
    <skos:altLabel xml:lang="de">Kamelhaar</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14851-2">
    <skos:altLabel xml:lang="de">Angorawolle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14851-2">
    <skos:altLabel xml:lang="de">Alpakawolle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13474-4">
    <skos:altLabel xml:lang="de">Sortimentstiefe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13474-4">
    <skos:altLabel xml:lang="de">Sortimentspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13474-4">
    <skos:altLabel xml:lang="de">Sortimentsplanung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13474-4">
    <skos:altLabel xml:lang="de">Sortimentsgestaltung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13474-4">
    <skos:altLabel xml:lang="de">Sortimentsbreite</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13474-4">
    <skos:altLabel xml:lang="de">Handelssortiment</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14981-3">
    <skos:prefLabel xml:lang="en">Sparkling wine</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14981-3">
    <skos:prefLabel xml:lang="de">Schaumwein</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17199-1">
    <skos:narrower rdf:resource="descriptor/17433-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17199-1">
    <skos:narrower rdf:resource="descriptor/17198-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17199-1">
    <skos:narrower rdf:resource="descriptor/17413-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17199-1">
    <skos:narrower rdf:resource="descriptor/17374-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17199-1">
    <skos:narrower rdf:resource="descriptor/17255-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17199-1">
    <skos:narrower rdf:resource="descriptor/17201-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16007-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11170-6">
    <skos:prefLabel xml:lang="en">Policy analysis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11170-6">
    <skos:prefLabel xml:lang="de">Policy-Ansatz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13296-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17471-4">
    <skos:prefLabel xml:lang="en">Estonia</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17471-4">
    <skos:prefLabel xml:lang="de">Estland</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70934">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.12.01.03.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11633-6">
    <skos:broader rdf:resource="thsys/71072"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11633-6">
    <skos:broader rdf:resource="thsys/70860"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11633-6">
    <skos:broader rdf:resource="descriptor/10205-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18939-3">
    <skos:broader rdf:resource="thsys/70824"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18939-3">
    <skos:broader rdf:resource="descriptor/25742-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18253-2">
    <skos:broader rdf:resource="thsys/70463"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18253-2">
    <skos:broader rdf:resource="descriptor/14642-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10649-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10103-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70899">
    <skos:narrower rdf:resource="descriptor/13782-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70899">
    <skos:narrower rdf:resource="descriptor/18727-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70899">
    <skos:narrower rdf:resource="descriptor/13769-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70899">
    <skos:narrower rdf:resource="descriptor/12265-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70899">
    <skos:narrower rdf:resource="descriptor/12299-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70899">
    <skos:narrower rdf:resource="descriptor/13790-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70899">
    <skos:narrower rdf:resource="descriptor/13761-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70899">
    <skos:narrower rdf:resource="descriptor/13763-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70899">
    <skos:narrower rdf:resource="descriptor/19556-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70899">
    <skos:narrower rdf:resource="descriptor/13762-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70899">
    <skos:narrower rdf:resource="descriptor/19282-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70899">
    <skos:narrower rdf:resource="descriptor/12253-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70899">
    <skos:narrower rdf:resource="descriptor/18725-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70899">
    <skos:narrower rdf:resource="descriptor/18178-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70899">
    <skos:narrower rdf:resource="descriptor/12252-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70899">
    <skos:narrower rdf:resource="descriptor/13777-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70899">
    <skos:narrower rdf:resource="descriptor/13776-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70899">
    <skos:narrower rdf:resource="descriptor/12251-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70899">
    <skos:narrower rdf:resource="descriptor/13775-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70899">
    <skos:narrower rdf:resource="descriptor/19137-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70899">
    <skos:narrower rdf:resource="descriptor/18667-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70899">
    <skos:narrower rdf:resource="descriptor/13785-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70899">
    <skos:narrower rdf:resource="descriptor/13784-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70899">
    <skos:narrower rdf:resource="descriptor/13795-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70899">
    <skos:narrower rdf:resource="descriptor/13794-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70899">
    <skos:narrower rdf:resource="descriptor/18084-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70899">
    <skos:narrower rdf:resource="descriptor/11563-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70899">
    <skos:narrower rdf:resource="descriptor/20733-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70899">
    <skos:narrower rdf:resource="descriptor/13756-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70899">
    <skos:narrower rdf:resource="descriptor/12236-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70899">
    <skos:narrower rdf:resource="descriptor/18141-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70899">
    <skos:narrower rdf:resource="descriptor/19535-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18426-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18249-0">
    <skos:related rdf:resource="descriptor/13653-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18249-0">
    <skos:related rdf:resource="descriptor/14534-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19238-0">
    <skos:broader rdf:resource="thsys/70020"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18685-1">
    <skos:related rdf:resource="descriptor/12241-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19128-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11664-2">
    <skos:prefLabel xml:lang="en">Financial transaction tax</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11664-2">
    <skos:prefLabel xml:lang="de">Kapitalverkehrsteuer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19430-1">
    <skos:altLabel xml:lang="de">Chicago Wettbewerbstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19430-1">
    <skos:altLabel xml:lang="de">Chicago Schule (Wettbewerb)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19625-0">
    <skos:altLabel xml:lang="en">Genetic resources</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19625-0">
    <skos:altLabel xml:lang="en">Genetic diversity</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19625-0">
    <skos:altLabel xml:lang="en">Endangered species</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19625-0">
    <skos:altLabel xml:lang="de">Genpool</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19625-0">
    <skos:altLabel xml:lang="de">Genetische Ressourcen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19625-0">
    <skos:altLabel xml:lang="de">Bedrohte Arten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19625-0">
    <skos:altLabel xml:lang="de">Artenvielfalt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19625-0">
    <skos:altLabel xml:lang="de">Artenschutz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19625-0">
    <skos:altLabel xml:lang="de">Artenreichtum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70859">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18166-4">
    <skos:broader rdf:resource="thsys/71059"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18166-4">
    <skos:broader rdf:resource="thsys/71050"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18166-4">
    <skos:broader rdf:resource="descriptor/11248-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18166-4">
    <skos:broader rdf:resource="descriptor/11187-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11269-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16504-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id278">
    <rdf:value xml:lang="de">für Rechtsschutzversicherungen</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19762-4">
    <skos:related rdf:resource="descriptor/18970-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19762-4">
    <skos:related rdf:resource="descriptor/18968-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18156-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13252-1">
    <skos:broader rdf:resource="thsys/70936"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13252-1">
    <skos:broader rdf:resource="descriptor/13264-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15995-1">
    <skos:altLabel xml:lang="de">Stadtverfassung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15995-1">
    <skos:altLabel xml:lang="de">Städteordnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15995-1">
    <skos:altLabel xml:lang="de">Magistratsverfassung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15995-1">
    <skos:altLabel xml:lang="de">Landkreisordnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15995-1">
    <skos:altLabel xml:lang="de">Landgemeindeordnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15995-1">
    <skos:altLabel xml:lang="de">Kommunalverfassung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15995-1">
    <skos:altLabel xml:lang="de">Gemeindeverfassung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15995-1">
    <skos:altLabel xml:lang="de">Gemeinderecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15995-1">
    <skos:altLabel xml:lang="de">Gemeindeordnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13242-4">
    <skos:prefLabel xml:lang="en">Inland water transport</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13242-4">
    <skos:prefLabel xml:lang="de">Binnenschifffahrt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18744-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18176-1">
    <skos:altLabel xml:lang="en">IPR (Intellectual property rights)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18176-1">
    <skos:altLabel xml:lang="de">Schutz des geistigen Eigentums</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18176-1">
    <skos:altLabel xml:lang="de">Rechtsschutz des geistigen Eigentums</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18176-1">
    <skos:altLabel xml:lang="de">Immaterialgüterrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18176-1">
    <skos:altLabel xml:lang="de">Geistige Eigentumsrechte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18118-1">
    <skos:broader rdf:resource="thsys/70485"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18118-1">
    <skos:broader rdf:resource="descriptor/12085-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10210-4">
    <skos:narrower rdf:resource="descriptor/19280-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70101">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13331-5">
    <skos:prefLabel xml:lang="en">Sewage plant</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13331-5">
    <skos:prefLabel xml:lang="de">Kläranlage</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11186-5">
    <skos:prefLabel xml:lang="en">Division of labour</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11186-5">
    <skos:prefLabel xml:lang="de">Arbeitsteilung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10872-0">
    <skos:prefLabel xml:lang="en">Import trade</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10872-0">
    <skos:prefLabel xml:lang="de">Importhandel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10692-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10261-1">
    <skos:prefLabel xml:lang="en">Short-time work</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10261-1">
    <skos:prefLabel xml:lang="de">Kurzarbeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19416-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10010-5">
    <skos:broader rdf:resource="thsys/71112"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71044">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">B.02.02.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11087-0">
    <skos:related rdf:resource="descriptor/16286-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11087-0">
    <skos:related rdf:resource="descriptor/11070-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11087-0">
    <skos:related rdf:resource="descriptor/16387-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71019">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17726-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14679-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13047-2">
    <skos:narrower rdf:resource="descriptor/13065-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13047-2">
    <skos:narrower rdf:resource="descriptor/13066-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13047-2">
    <skos:narrower rdf:resource="descriptor/13063-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13047-2">
    <skos:narrower rdf:resource="descriptor/13056-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11905-6">
    <skos:narrower rdf:resource="descriptor/18546-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11905-6">
    <skos:narrower rdf:resource="descriptor/18001-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10052-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10145-3">
    <skos:prefLabel xml:lang="en">Microfoundations</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10145-3">
    <skos:prefLabel xml:lang="de">Mikroökonomische Fundierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19836-1">
    <zbwext:useInsteadNote rdf:nodeID="id337"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19836-1">
    <zbwext:useInsteadNote rdf:nodeID="id336"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12737-3">
    <skos:prefLabel xml:lang="en">Robot</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12737-3">
    <skos:prefLabel xml:lang="de">Roboter</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18771-1">
    <skos:prefLabel xml:lang="en">Online services</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18771-1">
    <skos:prefLabel xml:lang="de">Online-Dienst</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18695-5">
    <zbwext:useInsteadNote rdf:nodeID="id15"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11032-4">
    <skos:related rdf:resource="descriptor/10269-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11032-4">
    <skos:related rdf:resource="descriptor/19236-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11032-4">
    <skos:related rdf:resource="descriptor/13398-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11032-4">
    <skos:related rdf:resource="descriptor/15069-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11032-4">
    <skos:related rdf:resource="descriptor/15088-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11032-4">
    <skos:related rdf:resource="descriptor/15940-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19258-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13708-5">
    <skos:scopeNote xml:lang="de">Banken mit regional begrenztem Geschäftsgebiet.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70573">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19413-1">
    <skos:related rdf:resource="descriptor/16092-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19413-1">
    <skos:related rdf:resource="descriptor/11650-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15872-3">
    <skos:prefLabel xml:lang="en">Geography</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15872-3">
    <skos:prefLabel xml:lang="de">Geographie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16269-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18364-0">
    <skos:prefLabel xml:lang="en">Sustainable energy policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18364-0">
    <skos:prefLabel xml:lang="de">Umweltverträgliche Energiepolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16059-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12667-5">
    <skos:altLabel xml:lang="de">Selbermachen oder kaufen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12667-5">
    <skos:altLabel xml:lang="de">Fremdproduktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12667-5">
    <skos:altLabel xml:lang="de">Fremdleistung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12667-5">
    <skos:altLabel xml:lang="de">Fremdfertigung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12667-5">
    <skos:altLabel xml:lang="de">Fremdbezug</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12667-5">
    <skos:altLabel xml:lang="de">Fremd- oder Eigenleistung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12667-5">
    <skos:altLabel xml:lang="de">Eigenproduktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12667-5">
    <skos:altLabel xml:lang="de">Eigenleistung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12667-5">
    <skos:altLabel xml:lang="de">Eigenfertigung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12667-5">
    <skos:altLabel xml:lang="de">Eigenerstellung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12155-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10446-3">
    <skos:broader rdf:resource="thsys/71109"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10446-3">
    <skos:broader rdf:resource="descriptor/10466-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10774-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12969-3">
    <skos:scopeNote xml:lang="de">Für die Zucht sämtlicher Meeresprodukte.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10413-4">
    <skos:broader rdf:resource="thsys/70406"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10413-4">
    <skos:broader rdf:resource="descriptor/10414-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17476-1">
    <skos:prefLabel xml:lang="en">Latvia</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17476-1">
    <skos:prefLabel xml:lang="de">Lettland</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20756-3">
    <skos:altLabel xml:lang="en">Discrete choice model</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20756-3">
    <skos:altLabel xml:lang="de">Diskrete Entscheidungen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20756-3">
    <skos:altLabel xml:lang="de">Discrete-Choice-Analyse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14927-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19792-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17255-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18750-2">
    <skos:prefLabel xml:lang="en">Real estate financing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18750-2">
    <skos:prefLabel xml:lang="de">Immobilienfinanzierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16654-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17676-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15790-5">
    <skos:narrower rdf:resource="descriptor/15836-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15790-5">
    <skos:narrower rdf:resource="descriptor/15811-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15790-5">
    <skos:narrower rdf:resource="descriptor/15787-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15790-5">
    <skos:narrower rdf:resource="descriptor/15786-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18478-6">
    <skos:broader rdf:resource="thsys/71110"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18478-6">
    <skos:broader rdf:resource="thsys/71005"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18478-6">
    <skos:broader rdf:resource="descriptor/11790-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18183-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10779-4">
    <skos:related rdf:resource="descriptor/10773-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10779-4">
    <skos:related rdf:resource="descriptor/19337-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10779-4">
    <skos:related rdf:resource="descriptor/10781-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10779-4">
    <skos:related rdf:resource="descriptor/10780-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10779-4">
    <skos:related rdf:resource="descriptor/13757-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10779-4">
    <skos:related rdf:resource="descriptor/10776-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11543-0">
    <skos:narrower rdf:resource="descriptor/18583-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11543-0">
    <skos:narrower rdf:resource="descriptor/18646-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19829-5">
    <skos:scopeNote xml:lang="de">Für die Förderung spezieller Sektoren oder Regionen BENUTZE die entsprechenden Deskriptoren</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19829-5">
    <skos:scopeNote xml:lang="de">Allgemeine Literatur über wirtschaftsfördernde Maßnahmen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10930-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19435-5">
    <skos:altLabel xml:lang="de">Quote (Regulierungsquote)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19435-5">
    <skos:altLabel xml:lang="de">Mengenregulierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70799">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70235">
    <skos:narrower rdf:resource="descriptor/11067-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70235">
    <skos:narrower rdf:resource="descriptor/19491-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70235">
    <skos:narrower rdf:resource="descriptor/11349-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70235">
    <skos:narrower rdf:resource="descriptor/11348-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70235">
    <skos:narrower rdf:resource="descriptor/18159-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70235">
    <skos:narrower rdf:resource="descriptor/11350-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70235">
    <skos:narrower rdf:resource="descriptor/11347-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70235">
    <skos:narrower rdf:resource="descriptor/18125-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70235">
    <skos:narrower rdf:resource="descriptor/11310-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70235">
    <skos:narrower rdf:resource="descriptor/20449-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70235">
    <skos:narrower rdf:resource="descriptor/19503-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70235">
    <skos:narrower rdf:resource="descriptor/18123-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70235">
    <skos:narrower rdf:resource="descriptor/19502-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70235">
    <skos:narrower rdf:resource="descriptor/11313-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70235">
    <skos:narrower rdf:resource="descriptor/11814-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70235">
    <skos:narrower rdf:resource="descriptor/18142-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70235">
    <skos:narrower rdf:resource="descriptor/20455-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70235">
    <skos:narrower rdf:resource="descriptor/19956-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70235">
    <skos:narrower rdf:resource="descriptor/19005-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70235">
    <skos:narrower rdf:resource="descriptor/16136-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70235">
    <skos:narrower rdf:resource="descriptor/18169-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70235">
    <skos:narrower rdf:resource="descriptor/11311-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70235">
    <skos:narrower rdf:resource="descriptor/18792-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70235">
    <skos:narrower rdf:resource="descriptor/11340-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70235">
    <skos:narrower rdf:resource="descriptor/16131-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70235">
    <skos:narrower rdf:resource="descriptor/19902-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70235">
    <skos:narrower rdf:resource="descriptor/12609-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70235">
    <skos:narrower rdf:resource="descriptor/11185-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70235">
    <skos:narrower rdf:resource="descriptor/11339-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70235">
    <skos:narrower rdf:resource="descriptor/11307-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70235">
    <skos:narrower rdf:resource="descriptor/11306-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70235">
    <skos:narrower rdf:resource="descriptor/11314-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70235">
    <skos:narrower rdf:resource="descriptor/11305-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70235">
    <skos:narrower rdf:resource="descriptor/11769-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70235">
    <skos:narrower rdf:resource="descriptor/11304-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70235">
    <skos:narrower rdf:resource="descriptor/11036-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20321-3">
    <skos:broader rdf:resource="thsys/71067"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20321-3">
    <skos:broader rdf:resource="descriptor/11928-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19359-2">
    <skos:scopeNote xml:lang="de">Internationale Vereinbarungen zur Regelung von Verzinsung, Tilgung, Rückzahlung usw. von ausländischen Krediten an private und öffentliche Kreditgeber.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12291-3">
    <skos:broader rdf:resource="thsys/71043"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12291-3">
    <skos:broader rdf:resource="thsys/70036"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12291-3">
    <skos:broader rdf:resource="descriptor/12297-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12291-3">
    <skos:broader rdf:resource="descriptor/12591-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16203-6">
    <skos:broader rdf:resource="thsys/73392"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16203-6">
    <skos:broader rdf:resource="thsys/73334"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70572">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11907-2">
    <skos:prefLabel xml:lang="en">Spatial effect</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11907-2">
    <skos:prefLabel xml:lang="de">Regionale Inzidenz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10438-2">
    <skos:altLabel xml:lang="en">Investment subsidy</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10438-2">
    <skos:altLabel xml:lang="de">Investitionszulage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10438-2">
    <skos:altLabel xml:lang="de">Investitionslenkung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10438-2">
    <skos:altLabel xml:lang="de">Investitionshilfe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10438-2">
    <skos:altLabel xml:lang="de">Investitionsförderung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10438-2">
    <skos:altLabel xml:lang="de">Investitionsbeihilfe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10438-2">
    <skos:altLabel xml:lang="de">Investitionsanreiz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16058-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10725-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18176-1">
    <skos:scopeNote xml:lang="de">Rechte an unkörperlichen Gütern, an geistigem Eigentum. Die wichtigsten Verwertungsrechte sind Patent- und Urheberrecht.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13345-1">
    <skos:narrower rdf:resource="descriptor/13346-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13345-1">
    <skos:narrower rdf:resource="descriptor/15473-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13345-1">
    <skos:narrower rdf:resource="descriptor/18941-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16012-6">
    <zbwext:useInsteadNote rdf:nodeID="id287"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19216-3">
    <skos:prefLabel xml:lang="en">Bounded rationality</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19216-3">
    <skos:prefLabel xml:lang="de">Beschränkte Rationalität</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18353-5">
    <skos:broader rdf:resource="thsys/71016"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18353-5">
    <skos:broader rdf:resource="descriptor/10470-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14313-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19326-3">
    <skos:narrower rdf:resource="descriptor/20110-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19326-3">
    <skos:narrower rdf:resource="descriptor/19325-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19326-3">
    <skos:narrower rdf:resource="descriptor/19958-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70346">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">P.07.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15519-5">
    <skos:related rdf:resource="descriptor/13588-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15519-5">
    <skos:related rdf:resource="descriptor/18620-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15519-5">
    <skos:related rdf:resource="descriptor/12689-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70278">
    <skos:prefLabel xml:lang="en">V.13.02  Wages</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70278">
    <skos:prefLabel xml:lang="de">V.13.02  Lohn</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12647-4">
    <skos:broader rdf:resource="thsys/73343"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10587-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10691-4">
    <skos:prefLabel xml:lang="en">Embargo</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10691-4">
    <skos:prefLabel xml:lang="de">Embargo</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14073-3">
    <skos:narrower rdf:resource="descriptor/14084-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14073-3">
    <skos:narrower rdf:resource="descriptor/14118-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14073-3">
    <skos:narrower rdf:resource="descriptor/14082-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14073-3">
    <skos:narrower rdf:resource="descriptor/14076-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14073-3">
    <skos:narrower rdf:resource="descriptor/14070-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14073-3">
    <skos:narrower rdf:resource="descriptor/14068-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14073-3">
    <skos:narrower rdf:resource="descriptor/18199-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19739-6">
    <skos:related rdf:resource="descriptor/10574-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12004-4">
    <skos:altLabel xml:lang="de">Infrastrukturplanung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12004-4">
    <skos:altLabel xml:lang="de">Infrastrukturmaßnahme</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12004-4">
    <skos:altLabel xml:lang="de">Infrastrukturausbau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16621-2">
    <skos:narrower rdf:resource="descriptor/16395-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71045">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">B.02.01.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11417-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12511-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19494-3">
    <skos:altLabel xml:lang="de">Beveridge Kurve</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71050">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">B.04.04</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17567-5">
    <skos:altLabel xml:lang="en">ROC (Republic of China)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17567-5">
    <skos:altLabel xml:lang="en">Republic of China</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17567-5">
    <skos:altLabel xml:lang="de">Nationalchina</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17567-5">
    <skos:altLabel xml:lang="de">Formosa</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17567-5">
    <skos:altLabel xml:lang="de">China (Republik)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18412-1">
    <skos:altLabel xml:lang="de">Maschinenbau (Technologie)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21041-4">
    <skos:broader rdf:resource="thsys/70041"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21041-4">
    <skos:broader rdf:resource="descriptor/18100-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18531-0">
    <skos:prefLabel xml:lang="en">Trade management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18531-0">
    <skos:prefLabel xml:lang="de">Handelsmanagement</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14019-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15491-1">
    <skos:broader rdf:resource="thsys/71035"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15491-1">
    <skos:broader rdf:resource="descriptor/15055-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12812-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11496-4">
    <skos:related rdf:resource="descriptor/11506-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11833-5">
    <skos:broader rdf:resource="thsys/73368"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11833-5">
    <skos:broader rdf:resource="thsys/71110"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11833-5">
    <skos:broader rdf:resource="thsys/70926"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11833-5">
    <skos:broader rdf:resource="descriptor/10981-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19802-4">
    <skos:altLabel xml:lang="de">Systemmigration</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19802-4">
    <skos:altLabel xml:lang="de">Programm-Migration</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14339-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10550-1">
    <skos:prefLabel xml:lang="en">Development bank</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10550-1">
    <skos:prefLabel xml:lang="de">Entwicklungsbank</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70056">
    <rdfs:label xml:lang="de">Soziales, Gesundheit und Bildung</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19479-6">
    <skos:related rdf:resource="descriptor/15897-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19479-6">
    <skos:related rdf:resource="descriptor/13362-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19479-6">
    <skos:related rdf:resource="descriptor/11850-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15858-4">
    <skos:related rdf:resource="descriptor/12846-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15858-4">
    <skos:related rdf:resource="descriptor/12837-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15858-4">
    <skos:related rdf:resource="descriptor/15793-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15858-4">
    <skos:related rdf:resource="descriptor/16265-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15858-4">
    <skos:related rdf:resource="descriptor/10300-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15858-4">
    <skos:related rdf:resource="descriptor/19599-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12669-1">
    <skos:broader rdf:resource="thsys/73384"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12669-1">
    <skos:broader rdf:resource="thsys/70368"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12669-1">
    <skos:broader rdf:resource="descriptor/12665-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15787-1">
    <skos:broader rdf:resource="thsys/70017"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15787-1">
    <skos:broader rdf:resource="descriptor/15790-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19230-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16531-3">
    <skos:prefLabel xml:lang="en">Work disability</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16531-3">
    <skos:prefLabel xml:lang="de">Erwerbsunfähigkeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15340-1">
    <skos:prefLabel xml:lang="en">Regression</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15340-1">
    <skos:prefLabel xml:lang="de">Regression</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16441-4">
    <skos:broader rdf:resource="thsys/73366"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10470-6">
    <skos:editorialNote xml:lang="de">Möglichst spezifischer indexieren.</skos:editorialNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15703-5">
    <skos:prefLabel xml:lang="en">Industrial history</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15703-5">
    <skos:prefLabel xml:lang="de">Industriegeschichte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13113-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19310-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id67">
    <rdf:value xml:lang="de">Für statistische Analysen der Verweildauer</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18609-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19364-2">
    <skos:prefLabel xml:lang="en">Core-periphery relations</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19364-2">
    <skos:prefLabel xml:lang="de">Kern-Peripherie-Beziehung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19613-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14846-2">
    <skos:broader rdf:resource="thsys/70066"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18342-3">
    <skos:altLabel xml:lang="de">Fischfangflotte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13753-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12212-4">
    <skos:altLabel xml:lang="en">Portfolio selection</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12212-4">
    <skos:altLabel xml:lang="en">Portfolio planning</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12212-4">
    <skos:altLabel xml:lang="en">Portfolio performance</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12212-4">
    <skos:altLabel xml:lang="de">Wertpapierstreuung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12212-4">
    <skos:altLabel xml:lang="de">Theorie der optimalen Wertpapiermischung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12212-4">
    <skos:altLabel xml:lang="de">Separationstheorem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12212-4">
    <skos:altLabel xml:lang="de">Risikostreuung von Wertpapieren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12212-4">
    <skos:altLabel xml:lang="de">Risikodiversifizierung von Kapitalanlagen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12212-4">
    <skos:altLabel xml:lang="de">Portfolio-Theorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12212-4">
    <skos:altLabel xml:lang="de">Portfoliomanagement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12212-4">
    <skos:altLabel xml:lang="de">Portfolio-Analyse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12212-4">
    <skos:altLabel xml:lang="de">Portefeuille-Theorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12212-4">
    <skos:altLabel xml:lang="de">Portefeuilleplanung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12212-4">
    <skos:altLabel xml:lang="de">Portefeuille-Management</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12212-4">
    <skos:altLabel xml:lang="de">Markowitz-Theorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12212-4">
    <skos:altLabel xml:lang="de">Kapitaltheorie des Betriebs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12212-4">
    <skos:altLabel xml:lang="de">Kapitaltheorie des Betriebes</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12212-4">
    <skos:altLabel xml:lang="de">Kapitalanlagetheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11809-2">
    <skos:broader rdf:resource="thsys/71020"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11809-2">
    <skos:broader rdf:resource="descriptor/10994-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11809-2">
    <skos:broader rdf:resource="descriptor/11801-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18635-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19449-1">
    <skos:narrower rdf:resource="descriptor/19473-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15243-6">
    <skos:broader rdf:resource="thsys/72186"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15243-6">
    <skos:broader rdf:resource="descriptor/15111-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15243-6">
    <skos:broader rdf:resource="descriptor/15258-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11304-4">
    <skos:broader rdf:resource="thsys/73378"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11304-4">
    <skos:broader rdf:resource="thsys/70852"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11304-4">
    <skos:broader rdf:resource="thsys/70235"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11304-4">
    <skos:broader rdf:resource="descriptor/10966-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11304-4">
    <skos:broader rdf:resource="descriptor/11350-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11814-2">
    <skos:related rdf:resource="descriptor/12613-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11814-2">
    <skos:related rdf:resource="descriptor/16052-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11814-2">
    <skos:related rdf:resource="descriptor/11817-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11814-2">
    <skos:related rdf:resource="descriptor/12609-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18125-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18751-0">
    <skos:altLabel xml:lang="de">Werksiedlung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18751-0">
    <skos:altLabel xml:lang="de">Fabriksiedlung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id213">
    <rdf:value xml:lang="de">Für Mensch-Maschine-Schnittstellen</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13441-5">
    <skos:prefLabel xml:lang="en">Agricultural insurance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13441-5">
    <skos:prefLabel xml:lang="de">Landwirtschaftliche Versicherung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10455-2">
    <skos:related rdf:resource="descriptor/19797-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10455-2">
    <skos:related rdf:resource="descriptor/19798-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10455-2">
    <skos:related rdf:resource="descriptor/13074-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10455-2">
    <skos:related rdf:resource="descriptor/14480-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10455-2">
    <skos:related rdf:resource="descriptor/18425-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10455-2">
    <skos:related rdf:resource="descriptor/15606-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10455-2">
    <skos:related rdf:resource="descriptor/19660-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10054-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19667-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12205-1">
    <skos:prefLabel xml:lang="en">Corporate Finance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12205-1">
    <skos:prefLabel xml:lang="de">Unternehmensfinanzierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70539">
    <rdfs:label xml:lang="en">Energy policy</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70539">
    <rdfs:label xml:lang="de">Energiepolitik</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13278-4">
    <skos:related rdf:resource="descriptor/18654-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13278-4">
    <skos:related rdf:resource="descriptor/18643-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13278-4">
    <skos:related rdf:resource="descriptor/18653-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13278-4">
    <skos:related rdf:resource="descriptor/18663-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13278-4">
    <skos:related rdf:resource="descriptor/18649-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13278-4">
    <skos:related rdf:resource="descriptor/18428-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13278-4">
    <skos:related rdf:resource="descriptor/18651-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13278-4">
    <skos:related rdf:resource="descriptor/18112-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13278-4">
    <skos:related rdf:resource="descriptor/13274-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13278-4">
    <skos:related rdf:resource="descriptor/14490-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13278-4">
    <skos:related rdf:resource="descriptor/13682-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13278-4">
    <skos:related rdf:resource="descriptor/13658-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13278-4">
    <skos:related rdf:resource="descriptor/13671-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13409-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70882">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.19</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10718-3">
    <skos:altLabel xml:lang="en">Exchange rate appreciation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10699-2">
    <skos:altLabel xml:lang="de">Importschutz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10699-2">
    <skos:altLabel xml:lang="de">Importrestriktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10699-2">
    <skos:altLabel xml:lang="de">Importboykott</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10699-2">
    <skos:altLabel xml:lang="de">Einfuhrbeschränkung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70986">
    <skos:narrower rdf:resource="descriptor/13165-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70986">
    <skos:narrower rdf:resource="descriptor/13161-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70986">
    <skos:narrower rdf:resource="descriptor/18619-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70986">
    <skos:narrower rdf:resource="descriptor/13153-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70986">
    <skos:narrower rdf:resource="descriptor/13428-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70986">
    <skos:narrower rdf:resource="descriptor/11567-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70986">
    <skos:narrower rdf:resource="descriptor/18811-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70986">
    <skos:narrower rdf:resource="descriptor/18991-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70986">
    <skos:narrower rdf:resource="descriptor/19338-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70986">
    <skos:narrower rdf:resource="descriptor/19394-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70986">
    <skos:narrower rdf:resource="descriptor/18445-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70986">
    <skos:narrower rdf:resource="descriptor/13160-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70986">
    <skos:narrower rdf:resource="descriptor/13159-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70986">
    <skos:narrower rdf:resource="descriptor/13158-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70986">
    <skos:narrower rdf:resource="descriptor/19677-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70986">
    <skos:narrower rdf:resource="descriptor/19395-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70986">
    <skos:narrower rdf:resource="descriptor/18443-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70986">
    <skos:narrower rdf:resource="descriptor/16004-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70986">
    <skos:narrower rdf:resource="descriptor/18442-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70986">
    <skos:narrower rdf:resource="descriptor/19292-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70986">
    <skos:narrower rdf:resource="descriptor/13156-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70986">
    <skos:narrower rdf:resource="descriptor/13155-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70986">
    <skos:narrower rdf:resource="descriptor/13940-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70986">
    <skos:narrower rdf:resource="descriptor/19383-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70986">
    <skos:narrower rdf:resource="descriptor/13154-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70986">
    <skos:narrower rdf:resource="descriptor/18441-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70986">
    <skos:narrower rdf:resource="descriptor/13949-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70986">
    <skos:narrower rdf:resource="descriptor/18440-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70986">
    <skos:narrower rdf:resource="descriptor/18320-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70986">
    <skos:narrower rdf:resource="thsys/73295"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70986">
    <skos:narrower rdf:resource="thsys/73294"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10568-3">
    <skos:related rdf:resource="descriptor/10499-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10568-3">
    <skos:related rdf:resource="descriptor/19088-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10568-3">
    <skos:related rdf:resource="descriptor/10555-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10568-3">
    <skos:related rdf:resource="descriptor/10552-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18700-3">
    <skos:related rdf:resource="descriptor/18539-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18700-3">
    <skos:related rdf:resource="descriptor/12425-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18700-3">
    <skos:related rdf:resource="descriptor/12432-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12801-6">
    <skos:related rdf:resource="descriptor/19129-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12801-6">
    <skos:related rdf:resource="descriptor/12800-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18592-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13466-3">
    <skos:related rdf:resource="descriptor/13475-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13466-3">
    <skos:related rdf:resource="descriptor/12820-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13466-3">
    <skos:related rdf:resource="descriptor/13967-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13466-3">
    <skos:related rdf:resource="descriptor/18513-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13466-3">
    <skos:related rdf:resource="descriptor/18612-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13466-3">
    <skos:related rdf:resource="descriptor/12812-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70493">
    <skos:broader rdf:resource="thsys/70180"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16258-2">
    <skos:broader rdf:resource="thsys/73381"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16258-2">
    <skos:broader rdf:resource="thsys/73361"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73395">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70171">
    <skos:broader rdf:resource="thsys/70039"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13148-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15712-4">
    <skos:prefLabel xml:lang="en">Social relations</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15712-4">
    <skos:prefLabel xml:lang="de">Soziale Beziehungen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11310-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11379-1">
    <skos:altLabel xml:lang="de">Zusatzstudium</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11379-1">
    <skos:altLabel xml:lang="de">Universitätsstudium</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11379-1">
    <skos:altLabel xml:lang="de">Universitätsausbildung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11379-1">
    <skos:altLabel xml:lang="de">Kontaktstudium</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11379-1">
    <skos:altLabel xml:lang="de">Hochschulstudium</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11379-1">
    <skos:altLabel xml:lang="de">Fachhochschulstudium</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11379-1">
    <skos:altLabel xml:lang="de">Ergänzungsstudium</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11379-1">
    <skos:altLabel xml:lang="de">Aufbaustudium</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18926-5">
    <skos:broader rdf:resource="thsys/70959"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18926-5">
    <skos:broader rdf:resource="thsys/70828"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18926-5">
    <skos:broader rdf:resource="descriptor/18920-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18926-5">
    <skos:broader rdf:resource="descriptor/18522-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#broader">
    <rdfs:label xml:lang="en">has broader</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19392-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18642-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10005-5">
    <skos:altLabel xml:lang="en">Private property</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10005-5">
    <skos:altLabel xml:lang="en">Ownership</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10005-5">
    <skos:altLabel xml:lang="de">Privateigentum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18595-2">
    <skos:altLabel xml:lang="de">Containerschiffahrt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12821-0">
    <skos:prefLabel xml:lang="en">Shelf allocation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12821-0">
    <skos:prefLabel xml:lang="de">Verkaufsraumgestaltung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14466-5">
    <skos:broader rdf:resource="thsys/70135"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14466-5">
    <skos:broader rdf:resource="descriptor/14473-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18816-5">
    <skos:altLabel xml:lang="de">Standortkonversion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18816-5">
    <skos:altLabel xml:lang="de">Militärstützpunkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18816-5">
    <skos:altLabel xml:lang="de">Militärstationierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18816-5">
    <skos:altLabel xml:lang="de">Militärbasis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18816-5">
    <skos:altLabel xml:lang="de">Garnison</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19366-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18738-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12504-5">
    <skos:prefLabel xml:lang="en">Production costs</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12504-5">
    <skos:prefLabel xml:lang="de">Produktionskosten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70397">
    <skos:broader rdf:resource="thsys/70046"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16017-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19844-2">
    <skos:broader rdf:resource="thsys/73336"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19844-2">
    <skos:broader rdf:resource="descriptor/15971-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10683-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14743-5">
    <skos:broader rdf:resource="thsys/70014"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14743-5">
    <skos:broader rdf:resource="descriptor/14599-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24785-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19729-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18889-6">
    <zbwext:useInsteadNote rdf:nodeID="id95"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18429-5">
    <skos:related rdf:resource="descriptor/13378-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18429-5">
    <skos:related rdf:resource="descriptor/14331-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18429-5">
    <skos:related rdf:resource="descriptor/12996-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18429-5">
    <skos:related rdf:resource="descriptor/18408-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18429-5">
    <skos:related rdf:resource="descriptor/15638-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18429-5">
    <skos:related rdf:resource="descriptor/13327-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11313-3">
    <skos:narrower rdf:resource="descriptor/16438-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71030">
    <skos:narrower rdf:resource="descriptor/17934-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71030">
    <skos:narrower rdf:resource="descriptor/17933-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71030">
    <skos:narrower rdf:resource="descriptor/17932-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71030">
    <skos:narrower rdf:resource="descriptor/17931-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71030">
    <skos:narrower rdf:resource="descriptor/17928-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71030">
    <skos:narrower rdf:resource="descriptor/17927-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71030">
    <skos:narrower rdf:resource="descriptor/17923-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71030">
    <skos:narrower rdf:resource="descriptor/17922-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71030">
    <skos:narrower rdf:resource="descriptor/17920-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71030">
    <skos:narrower rdf:resource="descriptor/17919-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71030">
    <skos:narrower rdf:resource="descriptor/17915-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71030">
    <skos:narrower rdf:resource="descriptor/17914-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71030">
    <skos:narrower rdf:resource="descriptor/17911-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13279-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11525-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16761-0">
    <skos:broader rdf:resource="thsys/70825"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16761-0">
    <skos:broader rdf:resource="thsys/70013"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16761-0">
    <skos:broader rdf:resource="descriptor/16563-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19100-1">
    <skos:broader rdf:resource="thsys/70578"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19100-1">
    <skos:broader rdf:resource="descriptor/12480-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70958">
    <skos:narrower rdf:resource="descriptor/20136-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70958">
    <skos:narrower rdf:resource="descriptor/18619-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70958">
    <skos:narrower rdf:resource="descriptor/13321-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70958">
    <skos:narrower rdf:resource="descriptor/16672-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70958">
    <skos:narrower rdf:resource="descriptor/18126-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70958">
    <skos:narrower rdf:resource="descriptor/13316-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70958">
    <skos:narrower rdf:resource="descriptor/13871-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70958">
    <skos:narrower rdf:resource="descriptor/12782-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70958">
    <skos:narrower rdf:resource="descriptor/13481-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70958">
    <skos:narrower rdf:resource="descriptor/19604-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70958">
    <skos:narrower rdf:resource="descriptor/18013-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70958">
    <skos:narrower rdf:resource="descriptor/18821-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70958">
    <skos:narrower rdf:resource="descriptor/18101-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70958">
    <skos:narrower rdf:resource="descriptor/18717-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70958">
    <skos:narrower rdf:resource="descriptor/16601-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70958">
    <skos:narrower rdf:resource="descriptor/18535-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70958">
    <skos:narrower rdf:resource="descriptor/13301-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70958">
    <skos:narrower rdf:resource="descriptor/18454-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70958">
    <skos:narrower rdf:resource="descriptor/13315-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70958">
    <skos:narrower rdf:resource="descriptor/14346-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14109-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19800-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16336-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11545-3">
    <skos:narrower rdf:resource="descriptor/10710-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11545-3">
    <skos:narrower rdf:resource="descriptor/11547-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11545-3">
    <skos:narrower rdf:resource="descriptor/11544-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11545-3">
    <skos:narrower rdf:resource="descriptor/11543-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15868-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16549-5">
    <skos:related rdf:resource="descriptor/19697-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16549-5">
    <skos:related rdf:resource="descriptor/11998-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16549-5">
    <skos:related rdf:resource="descriptor/12004-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16549-5">
    <skos:related rdf:resource="descriptor/11852-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16549-5">
    <skos:related rdf:resource="descriptor/13639-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16549-5">
    <skos:related rdf:resource="descriptor/16266-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12239-5">
    <skos:scopeNote xml:lang="de">Bei einer Kapitalerhöhung sind den Aktionären junge Aktien im Verhältnis ihres Anteilsbesitzes zum Erhöhungsbetrag anzubieten.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18561-5">
    <skos:altLabel xml:lang="de">Elektronisches Reservierungssystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18561-5">
    <skos:altLabel xml:lang="de">Computerreservierungssystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18561-5">
    <skos:altLabel xml:lang="de">Computergestütztes Reservierungssystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13441-5">
    <skos:broader rdf:resource="thsys/71028"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13441-5">
    <skos:broader rdf:resource="thsys/70891"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13441-5">
    <skos:broader rdf:resource="descriptor/13297-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13622-1">
    <skos:broader rdf:resource="thsys/70930"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13622-1">
    <skos:broader rdf:resource="descriptor/13621-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18818-1">
    <skos:altLabel xml:lang="de">Waffenexportbeschränkung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18818-1">
    <skos:altLabel xml:lang="de">Rüstungsexportbeschränkung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18818-1">
    <skos:altLabel xml:lang="de">Exportkontrolle für Rüstungsgüter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18818-1">
    <skos:altLabel xml:lang="de">Exportbeschränkung für Rüstungsgüter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19394-0">
    <skos:altLabel xml:lang="de">Nahrungsmittelverbrauch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19394-0">
    <skos:altLabel xml:lang="de">Lebensmittelverbrauch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19394-0">
    <skos:altLabel xml:lang="de">Lebensmittelkonsum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71102">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18897-0">
    <skos:prefLabel xml:lang="en">Infections disease</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18897-0">
    <skos:prefLabel xml:lang="de">Infektionskrankheit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70253">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12713-3">
    <skos:narrower rdf:resource="descriptor/15066-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12713-3">
    <skos:narrower rdf:resource="descriptor/12155-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12713-3">
    <skos:narrower rdf:resource="descriptor/12809-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12713-3">
    <skos:narrower rdf:resource="descriptor/18821-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12713-3">
    <skos:narrower rdf:resource="descriptor/19117-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19077-5">
    <skos:broader rdf:resource="thsys/70174"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19077-5">
    <skos:broader rdf:resource="descriptor/16815-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15543-1">
    <skos:prefLabel xml:lang="en">Nonlinear programming</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15543-1">
    <skos:prefLabel xml:lang="de">Nichtlineare Optimierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18221-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18562-3">
    <skos:broader rdf:resource="thsys/70201"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12958-1">
    <skos:related rdf:resource="descriptor/14102-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12958-1">
    <skos:related rdf:resource="descriptor/19348-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12958-1">
    <skos:related rdf:resource="descriptor/13234-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12958-1">
    <skos:related rdf:resource="descriptor/14985-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12958-1">
    <skos:related rdf:resource="descriptor/13157-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13721-6">
    <skos:broader rdf:resource="thsys/71105"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13721-6">
    <skos:broader rdf:resource="descriptor/12299-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24705-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15669-0">
    <skos:related rdf:resource="descriptor/18401-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15669-0">
    <skos:related rdf:resource="descriptor/15628-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15669-0">
    <skos:related rdf:resource="descriptor/15876-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10195-2">
    <skos:prefLabel xml:lang="en">Bilateral monopoly</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10195-2">
    <skos:prefLabel xml:lang="de">Bilaterales Monopol</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14955-4">
    <skos:prefLabel xml:lang="en">Dairy product</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14955-4">
    <skos:prefLabel xml:lang="de">Milchprodukt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id361">
    <rdf:value xml:lang="de">Für die Forschung im Bereich des Raumfahrzeugbaus und der Weltraumtechnik</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10970-0">
    <skos:broader rdf:resource="thsys/73372"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10970-0">
    <skos:broader rdf:resource="thsys/70038"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10970-0">
    <skos:broader rdf:resource="descriptor/10987-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16029-3">
    <skos:prefLabel xml:lang="en">Law of inheritance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16029-3">
    <skos:prefLabel xml:lang="de">Erbrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14518-5">
    <skos:prefLabel xml:lang="en">In-car entertainment product</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14518-5">
    <skos:prefLabel xml:lang="de">Autoradio</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20368-5">
    <skos:prefLabel xml:lang="en">Stock option plan</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20368-5">
    <skos:prefLabel xml:lang="de">Aktienoptionsplan</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13613-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11819-6">
    <skos:related rdf:resource="descriptor/16055-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11819-6">
    <skos:related rdf:resource="descriptor/11818-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11819-6">
    <skos:related rdf:resource="descriptor/12910-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12987-1">
    <skos:prefLabel xml:lang="en">Seabed mining</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12987-1">
    <skos:prefLabel xml:lang="de">Meeresbergbau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12026-1">
    <skos:altLabel xml:lang="de">Immobilieneigentum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12026-1">
    <skos:altLabel xml:lang="de">Immobilienbesitz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12026-1">
    <skos:altLabel xml:lang="de">Hausbesitz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12026-1">
    <skos:altLabel xml:lang="de">Grundvermögen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12026-1">
    <skos:altLabel xml:lang="de">Grundeigentum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12026-1">
    <skos:altLabel xml:lang="de">Bodeneigentum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19564-1">
    <skos:altLabel xml:lang="de">Wollsiegel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19564-1">
    <skos:altLabel xml:lang="de">VDE-Sicherheitszeichen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19564-1">
    <skos:altLabel xml:lang="de">TÜV-Zeichen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19564-1">
    <skos:altLabel xml:lang="de">RAL-Gütezeichen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19564-1">
    <skos:altLabel xml:lang="de">Qualitätszeichen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19564-1">
    <skos:altLabel xml:lang="de">Produktkennzeichnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19564-1">
    <skos:altLabel xml:lang="de">Mindesthaltbarkeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19564-1">
    <skos:altLabel xml:lang="de">Handelsklasse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19564-1">
    <skos:altLabel xml:lang="de">Gütezeichen für Produkte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19564-1">
    <skos:altLabel xml:lang="de">Gütesiegel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19564-1">
    <skos:altLabel xml:lang="de">GS-Zeichen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19564-1">
    <skos:altLabel xml:lang="de">CE-Kennzeichen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15298-2">
    <skos:broader rdf:resource="thsys/72186"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15298-2">
    <skos:broader rdf:resource="descriptor/15258-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70838">
    <skos:narrower rdf:resource="descriptor/12620-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70838">
    <skos:narrower rdf:resource="descriptor/16531-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70838">
    <skos:narrower rdf:resource="descriptor/12615-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70838">
    <skos:narrower rdf:resource="descriptor/12618-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70838">
    <skos:narrower rdf:resource="descriptor/11388-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70838">
    <skos:narrower rdf:resource="descriptor/11269-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70838">
    <skos:narrower rdf:resource="descriptor/12610-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70838">
    <skos:narrower rdf:resource="descriptor/12609-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70838">
    <skos:narrower rdf:resource="descriptor/12617-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12609-5">
    <zbwext:useInsteadNote rdf:nodeID="id25"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71081">
    <skos:broader rdf:resource="thsys/70144"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12119-1">
    <skos:altLabel xml:lang="en">Work group</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12119-1">
    <skos:altLabel xml:lang="en">Team</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12119-1">
    <skos:altLabel xml:lang="en">Self-managed team</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12119-1">
    <skos:altLabel xml:lang="en">Autonomous work group</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12119-1">
    <skos:altLabel xml:lang="de">Teilautonome Arbeitsgruppe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12119-1">
    <skos:altLabel xml:lang="de">Teamarbeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12119-1">
    <skos:altLabel xml:lang="de">Selbststeuerndes Team</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12119-1">
    <skos:altLabel xml:lang="de">Selbststeuernde Gruppe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12119-1">
    <skos:altLabel xml:lang="de">Autonome Arbeitsgruppe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12119-1">
    <skos:altLabel xml:lang="de">Arbeitsgruppe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17678-3">
    <skos:prefLabel xml:lang="en">Tanzania</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17678-3">
    <skos:prefLabel xml:lang="de">Tansania</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18867-2">
    <skos:broader rdf:resource="thsys/70562"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18867-2">
    <skos:broader rdf:resource="thsys/70321"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18867-2">
    <skos:broader rdf:resource="descriptor/12105-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18388-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12708-3">
    <skos:related rdf:resource="descriptor/10239-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12708-3">
    <skos:related rdf:resource="descriptor/13610-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12708-3">
    <skos:related rdf:resource="descriptor/12754-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12708-3">
    <skos:related rdf:resource="descriptor/12700-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12708-3">
    <skos:related rdf:resource="descriptor/12038-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11810-3">
    <skos:prefLabel xml:lang="en">Energy policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11810-3">
    <skos:prefLabel xml:lang="de">Energiepolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17626-1">
    <skos:related rdf:resource="descriptor/18285-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11553-4">
    <skos:prefLabel xml:lang="en">Local public expenditure</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11553-4">
    <skos:prefLabel xml:lang="de">Kommunale Ausgaben</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12492-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12931-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14240-3">
    <skos:broader rdf:resource="thsys/70348"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14240-3">
    <skos:broader rdf:resource="descriptor/14236-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16182-2">
    <skos:related rdf:resource="descriptor/19705-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16182-2">
    <skos:related rdf:resource="descriptor/18772-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13545-0">
    <skos:prefLabel xml:lang="en">Shipping policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13545-0">
    <skos:prefLabel xml:lang="de">Schifffahrtspolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19306-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10329-0">
    <skos:prefLabel xml:lang="en">Putty-clay model</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10329-0">
    <skos:prefLabel xml:lang="de">Putty-Clay-Modell</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13724-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13490-6">
    <skos:scopeNote xml:lang="de">Großflächen-Einzelhandel mit vorwiegender Selbstbedienung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12563-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15480-6">
    <skos:narrower rdf:resource="descriptor/15479-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15480-6">
    <skos:narrower rdf:resource="descriptor/15478-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15480-6">
    <skos:narrower rdf:resource="descriptor/15477-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14328-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14105-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15460-5">
    <skos:prefLabel xml:lang="en">Repeated games</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15460-5">
    <skos:prefLabel xml:lang="de">Wiederholte Spiele</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12570-6">
    <skos:related rdf:resource="descriptor/13384-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12570-6">
    <skos:related rdf:resource="descriptor/11302-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19702-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16395-6">
    <skos:related rdf:resource="descriptor/16332-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16395-6">
    <skos:related rdf:resource="descriptor/16404-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16395-6">
    <skos:related rdf:resource="descriptor/16288-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16395-6">
    <skos:related rdf:resource="descriptor/16303-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16395-6">
    <skos:related rdf:resource="descriptor/16286-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16395-6">
    <skos:related rdf:resource="descriptor/16304-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16395-6">
    <skos:related rdf:resource="descriptor/16454-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16395-6">
    <skos:related rdf:resource="descriptor/16510-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16395-6">
    <skos:related rdf:resource="descriptor/16397-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19023-0">
    <skos:broader rdf:resource="thsys/70506"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19023-0">
    <skos:broader rdf:resource="thsys/70403"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19023-0">
    <skos:broader rdf:resource="descriptor/14160-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10605-2">
    <skos:altLabel xml:lang="de">Gesamtwirtschaftlicher Außenhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10605-2">
    <skos:altLabel xml:lang="de">Außenhandel der Gesamtwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70931">
    <skos:narrower rdf:resource="descriptor/13578-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70931">
    <skos:narrower rdf:resource="descriptor/16173-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70931">
    <skos:narrower rdf:resource="descriptor/15900-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70931">
    <skos:narrower rdf:resource="descriptor/15899-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70931">
    <skos:narrower rdf:resource="descriptor/13684-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70931">
    <skos:narrower rdf:resource="descriptor/18419-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70931">
    <skos:narrower rdf:resource="descriptor/13507-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16176-4">
    <skos:prefLabel xml:lang="en">Building legislation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16176-4">
    <skos:prefLabel xml:lang="de">Baurecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10622-2">
    <zbwext:useInsteadNote rdf:nodeID="id236"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10622-2">
    <zbwext:useInsteadNote rdf:nodeID="id235"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11788-5">
    <skos:altLabel xml:lang="de">Schrumpfender Markt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11788-5">
    <skos:altLabel xml:lang="de">Schrumpfende Märkte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13020-1">
    <skos:altLabel xml:lang="de">Steinmetzgewerbe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13020-1">
    <skos:altLabel xml:lang="de">Herstellung von Mineralerzeugnissen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10555-5">
    <skos:broader rdf:resource="thsys/73366"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10555-5">
    <skos:broader rdf:resource="thsys/71084"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10555-5">
    <skos:broader rdf:resource="descriptor/16489-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12693-4">
    <skos:prefLabel xml:lang="en">Materials-handling equipment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12693-4">
    <skos:prefLabel xml:lang="de">Fördermittel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14822-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18799-0">
    <skos:related rdf:resource="descriptor/18797-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18799-0">
    <skos:related rdf:resource="descriptor/12783-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18591-3">
    <skos:scopeNote xml:lang="de">Umfasst alle schifffahrtsrelevanten rechtlichen Bestimmungen, einschl. Hafenordnungen und Schiffsregister.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20921-0">
    <skos:altLabel xml:lang="en">B2B portal</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20921-0">
    <skos:altLabel xml:lang="de">Web-Portal</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20921-0">
    <skos:altLabel xml:lang="de">Unternehmensportal</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20921-0">
    <skos:altLabel xml:lang="de">Unternehmens-Portal</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20921-0">
    <skos:altLabel xml:lang="de">Portal (Internet)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20921-0">
    <skos:altLabel xml:lang="de">Internetportal</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16458-1">
    <skos:scopeNote xml:lang="de">Gesamtheit der verfügbaren militärischen Ressourcen eines Staates.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73396">
    <skos:prefLabel xml:lang="de">N.06.02.01  Gesellschaftsordnungen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70046">
    <skos:narrower rdf:resource="descriptor/11679-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70046">
    <skos:narrower rdf:resource="descriptor/11678-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70046">
    <skos:narrower rdf:resource="descriptor/11696-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70046">
    <skos:narrower rdf:resource="descriptor/11709-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70046">
    <skos:narrower rdf:resource="descriptor/12558-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70046">
    <skos:narrower rdf:resource="descriptor/11640-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70046">
    <skos:narrower rdf:resource="descriptor/12876-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70046">
    <skos:narrower rdf:resource="descriptor/19281-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70046">
    <skos:narrower rdf:resource="descriptor/24858-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70046">
    <skos:narrower rdf:resource="descriptor/11636-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70046">
    <skos:narrower rdf:resource="descriptor/11702-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70046">
    <skos:narrower rdf:resource="descriptor/11547-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70046">
    <skos:narrower rdf:resource="descriptor/11669-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70046">
    <skos:narrower rdf:resource="descriptor/19143-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70046">
    <skos:narrower rdf:resource="descriptor/11657-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70046">
    <skos:narrower rdf:resource="descriptor/12848-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70046">
    <skos:narrower rdf:resource="thsys/71040"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70046">
    <skos:narrower rdf:resource="thsys/70184"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70046">
    <skos:narrower rdf:resource="thsys/70397"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11862-5">
    <skos:prefLabel xml:lang="en">Village</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11862-5">
    <skos:prefLabel xml:lang="de">Dorf</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13507-1">
    <skos:prefLabel xml:lang="en">Space travel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13507-1">
    <skos:prefLabel xml:lang="de">Raumfahrt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10901-5">
    <skos:broader rdf:resource="thsys/72184"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10901-5">
    <skos:broader rdf:resource="thsys/70212"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10901-5">
    <skos:broader rdf:resource="descriptor/10913-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10901-5">
    <skos:broader rdf:resource="descriptor/15082-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12563-3">
    <skos:broader rdf:resource="thsys/70244"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12563-3">
    <skos:broader rdf:resource="descriptor/12761-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12563-3">
    <skos:broader rdf:resource="descriptor/12565-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11979-5">
    <skos:altLabel xml:lang="en">Standard metropolitan statistical area</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11979-5">
    <skos:altLabel xml:lang="de">Verdichtungsraum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11979-5">
    <skos:altLabel xml:lang="de">Verdichtungsgebiet</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11979-5">
    <skos:altLabel xml:lang="de">Städtischer Verdichtungsraum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11979-5">
    <skos:altLabel xml:lang="de">SMSA</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11979-5">
    <skos:altLabel xml:lang="de">Kontraktionsraum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11979-5">
    <skos:altLabel xml:lang="de">Ballungsgebiet</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11979-5">
    <skos:altLabel xml:lang="de">Agglomerationsraum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16312-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11417-5">
    <skos:narrower rdf:resource="descriptor/16046-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11417-5">
    <skos:narrower rdf:resource="descriptor/10780-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11703-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15907-3">
    <skos:broader rdf:resource="thsys/73358"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15907-3">
    <skos:broader rdf:resource="thsys/70142"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15907-3">
    <skos:broader rdf:resource="descriptor/15909-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14713-0">
    <skos:altLabel xml:lang="de">PUR (Polyurethan)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14413-5">
    <skos:altLabel xml:lang="de">Panzer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19142-6">
    <skos:related rdf:resource="descriptor/12326-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19142-6">
    <skos:related rdf:resource="descriptor/12357-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12848-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13904-5">
    <skos:prefLabel xml:lang="en">Leather professions</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13904-5">
    <skos:prefLabel xml:lang="de">Lederberufe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18163-3">
    <skos:related rdf:resource="descriptor/10211-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18163-3">
    <skos:related rdf:resource="descriptor/18812-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18163-3">
    <skos:related rdf:resource="descriptor/18360-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18163-3">
    <skos:related rdf:resource="descriptor/14174-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13057-6">
    <skos:altLabel xml:lang="de">Flugzeugwerft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13057-6">
    <skos:altLabel xml:lang="de">Flugzeugindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13057-6">
    <skos:altLabel xml:lang="de">Flugzeugbau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19666-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17750-0">
    <skos:broader rdf:resource="thsys/70085"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17750-0">
    <skos:broader rdf:resource="descriptor/17642-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18912-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17589-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19517-3">
    <skos:prefLabel xml:lang="en">Health care financing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19517-3">
    <skos:prefLabel xml:lang="de">Gesundheitsfinanzierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18199-3">
    <skos:related rdf:resource="descriptor/14723-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71053">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.13.06.02.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18704-2">
    <skos:broader rdf:resource="thsys/70904"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18704-2">
    <skos:broader rdf:resource="thsys/70305"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12509-2">
    <skos:prefLabel xml:lang="en">Variable costs</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12509-2">
    <skos:prefLabel xml:lang="de">Variable Kosten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16112-2">
    <skos:altLabel xml:lang="en">Tax evasion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16112-2">
    <skos:altLabel xml:lang="de">Steuervergehen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16112-2">
    <skos:altLabel xml:lang="de">Steuerstraftat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16112-2">
    <skos:altLabel xml:lang="de">Steuerstrafrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16112-2">
    <skos:altLabel xml:lang="de">Steuerhinterziehung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16112-2">
    <skos:altLabel xml:lang="de">Steuerhehlerei</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16112-2">
    <skos:altLabel xml:lang="de">Steuerbetrug</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18063-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id206">
    <rdfs:seeAlso rdf:resource="descriptor/18932-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19514-2">
    <skos:prefLabel xml:lang="en">Machine contribution</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19514-2">
    <skos:prefLabel xml:lang="de">Maschinensteuer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18851-3">
    <skos:broader rdf:resource="thsys/71020"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18851-3">
    <skos:broader rdf:resource="thsys/70852"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18851-3">
    <skos:broader rdf:resource="descriptor/10966-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11873-0">
    <skos:broader rdf:resource="thsys/71070"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11873-0">
    <skos:broader rdf:resource="descriptor/11951-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10065-1">
    <skos:prefLabel xml:lang="en">Economic model</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10065-1">
    <skos:prefLabel xml:lang="de">Wirtschaftsmodell</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12571-4">
    <skos:related rdf:resource="descriptor/12574-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12571-4">
    <skos:related rdf:resource="descriptor/11295-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12571-4">
    <skos:related rdf:resource="descriptor/11303-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13000-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18653-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15119-0">
    <skos:narrower rdf:resource="descriptor/15125-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15119-0">
    <skos:narrower rdf:resource="descriptor/15023-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10901-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13285-0">
    <skos:altLabel xml:lang="de">Kreditgarantiegemeinschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13285-0">
    <skos:altLabel xml:lang="de">Avalbank</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70928">
    <skos:broader rdf:resource="thsys/70943"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18007-3">
    <zbwext:useInsteadNote rdf:nodeID="id3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70455">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.05.04</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10208-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73306">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72184">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10452-1">
    <skos:altLabel xml:lang="de">Internationale wissenschaftlich-technische Zusammenarbeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12720-6">
    <skos:related rdf:resource="descriptor/12119-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18889-6">
    <skos:broader rdf:resource="thsys/73362"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18889-6">
    <skos:broader rdf:resource="descriptor/18772-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18889-6">
    <skos:broader rdf:resource="descriptor/11397-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10984-3">
    <skos:altLabel xml:lang="en">Pressure group</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10984-3">
    <skos:altLabel xml:lang="de">Verband</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10984-3">
    <skos:altLabel xml:lang="de">Lobby</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10984-3">
    <skos:altLabel xml:lang="de">Interessenvereinigung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10984-3">
    <skos:altLabel xml:lang="de">Interessenverband</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10984-3">
    <skos:altLabel xml:lang="de">Interessenorganisation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10984-3">
    <skos:altLabel xml:lang="de">Interessengruppe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10984-3">
    <skos:altLabel xml:lang="de">Interessengemeinschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19549-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71112">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19295-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19374-6">
    <skos:prefLabel xml:lang="en">Fiscal illusion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19374-6">
    <skos:prefLabel xml:lang="de">Fiskalillusion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19262-3">
    <skos:related rdf:resource="descriptor/11485-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19262-3">
    <skos:related rdf:resource="descriptor/19547-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18313-3">
    <skos:altLabel xml:lang="de">Plantagenwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18313-3">
    <skos:altLabel xml:lang="de">Plantage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18313-3">
    <skos:altLabel xml:lang="de">Pflanzung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18313-3">
    <skos:altLabel xml:lang="de">Landwirtschaftliche Betriebsform</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18313-3">
    <skos:altLabel xml:lang="de">Agrarbetriebstyp</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18313-3">
    <skos:altLabel xml:lang="de">Agrarbetriebssystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19306-2">
    <skos:prefLabel xml:lang="en">Sugar market</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19306-2">
    <skos:prefLabel xml:lang="de">Zuckermarkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19697-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15365-6">
    <skos:altLabel xml:lang="en">Method of instrumental variables</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15365-6">
    <skos:altLabel xml:lang="en">Instrumental variables</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10676-0">
    <skos:prefLabel xml:lang="en">International monetary theory</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10676-0">
    <skos:prefLabel xml:lang="de">Monetäre Außenwirtschaftstheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id143">
    <rdf:value xml:lang="de">Für den praktischen Einsatz und die Ausgestaltung dv-technischer Anlagen</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16104-1">
    <skos:prefLabel xml:lang="en">Corporate income tax law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16104-1">
    <skos:prefLabel xml:lang="de">Körperschaftsteuerrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15411-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15869-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13533-0">
    <skos:prefLabel xml:lang="en">Port</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13533-0">
    <skos:prefLabel xml:lang="de">Hafen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id265">
    <rdfs:seeAlso rdf:resource="descriptor/12741-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71002">
    <rdfs:label xml:lang="de">Maschinen- und Anlagenbau</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19046-2">
    <skos:related rdf:resource="descriptor/18851-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19046-2">
    <skos:related rdf:resource="descriptor/13429-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12055-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19300-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70328">
    <skos:broader rdf:resource="thsys/p"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15543-1">
    <skos:altLabel xml:lang="en">Separable programming</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15543-1">
    <skos:altLabel xml:lang="en">Nonconvex programming</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15543-1">
    <skos:altLabel xml:lang="en">Fractional programming</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15543-1">
    <skos:altLabel xml:lang="de">Quotientenprogrammierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15543-1">
    <skos:altLabel xml:lang="de">Quadratische Optimierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15543-1">
    <skos:altLabel xml:lang="de">Nichtlineare Programmierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15543-1">
    <skos:altLabel xml:lang="de">Nichtkonvexe Optimierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15543-1">
    <skos:altLabel xml:lang="de">Kuhn-Tucker-Bedingung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15543-1">
    <skos:altLabel xml:lang="de">Konvexe Optimierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15543-1">
    <skos:altLabel xml:lang="de">Geometrische Optimierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16124-2">
    <skos:broader rdf:resource="thsys/73336"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16124-2">
    <skos:broader rdf:resource="thsys/70835"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16124-2">
    <skos:broader rdf:resource="descriptor/11185-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19162-0">
    <skos:broader rdf:resource="thsys/70582"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id104">
    <rdf:value xml:lang="de">Für den technischen Fortschritt</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13394-2">
    <skos:prefLabel xml:lang="en">Association of local authorities</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13394-2">
    <skos:prefLabel xml:lang="de">Kommunaler Spitzenverband</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10459-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14955-4">
    <skos:narrower rdf:resource="descriptor/14995-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14955-4">
    <skos:narrower rdf:resource="descriptor/14954-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14955-4">
    <skos:narrower rdf:resource="descriptor/14953-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14955-4">
    <skos:narrower rdf:resource="descriptor/18217-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15891-6">
    <skos:related rdf:resource="descriptor/19614-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15891-6">
    <skos:related rdf:resource="descriptor/15887-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15891-6">
    <skos:related rdf:resource="descriptor/15872-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15115-1">
    <skos:narrower rdf:resource="descriptor/15135-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15115-1">
    <skos:narrower rdf:resource="descriptor/15124-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15115-1">
    <skos:narrower rdf:resource="descriptor/15119-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18996-5">
    <skos:prefLabel xml:lang="en">Energy market</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18996-5">
    <skos:prefLabel xml:lang="de">Energiemarkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11342-3">
    <skos:altLabel xml:lang="en">Wage setting</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11342-3">
    <skos:altLabel xml:lang="en">Wage formation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11342-3">
    <skos:altLabel xml:lang="de">Lohnfestsetzung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18216-1">
    <skos:narrower rdf:resource="descriptor/18215-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14073-3">
    <skos:broader rdf:resource="thsys/73351"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14073-3">
    <skos:broader rdf:resource="thsys/70341"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70085">
    <skos:broader rdf:resource="thsys/70076"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10428-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20855-1">
    <skos:related rdf:resource="descriptor/16642-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20855-1">
    <skos:related rdf:resource="descriptor/15806-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20855-1">
    <skos:related rdf:resource="descriptor/16329-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20855-1">
    <skos:related rdf:resource="descriptor/16207-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20855-1">
    <skos:related rdf:resource="descriptor/16209-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20855-1">
    <skos:related rdf:resource="descriptor/12009-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20855-1">
    <skos:related rdf:resource="descriptor/11161-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11969-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17999-4">
    <skos:prefLabel xml:lang="en">Western world</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17999-4">
    <skos:prefLabel xml:lang="de">Westliche Staaten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11365-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12603-3">
    <skos:altLabel xml:lang="de">Menschengerechte Gestaltung der Arbeitswelt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12603-3">
    <skos:altLabel xml:lang="de">Menschengerechte Arbeitsplatzgestaltung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17612-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14972-4">
    <skos:broader rdf:resource="thsys/73300"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14972-4">
    <skos:broader rdf:resource="thsys/73298"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14972-4">
    <skos:broader rdf:resource="descriptor/14976-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14972-4">
    <skos:broader rdf:resource="descriptor/18222-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11209-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13320-3">
    <skos:prefLabel xml:lang="en">Fitness center</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13320-3">
    <skos:prefLabel xml:lang="de">Fitnesszentrum</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19095-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16041-6">
    <skos:altLabel xml:lang="en">Civil procedure</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16041-6">
    <skos:altLabel xml:lang="de">Zivilprozessrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16041-6">
    <skos:altLabel xml:lang="de">Zivilprozessordnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16041-6">
    <skos:altLabel xml:lang="de">Zivilprozeßrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16041-6">
    <skos:altLabel xml:lang="de">Zivilprozeßordnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16041-6">
    <skos:altLabel xml:lang="de">Zivilprozeß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16041-6">
    <skos:altLabel xml:lang="de">Litigation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19046-2">
    <skos:narrower rdf:resource="descriptor/10576-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10541-2">
    <skos:altLabel xml:lang="en">Unbalanced-growth-strategy</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10541-2">
    <skos:altLabel xml:lang="en">Disequilibrium growth theory</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10541-2">
    <skos:altLabel xml:lang="de">Ungleichgewichtswachstum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10541-2">
    <skos:altLabel xml:lang="de">Unbalanced-Growth-Strategie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10541-2">
    <skos:altLabel xml:lang="de">Strategie des ungleichgewichtigen Wachstums</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10541-2">
    <skos:altLabel xml:lang="de">Nichtgleichgewichtiges Wachstumsmodell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18584-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14453-0">
    <skos:broader rdf:resource="thsys/70257"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14453-0">
    <skos:broader rdf:resource="descriptor/13513-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11090-4">
    <skos:prefLabel xml:lang="en">Economic theory of socialism</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11090-4">
    <skos:prefLabel xml:lang="de">Sozialistische Wirtschaftstheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17680-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71115">
    <skos:narrower rdf:resource="descriptor/19232-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71115">
    <skos:narrower rdf:resource="descriptor/19589-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71115">
    <skos:narrower rdf:resource="descriptor/10274-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71115">
    <skos:narrower rdf:resource="descriptor/10024-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71115">
    <skos:narrower rdf:resource="descriptor/11754-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71115">
    <skos:narrower rdf:resource="descriptor/11616-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71115">
    <skos:narrower rdf:resource="descriptor/19553-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71115">
    <skos:narrower rdf:resource="descriptor/11326-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71115">
    <skos:narrower rdf:resource="descriptor/11740-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71115">
    <skos:narrower rdf:resource="descriptor/18982-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71115">
    <skos:narrower rdf:resource="descriptor/19966-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71115">
    <skos:narrower rdf:resource="descriptor/18948-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71115">
    <skos:narrower rdf:resource="descriptor/11774-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71115">
    <skos:narrower rdf:resource="descriptor/18680-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71115">
    <skos:narrower rdf:resource="descriptor/15102-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71115">
    <skos:narrower rdf:resource="descriptor/10006-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71115">
    <skos:narrower rdf:resource="thsys/71114"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71115">
    <skos:narrower rdf:resource="thsys/71113"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18315-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73345">
    <skos:broader rdf:resource="thsys/70151"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11562-3">
    <skos:prefLabel xml:lang="en">Agricultural subsidy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11562-3">
    <skos:prefLabel xml:lang="de">Agrarsubvention</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15995-1">
    <skos:related rdf:resource="descriptor/16016-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15995-1">
    <skos:related rdf:resource="descriptor/18809-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15995-1">
    <skos:related rdf:resource="descriptor/11852-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15080-1">
    <skos:prefLabel xml:lang="en">Wholesale price index</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15080-1">
    <skos:prefLabel xml:lang="de">Großhandelspreisindex</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11272-5">
    <skos:altLabel xml:lang="en">Labour force participation theory</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11272-5">
    <skos:altLabel xml:lang="en">Labor force participation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11272-5">
    <skos:altLabel xml:lang="de">Erwerbsverhalten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11272-5">
    <skos:altLabel xml:lang="de">Erwerbsquote</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11272-5">
    <skos:altLabel xml:lang="de">Erwerbsbeteiligungstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11272-5">
    <skos:altLabel xml:lang="de">Erwerbsbeteiligung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19113-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18407-1">
    <skos:prefLabel xml:lang="en">Mining machines construction</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18407-1">
    <skos:prefLabel xml:lang="de">Bergwerksmaschinenbau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19754-3">
    <skos:broader rdf:resource="thsys/70120"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10177-4">
    <zbwext:useInsteadNote rdf:nodeID="id132"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12589-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14927-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12876-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19450-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19510-3">
    <skos:related rdf:resource="descriptor/19384-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11939-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15801-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70530">
    <skos:broader rdf:resource="thsys/70234"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13156-4">
    <skos:broader rdf:resource="thsys/70986"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13156-4">
    <skos:broader rdf:resource="descriptor/13160-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12884-4">
    <skos:narrower rdf:resource="descriptor/18184-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12884-4">
    <skos:narrower rdf:resource="descriptor/19001-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12884-4">
    <skos:narrower rdf:resource="descriptor/18833-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12884-4">
    <skos:narrower rdf:resource="descriptor/19104-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18560-0">
    <skos:related rdf:resource="descriptor/18565-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18560-0">
    <skos:related rdf:resource="descriptor/13975-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14026-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18004-2">
    <skos:altLabel xml:lang="en">Workstation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18004-2">
    <skos:altLabel xml:lang="en">Home computer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18004-2">
    <skos:altLabel xml:lang="de">Personalcomputer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18004-2">
    <skos:altLabel xml:lang="de">Mikrocomputer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18004-2">
    <skos:altLabel xml:lang="de">Arbeitsplatzrechner</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70030">
    <skos:broader rdf:resource="thsys/b"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71057">
    <skos:broader rdf:resource="thsys/70052"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16523-2">
    <skos:altLabel xml:lang="en">Civil alternative service</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16523-2">
    <skos:altLabel xml:lang="de">Ziviler Ersatzdienst</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16523-2">
    <skos:altLabel xml:lang="de">Zivildienstleistende</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16523-2">
    <skos:altLabel xml:lang="de">Wehrersatzdienst</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16523-2">
    <skos:altLabel xml:lang="de">Ersatzdienst</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13594-1">
    <skos:narrower rdf:resource="descriptor/13592-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19604-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10349-1">
    <skos:scopeNote xml:lang="de">Makroökonomische Betrachtung der Lagerhaltung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12437-1">
    <skos:related rdf:resource="descriptor/12558-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18182-6">
    <skos:prefLabel xml:lang="en">Defence conversion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18182-6">
    <skos:prefLabel xml:lang="de">Rüstungskonversion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10650-4">
    <skos:related rdf:resource="descriptor/10628-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10650-4">
    <skos:related rdf:resource="descriptor/10103-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10656-6">
    <skos:related rdf:resource="descriptor/10007-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10656-6">
    <skos:related rdf:resource="descriptor/18538-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10656-6">
    <skos:related rdf:resource="descriptor/10833-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20241-1">
    <skos:broader rdf:resource="thsys/71036"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20241-1">
    <skos:broader rdf:resource="descriptor/12072-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19345-6">
    <skos:related rdf:resource="descriptor/19346-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19345-6">
    <skos:related rdf:resource="descriptor/12953-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19345-6">
    <skos:related rdf:resource="descriptor/14144-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13053-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19822-5">
    <skos:broader rdf:resource="thsys/70154"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19822-5">
    <skos:broader rdf:resource="descriptor/15966-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17678-3">
    <skos:broader rdf:resource="thsys/70445"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17678-3">
    <skos:broader rdf:resource="descriptor/17672-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13231-2">
    <skos:related rdf:resource="descriptor/13083-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13231-2">
    <skos:related rdf:resource="descriptor/14543-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13231-2">
    <skos:related rdf:resource="descriptor/13095-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13231-2">
    <skos:related rdf:resource="descriptor/14593-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13231-2">
    <skos:related rdf:resource="descriptor/14227-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13231-2">
    <skos:related rdf:resource="descriptor/14296-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73297">
    <skos:broader rdf:resource="thsys/70996"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15823-2">
    <zbwext:useInsteadNote rdf:nodeID="id107"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14738-5">
    <skos:related rdf:resource="descriptor/14741-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14738-5">
    <skos:related rdf:resource="descriptor/14734-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14738-5">
    <skos:related rdf:resource="descriptor/14026-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20301-2">
    <skos:prefLabel xml:lang="en">Real option</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20301-2">
    <skos:prefLabel xml:lang="de">Realoption</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19224-4">
    <skos:broader rdf:resource="thsys/70178"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19224-4">
    <skos:broader rdf:resource="thsys/70058"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19224-4">
    <skos:broader rdf:resource="descriptor/11875-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11666-5">
    <skos:related rdf:resource="descriptor/12868-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11666-5">
    <skos:related rdf:resource="descriptor/16104-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11666-5">
    <skos:related rdf:resource="descriptor/11650-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11666-5">
    <skos:related rdf:resource="descriptor/19418-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13611-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14105-2">
    <skos:altLabel xml:lang="de">Wurzelgemüse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14105-2">
    <skos:altLabel xml:lang="de">Salatpflanze</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14105-2">
    <skos:altLabel xml:lang="de">Paprika</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14105-2">
    <skos:altLabel xml:lang="de">Kohl</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14105-2">
    <skos:altLabel xml:lang="de">Knollengemüse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14105-2">
    <skos:altLabel xml:lang="de">Frischgemüse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16853-2">
    <skos:prefLabel xml:lang="en">West Germany</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16853-2">
    <skos:prefLabel xml:lang="de">Alte Bundesländer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18805-3">
    <skos:prefLabel xml:lang="en">Concession</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18805-3">
    <skos:prefLabel xml:lang="de">Konzession</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19442-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18025-1">
    <skos:altLabel xml:lang="de">Pensionsreform</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10490-0">
    <skos:broader rdf:resource="thsys/71011"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13514-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11541-4">
    <skos:related rdf:resource="descriptor/11539-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11541-4">
    <skos:related rdf:resource="descriptor/16100-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11541-4">
    <skos:related rdf:resource="descriptor/19941-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10606-0">
    <skos:altLabel xml:lang="de">Außenwirtschaftsverordnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10606-0">
    <skos:altLabel xml:lang="de">Außenwirtschaftsgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10606-0">
    <skos:altLabel xml:lang="de">Außenhandelsrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10466-4">
    <skos:altLabel xml:lang="en">Technological innovation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10466-4">
    <skos:altLabel xml:lang="de">Wissenschaftlich-technischer Fortschritt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10466-4">
    <skos:altLabel xml:lang="de">Wissenschaftlich-technische Revolution</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10466-4">
    <skos:altLabel xml:lang="de">Technologischer Wandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10466-4">
    <skos:altLabel xml:lang="de">Technologischer Fortschritt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10466-4">
    <skos:altLabel xml:lang="de">Technologische Entwicklung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10466-4">
    <skos:altLabel xml:lang="de">Technischer Wandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10466-4">
    <skos:altLabel xml:lang="de">Technische Innovation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10466-4">
    <skos:altLabel xml:lang="de">Neue Technologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10466-4">
    <skos:altLabel xml:lang="de">Innovation der Technik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11293-4">
    <skos:related rdf:resource="descriptor/19112-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11293-4">
    <skos:related rdf:resource="descriptor/11394-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11293-4">
    <skos:related rdf:resource="descriptor/19505-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11293-4">
    <skos:related rdf:resource="descriptor/11290-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11293-4">
    <skos:related rdf:resource="descriptor/19767-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11293-4">
    <skos:related rdf:resource="descriptor/11303-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11079-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19626-5">
    <skos:prefLabel xml:lang="en">Fishery protection</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19626-5">
    <skos:prefLabel xml:lang="de">Fischschutz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18537-2">
    <skos:broader rdf:resource="thsys/73387"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18537-2">
    <skos:broader rdf:resource="thsys/70109"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18537-2">
    <skos:broader rdf:resource="descriptor/18534-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#example">
    <rdfs:label xml:lang="en">example</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11618-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14037-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15779-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11397-6">
    <skos:altLabel xml:lang="en">Education planning</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11397-6">
    <skos:altLabel xml:lang="de">Bildungsplanung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11397-6">
    <skos:altLabel xml:lang="de">Bildungsgesamtplan</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11397-6">
    <skos:altLabel xml:lang="de">Bildungsförderung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12136-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14273-2">
    <skos:narrower rdf:resource="descriptor/18229-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70855">
    <skos:broader rdf:resource="thsys/70856"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12035-0">
    <skos:altLabel xml:lang="en">Back orders</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12035-0">
    <skos:altLabel xml:lang="de">Auftragseingang</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12035-0">
    <skos:altLabel xml:lang="de">Auftragsbestand</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18440-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11271-0">
    <skos:related rdf:resource="descriptor/12573-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11271-0">
    <skos:related rdf:resource="descriptor/11224-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11271-0">
    <skos:related rdf:resource="descriptor/11272-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11271-0">
    <skos:related rdf:resource="descriptor/11183-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18773-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19223-6">
    <skos:related rdf:resource="descriptor/20522-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19223-6">
    <skos:related rdf:resource="descriptor/10208-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19223-6">
    <skos:related rdf:resource="descriptor/10198-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70051">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14507-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18696-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13983-4">
    <skos:related rdf:resource="descriptor/13256-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13983-4">
    <skos:related rdf:resource="descriptor/13252-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13983-4">
    <skos:related rdf:resource="descriptor/18599-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19338-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11529-1">
    <skos:prefLabel xml:lang="en">Cost-benefit analysis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11529-1">
    <skos:prefLabel xml:lang="de">Kosten-Nutzen-Analyse</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18149-4">
    <skos:altLabel xml:lang="en">Seigniorage gain</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18149-4">
    <skos:altLabel xml:lang="de">Prägegewinn</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18149-4">
    <skos:altLabel xml:lang="de">Münzprägegewinn</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18149-4">
    <skos:altLabel xml:lang="de">Geldschöpfungsgewinn</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19421-2">
    <skos:related rdf:resource="descriptor/13207-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19421-2">
    <skos:related rdf:resource="descriptor/14976-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19421-2">
    <skos:related rdf:resource="descriptor/13306-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19421-2">
    <skos:related rdf:resource="descriptor/11646-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10621-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11241-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19852-3">
    <skos:prefLabel xml:lang="en">Legal aid</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19852-3">
    <skos:prefLabel xml:lang="de">Prozesskostenhilfe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15938-6">
    <skos:broader rdf:resource="thsys/73357"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15938-6">
    <skos:broader rdf:resource="thsys/70814"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15938-6">
    <skos:broader rdf:resource="thsys/70142"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15938-6">
    <skos:broader rdf:resource="descriptor/18969-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17660-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17628-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18012-3">
    <zbwext:useInsteadNote rdf:nodeID="id68"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19525-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20206-3">
    <skos:scopeNote xml:lang="de">Preis-Mengen-Management für Dienstleistungen; Ertragsoptimierungsmethode</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16605-0">
    <skos:altLabel xml:lang="en">Affluent society</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16605-0">
    <skos:altLabel xml:lang="de">Wohlstandsgesellschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16605-0">
    <skos:altLabel xml:lang="de">Wegwerfgesellschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16605-0">
    <skos:altLabel xml:lang="de">Überflussgesellschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16605-0">
    <skos:altLabel xml:lang="de">Überflußgesellschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12134-5">
    <skos:broader rdf:resource="thsys/70562"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12134-5">
    <skos:broader rdf:resource="descriptor/19562-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11832-0">
    <skos:scopeNote xml:lang="de">vgl. auch: Verkehrspolitik.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11832-0">
    <skos:scopeNote xml:lang="de">Möglichst nur für konkrete Verkehrpläne und -konzepte.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18189-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11032-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10481-1">
    <skos:related rdf:resource="descriptor/16037-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10481-1">
    <skos:related rdf:resource="descriptor/10478-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10481-1">
    <skos:related rdf:resource="descriptor/16078-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10481-1">
    <skos:related rdf:resource="descriptor/18065-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10481-1">
    <skos:related rdf:resource="descriptor/10475-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19751-2">
    <skos:broader rdf:resource="thsys/70321"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10266-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19089-5">
    <skos:broader rdf:resource="thsys/70036"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14135-0">
    <skos:broader rdf:resource="thsys/70403"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14135-0">
    <skos:broader rdf:resource="descriptor/19022-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14135-0">
    <skos:broader rdf:resource="descriptor/14145-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15515-6">
    <skos:altLabel xml:lang="de">Sensibilitätsanalyse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11921-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18646-4">
    <skos:related rdf:resource="descriptor/18644-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18646-4">
    <skos:related rdf:resource="descriptor/13647-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18646-4">
    <skos:related rdf:resource="descriptor/13274-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14779-5">
    <skos:related rdf:resource="descriptor/13213-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14779-5">
    <skos:related rdf:resource="descriptor/14032-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12089-5">
    <skos:related rdf:resource="descriptor/12071-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12089-5">
    <skos:related rdf:resource="descriptor/12090-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12089-5">
    <skos:related rdf:resource="descriptor/12108-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12089-5">
    <skos:related rdf:resource="descriptor/12085-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12089-5">
    <skos:related rdf:resource="descriptor/10192-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71092">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10710-5">
    <skos:altLabel xml:lang="de">Schutzzoll</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10710-5">
    <skos:altLabel xml:lang="de">Fiskalischer Zoll</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10710-5">
    <skos:altLabel xml:lang="de">Finanzzoll</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17968-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19526-2">
    <skos:prefLabel xml:lang="en">Vaccination</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19526-2">
    <skos:prefLabel xml:lang="de">Impfung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12801-6">
    <skos:broader rdf:resource="thsys/70550"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12801-6">
    <skos:broader rdf:resource="descriptor/10016-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11913-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17374-2">
    <skos:altLabel xml:lang="en">Great Britain</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17374-2">
    <skos:altLabel xml:lang="de">Vereinigtes Königreich</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17374-2">
    <skos:altLabel xml:lang="de">UK (United Kingdom)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17374-2">
    <skos:altLabel xml:lang="de">England (Staat)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11362-4">
    <skos:broader rdf:resource="thsys/70845"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11362-4">
    <skos:broader rdf:resource="thsys/70843"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11362-4">
    <skos:broader rdf:resource="descriptor/11377-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11362-4">
    <skos:broader rdf:resource="descriptor/11360-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16301-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70996">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16674-2">
    <skos:broader rdf:resource="thsys/73356"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15101-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18030-1">
    <skos:altLabel xml:lang="de">Technischer Unfall</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18030-1">
    <skos:altLabel xml:lang="de">Störfallverordnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18030-1">
    <skos:altLabel xml:lang="de">Störfallfolgen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18030-1">
    <skos:altLabel xml:lang="de">Schnellabschaltung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18030-1">
    <skos:altLabel xml:lang="de">Industrieunfall</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10452-1">
    <skos:narrower rdf:resource="descriptor/10598-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12585-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id338">
    <rdf:value xml:lang="de">Für Unterernährung größerer Bevölkerungsgruppen infolge von Verarmung, Krieg oder Naturkatastrophen</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10650-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13406-0">
    <skos:broader rdf:resource="thsys/71061"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13406-0">
    <skos:broader rdf:resource="thsys/70868"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13406-0">
    <skos:broader rdf:resource="thsys/70832"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13406-0">
    <skos:broader rdf:resource="thsys/70458"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13406-0">
    <skos:broader rdf:resource="descriptor/13418-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13406-0">
    <skos:broader rdf:resource="descriptor/11240-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71071">
    <skos:narrower rdf:resource="descriptor/10964-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71071">
    <skos:narrower rdf:resource="descriptor/10963-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71071">
    <skos:narrower rdf:resource="descriptor/10962-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71071">
    <skos:narrower rdf:resource="descriptor/20522-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71071">
    <skos:narrower rdf:resource="descriptor/10961-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71071">
    <skos:narrower rdf:resource="descriptor/19560-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71071">
    <skos:narrower rdf:resource="descriptor/10893-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71071">
    <skos:narrower rdf:resource="descriptor/10894-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71071">
    <skos:narrower rdf:resource="descriptor/19435-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71071">
    <skos:narrower rdf:resource="descriptor/10960-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71071">
    <skos:narrower rdf:resource="descriptor/10958-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71071">
    <skos:narrower rdf:resource="descriptor/19434-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71071">
    <skos:narrower rdf:resource="descriptor/16051-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71071">
    <skos:narrower rdf:resource="descriptor/16059-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71071">
    <skos:narrower rdf:resource="descriptor/10957-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71071">
    <skos:narrower rdf:resource="descriptor/24662-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71071">
    <skos:narrower rdf:resource="descriptor/18011-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71071">
    <skos:narrower rdf:resource="descriptor/19433-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16205-2">
    <skos:scopeNote xml:lang="de">Interdisziplinärer, nicht genau abgrenzbarer Wissenschaftsbereich mit den Schwerpunkten auf den kommunal relevanten Aspekten der Rechts-, Verwaltungs-, Finanz- und Politikwissenschaften.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18212-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16336-1">
    <skos:prefLabel xml:lang="en">Administrative reform</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16336-1">
    <skos:prefLabel xml:lang="de">Verwaltungsreform</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13804-2">
    <skos:related rdf:resource="descriptor/16068-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13804-2">
    <skos:related rdf:resource="descriptor/11451-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13784-3">
    <skos:prefLabel xml:lang="en">Deposit-taking</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13784-3">
    <skos:prefLabel xml:lang="de">Einlagengeschäft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16176-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14511-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#memberList">
    <rdfs:domain rdf:resource="http://www.w3.org/2004/02/skos/core#OrderedCollection"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13979-2">
    <skos:broader rdf:resource="thsys/71055"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13979-2">
    <skos:broader rdf:resource="thsys/70943"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13979-2">
    <skos:broader rdf:resource="descriptor/18535-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11328-4">
    <skos:prefLabel xml:lang="en">Minimum wage</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11328-4">
    <skos:prefLabel xml:lang="de">Mindestlohn</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15720-5">
    <skos:prefLabel xml:lang="en">Social change</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15720-5">
    <skos:prefLabel xml:lang="de">Sozialer Wandel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16298-4">
    <skos:related rdf:resource="descriptor/16333-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16298-4">
    <skos:related rdf:resource="descriptor/19682-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16298-4">
    <skos:related rdf:resource="descriptor/16250-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16298-4">
    <skos:related rdf:resource="descriptor/16245-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16298-4">
    <skos:related rdf:resource="descriptor/19748-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18918-4">
    <skos:broader rdf:resource="thsys/70829"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18918-4">
    <skos:broader rdf:resource="descriptor/12907-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11544-5">
    <skos:narrower rdf:resource="descriptor/11653-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11554-2">
    <skos:broader rdf:resource="thsys/71081"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11554-2">
    <skos:broader rdf:resource="descriptor/11540-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11554-2">
    <skos:broader rdf:resource="descriptor/12306-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10170-4">
    <skos:altLabel xml:lang="en">Durable consumer goods</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10170-4">
    <skos:altLabel xml:lang="en">Consumer durable</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10170-4">
    <skos:altLabel xml:lang="de">Langlebiges Konsumgut</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13519-1">
    <skos:prefLabel xml:lang="en">Air transport policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13519-1">
    <skos:prefLabel xml:lang="de">Luftverkehrspolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18683-5">
    <skos:altLabel xml:lang="de">Portfolio-Versicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10602-1">
    <skos:related rdf:resource="descriptor/10836-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10602-1">
    <skos:related rdf:resource="descriptor/10823-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10602-1">
    <skos:related rdf:resource="descriptor/18001-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10602-1">
    <skos:related rdf:resource="descriptor/18794-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11271-0">
    <skos:scopeNote xml:lang="de">Schätzgröße des potenziellen Arbeitskräfteangebots.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19487-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12371-5">
    <skos:prefLabel xml:lang="en">Income statement</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12371-5">
    <skos:prefLabel xml:lang="de">Gewinn- und Verlustrechnung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11326-1">
    <skos:scopeNote xml:lang="de">Anteil des Einkommens aus unselbständiger Arbeit am Volkseinkommen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70816">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14127-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11518-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18871-4">
    <skos:broader rdf:resource="thsys/70844"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18871-4">
    <skos:broader rdf:resource="thsys/70175"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18871-4">
    <skos:broader rdf:resource="descriptor/11379-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70935">
    <skos:prefLabel xml:lang="en">W.12.01.03.01  Inland water transport</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70935">
    <skos:prefLabel xml:lang="de">W.12.01.03.01  Binnenschifffahrt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18619-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10950-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10543-5">
    <skos:broader rdf:resource="thsys/71084"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10543-5">
    <skos:broader rdf:resource="descriptor/10555-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10587-6">
    <skos:related rdf:resource="descriptor/10586-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10587-6">
    <skos:related rdf:resource="descriptor/10617-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10587-6">
    <skos:related rdf:resource="descriptor/10555-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11318-0">
    <skos:scopeNote xml:lang="de">Entlohnung des gesamten Personals im öffentlichen Dienst.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13247-1">
    <skos:related rdf:resource="descriptor/13519-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13247-1">
    <skos:related rdf:resource="descriptor/13980-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13247-1">
    <skos:related rdf:resource="descriptor/13246-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13247-1">
    <skos:related rdf:resource="descriptor/16170-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13247-1">
    <skos:related rdf:resource="descriptor/13814-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18602-3">
    <skos:broader rdf:resource="thsys/73389"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18602-3">
    <skos:broader rdf:resource="thsys/70933"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18602-3">
    <skos:broader rdf:resource="descriptor/18591-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18798-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12307-0">
    <skos:prefLabel xml:lang="en">Business accounting</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12307-0">
    <skos:prefLabel xml:lang="de">Betriebliches Rechnungswesen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18745-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70198">
    <skos:prefLabel xml:lang="de">G.00  Geographische Begriffe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10464-1">
    <skos:broader rdf:resource="thsys/73344"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10464-1">
    <skos:broader rdf:resource="thsys/71109"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10464-1">
    <skos:broader rdf:resource="thsys/71005"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10464-1">
    <skos:broader rdf:resource="descriptor/12993-6"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id138">
    <rdf:value xml:lang="de">Für Wohnungsbaufinanzierung</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18345-4">
    <skos:related rdf:resource="descriptor/18800-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18345-4">
    <skos:related rdf:resource="descriptor/19470-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70926">
    <skos:broader rdf:resource="thsys/70928"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18424-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11182-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15949-1">
    <skos:prefLabel xml:lang="en">Mortality</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15949-1">
    <skos:prefLabel xml:lang="de">Sterblichkeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10505-6">
    <skos:broader rdf:resource="thsys/71085"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10505-6">
    <skos:broader rdf:resource="thsys/71028"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10505-6">
    <skos:broader rdf:resource="descriptor/10524-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16802-5">
    <skos:broader rdf:resource="thsys/70822"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16802-5">
    <skos:broader rdf:resource="descriptor/12036-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16802-5">
    <skos:broader rdf:resource="descriptor/15599-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15624-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11718-5">
    <skos:related rdf:resource="descriptor/19210-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11718-5">
    <skos:related rdf:resource="descriptor/11723-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11718-5">
    <skos:related rdf:resource="descriptor/10164-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11718-5">
    <skos:related rdf:resource="descriptor/11722-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11718-5">
    <skos:related rdf:resource="descriptor/13303-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11718-5">
    <skos:related rdf:resource="descriptor/11717-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11718-5">
    <skos:related rdf:resource="descriptor/11054-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11718-5">
    <skos:related rdf:resource="descriptor/11177-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11718-5">
    <skos:related rdf:resource="descriptor/10092-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14849-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id296">
    <rdfs:seeAlso rdf:resource="descriptor/16396-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15640-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10964-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10963-4">
    <skos:broader rdf:resource="thsys/71071"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10963-4">
    <skos:broader rdf:resource="descriptor/10976-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19063-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19373-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12432-4">
    <skos:broader rdf:resource="thsys/71043"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12432-4">
    <skos:broader rdf:resource="descriptor/18118-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12432-4">
    <skos:broader rdf:resource="descriptor/12408-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13350-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13514-4">
    <skos:altLabel xml:lang="de">Wegenetz des Verkehrs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13514-4">
    <skos:altLabel xml:lang="de">Verkehrswegenetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13514-4">
    <skos:altLabel xml:lang="de">Verkehrsnetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13514-4">
    <skos:altLabel xml:lang="de">Transportnetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70817">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15835-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17740-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#changeNote">
    <rdfs:label xml:lang="en">change note</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11774-2">
    <skos:broader rdf:resource="thsys/71115"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19321-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15519-5">
    <skos:altLabel xml:lang="en">Transshipment problem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15519-5">
    <skos:altLabel xml:lang="en">Bottleneck transportation problem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15519-5">
    <skos:altLabel xml:lang="de">Stepping-Stone-Methode</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15519-5">
    <skos:altLabel xml:lang="de">Nord-West-Ecken-Regel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15519-5">
    <skos:altLabel xml:lang="de">Klassisches Transportproblem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14236-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13378-0">
    <skos:related rdf:resource="descriptor/18429-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13378-0">
    <skos:related rdf:resource="descriptor/13959-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13378-0">
    <skos:related rdf:resource="descriptor/12996-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13378-0">
    <skos:related rdf:resource="descriptor/18430-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13378-0">
    <skos:related rdf:resource="descriptor/15638-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10623-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12609-5">
    <skos:prefLabel xml:lang="en">Occupational safety</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12609-5">
    <skos:prefLabel xml:lang="de">Arbeitsschutz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19505-3">
    <skos:altLabel xml:lang="de">Berufsperspektive</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19505-3">
    <skos:altLabel xml:lang="de">Berufsaussichten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19421-2">
    <skos:prefLabel xml:lang="en">Beverage tax</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19421-2">
    <skos:prefLabel xml:lang="de">Getränkesteuer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14980-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73392">
    <skos:broader rdf:resource="thsys/70241"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id271">
    <rdfs:seeAlso rdf:resource="descriptor/15861-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12977-4">
    <skos:narrower rdf:resource="descriptor/18281-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17686-4">
    <skos:prefLabel xml:lang="en">Zimbabwe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17686-4">
    <skos:prefLabel xml:lang="de">Simbabwe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19012-5">
    <skos:related rdf:resource="descriptor/10440-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10395-1">
    <skos:prefLabel xml:lang="en">Policy mix</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10395-1">
    <skos:prefLabel xml:lang="de">Policy-Mix</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15731-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73376">
    <skos:broader rdf:resource="thsys/73382"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14915-2">
    <skos:prefLabel xml:lang="en">Outerwear</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14915-2">
    <skos:prefLabel xml:lang="de">Oberbekleidung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14902-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19069-4">
    <skos:narrower rdf:resource="descriptor/19107-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10401-4">
    <skos:scopeNote xml:lang="de">Formelle und informelle Vereinbarungen zwischen den Tarifparteien, gegebenenfalls unter Hinzuziehung der Regierung, über die zu befolgende Einkommenspolitik. Erweitert auf andere einkommenspolitische Verhandlungen, z.B. im Gesundheitswesen, sowie auf beschäftigungspolitische Übereinkünfte, z.B. Bündnis für Arbeit.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12598-5">
    <skos:related rdf:resource="descriptor/12594-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12598-5">
    <skos:related rdf:resource="descriptor/11320-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19313-5">
    <skos:scopeNote xml:lang="de">Entwicklung des Leistungsbilanzsaldos nach einer Abwertung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18119-6">
    <skos:altLabel xml:lang="de">Abhängiges Unternehmen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18032-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16249-3">
    <skos:prefLabel xml:lang="en">Fascism</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16249-3">
    <skos:prefLabel xml:lang="de">Faschismus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12071-3">
    <skos:prefLabel xml:lang="en">Firm growth</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12071-3">
    <skos:prefLabel xml:lang="de">Unternehmenswachstum</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12917-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12675-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12876-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16014-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13826-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19501-4">
    <skos:altLabel xml:lang="en">Value of future lost earnings</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19501-4">
    <skos:altLabel xml:lang="en">Lost earnings</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19501-4">
    <skos:altLabel xml:lang="en">Future lost earnings</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19501-4">
    <skos:altLabel xml:lang="en">Earnings loss</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19501-4">
    <skos:altLabel xml:lang="de">Entgangenes Einkommen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19501-4">
    <skos:altLabel xml:lang="de">Einkommensausfall</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19754-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10575-6">
    <skos:related rdf:resource="descriptor/10505-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10575-6">
    <skos:related rdf:resource="descriptor/19243-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10575-6">
    <skos:related rdf:resource="descriptor/18865-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10575-6">
    <skos:related rdf:resource="descriptor/10503-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10575-6">
    <skos:related rdf:resource="descriptor/19460-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10575-6">
    <skos:related rdf:resource="descriptor/19463-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10575-6">
    <skos:related rdf:resource="descriptor/19368-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10575-6">
    <skos:related rdf:resource="descriptor/18323-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10575-6">
    <skos:related rdf:resource="descriptor/19467-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18843-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12075-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16198-1">
    <skos:narrower rdf:resource="descriptor/16202-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16198-1">
    <skos:narrower rdf:resource="descriptor/19816-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12921-3">
    <skos:prefLabel xml:lang="en">Furniture industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12921-3">
    <skos:prefLabel xml:lang="de">Möbelindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73383">
    <rdfs:label xml:lang="de">Politische Theorien, politische Philosophien und Strömungen</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13543-4">
    <skos:altLabel xml:lang="de">Küstenschiffahrt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13543-4">
    <skos:altLabel xml:lang="de">Küstennahe Seeschifffahrt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13543-4">
    <skos:altLabel xml:lang="de">Küstennahe Seeschiffahrt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11909-5">
    <skos:broader rdf:resource="thsys/71070"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11909-5">
    <skos:broader rdf:resource="descriptor/10239-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10060-4">
    <skos:altLabel xml:lang="en">Transaction cost economics</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10060-4">
    <skos:altLabel xml:lang="de">Transaktionsökonomik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18852-1">
    <skos:altLabel xml:lang="de">Richtungsgewerkschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18852-1">
    <skos:altLabel xml:lang="de">Gewerkschaftsstruktur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18852-1">
    <skos:altLabel xml:lang="de">Einheitsgewerkschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12275-1">
    <skos:related rdf:resource="descriptor/12403-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12275-1">
    <skos:related rdf:resource="descriptor/11092-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12275-1">
    <skos:related rdf:resource="descriptor/12285-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12275-1">
    <skos:related rdf:resource="descriptor/12446-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12275-1">
    <skos:related rdf:resource="descriptor/11774-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12275-1">
    <skos:related rdf:resource="descriptor/12372-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12275-1">
    <skos:related rdf:resource="descriptor/12371-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12275-1">
    <skos:related rdf:resource="descriptor/18106-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12275-1">
    <skos:related rdf:resource="descriptor/11772-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12275-1">
    <skos:related rdf:resource="descriptor/10192-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19796-1">
    <skos:scopeNote xml:lang="de">für Messgeräte und messtechnische Produkte BENUTZE die entsprechenden Deskriptoren</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19796-1">
    <skos:scopeNote xml:lang="de">Für alle Arten von Maßen und Gewichten.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14350-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16699-0">
    <skos:related rdf:resource="descriptor/19839-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16699-0">
    <skos:related rdf:resource="descriptor/10179-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16699-0">
    <skos:related rdf:resource="descriptor/15925-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16699-0">
    <skos:related rdf:resource="descriptor/15924-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16699-0">
    <skos:related rdf:resource="descriptor/19209-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16699-0">
    <skos:related rdf:resource="descriptor/19376-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16699-0">
    <skos:related rdf:resource="descriptor/19759-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16699-0">
    <skos:related rdf:resource="descriptor/15904-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10761-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12217-1">
    <skos:related rdf:resource="descriptor/12876-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12217-1">
    <skos:related rdf:resource="descriptor/12297-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12217-1">
    <skos:related rdf:resource="descriptor/12296-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12217-1">
    <skos:related rdf:resource="descriptor/16067-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12217-1">
    <skos:related rdf:resource="descriptor/12381-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12217-1">
    <skos:related rdf:resource="descriptor/12458-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12217-1">
    <skos:related rdf:resource="descriptor/12326-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12217-1">
    <skos:related rdf:resource="descriptor/12408-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12217-1">
    <skos:related rdf:resource="descriptor/19535-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18232-3">
    <skos:related rdf:resource="descriptor/14984-4"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id183">
    <rdf:value xml:lang="de">Für private Endverbraucher/innen als Analyseobjekte volkswirtschaftlicher, wirtschaftspolitischer oder sozialwissenschaftlicher Untersuchungen</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73396">
    <rdfs:label xml:lang="de">Gesellschaftsordnungen</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10631-1">
    <skos:broader rdf:resource="thsys/71100"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10631-1">
    <skos:broader rdf:resource="descriptor/10635-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11716-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13742-5">
    <skos:narrower rdf:resource="descriptor/13744-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13742-5">
    <skos:narrower rdf:resource="descriptor/13724-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13742-5">
    <skos:narrower rdf:resource="descriptor/13729-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12993-6">
    <skos:altLabel xml:lang="en">Manufacturing industry</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12993-6">
    <skos:altLabel xml:lang="de">Verarbeitende Industrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12993-6">
    <skos:altLabel xml:lang="de">Prozessindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16713-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24662-6">
    <skos:prefLabel xml:lang="en">Essential facilities doctrine</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24662-6">
    <skos:prefLabel xml:lang="de">Essential-Facilities-Doktrin</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10000-1">
    <skos:narrower rdf:resource="descriptor/10241-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10000-1">
    <skos:narrower rdf:resource="descriptor/11418-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10000-1">
    <skos:narrower rdf:resource="descriptor/10169-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10944-1">
    <skos:altLabel xml:lang="de">Wirksamer Wettbewerb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10944-1">
    <skos:altLabel xml:lang="de">Arbeitsfähiger Wettbewerb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10944-1">
    <skos:altLabel xml:lang="de">Als-ob-Wettbewerb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13507-1">
    <skos:related rdf:resource="descriptor/13578-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13507-1">
    <skos:related rdf:resource="descriptor/16173-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13507-1">
    <skos:related rdf:resource="descriptor/15900-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13507-1">
    <skos:related rdf:resource="descriptor/15899-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13507-1">
    <skos:related rdf:resource="descriptor/14423-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13507-1">
    <skos:related rdf:resource="descriptor/18419-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13507-1">
    <skos:related rdf:resource="descriptor/13246-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13507-1">
    <skos:related rdf:resource="descriptor/13056-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12788-0">
    <skos:prefLabel xml:lang="en">Discount</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12788-0">
    <skos:prefLabel xml:lang="de">Rabatt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13173-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19059-0">
    <skos:prefLabel xml:lang="en">Motorcycle trade</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19059-0">
    <skos:prefLabel xml:lang="de">Motorradhandel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13971-4">
    <skos:prefLabel xml:lang="en">Advertising professions</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13971-4">
    <skos:prefLabel xml:lang="de">Werbeberufe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13410-2">
    <skos:altLabel xml:lang="de">Gerichtssystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13410-2">
    <skos:altLabel xml:lang="de">Gericht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11840-1">
    <skos:broader rdf:resource="thsys/73392"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11840-1">
    <skos:broader rdf:resource="thsys/70884"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11840-1">
    <skos:broader rdf:resource="descriptor/16039-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70058">
    <rdfs:label xml:lang="de">Regionalwirtschaft und Infrastruktur</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12083-3">
    <skos:broader rdf:resource="thsys/70485"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12083-3">
    <skos:broader rdf:resource="descriptor/12088-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12083-3">
    <skos:broader rdf:resource="descriptor/19163-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16477-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17964-2">
    <skos:broader rdf:resource="thsys/70198"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17964-2">
    <skos:broader rdf:resource="descriptor/17954-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17940-2">
    <skos:altLabel xml:lang="de">Föderierte Staaten von Mikronesien</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10074-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18540-6">
    <skos:altLabel xml:lang="de">Überseebank</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12980-1">
    <skos:related rdf:resource="descriptor/18256-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12980-1">
    <skos:related rdf:resource="descriptor/18388-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12980-1">
    <skos:related rdf:resource="descriptor/14236-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12980-1">
    <skos:related rdf:resource="descriptor/15873-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12980-1">
    <skos:related rdf:resource="descriptor/18383-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12980-1">
    <skos:related rdf:resource="descriptor/18407-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12980-1">
    <skos:related rdf:resource="descriptor/14368-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12980-1">
    <skos:related rdf:resource="descriptor/18390-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12980-1">
    <skos:related rdf:resource="descriptor/18352-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12980-1">
    <skos:related rdf:resource="descriptor/18386-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12980-1">
    <skos:related rdf:resource="descriptor/14192-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12980-1">
    <skos:related rdf:resource="descriptor/18375-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12980-1">
    <skos:related rdf:resource="descriptor/18384-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12980-1">
    <skos:related rdf:resource="descriptor/13881-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19592-3">
    <skos:prefLabel xml:lang="en">Stochastic game</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19592-3">
    <skos:prefLabel xml:lang="de">Stochastisches Spiel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14258-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19638-5">
    <skos:broader rdf:resource="thsys/70142"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19638-5">
    <skos:broader rdf:resource="descriptor/15905-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14222-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15779-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11369-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19739-6">
    <skos:altLabel xml:lang="de">Weltmission</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19739-6">
    <skos:altLabel xml:lang="de">Missionsgesellschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19739-6">
    <skos:altLabel xml:lang="de">Missionierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20002-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70143">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10764-3">
    <skos:broader rdf:resource="thsys/71095"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10764-3">
    <skos:broader rdf:resource="thsys/71094"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10764-3">
    <skos:broader rdf:resource="thsys/70055"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16039-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19635-4">
    <skos:related rdf:resource="descriptor/16563-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19635-4">
    <skos:related rdf:resource="descriptor/16209-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19635-4">
    <skos:related rdf:resource="descriptor/15690-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19635-4">
    <skos:related rdf:resource="descriptor/15644-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18368-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12393-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70169">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11909-5">
    <skos:prefLabel xml:lang="en">Regional economic resources</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11909-5">
    <skos:prefLabel xml:lang="de">Regionale Wirtschaftskraft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10859-6">
    <skos:related rdf:resource="descriptor/10880-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10859-6">
    <skos:related rdf:resource="descriptor/10604-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10859-6">
    <skos:related rdf:resource="descriptor/10852-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10859-6">
    <skos:related rdf:resource="descriptor/10833-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12785-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19485-4">
    <skos:related rdf:resource="descriptor/15889-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19485-4">
    <skos:related rdf:resource="descriptor/18820-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14041-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19629-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71054">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.13.06.02.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10307-3">
    <skos:prefLabel xml:lang="en">Cobb-Douglas production function</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10307-3">
    <skos:prefLabel xml:lang="de">Cobb-Douglas-Produktionsfunktion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12732-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14834-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13656-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10957-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11389-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19299-1">
    <skos:scopeNote xml:lang="de">Kampf um die wirtschaftliche und technologische Vorherrschaft auf den globalen Märkten.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18139-0">
    <skos:related rdf:resource="descriptor/15616-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18139-0">
    <skos:related rdf:resource="descriptor/18661-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73300">
    <rdfs:label xml:lang="de">Getränke</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13874-2">
    <skos:related rdf:resource="descriptor/11276-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13874-2">
    <skos:related rdf:resource="descriptor/18987-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13874-2">
    <skos:related rdf:resource="descriptor/13894-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13874-2">
    <skos:related rdf:resource="descriptor/12961-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16475-1">
    <skos:related rdf:resource="descriptor/16540-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16475-1">
    <skos:related rdf:resource="descriptor/16477-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16475-1">
    <skos:related rdf:resource="descriptor/16697-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16475-1">
    <skos:related rdf:resource="descriptor/15904-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16436-4">
    <skos:prefLabel xml:lang="en">World order</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16436-4">
    <skos:prefLabel xml:lang="de">Weltordnung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id58">
    <rdf:value xml:lang="de">Für strukturelle Branchendarstellungen</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15930-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id131">
    <rdf:value xml:lang="de">Für die ökonomische Bewertung der Hausarbeit</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19146-5">
    <skos:prefLabel xml:lang="en">Brainstorming</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19146-5">
    <skos:prefLabel xml:lang="de">Brainstorming</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16709-2">
    <skos:narrower rdf:resource="descriptor/18130-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16709-2">
    <skos:narrower rdf:resource="descriptor/19733-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16709-2">
    <skos:narrower rdf:resource="descriptor/10166-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16709-2">
    <skos:narrower rdf:resource="descriptor/18116-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16709-2">
    <skos:narrower rdf:resource="descriptor/16544-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16709-2">
    <skos:narrower rdf:resource="descriptor/16430-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16709-2">
    <skos:narrower rdf:resource="descriptor/18830-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16709-2">
    <skos:narrower rdf:resource="descriptor/18832-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16709-2">
    <skos:narrower rdf:resource="descriptor/11181-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16709-2">
    <skos:narrower rdf:resource="descriptor/10150-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11739-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12215-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13339-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14331-0">
    <skos:prefLabel xml:lang="en">Chemical plant equipment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14331-0">
    <skos:prefLabel xml:lang="de">Chemieindustrieausrüstung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14225-6">
    <skos:prefLabel xml:lang="en">Diamond</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14225-6">
    <skos:prefLabel xml:lang="de">Diamant</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14355-0">
    <skos:narrower rdf:resource="descriptor/14364-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14355-0">
    <skos:narrower rdf:resource="descriptor/14359-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14355-0">
    <skos:narrower rdf:resource="descriptor/14357-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14355-0">
    <skos:narrower rdf:resource="descriptor/14353-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10680-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12770-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12272-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19528-5">
    <skos:prefLabel xml:lang="en">Quarantine</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19528-5">
    <skos:prefLabel xml:lang="de">Quarantäne</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18732-4">
    <skos:broader rdf:resource="thsys/70893"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18732-4">
    <skos:broader rdf:resource="descriptor/13715-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70300">
    <skos:narrower rdf:resource="descriptor/15571-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70300">
    <skos:narrower rdf:resource="descriptor/12675-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70300">
    <skos:narrower rdf:resource="descriptor/12380-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70300">
    <skos:narrower rdf:resource="descriptor/19115-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70300">
    <skos:narrower rdf:resource="descriptor/12677-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11369-4">
    <skos:prefLabel xml:lang="en">Adult education</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11369-4">
    <skos:prefLabel xml:lang="de">Erwachsenenbildung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17654-3">
    <skos:altLabel xml:lang="de">Sozialistisches Äthiopien</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17654-3">
    <skos:altLabel xml:lang="de">Demokratische Bundesrepublik Äthiopien</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17654-3">
    <skos:altLabel xml:lang="de">Abessinien</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16187-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10733-0">
    <skos:related rdf:resource="descriptor/10771-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10733-0">
    <skos:related rdf:resource="descriptor/10768-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10733-0">
    <skos:related rdf:resource="descriptor/19353-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12733-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19533-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19312-0">
    <skos:scopeNote xml:lang="de">Verknüpfung industrieökonomischer und handelstheoretischer Modelle zur Erklärung einer protektionistischen Außenhandelspolitik auf oligopolistischen Märkten.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="http://zbw.eu/namespaces/skos-extensions/">
    <ns0:title xmlns:ns0="http://purl.org/dc/elements/1.1/" xml:lang="en">SKOS Extensions used by the German National Library of Economics (ZBW)</ns0:title>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15762-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15657-0">
    <skos:prefLabel xml:lang="en">Literature</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15657-0">
    <skos:prefLabel xml:lang="de">Literatur</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70941">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13372-5">
    <skos:related rdf:resource="descriptor/18907-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13372-5">
    <skos:related rdf:resource="descriptor/14007-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13372-5">
    <skos:related rdf:resource="descriptor/19518-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13372-5">
    <skos:related rdf:resource="descriptor/18903-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70324">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.14.01.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11591-3">
    <skos:prefLabel xml:lang="en">Municipal budget</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11591-3">
    <skos:prefLabel xml:lang="de">Kommunalhaushalt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19095-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13419-5">
    <skos:prefLabel xml:lang="en">Social administration</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13419-5">
    <skos:prefLabel xml:lang="de">Sozialverwaltung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70161">
    <rdfs:label xml:lang="de">Abfall und Sekundärrohstoffe</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18369-4">
    <skos:broader rdf:resource="thsys/70951"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18369-4">
    <skos:broader rdf:resource="thsys/70883"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18369-4">
    <skos:broader rdf:resource="descriptor/12254-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70030">
    <skos:prefLabel xml:lang="en">B.06  Production management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70030">
    <skos:prefLabel xml:lang="de">B.06  Produktionswirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12782-5">
    <skos:narrower rdf:resource="descriptor/18525-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14129-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13516-0">
    <skos:altLabel xml:lang="de">Verkehrslandeplatz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13516-0">
    <skos:altLabel xml:lang="de">Verkehrsflughafen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13516-0">
    <skos:altLabel xml:lang="de">Startbahn</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13516-0">
    <skos:altLabel xml:lang="de">Landebahn</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13516-0">
    <skos:altLabel xml:lang="de">Flugplatz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18565-4">
    <skos:prefLabel xml:lang="en">Motorway and railway catering services</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18565-4">
    <skos:prefLabel xml:lang="de">Verkehrsgastronomie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19453-3">
    <skos:altLabel xml:lang="en">Urban land</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19453-3">
    <skos:altLabel xml:lang="en">Lowry model</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19453-3">
    <skos:altLabel xml:lang="de">Städtischer Boden</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19453-3">
    <skos:altLabel xml:lang="de">Lowry-Modell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19463-0">
    <skos:prefLabel xml:lang="en">Rural infrastructure</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19463-0">
    <skos:prefLabel xml:lang="de">Ländliche Infrastruktur</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11590-5">
    <skos:altLabel xml:lang="de">Städtische Finanzplanung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11590-5">
    <skos:altLabel xml:lang="de">Gemeindefinanzplanung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11590-5">
    <skos:altLabel xml:lang="de">Gemeindefinanzierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19829-5">
    <skos:narrower rdf:resource="descriptor/19399-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19011-0">
    <skos:prefLabel xml:lang="en">Wage differential</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19011-0">
    <skos:prefLabel xml:lang="de">Lohndifferenzierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18030-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71080">
    <skos:prefLabel xml:lang="en">V.09.04  Public Transfer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71080">
    <skos:prefLabel xml:lang="de">V.09.04  Öffentlicher Transfer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12224-4">
    <skos:scopeNote xml:lang="de">Nur für betriebswirtschaftliche Literatur, nicht für volkswirtschaftliche Analysen der Investitionstätigkeit der Betriebe.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13756-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20733-1">
    <skos:broader rdf:resource="thsys/73366"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20733-1">
    <skos:broader rdf:resource="thsys/73332"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20733-1">
    <skos:broader rdf:resource="thsys/70899"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20733-1">
    <skos:broader rdf:resource="descriptor/16184-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71069">
    <skos:prefLabel xml:lang="en">V.11.02  Regional policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71069">
    <skos:prefLabel xml:lang="de">V.11.02  Regionalpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10969-6">
    <skos:broader rdf:resource="thsys/71097"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10969-6">
    <skos:broader rdf:resource="thsys/70179"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10969-6">
    <skos:broader rdf:resource="thsys/70038"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10969-6">
    <skos:broader rdf:resource="descriptor/10619-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10546-6">
    <skos:broader rdf:resource="thsys/70550"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10546-6">
    <skos:broader rdf:resource="thsys/70030"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10546-6">
    <skos:broader rdf:resource="descriptor/12808-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19086-4">
    <skos:altLabel xml:lang="x-other">Bosna i Hercegovina</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19086-4">
    <skos:altLabel xml:lang="de">Republik Bosnien und Herzegowina</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19086-4">
    <skos:altLabel xml:lang="de">Herzegowina</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18402-4">
    <skos:related rdf:resource="descriptor/18401-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18402-4">
    <skos:related rdf:resource="descriptor/13034-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18402-4">
    <skos:related rdf:resource="descriptor/18400-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18402-4">
    <skos:related rdf:resource="descriptor/14382-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18402-4">
    <skos:related rdf:resource="descriptor/14266-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18402-4">
    <skos:related rdf:resource="descriptor/13032-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18402-4">
    <skos:related rdf:resource="descriptor/14378-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18402-4">
    <skos:related rdf:resource="descriptor/15638-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18341-5">
    <skos:prefLabel xml:lang="en">Common Fisheries Policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18341-5">
    <skos:prefLabel xml:lang="de">EU-Fischereipolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19732-6">
    <skos:scopeNote xml:lang="de">In verschiedenen Gebieten verwendeter Ansatz. 1. Denkmethode, die den Vorrang des Allgemeinen, Ganzen gegenüber dem Besonderen, Einzelnen betont. 2. Umfassendes Denken und Wissen. 3. In der Wirtschaftswissenschaft eine von SPANN vertretene soziologische und nationalökonomische Richtung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16684-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19687-6">
    <skos:related rdf:resource="descriptor/19854-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19687-6">
    <skos:related rdf:resource="descriptor/19323-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19687-6">
    <skos:related rdf:resource="descriptor/16486-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19687-6">
    <skos:related rdf:resource="descriptor/19649-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19687-6">
    <skos:related rdf:resource="descriptor/15989-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19525-4">
    <skos:altLabel xml:lang="de">Schwangerschaftsunterbrechung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19525-4">
    <skos:altLabel xml:lang="de">Abtreibung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24740-5">
    <skos:prefLabel xml:lang="en">Hedge fund</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24740-5">
    <skos:prefLabel xml:lang="de">Hedgefonds</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17461-0">
    <skos:scopeNote xml:lang="de">Die Sowjetunion hat Ende 1991 als völkerrechtliches Subjekt aufgehört zu existieren</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19860-4">
    <skos:related rdf:resource="descriptor/16303-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19860-4">
    <skos:related rdf:resource="descriptor/15964-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19860-4">
    <skos:related rdf:resource="descriptor/15659-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19860-4">
    <skos:related rdf:resource="descriptor/16713-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19860-4">
    <skos:related rdf:resource="descriptor/15932-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11421-0">
    <skos:narrower rdf:resource="descriptor/10809-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13660-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12953-4">
    <skos:prefLabel xml:lang="en">Pig production</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12953-4">
    <skos:prefLabel xml:lang="de">Schweinehaltung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18595-2">
    <skos:related rdf:resource="descriptor/18242-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18595-2">
    <skos:related rdf:resource="descriptor/18586-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18595-2">
    <skos:related rdf:resource="descriptor/18241-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13239-0">
    <skos:altLabel xml:lang="de">Kraftstoffeinzelhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10219-0">
    <skos:broader rdf:resource="thsys/71070"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10219-0">
    <skos:broader rdf:resource="thsys/70178"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10219-0">
    <skos:broader rdf:resource="descriptor/11928-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10219-0">
    <skos:broader rdf:resource="descriptor/10217-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70817">
    <skos:broader rdf:resource="thsys/70818"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12987-1">
    <skos:narrower rdf:resource="descriptor/18394-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12254-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19336-0">
    <skos:narrower rdf:resource="descriptor/19354-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13481-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19264-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10534-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10340-5">
    <skos:narrower rdf:resource="descriptor/10346-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14236-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15821-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10164-6">
    <zbwext:useInsteadNote rdf:nodeID="id258"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16679-6">
    <skos:related rdf:resource="descriptor/11260-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16679-6">
    <skos:related rdf:resource="descriptor/19466-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16679-6">
    <skos:related rdf:resource="descriptor/18930-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16679-6">
    <skos:related rdf:resource="descriptor/16680-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16679-6">
    <skos:related rdf:resource="descriptor/18916-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16679-6">
    <skos:related rdf:resource="descriptor/11194-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18426-4">
    <skos:broader rdf:resource="thsys/70996"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18426-4">
    <skos:broader rdf:resource="descriptor/10470-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11637-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10374-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73351">
    <rdfs:label xml:lang="de">Umwelt, Natur und Ökologie</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15069-3">
    <skos:broader rdf:resource="thsys/70814"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15069-3">
    <skos:broader rdf:resource="thsys/70504"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15069-3">
    <skos:broader rdf:resource="thsys/70355"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15069-3">
    <skos:broader rdf:resource="descriptor/10011-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15972-6">
    <skos:related rdf:resource="descriptor/15981-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15972-6">
    <skos:related rdf:resource="descriptor/13295-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15972-6">
    <skos:related rdf:resource="descriptor/19607-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15972-6">
    <skos:related rdf:resource="descriptor/13389-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15972-6">
    <skos:related rdf:resource="descriptor/16301-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10641-5">
    <skos:broader rdf:resource="thsys/71100"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10641-5">
    <skos:broader rdf:resource="descriptor/10629-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10313-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13170-3">
    <skos:prefLabel xml:lang="en">Prefabricated house building</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13170-3">
    <skos:prefLabel xml:lang="de">Fertigbau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73335">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11782-3">
    <skos:editorialNote xml:lang="de">Möglichst spezifischer indexieren.</skos:editorialNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19748-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19241-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13060-3">
    <skos:related rdf:resource="descriptor/14438-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13060-3">
    <skos:related rdf:resource="descriptor/13054-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13060-3">
    <skos:related rdf:resource="descriptor/18409-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13060-3">
    <skos:related rdf:resource="descriptor/13045-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71057">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10161-5">
    <skos:related rdf:resource="descriptor/11111-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10161-5">
    <skos:related rdf:resource="descriptor/10166-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10161-5">
    <skos:related rdf:resource="descriptor/10181-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10161-5">
    <skos:related rdf:resource="descriptor/10177-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10161-5">
    <skos:related rdf:resource="descriptor/18832-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19817-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19545-5">
    <skos:prefLabel xml:lang="en">State milk regulations</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19545-5">
    <skos:prefLabel xml:lang="de">Milchmarktordnung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id169">
    <rdfs:seeAlso rdf:resource="descriptor/18930-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14068-3">
    <skos:prefLabel xml:lang="en">Herbs</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14068-3">
    <skos:prefLabel xml:lang="de">Kräuter</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19536-6">
    <skos:related rdf:resource="descriptor/11527-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19536-6">
    <skos:related rdf:resource="descriptor/11529-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19536-6">
    <skos:related rdf:resource="descriptor/12208-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14301-2">
    <skos:broader rdf:resource="thsys/70134"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14301-2">
    <skos:broader rdf:resource="descriptor/14274-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14301-2">
    <skos:broader rdf:resource="descriptor/14302-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14032-3">
    <skos:related rdf:resource="descriptor/18277-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14032-3">
    <skos:related rdf:resource="descriptor/14033-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14032-3">
    <skos:related rdf:resource="descriptor/13213-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14032-3">
    <skos:related rdf:resource="descriptor/14779-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14032-3">
    <skos:related rdf:resource="descriptor/14766-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14032-3">
    <skos:related rdf:resource="descriptor/14548-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19011-0">
    <skos:broader rdf:resource="thsys/70278"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19011-0">
    <skos:broader rdf:resource="descriptor/11327-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70239">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.12.01.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13600-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12340-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17713-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11903-3">
    <skos:altLabel xml:lang="en">Central place theory</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11903-3">
    <skos:altLabel xml:lang="de">Zentrale-Orte-Theorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11903-3">
    <skos:altLabel xml:lang="de">Zentrale Orte Theorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11903-3">
    <skos:altLabel xml:lang="de">Unterzentrum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11903-3">
    <skos:altLabel xml:lang="de">Oberzentrum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11903-3">
    <skos:altLabel xml:lang="de">Mittelzentrum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11903-3">
    <skos:altLabel xml:lang="de">Kleinzentrum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13888-5">
    <skos:prefLabel xml:lang="en">Chemical workers</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13888-5">
    <skos:prefLabel xml:lang="de">Chemieberufe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12283-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14464-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10923-2">
    <skos:prefLabel xml:lang="en">Cartel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10923-2">
    <skos:prefLabel xml:lang="de">Kartell</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19229-1">
    <skos:broader rdf:resource="thsys/70234"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19229-1">
    <skos:broader rdf:resource="descriptor/10249-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10089-1">
    <zbwext:useInsteadNote rdf:nodeID="id5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16533-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#scopeNote">
    <skos:definition xml:lang="en">A note that helps to clarify the meaning of a concept.</skos:definition>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14724-2">
    <skos:prefLabel xml:lang="en">Contraceptive</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14724-2">
    <skos:prefLabel xml:lang="de">Empfängnisverhütungsmittel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17666-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16411-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12821-0">
    <skos:related rdf:resource="descriptor/19135-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12821-0">
    <skos:related rdf:resource="descriptor/13486-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12821-0">
    <skos:related rdf:resource="descriptor/13186-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12821-0">
    <skos:related rdf:resource="descriptor/18101-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12821-0">
    <skos:related rdf:resource="descriptor/18553-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13392-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14384-0">
    <skos:altLabel xml:lang="de">Lebensmittelmaschine</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14384-0">
    <skos:altLabel xml:lang="de">Getränkemaschine</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14384-0">
    <skos:altLabel xml:lang="de">Genussmittelmaschine</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14384-0">
    <skos:altLabel xml:lang="de">Genußmittelmaschine</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15700-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12446-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15488-4">
    <skos:related rdf:resource="descriptor/15520-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19244-5">
    <skos:altLabel xml:lang="en">Endogenous growth theory</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19244-5">
    <skos:altLabel xml:lang="en">Endogenous growth</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19244-5">
    <skos:altLabel xml:lang="de">Endogenes Wachstum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19244-5">
    <skos:altLabel xml:lang="de">Endogene Wachstumstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18357-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19370-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12844-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18162-5">
    <skos:broader rdf:resource="thsys/70348"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18162-5">
    <skos:broader rdf:resource="descriptor/14236-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10504-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13131-6">
    <skos:prefLabel xml:lang="en">Home textiles industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13131-6">
    <skos:prefLabel xml:lang="de">Heimtextilienindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16628-2">
    <skos:altLabel xml:lang="en">Women's organization</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16628-2">
    <skos:altLabel xml:lang="en">Feminism</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16628-2">
    <skos:altLabel xml:lang="de">Frauenverband</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16628-2">
    <skos:altLabel xml:lang="de">Frauenrechtlerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16628-2">
    <skos:altLabel xml:lang="de">Frauenorganisation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16628-2">
    <skos:altLabel xml:lang="de">Feminismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24666-5">
    <skos:related rdf:resource="descriptor/12280-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24666-5">
    <skos:related rdf:resource="descriptor/12313-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16542-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12759-0">
    <skos:related rdf:resource="descriptor/13474-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12759-0">
    <skos:related rdf:resource="descriptor/11787-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12759-0">
    <skos:related rdf:resource="descriptor/12758-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12759-0">
    <skos:related rdf:resource="descriptor/12212-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12759-0">
    <skos:related rdf:resource="descriptor/10546-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18090-4">
    <skos:altLabel xml:lang="de">Unterwassertunnel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18090-4">
    <skos:altLabel xml:lang="de">Straßentunnel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18090-4">
    <skos:altLabel xml:lang="de">Gebirgstunnel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18090-4">
    <skos:altLabel xml:lang="de">Eisenbahntunnel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11675-4">
    <skos:prefLabel xml:lang="en">Natural resource taxation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11675-4">
    <skos:prefLabel xml:lang="de">Rohstoffsteuer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id259">
    <rdfs:seeAlso rdf:resource="descriptor/10978-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16603-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70051">
    <rdfs:label xml:lang="en">Labour market</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70051">
    <rdfs:label xml:lang="de">Arbeitsmarkt</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18844-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18262-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15340-1">
    <skos:broader rdf:resource="thsys/72186"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15340-1">
    <skos:broader rdf:resource="thsys/71035"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15340-1">
    <skos:broader rdf:resource="descriptor/15258-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70035">
    <skos:prefLabel xml:lang="en">G.01.03.01  Northeast Europe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70035">
    <skos:prefLabel xml:lang="de">G.01.03.01  Nordosteuropa</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11360-1">
    <zbwext:useInsteadNote rdf:nodeID="id47"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18395-3">
    <skos:broader rdf:resource="thsys/70041"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18395-3">
    <skos:broader rdf:resource="descriptor/12766-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18954-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19728-4">
    <skos:altLabel xml:lang="en">Enforcement of law</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19728-4">
    <skos:altLabel xml:lang="de">Vollzug von Rechnungsanforderungen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19728-4">
    <skos:altLabel xml:lang="de">Vollzug von Gesetzen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19728-4">
    <skos:altLabel xml:lang="de">Rechtsvollzug</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19728-4">
    <skos:altLabel xml:lang="de">Normvollzug</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19728-4">
    <skos:altLabel xml:lang="de">Gesetzesvollzug</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19728-4">
    <skos:altLabel xml:lang="de">Durchsetzung von Gesetzen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14088-4">
    <skos:narrower rdf:resource="descriptor/14098-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14088-4">
    <skos:narrower rdf:resource="descriptor/14096-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14088-4">
    <skos:narrower rdf:resource="descriptor/14095-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14088-4">
    <skos:narrower rdf:resource="descriptor/14093-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14088-4">
    <skos:narrower rdf:resource="descriptor/14091-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14088-4">
    <skos:narrower rdf:resource="descriptor/14090-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14088-4">
    <skos:narrower rdf:resource="descriptor/14087-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11673-1">
    <skos:prefLabel xml:lang="en">Petroleum tax</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11673-1">
    <skos:prefLabel xml:lang="de">Mineralölsteuer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19446-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16792-3">
    <skos:related rdf:resource="descriptor/16780-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15842-5">
    <skos:altLabel xml:lang="de">Konfliktmanagement im Betrieb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15842-5">
    <skos:altLabel xml:lang="de">Konflikt im Betrieb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id155">
    <rdf:value xml:lang="de">Für internationale Handelsabkommen</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11419-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14180-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18106-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10346-0">
    <skos:altLabel xml:lang="de">Lokomotiventheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10346-0">
    <skos:altLabel xml:lang="de">Konjunkturübertragung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10346-0">
    <skos:altLabel xml:lang="de">Internationaler Konjunkturzusammenhang</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10346-0">
    <skos:altLabel xml:lang="de">Internationaler Konjunkturverbund</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10346-0">
    <skos:altLabel xml:lang="de">Internationale Konjunkturübertragung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14581-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10029-5">
    <skos:narrower rdf:resource="descriptor/10904-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19542-4">
    <skos:altLabel xml:lang="en">Conjoint measurement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19542-4">
    <skos:altLabel xml:lang="de">Verbundmessung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19542-4">
    <skos:altLabel xml:lang="de">Conjoint Analyse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19803-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18905-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13530-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10030-6">
    <skos:broader rdf:resource="thsys/70175"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18582-4">
    <skos:prefLabel xml:lang="en">Trucking industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18582-4">
    <skos:prefLabel xml:lang="de">Güterkraftverkehrsgewerbe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10324-3">
    <skos:prefLabel xml:lang="en">Multistage production</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10324-3">
    <skos:prefLabel xml:lang="de">Mehrstufige Produktion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71105">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11934-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16031-2">
    <skos:broader rdf:resource="thsys/73391"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16031-2">
    <skos:broader rdf:resource="thsys/70854"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16031-2">
    <skos:broader rdf:resource="descriptor/16042-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13967-2">
    <skos:narrower rdf:resource="descriptor/12025-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13967-2">
    <skos:narrower rdf:resource="descriptor/13716-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#narrowerTransitive">
    <rdfs:comment xml:lang="en">By convention, skos:narrowerTransitive is not used to make
      assertions. Rather, the properties can be used to draw inferences about the transitive closure
      of the hierarchical relation, which is useful e.g. when implementing a simple query expansion
      algorithm in a search application.</rdfs:comment>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19124-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15609-4">
    <skos:scopeNote xml:lang="de">Nur für die Wissenschaft, die sich mit DV-Anlagen und den Grundlagen ihrer Anwendung beschäftigt.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10028-0">
    <skos:broader rdf:resource="thsys/71111"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10028-0">
    <skos:broader rdf:resource="descriptor/10272-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16252-0">
    <skos:altLabel xml:lang="de">Offizierinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16252-0">
    <skos:altLabel xml:lang="de">Offiziere</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16252-0">
    <skos:altLabel xml:lang="de">Konteradmirale</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16252-0">
    <skos:altLabel xml:lang="de">Armeeführung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14345-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71118">
    <skos:prefLabel xml:lang="en">V.02.05  Utility theory</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71118">
    <skos:prefLabel xml:lang="de">V.02.05  Nutzentheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71081">
    <skos:prefLabel xml:lang="en">V.09.03  Public Budget</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71081">
    <skos:prefLabel xml:lang="de">V.09.03  Öffentlicher Haushalt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12053-5">
    <skos:narrower rdf:resource="descriptor/12318-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12053-5">
    <skos:narrower rdf:resource="descriptor/12295-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12053-5">
    <skos:narrower rdf:resource="descriptor/12218-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12053-5">
    <skos:narrower rdf:resource="descriptor/12308-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12053-5">
    <skos:narrower rdf:resource="descriptor/12228-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19375-4">
    <skos:broader rdf:resource="thsys/71076"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19375-4">
    <skos:broader rdf:resource="descriptor/11599-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18732-4">
    <skos:prefLabel xml:lang="en">Issuing price</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18732-4">
    <skos:prefLabel xml:lang="de">Emissionskurs</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15902-6">
    <skos:related rdf:resource="descriptor/11916-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15902-6">
    <skos:related rdf:resource="descriptor/19611-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15902-6">
    <skos:related rdf:resource="descriptor/19438-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17951-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11042-1">
    <skos:prefLabel xml:lang="en">Monetary economy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11042-1">
    <skos:prefLabel xml:lang="de">Geldwirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18339-6">
    <skos:broader rdf:resource="thsys/73348"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18339-6">
    <skos:broader rdf:resource="thsys/71019"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18339-6">
    <skos:broader rdf:resource="descriptor/15685-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71098">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.07.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11646-4">
    <skos:broader rdf:resource="thsys/73295"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11646-4">
    <skos:broader rdf:resource="thsys/71074"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11646-4">
    <skos:broader rdf:resource="descriptor/19422-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11646-4">
    <skos:broader rdf:resource="descriptor/18992-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17938-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14741-2">
    <skos:narrower rdf:resource="descriptor/14739-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14741-2">
    <skos:narrower rdf:resource="descriptor/14622-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71087">
    <skos:broader rdf:resource="thsys/70179"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19085-6">
    <skos:prefLabel xml:lang="en">CEFTA countries</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19085-6">
    <skos:prefLabel xml:lang="de">Visegrad-Staaten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12165-1">
    <skos:narrower rdf:resource="descriptor/18092-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12165-1">
    <skos:narrower rdf:resource="descriptor/18664-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12165-1">
    <skos:narrower rdf:resource="descriptor/12564-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12165-1">
    <skos:narrower rdf:resource="descriptor/12768-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12165-1">
    <skos:narrower rdf:resource="descriptor/12079-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12165-1">
    <skos:narrower rdf:resource="descriptor/19114-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12165-1">
    <skos:narrower rdf:resource="descriptor/12473-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12165-1">
    <skos:narrower rdf:resource="descriptor/12166-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12165-1">
    <skos:narrower rdf:resource="descriptor/19833-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12165-1">
    <skos:narrower rdf:resource="descriptor/19104-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12165-1">
    <skos:narrower rdf:resource="descriptor/12883-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12165-1">
    <skos:narrower rdf:resource="descriptor/19115-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12165-1">
    <skos:narrower rdf:resource="descriptor/12330-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12165-1">
    <skos:narrower rdf:resource="descriptor/13671-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12165-1">
    <skos:narrower rdf:resource="descriptor/12730-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15446-6">
    <skos:altLabel xml:lang="de">Nichtkompetitives Spiel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15446-6">
    <skos:altLabel xml:lang="de">Kooperationsspiel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15054-2">
    <skos:broader rdf:resource="thsys/70269"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15054-2">
    <skos:broader rdf:resource="descriptor/15045-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71081">
    <skos:narrower rdf:resource="descriptor/10710-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71081">
    <skos:narrower rdf:resource="descriptor/11598-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71081">
    <skos:narrower rdf:resource="descriptor/11724-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71081">
    <skos:narrower rdf:resource="descriptor/11560-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71081">
    <skos:narrower rdf:resource="descriptor/15760-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71081">
    <skos:narrower rdf:resource="descriptor/19436-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71081">
    <skos:narrower rdf:resource="descriptor/11547-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71081">
    <skos:narrower rdf:resource="descriptor/11589-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71081">
    <skos:narrower rdf:resource="descriptor/10267-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71081">
    <skos:narrower rdf:resource="descriptor/11596-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71081">
    <skos:narrower rdf:resource="descriptor/11558-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71081">
    <skos:narrower rdf:resource="descriptor/11556-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71081">
    <skos:narrower rdf:resource="descriptor/11595-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71081">
    <skos:narrower rdf:resource="descriptor/11545-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71081">
    <skos:narrower rdf:resource="descriptor/11409-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71081">
    <skos:narrower rdf:resource="descriptor/11554-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71081">
    <skos:narrower rdf:resource="descriptor/11592-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71081">
    <skos:narrower rdf:resource="descriptor/19381-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71081">
    <skos:narrower rdf:resource="descriptor/11591-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71081">
    <skos:narrower rdf:resource="descriptor/11590-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71081">
    <skos:narrower rdf:resource="descriptor/11544-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71081">
    <skos:narrower rdf:resource="descriptor/11553-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71081">
    <skos:narrower rdf:resource="descriptor/11587-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71081">
    <skos:narrower rdf:resource="descriptor/11731-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71081">
    <skos:narrower rdf:resource="descriptor/11586-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71081">
    <skos:narrower rdf:resource="descriptor/11543-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71081">
    <skos:narrower rdf:resource="descriptor/19380-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71081">
    <skos:narrower rdf:resource="descriptor/11713-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71081">
    <skos:narrower rdf:resource="descriptor/11318-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71081">
    <skos:narrower rdf:resource="descriptor/18957-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13971-4">
    <skos:related rdf:resource="descriptor/12845-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13971-4">
    <skos:related rdf:resource="descriptor/18977-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13971-4">
    <skos:related rdf:resource="descriptor/18788-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19812-1">
    <skos:scopeNote xml:lang="de">Regelungen des Arbeitsrechts, die von zwei oder mehr Staaten in Form einer Vereinbarung oder eines Abkommens beschlossen worden sind. Regelungen des Internationalen Privatrechts über Arbeitsverträge.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11751-0">
    <skos:broader rdf:resource="thsys/71114"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12001-3">
    <skos:related rdf:resource="descriptor/15073-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12001-3">
    <skos:related rdf:resource="descriptor/12004-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10107-4">
    <skos:related rdf:resource="descriptor/16440-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10107-4">
    <skos:related rdf:resource="descriptor/11167-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10107-4">
    <skos:related rdf:resource="descriptor/10089-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10107-4">
    <skos:related rdf:resource="descriptor/19692-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19107-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14154-3">
    <skos:prefLabel xml:lang="en">Coniferous wood</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14154-3">
    <skos:prefLabel xml:lang="de">Nadelholz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19153-1">
    <skos:prefLabel xml:lang="en">Pretiale Lenkung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19153-1">
    <skos:prefLabel xml:lang="de">Pretiale Lenkung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11806-1">
    <skos:broader rdf:resource="thsys/73362"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11806-1">
    <skos:broader rdf:resource="thsys/71020"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11806-1">
    <skos:broader rdf:resource="descriptor/18332-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11806-1">
    <skos:broader rdf:resource="descriptor/11800-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11436-1">
    <skos:related rdf:resource="descriptor/11434-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11436-1">
    <skos:related rdf:resource="descriptor/19255-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11436-1">
    <skos:related rdf:resource="descriptor/11411-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16410-1">
    <skos:altLabel xml:lang="de">Traditionalismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16410-1">
    <skos:altLabel xml:lang="de">Neokonservatismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16410-1">
    <skos:altLabel xml:lang="de">Konservativismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18655-3">
    <skos:prefLabel xml:lang="en">Value added services</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18655-3">
    <skos:prefLabel xml:lang="de">Mehrwertdienst</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14015-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16680-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11164-1">
    <skos:broader rdf:resource="thsys/73395"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11164-1">
    <skos:broader rdf:resource="thsys/73386"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11164-1">
    <skos:broader rdf:resource="thsys/73382"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11164-1">
    <skos:broader rdf:resource="thsys/70274"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11164-1">
    <skos:broader rdf:resource="thsys/70140"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18799-0">
    <skos:broader rdf:resource="thsys/71067"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18799-0">
    <skos:broader rdf:resource="thsys/70873"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18799-0">
    <skos:broader rdf:resource="descriptor/11913-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21094-4">
    <skos:scopeNote xml:lang="de">Betrifft vertragliche Regelungen zwischen Erzeuger und Abnehmer landwirtschaftlicher Produkte, die dem Erzeuger einen garantierten Absatz und dem Abnehmer garantierte Mengen und Qualität sichern.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12228-3">
    <skos:broader rdf:resource="thsys/71042"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12228-3">
    <skos:broader rdf:resource="thsys/70037"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12228-3">
    <skos:broader rdf:resource="descriptor/12053-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17964-2">
    <skos:altLabel xml:lang="de">EU-Meer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17964-2">
    <skos:altLabel xml:lang="de">EG-Meer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16437-2">
    <skos:related rdf:resource="descriptor/16463-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16437-2">
    <skos:related rdf:resource="descriptor/16278-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16437-2">
    <skos:related rdf:resource="descriptor/18816-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16437-2">
    <skos:related rdf:resource="descriptor/19723-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16437-2">
    <skos:related rdf:resource="descriptor/13412-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16437-2">
    <skos:related rdf:resource="descriptor/19681-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19516-5">
    <skos:altLabel xml:lang="en">Returns to schooling</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13819-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12027-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16184-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71117">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72185">
    <skos:broader rdf:resource="thsys/72180"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15811-2">
    <skos:prefLabel xml:lang="en">Social psychology</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15811-2">
    <skos:prefLabel xml:lang="de">Sozialpsychologie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18381-0">
    <skos:prefLabel xml:lang="en">Resource economics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18381-0">
    <skos:prefLabel xml:lang="de">Ressourcenökonomik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14384-0">
    <skos:related rdf:resource="descriptor/18413-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14384-0">
    <skos:related rdf:resource="descriptor/13160-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14384-0">
    <skos:related rdf:resource="descriptor/14931-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14384-0">
    <skos:related rdf:resource="descriptor/18443-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14384-0">
    <skos:related rdf:resource="descriptor/18562-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18908-0">
    <skos:related rdf:resource="descriptor/16485-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18908-0">
    <skos:related rdf:resource="descriptor/13401-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18908-0">
    <skos:related rdf:resource="descriptor/25621-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18908-0">
    <skos:related rdf:resource="descriptor/18975-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17598-1">
    <skos:altLabel xml:lang="de">Königreich Bhutan</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12110-5">
    <skos:prefLabel xml:lang="en">Sociology of organizations</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12110-5">
    <skos:prefLabel xml:lang="de">Organisationssoziologie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70038">
    <rdfs:label xml:lang="de">Allgemeine Wirtschaftspolitik</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18273-3">
    <skos:prefLabel xml:lang="en">Reusable packaging</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18273-3">
    <skos:prefLabel xml:lang="de">Mehrwegverpackung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19369-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#hiddenLabel">
    <rdfs:subPropertyOf rdf:resource="http://www.w3.org/2000/01/rdf-schema#label"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19027-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70924">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#changeNote">
    <rdf:type rdf:resource="http://www.w3.org/1999/02/22-rdf-syntax-ns#Property"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#changeNote">
    <rdf:type rdf:resource="http://www.w3.org/2002/07/owl#AnnotationProperty"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15658-5">
    <skos:broader rdf:resource="thsys/73347"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15658-5">
    <skos:broader rdf:resource="thsys/70839"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15658-5">
    <skos:broader rdf:resource="thsys/70573"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15658-5">
    <skos:broader rdf:resource="descriptor/15685-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12046-2">
    <skos:related rdf:resource="descriptor/12069-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12046-2">
    <skos:related rdf:resource="descriptor/19519-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14926-4">
    <skos:altLabel xml:lang="de">Schlankheitskost</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14926-4">
    <skos:altLabel xml:lang="de">Reformkost</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14926-4">
    <skos:altLabel xml:lang="de">Light-Nahrungsmittel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14926-4">
    <skos:altLabel xml:lang="de">Kalorienreduziertes Nahrungsmittel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14926-4">
    <skos:altLabel xml:lang="de">Diätkost</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18216-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15051-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70823">
    <skos:prefLabel xml:lang="en">W.30  Information industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70823">
    <skos:prefLabel xml:lang="de">W.30  Informationswirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12758-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16416-3">
    <skos:altLabel xml:lang="de">Chauvinismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15225-1">
    <skos:broader rdf:resource="thsys/72185"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15225-1">
    <skos:broader rdf:resource="thsys/71035"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15225-1">
    <skos:broader rdf:resource="descriptor/15060-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14642-4">
    <skos:prefLabel xml:lang="en">Inorganic chemical</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14642-4">
    <skos:prefLabel xml:lang="de">Anorganische Chemikalie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10617-2">
    <skos:broader rdf:resource="thsys/73366"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10617-2">
    <skos:broader rdf:resource="thsys/70179"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10617-2">
    <skos:broader rdf:resource="descriptor/16441-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16460-0">
    <skos:altLabel xml:lang="de">Rüstungswettlauf</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16460-0">
    <skos:altLabel xml:lang="de">Rüstung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16460-0">
    <skos:altLabel xml:lang="de">Militärische Abschreckung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16460-0">
    <skos:altLabel xml:lang="de">Aufrüstung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18239-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11894-6">
    <skos:related rdf:resource="descriptor/11895-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11894-6">
    <skos:related rdf:resource="descriptor/11888-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13214-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19280-1">
    <skos:related rdf:resource="descriptor/19271-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70951">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11419-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19392-4">
    <skos:altLabel xml:lang="en">Public school (GB)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14488-2">
    <skos:broader rdf:resource="thsys/70135"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14488-2">
    <skos:broader rdf:resource="descriptor/14473-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11393-0">
    <skos:prefLabel xml:lang="en">Prevocational training</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11393-0">
    <skos:prefLabel xml:lang="de">Berufsvorbereitung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70049">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14323-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12618-4">
    <skos:prefLabel xml:lang="en">Occupational disease</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12618-4">
    <skos:prefLabel xml:lang="de">Berufskrankheit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10065-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18869-5">
    <skos:related rdf:resource="descriptor/19766-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18869-5">
    <skos:related rdf:resource="descriptor/19253-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18869-5">
    <skos:related rdf:resource="descriptor/10142-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18869-5">
    <skos:related rdf:resource="descriptor/18775-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13513-6">
    <skos:altLabel xml:lang="de">Transportmittel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13513-6">
    <skos:altLabel xml:lang="de">Reisemittel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13513-6">
    <skos:altLabel xml:lang="de">Fahrzeug</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13513-6">
    <skos:altLabel xml:lang="de">Beförderungsmittel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13324-2">
    <skos:broader rdf:resource="thsys/70816"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13324-2">
    <skos:broader rdf:resource="thsys/70239"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13324-2">
    <skos:broader rdf:resource="descriptor/13325-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11390-6">
    <skos:prefLabel xml:lang="en">Retraining</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11390-6">
    <skos:prefLabel xml:lang="de">Umschulung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16967-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20368-5">
    <skos:altLabel xml:lang="en">ESO (Employee stock options)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20368-5">
    <skos:altLabel xml:lang="en">Employee stock options</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20368-5">
    <skos:altLabel xml:lang="de">Mitarbeiter-Aktienoption</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20368-5">
    <skos:altLabel xml:lang="de">Aktienoptionsprogramm</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13045-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70257">
    <rdfs:label xml:lang="de">Fahrzeuge</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13297-0">
    <skos:altLabel xml:lang="en">Property and liability insurance</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13297-0">
    <skos:altLabel xml:lang="de">Schadensversicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13297-0">
    <skos:altLabel xml:lang="de">Schaden- und Unfallversicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15571-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11286-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12577-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11824-6">
    <skos:prefLabel xml:lang="en">Trade structure</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11824-6">
    <skos:prefLabel xml:lang="de">Handelsstruktur</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#definition">
    <rdfs:subPropertyOf rdf:resource="http://www.w3.org/2004/02/skos/core#note"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71041">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70815">
    <skos:prefLabel xml:lang="en">W.33  Environmental goods and services industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70815">
    <skos:prefLabel xml:lang="de">W.33  Umweltschutzsektor</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13499-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18397-6">
    <skos:broader rdf:resource="thsys/70876"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18397-6">
    <skos:broader rdf:resource="descriptor/13301-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10905-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12016-4">
    <skos:prefLabel xml:lang="en">Soil degradation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12016-4">
    <skos:prefLabel xml:lang="de">Bodenbelastung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70835">
    <skos:narrower rdf:resource="descriptor/18905-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70835">
    <skos:narrower rdf:resource="descriptor/18887-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70835">
    <skos:narrower rdf:resource="descriptor/16159-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70835">
    <skos:narrower rdf:resource="descriptor/15674-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70835">
    <skos:narrower rdf:resource="descriptor/16158-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70835">
    <skos:narrower rdf:resource="descriptor/15723-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70835">
    <skos:narrower rdf:resource="descriptor/16544-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70835">
    <skos:narrower rdf:resource="descriptor/16155-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70835">
    <skos:narrower rdf:resource="descriptor/18904-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70835">
    <skos:narrower rdf:resource="descriptor/16401-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70835">
    <skos:narrower rdf:resource="descriptor/11037-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70835">
    <skos:narrower rdf:resource="descriptor/16124-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70835">
    <skos:narrower rdf:resource="thsys/70832"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70835">
    <skos:narrower rdf:resource="thsys/70834"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70835">
    <skos:narrower rdf:resource="thsys/70833"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12075-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19191-0">
    <skos:prefLabel xml:lang="en">Farm economics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19191-0">
    <skos:prefLabel xml:lang="de">Landwirtschaftliche Betriebslehre</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71028">
    <skos:broader rdf:resource="thsys/w"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id123">
    <rdfs:seeAlso rdf:resource="descriptor/16013-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10223-2">
    <skos:narrower rdf:resource="descriptor/19226-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11067-6">
    <skos:narrower rdf:resource="descriptor/11310-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11067-6">
    <skos:narrower rdf:resource="descriptor/11036-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16258-2">
    <skos:related rdf:resource="descriptor/16013-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16258-2">
    <skos:related rdf:resource="descriptor/15974-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16258-2">
    <skos:related rdf:resource="descriptor/19607-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16258-2">
    <skos:related rdf:resource="descriptor/16292-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16258-2">
    <skos:related rdf:resource="descriptor/19753-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16723-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12330-5">
    <skos:altLabel xml:lang="de">Speicherbuchführung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12330-5">
    <skos:altLabel xml:lang="de">Rechnergestützte Buchführung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12330-5">
    <skos:altLabel xml:lang="de">EDV-gestützte Buchführung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12330-5">
    <skos:altLabel xml:lang="de">Automatisierte Buchhaltung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15478-0">
    <skos:prefLabel xml:lang="en">Risk-loving</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15478-0">
    <skos:prefLabel xml:lang="de">Risikofreude</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11356-6">
    <skos:altLabel xml:lang="en">University graduates</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11356-6">
    <skos:altLabel xml:lang="en">Graduates (brit.)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11356-6">
    <skos:altLabel xml:lang="en">College graduates</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11356-6">
    <skos:altLabel xml:lang="en">Academic staff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11356-6">
    <skos:altLabel xml:lang="de">Universitätsabsolventen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11356-6">
    <skos:altLabel xml:lang="de">Hochschulabsolventinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11356-6">
    <skos:altLabel xml:lang="de">Hochschulabsolventen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11356-6">
    <skos:altLabel xml:lang="de">Akademikerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11356-6">
    <skos:altLabel xml:lang="de">Akademiker</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10427-0">
    <skos:related rdf:resource="descriptor/10424-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15023-6">
    <skos:prefLabel xml:lang="en">Cohort analysis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15023-6">
    <skos:prefLabel xml:lang="de">Kohortenanalyse</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18505-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14091-1">
    <skos:broader rdf:resource="thsys/70070"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14091-1">
    <skos:broader rdf:resource="descriptor/14088-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13232-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18643-3">
    <skos:scopeNote xml:lang="de">Auch für die sendetechnischen Regelungen von Rundfunk und Fernsehen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12720-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16258-2">
    <skos:scopeNote xml:lang="de">Rechtfertigung des Staates, der Rechtsordnung und des staatlichen Handelns aus höheren Grundsätzen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18352-0">
    <skos:related rdf:resource="descriptor/16187-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18352-0">
    <skos:related rdf:resource="descriptor/15993-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18352-0">
    <skos:related rdf:resource="descriptor/18375-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18352-0">
    <skos:related rdf:resource="descriptor/12980-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19595-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18553-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18349-3">
    <skos:altLabel xml:lang="de">Betriebliches Energiemanagement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18349-3">
    <skos:altLabel xml:lang="de">Betriebliche Energiewirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18126-2">
    <skos:prefLabel xml:lang="en">Trade in services</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18126-2">
    <skos:prefLabel xml:lang="de">Internationaler Dienstleistungsverkehr</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11974-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70957">
    <skos:broader rdf:resource="thsys/70962"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19091-4">
    <skos:narrower rdf:resource="descriptor/12433-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10675-2">
    <skos:related rdf:resource="descriptor/10677-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10675-2">
    <skos:related rdf:resource="descriptor/10659-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10675-2">
    <skos:related rdf:resource="descriptor/10527-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10675-2">
    <skos:related rdf:resource="descriptor/10834-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10216-6">
    <skos:broader rdf:resource="thsys/70178"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10216-6">
    <skos:broader rdf:resource="thsys/70101"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10216-6">
    <skos:broader rdf:resource="descriptor/10217-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14093-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13051-4">
    <skos:altLabel xml:lang="en">Motor vehicle industry</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13051-4">
    <skos:altLabel xml:lang="en">Car manufacturer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13051-4">
    <skos:altLabel xml:lang="en">Automotive industry</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13051-4">
    <skos:altLabel xml:lang="de">Pkw-Bau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13051-4">
    <skos:altLabel xml:lang="de">Montage von Kraftfahrzeugen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13051-4">
    <skos:altLabel xml:lang="de">Kraftfahrzeugmontage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13051-4">
    <skos:altLabel xml:lang="de">Kraftfahrzeugbau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13051-4">
    <skos:altLabel xml:lang="de">Kfz-Industrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13051-4">
    <skos:altLabel xml:lang="de">Kfz-Bau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13051-4">
    <skos:altLabel xml:lang="de">Automobilindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13051-4">
    <skos:altLabel xml:lang="de">Autoindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14608-4">
    <skos:altLabel xml:lang="de">Ruß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14608-4">
    <skos:altLabel xml:lang="de">Graphit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14608-4">
    <skos:altLabel xml:lang="de">Grafit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19449-1">
    <skos:prefLabel xml:lang="en">Ecological Regional Planning</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19449-1">
    <skos:prefLabel xml:lang="de">Ökologische Raumplanung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18436-1">
    <skos:altLabel xml:lang="de">Textilfachgeschäft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18436-1">
    <skos:altLabel xml:lang="de">Bekleidungsfachgeschäft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18436-1">
    <skos:altLabel xml:lang="de">Bekleidungseinzelhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11709-6">
    <skos:prefLabel xml:lang="en">Tax avoidance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11709-6">
    <skos:prefLabel xml:lang="de">Steuervermeidung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18963-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16984-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13295-4">
    <skos:related rdf:resource="descriptor/15972-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18848-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12644-3">
    <skos:broader rdf:resource="thsys/70394"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12644-3">
    <skos:broader rdf:resource="descriptor/12670-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14446-4">
    <skos:altLabel xml:lang="de">Sportboot</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14446-4">
    <skos:altLabel xml:lang="de">Segelschiff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14446-4">
    <skos:altLabel xml:lang="de">Segelboot</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14446-4">
    <skos:altLabel xml:lang="de">Motorboot</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14446-4">
    <skos:altLabel xml:lang="de">Jacht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13635-6">
    <skos:prefLabel xml:lang="en">Rural tourism</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13635-6">
    <skos:prefLabel xml:lang="de">Agrotourismus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14954-6">
    <skos:altLabel xml:lang="de">Quark</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14954-6">
    <skos:altLabel xml:lang="de">Frischkäse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10173-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10617-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12621-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19141-1">
    <skos:prefLabel xml:lang="en">Customers</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19141-1">
    <skos:prefLabel xml:lang="de">Kunden</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19693-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19447-5">
    <skos:prefLabel xml:lang="en">Regional economic transformation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19447-5">
    <skos:prefLabel xml:lang="de">Regionale Erneuerung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73304">
    <rdfs:label xml:lang="de">Naturfasern</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13083-5">
    <skos:related rdf:resource="descriptor/14543-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13083-5">
    <skos:related rdf:resource="descriptor/13095-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13083-5">
    <skos:related rdf:resource="descriptor/13231-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13083-5">
    <skos:related rdf:resource="descriptor/13079-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13679-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16397-2">
    <skos:related rdf:resource="descriptor/16404-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16397-2">
    <skos:related rdf:resource="descriptor/16286-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16397-2">
    <skos:related rdf:resource="descriptor/16395-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13110-0">
    <skos:broader rdf:resource="thsys/70991"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13110-0">
    <skos:broader rdf:resource="descriptor/12912-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13399-6">
    <skos:broader rdf:resource="thsys/73388"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13399-6">
    <skos:broader rdf:resource="thsys/70850"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13399-6">
    <skos:broader rdf:resource="descriptor/10984-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13399-6">
    <skos:broader rdf:resource="descriptor/16587-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19517-3">
    <skos:narrower rdf:resource="descriptor/18903-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19613-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14725-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12469-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19693-4">
    <skos:related rdf:resource="descriptor/13984-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10114-0">
    <skos:related rdf:resource="descriptor/10328-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10114-0">
    <skos:related rdf:resource="descriptor/10460-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10114-0">
    <skos:related rdf:resource="descriptor/19208-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19684-5">
    <skos:narrower rdf:resource="descriptor/11853-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19303-1">
    <skos:prefLabel xml:lang="en">Wheat market</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19303-1">
    <skos:prefLabel xml:lang="de">Weizenmarkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19711-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19832-2">
    <skos:related rdf:resource="descriptor/19155-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19832-2">
    <skos:related rdf:resource="descriptor/12154-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73388">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18453-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11285-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10443-2">
    <skos:broader rdf:resource="thsys/71109"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10443-2">
    <skos:broader rdf:resource="thsys/70878"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10443-2">
    <skos:broader rdf:resource="thsys/70550"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10443-2">
    <skos:broader rdf:resource="descriptor/10436-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13157-2">
    <skos:narrower rdf:resource="descriptor/13152-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13157-2">
    <skos:narrower rdf:resource="descriptor/18221-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13157-2">
    <skos:narrower rdf:resource="descriptor/13151-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16550-6">
    <skos:prefLabel xml:lang="en">Local government</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16550-6">
    <skos:prefLabel xml:lang="de">Kommunalverwaltung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18635-2">
    <skos:broader rdf:resource="thsys/70924"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18635-2">
    <skos:broader rdf:resource="descriptor/11850-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12846-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71106">
    <skos:broader rdf:resource="thsys/71107"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16202-1">
    <skos:altLabel xml:lang="de">Wirtschaftsstrafgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14229-5">
    <skos:related rdf:resource="descriptor/19050-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16140-4">
    <skos:prefLabel xml:lang="en">Protection of minors at work</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16140-4">
    <skos:prefLabel xml:lang="de">Jugendarbeitsschutz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14184-1">
    <skos:related rdf:resource="descriptor/11813-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19796-1">
    <skos:altLabel xml:lang="en">Weights and measures</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19796-1">
    <skos:altLabel xml:lang="de">Metrisches System</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19796-1">
    <skos:altLabel xml:lang="de">Messeinheit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19796-1">
    <skos:altLabel xml:lang="de">Meßeinheit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19796-1">
    <skos:altLabel xml:lang="de">Mengenmaß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19796-1">
    <skos:altLabel xml:lang="de">Maße und Gewichte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19796-1">
    <skos:altLabel xml:lang="de">Maß und Gewicht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19796-1">
    <skos:altLabel xml:lang="de">Längenmaß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19796-1">
    <skos:altLabel xml:lang="de">Gewicht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19796-1">
    <skos:altLabel xml:lang="de">Flächenmaß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19796-1">
    <skos:altLabel xml:lang="de">Eichwesen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19796-1">
    <skos:altLabel xml:lang="de">Dezimalsystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70212">
    <skos:broader rdf:resource="thsys/70101"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12749-3">
    <skos:broader rdf:resource="thsys/70029"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12749-3">
    <skos:broader rdf:resource="descriptor/19074-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70085">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14554-1">
    <skos:prefLabel xml:lang="en">Cooker</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14554-1">
    <skos:prefLabel xml:lang="de">Herd</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11261-3">
    <skos:altLabel xml:lang="de">Altersarbeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11261-3">
    <skos:altLabel xml:lang="de">Alternde Belegschaften</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11261-3">
    <skos:altLabel xml:lang="de">Ältere Arbeitnehmerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11261-3">
    <skos:altLabel xml:lang="de">Ältere Arbeitnehmer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14193-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12341-0">
    <skos:scopeNote xml:lang="de">Nur für Konten in der Buchführung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14048-2">
    <skos:prefLabel xml:lang="en">Writing instrument</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14048-2">
    <skos:prefLabel xml:lang="de">Schreibgerät</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24855-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11578-2">
    <skos:related rdf:resource="descriptor/13179-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11578-2">
    <skos:related rdf:resource="descriptor/11580-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11578-2">
    <skos:related rdf:resource="descriptor/13176-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11578-2">
    <skos:related rdf:resource="descriptor/19398-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11578-2">
    <skos:related rdf:resource="descriptor/11579-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11578-2">
    <skos:related rdf:resource="descriptor/19372-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11578-2">
    <skos:related rdf:resource="descriptor/18507-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19432-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20301-2">
    <skos:related rdf:resource="descriptor/10008-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20301-2">
    <skos:related rdf:resource="descriptor/10294-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10433-5">
    <skos:related rdf:resource="descriptor/10329-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14461-1">
    <skos:broader rdf:resource="thsys/70415"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14461-1">
    <skos:broader rdf:resource="descriptor/14455-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18325-3">
    <skos:narrower rdf:resource="descriptor/19561-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18325-3">
    <skos:narrower rdf:resource="descriptor/19625-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16958-2">
    <skos:broader rdf:resource="thsys/70003"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16958-2">
    <skos:broader rdf:resource="descriptor/16972-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13773-1">
    <skos:prefLabel xml:lang="en">Bank secrecy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13773-1">
    <skos:prefLabel xml:lang="de">Bankgeheimnis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10719-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11266-0">
    <skos:related rdf:resource="descriptor/12573-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11266-0">
    <skos:related rdf:resource="descriptor/12572-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11266-0">
    <skos:related rdf:resource="descriptor/11270-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11266-0">
    <skos:related rdf:resource="descriptor/19505-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11266-0">
    <skos:related rdf:resource="descriptor/19111-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11266-0">
    <skos:related rdf:resource="descriptor/10169-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13813-1">
    <skos:related rdf:resource="descriptor/18581-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13813-1">
    <skos:related rdf:resource="descriptor/18580-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13813-1">
    <skos:related rdf:resource="descriptor/13558-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13813-1">
    <skos:related rdf:resource="descriptor/14433-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25743-1">
    <skos:broader rdf:resource="thsys/70141"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10652-0">
    <skos:prefLabel xml:lang="en">Foreign trade elasticities</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10652-0">
    <skos:prefLabel xml:lang="de">Außenhandelselastizität</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24665-0">
    <skos:related rdf:resource="descriptor/24782-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24665-0">
    <skos:related rdf:resource="descriptor/18712-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24665-0">
    <skos:related rdf:resource="descriptor/18718-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14428-6">
    <skos:narrower rdf:resource="descriptor/14427-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14428-6">
    <skos:narrower rdf:resource="descriptor/18294-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16029-3">
    <skos:altLabel xml:lang="de">Testament</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16029-3">
    <skos:altLabel xml:lang="de">Erbvertrag</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16029-3">
    <skos:altLabel xml:lang="de">Erbschein</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16029-3">
    <skos:altLabel xml:lang="de">Erbfolge</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10129-1">
    <skos:prefLabel xml:lang="en">Real interest rate</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10129-1">
    <skos:prefLabel xml:lang="de">Realzins</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18182-6">
    <skos:related rdf:resource="descriptor/18817-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18182-6">
    <skos:related rdf:resource="descriptor/10981-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18182-6">
    <skos:related rdf:resource="descriptor/18447-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18182-6">
    <skos:related rdf:resource="descriptor/15005-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18182-6">
    <skos:related rdf:resource="descriptor/18818-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18182-6">
    <skos:related rdf:resource="descriptor/18816-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18182-6">
    <skos:related rdf:resource="descriptor/10546-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11182-6">
    <skos:altLabel xml:lang="de">Sockelarbeitslosigkeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11182-6">
    <skos:altLabel xml:lang="de">Erwerbslosigkeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11182-6">
    <skos:altLabel xml:lang="de">Erwerbslose</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11182-6">
    <skos:altLabel xml:lang="de">Arbeitslose</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18226-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70922">
    <skos:broader rdf:resource="thsys/70923"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24783-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71117">
    <skos:narrower rdf:resource="descriptor/10225-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71117">
    <skos:narrower rdf:resource="descriptor/10222-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71117">
    <skos:narrower rdf:resource="descriptor/10147-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71117">
    <skos:narrower rdf:resource="descriptor/12170-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71117">
    <skos:narrower rdf:resource="descriptor/16759-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71117">
    <skos:narrower rdf:resource="descriptor/10142-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71117">
    <skos:narrower rdf:resource="descriptor/10141-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71117">
    <skos:narrower rdf:resource="descriptor/18943-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71117">
    <skos:narrower rdf:resource="descriptor/10140-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71117">
    <skos:narrower rdf:resource="descriptor/12168-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71117">
    <skos:narrower rdf:resource="descriptor/10194-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71117">
    <skos:narrower rdf:resource="descriptor/19227-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70881">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.19.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16030-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19816-0">
    <skos:altLabel xml:lang="en">Criminal prosecution</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19816-0">
    <skos:altLabel xml:lang="de">Strafverfolgung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19816-0">
    <skos:altLabel xml:lang="de">Strafprozessordnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19816-0">
    <skos:altLabel xml:lang="de">Strafprozess</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19816-0">
    <skos:altLabel xml:lang="de">Strafprozeßordnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19816-0">
    <skos:altLabel xml:lang="de">Strafprozeß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19816-0">
    <skos:altLabel xml:lang="de">Strafbefehl</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16416-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15038-0">
    <skos:broader rdf:resource="thsys/71035"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15038-0">
    <skos:broader rdf:resource="thsys/70269"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15038-0">
    <skos:broader rdf:resource="descriptor/15045-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12050-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14610-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19202-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12810-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70415">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19422-0">
    <skos:narrower rdf:resource="descriptor/19423-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19422-0">
    <skos:narrower rdf:resource="descriptor/11677-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19422-0">
    <skos:narrower rdf:resource="descriptor/11671-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19422-0">
    <skos:narrower rdf:resource="descriptor/19421-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19422-0">
    <skos:narrower rdf:resource="descriptor/18376-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19422-0">
    <skos:narrower rdf:resource="descriptor/11646-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19479-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18871-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17787-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71063">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12730-3">
    <skos:broader rdf:resource="thsys/71036"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12730-3">
    <skos:broader rdf:resource="thsys/70471"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12730-3">
    <skos:broader rdf:resource="descriptor/12725-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12730-3">
    <skos:broader rdf:resource="descriptor/12165-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12730-3">
    <skos:broader rdf:resource="descriptor/19660-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12837-6">
    <skos:altLabel xml:lang="en">Recognition test</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12837-6">
    <skos:altLabel xml:lang="en">Copytest</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12837-6">
    <skos:altLabel xml:lang="de">Werbewirkungsforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12837-6">
    <skos:altLabel xml:lang="de">Werbetest</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12837-6">
    <skos:altLabel xml:lang="de">Werbeindikator</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12837-6">
    <skos:altLabel xml:lang="de">Werbeerfolgskontrolle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12837-6">
    <skos:altLabel xml:lang="de">Werbeerfolg</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18071-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13555-4">
    <skos:prefLabel xml:lang="en">Motorway</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13555-4">
    <skos:prefLabel xml:lang="de">Autobahn</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12287-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11771-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11033-2">
    <skos:altLabel xml:lang="en">Centralized planning</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11033-2">
    <skos:altLabel xml:lang="de">Zentralisierung der Wirtschaftsplanung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11033-2">
    <skos:altLabel xml:lang="de">Wirtschaftslenkung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11033-2">
    <skos:altLabel xml:lang="de">Sozialistische Wirtschaftsplanung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11033-2">
    <skos:altLabel xml:lang="de">Imperative Planung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15640-3">
    <zbwext:useInsteadNote rdf:nodeID="id64"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15640-3">
    <zbwext:useInsteadNote rdf:nodeID="id63"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15725-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71017">
    <skos:narrower rdf:resource="descriptor/12931-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71017">
    <skos:narrower rdf:resource="descriptor/18345-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71017">
    <skos:narrower rdf:resource="thsys/71011"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71017">
    <skos:narrower rdf:resource="thsys/71016"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14346-1">
    <skos:altLabel xml:lang="de">Automatenverkauf</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14346-1">
    <skos:altLabel xml:lang="de">Automatenindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14346-1">
    <skos:altLabel xml:lang="de">Automatenhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14346-1">
    <skos:altLabel xml:lang="de">Automatenbau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14346-1">
    <skos:altLabel xml:lang="de">Automatenaufstellung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19034-2">
    <skos:related rdf:resource="descriptor/19564-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19034-2">
    <skos:related rdf:resource="descriptor/15066-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19034-2">
    <skos:related rdf:resource="descriptor/12809-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19034-2">
    <skos:related rdf:resource="descriptor/12647-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10437-4">
    <skos:altLabel xml:lang="en">Technology policy</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10437-4">
    <skos:altLabel xml:lang="de">Technologiepolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10437-4">
    <skos:altLabel xml:lang="de">Öffentliche Forschungsförderung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10437-4">
    <skos:altLabel xml:lang="de">FuE-Politik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10437-4">
    <skos:altLabel xml:lang="de">FuE-Förderung des Staates</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10437-4">
    <skos:altLabel xml:lang="de">Forschungspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10437-4">
    <skos:altLabel xml:lang="de">Forschungsförderung des Staates</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11735-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11020-4">
    <skos:altLabel xml:lang="de">Zentrales Planungsorgan</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11020-4">
    <skos:altLabel xml:lang="de">Staatliche Plankommission</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11020-4">
    <skos:altLabel xml:lang="de">Planungsorgan</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11020-4">
    <skos:altLabel xml:lang="de">Plankommission</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11020-4">
    <skos:altLabel xml:lang="de">Planamt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12718-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13364-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19473-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16825-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11443-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13714-3">
    <skos:broader rdf:resource="thsys/71106"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13714-3">
    <skos:broader rdf:resource="thsys/70087"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13714-3">
    <skos:broader rdf:resource="descriptor/19043-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10828-3">
    <skos:altLabel xml:lang="de">Steuerentstrickung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10828-3">
    <skos:altLabel xml:lang="de">Internationale Mehrfachbesteuerung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10828-3">
    <skos:altLabel xml:lang="de">Internationale Doppelbesteuerung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10828-3">
    <skos:altLabel xml:lang="de">Doppelbesteuerungsabkommen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10828-3">
    <skos:altLabel xml:lang="de">DBA (Doppelbesteuerungsabkommen)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10828-3">
    <skos:altLabel xml:lang="de">Abkommen zur Vermeidung der Doppelbesteuerung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19821-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id91">
    <rdfs:seeAlso rdf:resource="descriptor/16552-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70486">
    <skos:broader rdf:resource="thsys/70234"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10653-5">
    <skos:broader rdf:resource="thsys/71100"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10653-5">
    <skos:broader rdf:resource="descriptor/10030-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18741-3">
    <skos:narrower rdf:resource="descriptor/13819-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12178-6">
    <skos:related rdf:resource="descriptor/13992-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12178-6">
    <skos:related rdf:resource="descriptor/18083-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13076-2">
    <skos:related rdf:resource="descriptor/18396-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13076-2">
    <skos:related rdf:resource="descriptor/14473-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13076-2">
    <skos:related rdf:resource="descriptor/18426-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13076-2">
    <skos:related rdf:resource="descriptor/13077-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13076-2">
    <skos:related rdf:resource="descriptor/13202-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13076-2">
    <skos:related rdf:resource="descriptor/13890-4"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id43">
    <rdf:value xml:lang="de">Für den Einzelhandel mit Baustoffen</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16389-1">
    <skos:prefLabel xml:lang="en">National liberation movement</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16389-1">
    <skos:prefLabel xml:lang="de">Unabhängigkeitsbewegung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14032-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13970-6">
    <skos:related rdf:resource="descriptor/13210-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13974-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19743-1">
    <skos:altLabel xml:lang="de">Methodologischer Kollektivismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19743-1">
    <skos:altLabel xml:lang="de">Holistischer Ansatz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19743-1">
    <skos:altLabel xml:lang="de">Ganzheitlicher Ansatz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18721-2">
    <skos:broader rdf:resource="thsys/70900"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18721-2">
    <skos:broader rdf:resource="descriptor/18118-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18721-2">
    <skos:broader rdf:resource="descriptor/12217-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18721-2">
    <skos:broader rdf:resource="descriptor/18716-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15005-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73384">
    <skos:broader rdf:resource="thsys/70017"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17705-5">
    <skos:prefLabel xml:lang="en">Guinea</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17705-5">
    <skos:prefLabel xml:lang="de">Guinea</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18959-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15664-3">
    <skos:related rdf:resource="descriptor/15765-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15664-3">
    <skos:related rdf:resource="descriptor/10470-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15664-3">
    <skos:related rdf:resource="descriptor/14011-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15664-3">
    <skos:related rdf:resource="descriptor/15662-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15664-3">
    <skos:related rdf:resource="descriptor/15658-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15664-3">
    <skos:related rdf:resource="descriptor/15045-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15664-3">
    <skos:related rdf:resource="descriptor/19795-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13151-0">
    <skos:prefLabel xml:lang="en">Brewery</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13151-0">
    <skos:prefLabel xml:lang="de">Brauerei</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71017">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10472-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10713-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13809-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20449-5">
    <skos:altLabel xml:lang="de">Weltkonzernbetriebsrat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14807-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12805-5">
    <skos:prefLabel xml:lang="en">Product design</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12805-5">
    <skos:prefLabel xml:lang="de">Produktdesign</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24664-2">
    <skos:prefLabel xml:lang="en">Digital goods</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24664-2">
    <skos:prefLabel xml:lang="de">Digitale Güter</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11971-0">
    <skos:broader rdf:resource="thsys/71064"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11971-0">
    <skos:broader rdf:resource="thsys/70142"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11971-0">
    <skos:broader rdf:resource="descriptor/15909-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18436-1">
    <skos:related rdf:resource="descriptor/14846-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18436-1">
    <skos:related rdf:resource="descriptor/13141-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18436-1">
    <skos:related rdf:resource="descriptor/19061-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18436-1">
    <skos:related rdf:resource="descriptor/18094-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18436-1">
    <skos:related rdf:resource="descriptor/13232-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18436-1">
    <skos:related rdf:resource="descriptor/18295-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18436-1">
    <skos:related rdf:resource="descriptor/13220-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18436-1">
    <skos:related rdf:resource="descriptor/14899-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16484-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70036">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11336-5">
    <skos:related rdf:resource="descriptor/11765-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11336-5">
    <skos:related rdf:resource="descriptor/11345-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11336-5">
    <skos:related rdf:resource="descriptor/11322-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11336-5">
    <skos:related rdf:resource="descriptor/11159-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16868-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70872">
    <rdfs:label xml:lang="en">Public Sector</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70872">
    <rdfs:label xml:lang="de">Öffentlicher Sektor</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70253">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">G.02.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70970">
    <rdfs:label xml:lang="en">Construction industry</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70970">
    <rdfs:label xml:lang="de">Bauwirtschaft</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13970-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19417-0">
    <skos:broader rdf:resource="thsys/71074"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19417-0">
    <skos:broader rdf:resource="descriptor/11664-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19417-0">
    <skos:broader rdf:resource="descriptor/19270-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14249-6">
    <skos:broader rdf:resource="thsys/70348"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14249-6">
    <skos:broader rdf:resource="descriptor/14021-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10980-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14273-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11582-4">
    <skos:prefLabel xml:lang="en">Military procurement</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11582-4">
    <skos:prefLabel xml:lang="de">Militärische Beschaffung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12191-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11740-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14397-5">
    <skos:related rdf:resource="descriptor/14366-2"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id108">
    <rdf:value xml:lang="de">Für Vergütungen im öffentlichen Dienst</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16226-1">
    <zbwext:useInsteadNote rdf:nodeID="id247"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16226-1">
    <zbwext:useInsteadNote rdf:nodeID="id246"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18149-4">
    <skos:broader rdf:resource="thsys/70355"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18149-4">
    <skos:broader rdf:resource="thsys/70144"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18149-4">
    <skos:broader rdf:resource="thsys/70055"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13483-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18672-3">
    <skos:related rdf:resource="descriptor/13801-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73369">
    <skos:broader rdf:resource="thsys/73374"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16769-5">
    <skos:prefLabel xml:lang="en">Library construction</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16769-5">
    <skos:prefLabel xml:lang="de">Bibliotheksbau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71073">
    <rdfs:label xml:lang="en">Financial law</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71073">
    <rdfs:label xml:lang="de">Finanzrecht</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19351-4">
    <skos:prefLabel xml:lang="en">Trading bloc</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19351-4">
    <skos:prefLabel xml:lang="de">Handelsblock</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13867-6">
    <skos:prefLabel xml:lang="en">Residential mobility</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13867-6">
    <skos:prefLabel xml:lang="de">Wohnungswechsel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19855-4">
    <skos:broader rdf:resource="thsys/73375"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19855-4">
    <skos:broader rdf:resource="thsys/73334"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19855-4">
    <skos:broader rdf:resource="descriptor/16195-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14100-5">
    <skos:prefLabel xml:lang="en">Banana</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14100-5">
    <skos:prefLabel xml:lang="de">Banane</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11782-3">
    <skos:broader rdf:resource="thsys/71110"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11782-3">
    <skos:broader rdf:resource="thsys/70575"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16749-4">
    <skos:prefLabel xml:lang="en">Population theory</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16749-4">
    <skos:prefLabel xml:lang="de">Bevölkerungstheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19141-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11302-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18578-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13571-6">
    <skos:altLabel xml:lang="de">Autogesellschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10241-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12736-5">
    <skos:prefLabel xml:lang="en">Handling equipment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12736-5">
    <skos:prefLabel xml:lang="de">Handhabungsgerät</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12255-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19367-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13446-2">
    <skos:narrower rdf:resource="descriptor/13445-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13446-2">
    <skos:narrower rdf:resource="descriptor/18318-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13446-2">
    <skos:narrower rdf:resource="descriptor/13444-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13446-2">
    <skos:narrower rdf:resource="descriptor/13443-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13446-2">
    <skos:narrower rdf:resource="descriptor/18317-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15078-2">
    <skos:altLabel xml:lang="de">Index der Verkaufspreise für Exportgüter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15078-2">
    <skos:altLabel xml:lang="de">Index der Einkaufspreise für Auslandsgüter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15078-2">
    <skos:altLabel xml:lang="de">Index der Einfuhrpreise</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15078-2">
    <skos:altLabel xml:lang="de">Index der Ausfuhrpreise</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15078-2">
    <skos:altLabel xml:lang="de">Importpreisindex</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15078-2">
    <skos:altLabel xml:lang="de">Exportpreisindex</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13608-2">
    <skos:broader rdf:resource="thsys/70927"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13608-2">
    <skos:broader rdf:resource="descriptor/13588-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13608-2">
    <skos:broader rdf:resource="descriptor/12041-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18138-2">
    <skos:scopeNote xml:lang="de">Wirkung der Ankündigung wirtschaftspolitischer oder betrieblicher Maßnahmen auf ökonomische Variablen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14272-4">
    <skos:prefLabel xml:lang="en">Galvanized product</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14272-4">
    <skos:prefLabel xml:lang="de">Galvanisiertes Produkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11618-2">
    <skos:prefLabel xml:lang="en">Treasuries</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11618-2">
    <skos:prefLabel xml:lang="de">Schatzpapier</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15611-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73295">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.06.01.12.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19055-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17686-4">
    <skos:broader rdf:resource="thsys/70429"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17686-4">
    <skos:broader rdf:resource="descriptor/17687-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19440-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10823-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12558-3">
    <skos:related rdf:resource="descriptor/12437-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12558-3">
    <skos:related rdf:resource="descriptor/12396-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12558-3">
    <skos:related rdf:resource="descriptor/16092-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12558-3">
    <skos:related rdf:resource="descriptor/18302-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16943-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10698-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18340-0">
    <skos:altLabel xml:lang="en">Tropical rain forest</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18340-0">
    <skos:altLabel xml:lang="en">Rain forest</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18340-0">
    <skos:altLabel xml:lang="de">Tropenwald</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18340-0">
    <skos:altLabel xml:lang="de">Regenwald</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15722-1">
    <skos:narrower rdf:resource="descriptor/19772-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15722-1">
    <skos:narrower rdf:resource="descriptor/19009-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15722-1">
    <skos:narrower rdf:resource="descriptor/18832-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15722-1">
    <skos:narrower rdf:resource="descriptor/10150-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19623-4">
    <skos:broader rdf:resource="thsys/73352"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18196-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10797-2">
    <skos:related rdf:resource="descriptor/19563-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10797-2">
    <skos:related rdf:resource="descriptor/10816-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10797-2">
    <skos:related rdf:resource="descriptor/10786-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10797-2">
    <skos:related rdf:resource="descriptor/10812-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10797-2">
    <skos:related rdf:resource="descriptor/18694-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10797-2">
    <skos:related rdf:resource="descriptor/10811-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10797-2">
    <skos:related rdf:resource="descriptor/10805-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10797-2">
    <skos:related rdf:resource="descriptor/10804-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10797-2">
    <skos:related rdf:resource="descriptor/10552-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10797-2">
    <skos:related rdf:resource="descriptor/10798-0"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id352">
    <rdf:value xml:lang="de">Für volkswirtschaftliche Verluste</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19651-6">
    <skos:related rdf:resource="descriptor/18552-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19651-6">
    <skos:related rdf:resource="descriptor/18648-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19651-6">
    <skos:related rdf:resource="descriptor/19145-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19651-6">
    <skos:related rdf:resource="descriptor/18659-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19651-6">
    <skos:related rdf:resource="descriptor/19801-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11271-0">
    <skos:altLabel xml:lang="de">Erwerbspotenzial</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11271-0">
    <skos:altLabel xml:lang="de">Erwerbspotential</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11271-0">
    <skos:altLabel xml:lang="de">Erwerbspersonenpotential</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11271-0">
    <skos:altLabel xml:lang="de">Erwerbsbevölkerung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11271-0">
    <skos:altLabel xml:lang="de">Arbeitskräftepotenzial</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11271-0">
    <skos:altLabel xml:lang="de">Arbeitskräftepotential</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18639-1">
    <skos:altLabel xml:lang="de">Touristik-Marketing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18639-1">
    <skos:altLabel xml:lang="de">Fremdenverkehrsmarketing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10499-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70506">
    <rdfs:label xml:lang="de">Meeresprodukte</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16391-0">
    <skos:broader rdf:resource="thsys/73378"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16391-0">
    <skos:broader rdf:resource="descriptor/15711-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14817-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70038">
    <skos:broader rdf:resource="thsys/70039"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11993-4">
    <skos:related rdf:resource="descriptor/13593-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11993-4">
    <skos:related rdf:resource="descriptor/10512-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11993-4">
    <skos:related rdf:resource="descriptor/11976-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11993-4">
    <skos:related rdf:resource="descriptor/11991-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11993-4">
    <skos:related rdf:resource="descriptor/11902-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11993-4">
    <skos:related rdf:resource="descriptor/11970-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11993-4">
    <skos:related rdf:resource="descriptor/11876-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11993-4">
    <skos:related rdf:resource="descriptor/11893-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11993-4">
    <skos:related rdf:resource="descriptor/10503-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11993-4">
    <skos:related rdf:resource="descriptor/10504-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11993-4">
    <skos:related rdf:resource="descriptor/11863-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17712-1">
    <skos:broader rdf:resource="thsys/70560"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17712-1">
    <skos:broader rdf:resource="descriptor/17724-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12072-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14270-1">
    <skos:altLabel xml:lang="en">Tube</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13600-4">
    <skos:prefLabel xml:lang="en">Vehicles fleet</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13600-4">
    <skos:prefLabel xml:lang="de">Fuhrpark</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16186-1">
    <zbwext:useInsteadNote rdf:nodeID="id156"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18900-2">
    <skos:related rdf:resource="descriptor/14997-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18900-2">
    <skos:related rdf:resource="descriptor/11677-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18900-2">
    <skos:related rdf:resource="descriptor/13164-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18900-2">
    <skos:related rdf:resource="descriptor/16676-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18900-2">
    <skos:related rdf:resource="descriptor/19394-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18900-2">
    <skos:related rdf:resource="descriptor/16479-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24703-4">
    <skos:scopeNote xml:lang="de">Anrechnung bereits vorher angefallener Körperschaftsteuer bei Gewinnausschüttungen und Erträgen von Anteilsveräußerungen sowie vergleichbare Bestimmungen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15715-5">
    <skos:narrower rdf:resource="descriptor/11305-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12647-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11298-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15801-5">
    <skos:broader rdf:resource="thsys/70292"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15801-5">
    <skos:broader rdf:resource="thsys/70017"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14056-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18060-6">
    <skos:related rdf:resource="descriptor/11788-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18060-6">
    <skos:related rdf:resource="descriptor/19451-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18060-6">
    <skos:related rdf:resource="descriptor/18478-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18060-6">
    <skos:related rdf:resource="descriptor/10500-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18060-6">
    <skos:related rdf:resource="descriptor/19245-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18060-6">
    <skos:related rdf:resource="descriptor/19437-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12025-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10387-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10018-3">
    <skos:narrower rdf:resource="descriptor/11209-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19213-2">
    <skos:broader rdf:resource="thsys/70274"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19213-2">
    <skos:broader rdf:resource="thsys/70154"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19213-2">
    <skos:broader rdf:resource="descriptor/10032-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17590-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10987-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18133-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18591-3">
    <zbwext:useInsteadNote rdf:nodeID="id289"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18591-3">
    <zbwext:useInsteadNote rdf:nodeID="id288"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12247-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16142-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18699-4">
    <skos:related rdf:resource="descriptor/10821-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12201-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13244-0">
    <skos:altLabel xml:lang="de">Straßengütertransport</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13244-0">
    <skos:altLabel xml:lang="de">Schwerverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13244-0">
    <skos:altLabel xml:lang="de">Schwerlastverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13244-0">
    <skos:altLabel xml:lang="de">Lkw-Verkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13244-0">
    <skos:altLabel xml:lang="de">Güterkraftverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71056">
    <skos:broader rdf:resource="thsys/71057"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13465-5">
    <skos:related rdf:resource="descriptor/13210-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13465-5">
    <skos:related rdf:resource="descriptor/19604-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17636-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11864-1">
    <skos:broader rdf:resource="thsys/70925"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11864-1">
    <skos:broader rdf:resource="descriptor/13510-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10022-5">
    <skos:prefLabel xml:lang="en">Debt rescheduling</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10022-5">
    <skos:prefLabel xml:lang="de">Umschuldung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18788-5">
    <skos:altLabel xml:lang="de">Kommunikations-Designerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18788-5">
    <skos:altLabel xml:lang="de">Kommunikations-Designer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18788-5">
    <skos:altLabel xml:lang="de">Industrie-Designerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18788-5">
    <skos:altLabel xml:lang="de">Industrie-Designer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18788-5">
    <skos:altLabel xml:lang="de">Designleistungen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18788-5">
    <skos:altLabel xml:lang="de">Designerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73338">
    <skos:prefLabel xml:lang="de">N.10.07.04  Informatik-Anwendungen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13185-4">
    <skos:prefLabel xml:lang="en">Plumbing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13185-4">
    <skos:prefLabel xml:lang="de">Klempnerhandwerk</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15064-6">
    <skos:related rdf:resource="descriptor/15076-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15064-6">
    <skos:related rdf:resource="descriptor/18161-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15064-6">
    <skos:related rdf:resource="descriptor/19581-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15064-6">
    <skos:related rdf:resource="descriptor/19072-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15064-6">
    <skos:related rdf:resource="descriptor/15483-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15064-6">
    <skos:related rdf:resource="descriptor/15045-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19251-1">
    <skos:prefLabel xml:lang="en">Innovation policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19251-1">
    <skos:prefLabel xml:lang="de">Innovationspolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11460-4">
    <zbwext:useInsteadNote rdf:nodeID="id116"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11460-4">
    <zbwext:useInsteadNote rdf:nodeID="id115"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14626-2">
    <skos:altLabel xml:lang="de">Unkrautvertilgungsmittel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14626-2">
    <skos:altLabel xml:lang="de">Schädlingsbekämpfungsmittel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14626-2">
    <skos:altLabel xml:lang="de">Saatenschutzmittel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14626-2">
    <skos:altLabel xml:lang="de">Rodentizid</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14626-2">
    <skos:altLabel xml:lang="de">Pflanzenschutzmittel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14626-2">
    <skos:altLabel xml:lang="de">Nematizid</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14626-2">
    <skos:altLabel xml:lang="de">Herbizid</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14626-2">
    <skos:altLabel xml:lang="de">Fungizid</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14626-2">
    <skos:altLabel xml:lang="de">Biozid</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10092-5">
    <skos:narrower rdf:resource="descriptor/11931-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10092-5">
    <skos:narrower rdf:resource="descriptor/19431-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10092-5">
    <skos:narrower rdf:resource="descriptor/10172-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10092-5">
    <skos:narrower rdf:resource="descriptor/10094-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10092-5">
    <skos:narrower rdf:resource="descriptor/11919-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18489-1">
    <skos:prefLabel xml:lang="en">Industrial district</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18489-1">
    <skos:prefLabel xml:lang="de">Industriegebiet</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10214-3">
    <skos:prefLabel xml:lang="en">Price-response function</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10214-3">
    <skos:prefLabel xml:lang="de">Preis-Absatz-Funktion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18288-4">
    <skos:related rdf:resource="descriptor/15769-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18288-4">
    <skos:related rdf:resource="descriptor/18289-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18288-4">
    <skos:related rdf:resource="descriptor/15992-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21315-0">
    <skos:altLabel xml:lang="en">Spam</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21315-0">
    <skos:altLabel xml:lang="de">Online-Werbemarkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21315-0">
    <skos:altLabel xml:lang="de">Online-Marketing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21315-0">
    <skos:altLabel xml:lang="de">Internet-Werbung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21315-0">
    <skos:altLabel xml:lang="de">Internet-Marketing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21315-0">
    <skos:altLabel xml:lang="de">E-Mail-Werbung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21315-0">
    <skos:altLabel xml:lang="de">Banner-Werbung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21315-0">
    <skos:altLabel xml:lang="de">Banner</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13272-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70941">
    <skos:broader rdf:resource="thsys/70942"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id242">
    <rdf:value xml:lang="de">Für Maßnahmen zur Stimulierung der Leistung von Arbeitskräften</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11399-2">
    <skos:broader rdf:resource="thsys/73369"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11399-2">
    <skos:broader rdf:resource="thsys/70847"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11399-2">
    <skos:broader rdf:resource="thsys/70176"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11399-2">
    <skos:broader rdf:resource="descriptor/11215-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11399-2">
    <skos:broader rdf:resource="descriptor/11397-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12172-4">
    <skos:scopeNote xml:lang="de">Wissenschaft, die sich mit der Gestaltung rechnergestützter Informationssysteme in der Wirtschaft befasst.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10197-5">
    <skos:related rdf:resource="descriptor/10224-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10197-5">
    <skos:related rdf:resource="descriptor/10195-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13686-3">
    <skos:related rdf:resource="descriptor/13756-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13686-3">
    <skos:related rdf:resource="descriptor/13283-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11355-1">
    <skos:altLabel xml:lang="en">School leavers</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11355-1">
    <skos:altLabel xml:lang="de">Schulabgängerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11355-1">
    <skos:altLabel xml:lang="de">Schulabgänger</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11355-1">
    <skos:altLabel xml:lang="de">Ausbildungsabsolventinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11355-1">
    <skos:altLabel xml:lang="de">Ausbildungsabsolventen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11355-1">
    <skos:altLabel xml:lang="de">Absolventinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19675-6">
    <skos:broader rdf:resource="thsys/70993"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19675-6">
    <skos:broader rdf:resource="thsys/70957"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19675-6">
    <skos:broader rdf:resource="descriptor/10012-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71061">
    <skos:prefLabel xml:lang="en">V.13.01.02  Labour market policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71061">
    <skos:prefLabel xml:lang="de">V.13.01.02  Arbeitsmarktpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15768-5">
    <skos:broader rdf:resource="thsys/73371"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15768-5">
    <skos:broader rdf:resource="thsys/70815"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15768-5">
    <skos:broader rdf:resource="thsys/70239"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15768-5">
    <skos:broader rdf:resource="descriptor/16273-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11183-4">
    <skos:altLabel xml:lang="en">Labor market</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11183-4">
    <skos:altLabel xml:lang="de">Arbeitsmarktstruktur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18302-1">
    <skos:related rdf:resource="descriptor/12558-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18302-1">
    <skos:related rdf:resource="descriptor/12876-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18302-1">
    <skos:related rdf:resource="descriptor/12396-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18302-1">
    <skos:related rdf:resource="descriptor/12416-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18302-1">
    <skos:related rdf:resource="descriptor/12326-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14269-0">
    <skos:altLabel xml:lang="de">NE-Metallprodukt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14269-0">
    <skos:altLabel xml:lang="de">Metallhalbzeug</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14269-0">
    <skos:altLabel xml:lang="de">Leichtmetallkonstruktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14269-0">
    <skos:altLabel xml:lang="de">Bolzen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14269-0">
    <skos:altLabel xml:lang="de">Baugerüst</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13510-5">
    <skos:prefLabel xml:lang="en">Traffic</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13510-5">
    <skos:prefLabel xml:lang="de">Verkehr</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14041-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18772-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11612-0">
    <skos:altLabel xml:lang="de">Staatsbeteiligung an Unternehmen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11612-0">
    <skos:altLabel xml:lang="de">Staatliche Kapitalbeteiligung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11612-0">
    <skos:altLabel xml:lang="de">Öffentliche Unternehmensbeteiligung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11612-0">
    <skos:altLabel xml:lang="de">Öffentliche Beteiligung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11612-0">
    <skos:altLabel xml:lang="de">Landesbeteiligung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11612-0">
    <skos:altLabel xml:lang="de">Kapitalbeteiligung des Staates</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10730-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10129-1">
    <skos:broader rdf:resource="thsys/70455"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10129-1">
    <skos:broader rdf:resource="descriptor/10135-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15716-3">
    <skos:broader rdf:resource="thsys/73395"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17948-0">
    <skos:prefLabel xml:lang="en">Oceania</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17948-0">
    <skos:prefLabel xml:lang="de">Südpazifischer Inselraum</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18570-4">
    <skos:prefLabel xml:lang="en">Container train</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18570-4">
    <skos:prefLabel xml:lang="de">Eisenbahn-Containerverkehr</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10437-4">
    <skos:narrower rdf:resource="descriptor/18653-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10437-4">
    <skos:narrower rdf:resource="descriptor/18031-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10437-4">
    <skos:narrower rdf:resource="descriptor/18419-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10437-4">
    <skos:narrower rdf:resource="descriptor/18774-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10437-4">
    <skos:narrower rdf:resource="descriptor/18773-4"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id385">
    <rdfs:seeAlso rdf:resource="descriptor/11260-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10466-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18847-1">
    <skos:scopeNote xml:lang="de">Fragen von Vor- und Nachteilen der Mitgliedschaft in Verbänden und Organisationen, Ein- und Austritte aus denselben, usw.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70852">
    <rdfs:label xml:lang="de">Wirtschafts- und Berufsverbände</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10065-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19648-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70058">
    <skos:broader rdf:resource="thsys/v"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14145-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13379-5">
    <skos:altLabel xml:lang="de">OCR-Dienst</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13379-5">
    <skos:altLabel xml:lang="de">Informatik-Dienstleistung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13379-5">
    <skos:altLabel xml:lang="de">Hardware-Beratung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13379-5">
    <skos:altLabel xml:lang="de">EDV-Beratung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13379-5">
    <skos:altLabel xml:lang="de">DV-Dienstleistung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13379-5">
    <skos:altLabel xml:lang="de">Datenverarbeitungsdienst</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13379-5">
    <skos:altLabel xml:lang="de">Datenerfassungs-Dienstleistung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13379-5">
    <skos:altLabel xml:lang="de">COM-Dienst</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15730-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14789-2">
    <skos:prefLabel xml:lang="en">Wooden plate</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14789-2">
    <skos:prefLabel xml:lang="de">Holzplatte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18948-2">
    <skos:narrower rdf:resource="descriptor/10268-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18948-2">
    <skos:narrower rdf:resource="descriptor/19553-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18948-2">
    <skos:narrower rdf:resource="descriptor/18982-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19129-5">
    <skos:narrower rdf:resource="descriptor/12806-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70144">
    <rdfs:label xml:lang="de">Öffentliche Finanzen und Finanzwissenschaft</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id280">
    <rdfs:seeAlso rdf:resource="descriptor/11183-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13343-5">
    <skos:related rdf:resource="descriptor/13349-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13343-5">
    <skos:related rdf:resource="descriptor/15685-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13343-5">
    <skos:related rdf:resource="descriptor/10436-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11881-1">
    <skos:related rdf:resource="descriptor/18633-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11881-1">
    <skos:related rdf:resource="descriptor/11974-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11881-1">
    <skos:related rdf:resource="descriptor/11876-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11881-1">
    <skos:related rdf:resource="descriptor/11979-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17177-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19765-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16732-0">
    <skos:broader rdf:resource="thsys/73393"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16732-0">
    <skos:broader rdf:resource="descriptor/16719-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12033-4">
    <skos:related rdf:resource="descriptor/12785-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12033-4">
    <skos:related rdf:resource="descriptor/13464-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12033-4">
    <skos:related rdf:resource="descriptor/12787-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24859-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18564-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11692-4">
    <skos:altLabel xml:lang="de">Inzidenz der Steuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11692-4">
    <skos:altLabel xml:lang="de">Formale Steuerinzidenz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11692-4">
    <skos:altLabel xml:lang="de">Effektive Steuerinzidenz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70504">
    <rdfs:label xml:lang="de">Soziales</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10561-3">
    <skos:related rdf:resource="descriptor/12154-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10561-3">
    <skos:related rdf:resource="descriptor/10590-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10561-3">
    <skos:related rdf:resource="descriptor/10555-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10561-3">
    <skos:related rdf:resource="descriptor/10548-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10025-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12116-0">
    <skos:prefLabel xml:lang="en">Taylorism</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12116-0">
    <skos:prefLabel xml:lang="de">Taylorismus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18756-4">
    <skos:related rdf:resource="descriptor/18122-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19010-2">
    <skos:prefLabel xml:lang="en">Scientific research methods</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19010-2">
    <skos:prefLabel xml:lang="de">Wissenschaftliches Arbeiten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71031">
    <skos:prefLabel xml:lang="en">G.03.01.04  Central Africa</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71031">
    <skos:prefLabel xml:lang="de">G.03.01.04  Zentralafrika</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12907-4">
    <skos:altLabel xml:lang="en">Entertainment industry</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12907-4">
    <skos:altLabel xml:lang="de">Zoologischer Garten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12907-4">
    <skos:altLabel xml:lang="de">Zoo</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12907-4">
    <skos:altLabel xml:lang="de">Zirkus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12907-4">
    <skos:altLabel xml:lang="de">Vergnügungspark</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12907-4">
    <skos:altLabel xml:lang="de">Vergnügungsindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12907-4">
    <skos:altLabel xml:lang="de">Unterhaltungsindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12907-4">
    <skos:altLabel xml:lang="de">Tiergehege</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12907-4">
    <skos:altLabel xml:lang="de">Freizeitpark</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12907-4">
    <skos:altLabel xml:lang="de">Erholungspark</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11046-0">
    <skos:broader rdf:resource="thsys/73396"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11046-0">
    <skos:broader rdf:resource="thsys/73382"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11046-0">
    <skos:broader rdf:resource="thsys/71102"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11046-0">
    <skos:broader rdf:resource="descriptor/11068-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11046-0">
    <skos:broader rdf:resource="descriptor/16602-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13233-5">
    <skos:related rdf:resource="descriptor/13235-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13233-5">
    <skos:related rdf:resource="descriptor/13096-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13233-5">
    <skos:related rdf:resource="descriptor/14581-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12387-4">
    <skos:prefLabel xml:lang="en">Accrual accounting</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12387-4">
    <skos:prefLabel xml:lang="de">Rechnungsabgrenzung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14054-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15876-2">
    <skos:narrower rdf:resource="descriptor/15888-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15876-2">
    <skos:narrower rdf:resource="descriptor/15891-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15876-2">
    <skos:narrower rdf:resource="descriptor/15873-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15876-2">
    <skos:narrower rdf:resource="descriptor/15872-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18572-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13820-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12945-3">
    <skos:prefLabel xml:lang="en">Agriculture</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12945-3">
    <skos:prefLabel xml:lang="de">Landwirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19806-3">
    <skos:prefLabel xml:lang="en">Computer graphics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19806-3">
    <skos:prefLabel xml:lang="de">Computergraphik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70305">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11065-3">
    <skos:broader rdf:resource="thsys/70180"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11065-3">
    <skos:broader rdf:resource="descriptor/10975-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11065-3">
    <skos:broader rdf:resource="descriptor/10144-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18075-0">
    <skos:altLabel xml:lang="en">Compact disc read-only memory</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18075-0">
    <skos:altLabel xml:lang="de">Nur-Leseplattenspeicher</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18075-0">
    <skos:altLabel xml:lang="de">CD-ROM-Laufwerk</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14897-6">
    <skos:broader rdf:resource="thsys/70066"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14897-6">
    <skos:broader rdf:resource="descriptor/14895-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14897-6">
    <skos:broader rdf:resource="descriptor/14022-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18829-3">
    <skos:altLabel xml:lang="de">Recht des öffentlichen Dienstes</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18829-3">
    <skos:altLabel xml:lang="de">Bundesbeamtengesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18829-3">
    <skos:altLabel xml:lang="de">Beamtenrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18829-3">
    <skos:altLabel xml:lang="de">Beamtendisziplinarrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11212-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11892-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18813-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13113-1">
    <skos:related rdf:resource="descriptor/12921-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13113-1">
    <skos:related rdf:resource="descriptor/14787-6"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id39">
    <rdfs:seeAlso rdf:resource="descriptor/18001-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19581-1">
    <skos:broader rdf:resource="thsys/72182"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19581-1">
    <skos:broader rdf:resource="descriptor/15685-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#broadMatch">
    <owl:inverseOf rdf:resource="http://www.w3.org/2004/02/skos/core#narrowMatch"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11976-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70321">
    <skos:prefLabel xml:lang="de">N.06.02  Gesellschaft und sozialer Wandel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15763-1">
    <skos:prefLabel xml:lang="en">Environmental degradation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15763-1">
    <skos:prefLabel xml:lang="de">Umweltbelastung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10028-0">
    <skos:scopeNote xml:lang="de">Die in der Volkswirtschaftlichen Gesamtrechnung ausgewiesene erbrachte wirtschaftliche Leistung der einzelnen Sektoren.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14335-6">
    <skos:altLabel xml:lang="de">Warenautomat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14335-6">
    <skos:altLabel xml:lang="de">Unterhaltungsautomat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14335-6">
    <skos:altLabel xml:lang="de">Spielautomat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14335-6">
    <skos:altLabel xml:lang="de">Musikautomat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14335-6">
    <skos:altLabel xml:lang="de">Münzautomat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14335-6">
    <skos:altLabel xml:lang="de">Glücksspielautomat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14335-6">
    <skos:altLabel xml:lang="de">Geldspielautomat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14335-6">
    <skos:altLabel xml:lang="de">Dienstleistungsautomat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11870-6">
    <skos:related rdf:resource="descriptor/11869-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11870-6">
    <skos:related rdf:resource="descriptor/12020-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11870-6">
    <skos:related rdf:resource="descriptor/16176-4"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id3">
    <rdfs:seeAlso rdf:resource="descriptor/12766-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12146-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71117">
    <skos:broader rdf:resource="thsys/70180"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18467-4">
    <skos:prefLabel xml:lang="en">Industrial cooperation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18467-4">
    <skos:prefLabel xml:lang="de">Industrielle Kooperation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14350-3">
    <skos:related rdf:resource="descriptor/14214-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19532-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19504-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10785-2">
    <skos:narrower rdf:resource="descriptor/19360-3"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id317">
    <rdfs:seeAlso rdf:resource="descriptor/10828-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11061-4">
    <skos:related rdf:resource="descriptor/11033-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11061-4">
    <skos:related rdf:resource="descriptor/11030-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11061-4">
    <skos:related rdf:resource="descriptor/10985-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11061-4">
    <skos:related rdf:resource="descriptor/11101-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11061-4">
    <skos:related rdf:resource="descriptor/11142-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11061-4">
    <skos:related rdf:resource="descriptor/11090-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11061-4">
    <skos:related rdf:resource="descriptor/11143-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11061-4">
    <skos:related rdf:resource="descriptor/11123-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11061-4">
    <skos:related rdf:resource="descriptor/11060-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11061-4">
    <skos:related rdf:resource="descriptor/11047-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11061-4">
    <skos:related rdf:resource="descriptor/16409-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70506">
    <skos:narrower rdf:resource="descriptor/19023-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70506">
    <skos:narrower rdf:resource="descriptor/14164-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70506">
    <skos:narrower rdf:resource="descriptor/14163-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70506">
    <skos:narrower rdf:resource="descriptor/14160-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70506">
    <skos:narrower rdf:resource="descriptor/14161-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70506">
    <skos:narrower rdf:resource="descriptor/19019-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70506">
    <skos:narrower rdf:resource="descriptor/14158-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16687-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15682-1">
    <skos:altLabel xml:lang="en">New Age</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15682-1">
    <skos:altLabel xml:lang="de">Weltbild</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15682-1">
    <skos:altLabel xml:lang="de">Spiritualität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15682-1">
    <skos:altLabel xml:lang="de">Parapsychologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15682-1">
    <skos:altLabel xml:lang="de">Okkultismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15682-1">
    <skos:altLabel xml:lang="de">Magie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15682-1">
    <skos:altLabel xml:lang="de">Esoterik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15682-1">
    <skos:altLabel xml:lang="de">Atheismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15682-1">
    <skos:altLabel xml:lang="de">Astrologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15682-1">
    <skos:altLabel xml:lang="de">Aberglaube</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19544-0">
    <skos:prefLabel xml:lang="en">E-Business</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19544-0">
    <skos:prefLabel xml:lang="de">E-Business</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16187-6">
    <skos:broader rdf:resource="thsys/73389"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16187-6">
    <skos:broader rdf:resource="thsys/73332"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16187-6">
    <skos:broader rdf:resource="thsys/70934"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16187-6">
    <skos:broader rdf:resource="descriptor/15966-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16278-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14444-1">
    <skos:narrower rdf:resource="descriptor/14437-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14444-1">
    <skos:narrower rdf:resource="descriptor/14432-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#topConceptOf">
    <rdfs:label xml:lang="en">top concept in scheme</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11469-0">
    <skos:prefLabel xml:lang="en">Monetary neutrality</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11469-0">
    <skos:prefLabel xml:lang="de">Neutralität des Geldes</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18696-3">
    <skos:narrower rdf:resource="descriptor/11459-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11671-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24857-5">
    <skos:prefLabel xml:lang="en">Path dependence</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24857-5">
    <skos:prefLabel xml:lang="de">Pfadabhängigkeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19133-0">
    <skos:altLabel xml:lang="en">Customer relationship</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19133-0">
    <skos:altLabel xml:lang="en">CRM (Customer Relationship Management)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19133-0">
    <skos:altLabel xml:lang="en">B2C</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19133-0">
    <skos:altLabel xml:lang="de">Kundentreue</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19133-0">
    <skos:altLabel xml:lang="de">Kundenorientierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19133-0">
    <skos:altLabel xml:lang="de">Kundennähe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19133-0">
    <skos:altLabel xml:lang="de">Kundenmanagement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19133-0">
    <skos:altLabel xml:lang="de">Kundenbindungsmanagement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19133-0">
    <skos:altLabel xml:lang="de">Kundenbindung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id314">
    <rdf:value xml:lang="de">Für gesetzliche Vertretung im Sinne von Vormundschaft</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12978-2">
    <skos:altLabel xml:lang="de">Gaswerk</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12978-2">
    <skos:altLabel xml:lang="de">Erdgaswirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10211-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18687-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12912-4">
    <skos:prefLabel xml:lang="en">Timber industry and trade</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12912-4">
    <skos:prefLabel xml:lang="de">Holzwirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17959-2">
    <skos:prefLabel xml:lang="en">Indian ocean</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17959-2">
    <skos:prefLabel xml:lang="de">Indischer Ozean</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17707-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13830-1">
    <skos:broader rdf:resource="thsys/70888"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13830-1">
    <skos:broader rdf:resource="descriptor/19068-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17727-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16264-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19694-2">
    <skos:broader rdf:resource="thsys/73380"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19694-2">
    <skos:broader rdf:resource="descriptor/16329-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10062-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13063-4">
    <skos:altLabel xml:lang="en">Rolling stock industry</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13063-4">
    <skos:altLabel xml:lang="de">Waggonbau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13063-4">
    <skos:altLabel xml:lang="de">Lokomotivbau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13063-4">
    <skos:altLabel xml:lang="de">Eisenbahnindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13063-4">
    <skos:altLabel xml:lang="de">Eisenbahnfahrzeugindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13063-4">
    <skos:altLabel xml:lang="de">Eisenbahnfahrzeugbau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13063-4">
    <skos:altLabel xml:lang="de">Bahnindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18923-4">
    <skos:prefLabel xml:lang="en">Arts and cultural management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18923-4">
    <skos:prefLabel xml:lang="de">Kulturmanagement</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14885-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12997-5">
    <skos:prefLabel xml:lang="en">Fertilizer industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12997-5">
    <skos:prefLabel xml:lang="de">Düngemittelindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13667-0">
    <skos:broader rdf:resource="thsys/70922"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13667-0">
    <skos:broader rdf:resource="thsys/70135"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13667-0">
    <skos:broader rdf:resource="descriptor/13278-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13667-0">
    <skos:broader rdf:resource="descriptor/13277-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13667-0">
    <skos:broader rdf:resource="descriptor/14490-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11297-3">
    <skos:prefLabel xml:lang="en">Occupational statistics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11297-3">
    <skos:prefLabel xml:lang="de">Berufsstatistik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19558-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10498-5">
    <skos:scopeNote xml:lang="de">Beinhalten vor allem die Bedürfnisse nach ausreichender Nahrung und sauberem Wasser, nach Bekleidung und Wohnung, nach Bildung und Gesundheitsversorgung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18330-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12511-1">
    <skos:broader rdf:resource="thsys/70871"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12511-1">
    <skos:broader rdf:resource="thsys/70578"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12511-1">
    <skos:broader rdf:resource="thsys/70144"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12511-1">
    <skos:broader rdf:resource="descriptor/12492-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10858-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19600-2">
    <skos:prefLabel xml:lang="en">Humor</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19600-2">
    <skos:prefLabel xml:lang="de">Humor</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15264-5">
    <skos:prefLabel xml:lang="en">Maximum-likelihood estimation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15264-5">
    <skos:prefLabel xml:lang="de">Maximum-Likelihood-Methode</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15646-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19580-3">
    <skos:related rdf:resource="descriptor/15709-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19580-3">
    <skos:related rdf:resource="descriptor/15954-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18962-1">
    <skos:scopeNote xml:lang="de">Berücksichtigung aller Formen des Recycling bei der Produktentwicklung sowie der technischen Maßnahmen, die zur Durchführung des Recycling notwendig sind.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18073-4">
    <skos:altLabel xml:lang="en">Alternative economy</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18073-4">
    <skos:altLabel xml:lang="de">Ökologische Wirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18073-4">
    <skos:altLabel xml:lang="de">Ausweichökonomie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18073-4">
    <skos:altLabel xml:lang="de">Alternativsektor</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18073-4">
    <skos:altLabel xml:lang="de">Alternativ-ökonomischer Sektor</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18073-4">
    <skos:altLabel xml:lang="de">Alternativökonomie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18073-4">
    <skos:altLabel xml:lang="de">Alternative Wirtschaftstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18073-4">
    <skos:altLabel xml:lang="de">Alternative Wirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16338-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13406-0">
    <skos:altLabel xml:lang="en">Unemployment compensation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13406-0">
    <skos:altLabel xml:lang="en">Unemployment benefits</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13406-0">
    <skos:altLabel xml:lang="en">Unemployment assistance</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13406-0">
    <skos:altLabel xml:lang="de">Arbeitslosenunterstützung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13406-0">
    <skos:altLabel xml:lang="de">Arbeitslosenhilfe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13406-0">
    <skos:altLabel xml:lang="de">Arbeitslosengeld</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18421-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15905-0">
    <skos:altLabel xml:lang="en">Age structure</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15905-0">
    <skos:altLabel xml:lang="en">Age distribution</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15905-0">
    <skos:altLabel xml:lang="en">Age composition</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15905-0">
    <skos:altLabel xml:lang="de">Altersgliederung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13684-0">
    <skos:prefLabel xml:lang="en">Satellite communication</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13684-0">
    <skos:prefLabel xml:lang="de">Satellitenkommunikation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71070">
    <skos:broader rdf:resource="thsys/70058"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11200-2">
    <skos:broader rdf:resource="thsys/71060"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11200-2">
    <skos:broader rdf:resource="descriptor/11182-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18534-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18746-0">
    <skos:related rdf:resource="descriptor/12027-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18746-0">
    <skos:related rdf:resource="descriptor/13330-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18746-0">
    <skos:related rdf:resource="descriptor/13844-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18746-0">
    <skos:related rdf:resource="descriptor/18496-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18746-0">
    <skos:related rdf:resource="descriptor/13181-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15965-3">
    <skos:prefLabel xml:lang="en">Natural law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15965-3">
    <skos:prefLabel xml:lang="de">Naturrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18266-0">
    <skos:broader rdf:resource="thsys/70461"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18266-0">
    <skos:broader rdf:resource="descriptor/14626-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18381-0">
    <skos:altLabel xml:lang="de">Rohstoffökonomie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18381-0">
    <skos:altLabel xml:lang="de">Ressourcenökonomie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18381-0">
    <skos:altLabel xml:lang="de">Ökonomie der Naturressourcen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18381-0">
    <skos:altLabel xml:lang="de">Ökonomie der natürlichen Ressourcen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18381-0">
    <skos:altLabel xml:lang="de">Intergenerative Ressourcennutzung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13357-1">
    <skos:broader rdf:resource="thsys/70828"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13357-1">
    <skos:broader rdf:resource="descriptor/18920-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11070-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13964-1">
    <skos:broader rdf:resource="thsys/71056"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13964-1">
    <skos:broader rdf:resource="thsys/71052"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13964-1">
    <skos:broader rdf:resource="descriptor/13960-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70147">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13018-2">
    <skos:scopeNote xml:lang="de">Bergbau auf Mineralien, die weder zu den Metallen noch zu den Energierohstoffen zählen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18487-5">
    <skos:broader rdf:resource="thsys/71005"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18487-5">
    <skos:broader rdf:resource="descriptor/11322-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19332-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18571-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10137-2">
    <skos:prefLabel xml:lang="en">Theory of interest</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10137-2">
    <skos:prefLabel xml:lang="de">Zinstheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12165-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13839-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10143-0">
    <skos:related rdf:resource="descriptor/19495-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10143-0">
    <skos:related rdf:resource="descriptor/11213-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10143-0">
    <skos:related rdf:resource="descriptor/18140-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10143-0">
    <skos:related rdf:resource="descriptor/11184-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10143-0">
    <skos:related rdf:resource="descriptor/13406-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10143-0">
    <skos:related rdf:resource="descriptor/24623-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15405-6">
    <skos:prefLabel xml:lang="en">Multivariate analysis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15405-6">
    <skos:prefLabel xml:lang="de">Multivariate Analyse</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12725-3">
    <skos:broader rdf:resource="thsys/70310"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72185">
    <skos:narrower rdf:resource="descriptor/15176-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72185">
    <skos:narrower rdf:resource="descriptor/15060-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72185">
    <skos:narrower rdf:resource="descriptor/15225-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72185">
    <skos:narrower rdf:resource="descriptor/19572-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72185">
    <skos:narrower rdf:resource="descriptor/19571-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72185">
    <skos:narrower rdf:resource="descriptor/15220-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72185">
    <skos:narrower rdf:resource="descriptor/15163-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16063-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14793-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11090-4">
    <skos:related rdf:resource="descriptor/11101-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11090-4">
    <skos:related rdf:resource="descriptor/11060-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11090-4">
    <skos:related rdf:resource="descriptor/20289-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11090-4">
    <skos:related rdf:resource="descriptor/11174-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11090-4">
    <skos:related rdf:resource="descriptor/11061-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11090-4">
    <skos:related rdf:resource="descriptor/11088-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11090-4">
    <skos:related rdf:resource="descriptor/10348-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11090-4">
    <skos:related rdf:resource="descriptor/11083-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10085-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16990-6">
    <skos:narrower rdf:resource="descriptor/16989-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16990-6">
    <skos:narrower rdf:resource="descriptor/16985-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16990-6">
    <skos:narrower rdf:resource="descriptor/16984-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16990-6">
    <skos:narrower rdf:resource="descriptor/16982-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16990-6">
    <skos:narrower rdf:resource="descriptor/16995-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14028-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11271-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18940-4">
    <skos:altLabel xml:lang="de">Optische Archivierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18940-4">
    <skos:altLabel xml:lang="de">CD-ROM-Archiv</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10798-0">
    <skos:related rdf:resource="descriptor/12292-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10798-0">
    <skos:related rdf:resource="descriptor/10786-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10798-0">
    <skos:related rdf:resource="descriptor/10811-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10798-0">
    <skos:related rdf:resource="descriptor/10823-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10798-0">
    <skos:related rdf:resource="descriptor/10797-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11850-5">
    <skos:narrower rdf:resource="descriptor/18635-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14755-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19732-6">
    <skos:prefLabel xml:lang="en">Universalism</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19732-6">
    <skos:prefLabel xml:lang="de">Universalismus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14466-5">
    <skos:altLabel xml:lang="de">Steckdose</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14466-5">
    <skos:altLabel xml:lang="de">Relais</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14466-5">
    <skos:altLabel xml:lang="de">Elektrostecker</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14466-5">
    <skos:altLabel xml:lang="de">Elektroinstallationsmaterial</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14466-5">
    <skos:altLabel xml:lang="de">Elektrischer Schalter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14466-5">
    <skos:altLabel xml:lang="de">Elektrische Sicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18204-1">
    <skos:altLabel xml:lang="de">Restholz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18204-1">
    <skos:altLabel xml:lang="de">Altholz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18204-1">
    <skos:altLabel xml:lang="de">Abfallholz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15463-6">
    <skos:prefLabel xml:lang="en">Bargaining theory</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15463-6">
    <skos:prefLabel xml:lang="de">Verhandlungstheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19413-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11739-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19252-6">
    <skos:broader rdf:resource="thsys/71109"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19252-6">
    <skos:broader rdf:resource="thsys/71070"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19252-6">
    <skos:broader rdf:resource="descriptor/18082-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70355">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.05.03</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16633-2">
    <skos:altLabel xml:lang="de">Staatsgesinnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16633-2">
    <skos:altLabel xml:lang="de">Politisches Bewußtsein</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16633-2">
    <skos:altLabel xml:lang="de">Politisches Bewusstsein</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16633-2">
    <skos:altLabel xml:lang="de">Kollektivbewußtsein</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16633-2">
    <skos:altLabel xml:lang="de">Gesellschaftliches Bewußtsein</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18693-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19773-6">
    <skos:broader rdf:resource="thsys/70834"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19648-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72393">
    <skos:prefLabel xml:lang="en">V.16.07  Econometrics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72393">
    <skos:prefLabel xml:lang="de">V.16.07  Ökonometrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13445-4">
    <skos:altLabel xml:lang="de">Landsoziologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13445-4">
    <skos:altLabel xml:lang="de">Ländliche Soziologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11661-1">
    <skos:altLabel xml:lang="de">Indirekte Besteuerung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11994-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13389-2">
    <skos:altLabel xml:lang="en">Legal firm</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13389-2">
    <skos:altLabel xml:lang="en">Legal advice</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13389-2">
    <skos:altLabel xml:lang="en">Law firm</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13389-2">
    <skos:altLabel xml:lang="de">Rechtsanwaltskanzlei</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13389-2">
    <skos:altLabel xml:lang="de">Notariat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13389-2">
    <skos:altLabel xml:lang="de">Anwaltssozietät</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13389-2">
    <skos:altLabel xml:lang="de">Anwaltskanzlei</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14900-1">
    <skos:prefLabel xml:lang="en">Accessories</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14900-1">
    <skos:prefLabel xml:lang="de">Bekleidungszubehör</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16449-2">
    <skos:prefLabel xml:lang="en">Nuclear disarmament</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16449-2">
    <skos:prefLabel xml:lang="de">Atomare Abrüstung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11715-4">
    <skos:narrower rdf:resource="descriptor/19436-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11715-4">
    <skos:narrower rdf:resource="descriptor/18156-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11715-4">
    <skos:narrower rdf:resource="descriptor/11718-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11715-4">
    <skos:narrower rdf:resource="descriptor/19212-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11715-4">
    <skos:narrower rdf:resource="descriptor/19374-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11715-4">
    <skos:narrower rdf:resource="descriptor/11711-5"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id130">
    <rdf:value xml:lang="de">Für betriebswirtschaftliche Fragestellungen allein</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15017-1">
    <skos:prefLabel xml:lang="en">Hazardous waste</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15017-1">
    <skos:prefLabel xml:lang="de">Sonderabfall</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17706-3">
    <skos:prefLabel xml:lang="en">Guinea-Bissau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17706-3">
    <skos:prefLabel xml:lang="de">Guinea-Bissau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19209-0">
    <skos:prefLabel xml:lang="en">Intertemporal allocation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19209-0">
    <skos:prefLabel xml:lang="de">Intertemporale Allokation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15467-5">
    <skos:altLabel xml:lang="de">Risiko-Entscheidungsmodell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15467-5">
    <skos:altLabel xml:lang="de">Entscheidung unter Risiko</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70233">
    <skos:narrower rdf:resource="descriptor/10114-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70233">
    <skos:narrower rdf:resource="descriptor/10239-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70233">
    <skos:narrower rdf:resource="descriptor/10027-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70233">
    <skos:narrower rdf:resource="descriptor/19250-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70233">
    <skos:narrower rdf:resource="descriptor/10328-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70233">
    <skos:narrower rdf:resource="descriptor/10334-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70233">
    <skos:narrower rdf:resource="descriptor/10332-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70233">
    <skos:narrower rdf:resource="descriptor/10329-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70233">
    <skos:narrower rdf:resource="descriptor/10460-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70233">
    <skos:narrower rdf:resource="descriptor/11787-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70233">
    <skos:narrower rdf:resource="descriptor/15105-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70233">
    <skos:narrower rdf:resource="descriptor/12504-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70233">
    <skos:narrower rdf:resource="descriptor/10327-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70233">
    <skos:narrower rdf:resource="descriptor/10326-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70233">
    <skos:narrower rdf:resource="descriptor/10016-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70233">
    <skos:narrower rdf:resource="descriptor/19012-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70233">
    <skos:narrower rdf:resource="descriptor/10349-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70233">
    <skos:narrower rdf:resource="descriptor/10320-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70233">
    <skos:narrower rdf:resource="descriptor/10187-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70233">
    <skos:narrower rdf:resource="descriptor/10122-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70233">
    <skos:narrower rdf:resource="descriptor/10256-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70233">
    <skos:narrower rdf:resource="descriptor/19231-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70233">
    <skos:narrower rdf:resource="descriptor/10315-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70233">
    <skos:narrower rdf:resource="descriptor/10186-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70233">
    <skos:narrower rdf:resource="descriptor/10185-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70233">
    <skos:narrower rdf:resource="descriptor/10314-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70233">
    <skos:narrower rdf:resource="descriptor/10313-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70233">
    <skos:narrower rdf:resource="descriptor/10312-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70233">
    <skos:narrower rdf:resource="descriptor/10307-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70233">
    <skos:narrower rdf:resource="descriptor/10305-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70233">
    <skos:narrower rdf:resource="descriptor/11186-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70233">
    <skos:narrower rdf:resource="descriptor/10440-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70233">
    <skos:narrower rdf:resource="descriptor/19566-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70233">
    <skos:narrower rdf:resource="descriptor/10304-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70233">
    <skos:narrower rdf:resource="descriptor/10303-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14888-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13056-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19647-4">
    <skos:scopeNote xml:lang="de">Abwanderung bzw. Wegzug von einem inländischen Wohnort an einen anderen</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10094-1">
    <skos:narrower rdf:resource="descriptor/10111-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10094-1">
    <skos:narrower rdf:resource="descriptor/10091-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19468-4">
    <skos:altLabel xml:lang="de">Städtische Wirtschaftsentwicklung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19624-2">
    <skos:altLabel xml:lang="de">Wasserschaden</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19624-2">
    <skos:altLabel xml:lang="de">Tsunami</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19624-2">
    <skos:altLabel xml:lang="de">Sturmflut</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19624-2">
    <skos:altLabel xml:lang="de">Hochwasser</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19624-2">
    <skos:altLabel xml:lang="de">Flutwelle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19624-2">
    <skos:altLabel xml:lang="de">Flutkatastrophe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14712-2">
    <skos:broader rdf:resource="thsys/70476"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14712-2">
    <skos:broader rdf:resource="descriptor/14710-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18650-6">
    <skos:prefLabel xml:lang="en">Telephone industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18650-6">
    <skos:prefLabel xml:lang="de">Telefongesellschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18943-5">
    <skos:broader rdf:resource="thsys/71117"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18943-5">
    <skos:broader rdf:resource="thsys/70823"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18943-5">
    <skos:broader rdf:resource="descriptor/10012-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19335-2">
    <skos:altLabel xml:lang="en">Exchange rate system</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70383">
    <skos:narrower rdf:resource="descriptor/18664-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70383">
    <skos:narrower rdf:resource="descriptor/16067-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70383">
    <skos:narrower rdf:resource="descriptor/12775-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70383">
    <skos:narrower rdf:resource="descriptor/19008-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70383">
    <skos:narrower rdf:resource="descriptor/19151-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70383">
    <skos:narrower rdf:resource="descriptor/15798-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70383">
    <skos:narrower rdf:resource="descriptor/19835-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70383">
    <skos:narrower rdf:resource="descriptor/20921-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70383">
    <skos:narrower rdf:resource="descriptor/12171-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70383">
    <skos:narrower rdf:resource="descriptor/15598-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70383">
    <skos:narrower rdf:resource="descriptor/10141-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70383">
    <skos:narrower rdf:resource="descriptor/10140-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70383">
    <skos:narrower rdf:resource="descriptor/15473-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70383">
    <skos:narrower rdf:resource="descriptor/16754-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70383">
    <skos:narrower rdf:resource="descriptor/19848-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70383">
    <skos:narrower rdf:resource="descriptor/12370-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70383">
    <skos:narrower rdf:resource="descriptor/12166-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70383">
    <skos:narrower rdf:resource="descriptor/16782-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70383">
    <skos:narrower rdf:resource="descriptor/15605-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70383">
    <skos:narrower rdf:resource="descriptor/15604-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70383">
    <skos:narrower rdf:resource="descriptor/18207-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70383">
    <skos:narrower rdf:resource="descriptor/12165-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70383">
    <skos:narrower rdf:resource="descriptor/20404-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70383">
    <skos:narrower rdf:resource="descriptor/12164-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18127-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11919-2">
    <skos:prefLabel xml:lang="en">Agglomeration effect</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11919-2">
    <skos:prefLabel xml:lang="de">Agglomerationseffekt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10326-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15746-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14789-2">
    <skos:altLabel xml:lang="de">Tischlerplatte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14789-2">
    <skos:altLabel xml:lang="de">Sperrholz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14789-2">
    <skos:altLabel xml:lang="de">Spanplatte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14789-2">
    <skos:altLabel xml:lang="de">Holzwerkstoffplatte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14789-2">
    <skos:altLabel xml:lang="de">Furnierplatte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14789-2">
    <skos:altLabel xml:lang="de">Faserplatte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16990-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18105-3">
    <skos:altLabel xml:lang="en">Superannuation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18105-3">
    <skos:altLabel xml:lang="en">Super fund</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18105-3">
    <skos:altLabel xml:lang="en">Pension scheme</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18105-3">
    <skos:altLabel xml:lang="en">Funded pension scheme</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18105-3">
    <skos:altLabel xml:lang="de">Pensionskasse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18105-3">
    <skos:altLabel xml:lang="de">Pensionsfonds-Verwaltung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18105-3">
    <skos:altLabel xml:lang="de">Pensionsfonds-Management</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12952-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18022-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19552-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18718-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10835-6">
    <skos:narrower rdf:resource="descriptor/15078-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17949-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19806-3">
    <skos:altLabel xml:lang="en">Graphic data processing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19806-3">
    <skos:altLabel xml:lang="de">3-D-Graphik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19806-3">
    <skos:altLabel xml:lang="de">Graphisches Kernsystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19806-3">
    <skos:altLabel xml:lang="de">Graphische Datenverarbeitung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19806-3">
    <skos:altLabel xml:lang="de">GKS (Graphisches Kernsystem)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19806-3">
    <skos:altLabel xml:lang="de">Computeranimation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10884-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11365-5">
    <skos:altLabel xml:lang="en">Traineeship</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11365-5">
    <skos:altLabel xml:lang="en">In-plant training</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11365-5">
    <skos:altLabel xml:lang="de">Lehrlingsausbildung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11365-5">
    <skos:altLabel xml:lang="de">Lehre im Betrieb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11365-5">
    <skos:altLabel xml:lang="de">Betriebliche Lehre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11365-5">
    <skos:altLabel xml:lang="de">Betriebliche Ausbildung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11365-5">
    <skos:altLabel xml:lang="de">Ausbildung in Unternehmen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19722-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10204-6">
    <skos:related rdf:resource="descriptor/10198-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70109">
    <rdfs:label xml:lang="en">Economic history</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70109">
    <rdfs:label xml:lang="de">Wirtschaftsgeschichte</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11168-0">
    <skos:related rdf:resource="descriptor/11723-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11168-0">
    <skos:related rdf:resource="descriptor/15458-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11168-0">
    <skos:related rdf:resource="descriptor/18847-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11168-0">
    <skos:related rdf:resource="descriptor/11164-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11168-0">
    <skos:related rdf:resource="descriptor/11337-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11168-0">
    <skos:related rdf:resource="descriptor/10984-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11168-0">
    <skos:related rdf:resource="descriptor/10968-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11168-0">
    <skos:related rdf:resource="descriptor/11161-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11168-0">
    <skos:related rdf:resource="descriptor/15472-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11168-0">
    <skos:related rdf:resource="descriptor/11152-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70832">
    <skos:broader rdf:resource="thsys/70835"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18340-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11641-0">
    <skos:broader rdf:resource="thsys/72181"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11641-0">
    <skos:broader rdf:resource="thsys/71077"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11641-0">
    <skos:broader rdf:resource="descriptor/11534-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16777-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16704-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20136-5">
    <skos:prefLabel xml:lang="en">Universal service</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20136-5">
    <skos:prefLabel xml:lang="de">Universaldienst</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18271-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11311-0">
    <skos:narrower rdf:resource="descriptor/20449-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11424-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16032-0">
    <skos:prefLabel xml:lang="en">Private international law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16032-0">
    <skos:prefLabel xml:lang="de">Internationales Privatrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12262-3">
    <skos:prefLabel xml:lang="en">Venture capital</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12262-3">
    <skos:prefLabel xml:lang="de">Risikokapital</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10498-5">
    <skos:broader rdf:resource="thsys/70504"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10498-5">
    <skos:broader rdf:resource="thsys/70120"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10498-5">
    <skos:broader rdf:resource="thsys/70091"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10498-5">
    <skos:broader rdf:resource="descriptor/19220-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19080-2">
    <skos:altLabel xml:lang="en">Capital invested</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15466-0">
    <skos:broader rdf:resource="thsys/73103"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15466-0">
    <skos:broader rdf:resource="thsys/70485"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19269-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18949-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11161-0">
    <skos:narrower rdf:resource="descriptor/11160-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11161-0">
    <skos:narrower rdf:resource="descriptor/19283-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11161-0">
    <skos:narrower rdf:resource="descriptor/19211-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10815-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12437-1">
    <skos:broader rdf:resource="thsys/73337"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12437-1">
    <skos:broader rdf:resource="thsys/71043"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12437-1">
    <skos:broader rdf:resource="thsys/70184"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12437-1">
    <skos:broader rdf:resource="descriptor/12408-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11321-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14600-6">
    <skos:prefLabel xml:lang="en">Chemical weapon</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14600-6">
    <skos:prefLabel xml:lang="de">Chemischer Kampfstoff</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16639-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15708-2">
    <skos:scopeNote xml:lang="de">Quantitative Veränderungen durch Wachstum oder Schrumpfung; auch Stabilisierungstendenzen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19464-5">
    <skos:prefLabel xml:lang="en">Rural industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19464-5">
    <skos:prefLabel xml:lang="de">Ländliche Industrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19244-5">
    <skos:scopeNote xml:lang="de">Die Annahme eines exogenen technischen Fortschritts wird ersetzt durch verschiedene endogene Wachstumsmechanismen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14849-3">
    <skos:prefLabel xml:lang="en">Fibre</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14849-3">
    <skos:prefLabel xml:lang="de">Faser</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14752-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10497-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19195-6">
    <skos:narrower rdf:resource="descriptor/13564-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19195-6">
    <skos:narrower rdf:resource="descriptor/10121-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18558-1">
    <skos:related rdf:resource="descriptor/18092-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10164-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10215-1">
    <skos:related rdf:resource="descriptor/20206-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10215-1">
    <skos:related rdf:resource="descriptor/10211-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10215-1">
    <skos:related rdf:resource="descriptor/19224-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10215-1">
    <skos:related rdf:resource="descriptor/10219-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10215-1">
    <skos:related rdf:resource="descriptor/10202-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10215-1">
    <skos:related rdf:resource="descriptor/10180-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12950-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12110-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14999-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19132-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71078">
    <skos:prefLabel xml:lang="de">V.09.06  Öffentliches Vermögen und öffentliche Schulden</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18480-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14737-0">
    <skos:altLabel xml:lang="de">Parfüm</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14737-0">
    <skos:altLabel xml:lang="de">Duftwasser</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14809-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11230-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12487-0">
    <skos:prefLabel xml:lang="en">Return on investment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12487-0">
    <skos:prefLabel xml:lang="de">Return on Investment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18081-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13003-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12090-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13425-3">
    <skos:related rdf:resource="descriptor/10511-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13425-3">
    <skos:related rdf:resource="descriptor/19455-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13425-3">
    <skos:related rdf:resource="descriptor/16250-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13425-3">
    <skos:related rdf:resource="descriptor/10521-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13425-3">
    <skos:related rdf:resource="descriptor/13445-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10510-6">
    <skos:related rdf:resource="descriptor/10512-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10510-6">
    <skos:related rdf:resource="descriptor/19363-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10510-6">
    <skos:related rdf:resource="descriptor/19477-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10510-6">
    <skos:related rdf:resource="descriptor/11892-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10510-6">
    <skos:related rdf:resource="descriptor/15695-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10510-6">
    <skos:related rdf:resource="descriptor/19468-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10249-5">
    <skos:related rdf:resource="descriptor/10272-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10249-5">
    <skos:related rdf:resource="descriptor/10252-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10249-5">
    <skos:related rdf:resource="descriptor/10250-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10249-5">
    <skos:related rdf:resource="descriptor/10041-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10249-5">
    <skos:related rdf:resource="descriptor/10247-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10249-5">
    <skos:related rdf:resource="descriptor/10242-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73335">
    <skos:prefLabel xml:lang="en">N.05.07  Social legislation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73335">
    <skos:prefLabel xml:lang="de">N.05.07  Sozialrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21072-0">
    <skos:altLabel xml:lang="en">Uninsured</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21072-0">
    <skos:altLabel xml:lang="de">Versicherungsgrad</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21072-0">
    <skos:altLabel xml:lang="de">Versicherungsdeckung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21072-0">
    <skos:altLabel xml:lang="de">Unterversicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21072-0">
    <skos:altLabel xml:lang="de">Risikodeckung (Versicherung)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21072-0">
    <skos:altLabel xml:lang="de">Gefahrendeckung (Versicherung)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13846-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13270-6">
    <skos:prefLabel xml:lang="en">Postal and telecommunications services</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13270-6">
    <skos:prefLabel xml:lang="de">Post und Telekommunikation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13689-4">
    <skos:altLabel xml:lang="de">Bankstatistik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70891">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.16.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11914-5">
    <skos:related rdf:resource="descriptor/19404-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11914-5">
    <skos:related rdf:resource="descriptor/18160-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11914-5">
    <skos:related rdf:resource="descriptor/11915-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11914-5">
    <skos:related rdf:resource="descriptor/19291-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11914-5">
    <skos:related rdf:resource="descriptor/11913-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11914-5">
    <skos:related rdf:resource="descriptor/19566-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12727-6">
    <skos:related rdf:resource="descriptor/12724-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12727-6">
    <skos:related rdf:resource="descriptor/12755-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12727-6">
    <skos:related rdf:resource="descriptor/12719-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12009-1">
    <skos:broader rdf:resource="thsys/71063"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12009-1">
    <skos:broader rdf:resource="thsys/70039"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12032-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13350-1">
    <skos:prefLabel xml:lang="en">Academic library</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13350-1">
    <skos:prefLabel xml:lang="de">Wissenschaftliche Bibliothek</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11147-1">
    <zbwext:useInsteadNote rdf:nodeID="id6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15859-2">
    <skos:broader rdf:resource="thsys/73384"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15859-2">
    <skos:broader rdf:resource="thsys/70814"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15859-2">
    <skos:broader rdf:resource="thsys/70384"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15859-2">
    <skos:broader rdf:resource="descriptor/15714-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19115-2">
    <skos:broader rdf:resource="thsys/71036"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19115-2">
    <skos:broader rdf:resource="thsys/70300"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19115-2">
    <skos:broader rdf:resource="descriptor/12165-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19115-2">
    <skos:broader rdf:resource="descriptor/12677-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11051-0">
    <skos:prefLabel xml:lang="en">War economy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11051-0">
    <skos:prefLabel xml:lang="de">Kriegswirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18251-6">
    <skos:prefLabel xml:lang="en">Optoelectronics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18251-6">
    <skos:prefLabel xml:lang="de">Optoelektronik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19378-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11063-0">
    <skos:related rdf:resource="descriptor/11068-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11063-0">
    <skos:related rdf:resource="descriptor/19285-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11063-0">
    <skos:related rdf:resource="descriptor/11039-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11063-0">
    <skos:related rdf:resource="descriptor/11049-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11063-0">
    <skos:related rdf:resource="descriptor/19607-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11063-0">
    <skos:related rdf:resource="descriptor/16292-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11063-0">
    <skos:related rdf:resource="descriptor/16209-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11063-0">
    <skos:related rdf:resource="descriptor/11066-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11063-0">
    <skos:related rdf:resource="descriptor/10976-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11063-0">
    <skos:related rdf:resource="descriptor/16215-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71114">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.03.04.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12259-6">
    <skos:prefLabel xml:lang="en">Self-financing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12259-6">
    <skos:prefLabel xml:lang="de">Selbstfinanzierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13849-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70851">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.23.02.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19902-0">
    <skos:related rdf:resource="descriptor/11212-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19902-0">
    <skos:related rdf:resource="descriptor/11258-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19902-0">
    <skos:related rdf:resource="descriptor/16131-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13141-3">
    <skos:prefLabel xml:lang="en">Textile craft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13141-3">
    <skos:prefLabel xml:lang="de">Textilhandwerk</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19622-6">
    <skos:altLabel xml:lang="de">Witterung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19622-6">
    <skos:altLabel xml:lang="de">Wetterabhängigkeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19622-6">
    <skos:altLabel xml:lang="de">Regen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19622-6">
    <skos:altLabel xml:lang="de">Niederschlag</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19622-6">
    <skos:altLabel xml:lang="de">Hagel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18766-1">
    <skos:related rdf:resource="descriptor/15624-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18766-1">
    <skos:related rdf:resource="descriptor/15610-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18766-1">
    <skos:related rdf:resource="descriptor/13390-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18766-1">
    <skos:related rdf:resource="descriptor/19805-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18766-1">
    <skos:related rdf:resource="descriptor/15621-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18766-1">
    <skos:related rdf:resource="descriptor/18661-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18766-1">
    <skos:related rdf:resource="descriptor/15619-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18766-1">
    <skos:related rdf:resource="descriptor/13987-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18766-1">
    <skos:related rdf:resource="descriptor/18172-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18741-3">
    <skos:prefLabel xml:lang="en">Economics of insurance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18741-3">
    <skos:prefLabel xml:lang="de">Versicherungsökonomik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11813-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18248-2">
    <skos:altLabel xml:lang="en">Dry bulk carrier</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14473-1">
    <skos:related rdf:resource="descriptor/18426-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14473-1">
    <skos:related rdf:resource="descriptor/19794-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14473-1">
    <skos:related rdf:resource="descriptor/13076-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14473-1">
    <skos:related rdf:resource="descriptor/13077-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14473-1">
    <skos:related rdf:resource="descriptor/13202-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14473-1">
    <skos:related rdf:resource="descriptor/14431-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14993-3">
    <skos:broader rdf:resource="thsys/73299"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14993-3">
    <skos:broader rdf:resource="descriptor/14989-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11141-6">
    <skos:altLabel xml:lang="de">Sozialistische Intensivierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11141-6">
    <skos:altLabel xml:lang="de">Reproduktionsschema</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11141-6">
    <skos:altLabel xml:lang="de">Intensiv erweiterte Reproduktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11141-6">
    <skos:altLabel xml:lang="de">Grundfondsreproduktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11141-6">
    <skos:altLabel xml:lang="de">Erweiterte sozialistische Reproduktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11141-6">
    <skos:altLabel xml:lang="de">Erweiterte Reproduktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11141-6">
    <skos:altLabel xml:lang="de">Einfache Reproduktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13052-2">
    <skos:prefLabel xml:lang="en">Motor vehicle repair</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13052-2">
    <skos:prefLabel xml:lang="de">Kraftfahrzeughandwerk</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19042-3">
    <skos:altLabel xml:lang="de">Vorhersage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19042-3">
    <skos:altLabel xml:lang="de">Voraussage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18510-1">
    <skos:altLabel xml:lang="de">Handwerkergenossenschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18833-5">
    <skos:related rdf:resource="descriptor/18834-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11504-3">
    <skos:prefLabel xml:lang="en">Indexation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11504-3">
    <skos:prefLabel xml:lang="de">Indexierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14302-0">
    <skos:related rdf:resource="descriptor/13037-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14302-0">
    <skos:related rdf:resource="descriptor/12988-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70056">
    <skos:narrower rdf:resource="thsys/70176"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70056">
    <skos:narrower rdf:resource="thsys/70573"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70056">
    <skos:narrower rdf:resource="thsys/70504"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19112-1">
    <skos:broader rdf:resource="thsys/71058"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19112-1">
    <skos:broader rdf:resource="thsys/70244"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19755-1">
    <skos:broader rdf:resource="thsys/70120"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11154-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13746-4">
    <skos:prefLabel xml:lang="en">Bank liquidity</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13746-4">
    <skos:prefLabel xml:lang="de">Bankenliquidität</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18522-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18809-2">
    <skos:broader rdf:resource="thsys/70160"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18809-2">
    <skos:broader rdf:resource="descriptor/16007-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15714-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19660-5">
    <skos:editorialNote xml:lang="de">Möglichst spezifischer indexieren.</skos:editorialNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14948-1">
    <skos:related rdf:resource="descriptor/13161-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14948-1">
    <skos:related rdf:resource="descriptor/14980-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14948-1">
    <skos:related rdf:resource="descriptor/14110-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14948-1">
    <skos:related rdf:resource="descriptor/14105-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15473-3">
    <skos:altLabel xml:lang="en">Procurement of information</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15473-3">
    <skos:altLabel xml:lang="de">Informationsgewinnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70944">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12108-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10845-3">
    <skos:broader rdf:resource="thsys/73334"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10845-3">
    <skos:broader rdf:resource="thsys/71096"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10845-3">
    <skos:broader rdf:resource="descriptor/16112-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13558-5">
    <skos:altLabel xml:lang="de">Kraftverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13558-5">
    <skos:altLabel xml:lang="de">Automobilverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15743-0">
    <skos:altLabel xml:lang="en">Smog</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15743-0">
    <skos:altLabel xml:lang="de">Staub</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15743-0">
    <skos:altLabel xml:lang="de">Schwebestaub</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15743-0">
    <skos:altLabel xml:lang="de">Rauch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15743-0">
    <skos:altLabel xml:lang="de">Luftverschmutzung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14057-1">
    <skos:prefLabel xml:lang="en">Flax</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14057-1">
    <skos:prefLabel xml:lang="de">Flachs</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18376-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18488-3">
    <skos:prefLabel xml:lang="en">Industrial managers</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18488-3">
    <skos:prefLabel xml:lang="de">Industriemanager</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18277-2">
    <skos:broader rdf:resource="thsys/70201"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10635-0">
    <skos:related rdf:resource="descriptor/10612-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16943-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12988-6">
    <skos:related rdf:resource="descriptor/13037-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12988-6">
    <skos:related rdf:resource="descriptor/14204-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12988-6">
    <skos:related rdf:resource="descriptor/14302-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12331-3">
    <skos:broader rdf:resource="thsys/73331"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12331-3">
    <skos:broader rdf:resource="thsys/71043"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12331-3">
    <skos:broader rdf:resource="descriptor/16182-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12331-3">
    <skos:broader rdf:resource="descriptor/12326-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20409-3">
    <skos:prefLabel xml:lang="en">Drugs economy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20409-3">
    <skos:prefLabel xml:lang="de">Drogenwirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12246-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14094-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70184">
    <rdfs:label xml:lang="de">Steuerrechtliche Vermögensaufstellung</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71070">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.11.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70549">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12841-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16267-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18527-5">
    <skos:broader rdf:resource="thsys/70956"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11834-3">
    <skos:related rdf:resource="descriptor/13590-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11834-3">
    <skos:related rdf:resource="descriptor/11833-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11834-3">
    <skos:related rdf:resource="descriptor/13264-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11834-3">
    <skos:related rdf:resource="descriptor/11934-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11834-3">
    <skos:related rdf:resource="descriptor/10211-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15952-5">
    <skos:related rdf:resource="descriptor/16615-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15952-5">
    <skos:related rdf:resource="descriptor/15956-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15952-5">
    <skos:related rdf:resource="descriptor/19649-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13722-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15787-1">
    <skos:altLabel xml:lang="de">Persönlichkeitstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15787-1">
    <skos:altLabel xml:lang="de">Persönlichkeitsstruktur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15787-1">
    <skos:altLabel xml:lang="de">Persönlichkeitsstärke</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15787-1">
    <skos:altLabel xml:lang="de">Persönliche Identität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15787-1">
    <skos:altLabel xml:lang="de">Identitätsfindung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15787-1">
    <skos:altLabel xml:lang="de">Identitätsbildung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15787-1">
    <skos:altLabel xml:lang="de">Identität (Person)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15753-4">
    <skos:related rdf:resource="descriptor/19490-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15753-4">
    <skos:related rdf:resource="descriptor/15754-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15753-4">
    <skos:related rdf:resource="descriptor/15752-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15753-4">
    <skos:related rdf:resource="descriptor/18995-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15753-4">
    <skos:related rdf:resource="descriptor/14047-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19458-0">
    <skos:related rdf:resource="descriptor/19459-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19458-0">
    <skos:related rdf:resource="descriptor/11862-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19558-3">
    <skos:prefLabel xml:lang="en">Travel cost model</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19558-3">
    <skos:prefLabel xml:lang="de">Reisekostenansatz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19173-2">
    <skos:altLabel xml:lang="de">CACOM-Staaten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19173-2">
    <skos:altLabel xml:lang="de">CACOM-Länder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19173-2">
    <skos:altLabel xml:lang="de">CACM-Länder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19161-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11717-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13966-4">
    <skos:prefLabel xml:lang="en">Touristic professions</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13966-4">
    <skos:prefLabel xml:lang="de">Fremdenverkehrsberufe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10021-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18745-2">
    <skos:broader rdf:resource="thsys/71066"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18745-2">
    <skos:broader rdf:resource="thsys/70889"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18745-2">
    <skos:broader rdf:resource="descriptor/10213-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18119-6">
    <skos:scopeNote xml:lang="de">Von einer Muttergesellschaft abhängige Kapitalgesellschaft, deren Kapital zum großen Teil, meist zu 100 %, im Besitz der herrschenden Gesellschaft ist.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id88">
    <rdf:value xml:lang="de">Für konkrete Entwicklungspläne</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19314-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16623-5">
    <skos:prefLabel xml:lang="en">Social justice</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16623-5">
    <skos:prefLabel xml:lang="de">Soziale Gerechtigkeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13804-2">
    <skos:altLabel xml:lang="en">Traveller's cheque</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13804-2">
    <skos:altLabel xml:lang="de">Scheckverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13804-2">
    <skos:altLabel xml:lang="de">Scheckkarte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13804-2">
    <skos:altLabel xml:lang="de">Reisescheck</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13804-2">
    <skos:altLabel xml:lang="de">Euroscheckverfahren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13804-2">
    <skos:altLabel xml:lang="de">ec-Karte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13949-4">
    <skos:altLabel xml:lang="de">Tabakindustriearbeiterinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13949-4">
    <skos:altLabel xml:lang="de">Tabakindustriearbeiter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13949-4">
    <skos:altLabel xml:lang="de">Schnapsbrennerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13949-4">
    <skos:altLabel xml:lang="de">Schnapsbrenner</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13949-4">
    <skos:altLabel xml:lang="de">Molkereifachleute</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13949-4">
    <skos:altLabel xml:lang="de">Köchinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13949-4">
    <skos:altLabel xml:lang="de">Köche</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13949-4">
    <skos:altLabel xml:lang="de">Getränkeindustriearbeiterinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13949-4">
    <skos:altLabel xml:lang="de">Getränkeindustriearbeiter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13949-4">
    <skos:altLabel xml:lang="de">Fleischerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13949-4">
    <skos:altLabel xml:lang="de">Fleischer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13949-4">
    <skos:altLabel xml:lang="de">Bäckerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13949-4">
    <skos:altLabel xml:lang="de">Bäcker</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13451-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15644-2">
    <skos:prefLabel xml:lang="en">Humanities</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15644-2">
    <skos:prefLabel xml:lang="de">Geisteswissenschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11681-2">
    <skos:narrower rdf:resource="descriptor/19424-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11681-2">
    <skos:narrower rdf:resource="descriptor/11678-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11681-2">
    <skos:narrower rdf:resource="descriptor/11667-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11681-2">
    <skos:narrower rdf:resource="descriptor/11664-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11681-2">
    <skos:narrower rdf:resource="descriptor/11658-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11681-2">
    <skos:narrower rdf:resource="descriptor/11651-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18909-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19384-3">
    <skos:altLabel xml:lang="en">Voucher schemes</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19032-6">
    <skos:broader rdf:resource="thsys/70582"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13545-0">
    <skos:broader rdf:resource="thsys/70936"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13545-0">
    <skos:broader rdf:resource="thsys/70926"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13545-0">
    <skos:broader rdf:resource="descriptor/11833-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18010-0">
    <skos:related rdf:resource="descriptor/11790-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18010-0">
    <skos:related rdf:resource="descriptor/11784-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19051-2">
    <skos:broader rdf:resource="thsys/70346"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19051-2">
    <skos:broader rdf:resource="descriptor/14269-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11640-2">
    <skos:related rdf:resource="descriptor/16122-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11640-2">
    <skos:related rdf:resource="descriptor/11637-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11640-2">
    <skos:related rdf:resource="descriptor/11642-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11640-2">
    <skos:related rdf:resource="descriptor/16087-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11640-2">
    <skos:related rdf:resource="descriptor/11704-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11640-2">
    <skos:related rdf:resource="descriptor/16112-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11640-2">
    <skos:related rdf:resource="descriptor/11708-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11640-2">
    <skos:related rdf:resource="descriptor/12889-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11640-2">
    <skos:related rdf:resource="descriptor/11547-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11640-2">
    <skos:related rdf:resource="descriptor/16050-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11640-2">
    <skos:related rdf:resource="descriptor/11536-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13390-3">
    <skos:related rdf:resource="descriptor/18761-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13390-3">
    <skos:related rdf:resource="descriptor/15621-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13390-3">
    <skos:related rdf:resource="descriptor/18766-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19230-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10776-3">
    <skos:broader rdf:resource="thsys/71103"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10776-3">
    <skos:broader rdf:resource="thsys/71095"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10776-3">
    <skos:broader rdf:resource="thsys/71088"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10776-3">
    <skos:broader rdf:resource="descriptor/11415-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13370-2">
    <skos:broader rdf:resource="thsys/71053"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13370-2">
    <skos:broader rdf:resource="thsys/71052"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13370-2">
    <skos:broader rdf:resource="thsys/70839"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13370-2">
    <skos:broader rdf:resource="descriptor/18890-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13370-2">
    <skos:broader rdf:resource="descriptor/11356-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15703-5">
    <skos:broader rdf:resource="thsys/73387"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15703-5">
    <skos:broader rdf:resource="thsys/71005"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15703-5">
    <skos:broader rdf:resource="thsys/70109"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15703-5">
    <skos:broader rdf:resource="descriptor/15706-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19302-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18983-0">
    <skos:altLabel xml:lang="en">Farm income</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18983-0">
    <skos:altLabel xml:lang="de">Landwirtschaftliches Einkommen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18983-0">
    <skos:altLabel xml:lang="de">Einkommen von Landwirten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18983-0">
    <skos:altLabel xml:lang="de">Bäuerliches Einkommen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18980-6">
    <skos:related rdf:resource="descriptor/14054-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18980-6">
    <skos:related rdf:resource="descriptor/18983-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18404-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11563-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12337-5">
    <skos:altLabel xml:lang="en">Generally accepted accounting principles</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12337-5">
    <skos:altLabel xml:lang="de">US-GAAP</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12337-5">
    <skos:altLabel xml:lang="de">Stetigkeitsprinzip</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12337-5">
    <skos:altLabel xml:lang="de">Rechnungslegungsstandards (GoB)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12337-5">
    <skos:altLabel xml:lang="de">Ordnungsmäßigkeit der Buchführung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12337-5">
    <skos:altLabel xml:lang="de">Grundsätze ordnungsmäßiger Bilanzierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12337-5">
    <skos:altLabel xml:lang="de">GoB (Grundsätze ordnungsmäßiger Buchführung)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12337-5">
    <skos:altLabel xml:lang="de">Buchführungsgrundsätze</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12337-5">
    <skos:altLabel xml:lang="de">Bilanzwahrheit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12337-5">
    <skos:altLabel xml:lang="de">Bilanzkontinuität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12337-5">
    <skos:altLabel xml:lang="de">Bilanzklarheit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12337-5">
    <skos:altLabel xml:lang="de">Bilanzierungsgrundsätze</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12337-5">
    <skos:altLabel xml:lang="de">Bilanzidentität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11590-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11915-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18365-5">
    <skos:broader rdf:resource="thsys/73331"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18365-5">
    <skos:broader rdf:resource="descriptor/11580-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18365-5">
    <skos:broader rdf:resource="descriptor/16182-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18233-1">
    <skos:altLabel xml:lang="de">Grundwasserressourcen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70834">
    <skos:broader rdf:resource="thsys/70835"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14463-4">
    <skos:broader rdf:resource="thsys/70415"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14463-4">
    <skos:broader rdf:resource="descriptor/13513-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71032">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">G.02.05</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10150-3">
    <skos:prefLabel xml:lang="en">Altruism</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10150-3">
    <skos:prefLabel xml:lang="de">Altruismus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16353-1">
    <skos:broader rdf:resource="thsys/73376"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16353-1">
    <skos:broader rdf:resource="thsys/70013"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16353-1">
    <skos:broader rdf:resource="descriptor/18859-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17171-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15988-5">
    <skos:altLabel xml:lang="de">TA-Abfall</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15988-5">
    <skos:altLabel xml:lang="de">Kreislaufwirtschaftsgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15988-5">
    <skos:altLabel xml:lang="de">AbfG (Abfallgesetz)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15988-5">
    <skos:altLabel xml:lang="de">Abfallgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15988-5">
    <skos:altLabel xml:lang="de">Abfallbeseitigungsgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15949-1">
    <skos:broader rdf:resource="thsys/73347"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15949-1">
    <skos:broader rdf:resource="thsys/70182"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20948-1">
    <skos:altLabel xml:lang="en">Intrapreneurship</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20948-1">
    <skos:altLabel xml:lang="en">Intraorganizational entrepreneuring</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20948-1">
    <skos:altLabel xml:lang="en">Corporate entrepreneurship</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20948-1">
    <skos:altLabel xml:lang="de">Internes Unternehmertum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16112-2">
    <skos:prefLabel xml:lang="en">Tax fraud</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16112-2">
    <skos:prefLabel xml:lang="de">Steuerkriminalität</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70144">
    <skos:prefLabel xml:lang="de">V.09  Öffentliche Finanzen und Finanzwissenschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16556-1">
    <skos:scopeNote xml:lang="de">Wissen von den und Erforschung der Kulturen vorwiegend der sogenannten Naturvölker. Studien zum interkulturellen Vergleich (Cross-Cultural Studies).</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10838-0">
    <skos:prefLabel xml:lang="en">Import</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10838-0">
    <skos:prefLabel xml:lang="de">Import</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19185-2">
    <skos:broader rdf:resource="thsys/71007"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19185-2">
    <skos:broader rdf:resource="thsys/70957"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19185-2">
    <skos:broader rdf:resource="descriptor/19184-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73354">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.10.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12720-6">
    <skos:altLabel xml:lang="en">Group technology</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12720-6">
    <skos:altLabel xml:lang="de">Produktionsinsel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12720-6">
    <skos:altLabel xml:lang="de">Inselfertigung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12720-6">
    <skos:altLabel xml:lang="de">Gruppenfertigung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11452-3">
    <skos:related rdf:resource="descriptor/11473-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11452-3">
    <skos:related rdf:resource="descriptor/11426-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11452-3">
    <skos:related rdf:resource="descriptor/11449-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11168-0">
    <skos:altLabel xml:lang="de">Verbandstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11168-0">
    <skos:altLabel xml:lang="de">Theorie der Gruppenbildung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11168-0">
    <skos:altLabel xml:lang="de">Ökonomische Theorie des kollektiven Handelns</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11168-0">
    <skos:altLabel xml:lang="de">Ökonomische Theorie der Verbände</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11168-0">
    <skos:altLabel xml:lang="de">Ökonomische Theorie der Koalitionen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11168-0">
    <skos:altLabel xml:lang="de">Ökonomische Theorie der Gruppen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11168-0">
    <skos:altLabel xml:lang="de">Ökonomische Theorie der Gruppe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11168-0">
    <skos:altLabel xml:lang="de">Interessengruppentheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18368-6">
    <skos:prefLabel xml:lang="en">Reactor safety</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18368-6">
    <skos:prefLabel xml:lang="de">Reaktorsicherheit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11069-2">
    <skos:prefLabel xml:lang="en">Economic constitution</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11069-2">
    <skos:prefLabel xml:lang="de">Wirtschaftsverfassung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19467-6">
    <skos:related rdf:resource="descriptor/10575-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19467-6">
    <skos:related rdf:resource="descriptor/19468-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19467-6">
    <skos:related rdf:resource="descriptor/11852-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11235-4">
    <skos:broader rdf:resource="thsys/73372"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11235-4">
    <skos:broader rdf:resource="thsys/71061"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11235-4">
    <skos:broader rdf:resource="descriptor/10987-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70988">
    <skos:narrower rdf:resource="descriptor/13142-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70988">
    <skos:narrower rdf:resource="descriptor/13137-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70988">
    <skos:narrower rdf:resource="descriptor/13136-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70988">
    <skos:narrower rdf:resource="descriptor/13135-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70988">
    <skos:narrower rdf:resource="descriptor/13904-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19284-0">
    <skos:broader rdf:resource="thsys/70039"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19284-0">
    <skos:broader rdf:resource="thsys/70038"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19284-0">
    <skos:broader rdf:resource="descriptor/10979-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14524-3">
    <skos:broader rdf:resource="thsys/70373"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14524-3">
    <skos:broader rdf:resource="descriptor/14525-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14524-3">
    <skos:broader rdf:resource="descriptor/14513-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12236-4">
    <skos:broader rdf:resource="thsys/73392"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12236-4">
    <skos:broader rdf:resource="thsys/71045"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12236-4">
    <skos:broader rdf:resource="thsys/70899"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12236-4">
    <skos:broader rdf:resource="descriptor/12246-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10342-1">
    <zbwext:useInsteadNote rdf:nodeID="id374"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10342-1">
    <zbwext:useInsteadNote rdf:nodeID="id373"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10342-1">
    <zbwext:useInsteadNote rdf:nodeID="id372"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70935">
    <skos:broader rdf:resource="thsys/70936"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18244-3">
    <skos:altLabel xml:lang="de">Spindel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10831-0">
    <skos:prefLabel xml:lang="en">Tax haven</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10831-0">
    <skos:prefLabel xml:lang="de">Steueroase</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11709-6">
    <skos:broader rdf:resource="thsys/73337"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11709-6">
    <skos:broader rdf:resource="thsys/71075"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11709-6">
    <skos:broader rdf:resource="thsys/70046"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10000-1">
    <zbwext:useInsteadNote rdf:nodeID="id13"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10000-1">
    <zbwext:useInsteadNote rdf:nodeID="id12"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14915-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14205-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12072-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15970-3">
    <skos:prefLabel xml:lang="en">Legal policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15970-3">
    <skos:prefLabel xml:lang="de">Rechtspolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13844-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16985-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15874-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11701-1">
    <skos:related rdf:resource="descriptor/19404-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11701-1">
    <skos:related rdf:resource="descriptor/11653-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11701-1">
    <skos:related rdf:resource="descriptor/11904-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16402-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14692-3">
    <skos:altLabel xml:lang="de">Ethylenbenzol</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14692-3">
    <skos:altLabel xml:lang="de">Äthylenbenzol</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11507-4">
    <skos:prefLabel xml:lang="en">Inflation rate</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11507-4">
    <skos:prefLabel xml:lang="de">Inflationsrate</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73347">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14817-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13209-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19738-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14964-3">
    <skos:prefLabel xml:lang="en">Meat product</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14964-3">
    <skos:prefLabel xml:lang="de">Fleischwaren</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10815-5">
    <skos:broader rdf:resource="thsys/71089"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10815-5">
    <skos:broader rdf:resource="descriptor/10811-6"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id289">
    <rdfs:seeAlso rdf:resource="descriptor/16070-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18195-4">
    <skos:scopeNote xml:lang="de">Kaffee, Kakao, Tee, Gewürze, Zucker, Bananen, Ölpflanzen, Baumwolle, Kautschuk, Tropenhölzer usw. können zu den tropischen Erzeugnissen gerechnet werden.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14306-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70082">
    <skos:narrower rdf:resource="descriptor/19561-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70082">
    <skos:narrower rdf:resource="descriptor/19625-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70082">
    <skos:narrower rdf:resource="thsys/70209"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70082">
    <skos:narrower rdf:resource="thsys/70227"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70082">
    <skos:narrower rdf:resource="thsys/70811"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18121-5">
    <skos:altLabel xml:lang="en">Cooperative venture</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18121-5">
    <skos:altLabel xml:lang="de">Internationale Zusammenarbeit von Unternehmen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18121-5">
    <skos:altLabel xml:lang="de">Internationale Unternehmenszusammenarbeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19183-6">
    <skos:prefLabel xml:lang="en">Milk market</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19183-6">
    <skos:prefLabel xml:lang="de">Milchmarkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13279-2">
    <skos:editorialNote xml:lang="de">Möglichst spezifischer indexieren.</skos:editorialNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70943">
    <skos:narrower rdf:resource="descriptor/13572-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70943">
    <skos:narrower rdf:resource="descriptor/13577-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70943">
    <skos:narrower rdf:resource="descriptor/13564-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70943">
    <skos:narrower rdf:resource="descriptor/13570-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70943">
    <skos:narrower rdf:resource="descriptor/13569-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70943">
    <skos:narrower rdf:resource="descriptor/13513-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70943">
    <skos:narrower rdf:resource="descriptor/15777-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70943">
    <skos:narrower rdf:resource="descriptor/12013-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70943">
    <skos:narrower rdf:resource="descriptor/13264-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70943">
    <skos:narrower rdf:resource="descriptor/18565-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70943">
    <skos:narrower rdf:resource="descriptor/13979-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70943">
    <skos:narrower rdf:resource="descriptor/13583-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70943">
    <skos:narrower rdf:resource="descriptor/13510-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70943">
    <skos:narrower rdf:resource="descriptor/18090-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70943">
    <skos:narrower rdf:resource="descriptor/13509-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70943">
    <skos:narrower rdf:resource="descriptor/13259-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70943">
    <skos:narrower rdf:resource="descriptor/13508-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70943">
    <skos:narrower rdf:resource="descriptor/11629-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70943">
    <skos:narrower rdf:resource="descriptor/13504-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70943">
    <skos:narrower rdf:resource="descriptor/13502-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70943">
    <skos:narrower rdf:resource="descriptor/18567-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70943">
    <skos:narrower rdf:resource="descriptor/20897-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70943">
    <skos:narrower rdf:resource="descriptor/13501-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70943">
    <skos:narrower rdf:resource="descriptor/19066-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70943">
    <skos:narrower rdf:resource="descriptor/18566-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70943">
    <skos:narrower rdf:resource="descriptor/13498-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70943">
    <skos:narrower rdf:resource="descriptor/19610-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70943">
    <skos:narrower rdf:resource="thsys/70929"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70943">
    <skos:narrower rdf:resource="thsys/70924"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70943">
    <skos:narrower rdf:resource="thsys/70925"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70943">
    <skos:narrower rdf:resource="thsys/70942"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70943">
    <skos:narrower rdf:resource="thsys/70940"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70943">
    <skos:narrower rdf:resource="thsys/70939"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70943">
    <skos:narrower rdf:resource="thsys/70928"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14144-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11069-2">
    <skos:related rdf:resource="descriptor/16081-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11069-2">
    <skos:related rdf:resource="descriptor/11068-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11069-2">
    <skos:related rdf:resource="descriptor/11057-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11069-2">
    <skos:related rdf:resource="descriptor/16016-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11069-2">
    <skos:related rdf:resource="descriptor/12090-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11069-2">
    <skos:related rdf:resource="descriptor/10976-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11069-2">
    <skos:related rdf:resource="descriptor/12009-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11069-2">
    <skos:related rdf:resource="descriptor/11040-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11069-2">
    <skos:related rdf:resource="descriptor/16136-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18828-5">
    <skos:broader rdf:resource="thsys/70858"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18828-5">
    <skos:broader rdf:resource="thsys/70242"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18828-5">
    <skos:broader rdf:resource="descriptor/10975-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18828-5">
    <skos:broader rdf:resource="descriptor/11066-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70799">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16697-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11902-5">
    <skos:related rdf:resource="descriptor/11976-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11902-5">
    <skos:related rdf:resource="descriptor/11993-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11902-5">
    <skos:related rdf:resource="descriptor/11983-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11902-5">
    <skos:related rdf:resource="descriptor/19477-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11902-5">
    <skos:related rdf:resource="descriptor/10504-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16564-2">
    <skos:related rdf:resource="descriptor/15814-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16564-2">
    <skos:related rdf:resource="descriptor/15811-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16564-2">
    <skos:related rdf:resource="descriptor/15723-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16564-2">
    <skos:related rdf:resource="descriptor/16209-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11504-3">
    <skos:altLabel xml:lang="de">Zinsgleitklausel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11504-3">
    <skos:altLabel xml:lang="de">Wertsicherungsklausel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11504-3">
    <skos:altLabel xml:lang="de">Wertsicherungsindex</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11504-3">
    <skos:altLabel xml:lang="de">Währungsklausel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11504-3">
    <skos:altLabel xml:lang="de">Teuerungsausgleich</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11504-3">
    <skos:altLabel xml:lang="de">Preisgleitklausel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11504-3">
    <skos:altLabel xml:lang="de">Indexklausel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11504-3">
    <skos:altLabel xml:lang="de">Indexbindung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11504-3">
    <skos:altLabel xml:lang="de">Goldklausel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11504-3">
    <skos:altLabel xml:lang="de">Geldwertsicherungsklausel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70147">
    <skos:prefLabel xml:lang="en">G.02.02  East Asia</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70147">
    <skos:prefLabel xml:lang="de">G.02.02  Ostasien</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16547-2">
    <skos:prefLabel xml:lang="en">Social security contribution</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16547-2">
    <skos:prefLabel xml:lang="de">Sozialversicherungsbeitrag</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15396-2">
    <skos:narrower rdf:resource="descriptor/19570-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15396-2">
    <skos:narrower rdf:resource="descriptor/15392-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14630-4">
    <skos:prefLabel xml:lang="en">Textile finishing chemical</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14630-4">
    <skos:prefLabel xml:lang="de">Textilhilfsmittel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70005">
    <skos:broader rdf:resource="thsys/b"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14227-2">
    <skos:prefLabel xml:lang="en">Precious stone</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14227-2">
    <skos:prefLabel xml:lang="de">Edelstein</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10748-1">
    <skos:altLabel xml:lang="de">Japanischer Yen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70862">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12961-5">
    <skos:narrower rdf:resource="descriptor/19365-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12961-5">
    <skos:narrower rdf:resource="descriptor/18324-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19680-6">
    <skos:prefLabel xml:lang="en">Communitarianism</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19680-6">
    <skos:prefLabel xml:lang="de">Kommunitarismus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11504-3">
    <skos:scopeNote xml:lang="de">Gewährung eines Inflationsausgleichs zur Minderung der Inflationswirkung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10329-0">
    <skos:related rdf:resource="descriptor/10433-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19591-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14202-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13735-2">
    <skos:altLabel xml:lang="en">Put option</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13735-2">
    <skos:altLabel xml:lang="en">Exotic options</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13735-2">
    <skos:altLabel xml:lang="en">Call option</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13735-2">
    <skos:altLabel xml:lang="de">Optionsmarkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13735-2">
    <skos:altLabel xml:lang="de">Optionshandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14603-0">
    <skos:narrower rdf:resource="descriptor/18254-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14603-0">
    <skos:narrower rdf:resource="descriptor/14196-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14603-0">
    <skos:narrower rdf:resource="descriptor/14631-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15968-4">
    <skos:related rdf:resource="descriptor/15723-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15968-4">
    <skos:related rdf:resource="descriptor/15976-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15968-4">
    <skos:related rdf:resource="descriptor/15966-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18502-0">
    <skos:prefLabel xml:lang="en">Housing finance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18502-0">
    <skos:prefLabel xml:lang="de">Wohnungsbaufinanzierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10312-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13213-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19760-1">
    <skos:altLabel xml:lang="de">Alter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70244">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18124-6">
    <skos:broader rdf:resource="thsys/71060"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18124-6">
    <skos:broader rdf:resource="descriptor/11182-6"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id83">
    <rdfs:seeAlso rdf:resource="descriptor/13076-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12967-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17534-6">
    <skos:scopeNote xml:lang="de">Russland erklärte 1990 seine staatliche Souveränität und gründete im Dezember 1991 mit Weißrussland und der Ukraine die GUS.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10728-0">
    <skos:related rdf:resource="descriptor/10735-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19461-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19113-6">
    <skos:altLabel xml:lang="en">Contracting out</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19113-6">
    <skos:altLabel xml:lang="de">Auslagerung betrieblicher Leistungen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19113-6">
    <skos:altLabel xml:lang="de">Ausgliederung betrieblicher Leistungen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12112-1">
    <skos:narrower rdf:resource="descriptor/15825-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73387">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18081-5">
    <skos:broader rdf:resource="thsys/70922"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18081-5">
    <skos:broader rdf:resource="thsys/70135"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18081-5">
    <skos:broader rdf:resource="descriptor/13667-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18081-5">
    <skos:broader rdf:resource="descriptor/18666-5"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id212">
    <rdfs:seeAlso rdf:resource="descriptor/12774-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12354-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70109">
    <skos:prefLabel xml:lang="en">V.15  Economic history</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70109">
    <skos:prefLabel xml:lang="de">V.15  Wirtschaftsgeschichte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13371-0">
    <skos:broader rdf:resource="thsys/70839"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13371-0">
    <skos:broader rdf:resource="thsys/70573"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71094">
    <skos:broader rdf:resource="thsys/71095"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12254-2">
    <skos:broader rdf:resource="thsys/71045"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12254-2">
    <skos:broader rdf:resource="thsys/70883"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10758-5">
    <skos:altLabel xml:lang="en">SDR (Special Drawing Rights)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10758-5">
    <skos:altLabel xml:lang="de">SZR (Sonderziehungsrechte)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14034-6">
    <skos:prefLabel xml:lang="en">Potash fertilizer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14034-6">
    <skos:prefLabel xml:lang="de">Kalidünger</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19694-2">
    <skos:prefLabel xml:lang="en">Coalition government</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19694-2">
    <skos:prefLabel xml:lang="de">Regierungskoalition</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19674-1">
    <skos:prefLabel xml:lang="en">Agricultural price</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19674-1">
    <skos:prefLabel xml:lang="de">Agrarpreis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18763-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19316-6">
    <skos:prefLabel xml:lang="en">Rules of origin</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19316-6">
    <skos:prefLabel xml:lang="de">Herkunftsbezeichnung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20125-3">
    <skos:broader rdf:resource="thsys/71033"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20125-3">
    <skos:broader rdf:resource="thsys/70815"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20125-3">
    <skos:broader rdf:resource="descriptor/12881-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10603-6">
    <skos:altLabel xml:lang="de">Wirtschaftshilfe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10603-6">
    <skos:altLabel xml:lang="de">Internationale Finanzhilfe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10603-6">
    <skos:altLabel xml:lang="de">Auslandshilfe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19700-5">
    <skos:altLabel xml:lang="en">No-show paradox</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19700-5">
    <skos:altLabel xml:lang="en">Abstention paradox</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18574-3">
    <skos:broader rdf:resource="thsys/70941"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18574-3">
    <skos:broader rdf:resource="thsys/70939"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18574-3">
    <skos:broader rdf:resource="descriptor/13255-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18574-3">
    <skos:broader rdf:resource="descriptor/13632-5"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id47">
    <rdfs:seeAlso rdf:resource="descriptor/11361-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13006-2">
    <skos:altLabel xml:lang="de">Kunststoffwarenindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19495-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11228-1">
    <skos:related rdf:resource="descriptor/11365-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13066-5">
    <skos:related rdf:resource="descriptor/14463-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13066-5">
    <skos:related rdf:resource="descriptor/13576-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13066-5">
    <skos:related rdf:resource="descriptor/13256-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13066-5">
    <skos:related rdf:resource="descriptor/18417-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13066-5">
    <skos:related rdf:resource="descriptor/14317-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25743-1">
    <skos:altLabel xml:lang="en">Schumpeter's entrepreneur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25743-1">
    <skos:altLabel xml:lang="en">Schumpeterianism</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25743-1">
    <skos:altLabel xml:lang="en">Schumpeterian entrepreneur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25743-1">
    <skos:altLabel xml:lang="en">Schumpeterian competition</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25743-1">
    <skos:altLabel xml:lang="en">Creative entrepreneur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25743-1">
    <skos:altLabel xml:lang="de">Schumpeterscher Unternehmer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25743-1">
    <skos:altLabel xml:lang="de">Schöpferischer Unternehmer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16170-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13865-3">
    <skos:related rdf:resource="descriptor/13305-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13865-3">
    <skos:related rdf:resource="descriptor/13845-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13865-3">
    <skos:related rdf:resource="descriptor/13862-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18989-2">
    <skos:broader rdf:resource="thsys/71016"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18989-2">
    <skos:broader rdf:resource="thsys/70227"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18989-2">
    <skos:broader rdf:resource="descriptor/10010-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11367-1">
    <skos:altLabel xml:lang="en">Educational system</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11367-1">
    <skos:altLabel xml:lang="de">Erziehungswesen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11367-1">
    <skos:altLabel xml:lang="de">Erziehungssystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11367-1">
    <skos:altLabel xml:lang="de">Bildungssystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19527-0">
    <skos:altLabel xml:lang="de">Transplantationsrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19527-0">
    <skos:altLabel xml:lang="de">Transplantationsmedizin</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19527-0">
    <skos:altLabel xml:lang="de">Organverpflanzung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19527-0">
    <skos:altLabel xml:lang="de">Organübertragung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19527-0">
    <skos:altLabel xml:lang="de">Organspende</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19527-0">
    <skos:altLabel xml:lang="de">Organbank</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19527-0">
    <skos:altLabel xml:lang="de">Nierentransplantation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19527-0">
    <skos:altLabel xml:lang="de">Herztransplantation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16602-6">
    <skos:related rdf:resource="descriptor/11068-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16602-6">
    <skos:related rdf:resource="descriptor/11127-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16602-6">
    <skos:related rdf:resource="descriptor/16597-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16602-6">
    <skos:related rdf:resource="descriptor/15723-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16602-6">
    <skos:related rdf:resource="descriptor/19607-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16602-6">
    <skos:related rdf:resource="descriptor/16292-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16602-6">
    <skos:related rdf:resource="descriptor/16579-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16602-6">
    <skos:related rdf:resource="descriptor/16588-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18576-6">
    <skos:altLabel xml:lang="de">Vorortbahn</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18576-6">
    <skos:altLabel xml:lang="de">Untergrundbahnverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18576-6">
    <skos:altLabel xml:lang="de">U-Bahnverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18576-6">
    <skos:altLabel xml:lang="de">Straßenbahnverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18576-6">
    <skos:altLabel xml:lang="de">Stadtschnellbahnverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18576-6">
    <skos:altLabel xml:lang="de">Stadtbahnverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18576-6">
    <skos:altLabel xml:lang="de">Schienengebundener Nahverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18576-6">
    <skos:altLabel xml:lang="de">S-Bahnverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18576-6">
    <skos:altLabel xml:lang="de">Kleinbahn</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18831-2">
    <skos:scopeNote xml:lang="de">Steuerliche Begünstigung gemeinnütziger Einrichtungen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10458-3">
    <skos:related rdf:resource="descriptor/18782-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11872-2">
    <skos:prefLabel xml:lang="en">Regional income inequality</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11872-2">
    <skos:prefLabel xml:lang="de">Regionale Einkommensverteilung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70029">
    <rdfs:label xml:lang="de">Produktionsplanung und Fertigungssteuerung</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13351-6">
    <skos:narrower rdf:resource="descriptor/13350-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13351-6">
    <skos:narrower rdf:resource="descriptor/13358-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13351-6">
    <skos:narrower rdf:resource="descriptor/16769-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18324-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12727-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14173-6">
    <skos:broader rdf:resource="thsys/70538"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14173-6">
    <skos:broader rdf:resource="descriptor/14176-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19688-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18659-2">
    <skos:prefLabel xml:lang="en">Internet</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18659-2">
    <skos:prefLabel xml:lang="de">Internet</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18063-0">
    <skos:broader rdf:resource="thsys/73370"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18063-0">
    <skos:broader rdf:resource="thsys/70839"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18063-0">
    <skos:broader rdf:resource="thsys/70573"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18063-0">
    <skos:broader rdf:resource="descriptor/15725-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18063-0">
    <skos:broader rdf:resource="descriptor/16479-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12889-1">
    <skos:prefLabel xml:lang="en">Tax consultancy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12889-1">
    <skos:prefLabel xml:lang="de">Steuerberatung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19822-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12112-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18654-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18693-2">
    <skos:related rdf:resource="descriptor/18087-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18693-2">
    <skos:related rdf:resource="descriptor/10135-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18693-2">
    <skos:related rdf:resource="descriptor/13722-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15976-5">
    <skos:related rdf:resource="descriptor/16233-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15976-5">
    <skos:related rdf:resource="descriptor/15971-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15976-5">
    <skos:related rdf:resource="descriptor/15970-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15976-5">
    <skos:related rdf:resource="descriptor/15969-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15976-5">
    <skos:related rdf:resource="descriptor/19607-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15976-5">
    <skos:related rdf:resource="descriptor/19213-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15976-5">
    <skos:related rdf:resource="descriptor/15968-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15976-5">
    <skos:related rdf:resource="descriptor/15966-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15976-5">
    <skos:related rdf:resource="descriptor/16196-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16026-2">
    <skos:broader rdf:resource="thsys/73392"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16026-2">
    <skos:broader rdf:resource="descriptor/16042-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10115-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11823-1">
    <skos:scopeNote xml:lang="de">Für Außenhandelspolitik und Internationale Handelspolitik BENUTZE die entsprechenden Deskriptoren.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11823-1">
    <skos:scopeNote xml:lang="de">Nur Binnenhandelspolitik bezüglich Groß- und Einzelhandel.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14821-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15861-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15923-5">
    <skos:related rdf:resource="descriptor/16424-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15923-5">
    <skos:related rdf:resource="descriptor/15648-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15923-5">
    <skos:related rdf:resource="descriptor/16400-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16214-1">
    <skos:prefLabel xml:lang="en">Future studies</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16214-1">
    <skos:prefLabel xml:lang="de">Zukunftsforschung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18238-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70844">
    <skos:prefLabel xml:lang="de">W.24.02  Hochschulen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19310-4">
    <skos:broader rdf:resource="thsys/71098"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19310-4">
    <skos:broader rdf:resource="descriptor/10682-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73357">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70080">
    <skos:prefLabel xml:lang="en">G.04.02  North America</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70080">
    <skos:prefLabel xml:lang="de">G.04.02  Nordamerika</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19204-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11711-5">
    <skos:prefLabel xml:lang="en">Intergovernmental fiscal relations</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11711-5">
    <skos:prefLabel xml:lang="de">Finanzpolitische Aufgabenverteilung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15637-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19786-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10307-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15643-4">
    <skos:prefLabel xml:lang="en">Football (Sport)</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15643-4">
    <skos:prefLabel xml:lang="de">Fußballsport</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10671-3">
    <skos:related rdf:resource="descriptor/24855-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10671-3">
    <skos:related rdf:resource="descriptor/10672-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19405-0">
    <skos:prefLabel xml:lang="en">History of taxation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19405-0">
    <skos:prefLabel xml:lang="de">Steuergeschichte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70212">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.10.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15881-2">
    <skos:related rdf:resource="descriptor/18069-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15881-2">
    <skos:related rdf:resource="descriptor/15880-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16595-5">
    <skos:altLabel xml:lang="de">Soziale Netzwerkanalyse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16595-5">
    <skos:altLabel xml:lang="de">Guanxi</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11580-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70580">
    <skos:broader rdf:resource="thsys/70060"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15976-5">
    <skos:altLabel xml:lang="de">Rechtstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15976-5">
    <skos:altLabel xml:lang="de">Rechtssystematik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15976-5">
    <skos:altLabel xml:lang="de">Rechtsdogmatik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15976-5">
    <skos:altLabel xml:lang="de">Juristische Methodenlehre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15976-5">
    <skos:altLabel xml:lang="de">Jurisprudenz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10147-6">
    <skos:related rdf:resource="descriptor/10060-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10147-6">
    <skos:related rdf:resource="descriptor/15063-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10147-6">
    <skos:related rdf:resource="descriptor/10140-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15727-5">
    <skos:prefLabel xml:lang="en">Waste disposal method</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15727-5">
    <skos:prefLabel xml:lang="de">Entsorgungstechnik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71076">
    <skos:prefLabel xml:lang="en">V.09.07.01  Theory of taxation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71076">
    <skos:prefLabel xml:lang="de">V.09.07.01  Steuertheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12747-0">
    <skos:narrower rdf:resource="descriptor/12761-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12747-0">
    <skos:narrower rdf:resource="descriptor/12748-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19506-1">
    <skos:broader rdf:resource="thsys/73356"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19506-1">
    <skos:broader rdf:resource="thsys/70504"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19506-1">
    <skos:broader rdf:resource="descriptor/16639-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70242">
    <skos:prefLabel xml:lang="en">V.10.02  Competition theory</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70242">
    <skos:prefLabel xml:lang="de">V.10.02  Wettbewerbstheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13127-4">
    <skos:prefLabel xml:lang="en">Cotton industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13127-4">
    <skos:prefLabel xml:lang="de">Baumwollindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15142-5">
    <skos:related rdf:resource="descriptor/19150-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11272-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19492-0">
    <skos:scopeNote xml:lang="de">Wanderung von Arbeitskräften zwischen inländischen Standorten.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70324">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16476-6">
    <skos:prefLabel xml:lang="en">Drug policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16476-6">
    <skos:prefLabel xml:lang="de">Drogenpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15894-0">
    <skos:related rdf:resource="descriptor/15903-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15894-0">
    <skos:related rdf:resource="descriptor/15893-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15894-0">
    <skos:related rdf:resource="descriptor/19620-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19492-0">
    <skos:broader rdf:resource="thsys/70572"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19492-0">
    <skos:broader rdf:resource="thsys/70058"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19492-0">
    <skos:broader rdf:resource="thsys/70051"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19492-0">
    <skos:broader rdf:resource="descriptor/11184-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#semanticRelation">
    <rdf:type rdf:resource="http://www.w3.org/1999/02/22-rdf-syntax-ns#Property"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#semanticRelation">
    <rdf:type rdf:resource="http://www.w3.org/2002/07/owl#ObjectProperty"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14023-4">
    <skos:narrower rdf:resource="descriptor/18275-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14023-4">
    <skos:narrower rdf:resource="descriptor/14024-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14023-4">
    <skos:narrower rdf:resource="descriptor/14750-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14023-4">
    <skos:narrower rdf:resource="descriptor/13671-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10905-4">
    <skos:prefLabel xml:lang="en">Market structure</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10905-4">
    <skos:prefLabel xml:lang="de">Marktstruktur</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10923-2">
    <skos:broader rdf:resource="thsys/70212"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10923-2">
    <skos:broader rdf:resource="thsys/70140"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10923-2">
    <skos:broader rdf:resource="descriptor/10942-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10923-2">
    <skos:broader rdf:resource="descriptor/10911-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15381-1">
    <skos:related rdf:resource="descriptor/15376-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16184-5">
    <skos:broader rdf:resource="thsys/73366"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16184-5">
    <skos:broader rdf:resource="thsys/73332"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16184-5">
    <skos:broader rdf:resource="descriptor/16188-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14141-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10996-3">
    <skos:broader rdf:resource="thsys/71101"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10996-3">
    <skos:broader rdf:resource="thsys/71061"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10996-3">
    <skos:broader rdf:resource="descriptor/11033-2"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id146">
    <rdfs:seeAlso rdf:resource="descriptor/13381-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15416-1">
    <skos:prefLabel xml:lang="en">Man-machine systems</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15416-1">
    <skos:prefLabel xml:lang="de">Mensch-Maschine-System</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15644-2">
    <skos:related rdf:resource="descriptor/16563-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15644-2">
    <skos:related rdf:resource="descriptor/15790-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15644-2">
    <skos:related rdf:resource="descriptor/19635-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15989-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10675-2">
    <skos:altLabel xml:lang="en">Income terms of trade</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10675-2">
    <skos:altLabel xml:lang="en">Factoral terms of trade</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10675-2">
    <skos:altLabel xml:lang="en">Double-factoral terms of trade</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10675-2">
    <skos:altLabel xml:lang="en">Commodity terms of trade</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10675-2">
    <skos:altLabel xml:lang="en">Barter terms of trade</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10675-2">
    <skos:altLabel xml:lang="de">Singer-Prebisch-These</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10675-2">
    <skos:altLabel xml:lang="de">Reales Austauschverhältnis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10675-2">
    <skos:altLabel xml:lang="de">Prebisch-Singer-These</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11350-4">
    <skos:altLabel xml:lang="en">Social partner</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11350-4">
    <skos:altLabel xml:lang="de">Tarifpartner</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11350-4">
    <skos:altLabel xml:lang="de">Tarifparteien</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11350-4">
    <skos:altLabel xml:lang="de">Sozialpartnerschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18325-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11443-4">
    <skos:related rdf:resource="descriptor/11473-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11443-4">
    <skos:related rdf:resource="descriptor/11430-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11443-4">
    <skos:related rdf:resource="descriptor/11444-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11443-4">
    <skos:related rdf:resource="descriptor/11411-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13270-6">
    <skos:narrower rdf:resource="descriptor/13278-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13270-6">
    <skos:narrower rdf:resource="descriptor/13274-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18360-1">
    <skos:prefLabel xml:lang="en">Electricity policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18360-1">
    <skos:prefLabel xml:lang="de">Elektrizitätspolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14164-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10294-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id309">
    <rdf:value xml:lang="de">Für den Arbeitsmarkt einer bestimmten Stadt</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10560-5">
    <skos:prefLabel xml:lang="en">Development policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10560-5">
    <skos:prefLabel xml:lang="de">Entwicklungspolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16252-0">
    <skos:prefLabel xml:lang="en">Military leadership</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16252-0">
    <skos:prefLabel xml:lang="de">Militärführung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17413-4">
    <skos:broader rdf:resource="thsys/70557"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17413-4">
    <skos:broader rdf:resource="descriptor/17199-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17413-4">
    <skos:broader rdf:resource="descriptor/17983-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14810-2">
    <skos:related rdf:resource="descriptor/13125-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14810-2">
    <skos:related rdf:resource="descriptor/14802-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14810-2">
    <skos:related rdf:resource="descriptor/14788-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14810-2">
    <skos:related rdf:resource="descriptor/14883-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70879">
    <rdfs:label xml:lang="de">Forschung und Entwicklung</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11973-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18387-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12916-3">
    <skos:prefLabel xml:lang="en">Capital goods industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12916-3">
    <skos:prefLabel xml:lang="de">Investitionsgüterindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18579-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19423-5">
    <skos:prefLabel xml:lang="en">Amusement tax</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19423-5">
    <skos:prefLabel xml:lang="de">Vergnügungsteuer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13420-6">
    <skos:prefLabel xml:lang="en">State</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13420-6">
    <skos:prefLabel xml:lang="de">Staat</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73384">
    <rdfs:label xml:lang="de">Werbe- und Konsumentenpsychologie</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14176-0">
    <skos:narrower rdf:resource="descriptor/14184-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14176-0">
    <skos:narrower rdf:resource="descriptor/14202-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14176-0">
    <skos:narrower rdf:resource="descriptor/14177-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14176-0">
    <skos:narrower rdf:resource="descriptor/14174-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14176-0">
    <skos:narrower rdf:resource="descriptor/14173-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14198-4">
    <skos:narrower rdf:resource="descriptor/14204-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14198-4">
    <skos:narrower rdf:resource="descriptor/14195-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70874">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.21.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16953-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19568-0">
    <skos:broader rdf:resource="thsys/72182"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19568-0">
    <skos:broader rdf:resource="descriptor/15064-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12136-1">
    <skos:narrower rdf:resource="descriptor/12135-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70187">
    <skos:prefLabel xml:lang="de">V.05.06  Finanzmärkte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id369">
    <rdf:value xml:lang="de">Für internationale regionale ökonomische Integration</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70970">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.07</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12488-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70096">
    <skos:prefLabel xml:lang="de">N.01  Philosophie, Wissenschaftstheorie und Religion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13769-6">
    <skos:prefLabel xml:lang="en">Transactions in securities</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13769-6">
    <skos:prefLabel xml:lang="de">Wertpapiergeschäft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70080">
    <skos:narrower rdf:resource="descriptor/17829-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70080">
    <skos:narrower rdf:resource="descriptor/17806-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70080">
    <skos:narrower rdf:resource="descriptor/17811-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18215-3">
    <skos:altLabel xml:lang="de">Zuckerersatzstoff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18215-3">
    <skos:altLabel xml:lang="de">Saccharin</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18215-3">
    <skos:altLabel xml:lang="de">Künstlicher Süßstoff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19902-0">
    <skos:narrower rdf:resource="descriptor/19491-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19902-0">
    <skos:narrower rdf:resource="descriptor/18792-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16619-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11956-3">
    <skos:related rdf:resource="descriptor/11912-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11956-3">
    <skos:related rdf:resource="descriptor/11959-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20972-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10813-2">
    <skos:altLabel xml:lang="de">Kapitalausfuhr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16782-6">
    <skos:narrower rdf:resource="descriptor/16780-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16782-6">
    <skos:narrower rdf:resource="descriptor/20404-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19227-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12055-1">
    <skos:altLabel xml:lang="x-other">Jae bul</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12055-1">
    <skos:altLabel xml:lang="en">General trading company</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12055-1">
    <skos:altLabel xml:lang="de">Sogo shosha</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12055-1">
    <skos:altLabel xml:lang="de">Mischkonzern</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12055-1">
    <skos:altLabel xml:lang="de">Konglomerate Konzentration</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12055-1">
    <skos:altLabel xml:lang="de">Jaebol</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12055-1">
    <skos:altLabel xml:lang="de">Generalhandelsgesellschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12055-1">
    <skos:altLabel xml:lang="de">Chaebol</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19587-3">
    <skos:broader rdf:resource="thsys/72181"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19587-3">
    <skos:broader rdf:resource="thsys/70020"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19587-3">
    <skos:broader rdf:resource="descriptor/15076-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18270-2">
    <skos:prefLabel xml:lang="en">Parallel computer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18270-2">
    <skos:prefLabel xml:lang="de">Parallelrechner</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id187">
    <rdfs:seeAlso rdf:resource="descriptor/18993-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13125-1">
    <skos:related rdf:resource="descriptor/14810-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19160-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13131-6">
    <skos:altLabel xml:lang="de">Matratzenindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14509-6">
    <skos:prefLabel xml:lang="en">Refrigerator</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14509-6">
    <skos:prefLabel xml:lang="de">Kühlschrank</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19266-2">
    <skos:scopeNote xml:lang="de">Regionale und internationale Zinsdifferenz.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18428-0">
    <skos:prefLabel xml:lang="en">Telecommunication equipment industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18428-0">
    <skos:prefLabel xml:lang="de">Telekommunikationsgeräteindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14101-3">
    <skos:related rdf:resource="descriptor/14149-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14101-3">
    <skos:related rdf:resource="descriptor/12935-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18411-3">
    <skos:related rdf:resource="descriptor/14379-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70012">
    <skos:broader rdf:resource="thsys/n"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16695-1">
    <skos:broader rdf:resource="thsys/73355"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16695-1">
    <skos:broader rdf:resource="descriptor/16564-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16853-2">
    <skos:scopeNote xml:lang="de">Nur für Literatur zum Gebiet der alten Bundesrepublik nach der Wiedervereinigung ab 1990</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14779-5">
    <skos:altLabel xml:lang="de">Trinkglas</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14779-5">
    <skos:altLabel xml:lang="de">Laborglas</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14779-5">
    <skos:altLabel xml:lang="de">Glasschmuck</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14779-5">
    <skos:altLabel xml:lang="de">Glasinstrument</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14779-5">
    <skos:altLabel xml:lang="de">Glasbläsereiwaren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14779-5">
    <skos:altLabel xml:lang="de">Bleikristallwaren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14042-0">
    <skos:prefLabel xml:lang="en">Medical equipment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14042-0">
    <skos:prefLabel xml:lang="de">Medizinisches Produkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15668-2">
    <skos:related rdf:resource="descriptor/14139-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18299-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16950-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24666-5">
    <skos:broader rdf:resource="thsys/71043"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24666-5">
    <skos:broader rdf:resource="descriptor/12331-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24666-5">
    <skos:broader rdf:resource="descriptor/12408-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15859-2">
    <skos:altLabel xml:lang="en">Life-style segmentation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15859-2">
    <skos:altLabel xml:lang="de">Werbezielgruppe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15859-2">
    <skos:altLabel xml:lang="de">Verbrauchertypologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15859-2">
    <skos:altLabel xml:lang="de">Verbrauchersegmentierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15859-2">
    <skos:altLabel xml:lang="de">Konsumententypologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11174-5">
    <skos:related rdf:resource="descriptor/11101-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11174-5">
    <skos:related rdf:resource="descriptor/11090-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11174-5">
    <skos:related rdf:resource="descriptor/11088-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19034-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19376-2">
    <skos:related rdf:resource="descriptor/18156-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19376-2">
    <skos:related rdf:resource="descriptor/10179-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19376-2">
    <skos:related rdf:resource="descriptor/11613-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19376-2">
    <skos:related rdf:resource="descriptor/11739-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19376-2">
    <skos:related rdf:resource="descriptor/16699-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18272-5">
    <skos:broader rdf:resource="thsys/73360"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18272-5">
    <skos:broader rdf:resource="thsys/70985"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18272-5">
    <skos:broader rdf:resource="descriptor/15988-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10605-2">
    <skos:scopeNote xml:lang="de">Gesamtheit der internationalen Wirtschafts-, Finanz- und Handelsbeziehungen, meist aus Sicht einer Volkswirtschaft betrachtet.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18010-0">
    <skos:altLabel xml:lang="de">Wirtschaftszweigentwicklung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18010-0">
    <skos:altLabel xml:lang="de">Sektorale Wirtschaftsentwicklung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18010-0">
    <skos:altLabel xml:lang="de">Sektorale Entwicklung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13528-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17586-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24624-0">
    <skos:prefLabel xml:lang="en">Business angels</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24624-0">
    <skos:prefLabel xml:lang="de">Business Angels</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10899-1">
    <skos:related rdf:resource="descriptor/10912-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73331">
    <rdfs:label xml:lang="en">Community law</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73331">
    <rdfs:label xml:lang="de">EU-Recht</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19672-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20733-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73339">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15878-5">
    <skos:broader rdf:resource="thsys/73381"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19096-1">
    <skos:narrower rdf:resource="descriptor/11598-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19096-1">
    <skos:narrower rdf:resource="descriptor/12489-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19096-1">
    <skos:narrower rdf:resource="descriptor/19097-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11987-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19181-3">
    <skos:broader rdf:resource="thsys/71006"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19181-3">
    <skos:broader rdf:resource="thsys/70992"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19181-3">
    <skos:broader rdf:resource="thsys/70957"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19181-3">
    <skos:broader rdf:resource="descriptor/18382-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14337-2">
    <skos:narrower rdf:resource="descriptor/14408-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14337-2">
    <skos:narrower rdf:resource="descriptor/14509-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14157-4">
    <skos:altLabel xml:lang="de">Teak</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14157-4">
    <skos:altLabel xml:lang="de">Eukalyptus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19746-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19340-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19517-3">
    <skos:related rdf:resource="descriptor/21002-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19517-3">
    <skos:related rdf:resource="descriptor/18063-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19517-3">
    <skos:related rdf:resource="descriptor/11157-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19517-3">
    <skos:related rdf:resource="descriptor/11555-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19517-3">
    <skos:related rdf:resource="descriptor/13415-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19439-4">
    <skos:prefLabel xml:lang="en">Regional unemployment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19439-4">
    <skos:prefLabel xml:lang="de">Regionale Arbeitslosigkeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16727-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19650-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16619-3">
    <skos:broader rdf:resource="thsys/73395"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16619-3">
    <skos:broader rdf:resource="descriptor/15717-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15026-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16426-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11676-2">
    <skos:related rdf:resource="descriptor/12416-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11857-5">
    <skos:narrower rdf:resource="descriptor/11916-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11857-5">
    <skos:narrower rdf:resource="descriptor/19701-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11857-5">
    <skos:narrower rdf:resource="descriptor/11917-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11857-5">
    <skos:narrower rdf:resource="descriptor/11991-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11857-5">
    <skos:narrower rdf:resource="descriptor/11855-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11857-5">
    <skos:narrower rdf:resource="descriptor/10503-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11857-5">
    <skos:narrower rdf:resource="descriptor/15883-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11857-5">
    <skos:narrower rdf:resource="descriptor/11853-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11857-5">
    <skos:narrower rdf:resource="descriptor/13409-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11857-5">
    <skos:narrower rdf:resource="descriptor/11979-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19523-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18084-6">
    <skos:broader rdf:resource="thsys/70899"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18084-6">
    <skos:broader rdf:resource="thsys/70037"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18084-6">
    <skos:broader rdf:resource="descriptor/12215-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12087-2">
    <skos:related rdf:resource="descriptor/12088-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18001-1">
    <skos:related rdf:resource="descriptor/19323-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18001-1">
    <skos:related rdf:resource="descriptor/19308-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18001-1">
    <skos:related rdf:resource="descriptor/10842-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18001-1">
    <skos:related rdf:resource="descriptor/10823-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18001-1">
    <skos:related rdf:resource="descriptor/10826-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18001-1">
    <skos:related rdf:resource="descriptor/10822-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18001-1">
    <skos:related rdf:resource="descriptor/19315-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18001-1">
    <skos:related rdf:resource="descriptor/10602-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18001-1">
    <skos:related rdf:resource="descriptor/10605-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14433-6">
    <skos:scopeNote xml:lang="de">Landfahrzeuge, die von eigener Maschinenkraft bewegt werden und nicht an Schienen gebunden sind.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15722-1">
    <skos:scopeNote xml:lang="de">Verhalten von Individuen gegenüber anderen Individuen bzw. in sozialen Zusammenhängen. Nicht nur für positives soziales Verhalten.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13249-4">
    <skos:related rdf:resource="descriptor/13613-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13249-4">
    <skos:related rdf:resource="descriptor/18576-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13249-4">
    <skos:related rdf:resource="descriptor/11876-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13249-4">
    <skos:related rdf:resource="descriptor/18610-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13249-4">
    <skos:related rdf:resource="descriptor/18800-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12511-1">
    <skos:related rdf:resource="descriptor/12160-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12511-1">
    <skos:related rdf:resource="descriptor/13413-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21094-4">
    <skos:prefLabel xml:lang="en">Contract farming</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21094-4">
    <skos:prefLabel xml:lang="de">Vertragsanbau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14733-1">
    <skos:prefLabel xml:lang="en">Skin care product</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14733-1">
    <skos:prefLabel xml:lang="de">Hautpflegemittel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20733-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18634-4">
    <skos:broader rdf:resource="thsys/70924"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18634-4">
    <skos:broader rdf:resource="descriptor/13639-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14962-0">
    <skos:broader rdf:resource="thsys/70467"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14962-0">
    <skos:broader rdf:resource="descriptor/14129-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14962-0">
    <skos:broader rdf:resource="descriptor/14044-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70418">
    <skos:narrower rdf:resource="descriptor/14211-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70418">
    <skos:narrower rdf:resource="descriptor/14205-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70418">
    <skos:narrower rdf:resource="descriptor/14193-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70418">
    <skos:narrower rdf:resource="descriptor/14204-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70418">
    <skos:narrower rdf:resource="descriptor/14202-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70418">
    <skos:narrower rdf:resource="descriptor/14200-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70418">
    <skos:narrower rdf:resource="descriptor/14638-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70418">
    <skos:narrower rdf:resource="descriptor/14199-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70418">
    <skos:narrower rdf:resource="descriptor/18236-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70418">
    <skos:narrower rdf:resource="descriptor/14198-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70418">
    <skos:narrower rdf:resource="descriptor/14197-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70418">
    <skos:narrower rdf:resource="descriptor/14196-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70418">
    <skos:narrower rdf:resource="descriptor/14195-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70418">
    <skos:narrower rdf:resource="descriptor/14194-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70418">
    <skos:narrower rdf:resource="descriptor/14192-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70418">
    <skos:narrower rdf:resource="descriptor/14190-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11970-2">
    <skos:related rdf:resource="descriptor/11991-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11970-2">
    <skos:related rdf:resource="descriptor/11990-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11970-2">
    <skos:related rdf:resource="descriptor/11993-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11970-2">
    <skos:related rdf:resource="descriptor/11987-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11970-2">
    <skos:related rdf:resource="descriptor/11986-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11970-2">
    <skos:related rdf:resource="descriptor/18638-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11970-2">
    <skos:related rdf:resource="descriptor/19476-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15715-5">
    <skos:prefLabel xml:lang="en">Social conditions</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15715-5">
    <skos:prefLabel xml:lang="de">Soziale Lage</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15655-4">
    <skos:prefLabel xml:lang="en">Arts</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15655-4">
    <skos:prefLabel xml:lang="de">Kunst</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11900-2">
    <skos:related rdf:resource="descriptor/13593-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11900-2">
    <skos:related rdf:resource="descriptor/16552-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11900-2">
    <skos:related rdf:resource="descriptor/11951-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11900-2">
    <skos:related rdf:resource="descriptor/11857-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11900-2">
    <skos:related rdf:resource="descriptor/11869-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11900-2">
    <skos:related rdf:resource="descriptor/12004-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11900-2">
    <skos:related rdf:resource="descriptor/10523-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11167-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71001">
    <rdfs:label xml:lang="en">Steel industry</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71001">
    <rdfs:label xml:lang="de">Stahlindustrie</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10804-3">
    <skos:altLabel xml:lang="en">Foreign currency bonds</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10804-3">
    <skos:altLabel xml:lang="de">Währungsanleihe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10804-3">
    <skos:altLabel xml:lang="de">Globalanleihe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10804-3">
    <skos:altLabel xml:lang="de">Auslandsanleihe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19050-4">
    <skos:related rdf:resource="descriptor/14258-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19050-4">
    <skos:related rdf:resource="descriptor/14229-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19050-4">
    <skos:related rdf:resource="descriptor/14249-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13210-3">
    <skos:broader rdf:resource="thsys/70963"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19528-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13416-4">
    <skos:altLabel xml:lang="en">Public pension</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13416-4">
    <skos:altLabel xml:lang="de">Staatliche Rentenversicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13416-4">
    <skos:altLabel xml:lang="de">Staatliche Alterssicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13416-4">
    <skos:altLabel xml:lang="de">Sozialrentenversicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13416-4">
    <skos:altLabel xml:lang="de">Soziale Rentenversicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13416-4">
    <skos:altLabel xml:lang="de">Rente (GRV)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13416-4">
    <skos:altLabel xml:lang="de">Hinterbliebenenversicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13416-4">
    <skos:altLabel xml:lang="de">GRV (Gesetzliche Rentenversicherung)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13416-4">
    <skos:altLabel xml:lang="de">Gesetzliche Altersrente</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13416-4">
    <skos:altLabel xml:lang="de">Altersrente (GRV)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10151-1">
    <skos:prefLabel xml:lang="en">Expected utility</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10151-1">
    <skos:prefLabel xml:lang="de">Erwartungsnutzen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10471-4">
    <skos:related rdf:resource="descriptor/10470-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10471-4">
    <skos:related rdf:resource="descriptor/15025-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10471-4">
    <skos:related rdf:resource="descriptor/15072-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10185-5">
    <skos:related rdf:resource="descriptor/10190-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10185-5">
    <skos:related rdf:resource="descriptor/10326-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10185-5">
    <skos:related rdf:resource="descriptor/10186-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15709-0">
    <skos:broader rdf:resource="thsys/70925"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15709-0">
    <skos:broader rdf:resource="thsys/70572"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17676-0">
    <skos:altLabel xml:lang="de">Somaliland</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17676-0">
    <skos:altLabel xml:lang="de">Demokratische Republik Somalia</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10180-1">
    <skos:prefLabel xml:lang="en">Consumers surplus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10180-1">
    <skos:prefLabel xml:lang="de">Konsumentenrente</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11716-2">
    <skos:broader rdf:resource="thsys/71063"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id305">
    <rdfs:seeAlso rdf:resource="descriptor/19468-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71032">
    <skos:prefLabel xml:lang="en">G.02.05  Central Asia</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71032">
    <skos:prefLabel xml:lang="de">G.02.05  Zentralasien</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13667-0">
    <skos:related rdf:resource="descriptor/13666-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13667-0">
    <skos:related rdf:resource="descriptor/13678-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13667-0">
    <skos:related rdf:resource="descriptor/18650-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14412-0">
    <skos:related rdf:resource="descriptor/18514-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18502-0">
    <skos:broader rdf:resource="thsys/70967"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18502-0">
    <skos:broader rdf:resource="thsys/70888"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18502-0">
    <skos:broader rdf:resource="descriptor/18750-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19234-1">
    <skos:related rdf:resource="descriptor/11872-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19234-1">
    <skos:related rdf:resource="descriptor/11911-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11160-2">
    <skos:related rdf:resource="descriptor/10060-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11160-2">
    <skos:related rdf:resource="descriptor/10190-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11160-2">
    <skos:related rdf:resource="descriptor/12112-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11160-2">
    <skos:related rdf:resource="descriptor/11066-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11160-2">
    <skos:related rdf:resource="descriptor/19211-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11160-2">
    <skos:related rdf:resource="descriptor/19204-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11160-2">
    <skos:related rdf:resource="descriptor/19596-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11160-2">
    <skos:related rdf:resource="descriptor/11065-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19671-0">
    <skos:related rdf:resource="descriptor/24733-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19671-0">
    <skos:related rdf:resource="descriptor/18676-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19671-0">
    <skos:related rdf:resource="descriptor/13715-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10172-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11811-1">
    <skos:narrower rdf:resource="descriptor/18508-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12135-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14100-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19293-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16035-1">
    <skos:prefLabel xml:lang="en">Law of Property</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16035-1">
    <skos:prefLabel xml:lang="de">Sachenrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id230">
    <rdfs:seeAlso rdf:resource="descriptor/10985-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18205-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19668-3">
    <skos:scopeNote xml:lang="de">Ökonomische Sicherheit eines Staates lässt sich als Indikator ausdrücken, in den u.a. Variablen für politische Stabilität, Ausmaß an Demokratie und Rechtsstaatlichkeit sowie Wahrscheinlichkeit politischer Unruhen eingehen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12081-0">
    <skos:altLabel xml:lang="de">Werkmeisterinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12081-0">
    <skos:altLabel xml:lang="de">Werkmeister</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12081-0">
    <skos:altLabel xml:lang="de">Industriemeisterinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12081-0">
    <skos:altLabel xml:lang="de">Industriemeister</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71033">
    <skos:narrower rdf:resource="descriptor/10111-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71033">
    <skos:narrower rdf:resource="descriptor/15770-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71033">
    <skos:narrower rdf:resource="descriptor/18165-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71033">
    <skos:narrower rdf:resource="descriptor/18186-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71033">
    <skos:narrower rdf:resource="descriptor/19090-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71033">
    <skos:narrower rdf:resource="descriptor/20125-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71033">
    <skos:narrower rdf:resource="descriptor/15763-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71033">
    <skos:narrower rdf:resource="descriptor/12605-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71033">
    <skos:narrower rdf:resource="descriptor/19561-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71033">
    <skos:narrower rdf:resource="descriptor/10058-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71033">
    <skos:narrower rdf:resource="descriptor/19160-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71033">
    <skos:narrower rdf:resource="descriptor/16638-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71033">
    <skos:narrower rdf:resource="descriptor/15737-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71033">
    <skos:narrower rdf:resource="descriptor/18349-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71033">
    <skos:narrower rdf:resource="descriptor/18962-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71033">
    <skos:narrower rdf:resource="descriptor/18960-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71033">
    <skos:narrower rdf:resource="descriptor/15009-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12778-3">
    <skos:related rdf:resource="descriptor/19140-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12778-3">
    <skos:related rdf:resource="descriptor/20002-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12778-3">
    <skos:related rdf:resource="descriptor/12770-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12778-3">
    <skos:related rdf:resource="descriptor/18527-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12778-3">
    <skos:related rdf:resource="descriptor/18007-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13058-4">
    <skos:altLabel xml:lang="de">Maschinenindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13058-4">
    <skos:altLabel xml:lang="de">Maschinenbau (Industrie)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12647-4">
    <skos:prefLabel xml:lang="en">Materials testing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12647-4">
    <skos:prefLabel xml:lang="de">Materialprüfung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19094-5">
    <skos:prefLabel xml:lang="en">Tangible assets</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19094-5">
    <skos:prefLabel xml:lang="de">Betriebliches Sachanlagevermögen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11993-4">
    <skos:altLabel xml:lang="de">Umland</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11993-4">
    <skos:altLabel xml:lang="de">Stadt-Umland-Beziehung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11993-4">
    <skos:altLabel xml:lang="de">Stadtumland</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17011-3">
    <skos:altLabel xml:lang="de">Fürstentum Andorra</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11124-6">
    <skos:prefLabel xml:lang="en">State property</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11124-6">
    <skos:prefLabel xml:lang="de">Staatseigentum</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70938">
    <rdfs:label xml:lang="en">Road transport</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70938">
    <rdfs:label xml:lang="de">Straßenverkehr</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70985">
    <skos:broader rdf:resource="thsys/71004"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18702-6">
    <skos:broader rdf:resource="thsys/73390"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18702-6">
    <skos:broader rdf:resource="thsys/70187"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18702-6">
    <skos:broader rdf:resource="descriptor/13731-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18702-6">
    <skos:broader rdf:resource="descriptor/12277-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18702-6">
    <skos:broader rdf:resource="descriptor/16050-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10826-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70066">
    <skos:broader rdf:resource="thsys/p"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10148-4">
    <zbwext:useInsteadNote rdf:nodeID="id387"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10491-5">
    <skos:broader rdf:resource="thsys/70572"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10491-5">
    <skos:broader rdf:resource="thsys/70091"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10491-5">
    <skos:broader rdf:resource="thsys/70051"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10491-5">
    <skos:broader rdf:resource="descriptor/18869-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10491-5">
    <skos:broader rdf:resource="descriptor/10613-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15977-3">
    <skos:broader rdf:resource="thsys/70876"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15977-3">
    <skos:broader rdf:resource="thsys/70862"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15977-3">
    <skos:broader rdf:resource="thsys/70154"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15977-3">
    <skos:broader rdf:resource="descriptor/13410-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11404-0">
    <skos:prefLabel xml:lang="en">School policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11404-0">
    <skos:prefLabel xml:lang="de">Schulpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11840-1">
    <skos:related rdf:resource="descriptor/11848-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11840-1">
    <skos:related rdf:resource="descriptor/13304-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11840-1">
    <skos:related rdf:resource="descriptor/16033-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11840-1">
    <skos:related rdf:resource="descriptor/13839-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11840-1">
    <skos:related rdf:resource="descriptor/11839-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11840-1">
    <skos:related rdf:resource="descriptor/18755-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11840-1">
    <skos:related rdf:resource="descriptor/13837-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10612-5">
    <skos:related rdf:resource="descriptor/10628-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10612-5">
    <skos:related rdf:resource="descriptor/10714-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10612-5">
    <skos:related rdf:resource="descriptor/10627-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10612-5">
    <skos:related rdf:resource="descriptor/19309-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10612-5">
    <skos:related rdf:resource="descriptor/18181-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10612-5">
    <skos:related rdf:resource="descriptor/19351-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10612-5">
    <skos:related rdf:resource="descriptor/19317-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10612-5">
    <skos:related rdf:resource="descriptor/10635-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14123-0">
    <skos:related rdf:resource="descriptor/14131-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10471-4">
    <skos:prefLabel xml:lang="en">Technological forecasting</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10471-4">
    <skos:prefLabel xml:lang="de">Technologieprognose</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19025-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70938">
    <skos:prefLabel xml:lang="en">W.12.01.02  Road transport</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70938">
    <skos:prefLabel xml:lang="de">W.12.01.02  Straßenverkehr</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17920-1">
    <skos:prefLabel xml:lang="en">Ecuador</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17920-1">
    <skos:prefLabel xml:lang="de">Ecuador</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11274-1">
    <skos:related rdf:resource="descriptor/11302-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11274-1">
    <skos:related rdf:resource="descriptor/16658-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11274-1">
    <skos:related rdf:resource="descriptor/11356-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19390-1">
    <skos:altLabel xml:lang="de">Eisenmarkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12403-4">
    <zbwext:useInsteadNote rdf:nodeID="id352"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10074-0">
    <skos:broader rdf:resource="thsys/70049"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10074-0">
    <skos:broader rdf:resource="descriptor/10065-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15859-2">
    <skos:prefLabel xml:lang="en">Target group</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15859-2">
    <skos:prefLabel xml:lang="de">Zielgruppe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10625-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12872-4">
    <skos:broader rdf:resource="thsys/71040"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12872-4">
    <skos:broader rdf:resource="thsys/70934"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14439-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10091-0">
    <skos:broader rdf:resource="thsys/70057"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10091-0">
    <skos:broader rdf:resource="descriptor/10094-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16966-3">
    <skos:prefLabel xml:lang="en">Lucerne</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16966-3">
    <skos:prefLabel xml:lang="de">Luzern</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18919-2">
    <skos:altLabel xml:lang="de">Kino</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18919-2">
    <skos:altLabel xml:lang="de">Filmtheater</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11704-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19524-6">
    <skos:prefLabel xml:lang="en">Shock</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19524-6">
    <skos:prefLabel xml:lang="de">Schock</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12631-5">
    <skos:narrower rdf:resource="descriptor/19148-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14534-0">
    <skos:prefLabel xml:lang="en">Photocopy machine</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14534-0">
    <skos:prefLabel xml:lang="de">Kopiergerät</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70328">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19611-4">
    <skos:prefLabel xml:lang="en">Social geography</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19611-4">
    <skos:prefLabel xml:lang="de">Sozialgeographie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16521-6">
    <zbwext:useInsteadNote rdf:nodeID="id229"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71051">
    <rdfs:label xml:lang="de">Arbeitsrecht und Arbeitsbeziehungen</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17935-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70816">
    <rdfs:label xml:lang="en">Recycling</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70816">
    <rdfs:label xml:lang="de">Recycling</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19334-4">
    <skos:prefLabel xml:lang="en">Realignment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19334-4">
    <skos:prefLabel xml:lang="de">Realignment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73395">
    <skos:prefLabel xml:lang="de">N.06.02.02  Soziale Schichtung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16355-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19975-1">
    <skos:related rdf:resource="descriptor/16595-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19975-1">
    <skos:related rdf:resource="descriptor/16207-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19975-1">
    <skos:related rdf:resource="descriptor/16430-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19975-1">
    <skos:related rdf:resource="descriptor/16193-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19975-1">
    <skos:related rdf:resource="descriptor/10968-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13422-2">
    <skos:prefLabel xml:lang="en">Constitutional court</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13422-2">
    <skos:prefLabel xml:lang="de">Verfassungsgericht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19358-4">
    <skos:related rdf:resource="descriptor/10022-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19358-4">
    <skos:related rdf:resource="descriptor/19379-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12341-0">
    <skos:related rdf:resource="descriptor/12340-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12947-6">
    <skos:prefLabel xml:lang="en">Fruit crops</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12947-6">
    <skos:prefLabel xml:lang="de">Obstbau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18889-6">
    <skos:prefLabel xml:lang="en">EU educational policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18889-6">
    <skos:prefLabel xml:lang="de">EU-Bildungspolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18775-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18971-0">
    <skos:broader rdf:resource="thsys/73357"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18971-0">
    <skos:broader rdf:resource="thsys/70814"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18971-0">
    <skos:broader rdf:resource="thsys/70142"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18971-0">
    <skos:broader rdf:resource="descriptor/15909-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10565-2">
    <skos:related rdf:resource="descriptor/10614-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10565-2">
    <skos:related rdf:resource="descriptor/10570-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10565-2">
    <skos:related rdf:resource="descriptor/10415-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10565-2">
    <skos:related rdf:resource="descriptor/10836-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10565-2">
    <skos:related rdf:resource="descriptor/10684-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10824-4">
    <skos:broader rdf:resource="thsys/73392"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10824-4">
    <skos:broader rdf:resource="thsys/73332"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10824-4">
    <skos:broader rdf:resource="thsys/71098"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10824-4">
    <skos:broader rdf:resource="descriptor/16184-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10824-4">
    <skos:broader rdf:resource="descriptor/12246-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18165-6">
    <skos:altLabel xml:lang="de">Umweltschutzmanagement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18165-6">
    <skos:altLabel xml:lang="de">Umweltorientierte Unternehmensführung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18165-6">
    <skos:altLabel xml:lang="de">Ökologieorientiertes Unternehmensverhalten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18165-6">
    <skos:altLabel xml:lang="de">Ökologieorientiertes Management</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18165-6">
    <skos:altLabel xml:lang="de">Ökologieorientierte Unternehmensführung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18165-6">
    <skos:altLabel xml:lang="de">Betriebliches Umweltmanagement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18165-6">
    <skos:altLabel xml:lang="de">Betrieblicher Umweltschutz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18165-6">
    <skos:altLabel xml:lang="de">Betriebliche Umweltpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19316-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13687-1">
    <skos:prefLabel xml:lang="en">Banking supervision</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13687-1">
    <skos:prefLabel xml:lang="de">Bankenaufsicht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12285-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70324">
    <skos:prefLabel xml:lang="en">V.14.01.01  Social policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70324">
    <skos:prefLabel xml:lang="de">V.14.01.01  Sozialpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13433-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10685-6">
    <skos:related rdf:resource="descriptor/10691-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18685-1">
    <skos:broader rdf:resource="thsys/71105"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18685-1">
    <skos:broader rdf:resource="thsys/71045"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18685-1">
    <skos:broader rdf:resource="descriptor/18670-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18685-1">
    <skos:broader rdf:resource="descriptor/12234-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11602-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14942-6">
    <skos:broader rdf:resource="thsys/70498"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14942-6">
    <skos:broader rdf:resource="descriptor/14940-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19358-4">
    <skos:prefLabel xml:lang="en">External debt service</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19358-4">
    <skos:prefLabel xml:lang="de">Auslandsschuldendienst</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71035">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">B.10</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13247-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18115-0">
    <skos:related rdf:resource="descriptor/13992-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18115-0">
    <skos:related rdf:resource="descriptor/12069-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18115-0">
    <skos:related rdf:resource="descriptor/12057-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18115-0">
    <skos:related rdf:resource="descriptor/18143-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12518-1">
    <skos:broader rdf:resource="thsys/70578"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12518-1">
    <skos:broader rdf:resource="descriptor/12524-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12518-1">
    <skos:broader rdf:resource="descriptor/12523-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70985">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17829-1">
    <skos:broader rdf:resource="thsys/70080"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17829-1">
    <skos:broader rdf:resource="descriptor/17806-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17829-1">
    <skos:broader rdf:resource="descriptor/17991-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17829-1">
    <skos:broader rdf:resource="descriptor/19130-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17829-1">
    <skos:broader rdf:resource="descriptor/19178-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17829-1">
    <skos:broader rdf:resource="descriptor/19177-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11541-4">
    <skos:altLabel xml:lang="de">Rechnungshof</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11541-4">
    <skos:altLabel xml:lang="de">Finanzkontrollbehörde</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16209-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12643-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14216-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19648-2">
    <skos:altLabel xml:lang="de">Spätaussiedler</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19648-2">
    <skos:altLabel xml:lang="de">Aussiedlerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11521-3">
    <zbwext:useInsteadNote rdf:nodeID="id357"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18646-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="r1238766808r28278r2">
    <rdf:rest rdf:nodeID="r1238766808r28278r3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10037-6">
    <skos:related rdf:resource="descriptor/15373-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20921-0">
    <skos:scopeNote xml:lang="de">Spezielles Internet-Angebot als Einstieg für ein einzelnes Interessengebiet oder einen elektronischen Markt, das oft eine Suchmaschine und eine (kommentierte) Linksammlung enthält. Bei kommerziellen Anbietern kann direkt geordert werden. Manchmal werden darüber hinaus Neuigkeiten und weitere Dienste angeboten. In Unternehmen dienen solche Portale z.T. der internen Kommunikation.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19231-0">
    <skos:related rdf:resource="descriptor/11787-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19231-0">
    <skos:related rdf:resource="descriptor/15105-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19231-0">
    <skos:related rdf:resource="descriptor/12065-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19231-0">
    <skos:related rdf:resource="descriptor/10241-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19231-0">
    <skos:related rdf:resource="descriptor/10256-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13289-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19465-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70128">
    <skos:broader rdf:resource="thsys/g"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14737-0">
    <skos:prefLabel xml:lang="en">Perfume</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14737-0">
    <skos:prefLabel xml:lang="de">Parfum</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18418-3">
    <skos:related rdf:resource="descriptor/13578-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18418-3">
    <skos:related rdf:resource="descriptor/15900-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18418-3">
    <skos:related rdf:resource="descriptor/13684-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18418-3">
    <skos:related rdf:resource="descriptor/18814-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18418-3">
    <skos:related rdf:resource="descriptor/14423-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18418-3">
    <skos:related rdf:resource="descriptor/18419-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18418-3">
    <skos:related rdf:resource="descriptor/18607-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18418-3">
    <skos:related rdf:resource="descriptor/13246-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18418-3">
    <skos:related rdf:resource="descriptor/14421-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18418-3">
    <skos:related rdf:resource="descriptor/13575-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18418-3">
    <skos:related rdf:resource="descriptor/13056-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17935-2">
    <skos:related rdf:resource="descriptor/17948-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11642-5">
    <skos:scopeNote xml:lang="de">Nur für theoretische Analysen zur Steuersystematik.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12480-0">
    <skos:prefLabel xml:lang="en">Cost accounting</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12480-0">
    <skos:prefLabel xml:lang="de">Kostenrechnung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13769-6">
    <skos:broader rdf:resource="thsys/70899"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13769-6">
    <skos:broader rdf:resource="descriptor/13756-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11236-2">
    <skos:broader rdf:resource="thsys/71061"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11236-2">
    <skos:broader rdf:resource="thsys/70868"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11236-2">
    <skos:broader rdf:resource="descriptor/11235-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18783-1">
    <skos:broader rdf:resource="thsys/73343"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18783-1">
    <skos:broader rdf:resource="thsys/70878"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18783-1">
    <skos:broader rdf:resource="descriptor/19782-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70819">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70876">
    <skos:narrower rdf:resource="descriptor/12903-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70876">
    <skos:narrower rdf:resource="descriptor/16120-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70876">
    <skos:narrower rdf:resource="descriptor/13392-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70876">
    <skos:narrower rdf:resource="descriptor/18397-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70876">
    <skos:narrower rdf:resource="descriptor/14003-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70876">
    <skos:narrower rdf:resource="descriptor/13385-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70876">
    <skos:narrower rdf:resource="descriptor/19067-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70876">
    <skos:narrower rdf:resource="descriptor/13339-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70876">
    <skos:narrower rdf:resource="descriptor/18298-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70876">
    <skos:narrower rdf:resource="descriptor/15977-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70876">
    <skos:narrower rdf:resource="descriptor/13338-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70876">
    <skos:narrower rdf:resource="descriptor/13383-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70876">
    <skos:narrower rdf:resource="descriptor/12774-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70876">
    <skos:narrower rdf:resource="descriptor/13967-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70876">
    <skos:narrower rdf:resource="descriptor/18065-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70876">
    <skos:narrower rdf:resource="descriptor/11203-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70876">
    <skos:narrower rdf:resource="descriptor/19046-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70876">
    <skos:narrower rdf:resource="descriptor/12283-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70876">
    <skos:narrower rdf:resource="descriptor/13799-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70876">
    <skos:narrower rdf:resource="descriptor/13381-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70876">
    <skos:narrower rdf:resource="descriptor/13330-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70876">
    <skos:narrower rdf:resource="descriptor/19047-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70876">
    <skos:narrower rdf:resource="descriptor/18788-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70876">
    <skos:narrower rdf:resource="descriptor/13378-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70876">
    <skos:narrower rdf:resource="descriptor/18787-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70876">
    <skos:narrower rdf:resource="descriptor/13377-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70876">
    <skos:narrower rdf:resource="descriptor/13322-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70876">
    <skos:narrower rdf:resource="descriptor/12175-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70876">
    <skos:narrower rdf:resource="descriptor/13376-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70876">
    <skos:narrower rdf:resource="thsys/70873"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70876">
    <skos:narrower rdf:resource="thsys/70874"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70876">
    <skos:narrower rdf:resource="thsys/70875"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70904">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.14.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13136-3">
    <skos:prefLabel xml:lang="en">Leather industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13136-3">
    <skos:prefLabel xml:lang="de">Lederindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18167-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18654-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20125-3">
    <skos:prefLabel xml:lang="en">Ecological consulting</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20125-3">
    <skos:prefLabel xml:lang="de">Umweltberatung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14142-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20301-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11552-6">
    <skos:prefLabel xml:lang="en">Infrastructure investment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11552-6">
    <skos:prefLabel xml:lang="de">Infrastrukturinvestition</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19618-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14634-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14750-1">
    <skos:prefLabel xml:lang="en">Office machine</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14750-1">
    <skos:prefLabel xml:lang="de">Büromaschine</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12282-4">
    <skos:altLabel xml:lang="de">Schachtelbeteiligung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12282-4">
    <skos:altLabel xml:lang="de">Beteiligungskapital</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12282-4">
    <skos:altLabel xml:lang="de">Beteiligungsfinanzierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71037">
    <rdfs:label xml:lang="de">IS-Entwicklung und -Betrieb</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12933-3">
    <skos:narrower rdf:resource="descriptor/18196-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12933-3">
    <skos:narrower rdf:resource="descriptor/12951-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12933-3">
    <skos:narrower rdf:resource="descriptor/19193-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12933-3">
    <skos:narrower rdf:resource="descriptor/18198-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12933-3">
    <skos:narrower rdf:resource="descriptor/18190-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10623-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13493-0">
    <skos:altLabel xml:lang="de">Kaufhaus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19673-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16470-4">
    <skos:prefLabel xml:lang="en">War</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16470-4">
    <skos:prefLabel xml:lang="de">Krieg</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13188-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10019-1">
    <skos:narrower rdf:resource="descriptor/12401-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10019-1">
    <skos:narrower rdf:resource="descriptor/18975-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10019-1">
    <skos:narrower rdf:resource="descriptor/11613-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10019-1">
    <skos:narrower rdf:resource="descriptor/10797-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17800-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18255-5">
    <skos:broader rdf:resource="thsys/70463"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18255-5">
    <skos:broader rdf:resource="descriptor/14642-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10623-0">
    <skos:broader rdf:resource="thsys/73366"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10623-0">
    <skos:broader rdf:resource="thsys/70179"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10623-0">
    <skos:broader rdf:resource="descriptor/16441-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10376-5">
    <skos:related rdf:resource="descriptor/10375-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10376-5">
    <skos:related rdf:resource="descriptor/19238-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10376-5">
    <skos:related rdf:resource="descriptor/10276-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16372-4">
    <skos:related rdf:resource="descriptor/16404-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16372-4">
    <skos:related rdf:resource="descriptor/16416-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71040">
    <skos:broader rdf:resource="thsys/70046"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15993-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73331">
    <skos:prefLabel xml:lang="en">N.05.08.02  Community law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73331">
    <skos:prefLabel xml:lang="de">N.05.08.02  EU-Recht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11534-1">
    <skos:related rdf:resource="descriptor/15076-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11534-1">
    <skos:related rdf:resource="descriptor/11540-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12154-6">
    <skos:broader rdf:resource="thsys/70562"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12154-6">
    <skos:broader rdf:resource="descriptor/12111-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13580-5">
    <skos:prefLabel xml:lang="en">Intermodal competition</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13580-5">
    <skos:prefLabel xml:lang="de">Verkehrsträgerwettbewerb</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13176-5">
    <skos:related rdf:resource="descriptor/13562-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13176-5">
    <skos:related rdf:resource="descriptor/13561-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13176-5">
    <skos:related rdf:resource="descriptor/13560-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13176-5">
    <skos:related rdf:resource="descriptor/11578-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16963-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11200-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12757-4">
    <skos:prefLabel xml:lang="en">Production organization</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12757-4">
    <skos:prefLabel xml:lang="de">Fertigungsorganisation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19379-3">
    <skos:broader rdf:resource="thsys/71082"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19379-3">
    <skos:broader rdf:resource="descriptor/12302-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19379-3">
    <skos:broader rdf:resource="descriptor/16295-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16730-4">
    <skos:prefLabel xml:lang="en">Value judgement</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16730-4">
    <skos:prefLabel xml:lang="de">Werturteil</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10517-6">
    <skos:related rdf:resource="descriptor/10541-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19122-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11024-3">
    <skos:related rdf:resource="descriptor/10978-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11024-3">
    <skos:related rdf:resource="descriptor/10220-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11024-3">
    <skos:related rdf:resource="descriptor/10213-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11032-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13155-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71010">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14028-1">
    <skos:related rdf:resource="descriptor/13170-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14028-1">
    <skos:related rdf:resource="descriptor/14020-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16699-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18511-6">
    <skos:narrower rdf:resource="descriptor/18510-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15761-5">
    <skos:altLabel xml:lang="en">Pollution standard</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15761-5">
    <skos:altLabel xml:lang="de">Umweltstandard</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15761-5">
    <skos:altLabel xml:lang="de">Umweltschutzauflage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15761-5">
    <skos:altLabel xml:lang="de">Umweltauflagen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11361-6">
    <skos:altLabel xml:lang="de">Berufliche Qualifizierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11361-6">
    <skos:altLabel xml:lang="de">Ausbildung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14341-4">
    <skos:broader rdf:resource="thsys/70549"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14341-4">
    <skos:broader rdf:resource="descriptor/14336-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11457-0">
    <skos:broader rdf:resource="thsys/70306"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11457-0">
    <skos:broader rdf:resource="descriptor/11449-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19301-5">
    <skos:prefLabel xml:lang="en">Tea industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19301-5">
    <skos:prefLabel xml:lang="de">Teewirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13742-5">
    <skos:broader rdf:resource="thsys/71107"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13742-5">
    <skos:broader rdf:resource="thsys/70893"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19558-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15992-0">
    <skos:altLabel xml:lang="de">Gesetz zum Schutz vor gefährlichen Stoffen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15992-0">
    <skos:altLabel xml:lang="de">Gefahrstoffverordnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15992-0">
    <skos:altLabel xml:lang="de">Gefahrstoffrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15992-0">
    <skos:altLabel xml:lang="de">DDT-Gesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15992-0">
    <skos:altLabel xml:lang="de">Chemikaliengesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15992-0">
    <skos:altLabel xml:lang="de">Chemierecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19630-0">
    <skos:altLabel xml:lang="en">Red list of threatened animals</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19630-0">
    <skos:altLabel xml:lang="en">Control of wildlife trade</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19630-0">
    <skos:altLabel xml:lang="en">Conservation of threatened animals</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19630-0">
    <skos:altLabel xml:lang="de">Wildschutz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19003-6">
    <skos:related rdf:resource="descriptor/12575-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19003-6">
    <skos:related rdf:resource="descriptor/16050-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19003-6">
    <skos:related rdf:resource="descriptor/11185-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13219-6">
    <skos:altLabel xml:lang="de">Nahrungsmittelhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13219-6">
    <skos:altLabel xml:lang="de">Genußmittelhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13219-6">
    <skos:altLabel xml:lang="de">Ernährungshandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15969-2">
    <skos:scopeNote xml:lang="de">Befasst sich mit Ursprung, Zweck, Struktur, Legitimation, gesellschaftlicher Interdependenz und Geltung des Rechts.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13220-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11560-0">
    <skos:related rdf:resource="descriptor/18817-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11560-0">
    <skos:related rdf:resource="descriptor/18815-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11560-0">
    <skos:related rdf:resource="descriptor/11582-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16722-3">
    <skos:broader rdf:resource="thsys/73394"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16722-3">
    <skos:broader rdf:resource="descriptor/19731-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19720-6">
    <skos:broader rdf:resource="thsys/73365"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19720-6">
    <skos:broader rdf:resource="thsys/73363"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19720-6">
    <skos:broader rdf:resource="thsys/70865"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19720-6">
    <skos:broader rdf:resource="descriptor/16463-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13715-1">
    <skos:narrower rdf:resource="descriptor/18732-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13715-1">
    <skos:narrower rdf:resource="descriptor/19538-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11184-2">
    <skos:scopeNote xml:lang="de">Berufliche Veränderungen von Arbeitskräften innerhalb von Betrieben sowie zwischen Betrieben, Berufen, sozialen Positionen und Standorten.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11373-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16409-0">
    <skos:broader rdf:resource="thsys/73383"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/n">
    <skos:prefLabel xml:lang="en">N  Related subject areas</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/n">
    <skos:prefLabel xml:lang="de">N  Nachbarwissenschaften</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18122-3">
    <skos:prefLabel xml:lang="en">Commercial real estate</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18122-3">
    <skos:prefLabel xml:lang="de">Gewerbeimmobilien</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11859-1">
    <skos:broader rdf:resource="thsys/71068"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11859-1">
    <skos:broader rdf:resource="thsys/71066"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70847">
    <rdfs:label xml:lang="en">Education system</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70847">
    <rdfs:label xml:lang="de">Bildungswesen</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10570-2">
    <skos:prefLabel xml:lang="en">Import substitution</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10570-2">
    <skos:prefLabel xml:lang="de">Importsubstitution</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16301-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73383">
    <skos:broader rdf:resource="thsys/70095"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19372-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14948-1">
    <skos:narrower rdf:resource="descriptor/14947-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14948-1">
    <skos:narrower rdf:resource="descriptor/14949-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13362-1">
    <skos:altLabel xml:lang="de">Turnanlage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13362-1">
    <skos:altLabel xml:lang="de">Sportzentrum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13362-1">
    <skos:altLabel xml:lang="de">Sportstätte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13362-1">
    <skos:altLabel xml:lang="de">Sporteinrichtung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13362-1">
    <skos:altLabel xml:lang="de">Freizeitsportanlage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14302-0">
    <skos:narrower rdf:resource="descriptor/14310-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14302-0">
    <skos:narrower rdf:resource="descriptor/14309-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14302-0">
    <skos:narrower rdf:resource="descriptor/14222-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14302-0">
    <skos:narrower rdf:resource="descriptor/18240-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14302-0">
    <skos:narrower rdf:resource="descriptor/18239-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14302-0">
    <skos:narrower rdf:resource="descriptor/14307-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14302-0">
    <skos:narrower rdf:resource="descriptor/14220-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14302-0">
    <skos:narrower rdf:resource="descriptor/14305-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14302-0">
    <skos:narrower rdf:resource="descriptor/18238-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14302-0">
    <skos:narrower rdf:resource="descriptor/14301-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14302-0">
    <skos:narrower rdf:resource="descriptor/14300-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14302-0">
    <skos:narrower rdf:resource="descriptor/14299-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14302-0">
    <skos:narrower rdf:resource="descriptor/14296-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14302-0">
    <skos:narrower rdf:resource="descriptor/14295-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14302-0">
    <skos:narrower rdf:resource="descriptor/18237-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14302-0">
    <skos:narrower rdf:resource="descriptor/14294-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14302-0">
    <skos:narrower rdf:resource="descriptor/14293-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14302-0">
    <skos:narrower rdf:resource="descriptor/14292-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18879-2">
    <skos:broader rdf:resource="thsys/70844"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18879-2">
    <skos:broader rdf:resource="thsys/70176"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18879-2">
    <skos:broader rdf:resource="descriptor/11406-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15050-3">
    <skos:broader rdf:resource="thsys/70269"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15050-3">
    <skos:broader rdf:resource="descriptor/15045-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18754-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13244-0">
    <skos:scopeNote xml:lang="de">Vorwiegend Kraftfahrzeugverkehr; aber auch für nichtmotorisierten Straßentransport, z.B. in Entwicklungsländern</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19072-1">
    <zbwext:useInsteadNote rdf:nodeID="id312"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19072-1">
    <zbwext:useInsteadNote rdf:nodeID="id311"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19677-2">
    <skos:broader rdf:resource="thsys/70986"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19677-2">
    <skos:broader rdf:resource="thsys/70960"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19677-2">
    <skos:broader rdf:resource="descriptor/18991-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19677-2">
    <skos:broader rdf:resource="descriptor/19674-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71015">
    <skos:narrower rdf:resource="descriptor/14364-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71015">
    <skos:narrower rdf:resource="descriptor/14053-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71015">
    <skos:narrower rdf:resource="descriptor/14189-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71015">
    <skos:narrower rdf:resource="descriptor/18163-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71015">
    <skos:narrower rdf:resource="descriptor/18361-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71015">
    <skos:narrower rdf:resource="descriptor/14186-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71015">
    <skos:narrower rdf:resource="descriptor/14407-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71015">
    <skos:narrower rdf:resource="descriptor/18274-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71015">
    <skos:narrower rdf:resource="descriptor/14039-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71015">
    <skos:narrower rdf:resource="descriptor/14037-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71015">
    <skos:narrower rdf:resource="descriptor/14035-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71015">
    <skos:narrower rdf:resource="descriptor/12977-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71015">
    <skos:narrower rdf:resource="descriptor/12974-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71015">
    <skos:narrower rdf:resource="descriptor/19838-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71015">
    <skos:narrower rdf:resource="descriptor/18360-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71015">
    <skos:narrower rdf:resource="descriptor/18359-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71015">
    <skos:narrower rdf:resource="descriptor/18281-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17623-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14074-1">
    <skos:related rdf:resource="descriptor/18195-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14074-1">
    <skos:related rdf:resource="descriptor/14083-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14074-1">
    <skos:related rdf:resource="descriptor/12949-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14074-1">
    <skos:related rdf:resource="descriptor/14073-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14074-1">
    <skos:related rdf:resource="descriptor/14065-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14074-1">
    <skos:related rdf:resource="descriptor/14064-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14074-1">
    <skos:related rdf:resource="descriptor/14060-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14074-1">
    <skos:related rdf:resource="descriptor/13434-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19817-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10698-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19661-3">
    <skos:broader rdf:resource="thsys/70582"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11262-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12435-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13810-0">
    <skos:related rdf:resource="descriptor/18994-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13810-0">
    <skos:related rdf:resource="descriptor/18067-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70902">
    <skos:prefLabel xml:lang="de">W.14.02.01  Spezialbanken</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14971-6">
    <skos:prefLabel xml:lang="en">Non-alcoholic beverage</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14971-6">
    <skos:prefLabel xml:lang="de">Alkoholfreies Getränk</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12019-5">
    <skos:altLabel xml:lang="de">Grundstückspreis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12019-5">
    <skos:altLabel xml:lang="de">Bodenwert</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17928-6">
    <skos:altLabel xml:lang="de">Republik Peru</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18916-1">
    <skos:related rdf:resource="descriptor/18932-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18916-1">
    <skos:related rdf:resource="descriptor/10300-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18916-1">
    <skos:related rdf:resource="descriptor/12907-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18916-1">
    <skos:related rdf:resource="descriptor/16679-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18916-1">
    <skos:related rdf:resource="descriptor/14029-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18916-1">
    <skos:related rdf:resource="descriptor/11194-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73300">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">P.20.05</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13452-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18695-5">
    <skos:altLabel xml:lang="en">World Bank Structural Adjustment Programme</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18695-5">
    <skos:altLabel xml:lang="en">Structural adjustment programme</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18695-5">
    <skos:altLabel xml:lang="en">SAP (Structural Adjustment Programme)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18695-5">
    <skos:altLabel xml:lang="en">IMF standby program</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18695-5">
    <skos:altLabel xml:lang="en">IMF stabilization program</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18695-5">
    <skos:altLabel xml:lang="en">IMF conditionality</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18695-5">
    <skos:altLabel xml:lang="de">Strukturanpassungsprogramm des IWF</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18695-5">
    <skos:altLabel xml:lang="de">Stabilisierungsprogramm des IWF</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18695-5">
    <skos:altLabel xml:lang="de">IWF-Stabilisierungsprogramm</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18695-5">
    <skos:altLabel xml:lang="de">IWF-Konditionen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18695-5">
    <skos:altLabel xml:lang="de">Bereitschaftskreditabkommen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12230-2">
    <skos:altLabel xml:lang="en">Price-earnings ratio</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12230-2">
    <skos:altLabel xml:lang="en">Chart reading</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12230-2">
    <skos:altLabel xml:lang="de">Wertpapierbewertung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12230-2">
    <skos:altLabel xml:lang="de">Technische Wertpapieranalyse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12230-2">
    <skos:altLabel xml:lang="de">Kurs-Gewinn-Verhältnis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12230-2">
    <skos:altLabel xml:lang="de">Fundamentalanalyse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12230-2">
    <skos:altLabel xml:lang="de">DVFA-Formel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12230-2">
    <skos:altLabel xml:lang="de">Chart-Analyse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12230-2">
    <skos:altLabel xml:lang="de">Bewertung von Wertpapieren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12230-2">
    <skos:altLabel xml:lang="de">Aktienanalyse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19564-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10307-3">
    <skos:related rdf:resource="descriptor/10334-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10307-3">
    <skos:related rdf:resource="descriptor/10332-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10307-3">
    <skos:related rdf:resource="descriptor/10305-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13394-2">
    <skos:scopeNote xml:lang="de">Hierunter zählen Organisationen wie der Deutsche Städtetag, der Deutsche Landkreistag und der Deutsche Gemeindetag</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10469-5">
    <skos:prefLabel xml:lang="en">Technocracy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10469-5">
    <skos:prefLabel xml:lang="de">Technokratie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12737-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19330-5">
    <skos:scopeNote xml:lang="de">Zölle auf landwirtschaftliche Produkte.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15671-6">
    <skos:altLabel xml:lang="de">Mythologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15671-6">
    <skos:altLabel xml:lang="de">Mystik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18169-5">
    <skos:altLabel xml:lang="de">Gesamtbetriebsvereinbarung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18169-5">
    <skos:altLabel xml:lang="de">Dienstvereinbarung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18169-5">
    <skos:altLabel xml:lang="de">Ablösende Betriebsvereinbarung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12418-5">
    <skos:prefLabel xml:lang="en">Equity method</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12418-5">
    <skos:prefLabel xml:lang="de">Equity-Methode</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11919-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19622-6">
    <skos:prefLabel xml:lang="en">Weather</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19622-6">
    <skos:prefLabel xml:lang="de">Wetter</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19667-5">
    <skos:altLabel xml:lang="en">Exit decision</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19667-5">
    <skos:altLabel xml:lang="en">Exit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18608-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18609-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15067-0">
    <skos:altLabel xml:lang="en">Waiting line theory</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15067-0">
    <skos:altLabel xml:lang="en">Queueing network</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15067-0">
    <skos:altLabel xml:lang="en">Queueing model</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15067-0">
    <skos:altLabel xml:lang="de">Warteschlangensystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15067-0">
    <skos:altLabel xml:lang="de">Warteschlangenmodell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12408-1">
    <skos:related rdf:resource="descriptor/12441-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12408-1">
    <skos:related rdf:resource="descriptor/12438-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12408-1">
    <skos:related rdf:resource="descriptor/12474-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12408-1">
    <skos:related rdf:resource="descriptor/12277-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12408-1">
    <skos:related rdf:resource="descriptor/12423-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12408-1">
    <skos:related rdf:resource="descriptor/12217-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18943-5">
    <skos:prefLabel xml:lang="en">Information market</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18943-5">
    <skos:prefLabel xml:lang="de">Informationsmarkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19305-4">
    <skos:related rdf:resource="descriptor/19304-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19305-4">
    <skos:related rdf:resource="descriptor/19303-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19305-4">
    <skos:related rdf:resource="descriptor/19302-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19305-4">
    <skos:related rdf:resource="descriptor/14098-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16712-6">
    <skos:altLabel xml:lang="de">Sozialrechtliche Schule</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16712-6">
    <skos:altLabel xml:lang="de">Jüngerer Historismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16712-6">
    <skos:altLabel xml:lang="de">Jüngere historische Schule</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16712-6">
    <skos:altLabel xml:lang="de">Historismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16712-6">
    <skos:altLabel xml:lang="de">Älterer Historismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16712-6">
    <skos:altLabel xml:lang="de">Ältere historische Schule</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13397-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70506">
    <skos:prefLabel xml:lang="de">P.03  Meeresprodukte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18869-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16783-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19643-5">
    <skos:broader rdf:resource="thsys/70142"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19643-5">
    <skos:broader rdf:resource="descriptor/15918-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14776-4">
    <skos:prefLabel xml:lang="en">Glass</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14776-4">
    <skos:prefLabel xml:lang="de">Glas</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11844-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id223">
    <rdf:value xml:lang="de">für Käufermotivation</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14056-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10993-2">
    <skos:narrower rdf:resource="descriptor/10410-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10993-2">
    <skos:narrower rdf:resource="descriptor/10414-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10993-2">
    <skos:narrower rdf:resource="descriptor/10406-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10993-2">
    <skos:narrower rdf:resource="descriptor/19262-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19708-3">
    <skos:broader rdf:resource="thsys/73366"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19708-3">
    <skos:broader rdf:resource="descriptor/16441-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14738-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13600-4">
    <skos:altLabel xml:lang="de">Geschäftsauto</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13600-4">
    <skos:altLabel xml:lang="de">Fuhrparkplanung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13600-4">
    <skos:altLabel xml:lang="de">Firmenwagen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13600-4">
    <skos:altLabel xml:lang="de">Fahrzeugpark</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13600-4">
    <skos:altLabel xml:lang="de">Dienstwagen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14127-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13817-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12047-0">
    <skos:related rdf:resource="descriptor/18956-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12047-0">
    <skos:related rdf:resource="descriptor/18097-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12047-0">
    <skos:related rdf:resource="descriptor/10975-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12047-0">
    <skos:related rdf:resource="descriptor/18823-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12047-0">
    <skos:related rdf:resource="descriptor/18834-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12047-0">
    <skos:related rdf:resource="descriptor/24705-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12047-0">
    <skos:related rdf:resource="descriptor/18830-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12047-0">
    <skos:related rdf:resource="descriptor/18831-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71087">
    <rdfs:label xml:lang="en">Balance of payments</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71087">
    <rdfs:label xml:lang="de">Zahlungsbilanz</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13995-4">
    <skos:altLabel xml:lang="de">Verwaltungsjuristen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13995-4">
    <skos:altLabel xml:lang="de">Staatsanwältinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13995-4">
    <skos:altLabel xml:lang="de">Staatsanwälte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13995-4">
    <skos:altLabel xml:lang="de">Richterinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13995-4">
    <skos:altLabel xml:lang="de">Richter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13995-4">
    <skos:altLabel xml:lang="de">Rechtspflegerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13995-4">
    <skos:altLabel xml:lang="de">Rechtspfleger</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13995-4">
    <skos:altLabel xml:lang="de">Rechtsanwältinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13995-4">
    <skos:altLabel xml:lang="de">Rechtsanwälte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13995-4">
    <skos:altLabel xml:lang="de">Notarinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13995-4">
    <skos:altLabel xml:lang="de">Notare</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13995-4">
    <skos:altLabel xml:lang="de">Justizbeamtinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13995-4">
    <skos:altLabel xml:lang="de">Justizbeamte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13995-4">
    <skos:altLabel xml:lang="de">Juristinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13995-4">
    <skos:altLabel xml:lang="de">Juristen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13995-4">
    <skos:altLabel xml:lang="de">Anwältinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13995-4">
    <skos:altLabel xml:lang="de">Anwälte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13995-4">
    <skos:altLabel xml:lang="de">Advokatinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13995-4">
    <skos:altLabel xml:lang="de">Advokaten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19146-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id350">
    <rdf:value xml:lang="de">Für Bargaining-Modelle und die Ökonomische Theorie der Verhandlungen</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11603-1">
    <skos:related rdf:resource="descriptor/11537-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14420-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13807-3">
    <zbwext:useInsteadNote rdf:nodeID="id382"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13807-3">
    <zbwext:useInsteadNote rdf:nodeID="id381"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18285-3">
    <skos:broader rdf:resource="thsys/70253"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18285-3">
    <skos:broader rdf:resource="descriptor/17608-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10173-5">
    <skos:altLabel xml:lang="en">Theory of marriage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10173-5">
    <skos:altLabel xml:lang="en">Economics of the family</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10173-5">
    <skos:altLabel xml:lang="de">Ökonomische Theorie der Familie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10173-5">
    <skos:altLabel xml:lang="de">Ökonomische Theorie der Ehe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10173-5">
    <skos:altLabel xml:lang="de">Intrafamiliale Zeitallokation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10173-5">
    <skos:altLabel xml:lang="de">Familienökonomie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19032-6">
    <skos:altLabel xml:lang="de">Überprüfung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19032-6">
    <skos:altLabel xml:lang="de">Kontrollsystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18705-0">
    <skos:narrower rdf:resource="descriptor/13289-6"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id377">
    <rdf:value xml:lang="de">Für betriebswirtschaftliche Theorie</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19126-4">
    <skos:scopeNote xml:lang="de">Nachlass auf den Kaufpreis bei sofortiger Zahlung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18941-2">
    <skos:related rdf:resource="descriptor/18655-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18941-2">
    <skos:related rdf:resource="descriptor/20921-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18941-2">
    <skos:related rdf:resource="descriptor/18110-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18941-2">
    <skos:related rdf:resource="descriptor/24664-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18941-2">
    <skos:related rdf:resource="descriptor/18763-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18941-2">
    <skos:related rdf:resource="descriptor/16774-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18553-4">
    <skos:broader rdf:resource="thsys/70947"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18553-4">
    <skos:broader rdf:resource="descriptor/12818-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16481-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11864-1">
    <skos:related rdf:resource="descriptor/15518-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11864-1">
    <skos:related rdf:resource="descriptor/11876-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11864-1">
    <skos:related rdf:resource="descriptor/18631-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13828-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13499-2">
    <skos:prefLabel xml:lang="en">Long-distance traffic</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13499-2">
    <skos:prefLabel xml:lang="de">Fernverkehr</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19577-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13669-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19813-6">
    <skos:altLabel xml:lang="en">Punishment</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19813-6">
    <skos:altLabel xml:lang="de">Verurteilung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19813-6">
    <skos:altLabel xml:lang="de">Todesstrafe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19813-6">
    <skos:altLabel xml:lang="de">Geldstrafe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19813-6">
    <skos:altLabel xml:lang="de">Freiheitsstrafe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13969-5">
    <skos:prefLabel xml:lang="en">Insurance employees</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13969-5">
    <skos:prefLabel xml:lang="de">Versicherungsberufe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id337">
    <rdf:value xml:lang="de">für längerfristige Betrachtungen</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16384-4">
    <skos:prefLabel xml:lang="en">Corporatism</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16384-4">
    <skos:prefLabel xml:lang="de">Korporatismus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19649-0">
    <skos:prefLabel xml:lang="en">Immigration law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19649-0">
    <skos:prefLabel xml:lang="de">Einwanderungsrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13679-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15080-1">
    <skos:broader rdf:resource="thsys/72184"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15080-1">
    <skos:broader rdf:resource="thsys/70953"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15080-1">
    <skos:broader rdf:resource="descriptor/15093-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15080-1">
    <skos:broader rdf:resource="descriptor/13465-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71026">
    <skos:prefLabel xml:lang="en">W.01.01.01  Horticulture</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71026">
    <skos:prefLabel xml:lang="de">W.01.01.01  Gartenbau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10904-6">
    <skos:broader rdf:resource="thsys/70212"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10904-6">
    <skos:broader rdf:resource="descriptor/10029-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18307-5">
    <skos:related rdf:resource="descriptor/18965-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18307-5">
    <skos:related rdf:resource="descriptor/15775-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18307-5">
    <skos:related rdf:resource="descriptor/18559-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18307-5">
    <skos:related rdf:resource="descriptor/18273-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15101-5">
    <skos:related rdf:resource="descriptor/13831-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15101-5">
    <skos:related rdf:resource="descriptor/12905-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70902">
    <skos:narrower rdf:resource="descriptor/13291-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70902">
    <skos:narrower rdf:resource="descriptor/13272-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70902">
    <skos:narrower rdf:resource="descriptor/13702-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70902">
    <skos:narrower rdf:resource="descriptor/18086-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70902">
    <skos:narrower rdf:resource="descriptor/13703-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70902">
    <skos:narrower rdf:resource="descriptor/10550-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70902">
    <skos:narrower rdf:resource="descriptor/13285-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70902">
    <skos:narrower rdf:resource="descriptor/13284-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70902">
    <skos:narrower rdf:resource="descriptor/18540-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10514-5">
    <skos:related rdf:resource="descriptor/10506-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10514-5">
    <skos:related rdf:resource="descriptor/10513-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18435-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19204-3">
    <skos:broader rdf:resource="thsys/73393"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19204-3">
    <skos:broader rdf:resource="thsys/70141"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19204-3">
    <skos:broader rdf:resource="descriptor/10032-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70176">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15769-3">
    <skos:related rdf:resource="descriptor/15770-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15769-3">
    <skos:related rdf:resource="descriptor/15768-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15769-3">
    <skos:related rdf:resource="descriptor/15766-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15769-3">
    <skos:related rdf:resource="descriptor/18186-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15769-3">
    <skos:related rdf:resource="descriptor/15762-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15769-3">
    <skos:related rdf:resource="descriptor/15761-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15769-3">
    <skos:related rdf:resource="descriptor/18288-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15769-3">
    <skos:related rdf:resource="descriptor/15993-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15769-3">
    <skos:related rdf:resource="descriptor/15992-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15769-3">
    <skos:related rdf:resource="descriptor/13426-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15769-3">
    <skos:related rdf:resource="descriptor/15988-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13050-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10239-1">
    <skos:broader rdf:resource="thsys/70233"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10239-1">
    <skos:broader rdf:resource="thsys/70021"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70844">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18689-0">
    <skos:prefLabel xml:lang="en">Stock index futures</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18689-0">
    <skos:prefLabel xml:lang="de">Index-Futures</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id128">
    <rdfs:seeAlso rdf:resource="descriptor/13528-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10199-1">
    <skos:scopeNote xml:lang="de">Schattenpreise einzelner Eigenschaften eines Gutes, die zusammen den Produktpreis determinieren.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13743-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13479-1">
    <skos:prefLabel xml:lang="en">Customer service system</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13479-1">
    <skos:prefLabel xml:lang="de">Kundenbedienungssystem</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20206-3">
    <skos:altLabel xml:lang="en">Revenue management</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20206-3">
    <skos:altLabel xml:lang="de">Ertragsmanagement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16204-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16774-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14921-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71087">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.07.08</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10002-4">
    <skos:broader rdf:resource="thsys/70394"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18785-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10112-4">
    <skos:related rdf:resource="descriptor/10991-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10112-4">
    <skos:related rdf:resource="descriptor/10058-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10112-4">
    <skos:related rdf:resource="descriptor/10160-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10112-4">
    <skos:related rdf:resource="descriptor/19666-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10112-4">
    <skos:related rdf:resource="descriptor/11529-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10112-4">
    <skos:related rdf:resource="descriptor/10097-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10112-4">
    <skos:related rdf:resource="descriptor/10095-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10112-4">
    <skos:related rdf:resource="descriptor/11685-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17723-3">
    <skos:altLabel xml:lang="de">Republik Togo</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17726-4">
    <skos:broader rdf:resource="thsys/71031"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17726-4">
    <skos:broader rdf:resource="descriptor/17743-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16017-3">
    <skos:altLabel xml:lang="de">Verfassungsrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16017-3">
    <skos:altLabel xml:lang="de">Abgeordnetengesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18068-4">
    <skos:related rdf:resource="descriptor/19703-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18068-4">
    <skos:related rdf:resource="descriptor/16283-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19361-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10109-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13170-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70860">
    <skos:broader rdf:resource="thsys/70872"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17578-0">
    <skos:related rdf:resource="descriptor/21042-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15036-4">
    <skos:altLabel xml:lang="en">Fuzzy theory on decision making</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15036-4">
    <skos:altLabel xml:lang="en">Fuzzy set theory</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15036-4">
    <skos:altLabel xml:lang="en">Fuzzy logic</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15036-4">
    <skos:altLabel xml:lang="de">Unscharfe Mengen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15036-4">
    <skos:altLabel xml:lang="de">Unscharfe Logik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15036-4">
    <skos:altLabel xml:lang="de">Unscharfe Entscheidung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15036-4">
    <skos:altLabel xml:lang="de">Fuzzy-Technik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15036-4">
    <skos:altLabel xml:lang="de">Fuzzy-Set-Theorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15036-4">
    <skos:altLabel xml:lang="de">Fuzzy-Logik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15036-4">
    <skos:altLabel xml:lang="de">Fuzzy Logik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19510-3">
    <skos:prefLabel xml:lang="en">In-kind transfer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19510-3">
    <skos:prefLabel xml:lang="de">Sachtransfer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15078-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19115-2">
    <skos:related rdf:resource="descriptor/18092-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19115-2">
    <skos:related rdf:resource="descriptor/12681-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19115-2">
    <skos:related rdf:resource="descriptor/18002-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15346-3">
    <skos:altLabel xml:lang="en">Truncated regression model</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15346-3">
    <skos:altLabel xml:lang="en">Truncated model</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15346-3">
    <skos:altLabel xml:lang="en">Truncated dependent variable</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15346-3">
    <skos:altLabel xml:lang="en">Limited dependent variable model</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15346-3">
    <skos:altLabel xml:lang="en">Censored model</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15346-3">
    <skos:altLabel xml:lang="en">Censored data</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15346-3">
    <skos:altLabel xml:lang="de">Zensiertes Tobit-Modell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15346-3">
    <skos:altLabel xml:lang="de">Tobit Modell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15346-3">
    <skos:altLabel xml:lang="de">Gestutztes Tobit-Modell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15346-3">
    <skos:altLabel xml:lang="de">Friktionsmodell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19707-5">
    <skos:prefLabel xml:lang="en">Language policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19707-5">
    <skos:prefLabel xml:lang="de">Sprachpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12274-3">
    <skos:altLabel xml:lang="en">External funds</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12274-3">
    <skos:altLabel xml:lang="de">Fremdfinanzierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14975-5">
    <skos:scopeNote xml:lang="de">Einschließlich künstlicher Heiß- und Kaltgetränke.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15117-4">
    <skos:narrower rdf:resource="descriptor/15251-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18687-4">
    <skos:narrower rdf:resource="descriptor/20368-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16405-1">
    <skos:related rdf:resource="descriptor/15682-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16405-1">
    <skos:related rdf:resource="descriptor/16353-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16405-1">
    <skos:related rdf:resource="descriptor/16233-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16405-1">
    <skos:related rdf:resource="descriptor/16642-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16405-1">
    <skos:related rdf:resource="descriptor/13397-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16405-1">
    <skos:related rdf:resource="descriptor/16430-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16405-1">
    <skos:related rdf:resource="descriptor/16209-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16405-1">
    <skos:related rdf:resource="descriptor/16719-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16405-1">
    <skos:related rdf:resource="descriptor/15821-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16405-1">
    <skos:related rdf:resource="descriptor/16633-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18112-6">
    <skos:altLabel xml:lang="en">Teleworking</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18112-6">
    <skos:altLabel xml:lang="en">Telecommuting</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18112-6">
    <skos:altLabel xml:lang="en">Distance working</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18112-6">
    <skos:altLabel xml:lang="de">Teleheimarbeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18112-6">
    <skos:altLabel xml:lang="de">Elektronische Heimarbeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18112-6">
    <skos:altLabel xml:lang="de">EDV-Heimarbeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73358">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.06.03.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14135-0">
    <skos:altLabel xml:lang="de">Truthahn</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14135-0">
    <skos:altLabel xml:lang="de">Pute</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14135-0">
    <skos:altLabel xml:lang="de">Huhn</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14135-0">
    <skos:altLabel xml:lang="de">Gans</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14135-0">
    <skos:altLabel xml:lang="de">Ente</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18939-3">
    <skos:altLabel xml:lang="de">Rundfunkgebühr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18939-3">
    <skos:altLabel xml:lang="de">Rundfunk- und Fernsehfinanzierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18939-3">
    <skos:altLabel xml:lang="de">Hörfunkfinanzierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18939-3">
    <skos:altLabel xml:lang="de">Fernsehgebühr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18939-3">
    <skos:altLabel xml:lang="de">Fernsehfinanzierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19245-3">
    <skos:altLabel xml:lang="de">Holländische Krankheit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18276-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16171-0">
    <skos:broader rdf:resource="thsys/73389"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16171-0">
    <skos:broader rdf:resource="thsys/70938"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16171-0">
    <skos:broader rdf:resource="descriptor/13569-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20722-6">
    <skos:broader rdf:resource="thsys/73371"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20722-6">
    <skos:broader rdf:resource="thsys/73366"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20722-6">
    <skos:broader rdf:resource="descriptor/15738-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18515-5">
    <skos:broader rdf:resource="thsys/70961"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18515-5">
    <skos:broader rdf:resource="descriptor/13239-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18515-5">
    <skos:broader rdf:resource="descriptor/18550-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10120-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15135-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12730-3">
    <skos:prefLabel xml:lang="en">Computer-aided manufacturing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12730-3">
    <skos:prefLabel xml:lang="de">Automatisierte Produktion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71100">
    <rdfs:label xml:lang="en">International economics</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71100">
    <rdfs:label xml:lang="de">Außenwirtschaftstheorie</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13681-6">
    <skos:prefLabel xml:lang="en">Cable communication</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13681-6">
    <skos:prefLabel xml:lang="de">Kabelkommunikation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13436-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12979-0">
    <skos:related rdf:resource="descriptor/18378-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12979-0">
    <skos:related rdf:resource="descriptor/16023-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12979-0">
    <skos:related rdf:resource="descriptor/18990-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12979-0">
    <skos:related rdf:resource="descriptor/18379-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12979-0">
    <skos:related rdf:resource="descriptor/13179-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12979-0">
    <skos:related rdf:resource="descriptor/14052-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12979-0">
    <skos:related rdf:resource="descriptor/14188-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12979-0">
    <skos:related rdf:resource="descriptor/13329-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19057-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11125-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70151">
    <skos:narrower rdf:resource="descriptor/19782-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70151">
    <skos:narrower rdf:resource="descriptor/15664-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70151">
    <skos:narrower rdf:resource="thsys/73342"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70151">
    <skos:narrower rdf:resource="thsys/73347"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70151">
    <skos:narrower rdf:resource="thsys/73345"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70151">
    <skos:narrower rdf:resource="thsys/73348"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70151">
    <skos:narrower rdf:resource="thsys/73349"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70151">
    <skos:narrower rdf:resource="thsys/73350"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70151">
    <skos:narrower rdf:resource="thsys/73354"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13443-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11487-5">
    <skos:prefLabel xml:lang="en">Open market operations</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11487-5">
    <skos:prefLabel xml:lang="de">Offenmarktpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11784-6">
    <zbwext:useInsteadNote rdf:nodeID="id58"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18177-6">
    <skos:related rdf:resource="descriptor/19268-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18177-6">
    <skos:related rdf:resource="descriptor/17988-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18177-6">
    <skos:related rdf:resource="descriptor/15647-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18177-6">
    <skos:related rdf:resource="descriptor/17978-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70889">
    <skos:broader rdf:resource="thsys/w"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19329-4">
    <skos:prefLabel xml:lang="en">International trade law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19329-4">
    <skos:prefLabel xml:lang="de">Internationales Handelsrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13967-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19483-1">
    <skos:related rdf:resource="descriptor/15775-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19483-1">
    <skos:related rdf:resource="descriptor/15770-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19483-1">
    <skos:related rdf:resource="descriptor/15762-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15941-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19723-0">
    <skos:altLabel xml:lang="de">Waffenhilfe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19723-0">
    <skos:altLabel xml:lang="de">Rüstungshilfe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11991-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18301-3">
    <skos:related rdf:resource="descriptor/13398-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18301-3">
    <skos:related rdf:resource="descriptor/18442-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18301-3">
    <skos:related rdf:resource="descriptor/13974-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18301-3">
    <skos:related rdf:resource="descriptor/10176-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18301-3">
    <skos:related rdf:resource="descriptor/10177-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18301-3">
    <skos:related rdf:resource="descriptor/18335-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18301-3">
    <skos:related rdf:resource="descriptor/10495-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10523-4">
    <skos:altLabel xml:lang="en">Development programming</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10523-4">
    <skos:altLabel xml:lang="de">Entwicklungsprogrammierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12806-3">
    <skos:broader rdf:resource="thsys/73384"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12806-3">
    <skos:broader rdf:resource="thsys/70550"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12806-3">
    <skos:broader rdf:resource="descriptor/19129-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15410-6">
    <skos:broader rdf:resource="thsys/73339"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15410-6">
    <skos:broader rdf:resource="descriptor/15609-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15410-6">
    <skos:broader rdf:resource="descriptor/18170-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19298-3">
    <skos:broader rdf:resource="thsys/71029"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19298-3">
    <skos:broader rdf:resource="thsys/71020"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19298-3">
    <skos:broader rdf:resource="descriptor/19187-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14702-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15723-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10546-6">
    <skos:altLabel xml:lang="en">Commodity diversification</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10546-6">
    <skos:altLabel xml:lang="de">Produktionsdiversifikation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10546-6">
    <skos:altLabel xml:lang="de">Produktdiversifikation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10546-6">
    <skos:altLabel xml:lang="de">Diversifizierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10803-5">
    <skos:related rdf:resource="descriptor/18087-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10803-5">
    <skos:related rdf:resource="descriptor/10821-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10803-5">
    <skos:related rdf:resource="descriptor/10884-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10803-5">
    <skos:related rdf:resource="descriptor/18539-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10803-5">
    <skos:related rdf:resource="descriptor/10221-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10803-5">
    <skos:related rdf:resource="descriptor/24740-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14292-5">
    <skos:altLabel xml:lang="de">Rubidium</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14292-5">
    <skos:altLabel xml:lang="de">Natrium</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14292-5">
    <skos:altLabel xml:lang="de">Lithium</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14292-5">
    <skos:altLabel xml:lang="de">Cäsium</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12492-0">
    <skos:altLabel xml:lang="de">Gemeinkostenrechnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19607-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10481-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73332">
    <skos:prefLabel xml:lang="en">N.05.08.01  Public international law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73332">
    <skos:prefLabel xml:lang="de">N.05.08.01  Völkerrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12594-6">
    <skos:altLabel xml:lang="en">Performance pay</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12594-6">
    <skos:altLabel xml:lang="de">Variables Vergütungssystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12594-6">
    <skos:altLabel xml:lang="de">Qualifikationslohn</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12594-6">
    <skos:altLabel xml:lang="de">Prämienlohn</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12594-6">
    <skos:altLabel xml:lang="de">Leistungslohn</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12594-6">
    <skos:altLabel xml:lang="de">Gratifikation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12594-6">
    <skos:altLabel xml:lang="de">Flexibles Vergütungssystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12594-6">
    <skos:altLabel xml:lang="de">Flexibles Entgeltsystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13785-1">
    <skos:related rdf:resource="descriptor/12265-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13785-1">
    <skos:related rdf:resource="descriptor/18732-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11629-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17592-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11335-0">
    <skos:broader rdf:resource="thsys/71062"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11335-0">
    <skos:broader rdf:resource="thsys/70278"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11335-0">
    <skos:broader rdf:resource="descriptor/19222-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11335-0">
    <skos:broader rdf:resource="descriptor/11336-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19631-5">
    <skos:broader rdf:resource="thsys/73366"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19631-5">
    <skos:broader rdf:resource="descriptor/16273-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70865">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.22.01.06</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10613-3">
    <skos:broader rdf:resource="thsys/70572"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10613-3">
    <skos:broader rdf:resource="thsys/70179"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10613-3">
    <skos:broader rdf:resource="thsys/70051"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10613-3">
    <skos:broader rdf:resource="descriptor/15954-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10613-3">
    <skos:broader rdf:resource="descriptor/10615-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10613-3">
    <skos:broader rdf:resource="descriptor/11184-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12253-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71096">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.07.02.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14627-0">
    <skos:broader rdf:resource="thsys/70463"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14627-0">
    <skos:broader rdf:resource="descriptor/14236-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14627-0">
    <skos:broader rdf:resource="descriptor/14601-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14627-0">
    <skos:broader rdf:resource="descriptor/14642-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11685-1">
    <skos:scopeNote xml:lang="de">Zusatzlast der Besteuerung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10287-4">
    <skos:prefLabel xml:lang="en">Equipment investment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10287-4">
    <skos:prefLabel xml:lang="de">Anlageinvestition</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14017-6">
    <skos:altLabel xml:lang="en">Business economists</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14017-6">
    <skos:altLabel xml:lang="de">Wirtschaftswissenschaftlicher Beruf</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14017-6">
    <skos:altLabel xml:lang="de">Wirtschaftswissenschaftlerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14017-6">
    <skos:altLabel xml:lang="de">Wirtschaftshistorikerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14017-6">
    <skos:altLabel xml:lang="de">Wirtschaftshistoriker</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14017-6">
    <skos:altLabel xml:lang="de">Wirtschaftsakademikerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14017-6">
    <skos:altLabel xml:lang="de">Wirtschaftsakademiker</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14017-6">
    <skos:altLabel xml:lang="de">Volkswirtinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14017-6">
    <skos:altLabel xml:lang="de">Volkswirte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14017-6">
    <skos:altLabel xml:lang="de">Verwaltungswissenschaftlerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14017-6">
    <skos:altLabel xml:lang="de">Verwaltungswissenschaftler</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14017-6">
    <skos:altLabel xml:lang="de">Ökonominnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14017-6">
    <skos:altLabel xml:lang="de">Ökonomen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14017-6">
    <skos:altLabel xml:lang="de">Diplom-Volkswirtinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14017-6">
    <skos:altLabel xml:lang="de">Diplom-Volkswirte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14017-6">
    <skos:altLabel xml:lang="de">Diplom-Kaufleute</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14017-6">
    <skos:altLabel xml:lang="de">Diplom-Betriebswirtinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14017-6">
    <skos:altLabel xml:lang="de">Diplom-Betriebswirte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14017-6">
    <skos:altLabel xml:lang="de">Betriebswirtinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14017-6">
    <skos:altLabel xml:lang="de">Betriebswirte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12793-0">
    <skos:broader rdf:resource="thsys/70040"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12793-0">
    <skos:broader rdf:resource="descriptor/19565-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12793-0">
    <skos:broader rdf:resource="descriptor/12794-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13358-6">
    <skos:broader rdf:resource="thsys/70872"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13358-6">
    <skos:broader rdf:resource="thsys/70822"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13358-6">
    <skos:broader rdf:resource="descriptor/13303-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13358-6">
    <skos:broader rdf:resource="descriptor/13351-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18014-6">
    <skos:altLabel xml:lang="de">Grünbrache</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18014-6">
    <skos:altLabel xml:lang="de">Flächenstillegung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18014-6">
    <skos:altLabel xml:lang="de">Brachland</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18014-6">
    <skos:altLabel xml:lang="de">Brache</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10256-1">
    <zbwext:useInsteadNote rdf:nodeID="id119"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70845">
    <rdfs:label xml:lang="de">Schulen</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18617-4">
    <skos:related rdf:resource="descriptor/18620-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18617-4">
    <skos:related rdf:resource="descriptor/13622-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18617-4">
    <skos:related rdf:resource="descriptor/20897-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18617-4">
    <skos:related rdf:resource="descriptor/13620-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19768-6">
    <skos:scopeNote xml:lang="de">Grundlegendes Ideal der Rechtsordnung als Prinzip der gerechten Regelung der Verhältnisse der einzelnen untereinander sowie der Regelung von Rechten und Pflichten des einzelnen gegenüber der Gemeinschaft.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18097-4">
    <skos:altLabel xml:lang="en">Charity donation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18097-4">
    <skos:altLabel xml:lang="en">Charitable contribution</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18655-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70979">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.06.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73342">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12475-0">
    <skos:related rdf:resource="descriptor/10978-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12475-0">
    <skos:related rdf:resource="descriptor/12540-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12475-0">
    <skos:related rdf:resource="descriptor/12514-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18948-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15759-6">
    <skos:altLabel xml:lang="en">Ecosystems</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15759-6">
    <skos:altLabel xml:lang="en">Ecosystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15759-6">
    <skos:altLabel xml:lang="en">Ecological system</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15759-6">
    <skos:altLabel xml:lang="de">Ökosystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15759-6">
    <skos:altLabel xml:lang="de">Ökologisches System</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18359-0">
    <skos:altLabel xml:lang="de">Stromerzeugungstechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18359-0">
    <skos:altLabel xml:lang="de">Kraftwerkstechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18359-0">
    <skos:altLabel xml:lang="de">Elektrische Energieerzeugung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18544-5">
    <skos:altLabel xml:lang="de">Internationale Handelskammer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19544-0">
    <skos:related rdf:resource="descriptor/20241-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19544-0">
    <skos:related rdf:resource="descriptor/18654-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19544-0">
    <skos:related rdf:resource="descriptor/18648-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19544-0">
    <skos:related rdf:resource="descriptor/21020-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19544-0">
    <skos:related rdf:resource="descriptor/21315-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19544-0">
    <skos:related rdf:resource="descriptor/24782-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19544-0">
    <skos:related rdf:resource="descriptor/20921-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19544-0">
    <skos:related rdf:resource="descriptor/19657-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19544-0">
    <skos:related rdf:resource="descriptor/18660-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19544-0">
    <skos:related rdf:resource="descriptor/18712-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16635-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14635-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14030-0">
    <skos:prefLabel xml:lang="en">Garden product</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14030-0">
    <skos:prefLabel xml:lang="de">Gartenartikel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10006-3">
    <skos:related rdf:resource="descriptor/10024-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10006-3">
    <skos:related rdf:resource="descriptor/19501-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10006-3">
    <skos:related rdf:resource="descriptor/18910-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10006-3">
    <skos:related rdf:resource="descriptor/15102-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10006-3">
    <skos:related rdf:resource="descriptor/10282-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19055-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13534-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11418-3">
    <skos:prefLabel xml:lang="en">Money supply</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11418-3">
    <skos:prefLabel xml:lang="de">Geldangebot</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12571-4">
    <skos:prefLabel xml:lang="en">Personnel selection</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12571-4">
    <skos:prefLabel xml:lang="de">Personalauswahl</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19315-1">
    <skos:scopeNote xml:lang="de">Nationale rechtliche Rahmenbedingungen für ausländische Investoren, z.B. hinsichtlich Besteuerung, Gewinntransfer, Erwerb von Grundeigentum, usw.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18115-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14196-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11449-6">
    <skos:narrower rdf:resource="descriptor/11457-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11449-6">
    <skos:narrower rdf:resource="descriptor/11454-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11449-6">
    <skos:narrower rdf:resource="descriptor/19257-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11449-6">
    <skos:narrower rdf:resource="descriptor/19256-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18397-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16608-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17777-1">
    <skos:broader rdf:resource="thsys/70086"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17777-1">
    <skos:broader rdf:resource="descriptor/17791-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15737-2">
    <skos:broader rdf:resource="thsys/73360"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15737-2">
    <skos:broader rdf:resource="thsys/71033"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15737-2">
    <skos:broader rdf:resource="thsys/70239"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15737-2">
    <skos:broader rdf:resource="descriptor/15770-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17935-2">
    <skos:broader rdf:resource="thsys/70128"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10310-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17990-1">
    <skos:broader rdf:resource="thsys/70307"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19714-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13839-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14274-0">
    <skos:broader rdf:resource="thsys/70358"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14274-0">
    <skos:broader rdf:resource="descriptor/14266-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71004">
    <skos:prefLabel xml:lang="de">W.06.01  Industriezweige</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13714-3">
    <skos:prefLabel xml:lang="en">Shareholders</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13714-3">
    <skos:prefLabel xml:lang="de">Aktionäre</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20972-4">
    <skos:broader rdf:resource="thsys/70212"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20972-4">
    <skos:broader rdf:resource="thsys/70140"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20972-4">
    <skos:broader rdf:resource="descriptor/19428-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18985-3">
    <skos:broader rdf:resource="thsys/71023"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18985-3">
    <skos:broader rdf:resource="thsys/70969"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18985-3">
    <skos:broader rdf:resource="descriptor/13179-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11995-0">
    <skos:prefLabel xml:lang="en">Urban development plan</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11995-0">
    <skos:prefLabel xml:lang="de">Bauleitplan</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18260-5">
    <skos:related rdf:resource="descriptor/15635-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18260-5">
    <skos:related rdf:resource="descriptor/14722-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18260-5">
    <skos:related rdf:resource="descriptor/18777-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14300-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16202-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17634-2">
    <skos:altLabel xml:lang="de">Königreich Saudi-Arabien</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71019">
    <skos:prefLabel xml:lang="en">W.02  Forestry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71019">
    <skos:prefLabel xml:lang="de">W.02  Forstwirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14307-4">
    <skos:broader rdf:resource="thsys/70134"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14307-4">
    <skos:broader rdf:resource="descriptor/14274-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14307-4">
    <skos:broader rdf:resource="descriptor/14302-0"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id366">
    <rdf:value xml:lang="de">für Anpassungsmaßnahmen in Unternehmen</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19197-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72186">
    <rdfs:label xml:lang="en">Statistical inference</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72186">
    <rdfs:label xml:lang="de">Inferenzstatistik</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19640-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://zbw.eu/namespaces/skos-extensions/useInsteadNote">
    <rdfs:isDefinedBy rdf:resource="../namespaces/skos-extensions/skos-extensions.rdf"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12612-2">
    <skos:narrower rdf:resource="descriptor/19805-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12612-2">
    <skos:narrower rdf:resource="descriptor/12609-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70324">
    <skos:broader rdf:resource="thsys/70504"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13662-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19100-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19151-5">
    <skos:related rdf:resource="descriptor/19779-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19151-5">
    <skos:related rdf:resource="descriptor/16440-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19151-5">
    <skos:related rdf:resource="descriptor/18564-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19151-5">
    <skos:related rdf:resource="descriptor/12783-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19151-5">
    <skos:related rdf:resource="descriptor/18612-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19151-5">
    <skos:related rdf:resource="descriptor/19146-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16275-2">
    <skos:broader rdf:resource="thsys/73376"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16275-2">
    <skos:broader rdf:resource="thsys/70143"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16275-2">
    <skos:broader rdf:resource="descriptor/11353-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25742-3">
    <skos:broader rdf:resource="thsys/70187"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25742-3">
    <skos:broader rdf:resource="thsys/70037"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14636-6">
    <skos:related rdf:resource="descriptor/14049-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17686-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14664-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70235">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15705-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18346-2">
    <skos:broader rdf:resource="thsys/71012"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18346-2">
    <skos:broader rdf:resource="thsys/70539"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18346-2">
    <skos:broader rdf:resource="descriptor/11812-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15410-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13181-5">
    <skos:prefLabel xml:lang="en">Domestic installation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13181-5">
    <skos:prefLabel xml:lang="de">Bauinstallation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12470-3">
    <skos:prefLabel xml:lang="en">Breakeven analysis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12470-3">
    <skos:prefLabel xml:lang="de">Break-Even-Analyse</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19318-2">
    <skos:related rdf:resource="descriptor/10836-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19318-2">
    <skos:related rdf:resource="descriptor/10854-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18100-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71110">
    <skos:broader rdf:resource="thsys/70021"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12265-4">
    <skos:related rdf:resource="descriptor/18682-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12265-4">
    <skos:related rdf:resource="descriptor/12299-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12265-4">
    <skos:related rdf:resource="descriptor/12256-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12265-4">
    <skos:related rdf:resource="descriptor/19652-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12265-4">
    <skos:related rdf:resource="descriptor/18086-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12265-4">
    <skos:related rdf:resource="descriptor/13729-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12265-4">
    <skos:related rdf:resource="descriptor/24733-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12265-4">
    <skos:related rdf:resource="descriptor/18732-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12265-4">
    <skos:related rdf:resource="descriptor/13785-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12265-4">
    <skos:related rdf:resource="descriptor/12239-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10814-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19202-0">
    <skos:altLabel xml:lang="en">Time inconsistency</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19202-0">
    <skos:altLabel xml:lang="en">Dynamic inconsistency</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19202-0">
    <skos:altLabel xml:lang="de">Zeitinkonsistenz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19202-0">
    <skos:altLabel xml:lang="de">Dynamische Inkonsistenz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19416-2">
    <skos:related rdf:resource="descriptor/19381-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15897-1">
    <skos:related rdf:resource="descriptor/15779-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15897-1">
    <skos:related rdf:resource="descriptor/14150-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15897-1">
    <skos:related rdf:resource="descriptor/19479-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15897-1">
    <skos:related rdf:resource="descriptor/12961-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15897-1">
    <skos:related rdf:resource="descriptor/12935-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15897-1">
    <skos:related rdf:resource="descriptor/18336-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15897-1">
    <skos:related rdf:resource="descriptor/18337-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16769-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11436-1">
    <skos:altLabel xml:lang="en">Fiat standard</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11436-1">
    <skos:altLabel xml:lang="en">Fiat money</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11436-1">
    <skos:altLabel xml:lang="de">Papiergeld</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17920-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20321-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18631-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11881-1">
    <skos:prefLabel xml:lang="en">Agglomeration traffic</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11881-1">
    <skos:prefLabel xml:lang="de">Verdichtungsverkehr</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12014-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15315-0">
    <skos:narrower rdf:resource="descriptor/19609-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12209-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71090">
    <rdfs:label xml:lang="de">Multinationale Unternehmen</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17983-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70963">
    <rdfs:label xml:lang="en">Trade</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70963">
    <rdfs:label xml:lang="de">Handel</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19582-6">
    <skos:prefLabel xml:lang="en">Error correction model</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19582-6">
    <skos:prefLabel xml:lang="de">Fehlerkorrekturmodell</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19377-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70361">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">G.01.01.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12570-6">
    <skos:prefLabel xml:lang="en">Executive recruitment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12570-6">
    <skos:prefLabel xml:lang="de">Führungskräfteauswahl</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14698-5">
    <skos:prefLabel xml:lang="en">Elastomer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14698-5">
    <skos:prefLabel xml:lang="de">Elastomere</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19350-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15926-6">
    <skos:altLabel xml:lang="de">Samis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15926-6">
    <skos:altLabel xml:lang="de">Sami</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15926-6">
    <skos:altLabel xml:lang="de">Samen (Lappland)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11800-6">
    <skos:altLabel xml:lang="de">Regulierung des Agrarmarktes</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11800-6">
    <skos:altLabel xml:lang="de">Interventionssystem am Agrarmarkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11800-6">
    <skos:altLabel xml:lang="de">Agrarmarktpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11800-6">
    <skos:altLabel xml:lang="de">Agrarmarktorganisation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14897-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14803-6">
    <skos:related rdf:resource="descriptor/13226-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14803-6">
    <skos:related rdf:resource="descriptor/14805-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14803-6">
    <skos:related rdf:resource="descriptor/13117-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14803-6">
    <skos:related rdf:resource="descriptor/14617-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14803-6">
    <skos:related rdf:resource="descriptor/14802-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14803-6">
    <skos:related rdf:resource="descriptor/14796-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16522-4">
    <skos:related rdf:resource="descriptor/16523-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16522-4">
    <skos:related rdf:resource="descriptor/16024-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16522-4">
    <skos:related rdf:resource="descriptor/13998-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16522-4">
    <skos:related rdf:resource="descriptor/13412-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12845-0">
    <skos:related rdf:resource="descriptor/12846-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12845-0">
    <skos:related rdf:resource="descriptor/12844-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12845-0">
    <skos:related rdf:resource="descriptor/13971-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12845-0">
    <skos:related rdf:resource="descriptor/12775-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12845-0">
    <skos:related rdf:resource="descriptor/18788-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14094-2">
    <skos:broader rdf:resource="thsys/70341"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14094-2">
    <skos:broader rdf:resource="descriptor/14074-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14833-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10185-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18517-1">
    <skos:prefLabel xml:lang="en">Grain trade</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18517-1">
    <skos:prefLabel xml:lang="de">Getreidehandel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17744-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16963-2">
    <skos:broader rdf:resource="thsys/70003"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16963-2">
    <skos:broader rdf:resource="descriptor/16972-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19085-6">
    <skos:related rdf:resource="descriptor/19084-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13576-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13595-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10237-5">
    <skos:broader rdf:resource="thsys/70234"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10237-5">
    <skos:broader rdf:resource="descriptor/10025-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10237-5">
    <skos:broader rdf:resource="descriptor/10013-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10813-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12824-1">
    <skos:altLabel xml:lang="en">Direct mailing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12824-1">
    <skos:altLabel xml:lang="de">Wurfsendung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12824-1">
    <skos:altLabel xml:lang="de">Werbebrief</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12824-1">
    <skos:altLabel xml:lang="de">Haushaltswerbung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12824-1">
    <skos:altLabel xml:lang="de">Einzelumwerbung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18619-0">
    <skos:narrower rdf:resource="descriptor/18618-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10512-2">
    <skos:related rdf:resource="descriptor/10542-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10512-2">
    <skos:related rdf:resource="descriptor/19444-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10512-2">
    <skos:related rdf:resource="descriptor/11976-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10512-2">
    <skos:related rdf:resource="descriptor/19363-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10512-2">
    <skos:related rdf:resource="descriptor/11991-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10512-2">
    <skos:related rdf:resource="descriptor/11990-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10512-2">
    <skos:related rdf:resource="descriptor/11972-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10512-2">
    <skos:related rdf:resource="descriptor/11993-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10512-2">
    <skos:related rdf:resource="descriptor/10510-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10512-2">
    <skos:related rdf:resource="descriptor/11897-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10512-2">
    <skos:related rdf:resource="descriptor/10504-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10512-2">
    <skos:related rdf:resource="descriptor/10501-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10512-2">
    <skos:related rdf:resource="descriptor/15910-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10512-2">
    <skos:related rdf:resource="descriptor/11979-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19471-1">
    <skos:related rdf:resource="descriptor/11983-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19471-1">
    <skos:related rdf:resource="descriptor/19472-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19471-1">
    <skos:related rdf:resource="descriptor/19474-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19471-1">
    <skos:related rdf:resource="descriptor/19439-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71054">
    <skos:prefLabel xml:lang="de">V.13.06.02.01  Kaufmännische, Organisations- und Verwaltungsberufe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10593-4">
    <skos:narrower rdf:resource="descriptor/18200-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10593-4">
    <skos:narrower rdf:resource="descriptor/18194-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10593-4">
    <skos:narrower rdf:resource="descriptor/18191-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13301-0">
    <skos:narrower rdf:resource="descriptor/18397-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13301-0">
    <skos:narrower rdf:resource="descriptor/20136-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13301-0">
    <skos:narrower rdf:resource="descriptor/13303-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13301-0">
    <skos:narrower rdf:resource="descriptor/12786-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13301-0">
    <skos:narrower rdf:resource="descriptor/13345-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13301-0">
    <skos:narrower rdf:resource="descriptor/18085-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13301-0">
    <skos:narrower rdf:resource="descriptor/13379-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13301-0">
    <skos:narrower rdf:resource="descriptor/19541-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70257">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19844-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16041-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16479-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18920-3">
    <skos:broader rdf:resource="thsys/70828"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18920-3">
    <skos:broader rdf:resource="descriptor/12907-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17464-1">
    <skos:altLabel xml:lang="de">Republik Armenien</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17464-1">
    <skos:altLabel xml:lang="de">Armenische SSR</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16679-6">
    <skos:prefLabel xml:lang="en">Leisure research</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16679-6">
    <skos:prefLabel xml:lang="de">Freizeitforschung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13343-5">
    <skos:altLabel xml:lang="de">Wissenschaftliches Forschungsinstitut</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13343-5">
    <skos:altLabel xml:lang="de">Großforschungseinrichtung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13343-5">
    <skos:altLabel xml:lang="de">Forschungseinrichtung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14012-2">
    <skos:related rdf:resource="descriptor/15678-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14012-2">
    <skos:related rdf:resource="descriptor/14014-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14012-2">
    <skos:related rdf:resource="descriptor/13399-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14012-2">
    <skos:related rdf:resource="descriptor/15671-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14012-2">
    <skos:related rdf:resource="descriptor/11356-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16483-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15375-3">
    <skos:altLabel xml:lang="en">Simultaneous equations model</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15375-3">
    <skos:altLabel xml:lang="en">Simultaneous equations</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15375-3">
    <skos:altLabel xml:lang="en">Recursive model</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15375-3">
    <skos:altLabel xml:lang="de">Simultangleichungsmodell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15375-3">
    <skos:altLabel xml:lang="de">Rekursives Modell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15375-3">
    <skos:altLabel xml:lang="de">Reduzierte Form rekursiver Modelle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15375-3">
    <skos:altLabel xml:lang="de">Ökonometrisches Mehrgleichungsmodell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15375-3">
    <skos:altLabel xml:lang="de">Ökonometrisches Gleichungssystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15375-3">
    <skos:altLabel xml:lang="de">Mehrgleichungsmodell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15375-3">
    <skos:altLabel xml:lang="de">Interdependentes ökonometrisches Modell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73338">
    <skos:narrower rdf:resource="descriptor/19810-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73338">
    <skos:narrower rdf:resource="descriptor/13673-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73338">
    <skos:narrower rdf:resource="descriptor/18647-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73338">
    <skos:narrower rdf:resource="descriptor/19657-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73338">
    <skos:narrower rdf:resource="descriptor/19832-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73338">
    <skos:narrower rdf:resource="descriptor/18003-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73338">
    <skos:narrower rdf:resource="descriptor/18110-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73338">
    <skos:narrower rdf:resource="descriptor/18767-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73338">
    <skos:narrower rdf:resource="descriptor/18074-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12282-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10626-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10671-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11992-6">
    <skos:altLabel xml:lang="de">Soziologische Stadtforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11992-6">
    <skos:altLabel xml:lang="de">Sozialwissenschaftliche Stadtforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12019-5">
    <skos:broader rdf:resource="thsys/71066"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12019-5">
    <skos:broader rdf:resource="thsys/70889"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12019-5">
    <skos:broader rdf:resource="descriptor/18745-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12019-5">
    <skos:broader rdf:resource="descriptor/10186-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11167-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15690-2">
    <skos:prefLabel xml:lang="en">Historical perspective</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15690-2">
    <skos:prefLabel xml:lang="de">Geschichte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14931-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12014-1">
    <skos:prefLabel xml:lang="en">Residential land</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12014-1">
    <skos:prefLabel xml:lang="de">Bauland</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70186">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">B.03.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10734-5">
    <skos:narrower rdf:resource="descriptor/10733-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10437-4">
    <skos:related rdf:resource="descriptor/16487-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10437-4">
    <skos:related rdf:resource="descriptor/10987-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10437-4">
    <skos:related rdf:resource="descriptor/10439-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10437-4">
    <skos:related rdf:resource="descriptor/10470-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10437-4">
    <skos:related rdf:resource="descriptor/19251-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10437-4">
    <skos:related rdf:resource="descriptor/10464-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10437-4">
    <skos:related rdf:resource="descriptor/11402-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10437-4">
    <skos:related rdf:resource="descriptor/10436-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16545-6">
    <skos:prefLabel xml:lang="en">Social policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16545-6">
    <skos:prefLabel xml:lang="de">Sozialpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11063-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18551-1">
    <skos:narrower rdf:resource="descriptor/18520-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19703-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10080-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19439-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11957-1">
    <skos:altLabel xml:lang="de">Vermessungskunde</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11957-1">
    <skos:altLabel xml:lang="de">Topographie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11957-1">
    <skos:altLabel xml:lang="de">Landvermessung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11957-1">
    <skos:altLabel xml:lang="de">Geodäsie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11957-1">
    <skos:altLabel xml:lang="de">Erdvermessung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12572-2">
    <skos:prefLabel xml:lang="en">Staff requirement</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12572-2">
    <skos:prefLabel xml:lang="de">Personalbedarf</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13703-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13004-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18193-1">
    <skos:related rdf:resource="descriptor/19296-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18193-1">
    <skos:related rdf:resource="descriptor/18194-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18193-1">
    <skos:related rdf:resource="descriptor/14065-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19212-4">
    <skos:scopeNote xml:lang="de">Ansatz, nach dem Politiker und Staatsbedienstete als Maximierer ihres Individualnutzens für eine überproportional hohe Staatstätigkeit sorgen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10631-1">
    <skos:altLabel xml:lang="de">Wohlfahrtsgewinn des Außenhandels</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14652-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15929-0">
    <skos:broader rdf:resource="thsys/73358"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15929-0">
    <skos:broader rdf:resource="thsys/70142"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15929-0">
    <skos:broader rdf:resource="descriptor/15918-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16233-4">
    <skos:broader rdf:resource="thsys/73383"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16233-4">
    <skos:broader rdf:resource="thsys/73381"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16233-4">
    <skos:broader rdf:resource="thsys/70154"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16233-4">
    <skos:broader rdf:resource="descriptor/16209-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13537-6">
    <skos:altLabel xml:lang="en">Port handling</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13537-6">
    <skos:altLabel xml:lang="de">Güterumschlag im Hafen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13228-5">
    <skos:scopeNote xml:lang="de">Handel mit Unterhaltungselektronik, einschließlich Reparatur, Geräte-Leasing und Handwerk</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13339-3">
    <skos:prefLabel xml:lang="en">Chimney sweep</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13339-3">
    <skos:prefLabel xml:lang="de">Schornsteinfegerhandwerk</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16556-1">
    <skos:broader rdf:resource="thsys/70013"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16556-1">
    <skos:broader rdf:resource="descriptor/16563-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10603-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13794-0">
    <skos:altLabel xml:lang="de">Wertpapierverwaltung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13794-0">
    <skos:altLabel xml:lang="de">Wertpapierverwahrung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19728-4">
    <skos:broader rdf:resource="thsys/70154"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10123-6">
    <skos:altLabel xml:lang="en">Marginal capital-output ratio</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10123-6">
    <skos:altLabel xml:lang="en">Marginal capital output ratio</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10123-6">
    <skos:altLabel xml:lang="en">Capital turnover</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10123-6">
    <skos:altLabel xml:lang="en">Capital output ratio</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13564-3">
    <skos:broader rdf:resource="thsys/70943"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13564-3">
    <skos:broader rdf:resource="descriptor/13583-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13564-3">
    <skos:broader rdf:resource="descriptor/19195-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18018-5">
    <skos:prefLabel xml:lang="en">Consumer cooperative</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18018-5">
    <skos:prefLabel xml:lang="de">Konsumgenossenschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12543-2">
    <skos:related rdf:resource="descriptor/10015-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12543-2">
    <skos:related rdf:resource="descriptor/19094-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19844-2">
    <skos:related rdf:resource="descriptor/11185-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19844-2">
    <skos:related rdf:resource="descriptor/11339-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19844-2">
    <skos:related rdf:resource="descriptor/11307-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16449-2">
    <skos:altLabel xml:lang="en">Nuclear test ban</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16449-2">
    <skos:altLabel xml:lang="en">Nonproliferation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16449-2">
    <skos:altLabel xml:lang="de">Nukleare Abrüstung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16449-2">
    <skos:altLabel xml:lang="de">Atomwaffensperrvertrag</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16449-2">
    <skos:altLabel xml:lang="de">Atomwaffenfreie Zone</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16449-2">
    <skos:altLabel xml:lang="de">Atomtestverbot</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13340-4">
    <skos:altLabel xml:lang="de">Straßenreinigung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13340-4">
    <skos:altLabel xml:lang="de">Stadtreinigung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13340-4">
    <skos:altLabel xml:lang="de">Schneebeseitigung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12326-3">
    <skos:related rdf:resource="descriptor/16067-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12326-3">
    <skos:related rdf:resource="descriptor/12381-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12326-3">
    <skos:related rdf:resource="descriptor/12313-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12326-3">
    <skos:related rdf:resource="descriptor/12325-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12326-3">
    <skos:related rdf:resource="descriptor/18302-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12326-3">
    <skos:related rdf:resource="descriptor/12217-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12326-3">
    <skos:related rdf:resource="descriptor/19142-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12326-3">
    <skos:related rdf:resource="descriptor/12357-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16002-2">
    <skos:prefLabel xml:lang="en">Ecclesiastical law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16002-2">
    <skos:prefLabel xml:lang="de">Kirchenrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19335-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16512-0">
    <skos:prefLabel xml:lang="en">Minority politics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16512-0">
    <skos:prefLabel xml:lang="de">Minderheitenpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10267-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18003-4">
    <skos:related rdf:resource="descriptor/19253-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18003-4">
    <skos:related rdf:resource="descriptor/15611-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19114-4">
    <skos:prefLabel xml:lang="en">Computer-integrated logistics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19114-4">
    <skos:prefLabel xml:lang="de">Logistik-Informationssystem</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11640-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12558-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18579-0">
    <skos:narrower rdf:resource="descriptor/11827-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18579-0">
    <skos:narrower rdf:resource="descriptor/18583-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18579-0">
    <skos:narrower rdf:resource="descriptor/13562-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18579-0">
    <skos:narrower rdf:resource="descriptor/11667-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18579-0">
    <skos:narrower rdf:resource="descriptor/11826-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10008-6">
    <skos:prefLabel xml:lang="en">Investment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10008-6">
    <skos:prefLabel xml:lang="de">Investition</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15909-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12577-6">
    <skos:prefLabel xml:lang="en">Employee relations</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12577-6">
    <skos:prefLabel xml:lang="de">Betriebsklima</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10821-3">
    <skos:prefLabel xml:lang="en">Exchange rate risk</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10821-3">
    <skos:prefLabel xml:lang="de">Wechselkursrisiko</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18117-3">
    <skos:broader rdf:resource="thsys/70140"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15519-5">
    <skos:broader rdf:resource="thsys/71035"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15519-5">
    <skos:broader rdf:resource="thsys/70927"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16749-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13128-2">
    <skos:prefLabel xml:lang="en">Clothing industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13128-2">
    <skos:prefLabel xml:lang="de">Bekleidungsindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11268-3">
    <skos:prefLabel xml:lang="en">Foreign workers</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11268-3">
    <skos:prefLabel xml:lang="de">Ausländische Arbeitskräfte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70530">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.03.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12846-5">
    <skos:narrower rdf:resource="descriptor/12841-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12846-5">
    <skos:narrower rdf:resource="descriptor/12832-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12846-5">
    <skos:narrower rdf:resource="descriptor/18098-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12846-5">
    <skos:narrower rdf:resource="descriptor/19131-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12846-5">
    <skos:narrower rdf:resource="descriptor/18543-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12846-5">
    <skos:narrower rdf:resource="descriptor/12824-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17497-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12744-6">
    <skos:related rdf:resource="descriptor/10190-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12744-6">
    <skos:related rdf:resource="descriptor/12712-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12744-6">
    <skos:related rdf:resource="descriptor/12065-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12744-6">
    <skos:related rdf:resource="descriptor/12742-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12744-6">
    <skos:related rdf:resource="descriptor/10256-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24788-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19147-3">
    <skos:prefLabel xml:lang="en">Decision table</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19147-3">
    <skos:prefLabel xml:lang="de">Entscheidungstabelle</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10500-2">
    <skos:related rdf:resource="descriptor/10508-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10500-2">
    <skos:related rdf:resource="descriptor/10533-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10500-2">
    <skos:related rdf:resource="descriptor/25501-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10500-2">
    <skos:related rdf:resource="descriptor/18478-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10500-2">
    <skos:related rdf:resource="descriptor/16603-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10500-2">
    <skos:related rdf:resource="descriptor/15703-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10500-2">
    <skos:related rdf:resource="descriptor/12913-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10500-2">
    <skos:related rdf:resource="descriptor/18481-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10500-2">
    <skos:related rdf:resource="descriptor/10537-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10500-2">
    <skos:related rdf:resource="descriptor/18060-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11866-4">
    <skos:prefLabel xml:lang="en">Landcare</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11866-4">
    <skos:prefLabel xml:lang="de">Landschaftspflege</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12905-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17754-6">
    <skos:altLabel xml:lang="de">Tunesische Republik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19414-6">
    <skos:broader rdf:resource="thsys/71098"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19414-6">
    <skos:broader rdf:resource="thsys/71075"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19414-6">
    <skos:broader rdf:resource="thsys/71074"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19414-6">
    <skos:broader rdf:resource="descriptor/11547-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19414-6">
    <skos:broader rdf:resource="descriptor/10692-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73347">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.10.05</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70318">
    <skos:broader rdf:resource="thsys/70549"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19650-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13644-5">
    <skos:broader rdf:resource="thsys/70924"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13644-5">
    <skos:broader rdf:resource="descriptor/13632-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10239-1">
    <skos:altLabel xml:lang="en">Macroeconomic capacity</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10239-1">
    <skos:altLabel xml:lang="de">Wirtschaftspotential</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10239-1">
    <skos:altLabel xml:lang="de">Volkswirtschaftliches Produktionspotenzial</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10239-1">
    <skos:altLabel xml:lang="de">Volkswirtschaftliches Produktionspotential</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10239-1">
    <skos:altLabel xml:lang="de">Produktionspotenzial der Gesamtwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10239-1">
    <skos:altLabel xml:lang="de">Produktionspotential der Gesamtwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10239-1">
    <skos:altLabel xml:lang="de">Produktionskapazität der Gesamtwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10239-1">
    <skos:altLabel xml:lang="de">Kapazität der Gesamtwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10239-1">
    <skos:altLabel xml:lang="de">Gesamtwirtschaftliches Produktionspotenzial</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10239-1">
    <skos:altLabel xml:lang="de">Gesamtwirtschaftliches Produktionspotential</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10239-1">
    <skos:altLabel xml:lang="de">Gesamtwirtschaftliche Produktionskapazität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10239-1">
    <skos:altLabel xml:lang="de">Endogenes Potenzial</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10239-1">
    <skos:altLabel xml:lang="de">Endogenes Potential</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11811-1">
    <skos:related rdf:resource="descriptor/19487-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11811-1">
    <skos:related rdf:resource="descriptor/18376-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11811-1">
    <skos:related rdf:resource="descriptor/18349-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11811-1">
    <skos:related rdf:resource="descriptor/18989-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73363">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12021-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19156-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11310-2">
    <skos:broader rdf:resource="thsys/73336"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11310-2">
    <skos:broader rdf:resource="thsys/71051"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11310-2">
    <skos:broader rdf:resource="thsys/70235"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11310-2">
    <skos:broader rdf:resource="descriptor/11067-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11310-2">
    <skos:broader rdf:resource="descriptor/16226-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10097-2">
    <skos:altLabel xml:lang="de">Kompensationsprinzip</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18063-0">
    <skos:prefLabel xml:lang="en">Health care reform</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18063-0">
    <skos:prefLabel xml:lang="de">Gesundheitsreform</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13226-2">
    <skos:related rdf:resource="descriptor/18450-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13226-2">
    <skos:related rdf:resource="descriptor/13117-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13226-2">
    <skos:related rdf:resource="descriptor/14803-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13226-2">
    <skos:related rdf:resource="descriptor/18454-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13226-2">
    <skos:related rdf:resource="descriptor/14024-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13226-2">
    <skos:related rdf:resource="descriptor/13197-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19360-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14640-1">
    <skos:prefLabel xml:lang="en">Carbon dioxide</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14640-1">
    <skos:prefLabel xml:lang="de">Kohlendioxid</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17977-0">
    <skos:related rdf:resource="descriptor/17989-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17977-0">
    <skos:related rdf:resource="descriptor/16066-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17977-0">
    <skos:related rdf:resource="descriptor/17642-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18709-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19594-6">
    <skos:prefLabel xml:lang="en">Extensive form game</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19594-6">
    <skos:prefLabel xml:lang="de">Extensives Spiel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16551-4">
    <skos:scopeNote xml:lang="de">Politik auf der Ebene eines Teilstaates bzw. Bundeslandes</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19129-5">
    <skos:altLabel xml:lang="en">Branding</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19129-5">
    <skos:altLabel xml:lang="de">Markenführung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10013-6">
    <skos:related rdf:resource="descriptor/10907-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10013-6">
    <skos:related rdf:resource="descriptor/10182-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10013-6">
    <skos:related rdf:resource="descriptor/19123-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10013-6">
    <skos:related rdf:resource="descriptor/10000-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18595-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11677-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70086">
    <skos:prefLabel xml:lang="en">G.04.01  Central America</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70086">
    <skos:prefLabel xml:lang="de">G.04.01  Mittelamerika</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19248-4">
    <skos:related rdf:resource="descriptor/10414-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19248-4">
    <skos:related rdf:resource="descriptor/11793-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16650-2">
    <skos:prefLabel xml:lang="en">Self help</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16650-2">
    <skos:prefLabel xml:lang="de">Selbsthilfe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15676-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16084-2">
    <skos:altLabel xml:lang="de">Preisdiskriminierung im Außenhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16084-2">
    <skos:altLabel xml:lang="de">Außenwirtschaftsdiskriminierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19556-0">
    <skos:prefLabel xml:lang="en">Private banking (Privatkundengeschäft)</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19556-0">
    <skos:prefLabel xml:lang="de">Privatkundengeschäft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18508-2">
    <skos:related rdf:resource="descriptor/13860-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18508-2">
    <skos:related rdf:resource="descriptor/18495-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18508-2">
    <skos:related rdf:resource="descriptor/18493-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14915-2">
    <skos:broader rdf:resource="thsys/70031"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14915-2">
    <skos:broader rdf:resource="descriptor/14899-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16354-6">
    <skos:broader rdf:resource="thsys/73376"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13299-3">
    <skos:broader rdf:resource="thsys/70940"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13299-3">
    <skos:broader rdf:resource="thsys/70891"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13299-3">
    <skos:broader rdf:resource="descriptor/13297-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19242-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12782-5">
    <skos:prefLabel xml:lang="en">Franchising</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12782-5">
    <skos:prefLabel xml:lang="de">Franchising</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11043-6">
    <skos:narrower rdf:resource="descriptor/11048-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11937-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18495-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13096-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19329-4">
    <zbwext:useInsteadNote rdf:nodeID="id155"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#broaderTransitive">
    <skos:definition>skos:broaderTransitive is a transitive superproperty of skos:broader.</skos:definition>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13418-0">
    <skos:prefLabel xml:lang="en">Social insurance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13418-0">
    <skos:prefLabel xml:lang="de">Sozialversicherung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18499-5">
    <skos:broader rdf:resource="thsys/70969"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18499-5">
    <skos:broader rdf:resource="descriptor/12915-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17770-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15718-6">
    <skos:altLabel xml:lang="de">Unterprivilegierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15718-6">
    <skos:altLabel xml:lang="de">Stigmatisierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15718-6">
    <skos:altLabel xml:lang="de">Soziale Deprivation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19607-2">
    <skos:narrower rdf:resource="descriptor/19820-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10197-5">
    <skos:broader rdf:resource="thsys/71119"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10197-5">
    <skos:broader rdf:resource="descriptor/10209-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17471-4">
    <skos:broader rdf:resource="thsys/70035"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17471-4">
    <skos:broader rdf:resource="descriptor/19083-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17471-4">
    <skos:broader rdf:resource="descriptor/17983-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19780-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15753-4">
    <skos:prefLabel xml:lang="en">Commodity provision</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15753-4">
    <skos:prefLabel xml:lang="de">Rohstoffversorgung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12974-3">
    <skos:broader rdf:resource="thsys/71015"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12974-3">
    <skos:broader rdf:resource="descriptor/12975-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71036">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14021-1">
    <skos:altLabel xml:lang="de">Schlackenprodukt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14021-1">
    <skos:altLabel xml:lang="de">Dämmplatte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14021-1">
    <skos:altLabel xml:lang="de">Baumaterial</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14021-1">
    <skos:altLabel xml:lang="de">Asphalt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19548-6">
    <skos:scopeNote xml:lang="de">Betriebliche Innovationspolitik und -strategie zur Gestaltung und Vermarktung neuer Produkte (Produktinnovation) bzw. zur Einführung neuer Produktionsverfahren (Prozessinnovation).</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12947-6">
    <skos:altLabel xml:lang="de">Obstanbau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15429-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13444-6">
    <skos:broader rdf:resource="thsys/73387"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13444-6">
    <skos:broader rdf:resource="thsys/71028"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13444-6">
    <skos:broader rdf:resource="thsys/70109"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13444-6">
    <skos:broader rdf:resource="descriptor/15706-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13444-6">
    <skos:broader rdf:resource="descriptor/13446-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11406-3">
    <skos:altLabel xml:lang="de">Finanzierung der Berufsausbildung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11406-3">
    <skos:altLabel xml:lang="de">Finanzielle Ausbildungsförderung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11406-3">
    <skos:altLabel xml:lang="de">Bundesausbildungsförderungsgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11406-3">
    <skos:altLabel xml:lang="de">Berufsbildungsfinanzierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11406-3">
    <skos:altLabel xml:lang="de">BAföG</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11406-3">
    <skos:altLabel xml:lang="de">Ausbildungskosten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11406-3">
    <skos:altLabel xml:lang="de">Ausbildungsförderung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15752-6">
    <skos:broader rdf:resource="thsys/73371"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15752-6">
    <skos:broader rdf:resource="thsys/71010"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15752-6">
    <skos:broader rdf:resource="thsys/70811"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15752-6">
    <skos:broader rdf:resource="descriptor/10987-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16645-2">
    <skos:related rdf:resource="descriptor/10024-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16645-2">
    <skos:related rdf:resource="descriptor/15718-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16645-2">
    <skos:related rdf:resource="descriptor/11671-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16645-2">
    <skos:related rdf:resource="descriptor/10302-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16645-2">
    <skos:related rdf:resource="descriptor/16605-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16645-2">
    <skos:related rdf:resource="descriptor/10005-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16645-2">
    <skos:related rdf:resource="descriptor/10488-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10287-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13382-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18919-2">
    <skos:broader rdf:resource="thsys/70828"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18919-2">
    <skos:broader rdf:resource="descriptor/13352-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19719-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14412-0">
    <skos:prefLabel xml:lang="en">Used car</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14412-0">
    <skos:prefLabel xml:lang="de">Gebrauchtfahrzeug</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13381-4">
    <skos:altLabel xml:lang="de">Ingenieurfirma</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13381-4">
    <skos:altLabel xml:lang="de">Beratende Ingenieurfirma</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15770-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15787-1">
    <skos:prefLabel xml:lang="en">Personality psychology</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15787-1">
    <skos:prefLabel xml:lang="de">Persönlichkeitspsychologie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16003-0">
    <skos:scopeNote xml:lang="de">Gesamtheit der innerstaatlichen und völkerrechtlichen Regelungen, die während eines Krieges gelten.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17702-4">
    <skos:altLabel xml:lang="de">Gabunische Republik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11664-2">
    <skos:narrower rdf:resource="descriptor/19425-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11664-2">
    <skos:narrower rdf:resource="descriptor/19333-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11664-2">
    <skos:narrower rdf:resource="descriptor/19417-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11362-4">
    <skos:prefLabel xml:lang="en">Vocational school</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11362-4">
    <skos:prefLabel xml:lang="de">Berufsbildende Schule</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12677-2">
    <skos:altLabel xml:lang="de">Vorratshaltung des Betriebs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12677-2">
    <skos:altLabel xml:lang="de">Vorrat des Betriebs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12677-2">
    <skos:altLabel xml:lang="de">Optimale Lagerhaltung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12677-2">
    <skos:altLabel xml:lang="de">Materialbestand</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12677-2">
    <skos:altLabel xml:lang="de">Lieferbereitschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12677-2">
    <skos:altLabel xml:lang="de">Lagerwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12677-2">
    <skos:altLabel xml:lang="de">Lagerverwaltung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12677-2">
    <skos:altLabel xml:lang="de">Lagerplanung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12677-2">
    <skos:altLabel xml:lang="de">Lagerhaltungstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12677-2">
    <skos:altLabel xml:lang="de">Lagerhaltung des Betriebs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12677-2">
    <skos:altLabel xml:lang="de">Lagerbestand des Betriebs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12677-2">
    <skos:altLabel xml:lang="de">Bestandsmanagement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10680-2">
    <skos:related rdf:resource="descriptor/10410-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19019-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71090">
    <skos:narrower rdf:resource="descriptor/10847-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71090">
    <skos:narrower rdf:resource="descriptor/12796-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71090">
    <skos:narrower rdf:resource="descriptor/12292-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71090">
    <skos:narrower rdf:resource="descriptor/12062-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71090">
    <skos:narrower rdf:resource="descriptor/19308-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71090">
    <skos:narrower rdf:resource="descriptor/10572-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71090">
    <skos:narrower rdf:resource="descriptor/18121-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71090">
    <skos:narrower rdf:resource="descriptor/10806-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71090">
    <skos:narrower rdf:resource="descriptor/18437-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71090">
    <skos:narrower rdf:resource="descriptor/10823-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71090">
    <skos:narrower rdf:resource="descriptor/10826-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71090">
    <skos:narrower rdf:resource="descriptor/10822-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71090">
    <skos:narrower rdf:resource="descriptor/18001-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71090">
    <skos:narrower rdf:resource="descriptor/10602-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13674-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14734-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14030-0">
    <skos:altLabel xml:lang="de">Rasenmäher</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14030-0">
    <skos:altLabel xml:lang="de">Handrasenmäher</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14030-0">
    <skos:altLabel xml:lang="de">Gartenhaus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14030-0">
    <skos:altLabel xml:lang="de">Gartengrillgerät</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14030-0">
    <skos:altLabel xml:lang="de">Gartengerät</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14030-0">
    <skos:altLabel xml:lang="de">Gartenbedarf</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14030-0">
    <skos:altLabel xml:lang="de">Elektrischer Rasenmäher</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10824-4">
    <skos:scopeNote xml:lang="de">Bi- und multilaterale Schutzabkommen für Auslandsinvestitionen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12849-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14249-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12908-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11272-5">
    <skos:related rdf:resource="descriptor/10176-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11272-5">
    <skos:related rdf:resource="descriptor/11271-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11272-5">
    <skos:related rdf:resource="descriptor/10169-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18962-1">
    <skos:broader rdf:resource="thsys/71033"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18962-1">
    <skos:broader rdf:resource="thsys/70816"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18962-1">
    <skos:broader rdf:resource="thsys/70301"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18962-1">
    <skos:broader rdf:resource="descriptor/13324-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18962-1">
    <skos:broader rdf:resource="descriptor/18960-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73346">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16700-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18255-5">
    <skos:prefLabel xml:lang="en">Alumina</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18255-5">
    <skos:prefLabel xml:lang="de">Aluminiumoxid</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14623-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14442-5">
    <skos:prefLabel xml:lang="en">Mobile home</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14442-5">
    <skos:prefLabel xml:lang="de">Wohnmobil</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16004-5">
    <skos:prefLabel xml:lang="en">Food law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16004-5">
    <skos:prefLabel xml:lang="de">Lebensmittelrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18993-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14549-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24621-6">
    <zbwext:useInsteadNote rdf:nodeID="id1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19857-0">
    <skos:related rdf:resource="descriptor/19689-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19857-0">
    <skos:related rdf:resource="descriptor/16277-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19857-0">
    <skos:related rdf:resource="descriptor/16100-2"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id12">
    <rdf:value xml:lang="de">Nur zur Postkoordination verwenden, sonst</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18239-3">
    <skos:prefLabel xml:lang="en">Rare earth metal</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18239-3">
    <skos:prefLabel xml:lang="de">Seltenerdmetall</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73104">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14635-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14025-0">
    <skos:related rdf:resource="descriptor/13375-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14025-0">
    <skos:related rdf:resource="descriptor/14766-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11827-0">
    <skos:broader rdf:resource="thsys/70938"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11827-0">
    <skos:broader rdf:resource="thsys/70926"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11827-0">
    <skos:broader rdf:resource="thsys/70925"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11827-0">
    <skos:broader rdf:resource="descriptor/18628-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11827-0">
    <skos:broader rdf:resource="descriptor/18579-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11827-0">
    <skos:broader rdf:resource="descriptor/18633-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16716-5">
    <skos:altLabel xml:lang="de">Mathematische Logik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16716-5">
    <skos:altLabel xml:lang="de">Formale Logik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15821-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10186-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13221-5">
    <skos:narrower rdf:resource="descriptor/13239-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19507-6">
    <skos:altLabel xml:lang="de">Sozialhilfeempfängerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10754-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10037-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12940-6">
    <skos:prefLabel xml:lang="en">Horticulture</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12940-6">
    <skos:prefLabel xml:lang="de">Gartenbau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13253-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18384-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16354-6">
    <skos:altLabel xml:lang="de">Parlamentswahl</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16354-6">
    <skos:altLabel xml:lang="de">Landtagswahl</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16354-6">
    <skos:altLabel xml:lang="de">Bundestagswahl</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19705-2">
    <skos:broader rdf:resource="thsys/73373"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19705-2">
    <skos:broader rdf:resource="thsys/73333"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19705-2">
    <skos:broader rdf:resource="descriptor/15970-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19705-2">
    <skos:broader rdf:resource="descriptor/16435-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14176-0">
    <skos:broader rdf:resource="thsys/70538"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14176-0">
    <skos:broader rdf:resource="thsys/70227"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70967">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10852-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12452-5">
    <skos:broader rdf:resource="thsys/70186"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12452-5">
    <skos:broader rdf:resource="descriptor/12381-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10328-2">
    <skos:prefLabel xml:lang="en">Technical efficiency</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10328-2">
    <skos:prefLabel xml:lang="de">Technische Effizienz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10733-0">
    <skos:altLabel xml:lang="en">European Currency Unit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10733-0">
    <skos:altLabel xml:lang="de">Europäische Währungseinheit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19385-1">
    <skos:prefLabel xml:lang="en">Natural gas market</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19385-1">
    <skos:prefLabel xml:lang="de">Erdgasmarkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19902-0">
    <skos:broader rdf:resource="thsys/70235"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19902-0">
    <skos:broader rdf:resource="descriptor/11272-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18397-6">
    <skos:prefLabel xml:lang="en">Business services</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18397-6">
    <skos:prefLabel xml:lang="de">Unternehmensdienstleistung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10347-5">
    <skos:related rdf:resource="descriptor/10359-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10347-5">
    <skos:related rdf:resource="descriptor/10434-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10347-5">
    <skos:related rdf:resource="descriptor/10344-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10347-5">
    <skos:related rdf:resource="descriptor/10251-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10347-5">
    <skos:related rdf:resource="descriptor/10387-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10347-5">
    <skos:related rdf:resource="descriptor/10361-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10347-5">
    <skos:related rdf:resource="descriptor/10338-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10347-5">
    <skos:related rdf:resource="descriptor/10294-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70935">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19139-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17593-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19795-3">
    <skos:broader rdf:resource="thsys/73345"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19795-3">
    <skos:broader rdf:resource="descriptor/15685-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10328-2">
    <skos:related rdf:resource="descriptor/10114-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10328-2">
    <skos:related rdf:resource="descriptor/12073-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10328-2">
    <skos:related rdf:resource="descriptor/19609-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10328-2">
    <skos:related rdf:resource="descriptor/19208-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18525-2">
    <skos:prefLabel xml:lang="en">Franchise contract</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18525-2">
    <skos:prefLabel xml:lang="de">Franchise-Vertrag</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12475-0">
    <skos:prefLabel xml:lang="en">Cost recovery</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12475-0">
    <skos:prefLabel xml:lang="de">Kostendeckung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13726-3">
    <skos:prefLabel xml:lang="en">Money market securities</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13726-3">
    <skos:prefLabel xml:lang="de">Geldmarktpapier</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id220">
    <rdf:value xml:lang="de">Für ethnische Minderheiten</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16436-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19232-5">
    <skos:broader rdf:resource="thsys/71115"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19474-2">
    <skos:related rdf:resource="descriptor/11992-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19474-2">
    <skos:related rdf:resource="descriptor/11983-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19474-2">
    <skos:related rdf:resource="descriptor/19471-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19474-2">
    <skos:related rdf:resource="descriptor/11969-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71035">
    <rdfs:label xml:lang="de">Unternehmensforschung und Operations Research</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17739-2">
    <skos:broader rdf:resource="thsys/71031"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17739-2">
    <skos:broader rdf:resource="descriptor/17743-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16601-1">
    <skos:related rdf:resource="descriptor/16606-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16601-1">
    <skos:related rdf:resource="descriptor/16604-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16601-1">
    <skos:related rdf:resource="descriptor/10521-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16601-1">
    <skos:related rdf:resource="descriptor/18013-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16601-1">
    <skos:related rdf:resource="descriptor/13301-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10142-2">
    <skos:prefLabel xml:lang="en">Information dissemination</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10142-2">
    <skos:prefLabel xml:lang="de">Informationsverbreitung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10201-5">
    <skos:altLabel xml:lang="en">Price mechanism</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10201-5">
    <skos:altLabel xml:lang="de">Wettbewerbsmechanismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10201-5">
    <skos:altLabel xml:lang="de">Preismechanismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10201-5">
    <skos:altLabel xml:lang="de">Marktsteuerung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10201-5">
    <skos:altLabel xml:lang="de">Marktprozess</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10201-5">
    <skos:altLabel xml:lang="de">Marktprozeß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10201-5">
    <skos:altLabel xml:lang="de">Markt-Preis-Mechanismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10201-5">
    <skos:altLabel xml:lang="de">Marktkoordinierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10201-5">
    <skos:altLabel xml:lang="de">Marktkoordination</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11498-0">
    <skos:prefLabel xml:lang="en">Fisher effect</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11498-0">
    <skos:prefLabel xml:lang="de">Fisher-Effekt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11774-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19050-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70383">
    <skos:prefLabel xml:lang="de">B.01.03  Betriebliche Information und Kommunikation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12424-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10381-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18717-0">
    <skos:broader rdf:resource="thsys/70958"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18717-0">
    <skos:broader rdf:resource="descriptor/12085-6"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id177">
    <rdf:value xml:lang="de">Für den Ressourcenkreislauf</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73392">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71039">
    <skos:prefLabel xml:lang="en">B.09  Business information technology management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71039">
    <skos:prefLabel xml:lang="de">B.09  Wirtschaftsinformatik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19313-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10575-6">
    <skos:narrower rdf:resource="descriptor/18459-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10575-6">
    <skos:narrower rdf:resource="descriptor/19458-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13828-2">
    <skos:broader rdf:resource="thsys/70892"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14916-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16579-3">
    <skos:broader rdf:resource="thsys/70011"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16579-3">
    <skos:broader rdf:resource="descriptor/16563-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15931-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#ConceptScheme">
    <owl:disjointWith rdf:resource="http://www.w3.org/2004/02/skos/core#Concept"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16485-5">
    <skos:prefLabel xml:lang="en">Consumer protection</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16485-5">
    <skos:prefLabel xml:lang="de">Verbraucherschutz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13231-2">
    <skos:altLabel xml:lang="de">Uhrenhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13231-2">
    <skos:altLabel xml:lang="de">Schmuckwarengroßhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13231-2">
    <skos:altLabel xml:lang="de">Schmuckwareneinzelhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13231-2">
    <skos:altLabel xml:lang="de">Juwelier</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14638-2">
    <skos:related rdf:resource="descriptor/12986-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11501-2">
    <skos:broader rdf:resource="thsys/70355"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16210-2">
    <skos:prefLabel xml:lang="en">History of socialism</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16210-2">
    <skos:prefLabel xml:lang="de">Sozialismusgeschichte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17731-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70573">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.14.03</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19371-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18643-3">
    <skos:prefLabel xml:lang="en">Telecommunications law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18643-3">
    <skos:prefLabel xml:lang="de">Telekommunikationsrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17785-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10948-0">
    <skos:prefLabel xml:lang="en">Potential competition</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10948-0">
    <skos:prefLabel xml:lang="de">Potenzieller Wettbewerb</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11409-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15396-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15671-6">
    <skos:narrower rdf:resource="descriptor/19738-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15671-6">
    <skos:narrower rdf:resource="descriptor/15648-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15671-6">
    <skos:narrower rdf:resource="descriptor/15647-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15671-6">
    <skos:narrower rdf:resource="descriptor/15646-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15671-6">
    <skos:narrower rdf:resource="descriptor/16709-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15671-6">
    <skos:narrower rdf:resource="descriptor/15640-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15671-6">
    <skos:narrower rdf:resource="descriptor/15637-6"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id109">
    <rdf:value xml:lang="de">Für Geldangebot im Sinne von Geldmenge</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16629-0">
    <skos:broader rdf:resource="thsys/73378"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16629-0">
    <skos:broader rdf:resource="thsys/70851"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16629-0">
    <skos:broader rdf:resource="descriptor/15711-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10392-0">
    <skos:altLabel xml:lang="en">WIPP (Wage-Increase Permit Plan)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10392-0">
    <skos:altLabel xml:lang="en">Wage-increase permit plan</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10392-0">
    <skos:altLabel xml:lang="de">Lohnstopp</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10392-0">
    <skos:altLabel xml:lang="de">Lohnerhöhungslizenz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18395-3">
    <skos:narrower rdf:resource="descriptor/20442-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14984-4">
    <skos:altLabel xml:lang="de">Tafelwasser</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14984-4">
    <skos:altLabel xml:lang="de">Sodawasser</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14984-4">
    <skos:altLabel xml:lang="de">Selterswasser</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12919-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70030">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13161-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12984-0">
    <skos:broader rdf:resource="thsys/71006"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12984-0">
    <skos:broader rdf:resource="descriptor/12920-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12984-0">
    <skos:broader rdf:resource="descriptor/12980-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11390-6">
    <skos:related rdf:resource="descriptor/11389-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15940-5">
    <skos:scopeNote xml:lang="de">Statistik über Eheschließungen, Scheidungen, Familiengröße usw.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11299-6">
    <skos:prefLabel xml:lang="en">Occupational attainment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11299-6">
    <skos:prefLabel xml:lang="de">Erwerbsverlauf</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16369-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10558-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13217-3">
    <skos:prefLabel xml:lang="en">Motor vehicle trade</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13217-3">
    <skos:prefLabel xml:lang="de">Kraftfahrzeughandel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10772-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id237">
    <rdfs:seeAlso rdf:resource="descriptor/11825-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13985-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17198-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20189-5">
    <skos:altLabel xml:lang="en">Earmarked taxation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18636-0">
    <skos:prefLabel xml:lang="en">Ecotourism</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18636-0">
    <skos:prefLabel xml:lang="de">Öko-Tourismus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18722-0">
    <skos:broader rdf:resource="thsys/73391"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18722-0">
    <skos:broader rdf:resource="thsys/70900"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18722-0">
    <skos:broader rdf:resource="descriptor/12326-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18722-0">
    <skos:broader rdf:resource="descriptor/13693-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13378-0">
    <skos:prefLabel xml:lang="en">Chemical laboratory</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13378-0">
    <skos:prefLabel xml:lang="de">Chemisches Labor</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id300">
    <rdfs:seeAlso rdf:resource="descriptor/18209-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11902-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10828-3">
    <zbwext:useInsteadNote rdf:nodeID="id74"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19276-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15683-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24857-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10266-5">
    <skos:related rdf:resource="descriptor/10265-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11210-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14545-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14188-0">
    <skos:narrower rdf:resource="descriptor/18232-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14188-0">
    <skos:narrower rdf:resource="descriptor/18233-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14188-0">
    <skos:narrower rdf:resource="descriptor/15729-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73347">
    <rdfs:label xml:lang="de">Medizin und Ernährungswissenschaft</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13285-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id384">
    <rdf:value xml:lang="de">für Zeitdauer</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id150">
    <rdfs:seeAlso rdf:resource="descriptor/10966-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12120-2">
    <skos:related rdf:resource="descriptor/12085-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12120-2">
    <skos:related rdf:resource="descriptor/19154-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20897-6">
    <skos:broader rdf:resource="thsys/70943"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20897-6">
    <skos:broader rdf:resource="descriptor/18620-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17968-1">
    <skos:broader rdf:resource="thsys/70198"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17968-1">
    <skos:broader rdf:resource="descriptor/15887-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18443-4">
    <skos:prefLabel xml:lang="en">Food processing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18443-4">
    <skos:prefLabel xml:lang="de">Lebensmitteltechnik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12292-1">
    <skos:altLabel xml:lang="en">Cross-border securities investment</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12292-1">
    <skos:altLabel xml:lang="de">Portfolioinvestition</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12292-1">
    <skos:altLabel xml:lang="de">Indirekte Auslandsinvestition</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13498-4">
    <skos:altLabel xml:lang="de">Gelegenheitsverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13498-4">
    <skos:altLabel xml:lang="de">Bedarfsverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11782-3">
    <skos:related rdf:resource="descriptor/10025-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11782-3">
    <skos:related rdf:resource="descriptor/11790-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11782-3">
    <skos:related rdf:resource="descriptor/10981-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11782-3">
    <skos:related rdf:resource="descriptor/18310-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11782-3">
    <skos:related rdf:resource="descriptor/10049-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11782-3">
    <skos:related rdf:resource="descriptor/11327-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11782-3">
    <skos:related rdf:resource="descriptor/10243-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11782-3">
    <skos:related rdf:resource="descriptor/10521-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11782-3">
    <skos:related rdf:resource="descriptor/18456-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11782-3">
    <skos:related rdf:resource="descriptor/11298-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10561-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id316">
    <rdf:value xml:lang="de">Für EU-Steuerharmonisierung</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14486-6">
    <skos:altLabel xml:lang="de">Neonröhre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14486-6">
    <skos:altLabel xml:lang="de">Leuchtröhre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14486-6">
    <skos:altLabel xml:lang="de">Glühlampe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14486-6">
    <skos:altLabel xml:lang="de">Elektrische Lampe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10572-5">
    <skos:scopeNote xml:lang="de">Kapitalbeteiligung mehrerer Unternehmen aus verschiedenen Ländern an Gemeinschaftsunternehmen in einem Drittland mit einheimischen Partnern.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72180">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16594-0">
    <skos:broader rdf:resource="thsys/70321"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13156-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70930">
    <skos:broader rdf:resource="thsys/70940"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11518-6">
    <skos:scopeNote xml:lang="de">Hypothese über den Zusammenhang zwischen Lohn- und Preissteigerungen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10010-5">
    <skos:prefLabel xml:lang="en">Consumption</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10010-5">
    <skos:prefLabel xml:lang="de">Konsum</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10791-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11303-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17991-6">
    <skos:narrower rdf:resource="descriptor/17829-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17991-6">
    <skos:narrower rdf:resource="descriptor/17638-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17991-6">
    <skos:narrower rdf:resource="descriptor/17161-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17991-6">
    <skos:narrower rdf:resource="descriptor/17104-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17991-6">
    <skos:narrower rdf:resource="descriptor/16985-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17991-6">
    <skos:narrower rdf:resource="descriptor/17433-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17991-6">
    <skos:narrower rdf:resource="descriptor/17198-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17991-6">
    <skos:narrower rdf:resource="descriptor/17811-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17991-6">
    <skos:narrower rdf:resource="descriptor/17039-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17991-6">
    <skos:narrower rdf:resource="descriptor/16984-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17991-6">
    <skos:narrower rdf:resource="descriptor/17374-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17991-6">
    <skos:narrower rdf:resource="descriptor/17184-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17991-6">
    <skos:narrower rdf:resource="descriptor/17255-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17991-6">
    <skos:narrower rdf:resource="descriptor/18012-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17991-6">
    <skos:narrower rdf:resource="descriptor/16995-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17991-6">
    <skos:narrower rdf:resource="descriptor/17201-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15823-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70866">
    <skos:prefLabel xml:lang="de">W.22.01.05  Auswärtige Angelegenheiten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12485-4">
    <skos:altLabel xml:lang="en">Total cost of ownership</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12485-4">
    <skos:altLabel xml:lang="en">TCO (Total cost of ownership)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19958-1">
    <skos:altLabel xml:lang="en">General agreement on trade in services</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19958-1">
    <skos:altLabel xml:lang="de">Allgemeines Übereinkommen über den Dienstleistungsverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10181-6">
    <skos:broader rdf:resource="thsys/71112"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10181-6">
    <skos:broader rdf:resource="thsys/70493"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10181-6">
    <skos:broader rdf:resource="descriptor/10062-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10181-6">
    <skos:broader rdf:resource="descriptor/10177-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14712-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12825-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13412-5">
    <skos:narrower rdf:resource="descriptor/16326-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13412-5">
    <skos:narrower rdf:resource="descriptor/16325-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13412-5">
    <skos:narrower rdf:resource="descriptor/16455-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71054">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16806-4">
    <skos:prefLabel xml:lang="en">Arctic region</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16806-4">
    <skos:prefLabel xml:lang="de">Arktis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10562-1">
    <skos:altLabel xml:lang="de">Wachstumszentrum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10562-1">
    <skos:altLabel xml:lang="de">Wachstumsschwerpunkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10562-1">
    <skos:altLabel xml:lang="de">Wachstumspol</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10562-1">
    <skos:altLabel xml:lang="de">Entwicklungszentrum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18348-5">
    <skos:related rdf:resource="descriptor/12975-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18348-5">
    <skos:related rdf:resource="descriptor/14176-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18348-5">
    <skos:related rdf:resource="descriptor/18353-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18348-5">
    <skos:related rdf:resource="descriptor/11810-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18348-5">
    <skos:related rdf:resource="descriptor/18350-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10045-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10872-0">
    <skos:related rdf:resource="descriptor/20002-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10872-0">
    <skos:related rdf:resource="descriptor/10838-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10064-3">
    <skos:broader rdf:resource="thsys/70234"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10064-3">
    <skos:broader rdf:resource="descriptor/10065-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10912-0">
    <skos:broader rdf:resource="thsys/70212"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10912-0">
    <skos:broader rdf:resource="descriptor/10913-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14129-2">
    <skos:related rdf:resource="descriptor/14633-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14129-2">
    <skos:related rdf:resource="descriptor/12955-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14129-2">
    <skos:related rdf:resource="descriptor/14169-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14129-2">
    <skos:related rdf:resource="descriptor/19020-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14129-2">
    <skos:related rdf:resource="descriptor/14054-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16608-1">
    <skos:related rdf:resource="descriptor/16611-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12277-4">
    <skos:narrower rdf:resource="descriptor/12252-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12277-4">
    <skos:narrower rdf:resource="descriptor/13694-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12277-4">
    <skos:narrower rdf:resource="descriptor/18702-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14622-3">
    <skos:broader rdf:resource="thsys/70536"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14622-3">
    <skos:broader rdf:resource="descriptor/14741-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12015-6">
    <skos:prefLabel xml:lang="en">Land</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12015-6">
    <skos:prefLabel xml:lang="de">Boden</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16516-6">
    <zbwext:useInsteadNote rdf:nodeID="id207"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18753-3">
    <skos:related rdf:resource="descriptor/19456-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18753-3">
    <skos:related rdf:resource="descriptor/11994-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18753-3">
    <skos:related rdf:resource="descriptor/19453-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18753-3">
    <skos:related rdf:resource="descriptor/11844-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18292-6">
    <skos:related rdf:resource="descriptor/11382-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18292-6">
    <skos:related rdf:resource="descriptor/16797-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18292-6">
    <skos:related rdf:resource="descriptor/15656-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18292-6">
    <skos:related rdf:resource="descriptor/11401-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18292-6">
    <skos:related rdf:resource="descriptor/14812-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18292-6">
    <skos:related rdf:resource="descriptor/18868-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73357">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11181-1">
    <skos:related rdf:resource="descriptor/18130-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11181-1">
    <skos:related rdf:resource="descriptor/15839-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11181-1">
    <skos:related rdf:resource="descriptor/12602-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11181-1">
    <skos:related rdf:resource="descriptor/11180-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10949-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11266-0">
    <skos:altLabel xml:lang="en">Labor demand</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11266-0">
    <skos:altLabel xml:lang="de">Stellenangebot</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11266-0">
    <skos:altLabel xml:lang="de">Nachfrage am Arbeitsmarkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11266-0">
    <skos:altLabel xml:lang="de">Bedarf am Arbeitsmarkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11266-0">
    <skos:altLabel xml:lang="de">Arbeitsplatzangebot</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11266-0">
    <skos:altLabel xml:lang="de">Arbeitsnachfragetheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11266-0">
    <skos:altLabel xml:lang="de">Arbeitskräftenachfrage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11266-0">
    <skos:altLabel xml:lang="de">Arbeitskräftebedarf</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14240-3">
    <skos:prefLabel xml:lang="en">Sand</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14240-3">
    <skos:prefLabel xml:lang="de">Sand</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16808-0">
    <skos:related rdf:resource="descriptor/19621-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15654-6">
    <skos:editorialNote xml:lang="de">Möglichst spezifischer indexieren</skos:editorialNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16104-1">
    <skos:narrower rdf:resource="descriptor/12855-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11907-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18566-2">
    <skos:related rdf:resource="descriptor/12761-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18566-2">
    <skos:related rdf:resource="descriptor/13504-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19418-5">
    <skos:broader rdf:resource="thsys/71074"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19418-5">
    <skos:broader rdf:resource="descriptor/11679-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18927-3">
    <skos:prefLabel xml:lang="en">Handicrafts</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18927-3">
    <skos:prefLabel xml:lang="de">Kunsthandwerk</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16035-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10894-4">
    <skos:narrower rdf:resource="descriptor/20522-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10894-4">
    <skos:narrower rdf:resource="descriptor/10893-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10894-4">
    <skos:narrower rdf:resource="descriptor/19435-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10894-4">
    <skos:narrower rdf:resource="descriptor/19270-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10894-4">
    <skos:narrower rdf:resource="descriptor/10683-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10894-4">
    <skos:narrower rdf:resource="descriptor/19433-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10894-4">
    <skos:narrower rdf:resource="descriptor/11800-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19660-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18136-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10500-2">
    <zbwext:useInsteadNote rdf:nodeID="id141"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18861-0">
    <skos:prefLabel xml:lang="en">Adults</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18861-0">
    <skos:prefLabel xml:lang="de">Erwachsene</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18815-0">
    <skos:prefLabel xml:lang="en">Economics of military affairs</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18815-0">
    <skos:prefLabel xml:lang="de">Militärökonomik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12693-4">
    <skos:narrower rdf:resource="descriptor/14397-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12693-4">
    <skos:narrower rdf:resource="descriptor/14396-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15103-1">
    <skos:prefLabel xml:lang="en">Industrial statistics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15103-1">
    <skos:prefLabel xml:lang="de">Industriestatistik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13612-4">
    <skos:scopeNote xml:lang="de">Es existiert keine allgemein gültige Definition; meist: Verkehrsmenge x Weg : Zeit; oder: erbrachte Beförderungsdienste.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18756-4">
    <skos:prefLabel xml:lang="en">Rent for commercial real estate</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18756-4">
    <skos:prefLabel xml:lang="de">Gewerbemiete</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19417-0">
    <skos:related rdf:resource="descriptor/11678-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19417-0">
    <skos:related rdf:resource="descriptor/19669-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11966-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17972-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70889">
    <skos:narrower rdf:resource="descriptor/18329-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70889">
    <skos:narrower rdf:resource="descriptor/18747-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70889">
    <skos:narrower rdf:resource="descriptor/12027-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70889">
    <skos:narrower rdf:resource="descriptor/18745-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70889">
    <skos:narrower rdf:resource="descriptor/12031-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70889">
    <skos:narrower rdf:resource="descriptor/12025-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70889">
    <skos:narrower rdf:resource="descriptor/12030-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70889">
    <skos:narrower rdf:resource="descriptor/13704-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70889">
    <skos:narrower rdf:resource="descriptor/18750-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70889">
    <skos:narrower rdf:resource="descriptor/12029-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70889">
    <skos:narrower rdf:resource="descriptor/12028-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70889">
    <skos:narrower rdf:resource="descriptor/19065-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70889">
    <skos:narrower rdf:resource="descriptor/11659-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70889">
    <skos:narrower rdf:resource="descriptor/12026-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70889">
    <skos:narrower rdf:resource="descriptor/18122-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70889">
    <skos:narrower rdf:resource="descriptor/18746-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70889">
    <skos:narrower rdf:resource="descriptor/18496-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70889">
    <skos:narrower rdf:resource="descriptor/12023-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70889">
    <skos:narrower rdf:resource="descriptor/12019-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70889">
    <skos:narrower rdf:resource="descriptor/12018-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70889">
    <skos:narrower rdf:resource="descriptor/13846-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70889">
    <skos:narrower rdf:resource="thsys/70884"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70889">
    <skos:narrower rdf:resource="thsys/70888"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18954-0">
    <skos:scopeNote xml:lang="de">Vermarktung von Sport und Sportveranstaltungen. Nicht zu verwenden für das Marketing von Sportartikeln außerhalb dieses Rahmens.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16266-3">
    <skos:scopeNote xml:lang="de">Beteiligung der Bürger/innen an politischen Planungen bzw. in Entwicklungsländern an der Entwicklungsplanung jeweils im eigenen Nahbereich (z.B. der eigenen Gemeinde).</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13871-1">
    <skos:prefLabel xml:lang="en">Professional services</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13871-1">
    <skos:prefLabel xml:lang="de">Freie Berufe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13157-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18812-6">
    <skos:altLabel xml:lang="en">Energy pricing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18110-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19772-1">
    <skos:broader rdf:resource="thsys/73386"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19772-1">
    <skos:broader rdf:resource="descriptor/15722-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16410-1">
    <skos:related rdf:resource="descriptor/16416-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16410-1">
    <skos:related rdf:resource="descriptor/16717-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16410-1">
    <skos:related rdf:resource="descriptor/16362-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18640-2">
    <skos:related rdf:resource="descriptor/18636-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17674-4">
    <skos:prefLabel xml:lang="en">Zambia</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17674-4">
    <skos:prefLabel xml:lang="de">Sambia</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20810-2">
    <skos:related rdf:resource="descriptor/19352-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20810-2">
    <skos:related rdf:resource="descriptor/19663-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10834-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11664-2">
    <skos:broader rdf:resource="thsys/71074"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11664-2">
    <skos:broader rdf:resource="thsys/70893"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11664-2">
    <skos:broader rdf:resource="descriptor/11681-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12932-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11589-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17962-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12216-3">
    <skos:prefLabel xml:lang="en">Financial statement analysis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12216-3">
    <skos:prefLabel xml:lang="de">Bilanzanalyse</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16188-4">
    <skos:scopeNote xml:lang="de">Das Völkerrecht regelt vorwiegend das Verhältnis souveräner Staaten und der Staatengemeinschaften untereinander. Bestimmte Normen richten sich auch an Einzelpersonen, z.B. die Menschenrechte. Völkerrechtsnormen können Gewohnheitsrecht oder durch Abkommen kodifiziert sein.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18780-0">
    <skos:prefLabel xml:lang="en">Institute of economic research</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18780-0">
    <skos:prefLabel xml:lang="de">Wirtschaftsforschungsinstitut</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11589-4">
    <skos:broader rdf:resource="thsys/71081"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11589-4">
    <skos:broader rdf:resource="thsys/70859"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11589-4">
    <skos:broader rdf:resource="thsys/70005"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18725-1">
    <skos:related rdf:resource="descriptor/12252-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18725-1">
    <skos:related rdf:resource="descriptor/18698-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18725-1">
    <skos:related rdf:resource="descriptor/12251-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18725-1">
    <skos:related rdf:resource="descriptor/19088-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17800-4">
    <skos:prefLabel xml:lang="en">St. Lucia</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17800-4">
    <skos:prefLabel xml:lang="de">St. Lucia</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73371">
    <rdfs:label xml:lang="de">Umwelt-, Rohstoff- und Energiepolitik</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13689-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13992-3">
    <zbwext:useInsteadNote rdf:nodeID="id341"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13401-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11427-2">
    <skos:broader rdf:resource="thsys/70055"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12529-3">
    <skos:broader rdf:resource="thsys/70956"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12529-3">
    <skos:broader rdf:resource="thsys/70041"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12529-3">
    <skos:broader rdf:resource="descriptor/12531-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18310-2">
    <skos:broader rdf:resource="thsys/70575"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18310-2">
    <skos:broader rdf:resource="thsys/70049"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18310-2">
    <skos:broader rdf:resource="descriptor/10062-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16805-6">
    <skos:altLabel xml:lang="de">Südpolargebiet</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16805-6">
    <skos:altLabel xml:lang="de">Südpol</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18782-3">
    <skos:altLabel xml:lang="en">Academic research</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12299-1">
    <skos:prefLabel xml:lang="en">Securities</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12299-1">
    <skos:prefLabel xml:lang="de">Wertpapier</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10440-1">
    <skos:broader rdf:resource="thsys/70233"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10440-1">
    <skos:broader rdf:resource="thsys/70052"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10440-1">
    <skos:broader rdf:resource="descriptor/10460-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13594-1">
    <skos:altLabel xml:lang="de">Verkehrsräumliche Gliederung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13594-1">
    <skos:altLabel xml:lang="de">Verkehrsraum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13594-1">
    <skos:altLabel xml:lang="de">Transportgeographie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12304-6">
    <skos:broader rdf:resource="thsys/71034"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14202-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17738-4">
    <skos:broader rdf:resource="thsys/70445"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17738-4">
    <skos:broader rdf:resource="descriptor/17672-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20762-1">
    <skos:altLabel xml:lang="de">Zertifikatemärkte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20762-1">
    <skos:altLabel xml:lang="de">Zertifikatehandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20762-1">
    <skos:altLabel xml:lang="de">Umweltzertifikathandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20762-1">
    <skos:altLabel xml:lang="de">Emissionszertifikatehandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20762-1">
    <skos:altLabel xml:lang="de">Emissionsrechtehandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20762-1">
    <skos:altLabel xml:lang="de">CO2-Zertifikatehandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20762-1">
    <skos:altLabel xml:lang="de">CO2-Emissionshandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13757-6">
    <skos:narrower rdf:resource="descriptor/13766-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19712-5">
    <skos:prefLabel xml:lang="en">International scientific cooperation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19712-5">
    <skos:prefLabel xml:lang="de">Internationale wissenschaftliche Zusammenarbeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16124-2">
    <zbwext:useInsteadNote rdf:nodeID="id18"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70337">
    <skos:prefLabel xml:lang="de">P.22  Rüstungsgüter</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18922-6">
    <skos:related rdf:resource="descriptor/18939-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18922-6">
    <skos:related rdf:resource="descriptor/18920-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18922-6">
    <skos:related rdf:resource="descriptor/16515-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18922-6">
    <skos:related rdf:resource="descriptor/18921-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18922-6">
    <skos:related rdf:resource="descriptor/18884-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13409-1">
    <skos:altLabel xml:lang="en">Territorial authorities</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13409-1">
    <skos:altLabel xml:lang="de">Verwaltungsraum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13409-1">
    <skos:altLabel xml:lang="de">Verwaltungseinheit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11054-1">
    <skos:related rdf:resource="descriptor/20522-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11054-1">
    <skos:related rdf:resource="descriptor/10980-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11054-1">
    <skos:related rdf:resource="descriptor/10894-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11054-1">
    <skos:related rdf:resource="descriptor/11718-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11054-1">
    <skos:related rdf:resource="descriptor/10092-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13097-1">
    <skos:related rdf:resource="descriptor/13235-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13097-1">
    <skos:related rdf:resource="descriptor/14586-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13097-1">
    <skos:related rdf:resource="descriptor/15675-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13097-1">
    <skos:related rdf:resource="descriptor/13096-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13097-1">
    <skos:related rdf:resource="descriptor/14029-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15746-1">
    <skos:broader rdf:resource="thsys/73360"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15746-1">
    <skos:broader rdf:resource="thsys/73351"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15746-1">
    <skos:broader rdf:resource="thsys/70815"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15746-1">
    <skos:broader rdf:resource="thsys/70239"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15746-1">
    <skos:broader rdf:resource="descriptor/15770-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14190-6">
    <skos:related rdf:resource="descriptor/13025-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14190-6">
    <skos:related rdf:resource="descriptor/14293-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73334">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16809-5">
    <skos:related rdf:resource="descriptor/10624-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16809-5">
    <skos:related rdf:resource="descriptor/16502-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16809-5">
    <skos:related rdf:resource="descriptor/19299-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18733-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16203-6">
    <skos:related rdf:resource="descriptor/19268-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16203-6">
    <skos:related rdf:resource="descriptor/16201-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16203-6">
    <skos:related rdf:resource="descriptor/16485-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16203-6">
    <skos:related rdf:resource="descriptor/18116-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19057-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13784-3">
    <skos:related rdf:resource="descriptor/11448-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13784-3">
    <skos:related rdf:resource="descriptor/19276-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12215-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70964">
    <rdfs:label xml:lang="de">Handwerkszweige</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11215-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19038-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11938-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71042">
    <skos:narrower rdf:resource="descriptor/12073-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71042">
    <skos:narrower rdf:resource="descriptor/12403-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71042">
    <skos:narrower rdf:resource="descriptor/19095-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71042">
    <skos:narrower rdf:resource="descriptor/12488-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71042">
    <skos:narrower rdf:resource="descriptor/12318-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71042">
    <skos:narrower rdf:resource="descriptor/12391-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71042">
    <skos:narrower rdf:resource="descriptor/12487-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71042">
    <skos:narrower rdf:resource="descriptor/12295-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71042">
    <skos:narrower rdf:resource="descriptor/12372-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71042">
    <skos:narrower rdf:resource="descriptor/12371-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71042">
    <skos:narrower rdf:resource="descriptor/12275-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71042">
    <skos:narrower rdf:resource="descriptor/12485-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71042">
    <skos:narrower rdf:resource="descriptor/19082-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71042">
    <skos:narrower rdf:resource="descriptor/12312-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71042">
    <skos:narrower rdf:resource="descriptor/12218-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71042">
    <skos:narrower rdf:resource="descriptor/19069-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71042">
    <skos:narrower rdf:resource="descriptor/12039-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71042">
    <skos:narrower rdf:resource="descriptor/12308-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71042">
    <skos:narrower rdf:resource="descriptor/19108-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71042">
    <skos:narrower rdf:resource="descriptor/12320-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71042">
    <skos:narrower rdf:resource="descriptor/12228-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71042">
    <skos:narrower rdf:resource="descriptor/12053-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71042">
    <skos:narrower rdf:resource="descriptor/19107-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18076-5">
    <skos:prefLabel xml:lang="en">Causality analysis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18076-5">
    <skos:prefLabel xml:lang="de">Kausalanalyse</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13462-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13039-1">
    <skos:broader rdf:resource="thsys/71001"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13039-1">
    <skos:broader rdf:resource="descriptor/13034-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12950-3">
    <skos:broader rdf:resource="thsys/71025"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12950-3">
    <skos:broader rdf:resource="descriptor/12955-0"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id38">
    <rdfs:seeAlso rdf:resource="descriptor/10602-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12185-2">
    <skos:prefLabel xml:lang="en">General meeting of shareholders</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12185-2">
    <skos:prefLabel xml:lang="de">Hauptversammlung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11724-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11371-3">
    <skos:related rdf:resource="descriptor/18074-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70882">
    <skos:broader rdf:resource="thsys/w"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17553-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12942-2">
    <skos:related rdf:resource="descriptor/13161-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12942-2">
    <skos:related rdf:resource="descriptor/19341-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12942-2">
    <skos:related rdf:resource="descriptor/14105-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12942-2">
    <skos:related rdf:resource="descriptor/13225-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13082-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18103-0">
    <skos:narrower rdf:resource="descriptor/14522-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18103-0">
    <skos:narrower rdf:resource="descriptor/18251-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18103-0">
    <skos:narrower rdf:resource="descriptor/18223-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18103-0">
    <skos:narrower rdf:resource="descriptor/14472-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18103-0">
    <skos:narrower rdf:resource="descriptor/14751-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19762-4">
    <skos:broader rdf:resource="thsys/73357"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19762-4">
    <skos:broader rdf:resource="descriptor/15927-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19762-4">
    <skos:broader rdf:resource="descriptor/19759-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15742-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18449-6">
    <skos:related rdf:resource="descriptor/12847-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18449-6">
    <skos:related rdf:resource="descriptor/18450-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18449-6">
    <skos:related rdf:resource="descriptor/16010-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18449-6">
    <skos:related rdf:resource="descriptor/18211-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18449-6">
    <skos:related rdf:resource="descriptor/18768-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15072-0">
    <skos:prefLabel xml:lang="en">Forecasting technique</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15072-0">
    <skos:prefLabel xml:lang="de">Prognoseverfahren</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10032-2">
    <zbwext:useInsteadNote rdf:nodeID="id378"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70085">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12868-2">
    <skos:broader rdf:resource="thsys/71074"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12868-2">
    <skos:broader rdf:resource="thsys/71040"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12868-2">
    <skos:broader rdf:resource="descriptor/11679-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12868-2">
    <skos:broader rdf:resource="descriptor/11650-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19068-6">
    <skos:broader rdf:resource="thsys/70140"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16104-1">
    <skos:related rdf:resource="descriptor/12849-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16104-1">
    <skos:related rdf:resource="descriptor/12880-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16104-1">
    <skos:related rdf:resource="descriptor/12874-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16104-1">
    <skos:related rdf:resource="descriptor/11666-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16104-1">
    <skos:related rdf:resource="descriptor/16092-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13839-4">
    <skos:broader rdf:resource="thsys/70884"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13839-4">
    <skos:broader rdf:resource="descriptor/13829-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12722-2">
    <skos:broader rdf:resource="thsys/70310"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12722-2">
    <skos:broader rdf:resource="descriptor/12727-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73365">
    <skos:prefLabel xml:lang="en">N.04.04.01  Security policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73365">
    <skos:prefLabel xml:lang="de">N.04.04.01  Sicherheitspolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10294-0">
    <skos:related rdf:resource="descriptor/10137-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10294-0">
    <skos:related rdf:resource="descriptor/20301-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10294-0">
    <skos:related rdf:resource="descriptor/10347-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10294-0">
    <skos:related rdf:resource="descriptor/10126-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10294-0">
    <skos:related rdf:resource="descriptor/10008-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10294-0">
    <skos:related rdf:resource="descriptor/12209-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17497-0">
    <skos:scopeNote xml:lang="de">Erklärte sich 1991 unabhängig von der Sowjetunion und trat der GUS bei</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16699-0">
    <skos:prefLabel xml:lang="en">Generation relations</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16699-0">
    <skos:prefLabel xml:lang="de">Generationenbeziehungen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11888-1">
    <skos:related rdf:resource="descriptor/15958-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11888-1">
    <skos:related rdf:resource="descriptor/11894-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11888-1">
    <skos:related rdf:resource="descriptor/19492-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11888-1">
    <skos:related rdf:resource="descriptor/15954-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14435-2">
    <skos:broader rdf:resource="thsys/70258"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14435-2">
    <skos:broader rdf:resource="descriptor/14433-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15491-1">
    <skos:prefLabel xml:lang="en">Decomposition method</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15491-1">
    <skos:prefLabel xml:lang="de">Dekompositionsverfahren</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19129-5">
    <skos:prefLabel xml:lang="en">Brand management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19129-5">
    <skos:prefLabel xml:lang="de">Markenpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10537-0">
    <skos:broader rdf:resource="thsys/71086"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10537-0">
    <skos:broader rdf:resource="descriptor/10525-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18089-3">
    <skos:related rdf:resource="descriptor/14427-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18089-3">
    <skos:related rdf:resource="descriptor/18294-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73349">
    <rdfs:label xml:lang="en">Chemistry</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73349">
    <rdfs:label xml:lang="de">Chemie</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18370-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18089-3">
    <skos:broader rdf:resource="thsys/70941"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18089-3">
    <skos:broader rdf:resource="descriptor/13508-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18089-3">
    <skos:broader rdf:resource="descriptor/13255-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19179-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13007-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17654-3">
    <skos:broader rdf:resource="thsys/70445"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17654-3">
    <skos:broader rdf:resource="descriptor/17672-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10845-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19037-3">
    <skos:prefLabel xml:lang="en">Estimation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19037-3">
    <skos:prefLabel xml:lang="de">Schätzung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19162-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12320-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70175">
    <rdfs:label xml:lang="en">Economy</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70175">
    <rdfs:label xml:lang="de">Volkswirtschaft</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17790-2">
    <skos:altLabel xml:lang="de">Vereinigte Mexikanische Staaten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11993-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14380-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16754-4">
    <skos:broader rdf:resource="thsys/71039"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16754-4">
    <skos:broader rdf:resource="thsys/70823"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16754-4">
    <skos:broader rdf:resource="thsys/70383"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10410-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20536-3">
    <skos:prefLabel xml:lang="en">Country Image</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20536-3">
    <skos:prefLabel xml:lang="de">Länderimage</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11164-1">
    <skos:altLabel xml:lang="de">Machttheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19833-0">
    <skos:altLabel xml:lang="en">ERP (Enterprise-Resource-Planning)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19833-0">
    <skos:altLabel xml:lang="en">Enterprise-resource system</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19833-0">
    <skos:altLabel xml:lang="de">ERP-System</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71033">
    <rdfs:label xml:lang="en">Environmental management</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71033">
    <rdfs:label xml:lang="de">Umweltmanagement</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18640-2">
    <skos:broader rdf:resource="thsys/73351"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18640-2">
    <skos:broader rdf:resource="thsys/70924"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18640-2">
    <skos:broader rdf:resource="thsys/70239"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18640-2">
    <skos:broader rdf:resource="descriptor/18623-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19655-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71014">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11523-6">
    <skos:altLabel xml:lang="de">Versteckte Inflation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11523-6">
    <skos:altLabel xml:lang="de">Inflationsstau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11262-1">
    <skos:broader rdf:resource="thsys/70113"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11262-1">
    <skos:broader rdf:resource="thsys/70052"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11262-1">
    <skos:broader rdf:resource="descriptor/11271-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12171-6">
    <skos:broader rdf:resource="thsys/73385"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12171-6">
    <skos:broader rdf:resource="thsys/70383"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12171-6">
    <skos:broader rdf:resource="descriptor/15798-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70959">
    <skos:narrower rdf:resource="descriptor/18450-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70959">
    <skos:narrower rdf:resource="descriptor/13228-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70959">
    <skos:narrower rdf:resource="descriptor/19061-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70959">
    <skos:narrower rdf:resource="descriptor/18436-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70959">
    <skos:narrower rdf:resource="descriptor/13235-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70959">
    <skos:narrower rdf:resource="descriptor/13233-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70959">
    <skos:narrower rdf:resource="descriptor/19391-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70959">
    <skos:narrower rdf:resource="descriptor/13232-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70959">
    <skos:narrower rdf:resource="descriptor/13226-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70959">
    <skos:narrower rdf:resource="descriptor/13231-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70959">
    <skos:narrower rdf:resource="descriptor/13222-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70959">
    <skos:narrower rdf:resource="descriptor/13220-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70959">
    <skos:narrower rdf:resource="descriptor/18926-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70959">
    <skos:narrower rdf:resource="descriptor/18522-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70959">
    <skos:narrower rdf:resource="descriptor/13213-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70959">
    <skos:narrower rdf:resource="descriptor/13229-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70959">
    <skos:narrower rdf:resource="descriptor/19396-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70959">
    <skos:narrower rdf:resource="descriptor/13202-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70959">
    <skos:narrower rdf:resource="descriptor/18523-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70959">
    <skos:narrower rdf:resource="descriptor/13197-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13207-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17590-3">
    <skos:altLabel xml:lang="de">Siam</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17590-3">
    <skos:altLabel xml:lang="de">Königreich Thailand</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14762-1">
    <skos:prefLabel xml:lang="en">Scanner</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14762-1">
    <skos:prefLabel xml:lang="de">Scanner</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11742-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14759-4">
    <skos:scopeNote xml:lang="de">Beinhaltet auch Laufwerk und Adapterkarte (Controller).</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15555-1">
    <skos:altLabel xml:lang="en">Optimal control theory</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11194-6">
    <skos:related rdf:resource="descriptor/11260-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11194-6">
    <skos:related rdf:resource="descriptor/10184-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11194-6">
    <skos:related rdf:resource="descriptor/13644-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11194-6">
    <skos:related rdf:resource="descriptor/12907-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11194-6">
    <skos:related rdf:resource="descriptor/18916-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11194-6">
    <skos:related rdf:resource="descriptor/18917-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11194-6">
    <skos:related rdf:resource="descriptor/16679-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11194-6">
    <skos:related rdf:resource="descriptor/14029-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11194-6">
    <skos:related rdf:resource="descriptor/11850-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11194-6">
    <skos:related rdf:resource="descriptor/11187-3"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id108">
    <rdfs:seeAlso rdf:resource="descriptor/11318-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20553-3">
    <skos:scopeNote xml:lang="de">Die Programme zur Mikrofinanzierung sollen im Rahmen der Armutsbekämpfung Kreditnehmern, die keinen anderen Zugang zum institutionellen Kapitalmarkt haben, Finanzierungsmöglichkeiten eröffnen. Mikrofinanzierung stellt z.B. Kleinkredite zur Verfügung für Bedürftige, die sich selbstständig machen wollen, und stellt vielfach eine Alternative zu herkömmlichen Sozialhilfeprogrammen dar.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19824-1">
    <skos:related rdf:resource="descriptor/16191-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19824-1">
    <skos:related rdf:resource="descriptor/16052-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19824-1">
    <skos:related rdf:resource="descriptor/19848-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19824-1">
    <skos:related rdf:resource="descriptor/19847-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19824-1">
    <skos:related rdf:resource="descriptor/19818-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19824-1">
    <skos:related rdf:resource="descriptor/16131-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19377-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11906-4">
    <skos:broader rdf:resource="thsys/73354"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11906-4">
    <skos:broader rdf:resource="thsys/71067"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11906-4">
    <skos:broader rdf:resource="thsys/71005"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11906-4">
    <skos:broader rdf:resource="descriptor/15902-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18212-2">
    <skos:altLabel xml:lang="de">Wirtschaftszeitung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18212-2">
    <skos:altLabel xml:lang="de">Wirtschaftspresse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18212-2">
    <skos:altLabel xml:lang="de">Wirtschaftsmagazin</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18212-2">
    <skos:altLabel xml:lang="de">Finanzpresse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19591-5">
    <skos:prefLabel xml:lang="en">Numerical method</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19591-5">
    <skos:prefLabel xml:lang="de">Numerisches Verfahren</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19459-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13152-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16382-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18358-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10771-6">
    <skos:altLabel xml:lang="en">Currency union</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10771-6">
    <skos:altLabel xml:lang="de">Währungsvereinheitlichung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10771-6">
    <skos:altLabel xml:lang="de">Währungsverbund</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10771-6">
    <skos:altLabel xml:lang="de">Münzunion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10524-2">
    <skos:prefLabel xml:lang="en">Development strategy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10524-2">
    <skos:prefLabel xml:lang="de">Entwicklungsstrategie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14047-4">
    <skos:related rdf:resource="descriptor/12922-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14047-4">
    <skos:related rdf:resource="descriptor/15753-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14047-4">
    <skos:related rdf:resource="descriptor/15754-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14047-4">
    <skos:related rdf:resource="descriptor/18382-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14047-4">
    <skos:related rdf:resource="descriptor/15752-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14047-4">
    <skos:related rdf:resource="descriptor/18995-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14047-4">
    <skos:related rdf:resource="descriptor/18381-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14047-4">
    <skos:related rdf:resource="descriptor/15745-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19086-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18410-5">
    <skos:broader rdf:resource="thsys/71002"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18410-5">
    <skos:broader rdf:resource="descriptor/12914-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10787-5">
    <skos:related rdf:resource="descriptor/19313-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10787-5">
    <skos:related rdf:resource="descriptor/18126-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10787-5">
    <skos:related rdf:resource="descriptor/10642-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13665-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10001-6">
    <skos:scopeNote xml:lang="de">Ausnutzung und Ausgleich interlokaler Preis- bzw. Kursunterschiede.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16131-5">
    <skos:scopeNote xml:lang="de">Alle rechtlichen Grundlagen des einzelnen Arbeitsverhältnisses zwischen Arbeitnehmer/in und Arbeitgeber/in.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12742-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12788-0">
    <skos:altLabel xml:lang="de">Rabattverbot</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12788-0">
    <skos:altLabel xml:lang="de">Rabattmarke</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12788-0">
    <skos:altLabel xml:lang="de">Rabattgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13720-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10048-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15801-5">
    <zbwext:useInsteadNote rdf:nodeID="id223"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15801-5">
    <zbwext:useInsteadNote rdf:nodeID="id222"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16495-2">
    <skos:narrower rdf:resource="descriptor/16349-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16495-2">
    <skos:narrower rdf:resource="descriptor/19336-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16495-2">
    <skos:narrower rdf:resource="descriptor/19352-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16495-2">
    <skos:narrower rdf:resource="descriptor/10608-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16495-2">
    <skos:narrower rdf:resource="descriptor/19663-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18567-0">
    <skos:related rdf:resource="descriptor/16187-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18567-0">
    <skos:related rdf:resource="descriptor/10621-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18567-0">
    <skos:related rdf:resource="descriptor/13501-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15947-5">
    <skos:altLabel xml:lang="de">Krankheitsverbreitung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15947-5">
    <skos:altLabel xml:lang="de">Krankheitsrate</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15947-5">
    <skos:altLabel xml:lang="de">Erkrankungshäufigkeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19654-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19167-4">
    <skos:narrower rdf:resource="descriptor/17612-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19167-4">
    <skos:narrower rdf:resource="descriptor/17610-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19167-4">
    <skos:narrower rdf:resource="descriptor/17609-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19167-4">
    <skos:narrower rdf:resource="descriptor/17607-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19167-4">
    <skos:narrower rdf:resource="descriptor/17602-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19167-4">
    <skos:narrower rdf:resource="descriptor/17598-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19167-4">
    <skos:narrower rdf:resource="descriptor/17597-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14627-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18388-0">
    <skos:scopeNote xml:lang="de">Alle mineralischen Rohstoffvorräte, z.B. auch von Erdöl und Erdgas oder von Metallen. Wird aber in der Literatur teilweise verstanden als: Nicht-metallische Ressourcen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15117-4">
    <skos:altLabel xml:lang="en">Random error</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15117-4">
    <skos:altLabel xml:lang="en">Measurement error</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15117-4">
    <skos:altLabel xml:lang="en">Errors-in-variables model</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15117-4">
    <skos:altLabel xml:lang="de">Zufallsfehler</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15117-4">
    <skos:altLabel xml:lang="de">Stochastischer Fehler</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15117-4">
    <skos:altLabel xml:lang="de">Messfehler</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15117-4">
    <skos:altLabel xml:lang="de">Meßfehler</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15117-4">
    <skos:altLabel xml:lang="de">Fehlertheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15117-4">
    <skos:altLabel xml:lang="de">Fehlerrechnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15117-4">
    <skos:altLabel xml:lang="de">Fehler-in-den-Variablen-Modell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15117-4">
    <skos:altLabel xml:lang="de">Fehler (Statistik)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15117-4">
    <skos:altLabel xml:lang="de">Beobachtungsfehler</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18945-1">
    <skos:related rdf:resource="descriptor/13351-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19042-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19550-5">
    <skos:prefLabel xml:lang="en">Low-income countries</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19550-5">
    <skos:prefLabel xml:lang="de">Low-Income Countries</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13434-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16245-4">
    <skos:related rdf:resource="descriptor/19682-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16245-4">
    <skos:related rdf:resource="descriptor/16298-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16245-4">
    <skos:related rdf:resource="descriptor/15704-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16245-4">
    <skos:related rdf:resource="descriptor/16250-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16245-4">
    <skos:related rdf:resource="descriptor/19753-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11360-1">
    <skos:altLabel xml:lang="de">Berufsbildungssystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11360-1">
    <skos:altLabel xml:lang="de">Berufliche Bildung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16599-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12567-2">
    <skos:related rdf:resource="descriptor/11185-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12567-2">
    <skos:related rdf:resource="descriptor/11219-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18983-0">
    <skos:prefLabel xml:lang="en">Agricultural income</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18983-0">
    <skos:prefLabel xml:lang="de">Agrareinkommen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10055-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11320-6">
    <skos:broader rdf:resource="thsys/70278"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11320-6">
    <skos:broader rdf:resource="thsys/70245"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11320-6">
    <skos:broader rdf:resource="descriptor/11322-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13682-4">
    <skos:altLabel xml:lang="de">Vermittlungstechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13682-4">
    <skos:altLabel xml:lang="de">Übertragungstechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13682-4">
    <skos:altLabel xml:lang="de">Übertragungsmedium</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13682-4">
    <skos:altLabel xml:lang="de">Telekommunikationstechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13682-4">
    <skos:altLabel xml:lang="de">Fernsprechtechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13682-4">
    <skos:altLabel xml:lang="de">Fernschreibtechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13682-4">
    <skos:altLabel xml:lang="de">Fernmeldetechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13682-4">
    <skos:altLabel xml:lang="de">Datenübertragungstechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13682-4">
    <skos:altLabel xml:lang="de">Datentechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12655-5">
    <skos:scopeNote xml:lang="de">Verzeichnis aller zur Herstellung eines übergeordneten Teils benötigten untergeordneten Teile mit ihren Mengenkoeffizienten.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13655-0">
    <skos:related rdf:resource="descriptor/18552-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13655-0">
    <skos:related rdf:resource="descriptor/16012-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13655-0">
    <skos:related rdf:resource="descriptor/13665-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13655-0">
    <skos:related rdf:resource="descriptor/18936-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13655-0">
    <skos:related rdf:resource="descriptor/18934-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13655-0">
    <skos:related rdf:resource="descriptor/14515-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13655-0">
    <skos:related rdf:resource="descriptor/18935-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19832-2">
    <skos:broader rdf:resource="thsys/73338"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19832-2">
    <skos:broader rdf:resource="thsys/71036"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19832-2">
    <skos:broader rdf:resource="descriptor/12079-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19832-2">
    <skos:broader rdf:resource="descriptor/12119-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12562-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18591-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14772-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13572-4">
    <skos:related rdf:resource="descriptor/13570-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10252-2">
    <skos:prefLabel xml:lang="en">Neoclassical economics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10252-2">
    <skos:prefLabel xml:lang="de">Neoklassik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18395-3">
    <skos:prefLabel xml:lang="en">Consumer goods marketing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18395-3">
    <skos:prefLabel xml:lang="de">Konsumgütermarketing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18345-4">
    <skos:narrower rdf:resource="descriptor/18357-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11708-1">
    <skos:narrower rdf:resource="descriptor/18154-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13360-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16030-4">
    <skos:altLabel xml:lang="de">Vormundschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16030-4">
    <skos:altLabel xml:lang="de">Familiengericht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16030-4">
    <skos:altLabel xml:lang="de">Elterliche Gewalt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16030-4">
    <skos:altLabel xml:lang="de">Adoption</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14821-4">
    <skos:altLabel xml:lang="de">Plastikverpackung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14821-4">
    <skos:altLabel xml:lang="de">Plastiktüte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14821-4">
    <skos:altLabel xml:lang="de">Plastikflasche</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14821-4">
    <skos:altLabel xml:lang="de">Plastikbeutel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14821-4">
    <skos:altLabel xml:lang="de">Kunststoffflasche</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14821-4">
    <skos:altLabel xml:lang="de">Kunststoffbeutel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70995">
    <skos:prefLabel xml:lang="de">W.06.01.05  Feinmechanische und Optische Industrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18539-5">
    <skos:scopeNote xml:lang="de">Management von Fremdwährungsrisiken, vor allem Absicherung von Wechselkursrisiken vorzugsweise durch Hedging oder Devisentermingeschäfte.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19424-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10993-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12188-3">
    <skos:prefLabel xml:lang="en">Company limited by shares</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12188-3">
    <skos:prefLabel xml:lang="de">Kommanditgesellschaft auf Aktien</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18535-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13441-5">
    <skos:related rdf:resource="descriptor/13808-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13441-5">
    <skos:related rdf:resource="descriptor/19194-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12387-4">
    <skos:altLabel xml:lang="de">Transitorische Rechnungsabgrenzung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12387-4">
    <skos:altLabel xml:lang="de">Rechnungsabgrenzungsposten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12387-4">
    <skos:altLabel xml:lang="de">Antizipative Rechnungsabgrenzung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12387-4">
    <skos:altLabel xml:lang="de">Abgrenzungsrechnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70096">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11007-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13312-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70822">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.30.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12910-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13971-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12181-3">
    <skos:related rdf:resource="descriptor/19157-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12181-3">
    <skos:related rdf:resource="descriptor/16026-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15606-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10347-5">
    <skos:prefLabel xml:lang="en">Business cycle theory</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10347-5">
    <skos:prefLabel xml:lang="de">Konjunkturtheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13988-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71117">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18267-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14945-0">
    <skos:broader rdf:resource="thsys/70498"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14945-0">
    <skos:broader rdf:resource="descriptor/14936-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14945-0">
    <skos:broader rdf:resource="descriptor/14926-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70649">
    <rdfs:label xml:lang="de">Kraftstoffe</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18484-4">
    <skos:narrower rdf:resource="descriptor/13888-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/g">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19722-2">
    <skos:altLabel xml:lang="de">Kriegswirkungen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19722-2">
    <skos:altLabel xml:lang="de">Kriegsfolgelasten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19722-2">
    <skos:altLabel xml:lang="de">Kriegsfolgeerscheinungen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19722-2">
    <skos:altLabel xml:lang="de">Kriegsfolgeerscheinung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11285-3">
    <skos:altLabel xml:lang="en">Salary workers</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12531-2">
    <skos:narrower rdf:resource="descriptor/13753-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12531-2">
    <skos:narrower rdf:resource="descriptor/20906-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12531-2">
    <skos:narrower rdf:resource="descriptor/12529-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11717-0">
    <skos:prefLabel xml:lang="en">Merit good</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11717-0">
    <skos:prefLabel xml:lang="de">Meritorisches Gut</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14317-1">
    <skos:broader rdf:resource="thsys/71006"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14317-1">
    <skos:broader rdf:resource="thsys/70936"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14317-1">
    <skos:broader rdf:resource="thsys/70415"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18992-6">
    <skos:scopeNote xml:lang="de">Staatliche Maßnahmen zur Steuerung der Spirituosenindustrie.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73388">
    <rdfs:label xml:lang="en">Religion</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73388">
    <rdfs:label xml:lang="de">Religion</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10638-1">
    <skos:prefLabel xml:lang="en">Factor price equalization</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10638-1">
    <skos:prefLabel xml:lang="de">Faktorpreisausgleich</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14060-5">
    <skos:related rdf:resource="descriptor/19024-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14060-5">
    <skos:related rdf:resource="descriptor/14934-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14060-5">
    <skos:related rdf:resource="descriptor/14205-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14060-5">
    <skos:related rdf:resource="descriptor/14074-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14060-5">
    <skos:related rdf:resource="descriptor/14936-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14060-5">
    <skos:related rdf:resource="descriptor/14068-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10166-2">
    <skos:scopeNote xml:lang="de">Der Utilitarismus sieht im Nützlichen die Grundlage sittlichen Handelns.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id79">
    <rdf:value xml:lang="de">Für Einproduktproduktion</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11237-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15671-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12257-3">
    <skos:related rdf:resource="descriptor/13736-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12257-3">
    <skos:related rdf:resource="descriptor/18750-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12257-3">
    <skos:related rdf:resource="descriptor/13703-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12257-3">
    <skos:related rdf:resource="descriptor/13775-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11711-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14101-3">
    <skos:broader rdf:resource="thsys/70169"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14101-3">
    <skos:broader rdf:resource="descriptor/14104-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13456-6">
    <skos:altLabel xml:lang="de">Einkaufskooperative</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18104-5">
    <skos:altLabel xml:lang="de">Zulieferindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18104-5">
    <skos:altLabel xml:lang="de">Zulieferer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18104-5">
    <skos:altLabel xml:lang="de">Zulieferbetrieb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18104-5">
    <skos:altLabel xml:lang="de">Unterlieferant</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18104-5">
    <skos:altLabel xml:lang="de">Subunternehmer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18104-5">
    <skos:altLabel xml:lang="de">Subcontracting</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18104-5">
    <skos:altLabel xml:lang="de">Lieferer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16486-3">
    <skos:broader rdf:resource="thsys/73373"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16486-3">
    <skos:broader rdf:resource="thsys/70572"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16486-3">
    <skos:broader rdf:resource="descriptor/16509-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15211-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19603-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11683-5">
    <skos:related rdf:resource="descriptor/19232-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11683-5">
    <skos:related rdf:resource="descriptor/11682-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11683-5">
    <skos:related rdf:resource="descriptor/19095-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13416-4">
    <skos:prefLabel xml:lang="en">Public pension system</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13416-4">
    <skos:prefLabel xml:lang="de">Gesetzliche Rentenversicherung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12820-2">
    <skos:prefLabel xml:lang="en">Extra pay</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12820-2">
    <skos:prefLabel xml:lang="de">Provision</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id10">
    <rdfs:seeAlso rdf:resource="descriptor/11357-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17941-0">
    <skos:altLabel xml:lang="de">Republik Nauru</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18172-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16550-6">
    <skos:altLabel xml:lang="en">Municipality (Administration)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16550-6">
    <skos:altLabel xml:lang="de">Stadtverwaltung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16550-6">
    <skos:altLabel xml:lang="de">Gemeindeverwaltung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20402-3">
    <skos:scopeNote xml:lang="de">Beachte auch: Kooperation.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10768-2">
    <skos:prefLabel xml:lang="en">Currency basket</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10768-2">
    <skos:prefLabel xml:lang="de">Währungskorb</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18211-4">
    <skos:broader rdf:resource="thsys/70265"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18211-4">
    <skos:broader rdf:resource="descriptor/13658-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18211-4">
    <skos:broader rdf:resource="descriptor/14812-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18517-1">
    <skos:broader rdf:resource="thsys/70960"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18517-1">
    <skos:broader rdf:resource="descriptor/13188-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16078-4">
    <skos:broader rdf:resource="thsys/73391"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16078-4">
    <skos:broader rdf:resource="thsys/70550"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16078-4">
    <skos:broader rdf:resource="descriptor/16053-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18992-6">
    <skos:related rdf:resource="descriptor/11633-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18992-6">
    <skos:related rdf:resource="descriptor/13152-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18992-6">
    <skos:related rdf:resource="descriptor/16666-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18992-6">
    <skos:related rdf:resource="descriptor/14972-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13047-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13840-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12786-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16150-1">
    <skos:altLabel xml:lang="en">Collective labour law</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16150-1">
    <skos:altLabel xml:lang="en">Collective labor agreement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16150-1">
    <skos:altLabel xml:lang="en">Collective bargaining legislation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16150-1">
    <skos:altLabel xml:lang="de">Tarifvertragsrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16150-1">
    <skos:altLabel xml:lang="de">Tarifvertragsgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16150-1">
    <skos:altLabel xml:lang="de">Kollektives Arbeitsrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16150-1">
    <skos:altLabel xml:lang="de">Kollektiver Arbeitsvertrag</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10714-4">
    <skos:altLabel xml:lang="en">GSP (Generalized System of Preferences)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10714-4">
    <skos:altLabel xml:lang="en">Generalized system of preferences</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10714-4">
    <skos:altLabel xml:lang="de">Vorzugszoll</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10714-4">
    <skos:altLabel xml:lang="de">Präferenzzone</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10714-4">
    <skos:altLabel xml:lang="de">Präferenzzoll</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12270-4">
    <skos:altLabel xml:lang="en">Net earnings</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20810-2">
    <skos:prefLabel xml:lang="en">European stability pact</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20810-2">
    <skos:prefLabel xml:lang="de">EU-Stabilitätspakt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17811-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16780-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12953-4">
    <skos:altLabel xml:lang="de">Schweinezucht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12953-4">
    <skos:altLabel xml:lang="de">Schweinemast</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12953-4">
    <skos:altLabel xml:lang="de">Ebermast</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12713-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14205-5">
    <skos:related rdf:resource="descriptor/14060-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14205-5">
    <skos:related rdf:resource="descriptor/14642-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19804-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id238">
    <rdfs:seeAlso rdf:resource="descriptor/19728-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18597-5">
    <skos:related rdf:resource="descriptor/16070-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18597-5">
    <skos:related rdf:resource="descriptor/18596-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18597-5">
    <skos:related rdf:resource="descriptor/16169-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70847">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70071">
    <skos:prefLabel xml:lang="de">P.01  Agrarprodukte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19281-6">
    <skos:related rdf:resource="descriptor/12876-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19281-6">
    <skos:related rdf:resource="descriptor/19410-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19281-6">
    <skos:related rdf:resource="descriptor/11636-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11042-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13756-1">
    <skos:narrower rdf:resource="descriptor/13769-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13756-1">
    <skos:narrower rdf:resource="descriptor/13761-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13756-1">
    <skos:narrower rdf:resource="descriptor/19556-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13756-1">
    <skos:narrower rdf:resource="descriptor/19282-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13756-1">
    <skos:narrower rdf:resource="descriptor/13776-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13756-1">
    <skos:narrower rdf:resource="descriptor/19137-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13756-1">
    <skos:narrower rdf:resource="descriptor/18141-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17016-0">
    <skos:broader rdf:resource="thsys/70131"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17016-0">
    <skos:broader rdf:resource="descriptor/16815-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12787-2">
    <skos:prefLabel xml:lang="en">Trade terms</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12787-2">
    <skos:prefLabel xml:lang="de">Handelsbrauch</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14004-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70996">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id151">
    <rdf:value xml:lang="de">Für Sozialstandards in Außenhandelsabkommen</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13390-3">
    <skos:altLabel xml:lang="de">Softwarehaus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13390-3">
    <skos:altLabel xml:lang="de">Softwareberatung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19339-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19166-6">
    <skos:scopeNote xml:lang="de">Für Literatur über das historische Korea bis 1948 und über beide koreanischen Staaten.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13100-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14385-5">
    <skos:prefLabel xml:lang="en">Paper machinery</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14385-5">
    <skos:prefLabel xml:lang="de">Papiermaschine</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13424-5">
    <skos:broader rdf:resource="thsys/73378"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13424-5">
    <skos:broader rdf:resource="thsys/73367"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13424-5">
    <skos:broader rdf:resource="thsys/70870"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13424-5">
    <skos:broader rdf:resource="descriptor/10984-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13424-5">
    <skos:broader rdf:resource="descriptor/13409-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11822-3">
    <skos:prefLabel xml:lang="en">Chamber of commerce</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11822-3">
    <skos:prefLabel xml:lang="de">Handelskammer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15964-5">
    <skos:altLabel xml:lang="de">Menschenrechtskonvention</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15964-5">
    <skos:altLabel xml:lang="de">Menschenrechts-Konvention</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15964-5">
    <skos:altLabel xml:lang="de">Menschenrechtscharta</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15964-5">
    <skos:altLabel xml:lang="de">Menschenrechts-Charta</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15964-5">
    <skos:altLabel xml:lang="de">Menschenrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15964-5">
    <skos:altLabel xml:lang="de">Charta der Menschenrechte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11792-0">
    <skos:altLabel xml:lang="de">Wirtschaftszweigkrise</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11792-0">
    <skos:altLabel xml:lang="de">Strukturkrise einer Branche</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11792-0">
    <skos:altLabel xml:lang="de">Sektorale Strukturkrise</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11792-0">
    <skos:altLabel xml:lang="de">Branchenstrukturkrise</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19742-3">
    <skos:prefLabel xml:lang="en">Axiomatic approach</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19742-3">
    <skos:prefLabel xml:lang="de">Axiomatisierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16349-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13413-3">
    <skos:altLabel xml:lang="en">Public authority</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13413-3">
    <skos:altLabel xml:lang="en">Administrative authority</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13413-3">
    <skos:altLabel xml:lang="de">Verwaltungsbehörde</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13413-3">
    <skos:altLabel xml:lang="de">Verwaltung (Öffentlich)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13413-3">
    <skos:altLabel xml:lang="de">Staatliche Verwaltung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13413-3">
    <skos:altLabel xml:lang="de">Leistungsverwaltung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13413-3">
    <skos:altLabel xml:lang="de">Hoheitsverwaltung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13413-3">
    <skos:altLabel xml:lang="de">Behörde</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13413-3">
    <skos:altLabel xml:lang="de">Amt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73382">
    <skos:prefLabel xml:lang="en">N.04.02  Political system</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73382">
    <skos:prefLabel xml:lang="de">N.04.02  Politisches System</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14450-6">
    <skos:broader rdf:resource="thsys/70415"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14450-6">
    <skos:broader rdf:resource="thsys/70337"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14450-6">
    <skos:broader rdf:resource="descriptor/14455-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14450-6">
    <skos:broader rdf:resource="descriptor/14413-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17684-1">
    <skos:prefLabel xml:lang="en">Namibia</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17684-1">
    <skos:prefLabel xml:lang="de">Namibia</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13384-5">
    <skos:prefLabel xml:lang="en">Recruitment services</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13384-5">
    <skos:prefLabel xml:lang="de">Personalberatung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15045-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14851-2">
    <skos:narrower rdf:resource="descriptor/14130-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14851-2">
    <skos:narrower rdf:resource="descriptor/14125-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14851-2">
    <skos:narrower rdf:resource="descriptor/14063-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14851-2">
    <skos:narrower rdf:resource="descriptor/19017-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14851-2">
    <skos:narrower rdf:resource="descriptor/14057-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14851-2">
    <skos:narrower rdf:resource="descriptor/14055-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18678-5">
    <skos:broader rdf:resource="thsys/70187"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18678-5">
    <skos:broader rdf:resource="descriptor/19784-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11246-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19000-5">
    <skos:broader rdf:resource="thsys/70307"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19000-5">
    <skos:broader rdf:resource="descriptor/17999-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16554-5">
    <skos:altLabel xml:lang="de">Empirische Sozialforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16554-5">
    <skos:altLabel xml:lang="de">Angewandte Sozialforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14176-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12320-1">
    <skos:related rdf:resource="descriptor/10028-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12320-1">
    <skos:related rdf:resource="descriptor/12312-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19748-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10018-3">
    <skos:broader rdf:resource="thsys/70171"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10018-3">
    <skos:broader rdf:resource="descriptor/10025-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13493-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17992-4">
    <skos:altLabel xml:lang="de">OAU-Länder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70575">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12806-3">
    <skos:prefLabel xml:lang="en">Product image</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12806-3">
    <skos:prefLabel xml:lang="de">Produktimage</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19395-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15904-2">
    <skos:prefLabel xml:lang="en">Elderly people</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15904-2">
    <skos:prefLabel xml:lang="de">Alte Menschen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71106">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.05.06.02.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16026-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19462-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13239-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14392-1">
    <skos:prefLabel xml:lang="en">Textile machinery</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14392-1">
    <skos:prefLabel xml:lang="de">Textilmaschine</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13580-5">
    <skos:broader rdf:resource="thsys/70942"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13580-5">
    <skos:broader rdf:resource="thsys/70928"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13580-5">
    <skos:broader rdf:resource="descriptor/10895-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15811-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18284-5">
    <skos:related rdf:resource="descriptor/17654-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10326-6">
    <skos:narrower rdf:resource="descriptor/10120-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10326-6">
    <skos:narrower rdf:resource="descriptor/11224-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10326-6">
    <skos:narrower rdf:resource="descriptor/12015-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10326-6">
    <skos:narrower rdf:resource="descriptor/11180-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10413-4">
    <skos:related rdf:resource="descriptor/10343-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10413-4">
    <skos:related rdf:resource="descriptor/10412-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19499-0">
    <skos:broader rdf:resource="thsys/70278"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19499-0">
    <skos:broader rdf:resource="descriptor/11322-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19161-2">
    <skos:related rdf:resource="descriptor/13992-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19161-2">
    <skos:related rdf:resource="descriptor/11040-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19161-2">
    <skos:related rdf:resource="descriptor/19522-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19161-2">
    <skos:related rdf:resource="descriptor/10005-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10246-4">
    <skos:broader rdf:resource="thsys/70234"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10246-4">
    <skos:broader rdf:resource="descriptor/10247-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12747-0">
    <zbwext:useInsteadNote rdf:nodeID="id2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10424-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/w">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19367-3">
    <skos:prefLabel xml:lang="en">Wood market</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19367-3">
    <skos:prefLabel xml:lang="de">Holzmarkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71065">
    <skos:broader rdf:resource="thsys/70058"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13499-2">
    <skos:broader rdf:resource="thsys/70925"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13499-2">
    <skos:broader rdf:resource="descriptor/13510-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18139-0">
    <skos:scopeNote xml:lang="de">Gesamtheit der Systemprogramme, z.B. UNIX, LINUX oder MS-Windows.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24666-5">
    <skos:scopeNote xml:lang="de">Die Fair-Value-Richtlinie der EU strebt eine Annäherung der Bilanzierung von Finanzinstrumenten an die IFRS an. Dabei wird Fair Value als Betrag verstanden, zu dem zwischen sachverständigen, vertragswilligen und voneinander unabhängigen Geschäftspartnern ein Vermögenswert getauscht oder eine Verbindlichkeit beglichen wird.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12884-4">
    <skos:broader rdf:resource="thsys/71041"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13712-0">
    <skos:prefLabel xml:lang="en">Stock index</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13712-0">
    <skos:prefLabel xml:lang="de">Aktienindex</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15878-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11802-2">
    <skos:prefLabel xml:lang="en">Agricultural price policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11802-2">
    <skos:prefLabel xml:lang="de">Agrarpreispolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10202-3">
    <skos:altLabel xml:lang="en">Segmented markets</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10202-3">
    <skos:altLabel xml:lang="de">Teilmarkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10202-3">
    <skos:altLabel xml:lang="de">Markteinteilung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18372-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18603-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10692-2">
    <skos:related rdf:resource="descriptor/19560-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10692-2">
    <skos:related rdf:resource="descriptor/10706-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10692-2">
    <skos:related rdf:resource="descriptor/10836-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15520-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12757-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13424-5">
    <skos:scopeNote xml:lang="de">Zusammenschluss zweier oder mehrerer Gebietskörperschaften zur Erledigung bestimmter Verwaltungsaufgaben auf überörtlicher Ebene.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19385-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73345">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10956-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11174-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11481-3">
    <skos:altLabel xml:lang="de">Währungsordnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11481-3">
    <skos:altLabel xml:lang="de">Münzregal</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11481-3">
    <skos:altLabel xml:lang="de">Geldordnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18097-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10300-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71063">
    <skos:narrower rdf:resource="descriptor/12012-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71063">
    <skos:narrower rdf:resource="descriptor/11828-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71063">
    <skos:narrower rdf:resource="descriptor/12013-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71063">
    <skos:narrower rdf:resource="descriptor/11723-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71063">
    <skos:narrower rdf:resource="descriptor/18663-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71063">
    <skos:narrower rdf:resource="descriptor/18813-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71063">
    <skos:narrower rdf:resource="descriptor/18803-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71063">
    <skos:narrower rdf:resource="descriptor/19678-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71063">
    <skos:narrower rdf:resource="descriptor/12010-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71063">
    <skos:narrower rdf:resource="descriptor/19463-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71063">
    <skos:narrower rdf:resource="descriptor/19632-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71063">
    <skos:narrower rdf:resource="descriptor/18800-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71063">
    <skos:narrower rdf:resource="descriptor/12009-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71063">
    <skos:narrower rdf:resource="descriptor/12004-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71063">
    <skos:narrower rdf:resource="descriptor/11552-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71063">
    <skos:narrower rdf:resource="descriptor/12001-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71063">
    <skos:narrower rdf:resource="descriptor/11716-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11020-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16430-2">
    <skos:altLabel xml:lang="de">Politische Moral</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16430-2">
    <skos:altLabel xml:lang="de">Ethik und Politik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14645-5">
    <skos:altLabel xml:lang="de">Salpeter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14645-5">
    <skos:altLabel xml:lang="de">Nitrit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14645-5">
    <skos:altLabel xml:lang="de">Nitrid</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id171">
    <rdf:value xml:lang="de">Für das Maß der Einkommensverteilung</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="http://zbw.eu/namespaces/skos-extensions/Descriptor">
    <rdfs:subClassOf rdf:resource="http://www.w3.org/2004/02/skos/core#Concept"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17571-0">
    <skos:related rdf:resource="descriptor/17589-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17571-0">
    <skos:related rdf:resource="descriptor/19172-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19159-3">
    <skos:related rdf:resource="descriptor/12302-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19159-3">
    <skos:related rdf:resource="descriptor/19667-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19159-3">
    <skos:related rdf:resource="descriptor/12058-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19159-3">
    <skos:related rdf:resource="descriptor/12057-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19159-3">
    <skos:related rdf:resource="descriptor/18791-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19159-3">
    <skos:related rdf:resource="descriptor/16058-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12073-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12142-6">
    <skos:prefLabel xml:lang="en">Competence</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12142-6">
    <skos:prefLabel xml:lang="de">Kompetenz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18005-0">
    <skos:broader rdf:resource="thsys/70824"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18005-0">
    <skos:broader rdf:resource="descriptor/13360-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18005-0">
    <skos:broader rdf:resource="descriptor/12064-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19101-6">
    <skos:prefLabel xml:lang="en">Activity-based costs</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19101-6">
    <skos:prefLabel xml:lang="de">Prozesskosten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11030-1">
    <skos:broader rdf:resource="thsys/71101"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11030-1">
    <skos:broader rdf:resource="descriptor/11033-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12693-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16951-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18286-1">
    <skos:narrower rdf:resource="descriptor/18024-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18286-1">
    <skos:narrower rdf:resource="descriptor/15741-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17477-6">
    <skos:prefLabel xml:lang="en">Lithuania</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17477-6">
    <skos:prefLabel xml:lang="de">Litauen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19092-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12106-3">
    <skos:related rdf:resource="descriptor/12091-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12106-3">
    <skos:related rdf:resource="descriptor/12617-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19533-5">
    <skos:broader rdf:resource="thsys/70180"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10202-3">
    <zbwext:useInsteadNote rdf:nodeID="id203"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13197-4">
    <skos:prefLabel xml:lang="en">Book trade</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13197-4">
    <skos:prefLabel xml:lang="de">Buchhandel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10622-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15010-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11388-0">
    <skos:related rdf:resource="descriptor/13411-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11388-0">
    <skos:related rdf:resource="descriptor/13415-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11388-0">
    <skos:related rdf:resource="descriptor/16531-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11388-0">
    <skos:related rdf:resource="descriptor/12615-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11388-0">
    <skos:related rdf:resource="descriptor/11269-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12928-3">
    <skos:scopeNote xml:lang="de">Unternehmen, die Produkte oder Techniken zum Schutz der Umwelt herstellen oder einschlägige Dienstleistungen anbieten.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10087-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13852-5">
    <skos:scopeNote xml:lang="de">Markt der Bauwirtschaft.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13303-3">
    <skos:related rdf:resource="descriptor/12931-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13303-3">
    <skos:related rdf:resource="descriptor/11718-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13303-3">
    <skos:related rdf:resource="descriptor/13413-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13303-3">
    <skos:related rdf:resource="descriptor/18803-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13303-3">
    <skos:related rdf:resource="descriptor/11236-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18894-6">
    <skos:related rdf:resource="descriptor/18895-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18894-6">
    <skos:related rdf:resource="descriptor/19465-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18894-6">
    <skos:related rdf:resource="descriptor/10498-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10173-5">
    <skos:related rdf:resource="descriptor/10176-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10173-5">
    <skos:related rdf:resource="descriptor/15941-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10173-5">
    <skos:related rdf:resource="descriptor/11273-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10173-5">
    <skos:related rdf:resource="descriptor/19205-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10173-5">
    <skos:related rdf:resource="descriptor/16698-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10173-5">
    <skos:related rdf:resource="descriptor/15938-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10173-5">
    <skos:related rdf:resource="descriptor/15937-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13611-6">
    <skos:altLabel xml:lang="de">Verkehrsträgerkooperation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13611-6">
    <skos:altLabel xml:lang="de">Transportbörse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13611-6">
    <skos:altLabel xml:lang="de">Frachtbörse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16373-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19845-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71007">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.05.01.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10077-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13120-4">
    <skos:altLabel xml:lang="de">Sägewerk</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12598-5">
    <skos:broader rdf:resource="thsys/70245"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12598-5">
    <skos:broader rdf:resource="descriptor/11341-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12598-5">
    <skos:broader rdf:resource="descriptor/11322-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11667-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12865-1">
    <skos:narrower rdf:resource="descriptor/13440-0"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id46">
    <rdf:value xml:lang="de">für die Lehrlingsausbildung im Betrieb</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70254">
    <skos:prefLabel xml:lang="de">P.10.03  Elektronische Produkte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19047-0">
    <skos:related rdf:resource="descriptor/12984-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19047-0">
    <skos:related rdf:resource="descriptor/18393-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10495-4">
    <skos:altLabel xml:lang="de">Nutrient</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10495-4">
    <skos:altLabel xml:lang="de">Nährstoff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12183-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18602-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14262-0">
    <zbwext:useInsteadNote rdf:nodeID="id75"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15873-1">
    <skos:prefLabel xml:lang="en">Geology</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15873-1">
    <skos:prefLabel xml:lang="de">Geologie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15082-4">
    <skos:editorialNote xml:lang="de">Vorwiegend zur Postkoordination verwenden.</skos:editorialNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14125-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18573-5">
    <skos:prefLabel xml:lang="en">Rail freight transport</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18573-5">
    <skos:prefLabel xml:lang="de">Eisenbahngüterverkehr</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13572-4">
    <skos:editorialNote xml:lang="de">Möglichst spezifischer indexieren.</skos:editorialNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19490-4">
    <skos:prefLabel xml:lang="en">Resource substitution</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19490-4">
    <skos:prefLabel xml:lang="de">Rohstoffsubstitution</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15706-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13518-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19181-3">
    <skos:prefLabel xml:lang="en">Oil price</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19181-3">
    <skos:prefLabel xml:lang="de">Erdölpreis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11680-4">
    <skos:altLabel xml:lang="en">Excise tax</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11680-4">
    <skos:altLabel xml:lang="en">Commodity tax</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11680-4">
    <skos:altLabel xml:lang="de">Verbrauchsteuerrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11680-4">
    <skos:altLabel xml:lang="de">Verbrauchsbesteuerung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11680-4">
    <skos:altLabel xml:lang="de">Verbrauchsabgaben</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11680-4">
    <skos:altLabel xml:lang="de">Produktsteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11680-4">
    <skos:altLabel xml:lang="de">Konsumsteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11236-2">
    <skos:altLabel xml:lang="de">Stellenvermittlung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11236-2">
    <skos:altLabel xml:lang="de">Förderung der Arbeitsaufnahme</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11236-2">
    <skos:altLabel xml:lang="de">Arbeitsnachweis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11236-2">
    <skos:altLabel xml:lang="de">Arbeitslosenberatung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11236-2">
    <skos:altLabel xml:lang="de">Arbeitsberatung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18329-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18843-2">
    <skos:related rdf:resource="descriptor/11036-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12594-6">
    <skos:prefLabel xml:lang="en">Performance pay scheme</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12594-6">
    <skos:prefLabel xml:lang="de">Leistungsorientierte Vergütung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12165-1">
    <skos:broader rdf:resource="thsys/71036"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12165-1">
    <skos:broader rdf:resource="thsys/70383"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12165-1">
    <skos:broader rdf:resource="descriptor/15598-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13509-4">
    <skos:altLabel xml:lang="de">Transitverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13509-4">
    <skos:altLabel xml:lang="de">Transithandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10728-0">
    <skos:prefLabel xml:lang="en">Bretton Woods System</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10728-0">
    <skos:prefLabel xml:lang="de">Bretton-Woods-System</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19842-6">
    <skos:prefLabel xml:lang="en">Distraint law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19842-6">
    <skos:prefLabel xml:lang="de">Zwangsvollstreckung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13593-3">
    <skos:broader rdf:resource="thsys/70925"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18381-0">
    <skos:narrower rdf:resource="descriptor/15751-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18381-0">
    <skos:narrower rdf:resource="descriptor/18286-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18381-0">
    <skos:narrower rdf:resource="descriptor/19488-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14934-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10342-1">
    <skos:altLabel xml:lang="de">Wirtschaftsverlauf</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10342-1">
    <skos:altLabel xml:lang="de">Wirtschaftsbeschreibung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10342-1">
    <skos:altLabel xml:lang="de">Wirtschaftsbericht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10342-1">
    <skos:altLabel xml:lang="de">Wirtschaftliche Lage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10342-1">
    <skos:altLabel xml:lang="de">Economic conditions</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10833-3">
    <skos:related rdf:resource="descriptor/19357-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10833-3">
    <skos:related rdf:resource="descriptor/18126-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10833-3">
    <skos:related rdf:resource="descriptor/10656-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10833-3">
    <skos:related rdf:resource="descriptor/18536-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10833-3">
    <skos:related rdf:resource="descriptor/10604-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10833-3">
    <skos:related rdf:resource="descriptor/10835-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10833-3">
    <skos:related rdf:resource="descriptor/10686-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10833-3">
    <skos:related rdf:resource="descriptor/18534-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10833-3">
    <skos:related rdf:resource="descriptor/10859-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10833-3">
    <skos:related rdf:resource="descriptor/18008-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16271-3">
    <skos:prefLabel xml:lang="en">Military-industrial complex</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16271-3">
    <skos:prefLabel xml:lang="de">Militärisch-industrieller Komplex</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19053-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10686-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13034-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17962-6">
    <skos:prefLabel xml:lang="en">Mediterranean Sea</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17962-6">
    <skos:prefLabel xml:lang="de">Mittelmeer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70581">
    <skos:narrower rdf:resource="descriptor/16599-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70581">
    <skos:narrower rdf:resource="descriptor/15683-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70581">
    <skos:narrower rdf:resource="descriptor/15675-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70581">
    <skos:narrower rdf:resource="descriptor/19750-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70581">
    <skos:narrower rdf:resource="descriptor/15668-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70581">
    <skos:narrower rdf:resource="descriptor/15661-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70581">
    <skos:narrower rdf:resource="descriptor/18295-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70581">
    <skos:narrower rdf:resource="descriptor/15657-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70581">
    <skos:narrower rdf:resource="descriptor/14040-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70581">
    <skos:narrower rdf:resource="descriptor/15655-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70581">
    <skos:narrower rdf:resource="descriptor/19781-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70581">
    <skos:narrower rdf:resource="descriptor/15694-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70581">
    <skos:narrower rdf:resource="descriptor/15654-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70581">
    <skos:narrower rdf:resource="descriptor/15643-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70581">
    <skos:narrower rdf:resource="descriptor/19780-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70581">
    <skos:narrower rdf:resource="descriptor/19747-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70581">
    <skos:narrower rdf:resource="descriptor/18492-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11915-3">
    <skos:narrower rdf:resource="descriptor/11904-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16760-2">
    <skos:narrower rdf:resource="descriptor/16782-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16760-2">
    <skos:narrower rdf:resource="descriptor/16771-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15437-0">
    <skos:altLabel xml:lang="en">Multistage game</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15437-0">
    <skos:altLabel xml:lang="de">Mehrstufiges Spiel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15767-0">
    <skos:related rdf:resource="descriptor/10111-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15767-0">
    <skos:related rdf:resource="descriptor/12928-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15767-0">
    <skos:related rdf:resource="descriptor/15773-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15767-0">
    <skos:related rdf:resource="descriptor/15771-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15767-0">
    <skos:related rdf:resource="descriptor/15770-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15767-0">
    <skos:related rdf:resource="descriptor/15768-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15767-0">
    <skos:related rdf:resource="descriptor/18165-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15767-0">
    <skos:related rdf:resource="descriptor/19633-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15767-0">
    <skos:related rdf:resource="descriptor/15765-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15767-0">
    <skos:related rdf:resource="descriptor/19442-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15767-0">
    <skos:related rdf:resource="descriptor/18183-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15767-0">
    <skos:related rdf:resource="descriptor/18325-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15767-0">
    <skos:related rdf:resource="descriptor/10097-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15767-0">
    <skos:related rdf:resource="descriptor/10092-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18131-2">
    <skos:altLabel xml:lang="de">Östliche Bundesländer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18131-2">
    <skos:altLabel xml:lang="de">Ostdeutschland</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18131-2">
    <skos:altLabel xml:lang="de">FNL (Fünf neue Länder)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18131-2">
    <skos:altLabel xml:lang="de">Ehemalige DDR</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18131-2">
    <skos:altLabel xml:lang="de">Deutschland (Neue Bundesländer)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18131-2">
    <skos:altLabel xml:lang="de">Beitrittsgebiet</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21408-0">
    <skos:altLabel xml:lang="de">Kupon (Wertgutschein)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21408-0">
    <skos:altLabel xml:lang="de">Gutschein-Werbung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21408-0">
    <skos:altLabel xml:lang="de">Coupon-Marketing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21408-0">
    <skos:altLabel xml:lang="de">Coupon (Wertgutschein)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19269-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13075-4">
    <skos:altLabel xml:lang="de">Haushaltsgeräteindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16544-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10460-2">
    <skos:related rdf:resource="descriptor/10114-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10460-2">
    <skos:related rdf:resource="descriptor/10239-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10460-2">
    <skos:related rdf:resource="descriptor/12073-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10460-2">
    <skos:related rdf:resource="descriptor/10466-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10460-2">
    <skos:related rdf:resource="descriptor/10332-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10460-2">
    <skos:related rdf:resource="descriptor/12295-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10460-2">
    <skos:related rdf:resource="descriptor/10123-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18859-1">
    <skos:prefLabel xml:lang="en">Behaviour</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18859-1">
    <skos:prefLabel xml:lang="de">Verhalten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11983-0">
    <skos:altLabel xml:lang="en">Urban crisis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11983-0">
    <skos:altLabel xml:lang="de">Stadtkrise</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11983-0">
    <skos:altLabel xml:lang="de">Städtische Krise</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10010-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21011-6">
    <skos:related rdf:resource="descriptor/11636-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16325-6">
    <skos:prefLabel xml:lang="en">Air force</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16325-6">
    <skos:prefLabel xml:lang="de">Luftwaffe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18312-5">
    <skos:related rdf:resource="descriptor/16187-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18312-5">
    <skos:related rdf:resource="descriptor/19631-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18312-5">
    <skos:related rdf:resource="descriptor/19616-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18312-5">
    <skos:related rdf:resource="descriptor/19626-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18312-5">
    <skos:related rdf:resource="descriptor/11807-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18312-5">
    <skos:related rdf:resource="descriptor/12964-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18312-5">
    <skos:related rdf:resource="descriptor/13426-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17935-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13003-1">
    <skos:narrower rdf:resource="descriptor/12992-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13003-1">
    <skos:narrower rdf:resource="descriptor/12976-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70944">
    <skos:broader rdf:resource="thsys/w"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18302-1">
    <skos:broader rdf:resource="thsys/73337"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18302-1">
    <skos:broader rdf:resource="thsys/71043"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18302-1">
    <skos:broader rdf:resource="thsys/70397"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18302-1">
    <skos:broader rdf:resource="descriptor/11640-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18772-6">
    <skos:broader rdf:resource="thsys/73362"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18772-6">
    <skos:broader rdf:resource="descriptor/16435-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19617-6">
    <skos:broader rdf:resource="thsys/73353"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19617-6">
    <skos:broader rdf:resource="descriptor/15887-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16651-0">
    <skos:broader rdf:resource="thsys/73359"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12668-3">
    <skos:prefLabel xml:lang="en">Buying group</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12668-3">
    <skos:prefLabel xml:lang="de">Einkaufskooperation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16372-4">
    <skos:altLabel xml:lang="de">Rechtsextreme Partei</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15717-1">
    <skos:broader rdf:resource="thsys/73395"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12134-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19719-5">
    <skos:prefLabel xml:lang="en">Pan-Americanism</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19719-5">
    <skos:prefLabel xml:lang="de">Panamerikanismus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12591-5">
    <skos:prefLabel xml:lang="en">Occupational pension scheme</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12591-5">
    <skos:prefLabel xml:lang="de">Betriebliche Altersversorgung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12748-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19311-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14958-5">
    <skos:related rdf:resource="descriptor/18077-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14958-5">
    <skos:related rdf:resource="descriptor/13153-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18338-1">
    <skos:broader rdf:resource="thsys/71110"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18338-1">
    <skos:broader rdf:resource="thsys/71019"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18338-1">
    <skos:broader rdf:resource="descriptor/10981-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18756-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18353-5">
    <skos:narrower rdf:resource="descriptor/18359-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10653-5">
    <skos:prefLabel xml:lang="en">Tradable</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10653-5">
    <skos:prefLabel xml:lang="de">Handelbares Gut</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11571-2">
    <skos:narrower rdf:resource="descriptor/11563-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18149-4">
    <skos:prefLabel xml:lang="en">Seigniorage</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18149-4">
    <skos:prefLabel xml:lang="de">Münzgewinn</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13694-4">
    <skos:broader rdf:resource="thsys/73390"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13694-4">
    <skos:broader rdf:resource="thsys/70905"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13694-4">
    <skos:broader rdf:resource="descriptor/12277-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13694-4">
    <skos:broader rdf:resource="descriptor/13693-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18255-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17413-4">
    <skos:prefLabel xml:lang="en">Ireland</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17413-4">
    <skos:prefLabel xml:lang="de">Irland</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16011-1">
    <skos:prefLabel xml:lang="en">Right to work</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16011-1">
    <skos:prefLabel xml:lang="de">Recht auf Arbeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70154">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10772-4">
    <zbwext:useInsteadNote rdf:nodeID="id358"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18908-0">
    <skos:altLabel xml:lang="en">Debt advice</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18908-0">
    <skos:altLabel xml:lang="en">Credit counseling</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18908-0">
    <skos:altLabel xml:lang="en">Consumer credit counselling</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10753-1">
    <skos:broader rdf:resource="thsys/71095"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10753-1">
    <skos:broader rdf:resource="descriptor/10764-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15876-2">
    <skos:altLabel xml:lang="de">Mineralogie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15876-2">
    <skos:altLabel xml:lang="de">Geophysik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18652-2">
    <skos:broader rdf:resource="thsys/70922"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18652-2">
    <skos:broader rdf:resource="descriptor/18646-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12863-5">
    <skos:broader rdf:resource="thsys/73337"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12863-5">
    <skos:broader rdf:resource="thsys/71040"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12863-5">
    <skos:broader rdf:resource="descriptor/10911-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11165-6">
    <skos:prefLabel xml:lang="en">Median voter</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11165-6">
    <skos:prefLabel xml:lang="de">Median Voter</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70467">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18648-0">
    <skos:related rdf:resource="descriptor/19651-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18648-0">
    <skos:related rdf:resource="descriptor/19544-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18648-0">
    <skos:related rdf:resource="descriptor/19653-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18648-0">
    <skos:related rdf:resource="descriptor/19541-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15897-1">
    <skos:broader rdf:resource="thsys/73351"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15897-1">
    <skos:broader rdf:resource="thsys/71019"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15897-1">
    <skos:broader rdf:resource="descriptor/15745-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17940-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16699-0">
    <skos:broader rdf:resource="thsys/73357"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16699-0">
    <skos:broader rdf:resource="descriptor/15712-4"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id116">
    <rdfs:seeAlso rdf:resource="descriptor/11457-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21020-5">
    <skos:prefLabel xml:lang="en">Supply chain management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21020-5">
    <skos:prefLabel xml:lang="de">Supply Chain Management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11473-2">
    <skos:prefLabel xml:lang="en">Quantity theory of money</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11473-2">
    <skos:prefLabel xml:lang="de">Quantitätstheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17951-4">
    <skos:broader rdf:resource="thsys/70128"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17951-4">
    <skos:broader rdf:resource="descriptor/17948-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13142-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17632-6">
    <skos:prefLabel xml:lang="en">Oman</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17632-6">
    <skos:prefLabel xml:lang="de">Oman</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10852-6">
    <skos:scopeNote xml:lang="de">Gesamtheit der im- und exportierenden Unternehmen sowie der damit verbundenen Dienstleistungsunternehmen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19491-2">
    <skos:related rdf:resource="descriptor/19859-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19491-2">
    <skos:related rdf:resource="descriptor/11282-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19491-2">
    <skos:related rdf:resource="descriptor/11258-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19491-2">
    <skos:related rdf:resource="descriptor/11230-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19491-2">
    <skos:related rdf:resource="descriptor/19005-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19491-2">
    <skos:related rdf:resource="descriptor/18792-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19491-2">
    <skos:related rdf:resource="descriptor/11185-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19491-2">
    <skos:related rdf:resource="descriptor/19493-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16768-0">
    <skos:scopeNote xml:lang="de">Nur für Literatur über Bibliographien.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11339-6">
    <skos:broader rdf:resource="thsys/71051"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11339-6">
    <skos:broader rdf:resource="thsys/70235"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11339-6">
    <skos:broader rdf:resource="descriptor/15719-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13881-5">
    <skos:altLabel xml:lang="de">Bergleute</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13881-5">
    <skos:altLabel xml:lang="de">Bergingenieurinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13881-5">
    <skos:altLabel xml:lang="de">Bergingenieure</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13881-5">
    <skos:altLabel xml:lang="de">Bergarbeiterinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13881-5">
    <skos:altLabel xml:lang="de">Bergarbeiter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13881-5">
    <skos:altLabel xml:lang="de">Bergakademikerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13881-5">
    <skos:altLabel xml:lang="de">Bergakademiker</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13160-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19198-0">
    <skos:prefLabel xml:lang="en">Credibility</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19198-0">
    <skos:prefLabel xml:lang="de">Glaubwürdigkeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71039">
    <skos:narrower rdf:resource="descriptor/12172-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71039">
    <skos:narrower rdf:resource="descriptor/15381-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71039">
    <skos:narrower rdf:resource="descriptor/15798-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71039">
    <skos:narrower rdf:resource="descriptor/15598-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71039">
    <skos:narrower rdf:resource="descriptor/16754-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71039">
    <skos:narrower rdf:resource="descriptor/15609-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71039">
    <skos:narrower rdf:resource="descriptor/15606-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71039">
    <skos:narrower rdf:resource="thsys/71038"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71039">
    <skos:narrower rdf:resource="thsys/71036"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71039">
    <skos:narrower rdf:resource="thsys/71037"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72185">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14739-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11360-1">
    <skos:narrower rdf:resource="descriptor/11391-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11360-1">
    <skos:narrower rdf:resource="descriptor/11366-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11360-1">
    <skos:narrower rdf:resource="descriptor/11393-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11360-1">
    <skos:narrower rdf:resource="descriptor/11362-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11360-1">
    <skos:narrower rdf:resource="descriptor/11361-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11360-1">
    <skos:narrower rdf:resource="descriptor/19656-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70113">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18313-3">
    <skos:scopeNote xml:lang="de">Kein genau abgrenzbarer Begriff.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11101-4">
    <skos:prefLabel xml:lang="en">State capitalism</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11101-4">
    <skos:prefLabel xml:lang="de">Staatskapitalismus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12766-3">
    <skos:prefLabel xml:lang="en">Marketing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12766-3">
    <skos:prefLabel xml:lang="de">Marketing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18859-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18268-3">
    <skos:altLabel xml:lang="de">Raumspray</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11541-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19282-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18165-6">
    <skos:scopeNote xml:lang="de">Nur für Umweltschutzmaßnahmen der Unternehmen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71038">
    <skos:broader rdf:resource="thsys/71039"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16209-1">
    <skos:prefLabel xml:lang="en">Political science</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16209-1">
    <skos:prefLabel xml:lang="de">Politikwissenschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19757-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19236-4">
    <skos:altLabel xml:lang="de">Verbrauchsausgaben</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19236-4">
    <skos:altLabel xml:lang="de">Verbraucherausgabe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19236-4">
    <skos:altLabel xml:lang="de">Private Verbrauchsausgaben</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19236-4">
    <skos:altLabel xml:lang="de">Private Konsumausgaben</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19236-4">
    <skos:altLabel xml:lang="de">Konsumausgaben</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18901-0">
    <skos:altLabel xml:lang="de">Viehseuche</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19721-4">
    <skos:prefLabel xml:lang="en">War victims</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19721-4">
    <skos:prefLabel xml:lang="de">Kriegsopfer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19012-5">
    <skos:prefLabel xml:lang="en">Unit labour costs</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19012-5">
    <skos:prefLabel xml:lang="de">Lohnstückkosten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11724-3">
    <skos:broader rdf:resource="thsys/71081"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11724-3">
    <skos:broader rdf:resource="thsys/70858"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11724-3">
    <skos:broader rdf:resource="descriptor/11721-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24662-6">
    <skos:related rdf:resource="descriptor/10962-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24662-6">
    <skos:related rdf:resource="descriptor/18828-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24662-6">
    <skos:related rdf:resource="descriptor/19678-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24662-6">
    <skos:related rdf:resource="descriptor/10208-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18527-5">
    <skos:related rdf:resource="descriptor/12778-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18527-5">
    <skos:related rdf:resource="descriptor/13479-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18527-5">
    <skos:related rdf:resource="descriptor/13469-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18527-5">
    <skos:related rdf:resource="descriptor/12781-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18811-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18607-0">
    <skos:prefLabel xml:lang="en">Aviation safety</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18607-0">
    <skos:prefLabel xml:lang="de">Luftverkehrssicherheit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12205-1">
    <skos:altLabel xml:lang="de">Unternehmens-Finanzierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12205-1">
    <skos:altLabel xml:lang="de">Kapitalbeschaffung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11211-4">
    <skos:narrower rdf:resource="descriptor/11233-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11211-4">
    <skos:narrower rdf:resource="descriptor/11231-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11211-4">
    <skos:narrower rdf:resource="descriptor/11230-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15735-6">
    <skos:related rdf:resource="descriptor/15738-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11260-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18371-3">
    <skos:altLabel xml:lang="en">Gas policy</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18371-3">
    <skos:altLabel xml:lang="de">Gaspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16726-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11220-3">
    <skos:altLabel xml:lang="en">Labour market disequilibrium model</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11220-3">
    <skos:altLabel xml:lang="en">Labor market disequilibrium model</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11220-3">
    <skos:altLabel xml:lang="en">Labor market disequilibrium</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71011">
    <rdfs:label xml:lang="en">Water industry</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71011">
    <rdfs:label xml:lang="de">Wasserwirtschaft</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18871-4">
    <skos:altLabel xml:lang="de">Wirtschaftswissenschaftliches Studium</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18871-4">
    <skos:altLabel xml:lang="de">Volkswirtschaftsstudium</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18871-4">
    <skos:altLabel xml:lang="de">Studium der Wirtschaftswissenschaften</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18871-4">
    <skos:altLabel xml:lang="de">Ökonomiestudium</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16630-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15063-1">
    <skos:related rdf:resource="descriptor/10147-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15063-1">
    <skos:related rdf:resource="descriptor/15243-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15063-1">
    <skos:related rdf:resource="descriptor/15298-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15063-1">
    <skos:related rdf:resource="descriptor/15272-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13769-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13546-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16849-0">
    <skos:prefLabel xml:lang="en">Bremen (State)</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16849-0">
    <skos:prefLabel xml:lang="de">Bremen (Land)</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14522-0">
    <skos:prefLabel xml:lang="en">Audiovisual product</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14522-0">
    <skos:prefLabel xml:lang="de">Unterhaltungselektronik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16176-4">
    <skos:altLabel xml:lang="de">Privates Baurecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16176-4">
    <skos:altLabel xml:lang="de">Öffentliches Baurecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16176-4">
    <skos:altLabel xml:lang="de">Bundesbaugesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16176-4">
    <skos:altLabel xml:lang="de">Bauvertragsrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16176-4">
    <skos:altLabel xml:lang="de">Baupolizeiverordnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16176-4">
    <skos:altLabel xml:lang="de">Bauordnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16176-4">
    <skos:altLabel xml:lang="de">Baunutzungsverordnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16176-4">
    <skos:altLabel xml:lang="de">Baugesetzbuch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16176-4">
    <skos:altLabel xml:lang="de">Baugesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16176-4">
    <skos:altLabel xml:lang="de">BauGB (Baugesetzbuch)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14328-3">
    <skos:broader rdf:resource="thsys/70318"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14328-3">
    <skos:broader rdf:resource="descriptor/14380-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16825-0">
    <skos:related rdf:resource="descriptor/19084-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71036">
    <skos:narrower rdf:resource="descriptor/20241-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71036">
    <skos:narrower rdf:resource="descriptor/19633-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71036">
    <skos:narrower rdf:resource="descriptor/13673-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71036">
    <skos:narrower rdf:resource="descriptor/18112-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71036">
    <skos:narrower rdf:resource="descriptor/21020-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71036">
    <skos:narrower rdf:resource="descriptor/12737-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71036">
    <skos:narrower rdf:resource="descriptor/12564-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71036">
    <skos:narrower rdf:resource="descriptor/12768-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71036">
    <skos:narrower rdf:resource="descriptor/12079-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71036">
    <skos:narrower rdf:resource="descriptor/19114-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71036">
    <skos:narrower rdf:resource="descriptor/12473-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71036">
    <skos:narrower rdf:resource="descriptor/19832-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71036">
    <skos:narrower rdf:resource="descriptor/12735-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71036">
    <skos:narrower rdf:resource="descriptor/19833-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71036">
    <skos:narrower rdf:resource="descriptor/18561-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71036">
    <skos:narrower rdf:resource="descriptor/18940-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71036">
    <skos:narrower rdf:resource="descriptor/19544-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71036">
    <skos:narrower rdf:resource="descriptor/18767-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71036">
    <skos:narrower rdf:resource="descriptor/19654-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71036">
    <skos:narrower rdf:resource="descriptor/19653-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71036">
    <skos:narrower rdf:resource="descriptor/19104-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71036">
    <skos:narrower rdf:resource="descriptor/12883-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71036">
    <skos:narrower rdf:resource="descriptor/19115-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71036">
    <skos:narrower rdf:resource="descriptor/12330-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71036">
    <skos:narrower rdf:resource="descriptor/18002-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71036">
    <skos:narrower rdf:resource="descriptor/19120-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71036">
    <skos:narrower rdf:resource="descriptor/19117-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71036">
    <skos:narrower rdf:resource="descriptor/12732-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71036">
    <skos:narrower rdf:resource="descriptor/12733-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71036">
    <skos:narrower rdf:resource="descriptor/13671-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71036">
    <skos:narrower rdf:resource="descriptor/12165-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71036">
    <skos:narrower rdf:resource="descriptor/12730-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71036">
    <skos:narrower rdf:resource="descriptor/19660-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73339">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19227-5">
    <skos:broader rdf:resource="thsys/71117"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19507-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10324-3">
    <skos:altLabel xml:lang="en">Multi-stage production</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10324-3">
    <skos:altLabel xml:lang="de">Zweistufige Produktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13827-4">
    <skos:broader rdf:resource="thsys/70892"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12502-2">
    <skos:altLabel xml:lang="en">Non-wage labor costs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12502-2">
    <skos:altLabel xml:lang="en">Indirect labour costs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12502-2">
    <skos:altLabel xml:lang="en">Indirect labor costs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12502-2">
    <skos:altLabel xml:lang="de">Personalnebenkosten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12502-2">
    <skos:altLabel xml:lang="de">Lohnnebenkosten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11526-0">
    <skos:prefLabel xml:lang="en">Economics of public finance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11526-0">
    <skos:prefLabel xml:lang="de">Finanzwissenschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13351-6">
    <skos:altLabel xml:lang="de">Bücherei</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13351-6">
    <skos:altLabel xml:lang="de">Bibliothekswesen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16742-4">
    <skos:broader rdf:resource="thsys/70141"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16742-4">
    <skos:broader rdf:resource="descriptor/16745-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20987-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70141">
    <skos:broader rdf:resource="thsys/70049"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10928-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14269-0">
    <skos:broader rdf:resource="thsys/70346"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13443-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11339-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10677-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17608-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13742-5">
    <skos:altLabel xml:lang="de">Wertpapiermarkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13742-5">
    <skos:altLabel xml:lang="de">Wertpapierkommissionsgeschäft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13742-5">
    <skos:altLabel xml:lang="de">Tafelgeschäft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13742-5">
    <skos:altLabel xml:lang="de">Effektenkommissionsgeschäft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13742-5">
    <skos:altLabel xml:lang="de">Effektenhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11394-5">
    <skos:altLabel xml:lang="de">Empirische Bildungsforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11394-5">
    <skos:altLabel xml:lang="de">Bildungssoziologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19083-3">
    <skos:prefLabel xml:lang="en">Northeast Europe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19083-3">
    <skos:prefLabel xml:lang="de">Nordosteuropa</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15874-6">
    <skos:broader rdf:resource="thsys/73354"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15874-6">
    <skos:broader rdf:resource="descriptor/15873-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15874-6">
    <skos:broader rdf:resource="descriptor/15872-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14905-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12915-5">
    <skos:related rdf:resource="descriptor/18122-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12915-5">
    <skos:related rdf:resource="descriptor/12750-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12915-5">
    <skos:related rdf:resource="descriptor/12914-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15479-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19266-2">
    <skos:altLabel xml:lang="de">Zinsgeographie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19266-2">
    <skos:altLabel xml:lang="de">Regionale Zinsdifferenz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19266-2">
    <skos:altLabel xml:lang="de">Räumliche Zinsdifferenz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19266-2">
    <skos:altLabel xml:lang="de">Internationale Zinsdifferenz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id370">
    <rdfs:seeAlso rdf:resource="descriptor/11897-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15744-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14787-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15765-4">
    <skos:related rdf:resource="descriptor/15774-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15765-4">
    <skos:related rdf:resource="descriptor/12928-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15765-4">
    <skos:related rdf:resource="descriptor/15767-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15765-4">
    <skos:related rdf:resource="descriptor/15759-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15765-4">
    <skos:related rdf:resource="descriptor/15664-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15765-4">
    <skos:related rdf:resource="descriptor/15633-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19520-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12742-3">
    <skos:broader rdf:resource="thsys/70270"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12742-3">
    <skos:broader rdf:resource="descriptor/12041-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12039-6">
    <zbwext:useInsteadNote rdf:nodeID="id54"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12778-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id11">
    <rdfs:seeAlso rdf:resource="descriptor/18907-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19127-2">
    <skos:broader rdf:resource="thsys/73391"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19127-2">
    <skos:broader rdf:resource="thsys/70040"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19127-2">
    <skos:broader rdf:resource="descriptor/12785-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19127-2">
    <skos:broader rdf:resource="descriptor/12787-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16303-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15923-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18745-2">
    <skos:related rdf:resource="descriptor/19411-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18745-2">
    <skos:related rdf:resource="descriptor/11994-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18745-2">
    <skos:related rdf:resource="descriptor/16033-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18745-2">
    <skos:related rdf:resource="descriptor/13837-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18745-2">
    <skos:related rdf:resource="descriptor/12031-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18745-2">
    <skos:related rdf:resource="descriptor/13853-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12031-1">
    <skos:related rdf:resource="descriptor/18747-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12031-1">
    <skos:related rdf:resource="descriptor/18745-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12031-1">
    <skos:related rdf:resource="descriptor/12029-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12031-1">
    <skos:related rdf:resource="descriptor/12023-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12031-1">
    <skos:related rdf:resource="descriptor/13852-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19465-3">
    <skos:broader rdf:resource="thsys/71065"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19465-3">
    <skos:broader rdf:resource="thsys/70573"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19465-3">
    <skos:broader rdf:resource="descriptor/13371-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10214-3">
    <skos:broader rdf:resource="thsys/70178"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10214-3">
    <skos:broader rdf:resource="descriptor/10217-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18991-1">
    <skos:narrower rdf:resource="descriptor/19677-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16401-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id342">
    <rdf:value xml:lang="de">Für Kunden/Kundinnen als Adressaten betrieblicher Marketingmaßnahmen</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14073-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19362-6">
    <skos:related rdf:resource="descriptor/18982-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19362-6">
    <skos:related rdf:resource="descriptor/19233-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19362-6">
    <skos:related rdf:resource="descriptor/19461-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19362-6">
    <skos:related rdf:resource="descriptor/10504-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15163-4">
    <skos:broader rdf:resource="thsys/72185"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15163-4">
    <skos:broader rdf:resource="descriptor/15060-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15535-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13374-1">
    <skos:prefLabel xml:lang="en">Veterinary medicine</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13374-1">
    <skos:prefLabel xml:lang="de">Veterinärwesen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21041-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17721-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13712-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13299-3">
    <skos:altLabel xml:lang="de">Transport-Warenversicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13299-3">
    <skos:altLabel xml:lang="de">Frachtversicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10464-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19317-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10865-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16550-6">
    <skos:broader rdf:resource="thsys/73380"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16550-6">
    <skos:broader rdf:resource="thsys/73367"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16550-6">
    <skos:broader rdf:resource="thsys/70870"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16550-6">
    <skos:broader rdf:resource="descriptor/13413-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13014-3">
    <skos:editorialNote xml:lang="de">Möglichst spezifischer indexieren</skos:editorialNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14107-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13018-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71074">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70878">
    <skos:narrower rdf:resource="descriptor/16214-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70878">
    <skos:narrower rdf:resource="descriptor/15686-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70878">
    <skos:narrower rdf:resource="descriptor/18780-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70878">
    <skos:narrower rdf:resource="descriptor/10037-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70878">
    <skos:narrower rdf:resource="descriptor/18783-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70878">
    <skos:narrower rdf:resource="descriptor/15900-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70878">
    <skos:narrower rdf:resource="descriptor/15861-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70878">
    <skos:narrower rdf:resource="descriptor/15765-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70878">
    <skos:narrower rdf:resource="descriptor/19466-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70878">
    <skos:narrower rdf:resource="descriptor/18651-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70878">
    <skos:narrower rdf:resource="descriptor/11986-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70878">
    <skos:narrower rdf:resource="descriptor/16554-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70878">
    <skos:narrower rdf:resource="descriptor/18814-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70878">
    <skos:narrower rdf:resource="descriptor/11951-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70878">
    <skos:narrower rdf:resource="descriptor/12106-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70878">
    <skos:narrower rdf:resource="descriptor/10458-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70878">
    <skos:narrower rdf:resource="descriptor/18418-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70878">
    <skos:narrower rdf:resource="descriptor/18442-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70878">
    <skos:narrower rdf:resource="descriptor/16215-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70878">
    <skos:narrower rdf:resource="descriptor/18423-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70878">
    <skos:narrower rdf:resource="descriptor/10361-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70878">
    <skos:narrower rdf:resource="descriptor/18461-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70878">
    <skos:narrower rdf:resource="descriptor/18781-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70878">
    <skos:narrower rdf:resource="descriptor/18782-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70878">
    <skos:narrower rdf:resource="descriptor/10450-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70878">
    <skos:narrower rdf:resource="descriptor/16204-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70878">
    <skos:narrower rdf:resource="descriptor/16678-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70878">
    <skos:narrower rdf:resource="descriptor/19361-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70878">
    <skos:narrower rdf:resource="descriptor/18348-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70878">
    <skos:narrower rdf:resource="descriptor/18430-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70878">
    <skos:narrower rdf:resource="descriptor/11394-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70878">
    <skos:narrower rdf:resource="descriptor/10443-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70878">
    <skos:narrower rdf:resource="descriptor/11293-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70878">
    <skos:narrower rdf:resource="descriptor/18384-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70878">
    <skos:narrower rdf:resource="descriptor/18491-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70878">
    <skos:narrower rdf:resource="descriptor/16697-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16563-4">
    <skos:broader rdf:resource="thsys/70012"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16563-4">
    <skos:broader rdf:resource="descriptor/15685-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18215-3">
    <skos:broader rdf:resource="thsys/70074"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18215-3">
    <skos:broader rdf:resource="descriptor/18216-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70956">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.10.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20381-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18181-1">
    <skos:prefLabel xml:lang="en">Economic regionalism</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18181-1">
    <skos:prefLabel xml:lang="de">Handelsregionalismus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19019-5">
    <skos:prefLabel xml:lang="en">Crustacea</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19019-5">
    <skos:prefLabel xml:lang="de">Krebstier</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11857-5">
    <skos:prefLabel xml:lang="en">Region</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11857-5">
    <skos:prefLabel xml:lang="de">Region</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19758-2">
    <skos:broader rdf:resource="thsys/73358"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10291-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13331-5">
    <skos:altLabel xml:lang="de">Klärwerkstechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70013">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15605-5">
    <skos:prefLabel xml:lang="en">Data security</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15605-5">
    <skos:prefLabel xml:lang="de">Datensicherheit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18028-2">
    <skos:altLabel xml:lang="de">Special-Interest-Zeitschrift</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17991-6">
    <skos:altLabel xml:lang="de">NATO-Länder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12419-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70241">
    <skos:prefLabel xml:lang="en">N.05.02  Private law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70241">
    <skos:prefLabel xml:lang="de">N.05.02  Privatrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12974-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19745-4">
    <skos:altLabel xml:lang="en">Telephone interview</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19745-4">
    <skos:altLabel xml:lang="en">Online-research</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19745-4">
    <skos:altLabel xml:lang="en">Interview</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19745-4">
    <skos:altLabel xml:lang="en">Computer-assisted interviewing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19745-4">
    <skos:altLabel xml:lang="de">Telefonumfrage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19745-4">
    <skos:altLabel xml:lang="de">Telefoninterview</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19745-4">
    <skos:altLabel xml:lang="de">Telefon-Interview</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19745-4">
    <skos:altLabel xml:lang="de">Online-Umfrage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19745-4">
    <skos:altLabel xml:lang="de">Online-Befragung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19745-4">
    <skos:altLabel xml:lang="de">Interviewtechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19745-4">
    <skos:altLabel xml:lang="de">Internet-Umfrage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19745-4">
    <skos:altLabel xml:lang="de">Computergestütztes Interview</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19745-4">
    <skos:altLabel xml:lang="de">Computergestützte Befragung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14281-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17534-6">
    <skos:altLabel xml:lang="x-other">Rossijskaja Imperija</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17534-6">
    <skos:altLabel xml:lang="x-other">Rossijskaja Federacija</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17534-6">
    <skos:altLabel xml:lang="x-other">Rossija</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17534-6">
    <skos:altLabel xml:lang="de">Russisches Reich</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17534-6">
    <skos:altLabel xml:lang="de">Russische SFSR</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17534-6">
    <skos:altLabel xml:lang="de">Russische Föderation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17534-6">
    <skos:altLabel xml:lang="de">Rußland</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18593-6">
    <skos:broader rdf:resource="thsys/70936"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18593-6">
    <skos:broader rdf:resource="descriptor/13570-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15627-2">
    <zbwext:useInsteadNote rdf:nodeID="id17"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15627-2">
    <zbwext:useInsteadNote rdf:nodeID="id16"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13492-2">
    <skos:altLabel xml:lang="en">Mail order</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13492-2">
    <skos:altLabel xml:lang="en">Home shopping</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13492-2">
    <skos:altLabel xml:lang="en">Catalog sales</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13492-2">
    <skos:altLabel xml:lang="de">Versendungskauf</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13492-2">
    <skos:altLabel xml:lang="de">Versandhaus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13492-2">
    <skos:altLabel xml:lang="de">Kataloghandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13492-2">
    <skos:altLabel xml:lang="de">Großversand</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19478-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18976-4">
    <skos:narrower rdf:resource="descriptor/16485-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16623-5">
    <skos:related rdf:resource="descriptor/16544-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16623-5">
    <skos:related rdf:resource="descriptor/15718-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16623-5">
    <skos:related rdf:resource="descriptor/10093-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16623-5">
    <skos:related rdf:resource="descriptor/19772-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16623-5">
    <skos:related rdf:resource="descriptor/16429-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16623-5">
    <skos:related rdf:resource="descriptor/15713-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16623-5">
    <skos:related rdf:resource="descriptor/11400-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14643-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#changeNote">
    <rdfs:isDefinedBy rdf:resource="http://www.w3.org/2004/02/skos/core"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16834-6">
    <skos:broader rdf:resource="thsys/70361"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16834-6">
    <skos:broader rdf:resource="descriptor/18012-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10020-2">
    <skos:narrower rdf:resource="descriptor/10285-1"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id12">
    <rdfs:seeAlso rdf:resource="descriptor/10241-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12982-4">
    <skos:altLabel xml:lang="de">Eisenbergbau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13840-5">
    <skos:broader rdf:resource="thsys/70888"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13840-5">
    <skos:broader rdf:resource="descriptor/12026-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11039-4">
    <skos:altLabel xml:lang="de">Ökonomische Effizienz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11039-4">
    <skos:altLabel xml:lang="de">Mikroökonomische Effizienz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11039-4">
    <skos:altLabel xml:lang="de">Makroökonomische Effizienz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11039-4">
    <skos:altLabel xml:lang="de">Gesamtwirtschaftliche Effizienz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18307-5">
    <skos:broader rdf:resource="thsys/70333"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18307-5">
    <skos:broader rdf:resource="descriptor/14051-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15609-4">
    <skos:altLabel xml:lang="de">Theoretische Informatik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15609-4">
    <skos:altLabel xml:lang="de">Kerninformatik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15609-4">
    <skos:altLabel xml:lang="de">Computerwissenschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15609-4">
    <skos:altLabel xml:lang="de">Computerforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15609-4">
    <skos:altLabel xml:lang="de">Automatentheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15609-4">
    <skos:altLabel xml:lang="de">Angewandte Informatik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11441-1">
    <skos:narrower rdf:resource="descriptor/18670-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11441-1">
    <skos:narrower rdf:resource="descriptor/18668-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11441-1">
    <skos:narrower rdf:resource="descriptor/19276-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11441-1">
    <skos:narrower rdf:resource="descriptor/19275-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18395-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11430-6">
    <skos:broader rdf:resource="thsys/70055"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11430-6">
    <skos:broader rdf:resource="descriptor/19255-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11951-6">
    <skos:related rdf:resource="descriptor/13573-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11951-6">
    <skos:related rdf:resource="descriptor/11986-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11951-6">
    <skos:related rdf:resource="descriptor/11875-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11951-6">
    <skos:related rdf:resource="descriptor/11900-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11951-6">
    <skos:related rdf:resource="descriptor/11857-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11951-6">
    <skos:related rdf:resource="descriptor/11869-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11951-6">
    <skos:related rdf:resource="descriptor/15872-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16769-5">
    <skos:broader rdf:resource="thsys/70969"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16769-5">
    <skos:broader rdf:resource="thsys/70822"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16769-5">
    <skos:broader rdf:resource="descriptor/11578-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16769-5">
    <skos:broader rdf:resource="descriptor/13351-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71102">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.06.01.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16169-1">
    <skos:altLabel xml:lang="en">Freight conference</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16169-1">
    <skos:altLabel xml:lang="en">Conference line</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16169-1">
    <skos:altLabel xml:lang="de">Schifffahrtskonferenz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16169-1">
    <skos:altLabel xml:lang="de">Schiffahrtskonferenz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16169-1">
    <skos:altLabel xml:lang="de">Linienschifffahrtskonferenz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16169-1">
    <skos:altLabel xml:lang="de">Linienschiffahrtskonferenz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16169-1">
    <skos:altLabel xml:lang="de">Frachtkonferenz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19487-0">
    <skos:related rdf:resource="descriptor/11811-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19487-0">
    <skos:related rdf:resource="descriptor/18350-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19487-0">
    <skos:related rdf:resource="descriptor/18989-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12925-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16696-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11273-3">
    <skos:related rdf:resource="descriptor/11284-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11273-3">
    <skos:related rdf:resource="descriptor/11212-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11273-3">
    <skos:related rdf:resource="descriptor/18142-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11273-3">
    <skos:related rdf:resource="descriptor/18863-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11273-3">
    <skos:related rdf:resource="descriptor/19498-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11273-3">
    <skos:related rdf:resource="descriptor/19499-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11273-3">
    <skos:related rdf:resource="descriptor/10173-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11273-3">
    <skos:related rdf:resource="descriptor/11180-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11706-5">
    <skos:related rdf:resource="descriptor/19567-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11706-5">
    <skos:related rdf:resource="descriptor/21014-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11706-5">
    <skos:related rdf:resource="descriptor/11703-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11570-4">
    <skos:broader rdf:resource="thsys/71080"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11570-4">
    <skos:broader rdf:resource="thsys/70833"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11570-4">
    <skos:broader rdf:resource="thsys/70324"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11570-4">
    <skos:broader rdf:resource="descriptor/11568-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11570-4">
    <skos:broader rdf:resource="descriptor/11558-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11532-5">
    <skos:altLabel xml:lang="en">Fiscal court</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11532-5">
    <skos:altLabel xml:lang="de">Steuergericht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11532-5">
    <skos:altLabel xml:lang="de">Finanzgerichtsordnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11532-5">
    <skos:altLabel xml:lang="de">Finanzgerichtsbarkeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10012-1">
    <skos:prefLabel xml:lang="en">Market</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10012-1">
    <skos:prefLabel xml:lang="de">Markt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19699-6">
    <skos:broader rdf:resource="thsys/73376"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19699-6">
    <skos:broader rdf:resource="descriptor/25742-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14611-1">
    <skos:narrower rdf:resource="descriptor/18216-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12194-1">
    <skos:prefLabel xml:lang="en">Organizational form</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12194-1">
    <skos:prefLabel xml:lang="de">Rechtsform</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19098-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19256-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14190-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70245">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">B.04.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19066-3">
    <skos:prefLabel xml:lang="en">Vehicle leasing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19066-3">
    <skos:prefLabel xml:lang="de">Fahrzeug-Leasing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14487-4">
    <skos:broader rdf:resource="thsys/70254"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14487-4">
    <skos:broader rdf:resource="descriptor/18251-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12337-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14769-1">
    <skos:broader rdf:resource="thsys/70647"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16333-0">
    <skos:prefLabel xml:lang="en">Head of state</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16333-0">
    <skos:prefLabel xml:lang="de">Staatsoberhaupt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11531-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70843">
    <skos:prefLabel xml:lang="en">W.24.03  Vocational education</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70843">
    <skos:prefLabel xml:lang="de">W.24.03  Berufsbildung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18789-3">
    <skos:broader rdf:resource="thsys/70875"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16709-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10120-5">
    <skos:related rdf:resource="descriptor/10024-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10120-5">
    <skos:related rdf:resource="descriptor/10126-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10120-5">
    <skos:related rdf:resource="descriptor/10125-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10120-5">
    <skos:related rdf:resource="descriptor/10121-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10120-5">
    <skos:related rdf:resource="descriptor/10123-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10120-5">
    <skos:related rdf:resource="descriptor/10122-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10120-5">
    <skos:related rdf:resource="descriptor/12225-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10120-5">
    <skos:related rdf:resource="descriptor/12280-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10120-5">
    <skos:related rdf:resource="descriptor/11224-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15074-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13383-0">
    <skos:altLabel xml:lang="en">Public opinion polling</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13383-0">
    <skos:altLabel xml:lang="de">Umfrageforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13383-0">
    <skos:altLabel xml:lang="de">Meinungsumfrage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13383-0">
    <skos:altLabel xml:lang="de">Einstellungsforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13383-0">
    <skos:altLabel xml:lang="de">Demoskopie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13383-0">
    <skos:altLabel xml:lang="de">Bevölkerungsbefragung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13383-0">
    <skos:altLabel xml:lang="de">Attitüdenforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13383-0">
    <skos:altLabel xml:lang="de">Akzeptanzforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20104-4">
    <skos:related rdf:resource="descriptor/10398-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20104-4">
    <skos:related rdf:resource="descriptor/11506-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20536-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71057">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.13.06</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16760-2">
    <skos:prefLabel xml:lang="en">Information science</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16760-2">
    <skos:prefLabel xml:lang="de">Informationswissenschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18696-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11270-2">
    <skos:narrower rdf:resource="descriptor/10406-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11270-2">
    <skos:narrower rdf:resource="descriptor/11281-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11270-2">
    <skos:narrower rdf:resource="descriptor/10261-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11270-2">
    <skos:narrower rdf:resource="descriptor/18486-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11270-2">
    <skos:narrower rdf:resource="descriptor/11238-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18329-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17773-2">
    <skos:altLabel xml:lang="de">Britisch-Honduras</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12823-3">
    <skos:broader rdf:resource="thsys/70384"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12823-3">
    <skos:broader rdf:resource="descriptor/12829-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11446-5">
    <skos:altLabel xml:lang="en">High-powered money</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11446-5">
    <skos:altLabel xml:lang="de">Zentralbankgeld</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11446-5">
    <skos:altLabel xml:lang="de">Monetäre Basis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11446-5">
    <skos:altLabel xml:lang="de">Bereinigte Geldbasis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11446-5">
    <skos:altLabel xml:lang="de">Basisgeld</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70899">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15939-4">
    <skos:broader rdf:resource="thsys/70182"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15939-4">
    <skos:broader rdf:resource="descriptor/15911-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71099">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.07.01.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12935-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15076-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19102-4">
    <skos:broader rdf:resource="thsys/70004"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19102-4">
    <skos:broader rdf:resource="descriptor/19673-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70817">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.32.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10913-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70333">
    <skos:prefLabel xml:lang="de">P.21  Verpackungen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12138-4">
    <skos:related rdf:resource="descriptor/12137-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11558-1">
    <skos:prefLabel xml:lang="en">Public Social Expenditure</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11558-1">
    <skos:prefLabel xml:lang="de">Öffentliche Sozialausgaben</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13127-4">
    <skos:broader rdf:resource="thsys/70987"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13127-4">
    <skos:broader rdf:resource="descriptor/13145-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10815-5">
    <skos:related rdf:resource="descriptor/19360-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10815-5">
    <skos:related rdf:resource="descriptor/10838-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15892-4">
    <skos:broader rdf:resource="thsys/73353"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18336-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19552-1">
    <skos:prefLabel xml:lang="en">Demand system</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19552-1">
    <skos:prefLabel xml:lang="de">Nachfragesystem</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18750-2">
    <skos:altLabel xml:lang="de">Baufinanzierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18363-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18601-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10491-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10160-0">
    <skos:prefLabel xml:lang="en">Utility</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10160-0">
    <skos:prefLabel xml:lang="de">Nutzen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70904">
    <skos:prefLabel xml:lang="en">W.14.01  Central bank</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70904">
    <skos:prefLabel xml:lang="de">W.14.01  Zentralbank</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18994-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14007-2">
    <skos:broader rdf:resource="thsys/71053"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14007-2">
    <skos:broader rdf:resource="thsys/70834"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14007-2">
    <skos:broader rdf:resource="descriptor/14014-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13743-3">
    <skos:related rdf:resource="descriptor/13742-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13743-3">
    <skos:related rdf:resource="descriptor/18682-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13743-3">
    <skos:related rdf:resource="descriptor/12299-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13743-3">
    <skos:related rdf:resource="descriptor/10884-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13743-3">
    <skos:related rdf:resource="descriptor/19652-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13743-3">
    <skos:related rdf:resource="descriptor/10781-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13743-3">
    <skos:related rdf:resource="descriptor/13715-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10239-1">
    <skos:narrower rdf:resource="descriptor/11909-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10239-1">
    <skos:narrower rdf:resource="descriptor/11271-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19249-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18847-1">
    <skos:related rdf:resource="descriptor/10984-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18847-1">
    <skos:related rdf:resource="descriptor/11168-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18847-1">
    <skos:related rdf:resource="descriptor/19502-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18847-1">
    <skos:related rdf:resource="descriptor/13988-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18847-1">
    <skos:related rdf:resource="descriptor/19352-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18847-1">
    <skos:related rdf:resource="descriptor/19663-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18847-1">
    <skos:related rdf:resource="descriptor/19350-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18847-1">
    <skos:related rdf:resource="descriptor/11152-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14043-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18273-3">
    <skos:altLabel xml:lang="de">Pfandflaschensystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18273-3">
    <skos:altLabel xml:lang="de">Getränke-Mehrwegsystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10267-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11998-1">
    <skos:narrower rdf:resource="descriptor/19473-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11998-1">
    <skos:narrower rdf:resource="descriptor/11995-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18024-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18397-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14302-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70856">
    <rdfs:label xml:lang="de">Nonprofit-Sektor</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18163-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19567-2">
    <skos:broader rdf:resource="thsys/71075"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19567-2">
    <skos:broader rdf:resource="descriptor/11704-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13612-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19090-6">
    <skos:related rdf:resource="descriptor/18183-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19090-6">
    <skos:related rdf:resource="descriptor/19160-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19090-6">
    <skos:related rdf:resource="descriptor/25822-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11101-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15135-2">
    <skos:prefLabel xml:lang="en">Population census</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15135-2">
    <skos:prefLabel xml:lang="de">Volkszählung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19502-2">
    <skos:altLabel xml:lang="en">Unionisation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19502-2">
    <skos:altLabel xml:lang="en">Union density</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15717-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10392-0">
    <skos:broader rdf:resource="thsys/70278"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10392-0">
    <skos:broader rdf:resource="thsys/70020"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10392-0">
    <skos:broader rdf:resource="descriptor/10387-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15727-5">
    <skos:narrower rdf:resource="descriptor/13337-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15727-5">
    <skos:narrower rdf:resource="descriptor/13336-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15727-5">
    <skos:narrower rdf:resource="descriptor/15728-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14512-3">
    <skos:prefLabel xml:lang="en">Washing machine</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14512-3">
    <skos:prefLabel xml:lang="de">Waschmaschine</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19228-3">
    <skos:prefLabel xml:lang="en">Small country</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19228-3">
    <skos:prefLabel xml:lang="de">Kleinstaat</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19200-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18481-3">
    <skos:related rdf:resource="descriptor/10500-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10058-5">
    <skos:altLabel xml:lang="de">Volkswirtschaftliche Kosten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10058-5">
    <skos:altLabel xml:lang="de">Gesellschaftliche Kosten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16186-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19673-3">
    <skos:broader rdf:resource="thsys/70004"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19826-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12619-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11702-6">
    <skos:prefLabel xml:lang="en">Tax incentive</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11702-6">
    <skos:prefLabel xml:lang="de">Steuerbegünstigung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73361">
    <rdfs:label xml:lang="de">Staats- und Verfassungsrecht</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12356-1">
    <skos:related rdf:resource="descriptor/12314-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12356-1">
    <skos:related rdf:resource="descriptor/19080-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12356-1">
    <skos:related rdf:resource="descriptor/12406-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11152-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15598-4">
    <skos:narrower rdf:resource="descriptor/18825-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15598-4">
    <skos:narrower rdf:resource="descriptor/19633-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15598-4">
    <skos:narrower rdf:resource="descriptor/16774-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15598-4">
    <skos:narrower rdf:resource="descriptor/19654-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15598-4">
    <skos:narrower rdf:resource="descriptor/18568-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15598-4">
    <skos:narrower rdf:resource="descriptor/18731-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15598-4">
    <skos:narrower rdf:resource="descriptor/12165-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11066-1">
    <skos:prefLabel xml:lang="en">Theory of economic systems</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11066-1">
    <skos:prefLabel xml:lang="de">Ordnungstheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11210-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18750-2">
    <zbwext:useInsteadNote rdf:nodeID="id138"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10452-1">
    <skos:broader rdf:resource="thsys/73366"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10452-1">
    <skos:broader rdf:resource="thsys/71109"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10452-1">
    <skos:broader rdf:resource="thsys/71097"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10452-1">
    <skos:broader rdf:resource="descriptor/19708-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10452-1">
    <skos:broader rdf:resource="descriptor/10617-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14538-6">
    <skos:prefLabel xml:lang="en">Photographic camera</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14538-6">
    <skos:prefLabel xml:lang="de">Fotoapparat</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11273-3">
    <skos:altLabel xml:lang="en">Female labor</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11273-3">
    <skos:altLabel xml:lang="de">Phasenerwerbstätigkeit der Frau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11273-3">
    <skos:altLabel xml:lang="de">Frauenarbeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16952-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10815-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18637-5">
    <skos:altLabel xml:lang="de">Reisevertragsrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16440-6">
    <zbwext:useInsteadNote rdf:nodeID="id350"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id160">
    <rdf:value xml:lang="de">Für nicht-elektrische Drähte</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15666-6">
    <skos:prefLabel xml:lang="en">Education sciences</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15666-6">
    <skos:prefLabel xml:lang="de">Pädagogik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10622-2">
    <skos:related rdf:resource="descriptor/19553-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10622-2">
    <skos:related rdf:resource="descriptor/11328-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10622-2">
    <skos:related rdf:resource="descriptor/11327-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10622-2">
    <skos:related rdf:resource="descriptor/11325-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10622-2">
    <skos:related rdf:resource="descriptor/19308-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10622-2">
    <skos:related rdf:resource="descriptor/19506-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12394-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12978-2">
    <skos:scopeNote xml:lang="de">Einschließlich Transport, Verteilung und Lagerung von Erdgas.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16739-0">
    <skos:broader rdf:resource="thsys/71118"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16739-0">
    <skos:broader rdf:resource="descriptor/10161-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19540-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70355">
    <rdfs:label xml:lang="en">Inflation</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70355">
    <rdfs:label xml:lang="de">Inflation</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11734-0">
    <skos:broader rdf:resource="thsys/71082"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18948-2">
    <skos:altLabel xml:lang="de">Privates Einkommen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18948-2">
    <skos:altLabel xml:lang="de">Familieneinkommen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20468-1">
    <skos:scopeNote xml:lang="de">Konzeptionelle Idee, die es einem Unternehmen ermöglichen soll, mit Hilfe von effizient gestalteten Prozessen auf Erfolg versprechenden Märkten Erlöse zu erzielen</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14638-2">
    <skos:prefLabel xml:lang="en">Potash</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14638-2">
    <skos:prefLabel xml:lang="de">Kalisalz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71056">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15651-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17534-6">
    <skos:related rdf:resource="descriptor/17461-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11650-6">
    <skos:altLabel xml:lang="de">ESt (Einkommensteuer)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11650-6">
    <skos:altLabel xml:lang="de">Einkommensbesteuerung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70086">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15836-0">
    <skos:prefLabel xml:lang="en">Economic psychology</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15836-0">
    <skos:prefLabel xml:lang="de">Wirtschaftspsychologie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11859-1">
    <skos:altLabel xml:lang="de">Raumnutzung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11859-1">
    <skos:altLabel xml:lang="de">Grundstücksnutzung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11859-1">
    <skos:altLabel xml:lang="de">Flächennutzung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11859-1">
    <skos:altLabel xml:lang="de">Flächenbedarf</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19252-6">
    <skos:prefLabel xml:lang="en">Spatial innovation diffusion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19252-6">
    <skos:prefLabel xml:lang="de">Räumliche Innovationsdiffusion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16421-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13429-2">
    <skos:related rdf:resource="descriptor/11149-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13429-2">
    <skos:related rdf:resource="descriptor/19046-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13429-2">
    <skos:related rdf:resource="descriptor/13188-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13429-2">
    <skos:related rdf:resource="descriptor/13287-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13429-2">
    <skos:related rdf:resource="descriptor/18320-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13429-2">
    <skos:related rdf:resource="descriptor/13451-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18954-0">
    <skos:related rdf:resource="descriptor/18956-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18954-0">
    <skos:related rdf:resource="descriptor/18955-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18954-0">
    <skos:related rdf:resource="descriptor/13363-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18954-0">
    <skos:related rdf:resource="descriptor/18099-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18954-0">
    <skos:related rdf:resource="descriptor/18953-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10347-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11954-0">
    <skos:altLabel xml:lang="en">Export base model</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11954-0">
    <skos:altLabel xml:lang="de">Exportbasismodell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11954-0">
    <skos:altLabel xml:lang="de">Basic-Nonbasic-Konzept</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16389-1">
    <skos:broader rdf:resource="thsys/73378"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16389-1">
    <skos:broader rdf:resource="thsys/73366"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16389-1">
    <skos:broader rdf:resource="descriptor/15711-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18181-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13282-6">
    <skos:scopeNote xml:lang="de">Umfasst alle selbstständigen und angestellten Tätigkeiten der Versicherungsvermittlung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18290-3">
    <skos:broader rdf:resource="thsys/70254"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18290-3">
    <skos:broader rdf:resource="descriptor/18251-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11586-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16188-4">
    <skos:broader rdf:resource="thsys/73332"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16188-4">
    <skos:broader rdf:resource="descriptor/16186-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12197-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13341-2">
    <skos:altLabel xml:lang="de">Pressearchiv</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13341-2">
    <skos:altLabel xml:lang="de">Filmarchiv</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13341-2">
    <skos:altLabel xml:lang="de">Bildarchiv</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13341-2">
    <skos:altLabel xml:lang="de">Ausschnittarchiv</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13341-2">
    <skos:altLabel xml:lang="de">Archivwesen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13341-2">
    <skos:altLabel xml:lang="de">Archivkunde</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12185-2">
    <skos:broader rdf:resource="thsys/70087"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10858-1">
    <skos:altLabel xml:lang="de">Zahlungsrisiko im Außenhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10858-1">
    <skos:altLabel xml:lang="de">Importrisiko</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10858-1">
    <skos:altLabel xml:lang="de">Finanzierungsrisiko im Außenhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10858-1">
    <skos:altLabel xml:lang="de">Exportrisiko</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10858-1">
    <skos:altLabel xml:lang="de">Einfuhrrisiko</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10858-1">
    <skos:altLabel xml:lang="de">Ausfuhrrisiko</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19741-5">
    <skos:broader rdf:resource="thsys/73393"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19741-5">
    <skos:broader rdf:resource="thsys/70011"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19741-5">
    <skos:broader rdf:resource="descriptor/19070-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70952">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16212-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16517-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16789-6">
    <skos:prefLabel xml:lang="en">Subject indexing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16789-6">
    <skos:prefLabel xml:lang="de">Sacherschließung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15075-1">
    <skos:altLabel xml:lang="de">Volkswirtschaftliche Kennzahl</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21042-2">
    <skos:broader rdf:resource="thsys/70327"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21042-2">
    <skos:broader rdf:resource="descriptor/17589-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71068">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.11.02.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19714-1">
    <skos:altLabel xml:lang="de">Staatensukzession</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19714-1">
    <skos:altLabel xml:lang="de">Nachfolgestaat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18966-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id26">
    <rdf:value xml:lang="de">Für die betriebsinterne Arbeitsteilung</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14453-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18129-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15763-1">
    <skos:altLabel xml:lang="en">Pollution</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15763-1">
    <skos:altLabel xml:lang="en">Environmental pollution</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15763-1">
    <skos:altLabel xml:lang="de">Umweltverschmutzung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15763-1">
    <skos:altLabel xml:lang="de">Umweltschaden</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15763-1">
    <skos:altLabel xml:lang="de">Umweltkatastrophe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15763-1">
    <skos:altLabel xml:lang="de">Schadstoffemission</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15763-1">
    <skos:altLabel xml:lang="de">Emission von Schadstoffen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18208-0">
    <skos:broader rdf:resource="thsys/70265"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18208-0">
    <skos:broader rdf:resource="descriptor/18028-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19199-5">
    <skos:scopeNote xml:lang="de">Untersuchung permanenter Wirkungen temporärer Schocks; analog zur Physik, wo Hysteresis das Zurückbleiben einer Wirkung hinter dem jeweiligen Stand der sie bedingenden veränderlichen Kraft kennzeichnet</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17623-0">
    <skos:related rdf:resource="descriptor/18285-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19133-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12563-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16043-2">
    <skos:altLabel xml:lang="de">Aktiengesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16043-2">
    <skos:altLabel xml:lang="de">AktG</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19343-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10616-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11770-3">
    <skos:broader rdf:resource="thsys/71113"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11770-3">
    <skos:broader rdf:resource="descriptor/11779-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19664-4">
    <skos:altLabel xml:lang="en">Financial instability</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19664-4">
    <skos:altLabel xml:lang="de">Krise der Finanzmärkte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19664-4">
    <skos:altLabel xml:lang="de">Finanzkrise</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/w">
    <skos:narrower rdf:resource="thsys/70847"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/w">
    <skos:narrower rdf:resource="thsys/70818"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/w">
    <skos:narrower rdf:resource="thsys/70970"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/w">
    <skos:narrower rdf:resource="thsys/70829"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/w">
    <skos:narrower rdf:resource="thsys/70823"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/w">
    <skos:narrower rdf:resource="thsys/70815"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/w">
    <skos:narrower rdf:resource="thsys/70839"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/w">
    <skos:narrower rdf:resource="thsys/70883"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/w">
    <skos:narrower rdf:resource="thsys/70819"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/w">
    <skos:narrower rdf:resource="thsys/70965"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/w">
    <skos:narrower rdf:resource="thsys/70892"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/w">
    <skos:narrower rdf:resource="thsys/70893"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/w">
    <skos:narrower rdf:resource="thsys/70872"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/w">
    <skos:narrower rdf:resource="thsys/70958"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/w">
    <skos:narrower rdf:resource="thsys/71019"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/w">
    <skos:narrower rdf:resource="thsys/70889"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/w">
    <skos:narrower rdf:resource="thsys/71018"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/w">
    <skos:narrower rdf:resource="thsys/70856"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/w">
    <skos:narrower rdf:resource="thsys/71017"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/w">
    <skos:narrower rdf:resource="thsys/70905"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/w">
    <skos:narrower rdf:resource="thsys/70814"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/w">
    <skos:narrower rdf:resource="thsys/70828"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/w">
    <skos:narrower rdf:resource="thsys/70879"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/w">
    <skos:narrower rdf:resource="thsys/71010"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/w">
    <skos:narrower rdf:resource="thsys/70575"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/w">
    <skos:narrower rdf:resource="thsys/70825"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/w">
    <skos:narrower rdf:resource="thsys/70944"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/w">
    <skos:narrower rdf:resource="thsys/71005"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/w">
    <skos:narrower rdf:resource="thsys/70882"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/w">
    <skos:narrower rdf:resource="thsys/70943"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/w">
    <skos:narrower rdf:resource="thsys/71028"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/w">
    <skos:narrower rdf:resource="thsys/70963"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/w">
    <skos:narrower rdf:resource="thsys/70923"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/w">
    <skos:narrower rdf:resource="thsys/70876"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/w">
    <skos:narrower rdf:resource="thsys/70835"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11222-6">
    <skos:altLabel xml:lang="de">Humankapitalinvestition</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11222-6">
    <skos:altLabel xml:lang="de">Humankapitalbildung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11222-6">
    <skos:altLabel xml:lang="de">Ausbildungsinvestition</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15460-5">
    <skos:altLabel xml:lang="en">Super game</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15460-5">
    <skos:altLabel xml:lang="en">Repeated game</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15460-5">
    <skos:altLabel xml:lang="en">Infinite-horizon game</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15460-5">
    <skos:altLabel xml:lang="en">Continuum games</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15460-5">
    <skos:altLabel xml:lang="de">Wiederholtes Spiel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15460-5">
    <skos:altLabel xml:lang="de">Unendliches Spiel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15460-5">
    <skos:altLabel xml:lang="de">Superspiel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16524-0">
    <skos:broader rdf:resource="thsys/73364"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16524-0">
    <skos:broader rdf:resource="thsys/70862"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18399-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13529-5">
    <skos:related rdf:resource="descriptor/13179-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13529-5">
    <skos:related rdf:resource="descriptor/15892-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13529-5">
    <skos:related rdf:resource="descriptor/15868-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13529-5">
    <skos:related rdf:resource="descriptor/13242-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13529-5">
    <skos:related rdf:resource="descriptor/13528-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18821-5">
    <skos:broader rdf:resource="thsys/70958"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18821-5">
    <skos:broader rdf:resource="descriptor/12713-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10035-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14750-1">
    <skos:related rdf:resource="descriptor/18092-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14750-1">
    <skos:related rdf:resource="descriptor/13673-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14750-1">
    <skos:related rdf:resource="descriptor/14751-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14750-1">
    <skos:related rdf:resource="descriptor/13071-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14750-1">
    <skos:related rdf:resource="descriptor/13671-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19218-6">
    <skos:broader rdf:resource="thsys/70180"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19218-6">
    <skos:broader rdf:resource="descriptor/16727-0"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id314">
    <rdfs:seeAlso rdf:resource="descriptor/16030-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16170-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19170-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19529-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19080-2">
    <skos:scopeNote xml:lang="de">Gesamtheit der Bar- und Sachleistungen, mit denen sich Gesellschafter an einer Handelsgesellschaft beteiligen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10986-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11809-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15952-5">
    <skos:broader rdf:resource="thsys/70572"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15952-5">
    <skos:broader rdf:resource="descriptor/15954-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16526-3">
    <skos:broader rdf:resource="thsys/70832"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16526-3">
    <skos:broader rdf:resource="thsys/70458"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12275-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13445-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19455-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18196-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10555-5">
    <skos:related rdf:resource="descriptor/10587-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10555-5">
    <skos:related rdf:resource="descriptor/10583-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10555-5">
    <skos:related rdf:resource="descriptor/10572-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10555-5">
    <skos:related rdf:resource="descriptor/10603-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10555-5">
    <skos:related rdf:resource="descriptor/10568-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10555-5">
    <skos:related rdf:resource="descriptor/10525-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10555-5">
    <skos:related rdf:resource="descriptor/10561-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10555-5">
    <skos:related rdf:resource="descriptor/10560-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10555-5">
    <skos:related rdf:resource="descriptor/10558-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10555-5">
    <skos:related rdf:resource="descriptor/10492-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10555-5">
    <skos:related rdf:resource="descriptor/10556-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10555-5">
    <skos:related rdf:resource="descriptor/10552-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18342-3">
    <skos:broader rdf:resource="thsys/71018"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18342-3">
    <skos:broader rdf:resource="descriptor/13546-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19240-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70101">
    <rdfs:label xml:lang="de">Industrieökonomik und Wettbewerb</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19376-2">
    <skos:altLabel xml:lang="de">Intergenerativer Verteilungseffekt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16010-3">
    <skos:altLabel xml:lang="de">Pressegesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18459-3">
    <skos:broader rdf:resource="thsys/71065"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18459-3">
    <skos:broader rdf:resource="thsys/71005"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18459-3">
    <skos:broader rdf:resource="descriptor/10575-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18459-3">
    <skos:broader rdf:resource="descriptor/10500-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15023-6">
    <skos:related rdf:resource="descriptor/15125-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15023-6">
    <skos:related rdf:resource="descriptor/15061-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70269">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.09</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12010-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10917-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13871-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14150-4">
    <skos:narrower rdf:resource="descriptor/18201-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14150-4">
    <skos:narrower rdf:resource="descriptor/14157-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14150-4">
    <skos:narrower rdf:resource="descriptor/14154-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14150-4">
    <skos:narrower rdf:resource="descriptor/14153-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14150-4">
    <skos:narrower rdf:resource="descriptor/18204-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14150-4">
    <skos:narrower rdf:resource="descriptor/18202-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14150-4">
    <skos:narrower rdf:resource="descriptor/18203-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13000-0">
    <skos:broader rdf:resource="thsys/70994"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13000-0">
    <skos:broader rdf:resource="thsys/70989"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13000-0">
    <skos:broader rdf:resource="descriptor/12919-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13000-0">
    <skos:broader rdf:resource="descriptor/12996-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25822-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12053-5">
    <skos:broader rdf:resource="thsys/71042"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12053-5">
    <skos:broader rdf:resource="thsys/70037"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11004-2">
    <skos:altLabel xml:lang="en">National economic planning</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11004-2">
    <skos:altLabel xml:lang="en">Macroeconomic planning</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11004-2">
    <skos:altLabel xml:lang="en">Economic planning</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11004-2">
    <skos:altLabel xml:lang="en">Decentralized planning</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11004-2">
    <skos:altLabel xml:lang="de">Volkswirtschaftliche Planung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11004-2">
    <skos:altLabel xml:lang="de">Marktwirtschaftliche Planung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11004-2">
    <skos:altLabel xml:lang="de">Dezentrale Wirtschaftsplanung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12119-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15924-3">
    <skos:related rdf:resource="descriptor/19760-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15924-3">
    <skos:related rdf:resource="descriptor/15925-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15924-3">
    <skos:related rdf:resource="descriptor/16700-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15924-3">
    <skos:related rdf:resource="descriptor/16480-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15924-3">
    <skos:related rdf:resource="descriptor/16630-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15924-3">
    <skos:related rdf:resource="descriptor/16192-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15924-3">
    <skos:related rdf:resource="descriptor/16699-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15924-3">
    <skos:related rdf:resource="descriptor/18861-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24856-0">
    <skos:altLabel xml:lang="de">Performance-Messung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18821-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12171-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16264-0">
    <skos:altLabel xml:lang="en">Right of self-determination</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16264-0">
    <skos:altLabel xml:lang="en">National autonomy</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16264-0">
    <skos:altLabel xml:lang="de">Recht auf Selbstbestimmung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16264-0">
    <skos:altLabel xml:lang="de">Nationale Unabhängigkeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16264-0">
    <skos:altLabel xml:lang="de">Nationale Autonomie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16264-0">
    <skos:altLabel xml:lang="de">Autonome Region</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12423-5">
    <skos:broader rdf:resource="thsys/71043"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12423-5">
    <skos:broader rdf:resource="thsys/70004"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12423-5">
    <skos:broader rdf:resource="descriptor/12376-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16226-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11070-3">
    <skos:broader rdf:resource="thsys/73395"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11070-3">
    <skos:broader rdf:resource="descriptor/15717-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13281-1">
    <skos:related rdf:resource="descriptor/12252-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13281-1">
    <skos:related rdf:resource="descriptor/13776-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13320-3">
    <skos:broader rdf:resource="thsys/70829"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13320-3">
    <skos:broader rdf:resource="descriptor/12907-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19481-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18069-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14420-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11579-0">
    <skos:related rdf:resource="descriptor/11580-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11579-0">
    <skos:related rdf:resource="descriptor/10930-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11579-0">
    <skos:related rdf:resource="descriptor/11578-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11579-0">
    <skos:related rdf:resource="descriptor/11554-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11579-0">
    <skos:related rdf:resource="descriptor/11576-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19291-3">
    <skos:prefLabel xml:lang="en">Locational policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19291-3">
    <skos:prefLabel xml:lang="de">Standortpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13236-6">
    <skos:prefLabel xml:lang="en">Steel trade</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13236-6">
    <skos:prefLabel xml:lang="de">Stahlhandel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13714-3">
    <skos:altLabel xml:lang="en">Stockholder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13714-3">
    <skos:altLabel xml:lang="en">Shareholder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13714-3">
    <skos:altLabel xml:lang="de">Aktionärinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13714-3">
    <skos:altLabel xml:lang="de">Aktionär</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13819-3">
    <skos:prefLabel xml:lang="en">Theory of insurance management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13819-3">
    <skos:prefLabel xml:lang="de">Versicherungsbetriebslehre</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15469-1">
    <skos:prefLabel xml:lang="en">Decision tree</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15469-1">
    <skos:prefLabel xml:lang="de">Entscheidungsbaum</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10094-1">
    <skos:altLabel xml:lang="de">Pigou-Steuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10094-1">
    <skos:altLabel xml:lang="de">Pigou Steuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15887-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18503-5">
    <skos:broader rdf:resource="thsys/70967"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18503-5">
    <skos:broader rdf:resource="thsys/70888"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18503-5">
    <skos:broader rdf:resource="thsys/70854"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18503-5">
    <skos:broader rdf:resource="descriptor/12180-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18503-5">
    <skos:broader rdf:resource="descriptor/13302-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73372">
    <skos:prefLabel xml:lang="de">N.04.03.02  Wirtschafts- und Finanzpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18292-6">
    <skos:prefLabel xml:lang="en">Teaching aid</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18292-6">
    <skos:prefLabel xml:lang="de">Lehrmaterial</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17596-5">
    <skos:broader rdf:resource="thsys/70253"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17596-5">
    <skos:broader rdf:resource="descriptor/17608-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10575-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15124-0">
    <skos:related rdf:resource="descriptor/15135-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73370">
    <skos:narrower rdf:resource="descriptor/15725-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73370">
    <skos:narrower rdf:resource="descriptor/16545-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73370">
    <skos:narrower rdf:resource="descriptor/18025-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73370">
    <skos:narrower rdf:resource="descriptor/16540-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73370">
    <skos:narrower rdf:resource="descriptor/16481-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73370">
    <skos:narrower rdf:resource="descriptor/16480-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73370">
    <skos:narrower rdf:resource="descriptor/18063-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73370">
    <skos:narrower rdf:resource="descriptor/16479-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73370">
    <skos:narrower rdf:resource="descriptor/24663-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73370">
    <skos:narrower rdf:resource="descriptor/18142-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73370">
    <skos:narrower rdf:resource="descriptor/16477-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73370">
    <skos:narrower rdf:resource="descriptor/18334-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73370">
    <skos:narrower rdf:resource="descriptor/16476-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73370">
    <skos:narrower rdf:resource="descriptor/15911-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73370">
    <skos:narrower rdf:resource="descriptor/16529-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73370">
    <skos:narrower rdf:resource="descriptor/16533-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73370">
    <skos:narrower rdf:resource="descriptor/16475-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71055">
    <rdfs:label xml:lang="en">Service workers</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71055">
    <rdfs:label xml:lang="de">Dienstleistungsberufe</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12914-0">
    <skos:related rdf:resource="descriptor/12705-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12914-0">
    <skos:related rdf:resource="descriptor/13058-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12914-0">
    <skos:related rdf:resource="descriptor/12916-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12914-0">
    <skos:related rdf:resource="descriptor/12915-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12914-0">
    <skos:related rdf:resource="descriptor/12750-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12914-0">
    <skos:related rdf:resource="descriptor/12717-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16739-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19219-4">
    <skos:broader rdf:resource="thsys/73393"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19219-4">
    <skos:broader rdf:resource="thsys/70180"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19219-4">
    <skos:broader rdf:resource="thsys/70011"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19219-4">
    <skos:broader rdf:resource="descriptor/16727-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15951-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16003-0">
    <skos:related rdf:resource="descriptor/16024-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16003-0">
    <skos:related rdf:resource="descriptor/16188-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16003-0">
    <skos:related rdf:resource="descriptor/16295-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16003-0">
    <skos:related rdf:resource="descriptor/16417-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16003-0">
    <skos:related rdf:resource="descriptor/11051-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16003-0">
    <skos:related rdf:resource="descriptor/19721-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16003-0">
    <skos:related rdf:resource="descriptor/16470-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13424-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18372-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11519-4">
    <skos:related rdf:resource="descriptor/10397-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11519-4">
    <skos:related rdf:resource="descriptor/11518-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11519-4">
    <skos:related rdf:resource="descriptor/15093-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11519-4">
    <skos:related rdf:resource="descriptor/10213-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11519-4">
    <skos:related rdf:resource="descriptor/10659-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11519-4">
    <skos:related rdf:resource="descriptor/11432-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13410-2">
    <skos:narrower rdf:resource="descriptor/13423-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13410-2">
    <skos:narrower rdf:resource="descriptor/13422-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13410-2">
    <skos:narrower rdf:resource="descriptor/15977-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13410-2">
    <skos:narrower rdf:resource="descriptor/19851-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13410-2">
    <skos:narrower rdf:resource="descriptor/11532-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13410-2">
    <skos:narrower rdf:resource="descriptor/11307-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72186">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.16.05</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18210-6">
    <skos:broader rdf:resource="thsys/70265"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18210-6">
    <skos:broader rdf:resource="descriptor/18028-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73394">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.01.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14423-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18085-4">
    <skos:altLabel xml:lang="de">Finanz-Service</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18085-4">
    <skos:altLabel xml:lang="de">Allfinanz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16630-1">
    <skos:prefLabel xml:lang="en">Youth organization</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16630-1">
    <skos:prefLabel xml:lang="de">Jugendorganisation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10543-5">
    <skos:prefLabel xml:lang="en">Bilateral aid</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10543-5">
    <skos:prefLabel xml:lang="de">Bilaterale Entwicklungshilfe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10272-3">
    <skos:related rdf:resource="descriptor/10791-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10272-3">
    <skos:related rdf:resource="descriptor/15076-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10272-3">
    <skos:related rdf:resource="descriptor/10249-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10272-3">
    <skos:related rdf:resource="descriptor/10248-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10272-3">
    <skos:related rdf:resource="descriptor/10243-3"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id205">
    <rdfs:seeAlso rdf:resource="descriptor/10462-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14236-1">
    <skos:prefLabel xml:lang="en">Mineral</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14236-1">
    <skos:prefLabel xml:lang="de">Mineral</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13310-6">
    <skos:prefLabel xml:lang="en">Home for the elderly</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13310-6">
    <skos:prefLabel xml:lang="de">Altenheim</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70852">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16078-4">
    <skos:altLabel xml:lang="en">Trademark right</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16078-4">
    <skos:altLabel xml:lang="de">Warenzeichengesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16078-4">
    <skos:altLabel xml:lang="de">Recht der Marken</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16078-4">
    <skos:altLabel xml:lang="de">Musterrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16078-4">
    <skos:altLabel xml:lang="de">Markenrechtsreformgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16078-4">
    <skos:altLabel xml:lang="de">Geschmacksmustergesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16078-4">
    <skos:altLabel xml:lang="de">Gebrauchsmustergesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12564-1">
    <skos:broader rdf:resource="thsys/71036"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12564-1">
    <skos:broader rdf:resource="thsys/70113"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12564-1">
    <skos:broader rdf:resource="descriptor/12567-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12564-1">
    <skos:broader rdf:resource="descriptor/12165-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18273-3">
    <skos:broader rdf:resource="thsys/70333"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18273-3">
    <skos:broader rdf:resource="descriptor/14051-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16171-0">
    <skos:related rdf:resource="descriptor/18579-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16171-0">
    <skos:related rdf:resource="descriptor/13563-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19830-6">
    <skos:altLabel xml:lang="de">Soziale Unterstützung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19830-6">
    <skos:altLabel xml:lang="de">Private Armenfürsorge</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19830-6">
    <skos:altLabel xml:lang="de">Philanthropie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19830-6">
    <skos:altLabel xml:lang="de">Armenfürsorge durch Private</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71020">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.01.05</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11239-3">
    <skos:altLabel xml:lang="en">Job creation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11239-3">
    <skos:altLabel xml:lang="en">Employment program</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11239-3">
    <skos:altLabel xml:lang="de">Beschäftigungsprogramm</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11239-3">
    <skos:altLabel xml:lang="de">Arbeitsplatzschaffung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12916-3">
    <skos:altLabel xml:lang="de">Kapitalgüterindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71060">
    <rdfs:label xml:lang="en">Unemployment</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71060">
    <rdfs:label xml:lang="de">Arbeitslosigkeit</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71112">
    <skos:broader rdf:resource="thsys/70234"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19424-3">
    <skos:broader rdf:resource="thsys/71074"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19424-3">
    <skos:broader rdf:resource="thsys/70892"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19424-3">
    <skos:broader rdf:resource="descriptor/11681-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11685-1">
    <skos:altLabel xml:lang="de">Mehrbelastung durch Steuern</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10349-1">
    <skos:related rdf:resource="descriptor/10297-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10349-1">
    <skos:related rdf:resource="descriptor/12675-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10349-1">
    <skos:related rdf:resource="descriptor/12677-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70884">
    <rdfs:label xml:lang="de">Vermietung und Verpachtung von Immobilien</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#exactMatch">
    <skos:definition xml:lang="en">skos:exactMatch is used to link two concepts, indicating a high
      degree of confidence that the concepts can be used interchangeably across a wide range of
      information retrieval applications. skos:exactMatch is a transitive property, and is a
      sub-property of skos:closeMatch.</skos:definition>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71079">
    <skos:narrower rdf:resource="descriptor/18905-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71079">
    <skos:narrower rdf:resource="descriptor/11668-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71079">
    <skos:narrower rdf:resource="descriptor/11603-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71079">
    <skos:narrower rdf:resource="descriptor/11602-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71079">
    <skos:narrower rdf:resource="descriptor/11600-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71079">
    <skos:narrower rdf:resource="descriptor/11599-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15909-6">
    <skos:narrower rdf:resource="descriptor/10298-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15909-6">
    <skos:narrower rdf:resource="descriptor/13992-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15909-6">
    <skos:narrower rdf:resource="descriptor/11378-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15909-6">
    <skos:narrower rdf:resource="descriptor/19763-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15909-6">
    <skos:narrower rdf:resource="descriptor/11971-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15909-6">
    <skos:narrower rdf:resource="descriptor/19507-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15909-6">
    <skos:narrower rdf:resource="descriptor/11375-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15909-6">
    <skos:narrower rdf:resource="descriptor/19637-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15909-6">
    <skos:narrower rdf:resource="descriptor/15930-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15909-6">
    <skos:narrower rdf:resource="descriptor/19639-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15909-6">
    <skos:narrower rdf:resource="descriptor/13454-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15909-6">
    <skos:narrower rdf:resource="descriptor/19141-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15909-6">
    <skos:narrower rdf:resource="descriptor/19721-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15909-6">
    <skos:narrower rdf:resource="descriptor/15925-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15909-6">
    <skos:narrower rdf:resource="descriptor/15924-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15909-6">
    <skos:narrower rdf:resource="descriptor/19088-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15909-6">
    <skos:narrower rdf:resource="descriptor/15919-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15909-6">
    <skos:narrower rdf:resource="descriptor/15918-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15909-6">
    <skos:narrower rdf:resource="descriptor/18861-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15909-6">
    <skos:narrower rdf:resource="descriptor/15908-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15909-6">
    <skos:narrower rdf:resource="descriptor/15907-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15909-6">
    <skos:narrower rdf:resource="descriptor/18971-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15909-6">
    <skos:narrower rdf:resource="descriptor/11355-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15909-6">
    <skos:narrower rdf:resource="descriptor/18856-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10617-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12748-5">
    <skos:related rdf:resource="descriptor/12655-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12748-5">
    <skos:related rdf:resource="descriptor/12563-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12748-5">
    <skos:related rdf:resource="descriptor/12754-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12748-5">
    <skos:related rdf:resource="descriptor/12651-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12748-5">
    <skos:related rdf:resource="descriptor/12621-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19334-4">
    <skos:broader rdf:resource="thsys/71094"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19334-4">
    <skos:broader rdf:resource="descriptor/10722-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13470-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19790-6">
    <skos:prefLabel xml:lang="en">Epidemic</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19790-6">
    <skos:prefLabel xml:lang="de">Epidemie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19099-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11474-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16379-4">
    <skos:prefLabel xml:lang="en">Pluralism</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16379-4">
    <skos:prefLabel xml:lang="de">Pluralismus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14802-1">
    <skos:altLabel xml:lang="de">Packpapier</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10225-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14603-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id202">
    <rdfs:seeAlso rdf:resource="descriptor/10300-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13452-0">
    <skos:related rdf:resource="descriptor/12949-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13452-0">
    <skos:related rdf:resource="descriptor/12032-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13452-0">
    <skos:related rdf:resource="descriptor/12725-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15711-6">
    <skos:prefLabel xml:lang="en">Social movement</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15711-6">
    <skos:prefLabel xml:lang="de">Soziale Bewegung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18916-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14947-3">
    <skos:narrower rdf:resource="descriptor/14946-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10224-0">
    <skos:prefLabel xml:lang="en">Imperfect competition</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10224-0">
    <skos:prefLabel xml:lang="de">Unvollkommener Wettbewerb</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10612-5">
    <skos:prefLabel xml:lang="en">Free trade area</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10612-5">
    <skos:prefLabel xml:lang="de">Freihandelszone</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18234-6">
    <skos:prefLabel xml:lang="en">Thermal energy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18234-6">
    <skos:prefLabel xml:lang="de">Wärmeenergie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11088-5">
    <skos:related rdf:resource="descriptor/11101-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11088-5">
    <skos:related rdf:resource="descriptor/11090-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11088-5">
    <skos:related rdf:resource="descriptor/16210-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11088-5">
    <skos:related rdf:resource="descriptor/11060-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11088-5">
    <skos:related rdf:resource="descriptor/11174-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11088-5">
    <skos:related rdf:resource="descriptor/16717-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11088-5">
    <skos:related rdf:resource="descriptor/11047-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13890-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18668-1">
    <skos:narrower rdf:resource="descriptor/13766-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18668-1">
    <skos:narrower rdf:resource="descriptor/13735-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18668-1">
    <skos:narrower rdf:resource="descriptor/13722-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15989-3">
    <skos:altLabel xml:lang="de">Fremdenrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11833-5">
    <skos:editorialNote xml:lang="de">Möglichst spezifischer indexieren.</skos:editorialNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19456-4">
    <skos:prefLabel xml:lang="en">Urban land market</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19456-4">
    <skos:prefLabel xml:lang="de">Städtischer Immobilienmarkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19209-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17179-0">
    <skos:narrower rdf:resource="descriptor/18226-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17179-0">
    <skos:narrower rdf:resource="descriptor/19087-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17179-0">
    <skos:narrower rdf:resource="descriptor/17177-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17179-0">
    <skos:narrower rdf:resource="descriptor/17480-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17179-0">
    <skos:narrower rdf:resource="descriptor/18228-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17179-0">
    <skos:narrower rdf:resource="descriptor/18227-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17179-0">
    <skos:narrower rdf:resource="descriptor/17176-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17179-0">
    <skos:narrower rdf:resource="descriptor/17184-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17179-0">
    <skos:narrower rdf:resource="descriptor/17175-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17179-0">
    <skos:narrower rdf:resource="descriptor/19086-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17179-0">
    <skos:narrower rdf:resource="descriptor/17171-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12271-2">
    <skos:altLabel xml:lang="de">Dividendenpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16759-1">
    <skos:related rdf:resource="descriptor/19766-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16759-1">
    <skos:related rdf:resource="descriptor/10060-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16759-1">
    <skos:related rdf:resource="descriptor/18932-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16759-1">
    <skos:related rdf:resource="descriptor/12170-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16759-1">
    <skos:related rdf:resource="descriptor/10141-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16759-1">
    <skos:related rdf:resource="descriptor/10140-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16759-1">
    <skos:related rdf:resource="descriptor/12168-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16759-1">
    <skos:related rdf:resource="descriptor/15473-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16759-1">
    <skos:related rdf:resource="descriptor/18860-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16759-1">
    <skos:related rdf:resource="descriptor/16783-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13687-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11579-0">
    <skos:narrower rdf:resource="descriptor/11582-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19136-1">
    <skos:broader rdf:resource="thsys/70459"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19136-1">
    <skos:broader rdf:resource="descriptor/12818-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71103">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.05.06.04</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18125-4">
    <skos:altLabel xml:lang="en">Theory of union size</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18125-4">
    <skos:altLabel xml:lang="en">Microeconomics of unions</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18125-4">
    <skos:altLabel xml:lang="en">Economic theory of trade unions</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18125-4">
    <skos:altLabel xml:lang="de">Theorie der Gewerkschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18125-4">
    <skos:altLabel xml:lang="de">Politische Ökonomie der Gewerkschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18125-4">
    <skos:altLabel xml:lang="de">Ökonomische Gewerkschaftstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17806-6">
    <skos:related rdf:resource="descriptor/19130-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11533-3">
    <zbwext:useInsteadNote rdf:nodeID="id100"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11533-3">
    <zbwext:useInsteadNote rdf:nodeID="id99"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16439-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10139-5">
    <skos:scopeNote xml:lang="de">Ökonometrische Analyse von Individualdaten.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19057-4">
    <skos:related rdf:resource="descriptor/19062-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19057-4">
    <skos:related rdf:resource="descriptor/14381-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11222-6">
    <skos:prefLabel xml:lang="en">Human capital investment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11222-6">
    <skos:prefLabel xml:lang="de">Bildungsinvestition</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19401-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14740-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13072-3">
    <skos:scopeNote xml:lang="de">Auch als Oberbegriff für Büromaschinen- und Computerindustrie</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10847-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19598-5">
    <skos:broader rdf:resource="thsys/73103"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19598-5">
    <skos:broader rdf:resource="descriptor/15476-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70936">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.12.01.03</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13645-3">
    <skos:related rdf:resource="descriptor/11215-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13645-3">
    <skos:related rdf:resource="descriptor/18639-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13645-3">
    <skos:related rdf:resource="descriptor/19466-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18739-4">
    <skos:related rdf:resource="descriptor/13416-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16526-3">
    <skos:prefLabel xml:lang="en">Retirement</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16526-3">
    <skos:prefLabel xml:lang="de">Altersgrenze</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12072-1">
    <skos:altLabel xml:lang="en">Company</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12072-1">
    <skos:altLabel xml:lang="de">Wirtschaftsunternehmen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12072-1">
    <skos:altLabel xml:lang="de">Wirtschaft (Gewerbliche Wirtschaft)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12072-1">
    <skos:altLabel xml:lang="de">Unternehmung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12072-1">
    <skos:altLabel xml:lang="de">Unternehmensbereich</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12072-1">
    <skos:altLabel xml:lang="de">Gewerbliche Wirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12072-1">
    <skos:altLabel xml:lang="de">Gewerbebetrieb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12072-1">
    <skos:altLabel xml:lang="de">Firma</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12072-1">
    <skos:altLabel xml:lang="de">Betrieb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71000">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18771-1">
    <skos:altLabel xml:lang="de">Onlinedienst</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18771-1">
    <skos:altLabel xml:lang="de">Internationaler Online-Informationsdienst</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18771-1">
    <skos:altLabel xml:lang="de">Internationaler Online-Dienst</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18771-1">
    <skos:altLabel xml:lang="de">Datex-J</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18771-1">
    <skos:altLabel xml:lang="de">Bildschirmtext</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18165-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14744-3">
    <skos:altLabel xml:lang="de">Titandioxid</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14744-3">
    <skos:altLabel xml:lang="de">Textilfarbstoff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14744-3">
    <skos:altLabel xml:lang="de">Schwermetallpigment</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14744-3">
    <skos:altLabel xml:lang="de">Pigment</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14744-3">
    <skos:altLabel xml:lang="de">Mineralpigment</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14744-3">
    <skos:altLabel xml:lang="de">Mineralfarbe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14744-3">
    <skos:altLabel xml:lang="de">Farbstoff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14744-3">
    <skos:altLabel xml:lang="de">Farbpigment</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14744-3">
    <skos:altLabel xml:lang="de">Farboxid</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14744-3">
    <skos:altLabel xml:lang="de">Druckfarbe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13726-3">
    <skos:broader rdf:resource="thsys/71108"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13726-3">
    <skos:broader rdf:resource="descriptor/12299-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13051-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14626-2">
    <skos:prefLabel xml:lang="en">Pesticide</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14626-2">
    <skos:prefLabel xml:lang="de">Pestizid</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18024-3">
    <skos:related rdf:resource="descriptor/20888-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18024-3">
    <skos:related rdf:resource="descriptor/18235-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18024-3">
    <skos:related rdf:resource="descriptor/18362-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16780-3">
    <skos:broader rdf:resource="thsys/70822"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16780-3">
    <skos:broader rdf:resource="descriptor/16782-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10904-6">
    <skos:prefLabel xml:lang="en">Market power</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10904-6">
    <skos:prefLabel xml:lang="de">Marktmacht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14930-6">
    <skos:related rdf:resource="descriptor/18445-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14930-6">
    <skos:related rdf:resource="descriptor/14927-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19523-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19818-3">
    <skos:narrower rdf:resource="descriptor/16120-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18661-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16050-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13390-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11988-4">
    <skos:prefLabel xml:lang="en">City size</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11988-4">
    <skos:prefLabel xml:lang="de">Stadtgröße</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19048-5">
    <skos:prefLabel xml:lang="en">Slate</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19048-5">
    <skos:prefLabel xml:lang="de">Schiefer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10000-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11163-3">
    <skos:broader rdf:resource="thsys/73359"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11163-3">
    <skos:broader rdf:resource="thsys/73104"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11163-3">
    <skos:broader rdf:resource="thsys/70274"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17179-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10713-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14334-1">
    <skos:prefLabel xml:lang="en">Gears</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14334-1">
    <skos:prefLabel xml:lang="de">Getriebe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19521-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12708-3">
    <skos:altLabel xml:lang="de">Kapazität der Produktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12708-3">
    <skos:altLabel xml:lang="de">Fertigungskapazität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18770-3">
    <skos:broader rdf:resource="thsys/70880"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18770-3">
    <skos:broader rdf:resource="descriptor/18059-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18770-3">
    <skos:broader rdf:resource="descriptor/16774-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18363-2">
    <skos:broader rdf:resource="thsys/71014"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18363-2">
    <skos:broader rdf:resource="thsys/71012"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18363-2">
    <skos:broader rdf:resource="thsys/70539"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18363-2">
    <skos:broader rdf:resource="descriptor/18362-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12589-6">
    <skos:scopeNote xml:lang="de">Maßnahmen zur Förderung der Leistungsfähigkeit und -bereitschaft von Mitarbeitern im Betrieb.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11038-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19323-2">
    <skos:broader rdf:resource="thsys/73331"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19323-2">
    <skos:broader rdf:resource="thsys/71098"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19323-2">
    <skos:broader rdf:resource="thsys/71093"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19323-2">
    <skos:broader rdf:resource="thsys/70160"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19323-2">
    <skos:broader rdf:resource="descriptor/16007-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10338-6">
    <skos:related rdf:resource="descriptor/10343-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10338-6">
    <skos:related rdf:resource="descriptor/19241-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10338-6">
    <skos:related rdf:resource="descriptor/10347-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10338-6">
    <skos:related rdf:resource="descriptor/19587-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10338-6">
    <skos:related rdf:resource="descriptor/10387-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10338-6">
    <skos:related rdf:resource="descriptor/10361-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10338-6">
    <skos:related rdf:resource="descriptor/10339-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10338-6">
    <skos:related rdf:resource="descriptor/10237-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10889-4">
    <skos:altLabel xml:lang="de">Kostenpreis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19612-2">
    <skos:related rdf:resource="descriptor/11988-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19612-2">
    <skos:related rdf:resource="descriptor/19684-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19612-2">
    <skos:related rdf:resource="descriptor/19228-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19612-2">
    <skos:related rdf:resource="descriptor/10042-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19811-3">
    <skos:broader rdf:resource="thsys/73339"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19811-3">
    <skos:broader rdf:resource="descriptor/15410-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18259-4">
    <skos:prefLabel xml:lang="en">Pharmaceutical law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18259-4">
    <skos:prefLabel xml:lang="de">Arzneimittelrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70235">
    <skos:prefLabel xml:lang="en">V.13.03  Labour relations</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70235">
    <skos:prefLabel xml:lang="de">V.13.03  Arbeitsbeziehungen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70922">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.13.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15881-2">
    <skos:prefLabel xml:lang="en">Climatology</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15881-2">
    <skos:prefLabel xml:lang="de">Klimaforschung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19541-6">
    <skos:scopeNote xml:lang="de">Zentrale für Kundenkommunikation im Pre- und After-Sales-Bereich unter Einsatz der Telekommunikation.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18823-1">
    <skos:altLabel xml:lang="de">Verwaltungsbetriebslehre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18823-1">
    <skos:altLabel xml:lang="de">Betriebswirtschaftslehre für die Verwaltung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18823-1">
    <skos:altLabel xml:lang="de">Betriebswirtschaftliche Theorie öffentlicher Unternehmen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18732-4">
    <skos:altLabel xml:lang="de">Emissionspreis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18732-4">
    <skos:altLabel xml:lang="de">Ausgabekurs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11812-6">
    <skos:broader rdf:resource="thsys/71012"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11812-6">
    <skos:broader rdf:resource="thsys/70539"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11812-6">
    <skos:broader rdf:resource="descriptor/15753-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12780-2">
    <skos:narrower rdf:resource="descriptor/18654-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12780-2">
    <skos:narrower rdf:resource="descriptor/18549-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12780-2">
    <skos:narrower rdf:resource="descriptor/12824-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10534-6">
    <skos:related rdf:resource="descriptor/10595-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10534-6">
    <skos:related rdf:resource="descriptor/10515-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12489-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12365-0">
    <skos:broader rdf:resource="thsys/71043"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12365-0">
    <skos:broader rdf:resource="descriptor/12357-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11801-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15336-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12047-0">
    <skos:altLabel xml:lang="en">Nonprofit sector</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12047-0">
    <skos:altLabel xml:lang="en">Non-profit organization</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12047-0">
    <skos:altLabel xml:lang="en">Nonprofit organisation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12047-0">
    <skos:altLabel xml:lang="de">NPO (Nonprofit-Organisation)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12047-0">
    <skos:altLabel xml:lang="de">Gemeinnützige Organisation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12047-0">
    <skos:altLabel xml:lang="de">Gemeinnützige Einrichtung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12047-0">
    <skos:altLabel xml:lang="de">Dritter Sektor</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15939-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18906-4">
    <skos:prefLabel xml:lang="en">Social facilities</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18906-4">
    <skos:prefLabel xml:lang="de">Soziale Einrichtung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70901">
    <skos:prefLabel xml:lang="en">W.14.02.02  Investment fund</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70901">
    <skos:prefLabel xml:lang="de">W.14.02.02  Investmentfonds</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19828-0">
    <skos:broader rdf:resource="thsys/70485"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19828-0">
    <skos:broader rdf:resource="descriptor/18118-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12281-6">
    <skos:narrower rdf:resource="descriptor/13709-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12281-6">
    <skos:narrower rdf:resource="descriptor/13704-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12281-6">
    <skos:narrower rdf:resource="descriptor/24740-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12281-6">
    <skos:narrower rdf:resource="descriptor/18088-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12281-6">
    <skos:narrower rdf:resource="descriptor/13699-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18795-1">
    <skos:altLabel xml:lang="de">Messeteilnahme</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18795-1">
    <skos:altLabel xml:lang="de">Messeselektion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18795-1">
    <skos:altLabel xml:lang="de">Messeauswahl</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13082-0">
    <skos:related rdf:resource="descriptor/14539-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13082-0">
    <skos:related rdf:resource="descriptor/13229-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11366-3">
    <skos:altLabel xml:lang="de">Mitarbeiterschulung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11366-3">
    <skos:altLabel xml:lang="de">Mitarbeiterqualifizierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11366-3">
    <skos:altLabel xml:lang="de">Lernstatt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11366-3">
    <skos:altLabel xml:lang="de">Bildungsbetriebslehre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11366-3">
    <skos:altLabel xml:lang="de">Betriebliche Qualifizierungsmaßnahme</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11366-3">
    <skos:altLabel xml:lang="de">Betriebliche Bildungsmaßnahme</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12198-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19429-0">
    <skos:related rdf:resource="descriptor/11928-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19429-0">
    <skos:related rdf:resource="descriptor/11919-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19026-1">
    <skos:prefLabel xml:lang="en">Implementation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19026-1">
    <skos:prefLabel xml:lang="de">Einführung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13747-2">
    <skos:broader rdf:resource="thsys/70900"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13747-2">
    <skos:broader rdf:resource="descriptor/10213-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11326-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10546-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16134-6">
    <skos:related rdf:resource="descriptor/16140-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16134-6">
    <skos:related rdf:resource="descriptor/11392-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16134-6">
    <skos:related rdf:resource="descriptor/11360-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16134-6">
    <skos:related rdf:resource="descriptor/11406-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16134-6">
    <skos:related rdf:resource="descriptor/11185-0"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id106">
    <rdfs:seeAlso rdf:resource="descriptor/10143-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18644-1">
    <skos:narrower rdf:resource="descriptor/18649-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16566-5">
    <skos:scopeNote xml:lang="de">Sozio-kulturelle Faktoren, die das ökonomische Verhalten und das Wirtschaftsleben vorwiegend bei den sogenannten Naturvölkern bestimmen; Studien zum interkulturellen Vergleich des Wirtschaftsgeschehens.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12957-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19842-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11675-4">
    <skos:related rdf:resource="descriptor/18376-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10176-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14037-0">
    <skos:related rdf:resource="descriptor/18374-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14037-0">
    <skos:related rdf:resource="descriptor/14200-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19091-4">
    <skos:related rdf:resource="descriptor/12404-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19091-4">
    <skos:related rdf:resource="descriptor/12677-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15925-1">
    <skos:narrower rdf:resource="descriptor/19774-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15925-1">
    <skos:narrower rdf:resource="descriptor/18862-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11382-5">
    <skos:narrower rdf:resource="descriptor/11379-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11382-5">
    <skos:narrower rdf:resource="descriptor/11371-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11769-2">
    <skos:altLabel xml:lang="en">Wage-earner fund</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11769-2">
    <skos:altLabel xml:lang="de">Miteigentum von Arbeitnehmern</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11769-2">
    <skos:altLabel xml:lang="de">Mitarbeiterkapital</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11769-2">
    <skos:altLabel xml:lang="de">Mitarbeiterbeteiligung am Produktivkapital</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11769-2">
    <skos:altLabel xml:lang="de">Kapitalbeteiligung von Arbeitnehmern</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11769-2">
    <skos:altLabel xml:lang="de">Belegschaftsaktie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11769-2">
    <skos:altLabel xml:lang="de">Arbeitnehmerpartizipation am Kapital</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11769-2">
    <skos:altLabel xml:lang="de">Arbeitnehmerfonds</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18684-3">
    <skos:related rdf:resource="descriptor/12256-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18684-3">
    <skos:related rdf:resource="descriptor/18689-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18684-3">
    <skos:related rdf:resource="descriptor/13712-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16084-2">
    <skos:related rdf:resource="descriptor/15713-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16084-2">
    <skos:related rdf:resource="descriptor/10687-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10721-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19711-0">
    <skos:scopeNote xml:lang="de">Meist Konferenzen politischer Akteure</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18951-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18696-3">
    <skos:altLabel xml:lang="de">Theorie der Kreditmärkte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18696-3">
    <skos:altLabel xml:lang="de">Kreditnachfragetheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18696-3">
    <skos:altLabel xml:lang="de">Kreditmarkttheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18872-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70057">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19173-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17609-1">
    <skos:prefLabel xml:lang="en">Nepal</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17609-1">
    <skos:prefLabel xml:lang="de">Nepal</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72183">
    <rdfs:label xml:lang="de">Statistische Datengewinnung und deskriptive Statistik</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15082-4">
    <skos:narrower rdf:resource="descriptor/15093-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15082-4">
    <skos:narrower rdf:resource="descriptor/10901-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15082-4">
    <skos:narrower rdf:resource="descriptor/13712-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18791-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13558-5">
    <skos:narrower rdf:resource="descriptor/18584-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13558-5">
    <skos:narrower rdf:resource="descriptor/13250-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13558-5">
    <skos:narrower rdf:resource="descriptor/13630-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20921-0">
    <skos:related rdf:resource="descriptor/18771-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20921-0">
    <skos:related rdf:resource="descriptor/18941-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20921-0">
    <skos:related rdf:resource="descriptor/19544-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20921-0">
    <skos:related rdf:resource="descriptor/12165-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13371-0">
    <skos:related rdf:resource="descriptor/13375-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13371-0">
    <skos:related rdf:resource="descriptor/13374-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13371-0">
    <skos:related rdf:resource="descriptor/16541-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13371-0">
    <skos:related rdf:resource="descriptor/13348-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13371-0">
    <skos:related rdf:resource="descriptor/14728-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13371-0">
    <skos:related rdf:resource="descriptor/14042-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13371-0">
    <skos:related rdf:resource="descriptor/15658-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13371-0">
    <skos:related rdf:resource="descriptor/16670-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13371-0">
    <skos:related rdf:resource="descriptor/18635-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13371-0">
    <skos:related rdf:resource="descriptor/18892-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13371-0">
    <skos:related rdf:resource="descriptor/18891-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13371-0">
    <skos:related rdf:resource="descriptor/19578-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13371-0">
    <skos:related rdf:resource="descriptor/16479-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13371-0">
    <skos:related rdf:resource="descriptor/11157-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13371-0">
    <skos:related rdf:resource="descriptor/18890-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13371-0">
    <skos:related rdf:resource="descriptor/16635-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13371-0">
    <skos:related rdf:resource="descriptor/13415-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13371-0">
    <skos:related rdf:resource="descriptor/13370-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11740-5">
    <skos:prefLabel xml:lang="en">Lifetime income</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11740-5">
    <skos:prefLabel xml:lang="de">Lebenseinkommen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13137-1">
    <skos:prefLabel xml:lang="en">Leather processing industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13137-1">
    <skos:prefLabel xml:lang="de">Lederverarbeitende Industrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10265-0">
    <skos:narrower rdf:resource="descriptor/18183-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10265-0">
    <skos:narrower rdf:resource="descriptor/19231-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11268-3">
    <skos:broader rdf:resource="thsys/70113"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11268-3">
    <skos:broader rdf:resource="thsys/70052"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11268-3">
    <skos:broader rdf:resource="descriptor/15907-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11268-3">
    <skos:broader rdf:resource="descriptor/11262-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15937-1">
    <skos:broader rdf:resource="thsys/73357"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15937-1">
    <skos:broader rdf:resource="descriptor/18969-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16601-1">
    <skos:broader rdf:resource="thsys/73396"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16601-1">
    <skos:broader rdf:resource="thsys/70958"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16601-1">
    <skos:broader rdf:resource="descriptor/16602-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16601-1">
    <skos:broader rdf:resource="descriptor/16588-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13611-6">
    <skos:prefLabel xml:lang="en">Cooperation of transport companies</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13611-6">
    <skos:prefLabel xml:lang="de">Verkehrskooperation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10552-4">
    <skos:prefLabel xml:lang="en">Development finance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10552-4">
    <skos:prefLabel xml:lang="de">Entwicklungsfinanzierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18523-6">
    <skos:scopeNote xml:lang="de">Vorwiegend Handel mit tragbaren Kleincomputern für Endverbraucher/innen</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14034-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11598-3">
    <skos:altLabel xml:lang="en">ZBB (Zero-Base Budgeting)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11598-3">
    <skos:altLabel xml:lang="de">Null-Basis-Budgetierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19656-3">
    <skos:related rdf:resource="descriptor/12588-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19656-3">
    <skos:related rdf:resource="descriptor/10613-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19656-3">
    <skos:related rdf:resource="descriptor/18143-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19656-3">
    <skos:related rdf:resource="descriptor/11399-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10272-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13716-6">
    <skos:related rdf:resource="descriptor/13742-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13716-6">
    <skos:related rdf:resource="descriptor/18677-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13716-6">
    <skos:related rdf:resource="descriptor/13729-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13716-6">
    <skos:related rdf:resource="descriptor/13279-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13716-6">
    <skos:related rdf:resource="descriptor/13965-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11777-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id296">
    <rdf:value xml:lang="de">für grundlegende Formen des politischen Protests</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12781-0">
    <skos:altLabel xml:lang="de">Zweigstelle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12781-0">
    <skos:altLabel xml:lang="de">Zweigniederlassung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12781-0">
    <skos:altLabel xml:lang="de">Filialsystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12781-0">
    <skos:altLabel xml:lang="de">Filialbetrieb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10951-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17480-3">
    <skos:related rdf:resource="descriptor/17177-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10616-4">
    <skos:broader rdf:resource="thsys/71097"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10616-4">
    <skos:broader rdf:resource="descriptor/10619-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12510-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11676-2">
    <skos:scopeNote xml:lang="de">Sowohl für die betriebswirtschaftliche wie die finanzwissenschaftliche Bedeutung des Begriffs.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13464-0">
    <skos:prefLabel xml:lang="en">Commercial law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13464-0">
    <skos:prefLabel xml:lang="de">Handelsrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13430-3">
    <skos:related rdf:resource="descriptor/19399-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13430-3">
    <skos:related rdf:resource="descriptor/13702-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13430-3">
    <skos:related rdf:resource="descriptor/19273-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70884">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11356-6">
    <skos:related rdf:resource="descriptor/11379-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11356-6">
    <skos:related rdf:resource="descriptor/11378-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11356-6">
    <skos:related rdf:resource="descriptor/11373-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11356-6">
    <skos:related rdf:resource="descriptor/11274-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11356-6">
    <skos:related rdf:resource="descriptor/14012-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11356-6">
    <skos:related rdf:resource="descriptor/16658-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11356-6">
    <skos:related rdf:resource="descriptor/10491-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15108-5">
    <skos:narrower rdf:resource="descriptor/11032-4"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id121">
    <rdf:value xml:lang="de">Für ökologisch-alternativ produzierte Nahrungsmittel</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#inScheme">
    <rdfs:isDefinedBy rdf:resource="http://www.w3.org/2004/02/skos/core"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25501-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19714-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73363">
    <skos:prefLabel xml:lang="en">N.04.04.03  Armed forces</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73363">
    <skos:prefLabel xml:lang="de">N.04.04.03  Militär</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19725-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14232-2">
    <skos:broader rdf:resource="thsys/70348"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14232-2">
    <skos:broader rdf:resource="descriptor/14237-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14104-4">
    <skos:narrower rdf:resource="descriptor/14118-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14104-4">
    <skos:narrower rdf:resource="descriptor/14110-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14104-4">
    <skos:narrower rdf:resource="descriptor/19018-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14104-4">
    <skos:narrower rdf:resource="descriptor/14105-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14104-4">
    <skos:narrower rdf:resource="descriptor/14101-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17480-3">
    <skos:prefLabel xml:lang="en">Moldova</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17480-3">
    <skos:prefLabel xml:lang="de">Moldawien</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11900-2">
    <skos:altLabel xml:lang="de">Regionale Entwicklungskonzeption</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11900-2">
    <skos:altLabel xml:lang="de">Raumplanungspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11900-2">
    <skos:altLabel xml:lang="de">Raumplanung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11900-2">
    <skos:altLabel xml:lang="de">Raumordnungspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11900-2">
    <skos:altLabel xml:lang="de">Landesplanung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11900-2">
    <skos:altLabel xml:lang="de">Landesentwicklungsprogramm</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11900-2">
    <skos:altLabel xml:lang="de">Landesentwicklungsplan</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11900-2">
    <skos:altLabel xml:lang="de">Gebietsentwicklungsplan</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12154-6">
    <skos:prefLabel xml:lang="en">Project management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12154-6">
    <skos:prefLabel xml:lang="de">Projektmanagement</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10024-1">
    <skos:related rdf:resource="descriptor/19232-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10024-1">
    <skos:related rdf:resource="descriptor/11751-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10024-1">
    <skos:related rdf:resource="descriptor/11778-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10024-1">
    <skos:related rdf:resource="descriptor/16645-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10024-1">
    <skos:related rdf:resource="descriptor/10120-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10024-1">
    <skos:related rdf:resource="descriptor/10006-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10024-1">
    <skos:related rdf:resource="descriptor/10005-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15775-1">
    <skos:prefLabel xml:lang="en">Environmental sustainability</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15775-1">
    <skos:prefLabel xml:lang="de">Umweltverträglichkeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19238-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12563-3">
    <skos:altLabel xml:lang="de">Optimaler Personaleinsatz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10897-5">
    <skos:related rdf:resource="descriptor/12303-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10897-5">
    <skos:related rdf:resource="descriptor/12438-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10897-5">
    <skos:related rdf:resource="descriptor/16076-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10897-5">
    <skos:related rdf:resource="descriptor/12304-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10897-5">
    <skos:related rdf:resource="descriptor/12863-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10897-5">
    <skos:related rdf:resource="descriptor/12056-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10897-5">
    <skos:related rdf:resource="descriptor/10923-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10897-5">
    <skos:related rdf:resource="descriptor/10572-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10897-5">
    <skos:related rdf:resource="descriptor/10957-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10897-5">
    <skos:related rdf:resource="descriptor/18032-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17706-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15111-2">
    <skos:prefLabel xml:lang="en">Sample selection</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15111-2">
    <skos:prefLabel xml:lang="de">Statistisches Auswahlverfahren</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10151-1">
    <skos:broader rdf:resource="thsys/73103"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10151-1">
    <skos:broader rdf:resource="thsys/71118"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10151-1">
    <skos:broader rdf:resource="thsys/70180"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10151-1">
    <skos:broader rdf:resource="descriptor/10160-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10151-1">
    <skos:broader rdf:resource="descriptor/10038-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11252-4">
    <skos:altLabel xml:lang="en">Job counselling</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11252-4">
    <skos:altLabel xml:lang="en">Career guidance</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18445-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73384">
    <skos:prefLabel xml:lang="de">N.07.03  Werbe- und Konsumentenpsychologie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18061-4">
    <skos:related rdf:resource="descriptor/18157-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18061-4">
    <skos:related rdf:resource="descriptor/18178-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18061-4">
    <skos:related rdf:resource="descriptor/18694-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12110-5">
    <skos:related rdf:resource="descriptor/24847-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12110-5">
    <skos:related rdf:resource="descriptor/16684-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12110-5">
    <skos:related rdf:resource="descriptor/18867-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12110-5">
    <skos:related rdf:resource="descriptor/16692-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12110-5">
    <skos:related rdf:resource="descriptor/12617-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18642-5">
    <skos:prefLabel xml:lang="en">Postal monopoly</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18642-5">
    <skos:prefLabel xml:lang="de">Postmonopol</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12153-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12876-3">
    <skos:prefLabel xml:lang="en">Tax planning</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12876-3">
    <skos:prefLabel xml:lang="de">Steuerplanung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19072-1">
    <skos:related rdf:resource="descriptor/18161-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19072-1">
    <skos:related rdf:resource="descriptor/19581-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19072-1">
    <skos:related rdf:resource="descriptor/15064-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16391-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13593-3">
    <skos:related rdf:resource="descriptor/11832-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13593-3">
    <skos:related rdf:resource="descriptor/13595-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13593-3">
    <skos:related rdf:resource="descriptor/11828-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13593-3">
    <skos:related rdf:resource="descriptor/12013-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13593-3">
    <skos:related rdf:resource="descriptor/13594-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13593-3">
    <skos:related rdf:resource="descriptor/11993-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13593-3">
    <skos:related rdf:resource="descriptor/16552-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13593-3">
    <skos:related rdf:resource="descriptor/11900-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19847-3">
    <skos:altLabel xml:lang="de">Verschwiegenheitspflicht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19847-3">
    <skos:altLabel xml:lang="de">Verschwiegenheit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19847-3">
    <skos:altLabel xml:lang="de">Staatsgeheimnis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19847-3">
    <skos:altLabel xml:lang="de">Schweigepflicht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19847-3">
    <skos:altLabel xml:lang="de">Geheimnisschutz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19847-3">
    <skos:altLabel xml:lang="de">Geheimnis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19847-3">
    <skos:altLabel xml:lang="de">Geheimhaltungspflicht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19847-3">
    <skos:altLabel xml:lang="de">Dienstgeheimnis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19847-3">
    <skos:altLabel xml:lang="de">Berufsgeheimnis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19847-3">
    <skos:altLabel xml:lang="de">Amtsgeheimnis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19690-3">
    <skos:prefLabel xml:lang="en">National symbol</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19690-3">
    <skos:prefLabel xml:lang="de">Staatssymbol</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10854-2">
    <skos:prefLabel xml:lang="en">Trade finance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10854-2">
    <skos:prefLabel xml:lang="de">Außenhandelsfinanzierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18766-1">
    <skos:broader rdf:resource="thsys/73340"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18766-1">
    <skos:broader rdf:resource="thsys/71037"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18766-1">
    <skos:broader rdf:resource="thsys/70881"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14642-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13492-2">
    <skos:broader rdf:resource="thsys/70947"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13492-2">
    <skos:broader rdf:resource="descriptor/13200-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11834-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19633-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18808-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16146-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19407-3">
    <skos:related rdf:resource="descriptor/11648-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14480-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14599-0">
    <skos:altLabel xml:lang="de">Chemisches Produkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14599-0">
    <skos:altLabel xml:lang="de">Chemieprodukt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14437-5">
    <skos:broader rdf:resource="thsys/70258"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14437-5">
    <skos:broader rdf:resource="descriptor/14444-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14437-5">
    <skos:broader rdf:resource="descriptor/14433-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19402-6">
    <skos:prefLabel xml:lang="en">Average tax rate</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19402-6">
    <skos:prefLabel xml:lang="de">Durchschnittssteuersatz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10415-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11282-2">
    <skos:altLabel xml:lang="en">Low-skilled worker</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11282-2">
    <skos:altLabel xml:lang="de">Ungelernte Arbeiterinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11282-2">
    <skos:altLabel xml:lang="de">Ungelernte Arbeiter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11282-2">
    <skos:altLabel xml:lang="de">Hilfsarbeiterinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11282-2">
    <skos:altLabel xml:lang="de">Hilfsarbeiter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11282-2">
    <skos:altLabel xml:lang="de">Geringqualifizierte Arbeitskräfte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11282-2">
    <skos:altLabel xml:lang="de">Geringqualifizierte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11282-2">
    <skos:altLabel xml:lang="de">Gering qualifizierte Arbeitskräfte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14934-5">
    <skos:broader rdf:resource="thsys/70498"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14934-5">
    <skos:broader rdf:resource="descriptor/14931-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11637-5">
    <skos:altLabel xml:lang="de">Steuerlehre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18862-5">
    <skos:related rdf:resource="descriptor/18863-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18862-5">
    <skos:related rdf:resource="descriptor/15920-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70874">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11826-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10006-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19632-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10062-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17654-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16421-3">
    <skos:scopeNote xml:lang="de">NATIONAL: Bestrebungen von Bewohnern einer Region in einem zentralistischen Staat, ihre (zumeist ethnisch begründete) Eigenart zu wahren und zu fördern; INTERNATIONAL: Bestrebungen, die Zusammenarbeit der Staaten einer Region zu intensivieren.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18302-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10605-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13701-5">
    <skos:prefLabel xml:lang="en">Big bank</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13701-5">
    <skos:prefLabel xml:lang="de">Großbank</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15392-3">
    <skos:altLabel xml:lang="en">Seasonal adjustment</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15392-3">
    <skos:altLabel xml:lang="de">Saisonnormale</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15392-3">
    <skos:altLabel xml:lang="de">Saisonkomponente</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15392-3">
    <skos:altLabel xml:lang="de">Saisonbereinigte Zeitreihe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18917-6">
    <skos:prefLabel xml:lang="en">Economics of leisure</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18917-6">
    <skos:prefLabel xml:lang="de">Freizeitökonomik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13278-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18767-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14997-2">
    <skos:narrower rdf:resource="descriptor/14999-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14997-2">
    <skos:narrower rdf:resource="descriptor/14998-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10677-5">
    <skos:altLabel xml:lang="de">Wechselkurseffekt der Zahlungsbilanz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10677-5">
    <skos:altLabel xml:lang="de">Preiseffekt der Zahlungsbilanz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10677-5">
    <skos:altLabel xml:lang="de">Harberger-Laursen-Metzler-Effekt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10677-5">
    <skos:altLabel xml:lang="de">Geldmengen-Preis-Mechanismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10677-5">
    <skos:altLabel xml:lang="de">Elastizitätsansatz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10677-5">
    <skos:altLabel xml:lang="de">Absorptionsansatz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15962-2">
    <skos:broader rdf:resource="thsys/73380"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15962-2">
    <skos:broader rdf:resource="thsys/70862"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15962-2">
    <skos:broader rdf:resource="thsys/70154"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15962-2">
    <skos:broader rdf:resource="descriptor/13413-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15962-2">
    <skos:broader rdf:resource="descriptor/19841-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20206-3">
    <skos:broader rdf:resource="thsys/70040"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20206-3">
    <skos:broader rdf:resource="thsys/70029"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20206-3">
    <skos:broader rdf:resource="descriptor/12754-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20206-3">
    <skos:broader rdf:resource="descriptor/12794-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10103-5">
    <skos:prefLabel xml:lang="en">Second best</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10103-5">
    <skos:prefLabel xml:lang="de">Second Best</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11555-0">
    <skos:related rdf:resource="descriptor/19551-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11555-0">
    <skos:related rdf:resource="descriptor/13292-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11555-0">
    <skos:related rdf:resource="descriptor/18903-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11555-0">
    <skos:related rdf:resource="descriptor/19578-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11555-0">
    <skos:related rdf:resource="descriptor/18063-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11555-0">
    <skos:related rdf:resource="descriptor/11157-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11555-0">
    <skos:related rdf:resource="descriptor/19517-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11555-0">
    <skos:related rdf:resource="descriptor/19676-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71040">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20948-1">
    <skos:related rdf:resource="descriptor/13992-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20948-1">
    <skos:related rdf:resource="descriptor/11302-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13795-5">
    <skos:broader rdf:resource="thsys/73391"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13795-5">
    <skos:broader rdf:resource="thsys/70899"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13795-5">
    <skos:broader rdf:resource="descriptor/18726-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13795-5">
    <skos:broader rdf:resource="descriptor/16043-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73373">
    <skos:broader rdf:resource="thsys/73374"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11863-3">
    <skos:related rdf:resource="descriptor/13595-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11863-3">
    <skos:related rdf:resource="descriptor/13594-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11863-3">
    <skos:related rdf:resource="descriptor/11993-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11863-3">
    <skos:related rdf:resource="descriptor/11987-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11863-3">
    <skos:related rdf:resource="descriptor/11876-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14803-6">
    <skos:prefLabel xml:lang="en">Paper product</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14803-6">
    <skos:prefLabel xml:lang="de">Papierprodukt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11998-1">
    <skos:prefLabel xml:lang="en">Local planning</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11998-1">
    <skos:prefLabel xml:lang="de">Kommunalplanung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14645-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71068">
    <skos:prefLabel xml:lang="en">V.11.02.01  Regional planning</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71068">
    <skos:prefLabel xml:lang="de">V.11.02.01  Regionalplanung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14967-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18258-6">
    <skos:altLabel xml:lang="de">Selbstmedikationsmittel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18258-6">
    <skos:altLabel xml:lang="de">OTC-Arzneimittel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18258-6">
    <skos:altLabel xml:lang="de">Freiverkäufliches Arzneimittel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18105-3">
    <skos:scopeNote xml:lang="de">Verwaltung von Pensionsfonds und -kassen zur betrieblichen Altersversorgung einzelner Arbeitgeber durch Lebensversicherungsunternehmen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15753-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18082-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14170-5">
    <skos:related rdf:resource="descriptor/19630-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14170-5">
    <skos:related rdf:resource="descriptor/12962-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17665-5">
    <skos:altLabel xml:lang="de">Demokratische Republik Madagaskar</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13832-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13379-5">
    <skos:broader rdf:resource="thsys/70882"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13379-5">
    <skos:broader rdf:resource="descriptor/13301-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13379-5">
    <skos:broader rdf:resource="descriptor/18762-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70959">
    <skos:broader rdf:resource="thsys/70962"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20842-3">
    <skos:scopeNote xml:lang="de">Das Konzept ist im Rahmen der Hilfe für Entwicklungsländer entstanden und bezeichnet i.d.R. Bedingungen, die nach Meinung der Geberländer von seiten der Regierung eines Empfängerlandes erfüllt sein müssen, wenn es z.B. Mittel der Weltbank oder des IWF in Anspruch nehmen will. Gleichzeitig ein sozialwissenschaftlicher Ansatz zur Analyse dieses und vergleichbarer Konzepte.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13777-0">
    <skos:broader rdf:resource="thsys/70899"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12845-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13273-0">
    <skos:altLabel xml:lang="de">Postzustelldienst</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13273-0">
    <skos:altLabel xml:lang="de">Postversand</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13273-0">
    <skos:altLabel xml:lang="de">Postfrachtdienst</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13273-0">
    <skos:altLabel xml:lang="de">Paketdienst</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13273-0">
    <skos:altLabel xml:lang="de">Paketbeförderung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13273-0">
    <skos:altLabel xml:lang="de">Briefverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13273-0">
    <skos:altLabel xml:lang="de">Briefbeförderung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70020">
    <skos:prefLabel xml:lang="en">V.04.01  Business cycle</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70020">
    <skos:prefLabel xml:lang="de">V.04.01  Konjunktur</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70853">
    <skos:narrower rdf:resource="descriptor/13405-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70853">
    <skos:narrower rdf:resource="descriptor/13402-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70853">
    <skos:narrower rdf:resource="descriptor/13401-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70853">
    <skos:narrower rdf:resource="descriptor/16038-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70853">
    <skos:narrower rdf:resource="descriptor/13400-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70853">
    <skos:narrower rdf:resource="descriptor/16379-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70853">
    <skos:narrower rdf:resource="descriptor/16226-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70853">
    <skos:narrower rdf:resource="descriptor/13395-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70853">
    <skos:narrower rdf:resource="descriptor/18847-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70853">
    <skos:narrower rdf:resource="descriptor/18844-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70853">
    <skos:narrower rdf:resource="descriptor/16384-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70853">
    <skos:narrower rdf:resource="descriptor/13394-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70853">
    <skos:narrower rdf:resource="descriptor/16630-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70853">
    <skos:narrower rdf:resource="descriptor/10984-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70853">
    <skos:narrower rdf:resource="descriptor/10968-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70853">
    <skos:narrower rdf:resource="descriptor/11168-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70853">
    <skos:narrower rdf:resource="descriptor/15472-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70853">
    <skos:narrower rdf:resource="descriptor/13988-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70853">
    <skos:narrower rdf:resource="descriptor/10558-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70853">
    <skos:narrower rdf:resource="descriptor/18832-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70853">
    <skos:narrower rdf:resource="descriptor/19521-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70853">
    <skos:narrower rdf:resource="descriptor/16382-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70853">
    <skos:narrower rdf:resource="thsys/70852"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70853">
    <skos:narrower rdf:resource="thsys/70851"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70853">
    <skos:narrower rdf:resource="thsys/70850"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14382-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19843-4">
    <skos:prefLabel xml:lang="en">Right to political asylum</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19843-4">
    <skos:prefLabel xml:lang="de">Asylrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70253">
    <rdfs:label xml:lang="en">Middle East</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70253">
    <rdfs:label xml:lang="de">Mittlerer Osten</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16405-1">
    <skos:broader rdf:resource="thsys/73396"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16405-1">
    <skos:broader rdf:resource="thsys/73394"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16405-1">
    <skos:broader rdf:resource="thsys/73383"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id225">
    <rdfs:seeAlso rdf:resource="descriptor/14490-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13303-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19630-0">
    <skos:narrower rdf:resource="descriptor/19626-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19319-0">
    <skos:prefLabel xml:lang="en">Strategic trade policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19319-0">
    <skos:prefLabel xml:lang="de">Strategische Handelspolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11751-0">
    <skos:related rdf:resource="descriptor/11765-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11751-0">
    <skos:related rdf:resource="descriptor/11779-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11751-0">
    <skos:related rdf:resource="descriptor/11752-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11751-0">
    <skos:related rdf:resource="descriptor/10110-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11751-0">
    <skos:related rdf:resource="descriptor/19589-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11751-0">
    <skos:related rdf:resource="descriptor/11778-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11751-0">
    <skos:related rdf:resource="descriptor/10024-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11751-0">
    <skos:related rdf:resource="descriptor/11735-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11751-0">
    <skos:related rdf:resource="descriptor/11753-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15927-4">
    <skos:altLabel xml:lang="de">Mann</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18163-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11358-2">
    <skos:altLabel xml:lang="en">In-company trainers</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11358-2">
    <skos:altLabel xml:lang="de">Trainerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11358-2">
    <skos:altLabel xml:lang="de">Trainer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11358-2">
    <skos:altLabel xml:lang="de">Ausbilderinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70904">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11398-4">
    <skos:related rdf:resource="descriptor/11367-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18857-5">
    <skos:broader rdf:resource="thsys/70847"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18857-5">
    <skos:broader rdf:resource="descriptor/11367-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14229-5">
    <skos:prefLabel xml:lang="en">Gypsum</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14229-5">
    <skos:prefLabel xml:lang="de">Gips</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18171-4">
    <skos:prefLabel xml:lang="en">Chaos theory</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18171-4">
    <skos:prefLabel xml:lang="de">Chaostheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19689-2">
    <skos:prefLabel xml:lang="en">Government organ</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19689-2">
    <skos:prefLabel xml:lang="de">Staatsorgan</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11054-1">
    <skos:prefLabel xml:lang="en">Market failure</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11054-1">
    <skos:prefLabel xml:lang="de">Marktversagen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16797-0">
    <skos:related rdf:resource="descriptor/18292-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16797-0">
    <skos:related rdf:resource="descriptor/14812-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11943-5">
    <skos:prefLabel xml:lang="en">Regional growth</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11943-5">
    <skos:prefLabel xml:lang="de">Regionales Wachstum</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70963">
    <skos:broader rdf:resource="thsys/w"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11187-3">
    <skos:related rdf:resource="descriptor/11260-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11187-3">
    <skos:related rdf:resource="descriptor/10184-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11187-3">
    <skos:related rdf:resource="descriptor/19509-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11187-3">
    <skos:related rdf:resource="descriptor/11194-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11187-3">
    <skos:related rdf:resource="descriptor/19074-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11187-3">
    <skos:related rdf:resource="descriptor/12765-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11187-3">
    <skos:related rdf:resource="descriptor/11248-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11187-3">
    <skos:related rdf:resource="descriptor/12631-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11187-3">
    <skos:related rdf:resource="descriptor/10169-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11187-3">
    <skos:related rdf:resource="descriptor/11180-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11187-3">
    <skos:related rdf:resource="descriptor/16526-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70968">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14765-2">
    <skos:altLabel xml:lang="en">Monitor (Computer)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14765-2">
    <skos:altLabel xml:lang="de">Farbmonitor</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14765-2">
    <skos:altLabel xml:lang="de">Farbbildschirm</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14765-2">
    <skos:altLabel xml:lang="de">Display (Bildschirm)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14765-2">
    <skos:altLabel xml:lang="de">Datensichtgerät</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14765-2">
    <skos:altLabel xml:lang="de">Computerbildschirm</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15399-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13312-2">
    <skos:prefLabel xml:lang="en">Homes and hostels</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13312-2">
    <skos:prefLabel xml:lang="de">Heim</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19989-4">
    <skos:broader rdf:resource="thsys/73393"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19989-4">
    <skos:broader rdf:resource="thsys/70049"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19989-4">
    <skos:broader rdf:resource="descriptor/16727-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18312-5">
    <skos:altLabel xml:lang="de">Fischereizone</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16299-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15878-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16039-0">
    <skos:prefLabel xml:lang="en">Contract law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16039-0">
    <skos:prefLabel xml:lang="de">Vertragsrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12508-4">
    <skos:narrower rdf:resource="descriptor/19012-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11481-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17987-4">
    <skos:altLabel xml:lang="de">SADC-Länder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17987-4">
    <skos:altLabel xml:lang="de">SADCC-Staaten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17987-4">
    <skos:altLabel xml:lang="de">Frontstaaten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19738-1">
    <skos:broader rdf:resource="thsys/73388"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19738-1">
    <skos:broader rdf:resource="descriptor/15671-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19738-1">
    <skos:broader rdf:resource="descriptor/16719-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15899-4">
    <skos:prefLabel xml:lang="en">Outer space</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15899-4">
    <skos:prefLabel xml:lang="de">Weltraum</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18445-0">
    <skos:altLabel xml:lang="de">Strahlensterilisation von Nahrungsmitteln</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18445-0">
    <skos:altLabel xml:lang="de">Lebensmittelkonservierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18445-0">
    <skos:altLabel xml:lang="de">Lebensmittelbestrahlung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18445-0">
    <skos:altLabel xml:lang="de">Konservierung von Nahrungsmitteln</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18445-0">
    <skos:altLabel xml:lang="de">Bestrahlung von Nahrungsmitteln</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13417-2">
    <skos:broader rdf:resource="thsys/73335"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13417-2">
    <skos:broader rdf:resource="thsys/70862"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13417-2">
    <skos:broader rdf:resource="descriptor/13423-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13565-1">
    <skos:broader rdf:resource="thsys/73389"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13565-1">
    <skos:broader rdf:resource="thsys/70940"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13565-1">
    <skos:broader rdf:resource="descriptor/13569-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10905-4">
    <skos:scopeNote xml:lang="de">Theorie und Empirie der Marktformen. Auch für die Marktstruktur einzelner Branchen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70947">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11071-1">
    <skos:prefLabel xml:lang="en">Labour theory of value</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11071-1">
    <skos:prefLabel xml:lang="de">Arbeitswertlehre</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10395-1">
    <skos:broader rdf:resource="thsys/70038"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10395-1">
    <skos:broader rdf:resource="descriptor/10987-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19772-1">
    <skos:prefLabel xml:lang="en">Envy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19772-1">
    <skos:prefLabel xml:lang="de">Neid</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16191-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16680-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18556-5">
    <skos:altLabel xml:lang="en">Experiential marketing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18556-5">
    <skos:altLabel xml:lang="de">Erlebnisorientierung im Handel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18556-5">
    <skos:altLabel xml:lang="de">Erlebniskauf</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18556-5">
    <skos:altLabel xml:lang="de">Erlebnishandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18734-0">
    <skos:prefLabel xml:lang="en">EU insurance law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18734-0">
    <skos:prefLabel xml:lang="de">EU-Versicherungsrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11427-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19798-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10458-3">
    <skos:altLabel xml:lang="de">Öffentlicher Forschungsbereich</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18122-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13849-1">
    <skos:prefLabel xml:lang="en">Housing conditions</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13849-1">
    <skos:prefLabel xml:lang="de">Wohnungsversorgung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70140">
    <rdfs:label xml:lang="de">Betriebswirtschaft</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70938">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19415-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13962-5">
    <skos:related rdf:resource="descriptor/18966-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13962-5">
    <skos:related rdf:resource="descriptor/18947-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13962-5">
    <skos:related rdf:resource="descriptor/10470-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13962-5">
    <skos:related rdf:resource="descriptor/18789-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13962-5">
    <skos:related rdf:resource="descriptor/11358-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18272-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15493-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19566-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73385">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.07.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10499-3">
    <skos:broader rdf:resource="thsys/70307"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17642-3">
    <skos:altLabel xml:lang="de">Afrikanische Staaten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17642-3">
    <skos:altLabel xml:lang="de">Afrikanische Länder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12703-6">
    <skos:broader rdf:resource="thsys/73345"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12703-6">
    <skos:broader rdf:resource="thsys/70310"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12703-6">
    <skos:broader rdf:resource="descriptor/12711-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19248-4">
    <skos:prefLabel xml:lang="en">Emerging markets</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19248-4">
    <skos:prefLabel xml:lang="de">Aufstrebende Märkte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13572-4">
    <skos:broader rdf:resource="thsys/70943"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13572-4">
    <skos:broader rdf:resource="descriptor/15679-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13250-5">
    <skos:prefLabel xml:lang="en">Bus traffic</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13250-5">
    <skos:prefLabel xml:lang="de">Omnibusverkehr</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11566-2">
    <skos:altLabel xml:lang="de">Vermögensverlustausgleich</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11566-2">
    <skos:altLabel xml:lang="de">Lastenausgleichsfonds</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11566-2">
    <skos:altLabel xml:lang="de">Lastenausgleichsabgabe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11566-2">
    <skos:altLabel xml:lang="de">Kreditgewinnabgabe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11566-2">
    <skos:altLabel xml:lang="de">Hypothekengewinnabgabe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13214-2">
    <skos:prefLabel xml:lang="en">Do-it-yourself store</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13214-2">
    <skos:prefLabel xml:lang="de">Heimwerkermarkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12573-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16666-1">
    <skos:prefLabel xml:lang="en">Alcohol consumption</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16666-1">
    <skos:prefLabel xml:lang="de">Alkoholkonsum</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10627-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15888-2">
    <skos:related rdf:resource="descriptor/19622-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16697-4">
    <skos:related rdf:resource="descriptor/16554-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16697-4">
    <skos:related rdf:resource="descriptor/15658-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16697-4">
    <skos:related rdf:resource="descriptor/16475-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16697-4">
    <skos:related rdf:resource="descriptor/15904-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17199-1">
    <skos:scopeNote xml:lang="de">Wird sowohl für die geographische wie für die politische Abgrenzung der Region verwendet</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19790-6">
    <skos:related rdf:resource="descriptor/19528-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19795-3">
    <skos:narrower rdf:resource="descriptor/18402-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16087-3">
    <skos:scopeNote xml:lang="de">Entscheidung von Rechtsfragen durch Finanz-/Steuergerichte.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id84">
    <rdf:value xml:lang="de">Für Umweltsteuern allgemein</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18387-2">
    <skos:related rdf:resource="descriptor/19055-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18387-2">
    <skos:related rdf:resource="descriptor/15754-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19461-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70818">
    <skos:prefLabel xml:lang="de">W.32  Abfallwirtschaft und Recycling</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18777-3">
    <skos:altLabel xml:lang="en">Biotech research</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18777-3">
    <skos:altLabel xml:lang="de">Biotechnologieforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11707-3">
    <skos:prefLabel xml:lang="en">Tax revenue estimation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11707-3">
    <skos:prefLabel xml:lang="de">Steuerschätzung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18618-2">
    <skos:broader rdf:resource="thsys/70940"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18618-2">
    <skos:broader rdf:resource="descriptor/18619-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18618-2">
    <skos:broader rdf:resource="descriptor/13621-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12984-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13090-1">
    <skos:related rdf:resource="descriptor/13067-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13090-1">
    <skos:related rdf:resource="descriptor/14567-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16295-3">
    <skos:altLabel xml:lang="de">Staatsnotstand</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16295-3">
    <skos:altLabel xml:lang="de">Politischer Notstand</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16295-3">
    <skos:altLabel xml:lang="de">Notstandsverordnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16295-3">
    <skos:altLabel xml:lang="de">Notstandsrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16295-3">
    <skos:altLabel xml:lang="de">Katastrophennotstand</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16295-3">
    <skos:altLabel xml:lang="de">Innerer Notstand</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16295-3">
    <skos:altLabel xml:lang="de">Ausnahmezustand</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10454-4">
    <skos:broader rdf:resource="thsys/71109"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10454-4">
    <skos:broader rdf:resource="thsys/70310"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10454-4">
    <skos:broader rdf:resource="descriptor/10466-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70201">
    <skos:broader rdf:resource="thsys/p"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19250-3">
    <skos:altLabel xml:lang="en">Scope economies</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19250-3">
    <skos:altLabel xml:lang="de">Umfangsvorteil</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19250-3">
    <skos:altLabel xml:lang="de">Sortimentsertrag</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18108-4">
    <skos:related rdf:resource="descriptor/12191-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18108-4">
    <skos:related rdf:resource="descriptor/16048-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19059-0">
    <skos:broader rdf:resource="thsys/70961"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19059-0">
    <skos:broader rdf:resource="descriptor/13217-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70234">
    <skos:prefLabel xml:lang="en">V.03  Macroeconomics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70234">
    <skos:prefLabel xml:lang="de">V.03  Makroökonomik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18938-5">
    <skos:altLabel xml:lang="de">Privat-TV</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18938-5">
    <skos:altLabel xml:lang="de">Privatfernsehen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18938-5">
    <skos:altLabel xml:lang="de">Privater Fernsehsender</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18938-5">
    <skos:altLabel xml:lang="de">Pay-TV</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18938-5">
    <skos:altLabel xml:lang="de">Kommerzielles Fernsehen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12927-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14651-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14772-5">
    <skos:altLabel xml:lang="de">Technisches Porzellan</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14772-5">
    <skos:altLabel xml:lang="de">Schneidkeramik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14772-5">
    <skos:altLabel xml:lang="de">Keramischer Werkstoff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14772-5">
    <skos:altLabel xml:lang="de">Keramischer Verbundstoff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14772-5">
    <skos:altLabel xml:lang="de">Keramischer Isolator</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14772-5">
    <skos:altLabel xml:lang="de">Keramischer Hochleistungswerkstoff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14772-5">
    <skos:altLabel xml:lang="de">Keramikschneidewerkzeug</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14772-5">
    <skos:altLabel xml:lang="de">Hochleistungskeramik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14772-5">
    <skos:altLabel xml:lang="de">Elektrokeramik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16767-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id331">
    <rdfs:seeAlso rdf:resource="descriptor/19630-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70244">
    <skos:prefLabel xml:lang="de">B.04.01  Personalführung, -planung und -entwicklung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18751-0">
    <skos:broader rdf:resource="thsys/70888"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18751-0">
    <skos:broader rdf:resource="thsys/70245"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18751-0">
    <skos:broader rdf:resource="descriptor/13829-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18751-0">
    <skos:broader rdf:resource="descriptor/12592-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10247-2">
    <skos:narrower rdf:resource="descriptor/10251-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10247-2">
    <skos:narrower rdf:resource="descriptor/10246-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10247-2">
    <skos:narrower rdf:resource="descriptor/10383-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19236-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19038-1">
    <skos:scopeNote xml:lang="de">Auch für theoretische Literatur, z.B. zur experimentellen Wirtschaftsforschung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16475-1">
    <skos:narrower rdf:resource="descriptor/11253-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16475-1">
    <skos:narrower rdf:resource="descriptor/16525-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12278-2">
    <skos:related rdf:resource="descriptor/10015-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12278-2">
    <skos:related rdf:resource="descriptor/18100-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12278-2">
    <skos:related rdf:resource="descriptor/12916-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12278-2">
    <skos:related rdf:resource="descriptor/10008-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12278-2">
    <skos:related rdf:resource="descriptor/18107-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12278-2">
    <skos:related rdf:resource="descriptor/19094-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15704-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11705-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16552-2">
    <skos:related rdf:resource="descriptor/10567-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16552-2">
    <skos:related rdf:resource="descriptor/19829-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16552-2">
    <skos:related rdf:resource="descriptor/13593-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16552-2">
    <skos:related rdf:resource="descriptor/19291-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16552-2">
    <skos:related rdf:resource="descriptor/11900-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16552-2">
    <skos:related rdf:resource="descriptor/16421-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16552-2">
    <skos:related rdf:resource="descriptor/19246-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16552-2">
    <skos:related rdf:resource="descriptor/11910-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16552-2">
    <skos:related rdf:resource="descriptor/11886-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16552-2">
    <skos:related rdf:resource="descriptor/11895-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16552-2">
    <skos:related rdf:resource="descriptor/11857-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16552-2">
    <skos:related rdf:resource="descriptor/11928-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16552-2">
    <skos:related rdf:resource="descriptor/12004-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16552-2">
    <skos:related rdf:resource="descriptor/13639-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16552-2">
    <skos:related rdf:resource="descriptor/11599-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16552-2">
    <skos:related rdf:resource="descriptor/11804-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15661-2">
    <skos:narrower rdf:resource="descriptor/12842-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17184-0">
    <skos:prefLabel xml:lang="en">Greece</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17184-0">
    <skos:prefLabel xml:lang="de">Griechenland</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10548-2">
    <skos:altLabel xml:lang="de">Machbarkeitsstudie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11044-4">
    <skos:broader rdf:resource="thsys/71101"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11044-4">
    <skos:broader rdf:resource="thsys/70039"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11044-4">
    <skos:broader rdf:resource="descriptor/10005-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19627-3">
    <skos:altLabel xml:lang="en">Protection of pet animals</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19627-3">
    <skos:altLabel xml:lang="en">Protection of animals</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19627-3">
    <skos:altLabel xml:lang="en">Animal welfare</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19627-3">
    <skos:altLabel xml:lang="en">Animal rights</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19627-3">
    <skos:altLabel xml:lang="de">Tierschutzauflage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19627-3">
    <skos:altLabel xml:lang="de">Tierrechte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19627-3">
    <skos:altLabel xml:lang="de">Artgerechte Tierhaltung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12930-2">
    <skos:altLabel xml:lang="de">Verpackungsgewerbe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16493-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15825-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13756-1">
    <skos:prefLabel xml:lang="en">Banking</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13756-1">
    <skos:prefLabel xml:lang="de">Bankgeschäft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16019-6">
    <skos:narrower rdf:resource="descriptor/16024-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16019-6">
    <skos:narrower rdf:resource="descriptor/19858-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16019-6">
    <skos:narrower rdf:resource="descriptor/15769-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16019-6">
    <skos:narrower rdf:resource="descriptor/18870-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16019-6">
    <skos:narrower rdf:resource="descriptor/19854-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16019-6">
    <skos:narrower rdf:resource="descriptor/11870-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16019-6">
    <skos:narrower rdf:resource="descriptor/18829-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16019-6">
    <skos:narrower rdf:resource="descriptor/13660-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16019-6">
    <skos:narrower rdf:resource="descriptor/16004-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16019-6">
    <skos:narrower rdf:resource="descriptor/18805-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16019-6">
    <skos:narrower rdf:resource="descriptor/15995-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16019-6">
    <skos:narrower rdf:resource="descriptor/18876-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16019-6">
    <skos:narrower rdf:resource="descriptor/16052-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16019-6">
    <skos:narrower rdf:resource="descriptor/18288-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16019-6">
    <skos:narrower rdf:resource="descriptor/19821-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16019-6">
    <skos:narrower rdf:resource="descriptor/15604-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16019-6">
    <skos:narrower rdf:resource="descriptor/15992-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16019-6">
    <skos:narrower rdf:resource="descriptor/12020-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16019-6">
    <skos:narrower rdf:resource="descriptor/16134-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16019-6">
    <skos:narrower rdf:resource="descriptor/15989-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16019-6">
    <skos:narrower rdf:resource="descriptor/18259-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16019-6">
    <skos:narrower rdf:resource="descriptor/15988-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11533-3">
    <skos:narrower rdf:resource="descriptor/19405-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18120-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12993-6">
    <skos:prefLabel xml:lang="en">Processing industries</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12993-6">
    <skos:prefLabel xml:lang="de">Verarbeitendes Gewerbe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19852-3">
    <skos:altLabel xml:lang="de">Prozeßkostenhilfe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19852-3">
    <skos:altLabel xml:lang="de">Armenrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10684-1">
    <skos:prefLabel xml:lang="en">Trade promotion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10684-1">
    <skos:prefLabel xml:lang="de">Außenhandelsförderung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12026-1">
    <skos:narrower rdf:resource="descriptor/13840-5"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id233">
    <rdf:value xml:lang="de">Für niedrige Löhne und Gehälter</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70979">
    <skos:narrower rdf:resource="descriptor/18405-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70979">
    <skos:narrower rdf:resource="descriptor/18417-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70979">
    <skos:narrower rdf:resource="descriptor/18447-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70979">
    <skos:narrower rdf:resource="descriptor/18419-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70979">
    <skos:narrower rdf:resource="descriptor/18482-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70979">
    <skos:narrower rdf:resource="descriptor/11820-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70979">
    <skos:narrower rdf:resource="descriptor/18481-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70979">
    <skos:narrower rdf:resource="descriptor/10564-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70979">
    <skos:narrower rdf:resource="descriptor/18425-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70979">
    <skos:narrower rdf:resource="descriptor/18375-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18963-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18952-4">
    <skos:related rdf:resource="descriptor/16802-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18952-4">
    <skos:related rdf:resource="descriptor/13351-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14933-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18004-2">
    <skos:prefLabel xml:lang="en">Personal computer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18004-2">
    <skos:prefLabel xml:lang="de">PC (Personal Computer)</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#historyNote">
    <skos:definition xml:lang="en">A note about the past state/use/meaning of a concept.</skos:definition>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17780-5">
    <skos:broader rdf:resource="thsys/70086"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17780-5">
    <skos:broader rdf:resource="descriptor/17791-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14810-2">
    <skos:broader rdf:resource="thsys/70328"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12801-6">
    <skos:altLabel xml:lang="de">NoName-Produkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70269">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19753-5">
    <skos:broader rdf:resource="thsys/73395"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15033-3">
    <skos:broader rdf:resource="thsys/73350"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15033-3">
    <skos:broader rdf:resource="thsys/71070"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15033-3">
    <skos:broader rdf:resource="thsys/70269"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15033-3">
    <skos:broader rdf:resource="descriptor/19442-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15033-3">
    <skos:broader rdf:resource="descriptor/15045-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13277-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10781-3">
    <skos:prefLabel xml:lang="en">Currency speculation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10781-3">
    <skos:prefLabel xml:lang="de">Devisenspekulation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10891-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14076-4">
    <skos:altLabel xml:lang="de">Zuchtpilz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14076-4">
    <skos:altLabel xml:lang="de">Champignon</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11694-0">
    <skos:related rdf:resource="descriptor/11704-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11694-0">
    <skos:related rdf:resource="descriptor/11547-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11694-0">
    <skos:related rdf:resource="descriptor/11715-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16430-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18233-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14473-1">
    <skos:broader rdf:resource="thsys/70135"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18307-5">
    <skos:scopeNote xml:lang="de">Auch für Verpackungen, die nur als umweltverträglich bezeichnet werden.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10968-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17993-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13341-2">
    <skos:related rdf:resource="descriptor/12164-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12914-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18011-5">
    <skos:broader rdf:resource="thsys/71071"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18011-5">
    <skos:broader rdf:resource="descriptor/10963-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15918-5">
    <skos:prefLabel xml:lang="en">Ethnic group</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15918-5">
    <skos:prefLabel xml:lang="de">Ethnische Gruppe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13985-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11101-4">
    <skos:altLabel xml:lang="de">Stamokap</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11101-4">
    <skos:altLabel xml:lang="de">Staatsmonopolistischer Kapitalismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11101-4">
    <skos:altLabel xml:lang="de">Neokapitalismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70550">
    <skos:prefLabel xml:lang="en">B.07.02  Product management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70550">
    <skos:prefLabel xml:lang="de">B.07.02  Produktpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18250-1">
    <skos:related rdf:resource="descriptor/14173-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19458-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12569-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13767-3">
    <skos:broader rdf:resource="thsys/71107"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13767-3">
    <skos:broader rdf:resource="descriptor/12299-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14040-4">
    <skos:related rdf:resource="descriptor/18277-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14040-4">
    <skos:related rdf:resource="descriptor/13357-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14040-4">
    <skos:related rdf:resource="descriptor/18927-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14040-4">
    <skos:related rdf:resource="descriptor/18926-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14040-4">
    <skos:related rdf:resource="descriptor/15655-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14040-4">
    <skos:related rdf:resource="descriptor/18921-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14040-4">
    <skos:related rdf:resource="descriptor/19781-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14040-4">
    <skos:related rdf:resource="descriptor/19747-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10249-5">
    <skos:narrower rdf:resource="descriptor/10434-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10249-5">
    <skos:narrower rdf:resource="descriptor/11765-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10249-5">
    <skos:narrower rdf:resource="descriptor/10087-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10249-5">
    <skos:narrower rdf:resource="descriptor/19229-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10249-5">
    <skos:narrower rdf:resource="descriptor/10251-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10249-5">
    <skos:narrower rdf:resource="descriptor/10145-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10249-5">
    <skos:narrower rdf:resource="descriptor/10248-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10249-5">
    <skos:narrower rdf:resource="descriptor/10347-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10249-5">
    <skos:narrower rdf:resource="descriptor/10126-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10249-5">
    <skos:narrower rdf:resource="descriptor/10256-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10249-5">
    <skos:narrower rdf:resource="descriptor/10294-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10249-5">
    <skos:narrower rdf:resource="descriptor/10286-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70651">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">P.04.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13170-3">
    <skos:altLabel xml:lang="de">Schlüsselfertigbau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13170-3">
    <skos:altLabel xml:lang="de">Fertigteilbau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13170-3">
    <skos:altLabel xml:lang="de">Fertighausbau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18855-2">
    <skos:related rdf:resource="descriptor/16401-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13762-6">
    <skos:related rdf:resource="descriptor/11487-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18203-3">
    <skos:altLabel xml:lang="en">Charcoal</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18203-3">
    <skos:altLabel xml:lang="de">Holzkohle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18203-3">
    <skos:altLabel xml:lang="de">Holzbrikett</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18203-3">
    <skos:altLabel xml:lang="de">Feuerholz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14173-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20485-1">
    <skos:scopeNote xml:lang="de">Literatur zur deutschen politischen Diskussion über bestimmte Gruppen von rechtlich Selbstständigen, die wirtschaftlich abhängig von anderen sind.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11994-2">
    <skos:altLabel xml:lang="de">Städtischer Wohnungsmarkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11994-2">
    <skos:altLabel xml:lang="de">Städtische Wohnverhältnisse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73396">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71016">
    <skos:narrower rdf:resource="descriptor/18357-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71016">
    <skos:narrower rdf:resource="descriptor/12978-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71016">
    <skos:narrower rdf:resource="descriptor/18356-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71016">
    <skos:narrower rdf:resource="descriptor/19385-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71016">
    <skos:narrower rdf:resource="descriptor/12975-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71016">
    <skos:narrower rdf:resource="descriptor/18354-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71016">
    <skos:narrower rdf:resource="descriptor/18353-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71016">
    <skos:narrower rdf:resource="descriptor/19487-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71016">
    <skos:narrower rdf:resource="descriptor/18351-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71016">
    <skos:narrower rdf:resource="descriptor/11813-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71016">
    <skos:narrower rdf:resource="descriptor/15993-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71016">
    <skos:narrower rdf:resource="descriptor/18174-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71016">
    <skos:narrower rdf:resource="descriptor/18350-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71016">
    <skos:narrower rdf:resource="descriptor/18996-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71016">
    <skos:narrower rdf:resource="descriptor/18349-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71016">
    <skos:narrower rdf:resource="descriptor/18989-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71016">
    <skos:narrower rdf:resource="descriptor/18348-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71016">
    <skos:narrower rdf:resource="descriptor/18347-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71016">
    <skos:narrower rdf:resource="thsys/71015"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71016">
    <skos:narrower rdf:resource="thsys/71014"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71016">
    <skos:narrower rdf:resource="thsys/71013"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71016">
    <skos:narrower rdf:resource="thsys/71012"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13374-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11820-0">
    <skos:prefLabel xml:lang="en">Industrial policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11820-0">
    <skos:prefLabel xml:lang="de">Industriepolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14923-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18828-5">
    <skos:scopeNote xml:lang="de">Nicht für die entwicklungstheoretische Regulation im Sinne der Regulationsschule.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70554">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">G.01.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17706-3">
    <skos:altLabel xml:lang="de">Republik Guinea-Bissau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17706-3">
    <skos:altLabel xml:lang="de">Portugiesisch-Guinea</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12713-3">
    <skos:broader rdf:resource="thsys/70485"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12713-3">
    <skos:broader rdf:resource="thsys/70030"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12713-3">
    <skos:broader rdf:resource="descriptor/12085-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15346-3">
    <skos:related rdf:resource="descriptor/10139-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71018">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.03</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71108">
    <skos:prefLabel xml:lang="en">V.05.06.01  Money market</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71108">
    <skos:prefLabel xml:lang="de">V.05.06.01  Geldmarkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10969-6">
    <skos:related rdf:resource="descriptor/10627-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10969-6">
    <skos:related rdf:resource="descriptor/19327-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18199-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16700-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16008-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10505-6">
    <skos:related rdf:resource="descriptor/18979-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10505-6">
    <skos:related rdf:resource="descriptor/10576-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10505-6">
    <skos:related rdf:resource="descriptor/12945-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10505-6">
    <skos:related rdf:resource="descriptor/10575-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10505-6">
    <skos:related rdf:resource="descriptor/13450-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10505-6">
    <skos:related rdf:resource="descriptor/11809-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10505-6">
    <skos:related rdf:resource="descriptor/11801-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10505-6">
    <skos:related rdf:resource="descriptor/13444-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12056-6">
    <skos:prefLabel xml:lang="en">Group of companies</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12056-6">
    <skos:prefLabel xml:lang="de">Konzern</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71082">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19781-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72393">
    <skos:broader rdf:resource="thsys/72180"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16382-1">
    <skos:broader rdf:resource="thsys/73378"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16382-1">
    <skos:broader rdf:resource="thsys/70853"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16382-1">
    <skos:broader rdf:resource="descriptor/15711-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16382-1">
    <skos:broader rdf:resource="descriptor/10984-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16678-1">
    <skos:related rdf:resource="descriptor/16429-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16678-1">
    <skos:related rdf:resource="descriptor/19757-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16678-1">
    <skos:related rdf:resource="descriptor/18142-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16678-1">
    <skos:related rdf:resource="descriptor/16628-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16678-1">
    <skos:related rdf:resource="descriptor/15920-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13004-6">
    <skos:prefLabel xml:lang="en">Rubber industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13004-6">
    <skos:prefLabel xml:lang="de">Gummiindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id286">
    <rdf:value xml:lang="de">Für Termingeschäfte auf Edelmetalle</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16874-1">
    <skos:broader rdf:resource="thsys/70361"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16874-1">
    <skos:broader rdf:resource="descriptor/18012-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18550-3">
    <skos:prefLabel xml:lang="en">Convenience store</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18550-3">
    <skos:prefLabel xml:lang="de">Nachbarschaftsladen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13440-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19234-1">
    <skos:prefLabel xml:lang="en">Spatial distribution of wealth</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19234-1">
    <skos:prefLabel xml:lang="de">Regionale Vermögensverteilung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19215-5">
    <skos:broader rdf:resource="thsys/70180"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19215-5">
    <skos:broader rdf:resource="descriptor/10144-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10680-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19177-1">
    <skos:altLabel xml:lang="de">G5-Staaten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19177-1">
    <skos:altLabel xml:lang="de">G5-Länder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19177-1">
    <skos:altLabel xml:lang="de">G-5-Länder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14003-3">
    <skos:broader rdf:resource="thsys/70876"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14003-3">
    <skos:broader rdf:resource="thsys/70580"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20522-0">
    <skos:broader rdf:resource="thsys/71071"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20522-0">
    <skos:broader rdf:resource="descriptor/10894-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12796-1">
    <skos:broader rdf:resource="thsys/71090"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12796-1">
    <skos:broader rdf:resource="thsys/71040"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12796-1">
    <skos:broader rdf:resource="descriptor/10220-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70300">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">B.05.02.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13302-5">
    <skos:narrower rdf:resource="descriptor/18503-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14192-2">
    <skos:related rdf:resource="descriptor/14236-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14192-2">
    <skos:related rdf:resource="descriptor/14173-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14192-2">
    <skos:related rdf:resource="descriptor/14368-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14192-2">
    <skos:related rdf:resource="descriptor/12980-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13979-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18387-2">
    <skos:altLabel xml:lang="de">Lagerstättenexploration</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18387-2">
    <skos:altLabel xml:lang="de">Exploration von Bodenschätzen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18387-2">
    <skos:altLabel xml:lang="de">Erkundung von Bodenschätzen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19450-2">
    <skos:narrower rdf:resource="descriptor/11995-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11565-4">
    <skos:related rdf:resource="descriptor/16156-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11565-4">
    <skos:related rdf:resource="descriptor/18912-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11565-4">
    <skos:related rdf:resource="descriptor/19721-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13283-4">
    <skos:narrower rdf:resource="descriptor/13697-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13283-4">
    <skos:narrower rdf:resource="descriptor/13291-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13283-4">
    <skos:narrower rdf:resource="descriptor/13290-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13283-4">
    <skos:narrower rdf:resource="descriptor/13708-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13283-4">
    <skos:narrower rdf:resource="descriptor/13706-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13283-4">
    <skos:narrower rdf:resource="descriptor/18705-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13283-4">
    <skos:narrower rdf:resource="descriptor/13287-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13283-4">
    <skos:narrower rdf:resource="descriptor/18177-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13283-4">
    <skos:narrower rdf:resource="descriptor/10806-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13283-4">
    <skos:narrower rdf:resource="descriptor/13701-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13283-4">
    <skos:narrower rdf:resource="descriptor/24665-0"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id214">
    <rdfs:seeAlso rdf:resource="descriptor/14465-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19036-5">
    <skos:prefLabel xml:lang="en">Reform</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19036-5">
    <skos:prefLabel xml:lang="de">Reform</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16531-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19684-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14078-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10029-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13819-3">
    <skos:broader rdf:resource="thsys/70892"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13819-3">
    <skos:broader rdf:resource="descriptor/18741-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13819-3">
    <skos:broader rdf:resource="descriptor/12041-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14538-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15694-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15176-2">
    <skos:prefLabel xml:lang="en">Random variable</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15176-2">
    <skos:prefLabel xml:lang="de">Zufallsvariable</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10770-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#exactMatch">
    <rdfs:label xml:lang="en">exact match</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18716-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15733-3">
    <skos:related rdf:resource="descriptor/14188-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15733-3">
    <skos:related rdf:resource="descriptor/15734-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15733-3">
    <skos:related rdf:resource="descriptor/15730-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15733-3">
    <skos:related rdf:resource="descriptor/15729-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13158-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13034-4">
    <skos:broader rdf:resource="thsys/71003"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13034-4">
    <skos:broader rdf:resource="descriptor/12993-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13462-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18264-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12841-1">
    <skos:altLabel xml:lang="de">Werbeprospekt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12841-1">
    <skos:altLabel xml:lang="de">Werbegeschenk</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12841-1">
    <skos:altLabel xml:lang="de">Werbebroschüre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12841-1">
    <skos:altLabel xml:lang="de">Werbeartikel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12841-1">
    <skos:altLabel xml:lang="de">Warenprobe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12841-1">
    <skos:altLabel xml:lang="de">Ladenwerbemittel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12841-1">
    <skos:altLabel xml:lang="de">Display-Material</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15860-3">
    <skos:altLabel xml:lang="en">Rational choice</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15860-3">
    <skos:altLabel xml:lang="de">Homo Oeconomicus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11589-4">
    <skos:altLabel xml:lang="en">Governmental accounting</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11589-4">
    <skos:altLabel xml:lang="de">Staatliches Rechnungswesen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11589-4">
    <skos:altLabel xml:lang="de">Öffentliche Rechnungslegung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11589-4">
    <skos:altLabel xml:lang="de">Kameralistische Buchführung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11589-4">
    <skos:altLabel xml:lang="de">Kameralistik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13490-6">
    <skos:altLabel xml:lang="x-other">Hypermarche</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13490-6">
    <skos:altLabel xml:lang="de">Verbrauchermarkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13490-6">
    <skos:altLabel xml:lang="de">Selbstbedienungswarenhaus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13490-6">
    <skos:altLabel xml:lang="de">SB-Warenhaus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13490-6">
    <skos:altLabel xml:lang="de">Hypermarkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12207-4">
    <skos:broader rdf:resource="thsys/71044"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12207-4">
    <skos:broader rdf:resource="thsys/71035"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12207-4">
    <skos:broader rdf:resource="thsys/70269"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12207-4">
    <skos:broader rdf:resource="descriptor/15045-3"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="r1238766808r28278r3">
    <rdf:first rdf:resource="http://www.w3.org/2004/02/skos/core#Collection"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20018-4">
    <skos:broader rdf:resource="thsys/70183"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20018-4">
    <skos:broader rdf:resource="descriptor/25742-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16515-1">
    <skos:narrower rdf:resource="descriptor/19707-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16515-1">
    <skos:narrower rdf:resource="descriptor/16516-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16515-1">
    <skos:narrower rdf:resource="descriptor/19706-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16515-1">
    <skos:narrower rdf:resource="descriptor/16514-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16515-1">
    <skos:narrower rdf:resource="descriptor/18945-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15661-2">
    <skos:prefLabel xml:lang="en">Music</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15661-2">
    <skos:prefLabel xml:lang="de">Musik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10194-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12946-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10303-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19322-4">
    <skos:related rdf:resource="descriptor/10704-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18901-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13309-5">
    <skos:related rdf:resource="descriptor/12592-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14118-0">
    <skos:altLabel xml:lang="en">Flower</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14118-0">
    <skos:altLabel xml:lang="de">Topfpflanze</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14118-0">
    <skos:altLabel xml:lang="de">Schnittblume</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14118-0">
    <skos:altLabel xml:lang="de">Blume</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18316-4">
    <skos:related rdf:resource="descriptor/19307-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18316-4">
    <skos:related rdf:resource="descriptor/14084-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18316-4">
    <skos:related rdf:resource="descriptor/19306-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18316-4">
    <skos:related rdf:resource="descriptor/13165-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13702-3">
    <skos:altLabel xml:lang="de">Agrarkreditinstitut</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13702-3">
    <skos:altLabel xml:lang="de">Agrarbank</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15675-5">
    <skos:related rdf:resource="descriptor/18956-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15675-5">
    <skos:related rdf:resource="descriptor/16483-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15675-5">
    <skos:related rdf:resource="descriptor/18955-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15675-5">
    <skos:related rdf:resource="descriptor/13363-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15675-5">
    <skos:related rdf:resource="descriptor/13097-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15675-5">
    <skos:related rdf:resource="descriptor/13235-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15675-5">
    <skos:related rdf:resource="descriptor/14586-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15675-5">
    <skos:related rdf:resource="descriptor/13362-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15675-5">
    <skos:related rdf:resource="descriptor/15654-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15675-5">
    <skos:related rdf:resource="descriptor/12962-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15675-5">
    <skos:related rdf:resource="descriptor/13354-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15675-5">
    <skos:related rdf:resource="descriptor/13320-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14837-3">
    <skos:prefLabel xml:lang="en">Fleece</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14837-3">
    <skos:prefLabel xml:lang="de">Vliesstoff</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18644-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12262-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13715-1">
    <skos:broader rdf:resource="thsys/70893"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13715-1">
    <skos:broader rdf:resource="thsys/70187"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13715-1">
    <skos:broader rdf:resource="descriptor/10213-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18543-0">
    <skos:prefLabel xml:lang="en">Global advertising</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18543-0">
    <skos:prefLabel xml:lang="de">Internationale Werbung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10120-5">
    <skos:prefLabel xml:lang="en">Capital</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10120-5">
    <skos:prefLabel xml:lang="de">Kapital</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10213-5">
    <skos:prefLabel xml:lang="en">Price</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10213-5">
    <skos:prefLabel xml:lang="de">Preis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13431-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10687-2">
    <skos:broader rdf:resource="thsys/71098"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10687-2">
    <skos:broader rdf:resource="descriptor/10684-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11302-1">
    <skos:altLabel xml:lang="en">Senior staff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11302-1">
    <skos:altLabel xml:lang="en">Corporate executives</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11302-1">
    <skos:altLabel xml:lang="en">Chief Executive Officers</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11302-1">
    <skos:altLabel xml:lang="en">CEO (Chief Executive Officers)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11302-1">
    <skos:altLabel xml:lang="de">Manager</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11302-1">
    <skos:altLabel xml:lang="de">Leitende Angestellte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11302-1">
    <skos:altLabel xml:lang="de">Geschäftsführer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11302-1">
    <skos:altLabel xml:lang="de">Generaldirektoren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11302-1">
    <skos:altLabel xml:lang="de">Direktoren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11302-1">
    <skos:altLabel xml:lang="de">Betriebsleiter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70199">
    <skos:narrower rdf:resource="descriptor/14731-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70199">
    <skos:narrower rdf:resource="descriptor/14730-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70199">
    <skos:narrower rdf:resource="descriptor/18258-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70199">
    <skos:narrower rdf:resource="descriptor/14729-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70199">
    <skos:narrower rdf:resource="descriptor/14728-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70199">
    <skos:narrower rdf:resource="descriptor/14725-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70199">
    <skos:narrower rdf:resource="descriptor/14722-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16513-5">
    <skos:prefLabel xml:lang="en">Racial policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16513-5">
    <skos:prefLabel xml:lang="de">Rassenpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13077-0">
    <skos:broader rdf:resource="thsys/70968"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13077-0">
    <skos:broader rdf:resource="thsys/70964"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13077-0">
    <skos:broader rdf:resource="descriptor/13181-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/g">
    <skos:prefLabel xml:lang="en">G  Geographic names</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/g">
    <skos:prefLabel xml:lang="de">G  Geographische Begriffe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12489-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10102-0">
    <skos:related rdf:resource="descriptor/11039-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10102-0">
    <skos:related rdf:resource="descriptor/10227-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10102-0">
    <skos:related rdf:resource="descriptor/10103-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10102-0">
    <skos:related rdf:resource="descriptor/10099-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10102-0">
    <skos:related rdf:resource="descriptor/10097-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10102-0">
    <skos:related rdf:resource="descriptor/10072-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10102-0">
    <skos:related rdf:resource="descriptor/19208-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11415-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14158-2">
    <skos:prefLabel xml:lang="en">Fish</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14158-2">
    <skos:prefLabel xml:lang="de">Fisch</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10060-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19167-4">
    <skos:altLabel xml:lang="de">Südasiatische Staaten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19167-4">
    <skos:altLabel xml:lang="de">Südasiatische Länder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14176-0">
    <skos:altLabel xml:lang="de">Sekundärenergieträger</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70549">
    <skos:narrower rdf:resource="descriptor/14052-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70549">
    <skos:narrower rdf:resource="descriptor/14408-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70549">
    <skos:narrower rdf:resource="descriptor/14345-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70549">
    <skos:narrower rdf:resource="descriptor/21110-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70549">
    <skos:narrower rdf:resource="descriptor/12737-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70549">
    <skos:narrower rdf:resource="descriptor/14341-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70549">
    <skos:narrower rdf:resource="descriptor/12705-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70549">
    <skos:narrower rdf:resource="descriptor/18245-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70549">
    <skos:narrower rdf:resource="descriptor/14339-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70549">
    <skos:narrower rdf:resource="descriptor/14336-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70549">
    <skos:narrower rdf:resource="descriptor/12681-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70549">
    <skos:narrower rdf:resource="descriptor/14397-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70549">
    <skos:narrower rdf:resource="descriptor/14337-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70549">
    <skos:narrower rdf:resource="descriptor/12736-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70549">
    <skos:narrower rdf:resource="descriptor/14334-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70549">
    <skos:narrower rdf:resource="descriptor/12693-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70549">
    <skos:narrower rdf:resource="descriptor/14396-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70549">
    <skos:narrower rdf:resource="descriptor/14331-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70549">
    <skos:narrower rdf:resource="descriptor/14335-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70549">
    <skos:narrower rdf:resource="descriptor/14329-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70549">
    <skos:narrower rdf:resource="thsys/70424"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70549">
    <skos:narrower rdf:resource="thsys/70318"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11783-1">
    <skos:altLabel xml:lang="de">Unternehmensgrößenstruktur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11783-1">
    <skos:altLabel xml:lang="de">Betriebsgrößenverteilung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10817-1">
    <skos:related rdf:resource="descriptor/10831-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10817-1">
    <skos:related rdf:resource="descriptor/10830-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10817-1">
    <skos:related rdf:resource="descriptor/10806-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10817-1">
    <skos:related rdf:resource="descriptor/10808-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10817-1">
    <skos:related rdf:resource="descriptor/10827-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10753-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14819-5">
    <skos:prefLabel xml:lang="en">plastic film</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14819-5">
    <skos:prefLabel xml:lang="de">Kunststofffolie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24703-4">
    <skos:broader rdf:resource="thsys/73337"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24703-4">
    <skos:broader rdf:resource="thsys/70397"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24703-4">
    <skos:broader rdf:resource="descriptor/12849-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14603-0">
    <skos:broader rdf:resource="thsys/70538"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10659-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70481">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14955-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11790-4">
    <zbwext:useInsteadNote rdf:nodeID="id322"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11790-4">
    <zbwext:useInsteadNote rdf:nodeID="id321"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14258-5">
    <skos:prefLabel xml:lang="en">Cement</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14258-5">
    <skos:prefLabel xml:lang="de">Zement</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70227">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12881-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19485-4">
    <skos:prefLabel xml:lang="en">Fire</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19485-4">
    <skos:prefLabel xml:lang="de">Brand</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14154-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12697-3">
    <skos:broader rdf:resource="thsys/71110"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19020-6">
    <skos:altLabel xml:lang="en">Animal manure</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19020-6">
    <skos:altLabel xml:lang="de">Kompost</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19020-6">
    <skos:altLabel xml:lang="de">Gülle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19020-6">
    <skos:altLabel xml:lang="de">Guano</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19020-6">
    <skos:altLabel xml:lang="de">Dung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18275-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18970-2">
    <skos:prefLabel xml:lang="en">Mothers</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18970-2">
    <skos:prefLabel xml:lang="de">Mütter</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15647-3">
    <skos:broader rdf:resource="thsys/73388"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15647-3">
    <skos:broader rdf:resource="descriptor/15671-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12085-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19759-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19399-4">
    <skos:altLabel xml:lang="de">Staatliche Kreditvergabe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19399-4">
    <skos:altLabel xml:lang="de">Öffentliche Darlehensvergabe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11390-6">
    <skos:broader rdf:resource="thsys/70843"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11390-6">
    <skos:broader rdf:resource="descriptor/11361-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11390-6">
    <skos:broader rdf:resource="descriptor/11184-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25621-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19254-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19453-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19246-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19034-2">
    <skos:broader rdf:resource="thsys/73345"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19034-2">
    <skos:broader rdf:resource="thsys/70030"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10294-0">
    <zbwext:useInsteadNote rdf:nodeID="id113"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19680-6">
    <skos:scopeNote xml:lang="de">Ansätze der politischen Philosophie, die sich als Alternative zum Neoliberalismus verstehen und die Bedeutung der Gemeinschaft hervorheben</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10667-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11414-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14638-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id322">
    <rdfs:seeAlso rdf:resource="descriptor/18478-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12761-6">
    <skos:narrower rdf:resource="descriptor/15520-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12761-6">
    <skos:narrower rdf:resource="descriptor/15518-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12761-6">
    <skos:narrower rdf:resource="descriptor/12563-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18940-4">
    <skos:narrower rdf:resource="descriptor/18768-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18737-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14573-4">
    <skos:related rdf:resource="descriptor/14522-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14573-4">
    <skos:related rdf:resource="descriptor/18448-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14573-4">
    <skos:related rdf:resource="descriptor/15661-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14573-4">
    <skos:related rdf:resource="descriptor/14029-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17583-0">
    <skos:altLabel xml:lang="de">Laotische Demokratische Volksrepublik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17583-0">
    <skos:altLabel xml:lang="de">Demokratische Volksrepublik Laos</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19608-0">
    <skos:altLabel xml:lang="de">Sprachwissenschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19608-0">
    <skos:altLabel xml:lang="de">Sprachforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19608-0">
    <skos:altLabel xml:lang="de">Grammatik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19608-0">
    <skos:altLabel xml:lang="de">Etymologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11956-3">
    <skos:altLabel xml:lang="en">Theory of social gravity</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11956-3">
    <skos:altLabel xml:lang="en">Theory of social gravitation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11956-3">
    <skos:altLabel xml:lang="en">Gravity equation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18104-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12165-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18754-1">
    <skos:related rdf:resource="descriptor/13861-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18754-1">
    <skos:related rdf:resource="descriptor/13302-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10215-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17781-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10142-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16335-3">
    <skos:prefLabel xml:lang="en">Parliamentary control</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16335-3">
    <skos:prefLabel xml:lang="de">Verwaltungskontrolle</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13308-0">
    <skos:prefLabel xml:lang="en">Hotel industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13308-0">
    <skos:prefLabel xml:lang="de">Hotelgewerbe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73364">
    <skos:prefLabel xml:lang="de">N.04.04.02  Internationale Konflikte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11184-2">
    <skos:narrower rdf:resource="descriptor/11390-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11184-2">
    <skos:narrower rdf:resource="descriptor/19492-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11184-2">
    <skos:narrower rdf:resource="descriptor/11191-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11184-2">
    <skos:narrower rdf:resource="descriptor/10613-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11184-2">
    <skos:narrower rdf:resource="descriptor/11389-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18063-0">
    <skos:scopeNote xml:lang="de">Vgl. auch Gesetzliche Krankenversicherung, Gesundheitskosten und Gesundheitsfinanzierung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17790-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70835">
    <skos:broader rdf:resource="thsys/w"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71083">
    <rdfs:label xml:lang="en">Economics of public finance</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71083">
    <rdfs:label xml:lang="de">Finanzwissenschaft</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19266-2">
    <skos:prefLabel xml:lang="en">Interest rate differential</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19266-2">
    <skos:prefLabel xml:lang="de">Zinsdifferenz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70878">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.20.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id218">
    <rdf:value xml:lang="de">Für die Haltung von Milchvieh</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11547-6">
    <skos:prefLabel xml:lang="en">Tax</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11547-6">
    <skos:prefLabel xml:lang="de">Steuer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16630-1">
    <skos:broader rdf:resource="thsys/73378"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16630-1">
    <skos:broader rdf:resource="thsys/70853"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16630-1">
    <skos:broader rdf:resource="descriptor/10984-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19190-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11616-6">
    <skos:altLabel xml:lang="de">Verwaltungsvermögen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11616-6">
    <skos:altLabel xml:lang="de">Staatsvermögen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11616-6">
    <skos:altLabel xml:lang="de">Sondervermögen des Bundes</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11616-6">
    <skos:altLabel xml:lang="de">Öffentliches Finanzvermögen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11616-6">
    <skos:altLabel xml:lang="de">Finanzvermögen des Staates</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13020-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70648">
    <rdfs:label xml:lang="en">Water</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70648">
    <rdfs:label xml:lang="de">Wasser</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12620-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13457-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17176-6">
    <skos:broader rdf:resource="thsys/70130"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17176-6">
    <skos:broader rdf:resource="descriptor/17179-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12341-0">
    <skos:broader rdf:resource="thsys/71043"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12341-0">
    <skos:broader rdf:resource="descriptor/12325-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16789-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11599-1">
    <skos:related rdf:resource="descriptor/16552-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11599-1">
    <skos:related rdf:resource="descriptor/11711-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11599-1">
    <skos:related rdf:resource="descriptor/11600-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20497-1">
    <skos:altLabel xml:lang="de">Multikulturelles Management</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10433-5">
    <skos:altLabel xml:lang="de">Vintage-Wachstumstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10433-5">
    <skos:altLabel xml:lang="de">Kapitaljahrgang-Wachstumsmodell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10433-5">
    <skos:altLabel xml:lang="de">Jahrgangsmodell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14846-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10464-1">
    <skos:related rdf:resource="descriptor/10466-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10464-1">
    <skos:related rdf:resource="descriptor/10459-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10464-1">
    <skos:related rdf:resource="descriptor/10437-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10464-1">
    <skos:related rdf:resource="descriptor/12711-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12039-6">
    <skos:scopeNote xml:lang="de">Nur für betriebswirtschaftliche Statistik: Sammlung und Analyse von betriebsintern und -extern anfallendem Zahlenmaterial zu betriebseigenen Zwecken.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17584-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12296-0">
    <skos:prefLabel xml:lang="en">Reserve (Balance sheet)</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12296-0">
    <skos:prefLabel xml:lang="de">Rücklage</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18908-0">
    <skos:prefLabel xml:lang="en">Credit counselling</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18908-0">
    <skos:prefLabel xml:lang="de">Schuldnerberatung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18630-5">
    <skos:narrower rdf:resource="descriptor/13562-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14771-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11723-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10478-4">
    <skos:prefLabel xml:lang="en">Patent</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10478-4">
    <skos:prefLabel xml:lang="de">Patent</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18700-3">
    <skos:scopeNote xml:lang="de">Bilanzielle Behandlung von Fremdwährungsgeschäften und Umrechnung von Fremdwährungsabschlüssen ausländischer Tochterunternehmen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18424-1">
    <skos:related rdf:resource="descriptor/13084-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18424-1">
    <skos:related rdf:resource="descriptor/18103-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18424-1">
    <skos:related rdf:resource="descriptor/18425-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18424-1">
    <skos:related rdf:resource="descriptor/19396-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12719-5">
    <skos:altLabel xml:lang="en">Assembly-line production</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12719-5">
    <skos:altLabel xml:lang="de">Montagelinie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12719-5">
    <skos:altLabel xml:lang="de">Montageband</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12719-5">
    <skos:altLabel xml:lang="de">Linienfertigung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12719-5">
    <skos:altLabel xml:lang="de">Fordismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12719-5">
    <skos:altLabel xml:lang="de">Fließproduktionssystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12719-5">
    <skos:altLabel xml:lang="de">Fließproduktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12719-5">
    <skos:altLabel xml:lang="de">Fließbandproduktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12719-5">
    <skos:altLabel xml:lang="de">Fließbandfertigung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12719-5">
    <skos:altLabel xml:lang="de">Fließbandarbeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12719-5">
    <skos:altLabel xml:lang="de">Fließarbeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12719-5">
    <skos:altLabel xml:lang="de">Fertigungsstraße</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10822-1">
    <skos:altLabel xml:lang="de">Produktionsverlagerung ins Ausland</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10822-1">
    <skos:altLabel xml:lang="de">Produktion im Ausland</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10822-1">
    <skos:altLabel xml:lang="de">Fertigung im Ausland</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70199">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">P.12.04</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13692-1">
    <zbwext:useInsteadNote rdf:nodeID="id42"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15635-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11596-0">
    <skos:related rdf:resource="descriptor/19436-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11596-0">
    <skos:related rdf:resource="descriptor/11589-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11596-0">
    <skos:related rdf:resource="descriptor/11595-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11596-0">
    <skos:related rdf:resource="descriptor/11545-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11596-0">
    <skos:related rdf:resource="descriptor/11554-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11596-0">
    <skos:related rdf:resource="descriptor/16100-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11596-0">
    <skos:related rdf:resource="descriptor/11731-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11596-0">
    <skos:related rdf:resource="descriptor/19380-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11596-0">
    <skos:related rdf:resource="descriptor/12309-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#narrowerTransitive">
    <rdfs:label xml:lang="en">narrower transitive</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19675-6">
    <skos:related rdf:resource="descriptor/14728-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19675-6">
    <skos:related rdf:resource="descriptor/13002-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19675-6">
    <skos:related rdf:resource="descriptor/13193-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13694-4">
    <skos:altLabel xml:lang="en">Deposit protection</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13694-4">
    <skos:altLabel xml:lang="de">Einlagensicherungsfonds</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13694-4">
    <skos:altLabel xml:lang="de">Depositenversicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13694-4">
    <skos:altLabel xml:lang="de">Bankeinlagensicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16541-0">
    <skos:related rdf:resource="descriptor/16670-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16541-0">
    <skos:related rdf:resource="descriptor/13371-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16541-0">
    <skos:related rdf:resource="descriptor/16683-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18763-0">
    <skos:broader rdf:resource="thsys/70880"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18763-0">
    <skos:broader rdf:resource="descriptor/18655-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18763-0">
    <skos:broader rdf:resource="descriptor/18762-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20722-6">
    <skos:related rdf:resource="descriptor/15772-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20722-6">
    <skos:related rdf:resource="descriptor/15770-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20722-6">
    <skos:related rdf:resource="descriptor/18325-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16927-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19132-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14849-3">
    <skos:narrower rdf:resource="descriptor/14851-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14849-3">
    <skos:narrower rdf:resource="descriptor/14777-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14849-3">
    <skos:narrower rdf:resource="descriptor/14854-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17778-6">
    <skos:broader rdf:resource="thsys/70086"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17778-6">
    <skos:broader rdf:resource="descriptor/17791-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15993-5">
    <skos:related rdf:resource="descriptor/15769-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15993-5">
    <skos:related rdf:resource="descriptor/12975-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15993-5">
    <skos:related rdf:resource="descriptor/11810-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15993-5">
    <skos:related rdf:resource="descriptor/14175-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15993-5">
    <skos:related rdf:resource="descriptor/18352-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17772-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14066-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14777-2">
    <skos:related rdf:resource="descriptor/13679-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18994-2">
    <skos:broader rdf:resource="thsys/73391"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18994-2">
    <skos:broader rdf:resource="thsys/70892"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18994-2">
    <skos:broader rdf:resource="descriptor/13825-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70936">
    <skos:broader rdf:resource="thsys/70942"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14060-5">
    <skos:prefLabel xml:lang="en">Spice</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14060-5">
    <skos:prefLabel xml:lang="de">Gewürz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12987-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18398-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12761-6">
    <skos:prefLabel xml:lang="en">Scheduling problem</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12761-6">
    <skos:prefLabel xml:lang="de">Scheduling-Verfahren</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18132-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18980-6">
    <skos:narrower rdf:resource="descriptor/13434-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18980-6">
    <skos:narrower rdf:resource="descriptor/11805-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10624-5">
    <skos:prefLabel xml:lang="en">World economy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10624-5">
    <skos:prefLabel xml:lang="de">Weltwirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19592-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10179-0">
    <skos:related rdf:resource="descriptor/10156-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10179-0">
    <skos:related rdf:resource="descriptor/10106-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10179-0">
    <skos:related rdf:resource="descriptor/11527-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10179-0">
    <skos:related rdf:resource="descriptor/18156-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10179-0">
    <skos:related rdf:resource="descriptor/10077-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10179-0">
    <skos:related rdf:resource="descriptor/11739-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10179-0">
    <skos:related rdf:resource="descriptor/19209-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10179-0">
    <skos:related rdf:resource="descriptor/19376-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10179-0">
    <skos:related rdf:resource="descriptor/16699-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18032-4">
    <skos:broader rdf:resource="thsys/71034"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20522-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15604-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18685-1">
    <skos:scopeNote xml:lang="de">Anleihen, die mit Forderungen unterlegt sind.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18230-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11134-3">
    <skos:prefLabel xml:lang="en">Combine (Socialist countries)</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11134-3">
    <skos:prefLabel xml:lang="de">Kombinat</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19184-4">
    <skos:narrower rdf:resource="descriptor/19390-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19184-4">
    <skos:narrower rdf:resource="descriptor/19185-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14216-0">
    <skos:narrower rdf:resource="descriptor/14214-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14216-0">
    <skos:narrower rdf:resource="descriptor/18009-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14216-0">
    <skos:narrower rdf:resource="descriptor/14212-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12023-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15005-1">
    <skos:prefLabel xml:lang="en">Armaments</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15005-1">
    <skos:prefLabel xml:lang="de">Rüstungsgut</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11192-3">
    <skos:related rdf:resource="descriptor/11215-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11192-3">
    <skos:related rdf:resource="descriptor/15839-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11192-3">
    <skos:related rdf:resource="descriptor/16670-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11192-3">
    <skos:related rdf:resource="descriptor/18911-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11192-3">
    <skos:related rdf:resource="descriptor/16531-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11192-3">
    <skos:related rdf:resource="descriptor/12577-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11192-3">
    <skos:related rdf:resource="descriptor/15837-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13761-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11928-1">
    <skos:broader rdf:resource="thsys/71070"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11928-1">
    <skos:broader rdf:resource="descriptor/11912-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19144-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15755-0">
    <skos:related rdf:resource="descriptor/19622-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15755-0">
    <skos:related rdf:resource="descriptor/15779-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15755-0">
    <skos:related rdf:resource="descriptor/19016-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15755-0">
    <skos:related rdf:resource="descriptor/14651-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15755-0">
    <skos:related rdf:resource="descriptor/15743-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15755-0">
    <skos:related rdf:resource="descriptor/12016-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19681-4">
    <zbwext:useInsteadNote rdf:nodeID="id103"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10197-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16092-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12730-3">
    <skos:narrower rdf:resource="descriptor/12735-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12730-3">
    <skos:narrower rdf:resource="descriptor/18002-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12730-3">
    <skos:narrower rdf:resource="descriptor/12732-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18199-3">
    <skos:broader rdf:resource="thsys/70341"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18199-3">
    <skos:broader rdf:resource="descriptor/14073-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13460-1">
    <skos:narrower rdf:resource="descriptor/13459-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15515-6">
    <skos:broader rdf:resource="thsys/71044"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15515-6">
    <skos:broader rdf:resource="thsys/71035"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15515-6">
    <skos:broader rdf:resource="descriptor/15483-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14336-4">
    <skos:prefLabel xml:lang="en">Machine product</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14336-4">
    <skos:prefLabel xml:lang="de">Maschinenbauprodukt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71071">
    <skos:prefLabel xml:lang="en">V.10.03  Competition policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71071">
    <skos:prefLabel xml:lang="de">V.10.03  Wettbewerbspolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15861-1">
    <zbwext:useInsteadNote rdf:nodeID="id349"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18924-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15976-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id33">
    <rdf:value xml:lang="de">Für internationale Regelungen und Abkommen zum Steuerrecht</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15025-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14513-1">
    <skos:broader rdf:resource="thsys/70373"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14513-1">
    <skos:broader rdf:resource="descriptor/14522-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18255-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19626-5">
    <skos:related rdf:resource="descriptor/18343-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19626-5">
    <skos:related rdf:resource="descriptor/18312-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19626-5">
    <skos:related rdf:resource="descriptor/11807-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19626-5">
    <skos:related rdf:resource="descriptor/12964-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19626-5">
    <skos:related rdf:resource="descriptor/14158-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10533-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12160-4">
    <skos:prefLabel xml:lang="en">Administrative department</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12160-4">
    <skos:prefLabel xml:lang="de">Unternehmensverwaltung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24664-2">
    <skos:scopeNote xml:lang="de">Unter digitalen Gütern versteht man immaterielle Mittel zur Bedürfnisbefriedigung, die sich mit Hilfe von Informationssystemen entwickeln, vertreiben oder anwenden lassen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12750-4">
    <skos:altLabel xml:lang="en">Equipment layout</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12750-4">
    <skos:altLabel xml:lang="de">Layout-Fabrikplanung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19589-6">
    <skos:prefLabel xml:lang="en">Wealth statistics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19589-6">
    <skos:prefLabel xml:lang="de">Vermögensstatistik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13343-5">
    <skos:narrower rdf:resource="descriptor/18780-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13343-5">
    <skos:narrower rdf:resource="descriptor/18781-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11822-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18398-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11641-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12408-1">
    <skos:broader rdf:resource="thsys/71043"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12408-1">
    <skos:broader rdf:resource="descriptor/19142-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14971-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14816-4">
    <skos:prefLabel xml:lang="en">Reprography</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14816-4">
    <skos:prefLabel xml:lang="de">Reprographie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13331-5">
    <skos:broader rdf:resource="thsys/70817"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13331-5">
    <skos:broader rdf:resource="descriptor/13327-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10960-3">
    <zbwext:useInsteadNote rdf:nodeID="id259"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16250-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13389-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70901">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18632-1">
    <skos:prefLabel xml:lang="en">Regional transport policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18632-1">
    <skos:prefLabel xml:lang="de">Regionale Verkehrspolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15330-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10780-5">
    <skos:related rdf:resource="descriptor/10779-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10780-5">
    <skos:related rdf:resource="descriptor/13757-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10780-5">
    <skos:related rdf:resource="descriptor/10778-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10780-5">
    <skos:related rdf:resource="descriptor/10776-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13989-6">
    <skos:related rdf:resource="descriptor/16333-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13989-6">
    <skos:related rdf:resource="descriptor/16330-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13989-6">
    <skos:related rdf:resource="descriptor/16329-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13989-6">
    <skos:related rdf:resource="descriptor/13397-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13989-6">
    <skos:related rdf:resource="descriptor/16273-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13989-6">
    <skos:related rdf:resource="descriptor/13988-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18291-1">
    <skos:prefLabel xml:lang="en">Microfibre</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18291-1">
    <skos:prefLabel xml:lang="de">Mikrofaser</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16269-4">
    <skos:broader rdf:resource="thsys/73376"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13124-3">
    <skos:related rdf:resource="descriptor/13369-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13124-3">
    <skos:related rdf:resource="descriptor/13670-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16034-3">
    <skos:broader rdf:resource="thsys/73392"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16034-3">
    <skos:broader rdf:resource="thsys/70550"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16034-3">
    <skos:broader rdf:resource="descriptor/18067-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12366-5">
    <skos:related rdf:resource="descriptor/12368-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12366-5">
    <skos:related rdf:resource="descriptor/12418-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13155-6">
    <skos:broader rdf:resource="thsys/70986"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13155-6">
    <skos:broader rdf:resource="descriptor/13160-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19572-2">
    <skos:altLabel xml:lang="de">Random-Walk-Theorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14483-5">
    <skos:scopeNote xml:lang="de">Nur für elektrische Kabel und Drähte.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18475-5">
    <skos:altLabel xml:lang="en">Captive insurance company</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18475-5">
    <skos:altLabel xml:lang="de">Industrie-Selbstversicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18475-5">
    <skos:altLabel xml:lang="de">Industrieeigene Versicherungsgesellschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18475-5">
    <skos:altLabel xml:lang="de">Firmeneigene Versicherungsgesellschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18475-5">
    <skos:altLabel xml:lang="de">Captive-Versicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15892-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18829-3">
    <skos:prefLabel xml:lang="en">Civil service law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18829-3">
    <skos:prefLabel xml:lang="de">Öffentliches Dienstrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14472-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12021-4">
    <skos:related rdf:resource="descriptor/12024-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12021-4">
    <skos:related rdf:resource="descriptor/18327-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12021-4">
    <skos:related rdf:resource="descriptor/12016-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12021-4">
    <skos:related rdf:resource="descriptor/12015-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12021-4">
    <skos:related rdf:resource="descriptor/18336-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14052-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10506-4">
    <skos:broader rdf:resource="thsys/70307"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10506-4">
    <skos:broader rdf:resource="thsys/70091"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10506-4">
    <skos:broader rdf:resource="descriptor/10492-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10455-2">
    <skos:scopeNote xml:lang="de">Für die mikroelektronische Industrie, elektronische Produkte oder den Einsatz von Datenverarbeitung BENUTZE die entsprechenden Deskriptoren.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10455-2">
    <skos:scopeNote xml:lang="de">Nur verwenden für Forschung und Entwicklung der Mikroelektronik selbst.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16267-1">
    <skos:altLabel xml:lang="de">Guerillakrieg</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19338-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18602-3">
    <skos:altLabel xml:lang="de">Hafenordnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16156-3">
    <skos:altLabel xml:lang="de">Opferentschädigungsgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16156-3">
    <skos:altLabel xml:lang="de">Lastenausgleichsgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16156-3">
    <skos:altLabel xml:lang="de">Kriegsopferversorgungsgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16156-3">
    <skos:altLabel xml:lang="de">Häftlingshilfegesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12768-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12134-5">
    <skos:altLabel xml:lang="en">Process organization</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12134-5">
    <skos:altLabel xml:lang="en">Process organisation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12134-5">
    <skos:altLabel xml:lang="de">Prozessorganisation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12134-5">
    <skos:altLabel xml:lang="de">Prozeßorganisation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12134-5">
    <skos:altLabel xml:lang="de">Organisatorische Prozeßstrukturierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12134-5">
    <skos:altLabel xml:lang="de">Organisatorische Prozess-Strukturierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12134-5">
    <skos:altLabel xml:lang="de">Arbeitsablauforganisation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12134-5">
    <skos:altLabel xml:lang="de">Ablaufstrukturierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id264">
    <rdf:value xml:lang="de">Für die Produktion in volkswirtschaftlicher Sicht</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18938-5">
    <skos:prefLabel xml:lang="en">Pay television</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18938-5">
    <skos:prefLabel xml:lang="de">Private Fernsehanstalt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10102-0">
    <skos:altLabel xml:lang="en">Pareto improvement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10102-0">
    <skos:altLabel xml:lang="en">Pareto efficiency</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10102-0">
    <skos:altLabel xml:lang="en">Pareto criterion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10102-0">
    <skos:altLabel xml:lang="en">Pareto conditions</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10102-0">
    <skos:altLabel xml:lang="de">Pareto-Effizienz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10102-0">
    <skos:altLabel xml:lang="de">Kompensationskriterium</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71072">
    <skos:prefLabel xml:lang="de">V.09.09  Öffentliche Wirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12491-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19560-2">
    <zbwext:useInsteadNote rdf:nodeID="id294"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11267-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70201">
    <rdfs:label xml:lang="de">Produkte</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10049-6">
    <skos:prefLabel xml:lang="en">Multisectoral model</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10049-6">
    <skos:prefLabel xml:lang="de">Mehr-Sektoren-Modell</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10752-3">
    <skos:broader rdf:resource="thsys/71095"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10752-3">
    <skos:broader rdf:resource="descriptor/10764-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11825-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18522-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18539-5">
    <skos:prefLabel xml:lang="en">Foreign exchange management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18539-5">
    <skos:prefLabel xml:lang="de">Währungsmanagement</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70174">
    <skos:broader rdf:resource="thsys/70002"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id327">
    <rdfs:seeAlso rdf:resource="descriptor/18649-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70929">
    <skos:narrower rdf:resource="descriptor/13573-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70929">
    <skos:narrower rdf:resource="descriptor/13571-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70929">
    <skos:narrower rdf:resource="descriptor/18623-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70929">
    <skos:narrower rdf:resource="descriptor/13511-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19104-0">
    <skos:altLabel xml:lang="de">Controlling-Software</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19104-0">
    <skos:altLabel xml:lang="de">Controlling Informationssystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19104-0">
    <skos:altLabel xml:lang="de">Computer-integriertes Controlling</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19104-0">
    <skos:altLabel xml:lang="de">CIS (Controlling-Informationssystem)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19104-0">
    <skos:altLabel xml:lang="de">CIC (Computer-integriertes Controlling)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18553-4">
    <skos:prefLabel xml:lang="en">Category management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18553-4">
    <skos:prefLabel xml:lang="de">Category Management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13510-5">
    <skos:broader rdf:resource="thsys/70943"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18604-6">
    <skos:broader rdf:resource="thsys/70932"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18604-6">
    <skos:broader rdf:resource="descriptor/13264-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10521-1">
    <skos:broader rdf:resource="thsys/71110"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10521-1">
    <skos:broader rdf:resource="thsys/71086"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10521-1">
    <skos:broader rdf:resource="thsys/70575"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10521-1">
    <skos:broader rdf:resource="descriptor/10525-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10523-4">
    <skos:related rdf:resource="descriptor/10595-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10523-4">
    <skos:related rdf:resource="descriptor/11900-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10523-4">
    <skos:related rdf:resource="descriptor/10548-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11288-4">
    <skos:broader rdf:resource="thsys/71058"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11288-4">
    <skos:broader rdf:resource="thsys/70843"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11288-4">
    <skos:broader rdf:resource="thsys/70244"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11288-4">
    <skos:broader rdf:resource="descriptor/11275-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11288-4">
    <skos:broader rdf:resource="descriptor/11294-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18721-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18616-6">
    <skos:related rdf:resource="descriptor/18615-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18616-6">
    <skos:related rdf:resource="descriptor/13621-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18616-6">
    <skos:related rdf:resource="descriptor/13601-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18616-6">
    <skos:related rdf:resource="descriptor/13565-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10220-1">
    <skos:prefLabel xml:lang="en">Shadow price</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10220-1">
    <skos:prefLabel xml:lang="de">Schattenpreis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70040">
    <skos:prefLabel xml:lang="de">B.07.01  Konditionen- und Preispolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70318">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">P.08.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18820-0">
    <skos:related rdf:resource="descriptor/12613-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18820-0">
    <skos:related rdf:resource="descriptor/19485-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18537-2">
    <skos:prefLabel xml:lang="en">Hansa</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18537-2">
    <skos:prefLabel xml:lang="de">Hanse</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15704-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15627-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13340-4">
    <skos:scopeNote xml:lang="de">Alle Arten von Städtereinigungsmaßnahmen, z.B. auch Leeren von Abfallkörben und Senkgruben, Eisentfernung im Winter, Winterstreudienst</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18879-2">
    <skos:altLabel xml:lang="en">Tuition fee</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18879-2">
    <skos:altLabel xml:lang="de">Studienkosten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18879-2">
    <skos:altLabel xml:lang="de">Studiengebühr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18879-2">
    <skos:altLabel xml:lang="de">Stipendium</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18879-2">
    <skos:altLabel xml:lang="de">Finanzielle Studienförderung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18650-6">
    <skos:broader rdf:resource="thsys/70922"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18650-6">
    <skos:broader rdf:resource="descriptor/18649-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10427-0">
    <skos:altLabel xml:lang="de">Golden-Age-Wachstum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10427-0">
    <skos:altLabel xml:lang="de">Dynamisches Wachstumsgleichgewicht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11083-1">
    <skos:prefLabel xml:lang="en">Imperialism</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11083-1">
    <skos:prefLabel xml:lang="de">Imperialismus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10114-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12823-3">
    <skos:altLabel xml:lang="de">Plakatwerbung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12823-3">
    <skos:altLabel xml:lang="de">Luftwerbung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12823-3">
    <skos:altLabel xml:lang="de">Lichtwerbung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12823-3">
    <skos:altLabel xml:lang="de">Bandenwerbung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18656-1">
    <skos:broader rdf:resource="thsys/70921"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18656-1">
    <skos:broader rdf:resource="descriptor/13684-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18656-1">
    <skos:broader rdf:resource="descriptor/13655-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15666-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16357-0">
    <skos:broader rdf:resource="thsys/73376"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13222-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19646-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71086">
    <skos:prefLabel xml:lang="en">V.08.01  Development theory</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71086">
    <skos:prefLabel xml:lang="de">V.08.01  Entwicklungstheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10051-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70169">
    <skos:prefLabel xml:lang="de">P.01.01.02  Gartenbauprodukte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id29">
    <rdf:value xml:lang="de">Für Armenfürsorge</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18405-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10821-3">
    <skos:altLabel xml:lang="de">Währungsrisiko</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10821-3">
    <skos:altLabel xml:lang="de">Devisenkursrisiko</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17932-1">
    <skos:broader rdf:resource="thsys/71030"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17932-1">
    <skos:broader rdf:resource="descriptor/17931-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24708-1">
    <skos:prefLabel xml:lang="en">IT industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24708-1">
    <skos:prefLabel xml:lang="de">IT-Industrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14798-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16063-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73299">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12719-5">
    <skos:related rdf:resource="descriptor/12727-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12719-5">
    <skos:related rdf:resource="descriptor/12724-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12719-5">
    <skos:related rdf:resource="descriptor/12730-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11992-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19958-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16131-5">
    <skos:related rdf:resource="descriptor/19824-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16131-5">
    <skos:related rdf:resource="descriptor/19859-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16131-5">
    <skos:related rdf:resource="descriptor/16150-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16131-5">
    <skos:related rdf:resource="descriptor/16143-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16131-5">
    <skos:related rdf:resource="descriptor/11159-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16131-5">
    <skos:related rdf:resource="descriptor/18169-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16131-5">
    <skos:related rdf:resource="descriptor/19902-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70465">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18968-3">
    <skos:related rdf:resource="descriptor/19762-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18968-3">
    <skos:related rdf:resource="descriptor/19839-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18968-3">
    <skos:related rdf:resource="descriptor/19774-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18968-3">
    <skos:related rdf:resource="descriptor/18970-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18968-3">
    <skos:related rdf:resource="descriptor/18971-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18767-6">
    <skos:prefLabel xml:lang="en">Document management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18767-6">
    <skos:prefLabel xml:lang="de">Dokumentenmanagement</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19210-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12183-6">
    <skos:broader rdf:resource="thsys/70087"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12183-6">
    <skos:broader rdf:resource="descriptor/12192-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12183-6">
    <skos:broader rdf:resource="descriptor/12186-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11237-0">
    <skos:related rdf:resource="descriptor/18810-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11237-0">
    <skos:related rdf:resource="descriptor/11236-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11237-0">
    <skos:related rdf:resource="descriptor/11235-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11237-0">
    <skos:related rdf:resource="descriptor/13406-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13656-5">
    <skos:broader rdf:resource="thsys/70921"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13656-5">
    <skos:broader rdf:resource="descriptor/13681-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13656-5">
    <skos:broader rdf:resource="descriptor/13655-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10722-5">
    <skos:broader rdf:resource="thsys/71095"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10722-5">
    <skos:broader rdf:resource="descriptor/10721-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16374-0">
    <skos:related rdf:resource="descriptor/11060-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16374-0">
    <skos:related rdf:resource="descriptor/16373-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12032-6">
    <skos:prefLabel xml:lang="en">Agricultural land-use</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12032-6">
    <skos:prefLabel xml:lang="de">Landwirtschaftliche Bodennutzung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13989-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19695-0">
    <skos:related rdf:resource="descriptor/13397-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19695-0">
    <skos:related rdf:resource="descriptor/16370-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16226-1">
    <skos:related rdf:resource="descriptor/11067-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16226-1">
    <skos:related rdf:resource="descriptor/16354-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16226-1">
    <skos:related rdf:resource="descriptor/20261-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16226-1">
    <skos:related rdf:resource="descriptor/19686-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16226-1">
    <skos:related rdf:resource="descriptor/15711-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16226-1">
    <skos:related rdf:resource="descriptor/16308-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16226-1">
    <skos:related rdf:resource="descriptor/16650-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16226-1">
    <skos:related rdf:resource="descriptor/16287-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16226-1">
    <skos:related rdf:resource="descriptor/16277-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16226-1">
    <skos:related rdf:resource="descriptor/16379-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16226-1">
    <skos:related rdf:resource="descriptor/10968-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16226-1">
    <skos:related rdf:resource="descriptor/15472-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16226-1">
    <skos:related rdf:resource="descriptor/16268-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16226-1">
    <skos:related rdf:resource="descriptor/16246-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70799">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.06.01.04.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19484-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10359-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70481">
    <rdfs:label xml:lang="en">Theory of the firm</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70481">
    <rdfs:label xml:lang="de">Theorie der Unternehmung</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19813-6">
    <skos:scopeNote xml:lang="de">Nur für Bestrafungen durch die Justiz.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17682-5">
    <skos:altLabel xml:lang="de">Königreich Lesotho</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17682-5">
    <skos:altLabel xml:lang="de">Basutoland</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19513-4">
    <skos:altLabel xml:lang="de">Kapitaldeckung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12129-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73373">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70818">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.32</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12964-6">
    <skos:scopeNote xml:lang="de">Einschließlich Gewinnung und Fang anderer Meeresprodukte.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16487-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16438-0">
    <skos:altLabel xml:lang="de">Internationale Gewerkschaftsbewegung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13617-1">
    <skos:broader rdf:resource="thsys/70940"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13617-1">
    <skos:broader rdf:resource="descriptor/13621-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13820-4">
    <skos:altLabel xml:lang="de">Versicherungsschwindel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16068-0">
    <skos:broader rdf:resource="thsys/73391"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16068-0">
    <skos:broader rdf:resource="thsys/70114"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16068-0">
    <skos:broader rdf:resource="descriptor/16080-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19179-4">
    <skos:related rdf:resource="descriptor/17931-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15659-3">
    <skos:prefLabel xml:lang="en">Human being</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15659-3">
    <skos:prefLabel xml:lang="de">Mensch</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13474-4">
    <skos:related rdf:resource="descriptor/11787-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13474-4">
    <skos:related rdf:resource="descriptor/12759-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13474-4">
    <skos:related rdf:resource="descriptor/18553-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11943-5">
    <skos:broader rdf:resource="thsys/70406"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11943-5">
    <skos:broader rdf:resource="thsys/70058"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11943-5">
    <skos:broader rdf:resource="descriptor/10414-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10719-1">
    <skos:altLabel xml:lang="de">Devisenkontrollpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10719-1">
    <skos:altLabel xml:lang="de">Devisenbewirtschaftung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11305-2">
    <skos:narrower rdf:resource="descriptor/12607-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17576-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17922-4">
    <skos:prefLabel xml:lang="en">Guyana</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17922-4">
    <skos:prefLabel xml:lang="de">Guyana</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11772-6">
    <skos:altLabel xml:lang="en">Profit-and-loss sharing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11772-6">
    <skos:altLabel xml:lang="de">Tantiemen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11772-6">
    <skos:altLabel xml:lang="de">Mitarbeiter-Erfolgsbeteiligung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11772-6">
    <skos:altLabel xml:lang="de">Gewinnbeteiligung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11772-6">
    <skos:altLabel xml:lang="de">Ertragsbeteiligung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11772-6">
    <skos:altLabel xml:lang="de">Erfolgsorientierte Vergütung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11772-6">
    <skos:altLabel xml:lang="de">Betriebsergebnisbeteiligung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11772-6">
    <skos:altLabel xml:lang="de">Betriebliche Ertragsbeteiligung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11772-6">
    <skos:altLabel xml:lang="de">Beteiligungslohn</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19205-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12961-5">
    <skos:related rdf:resource="descriptor/15897-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12961-5">
    <skos:related rdf:resource="descriptor/12945-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12961-5">
    <skos:related rdf:resource="descriptor/12962-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12961-5">
    <skos:related rdf:resource="descriptor/12912-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12961-5">
    <skos:related rdf:resource="descriptor/19367-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12961-5">
    <skos:related rdf:resource="descriptor/18987-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12961-5">
    <skos:related rdf:resource="descriptor/18339-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12961-5">
    <skos:related rdf:resource="descriptor/19585-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12961-5">
    <skos:related rdf:resource="descriptor/18311-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12961-5">
    <skos:related rdf:resource="descriptor/14149-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12961-5">
    <skos:related rdf:resource="descriptor/18338-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12961-5">
    <skos:related rdf:resource="descriptor/13874-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12961-5">
    <skos:related rdf:resource="descriptor/12935-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15598-4">
    <skos:prefLabel xml:lang="en">Information system</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15598-4">
    <skos:prefLabel xml:lang="de">Informationssystem</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24784-6">
    <skos:broader rdf:resource="thsys/73340"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24784-6">
    <skos:broader rdf:resource="descriptor/15621-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18982-2">
    <skos:broader rdf:resource="thsys/71115"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18982-2">
    <skos:broader rdf:resource="thsys/71065"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18982-2">
    <skos:broader rdf:resource="descriptor/18948-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11032-4">
    <skos:scopeNote xml:lang="de">Erhebungen über die Struktur der Einnahmen und Ausgaben privater Haushalte; Einkommensquellen und -verteilung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10701-6">
    <skos:broader rdf:resource="thsys/71098"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10701-6">
    <skos:broader rdf:resource="descriptor/10684-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10428-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11516-3">
    <skos:prefLabel xml:lang="en">Optimal rate of inflation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11516-3">
    <skos:prefLabel xml:lang="de">Optimale Inflationsrate</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16548-0">
    <skos:related rdf:resource="descriptor/19507-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16548-0">
    <skos:related rdf:resource="descriptor/13416-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18236-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15958-0">
    <skos:prefLabel xml:lang="en">Resettlement</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15958-0">
    <skos:prefLabel xml:lang="de">Umsiedlung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18629-4">
    <skos:related rdf:resource="descriptor/18624-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12979-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13627-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11772-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10328-2">
    <skos:altLabel xml:lang="en">Production possibility curve</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10328-2">
    <skos:altLabel xml:lang="en">Production frontier</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10328-2">
    <skos:altLabel xml:lang="en">Frontier production function</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10328-2">
    <skos:altLabel xml:lang="en">Frontier function</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10328-2">
    <skos:altLabel xml:lang="de">Transformationskurve</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10328-2">
    <skos:altLabel xml:lang="de">Produktionsmöglichkeitskurve</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10328-2">
    <skos:altLabel xml:lang="de">Produktionsmöglichkeitsgrenze</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10328-2">
    <skos:altLabel xml:lang="de">Produktionseffizienz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19647-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16529-4">
    <skos:altLabel xml:lang="de">Schwerbehindertenpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12910-1">
    <skos:narrower rdf:resource="descriptor/13141-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12910-1">
    <skos:narrower rdf:resource="descriptor/13339-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12910-1">
    <skos:narrower rdf:resource="descriptor/13903-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12910-1">
    <skos:narrower rdf:resource="descriptor/13080-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12910-1">
    <skos:narrower rdf:resource="descriptor/18927-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12910-1">
    <skos:narrower rdf:resource="descriptor/13052-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12910-1">
    <skos:narrower rdf:resource="descriptor/13109-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12910-1">
    <skos:narrower rdf:resource="descriptor/13316-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12910-1">
    <skos:narrower rdf:resource="descriptor/13940-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12910-1">
    <skos:narrower rdf:resource="descriptor/13181-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14261-2">
    <skos:prefLabel xml:lang="en">Metal sheet</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14261-2">
    <skos:prefLabel xml:lang="de">Blech</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71119">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10294-0">
    <skos:prefLabel xml:lang="en">Theory of aggregate investment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10294-0">
    <skos:prefLabel xml:lang="de">Gesamtwirtschaftliche Investitionstheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18333-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24707-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19388-2">
    <skos:related rdf:resource="descriptor/19182-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19388-2">
    <skos:related rdf:resource="descriptor/19181-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19388-2">
    <skos:related rdf:resource="descriptor/14197-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13478-3">
    <skos:broader rdf:resource="thsys/70947"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13478-3">
    <skos:broader rdf:resource="descriptor/13200-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18881-1">
    <skos:related rdf:resource="descriptor/10032-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18881-1">
    <skos:related rdf:resource="descriptor/18059-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18881-1">
    <skos:related rdf:resource="descriptor/18880-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18881-1">
    <skos:related rdf:resource="descriptor/18780-0"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id35">
    <rdfs:seeAlso rdf:resource="descriptor/13641-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12818-3">
    <skos:related rdf:resource="descriptor/12846-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12818-3">
    <skos:related rdf:resource="descriptor/19139-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12818-3">
    <skos:related rdf:resource="descriptor/12793-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18907-2">
    <skos:broader rdf:resource="thsys/70834"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18907-2">
    <skos:broader rdf:resource="descriptor/16543-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18227-3">
    <skos:prefLabel xml:lang="en">Croatia</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18227-3">
    <skos:prefLabel xml:lang="de">Kroatien</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70070">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71083">
    <skos:prefLabel xml:lang="en">V.09.01  Economics of public finance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71083">
    <skos:prefLabel xml:lang="de">V.09.01  Finanzwissenschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14343-0">
    <skos:altLabel xml:lang="de">Füllmaschine</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14343-0">
    <skos:altLabel xml:lang="de">Etikettiermaschine</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14343-0">
    <skos:altLabel xml:lang="de">Abfüllmaschine</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18562-3">
    <skos:prefLabel xml:lang="en">Catering equipment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18562-3">
    <skos:prefLabel xml:lang="de">Gastronomieausstattung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18123-1">
    <skos:broader rdf:resource="thsys/73378"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18123-1">
    <skos:broader rdf:resource="thsys/70851"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18123-1">
    <skos:broader rdf:resource="thsys/70235"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18123-1">
    <skos:broader rdf:resource="descriptor/10968-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12936-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10687-2">
    <skos:altLabel xml:lang="en">PTA (Preferential trade agreement)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10687-2">
    <skos:altLabel xml:lang="de">Präferenzsystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14262-0">
    <skos:altLabel xml:lang="de">Stahlseil</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14262-0">
    <skos:altLabel xml:lang="de">Stahldraht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14262-0">
    <skos:altLabel xml:lang="de">Stacheldraht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14262-0">
    <skos:altLabel xml:lang="de">Metalldraht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14262-0">
    <skos:altLabel xml:lang="de">Drahtseil</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14262-0">
    <skos:altLabel xml:lang="de">Drahtprodukt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11345-4">
    <skos:altLabel xml:lang="de">Tariflohnpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11345-4">
    <skos:altLabel xml:lang="de">Tarifliche Lohnpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11345-4">
    <skos:altLabel xml:lang="de">Gewerkschaftliche Lohnpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15767-0">
    <skos:broader rdf:resource="thsys/70815"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15767-0">
    <skos:broader rdf:resource="thsys/70209"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15767-0">
    <skos:broader rdf:resource="descriptor/10032-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15932-4">
    <skos:narrower rdf:resource="descriptor/16400-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17702-4">
    <skos:prefLabel xml:lang="en">Gabon</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17702-4">
    <skos:prefLabel xml:lang="de">Gabun</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18722-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14359-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13340-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19426-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18814-2">
    <skos:altLabel xml:lang="de">Wehrtechnische Forschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17184-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12891-0">
    <skos:related rdf:resource="descriptor/11146-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12891-0">
    <skos:related rdf:resource="descriptor/19160-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11407-1">
    <skos:related rdf:resource="descriptor/11409-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11407-1">
    <skos:related rdf:resource="descriptor/11224-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11407-1">
    <skos:related rdf:resource="descriptor/11367-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11407-1">
    <skos:related rdf:resource="descriptor/18860-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11407-1">
    <skos:related rdf:resource="descriptor/11394-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11407-1">
    <skos:related rdf:resource="descriptor/18884-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15220-4">
    <skos:broader rdf:resource="thsys/72185"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15220-4">
    <skos:broader rdf:resource="thsys/71035"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15220-4">
    <skos:broader rdf:resource="descriptor/15225-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13673-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18901-0">
    <skos:broader rdf:resource="thsys/73346"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18901-0">
    <skos:broader rdf:resource="descriptor/16670-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18991-1">
    <skos:altLabel xml:lang="de">Lebensmittelpreis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13608-2">
    <skos:altLabel xml:lang="de">Transportbetriebslehre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18062-2">
    <skos:broader rdf:resource="thsys/73381"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18062-2">
    <skos:broader rdf:resource="thsys/73366"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18062-2">
    <skos:broader rdf:resource="descriptor/16278-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11302-1">
    <skos:broader rdf:resource="thsys/71058"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11302-1">
    <skos:broader rdf:resource="thsys/71052"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11302-1">
    <skos:broader rdf:resource="thsys/70485"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11302-1">
    <skos:broader rdf:resource="descriptor/11294-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11302-1">
    <skos:broader rdf:resource="descriptor/18535-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19765-5">
    <skos:related rdf:resource="descriptor/15775-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19765-5">
    <skos:related rdf:resource="descriptor/16696-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19765-5">
    <skos:related rdf:resource="descriptor/16674-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10169-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14081-4">
    <skos:broader rdf:resource="thsys/70169"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14081-4">
    <skos:broader rdf:resource="descriptor/14070-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14081-4">
    <skos:broader rdf:resource="descriptor/14106-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18444-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15931-6">
    <skos:altLabel xml:lang="en">Race relations</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15931-6">
    <skos:altLabel xml:lang="de">Rassenproblem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15931-6">
    <skos:altLabel xml:lang="de">Rassenbeziehungen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15931-6">
    <skos:altLabel xml:lang="de">Ethnische Probleme</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17602-1">
    <skos:prefLabel xml:lang="en">India</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17602-1">
    <skos:prefLabel xml:lang="de">Indien</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12274-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18915-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18825-4">
    <skos:related rdf:resource="descriptor/12165-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12621-1">
    <skos:narrower rdf:resource="descriptor/12116-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12621-1">
    <skos:narrower rdf:resource="descriptor/12119-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10267-3">
    <skos:broader rdf:resource="thsys/71112"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10267-3">
    <skos:broader rdf:resource="thsys/71081"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10267-3">
    <skos:broader rdf:resource="thsys/71072"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10267-3">
    <skos:broader rdf:resource="thsys/70860"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10267-3">
    <skos:broader rdf:resource="descriptor/11722-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10267-3">
    <skos:broader rdf:resource="descriptor/11554-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10267-3">
    <skos:broader rdf:resource="descriptor/10276-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12135-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17491-5">
    <skos:broader rdf:resource="thsys/71032"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17491-5">
    <skos:broader rdf:resource="descriptor/17595-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17491-5">
    <skos:broader rdf:resource="descriptor/19078-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71066">
    <rdfs:label xml:lang="de">Grund und Boden</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12939-5">
    <skos:related rdf:resource="descriptor/14990-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12939-5">
    <skos:related rdf:resource="descriptor/19292-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19184-4">
    <skos:prefLabel xml:lang="en">Metal market</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19184-4">
    <skos:prefLabel xml:lang="de">Metallmarkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10942-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12325-5">
    <skos:altLabel xml:lang="en">Accountancy</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12325-5">
    <skos:altLabel xml:lang="de">Lohnbuchführung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12325-5">
    <skos:altLabel xml:lang="de">Lohn- und Gehaltsabrechnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12325-5">
    <skos:altLabel xml:lang="de">Kaufmännische Buchführung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12325-5">
    <skos:altLabel xml:lang="de">Geschäftsbuchführung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12325-5">
    <skos:altLabel xml:lang="de">Finanzbuchhaltung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12325-5">
    <skos:altLabel xml:lang="de">Doppelte Buchführung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12325-5">
    <skos:altLabel xml:lang="de">Buchhaltung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12325-5">
    <skos:altLabel xml:lang="de">Buchführungstechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16497-5">
    <skos:broader rdf:resource="thsys/73365"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16497-5">
    <skos:broader rdf:resource="descriptor/16463-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11407-1">
    <skos:altLabel xml:lang="de">Ökonomie der Bildung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11407-1">
    <skos:altLabel xml:lang="de">Bildungsökonomie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14438-3">
    <skos:broader rdf:resource="thsys/70258"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14438-3">
    <skos:broader rdf:resource="descriptor/14433-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10221-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19027-6">
    <skos:broader rdf:resource="thsys/70582"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19642-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16184-5">
    <skos:narrower rdf:resource="descriptor/19480-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16184-5">
    <skos:narrower rdf:resource="descriptor/19359-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16184-5">
    <skos:narrower rdf:resource="descriptor/10824-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16184-5">
    <skos:narrower rdf:resource="descriptor/10698-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16184-5">
    <skos:narrower rdf:resource="descriptor/16066-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16184-5">
    <skos:narrower rdf:resource="descriptor/20733-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70958">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13483-3">
    <skos:altLabel xml:lang="de">Spezialhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13483-3">
    <skos:altLabel xml:lang="de">Spezialgeschäft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13483-3">
    <skos:altLabel xml:lang="de">Fachmarkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13483-3">
    <skos:altLabel xml:lang="de">Fachhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13483-3">
    <skos:altLabel xml:lang="de">Fachgeschäft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13483-3">
    <skos:altLabel xml:lang="de">Einzelhandelsfachgeschäft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18995-0">
    <skos:narrower rdf:resource="descriptor/19393-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18995-0">
    <skos:narrower rdf:resource="descriptor/19182-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18995-0">
    <skos:narrower rdf:resource="descriptor/19184-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18995-0">
    <skos:narrower rdf:resource="descriptor/19367-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19510-3">
    <skos:broader rdf:resource="thsys/70833"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19510-3">
    <skos:broader rdf:resource="thsys/70324"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19510-3">
    <skos:broader rdf:resource="descriptor/11570-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18774-2">
    <skos:broader rdf:resource="thsys/70879"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18774-2">
    <skos:broader rdf:resource="descriptor/11572-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18774-2">
    <skos:broader rdf:resource="descriptor/10437-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19743-1">
    <skos:related rdf:resource="descriptor/19732-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19743-1">
    <skos:related rdf:resource="descriptor/19219-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14223-3">
    <skos:broader rdf:resource="thsys/70348"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14223-3">
    <skos:broader rdf:resource="descriptor/14236-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14223-3">
    <skos:broader rdf:resource="descriptor/14251-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14105-2">
    <skos:prefLabel xml:lang="en">Vegetable</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14105-2">
    <skos:prefLabel xml:lang="de">Gemüse</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16315-2">
    <skos:prefLabel xml:lang="en">Separation of powers</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16315-2">
    <skos:prefLabel xml:lang="de">Gewaltenteilung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13006-2">
    <skos:broader rdf:resource="thsys/70989"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13006-2">
    <skos:broader rdf:resource="descriptor/12993-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13006-2">
    <skos:broader rdf:resource="descriptor/12919-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71075">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.09.07.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11105-3">
    <skos:prefLabel xml:lang="en">Transitional economy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11105-3">
    <skos:prefLabel xml:lang="de">Übergangswirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16417-1">
    <skos:broader rdf:resource="thsys/73366"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11943-5">
    <skos:related rdf:resource="descriptor/19363-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11943-5">
    <skos:related rdf:resource="descriptor/10426-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11943-5">
    <skos:related rdf:resource="descriptor/11895-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11943-5">
    <skos:related rdf:resource="descriptor/19243-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10056-2">
    <skos:broader rdf:resource="thsys/70180"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10056-2">
    <skos:broader rdf:resource="descriptor/15860-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10056-2">
    <skos:broader rdf:resource="descriptor/10038-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10898-3">
    <skos:altLabel xml:lang="de">Gegenmachtkonzept</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10898-3">
    <skos:altLabel xml:lang="de">Gegengewichtsprinzip</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10177-4">
    <skos:scopeNote xml:lang="de">Theorie über das ökonomische Verhalten der Haushalte. Marktentscheidungen der privaten Haushalte auf den Faktormärkten (Arbeits- und Kapitalangebot) und auf den Gütermärkten (Nachfrage nach Gütern).</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16301-6">
    <skos:altLabel xml:lang="de">Ombudsmann</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16301-6">
    <skos:altLabel xml:lang="de">Ombudsleute</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16301-6">
    <skos:altLabel xml:lang="de">Ombudsfrau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10237-5">
    <skos:related rdf:resource="descriptor/10338-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10237-5">
    <skos:related rdf:resource="descriptor/10246-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10237-5">
    <skos:related rdf:resource="descriptor/10008-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10237-5">
    <skos:related rdf:resource="descriptor/19356-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10237-5">
    <skos:related rdf:resource="descriptor/10241-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10237-5">
    <skos:related rdf:resource="descriptor/10278-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10237-5">
    <skos:related rdf:resource="descriptor/10381-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10237-5">
    <skos:related rdf:resource="descriptor/10229-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70956">
    <skos:narrower rdf:resource="descriptor/18092-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70956">
    <skos:narrower rdf:resource="descriptor/10940-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70956">
    <skos:narrower rdf:resource="descriptor/13474-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70956">
    <skos:narrower rdf:resource="descriptor/13473-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70956">
    <skos:narrower rdf:resource="descriptor/13479-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70956">
    <skos:narrower rdf:resource="descriptor/12529-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70956">
    <skos:narrower rdf:resource="descriptor/18532-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70956">
    <skos:narrower rdf:resource="descriptor/18531-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70956">
    <skos:narrower rdf:resource="descriptor/13469-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70956">
    <skos:narrower rdf:resource="descriptor/12787-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70956">
    <skos:narrower rdf:resource="descriptor/13459-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70956">
    <skos:narrower rdf:resource="descriptor/12782-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70956">
    <skos:narrower rdf:resource="descriptor/18525-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70956">
    <skos:narrower rdf:resource="descriptor/12781-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70956">
    <skos:narrower rdf:resource="descriptor/18022-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70956">
    <skos:narrower rdf:resource="descriptor/18527-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16754-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19167-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19213-2">
    <skos:related rdf:resource="descriptor/16081-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19213-2">
    <skos:related rdf:resource="descriptor/11057-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19213-2">
    <skos:related rdf:resource="descriptor/15976-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19213-2">
    <skos:related rdf:resource="descriptor/19849-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19213-2">
    <skos:related rdf:resource="descriptor/11065-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19213-2">
    <skos:related rdf:resource="descriptor/11040-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14976-3">
    <skos:related rdf:resource="descriptor/18305-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14976-3">
    <skos:related rdf:resource="descriptor/19421-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14976-3">
    <skos:related rdf:resource="descriptor/13157-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14976-3">
    <skos:related rdf:resource="descriptor/13207-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17989-0">
    <skos:related rdf:resource="descriptor/17931-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17989-0">
    <skos:related rdf:resource="descriptor/17791-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17989-0">
    <skos:related rdf:resource="descriptor/17990-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17989-0">
    <skos:related rdf:resource="descriptor/19174-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17989-0">
    <skos:related rdf:resource="descriptor/17977-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10979-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70939">
    <skos:prefLabel xml:lang="en">W.12.02  Passenger transport</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70939">
    <skos:prefLabel xml:lang="de">W.12.02  Personenverkehr</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19686-1">
    <skos:altLabel xml:lang="de">Staatseingriff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19686-1">
    <skos:altLabel xml:lang="de">Staatliches Handeln</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19686-1">
    <skos:altLabel xml:lang="de">Staatlicher Einfluss</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19686-1">
    <skos:altLabel xml:lang="de">Staatlicher Einfluß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19686-1">
    <skos:altLabel xml:lang="de">Staatliche Steuerung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19686-1">
    <skos:altLabel xml:lang="de">Staatliche Einflußnahme</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19686-1">
    <skos:altLabel xml:lang="de">Politische Steuerung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19393-2">
    <skos:prefLabel xml:lang="en">Fats and oils market</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19393-2">
    <skos:prefLabel xml:lang="de">Ölmarkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12750-4">
    <skos:broader rdf:resource="thsys/70029"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18947-4">
    <skos:prefLabel xml:lang="en">Telecommunications professions</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18947-4">
    <skos:prefLabel xml:lang="de">Telekommunikationsberufe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73388">
    <skos:prefLabel xml:lang="en">N.01.03  Religion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73388">
    <skos:prefLabel xml:lang="de">N.01.03  Religion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id247">
    <rdf:value xml:lang="de">für die Beteiligung an Planungen im regionalen Nahbereich</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18708-1">
    <skos:altLabel xml:lang="de">Reform des Bankwesens</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18708-1">
    <skos:altLabel xml:lang="de">Reform des Bankensystems</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18708-1">
    <skos:altLabel xml:lang="de">Bankreform</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14600-6">
    <skos:broader rdf:resource="thsys/70337"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14600-6">
    <skos:broader rdf:resource="thsys/70014"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14600-6">
    <skos:broader rdf:resource="descriptor/15005-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14600-6">
    <skos:broader rdf:resource="descriptor/18613-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14600-6">
    <skos:broader rdf:resource="descriptor/14599-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13084-3">
    <skos:related rdf:resource="descriptor/14522-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13084-3">
    <skos:related rdf:resource="descriptor/13228-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13084-3">
    <skos:related rdf:resource="descriptor/13096-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13084-3">
    <skos:related rdf:resource="descriptor/18448-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13084-3">
    <skos:related rdf:resource="descriptor/18424-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10652-0">
    <skos:related rdf:resource="descriptor/10676-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10652-0">
    <skos:related rdf:resource="descriptor/10642-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15272-6">
    <skos:prefLabel xml:lang="en">Estimation method</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15272-6">
    <skos:prefLabel xml:lang="de">Schätztheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12778-3">
    <skos:narrower rdf:resource="descriptor/12780-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14915-2">
    <skos:narrower rdf:resource="descriptor/14920-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14915-2">
    <skos:narrower rdf:resource="descriptor/14919-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10804-3">
    <skos:prefLabel xml:lang="en">Global bond</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10804-3">
    <skos:prefLabel xml:lang="de">Internationale Anleihe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18356-6">
    <skos:prefLabel xml:lang="en">Gas pipeline industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18356-6">
    <skos:prefLabel xml:lang="de">Gasleitungstransport</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14079-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16723-1">
    <skos:broader rdf:resource="thsys/73394"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16723-1">
    <skos:broader rdf:resource="descriptor/16719-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13967-2">
    <skos:altLabel xml:lang="de">Vermittlungsleistung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13967-2">
    <skos:altLabel xml:lang="de">Vermittlerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13967-2">
    <skos:altLabel xml:lang="de">Vermittler</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13967-2">
    <skos:altLabel xml:lang="de">Maklerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13967-2">
    <skos:altLabel xml:lang="de">Maklergeschäft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15943-6">
    <skos:related rdf:resource="descriptor/19525-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15943-6">
    <skos:related rdf:resource="descriptor/15939-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15943-6">
    <skos:related rdf:resource="descriptor/19645-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15943-6">
    <skos:related rdf:resource="descriptor/10489-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15943-6">
    <skos:related rdf:resource="descriptor/19638-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10160-0">
    <skos:scopeNote xml:lang="de">Nur für Nutzen und Nutzenmessung im Sinne der Nutzentheorie.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16309-4">
    <skos:related rdf:resource="descriptor/19690-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16309-4">
    <skos:related rdf:resource="descriptor/16013-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16309-4">
    <skos:related rdf:resource="descriptor/19684-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16309-4">
    <skos:related rdf:resource="descriptor/19714-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16309-4">
    <skos:related rdf:resource="descriptor/16264-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16309-4">
    <skos:related rdf:resource="descriptor/16471-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15606-3">
    <skos:prefLabel xml:lang="en">Data processing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15606-3">
    <skos:prefLabel xml:lang="de">Datenverarbeitung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12035-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18922-6">
    <skos:broader rdf:resource="thsys/70828"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18922-6">
    <skos:broader rdf:resource="descriptor/25742-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16030-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18453-1">
    <skos:altLabel xml:lang="de">Industrielle Umweltverschmutzung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18453-1">
    <skos:altLabel xml:lang="de">Industrielle Emission</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18453-1">
    <skos:altLabel xml:lang="de">Industrieemission</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16102-5">
    <skos:broader rdf:resource="thsys/73390"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16102-5">
    <skos:broader rdf:resource="descriptor/16081-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13307-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13681-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24704-2">
    <skos:altLabel xml:lang="en">Process-based management</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24704-2">
    <skos:altLabel xml:lang="de">Prozessorientiertes Management</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24704-2">
    <skos:altLabel xml:lang="de">Prozessorientierte Unternehmensführung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24704-2">
    <skos:altLabel xml:lang="de">Prozessmanagement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24704-2">
    <skos:altLabel xml:lang="de">Arbeitsablaufprozess</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24704-2">
    <skos:altLabel xml:lang="de">Arbeitsablaufprozeß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13493-0">
    <skos:prefLabel xml:lang="en">Department store</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13493-0">
    <skos:prefLabel xml:lang="de">Warenhaus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18010-0">
    <skos:broader rdf:resource="thsys/71110"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18010-0">
    <skos:broader rdf:resource="thsys/70575"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18010-0">
    <skos:broader rdf:resource="descriptor/10342-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12015-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19549-4">
    <skos:broader rdf:resource="thsys/70997"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19549-4">
    <skos:broader rdf:resource="thsys/70961"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19549-4">
    <skos:broader rdf:resource="descriptor/10012-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10780-5">
    <skos:narrower rdf:resource="descriptor/10719-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19818-3">
    <skos:broader rdf:resource="thsys/73390"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19818-3">
    <skos:broader rdf:resource="descriptor/16081-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16421-3">
    <skos:related rdf:resource="descriptor/16312-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16421-3">
    <skos:related rdf:resource="descriptor/16422-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16421-3">
    <skos:related rdf:resource="descriptor/16308-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16421-3">
    <skos:related rdf:resource="descriptor/16552-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16421-3">
    <skos:related rdf:resource="descriptor/16278-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16421-3">
    <skos:related rdf:resource="descriptor/16264-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16421-3">
    <skos:related rdf:resource="descriptor/16512-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16421-3">
    <skos:related rdf:resource="descriptor/16322-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16421-3">
    <skos:related rdf:resource="descriptor/16314-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16421-3">
    <skos:related rdf:resource="descriptor/15918-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16421-3">
    <skos:related rdf:resource="descriptor/16297-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13446-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19666-0">
    <skos:altLabel xml:lang="en">Macroeconomic impact</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19666-0">
    <skos:altLabel xml:lang="de">Volkswirtschaftlicher Effekt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19666-0">
    <skos:altLabel xml:lang="de">Makroökonomischer Einfluß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19666-0">
    <skos:altLabel xml:lang="de">Makroökonomische Wirkung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19666-0">
    <skos:altLabel xml:lang="de">Makroökonomische Effekte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19666-0">
    <skos:altLabel xml:lang="de">Gesamtwirtschaftlicher Einfluss</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19666-0">
    <skos:altLabel xml:lang="de">Gesamtwirtschaftlicher Einfluß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19666-0">
    <skos:altLabel xml:lang="de">Gesamtwirtschaftlicher Effekt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16740-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13383-0">
    <zbwext:useInsteadNote rdf:nodeID="id212"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13383-0">
    <zbwext:useInsteadNote rdf:nodeID="id211"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10626-1">
    <skos:prefLabel xml:lang="en">Economic integration</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10626-1">
    <skos:prefLabel xml:lang="de">Wirtschaftsintegration</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13347-4">
    <skos:prefLabel xml:lang="en">Media industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13347-4">
    <skos:prefLabel xml:lang="de">Medienwirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19719-5">
    <skos:broader rdf:resource="thsys/73366"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19719-5">
    <skos:broader rdf:resource="descriptor/16421-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14214-4">
    <skos:prefLabel xml:lang="en">Diesel fuel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14214-4">
    <skos:prefLabel xml:lang="de">Dieselkraftstoff</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16486-3">
    <skos:prefLabel xml:lang="en">Migration policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16486-3">
    <skos:prefLabel xml:lang="de">Migrationspolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18190-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70074">
    <skos:prefLabel xml:lang="de">P.12.07  Spezialchemikalien</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id193">
    <rdfs:seeAlso rdf:resource="descriptor/14486-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19642-0">
    <skos:scopeNote xml:lang="de">Nur für Länder, die sowohl eine farbige wie eine weiße Bevölkerung haben</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15965-3">
    <skos:related rdf:resource="descriptor/16651-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15965-3">
    <skos:related rdf:resource="descriptor/16741-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15965-3">
    <skos:related rdf:resource="descriptor/15964-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15965-3">
    <skos:related rdf:resource="descriptor/15998-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15965-3">
    <skos:related rdf:resource="descriptor/16704-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70871">
    <rdfs:label xml:lang="en">Public Administration</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70871">
    <rdfs:label xml:lang="de">Öffentliche Verwaltung</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16588-2">
    <skos:narrower rdf:resource="descriptor/16606-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16588-2">
    <skos:narrower rdf:resource="descriptor/16605-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16588-2">
    <skos:narrower rdf:resource="descriptor/16604-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16588-2">
    <skos:narrower rdf:resource="descriptor/16603-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16588-2">
    <skos:narrower rdf:resource="descriptor/16601-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16588-2">
    <skos:narrower rdf:resource="descriptor/13425-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19207-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70087">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18644-1">
    <skos:related rdf:resource="descriptor/13647-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18644-1">
    <skos:related rdf:resource="descriptor/18646-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18644-1">
    <skos:related rdf:resource="descriptor/13274-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18644-1">
    <skos:related rdf:resource="descriptor/13981-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18644-1">
    <skos:related rdf:resource="descriptor/13270-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15774-3">
    <skos:altLabel xml:lang="de">Umwelttechnologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15774-3">
    <skos:altLabel xml:lang="de">Umweltschutztechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15774-3">
    <skos:altLabel xml:lang="de">Technischer Umweltschutz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id115">
    <rdf:value xml:lang="de">Für die internationale Liquidität</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19051-2">
    <skos:prefLabel xml:lang="en">Steel product</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19051-2">
    <skos:prefLabel xml:lang="de">Stahlprodukt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11573-5">
    <skos:related rdf:resource="descriptor/11570-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11573-5">
    <skos:related rdf:resource="descriptor/18912-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11573-5">
    <skos:related rdf:resource="descriptor/19722-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15598-4">
    <skos:editorialNote xml:lang="de">Möglichst spezifischer indexieren.</skos:editorialNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11570-4">
    <skos:narrower rdf:resource="descriptor/11574-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11570-4">
    <skos:narrower rdf:resource="descriptor/11569-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11570-4">
    <skos:narrower rdf:resource="descriptor/19510-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11570-4">
    <skos:narrower rdf:resource="descriptor/11687-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11570-4">
    <skos:narrower rdf:resource="descriptor/11240-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11570-4">
    <skos:narrower rdf:resource="descriptor/11566-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11570-4">
    <skos:narrower rdf:resource="descriptor/11565-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18340-0">
    <skos:related rdf:resource="descriptor/14157-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18340-0">
    <skos:related rdf:resource="descriptor/15895-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18340-0">
    <skos:related rdf:resource="descriptor/18324-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16036-6">
    <skos:prefLabel xml:lang="en">Law of obligations</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16036-6">
    <skos:prefLabel xml:lang="de">Schuldrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13235-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18954-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13680-1">
    <skos:broader rdf:resource="thsys/70922"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13680-1">
    <skos:broader rdf:resource="descriptor/18000-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13680-1">
    <skos:broader rdf:resource="descriptor/13674-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18025-1">
    <skos:prefLabel xml:lang="en">Pension reform</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18025-1">
    <skos:prefLabel xml:lang="de">Rentenreform</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20817-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70938">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11324-5">
    <zbwext:useInsteadNote rdf:nodeID="id198"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70055">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12543-2">
    <skos:narrower rdf:resource="descriptor/12558-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13829-0">
    <skos:prefLabel xml:lang="en">Housing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13829-0">
    <skos:prefLabel xml:lang="de">Wohnung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18654-5">
    <skos:prefLabel xml:lang="en">Telemarketing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18654-5">
    <skos:prefLabel xml:lang="de">Telemarketing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18433-0">
    <skos:prefLabel xml:lang="en">Computer law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18433-0">
    <skos:prefLabel xml:lang="de">Computerrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16533-6">
    <zbwext:useInsteadNote rdf:nodeID="id29"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16280-2">
    <skos:prefLabel xml:lang="en">Political opposition</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16280-2">
    <skos:prefLabel xml:lang="de">Politische Opposition</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11510-1">
    <skos:broader rdf:resource="thsys/70355"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11510-1">
    <skos:broader rdf:resource="thsys/70278"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11510-1">
    <skos:broader rdf:resource="descriptor/11345-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11510-1">
    <skos:broader rdf:resource="descriptor/11504-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24856-0">
    <skos:prefLabel xml:lang="en">Performance measurement</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24856-0">
    <skos:prefLabel xml:lang="de">Performance Measurement</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id2">
    <rdf:value xml:lang="de">Für Operations-Research-Methoden zur Optimierung der Ablaufplanung</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14189-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10902-3">
    <skos:related rdf:resource="descriptor/10202-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10902-3">
    <skos:related rdf:resource="descriptor/10904-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10902-3">
    <skos:related rdf:resource="descriptor/19125-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10902-3">
    <skos:related rdf:resource="descriptor/10012-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10902-3">
    <skos:related rdf:resource="descriptor/10901-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10055-4">
    <skos:related rdf:resource="descriptor/19552-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13153-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70101">
    <skos:narrower rdf:resource="descriptor/10029-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70101">
    <skos:narrower rdf:resource="descriptor/21282-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70101">
    <skos:narrower rdf:resource="descriptor/10895-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70101">
    <skos:narrower rdf:resource="descriptor/12036-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70101">
    <skos:narrower rdf:resource="descriptor/10910-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70101">
    <skos:narrower rdf:resource="descriptor/19429-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70101">
    <skos:narrower rdf:resource="descriptor/10909-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70101">
    <skos:narrower rdf:resource="descriptor/10216-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70101">
    <skos:narrower rdf:resource="descriptor/10908-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70101">
    <skos:narrower rdf:resource="descriptor/10905-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70101">
    <skos:narrower rdf:resource="descriptor/19125-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70101">
    <skos:narrower rdf:resource="descriptor/10902-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70101">
    <skos:narrower rdf:resource="descriptor/10842-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70101">
    <skos:narrower rdf:resource="descriptor/10891-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70101">
    <skos:narrower rdf:resource="descriptor/10898-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70101">
    <skos:narrower rdf:resource="descriptor/19428-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70101">
    <skos:narrower rdf:resource="thsys/70212"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70101">
    <skos:narrower rdf:resource="thsys/70242"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70101">
    <skos:narrower rdf:resource="thsys/71071"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14249-6">
    <skos:related rdf:resource="descriptor/14258-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14249-6">
    <skos:related rdf:resource="descriptor/19050-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70239">
    <skos:prefLabel xml:lang="de">V.12.01.01  Umweltpolitik und Umweltschutz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19382-0">
    <skos:related rdf:resource="descriptor/19338-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19382-0">
    <skos:related rdf:resource="descriptor/19383-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19382-0">
    <skos:related rdf:resource="descriptor/14158-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19382-0">
    <skos:related rdf:resource="descriptor/13431-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15610-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10902-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70003">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19821-0">
    <skos:related rdf:resource="descriptor/18805-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11157-5">
    <skos:prefLabel xml:lang="en">Health economics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11157-5">
    <skos:prefLabel xml:lang="de">Gesundheitsökonomik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14423-2">
    <skos:broader rdf:resource="thsys/70256"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14423-2">
    <skos:broader rdf:resource="descriptor/13513-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19153-1">
    <skos:scopeNote xml:lang="de">Von Schmalenbach geprägter Begriff für eine vom Preis her erfolgende Lenkung betriebsinterner Vorgänge.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14487-4">
    <skos:prefLabel xml:lang="en">Laser</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14487-4">
    <skos:prefLabel xml:lang="de">Laser</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71010">
    <skos:narrower rdf:resource="descriptor/10884-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71010">
    <skos:narrower rdf:resource="descriptor/18256-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71010">
    <skos:narrower rdf:resource="descriptor/12922-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71010">
    <skos:narrower rdf:resource="descriptor/15753-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71010">
    <skos:narrower rdf:resource="descriptor/19490-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71010">
    <skos:narrower rdf:resource="descriptor/11675-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71010">
    <skos:narrower rdf:resource="descriptor/19588-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71010">
    <skos:narrower rdf:resource="descriptor/15754-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71010">
    <skos:narrower rdf:resource="descriptor/18382-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71010">
    <skos:narrower rdf:resource="descriptor/15752-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71010">
    <skos:narrower rdf:resource="descriptor/18995-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71010">
    <skos:narrower rdf:resource="descriptor/18438-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71010">
    <skos:narrower rdf:resource="descriptor/18690-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71010">
    <skos:narrower rdf:resource="descriptor/10593-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71010">
    <skos:narrower rdf:resource="descriptor/18381-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71010">
    <skos:narrower rdf:resource="descriptor/18200-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71010">
    <skos:narrower rdf:resource="descriptor/18194-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71010">
    <skos:narrower rdf:resource="descriptor/18191-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71010">
    <skos:narrower rdf:resource="descriptor/10545-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71010">
    <skos:narrower rdf:resource="thsys/71008"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71010">
    <skos:narrower rdf:resource="thsys/71009"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19627-3">
    <skos:narrower rdf:resource="descriptor/19630-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18403-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15733-3">
    <skos:prefLabel xml:lang="en">Water pollution</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15733-3">
    <skos:prefLabel xml:lang="de">Gewässerbelastung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12259-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18872-2">
    <skos:prefLabel xml:lang="en">Business education</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18872-2">
    <skos:prefLabel xml:lang="de">Betriebswirtschaftsstudium</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13704-6">
    <skos:broader rdf:resource="thsys/71066"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13704-6">
    <skos:broader rdf:resource="thsys/70901"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13704-6">
    <skos:broader rdf:resource="thsys/70889"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13704-6">
    <skos:broader rdf:resource="thsys/70187"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13704-6">
    <skos:broader rdf:resource="descriptor/12281-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13704-6">
    <skos:broader rdf:resource="descriptor/18747-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16100-2">
    <skos:altLabel xml:lang="de">Haushaltsgrundsätzegesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16100-2">
    <skos:altLabel xml:lang="de">Bundeshaushaltsordnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10928-6">
    <skos:broader rdf:resource="thsys/70212"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10928-6">
    <skos:broader rdf:resource="thsys/70041"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16995-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14029-6">
    <skos:related rdf:resource="descriptor/14442-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14029-6">
    <skos:related rdf:resource="descriptor/13097-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14029-6">
    <skos:related rdf:resource="descriptor/13235-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14029-6">
    <skos:related rdf:resource="descriptor/14586-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14029-6">
    <skos:related rdf:resource="descriptor/13096-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14029-6">
    <skos:related rdf:resource="descriptor/14581-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14029-6">
    <skos:related rdf:resource="descriptor/14573-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14029-6">
    <skos:related rdf:resource="descriptor/14033-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14029-6">
    <skos:related rdf:resource="descriptor/14030-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14029-6">
    <skos:related rdf:resource="descriptor/12907-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14029-6">
    <skos:related rdf:resource="descriptor/18916-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14029-6">
    <skos:related rdf:resource="descriptor/11194-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11628-6">
    <zbwext:useInsteadNote rdf:nodeID="id240"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18573-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19785-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12280-1">
    <skos:broader rdf:resource="thsys/70187"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12280-1">
    <skos:broader rdf:resource="thsys/70036"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19363-4">
    <skos:scopeNote xml:lang="de">Ökonomische Analysen städtischen Wachstums. Optimales Wachstum im Sinne der Stadtökonomik. Auch Wachstums- und Schrumpfungsprozesse einzelner Städte.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18872-2">
    <skos:related rdf:resource="descriptor/18143-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18872-2">
    <skos:related rdf:resource="descriptor/12041-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13590-2">
    <skos:broader rdf:resource="thsys/70928"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13590-2">
    <skos:broader rdf:resource="descriptor/10011-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10598-1">
    <skos:altLabel xml:lang="en">Technical cooperation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10598-1">
    <skos:altLabel xml:lang="de">Technische Entwicklungszusammenarbeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10598-1">
    <skos:altLabel xml:lang="de">Technische Entwicklungsberatung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19346-4">
    <skos:broader rdf:resource="thsys/71025"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19346-4">
    <skos:broader rdf:resource="thsys/70960"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19346-4">
    <skos:broader rdf:resource="descriptor/19674-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19516-5">
    <skos:prefLabel xml:lang="en">Returns to education</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19516-5">
    <skos:prefLabel xml:lang="de">Bildungsertrag</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19509-2">
    <skos:scopeNote xml:lang="de">Politik und Vereinbarungen über international verbindliche Sozialstandards und Arbeitsbedingungen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17687-2">
    <skos:prefLabel xml:lang="en">Southern Africa</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17687-2">
    <skos:prefLabel xml:lang="de">Südliches Afrika</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19312-0">
    <skos:altLabel xml:lang="de">Positive Theorie der Außenhandelspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19312-0">
    <skos:altLabel xml:lang="de">Politische Ökonomie der Protektion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19312-0">
    <skos:altLabel xml:lang="de">Neue Handelstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14218-3">
    <skos:related rdf:resource="descriptor/14217-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14218-3">
    <skos:related rdf:resource="descriptor/13001-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14218-3">
    <skos:related rdf:resource="descriptor/13221-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14218-3">
    <skos:related rdf:resource="descriptor/18373-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14218-3">
    <skos:related rdf:resource="descriptor/14197-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14218-3">
    <skos:related rdf:resource="descriptor/14173-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18337-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70883">
    <skos:prefLabel xml:lang="en">W.18  Leasing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70883">
    <skos:prefLabel xml:lang="de">W.18  Leasing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19521-5">
    <skos:altLabel xml:lang="en">Volunteer labor</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19521-5">
    <skos:altLabel xml:lang="en">Voluntary action</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19521-5">
    <skos:altLabel xml:lang="en">Charitable service</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19521-5">
    <skos:altLabel xml:lang="de">Unentgeltliche Arbeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19521-5">
    <skos:altLabel xml:lang="de">Freiwilligenarbeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19521-5">
    <skos:altLabel xml:lang="de">Freiwillige Arbeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19521-5">
    <skos:altLabel xml:lang="de">Ehrenamt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="../stw">
    <dcterms:creator xml:lang="en">German National Library of Economics (Kiel) / Hamburgisches Welt-Wirtschaft-Archiv (Hamburg) / ifo Institute (Munich) / GBI-Genios German Business Information (Munich) (for the 1995-1997 built und publicly funded german version of the STW)</dcterms:creator>
  </rdf:Description>
  <rdf:Description rdf:about="../stw">
    <dcterms:creator xml:lang="en">German National Library of Economics (ZBW) / Leibniz Information Center for Economics (for the current german-english version of the STW)</dcterms:creator>
  </rdf:Description>
  <rdf:Description rdf:about="../stw">
    <dcterms:creator xml:lang="de">Deutsche Zentralbibliothek für Wirtschaftswissenschaften (Kiel) / Hamburgisches Welt-Wirtschaft-Archiv (Hamburg) / ifo-Institut (München) / Gesellschaft für Betriebswirtschaftliche Information GBI-Genios (München) (für die 1995-1997 erstellte und mit öffentlichen Mitteln geförderte deutschsprachige Version des STW)</dcterms:creator>
  </rdf:Description>
  <rdf:Description rdf:about="../stw">
    <dcterms:creator xml:lang="de">ZBW - Deutsche Zentralbibliothek für Wirtschaftswissenschaften / Leibniz-Informationszentrum Wirtschaft (für die aktuelle deutsch-englische Version des STW)</dcterms:creator>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71061">
    <skos:broader rdf:resource="thsys/70051"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10125-2">
    <skos:broader rdf:resource="thsys/71109"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10125-2">
    <skos:broader rdf:resource="thsys/70486"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10125-2">
    <skos:broader rdf:resource="descriptor/10460-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70868">
    <skos:prefLabel xml:lang="de">W.22.01.03  Arbeits- und Sozialverwaltung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14084-5">
    <skos:related rdf:resource="descriptor/19307-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14084-5">
    <skos:related rdf:resource="descriptor/18316-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14084-5">
    <skos:related rdf:resource="descriptor/19306-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14084-5">
    <skos:related rdf:resource="descriptor/14934-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17777-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13541-1">
    <skos:altLabel xml:lang="de">Seekanal</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10185-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11692-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19663-6">
    <skos:altLabel xml:lang="de">Beitrittskriterien zur EU</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19663-6">
    <skos:altLabel xml:lang="de">Beitrittskriterien für die EU</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18106-1">
    <skos:related rdf:resource="descriptor/21282-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18106-1">
    <skos:related rdf:resource="descriptor/18029-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18106-1">
    <skos:related rdf:resource="descriptor/12083-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18106-1">
    <skos:related rdf:resource="descriptor/24856-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18106-1">
    <skos:related rdf:resource="descriptor/12275-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core">
    <ns0:description xmlns:ns0="http://purl.org/dc/elements/1.1/" xml:lang="en">An RDF vocabulary for describing the basic structure and content
      of concept schemes such as thesauri, classification schemes, subject heading lists,
      taxonomies, 'folksonomies', other types of controlled vocabulary, and also concept schemes
      embedded in glossaries and terminologies.</ns0:description>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15891-6">
    <skos:altLabel xml:lang="en">Marine scientific research</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15891-6">
    <skos:altLabel xml:lang="de">Ozeanologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15891-6">
    <skos:altLabel xml:lang="de">Ozeanographie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15891-6">
    <skos:altLabel xml:lang="de">Meereskunde</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18590-5">
    <skos:related rdf:resource="descriptor/18298-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18590-5">
    <skos:related rdf:resource="descriptor/13252-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18590-5">
    <skos:related rdf:resource="descriptor/13703-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12192-5">
    <skos:narrower rdf:resource="descriptor/19157-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12192-5">
    <skos:narrower rdf:resource="descriptor/12191-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12192-5">
    <skos:narrower rdf:resource="descriptor/12187-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12192-5">
    <skos:narrower rdf:resource="descriptor/12184-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12192-5">
    <skos:narrower rdf:resource="descriptor/12183-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12192-5">
    <skos:narrower rdf:resource="descriptor/12181-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18783-1">
    <skos:altLabel xml:lang="en">Materials science</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18783-1">
    <skos:altLabel xml:lang="de">Werkstoffwissenschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18783-1">
    <skos:altLabel xml:lang="de">Werkstofforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18783-1">
    <skos:altLabel xml:lang="de">Werkstoffkunde</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18783-1">
    <skos:altLabel xml:lang="de">Materialforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70002">
    <skos:narrower rdf:resource="descriptor/16815-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70002">
    <skos:narrower rdf:resource="thsys/70554"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70002">
    <skos:narrower rdf:resource="thsys/70505"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70002">
    <skos:narrower rdf:resource="thsys/70174"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70002">
    <skos:narrower rdf:resource="thsys/70557"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70002">
    <skos:narrower rdf:resource="thsys/70131"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11556-5">
    <skos:related rdf:resource="descriptor/10590-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11556-5">
    <skos:related rdf:resource="descriptor/11529-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11556-5">
    <skos:related rdf:resource="descriptor/11734-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15520-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14142-3">
    <skos:prefLabel xml:lang="en">Cattle</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14142-3">
    <skos:prefLabel xml:lang="de">Rind</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71015">
    <skos:prefLabel xml:lang="en">W.04.01.01  Electric utility industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71015">
    <skos:prefLabel xml:lang="de">W.04.01.01  Elektrizitätswirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10147-6">
    <zbwext:useInsteadNote rdf:nodeID="id323"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13853-3">
    <skos:broader rdf:resource="thsys/70970"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13853-3">
    <skos:broader rdf:resource="descriptor/10213-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19678-0">
    <skos:prefLabel xml:lang="en">Network access</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19678-0">
    <skos:prefLabel xml:lang="de">Netzzugang</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14204-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18653-0">
    <skos:prefLabel xml:lang="en">Telecommunications policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18653-0">
    <skos:prefLabel xml:lang="de">Telekommunikationspolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16290-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19386-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18376-0">
    <skos:narrower rdf:resource="descriptor/11673-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18141-6">
    <skos:related rdf:resource="descriptor/18719-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70860">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18686-6">
    <skos:prefLabel xml:lang="en">Eurobond</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18686-6">
    <skos:prefLabel xml:lang="de">Euro-Anleihe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19254-2">
    <skos:prefLabel xml:lang="en">Adjustment costs</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19254-2">
    <skos:prefLabel xml:lang="de">Anpassungskosten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13375-6">
    <skos:altLabel xml:lang="de">Zahnarztpraxis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13375-6">
    <skos:altLabel xml:lang="de">Zahnärztinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13375-6">
    <skos:altLabel xml:lang="de">Zahnarzt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15709-0">
    <skos:related rdf:resource="descriptor/19580-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15709-0">
    <skos:related rdf:resource="descriptor/13571-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15709-0">
    <skos:related rdf:resource="descriptor/13594-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15709-0">
    <skos:related rdf:resource="descriptor/13510-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15709-0">
    <skos:related rdf:resource="descriptor/15716-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15709-0">
    <skos:related rdf:resource="descriptor/15994-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15709-0">
    <skos:related rdf:resource="descriptor/10314-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19714-1">
    <skos:related rdf:resource="descriptor/16188-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19714-1">
    <skos:related rdf:resource="descriptor/16014-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19714-1">
    <skos:related rdf:resource="descriptor/19688-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19714-1">
    <skos:related rdf:resource="descriptor/19684-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19714-1">
    <skos:related rdf:resource="descriptor/13420-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19714-1">
    <skos:related rdf:resource="descriptor/16309-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19714-1">
    <skos:related rdf:resource="descriptor/16422-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19714-1">
    <skos:related rdf:resource="descriptor/19705-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73332">
    <skos:narrower rdf:resource="descriptor/10581-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73332">
    <skos:narrower rdf:resource="descriptor/16173-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73332">
    <skos:narrower rdf:resource="descriptor/16188-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73332">
    <skos:narrower rdf:resource="descriptor/19860-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73332">
    <skos:narrower rdf:resource="descriptor/19480-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73332">
    <skos:narrower rdf:resource="descriptor/19714-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73332">
    <skos:narrower rdf:resource="descriptor/19325-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73332">
    <skos:narrower rdf:resource="descriptor/16309-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73332">
    <skos:narrower rdf:resource="descriptor/16187-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73332">
    <skos:narrower rdf:resource="descriptor/18064-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73332">
    <skos:narrower rdf:resource="descriptor/10593-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73332">
    <skos:narrower rdf:resource="descriptor/18200-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73332">
    <skos:narrower rdf:resource="descriptor/16264-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73332">
    <skos:narrower rdf:resource="descriptor/15964-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73332">
    <skos:narrower rdf:resource="descriptor/16003-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73332">
    <skos:narrower rdf:resource="descriptor/19359-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73332">
    <skos:narrower rdf:resource="descriptor/18194-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73332">
    <skos:narrower rdf:resource="descriptor/18191-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73332">
    <skos:narrower rdf:resource="descriptor/10824-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73332">
    <skos:narrower rdf:resource="descriptor/18567-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73332">
    <skos:narrower rdf:resource="descriptor/10829-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73332">
    <skos:narrower rdf:resource="descriptor/18605-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73332">
    <skos:narrower rdf:resource="descriptor/16184-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73332">
    <skos:narrower rdf:resource="descriptor/19509-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73332">
    <skos:narrower rdf:resource="descriptor/19851-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73332">
    <skos:narrower rdf:resource="descriptor/10698-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73332">
    <skos:narrower rdf:resource="descriptor/18312-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73332">
    <skos:narrower rdf:resource="descriptor/10691-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73332">
    <skos:narrower rdf:resource="descriptor/10828-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73332">
    <skos:narrower rdf:resource="descriptor/16066-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73332">
    <skos:narrower rdf:resource="descriptor/20733-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73332">
    <skos:narrower rdf:resource="descriptor/19843-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73332">
    <skos:narrower rdf:resource="descriptor/19350-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16723-1">
    <skos:related rdf:resource="descriptor/16563-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16723-1">
    <skos:related rdf:resource="descriptor/16544-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16723-1">
    <skos:related rdf:resource="descriptor/16642-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16723-1">
    <skos:related rdf:resource="descriptor/19731-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16723-1">
    <skos:related rdf:resource="descriptor/16579-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14525-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12758-2">
    <skos:related rdf:resource="descriptor/20277-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12758-2">
    <skos:related rdf:resource="descriptor/12808-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12758-2">
    <skos:related rdf:resource="descriptor/12759-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12758-2">
    <skos:related rdf:resource="descriptor/12757-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12758-2">
    <skos:related rdf:resource="descriptor/12750-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14554-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14084-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15993-5">
    <skos:broader rdf:resource="thsys/73390"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15993-5">
    <skos:broader rdf:resource="thsys/71016"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15993-5">
    <skos:broader rdf:resource="thsys/70227"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15993-5">
    <skos:broader rdf:resource="descriptor/16081-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15993-5">
    <skos:broader rdf:resource="descriptor/16007-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19609-5">
    <skos:prefLabel xml:lang="en">Data envelopment analysis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19609-5">
    <skos:prefLabel xml:lang="de">Data-Envelopment-Analyse</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10159-6">
    <skos:related rdf:resource="descriptor/10106-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10159-6">
    <skos:related rdf:resource="descriptor/10164-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18614-3">
    <skos:prefLabel xml:lang="en">Cross-border freight transport</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18614-3">
    <skos:prefLabel xml:lang="de">Grenzüberschreitender Güterverkehr</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19532-0">
    <skos:prefLabel xml:lang="en">Tenure choice</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19532-0">
    <skos:prefLabel xml:lang="de">Tenure Choice</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19210-1">
    <skos:altLabel xml:lang="en">WTP (Willingness to pay)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19210-1">
    <skos:altLabel xml:lang="en">WTA (Willingness to accept)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19210-1">
    <skos:altLabel xml:lang="en">Willingness to accept</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19210-1">
    <skos:altLabel xml:lang="en">CVM (Contingent valuation method)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19210-1">
    <skos:altLabel xml:lang="en">Contingent valuation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19210-1">
    <skos:altLabel xml:lang="de">Zahlungsbereitschaftsanalyse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16100-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13165-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11322-2">
    <skos:narrower rdf:resource="descriptor/12598-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11322-2">
    <skos:narrower rdf:resource="descriptor/11331-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11322-2">
    <skos:narrower rdf:resource="descriptor/11330-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11322-2">
    <skos:narrower rdf:resource="descriptor/10622-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11322-2">
    <skos:narrower rdf:resource="descriptor/11328-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11322-2">
    <skos:narrower rdf:resource="descriptor/12594-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11322-2">
    <skos:narrower rdf:resource="descriptor/11321-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11322-2">
    <skos:narrower rdf:resource="descriptor/18487-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11322-2">
    <skos:narrower rdf:resource="descriptor/11320-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11322-2">
    <skos:narrower rdf:resource="descriptor/19499-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11322-2">
    <skos:narrower rdf:resource="descriptor/11318-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11322-2">
    <skos:narrower rdf:resource="descriptor/24623-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19458-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13873-4">
    <skos:prefLabel xml:lang="en">Fishermen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13873-4">
    <skos:prefLabel xml:lang="de">Fischereiberufe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13073-1">
    <skos:altLabel xml:lang="en">PC industry</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13073-1">
    <skos:altLabel xml:lang="de">Personalcomputerindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13073-1">
    <skos:altLabel xml:lang="de">Mikrocomputerindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19843-4">
    <skos:altLabel xml:lang="de">Recht auf Asyl</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19843-4">
    <skos:altLabel xml:lang="de">Flüchtlingsrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13158-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11731-6">
    <skos:altLabel xml:lang="en">Deficit cutting</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11731-6">
    <skos:altLabel xml:lang="en">Budget restraint</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11731-6">
    <skos:altLabel xml:lang="en">Budget reduction</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11731-6">
    <skos:altLabel xml:lang="de">Konsolidierter Haushalt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11731-6">
    <skos:altLabel xml:lang="de">Finanzwirtschaftliche Sparmaßnahme</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11731-6">
    <skos:altLabel xml:lang="de">Budgetkürzung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11731-6">
    <skos:altLabel xml:lang="de">Budgetkonsolidierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16391-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id211">
    <rdf:value xml:lang="de">Für die Ergebnisse von Erhebungen</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18615-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18437-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18089-3">
    <skos:prefLabel xml:lang="en">High-speed rail</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18089-3">
    <skos:prefLabel xml:lang="de">Hochgeschwindigkeitsverkehr</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10901-5">
    <skos:related rdf:resource="descriptor/10902-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10901-5">
    <skos:related rdf:resource="descriptor/15033-3"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id288">
    <rdfs:seeAlso rdf:resource="descriptor/16187-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70811">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14001-0">
    <skos:broader rdf:resource="thsys/71055"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14001-0">
    <skos:broader rdf:resource="thsys/70823"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14001-0">
    <skos:broader rdf:resource="descriptor/18535-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10438-2">
    <skos:broader rdf:resource="thsys/73372"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10438-2">
    <skos:broader rdf:resource="thsys/70530"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10438-2">
    <skos:broader rdf:resource="descriptor/10987-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73344">
    <skos:narrower rdf:resource="descriptor/19799-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73344">
    <skos:narrower rdf:resource="descriptor/14772-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73344">
    <skos:narrower rdf:resource="descriptor/18280-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73344">
    <skos:narrower rdf:resource="descriptor/19797-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73344">
    <skos:narrower rdf:resource="descriptor/19798-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73344">
    <skos:narrower rdf:resource="descriptor/10455-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73344">
    <skos:narrower rdf:resource="descriptor/18252-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73344">
    <skos:narrower rdf:resource="descriptor/16757-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73344">
    <skos:narrower rdf:resource="descriptor/10464-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73344">
    <skos:narrower rdf:resource="descriptor/18289-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73344">
    <skos:narrower rdf:resource="descriptor/15635-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12307-0">
    <skos:narrower rdf:resource="descriptor/12381-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12307-0">
    <skos:narrower rdf:resource="descriptor/12325-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10415-0">
    <skos:altLabel xml:lang="en">Import-led growth</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10415-0">
    <skos:altLabel xml:lang="en">Import-biased growth</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10415-0">
    <skos:altLabel xml:lang="en">Export-biased growth</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10415-0">
    <skos:altLabel xml:lang="de">Positiv-handelsorientiertes Wachstum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10415-0">
    <skos:altLabel xml:lang="de">Negativ-handelsorientiertes Wachstum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16515-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14237-6">
    <skos:altLabel xml:lang="de">Stein</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14237-6">
    <skos:altLabel xml:lang="de">Schotter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14237-6">
    <skos:altLabel xml:lang="de">Sandstein</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14237-6">
    <skos:altLabel xml:lang="de">Mergel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14237-6">
    <skos:altLabel xml:lang="de">Dolomit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14237-6">
    <skos:altLabel xml:lang="de">Bimsstein</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10692-2">
    <skos:prefLabel xml:lang="en">Export restrictions</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10692-2">
    <skos:prefLabel xml:lang="de">Exportbeschränkung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10603-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13417-2">
    <skos:prefLabel xml:lang="en">Social jurisdiction</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13417-2">
    <skos:prefLabel xml:lang="de">Sozialgericht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16543-3">
    <skos:narrower rdf:resource="descriptor/19511-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16543-3">
    <skos:narrower rdf:resource="descriptor/18908-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16543-3">
    <skos:narrower rdf:resource="descriptor/18907-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16543-3">
    <skos:narrower rdf:resource="descriptor/18855-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16549-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14285-2">
    <skos:broader rdf:resource="thsys/70358"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14285-2">
    <skos:broader rdf:resource="descriptor/14267-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19440-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18644-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18816-5">
    <skos:broader rdf:resource="thsys/73363"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18816-5">
    <skos:broader rdf:resource="thsys/71067"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18816-5">
    <skos:broader rdf:resource="thsys/70865"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18816-5">
    <skos:broader rdf:resource="descriptor/11913-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13199-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19286-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19259-6">
    <skos:prefLabel xml:lang="en">Monetary transmission</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19259-6">
    <skos:prefLabel xml:lang="de">Transmissionsmechanismus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11572-0">
    <skos:broader rdf:resource="thsys/71080"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11572-0">
    <skos:broader rdf:resource="thsys/71045"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11572-0">
    <skos:broader rdf:resource="thsys/70038"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11572-0">
    <skos:broader rdf:resource="descriptor/11568-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10150-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10542-0">
    <skos:altLabel xml:lang="de">Sektorale Polarisation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10542-0">
    <skos:altLabel xml:lang="de">Entwicklungspoltheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12028-4">
    <skos:broader rdf:resource="thsys/73392"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12028-4">
    <skos:broader rdf:resource="thsys/71066"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12028-4">
    <skos:broader rdf:resource="thsys/70889"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12028-4">
    <skos:broader rdf:resource="descriptor/16035-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11600-0">
    <skos:related rdf:resource="descriptor/11711-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11600-0">
    <skos:related rdf:resource="descriptor/11599-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19646-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18198-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13713-5">
    <skos:altLabel xml:lang="en">Share market</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19070-5">
    <skos:narrower rdf:resource="descriptor/19741-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19070-5">
    <skos:narrower rdf:resource="descriptor/19745-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16704-5">
    <skos:broader rdf:resource="thsys/73394"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16704-5">
    <skos:broader rdf:resource="descriptor/16719-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12299-1">
    <skos:broader rdf:resource="thsys/71107"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12299-1">
    <skos:broader rdf:resource="thsys/71043"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12299-1">
    <skos:broader rdf:resource="thsys/70899"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18694-0">
    <skos:scopeNote xml:lang="de">Theorie und Politik internationaler Kreditvergabe und Verschuldung; vor allem für die Kreditgewährung durch internationale Finanzinstitutionen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/p">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">P</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11295-0">
    <skos:altLabel xml:lang="de">Berufswunsch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11295-0">
    <skos:altLabel xml:lang="de">Berufsreife</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11295-0">
    <skos:altLabel xml:lang="de">Berufspsychologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11295-0">
    <skos:altLabel xml:lang="de">Berufserwartung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11295-0">
    <skos:altLabel xml:lang="de">Berufseignung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18909-5">
    <skos:related rdf:resource="descriptor/15725-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18909-5">
    <skos:related rdf:resource="descriptor/18904-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19476-5">
    <skos:scopeNote xml:lang="de">Beschreibung einzelner Städte.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id244">
    <rdf:value xml:lang="de">Für Unternehmensorganisation</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13720-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16362-0">
    <skos:altLabel xml:lang="de">Christdemokratische Partei</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19472-6">
    <skos:altLabel xml:lang="en">Urban labor market</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19635-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13827-4">
    <skos:related rdf:resource="descriptor/13826-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13827-4">
    <skos:related rdf:resource="descriptor/21072-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13827-4">
    <skos:related rdf:resource="descriptor/13820-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70834">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18234-6">
    <skos:narrower rdf:resource="descriptor/14177-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10019-1">
    <skos:prefLabel xml:lang="en">Debt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10019-1">
    <skos:prefLabel xml:lang="de">Schulden</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13635-6">
    <skos:broader rdf:resource="thsys/71028"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13635-6">
    <skos:broader rdf:resource="thsys/70924"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13635-6">
    <skos:broader rdf:resource="descriptor/13644-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17017-5">
    <skos:altLabel xml:lang="x-other">Citta del Vaticano</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17017-5">
    <skos:altLabel xml:lang="de">Staat Vatikanstadt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17017-5">
    <skos:altLabel xml:lang="de">Heiliger Stuhl</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14909-4">
    <skos:broader rdf:resource="thsys/70031"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14909-4">
    <skos:broader rdf:resource="descriptor/14899-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12880-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13817-0">
    <skos:scopeNote xml:lang="de">Versicherung technischer Anlagen, z.B. von Kraftwerken, chemischen Anlagen, EDV-Anlagen; sie betrifft Sach- und Vermögensschäden</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19651-6">
    <skos:prefLabel xml:lang="en">Homepage</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19651-6">
    <skos:prefLabel xml:lang="de">Homepage</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11469-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17472-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12239-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11626-3">
    <skos:prefLabel xml:lang="en">Mixed ownership corporation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11626-3">
    <skos:prefLabel xml:lang="de">Gemischtwirtschaftliches Unternehmen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18559-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11273-3">
    <skos:broader rdf:resource="thsys/70051"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11273-3">
    <skos:broader rdf:resource="descriptor/11272-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18192-3">
    <skos:broader rdf:resource="thsys/71029"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18192-3">
    <skos:broader rdf:resource="descriptor/12949-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19631-5">
    <skos:related rdf:resource="descriptor/16187-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19631-5">
    <skos:related rdf:resource="descriptor/15744-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19631-5">
    <skos:related rdf:resource="descriptor/19614-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19631-5">
    <skos:related rdf:resource="descriptor/19617-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19631-5">
    <skos:related rdf:resource="descriptor/15887-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19631-5">
    <skos:related rdf:resource="descriptor/16502-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19631-5">
    <skos:related rdf:resource="descriptor/18312-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19631-5">
    <skos:related rdf:resource="descriptor/11807-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71041">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">B.03.04</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19427-4">
    <skos:prefLabel xml:lang="en">Refinery</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19427-4">
    <skos:prefLabel xml:lang="de">Erdölraffinerie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12881-3">
    <skos:altLabel xml:lang="en">Management consulting</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12881-3">
    <skos:altLabel xml:lang="en">Consulting engineer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12881-3">
    <skos:altLabel xml:lang="en">Consulting</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12881-3">
    <skos:altLabel xml:lang="de">Wirtschaftsberatung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12881-3">
    <skos:altLabel xml:lang="de">Unternehmensberaterinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12881-3">
    <skos:altLabel xml:lang="de">Unternehmensberater</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12881-3">
    <skos:altLabel xml:lang="de">Managementberatung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12881-3">
    <skos:altLabel xml:lang="de">Betriebswirtschaftliche Beratung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12881-3">
    <skos:altLabel xml:lang="de">Betriebsberatung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id340">
    <rdf:value xml:lang="de">Für Netzwerke im Sinne von Unternehmenszusammenarbeit</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16562-6">
    <skos:prefLabel xml:lang="en">Social economics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16562-6">
    <skos:prefLabel xml:lang="de">Sozialökonomik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13904-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14586-2">
    <skos:narrower rdf:resource="descriptor/14834-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14586-2">
    <skos:narrower rdf:resource="descriptor/14920-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12950-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18333-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16019-6">
    <skos:prefLabel xml:lang="en">Administrative law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16019-6">
    <skos:prefLabel xml:lang="de">Verwaltungsrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18005-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12923-6">
    <skos:narrower rdf:resource="descriptor/18416-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16510-4">
    <skos:prefLabel xml:lang="en">Homeland security</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16510-4">
    <skos:prefLabel xml:lang="de">Innere Sicherheit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13454-3">
    <skos:altLabel xml:lang="de">Ländliche Bevölkerung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13454-3">
    <skos:altLabel xml:lang="de">Agrarbevölkerung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19127-2">
    <skos:related rdf:resource="descriptor/13807-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19127-2">
    <skos:related rdf:resource="descriptor/12794-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17634-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10893-6">
    <skos:altLabel xml:lang="en">Rate regulation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10893-6">
    <skos:altLabel xml:lang="en">Rate padding</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10893-6">
    <skos:altLabel xml:lang="en">Cost-plus-regulierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10893-6">
    <skos:altLabel xml:lang="de">Rentabilitätsregulierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10893-6">
    <skos:altLabel xml:lang="de">Optimale faire Rendite</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10893-6">
    <skos:altLabel xml:lang="de">Graduierte Renditenbeschränkung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10893-6">
    <skos:altLabel xml:lang="de">Faire Rendite</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10893-6">
    <skos:altLabel xml:lang="de">Averch-Johnson-Effekt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10893-6">
    <skos:altLabel xml:lang="de">A-J-Effekt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11834-3">
    <skos:narrower rdf:resource="descriptor/18583-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11834-3">
    <skos:narrower rdf:resource="descriptor/13601-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11834-3">
    <skos:narrower rdf:resource="descriptor/13605-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11834-3">
    <skos:narrower rdf:resource="descriptor/13525-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10604-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11401-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18294-2">
    <skos:related rdf:resource="descriptor/18089-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70550">
    <rdfs:label xml:lang="en">Product management</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70550">
    <rdfs:label xml:lang="de">Produktpolitik</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10831-0">
    <skos:broader rdf:resource="thsys/71075"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10831-0">
    <skos:broader rdf:resource="thsys/71040"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14110-2">
    <skos:broader rdf:resource="thsys/70169"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14110-2">
    <skos:broader rdf:resource="descriptor/14104-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70952">
    <skos:broader rdf:resource="thsys/70963"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19630-0">
    <skos:related rdf:resource="descriptor/14170-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19630-0">
    <skos:related rdf:resource="descriptor/12962-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19630-0">
    <skos:related rdf:resource="descriptor/19015-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19630-0">
    <skos:related rdf:resource="descriptor/19625-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16670-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70893">
    <skos:prefLabel xml:lang="de">W.15  Börsen und Wertpapierhandel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10227-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10931-3">
    <skos:broader rdf:resource="thsys/70212"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10931-3">
    <skos:broader rdf:resource="thsys/70040"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10931-3">
    <skos:broader rdf:resource="descriptor/10942-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10931-3">
    <skos:broader rdf:resource="descriptor/12794-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70925">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19798-4">
    <skos:broader rdf:resource="thsys/73344"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19798-4">
    <skos:broader rdf:resource="descriptor/10470-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14175-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19026-1">
    <skos:altLabel xml:lang="de">Implementierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id75">
    <rdf:value xml:lang="de">Für Elektrodraht und -kabel</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21408-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12283-2">
    <skos:altLabel xml:lang="de">Finanzierungsgesellschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19010-2">
    <skos:broader rdf:resource="thsys/73393"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20762-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12913-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13255-2">
    <skos:altLabel xml:lang="de">Spurgebundener Verkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13255-2">
    <skos:altLabel xml:lang="de">Schienentransport</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13255-2">
    <skos:altLabel xml:lang="de">Schienengebundener Verkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13255-2">
    <skos:altLabel xml:lang="de">Eisenbahnwesen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13255-2">
    <skos:altLabel xml:lang="de">Eisenbahnverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13255-2">
    <skos:altLabel xml:lang="de">Eisenbahn</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18286-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14116-4">
    <skos:broader rdf:resource="thsys/70169"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14116-4">
    <skos:broader rdf:resource="descriptor/14105-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19855-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12952-6">
    <skos:broader rdf:resource="thsys/71025"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12952-6">
    <skos:broader rdf:resource="descriptor/12955-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19650-1">
    <skos:prefLabel xml:lang="en">Migration theory</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19650-1">
    <skos:prefLabel xml:lang="de">Migrationstheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18169-5">
    <skos:broader rdf:resource="thsys/73336"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18169-5">
    <skos:broader rdf:resource="thsys/71051"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18169-5">
    <skos:broader rdf:resource="thsys/70235"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18169-5">
    <skos:broader rdf:resource="descriptor/16136-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12523-1">
    <skos:prefLabel xml:lang="en">Standard costing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12523-1">
    <skos:prefLabel xml:lang="de">Plankostenrechnung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14413-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10877-4">
    <skos:related rdf:resource="descriptor/10785-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10877-4">
    <skos:related rdf:resource="descriptor/10854-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11230-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14368-5">
    <skos:broader rdf:resource="thsys/70318"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14368-5">
    <skos:broader rdf:resource="descriptor/14381-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13645-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19368-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13125-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13687-1">
    <skos:altLabel xml:lang="en">Bank monitoring</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13687-1">
    <skos:altLabel xml:lang="de">Staatsaufsicht im Bankensektor</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13687-1">
    <skos:altLabel xml:lang="de">Bankenaufsichtsrechtliche Regelungen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13687-1">
    <skos:altLabel xml:lang="de">Bankenaufsichtsrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13687-1">
    <skos:altLabel xml:lang="de">Bankenaufsichtsbehörde</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13687-1">
    <skos:altLabel xml:lang="de">Bankaufsicht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12204-3">
    <skos:related rdf:resource="descriptor/19827-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12204-3">
    <skos:related rdf:resource="descriptor/12168-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12204-3">
    <skos:related rdf:resource="descriptor/19271-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18176-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10136-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18408-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12656-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11191-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18183-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12268-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19649-0">
    <skos:related rdf:resource="descriptor/19687-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19649-0">
    <skos:related rdf:resource="descriptor/19323-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19649-0">
    <skos:related rdf:resource="descriptor/16486-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19649-0">
    <skos:related rdf:resource="descriptor/19640-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19649-0">
    <skos:related rdf:resource="descriptor/15994-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19649-0">
    <skos:related rdf:resource="descriptor/15952-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19649-0">
    <skos:related rdf:resource="descriptor/15989-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19649-0">
    <skos:related rdf:resource="descriptor/19843-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18663-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11736-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20241-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19259-6">
    <skos:related rdf:resource="descriptor/19258-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15735-6">
    <skos:altLabel xml:lang="en">Trans-boundary pollution</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15735-6">
    <skos:altLabel xml:lang="de">Grenzüberschreitende Umweltverschmutzung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73381">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.04.02.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17477-6">
    <skos:altLabel xml:lang="x-other">Litovskaja SSR</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17477-6">
    <skos:altLabel xml:lang="x-other">Lietuva</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17477-6">
    <skos:altLabel xml:lang="de">Republik Litauen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16730-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10672-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13681-6">
    <skos:altLabel xml:lang="de">Verkabelung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13681-6">
    <skos:altLabel xml:lang="de">Kupferkabelübertragung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13681-6">
    <skos:altLabel xml:lang="de">Koaxialkabelübertragung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13681-6">
    <skos:altLabel xml:lang="de">Kabelübertragung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10961-1">
    <skos:prefLabel xml:lang="en">Divestiture</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10961-1">
    <skos:prefLabel xml:lang="de">Unternehmensentflechtung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13413-3">
    <skos:related rdf:resource="descriptor/16212-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13413-3">
    <skos:related rdf:resource="descriptor/12005-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13413-3">
    <skos:related rdf:resource="descriptor/16336-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13413-3">
    <skos:related rdf:resource="descriptor/16019-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13413-3">
    <skos:related rdf:resource="descriptor/12511-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13413-3">
    <skos:related rdf:resource="descriptor/16335-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13413-3">
    <skos:related rdf:resource="descriptor/13993-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13413-3">
    <skos:related rdf:resource="descriptor/12160-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13413-3">
    <skos:related rdf:resource="descriptor/16329-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13413-3">
    <skos:related rdf:resource="descriptor/18803-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13413-3">
    <skos:related rdf:resource="descriptor/13303-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13413-3">
    <skos:related rdf:resource="descriptor/13409-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13413-3">
    <skos:related rdf:resource="descriptor/11154-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13413-3">
    <skos:related rdf:resource="descriptor/12042-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10406-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10255-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16067-2">
    <skos:prefLabel xml:lang="en">Financial disclosure</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16067-2">
    <skos:prefLabel xml:lang="de">Publizitätspflicht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19673-3">
    <skos:narrower rdf:resource="descriptor/12436-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19673-3">
    <skos:narrower rdf:resource="descriptor/19102-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19673-3">
    <skos:narrower rdf:resource="descriptor/12419-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13001-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11033-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10032-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11404-0">
    <skos:related rdf:resource="descriptor/18870-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11404-0">
    <skos:related rdf:resource="descriptor/19559-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11404-0">
    <skos:related rdf:resource="descriptor/11377-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19369-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13569-0">
    <skos:narrower rdf:resource="descriptor/16171-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13569-0">
    <skos:narrower rdf:resource="descriptor/18591-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13569-0">
    <skos:narrower rdf:resource="descriptor/13647-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13569-0">
    <skos:narrower rdf:resource="descriptor/16170-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13569-0">
    <skos:narrower rdf:resource="descriptor/13502-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13569-0">
    <skos:narrower rdf:resource="descriptor/18567-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13569-0">
    <skos:narrower rdf:resource="descriptor/13565-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13569-0">
    <skos:narrower rdf:resource="descriptor/16166-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14975-5">
    <skos:broader rdf:resource="thsys/73300"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14975-5">
    <skos:broader rdf:resource="descriptor/14971-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18140-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18150-5">
    <skos:broader rdf:resource="thsys/71062"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18150-5">
    <skos:broader rdf:resource="thsys/71060"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18150-5">
    <skos:broader rdf:resource="descriptor/19495-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18009-6">
    <skos:scopeNote xml:lang="de">Kraftstoff aus nachwachsenden Rohstoffen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16956-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14017-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18722-0">
    <skos:prefLabel xml:lang="en">Capital adequacy rules</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18722-0">
    <skos:prefLabel xml:lang="de">Eigenkapitalvorschriften</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11734-0">
    <skos:scopeNote xml:lang="de">Hypothese von der Verdrängung privater durch staatliche Wirtschaftsaktivitäten.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16830-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10713-6">
    <skos:narrower rdf:resource="descriptor/10628-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10713-6">
    <skos:narrower rdf:resource="descriptor/10714-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10713-6">
    <skos:narrower rdf:resource="descriptor/19331-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11389-5">
    <skos:scopeNote xml:lang="de">Alle Formen des beruflichen Wiederbeginns nach Arbeitslosigkeit, Familienpause, Krankheit sowie der Arbeitsmarktintegration von Zuwanderern usw. Ausmaß der Integration einzelner Gruppen sowie staatliche Eingliederungshilfen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17677-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20810-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19634-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13478-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70318">
    <rdfs:label xml:lang="de">Maschinen</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19752-0">
    <skos:related rdf:resource="descriptor/16595-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19752-0">
    <skos:related rdf:resource="descriptor/16642-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19752-0">
    <skos:related rdf:resource="descriptor/15712-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19752-0">
    <skos:related rdf:resource="descriptor/16651-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19752-0">
    <skos:related rdf:resource="descriptor/19680-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19752-0">
    <skos:related rdf:resource="descriptor/19773-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19752-0">
    <skos:related rdf:resource="descriptor/16587-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19752-0">
    <skos:related rdf:resource="descriptor/19521-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21061-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19079-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11609-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19218-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70091">
    <skos:prefLabel xml:lang="de">V.08  Entwicklungsökonomik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14787-6">
    <skos:related rdf:resource="descriptor/13113-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13459-0">
    <skos:altLabel xml:lang="de">Wirtschaftslehre des Handels</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13459-0">
    <skos:altLabel xml:lang="de">Theorie des Handelsbetriebs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13459-0">
    <skos:altLabel xml:lang="de">Handelsverkehrslehre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19367-3">
    <skos:related rdf:resource="descriptor/12912-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19367-3">
    <skos:related rdf:resource="descriptor/18987-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19367-3">
    <skos:related rdf:resource="descriptor/14150-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19367-3">
    <skos:related rdf:resource="descriptor/12961-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70554">
    <skos:narrower rdf:resource="descriptor/16825-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70554">
    <skos:narrower rdf:resource="descriptor/16824-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70554">
    <skos:narrower rdf:resource="thsys/70438"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70554">
    <skos:narrower rdf:resource="thsys/70361"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70554">
    <skos:narrower rdf:resource="thsys/70003"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13293-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18768-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12247-6">
    <skos:related rdf:resource="descriptor/19275-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13027-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70900">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73374">
    <rdfs:label xml:lang="de">Felder der Politik</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15315-0">
    <skos:related rdf:resource="descriptor/15062-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18566-2">
    <skos:altLabel xml:lang="de">Schedule</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10219-0">
    <skos:prefLabel xml:lang="en">Spatial price theory</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10219-0">
    <skos:prefLabel xml:lang="de">Räumliche Preistheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16011-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19760-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12998-3">
    <skos:related rdf:resource="descriptor/14743-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16615-4">
    <skos:prefLabel xml:lang="en">Social integration</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16615-4">
    <skos:prefLabel xml:lang="de">Soziale Integration</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10246-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18888-1">
    <skos:related rdf:resource="descriptor/19584-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18888-1">
    <skos:related rdf:resource="descriptor/11367-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18888-1">
    <skos:related rdf:resource="descriptor/11354-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10962-6">
    <skos:prefLabel xml:lang="en">Competition authority</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10962-6">
    <skos:prefLabel xml:lang="de">Wettbewerbsaufsicht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id335">
    <rdf:value xml:lang="de">Für Untersuchungen zum Verhalten von Urlaubern</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14522-0">
    <skos:related rdf:resource="descriptor/13084-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14522-0">
    <skos:related rdf:resource="descriptor/13228-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14522-0">
    <skos:related rdf:resource="descriptor/14581-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14522-0">
    <skos:related rdf:resource="descriptor/18004-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14522-0">
    <skos:related rdf:resource="descriptor/14573-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14522-0">
    <skos:related rdf:resource="descriptor/18647-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18939-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10494-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19425-1">
    <skos:related rdf:resource="descriptor/13767-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19062-4">
    <skos:broader rdf:resource="thsys/70962"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19062-4">
    <skos:broader rdf:resource="descriptor/13209-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13509-4">
    <skos:broader rdf:resource="thsys/71091"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13509-4">
    <skos:broader rdf:resource="thsys/70952"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13509-4">
    <skos:broader rdf:resource="thsys/70943"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13278-4">
    <skos:editorialNote xml:lang="de">Möglichst spezifischer indexieren.</skos:editorialNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18370-5">
    <skos:related rdf:resource="descriptor/19727-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12042-3">
    <skos:prefLabel xml:lang="en">Office</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12042-3">
    <skos:prefLabel xml:lang="de">Büro</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13670-4">
    <skos:altLabel xml:lang="de">Wochenzeitung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13670-4">
    <skos:altLabel xml:lang="de">Tageszeitung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13670-4">
    <skos:altLabel xml:lang="de">Tagespresse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13670-4">
    <skos:altLabel xml:lang="de">Sonntagszeitung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13670-4">
    <skos:altLabel xml:lang="de">Regionalzeitung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13670-4">
    <skos:altLabel xml:lang="de">Regionalpresse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13670-4">
    <skos:altLabel xml:lang="de">Lokalzeitung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13670-4">
    <skos:altLabel xml:lang="de">Lokalpresse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13537-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18963-6">
    <skos:prefLabel xml:lang="en">Plastics recycling</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18963-6">
    <skos:prefLabel xml:lang="de">Kunststoff-Recycling</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11870-6">
    <skos:prefLabel xml:lang="en">Regional planning law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11870-6">
    <skos:prefLabel xml:lang="de">Raumordnungsrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19418-5">
    <skos:prefLabel xml:lang="en">Cash-flow tax</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19418-5">
    <skos:prefLabel xml:lang="de">Cash-Flow-Steuer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13242-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12057-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13814-6">
    <skos:prefLabel xml:lang="en">Air transport insurance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13814-6">
    <skos:prefLabel xml:lang="de">Luftfahrtversicherung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15954-1">
    <skos:narrower rdf:resource="descriptor/15956-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15954-1">
    <skos:narrower rdf:resource="descriptor/10613-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15954-1">
    <skos:narrower rdf:resource="descriptor/15952-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15954-1">
    <skos:narrower rdf:resource="descriptor/15951-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70649">
    <skos:prefLabel xml:lang="de">P.04.03  Kraftstoffe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19828-0">
    <skos:prefLabel xml:lang="en">Value at risk</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19828-0">
    <skos:prefLabel xml:lang="de">Value at Risk</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18326-1">
    <skos:prefLabel xml:lang="en">Irrigation farming</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18326-1">
    <skos:prefLabel xml:lang="de">Bewässerungslandwirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18966-0">
    <skos:broader rdf:resource="thsys/71055"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18966-0">
    <skos:broader rdf:resource="thsys/70815"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18966-0">
    <skos:broader rdf:resource="descriptor/18535-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14743-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17184-0">
    <skos:altLabel xml:lang="x-other">Elliniki Dimokratia</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17184-0">
    <skos:altLabel xml:lang="de">Republik Griechenland</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17184-0">
    <skos:altLabel xml:lang="de">Hellas</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70651">
    <skos:narrower rdf:resource="descriptor/18231-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70651">
    <skos:narrower rdf:resource="descriptor/14189-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70651">
    <skos:narrower rdf:resource="descriptor/14186-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70651">
    <skos:narrower rdf:resource="descriptor/18024-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70651">
    <skos:narrower rdf:resource="descriptor/14180-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70651">
    <skos:narrower rdf:resource="descriptor/18203-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70651">
    <skos:narrower rdf:resource="descriptor/14171-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73373">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13293-1">
    <skos:altLabel xml:lang="en">Loan insurance</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13293-1">
    <skos:altLabel xml:lang="en">Debt insurance</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13293-1">
    <skos:altLabel xml:lang="de">Restschuldversicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18810-3">
    <skos:related rdf:resource="descriptor/10987-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18810-3">
    <skos:related rdf:resource="descriptor/11237-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14928-0">
    <skos:related rdf:resource="descriptor/14973-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16489-4">
    <skos:narrower rdf:resource="descriptor/19717-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16489-4">
    <skos:narrower rdf:resource="descriptor/10603-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16489-4">
    <skos:narrower rdf:resource="descriptor/16496-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16489-4">
    <skos:narrower rdf:resource="descriptor/10555-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16489-4">
    <skos:narrower rdf:resource="descriptor/13407-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13100-3">
    <skos:altLabel xml:lang="de">Industrielle Buchbinderei</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13100-3">
    <skos:altLabel xml:lang="de">Buchbinderhandwerk</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11998-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11370-5">
    <skos:scopeNote xml:lang="de">Vermittlung von theoretischen und praktischen Kenntnissen, die zu einem akademischen Grad oder gleichwertigen Abschluss führt.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13727-1">
    <skos:related rdf:resource="descriptor/12259-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13727-1">
    <skos:related rdf:resource="descriptor/12282-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13727-1">
    <skos:related rdf:resource="descriptor/19079-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13727-1">
    <skos:related rdf:resource="descriptor/12273-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13727-1">
    <skos:related rdf:resource="descriptor/12174-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13336-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10344-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10301-1">
    <skos:broader rdf:resource="thsys/71112"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10301-1">
    <skos:broader rdf:resource="thsys/70493"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10301-1">
    <skos:broader rdf:resource="descriptor/10030-6"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id311">
    <rdf:value xml:lang="de">Für theoretische Literatur</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19738-1">
    <skos:prefLabel xml:lang="en">Confucianism</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19738-1">
    <skos:prefLabel xml:lang="de">Konfuzianismus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18978-0">
    <skos:altLabel xml:lang="de">Managerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18978-0">
    <skos:altLabel xml:lang="de">Geschäftsführerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18978-0">
    <skos:altLabel xml:lang="de">Generaldirektorinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18978-0">
    <skos:altLabel xml:lang="de">Direktorinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18978-0">
    <skos:altLabel xml:lang="de">Betriebsleiterinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18090-4">
    <skos:prefLabel xml:lang="en">Tunnel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18090-4">
    <skos:prefLabel xml:lang="de">Tunnel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10460-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70933">
    <skos:broader rdf:resource="thsys/70936"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20449-5">
    <skos:narrower rdf:resource="descriptor/19956-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19220-5">
    <skos:related rdf:resource="descriptor/11037-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70080">
    <skos:broader rdf:resource="thsys/70026"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15801-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11753-3">
    <skos:related rdf:resource="descriptor/11751-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11753-3">
    <skos:related rdf:resource="descriptor/15033-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11753-3">
    <skos:related rdf:resource="descriptor/11735-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14317-1">
    <skos:altLabel xml:lang="de">Taucherglocke</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14317-1">
    <skos:altLabel xml:lang="de">Tauchboot</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14317-1">
    <skos:altLabel xml:lang="de">Offshore-Ausrüstung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14317-1">
    <skos:altLabel xml:lang="de">Meerestechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14317-1">
    <skos:altLabel xml:lang="de">Meeresbergbautechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14317-1">
    <skos:altLabel xml:lang="de">Bohrschiff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14317-1">
    <skos:altLabel xml:lang="de">Bohrplattform</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14317-1">
    <skos:altLabel xml:lang="de">Bohrinsel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15397-0">
    <skos:related rdf:resource="descriptor/15410-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11800-6">
    <skos:scopeNote xml:lang="de">Direkte Mengen- und Preisregulierungen für einzelne Agrarprodukte, z.B. Garantiemengenregelungen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25742-3">
    <skos:prefLabel xml:lang="en">Financing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25742-3">
    <skos:prefLabel xml:lang="de">Finanzierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15473-3">
    <skos:related rdf:resource="descriptor/16788-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15473-3">
    <skos:related rdf:resource="descriptor/16759-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15473-3">
    <skos:related rdf:resource="descriptor/15961-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15473-3">
    <skos:related rdf:resource="descriptor/10140-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15473-3">
    <skos:related rdf:resource="descriptor/16767-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19492-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16272-1">
    <skos:broader rdf:resource="thsys/73376"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17576-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13004-6">
    <skos:broader rdf:resource="thsys/70989"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13004-6">
    <skos:broader rdf:resource="descriptor/12913-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17989-0">
    <skos:broader rdf:resource="thsys/70307"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14084-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70536">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11814-2">
    <skos:altLabel xml:lang="en">Labor inspection</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12276-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18986-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12309-3">
    <skos:related rdf:resource="descriptor/12523-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12309-3">
    <skos:related rdf:resource="descriptor/11596-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12309-3">
    <skos:related rdf:resource="descriptor/12311-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12309-3">
    <skos:related rdf:resource="descriptor/12306-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16250-4">
    <skos:prefLabel xml:lang="en">Feudalism</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16250-4">
    <skos:prefLabel xml:lang="de">Feudalismus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10565-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19735-0">
    <skos:broader rdf:resource="thsys/73388"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19735-0">
    <skos:broader rdf:resource="descriptor/15682-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19466-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20399-1">
    <skos:broader rdf:resource="thsys/70582"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16950-4">
    <skos:altLabel xml:lang="de">Republik Österreich</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15918-5">
    <skos:narrower rdf:resource="descriptor/19643-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15918-5">
    <skos:narrower rdf:resource="descriptor/15929-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15918-5">
    <skos:narrower rdf:resource="descriptor/15935-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15918-5">
    <skos:narrower rdf:resource="descriptor/15923-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15918-5">
    <skos:narrower rdf:resource="descriptor/19642-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16687-0">
    <skos:prefLabel xml:lang="en">Economic sociology</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16687-0">
    <skos:prefLabel xml:lang="de">Wirtschaftssoziologie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15684-4">
    <skos:scopeNote xml:lang="de">Hier verstanden im weiteren Sinne als Wissenschaft für die Ausbildung im gewerblichen und kaufmännischen Bereich sowie als Wirtschaftserziehung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id344">
    <rdfs:seeAlso rdf:resource="descriptor/11032-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18280-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11974-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19643-5">
    <skos:altLabel xml:lang="en">Aboriginal people</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19643-5">
    <skos:altLabel xml:lang="de">Naturvölker</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19643-5">
    <skos:altLabel xml:lang="de">Eingeborener</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19643-5">
    <skos:altLabel xml:lang="de">Eingeborene</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11554-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16551-4">
    <skos:broader rdf:resource="thsys/73367"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19010-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18362-4">
    <skos:broader rdf:resource="thsys/71014"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18362-4">
    <skos:broader rdf:resource="thsys/71012"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18362-4">
    <skos:broader rdf:resource="thsys/70539"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18362-4">
    <skos:broader rdf:resource="descriptor/11810-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12962-3">
    <skos:altLabel xml:lang="de">Jagdbetriebslehre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13136-3">
    <skos:related rdf:resource="descriptor/13904-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19137-6">
    <skos:related rdf:resource="descriptor/12036-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19137-6">
    <skos:related rdf:resource="descriptor/19556-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19137-6">
    <skos:related rdf:resource="descriptor/19119-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19137-6">
    <skos:related rdf:resource="descriptor/18667-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19137-6">
    <skos:related rdf:resource="descriptor/18084-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19810-5">
    <skos:broader rdf:resource="thsys/73338"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15684-4">
    <skos:prefLabel xml:lang="en">Teaching of economics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15684-4">
    <skos:prefLabel xml:lang="de">Wirtschaftspädagogik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15769-3">
    <skos:prefLabel xml:lang="en">Environmental law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15769-3">
    <skos:prefLabel xml:lang="de">Umweltrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10164-6">
    <skos:narrower rdf:resource="descriptor/19210-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10164-6">
    <skos:narrower rdf:resource="descriptor/10162-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20261-2">
    <skos:broader rdf:resource="thsys/70321"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20261-2">
    <skos:broader rdf:resource="thsys/70028"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14808-3">
    <skos:broader rdf:resource="thsys/70328"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14808-3">
    <skos:broader rdf:resource="descriptor/18277-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14808-3">
    <skos:broader rdf:resource="descriptor/14803-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11469-0">
    <skos:broader rdf:resource="thsys/70306"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12023-0">
    <skos:altLabel xml:lang="de">Grundstücksspekulation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13867-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16475-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15902-6">
    <skos:prefLabel xml:lang="en">Economic geography</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15902-6">
    <skos:prefLabel xml:lang="de">Wirtschaftsgeographie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16440-6">
    <skos:related rdf:resource="descriptor/15463-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16440-6">
    <skos:related rdf:resource="descriptor/19606-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16440-6">
    <skos:related rdf:resource="descriptor/10107-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16440-6">
    <skos:related rdf:resource="descriptor/16288-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16440-6">
    <skos:related rdf:resource="descriptor/19715-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16440-6">
    <skos:related rdf:resource="descriptor/19151-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16440-6">
    <skos:related rdf:resource="descriptor/19708-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16440-6">
    <skos:related rdf:resource="descriptor/16497-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16440-6">
    <skos:related rdf:resource="descriptor/16493-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16038-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10721-0">
    <skos:broader rdf:resource="thsys/71095"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10721-0">
    <skos:broader rdf:resource="thsys/70904"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10721-0">
    <skos:broader rdf:resource="descriptor/10382-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10721-0">
    <skos:broader rdf:resource="descriptor/10682-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10817-1">
    <skos:altLabel xml:lang="en">Offshore investment</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10817-1">
    <skos:altLabel xml:lang="en">Offshore deposit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10817-1">
    <skos:altLabel xml:lang="en">Offshore bank</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10817-1">
    <skos:altLabel xml:lang="de">Offshore-Kapitalmarkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10817-1">
    <skos:altLabel xml:lang="de">Offshore-Geldmarkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10676-0">
    <skos:altLabel xml:lang="de">Zahlungsbilanztheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10676-0">
    <skos:altLabel xml:lang="de">Zahlungsbilanzanpassungsprozess</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10676-0">
    <skos:altLabel xml:lang="de">Zahlungsbilanzanpassungsprozeß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10676-0">
    <skos:altLabel xml:lang="de">Währungstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10676-0">
    <skos:altLabel xml:lang="de">Theorie des Zahlungsbilanzausgleichs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19217-1">
    <skos:related rdf:resource="descriptor/21061-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19217-1">
    <skos:related rdf:resource="descriptor/11529-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19217-1">
    <skos:related rdf:resource="descriptor/12376-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19217-1">
    <skos:related rdf:resource="descriptor/18176-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19217-1">
    <skos:related rdf:resource="descriptor/16053-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14514-6">
    <skos:broader rdf:resource="thsys/73305"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14514-6">
    <skos:broader rdf:resource="thsys/70254"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14514-6">
    <skos:broader rdf:resource="descriptor/18251-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14514-6">
    <skos:broader rdf:resource="descriptor/14752-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10220-1">
    <skos:broader rdf:resource="thsys/70178"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10220-1">
    <skos:broader rdf:resource="descriptor/10217-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10220-1">
    <skos:broader rdf:resource="descriptor/10213-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18660-3">
    <skos:broader rdf:resource="thsys/71037"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18660-3">
    <skos:broader rdf:resource="thsys/70922"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18660-3">
    <skos:broader rdf:resource="descriptor/18655-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18660-3">
    <skos:broader rdf:resource="descriptor/18175-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14371-2">
    <skos:altLabel xml:lang="de">Offsetmaschine</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14371-2">
    <skos:altLabel xml:lang="de">Fotosetzmaschine</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10948-0">
    <skos:broader rdf:resource="thsys/70242"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10948-0">
    <skos:broader rdf:resource="descriptor/10956-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10948-0">
    <skos:broader rdf:resource="descriptor/10905-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://zbw.eu/namespaces/skos-extensions/Thsys">
    <rdfs:subClassOf rdf:resource="http://www.w3.org/2004/02/skos/core#Concept"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11220-3">
    <skos:related rdf:resource="descriptor/11281-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11220-3">
    <skos:related rdf:resource="descriptor/10261-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11220-3">
    <skos:related rdf:resource="descriptor/11182-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19639-3">
    <skos:related rdf:resource="descriptor/16615-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19639-3">
    <skos:related rdf:resource="descriptor/15930-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19639-3">
    <skos:related rdf:resource="descriptor/19775-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19639-3">
    <skos:related rdf:resource="descriptor/15954-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19639-3">
    <skos:related rdf:resource="descriptor/15919-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19639-3">
    <skos:related rdf:resource="descriptor/15918-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19639-3">
    <skos:related rdf:resource="descriptor/15932-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19639-3">
    <skos:related rdf:resource="descriptor/15907-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19639-3">
    <skos:related rdf:resource="descriptor/19843-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15373-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11339-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70902">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73386">
    <rdfs:label xml:lang="en">Social psychology</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73386">
    <rdfs:label xml:lang="de">Sozialpsychologie</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10303-4">
    <skos:related rdf:resource="descriptor/10323-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10303-4">
    <skos:related rdf:resource="descriptor/15055-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10303-4">
    <skos:related rdf:resource="descriptor/10243-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10269-6">
    <skos:broader rdf:resource="thsys/71111"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10269-6">
    <skos:broader rdf:resource="descriptor/10272-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19220-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10776-3">
    <skos:related rdf:resource="descriptor/10773-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10776-3">
    <skos:related rdf:resource="descriptor/10770-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10776-3">
    <skos:related rdf:resource="descriptor/10764-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10776-3">
    <skos:related rdf:resource="descriptor/10785-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10776-3">
    <skos:related rdf:resource="descriptor/10781-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10776-3">
    <skos:related rdf:resource="descriptor/10780-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10776-3">
    <skos:related rdf:resource="descriptor/10779-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10776-3">
    <skos:related rdf:resource="descriptor/13757-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13294-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10469-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15842-5">
    <skos:narrower rdf:resource="descriptor/19009-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12035-0">
    <skos:related rdf:resource="descriptor/12755-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12035-0">
    <skos:related rdf:resource="descriptor/12656-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12035-0">
    <skos:related rdf:resource="descriptor/12640-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12035-0">
    <skos:related rdf:resource="descriptor/11576-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12035-0">
    <skos:related rdf:resource="descriptor/12716-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12035-0">
    <skos:related rdf:resource="descriptor/10002-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14759-4">
    <skos:altLabel xml:lang="en">Floppy disk</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14759-4">
    <skos:altLabel xml:lang="de">Plattenspeicher</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14759-4">
    <skos:altLabel xml:lang="de">Magnetplatte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14759-4">
    <skos:altLabel xml:lang="de">Magnetband</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14759-4">
    <skos:altLabel xml:lang="de">Festplatte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14759-4">
    <skos:altLabel xml:lang="de">Diskette</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12565-6">
    <skos:prefLabel xml:lang="en">Manpower planning</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12565-6">
    <skos:prefLabel xml:lang="de">Personalplanung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15914-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16594-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18658-4">
    <skos:broader rdf:resource="thsys/73341"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18658-4">
    <skos:broader rdf:resource="thsys/71038"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18658-4">
    <skos:broader rdf:resource="descriptor/15599-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13060-3">
    <skos:narrower rdf:resource="descriptor/13055-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70814">
    <skos:prefLabel xml:lang="de">W.34  Private Haushalte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12816-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10978-5">
    <skos:broader rdf:resource="thsys/73372"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10978-5">
    <skos:broader rdf:resource="thsys/70355"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10978-5">
    <skos:broader rdf:resource="thsys/70038"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10978-5">
    <skos:broader rdf:resource="descriptor/10987-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11452-3">
    <skos:altLabel xml:lang="de">Umlaufgeschwindigkeit des Geldes</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72180">
    <skos:prefLabel xml:lang="de">V.16  Statistik und Ökonometrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10094-1">
    <skos:prefLabel xml:lang="en">Internalizing externalities</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10094-1">
    <skos:prefLabel xml:lang="de">Internalisierung externer Effekte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12613-0">
    <skos:prefLabel xml:lang="en">Safety engineering</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12613-0">
    <skos:prefLabel xml:lang="de">Sicherheitstechnik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#relatedMatch">
    <rdfs:subPropertyOf rdf:resource="http://www.w3.org/2004/02/skos/core#related"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#relatedMatch">
    <rdfs:subPropertyOf rdf:resource="http://www.w3.org/2004/02/skos/core#mappingRelation"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18116-5">
    <skos:broader rdf:resource="thsys/73394"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18116-5">
    <skos:broader rdf:resource="thsys/73385"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18116-5">
    <skos:broader rdf:resource="thsys/70485"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18116-5">
    <skos:broader rdf:resource="descriptor/16709-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19301-5">
    <skos:narrower rdf:resource="descriptor/18521-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19301-5">
    <skos:narrower rdf:resource="descriptor/18299-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18657-6">
    <skos:prefLabel xml:lang="en">Open systems interconnection</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18657-6">
    <skos:prefLabel xml:lang="de">Offenes Computersystem</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73353">
    <skos:prefLabel xml:lang="de">N.10.01.01  Physische Geographie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10977-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11271-0">
    <skos:broader rdf:resource="thsys/70051"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11271-0">
    <skos:broader rdf:resource="descriptor/10239-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11271-0">
    <skos:broader rdf:resource="descriptor/15914-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16556-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13652-6">
    <skos:prefLabel xml:lang="en">Electronic mail</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13652-6">
    <skos:prefLabel xml:lang="de">Elektronische Post</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70131">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19313-5">
    <skos:prefLabel xml:lang="en">J-curve</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19313-5">
    <skos:prefLabel xml:lang="de">J-Kurve</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13218-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13248-6">
    <skos:broader rdf:resource="thsys/70937"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13248-6">
    <skos:broader rdf:resource="thsys/70883"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13248-6">
    <skos:broader rdf:resource="descriptor/19066-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19856-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21002-0">
    <skos:prefLabel xml:lang="en">Deductible (Insurance)</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21002-0">
    <skos:prefLabel xml:lang="de">Selbstbeteiligung (Versicherung)</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10644-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12371-5">
    <skos:altLabel xml:lang="de">GuV (Gewinn- und Verlustrechnung)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15013-2">
    <skos:broader rdf:resource="thsys/70650"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15013-2">
    <skos:broader rdf:resource="thsys/70161"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15013-2">
    <skos:broader rdf:resource="descriptor/15017-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12980-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15958-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70880">
    <skos:broader rdf:resource="thsys/70882"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19468-4">
    <skos:narrower rdf:resource="descriptor/11983-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17578-0">
    <skos:prefLabel xml:lang="en">Indonesia</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17578-0">
    <skos:prefLabel xml:lang="de">Indonesien</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12239-5">
    <skos:broader rdf:resource="thsys/73391"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12239-5">
    <skos:broader rdf:resource="thsys/71106"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12239-5">
    <skos:broader rdf:resource="thsys/71046"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12239-5">
    <skos:broader rdf:resource="descriptor/16043-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17980-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10552-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15836-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18743-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12733-4">
    <skos:broader rdf:resource="thsys/71036"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12733-4">
    <skos:broader rdf:resource="thsys/70471"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12733-4">
    <skos:broader rdf:resource="descriptor/12702-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70037">
    <rdfs:label xml:lang="de">Investition und Finanzierung</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id230">
    <rdf:value xml:lang="de">Für die Verstaatlichung von Unternehmen allgemein</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18543-0">
    <skos:broader rdf:resource="thsys/70951"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18543-0">
    <skos:broader rdf:resource="thsys/70384"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18543-0">
    <skos:broader rdf:resource="descriptor/12846-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18543-0">
    <skos:broader rdf:resource="descriptor/10876-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19569-5">
    <skos:prefLabel xml:lang="en">Bootstrap method</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19569-5">
    <skos:prefLabel xml:lang="de">Bootstrap-Verfahren</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10196-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13151-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12668-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16542-5">
    <skos:scopeNote xml:lang="de">Nur für allgemeine Literatur über soziale Sicherungssysteme.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16761-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19841-1">
    <skos:related rdf:resource="descriptor/15971-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19841-1">
    <skos:related rdf:resource="descriptor/19607-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19841-1">
    <skos:related rdf:resource="descriptor/15966-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19841-1">
    <skos:related rdf:resource="descriptor/16315-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14266-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19303-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16954-3">
    <skos:prefLabel xml:lang="en">Vorarlberg</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16954-3">
    <skos:prefLabel xml:lang="de">Vorarlberg</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16676-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19114-4">
    <skos:altLabel xml:lang="de">Computergestützte Logistik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11432-2">
    <skos:broader rdf:resource="thsys/71112"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11432-2">
    <skos:broader rdf:resource="descriptor/11427-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13334-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70556">
    <skos:broader rdf:resource="thsys/70014"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17727-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14485-1">
    <skos:broader rdf:resource="thsys/70135"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14485-1">
    <skos:broader rdf:resource="descriptor/14435-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14485-1">
    <skos:broader rdf:resource="descriptor/18421-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14485-1">
    <skos:broader rdf:resource="descriptor/14473-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70305">
    <skos:prefLabel xml:lang="en">V.05.02  Monetary policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70305">
    <skos:prefLabel xml:lang="de">V.05.02  Geldpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16954-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13861-4">
    <skos:scopeNote xml:lang="de">Erforschung von Verflechtungen sozialer und räumlicher Organisation und deren Wandel sowie der gesellschaftlichen und kulturellen Bedingtheit des Wohnens; z.B. Entstehung neuer Wohn- und Lebensformen, Wohnen von Minderheiten.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13338-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16028-5">
    <skos:altLabel xml:lang="en">Legacy</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16028-5">
    <skos:altLabel xml:lang="en">Bequest</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16028-5">
    <skos:altLabel xml:lang="de">Erbschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14032-3">
    <skos:broader rdf:resource="thsys/70201"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18375-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15983-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13264-1">
    <skos:narrower rdf:resource="descriptor/18565-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13264-1">
    <skos:narrower rdf:resource="descriptor/18584-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13264-1">
    <skos:narrower rdf:resource="descriptor/13252-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13264-1">
    <skos:narrower rdf:resource="descriptor/11629-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13264-1">
    <skos:narrower rdf:resource="descriptor/13247-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13264-1">
    <skos:narrower rdf:resource="descriptor/18603-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13264-1">
    <skos:narrower rdf:resource="descriptor/18615-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13264-1">
    <skos:narrower rdf:resource="descriptor/18604-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13264-1">
    <skos:narrower rdf:resource="descriptor/18572-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13027-1">
    <skos:related rdf:resource="descriptor/14279-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19139-2">
    <skos:broader rdf:resource="thsys/70459"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12287-1">
    <skos:broader rdf:resource="thsys/70905"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12287-1">
    <skos:broader rdf:resource="thsys/70187"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16283-3">
    <skos:prefLabel xml:lang="en">Political reform</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16283-3">
    <skos:prefLabel xml:lang="de">Politische Reform</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19005-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14792-6">
    <skos:altLabel xml:lang="de">Rattan</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14792-6">
    <skos:altLabel xml:lang="de">Pinsel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14792-6">
    <skos:altLabel xml:lang="de">Pflanzlicher Flechtstoff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14792-6">
    <skos:altLabel xml:lang="de">Korbwaren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14792-6">
    <skos:altLabel xml:lang="de">Bürste</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14792-6">
    <skos:altLabel xml:lang="de">Besen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14792-6">
    <skos:altLabel xml:lang="de">Bast</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14792-6">
    <skos:altLabel xml:lang="de">Bambus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11552-6">
    <skos:related rdf:resource="descriptor/19372-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11552-6">
    <skos:related rdf:resource="descriptor/18806-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11552-6">
    <skos:related rdf:resource="descriptor/12004-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18838-2">
    <skos:related rdf:resource="descriptor/16210-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18838-2">
    <skos:related rdf:resource="descriptor/18841-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18838-2">
    <skos:related rdf:resource="descriptor/12180-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10805-1">
    <skos:broader rdf:resource="thsys/71089"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10805-1">
    <skos:broader rdf:resource="descriptor/25742-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11809-2">
    <skos:altLabel xml:lang="en">Rural reform</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18117-3">
    <skos:altLabel xml:lang="en">Synergetics</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18117-3">
    <skos:altLabel xml:lang="de">Verbundeffekt (Synergie)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18117-3">
    <skos:altLabel xml:lang="de">Synergiemanagement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18117-3">
    <skos:altLabel xml:lang="de">Synergieeffekt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18117-3">
    <skos:altLabel xml:lang="de">Synergetik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14581-5">
    <skos:prefLabel xml:lang="en">Toys</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14581-5">
    <skos:prefLabel xml:lang="de">Spielwaren</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18344-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20176-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13020-1">
    <skos:narrower rdf:resource="descriptor/13023-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13020-1">
    <skos:narrower rdf:resource="descriptor/13014-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13020-1">
    <skos:narrower rdf:resource="descriptor/13011-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13145-2">
    <skos:prefLabel xml:lang="en">Textile industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13145-2">
    <skos:prefLabel xml:lang="de">Textilindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10347-5">
    <skos:narrower rdf:resource="descriptor/19239-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10347-5">
    <skos:narrower rdf:resource="descriptor/10353-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10347-5">
    <skos:narrower rdf:resource="descriptor/10350-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10347-5">
    <skos:narrower rdf:resource="descriptor/10349-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10347-5">
    <skos:narrower rdf:resource="descriptor/10348-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11713-1">
    <skos:altLabel xml:lang="de">Verteilungswirkung öffentlicher Haushalte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11713-1">
    <skos:altLabel xml:lang="de">Inzidenz des Haushalts</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70952">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19664-4">
    <skos:related rdf:resource="descriptor/10807-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19583-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16480-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15476-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18982-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10113-2">
    <skos:altLabel xml:lang="de">Wohlfahrtsökonomik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10113-2">
    <skos:altLabel xml:lang="de">Edgeworth-Kontraktkurve</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18747-5">
    <skos:related rdf:resource="descriptor/12031-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18747-5">
    <skos:related rdf:resource="descriptor/12029-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18747-5">
    <skos:related rdf:resource="descriptor/12028-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18747-5">
    <skos:related rdf:resource="descriptor/12018-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18747-5">
    <skos:related rdf:resource="descriptor/12905-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14512-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16700-6">
    <skos:related rdf:resource="descriptor/16630-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16700-6">
    <skos:related rdf:resource="descriptor/15924-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16700-6">
    <skos:related rdf:resource="descriptor/16192-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16700-6">
    <skos:related rdf:resource="descriptor/11200-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13255-2">
    <skos:narrower rdf:resource="descriptor/18089-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13255-2">
    <skos:narrower rdf:resource="descriptor/18574-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13255-2">
    <skos:narrower rdf:resource="descriptor/18573-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13197-4">
    <skos:altLabel xml:lang="de">Sortimentsbuchhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13197-4">
    <skos:altLabel xml:lang="de">Buchhandlung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13197-4">
    <skos:altLabel xml:lang="de">Buchgroßhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13197-4">
    <skos:altLabel xml:lang="de">Bucheinzelhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13067-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11184-2">
    <zbwext:useInsteadNote rdf:nodeID="id24"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11184-2">
    <zbwext:useInsteadNote rdf:nodeID="id23"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12273-5">
    <skos:narrower rdf:resource="descriptor/12296-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12273-5">
    <skos:narrower rdf:resource="descriptor/12270-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71102">
    <rdfs:label xml:lang="en">Market economy</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71102">
    <rdfs:label xml:lang="de">Marktwirtschaft</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13272-2">
    <skos:prefLabel xml:lang="en">Postal banking</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13272-2">
    <skos:prefLabel xml:lang="de">Postbankdienst</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16472-0">
    <skos:related rdf:resource="descriptor/19684-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16472-0">
    <skos:related rdf:resource="descriptor/16264-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16472-0">
    <skos:related rdf:resource="descriptor/19722-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16472-0">
    <skos:related rdf:resource="descriptor/16471-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15659-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19778-3">
    <skos:scopeNote xml:lang="de">Gesamtheit der spezifischen Wörter und Fachausdrücke eines Fachgebietes oder eines Berufs. Ebenso die Wissenschaft von der Terminologie. Häufig sind Terminologien international genormt, vor allem in Medizin, Naturwissenschaft und Technik.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20176-0">
    <skos:broader rdf:resource="thsys/70493"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20176-0">
    <skos:broader rdf:resource="descriptor/10030-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11811-1">
    <skos:prefLabel xml:lang="en">Energy saving</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11811-1">
    <skos:prefLabel xml:lang="de">Energiesparen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13987-3">
    <skos:prefLabel xml:lang="en">EDP personnel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13987-3">
    <skos:prefLabel xml:lang="de">EDV-Berufe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19364-2">
    <skos:broader rdf:resource="thsys/71086"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19364-2">
    <skos:broader rdf:resource="thsys/71067"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19097-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14194-5">
    <skos:related rdf:resource="descriptor/12981-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18607-0">
    <skos:related rdf:resource="descriptor/18296-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18607-0">
    <skos:related rdf:resource="descriptor/13980-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18607-0">
    <skos:related rdf:resource="descriptor/13814-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18607-0">
    <skos:related rdf:resource="descriptor/13575-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18607-0">
    <skos:related rdf:resource="descriptor/18418-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18607-0">
    <skos:related rdf:resource="descriptor/18604-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18607-0">
    <skos:related rdf:resource="descriptor/13516-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16642-1">
    <skos:broader rdf:resource="thsys/70120"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14817-2">
    <skos:altLabel xml:lang="de">Schaumgummiwaren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14817-2">
    <skos:altLabel xml:lang="de">Kautschukprodukt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18124-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16315-2">
    <skos:related rdf:resource="descriptor/16329-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16315-2">
    <skos:related rdf:resource="descriptor/15974-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16315-2">
    <skos:related rdf:resource="descriptor/15971-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16315-2">
    <skos:related rdf:resource="descriptor/19841-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16315-2">
    <skos:related rdf:resource="descriptor/16338-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16315-2">
    <skos:related rdf:resource="descriptor/16314-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16315-2">
    <skos:related rdf:resource="descriptor/16246-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15555-1">
    <skos:broader rdf:resource="thsys/71035"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15555-1">
    <skos:broader rdf:resource="descriptor/15483-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19173-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11288-4">
    <skos:altLabel xml:lang="de">Lehrlinge</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19244-5">
    <skos:related rdf:resource="descriptor/12619-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19244-5">
    <skos:related rdf:resource="descriptor/10459-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19244-5">
    <skos:related rdf:resource="descriptor/10446-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14522-0">
    <skos:altLabel xml:lang="de">Unterhaltungselektronisches Produkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14522-0">
    <skos:altLabel xml:lang="de">Konsumelektronik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14522-0">
    <skos:altLabel xml:lang="de">Heimelektronik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14522-0">
    <skos:altLabel xml:lang="de">Antenne</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19187-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70091">
    <skos:narrower rdf:resource="descriptor/10506-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70091">
    <skos:narrower rdf:resource="descriptor/10512-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70091">
    <skos:narrower rdf:resource="descriptor/18314-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70091">
    <skos:narrower rdf:resource="descriptor/10514-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70091">
    <skos:narrower rdf:resource="descriptor/10511-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70091">
    <skos:narrower rdf:resource="descriptor/19363-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70091">
    <skos:narrower rdf:resource="descriptor/10510-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70091">
    <skos:narrower rdf:resource="descriptor/15720-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70091">
    <skos:narrower rdf:resource="descriptor/10508-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70091">
    <skos:narrower rdf:resource="descriptor/19550-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70091">
    <skos:narrower rdf:resource="descriptor/10503-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70091">
    <skos:narrower rdf:resource="descriptor/10575-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70091">
    <skos:narrower rdf:resource="descriptor/19362-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70091">
    <skos:narrower rdf:resource="descriptor/10504-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70091">
    <skos:narrower rdf:resource="descriptor/10501-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70091">
    <skos:narrower rdf:resource="descriptor/18673-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70091">
    <skos:narrower rdf:resource="descriptor/10500-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70091">
    <skos:narrower rdf:resource="descriptor/19840-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70091">
    <skos:narrower rdf:resource="descriptor/10498-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70091">
    <skos:narrower rdf:resource="descriptor/20817-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70091">
    <skos:narrower rdf:resource="descriptor/10495-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70091">
    <skos:narrower rdf:resource="descriptor/10492-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70091">
    <skos:narrower rdf:resource="descriptor/19242-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70091">
    <skos:narrower rdf:resource="descriptor/10494-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70091">
    <skos:narrower rdf:resource="descriptor/19361-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70091">
    <skos:narrower rdf:resource="descriptor/10513-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70091">
    <skos:narrower rdf:resource="descriptor/10522-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70091">
    <skos:narrower rdf:resource="descriptor/10491-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70091">
    <skos:narrower rdf:resource="descriptor/10488-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70091">
    <skos:narrower rdf:resource="descriptor/10486-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70091">
    <skos:narrower rdf:resource="thsys/71086"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70091">
    <skos:narrower rdf:resource="thsys/71084"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70091">
    <skos:narrower rdf:resource="thsys/71085"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13592-5">
    <skos:broader rdf:resource="thsys/73354"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13592-5">
    <skos:broader rdf:resource="thsys/70924"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13592-5">
    <skos:broader rdf:resource="descriptor/13594-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16295-3">
    <skos:prefLabel xml:lang="en">National crisis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16295-3">
    <skos:prefLabel xml:lang="de">Staatskrise</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12241-4">
    <skos:broader rdf:resource="thsys/71045"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18054-1">
    <skos:related rdf:resource="descriptor/17179-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18054-1">
    <skos:related rdf:resource="descriptor/17016-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18054-1">
    <skos:related rdf:resource="descriptor/17750-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18054-1">
    <skos:related rdf:resource="descriptor/17608-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18054-1">
    <skos:related rdf:resource="descriptor/17962-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14814-1">
    <skos:related rdf:resource="descriptor/19613-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14814-1">
    <skos:related rdf:resource="descriptor/11957-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10103-5">
    <skos:broader rdf:resource="thsys/70057"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10103-5">
    <skos:broader rdf:resource="descriptor/10113-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11809-2">
    <skos:related rdf:resource="descriptor/10505-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11809-2">
    <skos:related rdf:resource="descriptor/11803-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13811-5">
    <skos:broader rdf:resource="thsys/70892"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13811-5">
    <skos:broader rdf:resource="descriptor/13828-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13811-5">
    <skos:broader rdf:resource="descriptor/12064-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14310-1">
    <skos:prefLabel xml:lang="en">Tin</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14310-1">
    <skos:prefLabel xml:lang="de">Zinn</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14083-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11778-1">
    <skos:broader rdf:resource="thsys/71113"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11778-1">
    <skos:broader rdf:resource="descriptor/11779-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12509-2">
    <skos:related rdf:resource="descriptor/12491-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14302-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73298">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18160-2">
    <skos:broader rdf:resource="thsys/71070"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18160-2">
    <skos:broader rdf:resource="thsys/71067"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18160-2">
    <skos:broader rdf:resource="thsys/70179"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18160-2">
    <skos:broader rdf:resource="descriptor/10895-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15543-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18661-1">
    <skos:altLabel xml:lang="en">RISC (Reduced Instruction Set Computer)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18661-1">
    <skos:altLabel xml:lang="en">CRISP (Complex Reduced Instruction Set Process)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18661-1">
    <skos:altLabel xml:lang="en">CISC (Complex Instruction Set Computer)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18661-1">
    <skos:altLabel xml:lang="de">Systemarchitektur von Computern</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18661-1">
    <skos:altLabel xml:lang="de">Rechnerstruktur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18661-1">
    <skos:altLabel xml:lang="de">Rechnerkonfiguration</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18661-1">
    <skos:altLabel xml:lang="de">Rechnen mit vermindertem Befehlssatz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18661-1">
    <skos:altLabel xml:lang="de">Computerarchitektur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12988-6">
    <skos:altLabel xml:lang="de">Nichteisenmetall-Bergbau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12988-6">
    <skos:altLabel xml:lang="de">Buntmetallerzbergbau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18341-5">
    <skos:altLabel xml:lang="de">Gemeinsame Fischereipolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73298">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10501-0">
    <skos:narrower rdf:resource="descriptor/18673-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11159-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18816-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11240-4">
    <skos:altLabel xml:lang="de">Schlechtwettergeld</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11240-4">
    <skos:altLabel xml:lang="de">Mutterschaftsgeld</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11240-4">
    <skos:altLabel xml:lang="de">Lohnersatzleistung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11240-4">
    <skos:altLabel xml:lang="de">Kurzarbeitergeld</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10350-2">
    <skos:related rdf:resource="descriptor/10359-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10350-2">
    <skos:related rdf:resource="descriptor/10434-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10350-2">
    <skos:related rdf:resource="descriptor/10521-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17946-4">
    <skos:prefLabel xml:lang="en">Solomon Islands</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17946-4">
    <skos:prefLabel xml:lang="de">Salomonen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10208-5">
    <skos:broader rdf:resource="thsys/71119"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10208-5">
    <skos:broader rdf:resource="descriptor/10205-4"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id313">
    <rdfs:seeAlso rdf:resource="descriptor/19581-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12065-5">
    <skos:prefLabel xml:lang="en">Production</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12065-5">
    <skos:prefLabel xml:lang="de">Produktion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70818">
    <rdfs:label xml:lang="de">Abfallwirtschaft und Recycling</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16406-6">
    <skos:related rdf:resource="descriptor/10166-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16406-6">
    <skos:related rdf:resource="descriptor/16379-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16406-6">
    <skos:related rdf:resource="descriptor/19219-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16406-6">
    <skos:related rdf:resource="descriptor/19754-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16406-6">
    <skos:related rdf:resource="descriptor/16411-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16406-6">
    <skos:related rdf:resource="descriptor/16426-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16406-6">
    <skos:related rdf:resource="descriptor/18832-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13437-3">
    <skos:related rdf:resource="descriptor/18329-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19060-1">
    <skos:prefLabel xml:lang="en">Grocery wholesale</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19060-1">
    <skos:prefLabel xml:lang="de">Lebensmittelgroßhandel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11430-6">
    <skos:prefLabel xml:lang="en">Gold standard</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11430-6">
    <skos:prefLabel xml:lang="de">Goldwährung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14515-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12781-0">
    <skos:broader rdf:resource="thsys/70956"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12781-0">
    <skos:broader rdf:resource="thsys/70459"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13574-0">
    <skos:altLabel xml:lang="de">Schienenverkehrstechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13574-0">
    <skos:altLabel xml:lang="de">Rad-Schiene-Technik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13574-0">
    <skos:altLabel xml:lang="de">Eisenbahnelektrifizierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13574-0">
    <skos:altLabel xml:lang="de">Bahntechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10541-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11186-5">
    <skos:altLabel xml:lang="en">Division of labor</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11186-5">
    <skos:altLabel xml:lang="de">Spezialisierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13292-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13274-5">
    <zbwext:useInsteadNote rdf:nodeID="id256"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12387-4">
    <skos:broader rdf:resource="thsys/71043"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11845-5">
    <skos:prefLabel xml:lang="en">Housing construction policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11845-5">
    <skos:prefLabel xml:lang="de">Wohnungsbaupolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11912-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13245-5">
    <skos:related rdf:resource="descriptor/18500-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13245-5">
    <skos:related rdf:resource="descriptor/14323-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19587-3">
    <skos:altLabel xml:lang="en">Business cycle statistics</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70154">
    <skos:narrower rdf:resource="descriptor/16233-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70154">
    <skos:narrower rdf:resource="descriptor/15977-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70154">
    <skos:narrower rdf:resource="descriptor/15976-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70154">
    <skos:narrower rdf:resource="descriptor/15972-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70154">
    <skos:narrower rdf:resource="descriptor/18068-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70154">
    <skos:narrower rdf:resource="descriptor/15971-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70154">
    <skos:narrower rdf:resource="descriptor/15970-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70154">
    <skos:narrower rdf:resource="descriptor/15969-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70154">
    <skos:narrower rdf:resource="descriptor/19607-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70154">
    <skos:narrower rdf:resource="descriptor/19213-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70154">
    <skos:narrower rdf:resource="descriptor/15968-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70154">
    <skos:narrower rdf:resource="descriptor/19728-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70154">
    <skos:narrower rdf:resource="descriptor/13995-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70154">
    <skos:narrower rdf:resource="descriptor/13389-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70154">
    <skos:narrower rdf:resource="descriptor/15966-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70154">
    <skos:narrower rdf:resource="descriptor/19662-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70154">
    <skos:narrower rdf:resource="descriptor/15965-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70154">
    <skos:narrower rdf:resource="descriptor/19822-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70154">
    <skos:narrower rdf:resource="descriptor/15962-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70154">
    <skos:narrower rdf:resource="descriptor/19841-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70154">
    <skos:narrower rdf:resource="descriptor/19819-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70154">
    <skos:narrower rdf:resource="descriptor/13410-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70154">
    <skos:narrower rdf:resource="descriptor/19768-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70154">
    <skos:narrower rdf:resource="descriptor/19820-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70154">
    <skos:narrower rdf:resource="thsys/73389"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70154">
    <skos:narrower rdf:resource="thsys/73335"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70154">
    <skos:narrower rdf:resource="thsys/73336"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70154">
    <skos:narrower rdf:resource="thsys/70160"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70154">
    <skos:narrower rdf:resource="thsys/73337"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70154">
    <skos:narrower rdf:resource="thsys/73333"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70154">
    <skos:narrower rdf:resource="thsys/73390"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70154">
    <skos:narrower rdf:resource="thsys/70241"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13192-0">
    <skos:prefLabel xml:lang="en">Pharmacy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13192-0">
    <skos:prefLabel xml:lang="de">Apotheke</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19361-1">
    <skos:altLabel xml:lang="de">Entwicklungsländerforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14310-1">
    <skos:broader rdf:resource="thsys/70134"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14310-1">
    <skos:broader rdf:resource="descriptor/14274-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14310-1">
    <skos:broader rdf:resource="descriptor/14302-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13555-4">
    <skos:related rdf:resource="descriptor/13508-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12960-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18896-2">
    <skos:altLabel xml:lang="de">Süchtiges Kaufverhalten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18896-2">
    <skos:altLabel xml:lang="de">Kaufrausch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13766-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18729-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10856-5">
    <skos:altLabel xml:lang="de">Firmenauslandsvertretung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10856-5">
    <skos:altLabel xml:lang="de">Exportvertretung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10856-5">
    <skos:altLabel xml:lang="de">Exportagentur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10856-5">
    <skos:altLabel xml:lang="de">Auslandsagentur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10856-5">
    <skos:altLabel xml:lang="de">Ausfuhragentur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10856-5">
    <skos:altLabel xml:lang="de">Außenhandelsvermittlung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15661-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18351-2">
    <skos:related rdf:resource="descriptor/19588-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18351-2">
    <skos:related rdf:resource="descriptor/12975-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18351-2">
    <skos:related rdf:resource="descriptor/18174-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18137-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11314-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19748-5">
    <skos:altLabel xml:lang="en">Peerage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19748-5">
    <skos:altLabel xml:lang="de">Aristokratie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11438-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18575-1">
    <skos:altLabel xml:lang="de">Eisenbahnreform</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18575-1">
    <skos:altLabel xml:lang="de">Bahnreform</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18575-1">
    <skos:altLabel xml:lang="de">Bahnpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19452-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19618-4">
    <skos:prefLabel xml:lang="en">Ocean floor</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19618-4">
    <skos:prefLabel xml:lang="de">Meeresboden</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18230-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18256-3">
    <skos:related rdf:resource="descriptor/14627-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18256-3">
    <skos:related rdf:resource="descriptor/12995-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18256-3">
    <skos:related rdf:resource="descriptor/12980-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10533-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11951-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18525-2">
    <skos:broader rdf:resource="thsys/73392"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18525-2">
    <skos:broader rdf:resource="thsys/70956"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18525-2">
    <skos:broader rdf:resource="descriptor/15982-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18525-2">
    <skos:broader rdf:resource="descriptor/12782-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11906-4">
    <skos:related rdf:resource="descriptor/18480-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11906-4">
    <skos:related rdf:resource="descriptor/11786-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11906-4">
    <skos:related rdf:resource="descriptor/18490-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11906-4">
    <skos:related rdf:resource="descriptor/18489-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71100">
    <skos:narrower rdf:resource="descriptor/10068-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71100">
    <skos:narrower rdf:resource="descriptor/10650-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71100">
    <skos:narrower rdf:resource="descriptor/10649-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71100">
    <skos:narrower rdf:resource="descriptor/10675-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71100">
    <skos:narrower rdf:resource="descriptor/10644-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71100">
    <skos:narrower rdf:resource="descriptor/10051-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71100">
    <skos:narrower rdf:resource="descriptor/10661-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71100">
    <skos:narrower rdf:resource="descriptor/19312-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71100">
    <skos:narrower rdf:resource="descriptor/10048-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71100">
    <skos:narrower rdf:resource="descriptor/10641-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71100">
    <skos:narrower rdf:resource="descriptor/10043-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71100">
    <skos:narrower rdf:resource="descriptor/10042-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71100">
    <skos:narrower rdf:resource="descriptor/10615-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71100">
    <skos:narrower rdf:resource="descriptor/10653-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71100">
    <skos:narrower rdf:resource="descriptor/10637-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71100">
    <skos:narrower rdf:resource="descriptor/10638-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71100">
    <skos:narrower rdf:resource="descriptor/19311-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71100">
    <skos:narrower rdf:resource="descriptor/10647-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71100">
    <skos:narrower rdf:resource="descriptor/10629-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71100">
    <skos:narrower rdf:resource="descriptor/10636-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71100">
    <skos:narrower rdf:resource="descriptor/10631-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71100">
    <skos:narrower rdf:resource="descriptor/10652-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71100">
    <skos:narrower rdf:resource="descriptor/10635-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71100">
    <skos:narrower rdf:resource="thsys/71099"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19634-6">
    <skos:altLabel xml:lang="de">Altertumsforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20077-2">
    <skos:broader rdf:resource="thsys/70274"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20077-2">
    <skos:broader rdf:resource="thsys/70096"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11559-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18993-4">
    <skos:altLabel xml:lang="en">Rural small-scale industry</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18993-4">
    <skos:altLabel xml:lang="de">Ländliches Handwerk</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18993-4">
    <skos:altLabel xml:lang="de">Ländliche Kleinindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18993-4">
    <skos:altLabel xml:lang="de">Landhandwerk</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18993-4">
    <skos:altLabel xml:lang="de">Dorfhandwerk</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19436-3">
    <skos:altLabel xml:lang="de">Lehre vom öffentlichen Haushalt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19436-3">
    <skos:altLabel xml:lang="de">Finanzhaushaltstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12297-5">
    <skos:broader rdf:resource="thsys/71043"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12297-5">
    <skos:broader rdf:resource="thsys/70036"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12297-5">
    <skos:broader rdf:resource="descriptor/12274-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16155-5">
    <skos:altLabel xml:lang="de">Schwerbehindertengesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24847-1">
    <skos:altLabel xml:lang="en">Organizational behavior</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24847-1">
    <skos:altLabel xml:lang="de">Organisationshandeln</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24847-1">
    <skos:altLabel xml:lang="de">Organisational behavior</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24740-5">
    <skos:altLabel xml:lang="de">Risikofonds</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24740-5">
    <skos:altLabel xml:lang="de">Hedge-Fonds</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24740-5">
    <skos:altLabel xml:lang="de">Hedge Fonds</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18065-3">
    <skos:related rdf:resource="descriptor/13392-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18065-3">
    <skos:related rdf:resource="descriptor/18066-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18065-3">
    <skos:related rdf:resource="descriptor/10481-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18065-3">
    <skos:related rdf:resource="descriptor/10478-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18065-3">
    <skos:related rdf:resource="descriptor/18805-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18065-3">
    <skos:related rdf:resource="descriptor/18525-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70160">
    <rdfs:label xml:lang="en">Public Law</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70160">
    <rdfs:label xml:lang="de">Öffentliches Recht</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12252-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71109">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14784-5">
    <skos:prefLabel xml:lang="en">Timber</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14784-5">
    <skos:prefLabel xml:lang="de">Bauholz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13415-6">
    <skos:scopeNote xml:lang="de">Vgl. auch Gesundheitswesen, Gesundheitsreform und Gesundheitsfinanzierung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11527-5">
    <skos:prefLabel xml:lang="en">Social discount rate</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11527-5">
    <skos:prefLabel xml:lang="de">Soziale Diskontrate</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24782-3">
    <skos:altLabel xml:lang="de">Neue Ökonomie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15654-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12964-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18488-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10286-6">
    <skos:broader rdf:resource="thsys/70234"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10286-6">
    <skos:broader rdf:resource="descriptor/10249-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10773-2">
    <zbwext:useInsteadNote rdf:nodeID="id359"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13003-1">
    <zbwext:useInsteadNote rdf:nodeID="id239"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18072-6">
    <skos:related rdf:resource="descriptor/15743-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18072-6">
    <skos:related rdf:resource="descriptor/14640-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70844">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.24.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12725-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19024-5">
    <skos:broader rdf:resource="thsys/70169"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19024-5">
    <skos:broader rdf:resource="descriptor/14105-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12050-4">
    <skos:altLabel xml:lang="de">Management-Holding</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12050-4">
    <skos:altLabel xml:lang="de">Konzern-Holding</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12050-4">
    <skos:altLabel xml:lang="de">Holdinggesellschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12050-4">
    <skos:altLabel xml:lang="de">Führungs-Holding</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12050-4">
    <skos:altLabel xml:lang="de">Finanz-Holding</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18922-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11881-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16305-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14934-5">
    <skos:prefLabel xml:lang="en">Sugar</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14934-5">
    <skos:prefLabel xml:lang="de">Zucker</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18917-6">
    <skos:scopeNote xml:lang="de">Vorwiegend mikroökonomische Marktbetrachtung von Freizeitgütern.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10685-6">
    <skos:broader rdf:resource="thsys/71098"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10685-6">
    <skos:broader rdf:resource="descriptor/19435-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10685-6">
    <skos:broader rdf:resource="descriptor/10706-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71058">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.13.05</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70095">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18346-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18783-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18115-0">
    <skos:prefLabel xml:lang="en">Succession planning</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18115-0">
    <skos:prefLabel xml:lang="de">Unternehmensnachfolge</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19681-4">
    <skos:related rdf:resource="descriptor/16314-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19681-4">
    <skos:related rdf:resource="descriptor/16437-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19426-6">
    <skos:broader rdf:resource="thsys/70992"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19426-6">
    <skos:broader rdf:resource="descriptor/13001-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18558-1">
    <skos:scopeNote xml:lang="de">Meist im Handel verwendeter Strichcode, der betriebsintern relevante Produktinformationen enthält.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12624-2">
    <skos:altLabel xml:lang="de">Personalzeiterfassungssystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12624-2">
    <skos:altLabel xml:lang="de">Gleitende Arbeitszeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25501-4">
    <skos:related rdf:resource="descriptor/15704-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25501-4">
    <skos:related rdf:resource="descriptor/10500-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73335">
    <skos:narrower rdf:resource="descriptor/16160-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73335">
    <skos:narrower rdf:resource="descriptor/16159-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73335">
    <skos:narrower rdf:resource="descriptor/16158-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73335">
    <skos:narrower rdf:resource="descriptor/13417-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73335">
    <skos:narrower rdf:resource="descriptor/16156-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73335">
    <skos:narrower rdf:resource="descriptor/16155-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73335">
    <skos:narrower rdf:resource="descriptor/18914-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73335">
    <skos:narrower rdf:resource="descriptor/16481-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73335">
    <skos:narrower rdf:resource="descriptor/18904-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14932-2">
    <skos:prefLabel xml:lang="en">Organic food</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14932-2">
    <skos:prefLabel xml:lang="de">Biokost</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10968-1">
    <skos:broader rdf:resource="thsys/73378"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10968-1">
    <skos:broader rdf:resource="thsys/70853"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10968-1">
    <skos:broader rdf:resource="thsys/70274"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10968-1">
    <skos:broader rdf:resource="thsys/70038"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19417-0">
    <skos:altLabel xml:lang="de">Wertpapiersteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19417-0">
    <skos:altLabel xml:lang="de">Börsensteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18891-5">
    <skos:prefLabel xml:lang="en">Health care</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18891-5">
    <skos:prefLabel xml:lang="de">Gesundheitsversorgung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14889-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10126-0">
    <skos:scopeNote xml:lang="de">Theorien zur Analyse der Bedeutung des Kapitals und der Kapitalbildung im Rahmen der Gesamtwirtschaft; enge Verbindung zur Produktions-, Verteilungs- und Wachstumstheorie.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10206-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18629-4">
    <skos:prefLabel xml:lang="en">Transport subsidy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18629-4">
    <skos:prefLabel xml:lang="de">Verkehrssubvention</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12255-0">
    <skos:broader rdf:resource="thsys/71045"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12255-0">
    <skos:broader rdf:resource="thsys/70040"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12255-0">
    <skos:broader rdf:resource="descriptor/12665-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12255-0">
    <skos:broader rdf:resource="descriptor/12251-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70305">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.05.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18833-5">
    <skos:prefLabel xml:lang="en">Management control in nonprofit organizations</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18833-5">
    <skos:prefLabel xml:lang="de">Nonprofit-Controlling</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13385-3">
    <skos:prefLabel xml:lang="en">Private security industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13385-3">
    <skos:prefLabel xml:lang="de">Sicherheitsgewerbe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15880-4">
    <skos:related rdf:resource="descriptor/15895-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15880-4">
    <skos:related rdf:resource="descriptor/19481-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15880-4">
    <skos:related rdf:resource="descriptor/15881-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13056-1">
    <skos:narrower rdf:resource="descriptor/13057-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70968">
    <skos:prefLabel xml:lang="en">W.07.02  Construction finishings and fittings</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70968">
    <skos:prefLabel xml:lang="de">W.07.02  Ausbaugewerbe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19399-4">
    <skos:related rdf:resource="descriptor/10987-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19399-4">
    <skos:related rdf:resource="descriptor/18694-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19399-4">
    <skos:related rdf:resource="descriptor/10865-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19399-4">
    <skos:related rdf:resource="descriptor/10552-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19399-4">
    <skos:related rdf:resource="descriptor/10854-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19399-4">
    <skos:related rdf:resource="descriptor/13430-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70987">
    <rdfs:label xml:lang="en">Textile and clothing industry</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70987">
    <rdfs:label xml:lang="de">Textil- und Bekleidungsindustrie</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13742-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10851-1">
    <skos:related rdf:resource="descriptor/10882-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10851-1">
    <skos:related rdf:resource="descriptor/10876-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10851-1">
    <skos:related rdf:resource="descriptor/18794-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10851-1">
    <skos:related rdf:resource="descriptor/10604-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10851-1">
    <skos:related rdf:resource="descriptor/18544-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70035">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11646-4">
    <skos:prefLabel xml:lang="en">Alcohol tax</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11646-4">
    <skos:prefLabel xml:lang="de">Alkoholsteuer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11822-3">
    <skos:broader rdf:resource="thsys/70963"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11822-3">
    <skos:broader rdf:resource="thsys/70852"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11822-3">
    <skos:broader rdf:resource="descriptor/10966-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18184-2">
    <skos:prefLabel xml:lang="en">Strategic control</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18184-2">
    <skos:prefLabel xml:lang="de">Strategisches Controlling</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16726-2">
    <skos:broader rdf:resource="thsys/73393"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16726-2">
    <skos:broader rdf:resource="descriptor/16732-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16726-2">
    <skos:broader rdf:resource="descriptor/16727-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14044-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19437-1">
    <skos:broader rdf:resource="thsys/70058"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19437-1">
    <skos:broader rdf:resource="descriptor/11917-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19437-1">
    <skos:broader rdf:resource="descriptor/18489-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70235">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13159-5">
    <skos:broader rdf:resource="thsys/70986"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13159-5">
    <skos:broader rdf:resource="descriptor/13160-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13306-4">
    <skos:broader rdf:resource="thsys/70944"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13306-4">
    <skos:broader rdf:resource="descriptor/13307-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11990-3">
    <skos:altLabel xml:lang="de">Stadtökonomie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15765-4">
    <skos:altLabel xml:lang="en">Environmental research</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15765-4">
    <skos:altLabel xml:lang="de">Umweltwissenschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15765-4">
    <skos:altLabel xml:lang="de">Umweltschutzforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15765-4">
    <skos:altLabel xml:lang="de">Ökologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14924-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16280-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14003-3">
    <skos:altLabel xml:lang="de">Übersetzungswissenschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14003-3">
    <skos:altLabel xml:lang="de">Übersetzungsdienst</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14003-3">
    <skos:altLabel xml:lang="de">Translationswissenschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14003-3">
    <skos:altLabel xml:lang="de">Dolmetscherinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14003-3">
    <skos:altLabel xml:lang="de">Dolmetscher</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12956-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18884-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10734-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11552-6">
    <skos:narrower rdf:resource="descriptor/18813-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11552-6">
    <skos:narrower rdf:resource="descriptor/11222-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14043-5">
    <skos:narrower rdf:resource="descriptor/18276-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14043-5">
    <skos:narrower rdf:resource="descriptor/14787-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14043-5">
    <skos:narrower rdf:resource="descriptor/18275-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19776-0">
    <skos:scopeNote xml:lang="de">Wissenschaft, die sich mit der Fähigkeit zur (überzeugenden) Argumentation beschäftigt. Auch wissenschaftstheoretische Analyse von Stilistik/Argumentationsfiguren/Metaphern in der Wissenschaft oder einzelnen Disziplinen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10555-5">
    <skos:altLabel xml:lang="en">Foreign assistance</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10555-5">
    <skos:altLabel xml:lang="en">Foreign aid</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10555-5">
    <skos:altLabel xml:lang="en">Development assistance</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10555-5">
    <skos:altLabel xml:lang="de">Öffentliche Entwicklungshilfe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10555-5">
    <skos:altLabel xml:lang="de">Entwicklungszusammenarbeit (Entwicklungshilfe)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10555-5">
    <skos:altLabel xml:lang="de">Entwicklungspolitik der Geberländer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10555-5">
    <skos:altLabel xml:lang="de">Entwicklungshilfepolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71015">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16969-4">
    <skos:prefLabel xml:lang="en">Obwalden</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16969-4">
    <skos:prefLabel xml:lang="de">Obwalden</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13444-6">
    <skos:altLabel xml:lang="de">Landwirtschaftsgeschichte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10241-0">
    <skos:broader rdf:resource="thsys/70234"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10241-0">
    <skos:broader rdf:resource="descriptor/10025-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10241-0">
    <skos:broader rdf:resource="descriptor/10000-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13052-2">
    <skos:broader rdf:resource="thsys/70964"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13052-2">
    <skos:broader rdf:resource="thsys/70961"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13052-2">
    <skos:broader rdf:resource="descriptor/12910-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16233-4">
    <skos:altLabel xml:lang="en">State theory</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16233-4">
    <skos:altLabel xml:lang="de">Staatsziel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16233-4">
    <skos:altLabel xml:lang="de">Staatswissenschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16233-4">
    <skos:altLabel xml:lang="de">Staatsrechtslehre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16233-4">
    <skos:altLabel xml:lang="de">Staatsphilosophie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16233-4">
    <skos:altLabel xml:lang="de">Staatslehre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16233-4">
    <skos:altLabel xml:lang="de">Staatsidee</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16233-4">
    <skos:altLabel xml:lang="de">Staatsformenlehre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18506-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10979-3">
    <skos:related rdf:resource="descriptor/12064-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10979-3">
    <skos:related rdf:resource="descriptor/11628-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18609-3">
    <skos:prefLabel xml:lang="en">Cross-border passenger traffic</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18609-3">
    <skos:prefLabel xml:lang="de">Grenzüberschreitender Personenverkehr</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70905">
    <rdfs:label xml:lang="de">Finanz- und Kreditwesen</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18080-0">
    <skos:prefLabel xml:lang="en">Chlorine</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18080-0">
    <skos:prefLabel xml:lang="de">Chlor</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16719-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13429-2">
    <skos:broader rdf:resource="thsys/71024"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13429-2">
    <skos:broader rdf:resource="thsys/70854"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13429-2">
    <skos:broader rdf:resource="descriptor/18313-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13429-2">
    <skos:broader rdf:resource="descriptor/12180-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15744-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13460-1">
    <skos:scopeNote xml:lang="de">Für wirtschaftswissenschaftliche Bearbeitungen aller Aspekte des Handels.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11151-3">
    <skos:prefLabel xml:lang="en">Collectivized agriculture</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11151-3">
    <skos:prefLabel xml:lang="de">Sozialistische Landwirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70014">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12103-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18094-3">
    <skos:altLabel xml:lang="de">Textilwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14611-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10908-5">
    <skos:broader rdf:resource="thsys/70101"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10908-5">
    <skos:broader rdf:resource="descriptor/19565-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10908-5">
    <skos:broader rdf:resource="descriptor/10895-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10020-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12257-3">
    <skos:altLabel xml:lang="de">Immobiliarkredit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12257-3">
    <skos:altLabel xml:lang="de">Hypothekarkredit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12257-3">
    <skos:altLabel xml:lang="de">Grundkredit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13724-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10553-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12278-2">
    <skos:narrower rdf:resource="descriptor/12705-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11531-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16484-0">
    <skos:narrower rdf:resource="descriptor/16150-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16484-0">
    <skos:narrower rdf:resource="descriptor/11347-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16484-0">
    <skos:narrower rdf:resource="descriptor/11345-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16484-0">
    <skos:narrower rdf:resource="descriptor/20455-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13773-1">
    <skos:altLabel xml:lang="en">Banking secrecy</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13773-1">
    <skos:altLabel xml:lang="de">Bankauskunft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13773-1">
    <skos:altLabel xml:lang="de">Auskunftspflicht von Banken</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70996">
    <skos:narrower rdf:resource="descriptor/13084-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70996">
    <skos:narrower rdf:resource="descriptor/18428-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70996">
    <skos:narrower rdf:resource="descriptor/13682-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70996">
    <skos:narrower rdf:resource="descriptor/13079-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70996">
    <skos:narrower rdf:resource="descriptor/10462-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70996">
    <skos:narrower rdf:resource="descriptor/18426-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70996">
    <skos:narrower rdf:resource="descriptor/13076-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70996">
    <skos:narrower rdf:resource="descriptor/13075-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70996">
    <skos:narrower rdf:resource="descriptor/13890-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70996">
    <skos:narrower rdf:resource="thsys/73297"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70996">
    <skos:narrower rdf:resource="thsys/70799"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18617-4">
    <skos:broader rdf:resource="thsys/70940"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18617-4">
    <skos:broader rdf:resource="descriptor/12013-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12587-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18174-5">
    <skos:related rdf:resource="descriptor/11812-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18174-5">
    <skos:related rdf:resource="descriptor/18351-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18174-5">
    <skos:related rdf:resource="descriptor/11810-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15405-6">
    <skos:narrower rdf:resource="descriptor/15350-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15405-6">
    <skos:narrower rdf:resource="descriptor/15336-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15405-6">
    <skos:narrower rdf:resource="descriptor/15330-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15405-6">
    <skos:narrower rdf:resource="descriptor/15399-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15405-6">
    <skos:narrower rdf:resource="descriptor/15398-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15405-6">
    <skos:narrower rdf:resource="descriptor/19542-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15405-6">
    <skos:narrower rdf:resource="descriptor/15397-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11354-3">
    <skos:altLabel xml:lang="de">Bildungsstand</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11354-3">
    <skos:altLabel xml:lang="de">Ausbildungsniveau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10072-4">
    <skos:scopeNote xml:lang="de">Menge aller zulässigen Allokationen, die durch keine Koalition verbessert werden kann bzw. Menge aller zulässigen Ergebnisse, die nicht dominiert.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24786-2">
    <skos:broader rdf:resource="thsys/70891"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24786-2">
    <skos:broader rdf:resource="descriptor/18739-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19607-2">
    <skos:altLabel xml:lang="de">Rechtssystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18510-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14920-2">
    <skos:altLabel xml:lang="de">Windjacke</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14920-2">
    <skos:altLabel xml:lang="de">Strandbekleidung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14920-2">
    <skos:altLabel xml:lang="de">Skibekleidung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14920-2">
    <skos:altLabel xml:lang="de">Parka</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14920-2">
    <skos:altLabel xml:lang="de">Freizeitbekleidung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14920-2">
    <skos:altLabel xml:lang="de">Blouson</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14920-2">
    <skos:altLabel xml:lang="de">Badebekleidung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14920-2">
    <skos:altLabel xml:lang="de">Anorak</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13127-4">
    <skos:altLabel xml:lang="de">Baumwollweberei</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13127-4">
    <skos:altLabel xml:lang="de">Baumwollwarenindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13127-4">
    <skos:altLabel xml:lang="de">Baumwolltextilienindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13127-4">
    <skos:altLabel xml:lang="de">Baumwollspinnerei</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13127-4">
    <skos:altLabel xml:lang="de">Baumwollgarnindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15666-6">
    <skos:narrower rdf:resource="descriptor/15684-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15666-6">
    <skos:narrower rdf:resource="descriptor/15674-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15666-6">
    <skos:narrower rdf:resource="descriptor/18868-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15648-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16834-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70461">
    <rdfs:label xml:lang="de">Agrochemikalien</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12575-3">
    <skos:broader rdf:resource="thsys/70245"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12575-3">
    <skos:broader rdf:resource="thsys/70244"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12575-3">
    <skos:broader rdf:resource="descriptor/12592-3"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id52">
    <rdf:value xml:lang="de">Für gesamtwirtschaftliche Analysen des Anlagevermögens</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70046">
    <skos:broader rdf:resource="thsys/b"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19249-2">
    <skos:broader rdf:resource="thsys/71034"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19249-2">
    <skos:broader rdf:resource="thsys/70481"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19249-2">
    <skos:broader rdf:resource="descriptor/12038-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12236-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16984-1">
    <skos:altLabel xml:lang="x-other">Lydveldid Island</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16984-1">
    <skos:altLabel xml:lang="de">Republik Island</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id309">
    <rdfs:seeAlso rdf:resource="descriptor/11183-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17941-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11736-3">
    <skos:broader rdf:resource="thsys/71114"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11736-3">
    <skos:broader rdf:resource="descriptor/11735-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12292-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id129">
    <rdf:value xml:lang="de">Für betriebliche Handelsbilanzen</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21306-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10658-2">
    <skos:prefLabel xml:lang="en">Inter-regional trade</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10658-2">
    <skos:prefLabel xml:lang="de">Interregionaler Handel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19439-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71001">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18156-0">
    <skos:scopeNote xml:lang="de">Hypothese, dass unter Berücksichtigung intergenerationaler Transfers Deficit Spending nicht expansiver wirkt als steuerfinanzierte Staatsausgaben.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12996-0">
    <skos:narrower rdf:resource="descriptor/18261-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12996-0">
    <skos:narrower rdf:resource="descriptor/13002-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12996-0">
    <skos:narrower rdf:resource="descriptor/18431-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12996-0">
    <skos:narrower rdf:resource="descriptor/12998-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12996-0">
    <skos:narrower rdf:resource="descriptor/13000-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12996-0">
    <skos:narrower rdf:resource="descriptor/18262-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12996-0">
    <skos:narrower rdf:resource="descriptor/12997-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12996-0">
    <skos:narrower rdf:resource="descriptor/12995-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12996-0">
    <skos:narrower rdf:resource="descriptor/12994-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19776-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18170-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11211-4">
    <skos:altLabel xml:lang="en">Labour market stratification</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11211-4">
    <skos:altLabel xml:lang="en">Labor market stratification</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11211-4">
    <skos:altLabel xml:lang="en">Labor market segmentation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11211-4">
    <skos:altLabel xml:lang="de">Teilarbeitsmarkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11211-4">
    <skos:altLabel xml:lang="de">Segmentierter Arbeitsmarkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11211-4">
    <skos:altLabel xml:lang="de">Gespaltener Arbeitsmarkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11211-4">
    <skos:altLabel xml:lang="de">Arbeitsmarktsegmentationstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11211-4">
    <skos:altLabel xml:lang="de">Arbeitsmarktsegment</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18984-5">
    <skos:altLabel xml:lang="de">Polder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18984-5">
    <skos:altLabel xml:lang="de">Moor</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18984-5">
    <skos:altLabel xml:lang="de">Marschland</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18984-5">
    <skos:altLabel xml:lang="de">Koog</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18984-5">
    <skos:altLabel xml:lang="de">Flussdelta</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18984-5">
    <skos:altLabel xml:lang="de">Flußdelta</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11738-6">
    <skos:related rdf:resource="descriptor/11742-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13226-2">
    <skos:broader rdf:resource="thsys/70959"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13226-2">
    <skos:broader rdf:resource="descriptor/18522-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19373-1">
    <skos:prefLabel xml:lang="en">Off-budget public sector</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19373-1">
    <skos:prefLabel xml:lang="de">Parafiskus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12112-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16683-1">
    <skos:altLabel xml:lang="de">Sozialmedizin</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16683-1">
    <skos:altLabel xml:lang="de">Medizinsoziologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16683-1">
    <skos:altLabel xml:lang="de">Medizinische Soziologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14012-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13084-3">
    <skos:altLabel xml:lang="de">Konsumelektronische Industrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10112-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13014-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70406">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10757-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18168-0">
    <skos:prefLabel xml:lang="en">Employment forecast</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18168-0">
    <skos:prefLabel xml:lang="de">Arbeitsmarktprognose</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id34">
    <rdf:value xml:lang="de">Für die Offenlegungsverpflichtung von Unternehmen</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13113-1">
    <skos:prefLabel xml:lang="en">Wood-processing industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13113-1">
    <skos:prefLabel xml:lang="de">Holzverarbeitende Industrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14654-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11873-0">
    <skos:prefLabel xml:lang="en">Spatial interaction</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11873-0">
    <skos:prefLabel xml:lang="de">Räumliche Interaktion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12569-5">
    <skos:prefLabel xml:lang="en">Assessment centre</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12569-5">
    <skos:prefLabel xml:lang="de">Assessment Center</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12480-0">
    <skos:altLabel xml:lang="de">Theorie der Kostenrechnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12480-0">
    <skos:altLabel xml:lang="de">Kosten- und Leistungsrechnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15970-3">
    <skos:narrower rdf:resource="descriptor/18068-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15970-3">
    <skos:narrower rdf:resource="descriptor/19705-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15970-3">
    <skos:narrower rdf:resource="descriptor/19704-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19809-4">
    <skos:related rdf:resource="descriptor/20241-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19809-4">
    <skos:related rdf:resource="descriptor/18647-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19809-4">
    <skos:related rdf:resource="descriptor/19806-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19611-4">
    <skos:scopeNote xml:lang="de">Erforschung der Beziehungen zwischen Geographie und den sozialen sowie kulturellen Aktivitäten der Menschen im weitesten Sinne.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13256-0">
    <skos:broader rdf:resource="thsys/70936"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13256-0">
    <skos:broader rdf:resource="descriptor/13510-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19326-3">
    <skos:related rdf:resource="descriptor/10625-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19326-3">
    <skos:related rdf:resource="descriptor/10698-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16052-1">
    <skos:altLabel xml:lang="de">GewO</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16052-1">
    <skos:altLabel xml:lang="de">Gewerbeordnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16052-1">
    <skos:altLabel xml:lang="de">Gewerbefreiheit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16052-1">
    <skos:altLabel xml:lang="de">Gaststättengesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16052-1">
    <skos:altLabel xml:lang="de">Apothekengesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18067-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14335-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12993-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19562-5">
    <skos:altLabel xml:lang="de">Betriebsorganisation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20125-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16366-6">
    <skos:related rdf:resource="descriptor/16305-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16366-6">
    <skos:related rdf:resource="descriptor/13397-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16366-6">
    <skos:related rdf:resource="descriptor/16008-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16366-6">
    <skos:related rdf:resource="descriptor/16365-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16366-6">
    <skos:related rdf:resource="descriptor/11167-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19845-0">
    <skos:prefLabel xml:lang="en">Disclosure rules</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19845-0">
    <skos:prefLabel xml:lang="de">Auskunftspflicht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11307-5">
    <skos:broader rdf:resource="thsys/73336"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11307-5">
    <skos:broader rdf:resource="thsys/70862"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11307-5">
    <skos:broader rdf:resource="thsys/70235"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11307-5">
    <skos:broader rdf:resource="descriptor/13410-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14819-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10126-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10172-0">
    <skos:altLabel xml:lang="en">Consumption externalities</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10172-0">
    <skos:altLabel xml:lang="en">Conspicuous consumption</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10172-0">
    <skos:altLabel xml:lang="de">Veblen-Effekt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10172-0">
    <skos:altLabel xml:lang="de">Snob-Effekt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10172-0">
    <skos:altLabel xml:lang="de">Schotten-Effekt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10172-0">
    <skos:altLabel xml:lang="de">Nachfrageinterdependenz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10172-0">
    <skos:altLabel xml:lang="de">Mitläufer-Effekt des Konsums</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10172-0">
    <skos:altLabel xml:lang="de">Mitläufereffekt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10172-0">
    <skos:altLabel xml:lang="de">Interdependenter Konsum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10172-0">
    <skos:altLabel xml:lang="de">Externer Effekt (Konsum)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10172-0">
    <skos:altLabel xml:lang="de">Demonstrationseffekt des Konsums</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10172-0">
    <skos:altLabel xml:lang="de">Bandwagon-Effekt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16566-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10617-2">
    <skos:related rdf:resource="descriptor/10619-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10617-2">
    <skos:related rdf:resource="descriptor/10625-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10617-2">
    <skos:related rdf:resource="descriptor/10624-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10617-2">
    <skos:related rdf:resource="descriptor/10597-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10617-2">
    <skos:related rdf:resource="descriptor/10623-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10617-2">
    <skos:related rdf:resource="descriptor/10587-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10617-2">
    <skos:related rdf:resource="descriptor/10618-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10617-2">
    <skos:related rdf:resource="descriptor/19324-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10617-2">
    <skos:related rdf:resource="descriptor/19299-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10617-2">
    <skos:related rdf:resource="descriptor/10605-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19037-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10227-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11181-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17626-1">
    <skos:altLabel xml:lang="de">Haschemitisches Königreich Jordanien</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12896-4">
    <skos:altLabel xml:lang="de">Testat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12394-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10161-5">
    <skos:broader rdf:resource="thsys/71118"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10161-5">
    <skos:broader rdf:resource="descriptor/10144-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10057-0">
    <skos:broader rdf:resource="thsys/70582"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11309-1">
    <skos:altLabel xml:lang="de">Erwerbsleben</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11309-1">
    <skos:altLabel xml:lang="de">Berufsleben</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11309-1">
    <skos:altLabel xml:lang="de">Arbeitsleben</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70154">
    <skos:prefLabel xml:lang="de">N.05  Recht und Rechtswissenschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70993">
    <skos:prefLabel xml:lang="en">W.06.01.06.01  Pharmaceutical industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70993">
    <skos:prefLabel xml:lang="de">W.06.01.06.01  Pharmazeutische Industrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13393-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13149-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19847-3">
    <skos:prefLabel xml:lang="en">Secrecy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19847-3">
    <skos:prefLabel xml:lang="de">Geheimhaltung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12562-5">
    <skos:related rdf:resource="descriptor/12565-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12562-5">
    <skos:related rdf:resource="descriptor/20381-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12562-5">
    <skos:related rdf:resource="descriptor/12314-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12562-5">
    <skos:related rdf:resource="descriptor/11262-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18482-1">
    <skos:prefLabel xml:lang="en">Industrial subsidies</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18482-1">
    <skos:prefLabel xml:lang="de">Industriesubvention</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16602-6">
    <skos:broader rdf:resource="thsys/73396"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16304-0">
    <skos:altLabel xml:lang="en">Political prisoners</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16304-0">
    <skos:altLabel xml:lang="de">Politische Häftlinge</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16304-0">
    <skos:altLabel xml:lang="de">Politische Gefangene</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16304-0">
    <skos:altLabel xml:lang="de">KZ-Haft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16304-0">
    <skos:altLabel xml:lang="de">Konzentrationslager</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16304-0">
    <skos:altLabel xml:lang="de">Internierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12361-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18337-3">
    <skos:altLabel xml:lang="de">Waldrodung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18337-3">
    <skos:altLabel xml:lang="de">Rodung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18337-3">
    <skos:altLabel xml:lang="de">Kahlschlag</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19840-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12741-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12241-4">
    <skos:altLabel xml:lang="en">Import-factoring</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12241-4">
    <skos:altLabel xml:lang="en">Export-factoring</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12241-4">
    <skos:altLabel xml:lang="de">Internationales Factoring</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12241-4">
    <skos:altLabel xml:lang="de">Forderungsverkauf</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12241-4">
    <skos:altLabel xml:lang="de">Auslands-Factoring</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12241-4">
    <skos:altLabel xml:lang="de">Außenhandels-Factoring</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11602-3">
    <skos:broader rdf:resource="thsys/71079"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11602-3">
    <skos:broader rdf:resource="descriptor/11599-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14169-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14942-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16454-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16184-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16445-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id173">
    <rdf:value xml:lang="de">Für Produkte der dekorativen Kosmetik</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18761-4">
    <skos:prefLabel xml:lang="en">Computer software law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18761-4">
    <skos:prefLabel xml:lang="de">Software-Recht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17680-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16210-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13697-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19721-4">
    <skos:broader rdf:resource="thsys/73364"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19721-4">
    <skos:broader rdf:resource="thsys/73356"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19721-4">
    <skos:broader rdf:resource="thsys/70142"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19721-4">
    <skos:broader rdf:resource="descriptor/15909-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11881-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19362-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12357-6">
    <skos:narrower rdf:resource="descriptor/12396-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12357-6">
    <skos:narrower rdf:resource="descriptor/12394-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12357-6">
    <skos:narrower rdf:resource="descriptor/12393-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12357-6">
    <skos:narrower rdf:resource="descriptor/12365-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12357-6">
    <skos:narrower rdf:resource="descriptor/18719-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19154-6">
    <skos:broader rdf:resource="thsys/70485"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19154-6">
    <skos:broader rdf:resource="descriptor/20855-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10671-3">
    <skos:altLabel xml:lang="en">Monetary exchange rate model</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10671-3">
    <skos:altLabel xml:lang="en">Asset market approach</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10671-3">
    <skos:altLabel xml:lang="de">Neuere monetäre Wechselkurstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10671-3">
    <skos:altLabel xml:lang="de">Monetäre Devisenkurstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10671-3">
    <skos:altLabel xml:lang="de">Finanzmarkttheorie des Wechselkurses</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10671-3">
    <skos:altLabel xml:lang="de">Finanzmarktansatz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17943-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19762-4">
    <skos:altLabel xml:lang="de">Vater</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14141-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14485-1">
    <skos:prefLabel xml:lang="en">Automotive electronics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14485-1">
    <skos:prefLabel xml:lang="de">Kraftfahrzeugelektrik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15757-3">
    <skos:broader rdf:resource="thsys/70238"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15757-3">
    <skos:broader rdf:resource="descriptor/15763-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16540-2">
    <skos:altLabel xml:lang="en">Retirement incomes policy</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16540-2">
    <skos:altLabel xml:lang="en">Pension policy</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14608-4">
    <skos:broader rdf:resource="thsys/70463"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14608-4">
    <skos:broader rdf:resource="descriptor/14601-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14608-4">
    <skos:broader rdf:resource="descriptor/14642-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16664-5">
    <skos:broader rdf:resource="thsys/73358"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19501-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13214-2">
    <skos:broader rdf:resource="thsys/70957"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13214-2">
    <skos:broader rdf:resource="descriptor/13200-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13214-2">
    <skos:broader rdf:resource="descriptor/13194-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10701-6">
    <skos:related rdf:resource="descriptor/10838-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19261-5">
    <skos:broader rdf:resource="thsys/73387"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19261-5">
    <skos:broader rdf:resource="thsys/70305"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19261-5">
    <skos:broader rdf:resource="thsys/70109"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19261-5">
    <skos:broader rdf:resource="descriptor/11416-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15887-4">
    <skos:narrower rdf:resource="descriptor/17968-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15887-4">
    <skos:narrower rdf:resource="descriptor/19618-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15887-4">
    <skos:narrower rdf:resource="descriptor/19617-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15887-4">
    <skos:narrower rdf:resource="descriptor/17959-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15887-4">
    <skos:narrower rdf:resource="descriptor/17954-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12613-0">
    <skos:related rdf:resource="descriptor/13817-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12613-0">
    <skos:related rdf:resource="descriptor/18030-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12613-0">
    <skos:related rdf:resource="descriptor/13385-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12613-0">
    <skos:related rdf:resource="descriptor/11814-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12613-0">
    <skos:related rdf:resource="descriptor/18820-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12613-0">
    <skos:related rdf:resource="descriptor/19247-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12613-0">
    <skos:related rdf:resource="descriptor/12609-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19556-0">
    <skos:related rdf:resource="descriptor/19137-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19556-0">
    <skos:related rdf:resource="descriptor/18085-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19556-0">
    <skos:related rdf:resource="descriptor/12214-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18995-0">
    <skos:related rdf:resource="descriptor/10884-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18995-0">
    <skos:related rdf:resource="descriptor/12922-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18995-0">
    <skos:related rdf:resource="descriptor/15753-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18995-0">
    <skos:related rdf:resource="descriptor/18382-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18995-0">
    <skos:related rdf:resource="descriptor/15752-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18995-0">
    <skos:related rdf:resource="descriptor/18690-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18995-0">
    <skos:related rdf:resource="descriptor/14047-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18995-0">
    <skos:related rdf:resource="descriptor/18381-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18995-0">
    <skos:related rdf:resource="descriptor/18996-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18995-0">
    <skos:related rdf:resource="descriptor/13431-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13406-0">
    <skos:related rdf:resource="descriptor/19495-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13406-0">
    <skos:related rdf:resource="descriptor/11237-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13406-0">
    <skos:related rdf:resource="descriptor/10143-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13406-0">
    <skos:related rdf:resource="descriptor/11182-6"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id203">
    <rdfs:seeAlso rdf:resource="descriptor/11211-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14940-3">
    <skos:altLabel xml:lang="de">Torte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14940-3">
    <skos:altLabel xml:lang="de">Kuchen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14940-3">
    <skos:altLabel xml:lang="de">Konditoreiwaren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17626-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10798-0">
    <skos:prefLabel xml:lang="en">Foreign assets</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10798-0">
    <skos:prefLabel xml:lang="de">Auslandsvermögen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19688-4">
    <skos:prefLabel xml:lang="en">Creation of states</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19688-4">
    <skos:prefLabel xml:lang="de">Staatsgründung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18278-0">
    <skos:prefLabel xml:lang="en">Commodity science</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18278-0">
    <skos:prefLabel xml:lang="de">Warenlehre</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11241-2">
    <skos:prefLabel xml:lang="en">Wage subsidy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11241-2">
    <skos:prefLabel xml:lang="de">Lohnsubvention</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10960-3">
    <skos:related rdf:resource="descriptor/10913-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10960-3">
    <skos:related rdf:resource="descriptor/10951-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10960-3">
    <skos:related rdf:resource="descriptor/10978-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10960-3">
    <skos:related rdf:resource="descriptor/10909-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10960-3">
    <skos:related rdf:resource="descriptor/10904-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10812-4">
    <skos:altLabel xml:lang="de">Währungsbeistandskredit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10812-4">
    <skos:altLabel xml:lang="de">IMF-Kredit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10812-4">
    <skos:altLabel xml:lang="de">Beistandskredit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10812-4">
    <skos:altLabel xml:lang="de">Auslandskredit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14531-6">
    <skos:altLabel xml:lang="de">Regelgerät (Feinmechanik)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14531-6">
    <skos:altLabel xml:lang="de">Messgerät (Feinmechanik)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14531-6">
    <skos:altLabel xml:lang="de">Meßgerät (Feinmechanik)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14531-6">
    <skos:altLabel xml:lang="de">Feinmechanisches Messgerät</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14531-6">
    <skos:altLabel xml:lang="de">Feinmechanisches Meßgerät</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14531-6">
    <skos:altLabel xml:lang="de">Atemschutzgerät</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11159-1">
    <skos:related rdf:resource="descriptor/11336-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11159-1">
    <skos:related rdf:resource="descriptor/16131-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13588-3">
    <skos:prefLabel xml:lang="en">Transport economics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13588-3">
    <skos:prefLabel xml:lang="de">Verkehrsökonomik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70990">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16186-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11866-4">
    <skos:altLabel xml:lang="de">Landschaftsverbrauch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11866-4">
    <skos:altLabel xml:lang="de">Landschaftsschutz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11866-4">
    <skos:altLabel xml:lang="de">Landespflege</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10207-0">
    <skos:altLabel xml:lang="de">Moralisches Risiko</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10207-0">
    <skos:altLabel xml:lang="de">Moralische Versuchung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16604-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11051-0">
    <skos:narrower rdf:resource="descriptor/19381-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15729-1">
    <skos:prefLabel xml:lang="en">Waste water</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15729-1">
    <skos:prefLabel xml:lang="de">Abwasser</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13029-4">
    <skos:prefLabel xml:lang="en">Steel industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13029-4">
    <skos:prefLabel xml:lang="de">Stahlindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id293">
    <rdfs:seeAlso rdf:resource="descriptor/10595-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11959-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18556-5">
    <skos:broader rdf:resource="thsys/70947"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18556-5">
    <skos:broader rdf:resource="thsys/70829"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18556-5">
    <skos:broader rdf:resource="descriptor/18532-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20176-0">
    <skos:related rdf:resource="descriptor/19755-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20176-0">
    <skos:related rdf:resource="descriptor/12809-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20176-0">
    <skos:related rdf:resource="descriptor/12806-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20176-0">
    <skos:related rdf:resource="descriptor/10194-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15437-0">
    <skos:prefLabel xml:lang="en">Dynamic game</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15437-0">
    <skos:prefLabel xml:lang="de">Dynamisches Spiel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10856-5">
    <skos:broader rdf:resource="thsys/71054"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10856-5">
    <skos:broader rdf:resource="thsys/70952"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10856-5">
    <skos:broader rdf:resource="descriptor/13466-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10856-5">
    <skos:broader rdf:resource="descriptor/10852-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19772-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11286-1">
    <skos:altLabel xml:lang="en">Production workers</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11286-1">
    <skos:altLabel xml:lang="en">Nonproduction workers</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11286-1">
    <skos:altLabel xml:lang="en">Manual worker</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11286-1">
    <skos:altLabel xml:lang="de">Arbeiterinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13158-0">
    <skos:broader rdf:resource="thsys/70986"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13158-0">
    <skos:broader rdf:resource="descriptor/13160-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11123-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14368-5">
    <skos:related rdf:resource="descriptor/18407-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14368-5">
    <skos:related rdf:resource="descriptor/18390-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14368-5">
    <skos:related rdf:resource="descriptor/14192-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14368-5">
    <skos:related rdf:resource="descriptor/12980-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18435-3">
    <skos:related rdf:resource="descriptor/18095-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18435-3">
    <skos:related rdf:resource="descriptor/14392-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18435-3">
    <skos:related rdf:resource="descriptor/14630-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18435-3">
    <skos:related rdf:resource="descriptor/13141-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18435-3">
    <skos:related rdf:resource="descriptor/18094-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18964-4">
    <skos:prefLabel xml:lang="en">Recycling technology</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18964-4">
    <skos:prefLabel xml:lang="de">Recycling-Technik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11185-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18620-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19424-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12433-2">
    <skos:broader rdf:resource="thsys/71043"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12433-2">
    <skos:broader rdf:resource="descriptor/19091-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19631-5">
    <skos:prefLabel xml:lang="en">Marine policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19631-5">
    <skos:prefLabel xml:lang="de">Meerespolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19369-6">
    <skos:broader rdf:resource="thsys/71085"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19369-6">
    <skos:broader rdf:resource="thsys/71069"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19369-6">
    <skos:broader rdf:resource="descriptor/10552-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10171-2">
    <skos:scopeNote xml:lang="de">Überwiegend die Einkommenselastizität der Nachfrage.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17792-5">
    <skos:prefLabel xml:lang="en">Nicaragua</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17792-5">
    <skos:prefLabel xml:lang="de">Nicaragua</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17014-4">
    <skos:altLabel xml:lang="de">Fürstentum Monaco</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13292-3">
    <skos:related rdf:resource="descriptor/18180-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13292-3">
    <skos:related rdf:resource="descriptor/16670-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13292-3">
    <skos:related rdf:resource="descriptor/18444-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13292-3">
    <skos:related rdf:resource="descriptor/11157-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13292-3">
    <skos:related rdf:resource="descriptor/11555-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14752-4">
    <skos:altLabel xml:lang="en">Mass storage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14752-4">
    <skos:altLabel xml:lang="de">Speichermedium</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14752-4">
    <skos:altLabel xml:lang="de">Speicher (Datenverarbeitung)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14752-4">
    <skos:altLabel xml:lang="de">Massenspeicher</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14752-4">
    <skos:altLabel xml:lang="de">Großspeicher</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19028-4">
    <skos:prefLabel xml:lang="en">International</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19028-4">
    <skos:prefLabel xml:lang="de">International</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13794-0">
    <skos:related rdf:resource="descriptor/13795-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10237-5">
    <skos:narrower rdf:resource="descriptor/10276-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11275-6">
    <skos:altLabel xml:lang="en">Youth labor</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11275-6">
    <skos:altLabel xml:lang="de">Arbeitskräftenachwuchs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14518-5">
    <skos:broader rdf:resource="thsys/70373"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14518-5">
    <skos:broader rdf:resource="descriptor/14435-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14518-5">
    <skos:broader rdf:resource="descriptor/14516-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12081-0">
    <skos:broader rdf:resource="thsys/71052"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12081-0">
    <skos:broader rdf:resource="thsys/70485"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12081-0">
    <skos:broader rdf:resource="descriptor/11302-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10911-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id173">
    <rdfs:seeAlso rdf:resource="descriptor/14735-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13188-5">
    <skos:scopeNote xml:lang="de">Privatwirtschaftlich und genossenschaftlich organisierte Beschaffungs- und Absatzaktivitäten für die Landwirtschaft.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17669-4">
    <skos:prefLabel xml:lang="en">Mozambique</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17669-4">
    <skos:prefLabel xml:lang="de">Mosambik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13231-2">
    <skos:broader rdf:resource="thsys/70959"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13231-2">
    <skos:broader rdf:resource="descriptor/18522-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18863-3">
    <skos:scopeNote xml:lang="de">Bildung von Frauen und Mädchen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11181-1">
    <skos:altLabel xml:lang="de">Arbeitsphilosophie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11181-1">
    <skos:altLabel xml:lang="de">Arbeitsmoral</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11181-1">
    <skos:altLabel xml:lang="de">Arbeitsgesinnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11181-1">
    <skos:altLabel xml:lang="de">Arbeitsethos</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11215-3">
    <skos:prefLabel xml:lang="en">Holidays</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11215-3">
    <skos:prefLabel xml:lang="de">Urlaub</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16212-5">
    <skos:scopeNote xml:lang="de">Die Verwaltungswissenschaft behandelt vor allem juristische, ökonomische, soziologische und politologische Fragestellungen bezüglich des Erkenntnisobjekts "Öffentliche Verwaltung"; entweder interdisziplinär oder fachspezifisch.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11490-2">
    <skos:altLabel xml:lang="en">Easy money</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11490-2">
    <skos:altLabel xml:lang="de">Zinssteuerung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11490-2">
    <skos:altLabel xml:lang="de">Niedrigzinspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11490-2">
    <skos:altLabel xml:lang="de">Hochzinspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12610-6">
    <skos:related rdf:resource="descriptor/12615-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12610-6">
    <skos:related rdf:resource="descriptor/12609-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19833-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11703-4">
    <skos:broader rdf:resource="thsys/71093"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11703-4">
    <skos:broader rdf:resource="thsys/71075"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11703-4">
    <skos:broader rdf:resource="thsys/71040"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11703-4">
    <skos:broader rdf:resource="descriptor/11704-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10524-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11873-0">
    <skos:narrower rdf:resource="descriptor/11956-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24740-5">
    <skos:related rdf:resource="descriptor/12262-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24740-5">
    <skos:related rdf:resource="descriptor/10803-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24740-5">
    <skos:related rdf:resource="descriptor/11441-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24740-5">
    <skos:related rdf:resource="descriptor/18668-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18557-3">
    <skos:prefLabel xml:lang="en">Eco-label</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18557-3">
    <skos:prefLabel xml:lang="de">Umweltzeichen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71014">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15060-0">
    <skos:broader rdf:resource="thsys/72185"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15060-0">
    <skos:broader rdf:resource="thsys/71035"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15060-0">
    <skos:broader rdf:resource="thsys/70269"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15060-0">
    <skos:broader rdf:resource="descriptor/15064-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15060-0">
    <skos:broader rdf:resource="descriptor/15045-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18897-0">
    <skos:altLabel xml:lang="en">Smallpox</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18897-0">
    <skos:altLabel xml:lang="en">Severe acute respiratory syndrome</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18897-0">
    <skos:altLabel xml:lang="en">SARS (Severe Acute Respiratory Syndrome)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18897-0">
    <skos:altLabel xml:lang="de">Windpocken</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18897-0">
    <skos:altLabel xml:lang="de">Tuberkulose</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18897-0">
    <skos:altLabel xml:lang="de">Geschlechtskrankheit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13302-5">
    <skos:altLabel xml:lang="en">Nonprofit housing organization</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13302-5">
    <skos:altLabel xml:lang="en">Nonprofit housing organisation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13302-5">
    <skos:altLabel xml:lang="en">Nonprofit housing industry</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13302-5">
    <skos:altLabel xml:lang="en">Non-profit housing industry</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13302-5">
    <skos:altLabel xml:lang="de">Gemeinnütziges Wohnungsbauunternehmen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10267-3">
    <skos:prefLabel xml:lang="en">Public consumption</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10267-3">
    <skos:prefLabel xml:lang="de">Öffentlicher Konsum</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12112-1">
    <skos:related rdf:resource="descriptor/11160-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12112-1">
    <skos:related rdf:resource="descriptor/12103-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12112-1">
    <skos:related rdf:resource="descriptor/11038-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19761-6">
    <skos:prefLabel xml:lang="en">Life course</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19761-6">
    <skos:prefLabel xml:lang="de">Lebensverlauf</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14803-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18245-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70921">
    <rdfs:label xml:lang="de">Rundfunk und Fernsehen</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13078-5">
    <skos:broader rdf:resource="thsys/70995"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13078-5">
    <skos:broader rdf:resource="descriptor/12993-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10897-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14640-1">
    <skos:altLabel xml:lang="de">Kohlendioxyd</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14640-1">
    <skos:altLabel xml:lang="de">CO2</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19455-6">
    <skos:broader rdf:resource="thsys/71066"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19455-6">
    <skos:broader rdf:resource="thsys/71023"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19455-6">
    <skos:broader rdf:resource="descriptor/18329-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14118-0">
    <skos:broader rdf:resource="thsys/70169"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14118-0">
    <skos:broader rdf:resource="descriptor/14073-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14118-0">
    <skos:broader rdf:resource="descriptor/14104-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15881-2">
    <skos:broader rdf:resource="thsys/73352"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15881-2">
    <skos:broader rdf:resource="descriptor/15888-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10274-6">
    <skos:scopeNote xml:lang="de">Nur für die Vermögensrechnung im Rahmen der Volkswirtschaftlichen Gesamtrechnung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10560-5">
    <skos:narrower rdf:resource="descriptor/10524-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10560-5">
    <skos:narrower rdf:resource="descriptor/10523-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19597-0">
    <skos:prefLabel xml:lang="en">Folk Theorem</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19597-0">
    <skos:prefLabel xml:lang="de">Folk-Theorem</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14554-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70823">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.30</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15964-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10140-6">
    <skos:altLabel xml:lang="en">Search costs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10140-6">
    <skos:altLabel xml:lang="de">Suchkosten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70030">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13307-2">
    <skos:broader rdf:resource="thsys/70944"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13307-2">
    <skos:broader rdf:resource="descriptor/12907-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10284-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14610-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13795-5">
    <skos:related rdf:resource="descriptor/13794-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10917-4">
    <skos:broader rdf:resource="thsys/73390"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10917-4">
    <skos:broader rdf:resource="thsys/73331"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18829-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18195-4">
    <skos:related rdf:resource="descriptor/18315-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18195-4">
    <skos:related rdf:resource="descriptor/15895-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18195-4">
    <skos:related rdf:resource="descriptor/14074-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12049-3">
    <skos:prefLabel xml:lang="en">Large enterprise</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12049-3">
    <skos:prefLabel xml:lang="de">Großunternehmen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11713-1">
    <skos:prefLabel xml:lang="en">Budget incidence</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11713-1">
    <skos:prefLabel xml:lang="de">Budgetinzidenz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20948-1">
    <skos:broader rdf:resource="thsys/70562"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20948-1">
    <skos:broader rdf:resource="descriptor/12120-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13873-4">
    <skos:broader rdf:resource="thsys/71057"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13873-4">
    <skos:broader rdf:resource="thsys/71018"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19439-4">
    <skos:related rdf:resource="descriptor/19471-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19439-4">
    <skos:related rdf:resource="descriptor/11207-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19439-4">
    <skos:related rdf:resource="descriptor/19492-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19439-4">
    <skos:related rdf:resource="descriptor/19461-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73387">
    <rdfs:label xml:lang="en">Economic history</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73387">
    <rdfs:label xml:lang="de">Wirtschaftsgeschichte</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12192-5">
    <skos:altLabel xml:lang="de">Personalgesellschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19220-5">
    <skos:prefLabel xml:lang="en">Needs</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19220-5">
    <skos:prefLabel xml:lang="de">Bedürfnisse</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71078">
    <skos:broader rdf:resource="thsys/70144"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71085">
    <skos:narrower rdf:resource="descriptor/10595-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71085">
    <skos:narrower rdf:resource="descriptor/19369-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71085">
    <skos:narrower rdf:resource="descriptor/19235-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71085">
    <skos:narrower rdf:resource="descriptor/18325-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71085">
    <skos:narrower rdf:resource="descriptor/20553-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71085">
    <skos:narrower rdf:resource="descriptor/10505-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71085">
    <skos:narrower rdf:resource="descriptor/19368-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71085">
    <skos:narrower rdf:resource="descriptor/18323-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71085">
    <skos:narrower rdf:resource="descriptor/18481-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71085">
    <skos:narrower rdf:resource="descriptor/10570-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71085">
    <skos:narrower rdf:resource="descriptor/10497-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71085">
    <skos:narrower rdf:resource="descriptor/10565-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71085">
    <skos:narrower rdf:resource="descriptor/10564-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71085">
    <skos:narrower rdf:resource="descriptor/10524-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71085">
    <skos:narrower rdf:resource="descriptor/10561-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71085">
    <skos:narrower rdf:resource="descriptor/10560-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71085">
    <skos:narrower rdf:resource="descriptor/10562-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71085">
    <skos:narrower rdf:resource="descriptor/10523-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71085">
    <skos:narrower rdf:resource="descriptor/10559-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71085">
    <skos:narrower rdf:resource="descriptor/10558-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71085">
    <skos:narrower rdf:resource="descriptor/10553-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71085">
    <skos:narrower rdf:resource="descriptor/10552-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71085">
    <skos:narrower rdf:resource="descriptor/10550-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71085">
    <skos:narrower rdf:resource="descriptor/10548-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71085">
    <skos:narrower rdf:resource="descriptor/18894-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71085">
    <skos:narrower rdf:resource="descriptor/10534-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71085">
    <skos:narrower rdf:resource="descriptor/16533-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71085">
    <skos:narrower rdf:resource="descriptor/10487-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12594-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70997">
    <rdfs:label xml:lang="en">Road vehicle construction</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70997">
    <rdfs:label xml:lang="de">Straßenfahrzeugbau</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12153-1">
    <skos:prefLabel xml:lang="en">Profit centre</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12153-1">
    <skos:prefLabel xml:lang="de">Profit Center</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19794-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10055-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id371">
    <rdfs:seeAlso rdf:resource="descriptor/10913-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11660-3">
    <skos:altLabel xml:lang="de">Splittingverfahren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11660-3">
    <skos:altLabel xml:lang="de">Haushaltsteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11660-3">
    <skos:altLabel xml:lang="de">Haushaltssteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11660-3">
    <skos:altLabel xml:lang="de">Haushaltsbesteuerung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11660-3">
    <skos:altLabel xml:lang="de">Familieneinkommensbesteuerung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11660-3">
    <skos:altLabel xml:lang="de">Ehegattensplitting</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19456-4">
    <skos:altLabel xml:lang="de">Städtischer Bodenmarkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19512-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13742-5">
    <skos:prefLabel xml:lang="en">Securities trading</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13742-5">
    <skos:prefLabel xml:lang="de">Wertpapierhandel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19532-0">
    <skos:related rdf:resource="descriptor/13867-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19532-0">
    <skos:related rdf:resource="descriptor/13829-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19532-0">
    <skos:related rdf:resource="descriptor/13862-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19532-0">
    <skos:related rdf:resource="descriptor/13840-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19532-0">
    <skos:related rdf:resource="descriptor/13839-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17472-2">
    <skos:scopeNote xml:lang="de">Erklärte sich 1991 unabhängig von der Sowjetunion; Mitglied der GUS</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18186-5">
    <skos:broader rdf:resource="thsys/73392"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18186-5">
    <skos:broader rdf:resource="thsys/71033"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18186-5">
    <skos:broader rdf:resource="thsys/70239"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18186-5">
    <skos:broader rdf:resource="descriptor/18067-6"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id283">
    <rdf:value xml:lang="de">Für die berufliche Wiedereingliederung</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18333-4">
    <skos:altLabel xml:lang="de">Landreform</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17787-5">
    <skos:prefLabel xml:lang="en">Cuba</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17787-5">
    <skos:prefLabel xml:lang="de">Kuba</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14106-0">
    <skos:altLabel xml:lang="en">Pulse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14106-0">
    <skos:altLabel xml:lang="de">Leguminosen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14106-0">
    <skos:altLabel xml:lang="de">Bohne</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19309-3">
    <skos:altLabel xml:lang="en">Common market</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19309-3">
    <skos:altLabel xml:lang="de">Marktverschmelzung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10566-0">
    <skos:related rdf:resource="descriptor/10593-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10566-0">
    <skos:related rdf:resource="descriptor/10836-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10566-0">
    <skos:related rdf:resource="descriptor/12311-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10566-0">
    <skos:related rdf:resource="descriptor/16066-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10566-0">
    <skos:related rdf:resource="descriptor/10545-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13081-2">
    <skos:narrower rdf:resource="descriptor/18427-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13081-2">
    <skos:narrower rdf:resource="descriptor/13082-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16110-6">
    <skos:prefLabel xml:lang="en">Tax enforcement</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16110-6">
    <skos:prefLabel xml:lang="de">Steuerfahndung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14245-0">
    <skos:prefLabel xml:lang="en">Clay</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14245-0">
    <skos:prefLabel xml:lang="de">Ton</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19065-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70573">
    <skos:broader rdf:resource="thsys/70056"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19239-5">
    <skos:broader rdf:resource="thsys/70020"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19239-5">
    <skos:broader rdf:resource="descriptor/19229-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19239-5">
    <skos:broader rdf:resource="descriptor/10347-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14011-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19141-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70988">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18320-6">
    <skos:broader rdf:resource="thsys/71028"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18320-6">
    <skos:broader rdf:resource="thsys/70986"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18320-6">
    <skos:broader rdf:resource="descriptor/12913-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18685-1">
    <skos:altLabel xml:lang="en">Mortgage-backed securities</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18685-1">
    <skos:altLabel xml:lang="en">Asset-backed securities</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18685-1">
    <skos:altLabel xml:lang="en">ABS (Asset-backed security)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70321">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18736-3">
    <skos:scopeNote xml:lang="de">Rechtswidriges Verschaffen von Vermögensvorteilen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18760-6">
    <skos:related rdf:resource="descriptor/18759-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18760-6">
    <skos:related rdf:resource="descriptor/12254-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11611-2">
    <skos:broader rdf:resource="thsys/71105"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11611-2">
    <skos:broader rdf:resource="thsys/71078"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11611-2">
    <skos:broader rdf:resource="descriptor/11613-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11611-2">
    <skos:broader rdf:resource="descriptor/12234-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70883">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.18</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13391-1">
    <skos:narrower rdf:resource="descriptor/12901-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15899-4">
    <skos:related rdf:resource="descriptor/13578-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15899-4">
    <skos:related rdf:resource="descriptor/16173-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15899-4">
    <skos:related rdf:resource="descriptor/15900-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15899-4">
    <skos:related rdf:resource="descriptor/14423-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15899-4">
    <skos:related rdf:resource="descriptor/13507-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13214-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11170-6">
    <skos:broader rdf:resource="thsys/73383"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11170-6">
    <skos:broader rdf:resource="descriptor/16209-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19829-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14104-4">
    <skos:related rdf:resource="descriptor/14076-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14104-4">
    <skos:related rdf:resource="descriptor/12940-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14104-4">
    <skos:related rdf:resource="descriptor/14030-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16043-2">
    <skos:prefLabel xml:lang="en">Stock law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16043-2">
    <skos:prefLabel xml:lang="de">Aktienrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14218-3">
    <skos:broader rdf:resource="thsys/70014"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14218-3">
    <skos:broader rdf:resource="descriptor/14599-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70853">
    <skos:prefLabel xml:lang="de">W.23.02  Interessenvertretungen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11778-1">
    <skos:prefLabel xml:lang="en">Property policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11778-1">
    <skos:prefLabel xml:lang="de">Vermögenspolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12474-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15466-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12760-1">
    <skos:broader rdf:resource="thsys/70029"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12760-1">
    <skos:broader rdf:resource="descriptor/12758-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14449-5">
    <skos:prefLabel xml:lang="en">Cargo ship</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14449-5">
    <skos:prefLabel xml:lang="de">Frachtschiff</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17770-1">
    <skos:prefLabel xml:lang="en">Antigua and Barbuda</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17770-1">
    <skos:prefLabel xml:lang="de">Antigua und Barbuda</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11658-4">
    <skos:altLabel xml:lang="de">Grunderwerbsteuerzuschlag</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11658-4">
    <skos:altLabel xml:lang="de">Grunderwerbsteuerrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11658-4">
    <skos:altLabel xml:lang="de">Grunderwerbssteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11658-4">
    <skos:altLabel xml:lang="de">Grundbesitzwechselsteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13252-1">
    <skos:altLabel xml:lang="en">Ship owner</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13252-1">
    <skos:altLabel xml:lang="de">Schiffsmakler</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13252-1">
    <skos:altLabel xml:lang="de">Schiffsagentur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13252-1">
    <skos:altLabel xml:lang="de">Schifffahrtsgesellschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13252-1">
    <skos:altLabel xml:lang="de">Schiffahrtsgesellschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17703-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11568-5">
    <skos:broader rdf:resource="thsys/71080"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11568-5">
    <skos:broader rdf:resource="descriptor/11554-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18149-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19560-2">
    <skos:prefLabel xml:lang="en">Voluntary agreement</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19560-2">
    <skos:prefLabel xml:lang="de">Selbstverpflichtung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11160-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71023">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13222-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16527-1">
    <skos:related rdf:resource="descriptor/16526-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16527-1">
    <skos:related rdf:resource="descriptor/11261-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16028-5">
    <skos:related rdf:resource="descriptor/11651-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16028-5">
    <skos:related rdf:resource="descriptor/16029-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17104-3">
    <skos:altLabel xml:lang="x-other">Republica Portuguesa</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17104-3">
    <skos:altLabel xml:lang="de">Portugiesische Republik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14596-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14805-2">
    <skos:related rdf:resource="descriptor/13117-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14805-2">
    <skos:related rdf:resource="descriptor/14803-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14805-2">
    <skos:related rdf:resource="descriptor/14385-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14805-2">
    <skos:related rdf:resource="descriptor/14802-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11708-1">
    <skos:prefLabel xml:lang="en">Tax collection</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11708-1">
    <skos:prefLabel xml:lang="de">Steuererhebung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12194-1">
    <skos:narrower rdf:resource="descriptor/13402-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12194-1">
    <skos:narrower rdf:resource="descriptor/12197-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12194-1">
    <skos:narrower rdf:resource="descriptor/12192-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12194-1">
    <skos:narrower rdf:resource="descriptor/12189-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12194-1">
    <skos:narrower rdf:resource="descriptor/11628-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12194-1">
    <skos:narrower rdf:resource="descriptor/12186-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12194-1">
    <skos:narrower rdf:resource="descriptor/12180-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12194-1">
    <skos:narrower rdf:resource="descriptor/18108-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12194-1">
    <skos:narrower rdf:resource="descriptor/12178-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12194-1">
    <skos:narrower rdf:resource="descriptor/12177-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12915-5">
    <skos:broader rdf:resource="thsys/70969"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12915-5">
    <skos:broader rdf:resource="descriptor/12905-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18235-4">
    <skos:narrower rdf:resource="descriptor/14211-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18235-4">
    <skos:narrower rdf:resource="descriptor/14200-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18235-4">
    <skos:narrower rdf:resource="descriptor/14197-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18235-4">
    <skos:narrower rdf:resource="descriptor/14196-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16207-5">
    <skos:altLabel xml:lang="de">Politischer Stil</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16207-5">
    <skos:altLabel xml:lang="de">Politische Tradition</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19709-1">
    <skos:prefLabel xml:lang="en">Major power</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19709-1">
    <skos:prefLabel xml:lang="de">Großmacht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id41">
    <rdf:value xml:lang="de">Für die Geschichte einzelner Banken</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20399-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18376-0">
    <skos:broader rdf:resource="thsys/71074"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18376-0">
    <skos:broader rdf:resource="thsys/71012"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18376-0">
    <skos:broader rdf:resource="thsys/70539"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18376-0">
    <skos:broader rdf:resource="descriptor/18364-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18376-0">
    <skos:broader rdf:resource="descriptor/19422-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18376-0">
    <skos:broader rdf:resource="descriptor/18072-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19682-2">
    <skos:prefLabel xml:lang="en">Oligarchy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19682-2">
    <skos:prefLabel xml:lang="de">Oligarchie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11636-0">
    <skos:altLabel xml:lang="de">Teilsteuerrechnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11636-0">
    <skos:altLabel xml:lang="de">Steuerquote</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11636-0">
    <skos:altLabel xml:lang="de">Steuerlastrechnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11636-0">
    <skos:altLabel xml:lang="de">Steuerlastquote</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11636-0">
    <skos:altLabel xml:lang="de">Steuerlast</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11636-0">
    <skos:altLabel xml:lang="de">Steuerbelastungsrechnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11636-0">
    <skos:altLabel xml:lang="de">Abgabenquote</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12024-5">
    <skos:broader rdf:resource="thsys/71066"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12024-5">
    <skos:broader rdf:resource="thsys/71023"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19315-1">
    <skos:related rdf:resource="descriptor/19235-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19315-1">
    <skos:related rdf:resource="descriptor/10816-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19315-1">
    <skos:related rdf:resource="descriptor/10572-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19315-1">
    <skos:related rdf:resource="descriptor/10824-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19315-1">
    <skos:related rdf:resource="descriptor/10823-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19315-1">
    <skos:related rdf:resource="descriptor/18001-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17988-2">
    <skos:prefLabel xml:lang="en">Islamic countries</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17988-2">
    <skos:prefLabel xml:lang="de">Islamische Staaten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10637-3">
    <skos:related rdf:resource="descriptor/10641-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10637-3">
    <skos:related rdf:resource="descriptor/10614-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10637-3">
    <skos:related rdf:resource="descriptor/10638-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18094-3">
    <skos:related rdf:resource="descriptor/10581-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18094-3">
    <skos:related rdf:resource="descriptor/18435-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18094-3">
    <skos:related rdf:resource="descriptor/18095-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18094-3">
    <skos:related rdf:resource="descriptor/14392-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18094-3">
    <skos:related rdf:resource="descriptor/14630-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18094-3">
    <skos:related rdf:resource="descriptor/13141-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18094-3">
    <skos:related rdf:resource="descriptor/19061-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18094-3">
    <skos:related rdf:resource="descriptor/18436-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18094-3">
    <skos:related rdf:resource="descriptor/13935-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13375-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10566-0">
    <skos:broader rdf:resource="thsys/71097"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10566-0">
    <skos:broader rdf:resource="thsys/71084"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13159-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18677-0">
    <skos:broader rdf:resource="thsys/73390"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18677-0">
    <skos:broader rdf:resource="thsys/70893"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18677-0">
    <skos:broader rdf:resource="thsys/70187"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18677-0">
    <skos:broader rdf:resource="descriptor/13731-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15994-3">
    <skos:prefLabel xml:lang="en">Freedom of movement</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15994-3">
    <skos:prefLabel xml:lang="de">Freizügigkeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70324">
    <skos:narrower rdf:resource="descriptor/19508-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70324">
    <skos:narrower rdf:resource="descriptor/19511-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70324">
    <skos:narrower rdf:resource="descriptor/16160-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70324">
    <skos:narrower rdf:resource="descriptor/11570-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70324">
    <skos:narrower rdf:resource="descriptor/16318-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70324">
    <skos:narrower rdf:resource="descriptor/15725-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70324">
    <skos:narrower rdf:resource="descriptor/16159-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70324">
    <skos:narrower rdf:resource="descriptor/16545-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70324">
    <skos:narrower rdf:resource="descriptor/11569-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70324">
    <skos:narrower rdf:resource="descriptor/18754-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70324">
    <skos:narrower rdf:resource="descriptor/16542-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70324">
    <skos:narrower rdf:resource="descriptor/18910-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70324">
    <skos:narrower rdf:resource="descriptor/16650-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70324">
    <skos:narrower rdf:resource="descriptor/16155-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70324">
    <skos:narrower rdf:resource="descriptor/19510-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70324">
    <skos:narrower rdf:resource="descriptor/18025-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70324">
    <skos:narrower rdf:resource="descriptor/18914-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70324">
    <skos:narrower rdf:resource="descriptor/16540-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70324">
    <skos:narrower rdf:resource="descriptor/11558-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70324">
    <skos:narrower rdf:resource="descriptor/19372-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70324">
    <skos:narrower rdf:resource="descriptor/11567-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70324">
    <skos:narrower rdf:resource="descriptor/11566-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70324">
    <skos:narrower rdf:resource="descriptor/11565-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70324">
    <skos:narrower rdf:resource="descriptor/16481-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70324">
    <skos:narrower rdf:resource="descriptor/16480-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70324">
    <skos:narrower rdf:resource="descriptor/19509-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70324">
    <skos:narrower rdf:resource="descriptor/16479-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70324">
    <skos:narrower rdf:resource="descriptor/16477-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70324">
    <skos:narrower rdf:resource="descriptor/11564-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70324">
    <skos:narrower rdf:resource="descriptor/18909-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70324">
    <skos:narrower rdf:resource="descriptor/18911-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70324">
    <skos:narrower rdf:resource="descriptor/15911-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70324">
    <skos:narrower rdf:resource="descriptor/12592-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70324">
    <skos:narrower rdf:resource="descriptor/11388-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70324">
    <skos:narrower rdf:resource="descriptor/16529-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70324">
    <skos:narrower rdf:resource="descriptor/16533-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70324">
    <skos:narrower rdf:resource="descriptor/12609-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70324">
    <skos:narrower rdf:resource="descriptor/16475-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18879-2">
    <skos:prefLabel xml:lang="en">Student finance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18879-2">
    <skos:prefLabel xml:lang="de">Studienfinanzierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20756-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70578">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16671-1">
    <skos:altLabel xml:lang="de">Stadtstreicher</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16671-1">
    <skos:altLabel xml:lang="de">Obdachlose</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16671-1">
    <skos:altLabel xml:lang="de">Nichtsesshafte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16671-1">
    <skos:altLabel xml:lang="de">Nichtseßhafte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16671-1">
    <skos:altLabel xml:lang="de">Landstreicher</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18360-1">
    <skos:altLabel xml:lang="de">Strompolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18360-1">
    <skos:altLabel xml:lang="de">Elektrifizierungspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12134-5">
    <skos:prefLabel xml:lang="en">Process-oriented organization</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12134-5">
    <skos:prefLabel xml:lang="de">Ablauforganisation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19725-3">
    <skos:prefLabel xml:lang="en">Military science</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19725-3">
    <skos:prefLabel xml:lang="de">Militärwissenschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10560-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19484-6">
    <skos:altLabel xml:lang="de">Erosion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16201-3">
    <skos:prefLabel xml:lang="en">Economic crime</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16201-3">
    <skos:prefLabel xml:lang="de">Wirtschaftskriminalität</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15107-0">
    <skos:prefLabel xml:lang="en">Enterprise statistics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15107-0">
    <skos:prefLabel xml:lang="de">Statistik der Unternehmen und Arbeitsstätten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18518-6">
    <skos:prefLabel xml:lang="en">Coffee industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18518-6">
    <skos:prefLabel xml:lang="de">Kaffeeindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10037-6">
    <skos:altLabel xml:lang="de">Empirische Wirtschaftsforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10037-6">
    <skos:altLabel xml:lang="de">Empirische Strukturforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10037-6">
    <skos:altLabel xml:lang="de">Angewandte Wirtschaftsforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18324-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17724-1">
    <skos:related rdf:resource="descriptor/19176-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19076-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11541-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19663-6">
    <skos:scopeNote xml:lang="de">Kriterien für die Aufnahme neuer Mitglieder in die EU.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70561">
    <skos:prefLabel xml:lang="en">G.02.03  South Asia</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70561">
    <skos:prefLabel xml:lang="de">G.02.03  Südasien</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70847">
    <skos:broader rdf:resource="thsys/w"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15888-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16124-2">
    <skos:narrower rdf:resource="descriptor/16146-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16124-2">
    <skos:narrower rdf:resource="descriptor/16142-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16124-2">
    <skos:narrower rdf:resource="descriptor/16140-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16124-2">
    <skos:narrower rdf:resource="descriptor/19509-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16124-2">
    <skos:narrower rdf:resource="descriptor/12609-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11457-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16460-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12393-2">
    <skos:prefLabel xml:lang="en">Special purpose balance sheet</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12393-2">
    <skos:prefLabel xml:lang="de">Sonderbilanz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16245-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70940">
    <skos:prefLabel xml:lang="en">W.12.03  Freight transport</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70940">
    <skos:prefLabel xml:lang="de">W.12.03  Güterverkehr</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19197-2">
    <skos:broader rdf:resource="thsys/70493"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19197-2">
    <skos:broader rdf:resource="thsys/70049"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19197-2">
    <skos:broader rdf:resource="descriptor/10181-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19197-2">
    <skos:broader rdf:resource="descriptor/10177-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14055-5">
    <skos:prefLabel xml:lang="en">Cotton</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14055-5">
    <skos:prefLabel xml:lang="de">Baumwolle</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11669-6">
    <skos:broader rdf:resource="thsys/71074"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11669-6">
    <skos:broader rdf:resource="thsys/70046"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11669-6">
    <skos:broader rdf:resource="descriptor/11650-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18520-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71110">
    <skos:prefLabel xml:lang="en">V.04.03  Economic structure</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71110">
    <skos:prefLabel xml:lang="de">V.04.03  Wirtschaftsstruktur</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14769-1">
    <skos:narrower rdf:resource="descriptor/14252-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14769-1">
    <skos:narrower rdf:resource="descriptor/14766-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14181-0">
    <skos:altLabel xml:lang="de">Transurane</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14181-0">
    <skos:altLabel xml:lang="de">Spaltstoff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14181-0">
    <skos:altLabel xml:lang="de">Brutstoff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12533-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15797-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10722-5">
    <skos:related rdf:resource="descriptor/10678-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10722-5">
    <skos:related rdf:resource="descriptor/10773-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10722-5">
    <skos:related rdf:resource="descriptor/10770-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10722-5">
    <skos:related rdf:resource="descriptor/19337-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10722-5">
    <skos:related rdf:resource="descriptor/19263-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10722-5">
    <skos:related rdf:resource="descriptor/10676-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10722-5">
    <skos:related rdf:resource="descriptor/10745-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10722-5">
    <skos:related rdf:resource="descriptor/10719-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10514-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70186">
    <skos:broader rdf:resource="thsys/70005"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15970-3">
    <skos:broader rdf:resource="thsys/73373"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15970-3">
    <skos:broader rdf:resource="thsys/70154"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15970-3">
    <skos:broader rdf:resource="descriptor/16273-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10973-1">
    <skos:altLabel xml:lang="de">Mittelstandsförderung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#closeMatch">
    <rdfs:subPropertyOf rdf:resource="http://www.w3.org/2004/02/skos/core#mappingRelation"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13026-3">
    <skos:broader rdf:resource="thsys/71000"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13026-3">
    <skos:broader rdf:resource="descriptor/13037-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19634-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18922-6">
    <skos:altLabel xml:lang="de">Kulturausgaben</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18922-6">
    <skos:altLabel xml:lang="de">Kulturausgabe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10072-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19221-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19638-5">
    <skos:altLabel xml:lang="en">Population aging</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19638-5">
    <skos:altLabel xml:lang="en">Population ageing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19638-5">
    <skos:altLabel xml:lang="en">Aging society</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19638-5">
    <skos:altLabel xml:lang="de">Alterung der Bevölkerung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19638-5">
    <skos:altLabel xml:lang="de">Alternde Gesellschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19483-1">
    <skos:prefLabel xml:lang="en">Environmental planning</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19483-1">
    <skos:prefLabel xml:lang="de">Umweltplanung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19026-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18695-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10696-1">
    <skos:prefLabel xml:lang="en">Free trade</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10696-1">
    <skos:prefLabel xml:lang="de">Freihandel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13077-0">
    <skos:altLabel xml:lang="de">Elektroinstallation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10804-3">
    <skos:related rdf:resource="descriptor/10812-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10804-3">
    <skos:related rdf:resource="descriptor/10807-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10804-3">
    <skos:related rdf:resource="descriptor/10805-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10804-3">
    <skos:related rdf:resource="descriptor/10797-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14998-0">
    <skos:prefLabel xml:lang="en">Cigarette</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14998-0">
    <skos:prefLabel xml:lang="de">Zigarette</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11857-5">
    <skos:scopeNote xml:lang="de">Für Gebietskörperschaften BENUTZE Gebietskörperschaft und die entsprechenden Unterbegriffe.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13362-1">
    <skos:prefLabel xml:lang="en">Sports facilities</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13362-1">
    <skos:prefLabel xml:lang="de">Sportanlage</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70301">
    <skos:narrower rdf:resource="descriptor/12649-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70301">
    <skos:narrower rdf:resource="descriptor/10027-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70301">
    <skos:narrower rdf:resource="descriptor/12648-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70301">
    <skos:narrower rdf:resource="descriptor/20789-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70301">
    <skos:narrower rdf:resource="descriptor/18962-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70301">
    <skos:narrower rdf:resource="descriptor/18960-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70301">
    <skos:narrower rdf:resource="thsys/70394"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70301">
    <skos:narrower rdf:resource="thsys/70368"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13136-3">
    <skos:broader rdf:resource="thsys/70988"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13136-3">
    <skos:broader rdf:resource="descriptor/12993-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12270-4">
    <skos:related rdf:resource="descriptor/19143-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12270-4">
    <skos:related rdf:resource="descriptor/12276-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12270-4">
    <skos:related rdf:resource="descriptor/12271-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14948-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11365-5">
    <skos:related rdf:resource="descriptor/11267-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11365-5">
    <skos:related rdf:resource="descriptor/11228-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11365-5">
    <skos:related rdf:resource="descriptor/11288-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11365-5">
    <skos:related rdf:resource="descriptor/11358-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12823-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19583-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18758-0">
    <skos:broader rdf:resource="thsys/70883"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18758-0">
    <skos:broader rdf:resource="descriptor/12254-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21306-1">
    <skos:related rdf:resource="descriptor/18978-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21306-1">
    <skos:related rdf:resource="descriptor/12585-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21306-1">
    <skos:related rdf:resource="descriptor/20497-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21306-1">
    <skos:related rdf:resource="descriptor/24663-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21306-1">
    <skos:related rdf:resource="descriptor/18142-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16266-3">
    <skos:broader rdf:resource="thsys/73376"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16266-3">
    <skos:broader rdf:resource="thsys/71068"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16266-3">
    <skos:broader rdf:resource="descriptor/16226-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70649">
    <skos:narrower rdf:resource="descriptor/14216-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70649">
    <skos:narrower rdf:resource="descriptor/14214-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70649">
    <skos:narrower rdf:resource="descriptor/18009-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70649">
    <skos:narrower rdf:resource="descriptor/14212-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14967-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19395-5">
    <skos:related rdf:resource="descriptor/19792-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19395-5">
    <skos:related rdf:resource="descriptor/16004-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11367-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19174-0">
    <skos:related rdf:resource="descriptor/17989-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10550-1">
    <skos:broader rdf:resource="thsys/71089"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10550-1">
    <skos:broader rdf:resource="thsys/71085"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10550-1">
    <skos:broader rdf:resource="thsys/71084"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10550-1">
    <skos:broader rdf:resource="thsys/70902"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10550-1">
    <skos:broader rdf:resource="descriptor/13291-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12940-6">
    <skos:altLabel xml:lang="de">Gewächshaus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19630-0">
    <skos:prefLabel xml:lang="en">Wildlife conservation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19630-0">
    <skos:prefLabel xml:lang="de">Wildtierschutz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73341">
    <rdfs:label xml:lang="de">Informationstechnik und Systemarchitektur</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15142-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13810-0">
    <skos:prefLabel xml:lang="en">Liability insurance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13810-0">
    <skos:prefLabel xml:lang="de">Haftpflichtversicherung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14779-5">
    <skos:prefLabel xml:lang="en">Glassware</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14779-5">
    <skos:prefLabel xml:lang="de">Glaswaren</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18611-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18717-0">
    <skos:prefLabel xml:lang="en">Services management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18717-0">
    <skos:prefLabel xml:lang="de">Dienstleistungsmanagement</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70004">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13278-4">
    <skos:broader rdf:resource="thsys/71037"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13278-4">
    <skos:broader rdf:resource="thsys/70922"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13278-4">
    <skos:broader rdf:resource="descriptor/13270-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10159-6">
    <skos:prefLabel xml:lang="en">Utility function</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10159-6">
    <skos:prefLabel xml:lang="de">Nutzenfunktion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15097-5">
    <skos:broader rdf:resource="thsys/72184"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15097-5">
    <skos:broader rdf:resource="descriptor/15064-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18885-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70307">
    <rdfs:label xml:lang="de">Politische und ökonomische Staatengruppen</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10801-2">
    <skos:altLabel xml:lang="en">Eurocurrency market</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10801-2">
    <skos:altLabel xml:lang="de">Eurokapitalmarkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10801-2">
    <skos:altLabel xml:lang="de">Eurogeldmarkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10801-2">
    <skos:altLabel xml:lang="de">Eurofinanzmarkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10801-2">
    <skos:altLabel xml:lang="de">Eurodollarmarkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70942">
    <rdfs:label xml:lang="en">Transport mode</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70942">
    <rdfs:label xml:lang="de">Verkehrsträger</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10558-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13077-0">
    <skos:related rdf:resource="descriptor/14473-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13077-0">
    <skos:related rdf:resource="descriptor/13076-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13077-0">
    <skos:related rdf:resource="descriptor/13202-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13077-0">
    <skos:related rdf:resource="descriptor/13890-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16042-4">
    <skos:broader rdf:resource="thsys/70241"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16042-4">
    <skos:broader rdf:resource="descriptor/15966-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16297-6">
    <skos:scopeNote xml:lang="de">Nur für die Beziehungen zwischen Gesamtstaat und Regionen bzw. für Bestrebungen zur Dezentralisierung der staatlichen Verwaltung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13431-1">
    <skos:prefLabel xml:lang="en">Agricultural market</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13431-1">
    <skos:prefLabel xml:lang="de">Agrarmarkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19849-6">
    <skos:broader rdf:resource="thsys/70160"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19849-6">
    <skos:broader rdf:resource="descriptor/10011-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11834-3">
    <skos:altLabel xml:lang="de">Verkehrspreispolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11834-3">
    <skos:altLabel xml:lang="de">Beförderungspreis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14225-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14733-1">
    <skos:altLabel xml:lang="de">Sonnenschutzmittel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14733-1">
    <skos:altLabel xml:lang="de">Körperlotion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14733-1">
    <skos:altLabel xml:lang="de">Hautcreme</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16037-4">
    <skos:broader rdf:resource="thsys/73296"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16037-4">
    <skos:broader rdf:resource="thsys/70241"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16037-4">
    <skos:broader rdf:resource="descriptor/18176-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10503-3">
    <skos:altLabel xml:lang="de">Ländliches Gebiet</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10503-3">
    <skos:altLabel xml:lang="de">Agrarregion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70580">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13763-4">
    <skos:prefLabel xml:lang="en">Refinancing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13763-4">
    <skos:prefLabel xml:lang="de">Refinanzierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15543-1">
    <skos:broader rdf:resource="thsys/71035"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15543-1">
    <skos:broader rdf:resource="thsys/70269"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15543-1">
    <skos:broader rdf:resource="descriptor/15372-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15543-1">
    <skos:broader rdf:resource="descriptor/15055-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70199">
    <skos:prefLabel xml:lang="de">P.12.04  Pharmazeutische Produkte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12775-2">
    <skos:broader rdf:resource="thsys/70383"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12775-2">
    <skos:broader rdf:resource="thsys/70041"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18609-3">
    <skos:narrower rdf:resource="descriptor/13641-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13246-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13394-2">
    <skos:broader rdf:resource="thsys/73378"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13394-2">
    <skos:broader rdf:resource="thsys/70853"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13394-2">
    <skos:broader rdf:resource="descriptor/10984-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19123-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16026-2">
    <skos:altLabel xml:lang="de">Bürgerliches Gesetzbuch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16026-2">
    <skos:altLabel xml:lang="de">BGB (Bürgerliches Gesetzbuch)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10604-4">
    <skos:scopeNote xml:lang="de">Wird gebraucht im Sinne von Exportmarkt, d.h. es geht um Produkte, die im Inland hergestellt und dann exportiert werden.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14200-1">
    <skos:prefLabel xml:lang="en">Coal</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14200-1">
    <skos:prefLabel xml:lang="de">Kohle</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10824-4">
    <skos:altLabel xml:lang="en">Free investment area</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10824-4">
    <skos:altLabel xml:lang="de">Investitionsschutzabkommen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10824-4">
    <skos:altLabel xml:lang="de">Investitionsschutz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10824-4">
    <skos:altLabel xml:lang="de">Investitionsgarantiezone</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10824-4">
    <skos:altLabel xml:lang="de">Investitionsbürgschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14117-2">
    <skos:narrower rdf:resource="descriptor/14119-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14117-2">
    <skos:narrower rdf:resource="descriptor/14066-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14117-2">
    <skos:narrower rdf:resource="descriptor/14100-5"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id217">
    <rdf:value xml:lang="de">Für den Bereich der Europäischen Union</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12620-3">
    <skos:broader rdf:resource="thsys/73385"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12620-3">
    <skos:broader rdf:resource="thsys/71050"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12620-3">
    <skos:broader rdf:resource="thsys/70838"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15468-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11181-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19405-0">
    <skos:broader rdf:resource="thsys/73387"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19405-0">
    <skos:broader rdf:resource="thsys/71077"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19405-0">
    <skos:broader rdf:resource="thsys/70109"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19405-0">
    <skos:broader rdf:resource="descriptor/11533-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10608-3">
    <skos:altLabel xml:lang="en">Single market</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10608-3">
    <skos:altLabel xml:lang="de">Europäischer Binnenmarkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10608-3">
    <skos:altLabel xml:lang="de">EG-Binnenmarkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10608-3">
    <skos:altLabel xml:lang="de">Binnenmarkt der EU</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16721-5">
    <skos:scopeNote xml:lang="de">Annahme, dass es möglich ist, mit Hilfe der Vernunft die Wirklichkeit zu erkennen und angemessen zu handeln.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18127-0">
    <skos:altLabel xml:lang="en">Gresham's law</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18127-0">
    <skos:altLabel xml:lang="en">Dollarization</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18127-0">
    <skos:altLabel xml:lang="en">Currency composition</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18127-0">
    <skos:altLabel xml:lang="de">Währungsdualismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18127-0">
    <skos:altLabel xml:lang="de">Greshamsches Gesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18127-0">
    <skos:altLabel xml:lang="de">Dollarisierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18127-0">
    <skos:altLabel xml:lang="de">Dollarisation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70936">
    <skos:prefLabel xml:lang="en">W.12.01.03  Shipping</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70936">
    <skos:prefLabel xml:lang="de">W.12.01.03  Schifffahrt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#relatedMatch">
    <rdfs:label xml:lang="en">related match</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15937-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18797-4">
    <skos:related rdf:resource="descriptor/18564-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18797-4">
    <skos:related rdf:resource="descriptor/18799-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18797-4">
    <skos:related rdf:resource="descriptor/18798-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18797-4">
    <skos:related rdf:resource="descriptor/18499-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18797-4">
    <skos:related rdf:resource="descriptor/12783-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10649-3">
    <skos:related rdf:resource="descriptor/10713-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10649-3">
    <skos:related rdf:resource="descriptor/10710-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19763-2">
    <skos:related rdf:resource="descriptor/15979-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19763-2">
    <skos:related rdf:resource="descriptor/16198-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19763-2">
    <skos:related rdf:resource="descriptor/19511-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19763-2">
    <skos:related rdf:resource="descriptor/19813-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19763-2">
    <skos:related rdf:resource="descriptor/16196-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19763-2">
    <skos:related rdf:resource="descriptor/11156-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19763-2">
    <skos:related rdf:resource="descriptor/16195-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19610-6">
    <skos:related rdf:resource="descriptor/13173-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13460-1">
    <skos:prefLabel xml:lang="en">Retail and wholesale research</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13460-1">
    <skos:prefLabel xml:lang="de">Handelsforschung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14997-2">
    <skos:altLabel xml:lang="de">Schnupftabak</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14997-2">
    <skos:altLabel xml:lang="de">Rauchtabak</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14997-2">
    <skos:altLabel xml:lang="de">Kautabak</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11595-2">
    <skos:altLabel xml:lang="en">Planning-programming-budgeting system</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11595-2">
    <skos:altLabel xml:lang="en">Planning programming budgeting system</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11595-2">
    <skos:altLabel xml:lang="en">Planning programming budgeting</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11595-2">
    <skos:altLabel xml:lang="de">Programmhaushalt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11595-2">
    <skos:altLabel xml:lang="de">PPBS</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11595-2">
    <skos:altLabel xml:lang="de">Öffentlicher Finanzplan</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11595-2">
    <skos:altLabel xml:lang="de">Öffentliche Einnahmenplanung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11595-2">
    <skos:altLabel xml:lang="de">Öffentliche Ausgabenplanung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11595-2">
    <skos:altLabel xml:lang="de">Mittelfristige Finanzplanung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11595-2">
    <skos:altLabel xml:lang="de">Mehrjährige Finanzplanung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11595-2">
    <skos:altLabel xml:lang="de">Finanzplanung der Öffentlichen Hand</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11595-2">
    <skos:altLabel xml:lang="de">Einnahmenplanung der Öffentlichen Hand</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11595-2">
    <skos:altLabel xml:lang="de">Ausgabenplanung der Öffentlichen Hand</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18078-1">
    <skos:related rdf:resource="descriptor/18279-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18078-1">
    <skos:related rdf:resource="descriptor/14710-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12502-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73353">
    <skos:broader rdf:resource="thsys/73354"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20967-4">
    <skos:related rdf:resource="descriptor/19481-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20967-4">
    <skos:related rdf:resource="descriptor/15776-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20967-4">
    <skos:related rdf:resource="descriptor/20744-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18639-1">
    <skos:broader rdf:resource="thsys/70924"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18639-1">
    <skos:broader rdf:resource="descriptor/18101-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12997-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14822-2">
    <skos:prefLabel xml:lang="en">Plastic product</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14822-2">
    <skos:prefLabel xml:lang="de">Kunststoffwaren</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13904-5">
    <skos:broader rdf:resource="thsys/71056"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13904-5">
    <skos:broader rdf:resource="thsys/70988"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15774-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13492-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11706-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14252-3">
    <skos:related rdf:resource="descriptor/13024-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14252-3">
    <skos:related rdf:resource="descriptor/14251-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14252-3">
    <skos:related rdf:resource="descriptor/14021-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11516-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12640-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19691-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11061-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20261-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14920-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18454-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17686-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18555-0">
    <skos:broader rdf:resource="thsys/70947"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18555-0">
    <skos:broader rdf:resource="descriptor/12794-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13830-1">
    <skos:altLabel xml:lang="en">Total dwellings</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19080-2">
    <skos:prefLabel xml:lang="en">Capital contribution</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19080-2">
    <skos:prefLabel xml:lang="de">Einlage</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24708-1">
    <skos:broader rdf:resource="thsys/73297"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24708-1">
    <skos:broader rdf:resource="thsys/70882"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24708-1">
    <skos:broader rdf:resource="descriptor/12993-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11592-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14617-3">
    <skos:prefLabel xml:lang="en">Paper chemical</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14617-3">
    <skos:prefLabel xml:lang="de">Papierhilfsmittel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13840-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24785-4">
    <skos:broader rdf:resource="thsys/70292"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14281-3">
    <skos:altLabel xml:lang="de">Ferro-Silicon</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14281-3">
    <skos:altLabel xml:lang="de">Ferrosilicium</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14281-3">
    <skos:altLabel xml:lang="de">Ferrochrom</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15598-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12180-5">
    <skos:prefLabel xml:lang="en">Cooperative</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12180-5">
    <skos:prefLabel xml:lang="de">Genossenschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18345-4">
    <skos:broader rdf:resource="thsys/71017"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18345-4">
    <skos:broader rdf:resource="descriptor/12931-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18345-4">
    <skos:broader rdf:resource="descriptor/11627-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24665-0">
    <skos:altLabel xml:lang="en">Internet-only bank</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15555-1">
    <skos:related rdf:resource="descriptor/15429-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15555-1">
    <skos:related rdf:resource="descriptor/15554-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10728-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18201-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11596-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12313-5">
    <skos:altLabel xml:lang="en">International accounting standards</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12313-5">
    <skos:altLabel xml:lang="en">IFRS</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12313-5">
    <skos:altLabel xml:lang="en">IAS (International Accounting Standards)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12313-5">
    <skos:altLabel xml:lang="de">Rechnungslegungsstandards (IFRS)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12313-5">
    <skos:altLabel xml:lang="de">Internationale Rechnungslegungsstandards</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12313-5">
    <skos:altLabel xml:lang="de">Internationale Rechnungslegungsnorm</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12313-5">
    <skos:altLabel xml:lang="de">Internationale Rechnungslegungsgrundsätze</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19182-1">
    <skos:related rdf:resource="descriptor/14193-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19182-1">
    <skos:related rdf:resource="descriptor/18394-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19182-1">
    <skos:related rdf:resource="descriptor/12920-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19182-1">
    <skos:related rdf:resource="descriptor/19489-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19182-1">
    <skos:related rdf:resource="descriptor/13221-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19182-1">
    <skos:related rdf:resource="descriptor/19181-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19182-1">
    <skos:related rdf:resource="descriptor/18373-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19182-1">
    <skos:related rdf:resource="descriptor/19388-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19182-1">
    <skos:related rdf:resource="descriptor/14197-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19182-1">
    <skos:related rdf:resource="descriptor/18996-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18390-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70903">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11052-5">
    <skos:related rdf:resource="descriptor/12587-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11052-5">
    <skos:related rdf:resource="descriptor/12589-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11052-5">
    <skos:related rdf:resource="descriptor/19533-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11052-5">
    <skos:related rdf:resource="descriptor/12594-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11052-5">
    <skos:related rdf:resource="descriptor/15839-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11052-5">
    <skos:related rdf:resource="descriptor/12577-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11052-5">
    <skos:related rdf:resource="descriptor/12592-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11052-5">
    <skos:related rdf:resource="descriptor/15837-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11052-5">
    <skos:related rdf:resource="descriptor/25881-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11052-5">
    <skos:related rdf:resource="descriptor/24622-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11052-5">
    <skos:related rdf:resource="descriptor/20368-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11055-6">
    <skos:altLabel xml:lang="de">Freie Marktwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19307-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16122-6">
    <skos:altLabel xml:lang="de">Zollverordnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16122-6">
    <skos:altLabel xml:lang="de">Zollgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16122-6">
    <skos:altLabel xml:lang="de">Versandverfahren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16122-6">
    <skos:altLabel xml:lang="de">Allgemeine Zollordnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14463-4">
    <skos:prefLabel xml:lang="en">Watercraft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14463-4">
    <skos:prefLabel xml:lang="de">Wasserfahrzeug</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14652-1">
    <skos:altLabel xml:lang="de">Schwefelwasserstoff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14652-1">
    <skos:altLabel xml:lang="de">Oleum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10378-1">
    <skos:prefLabel xml:lang="en">Discretionary policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10378-1">
    <skos:prefLabel xml:lang="de">Diskretionäre Politik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12979-0">
    <skos:prefLabel xml:lang="en">Water industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12979-0">
    <skos:prefLabel xml:lang="de">Wasserwirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13573-2">
    <skos:related rdf:resource="descriptor/13577-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13573-2">
    <skos:related rdf:resource="descriptor/13510-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13573-2">
    <skos:related rdf:resource="descriptor/11951-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13573-2">
    <skos:related rdf:resource="descriptor/11869-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10907-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11506-6">
    <skos:broader rdf:resource="thsys/70355"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71094">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11400-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14911-3">
    <skos:broader rdf:resource="thsys/70031"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14911-3">
    <skos:broader rdf:resource="descriptor/14899-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16717-3">
    <skos:broader rdf:resource="thsys/73394"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16717-3">
    <skos:broader rdf:resource="descriptor/16719-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15771-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16946-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18895-4">
    <skos:broader rdf:resource="thsys/73347"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18895-4">
    <skos:broader rdf:resource="descriptor/15658-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12174-0">
    <skos:altLabel xml:lang="de">AG</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13689-4">
    <skos:related rdf:resource="descriptor/19604-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13689-4">
    <skos:related rdf:resource="descriptor/13283-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18995-0">
    <skos:scopeNote xml:lang="de">Nur für Literatur über den Rohstoffmarkt allgemein.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19825-6">
    <skos:related rdf:resource="descriptor/11295-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19825-6">
    <skos:related rdf:resource="descriptor/11389-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19825-6">
    <skos:related rdf:resource="descriptor/11290-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19825-6">
    <skos:related rdf:resource="descriptor/11355-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18700-3">
    <skos:broader rdf:resource="thsys/71043"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18700-3">
    <skos:broader rdf:resource="thsys/70951"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18700-3">
    <skos:broader rdf:resource="descriptor/12408-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73360">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18362-4">
    <skos:altLabel xml:lang="de">Energiepolitik zur Förderung regenerativer Energien</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18362-4">
    <skos:altLabel xml:lang="de">Alternative Energiepolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15842-5">
    <skos:broader rdf:resource="thsys/73385"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15842-5">
    <skos:broader rdf:resource="thsys/70244"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15842-5">
    <skos:broader rdf:resource="descriptor/15719-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15842-5">
    <skos:broader rdf:resource="descriptor/12577-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19320-1">
    <skos:related rdf:resource="descriptor/10713-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19320-1">
    <skos:related rdf:resource="descriptor/19324-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19320-1">
    <skos:related rdf:resource="descriptor/19314-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17948-0">
    <skos:narrower rdf:resource="descriptor/17951-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17948-0">
    <skos:narrower rdf:resource="descriptor/17950-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17948-0">
    <skos:narrower rdf:resource="descriptor/17949-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17948-0">
    <skos:narrower rdf:resource="descriptor/17947-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17948-0">
    <skos:narrower rdf:resource="descriptor/17946-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17948-0">
    <skos:narrower rdf:resource="descriptor/17943-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17948-0">
    <skos:narrower rdf:resource="descriptor/19169-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17948-0">
    <skos:narrower rdf:resource="descriptor/20987-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17948-0">
    <skos:narrower rdf:resource="descriptor/17942-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17948-0">
    <skos:narrower rdf:resource="descriptor/17941-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17948-0">
    <skos:narrower rdf:resource="descriptor/17940-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17948-0">
    <skos:narrower rdf:resource="descriptor/24788-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17948-0">
    <skos:narrower rdf:resource="descriptor/19168-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17948-0">
    <skos:narrower rdf:resource="descriptor/17938-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17948-0">
    <skos:narrower rdf:resource="descriptor/17937-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12311-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18151-3">
    <skos:prefLabel xml:lang="en">Economic principle</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18151-3">
    <skos:prefLabel xml:lang="de">Ökonomisches Prinzip</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17990-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13957-5">
    <skos:prefLabel xml:lang="en">Architects</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13957-5">
    <skos:prefLabel xml:lang="de">Architekten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16698-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11866-4">
    <skos:narrower rdf:resource="descriptor/18967-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18183-4">
    <skos:altLabel xml:lang="en">Green NNP</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18183-4">
    <skos:altLabel xml:lang="en">Green net national product</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18183-4">
    <skos:altLabel xml:lang="de">Umwelt-Satellitensystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18183-4">
    <skos:altLabel xml:lang="de">Umweltökonomische Gesamtrechnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18183-4">
    <skos:altLabel xml:lang="de">Umweltgesamtrechnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18183-4">
    <skos:altLabel xml:lang="de">Öko-Sozialprodukt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18613-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71107">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18736-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18989-2">
    <skos:prefLabel xml:lang="en">Energy consumption</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18989-2">
    <skos:prefLabel xml:lang="de">Energiekonsum</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16727-0">
    <skos:scopeNote xml:lang="de">Literatur zu methodischen und methodologischen Fragen der Wirtschaftswissenschaft, aber auch anderer Disziplinen oder der Wissenschaft im allgemeinen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16676-5">
    <skos:narrower rdf:resource="descriptor/18896-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16797-0">
    <skos:broader rdf:resource="thsys/70822"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16959-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18011-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18770-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11066-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19446-0">
    <skos:related rdf:resource="descriptor/19829-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19446-0">
    <skos:related rdf:resource="descriptor/19448-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19446-0">
    <skos:related rdf:resource="descriptor/18837-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19446-0">
    <skos:related rdf:resource="descriptor/19468-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19446-0">
    <skos:related rdf:resource="descriptor/18806-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19446-0">
    <skos:related rdf:resource="descriptor/19378-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19446-0">
    <skos:related rdf:resource="descriptor/11817-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19446-0">
    <skos:related rdf:resource="descriptor/11981-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10490-0">
    <skos:related rdf:resource="descriptor/18377-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10490-0">
    <skos:related rdf:resource="descriptor/13179-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10490-0">
    <skos:related rdf:resource="descriptor/14188-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10490-0">
    <skos:related rdf:resource="descriptor/12024-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10490-0">
    <skos:related rdf:resource="descriptor/18326-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18151-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10089-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71068">
    <skos:broader rdf:resource="thsys/71069"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70180">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10728-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18587-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19527-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73371">
    <skos:narrower rdf:resource="descriptor/18379-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73371">
    <skos:narrower rdf:resource="descriptor/18364-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73371">
    <skos:narrower rdf:resource="descriptor/15768-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73371">
    <skos:narrower rdf:resource="descriptor/15752-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73371">
    <skos:narrower rdf:resource="descriptor/11810-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73371">
    <skos:narrower rdf:resource="descriptor/20722-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73371">
    <skos:narrower rdf:resource="descriptor/18958-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19250-3">
    <skos:broader rdf:resource="thsys/70233"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11915-3">
    <skos:prefLabel xml:lang="en">Location choice</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11915-3">
    <skos:prefLabel xml:lang="de">Standortwahl</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12712-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15350-5">
    <skos:broader rdf:resource="thsys/72394"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15350-5">
    <skos:broader rdf:resource="descriptor/15405-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11850-5">
    <skos:related rdf:resource="descriptor/13832-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11850-5">
    <skos:related rdf:resource="descriptor/19628-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11850-5">
    <skos:related rdf:resource="descriptor/19479-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11850-5">
    <skos:related rdf:resource="descriptor/13639-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11850-5">
    <skos:related rdf:resource="descriptor/13592-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11850-5">
    <skos:related rdf:resource="descriptor/18634-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11850-5">
    <skos:related rdf:resource="descriptor/12907-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11850-5">
    <skos:related rdf:resource="descriptor/11194-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14600-6">
    <skos:altLabel xml:lang="de">Reizgas</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14600-6">
    <skos:altLabel xml:lang="de">Nervengift</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14600-6">
    <skos:altLabel xml:lang="de">Giftgas</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14600-6">
    <skos:altLabel xml:lang="de">C-Waffe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14600-6">
    <skos:altLabel xml:lang="de">Chemische Waffe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14600-6">
    <skos:altLabel xml:lang="de">Chemiewaffe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16014-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17632-6">
    <skos:altLabel xml:lang="de">Sultanat Oman</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17632-6">
    <skos:altLabel xml:lang="de">Maskat und Oman</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19814-4">
    <skos:broader rdf:resource="thsys/73334"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19814-4">
    <skos:broader rdf:resource="descriptor/16195-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12651-6">
    <skos:broader rdf:resource="thsys/70368"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12651-6">
    <skos:broader rdf:resource="descriptor/12088-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12651-6">
    <skos:broader rdf:resource="descriptor/12640-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18430-6">
    <skos:narrower rdf:resource="descriptor/19785-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15149-5">
    <skos:related rdf:resource="descriptor/15350-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15149-5">
    <skos:related rdf:resource="descriptor/19796-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19736-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18352-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10913-5">
    <skos:broader rdf:resource="thsys/70212"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11596-0">
    <skos:prefLabel xml:lang="en">Public Budget</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11596-0">
    <skos:prefLabel xml:lang="de">Öffentlicher Haushalt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18823-1">
    <skos:scopeNote xml:lang="de">Betriebswirtschaftliche Analyse des öffentlichen Bereichs, der die öffentliche Verwaltung, die öffentlichen und gemeinwirtschaftlichen Unternehmen sowie den sonstigen Nonprofit-Sektor umfasst.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10182-4">
    <zbwext:useInsteadNote rdf:nodeID="id216"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17688-0">
    <skos:broader rdf:resource="thsys/70429"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17688-0">
    <skos:broader rdf:resource="descriptor/17687-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14509-6">
    <skos:broader rdf:resource="thsys/70477"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14509-6">
    <skos:broader rdf:resource="descriptor/14337-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14509-6">
    <skos:broader rdf:resource="descriptor/14502-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14385-5">
    <skos:broader rdf:resource="thsys/70318"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14385-5">
    <skos:broader rdf:resource="descriptor/14381-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10027-2">
    <skos:related rdf:resource="descriptor/12012-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10027-2">
    <skos:related rdf:resource="descriptor/12649-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10027-2">
    <skos:related rdf:resource="descriptor/12648-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10027-2">
    <skos:related rdf:resource="descriptor/20789-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19696-5">
    <skos:prefLabel xml:lang="en">Party politics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19696-5">
    <skos:prefLabel xml:lang="de">Parteipolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15700-4">
    <skos:altLabel xml:lang="de">Geschichte des Zwanzigsten Jahrhunderts</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15700-4">
    <skos:altLabel xml:lang="de">Geschichte der Gegenwart</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15700-4">
    <skos:altLabel xml:lang="de">Gegenwartsgeschichte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11484-4">
    <skos:broader rdf:resource="thsys/70305"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11484-4">
    <skos:broader rdf:resource="descriptor/10382-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18909-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13281-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15868-1">
    <skos:prefLabel xml:lang="en">River</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15868-1">
    <skos:prefLabel xml:lang="de">Fluss</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18653-0">
    <skos:altLabel xml:lang="de">Fernmeldepolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73388">
    <skos:narrower rdf:resource="descriptor/15682-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73388">
    <skos:narrower rdf:resource="descriptor/15678-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73388">
    <skos:narrower rdf:resource="descriptor/19737-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73388">
    <skos:narrower rdf:resource="descriptor/13399-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73388">
    <skos:narrower rdf:resource="descriptor/15671-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73388">
    <skos:narrower rdf:resource="descriptor/19740-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73388">
    <skos:narrower rdf:resource="descriptor/19738-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73388">
    <skos:narrower rdf:resource="descriptor/13393-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73388">
    <skos:narrower rdf:resource="descriptor/19736-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73388">
    <skos:narrower rdf:resource="descriptor/15648-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73388">
    <skos:narrower rdf:resource="descriptor/15647-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73388">
    <skos:narrower rdf:resource="descriptor/15646-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73388">
    <skos:narrower rdf:resource="descriptor/14012-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73388">
    <skos:narrower rdf:resource="descriptor/16401-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73388">
    <skos:narrower rdf:resource="descriptor/19739-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73388">
    <skos:narrower rdf:resource="descriptor/15640-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73388">
    <skos:narrower rdf:resource="descriptor/15637-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73388">
    <skos:narrower rdf:resource="descriptor/19735-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14729-6">
    <skos:altLabel xml:lang="de">Tranquilizer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71066">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11061-4">
    <skos:altLabel xml:lang="en">Soviet type economy</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11061-4">
    <skos:altLabel xml:lang="en">Planned economy</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11061-4">
    <skos:altLabel xml:lang="en">Command economy</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11061-4">
    <skos:altLabel xml:lang="de">Zentralverwaltungswirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11061-4">
    <skos:altLabel xml:lang="de">Zentrale Planwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11061-4">
    <skos:altLabel xml:lang="de">Sozialistisches Wirtschaftssystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11061-4">
    <skos:altLabel xml:lang="de">Sozialistische Volkswirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11061-4">
    <skos:altLabel xml:lang="de">Sozialistische Produktionsverhältnisse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11061-4">
    <skos:altLabel xml:lang="de">Sozialistische Planwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11061-4">
    <skos:altLabel xml:lang="de">Kommunistische Wirtschaftsordnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11001-1">
    <skos:broader rdf:resource="thsys/71101"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11001-1">
    <skos:broader rdf:resource="descriptor/11030-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12796-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11362-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15777-4">
    <skos:related rdf:resource="descriptor/11827-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19077-5">
    <skos:scopeNote xml:lang="de">Wird sowohl für die geographische wie für die politische Abgrenzung der Region verwendet</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19364-2">
    <skos:narrower rdf:resource="descriptor/11993-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71010">
    <rdfs:label xml:lang="de">Rohstoffwirtschaft und Bergbau</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15519-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id378">
    <rdfs:seeAlso rdf:resource="descriptor/10062-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13029-4">
    <skos:related rdf:resource="descriptor/13039-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13029-4">
    <skos:related rdf:resource="descriptor/18406-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13029-4">
    <skos:related rdf:resource="descriptor/19051-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13029-4">
    <skos:related rdf:resource="descriptor/18405-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13029-4">
    <skos:related rdf:resource="descriptor/19390-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13029-4">
    <skos:related rdf:resource="descriptor/13236-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13029-4">
    <skos:related rdf:resource="descriptor/14285-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16171-0">
    <skos:prefLabel xml:lang="en">Road traffic regulations</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16171-0">
    <skos:prefLabel xml:lang="de">Straßenverkehrsrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10180-1">
    <skos:scopeNote xml:lang="de">Entstehung einer Differentialrente zugunsten des Verbrauchers.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11046-0">
    <skos:prefLabel xml:lang="en">Capitalism</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11046-0">
    <skos:prefLabel xml:lang="de">Kapitalismus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11060-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10460-2">
    <skos:prefLabel xml:lang="en">Productivity</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10460-2">
    <skos:prefLabel xml:lang="de">Produktivität</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15108-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18340-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18203-3">
    <skos:broader rdf:resource="thsys/70651"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18203-3">
    <skos:broader rdf:resource="descriptor/18024-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18203-3">
    <skos:broader rdf:resource="descriptor/14150-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18162-5">
    <skos:prefLabel xml:lang="en">Strategic mineral</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18162-5">
    <skos:prefLabel xml:lang="de">Strategisches Mineral</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16695-1">
    <skos:related rdf:resource="descriptor/10470-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16695-1">
    <skos:related rdf:resource="descriptor/10469-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16695-1">
    <skos:related rdf:resource="descriptor/10466-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16695-1">
    <skos:related rdf:resource="descriptor/16696-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15760-0">
    <skos:broader rdf:resource="thsys/71081"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15760-0">
    <skos:broader rdf:resource="thsys/70239"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15760-0">
    <skos:broader rdf:resource="thsys/70209"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15760-0">
    <skos:broader rdf:resource="descriptor/15768-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14095-0">
    <skos:prefLabel xml:lang="en">Rice</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14095-0">
    <skos:prefLabel xml:lang="de">Reis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13694-4">
    <skos:scopeNote xml:lang="de">Nur für die Sicherung von Bankeinlagen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14788-4">
    <skos:related rdf:resource="descriptor/14810-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14788-4">
    <skos:related rdf:resource="descriptor/12912-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14788-4">
    <skos:related rdf:resource="descriptor/18204-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14788-4">
    <skos:related rdf:resource="descriptor/14150-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14788-4">
    <skos:related rdf:resource="descriptor/14792-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73375">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.04.02.07</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16395-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11039-4">
    <skos:prefLabel xml:lang="en">Economic efficiency</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11039-4">
    <skos:prefLabel xml:lang="de">Wirtschaftliche Effizienz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16497-5">
    <skos:related rdf:resource="descriptor/16440-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16497-5">
    <skos:related rdf:resource="descriptor/16288-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16497-5">
    <skos:related rdf:resource="descriptor/16417-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16497-5">
    <skos:related rdf:resource="descriptor/19715-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16497-5">
    <skos:related rdf:resource="descriptor/19708-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16497-5">
    <skos:related rdf:resource="descriptor/16455-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16497-5">
    <skos:related rdf:resource="descriptor/19851-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16497-5">
    <skos:related rdf:resource="descriptor/16204-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16497-5">
    <skos:related rdf:resource="descriptor/16391-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16497-5">
    <skos:related rdf:resource="descriptor/16427-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16497-5">
    <skos:related rdf:resource="descriptor/16445-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19104-0">
    <skos:related rdf:resource="descriptor/12079-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10976-2">
    <skos:related rdf:resource="descriptor/11069-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10976-2">
    <skos:related rdf:resource="descriptor/11068-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10976-2">
    <skos:related rdf:resource="descriptor/11063-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10976-2">
    <skos:related rdf:resource="descriptor/11127-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10976-2">
    <skos:related rdf:resource="descriptor/10978-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10976-2">
    <skos:related rdf:resource="descriptor/11066-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11822-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13400-5">
    <skos:altLabel xml:lang="en">Environmental movement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13400-5">
    <skos:altLabel xml:lang="en">Environmental interest group</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13400-5">
    <skos:altLabel xml:lang="de">Umweltverband</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13400-5">
    <skos:altLabel xml:lang="de">Umweltschutzorganisation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13400-5">
    <skos:altLabel xml:lang="de">Ökologische Bewegung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13400-5">
    <skos:altLabel xml:lang="de">Grüne Bewegung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13691-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19150-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13481-0">
    <skos:prefLabel xml:lang="en">Shopping center</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13481-0">
    <skos:prefLabel xml:lang="de">Einkaufszentrum</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13633-3">
    <skos:prefLabel xml:lang="en">Parking</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13633-3">
    <skos:prefLabel xml:lang="de">Ruhender Verkehr</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16562-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15998-2">
    <zbwext:useInsteadNote rdf:nodeID="id125"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16972-1">
    <skos:broader rdf:resource="thsys/70003"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16972-1">
    <skos:broader rdf:resource="descriptor/16825-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16972-1">
    <skos:broader rdf:resource="descriptor/17982-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11349-3">
    <skos:prefLabel xml:lang="en">Wage autonomy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11349-3">
    <skos:prefLabel xml:lang="de">Tarifautonomie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10506-4">
    <skos:related rdf:resource="descriptor/10514-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10506-4">
    <skos:related rdf:resource="descriptor/10511-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17461-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#example">
    <rdf:type rdf:resource="http://www.w3.org/1999/02/22-rdf-syntax-ns#Property"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#example">
    <rdf:type rdf:resource="http://www.w3.org/2002/07/owl#AnnotationProperty"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11052-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18059-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19274-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19165-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18410-5">
    <skos:related rdf:resource="descriptor/14039-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18410-5">
    <skos:related rdf:resource="descriptor/18359-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16066-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id294">
    <rdf:value xml:lang="de">Für Selbstbeschränkungsabkommen im Außenhandel</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10213-5">
    <skos:broader rdf:resource="thsys/70355"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10213-5">
    <skos:broader rdf:resource="thsys/70178"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10213-5">
    <skos:broader rdf:resource="thsys/70040"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14888-0">
    <skos:prefLabel xml:lang="en">Fabric</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14888-0">
    <skos:prefLabel xml:lang="de">Stoff</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70959">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14181-0">
    <skos:prefLabel xml:lang="en">Nuclear fuel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14181-0">
    <skos:prefLabel xml:lang="de">Kernbrennstoff</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18487-5">
    <skos:related rdf:resource="descriptor/18484-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18487-5">
    <skos:related rdf:resource="descriptor/11198-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11903-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18987-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16001-4">
    <skos:related rdf:resource="descriptor/12605-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16001-4">
    <skos:related rdf:resource="descriptor/18368-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16001-4">
    <skos:related rdf:resource="descriptor/13003-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16001-4">
    <skos:related rdf:resource="descriptor/14035-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16001-4">
    <skos:related rdf:resource="descriptor/18367-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16001-4">
    <skos:related rdf:resource="descriptor/14182-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11421-0">
    <skos:broader rdf:resource="thsys/71108"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11421-0">
    <skos:broader rdf:resource="descriptor/13723-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18101-4">
    <skos:broader rdf:resource="thsys/70958"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18101-4">
    <skos:broader rdf:resource="descriptor/12766-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18952-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15119-0">
    <skos:broader rdf:resource="thsys/72183"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15119-0">
    <skos:broader rdf:resource="thsys/71035"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15119-0">
    <skos:broader rdf:resource="descriptor/15115-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16327-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15810-4">
    <zbwext:useInsteadNote rdf:nodeID="id298"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18555-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10488-1">
    <skos:related rdf:resource="descriptor/16619-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10488-1">
    <skos:related rdf:resource="descriptor/18314-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10488-1">
    <skos:related rdf:resource="descriptor/15718-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10488-1">
    <skos:related rdf:resource="descriptor/16645-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10488-1">
    <skos:related rdf:resource="descriptor/16671-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10488-1">
    <skos:related rdf:resource="descriptor/19553-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10488-1">
    <skos:related rdf:resource="descriptor/19840-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10488-1">
    <skos:related rdf:resource="descriptor/10498-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10488-1">
    <skos:related rdf:resource="descriptor/19506-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10488-1">
    <skos:related rdf:resource="descriptor/16533-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10202-3">
    <skos:broader rdf:resource="thsys/70041"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14033-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20189-5">
    <skos:broader rdf:resource="thsys/71075"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20189-5">
    <skos:broader rdf:resource="descriptor/11547-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70944">
    <rdfs:label xml:lang="en">Hospitality industry</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70944">
    <rdfs:label xml:lang="de">Gastgewerbe</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id345">
    <rdfs:seeAlso rdf:resource="descriptor/10966-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14383-2">
    <skos:altLabel xml:lang="de">Industrienähmaschine</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14383-2">
    <skos:altLabel xml:lang="de">Haushaltsnähmaschine</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18949-0">
    <skos:broader rdf:resource="thsys/70822"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11235-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18264-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18972-5">
    <skos:altLabel xml:lang="de">Familiäre Krankenpflege</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18972-5">
    <skos:altLabel xml:lang="de">Familiäre Altenversorgung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18972-5">
    <skos:altLabel xml:lang="de">Familiäre Altenpflege</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18972-5">
    <skos:altLabel xml:lang="de">Familiale Betreuung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20897-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11895-4">
    <skos:altLabel xml:lang="de">Regionalentwicklung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11895-4">
    <skos:altLabel xml:lang="de">Regionale Wirtschaftsentwicklung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11895-4">
    <skos:altLabel xml:lang="de">Räumliche Wirtschaftsentwicklung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11895-4">
    <skos:altLabel xml:lang="de">Räumliche Entwicklung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id355">
    <rdf:value xml:lang="de">Für Kriegs- bzw. Rüstungswaffen</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12976-6">
    <skos:broader rdf:resource="thsys/71013"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12976-6">
    <skos:broader rdf:resource="thsys/70238"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12976-6">
    <skos:broader rdf:resource="descriptor/13003-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12976-6">
    <skos:broader rdf:resource="descriptor/24621-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18570-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18606-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19567-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18542-2">
    <skos:altLabel xml:lang="de">Marktsegmentierung von Auslandsmärkten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18542-2">
    <skos:altLabel xml:lang="de">Ländersegmentierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70853">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12108-6">
    <skos:broader rdf:resource="thsys/73385"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12108-6">
    <skos:broader rdf:resource="thsys/70485"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12064-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14622-3">
    <skos:altLabel xml:lang="de">WC-Reiniger</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14622-3">
    <skos:altLabel xml:lang="de">Schuhpflegemittel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14622-3">
    <skos:altLabel xml:lang="de">Rohrreiniger</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14622-3">
    <skos:altLabel xml:lang="de">Putzmittel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14622-3">
    <skos:altLabel xml:lang="de">Putz- und Pflegemittel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14622-3">
    <skos:altLabel xml:lang="de">Poliermittel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14622-3">
    <skos:altLabel xml:lang="de">Möbelpflegemittel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14622-3">
    <skos:altLabel xml:lang="de">Haushaltsreiniger</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14622-3">
    <skos:altLabel xml:lang="de">Fußbodenreiniger</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14622-3">
    <skos:altLabel xml:lang="de">Fleckenentferner</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14622-3">
    <skos:altLabel xml:lang="de">Fensterputzmittel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11962-1">
    <skos:prefLabel xml:lang="en">Regional statistics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11962-1">
    <skos:prefLabel xml:lang="de">Regionalstatistik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15064-6">
    <skos:broader rdf:resource="thsys/72182"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15064-6">
    <skos:broader rdf:resource="descriptor/16727-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15480-6">
    <skos:prefLabel xml:lang="en">Risk attitude</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15480-6">
    <skos:prefLabel xml:lang="de">Risikopräferenz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16963-2">
    <skos:prefLabel xml:lang="en">Grisons</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16963-2">
    <skos:prefLabel xml:lang="de">Graubünden</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14899-2">
    <skos:altLabel xml:lang="en">Garment</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14899-2">
    <skos:altLabel xml:lang="en">Apparel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70056">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19623-4">
    <skos:related rdf:resource="descriptor/18254-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19623-4">
    <skos:related rdf:resource="descriptor/15743-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19623-4">
    <skos:related rdf:resource="descriptor/18268-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19623-4">
    <skos:related rdf:resource="descriptor/15742-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19623-4">
    <skos:related rdf:resource="descriptor/14603-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12189-1">
    <skos:prefLabel xml:lang="en">Public Law Corporation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12189-1">
    <skos:prefLabel xml:lang="de">Öffentlich-rechtliches Unternehmen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12306-2">
    <skos:prefLabel xml:lang="en">Expenditure</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12306-2">
    <skos:prefLabel xml:lang="de">Ausgaben</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18401-6">
    <skos:scopeNote xml:lang="de">Befasst sich mit Eigenschaften und Aufbau der Metalle und Legierungen sowie mit ihrer technischen und wirtschaftlichen Verwendung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16493-6">
    <skos:altLabel xml:lang="de">Konsulargesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16493-6">
    <skos:altLabel xml:lang="de">Diplomatisches Schutzrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16493-6">
    <skos:altLabel xml:lang="de">Diplomatische Immunität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16493-6">
    <skos:altLabel xml:lang="de">Diplomatenrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11931-5">
    <skos:altLabel xml:lang="en">Spillover</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11931-5">
    <skos:altLabel xml:lang="de">Überschwappeffekt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11931-5">
    <skos:altLabel xml:lang="de">Spillover Effekt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11931-5">
    <skos:altLabel xml:lang="de">Ausstrahlungseffekt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12602-5">
    <skos:prefLabel xml:lang="en">Human engineering</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12602-5">
    <skos:prefLabel xml:lang="de">Arbeitswissenschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18870-6">
    <skos:related rdf:resource="descriptor/11404-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18870-6">
    <skos:related rdf:resource="descriptor/11377-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70899">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.14.03.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70878">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11521-3">
    <skos:broader rdf:resource="thsys/70305"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11521-3">
    <skos:broader rdf:resource="descriptor/10994-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11521-3">
    <skos:broader rdf:resource="descriptor/10382-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18589-4">
    <skos:broader rdf:resource="thsys/70939"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18589-4">
    <skos:broader rdf:resource="thsys/70936"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18589-4">
    <skos:broader rdf:resource="descriptor/13256-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18589-4">
    <skos:broader rdf:resource="descriptor/13632-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16405-1">
    <skos:prefLabel xml:lang="en">Ideology</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16405-1">
    <skos:prefLabel xml:lang="de">Ideologie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id367">
    <rdf:value xml:lang="de">für direkte staatliche Beihilfen</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70947">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13393-4">
    <skos:related rdf:resource="descriptor/17017-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13393-4">
    <skos:related rdf:resource="descriptor/15678-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13393-4">
    <skos:related rdf:resource="descriptor/19373-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13393-4">
    <skos:related rdf:resource="descriptor/11665-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13393-4">
    <skos:related rdf:resource="descriptor/16002-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13393-4">
    <skos:related rdf:resource="descriptor/19706-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13393-4">
    <skos:related rdf:resource="descriptor/15640-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10637-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14603-0">
    <skos:prefLabel xml:lang="en">Gases</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14603-0">
    <skos:prefLabel xml:lang="de">Gas</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24859-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19444-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16100-2">
    <skos:related rdf:resource="descriptor/19857-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16100-2">
    <skos:related rdf:resource="descriptor/16017-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16100-2">
    <skos:related rdf:resource="descriptor/11541-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16100-2">
    <skos:related rdf:resource="descriptor/11596-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16100-2">
    <skos:related rdf:resource="descriptor/11539-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16100-2">
    <skos:related rdf:resource="descriptor/19380-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15600-1">
    <skos:narrower rdf:resource="descriptor/18102-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13330-0">
    <skos:altLabel xml:lang="de">Reinigungsdienst</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13330-0">
    <skos:altLabel xml:lang="de">Raumpflege</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13330-0">
    <skos:altLabel xml:lang="de">Putzdienst</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11357-4">
    <skos:prefLabel xml:lang="en">General schooling</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11357-4">
    <skos:prefLabel xml:lang="de">Allgemeinbildende Schule</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12913-2">
    <skos:broader rdf:resource="thsys/71005"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15638-4">
    <skos:narrower rdf:resource="descriptor/18430-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11001-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19341-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10899-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19796-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10923-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20022-6">
    <skos:broader rdf:resource="thsys/73393"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20022-6">
    <skos:broader rdf:resource="thsys/70049"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20022-6">
    <skos:broader rdf:resource="descriptor/16727-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14093-4">
    <skos:broader rdf:resource="thsys/70070"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14093-4">
    <skos:broader rdf:resource="descriptor/14088-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13644-5">
    <skos:related rdf:resource="descriptor/12927-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13644-5">
    <skos:related rdf:resource="descriptor/19466-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13644-5">
    <skos:related rdf:resource="descriptor/13639-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13644-5">
    <skos:related rdf:resource="descriptor/13638-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13644-5">
    <skos:related rdf:resource="descriptor/13592-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13644-5">
    <skos:related rdf:resource="descriptor/13966-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13644-5">
    <skos:related rdf:resource="descriptor/11194-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12588-1">
    <skos:broader rdf:resource="thsys/73385"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12588-1">
    <skos:broader rdf:resource="thsys/70244"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20125-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12091-4">
    <skos:scopeNote xml:lang="de">Der Organisationsbegriff umfasst sowohl die instrumental-funktionelle wie die institutionelle Bedeutung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13985-0">
    <skos:narrower rdf:resource="descriptor/13993-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13985-0">
    <skos:narrower rdf:resource="descriptor/12125-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11357-4">
    <skos:altLabel xml:lang="en">Secondary school</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11357-4">
    <skos:altLabel xml:lang="en">Primary education</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11357-4">
    <skos:altLabel xml:lang="de">Sonderschule</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11357-4">
    <skos:altLabel xml:lang="de">Sekundarschule</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11357-4">
    <skos:altLabel xml:lang="de">Schule für Behinderte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11357-4">
    <skos:altLabel xml:lang="de">Realschule</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11357-4">
    <skos:altLabel xml:lang="de">Primarschule</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11357-4">
    <skos:altLabel xml:lang="de">Hauptschule</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11357-4">
    <skos:altLabel xml:lang="de">Gymnasium</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11357-4">
    <skos:altLabel xml:lang="de">Grundschule</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11357-4">
    <skos:altLabel xml:lang="de">Gesamtschule</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11357-4">
    <skos:altLabel xml:lang="de">Allgemeinbildendes Schulwesen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19615-3">
    <skos:altLabel xml:lang="de">Seismologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19615-3">
    <skos:altLabel xml:lang="de">Seebeben</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15082-4">
    <skos:altLabel xml:lang="de">Mengenindex</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15082-4">
    <skos:altLabel xml:lang="de">Indexziffer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15082-4">
    <skos:altLabel xml:lang="de">Indexzahl</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11991-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13351-6">
    <skos:related rdf:resource="descriptor/16771-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13351-6">
    <skos:related rdf:resource="descriptor/16802-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13351-6">
    <skos:related rdf:resource="descriptor/18945-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13351-6">
    <skos:related rdf:resource="descriptor/18952-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70238">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11859-1">
    <skos:narrower rdf:resource="descriptor/19453-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11859-1">
    <skos:narrower rdf:resource="descriptor/12032-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11859-1">
    <skos:narrower rdf:resource="descriptor/19479-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11859-1">
    <skos:narrower rdf:resource="descriptor/18748-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10271-5">
    <skos:prefLabel xml:lang="en">Flow-of-funds</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10271-5">
    <skos:prefLabel xml:lang="de">Volkswirtschaftliche Finanzierungsrechnung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17707-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19496-6">
    <skos:altLabel xml:lang="en">Secondary employment</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19496-6">
    <skos:altLabel xml:lang="en">Second job</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19496-6">
    <skos:altLabel xml:lang="en">Off-duty employment</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19496-6">
    <skos:altLabel xml:lang="en">Dual job holding</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19496-6">
    <skos:altLabel xml:lang="de">Nebenerwerbstätigkeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19496-6">
    <skos:altLabel xml:lang="de">Nebenbeschäftigung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19496-6">
    <skos:altLabel xml:lang="de">Nebenberuf</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19496-6">
    <skos:altLabel xml:lang="de">Mehrfacherwerbstätigkeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11286-1">
    <skos:related rdf:resource="descriptor/11282-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11286-1">
    <skos:related rdf:resource="descriptor/11070-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11286-1">
    <skos:related rdf:resource="descriptor/11285-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18755-6">
    <skos:prefLabel xml:lang="en">Rent regulation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18755-6">
    <skos:prefLabel xml:lang="de">Mietenpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17743-4">
    <skos:broader rdf:resource="thsys/71031"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17743-4">
    <skos:broader rdf:resource="descriptor/17652-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13492-2">
    <skos:prefLabel xml:lang="en">Mail order business</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13492-2">
    <skos:prefLabel xml:lang="de">Versandhandel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17616-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18427-2">
    <skos:broader rdf:resource="thsys/70799"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18427-2">
    <skos:broader rdf:resource="descriptor/13081-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18427-2">
    <skos:broader rdf:resource="descriptor/18424-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16990-6">
    <skos:altLabel xml:lang="de">Skandinavische Staaten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16990-6">
    <skos:altLabel xml:lang="de">Skandinavische Länder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16990-6">
    <skos:altLabel xml:lang="de">Skandinavien</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15062-3">
    <skos:prefLabel xml:lang="en">Robust method</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15062-3">
    <skos:prefLabel xml:lang="de">Robustes Verfahren</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11804-5">
    <skos:broader rdf:resource="thsys/71020"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11804-5">
    <skos:broader rdf:resource="descriptor/11801-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11068-4">
    <skos:prefLabel xml:lang="en">Economic system</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11068-4">
    <skos:prefLabel xml:lang="de">Wirtschaftsordnung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18566-2">
    <skos:prefLabel xml:lang="en">Timetable</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18566-2">
    <skos:prefLabel xml:lang="de">Fahrplan</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15669-0">
    <skos:narrower rdf:resource="descriptor/19442-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15669-0">
    <skos:narrower rdf:resource="descriptor/19529-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14961-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12281-6">
    <skos:altLabel xml:lang="en">Mutual fund</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12281-6">
    <skos:altLabel xml:lang="en">Investment trust</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12281-6">
    <skos:altLabel xml:lang="en">Investment company</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12281-6">
    <skos:altLabel xml:lang="de">Kapitalanlagegesellschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12281-6">
    <skos:altLabel xml:lang="de">Investmentsparen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12281-6">
    <skos:altLabel xml:lang="de">Investmentgesellschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12281-6">
    <skos:altLabel xml:lang="de">Anlagefonds</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10040-3">
    <skos:broader rdf:resource="thsys/70057"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10040-3">
    <skos:broader rdf:resource="descriptor/10113-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71077">
    <skos:prefLabel xml:lang="de">V.09.07  Steuern und Steuerwissenschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11649-5">
    <skos:related rdf:resource="descriptor/19414-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11649-5">
    <skos:related rdf:resource="descriptor/11698-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11649-5">
    <skos:related rdf:resource="descriptor/10827-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#broadMatch">
    <skos:definition xml:lang="en">skos:broadMatch is used to state a hierarchical mapping link
      between two conceptual resources in different concept schemes.</skos:definition>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19043-1">
    <skos:related rdf:resource="descriptor/12282-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19043-1">
    <skos:related rdf:resource="descriptor/16050-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19043-1">
    <skos:related rdf:resource="descriptor/19079-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19043-1">
    <skos:related rdf:resource="descriptor/19080-2"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id375">
    <rdf:value xml:lang="de">Für die Struktur einer einzelnen Branche</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19813-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16430-2">
    <skos:related rdf:resource="descriptor/18130-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16430-2">
    <skos:related rdf:resource="descriptor/16544-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16430-2">
    <skos:related rdf:resource="descriptor/16642-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16430-2">
    <skos:related rdf:resource="descriptor/16432-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16430-2">
    <skos:related rdf:resource="descriptor/16207-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16430-2">
    <skos:related rdf:resource="descriptor/16275-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16430-2">
    <skos:related rdf:resource="descriptor/19975-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16430-2">
    <skos:related rdf:resource="descriptor/11164-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16430-2">
    <skos:related rdf:resource="descriptor/16193-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16430-2">
    <skos:related rdf:resource="descriptor/10968-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16430-2">
    <skos:related rdf:resource="descriptor/16405-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16430-2">
    <skos:related rdf:resource="descriptor/16633-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18564-6">
    <skos:broader rdf:resource="thsys/70944"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18564-6">
    <skos:broader rdf:resource="thsys/70924"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18564-6">
    <skos:broader rdf:resource="thsys/70873"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18564-6">
    <skos:broader rdf:resource="descriptor/13307-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13490-6">
    <skos:prefLabel xml:lang="en">Hypermarket</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13490-6">
    <skos:prefLabel xml:lang="de">SB-Center</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11165-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12844-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17630-3">
    <skos:broader rdf:resource="thsys/70253"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17630-3">
    <skos:broader rdf:resource="descriptor/17608-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17630-3">
    <skos:broader rdf:resource="descriptor/17978-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73372">
    <skos:broader rdf:resource="thsys/73374"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12524-6">
    <skos:related rdf:resource="descriptor/12514-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15679-4">
    <skos:narrower rdf:resource="descriptor/13572-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15679-4">
    <skos:narrower rdf:resource="descriptor/18030-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15679-4">
    <skos:narrower rdf:resource="descriptor/12610-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13389-2">
    <zbwext:useInsteadNote rdf:nodeID="id274"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11728-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12446-0">
    <skos:prefLabel xml:lang="en">Income shifting</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12446-0">
    <skos:prefLabel xml:lang="de">Gewinnverlagerung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17652-0">
    <skos:prefLabel xml:lang="en">Africa South of Sahara</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17652-0">
    <skos:prefLabel xml:lang="de">Afrika südlich der Sahara</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14216-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16716-5">
    <skos:broader rdf:resource="thsys/73393"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16716-5">
    <skos:broader rdf:resource="thsys/71035"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16716-5">
    <skos:broader rdf:resource="thsys/70269"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id111">
    <rdf:value xml:lang="de">für die Entwicklung ländlicher Gemeinden in Entwicklungsländern</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#broaderTransitive">
    <rdfs:subPropertyOf rdf:resource="http://www.w3.org/2004/02/skos/core#semanticRelation"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17500-2">
    <skos:broader rdf:resource="thsys/71032"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17500-2">
    <skos:broader rdf:resource="descriptor/17595-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17500-2">
    <skos:broader rdf:resource="descriptor/19078-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11537-2">
    <skos:broader rdf:resource="thsys/71064"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11537-2">
    <skos:broader rdf:resource="thsys/70144"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11537-2">
    <skos:broader rdf:resource="descriptor/11540-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13562-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12419-3">
    <skos:altLabel xml:lang="en">Gross rental method</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12419-3">
    <skos:altLabel xml:lang="de">Ertragswertverfahren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15916-2">
    <skos:broader rdf:resource="thsys/70142"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15916-2">
    <skos:broader rdf:resource="descriptor/16563-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19651-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19578-4">
    <skos:broader rdf:resource="thsys/72181"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19578-4">
    <skos:broader rdf:resource="thsys/70839"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19578-4">
    <skos:broader rdf:resource="thsys/70573"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19578-4">
    <skos:broader rdf:resource="descriptor/18887-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14327-5">
    <skos:broader rdf:resource="thsys/70074"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14327-5">
    <skos:broader rdf:resource="descriptor/14602-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18884-2">
    <skos:scopeNote xml:lang="de">Auch für Bildungskosten.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10843-0">
    <skos:broader rdf:resource="thsys/71091"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10843-0">
    <skos:broader rdf:resource="thsys/71005"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10843-0">
    <skos:broader rdf:resource="descriptor/10656-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19072-1">
    <skos:narrower rdf:resource="descriptor/15076-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19072-1">
    <skos:narrower rdf:resource="descriptor/18887-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19072-1">
    <skos:narrower rdf:resource="descriptor/11962-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19072-1">
    <skos:narrower rdf:resource="descriptor/11965-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19072-1">
    <skos:narrower rdf:resource="descriptor/19584-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19072-1">
    <skos:narrower rdf:resource="descriptor/11534-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19072-1">
    <skos:narrower rdf:resource="descriptor/12039-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73304">
    <skos:narrower rdf:resource="descriptor/14130-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73304">
    <skos:narrower rdf:resource="descriptor/14125-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73304">
    <skos:narrower rdf:resource="descriptor/14851-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73304">
    <skos:narrower rdf:resource="descriptor/14063-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73304">
    <skos:narrower rdf:resource="descriptor/19017-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73304">
    <skos:narrower rdf:resource="descriptor/14057-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73304">
    <skos:narrower rdf:resource="descriptor/14055-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11525-2">
    <skos:related rdf:resource="descriptor/16081-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11525-2">
    <skos:related rdf:resource="descriptor/11526-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11525-2">
    <skos:related rdf:resource="descriptor/11536-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11525-2">
    <skos:related rdf:resource="descriptor/11535-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11525-2">
    <skos:related rdf:resource="descriptor/11728-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11525-2">
    <skos:related rdf:resource="descriptor/11532-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24733-2">
    <skos:broader rdf:resource="thsys/71107"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24733-2">
    <skos:broader rdf:resource="thsys/70893"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24733-2">
    <skos:broader rdf:resource="thsys/70183"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19121-0">
    <skos:related rdf:resource="descriptor/10202-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19121-0">
    <skos:related rdf:resource="descriptor/10928-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19121-0">
    <skos:related rdf:resource="descriptor/12766-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19813-6">
    <skos:broader rdf:resource="thsys/73334"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11905-6">
    <skos:altLabel xml:lang="en">Location of the firm</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11905-6">
    <skos:altLabel xml:lang="de">Unternehmensstandort</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11905-6">
    <skos:altLabel xml:lang="de">Standort des Betriebs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11905-6">
    <skos:altLabel xml:lang="de">Innerbetrieblicher Standort</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11905-6">
    <skos:altLabel xml:lang="de">Firmenstandort</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11905-6">
    <skos:altLabel xml:lang="de">Betriebsstandort</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70306">
    <rdfs:label xml:lang="en">Monetary theory</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70306">
    <rdfs:label xml:lang="de">Geldtheorie</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19533-5">
    <skos:altLabel xml:lang="en">Market incentive</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19533-5">
    <skos:altLabel xml:lang="en">Economic disincentive</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19533-5">
    <skos:altLabel xml:lang="de">Wirtschaftlicher Anreiz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19533-5">
    <skos:altLabel xml:lang="de">Ökonomisches Anreizsystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19533-5">
    <skos:altLabel xml:lang="de">Ökonomisches Anreizinstrument</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19533-5">
    <skos:altLabel xml:lang="de">Ökonomischer Hebel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19533-5">
    <skos:altLabel xml:lang="de">Ökonomische Stimulierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19533-5">
    <skos:altLabel xml:lang="de">Anreiz zum ökonomischen Handeln</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10629-2">
    <skos:altLabel xml:lang="de">Reine Außenwirtschaftstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10629-2">
    <skos:altLabel xml:lang="de">Reale Außenwirtschaftstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10629-2">
    <skos:altLabel xml:lang="de">Außenhandelstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18444-2">
    <skos:narrower rdf:resource="descriptor/18314-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18444-2">
    <skos:narrower rdf:resource="descriptor/19525-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18444-2">
    <skos:narrower rdf:resource="descriptor/19792-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18444-2">
    <skos:narrower rdf:resource="descriptor/16668-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12712-5">
    <skos:broader rdf:resource="thsys/70030"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11869-5">
    <skos:prefLabel xml:lang="en">Spatial organization</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11869-5">
    <skos:prefLabel xml:lang="de">Raumordnung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18771-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72181">
    <skos:prefLabel xml:lang="de">V.16.01  Statistische Übersichten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71005">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.06</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24855-2">
    <skos:scopeNote xml:lang="de">Überschießen des nominellen Wechselkurses.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id4">
    <rdfs:seeAlso rdf:resource="descriptor/16354-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73104">
    <rdfs:label xml:lang="de">Spiel- und Verhandlungstheorie</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11692-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19365-0">
    <skos:altLabel xml:lang="de">Kautschukanbau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11956-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15243-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16535-2">
    <skos:altLabel xml:lang="de">Jugendhilferecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16535-2">
    <skos:altLabel xml:lang="de">Erziehungshilfe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16535-2">
    <skos:altLabel xml:lang="de">Erzieherische Hilfe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19043-1">
    <skos:narrower rdf:resource="descriptor/12867-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19043-1">
    <skos:narrower rdf:resource="descriptor/13714-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18500-4">
    <skos:altLabel xml:lang="de">Pipelinebau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18500-4">
    <skos:altLabel xml:lang="de">Leitungsbau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18194-6">
    <skos:prefLabel xml:lang="en">International cocoa agreement</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18194-6">
    <skos:prefLabel xml:lang="de">Kakaoabkommen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13816-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16909-1">
    <skos:prefLabel xml:lang="en">Rhineland-Palatinate</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16909-1">
    <skos:prefLabel xml:lang="de">Rheinland-Pfalz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18247-4">
    <skos:related rdf:resource="descriptor/13530-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12783-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10011-3">
    <skos:narrower rdf:resource="descriptor/13841-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10011-3">
    <skos:narrower rdf:resource="descriptor/13590-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10011-3">
    <skos:narrower rdf:resource="descriptor/12510-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10011-3">
    <skos:narrower rdf:resource="descriptor/12509-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10011-3">
    <skos:narrower rdf:resource="descriptor/15773-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10011-3">
    <skos:narrower rdf:resource="descriptor/11934-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10011-3">
    <skos:narrower rdf:resource="descriptor/10060-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10011-3">
    <skos:narrower rdf:resource="descriptor/19432-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10011-3">
    <skos:narrower rdf:resource="descriptor/12508-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10011-3">
    <skos:narrower rdf:resource="descriptor/10058-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10011-3">
    <skos:narrower rdf:resource="descriptor/12540-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10011-3">
    <skos:narrower rdf:resource="descriptor/19101-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10011-3">
    <skos:narrower rdf:resource="descriptor/12504-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10011-3">
    <skos:narrower rdf:resource="descriptor/12501-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10011-3">
    <skos:narrower rdf:resource="descriptor/10052-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10011-3">
    <skos:narrower rdf:resource="descriptor/15069-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10011-3">
    <skos:narrower rdf:resource="descriptor/12249-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10011-3">
    <skos:narrower rdf:resource="descriptor/12495-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10011-3">
    <skos:narrower rdf:resource="descriptor/10140-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10011-3">
    <skos:narrower rdf:resource="descriptor/12424-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10011-3">
    <skos:narrower rdf:resource="descriptor/20881-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10011-3">
    <skos:narrower rdf:resource="descriptor/11555-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10011-3">
    <skos:narrower rdf:resource="descriptor/19849-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10011-3">
    <skos:narrower rdf:resource="descriptor/12492-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10011-3">
    <skos:narrower rdf:resource="descriptor/18785-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10011-3">
    <skos:narrower rdf:resource="descriptor/11551-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10011-3">
    <skos:narrower rdf:resource="descriptor/12491-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10011-3">
    <skos:narrower rdf:resource="descriptor/12490-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10011-3">
    <skos:narrower rdf:resource="descriptor/13851-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10011-3">
    <skos:narrower rdf:resource="descriptor/19566-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10011-3">
    <skos:narrower rdf:resource="descriptor/12407-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10011-3">
    <skos:narrower rdf:resource="descriptor/19254-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73301">
    <skos:narrower rdf:resource="descriptor/14161-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73301">
    <skos:narrower rdf:resource="descriptor/14969-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73301">
    <skos:narrower rdf:resource="descriptor/14967-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73301">
    <skos:narrower rdf:resource="descriptor/14158-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13574-0">
    <skos:narrower rdf:resource="descriptor/18568-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10684-1">
    <skos:altLabel xml:lang="en">Trade facilitation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10684-1">
    <skos:altLabel xml:lang="de">Handelserleichterungen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10684-1">
    <skos:altLabel xml:lang="de">Handelserleichterung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10684-1">
    <skos:altLabel xml:lang="de">Exportförderung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10684-1">
    <skos:altLabel xml:lang="de">Ausfuhrförderung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10684-1">
    <skos:altLabel xml:lang="de">Außenwirtschaftsförderung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17628-4">
    <skos:prefLabel xml:lang="en">Kuwait</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17628-4">
    <skos:prefLabel xml:lang="de">Kuwait</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20314-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18817-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14328-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id286">
    <rdfs:seeAlso rdf:resource="descriptor/10221-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19260-0">
    <skos:prefLabel xml:lang="en">Currency board</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19260-0">
    <skos:prefLabel xml:lang="de">Currency Board</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70928">
    <skos:prefLabel xml:lang="en">W.12.05  Transport economics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70928">
    <skos:prefLabel xml:lang="de">W.12.05  Verkehrsökonomik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19729-2">
    <skos:broader rdf:resource="thsys/73392"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19729-2">
    <skos:broader rdf:resource="descriptor/18067-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14174-4">
    <skos:broader rdf:resource="thsys/73350"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14174-4">
    <skos:broader rdf:resource="thsys/70538"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14174-4">
    <skos:broader rdf:resource="descriptor/14176-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11223-4">
    <skos:prefLabel xml:lang="en">Signalling</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11223-4">
    <skos:prefLabel xml:lang="de">Signalling</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10962-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18865-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12340-2">
    <skos:related rdf:resource="descriptor/12341-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12340-2">
    <skos:related rdf:resource="descriptor/12337-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17199-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18311-0">
    <skos:broader rdf:resource="thsys/73390"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18311-0">
    <skos:broader rdf:resource="thsys/71019"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18311-0">
    <skos:broader rdf:resource="descriptor/16081-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18311-0">
    <skos:broader rdf:resource="descriptor/16007-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14905-5">
    <skos:prefLabel xml:lang="en">Gloves</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14905-5">
    <skos:prefLabel xml:lang="de">Handschuhe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70171">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15916-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11322-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12012-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15764-6">
    <skos:altLabel xml:lang="en">Environmental ethics</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15764-6">
    <skos:altLabel xml:lang="en">Environmental concern</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15764-6">
    <skos:altLabel xml:lang="en">Environmental behaviour</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15764-6">
    <skos:altLabel xml:lang="de">Umweltverhalten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15764-6">
    <skos:altLabel xml:lang="de">Umweltethik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15764-6">
    <skos:altLabel xml:lang="de">Umweltbewußtsein</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12978-2">
    <skos:prefLabel xml:lang="en">Gas industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12978-2">
    <skos:prefLabel xml:lang="de">Gaswirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73340">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.10.07.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13377-2">
    <skos:broader rdf:resource="thsys/70876"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18929-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13720-1">
    <skos:altLabel xml:lang="en">Futures exchange market</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13720-1">
    <skos:altLabel xml:lang="en">Forward foreign exchange market</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13720-1">
    <skos:altLabel xml:lang="en">Forward exchange market</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13720-1">
    <skos:altLabel xml:lang="en">Foreign currency futures</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13720-1">
    <skos:altLabel xml:lang="en">Currency futures</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13720-1">
    <skos:altLabel xml:lang="de">Währungs-Futures</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13720-1">
    <skos:altLabel xml:lang="de">Termindevisen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13720-1">
    <skos:altLabel xml:lang="de">Devisenterminmarkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18788-5">
    <skos:related rdf:resource="descriptor/12845-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18788-5">
    <skos:related rdf:resource="descriptor/13971-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18788-5">
    <skos:related rdf:resource="descriptor/13935-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18788-5">
    <skos:related rdf:resource="descriptor/12805-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18788-5">
    <skos:related rdf:resource="descriptor/18295-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70493">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19510-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12787-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13660-0">
    <skos:related rdf:resource="descriptor/16037-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13660-0">
    <skos:related rdf:resource="descriptor/13662-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13660-0">
    <skos:related rdf:resource="descriptor/13347-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13660-0">
    <skos:related rdf:resource="descriptor/16516-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13660-0">
    <skos:related rdf:resource="descriptor/13658-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13660-0">
    <skos:related rdf:resource="descriptor/15961-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13660-0">
    <skos:related rdf:resource="descriptor/15604-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10038-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11239-3">
    <zbwext:useInsteadNote rdf:nodeID="id49"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70924">
    <rdfs:label xml:lang="en">Tourism</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70924">
    <rdfs:label xml:lang="de">Tourismus</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15947-5">
    <skos:related rdf:resource="descriptor/15949-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15947-5">
    <skos:related rdf:resource="descriptor/16670-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15947-5">
    <skos:related rdf:resource="descriptor/16683-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16112-2">
    <skos:related rdf:resource="descriptor/16202-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16112-2">
    <skos:related rdf:resource="descriptor/16201-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16112-2">
    <skos:related rdf:resource="descriptor/11709-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16112-2">
    <skos:related rdf:resource="descriptor/11640-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16112-2">
    <skos:related rdf:resource="descriptor/11638-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16112-2">
    <skos:related rdf:resource="descriptor/11691-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16112-2">
    <skos:related rdf:resource="descriptor/16110-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16112-2">
    <skos:related rdf:resource="descriptor/11532-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16112-2">
    <skos:related rdf:resource="descriptor/18736-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11990-3">
    <skos:related rdf:resource="descriptor/10512-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11990-3">
    <skos:related rdf:resource="descriptor/11976-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11990-3">
    <skos:related rdf:resource="descriptor/11932-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11990-3">
    <skos:related rdf:resource="descriptor/18837-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11990-3">
    <skos:related rdf:resource="descriptor/19456-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11990-3">
    <skos:related rdf:resource="descriptor/11986-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11990-3">
    <skos:related rdf:resource="descriptor/11970-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11990-3">
    <skos:related rdf:resource="descriptor/11928-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11990-3">
    <skos:related rdf:resource="descriptor/19377-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11277-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16797-0">
    <skos:scopeNote xml:lang="de">Nur für Literatur über Nachschlagewerke.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13652-6">
    <skos:altLabel xml:lang="en">Mailbox system</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13652-6">
    <skos:altLabel xml:lang="en">Computer mail</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13652-6">
    <skos:altLabel xml:lang="de">E-Mail</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id193">
    <rdf:value xml:lang="de">Für Glühbirnen etc.</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13394-2">
    <skos:altLabel xml:lang="de">Städteverband</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13394-2">
    <skos:altLabel xml:lang="de">Kreisverband</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13394-2">
    <skos:altLabel xml:lang="de">Kommunaler Regionalverband</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13394-2">
    <skos:altLabel xml:lang="de">Gemeindeverband</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11038-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14237-6">
    <skos:prefLabel xml:lang="en">Natural stone</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14237-6">
    <skos:prefLabel xml:lang="de">Naturstein</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70130">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10690-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71037">
    <skos:narrower rdf:resource="descriptor/19144-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71037">
    <skos:narrower rdf:resource="descriptor/18664-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71037">
    <skos:narrower rdf:resource="descriptor/13666-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71037">
    <skos:narrower rdf:resource="descriptor/13278-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71037">
    <skos:narrower rdf:resource="descriptor/13387-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71037">
    <skos:narrower rdf:resource="descriptor/18766-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71037">
    <skos:narrower rdf:resource="descriptor/15619-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71037">
    <skos:narrower rdf:resource="descriptor/18662-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71037">
    <skos:narrower rdf:resource="descriptor/13269-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71037">
    <skos:narrower rdf:resource="descriptor/19145-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71037">
    <skos:narrower rdf:resource="descriptor/18175-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71037">
    <skos:narrower rdf:resource="descriptor/18017-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71037">
    <skos:narrower rdf:resource="descriptor/19651-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71037">
    <skos:narrower rdf:resource="descriptor/18003-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71037">
    <skos:narrower rdf:resource="descriptor/18660-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71037">
    <skos:narrower rdf:resource="descriptor/18767-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71037">
    <skos:narrower rdf:resource="descriptor/15605-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71037">
    <skos:narrower rdf:resource="descriptor/15604-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71037">
    <skos:narrower rdf:resource="descriptor/18172-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71037">
    <skos:narrower rdf:resource="descriptor/15600-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71037">
    <skos:narrower rdf:resource="descriptor/16774-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71037">
    <skos:narrower rdf:resource="descriptor/20341-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71037">
    <skos:narrower rdf:resource="descriptor/15599-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71037">
    <skos:narrower rdf:resource="descriptor/18139-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71037">
    <skos:narrower rdf:resource="descriptor/18102-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16265-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15773-5">
    <skos:scopeNote xml:lang="de">Auch Kosten zur Vermeidung von Umweltschäden.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16187-6">
    <skos:narrower rdf:resource="descriptor/16070-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14804-4">
    <skos:prefLabel xml:lang="en">Paper packaging</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14804-4">
    <skos:prefLabel xml:lang="de">Papierverpackung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18609-3">
    <skos:altLabel xml:lang="de">Internationaler Personenverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18400-1">
    <skos:narrower rdf:resource="descriptor/18422-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18400-1">
    <skos:narrower rdf:resource="descriptor/13881-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19318-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13308-0">
    <skos:altLabel xml:lang="de">Pension (Fremdenverkehr)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13308-0">
    <skos:altLabel xml:lang="de">Hotellerie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13308-0">
    <skos:altLabel xml:lang="de">Hotel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13308-0">
    <skos:altLabel xml:lang="de">Erholungsheim</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13308-0">
    <skos:altLabel xml:lang="de">Beherbergungsgewerbe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13763-4">
    <skos:related rdf:resource="descriptor/11490-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13763-4">
    <skos:related rdf:resource="descriptor/19665-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13763-4">
    <skos:related rdf:resource="descriptor/11421-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13763-4">
    <skos:related rdf:resource="descriptor/13746-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18198-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18571-2">
    <skos:broader rdf:resource="thsys/70941"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18571-2">
    <skos:broader rdf:resource="descriptor/18624-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18581-6">
    <skos:broader rdf:resource="thsys/70938"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18581-6">
    <skos:broader rdf:resource="descriptor/13572-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11706-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15925-1">
    <skos:altLabel xml:lang="de">Kleinkinder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15925-1">
    <skos:altLabel xml:lang="de">Kleinkind</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15925-1">
    <skos:altLabel xml:lang="de">Kind</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15925-1">
    <skos:altLabel xml:lang="de">Babys</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18300-5">
    <skos:broader rdf:resource="thsys/70201"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16033-5">
    <skos:prefLabel xml:lang="en">Lease</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16033-5">
    <skos:prefLabel xml:lang="de">Pacht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18279-5">
    <skos:prefLabel xml:lang="en">Composites</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18279-5">
    <skos:prefLabel xml:lang="de">Verbundwerkstoff</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13289-6">
    <skos:broader rdf:resource="thsys/70903"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13289-6">
    <skos:broader rdf:resource="descriptor/18705-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11326-1">
    <skos:broader rdf:resource="thsys/71115"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11326-1">
    <skos:broader rdf:resource="thsys/70278"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13723-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18078-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16212-5">
    <skos:prefLabel xml:lang="en">Administrative sciences</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16212-5">
    <skos:prefLabel xml:lang="de">Verwaltungswissenschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13816-2">
    <skos:broader rdf:resource="thsys/70891"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13816-2">
    <skos:broader rdf:resource="descriptor/13297-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18496-4">
    <skos:related rdf:resource="descriptor/13330-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18496-4">
    <skos:related rdf:resource="descriptor/18746-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18496-4">
    <skos:related rdf:resource="descriptor/13844-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18496-4">
    <skos:related rdf:resource="descriptor/13846-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10624-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19321-6">
    <skos:related rdf:resource="descriptor/11664-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19321-6">
    <skos:related rdf:resource="descriptor/10725-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19321-6">
    <skos:related rdf:resource="descriptor/13731-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19321-6">
    <skos:related rdf:resource="descriptor/10807-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19321-6">
    <skos:related rdf:resource="descriptor/10811-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19321-6">
    <skos:related rdf:resource="descriptor/13723-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19321-6">
    <skos:related rdf:resource="descriptor/19270-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19321-6">
    <skos:related rdf:resource="descriptor/19317-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11779-6">
    <skos:related rdf:resource="descriptor/10110-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11779-6">
    <skos:related rdf:resource="descriptor/11751-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11779-6">
    <skos:related rdf:resource="descriptor/16545-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11779-6">
    <skos:related rdf:resource="descriptor/11568-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11779-6">
    <skos:related rdf:resource="descriptor/11771-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11779-6">
    <skos:related rdf:resource="descriptor/11735-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14743-5">
    <skos:altLabel xml:lang="de">Pulverlack</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14743-5">
    <skos:altLabel xml:lang="de">Lack</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14743-5">
    <skos:altLabel xml:lang="de">Kunstharzlack</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14743-5">
    <skos:altLabel xml:lang="de">Farben</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14743-5">
    <skos:altLabel xml:lang="de">Dispersionsfarbe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14743-5">
    <skos:altLabel xml:lang="de">Bautenanstrichstoff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14743-5">
    <skos:altLabel xml:lang="de">Anstrichstoff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13102-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14926-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19498-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73343">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19333-6">
    <skos:altLabel xml:lang="de">Tobin-Steuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19333-6">
    <skos:altLabel xml:lang="de">Devisensteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71096">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19706-0">
    <skos:prefLabel xml:lang="en">Church and state</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19706-0">
    <skos:prefLabel xml:lang="de">Kirchenpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71020">
    <rdfs:label xml:lang="en">Agricultural Policy</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71020">
    <rdfs:label xml:lang="de">Agrarpolitik</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16761-0">
    <skos:narrower rdf:resource="descriptor/18930-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15668-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10905-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13563-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id7">
    <rdfs:seeAlso rdf:resource="descriptor/11806-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19165-1">
    <skos:prefLabel xml:lang="en">Greater China</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19165-1">
    <skos:prefLabel xml:lang="de">Greater China</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19614-5">
    <skos:broader rdf:resource="thsys/73354"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18746-0">
    <skos:prefLabel xml:lang="en">Facility management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18746-0">
    <skos:prefLabel xml:lang="de">Gebäudemanagement</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70035">
    <skos:broader rdf:resource="thsys/70174"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14380-1">
    <skos:narrower rdf:resource="descriptor/14328-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71031">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14509-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11612-0">
    <skos:prefLabel xml:lang="en">State participation in private enterprises</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11612-0">
    <skos:prefLabel xml:lang="de">Öffentliche Kapitalbeteiligung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11809-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13391-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id78">
    <rdfs:seeAlso rdf:resource="descriptor/18702-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73365">
    <skos:narrower rdf:resource="descriptor/16463-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73365">
    <skos:narrower rdf:resource="descriptor/18182-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73365">
    <skos:narrower rdf:resource="descriptor/18818-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73365">
    <skos:narrower rdf:resource="descriptor/16521-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73365">
    <skos:narrower rdf:resource="descriptor/19720-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73365">
    <skos:narrower rdf:resource="descriptor/16454-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73365">
    <skos:narrower rdf:resource="descriptor/16497-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73365">
    <skos:narrower rdf:resource="descriptor/16437-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73365">
    <skos:narrower rdf:resource="descriptor/16449-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73365">
    <skos:narrower rdf:resource="descriptor/16445-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17915-1">
    <skos:prefLabel xml:lang="en">Brazil</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17915-1">
    <skos:prefLabel xml:lang="de">Brasilien</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18612-0">
    <skos:broader rdf:resource="thsys/70939"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18612-0">
    <skos:broader rdf:resource="thsys/70924"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18612-0">
    <skos:broader rdf:resource="descriptor/13644-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18612-0">
    <skos:broader rdf:resource="descriptor/13629-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11358-2">
    <skos:related rdf:resource="descriptor/13962-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11358-2">
    <skos:related rdf:resource="descriptor/12081-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11358-2">
    <skos:related rdf:resource="descriptor/11365-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13056-1">
    <skos:altLabel xml:lang="en">Aerospace industry</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13056-1">
    <skos:altLabel xml:lang="de">Raumfahrtindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70967">
    <skos:broader rdf:resource="thsys/70970"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19672-5">
    <skos:related rdf:resource="descriptor/19253-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19672-5">
    <skos:related rdf:resource="descriptor/19565-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19672-5">
    <skos:related rdf:resource="descriptor/12142-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19672-5">
    <skos:related rdf:resource="descriptor/12314-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14033-1">
    <skos:altLabel xml:lang="de">Heimwerkerbedarf</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14033-1">
    <skos:altLabel xml:lang="de">Do-It-Yourself-Artikel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14033-1">
    <skos:altLabel xml:lang="de">DIY-Produkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14033-1">
    <skos:altLabel xml:lang="de">Bastelartikel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19377-0">
    <skos:altLabel xml:lang="de">Lokale öffentliche Güter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10847-6">
    <skos:narrower rdf:resource="descriptor/19308-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19385-1">
    <skos:altLabel xml:lang="de">Gasmarkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19711-0">
    <skos:broader rdf:resource="thsys/73366"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19711-0">
    <skos:broader rdf:resource="descriptor/19151-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19711-0">
    <skos:broader rdf:resource="descriptor/16441-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70838">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70900">
    <skos:broader rdf:resource="thsys/70905"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19707-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11266-0">
    <skos:narrower rdf:resource="descriptor/11278-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21283-1">
    <skos:prefLabel xml:lang="en">Mobile commerce</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21283-1">
    <skos:prefLabel xml:lang="de">Mobile Commerce</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14507-3">
    <skos:prefLabel xml:lang="en">Dishwasher</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14507-3">
    <skos:prefLabel xml:lang="de">Geschirrspülmaschine</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18186-5">
    <skos:prefLabel xml:lang="en">Environmental liability</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18186-5">
    <skos:prefLabel xml:lang="de">Umwelthaftung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17992-4">
    <skos:prefLabel xml:lang="en">OAU countries</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17992-4">
    <skos:prefLabel xml:lang="de">OAU-Staaten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16188-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12016-4">
    <skos:scopeNote xml:lang="de">Belastungen des Bodens durch chemische Verunreinigungen, Versiegelung des Bodens etc.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15488-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12942-2">
    <skos:broader rdf:resource="thsys/71026"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12942-2">
    <skos:broader rdf:resource="descriptor/12940-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11307-5">
    <skos:altLabel xml:lang="en">Labor court</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11307-5">
    <skos:altLabel xml:lang="de">Arbeitsgerichtsgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11307-5">
    <skos:altLabel xml:lang="de">Arbeitsgerichtsbarkeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18607-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16485-5">
    <skos:altLabel xml:lang="en">Consumerism</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16485-5">
    <skos:altLabel xml:lang="en">Consumer law</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16485-5">
    <skos:altLabel xml:lang="de">Verbraucherrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16485-5">
    <skos:altLabel xml:lang="de">Konsumerismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12416-2">
    <skos:altLabel xml:lang="en">Rateable value</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70893">
    <rdfs:label xml:lang="de">Börsen und Wertpapierhandel</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10835-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13193-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16267-1">
    <skos:related rdf:resource="descriptor/16389-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16267-1">
    <skos:related rdf:resource="descriptor/16422-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16267-1">
    <skos:related rdf:resource="descriptor/16470-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73390">
    <skos:prefLabel xml:lang="en">N.05.03  Economic law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73390">
    <skos:prefLabel xml:lang="de">N.05.03  Wirtschaftsrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12592-3">
    <skos:narrower rdf:resource="descriptor/18751-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12592-3">
    <skos:narrower rdf:resource="descriptor/12575-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12592-3">
    <skos:narrower rdf:resource="descriptor/12615-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12592-3">
    <skos:narrower rdf:resource="descriptor/12591-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70037">
    <skos:prefLabel xml:lang="de">B.02  Investition und Finanzierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11037-1">
    <skos:scopeNote xml:lang="de">Nur als Gegensatz zu Leistungs- und erwerbswirtschaftlichem Prinzip.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18815-0">
    <skos:broader rdf:resource="thsys/73363"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18815-0">
    <skos:broader rdf:resource="thsys/70865"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18815-0">
    <skos:broader rdf:resource="descriptor/19725-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18815-0">
    <skos:broader rdf:resource="descriptor/12041-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15916-2">
    <skos:prefLabel xml:lang="en">Demography</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15916-2">
    <skos:prefLabel xml:lang="de">Demographie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19777-5">
    <skos:altLabel xml:lang="de">Semiologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13801-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70183">
    <skos:prefLabel xml:lang="de">B.02.01  Kapitalbeschaffung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70017">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14397-5">
    <skos:altLabel xml:lang="de">Turmdrehkran</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14397-5">
    <skos:altLabel xml:lang="de">Teleskopkran</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14397-5">
    <skos:altLabel xml:lang="de">Portalkran</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14397-5">
    <skos:altLabel xml:lang="de">Drehkran</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14397-5">
    <skos:altLabel xml:lang="de">Baukran</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12029-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11349-3">
    <skos:related rdf:resource="descriptor/16484-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11349-3">
    <skos:related rdf:resource="descriptor/11350-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11349-3">
    <skos:related rdf:resource="descriptor/19503-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13229-3">
    <skos:broader rdf:resource="thsys/70959"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13229-3">
    <skos:broader rdf:resource="descriptor/18522-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13229-3">
    <skos:broader rdf:resource="descriptor/13200-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18628-6">
    <skos:scopeNote xml:lang="de">Entwurf umweltverträglicher Verkehrssysteme.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11481-3">
    <skos:related rdf:resource="descriptor/18704-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11481-3">
    <skos:related rdf:resource="descriptor/11417-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11481-3">
    <skos:related rdf:resource="descriptor/10764-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11481-3">
    <skos:related rdf:resource="descriptor/10746-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11481-3">
    <skos:related rdf:resource="descriptor/11042-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11481-3">
    <skos:related rdf:resource="descriptor/10382-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11481-3">
    <skos:related rdf:resource="descriptor/11416-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70572">
    <skos:narrower rdf:resource="descriptor/19580-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70572">
    <skos:narrower rdf:resource="descriptor/15958-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70572">
    <skos:narrower rdf:resource="descriptor/11996-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70572">
    <skos:narrower rdf:resource="descriptor/11902-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70572">
    <skos:narrower rdf:resource="descriptor/15956-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70572">
    <skos:narrower rdf:resource="descriptor/19492-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70572">
    <skos:narrower rdf:resource="descriptor/15930-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70572">
    <skos:narrower rdf:resource="descriptor/15709-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70572">
    <skos:narrower rdf:resource="descriptor/19650-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70572">
    <skos:narrower rdf:resource="descriptor/16486-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70572">
    <skos:narrower rdf:resource="descriptor/19639-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70572">
    <skos:narrower rdf:resource="descriptor/10504-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70572">
    <skos:narrower rdf:resource="descriptor/15954-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70572">
    <skos:narrower rdf:resource="descriptor/10613-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70572">
    <skos:narrower rdf:resource="descriptor/19640-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70572">
    <skos:narrower rdf:resource="descriptor/15919-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70572">
    <skos:narrower rdf:resource="descriptor/19649-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70572">
    <skos:narrower rdf:resource="descriptor/15952-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70572">
    <skos:narrower rdf:resource="descriptor/10491-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70572">
    <skos:narrower rdf:resource="descriptor/11888-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70572">
    <skos:narrower rdf:resource="descriptor/15951-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70572">
    <skos:narrower rdf:resource="descriptor/19648-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70572">
    <skos:narrower rdf:resource="descriptor/19647-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16727-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19003-6">
    <skos:broader rdf:resource="thsys/73336"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19003-6">
    <skos:broader rdf:resource="thsys/70244"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19697-3">
    <skos:broader rdf:resource="thsys/73376"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19697-3">
    <skos:broader rdf:resource="descriptor/16354-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14948-1">
    <skos:prefLabel xml:lang="en">Processed food and vegetables</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14948-1">
    <skos:prefLabel xml:lang="de">Verarbeitetes Obst und Gemüse</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18774-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18332-6">
    <skos:prefLabel xml:lang="en">Common agricultural policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18332-6">
    <skos:prefLabel xml:lang="de">EU-Agrarpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11248-2">
    <skos:broader rdf:resource="thsys/71059"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11248-2">
    <skos:broader rdf:resource="thsys/71050"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16027-0">
    <skos:broader rdf:resource="thsys/73392"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16027-0">
    <skos:broader rdf:resource="descriptor/16030-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70178">
    <skos:prefLabel xml:lang="de">V.02.03  Preis- und Werttheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12446-0">
    <skos:broader rdf:resource="thsys/71040"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12446-0">
    <skos:broader rdf:resource="descriptor/12876-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12320-1">
    <skos:broader rdf:resource="thsys/71042"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12250-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19240-6">
    <skos:prefLabel xml:lang="en">Regional business cycle</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19240-6">
    <skos:prefLabel xml:lang="de">Regionaler Konjunkturzyklus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11591-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10678-3">
    <skos:prefLabel xml:lang="en">Exchange rate theory</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10678-3">
    <skos:prefLabel xml:lang="de">Wechselkurstheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20136-5">
    <skos:related rdf:resource="descriptor/12931-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20136-5">
    <skos:related rdf:resource="descriptor/13270-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11607-0">
    <skos:scopeNote xml:lang="de">Staatliches Schuldenmanagement: Staatsschuldformen, Laufzeiten, Zinsstruktur, usw.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19003-6">
    <skos:altLabel xml:lang="en">Severance payments</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19003-6">
    <skos:altLabel xml:lang="en">Severance pay</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19003-6">
    <skos:altLabel xml:lang="en">Redundancy payment</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13392-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11264-4">
    <skos:altLabel xml:lang="en">Labor market statistics</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11264-4">
    <skos:altLabel xml:lang="de">Erwerbstätigenstatistik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11264-4">
    <skos:altLabel xml:lang="de">Erwerbsstatistik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11264-4">
    <skos:altLabel xml:lang="de">Beschäftigungsstatistik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11264-4">
    <skos:altLabel xml:lang="de">Beschäftigtenstatistik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11264-4">
    <skos:altLabel xml:lang="de">Arbeitsstatistik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11264-4">
    <skos:altLabel xml:lang="de">Arbeitslosenstatistik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18665-0">
    <skos:altLabel xml:lang="en">Wireless access</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18665-0">
    <skos:altLabel xml:lang="de">Drahtlose Computerkommunikation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14045-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70011">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19843-4">
    <skos:broader rdf:resource="thsys/73361"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19843-4">
    <skos:broader rdf:resource="thsys/73332"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19843-4">
    <skos:broader rdf:resource="descriptor/15964-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19843-4">
    <skos:broader rdf:resource="descriptor/15998-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10696-1">
    <skos:broader rdf:resource="thsys/71098"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10696-1">
    <skos:broader rdf:resource="descriptor/10682-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10631-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19831-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id361">
    <rdfs:seeAlso rdf:resource="descriptor/18418-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70049">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#changeNote">
    <rdfs:subPropertyOf rdf:resource="http://www.w3.org/2004/02/skos/core#note"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15350-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id25">
    <rdfs:seeAlso rdf:resource="descriptor/16124-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10439-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11406-3">
    <skos:related rdf:resource="descriptor/16159-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11406-3">
    <skos:related rdf:resource="descriptor/16477-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11406-3">
    <skos:related rdf:resource="descriptor/16134-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11406-3">
    <skos:related rdf:resource="descriptor/11392-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11406-3">
    <skos:related rdf:resource="descriptor/11361-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10897-5">
    <skos:prefLabel xml:lang="en">Merger</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10897-5">
    <skos:prefLabel xml:lang="de">Fusion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16524-0">
    <skos:prefLabel xml:lang="en">Disaster relief</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16524-0">
    <skos:prefLabel xml:lang="de">Katastrophenhilfe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16027-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11401-6">
    <skos:narrower rdf:resource="descriptor/11384-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16308-6">
    <skos:prefLabel xml:lang="en">Self-government</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16308-6">
    <skos:prefLabel xml:lang="de">Selbstverwaltung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20110-2">
    <skos:altLabel xml:lang="en">Trade-related intellectual property</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20110-2">
    <skos:altLabel xml:lang="en">Agreement on trade-related aspects of intellectual property rights</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20110-2">
    <skos:altLabel xml:lang="de">Welthandelsregeln zum Schutz geistigen Eigentums</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20110-2">
    <skos:altLabel xml:lang="de">TRIPS-Abkommen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10852-6">
    <skos:prefLabel xml:lang="en">Foreign trade industries</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10852-6">
    <skos:prefLabel xml:lang="de">Außenhandelswirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18170-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17716-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70341">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">P.01.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10970-0">
    <skos:altLabel xml:lang="de">Staatsinterventionismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15600-1">
    <skos:broader rdf:resource="thsys/73342"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15600-1">
    <skos:broader rdf:resource="thsys/71037"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11273-3">
    <skos:prefLabel xml:lang="en">Women's labour force participation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11273-3">
    <skos:prefLabel xml:lang="de">Frauenerwerbstätigkeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16051-3">
    <skos:broader rdf:resource="thsys/73391"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16051-3">
    <skos:broader rdf:resource="thsys/71071"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16051-3">
    <skos:broader rdf:resource="descriptor/10964-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18971-0">
    <skos:altLabel xml:lang="en">Singles</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18971-0">
    <skos:altLabel xml:lang="en">Single persons</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18971-0">
    <skos:altLabel xml:lang="de">Ein-Personen-Haushalte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18971-0">
    <skos:altLabel xml:lang="de">Alleinstehende</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18013-1">
    <skos:broader rdf:resource="thsys/70958"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19508-4">
    <skos:broader rdf:resource="thsys/70833"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19508-4">
    <skos:broader rdf:resource="thsys/70324"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19508-4">
    <skos:broader rdf:resource="descriptor/11569-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19516-5">
    <skos:broader rdf:resource="thsys/70847"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19516-5">
    <skos:broader rdf:resource="thsys/70176"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19516-5">
    <skos:broader rdf:resource="descriptor/11407-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10007-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16638-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15755-0">
    <skos:prefLabel xml:lang="en">Acid rain</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15755-0">
    <skos:prefLabel xml:lang="de">Saurer Regen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18957-1">
    <skos:altLabel xml:lang="de">Müllgebühr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18957-1">
    <skos:altLabel xml:lang="de">Kehrichtsackgebühr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18957-1">
    <skos:altLabel xml:lang="de">Entsorgungsgebühr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18957-1">
    <skos:altLabel xml:lang="de">Abfallgebühr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19017-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15729-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19646-6">
    <skos:altLabel xml:lang="de">Suizid</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19646-6">
    <skos:altLabel xml:lang="de">Selbsttötung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19646-6">
    <skos:altLabel xml:lang="de">Freitod</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10112-4">
    <skos:prefLabel xml:lang="en">Welfare effect</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10112-4">
    <skos:prefLabel xml:lang="de">Wohlfahrtseffekt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13530-6">
    <skos:altLabel xml:lang="de">Fährschifffahrt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13530-6">
    <skos:altLabel xml:lang="de">Fährschiffahrt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18514-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10382-3">
    <skos:altLabel xml:lang="de">Zentralbankpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10382-3">
    <skos:altLabel xml:lang="de">Notenbankpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19455-6">
    <skos:altLabel xml:lang="de">Latifundium</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14129-2">
    <skos:narrower rdf:resource="descriptor/14130-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14129-2">
    <skos:narrower rdf:resource="descriptor/14962-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14129-2">
    <skos:narrower rdf:resource="descriptor/14125-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14129-2">
    <skos:narrower rdf:resource="descriptor/14916-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14129-2">
    <skos:narrower rdf:resource="descriptor/14955-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14129-2">
    <skos:narrower rdf:resource="descriptor/14124-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14129-2">
    <skos:narrower rdf:resource="descriptor/14829-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14129-2">
    <skos:narrower rdf:resource="descriptor/14123-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14129-2">
    <skos:narrower rdf:resource="descriptor/14127-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14129-2">
    <skos:narrower rdf:resource="descriptor/14964-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14129-2">
    <skos:narrower rdf:resource="descriptor/14122-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14129-2">
    <skos:narrower rdf:resource="descriptor/14969-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14129-2">
    <skos:narrower rdf:resource="descriptor/14121-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11880-3">
    <skos:prefLabel xml:lang="en">Settlement structure</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11880-3">
    <skos:prefLabel xml:lang="de">Siedlungsstruktur</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id124">
    <rdf:value xml:lang="de">Für die Privatisierung von Unternehmen</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11939-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70004">
    <skos:broader rdf:resource="thsys/70005"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16967-1">
    <skos:broader rdf:resource="thsys/70003"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16967-1">
    <skos:broader rdf:resource="descriptor/16972-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71005">
    <skos:prefLabel xml:lang="en">W.06  Industries</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71005">
    <skos:prefLabel xml:lang="de">W.06  Industrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10768-2">
    <skos:broader rdf:resource="thsys/71094"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10768-2">
    <skos:broader rdf:resource="descriptor/10746-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18597-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19435-5">
    <skos:narrower rdf:resource="descriptor/10685-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18765-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10784-4">
    <skos:related rdf:resource="descriptor/10605-2"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id198">
    <rdfs:seeAlso rdf:resource="descriptor/19011-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11895-4">
    <skos:narrower rdf:resource="descriptor/10575-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11895-4">
    <skos:narrower rdf:resource="descriptor/10562-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13515-2">
    <skos:broader rdf:resource="thsys/70942"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18589-4">
    <skos:altLabel xml:lang="de">Passagierschiffahrt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18589-4">
    <skos:altLabel xml:lang="de">Fahrgastschifffahrt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18589-4">
    <skos:altLabel xml:lang="de">Fahrgastschiffahrt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16169-1">
    <skos:related rdf:resource="descriptor/18597-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16169-1">
    <skos:related rdf:resource="descriptor/18588-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16169-1">
    <skos:related rdf:resource="descriptor/10923-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10346-0">
    <skos:broader rdf:resource="thsys/70179"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10346-0">
    <skos:broader rdf:resource="thsys/70020"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10346-0">
    <skos:broader rdf:resource="descriptor/10340-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16342-6">
    <skos:altLabel xml:lang="de">Zweikammersystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16342-6">
    <skos:altLabel xml:lang="de">Parlamentarisches System</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16342-6">
    <skos:altLabel xml:lang="de">Einkammersystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19634-6">
    <skos:broader rdf:resource="thsys/70007"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19634-6">
    <skos:broader rdf:resource="descriptor/19635-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70862">
    <skos:prefLabel xml:lang="de">W.22.01.07  Öffentliche Sicherheit und Justizverwaltung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13219-6">
    <skos:narrower rdf:resource="descriptor/18521-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13219-6">
    <skos:narrower rdf:resource="descriptor/13237-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13219-6">
    <skos:narrower rdf:resource="descriptor/19060-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13219-6">
    <skos:narrower rdf:resource="descriptor/13218-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13219-6">
    <skos:narrower rdf:resource="descriptor/18519-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13219-6">
    <skos:narrower rdf:resource="descriptor/13207-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13219-6">
    <skos:narrower rdf:resource="descriptor/13225-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13219-6">
    <skos:narrower rdf:resource="descriptor/13205-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15839-1">
    <skos:prefLabel xml:lang="en">Work motivation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15839-1">
    <skos:prefLabel xml:lang="de">Leistungsmotivation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11358-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12958-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10211-2">
    <skos:related rdf:resource="descriptor/11834-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10211-2">
    <skos:related rdf:resource="descriptor/18163-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10211-2">
    <skos:related rdf:resource="descriptor/10215-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18917-6">
    <skos:related rdf:resource="descriptor/10148-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18917-6">
    <skos:related rdf:resource="descriptor/18955-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18917-6">
    <skos:related rdf:resource="descriptor/18928-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18917-6">
    <skos:related rdf:resource="descriptor/19754-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18917-6">
    <skos:related rdf:resource="descriptor/18921-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18917-6">
    <skos:related rdf:resource="descriptor/12907-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18917-6">
    <skos:related rdf:resource="descriptor/11194-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18917-6">
    <skos:related rdf:resource="descriptor/18556-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10557-1">
    <skos:altLabel xml:lang="en">Multilateral aid</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10557-1">
    <skos:altLabel xml:lang="en">Coordination of aid</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10557-1">
    <skos:altLabel xml:lang="de">Multilaterale Entwicklungshilfe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10557-1">
    <skos:altLabel xml:lang="de">Entwicklungszusammenarbeit (Entwicklungskooperation)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10557-1">
    <skos:altLabel xml:lang="de">Entwicklungspolitische Zusammenarbeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10557-1">
    <skos:altLabel xml:lang="de">Drittlandkooperation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12018-0">
    <skos:narrower rdf:resource="descriptor/18333-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15009-0">
    <skos:related rdf:resource="descriptor/15727-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15009-0">
    <skos:related rdf:resource="descriptor/13325-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15009-0">
    <skos:related rdf:resource="descriptor/18959-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15009-0">
    <skos:related rdf:resource="descriptor/15988-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73376">
    <skos:prefLabel xml:lang="de">N.04.02.06  Politische Kultur und Partizipation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16410-1">
    <skos:broader rdf:resource="thsys/73383"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16410-1">
    <skos:broader rdf:resource="descriptor/16405-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24621-6">
    <skos:broader rdf:resource="thsys/70818"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13085-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16539-1">
    <skos:broader rdf:resource="thsys/70832"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16539-1">
    <skos:broader rdf:resource="thsys/70458"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16539-1">
    <skos:broader rdf:resource="descriptor/18913-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19267-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70814">
    <skos:narrower rdf:resource="descriptor/15859-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70814">
    <skos:narrower rdf:resource="descriptor/16485-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70814">
    <skos:narrower rdf:resource="descriptor/18976-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70814">
    <skos:narrower rdf:resource="descriptor/13401-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70814">
    <skos:narrower rdf:resource="descriptor/13780-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70814">
    <skos:narrower rdf:resource="descriptor/19846-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70814">
    <skos:narrower rdf:resource="descriptor/19236-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70814">
    <skos:narrower rdf:resource="descriptor/10298-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70814">
    <skos:narrower rdf:resource="descriptor/25621-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70814">
    <skos:narrower rdf:resource="descriptor/13398-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70814">
    <skos:narrower rdf:resource="descriptor/18975-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70814">
    <skos:narrower rdf:resource="descriptor/18301-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70814">
    <skos:narrower rdf:resource="descriptor/18970-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70814">
    <skos:narrower rdf:resource="descriptor/11839-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70814">
    <skos:narrower rdf:resource="descriptor/18844-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70814">
    <skos:narrower rdf:resource="descriptor/15069-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70814">
    <skos:narrower rdf:resource="descriptor/15088-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70814">
    <skos:narrower rdf:resource="descriptor/18969-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70814">
    <skos:narrower rdf:resource="descriptor/13440-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70814">
    <skos:narrower rdf:resource="descriptor/13485-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70814">
    <skos:narrower rdf:resource="descriptor/10300-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70814">
    <skos:narrower rdf:resource="descriptor/10180-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70814">
    <skos:narrower rdf:resource="descriptor/18864-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70814">
    <skos:narrower rdf:resource="descriptor/18915-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70814">
    <skos:narrower rdf:resource="descriptor/13974-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70814">
    <skos:narrower rdf:resource="descriptor/10176-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70814">
    <skos:narrower rdf:resource="descriptor/10177-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70814">
    <skos:narrower rdf:resource="descriptor/18948-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70814">
    <skos:narrower rdf:resource="descriptor/11032-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70814">
    <skos:narrower rdf:resource="descriptor/18144-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70814">
    <skos:narrower rdf:resource="descriptor/15941-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70814">
    <skos:narrower rdf:resource="descriptor/12865-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70814">
    <skos:narrower rdf:resource="descriptor/15940-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70814">
    <skos:narrower rdf:resource="descriptor/16698-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70814">
    <skos:narrower rdf:resource="descriptor/16030-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70814">
    <skos:narrower rdf:resource="descriptor/16477-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70814">
    <skos:narrower rdf:resource="descriptor/18972-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70814">
    <skos:narrower rdf:resource="descriptor/10173-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70814">
    <skos:narrower rdf:resource="descriptor/11660-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70814">
    <skos:narrower rdf:resource="descriptor/15938-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70814">
    <skos:narrower rdf:resource="descriptor/18971-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70814">
    <skos:narrower rdf:resource="descriptor/18968-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19367-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70198">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">G.00</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16124-2">
    <skos:prefLabel xml:lang="en">Employment protection legislation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16124-2">
    <skos:prefLabel xml:lang="de">Arbeitnehmerschutz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13322-6">
    <skos:related rdf:resource="descriptor/10138-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25742-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10463-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id99">
    <rdf:value xml:lang="de">Für die Geschichte des Bankwesens</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18252-4">
    <skos:related rdf:resource="descriptor/18396-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14434-4">
    <skos:related rdf:resource="descriptor/14442-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14434-4">
    <skos:related rdf:resource="descriptor/19052-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11886-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16136-2">
    <skos:related rdf:resource="descriptor/11069-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16136-2">
    <skos:related rdf:resource="descriptor/12090-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16136-2">
    <skos:related rdf:resource="descriptor/11310-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16136-2">
    <skos:related rdf:resource="descriptor/19110-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16136-2">
    <skos:related rdf:resource="descriptor/11311-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19532-0">
    <skos:broader rdf:resource="thsys/70888"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15073-5">
    <skos:altLabel xml:lang="de">Sozialindikator</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15073-5">
    <skos:altLabel xml:lang="de">Gesellschaftlicher Indikator</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14812-5">
    <skos:broader rdf:resource="thsys/70265"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11527-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19120-2">
    <skos:prefLabel xml:lang="en">Computer-aided selling</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19120-2">
    <skos:prefLabel xml:lang="de">CAS</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11659-2">
    <skos:related rdf:resource="descriptor/11682-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11659-2">
    <skos:related rdf:resource="descriptor/12029-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11659-2">
    <skos:related rdf:resource="descriptor/11658-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11659-2">
    <skos:related rdf:resource="descriptor/12026-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11659-2">
    <skos:related rdf:resource="descriptor/12018-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70120">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16495-2">
    <skos:broader rdf:resource="thsys/73362"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16495-2">
    <skos:broader rdf:resource="thsys/71092"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16495-2">
    <skos:broader rdf:resource="descriptor/16278-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10010-5">
    <zbwext:useInsteadNote rdf:nodeID="id170"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19535-1">
    <skos:related rdf:resource="descriptor/24733-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19535-1">
    <skos:related rdf:resource="descriptor/19522-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19535-1">
    <skos:related rdf:resource="descriptor/12217-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19535-1">
    <skos:related rdf:resource="descriptor/16043-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19535-1">
    <skos:related rdf:resource="descriptor/12232-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16540-2">
    <skos:broader rdf:resource="thsys/73370"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16540-2">
    <skos:broader rdf:resource="thsys/70833"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16540-2">
    <skos:broader rdf:resource="thsys/70324"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16540-2">
    <skos:broader rdf:resource="descriptor/16545-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16767-2">
    <skos:broader rdf:resource="thsys/70822"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16767-2">
    <skos:broader rdf:resource="descriptor/12640-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13106-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12621-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16510-4">
    <skos:related rdf:resource="descriptor/16332-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16510-4">
    <skos:related rdf:resource="descriptor/16295-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16510-4">
    <skos:related rdf:resource="descriptor/16328-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16510-4">
    <skos:related rdf:resource="descriptor/16286-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16510-4">
    <skos:related rdf:resource="descriptor/19683-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16510-4">
    <skos:related rdf:resource="descriptor/16304-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16510-4">
    <skos:related rdf:resource="descriptor/16395-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16510-4">
    <skos:related rdf:resource="descriptor/19668-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16510-4">
    <skos:related rdf:resource="descriptor/16521-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16510-4">
    <skos:related rdf:resource="descriptor/16509-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19418-5">
    <skos:related rdf:resource="descriptor/11666-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19418-5">
    <skos:related rdf:resource="descriptor/11657-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19418-5">
    <skos:related rdf:resource="descriptor/12218-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70180">
    <skos:narrower rdf:resource="descriptor/19215-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70180">
    <skos:narrower rdf:resource="descriptor/15861-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70180">
    <skos:narrower rdf:resource="descriptor/10344-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70180">
    <skos:narrower rdf:resource="descriptor/10334-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70180">
    <skos:narrower rdf:resource="descriptor/16562-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70180">
    <skos:narrower rdf:resource="descriptor/15860-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70180">
    <skos:narrower rdf:resource="descriptor/10056-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70180">
    <skos:narrower rdf:resource="descriptor/18151-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70180">
    <skos:narrower rdf:resource="descriptor/19533-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70180">
    <skos:narrower rdf:resource="descriptor/10207-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70180">
    <skos:narrower rdf:resource="descriptor/10144-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70180">
    <skos:narrower rdf:resource="descriptor/10139-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70180">
    <skos:narrower rdf:resource="descriptor/19219-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70180">
    <skos:narrower rdf:resource="descriptor/19218-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70180">
    <skos:narrower rdf:resource="descriptor/11156-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70180">
    <skos:narrower rdf:resource="descriptor/19217-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70180">
    <skos:narrower rdf:resource="descriptor/10038-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70180">
    <skos:narrower rdf:resource="descriptor/10151-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70180">
    <skos:narrower rdf:resource="descriptor/11065-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70180">
    <skos:narrower rdf:resource="descriptor/18832-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70180">
    <skos:narrower rdf:resource="descriptor/19540-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70180">
    <skos:narrower rdf:resource="descriptor/19216-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70180">
    <skos:narrower rdf:resource="descriptor/24622-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70180">
    <skos:narrower rdf:resource="descriptor/10150-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70180">
    <skos:narrower rdf:resource="descriptor/19208-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70180">
    <skos:narrower rdf:resource="descriptor/11153-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70180">
    <skos:narrower rdf:resource="descriptor/10034-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70180">
    <skos:narrower rdf:resource="thsys/71116"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70180">
    <skos:narrower rdf:resource="thsys/71117"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70180">
    <skos:narrower rdf:resource="thsys/71118"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70180">
    <skos:narrower rdf:resource="thsys/71119"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70180">
    <skos:narrower rdf:resource="thsys/70493"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70180">
    <skos:narrower rdf:resource="thsys/70481"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70180">
    <skos:narrower rdf:resource="thsys/70178"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16623-5">
    <skos:narrower rdf:resource="descriptor/10110-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16623-5">
    <skos:narrower rdf:resource="descriptor/11691-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18874-5">
    <skos:related rdf:resource="descriptor/18879-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18874-5">
    <skos:related rdf:resource="descriptor/19559-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18874-5">
    <skos:related rdf:resource="descriptor/11402-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18874-5">
    <skos:related rdf:resource="descriptor/11373-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18874-5">
    <skos:related rdf:resource="descriptor/18774-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18874-5">
    <skos:related rdf:resource="descriptor/18784-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19700-5">
    <skos:prefLabel xml:lang="en">Voter turnout</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19700-5">
    <skos:prefLabel xml:lang="de">Wahlbeteiligung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73385">
    <skos:prefLabel xml:lang="de">N.07.02  Arbeits- und Organisationspsychologie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16621-2">
    <zbwext:useInsteadNote rdf:nodeID="id123"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16621-2">
    <zbwext:useInsteadNote rdf:nodeID="id122"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24787-0">
    <skos:broader rdf:resource="thsys/70940"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24787-0">
    <skos:broader rdf:resource="descriptor/18615-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19611-4">
    <skos:broader rdf:resource="thsys/73354"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19611-4">
    <skos:broader rdf:resource="thsys/70013"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19611-4">
    <skos:broader rdf:resource="descriptor/16563-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19611-4">
    <skos:broader rdf:resource="descriptor/15872-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17932-1">
    <skos:altLabel xml:lang="de">Surinam</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17932-1">
    <skos:altLabel xml:lang="de">Republik Surinam</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17932-1">
    <skos:altLabel xml:lang="de">Niederländisch-Guayana</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70942">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.12.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19196-4">
    <skos:altLabel xml:lang="en">Speculative bubbles</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19196-4">
    <skos:altLabel xml:lang="de">Theorie der spekulativen Blasen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19196-4">
    <skos:altLabel xml:lang="de">Spekulative Blasen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12244-5">
    <skos:related rdf:resource="descriptor/12249-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12244-5">
    <skos:related rdf:resource="descriptor/12274-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12244-5">
    <skos:related rdf:resource="descriptor/12206-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12244-5">
    <skos:related rdf:resource="descriptor/12273-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16017-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14518-5">
    <skos:altLabel xml:lang="de">Auto-Kassettenrecorder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14518-5">
    <skos:altLabel xml:lang="de">Auto-CD-Spieler</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12903-5">
    <skos:narrower rdf:resource="descriptor/19106-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12903-5">
    <skos:narrower rdf:resource="descriptor/12898-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12903-5">
    <skos:narrower rdf:resource="descriptor/12896-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70854">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13385-3">
    <skos:altLabel xml:lang="en">Private security</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13385-3">
    <skos:altLabel xml:lang="de">Werttransport</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13385-3">
    <skos:altLabel xml:lang="de">Werkschutz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13385-3">
    <skos:altLabel xml:lang="de">Wachdienst</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13385-3">
    <skos:altLabel xml:lang="de">Wach- und Schließgesellschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13385-3">
    <skos:altLabel xml:lang="de">Sicherheitsservice</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13385-3">
    <skos:altLabel xml:lang="de">Sicherheitsdienst</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13385-3">
    <skos:altLabel xml:lang="de">Sicherheitsberatung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13385-3">
    <skos:altLabel xml:lang="de">Schutzdienst</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13385-3">
    <skos:altLabel xml:lang="de">Private Sicherheit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13385-3">
    <skos:altLabel xml:lang="de">Personenschutzdienst</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13385-3">
    <skos:altLabel xml:lang="de">Personenschutz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13385-3">
    <skos:altLabel xml:lang="de">Objektschutzdienst</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13385-3">
    <skos:altLabel xml:lang="de">Bewachungsgewerbe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12748-5">
    <skos:broader rdf:resource="thsys/70029"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12748-5">
    <skos:broader rdf:resource="descriptor/12747-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11676-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18009-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14479-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10206-2">
    <skos:related rdf:resource="descriptor/10907-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10206-2">
    <skos:related rdf:resource="descriptor/19226-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10206-2">
    <skos:related rdf:resource="descriptor/10205-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12028-4">
    <skos:related rdf:resource="descriptor/18747-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12028-4">
    <skos:related rdf:resource="descriptor/12029-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12028-4">
    <skos:related rdf:resource="descriptor/12026-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12028-4">
    <skos:related rdf:resource="descriptor/12020-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18545-3">
    <skos:broader rdf:resource="thsys/70947"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18545-3">
    <skos:broader rdf:resource="descriptor/11823-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16970-5">
    <skos:altLabel xml:lang="de">St. Gallen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id254">
    <rdfs:seeAlso rdf:resource="descriptor/13278-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70969">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16982-5">
    <skos:altLabel xml:lang="de">Suomi</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16982-5">
    <skos:altLabel xml:lang="de">Republik Finnland</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18892-3">
    <skos:narrower rdf:resource="descriptor/19528-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18892-3">
    <skos:narrower rdf:resource="descriptor/19526-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11374-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12186-0">
    <skos:broader rdf:resource="thsys/70087"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12186-0">
    <skos:broader rdf:resource="descriptor/12194-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19322-4">
    <skos:scopeNote xml:lang="de">Eine Volkswirtschaft ist dem Handel aller durch Abmachungen verbundenen Staaten in gleicher Weise geöffnet</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16792-3">
    <skos:altLabel xml:lang="de">Sachkatalog</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16792-3">
    <skos:altLabel xml:lang="de">Formalkatalog</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16792-3">
    <skos:altLabel xml:lang="de">Alphabetischer Katalog</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70244">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">B.04.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13180-0">
    <skos:altLabel xml:lang="de">Malerhandwerk</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13180-0">
    <skos:altLabel xml:lang="de">Glaserhandwerk</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13180-0">
    <skos:altLabel xml:lang="de">Fliesenlegerei</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13180-0">
    <skos:altLabel xml:lang="de">Baunebengewerbe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15635-3">
    <skos:broader rdf:resource="thsys/73348"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15635-3">
    <skos:broader rdf:resource="thsys/73344"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15635-3">
    <skos:broader rdf:resource="thsys/71109"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15635-3">
    <skos:broader rdf:resource="thsys/70993"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15635-3">
    <skos:broader rdf:resource="descriptor/15677-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18060-6">
    <skos:broader rdf:resource="thsys/71110"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18060-6">
    <skos:broader rdf:resource="descriptor/11790-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70036">
    <skos:narrower rdf:resource="descriptor/12855-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70036">
    <skos:narrower rdf:resource="descriptor/12298-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70036">
    <skos:narrower rdf:resource="descriptor/10021-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70036">
    <skos:narrower rdf:resource="descriptor/12297-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70036">
    <skos:narrower rdf:resource="descriptor/12296-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70036">
    <skos:narrower rdf:resource="descriptor/12295-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70036">
    <skos:narrower rdf:resource="descriptor/12272-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70036">
    <skos:narrower rdf:resource="descriptor/12291-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70036">
    <skos:narrower rdf:resource="descriptor/13837-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70036">
    <skos:narrower rdf:resource="descriptor/12285-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70036">
    <skos:narrower rdf:resource="descriptor/12284-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70036">
    <skos:narrower rdf:resource="descriptor/19089-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70036">
    <skos:narrower rdf:resource="descriptor/12282-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70036">
    <skos:narrower rdf:resource="descriptor/12280-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70036">
    <skos:narrower rdf:resource="descriptor/19088-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70036">
    <skos:narrower rdf:resource="descriptor/12276-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70036">
    <skos:narrower rdf:resource="descriptor/12275-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70036">
    <skos:narrower rdf:resource="descriptor/19082-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70036">
    <skos:narrower rdf:resource="descriptor/11772-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70036">
    <skos:narrower rdf:resource="descriptor/12271-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70036">
    <skos:narrower rdf:resource="descriptor/12270-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70036">
    <skos:narrower rdf:resource="descriptor/12366-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70036">
    <skos:narrower rdf:resource="thsys/71044"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10072-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12874-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18790-4">
    <skos:altLabel xml:lang="en">Dispute settlement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18790-4">
    <skos:altLabel xml:lang="de">Internationales Schiedsverfahren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18790-4">
    <skos:altLabel xml:lang="de">Internationales Schiedsrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18790-4">
    <skos:altLabel xml:lang="de">Internationales Schiedsgericht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18790-4">
    <skos:altLabel xml:lang="de">Internationaler Schiedsspruch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18790-4">
    <skos:altLabel xml:lang="de">Internationale Wirtschaftsschiedsgerichtsbarkeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18790-4">
    <skos:altLabel xml:lang="de">Internationale Handelsschiedsgerichtsbarkeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18790-4">
    <skos:altLabel xml:lang="de">Internationale Handelsarbitrage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18790-4">
    <skos:altLabel xml:lang="de">Außenwirtschaftsarbitrage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70926">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19496-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16959-0">
    <skos:broader rdf:resource="thsys/70003"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16959-0">
    <skos:broader rdf:resource="descriptor/16972-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12648-2">
    <skos:prefLabel xml:lang="en">Materials management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12648-2">
    <skos:prefLabel xml:lang="de">Materialwirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18378-3">
    <skos:broader rdf:resource="thsys/71011"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18378-3">
    <skos:broader rdf:resource="thsys/70811"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18378-3">
    <skos:broader rdf:resource="descriptor/15753-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12620-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11769-2">
    <skos:scopeNote xml:lang="de">Für die allgemeine Beteiligung von Arbeitnehmern an betrieblichen Entscheidungsprozessen BENUTZE Mitbestimmung bzw. Kooperative Führung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11769-2">
    <skos:scopeNote xml:lang="de">Alle Formen der betrieblichen und überbetrieblichen Beteiligung von Arbeitnehmern am Produktivkapital.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12922-1">
    <skos:scopeNote xml:lang="de">Die Rohstoffwirtschaft umfasst das Aufsuchen, die Gewinnung und die Verarbeitung von Rohstoffen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20341-4">
    <skos:broader rdf:resource="thsys/73341"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20341-4">
    <skos:broader rdf:resource="thsys/71037"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20341-4">
    <skos:broader rdf:resource="descriptor/12079-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20341-4">
    <skos:broader rdf:resource="descriptor/18017-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15767-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18348-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10327-4">
    <skos:scopeNote xml:lang="de">Sowohl für mikro- wie auch makroökonomische Produktionsfunktionen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11353-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10147-6">
    <skos:broader rdf:resource="thsys/71117"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10147-6">
    <skos:broader rdf:resource="descriptor/10141-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11862-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14819-5">
    <skos:altLabel xml:lang="de">Plastikfolie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14819-5">
    <skos:altLabel xml:lang="de">Kunststoffolie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19383-5">
    <skos:broader rdf:resource="thsys/71018"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19383-5">
    <skos:broader rdf:resource="thsys/70986"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11030-1">
    <zbwext:useInsteadNote rdf:nodeID="id354"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14611-1">
    <skos:altLabel xml:lang="de">Nahrungsmittelzusatzstoff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14611-1">
    <skos:altLabel xml:lang="de">Nahrungsmittelchemikalie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14611-1">
    <skos:altLabel xml:lang="de">Lebensmittelzusatzstoff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14611-1">
    <skos:altLabel xml:lang="de">Lebensmittelkonservierungsstoff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14611-1">
    <skos:altLabel xml:lang="de">Konservierungsstoff für Nahrungsmittel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14611-1">
    <skos:altLabel xml:lang="de">Geschmacksverstärker</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10522-6">
    <skos:broader rdf:resource="thsys/71110"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10522-6">
    <skos:broader rdf:resource="thsys/70091"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10522-6">
    <skos:broader rdf:resource="descriptor/11782-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19558-3">
    <skos:broader rdf:resource="thsys/71118"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19558-3">
    <skos:broader rdf:resource="thsys/70057"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19558-3">
    <skos:broader rdf:resource="descriptor/19210-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18421-0">
    <skos:prefLabel xml:lang="en">Automotive engineering</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18421-0">
    <skos:prefLabel xml:lang="de">Kraftfahrzeugtechnik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10642-3">
    <skos:related rdf:resource="descriptor/10787-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10642-3">
    <skos:related rdf:resource="descriptor/19313-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10642-3">
    <skos:related rdf:resource="descriptor/10652-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10268-1">
    <skos:broader rdf:resource="thsys/71111"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10268-1">
    <skos:broader rdf:resource="descriptor/18948-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15713-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73379">
    <skos:narrower rdf:resource="descriptor/19857-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73379">
    <skos:narrower rdf:resource="descriptor/16344-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73379">
    <skos:narrower rdf:resource="descriptor/16343-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73379">
    <skos:narrower rdf:resource="descriptor/16342-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73379">
    <skos:narrower rdf:resource="descriptor/16340-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73379">
    <skos:narrower rdf:resource="descriptor/16338-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73379">
    <skos:narrower rdf:resource="descriptor/19693-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73379">
    <skos:narrower rdf:resource="descriptor/19692-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73379">
    <skos:narrower rdf:resource="descriptor/13984-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19575-3">
    <skos:related rdf:resource="descriptor/15225-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11665-0">
    <skos:altLabel xml:lang="en">Church rate</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11665-0">
    <skos:altLabel xml:lang="de">Lohnkirchensteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11665-0">
    <skos:altLabel xml:lang="de">Kirchgeld</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11913-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12257-3">
    <skos:broader rdf:resource="thsys/71045"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12257-3">
    <skos:broader rdf:resource="thsys/70259"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12257-3">
    <skos:broader rdf:resource="descriptor/12251-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15689-1">
    <skos:broader rdf:resource="thsys/70007"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15689-1">
    <skos:broader rdf:resource="descriptor/15690-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11342-3">
    <skos:prefLabel xml:lang="en">Wage determination</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11342-3">
    <skos:prefLabel xml:lang="de">Lohnbildung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13678-2">
    <skos:related rdf:resource="descriptor/18650-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13678-2">
    <skos:related rdf:resource="descriptor/13667-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73298">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">P.20.07</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16666-1">
    <skos:broader rdf:resource="thsys/73356"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16666-1">
    <skos:broader rdf:resource="thsys/73346"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16666-1">
    <skos:broader rdf:resource="descriptor/16668-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11092-0">
    <skos:related rdf:resource="descriptor/11765-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11092-0">
    <skos:related rdf:resource="descriptor/11073-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11092-0">
    <skos:related rdf:resource="descriptor/12275-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11052-5">
    <skos:broader rdf:resource="thsys/73385"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11052-5">
    <skos:broader rdf:resource="thsys/70244"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10973-1">
    <skos:prefLabel xml:lang="en">Small business policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10973-1">
    <skos:prefLabel xml:lang="de">Mittelstandspolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10589-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13287-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18354-3">
    <skos:prefLabel xml:lang="en">Energy network</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18354-3">
    <skos:prefLabel xml:lang="de">Energieverbund</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13529-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15605-5">
    <zbwext:useInsteadNote rdf:nodeID="id66"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14771-0">
    <skos:prefLabel xml:lang="en">Ceramic sanitaryware</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14771-0">
    <skos:prefLabel xml:lang="de">Sanitärkeramik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10255-3">
    <skos:prefLabel xml:lang="en">Phillips curve</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10255-3">
    <skos:prefLabel xml:lang="de">Phillips-Kurve</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18572-0">
    <skos:broader rdf:resource="thsys/70941"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18572-0">
    <skos:broader rdf:resource="descriptor/13264-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11852-1">
    <skos:related rdf:resource="descriptor/11879-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11852-1">
    <skos:related rdf:resource="descriptor/16205-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11852-1">
    <skos:related rdf:resource="descriptor/19697-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11852-1">
    <skos:related rdf:resource="descriptor/16550-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11852-1">
    <skos:related rdf:resource="descriptor/15995-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11852-1">
    <skos:related rdf:resource="descriptor/16549-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11852-1">
    <skos:related rdf:resource="descriptor/11998-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11852-1">
    <skos:related rdf:resource="descriptor/13394-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11852-1">
    <skos:related rdf:resource="descriptor/19691-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11852-1">
    <skos:related rdf:resource="descriptor/11965-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11852-1">
    <skos:related rdf:resource="descriptor/19467-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15066-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19088-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18861-0">
    <skos:broader rdf:resource="thsys/73357"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18861-0">
    <skos:broader rdf:resource="thsys/70142"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18861-0">
    <skos:broader rdf:resource="descriptor/15909-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11530-2">
    <skos:altLabel xml:lang="en">Cost-efficiency analysis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11530-2">
    <skos:altLabel xml:lang="de">Kostenwirksamkeitsanalyse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11530-2">
    <skos:altLabel xml:lang="de">Kosten-Wirksamkeitsanalyse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12898-0">
    <skos:altLabel xml:lang="de">Jahresabschlußprüfung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12898-0">
    <skos:altLabel xml:lang="de">Abschlussprüfung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12898-0">
    <skos:altLabel xml:lang="de">Abschlußprüfung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15722-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71032">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71071">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12705-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14926-4">
    <skos:narrower rdf:resource="descriptor/18216-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14926-4">
    <skos:narrower rdf:resource="descriptor/14945-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14926-4">
    <skos:narrower rdf:resource="descriptor/14932-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14679-2">
    <skos:broader rdf:resource="thsys/70556"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14679-2">
    <skos:broader rdf:resource="descriptor/14656-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16458-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11068-4">
    <skos:related rdf:resource="descriptor/11069-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11068-4">
    <skos:related rdf:resource="descriptor/19285-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11068-4">
    <skos:related rdf:resource="descriptor/11063-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11068-4">
    <skos:related rdf:resource="descriptor/11127-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11068-4">
    <skos:related rdf:resource="descriptor/11049-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11068-4">
    <skos:related rdf:resource="descriptor/19686-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11068-4">
    <skos:related rdf:resource="descriptor/19607-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11068-4">
    <skos:related rdf:resource="descriptor/16292-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11068-4">
    <skos:related rdf:resource="descriptor/11066-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11068-4">
    <skos:related rdf:resource="descriptor/10976-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11068-4">
    <skos:related rdf:resource="descriptor/15704-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11068-4">
    <skos:related rdf:resource="descriptor/16384-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11068-4">
    <skos:related rdf:resource="descriptor/12009-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11068-4">
    <skos:related rdf:resource="descriptor/16602-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11068-4">
    <skos:related rdf:resource="descriptor/11040-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11370-5">
    <skos:related rdf:resource="descriptor/11362-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71072">
    <skos:broader rdf:resource="thsys/70144"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14812-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13295-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13398-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70822">
    <rdfs:label xml:lang="de">Bibliotheken</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id207">
    <rdfs:seeAlso rdf:resource="descriptor/16517-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11418-3">
    <skos:altLabel xml:lang="de">Geldangebotstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11418-3">
    <skos:altLabel xml:lang="de">Geldangebotsfunktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19701-3">
    <skos:prefLabel xml:lang="en">Constituency</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19701-3">
    <skos:prefLabel xml:lang="de">Wahlkreis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10380-0">
    <skos:prefLabel xml:lang="en">Incomes policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10380-0">
    <skos:prefLabel xml:lang="de">Einkommenspolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18967-5">
    <skos:prefLabel xml:lang="en">Recultivation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18967-5">
    <skos:prefLabel xml:lang="de">Rekultivierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19453-3">
    <skos:related rdf:resource="descriptor/18753-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19453-3">
    <skos:related rdf:resource="descriptor/19456-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19453-3">
    <skos:related rdf:resource="descriptor/19452-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19453-3">
    <skos:related rdf:resource="descriptor/11844-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19453-3">
    <skos:related rdf:resource="descriptor/11995-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70174">
    <skos:prefLabel xml:lang="en">G.01.03  Eastern Europe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70174">
    <skos:prefLabel xml:lang="de">G.01.03  Osteuropa</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id263">
    <rdf:value xml:lang="de">Für materielle Mittel der Bedürfnisbefriedigung im Sinne der Volkswirtschaftslehre</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19565-6">
    <skos:scopeNote xml:lang="de">Strategie von Unternehmen zur Sicherung ihrer Wettbewerbsfähigkeit, z.B. im Bereich der Produktpolitik, der Produktionstechnik oder der Organisation.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17753-1">
    <skos:prefLabel xml:lang="en">Western Sahara</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17753-1">
    <skos:prefLabel xml:lang="de">Westsahara</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17735-3">
    <skos:altLabel xml:lang="de">Volksrepublik Kongo</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17735-3">
    <skos:altLabel xml:lang="de">Republik Kongo</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17735-3">
    <skos:altLabel xml:lang="de">Kongo-Brazzaville</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17735-3">
    <skos:altLabel xml:lang="de">Französisch-Kongo</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13903-0">
    <skos:prefLabel xml:lang="en">Home decorating crafts</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13903-0">
    <skos:prefLabel xml:lang="de">Raumausstatterhandwerk</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18278-0">
    <skos:broader rdf:resource="thsys/70201"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19195-6">
    <skos:scopeNote xml:lang="de">Literatur über Engpass-Situationen, bei denen negative externe Effekte auftreten können, sobald die gemeinsame Inanspruchnahme eines Gutes zu Nutzenverlust führt, oder bei denen die Knappheit eines Faktors die ökonomische Entwicklung hemmt.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12418-5">
    <skos:related rdf:resource="descriptor/12452-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12418-5">
    <skos:related rdf:resource="descriptor/12282-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12418-5">
    <skos:related rdf:resource="descriptor/12366-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10020-2">
    <skos:related rdf:resource="descriptor/13790-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10020-2">
    <skos:related rdf:resource="descriptor/10284-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10020-2">
    <skos:related rdf:resource="descriptor/11571-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10020-2">
    <skos:related rdf:resource="descriptor/12057-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10020-2">
    <skos:related rdf:resource="descriptor/18674-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15441-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15600-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16949-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18620-1">
    <skos:related rdf:resource="descriptor/13584-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18620-1">
    <skos:related rdf:resource="descriptor/12013-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18620-1">
    <skos:related rdf:resource="descriptor/15519-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18620-1">
    <skos:related rdf:resource="descriptor/15518-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18620-1">
    <skos:related rdf:resource="descriptor/21020-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18620-1">
    <skos:related rdf:resource="descriptor/24787-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18620-1">
    <skos:related rdf:resource="descriptor/19116-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18620-1">
    <skos:related rdf:resource="descriptor/13622-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18620-1">
    <skos:related rdf:resource="descriptor/12643-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18620-1">
    <skos:related rdf:resource="descriptor/18617-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18620-1">
    <skos:related rdf:resource="descriptor/18615-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18620-1">
    <skos:related rdf:resource="descriptor/13620-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18620-1">
    <skos:related rdf:resource="descriptor/12689-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10272-3">
    <skos:prefLabel xml:lang="en">National accounts</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10272-3">
    <skos:prefLabel xml:lang="de">Volkswirtschaftliche Gesamtrechnung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18300-5">
    <skos:related rdf:resource="descriptor/14931-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18300-5">
    <skos:related rdf:resource="descriptor/13218-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19358-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13395-0">
    <skos:related rdf:resource="descriptor/10589-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13395-0">
    <skos:related rdf:resource="descriptor/16439-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13395-0">
    <skos:related rdf:resource="descriptor/20826-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13395-0">
    <skos:related rdf:resource="descriptor/10558-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18001-1">
    <skos:prefLabel xml:lang="en">Foreign affiliate</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18001-1">
    <skos:prefLabel xml:lang="de">Auslandsniederlassung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12036-5">
    <skos:prefLabel xml:lang="en">Interfirm cooperation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12036-5">
    <skos:prefLabel xml:lang="de">Unternehmenskooperation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15373-0">
    <skos:altLabel xml:lang="en">Econometric method</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15373-0">
    <skos:altLabel xml:lang="de">Ökonometrik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14504-2">
    <skos:prefLabel xml:lang="en">Small electric appliances</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14504-2">
    <skos:prefLabel xml:lang="de">Elektrokleingerät</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70140">
    <skos:broader rdf:resource="thsys/b"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11282-2">
    <skos:related rdf:resource="descriptor/19491-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11282-2">
    <skos:related rdf:resource="descriptor/11286-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16971-3">
    <skos:broader rdf:resource="thsys/70003"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16971-3">
    <skos:broader rdf:resource="descriptor/16972-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18540-6">
    <skos:prefLabel xml:lang="en">Import-export bank</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18540-6">
    <skos:prefLabel xml:lang="de">Außenhandelsbank</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10178-2">
    <skos:related rdf:resource="descriptor/10182-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18093-5">
    <skos:altLabel xml:lang="en">Smallholder agriculture</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18093-5">
    <skos:altLabel xml:lang="de">Small-scale farming</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18093-5">
    <skos:altLabel xml:lang="de">Kleinbauern</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18093-5">
    <skos:altLabel xml:lang="de">Kleinbäuerlicher Betrieb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18093-5">
    <skos:altLabel xml:lang="de">Agrarwirtschaftlicher Kleinbetrieb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19470-3">
    <skos:scopeNote xml:lang="de">Nur für öffentliche Dienstleistungen der Städte und Gemeinden.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11246-6">
    <skos:prefLabel xml:lang="en">Job security</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11246-6">
    <skos:prefLabel xml:lang="de">Arbeitsplatzsicherung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10255-3">
    <skos:altLabel xml:lang="en">Unemployment inflation trade-off</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10255-3">
    <skos:altLabel xml:lang="en">Inflation unemployment trade-off</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10276-2">
    <skos:narrower rdf:resource="descriptor/10267-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16267-1">
    <skos:broader rdf:resource="thsys/73375"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16267-1">
    <skos:broader rdf:resource="descriptor/16286-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19404-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19077-5">
    <skos:altLabel xml:lang="de">Transformationsländer Osteuropas</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19077-5">
    <skos:altLabel xml:lang="de">Postsozialistische Länder Europas</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19077-5">
    <skos:altLabel xml:lang="de">Postsocialist European Economies</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19077-5">
    <skos:altLabel xml:lang="de">Post-Socialist European Economies</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19077-5">
    <skos:altLabel xml:lang="de">Osteuropäische Staaten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19077-5">
    <skos:altLabel xml:lang="de">Osteuropäische Länder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19077-5">
    <skos:altLabel xml:lang="de">Ost-, Mittel- und Südosteuropa</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19077-5">
    <skos:altLabel xml:lang="de">Mittel- und Osteuropa</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19077-5">
    <skos:altLabel xml:lang="de">European Transformation States</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19077-5">
    <skos:altLabel xml:lang="de">European Transformation Countries</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19077-5">
    <skos:altLabel xml:lang="de">Europäische Transformationsstaaten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19077-5">
    <skos:altLabel xml:lang="de">Europäische Transformationsländer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19077-5">
    <skos:altLabel xml:lang="de">Ehemals sozialistische Staaten Europas</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19077-5">
    <skos:altLabel xml:lang="de">Central and Eastern European Countries</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19077-5">
    <skos:altLabel xml:lang="de">CEEC (Central and Eastern European Countries)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19077-5">
    <skos:altLabel xml:lang="de">CEE (Central and East Europe)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10750-0">
    <skos:related rdf:resource="descriptor/10719-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17654-3">
    <skos:related rdf:resource="descriptor/18284-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13960-2">
    <skos:altLabel xml:lang="de">Ingenieurinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19138-4">
    <skos:related rdf:resource="descriptor/11064-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19138-4">
    <skos:related rdf:resource="descriptor/19118-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16201-3">
    <skos:altLabel xml:lang="en">White-collar crime</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16201-3">
    <skos:altLabel xml:lang="en">White collar crime</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16201-3">
    <skos:altLabel xml:lang="de">Wirtschaftsstraftat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16201-3">
    <skos:altLabel xml:lang="de">Wirtschaftsdelikt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13293-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10087-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11932-3">
    <skos:prefLabel xml:lang="en">Location theory</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11932-3">
    <skos:prefLabel xml:lang="de">Standorttheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15142-5">
    <skos:narrower rdf:resource="descriptor/19605-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15142-5">
    <skos:narrower rdf:resource="descriptor/15320-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15142-5">
    <skos:narrower rdf:resource="descriptor/15149-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13046-4">
    <skos:related rdf:resource="descriptor/14432-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15635-3">
    <skos:narrower rdf:resource="descriptor/18289-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15635-3">
    <skos:narrower rdf:resource="descriptor/18777-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15635-3">
    <skos:narrower rdf:resource="descriptor/18317-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14817-2">
    <skos:narrower rdf:resource="descriptor/14824-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13039-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70458">
    <skos:prefLabel xml:lang="en">V.14.01.02  Social insurance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70458">
    <skos:prefLabel xml:lang="de">V.14.01.02  Sozialversicherung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19664-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11154-4">
    <skos:broader rdf:resource="thsys/73380"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16514-3">
    <skos:broader rdf:resource="thsys/73369"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16514-3">
    <skos:broader rdf:resource="thsys/70828"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16514-3">
    <skos:broader rdf:resource="descriptor/16515-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19661-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70183">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70176">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.14.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11170-6">
    <skos:altLabel xml:lang="en">Policy science</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11170-6">
    <skos:altLabel xml:lang="en">Policy evaluation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11170-6">
    <skos:altLabel xml:lang="de">Policy-Studie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11170-6">
    <skos:altLabel xml:lang="de">Policy-Forschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11170-6">
    <skos:altLabel xml:lang="de">Policy Forschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18972-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70967">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.07.03</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73363">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11248-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12423-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13563-5">
    <skos:broader rdf:resource="thsys/70938"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13563-5">
    <skos:broader rdf:resource="descriptor/13510-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11810-3">
    <skos:related rdf:resource="descriptor/15752-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11810-3">
    <skos:related rdf:resource="descriptor/12975-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11810-3">
    <skos:related rdf:resource="descriptor/11812-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11810-3">
    <skos:related rdf:resource="descriptor/15993-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11810-3">
    <skos:related rdf:resource="descriptor/18174-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11810-3">
    <skos:related rdf:resource="descriptor/18350-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11810-3">
    <skos:related rdf:resource="descriptor/18996-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11810-3">
    <skos:related rdf:resource="descriptor/18348-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11810-3">
    <skos:related rdf:resource="descriptor/14175-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11810-3">
    <skos:related rdf:resource="descriptor/18375-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10809-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19700-5">
    <skos:broader rdf:resource="thsys/73376"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19700-5">
    <skos:broader rdf:resource="descriptor/16353-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17586-1">
    <skos:prefLabel xml:lang="en">Philippines</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17586-1">
    <skos:prefLabel xml:lang="de">Philippinen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10350-2">
    <skos:altLabel xml:lang="en">Long waves</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10350-2">
    <skos:altLabel xml:lang="en">Long swings</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10350-2">
    <skos:altLabel xml:lang="en">Long cycles</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10350-2">
    <skos:altLabel xml:lang="de">Kondratieff-Zyklus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13056-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19557-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10537-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19307-0">
    <skos:prefLabel xml:lang="en">Sugar policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19307-0">
    <skos:prefLabel xml:lang="de">Zuckerpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13981-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70874">
    <rdfs:label xml:lang="en">Advertising industry</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70874">
    <rdfs:label xml:lang="de">Werbewirtschaft</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18489-1">
    <skos:scopeNote xml:lang="de">Nur für großräumige Industriegebiete.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17593-4">
    <skos:broader rdf:resource="thsys/71032"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17593-4">
    <skos:broader rdf:resource="descriptor/17595-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19699-6">
    <skos:prefLabel xml:lang="en">Campaign finance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19699-6">
    <skos:prefLabel xml:lang="de">Wahlkampffinanzierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18889-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18346-2">
    <skos:altLabel xml:lang="de">Dezentrale Energieerzeugung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12230-2">
    <skos:related rdf:resource="descriptor/18681-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12230-2">
    <skos:related rdf:resource="descriptor/12299-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12230-2">
    <skos:related rdf:resource="descriptor/12272-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12230-2">
    <skos:related rdf:resource="descriptor/12212-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12230-2">
    <skos:related rdf:resource="descriptor/19265-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12230-2">
    <skos:related rdf:resource="descriptor/12218-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12230-2">
    <skos:related rdf:resource="descriptor/19669-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12230-2">
    <skos:related rdf:resource="descriptor/18674-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12230-2">
    <skos:related rdf:resource="descriptor/12214-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12230-2">
    <skos:related rdf:resource="descriptor/13712-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20402-3">
    <skos:broader rdf:resource="thsys/70582"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70851">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11681-2">
    <skos:altLabel xml:lang="de">Verkehrsteuerrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11681-2">
    <skos:altLabel xml:lang="de">Spielbankabgabe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11681-2">
    <skos:altLabel xml:lang="de">Schankerlaubnissteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11681-2">
    <skos:altLabel xml:lang="de">Rennwett- und Lotteriesteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11681-2">
    <skos:altLabel xml:lang="de">Feuerschutzsteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11681-2">
    <skos:altLabel xml:lang="de">Beförderungsteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10070-1">
    <skos:altLabel xml:lang="en">Total equilibrium model</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10070-1">
    <skos:altLabel xml:lang="en">Computational general equilibrium model</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10070-1">
    <skos:altLabel xml:lang="en">CGE (Computational General Equilibrium)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10070-1">
    <skos:altLabel xml:lang="en">Applied general equilibrium model</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10070-1">
    <skos:altLabel xml:lang="de">Walrasianische Gleichgewichtstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10070-1">
    <skos:altLabel xml:lang="de">Totalgleichgewicht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10070-1">
    <skos:altLabel xml:lang="de">Totales Gleichgewicht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10070-1">
    <skos:altLabel xml:lang="de">Tatonnement-Prozess</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10070-1">
    <skos:altLabel xml:lang="de">Tatonnement-Prozeß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10070-1">
    <skos:altLabel xml:lang="de">Tatonnement-Gleichgewicht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10070-1">
    <skos:altLabel xml:lang="de">Auktionator-Preisbildung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10070-1">
    <skos:altLabel xml:lang="de">Arrow-Debreu-Gleichgewichtsmodell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10070-1">
    <skos:altLabel xml:lang="de">Arrow-Debreu Gleichgewichtstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10070-1">
    <skos:altLabel xml:lang="de">Allgemeines Gleichgewichtsmodell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10070-1">
    <skos:altLabel xml:lang="de">Allgemeine Gleichgewichtstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17992-4">
    <skos:broader rdf:resource="thsys/70307"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12252-6">
    <skos:prefLabel xml:lang="en">Credit risk collateralization</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12252-6">
    <skos:prefLabel xml:lang="de">Kreditsicherung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21020-5">
    <skos:related rdf:resource="descriptor/18620-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21020-5">
    <skos:related rdf:resource="descriptor/19114-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21020-5">
    <skos:related rdf:resource="descriptor/19544-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21020-5">
    <skos:related rdf:resource="descriptor/19654-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13453-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10856-5">
    <zbwext:useInsteadNote rdf:nodeID="id32"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20402-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16336-1">
    <skos:broader rdf:resource="thsys/73380"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16336-1">
    <skos:broader rdf:resource="thsys/73360"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16336-1">
    <skos:broader rdf:resource="thsys/70871"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13571-6">
    <skos:scopeNote xml:lang="de">Stellung und Bewertung einzelner Verkehrsmittel in der Gesellschaft. Rolle der Mobilität in der Gesellschaft. Analyse gruppenspezifischen Verkehrsverhaltens.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19812-1">
    <skos:narrower rdf:resource="descriptor/19509-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12253-4">
    <skos:altLabel xml:lang="en">Creditworthiness</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12253-4">
    <skos:altLabel xml:lang="en">Credit scoring</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12253-4">
    <skos:altLabel xml:lang="de">Kreditwürdigkeitsprüfung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12253-4">
    <skos:altLabel xml:lang="de">Kreditrating</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12253-4">
    <skos:altLabel xml:lang="de">Kredit-Rating</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12253-4">
    <skos:altLabel xml:lang="de">Kreditfähigkeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12253-4">
    <skos:altLabel xml:lang="de">Bonitätsbeurteilung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12253-4">
    <skos:altLabel xml:lang="de">Bonitätsanalyse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12253-4">
    <skos:altLabel xml:lang="de">Bonität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10923-2">
    <skos:related rdf:resource="descriptor/10962-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10923-2">
    <skos:related rdf:resource="descriptor/12036-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10923-2">
    <skos:related rdf:resource="descriptor/16169-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10923-2">
    <skos:related rdf:resource="descriptor/16051-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10923-2">
    <skos:related rdf:resource="descriptor/10897-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17491-5">
    <skos:scopeNote xml:lang="de">Erklärte sich 1991 unabhängig von der Sowjetunion; Mitglied der GUS</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16982-5">
    <skos:prefLabel xml:lang="en">Finland</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16982-5">
    <skos:prefLabel xml:lang="de">Finnland</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13080-4">
    <skos:prefLabel xml:lang="en">Opticians</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13080-4">
    <skos:prefLabel xml:lang="de">Optikerhandwerk</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10092-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12441-3">
    <skos:prefLabel xml:lang="en">Value adjustment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12441-3">
    <skos:prefLabel xml:lang="de">Wertberichtigung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13723-2">
    <skos:altLabel xml:lang="en">Financial asset market</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13723-2">
    <skos:altLabel xml:lang="en">Capital market</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13723-2">
    <skos:altLabel xml:lang="en">Asset market</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13723-2">
    <skos:altLabel xml:lang="de">Markt der Finanztitel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13723-2">
    <skos:altLabel xml:lang="de">Kapitalmarkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12403-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70056">
    <skos:prefLabel xml:lang="de">V.14  Soziales, Gesundheit und Bildung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15654-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10207-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13278-4">
    <skos:prefLabel xml:lang="en">Telecommunications</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13278-4">
    <skos:prefLabel xml:lang="de">Telekommunikation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13776-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13384-5">
    <skos:altLabel xml:lang="en">Labour contracting</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13384-5">
    <skos:altLabel xml:lang="en">Labour brokering</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13384-5">
    <skos:altLabel xml:lang="en">Labor contracting</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13384-5">
    <skos:altLabel xml:lang="en">Labor brokering</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13384-5">
    <skos:altLabel xml:lang="en">Headhunter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13384-5">
    <skos:altLabel xml:lang="en">Executive search</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13384-5">
    <skos:altLabel xml:lang="de">Private Personalvermittlung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13384-5">
    <skos:altLabel xml:lang="de">Private Arbeitsvermittlung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13384-5">
    <skos:altLabel xml:lang="de">Personalvermittlung durch Private</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13384-5">
    <skos:altLabel xml:lang="de">Gewerbliche Arbeitsvermittlung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13384-5">
    <skos:altLabel xml:lang="de">Führungskräftevermittlung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12794-5">
    <skos:narrower rdf:resource="descriptor/20206-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12794-5">
    <skos:narrower rdf:resource="descriptor/12788-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12794-5">
    <skos:narrower rdf:resource="descriptor/19555-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12794-5">
    <skos:narrower rdf:resource="descriptor/10215-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12794-5">
    <skos:narrower rdf:resource="descriptor/10931-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12794-5">
    <skos:narrower rdf:resource="descriptor/10930-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12794-5">
    <skos:narrower rdf:resource="descriptor/12793-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12794-5">
    <skos:narrower rdf:resource="descriptor/10204-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12794-5">
    <skos:narrower rdf:resource="descriptor/18555-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18977-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13513-6">
    <skos:related rdf:resource="descriptor/13580-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13513-6">
    <skos:related rdf:resource="descriptor/13587-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13513-6">
    <skos:related rdf:resource="descriptor/13584-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13513-6">
    <skos:related rdf:resource="descriptor/13510-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13513-6">
    <skos:related rdf:resource="descriptor/15518-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13513-6">
    <skos:related rdf:resource="descriptor/13546-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13513-6">
    <skos:related rdf:resource="descriptor/13600-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13513-6">
    <skos:related rdf:resource="descriptor/12693-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13513-6">
    <skos:related rdf:resource="descriptor/19066-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13513-6">
    <skos:related rdf:resource="descriptor/13047-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12620-3">
    <skos:related rdf:resource="descriptor/19791-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12620-3">
    <skos:related rdf:resource="descriptor/15842-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12620-3">
    <skos:related rdf:resource="descriptor/12615-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12620-3">
    <skos:related rdf:resource="descriptor/12618-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12620-3">
    <skos:related rdf:resource="descriptor/15837-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12620-3">
    <skos:related rdf:resource="descriptor/12617-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12620-3">
    <skos:related rdf:resource="descriptor/12631-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12620-3">
    <skos:related rdf:resource="descriptor/11305-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19082-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17735-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12670-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12984-0">
    <skos:related rdf:resource="descriptor/14193-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12984-0">
    <skos:related rdf:resource="descriptor/18394-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12984-0">
    <skos:related rdf:resource="descriptor/19387-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12984-0">
    <skos:related rdf:resource="descriptor/18372-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12984-0">
    <skos:related rdf:resource="descriptor/18393-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12984-0">
    <skos:related rdf:resource="descriptor/19047-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73382">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14091-1">
    <skos:altLabel xml:lang="en">Sorghum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14679-2">
    <skos:prefLabel xml:lang="en">Urea</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14679-2">
    <skos:prefLabel xml:lang="de">Harnstoff</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13708-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14196-1">
    <skos:prefLabel xml:lang="en">Natural gas</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14196-1">
    <skos:prefLabel xml:lang="de">Erdgas</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18856-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18272-5">
    <skos:prefLabel xml:lang="en">Packaging waste law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18272-5">
    <skos:prefLabel xml:lang="de">Verpackungsrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12912-4">
    <skos:related rdf:resource="descriptor/13117-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12912-4">
    <skos:related rdf:resource="descriptor/14788-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12912-4">
    <skos:related rdf:resource="descriptor/19367-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12912-4">
    <skos:related rdf:resource="descriptor/13894-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12912-4">
    <skos:related rdf:resource="descriptor/14150-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12912-4">
    <skos:related rdf:resource="descriptor/12961-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19292-1">
    <skos:prefLabel xml:lang="en">Feed market</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19292-1">
    <skos:prefLabel xml:lang="de">Futtermittelmarkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11426-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11821-5">
    <skos:narrower rdf:resource="descriptor/18480-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11821-5">
    <skos:narrower rdf:resource="descriptor/18479-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11821-5">
    <skos:narrower rdf:resource="descriptor/12917-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11821-5">
    <skos:narrower rdf:resource="descriptor/12908-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19756-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19263-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10337-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19432-4">
    <skos:altLabel xml:lang="de">Versunkene Kosten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13364-4">
    <skos:related rdf:resource="descriptor/13352-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11772-6">
    <skos:prefLabel xml:lang="en">Profit sharing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11772-6">
    <skos:prefLabel xml:lang="de">Erfolgsbeteiligung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13747-2">
    <skos:altLabel xml:lang="de">Kontogebühr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14685-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#inScheme">
    <rdfs:range rdf:resource="http://www.w3.org/2004/02/skos/core#ConceptScheme"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16233-4">
    <skos:prefLabel xml:lang="en">Theory of the state</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16233-4">
    <skos:prefLabel xml:lang="de">Staatstheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13465-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19064-0">
    <skos:related rdf:resource="descriptor/18619-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19064-0">
    <skos:related rdf:resource="descriptor/12681-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19064-0">
    <skos:related rdf:resource="descriptor/12675-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19064-0">
    <skos:related rdf:resource="descriptor/13620-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://zbw.eu/namespaces/skos-extensions/indexedItem">
    <rdfs:label xml:lang="en">indexes</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12451-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12722-2">
    <skos:prefLabel xml:lang="en">Small-lot production</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12722-2">
    <skos:prefLabel xml:lang="de">Kleinserienfertigung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19136-1">
    <skos:altLabel xml:lang="de">POS-Werbung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19136-1">
    <skos:altLabel xml:lang="de">POP-Werbung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19136-1">
    <skos:altLabel xml:lang="de">Point-of-Sale-Werbung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19136-1">
    <skos:altLabel xml:lang="de">Point-of-Purchase-Werbung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14789-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11545-3">
    <skos:broader rdf:resource="thsys/71081"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11545-3">
    <skos:broader rdf:resource="descriptor/11540-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11545-3">
    <skos:broader rdf:resource="descriptor/12311-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18096-6">
    <skos:narrower rdf:resource="descriptor/12945-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18096-6">
    <skos:narrower rdf:resource="descriptor/12961-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13160-6">
    <skos:prefLabel xml:lang="en">Food industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13160-6">
    <skos:prefLabel xml:lang="de">Nahrungsmittelgewerbe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13658-1">
    <skos:prefLabel xml:lang="en">Communication media</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13658-1">
    <skos:prefLabel xml:lang="de">Kommunikationsmedien</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10512-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11994-2">
    <skos:related rdf:resource="descriptor/18753-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11994-2">
    <skos:related rdf:resource="descriptor/19456-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11994-2">
    <skos:related rdf:resource="descriptor/11844-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11994-2">
    <skos:related rdf:resource="descriptor/11969-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11994-2">
    <skos:related rdf:resource="descriptor/18745-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20104-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19764-0">
    <skos:prefLabel xml:lang="en">Sexual harassment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19764-0">
    <skos:prefLabel xml:lang="de">Sexuelle Belästigung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24708-1">
    <skos:altLabel xml:lang="de">IT-Branche</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24708-1">
    <skos:altLabel xml:lang="de">Informationstechnische Industrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24708-1">
    <skos:altLabel xml:lang="de">Informationstechnische Branche</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19498-2">
    <skos:prefLabel xml:lang="en">Women's unemployment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19498-2">
    <skos:prefLabel xml:lang="de">Frauenarbeitslosigkeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10492-3">
    <skos:narrower rdf:resource="descriptor/10506-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10492-3">
    <skos:narrower rdf:resource="descriptor/19180-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10492-3">
    <skos:narrower rdf:resource="descriptor/19179-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10492-3">
    <skos:narrower rdf:resource="descriptor/19176-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10492-3">
    <skos:narrower rdf:resource="descriptor/19174-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10492-3">
    <skos:narrower rdf:resource="descriptor/19173-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10492-3">
    <skos:narrower rdf:resource="descriptor/19170-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10492-3">
    <skos:narrower rdf:resource="descriptor/17977-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15860-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18915-3">
    <skos:prefLabel xml:lang="en">Do-it-yourself</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18915-3">
    <skos:prefLabel xml:lang="de">Heimwerken</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16268-6">
    <skos:prefLabel xml:lang="en">Democratization</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16268-6">
    <skos:prefLabel xml:lang="de">Demokratisierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14954-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10353-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12928-3">
    <skos:prefLabel xml:lang="en">Environmental goods and services industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12928-3">
    <skos:prefLabel xml:lang="de">Umweltschutzsektor</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15009-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12393-2">
    <skos:broader rdf:resource="thsys/70005"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12393-2">
    <skos:broader rdf:resource="descriptor/12357-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10359-5">
    <skos:broader rdf:resource="thsys/70406"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10359-5">
    <skos:broader rdf:resource="descriptor/10434-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18590-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19286-3">
    <skos:broader rdf:resource="thsys/70171"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19286-3">
    <skos:broader rdf:resource="descriptor/11068-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12572-2">
    <skos:broader rdf:resource="thsys/70244"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17578-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14065-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10527-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16274-4">
    <skos:prefLabel xml:lang="en">Policy advisory service</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16274-4">
    <skos:prefLabel xml:lang="de">Politikberatung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11037-1">
    <skos:altLabel xml:lang="en">Needs-based approach</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11037-1">
    <skos:altLabel xml:lang="de">Bedürfnisprinzip</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11037-1">
    <skos:altLabel xml:lang="de">Bedürfnisorientierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11037-1">
    <skos:altLabel xml:lang="de">Bedarfsorientierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70581">
    <skos:prefLabel xml:lang="de">N.08.03  Kultur und Kunst</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#mappingRelation">
    <rdf:type rdf:resource="http://www.w3.org/1999/02/22-rdf-syntax-ns#Property"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#mappingRelation">
    <rdf:type rdf:resource="http://www.w3.org/2002/07/owl#ObjectProperty"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20136-5">
    <skos:altLabel xml:lang="en">Universal service obligation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20136-5">
    <skos:altLabel xml:lang="de">Universaldienste</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16656-4">
    <skos:broader rdf:resource="thsys/70321"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16661-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73350">
    <rdfs:label xml:lang="en">Physics</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73350">
    <rdfs:label xml:lang="de">Physik</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18000-3">
    <skos:scopeNote xml:lang="de">Vorwiegend für technische und rechtliche Fragen von Datenübertragungsnetzen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id158">
    <rdfs:seeAlso rdf:resource="descriptor/10970-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15656-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11377-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17948-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13553-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70274">
    <skos:prefLabel xml:lang="de">V.01.04  Neue Politische Ökonomie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73358">
    <skos:broader rdf:resource="thsys/70120"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18138-2">
    <skos:prefLabel xml:lang="en">Announcement effect</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18138-2">
    <skos:prefLabel xml:lang="de">Ankündigungseffekt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18594-4">
    <skos:prefLabel xml:lang="en">Marine accident</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18594-4">
    <skos:prefLabel xml:lang="de">Schiffsunfall</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19259-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13191-2">
    <skos:prefLabel xml:lang="en">Waste paper industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13191-2">
    <skos:prefLabel xml:lang="de">Altpapierwirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17927-1">
    <skos:broader rdf:resource="thsys/71030"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17927-1">
    <skos:broader rdf:resource="descriptor/17931-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17927-1">
    <skos:broader rdf:resource="descriptor/19179-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16286-4">
    <skos:narrower rdf:resource="descriptor/16267-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11894-6">
    <skos:narrower rdf:resource="descriptor/19477-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18538-0">
    <skos:altLabel xml:lang="de">Direktexport</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18215-3">
    <skos:prefLabel xml:lang="en">Sweetener</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18215-3">
    <skos:prefLabel xml:lang="de">Süßstoff</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18833-5">
    <skos:broader rdf:resource="thsys/70859"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18833-5">
    <skos:broader rdf:resource="thsys/70856"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18833-5">
    <skos:broader rdf:resource="descriptor/12884-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14960-4">
    <skos:broader rdf:resource="thsys/70498"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14960-4">
    <skos:broader rdf:resource="descriptor/14958-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14960-4">
    <skos:broader rdf:resource="descriptor/14961-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18157-5">
    <skos:altLabel xml:lang="en">Debt-for-nature-swap</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18157-5">
    <skos:altLabel xml:lang="en">Debt-for-nature swap</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18157-5">
    <skos:altLabel xml:lang="en">Debt-for-export swap</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18157-5">
    <skos:altLabel xml:lang="en">Debt-for-equity-swap</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18157-5">
    <skos:altLabel xml:lang="en">Debt-for-equity swap</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18157-5">
    <skos:altLabel xml:lang="en">Debt equity swap</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18157-5">
    <skos:altLabel xml:lang="en">Debt commodity swap</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19723-0">
    <skos:related rdf:resource="descriptor/10598-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19723-0">
    <skos:related rdf:resource="descriptor/18447-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19723-0">
    <skos:related rdf:resource="descriptor/18818-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19723-0">
    <skos:related rdf:resource="descriptor/16437-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10629-2">
    <skos:narrower rdf:resource="descriptor/10650-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10629-2">
    <skos:narrower rdf:resource="descriptor/10649-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10629-2">
    <skos:narrower rdf:resource="descriptor/19312-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10629-2">
    <skos:narrower rdf:resource="descriptor/10676-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10629-2">
    <skos:narrower rdf:resource="descriptor/10641-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10629-2">
    <skos:narrower rdf:resource="descriptor/10637-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10629-2">
    <skos:narrower rdf:resource="descriptor/10638-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10629-2">
    <skos:narrower rdf:resource="descriptor/19311-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10629-2">
    <skos:narrower rdf:resource="descriptor/10635-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18700-3">
    <skos:altLabel xml:lang="de">Bilanzierung von Valutaforderungen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18700-3">
    <skos:altLabel xml:lang="de">Bilanzierung von Fremdwährungstransaktionen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#hasTopConcept">
    <rdfs:domain rdf:resource="http://www.w3.org/2004/02/skos/core#ConceptScheme"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14600-6">
    <skos:related rdf:resource="descriptor/14632-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18511-6">
    <skos:broader rdf:resource="thsys/70965"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18511-6">
    <skos:broader rdf:resource="descriptor/12036-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14070-2">
    <skos:narrower rdf:resource="descriptor/14081-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14070-2">
    <skos:narrower rdf:resource="descriptor/14078-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14070-2">
    <skos:narrower rdf:resource="descriptor/14066-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14070-2">
    <skos:narrower rdf:resource="descriptor/14056-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19621-1">
    <skos:related rdf:resource="descriptor/16808-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11534-1">
    <skos:broader rdf:resource="thsys/72181"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11534-1">
    <skos:broader rdf:resource="thsys/70869"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11534-1">
    <skos:broader rdf:resource="thsys/70144"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11534-1">
    <skos:broader rdf:resource="descriptor/19072-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18125-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14070-2">
    <skos:altLabel xml:lang="de">Sonnenblumenkerne</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14070-2">
    <skos:altLabel xml:lang="de">Sesam</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14070-2">
    <skos:altLabel xml:lang="de">Rizinus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14070-2">
    <skos:altLabel xml:lang="de">Ölsaat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14070-2">
    <skos:altLabel xml:lang="de">Ölpalme</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14070-2">
    <skos:altLabel xml:lang="de">Olive</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14070-2">
    <skos:altLabel xml:lang="de">Ölfrucht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14070-2">
    <skos:altLabel xml:lang="de">Leinsamen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14070-2">
    <skos:altLabel xml:lang="de">Baumwollsamen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13234-3">
    <skos:prefLabel xml:lang="en">Wine trade</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13234-3">
    <skos:prefLabel xml:lang="de">Weinhandel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13658-1">
    <skos:broader rdf:resource="thsys/70825"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id46">
    <rdfs:seeAlso rdf:resource="descriptor/11365-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18718-5">
    <skos:broader rdf:resource="thsys/70900"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18718-5">
    <skos:broader rdf:resource="descriptor/18101-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12004-4">
    <skos:broader rdf:resource="thsys/73368"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12004-4">
    <skos:broader rdf:resource="thsys/71063"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12004-4">
    <skos:broader rdf:resource="descriptor/10981-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13327-3">
    <skos:broader rdf:resource="thsys/70817"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13327-3">
    <skos:broader rdf:resource="descriptor/18071-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13327-3">
    <skos:broader rdf:resource="descriptor/15731-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73306">
    <rdfs:label xml:lang="de">Glas und Glaswaren</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12957-3">
    <skos:broader rdf:resource="thsys/71029"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12957-3">
    <skos:broader rdf:resource="descriptor/12939-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14914-4">
    <skos:narrower rdf:resource="descriptor/14921-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11252-4">
    <skos:related rdf:resource="descriptor/11295-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11252-4">
    <skos:related rdf:resource="descriptor/19505-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11252-4">
    <skos:related rdf:resource="descriptor/11290-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11252-4">
    <skos:related rdf:resource="descriptor/11236-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19062-4">
    <skos:related rdf:resource="descriptor/19186-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19062-4">
    <skos:related rdf:resource="descriptor/19057-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19062-4">
    <skos:related rdf:resource="descriptor/14381-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19125-6">
    <skos:altLabel xml:lang="de">Marktherausforderer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19125-6">
    <skos:altLabel xml:lang="de">Marktführerschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13446-2">
    <skos:prefLabel xml:lang="en">Agricultural sciences</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13446-2">
    <skos:prefLabel xml:lang="de">Agrarwissenschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18583-2">
    <skos:prefLabel xml:lang="en">Road pricing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18583-2">
    <skos:prefLabel xml:lang="de">Straßenbenutzungsgebühr</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16973-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11671-5">
    <skos:broader rdf:resource="thsys/71074"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11671-5">
    <skos:broader rdf:resource="descriptor/19422-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18139-0">
    <skos:altLabel xml:lang="de">Systemsoftware</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18139-0">
    <skos:altLabel xml:lang="de">Maschinensprache</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18139-0">
    <skos:altLabel xml:lang="de">Compiler</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13492-2">
    <skos:related rdf:resource="descriptor/13479-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16013-4">
    <skos:related rdf:resource="descriptor/16332-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16013-4">
    <skos:related rdf:resource="descriptor/16017-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16013-4">
    <skos:related rdf:resource="descriptor/19684-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16013-4">
    <skos:related rdf:resource="descriptor/19379-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16013-4">
    <skos:related rdf:resource="descriptor/13420-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16013-4">
    <skos:related rdf:resource="descriptor/16309-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16013-4">
    <skos:related rdf:resource="descriptor/16328-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16013-4">
    <skos:related rdf:resource="descriptor/11164-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16013-4">
    <skos:related rdf:resource="descriptor/16258-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16013-4">
    <skos:related rdf:resource="descriptor/16621-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16013-4">
    <skos:related rdf:resource="descriptor/19753-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11157-5">
    <skos:related rdf:resource="descriptor/13292-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11157-5">
    <skos:related rdf:resource="descriptor/13371-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11157-5">
    <skos:related rdf:resource="descriptor/16479-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11157-5">
    <skos:related rdf:resource="descriptor/11555-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11157-5">
    <skos:related rdf:resource="descriptor/19517-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11157-5">
    <skos:related rdf:resource="descriptor/16635-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14961-2">
    <skos:narrower rdf:resource="descriptor/18077-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14961-2">
    <skos:narrower rdf:resource="descriptor/14960-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11377-5">
    <skos:related rdf:resource="descriptor/18870-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11377-5">
    <skos:related rdf:resource="descriptor/11404-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11377-5">
    <skos:related rdf:resource="descriptor/19559-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11377-5">
    <skos:related rdf:resource="descriptor/11375-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73352">
    <skos:broader rdf:resource="thsys/73354"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73302">
    <rdfs:label xml:lang="de">Fleisch und Fleischwaren</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13761-1">
    <skos:related rdf:resource="descriptor/13780-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13198-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11399-2">
    <skos:related rdf:resource="descriptor/11391-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11399-2">
    <skos:related rdf:resource="descriptor/12589-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11399-2">
    <skos:related rdf:resource="descriptor/19656-3"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id32">
    <rdfs:seeAlso rdf:resource="descriptor/10882-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12610-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10682-5">
    <skos:narrower rdf:resource="descriptor/10721-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10682-5">
    <skos:narrower rdf:resource="descriptor/19310-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10682-5">
    <skos:narrower rdf:resource="descriptor/10725-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10682-5">
    <skos:narrower rdf:resource="descriptor/10696-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10682-5">
    <skos:narrower rdf:resource="descriptor/10686-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19027-6">
    <skos:scopeNote xml:lang="de">Nicht im Sinne gezielter politischer Anpassungsmaßnahmen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18412-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25743-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18242-0">
    <skos:prefLabel xml:lang="en">Container ship</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18242-0">
    <skos:prefLabel xml:lang="de">Containerschiff</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12914-0">
    <skos:broader rdf:resource="thsys/71002"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15069-3">
    <skos:prefLabel xml:lang="en">Cost of living</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15069-3">
    <skos:prefLabel xml:lang="de">Lebenshaltungskosten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19835-3">
    <skos:prefLabel xml:lang="en">Investor relations</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19835-3">
    <skos:prefLabel xml:lang="de">Investor Relations</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11402-4">
    <skos:narrower rdf:resource="descriptor/18877-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12188-3">
    <skos:altLabel xml:lang="de">KGaA</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11345-4">
    <skos:prefLabel xml:lang="en">Wage policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11345-4">
    <skos:prefLabel xml:lang="de">Lohnpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18753-3">
    <skos:broader rdf:resource="thsys/71064"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18753-3">
    <skos:broader rdf:resource="thsys/70967"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18753-3">
    <skos:broader rdf:resource="thsys/70888"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18753-3">
    <skos:broader rdf:resource="descriptor/12932-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18294-2">
    <skos:broader rdf:resource="thsys/70258"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18294-2">
    <skos:broader rdf:resource="descriptor/14428-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13348-2">
    <skos:altLabel xml:lang="de">Psychologische Psychotherapeutinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13348-2">
    <skos:altLabel xml:lang="de">Psychologische Psychotherapeuten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13348-2">
    <skos:altLabel xml:lang="de">Psychologinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13348-2">
    <skos:altLabel xml:lang="de">Klinische Psychologinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13348-2">
    <skos:altLabel xml:lang="de">Klinische Psychologen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12975-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11547-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10478-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19110-5">
    <skos:related rdf:resource="descriptor/16136-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19110-5">
    <skos:related rdf:resource="descriptor/11311-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10833-3">
    <skos:narrower rdf:resource="descriptor/10838-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10833-3">
    <skos:narrower rdf:resource="descriptor/10836-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10812-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15647-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14975-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19755-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11786-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16411-6">
    <skos:narrower rdf:resource="descriptor/10986-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70823">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19678-0">
    <skos:broader rdf:resource="thsys/71063"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19347-2">
    <skos:narrower rdf:resource="descriptor/19345-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19347-2">
    <skos:narrower rdf:resource="descriptor/19343-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18240-4">
    <zbwext:useInsteadNote rdf:nodeID="id333"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70209">
    <rdfs:label xml:lang="en">Environmental economics</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70209">
    <rdfs:label xml:lang="de">Umweltökonomik</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16967-1">
    <skos:prefLabel xml:lang="en">Neuchatel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16967-1">
    <skos:prefLabel xml:lang="de">Neuenburg</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19279-0">
    <skos:prefLabel xml:lang="en">Small shareholders</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19279-0">
    <skos:prefLabel xml:lang="de">Kleinaktionäre</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11177-6">
    <skos:broader rdf:resource="thsys/70274"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11177-6">
    <skos:broader rdf:resource="descriptor/11168-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71110">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11670-0">
    <skos:scopeNote xml:lang="de">Bemessungsgrundlage ist die gesamte Lohn- und Gehaltssumme.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70307">
    <skos:narrower rdf:resource="descriptor/19000-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70307">
    <skos:narrower rdf:resource="descriptor/17999-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70307">
    <skos:narrower rdf:resource="descriptor/19085-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70307">
    <skos:narrower rdf:resource="descriptor/10506-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70307">
    <skos:narrower rdf:resource="descriptor/15895-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70307">
    <skos:narrower rdf:resource="descriptor/19837-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70307">
    <skos:narrower rdf:resource="descriptor/11123-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70307">
    <skos:narrower rdf:resource="descriptor/10508-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70307">
    <skos:narrower rdf:resource="descriptor/17987-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70307">
    <skos:narrower rdf:resource="descriptor/19180-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70307">
    <skos:narrower rdf:resource="descriptor/17994-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70307">
    <skos:narrower rdf:resource="descriptor/18019-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70307">
    <skos:narrower rdf:resource="descriptor/17985-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70307">
    <skos:narrower rdf:resource="descriptor/17993-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70307">
    <skos:narrower rdf:resource="descriptor/17992-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70307">
    <skos:narrower rdf:resource="descriptor/17991-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70307">
    <skos:narrower rdf:resource="descriptor/19130-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70307">
    <skos:narrower rdf:resource="descriptor/18054-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70307">
    <skos:narrower rdf:resource="descriptor/19179-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70307">
    <skos:narrower rdf:resource="descriptor/19550-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70307">
    <skos:narrower rdf:resource="descriptor/17990-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70307">
    <skos:narrower rdf:resource="descriptor/17989-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70307">
    <skos:narrower rdf:resource="descriptor/17988-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70307">
    <skos:narrower rdf:resource="descriptor/10499-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70307">
    <skos:narrower rdf:resource="descriptor/19178-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70307">
    <skos:narrower rdf:resource="descriptor/19177-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70307">
    <skos:narrower rdf:resource="descriptor/19078-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70307">
    <skos:narrower rdf:resource="descriptor/17986-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70307">
    <skos:narrower rdf:resource="descriptor/17983-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70307">
    <skos:narrower rdf:resource="descriptor/10492-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70307">
    <skos:narrower rdf:resource="descriptor/17982-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70307">
    <skos:narrower rdf:resource="descriptor/19176-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70307">
    <skos:narrower rdf:resource="descriptor/19175-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70307">
    <skos:narrower rdf:resource="descriptor/19174-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70307">
    <skos:narrower rdf:resource="descriptor/19173-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70307">
    <skos:narrower rdf:resource="descriptor/17980-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70307">
    <skos:narrower rdf:resource="descriptor/19172-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70307">
    <skos:narrower rdf:resource="descriptor/17979-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70307">
    <skos:narrower rdf:resource="descriptor/17978-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70307">
    <skos:narrower rdf:resource="descriptor/17616-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70307">
    <skos:narrower rdf:resource="descriptor/19171-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70307">
    <skos:narrower rdf:resource="descriptor/17984-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70307">
    <skos:narrower rdf:resource="descriptor/19170-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70307">
    <skos:narrower rdf:resource="descriptor/17977-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15735-6">
    <skos:broader rdf:resource="thsys/73351"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15735-6">
    <skos:broader rdf:resource="thsys/70238"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15735-6">
    <skos:broader rdf:resource="descriptor/15763-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19860-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14345-3">
    <skos:prefLabel xml:lang="en">Antifriction bearings</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14345-3">
    <skos:prefLabel xml:lang="de">Wälzlager</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11963-6">
    <skos:altLabel xml:lang="de">Shift-Analyse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id226">
    <rdfs:seeAlso rdf:resource="descriptor/18428-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70028">
    <skos:prefLabel xml:lang="de">B.01  Unternehmensführung und Organisation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16104-1">
    <skos:broader rdf:resource="thsys/73337"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16104-1">
    <skos:broader rdf:resource="thsys/71073"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16104-1">
    <skos:broader rdf:resource="thsys/70397"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16104-1">
    <skos:broader rdf:resource="descriptor/11640-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15664-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19513-4">
    <skos:broader rdf:resource="thsys/70832"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19513-4">
    <skos:broader rdf:resource="thsys/70458"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19513-4">
    <skos:broader rdf:resource="descriptor/13823-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19513-4">
    <skos:broader rdf:resource="descriptor/16539-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13873-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12974-3">
    <skos:altLabel xml:lang="de">Stromwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12974-3">
    <skos:altLabel xml:lang="de">Stromverteilung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12974-3">
    <skos:altLabel xml:lang="de">Stromerzeugung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12974-3">
    <skos:altLabel xml:lang="de">EVU (Elektrizitätsversorgungsunternehmen)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12974-3">
    <skos:altLabel xml:lang="de">Elektrizitätsversorgung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13688-6">
    <skos:scopeNote xml:lang="de">Nur für größere Krisen des gesamten Bankensektors.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19529-3">
    <skos:scopeNote xml:lang="de">Wissenschaft von den ionisierenden Strahlen und ihrer Anwendung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18282-2">
    <skos:prefLabel xml:lang="en">Baby care product</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18282-2">
    <skos:prefLabel xml:lang="de">Babypflegemittel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19850-0">
    <skos:prefLabel xml:lang="en">Business register</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19850-0">
    <skos:prefLabel xml:lang="de">Handelsregister</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15797-5">
    <skos:altLabel xml:lang="de">Völkerpsychologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15797-5">
    <skos:altLabel xml:lang="de">Patriotismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15797-5">
    <skos:altLabel xml:lang="de">Nationalgefühl</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15797-5">
    <skos:altLabel xml:lang="de">Nationale Mentalität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15797-5">
    <skos:altLabel xml:lang="de">Nationale Identität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15797-5">
    <skos:altLabel xml:lang="de">Nationalcharakter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15797-5">
    <skos:altLabel xml:lang="de">Nationalbewußtsein</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15797-5">
    <skos:altLabel xml:lang="de">Nationalbewusstsein</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10382-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11774-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10510-6">
    <skos:broader rdf:resource="thsys/71064"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10510-6">
    <skos:broader rdf:resource="thsys/70091"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10510-6">
    <skos:broader rdf:resource="descriptor/19467-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10052-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19412-3">
    <skos:scopeNote xml:lang="de">Persönliche allgemeine Ausgabensteuer.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11210-6">
    <skos:prefLabel xml:lang="en">Structural unemployment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11210-6">
    <skos:prefLabel xml:lang="de">Strukturelle Arbeitslosigkeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12972-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15009-0">
    <skos:broader rdf:resource="thsys/71033"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15009-0">
    <skos:broader rdf:resource="thsys/70238"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15009-0">
    <skos:broader rdf:resource="thsys/70161"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16962-4">
    <skos:broader rdf:resource="thsys/70003"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16962-4">
    <skos:broader rdf:resource="descriptor/16972-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15477-2">
    <skos:prefLabel xml:lang="en">Risk aversion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15477-2">
    <skos:prefLabel xml:lang="de">Risikoaversion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18431-4">
    <skos:prefLabel xml:lang="en">Pesticide industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18431-4">
    <skos:prefLabel xml:lang="de">Pestizidindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17500-2">
    <skos:altLabel xml:lang="de">Republik Tadschikistan</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13464-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10958-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16068-0">
    <skos:related rdf:resource="descriptor/13804-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12908-2">
    <skos:broader rdf:resource="thsys/71005"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12908-2">
    <skos:broader rdf:resource="descriptor/11821-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12908-2">
    <skos:broader rdf:resource="descriptor/12049-3"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id270">
    <rdf:value xml:lang="de">Für Röntgengeräte</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18064-5">
    <skos:altLabel xml:lang="en">Economic sanction</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18064-5">
    <skos:altLabel xml:lang="de">Wirtschaftssanktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18064-5">
    <skos:altLabel xml:lang="de">Politische Sanktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18064-5">
    <skos:altLabel xml:lang="de">Militärische Sanktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18064-5">
    <skos:altLabel xml:lang="de">Handelssanktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18064-5">
    <skos:altLabel xml:lang="de">Außenwirtschaftssanktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13703-1">
    <skos:broader rdf:resource="thsys/70902"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13703-1">
    <skos:broader rdf:resource="descriptor/13291-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11324-5">
    <skos:altLabel xml:lang="en">Wage gap (Wage drift)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11324-5">
    <skos:altLabel xml:lang="en">Real wage gap</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11324-5">
    <skos:altLabel xml:lang="de">Lohn-Gap</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11324-5">
    <skos:altLabel xml:lang="de">Effektivlohn</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18017-0">
    <skos:altLabel xml:lang="en">Info-centre software</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18017-0">
    <skos:altLabel xml:lang="en">Development-centre software</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18017-0">
    <skos:altLabel xml:lang="de">Datenverwaltung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18017-0">
    <skos:altLabel xml:lang="de">Datenmanagement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18017-0">
    <skos:altLabel xml:lang="de">Datenadministration</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id163">
    <rdf:value xml:lang="de">Für Steuern auf Capital Gains</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11705-0">
    <skos:altLabel xml:lang="de">Progressivsteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11705-0">
    <skos:altLabel xml:lang="de">Progressive Steuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11705-0">
    <skos:altLabel xml:lang="de">Progressive Besteuerung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10872-0">
    <skos:scopeNote xml:lang="de">Handelsfirmen, die das Importgeschäft betreiben sowie der entsprechende Einfuhrhandel.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15686-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14502-6">
    <skos:narrower rdf:resource="descriptor/14512-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14502-6">
    <skos:narrower rdf:resource="descriptor/14511-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14502-6">
    <skos:narrower rdf:resource="descriptor/14509-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14502-6">
    <skos:narrower rdf:resource="descriptor/14507-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14502-6">
    <skos:narrower rdf:resource="descriptor/14504-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15251-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19669-1">
    <skos:related rdf:resource="descriptor/13742-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19669-1">
    <skos:related rdf:resource="descriptor/12230-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19669-1">
    <skos:related rdf:resource="descriptor/18679-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19669-1">
    <skos:related rdf:resource="descriptor/19417-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19669-1">
    <skos:related rdf:resource="descriptor/13715-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10437-4">
    <skos:prefLabel xml:lang="en">Science and technology policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10437-4">
    <skos:prefLabel xml:lang="de">Forschungs- und Technologiepolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10605-2">
    <skos:narrower rdf:resource="descriptor/19357-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10605-2">
    <skos:narrower rdf:resource="descriptor/10877-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10605-2">
    <skos:narrower rdf:resource="descriptor/18126-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10605-2">
    <skos:narrower rdf:resource="descriptor/10656-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10605-2">
    <skos:narrower rdf:resource="descriptor/10872-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10605-2">
    <skos:narrower rdf:resource="descriptor/18437-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10605-2">
    <skos:narrower rdf:resource="descriptor/10864-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10605-2">
    <skos:narrower rdf:resource="descriptor/18536-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10605-2">
    <skos:narrower rdf:resource="descriptor/10823-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10605-2">
    <skos:narrower rdf:resource="descriptor/10602-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10605-2">
    <skos:narrower rdf:resource="descriptor/10833-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10605-2">
    <skos:narrower rdf:resource="descriptor/18008-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19482-3">
    <skos:related rdf:resource="descriptor/18069-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19482-3">
    <skos:related rdf:resource="descriptor/19481-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19482-3">
    <skos:related rdf:resource="descriptor/18079-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17011-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14380-1">
    <skos:related rdf:resource="descriptor/13054-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14380-1">
    <skos:related rdf:resource="descriptor/13450-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18565-4">
    <skos:related rdf:resource="descriptor/13239-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18565-4">
    <skos:related rdf:resource="descriptor/18560-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10187-1">
    <skos:related rdf:resource="descriptor/10327-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10187-1">
    <skos:related rdf:resource="descriptor/12741-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15689-1">
    <skos:altLabel xml:lang="de">Antike</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13762-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19611-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13298-5">
    <skos:altLabel xml:lang="de">Seewarenversicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13298-5">
    <skos:altLabel xml:lang="de">Seeversicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13298-5">
    <skos:altLabel xml:lang="de">Seetransportversicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13298-5">
    <skos:altLabel xml:lang="de">Seekasko</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13298-5">
    <skos:altLabel xml:lang="de">Schiffskaskoversicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13298-5">
    <skos:altLabel xml:lang="de">Flußtransportversicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13298-5">
    <skos:altLabel xml:lang="de">Flusstransportversicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13298-5">
    <skos:altLabel xml:lang="de">Flusskaskoversicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13298-5">
    <skos:altLabel xml:lang="de">Flußkaskoversicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13298-5">
    <skos:altLabel xml:lang="de">Binnenwarenversicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19389-0">
    <skos:prefLabel xml:lang="en">Paper market</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19389-0">
    <skos:prefLabel xml:lang="de">Papiermarkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14294-1">
    <skos:broader rdf:resource="thsys/70134"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14294-1">
    <skos:broader rdf:resource="descriptor/14274-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14294-1">
    <skos:broader rdf:resource="descriptor/14302-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71115">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18847-1">
    <skos:prefLabel xml:lang="en">Membership</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18847-1">
    <skos:prefLabel xml:lang="de">Mitgliedschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14158-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18733-2">
    <skos:scopeNote xml:lang="de">Ansatz zur Analyse der optimalen Ausgestaltung von Börsenmärkten.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20277-1">
    <skos:prefLabel xml:lang="en">Simultan planning</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20277-1">
    <skos:prefLabel xml:lang="de">Simultanplanung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18739-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11523-6">
    <skos:prefLabel xml:lang="en">Repressed inflation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11523-6">
    <skos:prefLabel xml:lang="de">Zurückgestaute Inflation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18843-2">
    <skos:prefLabel xml:lang="en">Producer cooperative</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18843-2">
    <skos:prefLabel xml:lang="de">Produktivgenossenschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16298-4">
    <skos:broader rdf:resource="thsys/73382"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16298-4">
    <skos:broader rdf:resource="descriptor/16305-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71104">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14821-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14418-2">
    <skos:altLabel xml:lang="de">Verkehrsflugzeug</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14418-2">
    <skos:altLabel xml:lang="de">Düsenflugzeug</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11970-2">
    <skos:prefLabel xml:lang="en">Town</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11970-2">
    <skos:prefLabel xml:lang="de">Stadt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15038-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11962-1">
    <skos:related rdf:resource="descriptor/15076-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11962-1">
    <skos:related rdf:resource="descriptor/11875-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11962-1">
    <skos:related rdf:resource="descriptor/11857-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11962-1">
    <skos:related rdf:resource="descriptor/19438-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11962-1">
    <skos:related rdf:resource="descriptor/11965-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16968-6">
    <skos:broader rdf:resource="thsys/70003"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16968-6">
    <skos:broader rdf:resource="descriptor/16972-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11165-6">
    <skos:broader rdf:resource="thsys/70274"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11165-6">
    <skos:broader rdf:resource="descriptor/11167-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19825-6">
    <skos:altLabel xml:lang="en">School-to-work transition</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19825-6">
    <skos:altLabel xml:lang="de">Berufseintritt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19825-6">
    <skos:altLabel xml:lang="de">Berufseinmündung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19825-6">
    <skos:altLabel xml:lang="de">Berufsbeginn</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19825-6">
    <skos:altLabel xml:lang="de">Berufsanfängerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19825-6">
    <skos:altLabel xml:lang="de">Berufsanfänger</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19825-6">
    <skos:altLabel xml:lang="de">Berufsanfang</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12030-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14831-1">
    <skos:broader rdf:resource="thsys/70313"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11544-5">
    <skos:prefLabel xml:lang="en">Local revenues</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11544-5">
    <skos:prefLabel xml:lang="de">Kommunale Einnahmen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14814-1">
    <skos:altLabel xml:lang="en">City map</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14814-1">
    <skos:altLabel xml:lang="de">Stadtplan</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14814-1">
    <skos:altLabel xml:lang="de">Stadtkarte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14814-1">
    <skos:altLabel xml:lang="de">Karte (Landkarte)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14814-1">
    <skos:altLabel xml:lang="de">Globus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19651-6">
    <skos:broader rdf:resource="thsys/73341"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19651-6">
    <skos:broader rdf:resource="thsys/71037"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19651-6">
    <skos:broader rdf:resource="thsys/70922"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19651-6">
    <skos:broader rdf:resource="thsys/70384"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14652-1">
    <skos:related rdf:resource="descriptor/14627-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11897-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15633-0">
    <skos:altLabel xml:lang="en">Life sciences</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15633-0">
    <skos:altLabel xml:lang="de">Biowissenschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19209-0">
    <skos:related rdf:resource="descriptor/10156-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19209-0">
    <skos:related rdf:resource="descriptor/10148-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19209-0">
    <skos:related rdf:resource="descriptor/10179-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19209-0">
    <skos:related rdf:resource="descriptor/16699-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12187-5">
    <skos:broader rdf:resource="thsys/70087"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12187-5">
    <skos:broader rdf:resource="descriptor/12192-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10668-6">
    <skos:related rdf:resource="descriptor/18188-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10668-6">
    <skos:related rdf:resource="descriptor/10217-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10668-6">
    <skos:related rdf:resource="descriptor/10213-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10668-6">
    <skos:related rdf:resource="descriptor/10659-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10668-6">
    <skos:related rdf:resource="descriptor/10667-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10668-6">
    <skos:related rdf:resource="descriptor/10638-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10668-6">
    <skos:related rdf:resource="descriptor/10834-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17709-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12036-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19080-2">
    <zbwext:useInsteadNote rdf:nodeID="id77"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18500-4">
    <skos:related rdf:resource="descriptor/14323-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18500-4">
    <skos:related rdf:resource="descriptor/13245-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18066-1">
    <skos:broader rdf:resource="thsys/73334"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18066-1">
    <skos:broader rdf:resource="thsys/70550"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18066-1">
    <skos:broader rdf:resource="descriptor/16201-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14514-6">
    <skos:related rdf:resource="descriptor/18940-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10889-4">
    <skos:broader rdf:resource="thsys/70242"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10889-4">
    <skos:broader rdf:resource="descriptor/10978-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10889-4">
    <skos:broader rdf:resource="descriptor/10213-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73367">
    <skos:narrower rdf:resource="descriptor/13424-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73367">
    <skos:narrower rdf:resource="descriptor/13421-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73367">
    <skos:narrower rdf:resource="descriptor/19291-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73367">
    <skos:narrower rdf:resource="descriptor/11844-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73367">
    <skos:narrower rdf:resource="descriptor/18808-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73367">
    <skos:narrower rdf:resource="descriptor/11900-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73367">
    <skos:narrower rdf:resource="descriptor/13414-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73367">
    <skos:narrower rdf:resource="descriptor/16551-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73367">
    <skos:narrower rdf:resource="descriptor/11592-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73367">
    <skos:narrower rdf:resource="descriptor/16550-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73367">
    <skos:narrower rdf:resource="descriptor/15995-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73367">
    <skos:narrower rdf:resource="descriptor/16549-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73367">
    <skos:narrower rdf:resource="descriptor/11591-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73367">
    <skos:narrower rdf:resource="descriptor/19446-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73367">
    <skos:narrower rdf:resource="descriptor/19378-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73367">
    <skos:narrower rdf:resource="descriptor/11590-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73367">
    <skos:narrower rdf:resource="descriptor/11852-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73367">
    <skos:narrower rdf:resource="descriptor/13409-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73367">
    <skos:narrower rdf:resource="descriptor/13639-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17743-4">
    <skos:narrower rdf:resource="descriptor/17744-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17743-4">
    <skos:narrower rdf:resource="descriptor/17740-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17743-4">
    <skos:narrower rdf:resource="descriptor/17739-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17743-4">
    <skos:narrower rdf:resource="descriptor/17735-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17743-4">
    <skos:narrower rdf:resource="descriptor/17742-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17743-4">
    <skos:narrower rdf:resource="descriptor/17732-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17743-4">
    <skos:narrower rdf:resource="descriptor/17726-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17743-4">
    <skos:narrower rdf:resource="descriptor/17727-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11872-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13301-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11163-3">
    <skos:related rdf:resource="descriptor/15458-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11163-3">
    <skos:related rdf:resource="descriptor/15811-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11163-3">
    <skos:related rdf:resource="descriptor/19715-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11163-3">
    <skos:related rdf:resource="descriptor/11337-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11163-3">
    <skos:related rdf:resource="descriptor/16204-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16697-4">
    <skos:broader rdf:resource="thsys/73357"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16697-4">
    <skos:broader rdf:resource="thsys/73347"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16697-4">
    <skos:broader rdf:resource="thsys/70878"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16697-4">
    <skos:broader rdf:resource="descriptor/10436-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13688-6">
    <skos:broader rdf:resource="thsys/70905"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13688-6">
    <skos:broader rdf:resource="descriptor/19664-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13688-6">
    <skos:broader rdf:resource="descriptor/11792-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12304-6">
    <skos:scopeNote xml:lang="de">Nur für Übernahme und Kauf von Unternehmen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71118">
    <rdfs:label xml:lang="en">Utility theory</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71118">
    <rdfs:label xml:lang="de">Nutzentheorie</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70076">
    <skos:prefLabel xml:lang="en">G.03  Africa</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70076">
    <skos:prefLabel xml:lang="de">G.03  Afrika</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16809-5">
    <skos:broader rdf:resource="thsys/70198"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12393-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10242-5">
    <skos:broader rdf:resource="thsys/70355"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10242-5">
    <skos:broader rdf:resource="descriptor/10236-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18977-2">
    <skos:narrower rdf:resource="descriptor/13969-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18977-2">
    <skos:narrower rdf:resource="descriptor/13970-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18977-2">
    <skos:narrower rdf:resource="descriptor/13965-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12452-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11717-0">
    <skos:broader rdf:resource="thsys/71083"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11717-0">
    <skos:broader rdf:resource="descriptor/10030-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19733-4">
    <skos:related rdf:resource="descriptor/16642-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19733-4">
    <skos:related rdf:resource="descriptor/15806-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19733-4">
    <skos:related rdf:resource="descriptor/12142-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19733-4">
    <skos:related rdf:resource="descriptor/11158-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19733-4">
    <skos:related rdf:resource="descriptor/18067-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19733-4">
    <skos:related rdf:resource="descriptor/20842-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19733-4">
    <skos:related rdf:resource="descriptor/16633-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19733-4">
    <skos:related rdf:resource="descriptor/25822-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73380">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15937-1">
    <skos:altLabel xml:lang="en">Wedlock</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15937-1">
    <skos:altLabel xml:lang="de">Mischehe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15937-1">
    <skos:altLabel xml:lang="de">Eheschließung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15821-6">
    <skos:narrower rdf:resource="descriptor/13661-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19101-6">
    <skos:broader rdf:resource="thsys/70578"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19101-6">
    <skos:broader rdf:resource="thsys/70270"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19101-6">
    <skos:broader rdf:resource="descriptor/10011-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11236-2">
    <skos:related rdf:resource="descriptor/12573-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11236-2">
    <skos:related rdf:resource="descriptor/13384-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11236-2">
    <skos:related rdf:resource="descriptor/13303-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11236-2">
    <skos:related rdf:resource="descriptor/11241-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11236-2">
    <skos:related rdf:resource="descriptor/11252-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11236-2">
    <skos:related rdf:resource="descriptor/11237-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11236-2">
    <skos:related rdf:resource="descriptor/18140-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13154-1">
    <skos:related rdf:resource="descriptor/13205-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13154-1">
    <skos:related rdf:resource="descriptor/12964-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13601-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16140-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19941-4">
    <skos:prefLabel xml:lang="en">EU state aid control</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19941-4">
    <skos:prefLabel xml:lang="de">EU-Beihilfenkontrolle</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10636-5">
    <skos:broader rdf:resource="thsys/71100"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10636-5">
    <skos:broader rdf:resource="descriptor/10635-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18437-6">
    <skos:broader rdf:resource="thsys/71091"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18437-6">
    <skos:broader rdf:resource="thsys/71090"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18437-6">
    <skos:broader rdf:resource="descriptor/10605-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19211-6">
    <zbwext:useInsteadNote rdf:nodeID="id148"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13981-1">
    <skos:narrower rdf:resource="descriptor/18947-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70039">
    <skos:broader rdf:resource="thsys/v"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19572-2">
    <skos:scopeNote xml:lang="de">"Weg des Betrunkenen".</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12935-6">
    <skos:related rdf:resource="descriptor/15897-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12935-6">
    <skos:related rdf:resource="descriptor/12961-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12935-6">
    <skos:related rdf:resource="descriptor/14101-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15026-0">
    <skos:narrower rdf:resource="descriptor/19583-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10576-4">
    <skos:broader rdf:resource="thsys/71084"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10576-4">
    <skos:broader rdf:resource="thsys/71028"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10576-4">
    <skos:broader rdf:resource="thsys/70875"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10576-4">
    <skos:broader rdf:resource="descriptor/12881-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10576-4">
    <skos:broader rdf:resource="descriptor/19046-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18308-3">
    <skos:related rdf:resource="descriptor/18309-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18308-3">
    <skos:related rdf:resource="descriptor/11270-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18308-3">
    <skos:related rdf:resource="descriptor/11298-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12497-4">
    <skos:broader rdf:resource="thsys/70578"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12497-4">
    <skos:broader rdf:resource="descriptor/12480-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18698-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19490-4">
    <skos:broader rdf:resource="thsys/71010"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19490-4">
    <skos:broader rdf:resource="thsys/70811"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16004-5">
    <skos:narrower rdf:resource="descriptor/18441-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12923-6">
    <skos:altLabel xml:lang="de">Reparaturhandwerk</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14198-4">
    <skos:altLabel xml:lang="de">Erzkonzentrat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14513-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15472-5">
    <skos:related rdf:resource="descriptor/15714-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15472-5">
    <skos:related rdf:resource="descriptor/16287-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15472-5">
    <skos:related rdf:resource="descriptor/16277-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15472-5">
    <skos:related rdf:resource="descriptor/16226-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15472-5">
    <skos:related rdf:resource="descriptor/11168-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15320-0">
    <skos:altLabel xml:lang="en">Statistical ranking</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15320-0">
    <skos:altLabel xml:lang="en">Rank correlation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15320-0">
    <skos:altLabel xml:lang="en">Order statistics</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15320-0">
    <skos:altLabel xml:lang="de">Rangtest</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15320-0">
    <skos:altLabel xml:lang="de">Rangstatistik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15320-0">
    <skos:altLabel xml:lang="de">Rangreihenverfahren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15320-0">
    <skos:altLabel xml:lang="de">Rangordnungsverfahren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15320-0">
    <skos:altLabel xml:lang="de">Rangmaßzahl</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15320-0">
    <skos:altLabel xml:lang="de">Rangkorrelation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15320-0">
    <skos:altLabel xml:lang="de">Rangfolgeverfahren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10505-6">
    <skos:altLabel xml:lang="de">Agrarentwicklungspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10505-6">
    <skos:altLabel xml:lang="de">Agrarentwicklung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18568-5">
    <skos:altLabel xml:lang="de">Stellwerkelektronik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18568-5">
    <skos:altLabel xml:lang="de">Eisenbahnelektronik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18568-5">
    <skos:altLabel xml:lang="de">Computergestützte Zugüberwachung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18568-5">
    <skos:altLabel xml:lang="de">Computergestützte Zugkontrolle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10647-0">
    <skos:broader rdf:resource="thsys/71100"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10647-0">
    <skos:broader rdf:resource="descriptor/19310-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10910-4">
    <skos:prefLabel xml:lang="en">Unfair competition</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10910-4">
    <skos:prefLabel xml:lang="de">Unlauterer Wettbewerb</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12022-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10909-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15733-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11181-1">
    <skos:broader rdf:resource="thsys/73394"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11181-1">
    <skos:broader rdf:resource="thsys/73385"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11181-1">
    <skos:broader rdf:resource="thsys/70052"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11181-1">
    <skos:broader rdf:resource="descriptor/16709-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19317-4">
    <skos:related rdf:resource="descriptor/19321-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19317-4">
    <skos:related rdf:resource="descriptor/10612-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19317-4">
    <skos:related rdf:resource="descriptor/10696-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19317-4">
    <skos:related rdf:resource="descriptor/10684-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13106-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18071-1">
    <skos:related rdf:resource="descriptor/15763-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18071-1">
    <skos:related rdf:resource="descriptor/12976-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18071-1">
    <skos:related rdf:resource="descriptor/12016-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18071-1">
    <skos:related rdf:resource="descriptor/13325-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19658-6">
    <skos:related rdf:resource="descriptor/18659-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19658-6">
    <skos:related rdf:resource="descriptor/13674-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11217-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70823">
    <rdfs:label xml:lang="en">Information industry</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70823">
    <rdfs:label xml:lang="de">Informationswirtschaft</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14037-0">
    <skos:broader rdf:resource="thsys/71015"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14037-0">
    <skos:broader rdf:resource="descriptor/14039-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19240-6">
    <skos:broader rdf:resource="thsys/70058"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19240-6">
    <skos:broader rdf:resource="thsys/70020"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19240-6">
    <skos:broader rdf:resource="descriptor/10338-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19729-2">
    <skos:related rdf:resource="descriptor/12183-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19729-2">
    <skos:related rdf:resource="descriptor/12182-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19729-2">
    <skos:related rdf:resource="descriptor/16050-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24705-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19124-1">
    <skos:related rdf:resource="descriptor/19134-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19124-1">
    <skos:related rdf:resource="descriptor/16034-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19124-1">
    <skos:related rdf:resource="descriptor/12246-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10278-5">
    <skos:broader rdf:resource="thsys/71112"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10278-5">
    <skos:broader rdf:resource="descriptor/10181-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12913-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13232-0">
    <skos:broader rdf:resource="thsys/70959"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13232-0">
    <skos:broader rdf:resource="descriptor/18522-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13232-0">
    <skos:broader rdf:resource="descriptor/13200-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10495-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18205-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18544-5">
    <skos:broader rdf:resource="thsys/71091"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18544-5">
    <skos:broader rdf:resource="thsys/70952"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18544-5">
    <skos:broader rdf:resource="thsys/70852"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18544-5">
    <skos:broader rdf:resource="descriptor/11822-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19436-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16070-6">
    <skos:altLabel xml:lang="en">Hague Rules</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16070-6">
    <skos:altLabel xml:lang="de">York-Antwerpen-Regel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16070-6">
    <skos:altLabel xml:lang="de">Seeprivatrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16070-6">
    <skos:altLabel xml:lang="de">Seefrachtrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16070-6">
    <skos:altLabel xml:lang="de">Havarierecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16070-6">
    <skos:altLabel xml:lang="de">Haager Regeln</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12366-5">
    <skos:scopeNote xml:lang="de">Nur für betriebswirtschaftliche Literatur.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71017">
    <rdfs:label xml:lang="de">Energie- und Wasserwirtschaft</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14218-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19589-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15676-3">
    <skos:related rdf:resource="descriptor/19779-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15676-3">
    <skos:related rdf:resource="descriptor/19778-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15676-3">
    <skos:related rdf:resource="descriptor/13277-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15676-3">
    <skos:related rdf:resource="descriptor/19707-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15676-3">
    <skos:related rdf:resource="descriptor/19776-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15676-3">
    <skos:related rdf:resource="descriptor/15657-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15676-3">
    <skos:related rdf:resource="descriptor/19608-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15676-3">
    <skos:related rdf:resource="descriptor/15798-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15676-3">
    <skos:related rdf:resource="descriptor/19601-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15676-3">
    <skos:related rdf:resource="descriptor/10486-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70953">
    <skos:broader rdf:resource="thsys/70963"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17476-1">
    <skos:broader rdf:resource="thsys/70035"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17476-1">
    <skos:broader rdf:resource="descriptor/19083-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17476-1">
    <skos:broader rdf:resource="descriptor/17983-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11792-0">
    <skos:broader rdf:resource="thsys/71110"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11792-0">
    <skos:broader rdf:resource="thsys/70575"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11792-0">
    <skos:broader rdf:resource="descriptor/10343-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11792-0">
    <skos:broader rdf:resource="descriptor/18010-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14554-1">
    <skos:related rdf:resource="descriptor/14502-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13516-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11701-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11400-1">
    <skos:scopeNote xml:lang="de">Grundsatz der Chancengleichheit auf Zugang zu höherwertiger Bildung und Ausbildung. Betrifft besonders Frauen und andere traditionell benachteiligte Gruppen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73387">
    <skos:narrower rdf:resource="descriptor/19636-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73387">
    <skos:narrower rdf:resource="descriptor/15706-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73387">
    <skos:narrower rdf:resource="descriptor/10772-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73387">
    <skos:narrower rdf:resource="descriptor/13511-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73387">
    <skos:narrower rdf:resource="descriptor/10465-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73387">
    <skos:narrower rdf:resource="descriptor/19405-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73387">
    <skos:narrower rdf:resource="descriptor/15723-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73387">
    <skos:narrower rdf:resource="descriptor/15698-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73387">
    <skos:narrower rdf:resource="descriptor/19255-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73387">
    <skos:narrower rdf:resource="descriptor/15704-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73387">
    <skos:narrower rdf:resource="descriptor/25501-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73387">
    <skos:narrower rdf:resource="descriptor/19288-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73387">
    <skos:narrower rdf:resource="descriptor/10574-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73387">
    <skos:narrower rdf:resource="descriptor/15703-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73387">
    <skos:narrower rdf:resource="descriptor/10500-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73387">
    <skos:narrower rdf:resource="descriptor/18537-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73387">
    <skos:narrower rdf:resource="descriptor/19520-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73387">
    <skos:narrower rdf:resource="descriptor/13462-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73387">
    <skos:narrower rdf:resource="descriptor/18838-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73387">
    <skos:narrower rdf:resource="descriptor/11416-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73387">
    <skos:narrower rdf:resource="descriptor/19261-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73387">
    <skos:narrower rdf:resource="descriptor/12046-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73387">
    <skos:narrower rdf:resource="descriptor/11533-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73387">
    <skos:narrower rdf:resource="descriptor/15705-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73387">
    <skos:narrower rdf:resource="descriptor/19519-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73387">
    <skos:narrower rdf:resource="descriptor/18729-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73387">
    <skos:narrower rdf:resource="descriptor/13691-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73387">
    <skos:narrower rdf:resource="descriptor/18534-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73387">
    <skos:narrower rdf:resource="descriptor/13444-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11530-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19754-3">
    <skos:prefLabel xml:lang="en">Life style</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19754-3">
    <skos:prefLabel xml:lang="de">Lifestyle</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10986-6">
    <skos:altLabel xml:lang="de">Neoliberalismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10986-6">
    <skos:altLabel xml:lang="de">Manchestertum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10986-6">
    <skos:altLabel xml:lang="de">Manchesterliberalismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10986-6">
    <skos:altLabel xml:lang="de">Laissez-Faire-Liberalismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70333">
    <rdfs:label xml:lang="de">Verpackungen</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11613-5">
    <skos:prefLabel xml:lang="en">Public Debt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11613-5">
    <skos:prefLabel xml:lang="de">Öffentliche Schulden</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13413-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10497-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10253-0">
    <skos:prefLabel xml:lang="en">Macro-econometric model</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10253-0">
    <skos:prefLabel xml:lang="de">Ökonometrisches Makromodell</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13361-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18025-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10659-0">
    <skos:related rdf:resource="descriptor/10675-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10659-0">
    <skos:related rdf:resource="descriptor/11519-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10659-0">
    <skos:related rdf:resource="descriptor/11432-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10659-0">
    <skos:related rdf:resource="descriptor/10668-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10659-0">
    <skos:related rdf:resource="descriptor/10667-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18018-5">
    <skos:altLabel xml:lang="de">Verbrauchergenossenschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18018-5">
    <skos:altLabel xml:lang="de">Konsumverein</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70941">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.12.01.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16982-5">
    <skos:broader rdf:resource="thsys/70505"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16982-5">
    <skos:broader rdf:resource="descriptor/16990-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16982-5">
    <skos:broader rdf:resource="descriptor/17983-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18387-2">
    <skos:prefLabel xml:lang="en">Mineral exploration</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18387-2">
    <skos:prefLabel xml:lang="de">Exploration</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18690-1">
    <skos:related rdf:resource="descriptor/10884-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18690-1">
    <skos:related rdf:resource="descriptor/18995-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18690-1">
    <skos:related rdf:resource="descriptor/13472-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18690-1">
    <skos:related rdf:resource="descriptor/13722-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16311-3">
    <skos:related rdf:resource="descriptor/16422-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16311-3">
    <skos:related rdf:resource="descriptor/16264-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16311-3">
    <skos:related rdf:resource="descriptor/16266-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13692-1">
    <skos:prefLabel xml:lang="en">Bank insolvency</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13692-1">
    <skos:prefLabel xml:lang="de">Bankinsolvenz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12668-3">
    <skos:related rdf:resource="descriptor/12665-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19346-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12574-5">
    <skos:narrower rdf:resource="descriptor/12569-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11723-5">
    <skos:related rdf:resource="descriptor/19210-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11723-5">
    <skos:related rdf:resource="descriptor/11718-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11723-5">
    <skos:related rdf:resource="descriptor/11168-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16676-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15776-6">
    <skos:broader rdf:resource="thsys/70209"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15776-6">
    <skos:broader rdf:resource="descriptor/15767-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13851-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12270-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18353-5">
    <skos:scopeNote xml:lang="de">Alle Verfahren, Vorrichtungen und Anlagen zur optimalen Umwandlung von Primärenergie in Sekundärenergie und deren Bereitstellung als Nutzenergie.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12855-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11504-3">
    <skos:broader rdf:resource="thsys/70355"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13848-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16480-1">
    <skos:prefLabel xml:lang="en">Youth policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16480-1">
    <skos:prefLabel xml:lang="de">Jugendpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10252-2">
    <skos:broader rdf:resource="thsys/70141"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10252-2">
    <skos:broader rdf:resource="descriptor/16745-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15119-0">
    <skos:related rdf:resource="descriptor/15243-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15717-1">
    <skos:altLabel xml:lang="en">Social stratum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15717-1">
    <skos:altLabel xml:lang="en">Social background</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15717-1">
    <skos:altLabel xml:lang="en">Caste</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15717-1">
    <skos:altLabel xml:lang="de">Soziale Schichtung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15717-1">
    <skos:altLabel xml:lang="de">Soziale Klasse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15717-1">
    <skos:altLabel xml:lang="de">Soziale Herkunft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15717-1">
    <skos:altLabel xml:lang="de">Schichtzugehörigkeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15717-1">
    <skos:altLabel xml:lang="de">Klasse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15717-1">
    <skos:altLabel xml:lang="de">Kaste</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15717-1">
    <skos:altLabel xml:lang="de">Gesellschaftsklasse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15717-1">
    <skos:altLabel xml:lang="de">Bevölkerungsschicht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19310-4">
    <skos:altLabel xml:lang="de">Handelsprotektionismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19310-4">
    <skos:altLabel xml:lang="de">Außenhandelsprotektionismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70011">
    <rdfs:label xml:lang="de">Sozialwissenschaftliche Theorien, Forschungsansätze und Methoden</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18108-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14281-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12040-0">
    <skos:prefLabel xml:lang="en">Evaluation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12040-0">
    <skos:prefLabel xml:lang="de">Bewertung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14135-0">
    <skos:related rdf:resource="descriptor/12941-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14135-0">
    <skos:related rdf:resource="descriptor/18218-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14135-0">
    <skos:related rdf:resource="descriptor/14121-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19305-4">
    <skos:prefLabel xml:lang="en">Wheat price</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19305-4">
    <skos:prefLabel xml:lang="de">Weizenpreis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19229-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12058-2">
    <skos:prefLabel xml:lang="en">Liquidation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12058-2">
    <skos:prefLabel xml:lang="de">Liquidation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19597-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10381-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13346-6">
    <skos:altLabel xml:lang="en">Information agent</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13346-6">
    <skos:altLabel xml:lang="de">Informationsvermittlungsstelle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13346-6">
    <skos:altLabel xml:lang="de">Informationsvermittlerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13346-6">
    <skos:altLabel xml:lang="de">Informationsvermittler</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13346-6">
    <skos:altLabel xml:lang="de">Informations-Broking</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16134-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10711-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12186-0">
    <skos:narrower rdf:resource="descriptor/12188-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12186-0">
    <skos:narrower rdf:resource="descriptor/12184-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12186-0">
    <skos:narrower rdf:resource="descriptor/12183-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12186-0">
    <skos:narrower rdf:resource="descriptor/12182-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12186-0">
    <skos:narrower rdf:resource="descriptor/11626-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12186-0">
    <skos:narrower rdf:resource="descriptor/12174-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16272-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19795-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12818-3">
    <skos:narrower rdf:resource="descriptor/12821-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12818-3">
    <skos:narrower rdf:resource="descriptor/12816-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12818-3">
    <skos:narrower rdf:resource="descriptor/12820-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12818-3">
    <skos:narrower rdf:resource="descriptor/19136-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12818-3">
    <skos:narrower rdf:resource="descriptor/21408-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12818-3">
    <skos:narrower rdf:resource="descriptor/18553-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11230-0">
    <skos:related rdf:resource="descriptor/19491-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11230-0">
    <skos:related rdf:resource="descriptor/11233-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11230-0">
    <skos:related rdf:resource="descriptor/11231-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11230-0">
    <skos:related rdf:resource="descriptor/10522-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13457-4">
    <skos:prefLabel xml:lang="en">Retail chain</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13457-4">
    <skos:prefLabel xml:lang="de">Handelskette</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16688-5">
    <skos:scopeNote xml:lang="de">Soziologische Untersuchungen zu sozialen Verflechtungen in der Nachbarschaft, im Stadtteil oder der Gemeinde.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14490-1">
    <skos:broader rdf:resource="thsys/70135"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14490-1">
    <skos:broader rdf:resource="descriptor/14473-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71057">
    <rdfs:label xml:lang="de">Berufsgruppen</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11143-2">
    <skos:narrower rdf:resource="descriptor/11146-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10543-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19389-0">
    <skos:broader rdf:resource="thsys/70990"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19389-0">
    <skos:broader rdf:resource="descriptor/10012-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71043">
    <rdfs:label xml:lang="de">Buchführung und Bilanzierung</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14814-1">
    <skos:scopeNote xml:lang="de">Nur für Literatur über Kartenwerke.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16723-1">
    <skos:prefLabel xml:lang="en">Social philosophy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16723-1">
    <skos:prefLabel xml:lang="de">Sozialphilosophie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13283-4">
    <skos:broader rdf:resource="thsys/70905"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73343">
    <rdfs:label xml:lang="en">Materials research</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73343">
    <rdfs:label xml:lang="de">Werkstoffforschung</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71068">
    <rdfs:label xml:lang="en">Regional planning</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71068">
    <rdfs:label xml:lang="de">Regionalplanung</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70130">
    <rdfs:label xml:lang="en">Southeastern Europe</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70130">
    <rdfs:label xml:lang="de">Südosteuropa</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17374-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11521-3">
    <skos:scopeNote xml:lang="de">Bezeichnet vor allem die Neuordnung des - meist zerrütteten - Geldwesens eines Staates durch gesetzliche Maßnahmen, z.B. die deutsche Währungsreform 1923/24 als Folge der Hyperinflation.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70897">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.14.03.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70143">
    <skos:narrower rdf:resource="descriptor/15684-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70143">
    <skos:narrower rdf:resource="descriptor/11382-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70143">
    <skos:narrower rdf:resource="descriptor/15674-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70143">
    <skos:narrower rdf:resource="descriptor/16275-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70143">
    <skos:narrower rdf:resource="descriptor/15666-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70143">
    <skos:narrower rdf:resource="descriptor/15786-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70143">
    <skos:narrower rdf:resource="descriptor/12619-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70143">
    <skos:narrower rdf:resource="descriptor/15656-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70143">
    <skos:narrower rdf:resource="descriptor/11401-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70143">
    <skos:narrower rdf:resource="descriptor/18292-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70143">
    <skos:narrower rdf:resource="descriptor/18863-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70143">
    <skos:narrower rdf:resource="descriptor/11369-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70143">
    <skos:narrower rdf:resource="descriptor/18868-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70143">
    <skos:narrower rdf:resource="descriptor/18074-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70143">
    <skos:narrower rdf:resource="descriptor/18860-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70143">
    <skos:narrower rdf:resource="descriptor/11398-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70143">
    <skos:narrower rdf:resource="descriptor/11354-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70143">
    <skos:narrower rdf:resource="descriptor/11394-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70143">
    <skos:narrower rdf:resource="descriptor/19916-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70143">
    <skos:narrower rdf:resource="descriptor/11353-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70143">
    <skos:narrower rdf:resource="descriptor/10486-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70143">
    <skos:narrower rdf:resource="descriptor/11352-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14055-5">
    <skos:broader rdf:resource="thsys/73304"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14055-5">
    <skos:broader rdf:resource="thsys/70341"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14055-5">
    <skos:broader rdf:resource="descriptor/14074-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14055-5">
    <skos:broader rdf:resource="descriptor/14851-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11836-6">
    <skos:prefLabel xml:lang="en">Rent control</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11836-6">
    <skos:prefLabel xml:lang="de">Mietpreisbindung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11849-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19705-2">
    <skos:scopeNote xml:lang="de">Für die Harmonisierung der Rechtsvorschriften innerhalb der Europäischen Union BENUTZE EU-Recht oder die entsprechenden Unterbegriffe.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19705-2">
    <skos:scopeNote xml:lang="de">Im allgemeinen für die Rechtsangleichung zwischen verschiedenen Staaten. In Einzelfällen auch bei Änderungen bezüglich der Rechtshoheit über ein Staatsgebiet.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19800-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id112">
    <rdfs:seeAlso rdf:resource="descriptor/16623-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10138-0">
    <skos:prefLabel xml:lang="en">Auction theory</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10138-0">
    <skos:prefLabel xml:lang="de">Auktionstheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19128-0">
    <skos:altLabel xml:lang="de">Gebrauchsanweisung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19128-0">
    <skos:altLabel xml:lang="de">Bedienungsanleitung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11302-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18834-3">
    <skos:prefLabel xml:lang="en">Nonprofit management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18834-3">
    <skos:prefLabel xml:lang="de">Nonprofit-Management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13164-5">
    <skos:prefLabel xml:lang="en">Tobacco industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13164-5">
    <skos:prefLabel xml:lang="de">Tabakindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16299-2">
    <skos:broader rdf:resource="thsys/73381"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19070-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12210-1">
    <skos:prefLabel xml:lang="en">Financial economics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12210-1">
    <skos:prefLabel xml:lang="de">Kapitalmarkttheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14186-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18304-4">
    <skos:narrower rdf:resource="descriptor/18306-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18304-4">
    <skos:narrower rdf:resource="descriptor/18305-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16551-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11792-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13011-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10432-0">
    <skos:broader rdf:resource="thsys/70406"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10432-0">
    <skos:broader rdf:resource="descriptor/10434-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18364-0">
    <skos:broader rdf:resource="thsys/73371"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18364-0">
    <skos:broader rdf:resource="thsys/71014"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18364-0">
    <skos:broader rdf:resource="thsys/71012"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18364-0">
    <skos:broader rdf:resource="thsys/70539"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18364-0">
    <skos:broader rdf:resource="descriptor/15768-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18364-0">
    <skos:broader rdf:resource="descriptor/11810-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13957-5">
    <skos:scopeNote xml:lang="de">Nur für Berufs- und Ausbildungsfragen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19650-1">
    <skos:broader rdf:resource="thsys/70572"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19650-1">
    <skos:broader rdf:resource="descriptor/10032-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13995-4">
    <skos:related rdf:resource="descriptor/15979-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13995-4">
    <skos:related rdf:resource="descriptor/15971-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13995-4">
    <skos:related rdf:resource="descriptor/13389-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13995-4">
    <skos:related rdf:resource="descriptor/15966-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13995-4">
    <skos:related rdf:resource="descriptor/15962-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13995-4">
    <skos:related rdf:resource="descriptor/13410-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25881-3">
    <skos:broader rdf:resource="thsys/73385"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25881-3">
    <skos:broader rdf:resource="thsys/70244"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25881-3">
    <skos:broader rdf:resource="descriptor/18859-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10073-2">
    <skos:broader rdf:resource="thsys/70188"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10073-2">
    <skos:broader rdf:resource="descriptor/10076-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13984-2">
    <skos:related rdf:resource="descriptor/16342-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13984-2">
    <skos:related rdf:resource="descriptor/16340-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13984-2">
    <skos:related rdf:resource="descriptor/19693-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10350-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15894-0">
    <skos:broader rdf:resource="thsys/73352"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15894-0">
    <skos:broader rdf:resource="descriptor/15880-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15376-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18116-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70148">
    <skos:narrower rdf:resource="descriptor/17553-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70148">
    <skos:narrower rdf:resource="thsys/70561"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70148">
    <skos:narrower rdf:resource="thsys/70327"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70148">
    <skos:narrower rdf:resource="thsys/70147"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70148">
    <skos:narrower rdf:resource="thsys/70253"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70148">
    <skos:narrower rdf:resource="thsys/71032"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20022-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14461-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19841-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15125-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10397-4">
    <skos:broader rdf:resource="thsys/70355"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10397-4">
    <skos:broader rdf:resource="descriptor/19262-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13019-0">
    <skos:narrower rdf:resource="descriptor/13020-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13019-0">
    <skos:narrower rdf:resource="descriptor/13018-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18573-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12609-5">
    <skos:scopeNote xml:lang="de">Nur für Gefahren- und Unfallschutz am Arbeitsplatz.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11626-3">
    <skos:broader rdf:resource="thsys/71072"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11626-3">
    <skos:broader rdf:resource="thsys/70860"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11626-3">
    <skos:broader rdf:resource="thsys/70087"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11626-3">
    <skos:broader rdf:resource="descriptor/11628-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11626-3">
    <skos:broader rdf:resource="descriptor/12186-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18729-0">
    <skos:broader rdf:resource="thsys/73387"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18729-0">
    <skos:broader rdf:resource="thsys/70893"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18729-0">
    <skos:broader rdf:resource="thsys/70187"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18729-0">
    <skos:broader rdf:resource="thsys/70109"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18729-0">
    <skos:broader rdf:resource="descriptor/11416-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17662-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15920-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19269-3">
    <skos:prefLabel xml:lang="en">Arbitrage pricing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19269-3">
    <skos:prefLabel xml:lang="de">Arbitrage Pricing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13273-0">
    <zbwext:useInsteadNote rdf:nodeID="id255"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10606-0">
    <skos:prefLabel xml:lang="en">Foreign trade law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10606-0">
    <skos:prefLabel xml:lang="de">Außenwirtschaftsrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14244-2">
    <skos:altLabel xml:lang="de">Talkum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14244-2">
    <skos:altLabel xml:lang="de">Steatit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14244-2">
    <skos:altLabel xml:lang="de">Speckstein</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15757-3">
    <skos:narrower rdf:resource="descriptor/19794-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18550-3">
    <skos:altLabel xml:lang="en">C-store</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73346">
    <rdfs:label xml:lang="de">Gesundheitsrisiken und Krankheiten</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16679-6">
    <skos:altLabel xml:lang="de">Freizeitsoziologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12667-5">
    <skos:related rdf:resource="descriptor/18397-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12667-5">
    <skos:related rdf:resource="descriptor/12665-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12667-5">
    <skos:related rdf:resource="descriptor/12651-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12667-5">
    <skos:related rdf:resource="descriptor/11186-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14180-2">
    <skos:prefLabel xml:lang="en">Geothermal energy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14180-2">
    <skos:prefLabel xml:lang="de">Geothermische Energie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11569-3">
    <skos:broader rdf:resource="thsys/71080"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11569-3">
    <skos:broader rdf:resource="thsys/70833"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11569-3">
    <skos:broader rdf:resource="thsys/70324"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11569-3">
    <skos:broader rdf:resource="descriptor/11570-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11569-3">
    <skos:broader rdf:resource="descriptor/18910-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17013-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13478-3">
    <skos:related rdf:resource="descriptor/13470-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14198-4">
    <skos:related rdf:resource="descriptor/12985-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70506">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70175">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16545-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16342-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11067-6">
    <skos:altLabel xml:lang="en">Industrial democracy</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19569-5">
    <skos:broader rdf:resource="thsys/72182"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19569-5">
    <skos:broader rdf:resource="descriptor/19568-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11448-1">
    <skos:prefLabel xml:lang="en">Money stock</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11448-1">
    <skos:prefLabel xml:lang="de">Geldmenge</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18459-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71085">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.08.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12216-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13721-6">
    <skos:narrower rdf:resource="descriptor/12234-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18991-1">
    <skos:prefLabel xml:lang="en">Food price</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18991-1">
    <skos:prefLabel xml:lang="de">Nahrungsmittelpreis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19098-4">
    <skos:broader rdf:resource="thsys/70578"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18518-6">
    <skos:altLabel xml:lang="de">Kaffeerösterei</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16079-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13270-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70151">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.10</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73331">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12425-1">
    <skos:related rdf:resource="descriptor/18700-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12425-1">
    <skos:related rdf:resource="descriptor/12435-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12425-1">
    <skos:related rdf:resource="descriptor/12391-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12425-1">
    <skos:related rdf:resource="descriptor/11506-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12425-1">
    <skos:related rdf:resource="descriptor/12432-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14302-0">
    <skos:altLabel xml:lang="de">Nichteisenmetall</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14302-0">
    <skos:altLabel xml:lang="de">Messing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14302-0">
    <skos:altLabel xml:lang="de">Buntmetall</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14293-3">
    <skos:related rdf:resource="descriptor/14190-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14293-3">
    <skos:related rdf:resource="descriptor/18255-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14293-3">
    <skos:related rdf:resource="descriptor/13025-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18894-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15769-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11273-3">
    <skos:scopeNote xml:lang="de">Theoretische und empirische Untersuchungen zur Beteiligung von Frauen am Erwerbsleben.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14091-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11325-3">
    <skos:prefLabel xml:lang="en">Wage level</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11325-3">
    <skos:prefLabel xml:lang="de">Lohnniveau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10781-3">
    <skos:altLabel xml:lang="de">Wechselkursspekulation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10781-3">
    <skos:altLabel xml:lang="de">Währungsspekulation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19356-1">
    <skos:prefLabel xml:lang="en">Import demand</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19356-1">
    <skos:prefLabel xml:lang="de">Importnachfrage</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13003-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12012-5">
    <skos:prefLabel xml:lang="en">Business oriented infrastructure</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12012-5">
    <skos:prefLabel xml:lang="de">Wirtschaftsnahe Infrastruktur</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19073-6">
    <skos:broader rdf:resource="thsys/71035"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19073-6">
    <skos:broader rdf:resource="thsys/70175"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12275-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13967-2">
    <skos:prefLabel xml:lang="en">Brokers</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13967-2">
    <skos:prefLabel xml:lang="de">Makler</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19265-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13940-1">
    <skos:prefLabel xml:lang="en">Butchery</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13940-1">
    <skos:prefLabel xml:lang="de">Fleischerhandwerk</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71010">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18221-1">
    <skos:broader rdf:resource="thsys/73295"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18221-1">
    <skos:broader rdf:resource="descriptor/13157-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14617-3">
    <skos:broader rdf:resource="thsys/70074"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14617-3">
    <skos:broader rdf:resource="descriptor/14602-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10649-3">
    <skos:altLabel xml:lang="de">Zollwirkung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12675-6">
    <skos:broader rdf:resource="thsys/70300"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73294">
    <skos:narrower rdf:resource="descriptor/18167-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73294">
    <skos:narrower rdf:resource="descriptor/11677-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73294">
    <skos:narrower rdf:resource="descriptor/13164-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73294">
    <skos:narrower rdf:resource="descriptor/13163-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73294">
    <skos:narrower rdf:resource="descriptor/18900-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73294">
    <skos:narrower rdf:resource="descriptor/19296-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73294">
    <skos:narrower rdf:resource="descriptor/19295-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73294">
    <skos:narrower rdf:resource="descriptor/19294-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73294">
    <skos:narrower rdf:resource="descriptor/18518-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73396">
    <skos:narrower rdf:resource="descriptor/11060-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73396">
    <skos:narrower rdf:resource="descriptor/15711-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73396">
    <skos:narrower rdf:resource="descriptor/16606-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73396">
    <skos:narrower rdf:resource="descriptor/16605-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73396">
    <skos:narrower rdf:resource="descriptor/11047-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73396">
    <skos:narrower rdf:resource="descriptor/11046-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73396">
    <skos:narrower rdf:resource="descriptor/16604-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73396">
    <skos:narrower rdf:resource="descriptor/16603-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73396">
    <skos:narrower rdf:resource="descriptor/16405-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73396">
    <skos:narrower rdf:resource="descriptor/16602-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73396">
    <skos:narrower rdf:resource="descriptor/16250-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73396">
    <skos:narrower rdf:resource="descriptor/16601-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73396">
    <skos:narrower rdf:resource="descriptor/13425-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10350-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10472-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19456-4">
    <skos:broader rdf:resource="thsys/71066"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19456-4">
    <skos:broader rdf:resource="thsys/71064"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19456-4">
    <skos:broader rdf:resource="descriptor/12031-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15798-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10682-5">
    <skos:related rdf:resource="descriptor/10619-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10682-5">
    <skos:related rdf:resource="descriptor/18064-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10682-5">
    <skos:related rdf:resource="descriptor/10629-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10682-5">
    <skos:related rdf:resource="descriptor/10606-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10682-5">
    <skos:related rdf:resource="descriptor/10605-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12380-4">
    <skos:broader rdf:resource="thsys/70300"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12380-4">
    <skos:broader rdf:resource="thsys/70186"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12380-4">
    <skos:broader rdf:resource="descriptor/12381-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18262-1">
    <skos:broader rdf:resource="thsys/70994"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18262-1">
    <skos:broader rdf:resource="descriptor/12996-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12747-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16249-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10436-6">
    <skos:editorialNote xml:lang="de">Möglichst spezifischer indexieren.</skos:editorialNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12019-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18450-0">
    <skos:prefLabel xml:lang="en">Newsagent</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18450-0">
    <skos:prefLabel xml:lang="de">Zeitschriftenhandel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13348-2">
    <skos:prefLabel xml:lang="en">Psychologists</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13348-2">
    <skos:prefLabel xml:lang="de">Psychologen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14236-1">
    <skos:altLabel xml:lang="de">Vermiculit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14236-1">
    <skos:altLabel xml:lang="de">Schwerspat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14236-1">
    <skos:altLabel xml:lang="de">Mineralischer Rohstoff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14236-1">
    <skos:altLabel xml:lang="de">Industriemineral</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14236-1">
    <skos:altLabel xml:lang="de">Feldspat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14236-1">
    <skos:altLabel xml:lang="de">Baryt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18935-4">
    <skos:prefLabel xml:lang="en">Television corporation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18935-4">
    <skos:prefLabel xml:lang="de">Fernsehanstalt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15643-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18734-0">
    <skos:broader rdf:resource="thsys/73331"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18734-0">
    <skos:broader rdf:resource="thsys/70892"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18734-0">
    <skos:broader rdf:resource="descriptor/13825-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18734-0">
    <skos:broader rdf:resource="descriptor/16048-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71033">
    <skos:prefLabel xml:lang="en">B.01.06  Environmental management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71033">
    <skos:prefLabel xml:lang="de">B.01.06  Umweltmanagement</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11276-4">
    <skos:broader rdf:resource="thsys/71057"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11276-4">
    <skos:broader rdf:resource="thsys/71028"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11970-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70085">
    <rdfs:label xml:lang="en">North Africa</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70085">
    <rdfs:label xml:lang="de">Nordafrika</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19818-3">
    <skos:related rdf:resource="descriptor/19824-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19818-3">
    <skos:related rdf:resource="descriptor/19132-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19818-3">
    <skos:related rdf:resource="descriptor/19847-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19818-3">
    <skos:related rdf:resource="descriptor/13871-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19818-3">
    <skos:related rdf:resource="descriptor/18850-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70968">
    <skos:broader rdf:resource="thsys/70970"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14200-1">
    <skos:related rdf:resource="descriptor/14202-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14200-1">
    <skos:related rdf:resource="descriptor/15651-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14200-1">
    <skos:related rdf:resource="descriptor/18374-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14200-1">
    <skos:related rdf:resource="descriptor/19192-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14200-1">
    <skos:related rdf:resource="descriptor/12990-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14200-1">
    <skos:related rdf:resource="descriptor/14037-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16927-6">
    <skos:broader rdf:resource="thsys/70361"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16927-6">
    <skos:broader rdf:resource="descriptor/18012-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13543-4">
    <skos:related rdf:resource="descriptor/15883-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13543-4">
    <skos:related rdf:resource="descriptor/19616-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13543-4">
    <skos:related rdf:resource="descriptor/12967-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14093-4">
    <skos:prefLabel xml:lang="en">Maize</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14093-4">
    <skos:prefLabel xml:lang="de">Mais</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12699-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13618-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18958-6">
    <skos:related rdf:resource="descriptor/15768-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18958-6">
    <skos:related rdf:resource="descriptor/16479-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18958-6">
    <skos:related rdf:resource="descriptor/13325-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18958-6">
    <skos:related rdf:resource="descriptor/18959-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18958-6">
    <skos:related rdf:resource="descriptor/15988-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15463-6">
    <skos:altLabel xml:lang="en">Negotiation behaviour theory</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15463-6">
    <skos:altLabel xml:lang="en">Bargaining model</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15463-6">
    <skos:altLabel xml:lang="de">Verhandlungsspiel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15463-6">
    <skos:altLabel xml:lang="de">Theorie der Verhandlung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15463-6">
    <skos:altLabel xml:lang="de">Ökonomische Theorie der Verhandlungen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15463-6">
    <skos:altLabel xml:lang="de">Harsanyi-Modell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15463-6">
    <skos:altLabel xml:lang="de">Cross-Coddington-Modell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15463-6">
    <skos:altLabel xml:lang="de">Bishop-Foldes-Modell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17634-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13577-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15610-5">
    <skos:prefLabel xml:lang="en">Software portability</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15610-5">
    <skos:prefLabel xml:lang="de">Software-Portabilität</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19444-4">
    <skos:prefLabel xml:lang="en">Urbanization economies</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19444-4">
    <skos:prefLabel xml:lang="de">Urbanisierungseffekt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12927-5">
    <skos:altLabel xml:lang="de">Tourismusindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12927-5">
    <skos:altLabel xml:lang="de">Fremdenverkehrswirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15055-0">
    <skos:narrower rdf:resource="descriptor/12761-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15055-0">
    <skos:narrower rdf:resource="descriptor/15543-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15055-0">
    <skos:narrower rdf:resource="descriptor/15571-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15055-0">
    <skos:narrower rdf:resource="descriptor/15535-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15055-0">
    <skos:narrower rdf:resource="descriptor/15554-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15055-0">
    <skos:narrower rdf:resource="descriptor/15493-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15055-0">
    <skos:narrower rdf:resource="descriptor/15491-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15966-1">
    <skos:narrower rdf:resource="descriptor/16081-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15966-1">
    <skos:narrower rdf:resource="descriptor/16038-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15966-1">
    <skos:narrower rdf:resource="descriptor/16187-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15966-1">
    <skos:narrower rdf:resource="descriptor/16042-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15966-1">
    <skos:narrower rdf:resource="descriptor/16007-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15966-1">
    <skos:narrower rdf:resource="descriptor/15965-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15966-1">
    <skos:narrower rdf:resource="descriptor/19822-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15966-1">
    <skos:narrower rdf:resource="descriptor/16186-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15966-1">
    <skos:narrower rdf:resource="descriptor/19819-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15966-1">
    <skos:narrower rdf:resource="descriptor/19820-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15966-1">
    <skos:narrower rdf:resource="descriptor/16176-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15966-1">
    <skos:narrower rdf:resource="descriptor/11185-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16136-2">
    <skos:broader rdf:resource="thsys/73336"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16136-2">
    <skos:broader rdf:resource="thsys/71051"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16136-2">
    <skos:broader rdf:resource="thsys/70235"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16136-2">
    <skos:broader rdf:resource="descriptor/11185-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12992-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18097-4">
    <skos:related rdf:resource="descriptor/19830-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18097-4">
    <skos:related rdf:resource="descriptor/19699-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18097-4">
    <skos:related rdf:resource="descriptor/19221-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18097-4">
    <skos:related rdf:resource="descriptor/16365-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18097-4">
    <skos:related rdf:resource="descriptor/12047-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18097-4">
    <skos:related rdf:resource="descriptor/18831-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18996-5">
    <skos:scopeNote xml:lang="de">Nur für Literatur zum Energiemarkt allgemein.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20855-1">
    <skos:broader rdf:resource="thsys/73383"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20855-1">
    <skos:broader rdf:resource="thsys/73376"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16481-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16366-6">
    <skos:broader rdf:resource="thsys/73378"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16366-6">
    <skos:broader rdf:resource="descriptor/16292-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15671-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18615-1">
    <skos:prefLabel xml:lang="en">Freight industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18615-1">
    <skos:prefLabel xml:lang="de">Güterverkehrsgewerbe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70463">
    <rdfs:label xml:lang="de">Anorganische Chemikalien</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18379-1">
    <skos:related rdf:resource="descriptor/12979-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18379-1">
    <skos:related rdf:resource="descriptor/18378-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18379-1">
    <skos:related rdf:resource="descriptor/16023-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18379-1">
    <skos:related rdf:resource="descriptor/18990-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18379-1">
    <skos:related rdf:resource="descriptor/13329-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18792-0">
    <skos:altLabel xml:lang="en">Temporary work</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18792-0">
    <skos:altLabel xml:lang="en">Fixed-term labor contract</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18792-0">
    <skos:altLabel xml:lang="en">Fixed term labor contract</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18792-0">
    <skos:altLabel xml:lang="de">Zeitvertrag</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18792-0">
    <skos:altLabel xml:lang="de">Temporäre Arbeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19715-6">
    <skos:related rdf:resource="descriptor/16440-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19715-6">
    <skos:related rdf:resource="descriptor/15719-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19715-6">
    <skos:related rdf:resource="descriptor/11347-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19715-6">
    <skos:related rdf:resource="descriptor/15977-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19715-6">
    <skos:related rdf:resource="descriptor/16288-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19715-6">
    <skos:related rdf:resource="descriptor/19662-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19715-6">
    <skos:related rdf:resource="descriptor/11163-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19715-6">
    <skos:related rdf:resource="descriptor/19708-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19715-6">
    <skos:related rdf:resource="descriptor/16497-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19715-6">
    <skos:related rdf:resource="descriptor/16301-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73375">
    <skos:narrower rdf:resource="descriptor/19703-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73375">
    <skos:narrower rdf:resource="descriptor/11127-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73375">
    <skos:narrower rdf:resource="descriptor/19855-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73375">
    <skos:narrower rdf:resource="descriptor/16422-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73375">
    <skos:narrower rdf:resource="descriptor/16421-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73375">
    <skos:narrower rdf:resource="descriptor/16306-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73375">
    <skos:narrower rdf:resource="descriptor/16396-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73375">
    <skos:narrower rdf:resource="descriptor/16290-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73375">
    <skos:narrower rdf:resource="descriptor/16288-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73375">
    <skos:narrower rdf:resource="descriptor/16303-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73375">
    <skos:narrower rdf:resource="descriptor/16286-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73375">
    <skos:narrower rdf:resource="descriptor/16283-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73375">
    <skos:narrower rdf:resource="descriptor/16395-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73375">
    <skos:narrower rdf:resource="descriptor/16277-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73375">
    <skos:narrower rdf:resource="descriptor/16297-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73375">
    <skos:narrower rdf:resource="descriptor/16268-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73375">
    <skos:narrower rdf:resource="descriptor/16267-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15018-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18690-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19853-1">
    <skos:altLabel xml:lang="de">Rechtshilfeabkommen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19853-1">
    <skos:altLabel xml:lang="de">Internationaler Rechtsverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19853-1">
    <skos:altLabel xml:lang="de">Internationale Rechtsverfolgung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19853-1">
    <skos:altLabel xml:lang="de">Auslieferungsersuchen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70968">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.07.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19848-1">
    <skos:prefLabel xml:lang="en">Trade secret</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19848-1">
    <skos:prefLabel xml:lang="de">Geschäftsgeheimnis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18503-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11976-4">
    <skos:prefLabel xml:lang="en">Suburbanization</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11976-4">
    <skos:prefLabel xml:lang="de">Suburbanisierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18496-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70557">
    <skos:narrower rdf:resource="descriptor/17199-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70557">
    <skos:narrower rdf:resource="descriptor/17433-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70557">
    <skos:narrower rdf:resource="descriptor/17198-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70557">
    <skos:narrower rdf:resource="descriptor/17413-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70557">
    <skos:narrower rdf:resource="descriptor/17374-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70557">
    <skos:narrower rdf:resource="descriptor/17255-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70557">
    <skos:narrower rdf:resource="descriptor/17201-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13949-4">
    <skos:broader rdf:resource="thsys/71056"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13949-4">
    <skos:broader rdf:resource="thsys/70986"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18198-5">
    <skos:broader rdf:resource="thsys/71029"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18198-5">
    <skos:broader rdf:resource="descriptor/12933-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13106-5">
    <skos:altLabel xml:lang="en">Plywood industry</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13106-5">
    <skos:altLabel xml:lang="de">Sperrholzindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13106-5">
    <skos:altLabel xml:lang="de">Spanplattenindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13106-5">
    <skos:altLabel xml:lang="de">Holzwerkstoffindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13106-5">
    <skos:altLabel xml:lang="de">Faserplattenindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19307-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14045-1">
    <skos:broader rdf:resource="thsys/70461"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14045-1">
    <skos:broader rdf:resource="descriptor/14646-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14045-1">
    <skos:broader rdf:resource="descriptor/13427-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70951">
    <rdfs:label xml:lang="en">Foreign trade company</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70951">
    <rdfs:label xml:lang="de">Außenhandelsbetriebslehre</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13475-2">
    <skos:prefLabel xml:lang="en">Vendors</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13475-2">
    <skos:prefLabel xml:lang="de">Verkaufspersonal</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14623-1">
    <skos:broader rdf:resource="thsys/70556"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14623-1">
    <skos:broader rdf:resource="descriptor/14656-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14627-0">
    <skos:altLabel xml:lang="de">Schwefelkies</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11430-6">
    <skos:scopeNote xml:lang="de">Internationaler Goldstandard ca. 1879-1913.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13364-4">
    <skos:altLabel xml:lang="de">Stadttheater</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13364-4">
    <skos:altLabel xml:lang="de">Staatstheater</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13364-4">
    <skos:altLabel xml:lang="de">Schauspiel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13364-4">
    <skos:altLabel xml:lang="de">Kabarett</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10350-2">
    <skos:prefLabel xml:lang="en">Long wave theory</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10350-2">
    <skos:prefLabel xml:lang="de">Lange Wellen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19488-5">
    <skos:related rdf:resource="descriptor/15745-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19488-5">
    <skos:related rdf:resource="descriptor/11044-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12737-3">
    <skos:narrower rdf:resource="descriptor/18245-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70985">
    <skos:narrower rdf:resource="descriptor/18398-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70985">
    <skos:narrower rdf:resource="descriptor/18272-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70985">
    <skos:narrower rdf:resource="descriptor/18415-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70985">
    <skos:narrower rdf:resource="descriptor/12930-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24623-2">
    <skos:related rdf:resource="descriptor/18140-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24623-2">
    <skos:related rdf:resource="descriptor/10143-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14007-2">
    <skos:prefLabel xml:lang="en">Nursing profession</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14007-2">
    <skos:prefLabel xml:lang="de">Pflegeberufe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73379">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.04.02.03</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12308-5">
    <skos:narrower rdf:resource="descriptor/12403-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12308-5">
    <skos:narrower rdf:resource="descriptor/12275-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/g">
    <skos:narrower rdf:resource="thsys/70026"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/g">
    <skos:narrower rdf:resource="thsys/70198"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/g">
    <skos:narrower rdf:resource="thsys/70076"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/g">
    <skos:narrower rdf:resource="thsys/70148"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/g">
    <skos:narrower rdf:resource="thsys/70128"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/g">
    <skos:narrower rdf:resource="thsys/70002"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/g">
    <skos:narrower rdf:resource="thsys/70307"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71042">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">B.03.01.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10653-5">
    <skos:related rdf:resource="descriptor/10661-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15992-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14105-2">
    <skos:broader rdf:resource="thsys/70169"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14105-2">
    <skos:broader rdf:resource="descriptor/14104-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13209-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10049-6">
    <skos:related rdf:resource="descriptor/11782-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10049-6">
    <skos:related rdf:resource="descriptor/10256-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18342-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11774-2">
    <skos:altLabel xml:lang="de">Profitquote</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73306">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70007">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19379-3">
    <skos:related rdf:resource="descriptor/16013-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19379-3">
    <skos:related rdf:resource="descriptor/11613-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19379-3">
    <skos:related rdf:resource="descriptor/19358-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19455-6">
    <skos:prefLabel xml:lang="en">Large-scale agriculture</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19455-6">
    <skos:prefLabel xml:lang="de">Großgrundbesitz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19720-6">
    <skos:narrower rdf:resource="descriptor/16460-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19720-6">
    <skos:narrower rdf:resource="descriptor/16471-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11893-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14413-5">
    <skos:broader rdf:resource="thsys/70337"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14413-5">
    <skos:broader rdf:resource="thsys/70257"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14413-5">
    <skos:broader rdf:resource="descriptor/13513-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14413-5">
    <skos:broader rdf:resource="descriptor/15005-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12821-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19134-5">
    <skos:prefLabel xml:lang="en">Consumer complaint</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19134-5">
    <skos:prefLabel xml:lang="de">Reklamation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13037-5">
    <skos:altLabel xml:lang="de">Nichteisenmetall-Industrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13037-5">
    <skos:altLabel xml:lang="de">NE-Metallerzeugende Industrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13037-5">
    <skos:altLabel xml:lang="de">Buntmetallwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19445-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13983-4">
    <skos:prefLabel xml:lang="en">Seafarers</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13983-4">
    <skos:prefLabel xml:lang="de">Schifffahrtsberufe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16780-3">
    <skos:narrower rdf:resource="descriptor/16789-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11850-5">
    <skos:prefLabel xml:lang="en">Recreation area</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11850-5">
    <skos:prefLabel xml:lang="de">Erholungsgebiet</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13400-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15880-4">
    <skos:prefLabel xml:lang="en">Climate</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15880-4">
    <skos:prefLabel xml:lang="de">Klima</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15051-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19718-0">
    <skos:prefLabel xml:lang="en">Pan-Africanism</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19718-0">
    <skos:prefLabel xml:lang="de">Panafrikanismus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18857-5">
    <skos:prefLabel xml:lang="en">Educational institution</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18857-5">
    <skos:prefLabel xml:lang="de">Bildungseinrichtung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18926-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11024-3">
    <skos:altLabel xml:lang="de">Sozialistische Preistheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11024-3">
    <skos:altLabel xml:lang="de">Sozialistische Preispolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11024-3">
    <skos:altLabel xml:lang="de">Planpreis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11024-3">
    <skos:altLabel xml:lang="de">Planmäßige Preisbildung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11024-3">
    <skos:altLabel xml:lang="de">Marxistische Preistheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13073-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20306-6">
    <skos:altLabel xml:lang="de">Gerätesicherheit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19353-0">
    <skos:altLabel xml:lang="de">Europawährung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12705-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id250">
    <rdfs:seeAlso rdf:resource="descriptor/11270-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73396">
    <skos:broader rdf:resource="thsys/70321"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16896-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12206-6">
    <skos:related rdf:resource="descriptor/12212-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12206-6">
    <skos:related rdf:resource="descriptor/12244-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12206-6">
    <skos:related rdf:resource="descriptor/12209-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12206-6">
    <skos:related rdf:resource="descriptor/12224-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12206-6">
    <skos:related rdf:resource="descriptor/12215-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12940-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10574-1">
    <skos:related rdf:resource="descriptor/16389-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10574-1">
    <skos:related rdf:resource="descriptor/15698-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10574-1">
    <skos:related rdf:resource="descriptor/19717-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10574-1">
    <skos:related rdf:resource="descriptor/10515-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10574-1">
    <skos:related rdf:resource="descriptor/19739-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70198">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18837-4">
    <skos:altLabel xml:lang="en">City-marketing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18837-4">
    <skos:altLabel xml:lang="de">Kommunales Marketing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73379">
    <rdfs:label xml:lang="de">Parlament und Parlamentarismus</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12966-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19722-2">
    <skos:narrower rdf:resource="descriptor/11565-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10672-1">
    <skos:broader rdf:resource="thsys/71099"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10672-1">
    <skos:broader rdf:resource="descriptor/10676-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16726-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16017-3">
    <skos:related rdf:resource="descriptor/13422-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16017-3">
    <skos:related rdf:resource="descriptor/16016-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16017-3">
    <skos:related rdf:resource="descriptor/18905-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16017-3">
    <skos:related rdf:resource="descriptor/16233-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16017-3">
    <skos:related rdf:resource="descriptor/19689-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16017-3">
    <skos:related rdf:resource="descriptor/16013-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16017-3">
    <skos:related rdf:resource="descriptor/16305-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16017-3">
    <skos:related rdf:resource="descriptor/13420-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16017-3">
    <skos:related rdf:resource="descriptor/19323-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16017-3">
    <skos:related rdf:resource="descriptor/16002-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16017-3">
    <skos:related rdf:resource="descriptor/16100-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73301">
    <skos:prefLabel xml:lang="de">P.20.04  Fisch und Fischprodukte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18510-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18273-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70970">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16416-3">
    <skos:narrower rdf:resource="descriptor/16424-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13808-1">
    <skos:broader rdf:resource="thsys/70891"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13808-1">
    <skos:broader rdf:resource="thsys/70485"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13808-1">
    <skos:broader rdf:resource="descriptor/13828-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11518-6">
    <skos:related rdf:resource="descriptor/11519-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11518-6">
    <skos:related rdf:resource="descriptor/11322-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19256-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12384-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12510-3">
    <skos:related rdf:resource="descriptor/19140-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18427-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19553-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11301-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19211-6">
    <skos:broader rdf:resource="thsys/70274"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19211-6">
    <skos:broader rdf:resource="thsys/70039"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19211-6">
    <skos:broader rdf:resource="descriptor/11161-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11123-1">
    <skos:prefLabel xml:lang="en">Socialist countries</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11123-1">
    <skos:prefLabel xml:lang="de">Sozialistische Staaten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11534-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10123-6">
    <skos:prefLabel xml:lang="en">Capital-output ratio</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10123-6">
    <skos:prefLabel xml:lang="de">Kapitalkoeffizient</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18928-1">
    <skos:prefLabel xml:lang="en">Media economics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18928-1">
    <skos:prefLabel xml:lang="de">Medienökonomik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20002-5">
    <skos:scopeNote xml:lang="de">In der Regel handelt es sich um nichtautorisierte Importe</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13595-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15410-6">
    <skos:narrower rdf:resource="descriptor/19811-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15410-6">
    <skos:narrower rdf:resource="descriptor/18558-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19195-6">
    <skos:altLabel xml:lang="en">Congestion costs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19195-6">
    <skos:altLabel xml:lang="en">Bottleneck model</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19195-6">
    <skos:altLabel xml:lang="de">Engpaßfaktor</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18661-1">
    <skos:scopeNote xml:lang="de">Potenzial eines Computers, bestehend aus der Kombination seiner Hard- und Software.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18131-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19078-3">
    <skos:prefLabel xml:lang="en">CIS countries</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19078-3">
    <skos:prefLabel xml:lang="de">GUS-Staaten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16404-3">
    <skos:broader rdf:resource="thsys/73383"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16404-3">
    <skos:broader rdf:resource="thsys/73356"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16404-3">
    <skos:broader rdf:resource="descriptor/16405-1"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id181">
    <rdfs:seeAlso rdf:resource="descriptor/16515-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15698-0">
    <skos:broader rdf:resource="thsys/73387"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15698-0">
    <skos:broader rdf:resource="thsys/70109"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14171-3">
    <skos:narrower rdf:resource="descriptor/18009-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18761-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12381-2">
    <skos:altLabel xml:lang="de">Rechnungslegung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12381-2">
    <skos:altLabel xml:lang="de">Jahresbilanz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12381-2">
    <skos:altLabel xml:lang="de">Jahresabschluß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12381-2">
    <skos:altLabel xml:lang="de">Handelsrechtlicher Jahresabschluss</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12381-2">
    <skos:altLabel xml:lang="de">Handelsrechtlicher Jahresabschluß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12381-2">
    <skos:altLabel xml:lang="de">Abschlussrechnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12381-2">
    <skos:altLabel xml:lang="de">Abschlußrechnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13451-2">
    <skos:prefLabel xml:lang="en">Agrarian constitution</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13451-2">
    <skos:prefLabel xml:lang="de">Agrarverfassung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18498-0">
    <skos:broader rdf:resource="thsys/70969"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18498-0">
    <skos:broader rdf:resource="descriptor/13176-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11986-1">
    <skos:narrower rdf:resource="descriptor/11992-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11986-1">
    <skos:narrower rdf:resource="descriptor/11987-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19462-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19155-4">
    <skos:broader rdf:resource="thsys/70562"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19155-4">
    <skos:broader rdf:resource="descriptor/12134-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12761-6">
    <skos:related rdf:resource="descriptor/13504-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12761-6">
    <skos:related rdf:resource="descriptor/18566-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19255-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13018-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13807-3">
    <skos:altLabel xml:lang="de">Zahlungsverkehrssystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13807-3">
    <skos:altLabel xml:lang="de">Kassenverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13807-3">
    <skos:altLabel xml:lang="de">Barzahlungsverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18817-3">
    <skos:broader rdf:resource="thsys/73363"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18817-3">
    <skos:broader rdf:resource="thsys/70865"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18817-3">
    <skos:broader rdf:resource="descriptor/10032-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18817-3">
    <skos:broader rdf:resource="descriptor/19725-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11991-1">
    <skos:prefLabel xml:lang="en">City region</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11991-1">
    <skos:prefLabel xml:lang="de">Stadtregion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11055-6">
    <skos:narrower rdf:resource="descriptor/11059-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20897-6">
    <skos:prefLabel xml:lang="en">Hub-and-spoke network</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20897-6">
    <skos:prefLabel xml:lang="de">Hub-and-Spoke System</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13683-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19092-2">
    <skos:broader rdf:resource="thsys/71043"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19092-2">
    <skos:broader rdf:resource="descriptor/12408-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13360-5">
    <skos:narrower rdf:resource="descriptor/18005-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13360-5">
    <skos:narrower rdf:resource="descriptor/18935-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70927">
    <rdfs:label xml:lang="en">Transport Management Science</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70927">
    <rdfs:label xml:lang="de">Verkehrsbetriebslehre</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10859-6">
    <skos:prefLabel xml:lang="en">Foreign trade company</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10859-6">
    <skos:prefLabel xml:lang="de">Außenhandelsbetriebslehre</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10722-5">
    <skos:narrower rdf:resource="descriptor/19335-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10722-5">
    <skos:narrower rdf:resource="descriptor/19334-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10722-5">
    <skos:narrower rdf:resource="descriptor/10718-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10722-5">
    <skos:narrower rdf:resource="descriptor/10717-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14941-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12665-2">
    <skos:related rdf:resource="descriptor/21094-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12665-2">
    <skos:related rdf:resource="descriptor/12108-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12665-2">
    <skos:related rdf:resource="descriptor/12766-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12665-2">
    <skos:related rdf:resource="descriptor/12667-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12665-2">
    <skos:related rdf:resource="descriptor/18104-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12665-2">
    <skos:related rdf:resource="descriptor/24785-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12665-2">
    <skos:related rdf:resource="descriptor/19141-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12665-2">
    <skos:related rdf:resource="descriptor/12785-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12665-2">
    <skos:related rdf:resource="descriptor/12787-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12665-2">
    <skos:related rdf:resource="descriptor/12668-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12665-2">
    <skos:related rdf:resource="descriptor/20442-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12665-2">
    <skos:related rdf:resource="descriptor/19133-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12665-2">
    <skos:related rdf:resource="descriptor/18553-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12665-2">
    <skos:related rdf:resource="descriptor/19428-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12665-2">
    <skos:related rdf:resource="descriptor/12640-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12665-2">
    <skos:related rdf:resource="descriptor/12777-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18120-0">
    <skos:broader rdf:resource="thsys/71034"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18120-0">
    <skos:broader rdf:resource="descriptor/12072-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12644-3">
    <skos:narrower rdf:resource="descriptor/12643-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18682-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11734-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18817-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19163-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19245-3">
    <skos:prefLabel xml:lang="en">Dutch disease</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19245-3">
    <skos:prefLabel xml:lang="de">Dutch Disease</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19513-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12165-1">
    <skos:altLabel xml:lang="en">Enterprise information system</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12165-1">
    <skos:altLabel xml:lang="de">Informationstechnik des Betriebs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12165-1">
    <skos:altLabel xml:lang="de">Informationssystem des Betriebs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12165-1">
    <skos:altLabel xml:lang="de">Datenverarbeitung des Betriebs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12165-1">
    <skos:altLabel xml:lang="de">Betriebliches Kommunikationssystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12165-1">
    <skos:altLabel xml:lang="de">Betriebliche EDV</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12165-1">
    <skos:altLabel xml:lang="de">Betriebliche Datenverarbeitung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10436-6">
    <skos:broader rdf:resource="thsys/73393"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10436-6">
    <skos:broader rdf:resource="thsys/71109"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10436-6">
    <skos:broader rdf:resource="thsys/70879"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11203-3">
    <skos:prefLabel xml:lang="en">Temporary agency work</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11203-3">
    <skos:prefLabel xml:lang="de">Leiharbeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14129-2">
    <skos:prefLabel xml:lang="en">Animal product</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14129-2">
    <skos:prefLabel xml:lang="de">Tierisches Produkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10190-5">
    <skos:prefLabel xml:lang="en">Theory of the firm</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10190-5">
    <skos:prefLabel xml:lang="de">Theorie der Unternehmung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17696-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19821-0">
    <skos:broader rdf:resource="thsys/73360"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19821-0">
    <skos:broader rdf:resource="descriptor/16019-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19821-0">
    <skos:broader rdf:resource="descriptor/15981-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18561-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11993-4">
    <skos:broader rdf:resource="thsys/71064"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11993-4">
    <skos:broader rdf:resource="descriptor/19364-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11149-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19580-3">
    <skos:broader rdf:resource="thsys/72181"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19580-3">
    <skos:broader rdf:resource="thsys/70572"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19580-3">
    <skos:broader rdf:resource="descriptor/15061-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18032-4">
    <skos:related rdf:resource="descriptor/19139-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18032-4">
    <skos:related rdf:resource="descriptor/19095-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18032-4">
    <skos:related rdf:resource="descriptor/12438-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18032-4">
    <skos:related rdf:resource="descriptor/12196-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18032-4">
    <skos:related rdf:resource="descriptor/12304-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18032-4">
    <skos:related rdf:resource="descriptor/12058-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18032-4">
    <skos:related rdf:resource="descriptor/10897-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18032-4">
    <skos:related rdf:resource="descriptor/18107-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13639-5">
    <skos:prefLabel xml:lang="en">Tourism policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13639-5">
    <skos:prefLabel xml:lang="de">Fremdenverkehrspolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10672-1">
    <skos:prefLabel xml:lang="en">Monetary approach to the balance of payments</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10672-1">
    <skos:prefLabel xml:lang="de">Monetäre Zahlungsbilanztheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17742-6">
    <skos:prefLabel xml:lang="en">Congo (Democratic Republic)</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17742-6">
    <skos:prefLabel xml:lang="de">Kongo (Demokratische Republik)</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13845-2">
    <skos:narrower rdf:resource="descriptor/11994-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70055">
    <skos:prefLabel xml:lang="de">V.05  Geld- und Finanzmärkte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20972-4">
    <skos:related rdf:resource="descriptor/19164-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20972-4">
    <skos:related rdf:resource="descriptor/12055-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20972-4">
    <skos:related rdf:resource="descriptor/10572-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17790-2">
    <skos:prefLabel xml:lang="en">Mexico</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17790-2">
    <skos:prefLabel xml:lang="de">Mexiko</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70931">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.12.01.04.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71097">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14921-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70834">
    <skos:prefLabel xml:lang="de">W.26.01  Soziale Dienste</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17596-5">
    <skos:prefLabel xml:lang="en">Afghanistan</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17596-5">
    <skos:prefLabel xml:lang="de">Afghanistan</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18912-2">
    <skos:scopeNote xml:lang="de">Materielle und Vermögensschäden, die durch einen Krieg verursacht werden sowie Fragen der Beseitigung dieser Schäden</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71046">
    <rdfs:label xml:lang="de">Eigenkapitalbeschaffung</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19238-0">
    <skos:related rdf:resource="descriptor/10376-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19238-0">
    <skos:related rdf:resource="descriptor/10375-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15731-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11105-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19705-2">
    <skos:related rdf:resource="descriptor/19688-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19705-2">
    <skos:related rdf:resource="descriptor/19714-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19705-2">
    <skos:related rdf:resource="descriptor/16278-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19705-2">
    <skos:related rdf:resource="descriptor/19822-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19705-2">
    <skos:related rdf:resource="descriptor/18062-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19705-2">
    <skos:related rdf:resource="descriptor/16182-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14729-6">
    <skos:prefLabel xml:lang="en">Psychotherapeutic agent</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14729-6">
    <skos:prefLabel xml:lang="de">Psychopharmakon</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14723-4">
    <zbwext:useInsteadNote rdf:nodeID="id76"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70074">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19339-1">
    <skos:scopeNote xml:lang="de">Nur für den Boden selbst, z.B. Bodentyp.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15787-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id275">
    <rdf:value xml:lang="de">Für Anwaltskanzleien</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18330-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18386-4">
    <skos:broader rdf:resource="thsys/72181"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18386-4">
    <skos:broader rdf:resource="thsys/71009"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18386-4">
    <skos:broader rdf:resource="descriptor/15103-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11037-1">
    <skos:related rdf:resource="descriptor/18910-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11037-1">
    <skos:related rdf:resource="descriptor/11741-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11037-1">
    <skos:related rdf:resource="descriptor/18830-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11037-1">
    <skos:related rdf:resource="descriptor/19220-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19435-5">
    <skos:scopeNote xml:lang="de">Marktregulierung über die Festlegung von Mengen bzw. Quoten.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18195-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19576-1">
    <skos:prefLabel xml:lang="en">Cointegration</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19576-1">
    <skos:prefLabel xml:lang="de">Kointegration</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16258-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12941-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12427-4">
    <skos:altLabel xml:lang="de">Zeitwert</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12427-4">
    <skos:altLabel xml:lang="de">Wiederbeschaffungswert</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12427-4">
    <skos:altLabel xml:lang="de">Verkehrswert</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12427-4">
    <skos:altLabel xml:lang="de">Verkaufswert</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12427-4">
    <skos:altLabel xml:lang="de">Tageswert</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19380-4">
    <skos:related rdf:resource="descriptor/11596-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19380-4">
    <skos:related rdf:resource="descriptor/11539-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19380-4">
    <skos:related rdf:resource="descriptor/16100-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19380-4">
    <skos:related rdf:resource="descriptor/11535-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14636-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16409-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19781-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12204-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13874-2">
    <skos:broader rdf:resource="thsys/71057"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13874-2">
    <skos:broader rdf:resource="thsys/71019"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10148-4">
    <skos:scopeNote xml:lang="de">Berücksichtigung zeitlicher Aspekte in ökonomischen Modellen. Möglichst spezifischer indexieren.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10991-6">
    <skos:altLabel xml:lang="en">Impact analysis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10991-6">
    <skos:altLabel xml:lang="de">Wirkungsanalyse der Wirtschaftspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10991-6">
    <skos:altLabel xml:lang="de">Ökonomische Effizienz wirtschaftspolitischer Maßnahmen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10991-6">
    <skos:altLabel xml:lang="de">Evaluierung wirtschaftspolitischer Maßnahmen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10991-6">
    <skos:altLabel xml:lang="de">Evaluierung der Wirtschaftspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10991-6">
    <skos:altLabel xml:lang="de">Effizienz der Wirtschaftspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19176-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18878-4">
    <skos:related rdf:resource="descriptor/11379-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18878-4">
    <skos:related rdf:resource="descriptor/11378-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18878-4">
    <skos:related rdf:resource="descriptor/18781-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14674-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12577-6">
    <skos:narrower rdf:resource="descriptor/15842-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18234-6">
    <skos:broader rdf:resource="thsys/70538"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18234-6">
    <skos:broader rdf:resource="descriptor/14175-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73302">
    <skos:broader rdf:resource="thsys/70498"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19656-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19406-5">
    <skos:broader rdf:resource="thsys/71076"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19406-5">
    <skos:broader rdf:resource="descriptor/11684-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19722-2">
    <skos:scopeNote xml:lang="de">Politische und soziale, aber auch wirtschaftliche Folgen eines Krieges</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17744-2">
    <skos:prefLabel xml:lang="en">Central African Republic</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17744-2">
    <skos:prefLabel xml:lang="de">Zentralafrikanische Republik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10746-5">
    <skos:narrower rdf:resource="descriptor/19335-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10746-5">
    <skos:narrower rdf:resource="descriptor/10771-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10746-5">
    <skos:narrower rdf:resource="descriptor/10769-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10746-5">
    <skos:narrower rdf:resource="descriptor/10768-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10746-5">
    <skos:narrower rdf:resource="descriptor/10734-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10746-5">
    <skos:narrower rdf:resource="descriptor/10728-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18851-3">
    <skos:related rdf:resource="descriptor/18979-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18851-3">
    <skos:related rdf:resource="descriptor/19046-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18851-3">
    <skos:related rdf:resource="descriptor/13426-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18851-3">
    <skos:related rdf:resource="descriptor/11801-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17668-6">
    <skos:broader rdf:resource="thsys/70445"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17668-6">
    <skos:broader rdf:resource="descriptor/17672-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18847-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11864-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19504-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18899-3">
    <skos:altLabel xml:lang="de">Leukämie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18899-3">
    <skos:altLabel xml:lang="de">Krebserkrankung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18899-3">
    <skos:altLabel xml:lang="de">Brustkrebs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18305-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16761-0">
    <zbwext:useInsteadNote rdf:nodeID="id169"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19254-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15142-5">
    <skos:broader rdf:resource="thsys/72183"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15142-5">
    <skos:broader rdf:resource="descriptor/15064-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18480-5">
    <skos:prefLabel xml:lang="en">Regional industrial structure</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18480-5">
    <skos:prefLabel xml:lang="de">Regionale Industriestruktur</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73394">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15060-0">
    <skos:narrower rdf:resource="descriptor/15176-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15060-0">
    <skos:narrower rdf:resource="descriptor/15067-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15060-0">
    <skos:narrower rdf:resource="descriptor/19592-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15060-0">
    <skos:narrower rdf:resource="descriptor/15225-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15060-0">
    <skos:narrower rdf:resource="descriptor/15467-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15060-0">
    <skos:narrower rdf:resource="descriptor/15163-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18673-1">
    <skos:broader rdf:resource="thsys/70905"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18673-1">
    <skos:broader rdf:resource="thsys/70187"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18673-1">
    <skos:broader rdf:resource="thsys/70091"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18673-1">
    <skos:broader rdf:resource="descriptor/10501-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18673-1">
    <skos:broader rdf:resource="descriptor/18671-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18125-4">
    <skos:related rdf:resource="descriptor/19502-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18125-4">
    <skos:related rdf:resource="descriptor/11313-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18125-4">
    <skos:related rdf:resource="descriptor/11314-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12996-0">
    <skos:related rdf:resource="descriptor/13004-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12996-0">
    <skos:related rdf:resource="descriptor/13378-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12996-0">
    <skos:related rdf:resource="descriptor/18429-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12996-0">
    <skos:related rdf:resource="descriptor/13959-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12996-0">
    <skos:related rdf:resource="descriptor/15992-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12996-0">
    <skos:related rdf:resource="descriptor/13198-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12996-0">
    <skos:related rdf:resource="descriptor/14599-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12996-0">
    <skos:related rdf:resource="descriptor/14331-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12996-0">
    <skos:related rdf:resource="descriptor/18430-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12996-0">
    <skos:related rdf:resource="descriptor/13888-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12996-0">
    <skos:related rdf:resource="descriptor/18408-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12996-0">
    <skos:related rdf:resource="descriptor/15638-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15968-4">
    <skos:prefLabel xml:lang="en">History of law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15968-4">
    <skos:prefLabel xml:lang="de">Rechtsgeschichte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10515-3">
    <skos:prefLabel xml:lang="en">Dependency theory</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10515-3">
    <skos:prefLabel xml:lang="de">Dependenztheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14923-3">
    <skos:broader rdf:resource="thsys/70031"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14923-3">
    <skos:broader rdf:resource="descriptor/14899-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12038-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18771-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70176">
    <skos:narrower rdf:resource="descriptor/18879-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70176">
    <skos:narrower rdf:resource="descriptor/18870-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70176">
    <skos:narrower rdf:resource="descriptor/11404-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70176">
    <skos:narrower rdf:resource="descriptor/11377-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70176">
    <skos:narrower rdf:resource="descriptor/11409-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70176">
    <skos:narrower rdf:resource="descriptor/11267-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70176">
    <skos:narrower rdf:resource="descriptor/18865-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70176">
    <skos:narrower rdf:resource="descriptor/11224-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70176">
    <skos:narrower rdf:resource="descriptor/18877-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70176">
    <skos:narrower rdf:resource="descriptor/11402-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70176">
    <skos:narrower rdf:resource="descriptor/18782-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70176">
    <skos:narrower rdf:resource="descriptor/18874-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70176">
    <skos:narrower rdf:resource="descriptor/11373-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70176">
    <skos:narrower rdf:resource="descriptor/18863-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70176">
    <skos:narrower rdf:resource="descriptor/11369-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70176">
    <skos:narrower rdf:resource="descriptor/19543-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70176">
    <skos:narrower rdf:resource="descriptor/11367-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70176">
    <skos:narrower rdf:resource="descriptor/18860-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70176">
    <skos:narrower rdf:resource="descriptor/11399-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70176">
    <skos:narrower rdf:resource="descriptor/11398-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70176">
    <skos:narrower rdf:resource="descriptor/11397-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70176">
    <skos:narrower rdf:resource="descriptor/11407-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70176">
    <skos:narrower rdf:resource="descriptor/11354-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70176">
    <skos:narrower rdf:resource="descriptor/11222-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70176">
    <skos:narrower rdf:resource="descriptor/11394-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70176">
    <skos:narrower rdf:resource="descriptor/18884-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70176">
    <skos:narrower rdf:resource="descriptor/19516-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70176">
    <skos:narrower rdf:resource="descriptor/11400-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70176">
    <skos:narrower rdf:resource="descriptor/19916-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70176">
    <skos:narrower rdf:resource="descriptor/11353-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70176">
    <skos:narrower rdf:resource="descriptor/16134-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70176">
    <skos:narrower rdf:resource="descriptor/11392-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70176">
    <skos:narrower rdf:resource="descriptor/11360-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70176">
    <skos:narrower rdf:resource="descriptor/11384-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70176">
    <skos:narrower rdf:resource="descriptor/11406-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70176">
    <skos:narrower rdf:resource="descriptor/10486-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70176">
    <skos:narrower rdf:resource="descriptor/11352-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11092-0">
    <skos:prefLabel xml:lang="en">Surplus value</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11092-0">
    <skos:prefLabel xml:lang="de">Mehrwerttheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11469-0">
    <skos:altLabel xml:lang="de">Nichtneutralität des Geldes</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11469-0">
    <skos:altLabel xml:lang="de">Neutrales Geld</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11469-0">
    <skos:altLabel xml:lang="de">Geldschleier</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11469-0">
    <skos:altLabel xml:lang="de">Geldneutralität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19837-6">
    <skos:prefLabel xml:lang="en">Transition countries</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19837-6">
    <skos:prefLabel xml:lang="de">Transformationsstaaten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12717-2">
    <skos:prefLabel xml:lang="en">Construction-site production</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12717-2">
    <skos:prefLabel xml:lang="de">Baustellenfertigung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70131">
    <skos:narrower rdf:resource="descriptor/17017-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70131">
    <skos:narrower rdf:resource="descriptor/17016-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70131">
    <skos:narrower rdf:resource="descriptor/17161-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70131">
    <skos:narrower rdf:resource="descriptor/17015-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70131">
    <skos:narrower rdf:resource="descriptor/17104-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70131">
    <skos:narrower rdf:resource="descriptor/17014-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70131">
    <skos:narrower rdf:resource="descriptor/17013-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70131">
    <skos:narrower rdf:resource="descriptor/17039-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70131">
    <skos:narrower rdf:resource="descriptor/17011-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20503-4">
    <skos:prefLabel xml:lang="en">Global sourcing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20503-4">
    <skos:prefLabel xml:lang="de">Internationale Beschaffung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13593-3">
    <skos:prefLabel xml:lang="en">Transportation network</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13593-3">
    <skos:prefLabel xml:lang="de">Verkehrserschließung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14149-3">
    <skos:related rdf:resource="descriptor/14076-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14149-3">
    <skos:related rdf:resource="descriptor/14703-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14149-3">
    <skos:related rdf:resource="descriptor/12961-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14149-3">
    <skos:related rdf:resource="descriptor/14101-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14149-3">
    <skos:related rdf:resource="descriptor/14054-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19016-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17666-3">
    <skos:broader rdf:resource="thsys/70445"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17666-3">
    <skos:broader rdf:resource="descriptor/17672-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12381-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16120-3">
    <skos:altLabel xml:lang="en">Auditor Independence</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16120-3">
    <skos:altLabel xml:lang="de">Wirtschaftsprüferordnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16120-3">
    <skos:altLabel xml:lang="de">Rechnungsprüfer-Unabhängigkeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18096-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14052-4">
    <skos:prefLabel xml:lang="en">Water treatment plant</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14052-4">
    <skos:prefLabel xml:lang="de">Wasseraufbereitungsanlage</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12780-2">
    <skos:broader rdf:resource="thsys/70459"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12780-2">
    <skos:broader rdf:resource="descriptor/12778-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12780-2">
    <skos:broader rdf:resource="descriptor/12766-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11944-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12252-6">
    <skos:broader rdf:resource="thsys/73392"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12252-6">
    <skos:broader rdf:resource="thsys/71045"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12252-6">
    <skos:broader rdf:resource="thsys/70899"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12252-6">
    <skos:broader rdf:resource="descriptor/12277-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15467-5">
    <skos:narrower rdf:resource="descriptor/15480-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19780-2">
    <skos:prefLabel xml:lang="en">Recreational fishing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19780-2">
    <skos:prefLabel xml:lang="de">Fischereisport</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12565-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14088-4">
    <skos:altLabel xml:lang="de">Dinkel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14436-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10135-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10504-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11972-5">
    <skos:scopeNote xml:lang="de">Ausgedehntes verstädtertes Gebiet, wobei mehrere Städte zu einem Großraum zusammengewachsen sind.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13794-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72394">
    <skos:prefLabel xml:lang="en">V.16.06  Multivariate analysis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72394">
    <skos:prefLabel xml:lang="de">V.16.06  Multivariate Analyse</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17496-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70056">
    <skos:broader rdf:resource="thsys/v"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17039-2">
    <skos:broader rdf:resource="thsys/70131"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17039-2">
    <skos:broader rdf:resource="descriptor/17016-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17039-2">
    <skos:broader rdf:resource="descriptor/17991-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17039-2">
    <skos:broader rdf:resource="descriptor/19178-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17039-2">
    <skos:broader rdf:resource="descriptor/17983-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70134">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19848-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16048-6">
    <skos:altLabel xml:lang="de">EG-Wirtschaftsrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12956-5">
    <skos:related rdf:resource="descriptor/18317-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12172-4">
    <skos:prefLabel xml:lang="en">Business information technology management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12172-4">
    <skos:prefLabel xml:lang="de">Wirtschaftsinformatik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70259">
    <skos:narrower rdf:resource="descriptor/13780-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70259">
    <skos:narrower rdf:resource="descriptor/12257-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70259">
    <skos:narrower rdf:resource="descriptor/18696-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70259">
    <skos:narrower rdf:resource="descriptor/11459-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70259">
    <skos:narrower rdf:resource="descriptor/18698-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70259">
    <skos:narrower rdf:resource="descriptor/11433-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70259">
    <skos:narrower rdf:resource="descriptor/12251-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70259">
    <skos:narrower rdf:resource="descriptor/12250-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70259">
    <skos:narrower rdf:resource="descriptor/10865-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70259">
    <skos:narrower rdf:resource="descriptor/13430-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11738-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16326-4">
    <skos:broader rdf:resource="thsys/73363"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16326-4">
    <skos:broader rdf:resource="thsys/70865"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16326-4">
    <skos:broader rdf:resource="descriptor/13412-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13485-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11963-6">
    <skos:prefLabel xml:lang="en">Shift-share analysis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11963-6">
    <skos:prefLabel xml:lang="de">Shift-Share-Analyse</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10361-4">
    <skos:broader rdf:resource="thsys/70878"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10361-4">
    <skos:broader rdf:resource="thsys/70020"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10361-4">
    <skos:broader rdf:resource="descriptor/10037-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10426-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12850-0">
    <skos:prefLabel xml:lang="en">Operating costs</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12850-0">
    <skos:prefLabel xml:lang="de">Betriebsausgaben</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19378-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70056">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id327">
    <rdf:value xml:lang="de">Für die Anbieter von Telekommunikationsdiensten</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13413-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13003-1">
    <skos:altLabel xml:lang="de">Wiederaufarbeitung von Kernbrennstoffen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13003-1">
    <skos:altLabel xml:lang="de">Urananreicherungsanlage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13003-1">
    <skos:altLabel xml:lang="de">Kernenergiewirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13003-1">
    <skos:altLabel xml:lang="de">Kernenergieindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13003-1">
    <skos:altLabel xml:lang="de">Kernbrennstoffwiederaufbereitung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13003-1">
    <skos:altLabel xml:lang="de">Kernbrennstoffwiederaufarbeitung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13003-1">
    <skos:altLabel xml:lang="de">Kernbrennstoffindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13003-1">
    <skos:altLabel xml:lang="de">Kernbrennstofffertigung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13003-1">
    <skos:altLabel xml:lang="de">Kernbrennstoffertigung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13003-1">
    <skos:altLabel xml:lang="de">Kernbrennstoffanreicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13003-1">
    <skos:altLabel xml:lang="de">Aufbereitungsanlage für Kernbrennstoffe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13003-1">
    <skos:altLabel xml:lang="de">Atomwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13003-1">
    <skos:altLabel xml:lang="de">Atomindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13003-1">
    <skos:altLabel xml:lang="de">Atomenergiewirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13003-1">
    <skos:altLabel xml:lang="de">Atomare Wiederaufbereitungsanlage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14686-5">
    <skos:prefLabel xml:lang="en">Phthalic acid</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14686-5">
    <skos:prefLabel xml:lang="de">Phthalsäure</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13860-6">
    <skos:broader rdf:resource="thsys/70888"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13860-6">
    <skos:broader rdf:resource="descriptor/15685-2"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id15">
    <rdfs:seeAlso rdf:resource="descriptor/12697-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19767-1">
    <skos:related rdf:resource="descriptor/19112-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19767-1">
    <skos:related rdf:resource="descriptor/11293-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19767-1">
    <skos:related rdf:resource="descriptor/11184-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19767-1">
    <skos:related rdf:resource="descriptor/11219-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19767-1">
    <skos:related rdf:resource="descriptor/11235-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19767-1">
    <skos:related rdf:resource="descriptor/19493-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19767-1">
    <skos:related rdf:resource="descriptor/11189-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19767-1">
    <skos:related rdf:resource="descriptor/11183-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12254-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10619-5">
    <skos:broader rdf:resource="thsys/73366"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10619-5">
    <skos:broader rdf:resource="thsys/71097"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10619-5">
    <skos:broader rdf:resource="descriptor/10987-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10619-5">
    <skos:broader rdf:resource="descriptor/16435-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12757-4">
    <skos:broader rdf:resource="thsys/70562"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12757-4">
    <skos:broader rdf:resource="thsys/70029"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12757-4">
    <skos:broader rdf:resource="descriptor/19562-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12757-4">
    <skos:broader rdf:resource="descriptor/12712-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16196-5">
    <skos:related rdf:resource="descriptor/19763-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16196-5">
    <skos:related rdf:resource="descriptor/16198-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16196-5">
    <skos:related rdf:resource="descriptor/15976-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16196-5">
    <skos:related rdf:resource="descriptor/19704-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16196-5">
    <skos:related rdf:resource="descriptor/11156-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16196-5">
    <skos:related rdf:resource="descriptor/16195-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16958-2">
    <skos:prefLabel xml:lang="en">Basle</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16958-2">
    <skos:prefLabel xml:lang="de">Basel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12922-1">
    <skos:related rdf:resource="descriptor/19301-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12922-1">
    <skos:related rdf:resource="descriptor/19588-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12922-1">
    <skos:related rdf:resource="descriptor/18382-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12922-1">
    <skos:related rdf:resource="descriptor/15752-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12922-1">
    <skos:related rdf:resource="descriptor/18995-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12922-1">
    <skos:related rdf:resource="descriptor/14047-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12922-1">
    <skos:related rdf:resource="descriptor/18381-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12922-1">
    <skos:related rdf:resource="descriptor/13324-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12922-1">
    <skos:related rdf:resource="descriptor/12648-2"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id14">
    <rdfs:seeAlso rdf:resource="descriptor/12915-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18752-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70937">
    <skos:narrower rdf:resource="descriptor/18585-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70937">
    <skos:narrower rdf:resource="descriptor/18584-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70937">
    <skos:narrower rdf:resource="descriptor/13244-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70937">
    <skos:narrower rdf:resource="descriptor/18583-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70937">
    <skos:narrower rdf:resource="descriptor/13250-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70937">
    <skos:narrower rdf:resource="descriptor/13630-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70937">
    <skos:narrower rdf:resource="descriptor/13558-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70937">
    <skos:narrower rdf:resource="descriptor/11667-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70937">
    <skos:narrower rdf:resource="descriptor/13813-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70937">
    <skos:narrower rdf:resource="descriptor/18423-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70937">
    <skos:narrower rdf:resource="descriptor/18582-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70937">
    <skos:narrower rdf:resource="descriptor/13248-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70937">
    <skos:narrower rdf:resource="descriptor/13555-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16332-2">
    <skos:scopeNote xml:lang="de">Geheimdienstliche Maßnahmen des Staates zur Abwehr innerer und äußerer Gefährdungen</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13270-6">
    <skos:broader rdf:resource="thsys/70923"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12401-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10622-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14788-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70160">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19052-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19335-2">
    <skos:broader rdf:resource="thsys/71094"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19335-2">
    <skos:broader rdf:resource="descriptor/10722-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19335-2">
    <skos:broader rdf:resource="descriptor/10746-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15340-1">
    <skos:narrower rdf:resource="descriptor/15336-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15340-1">
    <skos:narrower rdf:resource="descriptor/15362-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11824-6">
    <skos:narrower rdf:resource="descriptor/18524-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11824-6">
    <skos:narrower rdf:resource="descriptor/19355-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12404-2">
    <skos:prefLabel xml:lang="en">Inventories</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12404-2">
    <skos:prefLabel xml:lang="de">Vorratsvermögen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id312">
    <rdfs:seeAlso rdf:resource="descriptor/15064-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15677-1">
    <skos:prefLabel xml:lang="en">Process engineering</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15677-1">
    <skos:prefLabel xml:lang="de">Verfahrenstechnik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18429-5">
    <skos:altLabel xml:lang="de">Technische Chemie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18429-5">
    <skos:altLabel xml:lang="de">Chemische Verfahrenstechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18429-5">
    <skos:altLabel xml:lang="de">Chemietechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16441-4">
    <skos:related rdf:resource="descriptor/16436-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16441-4">
    <skos:related rdf:resource="descriptor/16209-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16441-4">
    <skos:related rdf:resource="descriptor/16215-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16441-4">
    <skos:related rdf:resource="descriptor/16186-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16441-4">
    <skos:related rdf:resource="descriptor/16435-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16441-4">
    <skos:related rdf:resource="descriptor/16502-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16441-4">
    <skos:related rdf:resource="descriptor/19716-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16566-5">
    <skos:broader rdf:resource="thsys/73355"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16566-5">
    <skos:broader rdf:resource="thsys/70171"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16566-5">
    <skos:broader rdf:resource="descriptor/16556-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19368-1">
    <skos:altLabel xml:lang="en">Rural development policy</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19368-1">
    <skos:altLabel xml:lang="de">Ländliches Entwicklungsprogramm</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19368-1">
    <skos:altLabel xml:lang="de">Ländliche Raumplanung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19368-1">
    <skos:altLabel xml:lang="de">Ländliche Raumordnungspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19368-1">
    <skos:altLabel xml:lang="de">Ländliche Entwicklungsstrategie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19368-1">
    <skos:altLabel xml:lang="de">Ländliche Entwicklungspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16663-0">
    <skos:altLabel xml:lang="de">Subkultur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16663-0">
    <skos:altLabel xml:lang="de">Squatters</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16663-0">
    <skos:altLabel xml:lang="de">Soziale Randgruppe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16663-0">
    <skos:altLabel xml:lang="de">Soziale Marginalisierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16663-0">
    <skos:altLabel xml:lang="de">Gesellschaftliche Außenseiter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16663-0">
    <skos:altLabel xml:lang="de">Außenseiter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11040-5">
    <skos:altLabel xml:lang="en">Ownership right</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11040-5">
    <skos:altLabel xml:lang="de">Eigentumsordnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11959-4">
    <skos:broader rdf:resource="thsys/71111"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11959-4">
    <skos:broader rdf:resource="thsys/71070"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11959-4">
    <skos:broader rdf:resource="descriptor/10243-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11057-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18774-2">
    <skos:prefLabel xml:lang="en">R&amp;amp;D subsidy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18774-2">
    <skos:prefLabel xml:lang="de">Forschungssubvention</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19035-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14737-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10177-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70160">
    <skos:prefLabel xml:lang="en">N.05.01  Public Law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70160">
    <skos:prefLabel xml:lang="de">N.05.01  Öffentliches Recht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19258-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13689-4">
    <skos:broader rdf:resource="thsys/72181"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13689-4">
    <skos:broader rdf:resource="thsys/70905"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13689-4">
    <skos:broader rdf:resource="descriptor/15076-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24624-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17914-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14958-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12515-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14014-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18434-5">
    <skos:altLabel xml:lang="de">Reproduktionstechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18434-5">
    <skos:altLabel xml:lang="de">Drucken</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16031-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19153-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19603-3">
    <skos:related rdf:resource="descriptor/11373-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70504">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#semanticRelation">
    <skos:definition xml:lang="en">A concept related by meaning.</skos:definition>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19284-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10866-2">
    <skos:related rdf:resource="descriptor/10865-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10866-2">
    <skos:related rdf:resource="descriptor/10693-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10866-2">
    <skos:related rdf:resource="descriptor/10836-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10866-2">
    <skos:related rdf:resource="descriptor/10858-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10866-2">
    <skos:related rdf:resource="descriptor/10684-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10614-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="../stw">
    <skos:hasTopConcept rdf:resource="thsys/w"/>
  </rdf:Description>
  <rdf:Description rdf:about="../stw">
    <skos:hasTopConcept rdf:resource="thsys/v"/>
  </rdf:Description>
  <rdf:Description rdf:about="../stw">
    <skos:hasTopConcept rdf:resource="thsys/p"/>
  </rdf:Description>
  <rdf:Description rdf:about="../stw">
    <skos:hasTopConcept rdf:resource="thsys/n"/>
  </rdf:Description>
  <rdf:Description rdf:about="../stw">
    <skos:hasTopConcept rdf:resource="thsys/g"/>
  </rdf:Description>
  <rdf:Description rdf:about="../stw">
    <skos:hasTopConcept rdf:resource="thsys/b"/>
  </rdf:Description>
  <rdf:Description rdf:about="../stw">
    <skos:hasTopConcept rdf:resource="thsys/a"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18508-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11906-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18249-0">
    <skos:prefLabel xml:lang="en">Fax machine</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18249-0">
    <skos:prefLabel xml:lang="de">Telefaxgerät</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18212-2">
    <skos:broader rdf:resource="thsys/70265"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18212-2">
    <skos:broader rdf:resource="descriptor/18028-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19740-0">
    <skos:altLabel xml:lang="de">Puritanismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19740-0">
    <skos:altLabel xml:lang="de">Pietismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19740-0">
    <skos:altLabel xml:lang="de">Lutheraner</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19740-0">
    <skos:altLabel xml:lang="de">Hugenotten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19740-0">
    <skos:altLabel xml:lang="de">Calvinismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15922-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10682-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13464-0">
    <skos:broader rdf:resource="thsys/73391"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13464-0">
    <skos:broader rdf:resource="thsys/70963"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13464-0">
    <skos:broader rdf:resource="descriptor/16042-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10817-1">
    <skos:prefLabel xml:lang="en">Offshore financial centre</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10817-1">
    <skos:prefLabel xml:lang="de">Offshore-Finanzmarkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73362">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11322-2">
    <skos:related rdf:resource="descriptor/11518-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11322-2">
    <skos:related rdf:resource="descriptor/11336-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11322-2">
    <skos:related rdf:resource="descriptor/11327-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11322-2">
    <skos:related rdf:resource="descriptor/11669-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11322-2">
    <skos:related rdf:resource="descriptor/15089-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11322-2">
    <skos:related rdf:resource="descriptor/11326-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11322-2">
    <skos:related rdf:resource="descriptor/11345-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11322-2">
    <skos:related rdf:resource="descriptor/11325-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11322-2">
    <skos:related rdf:resource="descriptor/16143-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11322-2">
    <skos:related rdf:resource="descriptor/19966-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11322-2">
    <skos:related rdf:resource="descriptor/11180-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19685-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19134-5">
    <skos:broader rdf:resource="thsys/70384"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14094-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19667-5">
    <skos:related rdf:resource="descriptor/19432-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19667-5">
    <skos:related rdf:resource="descriptor/10928-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19667-5">
    <skos:related rdf:resource="descriptor/10012-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19667-5">
    <skos:related rdf:resource="descriptor/12770-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19667-5">
    <skos:related rdf:resource="descriptor/12058-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19667-5">
    <skos:related rdf:resource="descriptor/19159-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19784-1">
    <skos:related rdf:resource="descriptor/16214-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19784-1">
    <skos:related rdf:resource="descriptor/15396-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19784-1">
    <skos:related rdf:resource="descriptor/10148-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19784-1">
    <skos:related rdf:resource="descriptor/19202-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19784-1">
    <skos:related rdf:resource="descriptor/11260-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19784-1">
    <skos:related rdf:resource="descriptor/12057-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19784-1">
    <skos:related rdf:resource="descriptor/19761-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19784-1">
    <skos:related rdf:resource="descriptor/15690-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19784-1">
    <skos:related rdf:resource="descriptor/12765-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11971-0">
    <skos:prefLabel xml:lang="en">Urban population</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11971-0">
    <skos:prefLabel xml:lang="de">Stadtbevölkerung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19713-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14029-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13018-2">
    <skos:prefLabel xml:lang="en">Quarrying</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13018-2">
    <skos:prefLabel xml:lang="de">Gewinnung von Steinen und Erden</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19431-6">
    <skos:prefLabel xml:lang="en">Network externalities</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19431-6">
    <skos:prefLabel xml:lang="de">Network Externalities</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70838">
    <skos:broader rdf:resource="thsys/70839"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13322-6">
    <skos:altLabel xml:lang="de">Versteigerung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13322-6">
    <skos:altLabel xml:lang="de">Auktionshaus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12951-1">
    <skos:broader rdf:resource="thsys/71029"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12951-1">
    <skos:broader rdf:resource="descriptor/12933-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19449-1">
    <skos:related rdf:resource="descriptor/18623-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73303">
    <skos:narrower rdf:resource="descriptor/14995-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73303">
    <skos:narrower rdf:resource="descriptor/14955-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73303">
    <skos:narrower rdf:resource="descriptor/14124-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73303">
    <skos:narrower rdf:resource="descriptor/14954-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73303">
    <skos:narrower rdf:resource="descriptor/14953-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73303">
    <skos:narrower rdf:resource="descriptor/18217-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19079-1">
    <skos:related rdf:resource="descriptor/19043-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19079-1">
    <skos:related rdf:resource="descriptor/13727-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18679-3">
    <skos:scopeNote xml:lang="de">Maß für Häufigkeit und Intensität von Preisschwankungen (Kurse und Zinssätze) eines Basiswertes.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11711-5">
    <skos:broader rdf:resource="thsys/71083"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11711-5">
    <skos:broader rdf:resource="descriptor/11715-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70188">
    <rdfs:label xml:lang="en">Equilibrium model</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70188">
    <rdfs:label xml:lang="de">Gleichgewichtstheorie</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11068-4">
    <skos:altLabel xml:lang="de">Wirtschaftssystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11068-4">
    <skos:altLabel xml:lang="de">Wirtschaftsform</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11068-4">
    <skos:altLabel xml:lang="de">Produktionsverhältnisse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10889-4">
    <skos:related rdf:resource="descriptor/12540-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10889-4">
    <skos:related rdf:resource="descriptor/10931-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10889-4">
    <skos:related rdf:resource="descriptor/11543-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16696-6">
    <skos:prefLabel xml:lang="en">Technology assessment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16696-6">
    <skos:prefLabel xml:lang="de">Technikbewertung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11449-6">
    <skos:prefLabel xml:lang="en">Demand for money model</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11449-6">
    <skos:prefLabel xml:lang="de">Geldnachfragetheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18734-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14266-6">
    <skos:prefLabel xml:lang="en">Metal</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14266-6">
    <skos:prefLabel xml:lang="de">Metall</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14906-3">
    <skos:prefLabel xml:lang="en">Men's wear</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14906-3">
    <skos:prefLabel xml:lang="de">Herrenbekleidung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19145-0">
    <skos:scopeNote xml:lang="de">Betriebsinternes Netz mit den Funktionen des Internet.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11892-3">
    <skos:altLabel xml:lang="de">Stadtneugründung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11892-3">
    <skos:altLabel xml:lang="de">New-Towns-Konzept</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11892-3">
    <skos:altLabel xml:lang="de">Neue Städte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11892-3">
    <skos:altLabel xml:lang="de">Neue Stadt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13245-5">
    <skos:altLabel xml:lang="de">Rohrleitungstransport</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13245-5">
    <skos:altLabel xml:lang="de">Pipelinetransportindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13245-5">
    <skos:altLabel xml:lang="de">Pipelinetransport</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13245-5">
    <skos:altLabel xml:lang="de">Leitungstransportindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13245-5">
    <skos:altLabel xml:lang="de">Fernleitungstransport</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14444-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70582">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15958-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73386">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.07.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17784-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19765-5">
    <skos:prefLabel xml:lang="en">Social acceptability</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19765-5">
    <skos:prefLabel xml:lang="de">Sozialverträglichkeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18082-3">
    <skos:prefLabel xml:lang="en">Diffusion of innovations</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18082-3">
    <skos:prefLabel xml:lang="de">Innovationsdiffusion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18054-1">
    <skos:altLabel xml:lang="en">Mediterranean countries</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18054-1">
    <skos:altLabel xml:lang="de">Mittelmeerstaaten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18054-1">
    <skos:altLabel xml:lang="de">Mittelmeerländer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13577-1">
    <skos:related rdf:resource="descriptor/13573-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13577-1">
    <skos:related rdf:resource="descriptor/13570-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13577-1">
    <skos:related rdf:resource="descriptor/13510-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13577-1">
    <skos:related rdf:resource="descriptor/13682-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13577-1">
    <skos:related rdf:resource="descriptor/18412-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13577-1">
    <skos:related rdf:resource="descriptor/18353-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16159-4">
    <skos:altLabel xml:lang="de">Sozialgesetzgebung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16159-4">
    <skos:altLabel xml:lang="de">Sozialgesetzbuch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16159-4">
    <skos:altLabel xml:lang="de">SGB (Sozialgesetzbuch)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10097-2">
    <skos:related rdf:resource="descriptor/10112-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10097-2">
    <skos:related rdf:resource="descriptor/15767-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10097-2">
    <skos:related rdf:resource="descriptor/10102-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10097-2">
    <skos:related rdf:resource="descriptor/10094-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13192-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13299-3">
    <skos:related rdf:resource="descriptor/13621-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13299-3">
    <skos:related rdf:resource="descriptor/13565-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16463-1">
    <skos:broader rdf:resource="thsys/73365"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16463-1">
    <skos:broader rdf:resource="descriptor/16273-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12198-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16650-2">
    <skos:altLabel xml:lang="de">Selbsthilfegruppe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16650-2">
    <skos:altLabel xml:lang="de">Gegenseitige Hilfeleistung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70833">
    <skos:prefLabel xml:lang="en">W.26.02  Social policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70833">
    <skos:prefLabel xml:lang="de">W.26.02  Sozialpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11667-3">
    <skos:prefLabel xml:lang="en">Motor vehicle tax</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11667-3">
    <skos:prefLabel xml:lang="de">Kraftfahrzeugsteuer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19320-1">
    <skos:broader rdf:resource="thsys/71098"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19320-1">
    <skos:broader rdf:resource="descriptor/10686-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19535-1">
    <skos:scopeNote xml:lang="de">Kauf der eigenen, in Umlauf befindlichen Aktien.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10652-0">
    <skos:altLabel xml:lang="en">Import elasticity</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10652-0">
    <skos:altLabel xml:lang="en">Export elasticity</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10652-0">
    <skos:altLabel xml:lang="de">Importelastizität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10652-0">
    <skos:altLabel xml:lang="de">Exportelastizität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10652-0">
    <skos:altLabel xml:lang="de">Einfuhrelastizität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10652-0">
    <skos:altLabel xml:lang="de">Ausfuhrelastizität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10652-0">
    <skos:altLabel xml:lang="de">Außenwirtschaftselastizität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11555-0">
    <skos:narrower rdf:resource="descriptor/19518-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70994">
    <skos:prefLabel xml:lang="en">W.06.01.06  Chemical industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70994">
    <skos:prefLabel xml:lang="de">W.06.01.06  Chemieindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18840-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71090">
    <skos:prefLabel xml:lang="de">V.07.06  Multinationale Unternehmen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15111-2">
    <skos:broader rdf:resource="thsys/72183"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24663-4">
    <skos:broader rdf:resource="thsys/73370"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19197-2">
    <skos:related rdf:resource="descriptor/19200-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19197-2">
    <skos:related rdf:resource="descriptor/10171-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10081-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10595-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14631-2">
    <skos:related rdf:resource="descriptor/18268-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14631-2">
    <skos:related rdf:resource="descriptor/18079-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14432-1">
    <skos:prefLabel xml:lang="en">Bicycle</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14432-1">
    <skos:prefLabel xml:lang="de">Fahrrad</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13278-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id197">
    <rdfs:seeAlso rdf:resource="descriptor/11320-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12381-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70923">
    <skos:prefLabel xml:lang="en">W.13  Postal and telecommunications services</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70923">
    <skos:prefLabel xml:lang="de">W.13  Post und Telekommunikation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12209-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15616-0">
    <skos:broader rdf:resource="thsys/73341"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16732-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12880-5">
    <skos:broader rdf:resource="thsys/73337"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12880-5">
    <skos:broader rdf:resource="thsys/71041"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12880-5">
    <skos:broader rdf:resource="descriptor/12882-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15899-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12138-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18911-4">
    <skos:altLabel xml:lang="de">Erziehungsgeld</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18911-4">
    <skos:altLabel xml:lang="de">Elternzeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18911-4">
    <skos:altLabel xml:lang="de">Bundeserziehungsgeldgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12153-1">
    <skos:broader rdf:resource="thsys/70562"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12153-1">
    <skos:broader rdf:resource="descriptor/12111-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13574-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12891-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18789-3">
    <skos:altLabel xml:lang="de">Technisches Büro</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18789-3">
    <skos:altLabel xml:lang="de">Technische Untersuchungsstelle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18789-3">
    <skos:altLabel xml:lang="de">Technische Untersuchung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18789-3">
    <skos:altLabel xml:lang="de">Technische Prüfung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18789-3">
    <skos:altLabel xml:lang="de">Technische Prüfstelle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18789-3">
    <skos:altLabel xml:lang="de">Technische Gutachterinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18789-3">
    <skos:altLabel xml:lang="de">Technische Gutachter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13617-1">
    <skos:prefLabel xml:lang="en">Transport of dangerous goods</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13617-1">
    <skos:prefLabel xml:lang="de">Gefahrguttransport</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18639-1">
    <skos:prefLabel xml:lang="en">Tourism marketing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18639-1">
    <skos:prefLabel xml:lang="de">Tourismusmarketing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12810-5">
    <skos:prefLabel xml:lang="en">Product differentiation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12810-5">
    <skos:prefLabel xml:lang="de">Produktdifferenzierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14465-0">
    <skos:broader rdf:resource="thsys/70135"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14465-0">
    <skos:broader rdf:resource="descriptor/14473-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13424-5">
    <skos:prefLabel xml:lang="en">Joint authority</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13424-5">
    <skos:prefLabel xml:lang="de">Zweckverband</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10950-6">
    <skos:altLabel xml:lang="en">SCP-model</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10950-6">
    <skos:altLabel xml:lang="en">Market structure and performance</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10950-6">
    <skos:altLabel xml:lang="de">Marktstruktur-Marktverhalten-Marktergebnis-Modell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13690-5">
    <skos:broader rdf:resource="thsys/70903"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71040">
    <skos:prefLabel xml:lang="de">B.08.03  Besteuerung, Rechtsform- und Standortwahl</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14805-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10595-0">
    <skos:scopeNote xml:lang="de">Nur für Selbsthilfe in Entwicklungsländern.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14195-3">
    <skos:prefLabel xml:lang="en">Iron ore</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14195-3">
    <skos:prefLabel xml:lang="de">Eisenerz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15764-6">
    <skos:prefLabel xml:lang="en">Environmental consciousness</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15764-6">
    <skos:prefLabel xml:lang="de">Umweltbewusstsein</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19434-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18937-0">
    <skos:broader rdf:resource="thsys/70824"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18937-0">
    <skos:broader rdf:resource="descriptor/13665-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18258-6">
    <skos:prefLabel xml:lang="en">OTC pharmaceutical</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18258-6">
    <skos:prefLabel xml:lang="de">Rezeptfreies Arzneimittel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11589-4">
    <skos:related rdf:resource="descriptor/11596-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11589-4">
    <skos:related rdf:resource="descriptor/11539-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11589-4">
    <skos:related rdf:resource="descriptor/12307-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14735-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14927-2">
    <skos:prefLabel xml:lang="en">Ready meal</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14927-2">
    <skos:prefLabel xml:lang="de">Fertiggericht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18236-2">
    <skos:broader rdf:resource="thsys/70418"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18236-2">
    <skos:broader rdf:resource="descriptor/14196-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13420-6">
    <skos:editorialNote xml:lang="de">Möglichst spezifischer indexieren.</skos:editorialNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19588-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19585-0">
    <skos:related rdf:resource="descriptor/19588-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19585-0">
    <skos:related rdf:resource="descriptor/12961-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19585-0">
    <skos:related rdf:resource="descriptor/13433-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14854-3">
    <skos:related rdf:resource="descriptor/14702-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14854-3">
    <skos:related rdf:resource="descriptor/12994-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15411-4">
    <skos:related rdf:resource="descriptor/10139-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18441-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11194-6">
    <skos:prefLabel xml:lang="en">Leisure</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11194-6">
    <skos:prefLabel xml:lang="de">Freizeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71006">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.05.01.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14396-0">
    <skos:broader rdf:resource="thsys/70549"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14396-0">
    <skos:broader rdf:resource="descriptor/12693-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70003">
    <skos:prefLabel xml:lang="en">G.01.01.03  Switzerland</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70003">
    <skos:prefLabel xml:lang="de">G.01.01.03  Schweiz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18869-5">
    <skos:broader rdf:resource="thsys/71109"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18869-5">
    <skos:broader rdf:resource="thsys/71084"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18869-5">
    <skos:broader rdf:resource="thsys/70550"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16692-0">
    <skos:altLabel xml:lang="de">Betriebssoziologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16692-0">
    <skos:altLabel xml:lang="de">Arbeitssoziologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17665-5">
    <skos:broader rdf:resource="thsys/70445"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17665-5">
    <skos:broader rdf:resource="descriptor/17672-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10171-2">
    <skos:broader rdf:resource="thsys/70493"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10171-2">
    <skos:broader rdf:resource="descriptor/10177-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10171-2">
    <skos:broader rdf:resource="descriptor/10036-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19669-1">
    <skos:broader rdf:resource="thsys/70893"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19669-1">
    <skos:broader rdf:resource="thsys/70187"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19858-5">
    <skos:scopeNote xml:lang="de">Regelungen zur Beschränkung von Herstellung, Erwerb, Besitz und Führen von Waffen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11900-2">
    <skos:narrower rdf:resource="descriptor/19449-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11900-2">
    <skos:narrower rdf:resource="descriptor/19368-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11900-2">
    <skos:narrower rdf:resource="descriptor/11998-1"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id121">
    <rdfs:seeAlso rdf:resource="descriptor/14932-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10382-3">
    <skos:broader rdf:resource="thsys/73372"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10382-3">
    <skos:broader rdf:resource="thsys/70904"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10382-3">
    <skos:broader rdf:resource="thsys/70305"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10382-3">
    <skos:broader rdf:resource="descriptor/10987-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15119-0">
    <zbwext:useInsteadNote rdf:nodeID="id190"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18401-6">
    <skos:prefLabel xml:lang="en">Metals science</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18401-6">
    <skos:prefLabel xml:lang="de">Metallkunde</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18912-2">
    <skos:related rdf:resource="descriptor/11573-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18912-2">
    <skos:related rdf:resource="descriptor/11566-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18912-2">
    <skos:related rdf:resource="descriptor/11051-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18912-2">
    <skos:related rdf:resource="descriptor/11565-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18912-2">
    <skos:related rdf:resource="descriptor/19722-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18912-2">
    <skos:related rdf:resource="descriptor/16470-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18912-2">
    <skos:related rdf:resource="descriptor/16524-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18912-2">
    <skos:related rdf:resource="descriptor/18107-6"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id87">
    <rdfs:seeAlso rdf:resource="descriptor/10555-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19487-0">
    <skos:prefLabel xml:lang="en">Energy substitution</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19487-0">
    <skos:prefLabel xml:lang="de">Energiesubstitution</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16737-4">
    <skos:prefLabel xml:lang="en">Cambridge School</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16737-4">
    <skos:prefLabel xml:lang="de">Cambridger Schule</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15518-0">
    <skos:scopeNote xml:lang="de">Theorie und Anwendung optimaler Routenplanungen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19385-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13488-0">
    <skos:related rdf:resource="descriptor/18166-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13488-0">
    <skos:related rdf:resource="descriptor/18545-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13488-0">
    <skos:related rdf:resource="descriptor/13200-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13488-0">
    <skos:related rdf:resource="descriptor/18013-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13488-0">
    <skos:related rdf:resource="descriptor/18821-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70082">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11751-0">
    <skos:prefLabel xml:lang="en">Wealth distribution</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11751-0">
    <skos:prefLabel xml:lang="de">Vermögensverteilung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12936-4">
    <skos:prefLabel xml:lang="en">Upland agriculture</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12936-4">
    <skos:prefLabel xml:lang="de">Bergbauernwirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73344">
    <skos:prefLabel xml:lang="de">N.10.06.01  Schlüsseltechnologien</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10241-0">
    <skos:prefLabel xml:lang="en">Aggregate supply</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10241-0">
    <skos:prefLabel xml:lang="de">Gesamtwirtschaftliches Angebot</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18570-4">
    <skos:broader rdf:resource="thsys/70941"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18570-4">
    <skos:broader rdf:resource="thsys/70930"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18570-4">
    <skos:broader rdf:resource="descriptor/18573-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18570-4">
    <skos:broader rdf:resource="descriptor/13615-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14032-3">
    <skos:scopeNote xml:lang="de">Haushaltswaren sind keine inhaltlich genau abgegrenzte Gütergruppe.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19670-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18424-1">
    <skos:prefLabel xml:lang="en">Electronics industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18424-1">
    <skos:prefLabel xml:lang="de">Elektronikindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16338-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15472-5">
    <skos:prefLabel xml:lang="en">Group decision</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15472-5">
    <skos:prefLabel xml:lang="de">Gruppenentscheidung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70386">
    <rdfs:label xml:lang="en">Inplant transportation</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70386">
    <rdfs:label xml:lang="de">Betriebliches Transportwesen</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13291-5">
    <skos:altLabel xml:lang="de">Wertpapiersammelbank</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13291-5">
    <skos:altLabel xml:lang="de">Trennbankensystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13291-5">
    <skos:altLabel xml:lang="de">Teilzahlungsbank</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13291-5">
    <skos:altLabel xml:lang="de">Spezialbankprinzip</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13291-5">
    <skos:altLabel xml:lang="de">Realkreditinstitut</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13291-5">
    <skos:altLabel xml:lang="de">Kassenverein</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13291-5">
    <skos:altLabel xml:lang="de">Grundkreditanstalt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11207-2">
    <skos:altLabel xml:lang="en">Regional labor market</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11682-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18290-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16853-2">
    <skos:broader rdf:resource="thsys/70361"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16853-2">
    <skos:broader rdf:resource="descriptor/18012-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18988-4">
    <skos:prefLabel xml:lang="en">Forest economics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18988-4">
    <skos:prefLabel xml:lang="de">Forstökonomie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13723-2">
    <skos:narrower rdf:resource="descriptor/13738-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13723-2">
    <skos:narrower rdf:resource="descriptor/19272-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13723-2">
    <skos:narrower rdf:resource="descriptor/19273-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13723-2">
    <skos:narrower rdf:resource="descriptor/11433-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13723-2">
    <skos:narrower rdf:resource="descriptor/10807-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13723-2">
    <skos:narrower rdf:resource="descriptor/11421-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13723-2">
    <skos:narrower rdf:resource="descriptor/13713-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14171-3">
    <skos:prefLabel xml:lang="en">Bioenergy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14171-3">
    <skos:prefLabel xml:lang="de">Bioenergie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15744-5">
    <skos:altLabel xml:lang="de">Ölpest</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15744-5">
    <skos:altLabel xml:lang="de">Meeresverunreinigung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13949-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19571-4">
    <skos:broader rdf:resource="thsys/72185"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19571-4">
    <skos:broader rdf:resource="descriptor/15225-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18105-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73344">
    <skos:broader rdf:resource="thsys/73345"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20314-0">
    <skos:broader rdf:resource="thsys/70823"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20314-0">
    <skos:broader rdf:resource="thsys/70550"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20314-0">
    <skos:broader rdf:resource="descriptor/16754-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18443-4">
    <skos:related rdf:resource="descriptor/18413-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18443-4">
    <skos:related rdf:resource="descriptor/14384-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18443-4">
    <skos:related rdf:resource="descriptor/13160-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18443-4">
    <skos:related rdf:resource="descriptor/14931-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18443-4">
    <skos:related rdf:resource="descriptor/18304-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18443-4">
    <skos:related rdf:resource="descriptor/18442-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18443-4">
    <skos:related rdf:resource="descriptor/14611-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18443-4">
    <skos:related rdf:resource="descriptor/18289-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18443-4">
    <skos:related rdf:resource="descriptor/13450-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12712-5">
    <skos:altLabel xml:lang="en">Operations management</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12712-5">
    <skos:altLabel xml:lang="de">Produktionsmanagement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12712-5">
    <skos:altLabel xml:lang="de">Fertigungswirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12712-5">
    <skos:altLabel xml:lang="de">Betriebliche Produktionswirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18131-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19849-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19167-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15466-0">
    <skos:related rdf:resource="descriptor/18859-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15466-0">
    <skos:related rdf:resource="descriptor/12079-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15466-0">
    <skos:related rdf:resource="descriptor/19601-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15466-0">
    <skos:related rdf:resource="descriptor/15470-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15466-0">
    <skos:related rdf:resource="descriptor/10089-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13804-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10312-3">
    <skos:prefLabel xml:lang="en">Law of diminishing returns</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10312-3">
    <skos:prefLabel xml:lang="de">Ertragsgesetz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70905">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13622-1">
    <skos:related rdf:resource="descriptor/13587-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13622-1">
    <skos:related rdf:resource="descriptor/13611-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13622-1">
    <skos:related rdf:resource="descriptor/18620-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13622-1">
    <skos:related rdf:resource="descriptor/18617-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13622-1">
    <skos:related rdf:resource="descriptor/13620-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19327-1">
    <skos:prefLabel xml:lang="en">Economic conference</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19327-1">
    <skos:prefLabel xml:lang="de">Wirtschaftskonferenz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id49">
    <rdf:value xml:lang="de">Für arbeitsmarktpolitische Maßnahmen</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18085-4">
    <skos:broader rdf:resource="thsys/70905"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18085-4">
    <skos:broader rdf:resource="descriptor/13301-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14463-4">
    <skos:related rdf:resource="descriptor/13066-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14463-4">
    <skos:related rdf:resource="descriptor/14453-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13210-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18593-6">
    <skos:prefLabel xml:lang="en">Ship safety</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18593-6">
    <skos:prefLabel xml:lang="de">Schiffssicherheit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21002-0">
    <skos:scopeNote xml:lang="de">Kostenanteil des versicherten Schadens, der von den Versicherten selbst zu tragen ist.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10225-5">
    <skos:prefLabel xml:lang="en">Complete information</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10225-5">
    <skos:prefLabel xml:lang="de">Vollkommene Information</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15073-5">
    <skos:broader rdf:resource="thsys/70504"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15073-5">
    <skos:broader rdf:resource="thsys/70012"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15073-5">
    <skos:broader rdf:resource="descriptor/18887-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19460-6">
    <skos:related rdf:resource="descriptor/19243-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19460-6">
    <skos:related rdf:resource="descriptor/18982-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19460-6">
    <skos:related rdf:resource="descriptor/10503-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19460-6">
    <skos:related rdf:resource="descriptor/19273-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19460-6">
    <skos:related rdf:resource="descriptor/19462-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19460-6">
    <skos:related rdf:resource="descriptor/10575-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16656-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19622-6">
    <skos:broader rdf:resource="thsys/73352"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19622-6">
    <skos:broader rdf:resource="descriptor/15880-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18726-6">
    <skos:broader rdf:resource="thsys/73391"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18726-6">
    <skos:broader rdf:resource="thsys/70485"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18726-6">
    <skos:broader rdf:resource="descriptor/16050-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16305-5">
    <skos:altLabel xml:lang="de">Regierungssystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16305-5">
    <skos:altLabel xml:lang="de">Regierungsform</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19666-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19099-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18116-5">
    <skos:altLabel xml:lang="de">Betriebsethik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11067-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21011-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18103-0">
    <skos:related rdf:resource="descriptor/18229-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18103-0">
    <skos:related rdf:resource="descriptor/19396-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18103-0">
    <skos:related rdf:resource="descriptor/18424-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17801-2">
    <skos:altLabel xml:lang="en">Saint Vincent and the Grenadines</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17801-2">
    <skos:altLabel xml:lang="de">St. Vincent</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17801-2">
    <skos:altLabel xml:lang="de">Nordgrenadinen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19720-6">
    <skos:prefLabel xml:lang="en">Military policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19720-6">
    <skos:prefLabel xml:lang="de">Militärpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16042-4">
    <skos:altLabel xml:lang="de">Zivilrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13630-2">
    <skos:altLabel xml:lang="de">Pkw-Verkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13630-2">
    <skos:altLabel xml:lang="de">Motorradverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13630-2">
    <skos:altLabel xml:lang="de">MIV (Motorisierter Individualverkehr)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13364-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15399-3">
    <skos:prefLabel xml:lang="en">Factor analysis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15399-3">
    <skos:prefLabel xml:lang="de">Faktorenanalyse</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13282-6">
    <skos:broader rdf:resource="thsys/71054"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13282-6">
    <skos:broader rdf:resource="thsys/70892"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13282-6">
    <skos:broader rdf:resource="descriptor/13969-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14543-6">
    <skos:broader rdf:resource="thsys/70423"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13256-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id221">
    <rdfs:seeAlso rdf:resource="descriptor/11633-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11642-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#topConceptOf">
    <owl:inverseOf rdf:resource="http://www.w3.org/2004/02/skos/core#hasTopConcept"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18444-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13397-3">
    <skos:related rdf:resource="descriptor/19694-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13397-3">
    <skos:related rdf:resource="descriptor/13989-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13397-3">
    <skos:related rdf:resource="descriptor/19696-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13397-3">
    <skos:related rdf:resource="descriptor/19695-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13397-3">
    <skos:related rdf:resource="descriptor/16366-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13397-3">
    <skos:related rdf:resource="descriptor/16008-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13397-3">
    <skos:related rdf:resource="descriptor/16344-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13397-3">
    <skos:related rdf:resource="descriptor/16630-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13397-3">
    <skos:related rdf:resource="descriptor/16405-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18406-3">
    <skos:prefLabel xml:lang="en">Steelmaking</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18406-3">
    <skos:prefLabel xml:lang="de">Stahltechnik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16544-1">
    <skos:broader rdf:resource="thsys/73394"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16544-1">
    <skos:broader rdf:resource="thsys/70835"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16544-1">
    <skos:broader rdf:resource="thsys/70504"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16544-1">
    <skos:broader rdf:resource="descriptor/16709-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10036-1">
    <skos:scopeNote xml:lang="de">Wirkung einer unabhängigen Variablen auf eine abhängige Variable.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12941-4">
    <skos:related rdf:resource="descriptor/18218-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12941-4">
    <skos:related rdf:resource="descriptor/14135-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12941-4">
    <skos:related rdf:resource="descriptor/14121-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14534-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14220-2">
    <skos:broader rdf:resource="thsys/70650"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14220-2">
    <skos:broader rdf:resource="thsys/70134"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14220-2">
    <skos:broader rdf:resource="descriptor/14302-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14220-2">
    <skos:broader rdf:resource="descriptor/14181-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12064-0">
    <skos:prefLabel xml:lang="en">Private sector</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12064-0">
    <skos:prefLabel xml:lang="de">Privatwirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15720-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17977-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18747-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20967-4">
    <skos:prefLabel xml:lang="en">Joint implementation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20967-4">
    <skos:prefLabel xml:lang="de">Joint Implementation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20826-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10339-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#broader">
    <rdfs:isDefinedBy rdf:resource="http://www.w3.org/2004/02/skos/core"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19621-1">
    <skos:broader rdf:resource="thsys/73352"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70057">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70902">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.14.02.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17675-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13378-0">
    <skos:broader rdf:resource="thsys/70876"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11739-4">
    <skos:broader rdf:resource="thsys/71114"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11739-4">
    <skos:broader rdf:resource="descriptor/11735-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18238-5">
    <skos:prefLabel xml:lang="en">Molybdenum</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18238-5">
    <skos:prefLabel xml:lang="de">Molybdän</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18676-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19352-2">
    <skos:altLabel xml:lang="de">EU-Beitritt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10668-6">
    <skos:prefLabel xml:lang="en">Law of one price</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10668-6">
    <skos:prefLabel xml:lang="de">Internationaler Preiszusammenhang</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10126-0">
    <skos:broader rdf:resource="thsys/70486"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10126-0">
    <skos:broader rdf:resource="descriptor/10249-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19701-3">
    <skos:broader rdf:resource="thsys/73376"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19701-3">
    <skos:broader rdf:resource="descriptor/11857-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12198-0">
    <skos:prefLabel xml:lang="en">Dormant partnership</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12198-0">
    <skos:prefLabel xml:lang="de">Stille Gesellschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14646-3">
    <skos:prefLabel xml:lang="en">Phosphate</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14646-3">
    <skos:prefLabel xml:lang="de">Phosphat</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18318-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16566-5">
    <skos:prefLabel xml:lang="en">Economic anthropology</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16566-5">
    <skos:prefLabel xml:lang="de">Wirtschaftsethnologie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13511-3">
    <skos:scopeNote xml:lang="de">Geschichtliche Darstellungen des Verkehrswesens allgemein oder in bestimmten Regionen bzw. Literatur zur Verkehrsgeschichte als Wissenschaftsdisziplin.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71026">
    <skos:narrower rdf:resource="descriptor/12960-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71026">
    <skos:narrower rdf:resource="descriptor/19348-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71026">
    <skos:narrower rdf:resource="descriptor/12958-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71026">
    <skos:narrower rdf:resource="descriptor/19342-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71026">
    <skos:narrower rdf:resource="descriptor/12947-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71026">
    <skos:narrower rdf:resource="descriptor/19341-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71026">
    <skos:narrower rdf:resource="descriptor/12942-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71026">
    <skos:narrower rdf:resource="descriptor/12940-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71026">
    <skos:narrower rdf:resource="descriptor/12935-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16841-2">
    <skos:broader rdf:resource="thsys/70361"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16841-2">
    <skos:broader rdf:resource="descriptor/18012-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15976-5">
    <skos:broader rdf:resource="thsys/70154"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15976-5">
    <skos:broader rdf:resource="descriptor/16563-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13153-3">
    <skos:related rdf:resource="descriptor/14958-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13153-3">
    <skos:related rdf:resource="descriptor/19393-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14052-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18666-5">
    <skos:prefLabel xml:lang="en">Mobile communications</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18666-5">
    <skos:prefLabel xml:lang="de">Mobilkommunikation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19342-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14135-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11678-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10080-5">
    <skos:broader rdf:resource="thsys/70188"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10080-5">
    <skos:broader rdf:resource="descriptor/10087-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15954-1">
    <skos:altLabel xml:lang="de">Zwischenstaatliche Wanderung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15954-1">
    <skos:altLabel xml:lang="de">Migration</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15954-1">
    <skos:altLabel xml:lang="de">Internationale Migration</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/b">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">B</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18864-1">
    <skos:prefLabel xml:lang="en">Child care</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18864-1">
    <skos:prefLabel xml:lang="de">Kinderbetreuung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18733-2">
    <skos:prefLabel xml:lang="en">Market microstructure</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18733-2">
    <skos:prefLabel xml:lang="de">Mikrostrukturanalyse</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18700-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id199">
    <rdfs:seeAlso rdf:resource="descriptor/18104-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13980-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11652-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19092-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18509-0">
    <skos:broader rdf:resource="thsys/71057"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18509-0">
    <skos:broader rdf:resource="thsys/70965"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18509-0">
    <skos:broader rdf:resource="descriptor/11294-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19705-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14237-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19799-2">
    <skos:broader rdf:resource="thsys/73344"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19799-2">
    <skos:broader rdf:resource="thsys/73343"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19799-2">
    <skos:broader rdf:resource="descriptor/10470-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10197-5">
    <skos:altLabel xml:lang="de">Stackelberg-Lösung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10197-5">
    <skos:altLabel xml:lang="de">Launhardt-Hotelling-Lösung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10197-5">
    <skos:altLabel xml:lang="de">Krelle-Lösung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10197-5">
    <skos:altLabel xml:lang="de">Dyopol</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10197-5">
    <skos:altLabel xml:lang="de">Cournot-Duopol</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10197-5">
    <skos:altLabel xml:lang="de">Bertrand-Edgeworth-Lösung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13272-2">
    <skos:altLabel xml:lang="de">Postzahlungsverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13272-2">
    <skos:altLabel xml:lang="de">Postsparkasse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13272-2">
    <skos:altLabel xml:lang="de">Postscheckdienst</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13272-2">
    <skos:altLabel xml:lang="de">Postgirodienst</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13272-2">
    <skos:altLabel xml:lang="de">Postbankdienstleistung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13272-2">
    <skos:altLabel xml:lang="de">Bankdienst der Post</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18966-0">
    <skos:altLabel xml:lang="de">Umweltfachleute</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19357-6">
    <skos:broader rdf:resource="thsys/71091"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19357-6">
    <skos:broader rdf:resource="thsys/70058"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19357-6">
    <skos:broader rdf:resource="descriptor/10605-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11168-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10216-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10161-5">
    <skos:narrower rdf:resource="descriptor/10164-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10161-5">
    <skos:narrower rdf:resource="descriptor/10159-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10161-5">
    <skos:narrower rdf:resource="descriptor/10160-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10161-5">
    <skos:narrower rdf:resource="descriptor/16739-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16033-5">
    <skos:related rdf:resource="descriptor/13304-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16033-5">
    <skos:related rdf:resource="descriptor/11840-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16033-5">
    <skos:related rdf:resource="descriptor/18745-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10220-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id276">
    <rdf:value xml:lang="de">für Staatsanwaltschaft</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16273-6">
    <skos:broader rdf:resource="thsys/70095"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14212-1">
    <skos:altLabel xml:lang="en">Gasoline</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14212-1">
    <skos:altLabel xml:lang="de">Motorenbenzin</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14212-1">
    <skos:altLabel xml:lang="de">Kerosin</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14212-1">
    <skos:altLabel xml:lang="de">Flugbenzin</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18412-1">
    <skos:related rdf:resource="descriptor/13577-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18412-1">
    <skos:related rdf:resource="descriptor/14336-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18412-1">
    <skos:related rdf:resource="descriptor/13058-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14710-6">
    <skos:prefLabel xml:lang="en">Polymers</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14710-6">
    <skos:prefLabel xml:lang="de">Polymere</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71006">
    <rdfs:label xml:lang="de">Erdöl- und Erdgasförderung</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15362-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13019-0">
    <skos:prefLabel xml:lang="en">Nonmetallic minerals industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13019-0">
    <skos:prefLabel xml:lang="de">Industrie der Steine und Erden</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16463-1">
    <zbwext:useInsteadNote rdf:nodeID="id295"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10976-2">
    <skos:altLabel xml:lang="de">Wirtschaftsordnungspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10976-2">
    <skos:altLabel xml:lang="de">Rahmenpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16264-0">
    <skos:broader rdf:resource="thsys/73381"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16264-0">
    <skos:broader rdf:resource="thsys/73366"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16264-0">
    <skos:broader rdf:resource="thsys/73332"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16264-0">
    <skos:broader rdf:resource="descriptor/15964-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19149-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17477-6">
    <skos:broader rdf:resource="thsys/70035"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17477-6">
    <skos:broader rdf:resource="descriptor/19083-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17477-6">
    <skos:broader rdf:resource="descriptor/17983-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13731-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73368">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17534-6">
    <skos:prefLabel xml:lang="en">Russia</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17534-6">
    <skos:prefLabel xml:lang="de">Russland</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20733-1">
    <skos:prefLabel xml:lang="en">New Basel Capital Accord</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20733-1">
    <skos:prefLabel xml:lang="de">Basel II</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18168-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15943-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16204-4">
    <skos:broader rdf:resource="thsys/73364"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16204-4">
    <skos:broader rdf:resource="thsys/70878"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16204-4">
    <skos:broader rdf:resource="descriptor/16209-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16204-4">
    <skos:broader rdf:resource="descriptor/10436-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10062-0">
    <zbwext:useInsteadNote rdf:nodeID="id377"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13338-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19001-3">
    <skos:altLabel xml:lang="de">Personal-Controlling</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15952-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18646-4">
    <skos:narrower rdf:resource="descriptor/18652-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11853-6">
    <skos:broader rdf:resource="thsys/70058"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11853-6">
    <skos:broader rdf:resource="descriptor/19684-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11853-6">
    <skos:broader rdf:resource="descriptor/11857-5"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id347">
    <rdf:value xml:lang="de">Für das Vergleichsverfahren im Konkursrecht</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19355-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19235-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18159-1">
    <skos:altLabel xml:lang="de">Sprecherausschuss für leitende Angestellte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18159-1">
    <skos:altLabel xml:lang="de">Sprecherausschuß für leitende Angestellte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18159-1">
    <skos:altLabel xml:lang="de">Sprecherausschuß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13391-1">
    <skos:prefLabel xml:lang="en">Property management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13391-1">
    <skos:prefLabel xml:lang="de">Vermögensverwaltung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11238-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10143-0">
    <skos:broader rdf:resource="thsys/71062"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10143-0">
    <skos:broader rdf:resource="descriptor/10147-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10143-0">
    <skos:broader rdf:resource="descriptor/11219-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16014-2">
    <skos:altLabel xml:lang="de">Amtshaftung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11569-3">
    <skos:altLabel xml:lang="en">Temporary assistance to needy</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11569-3">
    <skos:altLabel xml:lang="en">Income support</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11569-3">
    <skos:altLabel xml:lang="de">Wohlfahrtsprogramm</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11569-3">
    <skos:altLabel xml:lang="de">TANF</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15932-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15375-3">
    <skos:prefLabel xml:lang="en">Simultaneous equation model</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15375-3">
    <skos:prefLabel xml:lang="de">Simultanes Gleichungssystem</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11224-2">
    <skos:broader rdf:resource="thsys/71062"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11224-2">
    <skos:broader rdf:resource="thsys/70847"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11224-2">
    <skos:broader rdf:resource="thsys/70176"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11224-2">
    <skos:broader rdf:resource="descriptor/10326-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19676-4">
    <skos:prefLabel xml:lang="en">Physician payment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19676-4">
    <skos:prefLabel xml:lang="de">Arzthonorar</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18020-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18458-5">
    <skos:prefLabel xml:lang="en">Industrial energy supply</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18458-5">
    <skos:prefLabel xml:lang="de">Industrielle Energieversorgung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14824-5">
    <skos:broader rdf:resource="thsys/70372"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14824-5">
    <skos:broader rdf:resource="descriptor/14817-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10169-3">
    <skos:related rdf:resource="descriptor/10184-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10169-3">
    <skos:related rdf:resource="descriptor/11272-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10169-3">
    <skos:related rdf:resource="descriptor/11187-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10169-3">
    <skos:related rdf:resource="descriptor/18140-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10169-3">
    <skos:related rdf:resource="descriptor/11266-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19590-0">
    <skos:prefLabel xml:lang="en">Balance of payments statistics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19590-0">
    <skos:prefLabel xml:lang="de">Zahlungsbilanzstatistik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19486-2">
    <skos:altLabel xml:lang="en">Urban ozone</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19486-2">
    <skos:altLabel xml:lang="de">Sommersmog</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19829-5">
    <skos:broader rdf:resource="thsys/70038"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19829-5">
    <skos:broader rdf:resource="descriptor/10987-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14063-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13763-4">
    <skos:narrower rdf:resource="descriptor/13762-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16602-6">
    <skos:altLabel xml:lang="de">Sozialordnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16602-6">
    <skos:altLabel xml:lang="de">Soziale Ordnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16602-6">
    <skos:altLabel xml:lang="de">Gesellschaftssystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16602-6">
    <skos:altLabel xml:lang="de">Gesellschaftsform</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18768-4">
    <skos:broader rdf:resource="thsys/70880"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18768-4">
    <skos:broader rdf:resource="descriptor/18940-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19755-1">
    <skos:related rdf:resource="descriptor/20176-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19755-1">
    <skos:related rdf:resource="descriptor/19758-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19755-1">
    <skos:related rdf:resource="descriptor/19683-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19755-1">
    <skos:related rdf:resource="descriptor/12253-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19755-1">
    <skos:related rdf:resource="descriptor/19198-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14996-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18777-3">
    <skos:broader rdf:resource="thsys/73348"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18777-3">
    <skos:broader rdf:resource="thsys/70993"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18777-3">
    <skos:broader rdf:resource="descriptor/19782-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18777-3">
    <skos:broader rdf:resource="descriptor/15635-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14539-4">
    <skos:broader rdf:resource="thsys/70423"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18823-1">
    <skos:prefLabel xml:lang="en">Public sector management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18823-1">
    <skos:prefLabel xml:lang="de">Öffentliche Betriebswirtschaftslehre</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11836-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13054-5">
    <skos:broader rdf:resource="thsys/71002"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13054-5">
    <skos:broader rdf:resource="descriptor/13058-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73343">
    <skos:narrower rdf:resource="descriptor/19799-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73343">
    <skos:narrower rdf:resource="descriptor/18783-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73343">
    <skos:narrower rdf:resource="descriptor/12649-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73343">
    <skos:narrower rdf:resource="descriptor/18279-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73343">
    <skos:narrower rdf:resource="descriptor/18406-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73343">
    <skos:narrower rdf:resource="descriptor/18402-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73343">
    <skos:narrower rdf:resource="descriptor/18401-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73343">
    <skos:narrower rdf:resource="descriptor/12647-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73343">
    <skos:narrower rdf:resource="descriptor/15651-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10489-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70536">
    <skos:prefLabel xml:lang="de">P.12.05  Waschmittel und Körperpflegemittel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15019-4">
    <skos:related rdf:resource="descriptor/18453-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15019-4">
    <skos:related rdf:resource="descriptor/12913-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12263-1">
    <skos:broader rdf:resource="thsys/71105"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12263-1">
    <skos:broader rdf:resource="thsys/71045"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12263-1">
    <skos:broader rdf:resource="descriptor/12234-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15998-2">
    <skos:narrower rdf:resource="descriptor/11349-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15998-2">
    <skos:narrower rdf:resource="descriptor/16011-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15998-2">
    <skos:narrower rdf:resource="descriptor/13662-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15998-2">
    <skos:narrower rdf:resource="descriptor/15994-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15998-2">
    <skos:narrower rdf:resource="descriptor/19843-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10603-6">
    <skos:broader rdf:resource="thsys/73366"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10603-6">
    <skos:broader rdf:resource="thsys/71097"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10603-6">
    <skos:broader rdf:resource="thsys/70866"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10603-6">
    <skos:broader rdf:resource="descriptor/19328-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10603-6">
    <skos:broader rdf:resource="descriptor/10617-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10603-6">
    <skos:broader rdf:resource="descriptor/16489-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16641-3">
    <skos:related rdf:resource="descriptor/16198-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16641-3">
    <skos:related rdf:resource="descriptor/16642-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16641-3">
    <skos:related rdf:resource="descriptor/15806-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16641-3">
    <skos:related rdf:resource="descriptor/15671-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16641-3">
    <skos:related rdf:resource="descriptor/16709-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15810-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13395-0">
    <skos:broader rdf:resource="thsys/73378"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13395-0">
    <skos:broader rdf:resource="thsys/73366"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13395-0">
    <skos:broader rdf:resource="thsys/71084"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13395-0">
    <skos:broader rdf:resource="thsys/70853"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13395-0">
    <skos:broader rdf:resource="descriptor/12091-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13395-0">
    <skos:broader rdf:resource="descriptor/10984-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13686-3">
    <skos:altLabel xml:lang="de">Betriebswirtschaftslehre der Banken</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13686-3">
    <skos:altLabel xml:lang="de">Bankbetriebswirtschaftslehre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13686-3">
    <skos:altLabel xml:lang="de">Bankbetriebswirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13686-3">
    <skos:altLabel xml:lang="de">Bankbetrieb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12209-0">
    <skos:prefLabel xml:lang="en">Investment theory (Business)</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12209-0">
    <skos:prefLabel xml:lang="de">Betriebswirtschaftliche Investitionstheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15696-4">
    <skos:related rdf:resource="descriptor/15689-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70905">
    <skos:prefLabel xml:lang="de">W.14  Finanz- und Kreditwesen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73347">
    <skos:broader rdf:resource="thsys/70151"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16278-3">
    <skos:scopeNote xml:lang="de">Zusammenschluss bzw. Zusammenwachsen von Staaten/Regionen, die vorher nicht verbunden waren.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19252-6">
    <skos:related rdf:resource="descriptor/11873-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12311-2">
    <skos:prefLabel xml:lang="en">Revenue</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12311-2">
    <skos:prefLabel xml:lang="de">Einnahmen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12273-5">
    <skos:altLabel xml:lang="en">Share capital</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12273-5">
    <skos:altLabel xml:lang="en">Net worth</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12273-5">
    <skos:altLabel xml:lang="en">Equity financing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12273-5">
    <skos:altLabel xml:lang="de">Stammkapital</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12273-5">
    <skos:altLabel xml:lang="de">Nennkapital</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12273-5">
    <skos:altLabel xml:lang="de">Grundkapital</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12273-5">
    <skos:altLabel xml:lang="de">Eigenfinanzierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14567-6">
    <skos:prefLabel xml:lang="en">Tool</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14567-6">
    <skos:prefLabel xml:lang="de">Werkzeug</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18271-0">
    <skos:related rdf:resource="descriptor/19810-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18271-0">
    <skos:related rdf:resource="descriptor/24706-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18271-0">
    <skos:related rdf:resource="descriptor/13801-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18271-0">
    <skos:related rdf:resource="descriptor/13793-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18763-0">
    <skos:prefLabel xml:lang="en">Online vendor</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18763-0">
    <skos:prefLabel xml:lang="de">Datenbankanbieter</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11252-4">
    <skos:broader rdf:resource="thsys/71058"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11252-4">
    <skos:broader rdf:resource="descriptor/11235-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18011-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14104-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19589-6">
    <skos:related rdf:resource="descriptor/11751-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19589-6">
    <skos:related rdf:resource="descriptor/10274-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13225-4">
    <skos:prefLabel xml:lang="en">Fruit trade</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13225-4">
    <skos:prefLabel xml:lang="de">Fruchthandel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12178-6">
    <skos:broader rdf:resource="thsys/70087"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12178-6">
    <skos:broader rdf:resource="descriptor/12194-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10109-0">
    <skos:related rdf:resource="descriptor/10106-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10109-0">
    <skos:related rdf:resource="descriptor/10975-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10109-0">
    <skos:related rdf:resource="descriptor/11167-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10109-0">
    <skos:related rdf:resource="descriptor/10088-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24781-5">
    <skos:altLabel xml:lang="en">Customer lifetime value</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24781-5">
    <skos:altLabel xml:lang="en">Customer equity</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15695-6">
    <skos:broader rdf:resource="thsys/71064"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15695-6">
    <skos:broader rdf:resource="thsys/70007"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15695-6">
    <skos:broader rdf:resource="descriptor/15690-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16344-2">
    <skos:prefLabel xml:lang="en">Parliamentary group</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16344-2">
    <skos:prefLabel xml:lang="de">Parlamentsfraktion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19247-6">
    <skos:related rdf:resource="descriptor/13808-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19247-6">
    <skos:related rdf:resource="descriptor/18030-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19247-6">
    <skos:related rdf:resource="descriptor/12613-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19247-6">
    <skos:related rdf:resource="descriptor/12078-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19247-6">
    <skos:related rdf:resource="descriptor/12699-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19247-6">
    <skos:related rdf:resource="descriptor/12166-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19247-6">
    <skos:related rdf:resource="descriptor/19800-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19247-6">
    <skos:related rdf:resource="descriptor/11339-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19085-6">
    <skos:altLabel xml:lang="de">Visegrad-Länder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19085-6">
    <skos:altLabel xml:lang="de">CEFTA-Staaten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19085-6">
    <skos:altLabel xml:lang="de">CEFTA-Länder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11379-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18399-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13417-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11722-0">
    <skos:prefLabel xml:lang="en">Public Sector</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11722-0">
    <skos:prefLabel xml:lang="de">Öffentlicher Sektor</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14804-4">
    <skos:altLabel xml:lang="de">Wellpappe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14804-4">
    <skos:altLabel xml:lang="de">Pappeverpackung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14804-4">
    <skos:altLabel xml:lang="de">Kartonage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12988-6">
    <skos:prefLabel xml:lang="en">Non-ferrous metals ore mining</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12988-6">
    <skos:prefLabel xml:lang="de">NE-Metallerzbergbau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18491-0">
    <skos:related rdf:resource="descriptor/13860-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18491-0">
    <skos:related rdf:resource="descriptor/12905-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18491-0">
    <skos:related rdf:resource="descriptor/18495-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18491-0">
    <skos:related rdf:resource="descriptor/14021-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18491-0">
    <skos:related rdf:resource="descriptor/18507-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18491-0">
    <skos:related rdf:resource="descriptor/18493-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18491-0">
    <skos:related rdf:resource="descriptor/18492-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11377-5">
    <skos:narrower rdf:resource="descriptor/19392-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11377-5">
    <skos:narrower rdf:resource="descriptor/11362-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11377-5">
    <skos:narrower rdf:resource="descriptor/11357-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13352-4">
    <skos:narrower rdf:resource="descriptor/18919-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16249-3">
    <skos:related rdf:resource="descriptor/16404-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16249-3">
    <skos:related rdf:resource="descriptor/16416-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16249-3">
    <skos:related rdf:resource="descriptor/16247-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16249-3">
    <skos:related rdf:resource="descriptor/16400-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19649-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14019-2">
    <skos:altLabel xml:lang="de">Babybedarf</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18219-2">
    <skos:broader rdf:resource="thsys/73302"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18219-2">
    <skos:broader rdf:resource="descriptor/14122-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71094">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.07.03.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16303-2">
    <skos:altLabel xml:lang="de">Politische Unterdrückung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16303-2">
    <skos:altLabel xml:lang="de">Politische Repression</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11828-5">
    <skos:related rdf:resource="descriptor/11829-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11828-5">
    <skos:related rdf:resource="descriptor/12013-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11828-5">
    <skos:related rdf:resource="descriptor/13593-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70096">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16989-5">
    <skos:broader rdf:resource="thsys/70505"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16989-5">
    <skos:broader rdf:resource="descriptor/16990-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16989-5">
    <skos:broader rdf:resource="descriptor/17983-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12794-5">
    <skos:broader rdf:resource="thsys/70040"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12794-5">
    <skos:broader rdf:resource="descriptor/12770-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16485-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16470-4">
    <skos:broader rdf:resource="thsys/73364"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16470-4">
    <skos:broader rdf:resource="descriptor/16288-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19841-1">
    <skos:prefLabel xml:lang="en">Judiciary</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19841-1">
    <skos:prefLabel xml:lang="de">Justiz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14116-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18491-0">
    <skos:prefLabel xml:lang="en">Construction research</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18491-0">
    <skos:prefLabel xml:lang="de">Bauforschung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13303-3">
    <skos:altLabel xml:lang="de">Staatliche Dienstleistung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18789-3">
    <skos:prefLabel xml:lang="en">Technical consulting</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18789-3">
    <skos:prefLabel xml:lang="de">Technische Beratung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16209-1">
    <skos:altLabel xml:lang="de">Theorie der Politik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16209-1">
    <skos:altLabel xml:lang="de">Politologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16209-1">
    <skos:altLabel xml:lang="de">Politische Wissenschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16209-1">
    <skos:altLabel xml:lang="de">Politische Theorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16209-1">
    <skos:altLabel xml:lang="de">Politiktheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15678-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19753-5">
    <skos:prefLabel xml:lang="en">Authority</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19753-5">
    <skos:prefLabel xml:lang="de">Autorität</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70970">
    <skos:broader rdf:resource="thsys/w"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17806-6">
    <skos:narrower rdf:resource="descriptor/17829-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17806-6">
    <skos:narrower rdf:resource="descriptor/17811-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14573-4">
    <skos:altLabel xml:lang="de">Streichinstrument</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14573-4">
    <skos:altLabel xml:lang="de">Schlaginstrument</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14573-4">
    <skos:altLabel xml:lang="de">Saiteninstrument</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14573-4">
    <skos:altLabel xml:lang="de">Konzertflügel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14573-4">
    <skos:altLabel xml:lang="de">Klavier</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14573-4">
    <skos:altLabel xml:lang="de">Elektronisches Musikinstrument</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14573-4">
    <skos:altLabel xml:lang="de">Blasinstrument</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18785-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13501-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16969-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12013-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14170-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13457-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18461-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19022-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18072-6">
    <skos:prefLabel xml:lang="en">Environmental tax</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18072-6">
    <skos:prefLabel xml:lang="de">Ökosteuer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19030-3">
    <skos:broader rdf:resource="thsys/70582"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12001-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11793-5">
    <skos:altLabel xml:lang="de">Wachstumsindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11793-5">
    <skos:altLabel xml:lang="de">Wachsender Markt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17777-1">
    <skos:altLabel xml:lang="de">Republik Costa Rica</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70234">
    <skos:broader rdf:resource="thsys/v"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70486">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11803-0">
    <skos:broader rdf:resource="thsys/71028"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11803-0">
    <skos:broader rdf:resource="descriptor/11782-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14308-2">
    <skos:related rdf:resource="descriptor/11438-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73388">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.01.03</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10944-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12254-2">
    <skos:altLabel xml:lang="en">Operate leasing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id345">
    <rdf:value xml:lang="de">Für Wirtschafts- und Branchenverbände in Form von Vereinen</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10374-2">
    <skos:broader rdf:resource="thsys/70020"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10374-2">
    <skos:broader rdf:resource="descriptor/10031-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10374-2">
    <skos:broader rdf:resource="descriptor/10361-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13292-3">
    <skos:narrower rdf:resource="descriptor/18738-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13292-3">
    <skos:narrower rdf:resource="descriptor/19551-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13292-3">
    <skos:narrower rdf:resource="descriptor/13415-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16656-4">
    <zbwext:useInsteadNote rdf:nodeID="id291"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18237-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19471-1">
    <skos:broader rdf:resource="thsys/71064"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19471-1">
    <skos:broader rdf:resource="thsys/71060"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19471-1">
    <skos:broader rdf:resource="descriptor/11990-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19471-1">
    <skos:broader rdf:resource="descriptor/11182-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11424-1">
    <skos:broader rdf:resource="thsys/70055"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19723-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18400-1">
    <skos:altLabel xml:lang="de">Schweißerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18400-1">
    <skos:altLabel xml:lang="de">Schweißer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18400-1">
    <skos:altLabel xml:lang="de">Revolverdreherinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18400-1">
    <skos:altLabel xml:lang="de">Revolverdreher</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18400-1">
    <skos:altLabel xml:lang="de">Metallarbeiterinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18400-1">
    <skos:altLabel xml:lang="de">Metallarbeiter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18400-1">
    <skos:altLabel xml:lang="de">Klempnerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18400-1">
    <skos:altLabel xml:lang="de">Klempner</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18400-1">
    <skos:altLabel xml:lang="de">Hüttenfacharbeiterinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18400-1">
    <skos:altLabel xml:lang="de">Hüttenfacharbeiter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18400-1">
    <skos:altLabel xml:lang="de">Galvaniseurinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18400-1">
    <skos:altLabel xml:lang="de">Galvaniseure</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18400-1">
    <skos:altLabel xml:lang="de">Dreherinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18400-1">
    <skos:altLabel xml:lang="de">Dreher</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18109-2">
    <skos:broader rdf:resource="thsys/70087"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18109-2">
    <skos:broader rdf:resource="descriptor/12174-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12655-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18844-0">
    <skos:altLabel xml:lang="de">Mietervertretung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18844-0">
    <skos:altLabel xml:lang="de">Mieterverein</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18844-0">
    <skos:altLabel xml:lang="de">Mieterinitiative</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18844-0">
    <skos:altLabel xml:lang="de">Mieterbund</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10558-6">
    <skos:scopeNote xml:lang="de">Sowohl für nationale wie für internationale Organisationen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14692-3">
    <skos:broader rdf:resource="thsys/70556"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14692-3">
    <skos:broader rdf:resource="descriptor/14683-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11569-3">
    <skos:narrower rdf:resource="descriptor/19508-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15970-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11578-2">
    <skos:narrower rdf:resource="descriptor/16769-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73334">
    <skos:broader rdf:resource="thsys/70160"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19852-3">
    <skos:related rdf:resource="descriptor/16041-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19852-3">
    <skos:related rdf:resource="descriptor/11569-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19852-3">
    <skos:related rdf:resource="descriptor/19849-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11915-3">
    <skos:related rdf:resource="descriptor/18160-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11915-3">
    <skos:related rdf:resource="descriptor/11932-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11915-3">
    <skos:related rdf:resource="descriptor/19291-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11915-3">
    <skos:related rdf:resource="descriptor/11914-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11915-3">
    <skos:related rdf:resource="descriptor/11913-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14019-2">
    <skos:prefLabel xml:lang="en">Baby product</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14019-2">
    <skos:prefLabel xml:lang="de">Babyartikel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14486-6">
    <skos:prefLabel xml:lang="en">Lamp</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14486-6">
    <skos:prefLabel xml:lang="de">Lampe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14337-2">
    <skos:related rdf:resource="descriptor/18619-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14337-2">
    <skos:related rdf:resource="descriptor/13185-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14337-2">
    <skos:related rdf:resource="descriptor/14404-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14337-2">
    <skos:related rdf:resource="descriptor/18562-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14337-2">
    <skos:related rdf:resource="descriptor/15728-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13079-3">
    <skos:related rdf:resource="descriptor/13083-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13079-3">
    <skos:related rdf:resource="descriptor/13081-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13079-3">
    <skos:related rdf:resource="descriptor/10462-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13079-3">
    <skos:related rdf:resource="descriptor/14465-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13079-3">
    <skos:related rdf:resource="descriptor/18396-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13079-3">
    <skos:related rdf:resource="descriptor/13078-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#hiddenLabel">
    <rdfs:comment xml:lang="en">skos:prefLabel, skos:altLabel and skos:hiddenLabel are pairwise
      disjoint properties.</rdfs:comment>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#hiddenLabel">
    <rdfs:comment xml:lang="en">The range of skos:hiddenLabel is the class of RDF plain literals.</rdfs:comment>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18488-3">
    <skos:altLabel xml:lang="de">Industriemanagerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18488-3">
    <skos:altLabel xml:lang="de">Industrielle Unternehmensleiterinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18488-3">
    <skos:altLabel xml:lang="de">Industrielle Unternehmensleiter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71076">
    <skos:narrower rdf:resource="descriptor/16119-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71076">
    <skos:narrower rdf:resource="descriptor/11696-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71076">
    <skos:narrower rdf:resource="descriptor/11695-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71076">
    <skos:narrower rdf:resource="descriptor/11694-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71076">
    <skos:narrower rdf:resource="descriptor/19408-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71076">
    <skos:narrower rdf:resource="descriptor/19411-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71076">
    <skos:narrower rdf:resource="descriptor/11692-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71076">
    <skos:narrower rdf:resource="descriptor/11691-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71076">
    <skos:narrower rdf:resource="descriptor/19410-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71076">
    <skos:narrower rdf:resource="descriptor/11688-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71076">
    <skos:narrower rdf:resource="descriptor/11687-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71076">
    <skos:narrower rdf:resource="descriptor/11686-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71076">
    <skos:narrower rdf:resource="descriptor/19400-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71076">
    <skos:narrower rdf:resource="descriptor/19371-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71076">
    <skos:narrower rdf:resource="descriptor/11661-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71076">
    <skos:narrower rdf:resource="descriptor/19403-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71076">
    <skos:narrower rdf:resource="descriptor/19375-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71076">
    <skos:narrower rdf:resource="descriptor/11685-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71076">
    <skos:narrower rdf:resource="descriptor/19409-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71076">
    <skos:narrower rdf:resource="descriptor/19402-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71076">
    <skos:narrower rdf:resource="descriptor/11648-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71076">
    <skos:narrower rdf:resource="descriptor/11684-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71076">
    <skos:narrower rdf:resource="descriptor/19406-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13246-3">
    <skos:broader rdf:resource="thsys/70932"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13246-3">
    <skos:broader rdf:resource="descriptor/13510-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18010-0">
    <skos:narrower rdf:resource="descriptor/13536-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18010-0">
    <skos:narrower rdf:resource="descriptor/11792-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18010-0">
    <skos:narrower rdf:resource="descriptor/10337-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13865-3">
    <skos:altLabel xml:lang="en">Theory of the housing market</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13865-3">
    <skos:altLabel xml:lang="en">Housing market model</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13865-3">
    <skos:altLabel xml:lang="de">Wohnungswirtschaftliche Betriebswirtschaftslehre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13865-3">
    <skos:altLabel xml:lang="de">Wohnungsmarkttheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12961-5">
    <skos:broader rdf:resource="thsys/71019"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12961-5">
    <skos:broader rdf:resource="descriptor/18096-6"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id303">
    <rdf:value xml:lang="de">für die Preissetzung in Planwirtschaften</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10842-2">
    <skos:prefLabel xml:lang="en">Global competition</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10842-2">
    <skos:prefLabel xml:lang="de">Internationaler Wettbewerb</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16619-3">
    <skos:prefLabel xml:lang="en">Underclass</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16619-3">
    <skos:prefLabel xml:lang="de">Unterschicht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73104">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.17</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15945-2">
    <skos:related rdf:resource="descriptor/15927-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15945-2">
    <skos:related rdf:resource="descriptor/19757-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15945-2">
    <skos:related rdf:resource="descriptor/19756-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15945-2">
    <skos:related rdf:resource="descriptor/15920-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15945-2">
    <skos:related rdf:resource="descriptor/15914-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16968-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12108-6">
    <skos:altLabel xml:lang="en">Organizational culture</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12108-6">
    <skos:altLabel xml:lang="en">Organisational culture</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12108-6">
    <skos:altLabel xml:lang="en">Corporate identity</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12108-6">
    <skos:altLabel xml:lang="de">Unternehmensphilosophie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12108-6">
    <skos:altLabel xml:lang="de">Unternehmensleitbild</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12108-6">
    <skos:altLabel xml:lang="de">Unternehmensidentität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12108-6">
    <skos:altLabel xml:lang="de">Organisationskultur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12108-6">
    <skos:altLabel xml:lang="de">Managementphilosophie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12108-6">
    <skos:altLabel xml:lang="de">Leitbild des Betriebs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15458-6">
    <skos:scopeNote xml:lang="de">Mathematische Theorie interaktiver Entscheidungsprobleme.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12291-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14404-6">
    <skos:prefLabel xml:lang="en">Heating appliance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14404-6">
    <skos:prefLabel xml:lang="de">Heizungsanlage</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15897-1">
    <skos:prefLabel xml:lang="en">Forest</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15897-1">
    <skos:prefLabel xml:lang="de">Wald</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13563-5">
    <skos:narrower rdf:resource="descriptor/18578-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13563-5">
    <skos:narrower rdf:resource="descriptor/13244-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13563-5">
    <skos:narrower rdf:resource="descriptor/13633-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13563-5">
    <skos:narrower rdf:resource="descriptor/13558-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10140-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14176-0">
    <skos:prefLabel xml:lang="en">Energy source</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14176-0">
    <skos:prefLabel xml:lang="de">Energieträger</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18809-2">
    <skos:prefLabel xml:lang="en">State law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18809-2">
    <skos:prefLabel xml:lang="de">Landesrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13470-5">
    <skos:narrower rdf:resource="descriptor/12783-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13470-5">
    <skos:narrower rdf:resource="descriptor/12779-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13470-5">
    <skos:narrower rdf:resource="descriptor/13279-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13470-5">
    <skos:narrower rdf:resource="descriptor/11576-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13470-5">
    <skos:narrower rdf:resource="descriptor/13322-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12205-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11683-5">
    <skos:altLabel xml:lang="de">Wertzuwachsbesteuerung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11683-5">
    <skos:altLabel xml:lang="de">Vermögenszuwachssteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11683-5">
    <skos:altLabel xml:lang="de">Planungswertausgleich</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11683-5">
    <skos:altLabel xml:lang="de">Kapitalgewinnsteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11683-5">
    <skos:altLabel xml:lang="de">Bodenwertzuwachssteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70879">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11875-3">
    <skos:related rdf:resource="descriptor/11962-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11875-3">
    <skos:related rdf:resource="descriptor/11951-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11875-3">
    <skos:related rdf:resource="descriptor/11857-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11875-3">
    <skos:related rdf:resource="descriptor/19438-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14432-1">
    <skos:related rdf:resource="descriptor/18577-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14432-1">
    <skos:related rdf:resource="descriptor/13046-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11192-3">
    <skos:altLabel xml:lang="en">Absenteeism</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11192-3">
    <skos:altLabel xml:lang="en">Absence from work</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11192-3">
    <skos:altLabel xml:lang="de">Betriebliche Fehlzeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11192-3">
    <skos:altLabel xml:lang="de">Arbeitszeitausfall</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11192-3">
    <skos:altLabel xml:lang="de">Absentismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15408-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10697-6">
    <skos:broader rdf:resource="thsys/71098"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10697-6">
    <skos:broader rdf:resource="thsys/71020"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10697-6">
    <skos:broader rdf:resource="descriptor/11800-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12990-5">
    <skos:altLabel xml:lang="en">Coal industry</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12990-5">
    <skos:altLabel xml:lang="de">Steinkohlenbergbau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12990-5">
    <skos:altLabel xml:lang="de">Kohleindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15054-2">
    <skos:altLabel xml:lang="en">Dynamic nonlinear systems</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15054-2">
    <skos:altLabel xml:lang="en">Catastrophe model</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15054-2">
    <skos:altLabel xml:lang="de">Katastrophentheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11658-4">
    <skos:scopeNote xml:lang="de">Besteuerung von Grundstücksübertragungen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18020-4">
    <skos:altLabel xml:lang="de">PC-Standardsoftware</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18020-4">
    <skos:altLabel xml:lang="de">PC-Programm</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18020-4">
    <skos:altLabel xml:lang="de">PC-gestütztes Programm</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18020-4">
    <skos:altLabel xml:lang="de">PC-gesteuertes Programm</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10678-3">
    <skos:related rdf:resource="descriptor/18188-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10678-3">
    <skos:related rdf:resource="descriptor/10722-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10678-3">
    <skos:related rdf:resource="descriptor/10773-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14048-2">
    <skos:broader rdf:resource="thsys/70201"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14048-2">
    <skos:broader rdf:resource="descriptor/14024-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13358-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14723-4">
    <skos:related rdf:resource="descriptor/14728-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14723-4">
    <skos:related rdf:resource="descriptor/18199-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14723-4">
    <skos:related rdf:resource="descriptor/19017-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14723-4">
    <skos:related rdf:resource="descriptor/18222-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14723-4">
    <skos:related rdf:resource="descriptor/16476-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14723-4">
    <skos:related rdf:resource="descriptor/16668-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16696-6">
    <skos:broader rdf:resource="thsys/73355"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16696-6">
    <skos:broader rdf:resource="thsys/73345"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16696-6">
    <skos:broader rdf:resource="thsys/71109"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19747-0">
    <skos:related rdf:resource="descriptor/12846-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19747-0">
    <skos:related rdf:resource="descriptor/15790-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19747-0">
    <skos:related rdf:resource="descriptor/12805-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19747-0">
    <skos:related rdf:resource="descriptor/14040-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19747-0">
    <skos:related rdf:resource="descriptor/15655-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10978-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19608-0">
    <skos:prefLabel xml:lang="en">Linguistics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19608-0">
    <skos:prefLabel xml:lang="de">Linguistik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17949-5">
    <skos:altLabel xml:lang="de">Königreich Tonga</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17949-5">
    <skos:altLabel xml:lang="de">Freundschaftsinseln</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id274">
    <rdfs:seeAlso rdf:resource="descriptor/13995-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11811-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14019-2">
    <skos:broader rdf:resource="thsys/70201"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16322-5">
    <skos:prefLabel xml:lang="en">Territorial reform</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16322-5">
    <skos:prefLabel xml:lang="de">Gebietsreform</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id49">
    <rdfs:seeAlso rdf:resource="descriptor/11235-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19041-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13357-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14046-6">
    <skos:broader rdf:resource="thsys/70265"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14046-6">
    <skos:broader rdf:resource="descriptor/14816-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10302-6">
    <skos:broader rdf:resource="thsys/71112"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10302-6">
    <skos:broader rdf:resource="thsys/70493"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10302-6">
    <skos:broader rdf:resource="descriptor/10301-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10166-2">
    <skos:related rdf:resource="descriptor/10113-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10166-2">
    <skos:related rdf:resource="descriptor/10161-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10166-2">
    <skos:related rdf:resource="descriptor/19219-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10166-2">
    <skos:related rdf:resource="descriptor/10041-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10166-2">
    <skos:related rdf:resource="descriptor/16406-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10166-2">
    <skos:related rdf:resource="descriptor/18832-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11828-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11092-0">
    <skos:broader rdf:resource="thsys/70178"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11092-0">
    <skos:broader rdf:resource="descriptor/11071-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12103-2">
    <skos:scopeNote xml:lang="de">Ansatz der Organisationsforschung, nach dem die Situation die Organisationsstruktur bestimmt und diese wiederum die Effizienz. Angewandt auf die Führungsforschung besagt der Ansatz, daß situativen Einflüssen eine entscheidende Bedeutung für den Führungserfolg eingeräumt wird.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18289-2">
    <skos:altLabel xml:lang="en">Cloning</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18289-2">
    <skos:altLabel xml:lang="de">Klonen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18289-2">
    <skos:altLabel xml:lang="de">Gentechnologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18289-2">
    <skos:altLabel xml:lang="de">Genomanalyse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18289-2">
    <skos:altLabel xml:lang="de">Genetische Forschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14488-2">
    <skos:scopeNote xml:lang="de">Leuchten sind die Beleuchtungskörper.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17717-5">
    <skos:broader rdf:resource="thsys/70560"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17717-5">
    <skos:broader rdf:resource="descriptor/17724-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12314-3">
    <skos:prefLabel xml:lang="en">Human resources</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12314-3">
    <skos:prefLabel xml:lang="de">Humanvermögen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13064-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71028">
    <skos:narrower rdf:resource="descriptor/18315-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71028">
    <skos:narrower rdf:resource="descriptor/11276-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71028">
    <skos:narrower rdf:resource="descriptor/13702-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71028">
    <skos:narrower rdf:resource="descriptor/13441-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71028">
    <skos:narrower rdf:resource="descriptor/10505-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71028">
    <skos:narrower rdf:resource="descriptor/19046-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71028">
    <skos:narrower rdf:resource="descriptor/19191-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71028">
    <skos:narrower rdf:resource="descriptor/10576-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71028">
    <skos:narrower rdf:resource="descriptor/12945-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71028">
    <skos:narrower rdf:resource="descriptor/13436-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71028">
    <skos:narrower rdf:resource="descriptor/10503-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71028">
    <skos:narrower rdf:resource="descriptor/13188-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71028">
    <skos:narrower rdf:resource="descriptor/18096-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71028">
    <skos:narrower rdf:resource="descriptor/19194-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71028">
    <skos:narrower rdf:resource="descriptor/13434-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71028">
    <skos:narrower rdf:resource="descriptor/10495-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71028">
    <skos:narrower rdf:resource="descriptor/13635-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71028">
    <skos:narrower rdf:resource="descriptor/18320-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71028">
    <skos:narrower rdf:resource="descriptor/13446-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71028">
    <skos:narrower rdf:resource="descriptor/13451-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71028">
    <skos:narrower rdf:resource="descriptor/11805-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71028">
    <skos:narrower rdf:resource="descriptor/13450-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71028">
    <skos:narrower rdf:resource="descriptor/11803-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71028">
    <skos:narrower rdf:resource="descriptor/13433-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71028">
    <skos:narrower rdf:resource="descriptor/13445-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71028">
    <skos:narrower rdf:resource="descriptor/13426-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71028">
    <skos:narrower rdf:resource="descriptor/18980-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71028">
    <skos:narrower rdf:resource="descriptor/19674-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71028">
    <skos:narrower rdf:resource="descriptor/18318-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71028">
    <skos:narrower rdf:resource="descriptor/13431-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71028">
    <skos:narrower rdf:resource="descriptor/13430-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71028">
    <skos:narrower rdf:resource="descriptor/11147-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71028">
    <skos:narrower rdf:resource="descriptor/13425-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71028">
    <skos:narrower rdf:resource="descriptor/13444-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71028">
    <skos:narrower rdf:resource="descriptor/13443-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71028">
    <skos:narrower rdf:resource="descriptor/18983-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71028">
    <skos:narrower rdf:resource="descriptor/18317-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71028">
    <skos:narrower rdf:resource="descriptor/18008-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71028">
    <skos:narrower rdf:resource="thsys/71023"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71028">
    <skos:narrower rdf:resource="thsys/71029"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71028">
    <skos:narrower rdf:resource="thsys/71025"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71028">
    <skos:narrower rdf:resource="thsys/71020"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71028">
    <skos:narrower rdf:resource="thsys/71024"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19606-4">
    <skos:broader rdf:resource="thsys/73104"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19606-4">
    <skos:broader rdf:resource="thsys/70140"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19606-4">
    <skos:broader rdf:resource="descriptor/11164-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73395">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13605-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16160-5">
    <skos:broader rdf:resource="thsys/73335"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16160-5">
    <skos:broader rdf:resource="thsys/70833"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16160-5">
    <skos:broader rdf:resource="thsys/70832"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16160-5">
    <skos:broader rdf:resource="thsys/70324"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16160-5">
    <skos:broader rdf:resource="descriptor/16159-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19550-5">
    <skos:altLabel xml:lang="en">Low-income regions</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19550-5">
    <skos:altLabel xml:lang="de">Niedrigeinkommensländer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19550-5">
    <skos:altLabel xml:lang="de">Länder mit niedrigem Pro-Kopf-Einkommen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70012">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70038">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.06.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15754-2">
    <skos:broader rdf:resource="thsys/73351"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15754-2">
    <skos:broader rdf:resource="thsys/71010"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15754-2">
    <skos:broader rdf:resource="thsys/70811"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15754-2">
    <skos:broader rdf:resource="descriptor/15745-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11394-5">
    <skos:prefLabel xml:lang="en">Educational research</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11394-5">
    <skos:prefLabel xml:lang="de">Bildungsforschung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11474-0">
    <skos:prefLabel xml:lang="en">Real balance effect</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11474-0">
    <skos:prefLabel xml:lang="de">Realkasseneffekt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11894-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13247-1">
    <skos:broader rdf:resource="thsys/70932"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13247-1">
    <skos:broader rdf:resource="descriptor/13264-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18859-1">
    <zbwext:useInsteadNote rdf:nodeID="id348"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11636-0">
    <skos:related rdf:resource="descriptor/19404-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11636-0">
    <skos:related rdf:resource="descriptor/21014-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11636-0">
    <skos:related rdf:resource="descriptor/12876-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11636-0">
    <skos:related rdf:resource="descriptor/19281-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11636-0">
    <skos:related rdf:resource="descriptor/11703-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11636-0">
    <skos:related rdf:resource="descriptor/11691-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11636-0">
    <skos:related rdf:resource="descriptor/21011-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11636-0">
    <skos:related rdf:resource="descriptor/24858-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11636-0">
    <skos:related rdf:resource="descriptor/19410-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19492-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70386">
    <skos:broader rdf:resource="thsys/70394"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13392-6">
    <skos:related rdf:resource="descriptor/16037-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13392-6">
    <skos:related rdf:resource="descriptor/18065-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13392-6">
    <skos:related rdf:resource="descriptor/13355-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13392-6">
    <skos:related rdf:resource="descriptor/18920-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13392-6">
    <skos:related rdf:resource="descriptor/18176-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16374-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13830-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19390-1">
    <skos:broader rdf:resource="thsys/71001"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19390-1">
    <skos:broader rdf:resource="thsys/70957"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19390-1">
    <skos:broader rdf:resource="descriptor/19184-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19088-0">
    <skos:related rdf:resource="descriptor/16036-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19088-0">
    <skos:related rdf:resource="descriptor/10019-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19088-0">
    <skos:related rdf:resource="descriptor/12253-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19088-0">
    <skos:related rdf:resource="descriptor/18725-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19088-0">
    <skos:related rdf:resource="descriptor/12251-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19088-0">
    <skos:related rdf:resource="descriptor/12277-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19088-0">
    <skos:related rdf:resource="descriptor/10568-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10080-5">
    <skos:altLabel xml:lang="en">Out-of-equilibrium trading</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10080-5">
    <skos:altLabel xml:lang="en">Non-Walrasian model</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10080-5">
    <skos:altLabel xml:lang="en">False trading</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10080-5">
    <skos:altLabel xml:lang="en">Dual-decision hypothesis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10080-5">
    <skos:altLabel xml:lang="de">Ungleichgewichtstransaktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10080-5">
    <skos:altLabel xml:lang="de">Non-Tatonnement-Gleichgewicht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10145-3">
    <skos:broader rdf:resource="thsys/70234"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10145-3">
    <skos:broader rdf:resource="descriptor/10249-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14215-2">
    <skos:altLabel xml:lang="en">Heating oil</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id335">
    <rdfs:seeAlso rdf:resource="descriptor/13645-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18815-0">
    <skos:altLabel xml:lang="de">Militärbetrieb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18815-0">
    <skos:altLabel xml:lang="de">Betriebswirtschaftslehre der Streitkräfte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13650-3">
    <skos:broader rdf:resource="thsys/70265"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13650-3">
    <skos:broader rdf:resource="descriptor/14812-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18898-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12272-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11703-4">
    <zbwext:useInsteadNote rdf:nodeID="id316"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13627-5">
    <skos:related rdf:resource="descriptor/18248-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19042-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19431-6">
    <skos:altLabel xml:lang="en">Network effect</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19431-6">
    <skos:altLabel xml:lang="de">Netzwerkexternalitäten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19431-6">
    <skos:altLabel xml:lang="de">Netzwerk-Externalität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19431-6">
    <skos:altLabel xml:lang="de">Netzwerkeffekt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19431-6">
    <skos:altLabel xml:lang="de">Netzwerk-Effekt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19431-6">
    <skos:altLabel xml:lang="de">Netzeffekt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71082">
    <skos:broader rdf:resource="thsys/70144"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18919-2">
    <skos:prefLabel xml:lang="en">Cinema</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18919-2">
    <skos:prefLabel xml:lang="de">Kinowirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12816-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70143">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.08.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10323-5">
    <skos:altLabel xml:lang="de">Mehrproduktfertigung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19333-6">
    <skos:broader rdf:resource="thsys/71103"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19333-6">
    <skos:broader rdf:resource="thsys/71095"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19333-6">
    <skos:broader rdf:resource="thsys/71088"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19333-6">
    <skos:broader rdf:resource="descriptor/11664-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19333-6">
    <skos:broader rdf:resource="descriptor/10725-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18163-3">
    <skos:altLabel xml:lang="de">Strompreis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18163-3">
    <skos:altLabel xml:lang="de">Elektrizitätstarif</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18163-3">
    <skos:altLabel xml:lang="de">Elektrizitätspreis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19435-5">
    <skos:broader rdf:resource="thsys/71071"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19435-5">
    <skos:broader rdf:resource="descriptor/10894-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18301-3">
    <skos:altLabel xml:lang="de">Oecotrophologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18301-3">
    <skos:altLabel xml:lang="de">Hauswirtschaftswissenschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18301-3">
    <skos:altLabel xml:lang="de">Haushaltswissenschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18301-3">
    <skos:altLabel xml:lang="de">Ernährungswissenschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18301-3">
    <skos:altLabel xml:lang="de">Diätetik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13078-5">
    <skos:related rdf:resource="descriptor/13079-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13078-5">
    <skos:related rdf:resource="descriptor/18396-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13078-5">
    <skos:related rdf:resource="descriptor/14531-6"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id60">
    <rdf:value xml:lang="de">Für die gesamte Büroausstattung</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13975-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18227-3">
    <skos:broader rdf:resource="thsys/70130"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18227-3">
    <skos:broader rdf:resource="descriptor/17179-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11216-1">
    <skos:broader rdf:resource="thsys/71060"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11216-1">
    <skos:broader rdf:resource="descriptor/11182-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19616-1">
    <skos:related rdf:resource="descriptor/19684-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19616-1">
    <skos:related rdf:resource="descriptor/16187-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19616-1">
    <skos:related rdf:resource="descriptor/15887-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19616-1">
    <skos:related rdf:resource="descriptor/13543-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19616-1">
    <skos:related rdf:resource="descriptor/15883-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19616-1">
    <skos:related rdf:resource="descriptor/12967-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19616-1">
    <skos:related rdf:resource="descriptor/15878-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19616-1">
    <skos:related rdf:resource="descriptor/18312-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16364-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12969-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70029">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70582">
    <skos:prefLabel xml:lang="de">A.00  Allgemeinwörter</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14775-6">
    <skos:broader rdf:resource="thsys/73306"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14775-6">
    <skos:broader rdf:resource="descriptor/14776-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19207-4">
    <skos:related rdf:resource="descriptor/10062-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19207-4">
    <skos:related rdf:resource="descriptor/15045-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12384-3">
    <skos:related rdf:resource="descriptor/12396-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73367">
    <rdfs:label xml:lang="de">Landes- und Kommunalpolitik</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11728-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10075-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16176-4">
    <skos:narrower rdf:resource="descriptor/11995-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14308-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12091-4">
    <skos:broader rdf:resource="thsys/73378"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12091-4">
    <skos:broader rdf:resource="thsys/70562"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12091-4">
    <skos:broader rdf:resource="thsys/70321"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11490-2">
    <skos:broader rdf:resource="thsys/70305"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11490-2">
    <skos:broader rdf:resource="descriptor/10382-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10247-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19805-5">
    <skos:altLabel xml:lang="de">Software Ergonomie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12015-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15679-4">
    <skos:related rdf:resource="descriptor/13411-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15679-4">
    <skos:related rdf:resource="descriptor/15949-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15679-4">
    <skos:related rdf:resource="descriptor/16670-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15679-4">
    <skos:related rdf:resource="descriptor/18444-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15679-4">
    <skos:related rdf:resource="descriptor/15908-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11586-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13135-5">
    <skos:altLabel xml:lang="de">Häuteindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13135-5">
    <skos:altLabel xml:lang="de">Gerberei</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19101-6">
    <skos:scopeNote xml:lang="de">Nur im Sinne der Prozesskostenrechnung: Kosten werden betrieblichen Abläufen zugerechnet.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73333">
    <skos:narrower rdf:resource="descriptor/16070-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73333">
    <skos:narrower rdf:resource="descriptor/19705-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73333">
    <skos:narrower rdf:resource="descriptor/16169-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73333">
    <skos:narrower rdf:resource="descriptor/10621-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73333">
    <skos:narrower rdf:resource="descriptor/16059-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73333">
    <skos:narrower rdf:resource="descriptor/16186-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73333">
    <skos:narrower rdf:resource="descriptor/16032-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73333">
    <skos:narrower rdf:resource="descriptor/16185-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73333">
    <skos:narrower rdf:resource="descriptor/19329-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73333">
    <skos:narrower rdf:resource="descriptor/19812-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73333">
    <skos:narrower rdf:resource="descriptor/18790-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73333">
    <skos:narrower rdf:resource="descriptor/19853-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73333">
    <skos:narrower rdf:resource="descriptor/10880-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73333">
    <skos:narrower rdf:resource="descriptor/12313-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73333">
    <skos:narrower rdf:resource="thsys/73332"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73333">
    <skos:narrower rdf:resource="thsys/73331"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19745-4">
    <skos:prefLabel xml:lang="en">Survey</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19745-4">
    <skos:prefLabel xml:lang="de">Befragung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15013-2">
    <skos:altLabel xml:lang="en">Nuclear waste</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15013-2">
    <skos:altLabel xml:lang="de">Strahlender Abfall</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15013-2">
    <skos:altLabel xml:lang="de">Kontaminierter Abfall</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15013-2">
    <skos:altLabel xml:lang="de">Atommüll</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15013-2">
    <skos:altLabel xml:lang="de">Atomarer Abfall</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71039">
    <skos:broader rdf:resource="thsys/b"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18608-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19736-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15922-0">
    <skos:scopeNote xml:lang="de">Für alle eingeborenen Völker in Nord-, Mittel- und Südamerika</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15725-2">
    <skos:prefLabel xml:lang="en">Social reform</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15725-2">
    <skos:prefLabel xml:lang="de">Sozialreform</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19625-0">
    <skos:scopeNote xml:lang="de">Genetische Vielfalt, Artenvielfalt sowie Bestrebungen zu ihrer Erhaltung und zum Erhalt von Ökosystemen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14007-2">
    <skos:altLabel xml:lang="de">Pflegepersonal</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14007-2">
    <skos:altLabel xml:lang="de">Krankenschwestern</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14007-2">
    <skos:altLabel xml:lang="de">Krankenpfleger</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14007-2">
    <skos:altLabel xml:lang="de">Krankenpflegeberufe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14007-2">
    <skos:altLabel xml:lang="de">Altenpflegerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14007-2">
    <skos:altLabel xml:lang="de">Altenpfleger</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14007-2">
    <skos:altLabel xml:lang="de">Altenpflegeberufe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18266-0">
    <skos:related rdf:resource="descriptor/15763-1"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id10">
    <rdf:value xml:lang="de">Für das allgemeinbildende Schulwesen</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16090-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12702-1">
    <skos:prefLabel xml:lang="en">Construction</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12702-1">
    <skos:prefLabel xml:lang="de">Konstruktion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15621-0">
    <skos:broader rdf:resource="thsys/73340"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15621-0">
    <skos:broader rdf:resource="thsys/73305"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15621-0">
    <skos:broader rdf:resource="thsys/71038"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11433-0">
    <skos:prefLabel xml:lang="en">Credit market</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11433-0">
    <skos:prefLabel xml:lang="de">Kreditmarkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18755-6">
    <skos:related rdf:resource="descriptor/13304-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18755-6">
    <skos:related rdf:resource="descriptor/13839-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18755-6">
    <skos:related rdf:resource="descriptor/11840-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18755-6">
    <skos:related rdf:resource="descriptor/18844-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18755-6">
    <skos:related rdf:resource="descriptor/13837-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71108">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10307-3">
    <skos:broader rdf:resource="thsys/70233"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10307-3">
    <skos:broader rdf:resource="descriptor/10327-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14442-5">
    <skos:related rdf:resource="descriptor/14434-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14442-5">
    <skos:related rdf:resource="descriptor/14029-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13242-4">
    <skos:broader rdf:resource="thsys/70935"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13242-4">
    <skos:broader rdf:resource="descriptor/13256-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14656-0">
    <skos:narrower rdf:resource="descriptor/14686-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14656-0">
    <skos:narrower rdf:resource="descriptor/14685-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14656-0">
    <skos:narrower rdf:resource="descriptor/14683-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14656-0">
    <skos:narrower rdf:resource="descriptor/14679-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14656-0">
    <skos:narrower rdf:resource="descriptor/14661-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14656-0">
    <skos:narrower rdf:resource="descriptor/14674-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14656-0">
    <skos:narrower rdf:resource="descriptor/14623-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14656-0">
    <skos:narrower rdf:resource="descriptor/14665-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15448-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70967">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19744-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18417-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20756-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19659-4">
    <skos:prefLabel xml:lang="en">Multicultural</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19659-4">
    <skos:prefLabel xml:lang="de">Multikulturell</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16439-5">
    <skos:related rdf:resource="descriptor/13395-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16439-5">
    <skos:related rdf:resource="descriptor/19708-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16439-5">
    <skos:related rdf:resource="descriptor/19711-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16439-5">
    <skos:related rdf:resource="descriptor/10558-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15105-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18825-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10608-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70993">
    <skos:narrower rdf:resource="descriptor/13002-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70993">
    <skos:narrower rdf:resource="descriptor/18288-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70993">
    <skos:narrower rdf:resource="descriptor/18289-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70993">
    <skos:narrower rdf:resource="descriptor/15635-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70993">
    <skos:narrower rdf:resource="descriptor/18260-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70993">
    <skos:narrower rdf:resource="descriptor/18777-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70993">
    <skos:narrower rdf:resource="descriptor/18259-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70993">
    <skos:narrower rdf:resource="descriptor/19675-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18262-1">
    <skos:altLabel xml:lang="de">Kosmetikindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13010-4">
    <skos:related rdf:resource="descriptor/13213-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13010-4">
    <skos:related rdf:resource="descriptor/14766-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15730-2">
    <skos:related rdf:resource="descriptor/16023-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15730-2">
    <skos:related rdf:resource="descriptor/15734-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15730-2">
    <skos:related rdf:resource="descriptor/15733-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15730-2">
    <skos:related rdf:resource="descriptor/11543-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15730-2">
    <skos:related rdf:resource="descriptor/13329-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15730-2">
    <skos:related rdf:resource="descriptor/15729-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16155-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11963-6">
    <skos:related rdf:resource="descriptor/11912-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13653-4">
    <skos:altLabel xml:lang="de">Telekopieren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13653-4">
    <skos:altLabel xml:lang="de">Telebrief</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13653-4">
    <skos:altLabel xml:lang="de">Fernkopieren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13653-4">
    <skos:altLabel xml:lang="de">Faxdienst</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13653-4">
    <skos:altLabel xml:lang="de">Faksimilekopieren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16579-3">
    <skos:related rdf:resource="descriptor/15429-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16579-3">
    <skos:related rdf:resource="descriptor/16723-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16579-3">
    <skos:related rdf:resource="descriptor/19751-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16579-3">
    <skos:related rdf:resource="descriptor/16602-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16579-3">
    <skos:related rdf:resource="descriptor/16588-2"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id238">
    <rdf:value xml:lang="de">Für staatliche Maßnahmen zur Erzwingung der Beachtung von Gesetzen</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11607-0">
    <skos:altLabel xml:lang="de">Staatliche Schuldenverwaltung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11607-0">
    <skos:altLabel xml:lang="de">Bundesschuldenverwaltung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10872-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14339-5">
    <skos:prefLabel xml:lang="en">Machine tool</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14339-5">
    <skos:prefLabel xml:lang="de">Maschinenwerkzeug</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10251-4">
    <skos:related rdf:resource="descriptor/10347-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10251-4">
    <skos:related rdf:resource="descriptor/10286-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12005-2">
    <skos:broader rdf:resource="thsys/71067"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12005-2">
    <skos:broader rdf:resource="descriptor/11913-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17829-1">
    <skos:prefLabel xml:lang="en">United States</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17829-1">
    <skos:prefLabel xml:lang="de">USA</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13233-5">
    <skos:broader rdf:resource="thsys/70959"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13233-5">
    <skos:broader rdf:resource="descriptor/18522-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19126-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10001-6">
    <skos:broader rdf:resource="thsys/70893"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10001-6">
    <skos:broader rdf:resource="thsys/70187"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11675-4">
    <skos:narrower rdf:resource="descriptor/11673-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16288-0">
    <skos:prefLabel xml:lang="en">Political conflict</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16288-0">
    <skos:prefLabel xml:lang="de">Politischer Konflikt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18289-2">
    <skos:related rdf:resource="descriptor/18443-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18289-2">
    <skos:related rdf:resource="descriptor/18288-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11704-2">
    <skos:related rdf:resource="descriptor/10713-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11704-2">
    <skos:related rdf:resource="descriptor/11637-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11704-2">
    <skos:related rdf:resource="descriptor/11696-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11704-2">
    <skos:related rdf:resource="descriptor/11694-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11704-2">
    <skos:related rdf:resource="descriptor/11642-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11704-2">
    <skos:related rdf:resource="descriptor/11640-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11704-2">
    <skos:related rdf:resource="descriptor/12876-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11704-2">
    <skos:related rdf:resource="descriptor/19405-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11704-2">
    <skos:related rdf:resource="descriptor/11708-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10418-1">
    <skos:altLabel xml:lang="de">Knife-Edge-Theorem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10418-1">
    <skos:altLabel xml:lang="de">Harrod-Instabilitätstheorem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10418-1">
    <skos:altLabel xml:lang="de">Harrod-Domar-Wachstumsmodell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10418-1">
    <skos:altLabel xml:lang="de">Harrod-Dilemma</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10418-1">
    <skos:altLabel xml:lang="de">Domar-Wachstumsmodell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15927-4">
    <skos:broader rdf:resource="thsys/73358"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15927-4">
    <skos:broader rdf:resource="thsys/70142"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15927-4">
    <skos:broader rdf:resource="descriptor/18861-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14442-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15416-1">
    <zbwext:useInsteadNote rdf:nodeID="id213"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11223-4">
    <skos:altLabel xml:lang="en">Signalling theory</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11223-4">
    <skos:altLabel xml:lang="en">Signaling model</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11223-4">
    <skos:altLabel xml:lang="en">Market signalling model</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11223-4">
    <skos:altLabel xml:lang="en">Market signaling model</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11223-4">
    <skos:altLabel xml:lang="de">Screening-Hypothese</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13411-0">
    <skos:related rdf:resource="descriptor/15679-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13411-0">
    <skos:related rdf:resource="descriptor/13418-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13411-0">
    <skos:related rdf:resource="descriptor/16532-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13411-0">
    <skos:related rdf:resource="descriptor/11388-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18421-0">
    <skos:narrower rdf:resource="descriptor/14485-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10827-5">
    <skos:related rdf:resource="descriptor/19404-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10827-5">
    <skos:related rdf:resource="descriptor/10831-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10827-5">
    <skos:related rdf:resource="descriptor/10830-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10827-5">
    <skos:related rdf:resource="descriptor/10817-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10827-5">
    <skos:related rdf:resource="descriptor/10621-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10827-5">
    <skos:related rdf:resource="descriptor/10829-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10827-5">
    <skos:related rdf:resource="descriptor/19414-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10827-5">
    <skos:related rdf:resource="descriptor/11649-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10827-5">
    <skos:related rdf:resource="descriptor/10828-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10827-5">
    <skos:related rdf:resource="descriptor/10823-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10827-5">
    <skos:related rdf:resource="descriptor/10826-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10827-5">
    <skos:related rdf:resource="descriptor/10606-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10522-6">
    <skos:altLabel xml:lang="en">Dualistic development</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10522-6">
    <skos:altLabel xml:lang="de">Dualismusmodell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13383-0">
    <skos:related rdf:resource="descriptor/18859-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13383-0">
    <skos:related rdf:resource="descriptor/13661-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13383-0">
    <skos:related rdf:resource="descriptor/15821-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13383-0">
    <skos:related rdf:resource="descriptor/12774-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13383-0">
    <skos:related rdf:resource="descriptor/19745-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14142-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12863-5">
    <skos:prefLabel xml:lang="en">Affiliation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12863-5">
    <skos:prefLabel xml:lang="de">Organschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19826-4">
    <skos:broader rdf:resource="thsys/70530"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19826-4">
    <skos:broader rdf:resource="thsys/70020"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19826-4">
    <skos:broader rdf:resource="descriptor/19238-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17954-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13342-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12985-5">
    <skos:prefLabel xml:lang="en">Metal mining</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12985-5">
    <skos:prefLabel xml:lang="de">Erzbergbau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19311-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19796-1">
    <zbwext:useInsteadNote rdf:nodeID="id205"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19796-1">
    <zbwext:useInsteadNote rdf:nodeID="id204"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12880-5">
    <skos:prefLabel xml:lang="en">Tax audit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12880-5">
    <skos:prefLabel xml:lang="de">Steuerliche Betriebsprüfung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13613-2">
    <skos:broader rdf:resource="thsys/70927"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13613-2">
    <skos:broader rdf:resource="descriptor/13611-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70182">
    <skos:narrower rdf:resource="descriptor/15135-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70182">
    <skos:narrower rdf:resource="descriptor/15949-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70182">
    <skos:narrower rdf:resource="descriptor/19477-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70182">
    <skos:narrower rdf:resource="descriptor/19646-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70182">
    <skos:narrower rdf:resource="descriptor/11894-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70182">
    <skos:narrower rdf:resource="descriptor/15947-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70182">
    <skos:narrower rdf:resource="descriptor/15124-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70182">
    <skos:narrower rdf:resource="descriptor/15946-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70182">
    <skos:narrower rdf:resource="descriptor/15943-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70182">
    <skos:narrower rdf:resource="descriptor/15941-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70182">
    <skos:narrower rdf:resource="descriptor/15940-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70182">
    <skos:narrower rdf:resource="descriptor/15939-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70182">
    <skos:narrower rdf:resource="descriptor/19645-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70182">
    <skos:narrower rdf:resource="descriptor/10489-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70182">
    <skos:narrower rdf:resource="descriptor/15708-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19028-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10557-1">
    <skos:broader rdf:resource="thsys/73366"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10557-1">
    <skos:broader rdf:resource="thsys/71084"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10557-1">
    <skos:broader rdf:resource="descriptor/19708-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10557-1">
    <skos:broader rdf:resource="descriptor/10555-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12420-4">
    <skos:broader rdf:resource="thsys/71043"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12420-4">
    <skos:broader rdf:resource="descriptor/12408-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19337-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16440-6">
    <skos:broader rdf:resource="thsys/73366"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16440-6">
    <skos:broader rdf:resource="thsys/73359"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16440-6">
    <skos:broader rdf:resource="thsys/70140"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18397-6">
    <zbwext:useInsteadNote rdf:nodeID="id339"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12571-4">
    <skos:altLabel xml:lang="en">Job matching</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12571-4">
    <skos:altLabel xml:lang="en">Employment interview</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12571-4">
    <skos:altLabel xml:lang="de">Vorstellungsgespräch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12571-4">
    <skos:altLabel xml:lang="de">Personalselektion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12571-4">
    <skos:altLabel xml:lang="de">Mitarbeiterauswahl</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12571-4">
    <skos:altLabel xml:lang="de">Einstellungsgespräch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12571-4">
    <skos:altLabel xml:lang="de">Bewerbergespräch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12571-4">
    <skos:altLabel xml:lang="de">Bewerberauslese</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12571-4">
    <skos:altLabel xml:lang="de">Auswahl von Personal</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10243-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11869-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20104-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19321-6">
    <skos:scopeNote xml:lang="de">Liberalisierung des nationalen und internationalen Kapitalverkehrs.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11742-1">
    <skos:related rdf:resource="descriptor/11738-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19556-0">
    <skos:broader rdf:resource="thsys/70899"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19556-0">
    <skos:broader rdf:resource="thsys/70459"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19556-0">
    <skos:broader rdf:resource="descriptor/13756-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71089">
    <rdfs:label xml:lang="de">Internationale Finanz- und Kapitalmärkte</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11310-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71028">
    <rdfs:label xml:lang="en">Agriculture</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71028">
    <rdfs:label xml:lang="de">Landwirtschaft</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70313">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">P.17</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10836-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19809-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18702-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12241-4">
    <skos:prefLabel xml:lang="en">Factoring</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12241-4">
    <skos:prefLabel xml:lang="de">Factoring</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14730-0">
    <skos:altLabel xml:lang="en">Pain killer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14730-0">
    <skos:altLabel xml:lang="de">Analgetikum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12755-1">
    <skos:prefLabel xml:lang="en">Lot size</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12755-1">
    <skos:prefLabel xml:lang="de">Losgröße</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18031-6">
    <skos:altLabel xml:lang="en">Technology center</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18031-6">
    <skos:altLabel xml:lang="en">Technological centre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18031-6">
    <skos:altLabel xml:lang="en">Science park</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18031-6">
    <skos:altLabel xml:lang="en">Innovation pole</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18031-6">
    <skos:altLabel xml:lang="de">Wissenschaftspark</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18031-6">
    <skos:altLabel xml:lang="de">Technologie-Transfer-Zentrum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18031-6">
    <skos:altLabel xml:lang="de">Technologiepark</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18031-6">
    <skos:altLabel xml:lang="de">Gründerzentrum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18031-6">
    <skos:altLabel xml:lang="de">Forschungspark</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12655-5">
    <skos:related rdf:resource="descriptor/12651-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12655-5">
    <skos:related rdf:resource="descriptor/12748-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12850-0">
    <skos:broader rdf:resource="thsys/73337"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12850-0">
    <skos:broader rdf:resource="thsys/70397"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12850-0">
    <skos:broader rdf:resource="descriptor/16092-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15874-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12841-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12330-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19958-1">
    <skos:related rdf:resource="descriptor/18126-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19958-1">
    <skos:related rdf:resource="descriptor/10698-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11825-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71086">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11769-2">
    <skos:prefLabel xml:lang="en">Employee ownership</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11769-2">
    <skos:prefLabel xml:lang="de">Arbeitnehmerbeteiligung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id204">
    <rdf:value xml:lang="de">Für statistische Maße</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11083-1">
    <skos:related rdf:resource="descriptor/11090-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11083-1">
    <skos:related rdf:resource="descriptor/11073-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11083-1">
    <skos:related rdf:resource="descriptor/16470-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11083-1">
    <skos:related rdf:resource="descriptor/11046-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11083-1">
    <skos:related rdf:resource="descriptor/19709-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11083-1">
    <skos:related rdf:resource="descriptor/10515-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11083-1">
    <skos:related rdf:resource="descriptor/16489-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18268-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19173-2">
    <skos:scopeNote xml:lang="de">Mitgliedsländer des Central American Common Market</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14651-3">
    <skos:prefLabel xml:lang="en">Sulfur dioxide</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14651-3">
    <skos:prefLabel xml:lang="de">Schwefeldioxid</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15013-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70239">
    <rdfs:label xml:lang="de">Umweltpolitik und Umweltschutz</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16794-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16956-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10864-6">
    <skos:related rdf:resource="descriptor/18538-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10864-6">
    <skos:related rdf:resource="descriptor/10836-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18988-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19197-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71070">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19502-2">
    <skos:scopeNote xml:lang="de">Anteil gewerkschaftlich organisierter Arbeitskräfte.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11611-2">
    <skos:related rdf:resource="descriptor/11618-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11611-2">
    <skos:related rdf:resource="descriptor/11487-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12387-4">
    <skos:related rdf:resource="descriptor/12297-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15801-5">
    <skos:altLabel xml:lang="de">Motivationstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15801-5">
    <skos:altLabel xml:lang="de">Motivationspsychologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15801-5">
    <skos:altLabel xml:lang="de">Motivationsforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15801-5">
    <skos:altLabel xml:lang="de">Herzberg-Motivationstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15801-5">
    <skos:altLabel xml:lang="de">Erwartungsvalenztheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15801-5">
    <skos:altLabel xml:lang="de">Austauschtheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15801-5">
    <skos:altLabel xml:lang="de">Anreiz-Beitrags-Theorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19215-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18000-3">
    <skos:altLabel xml:lang="de">Telekommunikationsverbund</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18000-3">
    <skos:altLabel xml:lang="de">Postnetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18000-3">
    <skos:altLabel xml:lang="de">Öffentliches Nachrichtennetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18000-3">
    <skos:altLabel xml:lang="de">Öffentliches Fernmeldenetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18000-3">
    <skos:altLabel xml:lang="de">Netz für Telekommunikation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18000-3">
    <skos:altLabel xml:lang="de">Fernmeldenetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18000-3">
    <skos:altLabel xml:lang="de">Datexnetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18000-3">
    <skos:altLabel xml:lang="de">Datenübertragungsnetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18242-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10334-0">
    <skos:prefLabel xml:lang="en">Elasticity of substitution</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10334-0">
    <skos:prefLabel xml:lang="de">Substitutionselastizität</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21306-1">
    <skos:broader rdf:resource="thsys/70485"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21306-1">
    <skos:broader rdf:resource="thsys/70244"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21306-1">
    <skos:broader rdf:resource="descriptor/12567-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21306-1">
    <skos:broader rdf:resource="descriptor/12085-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70845">
    <skos:narrower rdf:resource="descriptor/18870-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70845">
    <skos:narrower rdf:resource="descriptor/11404-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70845">
    <skos:narrower rdf:resource="descriptor/19559-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70845">
    <skos:narrower rdf:resource="descriptor/11375-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70845">
    <skos:narrower rdf:resource="descriptor/11377-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70845">
    <skos:narrower rdf:resource="descriptor/19392-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70845">
    <skos:narrower rdf:resource="descriptor/11362-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70845">
    <skos:narrower rdf:resource="descriptor/11352-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70845">
    <skos:narrower rdf:resource="descriptor/11357-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18704-2">
    <skos:altLabel xml:lang="de">Unabhängigkeit der Zentralbank</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18704-2">
    <skos:altLabel xml:lang="de">Notenbankautonomie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18704-2">
    <skos:altLabel xml:lang="de">Autonomie der Zentralbank</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19701-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14480-4">
    <skos:prefLabel xml:lang="en">Semiconductor</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14480-4">
    <skos:prefLabel xml:lang="de">Halbleiter</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18059-5">
    <skos:broader rdf:resource="thsys/70823"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18059-5">
    <skos:broader rdf:resource="descriptor/16754-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19825-6">
    <skos:prefLabel xml:lang="en">Transition to work</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19825-6">
    <skos:prefLabel xml:lang="de">Berufseinstieg</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11972-5">
    <skos:related rdf:resource="descriptor/10512-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11972-5">
    <skos:related rdf:resource="descriptor/19363-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11972-5">
    <skos:related rdf:resource="descriptor/11991-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11972-5">
    <skos:related rdf:resource="descriptor/11966-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70834">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.26.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15821-6">
    <skos:prefLabel xml:lang="en">Opinion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15821-6">
    <skos:prefLabel xml:lang="de">Meinung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id338">
    <rdfs:seeAlso rdf:resource="descriptor/19840-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70134">
    <skos:broader rdf:resource="thsys/70358"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20789-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12111-3">
    <skos:narrower rdf:resource="descriptor/12138-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12111-3">
    <skos:narrower rdf:resource="descriptor/12154-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12111-3">
    <skos:narrower rdf:resource="descriptor/12153-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12111-3">
    <skos:narrower rdf:resource="descriptor/19695-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12111-3">
    <skos:narrower rdf:resource="descriptor/20977-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12111-3">
    <skos:narrower rdf:resource="descriptor/12146-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12111-3">
    <skos:narrower rdf:resource="descriptor/19119-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12111-3">
    <skos:narrower rdf:resource="descriptor/18852-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12111-3">
    <skos:narrower rdf:resource="descriptor/12137-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10618-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#OrderedCollection">
    <rdfs:subClassOf rdf:resource="http://www.w3.org/2004/02/skos/core#Collection"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16789-6">
    <skos:narrower rdf:resource="descriptor/16777-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10552-4">
    <skos:altLabel xml:lang="en">Development financing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15491-1">
    <skos:altLabel xml:lang="en">Decomposition principle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15491-1">
    <skos:altLabel xml:lang="de">Dekompositionsalgorithmus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15491-1">
    <skos:altLabel xml:lang="de">Dantzig-Wolfe-Dekompositionsverfahren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15491-1">
    <skos:altLabel xml:lang="de">Benders-Dekompositionsverfahren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19221-3">
    <skos:broader rdf:resource="thsys/70493"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19221-3">
    <skos:broader rdf:resource="descriptor/10173-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11917-6">
    <skos:related rdf:resource="descriptor/11895-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11917-6">
    <skos:related rdf:resource="descriptor/11855-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11917-6">
    <skos:related rdf:resource="descriptor/10503-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10668-6">
    <skos:altLabel xml:lang="en">Price linkages</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10668-6">
    <skos:altLabel xml:lang="de">Theorie der internationalen Preisbildung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10668-6">
    <skos:altLabel xml:lang="de">Gesetz der Preisunterschiedslosigkeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14661-0">
    <skos:altLabel xml:lang="en">MTBE (Methyl Tertbutyl Ether)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14661-0">
    <skos:altLabel xml:lang="de">Methylether</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11301-3">
    <skos:altLabel xml:lang="de">Qualifizierte Arbeiter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11301-3">
    <skos:altLabel xml:lang="de">Facharbeiterinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13808-1">
    <skos:narrower rdf:resource="descriptor/13817-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13808-1">
    <skos:narrower rdf:resource="descriptor/18737-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13808-1">
    <skos:narrower rdf:resource="descriptor/10866-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16173-3">
    <skos:broader rdf:resource="thsys/73332"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16173-3">
    <skos:broader rdf:resource="thsys/70931"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16173-3">
    <skos:broader rdf:resource="descriptor/16188-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14125-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14266-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id290">
    <rdfs:seeAlso rdf:resource="descriptor/14024-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18909-5">
    <skos:altLabel xml:lang="en">European social charter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18909-5">
    <skos:altLabel xml:lang="en">Common social policy</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18909-5">
    <skos:altLabel xml:lang="de">Sozialunion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18909-5">
    <skos:altLabel xml:lang="de">EU-Sozialcharta</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18909-5">
    <skos:altLabel xml:lang="de">Europäische Sozialpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19302-3">
    <skos:prefLabel xml:lang="en">Wheat production</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19302-3">
    <skos:prefLabel xml:lang="de">Weizenanbau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70256">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19145-0">
    <skos:prefLabel xml:lang="en">Intranet</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19145-0">
    <skos:prefLabel xml:lang="de">Intranet</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17589-2">
    <skos:narrower rdf:resource="descriptor/17592-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17589-2">
    <skos:narrower rdf:resource="descriptor/21042-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17589-2">
    <skos:narrower rdf:resource="descriptor/17590-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17589-2">
    <skos:narrower rdf:resource="descriptor/17588-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17589-2">
    <skos:narrower rdf:resource="descriptor/17586-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17589-2">
    <skos:narrower rdf:resource="descriptor/17599-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17589-2">
    <skos:narrower rdf:resource="descriptor/17584-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17589-2">
    <skos:narrower rdf:resource="descriptor/17583-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17589-2">
    <skos:narrower rdf:resource="descriptor/17580-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17589-2">
    <skos:narrower rdf:resource="descriptor/17578-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17589-2">
    <skos:narrower rdf:resource="descriptor/17576-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13460-1">
    <skos:related rdf:resource="descriptor/11824-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13460-1">
    <skos:related rdf:resource="descriptor/11823-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13460-1">
    <skos:related rdf:resource="descriptor/13210-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12977-4">
    <skos:altLabel xml:lang="de">Heizkraftwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14991-0">
    <skos:altLabel xml:lang="de">Vogelfutter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14991-0">
    <skos:altLabel xml:lang="de">Tierfertignahrung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14991-0">
    <skos:altLabel xml:lang="de">Katzenfutter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14991-0">
    <skos:altLabel xml:lang="de">Hundefutter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14991-0">
    <skos:altLabel xml:lang="de">Heimtiernahrung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14991-0">
    <skos:altLabel xml:lang="de">Haustiernahrung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18359-0">
    <skos:prefLabel xml:lang="en">Electrical power engineering</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18359-0">
    <skos:prefLabel xml:lang="de">Elektrische Energietechnik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19839-2">
    <skos:prefLabel xml:lang="en">Maintenance (Domestic relations)</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19839-2">
    <skos:prefLabel xml:lang="de">Unterhaltspflicht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10621-4">
    <skos:broader rdf:resource="thsys/73390"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10621-4">
    <skos:broader rdf:resource="thsys/73333"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10621-4">
    <skos:broader rdf:resource="thsys/71097"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10621-4">
    <skos:broader rdf:resource="descriptor/16081-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10621-4">
    <skos:broader rdf:resource="descriptor/16186-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14846-2">
    <skos:prefLabel xml:lang="en">Textiles</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14846-2">
    <skos:prefLabel xml:lang="de">Textilien</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14724-2">
    <skos:altLabel xml:lang="de">Verhütungsmittel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14724-2">
    <skos:altLabel xml:lang="de">Orales Verhütungsmittel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14724-2">
    <skos:altLabel xml:lang="de">Kontrazeptivum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14724-2">
    <skos:altLabel xml:lang="de">Kondom</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14724-2">
    <skos:altLabel xml:lang="de">Hormonelles Verhütungsmittel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12490-4">
    <skos:altLabel xml:lang="de">Einzelkostenrechnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14522-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14916-0">
    <skos:related rdf:resource="descriptor/12971-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14916-0">
    <skos:related rdf:resource="descriptor/14915-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14916-0">
    <skos:related rdf:resource="descriptor/14127-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73357">
    <skos:broader rdf:resource="thsys/70120"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18860-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13250-5">
    <skos:broader rdf:resource="thsys/70939"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13250-5">
    <skos:broader rdf:resource="thsys/70937"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13250-5">
    <skos:broader rdf:resource="descriptor/18578-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13250-5">
    <skos:broader rdf:resource="descriptor/13558-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#narrowMatch">
    <rdf:type rdf:resource="http://www.w3.org/1999/02/22-rdf-syntax-ns#Property"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#narrowMatch">
    <rdf:type rdf:resource="http://www.w3.org/2002/07/owl#ObjectProperty"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10172-0">
    <skos:prefLabel xml:lang="en">Consumption interdependence</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10172-0">
    <skos:prefLabel xml:lang="de">Konsuminterdependenz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70833">
    <skos:narrower rdf:resource="descriptor/19508-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70833">
    <skos:narrower rdf:resource="descriptor/19511-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70833">
    <skos:narrower rdf:resource="descriptor/16160-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70833">
    <skos:narrower rdf:resource="descriptor/11570-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70833">
    <skos:narrower rdf:resource="descriptor/16318-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70833">
    <skos:narrower rdf:resource="descriptor/15725-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70833">
    <skos:narrower rdf:resource="descriptor/16159-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70833">
    <skos:narrower rdf:resource="descriptor/16545-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70833">
    <skos:narrower rdf:resource="descriptor/11569-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70833">
    <skos:narrower rdf:resource="descriptor/18754-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70833">
    <skos:narrower rdf:resource="descriptor/16542-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70833">
    <skos:narrower rdf:resource="descriptor/18910-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70833">
    <skos:narrower rdf:resource="descriptor/16650-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70833">
    <skos:narrower rdf:resource="descriptor/16155-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70833">
    <skos:narrower rdf:resource="descriptor/19510-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70833">
    <skos:narrower rdf:resource="descriptor/18025-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70833">
    <skos:narrower rdf:resource="descriptor/18914-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70833">
    <skos:narrower rdf:resource="descriptor/16540-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70833">
    <skos:narrower rdf:resource="descriptor/11558-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70833">
    <skos:narrower rdf:resource="descriptor/19372-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70833">
    <skos:narrower rdf:resource="descriptor/11567-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70833">
    <skos:narrower rdf:resource="descriptor/11566-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70833">
    <skos:narrower rdf:resource="descriptor/11565-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70833">
    <skos:narrower rdf:resource="descriptor/16481-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70833">
    <skos:narrower rdf:resource="descriptor/16480-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70833">
    <skos:narrower rdf:resource="descriptor/19509-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70833">
    <skos:narrower rdf:resource="descriptor/16479-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70833">
    <skos:narrower rdf:resource="descriptor/16477-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70833">
    <skos:narrower rdf:resource="descriptor/11564-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70833">
    <skos:narrower rdf:resource="descriptor/18909-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70833">
    <skos:narrower rdf:resource="descriptor/18911-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70833">
    <skos:narrower rdf:resource="descriptor/15911-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70833">
    <skos:narrower rdf:resource="descriptor/12592-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70833">
    <skos:narrower rdf:resource="descriptor/11388-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70833">
    <skos:narrower rdf:resource="descriptor/16529-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70833">
    <skos:narrower rdf:resource="descriptor/16533-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70833">
    <skos:narrower rdf:resource="descriptor/12609-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70833">
    <skos:narrower rdf:resource="descriptor/16475-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11855-2">
    <skos:related rdf:resource="descriptor/11917-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11855-2">
    <skos:related rdf:resource="descriptor/19364-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11855-2">
    <skos:related rdf:resource="descriptor/11853-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10190-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15721-3">
    <skos:broader rdf:resource="thsys/73356"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18760-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19490-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21042-2">
    <skos:prefLabel xml:lang="en">East Timor</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21042-2">
    <skos:prefLabel xml:lang="de">Timor Leste</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18658-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14784-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13256-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11545-3">
    <skos:altLabel xml:lang="de">Staatseinnahmen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11545-3">
    <skos:altLabel xml:lang="de">Staatseinkünfte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11545-3">
    <skos:altLabel xml:lang="de">Öffentliche Abgaben</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11545-3">
    <skos:altLabel xml:lang="de">Nichtsteuerliche Abgaben</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12523-1">
    <skos:related rdf:resource="descriptor/12309-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10907-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10994-0">
    <skos:related rdf:resource="descriptor/12697-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10994-0">
    <skos:related rdf:resource="descriptor/11127-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10994-0">
    <skos:related rdf:resource="descriptor/15725-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11869-5">
    <skos:narrower rdf:resource="descriptor/19450-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11869-5">
    <skos:narrower rdf:resource="descriptor/11866-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11869-5">
    <skos:narrower rdf:resource="descriptor/12018-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10013-6">
    <skos:editorialNote xml:lang="de">Nur zur Postkoordination verwenden.</skos:editorialNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19380-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18249-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15924-3">
    <skos:prefLabel xml:lang="en">Youth</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15924-3">
    <skos:prefLabel xml:lang="de">Jugendliche</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19537-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#prefLabel">
    <rdfs:subPropertyOf rdf:resource="http://www.w3.org/2000/01/rdf-schema#label"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15727-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10942-5">
    <skos:scopeNote xml:lang="de">Beschränkung des freien Wettbewerbs.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13555-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12564-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14822-2">
    <skos:narrower rdf:resource="descriptor/14821-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14822-2">
    <skos:narrower rdf:resource="descriptor/14819-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13465-5">
    <skos:altLabel xml:lang="de">Handelspanel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70095">
    <rdfs:label xml:lang="de">Politik und Politikwissenschaft</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12543-2">
    <skos:broader rdf:resource="thsys/71043"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12543-2">
    <skos:broader rdf:resource="descriptor/12441-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19402-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10638-1">
    <skos:altLabel xml:lang="de">Stolper-Samuelson-Theorem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10638-1">
    <skos:altLabel xml:lang="de">Internationaler Zinsausgleich</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10638-1">
    <skos:altLabel xml:lang="de">Internationaler Lohnausgleich</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10638-1">
    <skos:altLabel xml:lang="de">Internationaler Faktorpreisausgleich</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10638-1">
    <skos:altLabel xml:lang="de">Faktorpreisausgleichstheorem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20455-3">
    <skos:broader rdf:resource="thsys/71051"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20455-3">
    <skos:broader rdf:resource="thsys/70235"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20455-3">
    <skos:broader rdf:resource="descriptor/16484-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20455-3">
    <skos:broader rdf:resource="descriptor/16136-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19835-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16072-2">
    <skos:prefLabel xml:lang="en">Savings banks law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16072-2">
    <skos:prefLabel xml:lang="de">Sparkassenrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19551-3">
    <skos:related rdf:resource="descriptor/11555-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12567-2">
    <skos:broader rdf:resource="thsys/70113"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10574-1">
    <skos:altLabel xml:lang="en">Decolonization</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10574-1">
    <skos:altLabel xml:lang="de">Neokolonialismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10574-1">
    <skos:altLabel xml:lang="de">Kolonisierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10574-1">
    <skos:altLabel xml:lang="de">Kolonie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10574-1">
    <skos:altLabel xml:lang="de">Kolonialgeschichte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10574-1">
    <skos:altLabel xml:lang="de">Kolonialgebiet</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10574-1">
    <skos:altLabel xml:lang="de">Entkolonisierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10574-1">
    <skos:altLabel xml:lang="de">Dekolonisation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15611-3">
    <skos:related rdf:resource="descriptor/12737-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15611-3">
    <skos:related rdf:resource="descriptor/19808-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15611-3">
    <skos:related rdf:resource="descriptor/15410-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15611-3">
    <skos:related rdf:resource="descriptor/19601-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15611-3">
    <skos:related rdf:resource="descriptor/15498-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15611-3">
    <skos:related rdf:resource="descriptor/18003-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73294">
    <skos:broader rdf:resource="thsys/70986"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13733-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11222-6">
    <skos:related rdf:resource="descriptor/11409-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11222-6">
    <skos:related rdf:resource="descriptor/11224-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11222-6">
    <skos:related rdf:resource="descriptor/11397-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11222-6">
    <skos:related rdf:resource="descriptor/18884-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11222-6">
    <skos:related rdf:resource="descriptor/19516-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14733-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19020-6">
    <skos:related rdf:resource="descriptor/14129-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19020-6">
    <skos:related rdf:resource="descriptor/18137-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19020-6">
    <skos:related rdf:resource="descriptor/14171-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71002">
    <skos:prefLabel xml:lang="de">W.06.01.02  Maschinen- und Anlagenbau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11532-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19096-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11501-2">
    <skos:prefLabel xml:lang="en">Gibson paradox</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11501-2">
    <skos:prefLabel xml:lang="de">Gibson Paradox</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17688-0">
    <skos:prefLabel xml:lang="en">South Africa</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17688-0">
    <skos:prefLabel xml:lang="de">Südafrika</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11827-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70384">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13736-0">
    <skos:related rdf:resource="descriptor/12257-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13736-0">
    <skos:related rdf:resource="descriptor/13703-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13736-0">
    <skos:related rdf:resource="descriptor/13775-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12192-5">
    <skos:broader rdf:resource="thsys/70087"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12192-5">
    <skos:broader rdf:resource="descriptor/12194-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11485-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18362-4">
    <skos:related rdf:resource="descriptor/18364-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18362-4">
    <skos:related rdf:resource="descriptor/18024-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19038-1">
    <skos:prefLabel xml:lang="en">Experiment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19038-1">
    <skos:prefLabel xml:lang="de">Experiment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14789-2">
    <skos:broader rdf:resource="thsys/70416"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14789-2">
    <skos:broader rdf:resource="descriptor/14788-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12203-5">
    <skos:broader rdf:resource="thsys/71044"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12203-5">
    <skos:broader rdf:resource="descriptor/12208-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13218-1">
    <skos:broader rdf:resource="thsys/70960"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13218-1">
    <skos:broader rdf:resource="descriptor/13219-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13218-1">
    <skos:broader rdf:resource="descriptor/13200-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19735-0">
    <skos:prefLabel xml:lang="en">Anthroposophy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19735-0">
    <skos:prefLabel xml:lang="de">Anthroposophie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70839">
    <rdfs:label xml:lang="de">Gesundheits- und Veterinärwesen</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70342">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17569-1">
    <skos:related rdf:resource="descriptor/17570-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15498-1">
    <skos:broader rdf:resource="thsys/73103"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15498-1">
    <skos:broader rdf:resource="thsys/71035"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18896-2">
    <skos:related rdf:resource="descriptor/13486-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18896-2">
    <skos:related rdf:resource="descriptor/10300-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18896-2">
    <skos:related rdf:resource="descriptor/19118-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18896-2">
    <skos:related rdf:resource="descriptor/18556-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12598-5">
    <skos:altLabel xml:lang="de">Stundenlohn</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12598-5">
    <skos:altLabel xml:lang="de">Festlohn</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16273-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18556-5">
    <skos:related rdf:resource="descriptor/19754-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18556-5">
    <skos:related rdf:resource="descriptor/13486-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18556-5">
    <skos:related rdf:resource="descriptor/18896-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18556-5">
    <skos:related rdf:resource="descriptor/12907-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18556-5">
    <skos:related rdf:resource="descriptor/18917-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18556-5">
    <skos:related rdf:resource="descriptor/13481-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14463-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71095">
    <skos:narrower rdf:resource="descriptor/18188-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71095">
    <skos:narrower rdf:resource="descriptor/10748-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71095">
    <skos:narrower rdf:resource="descriptor/10722-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71095">
    <skos:narrower rdf:resource="descriptor/10772-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71095">
    <skos:narrower rdf:resource="descriptor/18127-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71095">
    <skos:narrower rdf:resource="descriptor/10721-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71095">
    <skos:narrower rdf:resource="descriptor/10764-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71095">
    <skos:narrower rdf:resource="descriptor/10763-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71095">
    <skos:narrower rdf:resource="descriptor/19333-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71095">
    <skos:narrower rdf:resource="descriptor/10757-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71095">
    <skos:narrower rdf:resource="descriptor/19332-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71095">
    <skos:narrower rdf:resource="descriptor/10753-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71095">
    <skos:narrower rdf:resource="descriptor/10752-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71095">
    <skos:narrower rdf:resource="descriptor/10750-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71095">
    <skos:narrower rdf:resource="descriptor/10667-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71095">
    <skos:narrower rdf:resource="descriptor/10738-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71095">
    <skos:narrower rdf:resource="descriptor/10730-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71095">
    <skos:narrower rdf:resource="descriptor/10719-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71095">
    <skos:narrower rdf:resource="descriptor/10776-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71095">
    <skos:narrower rdf:resource="descriptor/10718-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71095">
    <skos:narrower rdf:resource="descriptor/10717-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71095">
    <skos:narrower rdf:resource="thsys/71094"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10751-5">
    <skos:broader rdf:resource="thsys/71094"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10751-5">
    <skos:broader rdf:resource="descriptor/19335-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17553-2">
    <skos:prefLabel xml:lang="en">Asia</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17553-2">
    <skos:prefLabel xml:lang="de">Asien</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18323-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11370-5">
    <skos:altLabel xml:lang="de">Polytechnic</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11370-5">
    <skos:altLabel xml:lang="de">Hochschule für Angewandte Wissenschaften</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11370-5">
    <skos:altLabel xml:lang="de">HAW (Hochschule für Angewandte Wissenschaften)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12009-1">
    <skos:altLabel xml:lang="de">Institutionelle Rahmenbedingungen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21094-4">
    <skos:broader rdf:resource="thsys/71024"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21094-4">
    <skos:broader rdf:resource="descriptor/18313-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11398-4">
    <skos:broader rdf:resource="thsys/73369"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11398-4">
    <skos:broader rdf:resource="thsys/70847"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11398-4">
    <skos:broader rdf:resource="thsys/70176"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11398-4">
    <skos:broader rdf:resource="thsys/70143"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11398-4">
    <skos:broader rdf:resource="descriptor/16283-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11398-4">
    <skos:broader rdf:resource="descriptor/11397-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10521-1">
    <skos:related rdf:resource="descriptor/11782-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10521-1">
    <skos:related rdf:resource="descriptor/11790-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10521-1">
    <skos:related rdf:resource="descriptor/10350-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10521-1">
    <skos:related rdf:resource="descriptor/16603-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10521-1">
    <skos:related rdf:resource="descriptor/10537-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10521-1">
    <skos:related rdf:resource="descriptor/16601-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10521-1">
    <skos:related rdf:resource="descriptor/13425-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13270-6">
    <skos:altLabel xml:lang="de">Post- und Fernmeldewesen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12947-6">
    <skos:related rdf:resource="descriptor/13161-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12947-6">
    <skos:related rdf:resource="descriptor/19342-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12947-6">
    <skos:related rdf:resource="descriptor/14110-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12947-6">
    <skos:related rdf:resource="descriptor/13225-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14127-6">
    <skos:altLabel xml:lang="de">Fell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20241-1">
    <skos:altLabel xml:lang="en">Virtual organization</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20241-1">
    <skos:altLabel xml:lang="en">Virtual organisation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20241-1">
    <skos:altLabel xml:lang="de">Virtuelle Organisation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70355">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14788-4">
    <skos:narrower rdf:resource="descriptor/14794-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14788-4">
    <skos:narrower rdf:resource="descriptor/14793-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14788-4">
    <skos:narrower rdf:resource="descriptor/14789-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14788-4">
    <skos:narrower rdf:resource="descriptor/14787-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14788-4">
    <skos:narrower rdf:resource="descriptor/14784-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14258-5">
    <skos:broader rdf:resource="thsys/70348"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14258-5">
    <skos:broader rdf:resource="descriptor/14021-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15458-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14131-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10570-2">
    <skos:broader rdf:resource="thsys/71098"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10570-2">
    <skos:broader rdf:resource="thsys/71085"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10570-2">
    <skos:broader rdf:resource="descriptor/18481-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10570-2">
    <skos:broader rdf:resource="descriptor/10524-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10570-2">
    <skos:broader rdf:resource="descriptor/10686-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13069-6">
    <skos:broader rdf:resource="thsys/70997"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13069-6">
    <skos:broader rdf:resource="descriptor/13065-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73338">
    <skos:broader rdf:resource="thsys/73342"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18259-4">
    <skos:broader rdf:resource="thsys/73360"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18259-4">
    <skos:broader rdf:resource="thsys/70993"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18259-4">
    <skos:broader rdf:resource="descriptor/16019-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15798-3">
    <skos:prefLabel xml:lang="en">Communication</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15798-3">
    <skos:prefLabel xml:lang="de">Kommunikation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13553-1">
    <skos:prefLabel xml:lang="en">Cargo shipping</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13553-1">
    <skos:prefLabel xml:lang="de">Frachtschifffahrt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14151-2">
    <skos:related rdf:resource="descriptor/19366-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14151-2">
    <skos:related rdf:resource="descriptor/14817-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14151-2">
    <skos:related rdf:resource="descriptor/13004-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14444-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16522-4">
    <skos:prefLabel xml:lang="en">Military service</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16522-4">
    <skos:prefLabel xml:lang="de">Wehrdienst</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17731-4">
    <skos:broader rdf:resource="thsys/70445"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17731-4">
    <skos:broader rdf:resource="descriptor/17672-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10348-3">
    <skos:broader rdf:resource="thsys/70020"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10348-3">
    <skos:broader rdf:resource="descriptor/10347-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17947-2">
    <skos:altLabel xml:lang="de">Westsamoa</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17947-2">
    <skos:altLabel xml:lang="de">Unabhängiger Staat Westsamoa</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11586-3">
    <skos:broader rdf:resource="thsys/71081"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11586-3">
    <skos:broader rdf:resource="descriptor/11596-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15730-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24786-2">
    <skos:scopeNote xml:lang="de">Spezielle Form der privaten Rentenversicherung, wobei dem Gläubiger i.d.R. das Recht zum regelmäßigen Empfang einer bestimmten Geldsumme auf Lebenszeit zugesichert wird.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18345-4">
    <skos:prefLabel xml:lang="en">Local public utilities</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18345-4">
    <skos:prefLabel xml:lang="de">Kommunale Versorgungswirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13756-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12485-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15265-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19658-6">
    <skos:broader rdf:resource="thsys/70921"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19658-6">
    <skos:broader rdf:resource="descriptor/13682-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19658-6">
    <skos:broader rdf:resource="descriptor/13655-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19522-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16344-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73352">
    <skos:prefLabel xml:lang="de">N.10.01.02  Klima, Wetter, Luft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id216">
    <rdf:value xml:lang="de">Für die makroökonomische Konsumfunktion</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12407-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70011">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.06.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13274-5">
    <skos:altLabel xml:lang="de">Postwesen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13274-5">
    <skos:altLabel xml:lang="de">Post</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18098-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14139-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19625-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12925-2">
    <skos:related rdf:resource="descriptor/18366-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12925-2">
    <skos:related rdf:resource="descriptor/18447-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12925-2">
    <skos:related rdf:resource="descriptor/15005-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12925-2">
    <skos:related rdf:resource="descriptor/18814-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12925-2">
    <skos:related rdf:resource="descriptor/16271-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12925-2">
    <skos:related rdf:resource="descriptor/13034-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12925-2">
    <skos:related rdf:resource="descriptor/13058-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12925-2">
    <skos:related rdf:resource="descriptor/13047-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12925-2">
    <skos:related rdf:resource="descriptor/13072-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70561">
    <skos:broader rdf:resource="thsys/70148"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19827-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15918-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11698-6">
    <skos:prefLabel xml:lang="en">Destination principle</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11698-6">
    <skos:prefLabel xml:lang="de">Bestimmungslandprinzip</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16927-6">
    <skos:related rdf:resource="descriptor/18131-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16927-6">
    <skos:related rdf:resource="descriptor/16853-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12370-0">
    <skos:broader rdf:resource="thsys/70383"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12370-0">
    <skos:broader rdf:resource="thsys/70186"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12370-0">
    <skos:broader rdf:resource="descriptor/12381-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12370-0">
    <skos:broader rdf:resource="descriptor/12164-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11416-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12774-4">
    <skos:prefLabel xml:lang="en">Market research</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12774-4">
    <skos:prefLabel xml:lang="de">Marktforschung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12540-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18684-3">
    <skos:prefLabel xml:lang="en">Index bond</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18684-3">
    <skos:prefLabel xml:lang="de">Indexanleihe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11782-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15488-4">
    <skos:broader rdf:resource="thsys/73103"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15488-4">
    <skos:broader rdf:resource="thsys/71035"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15488-4">
    <skos:broader rdf:resource="descriptor/15469-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16010-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10733-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15734-1">
    <skos:altLabel xml:lang="en">Water preservation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15734-1">
    <skos:altLabel xml:lang="de">Wasserschutzgebiet</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15734-1">
    <skos:altLabel xml:lang="de">Wasserschutz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15734-1">
    <skos:altLabel xml:lang="de">Wasserreinhaltung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15734-1">
    <skos:altLabel xml:lang="de">Gewässerreinhaltung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18487-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19211-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10298-6">
    <skos:scopeNote xml:lang="de">Nur für private Endverbraucher/innen als Einzelpersonen, Haushalt oder größere Gruppe mit konsumrelevanten Merkmalen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13402-1">
    <skos:altLabel xml:lang="de">Gemeinnütziger Verein</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19793-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70040">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">B.07.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10378-1">
    <skos:related rdf:resource="descriptor/19202-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10378-1">
    <skos:related rdf:resource="descriptor/10398-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10378-1">
    <skos:related rdf:resource="descriptor/10395-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10378-1">
    <skos:related rdf:resource="descriptor/10970-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10378-1">
    <skos:related rdf:resource="descriptor/19198-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10378-1">
    <skos:related rdf:resource="descriptor/10382-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12558-3">
    <skos:prefLabel xml:lang="en">Tax depreciation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12558-3">
    <skos:prefLabel xml:lang="de">Steuerrechtliche Abschreibung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19680-6">
    <skos:broader rdf:resource="thsys/73383"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19680-6">
    <skos:broader rdf:resource="descriptor/16405-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19536-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12863-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19234-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18708-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70082">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.12</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16033-5">
    <skos:narrower rdf:resource="descriptor/13437-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13974-5">
    <skos:broader rdf:resource="thsys/71055"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13974-5">
    <skos:broader rdf:resource="thsys/70814"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13974-5">
    <skos:broader rdf:resource="descriptor/18535-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12125-6">
    <skos:broader rdf:resource="thsys/71054"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12125-6">
    <skos:broader rdf:resource="descriptor/13985-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10552-4">
    <skos:narrower rdf:resource="descriptor/19369-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10552-4">
    <skos:narrower rdf:resource="descriptor/10573-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10552-4">
    <skos:narrower rdf:resource="descriptor/10553-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15658-5">
    <skos:scopeNote xml:lang="de">Nur im Sinne der Lehre von Gesundheit und Krankheit.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14771-0">
    <skos:altLabel xml:lang="de">Waschbecken</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14771-0">
    <skos:altLabel xml:lang="de">Toilettenbecken</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14771-0">
    <skos:altLabel xml:lang="de">Installationskeramik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13674-3">
    <skos:prefLabel xml:lang="en">Broadband telecommunications</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13674-3">
    <skos:prefLabel xml:lang="de">Breitbandkommunikation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10187-1">
    <skos:broader rdf:resource="thsys/70481"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10187-1">
    <skos:broader rdf:resource="thsys/70270"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10187-1">
    <skos:broader rdf:resource="thsys/70233"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10187-1">
    <skos:broader rdf:resource="descriptor/12742-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18450-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12033-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16547-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19117-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12807-1">
    <skos:altLabel xml:lang="de">Produktzyklus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12807-1">
    <skos:altLabel xml:lang="de">Lebenszyklus des Produkts</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15761-5">
    <skos:prefLabel xml:lang="en">Environmental regulation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15761-5">
    <skos:prefLabel xml:lang="de">Umweltauflage</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18966-0">
    <skos:related rdf:resource="descriptor/15774-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18966-0">
    <skos:related rdf:resource="descriptor/12928-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18966-0">
    <skos:related rdf:resource="descriptor/13962-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18966-0">
    <skos:related rdf:resource="descriptor/12879-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19486-2">
    <skos:scopeNote xml:lang="de">Auch für bodennahes Ozon, Sommersmog.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15912-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19219-4">
    <skos:prefLabel xml:lang="en">Methodological individualism</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19219-4">
    <skos:prefLabel xml:lang="de">Methodologischer Individualismus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15437-0">
    <skos:broader rdf:resource="thsys/73104"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15437-0">
    <skos:broader rdf:resource="descriptor/15458-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15437-0">
    <skos:broader rdf:resource="descriptor/15555-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18540-6">
    <skos:broader rdf:resource="thsys/70952"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18540-6">
    <skos:broader rdf:resource="thsys/70902"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18540-6">
    <skos:broader rdf:resource="descriptor/13291-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24740-5">
    <skos:scopeNote xml:lang="de">Investmentfonds, die mit hochspekulativen Anlagestrategien hohe Renditen erwirtschaften wollen. Ihre Anlagepolitik unterliegt keinen Beschränkungen, so dass sie alle Arten von Finanzinstrumenten einsetzen können.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16532-1">
    <skos:prefLabel xml:lang="en">Disability benefits</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16532-1">
    <skos:prefLabel xml:lang="de">Erwerbsunfähigkeitsrente</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18066-1">
    <skos:prefLabel xml:lang="en">Counterfeiting</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18066-1">
    <skos:prefLabel xml:lang="de">Produktpiraterie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12850-0">
    <skos:altLabel xml:lang="de">Nicht abzugsfähige Betriebsausgaben</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12992-1">
    <skos:related rdf:resource="descriptor/14222-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11587-1">
    <skos:broader rdf:resource="thsys/71081"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11587-1">
    <skos:broader rdf:resource="descriptor/11596-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19684-5">
    <skos:related rdf:resource="descriptor/19688-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19684-5">
    <skos:related rdf:resource="descriptor/16013-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19684-5">
    <skos:related rdf:resource="descriptor/19714-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19684-5">
    <skos:related rdf:resource="descriptor/16309-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19684-5">
    <skos:related rdf:resource="descriptor/16288-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19684-5">
    <skos:related rdf:resource="descriptor/16472-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19684-5">
    <skos:related rdf:resource="descriptor/18062-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19684-5">
    <skos:related rdf:resource="descriptor/19612-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19684-5">
    <skos:related rdf:resource="descriptor/19616-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19684-5">
    <skos:related rdf:resource="descriptor/15878-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10310-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16008-4">
    <skos:prefLabel xml:lang="en">Party law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16008-4">
    <skos:prefLabel xml:lang="de">Parteienrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14716-1">
    <skos:broader rdf:resource="thsys/70476"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14716-1">
    <skos:broader rdf:resource="descriptor/14702-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10182-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12452-5">
    <skos:prefLabel xml:lang="en">Group accounting</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12452-5">
    <skos:prefLabel xml:lang="de">Konzernabschluss</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17567-5">
    <skos:broader rdf:resource="thsys/70147"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17567-5">
    <skos:broader rdf:resource="descriptor/19165-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14199-2">
    <skos:prefLabel xml:lang="en">Fluorspar</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14199-2">
    <skos:prefLabel xml:lang="de">Flussspat</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19683-0">
    <skos:prefLabel xml:lang="en">Political stability</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19683-0">
    <skos:prefLabel xml:lang="de">Politische Stabilität</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11473-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12808-6">
    <skos:narrower rdf:resource="descriptor/13474-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12808-6">
    <skos:narrower rdf:resource="descriptor/12759-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12808-6">
    <skos:narrower rdf:resource="descriptor/12810-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12808-6">
    <skos:narrower rdf:resource="descriptor/12805-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12808-6">
    <skos:narrower rdf:resource="descriptor/19129-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12808-6">
    <skos:narrower rdf:resource="descriptor/10546-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16100-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16192-6">
    <skos:broader rdf:resource="thsys/73356"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16192-6">
    <skos:broader rdf:resource="thsys/73334"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16192-6">
    <skos:broader rdf:resource="descriptor/16195-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14044-3">
    <skos:altLabel xml:lang="de">Technisches Öl</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14044-3">
    <skos:altLabel xml:lang="de">Technisches Fett</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14044-3">
    <skos:altLabel xml:lang="de">Technische Fettsäure</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14044-3">
    <skos:altLabel xml:lang="de">Ölsäure</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14044-3">
    <skos:altLabel xml:lang="de">Fett</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12823-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10060-4">
    <skos:prefLabel xml:lang="en">Transaction costs</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10060-4">
    <skos:prefLabel xml:lang="de">Transaktionskosten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12983-2">
    <skos:broader rdf:resource="thsys/71006"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12983-2">
    <skos:broader rdf:resource="descriptor/12980-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17621-4">
    <skos:broader rdf:resource="thsys/70253"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17621-4">
    <skos:broader rdf:resource="descriptor/17608-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17621-4">
    <skos:broader rdf:resource="descriptor/17978-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70321">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.06.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10256-1">
    <skos:altLabel xml:lang="de">Produktionstheorie der Gesamtwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10256-1">
    <skos:altLabel xml:lang="de">Makroökonomische Produktionstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14094-2">
    <skos:prefLabel xml:lang="en">Cassava</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14094-2">
    <skos:prefLabel xml:lang="de">Maniok</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18252-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16658-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71088">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18782-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71070">
    <skos:narrower rdf:resource="descriptor/11903-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71070">
    <skos:narrower rdf:resource="descriptor/10542-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71070">
    <skos:narrower rdf:resource="descriptor/19444-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71070">
    <skos:narrower rdf:resource="descriptor/11934-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71070">
    <skos:narrower rdf:resource="descriptor/18160-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71070">
    <skos:narrower rdf:resource="descriptor/11932-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71070">
    <skos:narrower rdf:resource="descriptor/11931-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71070">
    <skos:narrower rdf:resource="descriptor/11912-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71070">
    <skos:narrower rdf:resource="descriptor/11951-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71070">
    <skos:narrower rdf:resource="descriptor/11929-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71070">
    <skos:narrower rdf:resource="descriptor/11909-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71070">
    <skos:narrower rdf:resource="descriptor/10426-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71070">
    <skos:narrower rdf:resource="descriptor/11907-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71070">
    <skos:narrower rdf:resource="descriptor/11959-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71070">
    <skos:narrower rdf:resource="descriptor/11849-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71070">
    <skos:narrower rdf:resource="descriptor/11928-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71070">
    <skos:narrower rdf:resource="descriptor/19429-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71070">
    <skos:narrower rdf:resource="descriptor/10219-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71070">
    <skos:narrower rdf:resource="descriptor/11873-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71070">
    <skos:narrower rdf:resource="descriptor/19252-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71070">
    <skos:narrower rdf:resource="descriptor/11927-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71070">
    <skos:narrower rdf:resource="descriptor/19443-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71070">
    <skos:narrower rdf:resource="descriptor/11956-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71070">
    <skos:narrower rdf:resource="descriptor/11954-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71070">
    <skos:narrower rdf:resource="descriptor/10562-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71070">
    <skos:narrower rdf:resource="descriptor/15033-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71070">
    <skos:narrower rdf:resource="descriptor/11921-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71070">
    <skos:narrower rdf:resource="descriptor/11919-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10081-3">
    <skos:broader rdf:resource="thsys/70188"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10081-3">
    <skos:broader rdf:resource="descriptor/10076-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11253-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15063-1">
    <skos:prefLabel xml:lang="en">Sequential analysis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15063-1">
    <skos:prefLabel xml:lang="de">Sequentialanalyse</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19063-2">
    <skos:related rdf:resource="descriptor/13193-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19063-2">
    <skos:related rdf:resource="descriptor/14042-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19063-2">
    <skos:related rdf:resource="descriptor/19053-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10142-2">
    <skos:related rdf:resource="descriptor/18869-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10142-2">
    <skos:related rdf:resource="descriptor/19766-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10142-2">
    <skos:related rdf:resource="descriptor/16269-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10142-2">
    <skos:related rdf:resource="descriptor/12775-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10142-2">
    <skos:related rdf:resource="descriptor/13662-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10142-2">
    <skos:related rdf:resource="descriptor/18082-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10142-2">
    <skos:related rdf:resource="descriptor/15961-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10142-2">
    <skos:related rdf:resource="descriptor/10141-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10142-2">
    <skos:related rdf:resource="descriptor/18943-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10142-2">
    <skos:related rdf:resource="descriptor/16604-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18539-5">
    <skos:altLabel xml:lang="de">Wechselkursversicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18539-5">
    <skos:altLabel xml:lang="de">Wechselkursmanagement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18539-5">
    <skos:altLabel xml:lang="de">Währungsrisikomanagement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18539-5">
    <skos:altLabel xml:lang="de">Kurssicherung im Außenhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18539-5">
    <skos:altLabel xml:lang="de">Kursrisikomanagement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18539-5">
    <skos:altLabel xml:lang="de">Fremdwährungsmanagement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18539-5">
    <skos:altLabel xml:lang="de">Devisenmanagement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18539-5">
    <skos:altLabel xml:lang="de">Betriebliches Devisenmanagement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10294-0">
    <skos:narrower rdf:resource="descriptor/19523-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10294-0">
    <skos:narrower rdf:resource="descriptor/10291-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10211-2">
    <skos:altLabel xml:lang="en">Time-of-use pricing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10211-2">
    <skos:altLabel xml:lang="en">Peak-load-problem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10211-2">
    <skos:altLabel xml:lang="de">Intertemporale Preisdiskriminierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#ConceptScheme">
    <rdfs:comment xml:lang="en">Thesauri, classification schemes, subject heading lists, taxonomies,
      'folksonomies', and other types of controlled vocabulary are all examples of concept schemes.
      Concept schemes are also embedded in glossaries and terminologies.</rdfs:comment>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#ConceptScheme">
    <rdfs:comment xml:lang="en">A concept scheme may be defined to include concepts from different
      sources.</rdfs:comment>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17689-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15761-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://zbw.eu/namespaces/skos-extensions/">
    <ns0:creator xmlns:ns0="http://purl.org/dc/elements/1.1/">Joachim Neubert</ns0:creator>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18930-0">
    <zbwext:useInsteadNote rdf:nodeID="id206"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19324-0">
    <skos:related rdf:resource="descriptor/18064-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19324-0">
    <skos:related rdf:resource="descriptor/19320-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19324-0">
    <skos:related rdf:resource="descriptor/10617-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19324-0">
    <skos:related rdf:resource="descriptor/18790-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19324-0">
    <skos:related rdf:resource="descriptor/10616-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19324-0">
    <skos:related rdf:resource="descriptor/10690-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19324-0">
    <skos:related rdf:resource="descriptor/10686-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19324-0">
    <skos:related rdf:resource="descriptor/19314-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13273-0">
    <skos:prefLabel xml:lang="en">Postal delivery</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13273-0">
    <skos:prefLabel xml:lang="de">Postbeförderung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15766-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71017">
    <skos:broader rdf:resource="thsys/w"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13330-0">
    <skos:broader rdf:resource="thsys/70876"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10616-4">
    <skos:related rdf:resource="descriptor/18790-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10616-4">
    <skos:related rdf:resource="descriptor/10880-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10616-4">
    <skos:related rdf:resource="descriptor/19324-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10616-4">
    <skos:related rdf:resource="descriptor/10686-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19446-0">
    <skos:broader rdf:resource="thsys/73367"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19446-0">
    <skos:broader rdf:resource="thsys/71064"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19446-0">
    <skos:broader rdf:resource="descriptor/10987-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19446-0">
    <skos:broader rdf:resource="descriptor/16549-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11030-1">
    <skos:related rdf:resource="descriptor/11061-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11030-1">
    <skos:related rdf:resource="descriptor/11004-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11030-1">
    <skos:related rdf:resource="descriptor/10559-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13472-1">
    <skos:altLabel xml:lang="de">Rohstoffgroßhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15093-6">
    <skos:broader rdf:resource="thsys/72184"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15093-6">
    <skos:broader rdf:resource="descriptor/15104-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15093-6">
    <skos:broader rdf:resource="descriptor/15082-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15900-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15616-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15072-0">
    <skos:related rdf:resource="descriptor/16214-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15072-0">
    <skos:related rdf:resource="descriptor/15396-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15072-0">
    <skos:related rdf:resource="descriptor/10031-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15072-0">
    <skos:related rdf:resource="descriptor/10471-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15072-0">
    <skos:related rdf:resource="descriptor/15912-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13801-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10169-3">
    <skos:altLabel xml:lang="en">Labor supply</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10169-3">
    <skos:altLabel xml:lang="en">Individual labour supply</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10169-3">
    <skos:altLabel xml:lang="de">Arbeitsbereitschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10169-3">
    <skos:altLabel xml:lang="de">Arbeitsangebotstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11777-3">
    <skos:prefLabel xml:lang="en">Wealth accumulation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11777-3">
    <skos:prefLabel xml:lang="de">Vermögensbildung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11459-3">
    <skos:scopeNote xml:lang="de">Rationierung von Krediten bei einer Überschussnachfrage (bei festem Zins) mit nichtpreislichen Maßnahmen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11723-5">
    <skos:altLabel xml:lang="en">Free-rider-problem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11723-5">
    <skos:altLabel xml:lang="en">Free riding</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11723-5">
    <skos:altLabel xml:lang="de">Trittbrettfahrer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16330-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12977-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15752-6">
    <skos:prefLabel xml:lang="en">Resource policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15752-6">
    <skos:prefLabel xml:lang="de">Rohstoffpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15018-6">
    <skos:altLabel xml:lang="en">Residential waste</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15018-6">
    <skos:altLabel xml:lang="en">Household waste</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15018-6">
    <skos:altLabel xml:lang="de">Sperrmüll</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19285-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14252-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70397">
    <rdfs:label xml:lang="de">Steuerrechtliche Gewinnermittlung</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12721-4">
    <skos:prefLabel xml:lang="en">Industrial production</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12721-4">
    <skos:prefLabel xml:lang="de">Industrielle Produktion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12993-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10107-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18393-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19831-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16081-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16207-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19077-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70486">
    <skos:narrower rdf:resource="descriptor/10135-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70486">
    <skos:narrower rdf:resource="descriptor/10126-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70486">
    <skos:narrower rdf:resource="descriptor/10125-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70486">
    <skos:narrower rdf:resource="descriptor/10121-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70486">
    <skos:narrower rdf:resource="descriptor/10123-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70486">
    <skos:narrower rdf:resource="descriptor/10122-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70486">
    <skos:narrower rdf:resource="descriptor/10120-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70486">
    <skos:narrower rdf:resource="descriptor/10124-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70486">
    <skos:narrower rdf:resource="descriptor/10811-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70486">
    <skos:narrower rdf:resource="descriptor/10115-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18178-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13523-3">
    <skos:prefLabel xml:lang="en">Train station</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13523-3">
    <skos:prefLabel xml:lang="de">Bahnhof</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12955-0">
    <skos:prefLabel xml:lang="en">Animal husbandry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12955-0">
    <skos:prefLabel xml:lang="de">Tierhaltung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19157-0">
    <skos:altLabel xml:lang="de">Partnerschaftsgesellschaftsgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18552-6">
    <skos:related rdf:resource="descriptor/20241-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18552-6">
    <skos:related rdf:resource="descriptor/18654-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18552-6">
    <skos:related rdf:resource="descriptor/19651-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18552-6">
    <skos:related rdf:resource="descriptor/13655-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18552-6">
    <skos:related rdf:resource="descriptor/12780-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18552-6">
    <skos:related rdf:resource="descriptor/19541-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19542-4">
    <skos:related rdf:resource="descriptor/12774-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19542-4">
    <skos:related rdf:resource="descriptor/10199-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18621-6">
    <skos:narrower rdf:resource="descriptor/18586-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19069-4">
    <skos:broader rdf:resource="thsys/71042"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12884-4">
    <skos:prefLabel xml:lang="en">Management control</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12884-4">
    <skos:prefLabel xml:lang="de">Controlling</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70937">
    <rdfs:label xml:lang="en">Motor vehicle traffic</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70937">
    <rdfs:label xml:lang="de">Kraftfahrzeugverkehr</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15908-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12681-4">
    <skos:broader rdf:resource="thsys/70549"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12681-4">
    <skos:broader rdf:resource="descriptor/14336-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13799-4">
    <skos:broader rdf:resource="thsys/70876"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13799-4">
    <skos:broader rdf:resource="thsys/70114"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13799-4">
    <skos:broader rdf:resource="descriptor/13793-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12992-1">
    <skos:prefLabel xml:lang="en">Uranium mining</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12992-1">
    <skos:prefLabel xml:lang="de">Uranbergbau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11252-4">
    <skos:prefLabel xml:lang="en">Vocational counselling</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11252-4">
    <skos:prefLabel xml:lang="de">Berufsberatung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15760-0">
    <skos:altLabel xml:lang="en">Pollution charge</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15760-0">
    <skos:altLabel xml:lang="en">Effluent charge</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15760-0">
    <skos:altLabel xml:lang="de">Emissionsabgabe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11170-6">
    <skos:related rdf:resource="descriptor/16274-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11890-0">
    <skos:prefLabel xml:lang="en">Village renewal</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11890-0">
    <skos:prefLabel xml:lang="de">Dorferneuerung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73331">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.05.08.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14421-6">
    <skos:narrower rdf:resource="descriptor/14422-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14421-6">
    <skos:narrower rdf:resource="descriptor/14420-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14421-6">
    <skos:narrower rdf:resource="descriptor/14418-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13830-1">
    <skos:related rdf:resource="descriptor/13849-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13830-1">
    <skos:related rdf:resource="descriptor/13831-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13830-1">
    <skos:related rdf:resource="descriptor/13845-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13830-1">
    <skos:related rdf:resource="descriptor/13829-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18089-3">
    <zbwext:useInsteadNote rdf:nodeID="id134"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13488-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19096-1">
    <skos:prefLabel xml:lang="en">Cost management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19096-1">
    <skos:prefLabel xml:lang="de">Kostenmanagement</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71076">
    <rdfs:label xml:lang="en">Theory of taxation</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71076">
    <rdfs:label xml:lang="de">Steuertheorie</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24661-1">
    <skos:broader rdf:resource="thsys/70562"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24661-1">
    <skos:broader rdf:resource="descriptor/12094-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71019">
    <rdfs:label xml:lang="en">Forestry</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71019">
    <rdfs:label xml:lang="de">Forstwirtschaft</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16409-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11481-3">
    <skos:prefLabel xml:lang="en">Monetary system</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11481-3">
    <skos:prefLabel xml:lang="de">Geldverfassung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13735-2">
    <skos:narrower rdf:resource="descriptor/18688-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13735-2">
    <skos:narrower rdf:resource="descriptor/18687-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13050-6">
    <skos:related rdf:resource="descriptor/14318-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20399-1">
    <skos:prefLabel xml:lang="en">Flexibility</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20399-1">
    <skos:prefLabel xml:lang="de">Flexibilität</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12164-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19761-6">
    <skos:scopeNote xml:lang="de">für Lebenszyklen von z.B. Unternehmen/Produkten BENUTZE Lebenszyklus bzw. engere Deskriptoren.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19761-6">
    <skos:scopeNote xml:lang="de">Für Biographien BENUTZE Publikationsart Biographie</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19761-6">
    <skos:scopeNote xml:lang="de">Nur für menschliche Lebensläufe.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14049-0">
    <skos:altLabel xml:lang="de">Harnstoffdünger</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14049-0">
    <skos:altLabel xml:lang="de">Ammoniumsulfat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14049-0">
    <skos:altLabel xml:lang="de">Ammonium Sulphate</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14049-0">
    <skos:altLabel xml:lang="de">Ammoniakdünger</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14586-2">
    <skos:altLabel xml:lang="de">Turngerät</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14586-2">
    <skos:altLabel xml:lang="de">Sportgerät</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14586-2">
    <skos:altLabel xml:lang="de">Gymnastikgerät</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19623-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12990-5">
    <skos:narrower rdf:resource="descriptor/12981-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10950-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11260-5">
    <skos:broader rdf:resource="thsys/71116"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11260-5">
    <skos:broader rdf:resource="thsys/71059"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11260-5">
    <skos:broader rdf:resource="thsys/70011"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11260-5">
    <skos:broader rdf:resource="descriptor/16554-5"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id278">
    <rdfs:seeAlso rdf:resource="descriptor/13295-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13228-5">
    <skos:broader rdf:resource="thsys/70959"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13228-5">
    <skos:broader rdf:resource="descriptor/13202-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/w">
    <skos:prefLabel xml:lang="en">W  Economic sectors</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/w">
    <skos:prefLabel xml:lang="de">W  Wirtschaftssektoren und spezielle Wirtschaftslehren</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12735-0">
    <skos:broader rdf:resource="thsys/71036"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12735-0">
    <skos:broader rdf:resource="thsys/70471"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12735-0">
    <skos:broader rdf:resource="descriptor/12730-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16524-0">
    <skos:scopeNote xml:lang="de">Alle Schutz- und Hilfsmaßnahmen für die Zivilbevölkerung bei Naturkatastrophen und im Krieg; im Inland und im Ausland.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11873-0">
    <skos:related rdf:resource="descriptor/11928-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11873-0">
    <skos:related rdf:resource="descriptor/19252-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11873-0">
    <skos:related rdf:resource="descriptor/10658-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13325-0">
    <skos:related rdf:resource="descriptor/12928-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13325-0">
    <skos:related rdf:resource="descriptor/18071-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13325-0">
    <skos:related rdf:resource="descriptor/15763-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13325-0">
    <skos:related rdf:resource="descriptor/15727-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13325-0">
    <skos:related rdf:resource="descriptor/18959-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13325-0">
    <skos:related rdf:resource="descriptor/15988-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13325-0">
    <skos:related rdf:resource="descriptor/18958-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13325-0">
    <skos:related rdf:resource="descriptor/15009-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11800-6">
    <zbwext:useInsteadNote rdf:nodeID="id7"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19243-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19076-0">
    <skos:prefLabel xml:lang="en">Southern Germany</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19076-0">
    <skos:prefLabel xml:lang="de">Süddeutschland</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18703-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10968-1">
    <skos:narrower rdf:resource="descriptor/16484-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10968-1">
    <skos:narrower rdf:resource="descriptor/19696-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10968-1">
    <skos:narrower rdf:resource="descriptor/18123-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12580-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11397-6">
    <skos:narrower rdf:resource="descriptor/11404-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11397-6">
    <skos:narrower rdf:resource="descriptor/11402-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11397-6">
    <skos:narrower rdf:resource="descriptor/18889-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11397-6">
    <skos:narrower rdf:resource="descriptor/11399-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11397-6">
    <skos:narrower rdf:resource="descriptor/11398-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11397-6">
    <skos:narrower rdf:resource="descriptor/11392-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13382-2">
    <skos:altLabel xml:lang="de">Reporterinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13382-2">
    <skos:altLabel xml:lang="de">Reporter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13382-2">
    <skos:altLabel xml:lang="de">Redakteurinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13382-2">
    <skos:altLabel xml:lang="de">Redakteure</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13382-2">
    <skos:altLabel xml:lang="de">Journalistinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11702-6">
    <skos:altLabel xml:lang="en">Tax relief</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11702-6">
    <skos:altLabel xml:lang="en">Tax expenditure</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11702-6">
    <skos:altLabel xml:lang="en">Tax credit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11702-6">
    <skos:altLabel xml:lang="de">Verlustzuweisungsgesellschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11702-6">
    <skos:altLabel xml:lang="de">Verlustzuweisung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11702-6">
    <skos:altLabel xml:lang="de">Verlustvortrag</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11702-6">
    <skos:altLabel xml:lang="de">Verlustrücktrag</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11702-6">
    <skos:altLabel xml:lang="de">Verlustabzug</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11702-6">
    <skos:altLabel xml:lang="de">Steuervorteil</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11702-6">
    <skos:altLabel xml:lang="de">Steuervergünstigung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11702-6">
    <skos:altLabel xml:lang="de">Steuerpräferenz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11702-6">
    <skos:altLabel xml:lang="de">Steuerfreiheit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11702-6">
    <skos:altLabel xml:lang="de">Steuerentlastung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11702-6">
    <skos:altLabel xml:lang="de">Steuerbefreiung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11702-6">
    <skos:altLabel xml:lang="de">Steueranreiz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11702-6">
    <skos:altLabel xml:lang="de">Steuerabzug</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11702-6">
    <skos:altLabel xml:lang="de">Abschreibungsgesellschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19178-6">
    <skos:prefLabel xml:lang="en">G-7 countries</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19178-6">
    <skos:prefLabel xml:lang="de">G-7-Staaten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19629-6">
    <zbwext:useInsteadNote rdf:nodeID="id252"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19012-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17983-5">
    <skos:narrower rdf:resource="descriptor/17641-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17983-5">
    <skos:narrower rdf:resource="descriptor/17181-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17983-5">
    <skos:narrower rdf:resource="descriptor/18224-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17983-5">
    <skos:narrower rdf:resource="descriptor/17161-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17983-5">
    <skos:narrower rdf:resource="descriptor/18226-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17983-5">
    <skos:narrower rdf:resource="descriptor/18225-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17983-5">
    <skos:narrower rdf:resource="descriptor/16989-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17983-5">
    <skos:narrower rdf:resource="descriptor/17104-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17983-5">
    <skos:narrower rdf:resource="descriptor/16826-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17983-5">
    <skos:narrower rdf:resource="descriptor/16950-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17983-5">
    <skos:narrower rdf:resource="descriptor/17433-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17983-5">
    <skos:narrower rdf:resource="descriptor/17013-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17983-5">
    <skos:narrower rdf:resource="descriptor/17198-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17983-5">
    <skos:narrower rdf:resource="descriptor/17477-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17983-5">
    <skos:narrower rdf:resource="descriptor/17476-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17983-5">
    <skos:narrower rdf:resource="descriptor/17039-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17983-5">
    <skos:narrower rdf:resource="descriptor/17413-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17983-5">
    <skos:narrower rdf:resource="descriptor/17374-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17983-5">
    <skos:narrower rdf:resource="descriptor/17184-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17983-5">
    <skos:narrower rdf:resource="descriptor/17255-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17983-5">
    <skos:narrower rdf:resource="descriptor/16982-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17983-5">
    <skos:narrower rdf:resource="descriptor/17471-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17983-5">
    <skos:narrower rdf:resource="descriptor/18012-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17983-5">
    <skos:narrower rdf:resource="descriptor/16995-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17983-5">
    <skos:narrower rdf:resource="descriptor/17201-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19106-3">
    <skos:broader rdf:resource="thsys/71041"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19106-3">
    <skos:broader rdf:resource="descriptor/12903-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70969">
    <skos:prefLabel xml:lang="de">W.07.01  Bausparten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13682-4">
    <skos:broader rdf:resource="thsys/70996"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13682-4">
    <skos:broader rdf:resource="thsys/70922"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13682-4">
    <skos:broader rdf:resource="descriptor/18426-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10735-3">
    <skos:prefLabel xml:lang="en">Fixed exchange rate</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10735-3">
    <skos:prefLabel xml:lang="de">Fester Wechselkurs</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19038-1">
    <skos:altLabel xml:lang="de">Versuch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19038-1">
    <skos:altLabel xml:lang="de">Test</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19038-1">
    <skos:altLabel xml:lang="de">Feldversuch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71036">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18358-2">
    <skos:altLabel xml:lang="de">Ländliche Elektrifizierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19061-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12009-1">
    <skos:related rdf:resource="descriptor/11069-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12009-1">
    <skos:related rdf:resource="descriptor/11068-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12009-1">
    <skos:related rdf:resource="descriptor/19283-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12009-1">
    <skos:related rdf:resource="descriptor/19211-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12009-1">
    <skos:related rdf:resource="descriptor/11161-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12009-1">
    <skos:related rdf:resource="descriptor/11716-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12009-1">
    <skos:related rdf:resource="descriptor/20855-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12009-1">
    <skos:related rdf:resource="descriptor/20826-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18489-1">
    <skos:narrower rdf:resource="descriptor/19437-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12988-6">
    <skos:narrower rdf:resource="descriptor/12992-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12988-6">
    <skos:narrower rdf:resource="descriptor/18392-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12988-6">
    <skos:narrower rdf:resource="descriptor/18391-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10735-3">
    <skos:altLabel xml:lang="de">Fixer Wechselkurs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14184-1">
    <skos:narrower rdf:resource="descriptor/18024-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14184-1">
    <skos:narrower rdf:resource="descriptor/14181-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14184-1">
    <skos:narrower rdf:resource="descriptor/18235-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16825-0">
    <skos:narrower rdf:resource="descriptor/16972-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16825-0">
    <skos:narrower rdf:resource="descriptor/16950-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16825-0">
    <skos:narrower rdf:resource="descriptor/16824-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16825-0">
    <skos:narrower rdf:resource="descriptor/18012-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15917-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#notation">
    <rdfs:isDefinedBy rdf:resource="http://www.w3.org/2004/02/skos/core"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19620-3">
    <skos:prefLabel xml:lang="en">Drought</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19620-3">
    <skos:prefLabel xml:lang="de">Dürre</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73296">
    <skos:broader rdf:resource="thsys/70990"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14816-4">
    <skos:narrower rdf:resource="descriptor/14614-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14816-4">
    <skos:narrower rdf:resource="descriptor/14046-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14849-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19061-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14596-6">
    <skos:narrower rdf:resource="descriptor/14626-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14596-6">
    <skos:narrower rdf:resource="descriptor/13427-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19066-3">
    <zbwext:useInsteadNote rdf:nodeID="id97"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19565-6">
    <skos:prefLabel xml:lang="en">Competitive strategy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19565-6">
    <skos:prefLabel xml:lang="de">Wettbewerbsstrategie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16050-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15021-3">
    <skos:broader rdf:resource="thsys/73103"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15021-3">
    <skos:broader rdf:resource="descriptor/19150-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15021-3">
    <skos:broader rdf:resource="descriptor/15072-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19432-4">
    <skos:broader rdf:resource="thsys/70242"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19432-4">
    <skos:broader rdf:resource="descriptor/10011-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14916-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10495-4">
    <skos:related rdf:resource="descriptor/18314-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10495-4">
    <skos:related rdf:resource="descriptor/18301-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10495-4">
    <skos:related rdf:resource="descriptor/14931-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10495-4">
    <skos:related rdf:resource="descriptor/19840-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10495-4">
    <skos:related rdf:resource="descriptor/13306-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10495-4">
    <skos:related rdf:resource="descriptor/18334-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10495-4">
    <skos:related rdf:resource="descriptor/18335-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10495-4">
    <skos:related rdf:resource="descriptor/14054-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19744-6">
    <skos:prefLabel xml:lang="en">Interdisciplinary research</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19744-6">
    <skos:prefLabel xml:lang="de">Interdisziplinäre Forschung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14543-6">
    <skos:related rdf:resource="descriptor/13083-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14543-6">
    <skos:related rdf:resource="descriptor/13231-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12705-2">
    <skos:broader rdf:resource="thsys/70549"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12705-2">
    <skos:broader rdf:resource="thsys/70030"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12705-2">
    <skos:broader rdf:resource="descriptor/12278-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12705-2">
    <skos:broader rdf:resource="descriptor/19094-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18276-4">
    <skos:prefLabel xml:lang="en">Kitchen furniture</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18276-4">
    <skos:prefLabel xml:lang="de">Küchenmöbel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12274-3">
    <skos:narrower rdf:resource="descriptor/12401-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12274-3">
    <skos:narrower rdf:resource="descriptor/12297-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17928-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18100-6">
    <skos:narrower rdf:resource="descriptor/21041-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10514-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11790-4">
    <skos:related rdf:resource="descriptor/11782-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11790-4">
    <skos:related rdf:resource="descriptor/12697-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11790-4">
    <skos:related rdf:resource="descriptor/10981-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11790-4">
    <skos:related rdf:resource="descriptor/11210-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11790-4">
    <skos:related rdf:resource="descriptor/10521-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11790-4">
    <skos:related rdf:resource="descriptor/18010-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10088-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14421-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13830-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17792-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15466-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12750-4">
    <skos:related rdf:resource="descriptor/12758-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12750-4">
    <skos:related rdf:resource="descriptor/12705-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12750-4">
    <skos:related rdf:resource="descriptor/19249-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12750-4">
    <skos:related rdf:resource="descriptor/12915-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12750-4">
    <skos:related rdf:resource="descriptor/12914-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13324-2">
    <skos:related rdf:resource="descriptor/15011-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13324-2">
    <skos:related rdf:resource="descriptor/12922-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13324-2">
    <skos:related rdf:resource="descriptor/18964-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13324-2">
    <skos:related rdf:resource="descriptor/24621-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12243-0">
    <skos:prefLabel xml:lang="en">Financial leasing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12243-0">
    <skos:prefLabel xml:lang="de">Finanzierungsleasing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18391-4">
    <skos:prefLabel xml:lang="en">Gold mining</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18391-4">
    <skos:prefLabel xml:lang="de">Goldbergbau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18612-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15073-5">
    <skos:narrower rdf:resource="descriptor/15075-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15073-5">
    <skos:narrower rdf:resource="descriptor/15088-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15073-5">
    <skos:narrower rdf:resource="descriptor/10494-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18093-5">
    <skos:related rdf:resource="descriptor/13453-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18093-5">
    <skos:related rdf:resource="descriptor/13440-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18093-5">
    <skos:related rdf:resource="descriptor/18993-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14980-5">
    <skos:altLabel xml:lang="de">Fruchtwein</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14980-5">
    <skos:altLabel xml:lang="de">Apfelwein</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70461">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16173-3">
    <skos:related rdf:resource="descriptor/15900-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16173-3">
    <skos:related rdf:resource="descriptor/15899-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16173-3">
    <skos:related rdf:resource="descriptor/18419-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16173-3">
    <skos:related rdf:resource="descriptor/13507-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16173-3">
    <skos:related rdf:resource="descriptor/18605-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16898-1">
    <skos:broader rdf:resource="thsys/70361"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16898-1">
    <skos:broader rdf:resource="descriptor/18012-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12065-5">
    <skos:altLabel xml:lang="de">Herstellung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12065-5">
    <skos:altLabel xml:lang="de">Fertigung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12065-5">
    <skos:altLabel xml:lang="de">Erzeugung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19496-6">
    <skos:related rdf:resource="descriptor/11209-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19496-6">
    <skos:related rdf:resource="descriptor/11290-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73370">
    <rdfs:label xml:lang="de">Gesundheits- und Sozialpolitik</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19193-3">
    <skos:prefLabel xml:lang="en">Potato production</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19193-3">
    <skos:prefLabel xml:lang="de">Kartoffelanbau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12089-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19163-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12259-6">
    <skos:related rdf:resource="descriptor/12205-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12259-6">
    <skos:related rdf:resource="descriptor/12284-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12259-6">
    <skos:related rdf:resource="descriptor/13727-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12259-6">
    <skos:related rdf:resource="descriptor/12273-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18758-0">
    <skos:prefLabel xml:lang="en">Computer leasing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18758-0">
    <skos:prefLabel xml:lang="de">Computer-Leasing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70860">
    <skos:narrower rdf:resource="descriptor/12931-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70860">
    <skos:narrower rdf:resource="descriptor/11633-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70860">
    <skos:narrower rdf:resource="descriptor/11142-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70860">
    <skos:narrower rdf:resource="descriptor/18822-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70860">
    <skos:narrower rdf:resource="descriptor/11628-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70860">
    <skos:narrower rdf:resource="descriptor/13249-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70860">
    <skos:narrower rdf:resource="descriptor/10267-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70860">
    <skos:narrower rdf:resource="descriptor/11629-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70860">
    <skos:narrower rdf:resource="descriptor/11612-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70860">
    <skos:narrower rdf:resource="descriptor/11556-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70860">
    <skos:narrower rdf:resource="descriptor/18705-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70860">
    <skos:narrower rdf:resource="descriptor/11627-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70860">
    <skos:narrower rdf:resource="descriptor/11552-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70860">
    <skos:narrower rdf:resource="descriptor/11626-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70860">
    <skos:narrower rdf:resource="thsys/70859"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70860">
    <skos:narrower rdf:resource="thsys/70858"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18588-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16160-5">
    <skos:related rdf:resource="descriptor/13825-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16160-5">
    <skos:related rdf:resource="descriptor/13418-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13051-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13374-1">
    <skos:related rdf:resource="descriptor/18901-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13374-1">
    <skos:related rdf:resource="descriptor/14731-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13374-1">
    <skos:related rdf:resource="descriptor/14169-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13374-1">
    <skos:related rdf:resource="descriptor/13371-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13374-1">
    <skos:related rdf:resource="descriptor/18890-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13374-1">
    <skos:related rdf:resource="descriptor/13871-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11576-6">
    <skos:related rdf:resource="descriptor/11579-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11576-6">
    <skos:related rdf:resource="descriptor/12035-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19278-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20497-1">
    <skos:broader rdf:resource="thsys/70485"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20497-1">
    <skos:broader rdf:resource="descriptor/12085-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17599-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id190">
    <rdf:value xml:lang="de">Für wiederholte Stichprobenziehungen aus vorhandenen Daten</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19149-6">
    <skos:prefLabel xml:lang="en">Pretest</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19149-6">
    <skos:prefLabel xml:lang="de">Pretest</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/a">
    <skos:narrower rdf:resource="thsys/70582"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id60">
    <rdfs:seeAlso rdf:resource="descriptor/14023-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12384-3">
    <skos:prefLabel xml:lang="en">Principle of equality of treatment in the tax and commercial balance sheet</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12384-3">
    <skos:prefLabel xml:lang="de">Maßgeblichkeitsprinzip</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18171-4">
    <skos:altLabel xml:lang="de">Deterministisches Chaos</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73392">
    <skos:narrower rdf:resource="descriptor/13782-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73392">
    <skos:narrower rdf:resource="descriptor/16203-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73392">
    <skos:narrower rdf:resource="descriptor/19859-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73392">
    <skos:narrower rdf:resource="descriptor/16039-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73392">
    <skos:narrower rdf:resource="descriptor/15982-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73392">
    <skos:narrower rdf:resource="descriptor/16485-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73392">
    <skos:narrower rdf:resource="descriptor/19839-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73392">
    <skos:narrower rdf:resource="descriptor/18186-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73392">
    <skos:narrower rdf:resource="descriptor/19856-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73392">
    <skos:narrower rdf:resource="descriptor/16036-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73392">
    <skos:narrower rdf:resource="descriptor/16035-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73392">
    <skos:narrower rdf:resource="descriptor/18637-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73392">
    <skos:narrower rdf:resource="descriptor/16034-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73392">
    <skos:narrower rdf:resource="descriptor/19823-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73392">
    <skos:narrower rdf:resource="descriptor/11840-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73392">
    <skos:narrower rdf:resource="descriptor/11839-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73392">
    <skos:narrower rdf:resource="descriptor/18760-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73392">
    <skos:narrower rdf:resource="descriptor/18725-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73392">
    <skos:narrower rdf:resource="descriptor/12252-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73392">
    <skos:narrower rdf:resource="descriptor/19159-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73392">
    <skos:narrower rdf:resource="descriptor/10824-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73392">
    <skos:narrower rdf:resource="descriptor/13775-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73392">
    <skos:narrower rdf:resource="descriptor/18067-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73392">
    <skos:narrower rdf:resource="descriptor/12028-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73392">
    <skos:narrower rdf:resource="descriptor/12277-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73392">
    <skos:narrower rdf:resource="descriptor/19088-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73392">
    <skos:narrower rdf:resource="descriptor/19124-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73392">
    <skos:narrower rdf:resource="descriptor/18525-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73392">
    <skos:narrower rdf:resource="descriptor/16030-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73392">
    <skos:narrower rdf:resource="descriptor/10693-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73392">
    <skos:narrower rdf:resource="descriptor/16029-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73392">
    <skos:narrower rdf:resource="descriptor/16028-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73392">
    <skos:narrower rdf:resource="descriptor/11040-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73392">
    <skos:narrower rdf:resource="descriptor/16027-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73392">
    <skos:narrower rdf:resource="descriptor/18433-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73392">
    <skos:narrower rdf:resource="descriptor/12246-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73392">
    <skos:narrower rdf:resource="descriptor/16026-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73392">
    <skos:narrower rdf:resource="descriptor/19729-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73392">
    <skos:narrower rdf:resource="descriptor/12236-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73392">
    <skos:narrower rdf:resource="descriptor/12033-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12075-2">
    <skos:broader rdf:resource="thsys/73384"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12075-2">
    <skos:broader rdf:resource="thsys/70485"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11829-3">
    <skos:related rdf:resource="descriptor/11828-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11829-3">
    <skos:related rdf:resource="descriptor/12013-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11829-3">
    <skos:related rdf:resource="descriptor/18624-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14833-4">
    <skos:broader rdf:resource="thsys/70313"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18118-1">
    <skos:altLabel xml:lang="de">Risiko-Management</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18118-1">
    <skos:altLabel xml:lang="de">Risiko des Betriebs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18118-1">
    <skos:altLabel xml:lang="de">Betriebswirtschaftliches Risiko</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18118-1">
    <skos:altLabel xml:lang="de">Betriebsrisiko</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19258-1">
    <skos:prefLabel xml:lang="en">Liquidity effect</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19258-1">
    <skos:prefLabel xml:lang="de">Liquiditätseffekt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18595-2">
    <skos:broader rdf:resource="thsys/70934"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18595-2">
    <skos:broader rdf:resource="thsys/70930"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18595-2">
    <skos:broader rdf:resource="descriptor/13553-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18595-2">
    <skos:broader rdf:resource="descriptor/13615-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13156-4">
    <skos:related rdf:resource="descriptor/12955-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13156-4">
    <skos:related rdf:resource="descriptor/19292-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13156-4">
    <skos:related rdf:resource="descriptor/14989-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15971-1">
    <skos:related rdf:resource="descriptor/15976-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15971-1">
    <skos:related rdf:resource="descriptor/13995-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15971-1">
    <skos:related rdf:resource="descriptor/15966-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15971-1">
    <skos:related rdf:resource="descriptor/19841-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15971-1">
    <skos:related rdf:resource="descriptor/16315-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15971-1">
    <skos:related rdf:resource="descriptor/13410-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12757-4">
    <skos:altLabel xml:lang="de">Produktionsorganisation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12757-4">
    <skos:altLabel xml:lang="de">Fertigungssegmentierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14948-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15894-0">
    <skos:altLabel xml:lang="de">Semiarides Klima</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15894-0">
    <skos:altLabel xml:lang="de">Semi-Arid Zone</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15894-0">
    <skos:altLabel xml:lang="de">Arides Klima</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15894-0">
    <skos:altLabel xml:lang="de">Aride Zone</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71057">
    <skos:narrower rdf:resource="descriptor/18793-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71057">
    <skos:narrower rdf:resource="descriptor/11276-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71057">
    <skos:narrower rdf:resource="descriptor/13436-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71057">
    <skos:narrower rdf:resource="descriptor/18509-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71057">
    <skos:narrower rdf:resource="descriptor/13874-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71057">
    <skos:narrower rdf:resource="descriptor/13873-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71057">
    <skos:narrower rdf:resource="descriptor/13881-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71057">
    <skos:narrower rdf:resource="thsys/71052"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71057">
    <skos:narrower rdf:resource="thsys/71055"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71057">
    <skos:narrower rdf:resource="thsys/71056"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70239">
    <skos:narrower rdf:resource="descriptor/18379-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70239">
    <skos:narrower rdf:resource="descriptor/18623-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70239">
    <skos:narrower rdf:resource="descriptor/18557-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70239">
    <skos:narrower rdf:resource="descriptor/15775-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70239">
    <skos:narrower rdf:resource="descriptor/15774-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70239">
    <skos:narrower rdf:resource="descriptor/12928-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70239">
    <skos:narrower rdf:resource="descriptor/15773-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70239">
    <skos:narrower rdf:resource="descriptor/15772-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70239">
    <skos:narrower rdf:resource="descriptor/15771-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70239">
    <skos:narrower rdf:resource="descriptor/15770-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70239">
    <skos:narrower rdf:resource="descriptor/18071-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70239">
    <skos:narrower rdf:resource="descriptor/15769-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70239">
    <skos:narrower rdf:resource="descriptor/15768-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70239">
    <skos:narrower rdf:resource="descriptor/19483-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70239">
    <skos:narrower rdf:resource="descriptor/13400-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70239">
    <skos:narrower rdf:resource="descriptor/18186-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70239">
    <skos:narrower rdf:resource="descriptor/15764-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70239">
    <skos:narrower rdf:resource="descriptor/15762-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70239">
    <skos:narrower rdf:resource="descriptor/15761-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70239">
    <skos:narrower rdf:resource="descriptor/19480-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70239">
    <skos:narrower rdf:resource="descriptor/15760-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70239">
    <skos:narrower rdf:resource="descriptor/18640-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70239">
    <skos:narrower rdf:resource="descriptor/12605-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70239">
    <skos:narrower rdf:resource="descriptor/19473-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70239">
    <skos:narrower rdf:resource="descriptor/19560-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70239">
    <skos:narrower rdf:resource="descriptor/18964-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70239">
    <skos:narrower rdf:resource="descriptor/13324-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70239">
    <skos:narrower rdf:resource="descriptor/18072-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70239">
    <skos:narrower rdf:resource="descriptor/18183-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70239">
    <skos:narrower rdf:resource="descriptor/19449-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70239">
    <skos:narrower rdf:resource="descriptor/15746-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70239">
    <skos:narrower rdf:resource="descriptor/15742-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70239">
    <skos:narrower rdf:resource="descriptor/15740-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70239">
    <skos:narrower rdf:resource="descriptor/19632-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70239">
    <skos:narrower rdf:resource="descriptor/19481-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70239">
    <skos:narrower rdf:resource="descriptor/20967-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70239">
    <skos:narrower rdf:resource="descriptor/15738-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70239">
    <skos:narrower rdf:resource="descriptor/15737-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70239">
    <skos:narrower rdf:resource="descriptor/15734-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70239">
    <skos:narrower rdf:resource="descriptor/19727-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70239">
    <skos:narrower rdf:resource="descriptor/20762-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70239">
    <skos:narrower rdf:resource="descriptor/20744-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70239">
    <skos:narrower rdf:resource="descriptor/12021-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70239">
    <skos:narrower rdf:resource="descriptor/18327-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70239">
    <skos:narrower rdf:resource="descriptor/18336-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70239">
    <skos:narrower rdf:resource="descriptor/15730-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70239">
    <skos:narrower rdf:resource="descriptor/18958-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70239">
    <skos:narrower rdf:resource="descriptor/18957-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13853-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14163-2">
    <skos:broader rdf:resource="thsys/70506"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14163-2">
    <skos:broader rdf:resource="thsys/70403"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14163-2">
    <skos:broader rdf:resource="descriptor/14158-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18722-0">
    <skos:altLabel xml:lang="en">Capital adequacy standard</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18722-0">
    <skos:altLabel xml:lang="en">Capital adequacy regulation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18722-0">
    <skos:altLabel xml:lang="de">Liquiditätsvorschriften</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18722-0">
    <skos:altLabel xml:lang="de">Eigenmittelvorschriften</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18722-0">
    <skos:altLabel xml:lang="de">Eigenkapitalunterlegung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18722-0">
    <skos:altLabel xml:lang="de">Eigenkapitalgrundsatz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18722-0">
    <skos:altLabel xml:lang="de">Eigenkapitalanforderungen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11659-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11990-3">
    <skos:scopeNote xml:lang="de">Schriften, die sich auf theoretischer und empirischer Grundlage mit ökonomischen Problemen der Stadt und der Verstädterung befassen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11417-5">
    <skos:related rdf:resource="descriptor/10764-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11417-5">
    <skos:related rdf:resource="descriptor/11481-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11417-5">
    <skos:related rdf:resource="descriptor/11419-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11417-5">
    <skos:related rdf:resource="descriptor/11415-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11417-5">
    <skos:related rdf:resource="descriptor/13693-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11965-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10032-2">
    <skos:related rdf:resource="descriptor/16727-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10032-2">
    <skos:related rdf:resource="descriptor/14017-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10032-2">
    <skos:related rdf:resource="descriptor/18871-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10032-2">
    <skos:related rdf:resource="descriptor/16687-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10032-2">
    <skos:related rdf:resource="descriptor/15836-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10032-2">
    <skos:related rdf:resource="descriptor/15684-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10032-2">
    <skos:related rdf:resource="descriptor/18880-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10032-2">
    <skos:related rdf:resource="descriptor/18780-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10032-2">
    <skos:related rdf:resource="descriptor/16566-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10032-2">
    <skos:related rdf:resource="descriptor/18881-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10032-2">
    <skos:related rdf:resource="descriptor/10025-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10032-2">
    <skos:related rdf:resource="descriptor/16745-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70934">
    <rdfs:label xml:lang="en">Cargo shipping</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70934">
    <rdfs:label xml:lang="de">Frachtschifffahrt</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12825-6">
    <skos:narrower rdf:resource="descriptor/18936-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20888-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18394-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18269-1">
    <skos:altLabel xml:lang="en">Personal digital assistant</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18269-1">
    <skos:altLabel xml:lang="en">PDA (Personal Digital Assistant)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18269-1">
    <skos:altLabel xml:lang="en">Notepad PC</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18269-1">
    <skos:altLabel xml:lang="en">Notebook-PC</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18269-1">
    <skos:altLabel xml:lang="en">Handheld PC</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18269-1">
    <skos:altLabel xml:lang="de">Palmtop</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15900-3">
    <skos:broader rdf:resource="thsys/73354"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15900-3">
    <skos:broader rdf:resource="thsys/73350"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15900-3">
    <skos:broader rdf:resource="thsys/70931"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15900-3">
    <skos:broader rdf:resource="thsys/70878"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15900-3">
    <skos:broader rdf:resource="descriptor/19782-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18390-6">
    <skos:broader rdf:resource="thsys/71009"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18390-6">
    <skos:broader rdf:resource="descriptor/10470-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13415-6">
    <skos:prefLabel xml:lang="en">Public health insurance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13415-6">
    <skos:prefLabel xml:lang="de">Gesetzliche Krankenversicherung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12640-4">
    <skos:broader rdf:resource="thsys/70368"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19394-0">
    <skos:broader rdf:resource="thsys/70986"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19394-0">
    <skos:broader rdf:resource="thsys/70960"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19394-0">
    <skos:broader rdf:resource="descriptor/10010-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19640-4">
    <skos:related rdf:resource="descriptor/11209-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19640-4">
    <skos:related rdf:resource="descriptor/19649-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19640-4">
    <skos:related rdf:resource="descriptor/15989-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19640-4">
    <skos:related rdf:resource="descriptor/15907-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19640-4">
    <skos:related rdf:resource="descriptor/19843-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/p">
    <skos:narrower rdf:resource="thsys/70201"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/p">
    <skos:narrower rdf:resource="thsys/70549"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/p">
    <skos:narrower rdf:resource="thsys/70348"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/p">
    <skos:narrower rdf:resource="thsys/70135"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/p">
    <skos:narrower rdf:resource="thsys/70071"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/p">
    <skos:narrower rdf:resource="thsys/70257"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/p">
    <skos:narrower rdf:resource="thsys/70313"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/p">
    <skos:narrower rdf:resource="thsys/70066"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/p">
    <skos:narrower rdf:resource="thsys/70333"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/p">
    <skos:narrower rdf:resource="thsys/70498"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/p">
    <skos:narrower rdf:resource="thsys/70014"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/p">
    <skos:narrower rdf:resource="thsys/70538"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/p">
    <skos:narrower rdf:resource="thsys/70423"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/p">
    <skos:narrower rdf:resource="thsys/70358"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/p">
    <skos:narrower rdf:resource="thsys/70416"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/p">
    <skos:narrower rdf:resource="thsys/70328"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/p">
    <skos:narrower rdf:resource="thsys/70418"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/p">
    <skos:narrower rdf:resource="thsys/70337"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/p">
    <skos:narrower rdf:resource="thsys/70465"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/p">
    <skos:narrower rdf:resource="thsys/70031"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/p">
    <skos:narrower rdf:resource="thsys/70372"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/p">
    <skos:narrower rdf:resource="thsys/70161"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/p">
    <skos:narrower rdf:resource="thsys/70506"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/p">
    <skos:narrower rdf:resource="thsys/70265"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18542-2">
    <skos:broader rdf:resource="thsys/70951"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18542-2">
    <skos:broader rdf:resource="descriptor/10202-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18542-2">
    <skos:broader rdf:resource="descriptor/10876-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15901-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16080-3">
    <skos:prefLabel xml:lang="en">Securities law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16080-3">
    <skos:prefLabel xml:lang="de">Wertpapierrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19512-6">
    <skos:altLabel xml:lang="de">Pensionszahlungen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19512-6">
    <skos:altLabel xml:lang="de">Beamtenpension</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10567-5">
    <skos:broader rdf:resource="thsys/70038"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11160-2">
    <skos:altLabel xml:lang="en">Organizational choice</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11160-2">
    <skos:altLabel xml:lang="en">Institutional choice</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11160-2">
    <skos:altLabel xml:lang="de">Institutionenwahl</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11160-2">
    <skos:altLabel xml:lang="de">Institutionenfindung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18084-6">
    <skos:prefLabel xml:lang="en">Cash management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18084-6">
    <skos:prefLabel xml:lang="de">Cash Management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19077-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71116">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id93">
    <rdfs:seeAlso rdf:resource="descriptor/12588-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10395-1">
    <skos:related rdf:resource="descriptor/10993-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10395-1">
    <skos:related rdf:resource="descriptor/10378-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19591-5">
    <skos:narrower rdf:resource="descriptor/15267-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19591-5">
    <skos:narrower rdf:resource="descriptor/15265-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19801-6">
    <skos:broader rdf:resource="thsys/73341"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71104">
    <skos:broader rdf:resource="thsys/70187"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19151-5">
    <skos:altLabel xml:lang="en">Workshop</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19151-5">
    <skos:altLabel xml:lang="en">Meeting</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19151-5">
    <skos:altLabel xml:lang="de">Tagung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19151-5">
    <skos:altLabel xml:lang="de">Sitzung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19151-5">
    <skos:altLabel xml:lang="de">Seminar</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19151-5">
    <skos:altLabel xml:lang="de">Kongress</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19151-5">
    <skos:altLabel xml:lang="de">Kongreß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19151-5">
    <skos:altLabel xml:lang="de">Konferenztechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19151-5">
    <skos:altLabel xml:lang="de">Konferenzleitung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19151-5">
    <skos:altLabel xml:lang="de">Konferenzführung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19151-5">
    <skos:altLabel xml:lang="de">Besprechung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19102-4">
    <skos:scopeNote xml:lang="de">Konzept zur Messung von Unternehmenserfolg und Unternehmensbewertung mit Hilfe von Marktwertmodellen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19154-6">
    <skos:prefLabel xml:lang="en">Corporate governance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19154-6">
    <skos:prefLabel xml:lang="de">Corporate Governance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19540-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12399-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14472-3">
    <skos:altLabel xml:lang="de">Komponente der Elektronik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14472-3">
    <skos:altLabel xml:lang="de">Elektronische Komponente</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14472-3">
    <skos:altLabel xml:lang="de">Elektronenröhre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14472-3">
    <skos:altLabel xml:lang="de">Bauelement der Elektronik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14472-3">
    <skos:altLabel xml:lang="de">Aktive Komponente</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18688-2">
    <skos:prefLabel xml:lang="en">Currency option</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18688-2">
    <skos:prefLabel xml:lang="de">Devisenoptionsgeschäft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18173-0">
    <skos:altLabel xml:lang="en">Shirking model (Efficiency wages)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18173-0">
    <skos:altLabel xml:lang="en">Shirking efficiency wage model</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18173-0">
    <skos:altLabel xml:lang="en">Efficiency wage theory</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18173-0">
    <skos:altLabel xml:lang="de">Effizienzlohntheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18173-0">
    <skos:altLabel xml:lang="de">Effizienzlohnhypothese</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14640-1">
    <skos:related rdf:resource="descriptor/19486-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14640-1">
    <skos:related rdf:resource="descriptor/18072-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14640-1">
    <skos:related rdf:resource="descriptor/15743-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14640-1">
    <skos:related rdf:resource="descriptor/14608-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14640-1">
    <skos:related rdf:resource="descriptor/18069-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14640-1">
    <skos:related rdf:resource="descriptor/20888-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14640-1">
    <skos:related rdf:resource="descriptor/15010-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18794-3">
    <skos:related rdf:resource="descriptor/10882-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18794-3">
    <skos:related rdf:resource="descriptor/10876-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18794-3">
    <skos:related rdf:resource="descriptor/10822-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18794-3">
    <skos:related rdf:resource="descriptor/10851-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18794-3">
    <skos:related rdf:resource="descriptor/10604-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18794-3">
    <skos:related rdf:resource="descriptor/18544-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18794-3">
    <skos:related rdf:resource="descriptor/10602-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18794-3">
    <skos:related rdf:resource="descriptor/10605-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19934-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19391-6">
    <skos:related rdf:resource="descriptor/13142-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19391-6">
    <skos:related rdf:resource="descriptor/13232-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19391-6">
    <skos:related rdf:resource="descriptor/14833-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10803-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15086-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18504-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13594-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19054-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19242-2">
    <skos:prefLabel xml:lang="en">Economic convergence</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19242-2">
    <skos:prefLabel xml:lang="de">Entwicklungskonvergenz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19857-0">
    <skos:altLabel xml:lang="de">Widerspruchsrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19857-0">
    <skos:altLabel xml:lang="de">Vetorecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17593-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10586-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14802-1">
    <skos:related rdf:resource="descriptor/14810-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14802-1">
    <skos:related rdf:resource="descriptor/14805-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14802-1">
    <skos:related rdf:resource="descriptor/13117-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14802-1">
    <skos:related rdf:resource="descriptor/14803-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14802-1">
    <skos:related rdf:resource="descriptor/14385-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14802-1">
    <skos:related rdf:resource="descriptor/19389-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14802-1">
    <skos:related rdf:resource="descriptor/14617-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14088-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11680-4">
    <skos:related rdf:resource="descriptor/10010-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12272-0">
    <skos:altLabel xml:lang="en">Yield to maturity</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10032-2">
    <skos:prefLabel xml:lang="en">Economics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10032-2">
    <skos:prefLabel xml:lang="de">Wirtschaftswissenschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19373-1">
    <skos:altLabel xml:lang="de">Parafiskalität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19373-1">
    <skos:altLabel xml:lang="de">Nebenfiskus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19373-1">
    <skos:altLabel xml:lang="de">Hilfsfiskus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12474-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16404-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19396-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13374-1">
    <skos:broader rdf:resource="thsys/70839"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13374-1">
    <skos:broader rdf:resource="thsys/70573"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18524-4">
    <skos:related rdf:resource="descriptor/16611-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12119-1">
    <skos:narrower rdf:resource="descriptor/19832-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15619-1">
    <skos:prefLabel xml:lang="en">Programming language</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15619-1">
    <skos:prefLabel xml:lang="de">Programmiersprache</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13018-2">
    <skos:broader rdf:resource="thsys/71009"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13018-2">
    <skos:broader rdf:resource="thsys/71008"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13018-2">
    <skos:broader rdf:resource="descriptor/13019-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13018-2">
    <skos:broader rdf:resource="descriptor/12980-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18927-3">
    <skos:broader rdf:resource="thsys/70964"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18927-3">
    <skos:broader rdf:resource="thsys/70828"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18927-3">
    <skos:broader rdf:resource="descriptor/18920-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18927-3">
    <skos:broader rdf:resource="descriptor/12910-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11787-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70989">
    <skos:broader rdf:resource="thsys/71004"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18583-2">
    <skos:altLabel xml:lang="en">Toll</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18583-2">
    <skos:altLabel xml:lang="de">Straßenzoll</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18583-2">
    <skos:altLabel xml:lang="de">Maut</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11048-3">
    <skos:altLabel xml:lang="en">Market socialism</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11048-3">
    <skos:altLabel xml:lang="de">Sozialistische Marktwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11048-3">
    <skos:altLabel xml:lang="de">Konkurrenzsozialismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70572">
    <skos:broader rdf:resource="thsys/70142"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71074">
    <rdfs:label xml:lang="de">Steuerarten</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14626-2">
    <skos:related rdf:resource="descriptor/15763-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14626-2">
    <skos:related rdf:resource="descriptor/19016-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14626-2">
    <skos:related rdf:resource="descriptor/13338-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14626-2">
    <skos:related rdf:resource="descriptor/18431-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14626-2">
    <skos:related rdf:resource="descriptor/14673-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17704-0">
    <skos:altLabel xml:lang="de">Republik Ghana</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19218-6">
    <skos:related rdf:resource="descriptor/11765-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19218-6">
    <skos:related rdf:resource="descriptor/16739-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19218-6">
    <skos:related rdf:resource="descriptor/10198-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19218-6">
    <skos:related rdf:resource="descriptor/20881-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13177-3">
    <skos:altLabel xml:lang="de">Tunnelbau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13177-3">
    <skos:altLabel xml:lang="de">Brunnenbau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19820-2">
    <skos:related rdf:resource="descriptor/19819-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15777-4">
    <skos:prefLabel xml:lang="en">Traffic noise</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15777-4">
    <skos:prefLabel xml:lang="de">Verkehrslärm</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19278-2">
    <skos:broader rdf:resource="thsys/71106"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19278-2">
    <skos:broader rdf:resource="descriptor/12232-5"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id389">
    <rdf:value xml:lang="de">Für Darstellungen mit Hilfe der Szenario-Methode</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70986">
    <skos:prefLabel xml:lang="de">W.06.01.12  Nahrungs- und Genussmittelgewerbe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#prefLabel">
    <rdf:type rdf:resource="http://www.w3.org/2002/07/owl#DatatypeProperty"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#prefLabel">
    <rdf:type rdf:resource="http://www.w3.org/1999/02/22-rdf-syntax-ns#Property"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#prefLabel">
    <rdf:type rdf:resource="http://www.w3.org/2002/07/owl#AnnotationProperty"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12798-4">
    <skos:altLabel xml:lang="de">Marke (Handelsmarke)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12798-4">
    <skos:altLabel xml:lang="de">Händlermarke</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12798-4">
    <skos:altLabel xml:lang="de">Eigenmarke des Handels</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14433-6">
    <skos:narrower rdf:resource="descriptor/14442-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14433-6">
    <skos:narrower rdf:resource="descriptor/14440-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14433-6">
    <skos:narrower rdf:resource="descriptor/14438-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14433-6">
    <skos:narrower rdf:resource="descriptor/14437-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14433-6">
    <skos:narrower rdf:resource="descriptor/14435-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14433-6">
    <skos:narrower rdf:resource="descriptor/14412-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12191-0">
    <skos:related rdf:resource="descriptor/13464-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12191-0">
    <skos:related rdf:resource="descriptor/18108-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17553-2">
    <skos:narrower rdf:resource="descriptor/17595-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17553-2">
    <skos:narrower rdf:resource="descriptor/17589-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17553-2">
    <skos:narrower rdf:resource="descriptor/19167-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17553-2">
    <skos:narrower rdf:resource="descriptor/17571-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17553-2">
    <skos:narrower rdf:resource="descriptor/17608-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19594-6">
    <skos:altLabel xml:lang="en">Tournament game</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19594-6">
    <skos:altLabel xml:lang="en">Extensive game</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19594-6">
    <skos:altLabel xml:lang="de">Spiel in extensiver Form</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19594-6">
    <skos:altLabel xml:lang="de">Sequentielle Spielform</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19288-6">
    <skos:broader rdf:resource="thsys/73387"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19288-6">
    <skos:broader rdf:resource="thsys/70171"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19288-6">
    <skos:broader rdf:resource="thsys/70109"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19288-6">
    <skos:broader rdf:resource="descriptor/11068-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19288-6">
    <skos:broader rdf:resource="descriptor/10574-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70549">
    <skos:broader rdf:resource="thsys/p"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20982-1">
    <skos:broader rdf:resource="thsys/70406"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20982-1">
    <skos:broader rdf:resource="descriptor/10434-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18230-0">
    <skos:prefLabel xml:lang="en">Plastic waste</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18230-0">
    <skos:prefLabel xml:lang="de">Kunststoffabfall</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10780-5">
    <skos:broader rdf:resource="thsys/73390"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10780-5">
    <skos:broader rdf:resource="thsys/71103"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10780-5">
    <skos:broader rdf:resource="thsys/71098"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10780-5">
    <skos:broader rdf:resource="thsys/71088"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10780-5">
    <skos:broader rdf:resource="descriptor/11417-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10780-5">
    <skos:broader rdf:resource="descriptor/10606-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19428-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12304-6">
    <skos:altLabel xml:lang="en">Unfriendly takeover</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12304-6">
    <skos:altLabel xml:lang="en">Take-over</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12304-6">
    <skos:altLabel xml:lang="en">Mergers and Acquisitions</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12304-6">
    <skos:altLabel xml:lang="en">MBO (Management Buyout)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12304-6">
    <skos:altLabel xml:lang="en">Management buyout</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12304-6">
    <skos:altLabel xml:lang="en">M&amp;amp;A (Mergers and Acquisitions)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12304-6">
    <skos:altLabel xml:lang="en">Leveraged buyout</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12304-6">
    <skos:altLabel xml:lang="en">LBO (Leveraged Buyout)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12304-6">
    <skos:altLabel xml:lang="en">Hostile takeover</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12304-6">
    <skos:altLabel xml:lang="de">Unternehmensübernahme</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12304-6">
    <skos:altLabel xml:lang="de">Unternehmenskauf</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12304-6">
    <skos:altLabel xml:lang="de">Unternehmenshandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12304-6">
    <skos:altLabel xml:lang="de">Unternehmensakquisition</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12304-6">
    <skos:altLabel xml:lang="de">Kauf des Betriebs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12304-6">
    <skos:altLabel xml:lang="de">Feindliche Übernahme</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12304-6">
    <skos:altLabel xml:lang="de">Erwerb des Unternehmens</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12304-6">
    <skos:altLabel xml:lang="de">Betriebsübernahme</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13252-1">
    <skos:prefLabel xml:lang="en">Shipping company</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13252-1">
    <skos:prefLabel xml:lang="de">Reederei</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12009-1">
    <skos:prefLabel xml:lang="en">Institutional infrastructure</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12009-1">
    <skos:prefLabel xml:lang="de">Institutionelle Infrastruktur</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11993-4">
    <skos:prefLabel xml:lang="en">Rural-urban linkages</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11993-4">
    <skos:prefLabel xml:lang="de">Stadt-Land-Beziehung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13316-1">
    <skos:broader rdf:resource="thsys/70964"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13316-1">
    <skos:broader rdf:resource="thsys/70958"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13316-1">
    <skos:broader rdf:resource="descriptor/12910-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19512-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14993-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15463-6">
    <skos:narrower rdf:resource="descriptor/11337-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16666-1">
    <skos:related rdf:resource="descriptor/16676-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16666-1">
    <skos:related rdf:resource="descriptor/19394-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16666-1">
    <skos:related rdf:resource="descriptor/18992-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16666-1">
    <skos:related rdf:resource="descriptor/14972-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70649">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13007-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13248-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15922-0">
    <skos:altLabel xml:lang="en">American indian</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15922-0">
    <skos:altLabel xml:lang="de">Indianerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19166-6">
    <skos:broader rdf:resource="thsys/70147"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19166-6">
    <skos:broader rdf:resource="descriptor/17571-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18061-4">
    <skos:scopeNote xml:lang="de">Nur für den Erlass von Auslandsschulden.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13347-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10160-0">
    <skos:narrower rdf:resource="descriptor/10095-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10160-0">
    <skos:narrower rdf:resource="descriptor/10151-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10550-1">
    <skos:altLabel xml:lang="de">Regionale Entwicklungsbank</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10550-1">
    <skos:altLabel xml:lang="de">Internationale Entwicklungsbank</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14188-0">
    <skos:altLabel xml:lang="de">Wasserressourcen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14188-0">
    <skos:altLabel xml:lang="de">Brauchwasser</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19088-0">
    <skos:prefLabel xml:lang="en">Creditors</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19088-0">
    <skos:prefLabel xml:lang="de">Gläubiger</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19582-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11840-1">
    <skos:prefLabel xml:lang="en">Tenancy law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11840-1">
    <skos:prefLabel xml:lang="de">Mietrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19090-6">
    <skos:altLabel xml:lang="de">Umweltbezogene Rechnungslegung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19090-6">
    <skos:altLabel xml:lang="de">Ökobilanz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12741-5">
    <skos:altLabel xml:lang="de">Verbrauchsfunktion der Produktionstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12741-5">
    <skos:altLabel xml:lang="de">Produktionsfunktion der Betriebswirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12741-5">
    <skos:altLabel xml:lang="de">Gutenberg-Produktionsfunktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12385-1">
    <skos:broader rdf:resource="thsys/71043"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12385-1">
    <skos:broader rdf:resource="descriptor/19142-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18766-1">
    <skos:narrower rdf:resource="descriptor/19803-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18766-1">
    <skos:narrower rdf:resource="descriptor/18765-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10297-1">
    <skos:related rdf:resource="descriptor/10349-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10297-1">
    <skos:related rdf:resource="descriptor/12675-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16354-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16002-2">
    <skos:altLabel xml:lang="de">Staatskirchenvertrag</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16002-2">
    <skos:altLabel xml:lang="de">Konkordat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13255-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16527-1">
    <skos:prefLabel xml:lang="en">Old-age security</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16527-1">
    <skos:prefLabel xml:lang="de">Alterssicherung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18120-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11270-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15994-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13039-1">
    <skos:altLabel xml:lang="de">Zieherei</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13039-1">
    <skos:altLabel xml:lang="de">Stahlumformung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13039-1">
    <skos:altLabel xml:lang="de">Schmiedeindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13039-1">
    <skos:altLabel xml:lang="de">Oberflächenveredlung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13039-1">
    <skos:altLabel xml:lang="de">Kaltwalzwerk</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13039-1">
    <skos:altLabel xml:lang="de">Gesenkschmiede</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13039-1">
    <skos:altLabel xml:lang="de">Galvanisierungsindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13039-1">
    <skos:altLabel xml:lang="de">Freiformschmiede</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70467">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">P.01.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13158-0">
    <skos:altLabel xml:lang="de">Molkereiwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13158-0">
    <skos:altLabel xml:lang="de">Milchwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13158-0">
    <skos:altLabel xml:lang="de">Milchindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13158-0">
    <skos:altLabel xml:lang="de">Meierei</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13158-0">
    <skos:altLabel xml:lang="de">Käserei</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19860-4">
    <skos:altLabel xml:lang="de">Völkermord</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19860-4">
    <skos:altLabel xml:lang="de">Menschenrechtsverbrechen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19860-4">
    <skos:altLabel xml:lang="de">Holocaust</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19860-4">
    <skos:altLabel xml:lang="de">Genozid</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13669-3">
    <skos:broader rdf:resource="thsys/70265"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13669-3">
    <skos:broader rdf:resource="descriptor/18211-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15860-3">
    <skos:broader rdf:resource="thsys/70180"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15860-3">
    <skos:broader rdf:resource="descriptor/18859-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15860-3">
    <skos:broader rdf:resource="descriptor/10144-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24858-3">
    <skos:altLabel xml:lang="en">Tax assessment notice</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24858-3">
    <skos:altLabel xml:lang="en">Notice of assessment</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73364">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.04.04.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10966-5">
    <skos:related rdf:resource="descriptor/19327-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10966-5">
    <skos:related rdf:resource="descriptor/10029-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10966-5">
    <skos:related rdf:resource="descriptor/16384-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10966-5">
    <skos:related rdf:resource="descriptor/10618-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10966-5">
    <skos:related rdf:resource="descriptor/11784-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10966-5">
    <skos:related rdf:resource="descriptor/18850-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17565-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18532-5">
    <skos:broader rdf:resource="thsys/70956"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18532-5">
    <skos:broader rdf:resource="descriptor/18101-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17586-1">
    <skos:altLabel xml:lang="de">Republik der Philippinen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70888">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18818-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10115-5">
    <skos:scopeNote xml:lang="de">Wert der dauerhaften, reproduzierbaren, sachlichen Produktionsmittel einer Volkswirtschaft.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12504-5">
    <skos:related rdf:resource="descriptor/12424-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70130">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">G.01.03.03</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16684-6">
    <skos:broader rdf:resource="thsys/73363"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16684-6">
    <skos:broader rdf:resource="thsys/70865"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16684-6">
    <skos:broader rdf:resource="thsys/70013"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16684-6">
    <skos:broader rdf:resource="descriptor/16564-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16684-6">
    <skos:broader rdf:resource="descriptor/19725-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10150-3">
    <skos:altLabel xml:lang="de">Uneigennützigkeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10150-3">
    <skos:altLabel xml:lang="de">Selbstlosigkeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10150-3">
    <skos:altLabel xml:lang="de">Nächstenliebe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70300">
    <skos:prefLabel xml:lang="en">B.05.02.01  Inventory management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70300">
    <skos:prefLabel xml:lang="de">B.05.02.01  Betriebliche Lagerhaltung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19231-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11390-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16330-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14796-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14364-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10022-5">
    <skos:altLabel xml:lang="en">Debt conversion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12004-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19192-5">
    <skos:prefLabel xml:lang="en">Coal market</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19192-5">
    <skos:prefLabel xml:lang="de">Kohlenmarkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18970-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18631-3">
    <skos:prefLabel xml:lang="en">Rural transport</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18631-3">
    <skos:prefLabel xml:lang="de">Ländlicher Verkehr</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10979-3">
    <skos:altLabel xml:lang="de">Unternehmensprivatisierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10979-3">
    <skos:altLabel xml:lang="de">Reprivatisierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13152-5">
    <skos:related rdf:resource="descriptor/13234-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13152-5">
    <skos:related rdf:resource="descriptor/14983-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13152-5">
    <skos:related rdf:resource="descriptor/18992-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10214-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13814-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12846-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11649-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16532-1">
    <skos:altLabel xml:lang="en">Disability insurance</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16532-1">
    <skos:altLabel xml:lang="de">Invalidenrente</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16532-1">
    <skos:altLabel xml:lang="de">Erwerbsminderungsrente</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16532-1">
    <skos:altLabel xml:lang="de">Berufsunfähigkeitsrente</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16110-6">
    <skos:altLabel xml:lang="de">Zollfahndung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19695-0">
    <skos:prefLabel xml:lang="en">Party organization</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19695-0">
    <skos:prefLabel xml:lang="de">Parteiorganisation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12175-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11598-3">
    <skos:broader rdf:resource="thsys/71081"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11598-3">
    <skos:broader rdf:resource="thsys/70578"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11598-3">
    <skos:broader rdf:resource="descriptor/11595-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11598-3">
    <skos:broader rdf:resource="descriptor/19096-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15839-1">
    <skos:altLabel xml:lang="en">Work engagement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15839-1">
    <skos:altLabel xml:lang="de">Mitarbeitermotivation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15839-1">
    <skos:altLabel xml:lang="de">Leistungsbereitschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15839-1">
    <skos:altLabel xml:lang="de">Innere Kündigung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15839-1">
    <skos:altLabel xml:lang="de">Arbeitsmotivation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18562-3">
    <skos:scopeNote xml:lang="de">Alle technischen Geräte und Einrichtungsgegenstände, einschließlich Tischausstattung und Tischwäsche, für das Gastgewerbe.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14548-3">
    <skos:related rdf:resource="descriptor/14567-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14548-3">
    <skos:related rdf:resource="descriptor/19858-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14548-3">
    <skos:related rdf:resource="descriptor/21110-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14548-3">
    <skos:related rdf:resource="descriptor/14840-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14548-3">
    <skos:related rdf:resource="descriptor/15005-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14548-3">
    <skos:related rdf:resource="descriptor/13213-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14548-3">
    <skos:related rdf:resource="descriptor/14032-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14548-3">
    <skos:related rdf:resource="descriptor/13085-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73336">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10454-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10219-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12094-5">
    <skos:related rdf:resource="descriptor/12697-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12094-5">
    <skos:related rdf:resource="descriptor/12196-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12094-5">
    <skos:related rdf:resource="descriptor/19254-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70530">
    <rdfs:label xml:lang="de">Gesamtwirtschaftliche Investition</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13322-6">
    <skos:broader rdf:resource="thsys/70963"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13322-6">
    <skos:broader rdf:resource="thsys/70876"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13322-6">
    <skos:broader rdf:resource="descriptor/13470-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14354-2">
    <skos:broader rdf:resource="thsys/70424"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14354-2">
    <skos:broader rdf:resource="descriptor/14357-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14354-2">
    <skos:broader rdf:resource="descriptor/14435-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11801-4">
    <skos:related rdf:resource="descriptor/15752-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11801-4">
    <skos:related rdf:resource="descriptor/13428-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11801-4">
    <skos:related rdf:resource="descriptor/18979-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11801-4">
    <skos:related rdf:resource="descriptor/18851-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11801-4">
    <skos:related rdf:resource="descriptor/10505-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11801-4">
    <skos:related rdf:resource="descriptor/12945-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11801-4">
    <skos:related rdf:resource="descriptor/18338-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11801-4">
    <skos:related rdf:resource="descriptor/11807-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11801-4">
    <skos:related rdf:resource="descriptor/18334-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11801-4">
    <skos:related rdf:resource="descriptor/12018-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11801-4">
    <skos:related rdf:resource="descriptor/13426-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14129-2">
    <skos:broader rdf:resource="thsys/70467"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18442-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11706-5">
    <skos:prefLabel xml:lang="en">Tax reform</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11706-5">
    <skos:prefLabel xml:lang="de">Steuerreform</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70278">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18316-4">
    <skos:altLabel xml:lang="de">Zuckerrübenanbau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18316-4">
    <skos:altLabel xml:lang="de">Zuckerrohranbau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18316-4">
    <skos:altLabel xml:lang="de">Zuckeranbau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13821-2">
    <skos:broader rdf:resource="thsys/70892"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13821-2">
    <skos:broader rdf:resource="descriptor/15685-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13308-0">
    <skos:related rdf:resource="descriptor/13832-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13308-0">
    <skos:related rdf:resource="descriptor/18564-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13308-0">
    <skos:related rdf:resource="descriptor/18561-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13308-0">
    <skos:related rdf:resource="descriptor/13636-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14054-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18682-0">
    <skos:altLabel xml:lang="de">Effektenbörse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18682-0">
    <skos:altLabel xml:lang="de">Aktienbörse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11260-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12280-1">
    <skos:prefLabel xml:lang="en">Financial investment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12280-1">
    <skos:prefLabel xml:lang="de">Kapitalanlage</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15713-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18022-0">
    <skos:prefLabel xml:lang="en">Electronic money</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18022-0">
    <skos:prefLabel xml:lang="de">Electronic Cash</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19214-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10376-5">
    <skos:prefLabel xml:lang="en">Consumer confidence</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10376-5">
    <skos:prefLabel xml:lang="de">Konsumklima</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11827-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18650-6">
    <skos:altLabel xml:lang="en">Telephone company</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18650-6">
    <skos:altLabel xml:lang="en">Phone company</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18650-6">
    <skos:altLabel xml:lang="de">Telephongesellschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10377-3">
    <skos:prefLabel xml:lang="en">Supply-side economics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10377-3">
    <skos:prefLabel xml:lang="de">Angebotsorientierte Wirtschaftspolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19779-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13627-5">
    <skos:altLabel xml:lang="en">Bulk shipping</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13627-5">
    <skos:altLabel xml:lang="de">Massengutschiffahrt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13627-5">
    <skos:altLabel xml:lang="de">Bulk-Schifffahrt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13627-5">
    <skos:altLabel xml:lang="de">Bulk-Schiffahrt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10834-1">
    <skos:prefLabel xml:lang="en">Trade price</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10834-1">
    <skos:prefLabel xml:lang="de">Außenhandelspreis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12928-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15017-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19761-6">
    <skos:narrower rdf:resource="descriptor/11299-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15690-2">
    <skos:scopeNote xml:lang="de">Nur zur Indexierung von Literatur über zeitlich zurückliegende Ereignisse und für die Betrachtung historischer Entwicklungen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18662-6">
    <skos:related rdf:resource="descriptor/15599-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19714-1">
    <skos:prefLabel xml:lang="en">State succession</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19714-1">
    <skos:prefLabel xml:lang="de">Staatennachfolge</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="http://zbw.eu/namespaces/skos-extensions/Descriptor">
    <rdfs:isDefinedBy rdf:resource="../namespaces/skos-extensions/skos-extensions.rdf"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12956-5">
    <skos:broader rdf:resource="thsys/71025"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12956-5">
    <skos:broader rdf:resource="descriptor/12955-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19136-1">
    <skos:related rdf:resource="descriptor/18022-0"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id45">
    <rdfs:seeAlso rdf:resource="descriptor/11360-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15111-2">
    <skos:scopeNote xml:lang="de">Auswahl von Art und Gewinnung statistischer Einheiten.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10684-1">
    <skos:broader rdf:resource="thsys/71098"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10684-1">
    <skos:broader rdf:resource="descriptor/10686-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10038-4">
    <skos:related rdf:resource="descriptor/19202-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10038-4">
    <skos:related rdf:resource="descriptor/10344-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10038-4">
    <skos:related rdf:resource="descriptor/10045-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12425-1">
    <skos:altLabel xml:lang="en">Inflation-accounting</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12425-1">
    <skos:altLabel xml:lang="de">Rechnungslegung bei Inflation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12425-1">
    <skos:altLabel xml:lang="de">Inflationsbereinigte Rechnungslegung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15674-0">
    <skos:prefLabel xml:lang="en">Sociological pedagogy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15674-0">
    <skos:prefLabel xml:lang="de">Sozialpädagogik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13010-4">
    <skos:altLabel xml:lang="de">Porzellanindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13427-6">
    <skos:narrower rdf:resource="descriptor/14049-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13427-6">
    <skos:narrower rdf:resource="descriptor/14045-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13427-6">
    <skos:narrower rdf:resource="descriptor/19020-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13427-6">
    <skos:narrower rdf:resource="descriptor/14034-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18390-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17602-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13074-6">
    <skos:related rdf:resource="descriptor/10455-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13074-6">
    <skos:related rdf:resource="descriptor/14480-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13074-6">
    <skos:related rdf:resource="descriptor/13072-3"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id391">
    <rdfs:seeAlso rdf:resource="descriptor/19836-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16102-5">
    <skos:altLabel xml:lang="de">Investitionszulagengesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16102-5">
    <skos:altLabel xml:lang="de">Investitionshilfegesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="http://zbw.eu/namespaces/skos-extensions/">
    <ns0:publisher xmlns:ns0="http://purl.org/dc/elements/1.1/" xml:lang="en">German National Library of Economics (ZBW)</ns0:publisher>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16454-2">
    <skos:prefLabel xml:lang="en">International security</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16454-2">
    <skos:prefLabel xml:lang="de">Internationale Sicherheit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12049-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11180-3">
    <skos:altLabel xml:lang="en">Labor</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18242-0">
    <skos:related rdf:resource="descriptor/18595-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18242-0">
    <skos:related rdf:resource="descriptor/18586-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18242-0">
    <skos:related rdf:resource="descriptor/18241-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18242-0">
    <skos:related rdf:resource="descriptor/14318-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15635-3">
    <skos:scopeNote xml:lang="de">Für die Forschung, die Branche und die Produkte BENUTZE die entsprechenden Deskriptoren.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15635-3">
    <skos:scopeNote xml:lang="de">Anwendung von Biochemie, Mikrobiologie sowie chemischen Techniken für die Verarbeitung von Substanzen durch Mikroorganismen, Enzyme und Zellen. Da es sich bei der Biotechnologie um ein junges Forschungsgebiet handelt, ist die Abgrenzung nicht eindeutig, vor allem nicht zum Bereich der Gentechnik.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18288-4">
    <skos:broader rdf:resource="thsys/73360"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18288-4">
    <skos:broader rdf:resource="thsys/70993"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18288-4">
    <skos:broader rdf:resource="descriptor/16019-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70171">
    <rdfs:label xml:lang="de">Wirtschaftssysteme</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18725-1">
    <skos:altLabel xml:lang="de">VerbrKrG</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18725-1">
    <skos:altLabel xml:lang="de">Verbraucherkreditgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id306">
    <rdf:value xml:lang="de">für Stadtentwicklungsplanung</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13420-6">
    <skos:broader rdf:resource="thsys/73381"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13420-6">
    <skos:broader rdf:resource="thsys/70870"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13420-6">
    <skos:broader rdf:resource="descriptor/13409-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18286-1">
    <skos:scopeNote xml:lang="de">Auch für die Ökonomische Theorie erneuerbarer Ressourcen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12738-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19177-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10110-1">
    <skos:prefLabel xml:lang="en">Distributive justice</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10110-1">
    <skos:prefLabel xml:lang="de">Verteilungsgerechtigkeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12820-2">
    <skos:altLabel xml:lang="de">Courtage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13170-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10823-6">
    <zbwext:useInsteadNote rdf:nodeID="id72"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14462-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17660-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19329-4">
    <skos:related rdf:resource="descriptor/16070-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19329-4">
    <skos:related rdf:resource="descriptor/10621-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19329-4">
    <skos:related rdf:resource="descriptor/10606-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11030-1">
    <skos:narrower rdf:resource="descriptor/11001-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15977-3">
    <skos:altLabel xml:lang="de">Schiedsverfahren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15977-3">
    <skos:altLabel xml:lang="de">Schiedsspruch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15977-3">
    <skos:altLabel xml:lang="de">Schiedsrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15977-3">
    <skos:altLabel xml:lang="de">Schiedsgericht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15977-3">
    <skos:altLabel xml:lang="de">Handelsarbitrage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15977-3">
    <skos:altLabel xml:lang="de">Arbitrageklausel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15977-3">
    <skos:altLabel xml:lang="de">Arbitrage im Handelsrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11684-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15752-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18022-0">
    <skos:altLabel xml:lang="en">Point-of-sale system</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18022-0">
    <skos:altLabel xml:lang="en">EPOS (Electronic Point-of-Sale)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18022-0">
    <skos:altLabel xml:lang="en">Electronic point-of-sale</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18022-0">
    <skos:altLabel xml:lang="en">Electronic payment</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18022-0">
    <skos:altLabel xml:lang="de">POS-System</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18022-0">
    <skos:altLabel xml:lang="de">Elektronischer Zahlungsverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16982-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18890-0">
    <skos:related rdf:resource="descriptor/13374-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18890-0">
    <skos:related rdf:resource="descriptor/13348-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18890-0">
    <skos:related rdf:resource="descriptor/14007-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18890-0">
    <skos:related rdf:resource="descriptor/13371-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12714-1">
    <skos:broader rdf:resource="thsys/70029"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12714-1">
    <skos:broader rdf:resource="descriptor/19074-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18748-3">
    <skos:broader rdf:resource="thsys/71066"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18748-3">
    <skos:broader rdf:resource="descriptor/11859-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13713-5">
    <skos:broader rdf:resource="thsys/71106"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13713-5">
    <skos:broader rdf:resource="descriptor/13723-2"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id23">
    <rdf:value xml:lang="de">Für die internationale Wanderung von Arbeitskräften</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18614-3">
    <skos:broader rdf:resource="thsys/70940"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18614-3">
    <skos:broader rdf:resource="descriptor/13621-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18614-3">
    <skos:broader rdf:resource="descriptor/13501-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16830-0">
    <skos:broader rdf:resource="thsys/70361"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16830-0">
    <skos:broader rdf:resource="descriptor/18012-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19406-5">
    <skos:prefLabel xml:lang="en">Benefit equity principle</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19406-5">
    <skos:prefLabel xml:lang="de">Äquivalenzprinzip</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14381-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13025-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id290">
    <rdf:value xml:lang="de">Für den allgemeinen Büro- und Schulbedarf</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16445-3">
    <skos:narrower rdf:resource="descriptor/18182-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16445-3">
    <skos:narrower rdf:resource="descriptor/18818-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16445-3">
    <skos:narrower rdf:resource="descriptor/16449-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19610-6">
    <skos:prefLabel xml:lang="en">Bridge</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19610-6">
    <skos:prefLabel xml:lang="de">Brücke</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11966-0">
    <skos:prefLabel xml:lang="en">City</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11966-0">
    <skos:prefLabel xml:lang="de">Großstadt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13666-2">
    <skos:related rdf:resource="descriptor/13667-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13666-2">
    <skos:related rdf:resource="descriptor/19653-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13231-2">
    <skos:prefLabel xml:lang="en">Jewelry trade</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13231-2">
    <skos:prefLabel xml:lang="de">Schmuckhandel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16203-6">
    <skos:scopeNote xml:lang="de">Wuchergeschäfte sind nach bürgerlichem Recht sittenwidrig. Im Strafrecht werden Vermittlungs- und Leistungswucher unterschieden.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10024-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18560-0">
    <skos:prefLabel xml:lang="en">Food delivery service</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18560-0">
    <skos:prefLabel xml:lang="de">Catering</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18194-6">
    <skos:altLabel xml:lang="de">Internationales Kakaoabkommen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18721-2">
    <skos:scopeNote xml:lang="de">Steuerung der Risikoposition einer Bank durch Optimierung der Bilanzstruktur.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70052">
    <skos:broader rdf:resource="thsys/v"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71044">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19329-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11972-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18062-2">
    <skos:scopeNote xml:lang="de">Nationale Einheits- und Einigungsbestrebungen sowie Streben nach Wiedervereinigung getrennter Nationen</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19438-6">
    <skos:related rdf:resource="descriptor/15902-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19438-6">
    <skos:related rdf:resource="descriptor/11875-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19438-6">
    <skos:related rdf:resource="descriptor/11962-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19438-6">
    <skos:related rdf:resource="descriptor/11910-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70856">
    <skos:broader rdf:resource="thsys/w"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19264-6">
    <skos:related rdf:resource="descriptor/13721-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13080-4">
    <skos:altLabel xml:lang="de">Augenoptikerhandwerk</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16529-4">
    <skos:related rdf:resource="descriptor/16155-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16529-4">
    <skos:related rdf:resource="descriptor/15908-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14054-0">
    <skos:prefLabel xml:lang="en">Agricultural product</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14054-0">
    <skos:prefLabel xml:lang="de">Agrarprodukt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10686-4">
    <skos:prefLabel xml:lang="en">Foreign trade policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10686-4">
    <skos:prefLabel xml:lang="de">Außenhandelspolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12232-5">
    <skos:altLabel xml:lang="en">Shares</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12232-5">
    <skos:altLabel xml:lang="en">Equities</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12232-5">
    <skos:altLabel xml:lang="de">Stammaktie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12232-5">
    <skos:altLabel xml:lang="de">Inhaberaktie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14323-6">
    <skos:prefLabel xml:lang="en">Pipeline</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14323-6">
    <skos:prefLabel xml:lang="de">Rohrleitung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12665-2">
    <skos:altLabel xml:lang="en">Customer-supplier-relationship</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12665-2">
    <skos:altLabel xml:lang="en">B2B</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12665-2">
    <skos:altLabel xml:lang="en">Buyer-supplier-relationship</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12665-2">
    <skos:altLabel xml:lang="en">Business-to-business selling</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12665-2">
    <skos:altLabel xml:lang="en">Business-to-business relationships</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12665-2">
    <skos:altLabel xml:lang="de">Zulieferer-Hersteller-Kooperation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12665-2">
    <skos:altLabel xml:lang="de">Zuliefererbeziehungen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12665-2">
    <skos:altLabel xml:lang="de">Lieferanten-Kundenbeziehung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12665-2">
    <skos:altLabel xml:lang="de">Kunden-Lieferanten-Beziehung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12665-2">
    <skos:altLabel xml:lang="de">Hersteller-Handel-Kooperation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12665-2">
    <skos:altLabel xml:lang="de">Beschaffungsmarketing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12880-5">
    <skos:altLabel xml:lang="de">Steuerprüfung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12880-5">
    <skos:altLabel xml:lang="de">Steuerliche Buchführungsprüfung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12880-5">
    <skos:altLabel xml:lang="de">Außenprüfung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70040">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14309-0">
    <skos:prefLabel xml:lang="en">Zinc</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14309-0">
    <skos:prefLabel xml:lang="de">Zink</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18767-6">
    <skos:broader rdf:resource="thsys/73338"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18767-6">
    <skos:broader rdf:resource="thsys/71037"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18767-6">
    <skos:broader rdf:resource="thsys/71036"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18767-6">
    <skos:broader rdf:resource="descriptor/12079-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18767-6">
    <skos:broader rdf:resource="descriptor/18017-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11981-4">
    <skos:prefLabel xml:lang="en">Enterprise zone</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11981-4">
    <skos:prefLabel xml:lang="de">Gewerbegebiet</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14267-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13683-2">
    <skos:broader rdf:resource="thsys/73341"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13683-2">
    <skos:broader rdf:resource="descriptor/18000-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11457-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12880-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19292-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15659-3">
    <skos:broader rdf:resource="thsys/73348"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15659-3">
    <skos:broader rdf:resource="thsys/73347"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70964">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.08.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14431-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13622-1">
    <skos:prefLabel xml:lang="en">Combined transport</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13622-1">
    <skos:prefLabel xml:lang="de">Kombinierter Verkehr</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14897-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15722-1">
    <skos:related rdf:resource="descriptor/15811-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15722-1">
    <skos:related rdf:resource="descriptor/15810-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15722-1">
    <skos:related rdf:resource="descriptor/15807-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15722-1">
    <skos:related rdf:resource="descriptor/16654-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15722-1">
    <skos:related rdf:resource="descriptor/15712-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15722-1">
    <skos:related rdf:resource="descriptor/15798-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12012-5">
    <skos:altLabel xml:lang="en">Public intermediate good</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12012-5">
    <skos:altLabel xml:lang="en">Public input</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12012-5">
    <skos:altLabel xml:lang="en">Collective factor of production</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12012-5">
    <skos:altLabel xml:lang="de">Unternehmensorientierte Infrastruktur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12012-5">
    <skos:altLabel xml:lang="de">Produktive Infrastruktur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12012-5">
    <skos:altLabel xml:lang="de">Produktionsorientierte Infrastruktur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12012-5">
    <skos:altLabel xml:lang="de">Öffentliches Zwischenprodukt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12012-5">
    <skos:altLabel xml:lang="de">Öffentliche Vorleistung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11187-3">
    <skos:narrower rdf:resource="descriptor/18166-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11187-3">
    <skos:narrower rdf:resource="descriptor/11215-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11187-3">
    <skos:narrower rdf:resource="descriptor/11281-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11187-3">
    <skos:narrower rdf:resource="descriptor/11212-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11187-3">
    <skos:narrower rdf:resource="descriptor/12628-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11187-3">
    <skos:narrower rdf:resource="descriptor/13488-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11187-3">
    <skos:narrower rdf:resource="descriptor/19504-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11187-3">
    <skos:narrower rdf:resource="descriptor/11192-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13578-6">
    <skos:related rdf:resource="descriptor/15900-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13578-6">
    <skos:related rdf:resource="descriptor/15899-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13578-6">
    <skos:related rdf:resource="descriptor/13684-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13578-6">
    <skos:related rdf:resource="descriptor/14423-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13578-6">
    <skos:related rdf:resource="descriptor/13507-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13578-6">
    <skos:related rdf:resource="descriptor/13575-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13578-6">
    <skos:related rdf:resource="descriptor/18418-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13578-6">
    <skos:related rdf:resource="descriptor/13056-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18394-5">
    <skos:scopeNote xml:lang="de">Förderung von Bodenschätzen im Festlandsockel und in Küstengewässern; bisher vorwiegend Öl- und Gasförderung</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18352-0">
    <skos:broader rdf:resource="thsys/73390"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18352-0">
    <skos:broader rdf:resource="thsys/71009"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18352-0">
    <skos:broader rdf:resource="thsys/70811"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18352-0">
    <skos:broader rdf:resource="descriptor/16081-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18352-0">
    <skos:broader rdf:resource="descriptor/16007-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12977-4">
    <skos:prefLabel xml:lang="en">Heating plant</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12977-4">
    <skos:prefLabel xml:lang="de">Heizkraftwerk</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10911-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15018-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id331">
    <rdf:value xml:lang="de">Für Maßnahmen zur Erhaltung wildlebender Tierarten</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18884-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11802-2">
    <skos:broader rdf:resource="thsys/71020"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11802-2">
    <skos:broader rdf:resource="descriptor/10978-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11802-2">
    <skos:broader rdf:resource="descriptor/11800-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19766-3">
    <skos:prefLabel xml:lang="en">Sociology of knowledge</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19766-3">
    <skos:prefLabel xml:lang="de">Wissenssoziologie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19202-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13496-1">
    <skos:altLabel xml:lang="en">Cash &amp;amp; Carry</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13496-1">
    <skos:altLabel xml:lang="de">Selbstbedienungsgroßhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13496-1">
    <skos:altLabel xml:lang="de">SB-Großhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13496-1">
    <skos:altLabel xml:lang="de">C&amp;amp;C-Handel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13496-1">
    <skos:altLabel xml:lang="de">Cash-and-Carry-Markt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13496-1">
    <skos:altLabel xml:lang="de">Cash-and-Carry-Großhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13496-1">
    <skos:altLabel xml:lang="de">Cash und Carry Großhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13496-1">
    <skos:altLabel xml:lang="de">Abholgroßmarkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18506-6">
    <skos:broader rdf:resource="thsys/70970"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18506-6">
    <skos:broader rdf:resource="descriptor/10337-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12810-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70580">
    <skos:prefLabel xml:lang="de">N.08.02  Sprache und Literatur</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14849-3">
    <skos:related rdf:resource="descriptor/14846-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14849-3">
    <skos:related rdf:resource="descriptor/14700-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10471-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12069-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70423">
    <skos:prefLabel xml:lang="de">P.11  Feinmechanische und optische Produkte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19706-0">
    <skos:related rdf:resource="descriptor/13399-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19706-0">
    <skos:related rdf:resource="descriptor/15671-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19706-0">
    <skos:related rdf:resource="descriptor/16002-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19706-0">
    <skos:related rdf:resource="descriptor/13393-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10139-5">
    <skos:broader rdf:resource="thsys/72393"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10139-5">
    <skos:broader rdf:resource="thsys/70180"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10139-5">
    <skos:broader rdf:resource="descriptor/15373-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70921">
    <skos:broader rdf:resource="thsys/70923"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13282-6">
    <skos:altLabel xml:lang="de">Versicherungsvertreterinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13282-6">
    <skos:altLabel xml:lang="de">Versicherungsvertreter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13282-6">
    <skos:altLabel xml:lang="de">Versicherungsvermittlerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13282-6">
    <skos:altLabel xml:lang="de">Versicherungsvermittler</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13282-6">
    <skos:altLabel xml:lang="de">Versicherungsmaklerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13282-6">
    <skos:altLabel xml:lang="de">Versicherungsmakler</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13282-6">
    <skos:altLabel xml:lang="de">Versicherungsagentur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13282-6">
    <skos:altLabel xml:lang="de">Versicherungsagentinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13282-6">
    <skos:altLabel xml:lang="de">Versicherungsagenten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18296-5">
    <skos:altLabel xml:lang="de">Navigationsgerät</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19304-6">
    <skos:prefLabel xml:lang="en">Wheat policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19304-6">
    <skos:prefLabel xml:lang="de">Weizenpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10438-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18066-1">
    <skos:related rdf:resource="descriptor/20002-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18066-1">
    <skos:related rdf:resource="descriptor/16078-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18066-1">
    <skos:related rdf:resource="descriptor/19129-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18066-1">
    <skos:related rdf:resource="descriptor/12800-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18066-1">
    <skos:related rdf:resource="descriptor/18065-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18066-1">
    <skos:related rdf:resource="descriptor/18176-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18066-1">
    <skos:related rdf:resource="descriptor/10947-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18066-1">
    <skos:related rdf:resource="descriptor/11419-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10276-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15888-2">
    <skos:broader rdf:resource="thsys/73352"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15888-2">
    <skos:broader rdf:resource="descriptor/15876-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17707-1">
    <skos:broader rdf:resource="thsys/70560"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17707-1">
    <skos:broader rdf:resource="descriptor/17724-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14154-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19698-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10397-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16514-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10945-6">
    <skos:altLabel xml:lang="en">R&amp;amp;D competition</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10945-6">
    <skos:altLabel xml:lang="de">Technologiewettbewerb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10945-6">
    <skos:altLabel xml:lang="de">Schöpferischer Wettbewerb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10945-6">
    <skos:altLabel xml:lang="de">FuE-Wettbewerb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10945-6">
    <skos:altLabel xml:lang="de">Forschungswettbewerb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10945-6">
    <skos:altLabel xml:lang="de">Entwicklungswettbewerb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13983-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18920-3">
    <skos:prefLabel xml:lang="en">Cultural industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18920-3">
    <skos:prefLabel xml:lang="de">Kultursektor</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18682-0">
    <skos:broader rdf:resource="thsys/71107"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18682-0">
    <skos:broader rdf:resource="thsys/70893"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18682-0">
    <skos:broader rdf:resource="descriptor/13279-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14335-6">
    <skos:related rdf:resource="descriptor/13479-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14335-6">
    <skos:related rdf:resource="descriptor/13354-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14335-6">
    <skos:related rdf:resource="descriptor/14346-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19849-6">
    <skos:prefLabel xml:lang="en">Litigation costs</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19849-6">
    <skos:prefLabel xml:lang="de">Gerichtliche Prozesskosten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10462-5">
    <skos:broader rdf:resource="thsys/73345"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10462-5">
    <skos:broader rdf:resource="thsys/70996"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10462-5">
    <skos:broader rdf:resource="descriptor/18426-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14583-1">
    <skos:prefLabel xml:lang="en">Video game</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14583-1">
    <skos:prefLabel xml:lang="de">Videospiel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11210-6">
    <skos:altLabel xml:lang="en">NAWRU</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11210-6">
    <skos:altLabel xml:lang="de">Technologische Arbeitslosigkeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11210-6">
    <skos:altLabel xml:lang="de">Mismatch-Arbeitslosigkeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13341-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18727-4">
    <skos:scopeNote xml:lang="de">Übereignung von Wertpapieren auf Zeit mit der Verpflichtung, dass der Entleiher Papiere gleicher Art und Güte nach Ablauf der vereinbarten Frist zurückzuübereignen und für die Dauer der Überlassung ein Entgelt zu entrichten hat.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13612-4">
    <skos:related rdf:resource="descriptor/13584-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13612-4">
    <skos:related rdf:resource="descriptor/13583-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13612-4">
    <skos:related rdf:resource="descriptor/13591-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13612-4">
    <skos:related rdf:resource="descriptor/13610-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13612-4">
    <skos:related rdf:resource="descriptor/13301-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18373-6">
    <skos:broader rdf:resource="thsys/71012"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18373-6">
    <skos:broader rdf:resource="thsys/70992"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18373-6">
    <skos:broader rdf:resource="thsys/70539"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18373-6">
    <skos:broader rdf:resource="descriptor/11810-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18373-6">
    <skos:broader rdf:resource="descriptor/18375-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18810-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70860">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13400-5">
    <skos:broader rdf:resource="thsys/73378"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13400-5">
    <skos:broader rdf:resource="thsys/70853"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13400-5">
    <skos:broader rdf:resource="thsys/70239"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13400-5">
    <skos:broader rdf:resource="descriptor/10984-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19535-1">
    <skos:altLabel xml:lang="en">Stock repurchase</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19535-1">
    <skos:altLabel xml:lang="en">Stock buyback</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19535-1">
    <skos:altLabel xml:lang="de">Buyback</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19689-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13464-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18380-2">
    <skos:related rdf:resource="descriptor/14053-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18380-2">
    <skos:related rdf:resource="descriptor/13179-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13704-6">
    <skos:altLabel xml:lang="en">Real estate fund</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13704-6">
    <skos:altLabel xml:lang="de">Offener Immobilienfonds</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13704-6">
    <skos:altLabel xml:lang="de">Immobilienzertifikat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13704-6">
    <skos:altLabel xml:lang="de">Grundstücksfonds</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13704-6">
    <skos:altLabel xml:lang="de">Geschlossener Immobilienfonds</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12226-0">
    <skos:related rdf:resource="descriptor/12312-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12226-0">
    <skos:related rdf:resource="descriptor/12218-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12226-0">
    <skos:related rdf:resource="descriptor/12216-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12226-0">
    <skos:related rdf:resource="descriptor/12228-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16008-4">
    <skos:related rdf:resource="descriptor/19699-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16008-4">
    <skos:related rdf:resource="descriptor/13397-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16008-4">
    <skos:related rdf:resource="descriptor/16366-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16008-4">
    <skos:related rdf:resource="descriptor/16365-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16365-1">
    <skos:prefLabel xml:lang="en">Party finance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16365-1">
    <skos:prefLabel xml:lang="de">Parteienfinanzierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10374-2">
    <skos:prefLabel xml:lang="en">Short-term forecast</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10374-2">
    <skos:prefLabel xml:lang="de">Konjunkturprognose</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70021">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16249-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10560-5">
    <skos:scopeNote xml:lang="de">Nur für die Entwicklungspolitik der Entwicklungsländer selbst.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13248-6">
    <skos:related rdf:resource="descriptor/13558-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13248-6">
    <skos:related rdf:resource="descriptor/13217-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13248-6">
    <skos:related rdf:resource="descriptor/14433-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13776-2">
    <skos:broader rdf:resource="thsys/70899"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13776-2">
    <skos:broader rdf:resource="descriptor/13756-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11670-0">
    <skos:prefLabel xml:lang="en">Payroll tax</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11670-0">
    <skos:prefLabel xml:lang="de">Lohnsummensteuer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11873-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16246-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18619-0">
    <skos:broader rdf:resource="thsys/70986"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18619-0">
    <skos:broader rdf:resource="thsys/70958"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11418-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16409-0">
    <skos:related rdf:resource="descriptor/10985-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16409-0">
    <skos:related rdf:resource="descriptor/11142-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16409-0">
    <skos:related rdf:resource="descriptor/11060-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16409-0">
    <skos:related rdf:resource="descriptor/11061-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16409-0">
    <skos:related rdf:resource="descriptor/11047-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16409-0">
    <skos:related rdf:resource="descriptor/11044-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16409-0">
    <skos:related rdf:resource="descriptor/18830-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19202-0">
    <skos:related rdf:resource="descriptor/10148-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19202-0">
    <skos:related rdf:resource="descriptor/19784-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19202-0">
    <skos:related rdf:resource="descriptor/10056-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19202-0">
    <skos:related rdf:resource="descriptor/19198-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19202-0">
    <skos:related rdf:resource="descriptor/10038-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19202-0">
    <skos:related rdf:resource="descriptor/10378-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19202-0">
    <skos:related rdf:resource="descriptor/18138-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19283-2">
    <skos:broader rdf:resource="thsys/70039"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19283-2">
    <skos:broader rdf:resource="descriptor/18160-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19283-2">
    <skos:broader rdf:resource="descriptor/11161-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13715-1">
    <skos:altLabel xml:lang="de">Aktienpreis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13715-1">
    <skos:altLabel xml:lang="de">Aktienkurs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17986-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18718-5">
    <skos:narrower rdf:resource="descriptor/18648-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18575-1">
    <skos:broader rdf:resource="thsys/70941"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18575-1">
    <skos:broader rdf:resource="thsys/70926"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18575-1">
    <skos:broader rdf:resource="descriptor/11833-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14766-0">
    <skos:narrower rdf:resource="descriptor/14772-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14766-0">
    <skos:narrower rdf:resource="descriptor/14771-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14766-0">
    <skos:narrower rdf:resource="descriptor/18293-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10465-6">
    <skos:prefLabel xml:lang="en">History of technology</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10465-6">
    <skos:prefLabel xml:lang="de">Technikgeschichte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16032-0">
    <skos:broader rdf:resource="thsys/73333"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16032-0">
    <skos:broader rdf:resource="descriptor/16042-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16032-0">
    <skos:broader rdf:resource="descriptor/16186-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14926-4">
    <skos:prefLabel xml:lang="en">Health food</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14926-4">
    <skos:prefLabel xml:lang="de">Gesundheitskost</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18065-3">
    <skos:altLabel xml:lang="de">Lizenzhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18065-3">
    <skos:altLabel xml:lang="de">Lizenzgeschäft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14646-3">
    <skos:broader rdf:resource="thsys/70463"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14646-3">
    <skos:broader rdf:resource="descriptor/14205-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14646-3">
    <skos:broader rdf:resource="descriptor/14642-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18812-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13310-6">
    <skos:broader rdf:resource="thsys/70834"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13310-6">
    <skos:broader rdf:resource="descriptor/13312-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13310-6">
    <skos:broader rdf:resource="descriptor/16525-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70076">
    <skos:broader rdf:resource="thsys/g"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18838-2">
    <skos:prefLabel xml:lang="en">History of the cooperative movement</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18838-2">
    <skos:prefLabel xml:lang="de">Genossenschaftsgeschichte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15929-0">
    <skos:prefLabel xml:lang="en">Black people</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15929-0">
    <skos:prefLabel xml:lang="de">Schwarze</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19628-1">
    <skos:broader rdf:resource="thsys/73351"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19628-1">
    <skos:broader rdf:resource="descriptor/15746-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14834-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14465-0">
    <skos:related rdf:resource="descriptor/14537-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14465-0">
    <skos:related rdf:resource="descriptor/13079-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14465-0">
    <skos:related rdf:resource="descriptor/10462-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14465-0">
    <skos:related rdf:resource="descriptor/19796-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14465-0">
    <skos:related rdf:resource="descriptor/14531-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10499-3">
    <skos:prefLabel xml:lang="en">Industrialized countries</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10499-3">
    <skos:prefLabel xml:lang="de">Industriestaaten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16661-4">
    <skos:related rdf:resource="descriptor/16654-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16661-4">
    <skos:related rdf:resource="descriptor/16615-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16661-4">
    <skos:related rdf:resource="descriptor/16663-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16661-4">
    <skos:related rdf:resource="descriptor/16512-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16661-4">
    <skos:related rdf:resource="descriptor/15918-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16661-4">
    <skos:related rdf:resource="descriptor/15713-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18298-1">
    <skos:prefLabel xml:lang="en">Shipping investment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18298-1">
    <skos:prefLabel xml:lang="de">Schiffsbeteiligungsgesellschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13706-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21408-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18728-2">
    <skos:altLabel xml:lang="de">Wertpapieraufsicht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19839-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15416-1">
    <skos:broader rdf:resource="thsys/73339"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18937-0">
    <skos:altLabel xml:lang="de">Lokalradio</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18937-0">
    <skos:altLabel xml:lang="de">Lokaler Hörfunk</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10542-0">
    <skos:prefLabel xml:lang="en">Growth pole theory</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10542-0">
    <skos:prefLabel xml:lang="de">Wachstumspoltheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70925">
    <skos:broader rdf:resource="thsys/70943"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15376-1">
    <skos:broader rdf:resource="thsys/72393"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15376-1">
    <skos:broader rdf:resource="descriptor/10065-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15376-1">
    <skos:broader rdf:resource="descriptor/15373-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15624-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71076">
    <skos:broader rdf:resource="thsys/71077"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13246-3">
    <skos:related rdf:resource="descriptor/13507-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13246-3">
    <skos:related rdf:resource="descriptor/13519-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13246-3">
    <skos:related rdf:resource="descriptor/13247-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13246-3">
    <skos:related rdf:resource="descriptor/13980-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13246-3">
    <skos:related rdf:resource="descriptor/16170-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13246-3">
    <skos:related rdf:resource="descriptor/14421-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13246-3">
    <skos:related rdf:resource="descriptor/13814-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13246-3">
    <skos:related rdf:resource="descriptor/13575-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13246-3">
    <skos:related rdf:resource="descriptor/13057-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13246-3">
    <skos:related rdf:resource="descriptor/18418-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13246-3">
    <skos:related rdf:resource="descriptor/13516-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11912-2">
    <skos:scopeNote xml:lang="de">Ökonomische Analysen der räumlichen Verteilung wirtschaftlicher Aktivitäten und der interregionalen Wirtschaftsbeziehungen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12580-3">
    <skos:narrower rdf:resource="descriptor/12155-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11974-1">
    <skos:prefLabel xml:lang="en">Urban transport</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11974-1">
    <skos:prefLabel xml:lang="de">Stadtverkehr</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16196-5">
    <skos:scopeNote xml:lang="de">Interdisziplinäre Sozialwissenschaft, die sich vor allem mit Erscheinungsformen und Ursachen von Verbrechen befasst, aber auch mit Verbrechensbekämpfung und -verhütung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15764-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11051-0">
    <skos:related rdf:resource="descriptor/11217-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11051-0">
    <skos:related rdf:resource="descriptor/18817-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11051-0">
    <skos:related rdf:resource="descriptor/11034-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11051-0">
    <skos:related rdf:resource="descriptor/18912-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11051-0">
    <skos:related rdf:resource="descriptor/16003-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11051-0">
    <skos:related rdf:resource="descriptor/16470-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11051-0">
    <skos:related rdf:resource="descriptor/11004-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15773-5">
    <skos:broader rdf:resource="thsys/70815"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15773-5">
    <skos:broader rdf:resource="thsys/70239"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15773-5">
    <skos:broader rdf:resource="thsys/70209"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15773-5">
    <skos:broader rdf:resource="descriptor/10011-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17778-6">
    <skos:altLabel xml:lang="de">Dominicanischer Bund</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18831-2">
    <skos:prefLabel xml:lang="en">Nonprofit tax law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18831-2">
    <skos:prefLabel xml:lang="de">Gemeinnützigkeitsrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15365-6">
    <skos:broader rdf:resource="thsys/72186"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15365-6">
    <skos:broader rdf:resource="descriptor/15272-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19522-3">
    <skos:altLabel xml:lang="de">Aktionärsstruktur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18763-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13827-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11959-4">
    <skos:prefLabel xml:lang="en">Regional input-output analysis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11959-4">
    <skos:prefLabel xml:lang="de">Regionale Input-Output-Analyse</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10159-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10487-3">
    <skos:broader rdf:resource="thsys/73345"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10487-3">
    <skos:broader rdf:resource="thsys/71109"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10487-3">
    <skos:broader rdf:resource="thsys/71085"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10487-3">
    <skos:broader rdf:resource="descriptor/10470-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11668-1">
    <skos:scopeNote xml:lang="de">Steuerhoheit liegt bei den jeweiligen Teilstaaten eines Bundesstaates.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id26">
    <rdfs:seeAlso rdf:resource="descriptor/12621-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13226-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18431-4">
    <skos:altLabel xml:lang="de">Pflanzenschutzmittelindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14433-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16328-0">
    <skos:related rdf:resource="descriptor/16013-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16328-0">
    <skos:related rdf:resource="descriptor/13385-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16328-0">
    <skos:related rdf:resource="descriptor/19728-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16328-0">
    <skos:related rdf:resource="descriptor/16510-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16621-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19411-5">
    <skos:related rdf:resource="descriptor/11695-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19411-5">
    <skos:related rdf:resource="descriptor/12427-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19411-5">
    <skos:related rdf:resource="descriptor/18745-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15998-2">
    <skos:related rdf:resource="descriptor/15965-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15998-2">
    <skos:related rdf:resource="descriptor/15964-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15998-2">
    <skos:related rdf:resource="descriptor/15961-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15998-2">
    <skos:related rdf:resource="descriptor/16713-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15998-2">
    <skos:related rdf:resource="descriptor/10498-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15998-2">
    <skos:related rdf:resource="descriptor/16429-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15998-2">
    <skos:related rdf:resource="descriptor/19768-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15998-2">
    <skos:related rdf:resource="descriptor/16426-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13562-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18413-6">
    <skos:altLabel xml:lang="de">Lebensmittelmaschinenbau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18413-6">
    <skos:altLabel xml:lang="de">Getränkemaschinenbau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18261-3">
    <skos:related rdf:resource="descriptor/16023-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18261-3">
    <skos:related rdf:resource="descriptor/14741-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18261-3">
    <skos:related rdf:resource="descriptor/14740-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11449-6">
    <skos:broader rdf:resource="thsys/70306"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11449-6">
    <skos:broader rdf:resource="descriptor/10236-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19224-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12933-3">
    <skos:prefLabel xml:lang="en">Field crops</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12933-3">
    <skos:prefLabel xml:lang="de">Ackerbau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19750-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11772-6">
    <skos:narrower rdf:resource="descriptor/12820-2"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id135">
    <rdf:value xml:lang="de">Für die Holzindustrie</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14007-2">
    <skos:related rdf:resource="descriptor/18907-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14007-2">
    <skos:related rdf:resource="descriptor/13372-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14007-2">
    <skos:related rdf:resource="descriptor/13312-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14007-2">
    <skos:related rdf:resource="descriptor/18890-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18160-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14205-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19851-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19515-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70932">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.12.01.04</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15685-2">
    <skos:prefLabel xml:lang="en">Science</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15685-2">
    <skos:prefLabel xml:lang="de">Wissenschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18248-2">
    <skos:broader rdf:resource="thsys/70415"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18248-2">
    <skos:broader rdf:resource="descriptor/14449-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10556-3">
    <skos:prefLabel xml:lang="en">Terms of aid</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10556-3">
    <skos:prefLabel xml:lang="de">Entwicklungshilfekonditionen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17942-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10823-6">
    <skos:related rdf:resource="descriptor/10567-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10823-6">
    <skos:related rdf:resource="descriptor/10472-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10823-6">
    <skos:related rdf:resource="descriptor/19235-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10823-6">
    <skos:related rdf:resource="descriptor/10572-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10823-6">
    <skos:related rdf:resource="descriptor/10824-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10823-6">
    <skos:related rdf:resource="descriptor/18121-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10823-6">
    <skos:related rdf:resource="descriptor/10798-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10823-6">
    <skos:related rdf:resource="descriptor/18001-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10823-6">
    <skos:related rdf:resource="descriptor/19315-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10823-6">
    <skos:related rdf:resource="descriptor/10602-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10823-6">
    <skos:related rdf:resource="descriptor/10827-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11717-0">
    <skos:altLabel xml:lang="en">Merit wants</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11717-0">
    <skos:altLabel xml:lang="de">Meritorische Güter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70554">
    <skos:prefLabel xml:lang="en">G.01.01  Central Europe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70554">
    <skos:prefLabel xml:lang="de">G.01.01  Mitteleuropa</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70881">
    <rdfs:label xml:lang="en">Software industry</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70881">
    <rdfs:label xml:lang="de">Softwareindustrie</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19036-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18343-1">
    <skos:prefLabel xml:lang="en">Fishery resources</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18343-1">
    <skos:prefLabel xml:lang="de">Fischereiressourcen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13583-6">
    <skos:altLabel xml:lang="de">Verkehrsströme</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13583-6">
    <skos:altLabel xml:lang="de">Verkehrsmenge</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13583-6">
    <skos:altLabel xml:lang="de">Verkehrserzeugung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13583-6">
    <skos:altLabel xml:lang="de">Verkehrsdichte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13583-6">
    <skos:altLabel xml:lang="de">Verkehrsbelastung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13583-6">
    <skos:altLabel xml:lang="de">Transportaufkommen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13583-6">
    <skos:altLabel xml:lang="de">Quellenverkehrsaufkommen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19792-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18263-6">
    <skos:broader rdf:resource="thsys/70536"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18263-6">
    <skos:broader rdf:resource="descriptor/14734-6"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id273">
    <rdf:value xml:lang="de">Für den Aufbau von Computernetzen und -systemen</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id132">
    <rdfs:seeAlso rdf:resource="descriptor/10173-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#example">
    <skos:definition xml:lang="en">An example of the use of a concept.</skos:definition>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12108-6">
    <skos:related rdf:resource="descriptor/12090-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12108-6">
    <skos:related rdf:resource="descriptor/12089-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12108-6">
    <skos:related rdf:resource="descriptor/18116-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12108-6">
    <skos:related rdf:resource="descriptor/20261-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12108-6">
    <skos:related rdf:resource="descriptor/24847-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12108-6">
    <skos:related rdf:resource="descriptor/12085-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12108-6">
    <skos:related rdf:resource="descriptor/12665-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12108-6">
    <skos:related rdf:resource="descriptor/12075-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12108-6">
    <skos:related rdf:resource="descriptor/25822-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12108-6">
    <skos:related rdf:resource="descriptor/12577-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19706-0">
    <skos:scopeNote xml:lang="de">Politik des Staates gegenüber der Kirche und anderen religiösen Gemeinschaften sowie gegenseitiges Verhältnis zwischen Kirche/Religionsgemeinschaften und Staat</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15904-2">
    <skos:broader rdf:resource="thsys/73357"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15904-2">
    <skos:broader rdf:resource="thsys/70142"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15904-2">
    <skos:broader rdf:resource="descriptor/18861-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11033-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19108-6">
    <skos:prefLabel xml:lang="en">Operation analysis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19108-6">
    <skos:prefLabel xml:lang="de">Betriebsanalyse</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11156-0">
    <skos:prefLabel xml:lang="en">Economics of crime</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11156-0">
    <skos:prefLabel xml:lang="de">Kriminalitätsökonomik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12480-0">
    <skos:narrower rdf:resource="descriptor/12525-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12480-0">
    <skos:narrower rdf:resource="descriptor/12524-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12480-0">
    <skos:narrower rdf:resource="descriptor/19100-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12480-0">
    <skos:narrower rdf:resource="descriptor/12523-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12480-0">
    <skos:narrower rdf:resource="descriptor/12535-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12480-0">
    <skos:narrower rdf:resource="descriptor/12533-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12480-0">
    <skos:narrower rdf:resource="descriptor/12521-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12480-0">
    <skos:narrower rdf:resource="descriptor/12473-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12480-0">
    <skos:narrower rdf:resource="descriptor/12497-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12480-0">
    <skos:narrower rdf:resource="descriptor/12514-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11060-6">
    <skos:prefLabel xml:lang="en">Socialism</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11060-6">
    <skos:prefLabel xml:lang="de">Sozialismus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70245">
    <skos:narrower rdf:resource="descriptor/12598-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70245">
    <skos:narrower rdf:resource="descriptor/18751-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70245">
    <skos:narrower rdf:resource="descriptor/11341-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70245">
    <skos:narrower rdf:resource="descriptor/11331-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70245">
    <skos:narrower rdf:resource="descriptor/12575-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70245">
    <skos:narrower rdf:resource="descriptor/12820-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70245">
    <skos:narrower rdf:resource="descriptor/12502-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70245">
    <skos:narrower rdf:resource="descriptor/12501-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70245">
    <skos:narrower rdf:resource="descriptor/18105-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70245">
    <skos:narrower rdf:resource="descriptor/19012-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70245">
    <skos:narrower rdf:resource="descriptor/16143-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70245">
    <skos:narrower rdf:resource="descriptor/11322-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70245">
    <skos:narrower rdf:resource="descriptor/12594-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70245">
    <skos:narrower rdf:resource="descriptor/11321-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70245">
    <skos:narrower rdf:resource="descriptor/19966-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70245">
    <skos:narrower rdf:resource="descriptor/13309-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70245">
    <skos:narrower rdf:resource="descriptor/11320-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70245">
    <skos:narrower rdf:resource="descriptor/11772-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70245">
    <skos:narrower rdf:resource="descriptor/10006-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70245">
    <skos:narrower rdf:resource="descriptor/12592-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70245">
    <skos:narrower rdf:resource="descriptor/12615-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70245">
    <skos:narrower rdf:resource="descriptor/12591-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70245">
    <skos:narrower rdf:resource="descriptor/11769-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70245">
    <skos:narrower rdf:resource="descriptor/20368-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70245">
    <skos:narrower rdf:resource="descriptor/12590-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19813-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13277-6">
    <skos:prefLabel xml:lang="en">Speech processing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13277-6">
    <skos:prefLabel xml:lang="de">Sprachverarbeitung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14377-4">
    <skos:broader rdf:resource="thsys/70318"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14377-4">
    <skos:broader rdf:resource="descriptor/14393-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19199-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19192-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11666-5">
    <skos:prefLabel xml:lang="en">Corporate income tax</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11666-5">
    <skos:prefLabel xml:lang="de">Körperschaftsteuer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11379-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10056-2">
    <skos:prefLabel xml:lang="en">Rational expectations</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10056-2">
    <skos:prefLabel xml:lang="de">Rationale Erwartung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19444-4">
    <skos:broader rdf:resource="thsys/71070"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19444-4">
    <skos:broader rdf:resource="descriptor/11919-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14930-6">
    <skos:altLabel xml:lang="de">Nahrungsmittelkonserve</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14930-6">
    <skos:altLabel xml:lang="de">Lebensmittelkonserve</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14117-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14205-5">
    <skos:narrower rdf:resource="descriptor/14653-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14205-5">
    <skos:narrower rdf:resource="descriptor/14646-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14205-5">
    <skos:narrower rdf:resource="descriptor/14645-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14205-5">
    <skos:narrower rdf:resource="descriptor/14643-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14205-5">
    <skos:narrower rdf:resource="descriptor/14638-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18777-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18509-0">
    <skos:prefLabel xml:lang="en">Craftsmen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18509-0">
    <skos:prefLabel xml:lang="de">Handwerksberufe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70998">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17749-6">
    <skos:broader rdf:resource="thsys/70085"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17749-6">
    <skos:broader rdf:resource="descriptor/17750-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17749-6">
    <skos:broader rdf:resource="descriptor/17978-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73383">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.04.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71061">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12294-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11640-2">
    <skos:altLabel xml:lang="de">Reichsabgabenordnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11640-2">
    <skos:altLabel xml:lang="de">AO (Abgabenordnung)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11640-2">
    <skos:altLabel xml:lang="de">Abgabenordnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10171-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18979-5">
    <skos:related rdf:resource="descriptor/18851-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18979-5">
    <skos:related rdf:resource="descriptor/10505-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18979-5">
    <skos:related rdf:resource="descriptor/11801-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13231-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19737-3">
    <skos:broader rdf:resource="thsys/73388"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19737-3">
    <skos:broader rdf:resource="descriptor/13399-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19737-3">
    <skos:broader rdf:resource="descriptor/15640-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19009-1">
    <skos:broader rdf:resource="thsys/73385"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19009-1">
    <skos:broader rdf:resource="thsys/73356"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19009-1">
    <skos:broader rdf:resource="thsys/70244"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19009-1">
    <skos:broader rdf:resource="descriptor/15722-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19009-1">
    <skos:broader rdf:resource="descriptor/15842-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10256-1">
    <skos:related rdf:resource="descriptor/10434-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10256-1">
    <skos:related rdf:resource="descriptor/10190-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10256-1">
    <skos:related rdf:resource="descriptor/10466-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10256-1">
    <skos:related rdf:resource="descriptor/10049-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10256-1">
    <skos:related rdf:resource="descriptor/10126-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10256-1">
    <skos:related rdf:resource="descriptor/10243-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10256-1">
    <skos:related rdf:resource="descriptor/19231-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10256-1">
    <skos:related rdf:resource="descriptor/12744-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13474-4">
    <skos:prefLabel xml:lang="en">Assortment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13474-4">
    <skos:prefLabel xml:lang="de">Sortiment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14232-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12670-2">
    <skos:broader rdf:resource="thsys/70394"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id321">
    <rdfs:seeAlso rdf:resource="descriptor/19246-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19636-2">
    <skos:broader rdf:resource="thsys/73387"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19636-2">
    <skos:broader rdf:resource="thsys/70965"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19636-2">
    <skos:broader rdf:resource="thsys/70109"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18207-2">
    <skos:broader rdf:resource="thsys/70383"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18207-2">
    <skos:broader rdf:resource="thsys/70265"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18207-2">
    <skos:broader rdf:resource="thsys/70244"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18207-2">
    <skos:broader rdf:resource="descriptor/13669-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18207-2">
    <skos:broader rdf:resource="descriptor/12171-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10555-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14734-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16720-0">
    <skos:related rdf:resource="descriptor/16721-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16720-0">
    <skos:related rdf:resource="descriptor/20022-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18903-3">
    <skos:prefLabel xml:lang="en">Hospital financing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18903-3">
    <skos:prefLabel xml:lang="de">Krankenhausfinanzierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18663-4">
    <skos:narrower rdf:resource="descriptor/18000-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13650-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16196-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19700-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70109">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id189">
    <rdfs:seeAlso rdf:resource="descriptor/10575-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18547-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18083-1">
    <skos:prefLabel xml:lang="en">Self-employed</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18083-1">
    <skos:prefLabel xml:lang="de">Selbstständige</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14223-3">
    <skos:prefLabel xml:lang="en">Asbestos</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14223-3">
    <skos:prefLabel xml:lang="de">Asbest</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10196-0">
    <skos:altLabel xml:lang="de">Spinnwebtheorem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10196-0">
    <skos:altLabel xml:lang="de">CobwebTheorem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10196-0">
    <skos:altLabel xml:lang="de">Cobweb-Theorem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13940-1">
    <skos:related rdf:resource="descriptor/14145-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13940-1">
    <skos:related rdf:resource="descriptor/12955-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13940-1">
    <skos:related rdf:resource="descriptor/13218-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13940-1">
    <skos:related rdf:resource="descriptor/14964-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13940-1">
    <skos:related rdf:resource="descriptor/19340-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13940-1">
    <skos:related rdf:resource="descriptor/13155-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17592-6">
    <skos:broader rdf:resource="thsys/70327"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17592-6">
    <skos:broader rdf:resource="descriptor/17589-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17592-6">
    <skos:broader rdf:resource="descriptor/17979-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10784-4">
    <zbwext:useInsteadNote rdf:nodeID="id129"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71005">
    <skos:broader rdf:resource="thsys/w"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10626-1">
    <zbwext:useInsteadNote rdf:nodeID="id371"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10626-1">
    <zbwext:useInsteadNote rdf:nodeID="id370"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10626-1">
    <zbwext:useInsteadNote rdf:nodeID="id369"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15977-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13207-6">
    <skos:prefLabel xml:lang="en">Beverage trade</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13207-6">
    <skos:prefLabel xml:lang="de">Getränkehandel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10126-0">
    <skos:related rdf:resource="descriptor/10137-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10126-0">
    <skos:related rdf:resource="descriptor/10434-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10126-0">
    <skos:related rdf:resource="descriptor/11765-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10126-0">
    <skos:related rdf:resource="descriptor/10120-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10126-0">
    <skos:related rdf:resource="descriptor/10256-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10126-0">
    <skos:related rdf:resource="descriptor/10294-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16400-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19157-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73355">
    <skos:broader rdf:resource="thsys/70013"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11718-5">
    <skos:altLabel xml:lang="en">Social goods</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11718-5">
    <skos:altLabel xml:lang="en">Nonmarket good</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11718-5">
    <skos:altLabel xml:lang="en">Collective goods</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11718-5">
    <skos:altLabel xml:lang="en">Collective good</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11718-5">
    <skos:altLabel xml:lang="en">Bien collectif</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11718-5">
    <skos:altLabel xml:lang="de">Theorie der öffentlichen Güter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11718-5">
    <skos:altLabel xml:lang="de">Spezifisch-öffentliches Gut</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11718-5">
    <skos:altLabel xml:lang="de">Sozialgut</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11718-5">
    <skos:altLabel xml:lang="de">Soziale Güter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11718-5">
    <skos:altLabel xml:lang="de">Öffentliche Güter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11718-5">
    <skos:altLabel xml:lang="de">Nichtmarktliches Gut</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11718-5">
    <skos:altLabel xml:lang="de">Kollektivgut</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12988-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10192-1">
    <skos:broader rdf:resource="thsys/70485"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10192-1">
    <skos:broader rdf:resource="thsys/70481"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10087-5">
    <skos:narrower rdf:resource="descriptor/10080-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19138-4">
    <skos:broader rdf:resource="thsys/70459"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18270-2">
    <skos:broader rdf:resource="thsys/73341"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18270-2">
    <skos:broader rdf:resource="thsys/73305"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18270-2">
    <skos:broader rdf:resource="descriptor/14751-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17502-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id281">
    <rdfs:seeAlso rdf:resource="descriptor/18181-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11983-0">
    <skos:related rdf:resource="descriptor/19363-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11983-0">
    <skos:related rdf:resource="descriptor/11992-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11983-0">
    <skos:related rdf:resource="descriptor/11999-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11983-0">
    <skos:related rdf:resource="descriptor/19474-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11983-0">
    <skos:related rdf:resource="descriptor/19471-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11983-0">
    <skos:related rdf:resource="descriptor/11902-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11983-0">
    <skos:related rdf:resource="descriptor/11969-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11983-0">
    <skos:related rdf:resource="descriptor/19469-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11983-0">
    <skos:related rdf:resource="descriptor/19437-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18874-5">
    <skos:prefLabel xml:lang="en">University financing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18874-5">
    <skos:prefLabel xml:lang="de">Hochschulfinanzierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13200-6">
    <skos:altLabel xml:lang="de">Einzelhandelsgeschäft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13200-6">
    <skos:altLabel xml:lang="de">Detailhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10489-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14422-4">
    <skos:related rdf:resource="descriptor/16325-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11673-1">
    <skos:altLabel xml:lang="en">Gasoline tax</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11673-1">
    <skos:altLabel xml:lang="en">Fuel tax</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11673-1">
    <skos:altLabel xml:lang="de">Heizölsteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11673-1">
    <skos:altLabel xml:lang="de">Benzinsteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18467-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19438-6">
    <skos:broader rdf:resource="thsys/70058"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19534-3">
    <skos:altLabel xml:lang="de">Verweildauer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16679-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18345-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18521-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13486-4">
    <skos:broader rdf:resource="thsys/73334"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13486-4">
    <skos:broader rdf:resource="thsys/70947"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13486-4">
    <skos:broader rdf:resource="descriptor/16195-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19093-0">
    <skos:related rdf:resource="descriptor/11451-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19093-0">
    <skos:related rdf:resource="descriptor/12228-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16541-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13291-5">
    <skos:related rdf:resource="descriptor/13690-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10198-3">
    <skos:prefLabel xml:lang="en">Marginal cost pricing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10198-3">
    <skos:prefLabel xml:lang="de">Grenzkostenpreis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17942-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20449-5">
    <skos:broader rdf:resource="thsys/71051"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20449-5">
    <skos:broader rdf:resource="thsys/70235"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20449-5">
    <skos:broader rdf:resource="descriptor/11311-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13795-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14378-2">
    <skos:altLabel xml:lang="de">Walzwerksmaschine</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14378-2">
    <skos:altLabel xml:lang="de">Walzwerkmaschine</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14378-2">
    <skos:altLabel xml:lang="de">Hüttenmaschine</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13810-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18263-6">
    <skos:altLabel xml:lang="de">Rasierwasser</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18263-6">
    <skos:altLabel xml:lang="de">Rasierseife</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18263-6">
    <skos:altLabel xml:lang="de">Herrenparfum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12137-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19394-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13197-4">
    <skos:broader rdf:resource="thsys/70959"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13197-4">
    <skos:broader rdf:resource="descriptor/18522-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10143-0">
    <skos:scopeNote xml:lang="de">Theoretische Analysen von Dauer und Intensität der Arbeitsuche bei unvollständigen Informationen auf dem Arbeitsmarkt.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11968-3">
    <skos:related rdf:resource="descriptor/11988-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17676-0">
    <skos:broader rdf:resource="thsys/70445"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17676-0">
    <skos:broader rdf:resource="descriptor/17672-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15716-3">
    <skos:related rdf:resource="descriptor/16597-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15716-3">
    <skos:related rdf:resource="descriptor/16664-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15716-3">
    <skos:related rdf:resource="descriptor/15717-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15716-3">
    <skos:related rdf:resource="descriptor/16615-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15716-3">
    <skos:related rdf:resource="descriptor/15709-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15716-3">
    <skos:related rdf:resource="descriptor/11299-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15716-3">
    <skos:related rdf:resource="descriptor/11184-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18087-0">
    <skos:altLabel xml:lang="de">Zinsänderungsrisiko</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19856-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18297-3">
    <skos:broader rdf:resource="thsys/70256"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18297-3">
    <skos:broader rdf:resource="descriptor/14418-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11650-6">
    <skos:broader rdf:resource="thsys/71074"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11650-6">
    <skos:broader rdf:resource="thsys/70397"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11650-6">
    <skos:broader rdf:resource="descriptor/19407-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11650-6">
    <skos:broader rdf:resource="descriptor/11652-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14610-3">
    <skos:broader rdf:resource="thsys/70074"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14610-3">
    <skos:broader rdf:resource="descriptor/14602-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10171-2">
    <skos:prefLabel xml:lang="en">Income elasticity</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10171-2">
    <skos:prefLabel xml:lang="de">Einkommenselastizität</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11269-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13392-6">
    <skos:broader rdf:resource="thsys/70876"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13305-6">
    <skos:narrower rdf:resource="descriptor/13302-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19133-0">
    <skos:related rdf:resource="descriptor/19134-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19133-0">
    <skos:related rdf:resource="descriptor/12665-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19133-0">
    <skos:related rdf:resource="descriptor/24785-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19133-0">
    <skos:related rdf:resource="descriptor/24781-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19133-0">
    <skos:related rdf:resource="descriptor/12786-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19133-0">
    <skos:related rdf:resource="descriptor/19141-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19133-0">
    <skos:related rdf:resource="descriptor/20442-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16369-0">
    <skos:related rdf:resource="descriptor/16354-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16369-0">
    <skos:related rdf:resource="descriptor/16432-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16369-0">
    <skos:related rdf:resource="descriptor/16370-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13306-4">
    <skos:related rdf:resource="descriptor/13160-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13306-4">
    <skos:related rdf:resource="descriptor/14931-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13306-4">
    <skos:related rdf:resource="descriptor/13219-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13306-4">
    <skos:related rdf:resource="descriptor/19421-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13306-4">
    <skos:related rdf:resource="descriptor/18562-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13306-4">
    <skos:related rdf:resource="descriptor/10495-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16318-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71089">
    <skos:narrower rdf:resource="descriptor/18701-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71089">
    <skos:narrower rdf:resource="descriptor/18157-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71089">
    <skos:narrower rdf:resource="descriptor/19360-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71089">
    <skos:narrower rdf:resource="descriptor/12292-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71089">
    <skos:narrower rdf:resource="descriptor/10817-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71089">
    <skos:narrower rdf:resource="descriptor/10816-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71089">
    <skos:narrower rdf:resource="descriptor/19359-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71089">
    <skos:narrower rdf:resource="descriptor/10725-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71089">
    <skos:narrower rdf:resource="descriptor/19321-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71089">
    <skos:narrower rdf:resource="descriptor/10815-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71089">
    <skos:narrower rdf:resource="descriptor/10814-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71089">
    <skos:narrower rdf:resource="descriptor/10813-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71089">
    <skos:narrower rdf:resource="descriptor/10785-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71089">
    <skos:narrower rdf:resource="descriptor/10812-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71089">
    <skos:narrower rdf:resource="descriptor/10809-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71089">
    <skos:narrower rdf:resource="descriptor/10807-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71089">
    <skos:narrower rdf:resource="descriptor/18694-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71089">
    <skos:narrower rdf:resource="descriptor/10811-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71089">
    <skos:narrower rdf:resource="descriptor/10805-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71089">
    <skos:narrower rdf:resource="descriptor/10806-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71089">
    <skos:narrower rdf:resource="descriptor/10804-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71089">
    <skos:narrower rdf:resource="descriptor/10808-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71089">
    <skos:narrower rdf:resource="descriptor/19664-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71089">
    <skos:narrower rdf:resource="descriptor/19277-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71089">
    <skos:narrower rdf:resource="descriptor/10801-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71089">
    <skos:narrower rdf:resource="descriptor/18686-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71089">
    <skos:narrower rdf:resource="descriptor/10550-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71089">
    <skos:narrower rdf:resource="descriptor/10797-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71089">
    <skos:narrower rdf:resource="descriptor/10798-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71089">
    <skos:narrower rdf:resource="descriptor/19358-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71089">
    <skos:narrower rdf:resource="thsys/71088"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14205-5">
    <skos:broader rdf:resource="thsys/70463"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14205-5">
    <skos:broader rdf:resource="thsys/70418"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14205-5">
    <skos:broader rdf:resource="descriptor/14236-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12800-1">
    <skos:altLabel xml:lang="de">Markenware</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12800-1">
    <skos:altLabel xml:lang="de">Markenprodukt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12800-1">
    <skos:altLabel xml:lang="de">Marke (Markenartikel)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12800-1">
    <skos:altLabel xml:lang="de">Industriemarke</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12800-1">
    <skos:altLabel xml:lang="de">Herstellermarke</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12800-1">
    <skos:altLabel xml:lang="de">Fabrikmarke</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14268-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12031-1">
    <skos:broader rdf:resource="thsys/71066"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12031-1">
    <skos:broader rdf:resource="thsys/70889"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12031-1">
    <skos:broader rdf:resource="descriptor/10012-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15895-5">
    <skos:related rdf:resource="descriptor/18195-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15895-5">
    <skos:related rdf:resource="descriptor/18340-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15895-5">
    <skos:related rdf:resource="descriptor/18315-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15895-5">
    <skos:related rdf:resource="descriptor/19793-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15895-5">
    <skos:related rdf:resource="descriptor/15880-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18364-0">
    <skos:altLabel xml:lang="de">Umweltschonende Energiepolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18364-0">
    <skos:altLabel xml:lang="de">Nachhaltige Energiepolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18383-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14729-6">
    <skos:broader rdf:resource="thsys/70199"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14729-6">
    <skos:broader rdf:resource="descriptor/14728-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73104">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10631-1">
    <skos:related rdf:resource="descriptor/10636-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12166-6">
    <skos:scopeNote xml:lang="de">Nicht für militärische Frühwarnsysteme</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19733-4">
    <skos:altLabel xml:lang="en">Sense of responsibility</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19733-4">
    <skos:altLabel xml:lang="de">Verantwortungsbewußtsein</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19733-4">
    <skos:altLabel xml:lang="de">Verantwortungsbewusstsein</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19733-4">
    <skos:altLabel xml:lang="de">Verantwortliches Handeln</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19733-4">
    <skos:altLabel xml:lang="de">Soziale Verantwortung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19733-4">
    <skos:altLabel xml:lang="de">Selbstverantwortung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19733-4">
    <skos:altLabel xml:lang="de">Moralische Verantwortung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19733-4">
    <skos:altLabel xml:lang="de">Gesellschaftliche Verantwortung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19733-4">
    <skos:altLabel xml:lang="de">Ethische Verantwortung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18077-3">
    <skos:prefLabel xml:lang="en">Palm oil</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18077-3">
    <skos:prefLabel xml:lang="de">Palmöl</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11460-4">
    <skos:altLabel xml:lang="de">Liquidität der Gesamtwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14035-4">
    <skos:related rdf:resource="descriptor/18368-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14035-4">
    <skos:related rdf:resource="descriptor/15013-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14035-4">
    <skos:related rdf:resource="descriptor/13003-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14035-4">
    <skos:related rdf:resource="descriptor/15628-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14035-4">
    <skos:related rdf:resource="descriptor/16001-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14035-4">
    <skos:related rdf:resource="descriptor/18367-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14035-4">
    <skos:related rdf:resource="descriptor/14182-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14035-4">
    <skos:related rdf:resource="descriptor/14181-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71040">
    <rdfs:label xml:lang="de">Besteuerung, Rechtsform- und Standortwahl</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10302-6">
    <skos:prefLabel xml:lang="en">Luxury good</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10302-6">
    <skos:prefLabel xml:lang="de">Luxusgut</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16033-5">
    <skos:altLabel xml:lang="de">Pachtzins</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12711-0">
    <skos:narrower rdf:resource="descriptor/12703-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10093-3">
    <skos:altLabel xml:lang="en">Theory of fairness</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10093-3">
    <skos:altLabel xml:lang="en">Rawlsian principles of justice</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10093-3">
    <skos:altLabel xml:lang="de">Rawlssche Gerechtigkeitstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10093-3">
    <skos:altLabel xml:lang="de">Rawls Theorie der Gerechtigkeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10093-3">
    <skos:altLabel xml:lang="de">Fairneßtheorie der Verteilung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10093-3">
    <skos:altLabel xml:lang="de">Fairnesstheorie der Verteilung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18419-1">
    <skos:prefLabel xml:lang="en">Space policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18419-1">
    <skos:prefLabel xml:lang="de">Raumfahrtpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70130">
    <skos:broader rdf:resource="thsys/70174"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18396-1">
    <skos:broader rdf:resource="thsys/71004"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18396-1">
    <skos:broader rdf:resource="descriptor/12993-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19277-4">
    <skos:related rdf:resource="descriptor/10801-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13736-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14039-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13742-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70819">
    <skos:narrower rdf:resource="descriptor/15683-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70819">
    <skos:narrower rdf:resource="descriptor/18956-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70819">
    <skos:narrower rdf:resource="descriptor/16483-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70819">
    <skos:narrower rdf:resource="descriptor/18955-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70819">
    <skos:narrower rdf:resource="descriptor/18954-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70819">
    <skos:narrower rdf:resource="descriptor/13363-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70819">
    <skos:narrower rdf:resource="descriptor/13362-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70819">
    <skos:narrower rdf:resource="descriptor/15675-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70819">
    <skos:narrower rdf:resource="descriptor/18099-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70819">
    <skos:narrower rdf:resource="descriptor/18953-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70819">
    <skos:narrower rdf:resource="descriptor/15668-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70819">
    <skos:narrower rdf:resource="descriptor/15643-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11695-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15946-0">
    <skos:related rdf:resource="descriptor/15925-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19243-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15685-2">
    <skos:related rdf:resource="descriptor/16732-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15685-2">
    <skos:related rdf:resource="descriptor/16487-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15685-2">
    <skos:related rdf:resource="descriptor/15686-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15685-2">
    <skos:related rdf:resource="descriptor/19010-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15685-2">
    <skos:related rdf:resource="descriptor/13350-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15685-2">
    <skos:related rdf:resource="descriptor/13349-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15685-2">
    <skos:related rdf:resource="descriptor/13343-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15685-2">
    <skos:related rdf:resource="descriptor/10436-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15685-2">
    <skos:related rdf:resource="descriptor/16708-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18665-0">
    <skos:prefLabel xml:lang="en">Mobile computing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18665-0">
    <skos:prefLabel xml:lang="de">Mobile Computing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10552-4">
    <skos:broader rdf:resource="thsys/71085"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10552-4">
    <skos:broader rdf:resource="thsys/71084"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10552-4">
    <skos:broader rdf:resource="descriptor/25742-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70987">
    <skos:prefLabel xml:lang="en">W.06.01.11  Textile and clothing industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70987">
    <skos:prefLabel xml:lang="de">W.06.01.11  Textil- und Bekleidungsindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18917-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10733-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13154-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12261-5">
    <skos:prefLabel xml:lang="en">Preference share</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12261-5">
    <skos:prefLabel xml:lang="de">Vorzugsaktie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70085">
    <skos:narrower rdf:resource="descriptor/17753-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70085">
    <skos:narrower rdf:resource="descriptor/17754-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70085">
    <skos:narrower rdf:resource="descriptor/17750-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70085">
    <skos:narrower rdf:resource="descriptor/17749-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70085">
    <skos:narrower rdf:resource="descriptor/17748-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70085">
    <skos:narrower rdf:resource="descriptor/17746-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70085">
    <skos:narrower rdf:resource="descriptor/17745-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13712-0">
    <skos:related rdf:resource="descriptor/12230-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13712-0">
    <skos:related rdf:resource="descriptor/18689-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13712-0">
    <skos:related rdf:resource="descriptor/18684-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13712-0">
    <skos:related rdf:resource="descriptor/13713-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13712-0">
    <skos:related rdf:resource="descriptor/12232-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18086-2">
    <skos:broader rdf:resource="thsys/70902"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18086-2">
    <skos:broader rdf:resource="thsys/70187"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18086-2">
    <skos:broader rdf:resource="descriptor/13291-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16564-2">
    <skos:altLabel xml:lang="de">Spezielle Soziologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16564-2">
    <skos:altLabel xml:lang="de">Soziologische Forschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16564-2">
    <skos:altLabel xml:lang="de">Bindestrich-Soziologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15599-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14967-4">
    <skos:altLabel xml:lang="de">Ölsardinen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14967-4">
    <skos:altLabel xml:lang="de">Dosenfisch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16295-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13665-4">
    <skos:narrower rdf:resource="descriptor/18937-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10383-1">
    <skos:altLabel xml:lang="de">Niveausteuerung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10383-1">
    <skos:altLabel xml:lang="de">Nachfragesteuerung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10383-1">
    <skos:altLabel xml:lang="de">Nachfrageorientierte Konjunkturpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10383-1">
    <skos:altLabel xml:lang="de">Keynesianische Wirtschaftspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18111-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16370-1">
    <skos:prefLabel xml:lang="en">Party congress</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16370-1">
    <skos:prefLabel xml:lang="de">Parteitag</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12056-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18003-4">
    <skos:prefLabel xml:lang="en">Expert system</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18003-4">
    <skos:prefLabel xml:lang="de">Expertensystem</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10693-0">
    <skos:prefLabel xml:lang="en">Export credit guarantee</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10693-0">
    <skos:prefLabel xml:lang="de">Exportgarantie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16438-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11156-0">
    <skos:related rdf:resource="descriptor/19763-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11156-0">
    <skos:related rdf:resource="descriptor/16196-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11156-0">
    <skos:related rdf:resource="descriptor/16195-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16554-5">
    <skos:prefLabel xml:lang="en">Social research</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16554-5">
    <skos:prefLabel xml:lang="de">Sozialforschung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18919-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18598-3">
    <skos:altLabel xml:lang="de">Tankschiffahrt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14313-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25743-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14656-0">
    <skos:prefLabel xml:lang="en">Organic chemical</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14656-0">
    <skos:prefLabel xml:lang="de">Organische Chemikalie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18085-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18214-5">
    <skos:altLabel xml:lang="de">Schnellgericht (Nahrungsmittel)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18214-5">
    <skos:altLabel xml:lang="de">Imbissprodukt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18214-5">
    <skos:altLabel xml:lang="de">Imbißprodukt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17792-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14941-1">
    <skos:broader rdf:resource="thsys/70498"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14941-1">
    <skos:broader rdf:resource="descriptor/14940-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11038-6">
    <skos:altLabel xml:lang="de">Theorie der Bürokratie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11038-6">
    <skos:altLabel xml:lang="de">Ökonomische Theorie der Bürokratie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15751-1">
    <skos:scopeNote xml:lang="de">Auch für die Ökonomische Theorie erschöpfbarer Ressourcen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10854-2">
    <skos:related rdf:resource="descriptor/18539-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10854-2">
    <skos:related rdf:resource="descriptor/12205-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10854-2">
    <skos:related rdf:resource="descriptor/19399-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10854-2">
    <skos:related rdf:resource="descriptor/10816-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10854-2">
    <skos:related rdf:resource="descriptor/10877-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10854-2">
    <skos:related rdf:resource="descriptor/18369-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10854-2">
    <skos:related rdf:resource="descriptor/10785-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10854-2">
    <skos:related rdf:resource="descriptor/10880-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10854-2">
    <skos:related rdf:resource="descriptor/12241-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10854-2">
    <skos:related rdf:resource="descriptor/19318-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10854-2">
    <skos:related rdf:resource="descriptor/10693-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10854-2">
    <skos:related rdf:resource="descriptor/10858-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10854-2">
    <skos:related rdf:resource="descriptor/18540-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11366-3">
    <skos:broader rdf:resource="thsys/70843"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11366-3">
    <skos:broader rdf:resource="descriptor/12589-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11366-3">
    <skos:broader rdf:resource="descriptor/11360-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73346">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12766-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19756-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18532-5">
    <skos:narrower rdf:resource="descriptor/12816-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18532-5">
    <skos:narrower rdf:resource="descriptor/18556-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10772-4">
    <skos:prefLabel xml:lang="en">Currency competition</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10772-4">
    <skos:prefLabel xml:lang="de">Währungswettbewerb</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12846-5">
    <skos:broader rdf:resource="thsys/73384"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12846-5">
    <skos:broader rdf:resource="thsys/70874"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12846-5">
    <skos:broader rdf:resource="thsys/70384"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12964-6">
    <skos:narrower rdf:resource="descriptor/12967-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12964-6">
    <skos:narrower rdf:resource="descriptor/12966-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12964-6">
    <skos:narrower rdf:resource="descriptor/12963-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18181-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73295">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14012-2">
    <skos:altLabel xml:lang="de">Seelsorgerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14012-2">
    <skos:altLabel xml:lang="de">Seelsorger</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14012-2">
    <skos:altLabel xml:lang="de">Seelsorgeberufe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14012-2">
    <skos:altLabel xml:lang="de">Rabbinerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14012-2">
    <skos:altLabel xml:lang="de">Rabbiner</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14012-2">
    <skos:altLabel xml:lang="de">Priesterinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14012-2">
    <skos:altLabel xml:lang="de">Priester</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14012-2">
    <skos:altLabel xml:lang="de">Predigerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14012-2">
    <skos:altLabel xml:lang="de">Prediger</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14012-2">
    <skos:altLabel xml:lang="de">Pfarrerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14012-2">
    <skos:altLabel xml:lang="de">Pfarrer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14012-2">
    <skos:altLabel xml:lang="de">Pastorinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14012-2">
    <skos:altLabel xml:lang="de">Pastoren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14012-2">
    <skos:altLabel xml:lang="de">Mullah</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14012-2">
    <skos:altLabel xml:lang="de">Kirchenpersonal</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14012-2">
    <skos:altLabel xml:lang="de">Kirchenbeamte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14012-2">
    <skos:altLabel xml:lang="de">Diakoninnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14012-2">
    <skos:altLabel xml:lang="de">Diakone</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12491-2">
    <skos:prefLabel xml:lang="en">Fixed costs</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12491-2">
    <skos:prefLabel xml:lang="de">Fixkosten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18268-3">
    <skos:related rdf:resource="descriptor/19623-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18268-3">
    <skos:related rdf:resource="descriptor/14623-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18268-3">
    <skos:related rdf:resource="descriptor/14631-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15606-3">
    <skos:broader rdf:resource="thsys/73342"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15606-3">
    <skos:broader rdf:resource="thsys/71039"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15606-3">
    <skos:broader rdf:resource="thsys/70882"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12297-5">
    <skos:narrower rdf:resource="descriptor/12291-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12913-2">
    <skos:altLabel xml:lang="de">Warenproduzierendes Gewerbe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12913-2">
    <skos:altLabel xml:lang="de">Sekundärer Sektor</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12913-2">
    <skos:altLabel xml:lang="de">Produzierendes Gewerbe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18317-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19802-4">
    <skos:broader rdf:resource="thsys/73341"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16435-6">
    <skos:prefLabel xml:lang="en">International politics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16435-6">
    <skos:prefLabel xml:lang="de">Internationale Politik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15966-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12621-1">
    <skos:related rdf:resource="descriptor/19010-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12621-1">
    <skos:related rdf:resource="descriptor/12603-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12621-1">
    <skos:related rdf:resource="descriptor/12725-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12621-1">
    <skos:related rdf:resource="descriptor/12757-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12621-1">
    <skos:related rdf:resource="descriptor/12602-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12621-1">
    <skos:related rdf:resource="descriptor/11186-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12621-1">
    <skos:related rdf:resource="descriptor/12748-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12621-1">
    <skos:related rdf:resource="descriptor/12607-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12621-1">
    <skos:related rdf:resource="descriptor/11303-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17695-3">
    <skos:prefLabel xml:lang="en">Benin</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17695-3">
    <skos:prefLabel xml:lang="de">Benin</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19130-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12354-5">
    <skos:narrower rdf:resource="descriptor/12376-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12354-5">
    <skos:narrower rdf:resource="descriptor/19094-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12354-5">
    <skos:narrower rdf:resource="descriptor/12366-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20881-0">
    <skos:altLabel xml:lang="de">Marginalkosten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20881-0">
    <skos:altLabel xml:lang="de">Marginale Kosten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18317-2">
    <skos:broader rdf:resource="thsys/73348"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18317-2">
    <skos:broader rdf:resource="thsys/71028"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18317-2">
    <skos:broader rdf:resource="descriptor/15635-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18317-2">
    <skos:broader rdf:resource="descriptor/13446-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12976-6">
    <skos:related rdf:resource="descriptor/18071-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12976-6">
    <skos:related rdf:resource="descriptor/12605-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12976-6">
    <skos:related rdf:resource="descriptor/15757-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12976-6">
    <skos:related rdf:resource="descriptor/15013-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12976-6">
    <skos:related rdf:resource="descriptor/14181-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12976-6">
    <skos:related rdf:resource="descriptor/15727-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12675-6">
    <skos:altLabel xml:lang="de">Warenlager</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12675-6">
    <skos:altLabel xml:lang="de">Vorratslager</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12675-6">
    <skos:altLabel xml:lang="de">Lagertyp</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12675-6">
    <skos:altLabel xml:lang="de">Lagerhaus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18549-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16495-2">
    <skos:prefLabel xml:lang="en">European integration</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16495-2">
    <skos:prefLabel xml:lang="de">Europäische Integration</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18169-5">
    <skos:prefLabel xml:lang="en">Enterprise bargaining agreement</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18169-5">
    <skos:prefLabel xml:lang="de">Betriebsvereinbarung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10498-5">
    <skos:prefLabel xml:lang="en">Basic needs</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10498-5">
    <skos:prefLabel xml:lang="de">Grundbedürfnisse</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18284-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10297-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14076-4">
    <skos:broader rdf:resource="thsys/70465"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14076-4">
    <skos:broader rdf:resource="thsys/70169"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14076-4">
    <skos:broader rdf:resource="descriptor/14073-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71079">
    <rdfs:label xml:lang="en">Fiscal transfer</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71079">
    <rdfs:label xml:lang="de">Finanzausgleich</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14949-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18406-3">
    <skos:broader rdf:resource="thsys/73343"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18406-3">
    <skos:broader rdf:resource="thsys/71001"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18406-3">
    <skos:broader rdf:resource="descriptor/18402-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18234-6">
    <skos:related rdf:resource="descriptor/14404-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70348">
    <skos:narrower rdf:resource="descriptor/14258-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70348">
    <skos:narrower rdf:resource="descriptor/14245-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70348">
    <skos:narrower rdf:resource="descriptor/14244-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70348">
    <skos:narrower rdf:resource="descriptor/18162-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70348">
    <skos:narrower rdf:resource="descriptor/14255-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70348">
    <skos:narrower rdf:resource="descriptor/19048-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70348">
    <skos:narrower rdf:resource="descriptor/14240-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70348">
    <skos:narrower rdf:resource="descriptor/14237-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70348">
    <skos:narrower rdf:resource="descriptor/19050-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70348">
    <skos:narrower rdf:resource="descriptor/14236-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70348">
    <skos:narrower rdf:resource="descriptor/14234-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70348">
    <skos:narrower rdf:resource="descriptor/14232-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70348">
    <skos:narrower rdf:resource="descriptor/14231-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70348">
    <skos:narrower rdf:resource="descriptor/14230-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70348">
    <skos:narrower rdf:resource="descriptor/14229-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70348">
    <skos:narrower rdf:resource="descriptor/14227-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70348">
    <skos:narrower rdf:resource="descriptor/14225-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70348">
    <skos:narrower rdf:resource="descriptor/14249-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70348">
    <skos:narrower rdf:resource="descriptor/14223-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70348">
    <skos:narrower rdf:resource="thsys/70647"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70348">
    <skos:narrower rdf:resource="thsys/73306"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12936-4">
    <skos:altLabel xml:lang="de">Almwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17607-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12952-6">
    <skos:altLabel xml:lang="de">Schafzucht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12952-6">
    <skos:altLabel xml:lang="de">Schäferei</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14752-4">
    <skos:prefLabel xml:lang="en">Data storage device</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14752-4">
    <skos:prefLabel xml:lang="de">Datenträger</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70889">
    <rdfs:label xml:lang="en">Real estate services</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70889">
    <rdfs:label xml:lang="de">Immobilienwirtschaft</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17829-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19453-3">
    <skos:prefLabel xml:lang="en">Urban land-use</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19453-3">
    <skos:prefLabel xml:lang="de">Städtische Bodennutzung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10636-5">
    <skos:scopeNote xml:lang="de">Wird im Rahmen der realen Außenhandelstheorie benutzt bei der wohlstandsökonomischen Analyse internationaler Tauschbeziehungen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11715-4">
    <skos:related rdf:resource="descriptor/11694-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11715-4">
    <skos:related rdf:resource="descriptor/19375-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11715-4">
    <skos:related rdf:resource="descriptor/11728-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19544-0">
    <skos:broader rdf:resource="thsys/71036"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19544-0">
    <skos:broader rdf:resource="thsys/70922"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19544-0">
    <skos:broader rdf:resource="thsys/70459"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19544-0">
    <skos:broader rdf:resource="thsys/70175"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19544-0">
    <skos:broader rdf:resource="thsys/70140"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19544-0">
    <skos:broader rdf:resource="descriptor/10012-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16716-5">
    <skos:prefLabel xml:lang="en">Logic</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16716-5">
    <skos:prefLabel xml:lang="de">Logik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15115-1">
    <skos:altLabel xml:lang="de">Zensus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15115-1">
    <skos:altLabel xml:lang="de">Vollerhebung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15115-1">
    <skos:altLabel xml:lang="de">Totalerhebung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15115-1">
    <skos:altLabel xml:lang="de">Teilerhebung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15115-1">
    <skos:altLabel xml:lang="de">Statistische Datenbasis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15115-1">
    <skos:altLabel xml:lang="de">Erhebung (Statistik)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71105">
    <skos:prefLabel xml:lang="en">V.05.06.02.02  Bond market</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71105">
    <skos:prefLabel xml:lang="de">V.05.06.02.02  Rentenmarkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18822-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13535-3">
    <skos:prefLabel xml:lang="en">Harbour constructing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13535-3">
    <skos:prefLabel xml:lang="de">Hafenbau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70087">
    <rdfs:label xml:lang="de">Rechtsformen</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11914-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18746-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18387-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17689-5">
    <skos:prefLabel xml:lang="en">Swaziland</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17689-5">
    <skos:prefLabel xml:lang="de">Swasiland</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11845-5">
    <skos:broader rdf:resource="thsys/70967"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11845-5">
    <skos:broader rdf:resource="thsys/70888"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11845-5">
    <skos:broader rdf:resource="descriptor/11847-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11845-5">
    <skos:broader rdf:resource="descriptor/18507-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14011-4">
    <skos:narrower rdf:resource="descriptor/13959-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16195-0">
    <skos:related rdf:resource="descriptor/15979-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16195-0">
    <skos:related rdf:resource="descriptor/19763-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16195-0">
    <skos:related rdf:resource="descriptor/16198-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16195-0">
    <skos:related rdf:resource="descriptor/19511-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16195-0">
    <skos:related rdf:resource="descriptor/19813-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16195-0">
    <skos:related rdf:resource="descriptor/19662-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16195-0">
    <skos:related rdf:resource="descriptor/16196-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16195-0">
    <skos:related rdf:resource="descriptor/19704-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16195-0">
    <skos:related rdf:resource="descriptor/11156-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16195-0">
    <skos:related rdf:resource="descriptor/16621-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16159-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id217">
    <rdfs:seeAlso rdf:resource="descriptor/19546-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10404-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12619-2">
    <skos:related rdf:resource="descriptor/10332-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12619-2">
    <skos:related rdf:resource="descriptor/19244-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12619-2">
    <skos:related rdf:resource="descriptor/15786-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12619-2">
    <skos:related rdf:resource="descriptor/18867-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12619-2">
    <skos:related rdf:resource="descriptor/18868-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11991-1">
    <skos:related rdf:resource="descriptor/10512-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11991-1">
    <skos:related rdf:resource="descriptor/11976-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11991-1">
    <skos:related rdf:resource="descriptor/11972-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11991-1">
    <skos:related rdf:resource="descriptor/11993-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11991-1">
    <skos:related rdf:resource="descriptor/11996-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11991-1">
    <skos:related rdf:resource="descriptor/11970-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11991-1">
    <skos:related rdf:resource="descriptor/11966-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11991-1">
    <skos:related rdf:resource="descriptor/11979-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11731-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13635-6">
    <skos:related rdf:resource="descriptor/18636-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13635-6">
    <skos:related rdf:resource="descriptor/13440-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10065-1">
    <skos:narrower rdf:resource="descriptor/10064-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10065-1">
    <skos:narrower rdf:resource="descriptor/10179-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10065-1">
    <skos:narrower rdf:resource="descriptor/15376-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10065-1">
    <skos:narrower rdf:resource="descriptor/10051-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10065-1">
    <skos:narrower rdf:resource="descriptor/10049-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10065-1">
    <skos:narrower rdf:resource="descriptor/10048-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10065-1">
    <skos:narrower rdf:resource="descriptor/10042-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10065-1">
    <skos:narrower rdf:resource="descriptor/10074-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10065-1">
    <skos:narrower rdf:resource="descriptor/10035-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16713-4">
    <skos:related rdf:resource="descriptor/19860-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16713-4">
    <skos:related rdf:resource="descriptor/15964-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16713-4">
    <skos:related rdf:resource="descriptor/11073-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16713-4">
    <skos:related rdf:resource="descriptor/15998-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18649-5">
    <skos:scopeNote xml:lang="de">Betreiber und Anbieter von Telekommunikationsnetzen und -diensten</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13130-1">
    <skos:prefLabel xml:lang="en">Yarn industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13130-1">
    <skos:prefLabel xml:lang="de">Garnindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18708-1">
    <skos:broader rdf:resource="thsys/70905"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18708-1">
    <skos:broader rdf:resource="descriptor/10994-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18708-1">
    <skos:broader rdf:resource="descriptor/18707-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14234-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16516-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19187-5">
    <skos:narrower rdf:resource="descriptor/19304-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19187-5">
    <skos:narrower rdf:resource="descriptor/19298-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19187-5">
    <skos:narrower rdf:resource="descriptor/19190-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70073">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17977-0">
    <skos:altLabel xml:lang="en">Group of African, Caribbean and Pacific States</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17977-0">
    <skos:altLabel xml:lang="de">AKP-Länder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12612-2">
    <skos:broader rdf:resource="thsys/73347"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12612-2">
    <skos:broader rdf:resource="thsys/71050"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12612-2">
    <skos:broader rdf:resource="descriptor/12602-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13163-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12276-6">
    <skos:altLabel xml:lang="de">Gewinnverteilung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12276-6">
    <skos:altLabel xml:lang="de">Gewinnausschüttung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12276-6">
    <skos:altLabel xml:lang="de">Ausschüttungspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12276-6">
    <skos:altLabel xml:lang="de">Ausschüttung von Gewinn</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16012-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14110-2">
    <skos:narrower rdf:resource="descriptor/14102-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14110-2">
    <skos:narrower rdf:resource="descriptor/14117-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14110-2">
    <skos:narrower rdf:resource="descriptor/14109-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14110-2">
    <skos:narrower rdf:resource="descriptor/14107-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14121-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12425-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15927-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19118-3">
    <skos:prefLabel xml:lang="en">Purchase</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19118-3">
    <skos:prefLabel xml:lang="de">Kauf</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14407-0">
    <skos:related rdf:resource="descriptor/18363-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14407-0">
    <skos:related rdf:resource="descriptor/14186-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14407-0">
    <skos:related rdf:resource="descriptor/14496-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14407-0">
    <skos:related rdf:resource="descriptor/14404-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14407-0">
    <skos:related rdf:resource="descriptor/18346-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70169">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">P.01.01.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13995-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18484-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12340-2">
    <skos:prefLabel xml:lang="en">Chart of accounts</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12340-2">
    <skos:prefLabel xml:lang="de">Kontenrahmen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72185">
    <skos:prefLabel xml:lang="en">V.16.04  Probability theory</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72185">
    <skos:prefLabel xml:lang="de">V.16.04  Wahrscheinlichkeitsrechnung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18666-5">
    <skos:narrower rdf:resource="descriptor/18081-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18666-5">
    <skos:narrower rdf:resource="descriptor/18665-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18666-5">
    <skos:narrower rdf:resource="descriptor/21283-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18747-5">
    <skos:narrower rdf:resource="descriptor/13305-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18747-5">
    <skos:narrower rdf:resource="descriptor/13304-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18747-5">
    <skos:narrower rdf:resource="descriptor/12027-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18747-5">
    <skos:narrower rdf:resource="descriptor/12025-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18747-5">
    <skos:narrower rdf:resource="descriptor/12030-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18747-5">
    <skos:narrower rdf:resource="descriptor/13704-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18747-5">
    <skos:narrower rdf:resource="descriptor/19065-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19715-6">
    <skos:broader rdf:resource="thsys/73366"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19715-6">
    <skos:broader rdf:resource="thsys/70834"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19426-6">
    <skos:scopeNote xml:lang="de">Weiterverarbeitung der Raffinerieprodukte zu Petrochemikalien</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13464-0">
    <skos:narrower rdf:resource="descriptor/16070-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13464-0">
    <skos:narrower rdf:resource="descriptor/19329-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13464-0">
    <skos:narrower rdf:resource="descriptor/18513-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13464-0">
    <skos:narrower rdf:resource="descriptor/19850-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13464-0">
    <skos:narrower rdf:resource="descriptor/12326-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15715-5">
    <skos:altLabel xml:lang="de">Sozioökonomische Situation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15715-5">
    <skos:altLabel xml:lang="de">Soziale Verhältnisse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15715-5">
    <skos:altLabel xml:lang="de">Soziale Situation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15715-5">
    <skos:altLabel xml:lang="de">Sozialbericht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18930-0">
    <skos:broader rdf:resource="thsys/70825"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18930-0">
    <skos:broader rdf:resource="thsys/70013"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18930-0">
    <skos:broader rdf:resource="descriptor/16761-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11311-0">
    <skos:related rdf:resource="descriptor/18159-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11311-0">
    <skos:related rdf:resource="descriptor/11310-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11311-0">
    <skos:related rdf:resource="descriptor/19110-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11311-0">
    <skos:related rdf:resource="descriptor/16136-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14900-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19398-6">
    <skos:broader rdf:resource="thsys/71078"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19398-6">
    <skos:broader rdf:resource="thsys/71066"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19398-6">
    <skos:broader rdf:resource="descriptor/11616-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19398-6">
    <skos:broader rdf:resource="descriptor/12029-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70394">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">B.05.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12072-1">
    <skos:broader rdf:resource="thsys/70140"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10144-5">
    <skos:altLabel xml:lang="de">Mikroökonomisches Modell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10144-5">
    <skos:altLabel xml:lang="de">Mikroökonomie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14027-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15001-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12091-4">
    <skos:narrower rdf:resource="descriptor/19562-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12091-4">
    <skos:narrower rdf:resource="descriptor/12047-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12091-4">
    <skos:narrower rdf:resource="descriptor/13395-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12091-4">
    <skos:narrower rdf:resource="descriptor/16439-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12091-4">
    <skos:narrower rdf:resource="descriptor/10558-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17986-6">
    <skos:broader rdf:resource="thsys/70307"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17986-6">
    <skos:broader rdf:resource="descriptor/17642-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13268-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10940-2">
    <skos:related rdf:resource="descriptor/19140-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10940-2">
    <skos:related rdf:resource="descriptor/13469-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10940-2">
    <skos:related rdf:resource="descriptor/12777-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14914-4">
    <skos:broader rdf:resource="thsys/70066"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14914-4">
    <skos:broader rdf:resource="descriptor/14846-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19425-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10847-6">
    <skos:altLabel xml:lang="de">Veredlungsverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10745-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13575-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70445">
    <skos:narrower rdf:resource="descriptor/17679-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70445">
    <skos:narrower rdf:resource="descriptor/17678-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70445">
    <skos:narrower rdf:resource="descriptor/17677-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70445">
    <skos:narrower rdf:resource="descriptor/17676-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70445">
    <skos:narrower rdf:resource="descriptor/17675-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70445">
    <skos:narrower rdf:resource="descriptor/17674-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70445">
    <skos:narrower rdf:resource="descriptor/17738-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70445">
    <skos:narrower rdf:resource="descriptor/17672-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70445">
    <skos:narrower rdf:resource="descriptor/17669-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70445">
    <skos:narrower rdf:resource="descriptor/17668-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70445">
    <skos:narrower rdf:resource="descriptor/17666-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70445">
    <skos:narrower rdf:resource="descriptor/17665-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70445">
    <skos:narrower rdf:resource="descriptor/17662-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70445">
    <skos:narrower rdf:resource="descriptor/17660-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70445">
    <skos:narrower rdf:resource="descriptor/18284-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70445">
    <skos:narrower rdf:resource="descriptor/17658-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70445">
    <skos:narrower rdf:resource="descriptor/17731-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70445">
    <skos:narrower rdf:resource="descriptor/17654-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13846-0">
    <skos:related rdf:resource="descriptor/13844-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13846-0">
    <skos:related rdf:resource="descriptor/18496-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13846-0">
    <skos:related rdf:resource="descriptor/11901-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19794-5">
    <skos:altLabel xml:lang="de">EMV (Elektromagnetische Verträglichkeit)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19794-5">
    <skos:altLabel xml:lang="de">Elektro-Smog</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19794-5">
    <skos:altLabel xml:lang="de">Elektromagnetische Verträglichkeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19794-5">
    <skos:altLabel xml:lang="de">Elektromagnetische Strahlung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19794-5">
    <skos:altLabel xml:lang="de">Elektro Smog</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18279-5">
    <skos:related rdf:resource="descriptor/18078-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18279-5">
    <skos:related rdf:resource="descriptor/14772-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18616-6">
    <skos:broader rdf:resource="thsys/70940"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18616-6">
    <skos:broader rdf:resource="thsys/70926"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18616-6">
    <skos:broader rdf:resource="descriptor/11833-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12558-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15836-0">
    <skos:broader rdf:resource="thsys/70017"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15836-0">
    <skos:broader rdf:resource="descriptor/15790-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70557">
    <skos:broader rdf:resource="thsys/70002"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13047-2">
    <skos:editorialNote xml:lang="de">Möglichst spezifischer indexieren.</skos:editorialNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19478-1">
    <skos:altLabel xml:lang="de">Bevölkerungsaufbau der Gemeinden</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13066-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19148-1">
    <skos:broader rdf:resource="thsys/72186"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19148-1">
    <skos:broader rdf:resource="thsys/71035"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19148-1">
    <skos:broader rdf:resource="descriptor/15243-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19148-1">
    <skos:broader rdf:resource="descriptor/12631-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16605-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73387">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.02.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18562-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13633-3">
    <skos:broader rdf:resource="thsys/70938"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13633-3">
    <skos:broader rdf:resource="descriptor/13563-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18017-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12171-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19119-1">
    <skos:altLabel xml:lang="en">Special account management</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19119-1">
    <skos:altLabel xml:lang="en">National account management</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19119-1">
    <skos:altLabel xml:lang="en">NAM (National Account Management)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19119-1">
    <skos:altLabel xml:lang="en">Key account management</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19119-1">
    <skos:altLabel xml:lang="en">KAM (Key-account management)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19119-1">
    <skos:altLabel xml:lang="de">Stammkunden-Marketing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19119-1">
    <skos:altLabel xml:lang="de">Kundenstamm-Marketing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19119-1">
    <skos:altLabel xml:lang="de">Kundengruppenmanagement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19119-1">
    <skos:altLabel xml:lang="de">Key-Account-Management</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19119-1">
    <skos:altLabel xml:lang="de">Großkundenmanagement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18994-2">
    <skos:altLabel xml:lang="en">Compulsory insurance</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18994-2">
    <skos:altLabel xml:lang="de">Zwangsversicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18994-2">
    <skos:altLabel xml:lang="de">Versicherungszwang</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18994-2">
    <skos:altLabel xml:lang="de">Pflichtversicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70328">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14674-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18721-2">
    <skos:related rdf:resource="descriptor/13777-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18721-2">
    <skos:related rdf:resource="descriptor/18722-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18721-2">
    <skos:related rdf:resource="descriptor/18719-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11663-4">
    <skos:related rdf:resource="descriptor/12285-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19275-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70109">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.15</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10073-2">
    <skos:altLabel xml:lang="en">Equilibrium dynamics</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10073-2">
    <skos:altLabel xml:lang="en">Dynamic general equilibrium</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10073-2">
    <skos:altLabel xml:lang="de">Dynamische Gleichgewichtstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19688-4">
    <skos:broader rdf:resource="thsys/73381"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13507-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18540-6">
    <skos:related rdf:resource="descriptor/10806-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18540-6">
    <skos:related rdf:resource="descriptor/10854-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18953-2">
    <skos:prefLabel xml:lang="en">Professional sports</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18953-2">
    <skos:prefLabel xml:lang="de">Professioneller Sport</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12667-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15919-3">
    <skos:related rdf:resource="descriptor/16615-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15919-3">
    <skos:related rdf:resource="descriptor/16303-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15919-3">
    <skos:related rdf:resource="descriptor/19639-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15919-3">
    <skos:related rdf:resource="descriptor/19721-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15919-3">
    <skos:related rdf:resource="descriptor/19722-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15919-3">
    <skos:related rdf:resource="descriptor/15951-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15919-3">
    <skos:related rdf:resource="descriptor/19648-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15919-3">
    <skos:related rdf:resource="descriptor/15989-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15919-3">
    <skos:related rdf:resource="descriptor/15907-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15919-3">
    <skos:related rdf:resource="descriptor/19843-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10812-4">
    <skos:narrower rdf:resource="descriptor/10758-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18587-1">
    <skos:prefLabel xml:lang="en">Cruise</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18587-1">
    <skos:prefLabel xml:lang="de">Kreuzfahrt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13575-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19779-1">
    <skos:prefLabel xml:lang="en">Public speaking</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19779-1">
    <skos:prefLabel xml:lang="de">Vortragstechnik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18703-4">
    <skos:broader rdf:resource="thsys/73331"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18703-4">
    <skos:broader rdf:resource="thsys/70905"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18703-4">
    <skos:broader rdf:resource="descriptor/16048-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18703-4">
    <skos:broader rdf:resource="descriptor/13693-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13303-3">
    <skos:prefLabel xml:lang="en">Public services</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13303-3">
    <skos:prefLabel xml:lang="de">Öffentliche Dienstleistung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18366-3">
    <skos:broader rdf:resource="thsys/73363"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18366-3">
    <skos:broader rdf:resource="thsys/70982"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18366-3">
    <skos:broader rdf:resource="thsys/70865"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18366-3">
    <skos:broader rdf:resource="descriptor/10470-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12423-5">
    <skos:altLabel xml:lang="de">Firmenwert</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12423-5">
    <skos:altLabel xml:lang="de">Fassonwert</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18658-4">
    <skos:scopeNote xml:lang="de">Ein Teil der am Netz angeschlossenen Computer erbringt als Server Leistungen, die von anderen ebenfalls vernetzten Rechnern als Clients genutzt werden.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73388">
    <skos:broader rdf:resource="thsys/70096"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19750-4">
    <skos:scopeNote xml:lang="de">1. Philosophische Denkrichtung, die vor allem von französischen Soziologen und Philosophen vertreten wird und übergreifende Ideen und Bekenntnisse ablehnt. 2. Kunst- und Kulturrichtung, die für eine Pluralität von Stilmitteln vor allem in Kunst und Architektur steht.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12587-3">
    <skos:broader rdf:resource="thsys/70244"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70530">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18864-1">
    <skos:altLabel xml:lang="en">Childcare</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18864-1">
    <skos:altLabel xml:lang="en">Child day care</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18864-1">
    <skos:altLabel xml:lang="de">Tagesmutter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18864-1">
    <skos:altLabel xml:lang="de">Kleinkinderbetreuung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18864-1">
    <skos:altLabel xml:lang="de">Kindertagesstätte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18864-1">
    <skos:altLabel xml:lang="de">Kinderkrippe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18864-1">
    <skos:altLabel xml:lang="de">Kinderhort</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18864-1">
    <skos:altLabel xml:lang="de">Kinderheim</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18864-1">
    <skos:altLabel xml:lang="de">Kindergarten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18864-1">
    <skos:altLabel xml:lang="de">Kindererziehung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19281-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70182">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.03.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19966-2">
    <skos:prefLabel xml:lang="en">Fee</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19966-2">
    <skos:prefLabel xml:lang="de">Honorar</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16540-2">
    <skos:prefLabel xml:lang="en">Pensions policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16540-2">
    <skos:prefLabel xml:lang="de">Rentenpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16274-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10985-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11213-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13583-6">
    <skos:narrower rdf:resource="descriptor/13564-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17589-2">
    <skos:altLabel xml:lang="de">Südostasiatische Staaten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17589-2">
    <skos:altLabel xml:lang="de">Südostasiatische Länder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16271-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19269-3">
    <skos:altLabel xml:lang="en">Arbitrage pricing theory</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19269-3">
    <skos:altLabel xml:lang="en">Arbitrage pricing model</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19269-3">
    <skos:altLabel xml:lang="de">Arbitragetheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19269-3">
    <skos:altLabel xml:lang="de">Arbitrage-Preis-Theorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19269-3">
    <skos:altLabel xml:lang="de">APT (Arbitrage-Preis-Theorie)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12020-6">
    <skos:prefLabel xml:lang="en">Land tenure</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12020-6">
    <skos:prefLabel xml:lang="de">Bodenrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19750-4">
    <skos:prefLabel xml:lang="en">Postmodernism</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19750-4">
    <skos:prefLabel xml:lang="de">Postmoderne</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10859-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13294-6">
    <skos:altLabel xml:lang="de">Sterbekasse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13294-6">
    <skos:altLabel xml:lang="de">Sterbegeldversicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13294-6">
    <skos:altLabel xml:lang="de">Risikolebensversicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13294-6">
    <skos:altLabel xml:lang="de">Kapitalversicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13294-6">
    <skos:altLabel xml:lang="de">Aussteuerversicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13294-6">
    <skos:altLabel xml:lang="de">Ausbildungsversicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16100-2">
    <skos:broader rdf:resource="thsys/73337"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16100-2">
    <skos:broader rdf:resource="thsys/71073"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16100-2">
    <skos:broader rdf:resource="descriptor/11525-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10197-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17949-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16642-1">
    <skos:altLabel xml:lang="de">Wertwandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16642-1">
    <skos:altLabel xml:lang="de">Wertstruktur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16642-1">
    <skos:altLabel xml:lang="de">Wertorientierung (Soziale Werte)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16642-1">
    <skos:altLabel xml:lang="de">Wertewandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16642-1">
    <skos:altLabel xml:lang="de">Gesellschaftliche Grundwerte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14078-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15880-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14184-1">
    <skos:altLabel xml:lang="en">Primary fuel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14184-1">
    <skos:altLabel xml:lang="de">Energierohstoff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13452-0">
    <skos:prefLabel xml:lang="en">Cultivation system</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13452-0">
    <skos:prefLabel xml:lang="de">Anbausystem</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73341">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19694-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18550-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16984-1">
    <skos:broader rdf:resource="thsys/70505"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16984-1">
    <skos:broader rdf:resource="descriptor/16990-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16984-1">
    <skos:broader rdf:resource="descriptor/17991-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16984-1">
    <skos:broader rdf:resource="descriptor/17982-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17598-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19493-5">
    <skos:broader rdf:resource="thsys/71061"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19493-5">
    <skos:broader rdf:resource="thsys/70051"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19493-5">
    <skos:broader rdf:resource="descriptor/18011-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71012">
    <rdfs:label xml:lang="en">Energy policy</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71012">
    <rdfs:label xml:lang="de">Energiepolitik</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16395-6">
    <skos:prefLabel xml:lang="en">Political violence</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16395-6">
    <skos:prefLabel xml:lang="de">Politische Gewalt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12206-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id387">
    <rdfs:seeAlso rdf:resource="descriptor/12765-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70504">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17584-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16269-4">
    <skos:related rdf:resource="descriptor/16287-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16269-4">
    <skos:related rdf:resource="descriptor/12775-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16269-4">
    <skos:related rdf:resource="descriptor/13661-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16269-4">
    <skos:related rdf:resource="descriptor/16265-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16269-4">
    <skos:related rdf:resource="descriptor/10142-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16269-4">
    <skos:related rdf:resource="descriptor/15961-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19262-3">
    <skos:prefLabel xml:lang="en">Monetary target</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19262-3">
    <skos:prefLabel xml:lang="de">Geldpolitisches Ziel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19299-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10722-5">
    <skos:altLabel xml:lang="en">Exchange rate management</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10722-5">
    <skos:altLabel xml:lang="en">Exchange rate adjustment</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10722-5">
    <skos:altLabel xml:lang="de">Wechselkursstabilisierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10722-5">
    <skos:altLabel xml:lang="de">Stützungskäufe am Devisenmarkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10722-5">
    <skos:altLabel xml:lang="de">Kursstützungskäufe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10722-5">
    <skos:altLabel xml:lang="de">Kurssicherungspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10722-5">
    <skos:altLabel xml:lang="de">Intervention am Devisenmarkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10722-5">
    <skos:altLabel xml:lang="de">Devisenmarktpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10722-5">
    <skos:altLabel xml:lang="de">Devisenmarktintervention</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10722-5">
    <skos:altLabel xml:lang="de">Devisenkurspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19236-4">
    <skos:prefLabel xml:lang="en">Household expenditure</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19236-4">
    <skos:prefLabel xml:lang="de">Verbraucherausgaben</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13498-4">
    <skos:broader rdf:resource="thsys/70943"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13498-4">
    <skos:broader rdf:resource="descriptor/13510-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73338">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.10.07.04</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19311-2">
    <skos:related rdf:resource="descriptor/10667-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19311-2">
    <skos:related rdf:resource="descriptor/10834-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18484-4">
    <skos:broader rdf:resource="thsys/71056"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18484-4">
    <skos:broader rdf:resource="thsys/71005"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18484-4">
    <skos:broader rdf:resource="descriptor/11286-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71103">
    <skos:narrower rdf:resource="descriptor/18701-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71103">
    <skos:narrower rdf:resource="descriptor/10821-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71103">
    <skos:narrower rdf:resource="descriptor/10773-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71103">
    <skos:narrower rdf:resource="descriptor/18699-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71103">
    <skos:narrower rdf:resource="descriptor/10770-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71103">
    <skos:narrower rdf:resource="descriptor/19333-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71103">
    <skos:narrower rdf:resource="descriptor/13766-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71103">
    <skos:narrower rdf:resource="descriptor/13720-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71103">
    <skos:narrower rdf:resource="descriptor/10781-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71103">
    <skos:narrower rdf:resource="descriptor/10780-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71103">
    <skos:narrower rdf:resource="descriptor/18688-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71103">
    <skos:narrower rdf:resource="descriptor/10779-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71103">
    <skos:narrower rdf:resource="descriptor/10719-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71103">
    <skos:narrower rdf:resource="descriptor/13757-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71103">
    <skos:narrower rdf:resource="descriptor/10776-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16426-0">
    <skos:related rdf:resource="descriptor/16389-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16426-0">
    <skos:related rdf:resource="descriptor/16264-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16426-0">
    <skos:related rdf:resource="descriptor/15964-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16426-0">
    <skos:related rdf:resource="descriptor/13662-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16426-0">
    <skos:related rdf:resource="descriptor/16411-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16426-0">
    <skos:related rdf:resource="descriptor/16406-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16426-0">
    <skos:related rdf:resource="descriptor/15998-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11341-5">
    <skos:altLabel xml:lang="en">Remuneration system</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11341-5">
    <skos:altLabel xml:lang="en">Compensation structure</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11341-5">
    <skos:altLabel xml:lang="de">Lohnsystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11341-5">
    <skos:altLabel xml:lang="de">Lohnform</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11341-5">
    <skos:altLabel xml:lang="de">Lohnfindung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11341-5">
    <skos:altLabel xml:lang="de">Lohnermittlungssystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11341-5">
    <skos:altLabel xml:lang="de">Entlohnungssystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11341-5">
    <skos:altLabel xml:lang="de">Entgeltsystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11341-5">
    <skos:altLabel xml:lang="de">Betriebliche Lohnpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11341-5">
    <skos:altLabel xml:lang="de">Betriebliche Gehaltspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16563-4">
    <skos:altLabel xml:lang="de">Gesellschaftswissenschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11361-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16043-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19104-0">
    <skos:broader rdf:resource="thsys/71041"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19104-0">
    <skos:broader rdf:resource="thsys/71036"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19104-0">
    <skos:broader rdf:resource="descriptor/12884-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19104-0">
    <skos:broader rdf:resource="descriptor/12165-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11626-3">
    <skos:altLabel xml:lang="de">Unternehmen mit öffentlicher Beteiligung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18228-1">
    <skos:scopeNote xml:lang="de">Nur für die ehemalige jugoslawische Teilrepublik Mazedonien</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19101-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13720-1">
    <skos:narrower rdf:resource="descriptor/18688-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13001-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15696-4">
    <skos:broader rdf:resource="thsys/70007"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15696-4">
    <skos:broader rdf:resource="descriptor/15690-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19713-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16995-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11326-1">
    <skos:related rdf:resource="descriptor/11322-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11326-1">
    <skos:related rdf:resource="descriptor/11774-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11326-1">
    <skos:related rdf:resource="descriptor/11736-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18274-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70930">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.12.03.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18995-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11571-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id172">
    <rdfs:seeAlso rdf:resource="descriptor/18249-0"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id196">
    <rdfs:seeAlso rdf:resource="descriptor/19724-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15258-0">
    <skos:altLabel xml:lang="de">Statistische Inferenz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15258-0">
    <skos:altLabel xml:lang="de">Schließende Statistik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15258-0">
    <skos:altLabel xml:lang="de">Konfirmatorische Statistik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15258-0">
    <skos:altLabel xml:lang="de">Inferenzielle Statistik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15258-0">
    <skos:altLabel xml:lang="de">Inferentielle Statistik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15258-0">
    <skos:altLabel xml:lang="de">Induktive Statistik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18620-1">
    <skos:scopeNote xml:lang="de">Transportlogistik bezieht sich auf die Gestaltung und/oder Inanspruchnahme von logistischen Dienstleistungen durch die verladende Wirtschaft.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13297-0">
    <skos:prefLabel xml:lang="en">Indemnity insurance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13297-0">
    <skos:prefLabel xml:lang="de">Schadenversicherung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14736-2">
    <skos:prefLabel xml:lang="en">Oral hygiene product</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14736-2">
    <skos:prefLabel xml:lang="de">Mundpflegemittel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12919-4">
    <skos:prefLabel xml:lang="en">Plastics industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12919-4">
    <skos:prefLabel xml:lang="de">Kunststoffindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10209-3">
    <skos:related rdf:resource="descriptor/10224-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10209-3">
    <skos:related rdf:resource="descriptor/15458-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10209-3">
    <skos:related rdf:resource="descriptor/10216-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15075-1">
    <skos:prefLabel xml:lang="en">Economic indicator</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15075-1">
    <skos:prefLabel xml:lang="de">Wirtschaftsindikator</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16271-3">
    <skos:related rdf:resource="descriptor/18366-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16271-3">
    <skos:related rdf:resource="descriptor/16460-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16271-3">
    <skos:related rdf:resource="descriptor/18447-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16271-3">
    <skos:related rdf:resource="descriptor/12925-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16271-3">
    <skos:related rdf:resource="descriptor/15005-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16271-3">
    <skos:related rdf:resource="descriptor/18814-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16271-3">
    <skos:related rdf:resource="descriptor/16458-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12488-5">
    <skos:broader rdf:resource="thsys/71042"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12488-5">
    <skos:broader rdf:resource="descriptor/12312-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18256-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15066-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18664-2">
    <skos:broader rdf:resource="thsys/71037"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18664-2">
    <skos:broader rdf:resource="thsys/70383"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18664-2">
    <skos:broader rdf:resource="descriptor/15599-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18664-2">
    <skos:broader rdf:resource="descriptor/12165-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12800-1">
    <skos:related rdf:resource="descriptor/19564-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12800-1">
    <skos:related rdf:resource="descriptor/18066-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12800-1">
    <skos:related rdf:resource="descriptor/20002-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12800-1">
    <skos:related rdf:resource="descriptor/12801-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12800-1">
    <skos:related rdf:resource="descriptor/16078-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12800-1">
    <skos:related rdf:resource="descriptor/19129-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12800-1">
    <skos:related rdf:resource="descriptor/19316-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11224-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13780-4">
    <skos:altLabel xml:lang="en">Instalment credit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13780-4">
    <skos:altLabel xml:lang="de">Teilzahlungskredit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13780-4">
    <skos:altLabel xml:lang="de">Ratenzahlung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13780-4">
    <skos:altLabel xml:lang="de">Ratenkredit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13780-4">
    <skos:altLabel xml:lang="de">Konsumentenkredit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13780-4">
    <skos:altLabel xml:lang="de">Abzahlungskredit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70557">
    <rdfs:label xml:lang="en">Western Europe</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70557">
    <rdfs:label xml:lang="de">Westeuropa</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12975-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10392-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10473-0">
    <skos:broader rdf:resource="thsys/71109"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id248">
    <rdfs:seeAlso rdf:resource="descriptor/10968-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18265-2">
    <skos:altLabel xml:lang="de">Schaumbad</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18265-2">
    <skos:altLabel xml:lang="de">Duschzusatz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18265-2">
    <skos:altLabel xml:lang="de">Duschgel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18265-2">
    <skos:altLabel xml:lang="de">Badesalz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13146-0">
    <skos:prefLabel xml:lang="en">Textile finishing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13146-0">
    <skos:prefLabel xml:lang="de">Textilveredlungsindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18339-6">
    <skos:altLabel xml:lang="de">Forstliche Forschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18806-1">
    <skos:prefLabel xml:lang="en">Local public investment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18806-1">
    <skos:prefLabel xml:lang="de">Kommunale Investition</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19824-1">
    <skos:prefLabel xml:lang="en">Non-competition law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19824-1">
    <skos:prefLabel xml:lang="de">Wettbewerbsverbot</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15061-5">
    <skos:broader rdf:resource="thsys/72181"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15061-5">
    <skos:broader rdf:resource="thsys/70142"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15061-5">
    <skos:broader rdf:resource="descriptor/18887-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18541-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15656-2">
    <skos:narrower rdf:resource="descriptor/18074-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15483-0">
    <skos:related rdf:resource="descriptor/12088-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15483-0">
    <skos:related rdf:resource="descriptor/19581-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15483-0">
    <skos:related rdf:resource="descriptor/15064-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15483-0">
    <skos:related rdf:resource="descriptor/15057-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15483-0">
    <skos:related rdf:resource="descriptor/19150-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15483-0">
    <skos:related rdf:resource="descriptor/12085-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15483-0">
    <skos:related rdf:resource="descriptor/15498-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15483-0">
    <skos:related rdf:resource="descriptor/15470-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11703-4">
    <skos:altLabel xml:lang="de">Steuervereinheitlichung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11703-4">
    <skos:altLabel xml:lang="de">Steuerintegration</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11703-4">
    <skos:altLabel xml:lang="de">Internationale Steuerharmonisierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11411-3">
    <skos:broader rdf:resource="thsys/70114"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11411-3">
    <skos:broader rdf:resource="thsys/70055"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11411-3">
    <skos:broader rdf:resource="descriptor/11415-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13493-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19596-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70212">
    <rdfs:label xml:lang="de">Wirtschaftskonzentration und Wettbewerbsbeschränkung</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70892">
    <skos:prefLabel xml:lang="de">W.16  Versicherungswesen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id34">
    <rdfs:seeAlso rdf:resource="descriptor/16067-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18017-0">
    <skos:prefLabel xml:lang="en">Information management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18017-0">
    <skos:prefLabel xml:lang="de">Informationsmanagement</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70855">
    <rdfs:label xml:lang="en">Social economy</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70855">
    <rdfs:label xml:lang="de">Gemeinwirtschaft</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13055-3">
    <skos:broader rdf:resource="thsys/70997"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13055-3">
    <skos:broader rdf:resource="descriptor/13060-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21002-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13390-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19773-6">
    <skos:prefLabel xml:lang="en">Community-based initiative</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19773-6">
    <skos:prefLabel xml:lang="de">Gemeinwesenarbeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18030-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12806-3">
    <skos:related rdf:resource="descriptor/20176-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12806-3">
    <skos:related rdf:resource="descriptor/12809-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12806-3">
    <skos:related rdf:resource="descriptor/19758-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12806-3">
    <skos:related rdf:resource="descriptor/15821-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12806-3">
    <skos:related rdf:resource="descriptor/20536-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12806-3">
    <skos:related rdf:resource="descriptor/12075-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25881-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19236-4">
    <skos:broader rdf:resource="thsys/71112"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19236-4">
    <skos:broader rdf:resource="thsys/70814"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19236-4">
    <skos:broader rdf:resource="descriptor/12306-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15657-0">
    <skos:related rdf:resource="descriptor/15676-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15657-0">
    <skos:related rdf:resource="descriptor/14004-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15657-0">
    <skos:related rdf:resource="descriptor/19600-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15657-0">
    <skos:related rdf:resource="descriptor/13365-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15657-0">
    <skos:related rdf:resource="descriptor/13197-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15686-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70862">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.22.01.07</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10940-2">
    <skos:narrower rdf:resource="descriptor/12782-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14732-3">
    <skos:prefLabel xml:lang="en">Hair care product</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14732-3">
    <skos:prefLabel xml:lang="de">Haarpflegemittel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12055-1">
    <skos:broader rdf:resource="thsys/70212"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12055-1">
    <skos:broader rdf:resource="thsys/70087"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12055-1">
    <skos:broader rdf:resource="descriptor/12056-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12931-0">
    <skos:prefLabel xml:lang="en">Utilities industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12931-0">
    <skos:prefLabel xml:lang="de">Versorgungswirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11578-2">
    <skos:prefLabel xml:lang="en">Public building</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11578-2">
    <skos:prefLabel xml:lang="de">Öffentlicher Bau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19730-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19581-1">
    <skos:related rdf:resource="descriptor/15064-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19581-1">
    <skos:related rdf:resource="descriptor/19072-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19581-1">
    <skos:related rdf:resource="descriptor/15483-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19581-1">
    <skos:related rdf:resource="descriptor/15045-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19581-1">
    <skos:related rdf:resource="descriptor/15470-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10698-4">
    <skos:altLabel xml:lang="de">Internationales Handelsabkommen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14004-1">
    <skos:narrower rdf:resource="descriptor/13382-2"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id312">
    <rdf:value xml:lang="de">für die Anwendung statistischer Methoden</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19047-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12404-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15460-5">
    <skos:narrower rdf:resource="descriptor/19597-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19086-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11490-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18428-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19212-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12106-3">
    <skos:prefLabel xml:lang="en">Organizational research</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12106-3">
    <skos:prefLabel xml:lang="de">Organisationsforschung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19005-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24740-5">
    <skos:broader rdf:resource="thsys/70901"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24740-5">
    <skos:broader rdf:resource="descriptor/12281-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12116-0">
    <skos:broader rdf:resource="thsys/71050"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12116-0">
    <skos:broader rdf:resource="thsys/70562"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12116-0">
    <skos:broader rdf:resource="descriptor/11186-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12116-0">
    <skos:broader rdf:resource="descriptor/12621-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19482-3">
    <skos:broader rdf:resource="thsys/73352"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19482-3">
    <skos:broader rdf:resource="descriptor/19486-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18092-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10838-0">
    <skos:related rdf:resource="descriptor/10815-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10838-0">
    <skos:related rdf:resource="descriptor/10570-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10838-0">
    <skos:related rdf:resource="descriptor/19356-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10838-0">
    <skos:related rdf:resource="descriptor/10872-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10838-0">
    <skos:related rdf:resource="descriptor/10701-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10838-0">
    <skos:related rdf:resource="descriptor/10699-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15737-2">
    <skos:prefLabel xml:lang="en">Emission control</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15737-2">
    <skos:prefLabel xml:lang="de">Immissionsschutz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14567-6">
    <skos:related rdf:resource="descriptor/13090-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14567-6">
    <skos:related rdf:resource="descriptor/14255-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14567-6">
    <skos:related rdf:resource="descriptor/14033-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14567-6">
    <skos:related rdf:resource="descriptor/14548-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14053-2">
    <skos:prefLabel xml:lang="en">Hydroelectric power plant</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14053-2">
    <skos:prefLabel xml:lang="de">Wasserkraftwerk</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16967-1">
    <skos:altLabel xml:lang="de">Neuchatel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70990">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15103-1">
    <skos:narrower rdf:resource="descriptor/15086-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15103-1">
    <skos:narrower rdf:resource="descriptor/18386-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18192-3">
    <skos:prefLabel xml:lang="en">Coffee production</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18192-3">
    <skos:prefLabel xml:lang="de">Kaffeeanbau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18880-3">
    <skos:prefLabel xml:lang="en">School of economics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18880-3">
    <skos:prefLabel xml:lang="de">Wirtschaftshochschule</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/p">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11893-1">
    <skos:related rdf:resource="descriptor/11974-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11893-1">
    <skos:related rdf:resource="descriptor/11993-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11893-1">
    <skos:related rdf:resource="descriptor/11876-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11893-1">
    <skos:related rdf:resource="descriptor/18609-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11893-1">
    <skos:related rdf:resource="descriptor/13629-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19443-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11770-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12514-2">
    <skos:prefLabel xml:lang="en">Contribution margin accounting</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12514-2">
    <skos:prefLabel xml:lang="de">Deckungsbeitragsrechnung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12953-4">
    <skos:related rdf:resource="descriptor/19346-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12953-4">
    <skos:related rdf:resource="descriptor/19345-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12953-4">
    <skos:related rdf:resource="descriptor/18219-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12953-4">
    <skos:related rdf:resource="descriptor/14144-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12225-2">
    <skos:altLabel xml:lang="en">Funds required</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71018">
    <rdfs:label xml:lang="en">Fishery</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71018">
    <rdfs:label xml:lang="de">Fischerei</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16192-6">
    <skos:prefLabel xml:lang="en">Juvenile delinquency</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16192-6">
    <skos:prefLabel xml:lang="de">Jugendkriminalität</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72180">
    <rdfs:label xml:lang="de">Statistik und Ökonometrie</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15835-2">
    <skos:prefLabel xml:lang="en">Informal group</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15835-2">
    <skos:prefLabel xml:lang="de">Informelle Gruppe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13644-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12178-6">
    <skos:prefLabel xml:lang="en">Sole trader</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12178-6">
    <skos:prefLabel xml:lang="de">Einzelunternehmen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13136-3">
    <skos:narrower rdf:resource="descriptor/13137-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13136-3">
    <skos:narrower rdf:resource="descriptor/13135-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13272-2">
    <skos:broader rdf:resource="thsys/70923"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13272-2">
    <skos:broader rdf:resource="thsys/70902"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13272-2">
    <skos:broader rdf:resource="descriptor/13291-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13272-2">
    <skos:broader rdf:resource="descriptor/13274-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18389-5">
    <skos:prefLabel xml:lang="en">Small-scale mining</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18389-5">
    <skos:prefLabel xml:lang="de">Kleinbergbau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10684-1">
    <skos:narrower rdf:resource="descriptor/10701-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10684-1">
    <skos:narrower rdf:resource="descriptor/19318-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10684-1">
    <skos:narrower rdf:resource="descriptor/10693-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10684-1">
    <skos:narrower rdf:resource="descriptor/10687-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18339-6">
    <skos:narrower rdf:resource="descriptor/18988-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10958-4">
    <skos:altLabel xml:lang="de">Wirtschaftskonzentrationspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10958-4">
    <skos:altLabel xml:lang="de">Antitrustpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10958-4">
    <skos:altLabel xml:lang="de">Antitrustbewegung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19051-2">
    <skos:altLabel xml:lang="de">Stahlkonstruktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19051-2">
    <skos:altLabel xml:lang="de">Stahlbauprodukt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19051-2">
    <skos:altLabel xml:lang="de">Kaltprofil</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19051-2">
    <skos:altLabel xml:lang="de">Kaltband</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19051-2">
    <skos:altLabel xml:lang="de">Blankstahl</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18683-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11813-4">
    <skos:broader rdf:resource="thsys/71016"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11813-4">
    <skos:broader rdf:resource="thsys/70227"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11813-4">
    <skos:broader rdf:resource="descriptor/15754-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18728-2">
    <skos:prefLabel xml:lang="en">Financial market supervision</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18728-2">
    <skos:prefLabel xml:lang="de">Börsenaufsicht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19825-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18108-4">
    <skos:broader rdf:resource="thsys/70087"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18108-4">
    <skos:broader rdf:resource="descriptor/12194-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18108-4">
    <skos:broader rdf:resource="descriptor/18121-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18005-0">
    <skos:scopeNote xml:lang="de">Für Hörfunkanstalten sowie gemeinschaftliche Hörfunk- und Fernsehanstalten</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11240-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13683-2">
    <skos:prefLabel xml:lang="en">Network architecture</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13683-2">
    <skos:prefLabel xml:lang="de">Netzarchitektur</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17491-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12363-4">
    <skos:broader rdf:resource="thsys/71043"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14149-3">
    <skos:prefLabel xml:lang="en">Forest product</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14149-3">
    <skos:prefLabel xml:lang="de">Forstprodukt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17724-1">
    <skos:prefLabel xml:lang="en">West Africa</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17724-1">
    <skos:prefLabel xml:lang="de">Westafrika</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11751-0">
    <skos:altLabel xml:lang="de">Vermögensumverteilung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11751-0">
    <skos:altLabel xml:lang="de">Vermögenskonzentration</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11751-0">
    <skos:altLabel xml:lang="de">Ungleiche Vermögensverteilung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15674-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14181-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18538-0">
    <skos:scopeNote xml:lang="de">Industrieunternehmen, die ihre Exporte selbst organisieren und durchführen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17696-1">
    <skos:altLabel xml:lang="de">Obervolta</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16002-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16579-3">
    <skos:altLabel xml:lang="de">Theorie der Gesellschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16579-3">
    <skos:altLabel xml:lang="de">Gesellschaftsbegriff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16579-3">
    <skos:altLabel xml:lang="de">Gesellschaftsanalyse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10710-5">
    <skos:related rdf:resource="descriptor/10649-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10710-5">
    <skos:related rdf:resource="descriptor/10715-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10710-5">
    <skos:related rdf:resource="descriptor/16122-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10710-5">
    <skos:related rdf:resource="descriptor/10713-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10710-5">
    <skos:related rdf:resource="descriptor/10697-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11810-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19522-3">
    <skos:prefLabel xml:lang="en">Ownership structure</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19522-3">
    <skos:prefLabel xml:lang="de">Eigentümerstruktur</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19577-6">
    <skos:broader rdf:resource="thsys/72393"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19577-6">
    <skos:broader rdf:resource="descriptor/15225-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19577-6">
    <skos:broader rdf:resource="descriptor/15376-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11879-2">
    <skos:related rdf:resource="descriptor/11880-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11879-2">
    <skos:related rdf:resource="descriptor/16688-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11879-2">
    <skos:related rdf:resource="descriptor/11857-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11879-2">
    <skos:related rdf:resource="descriptor/11852-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15055-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10397-4">
    <skos:altLabel xml:lang="en">Zero inflation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10397-4">
    <skos:altLabel xml:lang="de">Währungsstabilität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10397-4">
    <skos:altLabel xml:lang="de">Sicherung der Währung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10397-4">
    <skos:altLabel xml:lang="de">Sicherung der Kaufkraft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10397-4">
    <skos:altLabel xml:lang="de">Preisstabilität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10397-4">
    <skos:altLabel xml:lang="de">Kaufkraftstabilität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10397-4">
    <skos:altLabel xml:lang="de">Geldwertstabilität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id383">
    <rdf:value xml:lang="de">Für Einbeziehung der Zeit in ökonomische Modelle</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18628-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21014-0">
    <skos:related rdf:resource="descriptor/19567-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21014-0">
    <skos:related rdf:resource="descriptor/11706-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21014-0">
    <skos:related rdf:resource="descriptor/19410-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21014-0">
    <skos:related rdf:resource="descriptor/11636-0"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id77">
    <rdf:value xml:lang="de">Für Sachleistungen</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#exactMatch">
    <rdfs:subPropertyOf rdf:resource="http://www.w3.org/2004/02/skos/core#closeMatch"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13746-4">
    <skos:broader rdf:resource="thsys/70900"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13746-4">
    <skos:broader rdf:resource="descriptor/12228-3"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id267">
    <rdf:value xml:lang="de">Für betriebswirtschaftliche Produktionszahlen</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12716-4">
    <skos:broader rdf:resource="thsys/70310"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12716-4">
    <skos:broader rdf:resource="descriptor/12725-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13624-4">
    <skos:broader rdf:resource="thsys/70934"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13624-4">
    <skos:broader rdf:resource="thsys/70930"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13624-4">
    <skos:broader rdf:resource="descriptor/13622-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13624-4">
    <skos:broader rdf:resource="descriptor/13553-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19309-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11566-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10125-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15949-1">
    <skos:altLabel xml:lang="en">Life expectancy</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15949-1">
    <skos:altLabel xml:lang="en">Death</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15949-1">
    <skos:altLabel xml:lang="de">Todesursache</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15949-1">
    <skos:altLabel xml:lang="de">Sterblichkeitsziffer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15949-1">
    <skos:altLabel xml:lang="de">Sterberate</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15949-1">
    <skos:altLabel xml:lang="de">Mortalität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15949-1">
    <skos:altLabel xml:lang="de">Letalität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15949-1">
    <skos:altLabel xml:lang="de">Lebenserwartung (Demographie)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15949-1">
    <skos:altLabel xml:lang="de">Lebensdauer (Demographie)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70893">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.15</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11069-2">
    <skos:narrower rdf:resource="descriptor/11481-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11069-2">
    <skos:narrower rdf:resource="descriptor/11535-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11069-2">
    <skos:narrower rdf:resource="descriptor/13451-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12777-5">
    <skos:broader rdf:resource="thsys/70459"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12777-5">
    <skos:broader rdf:resource="descriptor/12036-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13694-4">
    <zbwext:useInsteadNote rdf:nodeID="id78"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19767-1">
    <skos:broader rdf:resource="thsys/73355"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19767-1">
    <skos:broader rdf:resource="thsys/70051"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19767-1">
    <skos:broader rdf:resource="descriptor/10037-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19767-1">
    <skos:broader rdf:resource="descriptor/16554-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14480-4">
    <skos:related rdf:resource="descriptor/10455-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14480-4">
    <skos:related rdf:resource="descriptor/13074-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15107-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14534-0">
    <skos:related rdf:resource="descriptor/18249-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14534-0">
    <skos:related rdf:resource="descriptor/14816-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19345-6">
    <skos:prefLabel xml:lang="en">Pork market</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19345-6">
    <skos:prefLabel xml:lang="de">Schweinemarkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73339">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.10.07.03</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16670-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18601-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13584-4">
    <skos:prefLabel xml:lang="en">Choice of transport mode</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13584-4">
    <skos:prefLabel xml:lang="de">Verkehrsmittelwahl</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16678-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15924-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19165-1">
    <skos:broader rdf:resource="thsys/70147"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19165-1">
    <skos:broader rdf:resource="descriptor/17571-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19161-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19213-2">
    <skos:altLabel xml:lang="en">Economic analysis of law</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19213-2">
    <skos:altLabel xml:lang="de">Ökonomische Theorie des Rechts</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19213-2">
    <skos:altLabel xml:lang="de">Ökonomische Analyse des Rechts</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14626-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12154-6">
    <skos:altLabel xml:lang="de">Projektorientierte Führung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12154-6">
    <skos:altLabel xml:lang="de">Projektorganisation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12154-6">
    <skos:altLabel xml:lang="de">Parallelorganisation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18680-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70049">
    <skos:narrower rdf:resource="descriptor/19202-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70049">
    <skos:narrower rdf:resource="descriptor/10032-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70049">
    <skos:narrower rdf:resource="descriptor/10062-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70049">
    <skos:narrower rdf:resource="descriptor/10065-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70049">
    <skos:narrower rdf:resource="descriptor/11039-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70049">
    <skos:narrower rdf:resource="descriptor/19201-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70049">
    <skos:narrower rdf:resource="descriptor/16730-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70049">
    <skos:narrower rdf:resource="descriptor/18679-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70049">
    <skos:narrower rdf:resource="descriptor/10060-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70049">
    <skos:narrower rdf:resource="descriptor/19200-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70049">
    <skos:narrower rdf:resource="descriptor/10058-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70049">
    <skos:narrower rdf:resource="descriptor/11223-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70049">
    <skos:narrower rdf:resource="descriptor/19524-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70049">
    <skos:narrower rdf:resource="descriptor/11754-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70049">
    <skos:narrower rdf:resource="descriptor/18129-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70049">
    <skos:narrower rdf:resource="descriptor/10055-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70049">
    <skos:narrower rdf:resource="descriptor/20022-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70049">
    <skos:narrower rdf:resource="descriptor/24857-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70049">
    <skos:narrower rdf:resource="descriptor/10179-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70049">
    <skos:narrower rdf:resource="descriptor/10052-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70049">
    <skos:narrower rdf:resource="descriptor/18151-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70049">
    <skos:narrower rdf:resource="descriptor/19989-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70049">
    <skos:narrower rdf:resource="descriptor/18310-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70049">
    <skos:narrower rdf:resource="descriptor/10049-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70049">
    <skos:narrower rdf:resource="descriptor/19207-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70049">
    <skos:narrower rdf:resource="descriptor/12619-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70049">
    <skos:narrower rdf:resource="descriptor/12057-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70049">
    <skos:narrower rdf:resource="descriptor/10045-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70049">
    <skos:narrower rdf:resource="descriptor/19199-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70049">
    <skos:narrower rdf:resource="descriptor/19198-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70049">
    <skos:narrower rdf:resource="descriptor/10074-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70049">
    <skos:narrower rdf:resource="descriptor/10036-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70049">
    <skos:narrower rdf:resource="descriptor/19197-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70049">
    <skos:narrower rdf:resource="descriptor/10035-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70049">
    <skos:narrower rdf:resource="descriptor/10072-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70049">
    <skos:narrower rdf:resource="descriptor/19196-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70049">
    <skos:narrower rdf:resource="descriptor/19195-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70049">
    <skos:narrower rdf:resource="descriptor/18138-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70049">
    <skos:narrower rdf:resource="thsys/70057"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70049">
    <skos:narrower rdf:resource="thsys/70141"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70049">
    <skos:narrower rdf:resource="thsys/70274"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70049">
    <skos:narrower rdf:resource="thsys/70188"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70485">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">B.01.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12808-6">
    <skos:related rdf:resource="descriptor/12809-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12808-6">
    <skos:related rdf:resource="descriptor/12758-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12808-6">
    <skos:related rdf:resource="descriptor/16034-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12808-6">
    <skos:related rdf:resource="descriptor/10016-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12808-6">
    <skos:related rdf:resource="descriptor/10459-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12808-6">
    <skos:related rdf:resource="descriptor/12794-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70937">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.12.01.02.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11430-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10102-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19282-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19393-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13474-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15971-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18086-2">
    <skos:scopeNote xml:lang="de">Banken, die vorwiegend im Wertpapiergeschäft und der Unternehmensfinanzierung tätig sind.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15770-4">
    <skos:altLabel xml:lang="en">Pollution control</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15770-4">
    <skos:altLabel xml:lang="de">Umweltüberwachung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14258-5">
    <skos:related rdf:resource="descriptor/13023-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14258-5">
    <skos:related rdf:resource="descriptor/19050-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14258-5">
    <skos:related rdf:resource="descriptor/14232-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14258-5">
    <skos:related rdf:resource="descriptor/14249-6"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id44">
    <rdfs:seeAlso rdf:resource="descriptor/11845-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18090-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11603-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18878-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16594-0">
    <skos:scopeNote xml:lang="de">Nur im Sinne eines allgemeinen Fortschritt- und Modernisierungsdenkens.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18151-3">
    <skos:related rdf:resource="descriptor/12073-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18151-3">
    <skos:related rdf:resource="descriptor/11039-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18151-3">
    <skos:related rdf:resource="descriptor/15860-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18151-3">
    <skos:related rdf:resource="descriptor/18832-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12523-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16830-0">
    <skos:prefLabel xml:lang="en">Berlin</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16830-0">
    <skos:prefLabel xml:lang="de">Berlin</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19214-0">
    <skos:altLabel xml:lang="de">Relationaler Vertrag</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12451-0">
    <skos:altLabel xml:lang="de">Schuldenkonsolidierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12451-0">
    <skos:altLabel xml:lang="de">Konsolidierte Gewinn- und Verlustrechnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12451-0">
    <skos:altLabel xml:lang="de">Kapitalkonsolidierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12451-0">
    <skos:altLabel xml:lang="de">Gewinnkonsolidierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12451-0">
    <skos:altLabel xml:lang="de">Erfolgskonsolidierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19553-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12590-0">
    <skos:altLabel xml:lang="en">Piecework payment</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12590-0">
    <skos:altLabel xml:lang="de">Zeitakkord</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12590-0">
    <skos:altLabel xml:lang="de">Stückakkord</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12590-0">
    <skos:altLabel xml:lang="de">Akkordschere</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12590-0">
    <skos:altLabel xml:lang="de">Akkordarbeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71064">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.11.06</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12960-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10176-6">
    <skos:prefLabel xml:lang="en">Household production</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10176-6">
    <skos:prefLabel xml:lang="de">Haushaltsproduktion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12562-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19495-1">
    <skos:altLabel xml:lang="de">Arbeitslosigkeitstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18625-5">
    <skos:broader rdf:resource="thsys/72181"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18625-5">
    <skos:broader rdf:resource="thsys/70928"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18625-5">
    <skos:broader rdf:resource="descriptor/15076-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11652-2">
    <skos:narrower rdf:resource="descriptor/11666-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11652-2">
    <skos:narrower rdf:resource="descriptor/11650-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15076-6">
    <skos:narrower rdf:resource="descriptor/19590-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15076-6">
    <skos:narrower rdf:resource="descriptor/15075-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15076-6">
    <skos:narrower rdf:resource="descriptor/19589-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15076-6">
    <skos:narrower rdf:resource="descriptor/18625-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15076-6">
    <skos:narrower rdf:resource="descriptor/15108-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15076-6">
    <skos:narrower rdf:resource="descriptor/15107-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15076-6">
    <skos:narrower rdf:resource="descriptor/19588-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15076-6">
    <skos:narrower rdf:resource="descriptor/15105-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15076-6">
    <skos:narrower rdf:resource="descriptor/15104-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15076-6">
    <skos:narrower rdf:resource="descriptor/19602-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15076-6">
    <skos:narrower rdf:resource="descriptor/19587-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15076-6">
    <skos:narrower rdf:resource="descriptor/19586-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15076-6">
    <skos:narrower rdf:resource="descriptor/15103-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15076-6">
    <skos:narrower rdf:resource="descriptor/19579-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15076-6">
    <skos:narrower rdf:resource="descriptor/13465-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15076-6">
    <skos:narrower rdf:resource="descriptor/19585-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15076-6">
    <skos:narrower rdf:resource="descriptor/18351-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15076-6">
    <skos:narrower rdf:resource="descriptor/15102-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15076-6">
    <skos:narrower rdf:resource="descriptor/19604-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15076-6">
    <skos:narrower rdf:resource="descriptor/11297-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15076-6">
    <skos:narrower rdf:resource="descriptor/15101-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15076-6">
    <skos:narrower rdf:resource="descriptor/13689-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15076-6">
    <skos:narrower rdf:resource="descriptor/11264-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15076-6">
    <skos:narrower rdf:resource="descriptor/13433-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11863-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16512-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12933-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19274-3">
    <skos:altLabel xml:lang="en">Equity risk premium</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19274-3">
    <skos:altLabel xml:lang="en">Equity premium (Risikoprämie)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19274-3">
    <skos:altLabel xml:lang="de">Risikozuschlag</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10312-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14214-4">
    <skos:broader rdf:resource="thsys/70649"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14214-4">
    <skos:broader rdf:resource="descriptor/14218-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14214-4">
    <skos:broader rdf:resource="descriptor/14216-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70144">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18484-4">
    <skos:prefLabel xml:lang="en">Industrial workers</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18484-4">
    <skos:prefLabel xml:lang="de">Industriearbeiter</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15737-2">
    <skos:related rdf:resource="descriptor/18030-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15737-2">
    <skos:related rdf:resource="descriptor/12609-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11983-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18397-6">
    <skos:narrower rdf:resource="descriptor/19046-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18397-6">
    <skos:narrower rdf:resource="descriptor/18153-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18397-6">
    <skos:narrower rdf:resource="descriptor/19047-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11407-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id274">
    <rdf:value xml:lang="de">Für Berufs- und Ausbildungsfragen von Rechtsanwälten</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16712-6">
    <skos:scopeNote xml:lang="de">Nur für die Historische Schule der Nationalökonomie.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10239-1">
    <skos:prefLabel xml:lang="en">Potential output</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10239-1">
    <skos:prefLabel xml:lang="de">Wirtschaftspotenzial</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15704-3">
    <skos:altLabel xml:lang="de">Neomerkantilismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15704-3">
    <skos:altLabel xml:lang="de">Kameralismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15704-3">
    <skos:altLabel xml:lang="de">Colbertismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15704-3">
    <skos:altLabel xml:lang="de">Bullionismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19628-1">
    <skos:related rdf:resource="descriptor/11866-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19628-1">
    <skos:related rdf:resource="descriptor/12907-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19628-1">
    <skos:related rdf:resource="descriptor/11850-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70580">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15989-3">
    <skos:related rdf:resource="descriptor/19687-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15989-3">
    <skos:related rdf:resource="descriptor/19854-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15989-3">
    <skos:related rdf:resource="descriptor/19323-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15989-3">
    <skos:related rdf:resource="descriptor/16486-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15989-3">
    <skos:related rdf:resource="descriptor/19640-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15989-3">
    <skos:related rdf:resource="descriptor/15919-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15989-3">
    <skos:related rdf:resource="descriptor/19649-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15989-3">
    <skos:related rdf:resource="descriptor/15911-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15989-3">
    <skos:related rdf:resource="descriptor/15907-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15989-3">
    <skos:related rdf:resource="descriptor/19843-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11360-1">
    <skos:prefLabel xml:lang="en">Vocational education</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11360-1">
    <skos:prefLabel xml:lang="de">Berufsbildung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17779-4">
    <skos:prefLabel xml:lang="en">Dominican Republic</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17779-4">
    <skos:prefLabel xml:lang="de">Dominikanische Republik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19359-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11216-1">
    <skos:prefLabel xml:lang="en">Hidden unemployment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11216-1">
    <skos:prefLabel xml:lang="de">Versteckte Arbeitslosigkeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13861-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13277-6">
    <skos:broader rdf:resource="thsys/73339"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13277-6">
    <skos:broader rdf:resource="descriptor/15611-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13277-6">
    <skos:broader rdf:resource="descriptor/15606-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11153-6">
    <skos:altLabel xml:lang="de">Vertretungstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11153-6">
    <skos:altLabel xml:lang="de">Vertretertheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11153-6">
    <skos:altLabel xml:lang="de">Prinzipal-Agent-Theorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11153-6">
    <skos:altLabel xml:lang="de">Prinzipal-Agent</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11153-6">
    <skos:altLabel xml:lang="de">Principal-Agent</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11153-6">
    <skos:altLabel xml:lang="de">Agent-Principal</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11153-6">
    <skos:altLabel xml:lang="de">Agency-Theorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11153-6">
    <skos:altLabel xml:lang="de">Agency Theorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14412-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19402-6">
    <skos:broader rdf:resource="thsys/71076"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19402-6">
    <skos:broader rdf:resource="descriptor/19408-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19402-6">
    <skos:broader rdf:resource="descriptor/11636-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70060">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.08</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13412-5">
    <skos:prefLabel xml:lang="en">Armed forces</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13412-5">
    <skos:prefLabel xml:lang="de">Militär</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10928-6">
    <skos:related rdf:resource="descriptor/10942-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10928-6">
    <skos:related rdf:resource="descriptor/19432-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10928-6">
    <skos:related rdf:resource="descriptor/10894-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10928-6">
    <skos:related rdf:resource="descriptor/12807-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10928-6">
    <skos:related rdf:resource="descriptor/10948-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10928-6">
    <skos:related rdf:resource="descriptor/19667-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10928-6">
    <skos:related rdf:resource="descriptor/10012-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10928-6">
    <skos:related rdf:resource="descriptor/19121-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10928-6">
    <skos:related rdf:resource="descriptor/12770-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10928-6">
    <skos:related rdf:resource="descriptor/10926-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10928-6">
    <skos:related rdf:resource="descriptor/10459-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10928-6">
    <skos:related rdf:resource="descriptor/10891-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71092">
    <skos:broader rdf:resource="thsys/71093"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13638-0">
    <skos:broader rdf:resource="thsys/70924"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13638-0">
    <skos:broader rdf:resource="descriptor/13588-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16156-3">
    <skos:prefLabel xml:lang="en">Victim compensation law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16156-3">
    <skos:prefLabel xml:lang="de">Soziales Entschädigungsrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18448-1">
    <skos:broader rdf:resource="thsys/71004"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18448-1">
    <skos:broader rdf:resource="thsys/70828"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19459-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#narrower">
    <owl:inverseOf rdf:resource="http://www.w3.org/2004/02/skos/core#broader"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12729-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19135-3">
    <skos:prefLabel xml:lang="en">Shop window</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19135-3">
    <skos:prefLabel xml:lang="de">Schaufenster</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71107">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.05.06.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11931-5">
    <skos:prefLabel xml:lang="en">Spillover effect</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11931-5">
    <skos:prefLabel xml:lang="de">Spillover-Effekt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13993-1">
    <skos:broader rdf:resource="thsys/71054"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13993-1">
    <skos:broader rdf:resource="thsys/70872"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13993-1">
    <skos:broader rdf:resource="descriptor/13985-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13106-5">
    <skos:related rdf:resource="descriptor/14789-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13106-5">
    <skos:related rdf:resource="descriptor/14377-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10208-5">
    <skos:prefLabel xml:lang="en">Natural monopoly</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10208-5">
    <skos:prefLabel xml:lang="de">Natürliches Monopol</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18877-6">
    <skos:altLabel xml:lang="de">Universitätsreform</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18877-6">
    <skos:altLabel xml:lang="de">Studienreform</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11839-0">
    <skos:altLabel xml:lang="de">Wohnraumkündigungsschutzgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11839-0">
    <skos:altLabel xml:lang="de">Kündigungsschutz im Mietrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13525-6">
    <skos:prefLabel xml:lang="en">Railway tariff</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13525-6">
    <skos:prefLabel xml:lang="de">Eisenbahntarif</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19069-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19655-5">
    <skos:prefLabel xml:lang="en">Knowledge management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19655-5">
    <skos:prefLabel xml:lang="de">Wissensmanagement</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19821-0">
    <skos:prefLabel xml:lang="en">Permit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19821-0">
    <skos:prefLabel xml:lang="de">Genehmigung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11752-5">
    <skos:related rdf:resource="descriptor/10110-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11752-5">
    <skos:related rdf:resource="descriptor/11751-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11752-5">
    <skos:related rdf:resource="descriptor/10401-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11752-5">
    <skos:related rdf:resource="descriptor/11735-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11752-5">
    <skos:related rdf:resource="descriptor/11339-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15669-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#semanticRelation">
    <rdfs:label xml:lang="en">semantic relation</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18129-3">
    <skos:prefLabel xml:lang="en">Product substitut</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18129-3">
    <skos:prefLabel xml:lang="de">Produktsubstitution</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73374">
    <skos:broader rdf:resource="thsys/70095"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71025">
    <skos:narrower rdf:resource="descriptor/19349-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71025">
    <skos:narrower rdf:resource="descriptor/19347-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71025">
    <skos:narrower rdf:resource="descriptor/12956-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71025">
    <skos:narrower rdf:resource="descriptor/12955-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71025">
    <skos:narrower rdf:resource="descriptor/12972-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71025">
    <skos:narrower rdf:resource="descriptor/19346-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71025">
    <skos:narrower rdf:resource="descriptor/19345-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71025">
    <skos:narrower rdf:resource="descriptor/12953-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71025">
    <skos:narrower rdf:resource="descriptor/12952-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71025">
    <skos:narrower rdf:resource="descriptor/19344-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71025">
    <skos:narrower rdf:resource="descriptor/19343-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71025">
    <skos:narrower rdf:resource="descriptor/12950-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71025">
    <skos:narrower rdf:resource="descriptor/12971-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71025">
    <skos:narrower rdf:resource="descriptor/12946-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71025">
    <skos:narrower rdf:resource="descriptor/19183-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71025">
    <skos:narrower rdf:resource="descriptor/12962-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71025">
    <skos:narrower rdf:resource="descriptor/12941-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71025">
    <skos:narrower rdf:resource="descriptor/19340-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19442-1">
    <skos:broader rdf:resource="thsys/73351"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19442-1">
    <skos:broader rdf:resource="thsys/73350"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19442-1">
    <skos:broader rdf:resource="thsys/70209"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19442-1">
    <skos:broader rdf:resource="descriptor/15669-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11146-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19652-4">
    <skos:scopeNote xml:lang="de">Eigenes Handelssegment an der Frankfurter Wertpapierbörse zur Finanzierung wachstumsstarker Klein- und Mittelunternehmen aus innovativen Branchen. Auch für analoge institutionelle Regelungen in anderen Ländern.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17592-6">
    <skos:prefLabel xml:lang="en">Viet Nam</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17592-6">
    <skos:prefLabel xml:lang="de">Vietnam</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13861-4">
    <skos:related rdf:resource="descriptor/13867-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13861-4">
    <skos:related rdf:resource="descriptor/13849-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13861-4">
    <skos:related rdf:resource="descriptor/13862-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13861-4">
    <skos:related rdf:resource="descriptor/18754-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13861-4">
    <skos:related rdf:resource="descriptor/16688-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19695-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19361-1">
    <skos:prefLabel xml:lang="en">Development research</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19361-1">
    <skos:prefLabel xml:lang="de">Entwicklungsforschung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13277-6">
    <skos:narrower rdf:resource="descriptor/13666-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13277-6">
    <skos:narrower rdf:resource="descriptor/13667-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13200-6">
    <skos:broader rdf:resource="thsys/70947"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13200-6">
    <skos:broader rdf:resource="descriptor/13210-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71011">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11303-6">
    <skos:related rdf:resource="descriptor/19112-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11303-6">
    <skos:related rdf:resource="descriptor/12573-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11303-6">
    <skos:related rdf:resource="descriptor/12571-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11303-6">
    <skos:related rdf:resource="descriptor/12129-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11303-6">
    <skos:related rdf:resource="descriptor/11295-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11303-6">
    <skos:related rdf:resource="descriptor/11293-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11303-6">
    <skos:related rdf:resource="descriptor/11290-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11303-6">
    <skos:related rdf:resource="descriptor/19111-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11303-6">
    <skos:related rdf:resource="descriptor/12621-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11303-6">
    <skos:related rdf:resource="descriptor/12631-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13849-1">
    <skos:related rdf:resource="descriptor/13867-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13849-1">
    <skos:related rdf:resource="descriptor/13845-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13849-1">
    <skos:related rdf:resource="descriptor/13830-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13849-1">
    <skos:related rdf:resource="descriptor/13841-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13849-1">
    <skos:related rdf:resource="descriptor/11574-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13849-1">
    <skos:related rdf:resource="descriptor/13861-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13849-1">
    <skos:related rdf:resource="descriptor/15715-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13849-1">
    <skos:related rdf:resource="descriptor/13312-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73376">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.04.02.06</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12041-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18124-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16311-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19450-2">
    <skos:altLabel xml:lang="de">Landesraumordnungsplan</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19764-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15460-5">
    <skos:broader rdf:resource="thsys/73104"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15460-5">
    <skos:broader rdf:resource="descriptor/15437-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16529-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10346-0">
    <skos:related rdf:resource="descriptor/10667-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11637-5">
    <skos:narrower rdf:resource="descriptor/11694-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11637-5">
    <skos:narrower rdf:resource="descriptor/12848-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10375-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19357-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14384-0">
    <skos:prefLabel xml:lang="en">Food machinery</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14384-0">
    <skos:prefLabel xml:lang="de">Nahrungsmittelmaschine</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12259-6">
    <skos:altLabel xml:lang="en">Self financing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12259-6">
    <skos:altLabel xml:lang="de">Gewinnthesaurierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12748-5">
    <skos:altLabel xml:lang="de">Arbeitsvorbereitung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12748-5">
    <skos:altLabel xml:lang="de">Arbeitsverteilung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12314-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id76">
    <rdfs:seeAlso rdf:resource="descriptor/14728-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11310-2">
    <skos:altLabel xml:lang="de">Paritätische Mitbestimmung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11310-2">
    <skos:altLabel xml:lang="de">Montanmitbestimmungsgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11310-2">
    <skos:altLabel xml:lang="de">Mitbestimmungsgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11310-2">
    <skos:altLabel xml:lang="de">Gesetz über die Mitbestimmung der Arbeitnehmer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11310-2">
    <skos:altLabel xml:lang="de">Betriebliche Mitbestimmung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11310-2">
    <skos:altLabel xml:lang="de">Arbeitnehmerpartizipation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19746-2">
    <skos:broader rdf:resource="thsys/73393"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19746-2">
    <skos:broader rdf:resource="descriptor/15686-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10581-4">
    <skos:altLabel xml:lang="en">Multifibre arrangement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10581-4">
    <skos:altLabel xml:lang="en">ATC (Agreement on Textiles and Clothing)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10581-4">
    <skos:altLabel xml:lang="en">Agreement on textiles and clothing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10581-4">
    <skos:altLabel xml:lang="de">WTA (Welttextilabkommen)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10581-4">
    <skos:altLabel xml:lang="de">Textilabkommen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10581-4">
    <skos:altLabel xml:lang="de">Multifaserabkommen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11679-3">
    <skos:prefLabel xml:lang="en">Corporate taxation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11679-3">
    <skos:prefLabel xml:lang="de">Unternehmensbesteuerung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id114">
    <rdfs:seeAlso rdf:resource="descriptor/10182-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11607-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19297-5">
    <skos:broader rdf:resource="thsys/71029"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19297-5">
    <skos:broader rdf:resource="thsys/70960"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19297-5">
    <skos:broader rdf:resource="descriptor/19188-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18661-1">
    <skos:broader rdf:resource="thsys/73341"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18661-1">
    <skos:broader rdf:resource="descriptor/18426-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18661-1">
    <skos:broader rdf:resource="descriptor/14751-6"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id382">
    <rdfs:seeAlso rdf:resource="descriptor/19127-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16192-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12920-5">
    <skos:prefLabel xml:lang="en">Petroleum industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12920-5">
    <skos:prefLabel xml:lang="de">Mineralölwirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11314-1">
    <skos:altLabel xml:lang="en">Labor relations</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11314-1">
    <skos:altLabel xml:lang="en">Joint industrial committee</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11314-1">
    <skos:altLabel xml:lang="en">Joint consultation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11314-1">
    <skos:altLabel xml:lang="en">Industrial relations</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11314-1">
    <skos:altLabel xml:lang="de">Industrielle Beziehungen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11314-1">
    <skos:altLabel xml:lang="de">Industrielle Arbeitsbeziehungen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11314-1">
    <skos:altLabel xml:lang="de">Arbeitnehmer-Arbeitgeber-Beziehungen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11314-1">
    <skos:altLabel xml:lang="de">Arbeitgeber-Arbeitnehmer-Beziehungen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10199-1">
    <skos:broader rdf:resource="thsys/70178"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10199-1">
    <skos:broader rdf:resource="descriptor/10220-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14439-1">
    <skos:altLabel xml:lang="de">Autobus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10816-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13661-5">
    <skos:broader rdf:resource="thsys/73376"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13661-5">
    <skos:broader rdf:resource="thsys/70825"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13661-5">
    <skos:broader rdf:resource="descriptor/15821-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13365-2">
    <skos:prefLabel xml:lang="en">Book publishing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13365-2">
    <skos:prefLabel xml:lang="de">Buchverlag</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16309-4">
    <skos:broader rdf:resource="thsys/73381"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16309-4">
    <skos:broader rdf:resource="thsys/73332"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16309-4">
    <skos:broader rdf:resource="descriptor/16188-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16309-4">
    <skos:broader rdf:resource="descriptor/16017-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19213-2">
    <skos:prefLabel xml:lang="en">Economics of law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19213-2">
    <skos:prefLabel xml:lang="de">Rechtsökonomik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12197-2">
    <skos:altLabel xml:lang="de">Stiftungsrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12197-2">
    <skos:altLabel xml:lang="de">Stiftung des privaten Rechts</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12197-2">
    <skos:altLabel xml:lang="de">Private Stiftung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12197-2">
    <skos:altLabel xml:lang="de">Gewerbliche Stiftung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70212">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15672-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16514-3">
    <skos:related rdf:resource="descriptor/16516-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16514-3">
    <skos:related rdf:resource="descriptor/13352-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10340-5">
    <skos:scopeNote xml:lang="de">Lage der Weltwirtschaft</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10028-0">
    <skos:related rdf:resource="descriptor/12320-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13343-5">
    <skos:broader rdf:resource="thsys/70879"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12509-2">
    <skos:altLabel xml:lang="de">Proportionale Kosten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10187-1">
    <skos:prefLabel xml:lang="en">Cost function</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10187-1">
    <skos:prefLabel xml:lang="de">Kostenfunktion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10957-6">
    <skos:prefLabel xml:lang="en">Merger control</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10957-6">
    <skos:prefLabel xml:lang="de">Fusionskontrolle</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15723-6">
    <skos:related rdf:resource="descriptor/15706-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15723-6">
    <skos:related rdf:resource="descriptor/16564-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15723-6">
    <skos:related rdf:resource="descriptor/16545-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15723-6">
    <skos:related rdf:resource="descriptor/15720-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15723-6">
    <skos:related rdf:resource="descriptor/15968-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15723-6">
    <skos:related rdf:resource="descriptor/15694-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15723-6">
    <skos:related rdf:resource="descriptor/16602-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15723-6">
    <skos:related rdf:resource="descriptor/16588-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12194-1">
    <skos:related rdf:resource="descriptor/12072-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12194-1">
    <skos:related rdf:resource="descriptor/12196-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12194-1">
    <skos:related rdf:resource="descriptor/12195-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12194-1">
    <skos:related rdf:resource="descriptor/16050-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12432-4">
    <skos:related rdf:resource="descriptor/18700-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12432-4">
    <skos:related rdf:resource="descriptor/12297-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12432-4">
    <skos:related rdf:resource="descriptor/12262-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12432-4">
    <skos:related rdf:resource="descriptor/12495-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12432-4">
    <skos:related rdf:resource="descriptor/12425-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70931">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15694-1">
    <skos:prefLabel xml:lang="en">Cultural history</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15694-1">
    <skos:prefLabel xml:lang="de">Kulturgeschichte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19226-0">
    <skos:prefLabel xml:lang="en">Monopolistic competition</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19226-0">
    <skos:prefLabel xml:lang="de">Monopolistischer Wettbewerb</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13881-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71066">
    <skos:broader rdf:resource="thsys/70058"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70423">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">P.11</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16327-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10481-1">
    <skos:altLabel xml:lang="de">Patentschutz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10481-1">
    <skos:altLabel xml:lang="de">Patentgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10481-1">
    <skos:altLabel xml:lang="de">Patentamt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10481-1">
    <skos:altLabel xml:lang="de">Lizenzrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10481-1">
    <skos:altLabel xml:lang="de">Erfindungsschutz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19387-4">
    <skos:related rdf:resource="descriptor/14193-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19387-4">
    <skos:related rdf:resource="descriptor/12984-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19387-4">
    <skos:related rdf:resource="descriptor/14197-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15555-1">
    <skos:prefLabel xml:lang="en">Control theory</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15555-1">
    <skos:prefLabel xml:lang="de">Kontrolltheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id182">
    <rdfs:seeAlso rdf:resource="descriptor/15797-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14161-6">
    <skos:prefLabel xml:lang="en">Marine product</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14161-6">
    <skos:prefLabel xml:lang="de">Meeresprodukt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18306-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10410-3">
    <skos:altLabel xml:lang="de">Zahlungsbilanzausgleich</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10410-3">
    <skos:altLabel xml:lang="de">Außenwirtschaftliches Gleichgewicht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70120">
    <rdfs:label xml:lang="de">Soziales Verhalten und soziale Gruppen</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13371-0">
    <skos:scopeNote xml:lang="de">Vgl. auch Gesetzliche Krankenversicherung und Gesundheitsreform.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13426-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15089-4">
    <skos:broader rdf:resource="thsys/72181"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15089-4">
    <skos:broader rdf:resource="thsys/70278"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15089-4">
    <skos:broader rdf:resource="descriptor/15102-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70459">
    <skos:broader rdf:resource="thsys/70041"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19686-1">
    <skos:broader rdf:resource="thsys/73381"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19686-1">
    <skos:broader rdf:resource="thsys/70039"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14711-4">
    <skos:altLabel xml:lang="de">PP (Polypropylen)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10464-1">
    <skos:altLabel xml:lang="en">High tech-industry</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10464-1">
    <skos:altLabel xml:lang="en">High tech</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10464-1">
    <skos:altLabel xml:lang="de">Spitzentechnologieindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10464-1">
    <skos:altLabel xml:lang="de">Spitzentechnologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10464-1">
    <skos:altLabel xml:lang="de">Hochtechnologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10464-1">
    <skos:altLabel xml:lang="de">High-Tech-Industrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18170-6">
    <skos:prefLabel xml:lang="en">Image processing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18170-6">
    <skos:prefLabel xml:lang="de">Bildverarbeitung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13152-5">
    <skos:altLabel xml:lang="en">Liquor industry</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13152-5">
    <skos:altLabel xml:lang="de">Weinbrennerei</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13152-5">
    <skos:altLabel xml:lang="de">Brennerei</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14983-6">
    <skos:altLabel xml:lang="de">Wodka</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14983-6">
    <skos:altLabel xml:lang="de">Whisky</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14983-6">
    <skos:altLabel xml:lang="de">Weinbrand</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14983-6">
    <skos:altLabel xml:lang="de">Soft-Spirituosen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14983-6">
    <skos:altLabel xml:lang="de">Rum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14983-6">
    <skos:altLabel xml:lang="de">Obstbrand</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14983-6">
    <skos:altLabel xml:lang="de">Likör</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14983-6">
    <skos:altLabel xml:lang="de">Gin</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14983-6">
    <skos:altLabel xml:lang="de">Cognac</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14983-6">
    <skos:altLabel xml:lang="de">Branntwein</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14983-6">
    <skos:altLabel xml:lang="de">Aquavit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16950-4">
    <skos:broader rdf:resource="thsys/70438"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16950-4">
    <skos:broader rdf:resource="descriptor/16825-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16950-4">
    <skos:broader rdf:resource="descriptor/17983-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25822-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15745-3">
    <skos:altLabel xml:lang="de">Naturressourcen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18805-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11828-5">
    <skos:narrower rdf:resource="descriptor/18630-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14479-3">
    <skos:related rdf:resource="descriptor/14355-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14479-3">
    <skos:related rdf:resource="descriptor/18359-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15067-0">
    <skos:related rdf:resource="descriptor/15225-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15067-0">
    <skos:related rdf:resource="descriptor/15057-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18376-0">
    <skos:related rdf:resource="descriptor/11675-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18376-0">
    <skos:related rdf:resource="descriptor/11811-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13148-3">
    <skos:broader rdf:resource="thsys/70987"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13148-3">
    <skos:broader rdf:resource="descriptor/13145-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71064">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11544-5">
    <skos:broader rdf:resource="thsys/71081"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11544-5">
    <skos:broader rdf:resource="thsys/71064"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11544-5">
    <skos:broader rdf:resource="descriptor/11545-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11544-5">
    <skos:broader rdf:resource="descriptor/11537-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16215-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19632-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11370-5">
    <skos:broader rdf:resource="thsys/70844"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11370-5">
    <skos:broader rdf:resource="descriptor/11373-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14331-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10517-6">
    <skos:broader rdf:resource="thsys/71086"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10517-6">
    <skos:broader rdf:resource="thsys/70406"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10517-6">
    <skos:broader rdf:resource="descriptor/10414-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10517-6">
    <skos:broader rdf:resource="descriptor/10524-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18909-5">
    <skos:prefLabel xml:lang="en">European social policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18909-5">
    <skos:prefLabel xml:lang="de">EU-Sozialpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16292-2">
    <skos:prefLabel xml:lang="en">Political system</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16292-2">
    <skos:prefLabel xml:lang="de">Politisches System</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13683-2">
    <skos:altLabel xml:lang="de">Sterntopologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13683-2">
    <skos:altLabel xml:lang="de">Ringtopologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13683-2">
    <skos:altLabel xml:lang="de">Netzwerkarchitektur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13683-2">
    <skos:altLabel xml:lang="de">Netztopologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13683-2">
    <skos:altLabel xml:lang="de">Netzstruktur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13683-2">
    <skos:altLabel xml:lang="de">Netzkonfiguration</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13683-2">
    <skos:altLabel xml:lang="de">Feldbussystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13683-2">
    <skos:altLabel xml:lang="de">Bustopologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13683-2">
    <skos:altLabel xml:lang="de">Bussystem in Computernetzen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18295-0">
    <skos:prefLabel xml:lang="en">Fashion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18295-0">
    <skos:prefLabel xml:lang="de">Mode</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16531-3">
    <skos:related rdf:resource="descriptor/16143-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16531-3">
    <skos:related rdf:resource="descriptor/16670-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16531-3">
    <skos:related rdf:resource="descriptor/11192-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16531-3">
    <skos:related rdf:resource="descriptor/16532-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16531-3">
    <skos:related rdf:resource="descriptor/11388-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16531-3">
    <skos:related rdf:resource="descriptor/11269-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14683-4">
    <skos:altLabel xml:lang="de">Xylol</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14683-4">
    <skos:altLabel xml:lang="de">Toluol</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14683-4">
    <skos:altLabel xml:lang="de">Fluorkohlenwasserstoffe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14683-4">
    <skos:altLabel xml:lang="de">FKW (Fluorkohlenwasserstoffe)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14683-4">
    <skos:altLabel xml:lang="de">Chlorierte Kohlenwasserstoffe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14683-4">
    <skos:altLabel xml:lang="de">BTX-Aromaten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14683-4">
    <skos:altLabel xml:lang="de">Benzol</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14683-4">
    <skos:altLabel xml:lang="de">Aromaten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17616-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19827-2">
    <skos:related rdf:resource="descriptor/10905-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19827-2">
    <skos:related rdf:resource="descriptor/10201-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19827-2">
    <skos:related rdf:resource="descriptor/12168-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19827-2">
    <skos:related rdf:resource="descriptor/12204-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19827-2">
    <skos:related rdf:resource="descriptor/19208-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13333-1">
    <skos:altLabel xml:lang="de">Müllabfuhr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13333-1">
    <skos:altLabel xml:lang="de">Kommunale Entsorgungswirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13333-1">
    <skos:altLabel xml:lang="de">Kommunale Abfallbeseitigung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13027-1">
    <skos:broader rdf:resource="thsys/71001"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13027-1">
    <skos:broader rdf:resource="descriptor/13029-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16513-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19475-0">
    <skos:scopeNote xml:lang="de">Branche, die alle Arten von Informationsdienstleistungen erbringt.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19405-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16037-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13674-3">
    <skos:altLabel xml:lang="en">ATM (Asynchronous Transfer Mode)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13674-3">
    <skos:altLabel xml:lang="en">Asynchronous transfer mode</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13674-3">
    <skos:altLabel xml:lang="de">Zweiweg-Kabelkommunikation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13674-3">
    <skos:altLabel xml:lang="de">Breitbandübertragung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13674-3">
    <skos:altLabel xml:lang="de">Breitbandkabel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13674-3">
    <skos:altLabel xml:lang="de">Asynchroner Transfermodus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18638-3">
    <skos:altLabel xml:lang="de">Städtereise</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11818-1">
    <skos:related rdf:resource="descriptor/16055-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11818-1">
    <skos:related rdf:resource="descriptor/11819-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11167-2">
    <skos:broader rdf:resource="thsys/70858"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11167-2">
    <skos:broader rdf:resource="thsys/70274"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11167-2">
    <skos:broader rdf:resource="descriptor/10975-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18245-1">
    <skos:altLabel xml:lang="de">Autonomer mobiler Roboter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16004-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13268-0">
    <skos:related rdf:resource="descriptor/18296-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18103-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id30">
    <rdfs:seeAlso rdf:resource="descriptor/10138-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14123-0">
    <skos:prefLabel xml:lang="en">Honey</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14123-0">
    <skos:prefLabel xml:lang="de">Honig</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10413-4">
    <skos:altLabel xml:lang="de">Nullwachstum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70418">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">P.05</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18166-4">
    <skos:related rdf:resource="descriptor/12628-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18166-4">
    <skos:related rdf:resource="descriptor/13488-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18953-2">
    <skos:broader rdf:resource="thsys/70819"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18953-2">
    <skos:broader rdf:resource="descriptor/15675-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19785-6">
    <skos:related rdf:resource="descriptor/14728-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19785-6">
    <skos:related rdf:resource="descriptor/13002-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19785-6">
    <skos:related rdf:resource="descriptor/15658-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19785-6">
    <skos:related rdf:resource="descriptor/18777-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14110-2">
    <skos:related rdf:resource="descriptor/13161-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14110-2">
    <skos:related rdf:resource="descriptor/14948-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14110-2">
    <skos:related rdf:resource="descriptor/14980-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14110-2">
    <skos:related rdf:resource="descriptor/19342-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14110-2">
    <skos:related rdf:resource="descriptor/12947-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14110-2">
    <skos:related rdf:resource="descriptor/13225-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11105-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18598-3">
    <skos:prefLabel xml:lang="en">Tanker shipping</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18598-3">
    <skos:prefLabel xml:lang="de">Tankschifffahrt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10685-6">
    <skos:prefLabel xml:lang="en">Quota (foreign trade)</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10685-6">
    <skos:prefLabel xml:lang="de">Außenhandelskontingent</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19470-3">
    <zbwext:useInsteadNote rdf:nodeID="id167"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15964-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13181-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12590-0">
    <skos:prefLabel xml:lang="en">Piece rate</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12590-0">
    <skos:prefLabel xml:lang="de">Akkordlohn</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18300-5">
    <skos:altLabel xml:lang="de">Nicht-Lebensmittel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16053-6">
    <skos:narrower rdf:resource="descriptor/10481-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16053-6">
    <skos:narrower rdf:resource="descriptor/16078-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16053-6">
    <skos:narrower rdf:resource="descriptor/18065-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18582-4">
    <skos:broader rdf:resource="thsys/70937"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18582-4">
    <skos:broader rdf:resource="descriptor/18615-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10644-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10613-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15961-4">
    <skos:altLabel xml:lang="en">Freedom of information</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15961-4">
    <skos:altLabel xml:lang="de">Informationsfreiheit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73349">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19311-2">
    <skos:altLabel xml:lang="en">Pass-through</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19311-2">
    <skos:altLabel xml:lang="en">Exchange rate pass through</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19311-2">
    <skos:altLabel xml:lang="en">ERPT (Exchange rate pass-through)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19520-0">
    <skos:scopeNote xml:lang="de">Nur für historische Untersuchungen überseeischer Handelsgesellschaften.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14044-3">
    <skos:related rdf:resource="descriptor/14633-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14044-3">
    <skos:related rdf:resource="descriptor/14656-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14044-3">
    <skos:related rdf:resource="descriptor/19393-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14044-3">
    <skos:related rdf:resource="descriptor/14197-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11735-5">
    <skos:prefLabel xml:lang="en">Income distribution</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11735-5">
    <skos:prefLabel xml:lang="de">Einkommensverteilung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10182-4">
    <skos:broader rdf:resource="thsys/70493"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10182-4">
    <skos:broader rdf:resource="descriptor/10181-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17780-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14359-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16136-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14607-6">
    <skos:related rdf:resource="descriptor/14024-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14607-6">
    <skos:related rdf:resource="descriptor/14021-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18004-2">
    <skos:broader rdf:resource="thsys/73305"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18004-2">
    <skos:broader rdf:resource="thsys/71038"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18004-2">
    <skos:broader rdf:resource="descriptor/14751-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70853">
    <rdfs:label xml:lang="de">Interessenvertretungen</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11567-0">
    <skos:related rdf:resource="descriptor/18811-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11567-0">
    <skos:related rdf:resource="descriptor/10583-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11567-0">
    <skos:related rdf:resource="descriptor/14931-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70070">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">P.01.01.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10527-3">
    <skos:altLabel xml:lang="en">Impoverishing growth</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10527-3">
    <skos:altLabel xml:lang="de">Verelendungswachstum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10527-3">
    <skos:altLabel xml:lang="de">Verarmungswachstum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16674-2">
    <skos:prefLabel xml:lang="en">Social consequences</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16674-2">
    <skos:prefLabel xml:lang="de">Soziale Folgen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18750-2">
    <skos:related rdf:resource="descriptor/12257-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18750-2">
    <skos:related rdf:resource="descriptor/12030-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18750-2">
    <skos:related rdf:resource="descriptor/12029-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18750-2">
    <skos:related rdf:resource="descriptor/13775-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18750-2">
    <skos:related rdf:resource="descriptor/13851-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18750-2">
    <skos:related rdf:resource="descriptor/18505-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18808-4">
    <skos:broader rdf:resource="thsys/73380"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18808-4">
    <skos:broader rdf:resource="thsys/73367"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18808-4">
    <skos:broader rdf:resource="thsys/70870"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18808-4">
    <skos:broader rdf:resource="descriptor/13413-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70982">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12848-1">
    <skos:prefLabel xml:lang="en">Company tax</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12848-1">
    <skos:prefLabel xml:lang="de">Betriebswirtschaftliche Steuerlehre</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12807-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70929">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12249-2">
    <skos:altLabel xml:lang="de">Finanzierungskosten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20399-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13683-2">
    <skos:scopeNote xml:lang="de">Architektur von Telekommunikations- und Computernetzen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15045-3">
    <skos:altLabel xml:lang="de">Mathematische Methode</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13078-5">
    <skos:altLabel xml:lang="de">Feintechnische Industrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11753-3">
    <skos:narrower rdf:resource="descriptor/11742-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11753-3">
    <skos:narrower rdf:resource="descriptor/11738-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70012">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id184">
    <rdfs:seeAlso rdf:resource="descriptor/11839-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14427-1">
    <skos:altLabel xml:lang="de">Transrapid</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18491-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18029-0">
    <skos:narrower rdf:resource="descriptor/19565-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18029-0">
    <skos:narrower rdf:resource="descriptor/19548-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13045-6">
    <skos:prefLabel xml:lang="en">Construction machinery industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13045-6">
    <skos:prefLabel xml:lang="de">Baumaschinenindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16477-4">
    <skos:broader rdf:resource="thsys/73370"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16477-4">
    <skos:broader rdf:resource="thsys/70833"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16477-4">
    <skos:broader rdf:resource="thsys/70814"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16477-4">
    <skos:broader rdf:resource="thsys/70324"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16477-4">
    <skos:broader rdf:resource="descriptor/16545-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14549-1">
    <skos:altLabel xml:lang="de">Weißblechverpackung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14549-1">
    <skos:altLabel xml:lang="de">Metallfolienverpackung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14549-1">
    <skos:altLabel xml:lang="de">Feinblechverpackung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14549-1">
    <skos:altLabel xml:lang="de">Dose</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14549-1">
    <skos:altLabel xml:lang="de">Blechdose</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16990-6">
    <skos:prefLabel xml:lang="en">Northern Europe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16990-6">
    <skos:prefLabel xml:lang="de">Nordeuropa</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12180-5">
    <skos:narrower rdf:resource="descriptor/18503-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12180-5">
    <skos:narrower rdf:resource="descriptor/18843-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12180-5">
    <skos:narrower rdf:resource="descriptor/18842-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12180-5">
    <skos:narrower rdf:resource="descriptor/13287-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12180-5">
    <skos:narrower rdf:resource="descriptor/18510-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12180-5">
    <skos:narrower rdf:resource="descriptor/13456-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12180-5">
    <skos:narrower rdf:resource="descriptor/13429-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73335">
    <rdfs:label xml:lang="en">Social legislation</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73335">
    <rdfs:label xml:lang="de">Sozialrecht</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16824-2">
    <skos:prefLabel xml:lang="en">Liechtenstein</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16824-2">
    <skos:prefLabel xml:lang="de">Liechtenstein</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16502-3">
    <skos:prefLabel xml:lang="en">Geopolitics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16502-3">
    <skos:prefLabel xml:lang="de">Geopolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15101-5">
    <skos:prefLabel xml:lang="en">Construction statistics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15101-5">
    <skos:prefLabel xml:lang="de">Baustatistik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19859-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13401-3">
    <skos:prefLabel xml:lang="en">Consumers' association</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13401-3">
    <skos:prefLabel xml:lang="de">Verbraucherorganisation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19070-5">
    <skos:broader rdf:resource="thsys/73393"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19070-5">
    <skos:broader rdf:resource="thsys/72182"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19070-5">
    <skos:broader rdf:resource="thsys/71035"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19070-5">
    <skos:broader rdf:resource="descriptor/16727-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13638-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14377-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18628-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18427-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18327-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14118-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19601-0">
    <skos:narrower rdf:resource="descriptor/15793-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19601-0">
    <skos:narrower rdf:resource="descriptor/15656-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18362-4">
    <skos:narrower rdf:resource="descriptor/18363-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18362-4">
    <skos:narrower rdf:resource="descriptor/20888-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16842-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13643-0">
    <skos:broader rdf:resource="thsys/70924"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13643-0">
    <skos:broader rdf:resource="descriptor/13644-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11448-1">
    <skos:altLabel xml:lang="de">Geldvolumen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18082-3">
    <skos:related rdf:resource="descriptor/10472-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18082-3">
    <skos:related rdf:resource="descriptor/10466-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18082-3">
    <skos:related rdf:resource="descriptor/10945-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18082-3">
    <skos:related rdf:resource="descriptor/19251-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18082-3">
    <skos:related rdf:resource="descriptor/19548-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18082-3">
    <skos:related rdf:resource="descriptor/10459-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18082-3">
    <skos:related rdf:resource="descriptor/10142-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18082-3">
    <skos:related rdf:resource="descriptor/10947-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18082-3">
    <skos:related rdf:resource="descriptor/12711-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16389-1">
    <skos:altLabel xml:lang="de">Nationale Befreiungsbewegung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16389-1">
    <skos:altLabel xml:lang="de">Befreiungsfront</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16389-1">
    <skos:altLabel xml:lang="de">Befreiungsbewegung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18696-3">
    <skos:prefLabel xml:lang="en">Credit theory</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18696-3">
    <skos:prefLabel xml:lang="de">Kredittheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13157-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16365-1">
    <skos:broader rdf:resource="thsys/73378"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16365-1">
    <skos:broader rdf:resource="descriptor/25742-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14725-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14088-4">
    <skos:related rdf:resource="descriptor/19293-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14088-4">
    <skos:related rdf:resource="descriptor/19188-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14088-4">
    <skos:related rdf:resource="descriptor/18517-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14088-4">
    <skos:related rdf:resource="descriptor/18198-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14088-4">
    <skos:related rdf:resource="descriptor/14945-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13158-0">
    <skos:related rdf:resource="descriptor/12946-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13158-0">
    <skos:related rdf:resource="descriptor/14955-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13158-0">
    <skos:related rdf:resource="descriptor/19183-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13158-0">
    <skos:related rdf:resource="descriptor/14124-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15902-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10035-3">
    <skos:narrower rdf:resource="descriptor/10045-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19059-0">
    <skos:related rdf:resource="descriptor/14437-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15757-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15932-4">
    <skos:broader rdf:resource="thsys/73358"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15932-4">
    <skos:broader rdf:resource="descriptor/15931-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15932-4">
    <skos:broader rdf:resource="descriptor/15713-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18888-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13426-1">
    <skos:broader rdf:resource="thsys/73390"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13426-1">
    <skos:broader rdf:resource="thsys/71028"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13426-1">
    <skos:broader rdf:resource="descriptor/16081-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12940-6">
    <skos:related rdf:resource="descriptor/14211-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12940-6">
    <skos:related rdf:resource="descriptor/14104-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12940-6">
    <skos:related rdf:resource="descriptor/14030-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70647">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18585-5">
    <skos:related rdf:resource="descriptor/18580-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18585-5">
    <skos:related rdf:resource="descriptor/18579-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18585-5">
    <skos:related rdf:resource="descriptor/18296-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18585-5">
    <skos:related rdf:resource="descriptor/13558-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18585-5">
    <skos:related rdf:resource="descriptor/18421-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18585-5">
    <skos:related rdf:resource="descriptor/18423-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18549-2">
    <skos:altLabel xml:lang="en">Warehouse club</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18549-2">
    <skos:altLabel xml:lang="en">Factory outlet center</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18549-2">
    <skos:altLabel xml:lang="de">Fabrikladen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12624-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18410-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16475-1">
    <skos:broader rdf:resource="thsys/73370"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16475-1">
    <skos:broader rdf:resource="thsys/70833"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16475-1">
    <skos:broader rdf:resource="thsys/70324"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16475-1">
    <skos:broader rdf:resource="descriptor/16545-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13307-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14364-6">
    <skos:broader rdf:resource="thsys/71015"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14364-6">
    <skos:broader rdf:resource="thsys/71014"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14364-6">
    <skos:broader rdf:resource="thsys/70424"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14364-6">
    <skos:broader rdf:resource="descriptor/14039-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14364-6">
    <skos:broader rdf:resource="descriptor/14355-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16003-0">
    <skos:prefLabel xml:lang="en">Martial law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16003-0">
    <skos:prefLabel xml:lang="de">Kriegsrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24704-2">
    <skos:scopeNote xml:lang="de">Gesamtheit der Aktivitäten zur Erreichung des Unternehmensziels oder von Teilzielen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id268">
    <rdfs:seeAlso rdf:resource="descriptor/18842-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14686-5">
    <skos:broader rdf:resource="thsys/70556"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14686-5">
    <skos:broader rdf:resource="descriptor/14656-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70310">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12511-1">
    <skos:prefLabel xml:lang="en">Administrative costs</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12511-1">
    <skos:prefLabel xml:lang="de">Verwaltungskosten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10268-1">
    <skos:prefLabel xml:lang="en">Disposable income</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10268-1">
    <skos:prefLabel xml:lang="de">Verfügbares Einkommen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10525-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16355-4">
    <skos:prefLabel xml:lang="en">Election campaign</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16355-4">
    <skos:prefLabel xml:lang="de">Wahlkampf</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10614-1">
    <skos:narrower rdf:resource="descriptor/19308-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18739-4">
    <skos:narrower rdf:resource="descriptor/24786-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19063-2">
    <skos:prefLabel xml:lang="en">Medical device trade</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19063-2">
    <skos:prefLabel xml:lang="de">Sanitätshandel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14928-0">
    <skos:prefLabel xml:lang="en">Yeast</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14928-0">
    <skos:prefLabel xml:lang="de">Hefe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12514-2">
    <skos:broader rdf:resource="thsys/70578"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12514-2">
    <skos:broader rdf:resource="descriptor/12480-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12177-1">
    <skos:broader rdf:resource="thsys/71040"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12177-1">
    <skos:broader rdf:resource="thsys/70087"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12177-1">
    <skos:broader rdf:resource="descriptor/12194-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14418-2">
    <skos:broader rdf:resource="thsys/70256"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14418-2">
    <skos:broader rdf:resource="descriptor/14421-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18924-2">
    <skos:scopeNote xml:lang="de">Charakteristische Verhaltensweisen von Angehörigen eines bestimmten Kulturkreises - Bräuche, Normen, Konventionen, Mentalitäten.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18149-4">
    <skos:related rdf:resource="descriptor/11434-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18149-4">
    <skos:related rdf:resource="descriptor/10242-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18149-4">
    <skos:related rdf:resource="descriptor/19371-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18149-4">
    <skos:related rdf:resource="descriptor/11424-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70888">
    <skos:prefLabel xml:lang="en">W.17.01  Housing industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70888">
    <skos:prefLabel xml:lang="de">W.17.01  Wohnungswirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16041-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16229-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11718-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18287-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12523-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11635-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17700-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18090-4">
    <skos:broader rdf:resource="thsys/70943"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18090-4">
    <skos:broader rdf:resource="descriptor/12013-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#member">
    <rdfs:range rdf:nodeID="r1238766808r28278r1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17199-1">
    <skos:altLabel xml:lang="de">Westeuropäische Staaten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17199-1">
    <skos:altLabel xml:lang="de">Westeuropäische Länder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13985-0">
    <skos:prefLabel xml:lang="en">Office workers</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13985-0">
    <skos:prefLabel xml:lang="de">Büroberufe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73339">
    <skos:prefLabel xml:lang="en">N.10.07.03  Computer science</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73339">
    <skos:prefLabel xml:lang="de">N.10.07.03  Informatik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18336-5">
    <skos:broader rdf:resource="thsys/73351"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18336-5">
    <skos:broader rdf:resource="thsys/71019"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18336-5">
    <skos:broader rdf:resource="thsys/70239"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18336-5">
    <skos:broader rdf:resource="descriptor/15746-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18854-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13361-3">
    <skos:altLabel xml:lang="de">Wissenschaftliche Fachbibliothek</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11215-3">
    <skos:related rdf:resource="descriptor/13645-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11215-3">
    <skos:related rdf:resource="descriptor/11192-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18661-1">
    <zbwext:useInsteadNote rdf:nodeID="id273"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11207-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14534-0">
    <zbwext:useInsteadNote rdf:nodeID="id172"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16215-6">
    <skos:broader rdf:resource="thsys/73366"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16215-6">
    <skos:broader rdf:resource="thsys/73354"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16215-6">
    <skos:broader rdf:resource="thsys/70878"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16215-6">
    <skos:broader rdf:resource="descriptor/10436-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16611-5">
    <skos:broader rdf:resource="thsys/73395"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16611-5">
    <skos:broader rdf:resource="descriptor/15717-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13780-4">
    <skos:related rdf:resource="descriptor/13761-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18220-3">
    <skos:prefLabel xml:lang="en">Beef</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18220-3">
    <skos:prefLabel xml:lang="de">Rindfleisch</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18525-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19366-5">
    <skos:broader rdf:resource="thsys/71019"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19366-5">
    <skos:broader rdf:resource="thsys/70989"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19366-5">
    <skos:broader rdf:resource="thsys/70960"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19366-5">
    <skos:broader rdf:resource="descriptor/10012-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12088-0">
    <skos:narrower rdf:resource="descriptor/12083-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12088-0">
    <skos:narrower rdf:resource="descriptor/12876-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12088-0">
    <skos:narrower rdf:resource="descriptor/20277-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12088-0">
    <skos:narrower rdf:resource="descriptor/12758-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12088-0">
    <skos:narrower rdf:resource="descriptor/12565-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12088-0">
    <skos:narrower rdf:resource="descriptor/12651-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19056-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19274-3">
    <skos:broader rdf:resource="thsys/70187"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19634-6">
    <skos:related rdf:resource="descriptor/15699-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19634-6">
    <skos:related rdf:resource="descriptor/19781-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19634-6">
    <skos:related rdf:resource="descriptor/15694-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19634-6">
    <skos:related rdf:resource="descriptor/15689-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70002">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13406-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70074">
    <skos:narrower rdf:resource="descriptor/18267-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70074">
    <skos:narrower rdf:resource="descriptor/14630-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70074">
    <skos:narrower rdf:resource="descriptor/14740-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70074">
    <skos:narrower rdf:resource="descriptor/18215-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70074">
    <skos:narrower rdf:resource="descriptor/14602-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70074">
    <skos:narrower rdf:resource="descriptor/14217-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70074">
    <skos:narrower rdf:resource="descriptor/14617-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70074">
    <skos:narrower rdf:resource="descriptor/18216-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70074">
    <skos:narrower rdf:resource="descriptor/14611-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70074">
    <skos:narrower rdf:resource="descriptor/14610-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70074">
    <skos:narrower rdf:resource="descriptor/14327-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13605-1">
    <skos:broader rdf:resource="thsys/70939"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13605-1">
    <skos:broader rdf:resource="descriptor/11834-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10570-2">
    <skos:altLabel xml:lang="de">Importsubstitutionsstrategie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10515-3">
    <skos:altLabel xml:lang="de">Dominationstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10515-3">
    <skos:altLabel xml:lang="de">Dissoziationsstrategie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10515-3">
    <skos:altLabel xml:lang="de">Abkoppelungsstrategie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10515-3">
    <skos:altLabel xml:lang="de">Abhängigkeitstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10515-3">
    <skos:altLabel xml:lang="de">Abhängigkeit (Entwicklungsland)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16540-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13084-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13518-3">
    <skos:prefLabel xml:lang="en">Air freight transport</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13518-3">
    <skos:prefLabel xml:lang="de">Luftfrachtverkehr</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10524-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20289-1">
    <zbwext:useInsteadNote rdf:nodeID="id282"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12025-3">
    <skos:broader rdf:resource="thsys/71055"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12025-3">
    <skos:broader rdf:resource="thsys/70889"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12025-3">
    <skos:broader rdf:resource="descriptor/13967-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12025-3">
    <skos:broader rdf:resource="descriptor/18747-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19491-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id340">
    <rdfs:seeAlso rdf:resource="descriptor/19428-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11264-4">
    <skos:prefLabel xml:lang="en">Labour statistics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11264-4">
    <skos:prefLabel xml:lang="de">Arbeitsmarktstatistik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11033-2">
    <skos:prefLabel xml:lang="en">Central planning</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11033-2">
    <skos:prefLabel xml:lang="de">Zentrale Wirtschaftsplanung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14928-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16914-1">
    <skos:broader rdf:resource="thsys/70361"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16914-1">
    <skos:broader rdf:resource="descriptor/18012-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14214-4">
    <skos:related rdf:resource="descriptor/14350-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14214-4">
    <skos:related rdf:resource="descriptor/14212-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18260-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12863-5">
    <skos:altLabel xml:lang="de">Organträger</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12863-5">
    <skos:altLabel xml:lang="de">Organtheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10224-0">
    <skos:broader rdf:resource="thsys/71119"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10224-0">
    <skos:broader rdf:resource="thsys/70242"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10224-0">
    <skos:broader rdf:resource="descriptor/10956-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10224-0">
    <skos:broader rdf:resource="descriptor/10905-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18747-5">
    <skos:broader rdf:resource="thsys/70889"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18389-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10190-5">
    <skos:related rdf:resource="descriptor/12072-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10190-5">
    <skos:related rdf:resource="descriptor/11160-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10190-5">
    <skos:related rdf:resource="descriptor/10891-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10190-5">
    <skos:related rdf:resource="descriptor/10256-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10190-5">
    <skos:related rdf:resource="descriptor/10185-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10190-5">
    <skos:related rdf:resource="descriptor/12041-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10190-5">
    <skos:related rdf:resource="descriptor/10192-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10190-5">
    <skos:related rdf:resource="descriptor/12744-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10190-5">
    <skos:related rdf:resource="descriptor/11153-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19555-2">
    <skos:broader rdf:resource="thsys/70040"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19555-2">
    <skos:broader rdf:resource="descriptor/12794-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15599-2">
    <skos:narrower rdf:resource="descriptor/19144-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15599-2">
    <skos:narrower rdf:resource="descriptor/18664-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15599-2">
    <skos:narrower rdf:resource="descriptor/18657-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15599-2">
    <skos:narrower rdf:resource="descriptor/13269-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15599-2">
    <skos:narrower rdf:resource="descriptor/18659-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15599-2">
    <skos:narrower rdf:resource="descriptor/18175-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15599-2">
    <skos:narrower rdf:resource="descriptor/18658-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15599-2">
    <skos:narrower rdf:resource="descriptor/16802-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13011-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18844-0">
    <skos:broader rdf:resource="thsys/70884"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18844-0">
    <skos:broader rdf:resource="thsys/70853"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18844-0">
    <skos:broader rdf:resource="thsys/70814"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18844-0">
    <skos:broader rdf:resource="descriptor/13401-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18690-1">
    <zbwext:useInsteadNote rdf:nodeID="id286"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19557-5">
    <skos:broader rdf:resource="thsys/70037"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19557-5">
    <skos:broader rdf:resource="descriptor/25742-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14504-2">
    <skos:narrower rdf:resource="descriptor/14510-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18400-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11694-0">
    <skos:prefLabel xml:lang="en">Theory of taxation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11694-0">
    <skos:prefLabel xml:lang="de">Steuertheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id134">
    <rdfs:seeAlso rdf:resource="descriptor/13508-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12047-0">
    <skos:broader rdf:resource="thsys/70856"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12047-0">
    <skos:broader rdf:resource="thsys/70087"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12047-0">
    <skos:broader rdf:resource="descriptor/12091-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10015-2">
    <skos:altLabel xml:lang="de">Wirtschaftliche Nutzungsdauer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10015-2">
    <skos:altLabel xml:lang="de">Technische Nutzungsdauer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10015-2">
    <skos:altLabel xml:lang="de">Optimaler Ersatzzeitpunkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10015-2">
    <skos:altLabel xml:lang="de">Optimale Nutzungsdauer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10015-2">
    <skos:altLabel xml:lang="de">Lebensdauer von Anlagen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10015-2">
    <skos:altLabel xml:lang="de">Geplanter Verschleiß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10192-1">
    <skos:prefLabel xml:lang="en">Corporate objective</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10192-1">
    <skos:prefLabel xml:lang="de">Betriebswirtschaftliches Ziel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10978-5">
    <skos:scopeNote xml:lang="de">Auch für Preiskartelle rohstoffexportierender Staaten.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10617-2">
    <skos:altLabel xml:lang="de">Wirtschaftsbeziehungen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10617-2">
    <skos:altLabel xml:lang="de">Internationale Wirtschaftsverflechtung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10617-2">
    <skos:altLabel xml:lang="de">Internationale wirtschaftliche Zusammenarbeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10617-2">
    <skos:altLabel xml:lang="de">Außenwirtschaftsbeziehung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10617-2">
    <skos:altLabel xml:lang="de">Außenhandelsverflechtung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17740-3">
    <skos:broader rdf:resource="thsys/71031"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17740-3">
    <skos:broader rdf:resource="descriptor/17743-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12028-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13828-2">
    <skos:related rdf:resource="descriptor/13821-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13828-2">
    <skos:related rdf:resource="descriptor/19424-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13828-2">
    <skos:related rdf:resource="descriptor/13825-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13828-2">
    <skos:related rdf:resource="descriptor/19679-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13828-2">
    <skos:related rdf:resource="descriptor/13969-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13828-2">
    <skos:related rdf:resource="descriptor/13818-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13828-2">
    <skos:related rdf:resource="descriptor/13296-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13828-2">
    <skos:related rdf:resource="descriptor/11414-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12224-4">
    <skos:prefLabel xml:lang="en">Corporate investment decision</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12224-4">
    <skos:prefLabel xml:lang="de">Betriebliche Investitionspolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12009-1">
    <skos:scopeNote xml:lang="de">Verfassungsmäßige, rechtliche und kulturspezifische Gegebenheiten, in deren Rahmen sich das gesellschaftliche und wirtschaftliche Leben gestaltet.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10968-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18746-0">
    <skos:broader rdf:resource="thsys/70889"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19491-2">
    <skos:altLabel xml:lang="en">Non-standard employment</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19491-2">
    <skos:altLabel xml:lang="en">Non-standard employees</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19491-2">
    <skos:altLabel xml:lang="en">Contingent employment</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19491-2">
    <skos:altLabel xml:lang="en">Casual work</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19491-2">
    <skos:altLabel xml:lang="en">Casual employment</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19491-2">
    <skos:altLabel xml:lang="de">Minijob</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19491-2">
    <skos:altLabel xml:lang="de">Geringfügige Beschäftigung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19491-2">
    <skos:altLabel xml:lang="de">Gelegenheitsarbeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19491-2">
    <skos:altLabel xml:lang="de">Aushilfsarbeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19491-2">
    <skos:altLabel xml:lang="de">Atypische Beschäftigung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13095-5">
    <skos:related rdf:resource="descriptor/13083-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13095-5">
    <skos:related rdf:resource="descriptor/13231-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13095-5">
    <skos:related rdf:resource="descriptor/14593-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13095-5">
    <skos:related rdf:resource="descriptor/14227-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13095-5">
    <skos:related rdf:resource="descriptor/14296-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19487-0">
    <skos:broader rdf:resource="thsys/71016"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19487-0">
    <skos:broader rdf:resource="thsys/70227"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19487-0">
    <skos:broader rdf:resource="descriptor/19490-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19487-0">
    <skos:broader rdf:resource="descriptor/18129-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12670-2">
    <skos:prefLabel xml:lang="en">Logistics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12670-2">
    <skos:prefLabel xml:lang="de">Logistik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15954-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18054-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id96">
    <rdf:value xml:lang="de">Für die Politik der Europäischen Union und ihrer Organe</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19188-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19513-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19117-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15712-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11534-1">
    <skos:prefLabel xml:lang="en">Government finance statistics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11534-1">
    <skos:prefLabel xml:lang="de">Finanzstatistik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14047-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11771-1">
    <skos:altLabel xml:lang="en">Eminent domain proceedings</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11771-1">
    <skos:altLabel xml:lang="en">Condemnation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11771-1">
    <skos:altLabel xml:lang="de">Vermögenskontrolle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16522-4">
    <skos:altLabel xml:lang="en">Conscientious objection</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16522-4">
    <skos:altLabel xml:lang="en">Compulsory military service</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16522-4">
    <skos:altLabel xml:lang="de">Wehrpflicht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16522-4">
    <skos:altLabel xml:lang="de">Wehrersatzwesen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16522-4">
    <skos:altLabel xml:lang="de">Wehrdienstverweigerung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16522-4">
    <skos:altLabel xml:lang="de">Reserveübung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16522-4">
    <skos:altLabel xml:lang="de">Rekrutierung militärischer Einheiten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16522-4">
    <skos:altLabel xml:lang="de">Recht auf Kriegsdienstverweigerung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16522-4">
    <skos:altLabel xml:lang="de">Mobilisierung militärischer Einheiten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16522-4">
    <skos:altLabel xml:lang="de">Militärische Dienstpflicht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16522-4">
    <skos:altLabel xml:lang="de">Militärdienst</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16522-4">
    <skos:altLabel xml:lang="de">Kriegsdienstverweigerung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16522-4">
    <skos:altLabel xml:lang="de">Kriegsdienst</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16522-4">
    <skos:altLabel xml:lang="de">Grundwehrdienst</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16522-4">
    <skos:altLabel xml:lang="de">Gesetz über den Schutz des Arbeitsplatzes bei Einberufung zum Wehrdienst</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16522-4">
    <skos:altLabel xml:lang="de">Berufsheer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16522-4">
    <skos:altLabel xml:lang="de">Berufsarmee</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73349">
    <skos:prefLabel xml:lang="en">N.10.03  Chemistry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73349">
    <skos:prefLabel xml:lang="de">N.10.03  Chemie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19724-5">
    <skos:prefLabel xml:lang="en">Military logistics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19724-5">
    <skos:prefLabel xml:lang="de">Militärische Logistik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18065-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70257">
    <skos:narrower rdf:resource="descriptor/13513-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70257">
    <skos:narrower rdf:resource="descriptor/14413-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70257">
    <skos:narrower rdf:resource="descriptor/14453-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70257">
    <skos:narrower rdf:resource="descriptor/14431-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70257">
    <skos:narrower rdf:resource="thsys/70256"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70257">
    <skos:narrower rdf:resource="thsys/70258"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70257">
    <skos:narrower rdf:resource="thsys/70415"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13655-0">
    <skos:narrower rdf:resource="descriptor/18656-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13655-0">
    <skos:narrower rdf:resource="descriptor/13656-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13655-0">
    <skos:narrower rdf:resource="descriptor/19658-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19830-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10406-1">
    <skos:prefLabel xml:lang="en">Full employment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10406-1">
    <skos:prefLabel xml:lang="de">Vollbeschäftigung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19749-3">
    <skos:related rdf:resource="descriptor/16487-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19749-3">
    <skos:related rdf:resource="descriptor/16515-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19749-3">
    <skos:related rdf:resource="descriptor/19251-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72185">
    <rdfs:label xml:lang="en">Probability theory</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72185">
    <rdfs:label xml:lang="de">Wahrscheinlichkeitsrechnung</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13469-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10418-1">
    <skos:prefLabel xml:lang="en">Harrod-Domar growth model</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10418-1">
    <skos:prefLabel xml:lang="de">Harrod-Domar-Wachstumstheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16131-5">
    <skos:altLabel xml:lang="en">Labor contract</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16131-5">
    <skos:altLabel xml:lang="de">Gruppenarbeitsvertrag</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16131-5">
    <skos:altLabel xml:lang="de">Dienstvertrag</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16131-5">
    <skos:altLabel xml:lang="de">Arbeitsvertragsrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15895-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16306-3">
    <skos:related rdf:resource="descriptor/16354-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16306-3">
    <skos:related rdf:resource="descriptor/16329-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70828">
    <skos:narrower rdf:resource="descriptor/12842-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70828">
    <skos:narrower rdf:resource="descriptor/13364-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70828">
    <skos:narrower rdf:resource="descriptor/18099-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70828">
    <skos:narrower rdf:resource="descriptor/18448-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70828">
    <skos:narrower rdf:resource="descriptor/15661-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70828">
    <skos:narrower rdf:resource="descriptor/13357-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70828">
    <skos:narrower rdf:resource="descriptor/18295-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70828">
    <skos:narrower rdf:resource="descriptor/15657-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70828">
    <skos:narrower rdf:resource="descriptor/13355-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70828">
    <skos:narrower rdf:resource="descriptor/18927-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70828">
    <skos:narrower rdf:resource="descriptor/18926-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70828">
    <skos:narrower rdf:resource="descriptor/15655-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70828">
    <skos:narrower rdf:resource="descriptor/18920-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70828">
    <skos:narrower rdf:resource="descriptor/16515-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70828">
    <skos:narrower rdf:resource="descriptor/18921-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70828">
    <skos:narrower rdf:resource="descriptor/18923-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70828">
    <skos:narrower rdf:resource="descriptor/18922-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70828">
    <skos:narrower rdf:resource="descriptor/16648-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70828">
    <skos:narrower rdf:resource="descriptor/15654-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70828">
    <skos:narrower rdf:resource="descriptor/18919-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70828">
    <skos:narrower rdf:resource="descriptor/13352-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70828">
    <skos:narrower rdf:resource="descriptor/16514-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70828">
    <skos:narrower rdf:resource="descriptor/18492-5"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id142">
    <rdfs:seeAlso rdf:resource="descriptor/11981-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10604-4">
    <skos:altLabel xml:lang="de">Markt im Ausland</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10604-4">
    <skos:altLabel xml:lang="de">Exportmarkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10604-4">
    <skos:altLabel xml:lang="de">Absatzmarkt im Ausland</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10512-2">
    <skos:broader rdf:resource="thsys/71064"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10512-2">
    <skos:broader rdf:resource="thsys/70091"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16544-1">
    <skos:altLabel xml:lang="de">Sozialgesinnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14942-6">
    <skos:prefLabel xml:lang="en">Dry baked goods</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14942-6">
    <skos:prefLabel xml:lang="de">Dauerbackwaren</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73341">
    <skos:prefLabel xml:lang="de">N.10.07.01  Informationstechnik und Systemarchitektur</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70113">
    <skos:narrower rdf:resource="descriptor/12567-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70113">
    <skos:narrower rdf:resource="descriptor/12564-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70113">
    <skos:narrower rdf:resource="descriptor/20381-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70113">
    <skos:narrower rdf:resource="descriptor/12879-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70113">
    <skos:narrower rdf:resource="descriptor/11268-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70113">
    <skos:narrower rdf:resource="descriptor/11262-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70113">
    <skos:narrower rdf:resource="descriptor/11305-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70113">
    <skos:narrower rdf:resource="descriptor/11180-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70113">
    <skos:narrower rdf:resource="thsys/71051"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70113">
    <skos:narrower rdf:resource="thsys/70245"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70113">
    <skos:narrower rdf:resource="thsys/70244"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70113">
    <skos:narrower rdf:resource="thsys/71050"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18310-2">
    <skos:related rdf:resource="descriptor/11782-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18310-2">
    <skos:related rdf:resource="descriptor/10981-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70046">
    <rdfs:label xml:lang="de">Betriebliches Steuerwesen</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70187">
    <skos:broader rdf:resource="thsys/70055"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10545-1">
    <skos:altLabel xml:lang="de">Stützungsvorrat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10545-1">
    <skos:altLabel xml:lang="de">Pflichtlagerhaltung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10545-1">
    <skos:altLabel xml:lang="de">Ausgleichslager für Rohstoffe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18215-3">
    <skos:related rdf:resource="descriptor/14934-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18215-3">
    <skos:related rdf:resource="descriptor/14996-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70465">
    <skos:broader rdf:resource="thsys/p"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16566-5">
    <skos:altLabel xml:lang="de">Wirtschaftsstil</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16566-5">
    <skos:altLabel xml:lang="de">Wirtschaftskultur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16566-5">
    <skos:altLabel xml:lang="de">Wirtschaftsanthropologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14792-6">
    <skos:prefLabel xml:lang="en">Braided goods</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14792-6">
    <skos:prefLabel xml:lang="de">Flechtwaren</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17503-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11648-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13337-0">
    <skos:broader rdf:resource="thsys/70818"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13337-0">
    <skos:broader rdf:resource="descriptor/15727-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70361">
    <skos:narrower rdf:resource="descriptor/18136-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70361">
    <skos:narrower rdf:resource="descriptor/19076-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70361">
    <skos:narrower rdf:resource="descriptor/16914-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70361">
    <skos:narrower rdf:resource="descriptor/18135-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70361">
    <skos:narrower rdf:resource="descriptor/18134-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70361">
    <skos:narrower rdf:resource="descriptor/16912-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70361">
    <skos:narrower rdf:resource="descriptor/16909-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70361">
    <skos:narrower rdf:resource="descriptor/16898-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70361">
    <skos:narrower rdf:resource="descriptor/19075-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70361">
    <skos:narrower rdf:resource="descriptor/16896-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70361">
    <skos:narrower rdf:resource="descriptor/18131-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70361">
    <skos:narrower rdf:resource="descriptor/18133-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70361">
    <skos:narrower rdf:resource="descriptor/16874-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70361">
    <skos:narrower rdf:resource="descriptor/16868-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70361">
    <skos:narrower rdf:resource="descriptor/16834-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70361">
    <skos:narrower rdf:resource="descriptor/18012-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70361">
    <skos:narrower rdf:resource="descriptor/16927-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70361">
    <skos:narrower rdf:resource="descriptor/16849-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70361">
    <skos:narrower rdf:resource="descriptor/18132-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70361">
    <skos:narrower rdf:resource="descriptor/16830-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70361">
    <skos:narrower rdf:resource="descriptor/16842-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70361">
    <skos:narrower rdf:resource="descriptor/16841-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70361">
    <skos:narrower rdf:resource="descriptor/16853-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10122-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11215-3">
    <skos:narrower rdf:resource="descriptor/18911-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11215-3">
    <skos:narrower rdf:resource="descriptor/11399-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10949-5">
    <skos:related rdf:resource="descriptor/10963-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10949-5">
    <skos:related rdf:resource="descriptor/10895-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10949-5">
    <skos:related rdf:resource="descriptor/10948-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10949-5">
    <skos:related rdf:resource="descriptor/10904-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11907-2">
    <skos:broader rdf:resource="thsys/71070"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11907-2">
    <skos:broader rdf:resource="descriptor/21082-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11618-2">
    <skos:related rdf:resource="descriptor/11611-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10828-3">
    <skos:prefLabel xml:lang="en">Double taxation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10828-3">
    <skos:prefLabel xml:lang="de">Doppelbesteuerung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10181-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17571-0">
    <skos:altLabel xml:lang="en">Far East</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17571-0">
    <skos:altLabel xml:lang="de">Ostasiatische Staaten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17571-0">
    <skos:altLabel xml:lang="de">Ostasiatische Länder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17571-0">
    <skos:altLabel xml:lang="de">Ferner Osten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12065-5">
    <zbwext:useInsteadNote rdf:nodeID="id264"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70143">
    <rdfs:label xml:lang="de">Bildung und Lernen</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13960-2">
    <skos:prefLabel xml:lang="en">Engineers</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13960-2">
    <skos:prefLabel xml:lang="de">Ingenieure</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17632-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18156-0">
    <skos:altLabel xml:lang="de">Neutralitätstheorem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18156-0">
    <skos:altLabel xml:lang="de">Barro-Ricardo-Neutralitätstheorem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18156-0">
    <skos:altLabel xml:lang="de">Äquivalenztheorem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18717-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16719-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18169-5">
    <skos:related rdf:resource="descriptor/16150-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18169-5">
    <skos:related rdf:resource="descriptor/11346-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18169-5">
    <skos:related rdf:resource="descriptor/20455-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18169-5">
    <skos:related rdf:resource="descriptor/16131-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18014-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10411-1">
    <skos:related rdf:resource="descriptor/16214-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10411-1">
    <skos:related rdf:resource="descriptor/10414-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10411-1">
    <skos:related rdf:resource="descriptor/10434-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10411-1">
    <skos:related rdf:resource="descriptor/15745-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10411-1">
    <skos:related rdf:resource="descriptor/18325-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10411-1">
    <skos:related rdf:resource="descriptor/15751-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14357-3">
    <skos:altLabel xml:lang="en">Engine</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14357-3">
    <skos:altLabel xml:lang="de">Verbrennungsmotor</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14357-3">
    <skos:altLabel xml:lang="de">Elektromotor</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14646-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19464-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18315-6">
    <skos:narrower rdf:resource="descriptor/18324-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18651-4">
    <skos:broader rdf:resource="thsys/70922"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18651-4">
    <skos:broader rdf:resource="thsys/70878"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18651-4">
    <skos:broader rdf:resource="descriptor/10436-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12742-3">
    <skos:narrower rdf:resource="descriptor/10187-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18187-3">
    <skos:broader rdf:resource="thsys/70928"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18187-3">
    <skos:broader rdf:resource="descriptor/10031-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18162-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19238-0">
    <skos:prefLabel xml:lang="en">Business climate</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19238-0">
    <skos:prefLabel xml:lang="de">Geschäftsklima</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18624-0">
    <skos:prefLabel xml:lang="en">Transport finance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18624-0">
    <skos:prefLabel xml:lang="de">Verkehrsfinanzierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10562-1">
    <skos:prefLabel xml:lang="en">Growth centre</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10562-1">
    <skos:prefLabel xml:lang="de">Entwicklungspol</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16719-6">
    <skos:related rdf:resource="descriptor/15682-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16719-6">
    <skos:related rdf:resource="descriptor/15678-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16719-6">
    <skos:related rdf:resource="descriptor/19777-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16719-6">
    <skos:related rdf:resource="descriptor/15671-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16719-6">
    <skos:related rdf:resource="descriptor/16405-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16719-6">
    <skos:related rdf:resource="descriptor/15627-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16055-2">
    <skos:broader rdf:resource="thsys/73390"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16055-2">
    <skos:broader rdf:resource="thsys/70965"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16055-2">
    <skos:broader rdf:resource="descriptor/16052-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13782-0">
    <skos:broader rdf:resource="thsys/73392"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13782-0">
    <skos:broader rdf:resource="thsys/70899"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13782-0">
    <skos:broader rdf:resource="descriptor/12252-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19858-5">
    <skos:related rdf:resource="descriptor/21110-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19858-5">
    <skos:related rdf:resource="descriptor/18818-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19858-5">
    <skos:related rdf:resource="descriptor/14548-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19858-5">
    <skos:related rdf:resource="descriptor/13085-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12781-0">
    <skos:related rdf:resource="descriptor/13457-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12781-0">
    <skos:related rdf:resource="descriptor/18527-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14074-1">
    <skos:broader rdf:resource="thsys/70341"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14074-1">
    <skos:broader rdf:resource="descriptor/14054-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11809-2">
    <skos:narrower rdf:resource="descriptor/18333-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19285-5">
    <skos:prefLabel xml:lang="en">Macroeconomic instability</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19285-5">
    <skos:prefLabel xml:lang="de">Wirtschaftliche Instabilität</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13853-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10138-0">
    <skos:broader rdf:resource="thsys/73104"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10138-0">
    <skos:broader rdf:resource="thsys/71119"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10138-0">
    <skos:broader rdf:resource="descriptor/10144-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18225-0">
    <skos:broader rdf:resource="thsys/70073"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18225-0">
    <skos:broader rdf:resource="descriptor/19085-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18225-0">
    <skos:broader rdf:resource="descriptor/19084-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18225-0">
    <skos:broader rdf:resource="descriptor/17983-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19203-5">
    <skos:altLabel xml:lang="de">Sozialdarwinismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19203-5">
    <skos:altLabel xml:lang="de">Sozialbiologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19203-5">
    <skos:altLabel xml:lang="de">Biologismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19543-2">
    <skos:broader rdf:resource="thsys/70843"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19543-2">
    <skos:broader rdf:resource="thsys/70176"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19543-2">
    <skos:broader rdf:resource="descriptor/11365-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18461-2">
    <skos:related rdf:resource="descriptor/19251-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13641-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18315-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10603-6">
    <zbwext:useInsteadNote rdf:nodeID="id154"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18565-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13578-6">
    <skos:prefLabel xml:lang="en">Space engineering</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13578-6">
    <skos:prefLabel xml:lang="de">Weltraumtechnik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70880">
    <skos:narrower rdf:resource="descriptor/18770-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70880">
    <skos:narrower rdf:resource="descriptor/18769-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70880">
    <skos:narrower rdf:resource="descriptor/18768-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70880">
    <skos:narrower rdf:resource="descriptor/18941-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70880">
    <skos:narrower rdf:resource="descriptor/18763-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70880">
    <skos:narrower rdf:resource="descriptor/16774-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70880">
    <skos:narrower rdf:resource="descriptor/19654-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14840-0">
    <skos:altLabel xml:lang="de">Tüll</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14840-0">
    <skos:altLabel xml:lang="de">Textilspitze</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14840-0">
    <skos:altLabel xml:lang="de">Textilband</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14840-0">
    <skos:altLabel xml:lang="de">Tapisserie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14840-0">
    <skos:altLabel xml:lang="de">Stickerei</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14840-0">
    <skos:altLabel xml:lang="de">Reißverschluss</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14840-0">
    <skos:altLabel xml:lang="de">Reißverschluß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14840-0">
    <skos:altLabel xml:lang="de">Kurzwaren (Textilien)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14840-0">
    <skos:altLabel xml:lang="de">Kurzwaren für Textilien</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14840-0">
    <skos:altLabel xml:lang="de">Knopf</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14840-0">
    <skos:altLabel xml:lang="de">Handarbeitsartikel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18282-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18306-0">
    <skos:related rdf:resource="descriptor/14955-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18306-0">
    <skos:related rdf:resource="descriptor/14124-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18306-0">
    <skos:related rdf:resource="descriptor/18305-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11460-4">
    <skos:related rdf:resource="descriptor/10271-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11460-4">
    <skos:related rdf:resource="descriptor/19602-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11460-4">
    <skos:related rdf:resource="descriptor/11457-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11460-4">
    <skos:related rdf:resource="descriptor/19258-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11460-4">
    <skos:related rdf:resource="descriptor/11451-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11460-4">
    <skos:related rdf:resource="descriptor/11449-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11460-4">
    <skos:related rdf:resource="descriptor/11422-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11460-4">
    <skos:related rdf:resource="descriptor/11448-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11460-4">
    <skos:related rdf:resource="descriptor/12228-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18117-3">
    <skos:prefLabel xml:lang="en">Synergy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18117-3">
    <skos:prefLabel xml:lang="de">Synergie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15005-1">
    <skos:narrower rdf:resource="descriptor/14413-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15005-1">
    <skos:narrower rdf:resource="descriptor/18223-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15005-1">
    <skos:narrower rdf:resource="descriptor/14600-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15005-1">
    <skos:narrower rdf:resource="descriptor/15001-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12613-0">
    <skos:narrower rdf:resource="descriptor/18368-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12613-0">
    <skos:narrower rdf:resource="descriptor/15605-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16136-2">
    <skos:altLabel xml:lang="en">Shop rules</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16136-2">
    <skos:altLabel xml:lang="de">Bundespersonalvertretungsgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16136-2">
    <skos:altLabel xml:lang="de">BetrVG (Betriebsverfassungsgesetz)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16136-2">
    <skos:altLabel xml:lang="de">Betriebsverfassungsrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16136-2">
    <skos:altLabel xml:lang="de">Betriebsverfassungsgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19176-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#broadMatch">
    <rdfs:label xml:lang="en">broader match</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15672-4">
    <skos:altLabel xml:lang="de">Transsexualität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15672-4">
    <skos:altLabel xml:lang="de">Sexualwissenschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15672-4">
    <skos:altLabel xml:lang="de">Sexualverhalten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15672-4">
    <skos:altLabel xml:lang="de">Sexualpsychologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15672-4">
    <skos:altLabel xml:lang="de">Sexualmoral</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15672-4">
    <skos:altLabel xml:lang="de">Sexualforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15672-4">
    <skos:altLabel xml:lang="de">Sexualethik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15672-4">
    <skos:altLabel xml:lang="de">Erotik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15672-4">
    <skos:altLabel xml:lang="de">Bisexualität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18482-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10353-3">
    <skos:altLabel xml:lang="en">PBC (Political business cycle)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18698-6">
    <skos:broader rdf:resource="thsys/70904"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18698-6">
    <skos:broader rdf:resource="thsys/70305"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18698-6">
    <skos:broader rdf:resource="thsys/70259"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18698-6">
    <skos:broader rdf:resource="descriptor/10987-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18426-4">
    <skos:related rdf:resource="descriptor/14473-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18426-4">
    <skos:related rdf:resource="descriptor/13076-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18426-4">
    <skos:related rdf:resource="descriptor/13890-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10323-5">
    <skos:broader rdf:resource="thsys/70310"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10323-5">
    <skos:broader rdf:resource="descriptor/12725-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16176-4">
    <skos:broader rdf:resource="thsys/73360"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16176-4">
    <skos:broader rdf:resource="thsys/70970"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16176-4">
    <skos:broader rdf:resource="descriptor/15966-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17597-3">
    <skos:altLabel xml:lang="de">Volksrepublik Bangladesch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15675-5">
    <skos:prefLabel xml:lang="en">Sports</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15675-5">
    <skos:prefLabel xml:lang="de">Sport</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18576-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17991-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12749-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19621-1">
    <zbwext:useInsteadNote rdf:nodeID="id80"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18541-4">
    <skos:broader rdf:resource="thsys/70951"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18541-4">
    <skos:broader rdf:resource="descriptor/19565-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18541-4">
    <skos:broader rdf:resource="descriptor/10928-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18541-4">
    <skos:broader rdf:resource="descriptor/10876-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11572-0">
    <skos:narrower rdf:resource="descriptor/18629-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11572-0">
    <skos:narrower rdf:resource="descriptor/11567-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11572-0">
    <skos:narrower rdf:resource="descriptor/11241-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11572-0">
    <skos:narrower rdf:resource="descriptor/18482-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11572-0">
    <skos:narrower rdf:resource="descriptor/18774-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11572-0">
    <skos:narrower rdf:resource="descriptor/19318-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11572-0">
    <skos:narrower rdf:resource="descriptor/11562-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19389-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10269-6">
    <skos:altLabel xml:lang="de">Volkswirtschaftliche Einkommensrechnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10269-6">
    <skos:altLabel xml:lang="de">Nettosozialprodukt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10269-6">
    <skos:altLabel xml:lang="de">Nationaleinkommen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10269-6">
    <skos:altLabel xml:lang="de">Inländereinkommen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10269-6">
    <skos:altLabel xml:lang="de">Einkommensverwendungsrechnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10269-6">
    <skos:altLabel xml:lang="de">Einkommensentstehungsrechnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18326-1">
    <skos:broader rdf:resource="thsys/71024"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18326-1">
    <skos:broader rdf:resource="descriptor/18313-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13152-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19102-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19459-5">
    <skos:related rdf:resource="descriptor/18993-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19459-5">
    <skos:related rdf:resource="descriptor/19458-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19459-5">
    <skos:related rdf:resource="descriptor/11862-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11478-6">
    <skos:altLabel xml:lang="de">Rediskontregulierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11478-6">
    <skos:altLabel xml:lang="de">Rediskontpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11478-6">
    <skos:altLabel xml:lang="de">Rediskont</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11478-6">
    <skos:altLabel xml:lang="de">Diskontsatz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18260-5">
    <skos:broader rdf:resource="thsys/70993"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18260-5">
    <skos:broader rdf:resource="descriptor/13002-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15033-3">
    <skos:altLabel xml:lang="de">Entropiemodell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15033-3">
    <skos:altLabel xml:lang="de">Entropiemaß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13785-1">
    <skos:broader rdf:resource="thsys/70899"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13785-1">
    <skos:broader rdf:resource="descriptor/13769-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13785-1">
    <skos:broader rdf:resource="descriptor/19282-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16363-5">
    <skos:broader rdf:resource="thsys/73378"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16363-5">
    <skos:broader rdf:resource="descriptor/13397-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19132-2">
    <skos:broader rdf:resource="thsys/73391"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19132-2">
    <skos:broader rdf:resource="thsys/70384"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19132-2">
    <skos:broader rdf:resource="descriptor/10964-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17799-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11180-3">
    <skos:narrower rdf:resource="descriptor/11209-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11180-3">
    <skos:narrower rdf:resource="descriptor/19496-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11180-3">
    <skos:narrower rdf:resource="descriptor/11201-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11180-3">
    <skos:narrower rdf:resource="descriptor/11198-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11180-3">
    <skos:narrower rdf:resource="descriptor/11197-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11180-3">
    <skos:narrower rdf:resource="descriptor/18144-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11180-3">
    <skos:narrower rdf:resource="descriptor/19521-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11180-3">
    <skos:narrower rdf:resource="descriptor/18111-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19403-4">
    <skos:related rdf:resource="descriptor/19402-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19264-6">
    <skos:broader rdf:resource="thsys/71107"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19264-6">
    <skos:broader rdf:resource="thsys/70455"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19264-6">
    <skos:broader rdf:resource="descriptor/12299-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16298-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70991">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11707-3">
    <skos:broader rdf:resource="thsys/71075"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11707-3">
    <skos:broader rdf:resource="descriptor/10031-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19504-5">
    <skos:broader rdf:resource="thsys/71059"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19504-5">
    <skos:broader rdf:resource="descriptor/11248-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19504-5">
    <skos:broader rdf:resource="descriptor/11187-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18793-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18574-3">
    <skos:prefLabel xml:lang="en">Passenger railway</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18574-3">
    <skos:prefLabel xml:lang="de">Eisenbahnpersonenverkehr</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17804-3">
    <skos:altLabel xml:lang="de">Tobago</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17804-3">
    <skos:altLabel xml:lang="de">Republik Trinidad und Tobago</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18171-4">
    <skos:related rdf:resource="descriptor/15060-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18171-4">
    <skos:related rdf:resource="descriptor/15036-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70648">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10052-3">
    <skos:broader rdf:resource="thsys/70578"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10052-3">
    <skos:broader rdf:resource="thsys/70049"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10052-3">
    <skos:broader rdf:resource="descriptor/10011-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15658-5">
    <skos:narrower rdf:resource="descriptor/18895-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15658-5">
    <skos:narrower rdf:resource="descriptor/19016-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15658-5">
    <skos:narrower rdf:resource="descriptor/16683-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13427-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70313">
    <rdfs:label xml:lang="de">Leder, Lederwaren und Schuhe</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19588-1">
    <skos:related rdf:resource="descriptor/12922-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19588-1">
    <skos:related rdf:resource="descriptor/19585-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19588-1">
    <skos:related rdf:resource="descriptor/18351-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19588-1">
    <skos:related rdf:resource="descriptor/13433-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72181">
    <skos:narrower rdf:resource="descriptor/19590-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72181">
    <skos:narrower rdf:resource="descriptor/13831-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72181">
    <skos:narrower rdf:resource="descriptor/15076-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72181">
    <skos:narrower rdf:resource="descriptor/19580-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72181">
    <skos:narrower rdf:resource="descriptor/19589-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72181">
    <skos:narrower rdf:resource="descriptor/18625-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72181">
    <skos:narrower rdf:resource="descriptor/15108-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72181">
    <skos:narrower rdf:resource="descriptor/15074-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72181">
    <skos:narrower rdf:resource="descriptor/11641-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72181">
    <skos:narrower rdf:resource="descriptor/15107-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72181">
    <skos:narrower rdf:resource="descriptor/19072-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72181">
    <skos:narrower rdf:resource="descriptor/18887-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72181">
    <skos:narrower rdf:resource="descriptor/19588-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72181">
    <skos:narrower rdf:resource="descriptor/11962-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72181">
    <skos:narrower rdf:resource="descriptor/15105-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72181">
    <skos:narrower rdf:resource="descriptor/15104-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72181">
    <skos:narrower rdf:resource="descriptor/19602-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72181">
    <skos:narrower rdf:resource="descriptor/15089-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72181">
    <skos:narrower rdf:resource="descriptor/19587-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72181">
    <skos:narrower rdf:resource="descriptor/19586-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72181">
    <skos:narrower rdf:resource="descriptor/15103-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72181">
    <skos:narrower rdf:resource="descriptor/19603-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72181">
    <skos:narrower rdf:resource="descriptor/11032-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72181">
    <skos:narrower rdf:resource="descriptor/19579-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72181">
    <skos:narrower rdf:resource="descriptor/13465-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72181">
    <skos:narrower rdf:resource="descriptor/19578-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72181">
    <skos:narrower rdf:resource="descriptor/11965-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72181">
    <skos:narrower rdf:resource="descriptor/19585-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72181">
    <skos:narrower rdf:resource="descriptor/19584-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72181">
    <skos:narrower rdf:resource="descriptor/11534-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72181">
    <skos:narrower rdf:resource="descriptor/15940-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72181">
    <skos:narrower rdf:resource="descriptor/18351-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72181">
    <skos:narrower rdf:resource="descriptor/15102-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72181">
    <skos:narrower rdf:resource="descriptor/19604-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72181">
    <skos:narrower rdf:resource="descriptor/18888-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72181">
    <skos:narrower rdf:resource="descriptor/15061-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72181">
    <skos:narrower rdf:resource="descriptor/12039-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72181">
    <skos:narrower rdf:resource="descriptor/11297-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72181">
    <skos:narrower rdf:resource="descriptor/18386-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72181">
    <skos:narrower rdf:resource="descriptor/15101-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72181">
    <skos:narrower rdf:resource="descriptor/13689-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72181">
    <skos:narrower rdf:resource="descriptor/10835-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72181">
    <skos:narrower rdf:resource="descriptor/11264-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72181">
    <skos:narrower rdf:resource="descriptor/13433-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13584-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11659-2">
    <skos:scopeNote xml:lang="de">Besteuerung von Grund und Boden.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19176-3">
    <skos:related rdf:resource="descriptor/17724-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18874-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18835-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11540-6">
    <skos:broader rdf:resource="thsys/70869"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11540-6">
    <skos:broader rdf:resource="thsys/70144"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70476">
    <rdfs:label xml:lang="de">Kunststoffe und Chemiefasern</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11238-5">
    <skos:prefLabel xml:lang="en">Employment effect</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11238-5">
    <skos:prefLabel xml:lang="de">Beschäftigungseffekt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70932">
    <rdfs:label xml:lang="en">Air transport</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70932">
    <rdfs:label xml:lang="de">Luftverkehr</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11603-1">
    <skos:broader rdf:resource="thsys/71079"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11603-1">
    <skos:broader rdf:resource="thsys/71064"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11603-1">
    <skos:broader rdf:resource="descriptor/11599-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16014-2">
    <skos:broader rdf:resource="thsys/73360"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16014-2">
    <skos:broader rdf:resource="descriptor/18067-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16697-4">
    <skos:altLabel xml:lang="de">Sozialgerontologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16697-4">
    <skos:altLabel xml:lang="de">Gerontologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16697-4">
    <skos:altLabel xml:lang="de">Alterssoziologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16697-4">
    <skos:altLabel xml:lang="de">Alternspsychologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16697-4">
    <skos:altLabel xml:lang="de">Alternsforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13962-5">
    <skos:altLabel xml:lang="de">Technische Zeichnerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13962-5">
    <skos:altLabel xml:lang="de">Technische Zeichner</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13962-5">
    <skos:altLabel xml:lang="de">Technikerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13962-5">
    <skos:altLabel xml:lang="de">Techniker</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13962-5">
    <skos:altLabel xml:lang="de">Laborantinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13962-5">
    <skos:altLabel xml:lang="de">Laboranten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13173-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16205-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11043-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19682-2">
    <skos:scopeNote xml:lang="de">Herrschaft einer einflussreichen, besitzenden Minderheit im Staat.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18712-3">
    <skos:narrower rdf:resource="descriptor/18022-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18551-1">
    <skos:related rdf:resource="descriptor/13490-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70141">
    <skos:narrower rdf:resource="descriptor/10166-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70141">
    <skos:narrower rdf:resource="descriptor/16743-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70141">
    <skos:narrower rdf:resource="descriptor/16562-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70141">
    <skos:narrower rdf:resource="descriptor/11090-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70141">
    <skos:narrower rdf:resource="descriptor/16210-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70141">
    <skos:narrower rdf:resource="descriptor/25743-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70141">
    <skos:narrower rdf:resource="descriptor/16742-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70141">
    <skos:narrower rdf:resource="descriptor/20289-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70141">
    <skos:narrower rdf:resource="descriptor/11174-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70141">
    <skos:narrower rdf:resource="descriptor/10054-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70141">
    <skos:narrower rdf:resource="descriptor/16741-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70141">
    <skos:narrower rdf:resource="descriptor/16740-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70141">
    <skos:narrower rdf:resource="descriptor/10977-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70141">
    <skos:narrower rdf:resource="descriptor/10252-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70141">
    <skos:narrower rdf:resource="descriptor/10250-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70141">
    <skos:narrower rdf:resource="descriptor/15704-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70141">
    <skos:narrower rdf:resource="descriptor/16738-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70141">
    <skos:narrower rdf:resource="descriptor/10041-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70141">
    <skos:narrower rdf:resource="descriptor/10247-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70141">
    <skos:narrower rdf:resource="descriptor/11161-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70141">
    <skos:narrower rdf:resource="descriptor/16712-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70141">
    <skos:narrower rdf:resource="descriptor/19206-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70141">
    <skos:narrower rdf:resource="descriptor/19205-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70141">
    <skos:narrower rdf:resource="descriptor/19204-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70141">
    <skos:narrower rdf:resource="descriptor/16745-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70141">
    <skos:narrower rdf:resource="descriptor/11444-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70141">
    <skos:narrower rdf:resource="descriptor/16737-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70141">
    <skos:narrower rdf:resource="descriptor/19203-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70141">
    <skos:narrower rdf:resource="descriptor/11443-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18912-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14160-1">
    <skos:altLabel xml:lang="de">Weichtier</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14160-1">
    <skos:altLabel xml:lang="de">Tintenfisch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14160-1">
    <skos:altLabel xml:lang="de">Schaltier</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14160-1">
    <skos:altLabel xml:lang="de">Naturschwamm</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14160-1">
    <skos:altLabel xml:lang="de">Muschel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14160-1">
    <skos:altLabel xml:lang="de">Meeresschnecke</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14160-1">
    <skos:altLabel xml:lang="de">Meeresfrüchte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14160-1">
    <skos:altLabel xml:lang="de">Krill</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14160-1">
    <skos:altLabel xml:lang="de">Koralle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14160-1">
    <skos:altLabel xml:lang="de">Auster</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10056-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16826-5">
    <skos:broader rdf:resource="thsys/70073"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16826-5">
    <skos:broader rdf:resource="descriptor/19085-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16826-5">
    <skos:broader rdf:resource="descriptor/19084-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16826-5">
    <skos:broader rdf:resource="descriptor/17983-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14626-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10791-0">
    <skos:prefLabel xml:lang="en">Balance of payments</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10791-0">
    <skos:prefLabel xml:lang="de">Zahlungsbilanz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70060">
    <rdfs:label xml:lang="de">Kultur und Geisteswissenschaften</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19526-2">
    <skos:broader rdf:resource="thsys/73347"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19526-2">
    <skos:broader rdf:resource="thsys/70839"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19526-2">
    <skos:broader rdf:resource="descriptor/18892-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14171-3">
    <skos:altLabel xml:lang="de">Faulgas</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14171-3">
    <skos:altLabel xml:lang="de">Deponiegas</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14171-3">
    <skos:altLabel xml:lang="de">Biomasse-Energie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14171-3">
    <skos:altLabel xml:lang="de">Biogas</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11739-4">
    <skos:related rdf:resource="descriptor/10179-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11739-4">
    <skos:related rdf:resource="descriptor/11740-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11739-4">
    <skos:related rdf:resource="descriptor/19376-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12206-6">
    <skos:prefLabel xml:lang="en">Theory of finance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12206-6">
    <skos:prefLabel xml:lang="de">Finanzierungstheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15450-1">
    <skos:altLabel xml:lang="en">Noncooperative games</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15450-1">
    <skos:altLabel xml:lang="en">Non-cooperative games</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15450-1">
    <skos:altLabel xml:lang="de">Nichtkooperative Spiele</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15450-1">
    <skos:altLabel xml:lang="de">Konkurrenzspiel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15450-1">
    <skos:altLabel xml:lang="de">Kompetitives Spiel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71119">
    <rdfs:label xml:lang="de">Markttheorie</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11792-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14632-0">
    <skos:prefLabel xml:lang="en">Pollutant</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14632-0">
    <skos:prefLabel xml:lang="de">Umweltchemikalie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16621-2">
    <skos:broader rdf:resource="thsys/73356"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13652-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11533-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14380-1">
    <skos:prefLabel xml:lang="en">Agricultural machinery</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14380-1">
    <skos:prefLabel xml:lang="de">Landmaschine</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11330-3">
    <skos:prefLabel xml:lang="en">Real wages</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11330-3">
    <skos:prefLabel xml:lang="de">Reallohn</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13957-5">
    <zbwext:useInsteadNote rdf:nodeID="id28"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18613-5">
    <skos:narrower rdf:resource="descriptor/14632-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18613-5">
    <skos:narrower rdf:resource="descriptor/15017-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18613-5">
    <skos:narrower rdf:resource="descriptor/14600-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19860-4">
    <skos:broader rdf:resource="thsys/73332"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19860-4">
    <skos:broader rdf:resource="descriptor/16195-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18378-3">
    <skos:narrower rdf:resource="descriptor/18377-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11800-6">
    <skos:prefLabel xml:lang="en">Agricultural market regulations</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11800-6">
    <skos:prefLabel xml:lang="de">Agrarmarktordnung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21061-5">
    <skos:related rdf:resource="descriptor/12396-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21061-5">
    <skos:related rdf:resource="descriptor/19410-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21061-5">
    <skos:related rdf:resource="descriptor/10016-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21061-5">
    <skos:related rdf:resource="descriptor/19217-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21061-5">
    <skos:related rdf:resource="descriptor/12029-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21061-5">
    <skos:related rdf:resource="descriptor/10030-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70459">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">B.07.04</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19125-6">
    <skos:related rdf:resource="descriptor/10216-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19125-6">
    <skos:related rdf:resource="descriptor/10902-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18625-5">
    <skos:related rdf:resource="descriptor/18187-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18625-5">
    <skos:related rdf:resource="descriptor/13264-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18625-5">
    <skos:related rdf:resource="descriptor/13583-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18625-5">
    <skos:related rdf:resource="descriptor/13510-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19688-4">
    <skos:altLabel xml:lang="en">State creation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19688-4">
    <skos:altLabel xml:lang="de">Staatsneugründung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19688-4">
    <skos:altLabel xml:lang="de">Staatenneubildung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19688-4">
    <skos:altLabel xml:lang="de">Staatenbildung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70573">
    <rdfs:label xml:lang="en">Health</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70573">
    <rdfs:label xml:lang="de">Gesundheit</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18844-0">
    <skos:related rdf:resource="descriptor/11839-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18844-0">
    <skos:related rdf:resource="descriptor/18755-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19042-3">
    <skos:broader rdf:resource="thsys/70582"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14810-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18633-6">
    <skos:related rdf:resource="descriptor/11881-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18633-6">
    <skos:related rdf:resource="descriptor/11974-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18633-6">
    <skos:related rdf:resource="descriptor/18632-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18633-6">
    <skos:related rdf:resource="descriptor/18610-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16458-1">
    <skos:altLabel xml:lang="de">Wehrpotenzial</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16458-1">
    <skos:altLabel xml:lang="de">Wehrpotential</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16458-1">
    <skos:altLabel xml:lang="de">Verteidigungspotenzial</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16458-1">
    <skos:altLabel xml:lang="de">Verteidigungspotential</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16458-1">
    <skos:altLabel xml:lang="de">Militärpotential</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16458-1">
    <skos:altLabel xml:lang="de">Militärische Stärke</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16458-1">
    <skos:altLabel xml:lang="de">Militärische Macht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16458-1">
    <skos:altLabel xml:lang="de">Kriegspotenzial</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16458-1">
    <skos:altLabel xml:lang="de">Kriegspotential</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13225-4">
    <skos:related rdf:resource="descriptor/13161-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13225-4">
    <skos:related rdf:resource="descriptor/19342-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13225-4">
    <skos:related rdf:resource="descriptor/12947-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13225-4">
    <skos:related rdf:resource="descriptor/14110-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13225-4">
    <skos:related rdf:resource="descriptor/19341-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13225-4">
    <skos:related rdf:resource="descriptor/12942-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13225-4">
    <skos:related rdf:resource="descriptor/14105-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15119-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19363-4">
    <skos:broader rdf:resource="thsys/71064"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19363-4">
    <skos:broader rdf:resource="thsys/70091"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19363-4">
    <skos:broader rdf:resource="descriptor/10414-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19363-4">
    <skos:broader rdf:resource="descriptor/11990-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10812-4">
    <skos:prefLabel xml:lang="en">International credit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10812-4">
    <skos:prefLabel xml:lang="de">Internationaler Kredit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15606-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19014-1">
    <skos:prefLabel xml:lang="en">Apple</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19014-1">
    <skos:prefLabel xml:lang="de">Apfel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19804-0">
    <skos:scopeNote xml:lang="de">Programm, das nach Vorgaben des Programmierers selbständig agiert.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11721-2">
    <skos:scopeNote xml:lang="de">Bezeichnet den Grad der Inanspruchnahme der gesamten Volkswirtschaft durch den staatlichen Sektor: Verhältnis der Staatsausgaben zu einer Sozialproduktsgröße.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12903-5">
    <skos:related rdf:resource="descriptor/16120-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12903-5">
    <skos:related rdf:resource="descriptor/12901-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12903-5">
    <skos:related rdf:resource="descriptor/12882-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12903-5">
    <skos:related rdf:resource="descriptor/12897-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12903-5">
    <skos:related rdf:resource="descriptor/13871-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12903-5">
    <skos:related rdf:resource="descriptor/19154-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19663-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19663-6">
    <skos:related rdf:resource="descriptor/18847-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19663-6">
    <skos:related rdf:resource="descriptor/19354-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19663-6">
    <skos:related rdf:resource="descriptor/20810-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19663-6">
    <skos:related rdf:resource="descriptor/19336-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19663-6">
    <skos:related rdf:resource="descriptor/19352-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16274-4">
    <skos:narrower rdf:resource="descriptor/10988-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13106-5">
    <skos:broader rdf:resource="thsys/70991"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13106-5">
    <skos:broader rdf:resource="descriptor/13110-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16976-0">
    <skos:broader rdf:resource="thsys/70003"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16976-0">
    <skos:broader rdf:resource="descriptor/16972-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18133-5">
    <skos:prefLabel xml:lang="en">Mecklenburg-Western Pomerania</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18133-5">
    <skos:prefLabel xml:lang="de">Mecklenburg-Vorpommern</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18129-3">
    <skos:broader rdf:resource="thsys/70049"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11722-0">
    <skos:narrower rdf:resource="descriptor/11628-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11722-0">
    <skos:narrower rdf:resource="descriptor/10267-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11722-0">
    <skos:narrower rdf:resource="descriptor/13413-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11722-0">
    <skos:narrower rdf:resource="descriptor/11556-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11722-0">
    <skos:narrower rdf:resource="descriptor/13303-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11722-0">
    <skos:narrower rdf:resource="descriptor/19372-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11992-6">
    <skos:scopeNote xml:lang="de">Analysiert die Prozesse städtischen Wandels im Zusammenhang mit gesellschaftlichen Veränderungen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id109">
    <rdfs:seeAlso rdf:resource="descriptor/11448-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19590-0">
    <skos:related rdf:resource="descriptor/10791-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19590-0">
    <skos:related rdf:resource="descriptor/10835-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13988-1">
    <skos:related rdf:resource="descriptor/13989-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13988-1">
    <skos:related rdf:resource="descriptor/18847-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13988-1">
    <skos:related rdf:resource="descriptor/10984-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13988-1">
    <skos:related rdf:resource="descriptor/15823-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13988-1">
    <skos:related rdf:resource="descriptor/19521-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16788-1">
    <skos:prefLabel xml:lang="en">Information provision</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16788-1">
    <skos:prefLabel xml:lang="de">Informationsversorgung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12931-0">
    <skos:altLabel xml:lang="en">Public utilities</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12931-0">
    <skos:altLabel xml:lang="de">Öffentliche Versorgungswirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12607-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16543-3">
    <skos:prefLabel xml:lang="en">Social services</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16543-3">
    <skos:prefLabel xml:lang="de">Sozialer Dienst</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14007-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15721-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17171-2">
    <skos:broader rdf:resource="thsys/70130"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17171-2">
    <skos:broader rdf:resource="descriptor/17179-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17569-1">
    <skos:altLabel xml:lang="en">Democratic People's Republic of Korea</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17569-1">
    <skos:altLabel xml:lang="de">Koreanische Volksdemokratische Republik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17569-1">
    <skos:altLabel xml:lang="de">Korea (Demokratische Volksrepublik)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17569-1">
    <skos:altLabel xml:lang="de">Demokratische Volksrepublik Korea</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11357-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70855">
    <skos:narrower rdf:resource="descriptor/12540-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70855">
    <skos:narrower rdf:resource="descriptor/12189-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70855">
    <skos:narrower rdf:resource="descriptor/11628-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70855">
    <skos:narrower rdf:resource="descriptor/12475-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70855">
    <skos:narrower rdf:resource="descriptor/11825-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70855">
    <skos:narrower rdf:resource="descriptor/13302-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70855">
    <skos:narrower rdf:resource="descriptor/11037-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70855">
    <skos:narrower rdf:resource="thsys/70854"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19408-1">
    <skos:prefLabel xml:lang="en">Tax rate</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19408-1">
    <skos:prefLabel xml:lang="de">Steuertarif</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10300-3">
    <skos:broader rdf:resource="thsys/73384"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10300-3">
    <skos:broader rdf:resource="thsys/71112"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10300-3">
    <skos:broader rdf:resource="thsys/70814"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10300-3">
    <skos:broader rdf:resource="thsys/70493"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10300-3">
    <skos:broader rdf:resource="thsys/70292"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10300-3">
    <skos:broader rdf:resource="descriptor/15861-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70962">
    <skos:narrower rdf:resource="descriptor/19063-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70962">
    <skos:narrower rdf:resource="descriptor/19186-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70962">
    <skos:narrower rdf:resource="descriptor/19062-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70962">
    <skos:narrower rdf:resource="thsys/70957"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70962">
    <skos:narrower rdf:resource="thsys/70961"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70962">
    <skos:narrower rdf:resource="thsys/70959"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70962">
    <skos:narrower rdf:resource="thsys/70960"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70424">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12588-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14407-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16066-4">
    <skos:related rdf:resource="descriptor/10566-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16066-4">
    <skos:related rdf:resource="descriptor/17977-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12380-4">
    <skos:altLabel xml:lang="en">Stock-taking</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12380-4">
    <skos:altLabel xml:lang="en">Inventory</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12380-4">
    <skos:altLabel xml:lang="de">Inventaraufnahme</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12380-4">
    <skos:altLabel xml:lang="de">Inventar</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18165-6">
    <skos:broader rdf:resource="thsys/71033"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18165-6">
    <skos:broader rdf:resource="thsys/70815"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18880-3">
    <skos:altLabel xml:lang="en">Economics faculty</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18880-3">
    <skos:altLabel xml:lang="en">Economics department (University)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18880-3">
    <skos:altLabel xml:lang="en">Department of economics (University)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18880-3">
    <skos:altLabel xml:lang="en">Business school</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18880-3">
    <skos:altLabel xml:lang="de">Wirtschaftswissenschaftliche Hochschule</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18880-3">
    <skos:altLabel xml:lang="de">Wirtschaftswissenschaftliche Fakultät</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18880-3">
    <skos:altLabel xml:lang="de">Wirtschaftsuniversität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18880-3">
    <skos:altLabel xml:lang="de">Volkswirtschaftlicher Fachbereich</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18880-3">
    <skos:altLabel xml:lang="de">Fachbereich Wirtschaftswissenschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18880-3">
    <skos:altLabel xml:lang="de">Betriebswirtschaftlicher Fachbereich</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14232-2">
    <skos:related rdf:resource="descriptor/14258-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17477-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71096">
    <skos:broader rdf:resource="thsys/71098"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11610-4">
    <skos:narrower rdf:resource="descriptor/11609-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16602-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11314-1">
    <skos:narrower rdf:resource="descriptor/11067-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19289-4">
    <skos:altLabel xml:lang="en">Illegal market</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19289-4">
    <skos:altLabel xml:lang="en">Grey market</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19289-4">
    <skos:altLabel xml:lang="de">Schwarzhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19289-4">
    <skos:altLabel xml:lang="de">Schwarzer Markt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19289-4">
    <skos:altLabel xml:lang="de">Grauer Markt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11678-5">
    <skos:altLabel xml:lang="en">VAT (Value-Added Tax)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11678-5">
    <skos:altLabel xml:lang="en">Value-added tax</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11678-5">
    <skos:altLabel xml:lang="en">Value added tax</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11678-5">
    <skos:altLabel xml:lang="en">Purchase tax</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11678-5">
    <skos:altLabel xml:lang="de">Vorsteuerabzug</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11678-5">
    <skos:altLabel xml:lang="de">USTG (Umsatzsteuergesetz)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11678-5">
    <skos:altLabel xml:lang="de">Umsatzsteuerrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11678-5">
    <skos:altLabel xml:lang="de">Umsatzbesteuerung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11678-5">
    <skos:altLabel xml:lang="de">Mehrwertsteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16909-1">
    <skos:broader rdf:resource="thsys/70361"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16909-1">
    <skos:broader rdf:resource="descriptor/18012-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#inScheme">
    <rdfs:label xml:lang="en">in scheme</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13865-3">
    <skos:scopeNote xml:lang="de">Für volks- und betriebswirtschaftliche Literatur.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12981-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11904-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18991-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18568-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18433-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19770-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id120">
    <rdf:value xml:lang="de">Für Geschichte als wissenschaftliche Disziplin</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18400-1">
    <skos:prefLabel xml:lang="en">Metal professions</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18400-1">
    <skos:prefLabel xml:lang="de">Metallberufe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18910-6">
    <skos:narrower rdf:resource="descriptor/11569-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18910-6">
    <skos:narrower rdf:resource="descriptor/11687-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18910-6">
    <skos:narrower rdf:resource="descriptor/16548-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18910-6">
    <skos:narrower rdf:resource="descriptor/16143-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70300">
    <rdfs:label xml:lang="en">Inventory management</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70300">
    <rdfs:label xml:lang="de">Betriebliche Lagerhaltung</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16754-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18924-2">
    <skos:narrower rdf:resource="descriptor/15797-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18519-4">
    <skos:prefLabel xml:lang="en">Coffee trade</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18519-4">
    <skos:prefLabel xml:lang="de">Kaffeehandel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10126-0">
    <skos:prefLabel xml:lang="en">Capital theory</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10126-0">
    <skos:prefLabel xml:lang="de">Kapitaltheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18532-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18253-2">
    <skos:prefLabel xml:lang="en">Hydrogen peroxide</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18253-2">
    <skos:prefLabel xml:lang="de">Wasserstoffperoxid</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12891-0">
    <skos:altLabel xml:lang="de">Interne Betriebsprüfung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12891-0">
    <skos:altLabel xml:lang="de">Innenrevision</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#historyNote">
    <rdfs:isDefinedBy rdf:resource="http://www.w3.org/2004/02/skos/core"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10000-1">
    <skos:related rdf:resource="descriptor/10013-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18122-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13837-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14515-4">
    <skos:broader rdf:resource="thsys/70373"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14515-4">
    <skos:broader rdf:resource="descriptor/14525-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19216-3">
    <skos:altLabel xml:lang="en">Satisficing rationality</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19216-3">
    <skos:altLabel xml:lang="de">Eingeschränkte Rationalität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19216-3">
    <skos:altLabel xml:lang="de">Eingeschränkt rationales Verhalten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19216-3">
    <skos:altLabel xml:lang="de">Begrenzte Rationalität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18932-3">
    <skos:prefLabel xml:lang="en">Media usage</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18932-3">
    <skos:prefLabel xml:lang="de">Medienverhalten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18831-2">
    <skos:broader rdf:resource="thsys/73337"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18831-2">
    <skos:broader rdf:resource="thsys/71073"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18831-2">
    <skos:broader rdf:resource="thsys/70856"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18831-2">
    <skos:broader rdf:resource="descriptor/11640-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70057">
    <skos:narrower rdf:resource="descriptor/10113-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70057">
    <skos:narrower rdf:resource="descriptor/10112-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70057">
    <skos:narrower rdf:resource="descriptor/19210-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70057">
    <skos:narrower rdf:resource="descriptor/10111-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70057">
    <skos:narrower rdf:resource="descriptor/10110-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70057">
    <skos:narrower rdf:resource="descriptor/10109-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70057">
    <skos:narrower rdf:resource="descriptor/10106-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70057">
    <skos:narrower rdf:resource="descriptor/11527-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70057">
    <skos:narrower rdf:resource="descriptor/10103-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70057">
    <skos:narrower rdf:resource="descriptor/19558-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70057">
    <skos:narrower rdf:resource="descriptor/10093-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70057">
    <skos:narrower rdf:resource="descriptor/10102-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70057">
    <skos:narrower rdf:resource="descriptor/10099-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70057">
    <skos:narrower rdf:resource="descriptor/10097-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70057">
    <skos:narrower rdf:resource="descriptor/19209-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70057">
    <skos:narrower rdf:resource="descriptor/10095-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70057">
    <skos:narrower rdf:resource="descriptor/10094-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70057">
    <skos:narrower rdf:resource="descriptor/10092-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70057">
    <skos:narrower rdf:resource="descriptor/10091-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70057">
    <skos:narrower rdf:resource="descriptor/19208-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70057">
    <skos:narrower rdf:resource="descriptor/10040-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70057">
    <skos:narrower rdf:resource="descriptor/10089-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70057">
    <skos:narrower rdf:resource="descriptor/10088-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13113-1">
    <skos:broader rdf:resource="thsys/70991"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13113-1">
    <skos:broader rdf:resource="descriptor/13110-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10269-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12920-5">
    <skos:altLabel xml:lang="de">Erdölwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13205-3">
    <skos:related rdf:resource="descriptor/13154-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13205-3">
    <skos:related rdf:resource="descriptor/12964-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19559-1">
    <skos:broader rdf:resource="thsys/70845"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19559-1">
    <skos:broader rdf:resource="descriptor/18884-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11806-1">
    <skos:narrower rdf:resource="descriptor/19546-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11806-1">
    <skos:narrower rdf:resource="descriptor/18331-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13125-1">
    <skos:broader rdf:resource="thsys/70990"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13125-1">
    <skos:broader rdf:resource="descriptor/13117-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19225-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13270-6">
    <skos:related rdf:resource="descriptor/20136-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13270-6">
    <skos:related rdf:resource="descriptor/18644-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13270-6">
    <skos:related rdf:resource="descriptor/13647-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13270-6">
    <skos:related rdf:resource="descriptor/13981-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13270-6">
    <skos:related rdf:resource="descriptor/15798-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18508-2">
    <skos:prefLabel xml:lang="en">Energy efficient building</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18508-2">
    <skos:prefLabel xml:lang="de">Energiesparendes Bauen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15719-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18557-3">
    <skos:altLabel xml:lang="en">Environmental labelling</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18557-3">
    <skos:altLabel xml:lang="de">Öko-Label</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18557-3">
    <skos:altLabel xml:lang="de">Blauer Engel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10683-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14829-2">
    <skos:prefLabel xml:lang="en">Leather</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14829-2">
    <skos:prefLabel xml:lang="de">Leder</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15117-4">
    <skos:related rdf:resource="descriptor/19582-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10135-6">
    <skos:narrower rdf:resource="descriptor/10129-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10135-6">
    <skos:narrower rdf:resource="descriptor/19265-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18694-0">
    <skos:altLabel xml:lang="de">Theorie internationaler Finanzmärkte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18694-0">
    <skos:altLabel xml:lang="de">Theorie internationaler Finanzbeziehungen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18694-0">
    <skos:altLabel xml:lang="de">Internationale Kreditvergabepolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18694-0">
    <skos:altLabel xml:lang="de">Internationale Kreditpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10628-4">
    <skos:related rdf:resource="descriptor/10650-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10628-4">
    <skos:related rdf:resource="descriptor/10627-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10628-4">
    <skos:related rdf:resource="descriptor/19351-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10628-4">
    <skos:related rdf:resource="descriptor/10612-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12470-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13202-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19144-2">
    <skos:scopeNote xml:lang="de">Netz, das Datenstationen verbindet, die über einen geographisch größeren Raum verteilt sind.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19040-0">
    <skos:broader rdf:resource="thsys/70582"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19041-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15920-4">
    <skos:prefLabel xml:lang="en">Women</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15920-4">
    <skos:prefLabel xml:lang="de">Frauen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10414-2">
    <skos:related rdf:resource="descriptor/10434-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10414-2">
    <skos:related rdf:resource="descriptor/10439-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10414-2">
    <skos:related rdf:resource="descriptor/11793-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10414-2">
    <skos:related rdf:resource="descriptor/12071-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10414-2">
    <skos:related rdf:resource="descriptor/10411-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10414-2">
    <skos:related rdf:resource="descriptor/10513-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10414-2">
    <skos:related rdf:resource="descriptor/19248-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16003-0">
    <skos:altLabel xml:lang="en">War crime</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16003-0">
    <skos:altLabel xml:lang="en">Law of war</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16003-0">
    <skos:altLabel xml:lang="de">Kriegsvölkerrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16003-0">
    <skos:altLabel xml:lang="de">Kriegsverbrechen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19358-4">
    <skos:broader rdf:resource="thsys/71089"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19358-4">
    <skos:broader rdf:resource="thsys/71084"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19358-4">
    <skos:broader rdf:resource="descriptor/10797-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11537-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19672-5">
    <skos:altLabel xml:lang="en">Market-access competencies</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19672-5">
    <skos:altLabel xml:lang="en">Integrity-related competencies</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19672-5">
    <skos:altLabel xml:lang="en">Functionally-related competencies</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19672-5">
    <skos:altLabel xml:lang="en">Core capabilities</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10559-4">
    <skos:broader rdf:resource="thsys/71085"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10559-4">
    <skos:broader rdf:resource="thsys/71084"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10559-4">
    <skos:broader rdf:resource="descriptor/10523-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18008-1">
    <skos:scopeNote xml:lang="de">Nur für den Außenhandel mit Agrarerzeugnissen insgesamt. Der Außenhandel mit einzelnen Agrarprodukten wird postkoordiniert.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11379-1">
    <skos:broader rdf:resource="thsys/70844"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11379-1">
    <skos:broader rdf:resource="descriptor/11382-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17937-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18456-2">
    <skos:editorialNote xml:lang="de">Nicht als Formschlagwort zu verwenden.</skos:editorialNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18009-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13322-6">
    <skos:prefLabel xml:lang="en">Auction</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13322-6">
    <skos:prefLabel xml:lang="de">Auktion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16981-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19690-3">
    <skos:altLabel xml:lang="de">Staatswappen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19690-3">
    <skos:altLabel xml:lang="de">Staatssiegel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19690-3">
    <skos:altLabel xml:lang="de">Nationalsymbol</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19690-3">
    <skos:altLabel xml:lang="de">Nationalhymne</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19690-3">
    <skos:altLabel xml:lang="de">Nationalflagge</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19690-3">
    <skos:altLabel xml:lang="de">Nationalfeiertag</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19690-3">
    <skos:altLabel xml:lang="de">Nationaldenkmal</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19690-3">
    <skos:altLabel xml:lang="de">Hoheitszeichen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19617-6">
    <skos:related rdf:resource="descriptor/19631-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10583-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13674-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73359">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70086">
    <rdfs:label xml:lang="en">Central America</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70086">
    <rdfs:label xml:lang="de">Mittelamerika</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#member">
    <rdfs:domain rdf:resource="http://www.w3.org/2004/02/skos/core#Collection"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15220-4">
    <skos:prefLabel xml:lang="en">Markov process</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15220-4">
    <skos:prefLabel xml:lang="de">Markovscher Prozess</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14230-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12251-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10593-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70892">
    <skos:narrower rdf:resource="descriptor/13821-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70892">
    <skos:narrower rdf:resource="descriptor/13282-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70892">
    <skos:narrower rdf:resource="descriptor/19424-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70892">
    <skos:narrower rdf:resource="descriptor/13826-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70892">
    <skos:narrower rdf:resource="descriptor/13823-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70892">
    <skos:narrower rdf:resource="descriptor/21072-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70892">
    <skos:narrower rdf:resource="descriptor/13827-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70892">
    <skos:narrower rdf:resource="descriptor/13825-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70892">
    <skos:narrower rdf:resource="descriptor/13824-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70892">
    <skos:narrower rdf:resource="descriptor/18994-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70892">
    <skos:narrower rdf:resource="descriptor/18741-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70892">
    <skos:narrower rdf:resource="descriptor/19679-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70892">
    <skos:narrower rdf:resource="descriptor/18744-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70892">
    <skos:narrower rdf:resource="descriptor/18743-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70892">
    <skos:narrower rdf:resource="descriptor/13820-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70892">
    <skos:narrower rdf:resource="descriptor/13819-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70892">
    <skos:narrower rdf:resource="descriptor/13969-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70892">
    <skos:narrower rdf:resource="descriptor/13818-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70892">
    <skos:narrower rdf:resource="descriptor/13828-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70892">
    <skos:narrower rdf:resource="descriptor/21002-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70892">
    <skos:narrower rdf:resource="descriptor/13296-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70892">
    <skos:narrower rdf:resource="descriptor/13811-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70892">
    <skos:narrower rdf:resource="descriptor/10207-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70892">
    <skos:narrower rdf:resource="descriptor/18734-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70892">
    <skos:narrower rdf:resource="thsys/70891"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19467-6">
    <skos:broader rdf:resource="thsys/71064"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19467-6">
    <skos:broader rdf:resource="descriptor/10513-0"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id275">
    <rdfs:seeAlso rdf:resource="descriptor/13389-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13220-0">
    <skos:prefLabel xml:lang="en">Leather goods trade</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13220-0">
    <skos:prefLabel xml:lang="de">Lederwarenhandel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13811-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18605-4">
    <skos:related rdf:resource="descriptor/16173-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10187-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19844-2">
    <skos:altLabel xml:lang="en">Labor court decision</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19844-2">
    <skos:altLabel xml:lang="de">Arbeitsgerichtsentscheidung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19844-2">
    <skos:altLabel xml:lang="de">Arbeitsgerichtliche Rechtsprechung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13998-5">
    <skos:related rdf:resource="descriptor/16522-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13998-5">
    <skos:related rdf:resource="descriptor/16684-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13998-5">
    <skos:related rdf:resource="descriptor/13412-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13998-5">
    <skos:related rdf:resource="descriptor/19721-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15265-3">
    <skos:altLabel xml:lang="en">TSLS (Two-Stage Least Squares)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15265-3">
    <skos:altLabel xml:lang="en">Ordinary least squares</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15265-3">
    <skos:altLabel xml:lang="en">OLS (Ordinary Least Squares)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15265-3">
    <skos:altLabel xml:lang="en">GLS (Generalized Least Squares)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15265-3">
    <skos:altLabel xml:lang="en">Generalized least squares</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15265-3">
    <skos:altLabel xml:lang="en">Aitken-estimator</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15265-3">
    <skos:altLabel xml:lang="de">KQ-Schätzung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15265-3">
    <skos:altLabel xml:lang="de">Kleinstquadratschätzung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15265-3">
    <skos:altLabel xml:lang="de">Einfache Kleinstquadratschätzung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19836-1">
    <skos:altLabel xml:lang="de">Trendforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19836-1">
    <skos:altLabel xml:lang="de">Tendenz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10011-3">
    <skos:related rdf:resource="descriptor/12535-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10011-3">
    <skos:related rdf:resource="descriptor/12742-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10011-3">
    <skos:related rdf:resource="descriptor/19098-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10011-3">
    <skos:related rdf:resource="descriptor/12480-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10011-3">
    <skos:related rdf:resource="descriptor/11529-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10011-3">
    <skos:related rdf:resource="descriptor/19096-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10011-3">
    <skos:related rdf:resource="descriptor/12306-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19076-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19782-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12041-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/v">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11479-4">
    <skos:prefLabel xml:lang="en">Monetary targeting</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11479-4">
    <skos:prefLabel xml:lang="de">Geldmengensteuerung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14323-6">
    <skos:broader rdf:resource="thsys/70346"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14323-6">
    <skos:broader rdf:resource="descriptor/13513-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14323-6">
    <skos:broader rdf:resource="descriptor/14269-0"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id237">
    <rdf:value xml:lang="de">Für Nonprofit-Wirtschaftsunternehmen</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19185-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10537-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12755-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14996-4">
    <skos:narrower rdf:resource="descriptor/14995-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13023-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19340-2">
    <skos:broader rdf:resource="thsys/71025"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19340-2">
    <skos:broader rdf:resource="thsys/70960"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19340-2">
    <skos:broader rdf:resource="descriptor/13431-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70882">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18000-3">
    <zbwext:useInsteadNote rdf:nodeID="id329"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18000-3">
    <zbwext:useInsteadNote rdf:nodeID="id328"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18269-1">
    <skos:broader rdf:resource="thsys/73305"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18269-1">
    <skos:broader rdf:resource="descriptor/18004-2"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id388">
    <rdfs:seeAlso rdf:resource="descriptor/18316-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71109">
    <skos:broader rdf:resource="thsys/70021"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16266-3">
    <skos:related rdf:resource="descriptor/11832-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16266-3">
    <skos:related rdf:resource="descriptor/15768-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16266-3">
    <skos:related rdf:resource="descriptor/15711-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16266-3">
    <skos:related rdf:resource="descriptor/16311-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16266-3">
    <skos:related rdf:resource="descriptor/16272-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16266-3">
    <skos:related rdf:resource="descriptor/16549-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16266-3">
    <skos:related rdf:resource="descriptor/11998-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16266-3">
    <skos:related rdf:resource="descriptor/19773-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16266-3">
    <skos:related rdf:resource="descriptor/16382-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10851-1">
    <skos:broader rdf:resource="thsys/73384"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10851-1">
    <skos:broader rdf:resource="thsys/70951"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10851-1">
    <skos:broader rdf:resource="descriptor/18059-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10851-1">
    <skos:broader rdf:resource="descriptor/12774-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16169-1">
    <skos:prefLabel xml:lang="en">Shipping conference</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16169-1">
    <skos:prefLabel xml:lang="de">Linienkonferenz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13282-6">
    <skos:related rdf:resource="descriptor/12820-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13282-6">
    <skos:related rdf:resource="descriptor/13967-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13282-6">
    <skos:related rdf:resource="descriptor/12812-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13102-6">
    <skos:narrower rdf:resource="descriptor/13124-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15964-5">
    <skos:prefLabel xml:lang="en">Human rights</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15964-5">
    <skos:prefLabel xml:lang="de">Menschenrechte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11895-4">
    <skos:prefLabel xml:lang="en">Regional development</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11895-4">
    <skos:prefLabel xml:lang="de">Regionale Entwicklung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12939-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13065-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18707-3">
    <skos:scopeNote xml:lang="de">Nur für staatliche Maßnahmen in Bezug auf den Bankensektor.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18311-0">
    <skos:altLabel xml:lang="de">Jagdrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18311-0">
    <skos:altLabel xml:lang="de">Bundeswaldgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19080-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70818">
    <skos:narrower rdf:resource="descriptor/13340-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70818">
    <skos:narrower rdf:resource="descriptor/13337-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70818">
    <skos:narrower rdf:resource="descriptor/13336-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70818">
    <skos:narrower rdf:resource="descriptor/13333-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70818">
    <skos:narrower rdf:resource="descriptor/18961-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70818">
    <skos:narrower rdf:resource="descriptor/15727-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70818">
    <skos:narrower rdf:resource="descriptor/18960-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70818">
    <skos:narrower rdf:resource="descriptor/15728-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70818">
    <skos:narrower rdf:resource="descriptor/13325-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70818">
    <skos:narrower rdf:resource="descriptor/18959-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70818">
    <skos:narrower rdf:resource="descriptor/15988-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70818">
    <skos:narrower rdf:resource="descriptor/18958-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70818">
    <skos:narrower rdf:resource="descriptor/24621-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70818">
    <skos:narrower rdf:resource="descriptor/18957-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70818">
    <skos:narrower rdf:resource="thsys/70816"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70818">
    <skos:narrower rdf:resource="thsys/70817"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70578">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10253-0">
    <skos:broader rdf:resource="thsys/72393"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10253-0">
    <skos:broader rdf:resource="thsys/70234"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10253-0">
    <skos:broader rdf:resource="descriptor/15376-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19666-0">
    <zbwext:useInsteadNote rdf:nodeID="id201"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13090-1">
    <zbwext:useInsteadNote rdf:nodeID="id362"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11123-1">
    <skos:related rdf:resource="descriptor/19837-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11123-1">
    <skos:related rdf:resource="descriptor/11060-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11123-1">
    <skos:related rdf:resource="descriptor/17461-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11123-1">
    <skos:related rdf:resource="descriptor/11061-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11123-1">
    <skos:related rdf:resource="descriptor/11047-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11123-1">
    <skos:related rdf:resource="descriptor/17176-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17937-5">
    <skos:altLabel xml:lang="de">Fidschiinseln</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73370">
    <skos:broader rdf:resource="thsys/73374"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73334">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14146-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19711-0">
    <skos:altLabel xml:lang="de">Internationaler Kongress</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19711-0">
    <skos:altLabel xml:lang="de">Internationaler Kongreß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19711-0">
    <skos:altLabel xml:lang="de">Internationale Tagung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19711-0">
    <skos:altLabel xml:lang="de">Gipfeltreffen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19711-0">
    <skos:altLabel xml:lang="de">Gipfelkonferenz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19711-0">
    <skos:altLabel xml:lang="de">Diplomatische Konferenz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19711-0">
    <skos:altLabel xml:lang="de">Außenministerkonferenz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15743-0">
    <skos:narrower rdf:resource="descriptor/19486-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15743-0">
    <skos:narrower rdf:resource="descriptor/15010-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14049-0">
    <skos:prefLabel xml:lang="en">Nitrogen fertilizer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14049-0">
    <skos:prefLabel xml:lang="de">Stickstoffdünger</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24704-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18489-1">
    <zbwext:useInsteadNote rdf:nodeID="id142"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14821-4">
    <skos:broader rdf:resource="thsys/70372"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14821-4">
    <skos:broader rdf:resource="thsys/70333"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14821-4">
    <skos:broader rdf:resource="descriptor/14051-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14821-4">
    <skos:broader rdf:resource="descriptor/14822-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18157-5">
    <skos:prefLabel xml:lang="en">Debt swap</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18157-5">
    <skos:prefLabel xml:lang="de">Schuldenswap</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14145-4">
    <skos:altLabel xml:lang="de">Schlachtvieh</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14145-4">
    <skos:altLabel xml:lang="de">Mule</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14145-4">
    <skos:altLabel xml:lang="de">Maultier</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14145-4">
    <skos:altLabel xml:lang="de">Maulesel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14145-4">
    <skos:altLabel xml:lang="de">Kamel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14145-4">
    <skos:altLabel xml:lang="de">Esel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12540-1">
    <skos:related rdf:resource="descriptor/11580-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12540-1">
    <skos:related rdf:resource="descriptor/12475-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12540-1">
    <skos:related rdf:resource="descriptor/10889-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18239-3">
    <skos:broader rdf:resource="thsys/70134"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18239-3">
    <skos:broader rdf:resource="descriptor/14302-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12354-5">
    <skos:scopeNote xml:lang="de">Nur für betriebswirtschaftliche Literatur.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19289-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10587-6">
    <skos:broader rdf:resource="thsys/73366"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10587-6">
    <skos:broader rdf:resource="thsys/71084"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10587-6">
    <skos:broader rdf:resource="thsys/70179"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10587-6">
    <skos:broader rdf:resource="descriptor/16441-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19452-5">
    <skos:prefLabel xml:lang="en">Urban location model</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19452-5">
    <skos:prefLabel xml:lang="de">Städtische Standorttheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12697-3">
    <skos:related rdf:resource="descriptor/10994-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12697-3">
    <skos:related rdf:resource="descriptor/19285-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12697-3">
    <skos:related rdf:resource="descriptor/11105-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12697-3">
    <skos:related rdf:resource="descriptor/11127-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12697-3">
    <skos:related rdf:resource="descriptor/11790-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12697-3">
    <skos:related rdf:resource="descriptor/10981-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12697-3">
    <skos:related rdf:resource="descriptor/19291-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12697-3">
    <skos:related rdf:resource="descriptor/12094-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12697-3">
    <skos:related rdf:resource="descriptor/19211-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12697-3">
    <skos:related rdf:resource="descriptor/18695-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12697-3">
    <skos:related rdf:resource="descriptor/19254-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70833">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15772-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14378-2">
    <skos:broader rdf:resource="thsys/70318"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14378-2">
    <skos:broader rdf:resource="descriptor/14381-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16306-3">
    <skos:altLabel xml:lang="de">Regierungssturz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16306-3">
    <skos:altLabel xml:lang="de">Regierungsbildung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16306-3">
    <skos:altLabel xml:lang="de">Neue Regierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13058-4">
    <skos:prefLabel xml:lang="en">Machinery industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13058-4">
    <skos:prefLabel xml:lang="de">Maschinenbau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13291-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18133-5">
    <skos:altLabel xml:lang="de">Vorpommern</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10328-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14976-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15889-0">
    <skos:broader rdf:resource="thsys/73351"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20491-6">
    <skos:prefLabel xml:lang="en">Homosexuality</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20491-6">
    <skos:prefLabel xml:lang="de">Homosexualität</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17985-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12380-4">
    <skos:related rdf:resource="descriptor/12677-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12380-4">
    <skos:related rdf:resource="descriptor/19068-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18413-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10065-1">
    <skos:broader rdf:resource="thsys/70049"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12993-6">
    <skos:broader rdf:resource="thsys/71005"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12993-6">
    <skos:broader rdf:resource="descriptor/12913-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18067-6">
    <skos:narrower rdf:resource="descriptor/18186-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18067-6">
    <skos:narrower rdf:resource="descriptor/16014-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18067-6">
    <skos:narrower rdf:resource="descriptor/16034-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18067-6">
    <skos:narrower rdf:resource="descriptor/19729-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11954-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12931-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13420-6">
    <skos:scopeNote xml:lang="de">Nur für die gesamtstaatliche Ebene.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18371-3">
    <skos:broader rdf:resource="thsys/71012"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18371-3">
    <skos:broader rdf:resource="thsys/70539"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18371-3">
    <skos:broader rdf:resource="descriptor/11810-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18371-3">
    <skos:broader rdf:resource="descriptor/18375-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11494-1">
    <skos:broader rdf:resource="thsys/70355"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11494-1">
    <skos:broader rdf:resource="descriptor/10382-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10239-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19829-5">
    <skos:prefLabel xml:lang="en">Industrial promotion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19829-5">
    <skos:prefLabel xml:lang="de">Wirtschaftsförderung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70993">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11677-0">
    <skos:broader rdf:resource="thsys/73294"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11677-0">
    <skos:broader rdf:resource="thsys/71074"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11677-0">
    <skos:broader rdf:resource="descriptor/19422-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11677-0">
    <skos:broader rdf:resource="descriptor/16479-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14421-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18348-5">
    <skos:narrower rdf:resource="descriptor/15628-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15931-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71077">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.09.07</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10629-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71032">
    <rdfs:label xml:lang="en">Central Asia</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71032">
    <rdfs:label xml:lang="de">Zentralasien</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18598-3">
    <skos:broader rdf:resource="thsys/70940"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18598-3">
    <skos:broader rdf:resource="thsys/70934"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18598-3">
    <skos:broader rdf:resource="descriptor/13553-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70823">
    <skos:broader rdf:resource="thsys/w"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19155-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11910-6">
    <skos:scopeNote xml:lang="de">Die Wirtschaftsstruktur einer Region analysiert im Verhältnis zu einer übergeordneten räumlichen Einheit.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id260">
    <rdf:value xml:lang="de">Für Preiselastizitäten im Außenhandel</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16708-4">
    <skos:prefLabel xml:lang="en">Epistemology</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16708-4">
    <skos:prefLabel xml:lang="de">Erkenntnistheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10024-1">
    <skos:altLabel xml:lang="de">Privatvermögen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13653-4">
    <skos:broader rdf:resource="thsys/70922"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13653-4">
    <skos:broader rdf:resource="descriptor/18170-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14129-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15446-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70872">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.22</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14531-6">
    <skos:prefLabel xml:lang="en">Precision instrument</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14531-6">
    <skos:prefLabel xml:lang="de">Feinmechanisches Produkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13057-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11393-0">
    <skos:scopeNote xml:lang="de">Erste Einführungen in das berufliche Arbeiten sowie die Vermittlung von Grundkenntnissen für bestimmte Berufsgruppen - schulisch, betrieblich, überbetrieblich.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12318-2">
    <skos:prefLabel xml:lang="en">Turnover</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12318-2">
    <skos:prefLabel xml:lang="de">Umsatz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14364-6">
    <skos:altLabel xml:lang="de">Windrad</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14364-6">
    <skos:altLabel xml:lang="de">Windenergieanlage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16605-0">
    <skos:related rdf:resource="descriptor/16645-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16605-0">
    <skos:related rdf:resource="descriptor/10300-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16605-0">
    <skos:related rdf:resource="descriptor/16603-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14227-2">
    <skos:narrower rdf:resource="descriptor/14225-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24782-3">
    <skos:prefLabel xml:lang="en">New economy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24782-3">
    <skos:prefLabel xml:lang="de">New Economy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16111-4">
    <skos:altLabel xml:lang="de">Verletzung des Steuergeheimnisses</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id174">
    <rdfs:seeAlso rdf:resource="descriptor/14734-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18969-1">
    <skos:related rdf:resource="descriptor/13398-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18969-1">
    <skos:related rdf:resource="descriptor/19774-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12998-3">
    <skos:altLabel xml:lang="en">Dye industry</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12998-3">
    <skos:altLabel xml:lang="de">Farbenindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12998-3">
    <skos:altLabel xml:lang="de">Farben- und Lackindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18969-1">
    <skos:prefLabel xml:lang="en">Cohabitation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18969-1">
    <skos:prefLabel xml:lang="de">Lebensgemeinschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10603-6">
    <skos:related rdf:resource="descriptor/16524-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10603-6">
    <skos:related rdf:resource="descriptor/10573-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10603-6">
    <skos:related rdf:resource="descriptor/10555-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18461-2">
    <skos:broader rdf:resource="thsys/71109"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18461-2">
    <skos:broader rdf:resource="thsys/71005"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18461-2">
    <skos:broader rdf:resource="thsys/70878"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18461-2">
    <skos:broader rdf:resource="descriptor/10443-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13865-3">
    <skos:broader rdf:resource="thsys/70888"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13865-3">
    <skos:broader rdf:resource="descriptor/13860-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13865-3">
    <skos:broader rdf:resource="descriptor/10032-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14364-6">
    <skos:related rdf:resource="descriptor/18231-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14364-6">
    <skos:related rdf:resource="descriptor/18346-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11702-6">
    <skos:related rdf:resource="descriptor/11572-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11702-6">
    <skos:related rdf:resource="descriptor/19533-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11702-6">
    <skos:related rdf:resource="descriptor/19941-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14955-4">
    <skos:altLabel xml:lang="en">Dairy products</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14955-4">
    <skos:altLabel xml:lang="de">Sahne</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14955-4">
    <skos:altLabel xml:lang="de">Rahm</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14955-4">
    <skos:altLabel xml:lang="de">Molkereiprodukt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14955-4">
    <skos:altLabel xml:lang="de">Milchpulver</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14955-4">
    <skos:altLabel xml:lang="de">Milcherzeugnis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14955-4">
    <skos:altLabel xml:lang="de">Meiereiprodukt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14955-4">
    <skos:altLabel xml:lang="de">Kondensmilch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14955-4">
    <skos:altLabel xml:lang="de">Dosenmilch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16304-0">
    <skos:prefLabel xml:lang="en">Political imprisonment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16304-0">
    <skos:prefLabel xml:lang="de">Politische Haft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12016-4">
    <skos:narrower rdf:resource="descriptor/19484-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13193-5">
    <skos:broader rdf:resource="thsys/70957"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13193-5">
    <skos:broader rdf:resource="descriptor/13198-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14042-0">
    <skos:scopeNote xml:lang="de">Medizinprodukte sind keine inhaltlich genau abgegrenzte Gütergruppe.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11451-5">
    <skos:altLabel xml:lang="en">Near-money</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11451-5">
    <skos:altLabel xml:lang="de">Quasigeld</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11451-5">
    <skos:altLabel xml:lang="de">Monetäres Surrogat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11451-5">
    <skos:altLabel xml:lang="de">Geldnahe Forderungen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10947-2">
    <skos:related rdf:resource="descriptor/18066-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10947-2">
    <skos:related rdf:resource="descriptor/10478-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10947-2">
    <skos:related rdf:resource="descriptor/18082-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10947-2">
    <skos:related rdf:resource="descriptor/10459-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73302">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15941-3">
    <skos:prefLabel xml:lang="en">Fertility</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15941-3">
    <skos:prefLabel xml:lang="de">Fruchtbarkeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19601-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18414-4">
    <skos:prefLabel xml:lang="en">Robot industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18414-4">
    <skos:prefLabel xml:lang="de">Roboterindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id206">
    <rdf:value xml:lang="de">Für Untersuchungen zum Medienverhalten</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71093">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14240-3">
    <skos:altLabel xml:lang="de">Quarzsand</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14240-3">
    <skos:altLabel xml:lang="de">Quarzit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14240-3">
    <skos:altLabel xml:lang="de">Quarz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14240-3">
    <skos:altLabel xml:lang="de">Kieselgur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14240-3">
    <skos:altLabel xml:lang="de">Kies</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14240-3">
    <skos:altLabel xml:lang="de">Diatomeenerde</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18166-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18213-0">
    <skos:prefLabel xml:lang="en">Snack food</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18213-0">
    <skos:prefLabel xml:lang="de">Snackartikel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19744-6">
    <skos:altLabel xml:lang="en">Multidisciplinary research</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73302">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13575-5">
    <skos:prefLabel xml:lang="en">Aeronautical engineering</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13575-5">
    <skos:prefLabel xml:lang="de">Luftfahrttechnik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15814-3">
    <skos:broader rdf:resource="thsys/72182"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15814-3">
    <skos:broader rdf:resource="thsys/70504"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15814-3">
    <skos:broader rdf:resource="thsys/70011"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15814-3">
    <skos:broader rdf:resource="descriptor/15064-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19251-1">
    <skos:broader rdf:resource="thsys/73372"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19251-1">
    <skos:broader rdf:resource="thsys/71109"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19251-1">
    <skos:broader rdf:resource="thsys/70406"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19251-1">
    <skos:broader rdf:resource="descriptor/11820-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13848-3">
    <skos:broader rdf:resource="thsys/70967"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13848-3">
    <skos:broader rdf:resource="thsys/70888"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13848-3">
    <skos:broader rdf:resource="descriptor/18505-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15989-3">
    <skos:prefLabel xml:lang="en">Alien law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15989-3">
    <skos:prefLabel xml:lang="de">Ausländerrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18790-4">
    <skos:broader rdf:resource="thsys/73333"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18790-4">
    <skos:broader rdf:resource="thsys/71097"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18790-4">
    <skos:broader rdf:resource="descriptor/15977-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18790-4">
    <skos:broader rdf:resource="descriptor/16032-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18790-4">
    <skos:broader rdf:resource="descriptor/19851-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10589-2">
    <skos:related rdf:resource="descriptor/13395-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10589-2">
    <skos:related rdf:resource="descriptor/20817-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10589-2">
    <skos:related rdf:resource="descriptor/10558-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13367-5">
    <skos:narrower rdf:resource="descriptor/18449-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13367-5">
    <skos:narrower rdf:resource="descriptor/13365-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18993-4">
    <skos:related rdf:resource="descriptor/18093-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18993-4">
    <skos:related rdf:resource="descriptor/19464-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18993-4">
    <skos:related rdf:resource="descriptor/12917-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18993-4">
    <skos:related rdf:resource="descriptor/11197-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18993-4">
    <skos:related rdf:resource="descriptor/12910-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18993-4">
    <skos:related rdf:resource="descriptor/19459-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18672-3">
    <skos:broader rdf:resource="thsys/70905"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18672-3">
    <skos:broader rdf:resource="descriptor/18085-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11168-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10271-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13801-1">
    <skos:related rdf:resource="descriptor/24706-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13801-1">
    <skos:related rdf:resource="descriptor/18672-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13801-1">
    <skos:related rdf:resource="descriptor/12251-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13801-1">
    <skos:related rdf:resource="descriptor/18271-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11299-6">
    <skos:related rdf:resource="descriptor/15716-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11299-6">
    <skos:related rdf:resource="descriptor/12588-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11299-6">
    <skos:related rdf:resource="descriptor/11294-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11299-6">
    <skos:related rdf:resource="descriptor/19005-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11299-6">
    <skos:related rdf:resource="descriptor/11389-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11299-6">
    <skos:related rdf:resource="descriptor/11184-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70270">
    <skos:broader rdf:resource="thsys/70030"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13829-0">
    <skos:narrower rdf:resource="descriptor/13832-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13829-0">
    <skos:narrower rdf:resource="descriptor/18751-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13829-0">
    <skos:narrower rdf:resource="descriptor/13839-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19485-4">
    <skos:broader rdf:resource="thsys/73351"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19485-4">
    <skos:broader rdf:resource="thsys/70238"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70091">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.08</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11736-3">
    <skos:prefLabel xml:lang="en">Functional income distribution</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11736-3">
    <skos:prefLabel xml:lang="de">Funktionelle Einkommensverteilung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14161-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16214-1">
    <skos:broader rdf:resource="thsys/70878"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16214-1">
    <skos:broader rdf:resource="thsys/70095"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16214-1">
    <skos:broader rdf:resource="descriptor/10436-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12607-2">
    <skos:prefLabel xml:lang="en">Workplace performance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12607-2">
    <skos:prefLabel xml:lang="de">Arbeitsgestaltung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19221-3">
    <skos:related rdf:resource="descriptor/19830-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19221-3">
    <skos:related rdf:resource="descriptor/19839-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19221-3">
    <skos:related rdf:resource="descriptor/18097-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19221-3">
    <skos:related rdf:resource="descriptor/11568-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19221-3">
    <skos:related rdf:resource="descriptor/10150-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18416-0">
    <skos:broader rdf:resource="thsys/70999"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18416-0">
    <skos:broader rdf:resource="descriptor/13066-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18416-0">
    <skos:broader rdf:resource="descriptor/12923-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19628-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#relatedMatch">
    <skos:definition xml:lang="en">skos:relatedMatch is used to state an associative mapping link
      between two conceptual resources in different concept schemes.</skos:definition>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18874-5">
    <skos:altLabel xml:lang="de">Hochschulhaushalt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18874-5">
    <skos:altLabel xml:lang="de">Hochschulfinanzen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18874-5">
    <skos:altLabel xml:lang="de">Hochschuletat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19080-2">
    <skos:narrower rdf:resource="descriptor/19081-0"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id159">
    <rdf:value xml:lang="de">Für die Investitionspolitik von Unternehmen</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73344">
    <rdfs:label xml:lang="de">Schlüsseltechnologien</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16742-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18961-3">
    <skos:prefLabel xml:lang="en">EU waste law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18961-3">
    <skos:prefLabel xml:lang="de">EU-Abfallrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14792-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id180">
    <rdfs:seeAlso rdf:resource="descriptor/14040-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71029">
    <skos:prefLabel xml:lang="en">W.01.01  Plant production</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71029">
    <skos:prefLabel xml:lang="de">W.01.01  Pflanzenbau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73296">
    <skos:narrower rdf:resource="descriptor/13369-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73296">
    <skos:narrower rdf:resource="descriptor/13368-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73296">
    <skos:narrower rdf:resource="descriptor/13367-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73296">
    <skos:narrower rdf:resource="descriptor/16037-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73296">
    <skos:narrower rdf:resource="descriptor/14004-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73296">
    <skos:narrower rdf:resource="descriptor/18449-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73296">
    <skos:narrower rdf:resource="descriptor/16010-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73296">
    <skos:narrower rdf:resource="descriptor/13662-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73296">
    <skos:narrower rdf:resource="descriptor/15657-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73296">
    <skos:narrower rdf:resource="descriptor/13382-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73296">
    <skos:narrower rdf:resource="descriptor/13365-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15810-4">
    <skos:scopeNote xml:lang="de">Der Sozialisationsprozess umfasst die Gesamtheit der Phasen, durch die ein Mensch zur gesellschaftlich handlungsfähigen Person wird.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id249">
    <rdfs:seeAlso rdf:resource="descriptor/12571-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18332-6">
    <skos:altLabel xml:lang="en">CAP (Common Agricultural Policy)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18332-6">
    <skos:altLabel xml:lang="de">Gemeinsame Agrarpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18332-6">
    <skos:altLabel xml:lang="de">GAP (Gemeinsame Agrarpolitik)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18332-6">
    <skos:altLabel xml:lang="de">EG-Agrarpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12577-6">
    <skos:altLabel xml:lang="en">Labour-management relations</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12577-6">
    <skos:altLabel xml:lang="en">Labor-management relations</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12577-6">
    <skos:altLabel xml:lang="en">Labor management relations</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12577-6">
    <skos:altLabel xml:lang="en">Human relations</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12577-6">
    <skos:altLabel xml:lang="de">Soziale Beziehungen im Betrieb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12577-6">
    <skos:altLabel xml:lang="de">Gruppenbeziehungen im Betrieb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12577-6">
    <skos:altLabel xml:lang="de">Betriebliche Arbeitsbeziehungen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12577-6">
    <skos:altLabel xml:lang="de">Arbeitsklima</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15761-5">
    <skos:related rdf:resource="descriptor/15769-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15761-5">
    <skos:related rdf:resource="descriptor/19662-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10248-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15009-0">
    <skos:narrower rdf:resource="descriptor/15017-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15009-0">
    <skos:narrower rdf:resource="descriptor/15011-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15009-0">
    <skos:narrower rdf:resource="descriptor/14273-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15009-0">
    <skos:narrower rdf:resource="descriptor/18230-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15009-0">
    <skos:narrower rdf:resource="descriptor/15019-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15009-0">
    <skos:narrower rdf:resource="descriptor/18204-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15009-0">
    <skos:narrower rdf:resource="descriptor/15018-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15009-0">
    <skos:narrower rdf:resource="descriptor/14796-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15009-0">
    <skos:narrower rdf:resource="descriptor/15729-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15009-0">
    <skos:narrower rdf:resource="descriptor/15010-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18458-5">
    <skos:broader rdf:resource="thsys/71012"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18458-5">
    <skos:broader rdf:resource="thsys/71005"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18458-5">
    <skos:broader rdf:resource="thsys/70539"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18458-5">
    <skos:broader rdf:resource="descriptor/11812-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10917-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16955-1">
    <skos:scopeNote xml:lang="de">Für Literatur zur Region Wien BENUTZE: Wien (Region).</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16955-1">
    <skos:scopeNote xml:lang="de">Literatur zu Hauptstadt und Bundesland Wien.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19614-5">
    <skos:related rdf:resource="descriptor/14161-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19614-5">
    <skos:related rdf:resource="descriptor/19631-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19614-5">
    <skos:related rdf:resource="descriptor/15891-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19614-5">
    <skos:related rdf:resource="descriptor/15887-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19614-5">
    <skos:related rdf:resource="descriptor/12967-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19614-5">
    <skos:related rdf:resource="descriptor/12966-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73360">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.05.01.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14345-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11141-6">
    <skos:broader rdf:resource="thsys/71101"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11141-6">
    <skos:broader rdf:resource="descriptor/11090-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15599-2">
    <skos:altLabel xml:lang="en">Distributed data processing system</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15599-2">
    <skos:altLabel xml:lang="de">Verteiltes Datenbanksystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15599-2">
    <skos:altLabel xml:lang="de">Verteiltes Computersystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15599-2">
    <skos:altLabel xml:lang="de">Rechnerverbundsystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15599-2">
    <skos:altLabel xml:lang="de">Rechnernetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15599-2">
    <skos:altLabel xml:lang="de">Heterogenes Computernetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15599-2">
    <skos:altLabel xml:lang="de">Dezentrale Datenverarbeitung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15599-2">
    <skos:altLabel xml:lang="de">Computerverbund</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12874-0">
    <skos:altLabel xml:lang="en">Intercorporate privilege</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13814-6">
    <skos:related rdf:resource="descriptor/18607-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13814-6">
    <skos:related rdf:resource="descriptor/13247-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13814-6">
    <skos:related rdf:resource="descriptor/13246-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13814-6">
    <skos:related rdf:resource="descriptor/13057-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13814-6">
    <skos:related rdf:resource="descriptor/18604-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11782-3">
    <skos:narrower rdf:resource="descriptor/18308-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11782-3">
    <skos:narrower rdf:resource="descriptor/11910-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11782-3">
    <skos:narrower rdf:resource="descriptor/11787-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11782-3">
    <skos:narrower rdf:resource="descriptor/10905-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11782-3">
    <skos:narrower rdf:resource="descriptor/11821-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11782-3">
    <skos:narrower rdf:resource="descriptor/11824-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11782-3">
    <skos:narrower rdf:resource="descriptor/10522-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11782-3">
    <skos:narrower rdf:resource="descriptor/11783-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11782-3">
    <skos:narrower rdf:resource="descriptor/11803-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10206-2">
    <skos:altLabel xml:lang="de">Nachfragemonopol</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18955-5">
    <skos:altLabel xml:lang="de">Sportökonomie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10142-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11539-5">
    <skos:narrower rdf:resource="descriptor/19941-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16824-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10561-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13682-4">
    <skos:related rdf:resource="descriptor/13577-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13682-4">
    <skos:related rdf:resource="descriptor/18663-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13682-4">
    <skos:related rdf:resource="descriptor/18649-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13682-4">
    <skos:related rdf:resource="descriptor/18428-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13682-4">
    <skos:related rdf:resource="descriptor/18651-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13682-4">
    <skos:related rdf:resource="descriptor/13278-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13682-4">
    <skos:related rdf:resource="descriptor/14490-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13682-4">
    <skos:related rdf:resource="descriptor/16757-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18073-4">
    <skos:broader rdf:resource="thsys/70171"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19048-5">
    <skos:narrower rdf:resource="descriptor/14193-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13777-0">
    <skos:prefLabel xml:lang="en">Credit risk</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13777-0">
    <skos:prefLabel xml:lang="de">Kreditrisiko</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71042">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10692-2">
    <skos:narrower rdf:resource="descriptor/18818-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10692-2">
    <skos:narrower rdf:resource="descriptor/19414-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10692-2">
    <skos:narrower rdf:resource="descriptor/10691-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71066">
    <skos:prefLabel xml:lang="de">V.11.04  Grund und Boden</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18847-1">
    <skos:altLabel xml:lang="de">Verbandsmitgliedschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18847-1">
    <skos:altLabel xml:lang="de">Mitgliederstruktur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19056-6">
    <skos:broader rdf:resource="thsys/70969"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19056-6">
    <skos:broader rdf:resource="descriptor/12905-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18638-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11716-2">
    <skos:altLabel xml:lang="en">Social overhead capital</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11716-2">
    <skos:altLabel xml:lang="en">Public capital</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11716-2">
    <skos:altLabel xml:lang="en">Overhead capital</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11716-2">
    <skos:altLabel xml:lang="de">Öffentliches Kapital</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11716-2">
    <skos:altLabel xml:lang="de">Materielle Infrastruktur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11716-2">
    <skos:altLabel xml:lang="de">Infrastrukturversorgung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19011-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18441-1">
    <skos:broader rdf:resource="thsys/73331"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18441-1">
    <skos:broader rdf:resource="thsys/70986"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18441-1">
    <skos:broader rdf:resource="descriptor/16004-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18441-1">
    <skos:broader rdf:resource="descriptor/16182-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12905-1">
    <skos:broader rdf:resource="thsys/70970"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17503-3">
    <skos:broader rdf:resource="thsys/71032"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17503-3">
    <skos:broader rdf:resource="descriptor/17595-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17503-3">
    <skos:broader rdf:resource="descriptor/19078-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14483-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19966-2">
    <skos:related rdf:resource="descriptor/18083-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19966-2">
    <skos:related rdf:resource="descriptor/11322-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19966-2">
    <skos:related rdf:resource="descriptor/13871-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13667-0">
    <skos:prefLabel xml:lang="en">Telephone</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13667-0">
    <skos:prefLabel xml:lang="de">Telefon</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10945-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11369-4">
    <skos:altLabel xml:lang="en">Open university</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11369-4">
    <skos:altLabel xml:lang="de">Volkshochschule</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11369-4">
    <skos:altLabel xml:lang="de">Seniorenuniversität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11369-4">
    <skos:altLabel xml:lang="de">Abendgymnasium</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70199">
    <skos:broader rdf:resource="thsys/70014"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11715-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19208-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id323">
    <rdf:value xml:lang="de">Für die Analyse der Arbeitsplatzsuche</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17934-4">
    <skos:altLabel xml:lang="de">Republik Venezuela</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18597-5">
    <skos:altLabel xml:lang="de">Seegüterverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18989-2">
    <skos:related rdf:resource="descriptor/19487-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18989-2">
    <skos:related rdf:resource="descriptor/11811-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18989-2">
    <skos:related rdf:resource="descriptor/18350-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18989-2">
    <skos:related rdf:resource="descriptor/18996-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18989-2">
    <skos:related rdf:resource="descriptor/18347-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15104-6">
    <skos:narrower rdf:resource="descriptor/15093-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14109-1">
    <skos:prefLabel xml:lang="en">Nut</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14109-1">
    <skos:prefLabel xml:lang="de">Nuss</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13153-3">
    <skos:prefLabel xml:lang="en">Oil and fat industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13153-3">
    <skos:prefLabel xml:lang="de">Öl- und Fettindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11426-4">
    <skos:altLabel xml:lang="en">Circulation of money</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11426-4">
    <skos:altLabel xml:lang="de">Münzumlauf</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11426-4">
    <skos:altLabel xml:lang="de">Currency circulation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11426-4">
    <skos:altLabel xml:lang="de">Bargeldumlauf</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11426-4">
    <skos:altLabel xml:lang="de">Bargeld</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13535-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19742-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10736-1">
    <skos:altLabel xml:lang="en">Floating</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71044">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16028-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14068-3">
    <skos:related rdf:resource="descriptor/14083-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14068-3">
    <skos:related rdf:resource="descriptor/14728-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14068-3">
    <skos:related rdf:resource="descriptor/14060-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19415-4">
    <skos:broader rdf:resource="thsys/71075"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19415-4">
    <skos:broader rdf:resource="thsys/71074"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19415-4">
    <skos:broader rdf:resource="descriptor/11650-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72184">
    <rdfs:label xml:lang="de">Indexrechnung</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10805-1">
    <skos:prefLabel xml:lang="en">International finance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10805-1">
    <skos:prefLabel xml:lang="de">Internationale Finanzierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70859">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14723-4">
    <skos:altLabel xml:lang="de">Rauschmittel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14723-4">
    <skos:altLabel xml:lang="de">Rauschgift</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14723-4">
    <skos:altLabel xml:lang="de">Opium</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14723-4">
    <skos:altLabel xml:lang="de">Opiat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14723-4">
    <skos:altLabel xml:lang="de">Morphium</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14723-4">
    <skos:altLabel xml:lang="de">Marihuana</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14723-4">
    <skos:altLabel xml:lang="de">Heroin</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14723-4">
    <skos:altLabel xml:lang="de">Haschisch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14723-4">
    <skos:altLabel xml:lang="de">Halluzinogen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14723-4">
    <skos:altLabel xml:lang="de">Betäubungsmittel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19655-5">
    <skos:broader rdf:resource="thsys/70244"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19655-5">
    <skos:broader rdf:resource="descriptor/12085-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19260-0">
    <skos:scopeNote xml:lang="de">Währungsinstitution, die statt einer Zentralbank nationales Geld ausschließlich durch Ankauf einer Reservewährung schafft; unabhängig von Geldpolitik; meist in Staaten, deren Währung kein Vertrauen aufbauen konnte; historisch z.B. im britischen Empire, heute in den Staaten Osteuropas diskutiert.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16048-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11727-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12354-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17565-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19428-2">
    <skos:related rdf:resource="descriptor/20241-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19428-2">
    <skos:related rdf:resource="descriptor/16595-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19428-2">
    <skos:related rdf:resource="descriptor/12665-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18809-2">
    <skos:scopeNote xml:lang="de">Rechtliche Vorschriften von Teilstaaten innerhalb eines Bundesstaates</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11182-6">
    <skos:narrower rdf:resource="descriptor/11216-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11182-6">
    <skos:narrower rdf:resource="descriptor/11210-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11182-6">
    <skos:narrower rdf:resource="descriptor/19471-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11182-6">
    <skos:narrower rdf:resource="descriptor/19439-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11182-6">
    <skos:narrower rdf:resource="descriptor/18124-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11182-6">
    <skos:narrower rdf:resource="descriptor/19461-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11182-6">
    <skos:narrower rdf:resource="descriptor/10339-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11182-6">
    <skos:narrower rdf:resource="descriptor/11200-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11182-6">
    <skos:narrower rdf:resource="descriptor/11213-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11182-6">
    <skos:narrower rdf:resource="descriptor/19498-2"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id178">
    <rdfs:seeAlso rdf:resource="descriptor/16267-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10301-1">
    <skos:related rdf:resource="descriptor/18395-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10301-1">
    <skos:related rdf:resource="descriptor/12918-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10301-1">
    <skos:related rdf:resource="descriptor/18522-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10301-1">
    <skos:related rdf:resource="descriptor/10010-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16195-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16166-0">
    <skos:related rdf:resource="descriptor/13255-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16166-0">
    <skos:related rdf:resource="descriptor/13525-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16166-0">
    <skos:related rdf:resource="descriptor/18575-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16487-1">
    <skos:prefLabel xml:lang="en">Science policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16487-1">
    <skos:prefLabel xml:lang="de">Wissenschaftspolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14396-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18277-2">
    <skos:prefLabel xml:lang="en">Furnishings</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18277-2">
    <skos:prefLabel xml:lang="de">Wohnungseinrichtung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18882-6">
    <skos:broader rdf:resource="thsys/73362"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18882-6">
    <skos:broader rdf:resource="descriptor/18889-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18882-6">
    <skos:broader rdf:resource="descriptor/11392-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13413-3">
    <skos:narrower rdf:resource="descriptor/10774-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13413-3">
    <skos:narrower rdf:resource="descriptor/15762-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13413-3">
    <skos:narrower rdf:resource="descriptor/18161-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13413-3">
    <skos:narrower rdf:resource="descriptor/13419-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13413-3">
    <skos:narrower rdf:resource="descriptor/18808-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13413-3">
    <skos:narrower rdf:resource="descriptor/11541-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13413-3">
    <skos:narrower rdf:resource="descriptor/16328-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13413-3">
    <skos:narrower rdf:resource="descriptor/11020-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13413-3">
    <skos:narrower rdf:resource="descriptor/24783-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13413-3">
    <skos:narrower rdf:resource="descriptor/16550-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13413-3">
    <skos:narrower rdf:resource="descriptor/15962-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13413-3">
    <skos:narrower rdf:resource="descriptor/11536-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13413-3">
    <skos:narrower rdf:resource="descriptor/18810-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13413-3">
    <skos:narrower rdf:resource="descriptor/11237-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11587-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73374">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.04.03</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11962-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17985-1">
    <skos:narrower rdf:resource="descriptor/17616-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18574-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id117">
    <rdfs:seeAlso rdf:resource="descriptor/11033-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18329-2">
    <skos:prefLabel xml:lang="en">Agricultural real estate</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18329-2">
    <skos:prefLabel xml:lang="de">Landwirtschaftliche Immobilien</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11553-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18921-1">
    <skos:related rdf:resource="descriptor/18955-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18921-1">
    <skos:related rdf:resource="descriptor/18928-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18921-1">
    <skos:related rdf:resource="descriptor/14040-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18921-1">
    <skos:related rdf:resource="descriptor/18920-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18921-1">
    <skos:related rdf:resource="descriptor/18923-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18921-1">
    <skos:related rdf:resource="descriptor/18922-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18921-1">
    <skos:related rdf:resource="descriptor/18917-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11687-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11335-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21101-5">
    <skos:scopeNote xml:lang="de">Vgl. auch Währungsreform.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21101-5">
    <skos:scopeNote xml:lang="de">Anders als eine Währungsreform gilt hier eine Währungsumstellung als Übergang auf eine neu geschaffene Währung in einem Staat oder einem größeren Währungsgebiet, z.B. Einführung des Euro oder eigener Währungen in den neuen Staaten Osteuropas nach dem Zusammenbruch der Sowjetunion.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11330-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15519-5">
    <skos:prefLabel xml:lang="en">Transportation problem</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15519-5">
    <skos:prefLabel xml:lang="de">Transportproblem</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16792-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13888-5">
    <skos:altLabel xml:lang="de">Gummiherstellerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13888-5">
    <skos:altLabel xml:lang="de">Gummihersteller</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13888-5">
    <skos:altLabel xml:lang="de">Chemieindustriearbeiterinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13888-5">
    <skos:altLabel xml:lang="de">Chemieindustriearbeiter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13888-5">
    <skos:altLabel xml:lang="de">Chemiefacharbeiterinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13888-5">
    <skos:altLabel xml:lang="de">Chemiefacharbeiter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13888-5">
    <skos:altLabel xml:lang="de">Chemiearbeiterinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13888-5">
    <skos:altLabel xml:lang="de">Chemiearbeiter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15946-0">
    <skos:broader rdf:resource="thsys/70182"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15946-0">
    <skos:broader rdf:resource="descriptor/15949-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18634-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19564-1">
    <skos:narrower rdf:resource="descriptor/18557-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19564-1">
    <skos:narrower rdf:resource="descriptor/18558-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18081-5">
    <skos:prefLabel xml:lang="en">Mobile telephone</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18081-5">
    <skos:prefLabel xml:lang="de">Mobiltelefon</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19275-1">
    <skos:scopeNote xml:lang="de">Ungesicherte Geldmarktpapiere, die vorwiegend von erstklassigen Industrieunternehmen und Finanzierungsgesellschaften ausgestellt werden.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11219-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15477-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18836-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16187-6">
    <skos:altLabel xml:lang="en">LOS (Law of the Sea)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16187-6">
    <skos:altLabel xml:lang="en">Law of the sea</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16187-6">
    <skos:altLabel xml:lang="de">Seevölkerrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16187-6">
    <skos:altLabel xml:lang="de">Seeschifffahrtsrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16187-6">
    <skos:altLabel xml:lang="de">Seeschiffahrtsrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16187-6">
    <skos:altLabel xml:lang="de">Seerechtskonvention</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16187-6">
    <skos:altLabel xml:lang="de">Seerechtsabkommen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16187-6">
    <skos:altLabel xml:lang="de">Recht der See</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16187-6">
    <skos:altLabel xml:lang="de">Meeresfreiheit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16187-6">
    <skos:altLabel xml:lang="de">Freiheit der Meere</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16187-6">
    <skos:altLabel xml:lang="de">Flaggenrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11182-6">
    <skos:prefLabel xml:lang="en">Unemployment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11182-6">
    <skos:prefLabel xml:lang="de">Arbeitslosigkeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id113">
    <rdfs:seeAlso rdf:resource="descriptor/12209-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14741-2">
    <skos:broader rdf:resource="thsys/70536"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14741-2">
    <skos:broader rdf:resource="descriptor/14599-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11129-3">
    <skos:broader rdf:resource="thsys/71101"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10162-3">
    <skos:prefLabel xml:lang="en">Revealed preferences</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10162-3">
    <skos:prefLabel xml:lang="de">Offenbarte Präferenzen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16698-2">
    <skos:related rdf:resource="descriptor/19757-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16698-2">
    <skos:related rdf:resource="descriptor/10173-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16698-2">
    <skos:related rdf:resource="descriptor/15938-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16698-2">
    <skos:related rdf:resource="descriptor/15937-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10339-4">
    <skos:prefLabel xml:lang="en">Cyclical unemployment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10339-4">
    <skos:prefLabel xml:lang="de">Konjunkturelle Arbeitslosigkeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18141-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id91">
    <rdf:value xml:lang="de">für die Entwicklungspolitik unterentwickelter Regionen in Industrieländern</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16454-2">
    <skos:related rdf:resource="descriptor/16463-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16454-2">
    <skos:related rdf:resource="descriptor/16395-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16454-2">
    <skos:related rdf:resource="descriptor/16521-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16454-2">
    <skos:related rdf:resource="descriptor/16455-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16454-2">
    <skos:related rdf:resource="descriptor/16427-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11460-4">
    <skos:prefLabel xml:lang="en">Aggregate liquidity</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11460-4">
    <skos:prefLabel xml:lang="de">Gesamtwirtschaftliche Liquidität</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11464-3">
    <skos:narrower rdf:resource="descriptor/11448-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11464-3">
    <skos:narrower rdf:resource="descriptor/11418-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21014-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11426-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20553-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11784-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18481-3">
    <skos:prefLabel xml:lang="en">Industrialization policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18481-3">
    <skos:prefLabel xml:lang="de">Industrialisierungspolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13688-6">
    <skos:altLabel xml:lang="de">Krise des Bankensektors</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12951-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13608-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12876-3">
    <skos:narrower rdf:resource="descriptor/12446-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10075-5">
    <skos:altLabel xml:lang="de">Stabiles Gleichgewicht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10075-5">
    <skos:altLabel xml:lang="de">Labiles Gleichgewicht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70835">
    <skos:prefLabel xml:lang="de">W.26  Sozialwesen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70458">
    <skos:broader rdf:resource="thsys/70504"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18230-0">
    <skos:altLabel xml:lang="de">Plastikmüll</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18230-0">
    <skos:altLabel xml:lang="de">Kunststoffmüll</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15801-5">
    <skos:related rdf:resource="descriptor/12837-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15801-5">
    <skos:related rdf:resource="descriptor/18859-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15801-5">
    <skos:related rdf:resource="descriptor/19533-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15801-5">
    <skos:related rdf:resource="descriptor/15786-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15801-5">
    <skos:related rdf:resource="descriptor/10300-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12087-2">
    <skos:prefLabel xml:lang="en">Management game</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12087-2">
    <skos:prefLabel xml:lang="de">Planspiel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11938-5">
    <skos:altLabel xml:lang="de">Gemeinschaftsaufgabe "Verbesserung der Agrarstruktur und des Küstenschutzes"</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11938-5">
    <skos:altLabel xml:lang="de">GAK (Gemeinschaftsaufgabe Agrarstruktur und Küstenschutz)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id15">
    <rdf:value xml:lang="de">Für nationale Anpassungsprogramme</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16312-1">
    <skos:altLabel xml:lang="de">Zentralismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16312-1">
    <skos:altLabel xml:lang="de">Zentralisierung der Staatstätigkeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16312-1">
    <skos:altLabel xml:lang="de">Staatlicher Zentralismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16312-1">
    <skos:altLabel xml:lang="de">Einheitsstaat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14407-0">
    <skos:narrower rdf:resource="descriptor/18274-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19025-3">
    <skos:altLabel xml:lang="de">Prozessbetrachtung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19025-3">
    <skos:altLabel xml:lang="de">Prozeßbetrachtung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18748-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12855-4">
    <skos:prefLabel xml:lang="en">Hidden profit distribution</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12855-4">
    <skos:prefLabel xml:lang="de">Verdeckte Gewinnausschüttung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18752-5">
    <skos:narrower rdf:resource="descriptor/11563-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12356-1">
    <skos:narrower rdf:resource="descriptor/21061-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12356-1">
    <skos:narrower rdf:resource="descriptor/12399-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12356-1">
    <skos:narrower rdf:resource="descriptor/12354-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15051-1">
    <skos:related rdf:resource="descriptor/12154-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15051-1">
    <skos:related rdf:resource="descriptor/12765-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16601-1">
    <skos:prefLabel xml:lang="en">Service society</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16601-1">
    <skos:prefLabel xml:lang="de">Dienstleistungsgesellschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13428-4">
    <skos:related rdf:resource="descriptor/18314-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13428-4">
    <skos:related rdf:resource="descriptor/14931-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13428-4">
    <skos:related rdf:resource="descriptor/18334-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13428-4">
    <skos:related rdf:resource="descriptor/19286-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13428-4">
    <skos:related rdf:resource="descriptor/11801-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12749-3">
    <skos:altLabel xml:lang="de">Produktionsdurchlaufzeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12749-3">
    <skos:altLabel xml:lang="de">Fertigungsdurchlaufzeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19131-4">
    <skos:broader rdf:resource="thsys/73391"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19131-4">
    <skos:broader rdf:resource="thsys/70384"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19131-4">
    <skos:broader rdf:resource="descriptor/12846-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18733-2">
    <skos:related rdf:resource="descriptor/13279-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18074-2">
    <skos:related rdf:resource="descriptor/11371-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19234-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16439-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14269-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13666-2">
    <skos:broader rdf:resource="thsys/71037"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13666-2">
    <skos:broader rdf:resource="thsys/70922"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13666-2">
    <skos:broader rdf:resource="descriptor/13277-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13666-2">
    <skos:broader rdf:resource="descriptor/18655-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13666-2">
    <skos:broader rdf:resource="descriptor/19151-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12396-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18191-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13357-1">
    <skos:prefLabel xml:lang="en">Museum</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13357-1">
    <skos:prefLabel xml:lang="de">Museum</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13431-1">
    <skos:scopeNote xml:lang="de">Nur für Literatur über den Agrarmarkt allgemein.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18004-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id212">
    <rdf:value xml:lang="de">und für die Meinungsforschung von Verbrauchern und Unternehmen zu Produkten, Märkten, etc.</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15057-3">
    <skos:altLabel xml:lang="de">Simulationstechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15057-3">
    <skos:altLabel xml:lang="de">Simulationsrechnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15057-3">
    <skos:altLabel xml:lang="de">Simulationsmodell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15057-3">
    <skos:altLabel xml:lang="de">Computersimulation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12331-3">
    <skos:narrower rdf:resource="descriptor/24666-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13660-0">
    <skos:altLabel xml:lang="de">Mediengesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14053-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19541-6">
    <skos:broader rdf:resource="thsys/70922"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19541-6">
    <skos:broader rdf:resource="thsys/70459"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19541-6">
    <skos:broader rdf:resource="descriptor/12665-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19541-6">
    <skos:broader rdf:resource="descriptor/13301-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15969-2">
    <skos:prefLabel xml:lang="en">Philosophy of law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15969-2">
    <skos:prefLabel xml:lang="de">Rechtsphilosophie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18545-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15793-6">
    <skos:altLabel xml:lang="de">Wahrnehmungspsychologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15793-6">
    <skos:altLabel xml:lang="de">Wahrnehmungsmessung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15793-6">
    <skos:altLabel xml:lang="de">Unterschwellige Wahrnehmung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15793-6">
    <skos:altLabel xml:lang="de">Selektive Wahrnehmung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15793-6">
    <skos:altLabel xml:lang="de">Produktwahrnehmung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15793-6">
    <skos:altLabel xml:lang="de">Methode der Blickregistrierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15793-6">
    <skos:altLabel xml:lang="de">Interpersonale Wahrnehmung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15793-6">
    <skos:altLabel xml:lang="de">Blickverhalten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15793-6">
    <skos:altLabel xml:lang="de">Blickfanganalyse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15793-6">
    <skos:altLabel xml:lang="de">Aufmerksamkeitsforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15793-6">
    <skos:altLabel xml:lang="de">Anmutung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10459-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10338-6">
    <skos:altLabel xml:lang="en">Boom</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10338-6">
    <skos:altLabel xml:lang="de">Rezession</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10338-6">
    <skos:altLabel xml:lang="de">Konjunkturzyklus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10338-6">
    <skos:altLabel xml:lang="de">Konjunkturelle Lage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10338-6">
    <skos:altLabel xml:lang="de">Konjunkturelle Entwicklung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10338-6">
    <skos:altLabel xml:lang="de">Depression</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70504">
    <skos:broader rdf:resource="thsys/70056"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10236-0">
    <skos:altLabel xml:lang="de">Monetäre Analyse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10236-0">
    <skos:altLabel xml:lang="de">Geldmodell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18979-5">
    <skos:altLabel xml:lang="en">Agricultural marketing board</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18979-5">
    <skos:altLabel xml:lang="de">Landwirtschaftsbehörde</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18979-5">
    <skos:altLabel xml:lang="de">Agrarverwaltung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18979-5">
    <skos:altLabel xml:lang="de">Agrarmarketinggesellschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18979-5">
    <skos:altLabel xml:lang="de">Agrarbehörde</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71013">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70581">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.08.03</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10038-4">
    <skos:narrower rdf:resource="descriptor/10056-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10038-4">
    <skos:narrower rdf:resource="descriptor/11495-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10038-4">
    <skos:narrower rdf:resource="descriptor/10151-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10038-4">
    <skos:narrower rdf:resource="descriptor/10034-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11224-2">
    <skos:prefLabel xml:lang="en">Human capital</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11224-2">
    <skos:prefLabel xml:lang="de">Humankapital</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13992-3">
    <skos:prefLabel xml:lang="en">Entrepreneurs</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13992-3">
    <skos:prefLabel xml:lang="de">Unternehmer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10221-6">
    <skos:scopeNote xml:lang="de">Hier auch Termingeschäfte mit Edelmetallen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18611-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14229-5">
    <skos:broader rdf:resource="thsys/70348"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14229-5">
    <skos:broader rdf:resource="descriptor/14236-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16209-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13729-4">
    <skos:broader rdf:resource="thsys/70893"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13729-4">
    <skos:broader rdf:resource="thsys/70187"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13729-4">
    <skos:broader rdf:resource="descriptor/13742-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10293-2">
    <skos:related rdf:resource="descriptor/18118-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10293-2">
    <skos:related rdf:resource="descriptor/10816-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10293-2">
    <skos:related rdf:resource="descriptor/12208-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10293-2">
    <skos:related rdf:resource="descriptor/10824-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10293-2">
    <skos:related rdf:resource="descriptor/10008-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10293-2">
    <skos:related rdf:resource="descriptor/19537-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13125-1">
    <skos:prefLabel xml:lang="en">Pulp industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13125-1">
    <skos:prefLabel xml:lang="de">Zellstoffindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13766-5">
    <skos:altLabel xml:lang="en">Financial swaps</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13766-5">
    <skos:altLabel xml:lang="en">Financial swap</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13766-5">
    <skos:altLabel xml:lang="de">Swaption</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13766-5">
    <skos:altLabel xml:lang="de">Swapoption</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13766-5">
    <skos:altLabel xml:lang="de">Swapgeschäft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13766-5">
    <skos:altLabel xml:lang="de">Finanzswap</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19411-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21306-1">
    <skos:prefLabel xml:lang="en">Diversity management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21306-1">
    <skos:prefLabel xml:lang="de">Diversity Management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18077-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15669-0">
    <skos:broader rdf:resource="thsys/73350"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15669-0">
    <skos:broader rdf:resource="descriptor/15664-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16513-5">
    <skos:related rdf:resource="descriptor/16615-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16513-5">
    <skos:related rdf:resource="descriptor/16656-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16513-5">
    <skos:related rdf:resource="descriptor/15928-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16513-5">
    <skos:related rdf:resource="descriptor/16512-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16513-5">
    <skos:related rdf:resource="descriptor/19642-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16513-5">
    <skos:related rdf:resource="descriptor/15932-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16513-5">
    <skos:related rdf:resource="descriptor/15931-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14735-4">
    <skos:broader rdf:resource="thsys/70536"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14735-4">
    <skos:broader rdf:resource="descriptor/14734-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19649-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19776-0">
    <zbwext:useInsteadNote rdf:nodeID="id285"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20289-1">
    <skos:scopeNote xml:lang="de">In Frankreich entstandener Ansatz der Entwicklungstheorie. Mithilfe der makroökonomischen Analyse werden die historisch-sozialen Gesetzmäßigkeiten untersucht, die Produktion und Verteilung von Gütern in strukturierten Gesellschaften regeln. Theoretische Anleihen bei Fordismus, Taylorismus, Holismus und Marxismus.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13315-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13279-2">
    <zbwext:useInsteadNote rdf:nodeID="id57"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18677-0">
    <skos:related rdf:resource="descriptor/13729-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18677-0">
    <skos:related rdf:resource="descriptor/13716-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15330-4">
    <skos:scopeNote xml:lang="de">Maß der Stärke des Zusammenhangs zwischen mehreren Merkmalen bzw. Variablen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17722-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13831-6">
    <skos:broader rdf:resource="thsys/72181"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13831-6">
    <skos:broader rdf:resource="thsys/70888"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13831-6">
    <skos:broader rdf:resource="descriptor/18887-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10829-1">
    <skos:broader rdf:resource="thsys/73337"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10829-1">
    <skos:broader rdf:resource="thsys/73332"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10829-1">
    <skos:broader rdf:resource="thsys/71073"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10829-1">
    <skos:broader rdf:resource="descriptor/16188-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10829-1">
    <skos:broader rdf:resource="descriptor/11640-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16340-3">
    <skos:narrower rdf:resource="descriptor/16344-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16340-3">
    <skos:narrower rdf:resource="descriptor/16343-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12218-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15770-4">
    <skos:prefLabel xml:lang="en">Environmental protection</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15770-4">
    <skos:prefLabel xml:lang="de">Umweltschutz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19268-5">
    <skos:related rdf:resource="descriptor/16203-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19268-5">
    <skos:related rdf:resource="descriptor/16722-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19268-5">
    <skos:related rdf:resource="descriptor/18177-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19500-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16603-4">
    <skos:broader rdf:resource="thsys/73396"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16603-4">
    <skos:broader rdf:resource="thsys/71005"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16603-4">
    <skos:broader rdf:resource="descriptor/16602-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16603-4">
    <skos:broader rdf:resource="descriptor/16588-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10156-5">
    <skos:prefLabel xml:lang="en">Time preference</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10156-5">
    <skos:prefLabel xml:lang="de">Zeitpräferenz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19274-3">
    <skos:prefLabel xml:lang="en">Risk premium</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19274-3">
    <skos:prefLabel xml:lang="de">Risikoprämie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18501-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15460-5">
    <skos:scopeNote xml:lang="de">Spiele mit einem längeren bis unendlichen Zeithorizont.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12178-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16601-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16672-6">
    <skos:related rdf:resource="descriptor/15672-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16672-6">
    <skos:related rdf:resource="descriptor/16663-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16672-6">
    <skos:related rdf:resource="descriptor/19814-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16672-6">
    <skos:related rdf:resource="descriptor/10501-0"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id353">
    <rdf:value xml:lang="de">Für das Personal des öffentlichen Sektors</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15920-4">
    <skos:related rdf:resource="descriptor/15659-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15920-4">
    <skos:related rdf:resource="descriptor/15927-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15920-4">
    <skos:related rdf:resource="descriptor/18862-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15920-4">
    <skos:related rdf:resource="descriptor/15945-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15920-4">
    <skos:related rdf:resource="descriptor/18210-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15920-4">
    <skos:related rdf:resource="descriptor/18142-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15920-4">
    <skos:related rdf:resource="descriptor/16678-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15920-4">
    <skos:related rdf:resource="descriptor/18863-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15920-4">
    <skos:related rdf:resource="descriptor/16628-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15920-4">
    <skos:related rdf:resource="descriptor/15938-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10271-5">
    <skos:altLabel xml:lang="en">National financial account</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10271-5">
    <skos:altLabel xml:lang="en">Money flow analysis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10271-5">
    <skos:altLabel xml:lang="en">Flow-of-funds account</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10271-5">
    <skos:altLabel xml:lang="en">Flow of capital funds</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10271-5">
    <skos:altLabel xml:lang="de">Gesamtwirtschaftliche Finanzierungsrechnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10271-5">
    <skos:altLabel xml:lang="de">Geldstromanalyse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10271-5">
    <skos:altLabel xml:lang="de">Finanzierungsrechnung der Gesamtwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11200-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15327-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10942-5">
    <skos:related rdf:resource="descriptor/10913-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10942-5">
    <skos:related rdf:resource="descriptor/10956-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10942-5">
    <skos:related rdf:resource="descriptor/10964-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10942-5">
    <skos:related rdf:resource="descriptor/10963-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10942-5">
    <skos:related rdf:resource="descriptor/10895-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10942-5">
    <skos:related rdf:resource="descriptor/10951-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10942-5">
    <skos:related rdf:resource="descriptor/10894-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10942-5">
    <skos:related rdf:resource="descriptor/10904-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10942-5">
    <skos:related rdf:resource="descriptor/10928-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10942-5">
    <skos:related rdf:resource="descriptor/16051-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10942-5">
    <skos:related rdf:resource="descriptor/10891-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10942-5">
    <skos:related rdf:resource="descriptor/10690-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14561-4">
    <skos:broader rdf:resource="thsys/70477"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14561-4">
    <skos:broader rdf:resource="thsys/70346"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19142-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13052-2">
    <skos:altLabel xml:lang="de">Kraftfahrzeuginstandhaltung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13052-2">
    <skos:altLabel xml:lang="de">Kfz-Reparaturwerkstätte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13052-2">
    <skos:altLabel xml:lang="de">Kfz-Handwerk</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18882-6">
    <skos:prefLabel xml:lang="en">EU vocational training policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18882-6">
    <skos:prefLabel xml:lang="de">EU-Berufsbildungspolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14933-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19262-3">
    <skos:broader rdf:resource="thsys/70305"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19262-3">
    <skos:broader rdf:resource="descriptor/10993-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19262-3">
    <skos:broader rdf:resource="descriptor/10382-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14539-4">
    <skos:prefLabel xml:lang="en">Photographic product</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14539-4">
    <skos:prefLabel xml:lang="de">Fototechnisches Produkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16432-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14788-4">
    <skos:broader rdf:resource="thsys/70416"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71075">
    <skos:narrower rdf:resource="descriptor/20189-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71075">
    <skos:narrower rdf:resource="descriptor/11679-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71075">
    <skos:narrower rdf:resource="descriptor/19404-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71075">
    <skos:narrower rdf:resource="descriptor/11709-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71075">
    <skos:narrower rdf:resource="descriptor/19567-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71075">
    <skos:narrower rdf:resource="descriptor/19408-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71075">
    <skos:narrower rdf:resource="descriptor/21014-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71075">
    <skos:narrower rdf:resource="descriptor/11707-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71075">
    <skos:narrower rdf:resource="descriptor/11706-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71075">
    <skos:narrower rdf:resource="descriptor/11705-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71075">
    <skos:narrower rdf:resource="descriptor/11704-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71075">
    <skos:narrower rdf:resource="descriptor/10831-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71075">
    <skos:narrower rdf:resource="descriptor/11703-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71075">
    <skos:narrower rdf:resource="descriptor/10830-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71075">
    <skos:narrower rdf:resource="descriptor/21011-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71075">
    <skos:narrower rdf:resource="descriptor/11708-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71075">
    <skos:narrower rdf:resource="descriptor/11702-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71075">
    <skos:narrower rdf:resource="descriptor/19416-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71075">
    <skos:narrower rdf:resource="descriptor/11701-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71075">
    <skos:narrower rdf:resource="descriptor/18154-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71075">
    <skos:narrower rdf:resource="descriptor/19415-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71075">
    <skos:narrower rdf:resource="descriptor/11660-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71075">
    <skos:narrower rdf:resource="descriptor/19414-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71075">
    <skos:narrower rdf:resource="descriptor/19413-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71075">
    <skos:narrower rdf:resource="descriptor/10828-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18248-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70459">
    <rdfs:label xml:lang="de">Verkauf und Distribution</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15036-4">
    <skos:prefLabel xml:lang="en">Fuzzy sets</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15036-4">
    <skos:prefLabel xml:lang="de">Fuzzy Sets</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71036">
    <skos:prefLabel xml:lang="de">B.09.03  Betriebliche Anwendungssysteme</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id36">
    <rdfs:seeAlso rdf:resource="descriptor/10822-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71026">
    <rdfs:label xml:lang="en">Horticulture</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71026">
    <rdfs:label xml:lang="de">Gartenbau</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12821-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13743-3">
    <skos:prefLabel xml:lang="en">Stock market speculation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13743-3">
    <skos:prefLabel xml:lang="de">Wertpapierspekulation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15478-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11382-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18670-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73302">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">P.20.03</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13376-4">
    <skos:related rdf:resource="descriptor/18787-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13376-4">
    <skos:related rdf:resource="descriptor/18492-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13376-4">
    <skos:related rdf:resource="descriptor/13957-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18423-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19253-4">
    <skos:prefLabel xml:lang="en">Knowledge</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19253-4">
    <skos:prefLabel xml:lang="de">Wissen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13249-4">
    <skos:broader rdf:resource="thsys/71072"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13249-4">
    <skos:broader rdf:resource="thsys/70939"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13249-4">
    <skos:broader rdf:resource="thsys/70925"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13249-4">
    <skos:broader rdf:resource="thsys/70860"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13249-4">
    <skos:broader rdf:resource="descriptor/13632-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13249-4">
    <skos:broader rdf:resource="descriptor/11629-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73353">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10043-4">
    <skos:altLabel xml:lang="de">SMOPEC</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10043-4">
    <skos:altLabel xml:lang="de">Kleines-offenes-Land-Modell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10043-4">
    <skos:altLabel xml:lang="de">Kleines offenes Land</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13753-0">
    <skos:broader rdf:resource="thsys/70900"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13753-0">
    <skos:broader rdf:resource="descriptor/12531-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13613-2">
    <skos:prefLabel xml:lang="en">Transit authority</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13613-2">
    <skos:prefLabel xml:lang="de">Verkehrsverbund</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10876-6">
    <skos:prefLabel xml:lang="en">International marketing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10876-6">
    <skos:prefLabel xml:lang="de">Internationales Marketing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15924-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18571-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10070-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18337-3">
    <skos:related rdf:resource="descriptor/15779-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18337-3">
    <skos:related rdf:resource="descriptor/15897-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18337-3">
    <skos:related rdf:resource="descriptor/18987-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18337-3">
    <skos:related rdf:resource="descriptor/19484-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10284-3">
    <skos:broader rdf:resource="thsys/71112"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12078-3">
    <skos:altLabel xml:lang="en">Turnaround management</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12078-3">
    <skos:altLabel xml:lang="de">Unternehmenskrise</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12078-3">
    <skos:altLabel xml:lang="de">Krise des Betriebs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12078-3">
    <skos:altLabel xml:lang="de">Firmenkrise</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18110-3">
    <skos:related rdf:resource="descriptor/13673-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18110-3">
    <skos:related rdf:resource="descriptor/18941-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18110-3">
    <skos:related rdf:resource="descriptor/24664-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18368-6">
    <skos:broader rdf:resource="thsys/71013"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18368-6">
    <skos:broader rdf:resource="descriptor/12613-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12912-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14665-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15759-6">
    <skos:prefLabel xml:lang="en">Environment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15759-6">
    <skos:prefLabel xml:lang="de">Umwelt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15600-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11804-5">
    <skos:related rdf:resource="descriptor/16552-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11804-5">
    <skos:related rdf:resource="descriptor/12032-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11804-5">
    <skos:related rdf:resource="descriptor/11803-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14766-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18429-5">
    <skos:prefLabel xml:lang="en">Chemical engineering</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18429-5">
    <skos:prefLabel xml:lang="de">Chemische Technik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10858-1">
    <skos:related rdf:resource="descriptor/10821-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10858-1">
    <skos:related rdf:resource="descriptor/18539-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10858-1">
    <skos:related rdf:resource="descriptor/10816-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10858-1">
    <skos:related rdf:resource="descriptor/10866-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10858-1">
    <skos:related rdf:resource="descriptor/10693-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10858-1">
    <skos:related rdf:resource="descriptor/10854-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11852-1">
    <skos:narrower rdf:resource="descriptor/11970-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11852-1">
    <skos:narrower rdf:resource="descriptor/11862-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71045">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70538">
    <skos:broader rdf:resource="thsys/p"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10652-0">
    <skos:broader rdf:resource="thsys/71100"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10652-0">
    <skos:broader rdf:resource="descriptor/10036-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19840-3">
    <skos:prefLabel xml:lang="en">Famine</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19840-3">
    <skos:prefLabel xml:lang="de">Hungersnot</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19480-0">
    <skos:broader rdf:resource="thsys/73366"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19480-0">
    <skos:broader rdf:resource="thsys/73332"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19480-0">
    <skos:broader rdf:resource="thsys/70239"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19480-0">
    <skos:broader rdf:resource="descriptor/16184-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19480-0">
    <skos:broader rdf:resource="descriptor/15738-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11090-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10012-1">
    <skos:narrower rdf:resource="descriptor/19679-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10012-1">
    <skos:narrower rdf:resource="descriptor/19289-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10012-1">
    <skos:narrower rdf:resource="descriptor/19391-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10012-1">
    <skos:narrower rdf:resource="descriptor/18995-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10012-1">
    <skos:narrower rdf:resource="descriptor/10949-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10012-1">
    <skos:narrower rdf:resource="descriptor/19389-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10012-1">
    <skos:narrower rdf:resource="descriptor/19338-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10012-1">
    <skos:narrower rdf:resource="descriptor/10013-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10012-1">
    <skos:narrower rdf:resource="descriptor/19186-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10012-1">
    <skos:narrower rdf:resource="descriptor/19549-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10012-1">
    <skos:narrower rdf:resource="descriptor/19366-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10012-1">
    <skos:narrower rdf:resource="descriptor/18943-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10012-1">
    <skos:narrower rdf:resource="descriptor/12031-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10012-1">
    <skos:narrower rdf:resource="descriptor/18547-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10012-1">
    <skos:narrower rdf:resource="descriptor/19382-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10012-1">
    <skos:narrower rdf:resource="descriptor/13723-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10012-1">
    <skos:narrower rdf:resource="descriptor/10608-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10012-1">
    <skos:narrower rdf:resource="descriptor/18996-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10012-1">
    <skos:narrower rdf:resource="descriptor/19396-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10012-1">
    <skos:narrower rdf:resource="descriptor/19544-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10012-1">
    <skos:narrower rdf:resource="descriptor/19397-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10012-1">
    <skos:narrower rdf:resource="descriptor/13852-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10012-1">
    <skos:narrower rdf:resource="descriptor/10604-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10012-1">
    <skos:narrower rdf:resource="descriptor/19248-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10012-1">
    <skos:narrower rdf:resource="descriptor/19675-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10012-1">
    <skos:narrower rdf:resource="descriptor/11183-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10012-1">
    <skos:narrower rdf:resource="descriptor/10000-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10012-1">
    <skos:narrower rdf:resource="descriptor/13431-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13652-6">
    <skos:broader rdf:resource="thsys/70922"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13652-6">
    <skos:broader rdf:resource="descriptor/18660-3"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id224">
    <rdfs:seeAlso rdf:resource="descriptor/10237-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12779-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id359">
    <rdf:value xml:lang="de">Für den realen Wechselkurs</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11127-0">
    <skos:prefLabel xml:lang="en">Systemic transition</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11127-0">
    <skos:prefLabel xml:lang="de">Systemtransformation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16522-4">
    <skos:scopeNote xml:lang="de">Jede Art militärischen Dienstes unabhängig vom Status als Berufs- oder Zeitsoldat oder als Wehrpflichtiger.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14807-5">
    <skos:broader rdf:resource="thsys/70328"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14807-5">
    <skos:broader rdf:resource="descriptor/14803-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14807-5">
    <skos:broader rdf:resource="descriptor/14024-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16314-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10504-1">
    <skos:broader rdf:resource="thsys/71065"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10504-1">
    <skos:broader rdf:resource="thsys/70572"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10504-1">
    <skos:broader rdf:resource="thsys/70091"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10504-1">
    <skos:broader rdf:resource="descriptor/19647-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10198-3">
    <skos:altLabel xml:lang="de">Marginalpreis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10198-3">
    <skos:altLabel xml:lang="de">Grenzkostenkalkulation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14679-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12280-1">
    <skos:narrower rdf:resource="descriptor/18681-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12280-1">
    <skos:narrower rdf:resource="descriptor/10798-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73349">
    <skos:narrower rdf:resource="descriptor/19785-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73349">
    <skos:narrower rdf:resource="descriptor/18402-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73349">
    <skos:narrower rdf:resource="descriptor/18429-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73349">
    <skos:narrower rdf:resource="descriptor/18430-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73349">
    <skos:narrower rdf:resource="descriptor/15638-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13829-0">
    <skos:altLabel xml:lang="de">Wohnraum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18653-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16327-2">
    <skos:broader rdf:resource="thsys/73380"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16327-2">
    <skos:broader rdf:resource="descriptor/16329-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70020">
    <skos:narrower rdf:resource="descriptor/10340-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70020">
    <skos:narrower rdf:resource="descriptor/19241-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70020">
    <skos:narrower rdf:resource="descriptor/19240-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70020">
    <skos:narrower rdf:resource="descriptor/19239-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70020">
    <skos:narrower rdf:resource="descriptor/10353-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70020">
    <skos:narrower rdf:resource="descriptor/10392-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70020">
    <skos:narrower rdf:resource="descriptor/10350-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70020">
    <skos:narrower rdf:resource="descriptor/10349-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70020">
    <skos:narrower rdf:resource="descriptor/10348-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70020">
    <skos:narrower rdf:resource="descriptor/10376-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70020">
    <skos:narrower rdf:resource="descriptor/10346-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70020">
    <skos:narrower rdf:resource="descriptor/10375-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70020">
    <skos:narrower rdf:resource="descriptor/10347-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70020">
    <skos:narrower rdf:resource="descriptor/19587-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70020">
    <skos:narrower rdf:resource="descriptor/10374-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70020">
    <skos:narrower rdf:resource="descriptor/10387-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70020">
    <skos:narrower rdf:resource="descriptor/10373-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70020">
    <skos:narrower rdf:resource="descriptor/10361-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70020">
    <skos:narrower rdf:resource="descriptor/10339-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70020">
    <skos:narrower rdf:resource="descriptor/10338-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70020">
    <skos:narrower rdf:resource="descriptor/12700-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70020">
    <skos:narrower rdf:resource="descriptor/19826-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70020">
    <skos:narrower rdf:resource="descriptor/10383-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70020">
    <skos:narrower rdf:resource="descriptor/19238-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70020">
    <skos:narrower rdf:resource="descriptor/10337-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70020">
    <skos:narrower rdf:resource="descriptor/10381-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18239-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16308-6">
    <skos:narrower rdf:resource="descriptor/19691-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16308-6">
    <skos:narrower rdf:resource="descriptor/11036-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11197-0">
    <skos:prefLabel xml:lang="en">Homeworking</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11197-0">
    <skos:prefLabel xml:lang="de">Heimarbeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19648-2">
    <skos:related rdf:resource="descriptor/15956-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19648-2">
    <skos:related rdf:resource="descriptor/15919-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19648-2">
    <skos:related rdf:resource="descriptor/15951-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19404-2">
    <skos:prefLabel xml:lang="en">Tax competition</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19404-2">
    <skos:prefLabel xml:lang="de">Steuerwettbewerb</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15243-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12232-5">
    <skos:prefLabel xml:lang="en">Share</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12232-5">
    <skos:prefLabel xml:lang="de">Aktie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11793-5">
    <skos:broader rdf:resource="thsys/71110"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11793-5">
    <skos:broader rdf:resource="descriptor/11784-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19669-1">
    <skos:prefLabel xml:lang="en">Trading volume</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19669-1">
    <skos:prefLabel xml:lang="de">Börsenumsatz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19638-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15624-1">
    <skos:broader rdf:resource="thsys/73340"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14193-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11540-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12825-6">
    <skos:altLabel xml:lang="de">Werbefunk</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12825-6">
    <skos:altLabel xml:lang="de">Rundfunk- und Fernsehwerbung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12825-6">
    <skos:altLabel xml:lang="de">Radiowerbung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12825-6">
    <skos:altLabel xml:lang="de">Hörfunkwerbung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12825-6">
    <skos:altLabel xml:lang="de">Funkwerbung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16535-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70114">
    <skos:narrower rdf:resource="descriptor/13807-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70114">
    <skos:narrower rdf:resource="descriptor/10789-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70114">
    <skos:narrower rdf:resource="descriptor/16068-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70114">
    <skos:narrower rdf:resource="descriptor/13804-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70114">
    <skos:narrower rdf:resource="descriptor/11434-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70114">
    <skos:narrower rdf:resource="descriptor/13801-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70114">
    <skos:narrower rdf:resource="descriptor/10785-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70114">
    <skos:narrower rdf:resource="descriptor/13799-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70114">
    <skos:narrower rdf:resource="descriptor/18022-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70114">
    <skos:narrower rdf:resource="descriptor/18271-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70114">
    <skos:narrower rdf:resource="descriptor/13793-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70114">
    <skos:narrower rdf:resource="descriptor/11411-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18667-3">
    <skos:scopeNote xml:lang="de">Situationsbezogene Kombination von Finanzierungstechniken.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15977-3">
    <skos:related rdf:resource="descriptor/11347-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15977-3">
    <skos:related rdf:resource="descriptor/19715-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15977-3">
    <skos:related rdf:resource="descriptor/16058-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15977-3">
    <skos:related rdf:resource="descriptor/13464-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15977-3">
    <skos:related rdf:resource="descriptor/16301-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20906-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13180-0">
    <skos:broader rdf:resource="thsys/70968"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13180-0">
    <skos:broader rdf:resource="descriptor/12905-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11146-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24788-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18755-6">
    <skos:narrower rdf:resource="descriptor/11574-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18755-6">
    <skos:narrower rdf:resource="descriptor/11836-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11784-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18509-0">
    <skos:related rdf:resource="descriptor/16055-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18509-0">
    <skos:related rdf:resource="descriptor/12910-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18509-0">
    <skos:related rdf:resource="descriptor/11301-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19584-2">
    <skos:broader rdf:resource="thsys/72181"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19584-2">
    <skos:broader rdf:resource="thsys/70879"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19584-2">
    <skos:broader rdf:resource="descriptor/19072-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70573">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14732-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13179-6">
    <skos:narrower rdf:resource="descriptor/13535-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13179-6">
    <skos:narrower rdf:resource="descriptor/18985-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12201-2">
    <skos:altLabel xml:lang="de">Verwaltungsrat (Schweizer Aktienrecht)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14216-0">
    <skos:broader rdf:resource="thsys/70649"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14216-0">
    <skos:broader rdf:resource="descriptor/14173-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10093-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18811-1">
    <skos:related rdf:resource="descriptor/11567-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18811-1">
    <skos:related rdf:resource="descriptor/18991-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18811-1">
    <skos:related rdf:resource="descriptor/19338-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18811-1">
    <skos:related rdf:resource="descriptor/19394-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18811-1">
    <skos:related rdf:resource="descriptor/11802-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10617-2">
    <skos:prefLabel xml:lang="en">International economic relations</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10617-2">
    <skos:prefLabel xml:lang="de">Internationale Wirtschaftsbeziehungen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11040-5">
    <skos:prefLabel xml:lang="en">Property Rights</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11040-5">
    <skos:prefLabel xml:lang="de">Eigentumsrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10758-5">
    <skos:related rdf:resource="descriptor/19665-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14816-4">
    <skos:related rdf:resource="descriptor/14534-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14816-4">
    <skos:related rdf:resource="descriptor/13102-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13590-2">
    <skos:related rdf:resource="descriptor/18630-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13590-2">
    <skos:related rdf:resource="descriptor/13514-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13590-2">
    <skos:related rdf:resource="descriptor/11834-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13590-2">
    <skos:related rdf:resource="descriptor/11934-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16070-6">
    <skos:related rdf:resource="descriptor/18597-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16070-6">
    <skos:related rdf:resource="descriptor/13298-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16070-6">
    <skos:related rdf:resource="descriptor/18594-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16070-6">
    <skos:related rdf:resource="descriptor/19329-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16070-6">
    <skos:related rdf:resource="descriptor/10880-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16070-6">
    <skos:related rdf:resource="descriptor/13565-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12665-2">
    <skos:scopeNote xml:lang="de">Alle Beziehungen zwischen liefernden und beschaffenden Unternehmen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19629-6">
    <skos:prefLabel xml:lang="en">Plant protection</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19629-6">
    <skos:prefLabel xml:lang="de">Pflanzenschutz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10073-2">
    <skos:related rdf:resource="descriptor/10081-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10073-2">
    <skos:related rdf:resource="descriptor/10080-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10073-2">
    <skos:related rdf:resource="descriptor/10077-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10073-2">
    <skos:related rdf:resource="descriptor/10035-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10073-2">
    <skos:related rdf:resource="descriptor/10070-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12174-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14034-6">
    <skos:broader rdf:resource="thsys/70461"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14034-6">
    <skos:broader rdf:resource="descriptor/14638-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14034-6">
    <skos:broader rdf:resource="descriptor/13427-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70883">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10566-0">
    <skos:altLabel xml:lang="de">Stabex</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10566-0">
    <skos:altLabel xml:lang="de">Preisstabilisierung im Export</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10566-0">
    <skos:altLabel xml:lang="de">Exportpreisstabilisierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10566-0">
    <skos:altLabel xml:lang="de">Exporterlösstabilisierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10566-0">
    <skos:altLabel xml:lang="de">Exporterlösinstabilität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10566-0">
    <skos:altLabel xml:lang="de">Erlösstabilisierung im Export</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10566-0">
    <skos:altLabel xml:lang="de">Ausfuhrerlös</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19347-2">
    <skos:related rdf:resource="descriptor/19349-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19347-2">
    <skos:related rdf:resource="descriptor/14145-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19347-2">
    <skos:related rdf:resource="descriptor/12955-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19347-2">
    <skos:related rdf:resource="descriptor/19340-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19126-4">
    <skos:prefLabel xml:lang="en">Sales discount</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19126-4">
    <skos:prefLabel xml:lang="de">Skonto</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13011-2">
    <skos:related rdf:resource="descriptor/13213-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13011-2">
    <skos:related rdf:resource="descriptor/14776-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10178-2">
    <skos:prefLabel xml:lang="en">Inferior good</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10178-2">
    <skos:prefLabel xml:lang="de">Inferiores Gut</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12927-5">
    <skos:prefLabel xml:lang="en">Tourism industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12927-5">
    <skos:prefLabel xml:lang="de">Tourismuswirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18820-0">
    <skos:broader rdf:resource="thsys/70862"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18820-0">
    <skos:broader rdf:resource="descriptor/16524-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18313-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12292-1">
    <skos:related rdf:resource="descriptor/10798-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16826-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13638-0">
    <skos:prefLabel xml:lang="en">Tourism economics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13638-0">
    <skos:prefLabel xml:lang="de">Fremdenverkehrsökonomik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16952-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10859-6">
    <skos:altLabel xml:lang="de">Betriebswirtschaftslehre des Außenhandels</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10661-6">
    <skos:prefLabel xml:lang="en">Non-tradable</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10661-6">
    <skos:prefLabel xml:lang="de">Nicht-handelbares Gut</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19296-0">
    <skos:related rdf:resource="descriptor/13163-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19296-0">
    <skos:related rdf:resource="descriptor/18193-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19296-0">
    <skos:related rdf:resource="descriptor/18194-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19296-0">
    <skos:related rdf:resource="descriptor/14065-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16599-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70300">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11460-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12677-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16186-1">
    <skos:related rdf:resource="descriptor/19822-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16186-1">
    <skos:related rdf:resource="descriptor/16435-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16186-1">
    <skos:related rdf:resource="descriptor/16441-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24662-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16471-2">
    <skos:related rdf:resource="descriptor/16309-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16471-2">
    <skos:related rdf:resource="descriptor/16472-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16471-2">
    <skos:related rdf:resource="descriptor/16489-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18643-3">
    <skos:related rdf:resource="descriptor/18653-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18643-3">
    <skos:related rdf:resource="descriptor/18663-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18643-3">
    <skos:related rdf:resource="descriptor/18649-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18643-3">
    <skos:related rdf:resource="descriptor/13278-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18643-3">
    <skos:related rdf:resource="descriptor/16012-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18643-3">
    <skos:related rdf:resource="descriptor/15605-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18643-3">
    <skos:related rdf:resource="descriptor/15604-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18643-3">
    <skos:related rdf:resource="descriptor/18433-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18268-3">
    <skos:prefLabel xml:lang="en">Air freshener</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18268-3">
    <skos:prefLabel xml:lang="de">Luftverbesserer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12062-4">
    <skos:prefLabel xml:lang="en">Transnational corporation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12062-4">
    <skos:prefLabel xml:lang="de">Multinationales Unternehmen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19256-5">
    <skos:prefLabel xml:lang="en">Buffer Stock model</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19256-5">
    <skos:prefLabel xml:lang="de">Buffer-Stock-Ansatz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17780-5">
    <skos:prefLabel xml:lang="en">El Salvador</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17780-5">
    <skos:prefLabel xml:lang="de">El Salvador</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10515-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14380-1">
    <skos:altLabel xml:lang="de">Pflug</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14380-1">
    <skos:altLabel xml:lang="de">Melkmaschine</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14380-1">
    <skos:altLabel xml:lang="de">Mähdrescher</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14380-1">
    <skos:altLabel xml:lang="de">Agrarmaschine</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20206-3">
    <skos:related rdf:resource="descriptor/10909-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20206-3">
    <skos:related rdf:resource="descriptor/10215-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20206-3">
    <skos:related rdf:resource="descriptor/12793-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11724-3">
    <skos:prefLabel xml:lang="en">Wagner's law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11724-3">
    <skos:prefLabel xml:lang="de">Wagnersches Gesetz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11569-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19263-1">
    <skos:scopeNote xml:lang="de">Geld- oder währungspolitische Zielzonen</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10092-5">
    <skos:prefLabel xml:lang="en">Externalities</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10092-5">
    <skos:prefLabel xml:lang="de">Externer Effekt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19151-5">
    <skos:prefLabel xml:lang="en">Conference</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19151-5">
    <skos:prefLabel xml:lang="de">Konferenz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11384-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13632-5">
    <skos:related rdf:resource="descriptor/13605-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70997">
    <skos:broader rdf:resource="thsys/70999"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73392">
    <rdfs:label xml:lang="en">Civil code</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73392">
    <rdfs:label xml:lang="de">Bürgerliches Recht</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10220-1">
    <skos:narrower rdf:resource="descriptor/12796-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10220-1">
    <skos:narrower rdf:resource="descriptor/10199-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11253-2">
    <skos:broader rdf:resource="thsys/71061"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11253-2">
    <skos:broader rdf:resource="descriptor/11248-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11253-2">
    <skos:broader rdf:resource="descriptor/16526-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11253-2">
    <skos:broader rdf:resource="descriptor/16475-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19071-3">
    <skos:related rdf:resource="descriptor/10626-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19071-3">
    <skos:related rdf:resource="descriptor/16278-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19071-3">
    <skos:related rdf:resource="descriptor/12062-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19071-3">
    <skos:related rdf:resource="descriptor/19309-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19071-3">
    <skos:related rdf:resource="descriptor/10876-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19071-3">
    <skos:related rdf:resource="descriptor/18121-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19071-3">
    <skos:related rdf:resource="descriptor/20826-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19071-3">
    <skos:related rdf:resource="descriptor/19299-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19071-3">
    <skos:related rdf:resource="descriptor/10696-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19071-3">
    <skos:related rdf:resource="descriptor/20817-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12330-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70070">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19559-1">
    <skos:prefLabel xml:lang="en">School finance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19559-1">
    <skos:prefLabel xml:lang="de">Schulfinanzierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12941-4">
    <skos:broader rdf:resource="thsys/71025"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12941-4">
    <skos:broader rdf:resource="descriptor/12955-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16324-1">
    <skos:prefLabel xml:lang="en">Regional government</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16324-1">
    <skos:prefLabel xml:lang="de">Landesregierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70843">
    <rdfs:label xml:lang="en">Vocational education</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70843">
    <rdfs:label xml:lang="de">Berufsbildung</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17801-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19369-6">
    <skos:prefLabel xml:lang="en">Financing of regional development</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19369-6">
    <skos:prefLabel xml:lang="de">Regionale Entwicklungsfinanzierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18484-4">
    <skos:related rdf:resource="descriptor/16692-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18484-4">
    <skos:related rdf:resource="descriptor/18487-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18484-4">
    <skos:related rdf:resource="descriptor/18486-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18484-4">
    <skos:related rdf:resource="descriptor/11198-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18096-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16364-3">
    <skos:altLabel xml:lang="de">Umweltpartei</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16364-3">
    <skos:altLabel xml:lang="de">Grüne Partei</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18201-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70850">
    <skos:narrower rdf:resource="descriptor/15682-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70850">
    <skos:narrower rdf:resource="descriptor/13399-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70850">
    <skos:narrower rdf:resource="descriptor/15671-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70850">
    <skos:narrower rdf:resource="descriptor/18855-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70850">
    <skos:narrower rdf:resource="descriptor/11665-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70850">
    <skos:narrower rdf:resource="descriptor/16002-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70850">
    <skos:narrower rdf:resource="descriptor/13393-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70850">
    <skos:narrower rdf:resource="descriptor/14012-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70850">
    <skos:narrower rdf:resource="descriptor/16401-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70850">
    <skos:narrower rdf:resource="descriptor/15640-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16552-2">
    <skos:narrower rdf:resource="descriptor/18632-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16552-2">
    <skos:narrower rdf:resource="descriptor/11701-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16552-2">
    <skos:narrower rdf:resource="descriptor/19447-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16552-2">
    <skos:narrower rdf:resource="descriptor/11940-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16552-2">
    <skos:narrower rdf:resource="descriptor/11939-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16552-2">
    <skos:narrower rdf:resource="descriptor/11938-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16552-2">
    <skos:narrower rdf:resource="descriptor/11937-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18680-4">
    <skos:prefLabel xml:lang="en">Monetary assets</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18680-4">
    <skos:prefLabel xml:lang="de">Geldvermögen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11983-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13839-4">
    <skos:prefLabel xml:lang="en">Rental housing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13839-4">
    <skos:prefLabel xml:lang="de">Mietwohnung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19160-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18121-5">
    <skos:narrower rdf:resource="descriptor/10572-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18121-5">
    <skos:narrower rdf:resource="descriptor/18108-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11404-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19475-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16052-1">
    <skos:narrower rdf:resource="descriptor/16055-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11738-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12174-0">
    <skos:related rdf:resource="descriptor/12201-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12174-0">
    <skos:related rdf:resource="descriptor/12185-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12174-0">
    <skos:related rdf:resource="descriptor/13727-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12174-0">
    <skos:related rdf:resource="descriptor/12175-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12174-0">
    <skos:related rdf:resource="descriptor/13714-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12174-0">
    <skos:related rdf:resource="descriptor/16043-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12174-0">
    <skos:related rdf:resource="descriptor/12232-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10977-0">
    <skos:altLabel xml:lang="de">Freiburger Schule</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id145">
    <rdfs:seeAlso rdf:resource="descriptor/18762-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13998-5">
    <skos:prefLabel xml:lang="en">Soldiers</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13998-5">
    <skos:prefLabel xml:lang="de">Soldaten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18822-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12036-5">
    <skos:broader rdf:resource="thsys/70140"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12036-5">
    <skos:broader rdf:resource="thsys/70101"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71013">
    <skos:prefLabel xml:lang="de">W.04.01.03  Kernenergiewirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19500-6">
    <skos:altLabel xml:lang="en">Employee benefits</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19500-6">
    <skos:altLabel xml:lang="de">Sachbezüge</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19500-6">
    <skos:altLabel xml:lang="de">Geldwerte Leistung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19500-6">
    <skos:altLabel xml:lang="de">Belegschaftsrabatt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19670-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id334">
    <rdfs:seeAlso rdf:resource="descriptor/18001-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14377-4">
    <skos:altLabel xml:lang="de">Holzverarbeitungsmaschine</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19487-0">
    <skos:scopeNote xml:lang="de">Substitution zwischen Energiearten oder Energiequellen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11217-6">
    <skos:prefLabel xml:lang="en">Forced labour</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11217-6">
    <skos:prefLabel xml:lang="de">Zwangsarbeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10094-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18693-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16654-1">
    <skos:broader rdf:resource="thsys/73359"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70965">
    <rdfs:label xml:lang="en">Trade (Craft)</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70965">
    <rdfs:label xml:lang="de">Handwerk</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12617-6">
    <skos:altLabel xml:lang="de">Sozialpsychologie des Betriebes</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12617-6">
    <skos:altLabel xml:lang="de">Organisationspsychologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12617-6">
    <skos:altLabel xml:lang="de">Betriebspsychologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73365">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.04.04.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10204-6">
    <skos:broader rdf:resource="thsys/70178"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10204-6">
    <skos:broader rdf:resource="descriptor/10217-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10204-6">
    <skos:broader rdf:resource="descriptor/12794-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13401-3">
    <skos:narrower rdf:resource="descriptor/18844-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70041">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">B.07</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14619-6">
    <skos:broader rdf:resource="thsys/70014"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14619-6">
    <skos:broader rdf:resource="descriptor/14599-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19334-4">
    <skos:scopeNote xml:lang="de">Neufestsetzung eines Wechselkurses nach einer Zeit des Floatens.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12416-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18727-4">
    <skos:prefLabel xml:lang="en">Securities lending</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18727-4">
    <skos:prefLabel xml:lang="de">Wertpapierleihe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13210-3">
    <skos:related rdf:resource="descriptor/13470-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13210-3">
    <skos:related rdf:resource="descriptor/13465-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13210-3">
    <skos:related rdf:resource="descriptor/13464-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13210-3">
    <skos:related rdf:resource="descriptor/11823-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13210-3">
    <skos:related rdf:resource="descriptor/13462-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13210-3">
    <skos:related rdf:resource="descriptor/13460-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13210-3">
    <skos:related rdf:resource="descriptor/13970-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18712-3">
    <skos:altLabel xml:lang="en">Online banking</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18712-3">
    <skos:altLabel xml:lang="en">Home banking</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18674-6">
    <skos:broader rdf:resource="thsys/70187"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18674-6">
    <skos:broader rdf:resource="descriptor/15861-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13214-2">
    <skos:related rdf:resource="descriptor/14033-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13214-2">
    <skos:related rdf:resource="descriptor/18915-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13214-2">
    <skos:related rdf:resource="descriptor/13213-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13214-2">
    <skos:related rdf:resource="descriptor/12907-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13153-3">
    <skos:broader rdf:resource="thsys/70986"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13153-3">
    <skos:broader rdf:resource="descriptor/13160-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18011-5">
    <skos:scopeNote xml:lang="de">Verminderung der staatlichen Kontrolle eines Marktes.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10481-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10473-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19794-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14158-2">
    <skos:narrower rdf:resource="descriptor/14164-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14158-2">
    <skos:narrower rdf:resource="descriptor/14163-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12302-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11438-4">
    <skos:related rdf:resource="descriptor/14308-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11438-4">
    <skos:related rdf:resource="descriptor/11430-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14299-5">
    <skos:prefLabel xml:lang="en">Copper</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14299-5">
    <skos:prefLabel xml:lang="de">Kupfer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73359">
    <skos:prefLabel xml:lang="de">N.06.03.01  Verhalten und soziale Beziehungen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14151-2">
    <skos:prefLabel xml:lang="en">Rubber</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14151-2">
    <skos:prefLabel xml:lang="de">Kautschuk</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12005-2">
    <skos:prefLabel xml:lang="en">Location of government agencies</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12005-2">
    <skos:prefLabel xml:lang="de">Verwaltungsstandort</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10574-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15397-0">
    <skos:prefLabel xml:lang="en">Cluster analysis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15397-0">
    <skos:prefLabel xml:lang="de">Clusteranalyse</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19684-5">
    <skos:broader rdf:resource="thsys/73381"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73371">
    <skos:prefLabel xml:lang="de">N.04.03.03  Umwelt-, Rohstoff- und Energiepolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73383">
    <skos:prefLabel xml:lang="de">N.04.01  Politische Theorien, politische Philosophien und Strömungen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10129-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18187-3">
    <skos:related rdf:resource="descriptor/18625-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18187-3">
    <skos:related rdf:resource="descriptor/11833-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18187-3">
    <skos:related rdf:resource="descriptor/13583-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19688-4">
    <skos:related rdf:resource="descriptor/16389-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19688-4">
    <skos:related rdf:resource="descriptor/19684-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19688-4">
    <skos:related rdf:resource="descriptor/19714-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19688-4">
    <skos:related rdf:resource="descriptor/13420-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19688-4">
    <skos:related rdf:resource="descriptor/16422-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19688-4">
    <skos:related rdf:resource="descriptor/19705-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19688-4">
    <skos:related rdf:resource="descriptor/16278-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19688-4">
    <skos:related rdf:resource="descriptor/16264-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19688-4">
    <skos:related rdf:resource="descriptor/18062-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13046-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19280-1">
    <skos:prefLabel xml:lang="en">Black-Scholes model</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19280-1">
    <skos:prefLabel xml:lang="de">Black-Scholes-Modell</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11360-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24706-5">
    <skos:broader rdf:resource="thsys/70040"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24706-5">
    <skos:broader rdf:resource="descriptor/12788-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15910-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13232-0">
    <skos:related rdf:resource="descriptor/18436-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13232-0">
    <skos:related rdf:resource="descriptor/19391-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13232-0">
    <skos:related rdf:resource="descriptor/13142-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13232-0">
    <skos:related rdf:resource="descriptor/14833-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13232-0">
    <skos:related rdf:resource="descriptor/13220-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14365-4">
    <skos:altLabel xml:lang="de">Schaufellader</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14365-4">
    <skos:altLabel xml:lang="de">Hydraulikbagger</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70481">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18231-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16328-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11020-4">
    <skos:narrower rdf:resource="descriptor/11886-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15086-3">
    <skos:altLabel xml:lang="en">PPI (Producer price index)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15086-3">
    <skos:altLabel xml:lang="de">Index der Industriepreise</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15086-3">
    <skos:altLabel xml:lang="de">Index der Erzeugerpreise industrieller Produkte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15086-3">
    <skos:altLabel xml:lang="de">Index der Erzeugerpreise gewerblicher Produkte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18119-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18914-5">
    <skos:related rdf:resource="descriptor/16540-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18914-5">
    <skos:related rdf:resource="descriptor/13416-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20826-1">
    <skos:prefLabel xml:lang="en">Global governance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20826-1">
    <skos:prefLabel xml:lang="de">Global Governance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18377-5">
    <skos:related rdf:resource="descriptor/15889-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18377-5">
    <skos:related rdf:resource="descriptor/15751-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18377-5">
    <skos:related rdf:resource="descriptor/10490-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70505">
    <skos:narrower rdf:resource="descriptor/16989-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70505">
    <skos:narrower rdf:resource="descriptor/16985-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70505">
    <skos:narrower rdf:resource="descriptor/16990-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70505">
    <skos:narrower rdf:resource="descriptor/16984-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70505">
    <skos:narrower rdf:resource="descriptor/16982-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70505">
    <skos:narrower rdf:resource="descriptor/16995-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11907-2">
    <skos:scopeNote xml:lang="de">Regionale oder räumliche Wirkungen von politischen Maßnahmen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18367-1">
    <skos:related rdf:resource="descriptor/13003-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18367-1">
    <skos:related rdf:resource="descriptor/14035-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18367-1">
    <skos:related rdf:resource="descriptor/15628-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18367-1">
    <skos:related rdf:resource="descriptor/16001-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18367-1">
    <skos:related rdf:resource="descriptor/14182-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18367-1">
    <skos:related rdf:resource="descriptor/16449-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71117">
    <rdfs:label xml:lang="en">Economics of information</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71117">
    <rdfs:label xml:lang="de">Informationsökonomik</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24847-1">
    <skos:broader rdf:resource="thsys/70562"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24847-1">
    <skos:broader rdf:resource="descriptor/18859-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24847-1">
    <skos:broader rdf:resource="descriptor/12106-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15627-2">
    <skos:prefLabel xml:lang="en">Anthropology</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15627-2">
    <skos:prefLabel xml:lang="de">Anthropologie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18297-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19824-1">
    <skos:broader rdf:resource="thsys/73391"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19824-1">
    <skos:broader rdf:resource="thsys/73336"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14919-1">
    <skos:prefLabel xml:lang="en">Workwear</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14919-1">
    <skos:prefLabel xml:lang="de">Spezialbekleidung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19370-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14150-4">
    <skos:prefLabel xml:lang="en">Wood</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14150-4">
    <skos:prefLabel xml:lang="de">Holz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71034">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">B.01.05</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70384">
    <skos:broader rdf:resource="thsys/70041"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13076-2">
    <skos:broader rdf:resource="thsys/70996"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13076-2">
    <skos:broader rdf:resource="descriptor/12993-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20756-3">
    <skos:scopeNote xml:lang="de">Entscheidung zwischen nicht kontinuierlichen (nicht-teilbaren) Alternativen, z.B. beim Hauskauf.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15604-0">
    <skos:scopeNote xml:lang="de">Rechtliche Regelungen zum Schutz personenbezogener Daten.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16716-5">
    <skos:related rdf:resource="descriptor/15045-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16716-5">
    <skos:related rdf:resource="descriptor/15609-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16716-5">
    <skos:related rdf:resource="descriptor/15036-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16716-5">
    <skos:related rdf:resource="descriptor/16708-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16716-5">
    <skos:related rdf:resource="descriptor/19742-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18031-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14161-6">
    <skos:altLabel xml:lang="en">Seaweed</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14161-6">
    <skos:altLabel xml:lang="en">Seafood</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14161-6">
    <skos:altLabel xml:lang="de">Wasserpflanze</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14161-6">
    <skos:altLabel xml:lang="de">Seetang</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14161-6">
    <skos:altLabel xml:lang="de">Plankton</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14161-6">
    <skos:altLabel xml:lang="de">Fischereiprodukt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14161-6">
    <skos:altLabel xml:lang="de">Algen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id373">
    <rdf:value xml:lang="de">für quantitative Entwicklungsprozesse</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18159-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19462-2">
    <skos:broader rdf:resource="thsys/71065"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19462-2">
    <skos:broader rdf:resource="thsys/70051"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19462-2">
    <skos:broader rdf:resource="descriptor/11183-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15108-5">
    <skos:prefLabel xml:lang="en">Consumption statistics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15108-5">
    <skos:prefLabel xml:lang="de">Verbrauchsstatistik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10817-1">
    <skos:broader rdf:resource="thsys/71089"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10817-1">
    <skos:broader rdf:resource="descriptor/10807-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15982-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19164-3">
    <skos:prefLabel xml:lang="en">Strategic alliance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19164-3">
    <skos:prefLabel xml:lang="de">Strategische Allianz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14217-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18101-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15076-6">
    <skos:broader rdf:resource="thsys/72181"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15076-6">
    <skos:broader rdf:resource="thsys/70021"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15076-6">
    <skos:broader rdf:resource="descriptor/19072-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10813-2">
    <skos:prefLabel xml:lang="en">Capital exports</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10813-2">
    <skos:prefLabel xml:lang="de">Kapitalexport</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18837-4">
    <skos:related rdf:resource="descriptor/11990-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18837-4">
    <skos:related rdf:resource="descriptor/19446-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16656-4">
    <skos:related rdf:resource="descriptor/16654-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16656-4">
    <skos:related rdf:resource="descriptor/16513-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16656-4">
    <skos:related rdf:resource="descriptor/16663-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16656-4">
    <skos:related rdf:resource="descriptor/19469-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16656-4">
    <skos:related rdf:resource="descriptor/15713-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12473-4">
    <skos:prefLabel xml:lang="en">Cost information system</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12473-4">
    <skos:prefLabel xml:lang="de">Kosten-Informationssystem</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10789-1">
    <skos:altLabel xml:lang="en">RTGS</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10789-1">
    <skos:altLabel xml:lang="en">Real-time gross settlement system</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10789-1">
    <skos:altLabel xml:lang="en">Clearance system</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10789-1">
    <skos:altLabel xml:lang="de">Clearing im Zahlungsverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10789-1">
    <skos:altLabel xml:lang="de">Abrechnungsverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18832-0">
    <skos:related rdf:resource="descriptor/10166-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18832-0">
    <skos:related rdf:resource="descriptor/15860-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18832-0">
    <skos:related rdf:resource="descriptor/18151-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18832-0">
    <skos:related rdf:resource="descriptor/10161-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18832-0">
    <skos:related rdf:resource="descriptor/19219-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18832-0">
    <skos:related rdf:resource="descriptor/10968-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18832-0">
    <skos:related rdf:resource="descriptor/16406-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18832-0">
    <skos:related rdf:resource="descriptor/10150-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14431-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18097-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14163-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16203-6">
    <skos:altLabel xml:lang="en">Price gouging</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16203-6">
    <skos:altLabel xml:lang="de">Vermittlungswucher</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16203-6">
    <skos:altLabel xml:lang="de">Preiswucher</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16203-6">
    <skos:altLabel xml:lang="de">Mietwucher</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16203-6">
    <skos:altLabel xml:lang="de">Leistungswucher</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16203-6">
    <skos:altLabel xml:lang="de">Kreditwucher</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11612-0">
    <skos:broader rdf:resource="thsys/71078"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11612-0">
    <skos:broader rdf:resource="thsys/71072"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11612-0">
    <skos:broader rdf:resource="thsys/70860"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11612-0">
    <skos:broader rdf:resource="descriptor/11616-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11612-0">
    <skos:broader rdf:resource="descriptor/12282-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16449-2">
    <skos:related rdf:resource="descriptor/18367-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16449-2">
    <skos:related rdf:resource="descriptor/15001-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18635-2">
    <skos:altLabel xml:lang="de">Kurort</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18086-2">
    <skos:prefLabel xml:lang="en">Investment bank</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18086-2">
    <skos:prefLabel xml:lang="de">Investment Bank</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19377-0">
    <skos:broader rdf:resource="thsys/71083"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19377-0">
    <skos:broader rdf:resource="descriptor/11718-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15962-2">
    <skos:prefLabel xml:lang="en">Administration of justice</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15962-2">
    <skos:prefLabel xml:lang="de">Justizverwaltung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10242-5">
    <skos:narrower rdf:resource="descriptor/11495-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13386-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15398-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15910-0">
    <skos:broader rdf:resource="thsys/70142"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12993-6">
    <skos:editorialNote xml:lang="de">vgl. auch: Industrie.</skos:editorialNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10221-6">
    <skos:narrower rdf:resource="descriptor/13744-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10221-6">
    <skos:narrower rdf:resource="descriptor/18690-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10221-6">
    <skos:narrower rdf:resource="descriptor/13735-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10221-6">
    <skos:narrower rdf:resource="descriptor/13722-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10221-6">
    <skos:narrower rdf:resource="descriptor/13720-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15493-4">
    <skos:broader rdf:resource="thsys/71035"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15493-4">
    <skos:broader rdf:resource="descriptor/15055-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10581-4">
    <skos:related rdf:resource="descriptor/18094-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11037-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16191-1">
    <skos:prefLabel xml:lang="en">Industrial espionage</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16191-1">
    <skos:prefLabel xml:lang="de">Industriespionage</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10889-4">
    <skos:narrower rdf:resource="descriptor/18990-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10889-4">
    <skos:narrower rdf:resource="descriptor/18163-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19254-2">
    <skos:related rdf:resource="descriptor/12697-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19254-2">
    <skos:related rdf:resource="descriptor/10058-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19254-2">
    <skos:related rdf:resource="descriptor/12094-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10177-4">
    <skos:narrower rdf:resource="descriptor/10184-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10177-4">
    <skos:narrower rdf:resource="descriptor/19200-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10177-4">
    <skos:narrower rdf:resource="descriptor/10181-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10177-4">
    <skos:narrower rdf:resource="descriptor/10180-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10177-4">
    <skos:narrower rdf:resource="descriptor/10176-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10177-4">
    <skos:narrower rdf:resource="descriptor/10171-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10177-4">
    <skos:narrower rdf:resource="descriptor/19197-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10177-4">
    <skos:narrower rdf:resource="descriptor/10169-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11158-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14335-6">
    <skos:broader rdf:resource="thsys/70549"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14335-6">
    <skos:broader rdf:resource="descriptor/14336-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18758-0">
    <skos:altLabel xml:lang="de">Software-Leasing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18758-0">
    <skos:altLabel xml:lang="de">EDV-Leasing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18758-0">
    <skos:altLabel xml:lang="de">DV-Leasing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19654-0">
    <skos:related rdf:resource="descriptor/21020-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19654-0">
    <skos:related rdf:resource="descriptor/12768-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19654-0">
    <skos:related rdf:resource="descriptor/19114-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19654-0">
    <skos:related rdf:resource="descriptor/19833-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19654-0">
    <skos:related rdf:resource="descriptor/16774-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19654-0">
    <skos:related rdf:resource="descriptor/19807-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11284-5">
    <skos:prefLabel xml:lang="en">Women workers</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11284-5">
    <skos:prefLabel xml:lang="de">Weibliche Arbeitskräfte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13402-1">
    <zbwext:useInsteadNote rdf:nodeID="id345"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15089-4">
    <skos:altLabel xml:lang="de">Lohnindex</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15089-4">
    <skos:altLabel xml:lang="de">Gehaltsindex</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16597-1">
    <skos:prefLabel xml:lang="en">Social structure</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16597-1">
    <skos:prefLabel xml:lang="de">Sozialstruktur</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11826-2">
    <skos:prefLabel xml:lang="en">Speed limit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11826-2">
    <skos:prefLabel xml:lang="de">Geschwindigkeitsbegrenzung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12805-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16195-0">
    <skos:broader rdf:resource="thsys/73356"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16195-0">
    <skos:broader rdf:resource="thsys/73334"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13347-4">
    <skos:broader rdf:resource="thsys/70825"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13347-4">
    <skos:broader rdf:resource="descriptor/18920-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18920-3">
    <skos:related rdf:resource="descriptor/13392-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18920-3">
    <skos:related rdf:resource="descriptor/18448-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18920-3">
    <skos:related rdf:resource="descriptor/13355-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18920-3">
    <skos:related rdf:resource="descriptor/16515-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18920-3">
    <skos:related rdf:resource="descriptor/18921-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18920-3">
    <skos:related rdf:resource="descriptor/18922-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18920-3">
    <skos:related rdf:resource="descriptor/15654-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18920-3">
    <skos:related rdf:resource="descriptor/19475-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18920-3">
    <skos:related rdf:resource="descriptor/11367-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16470-4">
    <skos:altLabel xml:lang="de">Verteidigungsfall</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16470-4">
    <skos:altLabel xml:lang="de">Kriegsziel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16470-4">
    <skos:altLabel xml:lang="de">Kriegspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16470-4">
    <skos:altLabel xml:lang="de">Kriegführung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18667-3">
    <skos:related rdf:resource="descriptor/19137-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18667-3">
    <skos:related rdf:resource="descriptor/11441-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70373">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13610-1">
    <skos:altLabel xml:lang="en">Haulage capacity</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13610-1">
    <skos:altLabel xml:lang="de">Verkehrsmittelkapazität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13610-1">
    <skos:altLabel xml:lang="de">Kapazität von Verkehrsmitteln</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15836-0">
    <skos:altLabel xml:lang="de">Ökonomische Psychologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18300-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10711-3">
    <skos:altLabel xml:lang="en">Foreign-trade zone</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10711-3">
    <skos:altLabel xml:lang="en">Foreign trade zone</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10711-3">
    <skos:altLabel xml:lang="en">Export processing zone</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10711-3">
    <skos:altLabel xml:lang="de">Zollfreigebiet</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10711-3">
    <skos:altLabel xml:lang="de">Zollausschluss</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10711-3">
    <skos:altLabel xml:lang="de">Zollausschluß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10711-3">
    <skos:altLabel xml:lang="de">Freizone</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10711-3">
    <skos:altLabel xml:lang="de">Freilager</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19633-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16051-3">
    <skos:narrower rdf:resource="descriptor/19434-0"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id65">
    <rdf:value xml:lang="de">Für fachlich spezialisierte Informationsbanken</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13818-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13587-5">
    <skos:related rdf:resource="descriptor/13580-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13587-5">
    <skos:related rdf:resource="descriptor/13515-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13587-5">
    <skos:related rdf:resource="descriptor/13584-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13587-5">
    <skos:related rdf:resource="descriptor/13513-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13587-5">
    <skos:related rdf:resource="descriptor/13622-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20341-4">
    <skos:prefLabel xml:lang="en">Content management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20341-4">
    <skos:prefLabel xml:lang="de">Content Management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71107">
    <rdfs:label xml:lang="de">Wertpapiermärkte</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15925-1">
    <skos:prefLabel xml:lang="en">Children</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15925-1">
    <skos:prefLabel xml:lang="de">Kinder</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73345">
    <skos:narrower rdf:resource="descriptor/15677-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73345">
    <skos:narrower rdf:resource="descriptor/10471-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73345">
    <skos:narrower rdf:resource="descriptor/10470-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73345">
    <skos:narrower rdf:resource="descriptor/10466-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73345">
    <skos:narrower rdf:resource="descriptor/18942-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73345">
    <skos:narrower rdf:resource="descriptor/10465-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73345">
    <skos:narrower rdf:resource="descriptor/16696-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73345">
    <skos:narrower rdf:resource="descriptor/18030-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73345">
    <skos:narrower rdf:resource="descriptor/12613-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73345">
    <skos:narrower rdf:resource="descriptor/19034-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73345">
    <skos:narrower rdf:resource="descriptor/12703-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73345">
    <skos:narrower rdf:resource="descriptor/10462-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73345">
    <skos:narrower rdf:resource="descriptor/19796-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73345">
    <skos:narrower rdf:resource="descriptor/12702-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73345">
    <skos:narrower rdf:resource="descriptor/19795-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73345">
    <skos:narrower rdf:resource="descriptor/12711-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73345">
    <skos:narrower rdf:resource="descriptor/19794-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73345">
    <skos:narrower rdf:resource="descriptor/10487-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73345">
    <skos:narrower rdf:resource="thsys/73343"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73345">
    <skos:narrower rdf:resource="thsys/73344"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14024-2">
    <skos:narrower rdf:resource="descriptor/14807-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14024-2">
    <skos:narrower rdf:resource="descriptor/14048-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18141-6">
    <skos:altLabel xml:lang="en">Off-balance-sheet activity</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18141-6">
    <skos:altLabel xml:lang="de">Bilanzunwirksames Geschäft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10850-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id131">
    <rdfs:seeAlso rdf:resource="descriptor/10176-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18913-0">
    <skos:broader rdf:resource="thsys/70832"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18913-0">
    <skos:broader rdf:resource="thsys/70458"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18913-0">
    <skos:broader rdf:resource="descriptor/25742-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12120-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19472-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12665-2">
    <skos:narrower rdf:resource="descriptor/12669-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12665-2">
    <skos:narrower rdf:resource="descriptor/12255-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12665-2">
    <skos:narrower rdf:resource="descriptor/12786-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12665-2">
    <skos:narrower rdf:resource="descriptor/19119-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12665-2">
    <skos:narrower rdf:resource="descriptor/18100-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12665-2">
    <skos:narrower rdf:resource="descriptor/19541-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18852-1">
    <skos:broader rdf:resource="thsys/70851"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18852-1">
    <skos:broader rdf:resource="descriptor/12111-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18377-5">
    <skos:broader rdf:resource="thsys/71011"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18377-5">
    <skos:broader rdf:resource="thsys/70811"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18377-5">
    <skos:broader rdf:resource="descriptor/18378-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70241">
    <skos:narrower rdf:resource="descriptor/16038-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70241">
    <skos:narrower rdf:resource="descriptor/16037-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70241">
    <skos:narrower rdf:resource="descriptor/18761-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70241">
    <skos:narrower rdf:resource="descriptor/16042-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70241">
    <skos:narrower rdf:resource="descriptor/18176-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70241">
    <skos:narrower rdf:resource="thsys/73392"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70241">
    <skos:narrower rdf:resource="thsys/73391"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16545-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14803-6">
    <skos:broader rdf:resource="thsys/70328"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10113-2">
    <skos:prefLabel xml:lang="en">Welfare economics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10113-2">
    <skos:prefLabel xml:lang="de">Wohlfahrtstheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19062-4">
    <skos:prefLabel xml:lang="en">Machine wholesale</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19062-4">
    <skos:prefLabel xml:lang="de">Maschinengroßhandel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19261-5">
    <skos:prefLabel xml:lang="en">Free banking</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19261-5">
    <skos:prefLabel xml:lang="de">Free Banking</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16495-2">
    <skos:related rdf:resource="descriptor/17983-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16495-2">
    <skos:related rdf:resource="descriptor/16496-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16495-2">
    <skos:related rdf:resource="descriptor/18772-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13260-2">
    <skos:altLabel xml:lang="de">Transport-Dienstleistungsgewerbe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13260-2">
    <skos:altLabel xml:lang="de">Speditionsgewerbe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13260-2">
    <skos:altLabel xml:lang="de">Internationale Spedition</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11303-6">
    <skos:altLabel xml:lang="en">Job design</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11303-6">
    <skos:altLabel xml:lang="en">Job description</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11303-6">
    <skos:altLabel xml:lang="de">Tätigkeitsstruktur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11303-6">
    <skos:altLabel xml:lang="de">Tätigkeitsprofil</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11303-6">
    <skos:altLabel xml:lang="de">Tätigkeitsmerkmal</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11303-6">
    <skos:altLabel xml:lang="de">Tätigkeitsfeld</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11303-6">
    <skos:altLabel xml:lang="de">Stellenbeschreibung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11303-6">
    <skos:altLabel xml:lang="de">Stellenanzeige</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11303-6">
    <skos:altLabel xml:lang="de">Berufsprofil</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11303-6">
    <skos:altLabel xml:lang="de">Berufsbild</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11303-6">
    <skos:altLabel xml:lang="de">Berufsanforderung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11303-6">
    <skos:altLabel xml:lang="de">Berufliche Aufgabenstruktur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11303-6">
    <skos:altLabel xml:lang="de">Arbeitsplatzbeschreibung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11303-6">
    <skos:altLabel xml:lang="de">Arbeitsinhalt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11303-6">
    <skos:altLabel xml:lang="de">Arbeitsanforderung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15705-1">
    <skos:broader rdf:resource="thsys/73387"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15705-1">
    <skos:broader rdf:resource="thsys/70109"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15705-1">
    <skos:broader rdf:resource="descriptor/10032-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15746-1">
    <skos:prefLabel xml:lang="en">Nature conservancy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15746-1">
    <skos:prefLabel xml:lang="de">Naturschutz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18341-5">
    <skos:broader rdf:resource="thsys/73362"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18341-5">
    <skos:broader rdf:resource="thsys/71018"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18341-5">
    <skos:broader rdf:resource="descriptor/11807-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18341-5">
    <skos:broader rdf:resource="descriptor/18772-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13535-3">
    <skos:broader rdf:resource="thsys/70969"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13535-3">
    <skos:broader rdf:resource="descriptor/13179-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18271-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70313">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19598-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13393-4">
    <skos:scopeNote xml:lang="de">Die christlichen Kirchen aller Konfessionen als Institution.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14998-0">
    <skos:broader rdf:resource="thsys/73298"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14998-0">
    <skos:broader rdf:resource="descriptor/14997-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17911-2">
    <skos:broader rdf:resource="thsys/71030"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17911-2">
    <skos:broader rdf:resource="descriptor/17931-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17911-2">
    <skos:broader rdf:resource="descriptor/19179-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10695-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71008">
    <rdfs:label xml:lang="en">Nonmetallic minerals industry</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71008">
    <rdfs:label xml:lang="de">Industrie der Steine und Erden</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16463-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16299-2">
    <skos:prefLabel xml:lang="en">Nation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16299-2">
    <skos:prefLabel xml:lang="de">Nation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70429">
    <skos:prefLabel xml:lang="en">G.03.01.02  Southern Africa</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70429">
    <skos:prefLabel xml:lang="de">G.03.01.02  Südliches Afrika</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71009">
    <skos:broader rdf:resource="thsys/71010"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11236-2">
    <skos:prefLabel xml:lang="en">Employment services</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11236-2">
    <skos:prefLabel xml:lang="de">Arbeitsvermittlung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10523-4">
    <zbwext:useInsteadNote rdf:nodeID="id88"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18855-2">
    <skos:prefLabel xml:lang="en">Social work of the church</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18855-2">
    <skos:prefLabel xml:lang="de">Kirchliche Sozialarbeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10020-2">
    <skos:prefLabel xml:lang="en">Savings</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10020-2">
    <skos:prefLabel xml:lang="de">Sparen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13611-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15998-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19012-5">
    <skos:broader rdf:resource="thsys/70245"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19012-5">
    <skos:broader rdf:resource="thsys/70233"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19012-5">
    <skos:broader rdf:resource="descriptor/12508-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19012-5">
    <skos:broader rdf:resource="descriptor/19566-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10931-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18231-5">
    <skos:altLabel xml:lang="de">Windkraft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13828-2">
    <skos:prefLabel xml:lang="en">Insurance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13828-2">
    <skos:prefLabel xml:lang="de">Versicherung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70539">
    <skos:prefLabel xml:lang="en">V.12.02.01  Energy policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70539">
    <skos:prefLabel xml:lang="de">V.12.02.01  Energiepolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10786-0">
    <skos:broader rdf:resource="thsys/71087"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10786-0">
    <skos:broader rdf:resource="descriptor/10791-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10424-6">
    <skos:altLabel xml:lang="en">Optimal economic growth</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10424-6">
    <skos:altLabel xml:lang="en">Golden rule of accumulation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10424-6">
    <skos:altLabel xml:lang="de">Ramsey-Wachstumspfad</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10424-6">
    <skos:altLabel xml:lang="de">Pontrjagin-Wachstumspfad</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10424-6">
    <skos:altLabel xml:lang="de">Optimaler Wachstumspfad</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10424-6">
    <skos:altLabel xml:lang="de">Goldene Regel der Akkumulation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10424-6">
    <skos:altLabel xml:lang="de">Goldene Akkumulationsregel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10424-6">
    <skos:altLabel xml:lang="de">Euler-Wachstumspfad</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10424-6">
    <skos:altLabel xml:lang="de">DoSSo-Wachstumspfad</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10424-6">
    <skos:altLabel xml:lang="de">Dorfman-Samuelson-Solow-Wachstumspfad</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19661-3">
    <skos:scopeNote xml:lang="de">Vgl. auch Automation, Datenverarbeitung, Informationssystem und Mikroelektronik.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19079-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14186-4">
    <skos:altLabel xml:lang="de">Solarenergie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71025">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13420-6">
    <skos:related rdf:resource="descriptor/16233-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13420-6">
    <skos:related rdf:resource="descriptor/19690-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13420-6">
    <skos:related rdf:resource="descriptor/16017-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13420-6">
    <skos:related rdf:resource="descriptor/19689-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13420-6">
    <skos:related rdf:resource="descriptor/16295-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13420-6">
    <skos:related rdf:resource="descriptor/19688-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13420-6">
    <skos:related rdf:resource="descriptor/16013-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13420-6">
    <skos:related rdf:resource="descriptor/16305-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13420-6">
    <skos:related rdf:resource="descriptor/19686-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13420-6">
    <skos:related rdf:resource="descriptor/19714-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13420-6">
    <skos:related rdf:resource="descriptor/16292-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13420-6">
    <skos:related rdf:resource="descriptor/11722-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13420-6">
    <skos:related rdf:resource="descriptor/16299-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13057-6">
    <skos:related rdf:resource="descriptor/13246-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13057-6">
    <skos:related rdf:resource="descriptor/14421-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13057-6">
    <skos:related rdf:resource="descriptor/13814-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13057-6">
    <skos:related rdf:resource="descriptor/13575-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24621-6">
    <skos:prefLabel xml:lang="en">Waste disposal</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24621-6">
    <skos:prefLabel xml:lang="de">Abfallentsorgung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14267-4">
    <skos:narrower rdf:resource="descriptor/14285-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14267-4">
    <skos:narrower rdf:resource="descriptor/14281-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17993-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11573-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18941-2">
    <skos:broader rdf:resource="thsys/70880"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18941-2">
    <skos:broader rdf:resource="thsys/70823"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18941-2">
    <skos:broader rdf:resource="descriptor/13345-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18581-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14068-3">
    <skos:altLabel xml:lang="en">Medical plant</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14068-3">
    <skos:altLabel xml:lang="de">Küchenkräuter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14068-3">
    <skos:altLabel xml:lang="de">Heilpflanze</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14068-3">
    <skos:altLabel xml:lang="de">Heilkräuter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14068-3">
    <skos:altLabel xml:lang="de">Arzneipflanze</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10032-2">
    <skos:altLabel xml:lang="en">Political economy</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10032-2">
    <skos:altLabel xml:lang="de">VWL</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10032-2">
    <skos:altLabel xml:lang="de">Volkswirtschaftslehre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10032-2">
    <skos:altLabel xml:lang="de">Politische Ökonomie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10032-2">
    <skos:altLabel xml:lang="de">Nationalökonomie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14514-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id245">
    <rdfs:seeAlso rdf:resource="descriptor/12047-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14766-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id319">
    <rdf:value xml:lang="de">Für den Kfz-Verkehr</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13339-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12675-6">
    <skos:narrower rdf:resource="descriptor/18287-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12675-6">
    <skos:narrower rdf:resource="descriptor/10545-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18448-1">
    <skos:prefLabel xml:lang="en">Music industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18448-1">
    <skos:prefLabel xml:lang="de">Musikwirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16215-6">
    <skos:related rdf:resource="descriptor/11063-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16215-6">
    <skos:related rdf:resource="descriptor/19611-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16215-6">
    <skos:related rdf:resource="descriptor/16209-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16215-6">
    <skos:related rdf:resource="descriptor/16680-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16215-6">
    <skos:related rdf:resource="descriptor/16441-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16215-6">
    <skos:related rdf:resource="descriptor/15872-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16215-6">
    <skos:related rdf:resource="descriptor/15797-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16215-6">
    <skos:related rdf:resource="descriptor/19361-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12365-0">
    <skos:related rdf:resource="descriptor/12070-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12365-0">
    <skos:related rdf:resource="descriptor/12393-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19071-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15638-4">
    <skos:related rdf:resource="descriptor/18402-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15638-4">
    <skos:related rdf:resource="descriptor/13378-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15638-4">
    <skos:related rdf:resource="descriptor/18429-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15638-4">
    <skos:related rdf:resource="descriptor/13959-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15638-4">
    <skos:related rdf:resource="descriptor/14599-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15638-4">
    <skos:related rdf:resource="descriptor/12996-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15638-4">
    <skos:related rdf:resource="descriptor/15635-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71036">
    <skos:broader rdf:resource="thsys/71039"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id271">
    <rdf:value xml:lang="de">Für ökonomisches Verhalten, das von diesen idealtypischen Annahmen abweicht</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24705-0">
    <skos:broader rdf:resource="thsys/70321"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24705-0">
    <skos:broader rdf:resource="thsys/70039"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14026-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18559-6">
    <skos:broader rdf:resource="thsys/70201"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18559-6">
    <skos:broader rdf:resource="descriptor/10016-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14473-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12509-2">
    <skos:broader rdf:resource="thsys/70578"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12509-2">
    <skos:broader rdf:resource="thsys/70270"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12509-2">
    <skos:broader rdf:resource="descriptor/10011-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14972-4">
    <skos:related rdf:resource="descriptor/18992-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14972-4">
    <skos:related rdf:resource="descriptor/16666-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14972-4">
    <skos:related rdf:resource="descriptor/14665-6"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id127">
    <rdfs:seeAlso rdf:resource="descriptor/10016-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18397-6">
    <skos:related rdf:resource="descriptor/12667-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10608-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10205-4">
    <skos:broader rdf:resource="thsys/71119"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10205-4">
    <skos:broader rdf:resource="descriptor/10905-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11843-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13738-3">
    <skos:scopeNote xml:lang="de">Markt für festverzinsliche Wertpapiere, vorwiegend für Anleihen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10353-3">
    <skos:broader rdf:resource="thsys/70858"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10353-3">
    <skos:broader rdf:resource="thsys/70274"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10353-3">
    <skos:broader rdf:resource="thsys/70020"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10353-3">
    <skos:broader rdf:resource="descriptor/10975-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10353-3">
    <skos:broader rdf:resource="descriptor/10347-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16249-3">
    <skos:broader rdf:resource="thsys/73383"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16249-3">
    <skos:broader rdf:resource="descriptor/16405-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19543-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14749-0">
    <skos:broader rdf:resource="thsys/73305"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14749-0">
    <skos:broader rdf:resource="descriptor/14751-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10125-2">
    <skos:prefLabel xml:lang="en">Capital productivity</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10125-2">
    <skos:prefLabel xml:lang="de">Kapitalproduktivität</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15076-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19688-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10182-4">
    <skos:prefLabel xml:lang="en">Microeconomic consumption function</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10182-4">
    <skos:prefLabel xml:lang="de">Mikroökonomische Konsumfunktion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19055-1">
    <skos:related rdf:resource="descriptor/13177-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19055-1">
    <skos:related rdf:resource="descriptor/18789-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19055-1">
    <skos:related rdf:resource="descriptor/13381-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19055-1">
    <skos:related rdf:resource="descriptor/18387-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12562-5">
    <skos:prefLabel xml:lang="en">Staff level</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12562-5">
    <skos:prefLabel xml:lang="de">Personalbestand</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18978-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18646-4">
    <skos:broader rdf:resource="thsys/70923"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18646-4">
    <skos:broader rdf:resource="descriptor/11543-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11649-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19238-0">
    <skos:narrower rdf:resource="descriptor/19826-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17927-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14455-3">
    <skos:broader rdf:resource="thsys/70415"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14455-3">
    <skos:broader rdf:resource="descriptor/14463-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70856">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19212-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16441-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16603-4">
    <skos:related rdf:resource="descriptor/16606-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16603-4">
    <skos:related rdf:resource="descriptor/16605-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16603-4">
    <skos:related rdf:resource="descriptor/10499-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16603-4">
    <skos:related rdf:resource="descriptor/16692-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16603-4">
    <skos:related rdf:resource="descriptor/10500-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16603-4">
    <skos:related rdf:resource="descriptor/10521-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14382-4">
    <skos:prefLabel xml:lang="en">Metalworking machinery</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14382-4">
    <skos:prefLabel xml:lang="de">Metallbearbeitungsmaschine</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73365">
    <skos:broader rdf:resource="thsys/73366"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11826-2">
    <skos:altLabel xml:lang="de">Tempo-30-Zone</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11826-2">
    <skos:altLabel xml:lang="de">Tempolimit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18835-1">
    <skos:broader rdf:resource="thsys/70859"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18835-1">
    <skos:broader rdf:resource="thsys/70856"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18835-1">
    <skos:broader rdf:resource="descriptor/12766-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12394-0">
    <skos:broader rdf:resource="thsys/70005"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12394-0">
    <skos:broader rdf:resource="descriptor/12357-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14942-6">
    <skos:altLabel xml:lang="de">Zwieback</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14942-6">
    <skos:altLabel xml:lang="de">Lebkuchen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14942-6">
    <skos:altLabel xml:lang="de">Kekse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14942-6">
    <skos:altLabel xml:lang="de">Dauerbackware</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11981-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16727-0">
    <skos:prefLabel xml:lang="en">Research method</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16727-0">
    <skos:prefLabel xml:lang="de">Wissenschaftliche Methode</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19423-5">
    <skos:related rdf:resource="descriptor/11671-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19423-5">
    <skos:related rdf:resource="descriptor/12907-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16604-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13761-1">
    <skos:broader rdf:resource="thsys/70899"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13761-1">
    <skos:broader rdf:resource="descriptor/13756-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14608-4">
    <skos:prefLabel xml:lang="en">Carbon</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14608-4">
    <skos:prefLabel xml:lang="de">Kohlenstoff</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11574-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13638-0">
    <skos:scopeNote xml:lang="de">Volkswirtschaftliche Analyse des ökonomischen Verhaltens im Fremdenverkehrsbereich, Reaktion auf Knappheiten und Veränderungen, individuelle Nutzenabwägungen usw.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12655-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12996-0">
    <skos:altLabel xml:lang="de">Karbochemische Industrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12996-0">
    <skos:altLabel xml:lang="de">Chemische Industrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12218-6">
    <skos:altLabel xml:lang="en">Cash-flow</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12218-6">
    <skos:altLabel xml:lang="de">Kapitalfluss aus Umsatz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12218-6">
    <skos:altLabel xml:lang="de">Kapitalfluß aus Umsatz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12218-6">
    <skos:altLabel xml:lang="de">Cash-Flow-Analyse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19775-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18592-1">
    <skos:broader rdf:resource="thsys/73389"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18592-1">
    <skos:broader rdf:resource="thsys/70936"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18592-1">
    <skos:broader rdf:resource="descriptor/18591-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12883-6">
    <skos:related rdf:resource="descriptor/12473-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12883-6">
    <skos:related rdf:resource="descriptor/12330-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19172-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15480-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17927-1">
    <skos:prefLabel xml:lang="en">Paraguay</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17927-1">
    <skos:prefLabel xml:lang="de">Paraguay</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10397-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15483-0">
    <skos:narrower rdf:resource="descriptor/15067-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15483-0">
    <skos:narrower rdf:resource="descriptor/15515-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15483-0">
    <skos:narrower rdf:resource="descriptor/15055-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15483-0">
    <skos:narrower rdf:resource="descriptor/15555-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19771-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73352">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19064-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15411-4">
    <skos:prefLabel xml:lang="en">Probit model</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15411-4">
    <skos:prefLabel xml:lang="de">Probit-Modell</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16204-4">
    <skos:altLabel xml:lang="de">Konfliktforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16204-4">
    <skos:altLabel xml:lang="de">Friedens- und Konfliktforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18967-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18253-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13073-1">
    <skos:related rdf:resource="descriptor/18004-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18769-2">
    <skos:broader rdf:resource="thsys/70880"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18769-2">
    <skos:broader rdf:resource="descriptor/16774-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15923-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15753-4">
    <skos:narrower rdf:resource="descriptor/18378-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15753-4">
    <skos:narrower rdf:resource="descriptor/11812-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19645-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13238-2">
    <skos:related rdf:resource="descriptor/18450-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13238-2">
    <skos:related rdf:resource="descriptor/14997-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13238-2">
    <skos:related rdf:resource="descriptor/13164-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13238-2">
    <skos:related rdf:resource="descriptor/18196-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13238-2">
    <skos:related rdf:resource="descriptor/14082-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13238-2">
    <skos:related rdf:resource="descriptor/13219-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13238-2">
    <skos:related rdf:resource="descriptor/13188-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15806-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15125-5">
    <skos:prefLabel xml:lang="en">Panel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15125-5">
    <skos:prefLabel xml:lang="de">Panel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16769-5">
    <skos:altLabel xml:lang="de">Magazin in Bibliotheken</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16769-5">
    <skos:altLabel xml:lang="de">Bibliotheksmagazin</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10297-1">
    <skos:altLabel xml:lang="de">Gesamtwirtschaftliche Lagerhaltung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13329-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18111-1">
    <skos:related rdf:resource="descriptor/13671-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18111-1">
    <skos:related rdf:resource="descriptor/14765-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70052">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11044-4">
    <skos:scopeNote xml:lang="de">Für alle Wirtschafts- und Gesellschaftsordnungen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16031-2">
    <skos:related rdf:resource="descriptor/16038-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16031-2">
    <skos:related rdf:resource="descriptor/16050-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16031-2">
    <skos:related rdf:resource="descriptor/18841-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16031-2">
    <skos:related rdf:resource="descriptor/12180-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11353-5">
    <skos:prefLabel xml:lang="en">Education</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11353-5">
    <skos:prefLabel xml:lang="de">Bildung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10210-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18200-2">
    <skos:prefLabel xml:lang="en">International natural rubber agreement</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18200-2">
    <skos:prefLabel xml:lang="de">Naturkautschukabkommen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13684-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18968-3">
    <skos:broader rdf:resource="thsys/73357"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18968-3">
    <skos:broader rdf:resource="thsys/70814"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18968-3">
    <skos:broader rdf:resource="thsys/70142"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18968-3">
    <skos:broader rdf:resource="descriptor/19759-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16950-4">
    <skos:prefLabel xml:lang="en">Austria</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16950-4">
    <skos:prefLabel xml:lang="de">Österreich</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12239-5">
    <skos:related rdf:resource="descriptor/12265-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12239-5">
    <skos:related rdf:resource="descriptor/12284-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73300">
    <skos:narrower rdf:resource="descriptor/14985-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73300">
    <skos:narrower rdf:resource="descriptor/14983-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73300">
    <skos:narrower rdf:resource="descriptor/14981-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73300">
    <skos:narrower rdf:resource="descriptor/14980-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73300">
    <skos:narrower rdf:resource="descriptor/14984-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73300">
    <skos:narrower rdf:resource="descriptor/14976-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73300">
    <skos:narrower rdf:resource="descriptor/14949-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73300">
    <skos:narrower rdf:resource="descriptor/14975-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73300">
    <skos:narrower rdf:resource="descriptor/14973-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73300">
    <skos:narrower rdf:resource="descriptor/14972-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73300">
    <skos:narrower rdf:resource="descriptor/14971-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11932-3">
    <skos:related rdf:resource="descriptor/13862-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11932-3">
    <skos:related rdf:resource="descriptor/11915-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11932-3">
    <skos:related rdf:resource="descriptor/11913-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11932-3">
    <skos:related rdf:resource="descriptor/11990-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11932-3">
    <skos:related rdf:resource="descriptor/10641-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19150-0">
    <skos:broader rdf:resource="thsys/73103"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19150-0">
    <skos:broader rdf:resource="thsys/72182"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19150-0">
    <skos:broader rdf:resource="thsys/71044"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19150-0">
    <skos:broader rdf:resource="thsys/71035"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19150-0">
    <skos:broader rdf:resource="thsys/70011"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19150-0">
    <skos:broader rdf:resource="descriptor/15064-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19083-3">
    <skos:altLabel xml:lang="de">Nordosteuropäische Staaten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19083-3">
    <skos:altLabel xml:lang="de">Nordosteuropäische Länder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19083-3">
    <skos:altLabel xml:lang="de">Baltische Staaten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19083-3">
    <skos:altLabel xml:lang="de">Baltische Länder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19083-3">
    <skos:altLabel xml:lang="de">Baltikum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18558-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16525-5">
    <skos:related rdf:resource="descriptor/18907-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18227-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70493">
    <skos:prefLabel xml:lang="en">V.02.01  Household economy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70493">
    <skos:prefLabel xml:lang="de">V.02.01  Haushaltsökonomik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13890-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13397-3">
    <skos:editorialNote xml:lang="de">Möglichst spezifischer indexieren.</skos:editorialNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18293-4">
    <skos:prefLabel xml:lang="en">Ceramic tile</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18293-4">
    <skos:prefLabel xml:lang="de">Fliese</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19750-4">
    <skos:related rdf:resource="descriptor/16724-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19750-4">
    <skos:related rdf:resource="descriptor/16606-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19750-4">
    <skos:related rdf:resource="descriptor/15654-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13355-5">
    <skos:broader rdf:resource="thsys/71055"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13355-5">
    <skos:broader rdf:resource="thsys/70828"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13355-5">
    <skos:broader rdf:resource="descriptor/18535-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21110-4">
    <zbwext:useInsteadNote rdf:nodeID="id356"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21110-4">
    <zbwext:useInsteadNote rdf:nodeID="id355"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11289-2">
    <skos:broader rdf:resource="thsys/71058"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11289-2">
    <skos:broader rdf:resource="thsys/70872"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11289-2">
    <skos:broader rdf:resource="descriptor/11559-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11289-2">
    <skos:broader rdf:resource="descriptor/11294-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11289-2">
    <skos:broader rdf:resource="descriptor/11262-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11533-3">
    <skos:scopeNote xml:lang="de">Nur für die Geschichte der öffentlichen Finanzwirtschaft.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18960-5">
    <skos:narrower rdf:resource="descriptor/19561-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18960-5">
    <skos:narrower rdf:resource="descriptor/18962-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71007">
    <skos:narrower rdf:resource="descriptor/12992-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71007">
    <skos:narrower rdf:resource="descriptor/12988-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71007">
    <skos:narrower rdf:resource="descriptor/19184-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71007">
    <skos:narrower rdf:resource="descriptor/19185-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71007">
    <skos:narrower rdf:resource="descriptor/18392-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71007">
    <skos:narrower rdf:resource="descriptor/18391-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71007">
    <skos:narrower rdf:resource="descriptor/12985-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71007">
    <skos:narrower rdf:resource="descriptor/12982-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19602-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19073-6">
    <skos:scopeNote xml:lang="de">Mit "Theorie" werden wirtschaftswissenschaftliche Arbeiten indexiert, die ein Thema insgesamt oder zu wesentlichen Teilen theoretisch analysieren oder darstellen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70854">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.23.01.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15350-5">
    <skos:related rdf:resource="descriptor/15149-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18509-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13377-2">
    <skos:prefLabel xml:lang="en">Inquiry office</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13377-2">
    <skos:prefLabel xml:lang="de">Auskunftei</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16026-2">
    <skos:related rdf:resource="descriptor/16038-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16026-2">
    <skos:related rdf:resource="descriptor/16032-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16026-2">
    <skos:related rdf:resource="descriptor/12181-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18859-1">
    <skos:narrower rdf:resource="descriptor/16353-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18859-1">
    <skos:narrower rdf:resource="descriptor/15861-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18859-1">
    <skos:narrower rdf:resource="descriptor/15722-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18859-1">
    <skos:narrower rdf:resource="descriptor/19646-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18859-1">
    <skos:narrower rdf:resource="descriptor/15860-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18859-1">
    <skos:narrower rdf:resource="descriptor/24847-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18859-1">
    <skos:narrower rdf:resource="descriptor/16759-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18859-1">
    <skos:narrower rdf:resource="descriptor/18916-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18859-1">
    <skos:narrower rdf:resource="descriptor/18335-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18859-1">
    <skos:narrower rdf:resource="descriptor/18860-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18859-1">
    <skos:narrower rdf:resource="descriptor/25881-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15881-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18820-0">
    <skos:prefLabel xml:lang="en">Fire control</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18820-0">
    <skos:prefLabel xml:lang="de">Brandschutz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24707-3">
    <skos:prefLabel xml:lang="en">Herd behaviour</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24707-3">
    <skos:prefLabel xml:lang="de">Herdenverhalten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18209-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10680-2">
    <skos:altLabel xml:lang="de">Zahlungsbilanzüberschuss</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10680-2">
    <skos:altLabel xml:lang="de">Zahlungsbilanzüberschuß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10680-2">
    <skos:altLabel xml:lang="de">Zahlungsbilanzdefizit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18396-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13964-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13989-6">
    <skos:broader rdf:resource="thsys/73378"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13989-6">
    <skos:broader rdf:resource="thsys/71054"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13989-6">
    <skos:broader rdf:resource="descriptor/18535-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12015-6">
    <skos:narrower rdf:resource="descriptor/19618-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12015-6">
    <skos:narrower rdf:resource="descriptor/12014-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12015-6">
    <skos:narrower rdf:resource="descriptor/19339-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10251-4">
    <skos:altLabel xml:lang="de">Dynamischer Multiplikator</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16729-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11331-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11533-3">
    <skos:prefLabel xml:lang="en">History of public finance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11533-3">
    <skos:prefLabel xml:lang="de">Finanzgeschichte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14635-1">
    <skos:related rdf:resource="descriptor/14997-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14635-1">
    <skos:related rdf:resource="descriptor/15005-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13576-3">
    <skos:prefLabel xml:lang="en">Marine engineering</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13576-3">
    <skos:prefLabel xml:lang="de">Schifffahrtstechnik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16349-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14255-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11402-4">
    <skos:broader rdf:resource="thsys/73369"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11402-4">
    <skos:broader rdf:resource="thsys/70844"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11402-4">
    <skos:broader rdf:resource="thsys/70176"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11402-4">
    <skos:broader rdf:resource="descriptor/11397-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70876">
    <skos:prefLabel xml:lang="de">W.21  Unternehmensdienstleistungen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19801-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11213-0">
    <zbwext:useInsteadNote rdf:nodeID="id106"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18947-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15708-2">
    <skos:related rdf:resource="descriptor/15949-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15708-2">
    <skos:related rdf:resource="descriptor/15954-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15708-2">
    <skos:related rdf:resource="descriptor/15941-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15708-2">
    <skos:related rdf:resource="descriptor/15916-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15708-2">
    <skos:related rdf:resource="descriptor/15914-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15708-2">
    <skos:related rdf:resource="descriptor/15061-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15708-2">
    <skos:related rdf:resource="descriptor/15912-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15708-2">
    <skos:related rdf:resource="descriptor/15911-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15708-2">
    <skos:related rdf:resource="descriptor/15909-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16509-3">
    <skos:narrower rdf:resource="descriptor/16513-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16509-3">
    <skos:narrower rdf:resource="descriptor/16512-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16509-3">
    <skos:narrower rdf:resource="descriptor/16486-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16509-3">
    <skos:narrower rdf:resource="descriptor/16515-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16509-3">
    <skos:narrower rdf:resource="descriptor/16549-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18663-4">
    <skos:broader rdf:resource="thsys/71063"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18663-4">
    <skos:broader rdf:resource="thsys/70922"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18663-4">
    <skos:broader rdf:resource="descriptor/11716-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71104">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11892-3">
    <skos:prefLabel xml:lang="en">New towns</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11892-3">
    <skos:prefLabel xml:lang="de">Stadtgründung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14353-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12573-0">
    <skos:related rdf:resource="descriptor/19007-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12573-0">
    <skos:related rdf:resource="descriptor/12572-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12573-0">
    <skos:related rdf:resource="descriptor/11203-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12573-0">
    <skos:related rdf:resource="descriptor/11271-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12573-0">
    <skos:related rdf:resource="descriptor/11389-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12573-0">
    <skos:related rdf:resource="descriptor/11236-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12573-0">
    <skos:related rdf:resource="descriptor/18140-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12573-0">
    <skos:related rdf:resource="descriptor/11266-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12573-0">
    <skos:related rdf:resource="descriptor/11184-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12573-0">
    <skos:related rdf:resource="descriptor/11303-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14341-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18315-6">
    <skos:prefLabel xml:lang="en">Tropical agriculture</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18315-6">
    <skos:prefLabel xml:lang="de">Tropische Landwirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16209-1">
    <skos:narrower rdf:resource="descriptor/16233-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16209-1">
    <skos:narrower rdf:resource="descriptor/11170-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16209-1">
    <skos:narrower rdf:resource="descriptor/16204-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16001-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18213-0">
    <skos:broader rdf:resource="thsys/73298"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18213-0">
    <skos:broader rdf:resource="descriptor/14931-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18213-0">
    <skos:broader rdf:resource="descriptor/18222-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15336-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14145-4">
    <skos:broader rdf:resource="thsys/70403"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14145-4">
    <skos:broader rdf:resource="descriptor/14169-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14145-4">
    <skos:broader rdf:resource="descriptor/14054-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15725-2">
    <skos:broader rdf:resource="thsys/73370"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15725-2">
    <skos:broader rdf:resource="thsys/70833"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15725-2">
    <skos:broader rdf:resource="thsys/70324"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15725-2">
    <skos:broader rdf:resource="descriptor/16283-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19572-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73349">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18988-4">
    <skos:altLabel xml:lang="de">Forstliche Produktionslehre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18988-4">
    <skos:altLabel xml:lang="de">Forstliche Betriebslehre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18988-4">
    <skos:altLabel xml:lang="de">Forstbetriebswirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73362">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.04.04.04</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18490-2">
    <skos:prefLabel xml:lang="en">Industrial location</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18490-2">
    <skos:prefLabel xml:lang="de">Industriestandort</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10833-3">
    <skos:prefLabel xml:lang="en">Foreign trade</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10833-3">
    <skos:prefLabel xml:lang="de">Außenhandel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10340-5">
    <skos:broader rdf:resource="thsys/70020"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10340-5">
    <skos:broader rdf:resource="descriptor/10624-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10340-5">
    <skos:broader rdf:resource="descriptor/10338-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16579-3">
    <skos:scopeNote xml:lang="de">Allgemeine theoretische Analyse gesellschaftlicher Handlungszusammenhänge,sozialer Umbrüche, Krisen und sozialen Wandels. Meist soziologische Ansätze, von denen die Systemtheorie und die Kritische Theorie zu den bekanntesten gehören.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18099-0">
    <skos:related rdf:resource="descriptor/18954-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18099-0">
    <skos:related rdf:resource="descriptor/12394-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18099-0">
    <skos:related rdf:resource="descriptor/18098-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18099-0">
    <skos:related rdf:resource="descriptor/12775-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18099-0">
    <skos:related rdf:resource="descriptor/18831-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18099-0">
    <skos:related rdf:resource="descriptor/25822-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14063-6">
    <skos:prefLabel xml:lang="en">Jute</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14063-6">
    <skos:prefLabel xml:lang="de">Jute</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12932-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18254-0">
    <skos:altLabel xml:lang="de">Trockeneis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18254-0">
    <skos:altLabel xml:lang="de">Stickstoff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18254-0">
    <skos:altLabel xml:lang="de">Sauerstoff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18254-0">
    <skos:altLabel xml:lang="de">Industriegas</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11331-1">
    <skos:broader rdf:resource="thsys/70278"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11331-1">
    <skos:broader rdf:resource="thsys/70245"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11331-1">
    <skos:broader rdf:resource="descriptor/11345-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11331-1">
    <skos:broader rdf:resource="descriptor/11322-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70873">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15720-5">
    <skos:broader rdf:resource="thsys/70321"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15720-5">
    <skos:broader rdf:resource="thsys/70091"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19825-6">
    <skos:broader rdf:resource="thsys/71058"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14091-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19848-1">
    <skos:broader rdf:resource="thsys/73391"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19848-1">
    <skos:broader rdf:resource="thsys/70383"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19848-1">
    <skos:broader rdf:resource="descriptor/19847-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12788-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19066-3">
    <skos:related rdf:resource="descriptor/13513-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70368">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10614-1">
    <skos:prefLabel xml:lang="en">International division of labour</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10614-1">
    <skos:prefLabel xml:lang="de">Internationale Arbeitsteilung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13632-5">
    <skos:prefLabel xml:lang="en">Passenger transport</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13632-5">
    <skos:prefLabel xml:lang="de">Personenverkehr</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19671-0">
    <skos:altLabel xml:lang="en">Quotation (Exchange)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19671-0">
    <skos:altLabel xml:lang="en">Listing requirements</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19671-0">
    <skos:altLabel xml:lang="de">Notierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19671-0">
    <skos:altLabel xml:lang="de">Kursnotiz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19671-0">
    <skos:altLabel xml:lang="de">Kursnotierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19671-0">
    <skos:altLabel xml:lang="de">Kursfeststellung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12761-6">
    <skos:broader rdf:resource="thsys/71035"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12761-6">
    <skos:broader rdf:resource="thsys/70029"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12761-6">
    <skos:broader rdf:resource="descriptor/15055-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12761-6">
    <skos:broader rdf:resource="descriptor/12747-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18133-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15086-3">
    <skos:related rdf:resource="descriptor/10007-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19767-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13110-0">
    <skos:narrower rdf:resource="descriptor/13113-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13110-0">
    <skos:narrower rdf:resource="descriptor/13106-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13037-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14355-0">
    <skos:related rdf:resource="descriptor/14479-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17967-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14975-5">
    <skos:related rdf:resource="descriptor/18221-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15272-6">
    <skos:altLabel xml:lang="en">Statistical efficiency</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15272-6">
    <skos:altLabel xml:lang="en">Estimator</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15272-6">
    <skos:altLabel xml:lang="en">Confidence interval</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15272-6">
    <skos:altLabel xml:lang="de">Statistische Effizienz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15272-6">
    <skos:altLabel xml:lang="de">Schätzverfahren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15272-6">
    <skos:altLabel xml:lang="de">Schätzmethode</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15272-6">
    <skos:altLabel xml:lang="de">Schätzfunktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15272-6">
    <skos:altLabel xml:lang="de">Schätzer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15272-6">
    <skos:altLabel xml:lang="de">Konfidenzschätzung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15272-6">
    <skos:altLabel xml:lang="de">Hochrechnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14567-6">
    <skos:broader rdf:resource="thsys/70346"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13324-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12121-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18898-5">
    <skos:altLabel xml:lang="en">Acquired Immunodeficiency Syndrome</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18898-5">
    <skos:altLabel xml:lang="en">Acquired Immune Deficiency Syndrome</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18898-5">
    <skos:altLabel xml:lang="de">HIV</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18402-4">
    <skos:prefLabel xml:lang="en">Metallurgy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18402-4">
    <skos:prefLabel xml:lang="de">Metallurgie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14567-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19846-5">
    <skos:broader rdf:resource="thsys/73384"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19846-5">
    <skos:broader rdf:resource="thsys/70814"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19846-5">
    <skos:broader rdf:resource="descriptor/10300-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15703-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id33">
    <rdfs:seeAlso rdf:resource="descriptor/10829-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15375-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14900-1">
    <skos:altLabel xml:lang="de">Sockenhalter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14900-1">
    <skos:altLabel xml:lang="de">Schirm</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14900-1">
    <skos:altLabel xml:lang="de">Regenschirm</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14900-1">
    <skos:altLabel xml:lang="de">Krawatte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14900-1">
    <skos:altLabel xml:lang="de">Hosenträger</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14900-1">
    <skos:altLabel xml:lang="de">Halstuch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13453-5">
    <skos:related rdf:resource="descriptor/18093-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13453-5">
    <skos:related rdf:resource="descriptor/13440-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19542-4">
    <skos:prefLabel xml:lang="en">Conjoint analysis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19542-4">
    <skos:prefLabel xml:lang="de">Conjoint-Analyse</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11183-4">
    <skos:broader rdf:resource="thsys/70051"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11183-4">
    <skos:broader rdf:resource="descriptor/10012-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16195-0">
    <skos:altLabel xml:lang="en">Crime</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16195-0">
    <skos:altLabel xml:lang="en">Blackmail</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16195-0">
    <skos:altLabel xml:lang="de">Verbrechen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16195-0">
    <skos:altLabel xml:lang="de">Straftat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16195-0">
    <skos:altLabel xml:lang="de">Straffälligkeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16195-0">
    <skos:altLabel xml:lang="de">Kriminelles Verhalten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16195-0">
    <skos:altLabel xml:lang="de">Erpressung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16195-0">
    <skos:altLabel xml:lang="de">Diebstahl</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16195-0">
    <skos:altLabel xml:lang="de">Delinquenz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14161-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13827-4">
    <skos:prefLabel xml:lang="en">Loss (Insurance)</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13827-4">
    <skos:prefLabel xml:lang="de">Versicherungsschaden</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10194-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19373-1">
    <skos:scopeNote xml:lang="de">Zu den Parafiski zählen z.B. die Sozialversicherungen, berufsständische Organisationen und kirchliche Einrichtungen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19651-6">
    <skos:altLabel xml:lang="en">Website</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19651-6">
    <skos:altLabel xml:lang="en">Web-site</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19651-6">
    <skos:altLabel xml:lang="en">Web-page</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19651-6">
    <skos:altLabel xml:lang="en">Webdesign</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19651-6">
    <skos:altLabel xml:lang="en">Web-design</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19651-6">
    <skos:altLabel xml:lang="en">Internet-site</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19651-6">
    <skos:altLabel xml:lang="de">Web-Seite</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19651-6">
    <skos:altLabel xml:lang="de">Internet-Seite</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11068-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11709-6">
    <skos:related rdf:resource="descriptor/12855-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11709-6">
    <skos:related rdf:resource="descriptor/12876-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11709-6">
    <skos:related rdf:resource="descriptor/10831-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11709-6">
    <skos:related rdf:resource="descriptor/11638-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11709-6">
    <skos:related rdf:resource="descriptor/12880-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11709-6">
    <skos:related rdf:resource="descriptor/16112-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11709-6">
    <skos:related rdf:resource="descriptor/12889-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11709-6">
    <skos:related rdf:resource="descriptor/12214-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13073-1">
    <skos:broader rdf:resource="thsys/73297"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13073-1">
    <skos:broader rdf:resource="descriptor/13072-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14041-2">
    <skos:broader rdf:resource="thsys/70201"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14014-5">
    <skos:broader rdf:resource="thsys/71053"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14014-5">
    <skos:broader rdf:resource="thsys/70834"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14014-5">
    <skos:broader rdf:resource="descriptor/18535-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16489-4">
    <skos:related rdf:resource="descriptor/16463-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16489-4">
    <skos:related rdf:resource="descriptor/16417-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16489-4">
    <skos:related rdf:resource="descriptor/16648-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16489-4">
    <skos:related rdf:resource="descriptor/16471-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16489-4">
    <skos:related rdf:resource="descriptor/16435-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16489-4">
    <skos:related rdf:resource="descriptor/11083-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16489-4">
    <skos:related rdf:resource="descriptor/16502-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16489-4">
    <skos:related rdf:resource="descriptor/10557-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16489-4">
    <skos:related rdf:resource="descriptor/16493-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16489-4">
    <skos:related rdf:resource="descriptor/19716-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19704-4">
    <skos:altLabel xml:lang="en">Crime prevention</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19704-4">
    <skos:altLabel xml:lang="en">Crime deterrence</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19704-4">
    <skos:altLabel xml:lang="de">Verbrechensbekämpfung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19704-4">
    <skos:altLabel xml:lang="de">Strafrechtspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19704-4">
    <skos:altLabel xml:lang="de">Strafpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19704-4">
    <skos:altLabel xml:lang="de">Kriminalitätsvorbeugung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19704-4">
    <skos:altLabel xml:lang="de">Kriminalitätsprophylaxe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19704-4">
    <skos:altLabel xml:lang="de">Kriminalitätspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14601-4">
    <skos:broader rdf:resource="thsys/70014"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18061-4">
    <skos:altLabel xml:lang="en">Debt reduction</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18061-4">
    <skos:altLabel xml:lang="de">Schuldenerleichterung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18061-4">
    <skos:altLabel xml:lang="de">Schuldenerlaß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18061-4">
    <skos:altLabel xml:lang="de">Brady Plan</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19288-6">
    <skos:prefLabel xml:lang="en">Colonial economy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19288-6">
    <skos:prefLabel xml:lang="de">Kolonialwirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70310">
    <skos:prefLabel xml:lang="de">B.06.02  Fertigungssysteme und Fertigungstechnik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14739-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14969-0">
    <skos:altLabel xml:lang="de">Verarbeiteter Fisch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14969-0">
    <skos:altLabel xml:lang="de">Trockenfisch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14969-0">
    <skos:altLabel xml:lang="de">Surimi</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14969-0">
    <skos:altLabel xml:lang="de">Sardelle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14969-0">
    <skos:altLabel xml:lang="de">Salzfisch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14969-0">
    <skos:altLabel xml:lang="de">Rogen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14969-0">
    <skos:altLabel xml:lang="de">Räucherfisch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14969-0">
    <skos:altLabel xml:lang="de">Kaviar</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14969-0">
    <skos:altLabel xml:lang="de">Fischsalat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14969-0">
    <skos:altLabel xml:lang="de">Anchovis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10829-1">
    <skos:related rdf:resource="descriptor/10831-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10829-1">
    <skos:related rdf:resource="descriptor/11703-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10829-1">
    <skos:related rdf:resource="descriptor/10621-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10829-1">
    <skos:related rdf:resource="descriptor/10827-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17980-4">
    <skos:related rdf:resource="descriptor/17199-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14513-1">
    <skos:narrower rdf:resource="descriptor/14524-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15328-5">
    <skos:narrower rdf:resource="descriptor/20506-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18312-5">
    <skos:scopeNote xml:lang="de">Regelungen des öffentlichen und des Privatrechts sowie des Völkerrechts.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14329-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12153-1">
    <skos:related rdf:resource="descriptor/20977-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19797-6">
    <skos:scopeNote xml:lang="de">Technische Verfahren, die an den Eigenschaften einzelner Atome ansetzen und an Prozessen, die im Nano-Bereich ablaufen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15045-3">
    <skos:broader rdf:resource="thsys/70269"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15045-3">
    <skos:broader rdf:resource="descriptor/15644-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18630-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12718-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14549-1">
    <skos:broader rdf:resource="thsys/70346"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14549-1">
    <skos:broader rdf:resource="thsys/70333"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14549-1">
    <skos:broader rdf:resource="descriptor/14051-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14549-1">
    <skos:broader rdf:resource="descriptor/14548-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19687-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19018-0">
    <skos:related rdf:resource="descriptor/14973-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11417-5">
    <skos:altLabel xml:lang="de">Währungsgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11417-5">
    <skos:altLabel xml:lang="de">Münzrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11417-5">
    <skos:altLabel xml:lang="de">Geldrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10398-2">
    <skos:prefLabel xml:lang="en">Policy rules</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10398-2">
    <skos:prefLabel xml:lang="de">Regelgebundene Politik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18696-3">
    <skos:broader rdf:resource="thsys/70259"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18696-3">
    <skos:broader rdf:resource="descriptor/10236-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14991-0">
    <skos:broader rdf:resource="thsys/73299"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14991-0">
    <skos:broader rdf:resource="descriptor/14989-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18694-0">
    <skos:broader rdf:resource="thsys/71089"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16985-6">
    <skos:altLabel xml:lang="en">Norge</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16985-6">
    <skos:altLabel xml:lang="de">Königreich Norwegen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18282-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19819-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10293-2">
    <skos:altLabel xml:lang="de">Risiko von Investitionen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13186-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71080">
    <skos:narrower rdf:resource="descriptor/11574-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71080">
    <skos:narrower rdf:resource="descriptor/11573-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71080">
    <skos:narrower rdf:resource="descriptor/19384-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71080">
    <skos:narrower rdf:resource="descriptor/11572-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71080">
    <skos:narrower rdf:resource="descriptor/11570-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71080">
    <skos:narrower rdf:resource="descriptor/11569-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71080">
    <skos:narrower rdf:resource="descriptor/11568-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71080">
    <skos:narrower rdf:resource="descriptor/11567-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71080">
    <skos:narrower rdf:resource="descriptor/11241-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71080">
    <skos:narrower rdf:resource="descriptor/11240-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71080">
    <skos:narrower rdf:resource="descriptor/11566-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71080">
    <skos:narrower rdf:resource="descriptor/11565-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71080">
    <skos:narrower rdf:resource="descriptor/20906-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71080">
    <skos:narrower rdf:resource="descriptor/18482-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71080">
    <skos:narrower rdf:resource="descriptor/11564-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71080">
    <skos:narrower rdf:resource="descriptor/11562-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10334-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14985-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12876-3">
    <skos:altLabel xml:lang="en">Tax management</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12876-3">
    <skos:altLabel xml:lang="de">Steuertaktik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12876-3">
    <skos:altLabel xml:lang="de">Steuerpolitik des Betriebs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12876-3">
    <skos:altLabel xml:lang="de">Steuerminimierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12876-3">
    <skos:altLabel xml:lang="de">Steuerbilanzpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12876-3">
    <skos:altLabel xml:lang="de">Betriebliche Steuerpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12876-3">
    <skos:altLabel xml:lang="de">Betriebliche Steuergestaltung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13414-1">
    <skos:prefLabel xml:lang="en">Province</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13414-1">
    <skos:prefLabel xml:lang="de">Provinz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12700-5">
    <skos:altLabel xml:lang="en">Surplus capacity</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12700-5">
    <skos:altLabel xml:lang="en">Capital utilization</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12700-5">
    <skos:altLabel xml:lang="en">Capacity working</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12700-5">
    <skos:altLabel xml:lang="de">Überkapazität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12700-5">
    <skos:altLabel xml:lang="de">Kapazitätsnutzungsgrad</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12700-5">
    <skos:altLabel xml:lang="de">Beschäftigungsgrad</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12700-5">
    <skos:altLabel xml:lang="de">Auslastung des Wirtschaftspotenzials</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12700-5">
    <skos:altLabel xml:lang="de">Auslastung des Wirtschaftspotentials</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12603-3">
    <skos:related rdf:resource="descriptor/16545-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12603-3">
    <skos:related rdf:resource="descriptor/12580-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12603-3">
    <skos:related rdf:resource="descriptor/15837-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12603-3">
    <skos:related rdf:resource="descriptor/12602-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12603-3">
    <skos:related rdf:resource="descriptor/11309-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12603-3">
    <skos:related rdf:resource="descriptor/12621-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12603-3">
    <skos:related rdf:resource="descriptor/12607-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12603-3">
    <skos:related rdf:resource="descriptor/11314-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12603-3">
    <skos:related rdf:resource="descriptor/11305-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12603-3">
    <skos:related rdf:resource="descriptor/16124-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70562">
    <rdfs:label xml:lang="en">Organization</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70562">
    <rdfs:label xml:lang="de">Organisation</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12089-5">
    <skos:broader rdf:resource="thsys/70485"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12985-5">
    <skos:narrower rdf:resource="descriptor/12988-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12985-5">
    <skos:narrower rdf:resource="descriptor/12982-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73371">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19170-1">
    <skos:narrower rdf:resource="descriptor/17934-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19170-1">
    <skos:narrower rdf:resource="descriptor/17928-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19170-1">
    <skos:narrower rdf:resource="descriptor/17923-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19170-1">
    <skos:narrower rdf:resource="descriptor/17920-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19170-1">
    <skos:narrower rdf:resource="descriptor/17914-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11678-5">
    <skos:related rdf:resource="descriptor/19424-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11678-5">
    <skos:related rdf:resource="descriptor/11679-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11678-5">
    <skos:related rdf:resource="descriptor/12318-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11678-5">
    <skos:related rdf:resource="descriptor/19417-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11678-5">
    <skos:related rdf:resource="descriptor/12848-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16563-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12209-0">
    <skos:altLabel xml:lang="de">Investitionstheorie des Betriebs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16542-5">
    <zbwext:useInsteadNote rdf:nodeID="id297"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70982">
    <skos:narrower rdf:resource="descriptor/18366-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70982">
    <skos:narrower rdf:resource="descriptor/18182-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70982">
    <skos:narrower rdf:resource="descriptor/18447-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70982">
    <skos:narrower rdf:resource="descriptor/12925-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70982">
    <skos:narrower rdf:resource="descriptor/18814-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70982">
    <skos:narrower rdf:resource="descriptor/18818-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70982">
    <skos:narrower rdf:resource="descriptor/16271-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12945-3">
    <skos:altLabel xml:lang="de">Agrarwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11558-1">
    <skos:altLabel xml:lang="de">Sozialhaushalt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11558-1">
    <skos:altLabel xml:lang="de">Sozialetat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11558-1">
    <skos:altLabel xml:lang="de">Sozialbudget</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11558-1">
    <skos:altLabel xml:lang="de">Sozialausgaben der Öffentlichen Hand</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id72">
    <rdf:value xml:lang="de">Für Finanzinvestitionen im Ausland</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18209-5">
    <skos:scopeNote xml:lang="de">Für Publikumszeitschriften, die sich an einzelne Käuferschichten wenden, BENUTZE Spezialzeitschrift bzw. die entsprechenden Unterbegriffe.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18209-5">
    <skos:scopeNote xml:lang="de">Zeitschrift, die sich an eine bestimmte Fachdisziplin oder Berufsgruppe wendet.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18838-2">
    <skos:altLabel xml:lang="de">Genossenschaftssozialismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18838-2">
    <skos:altLabel xml:lang="de">Genossenschaftliche Ideengeschichte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13346-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10054-6">
    <skos:narrower rdf:resource="descriptor/10418-1"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id73">
    <rdf:value xml:lang="de">Für die Messung der Wirtschaftskonzentration</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11088-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10035-3">
    <skos:related rdf:resource="descriptor/15054-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10035-3">
    <skos:related rdf:resource="descriptor/10073-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10035-3">
    <skos:related rdf:resource="descriptor/15026-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19686-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18781-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70416">
    <skos:narrower rdf:resource="descriptor/14794-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70416">
    <skos:narrower rdf:resource="descriptor/14793-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70416">
    <skos:narrower rdf:resource="descriptor/14788-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70416">
    <skos:narrower rdf:resource="descriptor/14789-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70416">
    <skos:narrower rdf:resource="descriptor/14787-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70416">
    <skos:narrower rdf:resource="descriptor/14792-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70416">
    <skos:narrower rdf:resource="descriptor/14784-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17935-2">
    <skos:prefLabel xml:lang="en">Australia</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17935-2">
    <skos:prefLabel xml:lang="de">Australien</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18920-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16304-0">
    <skos:broader rdf:resource="thsys/73373"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16304-0">
    <skos:broader rdf:resource="thsys/73334"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16304-0">
    <skos:broader rdf:resource="descriptor/15979-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19386-6">
    <skos:related rdf:resource="descriptor/12983-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19386-6">
    <skos:related rdf:resource="descriptor/14196-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70839">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15787-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18705-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15916-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10143-0">
    <skos:prefLabel xml:lang="en">Job search theory</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10143-0">
    <skos:prefLabel xml:lang="de">Arbeitsplatzsuchmodell</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70144">
    <skos:broader rdf:resource="thsys/v"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14313-2">
    <skos:broader rdf:resource="thsys/70358"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14313-2">
    <skos:broader rdf:resource="descriptor/14236-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14313-2">
    <skos:broader rdf:resource="descriptor/14601-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11213-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10049-6">
    <skos:altLabel xml:lang="de">Zwei-Sektoren-Modell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10049-6">
    <skos:altLabel xml:lang="de">Zwei-Güter-Modell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10049-6">
    <skos:altLabel xml:lang="de">N-Sektoren-Modell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10049-6">
    <skos:altLabel xml:lang="de">Mehr-Güter-Modell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13979-2">
    <skos:altLabel xml:lang="de">Verkehrspersonal</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13979-2">
    <skos:altLabel xml:lang="de">Transportarbeiterinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13979-2">
    <skos:altLabel xml:lang="de">Transportarbeiter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13979-2">
    <skos:altLabel xml:lang="de">Straßenwartinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13979-2">
    <skos:altLabel xml:lang="de">Straßenwarte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13979-2">
    <skos:altLabel xml:lang="de">Lagerarbeiterinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13979-2">
    <skos:altLabel xml:lang="de">Lagerarbeiter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13979-2">
    <skos:altLabel xml:lang="de">Bierkutscherinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13979-2">
    <skos:altLabel xml:lang="de">Bierkutscher</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13979-2">
    <skos:altLabel xml:lang="de">Berufskraftfahrerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13979-2">
    <skos:altLabel xml:lang="de">Berufskraftfahrer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19145-0">
    <skos:related rdf:resource="descriptor/13269-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19145-0">
    <skos:related rdf:resource="descriptor/18659-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19145-0">
    <skos:related rdf:resource="descriptor/19651-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19145-0">
    <skos:related rdf:resource="descriptor/20341-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24703-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70828">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.28</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11685-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11069-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15765-4">
    <skos:narrower rdf:resource="descriptor/18623-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15765-4">
    <skos:narrower rdf:resource="descriptor/18493-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15315-0">
    <skos:broader rdf:resource="thsys/72186"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15315-0">
    <skos:broader rdf:resource="descriptor/15064-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73391">
    <skos:narrower rdf:resource="descriptor/19127-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73391">
    <skos:narrower rdf:resource="descriptor/19824-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73391">
    <skos:narrower rdf:resource="descriptor/10964-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73391">
    <skos:narrower rdf:resource="descriptor/10942-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73391">
    <skos:narrower rdf:resource="descriptor/16080-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73391">
    <skos:narrower rdf:resource="descriptor/19132-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73391">
    <skos:narrower rdf:resource="descriptor/16079-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73391">
    <skos:narrower rdf:resource="descriptor/13825-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73391">
    <skos:narrower rdf:resource="descriptor/18994-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73391">
    <skos:narrower rdf:resource="descriptor/12832-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73391">
    <skos:narrower rdf:resource="descriptor/10910-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73391">
    <skos:narrower rdf:resource="descriptor/16076-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73391">
    <skos:narrower rdf:resource="descriptor/12196-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73391">
    <skos:narrower rdf:resource="descriptor/18726-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73391">
    <skos:narrower rdf:resource="descriptor/16070-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73391">
    <skos:narrower rdf:resource="descriptor/16068-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73391">
    <skos:narrower rdf:resource="descriptor/12195-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73391">
    <skos:narrower rdf:resource="descriptor/12194-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73391">
    <skos:narrower rdf:resource="descriptor/16067-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73391">
    <skos:narrower rdf:resource="descriptor/10481-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73391">
    <skos:narrower rdf:resource="descriptor/16078-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73391">
    <skos:narrower rdf:resource="descriptor/18065-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73391">
    <skos:narrower rdf:resource="descriptor/16063-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73391">
    <skos:narrower rdf:resource="descriptor/19434-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73391">
    <skos:narrower rdf:resource="descriptor/16051-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73391">
    <skos:narrower rdf:resource="descriptor/19131-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73391">
    <skos:narrower rdf:resource="descriptor/18513-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73391">
    <skos:narrower rdf:resource="descriptor/19850-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73391">
    <skos:narrower rdf:resource="descriptor/13464-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73391">
    <skos:narrower rdf:resource="descriptor/12787-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73391">
    <skos:narrower rdf:resource="descriptor/12337-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73391">
    <skos:narrower rdf:resource="descriptor/16053-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73391">
    <skos:narrower rdf:resource="descriptor/16050-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73391">
    <skos:narrower rdf:resource="descriptor/19848-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73391">
    <skos:narrower rdf:resource="descriptor/16031-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73391">
    <skos:narrower rdf:resource="descriptor/18722-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73391">
    <skos:narrower rdf:resource="descriptor/13795-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73391">
    <skos:narrower rdf:resource="descriptor/12326-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73391">
    <skos:narrower rdf:resource="descriptor/12361-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73391">
    <skos:narrower rdf:resource="descriptor/12239-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73391">
    <skos:narrower rdf:resource="descriptor/16043-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19810-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16989-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13436-5">
    <skos:related rdf:resource="descriptor/13440-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19133-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10901-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12621-1">
    <skos:broader rdf:resource="thsys/71050"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12621-1">
    <skos:broader rdf:resource="thsys/70562"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12621-1">
    <skos:broader rdf:resource="descriptor/19562-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18123-1">
    <skos:altLabel xml:lang="de">Politik der Gewerkschaften</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18123-1">
    <skos:altLabel xml:lang="de">Gewerkschaftsprogramm</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18123-1">
    <skos:altLabel xml:lang="de">Gewerkschaftliche Politik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13324-2">
    <skos:prefLabel xml:lang="en">Recycling</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13324-2">
    <skos:prefLabel xml:lang="de">Recycling</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18718-5">
    <skos:prefLabel xml:lang="en">Bank marketing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18718-5">
    <skos:prefLabel xml:lang="de">Bankmarketing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11277-2">
    <skos:prefLabel xml:lang="en">Male workers</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11277-2">
    <skos:prefLabel xml:lang="de">Männliche Arbeitskräfte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20136-5">
    <skos:scopeNote xml:lang="de">Der Grundsatz des Universaldienstes verpflichtet die Anbieter einer Dienstleistung, allen Kund/innen mindestens einen Minimaldienst mit spezifizierter Qualität zu einem erschwinglichen Preis zu gewährleisten.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15074-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18256-3">
    <skos:broader rdf:resource="thsys/71010"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18256-3">
    <skos:broader rdf:resource="descriptor/12922-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19211-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13655-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13533-0">
    <skos:related rdf:resource="descriptor/18603-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13533-0">
    <skos:related rdf:resource="descriptor/18602-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13533-0">
    <skos:related rdf:resource="descriptor/18601-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13533-0">
    <skos:related rdf:resource="descriptor/13536-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10734-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13801-1">
    <skos:altLabel xml:lang="en">Plastic money</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13801-1">
    <skos:altLabel xml:lang="en">Bank card</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13801-1">
    <skos:altLabel xml:lang="de">Plastikgeld</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13801-1">
    <skos:altLabel xml:lang="de">Debitkarte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13801-1">
    <skos:altLabel xml:lang="de">Bankzahlungskarte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19756-6">
    <skos:prefLabel xml:lang="en">Gender discrimination</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19756-6">
    <skos:prefLabel xml:lang="de">Geschlechterdiskriminierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18361-6">
    <skos:altLabel xml:lang="de">Stromverteilungsanlage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18361-6">
    <skos:altLabel xml:lang="de">Höchstspannungsnetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18361-6">
    <skos:altLabel xml:lang="de">Hochspannungsnetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18361-6">
    <skos:altLabel xml:lang="de">Elektrizitätsversorgungsnetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18361-6">
    <skos:altLabel xml:lang="de">Elektrisches Verteilungsnetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19489-3">
    <skos:prefLabel xml:lang="en">Oil price shock</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19489-3">
    <skos:prefLabel xml:lang="de">Mineralölpreisschock</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18426-4">
    <skos:prefLabel xml:lang="en">Electrical engineering</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18426-4">
    <skos:prefLabel xml:lang="de">Elektrotechnik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73350">
    <skos:broader rdf:resource="thsys/70151"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10503-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12625-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15599-2">
    <skos:broader rdf:resource="thsys/73341"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15599-2">
    <skos:broader rdf:resource="thsys/71037"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15599-2">
    <skos:broader rdf:resource="descriptor/18000-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16544-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16516-6">
    <skos:related rdf:resource="descriptor/18653-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16516-6">
    <skos:related rdf:resource="descriptor/13347-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16516-6">
    <skos:related rdf:resource="descriptor/13660-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16516-6">
    <skos:related rdf:resource="descriptor/13658-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16516-6">
    <skos:related rdf:resource="descriptor/16514-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19490-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15836-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10641-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12218-6">
    <skos:prefLabel xml:lang="en">Cash flow</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12218-6">
    <skos:prefLabel xml:lang="de">Cash Flow</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12535-1">
    <skos:prefLabel xml:lang="en">Cost allocation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12535-1">
    <skos:prefLabel xml:lang="de">Kostenverrechnung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14955-4">
    <skos:related rdf:resource="descriptor/14957-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14955-4">
    <skos:related rdf:resource="descriptor/18306-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14955-4">
    <skos:related rdf:resource="descriptor/13158-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14955-4">
    <skos:related rdf:resource="descriptor/14124-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16754-4">
    <skos:narrower rdf:resource="descriptor/18059-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16754-4">
    <skos:narrower rdf:resource="descriptor/18942-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16754-4">
    <skos:narrower rdf:resource="descriptor/20314-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19235-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14175-2">
    <skos:broader rdf:resource="thsys/70538"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14175-2">
    <skos:broader rdf:resource="thsys/70227"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17721-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15410-6">
    <skos:related rdf:resource="descriptor/19810-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15410-6">
    <skos:related rdf:resource="descriptor/13277-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15410-6">
    <skos:related rdf:resource="descriptor/14762-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15410-6">
    <skos:related rdf:resource="descriptor/19150-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15410-6">
    <skos:related rdf:resource="descriptor/15611-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15410-6">
    <skos:related rdf:resource="descriptor/15398-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15410-6">
    <skos:related rdf:resource="descriptor/15397-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19619-2">
    <skos:related rdf:resource="descriptor/15889-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19619-2">
    <skos:related rdf:resource="descriptor/14180-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19619-2">
    <skos:related rdf:resource="descriptor/15869-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19619-2">
    <skos:related rdf:resource="descriptor/19615-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70057">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.01.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16102-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70265">
    <skos:prefLabel xml:lang="de">P.15  Druckereiprodukte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10746-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10525-0">
    <skos:altLabel xml:lang="en">Two-gap model</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10525-0">
    <skos:altLabel xml:lang="en">Three-gap model</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10525-0">
    <skos:altLabel xml:lang="en">Theory of economic development</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10525-0">
    <skos:altLabel xml:lang="en">Multi-gap approach</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10525-0">
    <skos:altLabel xml:lang="en">Development economics</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10525-0">
    <skos:altLabel xml:lang="de">Entwicklungsökonomik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10525-0">
    <skos:altLabel xml:lang="de">Entwicklungsökonomie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15694-1">
    <skos:broader rdf:resource="thsys/70581"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15694-1">
    <skos:broader rdf:resource="thsys/70007"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15694-1">
    <skos:broader rdf:resource="descriptor/15690-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12020-6">
    <skos:altLabel xml:lang="en">Land rights</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12020-6">
    <skos:altLabel xml:lang="de">Bodenordnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12020-6">
    <skos:altLabel xml:lang="de">Bodennutzungsrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10320-4">
    <skos:related rdf:resource="descriptor/18117-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11931-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14453-0">
    <skos:prefLabel xml:lang="en">Air cushion vehicle</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14453-0">
    <skos:prefLabel xml:lang="de">Luftkissenfahrzeug</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73380">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.04.02.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16912-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10239-1">
    <skos:related rdf:resource="descriptor/10460-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10239-1">
    <skos:related rdf:resource="descriptor/12708-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10239-1">
    <skos:related rdf:resource="descriptor/10121-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10239-1">
    <skos:related rdf:resource="descriptor/12700-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10239-1">
    <skos:related rdf:resource="descriptor/11716-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10239-1">
    <skos:related rdf:resource="descriptor/19195-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73386">
    <skos:broader rdf:resource="thsys/70017"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14953-1">
    <skos:broader rdf:resource="thsys/73303"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14953-1">
    <skos:broader rdf:resource="descriptor/14955-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16599-4">
    <skos:broader rdf:resource="thsys/70581"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15932-4">
    <skos:altLabel xml:lang="en">Racial discrimination</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15932-4">
    <skos:altLabel xml:lang="de">Xenophobie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15932-4">
    <skos:altLabel xml:lang="de">Rassismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15932-4">
    <skos:altLabel xml:lang="de">Rassenvorurteil</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15932-4">
    <skos:altLabel xml:lang="de">Rassenhass</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15932-4">
    <skos:altLabel xml:lang="de">Rassendiskriminierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15932-4">
    <skos:altLabel xml:lang="de">Fremdenhass</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15932-4">
    <skos:altLabel xml:lang="de">Fremdenhaß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15932-4">
    <skos:altLabel xml:lang="de">Fremdenfeindlichkeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15932-4">
    <skos:altLabel xml:lang="de">Ausländerfeindlichkeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10377-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14755-5">
    <skos:related rdf:resource="descriptor/13387-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19653-2">
    <skos:related rdf:resource="descriptor/18654-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19653-2">
    <skos:related rdf:resource="descriptor/13666-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19653-2">
    <skos:related rdf:resource="descriptor/18648-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19653-2">
    <skos:related rdf:resource="descriptor/19120-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19653-2">
    <skos:related rdf:resource="descriptor/19541-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19653-2">
    <skos:related rdf:resource="descriptor/13674-3"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id58">
    <rdfs:seeAlso rdf:resource="descriptor/11782-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18387-2">
    <skos:broader rdf:resource="thsys/71009"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18387-2">
    <skos:broader rdf:resource="thsys/70811"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18387-2">
    <skos:broader rdf:resource="descriptor/12980-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18018-5">
    <skos:broader rdf:resource="thsys/70953"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18018-5">
    <skos:broader rdf:resource="thsys/70854"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18018-5">
    <skos:broader rdf:resource="descriptor/13456-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12721-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13250-5">
    <skos:altLabel xml:lang="de">ZOB (Zentraler Omnibusbahnhof)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13250-5">
    <skos:altLabel xml:lang="de">Zentraler Omnibusbahnhof</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13250-5">
    <skos:altLabel xml:lang="de">Omnibusbahnhof</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13250-5">
    <skos:altLabel xml:lang="de">Busverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15005-1">
    <skos:broader rdf:resource="thsys/70337"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14130-3">
    <skos:broader rdf:resource="thsys/73304"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14130-3">
    <skos:broader rdf:resource="thsys/70467"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14130-3">
    <skos:broader rdf:resource="descriptor/14129-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14130-3">
    <skos:broader rdf:resource="descriptor/14851-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19270-4">
    <skos:related rdf:resource="descriptor/19321-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19270-4">
    <skos:related rdf:resource="descriptor/18676-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19270-4">
    <skos:related rdf:resource="descriptor/13279-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13691-3">
    <skos:related rdf:resource="descriptor/11416-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13691-3">
    <skos:related rdf:resource="descriptor/19261-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13691-3">
    <skos:related rdf:resource="descriptor/13283-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15956-4">
    <skos:broader rdf:resource="thsys/70572"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15956-4">
    <skos:broader rdf:resource="descriptor/15954-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17941-0">
    <skos:prefLabel xml:lang="en">Nauru</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17941-0">
    <skos:prefLabel xml:lang="de">Nauru</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14722-6">
    <skos:related rdf:resource="descriptor/15635-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14722-6">
    <skos:related rdf:resource="descriptor/18260-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14722-6">
    <skos:related rdf:resource="descriptor/18777-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11905-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14354-2">
    <skos:prefLabel xml:lang="en">Automobile engine</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14354-2">
    <skos:prefLabel xml:lang="de">Kraftfahrzeugmotor</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12172-4">
    <skos:related rdf:resource="descriptor/18017-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12172-4">
    <skos:related rdf:resource="descriptor/12165-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19244-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11047-5">
    <skos:broader rdf:resource="thsys/73396"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11047-5">
    <skos:broader rdf:resource="thsys/73382"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11047-5">
    <skos:broader rdf:resource="thsys/71101"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11047-5">
    <skos:broader rdf:resource="descriptor/16405-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11047-5">
    <skos:broader rdf:resource="descriptor/16602-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18654-5">
    <skos:altLabel xml:lang="de">Telewerbung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18654-5">
    <skos:altLabel xml:lang="de">Telephonmarketing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18654-5">
    <skos:altLabel xml:lang="de">Telefonwerbung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18654-5">
    <skos:altLabel xml:lang="de">Telefonmarketing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70856">
    <skos:prefLabel xml:lang="de">W.23  Nonprofit-Sektor</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13149-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14567-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13329-6">
    <skos:prefLabel xml:lang="en">Waste water management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13329-6">
    <skos:prefLabel xml:lang="de">Abwasserwirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10807-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71119">
    <skos:narrower rdf:resource="descriptor/10227-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71119">
    <skos:narrower rdf:resource="descriptor/10224-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71119">
    <skos:narrower rdf:resource="descriptor/10223-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71119">
    <skos:narrower rdf:resource="descriptor/10209-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71119">
    <skos:narrower rdf:resource="descriptor/10208-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71119">
    <skos:narrower rdf:resource="descriptor/10206-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71119">
    <skos:narrower rdf:resource="descriptor/19226-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71119">
    <skos:narrower rdf:resource="descriptor/10205-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71119">
    <skos:narrower rdf:resource="descriptor/10905-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71119">
    <skos:narrower rdf:resource="descriptor/10201-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71119">
    <skos:narrower rdf:resource="descriptor/19827-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71119">
    <skos:narrower rdf:resource="descriptor/10197-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71119">
    <skos:narrower rdf:resource="descriptor/10196-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71119">
    <skos:narrower rdf:resource="descriptor/10195-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71119">
    <skos:narrower rdf:resource="descriptor/10138-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10184-0">
    <skos:scopeNote xml:lang="de">Ökonomische Analyse der Zeitallokation zwischen verschiedenen Aktivitäten; Opportunitätskosten der Zeit.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71090">
    <skos:broader rdf:resource="thsys/70179"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12249-2">
    <skos:narrower rdf:resource="descriptor/10135-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14336-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18598-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18940-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18022-0">
    <skos:related rdf:resource="descriptor/18092-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18022-0">
    <skos:related rdf:resource="descriptor/14762-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18022-0">
    <skos:related rdf:resource="descriptor/19136-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20972-4">
    <skos:prefLabel xml:lang="en">Keiretsu</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20972-4">
    <skos:prefLabel xml:lang="de">Keiretsu</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10463-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12042-3">
    <skos:broader rdf:resource="thsys/70562"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14017-6">
    <skos:prefLabel xml:lang="en">Economists</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14017-6">
    <skos:prefLabel xml:lang="de">Wirtschaftswissenschaftler</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70041">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18251-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10636-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70964">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71080">
    <skos:broader rdf:resource="thsys/70144"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13347-4">
    <skos:scopeNote xml:lang="de">Keine allgemein gültige Definition. Teilweise unter Einbeziehung der Filmwirtschaft und/oder der Anbieter von Telekommunikationsdiensten.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14632-0">
    <skos:broader rdf:resource="thsys/70238"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14632-0">
    <skos:broader rdf:resource="thsys/70014"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14632-0">
    <skos:broader rdf:resource="descriptor/15763-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14632-0">
    <skos:broader rdf:resource="descriptor/18613-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14632-0">
    <skos:broader rdf:resource="descriptor/14599-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13082-0">
    <skos:broader rdf:resource="thsys/70995"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13082-0">
    <skos:broader rdf:resource="descriptor/13081-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16504-6">
    <skos:scopeNote xml:lang="de">Schutz vor oder Befreiung von Strafe. Auch Begnadigung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18732-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10699-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19329-4">
    <skos:narrower rdf:resource="descriptor/10880-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19329-4">
    <skos:narrower rdf:resource="descriptor/12313-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12765-5">
    <skos:prefLabel xml:lang="en">Time scheduling</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12765-5">
    <skos:prefLabel xml:lang="de">Betriebliche Terminplanung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70184">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18611-2">
    <skos:broader rdf:resource="thsys/70939"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18611-2">
    <skos:broader rdf:resource="thsys/70932"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18611-2">
    <skos:broader rdf:resource="descriptor/13632-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18611-2">
    <skos:broader rdf:resource="descriptor/13246-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18835-1">
    <skos:related rdf:resource="descriptor/18101-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14428-6">
    <skos:related rdf:resource="descriptor/13255-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14428-6">
    <skos:related rdf:resource="descriptor/13063-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14428-6">
    <skos:related rdf:resource="descriptor/13574-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13793-2">
    <skos:altLabel xml:lang="de">Unbarer Zahlungsverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13793-2">
    <skos:altLabel xml:lang="de">Überweisungsverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13793-2">
    <skos:altLabel xml:lang="de">Giroverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19426-6">
    <skos:prefLabel xml:lang="en">Petroleum processing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19426-6">
    <skos:prefLabel xml:lang="de">Mineralölverarbeitung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13217-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15350-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11974-1">
    <skos:related rdf:resource="descriptor/11881-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11974-1">
    <skos:related rdf:resource="descriptor/13563-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11974-1">
    <skos:related rdf:resource="descriptor/18633-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11974-1">
    <skos:related rdf:resource="descriptor/18576-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11974-1">
    <skos:related rdf:resource="descriptor/11893-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13148-3">
    <skos:related rdf:resource="descriptor/14888-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18071-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15669-0">
    <skos:prefLabel xml:lang="en">Physics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15669-0">
    <skos:prefLabel xml:lang="de">Physik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14119-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18772-6">
    <skos:scopeNote xml:lang="de">Politik der Europäischen Union und ihrer Organe.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71111">
    <skos:narrower rdf:resource="descriptor/10112-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71111">
    <skos:narrower rdf:resource="descriptor/10028-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71111">
    <skos:narrower rdf:resource="descriptor/10272-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71111">
    <skos:narrower rdf:resource="descriptor/10271-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71111">
    <skos:narrower rdf:resource="descriptor/10269-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71111">
    <skos:narrower rdf:resource="descriptor/10274-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71111">
    <skos:narrower rdf:resource="descriptor/10268-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71111">
    <skos:narrower rdf:resource="descriptor/10266-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71111">
    <skos:narrower rdf:resource="descriptor/10265-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71111">
    <skos:narrower rdf:resource="descriptor/10058-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71111">
    <skos:narrower rdf:resource="descriptor/19237-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71111">
    <skos:narrower rdf:resource="descriptor/10019-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71111">
    <skos:narrower rdf:resource="descriptor/11959-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71111">
    <skos:narrower rdf:resource="descriptor/18183-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71111">
    <skos:narrower rdf:resource="descriptor/10248-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71111">
    <skos:narrower rdf:resource="descriptor/10243-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71111">
    <skos:narrower rdf:resource="descriptor/10276-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71111">
    <skos:narrower rdf:resource="descriptor/19231-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15939-4">
    <skos:prefLabel xml:lang="en">Family planning</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15939-4">
    <skos:prefLabel xml:lang="de">Familienplanung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13370-2">
    <skos:related rdf:resource="descriptor/13375-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13370-2">
    <skos:related rdf:resource="descriptor/13371-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13370-2">
    <skos:related rdf:resource="descriptor/13871-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13370-2">
    <skos:related rdf:resource="descriptor/19676-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18721-2">
    <skos:prefLabel xml:lang="en">Asset-liability management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18721-2">
    <skos:prefLabel xml:lang="de">Bilanzstrukturmanagement</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13331-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16429-1">
    <skos:altLabel xml:lang="en">Gender justice</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16429-1">
    <skos:altLabel xml:lang="en">Equal opportunities</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16429-1">
    <skos:altLabel xml:lang="de">Gleichstellung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16429-1">
    <skos:altLabel xml:lang="de">Emanzipation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10838-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17779-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13611-6">
    <skos:narrower rdf:resource="descriptor/13613-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13611-6">
    <skos:narrower rdf:resource="descriptor/16169-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18354-3">
    <skos:related rdf:resource="descriptor/18361-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18354-3">
    <skos:related rdf:resource="descriptor/12974-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12759-0">
    <skos:altLabel xml:lang="de">Teilefamilienproduktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12759-0">
    <skos:altLabel xml:lang="de">Produktprogrammplanung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12759-0">
    <skos:altLabel xml:lang="de">Produktprogramm-Management</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12759-0">
    <skos:altLabel xml:lang="de">Produktprogramm</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12759-0">
    <skos:altLabel xml:lang="de">Produktlinie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12759-0">
    <skos:altLabel xml:lang="de">Produktionsprogrammplanung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12759-0">
    <skos:altLabel xml:lang="de">Produktgruppe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12759-0">
    <skos:altLabel xml:lang="de">Produktfeld</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12759-0">
    <skos:altLabel xml:lang="de">Fertigungstiefe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12759-0">
    <skos:altLabel xml:lang="de">Fertigungsprogrammplanung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12759-0">
    <skos:altLabel xml:lang="de">Fertigungsprogramm</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12759-0">
    <skos:altLabel xml:lang="de">Fertigungsbreite</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18370-5">
    <skos:broader rdf:resource="thsys/73362"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18370-5">
    <skos:broader rdf:resource="thsys/71012"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18370-5">
    <skos:broader rdf:resource="thsys/70539"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18370-5">
    <skos:broader rdf:resource="descriptor/18772-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18370-5">
    <skos:broader rdf:resource="descriptor/11810-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11327-6">
    <skos:narrower rdf:resource="descriptor/18309-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11327-6">
    <skos:narrower rdf:resource="descriptor/19440-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11327-6">
    <skos:narrower rdf:resource="descriptor/19011-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12085-6">
    <skos:narrower rdf:resource="descriptor/19655-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12085-6">
    <skos:narrower rdf:resource="descriptor/12201-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12085-6">
    <skos:narrower rdf:resource="descriptor/18029-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12085-6">
    <skos:narrower rdf:resource="descriptor/18118-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12085-6">
    <skos:narrower rdf:resource="descriptor/12713-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12085-6">
    <skos:narrower rdf:resource="descriptor/19153-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12085-6">
    <skos:narrower rdf:resource="descriptor/12585-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12085-6">
    <skos:narrower rdf:resource="descriptor/18834-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12085-6">
    <skos:narrower rdf:resource="descriptor/12079-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12085-6">
    <skos:narrower rdf:resource="descriptor/19152-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12085-6">
    <skos:narrower rdf:resource="descriptor/12078-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12085-6">
    <skos:narrower rdf:resource="descriptor/19096-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12085-6">
    <skos:narrower rdf:resource="descriptor/20497-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12085-6">
    <skos:narrower rdf:resource="descriptor/21306-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12085-6">
    <skos:narrower rdf:resource="descriptor/18717-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18583-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16333-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13200-6">
    <skos:narrower rdf:resource="descriptor/13493-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13200-6">
    <skos:narrower rdf:resource="descriptor/13492-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13200-6">
    <skos:narrower rdf:resource="descriptor/18436-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13200-6">
    <skos:narrower rdf:resource="descriptor/13239-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13200-6">
    <skos:narrower rdf:resource="descriptor/13232-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13200-6">
    <skos:narrower rdf:resource="descriptor/18550-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13200-6">
    <skos:narrower rdf:resource="descriptor/13218-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13200-6">
    <skos:narrower rdf:resource="descriptor/18548-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13200-6">
    <skos:narrower rdf:resource="descriptor/13214-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13200-6">
    <skos:narrower rdf:resource="descriptor/13229-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13200-6">
    <skos:narrower rdf:resource="descriptor/13483-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13200-6">
    <skos:narrower rdf:resource="descriptor/18549-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13200-6">
    <skos:narrower rdf:resource="descriptor/13199-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13200-6">
    <skos:narrower rdf:resource="descriptor/13480-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13200-6">
    <skos:narrower rdf:resource="descriptor/13192-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13200-6">
    <skos:narrower rdf:resource="descriptor/13478-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#memberList">
    <rdfs:label xml:lang="en">member list</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13430-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18511-6">
    <skos:prefLabel xml:lang="en">Craft cooperation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18511-6">
    <skos:prefLabel xml:lang="de">Handwerkskooperation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14932-2">
    <skos:broader rdf:resource="thsys/70498"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14932-2">
    <skos:broader rdf:resource="descriptor/14926-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19590-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14743-5">
    <skos:prefLabel xml:lang="en">Paints and coatings</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14743-5">
    <skos:prefLabel xml:lang="de">Farbe und Lack</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12977-4">
    <skos:related rdf:resource="descriptor/14404-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12977-4">
    <skos:related rdf:resource="descriptor/14177-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12977-4">
    <skos:related rdf:resource="descriptor/18346-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14645-5">
    <skos:prefLabel xml:lang="en">Nitrate</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14645-5">
    <skos:prefLabel xml:lang="de">Nitrat</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15272-6">
    <skos:narrower rdf:resource="descriptor/15266-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15272-6">
    <skos:narrower rdf:resource="descriptor/15265-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15272-6">
    <skos:narrower rdf:resource="descriptor/15264-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15272-6">
    <skos:narrower rdf:resource="descriptor/15365-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15272-6">
    <skos:narrower rdf:resource="descriptor/15251-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14186-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71046">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10556-3">
    <skos:altLabel xml:lang="en">Tied aid</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10556-3">
    <skos:altLabel xml:lang="en">Aid conditionality</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10556-3">
    <skos:altLabel xml:lang="de">Liefergebundene Entwicklungshilfe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10556-3">
    <skos:altLabel xml:lang="de">Lieferbindung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10556-3">
    <skos:altLabel xml:lang="de">Gebundene Entwicklungshilfe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16484-0">
    <skos:prefLabel xml:lang="en">System of collective agreements</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16484-0">
    <skos:prefLabel xml:lang="de">Tarifpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19829-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12832-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19606-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19658-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11555-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16615-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17171-2">
    <skos:prefLabel xml:lang="en">Albania</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17171-2">
    <skos:prefLabel xml:lang="de">Albanien</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19629-6">
    <skos:scopeNote xml:lang="de">Maßnahmen zur Erhaltung gefährdeter und seltener Pflanzenarten.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15476-4">
    <skos:narrower rdf:resource="descriptor/19598-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15258-0">
    <skos:broader rdf:resource="thsys/72186"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15258-0">
    <skos:broader rdf:resource="descriptor/15064-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13410-2">
    <skos:prefLabel xml:lang="en">Cognizance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13410-2">
    <skos:prefLabel xml:lang="de">Gerichtsbarkeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19163-5">
    <skos:prefLabel xml:lang="en">Strategy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19163-5">
    <skos:prefLabel xml:lang="de">Strategie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11345-4">
    <zbwext:useInsteadNote rdf:nodeID="id200"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10511-4">
    <skos:prefLabel xml:lang="en">Subsistence farming</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10511-4">
    <skos:prefLabel xml:lang="de">Subsistenzwirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11628-6">
    <skos:broader rdf:resource="thsys/71072"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11628-6">
    <skos:broader rdf:resource="thsys/70860"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11628-6">
    <skos:broader rdf:resource="thsys/70855"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11628-6">
    <skos:broader rdf:resource="thsys/70087"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11628-6">
    <skos:broader rdf:resource="descriptor/12072-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11628-6">
    <skos:broader rdf:resource="descriptor/12194-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11628-6">
    <skos:broader rdf:resource="descriptor/11722-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11628-6">
    <skos:broader rdf:resource="descriptor/11825-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12524-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11713-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14705-6">
    <skos:altLabel xml:lang="de">Polyäthylen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14705-6">
    <skos:altLabel xml:lang="de">PE (Polyethylen)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19539-0">
    <skos:related rdf:resource="descriptor/14728-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10988-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16051-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13572-4">
    <skos:prefLabel xml:lang="en">Traffic accident</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13572-4">
    <skos:prefLabel xml:lang="de">Verkehrsunfall</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#Collection">
    <owl:disjointWith rdf:resource="http://www.w3.org/2004/02/skos/core#ConceptScheme"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#Collection">
    <owl:disjointWith rdf:resource="http://www.w3.org/2004/02/skos/core#Concept"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19018-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19141-1">
    <zbwext:useInsteadNote rdf:nodeID="id183"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16979-1">
    <skos:broader rdf:resource="thsys/70003"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16979-1">
    <skos:broader rdf:resource="descriptor/16972-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10745-0">
    <skos:broader rdf:resource="thsys/71097"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10745-0">
    <skos:broader rdf:resource="thsys/71094"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10745-0">
    <skos:broader rdf:resource="descriptor/10619-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10745-0">
    <skos:broader rdf:resource="descriptor/10721-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18937-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18667-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18775-0">
    <skos:prefLabel xml:lang="en">Research collaboration</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18775-0">
    <skos:prefLabel xml:lang="de">Forschungskooperation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18398-4">
    <skos:prefLabel xml:lang="en">Packaging technology</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18398-4">
    <skos:prefLabel xml:lang="de">Verpackungstechnik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10734-5">
    <skos:altLabel xml:lang="en">Exchange Rate Mechanism</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10734-5">
    <skos:altLabel xml:lang="en">ERM (Exchange Rate Mechanism)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10734-5">
    <skos:altLabel xml:lang="en">EMS (European Monetary System)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10734-5">
    <skos:altLabel xml:lang="de">EWS (Europäisches Währungssystem)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10734-5">
    <skos:altLabel xml:lang="de">Europäisches Währungssystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10734-5">
    <skos:altLabel xml:lang="de">Europäische Währungsschlange</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10734-5">
    <skos:altLabel xml:lang="de">Europäische Währungsintegration</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19603-3">
    <skos:scopeNote xml:lang="de">Statistiken über Hochschulen, ihre Einrichtungen, die Hochschullehrer/innen und die Studierenden.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19175-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12129-5">
    <skos:broader rdf:resource="thsys/70562"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15930-1">
    <skos:prefLabel xml:lang="en">Nomads</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15930-1">
    <skos:prefLabel xml:lang="de">Nomaden</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12250-3">
    <skos:prefLabel xml:lang="en">Advance on current account</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12250-3">
    <skos:prefLabel xml:lang="de">Kontokorrentkredit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10205-4">
    <zbwext:useInsteadNote rdf:nodeID="id221"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16411-6">
    <zbwext:useInsteadNote rdf:nodeID="id194"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18115-0">
    <skos:altLabel xml:lang="de">Unternehmernachfolge</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18115-0">
    <skos:altLabel xml:lang="de">Unternehmenserbe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18115-0">
    <skos:altLabel xml:lang="de">Nachfolgeregelung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18115-0">
    <skos:altLabel xml:lang="de">Nachfolgeplanung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14212-1">
    <skos:prefLabel xml:lang="en">Petrol</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14212-1">
    <skos:prefLabel xml:lang="de">Benzin</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12732-6">
    <skos:related rdf:resource="descriptor/12738-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12732-6">
    <skos:related rdf:resource="descriptor/18002-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10910-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13164-5">
    <skos:narrower rdf:resource="descriptor/18167-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12940-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18677-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10894-4">
    <skos:related rdf:resource="descriptor/10942-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10894-4">
    <skos:related rdf:resource="descriptor/18828-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10894-4">
    <skos:related rdf:resource="descriptor/19686-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10894-4">
    <skos:related rdf:resource="descriptor/19560-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10894-4">
    <skos:related rdf:resource="descriptor/10208-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10894-4">
    <skos:related rdf:resource="descriptor/11054-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10894-4">
    <skos:related rdf:resource="descriptor/10928-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10894-4">
    <skos:related rdf:resource="descriptor/10970-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10894-4">
    <skos:related rdf:resource="descriptor/10891-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10894-4">
    <skos:related rdf:resource="descriptor/18011-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11687-4">
    <skos:scopeNote xml:lang="de">Verknüpfung von Einkommensteuer und Sozialtransfers.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10121-3">
    <skos:scopeNote xml:lang="de">Gesamtwirtschaftlicher Mangel an Kapital, so dass das Erwerbspersonenpotenzial nicht ausgelastet werden kann.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70040">
    <skos:narrower rdf:resource="descriptor/19127-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70040">
    <skos:narrower rdf:resource="descriptor/20206-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70040">
    <skos:narrower rdf:resource="descriptor/19126-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70040">
    <skos:narrower rdf:resource="descriptor/12788-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70040">
    <skos:narrower rdf:resource="descriptor/19555-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70040">
    <skos:narrower rdf:resource="descriptor/10931-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70040">
    <skos:narrower rdf:resource="descriptor/10213-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70040">
    <skos:narrower rdf:resource="descriptor/12793-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70040">
    <skos:narrower rdf:resource="descriptor/12255-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70040">
    <skos:narrower rdf:resource="descriptor/24706-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70040">
    <skos:narrower rdf:resource="descriptor/12785-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70040">
    <skos:narrower rdf:resource="descriptor/19124-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70040">
    <skos:narrower rdf:resource="descriptor/12794-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70040">
    <skos:narrower rdf:resource="descriptor/12033-4"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id329">
    <rdf:value xml:lang="de">für Computernetze</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11641-0">
    <skos:prefLabel xml:lang="en">Tax statistics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11641-0">
    <skos:prefLabel xml:lang="de">Steuerstatistik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12058-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11443-4">
    <skos:altLabel xml:lang="de">Banking-Schule</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12609-5">
    <skos:broader rdf:resource="thsys/73336"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12609-5">
    <skos:broader rdf:resource="thsys/71050"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12609-5">
    <skos:broader rdf:resource="thsys/70838"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12609-5">
    <skos:broader rdf:resource="thsys/70833"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12609-5">
    <skos:broader rdf:resource="thsys/70324"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12609-5">
    <skos:broader rdf:resource="thsys/70235"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12609-5">
    <skos:broader rdf:resource="descriptor/12612-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12609-5">
    <skos:broader rdf:resource="descriptor/16124-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18316-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11262-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15811-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73355">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12038-1">
    <skos:prefLabel xml:lang="en">Firm size</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12038-1">
    <skos:prefLabel xml:lang="de">Betriebsgröße</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11506-6">
    <skos:prefLabel xml:lang="en">Inflation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11506-6">
    <skos:prefLabel xml:lang="de">Inflation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70875">
    <skos:broader rdf:resource="thsys/70876"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13846-0">
    <skos:altLabel xml:lang="en">Housing rehabilitation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13846-0">
    <skos:altLabel xml:lang="en">Housing modernisation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13846-0">
    <skos:altLabel xml:lang="de">Wohnungsmodernisierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13846-0">
    <skos:altLabel xml:lang="de">Wohnraummodernisierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13846-0">
    <skos:altLabel xml:lang="de">Modernisierung von Altbauten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15723-6">
    <skos:narrower rdf:resource="descriptor/16210-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15706-6">
    <skos:prefLabel xml:lang="en">Economic history</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15706-6">
    <skos:prefLabel xml:lang="de">Wirtschaftsgeschichte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11999-6">
    <skos:broader rdf:resource="thsys/71064"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11999-6">
    <skos:broader rdf:resource="descriptor/11844-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11999-6">
    <skos:broader rdf:resource="descriptor/19447-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13635-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19014-1">
    <skos:broader rdf:resource="thsys/70169"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19014-1">
    <skos:broader rdf:resource="descriptor/14107-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18976-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12081-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70087">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11888-1">
    <skos:altLabel xml:lang="de">Regionale Wanderung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11888-1">
    <skos:altLabel xml:lang="de">Nahwanderung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11888-1">
    <skos:altLabel xml:lang="de">Kleinräumige Mobilität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11888-1">
    <skos:altLabel xml:lang="de">Kleinräumige Bevölkerungsbewegung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11888-1">
    <skos:altLabel xml:lang="de">Interne Migration</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11888-1">
    <skos:altLabel xml:lang="de">Innerstaatliche Wanderung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11888-1">
    <skos:altLabel xml:lang="de">Innerregionale Wanderung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16053-6">
    <skos:prefLabel xml:lang="en">Industrial property rights</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16053-6">
    <skos:prefLabel xml:lang="de">Gewerblicher Rechtsschutz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18818-1">
    <skos:prefLabel xml:lang="en">Arms export control</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18818-1">
    <skos:prefLabel xml:lang="de">Rüstungsexportkontrolle</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18261-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id354">
    <rdf:value xml:lang="de">Für Entwicklungspläne in Entwicklungsländern</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18860-2">
    <skos:altLabel xml:lang="en">School participation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18860-2">
    <skos:altLabel xml:lang="de">Bildungsbeteiligung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18860-2">
    <skos:altLabel xml:lang="de">Ausbildungsgewohnheiten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15685-2">
    <skos:narrower rdf:resource="descriptor/13860-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15685-2">
    <skos:narrower rdf:resource="descriptor/16727-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15685-2">
    <skos:narrower rdf:resource="descriptor/13821-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15685-2">
    <skos:narrower rdf:resource="descriptor/13573-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15685-2">
    <skos:narrower rdf:resource="descriptor/15429-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15685-2">
    <skos:narrower rdf:resource="descriptor/16724-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15685-2">
    <skos:narrower rdf:resource="descriptor/19581-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15685-2">
    <skos:narrower rdf:resource="descriptor/16563-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15685-2">
    <skos:narrower rdf:resource="descriptor/15790-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15685-2">
    <skos:narrower rdf:resource="descriptor/18301-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15685-2">
    <skos:narrower rdf:resource="descriptor/15664-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15685-2">
    <skos:narrower rdf:resource="descriptor/19725-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15685-2">
    <skos:narrower rdf:resource="descriptor/15658-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15685-2">
    <skos:narrower rdf:resource="descriptor/15415-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15685-2">
    <skos:narrower rdf:resource="descriptor/19795-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15685-2">
    <skos:narrower rdf:resource="descriptor/16760-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15685-2">
    <skos:narrower rdf:resource="descriptor/19635-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15685-2">
    <skos:narrower rdf:resource="descriptor/15644-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15685-2">
    <skos:narrower rdf:resource="descriptor/18339-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15685-2">
    <skos:narrower rdf:resource="descriptor/15470-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15685-2">
    <skos:narrower rdf:resource="descriptor/12602-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15685-2">
    <skos:narrower rdf:resource="descriptor/13446-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#hasTopConcept">
    <rdfs:isDefinedBy rdf:resource="http://www.w3.org/2004/02/skos/core"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19824-1">
    <skos:altLabel xml:lang="de">Konkurrenzverbot</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19824-1">
    <skos:altLabel xml:lang="de">Konkurrenzklausel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12628-1">
    <skos:broader rdf:resource="thsys/71059"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12628-1">
    <skos:broader rdf:resource="thsys/71050"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12628-1">
    <skos:broader rdf:resource="descriptor/11248-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12628-1">
    <skos:broader rdf:resource="descriptor/11187-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19276-6">
    <skos:related rdf:resource="descriptor/13784-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19276-6">
    <skos:related rdf:resource="descriptor/19275-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15741-4">
    <skos:related rdf:resource="descriptor/14171-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10722-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10730-6">
    <skos:altLabel xml:lang="de">Deutschmark</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10730-6">
    <skos:altLabel xml:lang="de">Deutsche Mark</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15754-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18837-4">
    <skos:prefLabel xml:lang="en">Urban marketing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18837-4">
    <skos:prefLabel xml:lang="de">Stadtmarketing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16971-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15616-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10466-4">
    <skos:prefLabel xml:lang="en">Technological change</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10466-4">
    <skos:prefLabel xml:lang="de">Technischer Fortschritt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10517-6">
    <skos:altLabel xml:lang="de">Strategie des gleichgewichtigen Wachstums</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10517-6">
    <skos:altLabel xml:lang="de">Balanced-Growth-Strategie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16024-6">
    <skos:broader rdf:resource="thsys/73363"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16024-6">
    <skos:broader rdf:resource="thsys/73360"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16024-6">
    <skos:broader rdf:resource="thsys/70865"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16024-6">
    <skos:broader rdf:resource="descriptor/16019-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71008">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18298-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19230-2">
    <skos:broader rdf:resource="thsys/72393"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19230-2">
    <skos:broader rdf:resource="thsys/70234"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19230-2">
    <skos:broader rdf:resource="descriptor/10253-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18061-4">
    <skos:prefLabel xml:lang="en">Debt relief</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18061-4">
    <skos:prefLabel xml:lang="de">Schuldenerlass</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13244-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18795-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10492-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70485">
    <skos:narrower rdf:resource="descriptor/21282-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70485">
    <skos:narrower rdf:resource="descriptor/19565-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70485">
    <skos:narrower rdf:resource="descriptor/19828-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70485">
    <skos:narrower rdf:resource="descriptor/13808-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70485">
    <skos:narrower rdf:resource="descriptor/12090-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70485">
    <skos:narrower rdf:resource="descriptor/12089-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70485">
    <skos:narrower rdf:resource="descriptor/12088-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70485">
    <skos:narrower rdf:resource="descriptor/18115-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70485">
    <skos:narrower rdf:resource="descriptor/12108-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70485">
    <skos:narrower rdf:resource="descriptor/18116-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70485">
    <skos:narrower rdf:resource="descriptor/15025-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70485">
    <skos:narrower rdf:resource="descriptor/18029-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70485">
    <skos:narrower rdf:resource="descriptor/12158-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70485">
    <skos:narrower rdf:resource="descriptor/18184-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70485">
    <skos:narrower rdf:resource="descriptor/12083-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70485">
    <skos:narrower rdf:resource="descriptor/18726-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70485">
    <skos:narrower rdf:resource="descriptor/18118-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70485">
    <skos:narrower rdf:resource="descriptor/12713-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70485">
    <skos:narrower rdf:resource="descriptor/19153-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70485">
    <skos:narrower rdf:resource="descriptor/12212-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70485">
    <skos:narrower rdf:resource="descriptor/12087-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70485">
    <skos:narrower rdf:resource="descriptor/24856-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70485">
    <skos:narrower rdf:resource="descriptor/12081-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70485">
    <skos:narrower rdf:resource="descriptor/16265-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70485">
    <skos:narrower rdf:resource="descriptor/12079-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70485">
    <skos:narrower rdf:resource="descriptor/12085-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70485">
    <skos:narrower rdf:resource="descriptor/19152-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70485">
    <skos:narrower rdf:resource="descriptor/12078-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70485">
    <skos:narrower rdf:resource="descriptor/18070-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70485">
    <skos:narrower rdf:resource="descriptor/12142-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70485">
    <skos:narrower rdf:resource="descriptor/19672-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70485">
    <skos:narrower rdf:resource="descriptor/20497-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70485">
    <skos:narrower rdf:resource="descriptor/19548-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70485">
    <skos:narrower rdf:resource="descriptor/10459-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70485">
    <skos:narrower rdf:resource="descriptor/20468-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70485">
    <skos:narrower rdf:resource="descriptor/15825-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70485">
    <skos:narrower rdf:resource="descriptor/12120-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70485">
    <skos:narrower rdf:resource="descriptor/11302-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70485">
    <skos:narrower rdf:resource="descriptor/12166-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70485">
    <skos:narrower rdf:resource="descriptor/12075-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70485">
    <skos:narrower rdf:resource="descriptor/18106-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70485">
    <skos:narrower rdf:resource="descriptor/15466-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70485">
    <skos:narrower rdf:resource="descriptor/21306-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70485">
    <skos:narrower rdf:resource="descriptor/25822-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70485">
    <skos:narrower rdf:resource="descriptor/19154-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70485">
    <skos:narrower rdf:resource="descriptor/19146-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70485">
    <skos:narrower rdf:resource="descriptor/10192-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70485">
    <skos:narrower rdf:resource="descriptor/19107-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70485">
    <skos:narrower rdf:resource="descriptor/19834-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19670-2">
    <skos:altLabel xml:lang="de">Volkswirtschaftlicher Verlust</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18323-0">
    <skos:broader rdf:resource="thsys/71085"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18323-0">
    <skos:broader rdf:resource="thsys/71065"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18323-0">
    <skos:broader rdf:resource="descriptor/19369-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19814-4">
    <skos:prefLabel xml:lang="en">Organized crime</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19814-4">
    <skos:prefLabel xml:lang="de">Organisierte Kriminalität</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16761-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71060">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10340-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18691-6">
    <skos:prefLabel xml:lang="en">Futures exchange</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18691-6">
    <skos:prefLabel xml:lang="de">Terminbörse</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19455-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10557-1">
    <zbwext:useInsteadNote rdf:nodeID="id87"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24857-5">
    <skos:editorialNote xml:lang="de">Vergleiche auch Hysteresis.</skos:editorialNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11306-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10717-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16214-1">
    <skos:altLabel xml:lang="de">Futurologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10216-6">
    <skos:altLabel xml:lang="de">Preisfixierer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10216-6">
    <skos:altLabel xml:lang="de">Leitpreis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19188-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19139-2">
    <skos:related rdf:resource="descriptor/19140-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19139-2">
    <skos:related rdf:resource="descriptor/13475-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19139-2">
    <skos:related rdf:resource="descriptor/12818-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19139-2">
    <skos:related rdf:resource="descriptor/12766-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19139-2">
    <skos:related rdf:resource="descriptor/18032-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19139-2">
    <skos:related rdf:resource="descriptor/18007-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14802-1">
    <skos:broader rdf:resource="thsys/70328"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14816-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11849-4">
    <skos:broader rdf:resource="thsys/71070"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11849-4">
    <skos:broader rdf:resource="thsys/71067"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13383-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19027-6">
    <skos:altLabel xml:lang="de">Anpassungsprozess</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19027-6">
    <skos:altLabel xml:lang="de">Anpassungsprozeß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19027-6">
    <skos:altLabel xml:lang="de">Adaption</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19159-3">
    <skos:altLabel xml:lang="en">Corporate failure</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19159-3">
    <skos:altLabel xml:lang="de">Bankrott</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14539-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16338-4">
    <skos:prefLabel xml:lang="en">Legislation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16338-4">
    <skos:prefLabel xml:lang="de">Gesetzgebung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18800-6">
    <skos:prefLabel xml:lang="en">Local infrastructure</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18800-6">
    <skos:prefLabel xml:lang="de">Kommunale Infrastruktur</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18953-2">
    <skos:related rdf:resource="descriptor/18954-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19325-5">
    <skos:broader rdf:resource="thsys/73336"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19325-5">
    <skos:broader rdf:resource="thsys/73332"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19325-5">
    <skos:broader rdf:resource="thsys/71098"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19325-5">
    <skos:broader rdf:resource="descriptor/19326-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19325-5">
    <skos:broader rdf:resource="descriptor/19509-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70052">
    <skos:narrower rdf:resource="descriptor/11217-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70052">
    <skos:narrower rdf:resource="descriptor/11284-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70052">
    <skos:narrower rdf:resource="descriptor/19491-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70052">
    <skos:narrower rdf:resource="descriptor/11282-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70052">
    <skos:narrower rdf:resource="descriptor/18112-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70052">
    <skos:narrower rdf:resource="descriptor/11209-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70052">
    <skos:narrower rdf:resource="descriptor/11258-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70052">
    <skos:narrower rdf:resource="descriptor/19360-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70052">
    <skos:narrower rdf:resource="descriptor/19372-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70052">
    <skos:narrower rdf:resource="descriptor/11073-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70052">
    <skos:narrower rdf:resource="descriptor/11277-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70052">
    <skos:narrower rdf:resource="descriptor/11203-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70052">
    <skos:narrower rdf:resource="descriptor/11201-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70052">
    <skos:narrower rdf:resource="descriptor/11275-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70052">
    <skos:narrower rdf:resource="descriptor/11274-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70052">
    <skos:narrower rdf:resource="descriptor/11197-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70052">
    <skos:narrower rdf:resource="descriptor/18144-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70052">
    <skos:narrower rdf:resource="descriptor/19521-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70052">
    <skos:narrower rdf:resource="descriptor/18111-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70052">
    <skos:narrower rdf:resource="descriptor/19005-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70052">
    <skos:narrower rdf:resource="descriptor/11269-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70052">
    <skos:narrower rdf:resource="descriptor/18792-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70052">
    <skos:narrower rdf:resource="descriptor/11268-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70052">
    <skos:narrower rdf:resource="descriptor/11071-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70052">
    <skos:narrower rdf:resource="descriptor/11309-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70052">
    <skos:narrower rdf:resource="descriptor/16131-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70052">
    <skos:narrower rdf:resource="descriptor/11186-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70052">
    <skos:narrower rdf:resource="descriptor/11185-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70052">
    <skos:narrower rdf:resource="descriptor/10440-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70052">
    <skos:narrower rdf:resource="descriptor/11262-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70052">
    <skos:narrower rdf:resource="descriptor/11181-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70052">
    <skos:narrower rdf:resource="descriptor/11180-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70052">
    <skos:narrower rdf:resource="descriptor/11261-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70052">
    <skos:narrower rdf:resource="thsys/70235"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70052">
    <skos:narrower rdf:resource="thsys/71059"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70052">
    <skos:narrower rdf:resource="thsys/70051"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70052">
    <skos:narrower rdf:resource="thsys/71057"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70052">
    <skos:narrower rdf:resource="thsys/71058"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70052">
    <skos:narrower rdf:resource="thsys/70278"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18444-2">
    <skos:related rdf:resource="descriptor/15679-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18444-2">
    <skos:related rdf:resource="descriptor/15763-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18444-2">
    <skos:related rdf:resource="descriptor/16004-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18444-2">
    <skos:related rdf:resource="descriptor/16670-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18444-2">
    <skos:related rdf:resource="descriptor/13292-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18444-2">
    <skos:related rdf:resource="descriptor/18892-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18444-2">
    <skos:related rdf:resource="descriptor/16683-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18444-2">
    <skos:related rdf:resource="descriptor/16479-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18444-2">
    <skos:related rdf:resource="descriptor/16635-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13558-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10785-2">
    <skos:related rdf:resource="descriptor/10791-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10785-2">
    <skos:related rdf:resource="descriptor/10789-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10785-2">
    <skos:related rdf:resource="descriptor/10877-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10785-2">
    <skos:related rdf:resource="descriptor/10746-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10785-2">
    <skos:related rdf:resource="descriptor/19328-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10785-2">
    <skos:related rdf:resource="descriptor/10880-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10785-2">
    <skos:related rdf:resource="descriptor/10805-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10785-2">
    <skos:related rdf:resource="descriptor/10698-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10785-2">
    <skos:related rdf:resource="descriptor/10776-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10785-2">
    <skos:related rdf:resource="descriptor/10854-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10785-2">
    <skos:related rdf:resource="descriptor/10850-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12078-3">
    <skos:prefLabel xml:lang="en">Crisis management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12078-3">
    <skos:prefLabel xml:lang="de">Krisenmanagement</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16521-6">
    <skos:related rdf:resource="descriptor/16332-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16521-6">
    <skos:related rdf:resource="descriptor/16295-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16521-6">
    <skos:related rdf:resource="descriptor/19855-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16521-6">
    <skos:related rdf:resource="descriptor/16463-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16521-6">
    <skos:related rdf:resource="descriptor/19683-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16521-6">
    <skos:related rdf:resource="descriptor/13412-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16521-6">
    <skos:related rdf:resource="descriptor/16454-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16521-6">
    <skos:related rdf:resource="descriptor/16510-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70924">
    <skos:broader rdf:resource="thsys/70943"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13304-1">
    <skos:prefLabel xml:lang="en">Rental of real property</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13304-1">
    <skos:prefLabel xml:lang="de">Vermietung von Immobilien</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71055">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16059-1">
    <skos:altLabel xml:lang="de">Internationales Kartellrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10980-4">
    <skos:altLabel xml:lang="de">Theorie des Staatsversagens</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19237-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13239-0">
    <skos:related rdf:resource="descriptor/18565-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13239-0">
    <skos:related rdf:resource="descriptor/14216-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13239-0">
    <skos:related rdf:resource="descriptor/13558-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13239-0">
    <skos:related rdf:resource="descriptor/18422-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13239-0">
    <skos:related rdf:resource="descriptor/13052-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20341-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18863-3">
    <skos:altLabel xml:lang="en">Girls education</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18863-3">
    <skos:altLabel xml:lang="de">Mädchenbildung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18309-1">
    <skos:altLabel xml:lang="de">Sektorale Lohndifferenzierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18309-1">
    <skos:altLabel xml:lang="de">Intersektorales Lohndifferenzial</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18309-1">
    <skos:altLabel xml:lang="de">Intersektorales Lohndifferential</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18309-1">
    <skos:altLabel xml:lang="de">Intersektorale Lohnstruktur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18309-1">
    <skos:altLabel xml:lang="de">Interindustrieller Lohnunterschied</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70904">
    <skos:narrower rdf:resource="descriptor/16046-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70904">
    <skos:narrower rdf:resource="descriptor/18704-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70904">
    <skos:narrower rdf:resource="descriptor/10774-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70904">
    <skos:narrower rdf:resource="descriptor/10721-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70904">
    <skos:narrower rdf:resource="descriptor/18698-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70904">
    <skos:narrower rdf:resource="descriptor/10382-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70904">
    <skos:narrower rdf:resource="descriptor/19260-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12995-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15874-6">
    <skos:scopeNote xml:lang="de">Wissenschaft von den Oberflächenformen der Erde</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16980-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19485-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13457-4">
    <skos:broader rdf:resource="thsys/70953"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13457-4">
    <skos:broader rdf:resource="descriptor/13469-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73356">
    <skos:broader rdf:resource="thsys/70120"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70021">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11527-5">
    <skos:altLabel xml:lang="en">Social time preference</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11527-5">
    <skos:altLabel xml:lang="en">Social time preferance rate</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11527-5">
    <skos:altLabel xml:lang="en">Social rate of discount</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11527-5">
    <skos:altLabel xml:lang="en">Social opportunity cost of capital</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11527-5">
    <skos:altLabel xml:lang="de">Gesellschaftlicher Nutzendiskont</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14661-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14653-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18438-4">
    <skos:broader rdf:resource="thsys/71104"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18438-4">
    <skos:broader rdf:resource="thsys/71010"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18438-4">
    <skos:broader rdf:resource="descriptor/18690-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18438-4">
    <skos:broader rdf:resource="descriptor/10803-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16671-1">
    <skos:prefLabel xml:lang="en">Homelessness</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16671-1">
    <skos:prefLabel xml:lang="de">Obdachlosigkeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12820-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14897-6">
    <skos:altLabel xml:lang="de">Webteppich</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14897-6">
    <skos:altLabel xml:lang="de">Textiler Teppich</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14897-6">
    <skos:altLabel xml:lang="de">Teppichboden</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14897-6">
    <skos:altLabel xml:lang="de">Orientteppich</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12724-5">
    <skos:prefLabel xml:lang="en">Mass production</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12724-5">
    <skos:prefLabel xml:lang="de">Massenproduktion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11904-1">
    <skos:prefLabel xml:lang="en">Firm location choice</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11904-1">
    <skos:prefLabel xml:lang="de">Betriebliche Standortwahl</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12766-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10827-5">
    <skos:prefLabel xml:lang="en">Border tax</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10827-5">
    <skos:prefLabel xml:lang="de">Außensteuerrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15675-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12775-2">
    <skos:related rdf:resource="descriptor/12845-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12775-2">
    <skos:related rdf:resource="descriptor/16269-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12775-2">
    <skos:related rdf:resource="descriptor/18099-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12775-2">
    <skos:related rdf:resource="descriptor/16067-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12775-2">
    <skos:related rdf:resource="descriptor/13661-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12775-2">
    <skos:related rdf:resource="descriptor/12770-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12775-2">
    <skos:related rdf:resource="descriptor/16265-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12775-2">
    <skos:related rdf:resource="descriptor/10142-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12775-2">
    <skos:related rdf:resource="descriptor/12370-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12775-2">
    <skos:related rdf:resource="descriptor/12075-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19713-3">
    <skos:broader rdf:resource="thsys/73366"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19713-3">
    <skos:broader rdf:resource="descriptor/13420-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15654-6">
    <skos:broader rdf:resource="thsys/70828"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15654-6">
    <skos:broader rdf:resource="thsys/70581"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13338-5">
    <skos:altLabel xml:lang="de">Biologische Schädlingsbekämpfung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71074">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.09.07.03</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19299-1">
    <skos:broader rdf:resource="thsys/70179"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11777-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18228-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10263-4">
    <skos:narrower rdf:resource="descriptor/11474-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15823-2">
    <skos:scopeNote xml:lang="de">Nur im persönlichkeitspsychologischen Sinne.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15810-4">
    <skos:altLabel xml:lang="de">Sozialisationstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19332-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14307-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19820-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24623-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11051-0">
    <skos:broader rdf:resource="thsys/73364"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11051-0">
    <skos:broader rdf:resource="thsys/71101"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10991-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12072-1">
    <skos:narrower rdf:resource="descriptor/20241-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12072-1">
    <skos:narrower rdf:resource="descriptor/11142-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12072-1">
    <skos:narrower rdf:resource="descriptor/11628-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12072-1">
    <skos:narrower rdf:resource="descriptor/12062-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12072-1">
    <skos:narrower rdf:resource="descriptor/18120-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12072-1">
    <skos:narrower rdf:resource="descriptor/12049-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12072-1">
    <skos:narrower rdf:resource="descriptor/12865-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11911-4">
    <skos:broader rdf:resource="thsys/70058"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11911-4">
    <skos:broader rdf:resource="descriptor/19438-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11052-5">
    <skos:scopeNote xml:lang="de">Einsatz von Instrumenten, um die Leistungsbereitschaft zu erhöhen (extrinsische Motivation).</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19822-5">
    <skos:scopeNote xml:lang="de">Wird nur indexiert bei Rechtsvergleichen mit internationalem Recht, z.B. wenn EU-Recht mit entsprechenden Regelungen des deutschen Rechts verglichen wird.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16683-1">
    <skos:related rdf:resource="descriptor/15949-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16683-1">
    <skos:related rdf:resource="descriptor/16541-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16683-1">
    <skos:related rdf:resource="descriptor/15947-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16683-1">
    <skos:related rdf:resource="descriptor/18892-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16683-1">
    <skos:related rdf:resource="descriptor/18444-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16683-1">
    <skos:related rdf:resource="descriptor/16668-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19201-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11330-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13466-3">
    <skos:altLabel xml:lang="de">Verkäuferinnen im Außendienst</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13466-3">
    <skos:altLabel xml:lang="de">Verkäufer im Außendienst</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13466-3">
    <skos:altLabel xml:lang="de">Kommissionsgeschäft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13466-3">
    <skos:altLabel xml:lang="de">Handelsvertreterinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13466-3">
    <skos:altLabel xml:lang="de">Handelsvertreter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13466-3">
    <skos:altLabel xml:lang="de">Handelsvermittlung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13466-3">
    <skos:altLabel xml:lang="de">Handelsmaklerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13466-3">
    <skos:altLabel xml:lang="de">Handelsmakler</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19090-6">
    <skos:broader rdf:resource="thsys/71033"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19090-6">
    <skos:broader rdf:resource="thsys/70005"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19090-6">
    <skos:broader rdf:resource="descriptor/18165-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19090-6">
    <skos:broader rdf:resource="descriptor/12394-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10162-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13860-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16303-2">
    <skos:prefLabel xml:lang="en">Political repression</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16303-2">
    <skos:prefLabel xml:lang="de">Politische Verfolgung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12765-5">
    <skos:broader rdf:resource="thsys/70029"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16016-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13678-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19413-1">
    <skos:narrower rdf:resource="descriptor/19409-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18377-5">
    <skos:altLabel xml:lang="de">Wasserknappheit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12041-5">
    <skos:related rdf:resource="descriptor/10190-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12041-5">
    <skos:related rdf:resource="descriptor/18872-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16445-3">
    <skos:broader rdf:resource="thsys/73365"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16445-3">
    <skos:broader rdf:resource="descriptor/16460-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10210-4">
    <skos:prefLabel xml:lang="en">Option pricing theory</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10210-4">
    <skos:prefLabel xml:lang="de">Optionspreistheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73356">
    <skos:prefLabel xml:lang="de">N.06.03.04  Soziale Probleme</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19187-5">
    <skos:related rdf:resource="descriptor/19293-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19187-5">
    <skos:related rdf:resource="descriptor/19188-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19187-5">
    <skos:related rdf:resource="descriptor/18517-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19187-5">
    <skos:related rdf:resource="descriptor/18198-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11352-0">
    <zbwext:useInsteadNote rdf:nodeID="id10"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18229-6">
    <skos:prefLabel xml:lang="en">Waste from electronic equipment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18229-6">
    <skos:prefLabel xml:lang="de">Elektronikschrott</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16039-0">
    <skos:narrower rdf:resource="descriptor/18637-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16039-0">
    <skos:narrower rdf:resource="descriptor/11840-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16039-0">
    <skos:narrower rdf:resource="descriptor/18725-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16039-0">
    <skos:narrower rdf:resource="descriptor/12033-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10593-4">
    <skos:prefLabel xml:lang="en">Commodity agreement</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10593-4">
    <skos:prefLabel xml:lang="de">Rohstoffabkommen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11817-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19365-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18102-2">
    <skos:prefLabel xml:lang="en">Operational data collection</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18102-2">
    <skos:prefLabel xml:lang="de">Betriebsdatenerfassung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11996-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12567-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70141">
    <rdfs:label xml:lang="en">History of thought</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70141">
    <rdfs:label xml:lang="de">Dogmengeschichte</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12187-5">
    <skos:related rdf:resource="descriptor/12188-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12187-5">
    <skos:related rdf:resource="descriptor/12183-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12979-0">
    <skos:altLabel xml:lang="de">Wasserwerk</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14343-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16720-0">
    <skos:broader rdf:resource="thsys/73393"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16720-0">
    <skos:broader rdf:resource="descriptor/16708-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24704-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16087-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19055-1">
    <skos:altLabel xml:lang="de">Drilling</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14197-6">
    <skos:altLabel xml:lang="en">Crude oil</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14197-6">
    <skos:altLabel xml:lang="de">Mineralöl</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14026-5">
    <skos:prefLabel xml:lang="en">Chemist's goods</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14026-5">
    <skos:prefLabel xml:lang="de">Drogeriewaren</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15979-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16633-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18640-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id262">
    <rdf:value xml:lang="de">Für die ökonomische Theorie privater Haushalte</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70956">
    <rdfs:label xml:lang="en">Commercial management</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70956">
    <rdfs:label xml:lang="de">Handelsbetriebslehre</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19514-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19071-3">
    <skos:broader rdf:resource="thsys/73366"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19071-3">
    <skos:broader rdf:resource="thsys/70179"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19071-3">
    <skos:broader rdf:resource="thsys/70140"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19071-3">
    <skos:broader rdf:resource="descriptor/10617-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19281-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10076-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18380-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12949-2">
    <skos:broader rdf:resource="thsys/71029"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12949-2">
    <skos:broader rdf:resource="descriptor/12945-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16696-6">
    <skos:related rdf:resource="descriptor/10473-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16696-6">
    <skos:related rdf:resource="descriptor/10470-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16696-6">
    <skos:related rdf:resource="descriptor/16695-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16696-6">
    <skos:related rdf:resource="descriptor/15025-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16696-6">
    <skos:related rdf:resource="descriptor/19765-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16696-6">
    <skos:related rdf:resource="descriptor/16674-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16696-6">
    <skos:related rdf:resource="descriptor/11529-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18004-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14661-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15108-5">
    <skos:altLabel xml:lang="de">Statistik des Verbrauchs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15108-5">
    <skos:altLabel xml:lang="de">Konsumstatistik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12064-0">
    <skos:related rdf:resource="descriptor/10985-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12064-0">
    <skos:related rdf:resource="descriptor/13992-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12064-0">
    <skos:related rdf:resource="descriptor/18822-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12064-0">
    <skos:related rdf:resource="descriptor/10979-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12064-0">
    <skos:related rdf:resource="descriptor/11722-0"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id228">
    <rdfs:seeAlso rdf:resource="descriptor/16309-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16914-1">
    <skos:prefLabel xml:lang="en">Schleswig-Holstein</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16914-1">
    <skos:prefLabel xml:lang="de">Schleswig-Holstein</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73347">
    <skos:narrower rdf:resource="descriptor/18314-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73347">
    <skos:narrower rdf:resource="descriptor/18895-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73347">
    <skos:narrower rdf:resource="descriptor/19016-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73347">
    <skos:narrower rdf:resource="descriptor/15949-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73347">
    <skos:narrower rdf:resource="descriptor/19788-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73347">
    <skos:narrower rdf:resource="descriptor/19529-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73347">
    <skos:narrower rdf:resource="descriptor/19528-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73347">
    <skos:narrower rdf:resource="descriptor/19527-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73347">
    <skos:narrower rdf:resource="descriptor/18301-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73347">
    <skos:narrower rdf:resource="descriptor/15947-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73347">
    <skos:narrower rdf:resource="descriptor/15659-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73347">
    <skos:narrower rdf:resource="descriptor/15658-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73347">
    <skos:narrower rdf:resource="descriptor/19526-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73347">
    <skos:narrower rdf:resource="descriptor/19787-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73347">
    <skos:narrower rdf:resource="descriptor/19840-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73347">
    <skos:narrower rdf:resource="descriptor/18892-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73347">
    <skos:narrower rdf:resource="descriptor/18891-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73347">
    <skos:narrower rdf:resource="descriptor/16683-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73347">
    <skos:narrower rdf:resource="descriptor/16635-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73347">
    <skos:narrower rdf:resource="descriptor/18335-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73347">
    <skos:narrower rdf:resource="descriptor/10495-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73347">
    <skos:narrower rdf:resource="descriptor/12612-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73347">
    <skos:narrower rdf:resource="descriptor/19539-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73347">
    <skos:narrower rdf:resource="descriptor/18894-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73347">
    <skos:narrower rdf:resource="descriptor/19786-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73347">
    <skos:narrower rdf:resource="descriptor/16697-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73347">
    <skos:narrower rdf:resource="thsys/73346"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18366-3">
    <skos:prefLabel xml:lang="en">Military technology</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18366-3">
    <skos:prefLabel xml:lang="de">Wehrtechnik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14775-6">
    <skos:altLabel xml:lang="de">Verbundglas</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14775-6">
    <skos:altLabel xml:lang="de">Spiegelglas</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14775-6">
    <skos:altLabel xml:lang="de">Sicherheitsglas</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14775-6">
    <skos:altLabel xml:lang="de">Fensterglas</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13297-0">
    <skos:narrower rdf:resource="descriptor/13411-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13297-0">
    <skos:narrower rdf:resource="descriptor/13299-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13297-0">
    <skos:narrower rdf:resource="descriptor/13817-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13297-0">
    <skos:narrower rdf:resource="descriptor/13816-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13297-0">
    <skos:narrower rdf:resource="descriptor/13295-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13297-0">
    <skos:narrower rdf:resource="descriptor/13814-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13297-0">
    <skos:narrower rdf:resource="descriptor/13441-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13297-0">
    <skos:narrower rdf:resource="descriptor/13293-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13297-0">
    <skos:narrower rdf:resource="descriptor/13813-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13297-0">
    <skos:narrower rdf:resource="descriptor/13810-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19335-2">
    <skos:prefLabel xml:lang="en">Exchange-rate regime</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19335-2">
    <skos:prefLabel xml:lang="de">Wechselkurssystem</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12897-2">
    <skos:prefLabel xml:lang="en">Generally accepted auditing standards</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12897-2">
    <skos:prefLabel xml:lang="de">Grundsätze ordnungsmäßiger Prüfung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11900-2">
    <skos:prefLabel xml:lang="en">Regional planning</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11900-2">
    <skos:prefLabel xml:lang="de">Regionalplanung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14581-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12103-2">
    <skos:related rdf:resource="descriptor/24857-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12103-2">
    <skos:related rdf:resource="descriptor/12112-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12103-2">
    <skos:related rdf:resource="descriptor/15825-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71108">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18507-4">
    <skos:narrower rdf:resource="descriptor/11845-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18507-4">
    <skos:narrower rdf:resource="descriptor/13562-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18507-4">
    <skos:narrower rdf:resource="descriptor/18508-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11879-2">
    <skos:narrower rdf:resource="descriptor/11970-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11879-2">
    <skos:narrower rdf:resource="descriptor/11862-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70952">
    <rdfs:label xml:lang="en">Foreign trade industries</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70952">
    <rdfs:label xml:lang="de">Außenhandelswirtschaft</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71085">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10696-1">
    <skos:related rdf:resource="descriptor/10986-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10696-1">
    <skos:related rdf:resource="descriptor/19071-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10696-1">
    <skos:related rdf:resource="descriptor/19317-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11199-3">
    <skos:broader rdf:resource="thsys/71059"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11199-3">
    <skos:broader rdf:resource="thsys/71050"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11199-3">
    <skos:broader rdf:resource="descriptor/11212-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16396-4">
    <skos:related rdf:resource="descriptor/16389-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16396-4">
    <skos:related rdf:resource="descriptor/15711-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16396-4">
    <skos:related rdf:resource="descriptor/16303-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16396-4">
    <skos:related rdf:resource="descriptor/16286-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16396-4">
    <skos:related rdf:resource="descriptor/16304-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12782-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14244-2">
    <skos:prefLabel xml:lang="en">Talc</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14244-2">
    <skos:prefLabel xml:lang="de">Talk</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10488-1">
    <skos:broader rdf:resource="thsys/73395"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10488-1">
    <skos:broader rdf:resource="thsys/73356"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10488-1">
    <skos:broader rdf:resource="thsys/70504"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10488-1">
    <skos:broader rdf:resource="thsys/70091"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10488-1">
    <skos:broader rdf:resource="descriptor/16639-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16774-5">
    <skos:scopeNote xml:lang="de">Für die Anbieter von Datenbanken BENUTZE: Datenbankanbieter. Vgl. auch: Informationssystem.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70995">
    <skos:narrower rdf:resource="descriptor/13083-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70995">
    <skos:narrower rdf:resource="descriptor/13081-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70995">
    <skos:narrower rdf:resource="descriptor/13080-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70995">
    <skos:narrower rdf:resource="descriptor/13082-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70995">
    <skos:narrower rdf:resource="descriptor/13078-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10267-3">
    <skos:scopeNote xml:lang="de">Staatliche konsumtive Ausgaben</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16606-5">
    <skos:altLabel xml:lang="de">Postmoderne Gesellschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16606-5">
    <skos:altLabel xml:lang="de">Nachindustrielle Gesellschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12615-3">
    <skos:prefLabel xml:lang="en">Occupational health</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12615-3">
    <skos:prefLabel xml:lang="de">Betriebliche Gesundheitspolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10199-1">
    <skos:prefLabel xml:lang="en">Hedonic price</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10199-1">
    <skos:prefLabel xml:lang="de">Hedonischer Preis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18522-1">
    <skos:narrower rdf:resource="descriptor/18450-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18522-1">
    <skos:narrower rdf:resource="descriptor/19061-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18522-1">
    <skos:narrower rdf:resource="descriptor/18436-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18522-1">
    <skos:narrower rdf:resource="descriptor/13235-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18522-1">
    <skos:narrower rdf:resource="descriptor/13233-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18522-1">
    <skos:narrower rdf:resource="descriptor/13232-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18522-1">
    <skos:narrower rdf:resource="descriptor/13226-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18522-1">
    <skos:narrower rdf:resource="descriptor/13231-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18522-1">
    <skos:narrower rdf:resource="descriptor/13222-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18522-1">
    <skos:narrower rdf:resource="descriptor/13220-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18522-1">
    <skos:narrower rdf:resource="descriptor/18926-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18522-1">
    <skos:narrower rdf:resource="descriptor/13213-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18522-1">
    <skos:narrower rdf:resource="descriptor/13229-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18522-1">
    <skos:narrower rdf:resource="descriptor/13202-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18522-1">
    <skos:narrower rdf:resource="descriptor/13197-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15646-5">
    <skos:related rdf:resource="descriptor/15637-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18828-5">
    <skos:prefLabel xml:lang="en">Economics of regulation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18828-5">
    <skos:prefLabel xml:lang="de">Theorie der Regulierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10024-1">
    <skos:broader rdf:resource="thsys/71115"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10024-1">
    <skos:broader rdf:resource="thsys/70184"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19066-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19557-5">
    <skos:altLabel xml:lang="de">Internationale Projektfinanzierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19966-2">
    <skos:narrower rdf:resource="descriptor/19676-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15677-1">
    <zbwext:useInsteadNote rdf:nodeID="id346"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10779-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70817">
    <skos:narrower rdf:resource="descriptor/16023-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70817">
    <skos:narrower rdf:resource="descriptor/18379-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70817">
    <skos:narrower rdf:resource="descriptor/13334-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70817">
    <skos:narrower rdf:resource="descriptor/13331-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70817">
    <skos:narrower rdf:resource="descriptor/15731-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70817">
    <skos:narrower rdf:resource="descriptor/15734-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70817">
    <skos:narrower rdf:resource="descriptor/15733-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70817">
    <skos:narrower rdf:resource="descriptor/13329-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70817">
    <skos:narrower rdf:resource="descriptor/13327-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70817">
    <skos:narrower rdf:resource="descriptor/15730-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19551-3">
    <skos:broader rdf:resource="thsys/70891"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19551-3">
    <skos:broader rdf:resource="thsys/70839"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19551-3">
    <skos:broader rdf:resource="thsys/70832"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19551-3">
    <skos:broader rdf:resource="thsys/70573"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19551-3">
    <skos:broader rdf:resource="thsys/70458"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19551-3">
    <skos:broader rdf:resource="descriptor/13292-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19551-3">
    <skos:broader rdf:resource="descriptor/18891-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16016-5">
    <skos:related rdf:resource="descriptor/11069-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16016-5">
    <skos:related rdf:resource="descriptor/19703-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16016-5">
    <skos:related rdf:resource="descriptor/11057-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16016-5">
    <skos:related rdf:resource="descriptor/13422-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16016-5">
    <skos:related rdf:resource="descriptor/16017-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16016-5">
    <skos:related rdf:resource="descriptor/19689-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16016-5">
    <skos:related rdf:resource="descriptor/15974-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16016-5">
    <skos:related rdf:resource="descriptor/15995-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16016-5">
    <skos:related rdf:resource="descriptor/16268-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70471">
    <skos:prefLabel xml:lang="en">B.06.03  Computer-aided manufacturing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70471">
    <skos:prefLabel xml:lang="de">B.06.03  Automatisierte Produktion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70799">
    <skos:narrower rdf:resource="descriptor/18427-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70799">
    <skos:narrower rdf:resource="descriptor/10455-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70799">
    <skos:narrower rdf:resource="descriptor/13074-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70799">
    <skos:narrower rdf:resource="descriptor/18425-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70799">
    <skos:narrower rdf:resource="descriptor/19396-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70799">
    <skos:narrower rdf:resource="descriptor/18424-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70799">
    <skos:narrower rdf:resource="descriptor/13072-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16757-5">
    <skos:related rdf:resource="descriptor/13817-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16757-5">
    <skos:related rdf:resource="descriptor/15624-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16757-5">
    <skos:related rdf:resource="descriptor/18661-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16757-5">
    <skos:related rdf:resource="descriptor/13682-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16757-5">
    <skos:related rdf:resource="descriptor/24708-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16757-5">
    <skos:related rdf:resource="descriptor/16760-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16757-5">
    <skos:related rdf:resource="descriptor/15598-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16757-5">
    <skos:related rdf:resource="descriptor/18017-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16757-5">
    <skos:related rdf:resource="descriptor/15609-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16757-5">
    <skos:related rdf:resource="descriptor/18425-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16757-5">
    <skos:related rdf:resource="descriptor/15606-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16757-5">
    <skos:related rdf:resource="descriptor/15605-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16757-5">
    <skos:related rdf:resource="descriptor/18758-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16757-5">
    <skos:related rdf:resource="descriptor/13072-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16757-5">
    <skos:related rdf:resource="descriptor/14751-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16757-5">
    <skos:related rdf:resource="descriptor/19660-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16283-3">
    <skos:scopeNote xml:lang="de">Für innen- und gesellschaftspolitische Reformen im weitesten Sinne</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18502-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19518-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15811-2">
    <skos:scopeNote xml:lang="de">Untersuchung von Interdependenzen zwischen sozialen Einflüssen und der Entwicklung sowie dem Verhalten des Individuums</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15741-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15837-5">
    <skos:prefLabel xml:lang="en">Job satisfaction</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15837-5">
    <skos:prefLabel xml:lang="de">Arbeitszufriedenheit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19804-0">
    <skos:prefLabel xml:lang="en">Software agent</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19804-0">
    <skos:prefLabel xml:lang="de">Software-Agent</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12816-0">
    <skos:altLabel xml:lang="de">Winterschlußverkauf</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12816-0">
    <skos:altLabel xml:lang="de">Winterschlussverkauf</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12816-0">
    <skos:altLabel xml:lang="de">Verkaufsfahrt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12816-0">
    <skos:altLabel xml:lang="de">Sonderangebot</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12816-0">
    <skos:altLabel xml:lang="de">Sommerschlußverkauf</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12816-0">
    <skos:altLabel xml:lang="de">Sommerschlussverkauf</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12816-0">
    <skos:altLabel xml:lang="de">Schlußverkauf</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12816-0">
    <skos:altLabel xml:lang="de">Schlussverkauf</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12816-0">
    <skos:altLabel xml:lang="de">Räumungsverkauf</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12816-0">
    <skos:altLabel xml:lang="de">Partievermarktung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12816-0">
    <skos:altLabel xml:lang="de">Jubiläumsverkauf</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12816-0">
    <skos:altLabel xml:lang="de">Ausverkauf</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18652-2">
    <skos:altLabel xml:lang="de">Telefontarif</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18652-2">
    <skos:altLabel xml:lang="de">Telefongebühr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18652-2">
    <skos:altLabel xml:lang="de">Fernmeldegebühr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11143-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19533-5">
    <skos:scopeNote xml:lang="de">Direkte oder indirekte wirtschaftspolitische Instrumente, welche die Wirtschaftssubjekte zu einem ökonomisch erwünschten Handeln veranlassen sollen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12265-4">
    <skos:altLabel xml:lang="en">Underwriting</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12265-4">
    <skos:altLabel xml:lang="en">IPO (Initial Public Offer)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12265-4">
    <skos:altLabel xml:lang="en">Initial public offer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12265-4">
    <skos:altLabel xml:lang="en">Going public</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12265-4">
    <skos:altLabel xml:lang="de">Prospekt (Börse)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12265-4">
    <skos:altLabel xml:lang="de">Neuemission von Wertpapieren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12265-4">
    <skos:altLabel xml:lang="de">Emission von Wertpapieren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12265-4">
    <skos:altLabel xml:lang="de">Börsenzulassung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12265-4">
    <skos:altLabel xml:lang="de">Börseneinführung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12265-4">
    <skos:altLabel xml:lang="de">Aktienerstemission</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12265-4">
    <skos:altLabel xml:lang="de">Aktienemission</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18784-6">
    <skos:prefLabel xml:lang="en">Research financing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18784-6">
    <skos:prefLabel xml:lang="de">Forschungsfinanzierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12361-1">
    <skos:prefLabel xml:lang="en">Optional inclusion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12361-1">
    <skos:prefLabel xml:lang="de">Bilanzierungswahlrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16055-2">
    <skos:related rdf:resource="descriptor/11819-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16055-2">
    <skos:related rdf:resource="descriptor/11818-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16055-2">
    <skos:related rdf:resource="descriptor/18509-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16055-2">
    <skos:related rdf:resource="descriptor/12910-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18475-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14796-5">
    <skos:altLabel xml:lang="de">Papierabfall</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id153">
    <rdfs:seeAlso rdf:resource="descriptor/10572-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12437-1">
    <skos:prefLabel xml:lang="en">Fraction value</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12437-1">
    <skos:prefLabel xml:lang="de">Teilwert</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16432-5">
    <skos:prefLabel xml:lang="en">Political goal</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16432-5">
    <skos:prefLabel xml:lang="de">Politisches Ziel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14293-3">
    <skos:broader rdf:resource="thsys/70134"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14293-3">
    <skos:broader rdf:resource="descriptor/14302-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19440-5">
    <skos:broader rdf:resource="thsys/70278"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19440-5">
    <skos:broader rdf:resource="thsys/70058"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19440-5">
    <skos:broader rdf:resource="descriptor/11875-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19440-5">
    <skos:broader rdf:resource="descriptor/11327-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18197-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11840-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16192-6">
    <skos:related rdf:resource="descriptor/16700-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16192-6">
    <skos:related rdf:resource="descriptor/15924-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17571-0">
    <skos:narrower rdf:resource="descriptor/19166-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17571-0">
    <skos:narrower rdf:resource="descriptor/17568-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17571-0">
    <skos:narrower rdf:resource="descriptor/19165-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73371">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18969-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11490-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13130-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10326-6">
    <skos:related rdf:resource="descriptor/16754-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10326-6">
    <skos:related rdf:resource="descriptor/10315-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10326-6">
    <skos:related rdf:resource="descriptor/10186-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10326-6">
    <skos:related rdf:resource="descriptor/10185-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10326-6">
    <skos:related rdf:resource="descriptor/10314-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10326-6">
    <skos:related rdf:resource="descriptor/10313-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10326-6">
    <skos:related rdf:resource="descriptor/10040-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12204-3">
    <skos:altLabel xml:lang="de">Theorie effizienter Märkte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12204-3">
    <skos:altLabel xml:lang="de">Effizienzmarkthypothese</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12204-3">
    <skos:altLabel xml:lang="de">Effizienter-Markt-Hypothese</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12204-3">
    <skos:altLabel xml:lang="de">Effizienter Markt Hypothese</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19361-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id277">
    <rdfs:seeAlso rdf:resource="descriptor/19841-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18011-5">
    <skos:prefLabel xml:lang="en">Deregulation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18011-5">
    <skos:prefLabel xml:lang="de">Deregulierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12404-2">
    <skos:broader rdf:resource="thsys/71043"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12404-2">
    <skos:broader rdf:resource="descriptor/12399-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10513-0">
    <skos:scopeNote xml:lang="de">Für die längerfristige Entwicklung von Industrieländern BENUTZE Wirtschaftswachstum oder Volkswirtschaft</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10513-0">
    <skos:scopeNote xml:lang="de">Nur für die wirtschaftliche Entwicklung unterentwickelter Länder, in deren Verlauf quantitativer und qualitativer Lebensstandard ansteigen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19799-2">
    <skos:narrower rdf:resource="descriptor/18402-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14712-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14175-2">
    <skos:prefLabel xml:lang="en">Energy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14175-2">
    <skos:prefLabel xml:lang="de">Energie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15927-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19665-2">
    <skos:prefLabel xml:lang="en">Lender of last resort</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19665-2">
    <skos:prefLabel xml:lang="de">Lender of Last Resort</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14601-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13067-3">
    <skos:related rdf:resource="descriptor/14393-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13067-3">
    <skos:related rdf:resource="descriptor/13090-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18765-3">
    <skos:altLabel xml:lang="en">Software tools</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18765-3">
    <skos:altLabel xml:lang="en">Computer-aided software engineering</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18765-3">
    <skos:altLabel xml:lang="de">Softwarewerkzeuge</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18765-3">
    <skos:altLabel xml:lang="de">Software-Werkzeuge</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18765-3">
    <skos:altLabel xml:lang="de">Rechnerunterstützte Software-Entwicklung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14060-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13799-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18914-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16023-1">
    <skos:broader rdf:resource="thsys/73390"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16023-1">
    <skos:broader rdf:resource="thsys/71011"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16023-1">
    <skos:broader rdf:resource="thsys/70817"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16023-1">
    <skos:broader rdf:resource="descriptor/16081-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16023-1">
    <skos:broader rdf:resource="descriptor/15769-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15742-2">
    <skos:related rdf:resource="descriptor/15743-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15742-2">
    <skos:related rdf:resource="descriptor/19623-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15742-2">
    <skos:related rdf:resource="descriptor/15728-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11695-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11124-6">
    <skos:broader rdf:resource="thsys/71101"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11124-6">
    <skos:broader rdf:resource="descriptor/11616-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11124-6">
    <skos:broader rdf:resource="descriptor/10005-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10278-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19692-6">
    <skos:prefLabel xml:lang="en">Voting</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19692-6">
    <skos:prefLabel xml:lang="de">Abstimmung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15940-5">
    <skos:related rdf:resource="descriptor/11032-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15940-5">
    <skos:related rdf:resource="descriptor/15938-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10621-4">
    <skos:scopeNote xml:lang="de">Keine einheitliche Abgrenzung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11968-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15770-4">
    <skos:broader rdf:resource="thsys/73351"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15770-4">
    <skos:broader rdf:resource="thsys/71033"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15770-4">
    <skos:broader rdf:resource="thsys/70815"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15770-4">
    <skos:broader rdf:resource="thsys/70239"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17986-6">
    <skos:prefLabel xml:lang="en">French speaking Africa</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17986-6">
    <skos:prefLabel xml:lang="de">Frankophones Afrika</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10572-5">
    <skos:prefLabel xml:lang="en">Joint venture</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10572-5">
    <skos:prefLabel xml:lang="de">Joint Venture</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13723-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11886-5">
    <skos:broader rdf:resource="thsys/71069"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11886-5">
    <skos:broader rdf:resource="thsys/70867"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11886-5">
    <skos:broader rdf:resource="descriptor/11020-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13451-2">
    <skos:broader rdf:resource="thsys/73390"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13451-2">
    <skos:broader rdf:resource="thsys/71028"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13451-2">
    <skos:broader rdf:resource="descriptor/11069-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70257">
    <skos:prefLabel xml:lang="de">P.09  Fahrzeuge</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12975-1">
    <skos:prefLabel xml:lang="en">Power industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12975-1">
    <skos:prefLabel xml:lang="de">Energiewirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16449-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19580-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16404-3">
    <skos:related rdf:resource="descriptor/16286-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16404-3">
    <skos:related rdf:resource="descriptor/16395-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16404-3">
    <skos:related rdf:resource="descriptor/16372-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16404-3">
    <skos:related rdf:resource="descriptor/16416-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16404-3">
    <skos:related rdf:resource="descriptor/16249-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16404-3">
    <skos:related rdf:resource="descriptor/15932-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16404-3">
    <skos:related rdf:resource="descriptor/16400-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16404-3">
    <skos:related rdf:resource="descriptor/16397-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18329-2">
    <skos:related rdf:resource="descriptor/13437-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18329-2">
    <skos:related rdf:resource="descriptor/18333-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18329-2">
    <skos:related rdf:resource="descriptor/19339-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18433-0">
    <skos:broader rdf:resource="thsys/73392"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18433-0">
    <skos:broader rdf:resource="thsys/73297"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18433-0">
    <skos:broader rdf:resource="thsys/70882"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18433-0">
    <skos:broader rdf:resource="descriptor/16026-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12618-4">
    <skos:broader rdf:resource="thsys/73346"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12618-4">
    <skos:broader rdf:resource="thsys/71050"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12618-4">
    <skos:broader rdf:resource="thsys/70838"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12618-4">
    <skos:broader rdf:resource="descriptor/16670-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10622-2">
    <skos:broader rdf:resource="thsys/70278"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10622-2">
    <skos:broader rdf:resource="descriptor/11322-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11540-6">
    <skos:prefLabel xml:lang="en">Public Finance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11540-6">
    <skos:prefLabel xml:lang="de">Öffentliche Finanzwirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19593-1">
    <skos:altLabel xml:lang="en">Strategic form game</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19593-1">
    <skos:altLabel xml:lang="en">Normal game</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19593-1">
    <skos:altLabel xml:lang="en">Matrix form game</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19593-1">
    <skos:altLabel xml:lang="de">Spiel in strategischer Form</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18363-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12250-3">
    <skos:altLabel xml:lang="en">Drawing credit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12250-3">
    <skos:altLabel xml:lang="de">Dispositionskredit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10029-5">
    <skos:prefLabel xml:lang="en">Economic power</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10029-5">
    <skos:prefLabel xml:lang="de">Wirtschaftliche Macht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14844-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10568-3">
    <skos:broader rdf:resource="thsys/71084"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19368-1">
    <skos:prefLabel xml:lang="en">Rural development planning</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19368-1">
    <skos:prefLabel xml:lang="de">Ländliche Entwicklungsplanung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15868-1">
    <skos:altLabel xml:lang="de">Fluß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17588-4">
    <skos:altLabel xml:lang="de">Republik Singapur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70504">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.14.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13558-5">
    <zbwext:useInsteadNote rdf:nodeID="id176"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18759-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70833">
    <skos:broader rdf:resource="thsys/70835"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10806-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19076-0">
    <skos:broader rdf:resource="thsys/70361"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19076-0">
    <skos:broader rdf:resource="descriptor/18012-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19432-4">
    <skos:related rdf:resource="descriptor/10948-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19432-4">
    <skos:related rdf:resource="descriptor/10928-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19432-4">
    <skos:related rdf:resource="descriptor/19667-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18583-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73360">
    <skos:narrower rdf:resource="descriptor/11573-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73360">
    <skos:narrower rdf:resource="descriptor/16024-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73360">
    <skos:narrower rdf:resource="descriptor/19858-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73360">
    <skos:narrower rdf:resource="descriptor/16336-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73360">
    <skos:narrower rdf:resource="descriptor/16019-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73360">
    <skos:narrower rdf:resource="descriptor/13423-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73360">
    <skos:narrower rdf:resource="descriptor/18272-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73360">
    <skos:narrower rdf:resource="descriptor/15769-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73360">
    <skos:narrower rdf:resource="descriptor/16014-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73360">
    <skos:narrower rdf:resource="descriptor/18870-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73360">
    <skos:narrower rdf:resource="descriptor/16012-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73360">
    <skos:narrower rdf:resource="descriptor/19854-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73360">
    <skos:narrower rdf:resource="descriptor/11870-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73360">
    <skos:narrower rdf:resource="descriptor/16010-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73360">
    <skos:narrower rdf:resource="descriptor/18829-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73360">
    <skos:narrower rdf:resource="descriptor/15746-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73360">
    <skos:narrower rdf:resource="descriptor/13660-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73360">
    <skos:narrower rdf:resource="descriptor/16004-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73360">
    <skos:narrower rdf:resource="descriptor/18805-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73360">
    <skos:narrower rdf:resource="descriptor/15995-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73360">
    <skos:narrower rdf:resource="descriptor/15737-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73360">
    <skos:narrower rdf:resource="descriptor/18876-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73360">
    <skos:narrower rdf:resource="descriptor/18288-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73360">
    <skos:narrower rdf:resource="descriptor/19821-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73360">
    <skos:narrower rdf:resource="descriptor/19847-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73360">
    <skos:narrower rdf:resource="descriptor/11771-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73360">
    <skos:narrower rdf:resource="descriptor/15604-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73360">
    <skos:narrower rdf:resource="descriptor/15992-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73360">
    <skos:narrower rdf:resource="descriptor/12020-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73360">
    <skos:narrower rdf:resource="descriptor/16134-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73360">
    <skos:narrower rdf:resource="descriptor/16176-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73360">
    <skos:narrower rdf:resource="descriptor/15989-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73360">
    <skos:narrower rdf:resource="descriptor/19845-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73360">
    <skos:narrower rdf:resource="descriptor/18259-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73360">
    <skos:narrower rdf:resource="descriptor/15988-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11284-5">
    <zbwext:useInsteadNote rdf:nodeID="id360"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16166-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71019">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14171-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15475-6">
    <skos:narrower rdf:resource="descriptor/15469-1"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id27">
    <rdfs:seeAlso rdf:resource="descriptor/10614-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73393">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.01.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16760-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12668-3">
    <skos:broader rdf:resource="thsys/70953"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12668-3">
    <skos:broader rdf:resource="descriptor/13469-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12668-3">
    <skos:broader rdf:resource="descriptor/12640-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13273-0">
    <skos:broader rdf:resource="thsys/70923"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13273-0">
    <skos:broader rdf:resource="descriptor/13274-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11228-1">
    <skos:broader rdf:resource="thsys/71058"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11228-1">
    <skos:broader rdf:resource="thsys/70843"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11228-1">
    <skos:broader rdf:resource="descriptor/11366-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11228-1">
    <skos:broader rdf:resource="descriptor/11361-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14466-5">
    <skos:prefLabel xml:lang="en">Power plug and socket equipment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14466-5">
    <skos:prefLabel xml:lang="de">Stromverteilungseinrichtung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12898-0">
    <skos:prefLabel xml:lang="en">Annual audit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12898-0">
    <skos:prefLabel xml:lang="de">Jahresabschlussprüfung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15298-2">
    <skos:prefLabel xml:lang="en">Statistical test</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15298-2">
    <skos:prefLabel xml:lang="de">Statistischer Test</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19309-3">
    <skos:scopeNote xml:lang="de">Gütermarkt- und Faktormarktintegration.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10471-4">
    <skos:broader rdf:resource="thsys/73345"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17979-3">
    <skos:altLabel xml:lang="de">ASEAN-Länder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70505">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">G.01.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13576-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13565-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13176-5">
    <skos:prefLabel xml:lang="en">Road construction</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13176-5">
    <skos:prefLabel xml:lang="de">Straßenbau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16972-1">
    <skos:prefLabel xml:lang="en">Switzerland</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16972-1">
    <skos:prefLabel xml:lang="de">Schweiz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10562-1">
    <skos:broader rdf:resource="thsys/71085"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10562-1">
    <skos:broader rdf:resource="thsys/71070"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10562-1">
    <skos:broader rdf:resource="thsys/71068"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10562-1">
    <skos:broader rdf:resource="descriptor/11895-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10562-1">
    <skos:broader rdf:resource="descriptor/10524-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16190-3">
    <skos:narrower rdf:resource="descriptor/19800-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15893-2">
    <skos:related rdf:resource="descriptor/15903-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15893-2">
    <skos:related rdf:resource="descriptor/15894-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15893-2">
    <skos:related rdf:resource="descriptor/15930-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15893-2">
    <skos:related rdf:resource="descriptor/19620-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14064-4">
    <skos:prefLabel xml:lang="en">Coffee</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14064-4">
    <skos:prefLabel xml:lang="de">Kaffee</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16949-3">
    <skos:broader rdf:resource="thsys/70438"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16949-3">
    <skos:broader rdf:resource="descriptor/16950-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10250-6">
    <skos:altLabel xml:lang="de">Neoquantitätstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10250-6">
    <skos:altLabel xml:lang="de">Friedmansche Quantitätstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10250-6">
    <skos:altLabel xml:lang="de">Chicago-Schule</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16806-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18251-6">
    <skos:narrower rdf:resource="descriptor/14496-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18251-6">
    <skos:narrower rdf:resource="descriptor/14514-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18251-6">
    <skos:narrower rdf:resource="descriptor/14487-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18251-6">
    <skos:narrower rdf:resource="descriptor/18290-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10833-3">
    <zbwext:useInsteadNote rdf:nodeID="id31"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17464-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16439-5">
    <skos:altLabel xml:lang="en">Intergovernmental organization</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16439-5">
    <skos:altLabel xml:lang="en">Intergovernmental organisation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16439-5">
    <skos:altLabel xml:lang="de">Multilaterale Institution</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16439-5">
    <skos:altLabel xml:lang="de">Internationale staatliche Organisation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16439-5">
    <skos:altLabel xml:lang="de">Internationale Regierungsorganisation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12615-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16729-3">
    <skos:altLabel xml:lang="de">Theorie-Praxis Problem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16729-3">
    <skos:altLabel xml:lang="de">Praxis-Theorie-Verhältnis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16729-3">
    <skos:altLabel xml:lang="de">Praxis-Theorie-Problem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19677-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id69">
    <rdfs:seeAlso rdf:resource="descriptor/11849-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#narrower">
    <skos:definition xml:lang="en">A concept that is more specific in meaning.</skos:definition>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13529-5">
    <skos:prefLabel xml:lang="en">Inland waterway</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13529-5">
    <skos:prefLabel xml:lang="de">Binnenwasserstraße</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18339-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16594-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16619-3">
    <skos:related rdf:resource="descriptor/19507-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16619-3">
    <skos:related rdf:resource="descriptor/16663-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16619-3">
    <skos:related rdf:resource="descriptor/16671-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16619-3">
    <skos:related rdf:resource="descriptor/19553-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16619-3">
    <skos:related rdf:resource="descriptor/10488-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16619-3">
    <skos:related rdf:resource="descriptor/11070-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15627-2">
    <skos:altLabel xml:lang="de">Menschenkunde</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15627-2">
    <skos:altLabel xml:lang="de">Anthropometrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19389-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12978-2">
    <skos:narrower rdf:resource="descriptor/18356-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16548-0">
    <skos:scopeNote xml:lang="de">Modell einer steuerfinanzierten Grundrente.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18352-0">
    <skos:narrower rdf:resource="descriptor/18372-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73392">
    <skos:prefLabel xml:lang="en">N.05.02.01  Civil code</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73392">
    <skos:prefLabel xml:lang="de">N.05.02.01  Bürgerliches Recht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70114">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.05.07</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14548-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15025-2">
    <skos:broader rdf:resource="thsys/72182"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15025-2">
    <skos:broader rdf:resource="thsys/70485"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15025-2">
    <skos:broader rdf:resource="descriptor/16214-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15025-2">
    <skos:broader rdf:resource="descriptor/15072-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11527-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11970-2">
    <skos:broader rdf:resource="thsys/71064"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11970-2">
    <skos:broader rdf:resource="thsys/70870"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11970-2">
    <skos:broader rdf:resource="descriptor/11879-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11970-2">
    <skos:broader rdf:resource="descriptor/11852-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19312-0">
    <skos:related rdf:resource="descriptor/19319-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19312-0">
    <skos:related rdf:resource="descriptor/20321-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16849-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12236-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16027-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70850">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70394">
    <skos:narrower rdf:resource="descriptor/18092-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70394">
    <skos:narrower rdf:resource="descriptor/12778-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70394">
    <skos:narrower rdf:resource="descriptor/18620-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70394">
    <skos:narrower rdf:resource="descriptor/21020-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70394">
    <skos:narrower rdf:resource="descriptor/12644-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70394">
    <skos:narrower rdf:resource="descriptor/19114-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70394">
    <skos:narrower rdf:resource="descriptor/12670-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70394">
    <skos:narrower rdf:resource="descriptor/12643-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70394">
    <skos:narrower rdf:resource="descriptor/10002-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70394">
    <skos:narrower rdf:resource="descriptor/12035-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70394">
    <skos:narrower rdf:resource="thsys/70386"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70394">
    <skos:narrower rdf:resource="thsys/70300"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73391">
    <skos:prefLabel xml:lang="de">N.05.02.02  Wirtschaftsprivatrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10758-5">
    <skos:broader rdf:resource="thsys/71094"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10758-5">
    <skos:broader rdf:resource="descriptor/10770-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10758-5">
    <skos:broader rdf:resource="descriptor/10812-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13348-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19496-6">
    <skos:narrower rdf:resource="descriptor/13453-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11612-0">
    <skos:related rdf:resource="descriptor/18822-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11612-0">
    <skos:related rdf:resource="descriptor/11628-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11612-0">
    <skos:related rdf:resource="descriptor/11626-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12770-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10121-3">
    <skos:altLabel xml:lang="en">Capital needs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16043-2">
    <skos:narrower rdf:resource="descriptor/13795-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16043-2">
    <skos:narrower rdf:resource="descriptor/12239-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17491-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15703-5">
    <skos:related rdf:resource="descriptor/10465-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15703-5">
    <skos:related rdf:resource="descriptor/12913-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15703-5">
    <skos:related rdf:resource="descriptor/10500-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16761-0">
    <skos:altLabel xml:lang="de">Zeitungswissenschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16761-0">
    <skos:altLabel xml:lang="de">Publizistikwissenschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16761-0">
    <skos:altLabel xml:lang="de">Kommunikationstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16761-0">
    <skos:altLabel xml:lang="de">Kommunikationsforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11810-3">
    <skos:broader rdf:resource="thsys/73371"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11810-3">
    <skos:broader rdf:resource="thsys/71110"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11810-3">
    <skos:broader rdf:resource="thsys/71012"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11810-3">
    <skos:broader rdf:resource="thsys/70539"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11810-3">
    <skos:broader rdf:resource="descriptor/10981-2"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id136">
    <rdfs:seeAlso rdf:resource="descriptor/12314-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19627-3">
    <skos:prefLabel xml:lang="en">Animal protection</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19627-3">
    <skos:prefLabel xml:lang="de">Tierschutz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14749-0">
    <skos:prefLabel xml:lang="en">Printer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14749-0">
    <skos:prefLabel xml:lang="de">Drucker</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10811-6">
    <skos:prefLabel xml:lang="en">International capital mobility</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10811-6">
    <skos:prefLabel xml:lang="de">Internationale Kapitalmobilität</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12730-3">
    <skos:related rdf:resource="descriptor/21041-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12730-3">
    <skos:related rdf:resource="descriptor/12738-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12730-3">
    <skos:related rdf:resource="descriptor/12681-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12730-3">
    <skos:related rdf:resource="descriptor/12736-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12730-3">
    <skos:related rdf:resource="descriptor/12693-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12730-3">
    <skos:related rdf:resource="descriptor/12719-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14158-2">
    <skos:broader rdf:resource="thsys/73301"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14158-2">
    <skos:broader rdf:resource="thsys/70506"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14158-2">
    <skos:broader rdf:resource="thsys/70403"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14158-2">
    <skos:broader rdf:resource="descriptor/14160-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16615-4">
    <skos:narrower rdf:resource="descriptor/11389-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16111-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10644-6">
    <skos:prefLabel xml:lang="en">Optimal tariff</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10644-6">
    <skos:prefLabel xml:lang="de">Optimalzoll</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14632-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19819-1">
    <skos:prefLabel xml:lang="en">Customary law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19819-1">
    <skos:prefLabel xml:lang="de">Gewohnheitsrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14435-2">
    <skos:prefLabel xml:lang="en">Automotive part</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14435-2">
    <skos:prefLabel xml:lang="de">Kraftfahrzeugteil</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12091-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17623-0">
    <skos:broader rdf:resource="thsys/70253"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17623-0">
    <skos:broader rdf:resource="descriptor/17608-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15904-2">
    <skos:narrower rdf:resource="descriptor/11261-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14807-5">
    <zbwext:useInsteadNote rdf:nodeID="id290"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11718-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70258">
    <skos:prefLabel xml:lang="de">P.09.02  Landfahrzeuge</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15994-3">
    <skos:related rdf:resource="descriptor/19854-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15994-3">
    <skos:related rdf:resource="descriptor/19323-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15994-3">
    <skos:related rdf:resource="descriptor/15709-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15994-3">
    <skos:related rdf:resource="descriptor/16486-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15994-3">
    <skos:related rdf:resource="descriptor/15954-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15994-3">
    <skos:related rdf:resource="descriptor/19649-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12976-6">
    <skos:altLabel xml:lang="de">Zwischenlagerung radioaktiver Abfälle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12976-6">
    <skos:altLabel xml:lang="de">Entsorgung von Brennelementen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12976-6">
    <skos:altLabel xml:lang="de">Entsorgung radioaktiver Abfälle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12976-6">
    <skos:altLabel xml:lang="de">Endlagerung radioaktiver Abfälle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12976-6">
    <skos:altLabel xml:lang="de">Atommüllbeseitigung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11503-5">
    <skos:broader rdf:resource="thsys/70355"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11503-5">
    <skos:broader rdf:resource="descriptor/11506-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14306-6">
    <skos:broader rdf:resource="thsys/70134"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14306-6">
    <skos:broader rdf:resource="descriptor/14296-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15220-4">
    <skos:altLabel xml:lang="en">Markov chain</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15220-4">
    <skos:altLabel xml:lang="de">Markovscher Prozeß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15220-4">
    <skos:altLabel xml:lang="de">Markovsche Kette</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15220-4">
    <skos:altLabel xml:lang="de">Markov-Kette</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15220-4">
    <skos:altLabel xml:lang="de">Markoffscher Prozess</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15220-4">
    <skos:altLabel xml:lang="de">Markoffscher Prozeß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15220-4">
    <skos:altLabel xml:lang="de">Markoff-Kette</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13533-0">
    <zbwext:useInsteadNote rdf:nodeID="id128"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71043">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10814-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15498-1">
    <skos:prefLabel xml:lang="en">Heuristic method</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15498-1">
    <skos:prefLabel xml:lang="de">Heuristisches Verfahren</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10310-0">
    <skos:altLabel xml:lang="de">Einproduktunternehmen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10310-0">
    <skos:altLabel xml:lang="de">Einfache Produktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18550-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14420-1">
    <skos:altLabel xml:lang="de">Helikopter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17942-5">
    <skos:prefLabel xml:lang="en">New Zealand</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17942-5">
    <skos:prefLabel xml:lang="de">Neuseeland</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10738-4">
    <skos:altLabel xml:lang="de">Franc</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13894-3">
    <skos:broader rdf:resource="thsys/71056"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13894-3">
    <skos:broader rdf:resource="thsys/70991"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12452-5">
    <skos:narrower rdf:resource="descriptor/12451-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19744-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13239-0">
    <skos:narrower rdf:resource="descriptor/18515-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14593-5">
    <skos:prefLabel xml:lang="en">Jewellery</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14593-5">
    <skos:prefLabel xml:lang="de">Schmuck</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19318-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19592-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12631-5">
    <skos:broader rdf:resource="thsys/71050"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11913-0">
    <skos:broader rdf:resource="thsys/71067"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10085-2">
    <skos:related rdf:resource="descriptor/10077-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10573-3">
    <skos:altLabel xml:lang="de">Finanzielle Entwicklungshilfe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10573-3">
    <skos:altLabel xml:lang="de">Finanzhilfe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14082-2">
    <skos:broader rdf:resource="thsys/70341"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14082-2">
    <skos:broader rdf:resource="descriptor/14073-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16389-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15609-4">
    <zbwext:useInsteadNote rdf:nodeID="id143"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19721-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14151-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19429-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73366">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71091">
    <skos:broader rdf:resource="thsys/70179"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17948-0">
    <skos:altLabel xml:lang="de">Polynesien</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17948-0">
    <skos:altLabel xml:lang="de">Ozeanien</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13369-1">
    <skos:broader rdf:resource="thsys/73296"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13369-1">
    <skos:broader rdf:resource="descriptor/18449-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16203-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70082">
    <skos:broader rdf:resource="thsys/v"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18822-3">
    <skos:related rdf:resource="descriptor/19686-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18822-3">
    <skos:related rdf:resource="descriptor/12064-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18822-3">
    <skos:related rdf:resource="descriptor/11722-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18822-3">
    <skos:related rdf:resource="descriptor/11612-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18822-3">
    <skos:related rdf:resource="descriptor/24783-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18822-3">
    <skos:related rdf:resource="descriptor/24705-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18822-3">
    <skos:related rdf:resource="descriptor/11626-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18822-3">
    <skos:related rdf:resource="descriptor/11043-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18822-3">
    <skos:related rdf:resource="descriptor/18775-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14092-6">
    <skos:prefLabel xml:lang="en">Potato</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14092-6">
    <skos:prefLabel xml:lang="de">Kartoffel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19902-0">
    <skos:prefLabel xml:lang="en">Employment relationship</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19902-0">
    <skos:prefLabel xml:lang="de">Arbeitsverhältnis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18480-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13825-1">
    <skos:broader rdf:resource="thsys/73391"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13825-1">
    <skos:broader rdf:resource="thsys/70892"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13825-1">
    <skos:broader rdf:resource="descriptor/16042-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19761-6">
    <skos:related rdf:resource="descriptor/19784-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19761-6">
    <skos:related rdf:resource="descriptor/12057-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19761-6">
    <skos:related rdf:resource="descriptor/11740-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19761-6">
    <skos:related rdf:resource="descriptor/19760-1"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id194">
    <rdfs:seeAlso rdf:resource="descriptor/10986-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19327-1">
    <skos:broader rdf:resource="thsys/71097"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19327-1">
    <skos:broader rdf:resource="thsys/70038"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19327-1">
    <skos:broader rdf:resource="descriptor/19151-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18359-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12356-1">
    <skos:altLabel xml:lang="de">Unternehmensvermögen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12356-1">
    <skos:altLabel xml:lang="de">Firmenvermögen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12356-1">
    <skos:altLabel xml:lang="de">Betriebliches Vermögen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14258-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12994-4">
    <skos:related rdf:resource="descriptor/13145-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12994-4">
    <skos:related rdf:resource="descriptor/13000-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12994-4">
    <skos:related rdf:resource="descriptor/14854-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18891-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12408-1">
    <skos:narrower rdf:resource="descriptor/18700-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12408-1">
    <skos:narrower rdf:resource="descriptor/12437-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12408-1">
    <skos:narrower rdf:resource="descriptor/12435-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12408-1">
    <skos:narrower rdf:resource="descriptor/12427-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12408-1">
    <skos:narrower rdf:resource="descriptor/12425-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12408-1">
    <skos:narrower rdf:resource="descriptor/12424-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12408-1">
    <skos:narrower rdf:resource="descriptor/12420-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12408-1">
    <skos:narrower rdf:resource="descriptor/24666-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12408-1">
    <skos:narrower rdf:resource="descriptor/12418-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12408-1">
    <skos:narrower rdf:resource="descriptor/19934-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12408-1">
    <skos:narrower rdf:resource="descriptor/12432-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12408-1">
    <skos:narrower rdf:resource="descriptor/19092-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12408-1">
    <skos:narrower rdf:resource="descriptor/19091-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12408-1">
    <skos:narrower rdf:resource="descriptor/12407-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17480-3">
    <skos:altLabel xml:lang="x-other">Republica Moldoveneasca</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17480-3">
    <skos:altLabel xml:lang="de">Moldavskaja SSR</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17480-3">
    <skos:altLabel xml:lang="de">Moldau (Staat)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70058">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19216-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12897-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10008-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19609-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12452-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70007">
    <skos:prefLabel xml:lang="en">N.02  Historical perspective</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70007">
    <skos:prefLabel xml:lang="de">N.02  Geschichte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18507-4">
    <skos:prefLabel xml:lang="en">Construction policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18507-4">
    <skos:prefLabel xml:lang="de">Baupolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71113">
    <skos:narrower rdf:resource="descriptor/11779-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71113">
    <skos:narrower rdf:resource="descriptor/10110-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71113">
    <skos:narrower rdf:resource="descriptor/11682-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71113">
    <skos:narrower rdf:resource="descriptor/11778-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71113">
    <skos:narrower rdf:resource="descriptor/11777-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71113">
    <skos:narrower rdf:resource="descriptor/10404-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71113">
    <skos:narrower rdf:resource="descriptor/11571-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71113">
    <skos:narrower rdf:resource="descriptor/19235-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71113">
    <skos:narrower rdf:resource="descriptor/11741-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71113">
    <skos:narrower rdf:resource="descriptor/11566-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71113">
    <skos:narrower rdf:resource="descriptor/11321-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71113">
    <skos:narrower rdf:resource="descriptor/11564-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71113">
    <skos:narrower rdf:resource="descriptor/11772-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71113">
    <skos:narrower rdf:resource="descriptor/11771-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71113">
    <skos:narrower rdf:resource="descriptor/11770-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71113">
    <skos:narrower rdf:resource="descriptor/10380-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71113">
    <skos:narrower rdf:resource="descriptor/11037-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71113">
    <skos:narrower rdf:resource="descriptor/11769-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11001-1">
    <skos:prefLabel xml:lang="en">Five-year plan</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11001-1">
    <skos:prefLabel xml:lang="de">Fünfjahrplan</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12824-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70254">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">P.10.03</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12648-2">
    <skos:narrower rdf:resource="descriptor/18960-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19847-3">
    <skos:scopeNote xml:lang="de">Für alle Regelungen zur Geheimhaltung des Verwaltungs- und Privatrechts. Amtsverschwiegenheit von Behörden. Verschwiegenheitspflicht verschiedener Berufe (z.B. Ärzte, Wirtschaftsprüfer, Hebammen).</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13032-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19803-2">
    <skos:broader rdf:resource="thsys/73340"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19803-2">
    <skos:broader rdf:resource="descriptor/18766-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13377-2">
    <skos:altLabel xml:lang="de">Privatdetektivinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13377-2">
    <skos:altLabel xml:lang="de">Privatdetektive</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13377-2">
    <skos:altLabel xml:lang="de">Informationsbüro</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13377-2">
    <skos:altLabel xml:lang="de">Detektei</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13377-2">
    <skos:altLabel xml:lang="de">Auskunftswesen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13377-2">
    <skos:altLabel xml:lang="de">Auskunftsdienst</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18575-1">
    <skos:prefLabel xml:lang="en">Railway policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18575-1">
    <skos:prefLabel xml:lang="de">Eisenbahnpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13269-5">
    <skos:altLabel xml:lang="en">Local area network</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13269-5">
    <skos:altLabel xml:lang="de">Lokales Netz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13269-5">
    <skos:altLabel xml:lang="de">Inhouse-Netz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13269-5">
    <skos:altLabel xml:lang="de">Ethernet</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14889-5">
    <skos:prefLabel xml:lang="en">Technical textiles</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14889-5">
    <skos:prefLabel xml:lang="de">Technische Textilien</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19013-3">
    <skos:broader rdf:resource="thsys/73346"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19013-3">
    <skos:broader rdf:resource="thsys/70017"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19013-3">
    <skos:broader rdf:resource="descriptor/16670-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10160-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14234-5">
    <skos:prefLabel xml:lang="en">Marble</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14234-5">
    <skos:prefLabel xml:lang="de">Marmor</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10334-0">
    <skos:broader rdf:resource="thsys/70233"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10334-0">
    <skos:broader rdf:resource="thsys/70180"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10334-0">
    <skos:broader rdf:resource="descriptor/10036-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13142-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24661-1">
    <skos:prefLabel xml:lang="en">Change management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24661-1">
    <skos:prefLabel xml:lang="de">Change Management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10410-3">
    <skos:related rdf:resource="descriptor/10680-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18416-0">
    <skos:altLabel xml:lang="de">Werft zur Schiffsreparatur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11235-4">
    <zbwext:useInsteadNote rdf:nodeID="id22"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11855-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16051-3">
    <skos:prefLabel xml:lang="en">Antitrust law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16051-3">
    <skos:prefLabel xml:lang="de">Kartellrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10597-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14377-4">
    <skos:prefLabel xml:lang="en">Wood-processing machinery</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14377-4">
    <skos:prefLabel xml:lang="de">Holzbearbeitungsmaschine</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12256-5">
    <skos:related rdf:resource="descriptor/12265-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12256-5">
    <skos:related rdf:resource="descriptor/12263-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12256-5">
    <skos:related rdf:resource="descriptor/18684-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18944-3">
    <skos:scopeNote xml:lang="de">Staatliche Maßnahmen mit Bezug auf die Informationswirtschaft.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12598-5">
    <skos:prefLabel xml:lang="en">Time rate</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12598-5">
    <skos:prefLabel xml:lang="de">Zeitlohn</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11451-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18747-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18831-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19019-5">
    <skos:altLabel xml:lang="de">Languste</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19019-5">
    <skos:altLabel xml:lang="de">Krustentier</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19019-5">
    <skos:altLabel xml:lang="de">Krabbe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19019-5">
    <skos:altLabel xml:lang="de">Hummer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19019-5">
    <skos:altLabel xml:lang="de">Garnele</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70941">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10181-6">
    <skos:prefLabel xml:lang="en">Consumer theory</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10181-6">
    <skos:prefLabel xml:lang="de">Konsumtheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12825-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70940">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73305">
    <skos:prefLabel xml:lang="de">P.10.04  Büromaschinen und Datenverarbeitungsgeräte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11038-6">
    <skos:prefLabel xml:lang="en">Theory of bureaucracy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11038-6">
    <skos:prefLabel xml:lang="de">Bürokratietheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70461">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">P.12.06</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16554-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11346-2">
    <skos:altLabel xml:lang="de">Tarifverhandlungen (Lohn)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70884">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12071-3">
    <skos:related rdf:resource="descriptor/10414-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12071-3">
    <skos:related rdf:resource="descriptor/12089-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12071-3">
    <skos:related rdf:resource="descriptor/10192-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12071-3">
    <skos:related rdf:resource="descriptor/12038-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11802-2">
    <skos:related rdf:resource="descriptor/18811-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11802-2">
    <skos:related rdf:resource="descriptor/19293-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11802-2">
    <skos:related rdf:resource="descriptor/19674-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11802-2">
    <skos:related rdf:resource="descriptor/18983-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16275-2">
    <skos:altLabel xml:lang="de">Staatspolitische Erziehung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16275-2">
    <skos:altLabel xml:lang="de">Staatsbürgerliche Erziehung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16275-2">
    <skos:altLabel xml:lang="de">Politische Erziehung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18822-3">
    <skos:altLabel xml:lang="de">ÖPP (Öffentlich-private Partnerschaft)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18822-3">
    <skos:altLabel xml:lang="de">Öffentlich-private Partnerschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12574-5">
    <skos:prefLabel xml:lang="en">Employee appraisal</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12574-5">
    <skos:prefLabel xml:lang="de">Personalbeurteilung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18870-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13731-3">
    <skos:scopeNote xml:lang="de">Rechtliche Regelungen des börslichen und außerbörslichen Geschehens auf den Finanzmärkten.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11362-4">
    <skos:related rdf:resource="descriptor/11370-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13574-0">
    <skos:broader rdf:resource="thsys/70941"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13574-0">
    <skos:broader rdf:resource="descriptor/13577-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11460-4">
    <skos:narrower rdf:resource="descriptor/10770-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14751-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10082-1">
    <skos:altLabel xml:lang="en">Quantity rationing theory</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10082-1">
    <skos:altLabel xml:lang="de">Theorie der Rationierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12308-5">
    <skos:prefLabel xml:lang="en">Operating result</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12308-5">
    <skos:prefLabel xml:lang="de">Betriebsergebnis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16357-0">
    <skos:narrower rdf:resource="descriptor/16311-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19916-3">
    <skos:related rdf:resource="descriptor/19112-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19916-3">
    <skos:related rdf:resource="descriptor/11224-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19916-3">
    <skos:related rdf:resource="descriptor/11354-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19916-3">
    <skos:related rdf:resource="descriptor/19516-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19916-3">
    <skos:related rdf:resource="descriptor/11353-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19916-3">
    <skos:related rdf:resource="descriptor/11355-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12700-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11544-5">
    <skos:altLabel xml:lang="de">Gemeindeeinnahmen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12224-4">
    <skos:broader rdf:resource="thsys/71044"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12224-4">
    <skos:broader rdf:resource="descriptor/12089-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11553-4">
    <skos:related rdf:resource="descriptor/18806-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14765-2">
    <skos:related rdf:resource="descriptor/18111-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14765-2">
    <skos:related rdf:resource="descriptor/19801-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12699-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10136-4">
    <skos:related rdf:resource="descriptor/19267-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10136-4">
    <skos:related rdf:resource="descriptor/11457-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12841-1">
    <skos:narrower rdf:resource="descriptor/12842-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15895-5">
    <skos:altLabel xml:lang="de">Tropische Staaten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15895-5">
    <skos:altLabel xml:lang="de">Tropische Länder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15895-5">
    <skos:altLabel xml:lang="de">Subtropen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15895-5">
    <skos:altLabel xml:lang="de">Savanne</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10256-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18959-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11266-0">
    <skos:broader rdf:resource="thsys/70051"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11266-0">
    <skos:broader rdf:resource="descriptor/10185-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11266-0">
    <skos:broader rdf:resource="descriptor/11183-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16564-2">
    <skos:broader rdf:resource="thsys/70012"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16564-2">
    <skos:broader rdf:resource="descriptor/16563-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70964">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18002-6">
    <skos:related rdf:resource="descriptor/19115-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18002-6">
    <skos:related rdf:resource="descriptor/12732-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18002-6">
    <skos:related rdf:resource="descriptor/13671-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16704-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15949-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19200-4">
    <skos:broader rdf:resource="thsys/70493"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19200-4">
    <skos:broader rdf:resource="thsys/70049"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19200-4">
    <skos:broader rdf:resource="descriptor/10181-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19200-4">
    <skos:broader rdf:resource="descriptor/10177-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11353-5">
    <skos:broader rdf:resource="thsys/70847"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11353-5">
    <skos:broader rdf:resource="thsys/70176"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11353-5">
    <skos:broader rdf:resource="thsys/70143"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19793-0">
    <skos:prefLabel xml:lang="en">Tropical disease</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19793-0">
    <skos:prefLabel xml:lang="de">Tropenkrankheit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18218-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17589-2">
    <skos:related rdf:resource="descriptor/19167-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17589-2">
    <skos:related rdf:resource="descriptor/17571-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17589-2">
    <skos:related rdf:resource="descriptor/19172-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17589-2">
    <skos:related rdf:resource="descriptor/17979-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15320-0">
    <skos:prefLabel xml:lang="en">Ranking method</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15320-0">
    <skos:prefLabel xml:lang="de">Ranking-Verfahren</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19295-2">
    <skos:prefLabel xml:lang="en">Coffee policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19295-2">
    <skos:prefLabel xml:lang="de">Kaffeepolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11274-1">
    <skos:altLabel xml:lang="en">High skilled labor</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13686-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id38">
    <rdf:value xml:lang="de">Für Auslandsaufträge des Dienstleistungssektors</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13400-5">
    <skos:related rdf:resource="descriptor/15768-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13400-5">
    <skos:related rdf:resource="descriptor/15764-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13400-5">
    <skos:related rdf:resource="descriptor/16364-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14515-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11416-0">
    <skos:related rdf:resource="descriptor/10764-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11416-0">
    <skos:related rdf:resource="descriptor/13462-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11416-0">
    <skos:related rdf:resource="descriptor/11042-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11416-0">
    <skos:related rdf:resource="descriptor/11481-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11416-0">
    <skos:related rdf:resource="descriptor/11415-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11416-0">
    <skos:related rdf:resource="descriptor/13691-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25881-3">
    <skos:prefLabel xml:lang="en">Work behaviour</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25881-3">
    <skos:prefLabel xml:lang="de">Arbeitsverhalten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15876-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13763-4">
    <skos:broader rdf:resource="thsys/70899"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14949-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19150-0">
    <skos:related rdf:resource="descriptor/15025-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19150-0">
    <skos:related rdf:resource="descriptor/10164-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19150-0">
    <skos:related rdf:resource="descriptor/15483-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19150-0">
    <skos:related rdf:resource="descriptor/11531-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19150-0">
    <skos:related rdf:resource="descriptor/15410-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19150-0">
    <skos:related rdf:resource="descriptor/12208-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19150-0">
    <skos:related rdf:resource="descriptor/15498-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19150-0">
    <skos:related rdf:resource="descriptor/15470-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19150-0">
    <skos:related rdf:resource="descriptor/15142-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71011">
    <skos:broader rdf:resource="thsys/71017"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10293-2">
    <skos:prefLabel xml:lang="en">Investment risk</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10293-2">
    <skos:prefLabel xml:lang="de">Investitionsrisiko</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18096-6">
    <skos:scopeNote xml:lang="de">Agrarsektor im weiteren Sinne.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18266-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13405-2">
    <skos:altLabel xml:lang="de">Wohlfahrtsverband</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13405-2">
    <skos:altLabel xml:lang="de">Sozial-karitative Organisation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13405-2">
    <skos:altLabel xml:lang="de">Freier Wohlfahrtsverband</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13405-2">
    <skos:altLabel xml:lang="de">Freie Wohlfahrtspflege</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/v">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13443-1">
    <skos:altLabel xml:lang="de">Landwirtschaftsgeographie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13443-1">
    <skos:altLabel xml:lang="de">Agrarraum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id61">
    <rdf:value xml:lang="de">Für unternehmensinterne Computernetze</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18548-4">
    <skos:prefLabel xml:lang="en">Rural retail trade</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18548-4">
    <skos:prefLabel xml:lang="de">Ländlicher Einzelhandel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14220-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12995-2">
    <skos:narrower rdf:resource="descriptor/13001-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16671-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14182-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10010-5">
    <skos:editorialNote xml:lang="de">Nur zur Postkoordination verwenden.</skos:editorialNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13416-4">
    <skos:related rdf:resource="descriptor/19637-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13416-4">
    <skos:related rdf:resource="descriptor/18914-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13416-4">
    <skos:related rdf:resource="descriptor/16540-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13416-4">
    <skos:related rdf:resource="descriptor/16539-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13416-4">
    <skos:related rdf:resource="descriptor/18739-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13416-4">
    <skos:related rdf:resource="descriptor/16548-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15555-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10642-3">
    <skos:broader rdf:resource="thsys/71099"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10642-3">
    <skos:broader rdf:resource="descriptor/10676-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15463-6">
    <skos:scopeNote xml:lang="de">Nur für die Ökonomische Theorie der Verhandlungen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18449-6">
    <skos:broader rdf:resource="thsys/73296"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18449-6">
    <skos:broader rdf:resource="thsys/70825"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18449-6">
    <skos:broader rdf:resource="descriptor/13367-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18449-6">
    <skos:broader rdf:resource="descriptor/13347-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70851">
    <rdfs:label xml:lang="de">Gewerkschaften</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11814-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16052-1">
    <skos:scopeNote xml:lang="de">Regelungen über die Zulassung und Ausübung eines Gewerbes sowie über die Einhaltung von Arbeitsschutzbestimmungen. Gilt nicht für Betriebe der Urproduktion und die freien Berufe.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10771-6">
    <skos:narrower rdf:resource="descriptor/19336-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16283-3">
    <skos:altLabel xml:lang="de">Reformpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71000">
    <skos:narrower rdf:resource="descriptor/13037-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71000">
    <skos:narrower rdf:resource="descriptor/13033-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71000">
    <skos:narrower rdf:resource="descriptor/13026-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71000">
    <skos:narrower rdf:resource="descriptor/13025-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70650">
    <rdfs:label xml:lang="en">Nuclear energy</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70650">
    <rdfs:label xml:lang="de">Kernenergie</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18613-5">
    <skos:prefLabel xml:lang="en">Dangerous good</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18613-5">
    <skos:prefLabel xml:lang="de">Gefahrgut</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10261-1">
    <skos:broader rdf:resource="thsys/70244"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10261-1">
    <skos:broader rdf:resource="thsys/70051"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10261-1">
    <skos:broader rdf:resource="descriptor/11270-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10847-6">
    <skos:broader rdf:resource="thsys/73337"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10847-6">
    <skos:broader rdf:resource="thsys/71091"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10847-6">
    <skos:broader rdf:resource="thsys/71090"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10847-6">
    <skos:broader rdf:resource="descriptor/16122-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17674-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10834-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14306-6">
    <skos:altLabel xml:lang="en">Platinum group metals</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14306-6">
    <skos:altLabel xml:lang="de">Ruthenium</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14306-6">
    <skos:altLabel xml:lang="de">Rhodium</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14306-6">
    <skos:altLabel xml:lang="de">Platinbeimetall</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14306-6">
    <skos:altLabel xml:lang="de">Platin</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14306-6">
    <skos:altLabel xml:lang="de">Palladium</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14306-6">
    <skos:altLabel xml:lang="de">Osmium</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14306-6">
    <skos:altLabel xml:lang="de">Iridium</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19250-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70458">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18693-2">
    <skos:prefLabel xml:lang="en">Interest rate futures</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18693-2">
    <skos:prefLabel xml:lang="de">Zinstermingeschäft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19144-2">
    <skos:altLabel xml:lang="en">Wide area network</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15842-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16024-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70041">
    <rdfs:label xml:lang="en">Marketing</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70041">
    <rdfs:label xml:lang="de">Marketing</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24621-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19023-0">
    <skos:altLabel xml:lang="de">Delphin</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19494-3">
    <skos:broader rdf:resource="thsys/70051"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19494-3">
    <skos:broader rdf:resource="descriptor/11264-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19001-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14990-2">
    <skos:broader rdf:resource="thsys/73299"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14990-2">
    <skos:broader rdf:resource="thsys/70341"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14990-2">
    <skos:broader rdf:resource="descriptor/14074-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14990-2">
    <skos:broader rdf:resource="descriptor/14989-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13149-1">
    <skos:broader rdf:resource="thsys/70987"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13149-1">
    <skos:broader rdf:resource="descriptor/13145-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11754-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19373-1">
    <skos:broader rdf:resource="thsys/70144"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13360-5">
    <skos:related rdf:resource="descriptor/12825-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13360-5">
    <skos:related rdf:resource="descriptor/16012-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13360-5">
    <skos:related rdf:resource="descriptor/18027-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13360-5">
    <skos:related rdf:resource="descriptor/16517-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13360-5">
    <skos:related rdf:resource="descriptor/18939-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13360-5">
    <skos:related rdf:resource="descriptor/13665-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16806-4">
    <skos:broader rdf:resource="thsys/70198"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16806-4">
    <skos:broader rdf:resource="descriptor/16808-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12717-2">
    <skos:broader rdf:resource="thsys/70310"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12717-2">
    <skos:broader rdf:resource="descriptor/12725-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12376-2">
    <skos:broader rdf:resource="thsys/71043"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12376-2">
    <skos:broader rdf:resource="descriptor/12354-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12177-1">
    <skos:altLabel xml:lang="en">Spin-off (Department)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12177-1">
    <skos:altLabel xml:lang="en">Corporate spinoff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12177-1">
    <skos:altLabel xml:lang="de">Unternehmensteilung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12177-1">
    <skos:altLabel xml:lang="de">Unternehmensaufspaltung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12177-1">
    <skos:altLabel xml:lang="de">Doppelgesellschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12177-1">
    <skos:altLabel xml:lang="de">Betriebsausgliederung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12177-1">
    <skos:altLabel xml:lang="de">Ausgliederung eines Betriebsteiles</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13808-1">
    <skos:scopeNote xml:lang="de">Versicherung betrieblicher Risiken</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19844-2">
    <skos:prefLabel xml:lang="en">Labour-relations court decision</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19844-2">
    <skos:prefLabel xml:lang="de">Arbeitsrechtsprechung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17911-2">
    <skos:altLabel xml:lang="de">Argentinische Republik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10043-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18104-5">
    <skos:broader rdf:resource="thsys/70368"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10326-6">
    <skos:broader rdf:resource="thsys/70270"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10326-6">
    <skos:broader rdf:resource="thsys/70233"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19434-0">
    <skos:prefLabel xml:lang="en">Antitrust enforcement</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19434-0">
    <skos:prefLabel xml:lang="de">Kartellverfahren</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13573-2">
    <skos:prefLabel xml:lang="en">Transport research</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13573-2">
    <skos:prefLabel xml:lang="de">Verkehrswissenschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11007-3">
    <skos:prefLabel xml:lang="en">Indicative planning</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11007-3">
    <skos:prefLabel xml:lang="de">Indikative Planung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11186-5">
    <skos:related rdf:resource="descriptor/12667-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11186-5">
    <skos:related rdf:resource="descriptor/10248-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11186-5">
    <skos:related rdf:resource="descriptor/12621-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17947-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18864-1">
    <skos:related rdf:resource="descriptor/16543-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18864-1">
    <skos:related rdf:resource="descriptor/15925-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18864-1">
    <skos:related rdf:resource="descriptor/13312-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18864-1">
    <skos:related rdf:resource="descriptor/16477-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18864-1">
    <skos:related rdf:resource="descriptor/18911-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18864-1">
    <skos:related rdf:resource="descriptor/19759-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18864-1">
    <skos:related rdf:resource="descriptor/18857-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12172-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71074">
    <skos:narrower rdf:resource="descriptor/10710-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71074">
    <skos:narrower rdf:resource="descriptor/11683-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71074">
    <skos:narrower rdf:resource="descriptor/19425-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71074">
    <skos:narrower rdf:resource="descriptor/19424-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71074">
    <skos:narrower rdf:resource="descriptor/11682-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71074">
    <skos:narrower rdf:resource="descriptor/11681-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71074">
    <skos:narrower rdf:resource="descriptor/19423-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71074">
    <skos:narrower rdf:resource="descriptor/11680-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71074">
    <skos:narrower rdf:resource="descriptor/12868-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71074">
    <skos:narrower rdf:resource="descriptor/11678-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71074">
    <skos:narrower rdf:resource="descriptor/11677-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71074">
    <skos:narrower rdf:resource="descriptor/11676-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71074">
    <skos:narrower rdf:resource="descriptor/11547-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71074">
    <skos:narrower rdf:resource="descriptor/19422-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71074">
    <skos:narrower rdf:resource="descriptor/11675-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71074">
    <skos:narrower rdf:resource="descriptor/19407-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71074">
    <skos:narrower rdf:resource="descriptor/18072-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71074">
    <skos:narrower rdf:resource="descriptor/11673-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71074">
    <skos:narrower rdf:resource="descriptor/11671-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71074">
    <skos:narrower rdf:resource="descriptor/11670-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71074">
    <skos:narrower rdf:resource="descriptor/11669-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71074">
    <skos:narrower rdf:resource="descriptor/11667-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71074">
    <skos:narrower rdf:resource="descriptor/11666-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71074">
    <skos:narrower rdf:resource="descriptor/19420-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71074">
    <skos:narrower rdf:resource="descriptor/11665-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71074">
    <skos:narrower rdf:resource="descriptor/11664-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71074">
    <skos:narrower rdf:resource="descriptor/11663-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71074">
    <skos:narrower rdf:resource="descriptor/11661-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71074">
    <skos:narrower rdf:resource="descriptor/11659-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71074">
    <skos:narrower rdf:resource="descriptor/11658-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71074">
    <skos:narrower rdf:resource="descriptor/11657-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71074">
    <skos:narrower rdf:resource="descriptor/19421-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71074">
    <skos:narrower rdf:resource="descriptor/11653-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71074">
    <skos:narrower rdf:resource="descriptor/19419-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71074">
    <skos:narrower rdf:resource="descriptor/19415-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71074">
    <skos:narrower rdf:resource="descriptor/19414-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71074">
    <skos:narrower rdf:resource="descriptor/11652-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71074">
    <skos:narrower rdf:resource="descriptor/11651-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71074">
    <skos:narrower rdf:resource="descriptor/18376-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71074">
    <skos:narrower rdf:resource="descriptor/11650-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71074">
    <skos:narrower rdf:resource="descriptor/11649-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71074">
    <skos:narrower rdf:resource="descriptor/11648-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71074">
    <skos:narrower rdf:resource="descriptor/19418-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71074">
    <skos:narrower rdf:resource="descriptor/19417-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71074">
    <skos:narrower rdf:resource="descriptor/19412-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71074">
    <skos:narrower rdf:resource="descriptor/11646-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10040-3">
    <skos:prefLabel xml:lang="en">Allocation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10040-3">
    <skos:prefLabel xml:lang="de">Allokation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18367-1">
    <skos:altLabel xml:lang="de">Kernenergieverzicht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18367-1">
    <skos:altLabel xml:lang="de">Kernenergieförderung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18367-1">
    <skos:altLabel xml:lang="de">Kernenergieausstieg</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18367-1">
    <skos:altLabel xml:lang="de">Atomenergiepolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16974-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17624-5">
    <skos:broader rdf:resource="thsys/70253"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17624-5">
    <skos:broader rdf:resource="descriptor/17608-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17624-5">
    <skos:broader rdf:resource="descriptor/17978-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10884-0">
    <skos:altLabel xml:lang="de">Rohstoffbörse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10884-0">
    <skos:altLabel xml:lang="de">Produktbörse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10434-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13451-2">
    <skos:related rdf:resource="descriptor/11151-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13451-2">
    <skos:related rdf:resource="descriptor/15930-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13451-2">
    <skos:related rdf:resource="descriptor/18313-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13451-2">
    <skos:related rdf:resource="descriptor/16250-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13451-2">
    <skos:related rdf:resource="descriptor/12020-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13451-2">
    <skos:related rdf:resource="descriptor/11803-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13451-2">
    <skos:related rdf:resource="descriptor/13445-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13451-2">
    <skos:related rdf:resource="descriptor/13426-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13451-2">
    <skos:related rdf:resource="descriptor/13443-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13451-2">
    <skos:related rdf:resource="descriptor/13429-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19458-0">
    <skos:prefLabel xml:lang="en">Village development</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19458-0">
    <skos:prefLabel xml:lang="de">Dorfentwicklung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18951-6">
    <skos:altLabel xml:lang="de">Zeitschriftenbestand</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18951-6">
    <skos:altLabel xml:lang="de">Magazinbestand in Bibliotheken</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16621-2">
    <skos:scopeNote xml:lang="de">Nur für Gewaltausübung im physischen Sinne.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19277-4">
    <skos:prefLabel xml:lang="en">Euronote</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19277-4">
    <skos:prefLabel xml:lang="de">Euronote</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18171-4">
    <skos:broader rdf:resource="thsys/70269"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18171-4">
    <skos:broader rdf:resource="descriptor/15054-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16484-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19017-2">
    <skos:related rdf:resource="descriptor/14723-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71057">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11374-4">
    <skos:narrower rdf:resource="descriptor/11372-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11374-4">
    <skos:narrower rdf:resource="descriptor/11358-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13305-6">
    <skos:altLabel xml:lang="de">Wohnungsunternehmen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13305-6">
    <skos:altLabel xml:lang="de">Wohnungssektor</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13305-6">
    <skos:altLabel xml:lang="de">Unternehmerische Wohnungswirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19115-2">
    <skos:prefLabel xml:lang="en">Computer-assisted inventory management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19115-2">
    <skos:prefLabel xml:lang="de">Computergestützte Lagerverwaltung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19860-4">
    <skos:prefLabel xml:lang="en">Crime against humanity</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19860-4">
    <skos:prefLabel xml:lang="de">Verbrechen gegen die Menschlichkeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14715-3">
    <skos:prefLabel xml:lang="en">PVC</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14715-3">
    <skos:prefLabel xml:lang="de">PVC</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70306">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13190-4">
    <skos:altLabel xml:lang="de">Shredderwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13190-4">
    <skos:altLabel xml:lang="de">Schrott-Recycling-Wirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13190-4">
    <skos:altLabel xml:lang="de">Schrotthandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13190-4">
    <skos:altLabel xml:lang="de">Altmetallhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71088">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.07.07.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10993-2">
    <skos:related rdf:resource="descriptor/10991-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10993-2">
    <skos:related rdf:resource="descriptor/10395-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10993-2">
    <skos:related rdf:resource="descriptor/10192-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19327-1">
    <skos:scopeNote xml:lang="de">Nationale und internationale Wirtschaftskonferenzen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11822-3">
    <skos:altLabel xml:lang="de">Industrie- und Handelskammer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17748-1">
    <skos:altLabel xml:lang="de">Sozialistische Libysch-Arabische Volks-Dschamahirija</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17748-1">
    <skos:altLabel xml:lang="de">Arabische Republik Libyen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10190-5">
    <skos:broader rdf:resource="thsys/70481"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10190-5">
    <skos:broader rdf:resource="descriptor/10144-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70960">
    <skos:prefLabel xml:lang="de">W.10.01.02  Lebensmittel- und Landhandel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12036-5">
    <skos:narrower rdf:resource="descriptor/13611-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12036-5">
    <skos:narrower rdf:resource="descriptor/19164-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12036-5">
    <skos:narrower rdf:resource="descriptor/12665-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12036-5">
    <skos:narrower rdf:resource="descriptor/18121-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12036-5">
    <skos:narrower rdf:resource="descriptor/18467-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12036-5">
    <skos:narrower rdf:resource="descriptor/18511-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12036-5">
    <skos:narrower rdf:resource="descriptor/13469-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12036-5">
    <skos:narrower rdf:resource="descriptor/18354-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12036-5">
    <skos:narrower rdf:resource="descriptor/19428-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12036-5">
    <skos:narrower rdf:resource="descriptor/16802-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12036-5">
    <skos:narrower rdf:resource="descriptor/12777-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18791-2">
    <skos:prefLabel xml:lang="en">Receivership</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18791-2">
    <skos:prefLabel xml:lang="de">Konkursverwaltung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18620-1">
    <skos:prefLabel xml:lang="en">Distribution logistics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18620-1">
    <skos:prefLabel xml:lang="de">Transportlogistik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19299-1">
    <skos:prefLabel xml:lang="en">Geo economics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19299-1">
    <skos:prefLabel xml:lang="de">Geoökonomik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18098-2">
    <skos:scopeNote xml:lang="de">Direkte Produktwerbung im Rahmen eines Spiel- oder Fernsehfilms.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10498-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70151">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13560-4">
    <skos:broader rdf:resource="thsys/70938"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13560-4">
    <skos:broader rdf:resource="descriptor/13514-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11657-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11151-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11158-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15793-6">
    <skos:prefLabel xml:lang="en">Perception</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15793-6">
    <skos:prefLabel xml:lang="de">Wahrnehmung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19851-5">
    <skos:altLabel xml:lang="de">Internationaler Strafgerichtshof</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19851-5">
    <skos:altLabel xml:lang="de">Internationaler Gerichtshof</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16455-0">
    <skos:prefLabel xml:lang="en">Multinational force</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16455-0">
    <skos:prefLabel xml:lang="de">Internationale Truppe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18904-1">
    <skos:broader rdf:resource="thsys/73335"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18904-1">
    <skos:broader rdf:resource="thsys/73331"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18904-1">
    <skos:broader rdf:resource="thsys/70835"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18904-1">
    <skos:broader rdf:resource="descriptor/16159-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18904-1">
    <skos:broader rdf:resource="descriptor/16182-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13639-5">
    <skos:altLabel xml:lang="de">Tourismuspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13639-5">
    <skos:altLabel xml:lang="de">Reiseverkehrspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13639-5">
    <skos:altLabel xml:lang="de">Fremdenverkehrsplanung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13639-5">
    <skos:altLabel xml:lang="de">Fremdenverkehrsleitbild</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13639-5">
    <skos:altLabel xml:lang="de">Fremdenverkehrsförderung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13639-5">
    <skos:altLabel xml:lang="de">Förderung des Tourismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18239-3">
    <skos:altLabel xml:lang="de">Seltene Erden</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18239-3">
    <skos:altLabel xml:lang="de">Scandium</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18239-3">
    <skos:altLabel xml:lang="de">Metalle der seltenen Erden</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11389-5">
    <skos:broader rdf:resource="thsys/73355"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11389-5">
    <skos:broader rdf:resource="descriptor/16615-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11389-5">
    <skos:broader rdf:resource="descriptor/11184-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18247-4">
    <skos:altLabel xml:lang="de">Fähre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16729-3">
    <skos:prefLabel xml:lang="en">Theory-practice divide</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16729-3">
    <skos:prefLabel xml:lang="de">Theorie-Praxis-Problem</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12340-2">
    <skos:altLabel xml:lang="en">Uniform cost accounting</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12340-2">
    <skos:altLabel xml:lang="de">Kontenplan</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12340-2">
    <skos:altLabel xml:lang="de">Einheitskontenrahmen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10210-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16964-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13473-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10016-0">
    <skos:narrower rdf:resource="descriptor/18559-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10016-0">
    <skos:narrower rdf:resource="descriptor/12801-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10016-0">
    <skos:narrower rdf:resource="descriptor/12800-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10016-0">
    <skos:narrower rdf:resource="descriptor/10007-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10016-0">
    <skos:narrower rdf:resource="descriptor/20789-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10016-0">
    <skos:narrower rdf:resource="descriptor/14054-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12136-1">
    <skos:prefLabel xml:lang="en">Organizational design</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12136-1">
    <skos:prefLabel xml:lang="de">Aufbauorganisation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16245-4">
    <skos:broader rdf:resource="thsys/73382"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16245-4">
    <skos:broader rdf:resource="descriptor/16292-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15571-3">
    <skos:broader rdf:resource="thsys/71035"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15571-3">
    <skos:broader rdf:resource="thsys/70300"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15571-3">
    <skos:broader rdf:resource="descriptor/15055-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12613-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15943-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71110">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17985-1">
    <skos:prefLabel xml:lang="en">OPEC countries</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17985-1">
    <skos:prefLabel xml:lang="de">OPEC-Staaten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10446-3">
    <skos:altLabel xml:lang="en">Induced technical progress</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10446-3">
    <skos:altLabel xml:lang="en">Induced innovation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10446-3">
    <skos:altLabel xml:lang="en">Endogenous technical change</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10446-3">
    <skos:altLabel xml:lang="de">Induzierter technischer Fortschritt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15836-0">
    <skos:related rdf:resource="descriptor/10032-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15836-0">
    <skos:related rdf:resource="descriptor/16687-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15836-0">
    <skos:related rdf:resource="descriptor/16566-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15836-0">
    <skos:related rdf:resource="descriptor/15861-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15836-0">
    <skos:related rdf:resource="descriptor/20497-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16287-2">
    <skos:related rdf:resource="descriptor/16353-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16287-2">
    <skos:related rdf:resource="descriptor/16269-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16287-2">
    <skos:related rdf:resource="descriptor/16277-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16287-2">
    <skos:related rdf:resource="descriptor/16273-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16287-2">
    <skos:related rdf:resource="descriptor/16226-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16287-2">
    <skos:related rdf:resource="descriptor/13661-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16287-2">
    <skos:related rdf:resource="descriptor/15821-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16287-2">
    <skos:related rdf:resource="descriptor/10984-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16287-2">
    <skos:related rdf:resource="descriptor/10968-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16287-2">
    <skos:related rdf:resource="descriptor/15472-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16287-2">
    <skos:related rdf:resource="descriptor/16633-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70131">
    <rdfs:label xml:lang="en">Southern Europe</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70131">
    <rdfs:label xml:lang="de">Südeuropa</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18978-0">
    <skos:related rdf:resource="descriptor/21306-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12261-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18285-3">
    <skos:prefLabel xml:lang="en">Palestine</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18285-3">
    <skos:prefLabel xml:lang="de">Palästina</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15993-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11769-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19059-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18583-2">
    <skos:broader rdf:resource="thsys/70937"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18583-2">
    <skos:broader rdf:resource="thsys/70926"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18583-2">
    <skos:broader rdf:resource="descriptor/11834-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18583-2">
    <skos:broader rdf:resource="descriptor/18579-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18583-2">
    <skos:broader rdf:resource="descriptor/11543-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11676-2">
    <skos:altLabel xml:lang="de">Substanzbesteuerung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11676-2">
    <skos:altLabel xml:lang="de">Sollsteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11676-2">
    <skos:altLabel xml:lang="de">Bestandsteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11676-2">
    <skos:altLabel xml:lang="de">Bestandssteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19235-6">
    <skos:scopeNote xml:lang="de">Überführung von Unternehmen an einheimische Kapitalgeber; staatlich vorgeschriebene Beteiligung heimischer Unternehmer; Förderung des heimischen Unternehmertums in Entwicklungsländern vor ausländischen Unternehmen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12640-4">
    <skos:prefLabel xml:lang="en">Procurement</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12640-4">
    <skos:prefLabel xml:lang="de">Beschaffung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15751-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70233">
    <skos:prefLabel xml:lang="en">V.03.02  Aggregate production</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70233">
    <skos:prefLabel xml:lang="de">V.03.02  Gesamtwirtschaftliche Produktion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19324-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13474-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19087-2">
    <skos:altLabel xml:lang="en">FR Yugoslavia</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19087-2">
    <skos:altLabel xml:lang="de">Jugoslawien (Bundesrepublik)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19087-2">
    <skos:altLabel xml:lang="de">Föderative Republik Jugoslawien</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19087-2">
    <skos:altLabel xml:lang="de">Bundesrepublik Jugoslawien</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19087-2">
    <skos:altLabel xml:lang="de">BR Jugoslawien</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18255-5">
    <skos:related rdf:resource="descriptor/14293-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12976-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70180">
    <skos:prefLabel xml:lang="en">V.02  Microeconomics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70180">
    <skos:prefLabel xml:lang="de">V.02  Mikroökonomik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19855-4">
    <skos:altLabel xml:lang="de">Verrat von Staatsgeheimnissen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19855-4">
    <skos:altLabel xml:lang="de">Landesverrat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14665-6">
    <skos:altLabel xml:lang="de">Methylalkohol</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14665-6">
    <skos:altLabel xml:lang="de">Methanol</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14665-6">
    <skos:altLabel xml:lang="de">Glycerin</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14665-6">
    <skos:altLabel xml:lang="de">Ethylalkohol</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14665-6">
    <skos:altLabel xml:lang="de">Ethanol</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14665-6">
    <skos:altLabel xml:lang="de">Butylalkohol</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14665-6">
    <skos:altLabel xml:lang="de">Butanol</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14665-6">
    <skos:altLabel xml:lang="de">Äthylalkohol</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13218-1">
    <skos:related rdf:resource="descriptor/18300-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13218-1">
    <skos:related rdf:resource="descriptor/18550-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13218-1">
    <skos:related rdf:resource="descriptor/13940-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16202-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16708-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16160-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15911-5">
    <skos:prefLabel xml:lang="en">Population policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15911-5">
    <skos:prefLabel xml:lang="de">Bevölkerungspolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70979">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10016-0">
    <skos:scopeNote xml:lang="de">Waren/Produkte im Sinne der Betriebswirtschaftslehre und der Wirtschaftspraxis.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12306-2">
    <skos:narrower rdf:resource="descriptor/19236-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12306-2">
    <skos:narrower rdf:resource="descriptor/11554-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18803-0">
    <skos:prefLabel xml:lang="en">Public Facility</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18803-0">
    <skos:prefLabel xml:lang="de">Öffentliche Einrichtung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13674-3">
    <skos:narrower rdf:resource="descriptor/13684-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13674-3">
    <skos:narrower rdf:resource="descriptor/13679-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13674-3">
    <skos:narrower rdf:resource="descriptor/13680-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11049-1">
    <skos:broader rdf:resource="thsys/73366"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11049-1">
    <skos:broader rdf:resource="thsys/70039"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15746-1">
    <skos:related rdf:resource="descriptor/15662-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15746-1">
    <skos:related rdf:resource="descriptor/11866-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15746-1">
    <skos:related rdf:resource="descriptor/15734-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18904-1">
    <skos:prefLabel xml:lang="en">EU social legislation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18904-1">
    <skos:prefLabel xml:lang="de">EU-Sozialrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18927-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12262-3">
    <skos:broader rdf:resource="thsys/70879"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12262-3">
    <skos:broader rdf:resource="thsys/70183"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12262-3">
    <skos:broader rdf:resource="descriptor/25742-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12158-5">
    <skos:prefLabel xml:lang="en">Strategic business unit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12158-5">
    <skos:prefLabel xml:lang="de">Strategisches Geschäftsfeld</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14436-0">
    <skos:altLabel xml:lang="en">Lorry</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14436-0">
    <skos:altLabel xml:lang="de">Straßenzugmaschine</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14436-0">
    <skos:altLabel xml:lang="de">Sattelzugmaschine</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14436-0">
    <skos:altLabel xml:lang="de">Lkw</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14436-0">
    <skos:altLabel xml:lang="de">Lastwagen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16169-1">
    <skos:broader rdf:resource="thsys/73389"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16169-1">
    <skos:broader rdf:resource="thsys/73333"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16169-1">
    <skos:broader rdf:resource="thsys/70934"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16169-1">
    <skos:broader rdf:resource="descriptor/13611-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16169-1">
    <skos:broader rdf:resource="descriptor/16070-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18454-6">
    <skos:scopeNote xml:lang="de">Herstellung und Vertrieb von Büroausstattungen und damit zusammenhängende Dienstleistungen</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16229-2">
    <skos:related rdf:resource="descriptor/16214-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12901-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18766-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11729-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#Collection">
    <rdfs:comment xml:lang="en">Labelled collections can be used with collectable semantic relation
      properties e.g. skos:narrower, where you would like a set of concepts to be displayed under a
      'node label' in the hierarchy.</rdfs:comment>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11793-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11559-6">
    <skos:altLabel xml:lang="de">Verwaltungspersonal im öffentlichen Dienst</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11559-6">
    <skos:altLabel xml:lang="de">Personal im öffentlichen Dienst</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18765-3">
    <skos:related rdf:resource="descriptor/19803-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18288-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14831-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71095">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.07.03</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21061-5">
    <skos:scopeNote xml:lang="de">Hier verstanden im Sinne des Steuerrechts. Umfasst bewegliches und unbewegliches Vermögen, aber auch nicht-gegenständliche Werte (positiv oder negativ), die bilanzierungsfähig sind.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19489-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10032-2">
    <skos:narrower rdf:resource="descriptor/13865-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10032-2">
    <skos:narrower rdf:resource="descriptor/10062-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10032-2">
    <skos:narrower rdf:resource="descriptor/10037-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10032-2">
    <skos:narrower rdf:resource="descriptor/18817-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10032-2">
    <skos:narrower rdf:resource="descriptor/18741-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10032-2">
    <skos:narrower rdf:resource="descriptor/13588-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10032-2">
    <skos:narrower rdf:resource="descriptor/15767-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10032-2">
    <skos:narrower rdf:resource="descriptor/18955-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10032-2">
    <skos:narrower rdf:resource="descriptor/16562-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10032-2">
    <skos:narrower rdf:resource="descriptor/11912-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10032-2">
    <skos:narrower rdf:resource="descriptor/19213-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10032-2">
    <skos:narrower rdf:resource="descriptor/10975-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10032-2">
    <skos:narrower rdf:resource="descriptor/11066-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10032-2">
    <skos:narrower rdf:resource="descriptor/19650-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10032-2">
    <skos:narrower rdf:resource="descriptor/18928-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10032-2">
    <skos:narrower rdf:resource="descriptor/18921-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10032-2">
    <skos:narrower rdf:resource="descriptor/11161-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10032-2">
    <skos:narrower rdf:resource="descriptor/13460-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10032-2">
    <skos:narrower rdf:resource="descriptor/11157-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10032-2">
    <skos:narrower rdf:resource="descriptor/18917-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10032-2">
    <skos:narrower rdf:resource="descriptor/11526-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10032-2">
    <skos:narrower rdf:resource="descriptor/19205-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10032-2">
    <skos:narrower rdf:resource="descriptor/10173-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10032-2">
    <skos:narrower rdf:resource="descriptor/19204-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10032-2">
    <skos:narrower rdf:resource="descriptor/18350-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10032-2">
    <skos:narrower rdf:resource="descriptor/15705-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10032-2">
    <skos:narrower rdf:resource="descriptor/19203-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10032-2">
    <skos:narrower rdf:resource="descriptor/11407-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10032-2">
    <skos:narrower rdf:resource="descriptor/12041-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10032-2">
    <skos:narrower rdf:resource="descriptor/18318-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20289-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13033-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14632-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18434-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12234-1">
    <skos:narrower rdf:resource="descriptor/13745-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12234-1">
    <skos:narrower rdf:resource="descriptor/12263-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12234-1">
    <skos:narrower rdf:resource="descriptor/13736-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12234-1">
    <skos:narrower rdf:resource="descriptor/12256-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12234-1">
    <skos:narrower rdf:resource="descriptor/11611-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12234-1">
    <skos:narrower rdf:resource="descriptor/10804-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12234-1">
    <skos:narrower rdf:resource="descriptor/12247-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12234-1">
    <skos:narrower rdf:resource="descriptor/18684-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12234-1">
    <skos:narrower rdf:resource="descriptor/18685-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15362-5">
    <skos:broader rdf:resource="thsys/72186"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15362-5">
    <skos:broader rdf:resource="descriptor/15340-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15064-6">
    <skos:narrower rdf:resource="descriptor/15396-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15064-6">
    <skos:narrower rdf:resource="descriptor/15060-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15064-6">
    <skos:narrower rdf:resource="descriptor/15211-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15064-6">
    <skos:narrower rdf:resource="descriptor/15814-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15064-6">
    <skos:narrower rdf:resource="descriptor/10266-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15064-6">
    <skos:narrower rdf:resource="descriptor/15063-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15064-6">
    <skos:narrower rdf:resource="descriptor/15062-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15064-6">
    <skos:narrower rdf:resource="descriptor/19568-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15064-6">
    <skos:narrower rdf:resource="descriptor/15126-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15064-6">
    <skos:narrower rdf:resource="descriptor/19150-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15064-6">
    <skos:narrower rdf:resource="descriptor/15072-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15064-6">
    <skos:narrower rdf:resource="descriptor/15373-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15064-6">
    <skos:narrower rdf:resource="descriptor/15315-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15064-6">
    <skos:narrower rdf:resource="descriptor/15372-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15064-6">
    <skos:narrower rdf:resource="descriptor/15405-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15064-6">
    <skos:narrower rdf:resource="descriptor/15258-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15064-6">
    <skos:narrower rdf:resource="descriptor/15097-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15064-6">
    <skos:narrower rdf:resource="descriptor/15142-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15064-6">
    <skos:narrower rdf:resource="descriptor/10229-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14023-4">
    <skos:scopeNote xml:lang="de">Beinhaltet Möbel, technische und Geräteausstattung, Software, Papier- und Schreibwaren sowie sonstiges Büromaterial.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16666-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73358">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12283-2">
    <skos:prefLabel xml:lang="en">Investment group</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12283-2">
    <skos:prefLabel xml:lang="de">Kapitalbeteiligungsgesellschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19739-6">
    <skos:broader rdf:resource="thsys/73388"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19739-6">
    <skos:broader rdf:resource="descriptor/15640-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11998-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11235-4">
    <skos:altLabel xml:lang="en">Labour policy</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11235-4">
    <skos:altLabel xml:lang="en">Labor policy</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11235-4">
    <skos:altLabel xml:lang="en">Labor market policy</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11235-4">
    <skos:altLabel xml:lang="de">Arbeitsmarktregulierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11235-4">
    <skos:altLabel xml:lang="de">Arbeitsförderungsgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11235-4">
    <skos:altLabel xml:lang="de">Aktive Arbeitsmarktpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11235-4">
    <skos:altLabel xml:lang="de">AFG (Arbeitsförderungsgesetz)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12285-5">
    <skos:related rdf:resource="descriptor/19232-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12285-5">
    <skos:related rdf:resource="descriptor/11663-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12285-5">
    <skos:related rdf:resource="descriptor/12280-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12285-5">
    <skos:related rdf:resource="descriptor/12275-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18407-1">
    <skos:broader rdf:resource="thsys/71002"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18407-1">
    <skos:broader rdf:resource="descriptor/13058-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70926">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11469-0">
    <skos:related rdf:resource="descriptor/11473-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70951">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19222-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16429-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10376-5">
    <skos:broader rdf:resource="thsys/71112"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10376-5">
    <skos:broader rdf:resource="thsys/70020"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10464-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18938-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73350">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19471-1">
    <zbwext:useInsteadNote rdf:nodeID="id308"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19471-1">
    <zbwext:useInsteadNote rdf:nodeID="id307"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73390">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10734-5">
    <skos:broader rdf:resource="thsys/73362"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10734-5">
    <skos:broader rdf:resource="thsys/71094"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10734-5">
    <skos:broader rdf:resource="thsys/71092"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10734-5">
    <skos:broader rdf:resource="descriptor/10746-5"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id81">
    <rdfs:seeAlso rdf:resource="descriptor/18078-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13419-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19056-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19136-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15684-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11897-0">
    <skos:narrower rdf:resource="descriptor/13595-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18324-5">
    <skos:prefLabel xml:lang="en">Agroforestry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18324-5">
    <skos:prefLabel xml:lang="de">Agroforstwirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12263-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19447-5">
    <skos:broader rdf:resource="thsys/71069"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19447-5">
    <skos:broader rdf:resource="descriptor/16552-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12057-4">
    <skos:altLabel xml:lang="en">Life-cycle theory</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12057-4">
    <skos:altLabel xml:lang="en">Life-cycle analysis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12057-4">
    <skos:altLabel xml:lang="de">Lebenszyklustheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12057-4">
    <skos:altLabel xml:lang="de">Lebenszyklushypothese</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73389">
    <skos:prefLabel xml:lang="en">N.05.04  Traffic law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73389">
    <skos:prefLabel xml:lang="de">N.05.04  Verkehrsrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71000">
    <rdfs:label xml:lang="en">Nonferrous metals industry</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71000">
    <rdfs:label xml:lang="de">NE-Metallindustrie</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10969-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11382-5">
    <skos:prefLabel xml:lang="en">Teaching</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11382-5">
    <skos:prefLabel xml:lang="de">Unterricht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17672-1">
    <skos:broader rdf:resource="thsys/70445"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17672-1">
    <skos:broader rdf:resource="descriptor/17652-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16084-2">
    <skos:scopeNote xml:lang="de">Benachteiligung von Handelspartnern durch Gewährung von Vorzugsrechten (Preise, Zölle, usw.) an andere Handelspartner.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19157-0">
    <skos:related rdf:resource="descriptor/12181-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19157-0">
    <skos:related rdf:resource="descriptor/13871-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15082-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19147-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15776-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11715-4">
    <skos:prefLabel xml:lang="en">Fiscal theory</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11715-4">
    <skos:prefLabel xml:lang="de">Finanztheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73302">
    <skos:prefLabel xml:lang="de">P.20.03  Fleisch und Fleischwaren</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12129-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13055-3">
    <skos:related rdf:resource="descriptor/14436-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19525-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12297-5">
    <skos:related rdf:resource="descriptor/12403-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12297-5">
    <skos:related rdf:resource="descriptor/12401-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12297-5">
    <skos:related rdf:resource="descriptor/12387-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12297-5">
    <skos:related rdf:resource="descriptor/12385-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12297-5">
    <skos:related rdf:resource="descriptor/12217-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12297-5">
    <skos:related rdf:resource="descriptor/12432-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18444-2">
    <skos:prefLabel xml:lang="en">Health hazard</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18444-2">
    <skos:prefLabel xml:lang="de">Gesundheitsrisiko</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14180-2">
    <skos:broader rdf:resource="thsys/70651"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14180-2">
    <skos:broader rdf:resource="thsys/70227"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14180-2">
    <skos:broader rdf:resource="descriptor/18024-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17567-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18642-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id227">
    <rdf:value xml:lang="de">Für die Subventionierung von Nahrungsmitteln</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10373-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18513-2">
    <skos:broader rdf:resource="thsys/73391"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18513-2">
    <skos:broader rdf:resource="thsys/70963"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18513-2">
    <skos:broader rdf:resource="descriptor/13464-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19701-3">
    <skos:altLabel xml:lang="de">Wahlbezirk</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12510-3">
    <skos:broader rdf:resource="thsys/70459"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12510-3">
    <skos:broader rdf:resource="descriptor/10011-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16290-6">
    <skos:broader rdf:resource="thsys/73375"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12065-5">
    <skos:related rdf:resource="descriptor/12712-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12065-5">
    <skos:related rdf:resource="descriptor/12705-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12065-5">
    <skos:related rdf:resource="descriptor/19231-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12065-5">
    <skos:related rdf:resource="descriptor/12725-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12065-5">
    <skos:related rdf:resource="descriptor/12757-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12065-5">
    <skos:related rdf:resource="descriptor/12744-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15658-5">
    <skos:prefLabel xml:lang="en">Medical sciences</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15658-5">
    <skos:prefLabel xml:lang="de">Medizin</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19378-5">
    <skos:altLabel xml:lang="de">Städtische Finanzpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19378-5">
    <skos:altLabel xml:lang="de">Gemeindefinanzpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18847-1">
    <skos:broader rdf:resource="thsys/70853"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14083-0">
    <skos:prefLabel xml:lang="en">Tea</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14083-0">
    <skos:prefLabel xml:lang="de">Tee</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11177-6">
    <skos:related rdf:resource="descriptor/11718-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11177-6">
    <skos:related rdf:resource="descriptor/19377-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10195-2">
    <skos:related rdf:resource="descriptor/10197-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19497-4">
    <skos:prefLabel xml:lang="en">Employment promotion company</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19497-4">
    <skos:prefLabel xml:lang="de">Beschäftigungsgesellschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18354-3">
    <skos:altLabel xml:lang="de">Verbundwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18354-3">
    <skos:altLabel xml:lang="de">Stromverbundwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18354-3">
    <skos:altLabel xml:lang="de">Stromverbund</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18354-3">
    <skos:altLabel xml:lang="de">Stromlieferungsvertrag</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18354-3">
    <skos:altLabel xml:lang="de">Elektrizitätsverbund</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10858-1">
    <skos:broader rdf:resource="thsys/70951"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10858-1">
    <skos:broader rdf:resource="descriptor/18118-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13821-2">
    <skos:prefLabel xml:lang="en">Insurance science</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13821-2">
    <skos:prefLabel xml:lang="de">Versicherungswissenschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70851">
    <skos:narrower rdf:resource="descriptor/18125-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70851">
    <skos:narrower rdf:resource="descriptor/16438-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70851">
    <skos:narrower rdf:resource="descriptor/18854-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70851">
    <skos:narrower rdf:resource="descriptor/18123-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70851">
    <skos:narrower rdf:resource="descriptor/18852-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70851">
    <skos:narrower rdf:resource="descriptor/16629-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70851">
    <skos:narrower rdf:resource="descriptor/19502-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70851">
    <skos:narrower rdf:resource="descriptor/11313-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70851">
    <skos:narrower rdf:resource="descriptor/11314-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14039-3">
    <skos:altLabel xml:lang="en">Power station</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14039-3">
    <skos:altLabel xml:lang="de">Elektrizitätswerk</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14039-3">
    <skos:altLabel xml:lang="de">Dampfkraftwerk</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10748-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70180">
    <rdfs:label xml:lang="en">Microeconomics</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70180">
    <rdfs:label xml:lang="de">Mikroökonomik</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70859">
    <skos:prefLabel xml:lang="en">W.22.02.01  Public sector management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70859">
    <skos:prefLabel xml:lang="de">W.22.02.01  Öffentliche Betriebswirtschaftslehre</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11901-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71011">
    <skos:prefLabel xml:lang="en">W.04.02  Water industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71011">
    <skos:prefLabel xml:lang="de">W.04.02  Wasserwirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10248-0">
    <skos:altLabel xml:lang="x-other">Tableau economique</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10248-0">
    <skos:altLabel xml:lang="de">Wirtschaftskreislauf</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10248-0">
    <skos:altLabel xml:lang="de">Quesnay-Kreislaufmodell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10248-0">
    <skos:altLabel xml:lang="de">Kreislauftheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10248-0">
    <skos:altLabel xml:lang="de">Kreislaufschema</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18281-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11537-2">
    <skos:altLabel xml:lang="de">Kommunalfinanzen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11537-2">
    <skos:altLabel xml:lang="de">Kommunale Finanzen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12236-4">
    <skos:related rdf:resource="descriptor/10693-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12236-4">
    <skos:related rdf:resource="descriptor/13285-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11398-4">
    <skos:prefLabel xml:lang="en">Education reform</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11398-4">
    <skos:prefLabel xml:lang="de">Bildungsreform</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11657-6">
    <skos:broader rdf:resource="thsys/71074"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11657-6">
    <skos:broader rdf:resource="thsys/70046"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11657-6">
    <skos:broader rdf:resource="descriptor/11679-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11657-6">
    <skos:broader rdf:resource="descriptor/11547-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19282-4">
    <skos:broader rdf:resource="thsys/70899"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19282-4">
    <skos:broader rdf:resource="descriptor/13756-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70301">
    <rdfs:label xml:lang="de">Materialwirtschaft und Logistik</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18302-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14343-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19769-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15751-1">
    <skos:prefLabel xml:lang="en">Non-renewable resources</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15751-1">
    <skos:prefLabel xml:lang="de">Erschöpfbare Ressourcen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11640-2">
    <skos:prefLabel xml:lang="en">Tax law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11640-2">
    <skos:prefLabel xml:lang="de">Steuerrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11973-3">
    <skos:narrower rdf:resource="descriptor/11975-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11973-3">
    <skos:narrower rdf:resource="descriptor/11969-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11973-3">
    <skos:narrower rdf:resource="descriptor/19469-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11910-6">
    <skos:narrower rdf:resource="descriptor/18480-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13405-2">
    <skos:broader rdf:resource="thsys/73378"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13405-2">
    <skos:broader rdf:resource="thsys/70853"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13405-2">
    <skos:broader rdf:resource="thsys/70834"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13405-2">
    <skos:broader rdf:resource="descriptor/10984-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14510-0">
    <skos:broader rdf:resource="thsys/70477"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14510-0">
    <skos:broader rdf:resource="descriptor/14504-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70982">
    <rdfs:label xml:lang="en">Defence industry</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70982">
    <rdfs:label xml:lang="de">Rüstungsindustrie</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19192-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13558-5">
    <skos:broader rdf:resource="thsys/70937"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13558-5">
    <skos:broader rdf:resource="descriptor/13563-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73364">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15716-3">
    <skos:scopeNote xml:lang="de">Veränderungen in der sozialen Schichtzugehörigkeit bzw. im sozialen Status.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11698-6">
    <skos:broader rdf:resource="thsys/73337"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11698-6">
    <skos:broader rdf:resource="thsys/71073"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11698-6">
    <skos:broader rdf:resource="descriptor/10829-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11698-6">
    <skos:broader rdf:resource="descriptor/11684-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10186-3">
    <skos:narrower rdf:resource="descriptor/10135-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10186-3">
    <skos:narrower rdf:resource="descriptor/11322-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10186-3">
    <skos:narrower rdf:resource="descriptor/12019-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11260-5">
    <skos:related rdf:resource="descriptor/10184-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11260-5">
    <skos:related rdf:resource="descriptor/19784-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11260-5">
    <skos:related rdf:resource="descriptor/16679-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11260-5">
    <skos:related rdf:resource="descriptor/11194-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11260-5">
    <skos:related rdf:resource="descriptor/11187-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14118-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13096-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70384">
    <skos:narrower rdf:resource="descriptor/15859-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70384">
    <skos:narrower rdf:resource="descriptor/12847-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70384">
    <skos:narrower rdf:resource="descriptor/12846-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70384">
    <skos:narrower rdf:resource="descriptor/12837-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70384">
    <skos:narrower rdf:resource="descriptor/12829-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70384">
    <skos:narrower rdf:resource="descriptor/15858-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70384">
    <skos:narrower rdf:resource="descriptor/12844-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70384">
    <skos:narrower rdf:resource="descriptor/12842-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70384">
    <skos:narrower rdf:resource="descriptor/12841-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70384">
    <skos:narrower rdf:resource="descriptor/19132-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70384">
    <skos:narrower rdf:resource="descriptor/12821-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70384">
    <skos:narrower rdf:resource="descriptor/12818-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70384">
    <skos:narrower rdf:resource="descriptor/12832-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70384">
    <skos:narrower rdf:resource="descriptor/18099-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70384">
    <skos:narrower rdf:resource="descriptor/12816-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70384">
    <skos:narrower rdf:resource="descriptor/19135-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70384">
    <skos:narrower rdf:resource="descriptor/12825-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70384">
    <skos:narrower rdf:resource="descriptor/19134-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70384">
    <skos:narrower rdf:resource="descriptor/12820-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70384">
    <skos:narrower rdf:resource="descriptor/18098-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70384">
    <skos:narrower rdf:resource="descriptor/21315-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70384">
    <skos:narrower rdf:resource="descriptor/16265-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70384">
    <skos:narrower rdf:resource="descriptor/12786-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70384">
    <skos:narrower rdf:resource="descriptor/19131-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70384">
    <skos:narrower rdf:resource="descriptor/20921-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70384">
    <skos:narrower rdf:resource="descriptor/18543-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70384">
    <skos:narrower rdf:resource="descriptor/19651-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70384">
    <skos:narrower rdf:resource="descriptor/18936-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70384">
    <skos:narrower rdf:resource="descriptor/12824-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70384">
    <skos:narrower rdf:resource="descriptor/21408-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70384">
    <skos:narrower rdf:resource="descriptor/12823-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19603-3">
    <skos:broader rdf:resource="thsys/72181"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19603-3">
    <skos:broader rdf:resource="thsys/70844"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19603-3">
    <skos:broader rdf:resource="descriptor/18888-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19226-0">
    <skos:altLabel xml:lang="de">Monopolistische Konkurrenz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19107-1">
    <skos:scopeNote xml:lang="de">Kontinuierlicher zwischenbetrieblicher Vergleich von Produktionsprozessen, Produkten, Dienstleistungen, usw. zur Erkennung und Schließung von Leistungslücken zum jeweils besten Unternehmen mit dem Ziel der Verbesserung der Wettbewerbsposition.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12945-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18688-2">
    <skos:altLabel xml:lang="en">Foreign currency option</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18688-2">
    <skos:altLabel xml:lang="de">Devisenoption</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11270-2">
    <skos:prefLabel xml:lang="en">Employment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11270-2">
    <skos:prefLabel xml:lang="de">Beschäftigung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12942-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19081-0">
    <skos:related rdf:resource="descriptor/19094-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10626-1">
    <skos:broader rdf:resource="thsys/71093"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10626-1">
    <skos:broader rdf:resource="descriptor/10617-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19009-1">
    <skos:prefLabel xml:lang="en">Mobbing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19009-1">
    <skos:prefLabel xml:lang="de">Mobbing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10811-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70647">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">P.06.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17746-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19724-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70891">
    <rdfs:label xml:lang="de">Versicherungszweige</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13428-4">
    <skos:prefLabel xml:lang="en">Food security</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13428-4">
    <skos:prefLabel xml:lang="de">Nahrungsmittelversorgung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19788-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70893">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19735-0">
    <skos:scopeNote xml:lang="de">Von R. STEINER begründete weltanschauliche Lehre.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14119-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11777-3">
    <skos:related rdf:resource="descriptor/11571-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11777-3">
    <skos:related rdf:resource="descriptor/19279-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11777-3">
    <skos:related rdf:resource="descriptor/12280-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18527-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19157-0">
    <skos:prefLabel xml:lang="en">Partnership enterprise</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19157-0">
    <skos:prefLabel xml:lang="de">Partnerschaftsgesellschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73333">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18353-5">
    <skos:prefLabel xml:lang="en">Energy technology</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18353-5">
    <skos:prefLabel xml:lang="de">Energietechnik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19830-6">
    <skos:broader rdf:resource="thsys/70834"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19830-6">
    <skos:broader rdf:resource="thsys/70504"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13504-0">
    <skos:narrower rdf:resource="descriptor/18588-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13504-0">
    <skos:narrower rdf:resource="descriptor/18606-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71017">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.04</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16526-3">
    <skos:related rdf:resource="descriptor/19760-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16526-3">
    <skos:related rdf:resource="descriptor/11248-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16526-3">
    <skos:related rdf:resource="descriptor/11187-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16526-3">
    <skos:related rdf:resource="descriptor/16527-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16526-3">
    <skos:related rdf:resource="descriptor/11261-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17744-2">
    <skos:altLabel xml:lang="de">Zentralafrikanisches Kaiserreich</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11125-4">
    <skos:related rdf:resource="descriptor/11633-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11125-4">
    <skos:related rdf:resource="descriptor/10882-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13430-3">
    <skos:prefLabel xml:lang="en">Agricultural credit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13430-3">
    <skos:prefLabel xml:lang="de">Agrarkredit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14914-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15243-6">
    <skos:related rdf:resource="descriptor/15066-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15243-6">
    <skos:related rdf:resource="descriptor/15063-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15243-6">
    <skos:related rdf:resource="descriptor/15272-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15243-6">
    <skos:related rdf:resource="descriptor/19568-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15243-6">
    <skos:related rdf:resource="descriptor/15119-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14046-6">
    <skos:prefLabel xml:lang="en">Photograph</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14046-6">
    <skos:prefLabel xml:lang="de">Fotografie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16295-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10458-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10051-5">
    <skos:prefLabel xml:lang="en">Open economy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10051-5">
    <skos:prefLabel xml:lang="de">Offene Volkswirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17674-4">
    <skos:broader rdf:resource="thsys/70445"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17674-4">
    <skos:broader rdf:resource="descriptor/17672-1"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id161">
    <rdf:value xml:lang="de">Für den Anbau von Kaffee</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14822-2">
    <skos:altLabel xml:lang="de">Plastikwaren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13213-4">
    <skos:altLabel xml:lang="de">Porzellanhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13213-4">
    <skos:altLabel xml:lang="de">Haushaltswarengeschäft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13213-4">
    <skos:altLabel xml:lang="de">Glaswarenhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13213-4">
    <skos:altLabel xml:lang="de">Glas-Porzellan-Keramik-Fachhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12761-6">
    <skos:altLabel xml:lang="en">Single-machine scheduling</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12761-6">
    <skos:altLabel xml:lang="en">Sequencing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12761-6">
    <skos:altLabel xml:lang="en">One-machine scheduling</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12761-6">
    <skos:altLabel xml:lang="en">Machine sequencing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12761-6">
    <skos:altLabel xml:lang="en">Machine scheduling</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12761-6">
    <skos:altLabel xml:lang="en">Job shop sequencing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12761-6">
    <skos:altLabel xml:lang="en">Job shop scheduling</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12761-6">
    <skos:altLabel xml:lang="en">Job shop problem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12761-6">
    <skos:altLabel xml:lang="en">Flow shop scheduling</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12761-6">
    <skos:altLabel xml:lang="en">Combinatorial optimization</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12761-6">
    <skos:altLabel xml:lang="de">Stundenplan-Optimierungsproblem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12761-6">
    <skos:altLabel xml:lang="de">Reihenfolge-Optimierungsproblem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12761-6">
    <skos:altLabel xml:lang="de">Maschinenbelegungsoptimierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12761-6">
    <skos:altLabel xml:lang="de">Kombinatorische Optimierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12761-6">
    <skos:altLabel xml:lang="de">Auftragsfolgenoptimierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70477">
    <skos:prefLabel xml:lang="de">P.10.01  Elektrohaushaltsgeräte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id204">
    <rdfs:seeAlso rdf:resource="descriptor/15149-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11627-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11372-1">
    <skos:related rdf:resource="descriptor/18782-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11372-1">
    <skos:related rdf:resource="descriptor/11373-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10661-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14833-4">
    <skos:related rdf:resource="descriptor/19391-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14833-4">
    <skos:related rdf:resource="descriptor/13142-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14833-4">
    <skos:related rdf:resource="descriptor/13232-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14833-4">
    <skos:related rdf:resource="descriptor/14831-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14833-4">
    <skos:related rdf:resource="descriptor/14817-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14833-4">
    <skos:related rdf:resource="descriptor/14899-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16340-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18313-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19667-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16615-4">
    <skos:altLabel xml:lang="en">Acculteration</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16615-4">
    <skos:altLabel xml:lang="de">Soziale Eingliederung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16615-4">
    <skos:altLabel xml:lang="de">Kulturelle Integration</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16615-4">
    <skos:altLabel xml:lang="de">Integrationspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16615-4">
    <skos:altLabel xml:lang="de">Eingliederung in die Gesellschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16615-4">
    <skos:altLabel xml:lang="de">Akkulturation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13989-6">
    <skos:altLabel xml:lang="de">Politikerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18108-4">
    <skos:prefLabel xml:lang="en">EEIG (European Economic Interest Group)</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18108-4">
    <skos:prefLabel xml:lang="de">EWIV</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18934-6">
    <skos:prefLabel xml:lang="en">Television programme</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18934-6">
    <skos:prefLabel xml:lang="de">Fernsehprogramm</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18354-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12689-2">
    <skos:prefLabel xml:lang="en">Inplant transportation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12689-2">
    <skos:prefLabel xml:lang="de">Betriebliches Transportwesen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13058-4">
    <skos:narrower rdf:resource="descriptor/13067-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13058-4">
    <skos:narrower rdf:resource="descriptor/18415-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13058-4">
    <skos:narrower rdf:resource="descriptor/18095-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13058-4">
    <skos:narrower rdf:resource="descriptor/18414-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13058-4">
    <skos:narrower rdf:resource="descriptor/18413-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13058-4">
    <skos:narrower rdf:resource="descriptor/13054-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13058-4">
    <skos:narrower rdf:resource="descriptor/18411-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13058-4">
    <skos:narrower rdf:resource="descriptor/18409-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13058-4">
    <skos:narrower rdf:resource="descriptor/18407-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13058-4">
    <skos:narrower rdf:resource="descriptor/13045-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14472-3">
    <skos:narrower rdf:resource="descriptor/14480-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70581">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18591-3">
    <skos:broader rdf:resource="thsys/73389"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18591-3">
    <skos:broader rdf:resource="thsys/70936"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18591-3">
    <skos:broader rdf:resource="descriptor/13569-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18285-3">
    <skos:related rdf:resource="descriptor/17626-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18285-3">
    <skos:related rdf:resource="descriptor/17623-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10649-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12793-0">
    <skos:altLabel xml:lang="en">Bargain price</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12793-0">
    <skos:altLabel xml:lang="de">Werbepreis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12793-0">
    <skos:altLabel xml:lang="de">Sonderpreis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12793-0">
    <skos:altLabel xml:lang="de">Saisonpreis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12793-0">
    <skos:altLabel xml:lang="de">Räumungspreis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12793-0">
    <skos:altLabel xml:lang="de">Penetration pricing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12793-0">
    <skos:altLabel xml:lang="de">Einführungspreis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12793-0">
    <skos:altLabel xml:lang="de">Diskontpreis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12793-0">
    <skos:altLabel xml:lang="de">Discountpreis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14364-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70871">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.22.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10276-2">
    <skos:altLabel xml:lang="en">Private consumption</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10276-2">
    <skos:altLabel xml:lang="de">Verbrauch der Gesamtwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10276-2">
    <skos:altLabel xml:lang="de">Privater Verbrauch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10276-2">
    <skos:altLabel xml:lang="de">Privater Konsum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10276-2">
    <skos:altLabel xml:lang="de">Konsum der Gesamtwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10276-2">
    <skos:altLabel xml:lang="de">Gesamtwirtschaftlicher Verbrauch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17712-1">
    <skos:prefLabel xml:lang="en">Mali</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17712-1">
    <skos:prefLabel xml:lang="de">Mali</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17942-5">
    <skos:broader rdf:resource="thsys/70128"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17942-5">
    <skos:broader rdf:resource="descriptor/17948-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13445-4">
    <skos:prefLabel xml:lang="en">Rural sociology</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13445-4">
    <skos:prefLabel xml:lang="de">Agrarsoziologie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70578">
    <skos:broader rdf:resource="thsys/70005"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71095">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17979-3">
    <skos:prefLabel xml:lang="en">ASEAN Countries</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17979-3">
    <skos:prefLabel xml:lang="de">ASEAN-Staaten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11365-5">
    <skos:broader rdf:resource="thsys/70843"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11365-5">
    <skos:broader rdf:resource="thsys/70244"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11365-5">
    <skos:broader rdf:resource="descriptor/11366-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11365-5">
    <skos:broader rdf:resource="descriptor/11361-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18288-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12314-3">
    <skos:scopeNote xml:lang="de">Summe aller Leistungspotenziale aller Arbeitnehmer/innen eines Betriebes.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19956-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16475-1">
    <skos:prefLabel xml:lang="en">Policy for senior citizens</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16475-1">
    <skos:prefLabel xml:lang="de">Altenpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13278-4">
    <zbwext:useInsteadNote rdf:nodeID="id326"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16495-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18610-4">
    <skos:related rdf:resource="descriptor/13613-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18610-4">
    <skos:related rdf:resource="descriptor/18633-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18610-4">
    <skos:related rdf:resource="descriptor/18576-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18610-4">
    <skos:related rdf:resource="descriptor/18632-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18610-4">
    <skos:related rdf:resource="descriptor/13249-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20455-3">
    <zbwext:useInsteadNote rdf:nodeID="id102"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16788-1">
    <skos:altLabel xml:lang="de">Literaturversorgung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14022-6">
    <skos:prefLabel xml:lang="en">Floorcovering</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14022-6">
    <skos:prefLabel xml:lang="de">Bodenbelag</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18314-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13472-1">
    <skos:scopeNote xml:lang="de">Für den über die Rohstoffbörse abgewickelten Handel BENUTZE Warenbörse bzw. Rohstoff-Futures.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19350-6">
    <skos:prefLabel xml:lang="en">Assocation agreement</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19350-6">
    <skos:prefLabel xml:lang="de">Assoziierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12744-6">
    <skos:altLabel xml:lang="de">Produktionswirtschaftliche Theorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12744-6">
    <skos:altLabel xml:lang="de">Produktionstheorie der Betriebswirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16067-2">
    <skos:related rdf:resource="descriptor/12775-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16067-2">
    <skos:related rdf:resource="descriptor/19835-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16067-2">
    <skos:related rdf:resource="descriptor/15961-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16067-2">
    <skos:related rdf:resource="descriptor/19850-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16067-2">
    <skos:related rdf:resource="descriptor/16050-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16067-2">
    <skos:related rdf:resource="descriptor/19848-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16067-2">
    <skos:related rdf:resource="descriptor/12370-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16067-2">
    <skos:related rdf:resource="descriptor/12326-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16067-2">
    <skos:related rdf:resource="descriptor/12217-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14931-4">
    <skos:broader rdf:resource="thsys/70498"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70013">
    <skos:narrower rdf:resource="descriptor/19766-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70013">
    <skos:narrower rdf:resource="descriptor/16353-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70013">
    <skos:narrower rdf:resource="descriptor/13645-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70013">
    <skos:narrower rdf:resource="descriptor/19466-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70013">
    <skos:narrower rdf:resource="descriptor/19611-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70013">
    <skos:narrower rdf:resource="descriptor/16684-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70013">
    <skos:narrower rdf:resource="descriptor/18932-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70013">
    <skos:narrower rdf:resource="descriptor/18930-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70013">
    <skos:narrower rdf:resource="descriptor/16680-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70013">
    <skos:narrower rdf:resource="descriptor/16761-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70013">
    <skos:narrower rdf:resource="descriptor/16759-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70013">
    <skos:narrower rdf:resource="descriptor/16683-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70013">
    <skos:narrower rdf:resource="descriptor/18916-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70013">
    <skos:narrower rdf:resource="descriptor/16679-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70013">
    <skos:narrower rdf:resource="descriptor/16556-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70013">
    <skos:narrower rdf:resource="descriptor/18335-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70013">
    <skos:narrower rdf:resource="descriptor/19361-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70013">
    <skos:narrower rdf:resource="descriptor/18860-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70013">
    <skos:narrower rdf:resource="descriptor/11394-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70013">
    <skos:narrower rdf:resource="thsys/73355"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19041-5">
    <skos:broader rdf:resource="thsys/70582"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12255-0">
    <skos:altLabel xml:lang="en">Supplier credit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12255-0">
    <skos:altLabel xml:lang="de">Warenkredit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18226-5">
    <skos:prefLabel xml:lang="en">Slovenia</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18226-5">
    <skos:prefLabel xml:lang="de">Slowenien</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71105">
    <skos:broader rdf:resource="thsys/71107"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11640-2">
    <skos:broader rdf:resource="thsys/73337"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11640-2">
    <skos:broader rdf:resource="thsys/71073"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11640-2">
    <skos:broader rdf:resource="thsys/70046"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11640-2">
    <skos:broader rdf:resource="descriptor/11525-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19407-3">
    <skos:narrower rdf:resource="descriptor/11682-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19407-3">
    <skos:narrower rdf:resource="descriptor/11651-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19407-3">
    <skos:narrower rdf:resource="descriptor/11650-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10566-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10957-6">
    <skos:broader rdf:resource="thsys/71071"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10957-6">
    <skos:broader rdf:resource="descriptor/10958-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19838-4">
    <skos:prefLabel xml:lang="en">Electric utility regulation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19838-4">
    <skos:prefLabel xml:lang="de">Elektrizitätsrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17915-1">
    <skos:broader rdf:resource="thsys/71030"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17915-1">
    <skos:broader rdf:resource="descriptor/17931-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17915-1">
    <skos:broader rdf:resource="descriptor/19179-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17735-3">
    <skos:prefLabel xml:lang="en">Congo (Republic)</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17735-3">
    <skos:prefLabel xml:lang="de">Kongo (Republik)</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18278-0">
    <skos:altLabel xml:lang="de">Wirtschaftliche Warenlehre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18278-0">
    <skos:altLabel xml:lang="de">Warenwirtschaftslehre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18278-0">
    <skos:altLabel xml:lang="de">Warenkunde</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18278-0">
    <skos:altLabel xml:lang="de">Ökonomische Produkttheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18278-0">
    <skos:altLabel xml:lang="de">Betriebswirtschaftliche Warenlehre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17782-1">
    <skos:broader rdf:resource="thsys/70086"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17782-1">
    <skos:broader rdf:resource="descriptor/17791-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18629-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15742-2">
    <skos:altLabel xml:lang="de">TA-Luft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15742-2">
    <skos:altLabel xml:lang="de">Großfeuerungsanlagenverordnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15742-2">
    <skos:altLabel xml:lang="de">Benzinbleigesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11048-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14408-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18830-4">
    <skos:altLabel xml:lang="en">Common good</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18830-4">
    <skos:altLabel xml:lang="de">Allgemeinwohl</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18887-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18989-2">
    <skos:altLabel xml:lang="de">Energieverbrauch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12256-5">
    <skos:prefLabel xml:lang="en">Warrant bond</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12256-5">
    <skos:prefLabel xml:lang="de">Optionsanleihe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19785-6">
    <skos:broader rdf:resource="thsys/73349"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19785-6">
    <skos:broader rdf:resource="descriptor/18430-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10237-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16513-5">
    <skos:scopeNote xml:lang="de">Alle politischen Maßnahmen, die an der rassischen Zugehörigkeit von Bevölkerungsgruppen ansetzen</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15072-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17704-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19451-0">
    <skos:altLabel xml:lang="de">Industrieumsiedlung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19451-0">
    <skos:altLabel xml:lang="de">Industrieauslagerung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#prefLabel">
    <rdfs:isDefinedBy rdf:resource="http://www.w3.org/2004/02/skos/core"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13594-1">
    <skos:prefLabel xml:lang="en">Transportation geography</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13594-1">
    <skos:prefLabel xml:lang="de">Verkehrsgeographie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17750-0">
    <skos:prefLabel xml:lang="en">North Africa</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17750-0">
    <skos:prefLabel xml:lang="de">Nordafrika</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15479-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18354-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13453-5">
    <skos:broader rdf:resource="thsys/71024"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13453-5">
    <skos:broader rdf:resource="descriptor/19496-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13453-5">
    <skos:broader rdf:resource="descriptor/18313-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13453-5">
    <skos:broader rdf:resource="descriptor/12945-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70832">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.26.03</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16948-5">
    <skos:related rdf:resource="descriptor/16955-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14431-3">
    <skos:prefLabel xml:lang="en">Electric vehicle</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14431-3">
    <skos:prefLabel xml:lang="de">Elektrofahrzeug</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25822-5">
    <skos:related rdf:resource="descriptor/19733-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25822-5">
    <skos:related rdf:resource="descriptor/12108-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25822-5">
    <skos:related rdf:resource="descriptor/18116-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25822-5">
    <skos:related rdf:resource="descriptor/19090-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25822-5">
    <skos:related rdf:resource="descriptor/18099-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25822-5">
    <skos:related rdf:resource="descriptor/12394-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14989-1">
    <skos:broader rdf:resource="thsys/73299"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14989-1">
    <skos:broader rdf:resource="descriptor/14931-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13428-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10548-2">
    <skos:related rdf:resource="descriptor/10590-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10548-2">
    <skos:related rdf:resource="descriptor/11529-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10548-2">
    <skos:related rdf:resource="descriptor/10561-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10548-2">
    <skos:related rdf:resource="descriptor/10523-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18643-3">
    <skos:altLabel xml:lang="de">TKO (Telekommunikationsordnung)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18643-3">
    <skos:altLabel xml:lang="de">Telekommunikationsordnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18643-3">
    <skos:altLabel xml:lang="de">Gesetz über Fernmeldeanlagen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18643-3">
    <skos:altLabel xml:lang="de">Fernmelderecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18643-3">
    <skos:altLabel xml:lang="de">Fernmeldebenutzungsrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18643-3">
    <skos:altLabel xml:lang="de">FAG (Fernmeldeanlagengesetz)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18161-0">
    <skos:prefLabel xml:lang="en">Statistical office</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18161-0">
    <skos:prefLabel xml:lang="de">Statistisches Amt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id31">
    <rdf:value xml:lang="de">Für die Gesamtheit des Außenhandels einer Volkswirtschaft</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12062-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13560-4">
    <skos:narrower rdf:resource="descriptor/13555-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12312-0">
    <skos:related rdf:resource="descriptor/12226-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12312-0">
    <skos:related rdf:resource="descriptor/12372-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12312-0">
    <skos:related rdf:resource="descriptor/12308-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12312-0">
    <skos:related rdf:resource="descriptor/12320-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10614-1">
    <skos:altLabel xml:lang="de">Spezialisierung im internationalen Wettbewerb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10614-1">
    <skos:altLabel xml:lang="de">Internationale Spezialisierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18129-3">
    <skos:altLabel xml:lang="de">Substitution von Produkten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18129-3">
    <skos:altLabel xml:lang="de">Substitution von Gütern</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18129-3">
    <skos:altLabel xml:lang="de">Gütersubstitution</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14157-4">
    <skos:broader rdf:resource="thsys/70465"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14157-4">
    <skos:broader rdf:resource="descriptor/18195-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14157-4">
    <skos:broader rdf:resource="descriptor/14150-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14428-6">
    <skos:altLabel xml:lang="de">Waggon</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14428-6">
    <skos:altLabel xml:lang="de">Triebwagen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14428-6">
    <skos:altLabel xml:lang="de">Schienenbus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14428-6">
    <skos:altLabel xml:lang="de">Rollendes Eisenbahnmaterial</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14428-6">
    <skos:altLabel xml:lang="de">Lokomotive</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14428-6">
    <skos:altLabel xml:lang="de">Eisenbahnfahrzeug</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18644-1">
    <zbwext:useInsteadNote rdf:nodeID="id257"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19555-2">
    <skos:related rdf:resource="descriptor/21041-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19555-2">
    <skos:related rdf:resource="descriptor/12785-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13247-1">
    <skos:prefLabel xml:lang="en">Airline</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13247-1">
    <skos:prefLabel xml:lang="de">Luftverkehrsgesellschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19110-5">
    <skos:altLabel xml:lang="de">Betriebsratswahl</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14685-0">
    <skos:prefLabel xml:lang="en">Phenol</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14685-0">
    <skos:prefLabel xml:lang="de">Phenol</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17750-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13431-1">
    <skos:broader rdf:resource="thsys/71028"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13431-1">
    <skos:broader rdf:resource="thsys/70960"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13431-1">
    <skos:broader rdf:resource="descriptor/10012-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14729-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19756-6">
    <skos:related rdf:resource="descriptor/19011-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19756-6">
    <skos:related rdf:resource="descriptor/19757-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19756-6">
    <skos:related rdf:resource="descriptor/15945-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19756-6">
    <skos:related rdf:resource="descriptor/24663-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19756-6">
    <skos:related rdf:resource="descriptor/18142-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19756-6">
    <skos:related rdf:resource="descriptor/16628-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19756-6">
    <skos:related rdf:resource="descriptor/11189-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15005-1">
    <skos:altLabel xml:lang="en">Arms</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15005-1">
    <skos:altLabel xml:lang="de">Wehrgut</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15005-1">
    <skos:altLabel xml:lang="de">Waffensystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15005-1">
    <skos:altLabel xml:lang="de">Waffe (Kriegswaffe)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15005-1">
    <skos:altLabel xml:lang="de">Raketenwaffe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15005-1">
    <skos:altLabel xml:lang="de">Raketenabwehrsystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15005-1">
    <skos:altLabel xml:lang="de">Munition</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15005-1">
    <skos:altLabel xml:lang="de">Militärische Ausrüstung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15005-1">
    <skos:altLabel xml:lang="de">Kriegswaffe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15005-1">
    <skos:altLabel xml:lang="de">Konventionelle Waffe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15005-1">
    <skos:altLabel xml:lang="de">B-Waffe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15005-1">
    <skos:altLabel xml:lang="de">Biologische Waffe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15005-1">
    <skos:altLabel xml:lang="de">Bakteriologische Waffe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18327-6">
    <skos:related rdf:resource="descriptor/18967-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18327-6">
    <skos:related rdf:resource="descriptor/12021-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18327-6">
    <skos:related rdf:resource="descriptor/11859-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18327-6">
    <skos:related rdf:resource="descriptor/12016-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16724-6">
    <skos:scopeNote xml:lang="de">Wissenschaftliche Betrachtungsweise, die Einzelphänomene aus ihrer Einbettung in größere Zusammenhänge analysiert und so versucht, analog zu den intersubjektiv überprüfbaren und exakten Verfahrensweisen der Naturwissenschaften auch in den Geistes- und Sozialwissenschaften vorzugehen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15107-0">
    <skos:related rdf:resource="descriptor/15103-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15107-0">
    <skos:related rdf:resource="descriptor/19579-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15107-0">
    <skos:related rdf:resource="descriptor/19604-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15107-0">
    <skos:related rdf:resource="descriptor/11297-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15107-0">
    <skos:related rdf:resource="descriptor/11264-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15107-0">
    <skos:related rdf:resource="descriptor/13433-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11532-5">
    <skos:related rdf:resource="descriptor/16087-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11532-5">
    <skos:related rdf:resource="descriptor/16112-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11532-5">
    <skos:related rdf:resource="descriptor/11535-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11532-5">
    <skos:related rdf:resource="descriptor/11525-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18977-2">
    <skos:prefLabel xml:lang="en">Commercial professions</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18977-2">
    <skos:prefLabel xml:lang="de">Kaufmännische Berufe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70002">
    <skos:broader rdf:resource="thsys/g"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15450-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10377-3">
    <skos:altLabel xml:lang="en">Thatcherism</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10377-3">
    <skos:altLabel xml:lang="en">Reagonomics</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10377-3">
    <skos:altLabel xml:lang="en">Reaganomics</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10377-3">
    <skos:altLabel xml:lang="de">Potenzialorientierte Wirtschaftspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10377-3">
    <skos:altLabel xml:lang="de">Potentialorientierte Wirtschaftspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10377-3">
    <skos:altLabel xml:lang="de">Angebotspolitik (Supply-Side Economics)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10377-3">
    <skos:altLabel xml:lang="de">Angebotsökonomik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14685-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10973-1">
    <skos:narrower rdf:resource="descriptor/11819-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11501-2">
    <skos:altLabel xml:lang="de">Gibson-Paradoxon</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18617-4">
    <skos:prefLabel xml:lang="en">Intermodal freight terminal</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18617-4">
    <skos:prefLabel xml:lang="de">Güterverkehrszentrum</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73395">
    <skos:narrower rdf:resource="descriptor/16619-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73395">
    <skos:narrower rdf:resource="descriptor/15718-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73395">
    <skos:narrower rdf:resource="descriptor/15717-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73395">
    <skos:narrower rdf:resource="descriptor/15716-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73395">
    <skos:narrower rdf:resource="descriptor/15715-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73395">
    <skos:narrower rdf:resource="descriptor/16623-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73395">
    <skos:narrower rdf:resource="descriptor/16645-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73395">
    <skos:narrower rdf:resource="descriptor/16611-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73395">
    <skos:narrower rdf:resource="descriptor/11164-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73395">
    <skos:narrower rdf:resource="descriptor/16639-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73395">
    <skos:narrower rdf:resource="descriptor/16638-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73395">
    <skos:narrower rdf:resource="descriptor/11087-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73395">
    <skos:narrower rdf:resource="descriptor/11158-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73395">
    <skos:narrower rdf:resource="descriptor/16429-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73395">
    <skos:narrower rdf:resource="descriptor/16658-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73395">
    <skos:narrower rdf:resource="descriptor/15713-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73395">
    <skos:narrower rdf:resource="descriptor/16608-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73395">
    <skos:narrower rdf:resource="descriptor/19753-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73395">
    <skos:narrower rdf:resource="descriptor/10488-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73395">
    <skos:narrower rdf:resource="descriptor/11070-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73395">
    <skos:narrower rdf:resource="descriptor/19748-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13546-5">
    <skos:altLabel xml:lang="de">Handelsflotte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12384-3">
    <skos:broader rdf:resource="thsys/71043"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12384-3">
    <skos:broader rdf:resource="descriptor/12337-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21041-4">
    <skos:altLabel xml:lang="de">Vernetztes Verkaufen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21041-4">
    <skos:altLabel xml:lang="de">System-Marketing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21041-4">
    <skos:altLabel xml:lang="de">Leistungssystem-Marketing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13600-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15876-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19818-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11358-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10683-3">
    <skos:broader rdf:resource="thsys/71098"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10683-3">
    <skos:broader rdf:resource="descriptor/10894-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10683-3">
    <skos:broader rdf:resource="descriptor/10686-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11451-5">
    <skos:prefLabel xml:lang="en">Near money</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11451-5">
    <skos:prefLabel xml:lang="de">Geldsubstitut</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12625-0">
    <skos:scopeNote xml:lang="de">Erweiterung des Aufgabenspektrums für einzelne Arbeitnehmer/innen zur Steigerung der Arbeitszufriedenheit.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12142-6">
    <skos:broader rdf:resource="thsys/70485"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70883">
    <skos:narrower rdf:resource="descriptor/19057-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70883">
    <skos:narrower rdf:resource="descriptor/11203-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70883">
    <skos:narrower rdf:resource="descriptor/18760-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70883">
    <skos:narrower rdf:resource="descriptor/18759-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70883">
    <skos:narrower rdf:resource="descriptor/12254-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70883">
    <skos:narrower rdf:resource="descriptor/18369-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70883">
    <skos:narrower rdf:resource="descriptor/12030-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70883">
    <skos:narrower rdf:resource="descriptor/12243-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70883">
    <skos:narrower rdf:resource="descriptor/19066-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70883">
    <skos:narrower rdf:resource="descriptor/18758-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70883">
    <skos:narrower rdf:resource="descriptor/19058-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70883">
    <skos:narrower rdf:resource="descriptor/13248-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12029-2">
    <skos:altLabel xml:lang="de">Grundstück</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16771-4">
    <skos:broader rdf:resource="thsys/70822"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16771-4">
    <skos:broader rdf:resource="descriptor/16760-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19324-0">
    <skos:broader rdf:resource="thsys/71097"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19324-0">
    <skos:broader rdf:resource="descriptor/16288-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70003">
    <rdfs:label xml:lang="en">Switzerland</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70003">
    <rdfs:label xml:lang="de">Schweiz</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18689-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19655-5">
    <skos:related rdf:resource="descriptor/19766-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19655-5">
    <skos:related rdf:resource="descriptor/19253-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19655-5">
    <skos:related rdf:resource="descriptor/18017-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19655-5">
    <skos:related rdf:resource="descriptor/12314-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19655-5">
    <skos:related rdf:resource="descriptor/20341-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10492-3">
    <skos:altLabel xml:lang="en">Underdeveloped countries</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10492-3">
    <skos:altLabel xml:lang="en">Less developed countries</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10492-3">
    <skos:altLabel xml:lang="en">LDC (Less Developed Countries)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10492-3">
    <skos:altLabel xml:lang="en">Backward countries</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10492-3">
    <skos:altLabel xml:lang="de">Unterentwickelte Länder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10492-3">
    <skos:altLabel xml:lang="de">Dritte Welt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10492-3">
    <skos:altLabel xml:lang="de">Arme Staaten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10492-3">
    <skos:altLabel xml:lang="de">Arme Länder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14051-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14392-1">
    <skos:broader rdf:resource="thsys/70318"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14392-1">
    <skos:broader rdf:resource="descriptor/14381-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19689-2">
    <skos:altLabel xml:lang="de">Verfassungsorgan</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18882-6">
    <skos:altLabel xml:lang="de">EU-Berufsbildungsprogramm</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18604-6">
    <skos:prefLabel xml:lang="en">Airport authority</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18604-6">
    <skos:prefLabel xml:lang="de">Flughafengesellschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18106-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14053-2">
    <skos:altLabel xml:lang="de">Wellenkraftwerk</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14053-2">
    <skos:altLabel xml:lang="de">Gezeitenkraftwerk</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14053-2">
    <skos:altLabel xml:lang="de">Flutkraftwerk</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16955-1">
    <skos:altLabel xml:lang="de">Wien (Stadt)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16955-1">
    <skos:altLabel xml:lang="de">Wien (Land)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19749-3">
    <skos:prefLabel xml:lang="en">Award</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19749-3">
    <skos:prefLabel xml:lang="de">Preisverleihung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18730-1">
    <skos:related rdf:resource="descriptor/19337-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18730-1">
    <skos:related rdf:resource="descriptor/13715-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18730-1">
    <skos:related rdf:resource="descriptor/18729-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18730-1">
    <skos:related rdf:resource="descriptor/18728-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18730-1">
    <skos:related rdf:resource="descriptor/13688-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18376-0">
    <skos:scopeNote xml:lang="de">Nur für energiebezogene Umweltsteuern.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12224-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10404-5">
    <skos:altLabel xml:lang="en">TIP (Tax-Based Incomes Policy)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10404-5">
    <skos:altLabel xml:lang="de">Steuerlich orientierte Einkommenspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10404-5">
    <skos:altLabel xml:lang="de">Lohnzuwachssteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id246">
    <rdfs:seeAlso rdf:resource="descriptor/10968-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19187-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19215-5">
    <skos:narrower rdf:resource="descriptor/19214-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19215-5">
    <skos:narrower rdf:resource="descriptor/11159-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19248-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14268-2">
    <skos:broader rdf:resource="thsys/70346"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14268-2">
    <skos:broader rdf:resource="descriptor/14269-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15064-6">
    <skos:scopeNote xml:lang="de">Für Literatur über die Anwendung statistischer Methoden.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13894-3">
    <skos:related rdf:resource="descriptor/12912-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13894-3">
    <skos:related rdf:resource="descriptor/13874-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71088">
    <skos:broader rdf:resource="thsys/71089"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11033-2">
    <skos:related rdf:resource="descriptor/11061-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11033-2">
    <skos:related rdf:resource="descriptor/10970-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11033-2">
    <skos:related rdf:resource="descriptor/11004-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11033-2">
    <skos:related rdf:resource="descriptor/10559-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14184-1">
    <skos:broader rdf:resource="thsys/70538"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14184-1">
    <skos:broader rdf:resource="descriptor/14047-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14184-1">
    <skos:broader rdf:resource="descriptor/14176-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19282-4">
    <skos:prefLabel xml:lang="en">Deposit operations</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19282-4">
    <skos:prefLabel xml:lang="de">Passivgeschäft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13935-1">
    <skos:altLabel xml:lang="de">Weberinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13935-1">
    <skos:altLabel xml:lang="de">Weber</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13935-1">
    <skos:altLabel xml:lang="de">Textilindustriearbeiterinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13935-1">
    <skos:altLabel xml:lang="de">Textilindustriearbeiter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13935-1">
    <skos:altLabel xml:lang="de">Textilfärberinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13935-1">
    <skos:altLabel xml:lang="de">Textilfärber</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13935-1">
    <skos:altLabel xml:lang="de">Textilarbeiterinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13935-1">
    <skos:altLabel xml:lang="de">Textilarbeiter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13935-1">
    <skos:altLabel xml:lang="de">Strickerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13935-1">
    <skos:altLabel xml:lang="de">Stricker</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13935-1">
    <skos:altLabel xml:lang="de">Stickerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13935-1">
    <skos:altLabel xml:lang="de">Sticker</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13935-1">
    <skos:altLabel xml:lang="de">Schneiderinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13935-1">
    <skos:altLabel xml:lang="de">Schneider</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13935-1">
    <skos:altLabel xml:lang="de">Näherinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13935-1">
    <skos:altLabel xml:lang="de">Näher</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13935-1">
    <skos:altLabel xml:lang="de">Modistinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13935-1">
    <skos:altLabel xml:lang="de">Modisten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13935-1">
    <skos:altLabel xml:lang="de">Hutmacherinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13935-1">
    <skos:altLabel xml:lang="de">Hutmacher</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13935-1">
    <skos:altLabel xml:lang="de">Bekleidungsberufe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19579-2">
    <skos:related rdf:resource="descriptor/15107-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19579-2">
    <skos:related rdf:resource="descriptor/12910-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19579-2">
    <skos:related rdf:resource="descriptor/19604-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19267-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18159-1">
    <skos:broader rdf:resource="thsys/71051"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18159-1">
    <skos:broader rdf:resource="thsys/70235"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18159-1">
    <skos:broader rdf:resource="descriptor/11310-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19421-2">
    <skos:altLabel xml:lang="de">Sektsteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19421-2">
    <skos:altLabel xml:lang="de">Schaumweinsteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10761-2">
    <skos:related rdf:resource="descriptor/10736-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12365-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18392-2">
    <skos:related rdf:resource="descriptor/19185-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18392-2">
    <skos:related rdf:resource="descriptor/13033-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18392-2">
    <skos:related rdf:resource="descriptor/14299-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20301-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10494-6">
    <skos:scopeNote xml:lang="de">Entwicklungsindikatoren sind z.B.: Pro-Kopf-Sozialprodukt, Lebenserwartung, medizinische Versorgung, Analphabetenquote, Bildungsstand.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71101">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.06.01.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19463-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70269">
    <skos:narrower rdf:resource="descriptor/15060-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70269">
    <skos:narrower rdf:resource="descriptor/13823-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70269">
    <skos:narrower rdf:resource="descriptor/15429-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70269">
    <skos:narrower rdf:resource="descriptor/15057-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70269">
    <skos:narrower rdf:resource="descriptor/19591-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70269">
    <skos:narrower rdf:resource="descriptor/15543-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70269">
    <skos:narrower rdf:resource="descriptor/15054-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70269">
    <skos:narrower rdf:resource="descriptor/15051-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70269">
    <skos:narrower rdf:resource="descriptor/15267-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70269">
    <skos:narrower rdf:resource="descriptor/15050-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70269">
    <skos:narrower rdf:resource="descriptor/15055-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70269">
    <skos:narrower rdf:resource="descriptor/15045-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70269">
    <skos:narrower rdf:resource="descriptor/11742-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70269">
    <skos:narrower rdf:resource="descriptor/16716-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70269">
    <skos:narrower rdf:resource="descriptor/15415-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70269">
    <skos:narrower rdf:resource="descriptor/15038-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70269">
    <skos:narrower rdf:resource="descriptor/11738-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70269">
    <skos:narrower rdf:resource="descriptor/15036-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70269">
    <skos:narrower rdf:resource="descriptor/12207-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70269">
    <skos:narrower rdf:resource="descriptor/19583-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70269">
    <skos:narrower rdf:resource="descriptor/15033-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70269">
    <skos:narrower rdf:resource="descriptor/18171-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70269">
    <skos:narrower rdf:resource="descriptor/15026-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70269">
    <skos:narrower rdf:resource="descriptor/19536-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73302">
    <skos:narrower rdf:resource="descriptor/18219-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73302">
    <skos:narrower rdf:resource="descriptor/19021-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73302">
    <skos:narrower rdf:resource="descriptor/18220-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73302">
    <skos:narrower rdf:resource="descriptor/18218-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73302">
    <skos:narrower rdf:resource="descriptor/14964-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73302">
    <skos:narrower rdf:resource="descriptor/14122-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11564-6">
    <skos:altLabel xml:lang="en">Family allowance</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11564-6">
    <skos:altLabel xml:lang="en">Child care tax credit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11564-6">
    <skos:altLabel xml:lang="en">Child care subsidy</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11564-6">
    <skos:altLabel xml:lang="en">Child benefit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11564-6">
    <skos:altLabel xml:lang="en">Aid for families with dependent children</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11564-6">
    <skos:altLabel xml:lang="de">Monetäre Familienpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11564-6">
    <skos:altLabel xml:lang="de">Kinderlastenausgleich</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11564-6">
    <skos:altLabel xml:lang="de">Kindergeld</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11564-6">
    <skos:altLabel xml:lang="de">Kinderfreibetrag</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11564-6">
    <skos:altLabel xml:lang="de">Kinderbeihilfe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11564-6">
    <skos:altLabel xml:lang="de">Familienzulage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11564-6">
    <skos:altLabel xml:lang="de">Familienlastenausgleich</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11564-6">
    <skos:altLabel xml:lang="de">Bundeskindergeldgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11564-6">
    <skos:altLabel xml:lang="de">AFDC</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11591-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12872-4">
    <skos:related rdf:resource="descriptor/10831-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12872-4">
    <skos:related rdf:resource="descriptor/16187-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12872-4">
    <skos:related rdf:resource="descriptor/18592-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14308-2">
    <skos:prefLabel xml:lang="en">Silver</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14308-2">
    <skos:prefLabel xml:lang="de">Silber</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12489-3">
    <skos:altLabel xml:lang="en">Value management</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12489-3">
    <skos:altLabel xml:lang="de">Wertmanagement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12489-3">
    <skos:altLabel xml:lang="de">Funktionswertanalyse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18916-1">
    <skos:altLabel xml:lang="en">Hobby</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18916-1">
    <skos:altLabel xml:lang="de">Freizeitgestaltung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18916-1">
    <skos:altLabel xml:lang="de">Freizeitbeschäftigung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15920-4">
    <skos:altLabel xml:lang="de">Frau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10564-4">
    <skos:prefLabel xml:lang="en">Infant industry protection</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10564-4">
    <skos:prefLabel xml:lang="de">Erziehungszoll</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10048-1">
    <skos:narrower rdf:resource="descriptor/10068-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19736-5">
    <skos:broader rdf:resource="thsys/73388"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19736-5">
    <skos:broader rdf:resource="descriptor/15640-3"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id89">
    <rdfs:seeAlso rdf:resource="descriptor/10542-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10830-2">
    <skos:broader rdf:resource="thsys/73334"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10830-2">
    <skos:broader rdf:resource="thsys/71075"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10830-2">
    <skos:broader rdf:resource="thsys/71073"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10830-2">
    <skos:broader rdf:resource="thsys/71040"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10830-2">
    <skos:broader rdf:resource="descriptor/11709-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10830-2">
    <skos:broader rdf:resource="descriptor/16112-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16266-3">
    <skos:altLabel xml:lang="en">Community participation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18740-5">
    <skos:related rdf:resource="descriptor/18186-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18740-5">
    <skos:related rdf:resource="descriptor/15763-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10650-4">
    <skos:prefLabel xml:lang="en">Customs union theory</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10650-4">
    <skos:prefLabel xml:lang="de">Zollunionstheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10397-4">
    <skos:prefLabel xml:lang="en">Price stability</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10397-4">
    <skos:prefLabel xml:lang="de">Preisniveaustabilität</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10005-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11367-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id243">
    <rdfs:seeAlso rdf:resource="descriptor/19182-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19649-0">
    <skos:broader rdf:resource="thsys/73361"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19649-0">
    <skos:broader rdf:resource="thsys/70572"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19649-0">
    <skos:broader rdf:resource="descriptor/16017-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11090-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19712-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70160">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18877-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17987-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11189-6">
    <skos:prefLabel xml:lang="en">Employment discrimination</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11189-6">
    <skos:prefLabel xml:lang="de">Arbeitsmarktdiskriminierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10412-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14110-2">
    <skos:altLabel xml:lang="de">Steinobst</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14110-2">
    <skos:altLabel xml:lang="de">Schalenobst</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14110-2">
    <skos:altLabel xml:lang="de">Pflaume</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14110-2">
    <skos:altLabel xml:lang="de">Mandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14110-2">
    <skos:altLabel xml:lang="de">Kirsche</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14110-2">
    <skos:altLabel xml:lang="de">Frucht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14110-2">
    <skos:altLabel xml:lang="de">Frischobst</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14110-2">
    <skos:altLabel xml:lang="de">Esskastanie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14110-2">
    <skos:altLabel xml:lang="de">Eßkastanie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14110-2">
    <skos:altLabel xml:lang="de">Beerenobst</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17742-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19630-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17758-5">
    <skos:altLabel xml:lang="de">Amerikanische Staaten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17758-5">
    <skos:altLabel xml:lang="de">Amerikanische Länder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13992-3">
    <skos:broader rdf:resource="thsys/71052"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13992-3">
    <skos:broader rdf:resource="thsys/71051"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13992-3">
    <skos:broader rdf:resource="thsys/70140"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13992-3">
    <skos:broader rdf:resource="descriptor/15909-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11220-3">
    <skos:prefLabel xml:lang="en">Labour market disequilibrium</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11220-3">
    <skos:prefLabel xml:lang="de">Arbeitsmarktungleichgewicht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16012-6">
    <skos:related rdf:resource="descriptor/18643-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16012-6">
    <skos:related rdf:resource="descriptor/16517-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16012-6">
    <skos:related rdf:resource="descriptor/18939-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16012-6">
    <skos:related rdf:resource="descriptor/13360-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16012-6">
    <skos:related rdf:resource="descriptor/13665-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16012-6">
    <skos:related rdf:resource="descriptor/13655-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19267-0">
    <skos:altLabel xml:lang="en">Term structure</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19267-0">
    <skos:altLabel xml:lang="en">Interest rate spread</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19267-0">
    <skos:altLabel xml:lang="de">Fristigkeitsstruktur der Zinssätze</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11680-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19659-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70341">
    <skos:prefLabel xml:lang="de">P.01.01  Pflanzenbauprodukte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10315-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70935">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.12.01.03.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19847-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73369">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.04.03.05</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10545-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70238">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.12.01.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71112">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.03.05</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13110-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13058-4">
    <skos:related rdf:resource="descriptor/12925-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13058-4">
    <skos:related rdf:resource="descriptor/19186-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13058-4">
    <skos:related rdf:resource="descriptor/18412-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13058-4">
    <skos:related rdf:resource="descriptor/14336-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13058-4">
    <skos:related rdf:resource="descriptor/12916-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13058-4">
    <skos:related rdf:resource="descriptor/13047-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13058-4">
    <skos:related rdf:resource="descriptor/14346-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13058-4">
    <skos:related rdf:resource="descriptor/12914-0"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id20">
    <rdf:value xml:lang="de">Für die Beziehungen im Betrieb</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18852-1">
    <skos:related rdf:resource="descriptor/18854-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18852-1">
    <skos:related rdf:resource="descriptor/11313-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70961">
    <rdfs:label xml:lang="en">Motor vehicle trade</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70961">
    <rdfs:label xml:lang="de">Kraftfahrzeughandel</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19567-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13998-5">
    <skos:altLabel xml:lang="en">Veterans</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13998-5">
    <skos:altLabel xml:lang="de">Veteranen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13998-5">
    <skos:altLabel xml:lang="de">Soldatinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14927-2">
    <skos:broader rdf:resource="thsys/70498"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14927-2">
    <skos:broader rdf:resource="descriptor/14931-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14377-4">
    <skos:related rdf:resource="descriptor/14385-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14377-4">
    <skos:related rdf:resource="descriptor/13106-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14377-4">
    <skos:related rdf:resource="descriptor/14150-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15938-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19094-5">
    <skos:related rdf:resource="descriptor/19081-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19094-5">
    <skos:related rdf:resource="descriptor/10015-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19094-5">
    <skos:related rdf:resource="descriptor/12278-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19094-5">
    <skos:related rdf:resource="descriptor/12543-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13456-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19534-3">
    <skos:prefLabel xml:lang="en">Duration</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19534-3">
    <skos:prefLabel xml:lang="de">Dauer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18718-5">
    <skos:related rdf:resource="descriptor/18712-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18718-5">
    <skos:related rdf:resource="descriptor/24665-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18718-5">
    <skos:related rdf:resource="descriptor/13747-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70471">
    <rdfs:label xml:lang="en">Computer-aided manufacturing</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70471">
    <rdfs:label xml:lang="de">Automatisierte Produktion</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10951-4">
    <skos:related rdf:resource="descriptor/10964-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10951-4">
    <skos:related rdf:resource="descriptor/10942-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10951-4">
    <skos:related rdf:resource="descriptor/10909-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10951-4">
    <skos:related rdf:resource="descriptor/10960-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10951-4">
    <skos:related rdf:resource="descriptor/12793-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10951-4">
    <skos:related rdf:resource="descriptor/10690-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14710-6">
    <skos:related rdf:resource="descriptor/18078-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18758-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10011-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18890-0">
    <skos:broader rdf:resource="thsys/71053"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18890-0">
    <skos:broader rdf:resource="thsys/70839"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18890-0">
    <skos:broader rdf:resource="descriptor/14014-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13039-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10864-6">
    <skos:altLabel xml:lang="en">Merchant shipper</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10864-6">
    <skos:altLabel xml:lang="de">Indirekter Export</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10864-6">
    <skos:altLabel xml:lang="de">Indirekter Außenhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10864-6">
    <skos:altLabel xml:lang="de">Indirekte Ausfuhr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10864-6">
    <skos:altLabel xml:lang="de">Exportgroßhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10864-6">
    <skos:altLabel xml:lang="de">Exporteur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10864-6">
    <skos:altLabel xml:lang="de">Ausfuhrhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10864-6">
    <skos:altLabel xml:lang="de">Außengroßhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18379-1">
    <skos:prefLabel xml:lang="en">Water policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18379-1">
    <skos:prefLabel xml:lang="de">Wasserpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15901-1">
    <skos:prefLabel xml:lang="en">Storm</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15901-1">
    <skos:prefLabel xml:lang="de">Sturm</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19155-4">
    <skos:altLabel xml:lang="de">Workflow-Managementsystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19155-4">
    <skos:altLabel xml:lang="de">Geschäftsprozessmanagement-System</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19155-4">
    <skos:altLabel xml:lang="de">Geschäftsprozeßmanagement-System</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14043-5">
    <skos:broader rdf:resource="thsys/70201"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14043-5">
    <skos:broader rdf:resource="descriptor/18277-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70354">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12094-5">
    <skos:narrower rdf:resource="descriptor/24661-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13733-6">
    <skos:broader rdf:resource="thsys/71106"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13733-6">
    <skos:broader rdf:resource="descriptor/12232-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18006-5">
    <skos:altLabel xml:lang="en">Fast-food-restaurant</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18006-5">
    <skos:altLabel xml:lang="de">Schnellrestaurant</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18006-5">
    <skos:altLabel xml:lang="de">Schnellimbiss</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18006-5">
    <skos:altLabel xml:lang="de">Schnellimbiß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18006-5">
    <skos:altLabel xml:lang="de">Imbisshalle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18006-5">
    <skos:altLabel xml:lang="de">Imbißhalle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10610-2">
    <skos:altLabel xml:lang="en">ERP (European Recovery Program)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10610-2">
    <skos:altLabel xml:lang="de">Marshallplan</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10610-2">
    <skos:altLabel xml:lang="de">Marshall-Plan</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10610-2">
    <skos:altLabel xml:lang="de">Europäisches Wiederaufbauprogramm</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19807-1">
    <skos:altLabel xml:lang="de">Datenmustererkennung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17801-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19685-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12497-4">
    <skos:prefLabel xml:lang="en">Cost-type accounting</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12497-4">
    <skos:prefLabel xml:lang="de">Kostenartenrechnung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15682-1">
    <skos:related rdf:resource="descriptor/13399-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15682-1">
    <skos:related rdf:resource="descriptor/16719-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15682-1">
    <skos:related rdf:resource="descriptor/15787-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15682-1">
    <skos:related rdf:resource="descriptor/15821-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15682-1">
    <skos:related rdf:resource="descriptor/15654-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15682-1">
    <skos:related rdf:resource="descriptor/16405-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18835-1">
    <skos:narrower rdf:resource="descriptor/19448-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18835-1">
    <skos:narrower rdf:resource="descriptor/18836-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12295-2">
    <skos:broader rdf:resource="thsys/71042"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12295-2">
    <skos:broader rdf:resource="thsys/70036"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12295-2">
    <skos:broader rdf:resource="descriptor/12053-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10139-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13050-6">
    <skos:broader rdf:resource="thsys/71003"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13050-6">
    <skos:broader rdf:resource="descriptor/18399-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10164-6">
    <skos:altLabel xml:lang="en">Multiple choice</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10164-6">
    <skos:altLabel xml:lang="en">Binary choice</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10164-6">
    <skos:altLabel xml:lang="de">Wahlhandlungstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10164-6">
    <skos:altLabel xml:lang="de">Veränderliche Präferenzen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10164-6">
    <skos:altLabel xml:lang="de">Stochastische Präferenzen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10164-6">
    <skos:altLabel xml:lang="de">Präferenzordnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70086">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">G.04.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19139-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13053-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id1">
    <rdf:value xml:lang="de">Für die Abfallentsorgungsbranche</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15045-3">
    <skos:narrower rdf:resource="descriptor/15060-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15045-3">
    <skos:narrower rdf:resource="descriptor/13823-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15045-3">
    <skos:narrower rdf:resource="descriptor/15458-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15045-3">
    <skos:narrower rdf:resource="descriptor/15057-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15045-3">
    <skos:narrower rdf:resource="descriptor/19591-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15045-3">
    <skos:narrower rdf:resource="descriptor/15054-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15045-3">
    <skos:narrower rdf:resource="descriptor/15050-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15045-3">
    <skos:narrower rdf:resource="descriptor/15055-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15045-3">
    <skos:narrower rdf:resource="descriptor/11742-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15045-3">
    <skos:narrower rdf:resource="descriptor/15038-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15045-3">
    <skos:narrower rdf:resource="descriptor/11738-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15045-3">
    <skos:narrower rdf:resource="descriptor/15036-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15045-3">
    <skos:narrower rdf:resource="descriptor/12207-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15045-3">
    <skos:narrower rdf:resource="descriptor/15033-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15045-3">
    <skos:narrower rdf:resource="descriptor/15026-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14549-1">
    <skos:prefLabel xml:lang="en">Packaging metal</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14549-1">
    <skos:prefLabel xml:lang="de">Metallverpackung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19772-1">
    <skos:related rdf:resource="descriptor/15718-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19772-1">
    <skos:related rdf:resource="descriptor/16623-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19772-1">
    <skos:related rdf:resource="descriptor/19599-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12203-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12755-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18455-4">
    <skos:prefLabel xml:lang="en">Commodity classification</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18455-4">
    <skos:prefLabel xml:lang="de">Produktklassifikation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24661-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19271-2">
    <skos:related rdf:resource="descriptor/19274-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19271-2">
    <skos:related rdf:resource="descriptor/10210-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19271-2">
    <skos:related rdf:resource="descriptor/12204-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19271-2">
    <skos:related rdf:resource="descriptor/19280-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19271-2">
    <skos:related rdf:resource="descriptor/19269-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13618-6">
    <skos:prefLabel xml:lang="en">Long-haul freight traffic</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13618-6">
    <skos:prefLabel xml:lang="de">Güterfernverkehr</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15735-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16188-4">
    <skos:related rdf:resource="descriptor/19714-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16188-4">
    <skos:related rdf:resource="descriptor/16187-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16188-4">
    <skos:related rdf:resource="descriptor/16003-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16188-4">
    <skos:related rdf:resource="descriptor/19851-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16188-4">
    <skos:related rdf:resource="descriptor/19819-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19115-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14169-4">
    <skos:narrower rdf:resource="descriptor/14170-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14169-4">
    <skos:narrower rdf:resource="descriptor/19022-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14169-4">
    <skos:narrower rdf:resource="descriptor/14145-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14169-4">
    <skos:narrower rdf:resource="descriptor/14160-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14169-4">
    <skos:narrower rdf:resource="descriptor/19015-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14169-4">
    <skos:narrower rdf:resource="descriptor/14131-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18586-3">
    <skos:broader rdf:resource="thsys/70934"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18586-3">
    <skos:broader rdf:resource="descriptor/13252-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18586-3">
    <skos:broader rdf:resource="descriptor/18621-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71081">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18100-6">
    <skos:prefLabel xml:lang="en">Investment goods marketing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18100-6">
    <skos:prefLabel xml:lang="de">Investitionsgütermarketing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14933-0">
    <skos:related rdf:resource="descriptor/18619-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14933-0">
    <skos:related rdf:resource="descriptor/14927-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71065">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13753-0">
    <skos:related rdf:resource="descriptor/19267-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13753-0">
    <skos:related rdf:resource="descriptor/12208-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13753-0">
    <skos:related rdf:resource="descriptor/18719-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13176-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19669-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16400-4">
    <skos:related rdf:resource="descriptor/16424-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16400-4">
    <skos:related rdf:resource="descriptor/16404-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16400-4">
    <skos:related rdf:resource="descriptor/16416-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16400-4">
    <skos:related rdf:resource="descriptor/15648-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16400-4">
    <skos:related rdf:resource="descriptor/15923-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16400-4">
    <skos:related rdf:resource="descriptor/16249-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10635-0">
    <skos:altLabel xml:lang="en">Trade diversion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10635-0">
    <skos:altLabel xml:lang="en">Trade creation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10635-0">
    <skos:altLabel xml:lang="de">Handelsschaffender Effekt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10635-0">
    <skos:altLabel xml:lang="de">Außenhandelsumlenkung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10635-0">
    <skos:altLabel xml:lang="de">Außenhandelsschaffung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14160-1">
    <skos:narrower rdf:resource="descriptor/19023-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14160-1">
    <skos:narrower rdf:resource="descriptor/19019-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14160-1">
    <skos:narrower rdf:resource="descriptor/14158-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11904-1">
    <skos:related rdf:resource="descriptor/12070-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11904-1">
    <skos:related rdf:resource="descriptor/11696-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11904-1">
    <skos:related rdf:resource="descriptor/18160-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11904-1">
    <skos:related rdf:resource="descriptor/11701-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11904-1">
    <skos:related rdf:resource="descriptor/11981-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11904-1">
    <skos:related rdf:resource="descriptor/11940-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11904-1">
    <skos:related rdf:resource="descriptor/11905-6"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id255">
    <rdfs:seeAlso rdf:resource="descriptor/13274-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19226-0">
    <skos:broader rdf:resource="thsys/71119"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19226-0">
    <skos:broader rdf:resource="thsys/70242"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19226-0">
    <skos:broader rdf:resource="descriptor/10224-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19226-0">
    <skos:broader rdf:resource="descriptor/10223-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14222-5">
    <skos:prefLabel xml:lang="en">Uranium</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14222-5">
    <skos:prefLabel xml:lang="de">Uran</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18155-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14664-1">
    <skos:narrower rdf:resource="descriptor/14212-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15267-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12520-0">
    <skos:broader rdf:resource="thsys/70578"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12520-0">
    <skos:broader rdf:resource="descriptor/12521-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16087-3">
    <skos:altLabel xml:lang="de">Oberste Steuerrechtsprechung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16087-3">
    <skos:altLabel xml:lang="de">Bundesfinanzhof-Rechtsprechung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16087-3">
    <skos:altLabel xml:lang="de">BFH-Rechtsprechung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10627-6">
    <skos:broader rdf:resource="thsys/71093"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10627-6">
    <skos:broader rdf:resource="descriptor/10626-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10627-6">
    <skos:broader rdf:resource="descriptor/16278-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70933">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.12.01.03.03</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18259-4">
    <skos:related rdf:resource="descriptor/14728-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18259-4">
    <skos:related rdf:resource="descriptor/13002-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18259-4">
    <skos:related rdf:resource="descriptor/13193-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18259-4">
    <skos:related rdf:resource="descriptor/16004-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18259-4">
    <skos:related rdf:resource="descriptor/15992-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19192-5">
    <skos:related rdf:resource="descriptor/18374-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19192-5">
    <skos:related rdf:resource="descriptor/12990-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19192-5">
    <skos:related rdf:resource="descriptor/14200-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11443-4">
    <skos:broader rdf:resource="thsys/70306"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11443-4">
    <skos:broader rdf:resource="thsys/70141"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11443-4">
    <skos:broader rdf:resource="descriptor/10236-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14066-0">
    <skos:altLabel xml:lang="de">Kopra</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14066-0">
    <skos:altLabel xml:lang="de">Kokosnuß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15478-0">
    <skos:altLabel xml:lang="de">Risikolust</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19989-4">
    <skos:related rdf:resource="descriptor/16732-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19989-4">
    <skos:related rdf:resource="descriptor/16730-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19989-4">
    <skos:related rdf:resource="descriptor/20022-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19989-4">
    <skos:related rdf:resource="descriptor/11066-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14731-5">
    <skos:prefLabel xml:lang="en">Veterinary medicinal product</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14731-5">
    <skos:prefLabel xml:lang="de">Tierarzneimittel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19245-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13969-5">
    <skos:altLabel xml:lang="de">Versicherungskaufleute</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13969-5">
    <skos:altLabel xml:lang="de">Versicherungsfachwirtinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13969-5">
    <skos:altLabel xml:lang="de">Versicherungsfachwirte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10285-1">
    <skos:broader rdf:resource="thsys/71112"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10285-1">
    <skos:broader rdf:resource="descriptor/10020-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12376-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10381-5">
    <skos:broader rdf:resource="thsys/71082"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10381-5">
    <skos:broader rdf:resource="thsys/70020"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10381-5">
    <skos:broader rdf:resource="descriptor/11728-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15258-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10773-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19331-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16650-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20506-5">
    <skos:broader rdf:resource="thsys/72186"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20506-5">
    <skos:broader rdf:resource="descriptor/15328-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19052-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15010-1">
    <skos:prefLabel xml:lang="en">Waste gas</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15010-1">
    <skos:prefLabel xml:lang="de">Abgas</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11975-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16545-6">
    <skos:related rdf:resource="descriptor/11779-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16545-6">
    <skos:related rdf:resource="descriptor/18905-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16545-6">
    <skos:related rdf:resource="descriptor/13419-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16545-6">
    <skos:related rdf:resource="descriptor/11570-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16545-6">
    <skos:related rdf:resource="descriptor/16318-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16545-6">
    <skos:related rdf:resource="descriptor/15725-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16545-6">
    <skos:related rdf:resource="descriptor/16159-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16545-6">
    <skos:related rdf:resource="descriptor/15723-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16545-6">
    <skos:related rdf:resource="descriptor/16543-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16545-6">
    <skos:related rdf:resource="descriptor/11558-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16545-6">
    <skos:related rdf:resource="descriptor/12603-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16545-6">
    <skos:related rdf:resource="descriptor/16479-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16545-6">
    <skos:related rdf:resource="descriptor/16588-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16545-6">
    <skos:related rdf:resource="descriptor/16124-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19146-5">
    <skos:altLabel xml:lang="de">Ideenfindung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13703-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18024-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15104-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14196-1">
    <skos:altLabel xml:lang="de">Erdölgas</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70818">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11030-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19745-4">
    <skos:narrower rdf:resource="descriptor/19008-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13131-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14107-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15690-2">
    <zbwext:useInsteadNote rdf:nodeID="id120"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18305-2">
    <skos:prefLabel xml:lang="en">Beverage packaging</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18305-2">
    <skos:prefLabel xml:lang="de">Getränkeverpackung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20125-3">
    <skos:altLabel xml:lang="de">Ökologische Unternehmensberatung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20125-3">
    <skos:altLabel xml:lang="de">Ökologische Beratung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18334-2">
    <skos:altLabel xml:lang="en">Food programme</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18334-2">
    <skos:altLabel xml:lang="en">Food policy</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18334-2">
    <skos:altLabel xml:lang="de">Nahrungssicherungspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18334-2">
    <skos:altLabel xml:lang="de">Ernährungsprogramm</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14776-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19701-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18910-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11728-2">
    <skos:prefLabel xml:lang="en">Fiscal policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11728-2">
    <skos:prefLabel xml:lang="de">Finanzpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18643-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18892-3">
    <skos:related rdf:resource="descriptor/15658-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18892-3">
    <skos:related rdf:resource="descriptor/19787-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18892-3">
    <skos:related rdf:resource="descriptor/13371-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18892-3">
    <skos:related rdf:resource="descriptor/16683-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18892-3">
    <skos:related rdf:resource="descriptor/18444-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18892-3">
    <skos:related rdf:resource="descriptor/16479-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18892-3">
    <skos:related rdf:resource="descriptor/14926-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18892-3">
    <skos:related rdf:resource="descriptor/16635-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18892-3">
    <skos:related rdf:resource="descriptor/13320-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18892-3">
    <skos:related rdf:resource="descriptor/18334-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18455-4">
    <skos:scopeNote xml:lang="de">Nur für Literatur über Produktklassifikationen</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18745-2">
    <skos:narrower rdf:resource="descriptor/12019-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12090-6">
    <skos:broader rdf:resource="thsys/70485"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15672-4">
    <skos:related rdf:resource="descriptor/18859-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15672-4">
    <skos:related rdf:resource="descriptor/19764-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15672-4">
    <skos:related rdf:resource="descriptor/19788-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15672-4">
    <skos:related rdf:resource="descriptor/16672-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15672-4">
    <skos:related rdf:resource="descriptor/15941-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15672-4">
    <skos:related rdf:resource="descriptor/15939-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15672-4">
    <skos:related rdf:resource="descriptor/14724-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16288-0">
    <skos:altLabel xml:lang="de">Militärischer Konflikt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16288-0">
    <skos:altLabel xml:lang="de">Internationaler Konflikt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16288-0">
    <skos:altLabel xml:lang="de">Innerstaatlicher Konflikt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16288-0">
    <skos:altLabel xml:lang="de">Bewaffneter Zwischenfall</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16288-0">
    <skos:altLabel xml:lang="de">Bewaffneter Konflikt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13341-2">
    <skos:narrower rdf:resource="descriptor/18940-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12876-3">
    <skos:related rdf:resource="descriptor/11679-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12876-3">
    <skos:related rdf:resource="descriptor/11709-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12876-3">
    <skos:related rdf:resource="descriptor/11704-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12876-3">
    <skos:related rdf:resource="descriptor/19281-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12876-3">
    <skos:related rdf:resource="descriptor/12880-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12876-3">
    <skos:related rdf:resource="descriptor/12396-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12876-3">
    <skos:related rdf:resource="descriptor/12889-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12876-3">
    <skos:related rdf:resource="descriptor/11636-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12876-3">
    <skos:related rdf:resource="descriptor/12276-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12876-3">
    <skos:related rdf:resource="descriptor/18302-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12876-3">
    <skos:related rdf:resource="descriptor/12217-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12876-3">
    <skos:related rdf:resource="descriptor/12848-1"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id241">
    <rdf:value xml:lang="de">Für makroökonomische Theoriemodelle</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14611-1">
    <skos:prefLabel xml:lang="en">Food additive</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14611-1">
    <skos:prefLabel xml:lang="de">Lebensmittelchemikalie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16110-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11940-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13799-4">
    <skos:altLabel xml:lang="en">Collection agency</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13799-4">
    <skos:altLabel xml:lang="de">Inkassounternehmen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13799-4">
    <skos:altLabel xml:lang="de">Inkassobüro</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10478-4">
    <skos:broader rdf:resource="thsys/71109"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12010-2">
    <skos:narrower rdf:resource="descriptor/18000-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12010-2">
    <skos:narrower rdf:resource="descriptor/18361-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12010-2">
    <skos:narrower rdf:resource="descriptor/15731-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19265-4">
    <skos:prefLabel xml:lang="en">Effective rate</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19265-4">
    <skos:prefLabel xml:lang="de">Effektivzins</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11752-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16635-5">
    <skos:broader rdf:resource="thsys/73347"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16635-5">
    <skos:broader rdf:resource="thsys/70839"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16635-5">
    <skos:broader rdf:resource="thsys/70573"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13618-6">
    <skos:related rdf:resource="descriptor/18614-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16672-6">
    <skos:prefLabel xml:lang="en">Prostitution</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16672-6">
    <skos:prefLabel xml:lang="de">Prostitution</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18411-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18689-0">
    <skos:broader rdf:resource="thsys/71106"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18689-0">
    <skos:broader rdf:resource="thsys/71104"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18689-0">
    <skos:broader rdf:resource="descriptor/13722-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19089-5">
    <skos:prefLabel xml:lang="en">Capital lock-up</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19089-5">
    <skos:prefLabel xml:lang="de">Kapitalbindung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13028-6">
    <skos:related rdf:resource="descriptor/14268-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13028-6">
    <skos:related rdf:resource="descriptor/14378-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19268-5">
    <skos:prefLabel xml:lang="en">Prohibition of interest</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19268-5">
    <skos:prefLabel xml:lang="de">Zinsverbot</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14945-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16972-1">
    <skos:altLabel xml:lang="x-other">Svizzra</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16972-1">
    <skos:altLabel xml:lang="x-other">Svizzera</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16972-1">
    <skos:altLabel xml:lang="x-other">Suisse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16972-1">
    <skos:altLabel xml:lang="x-other">Confoederatio Helvetica</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16972-1">
    <skos:altLabel xml:lang="de">Schweizerische Eidgenossenschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11278-0">
    <skos:broader rdf:resource="thsys/70051"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11278-0">
    <skos:broader rdf:resource="descriptor/11266-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11278-0">
    <skos:broader rdf:resource="descriptor/11264-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16527-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18688-2">
    <skos:broader rdf:resource="thsys/71104"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18688-2">
    <skos:broader rdf:resource="thsys/71103"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18688-2">
    <skos:broader rdf:resource="thsys/71088"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18688-2">
    <skos:broader rdf:resource="descriptor/13735-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18688-2">
    <skos:broader rdf:resource="descriptor/13720-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16738-2">
    <skos:prefLabel xml:lang="en">Lausanne tradition</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16738-2">
    <skos:prefLabel xml:lang="de">Lausanner Schule</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16437-2">
    <skos:broader rdf:resource="thsys/73365"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16437-2">
    <skos:broader rdf:resource="descriptor/19708-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10120-5">
    <skos:narrower rdf:resource="descriptor/10115-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10120-5">
    <skos:narrower rdf:resource="descriptor/12274-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10120-5">
    <skos:narrower rdf:resource="descriptor/12273-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15054-2">
    <zbwext:useInsteadNote rdf:nodeID="id232"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12812-1">
    <skos:related rdf:resource="descriptor/13282-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12812-1">
    <skos:related rdf:resource="descriptor/13466-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12812-1">
    <skos:related rdf:resource="descriptor/18612-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12812-1">
    <skos:related rdf:resource="descriptor/19120-2"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id27">
    <rdf:value xml:lang="de">für die internationale Arbeitsteilung</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70575">
    <skos:narrower rdf:resource="descriptor/11782-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70575">
    <skos:narrower rdf:resource="descriptor/11790-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70575">
    <skos:narrower rdf:resource="descriptor/10981-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70575">
    <skos:narrower rdf:resource="descriptor/18309-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70575">
    <skos:narrower rdf:resource="descriptor/18308-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70575">
    <skos:narrower rdf:resource="descriptor/12064-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70575">
    <skos:narrower rdf:resource="descriptor/18310-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70575">
    <skos:narrower rdf:resource="descriptor/10049-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70575">
    <skos:narrower rdf:resource="descriptor/10905-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70575">
    <skos:narrower rdf:resource="descriptor/10501-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70575">
    <skos:narrower rdf:resource="descriptor/10521-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70575">
    <skos:narrower rdf:resource="descriptor/11792-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70575">
    <skos:narrower rdf:resource="descriptor/10337-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70575">
    <skos:narrower rdf:resource="descriptor/18456-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70575">
    <skos:narrower rdf:resource="descriptor/18010-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70575">
    <skos:narrower rdf:resource="descriptor/11784-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18542-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15765-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70346">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14603-0">
    <skos:related rdf:resource="descriptor/19623-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14603-0">
    <skos:related rdf:resource="descriptor/14171-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14603-0">
    <skos:related rdf:resource="descriptor/14662-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14603-0">
    <skos:related rdf:resource="descriptor/14664-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14603-0">
    <skos:related rdf:resource="descriptor/15010-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15069-3">
    <skos:related rdf:resource="descriptor/19236-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15069-3">
    <skos:related rdf:resource="descriptor/16639-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15069-3">
    <skos:related rdf:resource="descriptor/15088-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15069-3">
    <skos:related rdf:resource="descriptor/11432-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15069-3">
    <skos:related rdf:resource="descriptor/11032-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15069-3">
    <skos:related rdf:resource="descriptor/19506-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11558-1">
    <skos:narrower rdf:resource="descriptor/11570-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11535-6">
    <skos:prefLabel xml:lang="en">Fiscal constitution</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11535-6">
    <skos:prefLabel xml:lang="de">Finanzverfassung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12062-4">
    <skos:narrower rdf:resource="descriptor/10806-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12062-4">
    <skos:narrower rdf:resource="descriptor/19520-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12062-4">
    <skos:narrower rdf:resource="descriptor/18001-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15745-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11709-6">
    <skos:altLabel xml:lang="de">Steuerumgehung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11709-6">
    <skos:altLabel xml:lang="de">Steuereinholung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18030-1">
    <skos:scopeNote xml:lang="de">Ein Störfall liegt vor, wenn aus einer technischen Anlage Stoffe unbeabsichtigt in die Umwelt freigesetzt werden (oder die Gefahr dazu besteht) und dadurch eine Gemeingefährdung für die Umwelt entsteht.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19508-4">
    <skos:related rdf:resource="descriptor/19507-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19508-4">
    <skos:related rdf:resource="descriptor/19372-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19508-4">
    <skos:related rdf:resource="descriptor/11243-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19422-0">
    <skos:prefLabel xml:lang="en">Specific excise taxation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19422-0">
    <skos:prefLabel xml:lang="de">Spezielle Verbrauchsteuer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19136-1">
    <skos:prefLabel xml:lang="en">Point of sale</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19136-1">
    <skos:prefLabel xml:lang="de">Point of Sale</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73362">
    <skos:prefLabel xml:lang="de">N.04.04.04  Europäische Integration und EU-Politik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70819">
    <rdfs:label xml:lang="en">Sports</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70819">
    <rdfs:label xml:lang="de">Sport</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13874-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14199-2">
    <skos:altLabel xml:lang="de">Flußspat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14199-2">
    <skos:altLabel xml:lang="de">Fluorit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14199-2">
    <skos:altLabel xml:lang="de">Calciumfluorid</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10180-1">
    <skos:related rdf:resource="descriptor/10215-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10180-1">
    <skos:related rdf:resource="descriptor/10182-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15330-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13562-0">
    <skos:related rdf:resource="descriptor/13561-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13562-0">
    <skos:related rdf:resource="descriptor/13176-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14973-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13248-6">
    <skos:scopeNote xml:lang="de">Sowohl für die Vermietung von Leihwagen als auch für das Leasing von Kraftfahrzeugen</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10773-2">
    <skos:related rdf:resource="descriptor/10678-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10773-2">
    <skos:related rdf:resource="descriptor/19335-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10773-2">
    <skos:related rdf:resource="descriptor/10821-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10773-2">
    <skos:related rdf:resource="descriptor/10722-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10773-2">
    <skos:related rdf:resource="descriptor/18679-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10773-2">
    <skos:related rdf:resource="descriptor/11427-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10773-2">
    <skos:related rdf:resource="descriptor/10781-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10773-2">
    <skos:related rdf:resource="descriptor/10779-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10773-2">
    <skos:related rdf:resource="descriptor/13757-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10773-2">
    <skos:related rdf:resource="descriptor/10776-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10773-2">
    <skos:related rdf:resource="descriptor/13715-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10773-2">
    <skos:related rdf:resource="descriptor/10001-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18788-5">
    <skos:prefLabel xml:lang="en">Designers</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18788-5">
    <skos:prefLabel xml:lang="de">Designer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18031-6">
    <skos:broader rdf:resource="thsys/71109"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18031-6">
    <skos:broader rdf:resource="thsys/71069"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18031-6">
    <skos:broader rdf:resource="thsys/70879"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18031-6">
    <skos:broader rdf:resource="descriptor/19291-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18031-6">
    <skos:broader rdf:resource="descriptor/10437-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19760-1">
    <skos:broader rdf:resource="thsys/73357"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19215-5">
    <skos:altLabel xml:lang="en">Theory of contract</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19215-5">
    <skos:altLabel xml:lang="en">Contract-theory</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19215-5">
    <skos:altLabel xml:lang="en">Contracting theory</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19215-5">
    <skos:altLabel xml:lang="de">Kontrakttheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16533-6">
    <skos:broader rdf:resource="thsys/73370"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16533-6">
    <skos:broader rdf:resource="thsys/71085"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16533-6">
    <skos:broader rdf:resource="thsys/70833"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16533-6">
    <skos:broader rdf:resource="thsys/70324"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16533-6">
    <skos:broader rdf:resource="descriptor/10987-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16533-6">
    <skos:broader rdf:resource="descriptor/16545-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14897-6">
    <skos:related rdf:resource="descriptor/13144-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70581">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13881-5">
    <skos:prefLabel xml:lang="en">Mining professions</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13881-5">
    <skos:prefLabel xml:lang="de">Bergbauberufe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17745-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12277-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13971-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18311-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10738-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14731-5">
    <skos:related rdf:resource="descriptor/13374-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14731-5">
    <skos:related rdf:resource="descriptor/18901-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14731-5">
    <skos:related rdf:resource="descriptor/14169-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14731-5">
    <skos:related rdf:resource="descriptor/14989-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14731-5">
    <skos:related rdf:resource="descriptor/14596-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19810-5">
    <skos:prefLabel xml:lang="en">User access control</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19810-5">
    <skos:prefLabel xml:lang="de">Zutrittskontrolle</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10613-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10763-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19819-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16195-0">
    <skos:prefLabel xml:lang="en">Delinquency</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16195-0">
    <skos:prefLabel xml:lang="de">Kriminalität</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14947-3">
    <skos:broader rdf:resource="thsys/70342"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14947-3">
    <skos:broader rdf:resource="descriptor/14948-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14947-3">
    <skos:broader rdf:resource="descriptor/14930-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16962-4">
    <skos:prefLabel xml:lang="en">Glarus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16962-4">
    <skos:prefLabel xml:lang="de">Glarus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10029-5">
    <skos:broader rdf:resource="thsys/70852"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10029-5">
    <skos:broader rdf:resource="thsys/70101"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10029-5">
    <skos:broader rdf:resource="thsys/70038"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10029-5">
    <skos:broader rdf:resource="descriptor/11164-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16303-2">
    <skos:broader rdf:resource="thsys/73375"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19533-5">
    <skos:related rdf:resource="descriptor/11039-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19533-5">
    <skos:related rdf:resource="descriptor/11702-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19533-5">
    <skos:related rdf:resource="descriptor/15860-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19533-5">
    <skos:related rdf:resource="descriptor/15801-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19533-5">
    <skos:related rdf:resource="descriptor/11052-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19533-5">
    <skos:related rdf:resource="descriptor/19433-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19533-5">
    <skos:related rdf:resource="descriptor/11153-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17590-3">
    <skos:broader rdf:resource="thsys/70327"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17590-3">
    <skos:broader rdf:resource="descriptor/17589-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17590-3">
    <skos:broader rdf:resource="descriptor/17979-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14846-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70859">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.22.02.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14142-3">
    <skos:broader rdf:resource="thsys/70403"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14142-3">
    <skos:broader rdf:resource="descriptor/14145-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12268-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10947-2">
    <skos:prefLabel xml:lang="en">Competitor imitation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10947-2">
    <skos:prefLabel xml:lang="de">Imitationswettbewerb</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16651-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15914-6">
    <skos:broader rdf:resource="thsys/70142"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11059-5">
    <skos:related rdf:resource="descriptor/16318-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11059-5">
    <skos:related rdf:resource="descriptor/10977-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11059-5">
    <skos:related rdf:resource="descriptor/11043-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10964-2">
    <skos:broader rdf:resource="thsys/73391"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10964-2">
    <skos:broader rdf:resource="thsys/71071"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10964-2">
    <skos:broader rdf:resource="descriptor/16042-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15981-5">
    <skos:broader rdf:resource="thsys/70160"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15981-5">
    <skos:broader rdf:resource="descriptor/16007-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16597-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18910-6">
    <skos:broader rdf:resource="thsys/70833"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18910-6">
    <skos:broader rdf:resource="thsys/70324"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18910-6">
    <skos:broader rdf:resource="descriptor/16542-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12849-6">
    <skos:narrower rdf:resource="descriptor/24703-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18587-1">
    <skos:altLabel xml:lang="de">Seetouristik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13716-6">
    <skos:broader rdf:resource="thsys/71055"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13716-6">
    <skos:broader rdf:resource="thsys/70893"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13716-6">
    <skos:broader rdf:resource="descriptor/13967-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11682-0">
    <skos:altLabel xml:lang="en">Net worth tax</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11682-0">
    <skos:altLabel xml:lang="de">VStG (Vermögensteuergesetz)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11682-0">
    <skos:altLabel xml:lang="de">Vermögensteuerrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11682-0">
    <skos:altLabel xml:lang="de">Vermögensteuergesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11682-0">
    <skos:altLabel xml:lang="de">Vermögensbesteuerung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15458-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15699-5">
    <skos:prefLabel xml:lang="en">Prehistory</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15699-5">
    <skos:prefLabel xml:lang="de">Vorgeschichte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19575-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10065-1">
    <skos:altLabel xml:lang="de">Zwei-Faktoren-Modell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10065-1">
    <skos:altLabel xml:lang="de">Ökonomisches Modell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10065-1">
    <skos:altLabel xml:lang="de">N-Faktoren-Modell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10065-1">
    <skos:altLabel xml:lang="de">Mehr-Faktoren-Modell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id260">
    <rdfs:seeAlso rdf:resource="descriptor/10652-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18426-4">
    <zbwext:useInsteadNote rdf:nodeID="id83"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18426-4">
    <zbwext:useInsteadNote rdf:nodeID="id82"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18708-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10012-1">
    <skos:broader rdf:resource="thsys/70175"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10012-1">
    <skos:broader rdf:resource="thsys/70041"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12189-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19053-5">
    <skos:prefLabel xml:lang="en">Disability vehicle</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19053-5">
    <skos:prefLabel xml:lang="de">Behindertenfahrzeug</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11321-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70927">
    <skos:prefLabel xml:lang="en">W.12.05.01  Transport Management Science</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70927">
    <skos:prefLabel xml:lang="de">W.12.05.01  Verkehrsbetriebslehre</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12168-2">
    <skos:broader rdf:resource="thsys/71117"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12168-2">
    <skos:broader rdf:resource="thsys/70823"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12168-2">
    <skos:broader rdf:resource="descriptor/11039-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12168-2">
    <skos:broader rdf:resource="descriptor/10141-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10661-6">
    <skos:broader rdf:resource="thsys/71100"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10661-6">
    <skos:broader rdf:resource="descriptor/10030-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71081">
    <rdfs:label xml:lang="en">Public Budget</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71081">
    <rdfs:label xml:lang="de">Öffentlicher Haushalt</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11716-2">
    <skos:narrower rdf:resource="descriptor/12012-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11716-2">
    <skos:narrower rdf:resource="descriptor/12013-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11716-2">
    <skos:narrower rdf:resource="descriptor/18663-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11716-2">
    <skos:narrower rdf:resource="descriptor/18803-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11716-2">
    <skos:narrower rdf:resource="descriptor/12010-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11716-2">
    <skos:narrower rdf:resource="descriptor/19463-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11716-2">
    <skos:narrower rdf:resource="descriptor/18800-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10597-3">
    <skos:broader rdf:resource="thsys/73366"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10597-3">
    <skos:broader rdf:resource="thsys/71084"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10597-3">
    <skos:broader rdf:resource="thsys/70179"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10597-3">
    <skos:broader rdf:resource="descriptor/16441-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17985-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13744-1">
    <skos:narrower rdf:resource="descriptor/18693-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13744-1">
    <skos:narrower rdf:resource="descriptor/18687-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19119-1">
    <skos:related rdf:resource="descriptor/24781-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19119-1">
    <skos:related rdf:resource="descriptor/19137-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19553-6">
    <skos:prefLabel xml:lang="en">Low income</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19553-6">
    <skos:prefLabel xml:lang="de">Niedrigeinkommen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71102">
    <skos:broader rdf:resource="thsys/70171"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19656-3">
    <skos:altLabel xml:lang="en">International assignment experience</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19656-3">
    <skos:altLabel xml:lang="en">International assignment</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19656-3">
    <skos:altLabel xml:lang="de">Auslandspraktikum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19656-3">
    <skos:altLabel xml:lang="de">Auslandserfahrung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19656-3">
    <skos:altLabel xml:lang="de">Auslandsentsendung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73368">
    <skos:broader rdf:resource="thsys/73374"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19469-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id137">
    <rdf:value xml:lang="de">Für volkswirtschaftliche Ansätze</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11555-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14041-2">
    <skos:narrower rdf:resource="descriptor/14380-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19759-0">
    <skos:broader rdf:resource="thsys/73357"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19759-0">
    <skos:broader rdf:resource="descriptor/15938-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11578-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17758-5">
    <skos:related rdf:resource="descriptor/19719-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16048-6">
    <skos:narrower rdf:resource="descriptor/18734-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16048-6">
    <skos:narrower rdf:resource="descriptor/18703-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20506-5">
    <skos:altLabel xml:lang="de">Mittelwert-Ergodizität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11092-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16007-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20485-1">
    <skos:related rdf:resource="descriptor/18083-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11904-1">
    <skos:altLabel xml:lang="de">Standortwahl des Betriebs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11904-1">
    <skos:altLabel xml:lang="de">Betrieblicher Standortwechsel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11904-1">
    <skos:altLabel xml:lang="de">Betriebliche Standortplanung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11904-1">
    <skos:altLabel xml:lang="de">Betriebliche Standortentscheidung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13498-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70862">
    <skos:broader rdf:resource="thsys/70871"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19509-2">
    <skos:broader rdf:resource="thsys/73366"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19509-2">
    <skos:broader rdf:resource="thsys/73336"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19509-2">
    <skos:broader rdf:resource="thsys/73332"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19509-2">
    <skos:broader rdf:resource="thsys/70833"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19509-2">
    <skos:broader rdf:resource="thsys/70324"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19509-2">
    <skos:broader rdf:resource="descriptor/16545-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19509-2">
    <skos:broader rdf:resource="descriptor/19812-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19509-2">
    <skos:broader rdf:resource="descriptor/16124-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18159-1">
    <skos:related rdf:resource="descriptor/11302-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18159-1">
    <skos:related rdf:resource="descriptor/11311-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11832-0">
    <skos:related rdf:resource="descriptor/11833-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11832-0">
    <skos:related rdf:resource="descriptor/13593-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11832-0">
    <skos:related rdf:resource="descriptor/16266-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70870">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.22.01.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10987-4">
    <skos:broader rdf:resource="thsys/73372"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10987-4">
    <skos:broader rdf:resource="thsys/70038"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10987-4">
    <skos:broader rdf:resource="descriptor/16273-6"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id261">
    <rdfs:seeAlso rdf:resource="descriptor/13283-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16035-1">
    <skos:narrower rdf:resource="descriptor/19823-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16035-1">
    <skos:narrower rdf:resource="descriptor/12028-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16035-1">
    <skos:narrower rdf:resource="descriptor/11040-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11560-0">
    <skos:altLabel xml:lang="en">Defence expenditures</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11560-0">
    <skos:altLabel xml:lang="de">Wehretat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11560-0">
    <skos:altLabel xml:lang="de">Wehrbudget</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11560-0">
    <skos:altLabel xml:lang="de">Verteidigungsbudget</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11560-0">
    <skos:altLabel xml:lang="de">Verteidigungsausgaben</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11560-0">
    <skos:altLabel xml:lang="de">Stationierungskosten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11560-0">
    <skos:altLabel xml:lang="de">Rüstungsfinanzierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11560-0">
    <skos:altLabel xml:lang="de">Rüstungsausgaben</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11560-0">
    <skos:altLabel xml:lang="de">Militärhaushalt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11560-0">
    <skos:altLabel xml:lang="de">Militäretat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11560-0">
    <skos:altLabel xml:lang="de">Militärausgaben</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12079-1">
    <skos:narrower rdf:resource="descriptor/19832-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12079-1">
    <skos:narrower rdf:resource="descriptor/18767-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12079-1">
    <skos:narrower rdf:resource="descriptor/20341-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19410-0">
    <skos:broader rdf:resource="thsys/73337"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19410-0">
    <skos:broader rdf:resource="thsys/71076"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19383-5">
    <skos:scopeNote xml:lang="de">Nur für den Gesamtsektor Fischerei, Fischindustrie und Fischhandel.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15765-4">
    <skos:broader rdf:resource="thsys/73351"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15765-4">
    <skos:broader rdf:resource="thsys/73348"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15765-4">
    <skos:broader rdf:resource="thsys/70878"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15765-4">
    <skos:broader rdf:resource="thsys/70815"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15765-4">
    <skos:broader rdf:resource="descriptor/10436-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10961-1">
    <skos:broader rdf:resource="thsys/71071"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10961-1">
    <skos:broader rdf:resource="descriptor/10958-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73341">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.10.07.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13322-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19026-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15666-6">
    <skos:related rdf:resource="descriptor/15786-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15666-6">
    <skos:related rdf:resource="descriptor/15656-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15666-6">
    <skos:related rdf:resource="descriptor/11374-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15666-6">
    <skos:related rdf:resource="descriptor/18860-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15666-6">
    <skos:related rdf:resource="descriptor/11394-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10758-5">
    <skos:prefLabel xml:lang="en">Special drawing rights</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10758-5">
    <skos:prefLabel xml:lang="de">Sonderziehungsrechte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10438-2">
    <skos:prefLabel xml:lang="en">Investment policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10438-2">
    <skos:prefLabel xml:lang="de">Investitionspolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12998-3">
    <skos:broader rdf:resource="thsys/70994"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12998-3">
    <skos:broader rdf:resource="descriptor/12996-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14268-2">
    <skos:altLabel xml:lang="de">Stahlguss</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14268-2">
    <skos:altLabel xml:lang="de">Stahlguß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14268-2">
    <skos:altLabel xml:lang="de">NE-Metallguss</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14268-2">
    <skos:altLabel xml:lang="de">NE-Metallguß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14268-2">
    <skos:altLabel xml:lang="de">Metallguß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14268-2">
    <skos:altLabel xml:lang="de">Gussprodukt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14268-2">
    <skos:altLabel xml:lang="de">Gusseisen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14268-2">
    <skos:altLabel xml:lang="de">Gußprodukt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14268-2">
    <skos:altLabel xml:lang="de">Gußeisen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14268-2">
    <skos:altLabel xml:lang="de">Gießereiprodukt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14268-2">
    <skos:altLabel xml:lang="de">Eisenguss</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14268-2">
    <skos:altLabel xml:lang="de">Eisenguß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14567-6">
    <skos:narrower rdf:resource="descriptor/14339-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11061-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13215-0">
    <skos:prefLabel xml:lang="en">Wood trade</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13215-0">
    <skos:prefLabel xml:lang="de">Holzhandel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14082-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14323-6">
    <skos:related rdf:resource="descriptor/14270-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14323-6">
    <skos:related rdf:resource="descriptor/18500-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14323-6">
    <skos:related rdf:resource="descriptor/13245-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12738-1">
    <skos:prefLabel xml:lang="en">NC machinery</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12738-1">
    <skos:prefLabel xml:lang="de">NC-Werkzeugmaschine</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70017">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11454-6">
    <skos:prefLabel xml:lang="en">Cash accounting</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11454-6">
    <skos:prefLabel xml:lang="de">Kassenhaltungsansatz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18404-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19820-2">
    <skos:prefLabel xml:lang="en">Common law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19820-2">
    <skos:prefLabel xml:lang="de">Common Law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17716-0">
    <skos:broader rdf:resource="thsys/70560"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17716-0">
    <skos:broader rdf:resource="descriptor/17724-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11299-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17791-0">
    <skos:broader rdf:resource="thsys/70086"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17791-0">
    <skos:broader rdf:resource="descriptor/17758-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19458-0">
    <skos:narrower rdf:resource="descriptor/11890-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19784-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19175-5">
    <skos:broader rdf:resource="thsys/70307"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16023-1">
    <skos:prefLabel xml:lang="en">Water law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16023-1">
    <skos:prefLabel xml:lang="de">Wasserrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18364-0">
    <skos:narrower rdf:resource="descriptor/18376-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71030">
    <skos:prefLabel xml:lang="en">G.04.03  South America</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71030">
    <skos:prefLabel xml:lang="de">G.04.03  Südamerika</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70982">
    <skos:broader rdf:resource="thsys/71004"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10949-5">
    <skos:broader rdf:resource="thsys/70242"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10949-5">
    <skos:broader rdf:resource="descriptor/10012-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11832-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18449-6">
    <skos:narrower rdf:resource="descriptor/13369-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18449-6">
    <skos:narrower rdf:resource="descriptor/13368-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19086-4">
    <skos:prefLabel xml:lang="en">Bosnia and Herzegovina</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19086-4">
    <skos:prefLabel xml:lang="de">Bosnien-Herzegowina</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19491-2">
    <skos:narrower rdf:resource="descriptor/11212-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19491-2">
    <skos:narrower rdf:resource="descriptor/11203-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19491-2">
    <skos:narrower rdf:resource="descriptor/11197-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10698-4">
    <skos:narrower rdf:resource="descriptor/10581-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10698-4">
    <skos:narrower rdf:resource="descriptor/10593-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12971-2">
    <skos:prefLabel xml:lang="en">Fur farming</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12971-2">
    <skos:prefLabel xml:lang="de">Pelztierzucht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12565-6">
    <skos:broader rdf:resource="thsys/70244"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12565-6">
    <skos:broader rdf:resource="descriptor/12088-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12565-6">
    <skos:broader rdf:resource="descriptor/12567-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12628-1">
    <skos:prefLabel xml:lang="en">Shift work</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12628-1">
    <skos:prefLabel xml:lang="de">Schichtarbeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11653-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19416-2">
    <skos:altLabel xml:lang="de">Solidaritätszuschlag</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18848-6">
    <skos:related rdf:resource="descriptor/11337-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12106-3">
    <skos:broader rdf:resource="thsys/70878"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12106-3">
    <skos:broader rdf:resource="thsys/70562"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12106-3">
    <skos:broader rdf:resource="thsys/70011"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12106-3">
    <skos:broader rdf:resource="descriptor/16563-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12106-3">
    <skos:broader rdf:resource="descriptor/10436-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16080-3">
    <skos:broader rdf:resource="thsys/73391"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16080-3">
    <skos:broader rdf:resource="thsys/71107"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16080-3">
    <skos:broader rdf:resource="descriptor/16042-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11203-3">
    <skos:broader rdf:resource="thsys/70883"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11203-3">
    <skos:broader rdf:resource="thsys/70876"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11203-3">
    <skos:broader rdf:resource="thsys/70244"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11203-3">
    <skos:broader rdf:resource="thsys/70052"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11203-3">
    <skos:broader rdf:resource="descriptor/19491-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13849-1">
    <skos:altLabel xml:lang="de">Wohnwunsch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13849-1">
    <skos:altLabel xml:lang="de">Wohnversorgung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13849-1">
    <skos:altLabel xml:lang="de">Wohnverhältnisse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13849-1">
    <skos:altLabel xml:lang="de">Wohnstandard</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13849-1">
    <skos:altLabel xml:lang="de">Wohnsituation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13849-1">
    <skos:altLabel xml:lang="de">Wohnraumversorgung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13849-1">
    <skos:altLabel xml:lang="de">Wohnpräferenz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13849-1">
    <skos:altLabel xml:lang="de">Wohnliche Versorgung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13849-1">
    <skos:altLabel xml:lang="de">Wohnbedingungen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16615-4">
    <skos:related rdf:resource="descriptor/15810-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16615-4">
    <skos:related rdf:resource="descriptor/15716-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16615-4">
    <skos:related rdf:resource="descriptor/15714-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16615-4">
    <skos:related rdf:resource="descriptor/16513-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16615-4">
    <skos:related rdf:resource="descriptor/16512-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16615-4">
    <skos:related rdf:resource="descriptor/16661-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16615-4">
    <skos:related rdf:resource="descriptor/16486-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16615-4">
    <skos:related rdf:resource="descriptor/19639-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16615-4">
    <skos:related rdf:resource="descriptor/16587-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16615-4">
    <skos:related rdf:resource="descriptor/15919-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16615-4">
    <skos:related rdf:resource="descriptor/15918-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16615-4">
    <skos:related rdf:resource="descriptor/15952-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11252-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15943-6">
    <skos:prefLabel xml:lang="en">Birth rate</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15943-6">
    <skos:prefLabel xml:lang="de">Geburtenrate</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24858-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13692-1">
    <skos:altLabel xml:lang="en">Bank failure</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13692-1">
    <skos:altLabel xml:lang="de">Insolvenz von Banken</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13692-1">
    <skos:altLabel xml:lang="de">Bankkonkurs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13721-6">
    <skos:altLabel xml:lang="en">Fixed income securities</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18868-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13486-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12251-1">
    <skos:narrower rdf:resource="descriptor/13780-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12251-1">
    <skos:narrower rdf:resource="descriptor/12257-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12251-1">
    <skos:narrower rdf:resource="descriptor/11613-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12251-1">
    <skos:narrower rdf:resource="descriptor/12255-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12251-1">
    <skos:narrower rdf:resource="descriptor/12250-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12251-1">
    <skos:narrower rdf:resource="descriptor/10812-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12251-1">
    <skos:narrower rdf:resource="descriptor/19079-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12251-1">
    <skos:narrower rdf:resource="descriptor/10865-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12251-1">
    <skos:narrower rdf:resource="descriptor/13430-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15739-5">
    <skos:prefLabel xml:lang="en">Noise</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15739-5">
    <skos:prefLabel xml:lang="de">Lärm</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11678-5">
    <skos:narrower rdf:resource="descriptor/11649-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13784-3">
    <skos:altLabel xml:lang="de">Depositengeschäft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19412-3">
    <skos:broader rdf:resource="thsys/71074"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19412-3">
    <skos:broader rdf:resource="descriptor/11680-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19090-6">
    <skos:prefLabel xml:lang="en">Environmental accounting</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19090-6">
    <skos:prefLabel xml:lang="de">Umweltbilanz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17597-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11979-5">
    <skos:prefLabel xml:lang="en">Metropolitan area</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11979-5">
    <skos:prefLabel xml:lang="de">Ballungsraum</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19671-0">
    <skos:prefLabel xml:lang="en">Exchange listing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19671-0">
    <skos:prefLabel xml:lang="de">Börsennotierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17713-6">
    <skos:altLabel xml:lang="de">Islamische Republik Mauretanien</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19020-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12757-4">
    <skos:related rdf:resource="descriptor/12758-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12757-4">
    <skos:related rdf:resource="descriptor/12065-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12757-4">
    <skos:related rdf:resource="descriptor/12725-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12757-4">
    <skos:related rdf:resource="descriptor/12621-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18074-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12216-3">
    <skos:altLabel xml:lang="de">Jahresabschlussanalyse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12216-3">
    <skos:altLabel xml:lang="de">Jahresabschlußanalyse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12216-3">
    <skos:altLabel xml:lang="de">Interne Bilanzanalyse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12216-3">
    <skos:altLabel xml:lang="de">Externe Bilanzanalyse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12216-3">
    <skos:altLabel xml:lang="de">Bilanzkritik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15945-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19824-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16344-2">
    <skos:broader rdf:resource="thsys/73379"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16344-2">
    <skos:broader rdf:resource="descriptor/16340-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15903-4">
    <skos:broader rdf:resource="thsys/73353"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18917-6">
    <skos:broader rdf:resource="thsys/70829"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18917-6">
    <skos:broader rdf:resource="descriptor/10032-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18798-2">
    <skos:broader rdf:resource="thsys/70873"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18798-2">
    <skos:broader rdf:resource="descriptor/18101-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19072-1">
    <skos:prefLabel xml:lang="en">Statistical data</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19072-1">
    <skos:prefLabel xml:lang="de">Statistik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70198">
    <skos:narrower rdf:resource="descriptor/16809-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70198">
    <skos:narrower rdf:resource="descriptor/17972-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70198">
    <skos:narrower rdf:resource="descriptor/16808-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70198">
    <skos:narrower rdf:resource="descriptor/17968-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70198">
    <skos:narrower rdf:resource="descriptor/17967-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70198">
    <skos:narrower rdf:resource="descriptor/17964-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70198">
    <skos:narrower rdf:resource="descriptor/17962-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70198">
    <skos:narrower rdf:resource="descriptor/17959-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70198">
    <skos:narrower rdf:resource="descriptor/17954-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70198">
    <skos:narrower rdf:resource="descriptor/16806-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70198">
    <skos:narrower rdf:resource="descriptor/16805-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19675-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19537-4">
    <skos:prefLabel xml:lang="en">Beta Estimate</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19537-4">
    <skos:prefLabel xml:lang="de">Beta-Faktor</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16684-6">
    <skos:prefLabel xml:lang="en">Military sociology</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16684-6">
    <skos:prefLabel xml:lang="de">Militärsoziologie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12185-2">
    <skos:altLabel xml:lang="en">General meeting</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19600-2">
    <skos:altLabel xml:lang="de">Satire</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19600-2">
    <skos:altLabel xml:lang="de">Lachen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16329-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19703-6">
    <skos:altLabel xml:lang="de">Verfassungsentwicklung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19703-6">
    <skos:altLabel xml:lang="de">Verfassungsänderung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19013-3">
    <skos:prefLabel xml:lang="en">Mental disease</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19013-3">
    <skos:prefLabel xml:lang="de">Psychische Störung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19083-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70847">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10501-0">
    <skos:scopeNote xml:lang="de">Keine einheitliche Definition. Kann z.B. Kleingewerbe, Subsistenzwirtschaft, illegale Aktivitäten oder Schattenwirtschaft umfassen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11156-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21282-3">
    <skos:broader rdf:resource="thsys/70485"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21282-3">
    <skos:broader rdf:resource="thsys/70101"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21282-3">
    <skos:broader rdf:resource="descriptor/19565-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10020-2">
    <skos:broader rdf:resource="thsys/71112"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10313-1">
    <skos:related rdf:resource="descriptor/10327-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10313-1">
    <skos:related rdf:resource="descriptor/10326-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10313-1">
    <skos:related rdf:resource="descriptor/10315-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16774-5">
    <skos:broader rdf:resource="thsys/73342"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16774-5">
    <skos:broader rdf:resource="thsys/71037"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16774-5">
    <skos:broader rdf:resource="thsys/70880"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16774-5">
    <skos:broader rdf:resource="descriptor/15598-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10135-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15777-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11142-4">
    <skos:prefLabel xml:lang="en">Socialist enterprise</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11142-4">
    <skos:prefLabel xml:lang="de">Sozialistisches Unternehmen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15102-3">
    <skos:related rdf:resource="descriptor/10006-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19902-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14831-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16853-2">
    <skos:related rdf:resource="descriptor/16927-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19343-3">
    <skos:prefLabel xml:lang="en">Cattle market</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19343-3">
    <skos:prefLabel xml:lang="de">Rindermarkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12312-0">
    <skos:broader rdf:resource="thsys/71042"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14972-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17687-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14439-1">
    <skos:related rdf:resource="descriptor/13250-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12766-3">
    <skos:narrower rdf:resource="descriptor/19007-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12766-3">
    <skos:narrower rdf:resource="descriptor/18835-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12766-3">
    <skos:narrower rdf:resource="descriptor/12770-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12766-3">
    <skos:narrower rdf:resource="descriptor/12768-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12766-3">
    <skos:narrower rdf:resource="descriptor/18395-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12766-3">
    <skos:narrower rdf:resource="descriptor/18100-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12766-3">
    <skos:narrower rdf:resource="descriptor/10876-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12766-3">
    <skos:narrower rdf:resource="descriptor/12780-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12766-3">
    <skos:narrower rdf:resource="descriptor/18101-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13020-1">
    <skos:broader rdf:resource="thsys/71008"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13020-1">
    <skos:broader rdf:resource="descriptor/12993-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13020-1">
    <skos:broader rdf:resource="descriptor/13019-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15938-6">
    <skos:altLabel xml:lang="de">Verwandtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15938-6">
    <skos:altLabel xml:lang="de">Verwandte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15938-6">
    <skos:altLabel xml:lang="de">Kleinfamilie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15938-6">
    <skos:altLabel xml:lang="de">Kinderreiche Familie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15938-6">
    <skos:altLabel xml:lang="de">Großfamilie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15938-6">
    <skos:altLabel xml:lang="de">Familienstruktur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15938-6">
    <skos:altLabel xml:lang="de">Familiengröße</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18944-3">
    <skos:related rdf:resource="descriptor/19475-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18944-3">
    <skos:related rdf:resource="descriptor/15961-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18944-3">
    <skos:related rdf:resource="descriptor/16604-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13738-3">
    <skos:related rdf:resource="descriptor/13709-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13738-3">
    <skos:related rdf:resource="descriptor/11487-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13738-3">
    <skos:related rdf:resource="descriptor/13721-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13738-3">
    <skos:related rdf:resource="descriptor/12234-1"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id134">
    <rdf:value xml:lang="de">Für andere Verkehrsträger</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10344-4">
    <skos:broader rdf:resource="thsys/70180"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10332-4">
    <skos:broader rdf:resource="thsys/70233"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19356-1">
    <skos:altLabel xml:lang="en">Import demand function</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19356-1">
    <skos:altLabel xml:lang="de">Importnachfragefunktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11919-2">
    <skos:altLabel xml:lang="en">Agglomeration economies</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11919-2">
    <skos:altLabel xml:lang="de">Ballungsvorteil</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11919-2">
    <skos:altLabel xml:lang="de">Ballungseffekt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11919-2">
    <skos:altLabel xml:lang="de">Agglomerationsvorteil</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11919-2">
    <skos:altLabel xml:lang="de">Agglomerationsnachteil</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17978-5">
    <skos:prefLabel xml:lang="en">Arab countries</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17978-5">
    <skos:prefLabel xml:lang="de">Arabische Staaten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11504-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14200-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11222-6">
    <skos:scopeNote xml:lang="de">Individuelle oder gesamtwirtschaftliche Investitionen in Bildung und Bildungseinrichtungen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12675-6">
    <skos:prefLabel xml:lang="en">Warehouse</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12675-6">
    <skos:prefLabel xml:lang="de">Lager</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16336-1">
    <skos:altLabel xml:lang="de">Verwaltungsinnovation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70956">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16134-6">
    <skos:prefLabel xml:lang="en">Vocational training law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16134-6">
    <skos:prefLabel xml:lang="de">Berufsbildungsrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14052-4">
    <skos:related rdf:resource="descriptor/12979-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14052-4">
    <skos:related rdf:resource="descriptor/18267-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14052-4">
    <skos:related rdf:resource="descriptor/13327-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11433-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71042">
    <skos:broader rdf:resource="thsys/70186"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14039-3">
    <skos:broader rdf:resource="thsys/71015"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14039-3">
    <skos:broader rdf:resource="descriptor/12705-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14039-3">
    <skos:broader rdf:resource="descriptor/12974-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19287-1">
    <skos:prefLabel xml:lang="en">Free-money theory (S. Gesell)</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19287-1">
    <skos:prefLabel xml:lang="de">Freiwirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19454-1">
    <skos:broader rdf:resource="thsys/71066"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19454-1">
    <skos:broader rdf:resource="descriptor/12031-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12051-2">
    <skos:altLabel xml:lang="de">Industriebetriebswirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12051-2">
    <skos:altLabel xml:lang="de">IBL (Industriebetriebslehre)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12051-2">
    <skos:altLabel xml:lang="de">Betriebswirtschaftslehre der Industrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73363">
    <rdfs:label xml:lang="en">Armed forces</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73363">
    <rdfs:label xml:lang="de">Militär</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14142-3">
    <skos:related rdf:resource="descriptor/18220-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14142-3">
    <skos:related rdf:resource="descriptor/19344-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14142-3">
    <skos:related rdf:resource="descriptor/19343-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14142-3">
    <skos:related rdf:resource="descriptor/12950-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14983-6">
    <skos:broader rdf:resource="thsys/73300"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14983-6">
    <skos:broader rdf:resource="descriptor/14972-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10581-4">
    <skos:broader rdf:resource="thsys/73332"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10581-4">
    <skos:broader rdf:resource="thsys/71084"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10581-4">
    <skos:broader rdf:resource="descriptor/10698-4"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id144">
    <rdfs:seeAlso rdf:resource="descriptor/16269-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71113">
    <skos:broader rdf:resource="thsys/71115"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12884-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17175-1">
    <skos:prefLabel xml:lang="en">Bulgaria</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17175-1">
    <skos:prefLabel xml:lang="de">Bulgarien</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70986">
    <skos:broader rdf:resource="thsys/71004"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18199-3">
    <skos:altLabel xml:lang="en">Cocaine</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18199-3">
    <skos:altLabel xml:lang="de">Kokain</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15671-6">
    <skos:related rdf:resource="descriptor/15678-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15671-6">
    <skos:related rdf:resource="descriptor/16641-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15671-6">
    <skos:related rdf:resource="descriptor/19731-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15671-6">
    <skos:related rdf:resource="descriptor/13399-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15671-6">
    <skos:related rdf:resource="descriptor/16719-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15671-6">
    <skos:related rdf:resource="descriptor/19706-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15671-6">
    <skos:related rdf:resource="descriptor/14012-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71101">
    <skos:broader rdf:resource="thsys/70171"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15873-1">
    <skos:scopeNote xml:lang="de">Geowissenschaftliche Disziplin von der Entstehung und Entwicklung der Erde; Analyse von Zusammensetzung und Veränderungen der Erdkruste.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14957-0">
    <skos:related rdf:resource="descriptor/14961-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14957-0">
    <skos:related rdf:resource="descriptor/14955-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13001-5">
    <skos:narrower rdf:resource="descriptor/19426-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13001-5">
    <skos:narrower rdf:resource="descriptor/19427-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18078-1">
    <skos:altLabel xml:lang="en">High-temperature plastics</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18078-1">
    <skos:altLabel xml:lang="de">Thermoplastische Elastomere</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18078-1">
    <skos:altLabel xml:lang="de">Thermoplaste</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18078-1">
    <skos:altLabel xml:lang="de">Spezialkunststoff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18078-1">
    <skos:altLabel xml:lang="de">Korrosionsbeständiger Kunststoff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18078-1">
    <skos:altLabel xml:lang="de">Hochtemperaturkunststoff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18078-1">
    <skos:altLabel xml:lang="de">Hochleistungskunststoff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18078-1">
    <skos:altLabel xml:lang="de">Hitzebeständiger Kunststoff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19043-1">
    <skos:prefLabel xml:lang="en">Associate</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19043-1">
    <skos:prefLabel xml:lang="de">Gesellschafter</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15126-3">
    <skos:altLabel xml:lang="en">Cross-sectional analysis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#ConceptScheme">
    <rdf:type rdf:resource="http://www.w3.org/2002/07/owl#Class"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11060-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71024">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15751-1">
    <skos:altLabel xml:lang="en">Exhaustible resources</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15751-1">
    <skos:altLabel xml:lang="en">Depletable resources</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15751-1">
    <skos:altLabel xml:lang="de">Nicht-regenerierbarer Rohstoff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15751-1">
    <skos:altLabel xml:lang="de">Nicht-regenerierbare Ressourcen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15751-1">
    <skos:altLabel xml:lang="de">Nicht-erneuerbare Ressourcen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15751-1">
    <skos:altLabel xml:lang="de">Erschöpfbarer Rohstoff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19428-2">
    <skos:narrower rdf:resource="descriptor/20972-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11729-0">
    <skos:prefLabel xml:lang="en">Fiscal reform</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11729-0">
    <skos:prefLabel xml:lang="de">Finanzreform</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19381-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73305">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70013">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.06.04</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15730-2">
    <skos:prefLabel xml:lang="en">Sewerage charge</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15730-2">
    <skos:prefLabel xml:lang="de">Abwasserabgabe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11211-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19141-1">
    <skos:scopeNote xml:lang="de"> Nur für Kunden/innen als Adressaten betrieblicher Marketingmaßnahmen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13233-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19529-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11696-3">
    <skos:broader rdf:resource="thsys/71076"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11696-3">
    <skos:broader rdf:resource="thsys/70046"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11696-3">
    <skos:broader rdf:resource="descriptor/11694-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18588-6">
    <skos:prefLabel xml:lang="en">Liner shipping</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18588-6">
    <skos:prefLabel xml:lang="de">Linienschifffahrt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70811">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.12.03</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11751-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13617-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12365-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10438-2">
    <skos:scopeNote xml:lang="de">Nur für staatliche Investitionspolitik.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18865-6">
    <skos:prefLabel xml:lang="en">Rural education</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18865-6">
    <skos:prefLabel xml:lang="de">Ländliches Bildungswesen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17799-5">
    <skos:prefLabel xml:lang="en">St. Kitts and Nevis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17799-5">
    <skos:prefLabel xml:lang="de">St. Kitts und Nevis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14735-4">
    <skos:scopeNote xml:lang="de">Nur für Produkte der dekorativen Kosmetik.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13144-4">
    <skos:related rdf:resource="descriptor/14897-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11314-1">
    <skos:broader rdf:resource="thsys/71051"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11314-1">
    <skos:broader rdf:resource="thsys/70851"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11314-1">
    <skos:broader rdf:resource="thsys/70235"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11314-1">
    <skos:broader rdf:resource="descriptor/15712-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18852-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10466-4">
    <skos:related rdf:resource="descriptor/10434-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10466-4">
    <skos:related rdf:resource="descriptor/10470-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10466-4">
    <skos:related rdf:resource="descriptor/16695-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10466-4">
    <skos:related rdf:resource="descriptor/10465-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10466-4">
    <skos:related rdf:resource="descriptor/10460-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10466-4">
    <skos:related rdf:resource="descriptor/10945-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10466-4">
    <skos:related rdf:resource="descriptor/19548-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10466-4">
    <skos:related rdf:resource="descriptor/18082-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10466-4">
    <skos:related rdf:resource="descriptor/10459-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10466-4">
    <skos:related rdf:resource="descriptor/10464-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10466-4">
    <skos:related rdf:resource="descriptor/10256-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18690-1">
    <skos:broader rdf:resource="thsys/71104"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18690-1">
    <skos:broader rdf:resource="thsys/71010"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18690-1">
    <skos:broader rdf:resource="descriptor/10221-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16102-5">
    <skos:narrower rdf:resource="descriptor/19315-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19630-0">
    <skos:broader rdf:resource="thsys/73351"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19630-0">
    <skos:broader rdf:resource="descriptor/19627-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13004-6">
    <skos:narrower rdf:resource="descriptor/13007-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71034">
    <skos:prefLabel xml:lang="de">B.01.05  Unternehmensentwicklung, Betriebsgröße und Standortwahl</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19773-6">
    <skos:altLabel xml:lang="en">Neighbourhood advocates</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19773-6">
    <skos:altLabel xml:lang="en">Neighborhood advisory council</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19773-6">
    <skos:altLabel xml:lang="en">Local action</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19773-6">
    <skos:altLabel xml:lang="en">Grassroots action</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19773-6">
    <skos:altLabel xml:lang="en">Grass roots</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19773-6">
    <skos:altLabel xml:lang="en">Community-based action</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19773-6">
    <skos:altLabel xml:lang="en">Community center</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19773-6">
    <skos:altLabel xml:lang="de">Soziale Gemeinschaftsarbeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18795-1">
    <skos:prefLabel xml:lang="en">Trade fair participation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18795-1">
    <skos:prefLabel xml:lang="de">Messebeteiligung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12564-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15072-0">
    <skos:broader rdf:resource="thsys/72393"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15072-0">
    <skos:broader rdf:resource="thsys/72183"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15072-0">
    <skos:broader rdf:resource="descriptor/15064-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18358-2">
    <skos:prefLabel xml:lang="en">Rural energy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18358-2">
    <skos:prefLabel xml:lang="de">Ländliche Energieversorgung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13995-4">
    <skos:broader rdf:resource="thsys/71055"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13995-4">
    <skos:broader rdf:resource="thsys/70154"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13995-4">
    <skos:broader rdf:resource="descriptor/18535-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16042-4">
    <skos:narrower rdf:resource="descriptor/10964-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16042-4">
    <skos:narrower rdf:resource="descriptor/16080-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16042-4">
    <skos:narrower rdf:resource="descriptor/13825-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16042-4">
    <skos:narrower rdf:resource="descriptor/16032-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16042-4">
    <skos:narrower rdf:resource="descriptor/18176-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16042-4">
    <skos:narrower rdf:resource="descriptor/13464-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16042-4">
    <skos:narrower rdf:resource="descriptor/16050-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16042-4">
    <skos:narrower rdf:resource="descriptor/16031-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16042-4">
    <skos:narrower rdf:resource="descriptor/16026-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12677-2">
    <skos:narrower rdf:resource="descriptor/19115-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13130-1">
    <skos:broader rdf:resource="thsys/70987"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13130-1">
    <skos:broader rdf:resource="descriptor/13145-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11189-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13701-5">
    <skos:broader rdf:resource="thsys/70903"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13701-5">
    <skos:broader rdf:resource="descriptor/12049-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13701-5">
    <skos:broader rdf:resource="descriptor/13283-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73295">
    <skos:narrower rdf:resource="descriptor/13152-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73295">
    <skos:narrower rdf:resource="descriptor/13157-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73295">
    <skos:narrower rdf:resource="descriptor/18221-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73295">
    <skos:narrower rdf:resource="descriptor/13151-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73295">
    <skos:narrower rdf:resource="descriptor/11646-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73295">
    <skos:narrower rdf:resource="descriptor/18992-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19768-6">
    <skos:prefLabel xml:lang="en">Justice</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19768-6">
    <skos:prefLabel xml:lang="de">Gerechtigkeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19750-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13459-0">
    <skos:prefLabel xml:lang="en">Commercial management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13459-0">
    <skos:prefLabel xml:lang="de">Handelsbetriebslehre</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12881-3">
    <skos:prefLabel xml:lang="en">Consultancy services</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12881-3">
    <skos:prefLabel xml:lang="de">Unternehmensberatung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11185-0">
    <skos:prefLabel xml:lang="en">Labour law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11185-0">
    <skos:prefLabel xml:lang="de">Arbeitsrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15415-3">
    <skos:broader rdf:resource="thsys/71035"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15415-3">
    <skos:broader rdf:resource="thsys/70269"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15415-3">
    <skos:broader rdf:resource="descriptor/15685-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11336-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70985">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.06.01.13</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11665-0">
    <skos:prefLabel xml:lang="en">Church tax</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11665-0">
    <skos:prefLabel xml:lang="de">Kirchensteuer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13643-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id295">
    <rdfs:seeAlso rdf:resource="descriptor/16510-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18406-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15060-0">
    <skos:related rdf:resource="descriptor/15211-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15060-0">
    <skos:related rdf:resource="descriptor/15458-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15060-0">
    <skos:related rdf:resource="descriptor/18171-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14778-0">
    <skos:prefLabel xml:lang="en">Glass packaging</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14778-0">
    <skos:prefLabel xml:lang="de">Glasverpackung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19570-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14408-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18958-6">
    <skos:narrower rdf:resource="descriptor/18957-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#hasTopConcept">
    <owl:inverseOf rdf:resource="http://www.w3.org/2004/02/skos/core#topConceptOf"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11194-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18762-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10204-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19564-1">
    <skos:prefLabel xml:lang="en">Product labelling</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19564-1">
    <skos:prefLabel xml:lang="de">Warenkennzeichnung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10695-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13649-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18797-4">
    <skos:broader rdf:resource="thsys/70873"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18630-5">
    <skos:altLabel xml:lang="de">Verkehrswegeplanung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18630-5">
    <skos:altLabel xml:lang="de">Verkehrswegebaupolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18630-5">
    <skos:altLabel xml:lang="de">Bundesverkehrswegeplan</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12038-1">
    <skos:broader rdf:resource="thsys/71034"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18670-0">
    <skos:broader rdf:resource="thsys/70905"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18670-0">
    <skos:broader rdf:resource="thsys/70187"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18670-0">
    <skos:broader rdf:resource="descriptor/11441-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14158-2">
    <skos:related rdf:resource="descriptor/19383-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14158-2">
    <skos:related rdf:resource="descriptor/19626-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14158-2">
    <skos:related rdf:resource="descriptor/14969-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14158-2">
    <skos:related rdf:resource="descriptor/19382-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14158-2">
    <skos:related rdf:resource="descriptor/18343-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71045">
    <rdfs:label xml:lang="de">Fremdkapitalbeschaffung</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19535-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13097-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70029">
    <skos:prefLabel xml:lang="de">B.06.04  Produktionsplanung und Fertigungssteuerung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10809-0">
    <skos:altLabel xml:lang="en">Chicago international money market</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14957-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19355-3">
    <skos:related rdf:resource="descriptor/10605-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19609-5">
    <skos:related rdf:resource="descriptor/10328-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13052-2">
    <skos:related rdf:resource="descriptor/13239-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13052-2">
    <skos:related rdf:resource="descriptor/12923-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13052-2">
    <skos:related rdf:resource="descriptor/18422-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13052-2">
    <skos:related rdf:resource="descriptor/13051-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13052-2">
    <skos:related rdf:resource="descriptor/13217-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13052-2">
    <skos:related rdf:resource="descriptor/14433-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18953-2">
    <skos:altLabel xml:lang="de">Profisport</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18953-2">
    <skos:altLabel xml:lang="de">Berufssport</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10656-6">
    <skos:narrower rdf:resource="descriptor/10843-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11141-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19235-6">
    <skos:broader rdf:resource="thsys/71113"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19235-6">
    <skos:broader rdf:resource="thsys/71085"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19235-6">
    <skos:broader rdf:resource="descriptor/11771-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12271-2">
    <skos:related rdf:resource="descriptor/12849-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12271-2">
    <skos:related rdf:resource="descriptor/12249-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12271-2">
    <skos:related rdf:resource="descriptor/12270-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12271-2">
    <skos:related rdf:resource="descriptor/12232-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15063-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14705-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70538">
    <skos:prefLabel xml:lang="de">P.04  Energie und Wasser</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70007">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12265-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10266-5">
    <skos:altLabel xml:lang="de">Sozialproduktsberechnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18495-6">
    <skos:prefLabel xml:lang="en">Structural engineering</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18495-6">
    <skos:prefLabel xml:lang="de">Bautechnik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10272-3">
    <skos:narrower rdf:resource="descriptor/10028-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10272-3">
    <skos:narrower rdf:resource="descriptor/10271-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10272-3">
    <skos:narrower rdf:resource="descriptor/10269-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10272-3">
    <skos:narrower rdf:resource="descriptor/10274-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10272-3">
    <skos:narrower rdf:resource="descriptor/10265-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10272-3">
    <skos:narrower rdf:resource="descriptor/19237-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19792-2">
    <skos:prefLabel xml:lang="en">Food contamination</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19792-2">
    <skos:prefLabel xml:lang="de">Nahrungsmittelbelastung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11888-1">
    <skos:narrower rdf:resource="descriptor/11996-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11888-1">
    <skos:narrower rdf:resource="descriptor/19647-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19414-6">
    <skos:related rdf:resource="descriptor/11710-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19414-6">
    <skos:related rdf:resource="descriptor/11649-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19414-6">
    <skos:related rdf:resource="descriptor/10827-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18272-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17017-5">
    <skos:broader rdf:resource="thsys/70131"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17017-5">
    <skos:broader rdf:resource="descriptor/17016-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19678-0">
    <skos:altLabel xml:lang="en">Open network provision</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19678-0">
    <skos:altLabel xml:lang="en">Network economics</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19678-0">
    <skos:altLabel xml:lang="en">Bypass competition</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19678-0">
    <skos:altLabel xml:lang="en">Access rules</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19678-0">
    <skos:altLabel xml:lang="en">Access pricing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19678-0">
    <skos:altLabel xml:lang="de">Ramsey-Zugangstarif</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19678-0">
    <skos:altLabel xml:lang="de">Offener Netzzugang</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19678-0">
    <skos:altLabel xml:lang="de">Netzzugangstarif</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19678-0">
    <skos:altLabel xml:lang="de">Netzökonomik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19678-0">
    <skos:altLabel xml:lang="de">Durchleitungsrechte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19678-0">
    <skos:altLabel xml:lang="de">Durchleitungsbedingungen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12820-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13508-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15839-1">
    <skos:broader rdf:resource="thsys/73385"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15839-1">
    <skos:broader rdf:resource="thsys/70244"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15839-1">
    <skos:broader rdf:resource="descriptor/15801-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15075-1">
    <skos:related rdf:resource="descriptor/10031-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15075-1">
    <skos:related rdf:resource="descriptor/10342-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15075-1">
    <skos:related rdf:resource="descriptor/10025-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15075-1">
    <skos:related rdf:resource="descriptor/10494-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15075-1">
    <skos:related rdf:resource="descriptor/12053-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18299-6">
    <skos:broader rdf:resource="thsys/71029"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18299-6">
    <skos:broader rdf:resource="descriptor/19301-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18299-6">
    <skos:broader rdf:resource="descriptor/12949-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14005-6">
    <skos:prefLabel xml:lang="en">Pharmacists</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14005-6">
    <skos:prefLabel xml:lang="de">Apotheker</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24733-2">
    <skos:prefLabel xml:lang="en">Going private</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24733-2">
    <skos:prefLabel xml:lang="de">Going Private</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14378-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19260-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19019-5">
    <skos:broader rdf:resource="thsys/70506"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19019-5">
    <skos:broader rdf:resource="thsys/70403"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19019-5">
    <skos:broader rdf:resource="descriptor/14160-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10714-4">
    <skos:prefLabel xml:lang="en">Preferential tariff</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10714-4">
    <skos:prefLabel xml:lang="de">Zollpräferenz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13514-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18268-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19349-5">
    <skos:broader rdf:resource="thsys/71025"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19349-5">
    <skos:broader rdf:resource="thsys/70960"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19349-5">
    <skos:broader rdf:resource="descriptor/13431-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18252-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70940">
    <skos:broader rdf:resource="thsys/70943"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id37">
    <rdfs:seeAlso rdf:resource="descriptor/18001-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12949-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16340-3">
    <skos:altLabel xml:lang="de">Volksvertretung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16340-3">
    <skos:altLabel xml:lang="de">Reichstag</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16340-3">
    <skos:altLabel xml:lang="de">Nationalversammlung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16340-3">
    <skos:altLabel xml:lang="de">Landtag</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16340-3">
    <skos:altLabel xml:lang="de">Kongreß (Parlament)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16340-3">
    <skos:altLabel xml:lang="de">Bürgerschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16340-3">
    <skos:altLabel xml:lang="de">Bundestag</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16340-3">
    <skos:altLabel xml:lang="de">Bundesparlament</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16445-3">
    <skos:altLabel xml:lang="en">Strategic arms limitation talks</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16445-3">
    <skos:altLabel xml:lang="en">SALT (Strategic Arms Limitation Talks)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16445-3">
    <skos:altLabel xml:lang="en">Mutual balanced forces reductions</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16445-3">
    <skos:altLabel xml:lang="en">MBFR</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16445-3">
    <skos:altLabel xml:lang="en">Arms reduction</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16445-3">
    <skos:altLabel xml:lang="en">Arms control</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16445-3">
    <skos:altLabel xml:lang="de">Truppenreduzierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16445-3">
    <skos:altLabel xml:lang="de">Rüstungssteuerung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16445-3">
    <skos:altLabel xml:lang="de">Rüstungskontrolle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16445-3">
    <skos:altLabel xml:lang="de">Rüstungsbeschränkung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16445-3">
    <skos:altLabel xml:lang="de">Demilitarisierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16445-3">
    <skos:altLabel xml:lang="de">Abrüstungspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/b">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15225-1">
    <skos:prefLabel xml:lang="en">Stochastic process</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15225-1">
    <skos:prefLabel xml:lang="de">Stochastischer Prozess</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10213-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15905-0">
    <skos:prefLabel xml:lang="en">Population age structure</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15905-0">
    <skos:prefLabel xml:lang="de">Altersstruktur der Bevölkerung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11852-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12016-4">
    <skos:related rdf:resource="descriptor/15903-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12016-4">
    <skos:related rdf:resource="descriptor/15779-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12016-4">
    <skos:related rdf:resource="descriptor/18071-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12016-4">
    <skos:related rdf:resource="descriptor/15755-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12016-4">
    <skos:related rdf:resource="descriptor/15743-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12016-4">
    <skos:related rdf:resource="descriptor/12021-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12016-4">
    <skos:related rdf:resource="descriptor/18327-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12016-4">
    <skos:related rdf:resource="descriptor/12015-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24787-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11563-1">
    <skos:altLabel xml:lang="de">Wohnungsbauprämie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11563-1">
    <skos:altLabel xml:lang="de">Bausparförderung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11563-1">
    <skos:altLabel xml:lang="de">Bausparfinanzierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19256-5">
    <skos:broader rdf:resource="thsys/70306"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19256-5">
    <skos:broader rdf:resource="descriptor/11449-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18661-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70979">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19398-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13867-6">
    <skos:related rdf:resource="descriptor/13849-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13867-6">
    <skos:related rdf:resource="descriptor/19532-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13867-6">
    <skos:related rdf:resource="descriptor/13861-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19837-6">
    <skos:altLabel xml:lang="de">Transformationsökonomien</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19837-6">
    <skos:altLabel xml:lang="de">Transformationsländer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19837-6">
    <skos:altLabel xml:lang="de">Ehemalige sozialistische Staaten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19837-6">
    <skos:altLabel xml:lang="de">Ehemalige sozialistische Länder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12039-6">
    <skos:related rdf:resource="descriptor/15076-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12039-6">
    <skos:related rdf:resource="descriptor/12069-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12039-6">
    <skos:related rdf:resource="descriptor/12884-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12039-6">
    <skos:related rdf:resource="descriptor/19069-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12039-6">
    <skos:related rdf:resource="descriptor/12165-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12039-6">
    <skos:related rdf:resource="descriptor/12053-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12039-6">
    <skos:related rdf:resource="descriptor/12164-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10969-6">
    <skos:altLabel xml:lang="en">International coordination of national economic policies</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10969-6">
    <skos:altLabel xml:lang="de">Internationale Koordination der Wirtschaftspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10969-6">
    <skos:altLabel xml:lang="de">Internationale Harmonisierung der Wirtschaftspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11311-0">
    <skos:broader rdf:resource="thsys/71054"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11311-0">
    <skos:broader rdf:resource="thsys/71051"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11311-0">
    <skos:broader rdf:resource="thsys/70235"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11311-0">
    <skos:broader rdf:resource="descriptor/13988-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15878-5">
    <skos:altLabel xml:lang="en">National boundary</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15878-5">
    <skos:altLabel xml:lang="de">Staatsgrenze</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11660-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15779-0">
    <skos:broader rdf:resource="thsys/73351"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15779-0">
    <skos:broader rdf:resource="thsys/71019"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15779-0">
    <skos:broader rdf:resource="thsys/70238"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16726-2">
    <skos:altLabel xml:lang="de">Positivismusstreit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12525-4">
    <skos:broader rdf:resource="thsys/70578"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12525-4">
    <skos:broader rdf:resource="descriptor/12480-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16201-3">
    <skos:broader rdf:resource="thsys/73334"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16201-3">
    <skos:broader rdf:resource="descriptor/16195-0"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id133">
    <rdfs:seeAlso rdf:resource="descriptor/12504-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13424-5">
    <skos:related rdf:resource="descriptor/11886-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13424-5">
    <skos:related rdf:resource="descriptor/16550-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13424-5">
    <skos:related rdf:resource="descriptor/13394-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16976-0">
    <skos:altLabel xml:lang="x-other">Ticino</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12591-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11052-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18286-1">
    <skos:prefLabel xml:lang="en">Renewable resources</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18286-1">
    <skos:prefLabel xml:lang="de">Erneuerbare Ressourcen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16517-4">
    <skos:broader rdf:resource="thsys/73369"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16517-4">
    <skos:broader rdf:resource="thsys/70824"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16517-4">
    <skos:broader rdf:resource="descriptor/16516-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12655-5">
    <skos:altLabel xml:lang="en">BOM (Bill of Materials)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10486-5">
    <skos:prefLabel xml:lang="en">Illiteracy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10486-5">
    <skos:prefLabel xml:lang="de">Analphabetismus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18489-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15981-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18054-1">
    <skos:broader rdf:resource="thsys/70307"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12136-1">
    <skos:broader rdf:resource="thsys/70562"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12136-1">
    <skos:broader rdf:resource="descriptor/19562-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17175-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12725-3">
    <skos:related rdf:resource="descriptor/12065-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12725-3">
    <skos:related rdf:resource="descriptor/12757-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12725-3">
    <skos:related rdf:resource="descriptor/12621-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12725-3">
    <skos:related rdf:resource="descriptor/13452-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12648-2">
    <skos:altLabel xml:lang="de">Materialwirtschaftslehre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12648-2">
    <skos:altLabel xml:lang="de">Material-Management</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12648-2">
    <skos:altLabel xml:lang="de">Betriebliche Materialwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72183">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id117">
    <rdf:value xml:lang="de">Für sozialistische Staaten</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11578-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10899-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19501-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18328-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16658-0">
    <skos:related rdf:resource="descriptor/16333-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16658-0">
    <skos:related rdf:resource="descriptor/16330-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16658-0">
    <skos:related rdf:resource="descriptor/11164-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16658-0">
    <skos:related rdf:resource="descriptor/11274-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16658-0">
    <skos:related rdf:resource="descriptor/15825-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16658-0">
    <skos:related rdf:resource="descriptor/15823-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16658-0">
    <skos:related rdf:resource="descriptor/11302-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16658-0">
    <skos:related rdf:resource="descriptor/11356-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16658-0">
    <skos:related rdf:resource="descriptor/19748-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14378-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10287-4">
    <skos:altLabel xml:lang="en">Investment in fixed assets</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10287-4">
    <skos:altLabel xml:lang="de">Ausrüstungsinvestition</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11653-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12714-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70845">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.24.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13502-4">
    <skos:broader rdf:resource="thsys/73389"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13502-4">
    <skos:broader rdf:resource="thsys/70943"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13502-4">
    <skos:broader rdf:resource="descriptor/13569-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15745-3">
    <skos:prefLabel xml:lang="en">Natural resources</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15745-3">
    <skos:prefLabel xml:lang="de">Natürliche Ressourcen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15463-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19364-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11741-3">
    <skos:broader rdf:resource="thsys/71113"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11741-3">
    <skos:broader rdf:resource="thsys/70278"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11741-3">
    <skos:broader rdf:resource="descriptor/10110-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11783-1">
    <skos:related rdf:resource="descriptor/12038-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12433-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11165-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10263-4">
    <skos:prefLabel xml:lang="en">Wealth effect</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10263-4">
    <skos:prefLabel xml:lang="de">Vermögenseffekt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12185-2">
    <skos:related rdf:resource="descriptor/12201-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12185-2">
    <skos:related rdf:resource="descriptor/18726-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12185-2">
    <skos:related rdf:resource="descriptor/19154-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12185-2">
    <skos:related rdf:resource="descriptor/13714-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12185-2">
    <skos:related rdf:resource="descriptor/12174-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12274-3">
    <skos:prefLabel xml:lang="en">Debt financing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12274-3">
    <skos:prefLabel xml:lang="de">Fremdkapital</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14269-0">
    <skos:narrower rdf:resource="descriptor/18287-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14269-0">
    <skos:narrower rdf:resource="descriptor/19051-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14269-0">
    <skos:narrower rdf:resource="descriptor/14323-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14269-0">
    <skos:narrower rdf:resource="descriptor/18243-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14269-0">
    <skos:narrower rdf:resource="descriptor/14268-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14269-0">
    <skos:narrower rdf:resource="descriptor/14272-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14269-0">
    <skos:narrower rdf:resource="descriptor/14548-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14269-0">
    <skos:narrower rdf:resource="descriptor/14318-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14269-0">
    <skos:narrower rdf:resource="descriptor/14261-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73380">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13731-3">
    <skos:prefLabel xml:lang="en">Financial regulation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13731-3">
    <skos:prefLabel xml:lang="de">Kapitalmarktrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14723-4">
    <skos:prefLabel xml:lang="en">Drug</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14723-4">
    <skos:prefLabel xml:lang="de">Droge</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11848-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18930-0">
    <skos:related rdf:resource="descriptor/12837-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18930-0">
    <skos:related rdf:resource="descriptor/12829-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18930-0">
    <skos:related rdf:resource="descriptor/18932-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18930-0">
    <skos:related rdf:resource="descriptor/12774-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18930-0">
    <skos:related rdf:resource="descriptor/13658-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18930-0">
    <skos:related rdf:resource="descriptor/16679-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16946-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17480-3">
    <skos:scopeNote xml:lang="de">Erklärte sich 1991 unabhängig von der Sowjetunion und trat der GUS bei</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12028-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11600-0">
    <skos:prefLabel xml:lang="en">Fiscal federalism</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11600-0">
    <skos:prefLabel xml:lang="de">Finanzföderalismus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12239-5">
    <skos:prefLabel xml:lang="en">Right</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12239-5">
    <skos:prefLabel xml:lang="de">Bezugsrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10876-6">
    <skos:altLabel xml:lang="de">Multinationales Marketing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10876-6">
    <skos:altLabel xml:lang="de">Internationaler Vertrieb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10876-6">
    <skos:altLabel xml:lang="de">Internationaler Absatz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10876-6">
    <skos:altLabel xml:lang="de">Interkulturelles Marketing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10876-6">
    <skos:altLabel xml:lang="de">Globalmarketing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10876-6">
    <skos:altLabel xml:lang="de">Globales Marketing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10876-6">
    <skos:altLabel xml:lang="de">Exportmarketing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73386">
    <skos:prefLabel xml:lang="en">N.07.01  Social psychology</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73386">
    <skos:prefLabel xml:lang="de">N.07.01  Sozialpsychologie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10699-2">
    <skos:broader rdf:resource="thsys/71098"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10699-2">
    <skos:broader rdf:resource="descriptor/10683-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11559-6">
    <skos:related rdf:resource="descriptor/13993-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11559-6">
    <skos:related rdf:resource="descriptor/13979-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11559-6">
    <skos:related rdf:resource="descriptor/13981-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11559-6">
    <skos:related rdf:resource="descriptor/18829-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11559-6">
    <skos:related rdf:resource="descriptor/11722-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11559-6">
    <skos:related rdf:resource="descriptor/11374-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11559-6">
    <skos:related rdf:resource="descriptor/11154-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11559-6">
    <skos:related rdf:resource="descriptor/11318-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12208-2">
    <skos:related rdf:resource="descriptor/13753-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12208-2">
    <skos:related rdf:resource="descriptor/10473-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12208-2">
    <skos:related rdf:resource="descriptor/15515-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12208-2">
    <skos:related rdf:resource="descriptor/19150-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12208-2">
    <skos:related rdf:resource="descriptor/10590-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12208-2">
    <skos:related rdf:resource="descriptor/10015-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12208-2">
    <skos:related rdf:resource="descriptor/11529-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12208-2">
    <skos:related rdf:resource="descriptor/10293-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12208-2">
    <skos:related rdf:resource="descriptor/12207-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12208-2">
    <skos:related rdf:resource="descriptor/18107-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12208-2">
    <skos:related rdf:resource="descriptor/12470-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12208-2">
    <skos:related rdf:resource="descriptor/19536-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12282-4">
    <skos:narrower rdf:resource="descriptor/11612-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12282-4">
    <skos:narrower rdf:resource="descriptor/12268-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12282-4">
    <skos:narrower rdf:resource="descriptor/11769-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18271-0">
    <skos:broader rdf:resource="thsys/73305"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18271-0">
    <skos:broader rdf:resource="thsys/70114"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18271-0">
    <skos:broader rdf:resource="descriptor/14752-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14293-3">
    <skos:altLabel xml:lang="de">Sekundäraluminium</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14293-3">
    <skos:altLabel xml:lang="de">Primäraluminium</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13975-3">
    <skos:broader rdf:resource="thsys/71055"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13975-3">
    <skos:broader rdf:resource="thsys/70944"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13975-3">
    <skos:broader rdf:resource="descriptor/18535-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14004-1">
    <skos:altLabel xml:lang="de">Verfasserinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14004-1">
    <skos:altLabel xml:lang="de">Verfasser</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14004-1">
    <skos:altLabel xml:lang="de">Schriftstellerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14004-1">
    <skos:altLabel xml:lang="de">Schriftsteller</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14004-1">
    <skos:altLabel xml:lang="de">Publizistinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14004-1">
    <skos:altLabel xml:lang="de">Publizisten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14004-1">
    <skos:altLabel xml:lang="de">Literatinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14004-1">
    <skos:altLabel xml:lang="de">Literaten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14004-1">
    <skos:altLabel xml:lang="de">Lektorinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14004-1">
    <skos:altLabel xml:lang="de">Lektoren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14004-1">
    <skos:altLabel xml:lang="de">Autorinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14004-1">
    <skos:altLabel xml:lang="de">Autoren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14004-1">
    <skos:altLabel xml:lang="de">Ansagerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14004-1">
    <skos:altLabel xml:lang="de">Ansager</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19556-0">
    <skos:altLabel xml:lang="de">Privatkundenbetreuung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19556-0">
    <skos:altLabel xml:lang="de">Privatkundenberatung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18467-4">
    <skos:broader rdf:resource="thsys/71005"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18467-4">
    <skos:broader rdf:resource="descriptor/12036-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15954-1">
    <skos:related rdf:resource="descriptor/19580-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15954-1">
    <skos:related rdf:resource="descriptor/19650-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15954-1">
    <skos:related rdf:resource="descriptor/16486-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15954-1">
    <skos:related rdf:resource="descriptor/19639-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15954-1">
    <skos:related rdf:resource="descriptor/15994-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15954-1">
    <skos:related rdf:resource="descriptor/11888-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15954-1">
    <skos:related rdf:resource="descriptor/15708-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15954-1">
    <skos:related rdf:resource="descriptor/15907-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18585-5">
    <skos:prefLabel xml:lang="en">Traffic control system</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18585-5">
    <skos:prefLabel xml:lang="de">Verkehrsleittechnik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10673-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11071-1">
    <skos:broader rdf:resource="thsys/70052"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11071-1">
    <skos:broader rdf:resource="descriptor/11111-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10007-1">
    <skos:broader rdf:resource="thsys/70201"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10007-1">
    <skos:broader rdf:resource="descriptor/10016-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13571-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19172-4">
    <skos:related rdf:resource="descriptor/17589-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19172-4">
    <skos:related rdf:resource="descriptor/17571-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19172-4">
    <skos:related rdf:resource="descriptor/17979-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19172-4">
    <skos:related rdf:resource="descriptor/19171-6"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id53">
    <rdf:value xml:lang="de">Für die institutionalisierten Beziehungen zwischen Arbeitgebern und Arbeitnehmern</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14143-1">
    <skos:broader rdf:resource="thsys/70403"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14143-1">
    <skos:broader rdf:resource="descriptor/14145-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12112-1">
    <skos:altLabel xml:lang="de">Organisationsmodell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12112-1">
    <skos:altLabel xml:lang="de">Organisationslehre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18392-2">
    <skos:prefLabel xml:lang="en">Copper mining</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18392-2">
    <skos:prefLabel xml:lang="de">Kupferbergbau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16523-2">
    <skos:related rdf:resource="descriptor/16024-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16523-2">
    <skos:related rdf:resource="descriptor/16522-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16523-2">
    <skos:related rdf:resource="descriptor/16543-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19832-2">
    <skos:altLabel xml:lang="en">Groupware-system</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19832-2">
    <skos:altLabel xml:lang="en">CSCW (Computer-Supported Cooperative Work)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19832-2">
    <skos:altLabel xml:lang="en">Computer-aided team</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19832-2">
    <skos:altLabel xml:lang="de">Computergestützte Gruppenarbeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70990">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.06.01.08</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12252-6">
    <skos:related rdf:resource="descriptor/13281-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12252-6">
    <skos:related rdf:resource="descriptor/12253-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12252-6">
    <skos:related rdf:resource="descriptor/18725-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12252-6">
    <skos:related rdf:resource="descriptor/13293-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12252-6">
    <skos:related rdf:resource="descriptor/13777-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12252-6">
    <skos:related rdf:resource="descriptor/13776-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16034-3">
    <skos:related rdf:resource="descriptor/16485-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16034-3">
    <skos:related rdf:resource="descriptor/19134-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16034-3">
    <skos:related rdf:resource="descriptor/12808-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16034-3">
    <skos:related rdf:resource="descriptor/10016-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16034-3">
    <skos:related rdf:resource="descriptor/18737-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16034-3">
    <skos:related rdf:resource="descriptor/19128-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16034-3">
    <skos:related rdf:resource="descriptor/19124-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13660-0">
    <zbwext:useInsteadNote rdf:nodeID="id208"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14177-5">
    <skos:broader rdf:resource="thsys/70538"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14177-5">
    <skos:broader rdf:resource="descriptor/18234-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14177-5">
    <skos:broader rdf:resource="descriptor/14176-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18192-3">
    <skos:related rdf:resource="descriptor/19295-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18192-3">
    <skos:related rdf:resource="descriptor/19294-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18192-3">
    <skos:related rdf:resource="descriptor/18518-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18192-3">
    <skos:related rdf:resource="descriptor/18519-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18192-3">
    <skos:related rdf:resource="descriptor/18191-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18192-3">
    <skos:related rdf:resource="descriptor/14064-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11671-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13472-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10991-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17946-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18118-1">
    <skos:related rdf:resource="descriptor/13808-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18118-1">
    <skos:related rdf:resource="descriptor/12078-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18118-1">
    <skos:related rdf:resource="descriptor/10293-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18118-1">
    <skos:related rdf:resource="descriptor/12166-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18118-1">
    <skos:related rdf:resource="descriptor/15467-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10138-0">
    <skos:scopeNote xml:lang="de">Theorie der Versteigerung und des Gebotsverhaltens, Theorie der Ausschreibung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70829">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.27</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14300-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11905-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19550-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12531-2">
    <skos:related rdf:resource="descriptor/12508-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12531-2">
    <skos:related rdf:resource="descriptor/12535-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12531-2">
    <skos:related rdf:resource="descriptor/12794-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18007-3">
    <skos:prefLabel xml:lang="en">Sales</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18007-3">
    <skos:prefLabel xml:lang="de">Absatz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19665-2">
    <skos:altLabel xml:lang="de">Refinanzierungsinstitut der letzten Instanz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19665-2">
    <skos:altLabel xml:lang="de">Letztinstanzlicher Kapitalgeber</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14961-2">
    <skos:related rdf:resource="descriptor/14957-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14961-2">
    <skos:related rdf:resource="descriptor/14070-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19543-2">
    <skos:related rdf:resource="descriptor/11267-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16512-0">
    <skos:scopeNote xml:lang="de">Politik gegenüber einheimischen ethnischen, sprachlichen oder religiösen Minderheiten</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13363-6">
    <skos:prefLabel xml:lang="en">Athletes</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13363-6">
    <skos:prefLabel xml:lang="de">Sportler</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70850">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12969-3">
    <skos:related rdf:resource="descriptor/14161-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12969-3">
    <skos:related rdf:resource="descriptor/12964-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16332-2">
    <skos:broader rdf:resource="thsys/73373"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16332-2">
    <skos:broader rdf:resource="thsys/73361"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16332-2">
    <skos:broader rdf:resource="thsys/70862"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10822-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19251-1">
    <skos:related rdf:resource="descriptor/16487-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19251-1">
    <skos:related rdf:resource="descriptor/10439-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19251-1">
    <skos:related rdf:resource="descriptor/10981-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19251-1">
    <skos:related rdf:resource="descriptor/12262-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19251-1">
    <skos:related rdf:resource="descriptor/19749-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19251-1">
    <skos:related rdf:resource="descriptor/10945-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19251-1">
    <skos:related rdf:resource="descriptor/18082-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19251-1">
    <skos:related rdf:resource="descriptor/10459-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19251-1">
    <skos:related rdf:resource="descriptor/18461-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19251-1">
    <skos:related rdf:resource="descriptor/10437-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19778-3">
    <skos:prefLabel xml:lang="en">Terminology</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19778-3">
    <skos:prefLabel xml:lang="de">Terminologie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19525-4">
    <skos:related rdf:resource="descriptor/19788-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19525-4">
    <skos:related rdf:resource="descriptor/15943-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19525-4">
    <skos:related rdf:resource="descriptor/15941-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19525-4">
    <skos:related rdf:resource="descriptor/15939-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17962-6">
    <skos:related rdf:resource="descriptor/18054-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12497-4">
    <skos:narrower rdf:resource="descriptor/19099-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19780-2">
    <skos:broader rdf:resource="thsys/70581"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19780-2">
    <skos:broader rdf:resource="descriptor/15675-5"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id234">
    <rdf:value xml:lang="de">für Länder und Regionen mit niedrigem Pro-Kopf-Einkommen</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14051-6">
    <skos:prefLabel xml:lang="en">Package</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14051-6">
    <skos:prefLabel xml:lang="de">Verpackung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14281-3">
    <skos:broader rdf:resource="thsys/70358"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14281-3">
    <skos:broader rdf:resource="descriptor/19051-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14281-3">
    <skos:broader rdf:resource="descriptor/14267-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16687-0">
    <skos:related rdf:resource="descriptor/10032-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16687-0">
    <skos:related rdf:resource="descriptor/15836-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16687-0">
    <skos:related rdf:resource="descriptor/19734-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16687-0">
    <skos:related rdf:resource="descriptor/16566-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16687-0">
    <skos:related rdf:resource="descriptor/15861-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16687-0">
    <skos:related rdf:resource="descriptor/16562-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16687-0">
    <skos:related rdf:resource="descriptor/16692-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16687-0">
    <skos:related rdf:resource="descriptor/19204-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16704-5">
    <skos:prefLabel xml:lang="en">Enlightenment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16704-5">
    <skos:prefLabel xml:lang="de">Aufklärung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11228-1">
    <skos:altLabel xml:lang="en">Training on the job</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11228-1">
    <skos:altLabel xml:lang="en">On-the-job-training</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11228-1">
    <skos:altLabel xml:lang="en">Learning by doing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18680-4">
    <skos:broader rdf:resource="thsys/71115"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18680-4">
    <skos:broader rdf:resource="descriptor/10024-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70650">
    <skos:broader rdf:resource="thsys/70538"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12072-1">
    <skos:prefLabel xml:lang="en">Enterprise</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12072-1">
    <skos:prefLabel xml:lang="de">Unternehmen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15956-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10173-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11805-3">
    <skos:broader rdf:resource="thsys/71028"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11805-3">
    <skos:broader rdf:resource="descriptor/18980-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16166-0">
    <skos:prefLabel xml:lang="en">Railroad law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16166-0">
    <skos:prefLabel xml:lang="de">Eisenbahnrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16985-6">
    <skos:prefLabel xml:lang="en">Norway</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16985-6">
    <skos:prefLabel xml:lang="de">Norwegen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18634-4">
    <skos:altLabel xml:lang="de">Tourismussteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18634-4">
    <skos:altLabel xml:lang="de">Tourismusabgabe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18634-4">
    <skos:altLabel xml:lang="de">Fremdenverkehrsteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15515-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13983-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12995-2">
    <skos:related rdf:resource="descriptor/18256-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12995-2">
    <skos:related rdf:resource="descriptor/14656-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12995-2">
    <skos:related rdf:resource="descriptor/14642-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12306-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70999">
    <rdfs:label xml:lang="en">Vehicle construction</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70999">
    <rdfs:label xml:lang="de">Fahrzeugbau</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16621-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10298-6">
    <skos:related rdf:resource="descriptor/15859-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10298-6">
    <skos:related rdf:resource="descriptor/18976-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10298-6">
    <skos:related rdf:resource="descriptor/13401-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10298-6">
    <skos:related rdf:resource="descriptor/19141-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10298-6">
    <skos:related rdf:resource="descriptor/18018-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10298-6">
    <skos:related rdf:resource="descriptor/10300-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10298-6">
    <skos:related rdf:resource="descriptor/10010-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10675-2">
    <skos:prefLabel xml:lang="en">Terms of trade</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10675-2">
    <skos:prefLabel xml:lang="de">Terms of Trade</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18590-5">
    <skos:altLabel xml:lang="de">Schiffskredit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18590-5">
    <skos:altLabel xml:lang="de">Schiffshypothek</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18590-5">
    <skos:altLabel xml:lang="de">Schiffsfinanzierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18590-5">
    <skos:altLabel xml:lang="de">Schiffahrtsfinanzierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19280-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70572">
    <rdfs:label xml:lang="de">Wanderungen und Mobilität der Bevölkerung</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14194-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13762-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15685-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70244">
    <skos:broader rdf:resource="thsys/70113"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12354-5">
    <zbwext:useInsteadNote rdf:nodeID="id52"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19697-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19633-1">
    <skos:scopeNote xml:lang="de">Informationssystem zur Verwaltung und Analyse von Umweltdaten. Auch für entsprechende Software.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id19">
    <rdf:value xml:lang="de">Für die Bewertung der Leistung von Mitarbeitern</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16786-5">
    <skos:altLabel xml:lang="de">Online-Recherche</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14003-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11185-0">
    <skos:altLabel xml:lang="en">Labor law</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10340-5">
    <skos:prefLabel xml:lang="en">International business cycle</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10340-5">
    <skos:prefLabel xml:lang="de">Weltkonjunktur</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24733-2">
    <skos:scopeNote xml:lang="de">Bezeichnet den Prozess, bei dem sich ein börsennotiertes Unternehmen von der Börse zurückzieht. Somit handelt es sich um das Gegenstück eines IPO oder "Going Public".</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24708-1">
    <skos:scopeNote xml:lang="de">Die IT-Industrie entwickelt und vermarktet die IuK-Technologie. Sie umfasst Hardware- und Softwarehersteller und -Dienstleister sowie Telekommunikations- und Internet-Provider.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16143-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18415-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11784-6">
    <skos:prefLabel xml:lang="en">Economic sector</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11784-6">
    <skos:prefLabel xml:lang="de">Branche</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14711-4">
    <skos:broader rdf:resource="thsys/70476"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14711-4">
    <skos:broader rdf:resource="descriptor/14710-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11305-2">
    <skos:prefLabel xml:lang="en">Working conditions</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11305-2">
    <skos:prefLabel xml:lang="de">Arbeitsbedingungen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71024">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id214">
    <rdf:value xml:lang="de">Für Produkte</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71094">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24661-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73351">
    <skos:prefLabel xml:lang="de">N.10.01.03  Umwelt, Natur und Ökologie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18240-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11778-1">
    <skos:related rdf:resource="descriptor/11751-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11778-1">
    <skos:related rdf:resource="descriptor/10024-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13413-3">
    <skos:prefLabel xml:lang="en">Public Administration</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13413-3">
    <skos:prefLabel xml:lang="de">Öffentliche Verwaltung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15927-4">
    <skos:narrower rdf:resource="descriptor/19762-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15927-4">
    <skos:narrower rdf:resource="descriptor/11277-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71084">
    <skos:prefLabel xml:lang="en">V.08.03  Development aid</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71084">
    <skos:prefLabel xml:lang="de">V.08.03  Entwicklungshilfe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19594-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10771-6">
    <skos:related rdf:resource="descriptor/10627-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10771-6">
    <skos:related rdf:resource="descriptor/21101-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10771-6">
    <skos:related rdf:resource="descriptor/10769-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10771-6">
    <skos:related rdf:resource="descriptor/10673-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10771-6">
    <skos:related rdf:resource="descriptor/10745-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10771-6">
    <skos:related rdf:resource="descriptor/10734-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10771-6">
    <skos:related rdf:resource="descriptor/10608-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10771-6">
    <skos:related rdf:resource="descriptor/10733-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14561-4">
    <skos:prefLabel xml:lang="en">Razor</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14561-4">
    <skos:prefLabel xml:lang="de">Rasierapparat</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12798-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20189-5">
    <skos:narrower rdf:resource="descriptor/19416-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18105-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10604-4">
    <skos:related rdf:resource="descriptor/10876-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10604-4">
    <skos:related rdf:resource="descriptor/10842-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10604-4">
    <skos:related rdf:resource="descriptor/18794-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10604-4">
    <skos:related rdf:resource="descriptor/10851-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10604-4">
    <skos:related rdf:resource="descriptor/10605-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10604-4">
    <skos:related rdf:resource="descriptor/10852-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10604-4">
    <skos:related rdf:resource="descriptor/10834-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10604-4">
    <skos:related rdf:resource="descriptor/10859-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10604-4">
    <skos:related rdf:resource="descriptor/10833-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20921-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12049-3">
    <skos:editorialNote xml:lang="de">Möglichst spezifischer indexieren.</skos:editorialNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10243-3">
    <skos:related rdf:resource="descriptor/11782-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10243-3">
    <skos:related rdf:resource="descriptor/10272-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10243-3">
    <skos:related rdf:resource="descriptor/19561-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10243-3">
    <skos:related rdf:resource="descriptor/15050-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10243-3">
    <skos:related rdf:resource="descriptor/11786-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10243-3">
    <skos:related rdf:resource="descriptor/10256-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10243-3">
    <skos:related rdf:resource="descriptor/10303-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15763-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16357-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19698-1">
    <skos:broader rdf:resource="thsys/73376"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19698-1">
    <skos:broader rdf:resource="descriptor/16354-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13402-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18280-6">
    <skos:related rdf:resource="descriptor/14772-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18280-6">
    <skos:related rdf:resource="descriptor/14266-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18280-6">
    <skos:related rdf:resource="descriptor/14599-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18504-3">
    <skos:related rdf:resource="descriptor/12905-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18504-3">
    <skos:related rdf:resource="descriptor/13853-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18504-3">
    <skos:related rdf:resource="descriptor/13851-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10568-3">
    <skos:altLabel xml:lang="de">Geberland</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10237-5">
    <skos:scopeNote xml:lang="de">Gesamtwirtschaftliche Nachfrage auf dem Gütermarkt.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id8">
    <rdfs:seeAlso rdf:resource="descriptor/18008-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12370-0">
    <skos:prefLabel xml:lang="en">Annual report (company)</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12370-0">
    <skos:prefLabel xml:lang="de">Geschäftsbericht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11339-6">
    <skos:prefLabel xml:lang="en">Labour dispute</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11339-6">
    <skos:prefLabel xml:lang="de">Arbeitskonflikt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71079">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.09.05</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10975-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11603-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10552-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18437-6">
    <skos:related rdf:resource="descriptor/12062-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18437-6">
    <skos:related rdf:resource="descriptor/10843-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18143-2">
    <skos:prefLabel xml:lang="en">Management education</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18143-2">
    <skos:prefLabel xml:lang="de">Führungskräfteausbildung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11790-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19528-5">
    <skos:related rdf:resource="descriptor/19790-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19507-6">
    <skos:broader rdf:resource="thsys/73356"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19507-6">
    <skos:broader rdf:resource="thsys/70504"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19507-6">
    <skos:broader rdf:resource="descriptor/15909-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12273-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15604-0">
    <skos:prefLabel xml:lang="en">Data protection</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15604-0">
    <skos:prefLabel xml:lang="de">Datenschutz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13593-3">
    <skos:altLabel xml:lang="de">Verkehrsbedienung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13593-3">
    <skos:altLabel xml:lang="de">Verkehrsanbindung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13593-3">
    <skos:altLabel xml:lang="de">Raumerschließung durch Verkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13593-3">
    <skos:altLabel xml:lang="de">Erreichbarkeit von Regionen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12391-6">
    <skos:broader rdf:resource="thsys/71042"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12391-6">
    <skos:broader rdf:resource="descriptor/12270-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16515-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11913-0">
    <zbwext:useInsteadNote rdf:nodeID="id310"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70878">
    <rdfs:label xml:lang="de">Forschungsbereiche</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73304">
    <skos:broader rdf:resource="thsys/70066"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10107-4">
    <skos:prefLabel xml:lang="en">Logrolling</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10107-4">
    <skos:prefLabel xml:lang="de">Stimmentausch</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19432-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11932-3">
    <skos:narrower rdf:resource="descriptor/19452-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11932-3">
    <skos:narrower rdf:resource="descriptor/11921-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15823-2">
    <skos:prefLabel xml:lang="en">Leadership</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15823-2">
    <skos:prefLabel xml:lang="de">Führungspersönlichkeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16806-4">
    <skos:altLabel xml:lang="de">Nordpolarmeer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16806-4">
    <skos:altLabel xml:lang="de">Nordpolargebiet</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16806-4">
    <skos:altLabel xml:lang="de">Nordpol</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70003">
    <skos:narrower rdf:resource="descriptor/16980-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70003">
    <skos:narrower rdf:resource="descriptor/16981-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70003">
    <skos:narrower rdf:resource="descriptor/16978-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70003">
    <skos:narrower rdf:resource="descriptor/16979-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70003">
    <skos:narrower rdf:resource="descriptor/16977-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70003">
    <skos:narrower rdf:resource="descriptor/16975-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70003">
    <skos:narrower rdf:resource="descriptor/16976-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70003">
    <skos:narrower rdf:resource="descriptor/16974-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70003">
    <skos:narrower rdf:resource="descriptor/16973-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70003">
    <skos:narrower rdf:resource="descriptor/16972-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70003">
    <skos:narrower rdf:resource="descriptor/16971-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70003">
    <skos:narrower rdf:resource="descriptor/16970-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70003">
    <skos:narrower rdf:resource="descriptor/16969-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70003">
    <skos:narrower rdf:resource="descriptor/16968-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70003">
    <skos:narrower rdf:resource="descriptor/16967-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70003">
    <skos:narrower rdf:resource="descriptor/16966-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70003">
    <skos:narrower rdf:resource="descriptor/16964-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70003">
    <skos:narrower rdf:resource="descriptor/16963-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70003">
    <skos:narrower rdf:resource="descriptor/16962-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70003">
    <skos:narrower rdf:resource="descriptor/16961-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70003">
    <skos:narrower rdf:resource="descriptor/16960-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70003">
    <skos:narrower rdf:resource="descriptor/16959-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70003">
    <skos:narrower rdf:resource="descriptor/16958-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70003">
    <skos:narrower rdf:resource="descriptor/16957-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70003">
    <skos:narrower rdf:resource="descriptor/16956-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11149-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id132">
    <rdf:value xml:lang="de">Für die ökonomische Analyse des Verhaltens von Familien</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21020-5">
    <skos:altLabel xml:lang="en">Supply chain management system</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21020-5">
    <skos:altLabel xml:lang="en">SCM system</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21020-5">
    <skos:altLabel xml:lang="en">SCM (Supply Chain Management)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14437-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10224-0">
    <skos:related rdf:resource="descriptor/10223-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10224-0">
    <skos:related rdf:resource="descriptor/10209-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10224-0">
    <skos:related rdf:resource="descriptor/10197-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16698-2">
    <skos:prefLabel xml:lang="en">Sociology of family</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16698-2">
    <skos:prefLabel xml:lang="de">Familiensoziologie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11373-6">
    <skos:related rdf:resource="descriptor/11379-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11373-6">
    <skos:related rdf:resource="descriptor/11378-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11373-6">
    <skos:related rdf:resource="descriptor/19603-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11373-6">
    <skos:related rdf:resource="descriptor/18876-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11373-6">
    <skos:related rdf:resource="descriptor/11402-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11373-6">
    <skos:related rdf:resource="descriptor/11372-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11373-6">
    <skos:related rdf:resource="descriptor/18782-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11373-6">
    <skos:related rdf:resource="descriptor/18874-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11373-6">
    <skos:related rdf:resource="descriptor/11356-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13200-6">
    <skos:related rdf:resource="descriptor/13475-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13200-6">
    <skos:related rdf:resource="descriptor/13488-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13200-6">
    <skos:related rdf:resource="descriptor/13486-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13200-6">
    <skos:related rdf:resource="descriptor/18547-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13200-6">
    <skos:related rdf:resource="descriptor/18555-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13200-6">
    <skos:related rdf:resource="descriptor/15077-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13200-6">
    <skos:related rdf:resource="descriptor/18545-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13200-6">
    <skos:related rdf:resource="descriptor/13481-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13200-6">
    <skos:related rdf:resource="descriptor/14346-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19665-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17161-5">
    <skos:altLabel xml:lang="x-other">Reino de Espana</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17161-5">
    <skos:altLabel xml:lang="x-other">Espana</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17161-5">
    <skos:altLabel xml:lang="de">Königreich Spanien</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10286-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10210-4">
    <skos:related rdf:resource="descriptor/10221-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10210-4">
    <skos:related rdf:resource="descriptor/13735-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10210-4">
    <skos:related rdf:resource="descriptor/12210-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10210-4">
    <skos:related rdf:resource="descriptor/10077-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10210-4">
    <skos:related rdf:resource="descriptor/19271-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13423-0">
    <skos:altLabel xml:lang="de">Verwaltungsgerichtsbarkeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19502-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19032-6">
    <skos:prefLabel xml:lang="en">Control</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19032-6">
    <skos:prefLabel xml:lang="de">Kontrolle</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15415-3">
    <skos:prefLabel xml:lang="en">Cybernetics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15415-3">
    <skos:prefLabel xml:lang="de">Kybernetik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70017">
    <skos:broader rdf:resource="thsys/n"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18265-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15752-6">
    <skos:altLabel xml:lang="en">Resources policy</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15752-6">
    <skos:altLabel xml:lang="en">Commodity policy</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15752-6">
    <skos:altLabel xml:lang="de">Ressourcenpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10801-2">
    <skos:prefLabel xml:lang="en">Euromarket</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10801-2">
    <skos:prefLabel xml:lang="de">Euromarkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16292-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11004-2">
    <skos:broader rdf:resource="thsys/70234"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11004-2">
    <skos:broader rdf:resource="thsys/70038"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19476-5">
    <skos:broader rdf:resource="thsys/71064"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11611-2">
    <skos:prefLabel xml:lang="en">Public bond</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11611-2">
    <skos:prefLabel xml:lang="de">Öffentliche Anleihe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16868-3">
    <skos:prefLabel xml:lang="en">Hamburg</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16868-3">
    <skos:prefLabel xml:lang="de">Hamburg</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19679-5">
    <skos:prefLabel xml:lang="en">Insurance market</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19679-5">
    <skos:prefLabel xml:lang="de">Versicherungsmarkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13563-5">
    <zbwext:useInsteadNote rdf:nodeID="id320"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13563-5">
    <zbwext:useInsteadNote rdf:nodeID="id319"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12184-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18349-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70956">
    <skos:broader rdf:resource="thsys/70963"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10027-2">
    <skos:broader rdf:resource="thsys/70301"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10027-2">
    <skos:broader rdf:resource="thsys/70233"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10027-2">
    <skos:broader rdf:resource="thsys/70201"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13957-5">
    <skos:altLabel xml:lang="de">Bauleiterinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13957-5">
    <skos:altLabel xml:lang="de">Bauleiter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13957-5">
    <skos:altLabel xml:lang="de">Bauingenieurinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13957-5">
    <skos:altLabel xml:lang="de">Bauingenieure</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13957-5">
    <skos:altLabel xml:lang="de">Architektinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12394-0">
    <skos:related rdf:resource="descriptor/18099-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12394-0">
    <skos:related rdf:resource="descriptor/15073-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12394-0">
    <skos:related rdf:resource="descriptor/10058-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12394-0">
    <skos:related rdf:resource="descriptor/19237-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12394-0">
    <skos:related rdf:resource="descriptor/12314-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12394-0">
    <skos:related rdf:resource="descriptor/25822-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12363-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19108-6">
    <skos:broader rdf:resource="thsys/71042"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12865-1">
    <skos:altLabel xml:lang="de">Familiengesellschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12865-1">
    <skos:altLabel xml:lang="de">Familienbetrieb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10497-0">
    <skos:related rdf:resource="descriptor/13446-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10497-0">
    <skos:related rdf:resource="descriptor/13450-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11847-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10327-4">
    <zbwext:useInsteadNote rdf:nodeID="id265"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24621-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18180-3">
    <skos:broader rdf:resource="thsys/70891"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18180-3">
    <skos:broader rdf:resource="thsys/70839"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18180-3">
    <skos:broader rdf:resource="thsys/70832"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18180-3">
    <skos:broader rdf:resource="thsys/70573"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18180-3">
    <skos:broader rdf:resource="thsys/70458"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18180-3">
    <skos:broader rdf:resource="descriptor/13418-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12812-1">
    <skos:prefLabel xml:lang="en">Sales force</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12812-1">
    <skos:prefLabel xml:lang="de">Außendienst</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16699-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id337">
    <rdfs:seeAlso rdf:resource="descriptor/16214-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11415-2">
    <skos:broader rdf:resource="thsys/70055"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18027-4">
    <skos:narrower rdf:resource="descriptor/18934-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10141-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16768-0">
    <skos:prefLabel xml:lang="en">Bibliography</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16768-0">
    <skos:prefLabel xml:lang="de">Bibliographie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11184-2">
    <skos:broader rdf:resource="thsys/70244"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11184-2">
    <skos:broader rdf:resource="thsys/70051"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11184-2">
    <skos:broader rdf:resource="descriptor/10314-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19750-4">
    <skos:broader rdf:resource="thsys/73394"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19750-4">
    <skos:broader rdf:resource="thsys/70581"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19750-4">
    <skos:broader rdf:resource="descriptor/16719-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12774-4">
    <skos:scopeNote xml:lang="de">Untersuchung der Einstellung und des Verhaltens von Verbrauchern, Produzenten und Dienstleistern zum Zweck der Verkaufsförderung sowie der Produkt- bzw. Dienstleistungs-Innovation.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12206-6">
    <skos:broader rdf:resource="thsys/70037"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12206-6">
    <skos:broader rdf:resource="descriptor/12041-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12120-2">
    <skos:altLabel xml:lang="de">Managementstruktur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12120-2">
    <skos:altLabel xml:lang="de">Managementorganisation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12120-2">
    <skos:altLabel xml:lang="de">Leitungsstruktur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12120-2">
    <skos:altLabel xml:lang="de">Leitungsorganisation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12120-2">
    <skos:altLabel xml:lang="de">Führungsstruktur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18655-3">
    <skos:scopeNote xml:lang="de">Keine allgemeingültige Definition. Meist ist damit gemeint: Angebot von erweiterten Dienstleistungen, die im Zusammenhang mit der Bereitstellung von Telekommunikationsnetzen stehen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19242-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12967-0">
    <skos:related rdf:resource="descriptor/19614-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12967-0">
    <skos:related rdf:resource="descriptor/13543-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12967-0">
    <skos:related rdf:resource="descriptor/15883-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12967-0">
    <skos:related rdf:resource="descriptor/19616-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12766-3">
    <skos:broader rdf:resource="thsys/73384"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12766-3">
    <skos:broader rdf:resource="thsys/70041"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14685-0">
    <skos:broader rdf:resource="thsys/70556"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14685-0">
    <skos:broader rdf:resource="descriptor/14656-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13969-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17989-0">
    <skos:altLabel xml:lang="de">Karibische Länder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12451-0">
    <skos:prefLabel xml:lang="en">Consolidation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12451-0">
    <skos:prefLabel xml:lang="de">Konsolidierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71000">
    <skos:prefLabel xml:lang="en">W.06.01.01.02  Nonferrous metals industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71000">
    <skos:prefLabel xml:lang="de">W.06.01.01.02  NE-Metallindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11995-0">
    <skos:broader rdf:resource="thsys/71064"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11995-0">
    <skos:broader rdf:resource="descriptor/19450-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11995-0">
    <skos:broader rdf:resource="descriptor/11998-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11995-0">
    <skos:broader rdf:resource="descriptor/16176-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10274-6">
    <skos:altLabel xml:lang="de">Volkswirtschaftliches Vermögen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10274-6">
    <skos:altLabel xml:lang="de">Volkswirtschaftliche Vermögensrechnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10274-6">
    <skos:altLabel xml:lang="de">Volksvermögensrechnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10274-6">
    <skos:altLabel xml:lang="de">Vermögen der Volkswirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10274-6">
    <skos:altLabel xml:lang="de">Gesamtwirtschaftliche Vermögensrechnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11447-3">
    <skos:related rdf:resource="descriptor/10255-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71089">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19165-1">
    <skos:scopeNote xml:lang="de">China im Sinne des historischen Chinas und der Gesamtheit von VR China, Taiwan und Hong Kong</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14895-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10559-4">
    <skos:related rdf:resource="descriptor/11033-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10559-4">
    <skos:related rdf:resource="descriptor/11030-1"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id104">
    <rdfs:seeAlso rdf:resource="descriptor/10466-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11436-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id136">
    <rdf:value xml:lang="de">Für betriebswirtschaftliche Ansätze</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18267-5">
    <skos:prefLabel xml:lang="en">Water treatment chemical</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18267-5">
    <skos:prefLabel xml:lang="de">Wasseraufbereitungsmittel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15621-0">
    <skos:altLabel xml:lang="de">Softwarewartung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15621-0">
    <skos:altLabel xml:lang="de">Software-Wartung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15621-0">
    <skos:altLabel xml:lang="de">Softwaretest</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15621-0">
    <skos:altLabel xml:lang="de">Softwareimplementierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15621-0">
    <skos:altLabel xml:lang="de">EDV-Programm</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15621-0">
    <skos:altLabel xml:lang="de">Computerprogramm</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15621-0">
    <skos:altLabel xml:lang="de">Anwendersoftware</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13224-6">
    <skos:narrower rdf:resource="descriptor/13236-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19778-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16826-5">
    <skos:altLabel xml:lang="de">Volksrepublik Polen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16826-5">
    <skos:altLabel xml:lang="de">Republik Polen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12524-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19639-3">
    <skos:narrower rdf:resource="descriptor/19648-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11992-6">
    <skos:prefLabel xml:lang="en">Urban sociology</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11992-6">
    <skos:prefLabel xml:lang="de">Stadtsoziologie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10432-0">
    <skos:prefLabel xml:lang="en">Turnpike theorem</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10432-0">
    <skos:prefLabel xml:lang="de">Turnpike-Theorem</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14635-1">
    <skos:prefLabel xml:lang="en">Flammables and explosives</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14635-1">
    <skos:prefLabel xml:lang="de">Zündwaren und Sprengstoff</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12992-1">
    <skos:broader rdf:resource="thsys/71013"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12992-1">
    <skos:broader rdf:resource="thsys/71007"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12992-1">
    <skos:broader rdf:resource="descriptor/13003-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12992-1">
    <skos:broader rdf:resource="descriptor/12988-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11876-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13209-2">
    <skos:related rdf:resource="descriptor/15080-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71065">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16628-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14027-3">
    <skos:broader rdf:resource="thsys/70333"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14027-3">
    <skos:broader rdf:resource="descriptor/14051-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19337-5">
    <skos:broader rdf:resource="thsys/71094"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19337-5">
    <skos:broader rdf:resource="descriptor/19664-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15365-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18400-1">
    <skos:related rdf:resource="descriptor/18402-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18400-1">
    <skos:related rdf:resource="descriptor/13034-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18400-1">
    <skos:related rdf:resource="descriptor/13185-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12142-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19683-0">
    <skos:altLabel xml:lang="en">Policy uncertainty</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19683-0">
    <skos:altLabel xml:lang="de">Verlässlichkeit der Politik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19683-0">
    <skos:altLabel xml:lang="de">Verläßlichkeit der Politik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19683-0">
    <skos:altLabel xml:lang="de">Politische Krise</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19683-0">
    <skos:altLabel xml:lang="de">Politische Instabilität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13188-5">
    <skos:prefLabel xml:lang="en">Agricultural trade</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13188-5">
    <skos:prefLabel xml:lang="de">Landhandel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13960-2">
    <skos:related rdf:resource="descriptor/19795-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13960-2">
    <skos:related rdf:resource="descriptor/13381-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13960-2">
    <skos:related rdf:resource="descriptor/18764-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13960-2">
    <skos:related rdf:resource="descriptor/13881-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13960-2">
    <skos:related rdf:resource="descriptor/13953-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13960-2">
    <skos:related rdf:resource="descriptor/13957-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14273-2">
    <skos:related rdf:resource="descriptor/13190-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14273-2">
    <skos:related rdf:resource="descriptor/14269-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13361-3">
    <skos:prefLabel xml:lang="en">Special library</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13361-3">
    <skos:prefLabel xml:lang="de">Spezialbibliothek</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18733-2">
    <skos:altLabel xml:lang="de">Markt-Mikrostruktur-Theorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19255-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19544-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11044-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19506-1">
    <skos:altLabel xml:lang="de">Lebenshaltungsminimum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11633-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18330-3">
    <skos:narrower rdf:resource="descriptor/19330-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19708-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14393-6">
    <skos:related rdf:resource="descriptor/13067-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14393-6">
    <skos:related rdf:resource="descriptor/14339-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16797-0">
    <skos:prefLabel xml:lang="en">Reference work</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16797-0">
    <skos:prefLabel xml:lang="de">Nachschlagewerk</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11165-6">
    <skos:altLabel xml:lang="en">Median voter approach</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11165-6">
    <skos:altLabel xml:lang="de">Median-Wähler-Ansatz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10383-1">
    <skos:related rdf:resource="descriptor/11004-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10383-1">
    <skos:related rdf:resource="descriptor/11239-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10383-1">
    <skos:related rdf:resource="descriptor/10381-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10463-3">
    <skos:prefLabel xml:lang="en">Rationalization</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10463-3">
    <skos:prefLabel xml:lang="de">Rationalisierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17461-0">
    <skos:broader rdf:resource="thsys/70174"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17934-4">
    <skos:prefLabel xml:lang="en">Venezuela</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17934-4">
    <skos:prefLabel xml:lang="de">Venezuela</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10541-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13056-1">
    <skos:broader rdf:resource="thsys/70998"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13056-1">
    <skos:broader rdf:resource="descriptor/13047-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14121-4">
    <skos:related rdf:resource="descriptor/12941-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14121-4">
    <skos:related rdf:resource="descriptor/14135-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14608-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13060-3">
    <skos:broader rdf:resource="thsys/70997"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13060-3">
    <skos:broader rdf:resource="descriptor/13051-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18518-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13301-0">
    <skos:prefLabel xml:lang="en">Service</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13301-0">
    <skos:prefLabel xml:lang="de">Dienstleistung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19641-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70556">
    <rdfs:label xml:lang="de">Organische Chemikalien</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13173-4">
    <skos:narrower rdf:resource="descriptor/13170-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19270-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71118">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16684-6">
    <skos:related rdf:resource="descriptor/13998-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16684-6">
    <skos:related rdf:resource="descriptor/12110-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16684-6">
    <skos:related rdf:resource="descriptor/16252-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16684-6">
    <skos:related rdf:resource="descriptor/13412-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16684-6">
    <skos:related rdf:resource="descriptor/16204-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10113-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70176">
    <skos:prefLabel xml:lang="en">V.14.02  Education</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70176">
    <skos:prefLabel xml:lang="de">V.14.02  Bildung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13234-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70187">
    <skos:narrower rdf:resource="descriptor/18087-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70187">
    <skos:narrower rdf:resource="descriptor/10884-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70187">
    <skos:narrower rdf:resource="descriptor/18679-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70187">
    <skos:narrower rdf:resource="descriptor/18691-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70187">
    <skos:narrower rdf:resource="descriptor/13766-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70187">
    <skos:narrower rdf:resource="descriptor/10021-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70187">
    <skos:narrower rdf:resource="descriptor/18670-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70187">
    <skos:narrower rdf:resource="descriptor/11618-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70187">
    <skos:narrower rdf:resource="descriptor/19274-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70187">
    <skos:narrower rdf:resource="descriptor/12272-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70187">
    <skos:narrower rdf:resource="descriptor/19272-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70187">
    <skos:narrower rdf:resource="descriptor/12212-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70187">
    <skos:narrower rdf:resource="descriptor/18683-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70187">
    <skos:narrower rdf:resource="descriptor/19554-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70187">
    <skos:narrower rdf:resource="descriptor/18733-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70187">
    <skos:narrower rdf:resource="descriptor/18678-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70187">
    <skos:narrower rdf:resource="descriptor/19273-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70187">
    <skos:narrower rdf:resource="descriptor/11433-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70187">
    <skos:narrower rdf:resource="descriptor/12210-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70187">
    <skos:narrower rdf:resource="descriptor/13731-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70187">
    <skos:narrower rdf:resource="descriptor/19321-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70187">
    <skos:narrower rdf:resource="descriptor/12285-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70187">
    <skos:narrower rdf:resource="descriptor/12280-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70187">
    <skos:narrower rdf:resource="descriptor/18086-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70187">
    <skos:narrower rdf:resource="descriptor/10124-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70187">
    <skos:narrower rdf:resource="descriptor/12287-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70187">
    <skos:narrower rdf:resource="descriptor/18677-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70187">
    <skos:narrower rdf:resource="descriptor/13729-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70187">
    <skos:narrower rdf:resource="descriptor/18673-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70187">
    <skos:narrower rdf:resource="descriptor/13704-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70187">
    <skos:narrower rdf:resource="descriptor/24707-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70187">
    <skos:narrower rdf:resource="descriptor/10803-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70187">
    <skos:narrower rdf:resource="descriptor/19664-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70187">
    <skos:narrower rdf:resource="descriptor/13723-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70187">
    <skos:narrower rdf:resource="descriptor/11414-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70187">
    <skos:narrower rdf:resource="descriptor/11441-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70187">
    <skos:narrower rdf:resource="descriptor/25742-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70187">
    <skos:narrower rdf:resource="descriptor/18668-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70187">
    <skos:narrower rdf:resource="descriptor/10801-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70187">
    <skos:narrower rdf:resource="descriptor/19522-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70187">
    <skos:narrower rdf:resource="descriptor/12204-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70187">
    <skos:narrower rdf:resource="descriptor/19265-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70187">
    <skos:narrower rdf:resource="descriptor/19271-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70187">
    <skos:narrower rdf:resource="descriptor/19196-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70187">
    <skos:narrower rdf:resource="descriptor/19669-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70187">
    <skos:narrower rdf:resource="descriptor/19270-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70187">
    <skos:narrower rdf:resource="descriptor/18676-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70187">
    <skos:narrower rdf:resource="descriptor/19671-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70187">
    <skos:narrower rdf:resource="descriptor/13715-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70187">
    <skos:narrower rdf:resource="descriptor/18730-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70187">
    <skos:narrower rdf:resource="descriptor/18729-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70187">
    <skos:narrower rdf:resource="descriptor/13279-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70187">
    <skos:narrower rdf:resource="descriptor/19538-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70187">
    <skos:narrower rdf:resource="descriptor/19537-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70187">
    <skos:narrower rdf:resource="descriptor/10001-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70187">
    <skos:narrower rdf:resource="descriptor/19269-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70187">
    <skos:narrower rdf:resource="descriptor/18702-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70187">
    <skos:narrower rdf:resource="descriptor/18674-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70187">
    <skos:narrower rdf:resource="thsys/71107"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70187">
    <skos:narrower rdf:resource="thsys/71103"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70187">
    <skos:narrower rdf:resource="thsys/71108"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70187">
    <skos:narrower rdf:resource="thsys/71104"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70091">
    <skos:broader rdf:resource="thsys/v"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18638-3">
    <skos:related rdf:resource="descriptor/11970-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12204-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12051-2">
    <skos:related rdf:resource="descriptor/12712-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12051-2">
    <skos:related rdf:resource="descriptor/12913-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12051-2">
    <skos:related rdf:resource="descriptor/18383-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19120-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18107-6">
    <skos:prefLabel xml:lang="en">Disinvestment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18107-6">
    <skos:prefLabel xml:lang="de">Desinvestition</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19350-6">
    <skos:related rdf:resource="descriptor/18847-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19350-6">
    <skos:related rdf:resource="descriptor/19352-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14194-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16435-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19065-5">
    <skos:broader rdf:resource="thsys/70889"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19065-5">
    <skos:broader rdf:resource="descriptor/18747-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70893">
    <skos:broader rdf:resource="thsys/w"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70141">
    <skos:prefLabel xml:lang="en">V.01.01  History of thought</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70141">
    <skos:prefLabel xml:lang="de">V.01.01  Dogmengeschichte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12865-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70049">
    <skos:prefLabel xml:lang="de">V.01  Allgemeine Volkswirtschaftstheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13714-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11618-2">
    <skos:broader rdf:resource="thsys/71108"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11618-2">
    <skos:broader rdf:resource="thsys/71078"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11618-2">
    <skos:broader rdf:resource="thsys/70187"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11618-2">
    <skos:broader rdf:resource="descriptor/11613-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11618-2">
    <skos:broader rdf:resource="descriptor/13726-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19989-4">
    <skos:prefLabel xml:lang="en">Normative economics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19989-4">
    <skos:prefLabel xml:lang="de">Normative Ökonomik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11695-5">
    <skos:prefLabel xml:lang="en">Tax shifting</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11695-5">
    <skos:prefLabel xml:lang="de">Steuerüberwälzung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14446-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15836-0">
    <skos:narrower rdf:resource="descriptor/15858-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15836-0">
    <skos:narrower rdf:resource="descriptor/12617-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16416-3">
    <skos:related rdf:resource="descriptor/16389-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16416-3">
    <skos:related rdf:resource="descriptor/16422-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16416-3">
    <skos:related rdf:resource="descriptor/16404-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16416-3">
    <skos:related rdf:resource="descriptor/16300-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16416-3">
    <skos:related rdf:resource="descriptor/16372-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16416-3">
    <skos:related rdf:resource="descriptor/19235-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16416-3">
    <skos:related rdf:resource="descriptor/16264-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16416-3">
    <skos:related rdf:resource="descriptor/18062-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16416-3">
    <skos:related rdf:resource="descriptor/16299-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16416-3">
    <skos:related rdf:resource="descriptor/16410-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16416-3">
    <skos:related rdf:resource="descriptor/16249-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16416-3">
    <skos:related rdf:resource="descriptor/15797-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16416-3">
    <skos:related rdf:resource="descriptor/15932-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16416-3">
    <skos:related rdf:resource="descriptor/16400-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19424-3">
    <skos:prefLabel xml:lang="en">Insurance tax</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19424-3">
    <skos:prefLabel xml:lang="de">Versicherungsteuer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#OrderedCollection">
    <rdfs:label xml:lang="en">Ordered Collection</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12025-3">
    <skos:prefLabel xml:lang="en">Real estate brokers</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12025-3">
    <skos:prefLabel xml:lang="de">Immobilienmakler</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70649">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">P.04.03</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14353-4">
    <skos:prefLabel xml:lang="en">Aircraft propulsion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14353-4">
    <skos:prefLabel xml:lang="de">Flugzeugantrieb</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18539-5">
    <skos:broader rdf:resource="thsys/70951"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18539-5">
    <skos:broader rdf:resource="thsys/70037"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18539-5">
    <skos:broader rdf:resource="descriptor/18118-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18539-5">
    <skos:broader rdf:resource="descriptor/12215-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18286-1">
    <skos:related rdf:resource="descriptor/15751-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14990-2">
    <skos:altLabel xml:lang="de">Spreu</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14990-2">
    <skos:altLabel xml:lang="de">Rauhfutter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14990-2">
    <skos:altLabel xml:lang="de">Heu</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14990-2">
    <skos:altLabel xml:lang="de">Grünfutter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14990-2">
    <skos:altLabel xml:lang="de">Futterpflanze</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16552-2">
    <skos:broader rdf:resource="thsys/73368"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16552-2">
    <skos:broader rdf:resource="thsys/71110"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16552-2">
    <skos:broader rdf:resource="thsys/71069"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16552-2">
    <skos:broader rdf:resource="descriptor/10981-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10969-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11444-2">
    <skos:related rdf:resource="descriptor/11473-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11444-2">
    <skos:related rdf:resource="descriptor/11430-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11444-2">
    <skos:related rdf:resource="descriptor/11411-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11444-2">
    <skos:related rdf:resource="descriptor/11443-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13336-2">
    <skos:altLabel xml:lang="en">Landfilling</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13336-2">
    <skos:altLabel xml:lang="en">Landfill site</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13336-2">
    <skos:altLabel xml:lang="de">Deponie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13336-2">
    <skos:altLabel xml:lang="de">Abfalllagerung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13336-2">
    <skos:altLabel xml:lang="de">Abfalldeponie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13336-2">
    <skos:altLabel xml:lang="de">Abfallagerung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14262-0">
    <skos:broader rdf:resource="thsys/70346"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14262-0">
    <skos:broader rdf:resource="descriptor/19051-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18501-2">
    <skos:related rdf:resource="descriptor/18630-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18501-2">
    <skos:related rdf:resource="descriptor/13527-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18501-2">
    <skos:related rdf:resource="descriptor/13574-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18501-2">
    <skos:related rdf:resource="descriptor/18575-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13564-3">
    <skos:altLabel xml:lang="de">Verkehrsstockung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13564-3">
    <skos:altLabel xml:lang="de">Verkehrsspitze</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13564-3">
    <skos:altLabel xml:lang="de">Stauung des Verkehrs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19490-4">
    <skos:narrower rdf:resource="descriptor/19487-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12995-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70176">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18394-5">
    <skos:altLabel xml:lang="de">Offshore-Erdölförderung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18394-5">
    <skos:altLabel xml:lang="de">Offshore-Erdgasförderung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18394-5">
    <skos:altLabel xml:lang="de">Offshore-Anlage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15117-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12046-2">
    <skos:broader rdf:resource="thsys/73387"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12046-2">
    <skos:broader rdf:resource="thsys/71034"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12046-2">
    <skos:broader rdf:resource="thsys/70109"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12046-2">
    <skos:broader rdf:resource="descriptor/15706-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12955-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11303-6">
    <skos:broader rdf:resource="thsys/73385"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11303-6">
    <skos:broader rdf:resource="thsys/71058"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11303-6">
    <skos:broader rdf:resource="thsys/71050"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11303-6">
    <skos:broader rdf:resource="thsys/70244"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14441-0">
    <skos:narrower rdf:resource="descriptor/14444-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14441-0">
    <skos:narrower rdf:resource="descriptor/14434-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14441-0">
    <skos:narrower rdf:resource="descriptor/14433-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14441-0">
    <skos:narrower rdf:resource="descriptor/19053-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14212-1">
    <skos:related rdf:resource="descriptor/15651-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14212-1">
    <skos:related rdf:resource="descriptor/14214-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70341">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15941-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16169-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21283-1">
    <skos:related rdf:resource="descriptor/18665-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20261-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73392">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.05.02.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15642-6">
    <skos:broader rdf:resource="thsys/70580"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15642-6">
    <skos:broader rdf:resource="descriptor/15676-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19073-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19030-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19346-4">
    <skos:prefLabel xml:lang="en">Pork price</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19346-4">
    <skos:prefLabel xml:lang="de">Schweinepreis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15981-5">
    <skos:altLabel xml:lang="de">Prozessrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15981-5">
    <skos:altLabel xml:lang="de">Prozeßrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15981-5">
    <skos:altLabel xml:lang="de">Kostenrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15981-5">
    <skos:altLabel xml:lang="de">Gerichtsverfassungsgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15981-5">
    <skos:altLabel xml:lang="de">Gerichtsverfassung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13848-3">
    <skos:related rdf:resource="descriptor/18502-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13848-3">
    <skos:related rdf:resource="descriptor/12932-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15709-0">
    <skos:altLabel xml:lang="de">Wanderung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15709-0">
    <skos:altLabel xml:lang="de">Räumliche Mobilität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15709-0">
    <skos:altLabel xml:lang="de">Räumliche Bevölkerungsbewegung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15709-0">
    <skos:altLabel xml:lang="de">Interregionale Mobilität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15709-0">
    <skos:altLabel xml:lang="de">Geographische Mobilität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12368-1">
    <skos:broader rdf:resource="thsys/71043"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12368-1">
    <skos:broader rdf:resource="descriptor/12399-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18273-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18170-6">
    <skos:related rdf:resource="descriptor/19806-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12273-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12868-2">
    <skos:related rdf:resource="descriptor/16076-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12868-2">
    <skos:related rdf:resource="descriptor/12196-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12868-2">
    <skos:related rdf:resource="descriptor/11666-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15893-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14715-3">
    <skos:altLabel xml:lang="de">Polyvinylchlorid</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71039">
    <rdfs:label xml:lang="en">Business information technology management</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71039">
    <rdfs:label xml:lang="de">Wirtschaftsinformatik</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15088-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10040-3">
    <skos:altLabel xml:lang="de">Reallokation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10040-3">
    <skos:altLabel xml:lang="de">Faktorallokation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10040-3">
    <skos:altLabel xml:lang="de">Allokationstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10040-3">
    <skos:altLabel xml:lang="de">Allokationssystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10040-3">
    <skos:altLabel xml:lang="de">Allokationsmechanismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14422-4">
    <skos:prefLabel xml:lang="en">Military aircraft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14422-4">
    <skos:prefLabel xml:lang="de">Militärluftfahrzeug</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id253">
    <rdf:value xml:lang="de">Für kollektive Formen politischer Gewalt, die geeignet sind, die gesamte Staats- und Rechtsordnung zu gefährden</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10826-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14772-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19662-1">
    <zbwext:useInsteadNote rdf:nodeID="id238"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id191">
    <rdf:value xml:lang="de">Für menschliche Lebensläufe</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15910-0">
    <skos:related rdf:resource="descriptor/10512-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15910-0">
    <skos:related rdf:resource="descriptor/15916-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15910-0">
    <skos:related rdf:resource="descriptor/15909-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15910-0">
    <skos:related rdf:resource="descriptor/11979-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18031-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11670-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13352-4">
    <skos:related rdf:resource="descriptor/13364-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13352-4">
    <skos:related rdf:resource="descriptor/18929-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13352-4">
    <skos:related rdf:resource="descriptor/13347-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13352-4">
    <skos:related rdf:resource="descriptor/16514-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19380-4">
    <skos:prefLabel xml:lang="en">Budget cycle</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19380-4">
    <skos:prefLabel xml:lang="de">Budgetkreislauf</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12299-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11988-4">
    <skos:broader rdf:resource="thsys/71064"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11988-4">
    <skos:broader rdf:resource="descriptor/11990-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17723-3">
    <skos:broader rdf:resource="thsys/70560"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17723-3">
    <skos:broader rdf:resource="descriptor/17724-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18572-0">
    <skos:related rdf:resource="descriptor/13255-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18572-0">
    <skos:related rdf:resource="descriptor/18575-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18572-0">
    <skos:related rdf:resource="descriptor/18571-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18572-0">
    <skos:related rdf:resource="descriptor/18569-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16977-5">
    <skos:broader rdf:resource="thsys/70003"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16977-5">
    <skos:broader rdf:resource="descriptor/16972-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13481-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15671-6">
    <skos:prefLabel xml:lang="en">Religion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15671-6">
    <skos:prefLabel xml:lang="de">Religion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12849-6">
    <skos:broader rdf:resource="thsys/73337"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12849-6">
    <skos:broader rdf:resource="thsys/70397"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12849-6">
    <skos:broader rdf:resource="descriptor/11640-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19445-2">
    <skos:scopeNote xml:lang="de">Literatur über die verschiedenen Strukturfonds der EU.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11929-6">
    <skos:related rdf:resource="descriptor/10426-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11929-6">
    <skos:related rdf:resource="descriptor/11954-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18911-4">
    <skos:related rdf:resource="descriptor/18970-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18911-4">
    <skos:related rdf:resource="descriptor/18864-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18911-4">
    <skos:related rdf:resource="descriptor/11192-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18911-4">
    <skos:related rdf:resource="descriptor/19759-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11678-5">
    <skos:broader rdf:resource="thsys/71074"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11678-5">
    <skos:broader rdf:resource="thsys/70046"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11678-5">
    <skos:broader rdf:resource="descriptor/11681-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14713-0">
    <skos:prefLabel xml:lang="en">Polyurethane</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14713-0">
    <skos:prefLabel xml:lang="de">Polyurethan</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12702-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14997-2">
    <skos:related rdf:resource="descriptor/14635-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14997-2">
    <skos:related rdf:resource="descriptor/11677-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14997-2">
    <skos:related rdf:resource="descriptor/13164-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14997-2">
    <skos:related rdf:resource="descriptor/13238-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14997-2">
    <skos:related rdf:resource="descriptor/14082-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14997-2">
    <skos:related rdf:resource="descriptor/18900-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14997-2">
    <skos:related rdf:resource="descriptor/16004-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19851-5">
    <skos:related rdf:resource="descriptor/16188-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19851-5">
    <skos:related rdf:resource="descriptor/16288-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19851-5">
    <skos:related rdf:resource="descriptor/16497-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13170-3">
    <skos:broader rdf:resource="thsys/70969"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13170-3">
    <skos:broader rdf:resource="descriptor/13173-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15411-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18205-6">
    <skos:related rdf:resource="descriptor/13102-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18205-6">
    <skos:related rdf:resource="descriptor/14812-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20906-3">
    <skos:broader rdf:resource="thsys/71080"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20906-3">
    <skos:broader rdf:resource="thsys/70578"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20906-3">
    <skos:broader rdf:resource="descriptor/12531-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12980-1">
    <skos:scopeNote xml:lang="de">Aufsuchen, Erschließen und Gewinnen von Bodenschätzen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16754-4">
    <skos:prefLabel xml:lang="en">Information</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16754-4">
    <skos:prefLabel xml:lang="de">Information</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14810-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15103-1">
    <skos:related rdf:resource="descriptor/15107-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15103-1">
    <skos:related rdf:resource="descriptor/15105-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15103-1">
    <skos:related rdf:resource="descriptor/12913-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11909-5">
    <skos:related rdf:resource="descriptor/11910-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11909-5">
    <skos:related rdf:resource="descriptor/11895-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70372">
    <rdfs:label xml:lang="de">Kunststoff- und Gummiwaren</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13443-1">
    <skos:scopeNote xml:lang="de">Wissenschaft von der durch die Landwirtschaft gestalteten Erdoberfläche.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20449-5">
    <skos:related rdf:resource="descriptor/12062-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20449-5">
    <skos:related rdf:resource="descriptor/16438-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10545-1">
    <skos:prefLabel xml:lang="en">Buffer stock</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10545-1">
    <skos:prefLabel xml:lang="de">Buffer Stock</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10889-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15839-1">
    <skos:scopeNote xml:lang="de">Eigene, intrinsische Motivation der Mitarbeiter/innen zur Leistung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11057-2">
    <skos:related rdf:resource="descriptor/11069-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11057-2">
    <skos:related rdf:resource="descriptor/16016-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11057-2">
    <skos:related rdf:resource="descriptor/20077-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11057-2">
    <skos:related rdf:resource="descriptor/19213-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11057-2">
    <skos:related rdf:resource="descriptor/11167-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70429">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">G.03.01.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18775-0">
    <skos:related rdf:resource="descriptor/18869-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18775-0">
    <skos:related rdf:resource="descriptor/12036-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18775-0">
    <skos:related rdf:resource="descriptor/18822-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70467">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71000">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19371-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17804-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11628-6">
    <skos:prefLabel xml:lang="en">Public enterprise</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11628-6">
    <skos:prefLabel xml:lang="de">Öffentliches Unternehmen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11821-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18760-6">
    <skos:prefLabel xml:lang="en">Lease contract</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18760-6">
    <skos:prefLabel xml:lang="de">Leasingvertrag</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19279-0">
    <skos:related rdf:resource="descriptor/11777-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19279-0">
    <skos:related rdf:resource="descriptor/11769-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12761-6">
    <skos:scopeNote xml:lang="de">Algorithmen und heuristische Operations-Research-Verfahren zur Bestimmung einer optimalen Reihenfolge, z.B. in der Produktionsablaufplanung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11684-3">
    <skos:prefLabel xml:lang="en">Taxation principle</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11684-3">
    <skos:prefLabel xml:lang="de">Besteuerungsgrundsatz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17616-4">
    <skos:broader rdf:resource="thsys/70307"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17616-4">
    <skos:broader rdf:resource="descriptor/17985-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17616-4">
    <skos:broader rdf:resource="descriptor/17978-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18926-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12972-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12495-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10025-6">
    <skos:broader rdf:resource="thsys/70021"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16721-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14623-1">
    <skos:prefLabel xml:lang="en">Aromatics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14623-1">
    <skos:prefLabel xml:lang="de">Aromastoff</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70939">
    <skos:broader rdf:resource="thsys/70943"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13200-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12040-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72184">
    <skos:prefLabel xml:lang="de">V.16.03.01  Indexrechnung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19538-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15946-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16387-5">
    <skos:broader rdf:resource="thsys/73378"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16387-5">
    <skos:broader rdf:resource="descriptor/15711-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13571-6">
    <skos:related rdf:resource="descriptor/13584-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13571-6">
    <skos:related rdf:resource="descriptor/15709-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18002-6">
    <skos:prefLabel xml:lang="en">Computer-Integrated Manufacturing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18002-6">
    <skos:prefLabel xml:lang="de">CIM</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10327-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11441-1">
    <skos:altLabel xml:lang="en">Transferable loan facilities</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11441-1">
    <skos:altLabel xml:lang="de">Innovative Finanzierungstechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10623-0">
    <skos:altLabel xml:lang="de">Ost-West-Handel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18594-4">
    <skos:broader rdf:resource="thsys/70936"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18594-4">
    <skos:broader rdf:resource="descriptor/13572-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11937-0">
    <skos:broader rdf:resource="thsys/73362"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11937-0">
    <skos:broader rdf:resource="thsys/71069"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11937-0">
    <skos:broader rdf:resource="descriptor/16552-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11937-0">
    <skos:broader rdf:resource="descriptor/18772-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12590-0">
    <skos:broader rdf:resource="thsys/70245"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12590-0">
    <skos:broader rdf:resource="descriptor/12594-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13284-2">
    <skos:related rdf:resource="descriptor/18502-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13284-2">
    <skos:related rdf:resource="descriptor/11563-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13354-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12026-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13298-5">
    <skos:related rdf:resource="descriptor/16070-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13298-5">
    <skos:related rdf:resource="descriptor/18594-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13298-5">
    <skos:related rdf:resource="descriptor/18593-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13298-5">
    <skos:related rdf:resource="descriptor/14455-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13298-5">
    <skos:related rdf:resource="descriptor/13252-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13298-5">
    <skos:related rdf:resource="descriptor/13553-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10770-1">
    <skos:narrower rdf:resource="descriptor/10758-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18627-1">
    <skos:prefLabel xml:lang="en">Common transport policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18627-1">
    <skos:prefLabel xml:lang="de">EU-Verkehrspolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13653-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18670-0">
    <skos:altLabel xml:lang="en">Securitisation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18670-0">
    <skos:altLabel xml:lang="de">Verbriefung von Forderungen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10415-0">
    <skos:broader rdf:resource="thsys/70406"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10415-0">
    <skos:broader rdf:resource="thsys/70179"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10415-0">
    <skos:broader rdf:resource="descriptor/10434-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12949-2">
    <skos:prefLabel xml:lang="en">Plant production</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12949-2">
    <skos:prefLabel xml:lang="de">Pflanzenbau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12956-5">
    <skos:prefLabel xml:lang="en">Animal breeding</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12956-5">
    <skos:prefLabel xml:lang="de">Tierzucht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18406-3">
    <skos:altLabel xml:lang="de">Stahlmetallurgie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18406-3">
    <skos:altLabel xml:lang="de">Stahlerzeugungsverfahren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13519-1">
    <skos:related rdf:resource="descriptor/13247-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13519-1">
    <skos:related rdf:resource="descriptor/13246-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13519-1">
    <skos:related rdf:resource="descriptor/16170-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18354-3">
    <skos:broader rdf:resource="thsys/71016"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18354-3">
    <skos:broader rdf:resource="thsys/70227"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18354-3">
    <skos:broader rdf:resource="descriptor/12036-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10076-3">
    <skos:altLabel xml:lang="en">Recontracting</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15119-0">
    <skos:prefLabel xml:lang="en">Continuous sample survey</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15119-0">
    <skos:prefLabel xml:lang="de">Laufende Stichprobenerhebung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12039-6">
    <skos:altLabel xml:lang="de">Unternehmensstatistik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12039-6">
    <skos:altLabel xml:lang="de">Betriebswirtschaftliche Statistik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20522-0">
    <skos:related rdf:resource="descriptor/19223-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20522-0">
    <skos:related rdf:resource="descriptor/10208-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20522-0">
    <skos:related rdf:resource="descriptor/11054-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18364-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10344-4">
    <skos:related rdf:resource="descriptor/10348-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10344-4">
    <skos:related rdf:resource="descriptor/10347-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10344-4">
    <skos:related rdf:resource="descriptor/10038-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16539-1">
    <skos:related rdf:resource="descriptor/13823-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16539-1">
    <skos:related rdf:resource="descriptor/16540-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16539-1">
    <skos:related rdf:resource="descriptor/13416-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13165-3">
    <skos:scopeNote xml:lang="de">Nur für die weiterverarbeitende Industrie.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70960">
    <skos:broader rdf:resource="thsys/70962"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14186-4">
    <skos:prefLabel xml:lang="en">Solar energy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14186-4">
    <skos:prefLabel xml:lang="de">Sonnenenergie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73363">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.04.04.03</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10056-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71104">
    <skos:prefLabel xml:lang="de">V.05.06.03  Terminmarkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11379-1">
    <skos:related rdf:resource="descriptor/19010-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11379-1">
    <skos:related rdf:resource="descriptor/11378-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11379-1">
    <skos:related rdf:resource="descriptor/18879-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11379-1">
    <skos:related rdf:resource="descriptor/18878-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11379-1">
    <skos:related rdf:resource="descriptor/11373-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11379-1">
    <skos:related rdf:resource="descriptor/11361-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11379-1">
    <skos:related rdf:resource="descriptor/11356-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70825">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.29</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10256-1">
    <skos:prefLabel xml:lang="en">Theory of production (Macroeconomics)</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10256-1">
    <skos:prefLabel xml:lang="de">Gesamtwirtschaftliche Produktionstheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70265">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13992-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13191-2">
    <skos:altLabel xml:lang="de">Altpapierverwertungswirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13191-2">
    <skos:altLabel xml:lang="de">Altpapier-Recycling-Wirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13191-2">
    <skos:altLabel xml:lang="de">Altpapierhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13191-2">
    <skos:altLabel xml:lang="de">Altpapierentsorgungswirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13191-2">
    <skos:altLabel xml:lang="de">Aktenvernichtungsfirma</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16332-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11812-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15346-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18258-6">
    <skos:broader rdf:resource="thsys/70199"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18258-6">
    <skos:broader rdf:resource="descriptor/14728-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13065-0">
    <skos:broader rdf:resource="thsys/70997"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13065-0">
    <skos:broader rdf:resource="descriptor/13047-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16435-6">
    <skos:scopeNote xml:lang="de">Politik mehrerer Staaten oder einer internationalen Organisation bezüglich bestimmter Politikfelder</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17750-0">
    <skos:narrower rdf:resource="descriptor/17753-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17750-0">
    <skos:narrower rdf:resource="descriptor/17754-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17750-0">
    <skos:narrower rdf:resource="descriptor/17749-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17750-0">
    <skos:narrower rdf:resource="descriptor/17748-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17750-0">
    <skos:narrower rdf:resource="descriptor/17746-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17750-0">
    <skos:narrower rdf:resource="descriptor/17745-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13198-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25743-1">
    <skos:scopeNote xml:lang="de">Schumpeterscher Ansatz der Ökonomie. Keine strukturierte Schule.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13587-5">
    <skos:prefLabel xml:lang="en">Modal split</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13587-5">
    <skos:prefLabel xml:lang="de">Verkehrsteilung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19792-2">
    <skos:related rdf:resource="descriptor/19016-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19792-2">
    <skos:related rdf:resource="descriptor/18445-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19792-2">
    <skos:related rdf:resource="descriptor/14931-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19792-2">
    <skos:related rdf:resource="descriptor/19395-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19792-2">
    <skos:related rdf:resource="descriptor/14611-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13671-2">
    <skos:broader rdf:resource="thsys/71036"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13671-2">
    <skos:broader rdf:resource="descriptor/14023-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13671-2">
    <skos:broader rdf:resource="descriptor/12165-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13423-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13155-6">
    <skos:prefLabel xml:lang="en">Meat industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13155-6">
    <skos:prefLabel xml:lang="de">Fleischindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10164-6">
    <skos:broader rdf:resource="thsys/71118"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10164-6">
    <skos:broader rdf:resource="descriptor/10161-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10714-4">
    <skos:broader rdf:resource="thsys/71096"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10714-4">
    <skos:broader rdf:resource="descriptor/10713-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10714-4">
    <skos:broader rdf:resource="descriptor/10687-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10222-4">
    <skos:related rdf:resource="descriptor/15441-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10222-4">
    <skos:related rdf:resource="descriptor/15468-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10222-4">
    <skos:related rdf:resource="descriptor/15467-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10222-4">
    <skos:related rdf:resource="descriptor/19216-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10222-4">
    <skos:related rdf:resource="descriptor/10194-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18667-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18992-6">
    <skos:prefLabel xml:lang="en">Alcohol policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18992-6">
    <skos:prefLabel xml:lang="de">Alkoholpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15938-6">
    <skos:narrower rdf:resource="descriptor/19759-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16595-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19705-2">
    <skos:prefLabel xml:lang="en">Harmonization of law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19705-2">
    <skos:prefLabel xml:lang="de">Rechtsangleichung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19457-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19427-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71095">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11269-1">
    <skos:related rdf:resource="descriptor/16531-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11269-1">
    <skos:related rdf:resource="descriptor/12615-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11269-1">
    <skos:related rdf:resource="descriptor/11388-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11698-6">
    <skos:related rdf:resource="descriptor/11649-5"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id344">
    <rdf:value xml:lang="de">Für Erhebungen zur Struktur der Einnahmen und Ausgaben privater Haushalte</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13144-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18000-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12766-3">
    <skos:related rdf:resource="descriptor/19140-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12766-3">
    <skos:related rdf:resource="descriptor/19139-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12766-3">
    <skos:related rdf:resource="descriptor/12774-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12766-3">
    <skos:related rdf:resource="descriptor/19121-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12766-3">
    <skos:related rdf:resource="descriptor/12665-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12766-3">
    <skos:related rdf:resource="descriptor/18007-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15970-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17491-5">
    <skos:prefLabel xml:lang="en">Azerbaijan</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17491-5">
    <skos:prefLabel xml:lang="de">Aserbaidschan</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12065-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12180-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18799-0">
    <skos:prefLabel xml:lang="en">Trade fair location</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18799-0">
    <skos:prefLabel xml:lang="de">Messestandort</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18678-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18404-0">
    <skos:broader rdf:resource="thsys/71001"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18404-0">
    <skos:broader rdf:resource="descriptor/13029-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18404-0">
    <skos:broader rdf:resource="descriptor/12917-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12587-3">
    <skos:prefLabel xml:lang="en">Suggestion system</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12587-3">
    <skos:prefLabel xml:lang="de">Vorschlagswesen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19216-3">
    <skos:broader rdf:resource="thsys/70180"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19216-3">
    <skos:broader rdf:resource="descriptor/15860-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11496-4">
    <skos:prefLabel xml:lang="en">Deflation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11496-4">
    <skos:prefLabel xml:lang="de">Deflation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19653-2">
    <skos:prefLabel xml:lang="en">Computer Telephony Integration</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19653-2">
    <skos:prefLabel xml:lang="de">CTI (Computer-Telefon-Integration)</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18871-4">
    <skos:related rdf:resource="descriptor/14017-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18871-4">
    <skos:related rdf:resource="descriptor/10032-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18871-4">
    <skos:related rdf:resource="descriptor/15684-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18871-4">
    <skos:related rdf:resource="descriptor/18880-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11207-2">
    <skos:prefLabel xml:lang="en">Regional labour market</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11207-2">
    <skos:prefLabel xml:lang="de">Regionaler Arbeitsmarkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10614-1">
    <skos:related rdf:resource="descriptor/11206-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10614-1">
    <skos:related rdf:resource="descriptor/10641-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10614-1">
    <skos:related rdf:resource="descriptor/10615-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10614-1">
    <skos:related rdf:resource="descriptor/10570-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10614-1">
    <skos:related rdf:resource="descriptor/10637-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10614-1">
    <skos:related rdf:resource="descriptor/10565-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10614-1">
    <skos:related rdf:resource="descriptor/10822-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18841-6">
    <skos:altLabel xml:lang="de">Theorie des Genossenschaftswesens</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18841-6">
    <skos:altLabel xml:lang="de">Theorie der Genossenschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18841-6">
    <skos:altLabel xml:lang="de">Kooperationswissenschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18841-6">
    <skos:altLabel xml:lang="de">Genossenschaftstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18841-6">
    <skos:altLabel xml:lang="de">Genossenschaftslehre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71050">
    <skos:narrower rdf:resource="descriptor/18166-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71050">
    <skos:narrower rdf:resource="descriptor/11212-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71050">
    <skos:narrower rdf:resource="descriptor/12116-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71050">
    <skos:narrower rdf:resource="descriptor/12620-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71050">
    <skos:narrower rdf:resource="descriptor/12605-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71050">
    <skos:narrower rdf:resource="descriptor/12613-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71050">
    <skos:narrower rdf:resource="descriptor/12628-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71050">
    <skos:narrower rdf:resource="descriptor/13488-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71050">
    <skos:narrower rdf:resource="descriptor/12619-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71050">
    <skos:narrower rdf:resource="descriptor/11199-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71050">
    <skos:narrower rdf:resource="descriptor/12625-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71050">
    <skos:narrower rdf:resource="descriptor/12603-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71050">
    <skos:narrower rdf:resource="descriptor/12119-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71050">
    <skos:narrower rdf:resource="descriptor/12624-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71050">
    <skos:narrower rdf:resource="descriptor/11194-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71050">
    <skos:narrower rdf:resource="descriptor/11192-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71050">
    <skos:narrower rdf:resource="descriptor/16531-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71050">
    <skos:narrower rdf:resource="descriptor/12612-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71050">
    <skos:narrower rdf:resource="descriptor/12618-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71050">
    <skos:narrower rdf:resource="descriptor/11388-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71050">
    <skos:narrower rdf:resource="descriptor/11269-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71050">
    <skos:narrower rdf:resource="descriptor/11248-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71050">
    <skos:narrower rdf:resource="descriptor/11187-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71050">
    <skos:narrower rdf:resource="descriptor/12602-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71050">
    <skos:narrower rdf:resource="descriptor/12610-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71050">
    <skos:narrower rdf:resource="descriptor/12609-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71050">
    <skos:narrower rdf:resource="descriptor/12617-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71050">
    <skos:narrower rdf:resource="descriptor/19111-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71050">
    <skos:narrower rdf:resource="descriptor/12621-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71050">
    <skos:narrower rdf:resource="descriptor/12607-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71050">
    <skos:narrower rdf:resource="descriptor/12631-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71050">
    <skos:narrower rdf:resource="descriptor/11303-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10028-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71090">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14923-3">
    <skos:related rdf:resource="descriptor/14914-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12185-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70998">
    <rdfs:label xml:lang="en">Aircraft and aerospace industry</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70998">
    <rdfs:label xml:lang="de">Luft- und Raumfahrtindustrie</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12988-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11607-0">
    <skos:broader rdf:resource="thsys/71078"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11607-0">
    <skos:broader rdf:resource="descriptor/11540-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17612-5">
    <skos:broader rdf:resource="thsys/70561"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17612-5">
    <skos:broader rdf:resource="descriptor/19167-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18917-6">
    <skos:altLabel xml:lang="de">Theorie der Freizeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18917-6">
    <skos:altLabel xml:lang="de">Freizeitökonomie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11627-1">
    <skos:altLabel xml:lang="en">Municipal enterprise</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11627-1">
    <skos:altLabel xml:lang="de">Kommunaler Eigenbetrieb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11627-1">
    <skos:altLabel xml:lang="de">Gemeindeunternehmen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11627-1">
    <skos:altLabel xml:lang="de">Gemeindebetrieb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11627-1">
    <skos:altLabel xml:lang="de">Eigenbetrieb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11354-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71005">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10647-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12470-3">
    <skos:broader rdf:resource="thsys/70578"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12470-3">
    <skos:broader rdf:resource="descriptor/19108-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10835-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16372-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14106-0">
    <skos:narrower rdf:resource="descriptor/14081-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14106-0">
    <skos:narrower rdf:resource="descriptor/14056-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18161-0">
    <skos:related rdf:resource="descriptor/15064-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18161-0">
    <skos:related rdf:resource="descriptor/15115-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18161-0">
    <skos:related rdf:resource="descriptor/18769-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18161-0">
    <skos:related rdf:resource="descriptor/19072-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13277-6">
    <skos:related rdf:resource="descriptor/15676-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13277-6">
    <skos:related rdf:resource="descriptor/15410-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13277-6">
    <skos:related rdf:resource="descriptor/19608-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11722-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10557-1">
    <skos:related rdf:resource="descriptor/10597-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10557-1">
    <skos:related rdf:resource="descriptor/10595-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10557-1">
    <skos:related rdf:resource="descriptor/10586-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10557-1">
    <skos:related rdf:resource="descriptor/16435-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10557-1">
    <skos:related rdf:resource="descriptor/10560-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10557-1">
    <skos:related rdf:resource="descriptor/16489-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11004-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13054-5">
    <skos:altLabel xml:lang="de">Traktorenindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13054-5">
    <skos:altLabel xml:lang="de">Landmaschinenindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13054-5">
    <skos:altLabel xml:lang="de">Agrarmaschinenindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19797-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17778-6">
    <skos:prefLabel xml:lang="en">Dominica</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17778-6">
    <skos:prefLabel xml:lang="de">Dominica</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13315-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70956">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11274-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14407-0">
    <skos:prefLabel xml:lang="en">Solar power plant</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14407-0">
    <skos:prefLabel xml:lang="de">Solaranlage</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11568-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10786-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10387-0">
    <skos:prefLabel xml:lang="en">Stabilization policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10387-0">
    <skos:prefLabel xml:lang="de">Konjunkturpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14098-1">
    <skos:related rdf:resource="descriptor/19305-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14098-1">
    <skos:related rdf:resource="descriptor/19304-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14098-1">
    <skos:related rdf:resource="descriptor/19303-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14098-1">
    <skos:related rdf:resource="descriptor/19302-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15101-5">
    <skos:broader rdf:resource="thsys/72181"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15101-5">
    <skos:broader rdf:resource="thsys/70970"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15101-5">
    <skos:broader rdf:resource="descriptor/15076-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16295-3">
    <skos:scopeNote xml:lang="de">Nur für allgemeine, tiefgreifende Krisen eines Staates bzw. eines Staatstyps, sowohl politisch als auch durch z.B. Naturkatastrophen verursacht. Auch für rechtliche Maßnahmen im Falle eines Staatsnotstands</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19729-2">
    <skos:scopeNote xml:lang="de">Der Schuldner hat nur mit einem Teil seines Vermögens für die Schuld einzustehen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18535-6">
    <skos:prefLabel xml:lang="en">Service workers</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18535-6">
    <skos:prefLabel xml:lang="de">Dienstleistungsberufe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18506-6">
    <skos:related rdf:resource="descriptor/13852-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18506-6">
    <skos:related rdf:resource="descriptor/18505-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15928-2">
    <skos:prefLabel xml:lang="en">Race</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15928-2">
    <skos:prefLabel xml:lang="de">Rasse</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15477-2">
    <skos:altLabel xml:lang="de">Risikoscheu</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15477-2">
    <skos:altLabel xml:lang="de">Risikofeindlichkeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11862-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16529-4">
    <skos:narrower rdf:resource="descriptor/16541-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14911-3">
    <skos:altLabel xml:lang="de">Mütze</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14911-3">
    <skos:altLabel xml:lang="de">Hut</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18542-2">
    <skos:prefLabel xml:lang="en">International market segmentation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18542-2">
    <skos:prefLabel xml:lang="de">Internationale Marktsegmentierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19464-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70241">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.05.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12932-5">
    <skos:related rdf:resource="descriptor/13305-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12932-5">
    <skos:related rdf:resource="descriptor/11845-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12932-5">
    <skos:related rdf:resource="descriptor/13848-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12932-5">
    <skos:related rdf:resource="descriptor/18503-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12932-5">
    <skos:related rdf:resource="descriptor/18502-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12932-5">
    <skos:related rdf:resource="descriptor/18749-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12932-5">
    <skos:related rdf:resource="descriptor/12014-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10201-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11206-4">
    <skos:broader rdf:resource="thsys/70058"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11206-4">
    <skos:broader rdf:resource="descriptor/11186-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10975-4">
    <skos:prefLabel xml:lang="en">Public choice</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10975-4">
    <skos:prefLabel xml:lang="de">Public Choice</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12667-5">
    <skos:prefLabel xml:lang="en">Make or buy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12667-5">
    <skos:prefLabel xml:lang="de">Make or Buy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17179-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19615-3">
    <skos:broader rdf:resource="thsys/73351"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19615-3">
    <skos:broader rdf:resource="descriptor/15889-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19757-4">
    <skos:broader rdf:resource="thsys/73358"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19757-4">
    <skos:broader rdf:resource="descriptor/16554-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14817-2">
    <skos:broader rdf:resource="thsys/70372"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15972-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10963-4">
    <skos:prefLabel xml:lang="en">Competition policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10963-4">
    <skos:prefLabel xml:lang="de">Wettbewerbspolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71113">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id1">
    <rdfs:seeAlso rdf:resource="descriptor/13325-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10034-5">
    <skos:broader rdf:resource="thsys/70180"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10034-5">
    <skos:broader rdf:resource="descriptor/10038-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16966-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11399-2">
    <skos:prefLabel xml:lang="en">Educational leave</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11399-2">
    <skos:prefLabel xml:lang="de">Bildungsurlaub</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16278-3">
    <skos:broader rdf:resource="thsys/73366"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72186">
    <skos:broader rdf:resource="thsys/72180"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18508-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70258">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14984-4">
    <skos:related rdf:resource="descriptor/18232-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70853">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.23.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10312-3">
    <skos:altLabel xml:lang="de">Turgot-Produktionsfunktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12849-6">
    <skos:prefLabel xml:lang="en">Tax credit system</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12849-6">
    <skos:prefLabel xml:lang="de">Steuerliches Anrechnungsverfahren</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11729-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18586-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10969-6">
    <skos:scopeNote xml:lang="de">Internationale Abstimmung nationaler Wirtschaftspolitiken.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14041-2">
    <skos:prefLabel xml:lang="en">Agricultural equipment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14041-2">
    <skos:prefLabel xml:lang="de">Landwirtschaftliches Gerät</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11301-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16510-4">
    <skos:broader rdf:resource="thsys/73373"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16510-4">
    <skos:broader rdf:resource="thsys/70862"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18749-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20789-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18411-3">
    <skos:broader rdf:resource="thsys/71002"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18411-3">
    <skos:broader rdf:resource="thsys/70989"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18411-3">
    <skos:broader rdf:resource="descriptor/13058-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18411-3">
    <skos:broader rdf:resource="descriptor/12919-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12085-6">
    <skos:altLabel xml:lang="de">Unternehmensleitung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12085-6">
    <skos:altLabel xml:lang="de">Unternehmensführung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12085-6">
    <skos:altLabel xml:lang="de">Geschäftsleitung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12085-6">
    <skos:altLabel xml:lang="de">Firmenleitung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12085-6">
    <skos:altLabel xml:lang="de">Dispositiver Faktor</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12085-6">
    <skos:altLabel xml:lang="de">Betriebsleitung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12085-6">
    <skos:altLabel xml:lang="de">Betriebsführung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18562-3">
    <skos:altLabel xml:lang="de">Kühltechnik für Gastronomie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18562-3">
    <skos:altLabel xml:lang="de">Gastrotechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18562-3">
    <skos:altLabel xml:lang="de">Gastronomiezulieferer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18562-3">
    <skos:altLabel xml:lang="de">Cafeteria-Bedarf</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12103-2">
    <skos:broader rdf:resource="thsys/73386"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12103-2">
    <skos:broader rdf:resource="thsys/73385"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12103-2">
    <skos:broader rdf:resource="thsys/71035"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12103-2">
    <skos:broader rdf:resource="descriptor/15811-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16815-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19734-2">
    <skos:related rdf:resource="descriptor/16687-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19734-2">
    <skos:related rdf:resource="descriptor/18130-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19734-2">
    <skos:related rdf:resource="descriptor/11111-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19734-2">
    <skos:related rdf:resource="descriptor/15860-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19734-2">
    <skos:related rdf:resource="descriptor/19219-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12368-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17669-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14024-2">
    <skos:broader rdf:resource="thsys/70201"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14024-2">
    <skos:broader rdf:resource="descriptor/14023-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16722-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70073">
    <skos:prefLabel xml:lang="en">G.01.03.02  Central-Eastern Europe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70073">
    <skos:prefLabel xml:lang="de">G.01.03.02  Ostmitteleuropa</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14024-2">
    <skos:prefLabel xml:lang="en">Office supplies</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14024-2">
    <skos:prefLabel xml:lang="de">Büromaterial</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70921">
    <skos:narrower rdf:resource="descriptor/18656-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70921">
    <skos:narrower rdf:resource="descriptor/13665-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70921">
    <skos:narrower rdf:resource="descriptor/13656-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70921">
    <skos:narrower rdf:resource="descriptor/13655-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70921">
    <skos:narrower rdf:resource="descriptor/19658-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19469-2">
    <skos:altLabel xml:lang="de">Getto</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18360-1">
    <skos:broader rdf:resource="thsys/71015"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18360-1">
    <skos:broader rdf:resource="thsys/71012"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18360-1">
    <skos:broader rdf:resource="thsys/70539"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18360-1">
    <skos:broader rdf:resource="descriptor/11810-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17642-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19787-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10940-2">
    <skos:broader rdf:resource="thsys/70956"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10940-2">
    <skos:broader rdf:resource="thsys/70459"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10940-2">
    <skos:broader rdf:resource="thsys/70212"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10940-2">
    <skos:broader rdf:resource="descriptor/10942-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12195-6">
    <skos:broader rdf:resource="thsys/73391"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12195-6">
    <skos:broader rdf:resource="thsys/71040"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12195-6">
    <skos:broader rdf:resource="thsys/70087"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16524-0">
    <skos:altLabel xml:lang="de">Zivilschutz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16524-0">
    <skos:altLabel xml:lang="de">Selbstschutz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16524-0">
    <skos:altLabel xml:lang="de">Katastrophenvorbeugung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16524-0">
    <skos:altLabel xml:lang="de">Katastrophenschutz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16524-0">
    <skos:altLabel xml:lang="de">Humanitäre Hilfe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16524-0">
    <skos:altLabel xml:lang="de">Evakuierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18678-5">
    <skos:prefLabel xml:lang="en">Holding period</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18678-5">
    <skos:prefLabel xml:lang="de">Laufzeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16305-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14153-5">
    <skos:prefLabel xml:lang="en">Deciduous wood</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14153-5">
    <skos:prefLabel xml:lang="de">Laubholz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16760-2">
    <skos:broader rdf:resource="thsys/70823"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16760-2">
    <skos:broader rdf:resource="descriptor/15685-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19230-2">
    <skos:altLabel xml:lang="en">MONASH model</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13735-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70073">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11787-0">
    <skos:related rdf:resource="descriptor/13474-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11787-0">
    <skos:related rdf:resource="descriptor/12759-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11787-0">
    <skos:related rdf:resource="descriptor/10241-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11787-0">
    <skos:related rdf:resource="descriptor/19231-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16471-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19434-0">
    <skos:altLabel xml:lang="en">Antitrust case</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19434-0">
    <skos:altLabel xml:lang="de">Kartellgericht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16214-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13511-3">
    <skos:broader rdf:resource="thsys/73387"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13511-3">
    <skos:broader rdf:resource="thsys/70929"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13511-3">
    <skos:broader rdf:resource="thsys/70109"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13511-3">
    <skos:broader rdf:resource="descriptor/13573-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13511-3">
    <skos:broader rdf:resource="descriptor/15690-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18209-5">
    <skos:related rdf:resource="descriptor/19746-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18417-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13736-0">
    <skos:broader rdf:resource="thsys/71105"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13736-0">
    <skos:broader rdf:resource="descriptor/12234-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17017-5">
    <skos:prefLabel xml:lang="en">Vatican City State</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17017-5">
    <skos:prefLabel xml:lang="de">Vatikanstadt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14485-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70035">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">G.01.03.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16985-6">
    <skos:broader rdf:resource="thsys/70505"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16985-6">
    <skos:broader rdf:resource="descriptor/16990-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16985-6">
    <skos:broader rdf:resource="descriptor/17991-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16985-6">
    <skos:broader rdf:resource="descriptor/17982-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18111-1">
    <skos:scopeNote xml:lang="de">Informationstechnisch unterstützte Büroarbeit.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id303">
    <rdfs:seeAlso rdf:resource="descriptor/11024-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18957-1">
    <skos:related rdf:resource="descriptor/11543-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12094-5">
    <skos:prefLabel xml:lang="en">Corporate restructuring</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12094-5">
    <skos:prefLabel xml:lang="de">Reorganisation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70958">
    <skos:broader rdf:resource="thsys/w"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11441-1">
    <skos:related rdf:resource="descriptor/18727-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11441-1">
    <skos:related rdf:resource="descriptor/24740-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11441-1">
    <skos:related rdf:resource="descriptor/18088-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11441-1">
    <skos:related rdf:resource="descriptor/18667-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10049-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17639-6">
    <skos:broader rdf:resource="thsys/70253"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17639-6">
    <skos:broader rdf:resource="descriptor/17608-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16595-5">
    <skos:prefLabel xml:lang="en">Social network</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16595-5">
    <skos:prefLabel xml:lang="de">Soziales Netzwerk</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13697-5">
    <skos:altLabel xml:lang="en">Universal bank</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13697-5">
    <skos:altLabel xml:lang="de">Universalbankensystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10787-5">
    <skos:narrower rdf:resource="descriptor/10784-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15062-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20314-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19488-5">
    <skos:broader rdf:resource="thsys/70811"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19488-5">
    <skos:broader rdf:resource="descriptor/18381-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19341-0">
    <skos:broader rdf:resource="thsys/71026"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19341-0">
    <skos:broader rdf:resource="thsys/70960"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19341-0">
    <skos:broader rdf:resource="descriptor/13431-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12302-3">
    <skos:broader rdf:resource="thsys/70037"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18381-0">
    <skos:broader rdf:resource="thsys/71010"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18381-0">
    <skos:broader rdf:resource="thsys/70811"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18381-0">
    <skos:broader rdf:resource="descriptor/15767-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11959-4">
    <skos:related rdf:resource="descriptor/11956-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11959-4">
    <skos:related rdf:resource="descriptor/11954-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19403-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12285-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11853-6">
    <skos:related rdf:resource="descriptor/11855-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11853-6">
    <skos:related rdf:resource="descriptor/15878-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70178">
    <skos:narrower rdf:resource="descriptor/11111-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70178">
    <skos:narrower rdf:resource="descriptor/10211-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70178">
    <skos:narrower rdf:resource="descriptor/10220-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70178">
    <skos:narrower rdf:resource="descriptor/19225-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70178">
    <skos:narrower rdf:resource="descriptor/19224-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70178">
    <skos:narrower rdf:resource="descriptor/10219-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70178">
    <skos:narrower rdf:resource="descriptor/19223-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70178">
    <skos:narrower rdf:resource="descriptor/10217-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70178">
    <skos:narrower rdf:resource="descriptor/19222-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70178">
    <skos:narrower rdf:resource="descriptor/10216-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70178">
    <skos:narrower rdf:resource="descriptor/10215-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70178">
    <skos:narrower rdf:resource="descriptor/10214-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70178">
    <skos:narrower rdf:resource="descriptor/10213-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70178">
    <skos:narrower rdf:resource="descriptor/11092-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70178">
    <skos:narrower rdf:resource="descriptor/10204-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70178">
    <skos:narrower rdf:resource="descriptor/10199-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70178">
    <skos:narrower rdf:resource="descriptor/10198-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70178">
    <skos:narrower rdf:resource="descriptor/11079-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70178">
    <skos:narrower rdf:resource="descriptor/10196-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19420-4">
    <skos:prefLabel xml:lang="en">Poll tax</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19420-4">
    <skos:prefLabel xml:lang="de">Kopfsteuer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16119-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18498-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18121-5">
    <skos:prefLabel xml:lang="en">International business cooperation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18121-5">
    <skos:prefLabel xml:lang="de">Internationale Unternehmenskooperation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18220-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11160-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11665-0">
    <skos:broader rdf:resource="thsys/71074"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11665-0">
    <skos:broader rdf:resource="thsys/70850"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11665-0">
    <skos:broader rdf:resource="descriptor/11547-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12986-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10949-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16440-6">
    <skos:altLabel xml:lang="de">Verhandlungstechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16440-6">
    <skos:altLabel xml:lang="de">Verhandlungstaktik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16440-6">
    <skos:altLabel xml:lang="de">Verhandlungsstrategie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16440-6">
    <skos:altLabel xml:lang="de">Verhandlung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16440-6">
    <skos:altLabel xml:lang="de">Internationale Verhandlungen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12230-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73389">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.05.04</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10828-3">
    <skos:related rdf:resource="descriptor/12849-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10828-3">
    <skos:related rdf:resource="descriptor/12874-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10828-3">
    <skos:related rdf:resource="descriptor/12863-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10828-3">
    <skos:related rdf:resource="descriptor/10827-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16400-4">
    <skos:prefLabel xml:lang="en">Antisemitism</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16400-4">
    <skos:prefLabel xml:lang="de">Antisemitismus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10661-6">
    <skos:related rdf:resource="descriptor/10653-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14900-1">
    <skos:broader rdf:resource="thsys/70031"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14900-1">
    <skos:broader rdf:resource="descriptor/14899-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73396">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.06.02.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15873-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12489-3">
    <skos:prefLabel xml:lang="en">Value analysis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12489-3">
    <skos:prefLabel xml:lang="de">Wertanalyse</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70897">
    <rdfs:label xml:lang="en">Financial economics</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70897">
    <rdfs:label xml:lang="de">Kapitalmarkttheorie</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18694-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18247-4">
    <skos:broader rdf:resource="thsys/70415"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18247-4">
    <skos:broader rdf:resource="descriptor/14455-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19611-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18679-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71107">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19586-5">
    <skos:prefLabel xml:lang="en">Investment statistics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19586-5">
    <skos:prefLabel xml:lang="de">Investitionsstatistik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19686-1">
    <skos:related rdf:resource="descriptor/11068-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19686-1">
    <skos:related rdf:resource="descriptor/10980-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19686-1">
    <skos:related rdf:resource="descriptor/11721-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19686-1">
    <skos:related rdf:resource="descriptor/13420-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19686-1">
    <skos:related rdf:resource="descriptor/10894-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19686-1">
    <skos:related rdf:resource="descriptor/18822-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19686-1">
    <skos:related rdf:resource="descriptor/16226-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19686-1">
    <skos:related rdf:resource="descriptor/10970-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19686-1">
    <skos:related rdf:resource="descriptor/20842-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16275-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70192">
    <skos:broader rdf:resource="thsys/n"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17686-4">
    <skos:altLabel xml:lang="de">Südrhodesien</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17686-4">
    <skos:altLabel xml:lang="de">Simbabwe-Rhodesien</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17686-4">
    <skos:altLabel xml:lang="de">Rhodesien</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17686-4">
    <skos:altLabel xml:lang="de">Republik Simbabwe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19170-1">
    <skos:related rdf:resource="descriptor/17990-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11281-4">
    <skos:altLabel xml:lang="de">Überbeschäftigung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11281-4">
    <skos:altLabel xml:lang="de">Mehrarbeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10177-4">
    <skos:broader rdf:resource="thsys/70814"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10177-4">
    <skos:broader rdf:resource="thsys/70493"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10177-4">
    <skos:broader rdf:resource="descriptor/10144-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19093-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17015-2">
    <skos:altLabel xml:lang="x-other">Repubblica di San Marino</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17015-2">
    <skos:altLabel xml:lang="de">Republik San Marino</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20442-5">
    <skos:broader rdf:resource="thsys/70368"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20442-5">
    <skos:broader rdf:resource="descriptor/18395-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70031">
    <skos:prefLabel xml:lang="en">P.19  Clothing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70031">
    <skos:prefLabel xml:lang="de">P.19  Bekleidung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18599-1">
    <skos:related rdf:resource="descriptor/13983-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18599-1">
    <skos:related rdf:resource="descriptor/18603-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18599-1">
    <skos:related rdf:resource="descriptor/13537-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18304-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10342-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18547-6">
    <skos:broader rdf:resource="thsys/70957"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18547-6">
    <skos:broader rdf:resource="descriptor/10012-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19284-0">
    <skos:related rdf:resource="descriptor/19384-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19284-0">
    <skos:related rdf:resource="descriptor/11127-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18178-4">
    <skos:altLabel xml:lang="de">Tilgung von Krediten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18178-4">
    <skos:altLabel xml:lang="de">Schuldentilgung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18178-4">
    <skos:altLabel xml:lang="de">Rückzahlung von Krediten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18178-4">
    <skos:altLabel xml:lang="de">Rückerstattung von Krediten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18178-4">
    <skos:altLabel xml:lang="de">Darlehenstilgung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18701-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id73">
    <rdfs:seeAlso rdf:resource="descriptor/10901-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14149-3">
    <skos:altLabel xml:lang="de">Naturharz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14149-3">
    <skos:altLabel xml:lang="de">Harz (Naturharz)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14149-3">
    <skos:altLabel xml:lang="de">Gummiharz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14149-3">
    <skos:altLabel xml:lang="de">Forstwirtschaftliches Produkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14149-3">
    <skos:altLabel xml:lang="de">Baumharz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19808-6">
    <skos:altLabel xml:lang="de">Neuronales Netzwerk</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19808-6">
    <skos:altLabel xml:lang="de">Neuronales Netz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19808-6">
    <skos:altLabel xml:lang="de">Neuronaler Algorithmus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19808-6">
    <skos:altLabel xml:lang="de">Neuro-Informatik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12631-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12206-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12967-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10300-3">
    <skos:narrower rdf:resource="descriptor/19846-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10300-3">
    <skos:narrower rdf:resource="descriptor/18932-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10300-3">
    <skos:narrower rdf:resource="descriptor/13485-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13019-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12848-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13591-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15698-0">
    <skos:prefLabel xml:lang="en">Slavery</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15698-0">
    <skos:prefLabel xml:lang="de">Sklaverei</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10676-0">
    <skos:narrower rdf:resource="descriptor/10677-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10676-0">
    <skos:narrower rdf:resource="descriptor/10678-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10676-0">
    <skos:narrower rdf:resource="descriptor/10672-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10676-0">
    <skos:narrower rdf:resource="descriptor/19313-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10676-0">
    <skos:narrower rdf:resource="descriptor/10668-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10676-0">
    <skos:narrower rdf:resource="descriptor/10642-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14586-2">
    <skos:broader rdf:resource="thsys/70201"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12656-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13364-4">
    <skos:broader rdf:resource="thsys/70828"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13364-4">
    <skos:broader rdf:resource="descriptor/18920-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70259">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.05.05</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73390">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#altLabel">
    <rdfs:label xml:lang="en">alternative label</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16299-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18121-5">
    <skos:related rdf:resource="descriptor/10626-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18121-5">
    <skos:related rdf:resource="descriptor/10472-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18121-5">
    <skos:related rdf:resource="descriptor/20503-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18121-5">
    <skos:related rdf:resource="descriptor/19071-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18121-5">
    <skos:related rdf:resource="descriptor/10823-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12103-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16564-2">
    <skos:prefLabel xml:lang="en">Sociology</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16564-2">
    <skos:prefLabel xml:lang="de">Soziologie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13584-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18104-5">
    <skos:prefLabel xml:lang="en">Supplier</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18104-5">
    <skos:prefLabel xml:lang="de">Lieferant</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15821-6">
    <skos:broader rdf:resource="thsys/70292"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15821-6">
    <skos:broader rdf:resource="thsys/70120"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18594-4">
    <skos:altLabel xml:lang="de">Seeunfall</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18594-4">
    <skos:altLabel xml:lang="de">Havarie von Schiffen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11937-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10459-1">
    <skos:altLabel xml:lang="de">Produktinnovation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10459-1">
    <skos:altLabel xml:lang="de">Produktideenfindung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10459-1">
    <skos:altLabel xml:lang="de">Produktentwicklung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10459-1">
    <skos:altLabel xml:lang="de">Produkteinführung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10459-1">
    <skos:altLabel xml:lang="de">Neuproduktentwicklung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10459-1">
    <skos:altLabel xml:lang="de">Neues Produkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10459-1">
    <skos:altLabel xml:lang="de">Markteinführung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10459-1">
    <skos:altLabel xml:lang="de">Betriebliche Innovation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10092-5">
    <skos:related rdf:resource="descriptor/15767-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10092-5">
    <skos:related rdf:resource="descriptor/10058-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10092-5">
    <skos:related rdf:resource="descriptor/11718-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10092-5">
    <skos:related rdf:resource="descriptor/11054-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19237-2">
    <skos:prefLabel xml:lang="en">Social accounting matrix</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19237-2">
    <skos:prefLabel xml:lang="de">Social Accounting Matrix</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15969-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16349-6">
    <skos:altLabel xml:lang="de">Wahlen zum Europäischen Parlament</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11819-6">
    <skos:broader rdf:resource="thsys/70965"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11819-6">
    <skos:broader rdf:resource="descriptor/10973-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15077-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14930-6">
    <skos:prefLabel xml:lang="en">Preserved food</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14930-6">
    <skos:prefLabel xml:lang="de">Konserve</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12829-5">
    <skos:narrower rdf:resource="descriptor/12847-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12829-5">
    <skos:narrower rdf:resource="descriptor/12825-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12829-5">
    <skos:narrower rdf:resource="descriptor/21315-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12829-5">
    <skos:narrower rdf:resource="descriptor/12823-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12829-5">
    <skos:narrower rdf:resource="descriptor/18206-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14549-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13228-5">
    <skos:prefLabel xml:lang="en">Consumer electronics trade</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13228-5">
    <skos:prefLabel xml:lang="de">UE-Handel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18503-5">
    <skos:altLabel xml:lang="en">Nonprofit housing company</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18503-5">
    <skos:altLabel xml:lang="de">Wohnungsgenossenschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18503-5">
    <skos:altLabel xml:lang="de">Genossenschaftliche Wohnungswirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18503-5">
    <skos:altLabel xml:lang="de">Baugenossenschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13776-2">
    <skos:altLabel xml:lang="de">Kreditvergabepolitik der Banken</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13776-2">
    <skos:altLabel xml:lang="de">Kreditpolitik der Banken</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13776-2">
    <skos:altLabel xml:lang="de">Kreditleihgeschäft der Banken</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13776-2">
    <skos:altLabel xml:lang="de">Geldleihgeschäft der Banken</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19652-4">
    <skos:altLabel xml:lang="de">Technologiebörse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19652-4">
    <skos:altLabel xml:lang="de">Nasdaq</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20881-0">
    <skos:prefLabel xml:lang="en">Marginal costs</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20881-0">
    <skos:prefLabel xml:lang="de">Grenzkosten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17624-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19660-5">
    <skos:scopeNote xml:lang="de">Vgl. auch Computergestütztes Verfahren, Informationssystem, Datenverarbeitung, Mikroelektronik und die jeweiligen Unterbegriffe.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70992">
    <rdfs:label xml:lang="en">Petroleum industry</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70992">
    <rdfs:label xml:lang="de">Mineralölwirtschaft</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10891-3">
    <skos:related rdf:resource="descriptor/10913-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10891-3">
    <skos:related rdf:resource="descriptor/10956-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10891-3">
    <skos:related rdf:resource="descriptor/10963-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10891-3">
    <skos:related rdf:resource="descriptor/10942-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10891-3">
    <skos:related rdf:resource="descriptor/10895-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10891-3">
    <skos:related rdf:resource="descriptor/10190-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10891-3">
    <skos:related rdf:resource="descriptor/10894-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10891-3">
    <skos:related rdf:resource="descriptor/10217-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10891-3">
    <skos:related rdf:resource="descriptor/10928-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14762-1">
    <skos:related rdf:resource="descriptor/15410-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14762-1">
    <skos:related rdf:resource="descriptor/18022-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13177-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15807-0">
    <skos:altLabel xml:lang="en">Role model</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15807-0">
    <skos:altLabel xml:lang="de">Rollenverständnis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15807-0">
    <skos:altLabel xml:lang="de">Rollenverhalten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15807-0">
    <skos:altLabel xml:lang="de">Rollentheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18862-5">
    <skos:prefLabel xml:lang="en">Girls</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18862-5">
    <skos:prefLabel xml:lang="de">Mädchen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10736-1">
    <skos:related rdf:resource="descriptor/10761-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10736-1">
    <skos:related rdf:resource="descriptor/10735-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13757-6">
    <skos:related rdf:resource="descriptor/10773-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13757-6">
    <skos:related rdf:resource="descriptor/10781-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13757-6">
    <skos:related rdf:resource="descriptor/10780-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13757-6">
    <skos:related rdf:resource="descriptor/10779-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13757-6">
    <skos:related rdf:resource="descriptor/10776-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15962-2">
    <skos:related rdf:resource="descriptor/15979-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15962-2">
    <skos:related rdf:resource="descriptor/13995-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15962-2">
    <skos:related rdf:resource="descriptor/13410-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15478-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18623-2">
    <skos:broader rdf:resource="thsys/73351"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18623-2">
    <skos:broader rdf:resource="thsys/70929"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18623-2">
    <skos:broader rdf:resource="thsys/70239"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18623-2">
    <skos:broader rdf:resource="descriptor/13573-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18623-2">
    <skos:broader rdf:resource="descriptor/15765-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19348-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11823-1">
    <skos:prefLabel xml:lang="en">Commercial policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11823-1">
    <skos:prefLabel xml:lang="de">Handelspolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10250-6">
    <skos:narrower rdf:resource="descriptor/11479-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19693-4">
    <skos:altLabel xml:lang="de">Abgeordnetenentschädigung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19693-4">
    <skos:altLabel xml:lang="de">Abgeordnetendiäten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70818">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11677-0">
    <skos:prefLabel xml:lang="en">Tobacco tax</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11677-0">
    <skos:prefLabel xml:lang="de">Tabaksteuer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12874-0">
    <skos:prefLabel xml:lang="en">Interlocking rights</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12874-0">
    <skos:prefLabel xml:lang="de">Schachtelprivileg</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19621-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10198-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12085-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12451-0">
    <skos:scopeNote xml:lang="de">Nur für Konsolidierung von Konzernabschlüssen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73338">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12427-4">
    <skos:related rdf:resource="descriptor/19673-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12427-4">
    <skos:related rdf:resource="descriptor/12438-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12427-4">
    <skos:related rdf:resource="descriptor/19411-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12427-4">
    <skos:related rdf:resource="descriptor/19934-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18022-0">
    <skos:broader rdf:resource="thsys/70956"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18022-0">
    <skos:broader rdf:resource="thsys/70900"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18022-0">
    <skos:broader rdf:resource="thsys/70114"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18022-0">
    <skos:broader rdf:resource="descriptor/18712-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18022-0">
    <skos:broader rdf:resource="descriptor/13793-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16630-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19153-1">
    <skos:altLabel xml:lang="de">Pretiale Betriebslenkung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19153-1">
    <skos:altLabel xml:lang="de">Indirekte Betriebslenkung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11727-4">
    <skos:related rdf:resource="descriptor/10398-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14777-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13252-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10725-6">
    <skos:prefLabel xml:lang="en">Capital controls policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10725-6">
    <skos:prefLabel xml:lang="de">Kapitalverkehrspolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10376-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16645-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11399-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10081-3">
    <skos:related rdf:resource="descriptor/10080-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10081-3">
    <skos:related rdf:resource="descriptor/10073-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10081-3">
    <skos:related rdf:resource="descriptor/10070-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19846-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16422-1">
    <skos:related rdf:resource="descriptor/16389-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16422-1">
    <skos:related rdf:resource="descriptor/19688-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16422-1">
    <skos:related rdf:resource="descriptor/19714-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16422-1">
    <skos:related rdf:resource="descriptor/16421-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16422-1">
    <skos:related rdf:resource="descriptor/16311-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16422-1">
    <skos:related rdf:resource="descriptor/16416-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16422-1">
    <skos:related rdf:resource="descriptor/16264-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16422-1">
    <skos:related rdf:resource="descriptor/16297-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16422-1">
    <skos:related rdf:resource="descriptor/16267-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13630-2">
    <skos:broader rdf:resource="thsys/70939"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13630-2">
    <skos:broader rdf:resource="thsys/70937"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13630-2">
    <skos:broader rdf:resource="descriptor/18578-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13630-2">
    <skos:broader rdf:resource="descriptor/13558-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70931">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10774-0">
    <skos:altLabel xml:lang="en">Reserve bank</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10774-0">
    <skos:altLabel xml:lang="en">National bank</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10774-0">
    <skos:altLabel xml:lang="de">Notenbank</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10774-0">
    <skos:altLabel xml:lang="de">Nationalbank</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71089">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.07.07</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10237-5">
    <skos:prefLabel xml:lang="en">Aggregate demand</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10237-5">
    <skos:prefLabel xml:lang="de">Gesamtwirtschaftliche Nachfrage</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#related">
    <rdfs:isDefinedBy rdf:resource="http://www.w3.org/2004/02/skos/core"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18245-1">
    <skos:broader rdf:resource="thsys/70549"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18245-1">
    <skos:broader rdf:resource="descriptor/12737-3"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id281">
    <rdf:value xml:lang="de">Für wirtschaftlichen Regionalismus</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11827-0">
    <skos:scopeNote xml:lang="de">Nicht für Verkehrsreduktion allgemein.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13398-1">
    <skos:related rdf:resource="descriptor/18975-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13398-1">
    <skos:related rdf:resource="descriptor/18301-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13398-1">
    <skos:related rdf:resource="descriptor/18969-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13398-1">
    <skos:related rdf:resource="descriptor/13974-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13398-1">
    <skos:related rdf:resource="descriptor/10177-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13398-1">
    <skos:related rdf:resource="descriptor/18948-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13398-1">
    <skos:related rdf:resource="descriptor/11032-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13398-1">
    <skos:related rdf:resource="descriptor/18144-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13398-1">
    <skos:related rdf:resource="descriptor/11660-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13398-1">
    <skos:related rdf:resource="descriptor/15938-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13398-1">
    <skos:related rdf:resource="descriptor/18971-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18456-2">
    <skos:scopeNote xml:lang="de">Nur für Literatur über Wirtschaftszweigsystematiken.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12281-6">
    <skos:related rdf:resource="descriptor/18681-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12281-6">
    <skos:related rdf:resource="descriptor/18105-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12281-6">
    <skos:related rdf:resource="descriptor/12283-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12281-6">
    <skos:related rdf:resource="descriptor/12280-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12281-6">
    <skos:related rdf:resource="descriptor/16061-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12281-6">
    <skos:related rdf:resource="descriptor/12287-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12281-6">
    <skos:related rdf:resource="descriptor/11414-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14102-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19283-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11529-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14020-3">
    <skos:broader rdf:resource="thsys/70201"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14020-3">
    <skos:broader rdf:resource="descriptor/14021-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15729-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10008-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18965-2">
    <skos:prefLabel xml:lang="en">Packaging recycling</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18965-2">
    <skos:prefLabel xml:lang="de">Verpackungs-Recycling</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12962-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15761-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10748-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19858-5">
    <skos:prefLabel xml:lang="en">Weapons law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19858-5">
    <skos:prefLabel xml:lang="de">Waffenrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11071-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10380-0">
    <skos:broader rdf:resource="thsys/71113"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10380-0">
    <skos:broader rdf:resource="descriptor/11779-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21020-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14702-5">
    <skos:narrower rdf:resource="descriptor/18078-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14702-5">
    <skos:narrower rdf:resource="descriptor/14716-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14702-5">
    <skos:narrower rdf:resource="descriptor/14710-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14702-5">
    <skos:narrower rdf:resource="descriptor/18230-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14702-5">
    <skos:narrower rdf:resource="descriptor/14703-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14702-5">
    <skos:narrower rdf:resource="descriptor/14700-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12621-1">
    <skos:altLabel xml:lang="de">Betriebsinterne Arbeitsteilung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12621-1">
    <skos:altLabel xml:lang="de">Arbeitstechnologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11142-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11829-3">
    <skos:altLabel xml:lang="de">Verkehrsinfrastrukturinvestition</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10636-5">
    <skos:altLabel xml:lang="de">Wohlfahrtsverlust des Außenhandels</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19237-2">
    <skos:related rdf:resource="descriptor/10058-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19237-2">
    <skos:related rdf:resource="descriptor/12394-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10589-2">
    <skos:prefLabel xml:lang="en">Private aid</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10589-2">
    <skos:prefLabel xml:lang="de">Private Entwicklungshilfe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12605-6">
    <skos:related rdf:resource="descriptor/15757-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12605-6">
    <skos:related rdf:resource="descriptor/18368-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12605-6">
    <skos:related rdf:resource="descriptor/19529-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12605-6">
    <skos:related rdf:resource="descriptor/15013-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12605-6">
    <skos:related rdf:resource="descriptor/12976-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12605-6">
    <skos:related rdf:resource="descriptor/16001-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71046">
    <skos:prefLabel xml:lang="de">B.02.01.01  Eigenkapitalbeschaffung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19682-2">
    <skos:altLabel xml:lang="de">Plutokratie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11973-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11893-1">
    <skos:broader rdf:resource="thsys/70939"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11893-1">
    <skos:broader rdf:resource="thsys/70925"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11893-1">
    <skos:broader rdf:resource="thsys/70058"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11893-1">
    <skos:broader rdf:resource="descriptor/13632-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14486-6">
    <skos:broader rdf:resource="thsys/70135"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14486-6">
    <skos:broader rdf:resource="descriptor/14473-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18809-2">
    <skos:related rdf:resource="descriptor/13421-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18809-2">
    <skos:related rdf:resource="descriptor/16324-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18809-2">
    <skos:related rdf:resource="descriptor/16551-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18809-2">
    <skos:related rdf:resource="descriptor/15995-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12950-3">
    <skos:related rdf:resource="descriptor/18220-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12950-3">
    <skos:related rdf:resource="descriptor/19344-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12950-3">
    <skos:related rdf:resource="descriptor/19343-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12950-3">
    <skos:related rdf:resource="descriptor/14142-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12950-3">
    <skos:related rdf:resource="descriptor/12946-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10025-6">
    <skos:related rdf:resource="descriptor/10032-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10025-6">
    <skos:related rdf:resource="descriptor/10062-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10025-6">
    <skos:related rdf:resource="descriptor/11782-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10025-6">
    <skos:related rdf:resource="descriptor/10031-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10025-6">
    <skos:related rdf:resource="descriptor/10987-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10025-6">
    <skos:related rdf:resource="descriptor/10342-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10025-6">
    <skos:related rdf:resource="descriptor/15075-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10025-6">
    <skos:related rdf:resource="descriptor/19666-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10274-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11147-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11198-5">
    <skos:related rdf:resource="descriptor/16692-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11198-5">
    <skos:related rdf:resource="descriptor/18487-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11198-5">
    <skos:related rdf:resource="descriptor/18486-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11198-5">
    <skos:related rdf:resource="descriptor/18484-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12998-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14143-1">
    <skos:related rdf:resource="descriptor/14130-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14143-1">
    <skos:related rdf:resource="descriptor/12952-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14143-1">
    <skos:related rdf:resource="descriptor/19021-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20306-6">
    <skos:broader rdf:resource="thsys/70550"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70161">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15983-1">
    <skos:prefLabel xml:lang="en">Electoral law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15983-1">
    <skos:prefLabel xml:lang="de">Wahlrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20409-3">
    <skos:scopeNote xml:lang="de">Ökonomische Analyse von Ursachen und Folgen des Drogenanbaus und -handels in den Erzeugerländern sowie von Handel, Konsum und Drogenpolitik in den Konsumentenländern</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19230-2">
    <skos:prefLabel xml:lang="en">Orani</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19230-2">
    <skos:prefLabel xml:lang="de">Orani</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70140">
    <skos:narrower rdf:resource="descriptor/10032-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70140">
    <skos:narrower rdf:resource="descriptor/19563-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70140">
    <skos:narrower rdf:resource="descriptor/19606-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70140">
    <skos:narrower rdf:resource="descriptor/16440-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70140">
    <skos:narrower rdf:resource="descriptor/13992-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70140">
    <skos:narrower rdf:resource="descriptor/12036-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70140">
    <skos:narrower rdf:resource="descriptor/12072-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70140">
    <skos:narrower rdf:resource="descriptor/18117-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70140">
    <skos:narrower rdf:resource="descriptor/19164-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70140">
    <skos:narrower rdf:resource="descriptor/19163-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70140">
    <skos:narrower rdf:resource="descriptor/10013-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70140">
    <skos:narrower rdf:resource="descriptor/12062-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70140">
    <skos:narrower rdf:resource="descriptor/11164-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70140">
    <skos:narrower rdf:resource="descriptor/20972-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70140">
    <skos:narrower rdf:resource="descriptor/10923-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70140">
    <skos:narrower rdf:resource="descriptor/10572-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70140">
    <skos:narrower rdf:resource="descriptor/18121-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70140">
    <skos:narrower rdf:resource="descriptor/19071-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70140">
    <skos:narrower rdf:resource="descriptor/19161-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70140">
    <skos:narrower rdf:resource="descriptor/19544-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70140">
    <skos:narrower rdf:resource="descriptor/18872-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70140">
    <skos:narrower rdf:resource="descriptor/12041-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70140">
    <skos:narrower rdf:resource="descriptor/19247-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70140">
    <skos:narrower rdf:resource="descriptor/19068-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70140">
    <skos:narrower rdf:resource="descriptor/10000-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70140">
    <skos:narrower rdf:resource="descriptor/19536-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19815-2">
    <skos:prefLabel xml:lang="en">Money laundering</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19815-2">
    <skos:prefLabel xml:lang="de">Geldwäsche</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19013-3">
    <skos:related rdf:resource="descriptor/16676-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19013-3">
    <skos:related rdf:resource="descriptor/14729-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19013-3">
    <skos:related rdf:resource="descriptor/15790-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19013-3">
    <skos:related rdf:resource="descriptor/13348-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19013-3">
    <skos:related rdf:resource="descriptor/16621-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11246-6">
    <skos:altLabel xml:lang="de">Sicherung von Arbeitsplätzen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11246-6">
    <skos:altLabel xml:lang="de">Erhaltung von Arbeitsplätzen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11246-6">
    <skos:altLabel xml:lang="de">Arbeitsplatzsicherheit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18009-6">
    <skos:broader rdf:resource="thsys/70649"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18009-6">
    <skos:broader rdf:resource="descriptor/14216-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18009-6">
    <skos:broader rdf:resource="descriptor/14171-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19570-6">
    <skos:broader rdf:resource="thsys/72183"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19570-6">
    <skos:broader rdf:resource="descriptor/15396-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70372">
    <skos:prefLabel xml:lang="de">P.16  Kunststoff- und Gummiwaren</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11901-0">
    <skos:broader rdf:resource="thsys/70969"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11901-0">
    <skos:broader rdf:resource="descriptor/12905-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11478-6">
    <skos:broader rdf:resource="thsys/70305"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11478-6">
    <skos:broader rdf:resource="descriptor/11490-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10786-0">
    <skos:related rdf:resource="descriptor/19360-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10786-0">
    <skos:related rdf:resource="descriptor/10811-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10786-0">
    <skos:related rdf:resource="descriptor/10797-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10786-0">
    <skos:related rdf:resource="descriptor/10798-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19313-5">
    <skos:related rdf:resource="descriptor/10677-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19313-5">
    <skos:related rdf:resource="descriptor/10787-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19313-5">
    <skos:related rdf:resource="descriptor/10642-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19313-5">
    <skos:related rdf:resource="descriptor/10717-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15773-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19184-4">
    <skos:related rdf:resource="descriptor/13034-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19184-4">
    <skos:related rdf:resource="descriptor/13224-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19184-4">
    <skos:related rdf:resource="descriptor/14266-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19184-4">
    <skos:related rdf:resource="descriptor/12985-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14752-4">
    <skos:narrower rdf:resource="descriptor/14514-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14752-4">
    <skos:narrower rdf:resource="descriptor/14759-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14752-4">
    <skos:narrower rdf:resource="descriptor/18271-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15706-6">
    <skos:related rdf:resource="descriptor/19636-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15706-6">
    <skos:related rdf:resource="descriptor/16566-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15706-6">
    <skos:related rdf:resource="descriptor/13511-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15706-6">
    <skos:related rdf:resource="descriptor/15723-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15706-6">
    <skos:related rdf:resource="descriptor/20289-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15706-6">
    <skos:related rdf:resource="descriptor/16745-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15706-6">
    <skos:related rdf:resource="descriptor/15705-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15072-0">
    <skos:altLabel xml:lang="en">Prediction model</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15072-0">
    <skos:altLabel xml:lang="en">Forecasting method</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15072-0">
    <skos:altLabel xml:lang="de">Prognosetechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15072-0">
    <skos:altLabel xml:lang="de">Prognosemethode</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19574-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14854-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18789-3">
    <skos:related rdf:resource="descriptor/19055-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18789-3">
    <skos:related rdf:resource="descriptor/10598-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18789-3">
    <skos:related rdf:resource="descriptor/13962-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18789-3">
    <skos:related rdf:resource="descriptor/18793-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18789-3">
    <skos:related rdf:resource="descriptor/13381-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12276-6">
    <skos:narrower rdf:resource="descriptor/12855-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12276-6">
    <skos:narrower rdf:resource="descriptor/12259-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12276-6">
    <skos:narrower rdf:resource="descriptor/12296-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12276-6">
    <skos:narrower rdf:resource="descriptor/19082-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12276-6">
    <skos:narrower rdf:resource="descriptor/11772-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12276-6">
    <skos:narrower rdf:resource="descriptor/12271-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13840-5">
    <skos:altLabel xml:lang="en">Owner-occupied house</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13840-5">
    <skos:altLabel xml:lang="de">Wohnungseigentum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13840-5">
    <skos:altLabel xml:lang="de">Selbstgenutztes Wohneigentum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13840-5">
    <skos:altLabel xml:lang="de">Einliegerwohnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13840-5">
    <skos:altLabel xml:lang="de">Eigentumswohnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13840-5">
    <skos:altLabel xml:lang="de">Eigenheim</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13840-5">
    <skos:altLabel xml:lang="de">Eigengenutzter Wohnraum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12340-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14614-2">
    <skos:related rdf:resource="descriptor/14619-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70233">
    <rdfs:label xml:lang="en">Aggregate production</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70233">
    <rdfs:label xml:lang="de">Gesamtwirtschaftliche Produktion</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11305-2">
    <skos:altLabel xml:lang="en">Quality of working life</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11305-2">
    <skos:altLabel xml:lang="en">Labour standards</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11305-2">
    <skos:altLabel xml:lang="en">Labor standards</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11305-2">
    <skos:altLabel xml:lang="de">Qualität des Arbeitslebens</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11305-2">
    <skos:altLabel xml:lang="de">Arbeitssituation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70889">
    <skos:prefLabel xml:lang="en">W.17  Real estate services</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70889">
    <skos:prefLabel xml:lang="de">W.17  Immobilienwirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18313-3">
    <skos:prefLabel xml:lang="en">Farming system</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18313-3">
    <skos:prefLabel xml:lang="de">Landwirtschaftliches Betriebssystem</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11034-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19486-2">
    <skos:narrower rdf:resource="descriptor/19482-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10514-5">
    <skos:prefLabel xml:lang="en">Underdevelopment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10514-5">
    <skos:prefLabel xml:lang="de">Unterentwicklung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20368-5">
    <skos:broader rdf:resource="thsys/70278"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20368-5">
    <skos:broader rdf:resource="thsys/70245"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20368-5">
    <skos:broader rdf:resource="descriptor/12594-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20368-5">
    <skos:broader rdf:resource="descriptor/11769-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20368-5">
    <skos:broader rdf:resource="descriptor/18687-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14652-1">
    <skos:prefLabel xml:lang="en">Sulphuric acid</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14652-1">
    <skos:prefLabel xml:lang="de">Schwefelsäure</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11962-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17175-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18382-5">
    <skos:related rdf:resource="descriptor/10884-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18382-5">
    <skos:related rdf:resource="descriptor/12922-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18382-5">
    <skos:related rdf:resource="descriptor/18995-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18382-5">
    <skos:related rdf:resource="descriptor/18438-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18382-5">
    <skos:related rdf:resource="descriptor/14047-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18382-5">
    <skos:related rdf:resource="descriptor/10545-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18382-5">
    <skos:related rdf:resource="descriptor/19674-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18416-0">
    <skos:prefLabel xml:lang="en">Shiprepair yard</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18416-0">
    <skos:prefLabel xml:lang="de">Schiffsreparaturindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19566-4">
    <skos:related rdf:resource="descriptor/11914-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19566-4">
    <skos:related rdf:resource="descriptor/12501-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19566-4">
    <skos:related rdf:resource="descriptor/11241-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12705-2">
    <skos:narrower rdf:resource="descriptor/14039-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12705-2">
    <skos:narrower rdf:resource="descriptor/14331-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13428-4">
    <skos:altLabel xml:lang="de">Nahrungsversorgung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13428-4">
    <skos:altLabel xml:lang="de">Nahrungsmittelversorgungssicherheit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13428-4">
    <skos:altLabel xml:lang="de">Nahrungsmittelsicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13428-4">
    <skos:altLabel xml:lang="de">Lebensmittelversorgung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13179-6">
    <skos:altLabel xml:lang="de">Wasserstraßenbau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13179-6">
    <skos:altLabel xml:lang="de">Wasserbautechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13179-6">
    <skos:altLabel xml:lang="de">Strombau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13179-6">
    <skos:altLabel xml:lang="de">Flussbau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13179-6">
    <skos:altLabel xml:lang="de">Flußbau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13179-6">
    <skos:altLabel xml:lang="de">Deichbau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12022-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18559-6">
    <skos:prefLabel xml:lang="en">Environmentally friendly product</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18559-6">
    <skos:prefLabel xml:lang="de">Umweltverträgliches Produkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12617-6">
    <skos:related rdf:resource="descriptor/19562-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12617-6">
    <skos:related rdf:resource="descriptor/18117-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12617-6">
    <skos:related rdf:resource="descriptor/12620-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12617-6">
    <skos:related rdf:resource="descriptor/12585-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12617-6">
    <skos:related rdf:resource="descriptor/12574-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12617-6">
    <skos:related rdf:resource="descriptor/24847-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12617-6">
    <skos:related rdf:resource="descriptor/12110-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12617-6">
    <skos:related rdf:resource="descriptor/12106-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12617-6">
    <skos:related rdf:resource="descriptor/15786-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12617-6">
    <skos:related rdf:resource="descriptor/16692-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12617-6">
    <skos:related rdf:resource="descriptor/15825-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12617-6">
    <skos:related rdf:resource="descriptor/12612-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12617-6">
    <skos:related rdf:resource="descriptor/12577-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12617-6">
    <skos:related rdf:resource="descriptor/12615-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12617-6">
    <skos:related rdf:resource="descriptor/11295-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12617-6">
    <skos:related rdf:resource="descriptor/15837-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12617-6">
    <skos:related rdf:resource="descriptor/12607-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10113-2">
    <skos:broader rdf:resource="thsys/70057"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10113-2">
    <skos:broader rdf:resource="descriptor/10062-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12986-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18342-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12947-6">
    <skos:narrower rdf:resource="descriptor/12958-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11350-4">
    <skos:narrower rdf:resource="descriptor/11313-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11350-4">
    <skos:narrower rdf:resource="descriptor/11304-4"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id269">
    <rdf:value xml:lang="de">Für Kosten von Gerichtsverfahren</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18666-5">
    <skos:broader rdf:resource="thsys/70922"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18666-5">
    <skos:broader rdf:resource="descriptor/13268-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10503-3">
    <skos:related rdf:resource="descriptor/11917-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10503-3">
    <skos:related rdf:resource="descriptor/11993-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10503-3">
    <skos:related rdf:resource="descriptor/12945-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10503-3">
    <skos:related rdf:resource="descriptor/19462-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10503-3">
    <skos:related rdf:resource="descriptor/19460-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10503-3">
    <skos:related rdf:resource="descriptor/19463-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10503-3">
    <skos:related rdf:resource="descriptor/10575-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10503-3">
    <skos:related rdf:resource="descriptor/10504-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10503-3">
    <skos:related rdf:resource="descriptor/13454-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10503-3">
    <skos:related rdf:resource="descriptor/11862-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10503-3">
    <skos:related rdf:resource="descriptor/13445-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10503-3">
    <skos:related rdf:resource="descriptor/13443-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14840-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19745-4">
    <skos:broader rdf:resource="thsys/73393"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19745-4">
    <skos:broader rdf:resource="thsys/70011"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19745-4">
    <skos:broader rdf:resource="descriptor/19070-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18513-2">
    <skos:altLabel xml:lang="de">Handelsvertretervertrag</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13194-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10249-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73337">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.05.05</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18890-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20972-4">
    <skos:scopeNote xml:lang="de">In Japan Bezeichnung für eine konzernähnlich verflochtene Gruppe von selbständigen Unternehmen mit koordinierten Unternehmenszielen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18793-5">
    <skos:altLabel xml:lang="de">Gutachterinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18793-5">
    <skos:altLabel xml:lang="de">Gutachter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18793-5">
    <skos:altLabel xml:lang="de">Expertinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18793-5">
    <skos:altLabel xml:lang="de">Experten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11682-0">
    <skos:related rdf:resource="descriptor/11683-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11682-0">
    <skos:related rdf:resource="descriptor/12849-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11682-0">
    <skos:related rdf:resource="descriptor/11566-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11682-0">
    <skos:related rdf:resource="descriptor/11659-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11682-0">
    <skos:related rdf:resource="descriptor/11651-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18822-3">
    <skos:narrower rdf:resource="descriptor/18813-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19171-6">
    <skos:broader rdf:resource="thsys/70307"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70866">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19183-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14157-4">
    <skos:related rdf:resource="descriptor/18340-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10122-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18990-3">
    <skos:broader rdf:resource="thsys/71011"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18990-3">
    <skos:broader rdf:resource="descriptor/10889-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11168-0">
    <skos:prefLabel xml:lang="en">Interest-group theory</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11168-0">
    <skos:prefLabel xml:lang="de">Gruppentheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16824-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70234">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.03</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11888-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19015-6">
    <skos:related rdf:resource="descriptor/19630-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70493">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.02.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15725-2">
    <skos:altLabel xml:lang="de">Gesellschaftsreform</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10553-2">
    <skos:prefLabel xml:lang="en">Development fund</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10553-2">
    <skos:prefLabel xml:lang="de">Entwicklungsfonds</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10261-1">
    <skos:altLabel xml:lang="en">Underemployment</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10261-1">
    <skos:altLabel xml:lang="en">Labour surplus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10261-1">
    <skos:altLabel xml:lang="en">Labor surplus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10261-1">
    <skos:altLabel xml:lang="de">Unterbeschäftigung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10261-1">
    <skos:altLabel xml:lang="de">Arbeitskräfteüberschuss</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10261-1">
    <skos:altLabel xml:lang="de">Arbeitskräfteüberschuß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15938-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12842-6">
    <skos:prefLabel xml:lang="en">Advertising music</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12842-6">
    <skos:prefLabel xml:lang="de">Werbemusik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18842-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12458-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13220-0">
    <skos:altLabel xml:lang="de">Lederwarenfachhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13220-0">
    <skos:altLabel xml:lang="de">Lederbekleidungshandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10834-1">
    <skos:altLabel xml:lang="de">Importpreis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10834-1">
    <skos:altLabel xml:lang="de">Exportpreis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10834-1">
    <skos:altLabel xml:lang="de">Einfuhrpreis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10834-1">
    <skos:altLabel xml:lang="de">Auslandspreis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10834-1">
    <skos:altLabel xml:lang="de">Ausfuhrpreis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19281-6">
    <skos:prefLabel xml:lang="en">Allocation for tax purposes</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19281-6">
    <skos:prefLabel xml:lang="de">Steuerliche Zurechnung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19003-6">
    <skos:scopeNote xml:lang="de">Abfindungen beim Ausscheiden von Gesellschaftern und bei Entlassung von Arbeitskräften.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71089">
    <skos:prefLabel xml:lang="de">V.07.07  Internationale Finanz- und Kapitalmärkte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id102">
    <rdf:value xml:lang="de">Für betriebliche Vereinbarungen mit dem Betriebsrat</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11233-1">
    <skos:prefLabel xml:lang="en">Secondary labour market</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11233-1">
    <skos:prefLabel xml:lang="de">Sekundärer Arbeitsmarkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24733-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14755-5">
    <skos:altLabel xml:lang="en">Supercomputer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14755-5">
    <skos:altLabel xml:lang="en">Mainframe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14755-5">
    <skos:altLabel xml:lang="de">Vektorrechner</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14755-5">
    <skos:altLabel xml:lang="de">Universalrechner</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14755-5">
    <skos:altLabel xml:lang="de">Großcomputer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18599-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11784-6">
    <skos:broader rdf:resource="thsys/71110"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11784-6">
    <skos:broader rdf:resource="thsys/70575"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18517-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17652-0">
    <skos:altLabel xml:lang="x-other">Afrique Sub-Saharienne</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17652-0">
    <skos:altLabel xml:lang="x-other">Afrique au Sud du Sahara</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17652-0">
    <skos:altLabel xml:lang="en">Sub-Saharan Africa</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17652-0">
    <skos:altLabel xml:lang="de">Schwarzafrika</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17984-3">
    <skos:altLabel xml:lang="de">Englischsprachiges Afrika</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17984-3">
    <skos:altLabel xml:lang="de">Afrika (Anglophon)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70925">
    <skos:narrower rdf:resource="descriptor/13514-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70925">
    <skos:narrower rdf:resource="descriptor/13595-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70925">
    <skos:narrower rdf:resource="descriptor/13594-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70925">
    <skos:narrower rdf:resource="descriptor/13593-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70925">
    <skos:narrower rdf:resource="descriptor/11827-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70925">
    <skos:narrower rdf:resource="descriptor/11881-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70925">
    <skos:narrower rdf:resource="descriptor/18633-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70925">
    <skos:narrower rdf:resource="descriptor/11974-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70925">
    <skos:narrower rdf:resource="descriptor/18576-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70925">
    <skos:narrower rdf:resource="descriptor/11876-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70925">
    <skos:narrower rdf:resource="descriptor/18632-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70925">
    <skos:narrower rdf:resource="descriptor/11893-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70925">
    <skos:narrower rdf:resource="descriptor/13249-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70925">
    <skos:narrower rdf:resource="descriptor/18610-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70925">
    <skos:narrower rdf:resource="descriptor/15709-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70925">
    <skos:narrower rdf:resource="descriptor/18631-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70925">
    <skos:narrower rdf:resource="descriptor/11863-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70925">
    <skos:narrower rdf:resource="descriptor/13618-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70925">
    <skos:narrower rdf:resource="descriptor/11864-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70925">
    <skos:narrower rdf:resource="descriptor/13499-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12474-2">
    <skos:related rdf:resource="descriptor/12489-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12474-2">
    <skos:related rdf:resource="descriptor/12796-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12474-2">
    <skos:related rdf:resource="descriptor/19153-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12474-2">
    <skos:related rdf:resource="descriptor/11529-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12474-2">
    <skos:related rdf:resource="descriptor/12520-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12474-2">
    <skos:related rdf:resource="descriptor/12408-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19845-0">
    <skos:narrower rdf:resource="descriptor/16067-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18980-6">
    <skos:altLabel xml:lang="de">Landwirtschaftsproduktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18980-6">
    <skos:altLabel xml:lang="de">Landwirtschaftliche Produktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18980-6">
    <skos:altLabel xml:lang="de">Landwirtschaftliche Erzeugung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18980-6">
    <skos:altLabel xml:lang="de">Agrarerzeugung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71097">
    <skos:prefLabel xml:lang="de">V.07.02.01  Internationale Wirtschaftspolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16978-3">
    <skos:altLabel xml:lang="de">Valais</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17991-6">
    <skos:related rdf:resource="descriptor/19000-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11278-0">
    <skos:prefLabel xml:lang="en">Job vacancies</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11278-0">
    <skos:prefLabel xml:lang="de">Offene Stellen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15689-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20110-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15714-0">
    <skos:narrower rdf:resource="descriptor/15859-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15714-0">
    <skos:narrower rdf:resource="descriptor/15711-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15714-0">
    <skos:narrower rdf:resource="descriptor/16663-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15714-0">
    <skos:narrower rdf:resource="descriptor/16661-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15714-0">
    <skos:narrower rdf:resource="descriptor/15835-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15714-0">
    <skos:narrower rdf:resource="descriptor/16587-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15714-0">
    <skos:narrower rdf:resource="descriptor/15918-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10082-1">
    <skos:broader rdf:resource="thsys/70188"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10082-1">
    <skos:broader rdf:resource="descriptor/11034-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13423-0">
    <skos:narrower rdf:resource="descriptor/13417-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16264-0">
    <skos:scopeNote xml:lang="de">Recht der nationalen Selbstbestimmung von Völkern bzw. Volksgruppen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14024-2">
    <skos:altLabel xml:lang="de">Schulbedarfsartikel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14024-2">
    <skos:altLabel xml:lang="de">Schreibunterlage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14024-2">
    <skos:altLabel xml:lang="de">Radiergummi</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14024-2">
    <skos:altLabel xml:lang="de">Federschale</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14024-2">
    <skos:altLabel xml:lang="de">Bürobedarf</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14024-2">
    <skos:altLabel xml:lang="de">Büroartikel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18102-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10048-1">
    <skos:altLabel xml:lang="en">Multi-regional model</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14336-4">
    <skos:related rdf:resource="descriptor/12705-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14336-4">
    <skos:related rdf:resource="descriptor/18412-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14336-4">
    <skos:related rdf:resource="descriptor/13058-4"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id118">
    <rdf:value xml:lang="de">für Entwicklungsländer</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19295-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19816-0">
    <skos:related rdf:resource="descriptor/16504-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19816-0">
    <skos:related rdf:resource="descriptor/19813-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19816-0">
    <skos:related rdf:resource="descriptor/19728-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19294-4">
    <skos:prefLabel xml:lang="en">Coffee market</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19294-4">
    <skos:prefLabel xml:lang="de">Kaffeemarkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12218-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10608-3">
    <skos:related rdf:resource="descriptor/10771-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10608-3">
    <skos:related rdf:resource="descriptor/19336-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10608-3">
    <skos:related rdf:resource="descriptor/18536-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10697-6">
    <skos:altLabel xml:lang="de">Abschöpfung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18728-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10550-1">
    <skos:related rdf:resource="descriptor/18705-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10550-1">
    <skos:related rdf:resource="descriptor/10558-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10550-1">
    <skos:related rdf:resource="descriptor/10552-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18970-2">
    <skos:broader rdf:resource="thsys/73357"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18970-2">
    <skos:broader rdf:resource="thsys/70814"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18970-2">
    <skos:broader rdf:resource="descriptor/15920-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18970-2">
    <skos:broader rdf:resource="descriptor/19759-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12246-1">
    <skos:broader rdf:resource="thsys/73392"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12246-1">
    <skos:broader rdf:resource="thsys/71045"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12246-1">
    <skos:broader rdf:resource="descriptor/16036-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16760-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11054-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71052">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12930-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12387-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18379-1">
    <skos:narrower rdf:resource="descriptor/15734-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18379-1">
    <skos:narrower rdf:resource="descriptor/15730-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13309-5">
    <skos:prefLabel xml:lang="en">Canteen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13309-5">
    <skos:prefLabel xml:lang="de">Gemeinschaftsgastronomie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15878-5">
    <skos:prefLabel xml:lang="en">National border</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15878-5">
    <skos:prefLabel xml:lang="de">Grenze</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10091-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19068-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18599-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17621-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70354">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13297-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70005">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">B.03</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14617-3">
    <skos:altLabel xml:lang="de">Papierchemikalie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12605-6">
    <skos:prefLabel xml:lang="en">Radiation protection</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12605-6">
    <skos:prefLabel xml:lang="de">Strahlenschutz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18608-5">
    <skos:prefLabel xml:lang="en">Commuter air transport</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18608-5">
    <skos:prefLabel xml:lang="de">Regionalluftverkehr</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12801-6">
    <skos:prefLabel xml:lang="en">Generic brand</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12801-6">
    <skos:prefLabel xml:lang="de">No-Name-Produkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17641-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10753-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18857-5">
    <skos:related rdf:resource="descriptor/11382-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18857-5">
    <skos:related rdf:resource="descriptor/11401-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18857-5">
    <skos:related rdf:resource="descriptor/11374-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18857-5">
    <skos:related rdf:resource="descriptor/18864-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16305-5">
    <skos:related rdf:resource="descriptor/16017-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16305-5">
    <skos:related rdf:resource="descriptor/19689-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16305-5">
    <skos:related rdf:resource="descriptor/13420-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16305-5">
    <skos:related rdf:resource="descriptor/16329-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16305-5">
    <skos:related rdf:resource="descriptor/16366-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16486-3">
    <skos:scopeNote xml:lang="de">Staatliche Maßnahmen zur Beeinflussung, Förderung oder Beschränkung der Aus- und Einwanderung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15055-0">
    <skos:broader rdf:resource="thsys/71035"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15055-0">
    <skos:broader rdf:resource="thsys/70269"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15055-0">
    <skos:broader rdf:resource="descriptor/15483-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15055-0">
    <skos:broader rdf:resource="descriptor/15045-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12980-1">
    <skos:prefLabel xml:lang="en">Mining</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12980-1">
    <skos:prefLabel xml:lang="de">Bergbau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16196-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19508-4">
    <skos:altLabel xml:lang="en">Work for benefit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19508-4">
    <skos:altLabel xml:lang="en">Welfare-to-work</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19508-4">
    <skos:altLabel xml:lang="de">Arbeit für Sozialhilfe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10500-2">
    <skos:altLabel xml:lang="en">Industrial development</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10500-2">
    <skos:altLabel xml:lang="de">Industrielle Revolution</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10500-2">
    <skos:altLabel xml:lang="de">Industrielle Entwicklung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12282-4">
    <skos:related rdf:resource="descriptor/10913-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12282-4">
    <skos:related rdf:resource="descriptor/20018-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12282-4">
    <skos:related rdf:resource="descriptor/12283-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12282-4">
    <skos:related rdf:resource="descriptor/19043-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12282-4">
    <skos:related rdf:resource="descriptor/13727-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12282-4">
    <skos:related rdf:resource="descriptor/12418-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12282-4">
    <skos:related rdf:resource="descriptor/19522-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20468-1">
    <skos:altLabel xml:lang="de">Geschäftskonzept</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20468-1">
    <skos:altLabel xml:lang="de">Geschäftsidee</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19681-4">
    <skos:prefLabel xml:lang="en">Federation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19681-4">
    <skos:prefLabel xml:lang="de">Föderation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14044-3">
    <skos:editorialNote xml:lang="de">Möglichst spezifischer indexieren.</skos:editorialNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10024-1">
    <skos:narrower rdf:resource="descriptor/10274-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10024-1">
    <skos:narrower rdf:resource="descriptor/11616-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10024-1">
    <skos:narrower rdf:resource="descriptor/12026-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10024-1">
    <skos:narrower rdf:resource="descriptor/10115-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10024-1">
    <skos:narrower rdf:resource="descriptor/18680-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10024-1">
    <skos:narrower rdf:resource="descriptor/12356-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10024-1">
    <skos:narrower rdf:resource="descriptor/10798-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70960">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11571-2">
    <skos:prefLabel xml:lang="en">Saving incentives</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11571-2">
    <skos:prefLabel xml:lang="de">Sparförderung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19777-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10001-6">
    <skos:prefLabel xml:lang="en">Arbitrage</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10001-6">
    <skos:prefLabel xml:lang="de">Arbitragegeschäft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18490-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16396-4">
    <skos:altLabel xml:lang="en">Resistance movement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16396-4">
    <skos:altLabel xml:lang="de">Widerstandsbewegung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16396-4">
    <skos:altLabel xml:lang="de">Untergrundbewegung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16396-4">
    <skos:altLabel xml:lang="de">Dissidenten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11529-1">
    <skos:broader rdf:resource="thsys/70578"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11529-1">
    <skos:broader rdf:resource="thsys/70144"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17595-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21283-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16774-5">
    <skos:related rdf:resource="descriptor/18941-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16774-5">
    <skos:related rdf:resource="descriptor/18172-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16774-5">
    <skos:related rdf:resource="descriptor/19654-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18140-1">
    <skos:altLabel xml:lang="de">Stellensuche</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18140-1">
    <skos:altLabel xml:lang="de">Bewerbung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18140-1">
    <skos:altLabel xml:lang="de">Arbeitssuche</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19186-0">
    <skos:broader rdf:resource="thsys/71002"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19186-0">
    <skos:broader rdf:resource="thsys/70962"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19186-0">
    <skos:broader rdf:resource="descriptor/10012-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18326-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18957-1">
    <skos:narrower rdf:resource="descriptor/15730-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15711-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13244-0">
    <skos:prefLabel xml:lang="en">Road freight transport</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13244-0">
    <skos:prefLabel xml:lang="de">Straßengüterverkehr</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18758-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15728-3">
    <skos:related rdf:resource="descriptor/15743-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15728-3">
    <skos:related rdf:resource="descriptor/15742-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15728-3">
    <skos:related rdf:resource="descriptor/14337-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15728-3">
    <skos:related rdf:resource="descriptor/14327-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15728-3">
    <skos:related rdf:resource="descriptor/15010-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18862-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16978-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73359">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.06.03.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11046-0">
    <skos:narrower rdf:resource="descriptor/11101-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16416-3">
    <skos:prefLabel xml:lang="en">Nationalism</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16416-3">
    <skos:prefLabel xml:lang="de">Nationalismus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18787-0">
    <skos:prefLabel xml:lang="en">Civil engineering firm</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18787-0">
    <skos:prefLabel xml:lang="de">Bauingenieurbüro</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13259-1">
    <skos:prefLabel xml:lang="en">Cable railway industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13259-1">
    <skos:prefLabel xml:lang="de">Seilbahnverkehr</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14274-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70148">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">G.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18295-0">
    <skos:related rdf:resource="descriptor/18436-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18295-0">
    <skos:related rdf:resource="descriptor/12805-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18295-0">
    <skos:related rdf:resource="descriptor/18070-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18295-0">
    <skos:related rdf:resource="descriptor/18788-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18295-0">
    <skos:related rdf:resource="descriptor/13128-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18295-0">
    <skos:related rdf:resource="descriptor/14899-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13316-1">
    <skos:altLabel xml:lang="de">Friseurinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13316-1">
    <skos:altLabel xml:lang="de">Friseur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13316-1">
    <skos:altLabel xml:lang="de">Coiffeurinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13316-1">
    <skos:altLabel xml:lang="de">Coiffeur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13316-1">
    <skos:altLabel xml:lang="de">Barbiere</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12285-5">
    <skos:broader rdf:resource="thsys/70187"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12285-5">
    <skos:broader rdf:resource="thsys/70036"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12285-5">
    <skos:broader rdf:resource="descriptor/10006-3"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id94">
    <rdfs:seeAlso rdf:resource="descriptor/18924-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70560">
    <skos:narrower rdf:resource="descriptor/17724-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70560">
    <skos:narrower rdf:resource="descriptor/17723-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70560">
    <skos:narrower rdf:resource="descriptor/17722-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70560">
    <skos:narrower rdf:resource="descriptor/17721-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70560">
    <skos:narrower rdf:resource="descriptor/17717-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70560">
    <skos:narrower rdf:resource="descriptor/17716-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70560">
    <skos:narrower rdf:resource="descriptor/17713-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70560">
    <skos:narrower rdf:resource="descriptor/17712-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70560">
    <skos:narrower rdf:resource="descriptor/17709-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70560">
    <skos:narrower rdf:resource="descriptor/17707-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70560">
    <skos:narrower rdf:resource="descriptor/17706-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70560">
    <skos:narrower rdf:resource="descriptor/17705-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70560">
    <skos:narrower rdf:resource="descriptor/17704-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70560">
    <skos:narrower rdf:resource="descriptor/17703-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70560">
    <skos:narrower rdf:resource="descriptor/17702-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70560">
    <skos:narrower rdf:resource="descriptor/17700-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70560">
    <skos:narrower rdf:resource="descriptor/17696-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70560">
    <skos:narrower rdf:resource="descriptor/17695-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11718-5">
    <skos:broader rdf:resource="thsys/71083"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11718-5">
    <skos:broader rdf:resource="thsys/70858"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11718-5">
    <skos:broader rdf:resource="descriptor/10030-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11718-5">
    <skos:broader rdf:resource="descriptor/11715-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19769-4">
    <skos:related rdf:resource="descriptor/19285-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19769-4">
    <skos:related rdf:resource="descriptor/15720-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19769-4">
    <skos:related rdf:resource="descriptor/16542-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19769-4">
    <skos:related rdf:resource="descriptor/19683-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12186-0">
    <skos:related rdf:resource="descriptor/19522-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14366-2">
    <skos:related rdf:resource="descriptor/14438-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14366-2">
    <skos:related rdf:resource="descriptor/14397-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14366-2">
    <skos:related rdf:resource="descriptor/12905-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14366-2">
    <skos:related rdf:resource="descriptor/18495-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14366-2">
    <skos:related rdf:resource="descriptor/12904-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14366-2">
    <skos:related rdf:resource="descriptor/14021-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14366-2">
    <skos:related rdf:resource="descriptor/19058-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14366-2">
    <skos:related rdf:resource="descriptor/13045-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19312-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70354">
    <skos:broader rdf:resource="thsys/70076"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14653-6">
    <skos:prefLabel xml:lang="en">Soda ash</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14653-6">
    <skos:prefLabel xml:lang="de">Soda</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19409-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19958-1">
    <skos:broader rdf:resource="thsys/71097"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19958-1">
    <skos:broader rdf:resource="descriptor/19326-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18693-2">
    <skos:broader rdf:resource="thsys/71104"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18693-2">
    <skos:broader rdf:resource="thsys/70455"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18693-2">
    <skos:broader rdf:resource="descriptor/13744-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19524-6">
    <skos:narrower rdf:resource="descriptor/19489-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16802-5">
    <skos:related rdf:resource="descriptor/18660-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16802-5">
    <skos:related rdf:resource="descriptor/18952-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16802-5">
    <skos:related rdf:resource="descriptor/13351-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11390-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13563-5">
    <skos:related rdf:resource="descriptor/16171-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13563-5">
    <skos:related rdf:resource="descriptor/18579-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13563-5">
    <skos:related rdf:resource="descriptor/13065-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13563-5">
    <skos:related rdf:resource="descriptor/14441-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13563-5">
    <skos:related rdf:resource="descriptor/13560-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13563-5">
    <skos:related rdf:resource="descriptor/11974-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18085-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13032-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71002">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.06.01.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15912-3">
    <skos:broader rdf:resource="thsys/70142"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15912-3">
    <skos:broader rdf:resource="descriptor/15916-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19116-0">
    <skos:scopeNote xml:lang="de">Realisierung von Kostensenkung und Kundenorientierung durch dezentrale und simultane Abläufe; Nachempfinden des japanischen Produktionsansatzes.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11373-6">
    <skos:altLabel xml:lang="en">College</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11373-6">
    <skos:altLabel xml:lang="de">Universität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11373-6">
    <skos:altLabel xml:lang="de">Hochschulwesen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10545-1">
    <skos:broader rdf:resource="thsys/71010"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10545-1">
    <skos:broader rdf:resource="thsys/70811"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10545-1">
    <skos:broader rdf:resource="descriptor/15752-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10545-1">
    <skos:broader rdf:resource="descriptor/12675-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14076-4">
    <skos:related rdf:resource="descriptor/14104-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14076-4">
    <skos:related rdf:resource="descriptor/14149-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15372-2">
    <skos:prefLabel xml:lang="en">Nonlinear approach</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15372-2">
    <skos:prefLabel xml:lang="de">Nichtlineares Verfahren</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15646-5">
    <skos:altLabel xml:lang="de">Brahmanismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15651-5">
    <skos:prefLabel xml:lang="en">Coal upgrading</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15651-5">
    <skos:prefLabel xml:lang="de">Kohleveredlung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18664-2">
    <skos:scopeNote xml:lang="de">Nur für unternehmensinterne Computernetze.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10199-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14703-3">
    <skos:broader rdf:resource="thsys/70476"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14703-3">
    <skos:broader rdf:resource="descriptor/14702-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14921-0">
    <skos:altLabel xml:lang="de">Strumpfhose</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14921-0">
    <skos:altLabel xml:lang="de">Strümpfe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16588-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12588-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13365-2">
    <skos:broader rdf:resource="thsys/73296"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13365-2">
    <skos:broader rdf:resource="descriptor/13367-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13082-0">
    <skos:prefLabel xml:lang="en">Photographic industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13082-0">
    <skos:prefLabel xml:lang="de">Fotoindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18689-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16404-3">
    <skos:altLabel xml:lang="de">Rechtsradikalismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16404-3">
    <skos:altLabel xml:lang="de">Rechtsextremismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16404-3">
    <skos:altLabel xml:lang="de">Politischer Radikalismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16404-3">
    <skos:altLabel xml:lang="de">Neue Rechte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16404-3">
    <skos:altLabel xml:lang="de">Neonazismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16404-3">
    <skos:altLabel xml:lang="de">Neofaschismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16404-3">
    <skos:altLabel xml:lang="de">Linksradikalismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16404-3">
    <skos:altLabel xml:lang="de">Linksextremismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16404-3">
    <skos:altLabel xml:lang="de">Fundamentalismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16404-3">
    <skos:altLabel xml:lang="de">Extremismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14656-0">
    <skos:altLabel xml:lang="de">Keton</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14656-0">
    <skos:altLabel xml:lang="de">Caprolactam</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14656-0">
    <skos:altLabel xml:lang="de">Aceton</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12730-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11034-0">
    <skos:narrower rdf:resource="descriptor/10082-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15731-0">
    <skos:prefLabel xml:lang="en">Sewerage system</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15731-0">
    <skos:prefLabel xml:lang="de">Kanalisation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13073-1">
    <skos:prefLabel xml:lang="en">Personal computer industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13073-1">
    <skos:prefLabel xml:lang="de">PC-Industrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10515-3">
    <skos:related rdf:resource="descriptor/10595-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10515-3">
    <skos:related rdf:resource="descriptor/19235-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10515-3">
    <skos:related rdf:resource="descriptor/10574-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10515-3">
    <skos:related rdf:resource="descriptor/19364-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10515-3">
    <skos:related rdf:resource="descriptor/11083-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10515-3">
    <skos:related rdf:resource="descriptor/10534-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10515-3">
    <skos:related rdf:resource="descriptor/19286-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10659-0">
    <skos:prefLabel xml:lang="en">Purchasing power parity</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10659-0">
    <skos:prefLabel xml:lang="de">Kaufkraftparität</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70039">
    <rdfs:label xml:lang="de">Wirtschaftsordnung und Wirtschaftspolitik</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15272-6">
    <skos:related rdf:resource="descriptor/15243-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15272-6">
    <skos:related rdf:resource="descriptor/15298-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15272-6">
    <skos:related rdf:resource="descriptor/15063-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15272-6">
    <skos:related rdf:resource="descriptor/19037-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15272-6">
    <skos:related rdf:resource="descriptor/15062-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15272-6">
    <skos:related rdf:resource="descriptor/15372-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15073-5">
    <skos:related rdf:resource="descriptor/16554-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15073-5">
    <skos:related rdf:resource="descriptor/15720-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15073-5">
    <skos:related rdf:resource="descriptor/15715-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15073-5">
    <skos:related rdf:resource="descriptor/12394-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15073-5">
    <skos:related rdf:resource="descriptor/19668-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15073-5">
    <skos:related rdf:resource="descriptor/16638-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15073-5">
    <skos:related rdf:resource="descriptor/12001-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16061-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16063-3">
    <skos:prefLabel xml:lang="en">Company law relating to groups</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16063-3">
    <skos:prefLabel xml:lang="de">Konzernrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13310-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73350">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.10.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12441-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10534-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16322-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11125-4">
    <skos:altLabel xml:lang="en">Foreign trade monopoly</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11125-4">
    <skos:altLabel xml:lang="de">Staatshandelsunternehmen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11125-4">
    <skos:altLabel xml:lang="de">Staatliche Außenwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11125-4">
    <skos:altLabel xml:lang="de">Außenhandelsmonopol</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24663-4">
    <skos:prefLabel xml:lang="en">Gender mainstreaming</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24663-4">
    <skos:prefLabel xml:lang="de">Gender Mainstreaming</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14561-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18424-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12744-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13424-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21110-4">
    <skos:broader rdf:resource="thsys/70549"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21110-4">
    <skos:broader rdf:resource="descriptor/14336-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15790-5">
    <skos:prefLabel xml:lang="en">Psychology</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15790-5">
    <skos:prefLabel xml:lang="de">Psychologie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18237-0">
    <skos:broader rdf:resource="thsys/70134"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18237-0">
    <skos:broader rdf:resource="descriptor/14274-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18237-0">
    <skos:broader rdf:resource="descriptor/14302-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10942-5">
    <skos:altLabel xml:lang="en">Collusive tendering</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71019">
    <skos:narrower rdf:resource="descriptor/15779-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71019">
    <skos:narrower rdf:resource="descriptor/15897-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71019">
    <skos:narrower rdf:resource="descriptor/18340-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71019">
    <skos:narrower rdf:resource="descriptor/19366-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71019">
    <skos:narrower rdf:resource="descriptor/19365-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71019">
    <skos:narrower rdf:resource="descriptor/12962-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71019">
    <skos:narrower rdf:resource="descriptor/19367-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71019">
    <skos:narrower rdf:resource="descriptor/18987-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71019">
    <skos:narrower rdf:resource="descriptor/18339-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71019">
    <skos:narrower rdf:resource="descriptor/12961-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71019">
    <skos:narrower rdf:resource="descriptor/19419-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71019">
    <skos:narrower rdf:resource="descriptor/19585-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71019">
    <skos:narrower rdf:resource="descriptor/18311-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71019">
    <skos:narrower rdf:resource="descriptor/18338-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71019">
    <skos:narrower rdf:resource="descriptor/18988-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71019">
    <skos:narrower rdf:resource="descriptor/13874-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71019">
    <skos:narrower rdf:resource="descriptor/18336-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71019">
    <skos:narrower rdf:resource="descriptor/18324-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71019">
    <skos:narrower rdf:resource="descriptor/18337-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16628-2">
    <skos:prefLabel xml:lang="en">Women's movement</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16628-2">
    <skos:prefLabel xml:lang="de">Frauenbewegung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11713-1">
    <skos:broader rdf:resource="thsys/71114"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11713-1">
    <skos:broader rdf:resource="thsys/71081"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11713-1">
    <skos:broader rdf:resource="descriptor/21082-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11713-1">
    <skos:broader rdf:resource="descriptor/19436-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19168-2">
    <skos:broader rdf:resource="thsys/70128"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19168-2">
    <skos:broader rdf:resource="descriptor/17948-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11426-4">
    <skos:broader rdf:resource="thsys/70055"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18670-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18699-4">
    <skos:broader rdf:resource="thsys/71103"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18699-4">
    <skos:broader rdf:resource="thsys/71088"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18699-4">
    <skos:broader rdf:resource="descriptor/13766-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19746-2">
    <skos:prefLabel xml:lang="en">Publication patterns</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19746-2">
    <skos:prefLabel xml:lang="de">Publikationsanalyse</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14251-5">
    <skos:related rdf:resource="descriptor/14252-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12154-6">
    <skos:related rdf:resource="descriptor/20277-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12154-6">
    <skos:related rdf:resource="descriptor/19557-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12154-6">
    <skos:related rdf:resource="descriptor/10590-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12154-6">
    <skos:related rdf:resource="descriptor/15051-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12154-6">
    <skos:related rdf:resource="descriptor/19832-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12154-6">
    <skos:related rdf:resource="descriptor/10561-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10414-2">
    <skos:prefLabel xml:lang="en">Economic growth</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10414-2">
    <skos:prefLabel xml:lang="de">Wirtschaftswachstum</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19249-2">
    <skos:prefLabel xml:lang="en">Optimal firm size</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19249-2">
    <skos:prefLabel xml:lang="de">Optimale Betriebsgröße</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10973-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15767-0">
    <skos:narrower rdf:resource="descriptor/15778-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15767-0">
    <skos:narrower rdf:resource="descriptor/18381-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15767-0">
    <skos:narrower rdf:resource="descriptor/19831-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15767-0">
    <skos:narrower rdf:resource="descriptor/15776-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19633-1">
    <skos:broader rdf:resource="thsys/71036"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19633-1">
    <skos:broader rdf:resource="thsys/70209"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19633-1">
    <skos:broader rdf:resource="descriptor/15598-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16587-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16723-1">
    <skos:scopeNote xml:lang="de">Philosophische Ansätze, die den Menschen als gesellschaftlich bestimmtes Wesen verstehen und deshalb die Gesellschaft zum Gegenstand haben.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19241-4">
    <skos:related rdf:resource="descriptor/18679-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19241-4">
    <skos:related rdf:resource="descriptor/15392-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19241-4">
    <skos:related rdf:resource="descriptor/11258-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19241-4">
    <skos:related rdf:resource="descriptor/10338-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11289-2">
    <skos:prefLabel xml:lang="en">Civil servants</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11289-2">
    <skos:prefLabel xml:lang="de">Beamte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16369-0">
    <skos:altLabel xml:lang="de">Wahlprogramm</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12786-4">
    <skos:broader rdf:resource="thsys/70384"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12786-4">
    <skos:broader rdf:resource="descriptor/12665-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12786-4">
    <skos:broader rdf:resource="descriptor/13301-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11164-1">
    <skos:related rdf:resource="descriptor/16013-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11164-1">
    <skos:related rdf:resource="descriptor/16430-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11164-1">
    <skos:related rdf:resource="descriptor/11168-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11164-1">
    <skos:related rdf:resource="descriptor/16621-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11164-1">
    <skos:related rdf:resource="descriptor/15825-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11164-1">
    <skos:related rdf:resource="descriptor/16658-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11164-1">
    <skos:related rdf:resource="descriptor/19753-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73377">
    <skos:prefLabel xml:lang="de">N.04.02.05  Politische Ziele und Werte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18379-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13888-5">
    <skos:broader rdf:resource="thsys/71056"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13888-5">
    <skos:broader rdf:resource="thsys/70994"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13888-5">
    <skos:broader rdf:resource="descriptor/18484-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19046-2">
    <skos:prefLabel xml:lang="en">Agricultural services</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19046-2">
    <skos:prefLabel xml:lang="de">Landwirtschaftliche Dienstleistung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13852-5">
    <skos:prefLabel xml:lang="en">Construction market</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13852-5">
    <skos:prefLabel xml:lang="de">Baumarkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/v">
    <skos:narrower rdf:resource="thsys/70175"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/v">
    <skos:narrower rdf:resource="thsys/70058"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/v">
    <skos:narrower rdf:resource="thsys/70091"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/v">
    <skos:narrower rdf:resource="thsys/70056"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/v">
    <skos:narrower rdf:resource="thsys/70179"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/v">
    <skos:narrower rdf:resource="thsys/70101"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/v">
    <skos:narrower rdf:resource="thsys/70039"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/v">
    <skos:narrower rdf:resource="thsys/72180"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/v">
    <skos:narrower rdf:resource="thsys/70055"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/v">
    <skos:narrower rdf:resource="thsys/70234"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/v">
    <skos:narrower rdf:resource="thsys/70082"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/v">
    <skos:narrower rdf:resource="thsys/70021"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/v">
    <skos:narrower rdf:resource="thsys/73104"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/v">
    <skos:narrower rdf:resource="thsys/70109"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/v">
    <skos:narrower rdf:resource="thsys/70049"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/v">
    <skos:narrower rdf:resource="thsys/73103"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/v">
    <skos:narrower rdf:resource="thsys/70180"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/v">
    <skos:narrower rdf:resource="thsys/70144"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/v">
    <skos:narrower rdf:resource="thsys/70052"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10772-4">
    <skos:scopeNote xml:lang="de">Für die historischen Fälle der Existenz von Doppelwährungen; auch für die Konkurrenz von Währungen um die Übernahme internationaler Geldfunktionen, z.B. als Zahlungsmittel oder als Wertaufbewahrungsmittel.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11805-3">
    <skos:prefLabel xml:lang="en">Agricultural surplus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11805-3">
    <skos:prefLabel xml:lang="de">Agrarüberschuss</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73358">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16460-0">
    <skos:prefLabel xml:lang="en">Armaments policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16460-0">
    <skos:prefLabel xml:lang="de">Rüstungspolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12282-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18367-1">
    <skos:broader rdf:resource="thsys/71013"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18367-1">
    <skos:broader rdf:resource="thsys/71012"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18367-1">
    <skos:broader rdf:resource="thsys/70539"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18367-1">
    <skos:broader rdf:resource="descriptor/11810-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12754-3">
    <skos:narrower rdf:resource="descriptor/20206-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18431-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24782-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70991">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18329-2">
    <skos:altLabel xml:lang="de">Landwirtschaftliches Gebäude</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18329-2">
    <skos:altLabel xml:lang="de">Landwirtschaftlicher Grundbesitz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18329-2">
    <skos:altLabel xml:lang="de">Agrarimmobilien</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18329-2">
    <skos:altLabel xml:lang="de">Agrargrundbesitz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13154-1">
    <skos:prefLabel xml:lang="en">Fishery industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13154-1">
    <skos:prefLabel xml:lang="de">Fischindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13002-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70459">
    <skos:prefLabel xml:lang="de">B.07.04  Verkauf und Distribution</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14730-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12247-6">
    <skos:broader rdf:resource="thsys/71105"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12247-6">
    <skos:broader rdf:resource="thsys/71045"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12247-6">
    <skos:broader rdf:resource="descriptor/12234-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10045-0">
    <skos:related rdf:resource="descriptor/15396-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10045-0">
    <skos:related rdf:resource="descriptor/10148-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10045-0">
    <skos:related rdf:resource="descriptor/10991-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10045-0">
    <skos:related rdf:resource="descriptor/10038-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10045-0">
    <skos:related rdf:resource="descriptor/15328-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19824-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12370-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16513-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15689-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12967-0">
    <skos:prefLabel xml:lang="en">Coastal fishery</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12967-0">
    <skos:prefLabel xml:lang="de">Küstenfischerei</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19162-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16562-6">
    <skos:related rdf:resource="descriptor/16687-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16562-6">
    <skos:related rdf:resource="descriptor/16566-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16562-6">
    <skos:related rdf:resource="descriptor/15861-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16562-6">
    <skos:related rdf:resource="descriptor/11161-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16562-6">
    <skos:related rdf:resource="descriptor/19216-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#narrowMatch">
    <skos:definition xml:lang="en">skos:narrowMatch is used to state a hierarchical mapping link
      between two conceptual resources in different concept schemes.</skos:definition>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17039-2">
    <skos:related rdf:resource="descriptor/17017-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70561">
    <skos:narrower rdf:resource="descriptor/19167-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70561">
    <skos:narrower rdf:resource="descriptor/17612-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70561">
    <skos:narrower rdf:resource="descriptor/17610-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70561">
    <skos:narrower rdf:resource="descriptor/17609-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70561">
    <skos:narrower rdf:resource="descriptor/17607-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70561">
    <skos:narrower rdf:resource="descriptor/17602-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70561">
    <skos:narrower rdf:resource="descriptor/17598-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70561">
    <skos:narrower rdf:resource="descriptor/17597-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19815-2">
    <skos:broader rdf:resource="thsys/73334"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19815-2">
    <skos:broader rdf:resource="descriptor/16201-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71091">
    <skos:prefLabel xml:lang="en">V.07.05  Foreign trade</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71091">
    <skos:prefLabel xml:lang="de">V.07.05  Außenhandel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12921-3">
    <skos:broader rdf:resource="thsys/71004"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12921-3">
    <skos:broader rdf:resource="descriptor/12993-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20110-2">
    <skos:broader rdf:resource="thsys/71097"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20110-2">
    <skos:broader rdf:resource="descriptor/19326-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13966-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14056-3">
    <skos:broader rdf:resource="thsys/70169"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14056-3">
    <skos:broader rdf:resource="descriptor/14070-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14056-3">
    <skos:broader rdf:resource="descriptor/14106-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19647-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19301-5">
    <skos:related rdf:resource="descriptor/14083-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19301-5">
    <skos:related rdf:resource="descriptor/12922-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19366-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11825-4">
    <skos:prefLabel xml:lang="en">Social economy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11825-4">
    <skos:prefLabel xml:lang="de">Gemeinwirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12919-4">
    <skos:related rdf:resource="descriptor/13004-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19228-3">
    <skos:related rdf:resource="descriptor/19612-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19228-3">
    <skos:related rdf:resource="descriptor/10042-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14837-3">
    <skos:broader rdf:resource="thsys/70066"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14837-3">
    <skos:broader rdf:resource="descriptor/14844-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11485-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70012">
    <skos:narrower rdf:resource="descriptor/16564-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70012">
    <skos:narrower rdf:resource="descriptor/16563-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70012">
    <skos:narrower rdf:resource="descriptor/18887-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70012">
    <skos:narrower rdf:resource="descriptor/15073-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70012">
    <skos:narrower rdf:resource="thsys/70321"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70012">
    <skos:narrower rdf:resource="thsys/70011"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70012">
    <skos:narrower rdf:resource="thsys/70013"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70012">
    <skos:narrower rdf:resource="thsys/70120"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11004-2">
    <zbwext:useInsteadNote rdf:nodeID="id118"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11004-2">
    <zbwext:useInsteadNote rdf:nodeID="id117"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16327-2">
    <skos:editorialNote xml:lang="de">Spezielle Ministerien sind ggf. mit dem entsprechenden Politik-Deskriptor zu postkoordinieren</skos:editorialNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18334-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19772-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18935-4">
    <skos:broader rdf:resource="thsys/70824"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18935-4">
    <skos:broader rdf:resource="descriptor/13360-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71038">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70256">
    <skos:prefLabel xml:lang="de">P.09.01  Luftfahrzeuge</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14366-2">
    <skos:prefLabel xml:lang="en">Construction machinery</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14366-2">
    <skos:prefLabel xml:lang="de">Baumaschine</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10797-2">
    <skos:narrower rdf:resource="descriptor/19358-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13527-2">
    <skos:related rdf:resource="descriptor/18501-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13527-2">
    <skos:related rdf:resource="descriptor/13255-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13527-2">
    <skos:related rdf:resource="descriptor/12010-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13527-2">
    <skos:related rdf:resource="descriptor/13574-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13527-2">
    <skos:related rdf:resource="descriptor/13523-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19553-6">
    <zbwext:useInsteadNote rdf:nodeID="id234"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19553-6">
    <zbwext:useInsteadNote rdf:nodeID="id233"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17980-4">
    <skos:prefLabel xml:lang="en">Benelux Countries</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17980-4">
    <skos:prefLabel xml:lang="de">Benelux-Staaten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18736-3">
    <skos:prefLabel xml:lang="en">Fraud</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18736-3">
    <skos:prefLabel xml:lang="de">Betrug</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11940-4">
    <skos:altLabel xml:lang="de">Regionale Industrieförderung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11940-4">
    <skos:altLabel xml:lang="de">Industrieansiedlung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11940-4">
    <skos:altLabel xml:lang="de">Betriebsansiedlung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70300">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12515-0">
    <skos:altLabel xml:lang="en">Variable costing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12515-0">
    <skos:altLabel xml:lang="de">Proportionalkostenrechnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12515-0">
    <skos:altLabel xml:lang="de">Direktkostenrechnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12515-0">
    <skos:altLabel xml:lang="de">Blockkostenrechnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18915-3">
    <skos:broader rdf:resource="thsys/70829"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18915-3">
    <skos:broader rdf:resource="thsys/70814"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18915-3">
    <skos:broader rdf:resource="descriptor/18916-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10202-3">
    <skos:prefLabel xml:lang="en">Market segmentation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10202-3">
    <skos:prefLabel xml:lang="de">Marktsegmentierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13553-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70903">
    <skos:narrower rdf:resource="descriptor/13697-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70903">
    <skos:narrower rdf:resource="descriptor/16072-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70903">
    <skos:narrower rdf:resource="descriptor/13290-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70903">
    <skos:narrower rdf:resource="descriptor/13708-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70903">
    <skos:narrower rdf:resource="descriptor/13706-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70903">
    <skos:narrower rdf:resource="descriptor/18705-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70903">
    <skos:narrower rdf:resource="descriptor/13289-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70903">
    <skos:narrower rdf:resource="descriptor/13287-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70903">
    <skos:narrower rdf:resource="descriptor/18177-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70903">
    <skos:narrower rdf:resource="descriptor/10806-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70903">
    <skos:narrower rdf:resource="descriptor/13701-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70903">
    <skos:narrower rdf:resource="descriptor/24665-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70903">
    <skos:narrower rdf:resource="descriptor/13690-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70903">
    <skos:narrower rdf:resource="thsys/70902"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70903">
    <skos:narrower rdf:resource="thsys/70901"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15839-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12057-4">
    <skos:related rdf:resource="descriptor/10184-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12057-4">
    <skos:related rdf:resource="descriptor/19784-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12057-4">
    <skos:related rdf:resource="descriptor/18115-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12057-4">
    <skos:related rdf:resource="descriptor/12070-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12057-4">
    <skos:related rdf:resource="descriptor/12069-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12057-4">
    <skos:related rdf:resource="descriptor/10020-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12057-4">
    <skos:related rdf:resource="descriptor/12058-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12057-4">
    <skos:related rdf:resource="descriptor/19761-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12057-4">
    <skos:related rdf:resource="descriptor/11740-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12057-4">
    <skos:related rdf:resource="descriptor/19159-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12057-4">
    <skos:related rdf:resource="descriptor/10282-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11342-3">
    <skos:related rdf:resource="descriptor/11346-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14692-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13528-0">
    <skos:prefLabel xml:lang="en">Inland port</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13528-0">
    <skos:prefLabel xml:lang="de">Binnenhafen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13304-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14244-2">
    <skos:broader rdf:resource="thsys/70348"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14244-2">
    <skos:broader rdf:resource="descriptor/14236-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18085-4">
    <skos:related rdf:resource="descriptor/19556-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18085-4">
    <skos:related rdf:resource="descriptor/18671-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18085-4">
    <skos:related rdf:resource="descriptor/13723-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18085-4">
    <skos:related rdf:resource="descriptor/11414-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18085-4">
    <skos:related rdf:resource="descriptor/25742-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14251-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12990-5">
    <skos:related rdf:resource="descriptor/18374-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12990-5">
    <skos:related rdf:resource="descriptor/19192-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12990-5">
    <skos:related rdf:resource="descriptor/14200-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19711-0">
    <skos:narrower rdf:resource="descriptor/15772-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19364-2">
    <skos:related rdf:resource="descriptor/11897-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19364-2">
    <skos:related rdf:resource="descriptor/11855-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19364-2">
    <skos:related rdf:resource="descriptor/20321-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19364-2">
    <skos:related rdf:resource="descriptor/10515-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12961-5">
    <skos:prefLabel xml:lang="en">Forestry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12961-5">
    <skos:prefLabel xml:lang="de">Forstwirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11691-6">
    <skos:prefLabel xml:lang="en">Tax fairness</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11691-6">
    <skos:prefLabel xml:lang="de">Steuergerechtigkeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19394-0">
    <skos:prefLabel xml:lang="en">Food consumption</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19394-0">
    <skos:prefLabel xml:lang="de">Nahrungsmittelkonsum</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10428-5">
    <skos:broader rdf:resource="thsys/70406"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10428-5">
    <skos:broader rdf:resource="descriptor/10434-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20762-1">
    <skos:prefLabel xml:lang="en">Emissions trading</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20762-1">
    <skos:prefLabel xml:lang="de">Emissionshandel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19181-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19204-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19328-6">
    <skos:scopeNote xml:lang="de">Transferzahlungen zwischen Staaten.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12042-3">
    <skos:related rdf:resource="descriptor/12160-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12042-3">
    <skos:related rdf:resource="descriptor/13413-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12042-3">
    <skos:related rdf:resource="descriptor/18454-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12042-3">
    <skos:related rdf:resource="descriptor/13985-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12042-3">
    <skos:related rdf:resource="descriptor/14023-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71079">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19310-4">
    <skos:prefLabel xml:lang="en">Protectionism</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19310-4">
    <skos:prefLabel xml:lang="de">Protektionismus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12134-5">
    <skos:narrower rdf:resource="descriptor/19155-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12134-5">
    <skos:narrower rdf:resource="descriptor/24704-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25742-3">
    <skos:narrower rdf:resource="descriptor/19699-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25742-3">
    <skos:narrower rdf:resource="descriptor/18624-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25742-3">
    <skos:narrower rdf:resource="descriptor/12205-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25742-3">
    <skos:narrower rdf:resource="descriptor/18097-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25742-3">
    <skos:narrower rdf:resource="descriptor/18913-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25742-3">
    <skos:narrower rdf:resource="descriptor/12259-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25742-3">
    <skos:narrower rdf:resource="descriptor/18939-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25742-3">
    <skos:narrower rdf:resource="descriptor/12262-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25742-3">
    <skos:narrower rdf:resource="descriptor/19557-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25742-3">
    <skos:narrower rdf:resource="descriptor/20018-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25742-3">
    <skos:narrower rdf:resource="descriptor/16365-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25742-3">
    <skos:narrower rdf:resource="descriptor/18922-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25742-3">
    <skos:narrower rdf:resource="descriptor/19381-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25742-3">
    <skos:narrower rdf:resource="descriptor/10805-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25742-3">
    <skos:narrower rdf:resource="descriptor/18750-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25742-3">
    <skos:narrower rdf:resource="descriptor/18784-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25742-3">
    <skos:narrower rdf:resource="descriptor/10552-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25742-3">
    <skos:narrower rdf:resource="descriptor/18884-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25742-3">
    <skos:narrower rdf:resource="descriptor/10854-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14392-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14940-3">
    <skos:broader rdf:resource="thsys/70498"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14940-3">
    <skos:broader rdf:resource="descriptor/14931-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10763-5">
    <skos:broader rdf:resource="thsys/71095"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10763-5">
    <skos:broader rdf:resource="descriptor/10764-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11840-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18124-6">
    <skos:altLabel xml:lang="de">Dauerarbeitslosigkeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14392-1">
    <skos:narrower rdf:resource="descriptor/18244-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70943">
    <skos:broader rdf:resource="thsys/w"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10179-0">
    <skos:altLabel xml:lang="en">Overlapping generations model of general equilibrium</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10179-0">
    <skos:altLabel xml:lang="en">OLG (Overlapping generations)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15733-3">
    <skos:narrower rdf:resource="descriptor/15744-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18127-0">
    <skos:related rdf:resource="descriptor/10772-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18127-0">
    <skos:related rdf:resource="descriptor/10770-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18127-0">
    <skos:related rdf:resource="descriptor/10764-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18127-0">
    <skos:related rdf:resource="descriptor/19260-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12120-2">
    <skos:narrower rdf:resource="descriptor/20948-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11416-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/a">
    <rdfs:label xml:lang="en">General descriptors</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/a">
    <rdfs:label xml:lang="de">Allgemeinwörter</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11170-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13490-6">
    <skos:broader rdf:resource="thsys/70947"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13490-6">
    <skos:broader rdf:resource="descriptor/13480-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11825-4">
    <skos:broader rdf:resource="thsys/70855"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11825-4">
    <skos:broader rdf:resource="descriptor/11068-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11825-4">
    <skos:broader rdf:resource="descriptor/12047-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15910-0">
    <skos:scopeNote xml:lang="de">Einwohnerzahl je Fläche</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18934-6">
    <skos:altLabel xml:lang="de">TV-Programm</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18934-6">
    <skos:altLabel xml:lang="de">Fernsehsendung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16868-3">
    <skos:altLabel xml:lang="de">Hansestadt Hamburg</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16868-3">
    <skos:altLabel xml:lang="de">Freie und Hansestadt Hamburg</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70235">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.13.03</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73339">
    <rdfs:label xml:lang="en">Computer science</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73339">
    <rdfs:label xml:lang="de">Informatik</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73333">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.05.08</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18242-0">
    <skos:broader rdf:resource="thsys/70415"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18242-0">
    <skos:broader rdf:resource="descriptor/14449-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13456-6">
    <skos:prefLabel xml:lang="en">Purchasing cooperative</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13456-6">
    <skos:prefLabel xml:lang="de">Einkaufsgenossenschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72393">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.16.07</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10433-5">
    <skos:prefLabel xml:lang="en">Vintage model</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10433-5">
    <skos:prefLabel xml:lang="de">Vintage-Modell</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12667-5">
    <skos:broader rdf:resource="thsys/70368"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12667-5">
    <skos:broader rdf:resource="thsys/70030"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15644-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19484-6">
    <skos:prefLabel xml:lang="en">Soil erosion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19484-6">
    <skos:prefLabel xml:lang="de">Bodenerosion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13393-4">
    <skos:prefLabel xml:lang="en">Church</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13393-4">
    <skos:prefLabel xml:lang="de">Kirche</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70368">
    <skos:prefLabel xml:lang="de">B.05.01  Materialwirtschaft und Beschaffung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13155-6">
    <skos:altLabel xml:lang="en">Meat processing industry</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13155-6">
    <skos:altLabel xml:lang="de">Schlachthof</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13155-6">
    <skos:altLabel xml:lang="de">Fleischwarenindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13155-6">
    <skos:altLabel xml:lang="de">Fleischverarbeitende Industrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13828-2">
    <skos:altLabel xml:lang="en">Insurance industry</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13828-2">
    <skos:altLabel xml:lang="de">Versicherungswirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13828-2">
    <skos:altLabel xml:lang="de">Versicherungswesen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13828-2">
    <skos:altLabel xml:lang="de">Assekuranz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70932">
    <skos:prefLabel xml:lang="en">W.12.01.04  Air transport</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70932">
    <skos:prefLabel xml:lang="de">W.12.01.04  Luftverkehr</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19199-5">
    <skos:altLabel xml:lang="en">State dependence (Labour Market)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19199-5">
    <skos:altLabel xml:lang="de">Hysteresis-Effekt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19199-5">
    <skos:altLabel xml:lang="de">Hysterese</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11879-2">
    <skos:broader rdf:resource="thsys/71068"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19775-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15392-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16486-3">
    <skos:altLabel xml:lang="de">Wanderungspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16486-3">
    <skos:altLabel xml:lang="de">Ausländerpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18523-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13469-4">
    <skos:narrower rdf:resource="descriptor/13457-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13469-4">
    <skos:narrower rdf:resource="descriptor/12668-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15606-3">
    <skos:related rdf:resource="descriptor/15621-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15606-3">
    <skos:related rdf:resource="descriptor/10455-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15606-3">
    <skos:related rdf:resource="descriptor/16757-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15606-3">
    <skos:related rdf:resource="descriptor/15598-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15606-3">
    <skos:related rdf:resource="descriptor/15609-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15606-3">
    <skos:related rdf:resource="descriptor/13987-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15606-3">
    <skos:related rdf:resource="descriptor/18762-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15606-3">
    <skos:related rdf:resource="descriptor/15605-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15606-3">
    <skos:related rdf:resource="descriptor/18433-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15606-3">
    <skos:related rdf:resource="descriptor/14751-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15606-3">
    <skos:related rdf:resource="descriptor/19660-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13073-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16427-5">
    <skos:altLabel xml:lang="de">Weltfrieden</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16427-5">
    <skos:altLabel xml:lang="de">Waffenstillstand</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16427-5">
    <skos:altLabel xml:lang="de">Friedensvertrag</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16427-5">
    <skos:altLabel xml:lang="de">Friedensschluss</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16427-5">
    <skos:altLabel xml:lang="de">Friedensschluß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19511-1">
    <skos:related rdf:resource="descriptor/15979-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19511-1">
    <skos:related rdf:resource="descriptor/19763-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19511-1">
    <skos:related rdf:resource="descriptor/16195-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19331-3">
    <skos:related rdf:resource="descriptor/11703-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18665-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11391-4">
    <skos:related rdf:resource="descriptor/11399-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17682-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19223-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18592-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70991">
    <rdfs:label xml:lang="en">Timber industry and trade</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70991">
    <rdfs:label xml:lang="de">Holzwirtschaft</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19445-2">
    <skos:prefLabel xml:lang="en">EU structural funds</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19445-2">
    <skos:prefLabel xml:lang="de">EU-Strukturfonds</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18325-3">
    <skos:altLabel xml:lang="en">Sustainable economics</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18325-3">
    <skos:altLabel xml:lang="de">Zukunftsfähige Entwicklung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18325-3">
    <skos:altLabel xml:lang="de">Umweltverträgliche Entwicklung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18325-3">
    <skos:altLabel xml:lang="de">Qualitatives Wachstum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18325-3">
    <skos:altLabel xml:lang="de">Ökologisch verträgliche Entwicklung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18325-3">
    <skos:altLabel xml:lang="de">Nachhaltiges Wirtschaften</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18325-3">
    <skos:altLabel xml:lang="de">Dauerhaft-umweltgerechte Entwicklung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18325-3">
    <skos:altLabel xml:lang="de">Dauerhafte Entwicklung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11347-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12808-6">
    <skos:broader rdf:resource="thsys/70550"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12808-6">
    <skos:broader rdf:resource="descriptor/12770-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15659-3">
    <skos:altLabel xml:lang="en">Mankind</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15659-3">
    <skos:altLabel xml:lang="en">Human nature</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15659-3">
    <skos:altLabel xml:lang="en">Human factor</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15659-3">
    <skos:altLabel xml:lang="de">Menschheit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15659-3">
    <skos:altLabel xml:lang="de">Menschenbild</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15659-3">
    <skos:altLabel xml:lang="de">Menschen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70114">
    <skos:broader rdf:resource="thsys/70055"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15050-3">
    <skos:altLabel xml:lang="de">Vektorrechnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15050-3">
    <skos:altLabel xml:lang="de">Matrixalgebra</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10030-6">
    <skos:narrower rdf:resource="descriptor/20176-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10030-6">
    <skos:narrower rdf:resource="descriptor/11718-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10030-6">
    <skos:narrower rdf:resource="descriptor/10661-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10030-6">
    <skos:narrower rdf:resource="descriptor/11717-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10030-6">
    <skos:narrower rdf:resource="descriptor/10301-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10030-6">
    <skos:narrower rdf:resource="descriptor/19217-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10030-6">
    <skos:narrower rdf:resource="descriptor/10653-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10030-6">
    <skos:narrower rdf:resource="descriptor/24664-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18078-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14728-1">
    <skos:altLabel xml:lang="de">Schlafmittel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14728-1">
    <skos:altLabel xml:lang="de">Salbe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14728-1">
    <skos:altLabel xml:lang="de">Pharmazeutikum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14728-1">
    <skos:altLabel xml:lang="de">Pflanzliches Arzneimittel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14728-1">
    <skos:altLabel xml:lang="de">Medizinische Salbe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14728-1">
    <skos:altLabel xml:lang="de">Medikament</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14728-1">
    <skos:altLabel xml:lang="de">Impfstoff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14728-1">
    <skos:altLabel xml:lang="de">Humanarzneimittel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14728-1">
    <skos:altLabel xml:lang="de">Homöopathisches Produkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14728-1">
    <skos:altLabel xml:lang="de">Heilmittel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14728-1">
    <skos:altLabel xml:lang="de">Desinfektionsmittel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14728-1">
    <skos:altLabel xml:lang="de">Arzneimittel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14728-1">
    <skos:altLabel xml:lang="de">Arzneidroge</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14728-1">
    <skos:altLabel xml:lang="de">Antibiotikum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11728-2">
    <skos:broader rdf:resource="thsys/73372"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11728-2">
    <skos:broader rdf:resource="thsys/71082"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11728-2">
    <skos:broader rdf:resource="descriptor/10987-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70819">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.31</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16072-2">
    <skos:scopeNote xml:lang="de">Alle bankrechtlichen Regelungen, welche die Sparkassen betreffen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19039-6">
    <skos:broader rdf:resource="thsys/70582"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18263-6">
    <skos:prefLabel xml:lang="en">Men's toiletries</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18263-6">
    <skos:prefLabel xml:lang="de">Herrenkosmetika</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12452-5">
    <skos:related rdf:resource="descriptor/12796-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12452-5">
    <skos:related rdf:resource="descriptor/16063-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12452-5">
    <skos:related rdf:resource="descriptor/12056-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12452-5">
    <skos:related rdf:resource="descriptor/12313-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12452-5">
    <skos:related rdf:resource="descriptor/12446-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12452-5">
    <skos:related rdf:resource="descriptor/12418-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13871-1">
    <skos:altLabel xml:lang="de">Freier Beruf</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11409-4">
    <skos:prefLabel xml:lang="en">Public education expenditures</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11409-4">
    <skos:prefLabel xml:lang="de">Öffentliche Bildungsausgaben</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18657-6">
    <skos:broader rdf:resource="thsys/73341"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18657-6">
    <skos:broader rdf:resource="descriptor/15599-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18573-5">
    <skos:altLabel xml:lang="de">Güterbahn</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24781-5">
    <skos:prefLabel xml:lang="en">Customer value</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24781-5">
    <skos:prefLabel xml:lang="de">Kundenwert</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10590-3">
    <skos:altLabel xml:lang="en">Project appraisal</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10590-3">
    <skos:altLabel xml:lang="de">Projektevaluierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10590-3">
    <skos:altLabel xml:lang="de">Monitoring von Projekten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10590-3">
    <skos:altLabel xml:lang="de">Evaluierung von Projekten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10590-3">
    <skos:altLabel xml:lang="de">Bewertung von Projekten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10173-5">
    <skos:narrower rdf:resource="descriptor/19221-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10488-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16193-4">
    <skos:broader rdf:resource="thsys/73376"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16193-4">
    <skos:broader rdf:resource="thsys/73334"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16193-4">
    <skos:broader rdf:resource="descriptor/16195-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11228-1">
    <skos:scopeNote xml:lang="de">Jede Art von beruflicher Qualifikation, die während der Beschäftigung erworben wird, mit Ausnahme der formalen Berufsausbildung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70418">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19449-1">
    <skos:altLabel xml:lang="de">Regionalökologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19449-1">
    <skos:altLabel xml:lang="de">Ökologisch orientierte Raumplanung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71077">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18513-2">
    <skos:related rdf:resource="descriptor/13466-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14981-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70929">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.12.04</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15903-4">
    <skos:prefLabel xml:lang="en">Desert</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15903-4">
    <skos:prefLabel xml:lang="de">Wüste</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13473-6">
    <skos:narrower rdf:resource="descriptor/18551-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13473-6">
    <skos:narrower rdf:resource="descriptor/13480-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13473-6">
    <skos:narrower rdf:resource="descriptor/13496-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13575-5">
    <skos:altLabel xml:lang="de">Luftverkehrstechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13575-5">
    <skos:altLabel xml:lang="de">Flugzeugtechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14299-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12136-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15111-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18457-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19741-5">
    <skos:altLabel xml:lang="en">Field study</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19741-5">
    <skos:altLabel xml:lang="de">Feldstudie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19741-5">
    <skos:altLabel xml:lang="de">Empirische Feldforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13434-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11804-5">
    <skos:altLabel xml:lang="de">Flurbereinigung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11770-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19717-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id24">
    <rdf:value xml:lang="de">für berufs- oder arbeitsmarktbedingte Wanderungen von Arbeitskräften innerhalb eines Staates</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18800-6">
    <skos:broader rdf:resource="thsys/71064"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18800-6">
    <skos:broader rdf:resource="thsys/71063"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18800-6">
    <skos:broader rdf:resource="descriptor/11716-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11872-2">
    <skos:narrower rdf:resource="descriptor/19233-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19451-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19273-5">
    <skos:broader rdf:resource="thsys/71065"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19273-5">
    <skos:broader rdf:resource="thsys/70187"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19273-5">
    <skos:broader rdf:resource="descriptor/13723-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12312-0">
    <skos:prefLabel xml:lang="en">Earnings statement</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12312-0">
    <skos:prefLabel xml:lang="de">Erfolgsrechnung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72183">
    <skos:narrower rdf:resource="descriptor/15396-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72183">
    <skos:narrower rdf:resource="descriptor/19570-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72183">
    <skos:narrower rdf:resource="descriptor/15111-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72183">
    <skos:narrower rdf:resource="descriptor/18161-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72183">
    <skos:narrower rdf:resource="descriptor/15117-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72183">
    <skos:narrower rdf:resource="descriptor/15115-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72183">
    <skos:narrower rdf:resource="descriptor/19605-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72183">
    <skos:narrower rdf:resource="descriptor/15392-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72183">
    <skos:narrower rdf:resource="descriptor/15320-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72183">
    <skos:narrower rdf:resource="descriptor/15126-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72183">
    <skos:narrower rdf:resource="descriptor/15072-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72183">
    <skos:narrower rdf:resource="descriptor/19149-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72183">
    <skos:narrower rdf:resource="descriptor/15125-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72183">
    <skos:narrower rdf:resource="descriptor/15124-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72183">
    <skos:narrower rdf:resource="descriptor/15149-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72183">
    <skos:narrower rdf:resource="descriptor/15119-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72183">
    <skos:narrower rdf:resource="descriptor/15023-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72183">
    <skos:narrower rdf:resource="descriptor/15142-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72183">
    <skos:narrower rdf:resource="descriptor/19609-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72183">
    <skos:narrower rdf:resource="thsys/72184"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14392-1">
    <skos:altLabel xml:lang="de">Wirkmaschine</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14392-1">
    <skos:altLabel xml:lang="de">Webmaschine</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14392-1">
    <skos:altLabel xml:lang="de">Strickmaschine</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14392-1">
    <skos:altLabel xml:lang="de">Spulmaschine</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14392-1">
    <skos:altLabel xml:lang="de">Bekleidungsmaschine</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19274-3">
    <skos:related rdf:resource="descriptor/13826-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19274-3">
    <skos:related rdf:resource="descriptor/12210-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19274-3">
    <skos:related rdf:resource="descriptor/19271-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19274-3">
    <skos:related rdf:resource="descriptor/13715-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10523-4">
    <skos:broader rdf:resource="thsys/71085"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10523-4">
    <skos:broader rdf:resource="descriptor/11004-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10523-4">
    <skos:broader rdf:resource="descriptor/10560-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16678-1">
    <skos:prefLabel xml:lang="en">Women's studies</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16678-1">
    <skos:prefLabel xml:lang="de">Frauenforschung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13364-4">
    <skos:prefLabel xml:lang="en">Theatre</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13364-4">
    <skos:prefLabel xml:lang="de">Theater</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id299">
    <rdfs:seeAlso rdf:resource="descriptor/13790-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14236-1">
    <skos:scopeNote xml:lang="de">Keine einheitliche Definition; wird in der Literatur teilweise verstanden als: Nicht-Metall.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12598-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10443-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16146-6">
    <skos:altLabel xml:lang="en">Maternity leave</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16146-6">
    <skos:altLabel xml:lang="de">Schwangerschaftsschutz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16146-6">
    <skos:altLabel xml:lang="de">Mutterschutzgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16146-6">
    <skos:altLabel xml:lang="de">Mutterschaftsurlaub</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16146-6">
    <skos:altLabel xml:lang="de">Gesetz zum Schutze der erwerbstätigen Mutter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13186-2">
    <skos:scopeNote xml:lang="de">Einbau von Geschäftseinrichtungen</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70992">
    <skos:prefLabel xml:lang="en">W.06.01.06.02  Petroleum industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70992">
    <skos:prefLabel xml:lang="de">W.06.01.06.02  Mineralölwirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14404-6">
    <skos:altLabel xml:lang="de">Zentralheizung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14404-6">
    <skos:altLabel xml:lang="de">Wärmetechnische Anlage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14404-6">
    <skos:altLabel xml:lang="de">Raumheizung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14404-6">
    <skos:altLabel xml:lang="de">Ofen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14404-6">
    <skos:altLabel xml:lang="de">Heizsystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14404-6">
    <skos:altLabel xml:lang="de">Heizkörper</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14404-6">
    <skos:altLabel xml:lang="de">Heizkessel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14404-6">
    <skos:altLabel xml:lang="de">Heizgerät</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70238">
    <skos:prefLabel xml:lang="en">V.12.01.02  Environmental degradation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70238">
    <skos:prefLabel xml:lang="de">V.12.01.02  Umweltbelastung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24666-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18203-3">
    <skos:prefLabel xml:lang="en">Fuelwood</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18203-3">
    <skos:prefLabel xml:lang="de">Brennholz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11336-5">
    <skos:narrower rdf:resource="descriptor/11335-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11336-5">
    <skos:narrower rdf:resource="descriptor/11324-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11336-5">
    <skos:narrower rdf:resource="descriptor/18848-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11336-5">
    <skos:narrower rdf:resource="descriptor/18173-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19716-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10342-1">
    <skos:broader rdf:resource="thsys/70021"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73303">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15712-4">
    <skos:broader rdf:resource="thsys/73359"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15709-0">
    <skos:prefLabel xml:lang="en">Mobility</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15709-0">
    <skos:prefLabel xml:lang="de">Mobilität</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70485">
    <skos:broader rdf:resource="thsys/70028"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18079-6">
    <skos:related rdf:resource="descriptor/19482-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18079-6">
    <skos:related rdf:resource="descriptor/18069-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18079-6">
    <skos:related rdf:resource="descriptor/14631-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14651-3">
    <skos:related rdf:resource="descriptor/15755-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10779-4">
    <skos:altLabel xml:lang="en">Currency market</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10779-4">
    <skos:altLabel xml:lang="de">Devisenkassamarkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16068-0">
    <skos:altLabel xml:lang="de">Scheckgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10305-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19613-0">
    <skos:related rdf:resource="descriptor/14814-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19613-0">
    <skos:related rdf:resource="descriptor/11957-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13060-3">
    <skos:prefLabel xml:lang="en">Commercial vehicle industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13060-3">
    <skos:prefLabel xml:lang="de">Nutzfahrzeugindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10801-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16948-5">
    <skos:prefLabel xml:lang="en">Lower Austria</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16948-5">
    <skos:prefLabel xml:lang="de">Niederösterreich</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18254-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13965-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10583-0">
    <skos:broader rdf:resource="thsys/73366"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10583-0">
    <skos:broader rdf:resource="thsys/71084"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12246-1">
    <skos:related rdf:resource="descriptor/19124-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11289-2">
    <skos:altLabel xml:lang="de">Berufsbeamtentum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11289-2">
    <skos:altLabel xml:lang="de">Beamtinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11289-2">
    <skos:altLabel xml:lang="de">Beamter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11289-2">
    <skos:altLabel xml:lang="de">Beamtentum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16120-3">
    <skos:scopeNote xml:lang="de">Alle berufsrechtlichen Regelungen für Wirtschaftsprüfer.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14651-3">
    <skos:broader rdf:resource="thsys/70463"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14651-3">
    <skos:broader rdf:resource="descriptor/14652-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13615-5">
    <skos:narrower rdf:resource="descriptor/18570-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13615-5">
    <skos:narrower rdf:resource="descriptor/18595-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70058">
    <skos:narrower rdf:resource="descriptor/11916-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70058">
    <skos:narrower rdf:resource="descriptor/11917-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70058">
    <skos:narrower rdf:resource="descriptor/11963-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70058">
    <skos:narrower rdf:resource="descriptor/11876-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70058">
    <skos:narrower rdf:resource="descriptor/11962-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70058">
    <skos:narrower rdf:resource="descriptor/18608-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70058">
    <skos:narrower rdf:resource="descriptor/11943-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70058">
    <skos:narrower rdf:resource="descriptor/19246-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70058">
    <skos:narrower rdf:resource="descriptor/19240-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70058">
    <skos:narrower rdf:resource="descriptor/19272-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70058">
    <skos:narrower rdf:resource="descriptor/19357-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70058">
    <skos:narrower rdf:resource="descriptor/11207-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70058">
    <skos:narrower rdf:resource="descriptor/11910-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70058">
    <skos:narrower rdf:resource="descriptor/19234-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70058">
    <skos:narrower rdf:resource="descriptor/19224-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70058">
    <skos:narrower rdf:resource="descriptor/19440-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70058">
    <skos:narrower rdf:resource="descriptor/18480-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70058">
    <skos:narrower rdf:resource="descriptor/11895-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70058">
    <skos:narrower rdf:resource="descriptor/11872-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70058">
    <skos:narrower rdf:resource="descriptor/11911-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70058">
    <skos:narrower rdf:resource="descriptor/11894-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70058">
    <skos:narrower rdf:resource="descriptor/11206-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70058">
    <skos:narrower rdf:resource="descriptor/19492-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70058">
    <skos:narrower rdf:resource="descriptor/19439-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70058">
    <skos:narrower rdf:resource="descriptor/13708-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70058">
    <skos:narrower rdf:resource="descriptor/11857-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70058">
    <skos:narrower rdf:resource="descriptor/19438-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70058">
    <skos:narrower rdf:resource="descriptor/13414-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70058">
    <skos:narrower rdf:resource="descriptor/11855-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70058">
    <skos:narrower rdf:resource="descriptor/11893-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70058">
    <skos:narrower rdf:resource="descriptor/15883-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70058">
    <skos:narrower rdf:resource="descriptor/10658-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70058">
    <skos:narrower rdf:resource="descriptor/10124-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70058">
    <skos:narrower rdf:resource="descriptor/18489-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70058">
    <skos:narrower rdf:resource="descriptor/11853-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70058">
    <skos:narrower rdf:resource="descriptor/11850-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70058">
    <skos:narrower rdf:resource="descriptor/11888-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70058">
    <skos:narrower rdf:resource="descriptor/19437-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70058">
    <skos:narrower rdf:resource="thsys/71070"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70058">
    <skos:narrower rdf:resource="thsys/71069"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70058">
    <skos:narrower rdf:resource="thsys/71067"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70058">
    <skos:narrower rdf:resource="thsys/71066"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70058">
    <skos:narrower rdf:resource="thsys/71065"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70058">
    <skos:narrower rdf:resource="thsys/71064"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70058">
    <skos:narrower rdf:resource="thsys/71063"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14232-2">
    <skos:prefLabel xml:lang="en">Lime</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14232-2">
    <skos:prefLabel xml:lang="de">Kalk</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10750-0">
    <skos:prefLabel xml:lang="en">Convertibility</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10750-0">
    <skos:prefLabel xml:lang="de">Konvertibilität</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11850-5">
    <skos:broader rdf:resource="thsys/70924"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11850-5">
    <skos:broader rdf:resource="thsys/70058"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16264-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19588-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11916-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10513-0">
    <skos:broader rdf:resource="thsys/70091"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19155-4">
    <skos:prefLabel xml:lang="en">Workflow management system</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19155-4">
    <skos:prefLabel xml:lang="de">Workflow-Management-System</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18495-6">
    <skos:related rdf:resource="descriptor/12702-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18495-6">
    <skos:related rdf:resource="descriptor/18508-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18495-6">
    <skos:related rdf:resource="descriptor/12905-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18495-6">
    <skos:related rdf:resource="descriptor/12904-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18495-6">
    <skos:related rdf:resource="descriptor/14021-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18495-6">
    <skos:related rdf:resource="descriptor/18493-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18495-6">
    <skos:related rdf:resource="descriptor/13045-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18495-6">
    <skos:related rdf:resource="descriptor/14366-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18495-6">
    <skos:related rdf:resource="descriptor/18491-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12714-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18010-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18135-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13959-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17732-2">
    <skos:prefLabel xml:lang="en">Cameroon</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17732-2">
    <skos:prefLabel xml:lang="de">Kamerun</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12296-0">
    <skos:narrower rdf:resource="descriptor/12298-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70234">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73335">
    <skos:broader rdf:resource="thsys/70154"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11393-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13699-1">
    <skos:broader rdf:resource="thsys/70901"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13699-1">
    <skos:broader rdf:resource="descriptor/12281-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17979-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14096-5">
    <skos:broader rdf:resource="thsys/70070"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14096-5">
    <skos:broader rdf:resource="descriptor/14088-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19568-0">
    <skos:altLabel xml:lang="de">Jackknife-Ansatz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16191-1">
    <skos:related rdf:resource="descriptor/19824-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16191-1">
    <skos:related rdf:resource="descriptor/18116-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16191-1">
    <skos:related rdf:resource="descriptor/19855-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16191-1">
    <skos:related rdf:resource="descriptor/19848-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19473-4">
    <skos:altLabel xml:lang="de">Stadtplanung und Umwelt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19473-4">
    <skos:altLabel xml:lang="de">Ökologische Stadtplanung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70922">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14522-0">
    <skos:narrower rdf:resource="descriptor/14525-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14522-0">
    <skos:narrower rdf:resource="descriptor/14513-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14522-0">
    <skos:narrower rdf:resource="descriptor/14516-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13152-5">
    <skos:broader rdf:resource="thsys/73295"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13152-5">
    <skos:broader rdf:resource="descriptor/13157-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71033">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">B.01.06</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15763-1">
    <skos:broader rdf:resource="thsys/73351"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15763-1">
    <skos:broader rdf:resource="thsys/71033"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15763-1">
    <skos:broader rdf:resource="thsys/70815"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15763-1">
    <skos:broader rdf:resource="thsys/70238"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12572-2">
    <skos:related rdf:resource="descriptor/12565-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12572-2">
    <skos:related rdf:resource="descriptor/12589-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12572-2">
    <skos:related rdf:resource="descriptor/12573-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12572-2">
    <skos:related rdf:resource="descriptor/20381-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12572-2">
    <skos:related rdf:resource="descriptor/11191-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12572-2">
    <skos:related rdf:resource="descriptor/11266-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14630-4">
    <skos:related rdf:resource="descriptor/18435-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14630-4">
    <skos:related rdf:resource="descriptor/14846-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14630-4">
    <skos:related rdf:resource="descriptor/18094-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14630-4">
    <skos:related rdf:resource="descriptor/14740-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14630-4">
    <skos:related rdf:resource="descriptor/13135-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14630-4">
    <skos:related rdf:resource="descriptor/14744-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11863-3">
    <skos:altLabel xml:lang="de">Einzugsgebiet</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70539">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70030">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">B.06</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12587-3">
    <skos:related rdf:resource="descriptor/10463-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12587-3">
    <skos:related rdf:resource="descriptor/12155-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12587-3">
    <skos:related rdf:resource="descriptor/11052-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12587-3">
    <skos:related rdf:resource="descriptor/18070-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12587-3">
    <skos:related rdf:resource="descriptor/10475-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70029">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12491-2">
    <skos:broader rdf:resource="thsys/70578"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12491-2">
    <skos:broader rdf:resource="thsys/70270"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12491-2">
    <skos:broader rdf:resource="descriptor/10011-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72182">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10492-3">
    <skos:related rdf:resource="descriptor/10508-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10492-3">
    <skos:related rdf:resource="descriptor/19550-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10492-3">
    <skos:related rdf:resource="descriptor/10525-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10492-3">
    <skos:related rdf:resource="descriptor/10560-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10492-3">
    <skos:related rdf:resource="descriptor/10555-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10492-3">
    <skos:related rdf:resource="descriptor/19361-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10492-3">
    <skos:related rdf:resource="descriptor/10513-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17914-3">
    <skos:broader rdf:resource="thsys/71030"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17914-3">
    <skos:broader rdf:resource="descriptor/17931-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17914-3">
    <skos:broader rdf:resource="descriptor/19170-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14427-1">
    <skos:prefLabel xml:lang="en">Magnetic levitation train</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14427-1">
    <skos:prefLabel xml:lang="de">Magnetschwebebahn</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18348-5">
    <skos:prefLabel xml:lang="en">Energy research</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18348-5">
    <skos:prefLabel xml:lang="de">Energieforschung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12244-5">
    <skos:broader rdf:resource="thsys/70183"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18447-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11444-2">
    <skos:altLabel xml:lang="de">Currency-Schule</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19582-6">
    <skos:altLabel xml:lang="en">Error-correction model</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19582-6">
    <skos:altLabel xml:lang="en">ECM (Error-correction model)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12736-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19514-2">
    <skos:broader rdf:resource="thsys/70832"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19514-2">
    <skos:broader rdf:resource="thsys/70458"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19514-2">
    <skos:broader rdf:resource="descriptor/16539-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16158-6">
    <skos:altLabel xml:lang="de">Bundessozialhilfegesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13389-2">
    <skos:broader rdf:resource="thsys/70875"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13389-2">
    <skos:broader rdf:resource="thsys/70154"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12407-3">
    <skos:broader rdf:resource="thsys/71043"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12407-3">
    <skos:broader rdf:resource="descriptor/10011-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12407-3">
    <skos:broader rdf:resource="descriptor/12408-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70012">
    <rdfs:label xml:lang="en">Sociology</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70012">
    <rdfs:label xml:lang="de">Soziologie</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19819-1">
    <skos:related rdf:resource="descriptor/16188-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19819-1">
    <skos:related rdf:resource="descriptor/13464-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19819-1">
    <skos:related rdf:resource="descriptor/12787-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19819-1">
    <skos:related rdf:resource="descriptor/19820-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13431-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18424-1">
    <skos:broader rdf:resource="thsys/70799"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18424-1">
    <skos:broader rdf:resource="descriptor/13076-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18424-1">
    <skos:broader rdf:resource="descriptor/18762-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14217-5">
    <skos:broader rdf:resource="thsys/70074"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14217-5">
    <skos:broader rdf:resource="descriptor/14602-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14217-5">
    <skos:broader rdf:resource="descriptor/14044-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10996-3">
    <skos:altLabel xml:lang="en">Labor regulation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10996-3">
    <skos:altLabel xml:lang="de">Arbeitskräftepolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12962-3">
    <skos:related rdf:resource="descriptor/19630-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12962-3">
    <skos:related rdf:resource="descriptor/14170-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12962-3">
    <skos:related rdf:resource="descriptor/12955-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12962-3">
    <skos:related rdf:resource="descriptor/15675-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12962-3">
    <skos:related rdf:resource="descriptor/12961-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71116">
    <skos:prefLabel xml:lang="en">V.02.07  Time in economics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71116">
    <skos:prefLabel xml:lang="de">V.02.07  Zeitökonomik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15376-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16664-5">
    <skos:related rdf:resource="descriptor/16597-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16664-5">
    <skos:related rdf:resource="descriptor/15717-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16664-5">
    <skos:related rdf:resource="descriptor/15716-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16664-5">
    <skos:related rdf:resource="descriptor/19758-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16191-1">
    <skos:altLabel xml:lang="de">Wirtschaftsspionage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16191-1">
    <skos:altLabel xml:lang="de">Werkspionage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16191-1">
    <skos:altLabel xml:lang="de">Betriebsspionage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18180-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10951-4">
    <skos:altLabel xml:lang="de">Ruinöser Wettbewerb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10951-4">
    <skos:altLabel xml:lang="de">Räuberische Preisunterbietung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13507-1">
    <skos:altLabel xml:lang="de">Weltraumfahrt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13289-6">
    <skos:altLabel xml:lang="de">Girozentrale</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11702-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18884-2">
    <skos:narrower rdf:resource="descriptor/19559-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18884-2">
    <skos:narrower rdf:resource="descriptor/18874-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18884-2">
    <skos:narrower rdf:resource="descriptor/11406-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19731-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18781-5">
    <skos:related rdf:resource="descriptor/18878-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16338-4">
    <skos:altLabel xml:lang="de">Legislative</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19411-5">
    <skos:prefLabel xml:lang="en">Tax capitalization</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19411-5">
    <skos:prefLabel xml:lang="de">Steuerkapitalisierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19670-2">
    <skos:scopeNote xml:lang="de">Materielle Verluste durch z.B. Naturkatastrophen oder andere Zerstörungen; aber auch Sozialproduktausfälle aufgrund von Fehlallokationen, Ineffizienzen, etc.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71088">
    <skos:prefLabel xml:lang="en">V.07.07.01  Foreign exchange market</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71088">
    <skos:prefLabel xml:lang="de">V.07.07.01  Devisenmarkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72394">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.16.06</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18934-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18891-5">
    <skos:altLabel xml:lang="de">Medizinische Versorgung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18891-5">
    <skos:altLabel xml:lang="de">Medizinische Infrastruktur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14642-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15001-2">
    <skos:altLabel xml:lang="de">Wasserstoffbombe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15001-2">
    <skos:altLabel xml:lang="de">Nuklearwaffe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15001-2">
    <skos:altLabel xml:lang="de">Neutronenbombe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15001-2">
    <skos:altLabel xml:lang="de">Kernwaffe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15001-2">
    <skos:altLabel xml:lang="de">A-Waffe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70880">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19842-6">
    <skos:altLabel xml:lang="de">Zwangsverwaltung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19842-6">
    <skos:altLabel xml:lang="de">Zwangsversteigerung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19842-6">
    <skos:altLabel xml:lang="de">Vollstreckungsschutz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19842-6">
    <skos:altLabel xml:lang="de">Gesamtvollstreckungsordnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10523-4">
    <skos:prefLabel xml:lang="en">Development planning</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10523-4">
    <skos:prefLabel xml:lang="de">Entwicklungsplanung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10944-1">
    <skos:prefLabel xml:lang="en">Workable competition</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10944-1">
    <skos:prefLabel xml:lang="de">Funktionsfähiger Wettbewerb</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24784-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70358">
    <skos:broader rdf:resource="thsys/p"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15105-4">
    <skos:prefLabel xml:lang="en">Production statistics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15105-4">
    <skos:prefLabel xml:lang="de">Produktionsstatistik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11641-0">
    <skos:related rdf:resource="descriptor/11707-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11641-0">
    <skos:related rdf:resource="descriptor/11635-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10779-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id159">
    <rdfs:seeAlso rdf:resource="descriptor/12224-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11716-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17675-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20536-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16142-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12295-2">
    <skos:narrower rdf:resource="descriptor/12487-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18273-3">
    <skos:related rdf:resource="descriptor/18307-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10055-4">
    <skos:broader rdf:resource="thsys/70049"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10055-4">
    <skos:broader rdf:resource="descriptor/10201-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10055-4">
    <skos:broader rdf:resource="descriptor/10036-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70884">
    <skos:prefLabel xml:lang="de">W.17.02  Vermietung und Verpachtung von Immobilien</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11847-1">
    <skos:altLabel xml:lang="de">Wohnpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19548-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15777-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73362">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18782-3">
    <skos:prefLabel xml:lang="en">University research</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18782-3">
    <skos:prefLabel xml:lang="de">Hochschulforschung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12644-3">
    <skos:prefLabel xml:lang="en">Material flow</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12644-3">
    <skos:prefLabel xml:lang="de">Materialfluss</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11039-4">
    <skos:narrower rdf:resource="descriptor/10114-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11039-4">
    <skos:narrower rdf:resource="descriptor/10328-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11039-4">
    <skos:narrower rdf:resource="descriptor/19827-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11039-4">
    <skos:narrower rdf:resource="descriptor/12168-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11039-4">
    <skos:narrower rdf:resource="descriptor/19208-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15825-5">
    <skos:related rdf:resource="descriptor/12585-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15825-5">
    <skos:related rdf:resource="descriptor/16252-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15825-5">
    <skos:related rdf:resource="descriptor/11164-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15825-5">
    <skos:related rdf:resource="descriptor/12103-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15825-5">
    <skos:related rdf:resource="descriptor/15823-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15825-5">
    <skos:related rdf:resource="descriptor/11302-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15825-5">
    <skos:related rdf:resource="descriptor/16658-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15825-5">
    <skos:related rdf:resource="descriptor/19753-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15825-5">
    <skos:related rdf:resource="descriptor/12617-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10082-1">
    <skos:scopeNote xml:lang="de">Nur für theoretische Ansätze zur Rationierung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18372-1">
    <skos:broader rdf:resource="thsys/73390"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18372-1">
    <skos:broader rdf:resource="thsys/71006"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18372-1">
    <skos:broader rdf:resource="thsys/70811"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18372-1">
    <skos:broader rdf:resource="descriptor/18805-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18372-1">
    <skos:broader rdf:resource="descriptor/18352-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13775-4">
    <skos:altLabel xml:lang="de">Grundschuld</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13775-4">
    <skos:altLabel xml:lang="de">Grundpfandrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13775-4">
    <skos:altLabel xml:lang="de">Grundpfandkredit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19654-0">
    <skos:prefLabel xml:lang="en">Data warehouse</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19654-0">
    <skos:prefLabel xml:lang="de">Data-Warehouse-System</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18681-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13001-5">
    <skos:related rdf:resource="descriptor/14218-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13001-5">
    <skos:related rdf:resource="descriptor/15651-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11149-4">
    <skos:broader rdf:resource="thsys/71101"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11149-4">
    <skos:broader rdf:resource="thsys/71024"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11149-4">
    <skos:broader rdf:resource="descriptor/11142-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11149-4">
    <skos:broader rdf:resource="descriptor/18313-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15814-3">
    <skos:prefLabel xml:lang="en">Sociometry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15814-3">
    <skos:prefLabel xml:lang="de">Soziometrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16396-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13224-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19054-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70961">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.10.01.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15892-4">
    <skos:prefLabel xml:lang="en">Lake</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15892-4">
    <skos:prefLabel xml:lang="de">See</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10411-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25681-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13102-6">
    <skos:broader rdf:resource="thsys/70990"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13102-6">
    <skos:broader rdf:resource="descriptor/13117-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16322-5">
    <skos:related rdf:resource="descriptor/18808-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16322-5">
    <skos:related rdf:resource="descriptor/16421-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16322-5">
    <skos:related rdf:resource="descriptor/13409-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16322-5">
    <skos:related rdf:resource="descriptor/16297-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18840-1">
    <skos:prefLabel xml:lang="en">Cooperative organization</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18840-1">
    <skos:prefLabel xml:lang="de">Genossenschaftsverband</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70134">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">P.07.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19545-5">
    <skos:altLabel xml:lang="de">Milchwirtschaftspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19545-5">
    <skos:altLabel xml:lang="de">Milchquote</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19545-5">
    <skos:altLabel xml:lang="de">Milchkontingentierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11235-4">
    <skos:prefLabel xml:lang="en">Labour market policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11235-4">
    <skos:prefLabel xml:lang="de">Arbeitsmarktpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70996">
    <skos:broader rdf:resource="thsys/71004"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15266-1">
    <skos:altLabel xml:lang="en">GMM (Generalized Method of Moments)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15266-1">
    <skos:altLabel xml:lang="en">Generalized method of moments</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18668-1">
    <skos:related rdf:resource="descriptor/24740-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10877-4">
    <skos:prefLabel xml:lang="en">Countertrade</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10877-4">
    <skos:prefLabel xml:lang="de">Kompensationsgeschäft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20368-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71099">
    <rdfs:label xml:lang="en">International monetary theory</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71099">
    <rdfs:label xml:lang="de">Monetäre Außenwirtschaftstheorie</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10361-4">
    <skos:altLabel xml:lang="de">Empirische Konjunkturforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14833-4">
    <skos:narrower rdf:resource="descriptor/14834-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13563-5">
    <skos:editorialNote xml:lang="de">Möglichst spezifischer indexieren.</skos:editorialNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10294-0">
    <skos:altLabel xml:lang="de">Volkswirtschaftliche Investitionstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10294-0">
    <skos:altLabel xml:lang="de">Makroökonomische Investitionstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10294-0">
    <skos:altLabel xml:lang="de">Investitionstheorie der Gesamtwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15225-1">
    <skos:narrower rdf:resource="descriptor/19577-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15225-1">
    <skos:narrower rdf:resource="descriptor/19572-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15225-1">
    <skos:narrower rdf:resource="descriptor/19571-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15225-1">
    <skos:narrower rdf:resource="descriptor/15220-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15225-1">
    <skos:narrower rdf:resource="descriptor/15327-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16597-1">
    <skos:related rdf:resource="descriptor/19611-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16597-1">
    <skos:related rdf:resource="descriptor/16664-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16597-1">
    <skos:related rdf:resource="descriptor/15719-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16597-1">
    <skos:related rdf:resource="descriptor/15718-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16597-1">
    <skos:related rdf:resource="descriptor/15717-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16597-1">
    <skos:related rdf:resource="descriptor/15716-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16597-1">
    <skos:related rdf:resource="descriptor/16602-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16597-1">
    <skos:related rdf:resource="descriptor/16588-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16597-1">
    <skos:related rdf:resource="descriptor/15914-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12577-6">
    <zbwext:useInsteadNote rdf:nodeID="id53"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18153-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18116-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11160-2">
    <skos:scopeNote xml:lang="de">Keine einheitliche Angrenzung zu Institutionalismus.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14673-0">
    <skos:related rdf:resource="descriptor/19016-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14673-0">
    <skos:related rdf:resource="descriptor/14626-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14435-2">
    <skos:narrower rdf:resource="descriptor/14354-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14435-2">
    <skos:narrower rdf:resource="descriptor/14485-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14435-2">
    <skos:narrower rdf:resource="descriptor/14518-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12158-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14969-0">
    <skos:related rdf:resource="descriptor/19383-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14969-0">
    <skos:related rdf:resource="descriptor/14158-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13294-6">
    <skos:related rdf:resource="descriptor/15949-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#exactMatch">
    <rdf:type rdf:resource="http://www.w3.org/1999/02/22-rdf-syntax-ns#Property"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#exactMatch">
    <rdf:type rdf:resource="http://www.w3.org/2002/07/owl#TransitiveProperty"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#exactMatch">
    <rdf:type rdf:resource="http://www.w3.org/2002/07/owl#SymmetricProperty"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#exactMatch">
    <rdf:type rdf:resource="http://www.w3.org/2002/07/owl#ObjectProperty"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16207-5">
    <skos:related rdf:resource="descriptor/16642-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16207-5">
    <skos:related rdf:resource="descriptor/16430-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16207-5">
    <skos:related rdf:resource="descriptor/16275-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16207-5">
    <skos:related rdf:resource="descriptor/19975-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16207-5">
    <skos:related rdf:resource="descriptor/20855-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16207-5">
    <skos:related rdf:resource="descriptor/16633-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70982">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15677-1">
    <skos:scopeNote xml:lang="de">Gegenstand der Verfahrenstechnik sind Entwurf, Projektierung und Betrieb von Verfahren und Anlagen zur Stoffwandlung, die u.a. in der chemischen Industrie, der Hüttenindustrie, der Energiewirtschaft und der Nahrungsmittelverarbeitung eingesetzt werden.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16424-4">
    <skos:broader rdf:resource="thsys/73383"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16424-4">
    <skos:broader rdf:resource="descriptor/16416-3"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id101">
    <rdfs:seeAlso rdf:resource="descriptor/19321-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#Collection">
    <rdfs:isDefinedBy rdf:resource="http://www.w3.org/2004/02/skos/core"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14703-3">
    <skos:related rdf:resource="descriptor/14149-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16621-2">
    <skos:related rdf:resource="descriptor/16013-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16621-2">
    <skos:related rdf:resource="descriptor/15719-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16621-2">
    <skos:related rdf:resource="descriptor/19013-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16621-2">
    <skos:related rdf:resource="descriptor/15787-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16621-2">
    <skos:related rdf:resource="descriptor/11164-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16621-2">
    <skos:related rdf:resource="descriptor/16195-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11474-0">
    <skos:broader rdf:resource="thsys/70306"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11474-0">
    <skos:broader rdf:resource="descriptor/10263-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16004-5">
    <skos:broader rdf:resource="thsys/73360"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16004-5">
    <skos:broader rdf:resource="thsys/70986"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16004-5">
    <skos:broader rdf:resource="descriptor/16019-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12781-0">
    <skos:prefLabel xml:lang="en">Branch facilities</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12781-0">
    <skos:prefLabel xml:lang="de">Filiale</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11934-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11880-3">
    <skos:related rdf:resource="descriptor/11903-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11880-3">
    <skos:related rdf:resource="descriptor/11987-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11880-3">
    <skos:related rdf:resource="descriptor/11879-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17665-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19671-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18545-3">
    <skos:prefLabel xml:lang="en">Retail trade policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18545-3">
    <skos:prefLabel xml:lang="de">Einzelhandelspolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16182-2">
    <skos:prefLabel xml:lang="en">Community law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16182-2">
    <skos:prefLabel xml:lang="de">EU-Recht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14518-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19477-3">
    <skos:altLabel xml:lang="en">Urban population growth</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73342">
    <rdfs:label xml:lang="de">Informationstechnik und Informatik</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10201-5">
    <skos:prefLabel xml:lang="en">Market mechanism</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10201-5">
    <skos:prefLabel xml:lang="de">Marktmechanismus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19031-1">
    <skos:altLabel xml:lang="de">Typologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19031-1">
    <skos:altLabel xml:lang="de">Systematik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11322-2">
    <zbwext:useInsteadNote rdf:nodeID="id197"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10895-2">
    <skos:narrower rdf:resource="descriptor/13580-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10895-2">
    <skos:narrower rdf:resource="descriptor/10951-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10895-2">
    <skos:narrower rdf:resource="descriptor/10910-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10895-2">
    <skos:narrower rdf:resource="descriptor/18160-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10895-2">
    <skos:narrower rdf:resource="descriptor/19429-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10895-2">
    <skos:narrower rdf:resource="descriptor/10909-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10895-2">
    <skos:narrower rdf:resource="descriptor/10908-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10895-2">
    <skos:narrower rdf:resource="descriptor/10842-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10895-2">
    <skos:narrower rdf:resource="descriptor/10945-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10895-2">
    <skos:narrower rdf:resource="descriptor/10947-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15771-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10091-0">
    <skos:related rdf:resource="descriptor/11065-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10091-0">
    <skos:related rdf:resource="descriptor/11040-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12889-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18936-2">
    <skos:related rdf:resource="descriptor/18098-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18936-2">
    <skos:related rdf:resource="descriptor/13655-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18936-2">
    <skos:related rdf:resource="descriptor/18935-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10007-1">
    <skos:altLabel xml:lang="en">Manufactured good</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10007-1">
    <skos:altLabel xml:lang="en">Industrial good</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10007-1">
    <skos:altLabel xml:lang="de">Industrielles Erzeugnis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10007-1">
    <skos:altLabel xml:lang="de">Industriegut</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13399-6">
    <skos:altLabel xml:lang="de">Sekte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13399-6">
    <skos:altLabel xml:lang="de">Religiöse Vereinigung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13399-6">
    <skos:altLabel xml:lang="de">Jugendsekte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13399-6">
    <skos:altLabel xml:lang="de">Jugendreligion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13399-6">
    <skos:altLabel xml:lang="de">Glaubensgemeinschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12137-6">
    <skos:related rdf:resource="descriptor/12138-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12137-6">
    <skos:related rdf:resource="descriptor/20977-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14337-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16036-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14895-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13769-6">
    <skos:altLabel xml:lang="de">Effektengeschäft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15727-5">
    <skos:altLabel xml:lang="de">Verklappung von Abfall</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15727-5">
    <skos:altLabel xml:lang="de">Abfalltechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19749-3">
    <skos:altLabel xml:lang="en">Awards</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14851-2">
    <skos:broader rdf:resource="thsys/73304"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14851-2">
    <skos:broader rdf:resource="descriptor/14849-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12592-3">
    <skos:altLabel xml:lang="en">Employee benefit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12592-3">
    <skos:altLabel xml:lang="de">Sozialpolitik des Betriebs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12592-3">
    <skos:altLabel xml:lang="de">Sozialleistungsmanagement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12592-3">
    <skos:altLabel xml:lang="de">Sozialleistungen des Betriebs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12592-3">
    <skos:altLabel xml:lang="de">Freiwillige Sozialleistungen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12592-3">
    <skos:altLabel xml:lang="de">Betriebliche Sozialpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12592-3">
    <skos:altLabel xml:lang="de">Betriebliche Sozialleistung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18534-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19507-6">
    <skos:related rdf:resource="descriptor/19508-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19507-6">
    <skos:related rdf:resource="descriptor/16619-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19507-6">
    <skos:related rdf:resource="descriptor/11569-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19507-6">
    <skos:related rdf:resource="descriptor/16663-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19507-6">
    <skos:related rdf:resource="descriptor/16548-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13357-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70862">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18177-6">
    <skos:prefLabel xml:lang="en">Islamic banking</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18177-6">
    <skos:prefLabel xml:lang="de">Islamische Bank</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14408-5">
    <skos:broader rdf:resource="thsys/70549"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14408-5">
    <skos:broader rdf:resource="thsys/70135"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14408-5">
    <skos:broader rdf:resource="descriptor/14337-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14408-5">
    <skos:broader rdf:resource="descriptor/14404-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13360-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10930-5">
    <skos:related rdf:resource="descriptor/11579-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19130-6">
    <skos:related rdf:resource="descriptor/17806-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16400-4">
    <skos:altLabel xml:lang="de">Antizionismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12166-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15520-6">
    <skos:related rdf:resource="descriptor/13588-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15520-6">
    <skos:related rdf:resource="descriptor/15518-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15520-6">
    <skos:related rdf:resource="descriptor/15488-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11628-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12995-2">
    <skos:scopeNote xml:lang="de">Chemische Grundstoffe sind die organischen und anorganischen Ausgangsstoffe</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14538-6">
    <skos:related rdf:resource="descriptor/14046-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14839-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70270">
    <skos:narrower rdf:resource="descriptor/12509-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70270">
    <skos:narrower rdf:resource="descriptor/12508-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70270">
    <skos:narrower rdf:resource="descriptor/19101-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70270">
    <skos:narrower rdf:resource="descriptor/10326-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70270">
    <skos:narrower rdf:resource="descriptor/12742-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70270">
    <skos:narrower rdf:resource="descriptor/10187-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70270">
    <skos:narrower rdf:resource="descriptor/10243-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70270">
    <skos:narrower rdf:resource="descriptor/20881-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70270">
    <skos:narrower rdf:resource="descriptor/12491-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70270">
    <skos:narrower rdf:resource="descriptor/10312-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70270">
    <skos:narrower rdf:resource="descriptor/12744-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70270">
    <skos:narrower rdf:resource="descriptor/12741-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17680-2">
    <skos:prefLabel xml:lang="en">Botswana</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17680-2">
    <skos:prefLabel xml:lang="de">Botsuana</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70947">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.10.05</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id105">
    <rdfs:seeAlso rdf:resource="descriptor/18916-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14488-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11713-1">
    <skos:related rdf:resource="descriptor/11692-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11713-1">
    <skos:related rdf:resource="descriptor/11907-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11713-1">
    <skos:related rdf:resource="descriptor/11735-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72184">
    <skos:broader rdf:resource="thsys/72183"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10207-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11599-1">
    <skos:altLabel xml:lang="en">Tax sharing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11599-1">
    <skos:altLabel xml:lang="en">Intergovernmental transfer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11599-1">
    <skos:altLabel xml:lang="en">Intergovernmental grant</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11599-1">
    <skos:altLabel xml:lang="en">Intergovernmental fiscal transfer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11599-1">
    <skos:altLabel xml:lang="de">Zweckzuweisung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11599-1">
    <skos:altLabel xml:lang="de">Steuerverbund</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11599-1">
    <skos:altLabel xml:lang="de">Schlüsselzuweisung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11599-1">
    <skos:altLabel xml:lang="de">Finanzzuweisung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19331-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11556-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70066">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12810-5">
    <skos:related rdf:resource="descriptor/12805-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12810-5">
    <skos:related rdf:resource="descriptor/10202-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12810-5">
    <skos:related rdf:resource="descriptor/10843-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12810-5">
    <skos:related rdf:resource="descriptor/10546-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18376-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17567-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13984-2">
    <skos:altLabel xml:lang="de">Volksvertreterinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13984-2">
    <skos:altLabel xml:lang="de">Volksvertreter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13984-2">
    <skos:altLabel xml:lang="de">Parlamentssitz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13984-2">
    <skos:altLabel xml:lang="de">Parlamentsmitglieder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13984-2">
    <skos:altLabel xml:lang="de">Parlamentsmandat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13984-2">
    <skos:altLabel xml:lang="de">Parlamentsabgeordnete</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13984-2">
    <skos:altLabel xml:lang="de">Parlamentarierinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13984-2">
    <skos:altLabel xml:lang="de">Parlamentarier</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19448-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18841-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10621-4">
    <skos:narrower rdf:resource="descriptor/16048-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18943-5">
    <skos:scopeNote xml:lang="de">Markt für das Gut "Information" und für Informationsdienstleistungen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20506-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18916-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19647-4">
    <skos:prefLabel xml:lang="en">Internal out-migration</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19647-4">
    <skos:prefLabel xml:lang="de">Abwanderung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11598-3">
    <skos:prefLabel xml:lang="en">Zero-base budgeting</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11598-3">
    <skos:prefLabel xml:lang="de">Zero-Base Budgeting</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70539">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.12.02.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10018-3">
    <skos:prefLabel xml:lang="en">Hidden economy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10018-3">
    <skos:prefLabel xml:lang="de">Schattenwirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70188">
    <skos:prefLabel xml:lang="en">V.01.03  Equilibrium model</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70188">
    <skos:prefLabel xml:lang="de">V.01.03  Gleichgewichtstheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70965">
    <skos:broader rdf:resource="thsys/w"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12955-0">
    <skos:scopeNote xml:lang="de">Nur für gewerbliche Tierhaltung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18934-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11913-0">
    <skos:prefLabel xml:lang="en">Location</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11913-0">
    <skos:prefLabel xml:lang="de">Standort</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18448-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19194-1">
    <skos:broader rdf:resource="thsys/71028"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14741-2">
    <skos:prefLabel xml:lang="en">Detergents</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14741-2">
    <skos:prefLabel xml:lang="de">Waschmittel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19839-2">
    <skos:altLabel xml:lang="en">Child support</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19839-2">
    <skos:altLabel xml:lang="de">Verwandtenunterhalt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19839-2">
    <skos:altLabel xml:lang="de">Unterhaltszahlung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19839-2">
    <skos:altLabel xml:lang="de">Unterhaltsrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19839-2">
    <skos:altLabel xml:lang="de">Unterhaltsanspruch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19839-2">
    <skos:altLabel xml:lang="de">Kindesunterhalt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19839-2">
    <skos:altLabel xml:lang="de">Familienunterhalt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19839-2">
    <skos:altLabel xml:lang="de">Elternunterhalt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19839-2">
    <skos:altLabel xml:lang="de">Alimente</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19381-2">
    <skos:altLabel xml:lang="de">Kriegssteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19381-2">
    <skos:altLabel xml:lang="de">Kriegsanleihe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19381-2">
    <skos:altLabel xml:lang="de">Kriegsabgabe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14833-4">
    <skos:prefLabel xml:lang="en">Footwear</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14833-4">
    <skos:prefLabel xml:lang="de">Schuhe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14270-1">
    <skos:related rdf:resource="descriptor/14323-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71054">
    <skos:narrower rdf:resource="descriptor/13993-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71054">
    <skos:narrower rdf:resource="descriptor/13282-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71054">
    <skos:narrower rdf:resource="descriptor/13969-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71054">
    <skos:narrower rdf:resource="descriptor/13475-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71054">
    <skos:narrower rdf:resource="descriptor/13989-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71054">
    <skos:narrower rdf:resource="descriptor/12125-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71054">
    <skos:narrower rdf:resource="descriptor/11559-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71054">
    <skos:narrower rdf:resource="descriptor/18977-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71054">
    <skos:narrower rdf:resource="descriptor/18764-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71054">
    <skos:narrower rdf:resource="descriptor/13466-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71054">
    <skos:narrower rdf:resource="descriptor/13970-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71054">
    <skos:narrower rdf:resource="descriptor/13988-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71054">
    <skos:narrower rdf:resource="descriptor/13987-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71054">
    <skos:narrower rdf:resource="descriptor/13985-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71054">
    <skos:narrower rdf:resource="descriptor/11311-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71054">
    <skos:narrower rdf:resource="descriptor/13965-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71054">
    <skos:narrower rdf:resource="descriptor/10856-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71054">
    <skos:narrower rdf:resource="descriptor/13984-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17374-2">
    <skos:broader rdf:resource="thsys/70557"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17374-2">
    <skos:broader rdf:resource="descriptor/17199-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17374-2">
    <skos:broader rdf:resource="descriptor/17991-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17374-2">
    <skos:broader rdf:resource="descriptor/19178-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17374-2">
    <skos:broader rdf:resource="descriptor/19177-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17374-2">
    <skos:broader rdf:resource="descriptor/17983-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13992-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18996-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11161-0">
    <skos:prefLabel xml:lang="en">Institutional economics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11161-0">
    <skos:prefLabel xml:lang="de">Institutionalismus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15470-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13245-5">
    <skos:broader rdf:resource="thsys/70940"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13245-5">
    <skos:broader rdf:resource="descriptor/13621-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19019-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18304-4">
    <skos:altLabel xml:lang="de">Nahrungsmittelverpackung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10327-4">
    <skos:broader rdf:resource="thsys/70233"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10327-4">
    <skos:broader rdf:resource="descriptor/10256-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19180-5">
    <skos:broader rdf:resource="thsys/70307"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19180-5">
    <skos:broader rdf:resource="descriptor/10492-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17672-1">
    <skos:altLabel xml:lang="de">Ostafrikanische Staaten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17672-1">
    <skos:altLabel xml:lang="de">Ostafrikanische Länder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16426-0">
    <skos:broader rdf:resource="thsys/73394"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16426-0">
    <skos:broader rdf:resource="thsys/73377"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id192">
    <rdfs:seeAlso rdf:resource="descriptor/11052-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19636-2">
    <skos:scopeNote xml:lang="de">Ständische Gliederungen der Handwerker, Kaufleute und anderer Berufsgruppen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11769-2">
    <skos:related rdf:resource="descriptor/11341-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11769-2">
    <skos:related rdf:resource="descriptor/19279-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11769-2">
    <skos:related rdf:resource="descriptor/11772-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11769-2">
    <skos:related rdf:resource="descriptor/11036-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19481-5">
    <skos:related rdf:resource="descriptor/19482-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19481-5">
    <skos:related rdf:resource="descriptor/18069-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19481-5">
    <skos:related rdf:resource="descriptor/15880-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19481-5">
    <skos:related rdf:resource="descriptor/20967-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19481-5">
    <skos:related rdf:resource="descriptor/20888-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19481-5">
    <skos:related rdf:resource="descriptor/20744-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13846-0">
    <skos:prefLabel xml:lang="en">Housing modernization</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13846-0">
    <skos:prefLabel xml:lang="de">Altbaumodernisierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12492-0">
    <skos:related rdf:resource="descriptor/11598-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12492-0">
    <skos:related rdf:resource="descriptor/12490-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12276-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13209-2">
    <skos:prefLabel xml:lang="en">Wholesale trade</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13209-2">
    <skos:prefLabel xml:lang="de">Großhandel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18461-2">
    <skos:altLabel xml:lang="en">Industrial R&amp;amp;D</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18461-2">
    <skos:altLabel xml:lang="de">Industrieforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18461-2">
    <skos:altLabel xml:lang="de">Forschung der Industrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10987-4">
    <skos:related rdf:resource="descriptor/10062-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10987-4">
    <skos:related rdf:resource="descriptor/10991-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10987-4">
    <skos:related rdf:resource="descriptor/10988-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10987-4">
    <skos:related rdf:resource="descriptor/10025-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10987-4">
    <skos:related rdf:resource="descriptor/11572-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10987-4">
    <skos:related rdf:resource="descriptor/19399-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10987-4">
    <skos:related rdf:resource="descriptor/19198-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10987-4">
    <skos:related rdf:resource="descriptor/11004-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10987-4">
    <skos:related rdf:resource="descriptor/10437-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10987-4">
    <skos:related rdf:resource="descriptor/10560-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10987-4">
    <skos:related rdf:resource="descriptor/18810-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10987-4">
    <skos:related rdf:resource="descriptor/18138-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73352">
    <rdfs:label xml:lang="de">Klima, Wetter, Luft</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10415-0">
    <skos:narrower rdf:resource="descriptor/10527-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19335-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10975-4">
    <skos:altLabel xml:lang="en">Social decision rule</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10975-4">
    <skos:altLabel xml:lang="en">Social choice</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10975-4">
    <skos:altLabel xml:lang="en">Public economics</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10975-4">
    <skos:altLabel xml:lang="en">Nonmarket decision making</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10975-4">
    <skos:altLabel xml:lang="en">Economic theory of politics</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10975-4">
    <skos:altLabel xml:lang="en">Collective choice</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10975-4">
    <skos:altLabel xml:lang="de">Soziale Wahl</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10975-4">
    <skos:altLabel xml:lang="de">Positive Politische Theorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10975-4">
    <skos:altLabel xml:lang="de">Positive politische Ökonomie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10975-4">
    <skos:altLabel xml:lang="de">Politische Wirtschaftslehre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10975-4">
    <skos:altLabel xml:lang="de">Ökonomische Theorie der Politik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10975-4">
    <skos:altLabel xml:lang="de">NPÖ (Neue Politische Ökonomie)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10975-4">
    <skos:altLabel xml:lang="de">Nicht-Markt-Ökonomik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10975-4">
    <skos:altLabel xml:lang="de">Neue Politische Ökonomie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10975-4">
    <skos:altLabel xml:lang="de">Moderne Politische Ökonomie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10975-4">
    <skos:altLabel xml:lang="de">Mathematische Politikwissenschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10975-4">
    <skos:altLabel xml:lang="de">Markttheorie der Politik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10975-4">
    <skos:altLabel xml:lang="de">Kollektive Wahl</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10975-4">
    <skos:altLabel xml:lang="de">Gesellschaftliche Wahl</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19082-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13079-3">
    <skos:prefLabel xml:lang="en">Measurement devices industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13079-3">
    <skos:prefLabel xml:lang="de">Messtechnische Industrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10055-4">
    <skos:altLabel xml:lang="de">Nachfrageelastizität (Preis)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10055-4">
    <skos:altLabel xml:lang="de">Angebotselastizität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16809-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11547-6">
    <skos:narrower rdf:resource="descriptor/20189-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11547-6">
    <skos:narrower rdf:resource="descriptor/11683-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11547-6">
    <skos:narrower rdf:resource="descriptor/11676-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11547-6">
    <skos:narrower rdf:resource="descriptor/19416-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11547-6">
    <skos:narrower rdf:resource="descriptor/11675-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11547-6">
    <skos:narrower rdf:resource="descriptor/19407-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11547-6">
    <skos:narrower rdf:resource="descriptor/18072-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11547-6">
    <skos:narrower rdf:resource="descriptor/11668-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11547-6">
    <skos:narrower rdf:resource="descriptor/19420-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11547-6">
    <skos:narrower rdf:resource="descriptor/11665-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11547-6">
    <skos:narrower rdf:resource="descriptor/11661-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11547-6">
    <skos:narrower rdf:resource="descriptor/11657-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11547-6">
    <skos:narrower rdf:resource="descriptor/11653-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11547-6">
    <skos:narrower rdf:resource="descriptor/19414-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11547-6">
    <skos:narrower rdf:resource="descriptor/11652-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11547-6">
    <skos:narrower rdf:resource="descriptor/11648-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18174-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11724-3">
    <skos:related rdf:resource="descriptor/19212-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10007-1">
    <skos:prefLabel xml:lang="en">Industrial product</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10007-1">
    <skos:prefLabel xml:lang="de">Industrieprodukt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13081-2">
    <skos:prefLabel xml:lang="en">Optical industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13081-2">
    <skos:prefLabel xml:lang="de">Optische Industrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70026">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18840-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13309-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17923-2">
    <skos:altLabel xml:lang="de">Republik Kolumbien</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18532-5">
    <skos:related rdf:resource="descriptor/13485-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18532-5">
    <skos:related rdf:resource="descriptor/10876-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18532-5">
    <skos:related rdf:resource="descriptor/12798-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18532-5">
    <skos:related rdf:resource="descriptor/18553-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10773-2">
    <skos:broader rdf:resource="thsys/71103"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10773-2">
    <skos:broader rdf:resource="thsys/71094"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10773-2">
    <skos:broader rdf:resource="thsys/71088"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10773-2">
    <skos:broader rdf:resource="descriptor/10213-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16287-2">
    <skos:broader rdf:resource="thsys/73376"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16081-1">
    <skos:broader rdf:resource="thsys/73390"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16081-1">
    <skos:broader rdf:resource="thsys/70039"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16081-1">
    <skos:broader rdf:resource="descriptor/15966-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15902-6">
    <skos:narrower rdf:resource="descriptor/13594-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15902-6">
    <skos:narrower rdf:resource="descriptor/11906-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15902-6">
    <skos:narrower rdf:resource="descriptor/13443-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19679-5">
    <skos:broader rdf:resource="thsys/70892"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19679-5">
    <skos:broader rdf:resource="descriptor/10012-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18603-1">
    <skos:altLabel xml:lang="en">Port operation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18603-1">
    <skos:altLabel xml:lang="de">Hafenverwaltung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18603-1">
    <skos:altLabel xml:lang="de">Hafenbetrieb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17985-1">
    <skos:altLabel xml:lang="de">OPEC-Länder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17985-1">
    <skos:altLabel xml:lang="de">Erdölexportierende Staaten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14924-1">
    <skos:altLabel xml:lang="de">Säuglingsnahrung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14924-1">
    <skos:altLabel xml:lang="de">Kleinkindernahrung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19376-2">
    <skos:scopeNote xml:lang="de">Messung langfristiger intergenerativer Umverteilungswirkungen der Staatsverschuldung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13591-0">
    <skos:broader rdf:resource="thsys/70928"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13591-0">
    <skos:broader rdf:resource="descriptor/19784-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18771-1">
    <skos:related rdf:resource="descriptor/20068-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18771-1">
    <skos:related rdf:resource="descriptor/21315-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18771-1">
    <skos:related rdf:resource="descriptor/20921-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72182">
    <skos:prefLabel xml:lang="de">V.16.02  Allgemeine statistische Methoden</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17161-5">
    <skos:prefLabel xml:lang="en">Spain</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17161-5">
    <skos:prefLabel xml:lang="de">Spanien</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10979-3">
    <skos:narrower rdf:resource="descriptor/19284-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16205-2">
    <skos:prefLabel xml:lang="en">Community research</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16205-2">
    <skos:prefLabel xml:lang="de">Kommunalwissenschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17674-4">
    <skos:altLabel xml:lang="de">Republik Sambia</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17674-4">
    <skos:altLabel xml:lang="de">Nordrhodesien</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73375">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15970-3">
    <skos:scopeNote xml:lang="de">Bestrebungen und Maßnahmen zur Weiterentwicklung und Änderung der bestehenden Rechtsordnung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14469-6">
    <skos:prefLabel xml:lang="en">Electromedical device</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14469-6">
    <skos:prefLabel xml:lang="de">Elektromedizinisches Gerät</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11523-6">
    <skos:broader rdf:resource="thsys/70355"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11523-6">
    <skos:broader rdf:resource="descriptor/11506-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13141-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12589-6">
    <skos:broader rdf:resource="thsys/70244"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12589-6">
    <skos:broader rdf:resource="descriptor/12567-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12589-6">
    <skos:broader rdf:resource="descriptor/12105-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18082-3">
    <skos:broader rdf:resource="thsys/71109"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18082-3">
    <skos:broader rdf:resource="thsys/70459"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13237-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12295-2">
    <skos:prefLabel xml:lang="en">Profitability</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12295-2">
    <skos:prefLabel xml:lang="de">Rentabilität</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11036-3">
    <skos:broader rdf:resource="thsys/70235"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11036-3">
    <skos:broader rdf:resource="thsys/70171"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11036-3">
    <skos:broader rdf:resource="descriptor/11067-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11036-3">
    <skos:broader rdf:resource="descriptor/16308-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17534-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11886-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13327-3">
    <skos:altLabel xml:lang="de">Aufbereitung von Abwasser</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13327-3">
    <skos:altLabel xml:lang="de">Abwasserverwertung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13327-3">
    <skos:altLabel xml:lang="de">Abwasseraufbereitung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14439-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16292-2">
    <skos:related rdf:resource="descriptor/11068-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16292-2">
    <skos:related rdf:resource="descriptor/11063-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16292-2">
    <skos:related rdf:resource="descriptor/11127-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16292-2">
    <skos:related rdf:resource="descriptor/11049-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16292-2">
    <skos:related rdf:resource="descriptor/16233-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16292-2">
    <skos:related rdf:resource="descriptor/13420-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16292-2">
    <skos:related rdf:resource="descriptor/19607-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16292-2">
    <skos:related rdf:resource="descriptor/16209-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16292-2">
    <skos:related rdf:resource="descriptor/16273-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16292-2">
    <skos:related rdf:resource="descriptor/16258-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16292-2">
    <skos:related rdf:resource="descriptor/16602-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18301-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10439-0">
    <skos:related rdf:resource="descriptor/10414-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10439-0">
    <skos:related rdf:resource="descriptor/10434-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10439-0">
    <skos:related rdf:resource="descriptor/10387-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10439-0">
    <skos:related rdf:resource="descriptor/10438-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10439-0">
    <skos:related rdf:resource="descriptor/19251-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10439-0">
    <skos:related rdf:resource="descriptor/10437-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10439-0">
    <skos:related rdf:resource="descriptor/10377-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19466-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19668-3">
    <skos:related rdf:resource="descriptor/19285-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19668-3">
    <skos:related rdf:resource="descriptor/16295-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19668-3">
    <skos:related rdf:resource="descriptor/15073-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19668-3">
    <skos:related rdf:resource="descriptor/19683-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19668-3">
    <skos:related rdf:resource="descriptor/10816-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19668-3">
    <skos:related rdf:resource="descriptor/16510-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70386">
    <skos:prefLabel xml:lang="en">B.05.02.02  Inplant transportation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70386">
    <skos:prefLabel xml:lang="de">B.05.02.02  Betriebliches Transportwesen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18944-3">
    <zbwext:useInsteadNote rdf:nodeID="id144"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16741-6">
    <skos:prefLabel xml:lang="en">Physiocracy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16741-6">
    <skos:prefLabel xml:lang="de">Physiokratie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71067">
    <skos:prefLabel xml:lang="en">V.11.03  Location</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71067">
    <skos:prefLabel xml:lang="de">V.11.03  Standort</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19185-2">
    <skos:prefLabel xml:lang="en">Copper market</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19185-2">
    <skos:prefLabel xml:lang="de">Kupfermarkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12081-0">
    <skos:related rdf:resource="descriptor/11358-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71072">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14948-1">
    <skos:altLabel xml:lang="de">Verarbeitetes Gemüse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14948-1">
    <skos:altLabel xml:lang="de">Obstprodukt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14948-1">
    <skos:altLabel xml:lang="de">Gemüseprodukt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71010">
    <skos:prefLabel xml:lang="de">W.05  Rohstoffwirtschaft und Bergbau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12318-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14473-1">
    <skos:narrower rdf:resource="descriptor/14466-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14473-1">
    <skos:narrower rdf:resource="descriptor/14490-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14473-1">
    <skos:narrower rdf:resource="descriptor/14465-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14473-1">
    <skos:narrower rdf:resource="descriptor/14488-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14473-1">
    <skos:narrower rdf:resource="descriptor/14486-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14473-1">
    <skos:narrower rdf:resource="descriptor/14485-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14473-1">
    <skos:narrower rdf:resource="descriptor/14483-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14473-1">
    <skos:narrower rdf:resource="descriptor/14479-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14473-1">
    <skos:narrower rdf:resource="descriptor/18103-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14473-1">
    <skos:narrower rdf:resource="descriptor/14469-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14473-1">
    <skos:narrower rdf:resource="descriptor/14502-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14473-1">
    <skos:narrower rdf:resource="descriptor/18250-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14473-1">
    <skos:narrower rdf:resource="descriptor/14464-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14931-4">
    <skos:altLabel xml:lang="de">Lebensmittel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18551-1">
    <skos:prefLabel xml:lang="en">Supermarket</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18551-1">
    <skos:prefLabel xml:lang="de">SB-Lebensmittelgeschäft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18456-2">
    <skos:prefLabel xml:lang="en">Industry classification</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18456-2">
    <skos:prefLabel xml:lang="de">Branchenklassifikation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16606-5">
    <skos:related rdf:resource="descriptor/19750-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16606-5">
    <skos:related rdf:resource="descriptor/16604-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16606-5">
    <skos:related rdf:resource="descriptor/16603-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16606-5">
    <skos:related rdf:resource="descriptor/16601-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15598-4">
    <skos:scopeNote xml:lang="de">Für betriebliche Informationssysteme BENUTZE Betriebliches Informationssystem. Vgl. auch Automation, Datenverarbeitung, Computergestütztes Verfahren und Datenbank.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18088-5">
    <skos:related rdf:resource="descriptor/13726-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18088-5">
    <skos:related rdf:resource="descriptor/11421-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18088-5">
    <skos:related rdf:resource="descriptor/11441-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18110-3">
    <skos:broader rdf:resource="thsys/73338"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16517-4">
    <skos:related rdf:resource="descriptor/16012-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16517-4">
    <skos:related rdf:resource="descriptor/18939-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16517-4">
    <skos:related rdf:resource="descriptor/13360-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16517-4">
    <skos:related rdf:resource="descriptor/13665-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70560">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10304-2">
    <skos:broader rdf:resource="thsys/70233"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10304-2">
    <skos:broader rdf:resource="descriptor/10313-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15017-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11567-0">
    <skos:scopeNote xml:lang="de">Subventionierung bzw. kostenlose Vergabe von Nahrungsmitteln an Bedürftige.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16679-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12732-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16621-2">
    <skos:prefLabel xml:lang="en">Violence</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16621-2">
    <skos:prefLabel xml:lang="de">Gewalt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71079">
    <skos:prefLabel xml:lang="en">V.09.05  Fiscal transfer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71079">
    <skos:prefLabel xml:lang="de">V.09.05  Finanzausgleich</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19966-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11801-4">
    <skos:broader rdf:resource="thsys/71110"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11801-4">
    <skos:broader rdf:resource="thsys/71020"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11801-4">
    <skos:broader rdf:resource="descriptor/10981-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13767-3">
    <skos:related rdf:resource="descriptor/19425-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13767-3">
    <skos:related rdf:resource="descriptor/16079-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13767-3">
    <skos:related rdf:resource="descriptor/11451-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13767-3">
    <skos:related rdf:resource="descriptor/10870-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73356">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.06.03.04</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14490-1">
    <skos:altLabel xml:lang="de">Telekommunikationsausrüstung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14490-1">
    <skos:altLabel xml:lang="de">Telegrafenausrüstung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14490-1">
    <skos:altLabel xml:lang="de">Sendeanlage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14490-1">
    <skos:altLabel xml:lang="de">Funktechnisches Gerät</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14490-1">
    <skos:altLabel xml:lang="de">Fernschreiber</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14490-1">
    <skos:altLabel xml:lang="de">Fernmeldegerät</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73338">
    <rdfs:label xml:lang="de">Informatik-Anwendungen</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70080">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13383-0">
    <skos:scopeNote xml:lang="de">Nur für Meinungsforschung als Forschungsbereich oder Wirtschaftssektor.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70148">
    <skos:broader rdf:resource="thsys/g"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14769-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70403">
    <rdfs:label xml:lang="de">Tiere</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19078-3">
    <skos:altLabel xml:lang="en">Commonwealth of Independent States</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19078-3">
    <skos:altLabel xml:lang="en">CIS (Commonwealth of Independent States)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19078-3">
    <skos:altLabel xml:lang="de">Sowjetunion (Nachfolgestaaten)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19078-3">
    <skos:altLabel xml:lang="de">Nachfolgestaaten (Sowjetunion)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19078-3">
    <skos:altLabel xml:lang="de">GUS-Länder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19078-3">
    <skos:altLabel xml:lang="de">Gemeinschaft Unabhängiger Staaten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15895-5">
    <skos:prefLabel xml:lang="en">Tropics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15895-5">
    <skos:prefLabel xml:lang="de">Tropen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19836-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14421-6">
    <skos:broader rdf:resource="thsys/70256"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14421-6">
    <skos:broader rdf:resource="descriptor/13513-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18311-0">
    <skos:related rdf:resource="descriptor/12961-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18311-0">
    <skos:related rdf:resource="descriptor/18338-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18311-0">
    <skos:related rdf:resource="descriptor/13426-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11510-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15731-0">
    <skos:related rdf:resource="descriptor/13179-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15731-0">
    <skos:related rdf:resource="descriptor/15727-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15731-0">
    <skos:related rdf:resource="descriptor/13329-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15731-0">
    <skos:related rdf:resource="descriptor/15729-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16551-4">
    <skos:related rdf:resource="descriptor/13421-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16551-4">
    <skos:related rdf:resource="descriptor/16324-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16551-4">
    <skos:related rdf:resource="descriptor/18809-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16551-4">
    <skos:related rdf:resource="descriptor/16509-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16551-4">
    <skos:related rdf:resource="descriptor/13639-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19084-1">
    <skos:narrower rdf:resource="descriptor/17181-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19084-1">
    <skos:narrower rdf:resource="descriptor/16828-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19084-1">
    <skos:narrower rdf:resource="descriptor/18224-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19084-1">
    <skos:narrower rdf:resource="descriptor/18225-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19084-1">
    <skos:narrower rdf:resource="descriptor/16826-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12181-3">
    <skos:altLabel xml:lang="de">BGB-Gesellschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13593-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18770-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14057-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10099-5">
    <skos:altLabel xml:lang="en">Optimal redistribution</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10099-5">
    <skos:altLabel xml:lang="de">Pareto-optimale Verteilung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10099-5">
    <skos:altLabel xml:lang="de">Optimale Umverteilung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19011-0">
    <skos:scopeNote xml:lang="de">Lohnunterschiede nach formalen Merkmalen; vor allem nach Alter, Geschlecht oder Rasse.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19285-5">
    <skos:broader rdf:resource="thsys/70039"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13096-3">
    <skos:related rdf:resource="descriptor/13084-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13096-3">
    <skos:related rdf:resource="descriptor/13097-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13096-3">
    <skos:related rdf:resource="descriptor/13233-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13096-3">
    <skos:related rdf:resource="descriptor/14581-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13096-3">
    <skos:related rdf:resource="descriptor/14029-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10016-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id252">
    <rdf:value xml:lang="de">Für Pflanzenschutzmittel</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20468-1">
    <skos:prefLabel xml:lang="en">Business model</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20468-1">
    <skos:prefLabel xml:lang="de">Geschäftsmodell</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18082-3">
    <skos:altLabel xml:lang="en">Innovation diffusion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18082-3">
    <skos:altLabel xml:lang="de">Diffusionsmodell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18082-3">
    <skos:altLabel xml:lang="de">Diffusionsforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18082-3">
    <skos:altLabel xml:lang="de">Diffusion (Innovation)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12005-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14548-3">
    <skos:prefLabel xml:lang="en">Ironware</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14548-3">
    <skos:prefLabel xml:lang="de">EBM-Waren</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15839-1">
    <skos:related rdf:resource="descriptor/12585-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15839-1">
    <skos:related rdf:resource="descriptor/12589-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15839-1">
    <skos:related rdf:resource="descriptor/11741-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15839-1">
    <skos:related rdf:resource="descriptor/12594-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15839-1">
    <skos:related rdf:resource="descriptor/19002-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15839-1">
    <skos:related rdf:resource="descriptor/11052-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15839-1">
    <skos:related rdf:resource="descriptor/11192-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15839-1">
    <skos:related rdf:resource="descriptor/12577-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15839-1">
    <skos:related rdf:resource="descriptor/12592-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15839-1">
    <skos:related rdf:resource="descriptor/15837-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15839-1">
    <skos:related rdf:resource="descriptor/25881-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15839-1">
    <skos:related rdf:resource="descriptor/11181-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15839-1">
    <skos:related rdf:resource="descriptor/24622-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71029">
    <skos:narrower rdf:resource="descriptor/19307-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71029">
    <skos:narrower rdf:resource="descriptor/18316-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71029">
    <skos:narrower rdf:resource="descriptor/19306-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71029">
    <skos:narrower rdf:resource="descriptor/19305-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71029">
    <skos:narrower rdf:resource="descriptor/19304-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71029">
    <skos:narrower rdf:resource="descriptor/19303-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71029">
    <skos:narrower rdf:resource="descriptor/19302-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71029">
    <skos:narrower rdf:resource="descriptor/12957-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71029">
    <skos:narrower rdf:resource="descriptor/19301-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71029">
    <skos:narrower rdf:resource="descriptor/18299-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71029">
    <skos:narrower rdf:resource="descriptor/18196-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71029">
    <skos:narrower rdf:resource="descriptor/12951-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71029">
    <skos:narrower rdf:resource="descriptor/19300-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71029">
    <skos:narrower rdf:resource="descriptor/19298-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71029">
    <skos:narrower rdf:resource="descriptor/19297-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71029">
    <skos:narrower rdf:resource="descriptor/18197-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71029">
    <skos:narrower rdf:resource="descriptor/12949-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71029">
    <skos:narrower rdf:resource="descriptor/19190-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71029">
    <skos:narrower rdf:resource="descriptor/19189-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71029">
    <skos:narrower rdf:resource="descriptor/18981-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71029">
    <skos:narrower rdf:resource="descriptor/19193-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71029">
    <skos:narrower rdf:resource="descriptor/19296-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71029">
    <skos:narrower rdf:resource="descriptor/18193-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71029">
    <skos:narrower rdf:resource="descriptor/19295-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71029">
    <skos:narrower rdf:resource="descriptor/19294-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71029">
    <skos:narrower rdf:resource="descriptor/18192-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71029">
    <skos:narrower rdf:resource="descriptor/19293-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71029">
    <skos:narrower rdf:resource="descriptor/19187-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71029">
    <skos:narrower rdf:resource="descriptor/19188-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71029">
    <skos:narrower rdf:resource="descriptor/18198-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71029">
    <skos:narrower rdf:resource="descriptor/19292-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71029">
    <skos:narrower rdf:resource="descriptor/12939-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71029">
    <skos:narrower rdf:resource="descriptor/20409-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71029">
    <skos:narrower rdf:resource="descriptor/19290-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71029">
    <skos:narrower rdf:resource="descriptor/18190-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71029">
    <skos:narrower rdf:resource="descriptor/12933-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71029">
    <skos:narrower rdf:resource="thsys/71026"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16329-5">
    <skos:broader rdf:resource="thsys/73380"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16329-5">
    <skos:broader rdf:resource="descriptor/19689-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21315-0">
    <skos:related rdf:resource="descriptor/18654-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21315-0">
    <skos:related rdf:resource="descriptor/18771-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21315-0">
    <skos:related rdf:resource="descriptor/13652-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21315-0">
    <skos:related rdf:resource="descriptor/19544-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21315-0">
    <skos:related rdf:resource="descriptor/12780-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15651-5">
    <skos:altLabel xml:lang="en">Synthetic fuels</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15651-5">
    <skos:altLabel xml:lang="en">Synfuels</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15651-5">
    <skos:altLabel xml:lang="en">Coal gasification</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15651-5">
    <skos:altLabel xml:lang="de">Kohlevergasung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15651-5">
    <skos:altLabel xml:lang="de">Kohleverflüssigung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15651-5">
    <skos:altLabel xml:lang="de">Kohleveredelung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15651-5">
    <skos:altLabel xml:lang="de">Kohlenverflüssigung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15651-5">
    <skos:altLabel xml:lang="de">Kohlehydrierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18716-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18063-0">
    <skos:altLabel xml:lang="de">Reform des Gesundheitswesens</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18063-0">
    <skos:altLabel xml:lang="de">Reform der gesetzlichen Krankenversicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18063-0">
    <skos:altLabel xml:lang="de">Krankenversicherungsreform</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18063-0">
    <skos:altLabel xml:lang="de">Gesundheitsstrukturgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72184">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.16.03.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15682-1">
    <skos:scopeNote xml:lang="de">Im Unterschied zum naturwissenschaftlichen Weltbild eine auf das Ganze der Welt und der menschlichen Existenz abzielende Sinndeutung. Weltanschauungen sind vor allem aus den verschiedenen Systemen der Philosophie entwickelt worden.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18903-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18935-4">
    <skos:altLabel xml:lang="de">TV-Station</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18935-4">
    <skos:altLabel xml:lang="de">Fernsehsender</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13407-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12699-6">
    <skos:broader rdf:resource="thsys/70030"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19761-6">
    <skos:altLabel xml:lang="de">Lebensweg</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19761-6">
    <skos:altLabel xml:lang="de">Lebensplanung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19761-6">
    <skos:altLabel xml:lang="de">Lebenslauf</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19761-6">
    <skos:altLabel xml:lang="de">Curriculum Vitae</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19761-6">
    <skos:altLabel xml:lang="de">Biographische Entwicklung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19761-6">
    <skos:altLabel xml:lang="de">Biografische Entwicklung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12274-3">
    <skos:related rdf:resource="descriptor/12254-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12274-3">
    <skos:related rdf:resource="descriptor/12251-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12274-3">
    <skos:related rdf:resource="descriptor/12244-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12274-3">
    <skos:related rdf:resource="descriptor/12241-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12274-3">
    <skos:related rdf:resource="descriptor/12273-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12274-3">
    <skos:related rdf:resource="descriptor/12234-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70095">
    <skos:prefLabel xml:lang="de">N.04  Politik und Politikwissenschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18241-2">
    <skos:altLabel xml:lang="de">Containerterminal</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70438">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11592-1">
    <skos:prefLabel xml:lang="en">State budget</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11592-1">
    <skos:prefLabel xml:lang="de">Länderhaushalt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15033-3">
    <skos:scopeNote xml:lang="de">Maß für die Ordnung/Unordnung/Verteilung in einem System.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10141-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11407-1">
    <skos:prefLabel xml:lang="en">Economics of education</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11407-1">
    <skos:prefLabel xml:lang="de">Bildungsökonomik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16366-6">
    <skos:prefLabel xml:lang="en">Party system</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16366-6">
    <skos:prefLabel xml:lang="de">Parteiensystem</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70403">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">P.01.03</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12563-3">
    <skos:prefLabel xml:lang="en">Crew scheduling</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12563-3">
    <skos:prefLabel xml:lang="de">Personaleinsatzplanung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18941-2">
    <skos:prefLabel xml:lang="en">Electronic information services</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18941-2">
    <skos:prefLabel xml:lang="de">Elektronische Informationsdienstleistung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10454-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70852">
    <skos:prefLabel xml:lang="de">W.23.02.01  Wirtschafts- und Berufsverbände</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11186-5">
    <zbwext:useInsteadNote rdf:nodeID="id27"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11186-5">
    <zbwext:useInsteadNote rdf:nodeID="id26"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id120">
    <rdfs:seeAlso rdf:resource="descriptor/19635-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19744-6">
    <skos:broader rdf:resource="thsys/73393"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19744-6">
    <skos:broader rdf:resource="descriptor/10436-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11063-0">
    <skos:altLabel xml:lang="en">Comparative politics</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11063-0">
    <skos:altLabel xml:lang="en">Comparative economic systems studies</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11063-0">
    <skos:altLabel xml:lang="en">Comparative economic systems</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11063-0">
    <skos:altLabel xml:lang="de">Wirtschaftsordnungsvergleich</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11063-0">
    <skos:altLabel xml:lang="de">Vergleich politischer Systeme</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11063-0">
    <skos:altLabel xml:lang="de">Politischer Systemvergleich</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11063-0">
    <skos:altLabel xml:lang="de">Ökonomischer Systemvergleich</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12125-6">
    <skos:prefLabel xml:lang="en">Organizers</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12125-6">
    <skos:prefLabel xml:lang="de">Organisatoren</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15905-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12922-1">
    <skos:prefLabel xml:lang="en">Resource sector</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12922-1">
    <skos:prefLabel xml:lang="de">Rohstoffwirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11928-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19100-1">
    <skos:related rdf:resource="descriptor/19101-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19100-1">
    <skos:related rdf:resource="descriptor/12473-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70354">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">G.03.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16562-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19408-1">
    <skos:narrower rdf:resource="descriptor/11705-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19408-1">
    <skos:narrower rdf:resource="descriptor/19403-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19408-1">
    <skos:narrower rdf:resource="descriptor/19409-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19408-1">
    <skos:narrower rdf:resource="descriptor/19402-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18417-5">
    <skos:altLabel xml:lang="de">Werftensubvention</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18417-5">
    <skos:altLabel xml:lang="de">Werftenpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18417-5">
    <skos:altLabel xml:lang="de">Werftenhilfe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18417-5">
    <skos:altLabel xml:lang="de">Schiffsbaupolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18417-5">
    <skos:altLabel xml:lang="de">Schiffbausubvention</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11224-2">
    <zbwext:useInsteadNote rdf:nodeID="id136"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18588-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19253-4">
    <skos:broader rdf:resource="thsys/73393"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19253-4">
    <skos:broader rdf:resource="thsys/71109"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19253-4">
    <skos:broader rdf:resource="thsys/70823"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id114">
    <rdf:value xml:lang="de">Für mikroökonomische Konsumfunktionen</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12089-5">
    <skos:prefLabel xml:lang="en">Corporate policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12089-5">
    <skos:prefLabel xml:lang="de">Unternehmenspolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18898-5">
    <skos:broader rdf:resource="thsys/73346"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18898-5">
    <skos:broader rdf:resource="descriptor/18897-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12287-1">
    <skos:prefLabel xml:lang="en">Institutional investor</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12287-1">
    <skos:prefLabel xml:lang="de">Institutioneller Anleger</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11910-6">
    <skos:prefLabel xml:lang="en">Regional economic structure</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11910-6">
    <skos:prefLabel xml:lang="de">Regionale Wirtschaftsstruktur</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14182-5">
    <skos:altLabel xml:lang="de">Nuklearenergie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14182-5">
    <skos:altLabel xml:lang="de">Kernkraft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14182-5">
    <skos:altLabel xml:lang="de">Atomkraft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14182-5">
    <skos:altLabel xml:lang="de">Atomkernenergie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14182-5">
    <skos:altLabel xml:lang="de">Atomenergie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10129-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11248-2">
    <skos:related rdf:resource="descriptor/11281-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11248-2">
    <skos:related rdf:resource="descriptor/18792-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11248-2">
    <skos:related rdf:resource="descriptor/11187-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11248-2">
    <skos:related rdf:resource="descriptor/19493-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11248-2">
    <skos:related rdf:resource="descriptor/12607-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11248-2">
    <skos:related rdf:resource="descriptor/16526-3"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id218">
    <rdfs:seeAlso rdf:resource="descriptor/12946-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18170-6">
    <skos:altLabel xml:lang="en">Electronic imaging</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18170-6">
    <skos:altLabel xml:lang="en">Digital imaging</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18170-6">
    <skos:altLabel xml:lang="en">Digital image processing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18170-6">
    <skos:altLabel xml:lang="de">Rechnergestütztes Sichtsystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18170-6">
    <skos:altLabel xml:lang="de">Optisches Sensorsystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18170-6">
    <skos:altLabel xml:lang="de">Elektronische Bildverarbeitung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18170-6">
    <skos:altLabel xml:lang="de">Digitale Bildverarbeitung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18170-6">
    <skos:altLabel xml:lang="de">Bilddigitalisierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19702-1">
    <skos:prefLabel xml:lang="en">Reelection</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19702-1">
    <skos:prefLabel xml:lang="de">Wiederwahl</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id359">
    <rdfs:seeAlso rdf:resource="descriptor/10659-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16957-4">
    <skos:broader rdf:resource="thsys/70003"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16957-4">
    <skos:broader rdf:resource="descriptor/16972-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15320-0">
    <skos:related rdf:resource="descriptor/15405-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12717-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10736-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19808-6">
    <skos:scopeNote xml:lang="de">Rechner-Netzarchitekturen, deren Struktur und Funktion sich an den Nervennetzen lebender Organismen orientiert.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10573-3">
    <skos:related rdf:resource="descriptor/10603-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15069-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18785-4">
    <skos:altLabel xml:lang="de">Innovationskosten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18785-4">
    <skos:altLabel xml:lang="de">FuE-Kosten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18785-4">
    <skos:altLabel xml:lang="de">Forschungsbudget</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18785-4">
    <skos:altLabel xml:lang="de">Forschungsausgaben</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18785-4">
    <skos:altLabel xml:lang="de">Forschungs- und Entwicklungskosten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18785-4">
    <skos:altLabel xml:lang="de">F&amp;amp;E-Kosten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18785-4">
    <skos:altLabel xml:lang="de">Entwicklungskosten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18625-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18918-4">
    <skos:altLabel xml:lang="de">Karussellbetreiber</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14318-6">
    <skos:related rdf:resource="descriptor/12693-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14318-6">
    <skos:related rdf:resource="descriptor/13615-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14318-6">
    <skos:related rdf:resource="descriptor/18621-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14318-6">
    <skos:related rdf:resource="descriptor/18242-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14318-6">
    <skos:related rdf:resource="descriptor/18241-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14318-6">
    <skos:related rdf:resource="descriptor/13050-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21101-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70060">
    <skos:prefLabel xml:lang="de">N.08  Kultur und Geisteswissenschaften</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15893-2">
    <skos:broader rdf:resource="thsys/73353"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19291-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17985-1">
    <skos:broader rdf:resource="thsys/70307"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19721-4">
    <zbwext:useInsteadNote rdf:nodeID="id179"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21011-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19633-1">
    <skos:prefLabel xml:lang="en">Environmental information system</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19633-1">
    <skos:prefLabel xml:lang="de">Umwelt-Informationssystem</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19436-3">
    <skos:prefLabel xml:lang="en">Budget theory</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19436-3">
    <skos:prefLabel xml:lang="de">Theorie des öffentlichen Haushalts</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16515-1">
    <skos:broader rdf:resource="thsys/73369"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16515-1">
    <skos:broader rdf:resource="thsys/70828"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16515-1">
    <skos:broader rdf:resource="descriptor/16509-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17922-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11282-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18800-6">
    <skos:related rdf:resource="descriptor/13249-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18800-6">
    <skos:related rdf:resource="descriptor/19463-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18800-6">
    <skos:related rdf:resource="descriptor/18345-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21282-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id315">
    <rdf:value xml:lang="de">für die wirtschaftstheoretische Untersuchung des Stellvertretungsproblems</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16526-3">
    <skos:narrower rdf:resource="descriptor/11253-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18664-2">
    <skos:related rdf:resource="descriptor/13269-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10754-6">
    <skos:related rdf:resource="descriptor/10772-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10754-6">
    <skos:related rdf:resource="descriptor/10770-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10754-6">
    <skos:related rdf:resource="descriptor/10768-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18498-0">
    <skos:altLabel xml:lang="de">Flughafenausbau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11793-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12716-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19749-3">
    <skos:scopeNote xml:lang="de">Literatur über Ehrungen und Preise aller Art, Preisträger, Verleihung, etc.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10058-5">
    <skos:broader rdf:resource="thsys/71111"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10058-5">
    <skos:broader rdf:resource="thsys/71033"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10058-5">
    <skos:broader rdf:resource="thsys/70049"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10058-5">
    <skos:broader rdf:resource="descriptor/10011-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13333-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18559-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id287">
    <rdfs:seeAlso rdf:resource="descriptor/18643-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13744-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11191-5">
    <skos:scopeNote xml:lang="de">Beendigung des Arbeitsverhältnisses durch Arbeitgeber/in oder Arbeitnehmer/in.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15837-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11572-0">
    <skos:related rdf:resource="descriptor/10987-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11572-0">
    <skos:related rdf:resource="descriptor/19829-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11572-0">
    <skos:related rdf:resource="descriptor/11702-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11572-0">
    <skos:related rdf:resource="descriptor/19310-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11572-0">
    <skos:related rdf:resource="descriptor/10438-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11572-0">
    <skos:related rdf:resource="descriptor/20906-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11572-0">
    <skos:related rdf:resource="descriptor/19941-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13790-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19021-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11238-5">
    <skos:broader rdf:resource="thsys/71061"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11238-5">
    <skos:broader rdf:resource="descriptor/10991-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11238-5">
    <skos:broader rdf:resource="descriptor/11270-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18513-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13165-3">
    <zbwext:useInsteadNote rdf:nodeID="id388"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15679-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18006-5">
    <skos:broader rdf:resource="thsys/70944"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18006-5">
    <skos:broader rdf:resource="descriptor/13306-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10511-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13514-4">
    <skos:prefLabel xml:lang="en">Transport route</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13514-4">
    <skos:prefLabel xml:lang="de">Verkehrsweg</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18557-3">
    <skos:related rdf:resource="descriptor/18559-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18557-3">
    <skos:related rdf:resource="descriptor/16078-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16187-6">
    <skos:related rdf:resource="descriptor/16188-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16187-6">
    <skos:related rdf:resource="descriptor/18591-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16187-6">
    <skos:related rdf:resource="descriptor/15744-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16187-6">
    <skos:related rdf:resource="descriptor/19631-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16187-6">
    <skos:related rdf:resource="descriptor/12987-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16187-6">
    <skos:related rdf:resource="descriptor/19616-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16187-6">
    <skos:related rdf:resource="descriptor/18567-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16187-6">
    <skos:related rdf:resource="descriptor/12872-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16187-6">
    <skos:related rdf:resource="descriptor/18312-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16187-6">
    <skos:related rdf:resource="descriptor/18352-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19636-2">
    <skos:related rdf:resource="descriptor/15706-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19636-2">
    <skos:related rdf:resource="descriptor/12910-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19636-2">
    <skos:related rdf:resource="descriptor/18850-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18739-4">
    <skos:altLabel xml:lang="en">Private pension scheme</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18739-4">
    <skos:altLabel xml:lang="de">Rente (Private Rentenversicherung)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18739-4">
    <skos:altLabel xml:lang="de">Private Altersvorsorge</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18739-4">
    <skos:altLabel xml:lang="de">Private Altersversorgung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18739-4">
    <skos:altLabel xml:lang="de">Altersrente (privat)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70498">
    <rdfs:label xml:lang="de">Nahrungs- und Genussmittel</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19826-4">
    <skos:altLabel xml:lang="de">Investitionsbarometer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70143">
    <skos:broader rdf:resource="thsys/70060"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71080">
    <rdfs:label xml:lang="en">Public Transfer</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71080">
    <rdfs:label xml:lang="de">Öffentlicher Transfer</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10176-6">
    <skos:altLabel xml:lang="en">New home economics</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11361-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13054-5">
    <skos:related rdf:resource="descriptor/13060-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13054-5">
    <skos:related rdf:resource="descriptor/14380-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13054-5">
    <skos:related rdf:resource="descriptor/18320-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13054-5">
    <skos:related rdf:resource="descriptor/13450-4"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id28">
    <rdf:value xml:lang="de">Für Architekturbüros</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14146-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11123-1">
    <skos:narrower rdf:resource="descriptor/17994-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14252-3">
    <skos:broader rdf:resource="thsys/70647"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14252-3">
    <skos:broader rdf:resource="descriptor/14769-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10225-5">
    <skos:broader rdf:resource="thsys/71117"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10225-5">
    <skos:broader rdf:resource="descriptor/10141-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14357-3">
    <skos:narrower rdf:resource="descriptor/14354-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14357-3">
    <skos:narrower rdf:resource="descriptor/14350-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19552-1">
    <skos:altLabel xml:lang="en">Linear expenditure system</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19552-1">
    <skos:altLabel xml:lang="en">LES (Linear Expenditure System)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19552-1">
    <skos:altLabel xml:lang="en">Expenditure system</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19552-1">
    <skos:altLabel xml:lang="en">Almost ideal demand system</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19552-1">
    <skos:altLabel xml:lang="en">AIDS (Almost Ideal Demand System)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19552-1">
    <skos:altLabel xml:lang="de">Ökonometrisches Nachfragesystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16683-1">
    <skos:scopeNote xml:lang="de">Untersucht vor allem soziale Ursachen und Auswirkungen von Krankheiten.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12270-4">
    <skos:narrower rdf:resource="descriptor/12391-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24786-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70259">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18028-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20491-6">
    <skos:broader rdf:resource="thsys/70017"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20491-6">
    <skos:broader rdf:resource="descriptor/15672-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11551-1">
    <skos:broader rdf:resource="thsys/70038"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11551-1">
    <skos:broader rdf:resource="descriptor/10011-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20967-4">
    <skos:broader rdf:resource="thsys/70239"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20967-4">
    <skos:broader rdf:resource="descriptor/15738-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15026-0">
    <skos:broader rdf:resource="thsys/70269"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15026-0">
    <skos:broader rdf:resource="descriptor/15045-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10211-2">
    <skos:prefLabel xml:lang="en">Peak-load pricing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10211-2">
    <skos:prefLabel xml:lang="de">Spitzenlastpreis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19160-4">
    <skos:related rdf:resource="descriptor/15775-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19160-4">
    <skos:related rdf:resource="descriptor/19090-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19160-4">
    <skos:related rdf:resource="descriptor/20125-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19160-4">
    <skos:related rdf:resource="descriptor/12891-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12225-2">
    <skos:related rdf:resource="descriptor/10121-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12225-2">
    <skos:related rdf:resource="descriptor/10120-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12225-2">
    <skos:related rdf:resource="descriptor/25742-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12225-2">
    <skos:related rdf:resource="descriptor/12228-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71042">
    <rdfs:label xml:lang="de">Erfolgsrechnung und betriebliche Kennzahlen</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12847-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14456-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15998-2">
    <skos:altLabel xml:lang="de">Versammlungsfreiheit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15998-2">
    <skos:altLabel xml:lang="de">Grundrechte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15998-2">
    <skos:altLabel xml:lang="de">Gleichheitssatz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15998-2">
    <skos:altLabel xml:lang="de">Gleichbehandlungsgrundsatz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73334">
    <skos:prefLabel xml:lang="en">N.05.01.03  Criminal law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73334">
    <skos:prefLabel xml:lang="de">N.05.01.03  Strafrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10338-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://zbw.eu/namespaces/skos-extensions/Thsys">
    <rdfs:label xml:lang="en">Thsys</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id150">
    <rdf:value xml:lang="de">Für Verbände der Wirtschaft</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13159-5">
    <skos:altLabel xml:lang="de">Windmühle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13159-5">
    <skos:altLabel xml:lang="de">Mühlenindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16972-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18546-1">
    <skos:broader rdf:resource="thsys/71067"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18546-1">
    <skos:broader rdf:resource="thsys/70947"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18546-1">
    <skos:broader rdf:resource="descriptor/11905-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11681-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15638-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70114">
    <rdfs:label xml:lang="en">Payments</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70114">
    <rdfs:label xml:lang="de">Zahlungsverkehr</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id51">
    <rdf:value xml:lang="de">Für gewerbliche Standorte aus volkswirtschaftlicher Sicht</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15635-3">
    <skos:related rdf:resource="descriptor/15638-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15635-3">
    <skos:related rdf:resource="descriptor/14722-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15635-3">
    <skos:related rdf:resource="descriptor/18260-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15635-3">
    <skos:related rdf:resource="descriptor/15633-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15635-3">
    <skos:related rdf:resource="descriptor/19625-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17800-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10487-3">
    <skos:prefLabel xml:lang="en">Appropriate technology</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10487-3">
    <skos:prefLabel xml:lang="de">Angepasste Technologie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13422-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10831-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18836-6">
    <skos:scopeNote xml:lang="de">Marketingkonzept zum geplanten sozialen Wandel, meist durch NPO, mit dem Ziel der Verhaltensänderung des angesprochenen Personenkreises, z.B. in der Gesundheitsvorsorge oder im Umweltschutz.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70582">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">A.00</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19807-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16684-6">
    <skos:scopeNote xml:lang="de">Befasst sich u.a. mit den Beziehungen zwischen Gesellschaft und Streitkräften und erörtert organisationssoziologische Fragen der Militärorganisation sowie berufssoziologische Fragen des militärischen Personals (besonders des Soldatenberufs).</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19777-5">
    <skos:prefLabel xml:lang="en">Semiotics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19777-5">
    <skos:prefLabel xml:lang="de">Semiotik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11503-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14722-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19975-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13163-0">
    <skos:broader rdf:resource="thsys/73294"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13163-0">
    <skos:broader rdf:resource="descriptor/13160-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16676-5">
    <skos:prefLabel xml:lang="en">Addiction</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16676-5">
    <skos:prefLabel xml:lang="de">Sucht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11684-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14318-6">
    <skos:altLabel xml:lang="de">Container-Transportsystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14318-6">
    <skos:altLabel xml:lang="de">Behälter-Transportsystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10025-6">
    <skos:prefLabel xml:lang="en">Economy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10025-6">
    <skos:prefLabel xml:lang="de">Volkswirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18375-2">
    <skos:related rdf:resource="descriptor/11810-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18375-2">
    <skos:related rdf:resource="descriptor/18352-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18375-2">
    <skos:related rdf:resource="descriptor/18384-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18375-2">
    <skos:related rdf:resource="descriptor/12980-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10148-4">
    <skos:narrower rdf:resource="descriptor/10184-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11238-5">
    <skos:altLabel xml:lang="de">Beschäftigungswirkung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13577-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16898-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12774-4">
    <skos:broader rdf:resource="thsys/73384"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12774-4">
    <skos:broader rdf:resource="thsys/70876"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12774-4">
    <skos:broader rdf:resource="thsys/70041"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12774-4">
    <skos:broader rdf:resource="descriptor/16554-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10220-1">
    <skos:related rdf:resource="descriptor/11024-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10220-1">
    <skos:related rdf:resource="descriptor/10058-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10220-1">
    <skos:related rdf:resource="descriptor/10052-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10220-1">
    <skos:related rdf:resource="descriptor/15493-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71078">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13560-4">
    <skos:prefLabel xml:lang="en">Road</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13560-4">
    <skos:prefLabel xml:lang="de">Straße</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11742-1">
    <skos:altLabel xml:lang="de">Konzentrationskurve des Einkommens</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19231-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16019-6">
    <skos:related rdf:resource="descriptor/16212-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16019-6">
    <skos:related rdf:resource="descriptor/16336-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16019-6">
    <skos:related rdf:resource="descriptor/16335-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16019-6">
    <skos:related rdf:resource="descriptor/13423-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16019-6">
    <skos:related rdf:resource="descriptor/16014-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16019-6">
    <skos:related rdf:resource="descriptor/13413-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16019-6">
    <skos:related rdf:resource="descriptor/11771-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16019-6">
    <skos:related rdf:resource="descriptor/16176-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13726-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15050-3">
    <skos:related rdf:resource="descriptor/10243-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18456-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id53">
    <rdfs:seeAlso rdf:resource="descriptor/11314-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16440-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14143-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12874-0">
    <skos:broader rdf:resource="thsys/73337"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12874-0">
    <skos:broader rdf:resource="thsys/70397"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12874-0">
    <skos:broader rdf:resource="descriptor/11640-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13483-3">
    <skos:broader rdf:resource="thsys/70947"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13483-3">
    <skos:broader rdf:resource="descriptor/13200-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12928-3">
    <skos:altLabel xml:lang="en">Environmental services</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12928-3">
    <skos:altLabel xml:lang="de">Umwelttechnische Industrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12928-3">
    <skos:altLabel xml:lang="de">Umweltschutzwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12928-3">
    <skos:altLabel xml:lang="de">Umweltschutzindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12928-3">
    <skos:altLabel xml:lang="de">Umweltschutzdienstleistung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19678-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10466-4">
    <skos:broader rdf:resource="thsys/73345"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10466-4">
    <skos:broader rdf:resource="thsys/71109"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10466-4">
    <skos:broader rdf:resource="thsys/70406"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10466-4">
    <skos:broader rdf:resource="descriptor/16594-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12249-2">
    <skos:broader rdf:resource="thsys/70037"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12249-2">
    <skos:broader rdf:resource="descriptor/10011-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18188-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19060-1">
    <skos:altLabel xml:lang="de">Nahrungsmittelgroßhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18005-0">
    <skos:prefLabel xml:lang="en">Commercial broadcasting</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18005-0">
    <skos:prefLabel xml:lang="de">Private Rundfunkanstalt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18672-3">
    <skos:altLabel xml:lang="en">Credit card organizer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19593-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14122-2">
    <skos:altLabel xml:lang="de">Frischfleisch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20506-5">
    <skos:prefLabel xml:lang="en">Mean reversion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20506-5">
    <skos:prefLabel xml:lang="de">Mean Reversion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11123-1">
    <skos:broader rdf:resource="thsys/70307"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14197-6">
    <skos:broader rdf:resource="thsys/70418"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14197-6">
    <skos:broader rdf:resource="descriptor/18235-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14197-6">
    <skos:broader rdf:resource="descriptor/14192-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11814-2">
    <skos:prefLabel xml:lang="en">Labour inspection</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11814-2">
    <skos:prefLabel xml:lang="de">Gewerbeaufsicht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15498-1">
    <skos:altLabel xml:lang="en">Heuristics</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15498-1">
    <skos:altLabel xml:lang="en">Heuristic procedure</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15498-1">
    <skos:altLabel xml:lang="de">Heuristische Planungsmethode</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15498-1">
    <skos:altLabel xml:lang="de">Heuristik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19014-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13826-6">
    <skos:broader rdf:resource="thsys/70892"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19607-2">
    <skos:broader rdf:resource="thsys/70154"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11153-6">
    <skos:scopeNote xml:lang="de">Ansatz zur Erklärung ökonomischen Verhaltens aufgrund des Interessenkonflikts zwischen einem "Agent" und seinem "Principal" und aufgrund der unzureichenden Kontrollmöglichkeiten des "Principal" über den "Agent".</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#closeMatch">
    <skos:definition xml:lang="en">skos:closeMatch is used to link two concepts that are
      sufficiently similar that they can be used interchangeably in some information retrieval
      applications. In order to avoid the possibility of "compound errors" when combining mappings
      across more than two concept schemes, skos:closeMatch is not declared to be a transitive
      property.</skos:definition>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11939-3">
    <skos:altLabel xml:lang="de">GRW (Gemeinschaftsaufgabe Regionale Wirtschaftsstruktur)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11939-3">
    <skos:altLabel xml:lang="de">Gemeinschaftsaufgabe "Verbesserung der regionalen Wirtschaftsstruktur"</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11819-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10265-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10564-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13691-3">
    <zbwext:useInsteadNote rdf:nodeID="id41"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19695-0">
    <skos:altLabel xml:lang="de">Parteistatut</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19695-0">
    <skos:altLabel xml:lang="de">Parteigericht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19695-0">
    <skos:altLabel xml:lang="de">Parteiapparat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11354-3">
    <skos:related rdf:resource="descriptor/11224-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11354-3">
    <skos:related rdf:resource="descriptor/11367-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11354-3">
    <skos:related rdf:resource="descriptor/18860-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11354-3">
    <skos:related rdf:resource="descriptor/18888-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11354-3">
    <skos:related rdf:resource="descriptor/11394-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11354-3">
    <skos:related rdf:resource="descriptor/19916-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11354-3">
    <skos:related rdf:resource="descriptor/11353-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13536-1">
    <skos:related rdf:resource="descriptor/13537-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13536-1">
    <skos:related rdf:resource="descriptor/18601-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13536-1">
    <skos:related rdf:resource="descriptor/13535-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13536-1">
    <skos:related rdf:resource="descriptor/13533-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13440-0">
    <skos:related rdf:resource="descriptor/13453-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13440-0">
    <skos:related rdf:resource="descriptor/18093-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13440-0">
    <skos:related rdf:resource="descriptor/13436-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13440-0">
    <skos:related rdf:resource="descriptor/13635-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11848-6">
    <skos:altLabel xml:lang="de">Wohnungsgemeinnützigkeitsgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11848-6">
    <skos:altLabel xml:lang="de">Heimstättenrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70154">
    <rdfs:label xml:lang="de">Recht und Rechtswissenschaft</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13697-5">
    <skos:scopeNote xml:lang="de">Bankensystem, in dem die Kreditinstitute alle Bankgeschäfte betreiben dürfen</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16599-4">
    <skos:prefLabel xml:lang="en">Civilization</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16599-4">
    <skos:prefLabel xml:lang="de">Zivilisation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10763-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12205-1">
    <skos:broader rdf:resource="thsys/70037"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12205-1">
    <skos:broader rdf:resource="descriptor/25742-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15908-1">
    <skos:narrower rdf:resource="descriptor/11269-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16777-6">
    <skos:broader rdf:resource="thsys/70822"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16777-6">
    <skos:broader rdf:resource="descriptor/15676-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16777-6">
    <skos:broader rdf:resource="descriptor/16789-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16185-3">
    <skos:prefLabel xml:lang="en">International patent law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16185-3">
    <skos:prefLabel xml:lang="de">Internationales Patentrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10031-4">
    <skos:altLabel xml:lang="de">Prognose der Gesamtwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10031-4">
    <skos:altLabel xml:lang="de">Gesamtwirtschaftliche Prognose</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#inScheme">
    <rdf:type rdf:resource="http://www.w3.org/1999/02/22-rdf-syntax-ns#Property"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#inScheme">
    <rdf:type rdf:resource="http://www.w3.org/2002/07/owl#ObjectProperty"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18419-1">
    <skos:related rdf:resource="descriptor/16173-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18419-1">
    <skos:related rdf:resource="descriptor/13684-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18419-1">
    <skos:related rdf:resource="descriptor/13507-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18419-1">
    <skos:related rdf:resource="descriptor/18418-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18419-1">
    <skos:related rdf:resource="descriptor/13056-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18645-6">
    <skos:related rdf:resource="descriptor/13273-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18645-6">
    <skos:related rdf:resource="descriptor/13621-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16697-4">
    <skos:prefLabel xml:lang="en">Gerontology</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16697-4">
    <skos:prefLabel xml:lang="de">Altersforschung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13966-4">
    <skos:altLabel xml:lang="de">Tourismusfachleute</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13966-4">
    <skos:altLabel xml:lang="de">Tourismusberufe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13966-4">
    <skos:altLabel xml:lang="de">Reiseleiterinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13966-4">
    <skos:altLabel xml:lang="de">Reiseleiter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13966-4">
    <skos:altLabel xml:lang="de">Reisebürokaufleute</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13966-4">
    <skos:altLabel xml:lang="de">Kurdirektorinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13966-4">
    <skos:altLabel xml:lang="de">Kurdirektoren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13966-4">
    <skos:altLabel xml:lang="de">Fremdenverkehrsfachleute</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12910-1">
    <skos:related rdf:resource="descriptor/19636-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12910-1">
    <skos:related rdf:resource="descriptor/16611-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12910-1">
    <skos:related rdf:resource="descriptor/18993-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12910-1">
    <skos:related rdf:resource="descriptor/12917-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12910-1">
    <skos:related rdf:resource="descriptor/18120-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12910-1">
    <skos:related rdf:resource="descriptor/11197-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12910-1">
    <skos:related rdf:resource="descriptor/19579-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12910-1">
    <skos:related rdf:resource="descriptor/16055-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12910-1">
    <skos:related rdf:resource="descriptor/11819-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12910-1">
    <skos:related rdf:resource="descriptor/18509-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12910-1">
    <skos:related rdf:resource="descriptor/12865-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12910-1">
    <skos:related rdf:resource="descriptor/13100-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12910-1">
    <skos:related rdf:resource="descriptor/18440-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16277-5">
    <skos:broader rdf:resource="thsys/73375"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16277-5">
    <skos:broader rdf:resource="descriptor/15466-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12134-5">
    <skos:related rdf:resource="descriptor/12747-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20306-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14751-6">
    <skos:broader rdf:resource="thsys/73341"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14751-6">
    <skos:broader rdf:resource="thsys/73305"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14751-6">
    <skos:broader rdf:resource="thsys/71038"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14751-6">
    <skos:broader rdf:resource="descriptor/18103-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18912-2">
    <skos:broader rdf:resource="thsys/73364"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18912-2">
    <skos:broader rdf:resource="descriptor/19670-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18839-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16961-6">
    <skos:broader rdf:resource="thsys/70003"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16961-6">
    <skos:broader rdf:resource="descriptor/16972-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14252-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12314-3">
    <zbwext:useInsteadNote rdf:nodeID="id137"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19800-1">
    <skos:broader rdf:resource="thsys/73342"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19800-1">
    <skos:broader rdf:resource="thsys/73334"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19800-1">
    <skos:broader rdf:resource="descriptor/16190-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18887-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16063-3">
    <skos:related rdf:resource="descriptor/12452-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16063-3">
    <skos:related rdf:resource="descriptor/12056-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15900-3">
    <skos:prefLabel xml:lang="en">Space science</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15900-3">
    <skos:prefLabel xml:lang="de">Weltraumforschung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13291-5">
    <skos:prefLabel xml:lang="en">Specialized bank</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13291-5">
    <skos:prefLabel xml:lang="de">Spezialbank</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19574-5">
    <skos:broader rdf:resource="thsys/72393"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19574-5">
    <skos:broader rdf:resource="descriptor/15376-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15907-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19600-2">
    <skos:related rdf:resource="descriptor/15787-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19600-2">
    <skos:related rdf:resource="descriptor/15657-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15019-4">
    <skos:altLabel xml:lang="de">Industrieabfall</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15019-4">
    <skos:altLabel xml:lang="de">Gewerbemüll</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15019-4">
    <skos:altLabel xml:lang="de">Gewerbeabfall</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19280-1">
    <skos:broader rdf:resource="thsys/71104"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19280-1">
    <skos:broader rdf:resource="descriptor/10210-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15627-2">
    <skos:broader rdf:resource="thsys/73348"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10404-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18313-3">
    <skos:broader rdf:resource="thsys/71024"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12612-2">
    <skos:prefLabel xml:lang="en">Ergonomics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12612-2">
    <skos:prefLabel xml:lang="de">Ergonomie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15861-1">
    <skos:related rdf:resource="descriptor/16687-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15861-1">
    <skos:related rdf:resource="descriptor/15836-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15861-1">
    <skos:related rdf:resource="descriptor/16566-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15861-1">
    <skos:related rdf:resource="descriptor/16353-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15861-1">
    <skos:related rdf:resource="descriptor/16562-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15861-1">
    <skos:related rdf:resource="descriptor/15860-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15861-1">
    <skos:related rdf:resource="descriptor/18860-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15861-1">
    <skos:related rdf:resource="descriptor/19216-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15211-5">
    <skos:related rdf:resource="descriptor/15060-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24856-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12125-6">
    <skos:altLabel xml:lang="en">Promoters</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12125-6">
    <skos:altLabel xml:lang="de">Organisatorinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19425-1">
    <skos:broader rdf:resource="thsys/71074"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19425-1">
    <skos:broader rdf:resource="descriptor/11664-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70179">
    <skos:narrower rdf:resource="descriptor/10624-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70179">
    <skos:narrower rdf:resource="descriptor/10472-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70179">
    <skos:narrower rdf:resource="descriptor/10597-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70179">
    <skos:narrower rdf:resource="descriptor/18160-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70179">
    <skos:narrower rdf:resource="descriptor/10623-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70179">
    <skos:narrower rdf:resource="descriptor/10587-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70179">
    <skos:narrower rdf:resource="descriptor/19309-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70179">
    <skos:narrower rdf:resource="descriptor/10346-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70179">
    <skos:narrower rdf:resource="descriptor/10842-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70179">
    <skos:narrower rdf:resource="descriptor/10969-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70179">
    <skos:narrower rdf:resource="descriptor/10617-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70179">
    <skos:narrower rdf:resource="descriptor/10811-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70179">
    <skos:narrower rdf:resource="descriptor/10614-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70179">
    <skos:narrower rdf:resource="descriptor/10613-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70179">
    <skos:narrower rdf:resource="descriptor/10527-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70179">
    <skos:narrower rdf:resource="descriptor/19071-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70179">
    <skos:narrower rdf:resource="descriptor/19299-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70179">
    <skos:narrower rdf:resource="descriptor/10415-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70179">
    <skos:narrower rdf:resource="descriptor/19245-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70179">
    <skos:narrower rdf:resource="descriptor/10605-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70179">
    <skos:narrower rdf:resource="thsys/71087"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70179">
    <skos:narrower rdf:resource="thsys/71100"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70179">
    <skos:narrower rdf:resource="thsys/71095"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70179">
    <skos:narrower rdf:resource="thsys/71098"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70179">
    <skos:narrower rdf:resource="thsys/71091"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70179">
    <skos:narrower rdf:resource="thsys/71089"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70179">
    <skos:narrower rdf:resource="thsys/71093"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70179">
    <skos:narrower rdf:resource="thsys/71090"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16233-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16668-4">
    <skos:broader rdf:resource="thsys/73356"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16668-4">
    <skos:broader rdf:resource="thsys/73346"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16668-4">
    <skos:broader rdf:resource="descriptor/10010-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16668-4">
    <skos:broader rdf:resource="descriptor/18444-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11857-5">
    <skos:broader rdf:resource="thsys/73354"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11857-5">
    <skos:broader rdf:resource="thsys/70058"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10261-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18284-5">
    <skos:broader rdf:resource="thsys/70445"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18284-5">
    <skos:broader rdf:resource="descriptor/17672-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70254">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71034">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15760-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70944">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18374-4">
    <skos:altLabel xml:lang="de">Kohlenpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70020">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70933">
    <skos:narrower rdf:resource="descriptor/18596-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70933">
    <skos:narrower rdf:resource="descriptor/18603-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70933">
    <skos:narrower rdf:resource="descriptor/13537-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70933">
    <skos:narrower rdf:resource="descriptor/18602-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70933">
    <skos:narrower rdf:resource="descriptor/18601-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70933">
    <skos:narrower rdf:resource="descriptor/13536-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70933">
    <skos:narrower rdf:resource="descriptor/18599-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70933">
    <skos:narrower rdf:resource="descriptor/13534-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70933">
    <skos:narrower rdf:resource="descriptor/13533-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70933">
    <skos:narrower rdf:resource="descriptor/10695-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70933">
    <skos:narrower rdf:resource="descriptor/18241-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70933">
    <skos:narrower rdf:resource="descriptor/13528-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13828-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16182-2">
    <skos:altLabel xml:lang="de">Rechtsharmonisierung der EU</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16182-2">
    <skos:altLabel xml:lang="de">Rechtsangleichung der EU</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16182-2">
    <skos:altLabel xml:lang="de">Gemeinschaftsrecht der EU</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16182-2">
    <skos:altLabel xml:lang="de">Gemeinschaftsrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16182-2">
    <skos:altLabel xml:lang="de">EU-Richtlinie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16182-2">
    <skos:altLabel xml:lang="de">EU-Rechtsharmonisierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16182-2">
    <skos:altLabel xml:lang="de">EU-Rechtsangleichung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16182-2">
    <skos:altLabel xml:lang="de">EG-Recht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11424-1">
    <skos:related rdf:resource="descriptor/18149-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11424-1">
    <skos:related rdf:resource="descriptor/11418-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11314-1">
    <skos:scopeNote xml:lang="de">Allgemeine Arbeitnehmer-Arbeitgeber-Beziehungen auf globaler Ebene, vorwiegend kollektiv und verbandsbezogen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13222-3">
    <skos:prefLabel xml:lang="en">Furniture trade</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13222-3">
    <skos:prefLabel xml:lang="de">Möbelhandel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11355-1">
    <skos:related rdf:resource="descriptor/11375-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11355-1">
    <skos:related rdf:resource="descriptor/11367-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11355-1">
    <skos:related rdf:resource="descriptor/18860-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11355-1">
    <skos:related rdf:resource="descriptor/19916-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11355-1">
    <skos:related rdf:resource="descriptor/19825-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11355-1">
    <skos:related rdf:resource="descriptor/11288-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11355-1">
    <skos:related rdf:resource="descriptor/18856-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19539-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18595-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18913-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16730-4">
    <skos:related rdf:resource="descriptor/16642-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16730-4">
    <skos:related rdf:resource="descriptor/19989-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16730-4">
    <skos:related rdf:resource="descriptor/15821-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11916-1">
    <skos:narrower rdf:resource="descriptor/18489-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11916-1">
    <skos:narrower rdf:resource="descriptor/11981-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10051-5">
    <skos:related rdf:resource="descriptor/10048-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10051-5">
    <skos:related rdf:resource="descriptor/10042-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10710-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18229-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12363-4">
    <skos:prefLabel xml:lang="en">Balance sheet date</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12363-4">
    <skos:prefLabel xml:lang="de">Bilanzstichtag</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19341-0">
    <skos:prefLabel xml:lang="en">Vegetable market</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19341-0">
    <skos:prefLabel xml:lang="de">Gemüsemarkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18580-1">
    <skos:related rdf:resource="descriptor/18585-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18580-1">
    <skos:related rdf:resource="descriptor/18581-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18580-1">
    <skos:related rdf:resource="descriptor/18579-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18580-1">
    <skos:related rdf:resource="descriptor/13813-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10495-4">
    <skos:broader rdf:resource="thsys/73347"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10495-4">
    <skos:broader rdf:resource="thsys/71028"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10495-4">
    <skos:broader rdf:resource="thsys/70091"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70183">
    <skos:broader rdf:resource="thsys/70037"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11073-4">
    <skos:related rdf:resource="descriptor/15718-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11073-4">
    <skos:related rdf:resource="descriptor/11092-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11073-4">
    <skos:related rdf:resource="descriptor/11046-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11073-4">
    <skos:related rdf:resource="descriptor/11083-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11073-4">
    <skos:related rdf:resource="descriptor/16713-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11073-4">
    <skos:related rdf:resource="descriptor/11071-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11073-4">
    <skos:related rdf:resource="descriptor/11070-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18930-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71018">
    <skos:prefLabel xml:lang="en">W.03  Fishery</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71018">
    <skos:prefLabel xml:lang="de">W.03  Fischerei</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19186-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16076-1">
    <skos:prefLabel xml:lang="en">Re-registration law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16076-1">
    <skos:prefLabel xml:lang="de">Umwandlungsrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18467-4">
    <skos:altLabel xml:lang="de">Kooperation der Industrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18329-2">
    <skos:broader rdf:resource="thsys/71066"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18329-2">
    <skos:broader rdf:resource="thsys/71023"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18329-2">
    <skos:broader rdf:resource="thsys/70889"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18329-2">
    <skos:broader rdf:resource="descriptor/12029-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16280-2">
    <skos:altLabel xml:lang="de">Parlamentarische Opposition</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16280-2">
    <skos:altLabel xml:lang="de">Opposition</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13097-1">
    <skos:prefLabel xml:lang="en">Sporting goods industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13097-1">
    <skos:prefLabel xml:lang="de">Sportartikelindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11648-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19855-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10242-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15013-2">
    <skos:related rdf:resource="descriptor/12605-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15013-2">
    <skos:related rdf:resource="descriptor/15757-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15013-2">
    <skos:related rdf:resource="descriptor/12976-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15013-2">
    <skos:related rdf:resource="descriptor/14035-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15013-2">
    <skos:related rdf:resource="descriptor/14182-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13747-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18261-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18488-3">
    <skos:broader rdf:resource="thsys/71052"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18488-3">
    <skos:broader rdf:resource="thsys/71005"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18488-3">
    <skos:broader rdf:resource="descriptor/11302-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10637-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15751-1">
    <skos:broader rdf:resource="thsys/73351"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15751-1">
    <skos:broader rdf:resource="thsys/70811"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15751-1">
    <skos:broader rdf:resource="descriptor/14047-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15751-1">
    <skos:broader rdf:resource="descriptor/18381-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15751-1">
    <skos:broader rdf:resource="descriptor/15745-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19206-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13110-0">
    <skos:prefLabel xml:lang="en">Timber industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13110-0">
    <skos:prefLabel xml:lang="de">Holzindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13330-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13743-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15811-2">
    <skos:related rdf:resource="descriptor/16564-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15811-2">
    <skos:related rdf:resource="descriptor/15810-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15811-2">
    <skos:related rdf:resource="descriptor/15722-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15811-2">
    <skos:related rdf:resource="descriptor/11163-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18561-5">
    <skos:related rdf:resource="descriptor/13253-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18561-5">
    <skos:related rdf:resource="descriptor/13308-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12987-1">
    <skos:broader rdf:resource="thsys/71009"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12987-1">
    <skos:broader rdf:resource="descriptor/12980-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12021-4">
    <skos:prefLabel xml:lang="en">Soil conservation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12021-4">
    <skos:prefLabel xml:lang="de">Bodenschutz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14051-6">
    <skos:related rdf:resource="descriptor/18398-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14051-6">
    <skos:related rdf:resource="descriptor/18272-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14051-6">
    <skos:related rdf:resource="descriptor/14343-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14051-6">
    <skos:related rdf:resource="descriptor/12930-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14051-6">
    <skos:related rdf:resource="descriptor/12805-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16683-1">
    <skos:broader rdf:resource="thsys/73347"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16683-1">
    <skos:broader rdf:resource="thsys/70839"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16683-1">
    <skos:broader rdf:resource="thsys/70573"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16683-1">
    <skos:broader rdf:resource="thsys/70013"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16683-1">
    <skos:broader rdf:resource="descriptor/16564-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16683-1">
    <skos:broader rdf:resource="descriptor/15658-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11901-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19378-5">
    <skos:broader rdf:resource="thsys/73367"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19378-5">
    <skos:broader rdf:resource="thsys/71082"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19378-5">
    <skos:broader rdf:resource="thsys/71064"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19378-5">
    <skos:broader rdf:resource="descriptor/16549-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19378-5">
    <skos:broader rdf:resource="descriptor/11728-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13697-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10219-0">
    <skos:altLabel xml:lang="de">Regionale Preistheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10219-0">
    <skos:altLabel xml:lang="de">Räumlicher Preiswettbewerb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16058-3">
    <skos:prefLabel xml:lang="en">Bankruptcy law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16058-3">
    <skos:prefLabel xml:lang="de">Insolvenzrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10263-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70962">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.10.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13191-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19024-5">
    <skos:related rdf:resource="descriptor/14060-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70355">
    <skos:broader rdf:resource="thsys/70055"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10054-6">
    <skos:altLabel xml:lang="en">Post-Keynesian theory</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10054-6">
    <skos:altLabel xml:lang="de">Postkeynesianische Theorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18380-2">
    <skos:prefLabel xml:lang="en">Reservoir</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18380-2">
    <skos:prefLabel xml:lang="de">Talsperre</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18536-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24706-5">
    <skos:altLabel xml:lang="de">Rabattkarte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24706-5">
    <skos:altLabel xml:lang="de">Pay-back Karte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24706-5">
    <skos:altLabel xml:lang="de">Kundenbonusprogramm</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24706-5">
    <skos:altLabel xml:lang="de">Bonuskarte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70120">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13375-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70109">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16250-4">
    <skos:altLabel xml:lang="de">Lehnsrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16250-4">
    <skos:altLabel xml:lang="de">Grundherrschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13553-1">
    <skos:narrower rdf:resource="descriptor/18598-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13553-1">
    <skos:narrower rdf:resource="descriptor/18597-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13553-1">
    <skos:narrower rdf:resource="descriptor/13627-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13553-1">
    <skos:narrower rdf:resource="descriptor/13624-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13553-1">
    <skos:narrower rdf:resource="descriptor/18595-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13077-0">
    <skos:prefLabel xml:lang="en">Electrical trade</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13077-0">
    <skos:prefLabel xml:lang="de">Elektrohandwerk</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15941-3">
    <skos:altLabel xml:lang="de">Reproduktion der Bevölkerung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15941-3">
    <skos:altLabel xml:lang="de">Fertilität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15941-3">
    <skos:altLabel xml:lang="de">Bevölkerungsreproduktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70253">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70933">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18972-5">
    <skos:broader rdf:resource="thsys/70814"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19231-0">
    <skos:narrower rdf:resource="descriptor/12721-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19231-0">
    <skos:narrower rdf:resource="descriptor/10176-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19231-0">
    <skos:narrower rdf:resource="descriptor/10822-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19231-0">
    <skos:narrower rdf:resource="descriptor/18980-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19351-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13757-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18647-2">
    <skos:broader rdf:resource="thsys/73338"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18647-2">
    <skos:broader rdf:resource="thsys/70922"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11236-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11360-1">
    <skos:scopeNote xml:lang="de">Nur für Schriften über die berufliche Bildung allgemein.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71014">
    <skos:prefLabel xml:lang="de">W.04.01.02  Regenerative Energiewirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18250-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17931-3">
    <skos:broader rdf:resource="thsys/71030"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17931-3">
    <skos:broader rdf:resource="descriptor/17758-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19273-5">
    <skos:prefLabel xml:lang="en">Rural financial market</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19273-5">
    <skos:prefLabel xml:lang="de">Ländlicher Finanzmarkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10757-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16959-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id148">
    <rdf:value xml:lang="de">Für Veränderungen in einzelnen Organisationen</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19519-6">
    <skos:scopeNote xml:lang="de">Wissenschaftlich orientierte Geschichtsschreibung zu Wirtschaftszweigen und der Bedeutung einzelner Firmen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18479-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14025-0">
    <skos:broader rdf:resource="thsys/70201"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14025-0">
    <skos:broader rdf:resource="descriptor/14042-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13076-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10184-0">
    <skos:related rdf:resource="descriptor/10156-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10184-0">
    <skos:related rdf:resource="descriptor/11260-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10184-0">
    <skos:related rdf:resource="descriptor/12057-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10184-0">
    <skos:related rdf:resource="descriptor/11194-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10184-0">
    <skos:related rdf:resource="descriptor/11187-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10184-0">
    <skos:related rdf:resource="descriptor/10169-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16556-1">
    <skos:related rdf:resource="descriptor/18895-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16556-1">
    <skos:related rdf:resource="descriptor/16724-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16556-1">
    <skos:related rdf:resource="descriptor/19611-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16556-1">
    <skos:related rdf:resource="descriptor/15928-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16556-1">
    <skos:related rdf:resource="descriptor/16680-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16556-1">
    <skos:related rdf:resource="descriptor/18924-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16556-1">
    <skos:related rdf:resource="descriptor/15694-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16556-1">
    <skos:related rdf:resource="descriptor/15654-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16556-1">
    <skos:related rdf:resource="descriptor/15918-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16556-1">
    <skos:related rdf:resource="descriptor/15931-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16556-1">
    <skos:related rdf:resource="descriptor/15627-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13558-5">
    <skos:prefLabel xml:lang="en">Motor vehicle traffic</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13558-5">
    <skos:prefLabel xml:lang="de">Kraftfahrzeugverkehr</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id205">
    <rdf:value xml:lang="de">für Messtechnik</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13393-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70270">
    <rdfs:label xml:lang="de">Produktions- und Kostentheorie</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18784-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id374">
    <rdfs:seeAlso rdf:resource="descriptor/10025-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12155-4">
    <skos:broader rdf:resource="thsys/70562"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12155-4">
    <skos:broader rdf:resource="thsys/70244"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12155-4">
    <skos:broader rdf:resource="descriptor/12713-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12155-4">
    <skos:broader rdf:resource="descriptor/12580-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11812-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11421-0">
    <skos:related rdf:resource="descriptor/13763-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11421-0">
    <skos:related rdf:resource="descriptor/11487-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11421-0">
    <skos:related rdf:resource="descriptor/13726-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11421-0">
    <skos:related rdf:resource="descriptor/18088-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11708-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15738-0">
    <skos:broader rdf:resource="thsys/73366"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15738-0">
    <skos:broader rdf:resource="thsys/70239"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15738-0">
    <skos:broader rdf:resource="descriptor/15768-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15738-0">
    <skos:broader rdf:resource="descriptor/16435-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11183-4">
    <skos:related rdf:resource="descriptor/11271-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11183-4">
    <skos:related rdf:resource="descriptor/19494-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11183-4">
    <skos:related rdf:resource="descriptor/11270-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11183-4">
    <skos:related rdf:resource="descriptor/11298-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11183-4">
    <skos:related rdf:resource="descriptor/11219-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11183-4">
    <skos:related rdf:resource="descriptor/18168-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11183-4">
    <skos:related rdf:resource="descriptor/11235-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11183-4">
    <skos:related rdf:resource="descriptor/19767-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11183-4">
    <skos:related rdf:resource="descriptor/19493-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11183-4">
    <skos:related rdf:resource="descriptor/11189-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11183-4">
    <skos:related rdf:resource="descriptor/11182-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11183-4">
    <skos:related rdf:resource="descriptor/11180-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15911-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18082-3">
    <skos:narrower rdf:resource="descriptor/19252-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19110-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18067-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70057">
    <skos:prefLabel xml:lang="en">V.01.02  Welfare economics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70057">
    <skos:prefLabel xml:lang="de">V.01.02  Wohlfahrtstheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70354">
    <skos:prefLabel xml:lang="en">G.03.01  Africa South of Sahara</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70354">
    <skos:prefLabel xml:lang="de">G.03.01  Afrika südlich der Sahara</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70071">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">P.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id85">
    <rdf:value xml:lang="de">für die allgemeine Beschreibung der aktuellen Wirtschaftslage eines Landes</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71115">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21072-0">
    <skos:broader rdf:resource="thsys/70892"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16472-0">
    <skos:prefLabel xml:lang="en">Occupation of territory</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16472-0">
    <skos:prefLabel xml:lang="de">Okkupation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10437-4">
    <skos:broader rdf:resource="thsys/73368"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10437-4">
    <skos:broader rdf:resource="thsys/71109"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10437-4">
    <skos:broader rdf:resource="thsys/70879"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10437-4">
    <skos:broader rdf:resource="thsys/70406"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10437-4">
    <skos:broader rdf:resource="descriptor/16273-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18547-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16335-3">
    <skos:narrower rdf:resource="descriptor/11539-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19500-6">
    <skos:scopeNote xml:lang="de">Alle Arten alternativer Vergütung, die als Teil des Einkommens zu versteuern sind,  z.B. Dienstwagen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19480-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10996-3">
    <skos:scopeNote xml:lang="de">Zentrale Lenkung des Einsatzes des Faktors Arbeit in der Planwirtschaft.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13054-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17630-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13709-3">
    <skos:related rdf:resource="descriptor/13738-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13709-3">
    <skos:related rdf:resource="descriptor/13721-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70839">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13647-6">
    <skos:altLabel xml:lang="de">Postgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13647-6">
    <skos:altLabel xml:lang="de">PostG (Postgesetz)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15598-4">
    <skos:broader rdf:resource="thsys/73341"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15598-4">
    <skos:broader rdf:resource="thsys/71039"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15598-4">
    <skos:broader rdf:resource="thsys/70383"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13361-3">
    <skos:narrower rdf:resource="descriptor/18881-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10618-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19561-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11626-3">
    <skos:related rdf:resource="descriptor/18822-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11626-3">
    <skos:related rdf:resource="descriptor/11612-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11626-3">
    <skos:related rdf:resource="descriptor/11043-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11570-4">
    <skos:related rdf:resource="descriptor/11573-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11570-4">
    <skos:related rdf:resource="descriptor/19384-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11570-4">
    <skos:related rdf:resource="descriptor/16545-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18654-5">
    <skos:scopeNote xml:lang="de">Direktmarketing mit Hilfe von Telefon, Telefax, Online-Diensten etc.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18199-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19677-2">
    <skos:prefLabel xml:lang="en">Milk price</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19677-2">
    <skos:prefLabel xml:lang="de">Milchpreis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19555-2">
    <skos:prefLabel xml:lang="en">Price bundling</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19555-2">
    <skos:prefLabel xml:lang="de">Price Bundling</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12385-1">
    <skos:prefLabel xml:lang="en">Carrying as liabilities</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12385-1">
    <skos:prefLabel xml:lang="de">Passivierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18952-4">
    <skos:prefLabel xml:lang="en">Electronic library</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18952-4">
    <skos:prefLabel xml:lang="de">Bibliotheksautomatisierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15968-4">
    <skos:broader rdf:resource="thsys/70154"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15968-4">
    <skos:broader rdf:resource="thsys/70007"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15968-4">
    <skos:broader rdf:resource="descriptor/15690-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18909-5">
    <skos:broader rdf:resource="thsys/73362"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18909-5">
    <skos:broader rdf:resource="thsys/70833"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18909-5">
    <skos:broader rdf:resource="thsys/70324"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18909-5">
    <skos:broader rdf:resource="descriptor/16545-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18909-5">
    <skos:broader rdf:resource="descriptor/18772-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18167-2">
    <skos:broader rdf:resource="thsys/73294"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18167-2">
    <skos:broader rdf:resource="descriptor/13164-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13638-0">
    <skos:related rdf:resource="descriptor/12927-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13638-0">
    <skos:related rdf:resource="descriptor/18639-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13638-0">
    <skos:related rdf:resource="descriptor/19466-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13638-0">
    <skos:related rdf:resource="descriptor/13644-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13638-0">
    <skos:related rdf:resource="descriptor/13592-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13638-0">
    <skos:related rdf:resource="descriptor/18634-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18401-6">
    <skos:altLabel xml:lang="en">Physical metallurgy</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18401-6">
    <skos:altLabel xml:lang="de">Physikalische Metallurgie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18401-6">
    <skos:altLabel xml:lang="de">Metallwissenschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18401-6">
    <skos:altLabel xml:lang="de">Metallographie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13573-2">
    <skos:altLabel xml:lang="de">Verkehrsforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14938-4">
    <skos:altLabel xml:lang="de">Teigware</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14938-4">
    <skos:altLabel xml:lang="de">Spaghetti</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14938-4">
    <skos:altLabel xml:lang="de">Spagetti</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14938-4">
    <skos:altLabel xml:lang="de">Nudeln</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14938-4">
    <skos:altLabel xml:lang="de">Makkaroni</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19451-0">
    <skos:broader rdf:resource="thsys/71067"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15396-2">
    <skos:altLabel xml:lang="en">Trend estimation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15396-2">
    <skos:altLabel xml:lang="en">Spectral analysis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15396-2">
    <skos:altLabel xml:lang="en">Exponential smoothing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15396-2">
    <skos:altLabel xml:lang="de">Zyklische Komponente</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15396-2">
    <skos:altLabel xml:lang="de">Zeitreihenzerlegung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15396-2">
    <skos:altLabel xml:lang="de">Zeitreihenkomponente</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15396-2">
    <skos:altLabel xml:lang="de">Trendschätzung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15396-2">
    <skos:altLabel xml:lang="de">Trendextrapolation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15396-2">
    <skos:altLabel xml:lang="de">Trendbestimmung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15396-2">
    <skos:altLabel xml:lang="de">Trendberechnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15396-2">
    <skos:altLabel xml:lang="de">Spektralanalyse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15396-2">
    <skos:altLabel xml:lang="de">Glättung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15396-2">
    <skos:altLabel xml:lang="de">Exponentielle Glättung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19615-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24708-1">
    <skos:related rdf:resource="descriptor/24782-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24708-1">
    <skos:related rdf:resource="descriptor/18647-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24708-1">
    <skos:related rdf:resource="descriptor/16757-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16976-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14807-5">
    <skos:altLabel xml:lang="de">Tischkalender</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14807-5">
    <skos:altLabel xml:lang="de">Schreibheft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14807-5">
    <skos:altLabel xml:lang="de">Papierschreibwaren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14807-5">
    <skos:altLabel xml:lang="de">Notizbuch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14807-5">
    <skos:altLabel xml:lang="de">Briefumschlag</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13296-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13190-4">
    <skos:broader rdf:resource="thsys/70957"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13190-4">
    <skos:broader rdf:resource="thsys/70816"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13190-4">
    <skos:broader rdf:resource="descriptor/13324-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18842-4">
    <skos:broader rdf:resource="thsys/70854"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18842-4">
    <skos:broader rdf:resource="descriptor/12180-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17583-0">
    <skos:broader rdf:resource="thsys/70327"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17583-0">
    <skos:broader rdf:resource="descriptor/17589-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18207-2">
    <skos:related rdf:resource="descriptor/19007-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13874-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11812-6">
    <skos:related rdf:resource="descriptor/11813-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11812-6">
    <skos:related rdf:resource="descriptor/18174-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11812-6">
    <skos:related rdf:resource="descriptor/11810-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11812-6">
    <skos:related rdf:resource="descriptor/18996-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11812-6">
    <skos:related rdf:resource="descriptor/14175-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11326-1">
    <skos:prefLabel xml:lang="en">Labour's share</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11326-1">
    <skos:prefLabel xml:lang="de">Lohnquote</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18560-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17695-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18566-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14527-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10908-5">
    <skos:altLabel xml:lang="en">Non-price competition</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10908-5">
    <skos:altLabel xml:lang="de">Servicewettbewerb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10908-5">
    <skos:altLabel xml:lang="de">Qualitätswettbewerb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10908-5">
    <skos:altLabel xml:lang="de">Leistungswettbewerb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17612-5">
    <skos:altLabel xml:lang="de">Demokratische Sozialistische Republik Sri Lanka</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17612-5">
    <skos:altLabel xml:lang="de">Ceylon</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73340">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18727-4">
    <skos:altLabel xml:lang="de">Wertpapierdarlehen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18740-5">
    <skos:prefLabel xml:lang="en">Environmental insurance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18740-5">
    <skos:prefLabel xml:lang="de">Umwelthaftpflichtversicherung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19092-2">
    <skos:related rdf:resource="descriptor/12361-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16970-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18834-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18497-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10334-0">
    <skos:altLabel xml:lang="en">Substitution elasticity</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14045-1">
    <skos:prefLabel xml:lang="en">Phosphate fertilizer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14045-1">
    <skos:prefLabel xml:lang="de">Phosphatdünger</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19298-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70073">
    <rdfs:label xml:lang="en">Central-Eastern Europe</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70073">
    <rdfs:label xml:lang="de">Ostmitteleuropa</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12504-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18203-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12964-6">
    <skos:related rdf:resource="descriptor/14161-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12964-6">
    <skos:related rdf:resource="descriptor/15887-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12964-6">
    <skos:related rdf:resource="descriptor/12945-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12964-6">
    <skos:related rdf:resource="descriptor/19626-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12964-6">
    <skos:related rdf:resource="descriptor/13154-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12964-6">
    <skos:related rdf:resource="descriptor/13205-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12964-6">
    <skos:related rdf:resource="descriptor/18344-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12964-6">
    <skos:related rdf:resource="descriptor/19780-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12964-6">
    <skos:related rdf:resource="descriptor/18343-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12964-6">
    <skos:related rdf:resource="descriptor/18312-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12964-6">
    <skos:related rdf:resource="descriptor/11807-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12964-6">
    <skos:related rdf:resource="descriptor/18342-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12964-6">
    <skos:related rdf:resource="descriptor/13873-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12964-6">
    <skos:related rdf:resource="descriptor/12969-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11803-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12994-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12470-3">
    <skos:altLabel xml:lang="en">Cost-volume-profit analysis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12470-3">
    <skos:altLabel xml:lang="en">Break-even point</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12470-3">
    <skos:altLabel xml:lang="de">Kosten-Volumen-Gewinn-Analyse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12470-3">
    <skos:altLabel xml:lang="de">Gewinnschwellenanalyse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10803-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70866">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.22.01.05</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19566-4">
    <skos:scopeNote xml:lang="de">Für Betrachtungen unter volkswirtschaftlichem Aspekt.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13081-2">
    <skos:altLabel xml:lang="de">Augenoptische Industrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18634-4">
    <skos:related rdf:resource="descriptor/13832-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18634-4">
    <skos:related rdf:resource="descriptor/13638-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18634-4">
    <skos:related rdf:resource="descriptor/11850-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13160-6">
    <skos:broader rdf:resource="thsys/70986"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13160-6">
    <skos:broader rdf:resource="descriptor/12993-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18873-0">
    <skos:broader rdf:resource="thsys/70844"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18873-0">
    <skos:broader rdf:resource="thsys/70822"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18873-0">
    <skos:broader rdf:resource="descriptor/13350-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18873-0">
    <skos:broader rdf:resource="descriptor/11373-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18982-2">
    <zbwext:useInsteadNote rdf:nodeID="id188"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14262-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10016-0">
    <skos:related rdf:resource="descriptor/18278-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10016-0">
    <skos:related rdf:resource="descriptor/21061-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10016-0">
    <skos:related rdf:resource="descriptor/12809-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10016-0">
    <skos:related rdf:resource="descriptor/12808-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10016-0">
    <skos:related rdf:resource="descriptor/18455-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10016-0">
    <skos:related rdf:resource="descriptor/20314-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10016-0">
    <skos:related rdf:resource="descriptor/16034-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10016-0">
    <skos:related rdf:resource="descriptor/10030-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10016-0">
    <skos:related rdf:resource="descriptor/13301-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70799">
    <skos:prefLabel xml:lang="en">W.06.01.04.01  Electronics industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70799">
    <skos:prefLabel xml:lang="de">W.06.01.04.01  Elektronikindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70988">
    <skos:broader rdf:resource="thsys/71004"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13057-6">
    <skos:prefLabel xml:lang="en">Aircraft industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13057-6">
    <skos:prefLabel xml:lang="de">Luftfahrtindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18189-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70578">
    <skos:narrower rdf:resource="descriptor/11598-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70578">
    <skos:narrower rdf:resource="descriptor/12489-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70578">
    <skos:narrower rdf:resource="descriptor/12525-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70578">
    <skos:narrower rdf:resource="descriptor/12511-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70578">
    <skos:narrower rdf:resource="descriptor/12509-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70578">
    <skos:narrower rdf:resource="descriptor/12524-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70578">
    <skos:narrower rdf:resource="descriptor/12508-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70578">
    <skos:narrower rdf:resource="descriptor/12540-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70578">
    <skos:narrower rdf:resource="descriptor/19100-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70578">
    <skos:narrower rdf:resource="descriptor/19101-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70578">
    <skos:narrower rdf:resource="descriptor/12523-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70578">
    <skos:narrower rdf:resource="descriptor/10052-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70578">
    <skos:narrower rdf:resource="descriptor/12535-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70578">
    <skos:narrower rdf:resource="descriptor/12533-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70578">
    <skos:narrower rdf:resource="descriptor/19098-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70578">
    <skos:narrower rdf:resource="descriptor/12521-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70578">
    <skos:narrower rdf:resource="descriptor/12480-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70578">
    <skos:narrower rdf:resource="descriptor/11529-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70578">
    <skos:narrower rdf:resource="descriptor/19096-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70578">
    <skos:narrower rdf:resource="descriptor/12473-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70578">
    <skos:narrower rdf:resource="descriptor/19099-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70578">
    <skos:narrower rdf:resource="descriptor/12475-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70578">
    <skos:narrower rdf:resource="descriptor/12474-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70578">
    <skos:narrower rdf:resource="descriptor/12497-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70578">
    <skos:narrower rdf:resource="descriptor/19097-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70578">
    <skos:narrower rdf:resource="descriptor/10011-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70578">
    <skos:narrower rdf:resource="descriptor/12495-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70578">
    <skos:narrower rdf:resource="descriptor/12531-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70578">
    <skos:narrower rdf:resource="descriptor/20906-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70578">
    <skos:narrower rdf:resource="descriptor/12520-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70578">
    <skos:narrower rdf:resource="descriptor/12518-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70578">
    <skos:narrower rdf:resource="descriptor/20881-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70578">
    <skos:narrower rdf:resource="descriptor/12492-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70578">
    <skos:narrower rdf:resource="descriptor/12491-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70578">
    <skos:narrower rdf:resource="descriptor/12490-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70578">
    <skos:narrower rdf:resource="descriptor/12515-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70578">
    <skos:narrower rdf:resource="descriptor/12514-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70578">
    <skos:narrower rdf:resource="descriptor/12470-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70578">
    <skos:narrower rdf:resource="descriptor/12513-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70578">
    <skos:narrower rdf:resource="descriptor/12469-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18848-6">
    <skos:prefLabel xml:lang="en">Insider-outsider model</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18848-6">
    <skos:prefLabel xml:lang="de">Insider-Outsider-Modell</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25881-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18013-1">
    <skos:related rdf:resource="descriptor/13488-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18013-1">
    <skos:related rdf:resource="descriptor/18126-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18013-1">
    <skos:related rdf:resource="descriptor/13871-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18013-1">
    <skos:related rdf:resource="descriptor/13481-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18013-1">
    <skos:related rdf:resource="descriptor/19604-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18013-1">
    <skos:related rdf:resource="descriptor/18717-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18013-1">
    <skos:related rdf:resource="descriptor/16601-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18013-1">
    <skos:related rdf:resource="descriptor/18535-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18013-1">
    <skos:related rdf:resource="descriptor/13301-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15176-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16487-1">
    <skos:broader rdf:resource="thsys/73369"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16487-1">
    <skos:broader rdf:resource="thsys/71109"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16487-1">
    <skos:broader rdf:resource="descriptor/16273-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19629-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12986-3">
    <skos:altLabel xml:lang="de">Kalisalzbergbau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12986-3">
    <skos:altLabel xml:lang="de">Kaliindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14961-2">
    <skos:broader rdf:resource="thsys/70201"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14961-2">
    <skos:broader rdf:resource="descriptor/14044-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18960-5">
    <skos:prefLabel xml:lang="en">Corporate waste management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18960-5">
    <skos:prefLabel xml:lang="de">Betriebliche Abfallwirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13826-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24663-4">
    <skos:scopeNote xml:lang="de">Gender Mainstreaming versucht in Abgrenzung zur Frauenpolitik, die unterschiedlichen Lebenssituationen und Interessen von Frauen und Männern gleichberechtigt zu berücksichtigen, da es keine geschlechtsneutrale Wirklichkeit gibt.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10993-2">
    <skos:altLabel xml:lang="de">Wirtschaftspolitischer Zielkonflikt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10993-2">
    <skos:altLabel xml:lang="de">Stabilitätsgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10993-2">
    <skos:altLabel xml:lang="de">Stabilitäts- und Wachstumsgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10993-2">
    <skos:altLabel xml:lang="de">Magisches Vieleck</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10993-2">
    <skos:altLabel xml:lang="de">Magisches Dreieck</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10993-2">
    <skos:altLabel xml:lang="de">Konjunkturpolitischer Zielkonflikt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16745-5">
    <skos:altLabel xml:lang="en">History of economic thought</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16745-5">
    <skos:altLabel xml:lang="de">Wissenschaftsgeschichte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16745-5">
    <skos:altLabel xml:lang="de">Wirtschaftswissenschaftliche Dogmengeschichte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16745-5">
    <skos:altLabel xml:lang="de">Theoriegeschichte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16745-5">
    <skos:altLabel xml:lang="de">Lehrmeinungen der Wirtschaftswissenschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16745-5">
    <skos:altLabel xml:lang="de">Ideengeschichte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16745-5">
    <skos:altLabel xml:lang="de">Geschichte der Wissenschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16745-5">
    <skos:altLabel xml:lang="de">Dogmengeschichte der Wirtschaftstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12829-5">
    <skos:altLabel xml:lang="en">Media selection</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12829-5">
    <skos:altLabel xml:lang="de">Media-Mix</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11125-4">
    <skos:prefLabel xml:lang="en">State trading</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11125-4">
    <skos:prefLabel xml:lang="de">Staatshandel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16896-5">
    <skos:broader rdf:resource="thsys/70361"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16896-5">
    <skos:broader rdf:resource="descriptor/18012-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70882">
    <skos:prefLabel xml:lang="en">W.19  Data processing industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70882">
    <skos:prefLabel xml:lang="de">W.19  Datenverarbeitungswirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10926-3">
    <skos:related rdf:resource="descriptor/19565-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10926-3">
    <skos:related rdf:resource="descriptor/10928-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12018-0">
    <skos:prefLabel xml:lang="en">Land policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12018-0">
    <skos:prefLabel xml:lang="de">Bodenpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14020-3">
    <skos:related rdf:resource="descriptor/12703-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14020-3">
    <skos:related rdf:resource="descriptor/14028-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14020-3">
    <skos:related rdf:resource="descriptor/13170-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12518-1">
    <skos:related rdf:resource="descriptor/19097-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12518-1">
    <skos:related rdf:resource="descriptor/10198-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13176-5">
    <skos:narrower rdf:resource="descriptor/18501-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13176-5">
    <skos:narrower rdf:resource="descriptor/18498-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18762-2">
    <skos:broader rdf:resource="thsys/70882"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18762-2">
    <skos:broader rdf:resource="descriptor/24708-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70017">
    <rdfs:label xml:lang="en">Psychology</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70017">
    <rdfs:label xml:lang="de">Psychologie</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14683-4">
    <skos:related rdf:resource="descriptor/14623-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19218-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16400-4">
    <skos:broader rdf:resource="thsys/73383"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16400-4">
    <skos:broader rdf:resource="thsys/73358"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16400-4">
    <skos:broader rdf:resource="descriptor/15932-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10323-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19399-4">
    <skos:prefLabel xml:lang="en">Government Credit Programme</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19399-4">
    <skos:prefLabel xml:lang="de">Öffentliche Kreditvergabe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10780-5">
    <skos:altLabel xml:lang="de">Devisenvorschriften</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10780-5">
    <skos:altLabel xml:lang="de">Devisenbestimmungen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19575-3">
    <skos:prefLabel xml:lang="en">Unit root test</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19575-3">
    <skos:prefLabel xml:lang="de">Unit Root Test</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18738-6">
    <skos:altLabel xml:lang="de">PKV (Private Krankenversicherung)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18119-6">
    <skos:narrower rdf:resource="descriptor/18001-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70824">
    <rdfs:label xml:lang="de">Rundfunk- und Fernsehanstalten</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73347">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12669-1">
    <skos:altLabel xml:lang="de">Zuliefererbeurteilung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12669-1">
    <skos:altLabel xml:lang="de">Lieferantenselektion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12669-1">
    <skos:altLabel xml:lang="de">Lieferantenbeurteilung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12669-1">
    <skos:altLabel xml:lang="de">Lieferantenanalyse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11444-2">
    <skos:broader rdf:resource="thsys/70306"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11444-2">
    <skos:broader rdf:resource="thsys/70141"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11444-2">
    <skos:broader rdf:resource="descriptor/10236-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70438">
    <rdfs:label xml:lang="en">Austria</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70438">
    <rdfs:label xml:lang="de">Österreich</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14734-6">
    <skos:narrower rdf:resource="descriptor/14737-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14734-6">
    <skos:narrower rdf:resource="descriptor/14736-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14734-6">
    <skos:narrower rdf:resource="descriptor/14735-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14734-6">
    <skos:narrower rdf:resource="descriptor/18263-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14734-6">
    <skos:narrower rdf:resource="descriptor/14733-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14734-6">
    <skos:narrower rdf:resource="descriptor/14732-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14734-6">
    <skos:narrower rdf:resource="descriptor/18264-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14734-6">
    <skos:narrower rdf:resource="descriptor/18265-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14734-6">
    <skos:narrower rdf:resource="descriptor/18282-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19223-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71009">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16427-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73368">
    <skos:prefLabel xml:lang="de">N.04.03.06  Struktur- und Entwicklungspolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14441-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19714-1">
    <skos:broader rdf:resource="thsys/73366"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19714-1">
    <skos:broader rdf:resource="thsys/73332"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16517-4">
    <skos:prefLabel xml:lang="en">Broadcasting policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16517-4">
    <skos:prefLabel xml:lang="de">Rundfunkpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14052-4">
    <skos:broader rdf:resource="thsys/70549"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14052-4">
    <skos:broader rdf:resource="descriptor/14336-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10166-2">
    <skos:broader rdf:resource="thsys/73394"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10166-2">
    <skos:broader rdf:resource="thsys/71118"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10166-2">
    <skos:broader rdf:resource="thsys/70141"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10166-2">
    <skos:broader rdf:resource="descriptor/16709-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10166-2">
    <skos:broader rdf:resource="descriptor/16745-5"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id143">
    <rdfs:seeAlso rdf:resource="descriptor/16757-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16771-4">
    <skos:altLabel xml:lang="de">Bibliothekskunde</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13457-4">
    <skos:altLabel xml:lang="en">Voluntary chain</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13457-4">
    <skos:altLabel xml:lang="de">Kettenladen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13457-4">
    <skos:altLabel xml:lang="de">Freiwillige Kette</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13457-4">
    <skos:altLabel xml:lang="de">Freiwillige Handelskette</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13457-4">
    <skos:altLabel xml:lang="de">Einzelhandelskette</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10504-1">
    <skos:prefLabel xml:lang="en">Rural-urban migration</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10504-1">
    <skos:prefLabel xml:lang="de">Landflucht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70135">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11711-5">
    <skos:related rdf:resource="descriptor/18905-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11711-5">
    <skos:related rdf:resource="descriptor/11728-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11711-5">
    <skos:related rdf:resource="descriptor/11600-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11711-5">
    <skos:related rdf:resource="descriptor/11599-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15468-3">
    <skos:related rdf:resource="descriptor/10222-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15468-3">
    <skos:related rdf:resource="descriptor/15467-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18582-4">
    <skos:altLabel xml:lang="de">Straßengüterverkehrswirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11054-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13239-0">
    <skos:broader rdf:resource="thsys/70961"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13239-0">
    <skos:broader rdf:resource="descriptor/13221-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13239-0">
    <skos:broader rdf:resource="descriptor/13200-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11307-5">
    <skos:prefLabel xml:lang="en">Labour court</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11307-5">
    <skos:prefLabel xml:lang="de">Arbeitsgericht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10187-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11721-2">
    <skos:related rdf:resource="descriptor/19686-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11721-2">
    <skos:related rdf:resource="descriptor/11722-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11721-2">
    <skos:related rdf:resource="descriptor/11554-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11721-2">
    <skos:related rdf:resource="descriptor/19212-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11646-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10678-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11876-1">
    <skos:scopeNote xml:lang="de">Keine allgemein gültige Definition. Häufigste Definitionen sind: Verkehr mit mittlerer Reichweite oder Verkehr innerhalb einer Region oder Stadt-Umland-Verkehr.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11852-1">
    <skos:altLabel xml:lang="de">Kommune</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12105-5">
    <skos:broader rdf:resource="thsys/70562"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12105-5">
    <skos:broader rdf:resource="thsys/70321"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19532-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19538-2">
    <skos:altLabel xml:lang="de">Geld-Brief-Spanne</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19538-2">
    <skos:altLabel xml:lang="de">Geld-Briefkurs-Spanne</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18620-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18403-2">
    <skos:broader rdf:resource="thsys/71001"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18403-2">
    <skos:broader rdf:resource="descriptor/13029-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11295-0">
    <skos:related rdf:resource="descriptor/19112-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11295-0">
    <skos:related rdf:resource="descriptor/12574-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11295-0">
    <skos:related rdf:resource="descriptor/12571-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11295-0">
    <skos:related rdf:resource="descriptor/12588-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11295-0">
    <skos:related rdf:resource="descriptor/19825-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11295-0">
    <skos:related rdf:resource="descriptor/11252-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11295-0">
    <skos:related rdf:resource="descriptor/19505-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11295-0">
    <skos:related rdf:resource="descriptor/11290-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11295-0">
    <skos:related rdf:resource="descriptor/12617-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11295-0">
    <skos:related rdf:resource="descriptor/11303-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10533-1">
    <skos:related rdf:resource="descriptor/11793-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10533-1">
    <skos:related rdf:resource="descriptor/10500-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70455">
    <skos:narrower rdf:resource="descriptor/19268-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70455">
    <skos:narrower rdf:resource="descriptor/19264-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70455">
    <skos:narrower rdf:resource="descriptor/10137-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70455">
    <skos:narrower rdf:resource="descriptor/18693-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70455">
    <skos:narrower rdf:resource="descriptor/18701-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70455">
    <skos:narrower rdf:resource="descriptor/10136-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70455">
    <skos:narrower rdf:resource="descriptor/19267-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70455">
    <skos:narrower rdf:resource="descriptor/18087-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70455">
    <skos:narrower rdf:resource="descriptor/18188-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70455">
    <skos:narrower rdf:resource="descriptor/19266-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70455">
    <skos:narrower rdf:resource="descriptor/10135-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70455">
    <skos:narrower rdf:resource="descriptor/12272-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70455">
    <skos:narrower rdf:resource="descriptor/10129-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70455">
    <skos:narrower rdf:resource="descriptor/19265-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19851-5">
    <skos:narrower rdf:resource="descriptor/18790-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11083-1">
    <skos:broader rdf:resource="thsys/73366"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11716-2">
    <skos:prefLabel xml:lang="en">Public infrastructure</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11716-2">
    <skos:prefLabel xml:lang="de">Infrastruktur</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15033-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16011-1">
    <skos:broader rdf:resource="thsys/73361"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16011-1">
    <skos:broader rdf:resource="descriptor/15998-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18748-3">
    <skos:altLabel xml:lang="en">Commercial land use</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18748-3">
    <skos:altLabel xml:lang="de">Gewerbefläche</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16611-5">
    <skos:related rdf:resource="descriptor/10973-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16611-5">
    <skos:related rdf:resource="descriptor/18524-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16611-5">
    <skos:related rdf:resource="descriptor/18120-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16611-5">
    <skos:related rdf:resource="descriptor/12910-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16611-5">
    <skos:related rdf:resource="descriptor/16608-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17980-4">
    <skos:altLabel xml:lang="de">Benelux-Länder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10161-5">
    <skos:prefLabel xml:lang="en">Utility theory</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10161-5">
    <skos:prefLabel xml:lang="de">Nutzentheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19120-2">
    <skos:related rdf:resource="descriptor/19653-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19120-2">
    <skos:related rdf:resource="descriptor/12812-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13370-2">
    <skos:altLabel xml:lang="de">Spezialärztinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13370-2">
    <skos:altLabel xml:lang="de">Spezialärzte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13370-2">
    <skos:altLabel xml:lang="de">Praxiszentrum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13370-2">
    <skos:altLabel xml:lang="de">Notarzt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13370-2">
    <skos:altLabel xml:lang="de">Niedergelassene Ärztinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13370-2">
    <skos:altLabel xml:lang="de">Niedergelassene Ärzte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13370-2">
    <skos:altLabel xml:lang="de">Krankenhausärztinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13370-2">
    <skos:altLabel xml:lang="de">Krankenhausärzte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13370-2">
    <skos:altLabel xml:lang="de">Kassenärztinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13370-2">
    <skos:altLabel xml:lang="de">Kassenärzte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13370-2">
    <skos:altLabel xml:lang="de">Gemeinschaftspraxis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13370-2">
    <skos:altLabel xml:lang="de">Fachärztinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13370-2">
    <skos:altLabel xml:lang="de">Fachärzte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13370-2">
    <skos:altLabel xml:lang="de">Arztpraxis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13370-2">
    <skos:altLabel xml:lang="de">Ärztinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13370-2">
    <skos:altLabel xml:lang="de">Arzt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13370-2">
    <skos:altLabel xml:lang="de">Amtsarzt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13370-2">
    <skos:altLabel xml:lang="de">Allgemeinärztinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13370-2">
    <skos:altLabel xml:lang="de">Allgemeinärzte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13440-0">
    <skos:prefLabel xml:lang="en">Family farm</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13440-0">
    <skos:prefLabel xml:lang="de">Landwirtschaftlicher Familienbetrieb</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12951-1">
    <skos:related rdf:resource="descriptor/14079-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12951-1">
    <skos:related rdf:resource="descriptor/18317-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15469-1">
    <skos:broader rdf:resource="thsys/73103"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15469-1">
    <skos:broader rdf:resource="thsys/71035"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15469-1">
    <skos:broader rdf:resource="descriptor/15475-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15731-0">
    <skos:altLabel xml:lang="en">Sewerage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15731-0">
    <skos:altLabel xml:lang="de">Sanitation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15731-0">
    <skos:altLabel xml:lang="de">Entwässerungsleitung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15731-0">
    <skos:altLabel xml:lang="de">Abwasserleitung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15731-0">
    <skos:altLabel xml:lang="de">Abwasserkanal</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11325-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10791-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id197">
    <rdf:value xml:lang="de">Für die Vergütung von Angestellten</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13630-2">
    <skos:prefLabel xml:lang="en">Private motor transport</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13630-2">
    <skos:prefLabel xml:lang="de">Motorisierter Individualverkehr</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19562-5">
    <skos:narrower rdf:resource="descriptor/12120-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19562-5">
    <skos:narrower rdf:resource="descriptor/12757-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19562-5">
    <skos:narrower rdf:resource="descriptor/12136-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19562-5">
    <skos:narrower rdf:resource="descriptor/12621-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19562-5">
    <skos:narrower rdf:resource="descriptor/12134-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70465">
    <rdfs:label xml:lang="de">Forstprodukte</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10313-1">
    <skos:prefLabel xml:lang="en">Factor intensity</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10313-1">
    <skos:prefLabel xml:lang="de">Faktorintensität</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11224-2">
    <skos:scopeNote xml:lang="de">Wert aller menschlichen Fähigkeiten, die im Erwerbsleben zur Erzielung von Einkommen einsetzbar sind.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16743-2">
    <skos:prefLabel xml:lang="en">Stockholm school</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16743-2">
    <skos:prefLabel xml:lang="de">Stockholmer Schule</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15051-1">
    <skos:broader rdf:resource="thsys/71035"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15051-1">
    <skos:broader rdf:resource="thsys/70269"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15051-1">
    <skos:broader rdf:resource="descriptor/15038-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12325-5">
    <skos:related rdf:resource="descriptor/12458-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12325-5">
    <skos:related rdf:resource="descriptor/12326-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10048-1">
    <skos:broader rdf:resource="thsys/71100"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10048-1">
    <skos:broader rdf:resource="thsys/70234"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10048-1">
    <skos:broader rdf:resource="descriptor/10065-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19672-5">
    <skos:broader rdf:resource="thsys/70485"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12073-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11377-5">
    <skos:broader rdf:resource="thsys/70845"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11377-5">
    <skos:broader rdf:resource="thsys/70176"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11377-5">
    <skos:broader rdf:resource="descriptor/18857-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17992-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14029-6">
    <skos:altLabel xml:lang="de">Hobbyartikel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14029-6">
    <skos:altLabel xml:lang="de">Campingartikel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id167">
    <rdfs:seeAlso rdf:resource="descriptor/18345-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10075-5">
    <skos:broader rdf:resource="thsys/70188"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10075-5">
    <skos:broader rdf:resource="descriptor/10076-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16430-2">
    <skos:prefLabel xml:lang="en">Political ethics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16430-2">
    <skos:prefLabel xml:lang="de">Politische Ethik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12515-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12487-0">
    <skos:altLabel xml:lang="en">ROI (Return on Investment)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18941-2">
    <skos:scopeNote xml:lang="de">Für die Bereitstellung von Netzdienstleistungen BENUTZE Mehrwertdienst bzw. die entsprechenden Unterbegriffe.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18941-2">
    <skos:scopeNote xml:lang="de">Kostenpflichtig oder unentgeltlich bereitgestellte Informationen auf Datenbank- oder Internet-Basis.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19786-4">
    <skos:altLabel xml:lang="de">Ambulante medizinische Versorgung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19786-4">
    <skos:altLabel xml:lang="de">Ambulante ärztliche Versorgung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18309-1">
    <skos:broader rdf:resource="thsys/70575"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18309-1">
    <skos:broader rdf:resource="thsys/70278"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18309-1">
    <skos:broader rdf:resource="descriptor/11327-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16384-4">
    <skos:related rdf:resource="descriptor/10966-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16384-4">
    <skos:related rdf:resource="descriptor/11068-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16384-4">
    <skos:related rdf:resource="descriptor/10401-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16384-4">
    <skos:related rdf:resource="descriptor/10984-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16384-4">
    <skos:related rdf:resource="descriptor/10968-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16384-4">
    <skos:related rdf:resource="descriptor/16588-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16384-4">
    <skos:related rdf:resource="descriptor/18850-5"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id48">
    <rdf:value xml:lang="de">Für die finanzielle Förderung der beruflichen Bildung</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19631-5">
    <skos:scopeNote xml:lang="de">Politische und geopolitische Aspekte von Meeresnutzung und Meereszugang.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15699-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71082">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.09.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10957-6">
    <skos:related rdf:resource="descriptor/10961-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10957-6">
    <skos:related rdf:resource="descriptor/10897-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16757-5">
    <skos:broader rdf:resource="thsys/73344"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16757-5">
    <skos:broader rdf:resource="thsys/73341"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16757-5">
    <skos:broader rdf:resource="descriptor/18426-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13998-5">
    <skos:narrower rdf:resource="descriptor/16252-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12520-0">
    <skos:prefLabel xml:lang="en">Intra-plant cost allocation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12520-0">
    <skos:prefLabel xml:lang="de">Innerbetriebliche Leistungsverrechnung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12945-3">
    <skos:broader rdf:resource="thsys/71028"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12945-3">
    <skos:broader rdf:resource="descriptor/18096-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18107-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16092-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73372">
    <rdfs:label xml:lang="de">Wirtschafts- und Finanzpolitik</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15554-3">
    <skos:related rdf:resource="descriptor/15225-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15554-3">
    <skos:related rdf:resource="descriptor/15054-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15554-3">
    <skos:related rdf:resource="descriptor/15475-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15554-3">
    <skos:related rdf:resource="descriptor/15571-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15554-3">
    <skos:related rdf:resource="descriptor/15555-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15554-3">
    <skos:related rdf:resource="descriptor/15491-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16400-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11537-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70348">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12916-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16336-1">
    <skos:related rdf:resource="descriptor/16019-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16336-1">
    <skos:related rdf:resource="descriptor/16283-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16336-1">
    <skos:related rdf:resource="descriptor/13413-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16336-1">
    <skos:related rdf:resource="descriptor/24783-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17799-5">
    <skos:broader rdf:resource="thsys/70086"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17799-5">
    <skos:broader rdf:resource="descriptor/17791-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18764-5">
    <skos:broader rdf:resource="thsys/71054"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18764-5">
    <skos:broader rdf:resource="thsys/71052"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18764-5">
    <skos:broader rdf:resource="thsys/70882"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18764-5">
    <skos:broader rdf:resource="descriptor/13987-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14012-2">
    <skos:prefLabel xml:lang="en">Clergy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14012-2">
    <skos:prefLabel xml:lang="de">Geistliche</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15899-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12396-3">
    <skos:related rdf:resource="descriptor/21061-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12396-3">
    <skos:related rdf:resource="descriptor/12558-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12396-3">
    <skos:related rdf:resource="descriptor/12876-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12396-3">
    <skos:related rdf:resource="descriptor/12384-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12396-3">
    <skos:related rdf:resource="descriptor/18302-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18066-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13656-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20261-2">
    <skos:scopeNote xml:lang="de">Ansatz, der eine Vielzahl von Anspruchsgruppen ins Unternehmenskonzept einbezieht, vor allem Kunden/innen und Mitarbeiter/innen, aber auch Interessengruppen aus Politik und Ökologie oder sogar die Gesellschaft als Ganzes.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18910-6">
    <skos:prefLabel xml:lang="en">Basic income</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18910-6">
    <skos:prefLabel xml:lang="de">Soziale Mindestsicherung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11910-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17811-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14996-4">
    <skos:related rdf:resource="descriptor/14934-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14996-4">
    <skos:related rdf:resource="descriptor/13163-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14996-4">
    <skos:related rdf:resource="descriptor/13237-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14996-4">
    <skos:related rdf:resource="descriptor/18215-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14996-4">
    <skos:related rdf:resource="descriptor/18213-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14996-4">
    <skos:related rdf:resource="descriptor/14065-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14996-4">
    <skos:related rdf:resource="descriptor/14942-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10780-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10533-1">
    <skos:altLabel xml:lang="en">Leading sector</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10533-1">
    <skos:altLabel xml:lang="de">Strategischer Sektor</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10533-1">
    <skos:altLabel xml:lang="de">Schlüsselindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10533-1">
    <skos:altLabel xml:lang="de">Schlüsselbranche</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17593-4">
    <skos:prefLabel xml:lang="en">Mongolia</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17593-4">
    <skos:prefLabel xml:lang="de">Mongolei</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14294-1">
    <skos:prefLabel xml:lang="en">Lead (heavy metal)</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14294-1">
    <skos:prefLabel xml:lang="de">Blei</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70327">
    <skos:narrower rdf:resource="descriptor/17592-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70327">
    <skos:narrower rdf:resource="descriptor/21042-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70327">
    <skos:narrower rdf:resource="descriptor/17590-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70327">
    <skos:narrower rdf:resource="descriptor/17589-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70327">
    <skos:narrower rdf:resource="descriptor/17588-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70327">
    <skos:narrower rdf:resource="descriptor/17586-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70327">
    <skos:narrower rdf:resource="descriptor/17599-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70327">
    <skos:narrower rdf:resource="descriptor/17584-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70327">
    <skos:narrower rdf:resource="descriptor/17583-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70327">
    <skos:narrower rdf:resource="descriptor/17580-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70327">
    <skos:narrower rdf:resource="descriptor/17578-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70327">
    <skos:narrower rdf:resource="descriptor/17576-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18425-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19126-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18388-0">
    <skos:altLabel xml:lang="de">Mineralvorkommen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18388-0">
    <skos:altLabel xml:lang="de">Mineralische Rohstoffreserven</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18388-0">
    <skos:altLabel xml:lang="de">Mineralische Lagerstätte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18388-0">
    <skos:altLabel xml:lang="de">Mineralische Bodenschätze</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18330-3">
    <skos:prefLabel xml:lang="en">Agricultural protectionism</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18330-3">
    <skos:prefLabel xml:lang="de">Agrarprotektionismus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11636-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19108-6">
    <skos:altLabel xml:lang="de">Unternehmensanalyse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19108-6">
    <skos:altLabel xml:lang="de">Potenzialanalyse des Betriebs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19108-6">
    <skos:altLabel xml:lang="de">Potentialanalyse des Betriebs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13247-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70031">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">P.19</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12047-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18399-2">
    <skos:narrower rdf:resource="descriptor/13064-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18399-2">
    <skos:narrower rdf:resource="descriptor/13050-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17717-5">
    <skos:prefLabel xml:lang="en">Nigeria</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17717-5">
    <skos:prefLabel xml:lang="de">Nigeria</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11657-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19304-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11034-0">
    <zbwext:useInsteadNote rdf:nodeID="id272"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14279-4">
    <skos:related rdf:resource="descriptor/13027-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18338-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16608-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71077">
    <skos:narrower rdf:resource="descriptor/11637-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71077">
    <skos:narrower rdf:resource="descriptor/11642-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71077">
    <skos:narrower rdf:resource="descriptor/11641-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71077">
    <skos:narrower rdf:resource="descriptor/11638-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71077">
    <skos:narrower rdf:resource="descriptor/16112-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71077">
    <skos:narrower rdf:resource="descriptor/19405-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71077">
    <skos:narrower rdf:resource="descriptor/11636-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71077">
    <skos:narrower rdf:resource="descriptor/11635-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71077">
    <skos:narrower rdf:resource="descriptor/19401-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71077">
    <skos:narrower rdf:resource="thsys/71074"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71077">
    <skos:narrower rdf:resource="thsys/71076"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71077">
    <skos:narrower rdf:resource="thsys/71075"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10889-4">
    <skos:prefLabel xml:lang="en">Administered price</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10889-4">
    <skos:prefLabel xml:lang="de">Administrierter Preis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12392-4">
    <skos:broader rdf:resource="thsys/71043"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12392-4">
    <skos:broader rdf:resource="descriptor/12432-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13252-1">
    <skos:narrower rdf:resource="descriptor/18586-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13309-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11972-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70178">
    <rdfs:label xml:lang="de">Preis- und Werttheorie</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14954-6">
    <skos:broader rdf:resource="thsys/73303"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14954-6">
    <skos:broader rdf:resource="descriptor/14955-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13376-4">
    <skos:broader rdf:resource="thsys/70876"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10470-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id254">
    <rdf:value xml:lang="de">Für den Fernmeldesektor</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11855-2">
    <skos:altLabel xml:lang="de">Peripherie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11855-2">
    <skos:altLabel xml:lang="de">Periphere Region</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73376">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10876-6">
    <skos:related rdf:resource="descriptor/20002-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10876-6">
    <skos:related rdf:resource="descriptor/12062-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10876-6">
    <skos:related rdf:resource="descriptor/18924-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10876-6">
    <skos:related rdf:resource="descriptor/10842-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10876-6">
    <skos:related rdf:resource="descriptor/18532-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10876-6">
    <skos:related rdf:resource="descriptor/19071-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10876-6">
    <skos:related rdf:resource="descriptor/18794-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10876-6">
    <skos:related rdf:resource="descriptor/10851-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10876-6">
    <skos:related rdf:resource="descriptor/10604-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13645-3">
    <skos:altLabel xml:lang="en">Travelling behaviour</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13645-3">
    <skos:altLabel xml:lang="de">Verhalten von Touristen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13645-3">
    <skos:altLabel xml:lang="de">Urlauberverhalten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13645-3">
    <skos:altLabel xml:lang="de">Touristenverhalten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13645-3">
    <skos:altLabel xml:lang="de">Reiseverhalten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16470-4">
    <skos:related rdf:resource="descriptor/16295-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16470-4">
    <skos:related rdf:resource="descriptor/10058-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16470-4">
    <skos:related rdf:resource="descriptor/16460-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16470-4">
    <skos:related rdf:resource="descriptor/15005-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16470-4">
    <skos:related rdf:resource="descriptor/19725-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16470-4">
    <skos:related rdf:resource="descriptor/13412-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16470-4">
    <skos:related rdf:resource="descriptor/11051-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16470-4">
    <skos:related rdf:resource="descriptor/18912-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16470-4">
    <skos:related rdf:resource="descriptor/16003-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16470-4">
    <skos:related rdf:resource="descriptor/19722-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16470-4">
    <skos:related rdf:resource="descriptor/11083-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16470-4">
    <skos:related rdf:resource="descriptor/16204-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16470-4">
    <skos:related rdf:resource="descriptor/16427-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16470-4">
    <skos:related rdf:resource="descriptor/16267-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10031-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18900-2">
    <skos:altLabel xml:lang="en">Smoking behaviour</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18900-2">
    <skos:altLabel xml:lang="de">Tabakmissbrauch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18900-2">
    <skos:altLabel xml:lang="de">Tabakmißbrauch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18900-2">
    <skos:altLabel xml:lang="de">Tabakkonsum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18900-2">
    <skos:altLabel xml:lang="de">Rauchverhalten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18900-2">
    <skos:altLabel xml:lang="de">Rauchverbot</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18900-2">
    <skos:altLabel xml:lang="de">Raucherverhalten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18900-2">
    <skos:altLabel xml:lang="de">Nikotinsucht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18900-2">
    <skos:altLabel xml:lang="de">Nikotinmissbrauch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18900-2">
    <skos:altLabel xml:lang="de">Nikotinmißbrauch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18900-2">
    <skos:altLabel xml:lang="de">Nikotinabhängigkeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18900-2">
    <skos:altLabel xml:lang="de">Nichtraucherschutz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18900-2">
    <skos:altLabel xml:lang="de">Anti-Raucher-Politik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18900-2">
    <skos:altLabel xml:lang="de">Anti-Raucher-Gesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18900-2">
    <skos:altLabel xml:lang="de">Anti-Raucher-Bewegung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10030-6">
    <skos:scopeNote xml:lang="de">Materielle Mittel zur Bedürfnisbefriedigung im Sinne der Volkswirtschaftslehre.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12154-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11649-5">
    <skos:broader rdf:resource="thsys/71098"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11649-5">
    <skos:broader rdf:resource="thsys/71074"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11649-5">
    <skos:broader rdf:resource="descriptor/11678-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11649-5">
    <skos:broader rdf:resource="descriptor/10699-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16131-5">
    <skos:narrower rdf:resource="descriptor/18792-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19558-3">
    <skos:related rdf:resource="descriptor/11529-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16214-1">
    <zbwext:useInsteadNote rdf:nodeID="id391"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16214-1">
    <zbwext:useInsteadNote rdf:nodeID="id390"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16214-1">
    <zbwext:useInsteadNote rdf:nodeID="id389"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16170-2">
    <skos:narrower rdf:resource="descriptor/18605-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16372-4">
    <skos:prefLabel xml:lang="en">Nationalist party</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16372-4">
    <skos:prefLabel xml:lang="de">Nationalistische Partei</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12432-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11790-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18574-3">
    <skos:narrower rdf:resource="descriptor/18576-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18175-3">
    <skos:altLabel xml:lang="de">Integriertes EDV-System</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18175-3">
    <skos:altLabel xml:lang="de">Integrierte Datenverarbeitung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17951-4">
    <skos:prefLabel xml:lang="en">Vanuatu</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17951-4">
    <skos:prefLabel xml:lang="de">Vanuatu</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11988-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18795-1">
    <skos:related rdf:resource="descriptor/12783-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13226-2">
    <skos:altLabel xml:lang="de">Papierwarenhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13226-2">
    <skos:altLabel xml:lang="de">Bürobedarfshandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11862-5">
    <skos:broader rdf:resource="thsys/71065"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11862-5">
    <skos:broader rdf:resource="thsys/70870"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11862-5">
    <skos:broader rdf:resource="descriptor/11879-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11862-5">
    <skos:broader rdf:resource="descriptor/11852-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12035-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12089-5">
    <skos:altLabel xml:lang="de">Geschäftspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12089-5">
    <skos:altLabel xml:lang="de">Firmenpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12089-5">
    <skos:altLabel xml:lang="de">Betriebswirtschaftspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12089-5">
    <skos:altLabel xml:lang="de">Betriebspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70922">
    <skos:prefLabel xml:lang="en">W.13.01  Telecommunications</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70922">
    <skos:prefLabel xml:lang="de">W.13.01  Telekommunikation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70817">
    <skos:prefLabel xml:lang="en">W.32.01  Waste water management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70817">
    <skos:prefLabel xml:lang="de">W.32.01  Abwasserwirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14088-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19257-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16745-5">
    <skos:broader rdf:resource="thsys/73393"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16745-5">
    <skos:broader rdf:resource="thsys/70141"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16745-5">
    <skos:broader rdf:resource="descriptor/15686-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70458">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.14.01.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13988-1">
    <skos:prefLabel xml:lang="en">Officials</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13988-1">
    <skos:prefLabel xml:lang="de">Funktionäre</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11912-2">
    <skos:prefLabel xml:lang="en">Regional economics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11912-2">
    <skos:prefLabel xml:lang="de">Regionalökonomik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13375-6">
    <skos:broader rdf:resource="thsys/71053"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13375-6">
    <skos:broader rdf:resource="thsys/71052"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13375-6">
    <skos:broader rdf:resource="thsys/70839"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13375-6">
    <skos:broader rdf:resource="descriptor/18890-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13375-6">
    <skos:broader rdf:resource="descriptor/11356-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18930-0">
    <skos:prefLabel xml:lang="en">Media research</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18930-0">
    <skos:prefLabel xml:lang="de">Medienforschung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13980-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71090">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10956-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11813-4">
    <skos:altLabel xml:lang="de">Energievorräte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11813-4">
    <skos:altLabel xml:lang="de">Energierohstoffreserven</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11813-4">
    <skos:altLabel xml:lang="de">Energiereserven</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10284-3">
    <skos:prefLabel xml:lang="en">Savings function</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10284-3">
    <skos:prefLabel xml:lang="de">Sparfunktion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18213-0">
    <skos:altLabel xml:lang="en">Savoury snack</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18213-0">
    <skos:altLabel xml:lang="de">Schokoriegel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18213-0">
    <skos:altLabel xml:lang="de">Salzstangen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18213-0">
    <skos:altLabel xml:lang="de">Salznüsse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18213-0">
    <skos:altLabel xml:lang="de">Salzgebäck</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18213-0">
    <skos:altLabel xml:lang="de">Müsliriegel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18213-0">
    <skos:altLabel xml:lang="de">Knabberartikel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18213-0">
    <skos:altLabel xml:lang="de">Käsegebäck</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18213-0">
    <skos:altLabel xml:lang="de">Kartoffelchips</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10089-1">
    <skos:prefLabel xml:lang="en">Voting rule</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10089-1">
    <skos:prefLabel xml:lang="de">Abstimmungsregel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13624-4">
    <skos:scopeNote xml:lang="de">Huckepack-Verkehr in der Binnen- und Seeschiffahrt.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13873-4">
    <skos:related rdf:resource="descriptor/11276-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13873-4">
    <skos:related rdf:resource="descriptor/12964-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10252-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18221-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13210-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12765-5">
    <skos:altLabel xml:lang="de">Zeitplanung des Betriebs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12765-5">
    <skos:altLabel xml:lang="de">Zeitmanagement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12765-5">
    <skos:altLabel xml:lang="de">Terminwesen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12765-5">
    <skos:altLabel xml:lang="de">Betriebliches Zeitmanagement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12765-5">
    <skos:altLabel xml:lang="de">Betriebliche Zeitplanung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12765-5">
    <skos:altLabel xml:lang="de">Betriebliche Terminsteuerung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70835">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11373-6">
    <skos:narrower rdf:resource="descriptor/18880-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11373-6">
    <skos:narrower rdf:resource="descriptor/18781-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11373-6">
    <skos:narrower rdf:resource="descriptor/18873-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11373-6">
    <skos:narrower rdf:resource="descriptor/11370-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19328-6">
    <skos:related rdf:resource="descriptor/11568-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19328-6">
    <skos:related rdf:resource="descriptor/10785-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10432-0">
    <skos:related rdf:resource="descriptor/10424-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10866-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13386-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18952-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18843-2">
    <skos:scopeNote xml:lang="de">Die Mitglieder sind Mitunternehmer/innen und Mitarbeiter/innen in Personalunion.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16309-4">
    <skos:prefLabel xml:lang="en">Sovereignty</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16309-4">
    <skos:prefLabel xml:lang="de">Souveränität</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15939-4">
    <skos:related rdf:resource="descriptor/15672-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15939-4">
    <skos:related rdf:resource="descriptor/19525-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15939-4">
    <skos:related rdf:resource="descriptor/15943-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15939-4">
    <skos:related rdf:resource="descriptor/15941-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15939-4">
    <skos:related rdf:resource="descriptor/10489-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19495-1">
    <skos:related rdf:resource="descriptor/11328-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19495-1">
    <skos:related rdf:resource="descriptor/18173-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19495-1">
    <skos:related rdf:resource="descriptor/10143-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19495-1">
    <skos:related rdf:resource="descriptor/11182-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19495-1">
    <skos:related rdf:resource="descriptor/13406-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19689-2">
    <skos:related rdf:resource="descriptor/19857-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19689-2">
    <skos:related rdf:resource="descriptor/16016-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19689-2">
    <skos:related rdf:resource="descriptor/16017-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19689-2">
    <skos:related rdf:resource="descriptor/16305-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19689-2">
    <skos:related rdf:resource="descriptor/13420-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14936-1">
    <skos:narrower rdf:resource="descriptor/14938-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14936-1">
    <skos:narrower rdf:resource="descriptor/14945-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70835">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.26</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13618-6">
    <skos:altLabel xml:lang="en">Long-distance road haulage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13618-6">
    <skos:altLabel xml:lang="en">Long-distance freight traffic</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70175">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16642-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10313-1">
    <skos:broader rdf:resource="thsys/70233"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10313-1">
    <skos:broader rdf:resource="descriptor/10256-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#broaderTransitive">
    <rdf:type rdf:resource="http://www.w3.org/1999/02/22-rdf-syntax-ns#Property"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#broaderTransitive">
    <rdf:type rdf:resource="http://www.w3.org/2002/07/owl#TransitiveProperty"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#broaderTransitive">
    <rdf:type rdf:resource="http://www.w3.org/2002/07/owl#ObjectProperty"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70649">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11192-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13975-3">
    <skos:altLabel xml:lang="de">Stewards</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13975-3">
    <skos:altLabel xml:lang="de">Stewardessen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13975-3">
    <skos:altLabel xml:lang="de">Restaurantfachleute</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13975-3">
    <skos:altLabel xml:lang="de">Kellnerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13975-3">
    <skos:altLabel xml:lang="de">Kellner</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13975-3">
    <skos:altLabel xml:lang="de">Hotelbesitzerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13975-3">
    <skos:altLabel xml:lang="de">Hotelbesitzer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13975-3">
    <skos:altLabel xml:lang="de">Hotelberufe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13975-3">
    <skos:altLabel xml:lang="de">Gastwirtinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13975-3">
    <skos:altLabel xml:lang="de">Gastwirte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13975-3">
    <skos:altLabel xml:lang="de">Gaststättenpersonal</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13975-3">
    <skos:altLabel xml:lang="de">Gaststättenberufe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13975-3">
    <skos:altLabel xml:lang="de">Gastronominnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13975-3">
    <skos:altLabel xml:lang="de">Gastronome</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13975-3">
    <skos:altLabel xml:lang="de">Gastgewerbepersonal</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13975-3">
    <skos:altLabel xml:lang="de">Flugbegleiterinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13975-3">
    <skos:altLabel xml:lang="de">Flugbegleiter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13975-3">
    <skos:altLabel xml:lang="de">Catering-Managerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13975-3">
    <skos:altLabel xml:lang="de">Catering-Manager</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18863-3">
    <skos:related rdf:resource="descriptor/18862-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18863-3">
    <skos:related rdf:resource="descriptor/18142-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18863-3">
    <skos:related rdf:resource="descriptor/11273-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18863-3">
    <skos:related rdf:resource="descriptor/15920-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18863-3">
    <skos:related rdf:resource="descriptor/11369-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18863-3">
    <skos:related rdf:resource="descriptor/11400-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18863-3">
    <skos:related rdf:resource="descriptor/10486-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12010-2">
    <skos:related rdf:resource="descriptor/13527-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12010-2">
    <skos:related rdf:resource="descriptor/19678-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12010-2">
    <skos:related rdf:resource="descriptor/20906-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18729-0">
    <skos:prefLabel xml:lang="en">History of financial markets</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18729-0">
    <skos:prefLabel xml:lang="de">Börsengeschichte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10979-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13176-5">
    <skos:broader rdf:resource="thsys/70969"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13176-5">
    <skos:broader rdf:resource="descriptor/12905-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12727-6">
    <skos:broader rdf:resource="thsys/70310"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12727-6">
    <skos:broader rdf:resource="descriptor/12725-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18141-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11326-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13686-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14537-1">
    <skos:prefLabel xml:lang="en">Optical product</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14537-1">
    <skos:prefLabel xml:lang="de">Optisches Produkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71093">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.07.04</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15755-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11224-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13186-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70485">
    <rdfs:label xml:lang="de">Unternehmensführung und Unternehmensplanung</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16743-2">
    <skos:broader rdf:resource="thsys/70141"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16743-2">
    <skos:broader rdf:resource="descriptor/16739-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16743-2">
    <skos:broader rdf:resource="descriptor/16745-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#note">
    <rdfs:comment xml:lang="en">This property may be used directly, or as a super-property for more
      specific note types.</rdfs:comment>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11449-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10987-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19763-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17722-5">
    <skos:prefLabel xml:lang="en">Sierra Leone</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17722-5">
    <skos:prefLabel xml:lang="de">Sierra Leone</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18956-3">
    <skos:prefLabel xml:lang="en">Sports association</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18956-3">
    <skos:prefLabel xml:lang="de">Sportverband</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18316-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11288-4">
    <skos:related rdf:resource="descriptor/11267-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11288-4">
    <skos:related rdf:resource="descriptor/11365-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11288-4">
    <skos:related rdf:resource="descriptor/11355-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11288-4">
    <skos:related rdf:resource="descriptor/18856-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71016">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13799-4">
    <skos:prefLabel xml:lang="en">Collection operations</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13799-4">
    <skos:prefLabel xml:lang="de">Inkassoverkehr</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14995-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71004">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70937">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70134">
    <skos:prefLabel xml:lang="de">P.07.01  NE-Metalle</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20888-0">
    <skos:related rdf:resource="descriptor/18024-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20888-0">
    <skos:related rdf:resource="descriptor/14640-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20888-0">
    <skos:related rdf:resource="descriptor/19481-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20888-0">
    <skos:related rdf:resource="descriptor/15776-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20888-0">
    <skos:related rdf:resource="descriptor/12974-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20888-0">
    <skos:related rdf:resource="descriptor/14174-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16732-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11243-5">
    <skos:prefLabel xml:lang="en">Public works programme</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11243-5">
    <skos:prefLabel xml:lang="de">Arbeitsbeschaffung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19140-3">
    <skos:broader rdf:resource="thsys/70459"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10140-6">
    <skos:prefLabel xml:lang="en">Information costs</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10140-6">
    <skos:prefLabel xml:lang="de">Informationskosten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13146-0">
    <skos:broader rdf:resource="thsys/70987"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13146-0">
    <skos:broader rdf:resource="descriptor/13145-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18140-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14393-6">
    <skos:prefLabel xml:lang="en">Tooling machine</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14393-6">
    <skos:prefLabel xml:lang="de">Werkzeugmaschine</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19375-4">
    <skos:scopeNote xml:lang="de">Benennt die Beobachtung, dass Kommunen nicht-zweckgebundene Zuweisungen von anderen öffentlichen Stellen nicht zur Senkung ihrer Steuersätze nutzen, sondern statt dessen ihre Ausgaben erhöhen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16656-4">
    <skos:scopeNote xml:lang="de">Freiwillige oder erzwungene räumliche Konzentration/Absonderung, meist bezüglich einzelner, oft benachteiligter Bevölkerungsgruppen in speziellen Wohngebieten. Aus dieser folgen dann besondere räumliche Verteilungsmuster (settlement pattern).</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73390">
    <skos:broader rdf:resource="thsys/70154"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11397-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70892">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14174-4">
    <skos:prefLabel xml:lang="en">Electricity</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14174-4">
    <skos:prefLabel xml:lang="de">Elektrizität</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13601-2">
    <skos:prefLabel xml:lang="en">Freight rate</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13601-2">
    <skos:prefLabel xml:lang="de">Gütertarif</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11282-2">
    <skos:broader rdf:resource="thsys/70052"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11282-2">
    <skos:broader rdf:resource="descriptor/11262-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20491-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19303-1">
    <skos:related rdf:resource="descriptor/19305-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19303-1">
    <skos:related rdf:resource="descriptor/19304-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19303-1">
    <skos:related rdf:resource="descriptor/19302-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19303-1">
    <skos:related rdf:resource="descriptor/14098-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19271-2">
    <skos:narrower rdf:resource="descriptor/19537-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11626-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71119">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.02.04</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18214-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11671-5">
    <skos:related rdf:resource="descriptor/19423-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11671-5">
    <skos:related rdf:resource="descriptor/16645-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11671-5">
    <skos:related rdf:resource="descriptor/10302-6"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id328">
    <rdf:value xml:lang="de">Für die Gesamtheit aller Telekommunikationseinrichtungen</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14365-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19069-4">
    <skos:related rdf:resource="descriptor/12039-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19069-4">
    <skos:related rdf:resource="descriptor/19108-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19069-4">
    <skos:related rdf:resource="descriptor/12053-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13807-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12265-4">
    <skos:broader rdf:resource="thsys/71107"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12265-4">
    <skos:broader rdf:resource="thsys/71046"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12265-4">
    <skos:broader rdf:resource="thsys/70899"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12265-4">
    <skos:broader rdf:resource="thsys/70893"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12265-4">
    <skos:broader rdf:resource="descriptor/12259-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17014-4">
    <skos:prefLabel xml:lang="en">Monaco</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17014-4">
    <skos:prefLabel xml:lang="de">Monaco</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18409-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11111-1">
    <skos:scopeNote xml:lang="de">Allgemein die Lehre von der Erkenntnis, Begründung, Geltung und Rangordnung von Werten. In der Volkswirtschaftslehre die Theorie über die Bestimmungsgründe des Tauschwertes der Güter.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15728-3">
    <skos:altLabel xml:lang="de">Rauchgasreinigung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15728-3">
    <skos:altLabel xml:lang="de">Abgasreinigung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15728-3">
    <skos:altLabel xml:lang="de">Abgasfilterung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15903-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18458-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13578-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11979-5">
    <skos:related rdf:resource="descriptor/11881-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11979-5">
    <skos:related rdf:resource="descriptor/10512-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11979-5">
    <skos:related rdf:resource="descriptor/11991-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11979-5">
    <skos:related rdf:resource="descriptor/11988-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11979-5">
    <skos:related rdf:resource="descriptor/11897-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11979-5">
    <skos:related rdf:resource="descriptor/19685-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11979-5">
    <skos:related rdf:resource="descriptor/11966-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11979-5">
    <skos:related rdf:resource="descriptor/10562-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11979-5">
    <skos:related rdf:resource="descriptor/15910-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11979-5">
    <skos:related rdf:resource="descriptor/11919-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14513-1">
    <skos:prefLabel xml:lang="en">Digital image and sound storage device</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14513-1">
    <skos:prefLabel xml:lang="de">Bildträger und Tonträger</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11185-0">
    <skos:scopeNote xml:lang="de">Regelungen des öffentlichen und des Privatrechts.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15916-2">
    <skos:related rdf:resource="descriptor/15061-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15916-2">
    <skos:related rdf:resource="descriptor/15708-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15916-2">
    <skos:related rdf:resource="descriptor/15910-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15916-2">
    <skos:related rdf:resource="descriptor/15909-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70904">
    <skos:broader rdf:resource="thsys/70905"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15097-5">
    <skos:prefLabel xml:lang="en">Index construction</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15097-5">
    <skos:prefLabel xml:lang="de">Indexberechnung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12531-2">
    <skos:broader rdf:resource="thsys/70578"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12531-2">
    <skos:broader rdf:resource="descriptor/12533-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19496-6">
    <skos:broader rdf:resource="thsys/71058"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19496-6">
    <skos:broader rdf:resource="descriptor/11272-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19496-6">
    <skos:broader rdf:resource="descriptor/11180-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14003-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18577-4">
    <skos:related rdf:resource="descriptor/14432-1"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id180">
    <rdf:value xml:lang="de">Für Kunstgegenstände</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16120-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18764-5">
    <skos:altLabel xml:lang="de">Informatikerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13051-4">
    <skos:narrower rdf:resource="descriptor/13060-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13051-4">
    <skos:narrower rdf:resource="descriptor/13053-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13051-4">
    <skos:narrower rdf:resource="descriptor/19052-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18887-3">
    <skos:broader rdf:resource="thsys/72181"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18887-3">
    <skos:broader rdf:resource="thsys/70835"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18887-3">
    <skos:broader rdf:resource="thsys/70504"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18887-3">
    <skos:broader rdf:resource="thsys/70012"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18887-3">
    <skos:broader rdf:resource="descriptor/19072-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18240-4">
    <skos:related rdf:resource="descriptor/14744-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15929-0">
    <skos:altLabel xml:lang="en">Afro-Americans</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15929-0">
    <skos:altLabel xml:lang="de">Schwarze Bevölkerung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15929-0">
    <skos:altLabel xml:lang="de">Negerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15929-0">
    <skos:altLabel xml:lang="de">Neger</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19408-1">
    <skos:broader rdf:resource="thsys/71076"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19408-1">
    <skos:broader rdf:resource="thsys/71075"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19408-1">
    <skos:broader rdf:resource="descriptor/11704-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19576-1">
    <skos:related rdf:resource="descriptor/19582-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12720-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19578-4">
    <skos:related rdf:resource="descriptor/13371-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19578-4">
    <skos:related rdf:resource="descriptor/11555-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19578-4">
    <skos:related rdf:resource="descriptor/16635-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10344-4">
    <skos:prefLabel xml:lang="en">Sunspot</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10344-4">
    <skos:prefLabel xml:lang="de">Sunspot</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70965">
    <skos:narrower rdf:resource="descriptor/19636-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70965">
    <skos:narrower rdf:resource="descriptor/19579-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70965">
    <skos:narrower rdf:resource="descriptor/16055-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70965">
    <skos:narrower rdf:resource="descriptor/11819-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70965">
    <skos:narrower rdf:resource="descriptor/18511-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70965">
    <skos:narrower rdf:resource="descriptor/11818-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70965">
    <skos:narrower rdf:resource="descriptor/18510-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70965">
    <skos:narrower rdf:resource="descriptor/18509-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70965">
    <skos:narrower rdf:resource="descriptor/12910-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70965">
    <skos:narrower rdf:resource="thsys/70964"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11303-6">
    <skos:scopeNote xml:lang="de">Funktionen und Aufgaben, die einem bestimmten Beruf oder Arbeitsplatz zugehörig sind.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14808-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12295-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19467-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19149-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16826-5">
    <skos:prefLabel xml:lang="en">Poland</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16826-5">
    <skos:prefLabel xml:lang="de">Polen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19580-3">
    <skos:prefLabel xml:lang="en">Migration statistics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19580-3">
    <skos:prefLabel xml:lang="de">Wanderungsstatistik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10204-6">
    <skos:altLabel xml:lang="en">Markup pricing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12295-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12912-4">
    <skos:narrower rdf:resource="descriptor/13110-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12912-4">
    <skos:narrower rdf:resource="descriptor/13109-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12912-4">
    <skos:narrower rdf:resource="descriptor/13215-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15904-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12612-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13302-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18698-6">
    <skos:prefLabel xml:lang="en">Credit policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18698-6">
    <skos:prefLabel xml:lang="de">Kreditpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11828-5">
    <skos:prefLabel xml:lang="en">Transport infrastructure policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11828-5">
    <skos:prefLabel xml:lang="de">Verkehrsinfrastrukturpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11704-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10808-2">
    <skos:altLabel xml:lang="de">Internationaler Finanzplatz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10808-2">
    <skos:altLabel xml:lang="de">Internationaler Bankplatz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10808-2">
    <skos:altLabel xml:lang="de">Internationaler Bankenplatz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10808-2">
    <skos:altLabel xml:lang="de">Finanzzentrum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10808-2">
    <skos:altLabel xml:lang="de">Bankplatz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12166-6">
    <skos:related rdf:resource="descriptor/18118-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12166-6">
    <skos:related rdf:resource="descriptor/12078-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12166-6">
    <skos:related rdf:resource="descriptor/19247-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18086-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18263-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18768-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11663-4">
    <skos:scopeNote xml:lang="de">Steuer auf Zinseinkünfte.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19374-6">
    <skos:scopeNote xml:lang="de">Hypothese, daß die Staatsbürger/innen annehmen, öffentliche Leistungen vom Staat ohne Gegenleistung zu erhalten.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15926-6">
    <skos:broader rdf:resource="thsys/70142"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15926-6">
    <skos:broader rdf:resource="descriptor/19643-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15754-2">
    <skos:altLabel xml:lang="de">Rohstoffvorrat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15754-2">
    <skos:altLabel xml:lang="de">Rohstoffvorkommen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15754-2">
    <skos:altLabel xml:lang="de">Rohstoffreserven</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15754-2">
    <skos:altLabel xml:lang="de">Rohstofflagerstätte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15754-2">
    <skos:altLabel xml:lang="de">Bodenschätze</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19048-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16981-0">
    <skos:broader rdf:resource="thsys/70003"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16981-0">
    <skos:broader rdf:resource="descriptor/16972-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12135-3">
    <skos:prefLabel xml:lang="en">Department</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12135-3">
    <skos:prefLabel xml:lang="de">Abteilung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id79">
    <rdfs:seeAlso rdf:resource="descriptor/10310-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11347-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16391-0">
    <skos:related rdf:resource="descriptor/16497-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16391-0">
    <skos:related rdf:resource="descriptor/16204-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16391-0">
    <skos:related rdf:resource="descriptor/16427-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18532-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18976-4">
    <skos:altLabel xml:lang="de">Konsumentenpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#changeNote">
    <skos:definition xml:lang="en">A note about a modification to a concept.</skos:definition>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20068-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19083-3">
    <skos:related rdf:resource="descriptor/19077-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19083-3">
    <skos:related rdf:resource="descriptor/16990-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18693-2">
    <skos:altLabel xml:lang="en">Forward rate agreement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18693-2">
    <skos:altLabel xml:lang="de">Zinsterminkontrakt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18693-2">
    <skos:altLabel xml:lang="de">Zinsterminhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18693-2">
    <skos:altLabel xml:lang="de">Zinsoption</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18693-2">
    <skos:altLabel xml:lang="de">Zins-Futures</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18693-2">
    <skos:altLabel xml:lang="de">Zinsforward</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12518-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13610-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11447-3">
    <skos:prefLabel xml:lang="en">Money illusion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11447-3">
    <skos:prefLabel xml:lang="de">Geldillusion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10401-4">
    <skos:broader rdf:resource="thsys/71051"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10401-4">
    <skos:broader rdf:resource="thsys/70038"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10401-4">
    <skos:broader rdf:resource="descriptor/11779-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16658-0">
    <skos:broader rdf:resource="thsys/73395"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16658-0">
    <skos:broader rdf:resource="thsys/73382"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16658-0">
    <skos:broader rdf:resource="descriptor/15717-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13949-4">
    <skos:prefLabel xml:lang="en">Food professions</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13949-4">
    <skos:prefLabel xml:lang="de">Ernährungsberufe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11817-3">
    <skos:prefLabel xml:lang="en">Business policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11817-3">
    <skos:prefLabel xml:lang="de">Gewerbepolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16012-6">
    <skos:altLabel xml:lang="de">Rundfunkverfassung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16012-6">
    <skos:altLabel xml:lang="de">Rundfunkstaatsvertrag</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16012-6">
    <skos:altLabel xml:lang="de">Rundfunkordnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16012-6">
    <skos:altLabel xml:lang="de">Rundfunkgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16012-6">
    <skos:altLabel xml:lang="de">Rundfunkfreiheit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16012-6">
    <skos:altLabel xml:lang="de">Rundfunk- und Fernsehrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16012-6">
    <skos:altLabel xml:lang="de">Fernsehrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70333">
    <skos:broader rdf:resource="thsys/p"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13641-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11595-2">
    <skos:broader rdf:resource="thsys/71081"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11595-2">
    <skos:broader rdf:resource="thsys/70869"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11595-2">
    <skos:broader rdf:resource="descriptor/11540-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73340">
    <skos:narrower rdf:resource="descriptor/15624-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73340">
    <skos:narrower rdf:resource="descriptor/15610-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73340">
    <skos:narrower rdf:resource="descriptor/19805-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73340">
    <skos:narrower rdf:resource="descriptor/19804-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73340">
    <skos:narrower rdf:resource="descriptor/15621-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73340">
    <skos:narrower rdf:resource="descriptor/18766-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73340">
    <skos:narrower rdf:resource="descriptor/15619-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73340">
    <skos:narrower rdf:resource="descriptor/18020-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73340">
    <skos:narrower rdf:resource="descriptor/24784-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73340">
    <skos:narrower rdf:resource="descriptor/19803-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73340">
    <skos:narrower rdf:resource="descriptor/18172-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73340">
    <skos:narrower rdf:resource="descriptor/18765-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73340">
    <skos:narrower rdf:resource="descriptor/18139-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71046">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10018-3">
    <skos:altLabel xml:lang="en">Unobserved economy</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10018-3">
    <skos:altLabel xml:lang="en">Underground economics</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10018-3">
    <skos:altLabel xml:lang="en">Shadow economy</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10018-3">
    <skos:altLabel xml:lang="en">Second economy</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10018-3">
    <skos:altLabel xml:lang="en">Moonlight economy</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10018-3">
    <skos:altLabel xml:lang="en">Irregular economy</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10018-3">
    <skos:altLabel xml:lang="de">Parallelwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18070-3">
    <skos:related rdf:resource="descriptor/12587-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18070-3">
    <skos:related rdf:resource="descriptor/12805-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18070-3">
    <skos:related rdf:resource="descriptor/15787-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18070-3">
    <skos:related rdf:resource="descriptor/18295-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18070-3">
    <skos:related rdf:resource="descriptor/13355-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18070-3">
    <skos:related rdf:resource="descriptor/15655-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18070-3">
    <skos:related rdf:resource="descriptor/19548-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18070-3">
    <skos:related rdf:resource="descriptor/10459-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18070-3">
    <skos:related rdf:resource="descriptor/20468-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18070-3">
    <skos:related rdf:resource="descriptor/10436-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18070-3">
    <skos:related rdf:resource="descriptor/10475-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18070-3">
    <skos:related rdf:resource="descriptor/19146-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19802-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11419-1">
    <skos:prefLabel xml:lang="en">Money counterfeiting</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11419-1">
    <skos:prefLabel xml:lang="de">Geldfälschung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16802-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14507-3">
    <skos:broader rdf:resource="thsys/70477"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14507-3">
    <skos:broader rdf:resource="descriptor/14502-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16315-2">
    <skos:altLabel xml:lang="de">Gewaltentrennung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16315-2">
    <skos:altLabel xml:lang="de">Gewaltenteilungslehre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16315-2">
    <skos:altLabel xml:lang="de">Gewaltengliederung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10008-6">
    <skos:related rdf:resource="descriptor/11141-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10008-6">
    <skos:related rdf:resource="descriptor/20301-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10008-6">
    <skos:related rdf:resource="descriptor/19089-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10008-6">
    <skos:related rdf:resource="descriptor/19586-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10008-6">
    <skos:related rdf:resource="descriptor/10293-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10008-6">
    <skos:related rdf:resource="descriptor/10438-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10008-6">
    <skos:related rdf:resource="descriptor/12278-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10008-6">
    <skos:related rdf:resource="descriptor/10237-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10008-6">
    <skos:related rdf:resource="descriptor/10294-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10008-6">
    <skos:related rdf:resource="descriptor/18107-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10008-6">
    <skos:related rdf:resource="descriptor/12209-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10008-6">
    <skos:related rdf:resource="descriptor/12224-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70348">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19308-5">
    <skos:altLabel xml:lang="en">In-bond assembly</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19308-5">
    <skos:altLabel xml:lang="en">Component assembly</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19308-5">
    <skos:altLabel xml:lang="de">Maquiladora</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19308-5">
    <skos:altLabel xml:lang="de">Lohnveredelung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18092-0">
    <skos:related rdf:resource="descriptor/18022-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18092-0">
    <skos:related rdf:resource="descriptor/19115-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18092-0">
    <skos:related rdf:resource="descriptor/14750-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18092-0">
    <skos:related rdf:resource="descriptor/18558-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18994-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13690-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11694-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10797-2">
    <skos:broader rdf:resource="thsys/71089"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10797-2">
    <skos:broader rdf:resource="descriptor/10019-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18217-6">
    <skos:prefLabel xml:lang="en">Convenience dessert</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18217-6">
    <skos:prefLabel xml:lang="de">Dessertprodukt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10303-4">
    <skos:broader rdf:resource="thsys/70233"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10303-4">
    <skos:broader rdf:resource="descriptor/10256-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10284-3">
    <skos:related rdf:resource="descriptor/10020-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10284-3">
    <skos:related rdf:resource="descriptor/10278-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15330-4">
    <skos:narrower rdf:resource="descriptor/15328-5"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id284">
    <rdfs:seeAlso rdf:resource="descriptor/12697-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13071-5">
    <skos:prefLabel xml:lang="en">Office machines industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13071-5">
    <skos:prefLabel xml:lang="de">Büromaschinenindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19310-4">
    <skos:narrower rdf:resource="descriptor/10564-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19310-4">
    <skos:narrower rdf:resource="descriptor/10647-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19310-4">
    <skos:narrower rdf:resource="descriptor/18330-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12394-0">
    <skos:narrower rdf:resource="descriptor/19090-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10826-0">
    <skos:related rdf:resource="descriptor/10831-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10826-0">
    <skos:related rdf:resource="descriptor/10830-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10826-0">
    <skos:related rdf:resource="descriptor/18001-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10826-0">
    <skos:related rdf:resource="descriptor/10827-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12881-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18165-6">
    <skos:narrower rdf:resource="descriptor/19090-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18165-6">
    <skos:narrower rdf:resource="descriptor/19160-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18165-6">
    <skos:narrower rdf:resource="descriptor/18349-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18165-6">
    <skos:narrower rdf:resource="descriptor/18960-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13221-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19263-1">
    <skos:altLabel xml:lang="de">Zielzone</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19263-1">
    <skos:altLabel xml:lang="de">Zielkorridor</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16661-4">
    <skos:altLabel xml:lang="de">Soziale Minderheit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16661-4">
    <skos:altLabel xml:lang="de">Minorität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12354-5">
    <skos:related rdf:resource="descriptor/10115-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12354-5">
    <skos:related rdf:resource="descriptor/11129-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73384">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70823">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70328">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">P.14</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10805-1">
    <skos:altLabel xml:lang="de">Global Financing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14730-0">
    <skos:prefLabel xml:lang="en">Analgesics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14730-0">
    <skos:prefLabel xml:lang="de">Schmerzmittel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11406-3">
    <skos:scopeNote xml:lang="de">Alle Finanzierungsaspekte der beruflichen Bildung: finanzielle Förderung durch den Staat, aber auch finanzielle Belastung z.B. der ausbildenden Betriebe.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11361-6">
    <skos:scopeNote xml:lang="de">Gezielte Ausbildungsmaßnahmen zur Vorbereitung auf einzelne Berufe oder Berufsfelder.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19132-2">
    <skos:related rdf:resource="descriptor/12832-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19132-2">
    <skos:related rdf:resource="descriptor/19131-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19132-2">
    <skos:related rdf:resource="descriptor/19818-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13486-4">
    <skos:prefLabel xml:lang="en">Shoplifting</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13486-4">
    <skos:prefLabel xml:lang="de">Ladendiebstahl</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14975-5">
    <skos:altLabel xml:lang="de">Tonic</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14975-5">
    <skos:altLabel xml:lang="de">Limonade</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14975-5">
    <skos:altLabel xml:lang="de">Light-Getränk</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14975-5">
    <skos:altLabel xml:lang="de">Koffeinhaltiges Erfrischungsgetränk</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14975-5">
    <skos:altLabel xml:lang="de">Kalorienreduziertes Getränk</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14975-5">
    <skos:altLabel xml:lang="de">Fruchtsaftgetränk</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14975-5">
    <skos:altLabel xml:lang="de">Diätgetränk</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14975-5">
    <skos:altLabel xml:lang="de">Cola-Getränk</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14975-5">
    <skos:altLabel xml:lang="de">Brause</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17943-3">
    <skos:broader rdf:resource="thsys/70128"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17943-3">
    <skos:broader rdf:resource="descriptor/17948-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19017-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18561-5">
    <skos:prefLabel xml:lang="en">Computer reservation system</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18561-5">
    <skos:prefLabel xml:lang="de">Elektronisches Buchungssystem</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70346">
    <skos:narrower rdf:resource="descriptor/14567-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70346">
    <skos:narrower rdf:resource="descriptor/18287-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70346">
    <skos:narrower rdf:resource="descriptor/14270-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70346">
    <skos:narrower rdf:resource="descriptor/19051-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70346">
    <skos:narrower rdf:resource="descriptor/14545-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70346">
    <skos:narrower rdf:resource="descriptor/14323-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70346">
    <skos:narrower rdf:resource="descriptor/14561-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70346">
    <skos:narrower rdf:resource="descriptor/18243-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70346">
    <skos:narrower rdf:resource="descriptor/14549-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70346">
    <skos:narrower rdf:resource="descriptor/14269-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70346">
    <skos:narrower rdf:resource="descriptor/14268-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70346">
    <skos:narrower rdf:resource="descriptor/14339-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70346">
    <skos:narrower rdf:resource="descriptor/14554-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70346">
    <skos:narrower rdf:resource="descriptor/14272-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70346">
    <skos:narrower rdf:resource="descriptor/14548-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70346">
    <skos:narrower rdf:resource="descriptor/14262-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70346">
    <skos:narrower rdf:resource="descriptor/14318-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70346">
    <skos:narrower rdf:resource="descriptor/14261-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14329-1">
    <skos:related rdf:resource="descriptor/14531-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70891">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18268-3">
    <skos:broader rdf:resource="thsys/70014"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18268-3">
    <skos:broader rdf:resource="descriptor/14599-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19427-4">
    <skos:altLabel xml:lang="de">Raffinerie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19427-4">
    <skos:altLabel xml:lang="de">Mineralölraffinerie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11987-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13433-4">
    <skos:broader rdf:resource="thsys/72181"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13433-4">
    <skos:broader rdf:resource="thsys/71028"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13433-4">
    <skos:broader rdf:resource="descriptor/15076-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15330-4">
    <skos:related rdf:resource="descriptor/15340-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70900">
    <rdfs:label xml:lang="en">Banking administration</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70900">
    <rdfs:label xml:lang="de">Bankbetriebslehre</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10784-4">
    <skos:prefLabel xml:lang="en">Balance of trade</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10784-4">
    <skos:prefLabel xml:lang="de">Handelsbilanz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70967">
    <skos:prefLabel xml:lang="en">W.07.03  Housing construction</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70967">
    <skos:prefLabel xml:lang="de">W.07.03  Wohnungsbau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15703-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16743-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19233-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70085">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">G.03.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10242-5">
    <skos:related rdf:resource="descriptor/18149-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10242-5">
    <skos:related rdf:resource="descriptor/10249-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10242-5">
    <skos:related rdf:resource="descriptor/10667-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10242-5">
    <skos:related rdf:resource="descriptor/11494-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10242-5">
    <skos:related rdf:resource="descriptor/11506-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12531-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19543-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70135">
    <rdfs:label xml:lang="de">Elektrotechnische und elektronische Produkte</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70151">
    <skos:broader rdf:resource="thsys/n"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13622-1">
    <skos:narrower rdf:resource="descriptor/13624-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13622-1">
    <skos:narrower rdf:resource="descriptor/13615-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14145-4">
    <skos:narrower rdf:resource="descriptor/14146-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14145-4">
    <skos:narrower rdf:resource="descriptor/14144-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14145-4">
    <skos:narrower rdf:resource="descriptor/14143-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14145-4">
    <skos:narrower rdf:resource="descriptor/14142-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14145-4">
    <skos:narrower rdf:resource="descriptor/14141-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14145-4">
    <skos:narrower rdf:resource="descriptor/14139-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14145-4">
    <skos:narrower rdf:resource="descriptor/14135-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10159-6">
    <skos:altLabel xml:lang="en">Utility tree</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10159-6">
    <skos:altLabel xml:lang="en">Indifference curve</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10159-6">
    <skos:altLabel xml:lang="de">Nutzenkurve</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10159-6">
    <skos:altLabel xml:lang="de">Nutzenbaum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10159-6">
    <skos:altLabel xml:lang="de">Indirekte Nutzenfunktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10159-6">
    <skos:altLabel xml:lang="de">Indifferenzkurve</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10436-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12733-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19171-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18527-5">
    <skos:altLabel xml:lang="de">Handelsbetriebsform</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18527-5">
    <skos:altLabel xml:lang="de">Betriebstyp des Handels</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18527-5">
    <skos:altLabel xml:lang="de">Angebotstyp des Handels</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19071-3">
    <skos:prefLabel xml:lang="en">Globalization</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19071-3">
    <skos:prefLabel xml:lang="de">Globalisierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17707-1">
    <skos:altLabel xml:lang="de">Republik Kap Verde</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17707-1">
    <skos:altLabel xml:lang="de">Kapverdische Inseln</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19764-0">
    <skos:related rdf:resource="descriptor/15672-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19764-0">
    <skos:related rdf:resource="descriptor/16124-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18952-4">
    <skos:narrower rdf:resource="descriptor/18179-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70238">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10690-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19233-3">
    <skos:related rdf:resource="descriptor/18982-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19233-3">
    <skos:related rdf:resource="descriptor/19362-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19233-3">
    <skos:related rdf:resource="descriptor/13454-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11047-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10608-3">
    <skos:prefLabel xml:lang="en">Single European market</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10608-3">
    <skos:prefLabel xml:lang="de">EU-Binnenmarkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18186-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19493-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13817-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15658-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10089-1">
    <skos:scopeNote xml:lang="de">Nur für Abstimmungsregeln im Rahmen der Public-Choice-Ansätze.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17682-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11426-4">
    <skos:related rdf:resource="descriptor/11434-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11426-4">
    <skos:related rdf:resource="descriptor/11452-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11426-4">
    <skos:related rdf:resource="descriptor/11415-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11426-4">
    <skos:related rdf:resource="descriptor/11411-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16401-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10638-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13722-4">
    <skos:prefLabel xml:lang="en">Futures market</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13722-4">
    <skos:prefLabel xml:lang="de">Financial Futures</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11573-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13399-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10985-1">
    <skos:altLabel xml:lang="de">Vergesellschaftung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10985-1">
    <skos:altLabel xml:lang="de">Sozialisierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10985-1">
    <skos:altLabel xml:lang="de">Sowjetisierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10985-1">
    <skos:altLabel xml:lang="de">Kollektivierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18450-0">
    <skos:altLabel xml:lang="de">Zeitungshandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18450-0">
    <skos:altLabel xml:lang="de">Pressegrosso</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18450-0">
    <skos:altLabel xml:lang="de">Pressegroßhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18450-0">
    <skos:altLabel xml:lang="de">Presseeinzelhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11591-3">
    <skos:altLabel xml:lang="de">Städtischer Haushalt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11591-3">
    <skos:altLabel xml:lang="de">Kommunalbudget</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11591-3">
    <skos:altLabel xml:lang="de">Gemeindehaushalt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11591-3">
    <skos:altLabel xml:lang="de">Gemeindeetat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11591-3">
    <skos:altLabel xml:lang="de">Gemeindebudget</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11591-3">
    <skos:altLabel xml:lang="de">Finanzhaushalt der Gemeinden</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18191-5">
    <skos:related rdf:resource="descriptor/19295-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18191-5">
    <skos:related rdf:resource="descriptor/19294-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18191-5">
    <skos:related rdf:resource="descriptor/18519-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18191-5">
    <skos:related rdf:resource="descriptor/18192-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18191-5">
    <skos:related rdf:resource="descriptor/14064-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18093-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#broadMatch">
    <rdfs:subPropertyOf rdf:resource="http://www.w3.org/2004/02/skos/core#broader"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#broadMatch">
    <rdfs:subPropertyOf rdf:resource="http://www.w3.org/2004/02/skos/core#mappingRelation"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12208-2">
    <skos:broader rdf:resource="thsys/71044"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12208-2">
    <skos:broader rdf:resource="descriptor/12209-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13109-6">
    <skos:broader rdf:resource="thsys/70991"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13109-6">
    <skos:broader rdf:resource="thsys/70969"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13109-6">
    <skos:broader rdf:resource="thsys/70964"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13109-6">
    <skos:broader rdf:resource="descriptor/12912-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13109-6">
    <skos:broader rdf:resource="descriptor/12910-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13109-6">
    <skos:broader rdf:resource="descriptor/12905-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16379-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18505-1">
    <skos:broader rdf:resource="thsys/70970"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18505-1">
    <skos:broader rdf:resource="descriptor/10008-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18254-0">
    <skos:prefLabel xml:lang="en">Industrial gas</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18254-0">
    <skos:prefLabel xml:lang="de">Technisches Gas</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18768-4">
    <skos:related rdf:resource="descriptor/18449-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18768-4">
    <skos:related rdf:resource="descriptor/16010-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18768-4">
    <skos:related rdf:resource="descriptor/18211-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13823-5">
    <skos:broader rdf:resource="thsys/70892"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13823-5">
    <skos:broader rdf:resource="thsys/70269"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13823-5">
    <skos:broader rdf:resource="descriptor/15045-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18782-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16119-2">
    <skos:prefLabel xml:lang="en">Professional expenditure</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16119-2">
    <skos:prefLabel xml:lang="de">Werbungskosten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70852">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.23.02.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18456-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10428-5">
    <skos:prefLabel xml:lang="en">Stochastic growth model</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10428-5">
    <skos:prefLabel xml:lang="de">Stochastisches Wachstumsmodell</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14822-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12469-2">
    <skos:altLabel xml:lang="de">ABC Analyse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13762-6">
    <skos:broader rdf:resource="thsys/70899"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13762-6">
    <skos:broader rdf:resource="descriptor/13763-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18493-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18867-2">
    <skos:altLabel xml:lang="en">Organizational learning</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18867-2">
    <skos:altLabel xml:lang="en">Organisational learning</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18867-2">
    <skos:altLabel xml:lang="en">Learning organisation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18867-2">
    <skos:altLabel xml:lang="de">Organisatorisches Lernen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18867-2">
    <skos:altLabel xml:lang="de">Organisatorischer Lernprozess</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18867-2">
    <skos:altLabel xml:lang="de">Organisatorischer Lernprozeß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18867-2">
    <skos:altLabel xml:lang="de">Lernendes Unternehmen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16306-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71016">
    <skos:broader rdf:resource="thsys/71017"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10473-0">
    <skos:altLabel xml:lang="en">Technology choice</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10314-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15045-3">
    <skos:prefLabel xml:lang="en">Mathematics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15045-3">
    <skos:prefLabel xml:lang="de">Mathematik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71101">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13530-6">
    <skos:prefLabel xml:lang="en">Ferry shipping</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13530-6">
    <skos:prefLabel xml:lang="de">Fährverkehr</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14015-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id107">
    <rdf:value xml:lang="de">Für Führungspersonal allgemein</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15644-2">
    <skos:narrower rdf:resource="descriptor/15678-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15644-2">
    <skos:narrower rdf:resource="descriptor/19777-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15644-2">
    <skos:narrower rdf:resource="descriptor/19776-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15644-2">
    <skos:narrower rdf:resource="descriptor/16719-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15644-2">
    <skos:narrower rdf:resource="descriptor/15045-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15644-2">
    <skos:narrower rdf:resource="descriptor/19608-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10148-4">
    <skos:broader rdf:resource="thsys/71116"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10148-4">
    <skos:broader rdf:resource="descriptor/10144-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16370-1">
    <skos:broader rdf:resource="thsys/73378"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16370-1">
    <skos:broader rdf:resource="descriptor/19151-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15778-2">
    <skos:prefLabel xml:lang="en">Pollution prevention</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15778-2">
    <skos:prefLabel xml:lang="de">Vorsorgeprinzip</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11859-1">
    <skos:prefLabel xml:lang="en">Land use</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11859-1">
    <skos:prefLabel xml:lang="de">Bodennutzung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10070-1">
    <skos:prefLabel xml:lang="en">General equilibrium</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10070-1">
    <skos:prefLabel xml:lang="de">Allgemeines Gleichgewicht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14107-5">
    <skos:altLabel xml:lang="de">Birne</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12708-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71043">
    <skos:broader rdf:resource="thsys/70186"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15473-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15956-4">
    <skos:related rdf:resource="descriptor/15958-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15956-4">
    <skos:related rdf:resource="descriptor/10613-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15956-4">
    <skos:related rdf:resource="descriptor/15952-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15956-4">
    <skos:related rdf:resource="descriptor/15951-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15956-4">
    <skos:related rdf:resource="descriptor/19648-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20897-6">
    <skos:scopeNote xml:lang="de">Beim Hub-and-Spoke-Verkehr (Nabe-Speiche-System) handelt es sich um ein Transportsystem mit einem zentralen Umschlagspunkt ("Nabe") für ein bestimmtes Einzugsgebiet, mit dem regionale Punkte über "Speichen" in regelmäßiger Verbindung stehen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13019-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71037">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13984-2">
    <skos:prefLabel xml:lang="en">Members of Parliament</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13984-2">
    <skos:prefLabel xml:lang="de">Abgeordnete</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16034-3">
    <skos:prefLabel xml:lang="en">Product liability</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16034-3">
    <skos:prefLabel xml:lang="de">Produkthaftung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10490-0">
    <skos:altLabel xml:lang="de">Bewässerungssystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19853-1">
    <skos:scopeNote xml:lang="de">Unterstützung bei der Rechtsverfolgung und Rechtsdurchsetzung eines Staates durch einen anderen Staat. Meist nur nach Abschluss internationaler oder zweiseitiger Vereinbarungen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19833-0">
    <skos:related rdf:resource="descriptor/19155-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19833-0">
    <skos:related rdf:resource="descriptor/19654-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12121-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70424">
    <skos:broader rdf:resource="thsys/70549"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10690-6">
    <skos:related rdf:resource="descriptor/10942-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10690-6">
    <skos:related rdf:resource="descriptor/10951-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10690-6">
    <skos:related rdf:resource="descriptor/12793-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10690-6">
    <skos:related rdf:resource="descriptor/19324-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10690-6">
    <skos:related rdf:resource="descriptor/19314-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71058">
    <skos:narrower rdf:resource="descriptor/18083-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71058">
    <skos:narrower rdf:resource="descriptor/19112-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71058">
    <skos:narrower rdf:resource="descriptor/19496-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71058">
    <skos:narrower rdf:resource="descriptor/11302-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71058">
    <skos:narrower rdf:resource="descriptor/11301-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71058">
    <skos:narrower rdf:resource="descriptor/11299-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71058">
    <skos:narrower rdf:resource="descriptor/11294-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71058">
    <skos:narrower rdf:resource="descriptor/11295-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71058">
    <skos:narrower rdf:resource="descriptor/11298-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71058">
    <skos:narrower rdf:resource="descriptor/11297-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71058">
    <skos:narrower rdf:resource="descriptor/11293-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71058">
    <skos:narrower rdf:resource="descriptor/19825-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71058">
    <skos:narrower rdf:resource="descriptor/11252-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71058">
    <skos:narrower rdf:resource="descriptor/11228-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71058">
    <skos:narrower rdf:resource="descriptor/19505-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71058">
    <skos:narrower rdf:resource="descriptor/11361-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71058">
    <skos:narrower rdf:resource="descriptor/11290-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71058">
    <skos:narrower rdf:resource="descriptor/11289-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71058">
    <skos:narrower rdf:resource="descriptor/11288-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71058">
    <skos:narrower rdf:resource="descriptor/11286-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71058">
    <skos:narrower rdf:resource="descriptor/11285-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71058">
    <skos:narrower rdf:resource="descriptor/11303-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18246-6">
    <skos:altLabel xml:lang="de">Propellerflugzeug</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18246-6">
    <skos:altLabel xml:lang="de">Privatflugzeug</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18275-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16286-4">
    <skos:altLabel xml:lang="de">Volksaufstand</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16286-4">
    <skos:altLabel xml:lang="de">Revolution</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16286-4">
    <skos:altLabel xml:lang="de">Revolte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16286-4">
    <skos:altLabel xml:lang="de">Rebellion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16286-4">
    <skos:altLabel xml:lang="de">Aufstand</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13722-4">
    <skos:scopeNote xml:lang="de">Standardisierte Finanztermingeschäfte im Devisenhandel, Edelmetallhandel und Aktienindexhandel.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19320-1">
    <skos:scopeNote xml:lang="de">Vergeltungsmaßnahme auf negative handelspolitische Maßnahmen eines Handelspartners.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15061-5">
    <skos:prefLabel xml:lang="en">Demographic statistics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15061-5">
    <skos:prefLabel xml:lang="de">Bevölkerungsstatistik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17595-0">
    <skos:narrower rdf:resource="descriptor/17503-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17595-0">
    <skos:narrower rdf:resource="descriptor/17502-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17595-0">
    <skos:narrower rdf:resource="descriptor/17500-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17595-0">
    <skos:narrower rdf:resource="descriptor/17593-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17595-0">
    <skos:narrower rdf:resource="descriptor/17497-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17595-0">
    <skos:narrower rdf:resource="descriptor/17496-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17595-0">
    <skos:narrower rdf:resource="descriptor/17472-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17595-0">
    <skos:narrower rdf:resource="descriptor/17491-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17595-0">
    <skos:narrower rdf:resource="descriptor/17464-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10106-6">
    <skos:broader rdf:resource="thsys/70274"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10106-6">
    <skos:broader rdf:resource="thsys/70057"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10106-6">
    <skos:broader rdf:resource="descriptor/10113-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10106-6">
    <skos:broader rdf:resource="descriptor/10975-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17565-2">
    <skos:broader rdf:resource="thsys/70147"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17565-2">
    <skos:broader rdf:resource="descriptor/19165-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70822">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id111">
    <rdfs:seeAlso rdf:resource="descriptor/10575-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11427-2">
    <skos:related rdf:resource="descriptor/10773-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11427-2">
    <skos:related rdf:resource="descriptor/10397-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11427-2">
    <skos:related rdf:resource="descriptor/11506-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11427-2">
    <skos:related rdf:resource="descriptor/11415-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12129-5">
    <skos:altLabel xml:lang="en">Organizational means</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12129-5">
    <skos:altLabel xml:lang="de">Organisatorisches Hilfsmittel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12129-5">
    <skos:altLabel xml:lang="de">Organisatorische Darstellung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12129-5">
    <skos:altLabel xml:lang="de">Organisationstechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12129-5">
    <skos:altLabel xml:lang="de">Organisationsschema</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12129-5">
    <skos:altLabel xml:lang="de">Organisationsschaubild</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12129-5">
    <skos:altLabel xml:lang="de">Organisationsplan</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12129-5">
    <skos:altLabel xml:lang="de">Organisationshandbuch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12129-5">
    <skos:altLabel xml:lang="de">Organigramm</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10184-0">
    <skos:broader rdf:resource="thsys/71116"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10184-0">
    <skos:broader rdf:resource="descriptor/10148-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10184-0">
    <skos:broader rdf:resource="descriptor/10177-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10184-0">
    <skos:broader rdf:resource="descriptor/10040-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15479-5">
    <skos:prefLabel xml:lang="en">Risk neutrality</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15479-5">
    <skos:prefLabel xml:lang="de">Risikoneutralität</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16527-1">
    <skos:narrower rdf:resource="descriptor/19512-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16527-1">
    <skos:narrower rdf:resource="descriptor/18739-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16527-1">
    <skos:narrower rdf:resource="descriptor/16548-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16527-1">
    <skos:narrower rdf:resource="descriptor/13416-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16527-1">
    <skos:narrower rdf:resource="descriptor/12591-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13525-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15460-5">
    <skos:related rdf:resource="descriptor/19596-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18154-4">
    <skos:prefLabel xml:lang="en">Withholding tax</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18154-4">
    <skos:prefLabel xml:lang="de">Quellensteuer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18765-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13264-1">
    <skos:prefLabel xml:lang="en">Transport sector</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13264-1">
    <skos:prefLabel xml:lang="de">Verkehrsgewerbe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10807-4">
    <skos:altLabel xml:lang="de">Internationaler Kapitalmarkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70862">
    <rdfs:label xml:lang="de">Öffentliche Sicherheit und Justizverwaltung</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18366-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18206-4">
    <skos:altLabel xml:lang="de">Kauf-Anzeigenblatt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18206-4">
    <skos:altLabel xml:lang="de">Gratismagazin</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18206-4">
    <skos:altLabel xml:lang="de">Annoncenblatt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11852-1">
    <skos:broader rdf:resource="thsys/73367"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11852-1">
    <skos:broader rdf:resource="thsys/71064"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11852-1">
    <skos:broader rdf:resource="thsys/70870"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11852-1">
    <skos:broader rdf:resource="descriptor/13409-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13831-6">
    <skos:related rdf:resource="descriptor/13305-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13831-6">
    <skos:related rdf:resource="descriptor/13830-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13831-6">
    <skos:related rdf:resource="descriptor/18749-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13831-6">
    <skos:related rdf:resource="descriptor/15101-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11996-5">
    <skos:broader rdf:resource="thsys/71064"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11996-5">
    <skos:broader rdf:resource="thsys/70572"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11996-5">
    <skos:broader rdf:resource="descriptor/11888-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16985-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15972-6">
    <skos:prefLabel xml:lang="en">Legal protection</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15972-6">
    <skos:prefLabel xml:lang="de">Rechtsschutz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71104">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.05.06.03</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16964-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16202-1">
    <skos:prefLabel xml:lang="en">Economic criminal law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16202-1">
    <skos:prefLabel xml:lang="de">Wirtschaftsstrafrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70003">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18602-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12294-4">
    <skos:broader rdf:resource="thsys/71044"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12294-4">
    <skos:broader rdf:resource="descriptor/10008-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10450-5">
    <skos:broader rdf:resource="thsys/71109"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10450-5">
    <skos:broader rdf:resource="thsys/70878"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10450-5">
    <skos:broader rdf:resource="descriptor/10436-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16226-1">
    <skos:scopeNote xml:lang="de">Möglichkeit und Ausmaß der Teilnahme von (nichtorganisierten) sozialen Gruppen an gesellschaftlichen und politischen Entscheidungsprozessen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10315-4">
    <skos:prefLabel xml:lang="en">Factor substitution</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10315-4">
    <skos:prefLabel xml:lang="de">Faktorsubstitution</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12225-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13965-6">
    <skos:related rdf:resource="descriptor/13716-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13965-6">
    <skos:related rdf:resource="descriptor/13283-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11707-3">
    <skos:related rdf:resource="descriptor/11641-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11707-3">
    <skos:related rdf:resource="descriptor/11635-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11707-3">
    <skos:related rdf:resource="descriptor/11595-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14702-5">
    <skos:broader rdf:resource="thsys/70476"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14702-5">
    <skos:broader rdf:resource="descriptor/14599-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15074-3">
    <skos:related rdf:resource="descriptor/19633-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15074-3">
    <skos:related rdf:resource="descriptor/15763-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15074-3">
    <skos:related rdf:resource="descriptor/15759-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11378-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13329-6">
    <skos:broader rdf:resource="thsys/70817"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13329-6">
    <skos:broader rdf:resource="descriptor/13325-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19600-2">
    <skos:scopeNote xml:lang="de">Sowohl für Literatur über Humor (z.B. in der Werbung) als auch für humoristische Darstellungen wirtschaftlicher oder wirtschaftswissenschaftlicher Zusammenhänge.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13064-2">
    <skos:broader rdf:resource="thsys/71001"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13064-2">
    <skos:broader rdf:resource="descriptor/13029-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13064-2">
    <skos:broader rdf:resource="descriptor/18399-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15619-1">
    <skos:altLabel xml:lang="de">Simulationssprache</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15619-1">
    <skos:altLabel xml:lang="de">Computersprache</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16322-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id385">
    <rdf:value xml:lang="de">Für empirische Untersuchungen zur tatsächlichen Zeitverwendung</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13995-4">
    <zbwext:useInsteadNote rdf:nodeID="id276"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13995-4">
    <zbwext:useInsteadNote rdf:nodeID="id275"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11595-2">
    <skos:narrower rdf:resource="descriptor/11598-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11595-2">
    <skos:narrower rdf:resource="descriptor/11590-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18099-0">
    <skos:altLabel xml:lang="de">Private Kulturförderung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18099-0">
    <skos:altLabel xml:lang="de">Mäzenatentum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18099-0">
    <skos:altLabel xml:lang="de">Mäzen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70057">
    <rdfs:label xml:lang="en">Welfare economics</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70057">
    <rdfs:label xml:lang="de">Wohlfahrtstheorie</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10073-2">
    <skos:prefLabel xml:lang="en">Dynamic equilibrium</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10073-2">
    <skos:prefLabel xml:lang="de">Dynamisches Gleichgewicht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11943-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17745-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13460-1">
    <zbwext:useInsteadNote rdf:nodeID="id130"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16182-2">
    <skos:scopeNote xml:lang="de">Supranationales Recht der Europäischen Union für die Gemeinschaftsstaaten.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12603-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10513-0">
    <skos:altLabel xml:lang="de">Wirtschaftliche Entwicklung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10513-0">
    <skos:altLabel xml:lang="de">Entwicklungsprozess</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10513-0">
    <skos:altLabel xml:lang="de">Entwicklungsprozeß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10719-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13731-3">
    <skos:altLabel xml:lang="de">Wertpapierhandelsgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13731-3">
    <skos:altLabel xml:lang="de">Prospekthaftung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13731-3">
    <skos:altLabel xml:lang="de">Finanzmarktförderungsgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15093-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70850">
    <skos:prefLabel xml:lang="de">W.23.02.03  Religionsgemeinschaften</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15637-6">
    <skos:related rdf:resource="descriptor/15646-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12716-4">
    <skos:related rdf:resource="descriptor/19113-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12716-4">
    <skos:related rdf:resource="descriptor/12643-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12716-4">
    <skos:related rdf:resource="descriptor/12717-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12716-4">
    <skos:related rdf:resource="descriptor/10002-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12716-4">
    <skos:related rdf:resource="descriptor/12035-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10981-2">
    <skos:related rdf:resource="descriptor/11847-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10981-2">
    <skos:related rdf:resource="descriptor/11782-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10981-2">
    <skos:related rdf:resource="descriptor/12697-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10981-2">
    <skos:related rdf:resource="descriptor/11790-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10981-2">
    <skos:related rdf:resource="descriptor/18182-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10981-2">
    <skos:related rdf:resource="descriptor/18310-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10981-2">
    <skos:related rdf:resource="descriptor/19251-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13776-2">
    <skos:related rdf:resource="descriptor/13281-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13776-2">
    <skos:related rdf:resource="descriptor/12252-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13776-2">
    <skos:related rdf:resource="descriptor/13777-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13776-2">
    <skos:related rdf:resource="descriptor/11433-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13776-2">
    <skos:related rdf:resource="descriptor/12251-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18657-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71037">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12518-1">
    <skos:altLabel xml:lang="de">Grenzplankosten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12518-1">
    <skos:altLabel xml:lang="de">Flexible Grenzplankostenrechnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70198">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12808-6">
    <skos:altLabel xml:lang="en">Nonstandard activities</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12808-6">
    <skos:altLabel xml:lang="en">Non-standard activities</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12808-6">
    <skos:altLabel xml:lang="de">Produktmanagement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12808-6">
    <skos:altLabel xml:lang="de">Modellpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16526-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10129-1">
    <skos:altLabel xml:lang="de">Realverzinsung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73381">
    <skos:broader rdf:resource="thsys/73382"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12758-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12910-1">
    <skos:broader rdf:resource="thsys/70965"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14889-5">
    <skos:broader rdf:resource="thsys/70066"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14889-5">
    <skos:broader rdf:resource="descriptor/14846-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16340-3">
    <skos:related rdf:resource="descriptor/16280-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16340-3">
    <skos:related rdf:resource="descriptor/16342-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16340-3">
    <skos:related rdf:resource="descriptor/16338-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16340-3">
    <skos:related rdf:resource="descriptor/16246-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16340-3">
    <skos:related rdf:resource="descriptor/13984-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13222-3">
    <skos:altLabel xml:lang="de">Möbelgroßhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13222-3">
    <skos:altLabel xml:lang="de">Möbelfachgeschäft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13222-3">
    <skos:altLabel xml:lang="de">Möbeleinzelhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13222-3">
    <skos:altLabel xml:lang="de">Einrichtungsfachhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13222-3">
    <skos:altLabel xml:lang="de">Einrichtungseinzelhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10249-5">
    <skos:broader rdf:resource="thsys/70234"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10249-5">
    <skos:broader rdf:resource="descriptor/10062-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13669-3">
    <skos:narrower rdf:resource="descriptor/18028-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13669-3">
    <skos:narrower rdf:resource="descriptor/18209-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13669-3">
    <skos:narrower rdf:resource="descriptor/18207-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10315-4">
    <skos:altLabel xml:lang="de">Substitution des Produktionsfaktors</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70418">
    <skos:prefLabel xml:lang="de">P.05  Bergbauprodukte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14199-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19084-1">
    <skos:broader rdf:resource="thsys/70073"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19084-1">
    <skos:broader rdf:resource="descriptor/16815-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14174-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13164-5">
    <skos:altLabel xml:lang="de">Tabakverarbeitende Industrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18156-0">
    <skos:prefLabel xml:lang="en">Ricardian equivalence</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18156-0">
    <skos:prefLabel xml:lang="de">Ricardianisches Äquivalenztheorem</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16692-0">
    <skos:broader rdf:resource="thsys/73355"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16692-0">
    <skos:broader rdf:resource="thsys/71005"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16692-0">
    <skos:broader rdf:resource="descriptor/16564-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17201-5">
    <skos:altLabel xml:lang="x-other">Belgique</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17201-5">
    <skos:altLabel xml:lang="de">Königreich Belgien</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11328-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12945-3">
    <skos:narrower rdf:resource="descriptor/18315-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12945-3">
    <skos:narrower rdf:resource="descriptor/12955-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12945-3">
    <skos:narrower rdf:resource="descriptor/11151-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12945-3">
    <skos:narrower rdf:resource="descriptor/12949-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12945-3">
    <skos:narrower rdf:resource="descriptor/13453-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12945-3">
    <skos:narrower rdf:resource="descriptor/19046-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17588-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18728-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14543-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11501-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://zbw.eu/namespaces/skos-extensions/Descriptor">
    <rdfs:comment xml:lang="en">Among themselves, the Descriptor entries are 
    linked in multiple skos:broader/skos:narrower hierarchies and skos:related 
    connections. To Thsys entries, only skos:broader connections are
    allowed.</rdfs:comment>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14199-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15778-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16815-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13218-1">
    <skos:altLabel xml:lang="de">Feinkosteinzelhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13218-1">
    <skos:altLabel xml:lang="de">Delikatessengeschäft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id232">
    <rdfs:seeAlso rdf:resource="descriptor/18171-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19515-0">
    <skos:prefLabel xml:lang="en">Pay-as-you-go</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19515-0">
    <skos:prefLabel xml:lang="de">Umlageverfahren</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18544-5">
    <skos:related rdf:resource="descriptor/10882-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18544-5">
    <skos:related rdf:resource="descriptor/18794-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18544-5">
    <skos:related rdf:resource="descriptor/10851-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18544-5">
    <skos:related rdf:resource="descriptor/10605-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18544-5">
    <skos:related rdf:resource="descriptor/10686-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19519-6">
    <skos:broader rdf:resource="thsys/73387"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19519-6">
    <skos:broader rdf:resource="thsys/70109"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19519-6">
    <skos:broader rdf:resource="descriptor/15706-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16504-6">
    <skos:broader rdf:resource="thsys/73334"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70968">
    <rdfs:label xml:lang="en">Construction finishings and fittings</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70968">
    <rdfs:label xml:lang="de">Ausbaugewerbe</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19425-1">
    <skos:prefLabel xml:lang="en">Bill tax</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19425-1">
    <skos:prefLabel xml:lang="de">Wechselsteuer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19179-4">
    <skos:prefLabel xml:lang="en">MERCOSUR countries</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19179-4">
    <skos:prefLabel xml:lang="de">MERCOSUR-Staaten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11404-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10342-1">
    <skos:narrower rdf:resource="descriptor/10338-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10342-1">
    <skos:narrower rdf:resource="descriptor/19468-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10342-1">
    <skos:narrower rdf:resource="descriptor/18010-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11754-1">
    <skos:prefLabel xml:lang="en">Economic rent</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11754-1">
    <skos:prefLabel xml:lang="de">Rententheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10930-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18030-1">
    <skos:prefLabel xml:lang="en">Industrial accident</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18030-1">
    <skos:prefLabel xml:lang="de">Störfall</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18405-5">
    <skos:prefLabel xml:lang="en">Steel policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18405-5">
    <skos:prefLabel xml:lang="de">Stahlpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11586-3">
    <skos:altLabel xml:lang="de">Staatsdefizit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11586-3">
    <skos:altLabel xml:lang="de">Staatliches Defizit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11586-3">
    <skos:altLabel xml:lang="de">Budgetdefizit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70997">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.06.01.03.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12905-1">
    <skos:prefLabel xml:lang="en">Construction industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12905-1">
    <skos:prefLabel xml:lang="de">Bauwirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18085-4">
    <skos:prefLabel xml:lang="en">Financial services</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18085-4">
    <skos:prefLabel xml:lang="de">Finanzdienstleistung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12524-6">
    <skos:narrower rdf:resource="descriptor/12518-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12524-6">
    <skos:narrower rdf:resource="descriptor/12515-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10097-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16300-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13617-1">
    <skos:altLabel xml:lang="de">Transport gefährlicher Güter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13617-1">
    <skos:altLabel xml:lang="de">Gefahrgutbeförderung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13617-1">
    <skos:altLabel xml:lang="de">Beförderung gefährlicher Güter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18189-6">
    <skos:altLabel xml:lang="de">Naturgummi</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18189-6">
    <skos:altLabel xml:lang="de">Latex</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18189-6">
    <skos:altLabel xml:lang="de">Guttapercha</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18189-6">
    <skos:altLabel xml:lang="de">Balata</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19266-2">
    <skos:related rdf:resource="descriptor/19267-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19266-2">
    <skos:related rdf:resource="descriptor/18188-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19266-2">
    <skos:related rdf:resource="descriptor/19272-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19266-2">
    <skos:related rdf:resource="descriptor/10124-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19266-2">
    <skos:related rdf:resource="descriptor/10001-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13363-6">
    <skos:altLabel xml:lang="en">Sportsmen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13363-6">
    <skos:altLabel xml:lang="de">Sportlerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13363-6">
    <skos:altLabel xml:lang="de">Berufssportlerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13363-6">
    <skos:altLabel xml:lang="de">Berufssportler</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10745-0">
    <skos:scopeNote xml:lang="de">Währungspolitische Zusammenarbeit mehrerer Staaten.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18306-0">
    <skos:broader rdf:resource="thsys/70333"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18306-0">
    <skos:broader rdf:resource="descriptor/18304-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17608-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20977-1">
    <skos:altLabel xml:lang="de">Modularprinzip</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20977-1">
    <skos:altLabel xml:lang="de">Modularisation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20977-1">
    <skos:altLabel xml:lang="de">Baukastenprinzip</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18834-3">
    <skos:scopeNote xml:lang="de">Management in nicht-erwerbswirtschaftlich ausgerichteten Organisationen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18703-4">
    <skos:altLabel xml:lang="de">EU-Bankrichtlinie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18703-4">
    <skos:altLabel xml:lang="de">EU-Bankenrichtlinie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18703-4">
    <skos:altLabel xml:lang="de">Bankenrichtlinie der Europäischen Union</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18703-4">
    <skos:altLabel xml:lang="de">Bankenrichtlinie der EU</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13694-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13702-3">
    <skos:prefLabel xml:lang="en">Agricultural bank</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13702-3">
    <skos:prefLabel xml:lang="de">Landwirtschaftsbank</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18888-1">
    <skos:prefLabel xml:lang="en">Statistics of education</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18888-1">
    <skos:prefLabel xml:lang="de">Bildungsstatistik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11372-1">
    <skos:broader rdf:resource="thsys/71052"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11372-1">
    <skos:broader rdf:resource="thsys/70844"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11372-1">
    <skos:broader rdf:resource="descriptor/13349-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11372-1">
    <skos:broader rdf:resource="descriptor/11374-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14971-6">
    <skos:related rdf:resource="descriptor/14083-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14971-6">
    <skos:related rdf:resource="descriptor/14124-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14971-6">
    <skos:related rdf:resource="descriptor/14065-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14971-6">
    <skos:related rdf:resource="descriptor/14064-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11044-4">
    <skos:altLabel xml:lang="de">Volkseigentum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11044-4">
    <skos:altLabel xml:lang="de">Sozialistisches Eigentum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11044-4">
    <skos:altLabel xml:lang="de">Kollektiveigentum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11044-4">
    <skos:altLabel xml:lang="de">Gemeineigentum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12201-2">
    <skos:narrower rdf:resource="descriptor/11306-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18100-6">
    <skos:altLabel xml:lang="en">Industrial goods marketing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18100-6">
    <skos:altLabel xml:lang="de">Industriegütermarketing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13807-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10172-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12435-5">
    <skos:related rdf:resource="descriptor/12436-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12435-5">
    <skos:related rdf:resource="descriptor/12391-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12435-5">
    <skos:related rdf:resource="descriptor/12425-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14883-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19200-4">
    <skos:related rdf:resource="descriptor/10334-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19200-4">
    <skos:related rdf:resource="descriptor/19490-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19200-4">
    <skos:related rdf:resource="descriptor/18129-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19200-4">
    <skos:related rdf:resource="descriptor/19197-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12571-4">
    <skos:narrower rdf:resource="descriptor/12570-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14279-4">
    <skos:altLabel xml:lang="de">Spezialstahl</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14279-4">
    <skos:altLabel xml:lang="de">Rostfreier Stahl</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14279-4">
    <skos:altLabel xml:lang="de">Nichtrostender Stahl</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11391-4">
    <skos:prefLabel xml:lang="en">Further training</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11391-4">
    <skos:prefLabel xml:lang="de">Weiterbildung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11409-4">
    <skos:related rdf:resource="descriptor/11397-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11409-4">
    <skos:related rdf:resource="descriptor/11407-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11409-4">
    <skos:related rdf:resource="descriptor/11222-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11409-4">
    <skos:related rdf:resource="descriptor/18884-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11554-2">
    <skos:prefLabel xml:lang="en">Public Expenditure</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11554-2">
    <skos:prefLabel xml:lang="de">Öffentliche Ausgaben</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13164-5">
    <zbwext:useInsteadNote rdf:nodeID="id325"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11857-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18749-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19101-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13413-3">
    <skos:broader rdf:resource="thsys/73380"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13413-3">
    <skos:broader rdf:resource="thsys/70871"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13413-3">
    <skos:broader rdf:resource="descriptor/11722-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10229-4">
    <skos:broader rdf:resource="thsys/72182"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10229-4">
    <skos:broader rdf:resource="thsys/70234"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10229-4">
    <skos:broader rdf:resource="descriptor/15064-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10711-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14964-3">
    <skos:broader rdf:resource="thsys/73302"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14964-3">
    <skos:broader rdf:resource="descriptor/14129-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14964-3">
    <skos:broader rdf:resource="descriptor/14931-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12376-2">
    <skos:prefLabel xml:lang="en">Intangible assets</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12376-2">
    <skos:prefLabel xml:lang="de">Immaterielles Anlagevermögen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18363-2">
    <skos:related rdf:resource="descriptor/14186-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18363-2">
    <skos:related rdf:resource="descriptor/14407-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18414-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19400-3">
    <skos:altLabel xml:lang="de">Swiftsches Steuereinmaleins</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19400-3">
    <skos:altLabel xml:lang="de">Laffer Kurve</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11921-1">
    <skos:prefLabel xml:lang="en">Land economics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11921-1">
    <skos:prefLabel xml:lang="de">Bodenökonomik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18251-6">
    <skos:broader rdf:resource="thsys/70423"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18251-6">
    <skos:broader rdf:resource="thsys/70254"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18251-6">
    <skos:broader rdf:resource="descriptor/14537-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18251-6">
    <skos:broader rdf:resource="descriptor/18103-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14844-6">
    <skos:altLabel xml:lang="de">Textilschlauch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14844-6">
    <skos:altLabel xml:lang="de">Textilsack</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14844-6">
    <skos:altLabel xml:lang="de">Textilnetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14844-6">
    <skos:altLabel xml:lang="de">Seil</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14844-6">
    <skos:altLabel xml:lang="de">Reinigungstuch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14844-6">
    <skos:altLabel xml:lang="de">Putztuch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14844-6">
    <skos:altLabel xml:lang="de">Filz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10787-5">
    <skos:broader rdf:resource="thsys/71087"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10787-5">
    <skos:broader rdf:resource="descriptor/10791-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#closeMatch">
    <rdfs:isDefinedBy rdf:resource="http://www.w3.org/2004/02/skos/core"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13829-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13250-5">
    <skos:related rdf:resource="descriptor/14439-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19071-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17931-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17696-1">
    <skos:prefLabel xml:lang="en">Burkina Faso</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17696-1">
    <skos:prefLabel xml:lang="de">Burkina Faso</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18880-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14511-5">
    <skos:prefLabel xml:lang="en">Vacuum cleaner</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14511-5">
    <skos:prefLabel xml:lang="de">Staubsauger</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18590-5">
    <skos:broader rdf:resource="thsys/70936"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18590-5">
    <skos:broader rdf:resource="descriptor/18624-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11341-5">
    <skos:broader rdf:resource="thsys/70278"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11341-5">
    <skos:broader rdf:resource="thsys/70245"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11341-5">
    <skos:broader rdf:resource="descriptor/12567-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11341-5">
    <skos:broader rdf:resource="descriptor/11345-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15708-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19612-2">
    <skos:prefLabel xml:lang="en">Country size</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19612-2">
    <skos:prefLabel xml:lang="de">Landesgröße</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11223-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19189-1">
    <skos:prefLabel xml:lang="en">Maize market</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19189-1">
    <skos:prefLabel xml:lang="de">Maismarkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11379-1">
    <skos:prefLabel xml:lang="en">University study</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11379-1">
    <skos:prefLabel xml:lang="de">Studium</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73332">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13460-1">
    <skos:altLabel xml:lang="de">Handlungswissenschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13460-1">
    <skos:altLabel xml:lang="de">Handelswissenschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13460-1">
    <skos:altLabel xml:lang="de">Handelsökonomie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13460-1">
    <skos:altLabel xml:lang="de">Empirische Handelsforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13745-6">
    <skos:broader rdf:resource="thsys/71105"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13745-6">
    <skos:broader rdf:resource="thsys/71045"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13745-6">
    <skos:broader rdf:resource="descriptor/12234-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70005">
    <skos:prefLabel xml:lang="de">B.03  Betriebswirtschaftliches Rechnungs- und Prüfungswesen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13369-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id326">
    <rdfs:seeAlso rdf:resource="descriptor/18649-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16630-1">
    <skos:related rdf:resource="descriptor/15711-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16630-1">
    <skos:related rdf:resource="descriptor/13397-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16630-1">
    <skos:related rdf:resource="descriptor/16700-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16630-1">
    <skos:related rdf:resource="descriptor/15924-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19319-0">
    <skos:altLabel xml:lang="en">New protectionism</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19319-0">
    <skos:altLabel xml:lang="de">Neuer Protektionismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19188-3">
    <skos:prefLabel xml:lang="en">Grain market</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19188-3">
    <skos:prefLabel xml:lang="de">Getreidemarkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11452-3">
    <skos:prefLabel xml:lang="en">Velocity of money</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11452-3">
    <skos:prefLabel xml:lang="de">Geldumlaufgeschwindigkeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18138-2">
    <skos:altLabel xml:lang="en">Self-fulfilling prophecy</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17713-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#broadMatch">
    <rdf:type rdf:resource="http://www.w3.org/1999/02/22-rdf-syntax-ns#Property"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#broadMatch">
    <rdf:type rdf:resource="http://www.w3.org/2002/07/owl#ObjectProperty"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11582-4">
    <skos:related rdf:resource="descriptor/11560-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11582-4">
    <skos:related rdf:resource="descriptor/18447-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11692-4">
    <skos:prefLabel xml:lang="en">Tax incidence</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11692-4">
    <skos:prefLabel xml:lang="de">Steuerinzidenz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15889-0">
    <skos:related rdf:resource="descriptor/18377-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15889-0">
    <skos:related rdf:resource="descriptor/19619-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15889-0">
    <skos:related rdf:resource="descriptor/16295-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15889-0">
    <skos:related rdf:resource="descriptor/16524-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15889-0">
    <skos:related rdf:resource="descriptor/19840-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15889-0">
    <skos:related rdf:resource="descriptor/13815-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15889-0">
    <skos:related rdf:resource="descriptor/19485-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14385-5">
    <skos:altLabel xml:lang="de">Pappemaschine</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15918-5">
    <skos:broader rdf:resource="thsys/73358"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15918-5">
    <skos:broader rdf:resource="thsys/70142"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15918-5">
    <skos:broader rdf:resource="descriptor/15714-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15918-5">
    <skos:broader rdf:resource="descriptor/15909-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19693-4">
    <skos:broader rdf:resource="thsys/73379"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19693-4">
    <skos:broader rdf:resource="descriptor/10006-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11415-2">
    <skos:prefLabel xml:lang="en">Money</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11415-2">
    <skos:prefLabel xml:lang="de">Geld</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11567-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19347-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10463-3">
    <skos:related rdf:resource="descriptor/12587-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10463-3">
    <skos:related rdf:resource="descriptor/18117-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10463-3">
    <skos:related rdf:resource="descriptor/12105-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10463-3">
    <skos:related rdf:resource="descriptor/19116-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10463-3">
    <skos:related rdf:resource="descriptor/19152-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10463-3">
    <skos:related rdf:resource="descriptor/19096-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19402-6">
    <skos:related rdf:resource="descriptor/19403-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16732-0">
    <skos:scopeNote xml:lang="de">Jener Teil der Philosophie, der sich mit den Grundlagen und Methoden der Wissenschaft befasst.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10219-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18425-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13393-4">
    <skos:broader rdf:resource="thsys/73388"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13393-4">
    <skos:broader rdf:resource="thsys/70850"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13393-4">
    <skos:broader rdf:resource="descriptor/13399-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11613-5">
    <skos:narrower rdf:resource="descriptor/11618-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11613-5">
    <skos:narrower rdf:resource="descriptor/11611-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11613-5">
    <skos:narrower rdf:resource="descriptor/11610-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18831-2">
    <skos:altLabel xml:lang="en">Non-profit tax law</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70530">
    <skos:prefLabel xml:lang="de">V.03.01  Gesamtwirtschaftliche Investition</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18670-0">
    <skos:related rdf:resource="descriptor/12299-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19684-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14814-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17989-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17946-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16176-4">
    <skos:related rdf:resource="descriptor/11848-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16176-4">
    <skos:related rdf:resource="descriptor/16019-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16176-4">
    <skos:related rdf:resource="descriptor/16039-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16176-4">
    <skos:related rdf:resource="descriptor/11580-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16176-4">
    <skos:related rdf:resource="descriptor/16036-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16176-4">
    <skos:related rdf:resource="descriptor/11870-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16176-4">
    <skos:related rdf:resource="descriptor/12020-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16176-4">
    <skos:related rdf:resource="descriptor/12905-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16176-4">
    <skos:related rdf:resource="descriptor/18507-4"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id282">
    <rdfs:seeAlso rdf:resource="descriptor/18828-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18930-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12942-2">
    <skos:prefLabel xml:lang="en">Vegetable crops</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12942-2">
    <skos:prefLabel xml:lang="de">Gemüsebau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11969-1">
    <skos:prefLabel xml:lang="en">Slum</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11969-1">
    <skos:prefLabel xml:lang="de">Slum</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71085">
    <skos:broader rdf:resource="thsys/70091"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17723-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13647-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19575-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19376-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11311-0">
    <skos:prefLabel xml:lang="en">Works council</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11311-0">
    <skos:prefLabel xml:lang="de">Betriebsrat</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14078-0">
    <skos:altLabel xml:lang="en">Canola</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="../stw">
    <skos:prefLabel xml:lang="en">STW Thesaurus for Economics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="../stw">
    <skos:prefLabel xml:lang="de">Standard-Thesaurus Wirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70463">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">P.12.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11258-6">
    <skos:prefLabel xml:lang="en">Seasonal workers</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11258-6">
    <skos:prefLabel xml:lang="de">Saisonarbeitskräfte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12535-1">
    <skos:related rdf:resource="descriptor/19153-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12535-1">
    <skos:related rdf:resource="descriptor/10011-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12535-1">
    <skos:related rdf:resource="descriptor/12531-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12535-1">
    <skos:related rdf:resource="descriptor/10094-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12535-1">
    <skos:related rdf:resource="descriptor/12520-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12535-1">
    <skos:related rdf:resource="descriptor/12490-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11286-1">
    <skos:prefLabel xml:lang="en">Workers</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11286-1">
    <skos:prefLabel xml:lang="de">Arbeiter</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12241-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15912-3">
    <skos:prefLabel xml:lang="en">Population forecast</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15912-3">
    <skos:prefLabel xml:lang="de">Bevölkerungsprognose</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19573-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13704-6">
    <skos:prefLabel xml:lang="en">Real estate investment trust</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13704-6">
    <skos:prefLabel xml:lang="de">Immobilienfonds</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13349-0">
    <skos:altLabel xml:lang="de">Wissenschaftliche Arbeitskräfte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13349-0">
    <skos:altLabel xml:lang="de">Wissenschaftlerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13349-0">
    <skos:altLabel xml:lang="de">Statistikerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13349-0">
    <skos:altLabel xml:lang="de">Statistiker</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13349-0">
    <skos:altLabel xml:lang="de">Privatgelehrte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13349-0">
    <skos:altLabel xml:lang="de">Forscherinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13349-0">
    <skos:altLabel xml:lang="de">Forscher</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10987-4">
    <skos:prefLabel xml:lang="en">Economic policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10987-4">
    <skos:prefLabel xml:lang="de">Wirtschaftspolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id216">
    <rdfs:seeAlso rdf:resource="descriptor/10278-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19271-2">
    <skos:altLabel xml:lang="en">CAPM (Capital Asset Pricing Model)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19271-2">
    <skos:altLabel xml:lang="en">Capital asset pricing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19271-2">
    <skos:altLabel xml:lang="en">Asset pricing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19271-2">
    <skos:altLabel xml:lang="de">CCAPM</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19271-2">
    <skos:altLabel xml:lang="de">Capital Asset Pricing Modell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13658-1">
    <skos:related rdf:resource="descriptor/12829-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13658-1">
    <skos:related rdf:resource="descriptor/13278-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13658-1">
    <skos:related rdf:resource="descriptor/14004-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13658-1">
    <skos:related rdf:resource="descriptor/13661-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13658-1">
    <skos:related rdf:resource="descriptor/13347-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13658-1">
    <skos:related rdf:resource="descriptor/18932-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13658-1">
    <skos:related rdf:resource="descriptor/13660-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13658-1">
    <skos:related rdf:resource="descriptor/16516-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13658-1">
    <skos:related rdf:resource="descriptor/18930-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13658-1">
    <skos:related rdf:resource="descriptor/15798-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13444-6">
    <skos:related rdf:resource="descriptor/10505-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19008-3">
    <skos:prefLabel xml:lang="en">Staff survey</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19008-3">
    <skos:prefLabel xml:lang="de">Mitarbeiterbefragung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17982-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18068-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11484-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70416">
    <skos:prefLabel xml:lang="de">P.13  Holzprodukte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12251-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18214-5">
    <skos:related rdf:resource="descriptor/18213-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18214-5">
    <skos:related rdf:resource="descriptor/18006-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18214-5">
    <skos:related rdf:resource="descriptor/14927-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13845-2">
    <skos:related rdf:resource="descriptor/13865-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13845-2">
    <skos:related rdf:resource="descriptor/13305-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13845-2">
    <skos:related rdf:resource="descriptor/13849-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13845-2">
    <skos:related rdf:resource="descriptor/13830-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13845-2">
    <skos:related rdf:resource="descriptor/13829-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11670-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18318-0">
    <skos:related rdf:resource="descriptor/13443-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70233">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.03.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14015-3">
    <skos:narrower rdf:resource="descriptor/13348-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16739-0">
    <skos:narrower rdf:resource="descriptor/10156-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16739-0">
    <skos:narrower rdf:resource="descriptor/16743-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16739-0">
    <skos:narrower rdf:resource="descriptor/16740-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16739-0">
    <skos:narrower rdf:resource="descriptor/16738-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16739-0">
    <skos:narrower rdf:resource="descriptor/16737-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17638-1">
    <skos:altLabel xml:lang="x-other">Türkiye</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17638-1">
    <skos:altLabel xml:lang="de">Republik Türkei</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18119-6">
    <zbwext:useInsteadNote rdf:nodeID="id334"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11275-6">
    <skos:broader rdf:resource="thsys/70052"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11275-6">
    <skos:broader rdf:resource="descriptor/15924-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11275-6">
    <skos:broader rdf:resource="descriptor/11262-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17571-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11541-4">
    <skos:prefLabel xml:lang="en">State audit institution</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11541-4">
    <skos:prefLabel xml:lang="de">Rechnungsprüfungsbehörde</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18133-5">
    <skos:broader rdf:resource="thsys/70361"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18133-5">
    <skos:broader rdf:resource="descriptor/18012-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10241-0">
    <skos:scopeNote xml:lang="de">Gesamtwirtschaftliches Angebot auf dem Gütermarkt.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19272-0">
    <skos:related rdf:resource="descriptor/19266-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19272-0">
    <skos:related rdf:resource="descriptor/19273-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19272-0">
    <skos:related rdf:resource="descriptor/10124-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19599-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18575-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11154-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19565-6">
    <skos:narrower rdf:resource="descriptor/21282-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19565-6">
    <skos:narrower rdf:resource="descriptor/10951-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19565-6">
    <skos:narrower rdf:resource="descriptor/19164-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19565-6">
    <skos:narrower rdf:resource="descriptor/10909-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19565-6">
    <skos:narrower rdf:resource="descriptor/12793-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19565-6">
    <skos:narrower rdf:resource="descriptor/10908-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19565-6">
    <skos:narrower rdf:resource="descriptor/18541-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19565-6">
    <skos:narrower rdf:resource="descriptor/10945-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19565-6">
    <skos:narrower rdf:resource="descriptor/10947-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10329-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16301-6">
    <skos:broader rdf:resource="thsys/73376"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16301-6">
    <skos:broader rdf:resource="thsys/71055"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16301-6">
    <skos:broader rdf:resource="descriptor/13995-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70120">
    <skos:broader rdf:resource="thsys/70012"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18674-6">
    <skos:related rdf:resource="descriptor/12230-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18674-6">
    <skos:related rdf:resource="descriptor/10021-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18674-6">
    <skos:related rdf:resource="descriptor/10020-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18674-6">
    <skos:related rdf:resource="descriptor/12210-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18674-6">
    <skos:related rdf:resource="descriptor/12280-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18674-6">
    <skos:related rdf:resource="descriptor/12214-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18674-6">
    <skos:related rdf:resource="descriptor/13714-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71041">
    <skos:broader rdf:resource="thsys/70005"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11066-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14543-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14025-0">
    <skos:prefLabel xml:lang="en">Dental products</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14025-0">
    <skos:prefLabel xml:lang="de">Dentalprodukt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10510-6">
    <skos:prefLabel xml:lang="en">Urban development</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10510-6">
    <skos:prefLabel xml:lang="de">Stadtentwicklung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13270-6">
    <zbwext:useInsteadNote rdf:nodeID="id254"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11570-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19002-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10985-1">
    <skos:prefLabel xml:lang="en">Nationalization (State public property)</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10985-1">
    <skos:prefLabel xml:lang="de">Verstaatlichung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13363-6">
    <skos:broader rdf:resource="thsys/71055"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13363-6">
    <skos:broader rdf:resource="thsys/70819"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13363-6">
    <skos:broader rdf:resource="descriptor/18535-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11152-1">
    <skos:related rdf:resource="descriptor/18847-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11152-1">
    <skos:related rdf:resource="descriptor/11168-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10406-1">
    <skos:related rdf:resource="descriptor/16011-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10406-1">
    <skos:related rdf:resource="descriptor/11239-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10406-1">
    <skos:related rdf:resource="descriptor/10996-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11143-2">
    <skos:related rdf:resource="descriptor/11142-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11143-2">
    <skos:related rdf:resource="descriptor/11141-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11143-2">
    <skos:related rdf:resource="descriptor/11061-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11143-2">
    <skos:related rdf:resource="descriptor/12085-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71075">
    <rdfs:label xml:lang="en">Tax policy</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71075">
    <rdfs:label xml:lang="de">Steuerpolitik</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71030">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14953-1">
    <skos:related rdf:resource="descriptor/14995-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13773-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12785-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19558-3">
    <skos:altLabel xml:lang="en">Travel cost approach</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19558-3">
    <skos:altLabel xml:lang="de">Reisekostenmodell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10682-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10011-3">
    <skos:prefLabel xml:lang="en">Costs</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10011-3">
    <skos:prefLabel xml:lang="de">Kosten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10036-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71064">
    <skos:prefLabel xml:lang="en">V.11.06  Urban economics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71064">
    <skos:prefLabel xml:lang="de">V.11.06  Stadtökonomik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73377">
    <rdfs:label xml:lang="de">Politische Ziele und Werte</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19757-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16729-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13095-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71008">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.05.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10548-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id126">
    <rdfs:seeAlso rdf:resource="descriptor/19365-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18197-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10038-4">
    <skos:broader rdf:resource="thsys/70180"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10038-4">
    <skos:broader rdf:resource="descriptor/10144-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18962-1">
    <skos:altLabel xml:lang="en">Reverse supply chains</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18962-1">
    <skos:altLabel xml:lang="en">Closed-loop supply chain management</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18962-1">
    <skos:altLabel xml:lang="de">Rückführlogistik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18962-1">
    <skos:altLabel xml:lang="de">Recycling-Management</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18962-1">
    <skos:altLabel xml:lang="de">Produktrückgabe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18962-1">
    <skos:altLabel xml:lang="de">Betriebliche Kreislaufwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18962-1">
    <skos:altLabel xml:lang="de">Altproduktrückführung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12372-3">
    <skos:broader rdf:resource="thsys/71042"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12372-3">
    <skos:broader rdf:resource="thsys/70397"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13427-6">
    <skos:broader rdf:resource="thsys/70461"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13427-6">
    <skos:broader rdf:resource="descriptor/14596-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70650">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19515-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14630-4">
    <skos:altLabel xml:lang="de">Textilchemikalie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14630-4">
    <skos:altLabel xml:lang="de">Textilausrüstungsmittel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14630-4">
    <skos:altLabel xml:lang="de">Pelzhilfsmittel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14630-4">
    <skos:altLabel xml:lang="de">Lederhilfsmittel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14630-4">
    <skos:altLabel xml:lang="de">Gerbstoff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14630-4">
    <skos:altLabel xml:lang="de">Appretiermittel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14885-6">
    <skos:altLabel xml:lang="de">Zwirn</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14885-6">
    <skos:altLabel xml:lang="de">Texturgarn</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14885-6">
    <skos:altLabel xml:lang="de">Stretchgarn</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14885-6">
    <skos:altLabel xml:lang="de">Streichgarn</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14885-6">
    <skos:altLabel xml:lang="de">Nähgarn</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14885-6">
    <skos:altLabel xml:lang="de">Kammgarn</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14885-6">
    <skos:altLabel xml:lang="de">Handstrickgarn</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14885-6">
    <skos:altLabel xml:lang="de">Handarbeitsgarn</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14885-6">
    <skos:altLabel xml:lang="de">Gespinst</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12018-0">
    <skos:related rdf:resource="descriptor/11844-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12018-0">
    <skos:related rdf:resource="descriptor/18747-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12018-0">
    <skos:related rdf:resource="descriptor/11659-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12018-0">
    <skos:related rdf:resource="descriptor/12020-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12018-0">
    <skos:related rdf:resource="descriptor/12015-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12018-0">
    <skos:related rdf:resource="descriptor/11801-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15745-3">
    <skos:related rdf:resource="descriptor/15759-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15745-3">
    <skos:related rdf:resource="descriptor/14047-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15745-3">
    <skos:related rdf:resource="descriptor/18381-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15745-3">
    <skos:related rdf:resource="descriptor/10411-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15745-3">
    <skos:related rdf:resource="descriptor/19488-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70058">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.11</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11498-0">
    <skos:broader rdf:resource="thsys/70355"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12268-5">
    <skos:altLabel xml:lang="en">Banking investments</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12268-5">
    <skos:altLabel xml:lang="de">Anteilsbesitz der Banken</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12978-2">
    <skos:broader rdf:resource="thsys/71016"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12978-2">
    <skos:broader rdf:resource="descriptor/12975-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14733-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12183-6">
    <skos:prefLabel xml:lang="en">Limited company &amp;amp; Co</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12183-6">
    <skos:prefLabel xml:lang="de">GmbH &amp;amp; Co</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id115">
    <rdfs:seeAlso rdf:resource="descriptor/10770-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73395">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.06.02.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10635-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16768-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19523-1">
    <skos:altLabel xml:lang="en">Q-Theory of investment</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13818-5">
    <skos:prefLabel xml:lang="en">Insurance supervision</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13818-5">
    <skos:prefLabel xml:lang="de">Versicherungsaufsicht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17931-3">
    <skos:narrower rdf:resource="descriptor/17934-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17931-3">
    <skos:narrower rdf:resource="descriptor/17933-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17931-3">
    <skos:narrower rdf:resource="descriptor/17932-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17931-3">
    <skos:narrower rdf:resource="descriptor/17928-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17931-3">
    <skos:narrower rdf:resource="descriptor/17927-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17931-3">
    <skos:narrower rdf:resource="descriptor/17923-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17931-3">
    <skos:narrower rdf:resource="descriptor/17922-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17931-3">
    <skos:narrower rdf:resource="descriptor/17920-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17931-3">
    <skos:narrower rdf:resource="descriptor/17919-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17931-3">
    <skos:narrower rdf:resource="descriptor/17915-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17931-3">
    <skos:narrower rdf:resource="descriptor/17914-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17931-3">
    <skos:narrower rdf:resource="descriptor/17911-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18868-0">
    <skos:scopeNote xml:lang="de">Wissenschaft der Lehr- und Bildungsinhalte sowie deren Vermittlung</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11738-6">
    <skos:altLabel xml:lang="de">Gini Ratio</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id272">
    <rdf:value xml:lang="de">Für die ökonomische Theorie der Rationierung</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18235-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71016">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.04.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11902-5">
    <skos:broader rdf:resource="thsys/71064"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11902-5">
    <skos:broader rdf:resource="thsys/70572"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11902-5">
    <skos:broader rdf:resource="descriptor/19647-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71098">
    <skos:narrower rdf:resource="descriptor/19319-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71098">
    <skos:narrower rdf:resource="descriptor/19325-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71098">
    <skos:narrower rdf:resource="descriptor/18818-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71098">
    <skos:narrower rdf:resource="descriptor/19320-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71098">
    <skos:narrower rdf:resource="descriptor/19310-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71098">
    <skos:narrower rdf:resource="descriptor/19322-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71098">
    <skos:narrower rdf:resource="descriptor/19323-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71098">
    <skos:narrower rdf:resource="descriptor/10706-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71098">
    <skos:narrower rdf:resource="descriptor/10704-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71098">
    <skos:narrower rdf:resource="descriptor/10725-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71098">
    <skos:narrower rdf:resource="descriptor/19321-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71098">
    <skos:narrower rdf:resource="descriptor/10824-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71098">
    <skos:narrower rdf:resource="descriptor/10570-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71098">
    <skos:narrower rdf:resource="descriptor/10701-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71098">
    <skos:narrower rdf:resource="descriptor/10699-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71098">
    <skos:narrower rdf:resource="descriptor/10697-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71098">
    <skos:narrower rdf:resource="descriptor/10696-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71098">
    <skos:narrower rdf:resource="descriptor/19318-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71098">
    <skos:narrower rdf:resource="descriptor/19414-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71098">
    <skos:narrower rdf:resource="descriptor/10693-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71098">
    <skos:narrower rdf:resource="descriptor/10565-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71098">
    <skos:narrower rdf:resource="descriptor/10692-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71098">
    <skos:narrower rdf:resource="descriptor/10691-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71098">
    <skos:narrower rdf:resource="descriptor/11649-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71098">
    <skos:narrower rdf:resource="descriptor/10690-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71098">
    <skos:narrower rdf:resource="descriptor/10780-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71098">
    <skos:narrower rdf:resource="descriptor/19315-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71098">
    <skos:narrower rdf:resource="descriptor/10606-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71098">
    <skos:narrower rdf:resource="descriptor/10682-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71098">
    <skos:narrower rdf:resource="descriptor/10827-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71098">
    <skos:narrower rdf:resource="descriptor/10687-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71098">
    <skos:narrower rdf:resource="descriptor/10686-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71098">
    <skos:narrower rdf:resource="descriptor/19317-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71098">
    <skos:narrower rdf:resource="descriptor/10685-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71098">
    <skos:narrower rdf:resource="descriptor/10684-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71098">
    <skos:narrower rdf:resource="descriptor/16084-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71098">
    <skos:narrower rdf:resource="descriptor/10683-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71098">
    <skos:narrower rdf:resource="descriptor/19314-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71098">
    <skos:narrower rdf:resource="thsys/71096"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71098">
    <skos:narrower rdf:resource="thsys/71097"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14001-0">
    <skos:altLabel xml:lang="de">Informationswissenschaftlerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14001-0">
    <skos:altLabel xml:lang="de">Informationswissenschaftler</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14001-0">
    <skos:altLabel xml:lang="de">Informationswirtinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14001-0">
    <skos:altLabel xml:lang="de">Informationswirte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14001-0">
    <skos:altLabel xml:lang="de">Informationsfachleute</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14001-0">
    <skos:altLabel xml:lang="de">Dokumentationsberufe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14001-0">
    <skos:altLabel xml:lang="de">Dokumentationsassistentinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14001-0">
    <skos:altLabel xml:lang="de">Dokumentationsassistenten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14001-0">
    <skos:altLabel xml:lang="de">Dokumentarinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14001-0">
    <skos:altLabel xml:lang="de">Dokumentare</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14001-0">
    <skos:altLabel xml:lang="de">BID-Berufe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14001-0">
    <skos:altLabel xml:lang="de">Bibliotheksassistentinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14001-0">
    <skos:altLabel xml:lang="de">Bibliotheksassistenten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14001-0">
    <skos:altLabel xml:lang="de">Bibliothekarinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14001-0">
    <skos:altLabel xml:lang="de">Bibliothekare</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14001-0">
    <skos:altLabel xml:lang="de">Archivarinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14001-0">
    <skos:altLabel xml:lang="de">Archivare</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15104-6">
    <skos:broader rdf:resource="thsys/72181"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15104-6">
    <skos:broader rdf:resource="thsys/70355"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15104-6">
    <skos:broader rdf:resource="descriptor/15076-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19130-6">
    <skos:prefLabel xml:lang="en">NAFTA countries</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19130-6">
    <skos:prefLabel xml:lang="de">NAFTA-Staaten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18333-4">
    <skos:broader rdf:resource="thsys/71066"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18333-4">
    <skos:broader rdf:resource="thsys/71020"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18333-4">
    <skos:broader rdf:resource="descriptor/12018-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18333-4">
    <skos:broader rdf:resource="descriptor/11809-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19053-5">
    <skos:altLabel xml:lang="de">Rollstuhl</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13824-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11197-0">
    <skos:broader rdf:resource="thsys/70052"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11197-0">
    <skos:broader rdf:resource="descriptor/19491-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11197-0">
    <skos:broader rdf:resource="descriptor/11180-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13269-5">
    <skos:broader rdf:resource="thsys/73341"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13269-5">
    <skos:broader rdf:resource="thsys/71037"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13269-5">
    <skos:broader rdf:resource="descriptor/15599-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11536-4">
    <skos:related rdf:resource="descriptor/11640-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11536-4">
    <skos:related rdf:resource="descriptor/12880-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11536-4">
    <skos:related rdf:resource="descriptor/11535-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11536-4">
    <skos:related rdf:resource="descriptor/11525-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11536-4">
    <skos:related rdf:resource="descriptor/11728-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17465-6">
    <skos:altLabel xml:lang="x-other">Respublika Belarus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17465-6">
    <skos:altLabel xml:lang="x-other">Belorusskaja SSR</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17465-6">
    <skos:altLabel xml:lang="de">Weißrußland</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17465-6">
    <skos:altLabel xml:lang="de">Republik Weißrussland</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17465-6">
    <skos:altLabel xml:lang="de">Republik Weißrußland</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17465-6">
    <skos:altLabel xml:lang="de">Belorussland</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17465-6">
    <skos:altLabel xml:lang="de">Belorußland</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18402-4">
    <skos:altLabel xml:lang="de">Schweißtechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18402-4">
    <skos:altLabel xml:lang="de">Schmelztechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18402-4">
    <skos:altLabel xml:lang="de">Hüttenkunde</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18402-4">
    <skos:altLabel xml:lang="de">Gießereitechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14285-2">
    <skos:altLabel xml:lang="de">Rohstahl</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14285-2">
    <skos:altLabel xml:lang="de">Roheisen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14285-2">
    <skos:altLabel xml:lang="de">Eisen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12241-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12480-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70987">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16041-6">
    <skos:narrower rdf:resource="descriptor/19842-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14296-4">
    <skos:prefLabel xml:lang="en">Precious metal</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14296-4">
    <skos:prefLabel xml:lang="de">Edelmetall</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18770-3">
    <skos:prefLabel xml:lang="en">Business Database</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18770-3">
    <skos:prefLabel xml:lang="de">Wirtschaftsdatenbank</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19222-1">
    <skos:broader rdf:resource="thsys/70178"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14331-0">
    <skos:related rdf:resource="descriptor/18429-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14331-0">
    <skos:related rdf:resource="descriptor/12996-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14331-0">
    <skos:related rdf:resource="descriptor/18408-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16315-2">
    <skos:scopeNote xml:lang="de">Aufteilung der Staatsgewalt in gesetzgebende, vollziehende und rechtsprechende Gewalt.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18227-3">
    <skos:altLabel xml:lang="x-other">Hrvatska</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18227-3">
    <skos:altLabel xml:lang="de">Republik Kroatien</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19369-6">
    <skos:narrower rdf:resource="descriptor/18323-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15621-0">
    <skos:related rdf:resource="descriptor/15624-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15621-0">
    <skos:related rdf:resource="descriptor/18761-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15621-0">
    <skos:related rdf:resource="descriptor/15610-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15621-0">
    <skos:related rdf:resource="descriptor/13390-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15621-0">
    <skos:related rdf:resource="descriptor/18766-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15621-0">
    <skos:related rdf:resource="descriptor/15606-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15621-0">
    <skos:related rdf:resource="descriptor/18208-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15621-0">
    <skos:related rdf:resource="descriptor/18758-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15621-0">
    <skos:related rdf:resource="descriptor/14751-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10198-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11734-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17588-4">
    <skos:broader rdf:resource="thsys/70327"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17588-4">
    <skos:broader rdf:resource="descriptor/17589-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17588-4">
    <skos:broader rdf:resource="descriptor/17979-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15738-0">
    <skos:related rdf:resource="descriptor/15772-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15738-0">
    <skos:related rdf:resource="descriptor/15735-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10897-5">
    <skos:broader rdf:resource="thsys/71034"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10897-5">
    <skos:broader rdf:resource="thsys/70212"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10897-5">
    <skos:broader rdf:resource="descriptor/10911-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12681-4">
    <skos:prefLabel xml:lang="en">Storeroom system</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12681-4">
    <skos:prefLabel xml:lang="de">Lagersystem</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18783-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18361-6">
    <skos:broader rdf:resource="thsys/71015"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18361-6">
    <skos:broader rdf:resource="descriptor/12010-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18605-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13980-3">
    <skos:prefLabel xml:lang="en">Aviation professions</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13980-3">
    <skos:prefLabel xml:lang="de">Luftverkehrsberufe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15067-0">
    <skos:prefLabel xml:lang="en">Queueing theory</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15067-0">
    <skos:prefLabel xml:lang="de">Warteschlangentheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10677-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19684-5">
    <skos:altLabel xml:lang="de">Territorialer Anspruch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19684-5">
    <skos:altLabel xml:lang="de">Hoheitsgebiet</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19684-5">
    <skos:altLabel xml:lang="de">Gebietsanspruch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18654-5">
    <skos:related rdf:resource="descriptor/20241-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18654-5">
    <skos:related rdf:resource="descriptor/18552-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18654-5">
    <skos:related rdf:resource="descriptor/13278-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18654-5">
    <skos:related rdf:resource="descriptor/21315-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18654-5">
    <skos:related rdf:resource="descriptor/19544-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18654-5">
    <skos:related rdf:resource="descriptor/19653-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18654-5">
    <skos:related rdf:resource="descriptor/19541-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13342-0">
    <skos:broader rdf:resource="thsys/70847"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13342-0">
    <skos:broader rdf:resource="descriptor/11367-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71028">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11158-3">
    <skos:altLabel xml:lang="de">Hierarchietheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11587-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17485-0">
    <skos:altLabel xml:lang="en">Ukraina</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13569-0">
    <skos:related rdf:resource="descriptor/11833-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13569-0">
    <skos:related rdf:resource="descriptor/13264-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13569-0">
    <skos:related rdf:resource="descriptor/13510-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19157-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12119-1">
    <skos:prefLabel xml:lang="en">Group work</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12119-1">
    <skos:prefLabel xml:lang="de">Gruppenarbeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18961-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16029-3">
    <skos:related rdf:resource="descriptor/16030-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16029-3">
    <skos:related rdf:resource="descriptor/11651-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16029-3">
    <skos:related rdf:resource="descriptor/16028-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10940-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#altLabel">
    <rdf:type rdf:resource="http://www.w3.org/1999/02/22-rdf-syntax-ns#Property"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#altLabel">
    <rdf:type rdf:resource="http://www.w3.org/2002/07/owl#AnnotationProperty"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14180-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19606-4">
    <skos:prefLabel xml:lang="en">Bargaining power</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19606-4">
    <skos:prefLabel xml:lang="de">Verhandlungsmacht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12419-3">
    <skos:related rdf:resource="descriptor/12416-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10680-2">
    <skos:broader rdf:resource="thsys/71099"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10680-2">
    <skos:broader rdf:resource="thsys/71087"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10680-2">
    <skos:broader rdf:resource="descriptor/10791-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12903-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14173-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19771-3">
    <skos:prefLabel xml:lang="en">Anxiety</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19771-3">
    <skos:prefLabel xml:lang="de">Angst</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11951-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14766-0">
    <skos:broader rdf:resource="thsys/70647"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14766-0">
    <skos:broader rdf:resource="descriptor/14769-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18708-1">
    <skos:prefLabel xml:lang="en">Banking reform</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18708-1">
    <skos:prefLabel xml:lang="de">Bankenreform</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10865-4">
    <skos:broader rdf:resource="thsys/71091"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10865-4">
    <skos:broader rdf:resource="thsys/70951"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10865-4">
    <skos:broader rdf:resource="thsys/70259"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10865-4">
    <skos:broader rdf:resource="descriptor/12251-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10865-4">
    <skos:broader rdf:resource="descriptor/10854-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24733-2">
    <skos:related rdf:resource="descriptor/12265-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24733-2">
    <skos:related rdf:resource="descriptor/19522-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24733-2">
    <skos:related rdf:resource="descriptor/19671-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24733-2">
    <skos:related rdf:resource="descriptor/19535-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10349-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id287">
    <rdf:value xml:lang="de">Für die rechtlichen Regelungen bezüglich der sendetechnischen Infrastruktur</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18121-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16080-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13186-2">
    <skos:broader rdf:resource="thsys/70968"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13186-2">
    <skos:broader rdf:resource="descriptor/12915-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13186-2">
    <skos:broader rdf:resource="descriptor/13180-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11765-3">
    <skos:altLabel xml:lang="de">Pasinetti-Verteilungstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11765-3">
    <skos:altLabel xml:lang="de">Kaldor-Verteilungstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12712-5">
    <skos:related rdf:resource="descriptor/12065-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12712-5">
    <skos:related rdf:resource="descriptor/12051-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12712-5">
    <skos:related rdf:resource="descriptor/12744-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14449-5">
    <skos:related rdf:resource="descriptor/13553-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20506-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14150-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19246-1">
    <skos:altLabel xml:lang="de">Strukturwandel der Region</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11873-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16440-6">
    <skos:prefLabel xml:lang="en">Negotiations</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16440-6">
    <skos:prefLabel xml:lang="de">Verhandlungen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19742-3">
    <skos:altLabel xml:lang="de">Deduktives System</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19742-3">
    <skos:altLabel xml:lang="de">Axiomensystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19742-3">
    <skos:altLabel xml:lang="de">Axiomatisches System</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19742-3">
    <skos:altLabel xml:lang="de">Axiomatische Methode</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19742-3">
    <skos:altLabel xml:lang="de">Axiomatik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19742-3">
    <skos:altLabel xml:lang="de">Axiom</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18766-1">
    <skos:altLabel xml:lang="en">Software engineering</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18766-1">
    <skos:altLabel xml:lang="en">Software development</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18766-1">
    <skos:altLabel xml:lang="de">Softwaretechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18766-1">
    <skos:altLabel xml:lang="de">Softwarekonstruktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18766-1">
    <skos:altLabel xml:lang="de">Softwareentwicklung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18766-1">
    <skos:altLabel xml:lang="de">Prototyping von Software</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18766-1">
    <skos:altLabel xml:lang="de">Programmentwicklung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13051-4">
    <skos:broader rdf:resource="thsys/70997"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13051-4">
    <skos:broader rdf:resource="descriptor/13065-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12907-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12754-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15961-4">
    <skos:related rdf:resource="descriptor/16037-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15961-4">
    <skos:related rdf:resource="descriptor/16269-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15961-4">
    <skos:related rdf:resource="descriptor/16067-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15961-4">
    <skos:related rdf:resource="descriptor/16272-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15961-4">
    <skos:related rdf:resource="descriptor/13662-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15961-4">
    <skos:related rdf:resource="descriptor/13660-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15961-4">
    <skos:related rdf:resource="descriptor/19475-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15961-4">
    <skos:related rdf:resource="descriptor/10142-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15961-4">
    <skos:related rdf:resource="descriptor/18944-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15961-4">
    <skos:related rdf:resource="descriptor/16604-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15961-4">
    <skos:related rdf:resource="descriptor/13345-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15961-4">
    <skos:related rdf:resource="descriptor/15473-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15961-4">
    <skos:related rdf:resource="descriptor/15998-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15961-4">
    <skos:related rdf:resource="descriptor/16050-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15961-4">
    <skos:related rdf:resource="descriptor/19847-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15961-4">
    <skos:related rdf:resource="descriptor/15604-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15961-4">
    <skos:related rdf:resource="descriptor/19845-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10377-3">
    <skos:related rdf:resource="descriptor/10986-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10377-3">
    <skos:related rdf:resource="descriptor/10439-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10377-3">
    <skos:related rdf:resource="descriptor/10250-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10377-3">
    <skos:related rdf:resource="descriptor/10387-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10377-3">
    <skos:related rdf:resource="descriptor/10241-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10377-3">
    <skos:related rdf:resource="descriptor/18011-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19264-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10676-0">
    <skos:related rdf:resource="descriptor/10791-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10676-0">
    <skos:related rdf:resource="descriptor/10722-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10676-0">
    <skos:related rdf:resource="descriptor/10721-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10676-0">
    <skos:related rdf:resource="descriptor/10764-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10676-0">
    <skos:related rdf:resource="descriptor/10236-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10676-0">
    <skos:related rdf:resource="descriptor/10652-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18346-2">
    <skos:prefLabel xml:lang="en">Decentralized energy supply</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18346-2">
    <skos:prefLabel xml:lang="de">Dezentrale Energieversorgung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12958-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14692-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10714-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17677-5">
    <skos:altLabel xml:lang="de">Republik Sudan</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17677-5">
    <skos:altLabel xml:lang="de">Demokratische Republik Sudan</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18868-0">
    <skos:related rdf:resource="descriptor/19779-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18868-0">
    <skos:related rdf:resource="descriptor/11382-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18868-0">
    <skos:related rdf:resource="descriptor/12619-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18868-0">
    <skos:related rdf:resource="descriptor/11401-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18868-0">
    <skos:related rdf:resource="descriptor/18292-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16166-0">
    <skos:altLabel xml:lang="de">Schienenverkehrsrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16166-0">
    <skos:altLabel xml:lang="de">Schienenbahnrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16166-0">
    <skos:altLabel xml:lang="de">Eisenbahnverkehrsordnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16166-0">
    <skos:altLabel xml:lang="de">Bundesbahngesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16166-0">
    <skos:altLabel xml:lang="de">Allgemeines Eisenbahngesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12775-2">
    <skos:altLabel xml:lang="en">PR (Public Relations)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12775-2">
    <skos:altLabel xml:lang="de">Imagepflege</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12775-2">
    <skos:altLabel xml:lang="de">Betriebliche Informationspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13109-6">
    <skos:related rdf:resource="descriptor/13903-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15675-5">
    <skos:narrower rdf:resource="descriptor/15683-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15675-5">
    <skos:narrower rdf:resource="descriptor/18953-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15675-5">
    <skos:narrower rdf:resource="descriptor/15668-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15675-5">
    <skos:narrower rdf:resource="descriptor/15643-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15675-5">
    <skos:narrower rdf:resource="descriptor/19780-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18315-6">
    <skos:broader rdf:resource="thsys/71028"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18315-6">
    <skos:broader rdf:resource="descriptor/12945-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13541-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18916-1">
    <skos:broader rdf:resource="thsys/70829"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18916-1">
    <skos:broader rdf:resource="thsys/70013"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18916-1">
    <skos:broader rdf:resource="descriptor/18859-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11610-4">
    <skos:prefLabel xml:lang="en">Municipal debt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11610-4">
    <skos:prefLabel xml:lang="de">Kommunalverschuldung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14960-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13479-1">
    <skos:narrower rdf:resource="descriptor/13473-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13145-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19733-4">
    <skos:broader rdf:resource="thsys/73394"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19733-4">
    <skos:broader rdf:resource="thsys/70120"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19733-4">
    <skos:broader rdf:resource="descriptor/16709-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73377">
    <skos:broader rdf:resource="thsys/73382"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11057-2">
    <skos:broader rdf:resource="thsys/70858"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11057-2">
    <skos:broader rdf:resource="thsys/70274"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11057-2">
    <skos:broader rdf:resource="descriptor/10975-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18028-2">
    <zbwext:useInsteadNote rdf:nodeID="id300"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14960-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11213-0">
    <skos:prefLabel xml:lang="en">Frictional unemployment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11213-0">
    <skos:prefLabel xml:lang="de">Friktionelle Arbeitslosigkeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13844-4">
    <skos:prefLabel xml:lang="en">Housing maintenance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13844-4">
    <skos:prefLabel xml:lang="de">Gebäudeinstandhaltung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20468-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14722-6">
    <skos:prefLabel xml:lang="en">Biotechnological product</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14722-6">
    <skos:prefLabel xml:lang="de">Biotechnisches Produkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11157-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14483-5">
    <skos:related rdf:resource="descriptor/18361-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14483-5">
    <skos:related rdf:resource="descriptor/13681-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14483-5">
    <skos:related rdf:resource="descriptor/18359-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14483-5">
    <skos:related rdf:resource="descriptor/14262-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16297-6">
    <zbwext:useInsteadNote rdf:nodeID="id70"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16297-6">
    <zbwext:useInsteadNote rdf:nodeID="id69"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20906-3">
    <skos:prefLabel xml:lang="en">Cross-subsidization</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20906-3">
    <skos:prefLabel xml:lang="de">Interne Subventionierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17471-4">
    <skos:altLabel xml:lang="x-other">Eesti</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17471-4">
    <skos:altLabel xml:lang="de">Republik Estland</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17471-4">
    <skos:altLabel xml:lang="de">Estnische SSR</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17471-4">
    <skos:altLabel xml:lang="de">Estnische Republik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10010-5">
    <skos:altLabel xml:lang="de">Verbrauch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19171-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12907-4">
    <skos:related rdf:resource="descriptor/19423-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12907-4">
    <skos:related rdf:resource="descriptor/19628-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12907-4">
    <skos:related rdf:resource="descriptor/13214-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12907-4">
    <skos:related rdf:resource="descriptor/18916-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12907-4">
    <skos:related rdf:resource="descriptor/18917-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12907-4">
    <skos:related rdf:resource="descriptor/14029-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12907-4">
    <skos:related rdf:resource="descriptor/11194-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12907-4">
    <skos:related rdf:resource="descriptor/18556-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12907-4">
    <skos:related rdf:resource="descriptor/11850-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12922-1">
    <skos:broader rdf:resource="thsys/71010"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12922-1">
    <skos:broader rdf:resource="thsys/70811"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18434-5">
    <skos:prefLabel xml:lang="en">Printing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18434-5">
    <skos:prefLabel xml:lang="de">Drucktechnik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18012-3">
    <skos:prefLabel xml:lang="en">Germany</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18012-3">
    <skos:prefLabel xml:lang="de">Deutschland</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18217-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73389">
    <skos:broader rdf:resource="thsys/70154"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11211-4">
    <skos:related rdf:resource="descriptor/11207-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11211-4">
    <skos:related rdf:resource="descriptor/10202-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11211-4">
    <skos:related rdf:resource="descriptor/19011-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11211-4">
    <skos:related rdf:resource="descriptor/11219-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11211-4">
    <skos:related rdf:resource="descriptor/11189-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13100-3">
    <skos:related rdf:resource="descriptor/12910-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13100-3">
    <skos:related rdf:resource="descriptor/13650-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18596-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15923-5">
    <skos:prefLabel xml:lang="en">Jews</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15923-5">
    <skos:prefLabel xml:lang="de">Juden</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12531-2">
    <skos:altLabel xml:lang="de">Zuschlagskalkulation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12531-2">
    <skos:altLabel xml:lang="de">Stückkostenrechnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12531-2">
    <skos:altLabel xml:lang="de">Preiskalkulation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12531-2">
    <skos:altLabel xml:lang="de">Plankalkulation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12531-2">
    <skos:altLabel xml:lang="de">Mischkalkulation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12531-2">
    <skos:altLabel xml:lang="de">Kostenträgerstückrechnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12531-2">
    <skos:altLabel xml:lang="de">Kostenkalkulation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12531-2">
    <skos:altLabel xml:lang="de">Kalkulationsverfahren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id179">
    <rdfs:seeAlso rdf:resource="descriptor/13998-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16391-0">
    <skos:altLabel xml:lang="de">Pazifismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14899-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13498-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11129-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18690-1">
    <skos:altLabel xml:lang="en">Commodity derivative</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18690-1">
    <skos:altLabel xml:lang="de">Warenterminmarkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18690-1">
    <skos:altLabel xml:lang="de">Warenterminhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18690-1">
    <skos:altLabel xml:lang="de">Warentermingeschäft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18690-1">
    <skos:altLabel xml:lang="de">Rohstoffterminhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18690-1">
    <skos:altLabel xml:lang="de">Rohstofftermingeschäft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15265-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12020-6">
    <skos:related rdf:resource="descriptor/15754-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12020-6">
    <skos:related rdf:resource="descriptor/11870-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12020-6">
    <skos:related rdf:resource="descriptor/12028-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12020-6">
    <skos:related rdf:resource="descriptor/11040-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12020-6">
    <skos:related rdf:resource="descriptor/12023-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12020-6">
    <skos:related rdf:resource="descriptor/18333-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12020-6">
    <skos:related rdf:resource="descriptor/12018-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12020-6">
    <skos:related rdf:resource="descriptor/12015-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12020-6">
    <skos:related rdf:resource="descriptor/16176-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12020-6">
    <skos:related rdf:resource="descriptor/13451-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12020-6">
    <skos:related rdf:resource="descriptor/13426-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18393-0">
    <skos:broader rdf:resource="thsys/71006"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18393-0">
    <skos:broader rdf:resource="thsys/70992"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18393-0">
    <skos:broader rdf:resource="descriptor/18390-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19454-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13785-1">
    <skos:altLabel xml:lang="de">Emissionskonsortium</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13785-1">
    <skos:altLabel xml:lang="de">Emissionsbank</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19562-5">
    <skos:prefLabel xml:lang="en">Business organization</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19562-5">
    <skos:prefLabel xml:lang="de">Unternehmensorganisation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13033-6">
    <skos:prefLabel xml:lang="en">Copper industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13033-6">
    <skos:prefLabel xml:lang="de">Kupferindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17565-2">
    <skos:prefLabel xml:lang="en">China</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17565-2">
    <skos:prefLabel xml:lang="de">China</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11340-0">
    <skos:prefLabel xml:lang="en">Lockout</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11340-0">
    <skos:prefLabel xml:lang="de">Aussperrung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10209-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19445-2">
    <skos:altLabel xml:lang="de">Strukturfonds</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19445-2">
    <skos:altLabel xml:lang="de">EU-Strukturpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19445-2">
    <skos:altLabel xml:lang="de">EU-Strukturhilfe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19445-2">
    <skos:altLabel xml:lang="de">Europäische Strukturpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19445-2">
    <skos:altLabel xml:lang="de">Europäische Strukturhilfe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19445-2">
    <skos:altLabel xml:lang="de">Europäische Strukturförderung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19445-2">
    <skos:altLabel xml:lang="de">Europäische Strukturfonds</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19445-2">
    <skos:altLabel xml:lang="de">EG-Strukturfonds</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11708-1">
    <skos:broader rdf:resource="thsys/73337"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11708-1">
    <skos:broader rdf:resource="thsys/71075"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16010-3">
    <skos:prefLabel xml:lang="en">Press law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16010-3">
    <skos:prefLabel xml:lang="de">Presserecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11040-5">
    <skos:broader rdf:resource="thsys/73392"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11040-5">
    <skos:broader rdf:resource="thsys/70039"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11040-5">
    <skos:broader rdf:resource="descriptor/16035-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18591-3">
    <skos:prefLabel xml:lang="en">Shipping law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18591-3">
    <skos:prefLabel xml:lang="de">Schifffahrtsrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11849-4">
    <skos:related rdf:resource="descriptor/11897-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11849-4">
    <skos:related rdf:resource="descriptor/16297-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11849-4">
    <skos:related rdf:resource="descriptor/11919-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16449-2">
    <skos:broader rdf:resource="thsys/73365"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16449-2">
    <skos:broader rdf:resource="descriptor/16445-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11839-0">
    <skos:related rdf:resource="descriptor/11840-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11839-0">
    <skos:related rdf:resource="descriptor/18844-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11820-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11903-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19775-2">
    <skos:prefLabel xml:lang="en">Multilingualism</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19775-2">
    <skos:prefLabel xml:lang="de">Mehrsprachigkeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13393-4">
    <skos:altLabel xml:lang="de">Christliche Kirche</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10625-3">
    <skos:prefLabel xml:lang="en">World economic order</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10625-3">
    <skos:prefLabel xml:lang="de">Weltwirtschaftsordnung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12049-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72393">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19030-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19493-5">
    <skos:altLabel xml:lang="en">Labour market deregulation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19493-5">
    <skos:altLabel xml:lang="en">Labor market flexibility</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19493-5">
    <skos:altLabel xml:lang="de">Deregulierung des Arbeitsmarktes</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19493-5">
    <skos:altLabel xml:lang="de">Arbeitsmarktliberalisierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19493-5">
    <skos:altLabel xml:lang="de">Arbeitsmarktderegulierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10126-0">
    <skos:narrower rdf:resource="descriptor/10123-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13519-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18401-6">
    <skos:related rdf:resource="descriptor/15669-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18401-6">
    <skos:related rdf:resource="descriptor/18402-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18401-6">
    <skos:related rdf:resource="descriptor/13034-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18401-6">
    <skos:related rdf:resource="descriptor/14266-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18401-6">
    <skos:related rdf:resource="descriptor/18384-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16191-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16898-1">
    <skos:prefLabel xml:lang="en">North Rhine-Westphalia</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16898-1">
    <skos:prefLabel xml:lang="de">Nordrhein-Westfalen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18890-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11484-4">
    <skos:related rdf:resource="descriptor/11479-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11484-4">
    <skos:related rdf:resource="descriptor/13746-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70927">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18109-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15078-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18396-1">
    <skos:prefLabel xml:lang="en">Medical equipment industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18396-1">
    <skos:prefLabel xml:lang="de">Medizintechnische Industrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11897-0">
    <skos:related rdf:resource="descriptor/10512-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11897-0">
    <skos:related rdf:resource="descriptor/11849-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11897-0">
    <skos:related rdf:resource="descriptor/20321-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11897-0">
    <skos:related rdf:resource="descriptor/19364-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11897-0">
    <skos:related rdf:resource="descriptor/10562-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11897-0">
    <skos:related rdf:resource="descriptor/11979-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11897-0">
    <skos:related rdf:resource="descriptor/11919-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18877-6">
    <skos:prefLabel xml:lang="en">Higher education reform</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18877-6">
    <skos:prefLabel xml:lang="de">Hochschulreform</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15021-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14392-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14379-0">
    <skos:prefLabel xml:lang="en">Plastics machinery</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14379-0">
    <skos:prefLabel xml:lang="de">Kunststoffmaschine</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12116-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19275-1">
    <skos:prefLabel xml:lang="en">Commercial paper</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19275-1">
    <skos:prefLabel xml:lang="de">Commercial Paper</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16131-5">
    <skos:broader rdf:resource="thsys/73336"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16131-5">
    <skos:broader rdf:resource="thsys/71051"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16131-5">
    <skos:broader rdf:resource="thsys/70235"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16131-5">
    <skos:broader rdf:resource="thsys/70052"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16131-5">
    <skos:broader rdf:resource="descriptor/15982-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16131-5">
    <skos:broader rdf:resource="descriptor/11185-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12330-5">
    <skos:prefLabel xml:lang="en">Accounting information system</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12330-5">
    <skos:prefLabel xml:lang="de">Computergestützte Buchführung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19428-2">
    <skos:prefLabel xml:lang="en">Business network</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19428-2">
    <skos:prefLabel xml:lang="de">Business Network</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13755-3">
    <skos:prefLabel xml:lang="en">Banking automation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13755-3">
    <skos:prefLabel xml:lang="de">Bankautomation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15604-0">
    <skos:altLabel xml:lang="en">Privacy</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15604-0">
    <skos:altLabel xml:lang="de">Datenschutzrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15604-0">
    <skos:altLabel xml:lang="de">Datenschutzgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15604-0">
    <skos:altLabel xml:lang="de">Bundesdatenschutzgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12708-3">
    <skos:broader rdf:resource="thsys/70030"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10864-6">
    <skos:broader rdf:resource="thsys/70952"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10864-6">
    <skos:broader rdf:resource="descriptor/10605-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10864-6">
    <skos:broader rdf:resource="descriptor/10852-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11320-6">
    <skos:related rdf:resource="descriptor/12598-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11320-6">
    <skos:related rdf:resource="descriptor/11302-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11320-6">
    <skos:related rdf:resource="descriptor/11318-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11320-6">
    <skos:related rdf:resource="descriptor/11285-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11710-0">
    <skos:altLabel xml:lang="en">Country-of-origin-principle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11710-0">
    <skos:altLabel xml:lang="de">Wohnsitzprinzip</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11710-0">
    <skos:altLabel xml:lang="de">Herkunftslandprinzip</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11710-0">
    <skos:altLabel xml:lang="de">Betriebsstättenprinzip</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16496-0">
    <skos:broader rdf:resource="thsys/73362"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16496-0">
    <skos:broader rdf:resource="descriptor/16489-4"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id8">
    <rdf:value xml:lang="de">Für den Außenhandel mit Agrarerzeugnissen allgemein</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11807-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19393-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13629-1">
    <skos:altLabel xml:lang="de">Pendelverkehr von Arbeitskräften</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13629-1">
    <skos:altLabel xml:lang="de">Berufspendlerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13629-1">
    <skos:altLabel xml:lang="de">Berufspendler</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13629-1">
    <skos:altLabel xml:lang="de">Berufspendelverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13629-1">
    <skos:altLabel xml:lang="de">Ausbildungspendlerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13629-1">
    <skos:altLabel xml:lang="de">Ausbildungspendler</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13629-1">
    <skos:altLabel xml:lang="de">Arbeitspendlerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13629-1">
    <skos:altLabel xml:lang="de">Arbeitspendler</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10327-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11979-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19143-4">
    <skos:broader rdf:resource="thsys/70046"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19143-4">
    <skos:broader rdf:resource="descriptor/11636-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13673-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15609-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18957-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16264-0">
    <zbwext:useInsteadNote rdf:nodeID="id228"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12427-4">
    <skos:prefLabel xml:lang="en">Market value</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12427-4">
    <skos:prefLabel xml:lang="de">Marktwert</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11765-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18808-4">
    <skos:prefLabel xml:lang="en">Regional authority</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18808-4">
    <skos:prefLabel xml:lang="de">Regionalverwaltung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16053-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id67">
    <rdfs:seeAlso rdf:resource="descriptor/19605-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13601-2">
    <skos:broader rdf:resource="thsys/70940"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13601-2">
    <skos:broader rdf:resource="descriptor/11834-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18948-2">
    <skos:prefLabel xml:lang="en">Household income</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18948-2">
    <skos:prefLabel xml:lang="de">Haushaltseinkommen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18612-0">
    <skos:altLabel xml:lang="de">Dienstreiseverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16311-3">
    <skos:prefLabel xml:lang="en">Referendum</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16311-3">
    <skos:prefLabel xml:lang="de">Referendum</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11566-2">
    <skos:scopeNote xml:lang="de">Öffentliche Maßnahmen zur gleichmäßigeren Verteilung von Vermögensverlusten durch Kriege oder Naturkatastrophen auf die Bevölkerung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12868-2">
    <skos:altLabel xml:lang="de">Umwandlungssteuerrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12868-2">
    <skos:altLabel xml:lang="de">Umwandlungssteuergesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12868-2">
    <skos:altLabel xml:lang="de">Steuerliches Reorganisationsrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11161-0">
    <skos:broader rdf:resource="thsys/70141"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11161-0">
    <skos:broader rdf:resource="descriptor/10032-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11161-0">
    <skos:broader rdf:resource="descriptor/16745-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18174-5">
    <skos:broader rdf:resource="thsys/71016"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18174-5">
    <skos:broader rdf:resource="thsys/70227"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18174-5">
    <skos:broader rdf:resource="descriptor/10031-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14084-5">
    <skos:altLabel xml:lang="de">Zuckerrübe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14084-5">
    <skos:altLabel xml:lang="de">Zuckerrohr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19107-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10573-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18990-3">
    <skos:related rdf:resource="descriptor/12979-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18990-3">
    <skos:related rdf:resource="descriptor/18379-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18990-3">
    <skos:related rdf:resource="descriptor/14188-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70324">
    <rdfs:label xml:lang="en">Social policy</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70324">
    <rdfs:label xml:lang="de">Sozialpolitik</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70182">
    <skos:prefLabel xml:lang="en">N.03.01  Population dynamics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70182">
    <skos:prefLabel xml:lang="de">N.03.01  Bevölkerungsentwicklung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13578-6">
    <skos:broader rdf:resource="thsys/70998"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13578-6">
    <skos:broader rdf:resource="thsys/70931"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13578-6">
    <skos:broader rdf:resource="descriptor/10470-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13164-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12253-4">
    <skos:broader rdf:resource="thsys/71045"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12253-4">
    <skos:broader rdf:resource="thsys/70899"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15708-2">
    <skos:broader rdf:resource="thsys/70182"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12403-4">
    <skos:scopeNote xml:lang="de">Nur für betriebswirtschaftliche Verluste.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12573-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16767-2">
    <skos:prefLabel xml:lang="en">Library acquisition</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16767-2">
    <skos:prefLabel xml:lang="de">Erwerbung in Bibliotheken</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13346-6">
    <skos:prefLabel xml:lang="en">Information broker</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13346-6">
    <skos:prefLabel xml:lang="de">Informationsvermittlung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14821-4">
    <skos:prefLabel xml:lang="en">Plastic packaging</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14821-4">
    <skos:prefLabel xml:lang="de">Kunststoffverpackung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18850-5">
    <skos:altLabel xml:lang="de">Standesverband</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18850-5">
    <skos:altLabel xml:lang="de">Standesorganisation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18850-5">
    <skos:altLabel xml:lang="de">Berufsorganisation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16670-3">
    <skos:narrower rdf:resource="descriptor/18901-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16670-3">
    <skos:narrower rdf:resource="descriptor/16676-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16670-3">
    <skos:narrower rdf:resource="descriptor/19013-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16670-3">
    <skos:narrower rdf:resource="descriptor/18899-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16670-3">
    <skos:narrower rdf:resource="descriptor/18897-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16670-3">
    <skos:narrower rdf:resource="descriptor/19791-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16670-3">
    <skos:narrower rdf:resource="descriptor/19789-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16670-3">
    <skos:narrower rdf:resource="descriptor/12618-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18726-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13321-1">
    <skos:altLabel xml:lang="de">Waschsalon</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13321-1">
    <skos:altLabel xml:lang="de">Wäscherei</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13321-1">
    <skos:altLabel xml:lang="de">Heißmangel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13321-1">
    <skos:altLabel xml:lang="de">Färberei für Textilien</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13321-1">
    <skos:altLabel xml:lang="de">Chemische Reinigung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13321-1">
    <skos:altLabel xml:lang="de">Bügelei</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13321-1">
    <skos:altLabel xml:lang="de">Bekleidungsfärberei</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12365-0">
    <skos:prefLabel xml:lang="en">Opening balance sheet</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12365-0">
    <skos:prefLabel xml:lang="de">Eröffnungsbilanz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71025">
    <skos:prefLabel xml:lang="en">W.01.02  Animal husbandry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71025">
    <skos:prefLabel xml:lang="de">W.01.02  Tierhaltung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15738-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15469-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19675-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12932-5">
    <skos:broader rdf:resource="thsys/70967"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12932-5">
    <skos:broader rdf:resource="thsys/70888"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12932-5">
    <skos:broader rdf:resource="descriptor/12905-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10905-4">
    <skos:broader rdf:resource="thsys/71119"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10905-4">
    <skos:broader rdf:resource="thsys/70575"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10905-4">
    <skos:broader rdf:resource="thsys/70101"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10905-4">
    <skos:broader rdf:resource="descriptor/11782-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10905-4">
    <skos:broader rdf:resource="descriptor/10891-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14794-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12775-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10829-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12241-4">
    <skos:related rdf:resource="descriptor/13391-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12241-4">
    <skos:related rdf:resource="descriptor/12274-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12241-4">
    <skos:related rdf:resource="descriptor/10854-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12241-4">
    <skos:related rdf:resource="descriptor/18685-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14743-5">
    <skos:narrower rdf:resource="descriptor/14744-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12057-4">
    <skos:narrower rdf:resource="descriptor/12807-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15713-2">
    <skos:prefLabel xml:lang="en">Discrimination</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15713-2">
    <skos:prefLabel xml:lang="de">Diskriminierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73296">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id351">
    <rdf:value xml:lang="de">Für die Gestaltung umweltverträglicher Verkehrssysteme</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71068">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11932-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14039-3">
    <skos:related rdf:resource="descriptor/18410-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14039-3">
    <skos:related rdf:resource="descriptor/18359-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10774-0">
    <skos:prefLabel xml:lang="en">Central bank</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10774-0">
    <skos:prefLabel xml:lang="de">Zentralbank</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11189-6">
    <skos:related rdf:resource="descriptor/19011-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11189-6">
    <skos:related rdf:resource="descriptor/19756-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11189-6">
    <skos:related rdf:resource="descriptor/10917-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11189-6">
    <skos:related rdf:resource="descriptor/11400-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11189-6">
    <skos:related rdf:resource="descriptor/11389-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11189-6">
    <skos:related rdf:resource="descriptor/11211-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11189-6">
    <skos:related rdf:resource="descriptor/19767-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11189-6">
    <skos:related rdf:resource="descriptor/11183-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14502-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15327-0">
    <skos:altLabel xml:lang="en">Autoregressive moving average</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15327-0">
    <skos:altLabel xml:lang="en">Autoregressive integrated moving average</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15327-0">
    <skos:altLabel xml:lang="en">Autoregressive fractionally integrated moving average</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15327-0">
    <skos:altLabel xml:lang="en">ARIMA model</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15327-0">
    <skos:altLabel xml:lang="en">ARFIMA model</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15327-0">
    <skos:altLabel xml:lang="de">ARIMA-Modell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15327-0">
    <skos:altLabel xml:lang="de">ARFIMA-Modell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12294-4">
    <skos:altLabel xml:lang="en">Replacement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12294-4">
    <skos:altLabel xml:lang="de">Ersatzinvestition</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11840-1">
    <skos:altLabel xml:lang="de">Soziales Mietrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11840-1">
    <skos:altLabel xml:lang="de">Pachtrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11840-1">
    <skos:altLabel xml:lang="de">Mietvertrag</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11840-1">
    <skos:altLabel xml:lang="de">Mietgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/g">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">G</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18153-6">
    <skos:broader rdf:resource="thsys/71005"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18153-6">
    <skos:broader rdf:resource="descriptor/18397-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18153-6">
    <skos:broader rdf:resource="descriptor/12913-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71035">
    <skos:prefLabel xml:lang="de">B.10  Unternehmensforschung und Operations Research</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12915-5">
    <skos:narrower rdf:resource="descriptor/18499-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12915-5">
    <skos:narrower rdf:resource="descriptor/13186-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13496-1">
    <skos:broader rdf:resource="thsys/70953"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13496-1">
    <skos:broader rdf:resource="descriptor/13473-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13496-1">
    <skos:broader rdf:resource="descriptor/13209-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20018-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19128-0">
    <skos:broader rdf:resource="thsys/70550"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19128-0">
    <skos:broader rdf:resource="descriptor/18942-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19128-0">
    <skos:broader rdf:resource="descriptor/20314-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14384-0">
    <skos:broader rdf:resource="thsys/70318"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14384-0">
    <skos:broader rdf:resource="descriptor/14381-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15719-4">
    <skos:prefLabel xml:lang="en">Social conflict</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15719-4">
    <skos:prefLabel xml:lang="de">Sozialer Konflikt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13981-1">
    <skos:prefLabel xml:lang="en">Postal workers</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13981-1">
    <skos:prefLabel xml:lang="de">Postberufe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11996-5">
    <skos:prefLabel xml:lang="en">Intra-urban migration</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11996-5">
    <skos:prefLabel xml:lang="de">Städtische Binnenwanderung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13736-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12573-0">
    <zbwext:useInsteadNote rdf:nodeID="id249"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18623-2">
    <skos:prefLabel xml:lang="en">Environmentally sustainable transport</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18623-2">
    <skos:prefLabel xml:lang="de">Verkehrsökologie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11184-2">
    <skos:related rdf:resource="descriptor/15716-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11184-2">
    <skos:related rdf:resource="descriptor/12573-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11184-2">
    <skos:related rdf:resource="descriptor/11299-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11184-2">
    <skos:related rdf:resource="descriptor/11294-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11184-2">
    <skos:related rdf:resource="descriptor/19005-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11184-2">
    <skos:related rdf:resource="descriptor/18140-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11184-2">
    <skos:related rdf:resource="descriptor/10143-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11184-2">
    <skos:related rdf:resource="descriptor/19767-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70578">
    <rdfs:label xml:lang="de">Kosten- und Leistungsrechnung</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14731-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13987-3">
    <skos:altLabel xml:lang="de">Systemanalytikerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13987-3">
    <skos:altLabel xml:lang="de">Systemanalytiker</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13987-3">
    <skos:altLabel xml:lang="de">Programmiererinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13987-3">
    <skos:altLabel xml:lang="de">Programmierer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13987-3">
    <skos:altLabel xml:lang="de">EDV-Fachleute</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13987-3">
    <skos:altLabel xml:lang="de">DV-Fachleute</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13987-3">
    <skos:altLabel xml:lang="de">Datenverarbeitungsberufe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13987-3">
    <skos:altLabel xml:lang="de">Computerspezialistinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13987-3">
    <skos:altLabel xml:lang="de">Computerspezialisten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13987-3">
    <skos:altLabel xml:lang="de">Computerberufe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70066">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">P.18</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12257-3">
    <skos:prefLabel xml:lang="en">Real estate loan</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12257-3">
    <skos:prefLabel xml:lang="de">Realkredit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70485">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19138-4">
    <skos:prefLabel xml:lang="en">Barter</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19138-4">
    <skos:prefLabel xml:lang="de">Tausch</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15642-6">
    <skos:related rdf:resource="descriptor/14003-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15642-6">
    <skos:related rdf:resource="descriptor/19775-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14218-3">
    <skos:altLabel xml:lang="de">Raffinerieprodukt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14218-3">
    <skos:altLabel xml:lang="de">Petrochemisches Produkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14218-3">
    <skos:altLabel xml:lang="de">Petrochemikalie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14218-3">
    <skos:altLabel xml:lang="de">Erdölprodukt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14218-3">
    <skos:altLabel xml:lang="de">Bitumen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18160-2">
    <skos:scopeNote xml:lang="de">Interregionale und internationale Wettbewerbsfähigkeit von Standorten hinsichtlich der Attraktion mobiler Produktionsfaktoren.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19492-0">
    <skos:prefLabel xml:lang="en">Regional labour mobility</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19492-0">
    <skos:prefLabel xml:lang="de">Regionale Arbeitsmobilität</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10216-6">
    <skos:prefLabel xml:lang="en">Price leadership</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10216-6">
    <skos:prefLabel xml:lang="de">Preisführer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18490-2">
    <skos:broader rdf:resource="thsys/71067"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18490-2">
    <skos:broader rdf:resource="thsys/71005"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18490-2">
    <skos:broader rdf:resource="descriptor/11913-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16297-6">
    <skos:prefLabel xml:lang="en">Decentralization</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16297-6">
    <skos:prefLabel xml:lang="de">Dezentralisierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13563-5">
    <skos:prefLabel xml:lang="en">Road transport</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13563-5">
    <skos:prefLabel xml:lang="de">Straßenverkehr</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16463-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10930-5">
    <skos:altLabel xml:lang="en">Price collusion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10930-5">
    <skos:altLabel xml:lang="de">Submissionskartell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10930-5">
    <skos:altLabel xml:lang="de">Preiskartell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18217-6">
    <skos:altLabel xml:lang="de">Quarkspeise</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18217-6">
    <skos:altLabel xml:lang="de">Gekühltes Fertigdessert</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18217-6">
    <skos:altLabel xml:lang="de">Fruchtdessert</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18217-6">
    <skos:altLabel xml:lang="de">Fertigpudding</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18217-6">
    <skos:altLabel xml:lang="de">Fertigdessert</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18217-6">
    <skos:altLabel xml:lang="de">Dessertspeise</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19255-0">
    <skos:prefLabel xml:lang="en">Metallic standard</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19255-0">
    <skos:prefLabel xml:lang="de">Metallwährung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10889-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18878-4">
    <skos:altLabel xml:lang="en">Subject of study</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18878-4">
    <skos:altLabel xml:lang="de">Universitätsfachbereich</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18878-4">
    <skos:altLabel xml:lang="de">Studiengang</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18878-4">
    <skos:altLabel xml:lang="de">Studienangebot</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18878-4">
    <skos:altLabel xml:lang="de">Hochschulfachbereich</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18878-4">
    <skos:altLabel xml:lang="de">Fakultät</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id301">
    <rdfs:seeAlso rdf:resource="descriptor/11125-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16405-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13214-2">
    <skos:altLabel xml:lang="de">Do-it-Yourself-Markt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13214-2">
    <skos:altLabel xml:lang="de">DIY-Markt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13214-2">
    <skos:altLabel xml:lang="de">Baumarkt (Heimwerkermarkt)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18285-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15480-6">
    <skos:altLabel xml:lang="en">Risk behaviour</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15480-6">
    <skos:altLabel xml:lang="de">Risikoeinstellung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71063">
    <rdfs:label xml:lang="en">Public infrastructure</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71063">
    <rdfs:label xml:lang="de">Infrastruktur</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16427-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12971-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17175-1">
    <skos:broader rdf:resource="thsys/70130"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17175-1">
    <skos:broader rdf:resource="descriptor/17179-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70011">
    <skos:prefLabel xml:lang="de">N.06.01  Sozialwissenschaftliche Theorien, Forschungsansätze und Methoden</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19007-5">
    <skos:related rdf:resource="descriptor/12573-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19007-5">
    <skos:related rdf:resource="descriptor/18207-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18599-1">
    <skos:prefLabel xml:lang="en">Dockers</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18599-1">
    <skos:prefLabel xml:lang="de">Hafenarbeiter</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18325-3">
    <skos:scopeNote xml:lang="de">Umweltpolitischer und entwicklungstheoretischer Ansatz zur Gestaltung einer ökologisch vertretbaren Entwicklung unter Betonung ethischer Prinzipien und Verzicht auf Wachstum.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12649-0">
    <skos:altLabel xml:lang="de">Neuer Werkstoff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12649-0">
    <skos:altLabel xml:lang="de">Hilfsstoff der Produktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12649-0">
    <skos:altLabel xml:lang="de">Betriebsstoff der Produktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15709-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14630-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71055">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11478-6">
    <skos:prefLabel xml:lang="en">Discount rate policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11478-6">
    <skos:prefLabel xml:lang="de">Diskontpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18890-0">
    <skos:altLabel xml:lang="de">Sprechstundenhelferinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18890-0">
    <skos:altLabel xml:lang="de">Nichtärztliche Gesundheitsberufe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18890-0">
    <skos:altLabel xml:lang="de">Medizinallaborantinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18890-0">
    <skos:altLabel xml:lang="de">Medizinallaboranten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18890-0">
    <skos:altLabel xml:lang="de">Masseurinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18890-0">
    <skos:altLabel xml:lang="de">Masseure</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18890-0">
    <skos:altLabel xml:lang="de">Krankengymnastinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18890-0">
    <skos:altLabel xml:lang="de">Krankengymnasten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18890-0">
    <skos:altLabel xml:lang="de">Heilpraktikerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18890-0">
    <skos:altLabel xml:lang="de">Heilpraktiker</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18890-0">
    <skos:altLabel xml:lang="de">Hebammen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18890-0">
    <skos:altLabel xml:lang="de">Ernährungsfachleute</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18890-0">
    <skos:altLabel xml:lang="de">Arzthelferinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id190">
    <rdfs:seeAlso rdf:resource="descriptor/19568-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19604-1">
    <skos:scopeNote xml:lang="de">Statistiken zum Dienstleistungssektor.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18687-4">
    <skos:altLabel xml:lang="de">Aktienoptionsmarkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18687-4">
    <skos:altLabel xml:lang="de">Aktienoptionshandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18687-4">
    <skos:altLabel xml:lang="de">Aktienoptionsgeschäft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11451-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10156-5">
    <skos:related rdf:resource="descriptor/10184-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10156-5">
    <skos:related rdf:resource="descriptor/10179-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10156-5">
    <skos:related rdf:resource="descriptor/10077-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10156-5">
    <skos:related rdf:resource="descriptor/19209-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12733-4">
    <skos:narrower rdf:resource="descriptor/12732-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13298-5">
    <skos:broader rdf:resource="thsys/70936"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13298-5">
    <skos:broader rdf:resource="thsys/70891"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13298-5">
    <skos:broader rdf:resource="descriptor/13299-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12699-6">
    <skos:related rdf:resource="descriptor/12923-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12699-6">
    <skos:related rdf:resource="descriptor/12294-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12699-6">
    <skos:related rdf:resource="descriptor/12705-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12699-6">
    <skos:related rdf:resource="descriptor/20789-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12699-6">
    <skos:related rdf:resource="descriptor/19247-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10991-6">
    <skos:broader rdf:resource="thsys/70038"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13466-3">
    <skos:broader rdf:resource="thsys/71054"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13466-3">
    <skos:broader rdf:resource="thsys/70963"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13466-3">
    <skos:broader rdf:resource="descriptor/13970-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14164-0">
    <skos:prefLabel xml:lang="en">Freshwater fish</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14164-0">
    <skos:prefLabel xml:lang="de">Süßwasserfisch</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70828">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14047-4">
    <skos:narrower rdf:resource="descriptor/15011-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14047-4">
    <skos:narrower rdf:resource="descriptor/14184-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14047-4">
    <skos:narrower rdf:resource="descriptor/14236-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14047-4">
    <skos:narrower rdf:resource="descriptor/15751-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14047-4">
    <skos:narrower rdf:resource="descriptor/18286-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13422-2">
    <skos:related rdf:resource="descriptor/16016-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13422-2">
    <skos:related rdf:resource="descriptor/16017-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12050-4">
    <skos:prefLabel xml:lang="en">Holding company</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12050-4">
    <skos:prefLabel xml:lang="de">Holding</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13450-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13074-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16726-2">
    <skos:prefLabel xml:lang="en">Methodenstreit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16726-2">
    <skos:prefLabel xml:lang="de">Methodenkritik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12974-3">
    <skos:narrower rdf:resource="descriptor/14039-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12529-3">
    <skos:altLabel xml:lang="de">Handelsaufschlag</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12529-3">
    <skos:altLabel xml:lang="de">Handelsabschlag</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70562">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11275-6">
    <skos:narrower rdf:resource="descriptor/11288-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12181-3">
    <skos:narrower rdf:resource="descriptor/12198-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15327-0">
    <skos:broader rdf:resource="thsys/72393"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15327-0">
    <skos:broader rdf:resource="descriptor/15225-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15327-0">
    <skos:broader rdf:resource="descriptor/15376-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18167-2">
    <skos:prefLabel xml:lang="en">Cigarette industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18167-2">
    <skos:prefLabel xml:lang="de">Zigarettenindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19398-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18721-2">
    <skos:altLabel xml:lang="en">Asset-liability-management</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#hasTopConcept">
    <rdf:type rdf:resource="http://www.w3.org/1999/02/22-rdf-syntax-ns#Property"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#hasTopConcept">
    <rdf:type rdf:resource="http://www.w3.org/2002/07/owl#ObjectProperty"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10181-6">
    <skos:altLabel xml:lang="en">Consumption theory</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10181-6">
    <skos:altLabel xml:lang="de">Verbrauchstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10181-6">
    <skos:altLabel xml:lang="de">Nachfragetheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18601-5">
    <skos:broader rdf:resource="thsys/70933"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18601-5">
    <skos:broader rdf:resource="thsys/70926"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18601-5">
    <skos:broader rdf:resource="descriptor/13545-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10202-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id269">
    <rdfs:seeAlso rdf:resource="descriptor/19849-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10624-5">
    <skos:narrower rdf:resource="descriptor/10340-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19837-6">
    <skos:related rdf:resource="descriptor/17595-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19837-6">
    <skos:related rdf:resource="descriptor/11127-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19837-6">
    <skos:related rdf:resource="descriptor/11123-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19837-6">
    <skos:related rdf:resource="descriptor/17461-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19837-6">
    <skos:related rdf:resource="descriptor/19084-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19837-6">
    <skos:related rdf:resource="descriptor/19077-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19837-6">
    <skos:related rdf:resource="descriptor/19078-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16952-0">
    <skos:broader rdf:resource="thsys/70438"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16952-0">
    <skos:broader rdf:resource="descriptor/16950-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18513-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19475-0">
    <skos:broader rdf:resource="thsys/70823"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13429-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11327-6">
    <skos:prefLabel xml:lang="en">Wage structure</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11327-6">
    <skos:prefLabel xml:lang="de">Lohnstruktur</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16717-3">
    <skos:related rdf:resource="descriptor/11088-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16717-3">
    <skos:related rdf:resource="descriptor/16410-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16717-3">
    <skos:related rdf:resource="descriptor/11047-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15861-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12487-0">
    <skos:broader rdf:resource="thsys/71042"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12487-0">
    <skos:broader rdf:resource="descriptor/12295-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70348">
    <skos:broader rdf:resource="thsys/p"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11421-0">
    <skos:prefLabel xml:lang="en">Money market</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11421-0">
    <skos:prefLabel xml:lang="de">Geldmarkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19096-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70269">
    <skos:broader rdf:resource="thsys/n"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14125-3">
    <skos:related rdf:resource="descriptor/12972-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19408-1">
    <skos:altLabel xml:lang="de">Steuersatz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19681-4">
    <skos:altLabel xml:lang="de">Staatenverbund</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19681-4">
    <skos:altLabel xml:lang="de">Staatengemeinschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19681-4">
    <skos:altLabel xml:lang="de">Staatenbund</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19681-4">
    <skos:altLabel xml:lang="de">Konföderation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19344-1">
    <skos:prefLabel xml:lang="en">Cattle price</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19344-1">
    <skos:prefLabel xml:lang="de">Rinderpreis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14407-0">
    <skos:altLabel xml:lang="de">Thermische Solaranlage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14407-0">
    <skos:altLabel xml:lang="de">Sonnenkollektor</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14407-0">
    <skos:altLabel xml:lang="de">Sonnenenergieanlage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18873-0">
    <skos:prefLabel xml:lang="en">University library</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18873-0">
    <skos:prefLabel xml:lang="de">Hochschulbibliothek</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18927-3">
    <skos:related rdf:resource="descriptor/18926-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18927-3">
    <skos:related rdf:resource="descriptor/14040-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18927-3">
    <skos:related rdf:resource="descriptor/15655-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15642-6">
    <skos:prefLabel xml:lang="en">Foreign language</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15642-6">
    <skos:prefLabel xml:lang="de">Fremdsprache</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12768-6">
    <skos:broader rdf:resource="thsys/71036"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12768-6">
    <skos:broader rdf:resource="thsys/70041"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12768-6">
    <skos:broader rdf:resource="descriptor/12766-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12768-6">
    <skos:broader rdf:resource="descriptor/12165-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19459-5">
    <skos:broader rdf:resource="thsys/71065"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19459-5">
    <skos:broader rdf:resource="descriptor/19460-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14030-0">
    <skos:broader rdf:resource="thsys/70201"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18309-1">
    <skos:prefLabel xml:lang="en">Interindustry wage structure</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18309-1">
    <skos:prefLabel xml:lang="de">Sektorale Lohnstruktur</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11850-5">
    <skos:altLabel xml:lang="de">Urlaubsgebiet</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11850-5">
    <skos:altLabel xml:lang="de">Naherholungsgebiet</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11850-5">
    <skos:altLabel xml:lang="de">Fremdenverkehrsgebiet</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11850-5">
    <skos:altLabel xml:lang="de">Freizeitraum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11850-5">
    <skos:altLabel xml:lang="de">Erholungsraum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19409-6">
    <skos:broader rdf:resource="thsys/71076"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19409-6">
    <skos:broader rdf:resource="descriptor/19408-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19409-6">
    <skos:broader rdf:resource="descriptor/19413-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72186">
    <skos:prefLabel xml:lang="en">V.16.05  Statistical inference</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72186">
    <skos:prefLabel xml:lang="de">V.16.05  Inferenzstatistik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11309-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19581-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70140">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">B.00</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15892-4">
    <skos:altLabel xml:lang="de">Binnensee</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13546-5">
    <skos:prefLabel xml:lang="en">Fleet</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13546-5">
    <skos:prefLabel xml:lang="de">Schiffsflotte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13399-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10612-5">
    <skos:broader rdf:resource="thsys/71093"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10612-5">
    <skos:broader rdf:resource="descriptor/10626-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18716-2">
    <skos:prefLabel xml:lang="en">Bank management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18716-2">
    <skos:prefLabel xml:lang="de">Bankmanagement</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20368-5">
    <skos:related rdf:resource="descriptor/11052-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20368-5">
    <skos:related rdf:resource="descriptor/11772-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10186-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19461-4">
    <skos:related rdf:resource="descriptor/19439-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19461-4">
    <skos:related rdf:resource="descriptor/18865-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19461-4">
    <skos:related rdf:resource="descriptor/19462-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19461-4">
    <skos:related rdf:resource="descriptor/19362-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19461-4">
    <skos:related rdf:resource="descriptor/10504-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19461-4">
    <skos:related rdf:resource="descriptor/13445-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19169-0">
    <skos:altLabel xml:lang="en">Palau Islands</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19169-0">
    <skos:altLabel xml:lang="de">Republik Palau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18643-3">
    <skos:broader rdf:resource="thsys/73389"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18643-3">
    <skos:broader rdf:resource="thsys/70922"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18643-3">
    <skos:broader rdf:resource="descriptor/13647-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11650-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16354-6">
    <skos:related rdf:resource="descriptor/16353-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16354-6">
    <skos:related rdf:resource="descriptor/16357-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16354-6">
    <skos:related rdf:resource="descriptor/15983-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16354-6">
    <skos:related rdf:resource="descriptor/19699-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16354-6">
    <skos:related rdf:resource="descriptor/16355-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16354-6">
    <skos:related rdf:resource="descriptor/16306-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16354-6">
    <skos:related rdf:resource="descriptor/16226-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16354-6">
    <skos:related rdf:resource="descriptor/16369-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16354-6">
    <skos:related rdf:resource="descriptor/19692-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71070">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13564-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17658-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70461">
    <skos:narrower rdf:resource="descriptor/14049-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70461">
    <skos:narrower rdf:resource="descriptor/14045-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70461">
    <skos:narrower rdf:resource="descriptor/14626-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70461">
    <skos:narrower rdf:resource="descriptor/14034-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70461">
    <skos:narrower rdf:resource="descriptor/18266-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70461">
    <skos:narrower rdf:resource="descriptor/13427-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70461">
    <skos:narrower rdf:resource="descriptor/14596-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18201-0">
    <skos:broader rdf:resource="thsys/70465"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18201-0">
    <skos:broader rdf:resource="descriptor/14150-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10305-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15265-3">
    <skos:broader rdf:resource="thsys/72186"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15265-3">
    <skos:broader rdf:resource="descriptor/15272-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15265-3">
    <skos:broader rdf:resource="descriptor/19591-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11813-4">
    <skos:narrower rdf:resource="descriptor/19387-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11813-4">
    <skos:narrower rdf:resource="descriptor/19386-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14294-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13959-1">
    <skos:broader rdf:resource="thsys/71056"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13959-1">
    <skos:broader rdf:resource="thsys/71052"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13959-1">
    <skos:broader rdf:resource="thsys/70994"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13959-1">
    <skos:broader rdf:resource="descriptor/13962-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13959-1">
    <skos:broader rdf:resource="descriptor/14011-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73382">
    <skos:broader rdf:resource="thsys/70095"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73104">
    <skos:prefLabel xml:lang="de">V.17  Spiel- und Verhandlungstheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19397-1">
    <skos:broader rdf:resource="thsys/70994"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19397-1">
    <skos:broader rdf:resource="thsys/70957"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19397-1">
    <skos:broader rdf:resource="descriptor/10012-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19127-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18944-3">
    <skos:altLabel xml:lang="de">Fachinformationsprogramm</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18944-3">
    <skos:altLabel xml:lang="de">Fachinformationspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19706-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19267-0">
    <skos:scopeNote xml:lang="de">Verhältnis von kurz- und langfristigen Zinssätzen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10141-4">
    <skos:broader rdf:resource="thsys/71117"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10141-4">
    <skos:broader rdf:resource="thsys/70823"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10141-4">
    <skos:broader rdf:resource="thsys/70383"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10141-4">
    <skos:broader rdf:resource="descriptor/10144-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14202-4">
    <skos:altLabel xml:lang="de">Teer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14202-4">
    <skos:altLabel xml:lang="de">Pechkoks</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14202-4">
    <skos:altLabel xml:lang="de">Pech</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14202-4">
    <skos:altLabel xml:lang="de">Kokskohle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14202-4">
    <skos:altLabel xml:lang="de">Kokereiprodukt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14202-4">
    <skos:altLabel xml:lang="de">Kohlenprodukt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10381-5">
    <skos:prefLabel xml:lang="en">Anti-cyclical fiscal policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10381-5">
    <skos:prefLabel xml:lang="de">Antizyklische Finanzpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18541-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16424-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17931-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12995-2">
    <skos:broader rdf:resource="thsys/70994"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12995-2">
    <skos:broader rdf:resource="descriptor/12922-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12995-2">
    <skos:broader rdf:resource="descriptor/12996-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12580-3">
    <skos:related rdf:resource="descriptor/11310-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12580-3">
    <skos:related rdf:resource="descriptor/12603-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12580-3">
    <skos:related rdf:resource="descriptor/12119-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18070-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12651-6">
    <skos:related rdf:resource="descriptor/12655-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12651-6">
    <skos:related rdf:resource="descriptor/12667-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12651-6">
    <skos:related rdf:resource="descriptor/12656-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12651-6">
    <skos:related rdf:resource="descriptor/12748-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10824-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10574-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24621-6">
    <skos:related rdf:resource="descriptor/18964-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24621-6">
    <skos:related rdf:resource="descriptor/13324-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24621-6">
    <skos:related rdf:resource="descriptor/13333-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24621-6">
    <skos:related rdf:resource="descriptor/15727-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24621-6">
    <skos:related rdf:resource="descriptor/18959-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12380-4">
    <skos:prefLabel xml:lang="en">Stocktaking</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12380-4">
    <skos:prefLabel xml:lang="de">Inventur</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18960-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10770-1">
    <skos:broader rdf:resource="thsys/71103"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10770-1">
    <skos:broader rdf:resource="thsys/71094"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10770-1">
    <skos:broader rdf:resource="thsys/71088"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10770-1">
    <skos:broader rdf:resource="descriptor/11460-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10106-6">
    <skos:related rdf:resource="descriptor/10109-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10106-6">
    <skos:related rdf:resource="descriptor/10164-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10106-6">
    <skos:related rdf:resource="descriptor/10179-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10106-6">
    <skos:related rdf:resource="descriptor/10159-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10106-6">
    <skos:related rdf:resource="descriptor/10095-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10106-6">
    <skos:related rdf:resource="descriptor/10089-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10106-6">
    <skos:related rdf:resource="descriptor/10088-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16460-0">
    <skos:narrower rdf:resource="descriptor/18447-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16460-0">
    <skos:narrower rdf:resource="descriptor/11582-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16460-0">
    <skos:narrower rdf:resource="descriptor/19723-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16460-0">
    <skos:narrower rdf:resource="descriptor/16445-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19858-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14924-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16738-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18501-2">
    <skos:altLabel xml:lang="de">Trassierung von Schienenstrecken</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18501-2">
    <skos:altLabel xml:lang="de">Streckenneubau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18501-2">
    <skos:altLabel xml:lang="de">Streckenausbau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18501-2">
    <skos:altLabel xml:lang="de">Neubaustrecke</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18501-2">
    <skos:altLabel xml:lang="de">Gleisbau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18501-2">
    <skos:altLabel xml:lang="de">Eisenbahnoberbau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18501-2">
    <skos:altLabel xml:lang="de">Ausbaustrecke</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11990-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18414-4">
    <skos:related rdf:resource="descriptor/12736-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72394">
    <rdfs:label xml:lang="en">Multivariate analysis</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72394">
    <rdfs:label xml:lang="de">Multivariate Analyse</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70406">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.04.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73339">
    <skos:broader rdf:resource="thsys/73342"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18951-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19193-3">
    <skos:broader rdf:resource="thsys/71029"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19193-3">
    <skos:broader rdf:resource="descriptor/12933-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14771-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11679-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16481-6">
    <skos:broader rdf:resource="thsys/73370"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16481-6">
    <skos:broader rdf:resource="thsys/73335"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16481-6">
    <skos:broader rdf:resource="thsys/70833"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16481-6">
    <skos:broader rdf:resource="thsys/70324"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16481-6">
    <skos:broader rdf:resource="descriptor/16159-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16481-6">
    <skos:broader rdf:resource="descriptor/16480-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16229-2">
    <skos:broader rdf:resource="thsys/73394"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16229-2">
    <skos:broader rdf:resource="thsys/73383"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16229-2">
    <skos:broader rdf:resource="descriptor/16405-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10222-4">
    <skos:broader rdf:resource="thsys/71117"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10222-4">
    <skos:broader rdf:resource="descriptor/10141-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16958-2">
    <skos:altLabel xml:lang="de">Basel-Stadt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16958-2">
    <skos:altLabel xml:lang="de">Basel-Landschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16958-2">
    <skos:altLabel xml:lang="de">Basel-Land</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16958-2">
    <skos:altLabel xml:lang="de">Basel (Stadt)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16326-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11228-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12982-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16526-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18406-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70143">
    <skos:prefLabel xml:lang="de">N.08.01  Bildung und Lernen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18546-1">
    <skos:related rdf:resource="descriptor/18548-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18546-1">
    <skos:related rdf:resource="descriptor/13481-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15759-6">
    <skos:related rdf:resource="descriptor/15074-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15759-6">
    <skos:related rdf:resource="descriptor/15770-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15759-6">
    <skos:related rdf:resource="descriptor/15768-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15759-6">
    <skos:related rdf:resource="descriptor/15765-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15759-6">
    <skos:related rdf:resource="descriptor/15745-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15759-6">
    <skos:related rdf:resource="descriptor/15662-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21101-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14359-6">
    <skos:altLabel xml:lang="de">Schiffsmotor</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14359-6">
    <skos:altLabel xml:lang="de">Schiffsmaschine</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11770-3">
    <skos:altLabel xml:lang="de">Umverteilung des Einkommens</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11770-3">
    <skos:altLabel xml:lang="de">Einkommensredistribution</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11770-3">
    <skos:altLabel xml:lang="de">Einkommensnivellierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18396-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18386-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73361">
    <skos:prefLabel xml:lang="de">N.05.01.01  Staats- und Verfassungsrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18668-1">
    <skos:prefLabel xml:lang="en">Financial derivative</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18668-1">
    <skos:prefLabel xml:lang="de">Finanzderivat</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70580">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.08.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18162-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12326-3">
    <skos:altLabel xml:lang="de">Rechnungslegungsvorschrift</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12326-3">
    <skos:altLabel xml:lang="de">Handelsbilanzrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12326-3">
    <skos:altLabel xml:lang="de">Buchführungspflicht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12326-3">
    <skos:altLabel xml:lang="de">Buchführungs- und Bilanzierungsvorschrift</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12326-3">
    <skos:altLabel xml:lang="de">Bilanzierungsvorschrift</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12326-3">
    <skos:altLabel xml:lang="de">Bilanzierungspflicht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12326-3">
    <skos:altLabel xml:lang="de">Bilanzgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12485-4">
    <skos:broader rdf:resource="thsys/71042"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12485-4">
    <skos:broader rdf:resource="descriptor/12312-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#mappingRelation">
    <rdfs:isDefinedBy rdf:resource="http://www.w3.org/2004/02/skos/core"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11185-0">
    <skos:broader rdf:resource="thsys/73336"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11185-0">
    <skos:broader rdf:resource="thsys/71051"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11185-0">
    <skos:broader rdf:resource="thsys/70235"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11185-0">
    <skos:broader rdf:resource="thsys/70052"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11185-0">
    <skos:broader rdf:resource="descriptor/15966-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11563-1">
    <skos:related rdf:resource="descriptor/13284-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18458-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12013-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16104-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16158-6">
    <skos:scopeNote xml:lang="de">Auch für vergleichbare rechtliche Regelungen anderer Staaten.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10042-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18249-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11261-3">
    <skos:prefLabel xml:lang="en">Elderly workforce</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11261-3">
    <skos:prefLabel xml:lang="de">Ältere Arbeitskräfte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19406-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14473-1">
    <skos:altLabel xml:lang="de">Elektrogerät</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14473-1">
    <skos:altLabel xml:lang="de">Elektrische Ausrüstung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15711-6">
    <skos:broader rdf:resource="thsys/73396"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15711-6">
    <skos:broader rdf:resource="thsys/73378"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15711-6">
    <skos:broader rdf:resource="descriptor/15714-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15786-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11147-1">
    <skos:altLabel xml:lang="de">AIK (Agrar-Industrie-Komplex)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11147-1">
    <skos:altLabel xml:lang="de">Agrar-Industriekomplex</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11203-3">
    <skos:related rdf:resource="descriptor/12573-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11203-3">
    <skos:related rdf:resource="descriptor/12254-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11203-3">
    <skos:related rdf:resource="descriptor/18792-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11147-1">
    <skos:scopeNote xml:lang="de">Nur für die Landwirtschaft in den sozialistischen und ehemals sozialistischen Staaten.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18769-2">
    <skos:prefLabel xml:lang="en">Statistical database</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18769-2">
    <skos:prefLabel xml:lang="de">Statistische Datenbank</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17946-4">
    <skos:altLabel xml:lang="de">Salomoninseln</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17946-4">
    <skos:altLabel xml:lang="de">Britische Salomonen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15974-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15011-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13197-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12016-4">
    <skos:broader rdf:resource="thsys/73351"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12016-4">
    <skos:broader rdf:resource="thsys/71066"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12016-4">
    <skos:broader rdf:resource="thsys/71023"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12016-4">
    <skos:broader rdf:resource="thsys/70238"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12016-4">
    <skos:broader rdf:resource="descriptor/15763-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14060-5">
    <skos:scopeNote xml:lang="de">Natürliche und industriell gefertigte Gewürze und Würzmittel.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11040-5">
    <skos:related rdf:resource="descriptor/11069-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11040-5">
    <skos:related rdf:resource="descriptor/11068-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11040-5">
    <skos:related rdf:resource="descriptor/19213-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11040-5">
    <skos:related rdf:resource="descriptor/18176-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11040-5">
    <skos:related rdf:resource="descriptor/11771-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11040-5">
    <skos:related rdf:resource="descriptor/11065-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11040-5">
    <skos:related rdf:resource="descriptor/19161-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11040-5">
    <skos:related rdf:resource="descriptor/10005-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11040-5">
    <skos:related rdf:resource="descriptor/10091-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11040-5">
    <skos:related rdf:resource="descriptor/12020-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19008-3">
    <skos:broader rdf:resource="thsys/70383"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19008-3">
    <skos:broader rdf:resource="thsys/70244"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19008-3">
    <skos:broader rdf:resource="descriptor/19007-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19008-3">
    <skos:broader rdf:resource="descriptor/12171-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19008-3">
    <skos:broader rdf:resource="descriptor/19745-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11535-6">
    <skos:broader rdf:resource="thsys/73337"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11535-6">
    <skos:broader rdf:resource="thsys/70144"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11535-6">
    <skos:broader rdf:resource="descriptor/11069-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15628-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10827-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12609-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12088-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16541-0">
    <skos:narrower rdf:resource="descriptor/11388-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14098-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16587-4">
    <skos:related rdf:resource="descriptor/16595-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16587-4">
    <skos:related rdf:resource="descriptor/16615-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16587-4">
    <skos:related rdf:resource="descriptor/19752-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16587-4">
    <skos:related rdf:resource="descriptor/16299-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16587-4">
    <skos:related rdf:resource="descriptor/19680-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16587-4">
    <skos:related rdf:resource="descriptor/18830-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16587-4">
    <skos:related rdf:resource="descriptor/19773-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14354-2">
    <skos:altLabel xml:lang="de">Kfz-Motor</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14354-2">
    <skos:altLabel xml:lang="de">Automotor</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16514-3">
    <skos:altLabel xml:lang="de">Staatliche Filmförderung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14728-1">
    <skos:broader rdf:resource="thsys/70199"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14728-1">
    <skos:broader rdf:resource="descriptor/14599-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19290-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17472-2">
    <skos:altLabel xml:lang="de">Republik Georgien</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17472-2">
    <skos:altLabel xml:lang="de">Grusinien</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19422-0">
    <skos:broader rdf:resource="thsys/71074"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19422-0">
    <skos:broader rdf:resource="descriptor/11680-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#memberList">
    <rdfs:isDefinedBy rdf:resource="http://www.w3.org/2004/02/skos/core"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10683-3">
    <skos:narrower rdf:resource="descriptor/10710-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10683-3">
    <skos:narrower rdf:resource="descriptor/10706-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10683-3">
    <skos:narrower rdf:resource="descriptor/10699-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10683-3">
    <skos:narrower rdf:resource="descriptor/19316-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10683-3">
    <skos:narrower rdf:resource="descriptor/10692-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10683-3">
    <skos:narrower rdf:resource="descriptor/16084-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10821-3">
    <skos:broader rdf:resource="thsys/71103"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10821-3">
    <skos:broader rdf:resource="thsys/71094"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10821-3">
    <skos:broader rdf:resource="thsys/71088"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13776-2">
    <skos:prefLabel xml:lang="en">Bank lending</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13776-2">
    <skos:prefLabel xml:lang="de">Kreditgeschäft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="http://zbw.eu/namespaces/skos-extensions/Thsys">
    <rdfs:comment xml:lang="en">Each entry has a rdfs:label property for each 
    supported language and one skos:notation property. skos:prefLabel 
    properties are generated by concatenation of these properties. 
    Among themselves, the thsys entries are linked in several
    skos:broader/skos:narrower mono-hierarchies under a top-level 
    "Subthesaurus" Thsys concept. To Descriptor entities, only skos:narrower 
    connections are allowed.</rdfs:comment>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16950-4">
    <skos:narrower rdf:resource="descriptor/16955-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16950-4">
    <skos:narrower rdf:resource="descriptor/16954-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16950-4">
    <skos:narrower rdf:resource="descriptor/16953-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16950-4">
    <skos:narrower rdf:resource="descriptor/16952-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16950-4">
    <skos:narrower rdf:resource="descriptor/16951-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16950-4">
    <skos:narrower rdf:resource="descriptor/16949-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16950-4">
    <skos:narrower rdf:resource="descriptor/16948-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16950-4">
    <skos:narrower rdf:resource="descriptor/16946-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16950-4">
    <skos:narrower rdf:resource="descriptor/16943-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70941">
    <rdfs:label xml:lang="en">Rail transport</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70941">
    <rdfs:label xml:lang="de">Schienenverkehr</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/g">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19526-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11278-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14450-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11199-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13198-2">
    <skos:narrower rdf:resource="descriptor/13193-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13198-2">
    <skos:narrower rdf:resource="descriptor/18516-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19635-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12689-2">
    <skos:broader rdf:resource="thsys/70386"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12689-2">
    <skos:broader rdf:resource="descriptor/13510-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12689-2">
    <skos:broader rdf:resource="descriptor/12670-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19818-3">
    <skos:prefLabel xml:lang="en">Code of professional conduct</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19818-3">
    <skos:prefLabel xml:lang="de">Berufsrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18823-1">
    <skos:related rdf:resource="descriptor/11722-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18823-1">
    <skos:related rdf:resource="descriptor/12047-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18823-1">
    <skos:related rdf:resource="descriptor/18834-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18823-1">
    <skos:related rdf:resource="descriptor/24783-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71020">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16298-4">
    <skos:prefLabel xml:lang="en">Monarchy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16298-4">
    <skos:prefLabel xml:lang="de">Monarchie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14900-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19126-4">
    <skos:altLabel xml:lang="de">Warenskonto</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11384-1">
    <skos:prefLabel xml:lang="en">Training regulations</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11384-1">
    <skos:prefLabel xml:lang="de">Ausbildungsordnung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10515-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19259-6">
    <skos:scopeNote xml:lang="de">Beschreibt im Rahmen der monetären Theorie die Übertragung monetärer Impulse auf den realen Sektor.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15093-6">
    <skos:related rdf:resource="descriptor/10397-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15093-6">
    <skos:related rdf:resource="descriptor/11519-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15093-6">
    <skos:related rdf:resource="descriptor/11507-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15093-6">
    <skos:related rdf:resource="descriptor/11504-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10910-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14341-4">
    <skos:prefLabel xml:lang="en">Pump</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14341-4">
    <skos:prefLabel xml:lang="de">Pumpe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18005-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16290-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19450-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18448-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14776-4">
    <skos:broader rdf:resource="thsys/73306"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18155-2">
    <skos:altLabel xml:lang="en">DUP (Directly Unproductive Profit-Seeking Activities)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18155-2">
    <skos:altLabel xml:lang="en">Directly unproductive profit-seeking activities</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18166-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18027-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70418">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16604-2">
    <skos:broader rdf:resource="thsys/73396"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16604-2">
    <skos:broader rdf:resource="thsys/70823"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16604-2">
    <skos:broader rdf:resource="descriptor/16602-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16604-2">
    <skos:broader rdf:resource="descriptor/16588-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12832-2">
    <skos:broader rdf:resource="thsys/73391"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12832-2">
    <skos:broader rdf:resource="thsys/70384"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12832-2">
    <skos:broader rdf:resource="descriptor/12846-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12982-4">
    <skos:broader rdf:resource="thsys/71007"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12982-4">
    <skos:broader rdf:resource="descriptor/12985-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19290-5">
    <skos:broader rdf:resource="thsys/71029"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19290-5">
    <skos:broader rdf:resource="thsys/70960"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19290-5">
    <skos:broader rdf:resource="descriptor/13431-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11079-6">
    <skos:broader rdf:resource="thsys/70178"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11079-6">
    <skos:broader rdf:resource="descriptor/11111-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15860-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19628-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13639-5">
    <skos:narrower rdf:resource="descriptor/18634-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70368">
    <rdfs:label xml:lang="de">Materialwirtschaft und Beschaffung</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19643-5">
    <skos:scopeNote xml:lang="de">Ethnische Gruppen, die ursprünglich in jenem Land beheimatet sind, in dem sie leben</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13481-0">
    <skos:altLabel xml:lang="en">Shopping mall</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13481-0">
    <skos:altLabel xml:lang="de">Einkaufspassage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19336-0">
    <skos:related rdf:resource="descriptor/21101-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19336-0">
    <skos:related rdf:resource="descriptor/16048-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19336-0">
    <skos:related rdf:resource="descriptor/10734-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19336-0">
    <skos:related rdf:resource="descriptor/19353-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19336-0">
    <skos:related rdf:resource="descriptor/19352-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19336-0">
    <skos:related rdf:resource="descriptor/10608-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19336-0">
    <skos:related rdf:resource="descriptor/19663-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10222-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14057-1">
    <skos:altLabel xml:lang="de">Leinen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14057-1">
    <skos:altLabel xml:lang="de">Lein</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15878-5">
    <skos:related rdf:resource="descriptor/13509-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15878-5">
    <skos:related rdf:resource="descriptor/19684-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15878-5">
    <skos:related rdf:resource="descriptor/16288-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15878-5">
    <skos:related rdf:resource="descriptor/19616-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15878-5">
    <skos:related rdf:resource="descriptor/11853-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20289-1">
    <skos:broader rdf:resource="thsys/70141"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20289-1">
    <skos:broader rdf:resource="descriptor/10525-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16401-2">
    <skos:prefLabel xml:lang="en">Christian social ethics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16401-2">
    <skos:prefLabel xml:lang="de">Christliche Soziallehre</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19731-1">
    <skos:related rdf:resource="descriptor/15678-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19731-1">
    <skos:related rdf:resource="descriptor/16723-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19731-1">
    <skos:related rdf:resource="descriptor/15671-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18749-1">
    <skos:narrower rdf:resource="descriptor/13829-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73103">
    <skos:narrower rdf:resource="descriptor/15480-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73103">
    <skos:narrower rdf:resource="descriptor/15479-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73103">
    <skos:narrower rdf:resource="descriptor/15478-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73103">
    <skos:narrower rdf:resource="descriptor/15477-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73103">
    <skos:narrower rdf:resource="descriptor/19150-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73103">
    <skos:narrower rdf:resource="descriptor/15476-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73103">
    <skos:narrower rdf:resource="descriptor/15475-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73103">
    <skos:narrower rdf:resource="descriptor/15498-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73103">
    <skos:narrower rdf:resource="descriptor/15472-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73103">
    <skos:narrower rdf:resource="descriptor/15036-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73103">
    <skos:narrower rdf:resource="descriptor/10151-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73103">
    <skos:narrower rdf:resource="descriptor/15494-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73103">
    <skos:narrower rdf:resource="descriptor/15470-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73103">
    <skos:narrower rdf:resource="descriptor/19147-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73103">
    <skos:narrower rdf:resource="descriptor/15469-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73103">
    <skos:narrower rdf:resource="descriptor/15468-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73103">
    <skos:narrower rdf:resource="descriptor/15467-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73103">
    <skos:narrower rdf:resource="descriptor/15466-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73103">
    <skos:narrower rdf:resource="descriptor/20756-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73103">
    <skos:narrower rdf:resource="descriptor/15021-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73103">
    <skos:narrower rdf:resource="descriptor/15488-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73103">
    <skos:narrower rdf:resource="descriptor/19598-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16476-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19519-6">
    <skos:related rdf:resource="descriptor/12046-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17598-1">
    <skos:prefLabel xml:lang="en">Bhutan</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17598-1">
    <skos:prefLabel xml:lang="de">Bhutan</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13014-3">
    <skos:prefLabel xml:lang="en">Ceramics industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13014-3">
    <skos:prefLabel xml:lang="de">Keramische Industrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13095-5">
    <skos:prefLabel xml:lang="en">Jewelry industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13095-5">
    <skos:prefLabel xml:lang="de">Schmuckindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18225-0">
    <skos:related rdf:resource="descriptor/16828-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18388-0">
    <skos:broader rdf:resource="thsys/71009"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18388-0">
    <skos:broader rdf:resource="descriptor/15754-2"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id63">
    <rdfs:seeAlso rdf:resource="descriptor/15678-6"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="r1238766808r28278r2">
    <rdf:first rdf:resource="http://www.w3.org/2004/02/skos/core#Concept"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18877-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19757-4">
    <skos:altLabel xml:lang="en">Gender issues</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19757-4">
    <skos:altLabel xml:lang="de">Geschlechterdemokratie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18676-2">
    <skos:related rdf:resource="descriptor/19270-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18676-2">
    <skos:related rdf:resource="descriptor/19671-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18676-2">
    <skos:related rdf:resource="descriptor/18728-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18676-2">
    <skos:related rdf:resource="descriptor/13279-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12206-6">
    <skos:scopeNote xml:lang="de">Theorie über Entscheidungskriterien zur optimalen Gestaltung von Investition, Finanzierung und Ausschüttung, bei expliziter Berücksichtigung der Beziehungen zwischen Kapitalanbietern und -nachfragern.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13960-2">
    <skos:narrower rdf:resource="descriptor/13964-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15105-4">
    <zbwext:useInsteadNote rdf:nodeID="id267"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14539-4">
    <skos:related rdf:resource="descriptor/14537-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14539-4">
    <skos:related rdf:resource="descriptor/13082-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14539-4">
    <skos:related rdf:resource="descriptor/13229-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14539-4">
    <skos:related rdf:resource="descriptor/14619-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19401-1">
    <skos:related rdf:resource="descriptor/19410-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14771-0">
    <skos:related rdf:resource="descriptor/13185-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13987-3">
    <skos:narrower rdf:resource="descriptor/18764-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73345">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.10.06</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13273-0">
    <skos:related rdf:resource="descriptor/18645-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13273-0">
    <skos:related rdf:resource="descriptor/13621-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13273-0">
    <skos:related rdf:resource="descriptor/13652-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18660-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70932">
    <skos:narrower rdf:resource="descriptor/18608-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70932">
    <skos:narrower rdf:resource="descriptor/18611-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70932">
    <skos:narrower rdf:resource="descriptor/18607-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70932">
    <skos:narrower rdf:resource="descriptor/13519-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70932">
    <skos:narrower rdf:resource="descriptor/13247-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70932">
    <skos:narrower rdf:resource="descriptor/13980-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70932">
    <skos:narrower rdf:resource="descriptor/13246-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70932">
    <skos:narrower rdf:resource="descriptor/16170-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70932">
    <skos:narrower rdf:resource="descriptor/13518-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70932">
    <skos:narrower rdf:resource="descriptor/13814-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70932">
    <skos:narrower rdf:resource="descriptor/13575-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70932">
    <skos:narrower rdf:resource="descriptor/18418-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70932">
    <skos:narrower rdf:resource="descriptor/18606-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70932">
    <skos:narrower rdf:resource="descriptor/18605-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70932">
    <skos:narrower rdf:resource="descriptor/18604-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70932">
    <skos:narrower rdf:resource="descriptor/13516-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70932">
    <skos:narrower rdf:resource="thsys/70931"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14339-5">
    <skos:related rdf:resource="descriptor/14393-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17502-5">
    <skos:broader rdf:resource="thsys/71032"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17502-5">
    <skos:broader rdf:resource="descriptor/17595-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17502-5">
    <skos:broader rdf:resource="descriptor/19078-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14765-2">
    <skos:broader rdf:resource="thsys/73305"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14765-2">
    <skos:broader rdf:resource="thsys/71038"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14765-2">
    <skos:broader rdf:resource="descriptor/14751-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11779-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18190-0">
    <skos:related rdf:resource="descriptor/19290-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18190-0">
    <skos:related rdf:resource="descriptor/13127-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18190-0">
    <skos:related rdf:resource="descriptor/14055-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14716-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14066-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18480-5">
    <skos:related rdf:resource="descriptor/18490-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18480-5">
    <skos:related rdf:resource="descriptor/11906-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18480-5">
    <skos:related rdf:resource="descriptor/18489-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12272-0">
    <skos:prefLabel xml:lang="en">Yield</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12272-0">
    <skos:prefLabel xml:lang="de">Rendite</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19521-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16788-1">
    <skos:related rdf:resource="descriptor/19475-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16788-1">
    <skos:related rdf:resource="descriptor/13345-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16788-1">
    <skos:related rdf:resource="descriptor/15473-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16788-1">
    <skos:related rdf:resource="descriptor/16767-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19784-1">
    <skos:prefLabel xml:lang="en">Time</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19784-1">
    <skos:prefLabel xml:lang="de">Zeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18957-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14610-3">
    <skos:related rdf:resource="descriptor/14702-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18678-5">
    <skos:altLabel xml:lang="en">Maturity of securities</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18678-5">
    <skos:altLabel xml:lang="de">Fälligkeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10638-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13316-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16476-6">
    <skos:altLabel xml:lang="en">War on drugs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16476-6">
    <skos:altLabel xml:lang="en">Drug war</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16476-6">
    <skos:altLabel xml:lang="en">Drug control</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16476-6">
    <skos:altLabel xml:lang="en">Anti-drug policy</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16476-6">
    <skos:altLabel xml:lang="de">Drogenkrieg</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16476-6">
    <skos:altLabel xml:lang="de">Drogenbekämpfung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16476-6">
    <skos:altLabel xml:lang="de">Antidrogenpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12733-4">
    <skos:prefLabel xml:lang="en">Computer-aided design</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12733-4">
    <skos:prefLabel xml:lang="de">CAD</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14741-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19662-1">
    <skos:altLabel xml:lang="de">Rechtstreue</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19662-1">
    <skos:altLabel xml:lang="de">Erfüllung von Rechtsnormen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19662-1">
    <skos:altLabel xml:lang="de">Erfüllung von Auflagen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19662-1">
    <skos:altLabel xml:lang="de">Einhaltung von Reglementierungen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19662-1">
    <skos:altLabel xml:lang="de">Einhaltung von Gesetzen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19662-1">
    <skos:altLabel xml:lang="de">Befolgung von Gesetzen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15872-3">
    <skos:related rdf:resource="descriptor/11951-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15872-3">
    <skos:related rdf:resource="descriptor/15891-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15872-3">
    <skos:related rdf:resource="descriptor/16215-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15872-3">
    <skos:related rdf:resource="descriptor/16502-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13305-6">
    <skos:prefLabel xml:lang="en">Housing industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13305-6">
    <skos:prefLabel xml:lang="de">Wohnungswirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12812-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19472-6">
    <skos:broader rdf:resource="thsys/71064"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19472-6">
    <skos:broader rdf:resource="thsys/70051"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19472-6">
    <skos:broader rdf:resource="descriptor/11990-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19472-6">
    <skos:broader rdf:resource="descriptor/11183-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13351-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70868">
    <skos:broader rdf:resource="thsys/70871"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13514-4">
    <skos:narrower rdf:resource="descriptor/13560-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13514-4">
    <skos:narrower rdf:resource="descriptor/13527-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13514-4">
    <skos:narrower rdf:resource="descriptor/13529-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70002">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15926-6">
    <skos:prefLabel xml:lang="en">Lapps</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15926-6">
    <skos:prefLabel xml:lang="de">Lappen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70187">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.05.06</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12263-1">
    <skos:related rdf:resource="descriptor/12256-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18780-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14049-0">
    <skos:related rdf:resource="descriptor/14645-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14049-0">
    <skos:related rdf:resource="descriptor/14679-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14049-0">
    <skos:related rdf:resource="descriptor/14636-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11787-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20842-3">
    <skos:related rdf:resource="descriptor/16335-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20842-3">
    <skos:related rdf:resource="descriptor/19733-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20842-3">
    <skos:related rdf:resource="descriptor/19686-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20842-3">
    <skos:related rdf:resource="descriptor/16329-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20842-3">
    <skos:related rdf:resource="descriptor/20826-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20842-3">
    <skos:related rdf:resource="descriptor/10560-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20842-3">
    <skos:related rdf:resource="descriptor/19361-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20842-3">
    <skos:related rdf:resource="descriptor/16268-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20842-3">
    <skos:related rdf:resource="descriptor/18695-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19590-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13443-1">
    <skos:prefLabel xml:lang="en">Agricultural geography</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13443-1">
    <skos:prefLabel xml:lang="de">Agrargeographie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16516-6">
    <skos:narrower rdf:resource="descriptor/16517-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12903-5">
    <skos:prefLabel xml:lang="en">Financial audit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12903-5">
    <skos:prefLabel xml:lang="de">Wirtschaftsprüfung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16548-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10947-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14437-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15097-5">
    <skos:related rdf:resource="descriptor/15082-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10884-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11666-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10427-0">
    <skos:prefLabel xml:lang="en">Steady-state growth</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10427-0">
    <skos:prefLabel xml:lang="de">Steady-State-Wachstum</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19533-5">
    <skos:prefLabel xml:lang="en">Economic incentive</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19533-5">
    <skos:prefLabel xml:lang="de">Ökonomischer Anreiz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13037-5">
    <skos:broader rdf:resource="thsys/71000"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13037-5">
    <skos:broader rdf:resource="descriptor/13034-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16246-2">
    <skos:broader rdf:resource="thsys/73382"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16246-2">
    <skos:broader rdf:resource="descriptor/16292-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70030">
    <skos:narrower rdf:resource="descriptor/12649-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70030">
    <skos:narrower rdf:resource="descriptor/15066-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70030">
    <skos:narrower rdf:resource="descriptor/12713-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70030">
    <skos:narrower rdf:resource="descriptor/12712-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70030">
    <skos:narrower rdf:resource="descriptor/12504-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70030">
    <skos:narrower rdf:resource="descriptor/12708-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70030">
    <skos:narrower rdf:resource="descriptor/12705-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70030">
    <skos:narrower rdf:resource="descriptor/12065-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70030">
    <skos:narrower rdf:resource="descriptor/10016-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70030">
    <skos:narrower rdf:resource="descriptor/19034-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70030">
    <skos:narrower rdf:resource="descriptor/12667-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70030">
    <skos:narrower rdf:resource="descriptor/12619-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70030">
    <skos:narrower rdf:resource="descriptor/19116-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70030">
    <skos:narrower rdf:resource="descriptor/12702-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70030">
    <skos:narrower rdf:resource="descriptor/12700-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70030">
    <skos:narrower rdf:resource="descriptor/12699-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70030">
    <skos:narrower rdf:resource="descriptor/10546-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70030">
    <skos:narrower rdf:resource="descriptor/11186-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70030">
    <skos:narrower rdf:resource="thsys/70471"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70030">
    <skos:narrower rdf:resource="thsys/70310"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70030">
    <skos:narrower rdf:resource="thsys/70029"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70030">
    <skos:narrower rdf:resource="thsys/70270"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11855-2">
    <skos:prefLabel xml:lang="en">Periphery</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11855-2">
    <skos:prefLabel xml:lang="de">Peripheres Gebiet</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18180-3">
    <skos:prefLabel xml:lang="en">Long-term care insurance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18180-3">
    <skos:prefLabel xml:lang="de">Pflegeversicherung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11142-4">
    <skos:related rdf:resource="descriptor/10985-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11142-4">
    <skos:related rdf:resource="descriptor/11143-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11142-4">
    <skos:related rdf:resource="descriptor/11060-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11142-4">
    <skos:related rdf:resource="descriptor/11061-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11142-4">
    <skos:related rdf:resource="descriptor/11628-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11142-4">
    <skos:related rdf:resource="descriptor/11134-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11142-4">
    <skos:related rdf:resource="descriptor/16409-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11142-4">
    <skos:related rdf:resource="descriptor/11044-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11142-4">
    <skos:related rdf:resource="descriptor/12180-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12879-4">
    <skos:related rdf:resource="descriptor/18165-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12879-4">
    <skos:related rdf:resource="descriptor/18966-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12879-4">
    <skos:related rdf:resource="descriptor/16301-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16140-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12957-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10121-3">
    <skos:broader rdf:resource="thsys/70486"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10121-3">
    <skos:broader rdf:resource="descriptor/19195-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11276-4">
    <skos:narrower rdf:resource="descriptor/13436-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70985">
    <skos:prefLabel xml:lang="de">W.06.01.13  Verpackungswirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15610-5">
    <skos:related rdf:resource="descriptor/15621-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15610-5">
    <skos:related rdf:resource="descriptor/18766-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15610-5">
    <skos:related rdf:resource="descriptor/19802-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14490-1">
    <skos:narrower rdf:resource="descriptor/13667-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14490-1">
    <skos:narrower rdf:resource="descriptor/18249-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14490-1">
    <skos:narrower rdf:resource="descriptor/18296-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18309-1">
    <skos:related rdf:resource="descriptor/18308-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19308-5">
    <zbwext:useInsteadNote rdf:nodeID="id199"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10525-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10378-1">
    <skos:broader rdf:resource="thsys/70038"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19315-1">
    <skos:prefLabel xml:lang="en">Rules on foreign investment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19315-1">
    <skos:prefLabel xml:lang="de">Auslandsinvestitionsrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id157">
    <rdf:value xml:lang="de">Für nationale steuerrechtliche Regelungen von Auslandsbeziehungen</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12168-2">
    <skos:related rdf:resource="descriptor/19827-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12168-2">
    <skos:related rdf:resource="descriptor/12170-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12168-2">
    <skos:related rdf:resource="descriptor/16759-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12168-2">
    <skos:related rdf:resource="descriptor/12204-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16971-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19663-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11047-5">
    <skos:altLabel xml:lang="de">Stalinismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11047-5">
    <skos:altLabel xml:lang="de">Spartakismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11047-5">
    <skos:altLabel xml:lang="de">Revisionismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11047-5">
    <skos:altLabel xml:lang="de">Eurokommunismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11047-5">
    <skos:altLabel xml:lang="de">Bolschewismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73334">
    <skos:narrower rdf:resource="descriptor/16203-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73334">
    <skos:narrower rdf:resource="descriptor/16202-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73334">
    <skos:narrower rdf:resource="descriptor/16201-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73334">
    <skos:narrower rdf:resource="descriptor/13820-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73334">
    <skos:narrower rdf:resource="descriptor/15766-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73334">
    <skos:narrower rdf:resource="descriptor/15979-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73334">
    <skos:narrower rdf:resource="descriptor/19816-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73334">
    <skos:narrower rdf:resource="descriptor/19763-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73334">
    <skos:narrower rdf:resource="descriptor/16198-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73334">
    <skos:narrower rdf:resource="descriptor/16504-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73334">
    <skos:narrower rdf:resource="descriptor/19813-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73334">
    <skos:narrower rdf:resource="descriptor/16112-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73334">
    <skos:narrower rdf:resource="descriptor/10830-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73334">
    <skos:narrower rdf:resource="descriptor/16110-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73334">
    <skos:narrower rdf:resource="descriptor/19855-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73334">
    <skos:narrower rdf:resource="descriptor/11209-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73334">
    <skos:narrower rdf:resource="descriptor/10845-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73334">
    <skos:narrower rdf:resource="descriptor/18066-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73334">
    <skos:narrower rdf:resource="descriptor/16304-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73334">
    <skos:narrower rdf:resource="descriptor/19814-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73334">
    <skos:narrower rdf:resource="descriptor/15963-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73334">
    <skos:narrower rdf:resource="descriptor/13486-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73334">
    <skos:narrower rdf:resource="descriptor/16196-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73334">
    <skos:narrower rdf:resource="descriptor/19704-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73334">
    <skos:narrower rdf:resource="descriptor/16195-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73334">
    <skos:narrower rdf:resource="descriptor/16193-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73334">
    <skos:narrower rdf:resource="descriptor/16192-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73334">
    <skos:narrower rdf:resource="descriptor/16191-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73334">
    <skos:narrower rdf:resource="descriptor/19815-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73334">
    <skos:narrower rdf:resource="descriptor/11419-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73334">
    <skos:narrower rdf:resource="descriptor/19800-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73334">
    <skos:narrower rdf:resource="descriptor/16190-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73334">
    <skos:narrower rdf:resource="descriptor/18736-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13632-5">
    <skos:broader rdf:resource="thsys/70939"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13632-5">
    <skos:broader rdf:resource="descriptor/13510-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13434-2">
    <skos:related rdf:resource="descriptor/14074-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13434-2">
    <skos:related rdf:resource="descriptor/12949-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13434-2">
    <skos:related rdf:resource="descriptor/12032-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13434-2">
    <skos:related rdf:resource="descriptor/18987-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13434-2">
    <skos:related rdf:resource="descriptor/19194-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16834-6">
    <skos:prefLabel xml:lang="en">Germany (until 1945)</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16834-6">
    <skos:prefLabel xml:lang="de">Deutschland (bis 1945)</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16648-3">
    <skos:altLabel xml:lang="de">Kulturelle Zusammenarbeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16648-3">
    <skos:altLabel xml:lang="de">Kulturaustausch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16648-3">
    <skos:altLabel xml:lang="de">Kulturabkommen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16648-3">
    <skos:altLabel xml:lang="de">Auswärtige Kulturpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19112-1">
    <skos:altLabel xml:lang="en">Skills</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19112-1">
    <skos:altLabel xml:lang="de">Berufsqualifikation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19112-1">
    <skos:altLabel xml:lang="de">Ausbildungsqualifikation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18912-2">
    <skos:prefLabel xml:lang="en">War damage</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18912-2">
    <skos:prefLabel xml:lang="de">Kriegsschaden</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70647">
    <skos:narrower rdf:resource="descriptor/14772-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70647">
    <skos:narrower rdf:resource="descriptor/14771-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70647">
    <skos:narrower rdf:resource="descriptor/14769-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70647">
    <skos:narrower rdf:resource="descriptor/14252-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70647">
    <skos:narrower rdf:resource="descriptor/18293-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70647">
    <skos:narrower rdf:resource="descriptor/14766-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10893-6">
    <skos:prefLabel xml:lang="en">Rate-of-return regulation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10893-6">
    <skos:prefLabel xml:lang="de">Renditebeschränkung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19602-5">
    <skos:altLabel xml:lang="de">Geldstatistik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15446-6">
    <skos:related rdf:resource="descriptor/15463-6"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id64">
    <rdf:value xml:lang="de">für die christliche Kirche als Institution</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24857-5">
    <skos:broader rdf:resource="thsys/70049"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24857-5">
    <skos:broader rdf:resource="descriptor/10040-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16639-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19601-0">
    <skos:altLabel xml:lang="de">Kognitives Verhalten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19601-0">
    <skos:altLabel xml:lang="de">Kognitiver Prozess</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19601-0">
    <skos:altLabel xml:lang="de">Kognitiver Prozeß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19601-0">
    <skos:altLabel xml:lang="de">Kognitive Struktur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19601-0">
    <skos:altLabel xml:lang="de">Kognitive Psychologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19601-0">
    <skos:altLabel xml:lang="de">Kognitive Leistung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19601-0">
    <skos:altLabel xml:lang="de">Kognitionstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19601-0">
    <skos:altLabel xml:lang="de">Kognitionspsychologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19601-0">
    <skos:altLabel xml:lang="de">Intelligenzforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19601-0">
    <skos:altLabel xml:lang="de">Denkstruktur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19601-0">
    <skos:altLabel xml:lang="de">Denken</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19601-0">
    <skos:altLabel xml:lang="de">Bewußtseinspsychologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19601-0">
    <skos:altLabel xml:lang="de">Bewusstseinspsychologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13592-5">
    <skos:prefLabel xml:lang="en">Geography of tourism</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13592-5">
    <skos:prefLabel xml:lang="de">Fremdenverkehrsgeographie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14334-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16027-0">
    <skos:altLabel xml:lang="de">Versorgungsausgleich</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16027-0">
    <skos:altLabel xml:lang="de">Gütertrennung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16027-0">
    <skos:altLabel xml:lang="de">Gütergemeinschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16027-0">
    <skos:altLabel xml:lang="de">Ehegesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14996-4">
    <skos:prefLabel xml:lang="en">Confectionery</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14996-4">
    <skos:prefLabel xml:lang="de">Süßwaren</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13229-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17700-1">
    <skos:altLabel xml:lang="en">Ivory Coast</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17700-1">
    <skos:altLabel xml:lang="de">Republik Elfenbeinküste</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17700-1">
    <skos:altLabel xml:lang="de">Republik Cote d'Ivoire</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12356-1">
    <skos:prefLabel xml:lang="en">Corporate assets</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12356-1">
    <skos:prefLabel xml:lang="de">Betriebsvermögen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18228-1">
    <skos:prefLabel xml:lang="en">Macedonia</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18228-1">
    <skos:prefLabel xml:lang="de">Mazedonien</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11772-6">
    <skos:broader rdf:resource="thsys/71113"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11772-6">
    <skos:broader rdf:resource="thsys/70245"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11772-6">
    <skos:broader rdf:resource="thsys/70036"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11772-6">
    <skos:broader rdf:resource="descriptor/11778-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11772-6">
    <skos:broader rdf:resource="descriptor/11341-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11772-6">
    <skos:broader rdf:resource="descriptor/12276-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18690-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12712-5">
    <skos:narrower rdf:resource="descriptor/12758-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12712-5">
    <skos:narrower rdf:resource="descriptor/19116-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12712-5">
    <skos:narrower rdf:resource="descriptor/12757-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10797-2">
    <skos:prefLabel xml:lang="en">External debt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10797-2">
    <skos:prefLabel xml:lang="de">Auslandsverschuldung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19015-6">
    <skos:prefLabel xml:lang="en">Elephant</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19015-6">
    <skos:prefLabel xml:lang="de">Elefant</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18094-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11246-6">
    <skos:related rdf:resource="descriptor/11241-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11246-6">
    <skos:related rdf:resource="descriptor/19005-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11246-6">
    <skos:related rdf:resource="descriptor/19111-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11246-6">
    <skos:related rdf:resource="descriptor/11235-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18905-6">
    <skos:prefLabel xml:lang="en">Subsidiarity</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18905-6">
    <skos:prefLabel xml:lang="de">Subsidiaritätsprinzip</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17702-4">
    <skos:broader rdf:resource="thsys/70560"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17702-4">
    <skos:broader rdf:resource="descriptor/17724-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#hiddenLabel">
    <rdf:type rdf:resource="http://www.w3.org/1999/02/22-rdf-syntax-ns#Property"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#hiddenLabel">
    <rdf:type rdf:resource="http://www.w3.org/2002/07/owl#AnnotationProperty"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18775-0">
    <skos:altLabel xml:lang="en">University-industry cooperation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18775-0">
    <skos:altLabel xml:lang="en">R&amp;amp;D cooperation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18775-0">
    <skos:altLabel xml:lang="en">Business-university relations</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18775-0">
    <skos:altLabel xml:lang="de">Kooperation in der Forschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18775-0">
    <skos:altLabel xml:lang="de">Gemeinschaftsforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18775-0">
    <skos:altLabel xml:lang="de">FuE-Kooperation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18775-0">
    <skos:altLabel xml:lang="de">Forschungs- und Entwicklungskooperation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19317-4">
    <skos:prefLabel xml:lang="en">Trade liberalization</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19317-4">
    <skos:prefLabel xml:lang="de">Außenhandelsliberalisierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19116-0">
    <skos:related rdf:resource="descriptor/18620-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19116-0">
    <skos:related rdf:resource="descriptor/10463-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19116-0">
    <skos:related rdf:resource="descriptor/19152-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19116-0">
    <skos:related rdf:resource="descriptor/12643-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19116-0">
    <skos:related rdf:resource="descriptor/12119-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18377-5">
    <skos:prefLabel xml:lang="en">Water scarcity</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18377-5">
    <skos:prefLabel xml:lang="de">Wassermangel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15903-4">
    <skos:altLabel xml:lang="de">Oase</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13810-0">
    <skos:narrower rdf:resource="descriptor/18740-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18269-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14732-3">
    <skos:broader rdf:resource="thsys/70536"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14732-3">
    <skos:broader rdf:resource="descriptor/14734-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12458-0">
    <skos:prefLabel xml:lang="en">Accounting theory</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12458-0">
    <skos:prefLabel xml:lang="de">Bilanztheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18213-0">
    <skos:scopeNote xml:lang="de">Snackartikel sind keine genau abgegrenzte Klasse von Produkten.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11087-0">
    <skos:prefLabel xml:lang="en">Class struggle</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11087-0">
    <skos:prefLabel xml:lang="de">Klassenkampf</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15599-2">
    <skos:related rdf:resource="descriptor/18662-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15599-2">
    <skos:related rdf:resource="descriptor/19800-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15599-2">
    <skos:related rdf:resource="descriptor/14751-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19566-4">
    <skos:prefLabel xml:lang="en">Labour costs</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19566-4">
    <skos:prefLabel xml:lang="de">Arbeitskosten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13226-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10455-2">
    <skos:broader rdf:resource="thsys/73344"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10455-2">
    <skos:broader rdf:resource="thsys/71109"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10455-2">
    <skos:broader rdf:resource="thsys/70799"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10455-2">
    <skos:broader rdf:resource="descriptor/18426-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10475-3">
    <skos:related rdf:resource="descriptor/12587-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10475-3">
    <skos:related rdf:resource="descriptor/10481-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10475-3">
    <skos:related rdf:resource="descriptor/10478-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10475-3">
    <skos:related rdf:resource="descriptor/18070-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10475-3">
    <skos:related rdf:resource="descriptor/19548-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10475-3">
    <skos:related rdf:resource="descriptor/10459-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10475-3">
    <skos:related rdf:resource="descriptor/10436-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18206-4">
    <skos:related rdf:resource="descriptor/12847-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19320-1">
    <skos:prefLabel xml:lang="en">Trade retaliation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19320-1">
    <skos:prefLabel xml:lang="de">Retorsionsmaßnahme</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13890-4">
    <skos:related rdf:resource="descriptor/18947-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13890-4">
    <skos:related rdf:resource="descriptor/18426-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13890-4">
    <skos:related rdf:resource="descriptor/13076-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13890-4">
    <skos:related rdf:resource="descriptor/13077-0"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id295">
    <rdf:value xml:lang="de">Für politische Maßnahmen gegen die Bedrohung der inneren Sicherheit durch Subversion und Unruhen</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14192-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12808-6">
    <skos:prefLabel xml:lang="en">Product management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12808-6">
    <skos:prefLabel xml:lang="de">Produktpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11207-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15704-3">
    <skos:related rdf:resource="descriptor/11068-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15704-3">
    <skos:related rdf:resource="descriptor/19310-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15704-3">
    <skos:related rdf:resource="descriptor/16741-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15704-3">
    <skos:related rdf:resource="descriptor/25501-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15704-3">
    <skos:related rdf:resource="descriptor/10970-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15704-3">
    <skos:related rdf:resource="descriptor/16245-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14711-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14899-2">
    <skos:broader rdf:resource="thsys/70031"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18162-5">
    <skos:scopeNote xml:lang="de">Minerale, die entweder nicht substituierbar oder nur begrenzt verfügbar sind; dazu zählen auch alle wichtigen Metalle.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18876-1">
    <skos:prefLabel xml:lang="en">Higher education legislation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18876-1">
    <skos:prefLabel xml:lang="de">Hochschulrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14381-6">
    <skos:related rdf:resource="descriptor/19186-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14381-6">
    <skos:related rdf:resource="descriptor/19057-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14381-6">
    <skos:related rdf:resource="descriptor/19062-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17739-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19299-1">
    <skos:altLabel xml:lang="de">Geo-Ökonomik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15038-0">
    <skos:prefLabel xml:lang="en">Graph theory</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15038-0">
    <skos:prefLabel xml:lang="de">Graphentheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19816-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11197-0">
    <skos:altLabel xml:lang="en">Sweating system</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11197-0">
    <skos:altLabel xml:lang="en">Home-based employment</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11197-0">
    <skos:altLabel xml:lang="en">Cottage industry</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11197-0">
    <skos:altLabel xml:lang="de">Verlagssystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11197-0">
    <skos:altLabel xml:lang="de">Heimarbeitsgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11197-0">
    <skos:altLabel xml:lang="de">Hausindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13501-6">
    <skos:broader rdf:resource="thsys/70943"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13501-6">
    <skos:broader rdf:resource="descriptor/13510-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13661-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19573-0">
    <skos:altLabel xml:lang="en">Vectorautoregressive Process</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19573-0">
    <skos:altLabel xml:lang="en">Vector autoregressive processes</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19573-0">
    <skos:altLabel xml:lang="en">Vector autoregressive model</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19573-0">
    <skos:altLabel xml:lang="en">Vector autoregression model</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18873-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16545-6">
    <skos:altLabel xml:lang="de">Gesellschaftspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19553-6">
    <skos:broader rdf:resource="thsys/71115"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19553-6">
    <skos:broader rdf:resource="descriptor/18948-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70253">
    <skos:broader rdf:resource="thsys/70148"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13753-0">
    <skos:prefLabel xml:lang="en">Interest margin</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13753-0">
    <skos:prefLabel xml:lang="de">Zinsspannenrechnung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11233-1">
    <skos:scopeNote xml:lang="de">Literatur über die Schaffung eines zweiten Arbeitsmarktes, der geschützt und subventioniert ist.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18369-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13321-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14090-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19346-4">
    <skos:related rdf:resource="descriptor/19345-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19346-4">
    <skos:related rdf:resource="descriptor/12953-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19346-4">
    <skos:related rdf:resource="descriptor/14144-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10440-1">
    <skos:altLabel xml:lang="en">Labor productivity</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10440-1">
    <skos:altLabel xml:lang="de">Arbeitskoeffizient</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10020-2">
    <skos:altLabel xml:lang="en">Saving</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10020-2">
    <skos:altLabel xml:lang="en">National savings</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10020-2">
    <skos:altLabel xml:lang="en">National saving</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10020-2">
    <skos:altLabel xml:lang="en">Aggregate saving</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10020-2">
    <skos:altLabel xml:lang="de">Sparverhalten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10020-2">
    <skos:altLabel xml:lang="de">Gesamtwirtschaftliches Sparen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10020-2">
    <skos:altLabel xml:lang="de">Ersparnis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13354-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17567-5">
    <skos:prefLabel xml:lang="en">Taiwan</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17567-5">
    <skos:prefLabel xml:lang="de">Taiwan</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18433-0">
    <skos:altLabel xml:lang="de">Halbleiterschutzrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18433-0">
    <skos:altLabel xml:lang="de">Halbleiterschutzgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18433-0">
    <skos:altLabel xml:lang="de">EDV-Recht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18433-0">
    <skos:altLabel xml:lang="de">DV-Recht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18433-0">
    <skos:altLabel xml:lang="de">Datenverarbeitungsrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18443-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17588-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70960">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.10.01.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12032-6">
    <skos:altLabel xml:lang="en">Shifting cultivation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12032-6">
    <skos:altLabel xml:lang="de">Landwirtschaftliche Flächennutzung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12032-6">
    <skos:altLabel xml:lang="de">Agrarwirtschaftliche Bodennutzung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12032-6">
    <skos:altLabel xml:lang="de">Agrarbodennutzung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10395-1">
    <skos:scopeNote xml:lang="de">Optimale Mittel-Ziel-Zuordnung in der Wirtschaftspolitik.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71059">
    <skos:prefLabel xml:lang="en">V.13.04  Working time</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71059">
    <skos:prefLabel xml:lang="de">V.13.04  Arbeitszeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18772-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13716-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13726-3">
    <skos:narrower rdf:resource="descriptor/11618-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13726-3">
    <skos:narrower rdf:resource="descriptor/19277-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13726-3">
    <skos:narrower rdf:resource="descriptor/19276-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13726-3">
    <skos:narrower rdf:resource="descriptor/19275-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11158-3">
    <skos:prefLabel xml:lang="en">Hierarchy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11158-3">
    <skos:prefLabel xml:lang="de">Hierarchie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10701-6">
    <skos:prefLabel xml:lang="en">Import promotion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10701-6">
    <skos:prefLabel xml:lang="de">Importförderung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18422-5">
    <skos:related rdf:resource="descriptor/13239-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18422-5">
    <skos:related rdf:resource="descriptor/18421-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18422-5">
    <skos:related rdf:resource="descriptor/13051-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18422-5">
    <skos:related rdf:resource="descriptor/13052-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19354-5">
    <skos:altLabel xml:lang="de">Maastricht-Kriterien</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15839-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14444-1">
    <skos:related rdf:resource="descriptor/13069-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12925-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11895-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18004-2">
    <skos:narrower rdf:resource="descriptor/18269-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18335-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13343-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12979-0">
    <skos:broader rdf:resource="thsys/71011"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12979-0">
    <skos:broader rdf:resource="descriptor/12931-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12979-0">
    <skos:broader rdf:resource="descriptor/12913-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12903-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10638-1">
    <skos:broader rdf:resource="thsys/71100"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10638-1">
    <skos:broader rdf:resource="descriptor/10629-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13502-4">
    <skos:prefLabel xml:lang="en">Cabotage</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13502-4">
    <skos:prefLabel xml:lang="de">Kabotage</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10813-2">
    <skos:narrower rdf:resource="descriptor/10814-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11729-0">
    <zbwext:useInsteadNote rdf:nodeID="id101"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16674-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14193-0">
    <skos:broader rdf:resource="thsys/70418"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14193-0">
    <skos:broader rdf:resource="descriptor/19048-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14193-0">
    <skos:broader rdf:resource="descriptor/14192-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19709-1">
    <skos:related rdf:resource="descriptor/16458-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19709-1">
    <skos:related rdf:resource="descriptor/19717-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19709-1">
    <skos:related rdf:resource="descriptor/11083-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19165-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13083-5">
    <skos:prefLabel xml:lang="en">Clock and watch making industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13083-5">
    <skos:prefLabel xml:lang="de">Uhrenindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13504-0">
    <skos:prefLabel xml:lang="en">Scheduled service</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13504-0">
    <skos:prefLabel xml:lang="de">Linienverkehr</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19101-6">
    <zbwext:useInsteadNote rdf:nodeID="id269"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17502-5">
    <skos:scopeNote xml:lang="de">Erklärte sich 1991 unabhängig von der Sowjetunion und trat der GUS bei</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18645-6">
    <skos:altLabel xml:lang="de">Privater Paketdienst</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18645-6">
    <skos:altLabel xml:lang="de">Privater Kurierdienst</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18645-6">
    <skos:altLabel xml:lang="de">Expreßdienst</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11055-6">
    <skos:related rdf:resource="descriptor/10986-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11055-6">
    <skos:related rdf:resource="descriptor/10977-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11055-6">
    <skos:related rdf:resource="descriptor/11048-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11055-6">
    <skos:related rdf:resource="descriptor/10201-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11055-6">
    <skos:related rdf:resource="descriptor/10012-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11055-6">
    <skos:related rdf:resource="descriptor/11046-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15725-2">
    <skos:narrower rdf:resource="descriptor/18025-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15725-2">
    <skos:narrower rdf:resource="descriptor/18063-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10566-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11459-3">
    <skos:related rdf:resource="descriptor/18698-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11459-3">
    <skos:related rdf:resource="descriptor/11433-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13543-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14262-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16475-1">
    <skos:scopeNote xml:lang="de">Politik für alte Menschen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12922-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70140">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10439-0">
    <skos:prefLabel xml:lang="en">Growth policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10439-0">
    <skos:prefLabel xml:lang="de">Wachstumspolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18142-4">
    <skos:altLabel xml:lang="de">Gleichstellungspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18142-4">
    <skos:altLabel xml:lang="de">Frauenförderung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71074">
    <skos:prefLabel xml:lang="de">V.09.07.03  Steuerarten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19463-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14511-5">
    <skos:broader rdf:resource="thsys/70477"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14511-5">
    <skos:broader rdf:resource="descriptor/14502-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71082">
    <skos:prefLabel xml:lang="en">V.09.02  Fiscal policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71082">
    <skos:prefLabel xml:lang="de">V.09.02  Finanzpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16305-5">
    <skos:prefLabel xml:lang="en">Government system</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16305-5">
    <skos:prefLabel xml:lang="de">Staatsform</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70151">
    <skos:prefLabel xml:lang="de">N.10  Naturwissenschaften und Technik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13888-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14810-2">
    <skos:altLabel xml:lang="de">Holzstoff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14810-2">
    <skos:altLabel xml:lang="de">Holzschliff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19407-3">
    <skos:altLabel xml:lang="de">Personensteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18596-0">
    <skos:altLabel xml:lang="de">Seehafenumschlag</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19791-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11141-6">
    <skos:related rdf:resource="descriptor/11143-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11141-6">
    <skos:related rdf:resource="descriptor/10248-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11141-6">
    <skos:related rdf:resource="descriptor/10008-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10977-0">
    <skos:related rdf:resource="descriptor/11059-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10977-0">
    <skos:related rdf:resource="descriptor/11055-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19477-3">
    <skos:prefLabel xml:lang="en">Urban population dynamics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19477-3">
    <skos:prefLabel xml:lang="de">Städtische Bevölkerungsentwicklung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14829-2">
    <skos:broader rdf:resource="thsys/70313"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14829-2">
    <skos:broader rdf:resource="descriptor/14129-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10498-5">
    <skos:altLabel xml:lang="en">Human needs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10498-5">
    <skos:altLabel xml:lang="de">Grundbedürfnisstrategie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10498-5">
    <skos:altLabel xml:lang="de">Grundbedarfskonzept</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13871-1">
    <skos:related rdf:resource="descriptor/13375-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13871-1">
    <skos:related rdf:resource="descriptor/12903-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13871-1">
    <skos:related rdf:resource="descriptor/13374-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13871-1">
    <skos:related rdf:resource="descriptor/13992-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13871-1">
    <skos:related rdf:resource="descriptor/12889-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13871-1">
    <skos:related rdf:resource="descriptor/18083-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13871-1">
    <skos:related rdf:resource="descriptor/13389-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13871-1">
    <skos:related rdf:resource="descriptor/19157-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13871-1">
    <skos:related rdf:resource="descriptor/13355-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13871-1">
    <skos:related rdf:resource="descriptor/19966-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13871-1">
    <skos:related rdf:resource="descriptor/18013-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13871-1">
    <skos:related rdf:resource="descriptor/18850-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13871-1">
    <skos:related rdf:resource="descriptor/19818-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13871-1">
    <skos:related rdf:resource="descriptor/13370-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13871-1">
    <skos:related rdf:resource="descriptor/13957-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16134-6">
    <skos:altLabel xml:lang="de">Recht der beruflichen Bildung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16134-6">
    <skos:altLabel xml:lang="de">Berufsbildungsgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16134-6">
    <skos:altLabel xml:lang="de">Berufsbildungsförderungsgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16134-6">
    <skos:altLabel xml:lang="de">Ausbildungsrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16134-6">
    <skos:altLabel xml:lang="de">Ausbildungsplatzförderungsgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73343">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.10.06.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18678-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18234-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18744-4">
    <skos:related rdf:resource="descriptor/13824-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18744-4">
    <skos:related rdf:resource="descriptor/19679-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17687-2">
    <skos:related rdf:resource="descriptor/17987-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17687-2">
    <skos:related rdf:resource="descriptor/19175-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11676-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11707-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15009-0">
    <skos:prefLabel xml:lang="en">Waste</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15009-0">
    <skos:prefLabel xml:lang="de">Abfall</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18841-6">
    <skos:prefLabel xml:lang="en">Cooperative science</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18841-6">
    <skos:prefLabel xml:lang="de">Genossenschaftswissenschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15952-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16692-0">
    <skos:prefLabel xml:lang="en">Industrial sociology</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16692-0">
    <skos:prefLabel xml:lang="de">Industriesoziologie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70828">
    <skos:broader rdf:resource="thsys/w"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70997">
    <skos:prefLabel xml:lang="en">W.06.01.03.02  Road vehicle construction</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70997">
    <skos:prefLabel xml:lang="de">W.06.01.03.02  Straßenfahrzeugbau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#broader">
    <skos:definition xml:lang="en">A concept that is more general in meaning.</skos:definition>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17622-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14441-0">
    <skos:prefLabel xml:lang="en">Road vehicle</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14441-0">
    <skos:prefLabel xml:lang="de">Straßenfahrzeug</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18982-2">
    <skos:prefLabel xml:lang="en">Rural income</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18982-2">
    <skos:prefLabel xml:lang="de">Ländliches Einkommen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12331-3">
    <skos:prefLabel xml:lang="en">EU accounting standards</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12331-3">
    <skos:prefLabel xml:lang="de">EU-Bilanzrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70942">
    <skos:narrower rdf:resource="descriptor/13580-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70942">
    <skos:narrower rdf:resource="descriptor/13515-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70942">
    <skos:narrower rdf:resource="descriptor/13587-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70942">
    <skos:narrower rdf:resource="thsys/70938"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70942">
    <skos:narrower rdf:resource="thsys/70941"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70942">
    <skos:narrower rdf:resource="thsys/70932"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70942">
    <skos:narrower rdf:resource="thsys/70936"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18344-6">
    <skos:prefLabel xml:lang="en">Fishery engineering</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18344-6">
    <skos:prefLabel xml:lang="de">Fischereitechnik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16424-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id96">
    <rdfs:seeAlso rdf:resource="descriptor/18772-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19589-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70292">
    <rdfs:label xml:lang="en">Consumer behaviour</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70292">
    <rdfs:label xml:lang="de">Konsumentenverhalten</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13694-4">
    <skos:related rdf:resource="descriptor/13692-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19814-4">
    <skos:related rdf:resource="descriptor/16201-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19814-4">
    <skos:related rdf:resource="descriptor/10845-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19814-4">
    <skos:related rdf:resource="descriptor/16672-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19814-4">
    <skos:related rdf:resource="descriptor/16193-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19814-4">
    <skos:related rdf:resource="descriptor/19815-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19814-4">
    <skos:related rdf:resource="descriptor/16668-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71069">
    <skos:narrower rdf:resource="descriptor/18031-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71069">
    <skos:narrower rdf:resource="descriptor/19291-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71069">
    <skos:narrower rdf:resource="descriptor/19448-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71069">
    <skos:narrower rdf:resource="descriptor/16552-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71069">
    <skos:narrower rdf:resource="descriptor/18632-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71069">
    <skos:narrower rdf:resource="descriptor/11701-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71069">
    <skos:narrower rdf:resource="descriptor/19447-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71069">
    <skos:narrower rdf:resource="descriptor/11886-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71069">
    <skos:narrower rdf:resource="descriptor/19369-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71069">
    <skos:narrower rdf:resource="descriptor/11940-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71069">
    <skos:narrower rdf:resource="descriptor/11939-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71069">
    <skos:narrower rdf:resource="descriptor/11938-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71069">
    <skos:narrower rdf:resource="descriptor/19445-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71069">
    <skos:narrower rdf:resource="descriptor/11937-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71069">
    <skos:narrower rdf:resource="descriptor/11944-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71069">
    <skos:narrower rdf:resource="thsys/71068"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70874">
    <skos:prefLabel xml:lang="en">W.21.02  Advertising industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70874">
    <skos:prefLabel xml:lang="de">W.21.02  Werbewirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70169">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11818-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17571-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70184">
    <skos:narrower rdf:resource="descriptor/11682-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70184">
    <skos:narrower rdf:resource="descriptor/21061-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70184">
    <skos:narrower rdf:resource="descriptor/11777-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70184">
    <skos:narrower rdf:resource="descriptor/10024-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70184">
    <skos:narrower rdf:resource="descriptor/12437-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70184">
    <skos:narrower rdf:resource="descriptor/11659-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70184">
    <skos:narrower rdf:resource="descriptor/12419-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70184">
    <skos:narrower rdf:resource="descriptor/11651-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70184">
    <skos:narrower rdf:resource="descriptor/12416-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70184">
    <skos:narrower rdf:resource="descriptor/12356-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12945-3">
    <skos:scopeNote xml:lang="de">Agrarsektor im engeren Sinne.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13507-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16332-2">
    <skos:altLabel xml:lang="de">Verfassungsschutz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16332-2">
    <skos:altLabel xml:lang="de">Staatssicherheitsdienst</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16332-2">
    <skos:altLabel xml:lang="de">Spionageabwehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16332-2">
    <skos:altLabel xml:lang="de">Nachrichtendienst des Staates</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16332-2">
    <skos:altLabel xml:lang="de">Geheimdienst</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19455-6">
    <skos:related rdf:resource="descriptor/16250-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19455-6">
    <skos:related rdf:resource="descriptor/13425-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19455-6">
    <skos:related rdf:resource="descriptor/19748-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10395-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15045-3">
    <skos:related rdf:resource="descriptor/16727-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15045-3">
    <skos:related rdf:resource="descriptor/19581-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15045-3">
    <skos:related rdf:resource="descriptor/15064-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15045-3">
    <skos:related rdf:resource="descriptor/15664-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15045-3">
    <skos:related rdf:resource="descriptor/19207-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15045-3">
    <skos:related rdf:resource="descriptor/16716-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15045-3">
    <skos:related rdf:resource="descriptor/15415-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15045-3">
    <skos:related rdf:resource="descriptor/15609-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15045-3">
    <skos:related rdf:resource="descriptor/15498-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10628-4">
    <skos:broader rdf:resource="thsys/71093"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10628-4">
    <skos:broader rdf:resource="descriptor/10713-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10628-4">
    <skos:broader rdf:resource="descriptor/10626-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18752-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13610-1">
    <skos:prefLabel xml:lang="en">Transportation capacity</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13610-1">
    <skos:prefLabel xml:lang="de">Transportkapazität</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19257-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10375-0">
    <skos:altLabel xml:lang="de">Verbraucher-Konjunkturumfrage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10375-0">
    <skos:altLabel xml:lang="de">Unternehmer-Konjunkturumfrage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10375-0">
    <skos:altLabel xml:lang="de">Unternehmer Konjunkturumfrage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10375-0">
    <skos:altLabel xml:lang="de">Qualitative Konjunkturumfrage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10375-0">
    <skos:altLabel xml:lang="de">Konjunkturtest</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10375-0">
    <skos:altLabel xml:lang="de">Ifo-Konjunkturtest</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10247-2">
    <skos:prefLabel xml:lang="en">Keynesianism</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10247-2">
    <skos:prefLabel xml:lang="de">Keynesianismus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13959-1">
    <skos:prefLabel xml:lang="en">Chemists</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13959-1">
    <skos:prefLabel xml:lang="de">Chemiker</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73351">
    <skos:broader rdf:resource="thsys/73354"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16635-5">
    <skos:altLabel xml:lang="en">Wellness</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16635-5">
    <skos:altLabel xml:lang="en">Health status</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16635-5">
    <skos:altLabel xml:lang="de">Gesundheitszustand</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13970-6">
    <skos:broader rdf:resource="thsys/71054"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13970-6">
    <skos:broader rdf:resource="thsys/70963"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13970-6">
    <skos:broader rdf:resource="descriptor/18977-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12997-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14794-2">
    <skos:prefLabel xml:lang="en">Sawn timber</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14794-2">
    <skos:prefLabel xml:lang="de">Schnittholz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12793-0">
    <skos:prefLabel xml:lang="en">Budget price</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12793-0">
    <skos:prefLabel xml:lang="de">Niedrigpreisstrategie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11239-3">
    <skos:scopeNote xml:lang="de">Staatliche Maßnahmen der Wirtschafts-, Finanz- und Sozialpolitik zur Erhöhung der Beschäftigung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id387">
    <rdf:value xml:lang="de">Für die Zeitplanung im Betrieb</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17607-5">
    <skos:broader rdf:resource="thsys/70561"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17607-5">
    <skos:broader rdf:resource="descriptor/19167-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18518-6">
    <skos:broader rdf:resource="thsys/73294"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18518-6">
    <skos:broader rdf:resource="descriptor/13160-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18631-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18606-2">
    <skos:prefLabel xml:lang="en">Scheduled air service</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18606-2">
    <skos:prefLabel xml:lang="de">Linienluftverkehr</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12239-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70471">
    <skos:narrower rdf:resource="descriptor/12737-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70471">
    <skos:narrower rdf:resource="descriptor/10463-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70471">
    <skos:narrower rdf:resource="descriptor/18023-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70471">
    <skos:narrower rdf:resource="descriptor/12735-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70471">
    <skos:narrower rdf:resource="descriptor/18002-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70471">
    <skos:narrower rdf:resource="descriptor/19117-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70471">
    <skos:narrower rdf:resource="descriptor/12732-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70471">
    <skos:narrower rdf:resource="descriptor/12733-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70471">
    <skos:narrower rdf:resource="descriptor/18102-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70471">
    <skos:narrower rdf:resource="descriptor/12730-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10088-3">
    <skos:prefLabel xml:lang="en">Paradox of voting</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10088-3">
    <skos:prefLabel xml:lang="de">Abstimmungsparadoxon</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19089-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19656-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70011">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18080-0">
    <skos:broader rdf:resource="thsys/70463"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18080-0">
    <skos:broader rdf:resource="descriptor/14601-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18080-0">
    <skos:broader rdf:resource="descriptor/14642-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19753-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19268-5">
    <skos:scopeNote xml:lang="de">Diskussion von Zinsverbot und Zinswucher aus weltanschaulichen oder religiösen Gründen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11024-3">
    <skos:broader rdf:resource="thsys/71101"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11024-3">
    <skos:broader rdf:resource="descriptor/11033-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18797-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73299">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">P.20.06</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18618-2">
    <skos:prefLabel xml:lang="en">Refrigerated transport</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18618-2">
    <skos:prefLabel xml:lang="de">Kühltransport</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16676-5">
    <skos:altLabel xml:lang="en">Addictive behaviour</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16676-5">
    <skos:altLabel xml:lang="de">Suchtkrankheit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18777-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13832-4">
    <skos:related rdf:resource="descriptor/12927-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13832-4">
    <skos:related rdf:resource="descriptor/13308-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13832-4">
    <skos:related rdf:resource="descriptor/18634-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13832-4">
    <skos:related rdf:resource="descriptor/11850-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16523-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16698-2">
    <skos:scopeNote xml:lang="de">U.a. Analyse des familiären Hintergrundes von Personen und des sozialen Wandels in Familiengröße und Rolle der Familie.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11321-4">
    <skos:prefLabel xml:lang="en">Wage-earners' investment fund</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11321-4">
    <skos:prefLabel xml:lang="de">Investivlohn</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16193-4">
    <skos:related rdf:resource="descriptor/16201-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16193-4">
    <skos:related rdf:resource="descriptor/18116-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16193-4">
    <skos:related rdf:resource="descriptor/16430-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16193-4">
    <skos:related rdf:resource="descriptor/19975-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16193-4">
    <skos:related rdf:resource="descriptor/19814-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18648-0">
    <skos:altLabel xml:lang="de">Telephon-Banking</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11362-4">
    <skos:altLabel xml:lang="en">Vocational secondary school</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11362-4">
    <skos:altLabel xml:lang="de">Gewerbeschule</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11362-4">
    <skos:altLabel xml:lang="de">Fachschule</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11362-4">
    <skos:altLabel xml:lang="de">Berufsschule</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11362-4">
    <skos:altLabel xml:lang="de">Berufsfachschule</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11362-4">
    <skos:altLabel xml:lang="de">Berufsakademie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11362-4">
    <skos:altLabel xml:lang="de">Berufliche Schule</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18725-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15679-4">
    <skos:prefLabel xml:lang="en">Accident</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15679-4">
    <skos:prefLabel xml:lang="de">Unfall</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17627-6">
    <skos:altLabel xml:lang="de">Staat Katar</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17627-6">
    <skos:altLabel xml:lang="de">Quatar</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18899-3">
    <skos:prefLabel xml:lang="en">Cancer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18899-3">
    <skos:prefLabel xml:lang="de">Krebskrankheit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19599-3">
    <skos:prefLabel xml:lang="en">Emotion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19599-3">
    <skos:prefLabel xml:lang="de">Emotion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18335-0">
    <skos:broader rdf:resource="thsys/73347"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18335-0">
    <skos:broader rdf:resource="thsys/70013"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18335-0">
    <skos:broader rdf:resource="descriptor/18859-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18803-0">
    <skos:altLabel xml:lang="de">Staatliche Einrichtung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18803-0">
    <skos:altLabel xml:lang="de">Öffentliche Institution</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10339-4">
    <skos:broader rdf:resource="thsys/71060"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10339-4">
    <skos:broader rdf:resource="thsys/70020"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10339-4">
    <skos:broader rdf:resource="descriptor/11182-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15375-3">
    <skos:broader rdf:resource="thsys/72393"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15375-3">
    <skos:broader rdf:resource="descriptor/15376-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13894-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12318-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14093-4">
    <skos:related rdf:resource="descriptor/19190-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14093-4">
    <skos:related rdf:resource="descriptor/19189-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14093-4">
    <skos:related rdf:resource="descriptor/18981-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72183">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.16.03</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12040-0">
    <skos:altLabel xml:lang="en">Rating</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12040-0">
    <skos:altLabel xml:lang="en">Estimate of the value</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12040-0">
    <skos:altLabel xml:lang="de">Wertermittlung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12040-0">
    <skos:altLabel xml:lang="de">Evaluierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12487-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71115">
    <skos:prefLabel xml:lang="de">V.03.04  Einkommen und Vermögen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12693-4">
    <skos:altLabel xml:lang="de">Transporttechnik (Betrieb)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12693-4">
    <skos:altLabel xml:lang="de">Stetigförderer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12693-4">
    <skos:altLabel xml:lang="de">Seilbahn</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12693-4">
    <skos:altLabel xml:lang="de">Schwebebahn</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12693-4">
    <skos:altLabel xml:lang="de">Rolltreppe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12693-4">
    <skos:altLabel xml:lang="de">Hebezeug</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12693-4">
    <skos:altLabel xml:lang="de">Fördersystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12693-4">
    <skos:altLabel xml:lang="de">Förderband</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12693-4">
    <skos:altLabel xml:lang="de">Fahrstuhl</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12693-4">
    <skos:altLabel xml:lang="de">Bergbahn</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12693-4">
    <skos:altLabel xml:lang="de">Aufzug</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18606-2">
    <skos:broader rdf:resource="thsys/70932"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18606-2">
    <skos:broader rdf:resource="descriptor/13246-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18606-2">
    <skos:broader rdf:resource="descriptor/13504-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16343-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15768-5">
    <skos:related rdf:resource="descriptor/15778-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15768-5">
    <skos:related rdf:resource="descriptor/10111-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15768-5">
    <skos:related rdf:resource="descriptor/15773-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15768-5">
    <skos:related rdf:resource="descriptor/15770-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15768-5">
    <skos:related rdf:resource="descriptor/15769-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15768-5">
    <skos:related rdf:resource="descriptor/13400-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15768-5">
    <skos:related rdf:resource="descriptor/15767-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15768-5">
    <skos:related rdf:resource="descriptor/18165-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15768-5">
    <skos:related rdf:resource="descriptor/15763-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15768-5">
    <skos:related rdf:resource="descriptor/15762-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15768-5">
    <skos:related rdf:resource="descriptor/15759-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15768-5">
    <skos:related rdf:resource="descriptor/16266-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15768-5">
    <skos:related rdf:resource="descriptor/18958-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16329-5">
    <skos:prefLabel xml:lang="en">Government</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16329-5">
    <skos:prefLabel xml:lang="de">Regierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11687-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19761-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18361-6">
    <skos:prefLabel xml:lang="en">Electricity grid</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18361-6">
    <skos:prefLabel xml:lang="de">Stromnetz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10636-5">
    <skos:prefLabel xml:lang="en">Welfare-reducing trade</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10636-5">
    <skos:prefLabel xml:lang="de">Außenhandelsverlust</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16543-3">
    <skos:broader rdf:resource="thsys/70834"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71004">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.06.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18002-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70893">
    <skos:narrower rdf:resource="descriptor/13742-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70893">
    <skos:narrower rdf:resource="descriptor/12265-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70893">
    <skos:narrower rdf:resource="descriptor/18682-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70893">
    <skos:narrower rdf:resource="descriptor/10884-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70893">
    <skos:narrower rdf:resource="descriptor/18691-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70893">
    <skos:narrower rdf:resource="descriptor/13724-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70893">
    <skos:narrower rdf:resource="descriptor/19554-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70893">
    <skos:narrower rdf:resource="descriptor/19652-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70893">
    <skos:narrower rdf:resource="descriptor/18733-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70893">
    <skos:narrower rdf:resource="descriptor/11664-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70893">
    <skos:narrower rdf:resource="descriptor/18677-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70893">
    <skos:narrower rdf:resource="descriptor/13729-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70893">
    <skos:narrower rdf:resource="descriptor/24733-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70893">
    <skos:narrower rdf:resource="descriptor/18732-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70893">
    <skos:narrower rdf:resource="descriptor/19669-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70893">
    <skos:narrower rdf:resource="descriptor/18676-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70893">
    <skos:narrower rdf:resource="descriptor/19671-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70893">
    <skos:narrower rdf:resource="descriptor/13716-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70893">
    <skos:narrower rdf:resource="descriptor/13715-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70893">
    <skos:narrower rdf:resource="descriptor/18730-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70893">
    <skos:narrower rdf:resource="descriptor/18731-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70893">
    <skos:narrower rdf:resource="descriptor/18729-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70893">
    <skos:narrower rdf:resource="descriptor/18728-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70893">
    <skos:narrower rdf:resource="descriptor/13279-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70893">
    <skos:narrower rdf:resource="descriptor/19538-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70893">
    <skos:narrower rdf:resource="descriptor/10001-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70893">
    <skos:narrower rdf:resource="descriptor/19535-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13079-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70891">
    <skos:broader rdf:resource="thsys/70892"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11981-4">
    <skos:broader rdf:resource="thsys/71064"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11981-4">
    <skos:broader rdf:resource="descriptor/11916-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19638-5">
    <skos:scopeNote xml:lang="de">Zunahme des Anteils alter Menschen in der Bevölkerung</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18887-3">
    <skos:narrower rdf:resource="descriptor/13831-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18887-3">
    <skos:narrower rdf:resource="descriptor/15074-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18887-3">
    <skos:narrower rdf:resource="descriptor/15073-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18887-3">
    <skos:narrower rdf:resource="descriptor/19578-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18887-3">
    <skos:narrower rdf:resource="descriptor/18888-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18887-3">
    <skos:narrower rdf:resource="descriptor/15061-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18887-3">
    <skos:narrower rdf:resource="descriptor/11264-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19670-2">
    <skos:prefLabel xml:lang="en">Economic loss</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19670-2">
    <skos:prefLabel xml:lang="de">Volkswirtschaftlicher Schaden</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18565-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11340-0">
    <skos:broader rdf:resource="thsys/71051"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11340-0">
    <skos:broader rdf:resource="thsys/70235"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11340-0">
    <skos:broader rdf:resource="descriptor/11339-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70901">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.14.02.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13055-3">
    <skos:altLabel xml:lang="de">Lkw-Industrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10894-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18254-0">
    <skos:narrower rdf:resource="descriptor/14634-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18254-0">
    <skos:narrower rdf:resource="descriptor/19486-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18254-0">
    <skos:narrower rdf:resource="descriptor/14640-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13340-4">
    <skos:prefLabel xml:lang="en">Street cleaning</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13340-4">
    <skos:prefLabel xml:lang="de">Städtereinigung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16556-1">
    <skos:narrower rdf:resource="descriptor/16566-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16556-1">
    <skos:narrower rdf:resource="descriptor/15797-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12254-2">
    <skos:prefLabel xml:lang="en">Leasing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12254-2">
    <skos:prefLabel xml:lang="de">Leasing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id185">
    <rdfs:seeAlso rdf:resource="descriptor/18008-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10964-2">
    <skos:prefLabel xml:lang="en">Competition law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10964-2">
    <skos:prefLabel xml:lang="de">Wettbewerbsrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18712-3">
    <skos:scopeNote xml:lang="de">Abwicklung des Geschäftsverkehrs zwischen Bank und Bankkunden mittels PC, Onlinediensten oder Cash-Management-Systemen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11288-4">
    <skos:prefLabel xml:lang="en">Apprentices</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11288-4">
    <skos:prefLabel xml:lang="de">Auszubildende</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13437-3">
    <skos:broader rdf:resource="thsys/71024"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13437-3">
    <skos:broader rdf:resource="descriptor/16033-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19859-3">
    <skos:altLabel xml:lang="de">Werklieferungsvertrag</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17017-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16548-0">
    <skos:altLabel xml:lang="de">Volksrente</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16548-0">
    <skos:altLabel xml:lang="de">Volkspension</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16548-0">
    <skos:altLabel xml:lang="de">Grundrentenmodell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16548-0">
    <skos:altLabel xml:lang="de">Einheitsrente</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14502-6">
    <skos:broader rdf:resource="thsys/70477"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14502-6">
    <skos:broader rdf:resource="descriptor/14032-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14502-6">
    <skos:broader rdf:resource="descriptor/14473-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11037-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19375-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12196-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18294-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10850-3">
    <skos:related rdf:resource="descriptor/10785-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14731-5">
    <skos:altLabel xml:lang="de">Veterinärmedizinisches Produkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14731-5">
    <skos:altLabel xml:lang="de">Fütterungsarzneimittel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71003">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16266-3">
    <skos:prefLabel xml:lang="en">People's participation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16266-3">
    <skos:prefLabel xml:lang="de">Bürgerbeteiligung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12175-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10494-6">
    <skos:related rdf:resource="descriptor/15075-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10494-6">
    <skos:related rdf:resource="descriptor/10498-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10494-6">
    <skos:related rdf:resource="descriptor/10537-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10494-6">
    <skos:related rdf:resource="descriptor/19242-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10494-6">
    <skos:related rdf:resource="descriptor/10513-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18690-1">
    <skos:narrower rdf:resource="descriptor/18438-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10179-0">
    <skos:prefLabel xml:lang="en">Overlapping generations</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10179-0">
    <skos:prefLabel xml:lang="de">Overlapping Generations</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10136-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18417-5">
    <skos:related rdf:resource="descriptor/13576-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18417-5">
    <skos:related rdf:resource="descriptor/13545-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18417-5">
    <skos:related rdf:resource="descriptor/13066-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14195-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15722-1">
    <skos:broader rdf:resource="thsys/73359"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15722-1">
    <skos:broader rdf:resource="descriptor/18859-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11592-1">
    <skos:related rdf:resource="descriptor/13421-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14365-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15821-6">
    <skos:altLabel xml:lang="de">Vorurteil</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15821-6">
    <skos:altLabel xml:lang="de">Soziale Akzeptanz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15821-6">
    <skos:altLabel xml:lang="de">Politische Einstellung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15821-6">
    <skos:altLabel xml:lang="de">Meinungsbildung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15821-6">
    <skos:altLabel xml:lang="de">Einstellung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15821-6">
    <skos:altLabel xml:lang="de">Beurteilung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15821-6">
    <skos:altLabel xml:lang="de">Attitude</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70876">
    <rdfs:label xml:lang="de">Unternehmensdienstleistungen</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14601-4">
    <skos:narrower rdf:resource="descriptor/14634-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14601-4">
    <skos:narrower rdf:resource="descriptor/14627-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14601-4">
    <skos:narrower rdf:resource="descriptor/14266-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14601-4">
    <skos:narrower rdf:resource="descriptor/14608-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14601-4">
    <skos:narrower rdf:resource="descriptor/14313-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14601-4">
    <skos:narrower rdf:resource="descriptor/18080-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#note">
    <rdfs:label xml:lang="en">note</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10717-5">
    <skos:prefLabel xml:lang="en">Currency devaluation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10717-5">
    <skos:prefLabel xml:lang="de">Abwertung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13074-6">
    <skos:prefLabel xml:lang="en">Semiconductor industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13074-6">
    <skos:prefLabel xml:lang="de">Halbleiterindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19181-3">
    <skos:altLabel xml:lang="de">Mineralölpreis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11847-1">
    <skos:prefLabel xml:lang="en">Housing policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11847-1">
    <skos:prefLabel xml:lang="de">Wohnungspolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70333">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14608-4">
    <skos:related rdf:resource="descriptor/14640-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11653-0">
    <skos:prefLabel xml:lang="en">Local tax</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11653-0">
    <skos:prefLabel xml:lang="de">Gemeindesteuer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15365-6">
    <skos:prefLabel xml:lang="en">Instrumental variables estimation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15365-6">
    <skos:prefLabel xml:lang="de">Instrumentalvariablen-Schätzmethode</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19040-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14888-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12855-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15647-3">
    <skos:prefLabel xml:lang="en">Islam</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15647-3">
    <skos:prefLabel xml:lang="de">Islam</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70560">
    <skos:broader rdf:resource="thsys/70354"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73103">
    <rdfs:label xml:lang="en">Decision theory</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73103">
    <rdfs:label xml:lang="de">Entscheidungstheorie</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10503-3">
    <skos:broader rdf:resource="thsys/71065"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10503-3">
    <skos:broader rdf:resource="thsys/71028"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10503-3">
    <skos:broader rdf:resource="thsys/70091"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10503-3">
    <skos:broader rdf:resource="descriptor/11857-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19742-3">
    <skos:broader rdf:resource="thsys/73393"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19742-3">
    <skos:broader rdf:resource="descriptor/16727-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14601-4">
    <skos:altLabel xml:lang="de">Brom</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13025-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12758-2">
    <skos:broader rdf:resource="thsys/70029"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12758-2">
    <skos:broader rdf:resource="descriptor/12088-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12758-2">
    <skos:broader rdf:resource="descriptor/12712-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21283-1">
    <skos:altLabel xml:lang="en">Mobile business</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21283-1">
    <skos:altLabel xml:lang="de">M-Commerce</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11231-5">
    <skos:related rdf:resource="descriptor/11230-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15962-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10347-5">
    <skos:altLabel xml:lang="de">Unterkonsumtionslehre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10347-5">
    <skos:altLabel xml:lang="de">Überspartheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10347-5">
    <skos:altLabel xml:lang="de">Überproduktionslehre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10347-5">
    <skos:altLabel xml:lang="de">Überinvestitionstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10347-5">
    <skos:altLabel xml:lang="de">Überakkumulationstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18564-6">
    <skos:altLabel xml:lang="de">Schulungswirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18564-6">
    <skos:altLabel xml:lang="de">Kongreßwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18564-6">
    <skos:altLabel xml:lang="de">Kongreßveranstalter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18564-6">
    <skos:altLabel xml:lang="de">Kongresswirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18564-6">
    <skos:altLabel xml:lang="de">Kongressveranstalter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18564-6">
    <skos:altLabel xml:lang="de">Konferenzwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18564-6">
    <skos:altLabel xml:lang="de">Konferenzveranstalter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18564-6">
    <skos:altLabel xml:lang="de">Konferenzservice</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12257-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19306-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18743-6">
    <skos:related rdf:resource="descriptor/13819-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12985-5">
    <skos:scopeNote xml:lang="de">Nur für Bergbau auf Metalle.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18312-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18285-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70258">
    <rdfs:label xml:lang="de">Landfahrzeuge</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10473-0">
    <skos:related rdf:resource="descriptor/10470-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10473-0">
    <skos:related rdf:resource="descriptor/16696-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10473-0">
    <skos:related rdf:resource="descriptor/12208-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10473-0">
    <skos:related rdf:resource="descriptor/10487-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17731-4">
    <skos:altLabel xml:lang="de">Republik Burundi</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13037-5">
    <skos:related rdf:resource="descriptor/12988-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13037-5">
    <skos:related rdf:resource="descriptor/14302-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14262-0">
    <skos:related rdf:resource="descriptor/14483-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16479-0">
    <skos:broader rdf:resource="thsys/73370"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16479-0">
    <skos:broader rdf:resource="thsys/70839"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16479-0">
    <skos:broader rdf:resource="thsys/70833"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16479-0">
    <skos:broader rdf:resource="thsys/70573"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16479-0">
    <skos:broader rdf:resource="thsys/70324"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16479-0">
    <skos:broader rdf:resource="descriptor/16273-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20206-3">
    <skos:prefLabel xml:lang="en">Yield management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20206-3">
    <skos:prefLabel xml:lang="de">Yield Management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17610-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19338-3">
    <skos:related rdf:resource="descriptor/19393-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19338-3">
    <skos:related rdf:resource="descriptor/18811-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19338-3">
    <skos:related rdf:resource="descriptor/18991-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19338-3">
    <skos:related rdf:resource="descriptor/19394-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19338-3">
    <skos:related rdf:resource="descriptor/13160-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19338-3">
    <skos:related rdf:resource="descriptor/14931-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19338-3">
    <skos:related rdf:resource="descriptor/13219-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19338-3">
    <skos:related rdf:resource="descriptor/19382-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18086-2">
    <skos:altLabel xml:lang="en">Merchant bank</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18086-2">
    <skos:altLabel xml:lang="en">Investment banking</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18086-2">
    <skos:altLabel xml:lang="de">Wertpapierhaus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18086-2">
    <skos:altLabel xml:lang="de">Wertpapierfirma</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11321-4">
    <skos:related rdf:resource="descriptor/11571-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11637-5">
    <skos:related rdf:resource="descriptor/11642-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11637-5">
    <skos:related rdf:resource="descriptor/11640-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11637-5">
    <skos:related rdf:resource="descriptor/11704-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11637-5">
    <skos:related rdf:resource="descriptor/19405-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11637-5">
    <skos:related rdf:resource="descriptor/11547-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12272-0">
    <skos:broader rdf:resource="thsys/70455"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12272-0">
    <skos:broader rdf:resource="thsys/70187"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12272-0">
    <skos:broader rdf:resource="thsys/70036"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12272-0">
    <skos:broader rdf:resource="descriptor/12285-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10981-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11892-3">
    <skos:broader rdf:resource="thsys/71064"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18344-6">
    <skos:related rdf:resource="descriptor/12964-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18344-6">
    <skos:related rdf:resource="descriptor/13450-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14139-6">
    <skos:prefLabel xml:lang="en">Horse</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14139-6">
    <skos:prefLabel xml:lang="de">Pferd</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12567-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15315-0">
    <skos:altLabel xml:lang="en">Semiparametric statistics</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15315-0">
    <skos:altLabel xml:lang="en">Semi-parametric statistics</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15315-0">
    <skos:altLabel xml:lang="en">Nonparametric model</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15315-0">
    <skos:altLabel xml:lang="en">Distribution-free statistics</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15315-0">
    <skos:altLabel xml:lang="de">Verteilungsfreie Statistik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15315-0">
    <skos:altLabel xml:lang="de">Semiparametrische Statistik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15315-0">
    <skos:altLabel xml:lang="de">Parameterfreies Verfahren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15315-0">
    <skos:altLabel xml:lang="de">Nichtparametrische Statistik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19087-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13384-5">
    <skos:related rdf:resource="descriptor/12570-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13384-5">
    <skos:related rdf:resource="descriptor/11236-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18340-0">
    <skos:broader rdf:resource="thsys/73351"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18340-0">
    <skos:broader rdf:resource="thsys/71019"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18340-0">
    <skos:broader rdf:resource="descriptor/15897-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19021-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10381-5">
    <skos:altLabel xml:lang="en">Deficit spending</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10381-5">
    <skos:altLabel xml:lang="de">Stabilitätszuschlag</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10381-5">
    <skos:altLabel xml:lang="de">Konjunkturzuschlag</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10381-5">
    <skos:altLabel xml:lang="de">Kompensatorische Finanzpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10381-5">
    <skos:altLabel xml:lang="de">Ergänzungsabgabe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10381-5">
    <skos:altLabel xml:lang="de">Antizyklische Fiskalpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12018-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13376-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19144-2">
    <skos:broader rdf:resource="thsys/73341"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19144-2">
    <skos:broader rdf:resource="thsys/71037"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19144-2">
    <skos:broader rdf:resource="descriptor/15599-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11580-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18286-1">
    <skos:broader rdf:resource="thsys/73351"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18286-1">
    <skos:broader rdf:resource="thsys/70811"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18286-1">
    <skos:broader rdf:resource="descriptor/14047-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18286-1">
    <skos:broader rdf:resource="descriptor/18381-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18286-1">
    <skos:broader rdf:resource="descriptor/15745-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12490-4">
    <skos:broader rdf:resource="thsys/70578"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12490-4">
    <skos:broader rdf:resource="descriptor/10011-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13510-5">
    <skos:altLabel xml:lang="en">Transport</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13510-5">
    <skos:altLabel xml:lang="de">Verkehrswesen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13510-5">
    <skos:altLabel xml:lang="de">Inlandverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13510-5">
    <skos:altLabel xml:lang="de">Binnenverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13510-5">
    <skos:altLabel xml:lang="de">Beförderung (Verkehr)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16298-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14765-2">
    <skos:prefLabel xml:lang="en">Computer monitor</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14765-2">
    <skos:prefLabel xml:lang="de">Bildschirm</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18447-3">
    <skos:prefLabel xml:lang="en">Arms industry policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18447-3">
    <skos:prefLabel xml:lang="de">Rüstungsindustriepolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19122-5">
    <skos:broader rdf:resource="thsys/70041"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19122-5">
    <skos:broader rdf:resource="descriptor/10202-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11320-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10970-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19140-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19464-5">
    <skos:related rdf:resource="descriptor/18993-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19464-5">
    <skos:related rdf:resource="descriptor/18459-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11302-1">
    <skos:prefLabel xml:lang="en">Managers</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11302-1">
    <skos:prefLabel xml:lang="de">Führungskräfte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18356-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18404-0">
    <skos:altLabel xml:lang="de">Ministahlwerk</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70424">
    <skos:narrower rdf:resource="descriptor/14364-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70424">
    <skos:narrower rdf:resource="descriptor/14359-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70424">
    <skos:narrower rdf:resource="descriptor/14357-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70424">
    <skos:narrower rdf:resource="descriptor/14355-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70424">
    <skos:narrower rdf:resource="descriptor/14354-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70424">
    <skos:narrower rdf:resource="descriptor/14353-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70424">
    <skos:narrower rdf:resource="descriptor/14350-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11907-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11765-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14440-2">
    <skos:prefLabel xml:lang="en">Passenger car</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14440-2">
    <skos:prefLabel xml:lang="de">Personenkraftwagen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15694-1">
    <skos:related rdf:resource="descriptor/15723-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15694-1">
    <skos:related rdf:resource="descriptor/19611-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15694-1">
    <skos:related rdf:resource="descriptor/16680-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15694-1">
    <skos:related rdf:resource="descriptor/18924-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15694-1">
    <skos:related rdf:resource="descriptor/19781-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15694-1">
    <skos:related rdf:resource="descriptor/15654-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15694-1">
    <skos:related rdf:resource="descriptor/16556-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15694-1">
    <skos:related rdf:resource="descriptor/19634-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14496-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11424-1">
    <skos:altLabel xml:lang="en">Money supply multiplier</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11424-1">
    <skos:altLabel xml:lang="en">Money multiplier</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11424-1">
    <skos:altLabel xml:lang="de">Kreditschöpfungsmultiplikator</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11424-1">
    <skos:altLabel xml:lang="de">Geldschöpfungsmultiplikator</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11424-1">
    <skos:altLabel xml:lang="de">Geldmultiplikator</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11424-1">
    <skos:altLabel xml:lang="de">Geldmengenmultiplikator</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11424-1">
    <skos:altLabel xml:lang="de">Geldangebotsmultiplikator</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16100-2">
    <skos:prefLabel xml:lang="en">Budget law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16100-2">
    <skos:prefLabel xml:lang="de">Haushaltsrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18419-1">
    <skos:broader rdf:resource="thsys/70998"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18419-1">
    <skos:broader rdf:resource="thsys/70979"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18419-1">
    <skos:broader rdf:resource="thsys/70931"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18419-1">
    <skos:broader rdf:resource="descriptor/11820-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18419-1">
    <skos:broader rdf:resource="descriptor/10437-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19436-3">
    <skos:narrower rdf:resource="descriptor/11713-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12038-1">
    <skos:narrower rdf:resource="descriptor/19249-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19058-2">
    <skos:broader rdf:resource="thsys/70883"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19058-2">
    <skos:broader rdf:resource="descriptor/19057-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18552-6">
    <skos:prefLabel xml:lang="en">Electronic shopping</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18552-6">
    <skos:prefLabel xml:lang="de">Teleshopping</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19077-5">
    <skos:related rdf:resource="descriptor/19837-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19077-5">
    <skos:related rdf:resource="descriptor/17179-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19077-5">
    <skos:related rdf:resource="descriptor/17461-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19077-5">
    <skos:related rdf:resource="descriptor/19084-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19077-5">
    <skos:related rdf:resource="descriptor/19083-3"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id42">
    <rdf:value xml:lang="de">Für gehäuftes Auftreten von Insolvenzen im Bankensektor</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12874-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13890-4">
    <skos:altLabel xml:lang="de">Radiotechnikerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13890-4">
    <skos:altLabel xml:lang="de">Radiotechniker</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13890-4">
    <skos:altLabel xml:lang="de">Fernsehtechnikerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13890-4">
    <skos:altLabel xml:lang="de">Fernsehtechniker</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13890-4">
    <skos:altLabel xml:lang="de">Elektromonteur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13890-4">
    <skos:altLabel xml:lang="de">Elektromechaniker</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13890-4">
    <skos:altLabel xml:lang="de">Elektroinstallateurinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13890-4">
    <skos:altLabel xml:lang="de">Elektroinstallateure</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13890-4">
    <skos:altLabel xml:lang="de">Elektroindustriearbeiterinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13890-4">
    <skos:altLabel xml:lang="de">Elektroindustriearbeiter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13890-4">
    <skos:altLabel xml:lang="de">Elektrikerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13890-4">
    <skos:altLabel xml:lang="de">Elektriker</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17477-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10343-6">
    <skos:broader rdf:resource="thsys/70021"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70368">
    <skos:broader rdf:resource="thsys/70301"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18367-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21041-4">
    <skos:scopeNote xml:lang="de">Angebot eines kompletten Anlagen-Dienstleistungs-Bündels (Hardware/Software-Kombination) im Investitionsgüter-Marketing, z.B. im Großanlagengeschäft. Grundsätzlich auch im Konsumgüter-Marketing möglich.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10603-6">
    <skos:prefLabel xml:lang="en">Economic assistance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10603-6">
    <skos:prefLabel xml:lang="de">Internationale Wirtschaftshilfe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13391-1">
    <skos:broader rdf:resource="thsys/70875"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18712-3">
    <skos:related rdf:resource="descriptor/19544-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18712-3">
    <skos:related rdf:resource="descriptor/24665-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18712-3">
    <skos:related rdf:resource="descriptor/18718-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11418-3">
    <skos:related rdf:resource="descriptor/11424-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11418-3">
    <skos:related rdf:resource="descriptor/10382-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11418-3">
    <skos:related rdf:resource="descriptor/11448-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11418-3">
    <skos:related rdf:resource="descriptor/11446-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17597-3">
    <skos:prefLabel xml:lang="en">Bangladesh</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17597-3">
    <skos:prefLabel xml:lang="de">Bangladesch</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19428-2">
    <zbwext:useInsteadNote rdf:nodeID="id61"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13176-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10908-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#narrowMatch">
    <rdfs:label xml:lang="en">narrower match</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13862-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70259">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15757-3">
    <skos:related rdf:resource="descriptor/12605-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15757-3">
    <skos:related rdf:resource="descriptor/18030-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15757-3">
    <skos:related rdf:resource="descriptor/18368-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15757-3">
    <skos:related rdf:resource="descriptor/19529-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15757-3">
    <skos:related rdf:resource="descriptor/15013-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15757-3">
    <skos:related rdf:resource="descriptor/12976-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15757-3">
    <skos:related rdf:resource="descriptor/18445-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16299-2">
    <skos:related rdf:resource="descriptor/19690-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16299-2">
    <skos:related rdf:resource="descriptor/13420-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16299-2">
    <skos:related rdf:resource="descriptor/16300-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16299-2">
    <skos:related rdf:resource="descriptor/16416-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16299-2">
    <skos:related rdf:resource="descriptor/19235-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16299-2">
    <skos:related rdf:resource="descriptor/16264-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16299-2">
    <skos:related rdf:resource="descriptor/18062-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16299-2">
    <skos:related rdf:resource="descriptor/16512-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16299-2">
    <skos:related rdf:resource="descriptor/16587-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16299-2">
    <skos:related rdf:resource="descriptor/15797-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16299-2">
    <skos:related rdf:resource="descriptor/15918-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13421-4">
    <skos:altLabel xml:lang="de">Teilrepublik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13421-4">
    <skos:altLabel xml:lang="de">Kanton</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13421-4">
    <skos:altLabel xml:lang="de">Gliedstaat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13421-4">
    <skos:altLabel xml:lang="de">Bundesland</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10572-5">
    <skos:related rdf:resource="descriptor/10472-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10572-5">
    <skos:related rdf:resource="descriptor/19235-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10572-5">
    <skos:related rdf:resource="descriptor/20972-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10572-5">
    <skos:related rdf:resource="descriptor/10897-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10572-5">
    <skos:related rdf:resource="descriptor/10555-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10572-5">
    <skos:related rdf:resource="descriptor/10823-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10572-5">
    <skos:related rdf:resource="descriptor/19315-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13124-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14713-0">
    <skos:broader rdf:resource="thsys/70476"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14713-0">
    <skos:broader rdf:resource="descriptor/14710-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70241">
    <skos:broader rdf:resource="thsys/70154"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18388-0">
    <skos:related rdf:resource="descriptor/14236-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18388-0">
    <skos:related rdf:resource="descriptor/11813-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18388-0">
    <skos:related rdf:resource="descriptor/12980-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18790-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70142">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14154-3">
    <skos:broader rdf:resource="thsys/70465"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14154-3">
    <skos:broader rdf:resource="descriptor/14150-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12280-1">
    <skos:related rdf:resource="descriptor/11777-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12280-1">
    <skos:related rdf:resource="descriptor/10021-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12280-1">
    <skos:related rdf:resource="descriptor/12212-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12280-1">
    <skos:related rdf:resource="descriptor/12285-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12280-1">
    <skos:related rdf:resource="descriptor/10120-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12280-1">
    <skos:related rdf:resource="descriptor/12281-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12280-1">
    <skos:related rdf:resource="descriptor/12287-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12280-1">
    <skos:related rdf:resource="descriptor/24666-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12280-1">
    <skos:related rdf:resource="descriptor/18674-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12280-1">
    <skos:related rdf:resource="descriptor/12214-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13610-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14175-2">
    <skos:related rdf:resource="descriptor/12975-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14175-2">
    <skos:related rdf:resource="descriptor/11812-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14175-2">
    <skos:related rdf:resource="descriptor/14176-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14175-2">
    <skos:related rdf:resource="descriptor/15993-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14175-2">
    <skos:related rdf:resource="descriptor/11810-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14175-2">
    <skos:related rdf:resource="descriptor/18350-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14175-2">
    <skos:related rdf:resource="descriptor/18996-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19643-5">
    <skos:related rdf:resource="descriptor/16566-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14227-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12779-1">
    <skos:altLabel xml:lang="en">World exposition</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12779-1">
    <skos:altLabel xml:lang="de">Weltausstellung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12779-1">
    <skos:altLabel xml:lang="de">Gartenschau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12779-1">
    <skos:altLabel xml:lang="de">Gartenbauausstellung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12779-1">
    <skos:altLabel xml:lang="de">Festival</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19798-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15017-1">
    <skos:altLabel xml:lang="en">Toxic waste</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15017-1">
    <skos:altLabel xml:lang="de">Sondermüll</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15017-1">
    <skos:altLabel xml:lang="de">Giftmüll</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15017-1">
    <skos:altLabel xml:lang="de">Giftiger Abfall</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15017-1">
    <skos:altLabel xml:lang="de">Gefährlicher Abfall</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13993-1">
    <zbwext:useInsteadNote rdf:nodeID="id353"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71073">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70486">
    <skos:prefLabel xml:lang="en">V.03.03  Capital</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70486">
    <skos:prefLabel xml:lang="de">V.03.03  Kapital</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13429-2">
    <skos:prefLabel xml:lang="en">Agricultural cooperative</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13429-2">
    <skos:prefLabel xml:lang="de">Agrargenossenschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71113">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24740-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12727-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18334-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11919-2">
    <skos:narrower rdf:resource="descriptor/19444-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11919-2">
    <skos:narrower rdf:resource="descriptor/19443-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10426-2">
    <skos:broader rdf:resource="thsys/71070"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10426-2">
    <skos:broader rdf:resource="thsys/70406"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10426-2">
    <skos:broader rdf:resource="descriptor/10434-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10426-2">
    <skos:broader rdf:resource="descriptor/11912-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13546-5">
    <skos:broader rdf:resource="thsys/70936"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id241">
    <rdfs:seeAlso rdf:resource="descriptor/10249-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70254">
    <skos:narrower rdf:resource="descriptor/14496-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70254">
    <skos:narrower rdf:resource="descriptor/18251-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70254">
    <skos:narrower rdf:resource="descriptor/14514-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70254">
    <skos:narrower rdf:resource="descriptor/18223-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70254">
    <skos:narrower rdf:resource="descriptor/14487-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70254">
    <skos:narrower rdf:resource="descriptor/14480-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70254">
    <skos:narrower rdf:resource="descriptor/18290-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70254">
    <skos:narrower rdf:resource="descriptor/18103-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70254">
    <skos:narrower rdf:resource="descriptor/14472-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70254">
    <skos:narrower rdf:resource="descriptor/18229-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13362-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19797-6">
    <skos:broader rdf:resource="thsys/73344"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19797-6">
    <skos:broader rdf:resource="descriptor/10470-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16739-0">
    <skos:altLabel xml:lang="de">Marginalismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16739-0">
    <skos:altLabel xml:lang="de">Grenznutzenschule</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18568-5">
    <skos:prefLabel xml:lang="en">Computer-integrated railroading</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18568-5">
    <skos:prefLabel xml:lang="de">Computer-Integrated Railroading</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12366-5">
    <skos:broader rdf:resource="thsys/71043"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12366-5">
    <skos:broader rdf:resource="thsys/70036"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12366-5">
    <skos:broader rdf:resource="descriptor/12354-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12174-0">
    <skos:broader rdf:resource="thsys/70087"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12174-0">
    <skos:broader rdf:resource="descriptor/12186-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19442-1">
    <skos:prefLabel xml:lang="en">Thermodynamics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19442-1">
    <skos:prefLabel xml:lang="de">Thermodynamik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13962-5">
    <skos:prefLabel xml:lang="en">Technical professions</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13962-5">
    <skos:prefLabel xml:lang="de">Technische Berufe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14170-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11510-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19586-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11686-6">
    <skos:altLabel xml:lang="en">Ability-to-pay principle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11686-6">
    <skos:altLabel xml:lang="en">Ability theory</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11686-6">
    <skos:altLabel xml:lang="de">Opfertheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11686-6">
    <skos:altLabel xml:lang="de">Leistungsfähigkeitstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12510-3">
    <skos:prefLabel xml:lang="en">Distribution costs</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12510-3">
    <skos:prefLabel xml:lang="de">Vertriebskosten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14055-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18518-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12203-5">
    <skos:related rdf:resource="descriptor/20277-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19740-0">
    <skos:broader rdf:resource="thsys/73388"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19740-0">
    <skos:broader rdf:resource="descriptor/15640-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17641-5">
    <skos:prefLabel xml:lang="en">Cyprus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17641-5">
    <skos:prefLabel xml:lang="de">Zypern</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18232-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16301-6">
    <skos:scopeNote xml:lang="de">Bevollmächtigte zur Rechtsschutzsicherung und Überwachung einzelner Verwaltungszweige, z.B. Wehrbeauftragter des Bundestages.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11201-0">
    <skos:altLabel xml:lang="en">Child labor</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18964-4">
    <skos:altLabel xml:lang="de">Abfallaufbereitungstechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13660-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10973-1">
    <skos:related rdf:resource="descriptor/19829-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10973-1">
    <skos:related rdf:resource="descriptor/16611-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10973-1">
    <skos:related rdf:resource="descriptor/18120-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10973-1">
    <skos:related rdf:resource="descriptor/11817-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19420-4">
    <skos:broader rdf:resource="thsys/71074"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19420-4">
    <skos:broader rdf:resource="descriptor/11547-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20921-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15021-3">
    <skos:scopeNote xml:lang="de">Qualitatives Prognoseverfahren durch Expertenbefragung und -beurteilung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18179-2">
    <skos:broader rdf:resource="thsys/70822"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18179-2">
    <skos:broader rdf:resource="descriptor/18941-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18179-2">
    <skos:broader rdf:resource="descriptor/16792-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18179-2">
    <skos:broader rdf:resource="descriptor/18952-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12265-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13992-3">
    <skos:altLabel xml:lang="en">Entrepreneurship</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13992-3">
    <skos:altLabel xml:lang="de">Unternehmertum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13992-3">
    <skos:altLabel xml:lang="de">Unternehmerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13992-3">
    <skos:altLabel xml:lang="de">Unternehmergeist</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13992-3">
    <skos:altLabel xml:lang="de">Kapitalistinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13992-3">
    <skos:altLabel xml:lang="de">Kapitalisten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13992-3">
    <skos:altLabel xml:lang="de">Industrielle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13992-3">
    <skos:altLabel xml:lang="de">Geschäftsinhaber</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13992-3">
    <skos:altLabel xml:lang="de">Fabrikbesitzerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13992-3">
    <skos:altLabel xml:lang="de">Fabrikbesitzer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13992-3">
    <skos:altLabel xml:lang="de">Fabrikantinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13992-3">
    <skos:altLabel xml:lang="de">Fabrikanten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13992-3">
    <skos:altLabel xml:lang="de">Betriebsinhaberinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13992-3">
    <skos:altLabel xml:lang="de">Betriebsinhaber</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13992-3">
    <skos:altLabel xml:lang="de">Arbeitgeberinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13992-3">
    <skos:altLabel xml:lang="de">Arbeitgeber</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15908-1">
    <skos:broader rdf:resource="thsys/73356"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15908-1">
    <skos:broader rdf:resource="thsys/70839"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15908-1">
    <skos:broader rdf:resource="thsys/70573"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15908-1">
    <skos:broader rdf:resource="thsys/70142"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15908-1">
    <skos:broader rdf:resource="descriptor/15909-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17630-3">
    <skos:altLabel xml:lang="de">Libanesische Republik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16502-3">
    <skos:related rdf:resource="descriptor/16809-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16502-3">
    <skos:related rdf:resource="descriptor/19631-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16502-3">
    <skos:related rdf:resource="descriptor/16435-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16502-3">
    <skos:related rdf:resource="descriptor/16441-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16502-3">
    <skos:related rdf:resource="descriptor/19299-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16502-3">
    <skos:related rdf:resource="descriptor/15872-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16502-3">
    <skos:related rdf:resource="descriptor/16489-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18647-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11559-6">
    <skos:broader rdf:resource="thsys/73380"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11559-6">
    <skos:broader rdf:resource="thsys/71054"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11559-6">
    <skos:broader rdf:resource="thsys/70872"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19043-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14802-1">
    <skos:narrower rdf:resource="descriptor/14809-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14802-1">
    <skos:narrower rdf:resource="descriptor/18205-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14802-1">
    <skos:narrower rdf:resource="descriptor/14796-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14599-0">
    <skos:broader rdf:resource="thsys/70014"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12187-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13245-5">
    <skos:narrower rdf:resource="descriptor/18356-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11036-3">
    <skos:altLabel xml:lang="en">Workers self-management</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11036-3">
    <skos:altLabel xml:lang="en">Labour-managed firm</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11036-3">
    <skos:altLabel xml:lang="en">Labour-managed economy</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11036-3">
    <skos:altLabel xml:lang="en">Labor-managed firm</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11036-3">
    <skos:altLabel xml:lang="en">Labor-managed economy</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11036-3">
    <skos:altLabel xml:lang="en">Labor managed firm</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11036-3">
    <skos:altLabel xml:lang="en">Labor managed economy</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11036-3">
    <skos:altLabel xml:lang="en">Illyrian firm</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11036-3">
    <skos:altLabel xml:lang="de">Selbstverwaltungsökonomie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11036-3">
    <skos:altLabel xml:lang="de">Selbstverwalteter Betrieb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id308">
    <rdfs:seeAlso rdf:resource="descriptor/11998-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13785-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70903">
    <skos:broader rdf:resource="thsys/70905"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12625-0">
    <skos:prefLabel xml:lang="en">Job enlargement</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12625-0">
    <skos:prefLabel xml:lang="de">Job Enlargement</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10966-5">
    <skos:prefLabel xml:lang="en">Business association</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10966-5">
    <skos:prefLabel xml:lang="de">Wirtschaftsverband</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16484-0">
    <skos:related rdf:resource="descriptor/11349-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16484-0">
    <skos:related rdf:resource="descriptor/11350-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16484-0">
    <skos:related rdf:resource="descriptor/10401-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16484-0">
    <skos:related rdf:resource="descriptor/11337-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16484-0">
    <skos:related rdf:resource="descriptor/11346-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16484-0">
    <skos:related rdf:resource="descriptor/19503-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16484-0">
    <skos:related rdf:resource="descriptor/18123-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17722-5">
    <skos:broader rdf:resource="thsys/70560"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17722-5">
    <skos:broader rdf:resource="descriptor/17724-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16523-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18879-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19486-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10604-4">
    <skos:prefLabel xml:lang="en">Export market</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10604-4">
    <skos:prefLabel xml:lang="de">Auslandsmarkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12137-6">
    <skos:prefLabel xml:lang="en">Decentralized management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12137-6">
    <skos:prefLabel xml:lang="de">Dezentrale Organisation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13431-1">
    <skos:narrower rdf:resource="descriptor/19306-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13431-1">
    <skos:narrower rdf:resource="descriptor/19349-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13431-1">
    <skos:narrower rdf:resource="descriptor/19347-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13431-1">
    <skos:narrower rdf:resource="descriptor/19342-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13431-1">
    <skos:narrower rdf:resource="descriptor/19183-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13431-1">
    <skos:narrower rdf:resource="descriptor/19296-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13431-1">
    <skos:narrower rdf:resource="descriptor/19294-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13431-1">
    <skos:narrower rdf:resource="descriptor/19188-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13431-1">
    <skos:narrower rdf:resource="descriptor/19341-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13431-1">
    <skos:narrower rdf:resource="descriptor/19340-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13431-1">
    <skos:narrower rdf:resource="descriptor/19290-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18074-2">
    <skos:altLabel xml:lang="en">Teachware</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18074-2">
    <skos:altLabel xml:lang="en">Electronic learning</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18074-2">
    <skos:altLabel xml:lang="en">Computer-based training</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18074-2">
    <skos:altLabel xml:lang="en">Computer-assisted learning</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18074-2">
    <skos:altLabel xml:lang="en">Computer-assisted instruction</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18074-2">
    <skos:altLabel xml:lang="en">CBT (Computer-Based Training)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18074-2">
    <skos:altLabel xml:lang="en">CAL (Computer-Assisted Learning)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18074-2">
    <skos:altLabel xml:lang="en">CAI (Computer-Assisted Instruction)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18074-2">
    <skos:altLabel xml:lang="de">Lernprogramm</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18074-2">
    <skos:altLabel xml:lang="de">E-Learning</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18074-2">
    <skos:altLabel xml:lang="de">Computergestützte Ausbildung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15771-2">
    <skos:related rdf:resource="descriptor/12928-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15771-2">
    <skos:related rdf:resource="descriptor/15773-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15771-2">
    <skos:related rdf:resource="descriptor/15767-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15771-2">
    <skos:related rdf:resource="descriptor/18165-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11298-1">
    <skos:prefLabel xml:lang="en">Occupational structure</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11298-1">
    <skos:prefLabel xml:lang="de">Berufsstruktur</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14924-1">
    <skos:broader rdf:resource="thsys/70498"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14924-1">
    <skos:broader rdf:resource="descriptor/14931-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14924-1">
    <skos:broader rdf:resource="descriptor/14019-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70134">
    <skos:narrower rdf:resource="descriptor/14310-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70134">
    <skos:narrower rdf:resource="descriptor/14309-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70134">
    <skos:narrower rdf:resource="descriptor/14222-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70134">
    <skos:narrower rdf:resource="descriptor/18240-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70134">
    <skos:narrower rdf:resource="descriptor/14308-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70134">
    <skos:narrower rdf:resource="descriptor/18239-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70134">
    <skos:narrower rdf:resource="descriptor/14307-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70134">
    <skos:narrower rdf:resource="descriptor/14220-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70134">
    <skos:narrower rdf:resource="descriptor/14306-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70134">
    <skos:narrower rdf:resource="descriptor/14305-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70134">
    <skos:narrower rdf:resource="descriptor/14302-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70134">
    <skos:narrower rdf:resource="descriptor/18238-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70134">
    <skos:narrower rdf:resource="descriptor/14301-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70134">
    <skos:narrower rdf:resource="descriptor/14300-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70134">
    <skos:narrower rdf:resource="descriptor/14299-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70134">
    <skos:narrower rdf:resource="descriptor/14298-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70134">
    <skos:narrower rdf:resource="descriptor/14296-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70134">
    <skos:narrower rdf:resource="descriptor/14295-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70134">
    <skos:narrower rdf:resource="descriptor/18237-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70134">
    <skos:narrower rdf:resource="descriptor/14294-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70134">
    <skos:narrower rdf:resource="descriptor/14293-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70134">
    <skos:narrower rdf:resource="descriptor/14292-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18882-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15605-5">
    <skos:related rdf:resource="descriptor/19810-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15605-5">
    <skos:related rdf:resource="descriptor/18643-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15605-5">
    <skos:related rdf:resource="descriptor/16757-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15605-5">
    <skos:related rdf:resource="descriptor/15606-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15605-5">
    <skos:related rdf:resource="descriptor/14752-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15605-5">
    <skos:related rdf:resource="descriptor/15604-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15605-5">
    <skos:related rdf:resource="descriptor/19800-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15605-5">
    <skos:related rdf:resource="descriptor/18433-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15605-5">
    <skos:related rdf:resource="descriptor/16190-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15488-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11844-0">
    <skos:altLabel xml:lang="de">Stadtumbau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11844-0">
    <skos:altLabel xml:lang="de">Stadthausmodell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11844-0">
    <skos:altLabel xml:lang="de">Stadtgestaltung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11844-0">
    <skos:altLabel xml:lang="de">Stadterhaltung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11844-0">
    <skos:altLabel xml:lang="de">Städtebaurecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11844-0">
    <skos:altLabel xml:lang="de">Städtebauförderungsgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11844-0">
    <skos:altLabel xml:lang="de">Städtebauförderung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19637-0">
    <skos:broader rdf:resource="thsys/70142"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19637-0">
    <skos:broader rdf:resource="descriptor/15909-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73381">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18312-5">
    <skos:broader rdf:resource="thsys/73390"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18312-5">
    <skos:broader rdf:resource="thsys/73332"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18312-5">
    <skos:broader rdf:resource="thsys/71018"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18312-5">
    <skos:broader rdf:resource="descriptor/16081-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19761-6">
    <skos:broader rdf:resource="thsys/73357"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18534-1">
    <skos:prefLabel xml:lang="en">History of foreign trade</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18534-1">
    <skos:prefLabel xml:lang="de">Außenhandelsgeschichte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16602-6">
    <skos:prefLabel xml:lang="en">Social order</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16602-6">
    <skos:prefLabel xml:lang="de">Gesellschaftsordnung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14738-5">
    <skos:broader rdf:resource="thsys/70536"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14738-5">
    <skos:broader rdf:resource="descriptor/14599-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15117-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10576-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15740-6">
    <skos:prefLabel xml:lang="en">Noise control</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15740-6">
    <skos:prefLabel xml:lang="de">Lärmschutz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13873-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19712-5">
    <skos:broader rdf:resource="thsys/73366"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19712-5">
    <skos:broader rdf:resource="descriptor/19708-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19712-5">
    <skos:broader rdf:resource="descriptor/18775-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19067-1">
    <skos:related rdf:resource="descriptor/14003-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18060-6">
    <skos:prefLabel xml:lang="en">Deindustrialization</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18060-6">
    <skos:prefLabel xml:lang="de">Deindustrialisierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18320-6">
    <skos:prefLabel xml:lang="en">Agroindustry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18320-6">
    <skos:prefLabel xml:lang="de">Agroindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19482-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19398-6">
    <skos:prefLabel xml:lang="en">Public Land</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19398-6">
    <skos:prefLabel xml:lang="de">Öffentliche Immobilien</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10011-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19747-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18534-1">
    <skos:broader rdf:resource="thsys/73387"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18534-1">
    <skos:broader rdf:resource="thsys/71091"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18534-1">
    <skos:broader rdf:resource="thsys/70109"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18534-1">
    <skos:broader rdf:resource="descriptor/13462-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73366">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13527-2">
    <skos:altLabel xml:lang="de">Schienennetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13527-2">
    <skos:altLabel xml:lang="de">Schieneninfrastruktur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13527-2">
    <skos:altLabel xml:lang="de">Eisenbahnstrecke</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13527-2">
    <skos:altLabel xml:lang="de">Bahnstrecke</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15518-0">
    <skos:broader rdf:resource="thsys/71035"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15518-0">
    <skos:broader rdf:resource="thsys/70927"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15518-0">
    <skos:broader rdf:resource="thsys/70386"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15518-0">
    <skos:broader rdf:resource="descriptor/12761-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13773-1">
    <skos:broader rdf:resource="thsys/70905"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13773-1">
    <skos:broader rdf:resource="descriptor/19847-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18825-4">
    <skos:altLabel xml:lang="de">Verwaltungsinformationssystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10068-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18102-2">
    <skos:altLabel xml:lang="de">Elektronische Betriebsdatenerfassung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18102-2">
    <skos:altLabel xml:lang="de">BDE (Betriebsdatenerfassung)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="../stw">
    <cc:morePermissions xml:lang="en">http://zbw.eu/e_imprint.htm</cc:morePermissions>
  </rdf:Description>
  <rdf:Description rdf:about="../stw">
    <cc:morePermissions xml:lang="de">http://zbw.eu/impressum.htm</cc:morePermissions>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11473-2">
    <skos:related rdf:resource="descriptor/11469-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11473-2">
    <skos:related rdf:resource="descriptor/10250-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11473-2">
    <skos:related rdf:resource="descriptor/11452-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11473-2">
    <skos:related rdf:resource="descriptor/11498-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11473-2">
    <skos:related rdf:resource="descriptor/11444-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11473-2">
    <skos:related rdf:resource="descriptor/11443-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70076">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">G.03</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10057-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19385-1">
    <skos:related rdf:resource="descriptor/12978-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19385-1">
    <skos:related rdf:resource="descriptor/18371-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19385-1">
    <skos:related rdf:resource="descriptor/12983-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19385-1">
    <skos:related rdf:resource="descriptor/14196-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11806-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14273-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12406-5">
    <skos:related rdf:resource="descriptor/12385-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12406-5">
    <skos:related rdf:resource="descriptor/12361-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12406-5">
    <skos:related rdf:resource="descriptor/12356-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15628-0">
    <skos:broader rdf:resource="thsys/73350"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15628-0">
    <skos:broader rdf:resource="thsys/71013"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15628-0">
    <skos:broader rdf:resource="thsys/70227"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15628-0">
    <skos:broader rdf:resource="descriptor/18348-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16597-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72180">
    <skos:broader rdf:resource="thsys/v"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11267-5">
    <skos:altLabel xml:lang="de">Lehrstellenangebot</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11267-5">
    <skos:altLabel xml:lang="de">Ausbildungsstellenmarkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11267-5">
    <skos:altLabel xml:lang="de">Ausbildungsplatzsituation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11267-5">
    <skos:altLabel xml:lang="de">Ausbildungsplatznachfrage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11267-5">
    <skos:altLabel xml:lang="de">Ausbildungsplatzangebot</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11267-5">
    <skos:altLabel xml:lang="de">Ausbildungsbedarf</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14019-2">
    <skos:related rdf:resource="descriptor/14581-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14019-2">
    <skos:related rdf:resource="descriptor/14909-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14019-2">
    <skos:related rdf:resource="descriptor/15925-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14024-2">
    <skos:scopeNote xml:lang="de">Bürobedarf aus Holz, Kunststoff und anderen Materialien sowie aus dem Bereich der Chemikalien, z.B. Spezialpapier, Tusche, Tinte, usw.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11127-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10043-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18168-0">
    <skos:altLabel xml:lang="en">Labor market forecast</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18168-0">
    <skos:altLabel xml:lang="de">Beschäftigungsprognose</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18168-0">
    <skos:altLabel xml:lang="de">Arbeitslosigkeitsprognose</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13371-0">
    <skos:altLabel xml:lang="de">Gesundheitssektor</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13371-0">
    <skos:altLabel xml:lang="de">Gesundheitspflege</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16794-6">
    <skos:broader rdf:resource="thsys/70822"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16794-6">
    <skos:broader rdf:resource="descriptor/18949-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10252-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11915-3">
    <skos:altLabel xml:lang="de">Standortwahlverhalten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11915-3">
    <skos:altLabel xml:lang="de">Standortverhalten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11915-3">
    <skos:altLabel xml:lang="de">Standortentscheidung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id310">
    <rdfs:seeAlso rdf:resource="descriptor/11905-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14274-0">
    <skos:prefLabel xml:lang="en">Heavy metal</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14274-0">
    <skos:prefLabel xml:lang="de">Schwermetall</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18687-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12603-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15797-5">
    <zbwext:useInsteadNote rdf:nodeID="id94"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18765-3">
    <skos:broader rdf:resource="thsys/73340"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18765-3">
    <skos:broader rdf:resource="descriptor/18766-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70242">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18702-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16384-4">
    <skos:broader rdf:resource="thsys/73382"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16384-4">
    <skos:broader rdf:resource="thsys/70853"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16384-4">
    <skos:broader rdf:resource="descriptor/16292-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13687-1">
    <skos:related rdf:resource="descriptor/12277-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13687-1">
    <skos:related rdf:resource="descriptor/18728-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13687-1">
    <skos:related rdf:resource="descriptor/13693-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13687-1">
    <skos:related rdf:resource="descriptor/13692-1"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id18">
    <rdfs:seeAlso rdf:resource="descriptor/12609-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15897-1">
    <skos:altLabel xml:lang="de">Waldressourcen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15897-1">
    <skos:altLabel xml:lang="de">Waldbestand</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15897-1">
    <skos:altLabel xml:lang="de">Forst</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18971-0">
    <skos:related rdf:resource="descriptor/13398-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18971-0">
    <skos:related rdf:resource="descriptor/15938-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18971-0">
    <skos:related rdf:resource="descriptor/18968-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18442-6">
    <skos:altLabel xml:lang="de">Nahrungsmittelforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11929-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13379-5">
    <skos:related rdf:resource="descriptor/18017-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12521-5">
    <skos:broader rdf:resource="thsys/70578"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12521-5">
    <skos:broader rdf:resource="descriptor/12480-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18944-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10156-5">
    <skos:altLabel xml:lang="en">Intertemporal utility</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10156-5">
    <skos:altLabel xml:lang="de">Langfristige Nutzenfunktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10156-5">
    <skos:altLabel xml:lang="de">Intertemporaler Nutzen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10156-5">
    <skos:altLabel xml:lang="de">Intertemporale Präferenzen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10156-5">
    <skos:altLabel xml:lang="de">Intertemporale Nutzenfunktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14981-3">
    <skos:broader rdf:resource="thsys/73300"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14981-3">
    <skos:broader rdf:resource="descriptor/14985-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18716-2">
    <skos:narrower rdf:resource="descriptor/18721-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11161-0">
    <skos:altLabel xml:lang="de">Theorie der Institutionen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11161-0">
    <skos:altLabel xml:lang="de">Institutionenökonomik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11161-0">
    <skos:altLabel xml:lang="de">Institutionelle Ökonomie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11161-0">
    <skos:altLabel xml:lang="de">Institutionalistische Schule</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10024-1">
    <skos:prefLabel xml:lang="en">Assets</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10024-1">
    <skos:prefLabel xml:lang="de">Vermögen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17991-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19535-1">
    <skos:broader rdf:resource="thsys/71107"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19535-1">
    <skos:broader rdf:resource="thsys/71046"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19535-1">
    <skos:broader rdf:resource="thsys/70899"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19535-1">
    <skos:broader rdf:resource="thsys/70893"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12930-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17773-2">
    <skos:broader rdf:resource="thsys/70086"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17773-2">
    <skos:broader rdf:resource="descriptor/17791-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16964-0">
    <skos:broader rdf:resource="thsys/70003"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16964-0">
    <skos:broader rdf:resource="descriptor/16972-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14300-4">
    <skos:related rdf:resource="descriptor/14643-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19520-0">
    <skos:prefLabel xml:lang="en">Trading company</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19520-0">
    <skos:prefLabel xml:lang="de">Handelskompanie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11111-1">
    <skos:prefLabel xml:lang="en">Theory of value</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11111-1">
    <skos:prefLabel xml:lang="de">Werttheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15408-0">
    <skos:related rdf:resource="descriptor/10139-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11325-3">
    <skos:related rdf:resource="descriptor/10622-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11325-3">
    <skos:related rdf:resource="descriptor/15089-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11325-3">
    <skos:related rdf:resource="descriptor/11322-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19209-0">
    <skos:altLabel xml:lang="de">Intertemporale Faktorallokation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19209-0">
    <skos:altLabel xml:lang="de">Dynamische Faktorallokation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19209-0">
    <skos:altLabel xml:lang="de">Dynamische Allokation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14049-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id348">
    <rdfs:seeAlso rdf:resource="descriptor/15861-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18154-4">
    <skos:altLabel xml:lang="de">Quellenabzugsverfahren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18154-4">
    <skos:altLabel xml:lang="de">Quellenabzugsprinzip</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18388-0">
    <skos:prefLabel xml:lang="en">Mineral resources</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18388-0">
    <skos:prefLabel xml:lang="de">Mineralische Ressourcen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13349-0">
    <skos:prefLabel xml:lang="en">Scientists</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13349-0">
    <skos:prefLabel xml:lang="de">Wissenschaftler</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13846-0">
    <skos:broader rdf:resource="thsys/70969"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13846-0">
    <skos:broader rdf:resource="thsys/70889"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13846-0">
    <skos:broader rdf:resource="descriptor/12905-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19505-3">
    <skos:prefLabel xml:lang="en">Occupational employment projection</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19505-3">
    <skos:prefLabel xml:lang="de">Berufsbedarf</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19934-1">
    <skos:related rdf:resource="descriptor/12438-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19934-1">
    <skos:related rdf:resource="descriptor/12427-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10843-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18200-2">
    <skos:altLabel xml:lang="de">Internationales Naturkautschuk-Übereinkommen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18200-2">
    <skos:altLabel xml:lang="de">Internationales Naturkautschukabkommen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18200-2">
    <skos:altLabel xml:lang="de">Internationales Kautschukabkommen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id16">
    <rdf:value xml:lang="de">Für Kultur/Sozialanthropologie</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11337-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18288-4">
    <skos:altLabel xml:lang="de">Gesetz zur Regelung von Fragen der Gentechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18288-4">
    <skos:altLabel xml:lang="de">GenTG (Gentechnikgesetz)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18288-4">
    <skos:altLabel xml:lang="de">Gentechnikgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18550-3">
    <skos:broader rdf:resource="thsys/70947"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18550-3">
    <skos:broader rdf:resource="descriptor/13200-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16525-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13632-5">
    <skos:narrower rdf:resource="descriptor/13644-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13632-5">
    <skos:narrower rdf:resource="descriptor/18578-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13632-5">
    <skos:narrower rdf:resource="descriptor/11893-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13632-5">
    <skos:narrower rdf:resource="descriptor/18589-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13632-5">
    <skos:narrower rdf:resource="descriptor/18611-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13632-5">
    <skos:narrower rdf:resource="descriptor/13249-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13632-5">
    <skos:narrower rdf:resource="descriptor/18609-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13632-5">
    <skos:narrower rdf:resource="descriptor/18574-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13632-5">
    <skos:narrower rdf:resource="descriptor/13629-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10624-5">
    <skos:related rdf:resource="descriptor/10619-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10624-5">
    <skos:related rdf:resource="descriptor/10625-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10624-5">
    <skos:related rdf:resource="descriptor/10064-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10624-5">
    <skos:related rdf:resource="descriptor/16809-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10624-5">
    <skos:related rdf:resource="descriptor/10617-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10624-5">
    <skos:related rdf:resource="descriptor/19299-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20261-2">
    <skos:related rdf:resource="descriptor/12108-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20261-2">
    <skos:related rdf:resource="descriptor/15711-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20261-2">
    <skos:related rdf:resource="descriptor/16226-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20261-2">
    <skos:related rdf:resource="descriptor/19154-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20261-2">
    <skos:related rdf:resource="descriptor/13714-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18542-2">
    <skos:related rdf:resource="descriptor/10816-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16159-4">
    <skos:related rdf:resource="descriptor/13419-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16159-4">
    <skos:related rdf:resource="descriptor/16318-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16159-4">
    <skos:related rdf:resource="descriptor/16545-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16159-4">
    <skos:related rdf:resource="descriptor/13417-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16159-4">
    <skos:related rdf:resource="descriptor/11406-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16159-4">
    <skos:related rdf:resource="descriptor/11185-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19552-1">
    <skos:broader rdf:resource="thsys/72393"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19552-1">
    <skos:broader rdf:resource="thsys/71112"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19552-1">
    <skos:broader rdf:resource="descriptor/10253-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19552-1">
    <skos:broader rdf:resource="descriptor/10181-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18587-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16695-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11233-1">
    <skos:broader rdf:resource="thsys/71061"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11233-1">
    <skos:broader rdf:resource="descriptor/11211-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11484-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11049-1">
    <skos:related rdf:resource="descriptor/11068-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11049-1">
    <skos:related rdf:resource="descriptor/11105-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11049-1">
    <skos:related rdf:resource="descriptor/11063-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11049-1">
    <skos:related rdf:resource="descriptor/11127-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11049-1">
    <skos:related rdf:resource="descriptor/16292-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11049-1">
    <skos:related rdf:resource="descriptor/11066-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11049-1">
    <skos:related rdf:resource="descriptor/19242-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10958-4">
    <skos:prefLabel xml:lang="en">Antitrust policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10958-4">
    <skos:prefLabel xml:lang="de">Konzentrationspolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19505-3">
    <skos:related rdf:resource="descriptor/11267-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19505-3">
    <skos:related rdf:resource="descriptor/11394-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19505-3">
    <skos:related rdf:resource="descriptor/11295-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19505-3">
    <skos:related rdf:resource="descriptor/11293-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19505-3">
    <skos:related rdf:resource="descriptor/11252-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19505-3">
    <skos:related rdf:resource="descriptor/11290-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19505-3">
    <skos:related rdf:resource="descriptor/11266-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19505-3">
    <skos:related rdf:resource="descriptor/10996-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18492-5">
    <skos:prefLabel xml:lang="en">Architecture</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18492-5">
    <skos:prefLabel xml:lang="de">Architektur</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12065-5">
    <skos:broader rdf:resource="thsys/70030"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19535-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17979-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19035-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18636-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13185-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10956-1">
    <skos:prefLabel xml:lang="en">Competition theory</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10956-1">
    <skos:prefLabel xml:lang="de">Wettbewerbstheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18750-2">
    <skos:narrower rdf:resource="descriptor/18502-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14117-2">
    <skos:broader rdf:resource="thsys/70169"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14117-2">
    <skos:broader rdf:resource="descriptor/18195-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14117-2">
    <skos:broader rdf:resource="descriptor/14110-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15554-3">
    <skos:broader rdf:resource="thsys/71035"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15554-3">
    <skos:broader rdf:resource="descriptor/15055-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11951-6">
    <skos:broader rdf:resource="thsys/71070"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11951-6">
    <skos:broader rdf:resource="thsys/70878"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11951-6">
    <skos:broader rdf:resource="descriptor/10436-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18911-4">
    <skos:broader rdf:resource="thsys/71059"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18911-4">
    <skos:broader rdf:resource="thsys/70833"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18911-4">
    <skos:broader rdf:resource="thsys/70324"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18911-4">
    <skos:broader rdf:resource="descriptor/11215-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18911-4">
    <skos:broader rdf:resource="descriptor/16477-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11727-4">
    <skos:prefLabel xml:lang="en">Built-in flexibility</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11727-4">
    <skos:prefLabel xml:lang="de">Eingebauter Stabilisator</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17716-0">
    <skos:prefLabel xml:lang="en">Niger</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17716-0">
    <skos:prefLabel xml:lang="de">Niger</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11243-5">
    <skos:broader rdf:resource="thsys/71061"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11243-5">
    <skos:broader rdf:resource="descriptor/11235-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13615-5">
    <skos:broader rdf:resource="thsys/70930"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13615-5">
    <skos:broader rdf:resource="descriptor/13622-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19653-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18605-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13480-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12727-6">
    <skos:narrower rdf:resource="descriptor/12722-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19211-6">
    <skos:related rdf:resource="descriptor/12697-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19211-6">
    <skos:related rdf:resource="descriptor/11160-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19211-6">
    <skos:related rdf:resource="descriptor/12009-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19437-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13690-5">
    <skos:prefLabel xml:lang="en">Banking system</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13690-5">
    <skos:prefLabel xml:lang="de">Bankensystem</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14914-4">
    <skos:altLabel xml:lang="de">Wirkwaren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14914-4">
    <skos:altLabel xml:lang="de">Trikotwaren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14914-4">
    <skos:altLabel xml:lang="de">Strickwaren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19845-0">
    <skos:broader rdf:resource="thsys/73360"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12015-6">
    <skos:related rdf:resource="descriptor/15754-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12015-6">
    <skos:related rdf:resource="descriptor/12029-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12015-6">
    <skos:related rdf:resource="descriptor/12026-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12015-6">
    <skos:related rdf:resource="descriptor/15873-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12015-6">
    <skos:related rdf:resource="descriptor/12021-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12015-6">
    <skos:related rdf:resource="descriptor/12020-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12015-6">
    <skos:related rdf:resource="descriptor/12018-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12015-6">
    <skos:related rdf:resource="descriptor/11921-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12015-6">
    <skos:related rdf:resource="descriptor/11859-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12015-6">
    <skos:related rdf:resource="descriptor/19454-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12015-6">
    <skos:related rdf:resource="descriptor/12016-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70192">
    <skos:prefLabel xml:lang="de">N.00  Nachbarwissenschaften</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16226-1">
    <skos:narrower rdf:resource="descriptor/11310-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16226-1">
    <skos:narrower rdf:resource="descriptor/16266-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16531-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14267-4">
    <skos:prefLabel xml:lang="en">Alloy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14267-4">
    <skos:prefLabel xml:lang="de">Legierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12309-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15888-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18144-0">
    <skos:related rdf:resource="descriptor/13398-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18144-0">
    <skos:related rdf:resource="descriptor/13974-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18144-0">
    <skos:related rdf:resource="descriptor/10176-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18144-0">
    <skos:related rdf:resource="descriptor/19521-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14751-6">
    <skos:narrower rdf:resource="descriptor/14762-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14751-6">
    <skos:narrower rdf:resource="descriptor/18661-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14751-6">
    <skos:narrower rdf:resource="descriptor/18004-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14751-6">
    <skos:narrower rdf:resource="descriptor/18270-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14751-6">
    <skos:narrower rdf:resource="descriptor/18665-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14751-6">
    <skos:narrower rdf:resource="descriptor/14755-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14751-6">
    <skos:narrower rdf:resource="descriptor/14749-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14751-6">
    <skos:narrower rdf:resource="descriptor/14765-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15774-3">
    <skos:broader rdf:resource="thsys/70815"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15774-3">
    <skos:broader rdf:resource="thsys/70239"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15774-3">
    <skos:broader rdf:resource="descriptor/10470-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13513-6">
    <skos:prefLabel xml:lang="en">Mean of transport</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13513-6">
    <skos:prefLabel xml:lang="de">Verkehrsmittel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20722-6">
    <skos:scopeNote xml:lang="de">Aktionsprogramm zu Umwelt- und Entwicklungsvorhaben der UN.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70184">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">B.08.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12640-4">
    <skos:related rdf:resource="descriptor/12648-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12640-4">
    <skos:related rdf:resource="descriptor/12665-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12640-4">
    <skos:related rdf:resource="descriptor/12035-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15776-6">
    <skos:prefLabel xml:lang="en">Emissions permit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15776-6">
    <skos:prefLabel xml:lang="de">Emissionsrechte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14057-1">
    <skos:broader rdf:resource="thsys/73304"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14057-1">
    <skos:broader rdf:resource="thsys/70341"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14057-1">
    <skos:broader rdf:resource="descriptor/14074-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14057-1">
    <skos:broader rdf:resource="descriptor/14851-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13747-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18521-3">
    <skos:broader rdf:resource="thsys/70960"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18521-3">
    <skos:broader rdf:resource="descriptor/19301-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18521-3">
    <skos:broader rdf:resource="descriptor/13219-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18521-3">
    <skos:broader rdf:resource="descriptor/13188-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14433-6">
    <zbwext:useInsteadNote rdf:nodeID="id175"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14642-4">
    <skos:related rdf:resource="descriptor/18239-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14642-4">
    <skos:related rdf:resource="descriptor/14205-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14642-4">
    <skos:related rdf:resource="descriptor/12995-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14642-4">
    <skos:related rdf:resource="descriptor/14292-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19096-1">
    <skos:broader rdf:resource="thsys/70578"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19096-1">
    <skos:broader rdf:resource="descriptor/12085-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10884-0">
    <skos:prefLabel xml:lang="en">Commodity exchange</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10884-0">
    <skos:prefLabel xml:lang="de">Warenbörse</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16324-1">
    <skos:altLabel xml:lang="de">Provinzregierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14942-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14973-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19132-2">
    <skos:scopeNote xml:lang="de">Alle Arten der Werbebeschränkung, z.B. bezüglich des Inhalts von Werbeaussagen oder der Nutzung bestimmter Medien, bis hin zum Werbeverbot.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16382-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70994">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19643-5">
    <skos:narrower rdf:resource="descriptor/15926-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19643-5">
    <skos:narrower rdf:resource="descriptor/15917-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19643-5">
    <skos:narrower rdf:resource="descriptor/15922-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19636-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17171-2">
    <skos:altLabel xml:lang="x-other">Shqiperise</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17171-2">
    <skos:altLabel xml:lang="de">Sozialistische Volksrepublik Albanien</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17171-2">
    <skos:altLabel xml:lang="de">Republik Albanien</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12647-4">
    <skos:altLabel xml:lang="de">Werkstoffprüfung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12647-4">
    <skos:altLabel xml:lang="de">Werkstoffkontrolle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12647-4">
    <skos:altLabel xml:lang="de">Warenprüfung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12647-4">
    <skos:altLabel xml:lang="de">Materialkontrolle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18350-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15721-3">
    <skos:altLabel xml:lang="de">Soziale Probleme</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15721-3">
    <skos:altLabel xml:lang="de">Soziale Frage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10107-4">
    <skos:altLabel xml:lang="de">Stimmentauschparadoxon</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id9">
    <rdf:value xml:lang="de">Für Activity-Based Costing</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24781-5">
    <skos:scopeNote xml:lang="de">"Kundenwert" ist einerseits die wirtschaftliche Langzeitbetrachtung eines Kunden aus Sicht eines Unternehmens; andererseits ist es aus der Kundenperspektive gesehen die Bewertung von Produkten/Dienstleistungen eines Unternehmens.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id21">
    <rdf:value xml:lang="de">Für betriebswirtschaftliche Arbeitskostenbetrachtungen</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19278-2">
    <skos:prefLabel xml:lang="en">Stock split</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19278-2">
    <skos:prefLabel xml:lang="de">Aktiensplit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13853-3">
    <skos:prefLabel xml:lang="en">Construction price</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13853-3">
    <skos:prefLabel xml:lang="de">Baupreis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11158-3">
    <skos:related rdf:resource="descriptor/19733-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11158-3">
    <skos:related rdf:resource="descriptor/12111-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11158-3">
    <skos:related rdf:resource="descriptor/12142-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11158-3">
    <skos:related rdf:resource="descriptor/11154-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11158-3">
    <skos:related rdf:resource="descriptor/19753-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11158-3">
    <skos:related rdf:resource="descriptor/12136-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13425-3">
    <skos:broader rdf:resource="thsys/73396"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13425-3">
    <skos:broader rdf:resource="thsys/71028"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13425-3">
    <skos:broader rdf:resource="descriptor/16602-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13425-3">
    <skos:broader rdf:resource="descriptor/16588-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18610-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10138-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19163-5">
    <skos:broader rdf:resource="thsys/70140"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19384-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18445-0">
    <skos:prefLabel xml:lang="en">Food preservation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18445-0">
    <skos:prefLabel xml:lang="de">Nahrungsmittelkonservierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11125-4">
    <skos:broader rdf:resource="thsys/71101"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11125-4">
    <skos:broader rdf:resource="thsys/71091"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11125-4">
    <skos:broader rdf:resource="thsys/70952"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11125-4">
    <skos:broader rdf:resource="thsys/70866"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11125-4">
    <skos:broader rdf:resource="descriptor/10852-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19118-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19540-1">
    <skos:prefLabel xml:lang="en">Budget constraint</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19540-1">
    <skos:prefLabel xml:lang="de">Budgetrestriktion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19637-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id42">
    <rdfs:seeAlso rdf:resource="descriptor/13688-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10842-2">
    <skos:scopeNote xml:lang="de">Wettbewerb und Wettbewerbsfähigkeit auf den Weltmärkten; Marktposition auf den internationalen Märkten.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70174">
    <rdfs:label xml:lang="en">Eastern Europe</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70174">
    <rdfs:label xml:lang="de">Osteuropa</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10722-5">
    <skos:prefLabel xml:lang="en">Exchange rate policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10722-5">
    <skos:prefLabel xml:lang="de">Wechselkurspolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13713-5">
    <skos:related rdf:resource="descriptor/18682-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13713-5">
    <skos:related rdf:resource="descriptor/16043-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13713-5">
    <skos:related rdf:resource="descriptor/13712-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13713-5">
    <skos:related rdf:resource="descriptor/13699-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13713-5">
    <skos:related rdf:resource="descriptor/12232-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15737-2">
    <skos:altLabel xml:lang="de">Immissionsschutzrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15737-2">
    <skos:altLabel xml:lang="de">Emissionsschutz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15737-2">
    <skos:altLabel xml:lang="de">Emissionsbegrenzung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15737-2">
    <skos:altLabel xml:lang="de">Bundesimmissionsschutzgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19494-3">
    <skos:prefLabel xml:lang="en">Beveridge curve</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19494-3">
    <skos:prefLabel xml:lang="de">Beveridge-Kurve</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70066">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19287-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20881-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14883-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18659-2">
    <skos:narrower rdf:resource="descriptor/20921-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71033">
    <skos:broader rdf:resource="thsys/70028"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18093-5">
    <skos:broader rdf:resource="thsys/71024"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18093-5">
    <skos:broader rdf:resource="descriptor/18313-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18093-5">
    <skos:broader rdf:resource="descriptor/18120-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10822-1">
    <skos:scopeNote xml:lang="de">Umfang des im Ausland erzeugten Produktionsvolumens des produzierenden Gewerbes.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12170-1">
    <skos:altLabel xml:lang="en">Value of information</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12170-1">
    <skos:altLabel xml:lang="en">Information content</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12170-1">
    <skos:altLabel xml:lang="de">Informationsnutzen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12170-1">
    <skos:altLabel xml:lang="de">Informationsgehalt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12170-1">
    <skos:altLabel xml:lang="de">Informationsbewertung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15725-2">
    <skos:scopeNote xml:lang="de">Sowohl für umfassende Änderungen der sozialen Sicherungssysteme als auch für eine Reform der gesellschaftlichen Verhältnisse unter sozialen Vorzeichen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18962-1">
    <skos:prefLabel xml:lang="en">Reverse logistics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18962-1">
    <skos:prefLabel xml:lang="de">Betriebliches Recycling</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18307-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14969-0">
    <skos:scopeNote xml:lang="de">Nur für verarbeitete Fische.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17967-3">
    <skos:altLabel xml:lang="de">Östersjön</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17967-3">
    <skos:altLabel xml:lang="de">Baltisches Meer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19671-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18087-0">
    <skos:broader rdf:resource="thsys/70455"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18087-0">
    <skos:broader rdf:resource="thsys/70187"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17695-3">
    <skos:broader rdf:resource="thsys/70560"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17695-3">
    <skos:broader rdf:resource="descriptor/17724-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16258-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14273-2">
    <skos:prefLabel xml:lang="en">Metal scrap</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14273-2">
    <skos:prefLabel xml:lang="de">Schrott</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10877-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16722-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70832">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13205-3">
    <skos:prefLabel xml:lang="en">Fish trade</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13205-3">
    <skos:prefLabel xml:lang="de">Fischhandel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14822-2">
    <skos:related rdf:resource="descriptor/13006-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14822-2">
    <skos:related rdf:resource="descriptor/14379-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14822-2">
    <skos:related rdf:resource="descriptor/18230-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14822-2">
    <skos:related rdf:resource="descriptor/14702-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18225-0">
    <skos:altLabel xml:lang="x-other">Slovensko</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18225-0">
    <skos:altLabel xml:lang="de">Slowakische Republik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19692-6">
    <skos:scopeNote xml:lang="de">Für Abstimmungen aller Art, aber nur für den Vorgang der Stimmabgabe; vor allem für das Abstimmungsverhalten der Abgeordneten im Parlament.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15089-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19809-4">
    <skos:broader rdf:resource="thsys/73339"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19809-4">
    <skos:broader rdf:resource="descriptor/15416-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11534-1">
    <skos:narrower rdf:resource="descriptor/11641-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16363-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10190-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13095-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16301-6">
    <skos:prefLabel xml:lang="en">Ombudsmen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16301-6">
    <skos:prefLabel xml:lang="de">Bürgerbeauftragte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11937-0">
    <skos:prefLabel xml:lang="en">European regional policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11937-0">
    <skos:prefLabel xml:lang="de">EU-Regionalpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12263-1">
    <skos:altLabel xml:lang="de">Wandelschuldverschreibung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12263-1">
    <skos:altLabel xml:lang="de">Wandelobligation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12263-1">
    <skos:altLabel xml:lang="de">Going-Public-Wandelanleihe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14645-5">
    <skos:related rdf:resource="descriptor/14049-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11872-2">
    <skos:broader rdf:resource="thsys/71114"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11872-2">
    <skos:broader rdf:resource="thsys/70058"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11872-2">
    <skos:broader rdf:resource="descriptor/11735-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10511-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11826-2">
    <skos:broader rdf:resource="thsys/70938"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11826-2">
    <skos:broader rdf:resource="thsys/70926"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11826-2">
    <skos:broader rdf:resource="descriptor/18628-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11826-2">
    <skos:broader rdf:resource="descriptor/18579-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#definition">
    <rdf:type rdf:resource="http://www.w3.org/1999/02/22-rdf-syntax-ns#Property"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#definition">
    <rdf:type rdf:resource="http://www.w3.org/2002/07/owl#AnnotationProperty"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12958-1">
    <skos:prefLabel xml:lang="en">Viticulture</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12958-1">
    <skos:prefLabel xml:lang="de">Weinbau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13735-2">
    <skos:prefLabel xml:lang="en">Option trading</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13735-2">
    <skos:prefLabel xml:lang="de">Optionsgeschäft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10443-2">
    <zbwext:useInsteadNote rdf:nodeID="id50"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12046-2">
    <skos:prefLabel xml:lang="en">Company history</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12046-2">
    <skos:prefLabel xml:lang="de">Firmengeschichte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18398-4">
    <skos:related rdf:resource="descriptor/18272-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18398-4">
    <skos:related rdf:resource="descriptor/18415-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18398-4">
    <skos:related rdf:resource="descriptor/14343-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18398-4">
    <skos:related rdf:resource="descriptor/12930-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18398-4">
    <skos:related rdf:resource="descriptor/14051-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10314-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18144-0">
    <zbwext:useInsteadNote rdf:nodeID="id131"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19329-4">
    <skos:altLabel xml:lang="en">Vienna sale convention</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19329-4">
    <skos:altLabel xml:lang="en">Vienna convention on international sales</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19329-4">
    <skos:altLabel xml:lang="en">Convention on the international sale of goods</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19329-4">
    <skos:altLabel xml:lang="en">CISG (Convention on the International Sale of Goods)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19329-4">
    <skos:altLabel xml:lang="de">UN-Kaufrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19329-4">
    <skos:altLabel xml:lang="de">Internationales Kaufvertragsrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13823-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19069-4">
    <skos:altLabel xml:lang="de">Vergleich von Betrieben</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19069-4">
    <skos:altLabel xml:lang="de">Betrieblicher Vergleich</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11240-4">
    <skos:narrower rdf:resource="descriptor/13406-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13284-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14603-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19803-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15097-5">
    <skos:altLabel xml:lang="en">Index number theory</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15097-5">
    <skos:altLabel xml:lang="de">Warenkorb für Preisindizes</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15097-5">
    <skos:altLabel xml:lang="de">Wägungsschema für Preisindizes</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15097-5">
    <skos:altLabel xml:lang="de">Umbasierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15097-5">
    <skos:altLabel xml:lang="de">Paasche-Index</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15097-5">
    <skos:altLabel xml:lang="de">Laspeyres-Index</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15097-5">
    <skos:altLabel xml:lang="de">Indexverkettung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15097-5">
    <skos:altLabel xml:lang="de">Indextheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15097-5">
    <skos:altLabel xml:lang="de">Indexrechnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18023-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16672-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16802-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71016">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11345-4">
    <skos:broader rdf:resource="thsys/70278"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11345-4">
    <skos:broader rdf:resource="descriptor/16484-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15476-4">
    <skos:broader rdf:resource="thsys/73103"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15476-4">
    <skos:broader rdf:resource="descriptor/15470-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15476-4">
    <skos:broader rdf:resource="descriptor/15466-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19807-1">
    <skos:related rdf:resource="descriptor/19654-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17011-3">
    <skos:prefLabel xml:lang="en">Andorra</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17011-3">
    <skos:prefLabel xml:lang="de">Andorra</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12473-4">
    <skos:broader rdf:resource="thsys/71036"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12473-4">
    <skos:broader rdf:resource="thsys/70578"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12473-4">
    <skos:broader rdf:resource="descriptor/12480-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12473-4">
    <skos:broader rdf:resource="descriptor/12165-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18251-6">
    <skos:related rdf:resource="descriptor/18427-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10816-3">
    <skos:broader rdf:resource="thsys/71089"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17634-2">
    <skos:broader rdf:resource="thsys/70253"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17634-2">
    <skos:broader rdf:resource="descriptor/17608-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17634-2">
    <skos:broader rdf:resource="descriptor/17978-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11992-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18968-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10082-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13102-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18454-6">
    <skos:related rdf:resource="descriptor/13226-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18454-6">
    <skos:related rdf:resource="descriptor/13671-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18454-6">
    <skos:related rdf:resource="descriptor/14023-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18454-6">
    <skos:related rdf:resource="descriptor/12042-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12619-2">
    <skos:broader rdf:resource="thsys/71050"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12619-2">
    <skos:broader rdf:resource="thsys/70143"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12619-2">
    <skos:broader rdf:resource="thsys/70049"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12619-2">
    <skos:broader rdf:resource="thsys/70030"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11101-4">
    <skos:related rdf:resource="descriptor/11090-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11101-4">
    <skos:related rdf:resource="descriptor/11174-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11101-4">
    <skos:related rdf:resource="descriptor/11061-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11101-4">
    <skos:related rdf:resource="descriptor/11088-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11101-4">
    <skos:related rdf:resource="descriptor/11048-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11101-4">
    <skos:related rdf:resource="descriptor/10970-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11101-4">
    <skos:related rdf:resource="descriptor/11043-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11724-3">
    <skos:altLabel xml:lang="de">Staatsausgabengesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11724-3">
    <skos:altLabel xml:lang="de">Gesetz der wachsenden Staatstätigkeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10988-2">
    <skos:prefLabel xml:lang="en">Economic policy advice</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10988-2">
    <skos:prefLabel xml:lang="de">Wirtschaftspolitische Beratung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10621-4">
    <skos:altLabel xml:lang="de">Weltwirtschaftsrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10621-4">
    <skos:altLabel xml:lang="de">Internationales Wirtschaftsabkommen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19199-5">
    <skos:related rdf:resource="descriptor/19524-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19199-5">
    <skos:related rdf:resource="descriptor/24857-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19199-5">
    <skos:related rdf:resource="descriptor/18124-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10197-5">
    <skos:prefLabel xml:lang="en">Duopoly</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10197-5">
    <skos:prefLabel xml:lang="de">Duopol</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18760-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13190-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12194-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17949-5">
    <skos:prefLabel xml:lang="en">Tonga</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17949-5">
    <skos:prefLabel xml:lang="de">Tonga</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16008-4">
    <skos:altLabel xml:lang="de">Verbot politischer Parteien</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16008-4">
    <skos:altLabel xml:lang="de">PartG</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16008-4">
    <skos:altLabel xml:lang="de">Parteiverbot</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16008-4">
    <skos:altLabel xml:lang="de">Parteiengesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13605-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18002-6">
    <skos:altLabel xml:lang="en">Computer-integrated enterprise</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18002-6">
    <skos:altLabel xml:lang="en">Computer-aided industry</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18002-6">
    <skos:altLabel xml:lang="en">CAI (Computer-Aided Industry)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18002-6">
    <skos:altLabel xml:lang="de">Rechnerintegrierte Produktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18002-6">
    <skos:altLabel xml:lang="de">Computerintegrierte Produktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16481-6">
    <skos:prefLabel xml:lang="en">Protection of children and young people</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16481-6">
    <skos:prefLabel xml:lang="de">Jugendschutz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10940-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70087">
    <skos:narrower rdf:resource="descriptor/12201-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70087">
    <skos:narrower rdf:resource="descriptor/13402-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70087">
    <skos:narrower rdf:resource="descriptor/16076-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70087">
    <skos:narrower rdf:resource="descriptor/12196-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70087">
    <skos:narrower rdf:resource="descriptor/18119-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70087">
    <skos:narrower rdf:resource="descriptor/12198-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70087">
    <skos:narrower rdf:resource="descriptor/12197-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70087">
    <skos:narrower rdf:resource="descriptor/12195-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70087">
    <skos:narrower rdf:resource="descriptor/12194-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70087">
    <skos:narrower rdf:resource="descriptor/12192-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70087">
    <skos:narrower rdf:resource="descriptor/19157-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70087">
    <skos:narrower rdf:resource="descriptor/12189-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70087">
    <skos:narrower rdf:resource="descriptor/11628-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70087">
    <skos:narrower rdf:resource="descriptor/12191-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70087">
    <skos:narrower rdf:resource="descriptor/12047-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70087">
    <skos:narrower rdf:resource="descriptor/12056-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70087">
    <skos:narrower rdf:resource="descriptor/12055-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70087">
    <skos:narrower rdf:resource="descriptor/12188-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70087">
    <skos:narrower rdf:resource="descriptor/12187-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70087">
    <skos:narrower rdf:resource="descriptor/12186-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70087">
    <skos:narrower rdf:resource="descriptor/12050-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70087">
    <skos:narrower rdf:resource="descriptor/12185-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70087">
    <skos:narrower rdf:resource="descriptor/12184-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70087">
    <skos:narrower rdf:resource="descriptor/12183-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70087">
    <skos:narrower rdf:resource="descriptor/12182-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70087">
    <skos:narrower rdf:resource="descriptor/16050-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70087">
    <skos:narrower rdf:resource="descriptor/19043-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70087">
    <skos:narrower rdf:resource="descriptor/12181-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70087">
    <skos:narrower rdf:resource="descriptor/12180-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70087">
    <skos:narrower rdf:resource="descriptor/11626-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70087">
    <skos:narrower rdf:resource="descriptor/12865-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70087">
    <skos:narrower rdf:resource="descriptor/18108-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70087">
    <skos:narrower rdf:resource="descriptor/18109-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70087">
    <skos:narrower rdf:resource="descriptor/12178-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70087">
    <skos:narrower rdf:resource="descriptor/12177-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70087">
    <skos:narrower rdf:resource="descriptor/19156-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70087">
    <skos:narrower rdf:resource="descriptor/12175-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70087">
    <skos:narrower rdf:resource="descriptor/13714-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70087">
    <skos:narrower rdf:resource="descriptor/16043-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70087">
    <skos:narrower rdf:resource="descriptor/12174-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24622-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16544-1">
    <skos:related rdf:resource="descriptor/18130-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16544-1">
    <skos:related rdf:resource="descriptor/11691-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16544-1">
    <skos:related rdf:resource="descriptor/16318-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16544-1">
    <skos:related rdf:resource="descriptor/16723-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16544-1">
    <skos:related rdf:resource="descriptor/16642-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16544-1">
    <skos:related rdf:resource="descriptor/16623-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16544-1">
    <skos:related rdf:resource="descriptor/16651-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16544-1">
    <skos:related rdf:resource="descriptor/16430-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16544-1">
    <skos:related rdf:resource="descriptor/19680-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16544-1">
    <skos:related rdf:resource="descriptor/16633-2"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id41">
    <rdfs:seeAlso rdf:resource="descriptor/12046-2"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id378">
    <rdf:value xml:lang="de">Für Volkswirtschaftstheorie</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10179-0">
    <skos:broader rdf:resource="thsys/71112"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10179-0">
    <skos:broader rdf:resource="thsys/70049"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10179-0">
    <skos:broader rdf:resource="descriptor/10065-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73332">
    <rdfs:label xml:lang="en">Public international law</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73332">
    <rdfs:label xml:lang="de">Völkerrecht</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15719-4">
    <skos:broader rdf:resource="thsys/73386"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15719-4">
    <skos:broader rdf:resource="thsys/70321"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15719-4">
    <skos:broader rdf:resource="descriptor/11163-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71061">
    <skos:narrower rdf:resource="descriptor/11233-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71061">
    <skos:narrower rdf:resource="descriptor/19372-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71061">
    <skos:narrower rdf:resource="descriptor/11241-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71061">
    <skos:narrower rdf:resource="descriptor/11240-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71061">
    <skos:narrower rdf:resource="descriptor/11253-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71061">
    <skos:narrower rdf:resource="descriptor/11239-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71061">
    <skos:narrower rdf:resource="descriptor/19497-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71061">
    <skos:narrower rdf:resource="descriptor/11238-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71061">
    <skos:narrower rdf:resource="descriptor/11237-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71061">
    <skos:narrower rdf:resource="descriptor/11236-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71061">
    <skos:narrower rdf:resource="descriptor/11246-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71061">
    <skos:narrower rdf:resource="descriptor/11235-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71061">
    <skos:narrower rdf:resource="descriptor/19493-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71061">
    <skos:narrower rdf:resource="descriptor/13406-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71061">
    <skos:narrower rdf:resource="descriptor/10996-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71061">
    <skos:narrower rdf:resource="descriptor/11243-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16052-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13163-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12085-6">
    <skos:prefLabel xml:lang="en">Management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12085-6">
    <skos:prefLabel xml:lang="de">Management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14309-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10250-6">
    <skos:broader rdf:resource="thsys/70306"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10250-6">
    <skos:broader rdf:resource="thsys/70141"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10250-6">
    <skos:broader rdf:resource="descriptor/10236-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10250-6">
    <skos:broader rdf:resource="descriptor/16745-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11543-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70141">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.01.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13144-4">
    <skos:prefLabel xml:lang="en">Carpet industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13144-4">
    <skos:prefLabel xml:lang="de">Teppichindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70305">
    <skos:narrower rdf:resource="descriptor/11490-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70305">
    <skos:narrower rdf:resource="descriptor/16046-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70305">
    <skos:narrower rdf:resource="descriptor/18704-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70305">
    <skos:narrower rdf:resource="descriptor/10774-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70305">
    <skos:narrower rdf:resource="descriptor/10772-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70305">
    <skos:narrower rdf:resource="descriptor/21101-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70305">
    <skos:narrower rdf:resource="descriptor/11521-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70305">
    <skos:narrower rdf:resource="descriptor/20104-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70305">
    <skos:narrower rdf:resource="descriptor/19263-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70305">
    <skos:narrower rdf:resource="descriptor/11487-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70305">
    <skos:narrower rdf:resource="descriptor/11485-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70305">
    <skos:narrower rdf:resource="descriptor/11484-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70305">
    <skos:narrower rdf:resource="descriptor/11483-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70305">
    <skos:narrower rdf:resource="descriptor/19665-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70305">
    <skos:narrower rdf:resource="descriptor/11459-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70305">
    <skos:narrower rdf:resource="descriptor/18698-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70305">
    <skos:narrower rdf:resource="descriptor/19547-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70305">
    <skos:narrower rdf:resource="descriptor/11481-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70305">
    <skos:narrower rdf:resource="descriptor/19262-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70305">
    <skos:narrower rdf:resource="descriptor/10382-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70305">
    <skos:narrower rdf:resource="descriptor/11479-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70305">
    <skos:narrower rdf:resource="descriptor/11446-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70305">
    <skos:narrower rdf:resource="descriptor/19261-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70305">
    <skos:narrower rdf:resource="descriptor/11478-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70305">
    <skos:narrower rdf:resource="descriptor/19260-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19561-0">
    <skos:altLabel xml:lang="en">Materials balances</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19561-0">
    <skos:altLabel xml:lang="de">Stoffstrommanagement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19561-0">
    <skos:altLabel xml:lang="de">Stoffstrombilanz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19561-0">
    <skos:altLabel xml:lang="de">Stoffstromanalyse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19561-0">
    <skos:altLabel xml:lang="de">Stoffpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12371-5">
    <skos:broader rdf:resource="thsys/71042"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12371-5">
    <skos:broader rdf:resource="descriptor/12381-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12371-5">
    <skos:broader rdf:resource="descriptor/12312-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70227">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.12.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14019-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19333-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12918-6">
    <skos:related rdf:resource="descriptor/18395-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12918-6">
    <skos:related rdf:resource="descriptor/18522-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12918-6">
    <skos:related rdf:resource="descriptor/10301-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16252-0">
    <skos:broader rdf:resource="thsys/73363"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16252-0">
    <skos:broader rdf:resource="thsys/71052"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16252-0">
    <skos:broader rdf:resource="thsys/70865"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16252-0">
    <skos:broader rdf:resource="descriptor/13998-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16252-0">
    <skos:broader rdf:resource="descriptor/11302-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11306-0">
    <skos:altLabel xml:lang="en">Labour relations director</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11306-0">
    <skos:altLabel xml:lang="en">Labor relations director</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11306-0">
    <skos:altLabel xml:lang="de">Arbeitsdirektorinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11306-0">
    <skos:altLabel xml:lang="de">Arbeitsdirektor</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10635-0">
    <skos:prefLabel xml:lang="en">Trade effect</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10635-0">
    <skos:prefLabel xml:lang="de">Außenhandelseffekt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10910-4">
    <skos:altLabel xml:lang="de">UWG (Gesetz gegen den unlauteren Wettbewerb)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10910-4">
    <skos:altLabel xml:lang="de">Gesetz gegen den unlauteren Wettbewerb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12912-4">
    <zbwext:useInsteadNote rdf:nodeID="id135"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18955-5">
    <skos:prefLabel xml:lang="en">Economics of sport</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18955-5">
    <skos:prefLabel xml:lang="de">Sportökonomik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18814-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14737-0">
    <skos:related rdf:resource="descriptor/14623-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11742-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16912-5">
    <skos:prefLabel xml:lang="en">Saarland</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16912-5">
    <skos:prefLabel xml:lang="de">Saarland</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17374-2">
    <skos:prefLabel xml:lang="en">United Kingdom</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17374-2">
    <skos:prefLabel xml:lang="de">Großbritannien</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18115-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19695-0">
    <skos:scopeNote xml:lang="de">Innere Organisation von Parteien: ihre Organisationsprinzipien, ihr Aufbau und ihre organisatorische Gestaltung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70982">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.06.01.14</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13058-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16951-2">
    <skos:prefLabel xml:lang="en">Salzburg (State)</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16951-2">
    <skos:prefLabel xml:lang="de">Salzburg (Land)</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11020-4">
    <skos:broader rdf:resource="thsys/73380"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11020-4">
    <skos:broader rdf:resource="thsys/71101"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11020-4">
    <skos:broader rdf:resource="thsys/70871"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11020-4">
    <skos:broader rdf:resource="descriptor/13413-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11927-3">
    <skos:prefLabel xml:lang="en">Spatial equilibrium model</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11927-3">
    <skos:prefLabel xml:lang="de">Räumliche Gleichgewichtstheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18884-2">
    <skos:related rdf:resource="descriptor/11409-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18884-2">
    <skos:related rdf:resource="descriptor/18922-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18884-2">
    <skos:related rdf:resource="descriptor/11397-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18884-2">
    <skos:related rdf:resource="descriptor/11407-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18884-2">
    <skos:related rdf:resource="descriptor/11222-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11587-1">
    <skos:related rdf:resource="descriptor/12296-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11633-6">
    <skos:prefLabel xml:lang="en">Public monopoly</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11633-6">
    <skos:prefLabel xml:lang="de">Staatliches Monopol</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15879-3">
    <skos:broader rdf:resource="thsys/73353"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/b">
    <skos:narrower rdf:resource="thsys/70301"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/b">
    <skos:narrower rdf:resource="thsys/70113"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/b">
    <skos:narrower rdf:resource="thsys/70041"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/b">
    <skos:narrower rdf:resource="thsys/70030"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/b">
    <skos:narrower rdf:resource="thsys/71035"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/b">
    <skos:narrower rdf:resource="thsys/71039"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/b">
    <skos:narrower rdf:resource="thsys/70028"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/b">
    <skos:narrower rdf:resource="thsys/70140"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/b">
    <skos:narrower rdf:resource="thsys/70046"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/b">
    <skos:narrower rdf:resource="thsys/70005"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/b">
    <skos:narrower rdf:resource="thsys/70037"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13790-1">
    <skos:altLabel xml:lang="en">Thrift business</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13790-1">
    <skos:altLabel xml:lang="de">Sparverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13790-1">
    <skos:altLabel xml:lang="de">Spareinlagengeschäft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14074-1">
    <skos:narrower rdf:resource="descriptor/14079-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14074-1">
    <skos:narrower rdf:resource="descriptor/14990-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14074-1">
    <skos:narrower rdf:resource="descriptor/15741-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14074-1">
    <skos:narrower rdf:resource="descriptor/14094-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14074-1">
    <skos:narrower rdf:resource="descriptor/14092-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14074-1">
    <skos:narrower rdf:resource="descriptor/14063-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14074-1">
    <skos:narrower rdf:resource="descriptor/14088-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14074-1">
    <skos:narrower rdf:resource="descriptor/14104-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14074-1">
    <skos:narrower rdf:resource="descriptor/14057-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14074-1">
    <skos:narrower rdf:resource="descriptor/14055-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13845-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12189-1">
    <skos:broader rdf:resource="thsys/70855"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12189-1">
    <skos:broader rdf:resource="thsys/70087"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12189-1">
    <skos:broader rdf:resource="descriptor/12194-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12189-1">
    <skos:broader rdf:resource="descriptor/11825-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70403">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16742-4">
    <skos:prefLabel xml:lang="en">Romanticism</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16742-4">
    <skos:prefLabel xml:lang="de">Romantik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18629-4">
    <skos:broader rdf:resource="thsys/70926"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18629-4">
    <skos:broader rdf:resource="descriptor/11833-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18629-4">
    <skos:broader rdf:resource="descriptor/11572-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11258-6">
    <skos:altLabel xml:lang="en">Temporary worker</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11258-6">
    <skos:altLabel xml:lang="de">Saisonarbeiterinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11258-6">
    <skos:altLabel xml:lang="de">Saisonarbeiter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12933-3">
    <skos:broader rdf:resource="thsys/71029"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12933-3">
    <skos:broader rdf:resource="descriptor/12949-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12847-3">
    <skos:related rdf:resource="descriptor/13670-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12847-3">
    <skos:related rdf:resource="descriptor/13669-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12847-3">
    <skos:related rdf:resource="descriptor/18449-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12847-3">
    <skos:related rdf:resource="descriptor/16010-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12847-3">
    <skos:related rdf:resource="descriptor/18206-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13574-0">
    <skos:related rdf:resource="descriptor/18501-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13574-0">
    <skos:related rdf:resource="descriptor/13255-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13574-0">
    <skos:related rdf:resource="descriptor/13527-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13574-0">
    <skos:related rdf:resource="descriptor/13063-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13574-0">
    <skos:related rdf:resource="descriptor/14428-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18445-0">
    <skos:broader rdf:resource="thsys/70986"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18445-0">
    <skos:broader rdf:resource="descriptor/18443-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18676-2">
    <skos:prefLabel xml:lang="en">Stock exchange law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18676-2">
    <skos:prefLabel xml:lang="de">Börsenrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11083-1">
    <skos:narrower rdf:resource="descriptor/10574-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12010-2">
    <skos:broader rdf:resource="thsys/71063"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12010-2">
    <skos:broader rdf:resource="descriptor/11716-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14817-2">
    <skos:prefLabel xml:lang="en">Rubber product</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14817-2">
    <skos:prefLabel xml:lang="de">Gummiwaren</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11648-0">
    <skos:altLabel xml:lang="de">Direkte Besteuerung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17749-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17920-1">
    <skos:broader rdf:resource="thsys/71030"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17920-1">
    <skos:broader rdf:resource="descriptor/17931-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17920-1">
    <skos:broader rdf:resource="descriptor/19170-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11276-4">
    <skos:altLabel xml:lang="de">Landwirtschaftliche Berufe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11276-4">
    <skos:altLabel xml:lang="de">Landarbeiterinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11276-4">
    <skos:altLabel xml:lang="de">Landarbeiter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11276-4">
    <skos:altLabel xml:lang="de">Gärtnerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11276-4">
    <skos:altLabel xml:lang="de">Gärtner</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11276-4">
    <skos:altLabel xml:lang="de">Gartenarbeiterinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11276-4">
    <skos:altLabel xml:lang="de">Gartenarbeiter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11276-4">
    <skos:altLabel xml:lang="de">Agrarberufe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16502-3">
    <skos:scopeNote xml:lang="de">Beziehungen zwischen Geographie und politischen Konstellationen allgemein. Außerdem Bestrebungen und Fähigkeit einzelner Staaten, weltpolitisch Einfluss zu nehmen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12396-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19826-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16595-5">
    <skos:related rdf:resource="descriptor/15712-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16595-5">
    <skos:related rdf:resource="descriptor/15711-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16595-5">
    <skos:related rdf:resource="descriptor/19752-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16595-5">
    <skos:related rdf:resource="descriptor/19975-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16595-5">
    <skos:related rdf:resource="descriptor/24784-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16595-5">
    <skos:related rdf:resource="descriptor/16587-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16595-5">
    <skos:related rdf:resource="descriptor/19428-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12278-2">
    <skos:altLabel xml:lang="en">Capital equipment</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12278-2">
    <skos:altLabel xml:lang="de">Produktionsgut</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12278-2">
    <skos:altLabel xml:lang="de">Kapitalgut</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12278-2">
    <skos:altLabel xml:lang="de">Ausrüstungsgut</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12278-2">
    <skos:altLabel xml:lang="de">Anlagegut</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13420-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19769-4">
    <skos:scopeNote xml:lang="de">Literatur zur zukünftigen Stabilität des traditionellen Systems der sozialen Sicherung und zur allgemeinen gesellschaftlichen Stabilität bzw. zu gesellschaftlichen Krisen vor allem in Zeiten des Umbruchs (z.B. in den Transformationsländern).</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12750-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18635-2">
    <skos:related rdf:resource="descriptor/13371-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17662-4">
    <skos:broader rdf:resource="thsys/70445"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17662-4">
    <skos:broader rdf:resource="descriptor/17672-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18066-1">
    <skos:altLabel xml:lang="en">Counterfeit good</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18066-1">
    <skos:altLabel xml:lang="de">Nachahmung von Markenartikeln</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18066-1">
    <skos:altLabel xml:lang="de">Markenpiraterie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18066-1">
    <skos:altLabel xml:lang="de">Markenartikelfälschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18066-1">
    <skos:altLabel xml:lang="de">Fälschung von Markenartikeln</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19560-2">
    <skos:related rdf:resource="descriptor/10894-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19560-2">
    <skos:related rdf:resource="descriptor/19662-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19560-2">
    <skos:related rdf:resource="descriptor/10692-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11183-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70933">
    <rdfs:label xml:lang="en">Port management</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70933">
    <rdfs:label xml:lang="de">Hafenwirtschaft</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10190-5">
    <skos:scopeNote xml:lang="de">Mikroökonomische Theorie zur Erklärung der Entstehung, Entwicklung und des Vorgehens von Unternehmen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14933-0">
    <skos:broader rdf:resource="thsys/70498"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14933-0">
    <skos:broader rdf:resource="descriptor/14931-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14455-3">
    <skos:prefLabel xml:lang="en">Ship</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14455-3">
    <skos:prefLabel xml:lang="de">Schiff</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10222-4">
    <skos:prefLabel xml:lang="en">Incomplete information</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10222-4">
    <skos:prefLabel xml:lang="de">Unvollkommene Information</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14129-2">
    <skos:altLabel xml:lang="de">Vogelfeder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14129-2">
    <skos:altLabel xml:lang="de">Schildpatt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14129-2">
    <skos:altLabel xml:lang="de">Elfenbein</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12703-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14074-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19754-3">
    <skos:altLabel xml:lang="de">Lebensweise</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19754-3">
    <skos:altLabel xml:lang="de">Lebensstil</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16729-3">
    <skos:related rdf:resource="descriptor/16726-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13953-6">
    <skos:related rdf:resource="descriptor/13960-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13953-6">
    <skos:related rdf:resource="descriptor/12905-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13953-6">
    <skos:related rdf:resource="descriptor/13957-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18443-4">
    <skos:altLabel xml:lang="de">Tabaktechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18443-4">
    <skos:altLabel xml:lang="de">Nahrungsmitteltechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18443-4">
    <skos:altLabel xml:lang="de">Müllereitechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18443-4">
    <skos:altLabel xml:lang="de">Molkereitechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18443-4">
    <skos:altLabel xml:lang="de">Backtechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18443-4">
    <skos:altLabel xml:lang="de">Bäckereitechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13268-0">
    <skos:narrower rdf:resource="descriptor/13665-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13268-0">
    <skos:narrower rdf:resource="descriptor/18666-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16370-1">
    <skos:related rdf:resource="descriptor/16369-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16370-1">
    <skos:related rdf:resource="descriptor/19695-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18499-5">
    <skos:prefLabel xml:lang="en">Exhibition services</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18499-5">
    <skos:prefLabel xml:lang="de">Messebau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15982-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18621-6">
    <skos:altLabel xml:lang="de">Container-Leasing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14725-0">
    <skos:broader rdf:resource="thsys/70199"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14725-0">
    <skos:broader rdf:resource="descriptor/14722-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10812-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13185-4">
    <skos:related rdf:resource="descriptor/14771-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13185-4">
    <skos:related rdf:resource="descriptor/18400-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13185-4">
    <skos:related rdf:resource="descriptor/14337-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13185-4">
    <skos:related rdf:resource="descriptor/14404-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11782-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12224-4">
    <skos:related rdf:resource="descriptor/10438-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12224-4">
    <skos:related rdf:resource="descriptor/10008-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12224-4">
    <skos:related rdf:resource="descriptor/12206-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12224-4">
    <skos:related rdf:resource="descriptor/12209-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12224-4">
    <skos:related rdf:resource="descriptor/12038-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12224-4">
    <skos:related rdf:resource="descriptor/12215-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13688-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70192">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.00</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11507-4">
    <skos:broader rdf:resource="thsys/70355"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12845-0">
    <skos:altLabel xml:lang="en">Marketing services</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12845-0">
    <skos:altLabel xml:lang="en">Advertising agency</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12845-0">
    <skos:altLabel xml:lang="de">Werbeberatung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12845-0">
    <skos:altLabel xml:lang="de">Werbeagentur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12845-0">
    <skos:altLabel xml:lang="de">Public-Relations-Agentur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12845-0">
    <skos:altLabel xml:lang="de">Media-Agentur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12951-1">
    <skos:prefLabel xml:lang="en">Seed industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12951-1">
    <skos:prefLabel xml:lang="de">Saatzucht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10784-4">
    <skos:broader rdf:resource="thsys/71087"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10784-4">
    <skos:broader rdf:resource="descriptor/10787-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18699-4">
    <skos:prefLabel xml:lang="en">Currency swap</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18699-4">
    <skos:prefLabel xml:lang="de">Währungsswap</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16481-6">
    <skos:related rdf:resource="descriptor/16140-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11579-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10912-0">
    <skos:related rdf:resource="descriptor/10899-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12610-6">
    <skos:altLabel xml:lang="de">Betriebsunfall</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12610-6">
    <skos:altLabel xml:lang="de">Berufsunfall</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12644-3">
    <skos:related rdf:resource="descriptor/19561-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12644-3">
    <skos:related rdf:resource="descriptor/12681-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12644-3">
    <skos:related rdf:resource="descriptor/12693-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12644-3">
    <skos:related rdf:resource="descriptor/12747-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11911-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21408-0">
    <skos:related rdf:resource="descriptor/12788-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21408-0">
    <skos:related rdf:resource="descriptor/12780-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12186-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11665-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11059-5">
    <skos:prefLabel xml:lang="en">Social market economy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11059-5">
    <skos:prefLabel xml:lang="de">Soziale Marktwirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15661-2">
    <skos:altLabel xml:lang="de">Gesang</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14064-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70965">
    <skos:prefLabel xml:lang="en">W.08  Trade (Craft)</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70965">
    <skos:prefLabel xml:lang="de">W.08  Handwerk</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11487-5">
    <skos:altLabel xml:lang="en">Open market operation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id227">
    <rdfs:seeAlso rdf:resource="descriptor/11567-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10583-0">
    <skos:related rdf:resource="descriptor/18314-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10583-0">
    <skos:related rdf:resource="descriptor/11567-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10583-0">
    <skos:related rdf:resource="descriptor/14931-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10583-0">
    <skos:related rdf:resource="descriptor/16524-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10583-0">
    <skos:related rdf:resource="descriptor/19840-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10583-0">
    <skos:related rdf:resource="descriptor/10555-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14005-6">
    <skos:broader rdf:resource="thsys/71053"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14005-6">
    <skos:broader rdf:resource="thsys/71052"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14005-6">
    <skos:broader rdf:resource="descriptor/18890-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14005-6">
    <skos:broader rdf:resource="descriptor/11356-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15067-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19249-2">
    <skos:related rdf:resource="descriptor/10332-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19249-2">
    <skos:related rdf:resource="descriptor/12754-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19249-2">
    <skos:related rdf:resource="descriptor/12750-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12401-1">
    <skos:prefLabel xml:lang="en">Corporate debt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12401-1">
    <skos:prefLabel xml:lang="de">Verbindlichkeiten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18589-4">
    <skos:prefLabel xml:lang="en">Passenger shipping</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18589-4">
    <skos:prefLabel xml:lang="de">Passagierschifffahrt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19443-6">
    <skos:related rdf:resource="descriptor/19444-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11397-6">
    <skos:broader rdf:resource="thsys/73369"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11397-6">
    <skos:broader rdf:resource="thsys/70847"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11397-6">
    <skos:broader rdf:resource="thsys/70176"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11397-6">
    <skos:broader rdf:resource="descriptor/16273-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15061-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15392-3">
    <skos:related rdf:resource="descriptor/19241-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14711-4">
    <skos:prefLabel xml:lang="en">Polypropylene</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14711-4">
    <skos:prefLabel xml:lang="de">Polypropylen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17014-4">
    <skos:broader rdf:resource="thsys/70131"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17014-4">
    <skos:broader rdf:resource="descriptor/17016-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16556-1">
    <skos:prefLabel xml:lang="en">Ethnology</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16556-1">
    <skos:prefLabel xml:lang="de">Ethnologie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19451-0">
    <skos:prefLabel xml:lang="en">Industrial mobility</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19451-0">
    <skos:prefLabel xml:lang="de">Industrieverlagerung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18645-6">
    <skos:scopeNote xml:lang="de">Postdienstleistungen, die durch private Unternehmen ausgeführt werden</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18859-1">
    <skos:related rdf:resource="descriptor/16563-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18859-1">
    <skos:related rdf:resource="descriptor/15672-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18859-1">
    <skos:related rdf:resource="descriptor/15790-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18859-1">
    <skos:related rdf:resource="descriptor/15801-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18859-1">
    <skos:related rdf:resource="descriptor/13383-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18859-1">
    <skos:related rdf:resource="descriptor/15821-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18859-1">
    <skos:related rdf:resource="descriptor/19601-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18859-1">
    <skos:related rdf:resource="descriptor/15466-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18859-1">
    <skos:related rdf:resource="descriptor/15633-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71038">
    <skos:prefLabel xml:lang="de">B.09.01  IS-Komponenten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70549">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">P.08</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15753-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13969-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13701-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13479-1">
    <skos:altLabel xml:lang="de">Verkäuferbedienung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13479-1">
    <skos:altLabel xml:lang="de">Fremdbedienung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13064-2">
    <skos:prefLabel xml:lang="en">Steel construction</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13064-2">
    <skos:prefLabel xml:lang="de">Stahlbau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19007-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13989-6">
    <skos:prefLabel xml:lang="en">Politicians</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13989-6">
    <skos:prefLabel xml:lang="de">Politiker</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18850-5">
    <skos:related rdf:resource="descriptor/19636-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18850-5">
    <skos:related rdf:resource="descriptor/10966-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18850-5">
    <skos:related rdf:resource="descriptor/10029-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18850-5">
    <skos:related rdf:resource="descriptor/19373-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18850-5">
    <skos:related rdf:resource="descriptor/16384-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18850-5">
    <skos:related rdf:resource="descriptor/11313-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18850-5">
    <skos:related rdf:resource="descriptor/13871-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18850-5">
    <skos:related rdf:resource="descriptor/19818-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18850-5">
    <skos:related rdf:resource="descriptor/11290-3"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id321">
    <rdf:value xml:lang="de">Für den regionalen Strukturwandel</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19662-1">
    <skos:broader rdf:resource="thsys/70154"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11298-1">
    <skos:related rdf:resource="descriptor/11782-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11298-1">
    <skos:related rdf:resource="descriptor/18308-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11298-1">
    <skos:related rdf:resource="descriptor/11327-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11298-1">
    <skos:related rdf:resource="descriptor/11297-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11298-1">
    <skos:related rdf:resource="descriptor/11290-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11298-1">
    <skos:related rdf:resource="descriptor/11183-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19081-0">
    <skos:scopeNote xml:lang="de">Einlage, die durch Einbringung von Vermögenswerten (Immobilien, Maschinen) statt durch Bareinzahlung geleistet wird.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10659-0">
    <skos:altLabel xml:lang="en">Real exchange rate</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10659-0">
    <skos:altLabel xml:lang="en">PPP (Purchasing Power Parity)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10659-0">
    <skos:altLabel xml:lang="de">Realer Wechselkurs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10659-0">
    <skos:altLabel xml:lang="de">Kaufkraftvergleich</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10659-0">
    <skos:altLabel xml:lang="de">Kaufkraftparitätentheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13292-3">
    <skos:prefLabel xml:lang="en">Health insurance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13292-3">
    <skos:prefLabel xml:lang="de">Krankenversicherung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17704-0">
    <skos:prefLabel xml:lang="en">Ghana</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17704-0">
    <skos:prefLabel xml:lang="de">Ghana</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18020-4">
    <skos:broader rdf:resource="thsys/73340"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18020-4">
    <skos:broader rdf:resource="descriptor/15621-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13629-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13446-2">
    <skos:related rdf:resource="descriptor/18442-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13446-2">
    <skos:related rdf:resource="descriptor/12945-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13446-2">
    <skos:related rdf:resource="descriptor/10497-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13446-2">
    <skos:related rdf:resource="descriptor/18339-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13446-2">
    <skos:related rdf:resource="descriptor/13450-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14962-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70995">
    <skos:broader rdf:resource="thsys/71004"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18491-0">
    <skos:broader rdf:resource="thsys/70970"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18491-0">
    <skos:broader rdf:resource="thsys/70878"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18491-0">
    <skos:broader rdf:resource="descriptor/10436-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19045-4">
    <skos:prefLabel xml:lang="en">Mixed farming</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19045-4">
    <skos:prefLabel xml:lang="de">Gemischte Landwirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16739-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14735-4">
    <skos:prefLabel xml:lang="en">Cosmetics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14735-4">
    <skos:prefLabel xml:lang="de">Kosmetika</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18525-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13083-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14196-1">
    <skos:narrower rdf:resource="descriptor/18236-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14469-6">
    <skos:altLabel xml:lang="en">Medical electronics</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14469-6">
    <skos:altLabel xml:lang="de">Röntgengerät</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14469-6">
    <skos:altLabel xml:lang="de">Medizinelektronik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14469-6">
    <skos:altLabel xml:lang="de">Hörgerät</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14469-6">
    <skos:altLabel xml:lang="de">Elektrokardiograph</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14469-6">
    <skos:altLabel xml:lang="de">Elektroenzephalograph</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14355-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16159-4">
    <skos:broader rdf:resource="thsys/73335"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16159-4">
    <skos:broader rdf:resource="thsys/70835"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16159-4">
    <skos:broader rdf:resource="thsys/70833"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16159-4">
    <skos:broader rdf:resource="thsys/70324"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16159-4">
    <skos:broader rdf:resource="descriptor/16007-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15712-4">
    <skos:related rdf:resource="descriptor/15722-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15712-4">
    <skos:related rdf:resource="descriptor/16595-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15712-4">
    <skos:related rdf:resource="descriptor/16654-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15712-4">
    <skos:related rdf:resource="descriptor/15714-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15712-4">
    <skos:related rdf:resource="descriptor/19752-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19584-2">
    <skos:related rdf:resource="descriptor/15076-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19584-2">
    <skos:related rdf:resource="descriptor/18785-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19584-2">
    <skos:related rdf:resource="descriptor/18784-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19584-2">
    <skos:related rdf:resource="descriptor/10436-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19584-2">
    <skos:related rdf:resource="descriptor/18888-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18557-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19620-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17638-1">
    <skos:broader rdf:resource="thsys/70253"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17638-1">
    <skos:broader rdf:resource="descriptor/17991-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17638-1">
    <skos:broader rdf:resource="descriptor/17608-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12155-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13745-6">
    <skos:altLabel xml:lang="en">Zero-coupon bonds</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13745-6">
    <skos:altLabel xml:lang="en">Zerobond</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13745-6">
    <skos:altLabel xml:lang="en">Zero-bond</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13745-6">
    <skos:altLabel xml:lang="de">Nullkuponanleihe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13745-6">
    <skos:altLabel xml:lang="de">Nullkupon-Anleihe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14479-3">
    <skos:broader rdf:resource="thsys/70135"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14479-3">
    <skos:broader rdf:resource="descriptor/14473-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14998-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70562">
    <skos:broader rdf:resource="thsys/70028"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13235-1">
    <skos:related rdf:resource="descriptor/13097-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13235-1">
    <skos:related rdf:resource="descriptor/14586-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13235-1">
    <skos:related rdf:resource="descriptor/15675-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13235-1">
    <skos:related rdf:resource="descriptor/13233-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13235-1">
    <skos:related rdf:resource="descriptor/14029-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70130">
    <skos:prefLabel xml:lang="en">G.01.03.03  Southeastern Europe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70130">
    <skos:prefLabel xml:lang="de">G.01.03.03  Südosteuropa</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13661-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73342">
    <skos:prefLabel xml:lang="de">N.10.07  Informationstechnik und Informatik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18406-3">
    <skos:related rdf:resource="descriptor/13039-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18406-3">
    <skos:related rdf:resource="descriptor/13029-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18213-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18010-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12217-1">
    <skos:altLabel xml:lang="en">Earnings management</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12217-1">
    <skos:altLabel xml:lang="en">Creative accounting</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12217-1">
    <skos:altLabel xml:lang="de">Jahresabschlusspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12217-1">
    <skos:altLabel xml:lang="de">Jahresabschlußpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12217-1">
    <skos:altLabel xml:lang="de">Bilanztaktik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70476">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14990-2">
    <skos:prefLabel xml:lang="en">Fodder</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14990-2">
    <skos:prefLabel xml:lang="de">Pflanzenfutter</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13354-0">
    <skos:altLabel xml:lang="de">Zahlenlotto</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13354-0">
    <skos:altLabel xml:lang="de">Wette</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13354-0">
    <skos:altLabel xml:lang="de">Wettbüro</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13354-0">
    <skos:altLabel xml:lang="de">Toto</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13354-0">
    <skos:altLabel xml:lang="de">Spielhalle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13354-0">
    <skos:altLabel xml:lang="de">Spielcasino</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13354-0">
    <skos:altLabel xml:lang="de">Spielbank</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13354-0">
    <skos:altLabel xml:lang="de">Lotto</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13354-0">
    <skos:altLabel xml:lang="de">Lotterie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13354-0">
    <skos:altLabel xml:lang="de">Buchmacher</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19616-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19146-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10328-2">
    <skos:broader rdf:resource="thsys/70233"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10328-2">
    <skos:broader rdf:resource="descriptor/11039-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10328-2">
    <skos:broader rdf:resource="descriptor/10327-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15018-6">
    <skos:prefLabel xml:lang="en">Domestic waste</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15018-6">
    <skos:prefLabel xml:lang="de">Hausmüll</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19294-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13733-6">
    <skos:prefLabel xml:lang="en">Inscribed stock</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13733-6">
    <skos:prefLabel xml:lang="de">Namensaktie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14035-4">
    <skos:prefLabel xml:lang="en">Nuclear power plant</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14035-4">
    <skos:prefLabel xml:lang="de">Kernkraftwerk</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18260-5">
    <skos:prefLabel xml:lang="en">Biotechnological industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18260-5">
    <skos:prefLabel xml:lang="de">Biotechnische Industrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11636-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19252-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11987-6">
    <skos:altLabel xml:lang="de">Stadtstruktur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11987-6">
    <skos:altLabel xml:lang="de">Städtische Siedlungsstruktur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11987-6">
    <skos:altLabel xml:lang="de">Räumliche Stadtstruktur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11892-3">
    <skos:related rdf:resource="descriptor/10542-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11892-3">
    <skos:related rdf:resource="descriptor/15695-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11892-3">
    <skos:related rdf:resource="descriptor/10510-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11892-3">
    <skos:related rdf:resource="descriptor/10562-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10342-1">
    <skos:prefLabel xml:lang="en">Recent economic development</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10342-1">
    <skos:prefLabel xml:lang="de">Wirtschaftslage</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11636-0">
    <skos:broader rdf:resource="thsys/71077"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11636-0">
    <skos:broader rdf:resource="thsys/70046"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16379-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19389-0">
    <skos:related rdf:resource="descriptor/13117-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19389-0">
    <skos:related rdf:resource="descriptor/14802-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11367-1">
    <skos:prefLabel xml:lang="en">Education system</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11367-1">
    <skos:prefLabel xml:lang="de">Bildungswesen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19365-0">
    <skos:prefLabel xml:lang="en">Natural rubber production</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19365-0">
    <skos:prefLabel xml:lang="de">Kautschukgewinnung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24708-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71058">
    <skos:broader rdf:resource="thsys/70052"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70867">
    <rdfs:label xml:lang="de">Wirtschaftsaufsicht und Wirtschaftsförderung</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13658-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18187-3">
    <skos:prefLabel xml:lang="en">Traffic forecast</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18187-3">
    <skos:prefLabel xml:lang="de">Verkehrsprognose</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25743-1">
    <skos:related rdf:resource="descriptor/13992-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12508-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16724-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16740-1">
    <skos:altLabel xml:lang="de">Wiener Schule</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19836-1">
    <skos:broader rdf:resource="thsys/70582"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16780-3">
    <skos:altLabel xml:lang="de">Titelaufnahme</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16780-3">
    <skos:altLabel xml:lang="de">Regeln für die alphabetische Katalogisierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16780-3">
    <skos:altLabel xml:lang="de">RAK-Regeln</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16780-3">
    <skos:altLabel xml:lang="de">Formalerschließung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16780-3">
    <skos:altLabel xml:lang="de">Formalerfassung von Dokumenten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16780-3">
    <skos:altLabel xml:lang="de">Bibliothekskatalogisierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16780-3">
    <skos:altLabel xml:lang="de">Alphabetische Katalogisierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70859">
    <skos:broader rdf:resource="thsys/70860"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10512-2">
    <skos:prefLabel xml:lang="en">Urbanization</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10512-2">
    <skos:prefLabel xml:lang="de">Urbanisierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16173-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14512-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12922-1">
    <skos:narrower rdf:resource="descriptor/18256-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12922-1">
    <skos:narrower rdf:resource="descriptor/13472-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12922-1">
    <skos:narrower rdf:resource="descriptor/18096-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12922-1">
    <skos:narrower rdf:resource="descriptor/13019-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12922-1">
    <skos:narrower rdf:resource="descriptor/12964-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12922-1">
    <skos:narrower rdf:resource="descriptor/12995-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12922-1">
    <skos:narrower rdf:resource="descriptor/12980-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15243-6">
    <skos:narrower rdf:resource="descriptor/19148-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13515-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12847-3">
    <skos:broader rdf:resource="thsys/70384"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12847-3">
    <skos:broader rdf:resource="descriptor/12829-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13102-6">
    <skos:related rdf:resource="descriptor/18450-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13102-6">
    <skos:related rdf:resource="descriptor/13367-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13102-6">
    <skos:related rdf:resource="descriptor/14816-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13102-6">
    <skos:related rdf:resource="descriptor/18434-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13102-6">
    <skos:related rdf:resource="descriptor/18205-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13102-6">
    <skos:related rdf:resource="descriptor/14371-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13102-6">
    <skos:related rdf:resource="descriptor/14812-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73104">
    <skos:narrower rdf:resource="descriptor/15460-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73104">
    <skos:narrower rdf:resource="descriptor/15463-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73104">
    <skos:narrower rdf:resource="descriptor/19606-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73104">
    <skos:narrower rdf:resource="descriptor/19592-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73104">
    <skos:narrower rdf:resource="descriptor/15458-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73104">
    <skos:narrower rdf:resource="descriptor/19595-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73104">
    <skos:narrower rdf:resource="descriptor/19593-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73104">
    <skos:narrower rdf:resource="descriptor/15450-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73104">
    <skos:narrower rdf:resource="descriptor/15448-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73104">
    <skos:narrower rdf:resource="descriptor/11337-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73104">
    <skos:narrower rdf:resource="descriptor/15446-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73104">
    <skos:narrower rdf:resource="descriptor/11163-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73104">
    <skos:narrower rdf:resource="descriptor/15441-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73104">
    <skos:narrower rdf:resource="descriptor/19597-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73104">
    <skos:narrower rdf:resource="descriptor/19594-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73104">
    <skos:narrower rdf:resource="descriptor/19596-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73104">
    <skos:narrower rdf:resource="descriptor/15437-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73104">
    <skos:narrower rdf:resource="descriptor/10072-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73104">
    <skos:narrower rdf:resource="descriptor/10138-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13808-1">
    <skos:altLabel xml:lang="de">Gewerbliche Versicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13808-1">
    <skos:altLabel xml:lang="de">Betriebsunterbrechungsversicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13808-1">
    <skos:altLabel xml:lang="de">Betriebsrisikoversicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10522-6">
    <skos:scopeNote xml:lang="de">Koexistenz zwischen einem modernen, kapitalintensiven Sektor und einem schwächeren Randbereich in Industrieländern bzw. einem randständigen traditionellen Sektor in Entwicklungsländern.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16084-2">
    <skos:broader rdf:resource="thsys/71098"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16084-2">
    <skos:broader rdf:resource="descriptor/10683-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18423-3">
    <skos:broader rdf:resource="thsys/70997"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18423-3">
    <skos:broader rdf:resource="thsys/70937"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18423-3">
    <skos:broader rdf:resource="thsys/70878"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18423-3">
    <skos:broader rdf:resource="descriptor/13573-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18423-3">
    <skos:broader rdf:resource="descriptor/10436-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17754-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15119-0">
    <skos:scopeNote xml:lang="de">Regelmäßig wiederkehrende statistische Teilerhebungen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11125-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12513-4">
    <skos:prefLabel xml:lang="en">Operational accounting</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12513-4">
    <skos:prefLabel xml:lang="de">Betriebsabrechnung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19839-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70572">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.03.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19327-1">
    <skos:altLabel xml:lang="de">Weltwirtschaftskonferenz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19327-1">
    <skos:altLabel xml:lang="de">Weltwirtschaftsgipfel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19327-1">
    <skos:altLabel xml:lang="de">Internationaler Wirtschaftsgipfel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19327-1">
    <skos:altLabel xml:lang="de">Außenwirtschaftskonferenz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10557-1">
    <skos:scopeNote xml:lang="de">Zusammenarbeit mehrerer Partner in der Entwicklungshilfe, z.B. international oder öffentliche mit privaten Trägern.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18930-0">
    <skos:altLabel xml:lang="de">Zuschauerforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18930-0">
    <skos:altLabel xml:lang="de">Programmforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18930-0">
    <skos:altLabel xml:lang="de">Mediaforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19151-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19590-0">
    <skos:broader rdf:resource="thsys/72181"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19590-0">
    <skos:broader rdf:resource="thsys/71087"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19590-0">
    <skos:broader rdf:resource="descriptor/15076-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11539-5">
    <skos:prefLabel xml:lang="en">Budget Control</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11539-5">
    <skos:prefLabel xml:lang="de">Öffentliche Finanzkontrolle</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18459-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15082-4">
    <skos:prefLabel xml:lang="en">Index number</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15082-4">
    <skos:prefLabel xml:lang="de">Index</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18478-6">
    <skos:prefLabel xml:lang="en">Industrial change</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18478-6">
    <skos:prefLabel xml:lang="de">Industrieller Strukturwandel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70270">
    <skos:prefLabel xml:lang="de">B.06.01  Produktions- und Kostentheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11266-0">
    <skos:scopeNote xml:lang="de">Für theoretische oder empirische Untersuchungen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10049-6">
    <skos:narrower rdf:resource="descriptor/10243-3"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id363">
    <rdf:value xml:lang="de">Für die Wertschöpfung im betriebswirtschaftlichen Sinne</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15054-2">
    <skos:related rdf:resource="descriptor/15429-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15054-2">
    <skos:related rdf:resource="descriptor/15372-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15054-2">
    <skos:related rdf:resource="descriptor/15415-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15054-2">
    <skos:related rdf:resource="descriptor/10035-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15054-2">
    <skos:related rdf:resource="descriptor/15554-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18636-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70998">
    <skos:prefLabel xml:lang="en">W.06.01.03.01  Aircraft and aerospace industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70998">
    <skos:prefLabel xml:lang="de">W.06.01.03.01  Luft- und Raumfahrtindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18289-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14295-6">
    <skos:prefLabel xml:lang="en">Chromium</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14295-6">
    <skos:prefLabel xml:lang="de">Chrom</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70839">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.25</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19584-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10176-6">
    <skos:broader rdf:resource="thsys/70814"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10176-6">
    <skos:broader rdf:resource="thsys/70493"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10176-6">
    <skos:broader rdf:resource="descriptor/10177-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10176-6">
    <skos:broader rdf:resource="descriptor/19231-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12668-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13422-2">
    <skos:broader rdf:resource="thsys/73361"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13422-2">
    <skos:broader rdf:resource="thsys/70862"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13422-2">
    <skos:broader rdf:resource="descriptor/19689-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13422-2">
    <skos:broader rdf:resource="descriptor/13410-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19541-6">
    <skos:related rdf:resource="descriptor/20241-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19541-6">
    <skos:related rdf:resource="descriptor/18552-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19541-6">
    <skos:related rdf:resource="descriptor/18654-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19541-6">
    <skos:related rdf:resource="descriptor/18648-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19541-6">
    <skos:related rdf:resource="descriptor/12786-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19541-6">
    <skos:related rdf:resource="descriptor/19653-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19078-3">
    <skos:scopeNote xml:lang="de">Gründung der Gemeinschaft Unabhängiger Staaten im Dezember 1991 durch die Mehrzahl der ehemaligen Sowjetrepubliken</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73348">
    <skos:narrower rdf:resource="descriptor/15765-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73348">
    <skos:narrower rdf:resource="descriptor/15928-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73348">
    <skos:narrower rdf:resource="descriptor/15659-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73348">
    <skos:narrower rdf:resource="descriptor/18289-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73348">
    <skos:narrower rdf:resource="descriptor/18339-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73348">
    <skos:narrower rdf:resource="descriptor/15635-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73348">
    <skos:narrower rdf:resource="descriptor/18777-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73348">
    <skos:narrower rdf:resource="descriptor/15633-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73348">
    <skos:narrower rdf:resource="descriptor/19625-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73348">
    <skos:narrower rdf:resource="descriptor/15627-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73348">
    <skos:narrower rdf:resource="descriptor/13446-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73348">
    <skos:narrower rdf:resource="descriptor/18317-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17476-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11286-1">
    <skos:narrower rdf:resource="descriptor/18484-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11286-1">
    <skos:narrower rdf:resource="descriptor/11301-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12927-5">
    <skos:related rdf:resource="descriptor/13832-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12927-5">
    <skos:related rdf:resource="descriptor/13264-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12927-5">
    <skos:related rdf:resource="descriptor/13644-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12927-5">
    <skos:related rdf:resource="descriptor/13307-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12927-5">
    <skos:related rdf:resource="descriptor/13639-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12927-5">
    <skos:related rdf:resource="descriptor/13638-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12927-5">
    <skos:related rdf:resource="descriptor/13966-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18110-3">
    <skos:altLabel xml:lang="en">Workstation publishing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18110-3">
    <skos:altLabel xml:lang="en">Mainframe publishing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18110-3">
    <skos:altLabel xml:lang="en">Inhouse publishing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18110-3">
    <skos:altLabel xml:lang="en">DTP (Desktop Publishing)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18110-3">
    <skos:altLabel xml:lang="en">Document publishing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18110-3">
    <skos:altLabel xml:lang="en">Desktop publishing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18110-3">
    <skos:altLabel xml:lang="en">Corporate electronic publishing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18110-3">
    <skos:altLabel xml:lang="en">Computer-aided publishing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18110-3">
    <skos:altLabel xml:lang="en">Computer aided publishing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18110-3">
    <skos:altLabel xml:lang="en">CAP (Computer-Aided Publishing)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11345-4">
    <skos:narrower rdf:resource="descriptor/11341-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11345-4">
    <skos:narrower rdf:resource="descriptor/11331-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11345-4">
    <skos:narrower rdf:resource="descriptor/11328-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11345-4">
    <skos:narrower rdf:resource="descriptor/11346-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11345-4">
    <skos:narrower rdf:resource="descriptor/11510-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11345-4">
    <skos:narrower rdf:resource="descriptor/16143-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11345-4">
    <skos:narrower rdf:resource="descriptor/11342-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12981-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19094-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19038-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10338-6">
    <skos:broader rdf:resource="thsys/70020"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10338-6">
    <skos:broader rdf:resource="descriptor/10342-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19246-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11168-0">
    <skos:broader rdf:resource="thsys/70853"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11168-0">
    <skos:broader rdf:resource="thsys/70274"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11168-0">
    <skos:broader rdf:resource="descriptor/10975-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10414-2">
    <skos:altLabel xml:lang="de">Wirtschaftliches Wachstum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10414-2">
    <skos:altLabel xml:lang="de">Wachstum der Gesamtwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72181">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11037-1">
    <skos:prefLabel xml:lang="en">Needs-based principle</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11037-1">
    <skos:prefLabel xml:lang="de">Bedarfsprinzip</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12366-5">
    <skos:narrower rdf:resource="descriptor/12282-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14833-4">
    <skos:altLabel xml:lang="de">Fußbekleidung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14928-0">
    <skos:altLabel xml:lang="de">Brennereihefe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14928-0">
    <skos:altLabel xml:lang="de">Bierhefe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14928-0">
    <skos:altLabel xml:lang="de">Backhefe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18365-5">
    <skos:prefLabel xml:lang="en">EU contract award regulation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18365-5">
    <skos:prefLabel xml:lang="de">EU-Vergaberecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18188-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18390-6">
    <skos:altLabel xml:lang="de">Grubentechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18390-6">
    <skos:altLabel xml:lang="de">Fördertechnik im Bergbau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18390-6">
    <skos:altLabel xml:lang="de">Bergbautechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71107">
    <skos:narrower rdf:resource="descriptor/19264-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71107">
    <skos:narrower rdf:resource="descriptor/13744-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71107">
    <skos:narrower rdf:resource="descriptor/13743-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71107">
    <skos:narrower rdf:resource="descriptor/16080-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71107">
    <skos:narrower rdf:resource="descriptor/18727-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71107">
    <skos:narrower rdf:resource="descriptor/13742-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71107">
    <skos:narrower rdf:resource="descriptor/12265-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71107">
    <skos:narrower rdf:resource="descriptor/18682-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71107">
    <skos:narrower rdf:resource="descriptor/18681-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71107">
    <skos:narrower rdf:resource="descriptor/12230-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71107">
    <skos:narrower rdf:resource="descriptor/12299-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71107">
    <skos:narrower rdf:resource="descriptor/16079-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71107">
    <skos:narrower rdf:resource="descriptor/13767-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71107">
    <skos:narrower rdf:resource="descriptor/13724-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71107">
    <skos:narrower rdf:resource="descriptor/24733-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71107">
    <skos:narrower rdf:resource="descriptor/13727-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71107">
    <skos:narrower rdf:resource="descriptor/19535-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71107">
    <skos:narrower rdf:resource="thsys/71105"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71107">
    <skos:narrower rdf:resource="thsys/71106"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17716-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12848-1">
    <skos:altLabel xml:lang="de">Steuerwirkungslehre (BWL)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12848-1">
    <skos:altLabel xml:lang="de">Steuerlehre des Betriebs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12848-1">
    <skos:altLabel xml:lang="de">Steuerbetriebslehre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12848-1">
    <skos:altLabel xml:lang="de">Betriebliche Steuerlehre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19735-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13630-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70967">
    <rdfs:label xml:lang="en">Housing construction</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70967">
    <rdfs:label xml:lang="de">Wohnungsbau</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12069-4">
    <skos:altLabel xml:lang="de">Unternehmenserfolg</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12069-4">
    <skos:altLabel xml:lang="de">Business performance</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15770-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11592-1">
    <skos:altLabel xml:lang="de">Landeshaushalt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10062-0">
    <skos:scopeNote xml:lang="de">Nur für Volkswirtschaftstheorie.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16680-0">
    <skos:scopeNote xml:lang="de">Im engeren Sinne die soziologische Analyse klassischer Kulturformen (Musik, Literatur, usw.). Im weiteren Sinne die soziologische Betrachtung aller kulturellen Erscheinungen einer Gesellschaft: der herrschenden und der Subkultur, der politischen, Medien-, Konsum- und Esskultur, aber auch der Religion und der Lebensstile</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11422-5">
    <skos:broader rdf:resource="thsys/70055"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13327-3">
    <skos:related rdf:resource="descriptor/18267-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13327-3">
    <skos:related rdf:resource="descriptor/14052-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13327-3">
    <skos:related rdf:resource="descriptor/15734-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13327-3">
    <skos:related rdf:resource="descriptor/18429-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14524-3">
    <skos:altLabel xml:lang="de">Videofilm</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14524-3">
    <skos:altLabel xml:lang="de">Bespielte Videokassette</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10816-3">
    <skos:altLabel xml:lang="en">Sovereign risk</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10816-3">
    <skos:altLabel xml:lang="de">Länder-Rating</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10816-3">
    <skos:altLabel xml:lang="de">Ländergruppenrisiko</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10816-3">
    <skos:altLabel xml:lang="de">Kreditwürdigkeit von Staaten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18309-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16278-3">
    <skos:related rdf:resource="descriptor/10626-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16278-3">
    <skos:related rdf:resource="descriptor/16436-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16278-3">
    <skos:related rdf:resource="descriptor/19688-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16278-3">
    <skos:related rdf:resource="descriptor/16421-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16278-3">
    <skos:related rdf:resource="descriptor/19705-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16278-3">
    <skos:related rdf:resource="descriptor/19708-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16278-3">
    <skos:related rdf:resource="descriptor/19071-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16278-3">
    <skos:related rdf:resource="descriptor/16437-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19691-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13026-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19179-4">
    <skos:narrower rdf:resource="descriptor/17933-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19179-4">
    <skos:narrower rdf:resource="descriptor/17927-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19179-4">
    <skos:narrower rdf:resource="descriptor/17915-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19179-4">
    <skos:narrower rdf:resource="descriptor/17911-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15251-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13577-1">
    <skos:altLabel xml:lang="de">Verkehrstechnologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13577-1">
    <skos:altLabel xml:lang="de">Verkehrstechnische Einrichtung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13577-1">
    <skos:altLabel xml:lang="de">Verkehrssystemtechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13577-1">
    <skos:altLabel xml:lang="de">Transportsystemtechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13577-1">
    <skos:altLabel xml:lang="de">Technisches Verkehrssystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18895-4">
    <skos:altLabel xml:lang="en">Conventional medicine</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18895-4">
    <skos:altLabel xml:lang="de">Volksmedizin</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18895-4">
    <skos:altLabel xml:lang="de">Ethnomedizin</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70957">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.10.01.04</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70844">
    <rdfs:label xml:lang="de">Hochschulen</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12978-2">
    <skos:related rdf:resource="descriptor/18371-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12978-2">
    <skos:related rdf:resource="descriptor/19385-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12978-2">
    <skos:related rdf:resource="descriptor/12983-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12978-2">
    <skos:related rdf:resource="descriptor/14196-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18306-0">
    <skos:prefLabel xml:lang="en">Milk packaging</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18306-0">
    <skos:prefLabel xml:lang="de">Milchverpackung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15741-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#memberList">
    <rdfs:range rdf:resource="http://www.w3.org/1999/02/22-rdf-syntax-ns#List"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13402-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18265-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18548-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18581-6">
    <skos:related rdf:resource="descriptor/18580-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18581-6">
    <skos:related rdf:resource="descriptor/13813-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16788-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14716-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17500-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12105-5">
    <skos:scopeNote xml:lang="de">Nicht im Sinne der historischen Entwicklung einer Organisation, sondern im Sinne gestaltender Eingriffe in die Organisationsstruktur</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71093">
    <skos:broader rdf:resource="thsys/70179"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14125-3">
    <skos:broader rdf:resource="thsys/73304"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14125-3">
    <skos:broader rdf:resource="thsys/70467"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14125-3">
    <skos:broader rdf:resource="descriptor/14129-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14125-3">
    <skos:broader rdf:resource="descriptor/14851-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15415-3">
    <skos:altLabel xml:lang="de">Steuerungstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15415-3">
    <skos:altLabel xml:lang="de">Rückkoppelungstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15415-3">
    <skos:altLabel xml:lang="de">Regelungstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15415-3">
    <skos:altLabel xml:lang="de">Regelsystem (Kybernetik)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15415-3">
    <skos:altLabel xml:lang="de">Regelkreis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15415-3">
    <skos:altLabel xml:lang="de">Kybernetisches System</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15415-3">
    <skos:altLabel xml:lang="de">Kybernetische Systemtheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15415-3">
    <skos:altLabel xml:lang="de">Blockschaltbild</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10545-1">
    <skos:related rdf:resource="descriptor/18382-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10545-1">
    <skos:related rdf:resource="descriptor/10593-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10545-1">
    <skos:related rdf:resource="descriptor/10566-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19529-3">
    <skos:prefLabel xml:lang="en">Radiology</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19529-3">
    <skos:prefLabel xml:lang="de">Radiologie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18913-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16102-5">
    <skos:related rdf:resource="descriptor/10438-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15023-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19645-1">
    <skos:related rdf:resource="descriptor/15949-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19645-1">
    <skos:related rdf:resource="descriptor/15943-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19645-1">
    <skos:related rdf:resource="descriptor/10489-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14802-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18060-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19916-3">
    <skos:broader rdf:resource="thsys/70847"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19916-3">
    <skos:broader rdf:resource="thsys/70176"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19916-3">
    <skos:broader rdf:resource="thsys/70143"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18130-4">
    <skos:prefLabel xml:lang="en">Economic ethics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18130-4">
    <skos:prefLabel xml:lang="de">Wirtschaftsethik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13367-5">
    <skos:prefLabel xml:lang="en">Publishing industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13367-5">
    <skos:prefLabel xml:lang="de">Verlagsgewerbe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10091-0">
    <skos:prefLabel xml:lang="en">Coase Theorem</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10091-0">
    <skos:prefLabel xml:lang="de">Coase-Theorem</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19397-1">
    <skos:prefLabel xml:lang="en">Fertilizer market</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19397-1">
    <skos:prefLabel xml:lang="de">Düngemittelmarkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16651-0">
    <skos:altLabel xml:lang="de">Solidarismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20522-0">
    <skos:prefLabel xml:lang="en">Regulated firm</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20522-0">
    <skos:prefLabel xml:lang="de">Unternehmensregulierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18729-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10699-2">
    <skos:prefLabel xml:lang="en">Import restrictions</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10699-2">
    <skos:prefLabel xml:lang="de">Importbeschränkung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18558-1">
    <skos:altLabel xml:lang="en">UPC (Universal Product Code)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18558-1">
    <skos:altLabel xml:lang="en">Universal Product Code</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18558-1">
    <skos:altLabel xml:lang="en">European article number</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18558-1">
    <skos:altLabel xml:lang="de">Strichcode</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18558-1">
    <skos:altLabel xml:lang="de">EAN (Europäische Artikelnummer)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18558-1">
    <skos:altLabel xml:lang="de">Balkencode</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70557">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19771-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16510-4">
    <skos:altLabel xml:lang="en">Domestic security</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16510-4">
    <skos:altLabel xml:lang="de">Politik der inneren Sicherheit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12782-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12832-2">
    <skos:prefLabel xml:lang="en">Comparative advertising</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12832-2">
    <skos:prefLabel xml:lang="de">Vergleichende Werbung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12787-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16604-2">
    <skos:prefLabel xml:lang="en">Information society</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16604-2">
    <skos:prefLabel xml:lang="de">Informationsgesellschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20972-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15643-4">
    <skos:broader rdf:resource="thsys/70819"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15643-4">
    <skos:broader rdf:resource="thsys/70581"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15643-4">
    <skos:broader rdf:resource="descriptor/15675-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21101-5">
    <skos:altLabel xml:lang="en">Changeover</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10102-0">
    <skos:broader rdf:resource="thsys/70057"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10102-0">
    <skos:broader rdf:resource="descriptor/10113-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19285-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15010-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19541-6">
    <skos:prefLabel xml:lang="en">Call centre</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19541-6">
    <skos:prefLabel xml:lang="de">Call Center</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19470-3">
    <skos:altLabel xml:lang="de">Städtische Dienstleistung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18247-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17634-2">
    <skos:prefLabel xml:lang="en">Saudi Arabia</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17634-2">
    <skos:prefLabel xml:lang="de">Saudi-Arabien</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14048-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14407-0">
    <skos:broader rdf:resource="thsys/71015"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14407-0">
    <skos:broader rdf:resource="thsys/71014"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14407-0">
    <skos:broader rdf:resource="descriptor/14039-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19718-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18587-1">
    <skos:broader rdf:resource="thsys/70939"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18587-1">
    <skos:broader rdf:resource="thsys/70936"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18587-1">
    <skos:broader rdf:resource="thsys/70924"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18587-1">
    <skos:broader rdf:resource="descriptor/13644-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18587-1">
    <skos:broader rdf:resource="descriptor/18589-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70573">
    <skos:narrower rdf:resource="descriptor/13374-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70573">
    <skos:narrower rdf:resource="descriptor/21002-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70573">
    <skos:narrower rdf:resource="descriptor/16541-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70573">
    <skos:narrower rdf:resource="descriptor/18738-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70573">
    <skos:narrower rdf:resource="descriptor/18180-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70573">
    <skos:narrower rdf:resource="descriptor/15947-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70573">
    <skos:narrower rdf:resource="descriptor/15658-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70573">
    <skos:narrower rdf:resource="descriptor/19551-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70573">
    <skos:narrower rdf:resource="descriptor/19465-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70573">
    <skos:narrower rdf:resource="descriptor/16670-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70573">
    <skos:narrower rdf:resource="descriptor/13292-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70573">
    <skos:narrower rdf:resource="descriptor/19518-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70573">
    <skos:narrower rdf:resource="descriptor/18903-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70573">
    <skos:narrower rdf:resource="descriptor/13372-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70573">
    <skos:narrower rdf:resource="descriptor/13371-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70573">
    <skos:narrower rdf:resource="descriptor/18892-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70573">
    <skos:narrower rdf:resource="descriptor/18891-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70573">
    <skos:narrower rdf:resource="descriptor/19578-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70573">
    <skos:narrower rdf:resource="descriptor/16683-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70573">
    <skos:narrower rdf:resource="descriptor/18063-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70573">
    <skos:narrower rdf:resource="descriptor/16479-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70573">
    <skos:narrower rdf:resource="descriptor/11157-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70573">
    <skos:narrower rdf:resource="descriptor/11555-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70573">
    <skos:narrower rdf:resource="descriptor/19517-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70573">
    <skos:narrower rdf:resource="descriptor/16635-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70573">
    <skos:narrower rdf:resource="descriptor/13415-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70573">
    <skos:narrower rdf:resource="descriptor/16531-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70573">
    <skos:narrower rdf:resource="descriptor/18334-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70573">
    <skos:narrower rdf:resource="descriptor/15908-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70573">
    <skos:narrower rdf:resource="descriptor/18894-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19746-2">
    <skos:scopeNote xml:lang="de">Quantitative und qualitative Analyse wissenschaftlicher Literaturproduktion. Untersuchungen über die Zitierhäufigkeit und das Zitationsmuster. Rezeption wissenschaftlicher Literatur. Statistische Untersuchungen über den Wandel der Thematik. Wissenschaftlicher Wert einer Veröffentlichung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19587-3">
    <skos:related rdf:resource="descriptor/15105-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19587-3">
    <skos:related rdf:resource="descriptor/10374-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19587-3">
    <skos:related rdf:resource="descriptor/10338-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19587-3">
    <skos:related rdf:resource="descriptor/19586-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11663-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17181-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19707-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14122-2">
    <skos:related rdf:resource="descriptor/14964-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14122-2">
    <skos:related rdf:resource="descriptor/19340-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19159-3">
    <skos:prefLabel xml:lang="en">Bankruptcy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19159-3">
    <skos:prefLabel xml:lang="de">Konkurs</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19786-4">
    <zbwext:useInsteadNote rdf:nodeID="id11"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11786-2">
    <skos:scopeNote xml:lang="de">Verflechtung der Branchen</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11569-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18333-4">
    <skos:related rdf:resource="descriptor/18329-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18333-4">
    <skos:related rdf:resource="descriptor/12020-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13622-1">
    <skos:altLabel xml:lang="en">Multimodal transport</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13622-1">
    <skos:altLabel xml:lang="en">Intermodal transport</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13622-1">
    <skos:altLabel xml:lang="de">Transportkette</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13622-1">
    <skos:altLabel xml:lang="de">Ro-Ro-Verkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13622-1">
    <skos:altLabel xml:lang="de">Roll-on-Roll-off-Verkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13622-1">
    <skos:altLabel xml:lang="de">Multimodaler Verkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13622-1">
    <skos:altLabel xml:lang="de">KLV (Kombinierter Ladungsverkehr)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13622-1">
    <skos:altLabel xml:lang="de">Intermodaler Güterverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13622-1">
    <skos:altLabel xml:lang="de">Huckepackverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18374-4">
    <skos:broader rdf:resource="thsys/71012"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18374-4">
    <skos:broader rdf:resource="thsys/70539"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18374-4">
    <skos:broader rdf:resource="descriptor/11810-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18374-4">
    <skos:broader rdf:resource="descriptor/18375-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18228-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16004-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19732-6">
    <skos:related rdf:resource="descriptor/19743-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12722-2">
    <skos:altLabel xml:lang="de">Kleinserienproduktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11683-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73370">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12620-3">
    <skos:altLabel xml:lang="en">Stress at work</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12620-3">
    <skos:altLabel xml:lang="de">Stressmanagement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12620-3">
    <skos:altLabel xml:lang="de">Streßmanagement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12620-3">
    <skos:altLabel xml:lang="de">Streß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12620-3">
    <skos:altLabel xml:lang="de">Psychosozialer Stress</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12620-3">
    <skos:altLabel xml:lang="de">Psychosozialer Streß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12620-3">
    <skos:altLabel xml:lang="de">Physischer Stress</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12620-3">
    <skos:altLabel xml:lang="de">Physischer Streß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12620-3">
    <skos:altLabel xml:lang="de">Arbeitsstress</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12620-3">
    <skos:altLabel xml:lang="de">Arbeitsstreß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12620-3">
    <skos:altLabel xml:lang="de">Arbeitsbelastung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12620-3">
    <skos:altLabel xml:lang="de">Arbeitsbeanspruchung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18101-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12523-1">
    <skos:altLabel xml:lang="en">Standard cost accounting</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12523-1">
    <skos:altLabel xml:lang="en">Budget accounting</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12523-1">
    <skos:altLabel xml:lang="de">Standardkostenrechnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12523-1">
    <skos:altLabel xml:lang="de">Richtkostenrechnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12523-1">
    <skos:altLabel xml:lang="de">Plankosten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12523-1">
    <skos:altLabel xml:lang="de">Kostenplanung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12523-1">
    <skos:altLabel xml:lang="de">Flexible Plankostenrechnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12523-1">
    <skos:altLabel xml:lang="de">Budgetkostenrechnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12567-2">
    <skos:prefLabel xml:lang="en">Human resource management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12567-2">
    <skos:prefLabel xml:lang="de">Personalwirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16416-3">
    <skos:broader rdf:resource="thsys/73383"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16416-3">
    <skos:broader rdf:resource="descriptor/16405-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71074">
    <skos:broader rdf:resource="thsys/71077"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17933-6">
    <skos:broader rdf:resource="thsys/71030"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17933-6">
    <skos:broader rdf:resource="descriptor/17931-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17933-6">
    <skos:broader rdf:resource="descriptor/19179-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18348-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14515-4">
    <skos:altLabel xml:lang="en">TV set</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14515-4">
    <skos:altLabel xml:lang="en">Colour TV</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14515-4">
    <skos:altLabel xml:lang="de">Farbfernsehgerät</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15690-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73394">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14223-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10215-1">
    <skos:prefLabel xml:lang="en">Price discrimination</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10215-1">
    <skos:prefLabel xml:lang="de">Preisdifferenzierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id239">
    <rdf:value xml:lang="de">Für Kernkraftwerke</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18430-6">
    <skos:prefLabel xml:lang="en">Chemical research</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18430-6">
    <skos:prefLabel xml:lang="de">Chemieforschung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19293-6">
    <skos:broader rdf:resource="thsys/71029"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19293-6">
    <skos:broader rdf:resource="thsys/70960"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19293-6">
    <skos:broader rdf:resource="descriptor/19674-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19030-3">
    <skos:altLabel xml:lang="de">Begriffsbestimmung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12513-4">
    <skos:broader rdf:resource="thsys/70578"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12513-4">
    <skos:broader rdf:resource="descriptor/12521-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19703-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11065-3">
    <skos:related rdf:resource="descriptor/18155-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11065-3">
    <skos:related rdf:resource="descriptor/19213-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11065-3">
    <skos:related rdf:resource="descriptor/11066-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11065-3">
    <skos:related rdf:resource="descriptor/11160-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11065-3">
    <skos:related rdf:resource="descriptor/10094-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11065-3">
    <skos:related rdf:resource="descriptor/11161-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11065-3">
    <skos:related rdf:resource="descriptor/11040-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11065-3">
    <skos:related rdf:resource="descriptor/10091-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16007-6">
    <skos:broader rdf:resource="thsys/70160"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16007-6">
    <skos:broader rdf:resource="descriptor/15966-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15105-4">
    <skos:scopeNote xml:lang="de">Nur für gesamtwirtschaftliche Produktionsstatistiken.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10246-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13221-5">
    <skos:altLabel xml:lang="de">Treibstoffhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13221-5">
    <skos:altLabel xml:lang="de">Mineralölgroßhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13221-5">
    <skos:altLabel xml:lang="de">Kraftstoffhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13221-5">
    <skos:altLabel xml:lang="de">Kraftstoffgroßhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13221-5">
    <skos:altLabel xml:lang="de">Heizölhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13221-5">
    <skos:altLabel xml:lang="de">Erdölhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12754-3">
    <skos:altLabel xml:lang="de">Optimale Kapazität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12754-3">
    <skos:altLabel xml:lang="de">Kapazitätsterminierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12754-3">
    <skos:altLabel xml:lang="de">Kapazitätsoptimierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13475-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16486-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10038-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11392-2">
    <skos:prefLabel xml:lang="en">Vocational training policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11392-2">
    <skos:prefLabel xml:lang="de">Berufsbildungspolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12837-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19723-0">
    <skos:broader rdf:resource="thsys/73363"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19723-0">
    <skos:broader rdf:resource="thsys/70865"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19723-0">
    <skos:broader rdf:resource="descriptor/16460-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19723-0">
    <skos:broader rdf:resource="descriptor/19328-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13708-5">
    <skos:prefLabel xml:lang="en">Regional bank</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13708-5">
    <skos:prefLabel xml:lang="de">Regionalbank</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11284-5">
    <skos:altLabel xml:lang="de">Mitarbeiterinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11284-5">
    <skos:altLabel xml:lang="de">Lohnempfängerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11284-5">
    <skos:altLabel xml:lang="de">Arbeitnehmerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11853-6">
    <skos:prefLabel xml:lang="en">Border region</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11853-6">
    <skos:prefLabel xml:lang="de">Grenzgebiet</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16651-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21315-0">
    <skos:prefLabel xml:lang="en">Online marketing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21315-0">
    <skos:prefLabel xml:lang="de">Online-Werbung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14272-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10164-6">
    <skos:related rdf:resource="descriptor/10106-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10164-6">
    <skos:related rdf:resource="descriptor/11527-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10164-6">
    <skos:related rdf:resource="descriptor/19150-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10164-6">
    <skos:related rdf:resource="descriptor/11718-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10164-6">
    <skos:related rdf:resource="descriptor/10159-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10164-6">
    <skos:related rdf:resource="descriptor/15470-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16825-0">
    <skos:prefLabel xml:lang="en">Central Europe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16825-0">
    <skos:prefLabel xml:lang="de">Mitteleuropa</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13480-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13693-6">
    <skos:altLabel xml:lang="de">Kreditwesenrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13693-6">
    <skos:altLabel xml:lang="de">Kreditwesengesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13693-6">
    <skos:altLabel xml:lang="de">Depotrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13693-6">
    <skos:altLabel xml:lang="de">Depotgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13693-6">
    <skos:altLabel xml:lang="de">Bankvertragsrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70986">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id110">
    <rdfs:seeAlso rdf:resource="descriptor/19468-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18374-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19210-1">
    <skos:prefLabel xml:lang="en">Willingness to pay</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19210-1">
    <skos:prefLabel xml:lang="de">Willingness to pay</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16288-0">
    <skos:scopeNote xml:lang="de">Nationale und internationale schwerwiegende politische Konfliktsituationen</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11389-5">
    <skos:narrower rdf:resource="descriptor/11388-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70004">
    <skos:prefLabel xml:lang="en">B.03.03  Company valuation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70004">
    <skos:prefLabel xml:lang="de">B.03.03  Unternehmensbewertung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70238">
    <rdfs:label xml:lang="en">Environmental degradation</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70238">
    <rdfs:label xml:lang="de">Umweltbelastung</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18487-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19468-4">
    <skos:related rdf:resource="descriptor/19472-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19468-4">
    <skos:related rdf:resource="descriptor/10510-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19468-4">
    <skos:related rdf:resource="descriptor/19446-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19468-4">
    <skos:related rdf:resource="descriptor/19467-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19185-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13499-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16676-5">
    <skos:related rdf:resource="descriptor/18900-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16676-5">
    <skos:related rdf:resource="descriptor/19013-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16676-5">
    <skos:related rdf:resource="descriptor/13354-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16676-5">
    <skos:related rdf:resource="descriptor/18222-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16676-5">
    <skos:related rdf:resource="descriptor/16668-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16676-5">
    <skos:related rdf:resource="descriptor/16666-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14967-4">
    <skos:prefLabel xml:lang="en">Canned fish</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14967-4">
    <skos:prefLabel xml:lang="de">Fischkonserve</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19716-4">
    <skos:related rdf:resource="descriptor/16441-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19716-4">
    <skos:related rdf:resource="descriptor/16489-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70254">
    <rdfs:label xml:lang="de">Elektronische Produkte</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13605-1">
    <skos:related rdf:resource="descriptor/13632-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16696-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18341-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12994-4">
    <skos:prefLabel xml:lang="en">Synthethic fiber industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12994-4">
    <skos:prefLabel xml:lang="de">Chemiefaserindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11192-3">
    <skos:prefLabel xml:lang="en">Work absence</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11192-3">
    <skos:prefLabel xml:lang="de">Fehlzeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18415-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11060-6">
    <skos:narrower rdf:resource="descriptor/11048-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13100-3">
    <skos:prefLabel xml:lang="en">Bookbinding</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13100-3">
    <skos:prefLabel xml:lang="de">Buchbinderei</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12592-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12558-3">
    <skos:altLabel xml:lang="de">Teilwertabschreibung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12558-3">
    <skos:altLabel xml:lang="de">Sonderabschreibung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12558-3">
    <skos:altLabel xml:lang="de">AfA (Absetzung für Abnutzung)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12558-3">
    <skos:altLabel xml:lang="de">Absetzung für Abnutzung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70242">
    <rdfs:label xml:lang="en">Competition theory</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70242">
    <rdfs:label xml:lang="de">Wettbewerbstheorie</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18624-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18996-5">
    <skos:related rdf:resource="descriptor/18995-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18996-5">
    <skos:related rdf:resource="descriptor/19182-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18996-5">
    <skos:related rdf:resource="descriptor/12975-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18996-5">
    <skos:related rdf:resource="descriptor/11812-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18996-5">
    <skos:related rdf:resource="descriptor/11810-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18996-5">
    <skos:related rdf:resource="descriptor/18350-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18996-5">
    <skos:related rdf:resource="descriptor/18989-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18996-5">
    <skos:related rdf:resource="descriptor/14175-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16029-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18651-4">
    <skos:prefLabel xml:lang="en">Telecommunications research</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18651-4">
    <skos:prefLabel xml:lang="de">Telekommunikationsforschung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19668-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13530-6">
    <skos:related rdf:resource="descriptor/18589-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13530-6">
    <skos:related rdf:resource="descriptor/18247-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#altLabel">
    <rdfs:comment xml:lang="en">skos:prefLabel, skos:altLabel and skos:hiddenLabel are pairwise
      disjoint properties.</rdfs:comment>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#altLabel">
    <rdfs:comment xml:lang="en">The range of skos:altLabel is the class of RDF plain literals.</rdfs:comment>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#altLabel">
    <rdfs:comment xml:lang="en">Acronyms, abbreviations, spelling variants, and irregular
      plural/singular forms may be included among the alternative labels for a concept. Mis-spelled
      terms are normally included as hidden labels (see skos:hiddenLabel).</rdfs:comment>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10843-0">
    <skos:altLabel xml:lang="de">Intrasektoraler Außenhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18349-3">
    <skos:broader rdf:resource="thsys/71033"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18349-3">
    <skos:broader rdf:resource="thsys/71016"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18349-3">
    <skos:broader rdf:resource="thsys/70227"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18349-3">
    <skos:broader rdf:resource="descriptor/18165-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18349-3">
    <skos:broader rdf:resource="descriptor/18350-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11289-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15694-1">
    <skos:scopeNote xml:lang="de">Geschichtliche Darstellungen von Bräuchen, Normen, Konventionen oder Mentalitäten (Sprache, Religion, Ernährung, Mode, usw.) eines bestimmten Kulturkreises bzw. eines dieser Aspekte; auch die historische Entwicklung der kulturellen Aktivitäten im engeren Sinne (Literatur, Musik, Kunst, usw.)</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11217-6">
    <skos:related rdf:resource="descriptor/15979-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11217-6">
    <skos:related rdf:resource="descriptor/19813-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11217-6">
    <skos:related rdf:resource="descriptor/15698-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11217-6">
    <skos:related rdf:resource="descriptor/16304-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11217-6">
    <skos:related rdf:resource="descriptor/11051-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16283-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10521-1">
    <skos:altLabel xml:lang="de">Fourastie-Hypothese</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10521-1">
    <skos:altLabel xml:lang="de">Drei-Sektoren-Entwicklungstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10521-1">
    <skos:altLabel xml:lang="de">Drei-Sektoren Hypothese</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18265-2">
    <skos:broader rdf:resource="thsys/70536"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18265-2">
    <skos:broader rdf:resource="descriptor/14734-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70940">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12768-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16190-3">
    <skos:related rdf:resource="descriptor/19810-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16190-3">
    <skos:related rdf:resource="descriptor/15605-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16190-3">
    <skos:related rdf:resource="descriptor/18433-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16190-3">
    <skos:related rdf:resource="descriptor/14751-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11328-4">
    <skos:related rdf:resource="descriptor/19495-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11328-4">
    <skos:related rdf:resource="descriptor/18910-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11328-4">
    <skos:related rdf:resource="descriptor/10622-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11328-4">
    <skos:related rdf:resource="descriptor/19509-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11328-4">
    <skos:related rdf:resource="descriptor/19506-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70406">
    <rdfs:label xml:lang="en">Economic growth</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70406">
    <rdfs:label xml:lang="de">Wirtschaftswachstum</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18519-4">
    <skos:broader rdf:resource="thsys/70960"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18519-4">
    <skos:broader rdf:resource="descriptor/13219-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18519-4">
    <skos:broader rdf:resource="descriptor/13188-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19116-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18769-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10701-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14043-5">
    <skos:altLabel xml:lang="de">Wohnmöbel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14043-5">
    <skos:altLabel xml:lang="de">Schlafmöbel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14043-5">
    <skos:altLabel xml:lang="de">Mobiliar</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15051-1">
    <skos:prefLabel xml:lang="en">Network technique</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15051-1">
    <skos:prefLabel xml:lang="de">Netzplantechnik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13141-3">
    <skos:related rdf:resource="descriptor/18435-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13141-3">
    <skos:related rdf:resource="descriptor/14846-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13141-3">
    <skos:related rdf:resource="descriptor/19061-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13141-3">
    <skos:related rdf:resource="descriptor/18436-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13141-3">
    <skos:related rdf:resource="descriptor/13935-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13141-3">
    <skos:related rdf:resource="descriptor/18094-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13141-3">
    <skos:related rdf:resource="descriptor/14899-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15873-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11153-6">
    <skos:broader rdf:resource="thsys/70180"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16405-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12649-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13475-2">
    <skos:broader rdf:resource="thsys/71054"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13475-2">
    <skos:broader rdf:resource="thsys/70963"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13475-2">
    <skos:broader rdf:resource="thsys/70459"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13475-2">
    <skos:broader rdf:resource="descriptor/13970-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17915-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18274-1">
    <skos:altLabel xml:lang="de">Fotovoltaikanlage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19318-2">
    <skos:altLabel xml:lang="de">Ausfuhrprämie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70233">
    <skos:broader rdf:resource="thsys/70234"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70120">
    <skos:narrower rdf:resource="descriptor/19755-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70120">
    <skos:narrower rdf:resource="descriptor/19733-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70120">
    <skos:narrower rdf:resource="descriptor/15764-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70120">
    <skos:narrower rdf:resource="descriptor/16642-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70120">
    <skos:narrower rdf:resource="descriptor/15821-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70120">
    <skos:narrower rdf:resource="descriptor/19754-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70120">
    <skos:narrower rdf:resource="descriptor/10498-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70120">
    <skos:narrower rdf:resource="descriptor/19198-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70120">
    <skos:narrower rdf:resource="descriptor/16633-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70120">
    <skos:narrower rdf:resource="descriptor/16709-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70120">
    <skos:narrower rdf:resource="descriptor/19220-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70120">
    <skos:narrower rdf:resource="thsys/73358"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70120">
    <skos:narrower rdf:resource="thsys/73359"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70120">
    <skos:narrower rdf:resource="thsys/73357"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70120">
    <skos:narrower rdf:resource="thsys/73356"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70055">
    <rdfs:label xml:lang="de">Geld- und Finanzmärkte</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18907-2">
    <skos:related rdf:resource="descriptor/18180-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18907-2">
    <skos:related rdf:resource="descriptor/14007-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18907-2">
    <skos:related rdf:resource="descriptor/13372-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18907-2">
    <skos:related rdf:resource="descriptor/13312-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18907-2">
    <skos:related rdf:resource="descriptor/18972-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18907-2">
    <skos:related rdf:resource="descriptor/16525-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15927-4">
    <skos:related rdf:resource="descriptor/15659-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15927-4">
    <skos:related rdf:resource="descriptor/15945-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15927-4">
    <skos:related rdf:resource="descriptor/15920-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15927-4">
    <skos:related rdf:resource="descriptor/15938-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15883-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12016-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70372">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14824-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19518-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18907-2">
    <skos:prefLabel xml:lang="en">Long-term care</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18907-2">
    <skos:prefLabel xml:lang="de">Pflegedienst</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19845-0">
    <skos:related rdf:resource="descriptor/15961-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19845-0">
    <skos:related rdf:resource="descriptor/19847-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19845-0">
    <skos:related rdf:resource="descriptor/15604-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18547-6">
    <skos:altLabel xml:lang="de">Second-Hand-Markt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18547-6">
    <skos:altLabel xml:lang="de">Gebrauchtwarenmarkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18547-6">
    <skos:altLabel xml:lang="de">Gebrauchtwarenhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18547-6">
    <skos:altLabel xml:lang="de">Gebrauchtgütermarkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20888-0">
    <skos:scopeNote xml:lang="de">Instrument zur Förderung der Nutzung regenerativer Energieträger.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21042-2">
    <skos:related rdf:resource="descriptor/17578-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11142-4">
    <skos:narrower rdf:resource="descriptor/11149-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18888-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18514-0">
    <skos:broader rdf:resource="thsys/70961"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18514-0">
    <skos:broader rdf:resource="descriptor/13217-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18514-0">
    <skos:broader rdf:resource="descriptor/18547-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18546-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15771-2">
    <skos:prefLabel xml:lang="en">Environmental investment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15771-2">
    <skos:prefLabel xml:lang="de">Umweltschutzinvestition</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10736-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id229">
    <rdf:value xml:lang="de">Für Sicherheit vor innerer Subversion und Unruhen</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19122-5">
    <skos:altLabel xml:lang="en">Market niche</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19122-5">
    <skos:altLabel xml:lang="de">Strategische Nische</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19122-5">
    <skos:altLabel xml:lang="de">Nischenmarketing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15017-1">
    <skos:broader rdf:resource="thsys/70238"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15017-1">
    <skos:broader rdf:resource="thsys/70161"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15017-1">
    <skos:broader rdf:resource="descriptor/18613-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15017-1">
    <skos:broader rdf:resource="descriptor/15009-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16267-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11331-1">
    <skos:related rdf:resource="descriptor/16150-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11331-1">
    <skos:related rdf:resource="descriptor/11324-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14614-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14328-3">
    <skos:prefLabel xml:lang="en">Tractor</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14328-3">
    <skos:prefLabel xml:lang="de">Traktor</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73303">
    <skos:broader rdf:resource="thsys/70498"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16615-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12914-0">
    <skos:prefLabel xml:lang="en">Plant construction</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12914-0">
    <skos:prefLabel xml:lang="de">Anlagenbau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11219-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id102">
    <rdfs:seeAlso rdf:resource="descriptor/18169-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11979-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73299">
    <skos:narrower rdf:resource="descriptor/14990-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73299">
    <skos:narrower rdf:resource="descriptor/14993-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73299">
    <skos:narrower rdf:resource="descriptor/14991-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73299">
    <skos:narrower rdf:resource="descriptor/14989-1"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id81">
    <rdf:value xml:lang="de">Für thermoplastische Elastomere</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70481">
    <skos:prefLabel xml:lang="en">V.02.02  Theory of the firm</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70481">
    <skos:prefLabel xml:lang="de">V.02.02  Theorie der Unternehmung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11054-1">
    <skos:altLabel xml:lang="de">Theorie des Marktversagens</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11769-2">
    <skos:broader rdf:resource="thsys/71113"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11769-2">
    <skos:broader rdf:resource="thsys/71046"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11769-2">
    <skos:broader rdf:resource="thsys/70245"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11769-2">
    <skos:broader rdf:resource="thsys/70235"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11769-2">
    <skos:broader rdf:resource="descriptor/11778-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11769-2">
    <skos:broader rdf:resource="descriptor/12282-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18615-1">
    <skos:related rdf:resource="descriptor/18620-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18615-1">
    <skos:related rdf:resource="descriptor/18616-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18615-1">
    <skos:related rdf:resource="descriptor/13621-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18615-1">
    <skos:related rdf:resource="descriptor/13601-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18615-1">
    <skos:related rdf:resource="descriptor/13565-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16898-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19245-3">
    <skos:related rdf:resource="descriptor/18060-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11392-2">
    <skos:related rdf:resource="descriptor/11369-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11392-2">
    <skos:related rdf:resource="descriptor/16134-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11392-2">
    <skos:related rdf:resource="descriptor/11360-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11392-2">
    <skos:related rdf:resource="descriptor/11389-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11392-2">
    <skos:related rdf:resource="descriptor/11406-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11392-2">
    <skos:related rdf:resource="descriptor/11235-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24847-1">
    <skos:scopeNote xml:lang="de">Für das Organisationsverhalten von Individuen BENUTZE: Mitgliedschaft oder: Gewerkschaftlicher Organisationsgrad.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24847-1">
    <skos:scopeNote xml:lang="de">Hier definiert im Sinne des Verhaltens von Organisationen, z.B. von Unternehmen, Verbänden oder Verwaltungen. Analysiert wird dieses Verhalten zumeist als Folge relevanter interner oder externer Veränderungen (z.B. Herausforderungen durch Globalisierung, Änderung der Organisationsstruktur, etc.) im Rahmen organisationspsychologischer oder -soziologischer Untersuchungen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12271-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10282-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18409-4">
    <skos:related rdf:resource="descriptor/13060-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18409-4">
    <skos:related rdf:resource="descriptor/12693-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18409-4">
    <skos:related rdf:resource="descriptor/13045-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12978-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id147">
    <rdfs:seeAlso rdf:resource="descriptor/10142-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18651-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id381">
    <rdfs:seeAlso rdf:resource="descriptor/10785-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11782-3">
    <skos:prefLabel xml:lang="en">Economic structure</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11782-3">
    <skos:prefLabel xml:lang="de">Wirtschaftsstruktur</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73294">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71054">
    <skos:broader rdf:resource="thsys/71055"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13536-1">
    <skos:prefLabel xml:lang="en">Port development</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13536-1">
    <skos:prefLabel xml:lang="de">Hafenentwicklung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12381-2">
    <skos:broader rdf:resource="thsys/70186"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12381-2">
    <skos:broader rdf:resource="descriptor/12307-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12285-5">
    <skos:altLabel xml:lang="en">Stock returns</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12285-5">
    <skos:altLabel xml:lang="en">Investment income</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12285-5">
    <skos:altLabel xml:lang="de">Kapitalgewinn</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12285-5">
    <skos:altLabel xml:lang="de">Kapitaleinkommen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10908-5">
    <skos:prefLabel xml:lang="en">Nonprice competition</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10908-5">
    <skos:prefLabel xml:lang="de">Nichtpreiswettbewerb</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13638-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11554-2">
    <skos:altLabel xml:lang="en">Government spending</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11554-2">
    <skos:altLabel xml:lang="de">Staatsausgaben</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10806-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id78">
    <rdf:value xml:lang="de">Für die generelle Sicherung von Kapitalanlagen</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19746-2">
    <skos:altLabel xml:lang="en">Zipfs law</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19746-2">
    <skos:altLabel xml:lang="en">Referee-system</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19746-2">
    <skos:altLabel xml:lang="en">Invisible college</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19746-2">
    <skos:altLabel xml:lang="en">Content analysis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19746-2">
    <skos:altLabel xml:lang="en">Citation index</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19746-2">
    <skos:altLabel xml:lang="en">Bradfords law</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19746-2">
    <skos:altLabel xml:lang="de">Zitierkartell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19746-2">
    <skos:altLabel xml:lang="de">Zitierhäufigkeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19746-2">
    <skos:altLabel xml:lang="de">Zitationsanalyse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19746-2">
    <skos:altLabel xml:lang="de">Wissenschaftliches Publizieren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19746-2">
    <skos:altLabel xml:lang="de">Wissenschaftliche Publikationsaktivitäten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19746-2">
    <skos:altLabel xml:lang="de">Wirtschaftswissenschaftliche Literaturanalyse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19746-2">
    <skos:altLabel xml:lang="de">Peer-Review-Verfahren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19746-2">
    <skos:altLabel xml:lang="de">Online-Informetrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19746-2">
    <skos:altLabel xml:lang="de">Literaturanalyse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19746-2">
    <skos:altLabel xml:lang="de">Inhaltsanalyse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19746-2">
    <skos:altLabel xml:lang="de">Informetrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19746-2">
    <skos:altLabel xml:lang="de">Bibliometrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73357">
    <skos:prefLabel xml:lang="de">N.06.03.03  Lebensalter, Partnerschaft und Familie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11576-6">
    <zbwext:useInsteadNote rdf:nodeID="id40"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19260-0">
    <skos:broader rdf:resource="thsys/70904"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19260-0">
    <skos:broader rdf:resource="thsys/70305"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16143-5">
    <skos:broader rdf:resource="thsys/70278"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16143-5">
    <skos:broader rdf:resource="thsys/70245"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16143-5">
    <skos:broader rdf:resource="descriptor/16159-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16143-5">
    <skos:broader rdf:resource="descriptor/18910-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16143-5">
    <skos:broader rdf:resource="descriptor/11345-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19334-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11827-0">
    <skos:altLabel xml:lang="de">Wohnstraße</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11827-0">
    <skos:altLabel xml:lang="de">Straßenraumgestaltung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11827-0">
    <skos:altLabel xml:lang="de">Straßenmöblierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11827-0">
    <skos:altLabel xml:lang="de">Fußgängerzone</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11827-0">
    <skos:altLabel xml:lang="de">Fußgängerbereich</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11827-0">
    <skos:altLabel xml:lang="de">Flächenhafte Verkehrsberuhigung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11827-0">
    <skos:altLabel xml:lang="de">Autofreie Stadt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19255-0">
    <skos:narrower rdf:resource="descriptor/11438-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19255-0">
    <skos:narrower rdf:resource="descriptor/11430-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18904-1">
    <skos:related rdf:resource="descriptor/18909-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10527-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10541-2">
    <skos:prefLabel xml:lang="en">Unbalanced growth</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10541-2">
    <skos:prefLabel xml:lang="de">Ungleichgewichtiges Wachstum</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13708-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11602-3">
    <skos:related rdf:resource="descriptor/19370-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18023-5">
    <skos:broader rdf:resource="thsys/70471"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18023-5">
    <skos:broader rdf:resource="descriptor/12758-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18023-5">
    <skos:broader rdf:resource="descriptor/18002-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13349-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10738-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13545-0">
    <skos:altLabel xml:lang="de">Seeverkehrspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13545-0">
    <skos:altLabel xml:lang="de">Schiffahrtspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13545-0">
    <skos:altLabel xml:lang="de">Flaggendiskriminierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11161-0">
    <skos:related rdf:resource="descriptor/10060-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11161-0">
    <skos:related rdf:resource="descriptor/16562-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11161-0">
    <skos:related rdf:resource="descriptor/10984-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11161-0">
    <skos:related rdf:resource="descriptor/10968-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11161-0">
    <skos:related rdf:resource="descriptor/12009-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11161-0">
    <skos:related rdf:resource="descriptor/16712-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11161-0">
    <skos:related rdf:resource="descriptor/11168-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11161-0">
    <skos:related rdf:resource="descriptor/20855-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11161-0">
    <skos:related rdf:resource="descriptor/20826-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11161-0">
    <skos:related rdf:resource="descriptor/11065-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18803-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12750-4">
    <skos:prefLabel xml:lang="en">Plant layout</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12750-4">
    <skos:prefLabel xml:lang="de">Fabrikplanung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18153-6">
    <skos:scopeNote xml:lang="de">Dienstleistungen des produzierenden Gewerbes</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id358">
    <rdfs:seeAlso rdf:resource="descriptor/18127-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17712-1">
    <skos:altLabel xml:lang="de">Republik Mali</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16724-6">
    <skos:broader rdf:resource="thsys/73394"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16724-6">
    <skos:broader rdf:resource="thsys/70011"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16724-6">
    <skos:broader rdf:resource="descriptor/15685-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18347-0">
    <skos:prefLabel xml:lang="en">Energy balance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18347-0">
    <skos:prefLabel xml:lang="de">Energiebilanz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id343">
    <rdfs:seeAlso rdf:resource="descriptor/11032-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10378-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70994">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15695-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11701-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16314-4">
    <skos:broader rdf:resource="thsys/73382"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16314-4">
    <skos:broader rdf:resource="thsys/70870"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16314-4">
    <skos:broader rdf:resource="descriptor/16305-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18628-6">
    <skos:narrower rdf:resource="descriptor/11827-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18628-6">
    <skos:narrower rdf:resource="descriptor/11826-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10606-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71092">
    <skos:prefLabel xml:lang="en">V.07.04.01  European integration</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71092">
    <skos:prefLabel xml:lang="de">V.07.04.01  Europäische Integration</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18410-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18972-5">
    <skos:related rdf:resource="descriptor/18180-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18972-5">
    <skos:related rdf:resource="descriptor/18907-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18972-5">
    <skos:related rdf:resource="descriptor/15938-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11129-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10845-3">
    <skos:altLabel xml:lang="de">Schleichhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18479-4">
    <skos:broader rdf:resource="thsys/71005"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18479-4">
    <skos:broader rdf:resource="descriptor/18120-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18479-4">
    <skos:broader rdf:resource="descriptor/11821-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15924-3">
    <skos:narrower rdf:resource="descriptor/11275-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15011-6">
    <skos:broader rdf:resource="thsys/70161"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15011-6">
    <skos:broader rdf:resource="descriptor/14047-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15011-6">
    <skos:broader rdf:resource="descriptor/15009-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10091-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12407-3">
    <skos:altLabel xml:lang="de">Anschaffungswert</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10598-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19204-3">
    <skos:related rdf:resource="descriptor/16687-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19204-3">
    <skos:related rdf:resource="descriptor/11160-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19204-3">
    <skos:related rdf:resource="descriptor/19596-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19204-3">
    <skos:related rdf:resource="descriptor/10525-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19204-3">
    <skos:related rdf:resource="descriptor/19203-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19204-3">
    <skos:related rdf:resource="descriptor/15633-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11432-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12195-6">
    <skos:related rdf:resource="descriptor/12070-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12195-6">
    <skos:related rdf:resource="descriptor/11696-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12195-6">
    <skos:related rdf:resource="descriptor/12194-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12195-6">
    <skos:related rdf:resource="descriptor/16050-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18210-6">
    <skos:prefLabel xml:lang="en">Women's magazine</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18210-6">
    <skos:prefLabel xml:lang="de">Frauenzeitschrift</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18125-4">
    <skos:prefLabel xml:lang="en">Economic theory of union behaviour</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18125-4">
    <skos:prefLabel xml:lang="de">Ökonomische Theorie der Gewerkschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13069-6">
    <skos:narrower rdf:resource="descriptor/13046-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11778-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14346-1">
    <skos:broader rdf:resource="thsys/71004"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14346-1">
    <skos:broader rdf:resource="thsys/70958"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10809-0">
    <skos:prefLabel xml:lang="en">International money market</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10809-0">
    <skos:prefLabel xml:lang="de">Internationaler Geldmarkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14200-1">
    <skos:narrower rdf:resource="descriptor/14194-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10606-0">
    <skos:related rdf:resource="descriptor/10621-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10606-0">
    <skos:related rdf:resource="descriptor/19329-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10606-0">
    <skos:related rdf:resource="descriptor/18790-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10606-0">
    <skos:related rdf:resource="descriptor/10880-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10606-0">
    <skos:related rdf:resource="descriptor/10682-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10606-0">
    <skos:related rdf:resource="descriptor/10827-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10606-0">
    <skos:related rdf:resource="descriptor/10852-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18191-5">
    <skos:broader rdf:resource="thsys/73332"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18191-5">
    <skos:broader rdf:resource="thsys/71010"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18191-5">
    <skos:broader rdf:resource="descriptor/10593-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10625-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17748-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15872-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10830-2">
    <skos:prefLabel xml:lang="en">Tax evasion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10830-2">
    <skos:prefLabel xml:lang="de">Steuerflucht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13960-2">
    <skos:scopeNote xml:lang="de">Nur für Berufs- und Ausbildungsfragen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70944">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.11</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13832-4">
    <skos:broader rdf:resource="thsys/70924"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13832-4">
    <skos:broader rdf:resource="thsys/70888"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13832-4">
    <skos:broader rdf:resource="descriptor/13829-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12982-4">
    <skos:prefLabel xml:lang="en">Iron-ore mining</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12982-4">
    <skos:prefLabel xml:lang="de">Eisenerzbergbau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17688-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18357-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19064-0">
    <skos:broader rdf:resource="thsys/70940"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19064-0">
    <skos:broader rdf:resource="descriptor/24787-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11306-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14046-6">
    <skos:narrower rdf:resource="descriptor/19613-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16111-4">
    <skos:broader rdf:resource="thsys/73337"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16111-4">
    <skos:broader rdf:resource="thsys/71073"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16111-4">
    <skos:broader rdf:resource="descriptor/11640-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16111-4">
    <skos:broader rdf:resource="descriptor/19847-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12031-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19771-3">
    <skos:related rdf:resource="descriptor/15477-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19771-3">
    <skos:related rdf:resource="descriptor/15932-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13608-2">
    <skos:related rdf:resource="descriptor/12689-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18586-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19667-5">
    <skos:broader rdf:resource="thsys/70212"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19667-5">
    <skos:broader rdf:resource="thsys/70041"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15768-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13682-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18440-3">
    <skos:broader rdf:resource="thsys/70986"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18440-3">
    <skos:broader rdf:resource="descriptor/13160-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16719-6">
    <skos:narrower rdf:resource="descriptor/16732-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16719-6">
    <skos:narrower rdf:resource="descriptor/19734-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16719-6">
    <skos:narrower rdf:resource="descriptor/16723-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16719-6">
    <skos:narrower rdf:resource="descriptor/19731-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16719-6">
    <skos:narrower rdf:resource="descriptor/15969-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16719-6">
    <skos:narrower rdf:resource="descriptor/19750-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16719-6">
    <skos:narrower rdf:resource="descriptor/16717-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16719-6">
    <skos:narrower rdf:resource="descriptor/19738-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16719-6">
    <skos:narrower rdf:resource="descriptor/16713-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16719-6">
    <skos:narrower rdf:resource="descriptor/16709-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16719-6">
    <skos:narrower rdf:resource="descriptor/16708-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16719-6">
    <skos:narrower rdf:resource="descriptor/16704-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16719-6">
    <skos:narrower rdf:resource="descriptor/19747-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19734-2">
    <skos:prefLabel xml:lang="en">Economic philosophy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19734-2">
    <skos:prefLabel xml:lang="de">Wirtschaftsphilosophie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15605-5">
    <skos:broader rdf:resource="thsys/73342"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15605-5">
    <skos:broader rdf:resource="thsys/71037"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15605-5">
    <skos:broader rdf:resource="thsys/70383"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15605-5">
    <skos:broader rdf:resource="descriptor/12613-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19393-2">
    <zbwext:useInsteadNote rdf:nodeID="id243"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13391-1">
    <skos:related rdf:resource="descriptor/18791-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13391-1">
    <skos:related rdf:resource="descriptor/12241-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13391-1">
    <skos:related rdf:resource="descriptor/12214-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13109-6">
    <skos:altLabel xml:lang="de">Zimmererhandwerk</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13109-6">
    <skos:altLabel xml:lang="de">Zimmerei</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13109-6">
    <skos:altLabel xml:lang="de">Tischlerhandwerk</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13109-6">
    <skos:altLabel xml:lang="de">Polstererhandwerk</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13109-6">
    <skos:altLabel xml:lang="de">Möbeltischlerei</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13109-6">
    <skos:altLabel xml:lang="de">Dachdeckerhandwerk</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13109-6">
    <skos:altLabel xml:lang="de">Bautischlerhandwerk</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13109-6">
    <skos:altLabel xml:lang="de">Abdichtungsgewerbe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12303-1">
    <skos:altLabel xml:lang="de">Stille Sanierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12303-1">
    <skos:altLabel xml:lang="de">Sanierung des Betriebs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12303-1">
    <skos:altLabel xml:lang="de">Offene Sanierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12303-1">
    <skos:altLabel xml:lang="de">Kapitalherabsetzung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12303-1">
    <skos:altLabel xml:lang="de">Finanzielle Sanierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19727-6">
    <skos:broader rdf:resource="thsys/73362"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19727-6">
    <skos:broader rdf:resource="thsys/70239"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19727-6">
    <skos:broader rdf:resource="descriptor/15768-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19727-6">
    <skos:broader rdf:resource="descriptor/18772-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70186">
    <rdfs:label xml:lang="en">Annual accounts</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70186">
    <rdfs:label xml:lang="de">Jahresabschluss</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70924">
    <skos:prefLabel xml:lang="en">W.12.07  Tourism</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70924">
    <skos:prefLabel xml:lang="de">W.12.07  Tourismus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10293-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13402-1">
    <skos:broader rdf:resource="thsys/70853"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13402-1">
    <skos:broader rdf:resource="thsys/70087"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13402-1">
    <skos:broader rdf:resource="descriptor/12194-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13402-1">
    <skos:broader rdf:resource="descriptor/10984-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18019-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71080">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13516-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16786-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12990-5">
    <skos:broader rdf:resource="thsys/71009"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12990-5">
    <skos:broader rdf:resource="descriptor/12980-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17677-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10677-5">
    <skos:related rdf:resource="descriptor/10791-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10677-5">
    <skos:related rdf:resource="descriptor/10675-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10677-5">
    <skos:related rdf:resource="descriptor/10672-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10677-5">
    <skos:related rdf:resource="descriptor/19313-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12296-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12137-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14379-0">
    <skos:altLabel xml:lang="de">Stranggussmaschine</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14379-0">
    <skos:altLabel xml:lang="de">Stranggußmaschine</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14379-0">
    <skos:altLabel xml:lang="de">Spritzgussmaschine</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14379-0">
    <skos:altLabel xml:lang="de">Spritzgußmaschine</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14379-0">
    <skos:altLabel xml:lang="de">Kautschukmaschine</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14379-0">
    <skos:altLabel xml:lang="de">Gummimaschine</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12897-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11572-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14961-2">
    <skos:altLabel xml:lang="de">Pflanzliches Öl</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14961-2">
    <skos:altLabel xml:lang="de">Pflanzenfett</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18763-0">
    <skos:altLabel xml:lang="en">Online-host</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18763-0">
    <skos:altLabel xml:lang="de">Online-Datenbankdienst</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18763-0">
    <skos:altLabel xml:lang="de">Datenbank-Host</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18339-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14736-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19139-2">
    <skos:narrower rdf:resource="descriptor/12816-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12952-6">
    <skos:related rdf:resource="descriptor/19349-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12952-6">
    <skos:related rdf:resource="descriptor/13149-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12952-6">
    <skos:related rdf:resource="descriptor/14130-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12952-6">
    <skos:related rdf:resource="descriptor/19021-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12952-6">
    <skos:related rdf:resource="descriptor/14143-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12952-6">
    <skos:related rdf:resource="descriptor/12946-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10907-0">
    <skos:broader rdf:resource="thsys/70212"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10907-0">
    <skos:broader rdf:resource="descriptor/10904-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12610-6">
    <skos:prefLabel xml:lang="en">Occupational accident</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12610-6">
    <skos:prefLabel xml:lang="de">Arbeitsunfall</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id186">
    <rdf:value xml:lang="de">Für die Entwicklung der Landwirtschaft</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id390">
    <rdf:value xml:lang="de">für Prognosen</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10303-4">
    <skos:prefLabel xml:lang="en">Activity analysis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10303-4">
    <skos:prefLabel xml:lang="de">Aktivitätsanalyse</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13780-4">
    <skos:broader rdf:resource="thsys/71112"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13780-4">
    <skos:broader rdf:resource="thsys/70814"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13780-4">
    <skos:broader rdf:resource="thsys/70259"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13780-4">
    <skos:broader rdf:resource="descriptor/18975-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13780-4">
    <skos:broader rdf:resource="descriptor/12251-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11787-0">
    <skos:prefLabel xml:lang="en">Production structure</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11787-0">
    <skos:prefLabel xml:lang="de">Produktionsstruktur</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12974-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18810-3">
    <skos:narrower rdf:resource="descriptor/10962-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18810-3">
    <skos:narrower rdf:resource="descriptor/13818-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18810-3">
    <skos:narrower rdf:resource="descriptor/10882-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18810-3">
    <skos:narrower rdf:resource="descriptor/18979-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18810-3">
    <skos:narrower rdf:resource="descriptor/11814-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18810-3">
    <skos:narrower rdf:resource="descriptor/18728-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18810-3">
    <skos:narrower rdf:resource="descriptor/13687-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71023">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10575-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13224-6">
    <skos:prefLabel xml:lang="en">Metal trade</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13224-6">
    <skos:prefLabel xml:lang="de">Metallhandel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19315-1">
    <skos:broader rdf:resource="thsys/73390"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19315-1">
    <skos:broader rdf:resource="thsys/73337"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19315-1">
    <skos:broader rdf:resource="thsys/71098"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19315-1">
    <skos:broader rdf:resource="descriptor/16102-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19315-1">
    <skos:broader rdf:resource="descriptor/10606-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19791-4">
    <skos:altLabel xml:lang="de">Kreislaufstörung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19791-4">
    <skos:altLabel xml:lang="de">Herzinfarkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19791-4">
    <skos:altLabel xml:lang="de">Herz- und Kreislaufkrankheit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19371-5">
    <skos:broader rdf:resource="thsys/71083"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19371-5">
    <skos:broader rdf:resource="thsys/71076"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18649-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18421-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18628-6">
    <skos:altLabel xml:lang="en">Sustainable mobility</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18628-6">
    <skos:altLabel xml:lang="de">Umweltorientierte Verkehrspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18628-6">
    <skos:altLabel xml:lang="de">Umweltfreundliche Verkehrsgestaltung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18628-6">
    <skos:altLabel xml:lang="de">Ökologische Verkehrspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18001-1">
    <skos:altLabel xml:lang="en">Foreign subsidiary</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18001-1">
    <skos:altLabel xml:lang="de">Tochtergesellschaft im Ausland</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18001-1">
    <skos:altLabel xml:lang="de">Niederlassung im Ausland</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18001-1">
    <skos:altLabel xml:lang="de">Betriebsstätte im Ausland</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18001-1">
    <skos:altLabel xml:lang="de">Auslandsstandort</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16671-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18208-0">
    <skos:prefLabel xml:lang="en">Computer magazine</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18208-0">
    <skos:prefLabel xml:lang="de">Computerzeitschrift</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16205-2">
    <skos:related rdf:resource="descriptor/16212-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16205-2">
    <skos:related rdf:resource="descriptor/11986-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16205-2">
    <skos:related rdf:resource="descriptor/16209-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16205-2">
    <skos:related rdf:resource="descriptor/11852-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19844-2">
    <skos:scopeNote xml:lang="de">Enscheidung von Rechtsfragen durch Arbeitsgerichte/Schlichtungsstellen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20906-3">
    <skos:altLabel xml:lang="en">Cross-subsidisation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20906-3">
    <skos:altLabel xml:lang="en">Cross-subsidies</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20906-3">
    <skos:altLabel xml:lang="de">Quersubventionierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20906-3">
    <skos:altLabel xml:lang="de">Quersubvention</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20906-3">
    <skos:altLabel xml:lang="de">Querfinanzierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11207-2">
    <skos:related rdf:resource="descriptor/19472-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11207-2">
    <skos:related rdf:resource="descriptor/19440-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11207-2">
    <skos:related rdf:resource="descriptor/19439-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11207-2">
    <skos:related rdf:resource="descriptor/19462-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11207-2">
    <skos:related rdf:resource="descriptor/11211-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13629-1">
    <skos:prefLabel xml:lang="en">Journey to work</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13629-1">
    <skos:prefLabel xml:lang="de">Berufsverkehr</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13935-1">
    <skos:broader rdf:resource="thsys/71056"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13935-1">
    <skos:broader rdf:resource="thsys/70987"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16186-1">
    <skos:altLabel xml:lang="de">Supranationales Recht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17771-6">
    <skos:broader rdf:resource="thsys/70086"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17771-6">
    <skos:broader rdf:resource="descriptor/17791-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10717-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71039">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">B.09</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19196-4">
    <skos:related rdf:resource="descriptor/24707-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11702-6">
    <skos:narrower rdf:resource="descriptor/11701-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18717-0">
    <skos:related rdf:resource="descriptor/18834-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18717-0">
    <skos:related rdf:resource="descriptor/18013-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10678-3">
    <skos:broader rdf:resource="thsys/71099"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10678-3">
    <skos:broader rdf:resource="descriptor/10676-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14222-5">
    <skos:related rdf:resource="descriptor/12992-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19486-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id6">
    <rdfs:seeAlso rdf:resource="descriptor/18320-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12129-5">
    <skos:related rdf:resource="descriptor/12091-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12129-5">
    <skos:related rdf:resource="descriptor/11303-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17652-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18634-4">
    <skos:prefLabel xml:lang="en">Tourism tax</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18634-4">
    <skos:prefLabel xml:lang="de">Fremdenverkehrsabgabe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73375">
    <rdfs:label xml:lang="de">Politische Prozesse und politischer Wandel</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18193-1">
    <skos:broader rdf:resource="thsys/71029"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18193-1">
    <skos:broader rdf:resource="descriptor/12949-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16016-5">
    <skos:prefLabel xml:lang="en">Constitution</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16016-5">
    <skos:prefLabel xml:lang="de">Verfassung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12094-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15766-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19696-5">
    <skos:related rdf:resource="descriptor/13397-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12232-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18357-4">
    <skos:related rdf:resource="descriptor/18346-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10858-1">
    <skos:prefLabel xml:lang="en">Foreign trade risk</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10858-1">
    <skos:prefLabel xml:lang="de">Außenhandelsrisiko</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19443-6">
    <skos:altLabel xml:lang="de">Lokalisationsvorteil</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13825-1">
    <skos:altLabel xml:lang="de">VVG (Versicherungsvertragsgesetz)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13825-1">
    <skos:altLabel xml:lang="de">Versicherungsvertragsrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13825-1">
    <skos:altLabel xml:lang="de">Versicherungsvertragsgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13825-1">
    <skos:altLabel xml:lang="de">Privatversicherungsrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16087-3">
    <skos:prefLabel xml:lang="en">Tax court decision</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16087-3">
    <skos:prefLabel xml:lang="de">Steuerrechtsprechung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18694-0">
    <skos:prefLabel xml:lang="en">International lending</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18694-0">
    <skos:prefLabel xml:lang="de">Internationale Kreditvergabe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id188">
    <rdf:value xml:lang="de">Für Einkommen, das in der Landwirtschaft erzielt wird,</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18560-0">
    <skos:broader rdf:resource="thsys/70944"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18560-0">
    <skos:broader rdf:resource="descriptor/13306-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11876-1">
    <skos:broader rdf:resource="thsys/70925"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11876-1">
    <skos:broader rdf:resource="thsys/70058"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11876-1">
    <skos:broader rdf:resource="descriptor/13510-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11446-5">
    <skos:prefLabel xml:lang="en">Monetary base</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11446-5">
    <skos:prefLabel xml:lang="de">Geldbasis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11355-1">
    <skos:broader rdf:resource="thsys/70847"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11355-1">
    <skos:broader rdf:resource="descriptor/15909-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19747-0">
    <skos:broader rdf:resource="thsys/73394"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19747-0">
    <skos:broader rdf:resource="thsys/70581"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19747-0">
    <skos:broader rdf:resource="descriptor/16719-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12164-3">
    <skos:narrower rdf:resource="descriptor/12370-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13372-5">
    <skos:broader rdf:resource="thsys/70839"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13372-5">
    <skos:broader rdf:resource="thsys/70573"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13372-5">
    <skos:broader rdf:resource="descriptor/13371-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12490-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13106-5">
    <skos:prefLabel xml:lang="en">Wood-working industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13106-5">
    <skos:prefLabel xml:lang="de">Holzbearbeitende Industrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16318-3">
    <skos:broader rdf:resource="thsys/73381"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16318-3">
    <skos:broader rdf:resource="thsys/70833"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16318-3">
    <skos:broader rdf:resource="thsys/70324"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16318-3">
    <skos:broader rdf:resource="descriptor/13420-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19026-1">
    <skos:scopeNote xml:lang="de">Sparsam verwenden, z.B. für: Einführung von Steuern, einer Währung, usw.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17660-1">
    <skos:altLabel xml:lang="de">Republik Kenia</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10202-3">
    <skos:narrower rdf:resource="descriptor/19122-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10202-3">
    <skos:narrower rdf:resource="descriptor/18542-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19646-6">
    <skos:broader rdf:resource="thsys/73356"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19646-6">
    <skos:broader rdf:resource="thsys/70182"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19646-6">
    <skos:broader rdf:resource="descriptor/18859-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19646-6">
    <skos:broader rdf:resource="descriptor/15949-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14928-0">
    <skos:broader rdf:resource="thsys/70498"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14928-0">
    <skos:broader rdf:resource="descriptor/14931-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12770-5">
    <skos:broader rdf:resource="thsys/70041"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12770-5">
    <skos:broader rdf:resource="descriptor/12089-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12770-5">
    <skos:broader rdf:resource="descriptor/12766-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11692-4">
    <skos:scopeNote xml:lang="de">Steuerwirkung auf die Einkommensverteilung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13145-2">
    <skos:related rdf:resource="descriptor/14846-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13145-2">
    <skos:related rdf:resource="descriptor/19061-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13145-2">
    <skos:related rdf:resource="descriptor/12994-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15731-0">
    <skos:broader rdf:resource="thsys/70817"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15731-0">
    <skos:broader rdf:resource="descriptor/12010-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16650-2">
    <skos:broader rdf:resource="thsys/70833"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16650-2">
    <skos:broader rdf:resource="thsys/70324"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17595-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18985-3">
    <skos:prefLabel xml:lang="en">Drainage</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18985-3">
    <skos:prefLabel xml:lang="de">Entwässerung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13345-1">
    <skos:broader rdf:resource="thsys/70823"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13345-1">
    <skos:broader rdf:resource="descriptor/13301-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15879-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19857-0">
    <skos:broader rdf:resource="thsys/73379"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19857-0">
    <skos:broader rdf:resource="thsys/73361"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19857-0">
    <skos:broader rdf:resource="descriptor/19692-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10653-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14261-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13697-5">
    <skos:related rdf:resource="descriptor/13690-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12761-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70258">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18248-2">
    <skos:prefLabel xml:lang="en">Bulk carrier</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18248-2">
    <skos:prefLabel xml:lang="de">Massengutfrachter</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15251-0">
    <skos:altLabel xml:lang="en">Unbiasedness</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15251-0">
    <skos:altLabel xml:lang="de">Verzerrung (Schätzung)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15251-0">
    <skos:altLabel xml:lang="de">Unverzerrtheit (Schätzung)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15251-0">
    <skos:altLabel xml:lang="de">Systematischer Fehler</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15251-0">
    <skos:altLabel xml:lang="de">Nicht-Stichprobenfehler</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15251-0">
    <skos:altLabel xml:lang="de">Erwartungstreue</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11180-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16688-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12748-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id280">
    <rdf:value xml:lang="de">Für den Arbeitsmarkt einer bestimmten Region</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10359-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19658-6">
    <skos:prefLabel xml:lang="en">Digital television</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19658-6">
    <skos:prefLabel xml:lang="de">Digitales Fernsehen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72183">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12436-3">
    <skos:prefLabel xml:lang="en">Net asset value</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12436-3">
    <skos:prefLabel xml:lang="de">Substanzwert</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10488-1">
    <skos:narrower rdf:resource="descriptor/19474-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10488-1">
    <skos:narrower rdf:resource="descriptor/19362-6"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id174">
    <rdf:value xml:lang="de">Für Produkte zur allgemeinen Körperpflege</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16957-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11206-4">
    <skos:prefLabel xml:lang="en">Regional specialization</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11206-4">
    <skos:prefLabel xml:lang="de">Regionale Arbeitsteilung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18030-1">
    <skos:related rdf:resource="descriptor/18186-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18030-1">
    <skos:related rdf:resource="descriptor/15763-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18030-1">
    <skos:related rdf:resource="descriptor/13817-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18030-1">
    <skos:related rdf:resource="descriptor/15757-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18030-1">
    <skos:related rdf:resource="descriptor/12613-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18030-1">
    <skos:related rdf:resource="descriptor/15737-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18030-1">
    <skos:related rdf:resource="descriptor/19247-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70900">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.14.03</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18961-3">
    <skos:altLabel xml:lang="de">EU-Abfallrichtlinie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11540-6">
    <skos:narrower rdf:resource="descriptor/11616-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11540-6">
    <skos:narrower rdf:resource="descriptor/11596-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11540-6">
    <skos:narrower rdf:resource="descriptor/11613-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11540-6">
    <skos:narrower rdf:resource="descriptor/11595-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11540-6">
    <skos:narrower rdf:resource="descriptor/11539-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11540-6">
    <skos:narrower rdf:resource="descriptor/11545-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11540-6">
    <skos:narrower rdf:resource="descriptor/11554-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11540-6">
    <skos:narrower rdf:resource="descriptor/19370-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11540-6">
    <skos:narrower rdf:resource="descriptor/11537-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11540-6">
    <skos:narrower rdf:resource="descriptor/11599-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11540-6">
    <skos:narrower rdf:resource="descriptor/11607-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10070-1">
    <skos:broader rdf:resource="thsys/70188"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10070-1">
    <skos:broader rdf:resource="descriptor/10076-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12292-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17933-6">
    <skos:prefLabel xml:lang="en">Uruguay</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17933-6">
    <skos:prefLabel xml:lang="de">Uruguay</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10803-5">
    <skos:narrower rdf:resource="descriptor/18438-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10803-5">
    <skos:narrower rdf:resource="descriptor/18683-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24706-5">
    <skos:prefLabel xml:lang="en">Account card</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24706-5">
    <skos:prefLabel xml:lang="de">Kundenkarte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16484-0">
    <skos:altLabel xml:lang="en">Collective bargaining policy</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16484-0">
    <skos:altLabel xml:lang="de">Tarifsystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16484-0">
    <skos:altLabel xml:lang="de">Tarifordnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16484-0">
    <skos:altLabel xml:lang="de">Gewerkschaftliche Tarifpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13409-1">
    <skos:scopeNote xml:lang="de">Für Fragen der Verwaltung BENUTZE die entsprechenden Deskriptoren.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13409-1">
    <skos:scopeNote xml:lang="de">Gesamtstaat, Gliedstaaten, Kreise, Gemeinden, usw. Nur für mehrere Verwaltungsebenen gemeinsam zu verwenden; für die einzelnen Ebenen benutze die entsprechenden Deskriptoren.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id207">
    <rdf:value xml:lang="de">Für politische Maßnahmen bezüglich Rundfunk und Fernsehen</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12818-3">
    <skos:altLabel xml:lang="en">Merchandising</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12818-3">
    <skos:altLabel xml:lang="en">Consumer promotion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12818-3">
    <skos:altLabel xml:lang="de">Preisausschreiben</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12818-3">
    <skos:altLabel xml:lang="de">Absatzförderung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10650-4">
    <skos:broader rdf:resource="thsys/71100"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10650-4">
    <skos:broader rdf:resource="descriptor/10629-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13232-0">
    <skos:prefLabel xml:lang="en">Footwear retailer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13232-0">
    <skos:prefLabel xml:lang="de">Schuheinzelhandel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15998-2">
    <skos:scopeNote xml:lang="de">Die dem Einzelnen zustehenden, meist in der Verfassung verbürgten elementaren Rechte.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18491-0">
    <skos:altLabel xml:lang="de">Baustofforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18491-0">
    <skos:altLabel xml:lang="de">Baustoffforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70058">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19799-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70278">
    <rdfs:label xml:lang="en">Wages</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70278">
    <rdfs:label xml:lang="de">Lohn</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18798-2">
    <skos:scopeNote xml:lang="de">Marketing der Messeveranstalter.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70031">
    <skos:broader rdf:resource="thsys/p"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19815-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13421-4">
    <skos:prefLabel xml:lang="en">State of a federation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13421-4">
    <skos:prefLabel xml:lang="de">Teilstaat</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16654-1">
    <skos:altLabel xml:lang="en">Social exclusion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16654-1">
    <skos:altLabel xml:lang="de">Vereinsamung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16654-1">
    <skos:altLabel xml:lang="de">Soziale Vereinzelung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16654-1">
    <skos:altLabel xml:lang="de">Isolation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16654-1">
    <skos:altLabel xml:lang="de">Einsamkeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15677-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19503-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10438-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16663-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19789-5">
    <skos:broader rdf:resource="thsys/73346"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19789-5">
    <skos:broader rdf:resource="descriptor/16670-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19511-1">
    <skos:broader rdf:resource="thsys/70834"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19511-1">
    <skos:broader rdf:resource="thsys/70833"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19511-1">
    <skos:broader rdf:resource="thsys/70324"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19511-1">
    <skos:broader rdf:resource="descriptor/16543-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19511-1">
    <skos:broader rdf:resource="descriptor/19704-4"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id65">
    <rdfs:seeAlso rdf:resource="descriptor/18941-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19097-6">
    <skos:related rdf:resource="descriptor/19098-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19097-6">
    <skos:related rdf:resource="descriptor/12518-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13253-6">
    <skos:related rdf:resource="descriptor/18637-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13253-6">
    <skos:related rdf:resource="descriptor/13966-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13253-6">
    <skos:related rdf:resource="descriptor/18561-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11912-2">
    <skos:related rdf:resource="descriptor/11963-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11912-2">
    <skos:related rdf:resource="descriptor/11857-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11912-2">
    <skos:related rdf:resource="descriptor/11956-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18646-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10710-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16068-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70026">
    <skos:prefLabel xml:lang="en">G.04  America</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70026">
    <skos:prefLabel xml:lang="de">G.04  Amerika</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71023">
    <rdfs:label xml:lang="en">Agricultural land-use</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71023">
    <rdfs:label xml:lang="de">Landwirtschaftliche Bodennutzung</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12713-3">
    <skos:prefLabel xml:lang="en">Quality management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12713-3">
    <skos:prefLabel xml:lang="de">Qualitätsmanagement</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16956-6">
    <skos:prefLabel xml:lang="en">Aargau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16956-6">
    <skos:prefLabel xml:lang="de">Aargau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18516-3">
    <skos:prefLabel xml:lang="en">Fertilizer trade</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18516-3">
    <skos:prefLabel xml:lang="de">Düngemittelhandel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15914-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17472-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71020">
    <skos:prefLabel xml:lang="en">W.01.05  Agricultural Policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71020">
    <skos:prefLabel xml:lang="de">W.01.05  Agrarpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18072-6">
    <skos:narrower rdf:resource="descriptor/18376-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16395-6">
    <skos:broader rdf:resource="thsys/73375"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16395-6">
    <skos:broader rdf:resource="descriptor/16621-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17782-1">
    <skos:prefLabel xml:lang="en">Guatemala</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17782-1">
    <skos:prefLabel xml:lang="de">Guatemala</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14116-4">
    <skos:prefLabel xml:lang="en">Tomato</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14116-4">
    <skos:prefLabel xml:lang="de">Tomate</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10495-4">
    <skos:prefLabel xml:lang="en">Nutrition</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10495-4">
    <skos:prefLabel xml:lang="de">Ernährung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14017-6">
    <skos:broader rdf:resource="thsys/71052"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14017-6">
    <skos:broader rdf:resource="descriptor/13349-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16737-4">
    <skos:broader rdf:resource="thsys/70141"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16737-4">
    <skos:broader rdf:resource="descriptor/16739-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16737-4">
    <skos:broader rdf:resource="descriptor/16745-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18092-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19715-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73394">
    <skos:prefLabel xml:lang="en">N.01.01  Philosophy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73394">
    <skos:prefLabel xml:lang="de">N.01.01  Philosophie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70825">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71028">
    <skos:prefLabel xml:lang="en">W.01  Agriculture</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71028">
    <skos:prefLabel xml:lang="de">W.01  Landwirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14619-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18142-4">
    <skos:prefLabel xml:lang="en">Policy towards women</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18142-4">
    <skos:prefLabel xml:lang="de">Frauenpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18833-5">
    <skos:altLabel xml:lang="en">Non-profit controlling</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10958-4">
    <skos:broader rdf:resource="thsys/71071"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10958-4">
    <skos:broader rdf:resource="descriptor/10963-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70867">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70038">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16311-3">
    <skos:broader rdf:resource="thsys/73376"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16311-3">
    <skos:broader rdf:resource="descriptor/16357-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10387-0">
    <skos:broader rdf:resource="thsys/73372"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10387-0">
    <skos:broader rdf:resource="thsys/70020"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10387-0">
    <skos:broader rdf:resource="descriptor/10987-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11919-2">
    <skos:related rdf:resource="descriptor/10542-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11919-2">
    <skos:related rdf:resource="descriptor/11988-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11919-2">
    <skos:related rdf:resource="descriptor/11897-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11919-2">
    <skos:related rdf:resource="descriptor/11849-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11919-2">
    <skos:related rdf:resource="descriptor/19429-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11919-2">
    <skos:related rdf:resource="descriptor/20321-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11919-2">
    <skos:related rdf:resource="descriptor/10562-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11919-2">
    <skos:related rdf:resource="descriptor/19195-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11919-2">
    <skos:related rdf:resource="descriptor/11979-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18250-1">
    <skos:prefLabel xml:lang="en">Fuel cell</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18250-1">
    <skos:prefLabel xml:lang="de">Brennstoffzelle</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14728-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19807-1">
    <skos:scopeNote xml:lang="de">Forschungsgebiet der Künstlichen Intelligenz. Ziel ist die Entwicklung allgemein verwendbarer, effizienter Methoden, die autonom aus großen Datenmengen die bedeutsamsten und aussagekräftigsten Muster identifizieren und sie dem Anwender als interessantes Wissen präsentieren.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70184">
    <skos:prefLabel xml:lang="de">B.08.02  Steuerrechtliche Vermögensaufstellung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16969-4">
    <skos:broader rdf:resource="thsys/70003"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16969-4">
    <skos:broader rdf:resource="descriptor/16972-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13840-5">
    <skos:related rdf:resource="descriptor/18749-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13840-5">
    <skos:related rdf:resource="descriptor/18752-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13840-5">
    <skos:related rdf:resource="descriptor/19532-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70192">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19812-1">
    <skos:broader rdf:resource="thsys/73336"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19812-1">
    <skos:broader rdf:resource="thsys/73333"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19812-1">
    <skos:broader rdf:resource="descriptor/16186-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19812-1">
    <skos:broader rdf:resource="descriptor/11185-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11432-2">
    <skos:related rdf:resource="descriptor/11519-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11432-2">
    <skos:related rdf:resource="descriptor/16639-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11432-2">
    <skos:related rdf:resource="descriptor/15069-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11432-2">
    <skos:related rdf:resource="descriptor/10659-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18543-0">
    <skos:altLabel xml:lang="de">Globale Werbung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18543-0">
    <skos:altLabel xml:lang="de">Exportwerbung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18543-0">
    <skos:altLabel xml:lang="de">Auslandswerbung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19128-0">
    <skos:related rdf:resource="descriptor/20306-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19128-0">
    <skos:related rdf:resource="descriptor/16034-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19128-0">
    <skos:related rdf:resource="descriptor/12786-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17665-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15949-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11839-0">
    <skos:broader rdf:resource="thsys/73392"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11839-0">
    <skos:broader rdf:resource="thsys/70884"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11839-0">
    <skos:broader rdf:resource="thsys/70814"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11839-0">
    <skos:broader rdf:resource="descriptor/16485-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10735-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11635-2">
    <skos:prefLabel xml:lang="en">Tax revenues</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11635-2">
    <skos:prefLabel xml:lang="de">Steueraufkommen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16092-3">
    <skos:related rdf:resource="descriptor/12558-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16092-3">
    <skos:related rdf:resource="descriptor/12849-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16092-3">
    <skos:related rdf:resource="descriptor/16104-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16092-3">
    <skos:related rdf:resource="descriptor/19413-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16092-3">
    <skos:related rdf:resource="descriptor/11650-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18767-6">
    <skos:related rdf:resource="descriptor/19155-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18767-6">
    <skos:related rdf:resource="descriptor/18940-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18767-6">
    <skos:related rdf:resource="descriptor/16782-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18767-6">
    <skos:related rdf:resource="descriptor/20341-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18767-6">
    <skos:related rdf:resource="descriptor/20404-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18767-6">
    <skos:related rdf:resource="descriptor/12164-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11051-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id187">
    <rdf:value xml:lang="de">Für die ländliche Kleinindustrie</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10433-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70004">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13726-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13135-5">
    <skos:prefLabel xml:lang="en">Leather production</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13135-5">
    <skos:prefLabel xml:lang="de">Ledererzeugende Industrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10576-4">
    <skos:prefLabel xml:lang="en">Agricultural extension</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10576-4">
    <skos:prefLabel xml:lang="de">Landwirtschaftliche Beratung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10574-1">
    <skos:broader rdf:resource="thsys/73387"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10574-1">
    <skos:broader rdf:resource="thsys/73366"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10574-1">
    <skos:broader rdf:resource="thsys/70109"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10574-1">
    <skos:broader rdf:resource="descriptor/11083-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73353">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15983-1">
    <skos:scopeNote xml:lang="de">Die Gesamtheit aller rechtlichen Regelungen, die öffentliche Wahlen betreffen. Das Recht zu wählen und gewählt zu werden.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14169-4">
    <skos:altLabel xml:lang="de">Versuchstier</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14169-4">
    <skos:altLabel xml:lang="de">Nutztier</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14169-4">
    <skos:altLabel xml:lang="de">Kaninchen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14169-4">
    <skos:altLabel xml:lang="de">Insekt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14169-4">
    <skos:altLabel xml:lang="de">Hund</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13072-3">
    <skos:related rdf:resource="descriptor/12925-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13072-3">
    <skos:related rdf:resource="descriptor/16757-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13072-3">
    <skos:related rdf:resource="descriptor/13074-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13072-3">
    <skos:related rdf:resource="descriptor/18523-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13072-3">
    <skos:related rdf:resource="descriptor/14751-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19751-2">
    <skos:related rdf:resource="descriptor/15429-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19751-2">
    <skos:related rdf:resource="descriptor/12091-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19751-2">
    <skos:related rdf:resource="descriptor/15415-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19751-2">
    <skos:related rdf:resource="descriptor/16579-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18351-2">
    <skos:narrower rdf:resource="descriptor/18347-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70269">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18892-3">
    <skos:prefLabel xml:lang="en">Prophylaxis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18892-3">
    <skos:prefLabel xml:lang="de">Gesundheitsvorsorge</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12210-1">
    <skos:scopeNote xml:lang="de">Untersucht den Zusammenhang zwischen Risiko und Ertrag von Kapitalanlagen auf einem vollkommenen Kapitalmarkt.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14198-4">
    <skos:prefLabel xml:lang="en">Ore</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14198-4">
    <skos:prefLabel xml:lang="de">Erz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73393">
    <skos:broader rdf:resource="thsys/70096"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70055">
    <skos:narrower rdf:resource="descriptor/11417-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70055">
    <skos:narrower rdf:resource="descriptor/10764-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70055">
    <skos:narrower rdf:resource="descriptor/11438-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70055">
    <skos:narrower rdf:resource="descriptor/11436-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70055">
    <skos:narrower rdf:resource="descriptor/18149-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70055">
    <skos:narrower rdf:resource="descriptor/11434-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70055">
    <skos:narrower rdf:resource="descriptor/19602-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70055">
    <skos:narrower rdf:resource="descriptor/19255-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70055">
    <skos:narrower rdf:resource="descriptor/11430-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70055">
    <skos:narrower rdf:resource="descriptor/11460-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70055">
    <skos:narrower rdf:resource="descriptor/11042-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70055">
    <skos:narrower rdf:resource="descriptor/11427-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70055">
    <skos:narrower rdf:resource="descriptor/11426-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70055">
    <skos:narrower rdf:resource="descriptor/11424-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70055">
    <skos:narrower rdf:resource="descriptor/11422-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70055">
    <skos:narrower rdf:resource="descriptor/11448-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70055">
    <skos:narrower rdf:resource="descriptor/11416-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70055">
    <skos:narrower rdf:resource="descriptor/11419-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70055">
    <skos:narrower rdf:resource="descriptor/11418-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70055">
    <skos:narrower rdf:resource="descriptor/11415-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70055">
    <skos:narrower rdf:resource="descriptor/11411-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70055">
    <skos:narrower rdf:resource="thsys/70305"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70055">
    <skos:narrower rdf:resource="thsys/70114"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70055">
    <skos:narrower rdf:resource="thsys/70355"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70055">
    <skos:narrower rdf:resource="thsys/70306"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70055">
    <skos:narrower rdf:resource="thsys/70187"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70055">
    <skos:narrower rdf:resource="thsys/70455"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70055">
    <skos:narrower rdf:resource="thsys/70259"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13006-2">
    <skos:related rdf:resource="descriptor/14822-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13006-2">
    <skos:related rdf:resource="descriptor/14379-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14736-2">
    <skos:altLabel xml:lang="de">Zahnpulver</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14736-2">
    <skos:altLabel xml:lang="de">Zahnpasta</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14736-2">
    <skos:altLabel xml:lang="de">Mundwasser</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14736-2">
    <skos:altLabel xml:lang="de">Gebissreiniger</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14736-2">
    <skos:altLabel xml:lang="de">Gebißreiniger</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16741-6">
    <skos:broader rdf:resource="thsys/70141"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16741-6">
    <skos:broader rdf:resource="descriptor/16745-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14487-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19797-6">
    <skos:altLabel xml:lang="de">Nano-Werkstoff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19797-6">
    <skos:altLabel xml:lang="de">Nanotechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19797-6">
    <skos:altLabel xml:lang="de">Nano-Material</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19488-5">
    <skos:altLabel xml:lang="en">Open access resources</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19488-5">
    <skos:altLabel xml:lang="de">Umweltgemeingüter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12644-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10410-3">
    <skos:prefLabel xml:lang="en">Balance of payments equilibrium</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10410-3">
    <skos:prefLabel xml:lang="de">Zahlungsbilanzgleichgewicht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17015-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12495-1">
    <skos:prefLabel xml:lang="en">Imputed costs</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12495-1">
    <skos:prefLabel xml:lang="de">Kalkulatorische Kosten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13670-4">
    <skos:prefLabel xml:lang="en">Newspaper</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13670-4">
    <skos:prefLabel xml:lang="de">Zeitung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10813-2">
    <skos:broader rdf:resource="thsys/71089"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10813-2">
    <skos:broader rdf:resource="descriptor/10811-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16805-6">
    <skos:broader rdf:resource="thsys/70198"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16805-6">
    <skos:broader rdf:resource="descriptor/16808-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12966-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18663-4">
    <skos:prefLabel xml:lang="en">Telecommunications infrastructure</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18663-4">
    <skos:prefLabel xml:lang="de">Telekommunikationsinfrastruktur</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15751-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11288-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18172-2">
    <skos:scopeNote xml:lang="de">Konzeptionelles Schema einer Datenbank.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19718-0">
    <skos:related rdf:resource="descriptor/17642-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24781-5">
    <skos:related rdf:resource="descriptor/19119-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24781-5">
    <skos:related rdf:resource="descriptor/19133-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19376-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14607-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12175-5">
    <skos:related rdf:resource="descriptor/12201-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12175-5">
    <skos:related rdf:resource="descriptor/19154-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12175-5">
    <skos:related rdf:resource="descriptor/19156-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12175-5">
    <skos:related rdf:resource="descriptor/12174-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10851-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14946-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10011-3">
    <skos:broader rdf:resource="thsys/70578"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10011-3">
    <skos:broader rdf:resource="thsys/70175"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19572-2">
    <skos:prefLabel xml:lang="en">Random walk</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19572-2">
    <skos:prefLabel xml:lang="de">Random Walk</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71008">
    <skos:prefLabel xml:lang="en">W.05.02  Nonmetallic minerals industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71008">
    <skos:prefLabel xml:lang="de">W.05.02  Industrie der Steine und Erden</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14292-5">
    <skos:broader rdf:resource="thsys/70463"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14292-5">
    <skos:broader rdf:resource="thsys/70134"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14292-5">
    <skos:broader rdf:resource="descriptor/14302-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12282-4">
    <skos:broader rdf:resource="thsys/71046"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12282-4">
    <skos:broader rdf:resource="thsys/71043"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12282-4">
    <skos:broader rdf:resource="thsys/70036"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12282-4">
    <skos:broader rdf:resource="descriptor/12366-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70178">
    <skos:broader rdf:resource="thsys/70180"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11788-5">
    <skos:broader rdf:resource="thsys/71110"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11788-5">
    <skos:broader rdf:resource="descriptor/11784-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10472-2">
    <skos:prefLabel xml:lang="en">Technology transfer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10472-2">
    <skos:prefLabel xml:lang="de">Technologietransfer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12311-2">
    <skos:narrower rdf:resource="descriptor/11545-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11272-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16684-6">
    <skos:altLabel xml:lang="de">Soldatentum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16684-6">
    <skos:altLabel xml:lang="de">Militarismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16684-6">
    <skos:altLabel xml:lang="de">Militärische Sozialisation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14744-3">
    <skos:broader rdf:resource="thsys/70014"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14744-3">
    <skos:broader rdf:resource="descriptor/14743-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70867">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.22.01.04</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19029-2">
    <skos:prefLabel xml:lang="en">Calculation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19029-2">
    <skos:prefLabel xml:lang="de">Berechnung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18318-0">
    <skos:narrower rdf:resource="descriptor/19191-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11485-2">
    <skos:prefLabel xml:lang="en">Monetary indicator</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11485-2">
    <skos:prefLabel xml:lang="de">Monetärer Indikator</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10398-2">
    <skos:broader rdf:resource="thsys/70038"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10398-2">
    <skos:broader rdf:resource="descriptor/10987-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18419-1">
    <skos:scopeNote xml:lang="de">Planung und Durchführung von Raumfahrtprogrammen zur Unterstützung der einschlägigen technischen Entwicklung und der Luftfahrtindustrie.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70471">
    <skos:broader rdf:resource="thsys/70030"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11454-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11092-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16849-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70182">
    <skos:broader rdf:resource="thsys/70142"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16387-5">
    <skos:related rdf:resource="descriptor/11060-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16387-5">
    <skos:related rdf:resource="descriptor/16651-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16387-5">
    <skos:related rdf:resource="descriptor/11047-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16387-5">
    <skos:related rdf:resource="descriptor/11087-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16387-5">
    <skos:related rdf:resource="descriptor/16629-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16387-5">
    <skos:related rdf:resource="descriptor/11070-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12898-0">
    <skos:broader rdf:resource="thsys/71041"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12898-0">
    <skos:broader rdf:resource="descriptor/12903-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13095-5">
    <skos:broader rdf:resource="thsys/71004"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13095-5">
    <skos:broader rdf:resource="descriptor/12993-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70937">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14762-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11393-0">
    <skos:broader rdf:resource="thsys/70843"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11393-0">
    <skos:broader rdf:resource="descriptor/11360-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15163-4">
    <skos:altLabel xml:lang="en">Bayesian approach</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15163-4">
    <skos:altLabel xml:lang="en">Bayesian analysis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15163-4">
    <skos:altLabel xml:lang="de">Bayessches Modell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15163-4">
    <skos:altLabel xml:lang="de">Bayes-Regel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10931-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13372-5">
    <skos:prefLabel xml:lang="en">Hospital</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13372-5">
    <skos:prefLabel xml:lang="de">Krankenhaus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12423-5">
    <skos:prefLabel xml:lang="en">Goodwill</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12423-5">
    <skos:prefLabel xml:lang="de">Geschäftswert</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12575-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19286-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12337-5">
    <skos:narrower rdf:resource="descriptor/12384-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12337-5">
    <skos:narrower rdf:resource="descriptor/19158-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19244-5">
    <skos:prefLabel xml:lang="en">New growth theory</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19244-5">
    <skos:prefLabel xml:lang="de">Neue Wachstumstheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18069-2">
    <skos:broader rdf:resource="thsys/73352"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18069-2">
    <skos:broader rdf:resource="thsys/70238"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18069-2">
    <skos:broader rdf:resource="descriptor/15880-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11127-0">
    <skos:related rdf:resource="descriptor/10994-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11127-0">
    <skos:related rdf:resource="descriptor/11068-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11127-0">
    <skos:related rdf:resource="descriptor/12697-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11127-0">
    <skos:related rdf:resource="descriptor/19284-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11127-0">
    <skos:related rdf:resource="descriptor/19703-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11127-0">
    <skos:related rdf:resource="descriptor/11105-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11127-0">
    <skos:related rdf:resource="descriptor/19837-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11127-0">
    <skos:related rdf:resource="descriptor/11049-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11127-0">
    <skos:related rdf:resource="descriptor/16295-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11127-0">
    <skos:related rdf:resource="descriptor/15720-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11127-0">
    <skos:related rdf:resource="descriptor/16292-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11127-0">
    <skos:related rdf:resource="descriptor/16286-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11127-0">
    <skos:related rdf:resource="descriptor/16283-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11127-0">
    <skos:related rdf:resource="descriptor/11066-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11127-0">
    <skos:related rdf:resource="descriptor/10976-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11127-0">
    <skos:related rdf:resource="descriptor/16602-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18408-6">
    <skos:broader rdf:resource="thsys/71002"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18408-6">
    <skos:broader rdf:resource="descriptor/12914-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13629-1">
    <skos:broader rdf:resource="thsys/70939"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13629-1">
    <skos:broader rdf:resource="descriptor/13632-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16078-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19374-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11320-6">
    <skos:scopeNote xml:lang="de">Vergütung von Angestellten.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11806-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19585-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18747-5">
    <skos:prefLabel xml:lang="en">Real estate services</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18747-5">
    <skos:prefLabel xml:lang="de">Immobilienwirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19514-2">
    <skos:related rdf:resource="descriptor/16547-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13723-2">
    <skos:prefLabel xml:lang="en">Financial market</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13723-2">
    <skos:prefLabel xml:lang="de">Finanzmarkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70227">
    <skos:broader rdf:resource="thsys/70082"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14218-3">
    <skos:narrower rdf:resource="descriptor/14215-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14218-3">
    <skos:narrower rdf:resource="descriptor/14214-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14218-3">
    <skos:narrower rdf:resource="descriptor/14212-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19182-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70970">
    <skos:narrower rdf:resource="descriptor/13562-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70970">
    <skos:narrower rdf:resource="descriptor/13561-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70970">
    <skos:narrower rdf:resource="descriptor/18750-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70970">
    <skos:narrower rdf:resource="descriptor/18508-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70970">
    <skos:narrower rdf:resource="descriptor/12905-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70970">
    <skos:narrower rdf:resource="descriptor/18495-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70970">
    <skos:narrower rdf:resource="descriptor/15101-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70970">
    <skos:narrower rdf:resource="descriptor/16176-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70970">
    <skos:narrower rdf:resource="descriptor/13853-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70970">
    <skos:narrower rdf:resource="descriptor/18507-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70970">
    <skos:narrower rdf:resource="descriptor/18493-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70970">
    <skos:narrower rdf:resource="descriptor/13852-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70970">
    <skos:narrower rdf:resource="descriptor/13851-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70970">
    <skos:narrower rdf:resource="descriptor/18506-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70970">
    <skos:narrower rdf:resource="descriptor/18505-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70970">
    <skos:narrower rdf:resource="descriptor/18787-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70970">
    <skos:narrower rdf:resource="descriptor/18491-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70970">
    <skos:narrower rdf:resource="descriptor/18504-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70970">
    <skos:narrower rdf:resource="descriptor/13953-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70970">
    <skos:narrower rdf:resource="descriptor/18492-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70970">
    <skos:narrower rdf:resource="thsys/70968"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70970">
    <skos:narrower rdf:resource="thsys/70969"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70970">
    <skos:narrower rdf:resource="thsys/70967"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70987">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11219-2">
    <skos:prefLabel xml:lang="en">Labour market theory</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11219-2">
    <skos:prefLabel xml:lang="de">Arbeitsmarkttheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18384-1">
    <skos:altLabel xml:lang="de">Montanforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73337">
    <skos:broader rdf:resource="thsys/70154"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17654-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17583-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12081-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19541-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10902-3">
    <skos:broader rdf:resource="thsys/70101"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10902-3">
    <skos:broader rdf:resource="thsys/70041"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11374-4">
    <skos:prefLabel xml:lang="en">Teaching personnel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11374-4">
    <skos:prefLabel xml:lang="de">Lehrkräfte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17618-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73349">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.10.03</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18171-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18793-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15475-6">
    <skos:altLabel xml:lang="en">Sequential decision making</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15475-6">
    <skos:altLabel xml:lang="en">Multi-stage decision model</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15475-6">
    <skos:altLabel xml:lang="de">Mehrperiodige Entscheidung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13627-5">
    <skos:prefLabel xml:lang="en">Dry bulk shipping</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13627-5">
    <skos:prefLabel xml:lang="de">Massengutschifffahrt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70951">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.10.04.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15734-1">
    <skos:prefLabel xml:lang="en">Water protection</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15734-1">
    <skos:prefLabel xml:lang="de">Gewässerschutz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19645-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13050-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71098">
    <skos:prefLabel xml:lang="en">V.07.02  Foreign economic policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71098">
    <skos:prefLabel xml:lang="de">V.07.02  Außenwirtschaftspolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16202-1">
    <skos:related rdf:resource="descriptor/16081-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16202-1">
    <skos:related rdf:resource="descriptor/16201-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16202-1">
    <skos:related rdf:resource="descriptor/18130-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16202-1">
    <skos:related rdf:resource="descriptor/16112-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19005-2">
    <skos:altLabel xml:lang="en">Job length</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19005-2">
    <skos:altLabel xml:lang="de">Senioritätsprinzip im Betrieb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19005-2">
    <skos:altLabel xml:lang="de">Beschäftigungsdauer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18007-3">
    <skos:related rdf:resource="descriptor/12778-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18007-3">
    <skos:related rdf:resource="descriptor/19140-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18007-3">
    <skos:related rdf:resource="descriptor/19139-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18007-3">
    <skos:related rdf:resource="descriptor/12766-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18007-3">
    <skos:related rdf:resource="descriptor/12777-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19194-1">
    <skos:prefLabel xml:lang="en">Crop loss</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19194-1">
    <skos:prefLabel xml:lang="de">Ernteschaden</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13014-3">
    <skos:related rdf:resource="descriptor/14769-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13935-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21408-0">
    <skos:broader rdf:resource="thsys/70384"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21408-0">
    <skos:broader rdf:resource="descriptor/12818-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12876-3">
    <skos:broader rdf:resource="thsys/70046"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12876-3">
    <skos:broader rdf:resource="descriptor/12088-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11392-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16777-6">
    <skos:altLabel xml:lang="en">Universal decimal classification</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16777-6">
    <skos:altLabel xml:lang="en">UDC (Universal Decimal Classification)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16777-6">
    <skos:altLabel xml:lang="de">Thesaurus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16777-6">
    <skos:altLabel xml:lang="de">SWD (Schlagwortnormdatei)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16777-6">
    <skos:altLabel xml:lang="de">Schlagwortsystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16777-6">
    <skos:altLabel xml:lang="de">Schlagwortnormdatei</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16777-6">
    <skos:altLabel xml:lang="de">RSWK</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16777-6">
    <skos:altLabel xml:lang="de">Retrievalsprache</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16777-6">
    <skos:altLabel xml:lang="de">Regeln für den Schlagwortkatalog</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16777-6">
    <skos:altLabel xml:lang="de">Klassifikatorische Sacherschließung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16777-6">
    <skos:altLabel xml:lang="de">Indexierungssprache</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16777-6">
    <skos:altLabel xml:lang="de">DK (Dezimalklassifikation)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16777-6">
    <skos:altLabel xml:lang="de">Dezimalklassifikation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18244-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73369">
    <skos:prefLabel xml:lang="de">N.04.03.05  Kultur-, Bildungs- und Wissenschaftspolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17968-1">
    <skos:related rdf:resource="descriptor/18019-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12932-5">
    <skos:prefLabel xml:lang="en">Housing construction</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12932-5">
    <skos:prefLabel xml:lang="de">Wohnungsbau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18144-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12742-3">
    <skos:related rdf:resource="descriptor/12480-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12742-3">
    <skos:related rdf:resource="descriptor/10011-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12742-3">
    <skos:related rdf:resource="descriptor/12744-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13841-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20826-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70305">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12032-6">
    <skos:broader rdf:resource="thsys/71066"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12032-6">
    <skos:broader rdf:resource="thsys/71023"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12032-6">
    <skos:broader rdf:resource="descriptor/11859-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11324-5">
    <skos:broader rdf:resource="thsys/70278"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11324-5">
    <skos:broader rdf:resource="descriptor/11336-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19106-3">
    <skos:related rdf:resource="descriptor/12393-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13480-2">
    <skos:altLabel xml:lang="de">Discounthandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13480-2">
    <skos:altLabel xml:lang="de">Discountgeschäft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15772-0">
    <skos:prefLabel xml:lang="en">Environmental conference</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15772-0">
    <skos:prefLabel xml:lang="de">Umweltschutzkonferenz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11595-2">
    <skos:related rdf:resource="descriptor/11707-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11595-2">
    <skos:related rdf:resource="descriptor/11596-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11595-2">
    <skos:related rdf:resource="descriptor/11728-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16663-0">
    <skos:prefLabel xml:lang="en">Fringe group</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16663-0">
    <skos:prefLabel xml:lang="de">Randgruppe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19536-6">
    <skos:prefLabel xml:lang="en">Discounting</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19536-6">
    <skos:prefLabel xml:lang="de">Abzinsung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14270-1">
    <skos:prefLabel xml:lang="en">Steel pipe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14270-1">
    <skos:prefLabel xml:lang="de">Stahlrohr</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11539-5">
    <skos:related rdf:resource="descriptor/11541-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11539-5">
    <skos:related rdf:resource="descriptor/11589-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11539-5">
    <skos:related rdf:resource="descriptor/16100-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11539-5">
    <skos:related rdf:resource="descriptor/19380-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15740-6">
    <skos:related rdf:resource="descriptor/15739-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11853-6">
    <skos:altLabel xml:lang="de">Grenzraum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18077-3">
    <skos:altLabel xml:lang="de">Palmfett</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18453-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19509-2">
    <skos:altLabel xml:lang="en">Social standards</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19509-2">
    <skos:altLabel xml:lang="en">Minimum labour standards</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19509-2">
    <skos:altLabel xml:lang="de">Internationale Arbeitsstandards</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12050-4">
    <skos:related rdf:resource="descriptor/20018-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12050-4">
    <skos:related rdf:resource="descriptor/12283-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18981-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10432-0">
    <skos:altLabel xml:lang="en">Von-Neumann-Turnpike</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10432-0">
    <skos:altLabel xml:lang="en">Von Neumann Turnpike</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10432-0">
    <skos:altLabel xml:lang="en">Twisted turnpike</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10432-0">
    <skos:altLabel xml:lang="en">Turnpike theory</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10432-0">
    <skos:altLabel xml:lang="en">Catenary-turnpike</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id160">
    <rdfs:seeAlso rdf:resource="descriptor/14262-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19092-2">
    <skos:altLabel xml:lang="de">Bewertungsfreiheit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18493-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10683-3">
    <skos:scopeNote xml:lang="de">Vgl. auch: Außenwirtschaftsrecht.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11668-1">
    <skos:prefLabel xml:lang="en">State tax</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11668-1">
    <skos:prefLabel xml:lang="de">Ländersteuer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id293">
    <rdf:value xml:lang="de">Für Maßnahmen zur Selbsthilfeförderung in Entwicklungsländern</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16531-3">
    <skos:altLabel xml:lang="en">Disability (Work)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16531-3">
    <skos:altLabel xml:lang="de">Minderung der Erwerbsfähigkeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16531-3">
    <skos:altLabel xml:lang="de">Krankenstand</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16531-3">
    <skos:altLabel xml:lang="de">Invalidität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16531-3">
    <skos:altLabel xml:lang="de">Frühinvalidität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16531-3">
    <skos:altLabel xml:lang="de">Erwerbsminderung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16531-3">
    <skos:altLabel xml:lang="de">Berufsunfähigkeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16531-3">
    <skos:altLabel xml:lang="de">Arbeitsunfähigkeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15298-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13080-4">
    <skos:related rdf:resource="descriptor/14537-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13080-4">
    <skos:related rdf:resource="descriptor/13081-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14227-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19494-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16628-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14665-6">
    <skos:broader rdf:resource="thsys/70556"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14665-6">
    <skos:broader rdf:resource="descriptor/14656-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18019-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11270-2">
    <skos:altLabel xml:lang="de">Beschäftigungslage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11270-2">
    <skos:altLabel xml:lang="de">Beschäftigungsentwicklung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13960-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19466-1">
    <skos:related rdf:resource="descriptor/13645-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19466-1">
    <skos:related rdf:resource="descriptor/13644-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19466-1">
    <skos:related rdf:resource="descriptor/13638-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19466-1">
    <skos:related rdf:resource="descriptor/16679-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11540-6">
    <skos:related rdf:resource="descriptor/11722-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11540-6">
    <skos:related rdf:resource="descriptor/11526-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11540-6">
    <skos:related rdf:resource="descriptor/11535-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11540-6">
    <skos:related rdf:resource="descriptor/11534-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11540-6">
    <skos:related rdf:resource="descriptor/11728-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11540-6">
    <skos:related rdf:resource="descriptor/11533-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11540-6">
    <skos:related rdf:resource="descriptor/12215-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10616-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11534-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14299-5">
    <skos:broader rdf:resource="thsys/70134"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14299-5">
    <skos:broader rdf:resource="descriptor/14274-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14299-5">
    <skos:broader rdf:resource="descriptor/14302-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18305-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19131-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10752-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11469-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12480-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71108">
    <rdfs:label xml:lang="en">Money market</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71108">
    <rdfs:label xml:lang="de">Geldmarkt</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19853-1">
    <skos:prefLabel xml:lang="en">Judicial assistance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19853-1">
    <skos:prefLabel xml:lang="de">Internationale Rechtshilfe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73391">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11845-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15754-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13680-1">
    <skos:altLabel xml:lang="en">Integrated services digital network</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13680-1">
    <skos:altLabel xml:lang="de">Integriertes Text- und Datennetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13680-1">
    <skos:altLabel xml:lang="de">Integriertes Breitbandkommunikationsnetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13680-1">
    <skos:altLabel xml:lang="de">IDN (Integriertes Text- und Datennetz)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13680-1">
    <skos:altLabel xml:lang="de">Euro-ISDN</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13680-1">
    <skos:altLabel xml:lang="de">Dienstintegriertes digitales Fernmeldenetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13680-1">
    <skos:altLabel xml:lang="de">Breitband-ISDN</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13680-1">
    <skos:altLabel xml:lang="de">B-ISDN</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20136-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14623-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13516-0">
    <skos:broader rdf:resource="thsys/70932"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13516-0">
    <skos:broader rdf:resource="descriptor/12013-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14989-1">
    <skos:related rdf:resource="descriptor/14731-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14989-1">
    <skos:related rdf:resource="descriptor/14169-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14989-1">
    <skos:related rdf:resource="descriptor/19292-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14989-1">
    <skos:related rdf:resource="descriptor/13156-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18242-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12215-5">
    <skos:related rdf:resource="descriptor/12205-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12215-5">
    <skos:related rdf:resource="descriptor/12212-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12215-5">
    <skos:related rdf:resource="descriptor/11540-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12215-5">
    <skos:related rdf:resource="descriptor/12206-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12215-5">
    <skos:related rdf:resource="descriptor/12209-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12215-5">
    <skos:related rdf:resource="descriptor/12224-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15881-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15682-1">
    <skos:broader rdf:resource="thsys/73388"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15682-1">
    <skos:broader rdf:resource="thsys/70850"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18419-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10002-4">
    <skos:related rdf:resource="descriptor/12716-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10002-4">
    <skos:related rdf:resource="descriptor/12035-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10002-4">
    <skos:related rdf:resource="descriptor/12747-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11042-1">
    <skos:scopeNote xml:lang="de">Für das Geldwesen BENUTZE Geld oder Geldverfassung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11042-1">
    <skos:scopeNote xml:lang="de">Wirtschaftsordnung, in der jeder Tauschakt am Markt mit Hilfe von Geld ausgeführt wird.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18102-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12865-1">
    <skos:prefLabel xml:lang="en">Family business</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12865-1">
    <skos:prefLabel xml:lang="de">Familienunternehmen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70899">
    <skos:prefLabel xml:lang="en">W.14.03.01  Banking</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70899">
    <skos:prefLabel xml:lang="de">W.14.03.01  Bankgeschäft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#altLabel">
    <rdfs:subPropertyOf rdf:resource="http://www.w3.org/2000/01/rdf-schema#label"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18132-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12874-0">
    <skos:related rdf:resource="descriptor/12863-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12874-0">
    <skos:related rdf:resource="descriptor/16104-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12874-0">
    <skos:related rdf:resource="descriptor/10828-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12925-2">
    <skos:altLabel xml:lang="en">Defense industry</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12925-2">
    <skos:altLabel xml:lang="de">Wehrgüterproduzierendes Gewerbe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13398-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13081-2">
    <skos:related rdf:resource="descriptor/14537-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13081-2">
    <skos:related rdf:resource="descriptor/13080-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13081-2">
    <skos:related rdf:resource="descriptor/13079-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16828-1">
    <skos:broader rdf:resource="thsys/70073"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16828-1">
    <skos:broader rdf:resource="descriptor/19084-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19229-1">
    <skos:narrower rdf:resource="descriptor/19239-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70333">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">P.21</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15940-5">
    <skos:broader rdf:resource="thsys/72181"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15940-5">
    <skos:broader rdf:resource="thsys/70814"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15940-5">
    <skos:broader rdf:resource="thsys/70182"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15940-5">
    <skos:broader rdf:resource="descriptor/15061-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14124-5">
    <skos:altLabel xml:lang="de">Trinkmilch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14124-5">
    <skos:altLabel xml:lang="de">Rohmilch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14751-6">
    <skos:prefLabel xml:lang="en">Computer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14751-6">
    <skos:prefLabel xml:lang="de">Computer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12575-3">
    <skos:related rdf:resource="descriptor/12058-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12575-3">
    <skos:related rdf:resource="descriptor/11191-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12575-3">
    <skos:related rdf:resource="descriptor/16058-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12575-3">
    <skos:related rdf:resource="descriptor/19003-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11901-0">
    <skos:related rdf:resource="descriptor/11999-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11901-0">
    <skos:related rdf:resource="descriptor/13844-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11901-0">
    <skos:related rdf:resource="descriptor/11890-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11901-0">
    <skos:related rdf:resource="descriptor/13846-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14450-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11876-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12787-2">
    <skos:related rdf:resource="descriptor/12665-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12787-2">
    <skos:related rdf:resource="descriptor/12785-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12787-2">
    <skos:related rdf:resource="descriptor/13464-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12787-2">
    <skos:related rdf:resource="descriptor/13459-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12787-2">
    <skos:related rdf:resource="descriptor/19819-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12787-2">
    <skos:related rdf:resource="descriptor/12033-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13831-6">
    <skos:altLabel xml:lang="de">Wohnungszählung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13831-6">
    <skos:altLabel xml:lang="de">Wohnungsstichprobe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13831-6">
    <skos:altLabel xml:lang="de">Wohnungsbestandserhebung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13831-6">
    <skos:altLabel xml:lang="de">Gebäudezählung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13831-6">
    <skos:altLabel xml:lang="de">Gebäudestatistik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13831-6">
    <skos:altLabel xml:lang="de">Gebäude- und Wohnungszählung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14339-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10950-6">
    <skos:related rdf:resource="descriptor/10905-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14989-1">
    <skos:altLabel xml:lang="de">Tiermehl</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14989-1">
    <skos:altLabel xml:lang="de">Kraftfutter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14989-1">
    <skos:altLabel xml:lang="de">Fischmehl</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10126-0">
    <skos:altLabel xml:lang="en">Theory of capital</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10126-0">
    <skos:altLabel xml:lang="en">Reswitching</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10126-0">
    <skos:altLabel xml:lang="en">Capital reversing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10126-0">
    <skos:altLabel xml:lang="de">Wicksell-Effekt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10126-0">
    <skos:altLabel xml:lang="de">Verformbares Kapital</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10126-0">
    <skos:altLabel xml:lang="de">Unverformbares Kapital</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10126-0">
    <skos:altLabel xml:lang="de">Clark-Ramsey-Kapitaltheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10126-0">
    <skos:altLabel xml:lang="de">Clark-Ramsey Kapitaltheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10126-0">
    <skos:altLabel xml:lang="de">Cambridger Kontroverse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10126-0">
    <skos:altLabel xml:lang="de">Cambridge-Kontroverse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10126-0">
    <skos:altLabel xml:lang="de">Cambridge Kontroverse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14750-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18358-2">
    <skos:broader rdf:resource="thsys/71065"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18358-2">
    <skos:broader rdf:resource="thsys/71012"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18358-2">
    <skos:broader rdf:resource="thsys/70539"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18358-2">
    <skos:broader rdf:resource="descriptor/11812-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11602-3">
    <skos:altLabel xml:lang="de">Länderfinanzausgleich</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18368-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15763-1">
    <skos:narrower rdf:resource="descriptor/14632-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15763-1">
    <skos:narrower rdf:resource="descriptor/15757-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15763-1">
    <skos:narrower rdf:resource="descriptor/15755-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15763-1">
    <skos:narrower rdf:resource="descriptor/15743-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15763-1">
    <skos:narrower rdf:resource="descriptor/15739-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15763-1">
    <skos:narrower rdf:resource="descriptor/18453-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15763-1">
    <skos:narrower rdf:resource="descriptor/15735-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15763-1">
    <skos:narrower rdf:resource="descriptor/15733-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15763-1">
    <skos:narrower rdf:resource="descriptor/12016-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18990-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13446-2">
    <skos:altLabel xml:lang="en">On-farm research</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13446-2">
    <skos:altLabel xml:lang="en">Agricultural research</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13446-2">
    <skos:altLabel xml:lang="de">Landwirtschaftswissenschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13446-2">
    <skos:altLabel xml:lang="de">Landwirtschaftliche Forschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13446-2">
    <skos:altLabel xml:lang="de">Agronomie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13446-2">
    <skos:altLabel xml:lang="de">Agrarforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10781-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14060-5">
    <skos:broader rdf:resource="thsys/73298"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14060-5">
    <skos:broader rdf:resource="thsys/70341"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14060-5">
    <skos:broader rdf:resource="descriptor/18222-6"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id370">
    <rdf:value xml:lang="de">für innerstaatliche regionale Integration</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15469-1">
    <skos:narrower rdf:resource="descriptor/15494-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15469-1">
    <skos:narrower rdf:resource="descriptor/15488-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10398-2">
    <skos:related rdf:resource="descriptor/20104-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10398-2">
    <skos:related rdf:resource="descriptor/11727-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10398-2">
    <skos:related rdf:resource="descriptor/10378-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15115-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12195-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19556-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11995-0">
    <skos:scopeNote xml:lang="de">Bauleitpläne werden von den Gemeinden zur Steuerung der städtebaulichen Entwicklung aufgestellt.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16268-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24664-2">
    <skos:related rdf:resource="descriptor/18655-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24664-2">
    <skos:related rdf:resource="descriptor/18110-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24664-2">
    <skos:related rdf:resource="descriptor/18941-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11223-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19563-3">
    <skos:related rdf:resource="descriptor/10019-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19563-3">
    <skos:related rdf:resource="descriptor/18975-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19563-3">
    <skos:related rdf:resource="descriptor/19540-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19563-3">
    <skos:related rdf:resource="descriptor/10797-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18565-4">
    <skos:broader rdf:resource="thsys/70944"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18565-4">
    <skos:broader rdf:resource="thsys/70943"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18565-4">
    <skos:broader rdf:resource="descriptor/13264-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18565-4">
    <skos:broader rdf:resource="descriptor/13306-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13766-5">
    <skos:broader rdf:resource="thsys/71103"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13766-5">
    <skos:broader rdf:resource="thsys/71088"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13766-5">
    <skos:broader rdf:resource="thsys/70187"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13766-5">
    <skos:broader rdf:resource="descriptor/18668-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13766-5">
    <skos:broader rdf:resource="descriptor/13757-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11061-4">
    <skos:broader rdf:resource="thsys/71101"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11061-4">
    <skos:broader rdf:resource="descriptor/11068-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19236-4">
    <skos:related rdf:resource="descriptor/15069-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19236-4">
    <skos:related rdf:resource="descriptor/10300-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19236-4">
    <skos:related rdf:resource="descriptor/11032-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19236-4">
    <skos:related rdf:resource="descriptor/19412-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16671-1">
    <skos:broader rdf:resource="thsys/73356"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16671-1">
    <skos:broader rdf:resource="thsys/70888"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16671-1">
    <skos:broader rdf:resource="descriptor/13849-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70182">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13057-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19724-5">
    <skos:narrower rdf:resource="descriptor/11582-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11294-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10236-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18521-3">
    <skos:prefLabel xml:lang="en">Tea trade</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18521-3">
    <skos:prefLabel xml:lang="de">Teehandel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14723-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12416-2">
    <skos:related rdf:resource="descriptor/11676-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12416-2">
    <skos:related rdf:resource="descriptor/12419-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12416-2">
    <skos:related rdf:resource="descriptor/18302-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13743-3">
    <skos:broader rdf:resource="thsys/71107"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13743-3">
    <skos:broader rdf:resource="descriptor/10021-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15605-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11571-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19852-3">
    <skos:broader rdf:resource="thsys/70160"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15038-0">
    <skos:altLabel xml:lang="de">Petri-Netz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15038-0">
    <skos:altLabel xml:lang="de">Netztheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15038-0">
    <skos:altLabel xml:lang="de">Gozinto-Graph</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70327">
    <skos:broader rdf:resource="thsys/70148"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10986-6">
    <skos:narrower rdf:resource="descriptor/10977-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73373">
    <skos:prefLabel xml:lang="de">N.04.03.01  Innen- und Rechtspolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17959-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71114">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14274-0">
    <skos:altLabel xml:lang="de">Wolfram</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14274-0">
    <skos:altLabel xml:lang="de">Wismut</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14274-0">
    <skos:altLabel xml:lang="de">Vanadium</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14274-0">
    <skos:altLabel xml:lang="de">Tungsten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14274-0">
    <skos:altLabel xml:lang="de">Tantal</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14274-0">
    <skos:altLabel xml:lang="de">Kobalt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70881">
    <skos:prefLabel xml:lang="en">W.19.01  Software industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70881">
    <skos:prefLabel xml:lang="de">W.19.01  Softwareindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15518-0">
    <skos:altLabel xml:lang="en">Vehicle routing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15518-0">
    <skos:altLabel xml:lang="en">Vehicle dispatching</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15518-0">
    <skos:altLabel xml:lang="en">Routing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15518-0">
    <skos:altLabel xml:lang="en">Chinese-postman problem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15518-0">
    <skos:altLabel xml:lang="de">Verkehrsmitteleinsatzplanung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15518-0">
    <skos:altLabel xml:lang="de">Sweep-Algorithmus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15518-0">
    <skos:altLabel xml:lang="de">Savings-Verfahren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15518-0">
    <skos:altLabel xml:lang="de">Optimaler Fuhrparkeinsatz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15518-0">
    <skos:altLabel xml:lang="de">Mehrdepot-Tourenplanung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15518-0">
    <skos:altLabel xml:lang="de">Fuhrparkeinsatzplanung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15518-0">
    <skos:altLabel xml:lang="de">Fahrzeugeinsatzplanung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15518-0">
    <skos:altLabel xml:lang="de">Fahrplan-Optimierungsproblem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15518-0">
    <skos:altLabel xml:lang="de">Briefträgerproblem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11954-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11167-2">
    <skos:related rdf:resource="descriptor/16353-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11167-2">
    <skos:related rdf:resource="descriptor/11057-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11167-2">
    <skos:related rdf:resource="descriptor/10109-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11167-2">
    <skos:related rdf:resource="descriptor/10107-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11167-2">
    <skos:related rdf:resource="descriptor/10353-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11167-2">
    <skos:related rdf:resource="descriptor/16366-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11167-2">
    <skos:related rdf:resource="descriptor/16246-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18780-0">
    <skos:related rdf:resource="descriptor/14017-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18780-0">
    <skos:related rdf:resource="descriptor/10032-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18780-0">
    <skos:related rdf:resource="descriptor/18880-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18780-0">
    <skos:related rdf:resource="descriptor/18881-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73343">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19665-2">
    <skos:broader rdf:resource="thsys/70305"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12798-4">
    <skos:broader rdf:resource="thsys/70947"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12798-4">
    <skos:broader rdf:resource="thsys/70550"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12798-4">
    <skos:broader rdf:resource="descriptor/12800-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14109-1">
    <skos:altLabel xml:lang="de">Walnuss</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14109-1">
    <skos:altLabel xml:lang="de">Walnuß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14109-1">
    <skos:altLabel xml:lang="de">Nuß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14109-1">
    <skos:altLabel xml:lang="de">Cashewnuss</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14109-1">
    <skos:altLabel xml:lang="de">Cashewnuß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10610-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11223-4">
    <skos:related rdf:resource="descriptor/10141-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11223-4">
    <skos:related rdf:resource="descriptor/18138-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16720-0">
    <skos:scopeNote xml:lang="de">Macht das "Positive" zum Prinzip alles wissenschaftlichen Wissens und aller menschlichen Erkenntnis, wobei das "Positive" das durch Erfahrung Gegebene bezeichnet.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12033-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13213-4">
    <skos:broader rdf:resource="thsys/70959"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13213-4">
    <skos:broader rdf:resource="descriptor/18522-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71000">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.06.01.01.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19031-1">
    <skos:scopeNote xml:lang="de">Für bibliothekarische Klassifikationen BENUTZE Dokumentationssprache (Deskriptor bzw. Publikationsart).</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19031-1">
    <skos:scopeNote xml:lang="de">Nur für Literatur über Klassifikationen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id371">
    <rdf:value xml:lang="de">für die Formen ökonomischer Konzentration</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16160-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/v">
    <rdfs:label xml:lang="en">Economics</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/v">
    <rdfs:label xml:lang="de">Volkswirtschaft</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70938">
    <skos:narrower rdf:resource="descriptor/11827-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70938">
    <skos:narrower rdf:resource="descriptor/18581-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70938">
    <skos:narrower rdf:resource="descriptor/18580-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70938">
    <skos:narrower rdf:resource="descriptor/16171-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70938">
    <skos:narrower rdf:resource="descriptor/18579-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70938">
    <skos:narrower rdf:resource="descriptor/13563-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70938">
    <skos:narrower rdf:resource="descriptor/18578-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70938">
    <skos:narrower rdf:resource="descriptor/13562-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70938">
    <skos:narrower rdf:resource="descriptor/13561-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70938">
    <skos:narrower rdf:resource="descriptor/13560-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70938">
    <skos:narrower rdf:resource="descriptor/13633-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70938">
    <skos:narrower rdf:resource="descriptor/11826-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70938">
    <skos:narrower rdf:resource="descriptor/13557-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70938">
    <skos:narrower rdf:resource="descriptor/18577-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70938">
    <skos:narrower rdf:resource="thsys/70937"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19755-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20449-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14234-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10828-3">
    <skos:scopeNote xml:lang="de">Nur für internationale steuerliche Doppelbelastung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12996-0">
    <skos:prefLabel xml:lang="en">Chemical industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12996-0">
    <skos:prefLabel xml:lang="de">Chemieindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11552-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10271-5">
    <skos:related rdf:resource="descriptor/19602-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10271-5">
    <skos:related rdf:resource="descriptor/11460-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19243-0">
    <skos:prefLabel xml:lang="en">Rural growth</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19243-0">
    <skos:prefLabel xml:lang="de">Ländliches Wachstum</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14599-0">
    <skos:narrower rdf:resource="descriptor/14635-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14599-0">
    <skos:narrower rdf:resource="descriptor/14741-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14599-0">
    <skos:narrower rdf:resource="descriptor/14632-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14599-0">
    <skos:narrower rdf:resource="descriptor/14602-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14599-0">
    <skos:narrower rdf:resource="descriptor/14738-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14599-0">
    <skos:narrower rdf:resource="descriptor/14728-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14599-0">
    <skos:narrower rdf:resource="descriptor/14656-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14599-0">
    <skos:narrower rdf:resource="descriptor/14218-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14599-0">
    <skos:narrower rdf:resource="descriptor/18268-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14599-0">
    <skos:narrower rdf:resource="descriptor/14702-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14599-0">
    <skos:narrower rdf:resource="descriptor/14734-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14599-0">
    <skos:narrower rdf:resource="descriptor/14607-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14599-0">
    <skos:narrower rdf:resource="descriptor/14619-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14599-0">
    <skos:narrower rdf:resource="descriptor/14743-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14599-0">
    <skos:narrower rdf:resource="descriptor/14600-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14599-0">
    <skos:narrower rdf:resource="descriptor/14854-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14599-0">
    <skos:narrower rdf:resource="descriptor/14642-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14599-0">
    <skos:narrower rdf:resource="descriptor/14596-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14599-0">
    <skos:narrower rdf:resource="descriptor/14631-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18945-1">
    <skos:broader rdf:resource="thsys/73369"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18945-1">
    <skos:broader rdf:resource="thsys/70822"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18945-1">
    <skos:broader rdf:resource="descriptor/16515-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18945-1">
    <skos:broader rdf:resource="descriptor/18944-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16215-6">
    <skos:scopeNote xml:lang="de">Die Literatur wird mit dem Land indexiert, das die Forschung betreibt.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15814-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10678-3">
    <skos:altLabel xml:lang="de">Devisenkurstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10589-2">
    <skos:altLabel xml:lang="en">Nongovernmental development</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10589-2">
    <skos:altLabel xml:lang="en">Development NGO</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10589-2">
    <skos:altLabel xml:lang="de">Nichtstaatliche Entwicklungshilfe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10589-2">
    <skos:altLabel xml:lang="de">NGO-Entwicklungshilfe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73367">
    <skos:broader rdf:resource="thsys/73374"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14102-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11152-1">
    <skos:altLabel xml:lang="en">Voting with feet</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11152-1">
    <skos:altLabel xml:lang="en">Voice theory</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11152-1">
    <skos:altLabel xml:lang="de">Voice-Theorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11152-1">
    <skos:altLabel xml:lang="de">Exit-Voice-Ansatz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16533-6">
    <skos:altLabel xml:lang="en">Poverty reduction</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16533-6">
    <skos:altLabel xml:lang="de">Armutsbekämpfung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14611-1">
    <skos:related rdf:resource="descriptor/19792-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14611-1">
    <skos:related rdf:resource="descriptor/14931-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14611-1">
    <skos:related rdf:resource="descriptor/18443-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14611-1">
    <skos:related rdf:resource="descriptor/16004-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14611-1">
    <skos:related rdf:resource="descriptor/18442-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14611-1">
    <skos:related rdf:resource="descriptor/14744-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14611-1">
    <skos:related rdf:resource="descriptor/14623-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12967-0">
    <skos:broader rdf:resource="thsys/71018"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12967-0">
    <skos:broader rdf:resource="descriptor/12964-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13815-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14418-2">
    <skos:prefLabel xml:lang="en">Aeroplane</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14418-2">
    <skos:prefLabel xml:lang="de">Flugzeug</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13173-4">
    <skos:altLabel xml:lang="de">Brückenbau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13689-4">
    <skos:prefLabel xml:lang="en">Banking statistics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13689-4">
    <skos:prefLabel xml:lang="de">Bankenstatistik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10138-0">
    <skos:altLabel xml:lang="en">Vickrey auction game</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10138-0">
    <skos:altLabel xml:lang="en">Theory of auctions</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10138-0">
    <skos:altLabel xml:lang="en">Sealed offer market</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10138-0">
    <skos:altLabel xml:lang="en">Bidding model</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10138-0">
    <skos:altLabel xml:lang="en">Bidding behaviour</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10138-0">
    <skos:altLabel xml:lang="en">Auction market</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10138-0">
    <skos:altLabel xml:lang="en">Auction game</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10138-0">
    <skos:altLabel xml:lang="de">Competitive-Bidding-Modell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10138-0">
    <skos:altLabel xml:lang="de">Competitive-bidding Modell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10138-0">
    <skos:altLabel xml:lang="de">Bidding behavior</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id365">
    <rdfs:seeAlso rdf:resource="descriptor/18695-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15108-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14751-6">
    <skos:altLabel xml:lang="en">Hardware</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14751-6">
    <skos:altLabel xml:lang="de">Zentraleinheit (Computer)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14751-6">
    <skos:altLabel xml:lang="de">Rechner</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14751-6">
    <skos:altLabel xml:lang="de">Rechenanlage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14751-6">
    <skos:altLabel xml:lang="de">Elektronenrechner</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14751-6">
    <skos:altLabel xml:lang="de">EDV-Konfiguration</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14751-6">
    <skos:altLabel xml:lang="de">EDV-Anlage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14751-6">
    <skos:altLabel xml:lang="de">Datenverarbeitungsanlage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14751-6">
    <skos:altLabel xml:lang="de">Datenerfassungsgerät</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14751-6">
    <skos:altLabel xml:lang="de">Computerterminal</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11271-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18379-1">
    <skos:broader rdf:resource="thsys/73371"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18379-1">
    <skos:broader rdf:resource="thsys/71011"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18379-1">
    <skos:broader rdf:resource="thsys/70817"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18379-1">
    <skos:broader rdf:resource="thsys/70811"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18379-1">
    <skos:broader rdf:resource="thsys/70239"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18379-1">
    <skos:broader rdf:resource="descriptor/15752-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15017-1">
    <skos:narrower rdf:resource="descriptor/15013-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12160-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12030-3">
    <skos:altLabel xml:lang="de">Leasing von Immobilien</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12030-3">
    <skos:altLabel xml:lang="de">Immobilienleasing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16542-5">
    <skos:prefLabel xml:lang="en">Social security</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16542-5">
    <skos:prefLabel xml:lang="de">Soziale Sicherung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10675-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70002">
    <skos:prefLabel xml:lang="en">G.01  Europe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70002">
    <skos:prefLabel xml:lang="de">G.01  Europa</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15928-2">
    <skos:broader rdf:resource="thsys/73348"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15928-2">
    <skos:broader rdf:resource="thsys/70142"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70038">
    <skos:prefLabel xml:lang="de">V.06.02  Allgemeine Wirtschaftspolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70049">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10821-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14245-0">
    <skos:altLabel xml:lang="de">Schamotte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14245-0">
    <skos:altLabel xml:lang="de">Lehm</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14245-0">
    <skos:altLabel xml:lang="de">Keramischer Rohstoff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14245-0">
    <skos:altLabel xml:lang="de">Kaolin</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14245-0">
    <skos:altLabel xml:lang="de">Bleichton</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14245-0">
    <skos:altLabel xml:lang="de">Bentonit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17597-3">
    <skos:broader rdf:resource="thsys/70561"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17597-3">
    <skos:broader rdf:resource="descriptor/19167-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18448-1">
    <skos:scopeNote xml:lang="de">Sammelbegriff für die wirtschaftliche Verwertung von Musik im weitesten Sinne</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12580-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10771-6">
    <skos:prefLabel xml:lang="en">Monetary union</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10771-6">
    <skos:prefLabel xml:lang="de">Währungsunion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14048-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70265">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">P.15</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17914-3">
    <skos:prefLabel xml:lang="en">Bolivia</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17914-3">
    <skos:prefLabel xml:lang="de">Bolivien</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12602-5">
    <skos:narrower rdf:resource="descriptor/12612-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19128-0">
    <skos:prefLabel xml:lang="en">Instructions for use</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19128-0">
    <skos:prefLabel xml:lang="de">Gebrauchsanleitung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12475-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14441-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18334-2">
    <skos:narrower rdf:resource="descriptor/18811-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10491-5">
    <skos:altLabel xml:lang="en">Brain gain</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10491-5">
    <skos:altLabel xml:lang="de">Auswanderung von hochqualifiziertem Personal</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13260-2">
    <skos:broader rdf:resource="thsys/70940"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13260-2">
    <skos:broader rdf:resource="descriptor/24787-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13337-0">
    <skos:altLabel xml:lang="de">Thermische Müllverwertung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13337-0">
    <skos:altLabel xml:lang="de">Thermische Abfallbehandlung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13337-0">
    <skos:altLabel xml:lang="de">Müllverbrennungstechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13337-0">
    <skos:altLabel xml:lang="de">Müllverbrennungsanlage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13337-0">
    <skos:altLabel xml:lang="de">Abfallverbrennung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18869-5">
    <skos:altLabel xml:lang="en">Knowledge spillover</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18869-5">
    <skos:altLabel xml:lang="en">Know-how-transfer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18869-5">
    <skos:altLabel xml:lang="en">F&amp;amp;E transfer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18869-5">
    <skos:altLabel xml:lang="de">FuE-Transfer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18869-5">
    <skos:altLabel xml:lang="de">Forschungstransfer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18633-6">
    <skos:altLabel xml:lang="de">Stadtverkehrsplanung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18633-6">
    <skos:altLabel xml:lang="de">Städtische Verkehrsplanung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19182-1">
    <skos:broader rdf:resource="thsys/71006"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19182-1">
    <skos:broader rdf:resource="thsys/70992"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19182-1">
    <skos:broader rdf:resource="thsys/70957"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19182-1">
    <skos:broader rdf:resource="descriptor/18995-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70148">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10243-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17915-1">
    <skos:altLabel xml:lang="x-other">Republica Federativa do Brasil</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17915-1">
    <skos:altLabel xml:lang="de">Föderative Republik Brasilien</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11972-5">
    <skos:broader rdf:resource="thsys/71064"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11972-5">
    <skos:broader rdf:resource="descriptor/11979-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14809-1">
    <skos:broader rdf:resource="thsys/70328"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14809-1">
    <skos:broader rdf:resource="descriptor/14802-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12004-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18701-1">
    <skos:altLabel xml:lang="de">Zinssatzswap</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18422-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71055">
    <skos:prefLabel xml:lang="en">V.13.06.02  Service workers</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71055">
    <skos:prefLabel xml:lang="de">V.13.06.02  Dienstleistungsberufe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10070-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11832-0">
    <skos:broader rdf:resource="thsys/70926"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11629-4">
    <skos:prefLabel xml:lang="en">Public Transport Sector</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11629-4">
    <skos:prefLabel xml:lang="de">Öffentliche Verkehrswirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15876-2">
    <skos:prefLabel xml:lang="en">Earth sciences</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15876-2">
    <skos:prefLabel xml:lang="de">Geowissenschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14844-6">
    <skos:prefLabel xml:lang="en">Spinning material</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14844-6">
    <skos:prefLabel xml:lang="de">Spinnstoffwaren</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18580-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19719-5">
    <skos:related rdf:resource="descriptor/17758-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19075-2">
    <skos:broader rdf:resource="thsys/70361"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19075-2">
    <skos:broader rdf:resource="descriptor/18012-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16539-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11866-4">
    <skos:broader rdf:resource="thsys/73351"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11866-4">
    <skos:broader rdf:resource="thsys/71068"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11866-4">
    <skos:broader rdf:resource="descriptor/11869-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20189-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14906-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19692-6">
    <zbwext:useInsteadNote rdf:nodeID="id4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19412-3">
    <skos:altLabel xml:lang="en">Spendings tax</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19412-3">
    <skos:altLabel xml:lang="en">Expenditure tax</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19412-3">
    <skos:altLabel xml:lang="en">Consumption-based direct tax</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19412-3">
    <skos:altLabel xml:lang="en">Consumption tax (direct tax)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19412-3">
    <skos:altLabel xml:lang="de">Direkte Konsumbesteuerung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19412-3">
    <skos:altLabel xml:lang="de">Ausgabenkonsumsteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12026-1">
    <skos:related rdf:resource="descriptor/12029-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12026-1">
    <skos:related rdf:resource="descriptor/13775-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12026-1">
    <skos:related rdf:resource="descriptor/12028-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12026-1">
    <skos:related rdf:resource="descriptor/11659-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12026-1">
    <skos:related rdf:resource="descriptor/11658-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12026-1">
    <skos:related rdf:resource="descriptor/12023-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12026-1">
    <skos:related rdf:resource="descriptor/12015-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14573-4">
    <skos:prefLabel xml:lang="en">Musical instrument</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14573-4">
    <skos:prefLabel xml:lang="de">Musikinstrument</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18313-3">
    <skos:narrower rdf:resource="descriptor/21094-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18313-3">
    <skos:narrower rdf:resource="descriptor/11149-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18313-3">
    <skos:narrower rdf:resource="descriptor/18137-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18313-3">
    <skos:narrower rdf:resource="descriptor/13453-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18313-3">
    <skos:narrower rdf:resource="descriptor/18093-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18313-3">
    <skos:narrower rdf:resource="descriptor/13440-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18313-3">
    <skos:narrower rdf:resource="descriptor/19045-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18313-3">
    <skos:narrower rdf:resource="descriptor/18326-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18313-3">
    <skos:narrower rdf:resource="descriptor/12936-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18313-3">
    <skos:narrower rdf:resource="descriptor/13452-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18313-3">
    <skos:narrower rdf:resource="descriptor/13429-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18638-3">
    <skos:prefLabel xml:lang="en">City tourism</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18638-3">
    <skos:prefLabel xml:lang="de">Städtetourismus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19784-1">
    <skos:editorialNote xml:lang="de">Möglichst spezifischer indexieren.</skos:editorialNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10068-2">
    <skos:prefLabel xml:lang="en">Two-country model</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10068-2">
    <skos:prefLabel xml:lang="de">Zwei-Länder-Modell</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18901-0">
    <skos:prefLabel xml:lang="en">Animal disease</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18901-0">
    <skos:prefLabel xml:lang="de">Tierkrankheit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12730-3">
    <skos:altLabel xml:lang="en">Holonic production</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12730-3">
    <skos:altLabel xml:lang="en">Holonic manufacturing system</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12730-3">
    <skos:altLabel xml:lang="en">HMS (Holonic manufacturing system)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12730-3">
    <skos:altLabel xml:lang="en">Computer-aided production</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12730-3">
    <skos:altLabel xml:lang="en">CAP</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12730-3">
    <skos:altLabel xml:lang="en">Automated manufacturing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12730-3">
    <skos:altLabel xml:lang="de">Rechnergestützte Produktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12730-3">
    <skos:altLabel xml:lang="de">Rechnergestützte Fertigung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12730-3">
    <skos:altLabel xml:lang="de">Produktions-Informationssystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12730-3">
    <skos:altLabel xml:lang="de">Produktionsautomatisierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12730-3">
    <skos:altLabel xml:lang="de">Holonisches Produktionssystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12730-3">
    <skos:altLabel xml:lang="de">Fertigungs-Informationssystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12730-3">
    <skos:altLabel xml:lang="de">Fertigungsautomatisierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12730-3">
    <skos:altLabel xml:lang="de">Fabrik der Zukunft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12730-3">
    <skos:altLabel xml:lang="de">EDV-gestützte Produktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12730-3">
    <skos:altLabel xml:lang="de">Computergestützte Produktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12730-3">
    <skos:altLabel xml:lang="de">Computergestützte Fertigung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12730-3">
    <skos:altLabel xml:lang="de">CAM</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16061-0">
    <skos:altLabel xml:lang="de">Kapitalanlagegesellschaftsgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16061-0">
    <skos:altLabel xml:lang="de">KAGG (Kapitalanlagegesellschaftsgesetz)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16061-0">
    <skos:altLabel xml:lang="de">Investmentgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16061-0">
    <skos:altLabel xml:lang="de">Investmentfondsgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16527-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16497-5">
    <skos:altLabel xml:lang="en">Peaceful coexistence</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16497-5">
    <skos:altLabel xml:lang="de">Völkerverständigung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16497-5">
    <skos:altLabel xml:lang="de">Kriegsverhütung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16497-5">
    <skos:altLabel xml:lang="de">Kriegsprävention</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16497-5">
    <skos:altLabel xml:lang="de">Internationale Verständigung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16497-5">
    <skos:altLabel xml:lang="de">Internationale Friedenssicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16497-5">
    <skos:altLabel xml:lang="de">Internationale Friedensordnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16497-5">
    <skos:altLabel xml:lang="de">Friedliche Koexistenz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16497-5">
    <skos:altLabel xml:lang="de">Friedenspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16497-5">
    <skos:altLabel xml:lang="de">Entspannungspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16497-5">
    <skos:altLabel xml:lang="de">Detente</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17595-0">
    <skos:related rdf:resource="descriptor/19837-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17595-0">
    <skos:related rdf:resource="descriptor/17565-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70035">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18003-4">
    <skos:altLabel xml:lang="en">Knowledge-based system</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18003-4">
    <skos:altLabel xml:lang="de">Wissensbasiertes System</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18003-4">
    <skos:altLabel xml:lang="de">Expertisesystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12872-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70415">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">P.09.03</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id144">
    <rdf:value xml:lang="de">Für staatliche Öffentlichkeitsarbeit</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13979-2">
    <skos:prefLabel xml:lang="en">Transport workers</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13979-2">
    <skos:prefLabel xml:lang="de">Verkehrsberufe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20277-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16521-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11807-6">
    <skos:broader rdf:resource="thsys/71110"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11807-6">
    <skos:broader rdf:resource="thsys/71018"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11807-6">
    <skos:broader rdf:resource="descriptor/10981-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13575-5">
    <skos:related rdf:resource="descriptor/13578-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13575-5">
    <skos:related rdf:resource="descriptor/18366-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13575-5">
    <skos:related rdf:resource="descriptor/18296-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13575-5">
    <skos:related rdf:resource="descriptor/18607-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13575-5">
    <skos:related rdf:resource="descriptor/13246-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13575-5">
    <skos:related rdf:resource="descriptor/14421-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13575-5">
    <skos:related rdf:resource="descriptor/13057-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13575-5">
    <skos:related rdf:resource="descriptor/18418-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18098-2">
    <skos:prefLabel xml:lang="en">Product placement</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18098-2">
    <skos:prefLabel xml:lang="de">Product Placement</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17783-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70114">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70037">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">B.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18412-1">
    <skos:broader rdf:resource="thsys/71002"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18412-1">
    <skos:broader rdf:resource="descriptor/10470-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70850">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.23.02.03</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18222-6">
    <skos:altLabel xml:lang="de">Genussmittelpflanze</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18222-6">
    <skos:altLabel xml:lang="de">Genußmittelpflanze</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18222-6">
    <skos:altLabel xml:lang="de">Genußmittel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12794-5">
    <skos:altLabel xml:lang="de">Preisuntergrenze</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12794-5">
    <skos:altLabel xml:lang="de">Preisstrategie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12794-5">
    <skos:altLabel xml:lang="de">Preispolitische Strategie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12794-5">
    <skos:altLabel xml:lang="de">Preisobergrenze</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12794-5">
    <skos:altLabel xml:lang="de">Preisbildung des Betriebs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12794-5">
    <skos:altLabel xml:lang="de">Betriebliche Preisermittlung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12794-5">
    <skos:altLabel xml:lang="de">Betriebliche Preisbildung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11879-2">
    <skos:altLabel xml:lang="de">Siedlungsentwicklung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11356-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12651-6">
    <skos:altLabel xml:lang="de">Materialdisposition</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12651-6">
    <skos:altLabel xml:lang="de">Materialbereitstellung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12651-6">
    <skos:altLabel xml:lang="de">Materialbedarfsrechnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12651-6">
    <skos:altLabel xml:lang="de">Materialbedarfsplanung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12651-6">
    <skos:altLabel xml:lang="de">Einkaufsplanung des Betriebs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12651-6">
    <skos:altLabel xml:lang="de">Beschaffungsbedarf</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18905-6">
    <skos:broader rdf:resource="thsys/71079"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18905-6">
    <skos:broader rdf:resource="thsys/70835"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18905-6">
    <skos:broader rdf:resource="thsys/70504"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18905-6">
    <skos:broader rdf:resource="thsys/70160"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18905-6">
    <skos:broader rdf:resource="descriptor/16544-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10677-5">
    <skos:scopeNote xml:lang="de">Wirkungsweise von Veränderungen exogener Variablen auf die Zahlungsbilanz oder eine ihrer Teilbilanzen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18301-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18116-5">
    <skos:prefLabel xml:lang="en">Business ethics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18116-5">
    <skos:prefLabel xml:lang="de">Unternehmensethik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10413-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14700-2">
    <skos:broader rdf:resource="thsys/70476"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14700-2">
    <skos:broader rdf:resource="descriptor/14702-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14788-4">
    <skos:prefLabel xml:lang="en">Wood product</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14788-4">
    <skos:prefLabel xml:lang="de">Holzprodukt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11401-6">
    <skos:broader rdf:resource="thsys/70847"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11401-6">
    <skos:broader rdf:resource="thsys/70143"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71088">
    <rdfs:label xml:lang="en">Foreign exchange market</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71088">
    <rdfs:label xml:lang="de">Devisenmarkt</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14593-5">
    <skos:broader rdf:resource="thsys/70201"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14734-6">
    <skos:related rdf:resource="descriptor/14738-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14734-6">
    <skos:related rdf:resource="descriptor/14728-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14734-6">
    <skos:related rdf:resource="descriptor/18262-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14734-6">
    <skos:related rdf:resource="descriptor/14026-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18865-6">
    <skos:related rdf:resource="descriptor/19463-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18865-6">
    <skos:related rdf:resource="descriptor/10575-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18865-6">
    <skos:related rdf:resource="descriptor/19461-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12990-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12330-5">
    <skos:related rdf:resource="descriptor/12473-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12330-5">
    <skos:related rdf:resource="descriptor/12883-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12441-3">
    <skos:altLabel xml:lang="de">Sammelwertberichtigung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12441-3">
    <skos:altLabel xml:lang="de">Pauschalwertberichtigung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12441-3">
    <skos:altLabel xml:lang="de">Einzelwertberichtigung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10550-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15346-3">
    <skos:broader rdf:resource="thsys/72393"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15346-3">
    <skos:broader rdf:resource="descriptor/19150-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14427-1">
    <skos:related rdf:resource="descriptor/18089-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19445-2">
    <skos:broader rdf:resource="thsys/73362"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19445-2">
    <skos:broader rdf:resource="thsys/71069"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19445-2">
    <skos:broader rdf:resource="descriptor/11937-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10676-0">
    <skos:broader rdf:resource="thsys/71099"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10676-0">
    <skos:broader rdf:resource="descriptor/10629-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20744-3">
    <skos:altLabel xml:lang="en">CDM (Clean Development Mechanism)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20744-3">
    <skos:altLabel xml:lang="de">Mechanismus für umweltverträgliche Entwicklung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19774-4">
    <zbwext:useInsteadNote rdf:nodeID="id231"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17987-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19213-2">
    <skos:narrower rdf:resource="descriptor/11156-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15628-0">
    <skos:prefLabel xml:lang="en">Nuclear research</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15628-0">
    <skos:prefLabel xml:lang="de">Kernforschung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10300-3">
    <skos:altLabel xml:lang="en">Consumer research</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10300-3">
    <skos:altLabel xml:lang="en">Consumer preference</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10300-3">
    <skos:altLabel xml:lang="en">Consumer choice behaviour</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10300-3">
    <skos:altLabel xml:lang="en">Consumer choice</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10300-3">
    <skos:altLabel xml:lang="de">Verbrauchsmotivforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10300-3">
    <skos:altLabel xml:lang="de">Verbraucherverhalten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10300-3">
    <skos:altLabel xml:lang="de">Verbraucherforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10300-3">
    <skos:altLabel xml:lang="de">Verbrauchereinstellung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10300-3">
    <skos:altLabel xml:lang="de">Verbraucherbefragung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10300-3">
    <skos:altLabel xml:lang="de">Konsumverhalten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10300-3">
    <skos:altLabel xml:lang="de">Konsumentensouveränität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10300-3">
    <skos:altLabel xml:lang="de">Konsumentenpsychologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10300-3">
    <skos:altLabel xml:lang="de">Konsumentenpräferenz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10300-3">
    <skos:altLabel xml:lang="de">Konsumentenforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10300-3">
    <skos:altLabel xml:lang="de">Kaufmotivforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10300-3">
    <skos:altLabel xml:lang="de">Kaufmotiv</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10300-3">
    <skos:altLabel xml:lang="de">Käuferverhalten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10300-3">
    <skos:altLabel xml:lang="de">Käufermotivation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10300-3">
    <skos:altLabel xml:lang="de">Käufereinstellung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10300-3">
    <skos:altLabel xml:lang="de">Kaufentscheidung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70154">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.05</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10842-2">
    <skos:related rdf:resource="descriptor/18160-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10842-2">
    <skos:related rdf:resource="descriptor/10641-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10842-2">
    <skos:related rdf:resource="descriptor/10876-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10842-2">
    <skos:related rdf:resource="descriptor/19283-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10842-2">
    <skos:related rdf:resource="descriptor/19299-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10842-2">
    <skos:related rdf:resource="descriptor/18001-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10842-2">
    <skos:related rdf:resource="descriptor/10604-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17176-6">
    <skos:prefLabel xml:lang="en">Yugoslavia (until 1990)</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17176-6">
    <skos:prefLabel xml:lang="de">Jugoslawien (bis 1990)</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18111-1">
    <skos:altLabel xml:lang="en">Terminal workplace</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18111-1">
    <skos:altLabel xml:lang="en">Terminal work space</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18111-1">
    <skos:altLabel xml:lang="en">Computer-aided workplace</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18111-1">
    <skos:altLabel xml:lang="de">Computerarbeitsplatz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19430-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13354-0">
    <skos:related rdf:resource="descriptor/16676-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13354-0">
    <skos:related rdf:resource="descriptor/15675-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13354-0">
    <skos:related rdf:resource="descriptor/14346-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13354-0">
    <skos:related rdf:resource="descriptor/14335-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17662-4">
    <skos:prefLabel xml:lang="en">Comoros</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17662-4">
    <skos:prefLabel xml:lang="de">Komoren</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11698-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13294-6">
    <skos:prefLabel xml:lang="en">Life insurance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13294-6">
    <skos:prefLabel xml:lang="de">Lebensversicherung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18763-0">
    <zbwext:useInsteadNote rdf:nodeID="id65"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12158-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21101-5">
    <skos:related rdf:resource="descriptor/10771-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21101-5">
    <skos:related rdf:resource="descriptor/11521-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21101-5">
    <skos:related rdf:resource="descriptor/10721-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21101-5">
    <skos:related rdf:resource="descriptor/19336-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10255-3">
    <skos:related rdf:resource="descriptor/10353-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10255-3">
    <skos:related rdf:resource="descriptor/18150-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10255-3">
    <skos:related rdf:resource="descriptor/11507-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10255-3">
    <skos:related rdf:resource="descriptor/11495-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10255-3">
    <skos:related rdf:resource="descriptor/11447-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71105">
    <skos:narrower rdf:resource="descriptor/13745-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71105">
    <skos:narrower rdf:resource="descriptor/12263-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71105">
    <skos:narrower rdf:resource="descriptor/13738-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71105">
    <skos:narrower rdf:resource="descriptor/13736-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71105">
    <skos:narrower rdf:resource="descriptor/12256-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71105">
    <skos:narrower rdf:resource="descriptor/11611-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71105">
    <skos:narrower rdf:resource="descriptor/11609-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71105">
    <skos:narrower rdf:resource="descriptor/10804-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71105">
    <skos:narrower rdf:resource="descriptor/12247-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71105">
    <skos:narrower rdf:resource="descriptor/18684-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71105">
    <skos:narrower rdf:resource="descriptor/13721-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71105">
    <skos:narrower rdf:resource="descriptor/18686-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71105">
    <skos:narrower rdf:resource="descriptor/18685-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71105">
    <skos:narrower rdf:resource="descriptor/12234-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15735-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14961-2">
    <skos:prefLabel xml:lang="en">Vegetable oil</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14961-2">
    <skos:prefLabel xml:lang="de">Pflanzenöl</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12038-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73351">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.10.01.03</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72394">
    <skos:narrower rdf:resource="descriptor/15350-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72394">
    <skos:narrower rdf:resource="descriptor/15405-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72394">
    <skos:narrower rdf:resource="descriptor/15336-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72394">
    <skos:narrower rdf:resource="descriptor/15330-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72394">
    <skos:narrower rdf:resource="descriptor/15399-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72394">
    <skos:narrower rdf:resource="descriptor/15398-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72394">
    <skos:narrower rdf:resource="descriptor/19542-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72394">
    <skos:narrower rdf:resource="descriptor/15397-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13235-1">
    <skos:prefLabel xml:lang="en">Sporting goods trade</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13235-1">
    <skos:prefLabel xml:lang="de">Sportartikelhandel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10548-2">
    <skos:broader rdf:resource="thsys/71085"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13199-0">
    <skos:prefLabel xml:lang="en">Drugstore</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13199-0">
    <skos:prefLabel xml:lang="de">Drogerie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15992-0">
    <skos:prefLabel xml:lang="en">Chemicals law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15992-0">
    <skos:prefLabel xml:lang="de">Chemikalienrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16502-3">
    <skos:altLabel xml:lang="de">Weltpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16502-3">
    <skos:altLabel xml:lang="de">Politische Geographie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19024-5">
    <skos:prefLabel xml:lang="en">Onion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19024-5">
    <skos:prefLabel xml:lang="de">Zwiebel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10436-6">
    <skos:related rdf:resource="descriptor/13349-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10436-6">
    <skos:related rdf:resource="descriptor/15685-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10436-6">
    <skos:related rdf:resource="descriptor/18070-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10436-6">
    <skos:related rdf:resource="descriptor/19584-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10436-6">
    <skos:related rdf:resource="descriptor/13343-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10436-6">
    <skos:related rdf:resource="descriptor/10437-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10436-6">
    <skos:related rdf:resource="descriptor/10475-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11483-6">
    <skos:broader rdf:resource="thsys/70305"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11483-6">
    <skos:broader rdf:resource="descriptor/11490-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13368-3">
    <skos:related rdf:resource="descriptor/13669-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16666-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18204-1">
    <skos:prefLabel xml:lang="en">Wood waste</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18204-1">
    <skos:prefLabel xml:lang="de">Holzabfall</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13570-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10564-4">
    <skos:broader rdf:resource="thsys/71096"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10564-4">
    <skos:broader rdf:resource="thsys/71085"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10564-4">
    <skos:broader rdf:resource="thsys/70979"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10564-4">
    <skos:broader rdf:resource="descriptor/10649-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10564-4">
    <skos:broader rdf:resource="descriptor/19310-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10564-4">
    <skos:broader rdf:resource="descriptor/18481-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19078-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14479-3">
    <skos:prefLabel xml:lang="en">Generator</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14479-3">
    <skos:prefLabel xml:lang="de">Generator</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11827-0">
    <skos:prefLabel xml:lang="en">Traffic calming</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11827-0">
    <skos:prefLabel xml:lang="de">Verkehrsberuhigung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14020-3">
    <skos:prefLabel xml:lang="en">Component</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14020-3">
    <skos:prefLabel xml:lang="de">Baufertigteil</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14814-1">
    <skos:prefLabel xml:lang="en">Map</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14814-1">
    <skos:prefLabel xml:lang="de">Landkarte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19519-6">
    <skos:altLabel xml:lang="de">Gewerbegeschichte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19519-6">
    <skos:altLabel xml:lang="de">Branchengeschichte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14164-0">
    <skos:broader rdf:resource="thsys/70506"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14164-0">
    <skos:broader rdf:resource="thsys/70403"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14164-0">
    <skos:broader rdf:resource="descriptor/14158-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70556">
    <skos:prefLabel xml:lang="de">P.12.02  Organische Chemikalien</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13055-3">
    <skos:prefLabel xml:lang="en">Truck industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13055-3">
    <skos:prefLabel xml:lang="de">Lastwagenbau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16185-3">
    <skos:altLabel xml:lang="en">PCT (Patent cooperation treaty)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16185-3">
    <skos:altLabel xml:lang="en">Patent cooperation treaty</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16185-3">
    <skos:altLabel xml:lang="de">Internationaler Patentschutz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11105-3">
    <skos:scopeNote xml:lang="de">vgl. auch: Systemtransformation.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11105-3">
    <skos:scopeNote xml:lang="de">Volkswirtschaften, die sich nach einer Systemtransformation im Übergang zu einer anderen Wirtschafts- und Gesellschaftsordnung befinden.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18356-6">
    <skos:altLabel xml:lang="de">Gaspipelineindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18356-6">
    <skos:altLabel xml:lang="de">Gasleitungsindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18356-6">
    <skos:altLabel xml:lang="de">Gasfernversorgung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18356-6">
    <skos:altLabel xml:lang="de">Erdgasleitungstransport</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11209-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10373-4">
    <skos:broader rdf:resource="thsys/70020"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10373-4">
    <skos:broader rdf:resource="descriptor/15075-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10373-4">
    <skos:broader rdf:resource="descriptor/19587-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10373-4">
    <skos:broader rdf:resource="descriptor/10361-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19568-0">
    <skos:related rdf:resource="descriptor/15243-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73366">
    <skos:prefLabel xml:lang="de">N.04.04  Internationale Beziehungen und Außenpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17990-1">
    <skos:prefLabel xml:lang="en">Latin America</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17990-1">
    <skos:prefLabel xml:lang="de">Lateinamerika</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19241-4">
    <skos:broader rdf:resource="thsys/70020"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13708-5">
    <skos:broader rdf:resource="thsys/70903"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13708-5">
    <skos:broader rdf:resource="thsys/70058"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13708-5">
    <skos:broader rdf:resource="descriptor/13283-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13156-4">
    <skos:prefLabel xml:lang="en">Animal feed industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13156-4">
    <skos:prefLabel xml:lang="de">Futtermittelindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10242-5">
    <skos:altLabel xml:lang="en">Demand-shift-inflation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10242-5">
    <skos:altLabel xml:lang="en">Demand-shift inflation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10242-5">
    <skos:altLabel xml:lang="en">Demand-pull-inflation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10242-5">
    <skos:altLabel xml:lang="en">Demand-pull inflation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10242-5">
    <skos:altLabel xml:lang="en">Cost-push-inflation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10242-5">
    <skos:altLabel xml:lang="en">Cost-push inflation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10242-5">
    <skos:altLabel xml:lang="de">Inflatorische Lücke</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10242-5">
    <skos:altLabel xml:lang="de">Anspruchsinflation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12648-2">
    <skos:broader rdf:resource="thsys/70368"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12648-2">
    <skos:broader rdf:resource="thsys/70301"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15640-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12251-1">
    <skos:related rdf:resource="descriptor/10022-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12251-1">
    <skos:related rdf:resource="descriptor/10019-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12251-1">
    <skos:related rdf:resource="descriptor/20553-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12251-1">
    <skos:related rdf:resource="descriptor/12253-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12251-1">
    <skos:related rdf:resource="descriptor/18725-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12251-1">
    <skos:related rdf:resource="descriptor/18178-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12251-1">
    <skos:related rdf:resource="descriptor/18696-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12251-1">
    <skos:related rdf:resource="descriptor/13777-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12251-1">
    <skos:related rdf:resource="descriptor/18698-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12251-1">
    <skos:related rdf:resource="descriptor/11433-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12251-1">
    <skos:related rdf:resource="descriptor/13801-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12251-1">
    <skos:related rdf:resource="descriptor/13776-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12251-1">
    <skos:related rdf:resource="descriptor/19088-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12251-1">
    <skos:related rdf:resource="descriptor/12274-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19127-2">
    <skos:altLabel xml:lang="de">Zahlungsklauseln</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12621-1">
    <skos:prefLabel xml:lang="en">Work organization</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12621-1">
    <skos:prefLabel xml:lang="de">Arbeitsorganisation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id162">
    <rdfs:seeAlso rdf:resource="descriptor/10121-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12244-5">
    <skos:altLabel xml:lang="de">Verschuldungsgrad des Betriebs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12244-5">
    <skos:altLabel xml:lang="de">Modigliani-Miller-Theorem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12244-5">
    <skos:altLabel xml:lang="de">Finanzierungsstruktur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12244-5">
    <skos:altLabel xml:lang="de">Finanzierungsregel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12244-5">
    <skos:altLabel xml:lang="de">Finanzierungsgrundsatz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19562-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73336">
    <rdfs:label xml:lang="en">Labour law</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73336">
    <rdfs:label xml:lang="de">Arbeitsrecht</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12226-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19707-5">
    <skos:broader rdf:resource="thsys/73369"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19707-5">
    <skos:broader rdf:resource="descriptor/16515-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10082-1">
    <skos:related rdf:resource="descriptor/10087-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10082-1">
    <skos:related rdf:resource="descriptor/10074-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18019-3">
    <skos:broader rdf:resource="thsys/70307"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17742-6">
    <skos:broader rdf:resource="thsys/71031"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17742-6">
    <skos:broader rdf:resource="descriptor/17743-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12270-4">
    <skos:broader rdf:resource="thsys/71043"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12270-4">
    <skos:broader rdf:resource="thsys/70036"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12270-4">
    <skos:broader rdf:resource="descriptor/12275-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12270-4">
    <skos:broader rdf:resource="descriptor/12273-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13090-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14024-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72182">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19598-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12955-0">
    <skos:related rdf:resource="descriptor/12957-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12955-0">
    <skos:related rdf:resource="descriptor/19347-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12955-0">
    <skos:related rdf:resource="descriptor/19627-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12955-0">
    <skos:related rdf:resource="descriptor/14129-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12955-0">
    <skos:related rdf:resource="descriptor/14169-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12955-0">
    <skos:related rdf:resource="descriptor/12962-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12955-0">
    <skos:related rdf:resource="descriptor/19292-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12955-0">
    <skos:related rdf:resource="descriptor/13156-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12955-0">
    <skos:related rdf:resource="descriptor/19340-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12955-0">
    <skos:related rdf:resource="descriptor/13155-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12955-0">
    <skos:related rdf:resource="descriptor/13940-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14593-5">
    <skos:related rdf:resource="descriptor/13095-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14593-5">
    <skos:related rdf:resource="descriptor/13231-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14593-5">
    <skos:related rdf:resource="descriptor/14161-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14593-5">
    <skos:related rdf:resource="descriptor/14227-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14593-5">
    <skos:related rdf:resource="descriptor/14296-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19059-0">
    <skos:altLabel xml:lang="de">Motorradteilehandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19059-0">
    <skos:altLabel xml:lang="de">Motorradreparatur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14972-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10041-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14123-0">
    <skos:altLabel xml:lang="de">Bienenhonig</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15798-3">
    <skos:narrower rdf:resource="descriptor/12171-6"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id40">
    <rdf:value xml:lang="de">Für Competitive-Bidding-Modelle und Bidding-Behaviour-Modelle</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18596-0">
    <skos:related rdf:resource="descriptor/18597-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70002">
    <rdfs:label xml:lang="en">Europe</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70002">
    <rdfs:label xml:lang="de">Europa</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10894-4">
    <skos:altLabel xml:lang="de">Wirtschaftsregulierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10894-4">
    <skos:altLabel xml:lang="de">Wettbewerbsregulierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10894-4">
    <skos:altLabel xml:lang="de">Staatliche Regulierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10894-4">
    <skos:altLabel xml:lang="de">Öffentliche Regulierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10894-4">
    <skos:altLabel xml:lang="de">Marktregulierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10894-4">
    <skos:altLabel xml:lang="de">Marktpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10894-4">
    <skos:altLabel xml:lang="de">Marktordnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10894-4">
    <skos:altLabel xml:lang="de">Dirigismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18775-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16544-1">
    <skos:prefLabel xml:lang="en">Social ethics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16544-1">
    <skos:prefLabel xml:lang="de">Sozialethik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12309-3">
    <skos:altLabel xml:lang="de">Budgetplanung im Betrieb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12309-3">
    <skos:altLabel xml:lang="de">Betriebliche Budgetplanung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16483-2">
    <skos:related rdf:resource="descriptor/18956-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16483-2">
    <skos:related rdf:resource="descriptor/15675-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16483-2">
    <skos:related rdf:resource="descriptor/16515-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16483-2">
    <skos:related rdf:resource="descriptor/16479-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19194-1">
    <skos:altLabel xml:lang="de">Missernte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19194-1">
    <skos:altLabel xml:lang="de">Mißernte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21110-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18236-2">
    <skos:altLabel xml:lang="en">LPG (Liquefied Petroleum Gas)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18236-2">
    <skos:altLabel xml:lang="en">LNG (Liquefied Natural Gas)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18236-2">
    <skos:altLabel xml:lang="en">Liquefied petroleum gas</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17472-2">
    <skos:broader rdf:resource="thsys/71032"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17472-2">
    <skos:broader rdf:resource="descriptor/17595-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17472-2">
    <skos:broader rdf:resource="descriptor/19078-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19023-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12376-2">
    <skos:altLabel xml:lang="de">Immaterieller Vermögenswert</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18518-6">
    <skos:related rdf:resource="descriptor/19294-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18518-6">
    <skos:related rdf:resource="descriptor/18519-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18518-6">
    <skos:related rdf:resource="descriptor/18192-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18518-6">
    <skos:related rdf:resource="descriptor/14064-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14946-5">
    <skos:broader rdf:resource="thsys/70342"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14946-5">
    <skos:broader rdf:resource="descriptor/14947-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15769-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11938-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18949-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11201-0">
    <skos:related rdf:resource="descriptor/15925-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11201-0">
    <skos:related rdf:resource="descriptor/11275-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11201-0">
    <skos:related rdf:resource="descriptor/16140-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11201-0">
    <skos:related rdf:resource="descriptor/19509-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12607-2">
    <skos:related rdf:resource="descriptor/12603-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12607-2">
    <skos:related rdf:resource="descriptor/12612-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12607-2">
    <skos:related rdf:resource="descriptor/11248-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12607-2">
    <skos:related rdf:resource="descriptor/12617-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12607-2">
    <skos:related rdf:resource="descriptor/19111-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12607-2">
    <skos:related rdf:resource="descriptor/12621-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70101">
    <skos:broader rdf:resource="thsys/v"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11055-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18572-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13415-6">
    <skos:related rdf:resource="descriptor/13371-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13415-6">
    <skos:related rdf:resource="descriptor/18063-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13415-6">
    <skos:related rdf:resource="descriptor/19517-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13415-6">
    <skos:related rdf:resource="descriptor/11388-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19127-2">
    <skos:prefLabel xml:lang="en">Terms of payment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19127-2">
    <skos:prefLabel xml:lang="de">Zahlungsbedingungen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11717-0">
    <skos:related rdf:resource="descriptor/11718-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13205-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12249-2">
    <skos:prefLabel xml:lang="en">Cost of capital</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12249-2">
    <skos:prefLabel xml:lang="de">Kapitalkosten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11507-4">
    <skos:altLabel xml:lang="de">Teuerungsrate</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11507-4">
    <skos:altLabel xml:lang="de">Preissteigerungsrate</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10320-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18084-6">
    <skos:related rdf:resource="descriptor/19137-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18084-6">
    <skos:related rdf:resource="descriptor/12228-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18783-1">
    <skos:related rdf:resource="descriptor/19799-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18783-1">
    <skos:related rdf:resource="descriptor/12649-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18783-1">
    <skos:related rdf:resource="descriptor/12647-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11708-1">
    <skos:altLabel xml:lang="de">Veranlagungsverfahren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11708-1">
    <skos:altLabel xml:lang="de">Tarifsteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11708-1">
    <skos:altLabel xml:lang="de">Steuerveranlagung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11708-1">
    <skos:altLabel xml:lang="de">Steuertechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11708-1">
    <skos:altLabel xml:lang="de">Steuerkontrollverfahren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11708-1">
    <skos:altLabel xml:lang="de">Steuererhebungsverfahren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11708-1">
    <skos:altLabel xml:lang="de">Steuereinschätzung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11708-1">
    <skos:altLabel xml:lang="de">Stempelsteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11708-1">
    <skos:altLabel xml:lang="de">Katastersteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70951">
    <skos:prefLabel xml:lang="en">W.10.04.01  Foreign trade company</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70951">
    <skos:prefLabel xml:lang="de">W.10.04.01  Außenhandelsbetriebslehre</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18177-6">
    <skos:altLabel xml:lang="en">Islamic finance</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18177-6">
    <skos:altLabel xml:lang="de">Islamisches Bankwesen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11772-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17968-1">
    <skos:altLabel xml:lang="de">Stiller Ozean</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17968-1">
    <skos:altLabel xml:lang="de">Pazifik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71118">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14634-3">
    <skos:prefLabel xml:lang="en">Hydrogen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14634-3">
    <skos:prefLabel xml:lang="de">Wasserstoff</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12917-1">
    <skos:related rdf:resource="descriptor/18479-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12917-1">
    <skos:related rdf:resource="descriptor/18993-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12917-1">
    <skos:related rdf:resource="descriptor/12910-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10649-3">
    <skos:broader rdf:resource="thsys/71100"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10649-3">
    <skos:broader rdf:resource="thsys/71096"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10649-3">
    <skos:broader rdf:resource="descriptor/10629-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15605-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70953">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.10.03</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16717-3">
    <skos:scopeNote xml:lang="de">Annahme, dass alles Wirkliche nur Materie ist oder von materiellen Vorgängen abgeleitet werden kann.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19975-1">
    <zbwext:useInsteadNote rdf:nodeID="id248"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19566-4">
    <skos:broader rdf:resource="thsys/70233"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19566-4">
    <skos:broader rdf:resource="descriptor/10011-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18415-2">
    <skos:prefLabel xml:lang="en">Packaging machinery industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18415-2">
    <skos:prefLabel xml:lang="de">Verpackungsmaschinenbau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12998-3">
    <skos:prefLabel xml:lang="en">Paint and varnish industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12998-3">
    <skos:prefLabel xml:lang="de">Lackindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70799">
    <skos:broader rdf:resource="thsys/70996"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71005">
    <rdfs:label xml:lang="en">Industries</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71005">
    <rdfs:label xml:lang="de">Industrie</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19262-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18215-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70880">
    <rdfs:label xml:lang="de">Datenbanken</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11042-1">
    <skos:broader rdf:resource="thsys/70171"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11042-1">
    <skos:broader rdf:resource="thsys/70055"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11042-1">
    <skos:broader rdf:resource="descriptor/11068-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13953-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70131">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16975-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10773-2">
    <skos:altLabel xml:lang="en">Foreign exchange rate</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10773-2">
    <skos:altLabel xml:lang="de">Währungskurs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10773-2">
    <skos:altLabel xml:lang="de">Devisenkurs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10773-2">
    <skos:altLabel xml:lang="de">Außenwert des Geldes</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11391-4">
    <skos:altLabel xml:lang="de">Fortbildung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11391-4">
    <skos:altLabel xml:lang="de">Berufsfortbildung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11391-4">
    <skos:altLabel xml:lang="de">Berufliche Fortbildung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19575-3">
    <skos:scopeNote xml:lang="de">Familie statistischer Testverfahren zur Prüfung spezifizierter stochastischer Prozesse auf Stationarität.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11151-3">
    <skos:altLabel xml:lang="de">Sowjetisierte Landwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11151-3">
    <skos:altLabel xml:lang="de">Kollektivierte Landwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10439-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14692-3">
    <skos:related rdf:resource="descriptor/14712-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17534-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10398-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11576-6">
    <skos:broader rdf:resource="thsys/70859"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11576-6">
    <skos:broader rdf:resource="thsys/70144"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11576-6">
    <skos:broader rdf:resource="descriptor/13470-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15126-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19734-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10043-4">
    <skos:broader rdf:resource="thsys/71100"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10043-4">
    <skos:broader rdf:resource="thsys/70234"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10043-4">
    <skos:broader rdf:resource="descriptor/10051-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10043-4">
    <skos:broader rdf:resource="descriptor/10042-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12749-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15992-0">
    <skos:related rdf:resource="descriptor/15769-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15992-0">
    <skos:related rdf:resource="descriptor/18288-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15992-0">
    <skos:related rdf:resource="descriptor/13617-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15992-0">
    <skos:related rdf:resource="descriptor/18613-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15992-0">
    <skos:related rdf:resource="descriptor/13198-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15992-0">
    <skos:related rdf:resource="descriptor/14599-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15992-0">
    <skos:related rdf:resource="descriptor/12996-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15992-0">
    <skos:related rdf:resource="descriptor/18259-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11782-3">
    <zbwext:useInsteadNote rdf:nodeID="id376"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11782-3">
    <zbwext:useInsteadNote rdf:nodeID="id375"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19511-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18593-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19859-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70342">
    <skos:narrower rdf:resource="descriptor/14948-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70342">
    <skos:narrower rdf:resource="descriptor/14980-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70342">
    <skos:narrower rdf:resource="descriptor/14947-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70342">
    <skos:narrower rdf:resource="descriptor/14946-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70342">
    <skos:narrower rdf:resource="descriptor/14949-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19325-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12805-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18312-5">
    <skos:prefLabel xml:lang="en">Fishery legislation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18312-5">
    <skos:prefLabel xml:lang="de">Fischereirecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11976-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18017-0">
    <skos:related rdf:resource="descriptor/19655-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18017-0">
    <skos:related rdf:resource="descriptor/12172-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18017-0">
    <skos:related rdf:resource="descriptor/16757-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18017-0">
    <skos:related rdf:resource="descriptor/13379-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18017-0">
    <skos:related rdf:resource="descriptor/12884-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18017-0">
    <skos:related rdf:resource="descriptor/12165-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16067-2">
    <skos:altLabel xml:lang="de">Veröffentlichungspflicht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16067-2">
    <skos:altLabel xml:lang="de">Publizitätsgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13601-2">
    <skos:altLabel xml:lang="de">Tarif im Güterverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13601-2">
    <skos:altLabel xml:lang="de">Gütertransportpreis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13601-2">
    <skos:altLabel xml:lang="de">Frachttarif</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13601-2">
    <skos:altLabel xml:lang="de">Frachtrate</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13601-2">
    <skos:altLabel xml:lang="de">Frachtpreis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13970-6">
    <skos:narrower rdf:resource="descriptor/13475-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13970-6">
    <skos:narrower rdf:resource="descriptor/13466-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16210-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13426-1">
    <skos:prefLabel xml:lang="en">Agricultural legislation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13426-1">
    <skos:prefLabel xml:lang="de">Agrarrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16336-1">
    <skos:narrower rdf:resource="descriptor/16322-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14096-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19229-1">
    <skos:related rdf:resource="descriptor/10252-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19229-1">
    <skos:related rdf:resource="descriptor/10250-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14502-6">
    <skos:prefLabel xml:lang="en">Household appliances</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14502-6">
    <skos:prefLabel xml:lang="de">Elektrohaushaltsgerät</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16340-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14902-4">
    <skos:altLabel xml:lang="de">Damenwäsche</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14902-4">
    <skos:altLabel xml:lang="de">Damenoberbekleidung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11347-0">
    <skos:prefLabel xml:lang="en">Conciliation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11347-0">
    <skos:prefLabel xml:lang="de">Schlichtung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11063-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18184-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17724-1">
    <skos:broader rdf:resource="thsys/70560"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17724-1">
    <skos:broader rdf:resource="descriptor/17652-0"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id194">
    <rdf:value xml:lang="de">Für Liberalismus in der Gestaltung des Wirtschaftslebens</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19527-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12106-3">
    <skos:altLabel xml:lang="en">Organisational research</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12106-3">
    <skos:altLabel xml:lang="de">Organisationswissenschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12106-3">
    <skos:altLabel xml:lang="de">Empirische Organisationsforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73361">
    <skos:broader rdf:resource="thsys/70160"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11049-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15023-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18921-1">
    <skos:scopeNote xml:lang="de">Ökonomische Analyse kultureller Aktivitäten.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16067-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18179-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10015-2">
    <skos:broader rdf:resource="thsys/71044"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14983-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13580-5">
    <skos:altLabel xml:lang="de">Wasser-Schiene-Wettbewerb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13580-5">
    <skos:altLabel xml:lang="de">Wasser-Luft-Wettbewerb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13580-5">
    <skos:altLabel xml:lang="de">Verkehrswettbewerb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13580-5">
    <skos:altLabel xml:lang="de">Straße-Schiene-Wettbewerb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13580-5">
    <skos:altLabel xml:lang="de">Schiene-Wasser-Wettbewerb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13580-5">
    <skos:altLabel xml:lang="de">Schiene-Straße-Wettbewerb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13580-5">
    <skos:altLabel xml:lang="de">Schiene-Luft-Wettbewerb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13580-5">
    <skos:altLabel xml:lang="de">Luft-Schiene-Wettbewerb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19640-4">
    <skos:broader rdf:resource="thsys/73361"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19640-4">
    <skos:broader rdf:resource="thsys/70572"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19640-4">
    <skos:broader rdf:resource="descriptor/15952-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12760-1">
    <skos:altLabel xml:lang="de">Prozeßsteuerung der Produktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12760-1">
    <skos:altLabel xml:lang="de">Prozesssteuerung der Produktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12760-1">
    <skos:altLabel xml:lang="de">Produktionsüberwachung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12760-1">
    <skos:altLabel xml:lang="de">Produktionssteuerung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12760-1">
    <skos:altLabel xml:lang="de">Produktionsregelung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12760-1">
    <skos:altLabel xml:lang="de">Produktionsprozeßsteuerung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12760-1">
    <skos:altLabel xml:lang="de">Produktionsprozesssteuerung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12760-1">
    <skos:altLabel xml:lang="de">Produktionslenkung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12760-1">
    <skos:altLabel xml:lang="de">Produktionskontrolle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12760-1">
    <skos:altLabel xml:lang="de">Fertigungslenkung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18488-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16521-6">
    <skos:scopeNote xml:lang="de">Sicherheit vor Aggression von außen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71030">
    <rdfs:label xml:lang="en">South America</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71030">
    <rdfs:label xml:lang="de">Südamerika</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13709-3">
    <skos:broader rdf:resource="thsys/70901"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13709-3">
    <skos:broader rdf:resource="descriptor/12281-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19047-0">
    <skos:prefLabel xml:lang="en">Oil production services</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19047-0">
    <skos:prefLabel xml:lang="de">Erdölförderservice</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19768-6">
    <skos:related rdf:resource="descriptor/15969-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19768-6">
    <skos:related rdf:resource="descriptor/10093-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19768-6">
    <skos:related rdf:resource="descriptor/15964-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19768-6">
    <skos:related rdf:resource="descriptor/15998-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19768-6">
    <skos:related rdf:resource="descriptor/16429-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19768-6">
    <skos:related rdf:resource="descriptor/16709-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13200-6">
    <skos:prefLabel xml:lang="en">Retail trade</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13200-6">
    <skos:prefLabel xml:lang="de">Einzelhandel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19187-5">
    <skos:broader rdf:resource="thsys/71029"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19187-5">
    <skos:broader rdf:resource="thsys/71020"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19187-5">
    <skos:broader rdf:resource="descriptor/11801-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19292-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10301-1">
    <skos:prefLabel xml:lang="en">Consumer good</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10301-1">
    <skos:prefLabel xml:lang="de">Konsumgut</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70256">
    <skos:narrower rdf:resource="descriptor/14423-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70256">
    <skos:narrower rdf:resource="descriptor/14422-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70256">
    <skos:narrower rdf:resource="descriptor/14421-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70256">
    <skos:narrower rdf:resource="descriptor/14420-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70256">
    <skos:narrower rdf:resource="descriptor/18246-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70256">
    <skos:narrower rdf:resource="descriptor/18297-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70256">
    <skos:narrower rdf:resource="descriptor/14418-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17660-1">
    <skos:broader rdf:resource="thsys/70445"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17660-1">
    <skos:broader rdf:resource="descriptor/17672-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18070-3">
    <skos:prefLabel xml:lang="en">Creativity</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18070-3">
    <skos:prefLabel xml:lang="de">Kreativität</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15914-6">
    <skos:related rdf:resource="descriptor/16597-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15914-6">
    <skos:related rdf:resource="descriptor/15945-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15914-6">
    <skos:related rdf:resource="descriptor/15061-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15914-6">
    <skos:related rdf:resource="descriptor/15708-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15914-6">
    <skos:related rdf:resource="descriptor/15909-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12491-2">
    <skos:related rdf:resource="descriptor/12509-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12491-2">
    <skos:related rdf:resource="descriptor/12514-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13960-2">
    <skos:broader rdf:resource="thsys/71056"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13960-2">
    <skos:broader rdf:resource="thsys/71052"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13960-2">
    <skos:broader rdf:resource="descriptor/13962-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13960-2">
    <skos:broader rdf:resource="descriptor/11356-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18356-6">
    <skos:broader rdf:resource="thsys/71016"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18356-6">
    <skos:broader rdf:resource="thsys/70940"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18356-6">
    <skos:broader rdf:resource="descriptor/13245-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18356-6">
    <skos:broader rdf:resource="descriptor/12978-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19450-2">
    <skos:prefLabel xml:lang="en">Regional planning programme</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19450-2">
    <skos:prefLabel xml:lang="de">Raumordnungsplan</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19849-6">
    <skos:related rdf:resource="descriptor/15981-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19849-6">
    <skos:related rdf:resource="descriptor/19213-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19849-6">
    <skos:related rdf:resource="descriptor/19852-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15925-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13246-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18172-2">
    <skos:prefLabel xml:lang="en">Data model</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18172-2">
    <skos:prefLabel xml:lang="de">Datenmodell</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13283-4">
    <skos:altLabel xml:lang="de">Kreditwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13283-4">
    <skos:altLabel xml:lang="de">Kreditwesen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13283-4">
    <skos:altLabel xml:lang="de">Kreditinstitut</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13283-4">
    <skos:altLabel xml:lang="de">Kreditbank</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13283-4">
    <skos:altLabel xml:lang="de">Geschäftsbank</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13283-4">
    <skos:altLabel xml:lang="de">Geldinstitut</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13283-4">
    <skos:altLabel xml:lang="de">Bankwesen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13283-4">
    <skos:altLabel xml:lang="de">Bankgewerbe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10194-4">
    <skos:prefLabel xml:lang="en">Asymmetric information</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10194-4">
    <skos:prefLabel xml:lang="de">Asymmetrische Information</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15441-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10561-3">
    <skos:broader rdf:resource="thsys/71085"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10561-3">
    <skos:broader rdf:resource="thsys/71084"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10561-3">
    <skos:broader rdf:resource="descriptor/10523-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15842-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14805-2">
    <skos:broader rdf:resource="thsys/70328"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12794-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18876-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12170-1">
    <skos:prefLabel xml:lang="en">Information value</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12170-1">
    <skos:prefLabel xml:lang="de">Informationswert</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18154-4">
    <skos:broader rdf:resource="thsys/71075"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18154-4">
    <skos:broader rdf:resource="descriptor/11708-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13984-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18746-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18602-3">
    <skos:prefLabel xml:lang="en">Harbour law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18602-3">
    <skos:prefLabel xml:lang="de">Hafenrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10089-1">
    <skos:broader rdf:resource="thsys/70274"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10089-1">
    <skos:broader rdf:resource="thsys/70057"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10089-1">
    <skos:broader rdf:resource="descriptor/10975-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18232-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13483-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19608-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18243-5">
    <skos:prefLabel xml:lang="en">Metal powder</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18243-5">
    <skos:prefLabel xml:lang="de">Pulvermetallurgischer Werkstoff</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70459">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16142-0">
    <skos:prefLabel xml:lang="en">Dismissal protection</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16142-0">
    <skos:prefLabel xml:lang="de">Kündigungsschutz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18488-3">
    <skos:related rdf:resource="descriptor/12913-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70142">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.03</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15676-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20289-1">
    <skos:related rdf:resource="descriptor/15706-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20289-1">
    <skos:related rdf:resource="descriptor/11090-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13228-5">
    <skos:altLabel xml:lang="de">Videorecorder-Leasing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13228-5">
    <skos:altLabel xml:lang="de">UE-Fachhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13228-5">
    <skos:altLabel xml:lang="de">Radiofachhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13228-5">
    <skos:altLabel xml:lang="de">Fernsehgeräte-Leasing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13228-5">
    <skos:altLabel xml:lang="de">Fernsehfachhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17799-5">
    <skos:altLabel xml:lang="en">Saint Kitts and Nevis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17799-5">
    <skos:altLabel xml:lang="de">St. Christoph und Nevis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17799-5">
    <skos:altLabel xml:lang="de">Föderation St. Kitts und Nevis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17799-5">
    <skos:altLabel xml:lang="de">Föderation St. Christoph und Nevis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16437-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19633-1">
    <skos:related rdf:resource="descriptor/15074-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19633-1">
    <skos:related rdf:resource="descriptor/15767-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19633-1">
    <skos:related rdf:resource="descriptor/18165-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19633-1">
    <skos:related rdf:resource="descriptor/18183-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18154-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16131-5">
    <skos:prefLabel xml:lang="en">Labour contract</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16131-5">
    <skos:prefLabel xml:lang="de">Arbeitsvertrag</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70004">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">B.03.03</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13590-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12778-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13397-3">
    <skos:altLabel xml:lang="de">Parteitheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13397-3">
    <skos:altLabel xml:lang="de">Parteientheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13397-3">
    <skos:altLabel xml:lang="de">Partei</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18402-4">
    <skos:scopeNote xml:lang="de">Wissenschaft und Technologie vom Ausschmelzen der Metalle aus den Erzen, ihrer Raffination und Weiterverarbeitung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16788-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10667-1">
    <skos:altLabel xml:lang="de">Internationale Inflationsübertragung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10667-1">
    <skos:altLabel xml:lang="de">Inflationsimport</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10667-1">
    <skos:altLabel xml:lang="de">Importierte Inflation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13903-0">
    <skos:broader rdf:resource="thsys/70968"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13903-0">
    <skos:broader rdf:resource="thsys/70964"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13903-0">
    <skos:broader rdf:resource="descriptor/12910-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19555-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18332-6">
    <skos:related rdf:resource="descriptor/18341-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11243-5">
    <skos:related rdf:resource="descriptor/19508-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11243-5">
    <skos:related rdf:resource="descriptor/11233-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11243-5">
    <skos:related rdf:resource="descriptor/19372-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11243-5">
    <skos:related rdf:resource="descriptor/11241-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11243-5">
    <skos:related rdf:resource="descriptor/19773-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19271-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13804-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10328-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14927-2">
    <skos:altLabel xml:lang="de">Tütensuppe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14927-2">
    <skos:altLabel xml:lang="de">Fertigsuppe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14927-2">
    <skos:altLabel xml:lang="de">Fertignahrung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14927-2">
    <skos:altLabel xml:lang="de">Dosensuppe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12469-2">
    <skos:related rdf:resource="descriptor/12489-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19845-0">
    <skos:scopeNote xml:lang="de">Für alle Regelungen zur Auskunftspflicht des Verwaltungs- und Privatrechts.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19198-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11610-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12087-2">
    <skos:broader rdf:resource="thsys/70485"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12087-2">
    <skos:broader rdf:resource="descriptor/18143-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13334-6">
    <skos:prefLabel xml:lang="en">Local waste water management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13334-6">
    <skos:prefLabel xml:lang="de">Kommunale Abwasserwirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15806-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70151">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18810-3">
    <skos:prefLabel xml:lang="en">Regulatory authority</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18810-3">
    <skos:prefLabel xml:lang="de">Aufsichtsbehörde</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18585-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18621-6">
    <skos:broader rdf:resource="thsys/70930"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18621-6">
    <skos:broader rdf:resource="descriptor/18615-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12847-3">
    <skos:scopeNote xml:lang="de">Werbung in Zeitschriften und Zeitungen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13296-2">
    <skos:related rdf:resource="descriptor/13828-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16051-3">
    <skos:related rdf:resource="descriptor/10942-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16051-3">
    <skos:related rdf:resource="descriptor/10958-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16051-3">
    <skos:related rdf:resource="descriptor/10923-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13037-5">
    <skos:narrower rdf:resource="descriptor/13033-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13037-5">
    <skos:narrower rdf:resource="descriptor/13026-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13037-5">
    <skos:narrower rdf:resource="descriptor/13025-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12940-6">
    <skos:narrower rdf:resource="descriptor/12960-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12940-6">
    <skos:narrower rdf:resource="descriptor/12947-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12940-6">
    <skos:narrower rdf:resource="descriptor/12942-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12940-6">
    <skos:narrower rdf:resource="descriptor/12935-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19810-5">
    <skos:altLabel xml:lang="de">Zugriffskontrolle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19810-5">
    <skos:altLabel xml:lang="de">Zugangskontrolle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19810-5">
    <skos:altLabel xml:lang="de">Elektronische Zutrittskontrolle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19810-5">
    <skos:altLabel xml:lang="de">Berechtigungskontrolle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id310">
    <rdf:value xml:lang="de">Für Standortbetrachtungen aus betriebswirtschaftlicher Sicht</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11531-0">
    <skos:prefLabel xml:lang="en">Benefit analysis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11531-0">
    <skos:prefLabel xml:lang="de">Nutzwertanalyse</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15713-2">
    <skos:broader rdf:resource="thsys/73395"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15713-2">
    <skos:broader rdf:resource="thsys/73356"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13411-0">
    <skos:altLabel xml:lang="en">Workers compensation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13411-0">
    <skos:altLabel xml:lang="en">Workers' compensation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13411-0">
    <skos:altLabel xml:lang="de">Private Unfallversicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13411-0">
    <skos:altLabel xml:lang="de">Gesetzliche Unfallversicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13411-0">
    <skos:altLabel xml:lang="de">Berufsunfallversicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13411-0">
    <skos:altLabel xml:lang="de">Berufsgenossenschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13411-0">
    <skos:altLabel xml:lang="de">Arbeitsunfallversicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10215-1">
    <skos:broader rdf:resource="thsys/70178"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10215-1">
    <skos:broader rdf:resource="descriptor/10217-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10215-1">
    <skos:broader rdf:resource="descriptor/12794-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18612-0">
    <skos:prefLabel xml:lang="en">Business tourism</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18612-0">
    <skos:prefLabel xml:lang="de">Geschäftsreiseverkehr</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17433-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13709-3">
    <skos:scopeNote xml:lang="de">Investmentfonds für festverzinsliche Wertpapiere.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15381-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19568-0">
    <skos:prefLabel xml:lang="en">Resampling method</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19568-0">
    <skos:prefLabel xml:lang="de">Resampling</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10811-6">
    <skos:related rdf:resource="descriptor/10725-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10811-6">
    <skos:related rdf:resource="descriptor/19321-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10811-6">
    <skos:related rdf:resource="descriptor/10786-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10811-6">
    <skos:related rdf:resource="descriptor/10124-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10811-6">
    <skos:related rdf:resource="descriptor/10807-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10811-6">
    <skos:related rdf:resource="descriptor/10797-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10811-6">
    <skos:related rdf:resource="descriptor/10798-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15555-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18233-1">
    <skos:broader rdf:resource="thsys/70648"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18233-1">
    <skos:broader rdf:resource="descriptor/14188-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13000-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10137-2">
    <skos:narrower rdf:resource="descriptor/10136-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10137-2">
    <skos:narrower rdf:resource="descriptor/19258-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19480-0">
    <skos:prefLabel xml:lang="en">International environmental agreement</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19480-0">
    <skos:prefLabel xml:lang="de">Umweltabkommen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10994-0">
    <skos:broader rdf:resource="thsys/70038"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10994-0">
    <skos:broader rdf:resource="descriptor/16283-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10821-3">
    <skos:related rdf:resource="descriptor/10773-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10821-3">
    <skos:related rdf:resource="descriptor/18699-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10821-3">
    <skos:related rdf:resource="descriptor/18539-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10821-3">
    <skos:related rdf:resource="descriptor/10803-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10821-3">
    <skos:related rdf:resource="descriptor/13720-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10821-3">
    <skos:related rdf:resource="descriptor/10781-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10821-3">
    <skos:related rdf:resource="descriptor/10858-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14652-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19934-1">
    <skos:prefLabel xml:lang="en">Book value</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19934-1">
    <skos:prefLabel xml:lang="de">Buchwert</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16136-2">
    <skos:scopeNote xml:lang="de">Kollektive arbeitsrechtliche Grundordnung für die betriebliche Zusammenarbeit zwischen Arbeitgebern und Arbeitnehmern.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19699-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12275-1">
    <skos:prefLabel xml:lang="en">Profit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12275-1">
    <skos:prefLabel xml:lang="de">Gewinn</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12391-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11564-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19366-5">
    <skos:related rdf:resource="descriptor/14151-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19366-5">
    <skos:related rdf:resource="descriptor/13004-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18841-6">
    <skos:related rdf:resource="descriptor/16031-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18841-6">
    <skos:related rdf:resource="descriptor/18839-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18841-6">
    <skos:related rdf:resource="descriptor/18838-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18841-6">
    <skos:related rdf:resource="descriptor/12180-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19657-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10265-0">
    <skos:prefLabel xml:lang="en">National product</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10265-0">
    <skos:prefLabel xml:lang="de">Sozialprodukt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71024">
    <rdfs:label xml:lang="de">Landwirtschaftliche Betriebssysteme</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19181-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18959-4">
    <skos:prefLabel xml:lang="en">Waste prevention</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18959-4">
    <skos:prefLabel xml:lang="de">Abfallvermeidung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19360-3">
    <skos:prefLabel xml:lang="en">Migrants' remittances</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19360-3">
    <skos:prefLabel xml:lang="de">Rücküberweisung (Migranten)</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19565-6">
    <skos:related rdf:resource="descriptor/10895-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19565-6">
    <skos:related rdf:resource="descriptor/12770-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19565-6">
    <skos:related rdf:resource="descriptor/10926-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19565-6">
    <skos:related rdf:resource="descriptor/19672-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10347-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18988-4">
    <skos:broader rdf:resource="thsys/71019"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18988-4">
    <skos:broader rdf:resource="descriptor/18339-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18988-4">
    <skos:broader rdf:resource="descriptor/12041-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14053-2">
    <skos:broader rdf:resource="thsys/71015"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14053-2">
    <skos:broader rdf:resource="thsys/71014"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14053-2">
    <skos:broader rdf:resource="descriptor/14039-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11281-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20301-2">
    <skos:scopeNote xml:lang="de">Realoptionen sind Optionen, die sich bei Realinvestitionen ergeben, wenn die Investitionsentscheidung zeitlich flexibel ist, z.B. in Erwartung zusätzlicher Informationen. Mithilfe der Optionspreistheorie soll diese Flexibilität bewertet und der optimale Zeitpunkt der Investition bestimmt werden.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18914-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19308-5">
    <skos:prefLabel xml:lang="en">Offshore assembly</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19308-5">
    <skos:prefLabel xml:lang="de">Lohnfertigung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16143-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15627-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10246-4">
    <skos:related rdf:resource="descriptor/19222-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10246-4">
    <skos:related rdf:resource="descriptor/10237-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11973-3">
    <skos:altLabel xml:lang="de">Vorort</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11973-3">
    <skos:altLabel xml:lang="de">Stadtviertel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11973-3">
    <skos:altLabel xml:lang="de">Stadtbezirk</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14198-4">
    <skos:broader rdf:resource="thsys/70418"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14198-4">
    <skos:broader rdf:resource="descriptor/14236-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14198-4">
    <skos:broader rdf:resource="descriptor/14192-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12490-4">
    <skos:related rdf:resource="descriptor/12535-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12490-4">
    <skos:related rdf:resource="descriptor/12533-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12490-4">
    <skos:related rdf:resource="descriptor/12492-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16984-1">
    <skos:prefLabel xml:lang="en">Iceland</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16984-1">
    <skos:prefLabel xml:lang="de">Island</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17013-6">
    <skos:altLabel xml:lang="de">Republik Malta</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12154-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10087-5">
    <skos:altLabel xml:lang="en">New Keynesian economics</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10087-5">
    <skos:altLabel xml:lang="en">Disequilibrium macroeconomics</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10087-5">
    <skos:altLabel xml:lang="de">Neue keynesianische Makroökonomik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10087-5">
    <skos:altLabel xml:lang="de">Neokeynesianismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10087-5">
    <skos:altLabel xml:lang="de">Neokeynesianische Theorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10087-5">
    <skos:altLabel xml:lang="de">Monetäre Ungleichgewichtstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10087-5">
    <skos:altLabel xml:lang="de">Barro-Grossman-Modell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10087-5">
    <skos:altLabel xml:lang="de">Barro Grossman Modell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12091-4">
    <skos:prefLabel xml:lang="en">Organization</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12091-4">
    <skos:prefLabel xml:lang="de">Organisation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19760-1">
    <skos:related rdf:resource="descriptor/15949-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19760-1">
    <skos:related rdf:resource="descriptor/19761-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19760-1">
    <skos:related rdf:resource="descriptor/15925-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19760-1">
    <skos:related rdf:resource="descriptor/15924-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19760-1">
    <skos:related rdf:resource="descriptor/18861-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19760-1">
    <skos:related rdf:resource="descriptor/15905-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19760-1">
    <skos:related rdf:resource="descriptor/16526-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19760-1">
    <skos:related rdf:resource="descriptor/15904-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12689-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16742-4">
    <skos:scopeNote xml:lang="de">Nur für die romantische Schule in der Wirtschaftstheorie.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15668-2">
    <skos:prefLabel xml:lang="en">Equestrian sport</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15668-2">
    <skos:prefLabel xml:lang="de">Pferdesport</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10880-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13504-0">
    <skos:related rdf:resource="descriptor/12761-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13504-0">
    <skos:related rdf:resource="descriptor/18566-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13543-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14261-2">
    <skos:broader rdf:resource="thsys/70346"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14261-2">
    <skos:broader rdf:resource="descriptor/14269-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12072-1">
    <skos:related rdf:resource="descriptor/13992-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12072-1">
    <skos:related rdf:resource="descriptor/12069-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12072-1">
    <skos:related rdf:resource="descriptor/10190-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12072-1">
    <skos:related rdf:resource="descriptor/12194-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12252-6">
    <skos:altLabel xml:lang="en">Collateral lending</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12252-6">
    <skos:altLabel xml:lang="de">Verpfändung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12252-6">
    <skos:altLabel xml:lang="de">Sicherungsübereignung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12252-6">
    <skos:altLabel xml:lang="de">Sicherungseigentum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12252-6">
    <skos:altLabel xml:lang="de">Recht der Kreditsicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12252-6">
    <skos:altLabel xml:lang="de">Pfandrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12252-6">
    <skos:altLabel xml:lang="de">Pfand</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12252-6">
    <skos:altLabel xml:lang="de">Kreditsicherungsrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12252-6">
    <skos:altLabel xml:lang="de">Kreditsicherheit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12252-6">
    <skos:altLabel xml:lang="de">Kreditschutz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12252-6">
    <skos:altLabel xml:lang="de">Kreditauskunft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12252-6">
    <skos:altLabel xml:lang="de">Eigentumsvorbehalt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12296-0">
    <skos:related rdf:resource="descriptor/11587-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12296-0">
    <skos:related rdf:resource="descriptor/12277-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12296-0">
    <skos:related rdf:resource="descriptor/12217-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13066-5">
    <skos:prefLabel xml:lang="en">Shipbuilding</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13066-5">
    <skos:prefLabel xml:lang="de">Schiffbau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10426-2">
    <skos:related rdf:resource="descriptor/11943-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10426-2">
    <skos:related rdf:resource="descriptor/11929-6"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id249">
    <rdf:value xml:lang="de">Für den Entscheidungsprozess vor der Personaleinstellung</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12178-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20522-0">
    <skos:altLabel xml:lang="en">Enterprise regulation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20522-0">
    <skos:altLabel xml:lang="de">Reguliertes Unternehmen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19221-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12844-2">
    <skos:prefLabel xml:lang="en">Advertising strategy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12844-2">
    <skos:prefLabel xml:lang="de">Werbeplanung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17175-1">
    <skos:altLabel xml:lang="x-other">Balgarija</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17175-1">
    <skos:altLabel xml:lang="de">Volksrepublik Bulgarien</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17175-1">
    <skos:altLabel xml:lang="de">Republik Bulgarien</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13213-4">
    <skos:prefLabel xml:lang="en">Household goods trade</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13213-4">
    <skos:prefLabel xml:lang="de">Haushaltswarenhandel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18656-1">
    <skos:prefLabel xml:lang="en">Satellite television</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18656-1">
    <skos:prefLabel xml:lang="de">Satellitenfernsehen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16739-0">
    <skos:prefLabel xml:lang="en">Marginalism</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16739-0">
    <skos:prefLabel xml:lang="de">Grenznutzentheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10282-0">
    <skos:altLabel xml:lang="en">Ratchet effect</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10282-0">
    <skos:altLabel xml:lang="en">Permanent income hypothesis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10282-0">
    <skos:altLabel xml:lang="en">Habit-persistance-hypothesis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10282-0">
    <skos:altLabel xml:lang="de">Relative Einkommenshypothese</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10282-0">
    <skos:altLabel xml:lang="de">Permanente Einkommenshypothese</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10282-0">
    <skos:altLabel xml:lang="de">Lebenszeit-Einkommenshypothese</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10282-0">
    <skos:altLabel xml:lang="de">Habit-Persistence-Hypothese</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10282-0">
    <skos:altLabel xml:lang="de">Duesenberry-Hypothese</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10282-0">
    <skos:altLabel xml:lang="de">Absolute Einkommenshypothese</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18284-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70959">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.10.01.03</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19634-6">
    <skos:prefLabel xml:lang="en">Archaeology</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19634-6">
    <skos:prefLabel xml:lang="de">Archäologie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11852-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18848-6">
    <skos:broader rdf:resource="thsys/71062"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18848-6">
    <skos:broader rdf:resource="descriptor/11336-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70922">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18186-5">
    <skos:related rdf:resource="descriptor/15769-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18186-5">
    <skos:related rdf:resource="descriptor/18165-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18186-5">
    <skos:related rdf:resource="descriptor/15766-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18186-5">
    <skos:related rdf:resource="descriptor/18740-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18186-5">
    <skos:related rdf:resource="descriptor/15763-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18186-5">
    <skos:related rdf:resource="descriptor/18030-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16265-5">
    <skos:broader rdf:resource="thsys/73386"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16265-5">
    <skos:broader rdf:resource="thsys/73376"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16265-5">
    <skos:broader rdf:resource="thsys/70485"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16265-5">
    <skos:broader rdf:resource="thsys/70384"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16265-5">
    <skos:broader rdf:resource="thsys/70292"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16543-3">
    <skos:related rdf:resource="descriptor/16523-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16543-3">
    <skos:related rdf:resource="descriptor/13405-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16543-3">
    <skos:related rdf:resource="descriptor/13419-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16543-3">
    <skos:related rdf:resource="descriptor/16545-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16543-3">
    <skos:related rdf:resource="descriptor/15674-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16543-3">
    <skos:related rdf:resource="descriptor/18906-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16543-3">
    <skos:related rdf:resource="descriptor/14014-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16543-3">
    <skos:related rdf:resource="descriptor/18864-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14431-3">
    <skos:related rdf:resource="descriptor/14473-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14431-3">
    <skos:related rdf:resource="descriptor/18359-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14999-5">
    <skos:altLabel xml:lang="de">Zigarillo</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10092-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14274-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19391-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12407-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70578">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">B.03.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17770-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10147-6">
    <skos:narrower rdf:resource="descriptor/10143-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11318-0">
    <skos:altLabel xml:lang="de">Vergütung im öffentlichen Dienst</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11318-0">
    <skos:altLabel xml:lang="de">Personalausgaben des öffentliches Dienstes</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11318-0">
    <skos:altLabel xml:lang="de">Öffentliche Personalausgaben</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11318-0">
    <skos:altLabel xml:lang="de">Beamtenbezüge</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11318-0">
    <skos:altLabel xml:lang="de">Beamtenbesoldung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19799-2">
    <skos:prefLabel xml:lang="en">Materials technology</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19799-2">
    <skos:prefLabel xml:lang="de">Werkstofftechnik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14190-6">
    <skos:broader rdf:resource="thsys/70418"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14190-6">
    <skos:broader rdf:resource="descriptor/14204-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12594-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13412-5">
    <skos:altLabel xml:lang="de">Wehrmacht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13412-5">
    <skos:altLabel xml:lang="de">Verteidigungsstreitkräfte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13412-5">
    <skos:altLabel xml:lang="de">Streitkräfte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13412-5">
    <skos:altLabel xml:lang="de">Landesverteidigung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13412-5">
    <skos:altLabel xml:lang="de">Heer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13412-5">
    <skos:altLabel xml:lang="de">Bundeswehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13412-5">
    <skos:altLabel xml:lang="de">Armee</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#narrower">
    <rdfs:comment xml:lang="en">Narrower concepts are typically rendered as children in a concept
      hierarchy (tree).</rdfs:comment>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#narrower">
    <rdfs:comment xml:lang="en">By convention, skos:broader is only used to assert an immediate
      (i.e. direct) hierarchical link between two conceptual resources.</rdfs:comment>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19493-5">
    <skos:related rdf:resource="descriptor/19491-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19493-5">
    <skos:related rdf:resource="descriptor/11239-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19493-5">
    <skos:related rdf:resource="descriptor/11248-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19493-5">
    <skos:related rdf:resource="descriptor/11235-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19493-5">
    <skos:related rdf:resource="descriptor/19767-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19493-5">
    <skos:related rdf:resource="descriptor/11183-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19493-5">
    <skos:related rdf:resource="descriptor/11314-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11261-3">
    <skos:related rdf:resource="descriptor/16527-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11261-3">
    <skos:related rdf:resource="descriptor/16526-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10001-6">
    <skos:related rdf:resource="descriptor/18188-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10001-6">
    <skos:related rdf:resource="descriptor/19266-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10001-6">
    <skos:related rdf:resource="descriptor/10135-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10001-6">
    <skos:related rdf:resource="descriptor/10773-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10001-6">
    <skos:related rdf:resource="descriptor/13715-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19377-0">
    <skos:prefLabel xml:lang="en">Local public good</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19377-0">
    <skos:prefLabel xml:lang="de">Lokales öffentliches Gut</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12947-6">
    <skos:broader rdf:resource="thsys/71026"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12947-6">
    <skos:broader rdf:resource="descriptor/12940-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11064-5">
    <skos:altLabel xml:lang="en">Exchange economy</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11064-5">
    <skos:altLabel xml:lang="de">Warengeld</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11064-5">
    <skos:altLabel xml:lang="de">Tauschhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11064-5">
    <skos:altLabel xml:lang="de">Naturalwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70471">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13613-2">
    <skos:related rdf:resource="descriptor/18576-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13613-2">
    <skos:related rdf:resource="descriptor/11876-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13613-2">
    <skos:related rdf:resource="descriptor/13249-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13613-2">
    <skos:related rdf:resource="descriptor/18610-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70368">
    <skos:narrower rdf:resource="descriptor/12655-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70368">
    <skos:narrower rdf:resource="descriptor/19113-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70368">
    <skos:narrower rdf:resource="descriptor/12648-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70368">
    <skos:narrower rdf:resource="descriptor/12667-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70368">
    <skos:narrower rdf:resource="descriptor/12669-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70368">
    <skos:narrower rdf:resource="descriptor/12665-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70368">
    <skos:narrower rdf:resource="descriptor/18104-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70368">
    <skos:narrower rdf:resource="descriptor/20503-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70368">
    <skos:narrower rdf:resource="descriptor/20442-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70368">
    <skos:narrower rdf:resource="descriptor/12656-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70368">
    <skos:narrower rdf:resource="descriptor/12651-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70368">
    <skos:narrower rdf:resource="descriptor/12640-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14602-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15107-0">
    <skos:broader rdf:resource="thsys/72181"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15107-0">
    <skos:broader rdf:resource="descriptor/15076-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11612-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16424-4">
    <skos:related rdf:resource="descriptor/15648-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16424-4">
    <skos:related rdf:resource="descriptor/15923-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16424-4">
    <skos:related rdf:resource="descriptor/16400-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16212-5">
    <skos:narrower rdf:resource="descriptor/18823-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70238">
    <skos:broader rdf:resource="thsys/70209"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14487-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19215-5">
    <skos:related rdf:resource="descriptor/15982-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19215-5">
    <skos:related rdf:resource="descriptor/24622-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70833">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18223-4">
    <skos:broader rdf:resource="thsys/70337"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18223-4">
    <skos:broader rdf:resource="thsys/70254"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18223-4">
    <skos:broader rdf:resource="descriptor/15005-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18223-4">
    <skos:broader rdf:resource="descriptor/18103-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19698-1">
    <skos:prefLabel xml:lang="en">Presidential election</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19698-1">
    <skos:prefLabel xml:lang="de">Präsidentschaftswahl</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16720-0">
    <skos:prefLabel xml:lang="en">Positivism</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16720-0">
    <skos:prefLabel xml:lang="de">Positivismus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14279-4">
    <skos:broader rdf:resource="thsys/70358"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14279-4">
    <skos:broader rdf:resource="descriptor/14285-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10452-1">
    <zbwext:useInsteadNote rdf:nodeID="id152"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13708-5">
    <skos:altLabel xml:lang="de">Regionales Kreditinstitut</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13316-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15769-3">
    <skos:broader rdf:resource="thsys/73360"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15769-3">
    <skos:broader rdf:resource="thsys/70815"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15769-3">
    <skos:broader rdf:resource="thsys/70239"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15769-3">
    <skos:broader rdf:resource="descriptor/16019-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19379-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16325-6">
    <skos:related rdf:resource="descriptor/14422-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16196-5">
    <skos:altLabel xml:lang="de">Viktimologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16196-5">
    <skos:altLabel xml:lang="de">Victimologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16196-5">
    <skos:altLabel xml:lang="de">Kriminalsoziologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="http://purl.org/dc/terms/publisher">
    <rdf:type rdf:resource="http://www.w3.org/2002/07/owl#DatatypeProperty"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10975-4">
    <skos:narrower rdf:resource="descriptor/11057-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10975-4">
    <skos:narrower rdf:resource="descriptor/18828-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10975-4">
    <skos:narrower rdf:resource="descriptor/10107-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10975-4">
    <skos:narrower rdf:resource="descriptor/10106-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10975-4">
    <skos:narrower rdf:resource="descriptor/18155-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10975-4">
    <skos:narrower rdf:resource="descriptor/10353-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10975-4">
    <skos:narrower rdf:resource="descriptor/11167-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10975-4">
    <skos:narrower rdf:resource="descriptor/11160-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10975-4">
    <skos:narrower rdf:resource="descriptor/19212-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10975-4">
    <skos:narrower rdf:resource="descriptor/11168-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10975-4">
    <skos:narrower rdf:resource="descriptor/11065-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10975-4">
    <skos:narrower rdf:resource="descriptor/11038-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10975-4">
    <skos:narrower rdf:resource="descriptor/11152-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10975-4">
    <skos:narrower rdf:resource="descriptor/10089-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11298-1">
    <skos:broader rdf:resource="thsys/71058"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19548-6">
    <skos:broader rdf:resource="thsys/70485"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19548-6">
    <skos:broader rdf:resource="descriptor/18029-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16927-6">
    <skos:altLabel xml:lang="de">Sowjetzone</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16927-6">
    <skos:altLabel xml:lang="de">SBZ (Sowjetisch-besetzte Zone)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16927-6">
    <skos:altLabel xml:lang="de">Russische Zone Deutschlands</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16927-6">
    <skos:altLabel xml:lang="de">Deutsche Demokratische Republik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12356-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18454-6">
    <skos:altLabel xml:lang="de">Büroservice</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18454-6">
    <skos:altLabel xml:lang="de">Bürohandwerk</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18454-6">
    <skos:altLabel xml:lang="de">Bürofachhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18454-6">
    <skos:altLabel xml:lang="de">Büroausstattungsgewerbe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71029">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18194-6">
    <skos:broader rdf:resource="thsys/73332"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18194-6">
    <skos:broader rdf:resource="thsys/71010"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18194-6">
    <skos:broader rdf:resource="descriptor/10593-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15126-3">
    <skos:prefLabel xml:lang="en">Cross-section analysis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15126-3">
    <skos:prefLabel xml:lang="de">Querschnittsanalyse</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14958-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16070-6">
    <skos:prefLabel xml:lang="en">Merchant shipping law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16070-6">
    <skos:prefLabel xml:lang="de">Seehandelsrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id107">
    <rdfs:seeAlso rdf:resource="descriptor/11302-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10077-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16648-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19061-6">
    <skos:prefLabel xml:lang="en">Textile wholesale</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19061-6">
    <skos:prefLabel xml:lang="de">Textilgroßhandel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12735-0">
    <skos:prefLabel xml:lang="en">Flexible production</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12735-0">
    <skos:prefLabel xml:lang="de">Flexibles Fertigungssystem</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12778-3">
    <skos:broader rdf:resource="thsys/70459"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12778-3">
    <skos:broader rdf:resource="thsys/70394"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19313-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11285-3">
    <skos:broader rdf:resource="thsys/71058"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11285-3">
    <skos:broader rdf:resource="descriptor/11294-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11285-3">
    <skos:broader rdf:resource="descriptor/11262-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12832-2">
    <skos:related rdf:resource="descriptor/19132-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12904-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13233-5">
    <skos:prefLabel xml:lang="en">Toys trade</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13233-5">
    <skos:prefLabel xml:lang="de">Spielwarenhandel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16142-0">
    <skos:scopeNote xml:lang="de">Nur Kündigungsschutz für Arbeitskräfte.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18717-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14490-1">
    <skos:related rdf:resource="descriptor/18428-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14490-1">
    <skos:related rdf:resource="descriptor/13278-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14490-1">
    <skos:related rdf:resource="descriptor/14423-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14490-1">
    <skos:related rdf:resource="descriptor/13682-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19743-1">
    <skos:scopeNote xml:lang="de">Forschungsleitende These, die postuliert, dass das Ganze mehr sei als die Summe seiner Teile</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11348-5">
    <skos:related rdf:resource="descriptor/11340-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12399-4">
    <skos:narrower rdf:resource="descriptor/12404-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12399-4">
    <skos:narrower rdf:resource="descriptor/19093-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12399-4">
    <skos:narrower rdf:resource="descriptor/12368-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18549-2">
    <skos:broader rdf:resource="thsys/70947"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18549-2">
    <skos:broader rdf:resource="thsys/70459"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18549-2">
    <skos:broader rdf:resource="descriptor/13200-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18549-2">
    <skos:broader rdf:resource="descriptor/12780-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14619-6">
    <skos:prefLabel xml:lang="en">Photo chemical</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14619-6">
    <skos:prefLabel xml:lang="de">Fotochemikalie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19356-1">
    <skos:related rdf:resource="descriptor/10838-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19356-1">
    <skos:related rdf:resource="descriptor/10237-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19356-1">
    <skos:related rdf:resource="descriptor/10629-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12070-5">
    <skos:related rdf:resource="descriptor/13992-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12070-5">
    <skos:related rdf:resource="descriptor/18031-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12070-5">
    <skos:related rdf:resource="descriptor/18083-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12070-5">
    <skos:related rdf:resource="descriptor/12195-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12070-5">
    <skos:related rdf:resource="descriptor/12057-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12070-5">
    <skos:related rdf:resource="descriptor/12365-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12070-5">
    <skos:related rdf:resource="descriptor/24624-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12070-5">
    <skos:related rdf:resource="descriptor/11904-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15956-4">
    <skos:scopeNote xml:lang="de">Rückkehr in das eigene Herkunftsland oder das Herkunftsland der Vorfahren</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19046-2">
    <skos:altLabel xml:lang="de">Maschinenring</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19046-2">
    <skos:altLabel xml:lang="de">Dienstleistung in der Landwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19046-2">
    <skos:altLabel xml:lang="de">Dienstleistung in der Agrarwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19046-2">
    <skos:altLabel xml:lang="de">Agrarwirtschaftliche Dienstleistung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16373-2">
    <skos:broader rdf:resource="thsys/73378"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16373-2">
    <skos:broader rdf:resource="descriptor/13397-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19180-5">
    <skos:altLabel xml:lang="de">SAARC-Länder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15873-1">
    <skos:narrower rdf:resource="descriptor/15874-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18177-6">
    <skos:scopeNote xml:lang="de">Das Zinsverbot des Islam wird durch Einsetzen alternativer Gewinnanreize ausgeglichen</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19326-3">
    <skos:altLabel xml:lang="en">GATT-rules</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19326-3">
    <skos:altLabel xml:lang="en">GATT rules</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19326-3">
    <skos:altLabel xml:lang="de">WTO-Abkommen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19326-3">
    <skos:altLabel xml:lang="de">GATT-Regeln</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13159-5">
    <skos:prefLabel xml:lang="en">Milling industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13159-5">
    <skos:prefLabel xml:lang="de">Nährmittelindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11739-4">
    <skos:prefLabel xml:lang="en">Intertemporal income distribution</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11739-4">
    <skos:prefLabel xml:lang="de">Intertemporale Einkommensverteilung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14638-2">
    <skos:narrower rdf:resource="descriptor/14034-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15657-0">
    <skos:altLabel xml:lang="de">Poesie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15657-0">
    <skos:altLabel xml:lang="de">Lyrik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15657-0">
    <skos:altLabel xml:lang="de">Literaturwissenschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15657-0">
    <skos:altLabel xml:lang="de">Dichtkunst</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15657-0">
    <skos:altLabel xml:lang="de">Belletristik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11595-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70021">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.04</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14809-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19709-1">
    <skos:broader rdf:resource="thsys/73366"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19709-1">
    <skos:broader rdf:resource="descriptor/13420-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10216-6">
    <skos:related rdf:resource="descriptor/10909-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10216-6">
    <skos:related rdf:resource="descriptor/10209-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10216-6">
    <skos:related rdf:resource="descriptor/19125-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10216-6">
    <skos:related rdf:resource="descriptor/12794-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18501-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11660-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19554-4">
    <skos:broader rdf:resource="thsys/70893"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19554-4">
    <skos:broader rdf:resource="thsys/70187"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19554-4">
    <skos:broader rdf:resource="descriptor/13743-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12761-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11213-0">
    <skos:related rdf:resource="descriptor/18140-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11213-0">
    <skos:related rdf:resource="descriptor/10143-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17806-6">
    <skos:altLabel xml:lang="de">Nordamerikanische Staaten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17806-6">
    <skos:altLabel xml:lang="de">Nordamerikanische Länder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18061-4">
    <skos:broader rdf:resource="thsys/71084"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18061-4">
    <skos:broader rdf:resource="descriptor/19358-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19130-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19220-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10383-1">
    <skos:prefLabel xml:lang="en">Keynesian macroeconomic policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10383-1">
    <skos:prefLabel xml:lang="de">Globalsteuerung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10641-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15926-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19243-0">
    <skos:related rdf:resource="descriptor/11943-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19243-0">
    <skos:related rdf:resource="descriptor/19460-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19243-0">
    <skos:related rdf:resource="descriptor/10575-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12212-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12320-1">
    <skos:scopeNote xml:lang="de">Beitrag eines Betriebes zum Volkseinkommen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73382">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.04.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12800-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19540-1">
    <skos:related rdf:resource="descriptor/19563-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19540-1">
    <skos:related rdf:resource="descriptor/10181-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11271-0">
    <skos:narrower rdf:resource="descriptor/11262-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12160-4">
    <skos:altLabel xml:lang="de">Kaufmännische Verwaltung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12160-4">
    <skos:altLabel xml:lang="de">Betriebliche Verwaltung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12438-6">
    <skos:related rdf:resource="descriptor/19673-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12438-6">
    <skos:related rdf:resource="descriptor/12304-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12438-6">
    <skos:related rdf:resource="descriptor/12427-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12438-6">
    <skos:related rdf:resource="descriptor/12423-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12438-6">
    <skos:related rdf:resource="descriptor/10897-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12438-6">
    <skos:related rdf:resource="descriptor/19934-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12438-6">
    <skos:related rdf:resource="descriptor/12408-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12438-6">
    <skos:related rdf:resource="descriptor/18032-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19690-3">
    <skos:related rdf:resource="descriptor/13420-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19690-3">
    <skos:related rdf:resource="descriptor/16309-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19690-3">
    <skos:related rdf:resource="descriptor/16299-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10248-0">
    <skos:prefLabel xml:lang="en">Circular flow</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10248-0">
    <skos:prefLabel xml:lang="de">Kreislaufanalyse</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13576-3">
    <skos:related rdf:resource="descriptor/13256-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13576-3">
    <skos:related rdf:resource="descriptor/18417-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13576-3">
    <skos:related rdf:resource="descriptor/13066-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13576-3">
    <skos:related rdf:resource="descriptor/14455-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13576-3">
    <skos:related rdf:resource="descriptor/14317-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13576-3">
    <skos:related rdf:resource="descriptor/18296-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13576-3">
    <skos:related rdf:resource="descriptor/13534-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70131">
    <skos:broader rdf:resource="thsys/70002"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18640-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71093">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10769-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19382-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19081-0">
    <skos:altLabel xml:lang="en">Contributions in kind</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19081-0">
    <skos:altLabel xml:lang="de">Sachkapitaleinlage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15695-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73331">
    <skos:broader rdf:resource="thsys/73333"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18674-6">
    <skos:prefLabel xml:lang="en">Stockholding behaviour</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18674-6">
    <skos:prefLabel xml:lang="de">Anlageverhalten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19703-6">
    <skos:broader rdf:resource="thsys/73375"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19703-6">
    <skos:broader rdf:resource="thsys/73361"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19703-6">
    <skos:broader rdf:resource="descriptor/16017-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18992-6">
    <skos:altLabel xml:lang="en">Liquor licensing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18992-6">
    <skos:altLabel xml:lang="de">Branntweinmonopol</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18992-6">
    <skos:altLabel xml:lang="de">Alkoholverbot</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18992-6">
    <skos:altLabel xml:lang="de">Alkoholprohibition</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18992-6">
    <skos:altLabel xml:lang="de">Alkoholmonopol</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18992-6">
    <skos:altLabel xml:lang="de">Alkoholkontrolle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19576-1">
    <skos:altLabel xml:lang="en">Cointegration test</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19576-1">
    <skos:altLabel xml:lang="en">Autoregressive DL Model</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19576-1">
    <skos:altLabel xml:lang="en">Autoregressive distributed lag model</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19576-1">
    <skos:altLabel xml:lang="en">ARDL (Autoregressive DL Model)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19576-1">
    <skos:altLabel xml:lang="en">ADL (Autoregressive DL Model)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19576-1">
    <skos:altLabel xml:lang="de">Kointegrationstest</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16641-3">
    <skos:broader rdf:resource="thsys/73358"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18180-3">
    <skos:altLabel xml:lang="en">Long term care insurance</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18180-3">
    <skos:altLabel xml:lang="de">Soziale Pflegeversicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18180-3">
    <skos:altLabel xml:lang="de">Private Pflegeversicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18180-3">
    <skos:altLabel xml:lang="de">Pflegefallversicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18180-3">
    <skos:altLabel xml:lang="de">Gesetzliche Pflegeversicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14643-2">
    <skos:altLabel xml:lang="de">Magnesiumcarbonat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18917-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11724-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12441-3">
    <skos:broader rdf:resource="thsys/71043"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17666-3">
    <skos:altLabel xml:lang="de">Republik Malawi</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17666-3">
    <skos:altLabel xml:lang="de">Nyasaland</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17666-3">
    <skos:altLabel xml:lang="de">Njassaland</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11047-5">
    <skos:prefLabel xml:lang="en">Communism</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11047-5">
    <skos:prefLabel xml:lang="de">Kommunismus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18874-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15086-3">
    <skos:prefLabel xml:lang="en">Producer price index</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15086-3">
    <skos:prefLabel xml:lang="de">Industriepreisindex</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14840-0">
    <skos:prefLabel xml:lang="en">Textile notions</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14840-0">
    <skos:prefLabel xml:lang="de">Textilkurzwaren</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14902-4">
    <skos:broader rdf:resource="thsys/70031"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14902-4">
    <skos:broader rdf:resource="descriptor/14899-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16663-0">
    <skos:related rdf:resource="descriptor/16619-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16663-0">
    <skos:related rdf:resource="descriptor/19507-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16663-0">
    <skos:related rdf:resource="descriptor/15718-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16663-0">
    <skos:related rdf:resource="descriptor/16654-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16663-0">
    <skos:related rdf:resource="descriptor/11969-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16663-0">
    <skos:related rdf:resource="descriptor/16656-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16663-0">
    <skos:related rdf:resource="descriptor/16672-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16663-0">
    <skos:related rdf:resource="descriptor/16671-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16663-0">
    <skos:related rdf:resource="descriptor/16661-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13305-6">
    <skos:related rdf:resource="descriptor/13865-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13305-6">
    <skos:related rdf:resource="descriptor/13831-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13305-6">
    <skos:related rdf:resource="descriptor/11848-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13305-6">
    <skos:related rdf:resource="descriptor/11847-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13305-6">
    <skos:related rdf:resource="descriptor/13845-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13305-6">
    <skos:related rdf:resource="descriptor/12932-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13305-6">
    <skos:related rdf:resource="descriptor/13829-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12820-2">
    <skos:related rdf:resource="descriptor/13282-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12820-2">
    <skos:related rdf:resource="descriptor/13475-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12820-2">
    <skos:related rdf:resource="descriptor/13967-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12820-2">
    <skos:related rdf:resource="descriptor/13466-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19255-0">
    <skos:related rdf:resource="descriptor/11436-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11235-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16104-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11880-3">
    <skos:altLabel xml:lang="de">Siedlungsstrukturtheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11676-2">
    <skos:broader rdf:resource="thsys/71074"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11676-2">
    <skos:broader rdf:resource="descriptor/11547-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19339-1">
    <skos:altLabel xml:lang="de">Landwirtschaftlicher Boden</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19339-1">
    <skos:altLabel xml:lang="de">Farmland</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19339-1">
    <skos:altLabel xml:lang="de">Ackerboden</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13518-3">
    <skos:broader rdf:resource="thsys/70940"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13518-3">
    <skos:broader rdf:resource="thsys/70932"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13518-3">
    <skos:broader rdf:resource="descriptor/13246-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13518-3">
    <skos:broader rdf:resource="descriptor/13621-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12036-5">
    <skos:related rdf:resource="descriptor/10913-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12036-5">
    <skos:related rdf:resource="descriptor/10911-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12036-5">
    <skos:related rdf:resource="descriptor/10923-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12036-5">
    <skos:related rdf:resource="descriptor/12283-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12036-5">
    <skos:related rdf:resource="descriptor/18775-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12036-5">
    <skos:related rdf:resource="descriptor/19137-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12036-5">
    <skos:related rdf:resource="descriptor/18660-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12036-5">
    <skos:related rdf:resource="descriptor/10546-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70867">
    <skos:narrower rdf:resource="descriptor/10962-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70867">
    <skos:narrower rdf:resource="descriptor/13818-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70867">
    <skos:narrower rdf:resource="descriptor/11886-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70867">
    <skos:narrower rdf:resource="descriptor/10960-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70867">
    <skos:narrower rdf:resource="descriptor/19395-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70867">
    <skos:narrower rdf:resource="descriptor/18979-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70867">
    <skos:narrower rdf:resource="descriptor/11814-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70867">
    <skos:narrower rdf:resource="descriptor/24662-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70867">
    <skos:narrower rdf:resource="descriptor/18728-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70867">
    <skos:narrower rdf:resource="descriptor/13687-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70867">
    <skos:narrower rdf:resource="descriptor/18810-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70049">
    <skos:broader rdf:resource="thsys/v"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10893-6">
    <skos:broader rdf:resource="thsys/71071"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10893-6">
    <skos:broader rdf:resource="descriptor/20522-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10893-6">
    <skos:broader rdf:resource="descriptor/10894-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19395-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17782-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12182-1">
    <skos:broader rdf:resource="thsys/70087"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12182-1">
    <skos:broader rdf:resource="descriptor/12186-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10710-5">
    <skos:prefLabel xml:lang="en">Tariffs</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10710-5">
    <skos:prefLabel xml:lang="de">Zoll</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11917-6">
    <skos:narrower rdf:resource="descriptor/19437-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12708-3">
    <skos:prefLabel xml:lang="en">Production capacity</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12708-3">
    <skos:prefLabel xml:lang="de">Produktionskapazität</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16410-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18942-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18759-5">
    <skos:broader rdf:resource="thsys/70883"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10784-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19391-6">
    <skos:prefLabel xml:lang="en">Footwear market</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19391-6">
    <skos:prefLabel xml:lang="de">Schuhmarkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16303-2">
    <skos:related rdf:resource="descriptor/19860-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16303-2">
    <skos:related rdf:resource="descriptor/16396-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16303-2">
    <skos:related rdf:resource="descriptor/16304-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16303-2">
    <skos:related rdf:resource="descriptor/16395-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16303-2">
    <skos:related rdf:resource="descriptor/13662-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16303-2">
    <skos:related rdf:resource="descriptor/15919-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16303-2">
    <skos:related rdf:resource="descriptor/19843-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18064-5">
    <skos:prefLabel xml:lang="en">Sanction</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18064-5">
    <skos:prefLabel xml:lang="de">Sanktion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19266-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14298-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13367-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14936-1">
    <skos:altLabel xml:lang="de">Traubenzucker</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14936-1">
    <skos:altLabel xml:lang="de">Tapioka</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14936-1">
    <skos:altLabel xml:lang="de">Stärkeprodukt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14936-1">
    <skos:altLabel xml:lang="de">Sago</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14936-1">
    <skos:altLabel xml:lang="de">Puddingpulver</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14936-1">
    <skos:altLabel xml:lang="de">Mühlenprodukt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14936-1">
    <skos:altLabel xml:lang="de">Mehl</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14936-1">
    <skos:altLabel xml:lang="de">Kartoffelstärke</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14936-1">
    <skos:altLabel xml:lang="de">Grieß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14936-1">
    <skos:altLabel xml:lang="de">Getreideprodukt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14936-1">
    <skos:altLabel xml:lang="de">Gelatine</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14936-1">
    <skos:altLabel xml:lang="de">Backpulver</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14936-1">
    <skos:altLabel xml:lang="de">Backfertige Mischung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70926">
    <skos:prefLabel xml:lang="en">W.12.05.02  Transport policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70926">
    <skos:prefLabel xml:lang="de">W.12.05.02  Verkehrspolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11289-2">
    <skos:related rdf:resource="descriptor/16252-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11289-2">
    <skos:related rdf:resource="descriptor/11285-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10913-5">
    <skos:narrower rdf:resource="descriptor/10912-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10913-5">
    <skos:narrower rdf:resource="descriptor/10911-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10913-5">
    <skos:narrower rdf:resource="descriptor/10901-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10913-5">
    <skos:narrower rdf:resource="descriptor/10899-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15974-2">
    <skos:prefLabel xml:lang="en">Rule of law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15974-2">
    <skos:prefLabel xml:lang="de">Rechtsstaat</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10320-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18862-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19253-4">
    <skos:related rdf:resource="descriptor/18869-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19253-4">
    <skos:related rdf:resource="descriptor/19766-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19253-4">
    <skos:related rdf:resource="descriptor/19655-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19253-4">
    <skos:related rdf:resource="descriptor/19601-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19253-4">
    <skos:related rdf:resource="descriptor/19672-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19253-4">
    <skos:related rdf:resource="descriptor/16754-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19253-4">
    <skos:related rdf:resource="descriptor/18003-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19253-4">
    <skos:related rdf:resource="descriptor/16708-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19253-4">
    <skos:related rdf:resource="descriptor/11353-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11802-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15005-1">
    <skos:related rdf:resource="descriptor/14635-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15005-1">
    <skos:related rdf:resource="descriptor/18366-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15005-1">
    <skos:related rdf:resource="descriptor/21110-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15005-1">
    <skos:related rdf:resource="descriptor/16460-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15005-1">
    <skos:related rdf:resource="descriptor/18182-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15005-1">
    <skos:related rdf:resource="descriptor/12925-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15005-1">
    <skos:related rdf:resource="descriptor/18814-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15005-1">
    <skos:related rdf:resource="descriptor/16271-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15005-1">
    <skos:related rdf:resource="descriptor/13412-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15005-1">
    <skos:related rdf:resource="descriptor/16470-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15005-1">
    <skos:related rdf:resource="descriptor/14548-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16229-2">
    <skos:altLabel xml:lang="de">Staatsutopie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18257-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10181-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15648-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14969-0">
    <skos:prefLabel xml:lang="en">Fish product</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14969-0">
    <skos:prefLabel xml:lang="de">Fischprodukt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70953">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12404-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19135-3">
    <skos:related rdf:resource="descriptor/12821-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19135-3">
    <skos:related rdf:resource="descriptor/12823-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10440-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12896-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id164">
    <rdf:value xml:lang="de">Für betrieblichen Kapitalmangel</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10462-5">
    <skos:altLabel xml:lang="de">Überwachungstechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10462-5">
    <skos:altLabel xml:lang="de">Steuerungstechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10462-5">
    <skos:altLabel xml:lang="de">Regeltechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10462-5">
    <skos:altLabel xml:lang="de">Prüftechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10462-5">
    <skos:altLabel xml:lang="de">Meßtechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13304-1">
    <skos:altLabel xml:lang="de">Wohnungsvermietung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13304-1">
    <skos:altLabel xml:lang="de">Verpachtung von Immobilien</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13304-1">
    <skos:altLabel xml:lang="de">Verpachtung von Grundstücken</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13304-1">
    <skos:altLabel xml:lang="de">Immobilienvermietung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13795-5">
    <skos:prefLabel xml:lang="en">Proxy voting</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13795-5">
    <skos:prefLabel xml:lang="de">Depotstimmrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18109-2">
    <skos:altLabel xml:lang="de">Societas Europaea</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18109-2">
    <skos:altLabel xml:lang="de">SE (Societas Europaea)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13621-3">
    <skos:broader rdf:resource="thsys/70940"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13621-3">
    <skos:broader rdf:resource="descriptor/13510-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11976-4">
    <skos:altLabel xml:lang="en">Suburb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11869-5">
    <skos:broader rdf:resource="thsys/71068"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18444-2">
    <skos:broader rdf:resource="thsys/73346"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18568-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12485-4">
    <skos:prefLabel xml:lang="en">Total costs type of accounting</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12485-4">
    <skos:prefLabel xml:lang="de">Gesamtkostenverfahren</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16046-3">
    <skos:altLabel xml:lang="de">Notenbankrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16046-3">
    <skos:altLabel xml:lang="de">Nationalbankrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16046-3">
    <skos:altLabel xml:lang="de">Gesetz über die Deutsche Bundesbank</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16046-3">
    <skos:altLabel xml:lang="de">Bundesbankgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11752-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14883-3">
    <skos:broader rdf:resource="thsys/70476"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14883-3">
    <skos:broader rdf:resource="descriptor/14854-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18834-3">
    <skos:narrower rdf:resource="descriptor/18923-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18834-3">
    <skos:narrower rdf:resource="descriptor/18839-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12915-5">
    <skos:altLabel xml:lang="de">Wirtschaftsbau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12915-5">
    <skos:altLabel xml:lang="de">Verwaltungsbau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12915-5">
    <skos:altLabel xml:lang="de">Landwirtschaftsbau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12915-5">
    <skos:altLabel xml:lang="de">Industriebau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12915-5">
    <skos:altLabel xml:lang="de">Bürobau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17983-5">
    <skos:broader rdf:resource="thsys/70307"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17983-5">
    <skos:broader rdf:resource="descriptor/17999-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19160-4">
    <skos:prefLabel xml:lang="en">Environmental audit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19160-4">
    <skos:prefLabel xml:lang="de">Öko-Audit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73301">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70014">
    <skos:broader rdf:resource="thsys/p"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13190-4">
    <skos:prefLabel xml:lang="en">Scrap metal industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13190-4">
    <skos:prefLabel xml:lang="de">Schrottwirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18928-1">
    <skos:altLabel xml:lang="de">Rundfunkökonomik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18928-1">
    <skos:altLabel xml:lang="de">Rundfunkökonomie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18928-1">
    <skos:altLabel xml:lang="de">Ökonomik des Rundfunks</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18928-1">
    <skos:altLabel xml:lang="de">Ökonomik der Medienwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18928-1">
    <skos:altLabel xml:lang="de">Ökonomie des Rundfunks</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18928-1">
    <skos:altLabel xml:lang="de">Ökonomie der Medienwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18928-1">
    <skos:altLabel xml:lang="de">Medienökonomie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15793-6">
    <skos:related rdf:resource="descriptor/12837-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15793-6">
    <skos:related rdf:resource="descriptor/15858-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12574-5">
    <skos:related rdf:resource="descriptor/12589-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12574-5">
    <skos:related rdf:resource="descriptor/12571-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12574-5">
    <skos:related rdf:resource="descriptor/15821-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12574-5">
    <skos:related rdf:resource="descriptor/11741-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12574-5">
    <skos:related rdf:resource="descriptor/19002-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12574-5">
    <skos:related rdf:resource="descriptor/11295-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12574-5">
    <skos:related rdf:resource="descriptor/12617-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12574-5">
    <skos:related rdf:resource="descriptor/12631-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13692-1">
    <skos:broader rdf:resource="thsys/70900"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13692-1">
    <skos:broader rdf:resource="descriptor/12302-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12882-1">
    <skos:altLabel xml:lang="de">Betriebswirtschaftliches Prüfungswesen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12882-1">
    <skos:altLabel xml:lang="de">Betriebswirtschaftliche Prüfung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12975-1">
    <skos:broader rdf:resource="thsys/71016"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12975-1">
    <skos:broader rdf:resource="thsys/70227"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12975-1">
    <skos:broader rdf:resource="descriptor/12931-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12975-1">
    <skos:broader rdf:resource="descriptor/12913-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12907-4">
    <skos:broader rdf:resource="thsys/70829"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70020">
    <skos:broader rdf:resource="thsys/70021"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13466-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10454-4">
    <skos:related rdf:resource="descriptor/15416-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10454-4">
    <skos:related rdf:resource="descriptor/19660-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14161-6">
    <skos:broader rdf:resource="thsys/73301"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14161-6">
    <skos:broader rdf:resource="thsys/70506"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14433-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14831-1">
    <skos:prefLabel xml:lang="en">Leather good</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14831-1">
    <skos:prefLabel xml:lang="de">Lederwaren</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19720-6">
    <skos:altLabel xml:lang="de">Wehrpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19720-6">
    <skos:altLabel xml:lang="de">Verteidigungsstrategie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19720-6">
    <skos:altLabel xml:lang="de">Verteidigungspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19720-6">
    <skos:altLabel xml:lang="de">Militärstrategie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19499-0">
    <skos:related rdf:resource="descriptor/11284-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19499-0">
    <skos:related rdf:resource="descriptor/19011-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19499-0">
    <skos:related rdf:resource="descriptor/11273-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13769-6">
    <skos:related rdf:resource="descriptor/13744-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13769-6">
    <skos:related rdf:resource="descriptor/13742-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13769-6">
    <skos:related rdf:resource="descriptor/18682-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13769-6">
    <skos:related rdf:resource="descriptor/12299-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13769-6">
    <skos:related rdf:resource="descriptor/18086-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13769-6">
    <skos:related rdf:resource="descriptor/12214-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14023-4">
    <skos:related rdf:resource="descriptor/18454-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14023-4">
    <skos:related rdf:resource="descriptor/12042-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16353-1">
    <skos:prefLabel xml:lang="en">Voting behaviour</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16353-1">
    <skos:prefLabel xml:lang="de">Wahlverhalten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18558-1">
    <skos:broader rdf:resource="thsys/70947"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18558-1">
    <skos:broader rdf:resource="descriptor/19564-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18558-1">
    <skos:broader rdf:resource="descriptor/15410-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19045-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13082-0">
    <skos:altLabel xml:lang="de">Photoindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19440-5">
    <skos:altLabel xml:lang="en">Inter-regional wage differentials</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19440-5">
    <skos:altLabel xml:lang="de">Regionales Lohndifferenzial</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19440-5">
    <skos:altLabel xml:lang="de">Regionales Lohndifferential</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19440-5">
    <skos:altLabel xml:lang="de">Regionaler Lohnunterschied</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19440-5">
    <skos:altLabel xml:lang="de">Regionale Lohndifferenzierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19440-5">
    <skos:altLabel xml:lang="de">Lohngeographie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17781-3">
    <skos:broader rdf:resource="thsys/70086"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17781-3">
    <skos:broader rdf:resource="descriptor/17791-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13299-3">
    <skos:scopeNote xml:lang="de">Versicherung von Transportmitteln sowie der beförderten Güter und der mit dem Transport im Zusammenhang stehenden Risiken</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10991-6">
    <skos:related rdf:resource="descriptor/10112-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10991-6">
    <skos:related rdf:resource="descriptor/10993-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10991-6">
    <skos:related rdf:resource="descriptor/10987-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10991-6">
    <skos:related rdf:resource="descriptor/19666-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10991-6">
    <skos:related rdf:resource="descriptor/10045-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10991-6">
    <skos:related rdf:resource="descriptor/11529-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10991-6">
    <skos:related rdf:resource="descriptor/19198-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10991-6">
    <skos:related rdf:resource="descriptor/18138-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70556">
    <skos:narrower rdf:resource="descriptor/14633-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70556">
    <skos:narrower rdf:resource="descriptor/14692-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70556">
    <skos:narrower rdf:resource="descriptor/14686-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70556">
    <skos:narrower rdf:resource="descriptor/14685-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70556">
    <skos:narrower rdf:resource="descriptor/14656-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70556">
    <skos:narrower rdf:resource="descriptor/14683-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70556">
    <skos:narrower rdf:resource="descriptor/14679-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70556">
    <skos:narrower rdf:resource="descriptor/18079-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70556">
    <skos:narrower rdf:resource="descriptor/14661-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70556">
    <skos:narrower rdf:resource="descriptor/14674-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70556">
    <skos:narrower rdf:resource="descriptor/14673-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70556">
    <skos:narrower rdf:resource="descriptor/14623-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70556">
    <skos:narrower rdf:resource="descriptor/14665-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70556">
    <skos:narrower rdf:resource="descriptor/14662-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70556">
    <skos:narrower rdf:resource="descriptor/14664-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13191-2">
    <skos:related rdf:resource="descriptor/13117-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13191-2">
    <skos:related rdf:resource="descriptor/14796-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11129-3">
    <skos:related rdf:resource="descriptor/10115-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11129-3">
    <skos:related rdf:resource="descriptor/12354-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11337-3">
    <skos:broader rdf:resource="thsys/73104"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11337-3">
    <skos:broader rdf:resource="thsys/71062"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11337-3">
    <skos:broader rdf:resource="thsys/70278"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11337-3">
    <skos:broader rdf:resource="descriptor/15463-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11337-3">
    <skos:broader rdf:resource="descriptor/11219-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18686-6">
    <skos:broader rdf:resource="thsys/71105"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18686-6">
    <skos:broader rdf:resource="thsys/71089"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18686-6">
    <skos:broader rdf:resource="descriptor/10804-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13235-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13430-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13367-5">
    <skos:editorialNote xml:lang="de">Möglichst spezifischer indexieren</skos:editorialNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13837-1">
    <skos:narrower rdf:resource="descriptor/18756-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70928">
    <rdfs:label xml:lang="en">Transport economics</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70928">
    <rdfs:label xml:lang="de">Verkehrsökonomik</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11911-4">
    <skos:altLabel xml:lang="de">Regionales Ungleichgewicht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11911-4">
    <skos:altLabel xml:lang="de">Räumliches Ungleichgewicht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11911-4">
    <skos:altLabel xml:lang="de">Räumliche Disparität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18172-2">
    <skos:broader rdf:resource="thsys/73340"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18172-2">
    <skos:broader rdf:resource="thsys/71037"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18521-3">
    <skos:altLabel xml:lang="de">Teegroßhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18521-3">
    <skos:altLabel xml:lang="de">Tee-Einzelhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12513-4">
    <skos:altLabel xml:lang="de">Betriebsabrechnungsbogen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12513-4">
    <skos:altLabel xml:lang="de">BAB (Betriebsabrechnungsbogen)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10337-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18109-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13572-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70880">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.19.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14710-6">
    <skos:altLabel xml:lang="de">Polyolefin</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14710-6">
    <skos:altLabel xml:lang="de">Polyester</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14193-0">
    <skos:related rdf:resource="descriptor/19182-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14193-0">
    <skos:related rdf:resource="descriptor/19387-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14193-0">
    <skos:related rdf:resource="descriptor/12984-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14193-0">
    <skos:related rdf:resource="descriptor/14197-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12963-1">
    <skos:altLabel xml:lang="de">Seenfischerei</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12963-1">
    <skos:altLabel xml:lang="de">Flussfischerei</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12963-1">
    <skos:altLabel xml:lang="de">Flußfischerei</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10178-2">
    <skos:altLabel xml:lang="de">Giffen-Paradoxon</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10178-2">
    <skos:altLabel xml:lang="de">Giffen-Gut</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10421-5">
    <skos:related rdf:resource="descriptor/11516-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10421-5">
    <skos:related rdf:resource="descriptor/10236-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11284-5">
    <skos:broader rdf:resource="thsys/70052"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11284-5">
    <skos:broader rdf:resource="descriptor/15920-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11284-5">
    <skos:broader rdf:resource="descriptor/11262-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12939-5">
    <skos:altLabel xml:lang="de">Futtermittelanbau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12939-5">
    <skos:altLabel xml:lang="de">Futteranbau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14749-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73387">
    <skos:broader rdf:resource="thsys/70007"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10437-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16280-2">
    <skos:related rdf:resource="descriptor/16288-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16280-2">
    <skos:related rdf:resource="descriptor/16340-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19220-5">
    <skos:narrower rdf:resource="descriptor/10498-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11277-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10205-4">
    <skos:narrower rdf:resource="descriptor/11633-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10205-4">
    <skos:narrower rdf:resource="descriptor/10208-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10205-4">
    <skos:narrower rdf:resource="descriptor/10195-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13222-3">
    <skos:broader rdf:resource="thsys/70959"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13222-3">
    <skos:broader rdf:resource="descriptor/18522-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12073-6">
    <skos:broader rdf:resource="thsys/71042"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11284-5">
    <skos:related rdf:resource="descriptor/16146-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11284-5">
    <skos:related rdf:resource="descriptor/11273-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11284-5">
    <skos:related rdf:resource="descriptor/19498-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11284-5">
    <skos:related rdf:resource="descriptor/19499-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11284-5">
    <skos:related rdf:resource="descriptor/19205-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18740-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19357-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10036-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10966-5">
    <skos:altLabel xml:lang="de">Wirtschaftszweigverband</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10966-5">
    <skos:altLabel xml:lang="de">Wirtschaftlicher Interessenverband</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10966-5">
    <skos:altLabel xml:lang="de">Unternehmerverband</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10966-5">
    <skos:altLabel xml:lang="de">Unternehmensverband</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10966-5">
    <skos:altLabel xml:lang="de">Industrieverband</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10966-5">
    <skos:altLabel xml:lang="de">Gewerbeverein</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10966-5">
    <skos:altLabel xml:lang="de">Gewerbeverband</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10966-5">
    <skos:altLabel xml:lang="de">Fachverband</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10966-5">
    <skos:altLabel xml:lang="de">Branchenverband</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12501-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16700-6">
    <skos:broader rdf:resource="thsys/73357"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16700-6">
    <skos:broader rdf:resource="descriptor/16564-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18069-2">
    <skos:altLabel xml:lang="en">Greenhouse warming</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18069-2">
    <skos:altLabel xml:lang="en">Greenhouse effect</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18069-2">
    <skos:altLabel xml:lang="en">Global warming</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18069-2">
    <skos:altLabel xml:lang="de">Treibhauseffekt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18069-2">
    <skos:altLabel xml:lang="de">Klimawandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18987-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16648-3">
    <skos:prefLabel xml:lang="en">Cultural cooperation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16648-3">
    <skos:prefLabel xml:lang="de">Kulturelle Beziehungen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10612-5">
    <skos:scopeNote xml:lang="de">Wird nur verwendet für internationale Freihandelszonen wie z.B. EFTA.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19690-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12303-1">
    <skos:related rdf:resource="descriptor/12078-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12303-1">
    <skos:related rdf:resource="descriptor/16058-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12303-1">
    <skos:related rdf:resource="descriptor/10897-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13074-6">
    <skos:altLabel xml:lang="de">Mikroelektronikindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13074-6">
    <skos:altLabel xml:lang="de">Chipindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13074-6">
    <skos:altLabel xml:lang="de">Chip-Industrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11903-3">
    <skos:prefLabel xml:lang="en">Central place model</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11903-3">
    <skos:prefLabel xml:lang="de">Zentraler Ort</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13825-1">
    <skos:prefLabel xml:lang="en">Insurance law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13825-1">
    <skos:prefLabel xml:lang="de">Versicherungsrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10042-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18699-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14619-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11526-0">
    <skos:broader rdf:resource="thsys/71083"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11526-0">
    <skos:broader rdf:resource="thsys/70858"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11526-0">
    <skos:broader rdf:resource="descriptor/10032-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18430-6">
    <skos:related rdf:resource="descriptor/13378-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18430-6">
    <skos:related rdf:resource="descriptor/13959-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18430-6">
    <skos:related rdf:resource="descriptor/12996-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18430-6">
    <skos:related rdf:resource="descriptor/18777-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10382-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10072-4">
    <skos:broader rdf:resource="thsys/73104"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10072-4">
    <skos:broader rdf:resource="thsys/70049"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18995-0">
    <skos:prefLabel xml:lang="en">Commodity market</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18995-0">
    <skos:prefLabel xml:lang="de">Rohstoffmarkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16198-1">
    <skos:prefLabel xml:lang="en">Criminal law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16198-1">
    <skos:prefLabel xml:lang="de">Strafrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17742-6">
    <skos:scopeNote xml:lang="de">Ehemals Zaire</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14268-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14105-2">
    <skos:narrower rdf:resource="descriptor/19024-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14105-2">
    <skos:narrower rdf:resource="descriptor/14116-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14105-2">
    <skos:narrower rdf:resource="descriptor/14106-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10668-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13014-3">
    <skos:broader rdf:resource="thsys/71008"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13014-3">
    <skos:broader rdf:resource="descriptor/13020-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70965">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17999-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15754-2">
    <skos:prefLabel xml:lang="en">Raw materials resources</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15754-2">
    <skos:prefLabel xml:lang="de">Rohstoffressourcen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10545-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16529-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15064-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16246-2">
    <skos:altLabel xml:lang="de">Repräsentative Demokratie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16246-2">
    <skos:altLabel xml:lang="de">Parlamentarische Demokratie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16246-2">
    <skos:altLabel xml:lang="de">Direkte Demokratie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16246-2">
    <skos:altLabel xml:lang="de">Demokratietheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11551-1">
    <skos:related rdf:resource="descriptor/19662-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11551-1">
    <skos:related rdf:resource="descriptor/11154-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19118-3">
    <skos:broader rdf:resource="thsys/70041"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10346-0">
    <skos:prefLabel xml:lang="en">Business cycle transmission</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10346-0">
    <skos:prefLabel xml:lang="de">Konjunkturzusammenhang</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19201-2">
    <skos:broader rdf:resource="thsys/70049"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19201-2">
    <skos:broader rdf:resource="descriptor/12275-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11183-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17959-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12807-1">
    <skos:broader rdf:resource="thsys/70550"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12807-1">
    <skos:broader rdf:resource="descriptor/12057-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16357-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71077">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10882-4">
    <skos:related rdf:resource="descriptor/11125-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10882-4">
    <skos:related rdf:resource="descriptor/13407-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10882-4">
    <skos:related rdf:resource="descriptor/18794-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10882-4">
    <skos:related rdf:resource="descriptor/10851-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10882-4">
    <skos:related rdf:resource="descriptor/18544-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10882-4">
    <skos:related rdf:resource="descriptor/10852-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10882-4">
    <skos:related rdf:resource="descriptor/10686-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16421-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13703-1">
    <skos:prefLabel xml:lang="en">Mortgage bank</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13703-1">
    <skos:prefLabel xml:lang="de">Hypothekenbank</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14640-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10201-5">
    <skos:broader rdf:resource="thsys/71119"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10201-5">
    <skos:broader rdf:resource="thsys/71102"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18631-3">
    <skos:broader rdf:resource="thsys/71065"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18631-3">
    <skos:broader rdf:resource="thsys/70925"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18631-3">
    <skos:broader rdf:resource="descriptor/13510-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17465-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12774-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73364">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19664-4">
    <skos:broader rdf:resource="thsys/71089"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19664-4">
    <skos:broader rdf:resource="thsys/70187"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19664-4">
    <skos:broader rdf:resource="descriptor/10343-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19179-4">
    <skos:altLabel xml:lang="de">MERCOSUR-Länder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19460-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10583-0">
    <skos:scopeNote xml:lang="de">Nur für Nahrungsmittelhilfe im Sinne von Entwicklungshilfe.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10586-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19096-1">
    <skos:altLabel xml:lang="en">Target costing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19096-1">
    <skos:altLabel xml:lang="de">Zielkostenmanagement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19096-1">
    <skos:altLabel xml:lang="de">Kostenkontrolle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14698-5">
    <skos:broader rdf:resource="thsys/70476"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14698-5">
    <skos:broader rdf:resource="descriptor/14710-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70933">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19584-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14007-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id353">
    <rdfs:seeAlso rdf:resource="descriptor/11559-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14305-1">
    <skos:broader rdf:resource="thsys/70134"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14305-1">
    <skos:broader rdf:resource="descriptor/14274-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14305-1">
    <skos:broader rdf:resource="descriptor/14302-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19022-2">
    <skos:broader rdf:resource="thsys/70403"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19022-2">
    <skos:broader rdf:resource="descriptor/14169-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16274-4">
    <skos:broader rdf:resource="thsys/70095"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18423-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11635-2">
    <skos:altLabel xml:lang="de">Steuereinnahmen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11635-2">
    <skos:altLabel xml:lang="de">Steueraufkommensstruktur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18370-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15690-2">
    <skos:altLabel xml:lang="en">Historical development</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15690-2">
    <skos:altLabel xml:lang="en">Historical account</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15690-2">
    <skos:altLabel xml:lang="de">Historische Entwicklung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15690-2">
    <skos:altLabel xml:lang="de">Historische Betrachtung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15690-2">
    <skos:altLabel xml:lang="de">Geschichtliche Entwicklung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#OrderedCollection">
    <rdfs:isDefinedBy rdf:resource="http://www.w3.org/2004/02/skos/core"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18821-5">
    <skos:prefLabel xml:lang="en">Service quality</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18821-5">
    <skos:prefLabel xml:lang="de">Dienstleistungsqualität</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16146-6">
    <skos:prefLabel xml:lang="en">Maternity protection</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16146-6">
    <skos:prefLabel xml:lang="de">Mutterschutz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11418-3">
    <zbwext:useInsteadNote rdf:nodeID="id109"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10478-4">
    <skos:related rdf:resource="descriptor/10481-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10478-4">
    <skos:related rdf:resource="descriptor/18065-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10478-4">
    <skos:related rdf:resource="descriptor/10947-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10478-4">
    <skos:related rdf:resource="descriptor/10475-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11998-1">
    <skos:altLabel xml:lang="en">Urban planning</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11998-1">
    <skos:altLabel xml:lang="en">Local government planning</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11998-1">
    <skos:altLabel xml:lang="en">Community planning</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11998-1">
    <skos:altLabel xml:lang="de">Stadtplanung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11998-1">
    <skos:altLabel xml:lang="de">Städteplanung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11998-1">
    <skos:altLabel xml:lang="de">Stadtentwicklungspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11998-1">
    <skos:altLabel xml:lang="de">Stadtentwicklungsplanung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11998-1">
    <skos:altLabel xml:lang="de">Stadtentwicklungskonzeption</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11998-1">
    <skos:altLabel xml:lang="de">Ortsplanung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11998-1">
    <skos:altLabel xml:lang="de">Kommunale Entwicklungsplanung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11998-1">
    <skos:altLabel xml:lang="de">Gemeindeplanung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16362-0">
    <skos:broader rdf:resource="thsys/73378"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16362-0">
    <skos:broader rdf:resource="descriptor/13397-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11957-1">
    <skos:broader rdf:resource="thsys/73354"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11957-1">
    <skos:broader rdf:resource="descriptor/15872-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14440-2">
    <skos:altLabel xml:lang="en">Jeep</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14440-2">
    <skos:altLabel xml:lang="de">Pkw</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14440-2">
    <skos:altLabel xml:lang="de">Personenwagen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14440-2">
    <skos:altLabel xml:lang="de">Geländewagen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13234-3">
    <skos:related rdf:resource="descriptor/12958-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13234-3">
    <skos:related rdf:resource="descriptor/14985-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13234-3">
    <skos:related rdf:resource="descriptor/13152-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13234-3">
    <skos:related rdf:resource="descriptor/14983-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70560">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">G.03.01.03</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11845-5">
    <skos:scopeNote xml:lang="de">Alle Formen der Wohnungsbauförderung, z.B. durch öffentliche Darlehen, Subventionen, steuerliche Anreize.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10956-1">
    <skos:related rdf:resource="descriptor/10963-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10956-1">
    <skos:related rdf:resource="descriptor/10942-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10956-1">
    <skos:related rdf:resource="descriptor/10895-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10956-1">
    <skos:related rdf:resource="descriptor/18828-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10956-1">
    <skos:related rdf:resource="descriptor/10891-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19104-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73336">
    <skos:narrower rdf:resource="descriptor/19824-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73336">
    <skos:narrower rdf:resource="descriptor/16150-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73336">
    <skos:narrower rdf:resource="descriptor/19325-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73336">
    <skos:narrower rdf:resource="descriptor/19764-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73336">
    <skos:narrower rdf:resource="descriptor/11347-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73336">
    <skos:narrower rdf:resource="descriptor/20485-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73336">
    <skos:narrower rdf:resource="descriptor/16146-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73336">
    <skos:narrower rdf:resource="descriptor/11310-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73336">
    <skos:narrower rdf:resource="descriptor/16142-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73336">
    <skos:narrower rdf:resource="descriptor/16140-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73336">
    <skos:narrower rdf:resource="descriptor/19812-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73336">
    <skos:narrower rdf:resource="descriptor/19509-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73336">
    <skos:narrower rdf:resource="descriptor/19503-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73336">
    <skos:narrower rdf:resource="descriptor/16136-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73336">
    <skos:narrower rdf:resource="descriptor/18169-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73336">
    <skos:narrower rdf:resource="descriptor/18792-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73336">
    <skos:narrower rdf:resource="descriptor/16131-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73336">
    <skos:narrower rdf:resource="descriptor/12609-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73336">
    <skos:narrower rdf:resource="descriptor/19844-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73336">
    <skos:narrower rdf:resource="descriptor/11185-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73336">
    <skos:narrower rdf:resource="descriptor/11307-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73336">
    <skos:narrower rdf:resource="descriptor/16124-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73336">
    <skos:narrower rdf:resource="descriptor/19003-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11281-4">
    <skos:prefLabel xml:lang="en">Overtime</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11281-4">
    <skos:prefLabel xml:lang="de">Überstunden</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13412-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16679-6">
    <zbwext:useInsteadNote rdf:nodeID="id105"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19046-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20110-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18773-4">
    <skos:related rdf:resource="descriptor/18889-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15858-4">
    <skos:broader rdf:resource="thsys/73384"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15858-4">
    <skos:broader rdf:resource="thsys/70384"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15858-4">
    <skos:broader rdf:resource="thsys/70292"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15858-4">
    <skos:broader rdf:resource="descriptor/15836-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16156-3">
    <skos:related rdf:resource="descriptor/11566-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16156-3">
    <skos:related rdf:resource="descriptor/11565-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16369-0">
    <skos:prefLabel xml:lang="en">Party programme</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16369-0">
    <skos:prefLabel xml:lang="de">Parteiprogramm</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11651-4">
    <skos:altLabel xml:lang="en">Succession tax</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11651-4">
    <skos:altLabel xml:lang="en">Estate tax</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11651-4">
    <skos:altLabel xml:lang="en">Capital transfer tax</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11651-4">
    <skos:altLabel xml:lang="de">Schenkungsteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11651-4">
    <skos:altLabel xml:lang="de">Schenkungssteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11651-4">
    <skos:altLabel xml:lang="de">Nachlaßsteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11651-4">
    <skos:altLabel xml:lang="de">Nachlasssteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11651-4">
    <skos:altLabel xml:lang="de">Erbschaftsteuerrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11651-4">
    <skos:altLabel xml:lang="de">Erbschaftsteuergesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11651-4">
    <skos:altLabel xml:lang="de">Erbschaftssteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11651-4">
    <skos:altLabel xml:lang="de">Erbnachlaßsteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11651-4">
    <skos:altLabel xml:lang="de">Erbnachlasssteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11651-4">
    <skos:altLabel xml:lang="de">Erbanfallsteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70550">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19423-5">
    <skos:altLabel xml:lang="de">Vergnügungssteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20982-1">
    <skos:altLabel xml:lang="de">Neuklassische Theorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20982-1">
    <skos:altLabel xml:lang="de">Moderne Klassische Theorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16029-3">
    <skos:broader rdf:resource="thsys/73392"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16029-3">
    <skos:broader rdf:resource="descriptor/16026-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/w">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19830-6">
    <skos:related rdf:resource="descriptor/13405-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19830-6">
    <skos:related rdf:resource="descriptor/18097-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19830-6">
    <skos:related rdf:resource="descriptor/18906-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19830-6">
    <skos:related rdf:resource="descriptor/19221-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18007-3">
    <skos:broader rdf:resource="thsys/70459"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70953">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18361-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12737-3">
    <skos:altLabel xml:lang="en">Industrial robot</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12737-3">
    <skos:altLabel xml:lang="de">Industrieroboter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14299-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19574-5">
    <skos:prefLabel xml:lang="en">ARCH model</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19574-5">
    <skos:prefLabel xml:lang="de">ARCH-Modell</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70178">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17568-3">
    <skos:altLabel xml:lang="en">Nippon</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19411-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73353">
    <rdfs:label xml:lang="de">Physische Geographie</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19510-3">
    <skos:narrower rdf:resource="descriptor/11567-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18863-3">
    <skos:prefLabel xml:lang="en">Women's education</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18863-3">
    <skos:prefLabel xml:lang="de">Frauenbildung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16676-5">
    <skos:broader rdf:resource="thsys/73356"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16676-5">
    <skos:broader rdf:resource="thsys/73346"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16676-5">
    <skos:broader rdf:resource="descriptor/16670-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11894-6">
    <skos:broader rdf:resource="thsys/70182"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11894-6">
    <skos:broader rdf:resource="thsys/70058"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11894-6">
    <skos:broader rdf:resource="descriptor/15708-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16479-0">
    <skos:prefLabel xml:lang="en">Health policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16479-0">
    <skos:prefLabel xml:lang="de">Gesundheitspolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13533-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18928-1">
    <skos:related rdf:resource="descriptor/12829-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18928-1">
    <skos:related rdf:resource="descriptor/18939-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18928-1">
    <skos:related rdf:resource="descriptor/13347-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18928-1">
    <skos:related rdf:resource="descriptor/18921-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18928-1">
    <skos:related rdf:resource="descriptor/18917-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19575-3">
    <skos:broader rdf:resource="thsys/72186"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19575-3">
    <skos:broader rdf:resource="descriptor/15298-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10164-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18840-1">
    <skos:broader rdf:resource="thsys/70854"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18840-1">
    <skos:broader rdf:resource="thsys/70852"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18840-1">
    <skos:broader rdf:resource="descriptor/10966-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70397">
    <skos:prefLabel xml:lang="de">B.08.01  Steuerrechtliche Gewinnermittlung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15609-4">
    <skos:narrower rdf:resource="descriptor/12172-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15609-4">
    <skos:narrower rdf:resource="descriptor/15410-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15609-4">
    <skos:narrower rdf:resource="descriptor/15611-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12040-0">
    <skos:broader rdf:resource="thsys/70582"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18627-1">
    <skos:broader rdf:resource="thsys/73362"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18627-1">
    <skos:broader rdf:resource="thsys/70926"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18627-1">
    <skos:broader rdf:resource="descriptor/11833-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18627-1">
    <skos:broader rdf:resource="descriptor/18772-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18989-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18430-6">
    <skos:broader rdf:resource="thsys/73349"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18430-6">
    <skos:broader rdf:resource="thsys/70994"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18430-6">
    <skos:broader rdf:resource="thsys/70878"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18430-6">
    <skos:broader rdf:resource="descriptor/19782-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18430-6">
    <skos:broader rdf:resource="descriptor/15638-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13679-0">
    <skos:broader rdf:resource="thsys/70922"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13679-0">
    <skos:broader rdf:resource="descriptor/13681-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13679-0">
    <skos:broader rdf:resource="descriptor/13674-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17993-2">
    <skos:altLabel xml:lang="en">High-income countries (OECD)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17993-2">
    <skos:altLabel xml:lang="de">Westliche Industriestaaten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17993-2">
    <skos:altLabel xml:lang="de">Westliche Industrieländer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17993-2">
    <skos:altLabel xml:lang="de">OECD-Länder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10462-5">
    <skos:prefLabel xml:lang="en">Measurement technology</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10462-5">
    <skos:prefLabel xml:lang="de">Messtechnik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70943">
    <rdfs:label xml:lang="de">Verkehr und Tourismus</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10323-5">
    <skos:narrower rdf:resource="descriptor/10320-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14116-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id90">
    <rdfs:seeAlso rdf:resource="descriptor/10555-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16853-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13193-5">
    <skos:related rdf:resource="descriptor/19063-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13193-5">
    <skos:related rdf:resource="descriptor/14728-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13193-5">
    <skos:related rdf:resource="descriptor/13002-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13193-5">
    <skos:related rdf:resource="descriptor/18259-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13193-5">
    <skos:related rdf:resource="descriptor/19675-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12040-0">
    <zbwext:useInsteadNote rdf:nodeID="id55"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18012-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10695-3">
    <skos:broader rdf:resource="thsys/71096"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10695-3">
    <skos:broader rdf:resource="thsys/70933"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10695-3">
    <skos:broader rdf:resource="descriptor/13533-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10695-3">
    <skos:broader rdf:resource="descriptor/10711-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19477-3">
    <skos:related rdf:resource="descriptor/19478-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19477-3">
    <skos:related rdf:resource="descriptor/11902-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19477-3">
    <skos:related rdf:resource="descriptor/10510-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19477-3">
    <skos:related rdf:resource="descriptor/11971-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19477-3">
    <skos:related rdf:resource="descriptor/10504-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19477-3">
    <skos:related rdf:resource="descriptor/11965-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72181">
    <rdfs:label xml:lang="de">Statistische Übersichten</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19293-6">
    <skos:narrower rdf:resource="descriptor/19305-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19293-6">
    <skos:narrower rdf:resource="descriptor/19300-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13255-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19207-4">
    <skos:scopeNote xml:lang="de">Anwendung mathematischer Methoden und Modelle zur Darstellung und Herleitung ökonomischer Analysen. Nicht notwendigerweise mit empirischen Untersuchungen der Ökonometrie oder Operations Research verbunden.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11907-2">
    <skos:related rdf:resource="descriptor/11713-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14722-6">
    <skos:broader rdf:resource="thsys/70199"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14722-6">
    <skos:broader rdf:resource="descriptor/14728-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18839-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18507-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id98">
    <rdf:value xml:lang="de">Für Fertigungstechniken und -verfahren</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19329-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13045-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10488-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19310-4">
    <skos:scopeNote xml:lang="de">Staatlicher Schutz inländischer Produzenten gegen ausländische Wettbewerber.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15981-5">
    <skos:narrower rdf:resource="descriptor/16041-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15981-5">
    <skos:narrower rdf:resource="descriptor/19816-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15981-5">
    <skos:narrower rdf:resource="descriptor/19434-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15981-5">
    <skos:narrower rdf:resource="descriptor/16058-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15981-5">
    <skos:narrower rdf:resource="descriptor/19821-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18425-6">
    <skos:scopeNote xml:lang="de">Förderung der Elektronikindustrie.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16664-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19443-6">
    <skos:scopeNote xml:lang="de">Standortbedingte brancheninterne Ersparnis.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16737-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70423">
    <skos:narrower rdf:resource="descriptor/14543-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70423">
    <skos:narrower rdf:resource="descriptor/18251-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70423">
    <skos:narrower rdf:resource="descriptor/14537-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70423">
    <skos:narrower rdf:resource="descriptor/14534-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70423">
    <skos:narrower rdf:resource="descriptor/14539-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70423">
    <skos:narrower rdf:resource="descriptor/14538-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70423">
    <skos:narrower rdf:resource="descriptor/14531-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70423">
    <skos:narrower rdf:resource="descriptor/14527-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19793-0">
    <skos:broader rdf:resource="thsys/73346"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19793-0">
    <skos:broader rdf:resource="descriptor/18897-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11813-4">
    <skos:prefLabel xml:lang="en">Energy resources</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11813-4">
    <skos:prefLabel xml:lang="de">Energieressourcen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17915-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71025">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70147">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">G.02.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13236-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12774-4">
    <zbwext:useInsteadNote rdf:nodeID="id202"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14769-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18522-1">
    <skos:altLabel xml:lang="de">Verbrauchsgüterhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18522-1">
    <skos:altLabel xml:lang="de">Konsumgütergroßhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18522-1">
    <skos:altLabel xml:lang="de">Konsumgütereinzelhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18522-1">
    <skos:altLabel xml:lang="de">Gebrauchsgüterhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70143">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11999-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17920-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15904-2">
    <skos:altLabel xml:lang="de">Seniorinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15904-2">
    <skos:altLabel xml:lang="de">Senioren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15904-2">
    <skos:altLabel xml:lang="de">Ältere Menschen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11310-2">
    <skos:prefLabel xml:lang="en">Codetermination</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11310-2">
    <skos:prefLabel xml:lang="de">Mitbestimmung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19656-3">
    <skos:scopeNote xml:lang="de">Nur für beruflich bedingte Auslandsaufenthalte.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#ConceptScheme">
    <rdfs:label xml:lang="en">Concept Scheme</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15642-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13113-1">
    <skos:altLabel xml:lang="de">Holzveredelungsindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11881-1">
    <skos:broader rdf:resource="thsys/71064"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11881-1">
    <skos:broader rdf:resource="thsys/70925"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11881-1">
    <skos:broader rdf:resource="descriptor/13510-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15105-4">
    <skos:broader rdf:resource="thsys/72181"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15105-4">
    <skos:broader rdf:resource="thsys/70233"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15105-4">
    <skos:broader rdf:resource="descriptor/15076-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13845-2">
    <skos:prefLabel xml:lang="en">Housing market</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13845-2">
    <skos:prefLabel xml:lang="de">Wohnungsmarkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15220-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10223-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13444-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19536-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11253-2">
    <skos:altLabel xml:lang="en">Early retirement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11253-2">
    <skos:altLabel xml:lang="de">Vorruhestandsgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11253-2">
    <skos:altLabel xml:lang="de">Vorruhestand</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11253-2">
    <skos:altLabel xml:lang="de">Vorgezogenes Altersruhegeld</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11253-2">
    <skos:altLabel xml:lang="de">Lebensarbeitszeitverkürzung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11253-2">
    <skos:altLabel xml:lang="de">Gleitende Pensionierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11253-2">
    <skos:altLabel xml:lang="de">Frühverrentung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11253-2">
    <skos:altLabel xml:lang="de">Altersteilzeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15766-2">
    <skos:prefLabel xml:lang="en">Environmental crime</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15766-2">
    <skos:prefLabel xml:lang="de">Umweltkriminalität</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12738-1">
    <skos:broader rdf:resource="thsys/70318"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12738-1">
    <skos:broader rdf:resource="descriptor/14393-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19762-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13142-1">
    <skos:prefLabel xml:lang="en">Footwear industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13142-1">
    <skos:prefLabel xml:lang="de">Schuhindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25681-4">
    <skos:broader rdf:resource="thsys/73357"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10866-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10110-1">
    <skos:altLabel xml:lang="de">Gerechte Einkommensverteilung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18865-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10074-0">
    <skos:altLabel xml:lang="en">Fixed-price equilibrium</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10074-0">
    <skos:altLabel xml:lang="de">Festpreisgleichgewicht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70135">
    <skos:prefLabel xml:lang="de">P.10  Elektrotechnische und elektronische Produkte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18169-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19171-6">
    <skos:altLabel xml:lang="de">APEC-Länder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13219-6">
    <skos:broader rdf:resource="thsys/70960"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13219-6">
    <skos:broader rdf:resource="descriptor/13210-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13355-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19182-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10497-0">
    <skos:prefLabel xml:lang="en">Green revolution</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10497-0">
    <skos:prefLabel xml:lang="de">Grüne Revolution</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15328-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16542-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24783-1">
    <skos:prefLabel xml:lang="en">New public management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24783-1">
    <skos:prefLabel xml:lang="de">New Public Management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18793-5">
    <skos:broader rdf:resource="thsys/71057"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18793-5">
    <skos:broader rdf:resource="thsys/70875"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70560">
    <skos:prefLabel xml:lang="en">G.03.01.03  West Africa</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70560">
    <skos:prefLabel xml:lang="de">G.03.01.03  Westafrika</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70306">
    <skos:broader rdf:resource="thsys/70055"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16387-5">
    <skos:altLabel xml:lang="en">Working class movement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16387-5">
    <skos:altLabel xml:lang="en">Labor movement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16387-5">
    <skos:altLabel xml:lang="de">Internationale Arbeiterbewegung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12780-2">
    <skos:related rdf:resource="descriptor/18552-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12780-2">
    <skos:related rdf:resource="descriptor/21315-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12780-2">
    <skos:related rdf:resource="descriptor/21408-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13472-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24858-3">
    <skos:prefLabel xml:lang="en">Tax assessment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24858-3">
    <skos:prefLabel xml:lang="de">Steuerbescheid</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19448-3">
    <skos:related rdf:resource="descriptor/19446-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19448-3">
    <skos:related rdf:resource="descriptor/11940-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14023-4">
    <skos:prefLabel xml:lang="en">Office equipment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14023-4">
    <skos:prefLabel xml:lang="de">Büroausstattung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70925">
    <rdfs:label xml:lang="de">Verkehrsgeographie und regionale Verkehrspolitik</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19409-6">
    <skos:related rdf:resource="descriptor/11650-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14661-0">
    <skos:prefLabel xml:lang="en">Ether</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14661-0">
    <skos:prefLabel xml:lang="de">Ether</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15718-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19659-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18944-3">
    <skos:narrower rdf:resource="descriptor/18945-1"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id198">
    <rdf:value xml:lang="de">ABER: Für Lohnunterschiede (wage gap) nach Geschlecht, Alter, Rasse, etc.</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71083">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13406-0">
    <skos:prefLabel xml:lang="en">Unemployment insurance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13406-0">
    <skos:prefLabel xml:lang="de">Arbeitslosenversicherung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18990-3">
    <skos:altLabel xml:lang="de">Wassertarif</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13153-3">
    <skos:scopeNote xml:lang="de">Herstellung und Verarbeitung von pflanzlichen und tierischen Ölen und Fetten zu Nahrungszwecken</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19748-5">
    <skos:broader rdf:resource="thsys/73395"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19748-5">
    <skos:broader rdf:resource="descriptor/15717-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71003">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10376-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70142">
    <skos:broader rdf:resource="thsys/n"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10332-4">
    <skos:prefLabel xml:lang="en">Returns to scale</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10332-4">
    <skos:prefLabel xml:lang="de">Skalenertrag</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18027-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11267-5">
    <skos:related rdf:resource="descriptor/19543-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11267-5">
    <skos:related rdf:resource="descriptor/11365-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11267-5">
    <skos:related rdf:resource="descriptor/19505-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11267-5">
    <skos:related rdf:resource="descriptor/11288-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15778-2">
    <skos:scopeNote xml:lang="de">Nur für Vermeidungs- und Verhinderungsstrategien beim Umweltschutz.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12399-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10182-4">
    <skos:altLabel xml:lang="de">Superiores Gut</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10182-4">
    <skos:altLabel xml:lang="de">Neoklassische Nachfragetheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10182-4">
    <skos:altLabel xml:lang="de">Mikroökonomische Nachfragefunktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10182-4">
    <skos:altLabel xml:lang="de">Marshallsche Nachfragefunktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10182-4">
    <skos:altLabel xml:lang="de">Engel-Kurve</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12972-0">
    <skos:prefLabel xml:lang="en">Sericulture</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12972-0">
    <skos:prefLabel xml:lang="de">Seidenraupenzucht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18454-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12001-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10931-3">
    <skos:prefLabel xml:lang="en">Resale price maintenance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10931-3">
    <skos:prefLabel xml:lang="de">Preisbindung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11803-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18536-4">
    <skos:related rdf:resource="descriptor/10608-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18536-4">
    <skos:related rdf:resource="descriptor/10833-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10823-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16527-1">
    <skos:broader rdf:resource="thsys/70832"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16527-1">
    <skos:broader rdf:resource="thsys/70458"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16527-1">
    <skos:broader rdf:resource="descriptor/16542-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70074">
    <skos:broader rdf:resource="thsys/70014"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19748-5">
    <skos:prefLabel xml:lang="en">Aristocracy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19748-5">
    <skos:prefLabel xml:lang="de">Adel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13033-6">
    <skos:broader rdf:resource="thsys/71000"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13033-6">
    <skos:broader rdf:resource="descriptor/13037-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12049-3">
    <skos:altLabel xml:lang="de">Großbetrieb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10497-0">
    <skos:broader rdf:resource="thsys/71085"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10497-0">
    <skos:broader rdf:resource="thsys/71020"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10497-0">
    <skos:broader rdf:resource="descriptor/10505-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12879-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15619-1">
    <skos:broader rdf:resource="thsys/73340"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15619-1">
    <skos:broader rdf:resource="thsys/71037"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15619-1">
    <skos:broader rdf:resource="descriptor/15676-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71050">
    <rdfs:label xml:lang="en">Human engineering</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71050">
    <rdfs:label xml:lang="de">Arbeitswissenschaft</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14176-0">
    <skos:related rdf:resource="descriptor/14634-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14176-0">
    <skos:related rdf:resource="descriptor/12975-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14176-0">
    <skos:related rdf:resource="descriptor/18353-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14176-0">
    <skos:related rdf:resource="descriptor/18350-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14176-0">
    <skos:related rdf:resource="descriptor/18348-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14176-0">
    <skos:related rdf:resource="descriptor/14175-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12502-2">
    <skos:prefLabel xml:lang="en">Non-wage labour costs</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12502-2">
    <skos:prefLabel xml:lang="de">Personalzusatzkosten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10221-6">
    <skos:related rdf:resource="descriptor/18691-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10221-6">
    <skos:related rdf:resource="descriptor/10210-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10221-6">
    <skos:related rdf:resource="descriptor/10077-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10221-6">
    <skos:related rdf:resource="descriptor/10803-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17684-1">
    <skos:broader rdf:resource="thsys/70429"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17684-1">
    <skos:broader rdf:resource="descriptor/17687-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71094">
    <skos:prefLabel xml:lang="en">V.07.03.01  International monetary system</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71094">
    <skos:prefLabel xml:lang="de">V.07.03.01  Internationales Währungssystem</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16171-0">
    <skos:altLabel xml:lang="de">StVO (Straßenverkehrs-Ordnung)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16171-0">
    <skos:altLabel xml:lang="de">Straßenverkehrs-Ordnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16171-0">
    <skos:altLabel xml:lang="de">Straßenverkehrsgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16171-0">
    <skos:altLabel xml:lang="de">Personenbeförderungsgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16171-0">
    <skos:altLabel xml:lang="de">Bundesfernstraßengesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11055-6">
    <skos:prefLabel xml:lang="en">Market economy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11055-6">
    <skos:prefLabel xml:lang="de">Marktwirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14393-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19263-1">
    <skos:related rdf:resource="descriptor/10722-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19263-1">
    <skos:related rdf:resource="descriptor/19547-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19180-5">
    <skos:prefLabel xml:lang="en">SAARC countries</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19180-5">
    <skos:prefLabel xml:lang="de">SAARC-Staaten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id122">
    <rdfs:seeAlso rdf:resource="descriptor/11164-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24705-0">
    <skos:related rdf:resource="descriptor/18822-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24705-0">
    <skos:related rdf:resource="descriptor/11722-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24705-0">
    <skos:related rdf:resource="descriptor/12047-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24705-0">
    <skos:related rdf:resource="descriptor/10201-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24705-0">
    <skos:related rdf:resource="descriptor/10040-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14139-6">
    <skos:related rdf:resource="descriptor/15668-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10606-0">
    <skos:narrower rdf:resource="descriptor/10780-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10606-0">
    <skos:narrower rdf:resource="descriptor/19315-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12205-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12424-3">
    <skos:prefLabel xml:lang="en">Manufacturing costs</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12424-3">
    <skos:prefLabel xml:lang="de">Herstellungskosten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10327-4">
    <skos:narrower rdf:resource="descriptor/10328-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10327-4">
    <skos:narrower rdf:resource="descriptor/10315-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10327-4">
    <skos:narrower rdf:resource="descriptor/10312-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10327-4">
    <skos:narrower rdf:resource="descriptor/10307-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10327-4">
    <skos:narrower rdf:resource="descriptor/10305-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10326-6">
    <skos:prefLabel xml:lang="en">Production factor</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10326-6">
    <skos:prefLabel xml:lang="de">Produktionsfaktor</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19761-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18219-2">
    <skos:related rdf:resource="descriptor/12953-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18219-2">
    <skos:related rdf:resource="descriptor/14144-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13225-4">
    <skos:broader rdf:resource="thsys/70960"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13225-4">
    <skos:broader rdf:resource="descriptor/13219-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16604-2">
    <skos:altLabel xml:lang="en">Knowledge society</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16604-2">
    <skos:altLabel xml:lang="de">Wissensgesellschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16604-2">
    <skos:altLabel xml:lang="de">Informationszeitalter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10247-2">
    <skos:altLabel xml:lang="en">Keynesian macroeconomics</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10247-2">
    <skos:altLabel xml:lang="de">Keynesianische Theorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10247-2">
    <skos:altLabel xml:lang="de">Fiskalismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16193-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13248-6">
    <skos:altLabel xml:lang="de">Leihwagenvermietung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13248-6">
    <skos:altLabel xml:lang="de">Kraftfahrzeugvermietung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13248-6">
    <skos:altLabel xml:lang="de">Kraftfahrzeug-Leasing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13248-6">
    <skos:altLabel xml:lang="de">Auto-Leasing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19065-5">
    <skos:altLabel xml:lang="de">Erschließung von Bauland</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19065-5">
    <skos:altLabel xml:lang="de">Baulanderschließung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14371-2">
    <skos:prefLabel xml:lang="en">Publishing-related machinery</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14371-2">
    <skos:prefLabel xml:lang="de">Druckmaschine</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70423">
    <rdfs:label xml:lang="de">Feinmechanische und optische Produkte</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19419-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13269-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17950-6">
    <skos:prefLabel xml:lang="en">Tuvalu</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17950-6">
    <skos:prefLabel xml:lang="de">Tuvalu</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70026">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10835-6">
    <skos:related rdf:resource="descriptor/19590-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10835-6">
    <skos:related rdf:resource="descriptor/10852-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10835-6">
    <skos:related rdf:resource="descriptor/10833-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73395">
    <skos:broader rdf:resource="thsys/70321"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11943-5">
    <skos:altLabel xml:lang="en">Regional economic growth</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11943-5">
    <skos:altLabel xml:lang="de">Regionales Wirtschaftswachstum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19353-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11914-5">
    <skos:prefLabel xml:lang="en">Location factor</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11914-5">
    <skos:prefLabel xml:lang="de">Standortfaktor</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19067-1">
    <skos:altLabel xml:lang="en">Clerical service</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19067-1">
    <skos:altLabel xml:lang="de">Schreibdienst</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12326-3">
    <skos:broader rdf:resource="thsys/73391"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12326-3">
    <skos:broader rdf:resource="thsys/71043"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12326-3">
    <skos:broader rdf:resource="descriptor/13464-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19676-4">
    <skos:altLabel xml:lang="en">Physician income</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19676-4">
    <skos:altLabel xml:lang="en">Physician fees</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19676-4">
    <skos:altLabel xml:lang="de">Arzteinkommen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19676-4">
    <skos:altLabel xml:lang="de">Ärzteeinkommen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id166">
    <rdf:value xml:lang="de">Für die synthetische Kautschukherstellung</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19189-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12907-4">
    <skos:narrower rdf:resource="descriptor/12927-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12907-4">
    <skos:narrower rdf:resource="descriptor/18918-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12907-4">
    <skos:narrower rdf:resource="descriptor/18920-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12907-4">
    <skos:narrower rdf:resource="descriptor/13354-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12907-4">
    <skos:narrower rdf:resource="descriptor/13307-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12907-4">
    <skos:narrower rdf:resource="descriptor/13320-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13662-3">
    <skos:altLabel xml:lang="de">Zensur des Staates</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13662-3">
    <skos:altLabel xml:lang="de">Staatliche Zensur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13662-3">
    <skos:altLabel xml:lang="de">Pressefreiheit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13662-3">
    <skos:altLabel xml:lang="de">Politische Zensur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13662-3">
    <skos:altLabel xml:lang="de">Freiheit der Berichterstattung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19106-3">
    <skos:prefLabel xml:lang="en">Special audit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19106-3">
    <skos:prefLabel xml:lang="de">Sonderprüfung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17732-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15952-5">
    <skos:prefLabel xml:lang="en">Immigration</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15952-5">
    <skos:prefLabel xml:lang="de">Einwanderung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15062-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16521-6">
    <skos:broader rdf:resource="thsys/73365"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12996-0">
    <skos:broader rdf:resource="thsys/70994"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12996-0">
    <skos:broader rdf:resource="descriptor/12993-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13065-0">
    <skos:narrower rdf:resource="descriptor/13069-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13065-0">
    <skos:narrower rdf:resource="descriptor/13051-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18863-3">
    <skos:broader rdf:resource="thsys/70176"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18863-3">
    <skos:broader rdf:resource="thsys/70143"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18863-3">
    <skos:broader rdf:resource="descriptor/11353-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16608-1">
    <skos:prefLabel xml:lang="en">Bourgeoisie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16608-1">
    <skos:prefLabel xml:lang="de">Bürgertum</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19660-5">
    <skos:narrower rdf:resource="descriptor/18952-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19660-5">
    <skos:narrower rdf:resource="descriptor/13755-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19660-5">
    <skos:narrower rdf:resource="descriptor/12730-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13306-4">
    <skos:altLabel xml:lang="de">Tanzlokal</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13306-4">
    <skos:altLabel xml:lang="de">Restaurant</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13306-4">
    <skos:altLabel xml:lang="de">Gaststätte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13306-4">
    <skos:altLabel xml:lang="de">Diskothek</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13306-4">
    <skos:altLabel xml:lang="de">Cafe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13306-4">
    <skos:altLabel xml:lang="de">Bar</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16658-0">
    <skos:altLabel xml:lang="en">Ruling class</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16658-0">
    <skos:altLabel xml:lang="de">Politische Elite</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16658-0">
    <skos:altLabel xml:lang="de">Oberschicht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16658-0">
    <skos:altLabel xml:lang="de">Machtelite</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16658-0">
    <skos:altLabel xml:lang="de">Herrschende Klasse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16658-0">
    <skos:altLabel xml:lang="de">Führungsschicht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16658-0">
    <skos:altLabel xml:lang="de">Elitenbildung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11411-3">
    <skos:prefLabel xml:lang="en">Banknote</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11411-3">
    <skos:prefLabel xml:lang="de">Banknote</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13894-3">
    <skos:prefLabel xml:lang="en">Woodworking professions</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13894-3">
    <skos:prefLabel xml:lang="de">Holzberufe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14900-1">
    <skos:narrower rdf:resource="descriptor/14905-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11710-0">
    <skos:related rdf:resource="descriptor/20536-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11710-0">
    <skos:related rdf:resource="descriptor/19414-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10559-4">
    <skos:prefLabel xml:lang="en">Development plan</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10559-4">
    <skos:prefLabel xml:lang="de">Entwicklungsplan</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16834-6">
    <skos:altLabel xml:lang="de">Römisch-Deutsches Reich</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16834-6">
    <skos:altLabel xml:lang="de">Rheinbund</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16834-6">
    <skos:altLabel xml:lang="de">Norddeutscher Bund</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16834-6">
    <skos:altLabel xml:lang="de">Heiliges Römisches Reich Deutscher Nation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16834-6">
    <skos:altLabel xml:lang="de">Deutsches Reich</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16834-6">
    <skos:altLabel xml:lang="de">Deutscher Bund</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18409-4">
    <skos:broader rdf:resource="thsys/71002"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18409-4">
    <skos:broader rdf:resource="descriptor/13058-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16611-5">
    <skos:prefLabel xml:lang="en">Middle class</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16611-5">
    <skos:prefLabel xml:lang="de">Mittelstand</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73296">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.06.01.08.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16188-4">
    <skos:narrower rdf:resource="descriptor/16173-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16188-4">
    <skos:narrower rdf:resource="descriptor/16309-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16188-4">
    <skos:narrower rdf:resource="descriptor/18064-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16188-4">
    <skos:narrower rdf:resource="descriptor/15964-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16188-4">
    <skos:narrower rdf:resource="descriptor/18567-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16188-4">
    <skos:narrower rdf:resource="descriptor/10829-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16188-4">
    <skos:narrower rdf:resource="descriptor/16184-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16188-4">
    <skos:narrower rdf:resource="descriptor/19350-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10923-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18637-5">
    <skos:related rdf:resource="descriptor/16485-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18637-5">
    <skos:related rdf:resource="descriptor/13253-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18637-5">
    <skos:related rdf:resource="descriptor/19854-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18637-5">
    <skos:related rdf:resource="descriptor/13643-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18637-5">
    <skos:related rdf:resource="descriptor/13639-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14932-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id330">
    <rdfs:seeAlso rdf:resource="descriptor/18387-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12615-3">
    <skos:altLabel xml:lang="de">Werksärztliche Versorgung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12615-3">
    <skos:altLabel xml:lang="de">Gesundheitsschutz im Betrieb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12615-3">
    <skos:altLabel xml:lang="de">Gesundheitsfürsorge im Betrieb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12615-3">
    <skos:altLabel xml:lang="de">Betriebsärztliche Versorgung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12615-3">
    <skos:altLabel xml:lang="de">Betrieblicher Gesundheitsschutz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12615-3">
    <skos:altLabel xml:lang="de">Betriebliche Gesundheitsfürsorge</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13340-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18581-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12961-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73300">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70212">
    <skos:narrower rdf:resource="descriptor/10913-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70212">
    <skos:narrower rdf:resource="descriptor/10942-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70212">
    <skos:narrower rdf:resource="descriptor/10940-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70212">
    <skos:narrower rdf:resource="descriptor/10912-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70212">
    <skos:narrower rdf:resource="descriptor/10911-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70212">
    <skos:narrower rdf:resource="descriptor/10931-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70212">
    <skos:narrower rdf:resource="descriptor/10930-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70212">
    <skos:narrower rdf:resource="descriptor/10907-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70212">
    <skos:narrower rdf:resource="descriptor/10904-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70212">
    <skos:narrower rdf:resource="descriptor/10928-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70212">
    <skos:narrower rdf:resource="descriptor/19667-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70212">
    <skos:narrower rdf:resource="descriptor/10926-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70212">
    <skos:narrower rdf:resource="descriptor/10901-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70212">
    <skos:narrower rdf:resource="descriptor/12055-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70212">
    <skos:narrower rdf:resource="descriptor/20972-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70212">
    <skos:narrower rdf:resource="descriptor/10923-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70212">
    <skos:narrower rdf:resource="descriptor/12283-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70212">
    <skos:narrower rdf:resource="descriptor/10899-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70212">
    <skos:narrower rdf:resource="descriptor/10897-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19858-5">
    <skos:broader rdf:resource="thsys/73360"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19858-5">
    <skos:broader rdf:resource="descriptor/16019-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20497-1">
    <skos:prefLabel xml:lang="en">Cross-cultural management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20497-1">
    <skos:prefLabel xml:lang="de">Interkulturelles Management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12932-5">
    <skos:altLabel xml:lang="de">Privater Wohnungsbau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12932-5">
    <skos:altLabel xml:lang="de">Freifinanzierter Wohnungsbau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13851-0">
    <skos:related rdf:resource="descriptor/13841-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13851-0">
    <skos:related rdf:resource="descriptor/18750-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13851-0">
    <skos:related rdf:resource="descriptor/13853-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13851-0">
    <skos:related rdf:resource="descriptor/18504-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#narrowerTransitive">
    <rdfs:subPropertyOf rdf:resource="http://www.w3.org/2004/02/skos/core#semanticRelation"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10143-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17568-3">
    <skos:broader rdf:resource="thsys/70147"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17568-3">
    <skos:broader rdf:resource="descriptor/17571-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17568-3">
    <skos:broader rdf:resource="descriptor/19178-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17568-3">
    <skos:broader rdf:resource="descriptor/19177-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13971-4">
    <skos:altLabel xml:lang="en">Art-directors</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13971-4">
    <skos:altLabel xml:lang="de">Werbeleiterinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13971-4">
    <skos:altLabel xml:lang="de">Werbeleiter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13971-4">
    <skos:altLabel xml:lang="de">Werbekaufleute</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13971-4">
    <skos:altLabel xml:lang="de">Werbefachleute</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13971-4">
    <skos:altLabel xml:lang="de">Werbeberaterinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13971-4">
    <skos:altLabel xml:lang="de">Werbeberater</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13971-4">
    <skos:altLabel xml:lang="de">Public-Relations-Berufe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13971-4">
    <skos:altLabel xml:lang="de">PR-Berufe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13971-4">
    <skos:altLabel xml:lang="de">Medienberaterinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13971-4">
    <skos:altLabel xml:lang="de">Medienberater</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13971-4">
    <skos:altLabel xml:lang="de">Media-Fachleute</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13971-4">
    <skos:altLabel xml:lang="de">Art-Direktorinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13971-4">
    <skos:altLabel xml:lang="de">Art-Direktoren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16688-5">
    <skos:prefLabel xml:lang="en">Settlement research</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16688-5">
    <skos:prefLabel xml:lang="de">Siedlungssoziologie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21094-4">
    <skos:related rdf:resource="descriptor/12665-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16471-2">
    <skos:broader rdf:resource="thsys/73364"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16471-2">
    <skos:broader rdf:resource="descriptor/16288-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16471-2">
    <skos:broader rdf:resource="descriptor/19720-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17015-2">
    <skos:broader rdf:resource="thsys/70131"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17015-2">
    <skos:broader rdf:resource="descriptor/17016-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13525-6">
    <skos:broader rdf:resource="thsys/70941"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13525-6">
    <skos:broader rdf:resource="descriptor/11834-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18136-6">
    <skos:prefLabel xml:lang="en">Thuringia</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18136-6">
    <skos:prefLabel xml:lang="de">Thüringen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18247-4">
    <skos:prefLabel xml:lang="en">Ferry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18247-4">
    <skos:prefLabel xml:lang="de">Fährschiff</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18112-6">
    <skos:broader rdf:resource="thsys/71036"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18112-6">
    <skos:broader rdf:resource="thsys/70922"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18112-6">
    <skos:broader rdf:resource="thsys/70052"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18112-6">
    <skos:broader rdf:resource="descriptor/11197-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18112-6">
    <skos:broader rdf:resource="descriptor/18111-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17933-6">
    <skos:altLabel xml:lang="de">Republik Östlich des Uruguay</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19770-5">
    <skos:broader rdf:resource="thsys/73393"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19770-5">
    <skos:broader rdf:resource="descriptor/19749-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16435-6">
    <skos:related rdf:resource="descriptor/16436-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16435-6">
    <skos:related rdf:resource="descriptor/16463-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16435-6">
    <skos:related rdf:resource="descriptor/16288-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16435-6">
    <skos:related rdf:resource="descriptor/16186-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16435-6">
    <skos:related rdf:resource="descriptor/16441-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16435-6">
    <skos:related rdf:resource="descriptor/16502-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16435-6">
    <skos:related rdf:resource="descriptor/10557-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16435-6">
    <skos:related rdf:resource="descriptor/16489-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14064-4">
    <skos:related rdf:resource="descriptor/14074-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14064-4">
    <skos:related rdf:resource="descriptor/19295-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14064-4">
    <skos:related rdf:resource="descriptor/19294-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14064-4">
    <skos:related rdf:resource="descriptor/18518-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14064-4">
    <skos:related rdf:resource="descriptor/18519-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14064-4">
    <skos:related rdf:resource="descriptor/18192-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14064-4">
    <skos:related rdf:resource="descriptor/18191-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14064-4">
    <skos:related rdf:resource="descriptor/14971-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12201-2">
    <skos:related rdf:resource="descriptor/12185-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12201-2">
    <skos:related rdf:resource="descriptor/12175-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12201-2">
    <skos:related rdf:resource="descriptor/12174-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16070-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14771-0">
    <skos:broader rdf:resource="thsys/70647"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14771-0">
    <skos:broader rdf:resource="descriptor/14766-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16184-5">
    <skos:related rdf:resource="descriptor/15982-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16184-5">
    <skos:related rdf:resource="descriptor/10621-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16184-5">
    <skos:related rdf:resource="descriptor/19708-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16184-5">
    <skos:related rdf:resource="descriptor/19853-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11194-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13287-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12014-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17687-2">
    <skos:narrower rdf:resource="descriptor/17689-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17687-2">
    <skos:narrower rdf:resource="descriptor/17688-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17687-2">
    <skos:narrower rdf:resource="descriptor/17686-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17687-2">
    <skos:narrower rdf:resource="descriptor/17684-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17687-2">
    <skos:narrower rdf:resource="descriptor/17682-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17687-2">
    <skos:narrower rdf:resource="descriptor/17680-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14021-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15064-6">
    <skos:prefLabel xml:lang="en">Statistical method</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15064-6">
    <skos:prefLabel xml:lang="de">Statistische Methode</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18159-1">
    <skos:prefLabel xml:lang="en">Sprecherausschuss</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18159-1">
    <skos:prefLabel xml:lang="de">Sprecherausschuss</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14486-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10950-6">
    <skos:prefLabel xml:lang="en">Structure-conduct-performance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10950-6">
    <skos:prefLabel xml:lang="de">Struktur-Performance-Modell</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24859-1">
    <skos:prefLabel xml:lang="en">Payment behaviour</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24859-1">
    <skos:prefLabel xml:lang="de">Zahlungsmoral</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13688-6">
    <skos:prefLabel xml:lang="en">Banking crisis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13688-6">
    <skos:prefLabel xml:lang="de">Bankenkrise</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16587-4">
    <skos:broader rdf:resource="thsys/73359"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16587-4">
    <skos:broader rdf:resource="descriptor/15714-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14638-2">
    <skos:altLabel xml:lang="en">Potassium sulphate</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14638-2">
    <skos:altLabel xml:lang="de">Kaliumsulfat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14638-2">
    <skos:altLabel xml:lang="de">Kaliumchlorid</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14638-2">
    <skos:altLabel xml:lang="de">Kali</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13181-5">
    <skos:narrower rdf:resource="descriptor/13185-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13181-5">
    <skos:narrower rdf:resource="descriptor/13077-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10769-0">
    <skos:narrower rdf:resource="descriptor/10673-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16828-1">
    <skos:altLabel xml:lang="x-other">Ceskoslovensko</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16828-1">
    <skos:altLabel xml:lang="de">Tschechoslowakische Sozialistische Republik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16828-1">
    <skos:altLabel xml:lang="de">Tschechische und Slowakische Föderative Republik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16828-1">
    <skos:altLabel xml:lang="de">CSSR</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18823-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70944">
    <skos:narrower rdf:resource="descriptor/18565-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70944">
    <skos:narrower rdf:resource="descriptor/18564-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70944">
    <skos:narrower rdf:resource="descriptor/18006-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70944">
    <skos:narrower rdf:resource="descriptor/13308-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70944">
    <skos:narrower rdf:resource="descriptor/13309-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70944">
    <skos:narrower rdf:resource="descriptor/13306-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70944">
    <skos:narrower rdf:resource="descriptor/13975-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70944">
    <skos:narrower rdf:resource="descriptor/13307-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70944">
    <skos:narrower rdf:resource="descriptor/18561-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70944">
    <skos:narrower rdf:resource="descriptor/18560-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70944">
    <skos:narrower rdf:resource="descriptor/13636-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12497-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10302-6">
    <skos:related rdf:resource="descriptor/16645-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10302-6">
    <skos:related rdf:resource="descriptor/11671-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14656-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19586-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71030">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11769-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11143-2">
    <skos:scopeNote xml:lang="de">Führung und Planung in sozialistischen Unternehmen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16401-2">
    <skos:related rdf:resource="descriptor/18905-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16401-2">
    <skos:related rdf:resource="descriptor/16651-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16401-2">
    <skos:related rdf:resource="descriptor/18855-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16401-2">
    <skos:related rdf:resource="descriptor/10150-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19572-2">
    <skos:broader rdf:resource="thsys/72185"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19572-2">
    <skos:broader rdf:resource="descriptor/15225-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14313-2">
    <skos:altLabel xml:lang="de">Tellur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14313-2">
    <skos:altLabel xml:lang="de">Silicium</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14313-2">
    <skos:altLabel xml:lang="de">Selen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14313-2">
    <skos:altLabel xml:lang="de">Bor</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14313-2">
    <skos:altLabel xml:lang="de">Arsen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14313-2">
    <skos:altLabel xml:lang="de">Antimon</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10253-0">
    <zbwext:useInsteadNote rdf:nodeID="id241"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13466-3">
    <skos:prefLabel xml:lang="en">Sales representatives</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13466-3">
    <skos:prefLabel xml:lang="de">Handelsvertretung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13165-3">
    <skos:related rdf:resource="descriptor/19307-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13165-3">
    <skos:related rdf:resource="descriptor/18316-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13165-3">
    <skos:related rdf:resource="descriptor/19306-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13165-3">
    <skos:related rdf:resource="descriptor/14934-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19838-4">
    <skos:broader rdf:resource="thsys/73390"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19838-4">
    <skos:broader rdf:resource="thsys/71015"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19838-4">
    <skos:broader rdf:resource="thsys/70227"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19838-4">
    <skos:broader rdf:resource="descriptor/15993-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19380-4">
    <skos:altLabel xml:lang="de">Haushaltsvollzug</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19380-4">
    <skos:altLabel xml:lang="de">Haushaltskreislauf</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19380-4">
    <skos:altLabel xml:lang="de">Haushaltsführung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19380-4">
    <skos:altLabel xml:lang="de">Haushaltsaufstellung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19380-4">
    <skos:altLabel xml:lang="de">Budgetprozess</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19380-4">
    <skos:altLabel xml:lang="de">Budgetprozeß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11052-5">
    <skos:prefLabel xml:lang="en">Performance incentive</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11052-5">
    <skos:prefLabel xml:lang="de">Leistungsanreiz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14404-6">
    <skos:related rdf:resource="descriptor/18234-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14404-6">
    <skos:related rdf:resource="descriptor/14407-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14404-6">
    <skos:related rdf:resource="descriptor/13339-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14404-6">
    <skos:related rdf:resource="descriptor/14337-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14404-6">
    <skos:related rdf:resource="descriptor/13185-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14404-6">
    <skos:related rdf:resource="descriptor/12977-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19277-4">
    <skos:scopeNote xml:lang="de">Kurzfristige Schuldtitel, die am Euromarkt emittiert werden.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10166-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10864-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18222-6">
    <skos:related rdf:resource="descriptor/16676-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18222-6">
    <skos:related rdf:resource="descriptor/13160-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18222-6">
    <skos:related rdf:resource="descriptor/14931-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18222-6">
    <skos:related rdf:resource="descriptor/14723-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18222-6">
    <skos:related rdf:resource="descriptor/14054-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70327">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13679-0">
    <skos:altLabel xml:lang="de">LWL-System</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13679-0">
    <skos:altLabel xml:lang="de">Lichtwellenleitertechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13679-0">
    <skos:altLabel xml:lang="de">Lichtleiterübertragung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13679-0">
    <skos:altLabel xml:lang="de">Glasfaserübertragung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13679-0">
    <skos:altLabel xml:lang="de">Glasfasertechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13679-0">
    <skos:altLabel xml:lang="de">Glasfaserkabel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14423-2">
    <skos:related rdf:resource="descriptor/13578-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14423-2">
    <skos:related rdf:resource="descriptor/15899-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14423-2">
    <skos:related rdf:resource="descriptor/13684-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14423-2">
    <skos:related rdf:resource="descriptor/13507-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14423-2">
    <skos:related rdf:resource="descriptor/14490-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14423-2">
    <skos:related rdf:resource="descriptor/14421-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14423-2">
    <skos:related rdf:resource="descriptor/18418-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14423-2">
    <skos:related rdf:resource="descriptor/13056-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70940">
    <skos:narrower rdf:resource="descriptor/13299-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70940">
    <skos:narrower rdf:resource="descriptor/18620-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70940">
    <skos:narrower rdf:resource="descriptor/18598-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70940">
    <skos:narrower rdf:resource="descriptor/13244-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70940">
    <skos:narrower rdf:resource="descriptor/13260-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70940">
    <skos:narrower rdf:resource="descriptor/18597-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70940">
    <skos:narrower rdf:resource="descriptor/13627-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70940">
    <skos:narrower rdf:resource="descriptor/13518-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70940">
    <skos:narrower rdf:resource="descriptor/24787-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70940">
    <skos:narrower rdf:resource="descriptor/13245-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70940">
    <skos:narrower rdf:resource="descriptor/19064-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70940">
    <skos:narrower rdf:resource="descriptor/18618-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70940">
    <skos:narrower rdf:resource="descriptor/18617-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70940">
    <skos:narrower rdf:resource="descriptor/18616-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70940">
    <skos:narrower rdf:resource="descriptor/18615-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70940">
    <skos:narrower rdf:resource="descriptor/13621-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70940">
    <skos:narrower rdf:resource="descriptor/13620-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70940">
    <skos:narrower rdf:resource="descriptor/13601-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70940">
    <skos:narrower rdf:resource="descriptor/13618-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70940">
    <skos:narrower rdf:resource="descriptor/18614-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70940">
    <skos:narrower rdf:resource="descriptor/13617-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70940">
    <skos:narrower rdf:resource="descriptor/18356-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70940">
    <skos:narrower rdf:resource="descriptor/13553-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70940">
    <skos:narrower rdf:resource="descriptor/13565-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70940">
    <skos:narrower rdf:resource="descriptor/18573-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70940">
    <skos:narrower rdf:resource="thsys/70930"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18555-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10996-3">
    <skos:narrower rdf:resource="descriptor/11217-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13830-1">
    <skos:prefLabel xml:lang="en">Housing stock</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13830-1">
    <skos:prefLabel xml:lang="de">Wohnungsbestand</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10266-5">
    <skos:prefLabel xml:lang="en">Conceptual approach of the measurement of nationals acounts</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10266-5">
    <skos:prefLabel xml:lang="de">Sozialproduktskonzept</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16122-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18632-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11696-3">
    <skos:altLabel xml:lang="en">Tax distortion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11696-3">
    <skos:altLabel xml:lang="de">Allokative Steuerwirkung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10977-0">
    <skos:prefLabel xml:lang="en">Ordo-liberal conception</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10977-0">
    <skos:prefLabel xml:lang="de">Ordoliberalismus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13747-2">
    <skos:prefLabel xml:lang="en">Bank charge</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13747-2">
    <skos:prefLabel xml:lang="de">Bankgebühr</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10721-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11881-1">
    <skos:altLabel xml:lang="de">Ballungsverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11881-1">
    <skos:altLabel xml:lang="de">Agglomerationsverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17485-0">
    <skos:broader rdf:resource="thsys/70174"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17485-0">
    <skos:broader rdf:resource="descriptor/19077-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17485-0">
    <skos:broader rdf:resource="descriptor/19078-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12312-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19284-0">
    <skos:prefLabel xml:lang="en">Voucher privatization</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19284-0">
    <skos:prefLabel xml:lang="de">Voucher-Privatisierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19259-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12501-4">
    <skos:prefLabel xml:lang="en">Payroll costs</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12501-4">
    <skos:prefLabel xml:lang="de">Personalkosten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11543-0">
    <skos:altLabel xml:lang="de">Verwaltungsgebühr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11543-0">
    <skos:altLabel xml:lang="de">Öffentlicher Beitrag</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11543-0">
    <skos:altLabel xml:lang="de">Kommunale Gebühr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11543-0">
    <skos:altLabel xml:lang="de">Benutzungsgebühr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11543-0">
    <skos:altLabel xml:lang="de">Beitrag</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11543-0">
    <skos:altLabel xml:lang="de">Abgaben</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15075-1">
    <skos:broader rdf:resource="thsys/70021"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15075-1">
    <skos:broader rdf:resource="descriptor/15076-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15075-1">
    <skos:broader rdf:resource="descriptor/15073-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19276-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11928-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18267-5">
    <skos:broader rdf:resource="thsys/70074"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18267-5">
    <skos:broader rdf:resource="descriptor/14602-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24733-2">
    <zbwext:useInsteadNote rdf:nodeID="id124"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11627-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13462-4">
    <skos:related rdf:resource="descriptor/13511-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13462-4">
    <skos:related rdf:resource="descriptor/13210-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13462-4">
    <skos:related rdf:resource="descriptor/11416-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10980-4">
    <skos:broader rdf:resource="thsys/70274"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13808-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12594-6">
    <skos:related rdf:resource="descriptor/12598-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12594-6">
    <skos:related rdf:resource="descriptor/11741-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12594-6">
    <skos:related rdf:resource="descriptor/15839-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12594-6">
    <skos:related rdf:resource="descriptor/11052-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12594-6">
    <skos:related rdf:resource="descriptor/11772-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12594-6">
    <skos:related rdf:resource="descriptor/18173-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12594-6">
    <skos:related rdf:resource="descriptor/10440-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12594-6">
    <skos:related rdf:resource="descriptor/24622-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11495-6">
    <skos:related rdf:resource="descriptor/10255-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11495-6">
    <skos:related rdf:resource="descriptor/19547-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11495-6">
    <skos:related rdf:resource="descriptor/11498-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16815-3">
    <skos:altLabel xml:lang="de">Europäische Staaten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16815-3">
    <skos:altLabel xml:lang="de">Europäische Länder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id191">
    <rdfs:seeAlso rdf:resource="descriptor/19761-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14343-0">
    <skos:broader rdf:resource="thsys/70318"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14343-0">
    <skos:broader rdf:resource="descriptor/14381-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18986-1">
    <skos:prefLabel xml:lang="en">Monoculture</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18986-1">
    <skos:prefLabel xml:lang="de">Monokultur</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12308-5">
    <skos:broader rdf:resource="thsys/71042"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12308-5">
    <skos:broader rdf:resource="descriptor/12053-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12168-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17618-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14543-6">
    <skos:prefLabel xml:lang="en">Clock</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14543-6">
    <skos:prefLabel xml:lang="de">Uhr</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12575-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18835-1">
    <skos:prefLabel xml:lang="en">Nonprofit marketing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18835-1">
    <skos:prefLabel xml:lang="de">Nonprofit-Marketing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18894-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10785-2">
    <skos:altLabel xml:lang="en">Cross-border payments</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10785-2">
    <skos:altLabel xml:lang="de">Zahlungsverkehr mit dem Ausland</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10785-2">
    <skos:altLabel xml:lang="de">Zahlungsabkommen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10785-2">
    <skos:altLabel xml:lang="de">Swing-Vereinbarung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10785-2">
    <skos:altLabel xml:lang="de">Internationales Zahlungsabkommen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10785-2">
    <skos:altLabel xml:lang="de">Grenzüberschreitender Zahlungsverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10785-2">
    <skos:altLabel xml:lang="de">Auslandszahlungsverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12039-6">
    <skos:broader rdf:resource="thsys/72181"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12039-6">
    <skos:broader rdf:resource="thsys/71042"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12039-6">
    <skos:broader rdf:resource="descriptor/19072-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16170-2">
    <skos:related rdf:resource="descriptor/13519-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16170-2">
    <skos:related rdf:resource="descriptor/13247-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16170-2">
    <skos:related rdf:resource="descriptor/13246-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11683-5">
    <skos:broader rdf:resource="thsys/71074"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11683-5">
    <skos:broader rdf:resource="descriptor/11547-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14812-5">
    <skos:related rdf:resource="descriptor/13367-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14812-5">
    <skos:related rdf:resource="descriptor/16797-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14812-5">
    <skos:related rdf:resource="descriptor/18292-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14812-5">
    <skos:related rdf:resource="descriptor/18205-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14812-5">
    <skos:related rdf:resource="descriptor/14371-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14812-5">
    <skos:related rdf:resource="descriptor/13102-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18064-5">
    <skos:narrower rdf:resource="descriptor/10691-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14449-5">
    <skos:altLabel xml:lang="de">Frachter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12079-1">
    <skos:related rdf:resource="descriptor/12083-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12079-1">
    <skos:related rdf:resource="descriptor/24856-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12079-1">
    <skos:related rdf:resource="descriptor/15466-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12079-1">
    <skos:related rdf:resource="descriptor/19104-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13569-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10285-1">
    <skos:prefLabel xml:lang="en">Savings rate</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10285-1">
    <skos:prefLabel xml:lang="de">Sparquote</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19465-3">
    <skos:prefLabel xml:lang="en">Rural health care</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19465-3">
    <skos:prefLabel xml:lang="de">Ländliches Gesundheitswesen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18072-6">
    <skos:broader rdf:resource="thsys/71074"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18072-6">
    <skos:broader rdf:resource="thsys/70239"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18072-6">
    <skos:broader rdf:resource="thsys/70209"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18072-6">
    <skos:broader rdf:resource="descriptor/15760-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18072-6">
    <skos:broader rdf:resource="descriptor/11547-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17787-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13827-4">
    <skos:altLabel xml:lang="de">Schadenregulierung von Versicherungen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13827-4">
    <skos:altLabel xml:lang="de">Schadenmanagement von Versicherungen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16688-5">
    <skos:altLabel xml:lang="de">Siedlungsforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16688-5">
    <skos:altLabel xml:lang="de">Gemeindesoziologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16688-5">
    <skos:altLabel xml:lang="de">Gemeindeforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18561-5">
    <skos:broader rdf:resource="thsys/71036"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18561-5">
    <skos:broader rdf:resource="thsys/70944"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18561-5">
    <skos:broader rdf:resource="thsys/70924"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18561-5">
    <skos:broader rdf:resource="descriptor/12768-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70148">
    <rdfs:label xml:lang="en">Asia</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70148">
    <rdfs:label xml:lang="de">Asien</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id3">
    <rdf:value xml:lang="de">Für den Absatzbereich eines Unternehmens</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11646-4">
    <skos:related rdf:resource="descriptor/19421-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10907-0">
    <skos:related rdf:resource="descriptor/10013-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10907-0">
    <skos:related rdf:resource="descriptor/10206-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19773-6">
    <skos:scopeNote xml:lang="de">Arbeit von Bürgergruppen mit oder ohne professionelle Anleitung/Begleitung zugunsten benachteiligter Gruppen innerhalb der Gemeinde oder des Stadtteils. In Entwicklungsländern gekoppelt mit der allgemeinen Entwicklung der Gemeinde oder des Stadtteils.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13473-6">
    <skos:broader rdf:resource="thsys/70956"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13473-6">
    <skos:broader rdf:resource="descriptor/13479-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11360-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id145">
    <rdf:value xml:lang="de">Für Softwarehersteller und DV-Dienstleister</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12050-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19362-6">
    <skos:prefLabel xml:lang="en">Rural poverty</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19362-6">
    <skos:prefLabel xml:lang="de">Ländliche Armut</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10567-5">
    <skos:altLabel xml:lang="en">Free enterprise zone</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10567-5">
    <skos:altLabel xml:lang="en">Free economic zone</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10567-5">
    <skos:altLabel xml:lang="en">Enterprise zone</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10567-5">
    <skos:altLabel xml:lang="de">Unternehmensfreizone</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10567-5">
    <skos:altLabel xml:lang="de">Sonderwirtschaftszone</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10567-5">
    <skos:altLabel xml:lang="de">Industriefreizone</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19665-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19634-6">
    <skos:scopeNote xml:lang="de">Wissenschaft, die die materialen Überreste alter Kulturen untersucht</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70858">
    <rdfs:label xml:lang="de">Finanztheorie und Public Choice</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10060-4">
    <skos:related rdf:resource="descriptor/19214-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10060-4">
    <skos:related rdf:resource="descriptor/10147-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10060-4">
    <skos:related rdf:resource="descriptor/18155-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10060-4">
    <skos:related rdf:resource="descriptor/11160-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10060-4">
    <skos:related rdf:resource="descriptor/11161-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10060-4">
    <skos:related rdf:resource="descriptor/16759-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10060-4">
    <skos:related rdf:resource="descriptor/10140-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14885-6">
    <skos:related rdf:resource="descriptor/18244-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14885-6">
    <skos:related rdf:resource="descriptor/13130-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10750-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19423-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10291-6">
    <skos:altLabel xml:lang="de">Gesamtwirtschaftliche Investitionsfunktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10625-3">
    <skos:narrower rdf:resource="descriptor/10586-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10222-4">
    <skos:altLabel xml:lang="de">Unvollständige Information</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10222-4">
    <skos:altLabel xml:lang="de">Imperfect Information</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16556-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#prefLabel">
    <skos:definition xml:lang="en">The preferred lexical label for a resource, in a given language.</skos:definition>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14198-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19734-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14434-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15320-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70254">
    <skos:broader rdf:resource="thsys/70135"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17987-4">
    <skos:broader rdf:resource="thsys/70307"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70486">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.03.03</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24740-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11375-2">
    <skos:related rdf:resource="descriptor/11377-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11375-2">
    <skos:related rdf:resource="descriptor/15925-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11375-2">
    <skos:related rdf:resource="descriptor/11355-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11375-2">
    <skos:related rdf:resource="descriptor/18856-0"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id179">
    <rdf:value xml:lang="de">Für Veteranen</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19794-5">
    <skos:broader rdf:resource="thsys/73345"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19794-5">
    <skos:broader rdf:resource="descriptor/15757-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17726-4">
    <skos:prefLabel xml:lang="en">Equatorial Guinea</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17726-4">
    <skos:prefLabel xml:lang="de">Äquatorialguinea</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19715-6">
    <skos:scopeNote xml:lang="de">Vermittlungsbemühungen durch neutrale Instanzen, die meist jedoch keine eigenen formalen Vermittlungsvorschläge unterbreiten. Mediation gibt es im sozialen Bereich, bei Umweltschutzkonflikten und bei internationalen politischen Konflikten.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12027-6">
    <skos:broader rdf:resource="thsys/70889"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12027-6">
    <skos:broader rdf:resource="descriptor/18747-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18870-6">
    <skos:prefLabel xml:lang="en">School law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18870-6">
    <skos:prefLabel xml:lang="de">Schulrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14011-4">
    <skos:broader rdf:resource="thsys/71052"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14011-4">
    <skos:broader rdf:resource="descriptor/13349-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19351-4">
    <skos:altLabel xml:lang="en">Trade Block</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19351-4">
    <skos:altLabel xml:lang="en">Trade bloc</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18229-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16513-5">
    <skos:altLabel xml:lang="de">Rassentrennung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16513-5">
    <skos:altLabel xml:lang="de">Apartheid</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71055">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.13.06.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16112-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19577-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18173-0">
    <skos:prefLabel xml:lang="en">Efficiency wages</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18173-0">
    <skos:prefLabel xml:lang="de">Effizienzlohn</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14844-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14490-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18982-2">
    <skos:narrower rdf:resource="descriptor/18983-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14020-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18175-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12950-3">
    <skos:prefLabel xml:lang="en">Cattle production</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12950-3">
    <skos:prefLabel xml:lang="de">Rinderhaltung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13255-2">
    <skos:related rdf:resource="descriptor/13527-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13255-2">
    <skos:related rdf:resource="descriptor/13063-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13255-2">
    <skos:related rdf:resource="descriptor/14428-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13255-2">
    <skos:related rdf:resource="descriptor/13574-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13255-2">
    <skos:related rdf:resource="descriptor/16166-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13255-2">
    <skos:related rdf:resource="descriptor/18575-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13255-2">
    <skos:related rdf:resource="descriptor/18572-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13255-2">
    <skos:related rdf:resource="descriptor/18569-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13255-2">
    <skos:related rdf:resource="descriptor/13523-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12001-3">
    <skos:prefLabel xml:lang="en">Need of infrastructure</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12001-3">
    <skos:prefLabel xml:lang="de">Infrastrukturbedarf</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18119-6">
    <skos:broader rdf:resource="thsys/70087"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18119-6">
    <skos:broader rdf:resource="descriptor/12056-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18772-6">
    <skos:related rdf:resource="descriptor/16496-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18772-6">
    <skos:related rdf:resource="descriptor/16495-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18772-6">
    <skos:related rdf:resource="descriptor/16182-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16429-1">
    <skos:prefLabel xml:lang="en">Gender equality</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16429-1">
    <skos:prefLabel xml:lang="de">Gleichberechtigung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70929">
    <rdfs:label xml:lang="en">Transport research</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70929">
    <rdfs:label xml:lang="de">Verkehrswissenschaft</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11547-6">
    <skos:broader rdf:resource="thsys/71081"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11547-6">
    <skos:broader rdf:resource="thsys/71074"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11547-6">
    <skos:broader rdf:resource="thsys/70046"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11547-6">
    <skos:broader rdf:resource="descriptor/11545-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70866">
    <rdfs:label xml:lang="de">Auswärtige Angelegenheiten</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16250-4">
    <skos:related rdf:resource="descriptor/19682-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16250-4">
    <skos:related rdf:resource="descriptor/16298-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16250-4">
    <skos:related rdf:resource="descriptor/19455-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16250-4">
    <skos:related rdf:resource="descriptor/16245-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16250-4">
    <skos:related rdf:resource="descriptor/13451-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16250-4">
    <skos:related rdf:resource="descriptor/13425-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16250-4">
    <skos:related rdf:resource="descriptor/19748-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11565-4">
    <skos:altLabel xml:lang="de">Kriegsrente</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11565-4">
    <skos:altLabel xml:lang="de">Kriegspension</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11565-4">
    <skos:altLabel xml:lang="de">Kriegsopferversorgung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11392-2">
    <skos:narrower rdf:resource="descriptor/18882-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21101-5">
    <skos:prefLabel xml:lang="en">Currency changeover</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21101-5">
    <skos:prefLabel xml:lang="de">Währungsumstellung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13296-2">
    <skos:prefLabel xml:lang="en">Reinsurance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13296-2">
    <skos:prefLabel xml:lang="de">Rückversicherung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15759-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70021">
    <skos:narrower rdf:resource="descriptor/15076-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70021">
    <skos:narrower rdf:resource="descriptor/10031-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70021">
    <skos:narrower rdf:resource="descriptor/10239-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70021">
    <skos:narrower rdf:resource="descriptor/10342-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70021">
    <skos:narrower rdf:resource="descriptor/10343-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70021">
    <skos:narrower rdf:resource="descriptor/15075-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70021">
    <skos:narrower rdf:resource="descriptor/10025-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70021">
    <skos:narrower rdf:resource="thsys/70406"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70021">
    <skos:narrower rdf:resource="thsys/70020"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70021">
    <skos:narrower rdf:resource="thsys/71110"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70021">
    <skos:narrower rdf:resource="thsys/71109"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10459-1">
    <skos:prefLabel xml:lang="en">Innovation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10459-1">
    <skos:prefLabel xml:lang="de">Innovation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11039-4">
    <skos:related rdf:resource="descriptor/19285-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11039-4">
    <skos:related rdf:resource="descriptor/11063-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11039-4">
    <skos:related rdf:resource="descriptor/10102-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11039-4">
    <skos:related rdf:resource="descriptor/18151-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11039-4">
    <skos:related rdf:resource="descriptor/19533-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19529-3">
    <skos:related rdf:resource="descriptor/12605-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19529-3">
    <skos:related rdf:resource="descriptor/15757-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19529-3">
    <skos:related rdf:resource="descriptor/18445-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19529-3">
    <skos:related rdf:resource="descriptor/15658-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19529-3">
    <skos:related rdf:resource="descriptor/15628-0"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id97">
    <rdfs:seeAlso rdf:resource="descriptor/13248-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11545-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16028-5">
    <skos:broader rdf:resource="thsys/73392"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16526-3">
    <skos:altLabel xml:lang="de">Verrentung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16526-3">
    <skos:altLabel xml:lang="de">Renteneintrittsalter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16526-3">
    <skos:altLabel xml:lang="de">Renteneintritt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16526-3">
    <skos:altLabel xml:lang="de">Rentenalter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16526-3">
    <skos:altLabel xml:lang="de">Pensionsalter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16526-3">
    <skos:altLabel xml:lang="de">Pensionierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16526-3">
    <skos:altLabel xml:lang="de">Lebensarbeitszeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13371-0">
    <skos:prefLabel xml:lang="en">Health service</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13371-0">
    <skos:prefLabel xml:lang="de">Gesundheitswesen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11879-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73376">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13715-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13456-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16896-5">
    <skos:prefLabel xml:lang="en">Lower Saxony</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16896-5">
    <skos:prefLabel xml:lang="de">Niedersachsen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16951-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13715-1">
    <skos:prefLabel xml:lang="en">Stock price</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13715-1">
    <skos:prefLabel xml:lang="de">Börsenkurs</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15899-4">
    <skos:broader rdf:resource="thsys/73353"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15899-4">
    <skos:broader rdf:resource="thsys/70931"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15911-5">
    <skos:broader rdf:resource="thsys/73370"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15911-5">
    <skos:broader rdf:resource="thsys/70833"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15911-5">
    <skos:broader rdf:resource="thsys/70324"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15911-5">
    <skos:broader rdf:resource="thsys/70142"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15911-5">
    <skos:broader rdf:resource="descriptor/16545-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11301-3">
    <skos:related rdf:resource="descriptor/18509-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13345-1">
    <skos:related rdf:resource="descriptor/19475-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13345-1">
    <skos:related rdf:resource="descriptor/16788-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13345-1">
    <skos:related rdf:resource="descriptor/15961-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13345-1">
    <skos:related rdf:resource="descriptor/18943-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13345-1">
    <skos:related rdf:resource="descriptor/16604-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13345-1">
    <skos:related rdf:resource="descriptor/16754-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19212-4">
    <skos:broader rdf:resource="thsys/71083"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19212-4">
    <skos:broader rdf:resource="thsys/70274"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19212-4">
    <skos:broader rdf:resource="descriptor/10975-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19212-4">
    <skos:broader rdf:resource="descriptor/11715-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19080-2">
    <skos:broader rdf:resource="thsys/71046"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12963-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13993-1">
    <skos:related rdf:resource="descriptor/12160-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13993-1">
    <skos:related rdf:resource="descriptor/11559-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13993-1">
    <skos:related rdf:resource="descriptor/13413-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10834-1">
    <skos:broader rdf:resource="thsys/71091"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10834-1">
    <skos:broader rdf:resource="thsys/70952"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10834-1">
    <skos:broader rdf:resource="descriptor/10213-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11177-6">
    <skos:prefLabel xml:lang="en">Club theory</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11177-6">
    <skos:prefLabel xml:lang="de">Klubtheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14227-2">
    <skos:altLabel xml:lang="de">Zirkon</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14227-2">
    <skos:altLabel xml:lang="de">Schmuckstein</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14227-2">
    <skos:altLabel xml:lang="de">Halbedelstein</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12995-2">
    <skos:prefLabel xml:lang="en">Basic chemical industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12995-2">
    <skos:prefLabel xml:lang="de">Chemische Grundstoffindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18784-6">
    <skos:related rdf:resource="descriptor/12205-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18784-6">
    <skos:related rdf:resource="descriptor/12262-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18784-6">
    <skos:related rdf:resource="descriptor/18874-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18784-6">
    <skos:related rdf:resource="descriptor/18774-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18784-6">
    <skos:related rdf:resource="descriptor/19584-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18784-6">
    <skos:related rdf:resource="descriptor/18785-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18664-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11496-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18985-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70179">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14439-1">
    <skos:broader rdf:resource="thsys/70258"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14439-1">
    <skos:broader rdf:resource="descriptor/14438-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14021-1">
    <skos:narrower rdf:resource="descriptor/14258-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14021-1">
    <skos:narrower rdf:resource="descriptor/19050-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14021-1">
    <skos:narrower rdf:resource="descriptor/14251-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14021-1">
    <skos:narrower rdf:resource="descriptor/14249-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14021-1">
    <skos:narrower rdf:resource="descriptor/14784-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14021-1">
    <skos:narrower rdf:resource="descriptor/14020-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14567-6">
    <skos:altLabel xml:lang="de">Schweißgerät</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14567-6">
    <skos:altLabel xml:lang="de">Schleifapparat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14567-6">
    <skos:altLabel xml:lang="de">Handwerkzeug</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14567-6">
    <skos:altLabel xml:lang="de">Elektrowerkzeug</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15211-5">
    <skos:scopeNote xml:lang="de">Sammelbegriff für Häufigkeitsverteilungen, Verteilungsfunktionen, Dichtefunktionen, Summenfunktionen oder Wahrscheinlichkeitsfunktionen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19370-0">
    <skos:prefLabel xml:lang="en">State finances</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19370-0">
    <skos:prefLabel xml:lang="de">Länderfinanzen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18521-3">
    <skos:related rdf:resource="descriptor/18299-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15089-4">
    <skos:prefLabel xml:lang="en">Wage statistics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15089-4">
    <skos:prefLabel xml:lang="de">Lohnstatistik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14599-0">
    <skos:related rdf:resource="descriptor/18280-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14599-0">
    <skos:related rdf:resource="descriptor/14601-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14599-0">
    <skos:related rdf:resource="descriptor/15992-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14599-0">
    <skos:related rdf:resource="descriptor/13198-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14599-0">
    <skos:related rdf:resource="descriptor/12996-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14599-0">
    <skos:related rdf:resource="descriptor/15638-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19711-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19399-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13145-2">
    <skos:narrower rdf:resource="descriptor/13149-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13145-2">
    <skos:narrower rdf:resource="descriptor/13148-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13145-2">
    <skos:narrower rdf:resource="descriptor/13146-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13145-2">
    <skos:narrower rdf:resource="descriptor/13131-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13145-2">
    <skos:narrower rdf:resource="descriptor/13130-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13145-2">
    <skos:narrower rdf:resource="descriptor/13127-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18217-6">
    <skos:broader rdf:resource="thsys/73303"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18217-6">
    <skos:broader rdf:resource="descriptor/14955-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12573-0">
    <skos:prefLabel xml:lang="en">Hiring</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12573-0">
    <skos:prefLabel xml:lang="de">Personalbeschaffung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10192-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16680-0">
    <skos:prefLabel xml:lang="en">Cultural sociology</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16680-0">
    <skos:prefLabel xml:lang="de">Kultursoziologie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20455-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14778-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13537-6">
    <skos:broader rdf:resource="thsys/70933"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13537-6">
    <skos:broader rdf:resource="descriptor/13620-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70999">
    <skos:narrower rdf:resource="descriptor/18416-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70999">
    <skos:narrower rdf:resource="descriptor/18417-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70999">
    <skos:narrower rdf:resource="descriptor/13066-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70999">
    <skos:narrower rdf:resource="descriptor/13063-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70999">
    <skos:narrower rdf:resource="descriptor/13047-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70999">
    <skos:narrower rdf:resource="thsys/70997"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70999">
    <skos:narrower rdf:resource="thsys/70998"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73378">
    <skos:prefLabel xml:lang="de">N.04.02.04  Politische Organisationen und Bewegungen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16470-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10613-3">
    <skos:narrower rdf:resource="descriptor/10491-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12287-1">
    <skos:related rdf:resource="descriptor/18105-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12287-1">
    <skos:related rdf:resource="descriptor/12283-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12287-1">
    <skos:related rdf:resource="descriptor/12280-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12287-1">
    <skos:related rdf:resource="descriptor/19835-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12287-1">
    <skos:related rdf:resource="descriptor/12281-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12287-1">
    <skos:related rdf:resource="descriptor/18671-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12287-1">
    <skos:related rdf:resource="descriptor/13723-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12287-1">
    <skos:related rdf:resource="descriptor/11414-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12508-4">
    <skos:broader rdf:resource="thsys/70578"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12508-4">
    <skos:broader rdf:resource="thsys/70270"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12508-4">
    <skos:broader rdf:resource="descriptor/10011-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20306-6">
    <skos:prefLabel xml:lang="en">Product safety</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20306-6">
    <skos:prefLabel xml:lang="de">Produktsicherheit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19795-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19161-2">
    <skos:prefLabel xml:lang="en">Owner</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19161-2">
    <skos:prefLabel xml:lang="de">Eigentümer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13665-4">
    <skos:altLabel xml:lang="de">Rundfunkdienst</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13665-4">
    <skos:altLabel xml:lang="de">Radiodienst</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13665-4">
    <skos:altLabel xml:lang="de">Hörfunk</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71080">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.09.04</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18790-4">
    <skos:related rdf:resource="descriptor/10616-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18790-4">
    <skos:related rdf:resource="descriptor/19324-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18790-4">
    <skos:related rdf:resource="descriptor/10606-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10088-3">
    <skos:altLabel xml:lang="en">Voter's paradoxon</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10088-3">
    <skos:altLabel xml:lang="de">Zyklische Mehrheiten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10088-3">
    <skos:altLabel xml:lang="de">Wahlparadoxon</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10088-3">
    <skos:altLabel xml:lang="de">Condorcet-Paradoxon</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10088-3">
    <skos:altLabel xml:lang="de">Condorcet Paradoxon</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10088-3">
    <skos:altLabel xml:lang="de">Arrow-Paradoxon</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18817-3">
    <skos:scopeNote xml:lang="de">Volkswirtschaftliche Analyse des Verteidigungshaushalts sowie sicherheits-und rüstungspolitischer Problemstellungen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13157-2">
    <skos:broader rdf:resource="thsys/73295"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13157-2">
    <skos:broader rdf:resource="descriptor/13160-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73348">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19444-4">
    <skos:related rdf:resource="descriptor/10512-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19444-4">
    <skos:related rdf:resource="descriptor/19443-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12297-5">
    <skos:prefLabel xml:lang="en">Accrual</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12297-5">
    <skos:prefLabel xml:lang="de">Rückstellung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10073-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15725-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15974-2">
    <skos:altLabel xml:lang="de">Verfassungsstaat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15974-2">
    <skos:altLabel xml:lang="de">Rechtsstaatsprinzip</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15974-2">
    <skos:altLabel xml:lang="de">Rechtsstaatliche Ordnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11613-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19806-3">
    <skos:broader rdf:resource="thsys/73339"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11295-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14028-1">
    <skos:prefLabel xml:lang="en">Prefabricated house</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14028-1">
    <skos:prefLabel xml:lang="de">Fertighaus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18486-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18591-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11040-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12432-4">
    <skos:narrower rdf:resource="descriptor/12392-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12988-6">
    <skos:broader rdf:resource="thsys/71007"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12988-6">
    <skos:broader rdf:resource="descriptor/12985-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12326-3">
    <skos:prefLabel xml:lang="en">Accounting standards</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12326-3">
    <skos:prefLabel xml:lang="de">Bilanzrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19536-6">
    <skos:scopeNote xml:lang="de">Ermittlung des Anfangskapitals für ein gegebenes Endkapital bzw. Bestimmung des Bar- oder Gegenwartswertes zukünftiger Zahlungen durch Multiplikation mit dem Abzinsungsfaktor.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71086">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16429-1">
    <skos:broader rdf:resource="thsys/73395"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16429-1">
    <skos:broader rdf:resource="thsys/73377"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18221-1">
    <skos:prefLabel xml:lang="en">Soft drink industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18221-1">
    <skos:prefLabel xml:lang="de">Erfrischungsgetränkeindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11336-5">
    <skos:prefLabel xml:lang="en">Wage theory</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11336-5">
    <skos:prefLabel xml:lang="de">Lohntheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18281-4">
    <skos:scopeNote xml:lang="de">Gekoppelte Abgabe von elektrischem Strom und Fern- bzw. Prozesswärme</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19039-6">
    <skos:prefLabel xml:lang="en">Supply situation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19039-6">
    <skos:prefLabel xml:lang="de">Versorgung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19202-0">
    <skos:broader rdf:resource="thsys/70049"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11529-1">
    <skos:altLabel xml:lang="en">Cost-utility analysis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11529-1">
    <skos:altLabel xml:lang="en">Cost utility analysis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11529-1">
    <skos:altLabel xml:lang="de">Kosten-Nutzenanalyse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11529-1">
    <skos:altLabel xml:lang="de">Kosten-Ertragsanalyse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11529-1">
    <skos:altLabel xml:lang="de">Kosten-Ertrags-Analyse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12985-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18391-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13462-4">
    <skos:narrower rdf:resource="descriptor/18534-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12368-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14614-2">
    <skos:broader rdf:resource="thsys/73305"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14614-2">
    <skos:broader rdf:resource="thsys/70265"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14614-2">
    <skos:broader rdf:resource="thsys/70014"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14614-2">
    <skos:broader rdf:resource="descriptor/14816-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14614-2">
    <skos:broader rdf:resource="descriptor/14514-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71031">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16397-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19262-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11393-0">
    <skos:altLabel xml:lang="en">Basic training</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11393-0">
    <skos:altLabel xml:lang="de">Grundausbildung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11393-0">
    <skos:altLabel xml:lang="de">Betriebspraktikum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11393-0">
    <skos:altLabel xml:lang="de">Berufsgrundbildungsjahr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11393-0">
    <skos:altLabel xml:lang="de">Berufsgrundbildung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20409-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14490-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19676-4">
    <skos:broader rdf:resource="thsys/70839"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19676-4">
    <skos:broader rdf:resource="descriptor/19966-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19676-4">
    <skos:broader rdf:resource="descriptor/10006-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18551-1">
    <skos:altLabel xml:lang="de">Supermarkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18551-1">
    <skos:altLabel xml:lang="de">SB-Lebensmitteleinzelhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18551-1">
    <skos:altLabel xml:lang="de">Lebensmittel-SB-Markt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18551-1">
    <skos:altLabel xml:lang="de">Lebensmittel-SB-Geschäft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10778-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14056-3">
    <skos:altLabel xml:lang="de">Erdnuß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15089-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19756-6">
    <skos:altLabel xml:lang="en">Sex discrimination</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19756-6">
    <skos:altLabel xml:lang="de">Sexuelle Diskriminierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19756-6">
    <skos:altLabel xml:lang="de">Sexismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19756-6">
    <skos:altLabel xml:lang="de">Geschlechtsspezifische Diskriminierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19756-6">
    <skos:altLabel xml:lang="de">Geschlechtsbedingte Diskriminierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19756-6">
    <skos:altLabel xml:lang="de">Frauenfeindlichkeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19756-6">
    <skos:altLabel xml:lang="de">Frauendiskriminierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19756-6">
    <skos:altLabel xml:lang="de">Diskriminierung von Frauen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19560-2">
    <skos:broader rdf:resource="thsys/71071"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19560-2">
    <skos:broader rdf:resource="thsys/70239"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15346-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10052-3">
    <skos:prefLabel xml:lang="en">Opportunity costs</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10052-3">
    <skos:prefLabel xml:lang="de">Opportunitätskosten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19799-2">
    <skos:altLabel xml:lang="de">Werkstofftechnologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18660-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13279-2">
    <skos:broader rdf:resource="thsys/70893"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13279-2">
    <skos:broader rdf:resource="thsys/70187"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13279-2">
    <skos:broader rdf:resource="descriptor/13470-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70305">
    <rdfs:label xml:lang="en">Monetary policy</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70305">
    <rdfs:label xml:lang="de">Geldpolitik</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70161">
    <skos:narrower rdf:resource="descriptor/15017-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70161">
    <skos:narrower rdf:resource="descriptor/15011-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70161">
    <skos:narrower rdf:resource="descriptor/14273-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70161">
    <skos:narrower rdf:resource="descriptor/15013-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70161">
    <skos:narrower rdf:resource="descriptor/18230-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70161">
    <skos:narrower rdf:resource="descriptor/15019-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70161">
    <skos:narrower rdf:resource="descriptor/18204-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70161">
    <skos:narrower rdf:resource="descriptor/15018-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70161">
    <skos:narrower rdf:resource="descriptor/18229-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70161">
    <skos:narrower rdf:resource="descriptor/14796-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70161">
    <skos:narrower rdf:resource="descriptor/15729-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70161">
    <skos:narrower rdf:resource="descriptor/15010-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70161">
    <skos:narrower rdf:resource="descriptor/15009-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14778-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20321-3">
    <skos:scopeNote xml:lang="de">Die Theorie der NEG dient der Erklärung räumlicher Wirtschaftsstrukturen. Es wird untersucht, welche Faktoren dazu führen, dass es in einigen Regionen zur Ballung kommt. Der Ansatz von KRUGMAN basiert auf der Kombination von Elementen der neuen Außenhandelstheorie mit Ideen der traditionellen Standortlehre. Die Stärke dieser Modellstruktur ist ihre endogene Erklärungskraft für das Entstehen von ökonomischen Kern-Peripherie-Strukturen</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18123-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16611-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24787-0">
    <skos:altLabel xml:lang="en">3PL (Third party logistics)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24787-0">
    <skos:altLabel xml:lang="en">TPL (Third party logistics)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24787-0">
    <skos:altLabel xml:lang="en">Third party logistics</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24787-0">
    <skos:altLabel xml:lang="de">Logistikserviceunternehmen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24787-0">
    <skos:altLabel xml:lang="de">Logistikgewerbe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24787-0">
    <skos:altLabel xml:lang="de">Logistikdienstleister</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24787-0">
    <skos:altLabel xml:lang="de">Logistikbranche</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24787-0">
    <skos:altLabel xml:lang="de">Logistikbetrieb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24787-0">
    <skos:altLabel xml:lang="de">Kontraktlogistik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14466-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10221-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14449-5">
    <skos:narrower rdf:resource="descriptor/14462-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14449-5">
    <skos:narrower rdf:resource="descriptor/18248-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14449-5">
    <skos:narrower rdf:resource="descriptor/18242-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14393-6">
    <skos:narrower rdf:resource="descriptor/12738-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14393-6">
    <skos:narrower rdf:resource="descriptor/14382-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14393-6">
    <skos:narrower rdf:resource="descriptor/14377-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13591-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14662-5">
    <skos:broader rdf:resource="thsys/70556"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14662-5">
    <skos:broader rdf:resource="descriptor/14683-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16532-1">
    <skos:related rdf:resource="descriptor/13411-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16532-1">
    <skos:related rdf:resource="descriptor/16531-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19408-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24783-1">
    <skos:related rdf:resource="descriptor/16336-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24783-1">
    <skos:related rdf:resource="descriptor/18822-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24783-1">
    <skos:related rdf:resource="descriptor/18823-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15945-2">
    <skos:prefLabel xml:lang="en">Gender</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15945-2">
    <skos:prefLabel xml:lang="de">Geschlecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16298-4">
    <skos:altLabel xml:lang="de">Fürstentum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16298-4">
    <skos:altLabel xml:lang="de">Dynastie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15494-2">
    <skos:prefLabel xml:lang="en">Enumeration algorithm</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15494-2">
    <skos:prefLabel xml:lang="de">Enumeration</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11916-1">
    <skos:altLabel xml:lang="de">Wirtschaftsregion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11916-1">
    <skos:altLabel xml:lang="de">Wirtschaftsgebiet</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16300-1">
    <skos:scopeNote xml:lang="de">Sich als Nation begreifender souveräner Staat</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15611-3">
    <skos:broader rdf:resource="thsys/73339"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15611-3">
    <skos:broader rdf:resource="descriptor/15609-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12974-3">
    <skos:related rdf:resource="descriptor/20888-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12974-3">
    <skos:related rdf:resource="descriptor/18354-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12974-3">
    <skos:related rdf:resource="descriptor/19838-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12974-3">
    <skos:related rdf:resource="descriptor/18360-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12974-3">
    <skos:related rdf:resource="descriptor/14174-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73306">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">P.06.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11633-6">
    <skos:related rdf:resource="descriptor/11125-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11633-6">
    <skos:related rdf:resource="descriptor/11628-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11633-6">
    <skos:related rdf:resource="descriptor/11728-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11633-6">
    <skos:related rdf:resource="descriptor/18992-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12225-2">
    <skos:prefLabel xml:lang="en">Capital requirements</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12225-2">
    <skos:prefLabel xml:lang="de">Kapitalbedarf</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73346">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.10.05.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id112">
    <rdf:value xml:lang="de">Für Gerechtigkeit der sozialen Verhältnisse</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11847-1">
    <skos:broader rdf:resource="thsys/71110"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11847-1">
    <skos:broader rdf:resource="thsys/70888"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11847-1">
    <skos:broader rdf:resource="descriptor/16273-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13418-0">
    <skos:related rdf:resource="descriptor/18994-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13418-0">
    <skos:related rdf:resource="descriptor/13411-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13418-0">
    <skos:related rdf:resource="descriptor/16160-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13418-0">
    <skos:related rdf:resource="descriptor/18913-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13418-0">
    <skos:related rdf:resource="descriptor/13417-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13418-0">
    <skos:related rdf:resource="descriptor/19373-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13418-0">
    <skos:related rdf:resource="descriptor/16143-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17627-6">
    <skos:prefLabel xml:lang="en">Qatar</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17627-6">
    <skos:prefLabel xml:lang="de">Katar</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10984-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18806-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70135">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">P.10</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16058-3">
    <skos:altLabel xml:lang="de">Zwangsvergleich</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16058-3">
    <skos:altLabel xml:lang="de">Vergleichsverfahren bei Insolvenz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16058-3">
    <skos:altLabel xml:lang="de">Vergleichsordnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16058-3">
    <skos:altLabel xml:lang="de">Konkursrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16058-3">
    <skos:altLabel xml:lang="de">Konkursordnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16058-3">
    <skos:altLabel xml:lang="de">Insolvenzverfahren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16058-3">
    <skos:altLabel xml:lang="de">Insolvency code</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17621-4">
    <skos:prefLabel xml:lang="en">Iraq</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17621-4">
    <skos:prefLabel xml:lang="de">Irak</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16158-6">
    <skos:prefLabel xml:lang="en">Social welfare law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16158-6">
    <skos:prefLabel xml:lang="de">Sozialhilferecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18642-5">
    <skos:broader rdf:resource="thsys/70923"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18642-5">
    <skos:broader rdf:resource="descriptor/11633-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18642-5">
    <skos:broader rdf:resource="descriptor/13647-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18841-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70372">
    <skos:narrower rdf:resource="descriptor/14824-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70372">
    <skos:narrower rdf:resource="descriptor/14822-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70372">
    <skos:narrower rdf:resource="descriptor/14821-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70372">
    <skos:narrower rdf:resource="descriptor/14819-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70372">
    <skos:narrower rdf:resource="descriptor/14817-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19309-3">
    <skos:prefLabel xml:lang="en">Market integration</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19309-3">
    <skos:prefLabel xml:lang="de">Marktintegration</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70959">
    <rdfs:label xml:lang="en">Consumer goods trade</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70959">
    <rdfs:label xml:lang="de">Konsumgüterhandel</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14046-6">
    <skos:related rdf:resource="descriptor/13229-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14046-6">
    <skos:related rdf:resource="descriptor/14619-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14046-6">
    <skos:related rdf:resource="descriptor/14538-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18236-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16533-6">
    <skos:scopeNote xml:lang="de">Staatliche Maßnahmen zur Bekämpfung der Armut.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10395-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14215-2">
    <skos:broader rdf:resource="thsys/70538"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14215-2">
    <skos:broader rdf:resource="descriptor/14218-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14215-2">
    <skos:broader rdf:resource="descriptor/14173-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17624-5">
    <skos:altLabel xml:lang="de">Südjemen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17624-5">
    <skos:altLabel xml:lang="de">Republik Jemen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17624-5">
    <skos:altLabel xml:lang="de">Nordjemen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17624-5">
    <skos:altLabel xml:lang="de">Jemen (Demokratische Volksrepublik)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17624-5">
    <skos:altLabel xml:lang="de">Jemen (Arabische Republik)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17624-5">
    <skos:altLabel xml:lang="de">Demokratische Volksrepublik Jemen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17624-5">
    <skos:altLabel xml:lang="de">Arabische Republik Jemen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18381-0">
    <skos:related rdf:resource="descriptor/12922-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18381-0">
    <skos:related rdf:resource="descriptor/15752-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18381-0">
    <skos:related rdf:resource="descriptor/18995-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18381-0">
    <skos:related rdf:resource="descriptor/14047-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18381-0">
    <skos:related rdf:resource="descriptor/15745-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18381-0">
    <skos:related rdf:resource="descriptor/18325-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10833-3">
    <skos:altLabel xml:lang="de">Internationaler Handel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10626-1">
    <skos:narrower rdf:resource="descriptor/10628-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10626-1">
    <skos:narrower rdf:resource="descriptor/10627-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10626-1">
    <skos:narrower rdf:resource="descriptor/10771-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10626-1">
    <skos:narrower rdf:resource="descriptor/19309-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10626-1">
    <skos:narrower rdf:resource="descriptor/18181-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10626-1">
    <skos:narrower rdf:resource="descriptor/19351-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10626-1">
    <skos:narrower rdf:resource="descriptor/10612-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10626-1">
    <skos:narrower rdf:resource="descriptor/19350-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19033-4">
    <skos:broader rdf:resource="thsys/70582"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13415-6">
    <skos:broader rdf:resource="thsys/70839"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13415-6">
    <skos:broader rdf:resource="thsys/70832"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13415-6">
    <skos:broader rdf:resource="thsys/70573"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13415-6">
    <skos:broader rdf:resource="thsys/70458"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13415-6">
    <skos:broader rdf:resource="descriptor/13418-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13415-6">
    <skos:broader rdf:resource="descriptor/13292-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19354-5">
    <skos:related rdf:resource="descriptor/19663-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19354-5">
    <skos:related rdf:resource="descriptor/19242-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11457-0">
    <skos:related rdf:resource="descriptor/10136-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11457-0">
    <skos:related rdf:resource="descriptor/19258-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11457-0">
    <skos:related rdf:resource="descriptor/11454-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11457-0">
    <skos:related rdf:resource="descriptor/11460-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17589-2">
    <skos:broader rdf:resource="thsys/70327"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17589-2">
    <skos:broader rdf:resource="descriptor/17553-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14174-4">
    <skos:altLabel xml:lang="de">Strom (Energie)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14174-4">
    <skos:altLabel xml:lang="de">Elektroenergie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70386">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">B.05.02.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14335-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13563-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12244-5">
    <skos:prefLabel xml:lang="en">Capital structure</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12244-5">
    <skos:prefLabel xml:lang="de">Kapitalstruktur</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/n">
    <rdfs:label xml:lang="en">Related subject areas</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/n">
    <rdfs:label xml:lang="de">Nachbarwissenschaften</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14025-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19781-0">
    <skos:prefLabel xml:lang="en">Cultural asset</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19781-0">
    <skos:prefLabel xml:lang="de">Kulturgut</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13199-0">
    <skos:related rdf:resource="descriptor/14026-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13199-0">
    <skos:related rdf:resource="descriptor/13192-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13578-6">
    <skos:altLabel xml:lang="de">Raumfahrttechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10817-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id279">
    <rdf:value xml:lang="de">Für die Arbeitslosigkeit einer Region allein</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19335-2">
    <skos:related rdf:resource="descriptor/10773-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10443-2">
    <skos:prefLabel xml:lang="en">Corporate research</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10443-2">
    <skos:prefLabel xml:lang="de">Betriebliche Forschung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19766-3">
    <skos:broader rdf:resource="thsys/70013"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19766-3">
    <skos:broader rdf:resource="descriptor/16564-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14504-2">
    <skos:altLabel xml:lang="de">Toaster</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14504-2">
    <skos:altLabel xml:lang="de">Munddusche</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14504-2">
    <skos:altLabel xml:lang="de">Küchenmixer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14504-2">
    <skos:altLabel xml:lang="de">Kaffeemaschine</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14504-2">
    <skos:altLabel xml:lang="de">Haartrockner</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14504-2">
    <skos:altLabel xml:lang="de">Haarpflegegerät</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14504-2">
    <skos:altLabel xml:lang="de">Elektrozahnbürste</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14504-2">
    <skos:altLabel xml:lang="de">Bügeleisen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16210-2">
    <skos:related rdf:resource="descriptor/11060-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16210-2">
    <skos:related rdf:resource="descriptor/11088-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16210-2">
    <skos:related rdf:resource="descriptor/16629-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16210-2">
    <skos:related rdf:resource="descriptor/18838-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18772-6">
    <skos:altLabel xml:lang="de">Politik der Europäischen Union</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18772-6">
    <skos:altLabel xml:lang="de">Politik der EU</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12719-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70924">
    <skos:narrower rdf:resource="descriptor/13832-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70924">
    <skos:narrower rdf:resource="descriptor/13645-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70924">
    <skos:narrower rdf:resource="descriptor/11215-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70924">
    <skos:narrower rdf:resource="descriptor/12927-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70924">
    <skos:narrower rdf:resource="descriptor/18640-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70924">
    <skos:narrower rdf:resource="descriptor/18639-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70924">
    <skos:narrower rdf:resource="descriptor/19466-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70924">
    <skos:narrower rdf:resource="descriptor/13644-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70924">
    <skos:narrower rdf:resource="descriptor/18564-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70924">
    <skos:narrower rdf:resource="descriptor/18638-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70924">
    <skos:narrower rdf:resource="descriptor/13253-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70924">
    <skos:narrower rdf:resource="descriptor/18637-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70924">
    <skos:narrower rdf:resource="descriptor/13643-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70924">
    <skos:narrower rdf:resource="descriptor/18636-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70924">
    <skos:narrower rdf:resource="descriptor/18587-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70924">
    <skos:narrower rdf:resource="descriptor/13641-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70924">
    <skos:narrower rdf:resource="descriptor/18635-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70924">
    <skos:narrower rdf:resource="descriptor/18612-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70924">
    <skos:narrower rdf:resource="descriptor/13639-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70924">
    <skos:narrower rdf:resource="descriptor/13638-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70924">
    <skos:narrower rdf:resource="descriptor/13592-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70924">
    <skos:narrower rdf:resource="descriptor/13966-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70924">
    <skos:narrower rdf:resource="descriptor/18634-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70924">
    <skos:narrower rdf:resource="descriptor/11194-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70924">
    <skos:narrower rdf:resource="descriptor/11850-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70924">
    <skos:narrower rdf:resource="descriptor/18561-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70924">
    <skos:narrower rdf:resource="descriptor/13636-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70924">
    <skos:narrower rdf:resource="descriptor/13635-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73376">
    <rdfs:label xml:lang="de">Politische Kultur und Partizipation</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18196-2">
    <skos:broader rdf:resource="thsys/71029"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18196-2">
    <skos:broader rdf:resource="descriptor/12933-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16463-1">
    <skos:prefLabel xml:lang="en">Security policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16463-1">
    <skos:prefLabel xml:lang="de">Sicherheitspolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16422-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71069">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70961">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10278-5">
    <skos:prefLabel xml:lang="en">Aggregate consumption function</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10278-5">
    <skos:prefLabel xml:lang="de">Gesamtwirtschaftliche Konsumfunktion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15657-0">
    <skos:broader rdf:resource="thsys/73296"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15657-0">
    <skos:broader rdf:resource="thsys/70828"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15657-0">
    <skos:broader rdf:resource="thsys/70581"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15657-0">
    <skos:broader rdf:resource="thsys/70580"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15657-0">
    <skos:broader rdf:resource="descriptor/15655-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16081-1">
    <skos:altLabel xml:lang="de">Wirtschaftsverwaltungsrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10433-5">
    <skos:broader rdf:resource="thsys/70406"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10433-5">
    <skos:broader rdf:resource="descriptor/10434-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17016-0">
    <skos:related rdf:resource="descriptor/17179-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17016-0">
    <skos:related rdf:resource="descriptor/18054-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19630-0">
    <skos:scopeNote xml:lang="de">Maßnahmen zur Erhaltung wildlebender Tierarten</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#narrowerTransitive">
    <owl:inverseOf rdf:resource="http://www.w3.org/2004/02/skos/core#broaderTransitive"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18074-2">
    <skos:broader rdf:resource="thsys/73338"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18074-2">
    <skos:broader rdf:resource="thsys/70847"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18074-2">
    <skos:broader rdf:resource="thsys/70143"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18074-2">
    <skos:broader rdf:resource="descriptor/15656-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19619-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19343-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12667-5">
    <skos:narrower rdf:resource="descriptor/19113-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70310">
    <skos:broader rdf:resource="thsys/70030"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13395-0">
    <skos:prefLabel xml:lang="en">Non-governmental organization</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13395-0">
    <skos:prefLabel xml:lang="de">Nichtregierungsorganisation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70869">
    <skos:prefLabel xml:lang="de">W.22.01.02  Finanzverwaltung und öffentliche Finanzwirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19390-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16379-4">
    <skos:altLabel xml:lang="de">Pluralistische Gesellschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16379-4">
    <skos:altLabel xml:lang="de">Pluralismustheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16379-4">
    <skos:altLabel xml:lang="de">Meinungsvielfalt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16379-4">
    <skos:altLabel xml:lang="de">Meinungspluralismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16379-4">
    <skos:altLabel xml:lang="de">Interessenpluralismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14295-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16460-0">
    <skos:related rdf:resource="descriptor/15005-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16460-0">
    <skos:related rdf:resource="descriptor/16271-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16460-0">
    <skos:related rdf:resource="descriptor/19724-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16460-0">
    <skos:related rdf:resource="descriptor/13412-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16460-0">
    <skos:related rdf:resource="descriptor/16470-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14092-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id253">
    <rdfs:seeAlso rdf:resource="descriptor/16286-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13285-0">
    <skos:prefLabel xml:lang="en">Credit guarantee association</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13285-0">
    <skos:prefLabel xml:lang="de">Bürgschaftsbank</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19717-2">
    <skos:broader rdf:resource="thsys/73366"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19717-2">
    <skos:broader rdf:resource="descriptor/16489-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18872-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11295-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10602-1">
    <skos:broader rdf:resource="thsys/71090"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10602-1">
    <skos:broader rdf:resource="descriptor/10605-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70179">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19501-4">
    <skos:related rdf:resource="descriptor/18910-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19501-4">
    <skos:related rdf:resource="descriptor/10006-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12997-5">
    <skos:related rdf:resource="descriptor/19397-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12997-5">
    <skos:related rdf:resource="descriptor/18516-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12997-5">
    <skos:related rdf:resource="descriptor/13427-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15666-6">
    <skos:broader rdf:resource="thsys/70847"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15666-6">
    <skos:broader rdf:resource="thsys/70143"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15666-6">
    <skos:broader rdf:resource="descriptor/16563-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15879-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19729-2">
    <skos:prefLabel xml:lang="en">Limited liability</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19729-2">
    <skos:prefLabel xml:lang="de">Beschränkte Haftung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12736-5">
    <skos:related rdf:resource="descriptor/18414-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12736-5">
    <skos:related rdf:resource="descriptor/12681-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12736-5">
    <skos:related rdf:resource="descriptor/12693-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12736-5">
    <skos:related rdf:resource="descriptor/12730-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18153-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18904-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14255-4">
    <skos:prefLabel xml:lang="en">Abrasive</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14255-4">
    <skos:prefLabel xml:lang="de">Schleifmittel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19137-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18726-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14985-2">
    <skos:altLabel xml:lang="en">Sherry</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14985-2">
    <skos:altLabel xml:lang="de">Wermutwein</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14985-2">
    <skos:altLabel xml:lang="de">Weißwein</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14985-2">
    <skos:altLabel xml:lang="de">Süßwein</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14985-2">
    <skos:altLabel xml:lang="de">Südwein</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14985-2">
    <skos:altLabel xml:lang="de">Rotwein</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14985-2">
    <skos:altLabel xml:lang="de">Rosewein</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14985-2">
    <skos:altLabel xml:lang="de">Portwein</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19108-6">
    <skos:scopeNote xml:lang="de">Betrifft den Gesamtbetrieb, Teilbereiche oder einzelne Funktionen und kann sowohl internen Erkenntnissen als auch der Verbesserung der Wettbewerbsposition dienen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70963">
    <skos:narrower rdf:resource="descriptor/13475-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70963">
    <skos:narrower rdf:resource="descriptor/12923-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70963">
    <skos:narrower rdf:resource="descriptor/18524-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70963">
    <skos:narrower rdf:resource="descriptor/13470-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70963">
    <skos:narrower rdf:resource="descriptor/13466-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70963">
    <skos:narrower rdf:resource="descriptor/18513-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70963">
    <skos:narrower rdf:resource="descriptor/11824-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70963">
    <skos:narrower rdf:resource="descriptor/13465-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70963">
    <skos:narrower rdf:resource="descriptor/13464-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70963">
    <skos:narrower rdf:resource="descriptor/11823-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70963">
    <skos:narrower rdf:resource="descriptor/11822-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70963">
    <skos:narrower rdf:resource="descriptor/13462-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70963">
    <skos:narrower rdf:resource="descriptor/13460-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70963">
    <skos:narrower rdf:resource="descriptor/13970-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70963">
    <skos:narrower rdf:resource="descriptor/13210-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70963">
    <skos:narrower rdf:resource="descriptor/12779-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70963">
    <skos:narrower rdf:resource="descriptor/13322-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70963">
    <skos:narrower rdf:resource="thsys/70952"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70963">
    <skos:narrower rdf:resource="thsys/70962"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70963">
    <skos:narrower rdf:resource="thsys/70956"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70963">
    <skos:narrower rdf:resource="thsys/70947"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70963">
    <skos:narrower rdf:resource="thsys/70953"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11675-4">
    <skos:altLabel xml:lang="de">Ressourcensteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11496-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12403-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18213-0">
    <skos:related rdf:resource="descriptor/13163-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18213-0">
    <skos:related rdf:resource="descriptor/13237-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18213-0">
    <skos:related rdf:resource="descriptor/14996-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18213-0">
    <skos:related rdf:resource="descriptor/18214-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18213-0">
    <skos:related rdf:resource="descriptor/14942-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20744-3">
    <skos:scopeNote xml:lang="de">Neuer Mechanismus für eine Klimaschutz-Kooperation zwischen Industrie- und Entwicklungsländern nach dem Kyoto-Protokoll. Der projektbezogene Mechanismus soll zwei Ziele verknüpfen: Hilfe für die Industrieländer bei der Erreichung ihrer Emissionsziele und Unterstützung der Entwicklungsländer bei der nachhaltigen Entwicklung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10506-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10076-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12191-0">
    <skos:altLabel xml:lang="de">OHG</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19683-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13381-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15093-6">
    <skos:narrower rdf:resource="descriptor/15088-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15093-6">
    <skos:narrower rdf:resource="descriptor/15086-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15093-6">
    <skos:narrower rdf:resource="descriptor/15080-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15093-6">
    <skos:narrower rdf:resource="descriptor/15077-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15093-6">
    <skos:narrower rdf:resource="descriptor/15078-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19409-6">
    <skos:prefLabel xml:lang="en">Income tax rate</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19409-6">
    <skos:prefLabel xml:lang="de">Einkommensteuertarif</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11273-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14967-4">
    <skos:broader rdf:resource="thsys/73301"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14967-4">
    <skos:broader rdf:resource="descriptor/14930-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14967-4">
    <skos:broader rdf:resource="descriptor/14969-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11378-3">
    <skos:prefLabel xml:lang="en">Students</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11378-3">
    <skos:prefLabel xml:lang="de">Studierende</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70253">
    <skos:prefLabel xml:lang="en">G.02.01  Middle East</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70253">
    <skos:prefLabel xml:lang="de">G.02.01  Mittlerer Osten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15315-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15654-6">
    <skos:related rdf:resource="descriptor/16599-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15654-6">
    <skos:related rdf:resource="descriptor/15682-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15654-6">
    <skos:related rdf:resource="descriptor/15675-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15654-6">
    <skos:related rdf:resource="descriptor/19611-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15654-6">
    <skos:related rdf:resource="descriptor/19750-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15654-6">
    <skos:related rdf:resource="descriptor/19754-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15654-6">
    <skos:related rdf:resource="descriptor/16680-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15654-6">
    <skos:related rdf:resource="descriptor/18920-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15654-6">
    <skos:related rdf:resource="descriptor/18924-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15654-6">
    <skos:related rdf:resource="descriptor/16515-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15654-6">
    <skos:related rdf:resource="descriptor/19781-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15654-6">
    <skos:related rdf:resource="descriptor/15694-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15654-6">
    <skos:related rdf:resource="descriptor/16648-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15654-6">
    <skos:related rdf:resource="descriptor/16556-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19699-6">
    <skos:related rdf:resource="descriptor/16355-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19699-6">
    <skos:related rdf:resource="descriptor/16354-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19699-6">
    <skos:related rdf:resource="descriptor/18097-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19699-6">
    <skos:related rdf:resource="descriptor/16008-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19699-6">
    <skos:related rdf:resource="descriptor/16365-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19346-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16688-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11552-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12181-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19247-6">
    <skos:prefLabel xml:lang="en">Operational disruption</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19247-6">
    <skos:prefLabel xml:lang="de">Betriebliche Störung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11987-6">
    <skos:related rdf:resource="descriptor/11903-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11987-6">
    <skos:related rdf:resource="descriptor/19476-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11987-6">
    <skos:related rdf:resource="descriptor/11970-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11987-6">
    <skos:related rdf:resource="descriptor/11880-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11987-6">
    <skos:related rdf:resource="descriptor/11863-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19417-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10811-6">
    <skos:altLabel xml:lang="en">International capital flows</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10811-6">
    <skos:altLabel xml:lang="de">Internationaler Kapitalverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10811-6">
    <skos:altLabel xml:lang="de">Internationale Kapitalbewegung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70346">
    <rdfs:label xml:lang="de">Metallprodukte</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id7">
    <rdf:value xml:lang="de">Für den Bereich der Europäischen Union</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10491-5">
    <skos:scopeNote xml:lang="de">Internationale Wanderung von hochqualifiziertem Personal.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10993-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11765-3">
    <skos:prefLabel xml:lang="en">Theory of distribution</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11765-3">
    <skos:prefLabel xml:lang="de">Verteilungstheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11951-6">
    <skos:scopeNote xml:lang="de">Wissenschaft zur Analyse der physischen, sozialen und ökonomischen Gegebenheiten und Aktivitäten im Raum. Häufig auch interdisziplinär ausgerichtet.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16050-5">
    <skos:related rdf:resource="descriptor/16081-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16050-5">
    <skos:related rdf:resource="descriptor/16038-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16050-5">
    <skos:related rdf:resource="descriptor/12090-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16050-5">
    <skos:related rdf:resource="descriptor/11640-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16050-5">
    <skos:related rdf:resource="descriptor/12195-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16050-5">
    <skos:related rdf:resource="descriptor/12194-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16050-5">
    <skos:related rdf:resource="descriptor/16067-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16050-5">
    <skos:related rdf:resource="descriptor/13731-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16050-5">
    <skos:related rdf:resource="descriptor/15961-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16050-5">
    <skos:related rdf:resource="descriptor/13464-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16050-5">
    <skos:related rdf:resource="descriptor/19043-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16050-5">
    <skos:related rdf:resource="descriptor/16031-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16050-5">
    <skos:related rdf:resource="descriptor/19080-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16050-5">
    <skos:related rdf:resource="descriptor/19729-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16050-5">
    <skos:related rdf:resource="descriptor/19003-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18928-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10353-3">
    <skos:scopeNote xml:lang="de">Konjunkturbewegungen, die vor Wahlen von Regierungen erzeugt werden, um die eigene Wiederwahl zu begünstigen, sowie der Einfluss der Konjunktur auf das Wahlverhalten.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18916-1">
    <skos:prefLabel xml:lang="en">Leisure utilization</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18916-1">
    <skos:prefLabel xml:lang="de">Freizeitverhalten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10298-6">
    <zbwext:useInsteadNote rdf:nodeID="id342"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19031-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12806-3">
    <skos:altLabel xml:lang="en">Brand image</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12806-3">
    <skos:altLabel xml:lang="en">Brand asset</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12806-3">
    <skos:altLabel xml:lang="de">Produktpositionierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12806-3">
    <skos:altLabel xml:lang="de">Produktbild</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12806-3">
    <skos:altLabel xml:lang="de">Markenwert</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12806-3">
    <skos:altLabel xml:lang="de">Markenimage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12806-3">
    <skos:altLabel xml:lang="de">Markenbild</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11411-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10221-6">
    <skos:broader rdf:resource="thsys/71104"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12116-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18062-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11382-5">
    <skos:altLabel xml:lang="de">Schulunterricht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15700-4">
    <skos:broader rdf:resource="thsys/70095"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15700-4">
    <skos:broader rdf:resource="thsys/70007"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15700-4">
    <skos:broader rdf:resource="descriptor/15690-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19413-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13535-3">
    <skos:related rdf:resource="descriptor/18601-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13535-3">
    <skos:related rdf:resource="descriptor/13536-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13535-3">
    <skos:related rdf:resource="descriptor/13534-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18140-1">
    <skos:related rdf:resource="descriptor/12573-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18140-1">
    <skos:related rdf:resource="descriptor/11213-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18140-1">
    <skos:related rdf:resource="descriptor/11236-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18140-1">
    <skos:related rdf:resource="descriptor/10143-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18140-1">
    <skos:related rdf:resource="descriptor/19111-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18140-1">
    <skos:related rdf:resource="descriptor/11184-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18140-1">
    <skos:related rdf:resource="descriptor/10169-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18140-1">
    <skos:related rdf:resource="descriptor/24623-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15963-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11560-0">
    <skos:broader rdf:resource="thsys/73363"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11560-0">
    <skos:broader rdf:resource="thsys/71081"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11560-0">
    <skos:broader rdf:resource="thsys/70865"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11560-0">
    <skos:broader rdf:resource="descriptor/11596-0"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id149">
    <rdfs:seeAlso rdf:resource="descriptor/15149-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19178-6">
    <skos:narrower rdf:resource="descriptor/17829-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19178-6">
    <skos:narrower rdf:resource="descriptor/17811-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19178-6">
    <skos:narrower rdf:resource="descriptor/17568-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19178-6">
    <skos:narrower rdf:resource="descriptor/17039-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19178-6">
    <skos:narrower rdf:resource="descriptor/17374-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19178-6">
    <skos:narrower rdf:resource="descriptor/17255-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19178-6">
    <skos:narrower rdf:resource="descriptor/18012-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18948-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19281-6">
    <skos:broader rdf:resource="thsys/70046"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12473-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19473-4">
    <skos:related rdf:resource="descriptor/15762-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19473-4">
    <skos:related rdf:resource="descriptor/11986-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19609-5">
    <skos:broader rdf:resource="thsys/72183"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19609-5">
    <skos:broader rdf:resource="thsys/71035"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19609-5">
    <skos:broader rdf:resource="descriptor/15315-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19659-4">
    <skos:broader rdf:resource="thsys/70582"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19069-4">
    <skos:prefLabel xml:lang="en">Interfirm comparison</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19069-4">
    <skos:prefLabel xml:lang="de">Betriebsvergleich</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12103-2">
    <skos:prefLabel xml:lang="en">Contingency theory</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12103-2">
    <skos:prefLabel xml:lang="de">Kontingenztheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13281-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17464-1">
    <skos:broader rdf:resource="thsys/71032"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17464-1">
    <skos:broader rdf:resource="descriptor/17595-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17464-1">
    <skos:broader rdf:resource="descriptor/19078-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71020">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18500-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10500-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18025-1">
    <skos:broader rdf:resource="thsys/73370"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18025-1">
    <skos:broader rdf:resource="thsys/70833"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18025-1">
    <skos:broader rdf:resource="thsys/70324"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18025-1">
    <skos:broader rdf:resource="descriptor/15725-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18025-1">
    <skos:broader rdf:resource="descriptor/16540-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13693-6">
    <skos:prefLabel xml:lang="en">Banking law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13693-6">
    <skos:prefLabel xml:lang="de">Bankrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11167-2">
    <skos:narrower rdf:resource="descriptor/11165-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16602-6">
    <skos:narrower rdf:resource="descriptor/11060-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16602-6">
    <skos:narrower rdf:resource="descriptor/16606-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16602-6">
    <skos:narrower rdf:resource="descriptor/16605-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16602-6">
    <skos:narrower rdf:resource="descriptor/11047-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16602-6">
    <skos:narrower rdf:resource="descriptor/11046-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16602-6">
    <skos:narrower rdf:resource="descriptor/16604-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16602-6">
    <skos:narrower rdf:resource="descriptor/16603-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16602-6">
    <skos:narrower rdf:resource="descriptor/16250-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16602-6">
    <skos:narrower rdf:resource="descriptor/16601-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16602-6">
    <skos:narrower rdf:resource="descriptor/13425-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12805-5">
    <skos:broader rdf:resource="thsys/70550"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12805-5">
    <skos:broader rdf:resource="descriptor/12808-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19958-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11419-1">
    <skos:related rdf:resource="descriptor/16201-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11419-1">
    <skos:related rdf:resource="descriptor/11417-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11419-1">
    <skos:related rdf:resource="descriptor/18066-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11419-1">
    <skos:related rdf:resource="descriptor/11415-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11419-1">
    <skos:related rdf:resource="descriptor/11411-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15964-5">
    <skos:broader rdf:resource="thsys/73332"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15964-5">
    <skos:broader rdf:resource="descriptor/16188-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14769-1">
    <skos:editorialNote xml:lang="de">Möglichst spezifischer indexieren.</skos:editorialNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13345-1">
    <skos:prefLabel xml:lang="en">Information services</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13345-1">
    <skos:prefLabel xml:lang="de">Informationsdienstleistung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18012-3">
    <skos:altLabel xml:lang="x-other">Allemagne</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18012-3">
    <skos:altLabel xml:lang="de">Vereinigtes Wirtschaftsgebiet Deutschlands</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18012-3">
    <skos:altLabel xml:lang="de">Französische Zone Deutschlands</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18012-3">
    <skos:altLabel xml:lang="de">Bundesrepublik Deutschland</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18012-3">
    <skos:altLabel xml:lang="de">Britische Zone Deutschlands</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18012-3">
    <skos:altLabel xml:lang="de">BRD</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18012-3">
    <skos:altLabel xml:lang="de">Amerikanische Zone Deutschlands</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14653-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19668-3">
    <skos:altLabel xml:lang="en">Index of economic security</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19668-3">
    <skos:altLabel xml:lang="en">ECOSEC</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19668-3">
    <skos:altLabel xml:lang="de">Wirtschaftliche Sicherheit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19705-2">
    <skos:altLabel xml:lang="de">Rechtsvereinheitlichung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19705-2">
    <skos:altLabel xml:lang="de">Rechtsharmonisierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19705-2">
    <skos:altLabel xml:lang="de">Rechtseinheit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19705-2">
    <skos:altLabel xml:lang="de">Internationale Rechtsharmonisierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19705-2">
    <skos:altLabel xml:lang="de">Harmonisierung des Rechts</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18006-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71012">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14889-5">
    <skos:scopeNote xml:lang="de">Fasern und Textilien für Einsatzzwecke in der Medizin, Bauwirtschaft, Elektroindustrie, im Fahrzeugbau usw.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18344-6">
    <skos:broader rdf:resource="thsys/71018"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18344-6">
    <skos:broader rdf:resource="descriptor/10470-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14504-2">
    <skos:related rdf:resource="descriptor/14561-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18246-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18821-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18010-0">
    <skos:prefLabel xml:lang="en">Sectoral development</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18010-0">
    <skos:prefLabel xml:lang="de">Branchenentwicklung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19068-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19378-5">
    <skos:related rdf:resource="descriptor/19446-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19378-5">
    <skos:related rdf:resource="descriptor/18806-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19378-5">
    <skos:related rdf:resource="descriptor/11590-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19378-5">
    <skos:related rdf:resource="descriptor/11537-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10006-3">
    <skos:prefLabel xml:lang="en">Income</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10006-3">
    <skos:prefLabel xml:lang="de">Einkommen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11786-2">
    <skos:altLabel xml:lang="en">Interindustry relations</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11786-2">
    <skos:altLabel xml:lang="en">Industrial complex analysis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11786-2">
    <skos:altLabel xml:lang="de">Wirtschaftsverflechtung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11786-2">
    <skos:altLabel xml:lang="de">Verflechtungsanalyse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11786-2">
    <skos:altLabel xml:lang="de">Intersektorale Verflechtung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11786-2">
    <skos:altLabel xml:lang="de">Industrieverflechtung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11786-2">
    <skos:altLabel xml:lang="de">Industriekomplexanalyse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11786-2">
    <skos:altLabel xml:lang="de">Gewichtete intersektorale Verflechtungsanalyse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11786-2">
    <skos:altLabel xml:lang="de">Attraktivitätsanalyse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11786-2">
    <skos:altLabel xml:lang="de">Attraktionsanalyse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15330-4">
    <skos:altLabel xml:lang="en">Covariance</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15330-4">
    <skos:altLabel xml:lang="en">Canonical correlation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15330-4">
    <skos:altLabel xml:lang="de">Partielle Korrelation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15330-4">
    <skos:altLabel xml:lang="de">Multipler Korrelationskoeffizient</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15330-4">
    <skos:altLabel xml:lang="de">Kovarianz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15330-4">
    <skos:altLabel xml:lang="de">Korrelationsmatrix</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15330-4">
    <skos:altLabel xml:lang="de">Korrelationsmaß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15330-4">
    <skos:altLabel xml:lang="de">Korrelationskoeffizient</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15330-4">
    <skos:altLabel xml:lang="de">Korrelationsanalyse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15330-4">
    <skos:altLabel xml:lang="de">Kanonische Korrelation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15793-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12255-0">
    <skos:prefLabel xml:lang="en">Trade credit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12255-0">
    <skos:prefLabel xml:lang="de">Lieferantenkredit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10852-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12396-3">
    <skos:prefLabel xml:lang="en">Tax balance sheet</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12396-3">
    <skos:prefLabel xml:lang="de">Steuerbilanz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12028-4">
    <skos:prefLabel xml:lang="en">Real estate law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12028-4">
    <skos:prefLabel xml:lang="de">Grundstücksrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12308-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70128">
    <rdfs:label xml:lang="de">Australien und südpazifischer Inselraum</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12226-0">
    <skos:altLabel xml:lang="en">Flow-of-funds analysis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12226-0">
    <skos:altLabel xml:lang="de">Wertflussrechnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12226-0">
    <skos:altLabel xml:lang="de">Wertflußrechnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12226-0">
    <skos:altLabel xml:lang="de">Veränderungsbilanz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12226-0">
    <skos:altLabel xml:lang="de">Kapitalverwendungsrechnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12226-0">
    <skos:altLabel xml:lang="de">Kapitalflußrechnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12226-0">
    <skos:altLabel xml:lang="de">Fondsflussrechnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12226-0">
    <skos:altLabel xml:lang="de">Fondsflußrechnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12226-0">
    <skos:altLabel xml:lang="de">Finanzwirtschaftliche Bilanz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12226-0">
    <skos:altLabel xml:lang="de">Finanzflussrechnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12226-0">
    <skos:altLabel xml:lang="de">Finanzflußrechnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12226-0">
    <skos:altLabel xml:lang="de">Bewegungsbilanz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19514-2">
    <skos:scopeNote xml:lang="de">Umstellungsvorschlag zur Finanzierung des Arbeitgeberanteils der gesetzlichen Rentenversicherung: Bemessungsgrundlage soll nicht mehr nur der Einsatz des Faktors Arbeit, sondern auch der Einsatz des Faktors Kapital sein.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10987-4">
    <skos:narrower rdf:resource="descriptor/10993-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10987-4">
    <skos:narrower rdf:resource="descriptor/19829-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10987-4">
    <skos:narrower rdf:resource="descriptor/10619-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10987-4">
    <skos:narrower rdf:resource="descriptor/10439-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10987-4">
    <skos:narrower rdf:resource="descriptor/11779-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10987-4">
    <skos:narrower rdf:resource="descriptor/18976-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10987-4">
    <skos:narrower rdf:resource="descriptor/10981-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10987-4">
    <skos:narrower rdf:resource="descriptor/19291-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10987-4">
    <skos:narrower rdf:resource="descriptor/10978-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10987-4">
    <skos:narrower rdf:resource="descriptor/15752-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10987-4">
    <skos:narrower rdf:resource="descriptor/10398-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10987-4">
    <skos:narrower rdf:resource="descriptor/10395-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10987-4">
    <skos:narrower rdf:resource="descriptor/10976-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10987-4">
    <skos:narrower rdf:resource="descriptor/18698-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10987-4">
    <skos:narrower rdf:resource="descriptor/10387-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10987-4">
    <skos:narrower rdf:resource="descriptor/19446-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10987-4">
    <skos:narrower rdf:resource="descriptor/10438-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10987-4">
    <skos:narrower rdf:resource="descriptor/10970-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10987-4">
    <skos:narrower rdf:resource="descriptor/10382-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10987-4">
    <skos:narrower rdf:resource="descriptor/11728-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10987-4">
    <skos:narrower rdf:resource="descriptor/11239-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10987-4">
    <skos:narrower rdf:resource="descriptor/10682-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10987-4">
    <skos:narrower rdf:resource="descriptor/16533-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10987-4">
    <skos:narrower rdf:resource="descriptor/11235-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10987-4">
    <skos:narrower rdf:resource="descriptor/10377-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10028-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18107-6">
    <skos:related rdf:resource="descriptor/10015-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18107-6">
    <skos:related rdf:resource="descriptor/12058-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18107-6">
    <skos:related rdf:resource="descriptor/18912-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18107-6">
    <skos:related rdf:resource="descriptor/19089-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18107-6">
    <skos:related rdf:resource="descriptor/12208-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18107-6">
    <skos:related rdf:resource="descriptor/12278-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18107-6">
    <skos:related rdf:resource="descriptor/10008-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18107-6">
    <skos:related rdf:resource="descriptor/18032-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18503-5">
    <skos:related rdf:resource="descriptor/18502-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18503-5">
    <skos:related rdf:resource="descriptor/12932-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15963-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16564-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19272-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24663-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16524-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19113-6">
    <skos:prefLabel xml:lang="en">Outsourcing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19113-6">
    <skos:prefLabel xml:lang="de">Outsourcing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15057-3">
    <skos:broader rdf:resource="thsys/71035"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15057-3">
    <skos:broader rdf:resource="thsys/70269"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15057-3">
    <skos:broader rdf:resource="descriptor/15045-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18159-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19328-6">
    <skos:prefLabel xml:lang="en">International transfer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19328-6">
    <skos:prefLabel xml:lang="de">Internationaler Transfer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14014-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13417-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19347-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18928-1">
    <skos:scopeNote xml:lang="de">Volkswirtschaftliche Analysen zu einzelnen Medien und zum Medienbereich insgesamt.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19348-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13192-0">
    <skos:related rdf:resource="descriptor/13199-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13192-0">
    <skos:related rdf:resource="descriptor/14005-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16050-5">
    <skos:narrower rdf:resource="descriptor/16076-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16050-5">
    <skos:narrower rdf:resource="descriptor/18726-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16050-5">
    <skos:narrower rdf:resource="descriptor/16063-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16050-5">
    <skos:narrower rdf:resource="descriptor/18702-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16050-5">
    <skos:narrower rdf:resource="descriptor/16043-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10786-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13584-4">
    <skos:related rdf:resource="descriptor/13580-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13584-4">
    <skos:related rdf:resource="descriptor/13587-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13584-4">
    <skos:related rdf:resource="descriptor/13571-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13584-4">
    <skos:related rdf:resource="descriptor/13513-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13584-4">
    <skos:related rdf:resource="descriptor/13612-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13584-4">
    <skos:related rdf:resource="descriptor/13591-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13584-4">
    <skos:related rdf:resource="descriptor/18620-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11753-3">
    <skos:prefLabel xml:lang="en">Measurement of inequality</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11753-3">
    <skos:prefLabel xml:lang="de">Disparitätsmaß</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19416-2">
    <skos:prefLabel xml:lang="en">Special tax</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19416-2">
    <skos:prefLabel xml:lang="de">Sondersteuer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12879-4">
    <skos:broader rdf:resource="thsys/71055"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12879-4">
    <skos:broader rdf:resource="thsys/70113"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19754-3">
    <skos:related rdf:resource="descriptor/15859-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19754-3">
    <skos:related rdf:resource="descriptor/16642-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19754-3">
    <skos:related rdf:resource="descriptor/15717-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19754-3">
    <skos:related rdf:resource="descriptor/10202-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19754-3">
    <skos:related rdf:resource="descriptor/16639-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19754-3">
    <skos:related rdf:resource="descriptor/13485-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19754-3">
    <skos:related rdf:resource="descriptor/15654-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19754-3">
    <skos:related rdf:resource="descriptor/10300-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19754-3">
    <skos:related rdf:resource="descriptor/16406-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19754-3">
    <skos:related rdf:resource="descriptor/18917-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19754-3">
    <skos:related rdf:resource="descriptor/18556-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11628-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15923-5">
    <skos:altLabel xml:lang="de">Jude</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15923-5">
    <skos:altLabel xml:lang="de">Israeliten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16984-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19166-6">
    <skos:narrower rdf:resource="descriptor/17570-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19166-6">
    <skos:narrower rdf:resource="descriptor/17569-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16437-2">
    <skos:altLabel xml:lang="de">Verteidigungsgemeinschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16437-2">
    <skos:altLabel xml:lang="de">Verteidigungsbündnis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16437-2">
    <skos:altLabel xml:lang="de">Staatsbündnis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16437-2">
    <skos:altLabel xml:lang="de">Militärbündnis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18953-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14469-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19199-5">
    <skos:broader rdf:resource="thsys/70049"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10183-2">
    <skos:altLabel xml:lang="de">Optimaler Verbrauchsplan</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10183-2">
    <skos:altLabel xml:lang="de">Effizienzbedingungen des Konsums</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70278">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.13.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18618-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15025-2">
    <skos:scopeNote xml:lang="de">Von H. KAHN entwickelte Methode zur Abschätzung möglicher Ereignisse und Entwicklungen in einem bestimmten Bereich, einer bestimmten Zeitperiode und einer bestimmten Region unter Zuhilfenahme qualitativer und quantitativer Prognoseverfahren.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id339">
    <rdf:value xml:lang="de">Für Dienstleistungen der Industrie</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11073-4">
    <skos:scopeNote xml:lang="de">Marxsche und marxistische Ansätze zur Erklärung des ökonomischen Arbeitsbegriffs.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12437-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16169-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16055-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19460-6">
    <skos:narrower rdf:resource="descriptor/18993-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19460-6">
    <skos:narrower rdf:resource="descriptor/18548-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19460-6">
    <skos:narrower rdf:resource="descriptor/19464-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19460-6">
    <skos:narrower rdf:resource="descriptor/19459-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10771-6">
    <skos:broader rdf:resource="thsys/71094"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10771-6">
    <skos:broader rdf:resource="descriptor/10626-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10771-6">
    <skos:broader rdf:resource="descriptor/10746-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18564-6">
    <skos:related rdf:resource="descriptor/18797-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18564-6">
    <skos:related rdf:resource="descriptor/12783-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18564-6">
    <skos:related rdf:resource="descriptor/19151-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18564-6">
    <skos:related rdf:resource="descriptor/13308-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18564-6">
    <skos:related rdf:resource="descriptor/18612-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10027-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11281-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12928-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15149-5">
    <skos:broader rdf:resource="thsys/72183"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15149-5">
    <skos:broader rdf:resource="descriptor/15142-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11698-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16150-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18088-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70923">
    <rdfs:label xml:lang="en">Postal and telecommunications services</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70923">
    <rdfs:label xml:lang="de">Post und Telekommunikation</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10586-1">
    <skos:related rdf:resource="descriptor/10619-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10586-1">
    <skos:related rdf:resource="descriptor/10587-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10586-1">
    <skos:related rdf:resource="descriptor/10560-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10586-1">
    <skos:related rdf:resource="descriptor/10557-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19415-4">
    <skos:scopeNote xml:lang="de">Steuer(reform)konzept für eine proportionale Einkommensteuer.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73377">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.04.02.05</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13744-1">
    <skos:broader rdf:resource="thsys/71107"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13744-1">
    <skos:broader rdf:resource="thsys/71104"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13744-1">
    <skos:broader rdf:resource="descriptor/13742-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13744-1">
    <skos:broader rdf:resource="descriptor/10221-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14472-3">
    <skos:prefLabel xml:lang="en">Electronic component</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14472-3">
    <skos:prefLabel xml:lang="de">Elektronisches Bauelement</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18009-6">
    <skos:related rdf:resource="descriptor/14665-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20455-3">
    <skos:altLabel xml:lang="de">Betriebliches Bündnis für Arbeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20455-3">
    <skos:altLabel xml:lang="de">Betriebliche Tarifvereinbarung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12403-4">
    <skos:related rdf:resource="descriptor/19670-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12403-4">
    <skos:related rdf:resource="descriptor/12297-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12403-4">
    <skos:related rdf:resource="descriptor/12371-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12403-4">
    <skos:related rdf:resource="descriptor/12275-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12960-0">
    <skos:related rdf:resource="descriptor/14118-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18792-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15446-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11212-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13801-1">
    <skos:broader rdf:resource="thsys/70905"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13801-1">
    <skos:broader rdf:resource="thsys/70114"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13801-1">
    <skos:broader rdf:resource="descriptor/13793-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20855-1">
    <skos:narrower rdf:resource="descriptor/20842-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20855-1">
    <skos:narrower rdf:resource="descriptor/20826-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20855-1">
    <skos:narrower rdf:resource="descriptor/19154-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19728-4">
    <skos:prefLabel xml:lang="en">Law enforcement</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19728-4">
    <skos:prefLabel xml:lang="de">Rechtsdurchsetzung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15001-2">
    <skos:related rdf:resource="descriptor/15628-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15001-2">
    <skos:related rdf:resource="descriptor/14182-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15001-2">
    <skos:related rdf:resource="descriptor/16449-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11377-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13019-0">
    <skos:related rdf:resource="descriptor/14236-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13019-0">
    <skos:related rdf:resource="descriptor/12904-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14355-0">
    <skos:prefLabel xml:lang="en">Power engine</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14355-0">
    <skos:prefLabel xml:lang="de">Kraftmaschine</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16053-6">
    <skos:broader rdf:resource="thsys/73391"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16053-6">
    <skos:broader rdf:resource="descriptor/18176-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19549-4">
    <skos:related rdf:resource="descriptor/13051-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19549-4">
    <skos:related rdf:resource="descriptor/13217-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19549-4">
    <skos:related rdf:resource="descriptor/14433-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11910-6">
    <skos:broader rdf:resource="thsys/71110"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11910-6">
    <skos:broader rdf:resource="thsys/70058"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11910-6">
    <skos:broader rdf:resource="descriptor/11782-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11910-6">
    <skos:broader rdf:resource="descriptor/11875-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10691-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18263-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15678-6">
    <skos:broader rdf:resource="thsys/73388"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15678-6">
    <skos:broader rdf:resource="descriptor/15644-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13729-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11847-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12243-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14744-3">
    <skos:related rdf:resource="descriptor/18240-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14744-3">
    <skos:related rdf:resource="descriptor/14630-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14744-3">
    <skos:related rdf:resource="descriptor/14611-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16373-2">
    <skos:prefLabel xml:lang="en">Social Democratic Party</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16373-2">
    <skos:prefLabel xml:lang="de">Sozialdemokratische Partei</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14171-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11627-1">
    <skos:related rdf:resource="descriptor/19470-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16379-4">
    <skos:broader rdf:resource="thsys/73382"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16379-4">
    <skos:broader rdf:resource="thsys/70853"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70939">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.12.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12427-4">
    <skos:broader rdf:resource="thsys/71043"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12427-4">
    <skos:broader rdf:resource="descriptor/12408-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11932-3">
    <skos:altLabel xml:lang="de">Standortmodell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11932-3">
    <skos:altLabel xml:lang="de">Standortbestimmungsmodell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11932-3">
    <skos:altLabel xml:lang="de">Standortbestimmungslehre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11932-3">
    <skos:altLabel xml:lang="de">Optimaler Standort</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14633-5">
    <skos:broader rdf:resource="thsys/70556"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14633-5">
    <skos:broader rdf:resource="thsys/70467"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14380-1">
    <skos:broader rdf:resource="thsys/70318"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14380-1">
    <skos:broader rdf:resource="descriptor/14381-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14380-1">
    <skos:broader rdf:resource="descriptor/14041-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15907-3">
    <skos:altLabel xml:lang="en">Foreigner</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15907-3">
    <skos:altLabel xml:lang="en">Expatriates</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15907-3">
    <skos:altLabel xml:lang="de">Ausländische Bevölkerung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15907-3">
    <skos:altLabel xml:lang="de">Ausländerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18874-5">
    <skos:broader rdf:resource="thsys/70844"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18874-5">
    <skos:broader rdf:resource="thsys/70176"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18874-5">
    <skos:broader rdf:resource="descriptor/18884-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70963">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.10</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11504-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14418-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18196-2">
    <skos:related rdf:resource="descriptor/13164-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18196-2">
    <skos:related rdf:resource="descriptor/13238-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18196-2">
    <skos:related rdf:resource="descriptor/14082-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19325-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70498">
    <skos:prefLabel xml:lang="de">P.20  Nahrungs- und Genussmittel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14787-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15887-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13555-4">
    <skos:altLabel xml:lang="en">Highway</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13555-4">
    <skos:altLabel xml:lang="en">Freeway</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13555-4">
    <skos:altLabel xml:lang="de">Autoroute</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13555-4">
    <skos:altLabel xml:lang="de">Autobahnnetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14366-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11427-2">
    <skos:altLabel xml:lang="de">Binnenwert des Geldes</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12665-2">
    <skos:prefLabel xml:lang="en">Buyer-supplier relations</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12665-2">
    <skos:prefLabel xml:lang="de">Lieferanten-Kunden-Beziehung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11392-2">
    <skos:altLabel xml:lang="de">Förderung der beruflichen Bildung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11392-2">
    <skos:altLabel xml:lang="de">Berufsbildungsprogramm</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11392-2">
    <skos:altLabel xml:lang="de">Berufsbildungsförderung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11392-2">
    <skos:altLabel xml:lang="de">Berufsausbildungspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11392-2">
    <skos:altLabel xml:lang="de">Berufliche Qualifizierungspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11392-2">
    <skos:altLabel xml:lang="de">Berufliche Bildungsförderung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11392-2">
    <skos:altLabel xml:lang="de">Ausbildungspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id37">
    <rdf:value xml:lang="de">für Auslandsniederlassungen</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18985-3">
    <skos:altLabel xml:lang="de">Trockenlegung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18635-2">
    <skos:prefLabel xml:lang="en">Spa town</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18635-2">
    <skos:prefLabel xml:lang="de">Heilbad</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id153">
    <rdf:value xml:lang="de">Für internationale Kooperationen mit Kapitalbeteiligung</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18798-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19116-0">
    <skos:broader rdf:resource="thsys/70030"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19116-0">
    <skos:broader rdf:resource="descriptor/12712-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71008">
    <skos:broader rdf:resource="thsys/71010"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10642-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15949-1">
    <skos:narrower rdf:resource="descriptor/19646-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15949-1">
    <skos:narrower rdf:resource="descriptor/15946-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18112-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15061-5">
    <skos:narrower rdf:resource="descriptor/19580-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15061-5">
    <skos:narrower rdf:resource="descriptor/15135-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15061-5">
    <skos:narrower rdf:resource="descriptor/15124-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15061-5">
    <skos:narrower rdf:resource="descriptor/15940-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15061-5">
    <skos:narrower rdf:resource="descriptor/11297-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11373-6">
    <skos:broader rdf:resource="thsys/70844"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11373-6">
    <skos:broader rdf:resource="thsys/70176"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11373-6">
    <skos:broader rdf:resource="descriptor/18857-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15768-5">
    <skos:altLabel xml:lang="de">Umweltschutzpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15768-5">
    <skos:altLabel xml:lang="de">Umweltpolitisches Instrument</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id64">
    <rdfs:seeAlso rdf:resource="descriptor/13393-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19169-0">
    <skos:prefLabel xml:lang="en">Palau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19169-0">
    <skos:prefLabel xml:lang="de">Palau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12975-1">
    <skos:related rdf:resource="descriptor/19678-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12975-1">
    <skos:related rdf:resource="descriptor/14176-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12975-1">
    <skos:related rdf:resource="descriptor/18353-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12975-1">
    <skos:related rdf:resource="descriptor/18351-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12975-1">
    <skos:related rdf:resource="descriptor/15993-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12975-1">
    <skos:related rdf:resource="descriptor/11810-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12975-1">
    <skos:related rdf:resource="descriptor/18350-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12975-1">
    <skos:related rdf:resource="descriptor/18996-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12975-1">
    <skos:related rdf:resource="descriptor/18348-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12975-1">
    <skos:related rdf:resource="descriptor/14175-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16369-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18206-4">
    <skos:prefLabel xml:lang="en">Advertiser</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18206-4">
    <skos:prefLabel xml:lang="de">Anzeigenblatt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13194-3">
    <skos:broader rdf:resource="thsys/70957"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13194-3">
    <skos:broader rdf:resource="descriptor/13210-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19534-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11325-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10268-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14423-2">
    <skos:prefLabel xml:lang="en">Spacecraft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14423-2">
    <skos:prefLabel xml:lang="de">Raumfahrzeug</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13423-0">
    <skos:prefLabel xml:lang="en">Administrative court</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13423-0">
    <skos:prefLabel xml:lang="de">Verwaltungsgericht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16314-4">
    <skos:narrower rdf:resource="descriptor/13421-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16314-4">
    <skos:narrower rdf:resource="descriptor/11939-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16314-4">
    <skos:narrower rdf:resource="descriptor/11938-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16314-4">
    <skos:narrower rdf:resource="descriptor/11600-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15258-0">
    <skos:narrower rdf:resource="descriptor/15243-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15258-0">
    <skos:narrower rdf:resource="descriptor/15298-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15258-0">
    <skos:narrower rdf:resource="descriptor/15272-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15258-0">
    <skos:narrower rdf:resource="descriptor/15340-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16214-1">
    <skos:related rdf:resource="descriptor/15700-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16214-1">
    <skos:related rdf:resource="descriptor/19784-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16214-1">
    <skos:related rdf:resource="descriptor/10064-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16214-1">
    <skos:related rdf:resource="descriptor/16229-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16214-1">
    <skos:related rdf:resource="descriptor/15072-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16214-1">
    <skos:related rdf:resource="descriptor/16209-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16214-1">
    <skos:related rdf:resource="descriptor/10411-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18671-5">
    <skos:broader rdf:resource="thsys/70905"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18691-6">
    <skos:related rdf:resource="descriptor/10221-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17746-5">
    <skos:broader rdf:resource="thsys/70085"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17746-5">
    <skos:broader rdf:resource="descriptor/17750-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17746-5">
    <skos:broader rdf:resource="descriptor/17978-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17612-5">
    <skos:prefLabel xml:lang="en">Sri Lanka</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17612-5">
    <skos:prefLabel xml:lang="de">Sri Lanka</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12303-1">
    <skos:prefLabel xml:lang="en">Enterprise restructuring</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12303-1">
    <skos:prefLabel xml:lang="de">Unternehmenssanierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19155-4">
    <skos:scopeNote xml:lang="de">Nur für Software zur Gestaltung der Ablauforganisation.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11772-6">
    <skos:related rdf:resource="descriptor/12594-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11772-6">
    <skos:related rdf:resource="descriptor/12275-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11772-6">
    <skos:related rdf:resource="descriptor/11769-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11772-6">
    <skos:related rdf:resource="descriptor/20368-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10469-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12406-5">
    <skos:prefLabel xml:lang="en">Capitalization</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12406-5">
    <skos:prefLabel xml:lang="de">Aktivierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18637-5">
    <skos:broader rdf:resource="thsys/73392"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18637-5">
    <skos:broader rdf:resource="thsys/70924"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18637-5">
    <skos:broader rdf:resource="descriptor/16039-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12187-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18912-2">
    <skos:altLabel xml:lang="de">Wiederaufbau (Kriegsschaden)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18912-2">
    <skos:altLabel xml:lang="de">Kriegsschäden</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18912-2">
    <skos:altLabel xml:lang="de">Demontage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18912-2">
    <skos:altLabel xml:lang="de">Beseitigung von Kriegsschäden</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19509-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18524-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19545-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13348-2">
    <skos:broader rdf:resource="thsys/71053"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13348-2">
    <skos:broader rdf:resource="thsys/71052"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13348-2">
    <skos:broader rdf:resource="thsys/70839"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13348-2">
    <skos:broader rdf:resource="thsys/70017"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13348-2">
    <skos:broader rdf:resource="descriptor/14015-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16286-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20817-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18212-2">
    <skos:related rdf:resource="descriptor/18059-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13020-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18772-6">
    <skos:prefLabel xml:lang="en">EU policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18772-6">
    <skos:prefLabel xml:lang="de">EU-Politik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19780-2">
    <skos:related rdf:resource="descriptor/12964-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15518-0">
    <skos:prefLabel xml:lang="en">Vehicle scheduling</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15518-0">
    <skos:prefLabel xml:lang="de">Tourenplanung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25501-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11912-2">
    <skos:narrower rdf:resource="descriptor/11990-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11912-2">
    <skos:narrower rdf:resource="descriptor/10426-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11912-2">
    <skos:narrower rdf:resource="descriptor/11928-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16034-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12166-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10994-0">
    <skos:narrower rdf:resource="descriptor/11521-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10994-0">
    <skos:narrower rdf:resource="descriptor/11729-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10994-0">
    <skos:narrower rdf:resource="descriptor/18708-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10994-0">
    <skos:narrower rdf:resource="descriptor/11809-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10746-5">
    <skos:altLabel xml:lang="en">International monetary order</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10746-5">
    <skos:altLabel xml:lang="de">Weltwährungssystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10746-5">
    <skos:altLabel xml:lang="de">Internationale Währungsordnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18784-6">
    <skos:broader rdf:resource="thsys/71109"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18784-6">
    <skos:broader rdf:resource="thsys/70879"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18784-6">
    <skos:broader rdf:resource="descriptor/25742-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13612-4">
    <skos:broader rdf:resource="thsys/70927"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15695-6">
    <skos:prefLabel xml:lang="en">Urban history</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15695-6">
    <skos:prefLabel xml:lang="de">Stadtgeschichte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10255-3">
    <skos:scopeNote xml:lang="de">Hypothese über den Zusammenhang zwischen Arbeitslosenquote und Inflationsrate.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16797-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15842-5">
    <skos:related rdf:resource="descriptor/12620-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18120-0">
    <skos:altLabel xml:lang="en">SME (Small and Medium Sized Enterprises)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18120-0">
    <skos:altLabel xml:lang="en">Small and medium sized enterprises</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18120-0">
    <skos:altLabel xml:lang="en">Small and medium size firms</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18120-0">
    <skos:altLabel xml:lang="de">Mittleres Unternehmen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18120-0">
    <skos:altLabel xml:lang="de">Mittelstandsbetrieb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18120-0">
    <skos:altLabel xml:lang="de">Mittelständisches Unternehmen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18120-0">
    <skos:altLabel xml:lang="de">Mittelbetrieb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18120-0">
    <skos:altLabel xml:lang="de">KMU</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18120-0">
    <skos:altLabel xml:lang="de">Kleinunternehmen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18120-0">
    <skos:altLabel xml:lang="de">Kleingewerbe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18120-0">
    <skos:altLabel xml:lang="de">Kleine und mittlere Unternehmungen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18120-0">
    <skos:altLabel xml:lang="de">Kleinbetrieb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18120-0">
    <skos:altLabel xml:lang="de">Klein- und Mittelbetriebe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18120-0">
    <skos:altLabel xml:lang="de">Klein- und Mittelbetrieb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10148-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12718-0">
    <skos:prefLabel xml:lang="en">Single-item production</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12718-0">
    <skos:prefLabel xml:lang="de">Einzelfertigung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13382-2">
    <skos:prefLabel xml:lang="en">Journalists</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13382-2">
    <skos:prefLabel xml:lang="de">Journalisten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16424-4">
    <skos:prefLabel xml:lang="en">Zionism</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16424-4">
    <skos:prefLabel xml:lang="de">Zionismus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16188-4">
    <skos:prefLabel xml:lang="en">Public international law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16188-4">
    <skos:prefLabel xml:lang="de">Völkerrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11727-4">
    <skos:altLabel xml:lang="de">Automatischer Stabilisator</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19815-2">
    <skos:related rdf:resource="descriptor/19814-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11566-2">
    <skos:prefLabel xml:lang="en">Property loss compensation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11566-2">
    <skos:prefLabel xml:lang="de">Lastenausgleich</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18701-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70438">
    <skos:narrower rdf:resource="descriptor/16955-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70438">
    <skos:narrower rdf:resource="descriptor/16954-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70438">
    <skos:narrower rdf:resource="descriptor/16953-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70438">
    <skos:narrower rdf:resource="descriptor/16952-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70438">
    <skos:narrower rdf:resource="descriptor/16951-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70438">
    <skos:narrower rdf:resource="descriptor/16950-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70438">
    <skos:narrower rdf:resource="descriptor/16949-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70438">
    <skos:narrower rdf:resource="descriptor/16948-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70438">
    <skos:narrower rdf:resource="descriptor/16946-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70438">
    <skos:narrower rdf:resource="descriptor/16943-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15021-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16395-6">
    <skos:altLabel xml:lang="en">Terrorism</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16395-6">
    <skos:altLabel xml:lang="de">Terrorismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16395-6">
    <skos:altLabel xml:lang="de">Politischer Mord</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16395-6">
    <skos:altLabel xml:lang="de">Politische Sabotage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16395-6">
    <skos:altLabel xml:lang="de">Internationaler Terrorismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16395-6">
    <skos:altLabel xml:lang="de">Attentat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18425-6">
    <skos:broader rdf:resource="thsys/70979"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18425-6">
    <skos:broader rdf:resource="thsys/70799"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18425-6">
    <skos:broader rdf:resource="descriptor/11820-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70493">
    <rdfs:label xml:lang="en">Household economy</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70493">
    <rdfs:label xml:lang="de">Haushaltsökonomik</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10553-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17184-0">
    <skos:broader rdf:resource="thsys/70130"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17184-0">
    <skos:broader rdf:resource="descriptor/17179-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17184-0">
    <skos:broader rdf:resource="descriptor/17991-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17184-0">
    <skos:broader rdf:resource="descriptor/17983-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16650-2">
    <skos:scopeNote xml:lang="de">Freiwilliges Zusammenwirken von Menschen mit gleichgelagerten Problemen ohne Inanspruchnahme professioneller Unterstützung oder von Menschen, die sich von politischen/sozialen Missständen betroffen fühlen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11192-3">
    <skos:broader rdf:resource="thsys/71059"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11192-3">
    <skos:broader rdf:resource="thsys/71050"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11192-3">
    <skos:broader rdf:resource="descriptor/11187-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19536-6">
    <skos:broader rdf:resource="thsys/70269"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19536-6">
    <skos:broader rdf:resource="thsys/70140"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19536-6">
    <skos:broader rdf:resource="descriptor/12207-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19221-3">
    <skos:altLabel xml:lang="en">Inter-vivos transfer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19221-3">
    <skos:altLabel xml:lang="en">Intergenerational transfer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19221-3">
    <skos:altLabel xml:lang="de">Generationentransfer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18083-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11365-5">
    <skos:prefLabel xml:lang="en">Apprenticeship</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11365-5">
    <skos:prefLabel xml:lang="de">Betriebliche Berufsausbildung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12640-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12111-3">
    <skos:related rdf:resource="descriptor/19562-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12111-3">
    <skos:related rdf:resource="descriptor/12091-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12111-3">
    <skos:related rdf:resource="descriptor/12142-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12111-3">
    <skos:related rdf:resource="descriptor/11158-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12111-3">
    <skos:related rdf:resource="descriptor/12136-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10776-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11678-5">
    <skos:prefLabel xml:lang="en">Sales tax</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11678-5">
    <skos:prefLabel xml:lang="de">Umsatzsteuer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14421-6">
    <skos:prefLabel xml:lang="en">Aircraft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14421-6">
    <skos:prefLabel xml:lang="de">Luftfahrzeug</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18752-5">
    <skos:broader rdf:resource="thsys/70888"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18752-5">
    <skos:broader rdf:resource="descriptor/11847-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18752-5">
    <skos:broader rdf:resource="descriptor/11778-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18006-5">
    <skos:related rdf:resource="descriptor/18214-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15825-5">
    <skos:broader rdf:resource="thsys/73385"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15825-5">
    <skos:broader rdf:resource="thsys/70485"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15825-5">
    <skos:broader rdf:resource="thsys/70244"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15825-5">
    <skos:broader rdf:resource="descriptor/12112-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11576-6">
    <skos:prefLabel xml:lang="en">Competitive tendering</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11576-6">
    <skos:prefLabel xml:lang="de">Ausschreibung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11105-3">
    <skos:broader rdf:resource="thsys/70321"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11105-3">
    <skos:broader rdf:resource="thsys/70039"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16563-4">
    <skos:narrower rdf:resource="descriptor/10032-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16563-4">
    <skos:narrower rdf:resource="descriptor/16212-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16563-4">
    <skos:narrower rdf:resource="descriptor/16564-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16563-4">
    <skos:narrower rdf:resource="descriptor/15811-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16563-4">
    <skos:narrower rdf:resource="descriptor/15723-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16563-4">
    <skos:narrower rdf:resource="descriptor/19611-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16563-4">
    <skos:narrower rdf:resource="descriptor/16554-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16563-4">
    <skos:narrower rdf:resource="descriptor/15976-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16563-4">
    <skos:narrower rdf:resource="descriptor/16209-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16563-4">
    <skos:narrower rdf:resource="descriptor/15666-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16563-4">
    <skos:narrower rdf:resource="descriptor/12106-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16563-4">
    <skos:narrower rdf:resource="descriptor/16196-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16563-4">
    <skos:narrower rdf:resource="descriptor/16761-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16563-4">
    <skos:narrower rdf:resource="descriptor/16205-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16563-4">
    <skos:narrower rdf:resource="descriptor/16579-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16563-4">
    <skos:narrower rdf:resource="descriptor/18841-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16563-4">
    <skos:narrower rdf:resource="descriptor/16556-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16563-4">
    <skos:narrower rdf:resource="descriptor/15916-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15876-2">
    <skos:broader rdf:resource="thsys/73354"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15876-2">
    <skos:broader rdf:resource="descriptor/15664-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11565-4">
    <skos:broader rdf:resource="thsys/71080"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11565-4">
    <skos:broader rdf:resource="thsys/70833"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11565-4">
    <skos:broader rdf:resource="thsys/70324"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11565-4">
    <skos:broader rdf:resource="descriptor/11570-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11565-4">
    <skos:broader rdf:resource="descriptor/19722-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15033-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14954-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12475-0">
    <skos:altLabel xml:lang="de">Kostendeckungsprinzip</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12475-0">
    <skos:altLabel xml:lang="de">Eigenwirtschaftliche Preisbildung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10225-5">
    <skos:altLabel xml:lang="en">Symmetric information</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10225-5">
    <skos:altLabel xml:lang="en">Perfect information</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10225-5">
    <skos:altLabel xml:lang="de">Vollständige Information</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13790-1">
    <skos:related rdf:resource="descriptor/11571-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13790-1">
    <skos:related rdf:resource="descriptor/10020-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12946-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11356-6">
    <skos:prefLabel xml:lang="en">Professional workers</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11356-6">
    <skos:prefLabel xml:lang="de">Akademische Berufe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11422-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18211-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14602-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18442-6">
    <skos:prefLabel xml:lang="en">Food research</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18442-6">
    <skos:prefLabel xml:lang="de">Lebensmittelforschung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11592-1">
    <skos:broader rdf:resource="thsys/73367"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11592-1">
    <skos:broader rdf:resource="thsys/71081"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11592-1">
    <skos:broader rdf:resource="descriptor/11596-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11592-1">
    <skos:broader rdf:resource="descriptor/19370-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17919-0">
    <skos:broader rdf:resource="thsys/71030"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17919-0">
    <skos:broader rdf:resource="descriptor/17931-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71110">
    <skos:narrower rdf:resource="descriptor/11847-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71110">
    <skos:narrower rdf:resource="descriptor/11782-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71110">
    <skos:narrower rdf:resource="descriptor/12697-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71110">
    <skos:narrower rdf:resource="descriptor/11793-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71110">
    <skos:narrower rdf:resource="descriptor/11833-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71110">
    <skos:narrower rdf:resource="descriptor/11790-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71110">
    <skos:narrower rdf:resource="descriptor/10981-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71110">
    <skos:narrower rdf:resource="descriptor/11210-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71110">
    <skos:narrower rdf:resource="descriptor/11788-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71110">
    <skos:narrower rdf:resource="descriptor/18182-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71110">
    <skos:narrower rdf:resource="descriptor/16552-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71110">
    <skos:narrower rdf:resource="descriptor/19246-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71110">
    <skos:narrower rdf:resource="descriptor/11910-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71110">
    <skos:narrower rdf:resource="descriptor/11787-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71110">
    <skos:narrower rdf:resource="descriptor/24782-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71110">
    <skos:narrower rdf:resource="descriptor/11786-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71110">
    <skos:narrower rdf:resource="descriptor/11821-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71110">
    <skos:narrower rdf:resource="descriptor/11820-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71110">
    <skos:narrower rdf:resource="descriptor/18478-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71110">
    <skos:narrower rdf:resource="descriptor/10500-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71110">
    <skos:narrower rdf:resource="descriptor/11823-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71110">
    <skos:narrower rdf:resource="descriptor/11817-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71110">
    <skos:narrower rdf:resource="descriptor/13639-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71110">
    <skos:narrower rdf:resource="descriptor/18338-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71110">
    <skos:narrower rdf:resource="descriptor/11807-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71110">
    <skos:narrower rdf:resource="descriptor/11810-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71110">
    <skos:narrower rdf:resource="descriptor/19245-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71110">
    <skos:narrower rdf:resource="descriptor/10522-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71110">
    <skos:narrower rdf:resource="descriptor/20409-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71110">
    <skos:narrower rdf:resource="descriptor/10521-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71110">
    <skos:narrower rdf:resource="descriptor/18060-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71110">
    <skos:narrower rdf:resource="descriptor/11792-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71110">
    <skos:narrower rdf:resource="descriptor/18010-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71110">
    <skos:narrower rdf:resource="descriptor/11784-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71110">
    <skos:narrower rdf:resource="descriptor/11783-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71110">
    <skos:narrower rdf:resource="descriptor/18707-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71110">
    <skos:narrower rdf:resource="descriptor/19248-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71110">
    <skos:narrower rdf:resource="descriptor/19254-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71110">
    <skos:narrower rdf:resource="descriptor/11801-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11687-4">
    <skos:altLabel xml:lang="de">Steuer-Transfer-System</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11687-4">
    <skos:altLabel xml:lang="de">Negativsteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11687-4">
    <skos:altLabel xml:lang="de">Integriertes Steuer-Transfer-System</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11687-4">
    <skos:altLabel xml:lang="de">Bürgersteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11687-4">
    <skos:altLabel xml:lang="de">Bürgergeld</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16615-4">
    <skos:scopeNote xml:lang="de">Vorwiegend soziale Assimilierung und Gleichbehandlung von Immigranten, Flüchtlingen und anderen Minderheiten in einer neuen Umgebung sowie die auf Integration gerichtete Politik.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11394-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15362-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10817-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18181-1">
    <skos:related rdf:resource="descriptor/19351-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18181-1">
    <skos:related rdf:resource="descriptor/10612-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18181-1">
    <skos:related rdf:resource="descriptor/10687-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12249-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18443-4">
    <skos:broader rdf:resource="thsys/70986"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18443-4">
    <skos:broader rdf:resource="descriptor/10470-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16638-6">
    <skos:broader rdf:resource="thsys/73395"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16638-6">
    <skos:broader rdf:resource="thsys/71033"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16638-6">
    <skos:broader rdf:resource="thsys/70504"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19786-4">
    <skos:broader rdf:resource="thsys/73347"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19786-4">
    <skos:broader rdf:resource="descriptor/18891-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12744-6">
    <skos:prefLabel xml:lang="en">Corporate production theory</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12744-6">
    <skos:prefLabel xml:lang="de">Betriebswirtschaftliche Produktionstheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18684-3">
    <skos:altLabel xml:lang="de">Indexierte Anleihe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18684-3">
    <skos:altLabel xml:lang="de">Anleihenindexierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18684-3">
    <skos:altLabel xml:lang="de">Aktienindexanleihe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10511-4">
    <skos:altLabel xml:lang="de">Subsistenzbereich</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10511-4">
    <skos:altLabel xml:lang="de">Selbstversorgungswirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10511-4">
    <skos:altLabel xml:lang="de">Bedarfsdeckungswirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id68">
    <rdfs:seeAlso rdf:resource="descriptor/16834-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70201">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">P.00</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70253">
    <skos:narrower rdf:resource="descriptor/17641-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70253">
    <skos:narrower rdf:resource="descriptor/17639-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70253">
    <skos:narrower rdf:resource="descriptor/17638-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70253">
    <skos:narrower rdf:resource="descriptor/17636-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70253">
    <skos:narrower rdf:resource="descriptor/17634-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70253">
    <skos:narrower rdf:resource="descriptor/18285-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70253">
    <skos:narrower rdf:resource="descriptor/17632-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70253">
    <skos:narrower rdf:resource="descriptor/17608-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70253">
    <skos:narrower rdf:resource="descriptor/17630-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70253">
    <skos:narrower rdf:resource="descriptor/17628-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70253">
    <skos:narrower rdf:resource="descriptor/17627-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70253">
    <skos:narrower rdf:resource="descriptor/17626-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70253">
    <skos:narrower rdf:resource="descriptor/17624-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70253">
    <skos:narrower rdf:resource="descriptor/17623-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70253">
    <skos:narrower rdf:resource="descriptor/17622-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70253">
    <skos:narrower rdf:resource="descriptor/17621-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70253">
    <skos:narrower rdf:resource="descriptor/17618-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70253">
    <skos:narrower rdf:resource="descriptor/17596-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10834-1">
    <skos:related rdf:resource="descriptor/10675-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10834-1">
    <skos:related rdf:resource="descriptor/10668-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10834-1">
    <skos:related rdf:resource="descriptor/19311-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10834-1">
    <skos:related rdf:resource="descriptor/10604-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10834-1">
    <skos:related rdf:resource="descriptor/15078-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12518-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17932-1">
    <skos:prefLabel xml:lang="en">Suriname</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17932-1">
    <skos:prefLabel xml:lang="de">Suriname</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19401-1">
    <skos:altLabel xml:lang="de">Rückzahlung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19401-1">
    <skos:altLabel xml:lang="de">Rückerstattung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18701-1">
    <skos:prefLabel xml:lang="en">Interest rate swap</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18701-1">
    <skos:prefLabel xml:lang="de">Zinsswap</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19580-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18620-1">
    <skos:narrower rdf:resource="descriptor/20897-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18906-4">
    <skos:narrower rdf:resource="descriptor/13312-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25501-4">
    <skos:broader rdf:resource="thsys/73387"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25501-4">
    <skos:broader rdf:resource="thsys/70109"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16741-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14600-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10764-3">
    <skos:prefLabel xml:lang="en">Currency</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10764-3">
    <skos:prefLabel xml:lang="de">Währung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73296">
    <rdfs:label xml:lang="en">Publishing industry</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73296">
    <rdfs:label xml:lang="de">Verlagsgewerbe</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70825">
    <skos:prefLabel xml:lang="en">W.29  Media industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70825">
    <skos:prefLabel xml:lang="de">W.29  Medienwirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17738-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12793-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13699-1">
    <skos:related rdf:resource="descriptor/13713-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13699-1">
    <skos:related rdf:resource="descriptor/12232-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19043-1">
    <skos:broader rdf:resource="thsys/71052"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19043-1">
    <skos:broader rdf:resource="thsys/70087"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19043-1">
    <skos:broader rdf:resource="descriptor/13992-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14844-6">
    <skos:broader rdf:resource="thsys/70066"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14844-6">
    <skos:broader rdf:resource="descriptor/14846-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11374-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70869">
    <skos:broader rdf:resource="thsys/70871"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10058-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19102-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14735-4">
    <zbwext:useInsteadNote rdf:nodeID="id174"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70128">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">G.05</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19407-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18099-0">
    <skos:broader rdf:resource="thsys/70828"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18099-0">
    <skos:broader rdf:resource="thsys/70819"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18099-0">
    <skos:broader rdf:resource="thsys/70384"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18099-0">
    <skos:broader rdf:resource="descriptor/18922-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18940-4">
    <skos:broader rdf:resource="thsys/71036"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18940-4">
    <skos:broader rdf:resource="thsys/70823"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18940-4">
    <skos:broader rdf:resource="descriptor/16774-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18940-4">
    <skos:broader rdf:resource="descriptor/13341-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14346-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14455-3">
    <skos:related rdf:resource="descriptor/13298-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14455-3">
    <skos:related rdf:resource="descriptor/18594-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14455-3">
    <skos:related rdf:resource="descriptor/18593-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14455-3">
    <skos:related rdf:resource="descriptor/18592-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14455-3">
    <skos:related rdf:resource="descriptor/13546-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14455-3">
    <skos:related rdf:resource="descriptor/18298-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14455-3">
    <skos:related rdf:resource="descriptor/13576-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14455-3">
    <skos:related rdf:resource="descriptor/13256-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16661-4">
    <zbwext:useInsteadNote rdf:nodeID="id220"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16738-2">
    <skos:related rdf:resource="descriptor/16743-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16738-2">
    <skos:related rdf:resource="descriptor/16740-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16738-2">
    <skos:related rdf:resource="descriptor/10252-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16738-2">
    <skos:related rdf:resource="descriptor/16737-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13026-3">
    <skos:related rdf:resource="descriptor/14296-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10968-1">
    <skos:scopeNote xml:lang="de">Politische Einflussnahme von organisierten Gruppen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id367">
    <rdfs:seeAlso rdf:resource="descriptor/11572-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19547-1">
    <skos:altLabel xml:lang="en">Inflation-forecast targeting</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19547-1">
    <skos:altLabel xml:lang="de">Inflationsziel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19031-1">
    <skos:prefLabel xml:lang="en">Classification</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19031-1">
    <skos:prefLabel xml:lang="de">Klassifikation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11297-3">
    <skos:related rdf:resource="descriptor/15107-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11297-3">
    <skos:related rdf:resource="descriptor/11294-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11297-3">
    <skos:related rdf:resource="descriptor/11298-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11297-3">
    <skos:related rdf:resource="descriptor/11264-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13350-1">
    <skos:related rdf:resource="descriptor/15685-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11975-6">
    <skos:prefLabel xml:lang="en">City centre</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11975-6">
    <skos:prefLabel xml:lang="de">Stadtzentrum</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11304-4">
    <skos:related rdf:resource="descriptor/13992-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11304-4">
    <skos:related rdf:resource="descriptor/11313-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16382-1">
    <skos:related rdf:resource="descriptor/16266-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70651">
    <skos:prefLabel xml:lang="de">P.04.01  Regenerative Energieträger</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15480-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10414-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12281-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14232-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11787-0">
    <skos:broader rdf:resource="thsys/71110"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11787-0">
    <skos:broader rdf:resource="thsys/70233"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11787-0">
    <skos:broader rdf:resource="descriptor/11782-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10051-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10963-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10494-6">
    <skos:altLabel xml:lang="en">Human development index</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10494-6">
    <skos:altLabel xml:lang="en">HDI (Human development index)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70998">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13245-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15705-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15720-5">
    <skos:altLabel xml:lang="de">Soziostrukturelle Entwicklung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15720-5">
    <skos:altLabel xml:lang="de">Soziokulturelle Entwicklung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15720-5">
    <skos:altLabel xml:lang="de">Soziale Unterentwicklung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15720-5">
    <skos:altLabel xml:lang="de">Soziale Entwicklung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15720-5">
    <skos:altLabel xml:lang="de">Kulturwandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15720-5">
    <skos:altLabel xml:lang="de">Gesellschaftsentwicklung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15720-5">
    <skos:altLabel xml:lang="de">Gesellschaftlicher Wandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15720-5">
    <skos:altLabel xml:lang="de">Gesellschaftlicher Strukturwandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15720-5">
    <skos:altLabel xml:lang="de">Gesellschaftliche Entwicklung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11928-1">
    <skos:related rdf:resource="descriptor/11903-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11928-1">
    <skos:related rdf:resource="descriptor/11990-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11928-1">
    <skos:related rdf:resource="descriptor/16552-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11928-1">
    <skos:related rdf:resource="descriptor/11869-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11928-1">
    <skos:related rdf:resource="descriptor/19429-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11928-1">
    <skos:related rdf:resource="descriptor/11873-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="../stw">
    <dcterms:publisher xml:lang="en">German National Library of Economics (ZBW) / Leibniz Information Center for Economics</dcterms:publisher>
  </rdf:Description>
  <rdf:Description rdf:about="../stw">
    <dcterms:publisher xml:lang="de">ZBW - Deutsche Zentralbibliothek für Wirtschaftswissenschaften / Leibniz-Informationszentrum Wirtschaft</dcterms:publisher>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12918-6">
    <skos:prefLabel xml:lang="en">Consumer goods industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12918-6">
    <skos:prefLabel xml:lang="de">Konsumgüterindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12201-2">
    <skos:broader rdf:resource="thsys/70087"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12201-2">
    <skos:broader rdf:resource="descriptor/12085-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12292-1">
    <skos:prefLabel xml:lang="en">Foreign portfolio investment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12292-1">
    <skos:prefLabel xml:lang="de">Portfolio-Investition</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16171-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10838-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10332-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19312-0">
    <skos:prefLabel xml:lang="en">New trade theory</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19312-0">
    <skos:prefLabel xml:lang="de">Neue Außenhandelstheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11516-3">
    <skos:related rdf:resource="descriptor/10421-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18794-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18662-6">
    <skos:scopeNote xml:lang="de">Nur für das Management von Computernetzen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18856-0">
    <skos:prefLabel xml:lang="en">Dropouts</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18856-0">
    <skos:prefLabel xml:lang="de">Abbrecher</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16554-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19332-1">
    <skos:prefLabel xml:lang="en">Ruble</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19332-1">
    <skos:prefLabel xml:lang="de">Rubel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12649-0">
    <skos:scopeNote xml:lang="de">Rohstoffe, Halbfertig- und Fertigfabrikate, die als Ausgangs- und Grundstoffe in die Erzeugnisse eines Betriebes eingehen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19175-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10910-4">
    <skos:broader rdf:resource="thsys/73391"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10910-4">
    <skos:broader rdf:resource="thsys/70101"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10910-4">
    <skos:broader rdf:resource="descriptor/10964-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10910-4">
    <skos:broader rdf:resource="descriptor/10895-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11293-4">
    <skos:altLabel xml:lang="de">Berufssoziologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11293-4">
    <skos:altLabel xml:lang="de">Berufsdeckungsanalyse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11293-4">
    <skos:altLabel xml:lang="de">Berufliche Substitutionsanalyse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70829">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19170-1">
    <skos:altLabel xml:lang="x-other">Pacto Andino</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19170-1">
    <skos:altLabel xml:lang="x-other">Grupo Andino</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19170-1">
    <skos:altLabel xml:lang="de">Andenpakt-Länder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19170-1">
    <skos:altLabel xml:lang="de">Andengemeinschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13349-0">
    <skos:related rdf:resource="descriptor/15685-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13349-0">
    <skos:related rdf:resource="descriptor/19770-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13349-0">
    <skos:related rdf:resource="descriptor/13343-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13349-0">
    <skos:related rdf:resource="descriptor/10436-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16131-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12313-5">
    <skos:prefLabel xml:lang="en">International financial reporting standards</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12313-5">
    <skos:prefLabel xml:lang="de">International Financial Reporting Standards</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16654-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12083-3">
    <skos:narrower rdf:resource="descriptor/20468-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12083-3">
    <skos:narrower rdf:resource="descriptor/19834-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12992-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15350-5">
    <skos:altLabel xml:lang="en">Univariate analysis of variance</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15350-5">
    <skos:altLabel xml:lang="en">Multivariate analysis of variance</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15350-5">
    <skos:altLabel xml:lang="en">MANOVA model</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15350-5">
    <skos:altLabel xml:lang="en">Cross classification</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15350-5">
    <skos:altLabel xml:lang="en">ANOVA model</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15350-5">
    <skos:altLabel xml:lang="en">Analysis of covariance</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15350-5">
    <skos:altLabel xml:lang="de">Univariate Varianzanalyse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15350-5">
    <skos:altLabel xml:lang="de">Multivariate Varianzanalyse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15350-5">
    <skos:altLabel xml:lang="de">Kreuzklassifikationsanalyse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15350-5">
    <skos:altLabel xml:lang="de">Kovarianzanalyse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15350-5">
    <skos:altLabel xml:lang="de">Interaktionsanalyse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19150-0">
    <skos:prefLabel xml:lang="en">Qualitative data analysis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19150-0">
    <skos:prefLabel xml:lang="de">Qualitatives Verfahren</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15937-1">
    <skos:prefLabel xml:lang="en">Marriage</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15937-1">
    <skos:prefLabel xml:lang="de">Ehe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17758-5">
    <skos:broader rdf:resource="thsys/70026"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id219">
    <rdfs:seeAlso rdf:resource="descriptor/13158-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16542-5">
    <skos:broader rdf:resource="thsys/70833"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16542-5">
    <skos:broader rdf:resource="thsys/70324"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16542-5">
    <skos:broader rdf:resource="descriptor/16545-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15974-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18662-6">
    <skos:broader rdf:resource="thsys/71037"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18662-6">
    <skos:broader rdf:resource="descriptor/18017-0"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id189">
    <rdf:value xml:lang="de">Für die Entwicklung des ländlichen Raumes</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12031-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18409-4">
    <skos:prefLabel xml:lang="en">Conveyor industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18409-4">
    <skos:prefLabel xml:lang="de">Fördermittelindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15220-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19061-6">
    <skos:related rdf:resource="descriptor/13145-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19061-6">
    <skos:related rdf:resource="descriptor/14846-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19061-6">
    <skos:related rdf:resource="descriptor/13141-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19061-6">
    <skos:related rdf:resource="descriptor/18436-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19061-6">
    <skos:related rdf:resource="descriptor/18094-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19037-3">
    <skos:scopeNote xml:lang="de">Für theoretische Literatur und Beschreibung von Schätzverfahren BENUTZE Schätztheorie bzw. entsprechende Deskriptoren.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11684-3">
    <skos:narrower rdf:resource="descriptor/11710-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11684-3">
    <skos:narrower rdf:resource="descriptor/11691-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11684-3">
    <skos:narrower rdf:resource="descriptor/11686-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11684-3">
    <skos:narrower rdf:resource="descriptor/11698-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11684-3">
    <skos:narrower rdf:resource="descriptor/19406-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18222-6">
    <skos:broader rdf:resource="thsys/73298"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14776-4">
    <skos:related rdf:resource="descriptor/13011-2"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id141">
    <rdf:value xml:lang="de">Für staatliche Maßnahmen zur Förderung des Industrialisierungsprozesses</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11863-3">
    <skos:broader rdf:resource="thsys/71067"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11863-3">
    <skos:broader rdf:resource="thsys/70925"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11530-2">
    <skos:broader rdf:resource="thsys/70144"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10909-3">
    <skos:related rdf:resource="descriptor/20206-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10909-3">
    <skos:related rdf:resource="descriptor/10951-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10909-3">
    <skos:related rdf:resource="descriptor/10216-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10909-3">
    <skos:related rdf:resource="descriptor/10960-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10909-3">
    <skos:related rdf:resource="descriptor/10213-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10909-3">
    <skos:related rdf:resource="descriptor/10908-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10909-3">
    <skos:related rdf:resource="descriptor/12794-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13988-1">
    <skos:broader rdf:resource="thsys/73378"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13988-1">
    <skos:broader rdf:resource="thsys/71054"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13988-1">
    <skos:broader rdf:resource="thsys/70853"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13988-1">
    <skos:broader rdf:resource="descriptor/18535-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14412-0">
    <skos:altLabel xml:lang="en">Second-hand car</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14412-0">
    <skos:altLabel xml:lang="de">Gebrauchtwagen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14412-0">
    <skos:altLabel xml:lang="de">Gebrauchtkraftfahrzeug</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14819-5">
    <skos:related rdf:resource="descriptor/14821-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11157-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13632-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19186-0">
    <skos:prefLabel xml:lang="en">Machinery market</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19186-0">
    <skos:prefLabel xml:lang="de">Maschinenmarkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10673-6">
    <skos:prefLabel xml:lang="en">Optimum currency area</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10673-6">
    <skos:prefLabel xml:lang="de">Optimaler Währungsraum</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18384-1">
    <skos:related rdf:resource="descriptor/18401-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18384-1">
    <skos:related rdf:resource="descriptor/18390-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18384-1">
    <skos:related rdf:resource="descriptor/18375-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18384-1">
    <skos:related rdf:resource="descriptor/12980-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19272-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19462-2">
    <skos:prefLabel xml:lang="en">Rural labour market</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19462-2">
    <skos:prefLabel xml:lang="de">Ländlicher Arbeitsmarkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12485-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15463-6">
    <skos:related rdf:resource="descriptor/19606-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15463-6">
    <skos:related rdf:resource="descriptor/16440-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15463-6">
    <skos:related rdf:resource="descriptor/15446-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19561-0">
    <skos:prefLabel xml:lang="en">Industrial metabolism</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19561-0">
    <skos:prefLabel xml:lang="de">Stoffstrom-Management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71118">
    <skos:narrower rdf:resource="descriptor/10156-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71118">
    <skos:narrower rdf:resource="descriptor/19210-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71118">
    <skos:narrower rdf:resource="descriptor/10166-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71118">
    <skos:narrower rdf:resource="descriptor/19558-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71118">
    <skos:narrower rdf:resource="descriptor/10164-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71118">
    <skos:narrower rdf:resource="descriptor/10162-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71118">
    <skos:narrower rdf:resource="descriptor/10161-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71118">
    <skos:narrower rdf:resource="descriptor/10159-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71118">
    <skos:narrower rdf:resource="descriptor/10160-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71118">
    <skos:narrower rdf:resource="descriptor/10095-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71118">
    <skos:narrower rdf:resource="descriptor/16739-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71118">
    <skos:narrower rdf:resource="descriptor/10151-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12842-6">
    <skos:altLabel xml:lang="en">Sound branding</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12842-6">
    <skos:altLabel xml:lang="en">Sonic branding</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12842-6">
    <skos:altLabel xml:lang="en">Audio branding</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12842-6">
    <skos:altLabel xml:lang="en">Acoustic branding</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12842-6">
    <skos:altLabel xml:lang="de">Werbe-Jingle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12842-6">
    <skos:altLabel xml:lang="de">Akustische Markenführung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18501-2">
    <skos:prefLabel xml:lang="en">Railroad construction</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18501-2">
    <skos:prefLabel xml:lang="de">Streckenbau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71012">
    <skos:narrower rdf:resource="descriptor/18364-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71012">
    <skos:narrower rdf:resource="descriptor/18363-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71012">
    <skos:narrower rdf:resource="descriptor/18358-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71012">
    <skos:narrower rdf:resource="descriptor/18374-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71012">
    <skos:narrower rdf:resource="descriptor/18367-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71012">
    <skos:narrower rdf:resource="descriptor/18458-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71012">
    <skos:narrower rdf:resource="descriptor/20888-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71012">
    <skos:narrower rdf:resource="descriptor/18362-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71012">
    <skos:narrower rdf:resource="descriptor/18370-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71012">
    <skos:narrower rdf:resource="descriptor/18373-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71012">
    <skos:narrower rdf:resource="descriptor/18371-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71012">
    <skos:narrower rdf:resource="descriptor/11812-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71012">
    <skos:narrower rdf:resource="descriptor/18376-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71012">
    <skos:narrower rdf:resource="descriptor/11811-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71012">
    <skos:narrower rdf:resource="descriptor/18812-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71012">
    <skos:narrower rdf:resource="descriptor/11810-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71012">
    <skos:narrower rdf:resource="descriptor/18360-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71012">
    <skos:narrower rdf:resource="descriptor/18346-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13775-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18003-4">
    <skos:scopeNote xml:lang="de">Methode der künstlichen Intelligenz zur Lösung von Routineaufgaben von Experten eines Bereichs. Hardware und Software.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12495-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11067-6">
    <skos:scopeNote xml:lang="de">Übertragung von Elementen demokratischer Abstimmungsverfahren auf den ökonomischen Bereich.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10018-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10677-5">
    <skos:prefLabel xml:lang="en">Effects on the balance of payments</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10677-5">
    <skos:prefLabel xml:lang="de">Zahlungsbilanzmechanismus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73381">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14538-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17471-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10619-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11313-3">
    <skos:related rdf:resource="descriptor/18125-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11313-3">
    <skos:related rdf:resource="descriptor/19503-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11313-3">
    <skos:related rdf:resource="descriptor/18123-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11313-3">
    <skos:related rdf:resource="descriptor/18852-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11313-3">
    <skos:related rdf:resource="descriptor/16629-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11313-3">
    <skos:related rdf:resource="descriptor/19502-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11313-3">
    <skos:related rdf:resource="descriptor/18850-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11313-3">
    <skos:related rdf:resource="descriptor/11304-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10704-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15498-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18836-6">
    <skos:altLabel xml:lang="de">Sozio-Marketing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18836-6">
    <skos:altLabel xml:lang="de">Sozialmarketing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71117">
    <skos:prefLabel xml:lang="en">V.02.06  Economics of information</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71117">
    <skos:prefLabel xml:lang="de">V.02.06  Informationsökonomik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11398-4">
    <skos:narrower rdf:resource="descriptor/18877-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16794-6">
    <skos:scopeNote xml:lang="de">Nur für Leihverkehr zwischen Bibliotheken.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16019-6">
    <skos:altLabel xml:lang="de">Verwaltungsvorschrift</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16019-6">
    <skos:altLabel xml:lang="de">Verwaltungsverfahrensrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16019-6">
    <skos:altLabel xml:lang="de">Verwaltungsprozessrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16019-6">
    <skos:altLabel xml:lang="de">Verwaltungsprozeßrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16019-6">
    <skos:altLabel xml:lang="de">Verwaltungsgerichtsordnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16019-6">
    <skos:altLabel xml:lang="de">Polizeirecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16019-6">
    <skos:altLabel xml:lang="de">Planfeststellung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12055-1">
    <skos:related rdf:resource="descriptor/10323-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12055-1">
    <skos:related rdf:resource="descriptor/20972-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70313">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15696-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19827-2">
    <skos:broader rdf:resource="thsys/71119"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19827-2">
    <skos:broader rdf:resource="descriptor/11039-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10473-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12094-5">
    <zbwext:useInsteadNote rdf:nodeID="id284"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18070-3">
    <skos:altLabel xml:lang="de">Kreativitätstechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13347-4">
    <skos:narrower rdf:resource="descriptor/13360-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13347-4">
    <skos:narrower rdf:resource="descriptor/18929-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13347-4">
    <skos:narrower rdf:resource="descriptor/18449-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13347-4">
    <skos:narrower rdf:resource="descriptor/13386-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16186-1">
    <skos:prefLabel xml:lang="en">International law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16186-1">
    <skos:prefLabel xml:lang="de">Internationales Recht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14831-1">
    <skos:related rdf:resource="descriptor/14833-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14831-1">
    <skos:related rdf:resource="descriptor/13220-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14831-1">
    <skos:related rdf:resource="descriptor/13137-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14831-1">
    <skos:related rdf:resource="descriptor/14829-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14040-4">
    <skos:altLabel xml:lang="de">Kunstgewerbeprodukt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14040-4">
    <skos:altLabel xml:lang="de">Gemälde</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14040-4">
    <skos:altLabel xml:lang="de">Antiquitäten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18505-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18603-1">
    <skos:related rdf:resource="descriptor/18602-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18603-1">
    <skos:related rdf:resource="descriptor/18601-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18603-1">
    <skos:related rdf:resource="descriptor/18599-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18603-1">
    <skos:related rdf:resource="descriptor/13533-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14011-4">
    <skos:prefLabel xml:lang="en">Natural scientists</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14011-4">
    <skos:prefLabel xml:lang="de">Naturwissenschaftler</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21002-0">
    <skos:related rdf:resource="descriptor/13823-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21002-0">
    <skos:related rdf:resource="descriptor/13824-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21002-0">
    <skos:related rdf:resource="descriptor/16547-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21002-0">
    <skos:related rdf:resource="descriptor/19517-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11921-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16815-3">
    <skos:scopeNote xml:lang="de">Ohne Türkei</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12816-0">
    <skos:prefLabel xml:lang="en">Special sales</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12816-0">
    <skos:prefLabel xml:lang="de">Sonderverkauf</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10002-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11449-6">
    <skos:related rdf:resource="descriptor/10263-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11449-6">
    <skos:related rdf:resource="descriptor/11460-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11449-6">
    <skos:related rdf:resource="descriptor/11452-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11449-6">
    <skos:related rdf:resource="descriptor/11422-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14048-2">
    <skos:altLabel xml:lang="de">Kugelschreiber</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14048-2">
    <skos:altLabel xml:lang="de">Füllfederhalter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14048-2">
    <skos:altLabel xml:lang="de">Bleistift</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14686-5">
    <skos:altLabel xml:lang="de">Terephthalsäure</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14686-5">
    <skos:altLabel xml:lang="de">Phthalsäureanhydrid</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14686-5">
    <skos:altLabel xml:lang="de">Phthalat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16738-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10534-6">
    <skos:broader rdf:resource="thsys/71085"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10534-6">
    <skos:broader rdf:resource="descriptor/10524-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10960-3">
    <skos:broader rdf:resource="thsys/71071"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10960-3">
    <skos:broader rdf:resource="thsys/70867"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10960-3">
    <skos:broader rdf:resource="descriptor/10962-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10751-5">
    <skos:prefLabel xml:lang="en">Multiple exchange rate</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10751-5">
    <skos:prefLabel xml:lang="de">Multipler Wechselkurs</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10751-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18076-5">
    <skos:broader rdf:resource="thsys/72393"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18076-5">
    <skos:broader rdf:resource="descriptor/15373-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12166-6">
    <skos:broader rdf:resource="thsys/70485"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12166-6">
    <skos:broader rdf:resource="thsys/70383"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12166-6">
    <skos:broader rdf:resource="descriptor/12165-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19547-1">
    <skos:broader rdf:resource="thsys/70305"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19547-1">
    <skos:broader rdf:resource="descriptor/10382-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19636-2">
    <skos:altLabel xml:lang="en">Merchant guild</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19636-2">
    <skos:altLabel xml:lang="en">Craft guild</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19636-2">
    <skos:altLabel xml:lang="de">Zunftzwang</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19636-2">
    <skos:altLabel xml:lang="de">Zunftwesen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19636-2">
    <skos:altLabel xml:lang="de">Zunftordnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19636-2">
    <skos:altLabel xml:lang="de">Zünfte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19636-2">
    <skos:altLabel xml:lang="de">Gilde</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18624-0">
    <skos:broader rdf:resource="thsys/70928"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18624-0">
    <skos:broader rdf:resource="descriptor/25742-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18009-6">
    <skos:prefLabel xml:lang="en">Biofuel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18009-6">
    <skos:prefLabel xml:lang="de">Biokraftstoff</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15961-4">
    <skos:prefLabel xml:lang="en">Information law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15961-4">
    <skos:prefLabel xml:lang="de">Informationsrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13813-1">
    <skos:prefLabel xml:lang="en">Auto insurance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13813-1">
    <skos:prefLabel xml:lang="de">Kraftfahrtversicherung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16757-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19645-1">
    <skos:prefLabel xml:lang="en">Demographic transition</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19645-1">
    <skos:prefLabel xml:lang="de">Demographischer Übergang</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17753-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70307">
    <skos:broader rdf:resource="thsys/g"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15901-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15518-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14488-2">
    <skos:altLabel xml:lang="de">Scheinwerfer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14488-2">
    <skos:altLabel xml:lang="de">Elektrische Leuchte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12590-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14404-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10095-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15477-2">
    <skos:broader rdf:resource="thsys/73103"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15477-2">
    <skos:broader rdf:resource="descriptor/15480-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13236-6">
    <skos:related rdf:resource="descriptor/19390-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13236-6">
    <skos:related rdf:resource="descriptor/13029-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13236-6">
    <skos:related rdf:resource="descriptor/14285-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13691-3">
    <skos:prefLabel xml:lang="en">Banking history</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13691-3">
    <skos:prefLabel xml:lang="de">Bankgeschichte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10583-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19418-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71100">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.07.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18575-1">
    <skos:related rdf:resource="descriptor/18501-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18575-1">
    <skos:related rdf:resource="descriptor/13255-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18575-1">
    <skos:related rdf:resource="descriptor/13525-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18575-1">
    <skos:related rdf:resource="descriptor/16166-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18575-1">
    <skos:related rdf:resource="descriptor/18572-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18575-1">
    <skos:related rdf:resource="descriptor/18571-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73304">
    <skos:prefLabel xml:lang="de">P.18.01  Naturfasern</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11940-4">
    <skos:related rdf:resource="descriptor/18031-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11940-4">
    <skos:related rdf:resource="descriptor/19448-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11940-4">
    <skos:related rdf:resource="descriptor/19451-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11940-4">
    <skos:related rdf:resource="descriptor/18490-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11940-4">
    <skos:related rdf:resource="descriptor/18122-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11940-4">
    <skos:related rdf:resource="descriptor/11981-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11940-4">
    <skos:related rdf:resource="descriptor/11904-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14527-4">
    <skos:altLabel xml:lang="en">Glasses</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14527-4">
    <skos:altLabel xml:lang="de">Kontaktlinsen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14527-4">
    <skos:altLabel xml:lang="de">Brille</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18428-0">
    <zbwext:useInsteadNote rdf:nodeID="id327"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13721-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12091-4">
    <skos:related rdf:resource="descriptor/19751-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12091-4">
    <skos:related rdf:resource="descriptor/12105-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12091-4">
    <skos:related rdf:resource="descriptor/12125-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12091-4">
    <skos:related rdf:resource="descriptor/12111-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12091-4">
    <skos:related rdf:resource="descriptor/12129-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12091-4">
    <skos:related rdf:resource="descriptor/12106-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11685-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15009-0">
    <skos:altLabel xml:lang="en">Refuse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15009-0">
    <skos:altLabel xml:lang="de">Reststoff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15009-0">
    <skos:altLabel xml:lang="de">Müll</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15009-0">
    <skos:altLabel xml:lang="de">Altstoff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19135-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14973-2">
    <skos:altLabel xml:lang="de">Malz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14973-2">
    <skos:altLabel xml:lang="de">Alkoholfreies Bier</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14973-2">
    <skos:altLabel xml:lang="de">Alkoholarmes Bier</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13181-5">
    <skos:related rdf:resource="descriptor/13903-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13181-5">
    <skos:related rdf:resource="descriptor/18746-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11927-3">
    <skos:broader rdf:resource="thsys/71070"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11927-3">
    <skos:broader rdf:resource="thsys/70188"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11927-3">
    <skos:broader rdf:resource="descriptor/11928-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11927-3">
    <skos:broader rdf:resource="descriptor/10076-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14750-1">
    <skos:narrower rdf:resource="descriptor/18249-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14750-1">
    <skos:narrower rdf:resource="descriptor/14534-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18987-6">
    <skos:prefLabel xml:lang="en">Logging</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18987-6">
    <skos:prefLabel xml:lang="de">Holzeinschlag</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11182-6">
    <skos:related rdf:resource="descriptor/19495-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11182-6">
    <skos:related rdf:resource="descriptor/10261-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11182-6">
    <skos:related rdf:resource="descriptor/11191-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11182-6">
    <skos:related rdf:resource="descriptor/19494-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11182-6">
    <skos:related rdf:resource="descriptor/11220-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11182-6">
    <skos:related rdf:resource="descriptor/11264-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11182-6">
    <skos:related rdf:resource="descriptor/18168-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11182-6">
    <skos:related rdf:resource="descriptor/11235-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11182-6">
    <skos:related rdf:resource="descriptor/11183-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11182-6">
    <skos:related rdf:resource="descriptor/13406-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18945-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13706-2">
    <zbwext:useInsteadNote rdf:nodeID="id261"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17413-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15725-2">
    <skos:related rdf:resource="descriptor/10994-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15725-2">
    <skos:related rdf:resource="descriptor/16318-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15725-2">
    <skos:related rdf:resource="descriptor/16545-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15725-2">
    <skos:related rdf:resource="descriptor/15720-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15725-2">
    <skos:related rdf:resource="descriptor/18909-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12341-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10746-5">
    <skos:related rdf:resource="descriptor/10772-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10746-5">
    <skos:related rdf:resource="descriptor/10770-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10746-5">
    <skos:related rdf:resource="descriptor/19337-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10746-5">
    <skos:related rdf:resource="descriptor/10764-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10746-5">
    <skos:related rdf:resource="descriptor/19334-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10746-5">
    <skos:related rdf:resource="descriptor/10785-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10746-5">
    <skos:related rdf:resource="descriptor/10745-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10746-5">
    <skos:related rdf:resource="descriptor/11481-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11788-5">
    <skos:prefLabel xml:lang="en">Declining industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11788-5">
    <skos:prefLabel xml:lang="de">Schrumpfungsbranche</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70358">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19046-2">
    <skos:scopeNote xml:lang="de">Dazu gehören z.B. überbetriebliche Maschinenverwendung sowie die Tätigkeit von Zucht-, Prüf- und Kontrollverbänden.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70438">
    <skos:broader rdf:resource="thsys/70554"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12376-2">
    <skos:related rdf:resource="descriptor/19217-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14816-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14631-2">
    <skos:altLabel xml:lang="en">Spray</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14631-2">
    <skos:altLabel xml:lang="de">Aerosolprodukt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10564-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18968-3">
    <skos:prefLabel xml:lang="en">Single-parent family</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18968-3">
    <skos:prefLabel xml:lang="de">Alleinerziehende</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19565-6">
    <skos:broader rdf:resource="thsys/70485"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19565-6">
    <skos:broader rdf:resource="descriptor/12089-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19565-6">
    <skos:broader rdf:resource="descriptor/18029-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13453-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10103-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11919-2">
    <skos:broader rdf:resource="thsys/71070"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11919-2">
    <skos:broader rdf:resource="descriptor/10092-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11663-4">
    <skos:altLabel xml:lang="de">Kuponsteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11663-4">
    <skos:altLabel xml:lang="de">Kapitalrentensteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11663-4">
    <skos:altLabel xml:lang="de">Kapitalertragsbesteuerung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11663-4">
    <skos:altLabel xml:lang="de">Kapitaleinkommensteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11663-4">
    <skos:altLabel xml:lang="de">Couponsteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24785-4">
    <skos:related rdf:resource="descriptor/12665-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24785-4">
    <skos:related rdf:resource="descriptor/12786-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24785-4">
    <skos:related rdf:resource="descriptor/19133-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12913-2">
    <skos:prefLabel xml:lang="en">Industries</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12913-2">
    <skos:prefLabel xml:lang="de">Industrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12577-6">
    <skos:broader rdf:resource="thsys/73385"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12577-6">
    <skos:broader rdf:resource="thsys/70244"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12577-6">
    <skos:broader rdf:resource="descriptor/15712-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11599-1">
    <skos:broader rdf:resource="thsys/71079"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11599-1">
    <skos:broader rdf:resource="descriptor/11540-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16401-2">
    <skos:altLabel xml:lang="de">Sozialethik der Kirche</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16401-2">
    <skos:altLabel xml:lang="de">Protestantische Soziallehre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16401-2">
    <skos:altLabel xml:lang="de">Kirchliche Soziallehre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16401-2">
    <skos:altLabel xml:lang="de">Katholische Soziallehre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16401-2">
    <skos:altLabel xml:lang="de">Evangelische Soziallehre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16401-2">
    <skos:altLabel xml:lang="de">Christliche Sozialethik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12035-0">
    <skos:prefLabel xml:lang="en">Incoming orders</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12035-0">
    <skos:prefLabel xml:lang="de">Auftrag</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11820-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17731-4">
    <skos:prefLabel xml:lang="en">Burundi</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17731-4">
    <skos:prefLabel xml:lang="de">Burundi</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18277-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14357-3">
    <skos:broader rdf:resource="thsys/70424"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14357-3">
    <skos:broader rdf:resource="descriptor/14355-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19691-1">
    <zbwext:useInsteadNote rdf:nodeID="id168"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11673-1">
    <skos:related rdf:resource="descriptor/11667-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18725-1">
    <skos:prefLabel xml:lang="en">Consumer credit act</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18725-1">
    <skos:prefLabel xml:lang="de">Kreditvertragsrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19757-4">
    <skos:related rdf:resource="descriptor/19756-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19757-4">
    <skos:related rdf:resource="descriptor/15945-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19757-4">
    <skos:related rdf:resource="descriptor/16678-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19757-4">
    <skos:related rdf:resource="descriptor/16698-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73344">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.10.06.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18705-0">
    <skos:altLabel xml:lang="de">Öffentlich-rechtliches Kreditinstitut</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18705-0">
    <skos:altLabel xml:lang="de">Öffentliches Kreditinstitut</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18705-0">
    <skos:altLabel xml:lang="de">Öffentliche Kreditwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16336-1">
    <skos:scopeNote xml:lang="de">Reform der öffentlichen Verwaltung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18313-3">
    <skos:related rdf:resource="descriptor/10511-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18313-3">
    <skos:related rdf:resource="descriptor/15930-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18313-3">
    <skos:related rdf:resource="descriptor/19191-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18313-3">
    <skos:related rdf:resource="descriptor/13451-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18313-3">
    <skos:related rdf:resource="descriptor/11803-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14023-4">
    <skos:broader rdf:resource="thsys/70201"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10850-3">
    <skos:prefLabel xml:lang="en">Letter of credit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10850-3">
    <skos:prefLabel xml:lang="de">Akkreditiv</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19566-4">
    <zbwext:useInsteadNote rdf:nodeID="id21"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19074-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18410-5">
    <skos:prefLabel xml:lang="en">Power plant construction</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18410-5">
    <skos:prefLabel xml:lang="de">Kraftwerksbau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18372-1">
    <skos:prefLabel xml:lang="en">Oil and gas lease</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18372-1">
    <skos:prefLabel xml:lang="de">Erdölkonzession</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11872-2">
    <skos:altLabel xml:lang="de">Intraregionale Einkommensverteilung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11872-2">
    <skos:altLabel xml:lang="de">Interregionale Einkommensverteilung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71035">
    <skos:broader rdf:resource="thsys/b"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14357-3">
    <skos:prefLabel xml:lang="en">Motor</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14357-3">
    <skos:prefLabel xml:lang="de">Motor</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14593-5">
    <skos:altLabel xml:lang="de">Schmuckwaren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14593-5">
    <skos:altLabel xml:lang="de">Perlmutt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14593-5">
    <skos:altLabel xml:lang="de">Perle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14593-5">
    <skos:altLabel xml:lang="de">Modeschmuck</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14593-5">
    <skos:altLabel xml:lang="de">Juwelen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14593-5">
    <skos:altLabel xml:lang="de">Bijouterie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14593-5">
    <skos:altLabel xml:lang="de">Bernstein</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10151-1">
    <skos:altLabel xml:lang="de">Neumann-Morgenstern-Nutzen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10151-1">
    <skos:altLabel xml:lang="de">Erwartungswert des Nutzens</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10151-1">
    <skos:altLabel xml:lang="de">Bernoulli-Prinzip</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10151-1">
    <skos:altLabel xml:lang="de">Bernoulli-Nutzen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73378">
    <skos:broader rdf:resource="thsys/73382"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#semanticRelation">
    <rdfs:comment xml:lang="en">This property should not be used directly, but as a super-property
      for all properties denoting a relationship of meaning between concepts.</rdfs:comment>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11474-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id161">
    <rdfs:seeAlso rdf:resource="descriptor/18192-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70301">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">B.05</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19235-6">
    <zbwext:useInsteadNote rdf:nodeID="id230"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10109-0">
    <skos:broader rdf:resource="thsys/70057"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10109-0">
    <skos:broader rdf:resource="descriptor/10113-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10211-2">
    <skos:broader rdf:resource="thsys/70178"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10211-2">
    <skos:broader rdf:resource="descriptor/10217-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10211-2">
    <skos:broader rdf:resource="descriptor/12700-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19231-0">
    <skos:prefLabel xml:lang="en">Aggregate production</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19231-0">
    <skos:prefLabel xml:lang="de">Gesamtwirtschaftliche Produktion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10812-4">
    <skos:broader rdf:resource="thsys/71089"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10812-4">
    <skos:broader rdf:resource="descriptor/12251-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13528-0">
    <skos:related rdf:resource="descriptor/13529-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13528-0">
    <skos:related rdf:resource="descriptor/13242-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#broaderTransitive">
    <rdfs:comment xml:lang="en">By convention, skos:broaderTransitive is not used to make
      assertions. Rather, the properties can be used to draw inferences about the transitive closure
      of the hierarchical relation, which is useful e.g. when implementing a simple query expansion
      algorithm in a search application.</rdfs:comment>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10676-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19384-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12184-4">
    <skos:broader rdf:resource="thsys/70087"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12184-4">
    <skos:broader rdf:resource="descriptor/12192-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12184-4">
    <skos:broader rdf:resource="descriptor/12186-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10581-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12226-0">
    <skos:prefLabel xml:lang="en">Cash flow statement</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12226-0">
    <skos:prefLabel xml:lang="de">Kapitalflussrechnung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19088-0">
    <skos:altLabel xml:lang="de">Kreditgeberinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19088-0">
    <skos:altLabel xml:lang="de">Kreditgeber</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19088-0">
    <skos:altLabel xml:lang="de">Gläubigerverzug</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19088-0">
    <skos:altLabel xml:lang="de">Gläubigerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12189-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10897-5">
    <skos:altLabel xml:lang="de">Verschmelzung von Unternehmen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10897-5">
    <skos:altLabel xml:lang="de">Unternehmensverschmelzung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11328-4">
    <skos:broader rdf:resource="thsys/70278"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11328-4">
    <skos:broader rdf:resource="descriptor/11345-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11328-4">
    <skos:broader rdf:resource="descriptor/11322-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12588-1">
    <skos:prefLabel xml:lang="en">Career planning</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12588-1">
    <skos:prefLabel xml:lang="de">Karriereplanung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10562-1">
    <zbwext:useInsteadNote rdf:nodeID="id89"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18299-6">
    <skos:prefLabel xml:lang="en">Tea production</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18299-6">
    <skos:prefLabel xml:lang="de">Teeanbau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12225-2">
    <zbwext:useInsteadNote rdf:nodeID="id162"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73357">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.06.03.03</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15926-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18403-2">
    <skos:prefLabel xml:lang="en">Electric steel plant</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18403-2">
    <skos:prefLabel xml:lang="de">Elektrostahlwerk</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12013-3">
    <skos:related rdf:resource="descriptor/11829-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12013-3">
    <skos:related rdf:resource="descriptor/11828-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12013-3">
    <skos:related rdf:resource="descriptor/13593-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12013-3">
    <skos:related rdf:resource="descriptor/13510-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12013-3">
    <skos:related rdf:resource="descriptor/18620-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12013-3">
    <skos:related rdf:resource="descriptor/13633-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12013-3">
    <skos:related rdf:resource="descriptor/20897-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70239">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18068-4">
    <skos:broader rdf:resource="thsys/73373"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18068-4">
    <skos:broader rdf:resource="thsys/70154"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18068-4">
    <skos:broader rdf:resource="descriptor/15970-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14198-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14196-1">
    <skos:broader rdf:resource="thsys/70418"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14196-1">
    <skos:broader rdf:resource="descriptor/14603-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14196-1">
    <skos:broader rdf:resource="descriptor/18235-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14196-1">
    <skos:broader rdf:resource="descriptor/14192-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19442-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16038-2">
    <skos:altLabel xml:lang="de">Vereinsrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16038-2">
    <skos:altLabel xml:lang="de">Vereinsgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16038-2">
    <skos:altLabel xml:lang="de">Vereinsförderungsgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16038-2">
    <skos:altLabel xml:lang="de">Verbandsklage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16038-2">
    <skos:altLabel xml:lang="de">Verbändegesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12747-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10991-6">
    <skos:scopeNote xml:lang="de">Ökonomische Analysen staatlicher Maßnahmen, deren Zielerreichungsgrad und Effizienz.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18448-1">
    <skos:related rdf:resource="descriptor/13367-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18448-1">
    <skos:related rdf:resource="descriptor/13084-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18448-1">
    <skos:related rdf:resource="descriptor/14573-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18448-1">
    <skos:related rdf:resource="descriptor/15661-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18448-1">
    <skos:related rdf:resource="descriptor/18920-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18448-1">
    <skos:related rdf:resource="descriptor/14513-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11239-3">
    <skos:prefLabel xml:lang="en">Employment policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11239-3">
    <skos:prefLabel xml:lang="de">Beschäftigungspolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70905">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#scopeNote">
    <rdf:type rdf:resource="http://www.w3.org/1999/02/22-rdf-syntax-ns#Property"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#scopeNote">
    <rdf:type rdf:resource="http://www.w3.org/2002/07/owl#AnnotationProperty"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14329-1">
    <skos:prefLabel xml:lang="en">Fittings (Armature)</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14329-1">
    <skos:prefLabel xml:lang="de">Armatur</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70028">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13510-5">
    <skos:related rdf:resource="descriptor/13573-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13510-5">
    <skos:related rdf:resource="descriptor/13577-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13510-5">
    <skos:related rdf:resource="descriptor/18625-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13510-5">
    <skos:related rdf:resource="descriptor/13569-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13510-5">
    <skos:related rdf:resource="descriptor/11833-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13510-5">
    <skos:related rdf:resource="descriptor/13513-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13510-5">
    <skos:related rdf:resource="descriptor/12013-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13510-5">
    <skos:related rdf:resource="descriptor/13264-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13510-5">
    <skos:related rdf:resource="descriptor/13979-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13510-5">
    <skos:related rdf:resource="descriptor/15709-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14837-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19845-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19853-1">
    <skos:related rdf:resource="descriptor/19728-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19853-1">
    <skos:related rdf:resource="descriptor/16184-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11613-5">
    <skos:broader rdf:resource="thsys/71078"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11613-5">
    <skos:broader rdf:resource="descriptor/10019-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11613-5">
    <skos:broader rdf:resource="descriptor/11540-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11613-5">
    <skos:broader rdf:resource="descriptor/12251-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13350-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73343">
    <skos:prefLabel xml:lang="en">N.10.06.02  Materials research</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73343">
    <skos:prefLabel xml:lang="de">N.10.06.02  Werkstoffforschung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10985-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12406-5">
    <skos:broader rdf:resource="thsys/71043"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12406-5">
    <skos:broader rdf:resource="descriptor/19142-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://zbw.eu/namespaces/skos-extensions/useInsteadNote">
    <skos:definition xml:lang="en">For an aspect of the current concept use the 
    concept to which the note points instead of the current.</skos:definition>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70101">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19414-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10166-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18910-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70934">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70373">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18625-5">
    <skos:prefLabel xml:lang="en">Transport statistics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18625-5">
    <skos:prefLabel xml:lang="de">Verkehrsstatistik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19491-2">
    <skos:broader rdf:resource="thsys/70235"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19491-2">
    <skos:broader rdf:resource="thsys/70052"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19491-2">
    <skos:broader rdf:resource="descriptor/11272-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19491-2">
    <skos:broader rdf:resource="descriptor/19902-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16990-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18225-0">
    <skos:prefLabel xml:lang="en">Slovakia</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18225-0">
    <skos:prefLabel xml:lang="de">Slowakei</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12957-3">
    <skos:related rdf:resource="descriptor/12955-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12957-3">
    <skos:related rdf:resource="descriptor/14990-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12331-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18497-2">
    <skos:prefLabel xml:lang="en">Foreign construction work</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18497-2">
    <skos:prefLabel xml:lang="de">Auslandsbau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12108-6">
    <skos:prefLabel xml:lang="en">Corporate culture</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12108-6">
    <skos:prefLabel xml:lang="de">Unternehmenskultur</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18418-3">
    <skos:broader rdf:resource="thsys/70998"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18418-3">
    <skos:broader rdf:resource="thsys/70932"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18418-3">
    <skos:broader rdf:resource="thsys/70878"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18418-3">
    <skos:broader rdf:resource="descriptor/13573-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18418-3">
    <skos:broader rdf:resource="descriptor/10436-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13662-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18830-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18953-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71045">
    <skos:prefLabel xml:lang="de">B.02.01.02  Fremdkapitalbeschaffung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19808-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12492-0">
    <skos:narrower rdf:resource="descriptor/12511-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16124-2">
    <skos:altLabel xml:lang="de">Schutzrechte für Arbeitnehmer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16124-2">
    <skos:altLabel xml:lang="de">Gesetzlicher Arbeitnehmerschutz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16124-2">
    <skos:altLabel xml:lang="de">Arbeitnehmerschutzrechte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18585-5">
    <skos:broader rdf:resource="thsys/70937"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18585-5">
    <skos:broader rdf:resource="descriptor/13577-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18404-0">
    <skos:prefLabel xml:lang="en">Minimill</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18404-0">
    <skos:prefLabel xml:lang="de">Kleinstahlwerk</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14107-5">
    <skos:narrower rdf:resource="descriptor/19014-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16979-1">
    <skos:prefLabel xml:lang="en">Vaud</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16979-1">
    <skos:prefLabel xml:lang="de">Waadt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19153-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12023-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13370-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10966-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13669-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16324-1">
    <skos:broader rdf:resource="thsys/73380"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16324-1">
    <skos:broader rdf:resource="descriptor/16329-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13441-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14808-3">
    <skos:prefLabel xml:lang="en">Wallpaper</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14808-3">
    <skos:prefLabel xml:lang="de">Tapete</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16709-2">
    <skos:altLabel xml:lang="de">Wertethik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16709-2">
    <skos:altLabel xml:lang="de">Tugend</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16709-2">
    <skos:altLabel xml:lang="de">Moralphilosophie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16709-2">
    <skos:altLabel xml:lang="de">Morallehre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16709-2">
    <skos:altLabel xml:lang="de">Moral</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70875">
    <rdfs:label xml:lang="de">Beratungsdienstleistungen</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18843-2">
    <zbwext:useInsteadNote rdf:nodeID="id268"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16824-2">
    <skos:altLabel xml:lang="de">Fürstentum Liechtenstein</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18117-3">
    <skos:scopeNote xml:lang="de">Z.B. Nutzung von vorhandenen Ressourcen zur Herstellung artverwandter Produkte.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15661-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11688-2">
    <skos:broader rdf:resource="thsys/71076"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11688-2">
    <skos:broader rdf:resource="descriptor/11694-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11683-5">
    <skos:prefLabel xml:lang="en">Capital gains tax</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11683-5">
    <skos:prefLabel xml:lang="de">Wertzuwachssteuer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15125-5">
    <skos:broader rdf:resource="thsys/72183"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15125-5">
    <skos:broader rdf:resource="thsys/71035"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15125-5">
    <skos:broader rdf:resource="thsys/70292"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15125-5">
    <skos:broader rdf:resource="descriptor/15119-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10040-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10560-5">
    <zbwext:useInsteadNote rdf:nodeID="id91"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10560-5">
    <zbwext:useInsteadNote rdf:nodeID="id90"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13210-3">
    <skos:altLabel xml:lang="en">Commerce</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13210-3">
    <skos:altLabel xml:lang="de">Warenhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13210-3">
    <skos:altLabel xml:lang="de">Kaufmannshandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70990">
    <skos:prefLabel xml:lang="de">W.06.01.08  Papier- und Verlagsgewerbe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20314-0">
    <skos:related rdf:resource="descriptor/18942-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20314-0">
    <skos:related rdf:resource="descriptor/12809-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20314-0">
    <skos:related rdf:resource="descriptor/10016-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15662-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13180-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15945-2">
    <skos:altLabel xml:lang="de">Geschlechtsproportion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15945-2">
    <skos:altLabel xml:lang="de">Geschlechterproportion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18647-2">
    <skos:altLabel xml:lang="de">Multimediakommunikation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18647-2">
    <skos:altLabel xml:lang="de">Hypermedia</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70936">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19591-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12083-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13117-0">
    <skos:prefLabel xml:lang="en">Pulp and paper industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13117-0">
    <skos:prefLabel xml:lang="de">Papierwirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16265-5">
    <skos:prefLabel xml:lang="en">Manipulation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16265-5">
    <skos:prefLabel xml:lang="de">Manipulation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20341-4">
    <skos:related rdf:resource="descriptor/19655-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20341-4">
    <skos:related rdf:resource="descriptor/19145-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20341-4">
    <skos:related rdf:resource="descriptor/18659-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20341-4">
    <skos:related rdf:resource="descriptor/18940-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20341-4">
    <skos:related rdf:resource="descriptor/18767-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20341-4">
    <skos:related rdf:resource="descriptor/16782-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19085-6">
    <skos:broader rdf:resource="thsys/70307"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20855-1">
    <skos:prefLabel xml:lang="en">Governance approach</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20855-1">
    <skos:prefLabel xml:lang="de">Governance-Ansatz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71053">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18118-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19244-5">
    <skos:broader rdf:resource="thsys/70406"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19244-5">
    <skos:broader rdf:resource="descriptor/10434-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14524-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18950-1">
    <skos:broader rdf:resource="thsys/70822"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11663-4">
    <skos:broader rdf:resource="thsys/71074"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11663-4">
    <skos:broader rdf:resource="descriptor/11650-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19236-4">
    <skos:scopeNote xml:lang="de">Gesamt- und einzelwirtschaftliche Verbrauchsausgaben z.B. für Ernährung, Bekleidung, Wohnen, Freizeit.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11382-5">
    <skos:broader rdf:resource="thsys/70847"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11382-5">
    <skos:broader rdf:resource="thsys/70143"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13180-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70383">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">B.01.03</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14895-3">
    <skos:broader rdf:resource="thsys/70066"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14895-3">
    <skos:broader rdf:resource="descriptor/18277-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14895-3">
    <skos:broader rdf:resource="descriptor/14846-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14378-2">
    <skos:prefLabel xml:lang="en">Foundry machine</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14378-2">
    <skos:prefLabel xml:lang="de">Gießereimaschine</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13837-1">
    <skos:prefLabel xml:lang="en">Housing rent</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13837-1">
    <skos:prefLabel xml:lang="de">Miete</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70815">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11483-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71034">
    <skos:narrower rdf:resource="descriptor/12071-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71034">
    <skos:narrower rdf:resource="descriptor/12303-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71034">
    <skos:narrower rdf:resource="descriptor/12070-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71034">
    <skos:narrower rdf:resource="descriptor/12069-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71034">
    <skos:narrower rdf:resource="descriptor/12304-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71034">
    <skos:narrower rdf:resource="descriptor/19249-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71034">
    <skos:narrower rdf:resource="descriptor/12058-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71034">
    <skos:narrower rdf:resource="descriptor/12057-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71034">
    <skos:narrower rdf:resource="descriptor/19159-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71034">
    <skos:narrower rdf:resource="descriptor/18120-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71034">
    <skos:narrower rdf:resource="descriptor/16058-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71034">
    <skos:narrower rdf:resource="descriptor/12049-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71034">
    <skos:narrower rdf:resource="descriptor/10897-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71034">
    <skos:narrower rdf:resource="descriptor/12046-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71034">
    <skos:narrower rdf:resource="descriptor/24624-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71034">
    <skos:narrower rdf:resource="descriptor/18032-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71034">
    <skos:narrower rdf:resource="descriptor/12038-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71034">
    <skos:narrower rdf:resource="descriptor/19158-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71034">
    <skos:narrower rdf:resource="descriptor/11905-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71034">
    <skos:narrower rdf:resource="descriptor/11904-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71034">
    <skos:narrower rdf:resource="descriptor/18001-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10637-3">
    <skos:broader rdf:resource="thsys/71100"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10637-3">
    <skos:broader rdf:resource="descriptor/10629-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10139-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11216-1">
    <skos:altLabel xml:lang="en">Disguised unemployment</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11216-1">
    <skos:altLabel xml:lang="de">Verdeckte Arbeitslosigkeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11216-1">
    <skos:altLabel xml:lang="de">Stille Reserve am Arbeitsmarkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10628-4">
    <skos:prefLabel xml:lang="en">Customs union</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10628-4">
    <skos:prefLabel xml:lang="de">Zollunion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13564-3">
    <skos:prefLabel xml:lang="en">Traffic jam</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13564-3">
    <skos:prefLabel xml:lang="de">Verkehrsstau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14512-3">
    <skos:altLabel xml:lang="de">Wäschetrockner</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14512-3">
    <skos:altLabel xml:lang="de">Wäscheschleuder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14357-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14710-6">
    <skos:broader rdf:resource="thsys/70476"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14710-6">
    <skos:broader rdf:resource="descriptor/14702-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15073-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13411-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15135-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14976-3">
    <skos:narrower rdf:resource="descriptor/14972-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14976-3">
    <skos:narrower rdf:resource="descriptor/14971-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14995-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10320-4">
    <skos:broader rdf:resource="thsys/70310"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10320-4">
    <skos:broader rdf:resource="thsys/70233"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10320-4">
    <skos:broader rdf:resource="descriptor/10323-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12024-5">
    <skos:altLabel xml:lang="de">Urbarmachung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12024-5">
    <skos:altLabel xml:lang="de">Melioration</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12024-5">
    <skos:altLabel xml:lang="de">Kultivierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12024-5">
    <skos:altLabel xml:lang="de">Bodengewinnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17754-6">
    <skos:prefLabel xml:lang="en">Tunisia</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17754-6">
    <skos:prefLabel xml:lang="de">Tunesien</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19717-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13405-2">
    <skos:prefLabel xml:lang="en">Charitable organization</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13405-2">
    <skos:prefLabel xml:lang="de">Wohlfahrtsorganisation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11186-5">
    <skos:broader rdf:resource="thsys/70233"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11186-5">
    <skos:broader rdf:resource="thsys/70052"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11186-5">
    <skos:broader rdf:resource="thsys/70030"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15931-6">
    <skos:prefLabel xml:lang="en">Ethnic relations</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15931-6">
    <skos:prefLabel xml:lang="de">Ethnische Beziehungen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15743-0">
    <skos:related rdf:resource="descriptor/15779-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15743-0">
    <skos:related rdf:resource="descriptor/18623-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15743-0">
    <skos:related rdf:resource="descriptor/15755-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15743-0">
    <skos:related rdf:resource="descriptor/18072-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15743-0">
    <skos:related rdf:resource="descriptor/15742-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15743-0">
    <skos:related rdf:resource="descriptor/19623-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15743-0">
    <skos:related rdf:resource="descriptor/14640-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15743-0">
    <skos:related rdf:resource="descriptor/12016-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15743-0">
    <skos:related rdf:resource="descriptor/15728-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18611-2">
    <skos:prefLabel xml:lang="en">Air passenger transport</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18611-2">
    <skos:prefLabel xml:lang="de">Passagierluftfahrt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10302-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14433-6">
    <skos:prefLabel xml:lang="en">Motor vehicle</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14433-6">
    <skos:prefLabel xml:lang="de">Kraftfahrzeug</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13485-6">
    <skos:altLabel xml:lang="de">Kundenverkehrsanalyse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13485-6">
    <skos:altLabel xml:lang="de">Kundenstrukturanalyse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13485-6">
    <skos:altLabel xml:lang="de">Kundenbeobachtung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13485-6">
    <skos:altLabel xml:lang="de">Käuferstrukturanalyse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13782-0">
    <skos:scopeNote xml:lang="de">Abtretung einer Einzelforderung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14507-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71051">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">B.04.03</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18233-1">
    <skos:prefLabel xml:lang="en">Groundwater</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18233-1">
    <skos:prefLabel xml:lang="de">Grundwasser</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#memberList">
    <skos:definition xml:lang="en">An RDF list containing the members of an ordered collection.</skos:definition>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17622-2">
    <skos:altLabel xml:lang="de">Persien</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17622-2">
    <skos:altLabel xml:lang="de">Kaiserreich Iran</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17622-2">
    <skos:altLabel xml:lang="de">Islamische Republik Iran</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19267-0">
    <skos:broader rdf:resource="thsys/70455"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16050-5">
    <skos:broader rdf:resource="thsys/73391"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16050-5">
    <skos:broader rdf:resource="thsys/70087"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16050-5">
    <skos:broader rdf:resource="descriptor/16042-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10343-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11001-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11046-0">
    <skos:related rdf:resource="descriptor/10986-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11046-0">
    <skos:related rdf:resource="descriptor/13992-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11046-0">
    <skos:related rdf:resource="descriptor/11073-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11046-0">
    <skos:related rdf:resource="descriptor/11055-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11046-0">
    <skos:related rdf:resource="descriptor/10348-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11046-0">
    <skos:related rdf:resource="descriptor/11083-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21014-0">
    <skos:prefLabel xml:lang="en">Tax reduction</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21014-0">
    <skos:prefLabel xml:lang="de">Steuersenkung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13644-5">
    <skos:altLabel xml:lang="de">Urlaubsverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13644-5">
    <skos:altLabel xml:lang="de">Reiseverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13644-5">
    <skos:altLabel xml:lang="de">Fremdenverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13644-5">
    <skos:altLabel xml:lang="de">Freizeitverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13644-5">
    <skos:altLabel xml:lang="de">Erholungsreiseverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13644-5">
    <skos:altLabel xml:lang="de">Ausflugsverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18755-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11824-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16311-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16050-5">
    <skos:altLabel xml:lang="de">Unternehmensrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10996-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11281-4">
    <skos:broader rdf:resource="thsys/71059"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11281-4">
    <skos:broader rdf:resource="thsys/70244"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11281-4">
    <skos:broader rdf:resource="descriptor/11270-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11281-4">
    <skos:broader rdf:resource="descriptor/11187-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70867">
    <skos:prefLabel xml:lang="de">W.22.01.04  Wirtschaftsaufsicht und Wirtschaftsförderung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19768-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17039-2">
    <skos:altLabel xml:lang="x-other">Repubblica Italiana</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17039-2">
    <skos:altLabel xml:lang="en">Italia</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17039-2">
    <skos:altLabel xml:lang="de">Italienische Republik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10434-3">
    <skos:related rdf:resource="descriptor/10414-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10434-3">
    <skos:related rdf:resource="descriptor/10064-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10434-3">
    <skos:related rdf:resource="descriptor/10439-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10434-3">
    <skos:related rdf:resource="descriptor/10466-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10434-3">
    <skos:related rdf:resource="descriptor/10350-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10434-3">
    <skos:related rdf:resource="descriptor/10347-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10434-3">
    <skos:related rdf:resource="descriptor/10126-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10434-3">
    <skos:related rdf:resource="descriptor/10411-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10434-3">
    <skos:related rdf:resource="descriptor/10256-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10434-3">
    <skos:related rdf:resource="descriptor/19242-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19172-4">
    <skos:broader rdf:resource="thsys/70307"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19172-4">
    <skos:broader rdf:resource="descriptor/18019-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18246-6">
    <skos:related rdf:resource="descriptor/18612-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10501-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18750-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13024-0">
    <skos:altLabel xml:lang="de">Ziegelei</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13024-0">
    <skos:altLabel xml:lang="de">Mauerziegelindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13024-0">
    <skos:altLabel xml:lang="de">Dachziegelindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10870-4">
    <skos:related rdf:resource="descriptor/13767-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11879-2">
    <skos:prefLabel xml:lang="en">Settlement</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11879-2">
    <skos:prefLabel xml:lang="de">Siedlung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14554-1">
    <skos:altLabel xml:lang="de">Kohlenherd</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14554-1">
    <skos:altLabel xml:lang="de">Kochgerät</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14554-1">
    <skos:altLabel xml:lang="de">Gasherd</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14554-1">
    <skos:altLabel xml:lang="de">Elektroherd</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10882-4">
    <skos:broader rdf:resource="thsys/71091"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10882-4">
    <skos:broader rdf:resource="thsys/70952"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10882-4">
    <skos:broader rdf:resource="thsys/70866"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10882-4">
    <skos:broader rdf:resource="descriptor/18810-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10284-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19541-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10373-4">
    <skos:prefLabel xml:lang="en">Cyclical indicator</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10373-4">
    <skos:prefLabel xml:lang="de">Konjunkturindikator</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12452-5">
    <skos:altLabel xml:lang="de">Weltbilanz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12452-5">
    <skos:altLabel xml:lang="de">Weltabschluss</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12452-5">
    <skos:altLabel xml:lang="de">Weltabschluß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12452-5">
    <skos:altLabel xml:lang="de">Teilkonzernabschluss</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12452-5">
    <skos:altLabel xml:lang="de">Teilkonzernabschluß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12452-5">
    <skos:altLabel xml:lang="de">Konzernrechnungslegung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12452-5">
    <skos:altLabel xml:lang="de">Konzernbilanzierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12452-5">
    <skos:altLabel xml:lang="de">Konzernbilanz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12452-5">
    <skos:altLabel xml:lang="de">Konzernabschluß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15901-1">
    <skos:broader rdf:resource="thsys/73352"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15901-1">
    <skos:broader rdf:resource="descriptor/19622-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15901-1">
    <skos:broader rdf:resource="descriptor/15889-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18813-4">
    <skos:scopeNote xml:lang="de">Private Finanzierung von Infrastrukturinvestitionen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#Concept">
    <rdfs:label xml:lang="en">Concept</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11267-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12508-4">
    <skos:prefLabel xml:lang="en">Unit costs</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12508-4">
    <skos:prefLabel xml:lang="de">Stückkosten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11954-0">
    <skos:broader rdf:resource="thsys/71070"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11954-0">
    <skos:broader rdf:resource="descriptor/10426-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12106-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70995">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.06.01.05</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12829-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10637-3">
    <skos:prefLabel xml:lang="en">Factor proportions</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10637-3">
    <skos:prefLabel xml:lang="de">Faktorproportionentheorem</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10805-1">
    <skos:related rdf:resource="descriptor/10785-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10805-1">
    <skos:related rdf:resource="descriptor/10812-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10805-1">
    <skos:related rdf:resource="descriptor/10807-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10805-1">
    <skos:related rdf:resource="descriptor/18694-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10805-1">
    <skos:related rdf:resource="descriptor/10806-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10805-1">
    <skos:related rdf:resource="descriptor/10804-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10805-1">
    <skos:related rdf:resource="descriptor/10797-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71024">
    <skos:prefLabel xml:lang="de">W.01.03  Landwirtschaftliche Betriebssysteme</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19749-3">
    <skos:narrower rdf:resource="descriptor/19770-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11518-6">
    <skos:prefLabel xml:lang="en">Price-wage mechanism</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11518-6">
    <skos:prefLabel xml:lang="de">Preis-Lohn-Spirale</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15739-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17777-1">
    <skos:prefLabel xml:lang="en">Costa Rica</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17777-1">
    <skos:prefLabel xml:lang="de">Costa Rica</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19494-3">
    <skos:scopeNote xml:lang="de">Hypothese einer Beziehung zwischen Arbeitslosigkeit und offenen Stellen, analog zur Phillips-Kurve.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12212-4">
    <skos:related rdf:resource="descriptor/12230-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12212-4">
    <skos:related rdf:resource="descriptor/19828-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12212-4">
    <skos:related rdf:resource="descriptor/18029-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12212-4">
    <skos:related rdf:resource="descriptor/12158-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12212-4">
    <skos:related rdf:resource="descriptor/12759-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12212-4">
    <skos:related rdf:resource="descriptor/12280-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12212-4">
    <skos:related rdf:resource="descriptor/12206-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12212-4">
    <skos:related rdf:resource="descriptor/15467-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12212-4">
    <skos:related rdf:resource="descriptor/12209-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12212-4">
    <skos:related rdf:resource="descriptor/12215-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12212-4">
    <skos:related rdf:resource="descriptor/19537-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12212-4">
    <skos:related rdf:resource="descriptor/12214-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11290-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19678-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10964-2">
    <skos:narrower rdf:resource="descriptor/19132-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10964-2">
    <skos:narrower rdf:resource="descriptor/10910-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10964-2">
    <skos:narrower rdf:resource="descriptor/16051-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10964-2">
    <skos:narrower rdf:resource="descriptor/16059-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10964-2">
    <skos:narrower rdf:resource="descriptor/24662-6"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id244">
    <rdfs:seeAlso rdf:resource="descriptor/19562-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19008-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12719-5">
    <skos:broader rdf:resource="thsys/70310"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12719-5">
    <skos:broader rdf:resource="descriptor/12725-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70257">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">P.09</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15060-0">
    <skos:altLabel xml:lang="en">Probability (Statistics)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15060-0">
    <skos:altLabel xml:lang="de">Wahrscheinlichkeitstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15060-0">
    <skos:altLabel xml:lang="de">Wahrscheinlichkeit (Statistik)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15060-0">
    <skos:altLabel xml:lang="de">Statistische Wahrscheinlichkeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15060-0">
    <skos:altLabel xml:lang="de">Mathematische Wahrscheinlichkeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14105-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14700-2">
    <skos:prefLabel xml:lang="en">Fibre reinforced plastics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14700-2">
    <skos:prefLabel xml:lang="de">Faserverstärkter Kunststoff</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70142">
    <rdfs:label xml:lang="de">Demographie und Bevölkerung</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70463">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15786-3">
    <skos:scopeNote xml:lang="de">Auch für die Analyse von Lehr- und Lernsituationen sowie von Lernproblemen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24622-4">
    <skos:scopeNote xml:lang="de">Vertragsgestaltung, welche die Entlohnung vom Ausmaß der Vertrags-/Leistungserfüllung abhängig macht, um die Leistungsmotivation (z.B. von Führungskräften) zu steigern.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15628-0">
    <skos:related rdf:resource="descriptor/18814-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15628-0">
    <skos:related rdf:resource="descriptor/19529-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15628-0">
    <skos:related rdf:resource="descriptor/15669-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15628-0">
    <skos:related rdf:resource="descriptor/13003-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15628-0">
    <skos:related rdf:resource="descriptor/14035-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15628-0">
    <skos:related rdf:resource="descriptor/18367-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15628-0">
    <skos:related rdf:resource="descriptor/14182-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15628-0">
    <skos:related rdf:resource="descriptor/15001-2"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id391">
    <rdf:value xml:lang="de">für kurzfristig erwartete Tendenzen</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13186-2">
    <skos:prefLabel xml:lang="en">Shopfitting</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13186-2">
    <skos:prefLabel xml:lang="de">Ladenbau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14339-5">
    <skos:broader rdf:resource="thsys/70549"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14339-5">
    <skos:broader rdf:resource="thsys/70346"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14339-5">
    <skos:broader rdf:resource="descriptor/14567-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14339-5">
    <skos:broader rdf:resource="descriptor/14336-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15924-3">
    <skos:broader rdf:resource="thsys/73357"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15924-3">
    <skos:broader rdf:resource="thsys/70142"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15924-3">
    <skos:broader rdf:resource="descriptor/15909-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#note">
    <rdfs:isDefinedBy rdf:resource="http://www.w3.org/2004/02/skos/core"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11374-4">
    <skos:broader rdf:resource="thsys/71055"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11374-4">
    <skos:broader rdf:resource="thsys/71052"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11374-4">
    <skos:broader rdf:resource="thsys/70847"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11374-4">
    <skos:broader rdf:resource="descriptor/18535-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70903">
    <rdfs:label xml:lang="en">Banking system</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70903">
    <rdfs:label xml:lang="de">Bankensystem</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19776-0">
    <skos:altLabel xml:lang="en">Speech</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19776-0">
    <skos:altLabel xml:lang="de">Argument</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14854-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19695-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16283-3">
    <skos:narrower rdf:resource="descriptor/10994-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16283-3">
    <skos:narrower rdf:resource="descriptor/15725-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16283-3">
    <skos:narrower rdf:resource="descriptor/11398-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13459-0">
    <skos:narrower rdf:resource="descriptor/10859-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15760-0">
    <skos:prefLabel xml:lang="en">Environmental charge</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15760-0">
    <skos:prefLabel xml:lang="de">Umweltabgabe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18093-5">
    <skos:prefLabel xml:lang="en">Small farm</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18093-5">
    <skos:prefLabel xml:lang="de">Landwirtschaftlicher Kleinbetrieb</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19448-3">
    <skos:broader rdf:resource="thsys/71069"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19448-3">
    <skos:broader rdf:resource="thsys/71067"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19448-3">
    <skos:broader rdf:resource="descriptor/19291-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19448-3">
    <skos:broader rdf:resource="descriptor/18835-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19011-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14117-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19112-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14700-2">
    <skos:related rdf:resource="descriptor/14849-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13264-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19638-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10019-1">
    <skos:altLabel xml:lang="en">Indebtedness</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10019-1">
    <skos:altLabel xml:lang="de">Verschuldung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10019-1">
    <skos:altLabel xml:lang="de">Schuldendienst</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16411-6">
    <skos:related rdf:resource="descriptor/16363-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16411-6">
    <skos:related rdf:resource="descriptor/16406-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16411-6">
    <skos:related rdf:resource="descriptor/16426-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10590-3">
    <skos:broader rdf:resource="thsys/71084"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10590-3">
    <skos:broader rdf:resource="thsys/70144"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10550-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16724-6">
    <skos:related rdf:resource="descriptor/19750-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16724-6">
    <skos:related rdf:resource="descriptor/19608-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16724-6">
    <skos:related rdf:resource="descriptor/16556-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10106-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12901-2">
    <skos:altLabel xml:lang="en">Trust association</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12901-2">
    <skos:altLabel xml:lang="de">Treuhandgesellschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16003-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11701-1">
    <skos:scopeNote xml:lang="de">Nur für Steuerbegünstigungen innerhalb eines Staates.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13661-5">
    <skos:prefLabel xml:lang="en">Public Opinion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13661-5">
    <skos:prefLabel xml:lang="de">Öffentliche Meinung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19454-1">
    <skos:related rdf:resource="descriptor/12023-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19454-1">
    <skos:related rdf:resource="descriptor/12019-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19454-1">
    <skos:related rdf:resource="descriptor/11921-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19454-1">
    <skos:related rdf:resource="descriptor/11859-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19454-1">
    <skos:related rdf:resource="descriptor/12015-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11788-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70012">
    <skos:prefLabel xml:lang="en">N.06  Sociology</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70012">
    <skos:prefLabel xml:lang="de">N.06  Soziologie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11336-5">
    <skos:altLabel xml:lang="de">Lohnfondstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11336-5">
    <skos:altLabel xml:lang="de">Grenzproduktivitätstheorie des Lohnes</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11336-5">
    <skos:altLabel xml:lang="de">Gleichgewichtslohn</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11336-5">
    <skos:altLabel xml:lang="de">Existenzminimumtheorie des Lohnes</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11336-5">
    <skos:altLabel xml:lang="de">Ehernes Lohngesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10659-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11299-6">
    <skos:broader rdf:resource="thsys/73355"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11299-6">
    <skos:broader rdf:resource="thsys/71058"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11299-6">
    <skos:broader rdf:resource="descriptor/19761-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11828-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15674-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18910-6">
    <skos:related rdf:resource="descriptor/19501-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18910-6">
    <skos:related rdf:resource="descriptor/11328-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18910-6">
    <skos:related rdf:resource="descriptor/19506-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18910-6">
    <skos:related rdf:resource="descriptor/11770-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18910-6">
    <skos:related rdf:resource="descriptor/10006-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18910-6">
    <skos:related rdf:resource="descriptor/11037-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12754-3">
    <skos:related rdf:resource="descriptor/12708-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12754-3">
    <skos:related rdf:resource="descriptor/19249-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12754-3">
    <skos:related rdf:resource="descriptor/12700-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12754-3">
    <skos:related rdf:resource="descriptor/12748-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14298-0">
    <skos:related rdf:resource="descriptor/11430-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14298-0">
    <skos:related rdf:resource="descriptor/18391-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18532-5">
    <skos:prefLabel xml:lang="en">Trade marketing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18532-5">
    <skos:prefLabel xml:lang="de">Handelsmarketing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19088-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14131-1">
    <skos:prefLabel xml:lang="en">Bee</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14131-1">
    <skos:prefLabel xml:lang="de">Biene</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73391">
    <skos:broader rdf:resource="thsys/70241"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10268-1">
    <skos:related rdf:resource="descriptor/10269-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12689-2">
    <skos:altLabel xml:lang="de">Werkverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12689-2">
    <skos:altLabel xml:lang="de">Materialtransport</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12689-2">
    <skos:altLabel xml:lang="de">Lagertransport</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12689-2">
    <skos:altLabel xml:lang="de">Innerbetrieblicher Transport</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12689-2">
    <skos:altLabel xml:lang="de">Eigenverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12689-2">
    <skos:altLabel xml:lang="de">Eigentransport</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12689-2">
    <skos:altLabel xml:lang="de">Betriebliches Förderwesen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13960-2">
    <zbwext:useInsteadNote rdf:nodeID="id146"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16543-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13252-1">
    <skos:related rdf:resource="descriptor/13298-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13252-1">
    <skos:related rdf:resource="descriptor/13546-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13252-1">
    <skos:related rdf:resource="descriptor/18591-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13252-1">
    <skos:related rdf:resource="descriptor/13545-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13252-1">
    <skos:related rdf:resource="descriptor/18590-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13252-1">
    <skos:related rdf:resource="descriptor/13983-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13252-1">
    <skos:related rdf:resource="descriptor/13256-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12733-4">
    <skos:altLabel xml:lang="de">Rechnergestützte Konstruktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12733-4">
    <skos:altLabel xml:lang="de">EDV-gestützte Konstruktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12733-4">
    <skos:altLabel xml:lang="de">Computergestützte Konstruktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14296-4">
    <skos:narrower rdf:resource="descriptor/14308-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14296-4">
    <skos:narrower rdf:resource="descriptor/14306-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14296-4">
    <skos:narrower rdf:resource="descriptor/14298-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18296-5">
    <skos:prefLabel xml:lang="en">Navigation system</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18296-5">
    <skos:prefLabel xml:lang="de">Navigationssystem</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18857-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12714-1">
    <skos:altLabel xml:lang="en">Setup time</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12714-1">
    <skos:altLabel xml:lang="de">Umrüstzeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12714-1">
    <skos:altLabel xml:lang="de">Einrichtezeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19227-5">
    <skos:prefLabel xml:lang="en">Adverse selection</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19227-5">
    <skos:prefLabel xml:lang="de">Adverse Selection</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73370">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.04.03.04</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14396-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18408-6">
    <skos:prefLabel xml:lang="en">Chemical plant construction</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18408-6">
    <skos:prefLabel xml:lang="de">Chemieanlagenbau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16745-5">
    <skos:prefLabel xml:lang="en">History of thought</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16745-5">
    <skos:prefLabel xml:lang="de">Dogmengeschichte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16272-1">
    <skos:related rdf:resource="descriptor/13661-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16272-1">
    <skos:related rdf:resource="descriptor/15961-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16272-1">
    <skos:related rdf:resource="descriptor/18830-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16272-1">
    <skos:related rdf:resource="descriptor/16266-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73365">
    <rdfs:label xml:lang="en">Security policy</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73365">
    <rdfs:label xml:lang="de">Sicherheitspolitik</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70073">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">G.01.03.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16019-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18956-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12435-5">
    <skos:altLabel xml:lang="de">Substanzielle Kapitalerhaltung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12435-5">
    <skos:altLabel xml:lang="de">Substantielle Kapitalerhaltung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11047-5">
    <skos:related rdf:resource="descriptor/11123-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11047-5">
    <skos:related rdf:resource="descriptor/11060-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11047-5">
    <skos:related rdf:resource="descriptor/11061-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11047-5">
    <skos:related rdf:resource="descriptor/16717-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11047-5">
    <skos:related rdf:resource="descriptor/11088-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11047-5">
    <skos:related rdf:resource="descriptor/16361-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11047-5">
    <skos:related rdf:resource="descriptor/16409-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11047-5">
    <skos:related rdf:resource="descriptor/16247-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11047-5">
    <skos:related rdf:resource="descriptor/16387-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17662-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73348">
    <rdfs:label xml:lang="de">Biowissenschaften</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10346-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10013-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20897-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15860-3">
    <skos:prefLabel xml:lang="en">Rational behaviour</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15860-3">
    <skos:prefLabel xml:lang="de">Rationales Verhalten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18223-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12717-2">
    <skos:related rdf:resource="descriptor/12905-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12717-2">
    <skos:related rdf:resource="descriptor/12716-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12717-2">
    <skos:related rdf:resource="descriptor/12914-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70049">
    <rdfs:label xml:lang="de">Allgemeine Volkswirtschaftstheorie</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15647-3">
    <skos:related rdf:resource="descriptor/17988-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15647-3">
    <skos:related rdf:resource="descriptor/18177-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#example">
    <rdfs:isDefinedBy rdf:resource="http://www.w3.org/2004/02/skos/core"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id221">
    <rdf:value xml:lang="de">Für Monopole des Staates</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70300">
    <skos:broader rdf:resource="thsys/70394"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19137-6">
    <skos:altLabel xml:lang="de">Firmenkundenbetreuung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19137-6">
    <skos:altLabel xml:lang="de">Firmenkundenberatung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11814-2">
    <skos:scopeNote xml:lang="de">I.e.S. die staatliche Inspektion zur Überwachung der Einhaltung von Arbeitsschutzvorschriften.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11419-1">
    <skos:altLabel xml:lang="de">Wertzeichenfälschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11419-1">
    <skos:altLabel xml:lang="de">Münzvergehen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11419-1">
    <skos:altLabel xml:lang="de">Falschmünzerei</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11419-1">
    <skos:altLabel xml:lang="de">Falschgeld</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13325-0">
    <skos:altLabel xml:lang="de">Abfallentsorgungswirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17660-1">
    <skos:prefLabel xml:lang="en">Kenya</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17660-1">
    <skos:prefLabel xml:lang="de">Kenia</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10064-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10036-1">
    <skos:broader rdf:resource="thsys/70049"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15735-6">
    <skos:prefLabel xml:lang="en">Transboundary pollution</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15735-6">
    <skos:prefLabel xml:lang="de">Grenzüberschreitende Umweltbelastung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18761-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19662-1">
    <skos:scopeNote xml:lang="de">Literatur zur Beachtung bzw. Nichtbeachtung von vertraglich eingegangenen oder freiwilligen Verpflichtungen sowie von staatlichen Regelungen oder Gesetzen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19620-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13220-0">
    <skos:broader rdf:resource="thsys/70959"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13220-0">
    <skos:broader rdf:resource="descriptor/18522-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15825-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16013-4">
    <skos:broader rdf:resource="thsys/73381"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16013-4">
    <skos:broader rdf:resource="thsys/73361"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25621-1">
    <skos:related rdf:resource="descriptor/18908-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25621-1">
    <skos:related rdf:resource="descriptor/18975-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11836-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14964-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14926-4">
    <zbwext:useInsteadNote rdf:nodeID="id121"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12057-4">
    <zbwext:useInsteadNote rdf:nodeID="id191"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18625-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11940-4">
    <skos:broader rdf:resource="thsys/71069"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11940-4">
    <skos:broader rdf:resource="descriptor/19291-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11940-4">
    <skos:broader rdf:resource="descriptor/16552-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11940-4">
    <skos:broader rdf:resource="descriptor/19446-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14015-3">
    <skos:related rdf:resource="descriptor/16563-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14015-3">
    <skos:related rdf:resource="descriptor/14014-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15163-4">
    <skos:prefLabel xml:lang="en">Bayesian statistics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15163-4">
    <skos:prefLabel xml:lang="de">Bayes-Statistik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14752-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18730-1">
    <skos:broader rdf:resource="thsys/70893"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18730-1">
    <skos:broader rdf:resource="thsys/70187"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18730-1">
    <skos:broader rdf:resource="descriptor/19664-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18730-1">
    <skos:broader rdf:resource="descriptor/11792-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10489-6">
    <skos:altLabel xml:lang="en">Population explosion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10489-6">
    <skos:altLabel xml:lang="en">Overpopulation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10489-6">
    <skos:altLabel xml:lang="en">Optimum population</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10489-6">
    <skos:altLabel xml:lang="de">Überbevölkerung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10489-6">
    <skos:altLabel xml:lang="de">Bevölkerungsproblem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10489-6">
    <skos:altLabel xml:lang="de">Bevölkerungsoptimum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10489-6">
    <skos:altLabel xml:lang="de">Bevölkerungsexplosion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10489-6">
    <skos:altLabel xml:lang="de">Bevölkerungsdruck</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18990-3">
    <skos:prefLabel xml:lang="en">Water price</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18990-3">
    <skos:prefLabel xml:lang="de">Wasserpreis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13545-0">
    <skos:narrower rdf:resource="descriptor/18601-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12883-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13407-5">
    <skos:broader rdf:resource="thsys/73380"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13407-5">
    <skos:broader rdf:resource="thsys/73366"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13407-5">
    <skos:broader rdf:resource="thsys/70866"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13407-5">
    <skos:broader rdf:resource="descriptor/16489-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12724-5">
    <skos:altLabel xml:lang="de">Massenfertigung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11707-3">
    <skos:altLabel xml:lang="de">Steuerertragsschätzung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11707-3">
    <skos:altLabel xml:lang="de">Steueraufkommensschätzung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19586-5">
    <skos:broader rdf:resource="thsys/72181"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19586-5">
    <skos:broader rdf:resource="thsys/70530"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19586-5">
    <skos:broader rdf:resource="descriptor/15076-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18323-0">
    <skos:related rdf:resource="descriptor/13702-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18323-0">
    <skos:related rdf:resource="descriptor/19273-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18323-0">
    <skos:related rdf:resource="descriptor/19368-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18323-0">
    <skos:related rdf:resource="descriptor/10575-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18323-0">
    <skos:related rdf:resource="descriptor/18673-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10205-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16789-6">
    <skos:altLabel xml:lang="en">Abstracting</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16789-6">
    <skos:altLabel xml:lang="de">Inhaltserschließung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16789-6">
    <skos:altLabel xml:lang="de">Indexieren von Dokumenten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15264-5">
    <skos:altLabel xml:lang="de">ML-Schätzung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15264-5">
    <skos:altLabel xml:lang="de">Methode der größten Mutmaßlichkeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15264-5">
    <skos:altLabel xml:lang="de">Maximum-Likelihood-Schätzung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15264-5">
    <skos:altLabel xml:lang="de">Maximum-Likelihood-Schätzfunktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15264-5">
    <skos:altLabel xml:lang="de">Größte-Dichte-Methode</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15760-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11401-6">
    <skos:related rdf:resource="descriptor/11382-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11401-6">
    <skos:related rdf:resource="descriptor/15656-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11401-6">
    <skos:related rdf:resource="descriptor/18292-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11401-6">
    <skos:related rdf:resource="descriptor/18868-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11401-6">
    <skos:related rdf:resource="descriptor/18857-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13837-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12732-6">
    <skos:broader rdf:resource="thsys/71036"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12732-6">
    <skos:broader rdf:resource="thsys/70471"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12732-6">
    <skos:broader rdf:resource="descriptor/12733-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12732-6">
    <skos:broader rdf:resource="descriptor/12730-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11934-6">
    <skos:prefLabel xml:lang="en">Transport costs</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11934-6">
    <skos:prefLabel xml:lang="de">Transportkosten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11872-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19674-1">
    <skos:broader rdf:resource="thsys/71028"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19674-1">
    <skos:broader rdf:resource="thsys/70960"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19674-1">
    <skos:broader rdf:resource="descriptor/10213-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18978-0">
    <skos:prefLabel xml:lang="en">Women managers</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18978-0">
    <skos:prefLabel xml:lang="de">Weibliche Führungskräfte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14310-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12821-0">
    <skos:altLabel xml:lang="en">Shop-in-the-shop</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12821-0">
    <skos:altLabel xml:lang="en">Shop-in-shop</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12821-0">
    <skos:altLabel xml:lang="en">Instore-marketing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12821-0">
    <skos:altLabel xml:lang="de">Warenpräsentation im Handel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12821-0">
    <skos:altLabel xml:lang="de">Verkaufsfläche</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12821-0">
    <skos:altLabel xml:lang="de">Regalpflege im Handel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12821-0">
    <skos:altLabel xml:lang="de">Ladengestaltung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19388-2">
    <skos:altLabel xml:lang="de">Mineralölverbrauch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19388-2">
    <skos:altLabel xml:lang="de">Mineralölkonsum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19388-2">
    <skos:altLabel xml:lang="de">Erdölverbrauch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14365-4">
    <skos:broader rdf:resource="thsys/70318"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14365-4">
    <skos:broader rdf:resource="descriptor/14366-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19731-1">
    <skos:scopeNote xml:lang="de">Philosophische Analyse von Begriff und Wesen der Religion.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10115-5">
    <skos:prefLabel xml:lang="en">Capital stock</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10115-5">
    <skos:prefLabel xml:lang="de">Gesamtwirtschaftliches Anlagevermögen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19307-0">
    <skos:related rdf:resource="descriptor/18316-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19307-0">
    <skos:related rdf:resource="descriptor/14084-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19307-0">
    <skos:related rdf:resource="descriptor/19306-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19307-0">
    <skos:related rdf:resource="descriptor/13165-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19475-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11353-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70550">
    <skos:narrower rdf:resource="descriptor/18869-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70550">
    <skos:narrower rdf:resource="descriptor/14051-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70550">
    <skos:narrower rdf:resource="descriptor/13474-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70550">
    <skos:narrower rdf:resource="descriptor/20306-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70550">
    <skos:narrower rdf:resource="descriptor/12809-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70550">
    <skos:narrower rdf:resource="descriptor/12808-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70550">
    <skos:narrower rdf:resource="descriptor/18066-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70550">
    <skos:narrower rdf:resource="descriptor/12807-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70550">
    <skos:narrower rdf:resource="descriptor/12759-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70550">
    <skos:narrower rdf:resource="descriptor/20314-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70550">
    <skos:narrower rdf:resource="descriptor/12806-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70550">
    <skos:narrower rdf:resource="descriptor/16034-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70550">
    <skos:narrower rdf:resource="descriptor/12810-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70550">
    <skos:narrower rdf:resource="descriptor/12805-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70550">
    <skos:narrower rdf:resource="descriptor/10016-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70550">
    <skos:narrower rdf:resource="descriptor/12801-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70550">
    <skos:narrower rdf:resource="descriptor/16078-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70550">
    <skos:narrower rdf:resource="descriptor/19129-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70550">
    <skos:narrower rdf:resource="descriptor/12800-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70550">
    <skos:narrower rdf:resource="descriptor/10459-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70550">
    <skos:narrower rdf:resource="descriptor/12798-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70550">
    <skos:narrower rdf:resource="descriptor/19128-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70550">
    <skos:narrower rdf:resource="descriptor/10546-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70550">
    <skos:narrower rdf:resource="descriptor/10443-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18787-0">
    <skos:related rdf:resource="descriptor/13376-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18787-0">
    <skos:related rdf:resource="descriptor/13957-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10638-1">
    <skos:related rdf:resource="descriptor/10668-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10638-1">
    <skos:related rdf:resource="descriptor/10615-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10638-1">
    <skos:related rdf:resource="descriptor/10637-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10638-1">
    <skos:related rdf:resource="descriptor/10186-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18916-1">
    <skos:narrower rdf:resource="descriptor/13645-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18916-1">
    <skos:narrower rdf:resource="descriptor/18915-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10034-5">
    <skos:prefLabel xml:lang="en">Adaptive expectation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10034-5">
    <skos:prefLabel xml:lang="de">Adaptive Erwartung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19283-2">
    <skos:prefLabel xml:lang="en">Institutional competition</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19283-2">
    <skos:prefLabel xml:lang="de">Institutioneller Wettbewerb</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10097-2">
    <skos:broader rdf:resource="thsys/70209"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10097-2">
    <skos:broader rdf:resource="thsys/70057"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10097-2">
    <skos:broader rdf:resource="descriptor/10113-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70415">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19733-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10156-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13330-0">
    <skos:prefLabel xml:lang="en">House-cleaning service</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13330-0">
    <skos:prefLabel xml:lang="de">Gebäudereinigung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17596-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18920-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10392-0">
    <skos:prefLabel xml:lang="en">Wage control</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10392-0">
    <skos:prefLabel xml:lang="de">Lohnkontrolle</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16120-3">
    <skos:broader rdf:resource="thsys/73390"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16120-3">
    <skos:broader rdf:resource="thsys/71041"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16120-3">
    <skos:broader rdf:resource="thsys/70876"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16120-3">
    <skos:broader rdf:resource="descriptor/19818-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14063-6">
    <skos:broader rdf:resource="thsys/73304"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14063-6">
    <skos:broader rdf:resource="thsys/70341"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14063-6">
    <skos:broader rdf:resource="descriptor/14074-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14063-6">
    <skos:broader rdf:resource="descriptor/14851-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19620-3">
    <skos:related rdf:resource="descriptor/15903-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19620-3">
    <skos:related rdf:resource="descriptor/15894-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19620-3">
    <skos:related rdf:resource="descriptor/15893-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18695-5">
    <skos:prefLabel xml:lang="en">IMF structural adjustment programme</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18695-5">
    <skos:prefLabel xml:lang="de">Anpassungsprogramm des IWF</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11479-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11253-2">
    <skos:prefLabel xml:lang="en">Flexible retirement</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11253-2">
    <skos:prefLabel xml:lang="de">Flexible Altersgrenze</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73379">
    <skos:prefLabel xml:lang="de">N.04.02.03  Parlament und Parlamentarismus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17964-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15320-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18168-0">
    <skos:broader rdf:resource="thsys/70051"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18168-0">
    <skos:broader rdf:resource="descriptor/10031-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10555-5">
    <skos:editorialNote xml:lang="de">Möglichst spezifischer indexieren.</skos:editorialNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11992-6">
    <skos:broader rdf:resource="thsys/73355"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11992-6">
    <skos:broader rdf:resource="thsys/71064"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11992-6">
    <skos:broader rdf:resource="descriptor/11986-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11992-6">
    <skos:broader rdf:resource="descriptor/16564-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19435-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17589-2">
    <skos:prefLabel xml:lang="en">Southeast Asia</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17589-2">
    <skos:prefLabel xml:lang="de">Südostasien</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13819-3">
    <skos:related rdf:resource="descriptor/13823-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13819-3">
    <skos:related rdf:resource="descriptor/18743-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15823-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13350-1">
    <skos:broader rdf:resource="thsys/70822"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13350-1">
    <skos:broader rdf:resource="descriptor/13351-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12368-1">
    <skos:related rdf:resource="descriptor/12366-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13237-4">
    <skos:broader rdf:resource="thsys/70960"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13237-4">
    <skos:broader rdf:resource="descriptor/13219-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14596-6">
    <skos:altLabel xml:lang="de">Landwirtschaftschemikalie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14596-6">
    <skos:altLabel xml:lang="de">Bodenverbesserer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14596-6">
    <skos:altLabel xml:lang="de">Agrarchemikalie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18928-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12605-6">
    <skos:broader rdf:resource="thsys/71050"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12605-6">
    <skos:broader rdf:resource="thsys/71033"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12605-6">
    <skos:broader rdf:resource="thsys/70239"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12605-6">
    <skos:broader rdf:resource="descriptor/15737-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10315-4">
    <skos:broader rdf:resource="thsys/70233"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10315-4">
    <skos:broader rdf:resource="descriptor/10327-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19829-5">
    <zbwext:useInsteadNote rdf:nodeID="id367"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11875-3">
    <skos:broader rdf:resource="thsys/71068"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11897-0">
    <skos:altLabel xml:lang="en">Regional cluster</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11897-0">
    <skos:altLabel xml:lang="en">Industrial cluster</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11897-0">
    <skos:altLabel xml:lang="de">Regionaler Verdichtungsprozess</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11897-0">
    <skos:altLabel xml:lang="de">Regionaler Verdichtungsprozeß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11897-0">
    <skos:altLabel xml:lang="de">Regionale Zentralisation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11897-0">
    <skos:altLabel xml:lang="de">Räumlicher Verdichtungsprozess</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11897-0">
    <skos:altLabel xml:lang="de">Räumlicher Verdichtungsprozeß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11897-0">
    <skos:altLabel xml:lang="de">Räumliche Konzentration</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11897-0">
    <skos:altLabel xml:lang="de">Agglomerationsprozess</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11897-0">
    <skos:altLabel xml:lang="de">Agglomerationsprozeß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19197-2">
    <skos:altLabel xml:lang="de">Realeinkommenseffekt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19197-2">
    <skos:altLabel xml:lang="de">Kaufkrafteffekt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13173-4">
    <skos:broader rdf:resource="thsys/70969"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13173-4">
    <skos:broader rdf:resource="descriptor/12905-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12985-5">
    <skos:altLabel xml:lang="de">Metallerzbergbau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13416-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70341">
    <skos:narrower rdf:resource="descriptor/14084-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70341">
    <skos:narrower rdf:resource="descriptor/14083-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70341">
    <skos:narrower rdf:resource="descriptor/14082-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70341">
    <skos:narrower rdf:resource="descriptor/14079-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70341">
    <skos:narrower rdf:resource="descriptor/14078-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70341">
    <skos:narrower rdf:resource="descriptor/14990-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70341">
    <skos:narrower rdf:resource="descriptor/14074-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70341">
    <skos:narrower rdf:resource="descriptor/14073-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70341">
    <skos:narrower rdf:resource="descriptor/14070-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70341">
    <skos:narrower rdf:resource="descriptor/15741-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70341">
    <skos:narrower rdf:resource="descriptor/14094-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70341">
    <skos:narrower rdf:resource="descriptor/14068-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70341">
    <skos:narrower rdf:resource="descriptor/18199-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70341">
    <skos:narrower rdf:resource="descriptor/14092-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70341">
    <skos:narrower rdf:resource="descriptor/14065-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70341">
    <skos:narrower rdf:resource="descriptor/14064-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70341">
    <skos:narrower rdf:resource="descriptor/14063-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70341">
    <skos:narrower rdf:resource="descriptor/14060-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70341">
    <skos:narrower rdf:resource="descriptor/14057-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70341">
    <skos:narrower rdf:resource="descriptor/14055-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70341">
    <skos:narrower rdf:resource="thsys/70070"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70341">
    <skos:narrower rdf:resource="thsys/70169"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19205-1">
    <skos:related rdf:resource="descriptor/11284-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19205-1">
    <skos:related rdf:resource="descriptor/10176-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19205-1">
    <skos:related rdf:resource="descriptor/10173-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15699-5">
    <skos:broader rdf:resource="thsys/70007"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15699-5">
    <skos:broader rdf:resource="descriptor/15690-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15651-5">
    <skos:broader rdf:resource="thsys/73343"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15651-5">
    <skos:broader rdf:resource="descriptor/15677-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13831-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13462-4">
    <skos:altLabel xml:lang="de">Binnenhandelsgeschichte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14518-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18194-6">
    <skos:related rdf:resource="descriptor/19296-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18194-6">
    <skos:related rdf:resource="descriptor/18193-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18194-6">
    <skos:related rdf:resource="descriptor/14065-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14428-6">
    <skos:broader rdf:resource="thsys/70258"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14428-6">
    <skos:broader rdf:resource="descriptor/13513-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12610-6">
    <skos:broader rdf:resource="thsys/71050"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12610-6">
    <skos:broader rdf:resource="thsys/70838"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12610-6">
    <skos:broader rdf:resource="descriptor/15679-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10005-5">
    <skos:broader rdf:resource="thsys/70039"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18677-0">
    <skos:altLabel xml:lang="de">Insiderregeln</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12613-0">
    <skos:altLabel xml:lang="de">Sicherheitssystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15909-6">
    <skos:altLabel xml:lang="de">Einwohnerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15909-6">
    <skos:altLabel xml:lang="de">Einwohner</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15969-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10715-2">
    <skos:related rdf:resource="descriptor/10710-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19340-2">
    <skos:prefLabel xml:lang="en">Meat market</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19340-2">
    <skos:prefLabel xml:lang="de">Fleischmarkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10452-1">
    <skos:related rdf:resource="descriptor/19712-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19730-3">
    <skos:broader rdf:resource="thsys/73394"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19730-3">
    <skos:broader rdf:resource="descriptor/16708-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19599-3">
    <skos:altLabel xml:lang="en">Emotionality</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19599-3">
    <skos:altLabel xml:lang="en">Emotion theory</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19599-3">
    <skos:altLabel xml:lang="de">Liebe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19599-3">
    <skos:altLabel xml:lang="de">Gefühl</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19599-3">
    <skos:altLabel xml:lang="de">Emotionstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19599-3">
    <skos:altLabel xml:lang="de">Emotionalität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19599-3">
    <skos:altLabel xml:lang="de">Emotionale Wirkung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17740-3">
    <skos:altLabel xml:lang="de">Republik Tschad</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10073-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11551-1">
    <skos:prefLabel xml:lang="en">Compliance costs</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11551-1">
    <skos:prefLabel xml:lang="de">Folgekosten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16458-1">
    <skos:related rdf:resource="descriptor/19725-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16458-1">
    <skos:related rdf:resource="descriptor/19720-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16458-1">
    <skos:related rdf:resource="descriptor/16271-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16458-1">
    <skos:related rdf:resource="descriptor/19709-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14236-1">
    <skos:broader rdf:resource="thsys/70348"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14236-1">
    <skos:broader rdf:resource="descriptor/14047-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71097">
    <rdfs:label xml:lang="de">Internationale Wirtschaftspolitik</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15723-6">
    <skos:prefLabel xml:lang="en">Social history</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15723-6">
    <skos:prefLabel xml:lang="de">Sozialgeschichte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16090-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17553-2">
    <skos:broader rdf:resource="thsys/70148"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11572-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10297-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14636-6">
    <skos:altLabel xml:lang="de">Ammoniumhydroxid</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13003-1">
    <skos:broader rdf:resource="thsys/71013"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13003-1">
    <skos:broader rdf:resource="thsys/70227"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13003-1">
    <skos:broader rdf:resource="descriptor/12993-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10671-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#member">
    <rdf:type rdf:resource="http://www.w3.org/1999/02/22-rdf-syntax-ns#Property"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#member">
    <rdf:type rdf:resource="http://www.w3.org/2002/07/owl#ObjectProperty"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11778-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10278-5">
    <skos:related rdf:resource="descriptor/10284-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10278-5">
    <skos:related rdf:resource="descriptor/10182-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10278-5">
    <skos:related rdf:resource="descriptor/10276-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10278-5">
    <skos:related rdf:resource="descriptor/10237-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18481-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12837-6">
    <skos:prefLabel xml:lang="en">Advertising effectiveness</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12837-6">
    <skos:prefLabel xml:lang="de">Werbewirkung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13194-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14614-2">
    <skos:altLabel xml:lang="de">Mikrofilm</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14614-2">
    <skos:altLabel xml:lang="de">Mikrofiche</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10491-5">
    <skos:related rdf:resource="descriptor/11224-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10491-5">
    <skos:related rdf:resource="descriptor/15951-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10491-5">
    <skos:related rdf:resource="descriptor/11356-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12256-5">
    <skos:broader rdf:resource="thsys/71105"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12256-5">
    <skos:broader rdf:resource="thsys/71045"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12256-5">
    <skos:broader rdf:resource="descriptor/12234-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19112-1">
    <skos:related rdf:resource="descriptor/11224-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19112-1">
    <skos:related rdf:resource="descriptor/19916-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19112-1">
    <skos:related rdf:resource="descriptor/11295-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19112-1">
    <skos:related rdf:resource="descriptor/11293-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19112-1">
    <skos:related rdf:resource="descriptor/11361-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19112-1">
    <skos:related rdf:resource="descriptor/11290-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19112-1">
    <skos:related rdf:resource="descriptor/19767-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19112-1">
    <skos:related rdf:resource="descriptor/11303-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16780-3">
    <skos:prefLabel xml:lang="en">Cataloguing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16780-3">
    <skos:prefLabel xml:lang="de">Katalogisierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11944-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12718-0">
    <skos:related rdf:resource="descriptor/12722-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14586-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12796-1">
    <skos:altLabel xml:lang="en">Transfer price</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12796-1">
    <skos:altLabel xml:lang="de">Verrechnungspreis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12796-1">
    <skos:altLabel xml:lang="de">Schattenpreis des Betriebs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12796-1">
    <skos:altLabel xml:lang="de">Konzernverrechnungspreis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12796-1">
    <skos:altLabel xml:lang="de">Arms-length Preis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18736-3">
    <skos:narrower rdf:resource="descriptor/13820-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12226-0">
    <skos:broader rdf:resource="thsys/71043"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12226-0">
    <skos:broader rdf:resource="thsys/70037"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10754-6">
    <skos:prefLabel xml:lang="en">Reserve currency</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10754-6">
    <skos:prefLabel xml:lang="de">Reservewährung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13443-1">
    <skos:related rdf:resource="descriptor/12032-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13443-1">
    <skos:related rdf:resource="descriptor/10503-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13443-1">
    <skos:related rdf:resource="descriptor/13451-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13443-1">
    <skos:related rdf:resource="descriptor/11803-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13443-1">
    <skos:related rdf:resource="descriptor/18318-0"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id140">
    <rdf:value xml:lang="de">Für die inhaltliche Erschließung von Dokumenten</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16041-6">
    <skos:related rdf:resource="descriptor/19852-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16041-6">
    <skos:related rdf:resource="descriptor/16042-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#notation">
    <rdfs:label xml:lang="en">notation</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71051">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19732-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18811-1">
    <skos:broader rdf:resource="thsys/70986"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18811-1">
    <skos:broader rdf:resource="descriptor/10978-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18811-1">
    <skos:broader rdf:resource="descriptor/18334-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18073-4">
    <skos:prefLabel xml:lang="en">Alternative economics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18073-4">
    <skos:prefLabel xml:lang="de">Alternative Ökonomie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16076-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15682-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10221-6">
    <skos:prefLabel xml:lang="en">Futures contract</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10221-6">
    <skos:prefLabel xml:lang="de">Termingeschäft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71091">
    <rdfs:label xml:lang="en">Foreign trade</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71091">
    <rdfs:label xml:lang="de">Außenhandel</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11666-5">
    <skos:altLabel xml:lang="en">Corporation tax</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11666-5">
    <skos:altLabel xml:lang="de">Körperschaftssteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19308-5">
    <skos:broader rdf:resource="thsys/71090"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19308-5">
    <skos:broader rdf:resource="descriptor/10847-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19308-5">
    <skos:broader rdf:resource="descriptor/10614-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19308-5">
    <skos:broader rdf:resource="descriptor/10822-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17016-0">
    <skos:narrower rdf:resource="descriptor/17017-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17016-0">
    <skos:narrower rdf:resource="descriptor/17161-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17016-0">
    <skos:narrower rdf:resource="descriptor/17015-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17016-0">
    <skos:narrower rdf:resource="descriptor/17104-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17016-0">
    <skos:narrower rdf:resource="descriptor/17014-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17016-0">
    <skos:narrower rdf:resource="descriptor/17013-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17016-0">
    <skos:narrower rdf:resource="descriptor/17039-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17016-0">
    <skos:narrower rdf:resource="descriptor/17011-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10054-6">
    <skos:related rdf:resource="descriptor/10247-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11558-1">
    <skos:broader rdf:resource="thsys/71081"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11558-1">
    <skos:broader rdf:resource="thsys/70833"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11558-1">
    <skos:broader rdf:resource="thsys/70324"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11558-1">
    <skos:broader rdf:resource="descriptor/11554-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12058-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id289">
    <rdf:value xml:lang="de">für handelsrechtliche Bestimmungen</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18926-5">
    <skos:prefLabel xml:lang="en">Art trade</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18926-5">
    <skos:prefLabel xml:lang="de">Kunsthandel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17950-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19809-4">
    <skos:altLabel xml:lang="de">Virtuelle Welten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19809-4">
    <skos:altLabel xml:lang="de">Virtuelle Umgebung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19809-4">
    <skos:altLabel xml:lang="de">Cyberspace</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10656-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18306-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73380">
    <skos:prefLabel xml:lang="de">N.04.02.02  Regierung und öffentliche Verwaltung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19321-6">
    <skos:broader rdf:resource="thsys/71098"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19321-6">
    <skos:broader rdf:resource="thsys/71089"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19321-6">
    <skos:broader rdf:resource="thsys/70187"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19321-6">
    <skos:broader rdf:resource="descriptor/18011-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13298-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13811-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10911-2">
    <skos:editorialNote xml:lang="de">Möglichst spezifischer indexieren.</skos:editorialNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10210-4">
    <skos:broader rdf:resource="thsys/71104"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10210-4">
    <skos:broader rdf:resource="thsys/70897"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10210-4">
    <skos:broader rdf:resource="descriptor/12212-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18484-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19728-4">
    <skos:related rdf:resource="descriptor/19816-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19728-4">
    <skos:related rdf:resource="descriptor/16328-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19728-4">
    <skos:related rdf:resource="descriptor/19662-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19728-4">
    <skos:related rdf:resource="descriptor/19853-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18941-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16212-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13691-3">
    <skos:broader rdf:resource="thsys/73387"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13691-3">
    <skos:broader rdf:resource="thsys/70905"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13691-3">
    <skos:broader rdf:resource="thsys/70109"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13691-3">
    <skos:broader rdf:resource="descriptor/15706-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13327-3">
    <skos:narrower rdf:resource="descriptor/13331-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16759-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10725-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11993-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id25">
    <rdf:value xml:lang="de">Für allgemeine arbeitsrechtliche Schutzbestimmungen für Arbeitskräfte</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19422-0">
    <skos:altLabel xml:lang="en">Specific commodity tax</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19422-0">
    <skos:altLabel xml:lang="de">Zuckersteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19422-0">
    <skos:altLabel xml:lang="de">Teesteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19422-0">
    <skos:altLabel xml:lang="de">Salzsteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19422-0">
    <skos:altLabel xml:lang="de">Leuchtmittelsteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19422-0">
    <skos:altLabel xml:lang="de">Kaffeesteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19422-0">
    <skos:altLabel xml:lang="de">Jagdsteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19422-0">
    <skos:altLabel xml:lang="de">Jagd- und Fischereisteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19422-0">
    <skos:altLabel xml:lang="de">Hundesteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18808-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10504-1">
    <skos:altLabel xml:lang="de">Land-Stadt-Wanderung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10037-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16668-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16683-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70962">
    <skos:prefLabel xml:lang="de">W.10.01  Handelssparten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15972-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16038-2">
    <skos:broader rdf:resource="thsys/70853"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16038-2">
    <skos:broader rdf:resource="thsys/70241"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16038-2">
    <skos:broader rdf:resource="descriptor/15966-1"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id200">
    <rdf:value xml:lang="de">Für die betriebliche Lohnpolitik</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71002">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17679-1">
    <skos:altLabel xml:lang="de">Republik Uganda</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13149-1">
    <skos:altLabel xml:lang="de">Wollweberei</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13149-1">
    <skos:altLabel xml:lang="de">Wollwarenindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13149-1">
    <skos:altLabel xml:lang="de">Wolltextilienindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13149-1">
    <skos:altLabel xml:lang="de">Wollspinnerei</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13149-1">
    <skos:altLabel xml:lang="de">Wollgarnindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16687-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18241-2">
    <skos:broader rdf:resource="thsys/70934"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18241-2">
    <skos:broader rdf:resource="thsys/70933"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18241-2">
    <skos:broader rdf:resource="descriptor/13533-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12712-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10030-6">
    <zbwext:useInsteadNote rdf:nodeID="id127"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16182-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13225-4">
    <skos:altLabel xml:lang="de">Obsthandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13225-4">
    <skos:altLabel xml:lang="de">Gemüsegroßhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13225-4">
    <skos:altLabel xml:lang="de">Gemüseeinzelhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13225-4">
    <skos:altLabel xml:lang="de">Fruchtgroßhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13225-4">
    <skos:altLabel xml:lang="de">Fruchteinzelhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16067-2">
    <skos:broader rdf:resource="thsys/73391"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16067-2">
    <skos:broader rdf:resource="thsys/70383"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16067-2">
    <skos:broader rdf:resource="thsys/70186"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16067-2">
    <skos:broader rdf:resource="descriptor/19845-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71036">
    <rdfs:label xml:lang="de">Betriebliche Anwendungssysteme</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13469-4">
    <skos:broader rdf:resource="thsys/70956"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13469-4">
    <skos:broader rdf:resource="descriptor/12036-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15655-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70071">
    <skos:broader rdf:resource="thsys/p"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13191-2">
    <skos:broader rdf:resource="thsys/70957"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13191-2">
    <skos:broader rdf:resource="thsys/70816"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13191-2">
    <skos:broader rdf:resource="descriptor/13324-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13508-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14046-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14829-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12064-0">
    <skos:broader rdf:resource="thsys/70575"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12064-0">
    <skos:broader rdf:resource="thsys/70039"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12064-0">
    <skos:broader rdf:resource="descriptor/11068-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14749-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10590-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18109-2">
    <skos:prefLabel xml:lang="en">European Company (SE)</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18109-2">
    <skos:prefLabel xml:lang="de">Europäische Aktiengesellschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18428-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14171-3">
    <skos:related rdf:resource="descriptor/19020-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14171-3">
    <skos:related rdf:resource="descriptor/15741-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14171-3">
    <skos:related rdf:resource="descriptor/14603-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14171-3">
    <skos:related rdf:resource="descriptor/13450-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12314-3">
    <skos:related rdf:resource="descriptor/19655-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12314-3">
    <skos:related rdf:resource="descriptor/12394-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12314-3">
    <skos:related rdf:resource="descriptor/12562-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12314-3">
    <skos:related rdf:resource="descriptor/19672-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12314-3">
    <skos:related rdf:resource="descriptor/11224-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12314-3">
    <skos:related rdf:resource="descriptor/12356-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15869-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14087-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19640-4">
    <skos:prefLabel xml:lang="en">Illegal immigration</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19640-4">
    <skos:prefLabel xml:lang="de">Illegale Einwanderung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10991-6">
    <skos:prefLabel xml:lang="en">Economic policy analysis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10991-6">
    <skos:prefLabel xml:lang="de">Wirtschaftspolitische Wirkungsanalyse</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19544-0">
    <skos:altLabel xml:lang="en">Electronic market</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19544-0">
    <skos:altLabel xml:lang="en">Electronic commerce</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19544-0">
    <skos:altLabel xml:lang="en">Electronic business</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19544-0">
    <skos:altLabel xml:lang="de">Elektronischer Marktplatz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19544-0">
    <skos:altLabel xml:lang="de">Elektronischer Markt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19544-0">
    <skos:altLabel xml:lang="de">Elektronischer Handel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19544-0">
    <skos:altLabel xml:lang="de">E-Commerce</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11371-3">
    <skos:broader rdf:resource="thsys/70847"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11371-3">
    <skos:broader rdf:resource="descriptor/11382-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12945-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70876">
    <skos:broader rdf:resource="thsys/w"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16663-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70991">
    <skos:broader rdf:resource="thsys/71004"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id106">
    <rdf:value xml:lang="de">Für mikroökonomische Analysen zu Dauer und Intensität der Arbeitsuche</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10512-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13290-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12105-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11971-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19672-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14383-2">
    <skos:prefLabel xml:lang="en">Sewing machinery</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14383-2">
    <skos:prefLabel xml:lang="de">Nähmaschine</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11525-2">
    <skos:prefLabel xml:lang="en">Financial law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11525-2">
    <skos:prefLabel xml:lang="de">Finanzrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17980-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10487-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13527-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70986">
    <rdfs:label xml:lang="de">Nahrungs- und Genussmittelgewerbe</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70180">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13807-3">
    <skos:narrower rdf:resource="descriptor/10785-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13807-3">
    <skos:narrower rdf:resource="descriptor/13793-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11354-3">
    <skos:broader rdf:resource="thsys/70847"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11354-3">
    <skos:broader rdf:resource="thsys/70176"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11354-3">
    <skos:broader rdf:resource="thsys/70143"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13283-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14973-2">
    <skos:prefLabel xml:lang="en">Beer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14973-2">
    <skos:prefLabel xml:lang="de">Bier</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11786-2">
    <skos:related rdf:resource="descriptor/10913-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11786-2">
    <skos:related rdf:resource="descriptor/10243-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11786-2">
    <skos:related rdf:resource="descriptor/11906-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18243-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19773-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18418-3">
    <skos:scopeNote xml:lang="de">Forschung im Bereich Flugzeugbau und Raumfahrzeugbau</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11361-6">
    <skos:broader rdf:resource="thsys/71058"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11361-6">
    <skos:broader rdf:resource="thsys/70843"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11361-6">
    <skos:broader rdf:resource="descriptor/11360-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18672-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11213-0">
    <skos:altLabel xml:lang="en">Temporary lay-off</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11213-0">
    <skos:altLabel xml:lang="en">Search unemployment</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11213-0">
    <skos:altLabel xml:lang="de">Temporäre Arbeitslosigkeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11213-0">
    <skos:altLabel xml:lang="de">Sucharbeitslosigkeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19526-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14734-6">
    <skos:altLabel xml:lang="de">Puder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18201-0">
    <skos:prefLabel xml:lang="en">Softwood</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18201-0">
    <skos:prefLabel xml:lang="de">Weichholz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11228-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18813-4">
    <skos:prefLabel xml:lang="en">Private investment in public infrastructure</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18813-4">
    <skos:prefLabel xml:lang="de">Private Infrastrukturinvestition</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13874-2">
    <skos:prefLabel xml:lang="en">Forestry workers</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13874-2">
    <skos:prefLabel xml:lang="de">Forstberufe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17739-2">
    <skos:altLabel xml:lang="de">Demokratische Republik Sao Tome und Principe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14110-2">
    <skos:prefLabel xml:lang="en">Fruit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14110-2">
    <skos:prefLabel xml:lang="de">Obst</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19767-1">
    <skos:altLabel xml:lang="de">Empirische Arbeitsmarktforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19767-1">
    <skos:altLabel xml:lang="de">Betriebliche Arbeitsmarktforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19404-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18850-5">
    <skos:broader rdf:resource="thsys/73378"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18850-5">
    <skos:broader rdf:resource="thsys/70852"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18850-5">
    <skos:broader rdf:resource="descriptor/10984-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18435-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19368-1">
    <skos:broader rdf:resource="thsys/71085"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19368-1">
    <skos:broader rdf:resource="thsys/71065"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19368-1">
    <skos:broader rdf:resource="descriptor/11900-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19368-1">
    <skos:broader rdf:resource="descriptor/10523-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11913-0">
    <skos:altLabel xml:lang="de">Wirtschaftsstandort</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18309-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19450-2">
    <skos:related rdf:resource="descriptor/11998-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19075-2">
    <skos:prefLabel xml:lang="en">Northern Germany</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19075-2">
    <skos:prefLabel xml:lang="de">Norddeutschland</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13004-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11613-5">
    <skos:related rdf:resource="descriptor/19379-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11613-5">
    <skos:related rdf:resource="descriptor/11545-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11613-5">
    <skos:related rdf:resource="descriptor/19376-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11613-5">
    <skos:related rdf:resource="descriptor/11731-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11613-5">
    <skos:related rdf:resource="descriptor/11586-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11613-5">
    <skos:related rdf:resource="descriptor/19374-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11613-5">
    <skos:related rdf:resource="descriptor/11607-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11613-5">
    <skos:related rdf:resource="descriptor/11734-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18876-1">
    <skos:related rdf:resource="descriptor/11402-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18876-1">
    <skos:related rdf:resource="descriptor/11373-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19851-5">
    <skos:prefLabel xml:lang="en">International jurisdiction</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19851-5">
    <skos:prefLabel xml:lang="de">Internationale Gerichtsbarkeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19503-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19623-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12121-0">
    <skos:altLabel xml:lang="de">Organisationsentwicklungsberatung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12121-0">
    <skos:altLabel xml:lang="de">OE-Beratung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70147">
    <skos:broader rdf:resource="thsys/70148"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14268-2">
    <skos:prefLabel xml:lang="en">Metal casting</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14268-2">
    <skos:prefLabel xml:lang="de">Metallguss</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11260-5">
    <skos:scopeNote xml:lang="de">Empirische Untersuchungen über die tatsächliche Zeitverwendung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18657-6">
    <skos:altLabel xml:lang="en">OSI (Open Systems Interconnection)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18657-6">
    <skos:altLabel xml:lang="de">Offenes Telekommunikationsnetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18657-6">
    <skos:altLabel xml:lang="de">Offenes EDV-System</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13312-2">
    <skos:broader rdf:resource="thsys/70834"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13312-2">
    <skos:broader rdf:resource="descriptor/18906-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18730-1">
    <skos:altLabel xml:lang="en">Crash</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18730-1">
    <skos:altLabel xml:lang="de">Crash an der Börse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18730-1">
    <skos:altLabel xml:lang="de">Börsenschock</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18730-1">
    <skos:altLabel xml:lang="de">Börsenkrach</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18730-1">
    <skos:altLabel xml:lang="de">Börsen-Crash</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12849-6">
    <zbwext:useInsteadNote rdf:nodeID="id317"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19781-0">
    <zbwext:useInsteadNote rdf:nodeID="id181"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19781-0">
    <zbwext:useInsteadNote rdf:nodeID="id180"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14051-6">
    <skos:narrower rdf:resource="descriptor/18307-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14051-6">
    <skos:narrower rdf:resource="descriptor/14804-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14051-6">
    <skos:narrower rdf:resource="descriptor/14549-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14051-6">
    <skos:narrower rdf:resource="descriptor/18273-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14051-6">
    <skos:narrower rdf:resource="descriptor/18304-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14051-6">
    <skos:narrower rdf:resource="descriptor/14821-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14051-6">
    <skos:narrower rdf:resource="descriptor/14778-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14051-6">
    <skos:narrower rdf:resource="descriptor/14027-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12689-2">
    <skos:related rdf:resource="descriptor/13608-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12689-2">
    <skos:related rdf:resource="descriptor/15519-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12689-2">
    <skos:related rdf:resource="descriptor/18620-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12689-2">
    <skos:related rdf:resource="descriptor/13621-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12689-2">
    <skos:related rdf:resource="descriptor/13600-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12689-2">
    <skos:related rdf:resource="descriptor/12693-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19438-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16184-5">
    <skos:altLabel xml:lang="de">Internationales Übereinkommen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16184-5">
    <skos:altLabel xml:lang="de">Internationaler Vertrag</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16184-5">
    <skos:altLabel xml:lang="de">Internationaler Staatsvertrag</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16184-5">
    <skos:altLabel xml:lang="de">Internationale Vereinbarung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16184-5">
    <skos:altLabel xml:lang="de">Internationale Konvention</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16184-5">
    <skos:altLabel xml:lang="de">Diplomatisches Abkommen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21082-4">
    <skos:related rdf:resource="descriptor/10110-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21082-4">
    <skos:related rdf:resource="descriptor/11735-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14916-0">
    <skos:altLabel xml:lang="de">Rauchwaren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14916-0">
    <skos:altLabel xml:lang="de">Pelzbekleidung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14916-0">
    <skos:altLabel xml:lang="de">Pelz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18436-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14021-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16037-4">
    <skos:altLabel xml:lang="en">Copyright</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16037-4">
    <skos:altLabel xml:lang="de">Verlagsrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16037-4">
    <skos:altLabel xml:lang="de">Verlagsgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16037-4">
    <skos:altLabel xml:lang="de">Urhebervertrag</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16037-4">
    <skos:altLabel xml:lang="de">Urheberrechtsgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#related">
    <rdfs:subPropertyOf rdf:resource="http://www.w3.org/2004/02/skos/core#semanticRelation"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12520-0">
    <skos:altLabel xml:lang="de">Kostenstellenumlageverfahren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12520-0">
    <skos:altLabel xml:lang="de">Hauptkostenstellenverfahren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12196-4">
    <skos:prefLabel xml:lang="en">Company transformation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12196-4">
    <skos:prefLabel xml:lang="de">Umwandlung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19113-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10458-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71003">
    <skos:narrower rdf:resource="descriptor/13090-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71003">
    <skos:narrower rdf:resource="descriptor/18402-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71003">
    <skos:narrower rdf:resource="descriptor/18401-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71003">
    <skos:narrower rdf:resource="descriptor/13034-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71003">
    <skos:narrower rdf:resource="descriptor/18400-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71003">
    <skos:narrower rdf:resource="descriptor/18399-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71003">
    <skos:narrower rdf:resource="descriptor/13032-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71003">
    <skos:narrower rdf:resource="descriptor/13028-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71003">
    <skos:narrower rdf:resource="descriptor/13085-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71003">
    <skos:narrower rdf:resource="descriptor/13050-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71003">
    <skos:narrower rdf:resource="thsys/71001"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71003">
    <skos:narrower rdf:resource="thsys/71000"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18182-6">
    <skos:altLabel xml:lang="en">Peace dividend</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18182-6">
    <skos:altLabel xml:lang="de">Rüstungskonversionstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18182-6">
    <skos:altLabel xml:lang="de">Rüstungskonversionspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18182-6">
    <skos:altLabel xml:lang="de">Konversionsökonomie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18182-6">
    <skos:altLabel xml:lang="de">Friedensdividende</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12975-1">
    <skos:narrower rdf:resource="descriptor/18357-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12975-1">
    <skos:narrower rdf:resource="descriptor/12978-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12975-1">
    <skos:narrower rdf:resource="descriptor/12974-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11753-3">
    <skos:altLabel xml:lang="en">Measure of income inequality</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11753-3">
    <skos:altLabel xml:lang="de">Verteilungsmaß des Vermögens</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11753-3">
    <skos:altLabel xml:lang="de">Pareto-Alpha</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11753-3">
    <skos:altLabel xml:lang="de">Konzentrationsmaß des Einkommens</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11753-3">
    <skos:altLabel xml:lang="de">Einkommensverteilungsmaß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19975-1">
    <skos:prefLabel xml:lang="en">Clientelism</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19975-1">
    <skos:prefLabel xml:lang="de">Patronage</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15441-2">
    <skos:related rdf:resource="descriptor/10222-4"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id92">
    <rdfs:seeAlso rdf:resource="descriptor/11270-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15605-5">
    <skos:scopeNote xml:lang="de">Zustand, in dem Daten inhaltlich und formal unverändert bleiben und auch vor unberechtigtem Zugriff geschützt sind.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70201">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16481-6">
    <skos:scopeNote xml:lang="de">Rechtliche Maßnahmen zum besonderen Schutz von Kindern und Jugendlichen vor Alkohol, jugendgefährdenden Medien, Missbrauch, etc.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16788-1">
    <skos:broader rdf:resource="thsys/70823"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14181-0">
    <skos:broader rdf:resource="thsys/70650"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14181-0">
    <skos:broader rdf:resource="descriptor/14184-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14181-0">
    <skos:broader rdf:resource="descriptor/14173-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18076-5">
    <skos:altLabel xml:lang="en">Wold causality</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18076-5">
    <skos:altLabel xml:lang="en">Sims causality</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18076-5">
    <skos:altLabel xml:lang="en">Noncausality</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18076-5">
    <skos:altLabel xml:lang="en">Granger causality</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18076-5">
    <skos:altLabel xml:lang="en">Causal analysis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18076-5">
    <skos:altLabel xml:lang="de">Wiener-Granger causality</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10570-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14188-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17583-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10051-5">
    <skos:altLabel xml:lang="de">Offene-Volkswirtschaft-Modell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11071-1">
    <skos:narrower rdf:resource="descriptor/11092-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17672-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70485">
    <skos:prefLabel xml:lang="de">B.01.01  Unternehmensführung und Unternehmensplanung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19388-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14743-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11206-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11612-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14212-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18429-5">
    <skos:broader rdf:resource="thsys/73349"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18429-5">
    <skos:broader rdf:resource="thsys/70994"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18429-5">
    <skos:broader rdf:resource="descriptor/15677-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19465-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15925-1">
    <skos:broader rdf:resource="thsys/73357"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15925-1">
    <skos:broader rdf:resource="thsys/70142"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15925-1">
    <skos:broader rdf:resource="descriptor/15909-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18083-1">
    <skos:related rdf:resource="descriptor/13992-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18083-1">
    <skos:related rdf:resource="descriptor/12070-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18083-1">
    <skos:related rdf:resource="descriptor/20485-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18083-1">
    <skos:related rdf:resource="descriptor/19966-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18083-1">
    <skos:related rdf:resource="descriptor/11302-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18083-1">
    <skos:related rdf:resource="descriptor/13871-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18083-1">
    <skos:related rdf:resource="descriptor/12178-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25681-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15093-6">
    <skos:prefLabel xml:lang="en">Price index</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15093-6">
    <skos:prefLabel xml:lang="de">Preisindex</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18083-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19732-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11805-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14079-5">
    <skos:prefLabel xml:lang="en">Seed</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14079-5">
    <skos:prefLabel xml:lang="de">Saatgut</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14047-4">
    <skos:altLabel xml:lang="en">Raw material</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14047-4">
    <skos:altLabel xml:lang="en">Primary product</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14047-4">
    <skos:altLabel xml:lang="de">Primärgut</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14047-4">
    <skos:altLabel xml:lang="de">Grundstoff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13395-0">
    <skos:altLabel xml:lang="en">Nongovernmental organization</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13395-0">
    <skos:altLabel xml:lang="en">NGO (Nongovernmental Organization)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13395-0">
    <skos:altLabel xml:lang="de">NRO (Nichtregierungsorganisation)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13395-0">
    <skos:altLabel xml:lang="de">Nichtstaatliche Organisation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15861-1">
    <skos:prefLabel xml:lang="en">Behavioral economics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15861-1">
    <skos:prefLabel xml:lang="de">Verhaltensökonomik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13198-2">
    <skos:related rdf:resource="descriptor/15992-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13198-2">
    <skos:related rdf:resource="descriptor/14599-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13198-2">
    <skos:related rdf:resource="descriptor/12996-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14941-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14292-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#Collection">
    <skos:definition xml:lang="en">A meaningful collection of concepts.</skos:definition>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10436-6">
    <skos:altLabel xml:lang="en">Research and development</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10436-6">
    <skos:altLabel xml:lang="en">R&amp;amp;D</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10436-6">
    <skos:altLabel xml:lang="en">R and D</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10436-6">
    <skos:altLabel xml:lang="en">R &amp;amp; D</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10436-6">
    <skos:altLabel xml:lang="de">Forschung und Entwicklung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10436-6">
    <skos:altLabel xml:lang="de">F und E</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10436-6">
    <skos:altLabel xml:lang="de">F &amp;amp; E</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18508-2">
    <skos:broader rdf:resource="thsys/70970"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18508-2">
    <skos:broader rdf:resource="descriptor/11811-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18508-2">
    <skos:broader rdf:resource="descriptor/18507-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70843">
    <skos:broader rdf:resource="thsys/70847"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17602-1">
    <skos:broader rdf:resource="thsys/70561"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17602-1">
    <skos:broader rdf:resource="descriptor/19167-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12721-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16471-2">
    <skos:altLabel xml:lang="de">Militärische Intervention</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10291-6">
    <skos:related rdf:resource="descriptor/10286-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12273-5">
    <skos:broader rdf:resource="thsys/71046"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12273-5">
    <skos:broader rdf:resource="thsys/71043"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12273-5">
    <skos:broader rdf:resource="descriptor/10120-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16023-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11866-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14331-0">
    <skos:altLabel xml:lang="de">Raffinerieausrüstung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14331-0">
    <skos:altLabel xml:lang="de">Petrochemieausrüstung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14331-0">
    <skos:altLabel xml:lang="de">Chemische Produktionsanlage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14331-0">
    <skos:altLabel xml:lang="de">Chemieanlage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14331-0">
    <skos:altLabel xml:lang="de">Biotechnischer Apparat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13053-0">
    <skos:altLabel xml:lang="de">Kraftfahrzeugteileindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13053-0">
    <skos:altLabel xml:lang="de">Kfz-Zulieferer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13053-0">
    <skos:altLabel xml:lang="de">Automobilzulieferer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19007-5">
    <skos:scopeNote xml:lang="de">Anwendung des Marketinggedankens auf die Personalwirtschaft.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70174">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73358">
    <skos:narrower rdf:resource="descriptor/15810-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73358">
    <skos:narrower rdf:resource="descriptor/16664-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73358">
    <skos:narrower rdf:resource="descriptor/15807-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73358">
    <skos:narrower rdf:resource="descriptor/15806-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73358">
    <skos:narrower rdf:resource="descriptor/16641-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73358">
    <skos:narrower rdf:resource="descriptor/15714-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73358">
    <skos:narrower rdf:resource="descriptor/15929-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73358">
    <skos:narrower rdf:resource="descriptor/16663-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73358">
    <skos:narrower rdf:resource="descriptor/19758-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73358">
    <skos:narrower rdf:resource="descriptor/16661-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73358">
    <skos:narrower rdf:resource="descriptor/15927-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73358">
    <skos:narrower rdf:resource="descriptor/15835-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73358">
    <skos:narrower rdf:resource="descriptor/15472-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73358">
    <skos:narrower rdf:resource="descriptor/12119-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73358">
    <skos:narrower rdf:resource="descriptor/19757-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73358">
    <skos:narrower rdf:resource="descriptor/19756-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73358">
    <skos:narrower rdf:resource="descriptor/15945-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73358">
    <skos:narrower rdf:resource="descriptor/16678-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73358">
    <skos:narrower rdf:resource="descriptor/15920-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73358">
    <skos:narrower rdf:resource="descriptor/19642-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73358">
    <skos:narrower rdf:resource="descriptor/15918-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73358">
    <skos:narrower rdf:resource="descriptor/15932-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73358">
    <skos:narrower rdf:resource="descriptor/15931-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73358">
    <skos:narrower rdf:resource="descriptor/15907-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73358">
    <skos:narrower rdf:resource="descriptor/16400-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13198-2">
    <skos:broader rdf:resource="thsys/70957"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13198-2">
    <skos:broader rdf:resource="descriptor/13210-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13090-1">
    <skos:prefLabel xml:lang="en">Tool industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13090-1">
    <skos:prefLabel xml:lang="de">Werkzeugindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70970">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19180-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18929-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21408-0">
    <skos:scopeNote xml:lang="de">Coupons als Mittel der Verkaufsförderung können in unterschiedlicher Weise und zu verschiedenen Zwecken eingesetzt werden. Häufig werden sie Anzeigen beigefügt - als Postkarte oder zum Ausschneiden - und berechtigen zum preisreduzierten Bezug von Produkten/Dienstleistungen oder dienen der Anforderung von Prospekten/Katalogen. Es sind auch andere Formen möglich, z.B. gibt es auch schon E-Couponing.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73369">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11004-2">
    <skos:prefLabel xml:lang="en">Economic planning in market economies</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11004-2">
    <skos:prefLabel xml:lang="de">Gesamtwirtschaftliche Planung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17016-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11567-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11673-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11555-0">
    <skos:prefLabel xml:lang="en">Health care costs</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11555-0">
    <skos:prefLabel xml:lang="de">Gesundheitskosten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11987-6">
    <skos:broader rdf:resource="thsys/73354"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11987-6">
    <skos:broader rdf:resource="thsys/71064"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11987-6">
    <skos:broader rdf:resource="descriptor/11986-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11987-6">
    <skos:broader rdf:resource="descriptor/15872-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13809-6">
    <skos:prefLabel xml:lang="en">Fire insurance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13809-6">
    <skos:prefLabel xml:lang="de">Feuerversicherung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18208-0">
    <skos:altLabel xml:lang="de">Computerpresse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12675-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18731-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16410-1">
    <skos:prefLabel xml:lang="en">Conservatism</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16410-1">
    <skos:prefLabel xml:lang="de">Konservatismus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11141-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18505-1">
    <skos:related rdf:resource="descriptor/18750-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18505-1">
    <skos:related rdf:resource="descriptor/18506-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13057-6">
    <skos:broader rdf:resource="thsys/70998"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13057-6">
    <skos:broader rdf:resource="descriptor/13056-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14184-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11358-2">
    <skos:prefLabel xml:lang="en">Instructors</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11358-2">
    <skos:prefLabel xml:lang="de">Ausbilder</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18503-5">
    <skos:prefLabel xml:lang="en">Housing cooperative</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18503-5">
    <skos:prefLabel xml:lang="de">Wohnungsbaugenossenschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14573-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70348">
    <rdfs:label xml:lang="de">Steine, Erden, Keramik und Glas</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70274">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.01.04</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13681-6">
    <skos:related rdf:resource="descriptor/14483-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13647-6">
    <skos:prefLabel xml:lang="en">Postal law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13647-6">
    <skos:prefLabel xml:lang="de">Postrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19504-5">
    <skos:related rdf:resource="descriptor/12628-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10294-0">
    <skos:broader rdf:resource="thsys/70530"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10294-0">
    <skos:broader rdf:resource="descriptor/10249-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19354-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13853-3">
    <skos:related rdf:resource="descriptor/18745-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13853-3">
    <skos:related rdf:resource="descriptor/13852-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13853-3">
    <skos:related rdf:resource="descriptor/13851-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13853-3">
    <skos:related rdf:resource="descriptor/18504-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19676-4">
    <skos:related rdf:resource="descriptor/11555-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19676-4">
    <skos:related rdf:resource="descriptor/13370-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19227-5">
    <skos:altLabel xml:lang="de">Adverse Selektion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10377-3">
    <skos:broader rdf:resource="thsys/70038"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10377-3">
    <skos:broader rdf:resource="descriptor/10987-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70030">
    <rdfs:label xml:lang="en">Production management</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70030">
    <rdfs:label xml:lang="de">Produktionswirtschaft</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70873">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.21.03</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10016-0">
    <skos:altLabel xml:lang="de">Ware</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10016-0">
    <skos:altLabel xml:lang="de">Fertigware</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10016-0">
    <skos:altLabel xml:lang="de">Fertigerzeugnis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10016-0">
    <skos:altLabel xml:lang="de">Erzeugnis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10016-0">
    <skos:altLabel xml:lang="de">Endprodukt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18104-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10642-3">
    <skos:prefLabel xml:lang="en">Elasticity condition</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10642-3">
    <skos:prefLabel xml:lang="de">Elastizitätsbedingung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19108-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13808-1">
    <skos:prefLabel xml:lang="en">Corporate insurance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13808-1">
    <skos:prefLabel xml:lang="de">Unternehmensversicherung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14664-1">
    <skos:related rdf:resource="descriptor/14603-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70171">
    <skos:narrower rdf:resource="descriptor/16566-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70171">
    <skos:narrower rdf:resource="descriptor/11064-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70171">
    <skos:narrower rdf:resource="descriptor/19289-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70171">
    <skos:narrower rdf:resource="descriptor/10018-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70171">
    <skos:narrower rdf:resource="descriptor/19288-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70171">
    <skos:narrower rdf:resource="descriptor/11043-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70171">
    <skos:narrower rdf:resource="descriptor/11042-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70171">
    <skos:narrower rdf:resource="descriptor/19287-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70171">
    <skos:narrower rdf:resource="descriptor/19286-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70171">
    <skos:narrower rdf:resource="descriptor/11036-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70171">
    <skos:narrower rdf:resource="descriptor/18073-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70171">
    <skos:narrower rdf:resource="thsys/71102"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70171">
    <skos:narrower rdf:resource="thsys/71101"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13452-0">
    <skos:narrower rdf:resource="descriptor/18986-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13452-0">
    <skos:narrower rdf:resource="descriptor/18324-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20553-3">
    <skos:prefLabel xml:lang="en">Microfinance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20553-3">
    <skos:prefLabel xml:lang="de">Mikrofinanzierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15965-3">
    <skos:broader rdf:resource="thsys/73394"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15965-3">
    <skos:broader rdf:resource="thsys/70154"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15965-3">
    <skos:broader rdf:resource="descriptor/15969-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15965-3">
    <skos:broader rdf:resource="descriptor/15966-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14949-6">
    <skos:altLabel xml:lang="de">Traubensaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14949-6">
    <skos:altLabel xml:lang="de">Süßmost</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14949-6">
    <skos:altLabel xml:lang="de">Orangensaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14949-6">
    <skos:altLabel xml:lang="de">Obstsaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14949-6">
    <skos:altLabel xml:lang="de">Gemüsesaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14949-6">
    <skos:altLabel xml:lang="de">Fruchtsaftkonzentrat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14949-6">
    <skos:altLabel xml:lang="de">Fruchtnektar</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14949-6">
    <skos:altLabel xml:lang="de">Apfelsaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15672-4">
    <skos:prefLabel xml:lang="en">Sexuality</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15672-4">
    <skos:prefLabel xml:lang="de">Sexualität</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18183-4">
    <skos:related rdf:resource="descriptor/15767-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18183-4">
    <skos:related rdf:resource="descriptor/19633-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18183-4">
    <skos:related rdf:resource="descriptor/19090-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70582">
    <skos:narrower rdf:resource="descriptor/19039-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70582">
    <skos:narrower rdf:resource="descriptor/19044-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70582">
    <skos:narrower rdf:resource="descriptor/19836-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70582">
    <skos:narrower rdf:resource="descriptor/10057-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70582">
    <skos:narrower rdf:resource="descriptor/19036-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70582">
    <skos:narrower rdf:resource="descriptor/19042-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70582">
    <skos:narrower rdf:resource="descriptor/19035-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70582">
    <skos:narrower rdf:resource="descriptor/20402-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70582">
    <skos:narrower rdf:resource="descriptor/19659-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70582">
    <skos:narrower rdf:resource="descriptor/19033-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70582">
    <skos:narrower rdf:resource="descriptor/19041-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70582">
    <skos:narrower rdf:resource="descriptor/19162-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70582">
    <skos:narrower rdf:resource="descriptor/19040-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70582">
    <skos:narrower rdf:resource="descriptor/19032-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70582">
    <skos:narrower rdf:resource="descriptor/15444-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70582">
    <skos:narrower rdf:resource="descriptor/19031-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70582">
    <skos:narrower rdf:resource="descriptor/19028-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70582">
    <skos:narrower rdf:resource="descriptor/20401-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70582">
    <skos:narrower rdf:resource="descriptor/20399-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70582">
    <skos:narrower rdf:resource="descriptor/19038-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70582">
    <skos:narrower rdf:resource="descriptor/19026-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70582">
    <skos:narrower rdf:resource="descriptor/19030-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70582">
    <skos:narrower rdf:resource="descriptor/19534-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70582">
    <skos:narrower rdf:resource="descriptor/19661-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70582">
    <skos:narrower rdf:resource="descriptor/12040-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70582">
    <skos:narrower rdf:resource="descriptor/19029-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70582">
    <skos:narrower rdf:resource="descriptor/19027-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70582">
    <skos:narrower rdf:resource="descriptor/19025-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19746-2">
    <skos:related rdf:resource="descriptor/19010-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19746-2">
    <skos:related rdf:resource="descriptor/18209-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17700-1">
    <skos:broader rdf:resource="thsys/70560"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17700-1">
    <skos:broader rdf:resource="descriptor/17724-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13580-5">
    <skos:related rdf:resource="descriptor/13515-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13580-5">
    <skos:related rdf:resource="descriptor/13587-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13580-5">
    <skos:related rdf:resource="descriptor/13584-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13580-5">
    <skos:related rdf:resource="descriptor/13513-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13504-0">
    <skos:broader rdf:resource="thsys/70943"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13504-0">
    <skos:broader rdf:resource="descriptor/13510-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10421-5">
    <skos:scopeNote xml:lang="de">Integration des Geldes in die Wachstumstheorie.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16445-3">
    <skos:prefLabel xml:lang="en">Disarmament</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16445-3">
    <skos:prefLabel xml:lang="de">Abrüstung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14292-5">
    <skos:prefLabel xml:lang="en">Alkali metal</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14292-5">
    <skos:prefLabel xml:lang="de">Alkalimetall</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20733-1">
    <skos:related rdf:resource="descriptor/12253-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20733-1">
    <skos:related rdf:resource="descriptor/13777-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20733-1">
    <skos:related rdf:resource="descriptor/10621-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20733-1">
    <skos:related rdf:resource="descriptor/18722-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10062-0">
    <skos:broader rdf:resource="thsys/70049"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10062-0">
    <skos:broader rdf:resource="descriptor/10032-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14343-0">
    <skos:prefLabel xml:lang="en">Packaging machinery</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14343-0">
    <skos:prefLabel xml:lang="de">Verpackungsmaschine</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13427-6">
    <skos:altLabel xml:lang="de">Mischdünger</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13427-6">
    <skos:altLabel xml:lang="de">Mineraldünger</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13427-6">
    <skos:altLabel xml:lang="de">Mehrnährstoffdünger</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13427-6">
    <skos:altLabel xml:lang="de">Kunstdünger</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13427-6">
    <skos:altLabel xml:lang="de">Handelsdünger</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13427-6">
    <skos:altLabel xml:lang="de">Dünger</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13427-6">
    <skos:altLabel xml:lang="de">Chemisches Düngemittel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11901-0">
    <skos:prefLabel xml:lang="en">Building refurbishment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11901-0">
    <skos:prefLabel xml:lang="de">Bausanierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19329-4">
    <skos:scopeNote xml:lang="de">Regelungen des internationalen Privatrechts zum Handelsrecht.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15066-2">
    <skos:broader rdf:resource="thsys/70030"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15066-2">
    <skos:broader rdf:resource="descriptor/12713-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11539-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13186-2">
    <skos:related rdf:resource="descriptor/12821-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13186-2">
    <skos:related rdf:resource="descriptor/18499-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13186-2">
    <skos:related rdf:resource="descriptor/18562-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15102-3">
    <skos:narrower rdf:resource="descriptor/15089-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15102-3">
    <skos:narrower rdf:resource="descriptor/11032-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19461-4">
    <skos:broader rdf:resource="thsys/71065"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19461-4">
    <skos:broader rdf:resource="thsys/71060"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19461-4">
    <skos:broader rdf:resource="descriptor/11182-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18553-4">
    <skos:related rdf:resource="descriptor/12821-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18553-4">
    <skos:related rdf:resource="descriptor/13474-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18553-4">
    <skos:related rdf:resource="descriptor/12665-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18553-4">
    <skos:related rdf:resource="descriptor/18532-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18508-2">
    <skos:altLabel xml:lang="de">Niedrigenergiehaus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10305-0">
    <skos:related rdf:resource="descriptor/10334-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10305-0">
    <skos:related rdf:resource="descriptor/10307-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71056">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70180">
    <skos:broader rdf:resource="thsys/v"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10706-3">
    <skos:prefLabel xml:lang="en">Non-tariff barriers</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10706-3">
    <skos:prefLabel xml:lang="de">Nichttarifäre Handelshemmnisse</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13894-3">
    <skos:altLabel xml:lang="de">Sägewerkerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13894-3">
    <skos:altLabel xml:lang="de">Sägewerker</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13894-3">
    <skos:altLabel xml:lang="de">Korbmacherinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13894-3">
    <skos:altLabel xml:lang="de">Korbmacher</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13894-3">
    <skos:altLabel xml:lang="de">Holzverarbeitende Berufe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13894-3">
    <skos:altLabel xml:lang="de">Holzbildhauerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13894-3">
    <skos:altLabel xml:lang="de">Holzbildhauer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13894-3">
    <skos:altLabel xml:lang="de">Holzbearbeitende Berufe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13894-3">
    <skos:altLabel xml:lang="de">Holzarbeiter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13894-3">
    <skos:altLabel xml:lang="de">Herrgottschnitzerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13894-3">
    <skos:altLabel xml:lang="de">Herrgottschnitzer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13894-3">
    <skos:altLabel xml:lang="de">Drechslerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13894-3">
    <skos:altLabel xml:lang="de">Drechsler</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15088-6">
    <skos:altLabel xml:lang="en">CPI (Consumer Price Index)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15088-6">
    <skos:altLabel xml:lang="de">Verbraucherpreisindex</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15088-6">
    <skos:altLabel xml:lang="de">Preisindex der Lebenshaltung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15088-6">
    <skos:altLabel xml:lang="de">Lebenshaltungspreisindex</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15088-6">
    <skos:altLabel xml:lang="de">Index der Verbraucherpreise</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18019-3">
    <skos:related rdf:resource="descriptor/17968-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14769-1">
    <skos:related rdf:resource="descriptor/14245-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14769-1">
    <skos:related rdf:resource="descriptor/13014-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13277-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15211-5">
    <skos:broader rdf:resource="thsys/72182"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15211-5">
    <skos:broader rdf:resource="descriptor/15064-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16188-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14991-0">
    <skos:prefLabel xml:lang="en">Pet food</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14991-0">
    <skos:prefLabel xml:lang="de">Heimtierfutter</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12252-6">
    <skos:narrower rdf:resource="descriptor/13782-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12252-6">
    <skos:narrower rdf:resource="descriptor/13775-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14971-6">
    <skos:narrower rdf:resource="descriptor/14984-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14971-6">
    <skos:narrower rdf:resource="descriptor/14949-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14971-6">
    <skos:narrower rdf:resource="descriptor/14975-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18923-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14251-5">
    <skos:narrower rdf:resource="descriptor/14223-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10285-1">
    <skos:related rdf:resource="descriptor/10276-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70461">
    <skos:prefLabel xml:lang="de">P.12.06  Agrochemikalien</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11212-2">
    <skos:related rdf:resource="descriptor/11273-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11212-2">
    <skos:related rdf:resource="descriptor/19902-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12963-1">
    <skos:related rdf:resource="descriptor/14164-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12963-1">
    <skos:related rdf:resource="descriptor/15892-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12963-1">
    <skos:related rdf:resource="descriptor/15868-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10807-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11191-5">
    <skos:prefLabel xml:lang="en">Dismissal</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11191-5">
    <skos:prefLabel xml:lang="de">Kündigung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19231-0">
    <skos:broader rdf:resource="thsys/71111"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19231-0">
    <skos:broader rdf:resource="thsys/70233"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19231-0">
    <skos:broader rdf:resource="descriptor/10265-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70875">
    <skos:narrower rdf:resource="descriptor/13391-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70875">
    <skos:narrower rdf:resource="descriptor/12881-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70875">
    <skos:narrower rdf:resource="descriptor/12901-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70875">
    <skos:narrower rdf:resource="descriptor/18789-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70875">
    <skos:narrower rdf:resource="descriptor/12889-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70875">
    <skos:narrower rdf:resource="descriptor/18793-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70875">
    <skos:narrower rdf:resource="descriptor/13389-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70875">
    <skos:narrower rdf:resource="descriptor/13384-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70875">
    <skos:narrower rdf:resource="descriptor/12121-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70875">
    <skos:narrower rdf:resource="descriptor/10576-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70875">
    <skos:narrower rdf:resource="descriptor/18791-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70875">
    <skos:narrower rdf:resource="descriptor/12214-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11197-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10522-6">
    <skos:prefLabel xml:lang="en">Dual economy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10522-6">
    <skos:prefLabel xml:lang="de">Dualökonomie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70383">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12960-0">
    <skos:prefLabel xml:lang="en">Floriculture</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12960-0">
    <skos:prefLabel xml:lang="de">Zierpflanzenbau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71023">
    <skos:prefLabel xml:lang="en">W.01.04  Agricultural land-use</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71023">
    <skos:prefLabel xml:lang="de">W.01.04  Landwirtschaftliche Bodennutzung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12189-1">
    <skos:altLabel xml:lang="de">Wirtschaftsunternehmen des öffentlichen Rechts</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12189-1">
    <skos:altLabel xml:lang="de">Stiftung des öffentlichen Rechts</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12189-1">
    <skos:altLabel xml:lang="de">Öffentlich-rechtliche Körperschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12189-1">
    <skos:altLabel xml:lang="de">Öffentlich-rechtliche Anstalt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12189-1">
    <skos:altLabel xml:lang="de">Körperschaft des öffentlichen Rechts</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12189-1">
    <skos:altLabel xml:lang="de">Anstalt des öffentlichen Rechts</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70035">
    <skos:narrower rdf:resource="descriptor/19083-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70035">
    <skos:narrower rdf:resource="descriptor/17477-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70035">
    <skos:narrower rdf:resource="descriptor/17476-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70035">
    <skos:narrower rdf:resource="descriptor/17471-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16027-0">
    <skos:related rdf:resource="descriptor/25681-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16027-0">
    <skos:related rdf:resource="descriptor/15937-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70990">
    <skos:narrower rdf:resource="descriptor/13125-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70990">
    <skos:narrower rdf:resource="descriptor/13124-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70990">
    <skos:narrower rdf:resource="descriptor/13117-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70990">
    <skos:narrower rdf:resource="descriptor/19389-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70990">
    <skos:narrower rdf:resource="descriptor/18434-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70990">
    <skos:narrower rdf:resource="descriptor/13102-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70990">
    <skos:narrower rdf:resource="descriptor/13100-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70990">
    <skos:narrower rdf:resource="thsys/73296"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16874-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id365">
    <rdf:value xml:lang="de">Für wirtschaftspolitische Anpassungsmaßnahmen im Rahmen der IWF- und Weltbank-Programme</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12794-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13188-5">
    <skos:narrower rdf:resource="descriptor/18521-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13188-5">
    <skos:narrower rdf:resource="descriptor/18519-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13188-5">
    <skos:narrower rdf:resource="descriptor/18517-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13188-5">
    <skos:narrower rdf:resource="descriptor/18516-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12572-2">
    <skos:altLabel xml:lang="en">Manpower requirement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18785-4">
    <skos:broader rdf:resource="thsys/71109"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18785-4">
    <skos:broader rdf:resource="thsys/70879"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18785-4">
    <skos:broader rdf:resource="descriptor/10011-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19193-3">
    <skos:related rdf:resource="descriptor/14092-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18793-5">
    <skos:related rdf:resource="descriptor/10988-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18793-5">
    <skos:related rdf:resource="descriptor/10469-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18793-5">
    <skos:related rdf:resource="descriptor/18789-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18793-5">
    <skos:related rdf:resource="descriptor/16274-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18793-5">
    <skos:related rdf:resource="descriptor/13410-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18069-2">
    <skos:related rdf:resource="descriptor/19482-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18069-2">
    <skos:related rdf:resource="descriptor/14640-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18069-2">
    <skos:related rdf:resource="descriptor/19481-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18069-2">
    <skos:related rdf:resource="descriptor/15881-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18069-2">
    <skos:related rdf:resource="descriptor/18079-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18069-2">
    <skos:related rdf:resource="descriptor/13815-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10764-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16411-6">
    <skos:prefLabel xml:lang="en">Liberalism</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16411-6">
    <skos:prefLabel xml:lang="de">Liberalismus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11587-1">
    <skos:prefLabel xml:lang="en">Budget stabilization fund</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11587-1">
    <skos:prefLabel xml:lang="de">Haushaltsrücklage</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18457-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18475-5">
    <skos:scopeNote xml:lang="de">Nur für industrieeigene Versicherungsgesellschaften zur Absicherung von Firmenrisiken</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12798-4">
    <skos:prefLabel xml:lang="en">Private label</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12798-4">
    <skos:prefLabel xml:lang="de">Handelsmarke</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19247-6">
    <skos:altLabel xml:lang="en">Outage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19247-6">
    <skos:altLabel xml:lang="de">Betriebsunterbrechung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19247-6">
    <skos:altLabel xml:lang="de">Betriebsstörung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14066-0">
    <skos:broader rdf:resource="thsys/70169"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14066-0">
    <skos:broader rdf:resource="descriptor/14117-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14066-0">
    <skos:broader rdf:resource="descriptor/14070-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17590-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18674-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15644-2">
    <skos:broader rdf:resource="thsys/70060"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15644-2">
    <skos:broader rdf:resource="descriptor/15685-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18353-5">
    <skos:altLabel xml:lang="de">Wärmetechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10253-0">
    <skos:narrower rdf:resource="descriptor/19230-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10253-0">
    <skos:narrower rdf:resource="descriptor/19552-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10475-3">
    <skos:prefLabel xml:lang="en">Invention</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10475-3">
    <skos:prefLabel xml:lang="de">Erfindung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12983-2">
    <skos:related rdf:resource="descriptor/18394-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12983-2">
    <skos:related rdf:resource="descriptor/12978-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12983-2">
    <skos:related rdf:resource="descriptor/18372-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12983-2">
    <skos:related rdf:resource="descriptor/18393-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12983-2">
    <skos:related rdf:resource="descriptor/19386-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12983-2">
    <skos:related rdf:resource="descriptor/19385-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12983-2">
    <skos:related rdf:resource="descriptor/14196-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18324-5">
    <skos:related rdf:resource="descriptor/18340-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20762-1">
    <skos:broader rdf:resource="thsys/70239"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20762-1">
    <skos:broader rdf:resource="descriptor/15738-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18061-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16721-5">
    <skos:broader rdf:resource="thsys/73394"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16721-5">
    <skos:broader rdf:resource="descriptor/16708-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14920-2">
    <skos:prefLabel xml:lang="en">Sportswear</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14920-2">
    <skos:prefLabel xml:lang="de">Sportbekleidung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13226-2">
    <skos:prefLabel xml:lang="en">Stationer's</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13226-2">
    <skos:prefLabel xml:lang="de">Schreibwarenhandel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20842-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10236-0">
    <skos:broader rdf:resource="thsys/70306"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10236-0">
    <skos:broader rdf:resource="descriptor/10062-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14469-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10976-2">
    <skos:narrower rdf:resource="descriptor/10963-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13621-3">
    <skos:related rdf:resource="descriptor/13299-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13621-3">
    <skos:related rdf:resource="descriptor/13273-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13621-3">
    <skos:related rdf:resource="descriptor/18645-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13621-3">
    <skos:related rdf:resource="descriptor/18616-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13621-3">
    <skos:related rdf:resource="descriptor/18615-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13621-3">
    <skos:related rdf:resource="descriptor/13565-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13621-3">
    <skos:related rdf:resource="descriptor/12689-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70192">
    <rdfs:label xml:lang="de">Nachbarwissenschaften</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13367-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12914-0">
    <skos:narrower rdf:resource="descriptor/18410-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12914-0">
    <skos:narrower rdf:resource="descriptor/18408-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11782-3">
    <skos:altLabel xml:lang="de">Sektorstruktur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11782-3">
    <skos:altLabel xml:lang="de">Sektorale Wirtschaftsstruktur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11782-3">
    <skos:altLabel xml:lang="de">Sektorale Produktionsstruktur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11782-3">
    <skos:altLabel xml:lang="de">Branchenstruktur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19850-0">
    <skos:related rdf:resource="descriptor/16067-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11248-2">
    <skos:altLabel xml:lang="de">Kapovaz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11248-2">
    <skos:altLabel xml:lang="de">Kapazitätsorientierte variable Arbeitszeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11248-2">
    <skos:altLabel xml:lang="de">Flexible Arbeitszeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11248-2">
    <skos:altLabel xml:lang="de">Flexibilisierung der Arbeitszeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11248-2">
    <skos:altLabel xml:lang="de">Arbeitszeitverlängerung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11248-2">
    <skos:altLabel xml:lang="de">Arbeitszeitverkürzung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11248-2">
    <skos:altLabel xml:lang="de">Arbeitszeitregelung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11248-2">
    <skos:altLabel xml:lang="de">Arbeitszeitorganisation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11248-2">
    <skos:altLabel xml:lang="de">Arbeitszeitmodell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11248-2">
    <skos:altLabel xml:lang="de">Arbeitszeitmanagement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11248-2">
    <skos:altLabel xml:lang="de">Arbeitszeitkonto</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21110-4">
    <skos:altLabel xml:lang="de">Sportwaffe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21110-4">
    <skos:altLabel xml:lang="de">Sportgewehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21110-4">
    <skos:altLabel xml:lang="de">Revolver</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21110-4">
    <skos:altLabel xml:lang="de">Pistole</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21110-4">
    <skos:altLabel xml:lang="de">Jagdwaffe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21110-4">
    <skos:altLabel xml:lang="de">Jagdgewehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21110-4">
    <skos:altLabel xml:lang="de">Gaspistole</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11722-0">
    <skos:related rdf:resource="descriptor/11721-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11722-0">
    <skos:related rdf:resource="descriptor/13420-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11722-0">
    <skos:related rdf:resource="descriptor/18822-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11722-0">
    <skos:related rdf:resource="descriptor/12064-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11722-0">
    <skos:related rdf:resource="descriptor/11718-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11722-0">
    <skos:related rdf:resource="descriptor/11559-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11722-0">
    <skos:related rdf:resource="descriptor/11540-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11722-0">
    <skos:related rdf:resource="descriptor/18823-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11722-0">
    <skos:related rdf:resource="descriptor/24705-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11722-0">
    <skos:related rdf:resource="descriptor/11043-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19839-2">
    <skos:related rdf:resource="descriptor/19360-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19839-2">
    <skos:related rdf:resource="descriptor/19221-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19839-2">
    <skos:related rdf:resource="descriptor/19774-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19839-2">
    <skos:related rdf:resource="descriptor/16699-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19839-2">
    <skos:related rdf:resource="descriptor/18968-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13583-6">
    <skos:prefLabel xml:lang="en">Transport flows</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13583-6">
    <skos:prefLabel xml:lang="de">Verkehrsaufkommen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11821-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19738-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14427-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14098-1">
    <skos:broader rdf:resource="thsys/70070"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14098-1">
    <skos:broader rdf:resource="descriptor/14088-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11665-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11124-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11711-5">
    <skos:altLabel xml:lang="de">Popitzsches Gesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11711-5">
    <skos:altLabel xml:lang="de">Optimale Aufgabenverteilung der Finanzpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11711-5">
    <skos:altLabel xml:lang="de">Optimale Aufgabenteilung der Finanzpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11711-5">
    <skos:altLabel xml:lang="de">Aufgabenverteilung (Finanzpolitik)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10198-3">
    <skos:related rdf:resource="descriptor/19223-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10198-3">
    <skos:related rdf:resource="descriptor/10204-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10198-3">
    <skos:related rdf:resource="descriptor/19218-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10198-3">
    <skos:related rdf:resource="descriptor/12518-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10198-3">
    <skos:related rdf:resource="descriptor/20881-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10198-3">
    <skos:related rdf:resource="descriptor/12794-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12922-1">
    <skos:altLabel xml:lang="de">Rohstoffverarbeitung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12922-1">
    <skos:altLabel xml:lang="de">Rohstoffindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12922-1">
    <skos:altLabel xml:lang="de">Rohstoffgewinnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12922-1">
    <skos:altLabel xml:lang="de">Grundstoffindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18408-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16072-2">
    <skos:broader rdf:resource="thsys/73390"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16072-2">
    <skos:broader rdf:resource="thsys/70903"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16072-2">
    <skos:broader rdf:resource="descriptor/13693-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12920-5">
    <skos:broader rdf:resource="thsys/70992"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12920-5">
    <skos:broader rdf:resource="descriptor/12913-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19095-3">
    <skos:broader rdf:resource="thsys/71042"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19095-3">
    <skos:broader rdf:resource="descriptor/12275-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14272-4">
    <skos:broader rdf:resource="thsys/70346"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14272-4">
    <skos:broader rdf:resource="descriptor/14269-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16516-6">
    <skos:broader rdf:resource="thsys/73369"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16516-6">
    <skos:broader rdf:resource="descriptor/16515-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18797-4">
    <skos:altLabel xml:lang="de">Messeveranstalter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18797-4">
    <skos:altLabel xml:lang="de">Messeindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18797-4">
    <skos:altLabel xml:lang="de">Messegesellschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18797-4">
    <skos:altLabel xml:lang="de">Ausstellungsorganisation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15483-0">
    <skos:broader rdf:resource="thsys/71035"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15483-0">
    <skos:broader rdf:resource="descriptor/19207-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18644-1">
    <skos:prefLabel xml:lang="en">Post office</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18644-1">
    <skos:prefLabel xml:lang="de">Postunternehmen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10225-5">
    <skos:related rdf:resource="descriptor/10056-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13256-0">
    <skos:narrower rdf:resource="descriptor/18589-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13256-0">
    <skos:narrower rdf:resource="descriptor/18588-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13256-0">
    <skos:narrower rdf:resource="descriptor/13543-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13256-0">
    <skos:narrower rdf:resource="descriptor/13553-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13256-0">
    <skos:narrower rdf:resource="descriptor/13530-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13256-0">
    <skos:narrower rdf:resource="descriptor/13242-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13980-3">
    <skos:broader rdf:resource="thsys/71055"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13980-3">
    <skos:broader rdf:resource="thsys/70932"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13980-3">
    <skos:broader rdf:resource="descriptor/13979-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13014-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18624-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12325-5">
    <skos:narrower rdf:resource="descriptor/12341-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12325-5">
    <skos:narrower rdf:resource="descriptor/12340-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12325-5">
    <skos:narrower rdf:resource="descriptor/12330-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19593-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10310-0">
    <skos:broader rdf:resource="thsys/70310"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10310-0">
    <skos:broader rdf:resource="descriptor/12725-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10970-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15807-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14724-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14737-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18087-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14308-2">
    <skos:broader rdf:resource="thsys/70134"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14308-2">
    <skos:broader rdf:resource="descriptor/14296-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12820-2">
    <skos:broader rdf:resource="thsys/70384"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12820-2">
    <skos:broader rdf:resource="thsys/70278"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12820-2">
    <skos:broader rdf:resource="thsys/70245"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12820-2">
    <skos:broader rdf:resource="descriptor/12818-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12820-2">
    <skos:broader rdf:resource="descriptor/11772-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12649-0">
    <skos:related rdf:resource="descriptor/18783-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12649-0">
    <skos:related rdf:resource="descriptor/10027-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12649-0">
    <skos:related rdf:resource="descriptor/12648-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12649-0">
    <skos:related rdf:resource="descriptor/12647-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13193-5">
    <skos:prefLabel xml:lang="en">Pharmaceutical trade</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13193-5">
    <skos:prefLabel xml:lang="de">Pharmahandel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19537-4">
    <skos:broader rdf:resource="thsys/70897"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19537-4">
    <skos:broader rdf:resource="thsys/70187"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19537-4">
    <skos:broader rdf:resource="descriptor/19271-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15467-5">
    <skos:scopeNote xml:lang="de">Entscheidungssituation bei Vorliegen objektiver Wahrscheinlichkeiten.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10911-2">
    <skos:broader rdf:resource="thsys/70212"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10911-2">
    <skos:broader rdf:resource="descriptor/10913-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15951-0">
    <skos:related rdf:resource="descriptor/15956-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15951-0">
    <skos:related rdf:resource="descriptor/15919-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15951-0">
    <skos:related rdf:resource="descriptor/10491-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15951-0">
    <skos:related rdf:resource="descriptor/19648-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13756-1">
    <skos:broader rdf:resource="thsys/70899"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12318-2">
    <skos:broader rdf:resource="thsys/71042"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12318-2">
    <skos:broader rdf:resource="descriptor/12053-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18705-0">
    <skos:related rdf:resource="descriptor/13290-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18705-0">
    <skos:related rdf:resource="descriptor/10550-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11478-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20948-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19167-4">
    <skos:broader rdf:resource="thsys/70561"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19167-4">
    <skos:broader rdf:resource="descriptor/17553-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18228-1">
    <skos:related rdf:resource="descriptor/17176-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70505">
    <rdfs:label xml:lang="en">Northern Europe</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70505">
    <rdfs:label xml:lang="de">Nordeuropa</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19111-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18994-2">
    <skos:prefLabel xml:lang="en">Mandatory insurance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18994-2">
    <skos:prefLabel xml:lang="de">Versicherungspflicht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18308-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13181-5">
    <skos:broader rdf:resource="thsys/70968"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13181-5">
    <skos:broader rdf:resource="thsys/70964"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13181-5">
    <skos:broader rdf:resource="descriptor/12910-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13181-5">
    <skos:broader rdf:resource="descriptor/13180-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17616-4">
    <skos:prefLabel xml:lang="en">Arab oil-producing countries</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17616-4">
    <skos:prefLabel xml:lang="de">Arabische Ölstaaten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19647-4">
    <skos:broader rdf:resource="thsys/70572"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19647-4">
    <skos:broader rdf:resource="descriptor/11888-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73303">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">P.20.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18270-2">
    <skos:related rdf:resource="descriptor/15616-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13656-5">
    <skos:altLabel xml:lang="en">Cable TV</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13656-5">
    <skos:altLabel xml:lang="de">Kabelverteilfernsehen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15633-0">
    <skos:broader rdf:resource="thsys/73348"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15633-0">
    <skos:broader rdf:resource="descriptor/15664-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13279-2">
    <skos:altLabel xml:lang="de">Börsenhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13279-2">
    <skos:altLabel xml:lang="de">Amtlicher Handel an der Börse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18748-3">
    <skos:prefLabel xml:lang="en">Industrial land use</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18748-3">
    <skos:prefLabel xml:lang="de">Gewerbliche Bodennutzung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15598-4">
    <skos:altLabel xml:lang="en">Computer-based information system</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15598-4">
    <skos:altLabel xml:lang="de">Rechnergestütztes Informationssystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15598-4">
    <skos:altLabel xml:lang="de">EDV-unterstütztes Informationssystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15598-4">
    <skos:altLabel xml:lang="de">Computergestütztes Informationssystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16436-4">
    <skos:narrower rdf:resource="descriptor/10625-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18913-0">
    <skos:related rdf:resource="descriptor/13418-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18913-0">
    <skos:related rdf:resource="descriptor/20906-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18913-0">
    <skos:related rdf:resource="descriptor/18063-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17746-5">
    <skos:altLabel xml:lang="de">Demokratische Volksrepublik Algerien</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13504-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19295-2">
    <skos:broader rdf:resource="thsys/73294"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19295-2">
    <skos:broader rdf:resource="thsys/71029"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19295-2">
    <skos:broader rdf:resource="thsys/71020"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19295-2">
    <skos:broader rdf:resource="descriptor/15752-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19295-2">
    <skos:broader rdf:resource="descriptor/11801-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16288-0">
    <skos:broader rdf:resource="thsys/73375"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16288-0">
    <skos:broader rdf:resource="thsys/73364"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16288-0">
    <skos:broader rdf:resource="descriptor/11163-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14531-6">
    <skos:broader rdf:resource="thsys/70423"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10981-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18231-5">
    <skos:broader rdf:resource="thsys/70651"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18231-5">
    <skos:broader rdf:resource="thsys/70227"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18231-5">
    <skos:broader rdf:resource="descriptor/18024-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13702-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15399-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15734-1">
    <skos:related rdf:resource="descriptor/15746-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15734-1">
    <skos:related rdf:resource="descriptor/15733-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15734-1">
    <skos:related rdf:resource="descriptor/13327-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15734-1">
    <skos:related rdf:resource="descriptor/15730-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16700-6">
    <skos:prefLabel xml:lang="en">Sociology of youth</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16700-6">
    <skos:prefLabel xml:lang="de">Jugendsoziologie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19243-0">
    <skos:broader rdf:resource="thsys/71065"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19243-0">
    <skos:broader rdf:resource="thsys/70406"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19243-0">
    <skos:broader rdf:resource="descriptor/10414-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18229-6">
    <skos:related rdf:resource="descriptor/18103-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73337">
    <rdfs:label xml:lang="de">Finanz- und Steuerrecht</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19207-4">
    <skos:narrower rdf:resource="descriptor/15483-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19207-4">
    <skos:narrower rdf:resource="descriptor/15373-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11436-1">
    <skos:prefLabel xml:lang="en">Paper money</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11436-1">
    <skos:prefLabel xml:lang="de">Papierwährung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11161-0">
    <skos:scopeNote xml:lang="de">Ökonomische Analyse, welche die wirtschaftliche Entwicklung in enger Anlehnung an die jeweils gegebenen institutionellen Rahmenbedingungen, vorwiegend die der Rechtsordnung, zu erklären sucht.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18258-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12020-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11914-5">
    <skos:broader rdf:resource="thsys/71067"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19791-4">
    <skos:prefLabel xml:lang="en">Heart disease</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19791-4">
    <skos:prefLabel xml:lang="de">Herzkrankheit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18080-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id263">
    <rdfs:seeAlso rdf:resource="descriptor/10030-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13459-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16079-2">
    <skos:prefLabel xml:lang="en">Law of exchange</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16079-2">
    <skos:prefLabel xml:lang="de">Wechselrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19807-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18059-5">
    <skos:scopeNote xml:lang="de">Für die formale Kennzeichnung von Dokumenten als Wirtschafts/Brancheninformation BENUTZE die entsprechenden Publikationsart-Bezeichnungen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18059-5">
    <skos:scopeNote xml:lang="de">Nur im Sinne des Angebots an Wirtschaftsinformationsdienstleistungen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18362-4">
    <skos:prefLabel xml:lang="en">Renewable energy policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18362-4">
    <skos:prefLabel xml:lang="de">Förderung regenerativer Energien</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73294">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12573-0">
    <skos:narrower rdf:resource="descriptor/12571-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13153-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13379-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14548-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15711-6">
    <skos:altLabel xml:lang="en">Political movement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15711-6">
    <skos:altLabel xml:lang="de">Protestbewegung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15711-6">
    <skos:altLabel xml:lang="de">Politischer Protest</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15711-6">
    <skos:altLabel xml:lang="de">Politische Bewegung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15711-6">
    <skos:altLabel xml:lang="de">Gesellschaftliche Bewegung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15711-6">
    <skos:altLabel xml:lang="de">Alternativbewegung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18490-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19078-3">
    <skos:narrower rdf:resource="descriptor/17465-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19078-3">
    <skos:narrower rdf:resource="descriptor/17503-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19078-3">
    <skos:narrower rdf:resource="descriptor/17485-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19078-3">
    <skos:narrower rdf:resource="descriptor/17502-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19078-3">
    <skos:narrower rdf:resource="descriptor/17500-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19078-3">
    <skos:narrower rdf:resource="descriptor/17534-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19078-3">
    <skos:narrower rdf:resource="descriptor/17480-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19078-3">
    <skos:narrower rdf:resource="descriptor/17497-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19078-3">
    <skos:narrower rdf:resource="descriptor/17496-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19078-3">
    <skos:narrower rdf:resource="descriptor/17472-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19078-3">
    <skos:narrower rdf:resource="descriptor/17491-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19078-3">
    <skos:narrower rdf:resource="descriptor/17464-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19250-3">
    <skos:related rdf:resource="descriptor/18117-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19250-3">
    <skos:related rdf:resource="descriptor/10332-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19250-3">
    <skos:related rdf:resource="descriptor/10323-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19355-3">
    <skos:scopeNote xml:lang="de">Warenstruktur von Exporten oder Importen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18228-1">
    <skos:broader rdf:resource="thsys/70130"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18228-1">
    <skos:broader rdf:resource="descriptor/17179-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16532-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12624-2">
    <skos:prefLabel xml:lang="en">Flex-time</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12624-2">
    <skos:prefLabel xml:lang="de">Gleitzeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14787-6">
    <skos:altLabel xml:lang="de">Polstermöbel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14787-6">
    <skos:altLabel xml:lang="de">Korbmöbel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13246-3">
    <skos:prefLabel xml:lang="en">Air transport</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13246-3">
    <skos:prefLabel xml:lang="de">Luftverkehr</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17914-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10813-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16405-1">
    <skos:narrower rdf:resource="descriptor/16229-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16405-1">
    <skos:narrower rdf:resource="descriptor/11060-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16405-1">
    <skos:narrower rdf:resource="descriptor/16402-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16405-1">
    <skos:narrower rdf:resource="descriptor/16404-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16405-1">
    <skos:narrower rdf:resource="descriptor/16416-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16405-1">
    <skos:narrower rdf:resource="descriptor/11088-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16405-1">
    <skos:narrower rdf:resource="descriptor/16411-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16405-1">
    <skos:narrower rdf:resource="descriptor/16410-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16405-1">
    <skos:narrower rdf:resource="descriptor/19680-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16405-1">
    <skos:narrower rdf:resource="descriptor/11047-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16405-1">
    <skos:narrower rdf:resource="descriptor/16249-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16405-1">
    <skos:narrower rdf:resource="descriptor/16397-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10988-2">
    <skos:broader rdf:resource="thsys/70038"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10988-2">
    <skos:broader rdf:resource="descriptor/16274-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70838">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.25.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17472-2">
    <skos:prefLabel xml:lang="en">Georgia (Republic)</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17472-2">
    <skos:prefLabel xml:lang="de">Georgien</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14713-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19370-0">
    <skos:narrower rdf:resource="descriptor/11668-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19370-0">
    <skos:narrower rdf:resource="descriptor/11592-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16322-5">
    <skos:broader rdf:resource="thsys/73380"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16322-5">
    <skos:broader rdf:resource="thsys/70870"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16322-5">
    <skos:broader rdf:resource="descriptor/16336-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70021">
    <skos:broader rdf:resource="thsys/v"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16059-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70882">
    <skos:narrower rdf:resource="descriptor/20068-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70882">
    <skos:narrower rdf:resource="descriptor/13387-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70882">
    <skos:narrower rdf:resource="descriptor/24708-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70882">
    <skos:narrower rdf:resource="descriptor/13346-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70882">
    <skos:narrower rdf:resource="descriptor/18764-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70882">
    <skos:narrower rdf:resource="descriptor/15609-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70882">
    <skos:narrower rdf:resource="descriptor/13379-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70882">
    <skos:narrower rdf:resource="descriptor/13987-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70882">
    <skos:narrower rdf:resource="descriptor/18762-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70882">
    <skos:narrower rdf:resource="descriptor/15606-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70882">
    <skos:narrower rdf:resource="descriptor/18433-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70882">
    <skos:narrower rdf:resource="thsys/70881"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70882">
    <skos:narrower rdf:resource="thsys/70880"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18173-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12055-1">
    <skos:prefLabel xml:lang="en">Conglomerate</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12055-1">
    <skos:prefLabel xml:lang="de">Konglomerat</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19556-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11124-6">
    <skos:scopeNote xml:lang="de">Nur für sozialistische Staaten.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13486-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12261-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14715-3">
    <skos:broader rdf:resource="thsys/70476"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14715-3">
    <skos:broader rdf:resource="descriptor/14710-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73352">
    <skos:narrower rdf:resource="descriptor/19622-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73352">
    <skos:narrower rdf:resource="descriptor/19624-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73352">
    <skos:narrower rdf:resource="descriptor/15895-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73352">
    <skos:narrower rdf:resource="descriptor/15894-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73352">
    <skos:narrower rdf:resource="descriptor/15901-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73352">
    <skos:narrower rdf:resource="descriptor/19482-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73352">
    <skos:narrower rdf:resource="descriptor/19486-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73352">
    <skos:narrower rdf:resource="descriptor/15888-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73352">
    <skos:narrower rdf:resource="descriptor/19623-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73352">
    <skos:narrower rdf:resource="descriptor/18069-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73352">
    <skos:narrower rdf:resource="descriptor/19481-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73352">
    <skos:narrower rdf:resource="descriptor/15881-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73352">
    <skos:narrower rdf:resource="descriptor/15880-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73352">
    <skos:narrower rdf:resource="descriptor/19621-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73352">
    <skos:narrower rdf:resource="descriptor/19620-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16780-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12923-6">
    <skos:related rdf:resource="descriptor/13052-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12923-6">
    <skos:related rdf:resource="descriptor/12699-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19222-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70651">
    <rdfs:label xml:lang="de">Regenerative Energieträger</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70021">
    <rdfs:label xml:lang="de">Konjunktur, Wachstum und Wirtschaftsstruktur</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12182-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18221-1">
    <skos:related rdf:resource="descriptor/14975-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13215-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14382-4">
    <skos:related rdf:resource="descriptor/18402-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14382-4">
    <skos:related rdf:resource="descriptor/14269-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14382-4">
    <skos:related rdf:resource="descriptor/13034-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14382-4">
    <skos:related rdf:resource="descriptor/14266-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10909-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10957-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13248-6">
    <skos:prefLabel xml:lang="en">Car rental</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13248-6">
    <skos:prefLabel xml:lang="de">Autovermietung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15893-2">
    <skos:prefLabel xml:lang="en">Steppe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15893-2">
    <skos:prefLabel xml:lang="de">Steppe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13641-4">
    <skos:altLabel xml:lang="de">Grenzüberschreitender Reiseverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13641-4">
    <skos:altLabel xml:lang="de">Auslandstourismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13641-4">
    <skos:altLabel xml:lang="de">Auslandsreise</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18607-0">
    <skos:altLabel xml:lang="en">Airline safety</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18607-0">
    <skos:altLabel xml:lang="de">Luftverkehrskontrolle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18607-0">
    <skos:altLabel xml:lang="de">Landesystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18607-0">
    <skos:altLabel xml:lang="de">Flugsicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18607-0">
    <skos:altLabel xml:lang="de">Bodenkontrolle im Luftverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19228-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19216-3">
    <skos:related rdf:resource="descriptor/15861-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19216-3">
    <skos:related rdf:resource="descriptor/10222-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19216-3">
    <skos:related rdf:resource="descriptor/16562-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70823">
    <skos:narrower rdf:resource="descriptor/19253-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70823">
    <skos:narrower rdf:resource="descriptor/18059-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70823">
    <skos:narrower rdf:resource="descriptor/18942-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70823">
    <skos:narrower rdf:resource="descriptor/20314-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70823">
    <skos:narrower rdf:resource="descriptor/16760-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70823">
    <skos:narrower rdf:resource="descriptor/19475-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70823">
    <skos:narrower rdf:resource="descriptor/12170-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70823">
    <skos:narrower rdf:resource="descriptor/16788-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70823">
    <skos:narrower rdf:resource="descriptor/13346-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70823">
    <skos:narrower rdf:resource="descriptor/16759-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70823">
    <skos:narrower rdf:resource="descriptor/10142-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70823">
    <skos:narrower rdf:resource="descriptor/16786-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70823">
    <skos:narrower rdf:resource="descriptor/15961-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70823">
    <skos:narrower rdf:resource="descriptor/18944-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70823">
    <skos:narrower rdf:resource="descriptor/10141-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70823">
    <skos:narrower rdf:resource="descriptor/18943-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70823">
    <skos:narrower rdf:resource="descriptor/10140-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70823">
    <skos:narrower rdf:resource="descriptor/16604-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70823">
    <skos:narrower rdf:resource="descriptor/12168-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70823">
    <skos:narrower rdf:resource="descriptor/13345-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70823">
    <skos:narrower rdf:resource="descriptor/14001-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70823">
    <skos:narrower rdf:resource="descriptor/16754-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70823">
    <skos:narrower rdf:resource="descriptor/18940-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70823">
    <skos:narrower rdf:resource="descriptor/18941-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70823">
    <skos:narrower rdf:resource="descriptor/16782-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70823">
    <skos:narrower rdf:resource="descriptor/13341-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70823">
    <skos:narrower rdf:resource="thsys/70822"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13995-4">
    <skos:prefLabel xml:lang="en">Legal profession</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13995-4">
    <skos:prefLabel xml:lang="de">Rechtsberufe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id213">
    <rdfs:seeAlso rdf:resource="descriptor/19801-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14586-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12263-1">
    <skos:prefLabel xml:lang="en">Convertible bond</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12263-1">
    <skos:prefLabel xml:lang="de">Wandelanleihe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18257-1">
    <skos:prefLabel xml:lang="en">Synthetic rubber</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18257-1">
    <skos:prefLabel xml:lang="de">Synthesekautschuk</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14654-4">
    <skos:broader rdf:resource="thsys/70463"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14654-4">
    <skos:broader rdf:resource="descriptor/14642-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10178-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#ConceptScheme">
    <rdfs:isDefinedBy rdf:resource="http://www.w3.org/2004/02/skos/core"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19378-5">
    <skos:prefLabel xml:lang="en">Local fiscal policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19378-5">
    <skos:prefLabel xml:lang="de">Kommunale Finanzpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71039">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16440-6">
    <skos:narrower rdf:resource="descriptor/11346-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19210-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20762-1">
    <skos:related rdf:resource="descriptor/15776-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11246-6">
    <skos:broader rdf:resource="thsys/71061"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11246-6">
    <skos:broader rdf:resource="descriptor/11239-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12521-5">
    <skos:altLabel xml:lang="de">Stellenkosten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12521-5">
    <skos:altLabel xml:lang="de">Platzkostenrechnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12521-5">
    <skos:altLabel xml:lang="de">Kostenstellen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12521-5">
    <skos:altLabel xml:lang="de">Kostenplatzrechnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18105-3">
    <skos:related rdf:resource="descriptor/12281-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18105-3">
    <skos:related rdf:resource="descriptor/12287-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10968-1">
    <skos:altLabel xml:lang="de">Wirtschaftsinteressenpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10968-1">
    <skos:altLabel xml:lang="de">Wirtschaftsinteressen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10968-1">
    <skos:altLabel xml:lang="de">Verbandspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10968-1">
    <skos:altLabel xml:lang="de">Standespolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10968-1">
    <skos:altLabel xml:lang="de">Ökonomische Interessen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10968-1">
    <skos:altLabel xml:lang="de">Lobbytätigkeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10968-1">
    <skos:altLabel xml:lang="de">Lobbyismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11821-5">
    <skos:prefLabel xml:lang="en">Industrial structure</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11821-5">
    <skos:prefLabel xml:lang="de">Industriestruktur</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10567-5">
    <skos:prefLabel xml:lang="en">Special economic zone</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10567-5">
    <skos:prefLabel xml:lang="de">Wirtschaftssonderzone</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14909-4">
    <skos:related rdf:resource="descriptor/14019-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11070-3">
    <skos:related rdf:resource="descriptor/16619-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11070-3">
    <skos:related rdf:resource="descriptor/11060-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11070-3">
    <skos:related rdf:resource="descriptor/11073-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11070-3">
    <skos:related rdf:resource="descriptor/11087-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11070-3">
    <skos:related rdf:resource="descriptor/16387-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11070-3">
    <skos:related rdf:resource="descriptor/11286-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14645-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15115-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71085">
    <rdfs:label xml:lang="en">Development policy</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71085">
    <rdfs:label xml:lang="de">Entwicklungspolitik</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14153-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18253-2">
    <skos:related rdf:resource="descriptor/14634-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19263-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18135-1">
    <skos:prefLabel xml:lang="en">Saxony-Anhalt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18135-1">
    <skos:prefLabel xml:lang="de">Sachsen-Anhalt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18621-6">
    <skos:related rdf:resource="descriptor/13615-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18621-6">
    <skos:related rdf:resource="descriptor/14318-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12912-4">
    <skos:broader rdf:resource="thsys/70991"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10521-1">
    <skos:prefLabel xml:lang="en">Sector hypothesis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10521-1">
    <skos:prefLabel xml:lang="de">Drei-Sektoren-Hypothese</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12502-2">
    <skos:related rdf:resource="descriptor/16547-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12502-2">
    <skos:related rdf:resource="descriptor/11240-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12502-2">
    <skos:related rdf:resource="descriptor/12592-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11990-3">
    <skos:broader rdf:resource="thsys/71064"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11990-3">
    <skos:broader rdf:resource="descriptor/11912-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11307-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18204-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19274-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10253-0">
    <skos:related rdf:resource="descriptor/10031-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10253-0">
    <skos:related rdf:resource="descriptor/10064-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20277-1">
    <skos:broader rdf:resource="thsys/70029"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20277-1">
    <skos:broader rdf:resource="descriptor/12088-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11529-1">
    <skos:related rdf:resource="descriptor/10113-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11529-1">
    <skos:related rdf:resource="descriptor/10112-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11529-1">
    <skos:related rdf:resource="descriptor/10991-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11529-1">
    <skos:related rdf:resource="descriptor/16696-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11529-1">
    <skos:related rdf:resource="descriptor/11527-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11529-1">
    <skos:related rdf:resource="descriptor/19558-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11529-1">
    <skos:related rdf:resource="descriptor/10590-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11529-1">
    <skos:related rdf:resource="descriptor/11556-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11529-1">
    <skos:related rdf:resource="descriptor/11531-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11529-1">
    <skos:related rdf:resource="descriptor/11530-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11529-1">
    <skos:related rdf:resource="descriptor/12474-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11529-1">
    <skos:related rdf:resource="descriptor/10011-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11529-1">
    <skos:related rdf:resource="descriptor/12208-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11529-1">
    <skos:related rdf:resource="descriptor/19217-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11529-1">
    <skos:related rdf:resource="descriptor/10548-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11529-1">
    <skos:related rdf:resource="descriptor/19536-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13636-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10094-1">
    <skos:broader rdf:resource="thsys/70057"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10094-1">
    <skos:broader rdf:resource="descriptor/10092-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19272-0">
    <skos:altLabel xml:lang="de">Regionaler Kapitalmarkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71025">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.01.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18401-6">
    <skos:broader rdf:resource="thsys/73343"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18401-6">
    <skos:broader rdf:resource="thsys/71003"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18401-6">
    <skos:broader rdf:resource="descriptor/18783-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19617-6">
    <skos:prefLabel xml:lang="en">Strait</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19617-6">
    <skos:prefLabel xml:lang="de">Meerenge</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10095-6">
    <skos:prefLabel xml:lang="en">Interpersonal comparison of utility</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10095-6">
    <skos:prefLabel xml:lang="de">Interpersoneller Nutzenvergleich</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17695-3">
    <skos:altLabel xml:lang="de">Volksrepublik Benin</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17695-3">
    <skos:altLabel xml:lang="de">Republik Benin</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17695-3">
    <skos:altLabel xml:lang="de">Dahome</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14001-0">
    <skos:prefLabel xml:lang="en">Information professionals</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14001-0">
    <skos:prefLabel xml:lang="de">Informationsberufe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16594-0">
    <skos:prefLabel xml:lang="en">Progress</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16594-0">
    <skos:prefLabel xml:lang="de">Fortschritt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16357-0">
    <skos:altLabel xml:lang="en">Electoral rules</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16357-0">
    <skos:altLabel xml:lang="de">Wahlverfahren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16357-0">
    <skos:altLabel xml:lang="de">Verhältniswahlsystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16357-0">
    <skos:altLabel xml:lang="de">Verhältniswahl</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16357-0">
    <skos:altLabel xml:lang="de">Mehrheitswahlsystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16357-0">
    <skos:altLabel xml:lang="de">Mehrheitswahl</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16357-0">
    <skos:altLabel xml:lang="de">Klassenwahl</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18008-1">
    <skos:related rdf:resource="descriptor/13188-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18008-1">
    <skos:related rdf:resource="descriptor/10833-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18008-1">
    <skos:related rdf:resource="descriptor/18330-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71007">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13376-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12208-2">
    <skos:prefLabel xml:lang="en">Capital budgeting</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12208-2">
    <skos:prefLabel xml:lang="de">Investitionsrechnung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10077-1">
    <skos:broader rdf:resource="thsys/71116"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10077-1">
    <skos:broader rdf:resource="thsys/70188"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10077-1">
    <skos:broader rdf:resource="descriptor/10076-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12210-1">
    <skos:altLabel xml:lang="de">Theorie des Kapitalmarktgleichgewichts</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18380-2">
    <skos:altLabel xml:lang="en">Barrage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18380-2">
    <skos:altLabel xml:lang="de">Staumauer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18380-2">
    <skos:altLabel xml:lang="de">Staudamm</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70838">
    <rdfs:label xml:lang="en">Occupational health</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70838">
    <rdfs:label xml:lang="de">Betriebliche Gesundheitspolitik</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18297-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18737-1">
    <skos:prefLabel xml:lang="en">Industrial insurance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18737-1">
    <skos:prefLabel xml:lang="de">Industrieversicherung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11161-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11400-1">
    <skos:broader rdf:resource="thsys/73377"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11400-1">
    <skos:broader rdf:resource="thsys/70847"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11400-1">
    <skos:broader rdf:resource="thsys/70176"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70358">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">P.07</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11290-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10859-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71089">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13362-1">
    <skos:related rdf:resource="descriptor/18956-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13362-1">
    <skos:related rdf:resource="descriptor/15675-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13362-1">
    <skos:related rdf:resource="descriptor/19398-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13362-1">
    <skos:related rdf:resource="descriptor/18803-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13362-1">
    <skos:related rdf:resource="descriptor/19479-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13362-1">
    <skos:related rdf:resource="descriptor/13320-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13046-4">
    <skos:prefLabel xml:lang="en">Bicycle industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13046-4">
    <skos:prefLabel xml:lang="de">Fahrradindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12492-0">
    <skos:broader rdf:resource="thsys/70578"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12492-0">
    <skos:broader rdf:resource="descriptor/10011-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10686-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18656-1">
    <skos:altLabel xml:lang="en">Satellite TV</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16961-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73347">
    <skos:prefLabel xml:lang="de">N.10.05  Medizin und Ernährungswissenschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18580-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11134-3">
    <skos:related rdf:resource="descriptor/11142-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17658-2">
    <skos:prefLabel xml:lang="en">Djibouti</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17658-2">
    <skos:prefLabel xml:lang="de">Dschibuti</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16201-3">
    <skos:related rdf:resource="descriptor/16203-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16201-3">
    <skos:related rdf:resource="descriptor/16202-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16201-3">
    <skos:related rdf:resource="descriptor/18130-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16201-3">
    <skos:related rdf:resource="descriptor/15766-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16201-3">
    <skos:related rdf:resource="descriptor/16112-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16201-3">
    <skos:related rdf:resource="descriptor/10018-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16201-3">
    <skos:related rdf:resource="descriptor/19814-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16201-3">
    <skos:related rdf:resource="descriptor/16193-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16201-3">
    <skos:related rdf:resource="descriptor/11419-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16201-3">
    <skos:related rdf:resource="descriptor/18736-3"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id55">
    <rdf:value xml:lang="de">Für eine allgemeine kritische Einschätzung oder Beurteilung</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13001-5">
    <skos:altLabel xml:lang="en">Petrochemistry</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70923">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.13</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14054-0">
    <skos:altLabel xml:lang="de">Landwirtschaftliches Produkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19152-3">
    <skos:scopeNote xml:lang="de">Managementkonzept, das sich an den Prinzipien der Lean Production orientiert.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19188-3">
    <skos:broader rdf:resource="thsys/71029"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19188-3">
    <skos:broader rdf:resource="thsys/70960"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19188-3">
    <skos:broader rdf:resource="descriptor/13431-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18829-3">
    <skos:broader rdf:resource="thsys/73360"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18829-3">
    <skos:broader rdf:resource="thsys/70872"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18829-3">
    <skos:broader rdf:resource="descriptor/16019-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70648">
    <skos:prefLabel xml:lang="en">P.04.04  Water</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70648">
    <skos:prefLabel xml:lang="de">P.04.04  Wasser</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10877-4">
    <skos:altLabel xml:lang="en">Switch trade</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10877-4">
    <skos:altLabel xml:lang="en">Parallel trade</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10877-4">
    <skos:altLabel xml:lang="en">Counter purchase</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10877-4">
    <skos:altLabel xml:lang="en">Barter trade</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10877-4">
    <skos:altLabel xml:lang="de">Verbundgeschäft im Außenhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10877-4">
    <skos:altLabel xml:lang="de">Switchgeschäft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10877-4">
    <skos:altLabel xml:lang="de">Produktabnahmegeschäft im Außenhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10877-4">
    <skos:altLabel xml:lang="de">Parallelgeschäft im Außenhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10877-4">
    <skos:altLabel xml:lang="de">Offsetgeschäft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10877-4">
    <skos:altLabel xml:lang="de">Koppelungsgeschäft im Außenhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10877-4">
    <skos:altLabel xml:lang="de">Kompensationshandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10877-4">
    <skos:altLabel xml:lang="de">Junktimgeschäft im Außenhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10877-4">
    <skos:altLabel xml:lang="de">Gegengeschäft im Außenhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10877-4">
    <skos:altLabel xml:lang="de">Dreiecksgeschäft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10877-4">
    <skos:altLabel xml:lang="de">Bartergeschäft im Außenhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13971-4">
    <skos:broader rdf:resource="thsys/71055"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13971-4">
    <skos:broader rdf:resource="thsys/70874"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13971-4">
    <skos:broader rdf:resource="descriptor/18535-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12914-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19236-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10622-2">
    <skos:altLabel xml:lang="en">Low wage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10622-2">
    <skos:altLabel xml:lang="en">Low pay</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10622-2">
    <skos:altLabel xml:lang="de">Billiglohn</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10135-6">
    <skos:altLabel xml:lang="en">Prime rate</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10135-6">
    <skos:altLabel xml:lang="de">Zinssatz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10135-6">
    <skos:altLabel xml:lang="de">Nominalzins</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19567-2">
    <skos:related rdf:resource="descriptor/21014-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19567-2">
    <skos:related rdf:resource="descriptor/11706-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19567-2">
    <skos:related rdf:resource="descriptor/11703-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19567-2">
    <skos:related rdf:resource="descriptor/11684-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19223-6">
    <skos:prefLabel xml:lang="en">Ramsey pricing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19223-6">
    <skos:prefLabel xml:lang="de">Ramsey-Preis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11184-2">
    <skos:prefLabel xml:lang="en">Labour mobility</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11184-2">
    <skos:prefLabel xml:lang="de">Arbeitsmobilität</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13746-4">
    <skos:altLabel xml:lang="de">Mindestreserve</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13746-4">
    <skos:altLabel xml:lang="de">Liquiditätsreserve von Banken</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13746-4">
    <skos:altLabel xml:lang="de">Liquidität des Bankensektors</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19632-3">
    <skos:broader rdf:resource="thsys/73351"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19632-3">
    <skos:broader rdf:resource="thsys/71063"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19632-3">
    <skos:broader rdf:resource="thsys/70239"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19632-3">
    <skos:broader rdf:resource="descriptor/15770-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19742-3">
    <skos:related rdf:resource="descriptor/16716-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16974-4">
    <skos:broader rdf:resource="thsys/70003"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16974-4">
    <skos:broader rdf:resource="descriptor/16972-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15520-6">
    <skos:altLabel xml:lang="en">Symmetrisches travelling-salesman-problem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15520-6">
    <skos:altLabel xml:lang="en">M-travelling-salesman problem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15520-6">
    <skos:altLabel xml:lang="de">Rundreiseproblem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15520-6">
    <skos:altLabel xml:lang="de">Handlungsreisendenproblem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15520-6">
    <skos:altLabel xml:lang="de">Asymmetrisches Travelling-Salesman-Problem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19219-4">
    <skos:related rdf:resource="descriptor/19734-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19219-4">
    <skos:related rdf:resource="descriptor/10166-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19219-4">
    <skos:related rdf:resource="descriptor/15860-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19219-4">
    <skos:related rdf:resource="descriptor/16406-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19219-4">
    <skos:related rdf:resource="descriptor/19743-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19219-4">
    <skos:related rdf:resource="descriptor/18832-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18535-6">
    <skos:related rdf:resource="descriptor/18013-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13192-0">
    <skos:broader rdf:resource="thsys/70957"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13192-0">
    <skos:broader rdf:resource="descriptor/13193-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13192-0">
    <skos:broader rdf:resource="descriptor/13200-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12283-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18105-3">
    <skos:prefLabel xml:lang="en">Pension fund</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18105-3">
    <skos:prefLabel xml:lang="de">Pensionsfonds</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19383-5">
    <skos:related rdf:resource="descriptor/14161-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19383-5">
    <skos:related rdf:resource="descriptor/14969-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19383-5">
    <skos:related rdf:resource="descriptor/19382-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19383-5">
    <skos:related rdf:resource="descriptor/14158-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19297-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15956-4">
    <skos:altLabel xml:lang="en">Remigration</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17716-0">
    <skos:altLabel xml:lang="de">Republik Niger</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16539-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11894-6">
    <skos:prefLabel xml:lang="en">Regional population dynamics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11894-6">
    <skos:prefLabel xml:lang="de">Regionale Bevölkerungsentwicklung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13120-4">
    <skos:prefLabel xml:lang="en">Sawmill</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13120-4">
    <skos:prefLabel xml:lang="de">Sägeindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19552-1">
    <skos:related rdf:resource="descriptor/10055-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19552-1">
    <skos:related rdf:resource="descriptor/10171-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70892">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.16</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71067">
    <rdfs:label xml:lang="en">Location</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71067">
    <rdfs:label xml:lang="de">Standort</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12931-0">
    <skos:related rdf:resource="descriptor/20136-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12931-0">
    <skos:related rdf:resource="descriptor/11628-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12931-0">
    <skos:related rdf:resource="descriptor/13303-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12931-0">
    <skos:related rdf:resource="descriptor/20906-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12931-0">
    <skos:related rdf:resource="descriptor/11716-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#broader">
    <rdfs:comment xml:lang="en">By convention, skos:broader is only used to assert an immediate
      (i.e. direct) hierarchical link between two conceptual resources.</rdfs:comment>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#broader">
    <rdfs:comment xml:lang="en">Broader concepts are typically rendered as parents in a concept
      hierarchy (tree).</rdfs:comment>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13018-2">
    <skos:altLabel xml:lang="de">Steinegewinnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13018-2">
    <skos:altLabel xml:lang="de">Steinbruchgewerbe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13018-2">
    <skos:altLabel xml:lang="de">Erdengewinnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12675-6">
    <skos:related rdf:resource="descriptor/10349-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12675-6">
    <skos:related rdf:resource="descriptor/12681-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12675-6">
    <skos:related rdf:resource="descriptor/10297-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12675-6">
    <skos:related rdf:resource="descriptor/19064-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12675-6">
    <skos:related rdf:resource="descriptor/18549-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12675-6">
    <skos:related rdf:resource="descriptor/12677-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19537-4">
    <skos:altLabel xml:lang="en">Beta-factor</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19537-4">
    <skos:altLabel xml:lang="en">Beta factor</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19537-4">
    <skos:altLabel xml:lang="en">Beta estimation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19537-4">
    <skos:altLabel xml:lang="de">Beta-Koeffizient</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18299-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15868-1">
    <skos:related rdf:resource="descriptor/18984-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15868-1">
    <skos:related rdf:resource="descriptor/13529-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15868-1">
    <skos:related rdf:resource="descriptor/12963-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19178-6">
    <skos:broader rdf:resource="thsys/70307"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19178-6">
    <skos:broader rdf:resource="descriptor/17999-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19178-6">
    <skos:broader rdf:resource="descriptor/10499-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17968-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11459-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19975-1">
    <skos:broader rdf:resource="thsys/73376"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13259-1">
    <skos:altLabel xml:lang="de">Seilbahngewerbe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13259-1">
    <skos:altLabel xml:lang="de">Bergbahnverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13259-1">
    <skos:altLabel xml:lang="de">Bergbahngewerbe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13377-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14146-2">
    <skos:related rdf:resource="descriptor/14851-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12088-0">
    <skos:prefLabel xml:lang="en">Corporate planning</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12088-0">
    <skos:prefLabel xml:lang="de">Unternehmensplanung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70648">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">P.04.04</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14145-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70031">
    <skos:narrower rdf:resource="descriptor/14923-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70031">
    <skos:narrower rdf:resource="descriptor/14921-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70031">
    <skos:narrower rdf:resource="descriptor/14920-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70031">
    <skos:narrower rdf:resource="descriptor/14919-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70031">
    <skos:narrower rdf:resource="descriptor/14916-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70031">
    <skos:narrower rdf:resource="descriptor/14915-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70031">
    <skos:narrower rdf:resource="descriptor/14911-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70031">
    <skos:narrower rdf:resource="descriptor/14839-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70031">
    <skos:narrower rdf:resource="descriptor/14909-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70031">
    <skos:narrower rdf:resource="descriptor/14906-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70031">
    <skos:narrower rdf:resource="descriptor/14905-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70031">
    <skos:narrower rdf:resource="descriptor/14902-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70031">
    <skos:narrower rdf:resource="descriptor/14900-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70031">
    <skos:narrower rdf:resource="descriptor/14899-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18613-5">
    <skos:related rdf:resource="descriptor/15763-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18613-5">
    <skos:related rdf:resource="descriptor/19016-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18613-5">
    <skos:related rdf:resource="descriptor/13617-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18613-5">
    <skos:related rdf:resource="descriptor/15992-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13988-1">
    <skos:narrower rdf:resource="descriptor/11311-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10926-3">
    <skos:broader rdf:resource="thsys/70212"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10926-3">
    <skos:broader rdf:resource="descriptor/10942-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10926-3">
    <skos:broader rdf:resource="descriptor/10213-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13097-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19438-6">
    <skos:narrower rdf:resource="descriptor/11911-4"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id151">
    <rdfs:seeAlso rdf:resource="descriptor/19325-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17959-2">
    <skos:altLabel xml:lang="de">Indik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18182-6">
    <skos:broader rdf:resource="thsys/73365"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18182-6">
    <skos:broader rdf:resource="thsys/71110"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18182-6">
    <skos:broader rdf:resource="thsys/70982"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18182-6">
    <skos:broader rdf:resource="descriptor/16445-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16246-2">
    <skos:narrower rdf:resource="descriptor/11067-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15744-5">
    <skos:broader rdf:resource="thsys/70238"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15744-5">
    <skos:broader rdf:resource="descriptor/15733-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16366-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70041">
    <skos:broader rdf:resource="thsys/b"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19690-3">
    <skos:broader rdf:resource="thsys/73381"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19615-3">
    <skos:related rdf:resource="descriptor/19619-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10265-0">
    <skos:altLabel xml:lang="en">Gross national product</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10265-0">
    <skos:altLabel xml:lang="en">Gross domestic product</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10265-0">
    <skos:altLabel xml:lang="en">GNP (Gross National Product)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10265-0">
    <skos:altLabel xml:lang="en">GDP (Gross Domestic Product)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10265-0">
    <skos:altLabel xml:lang="de">Prokopfeinkommen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10265-0">
    <skos:altLabel xml:lang="de">Pro-Kopf-Einkommen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10265-0">
    <skos:altLabel xml:lang="de">Inlandsprodukt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10265-0">
    <skos:altLabel xml:lang="de">BSP (Bruttosozialprodukt)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10265-0">
    <skos:altLabel xml:lang="de">Bruttosozialprodukt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10265-0">
    <skos:altLabel xml:lang="de">Bruttonationaleinkommen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10265-0">
    <skos:altLabel xml:lang="de">Bruttoinlandsprodukt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10265-0">
    <skos:altLabel xml:lang="de">BIP (Bruttoinlandsprodukt)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10265-0">
    <skos:altLabel xml:lang="de">Bilanz des gesellschaftlichen Gesamtprodukts</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14711-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18712-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11807-6">
    <skos:narrower rdf:resource="descriptor/18341-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19266-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13738-3">
    <skos:broader rdf:resource="thsys/71105"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13738-3">
    <skos:broader rdf:resource="descriptor/13723-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10521-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16366-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19432-4">
    <skos:prefLabel xml:lang="en">Sunk costs</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19432-4">
    <skos:prefLabel xml:lang="de">Sunk Costs</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11657-6">
    <skos:related rdf:resource="descriptor/11652-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11657-6">
    <skos:related rdf:resource="descriptor/19418-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16361-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11419-1">
    <skos:broader rdf:resource="thsys/73334"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11419-1">
    <skos:broader rdf:resource="thsys/70055"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11419-1">
    <skos:broader rdf:resource="descriptor/16195-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13825-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14513-1">
    <skos:altLabel xml:lang="en">Compact disc</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14513-1">
    <skos:altLabel xml:lang="en">CD (Compact Disc)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14513-1">
    <skos:altLabel xml:lang="de">Tonträger</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14513-1">
    <skos:altLabel xml:lang="de">Tonband</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14513-1">
    <skos:altLabel xml:lang="de">Schallplatte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14513-1">
    <skos:altLabel xml:lang="de">Musikkassette</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14513-1">
    <skos:altLabel xml:lang="de">MusiCassette</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14513-1">
    <skos:altLabel xml:lang="de">Bespielter Bild- und Tonträger</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14513-1">
    <skos:altLabel xml:lang="de">Audiokassette</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17785-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id299">
    <rdf:value xml:lang="de">Für das Spargeschäft von Kreditinstituten</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10513-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10202-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12281-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11434-5">
    <skos:prefLabel xml:lang="en">Coins</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11434-5">
    <skos:prefLabel xml:lang="de">Münzen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73387">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18176-1">
    <skos:related rdf:resource="descriptor/13392-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18176-1">
    <skos:related rdf:resource="descriptor/20110-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18176-1">
    <skos:related rdf:resource="descriptor/18066-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18176-1">
    <skos:related rdf:resource="descriptor/19217-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18176-1">
    <skos:related rdf:resource="descriptor/11040-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11174-5">
    <skos:altLabel xml:lang="en">Radical political economics</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11174-5">
    <skos:altLabel xml:lang="en">New left economics</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11174-5">
    <skos:altLabel xml:lang="de">Radikale Schule</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12502-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11494-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11803-0">
    <skos:prefLabel xml:lang="en">Agricultural structure</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11803-0">
    <skos:prefLabel xml:lang="de">Agrarstruktur</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16815-3">
    <skos:prefLabel xml:lang="en">Europe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16815-3">
    <skos:prefLabel xml:lang="de">Europa</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13252-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12759-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73374">
    <skos:narrower rdf:resource="thsys/73368"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73374">
    <skos:narrower rdf:resource="thsys/73373"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73374">
    <skos:narrower rdf:resource="thsys/73370"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73374">
    <skos:narrower rdf:resource="thsys/73369"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73374">
    <skos:narrower rdf:resource="thsys/73372"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73374">
    <skos:narrower rdf:resource="thsys/73371"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73374">
    <skos:narrower rdf:resource="thsys/73367"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12401-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10558-6">
    <skos:prefLabel xml:lang="en">Development organization</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10558-6">
    <skos:prefLabel xml:lang="de">Entwicklungsorganisation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12716-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10060-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12069-4">
    <skos:broader rdf:resource="thsys/71034"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19975-1">
    <skos:altLabel xml:lang="en">Nepotism</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19975-1">
    <skos:altLabel xml:lang="en">Cronyism</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19975-1">
    <skos:altLabel xml:lang="de">Vetternwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19975-1">
    <skos:altLabel xml:lang="de">Seilschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19975-1">
    <skos:altLabel xml:lang="de">Politische Protektion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19975-1">
    <skos:altLabel xml:lang="de">Politische Klientel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19975-1">
    <skos:altLabel xml:lang="de">Nepotismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19975-1">
    <skos:altLabel xml:lang="de">Klüngel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19975-1">
    <skos:altLabel xml:lang="de">Klientelpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19975-1">
    <skos:altLabel xml:lang="de">Klientelismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19975-1">
    <skos:altLabel xml:lang="de">Günstlingswirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19975-1">
    <skos:altLabel xml:lang="de">Ämterpatronage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10348-3">
    <skos:scopeNote xml:lang="de">Erklärung plötzlicher, tiefgreifender und umfassender Einbrüche in der gesamtwirtschaftlichen Tätigkeit. Vorläufer der traditionellen Konjunkturtheorie.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13860-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18134-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19695-0">
    <skos:broader rdf:resource="thsys/73378"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19695-0">
    <skos:broader rdf:resource="descriptor/12111-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14368-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19583-4">
    <skos:broader rdf:resource="thsys/70269"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19583-4">
    <skos:broader rdf:resource="descriptor/15026-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14130-3">
    <skos:related rdf:resource="descriptor/19349-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14130-3">
    <skos:related rdf:resource="descriptor/13149-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14130-3">
    <skos:related rdf:resource="descriptor/12952-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14130-3">
    <skos:related rdf:resource="descriptor/14143-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73346">
    <skos:narrower rdf:resource="descriptor/15679-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73346">
    <skos:narrower rdf:resource="descriptor/19793-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73346">
    <skos:narrower rdf:resource="descriptor/18901-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73346">
    <skos:narrower rdf:resource="descriptor/16676-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73346">
    <skos:narrower rdf:resource="descriptor/19525-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73346">
    <skos:narrower rdf:resource="descriptor/18900-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73346">
    <skos:narrower rdf:resource="descriptor/19013-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73346">
    <skos:narrower rdf:resource="descriptor/19792-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73346">
    <skos:narrower rdf:resource="descriptor/18899-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73346">
    <skos:narrower rdf:resource="descriptor/16670-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73346">
    <skos:narrower rdf:resource="descriptor/18896-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73346">
    <skos:narrower rdf:resource="descriptor/18897-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73346">
    <skos:narrower rdf:resource="descriptor/19791-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73346">
    <skos:narrower rdf:resource="descriptor/18444-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73346">
    <skos:narrower rdf:resource="descriptor/19790-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73346">
    <skos:narrower rdf:resource="descriptor/16668-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73346">
    <skos:narrower rdf:resource="descriptor/19789-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73346">
    <skos:narrower rdf:resource="descriptor/12618-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73346">
    <skos:narrower rdf:resource="descriptor/16666-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73346">
    <skos:narrower rdf:resource="descriptor/18898-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12824-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10064-3">
    <skos:related rdf:resource="descriptor/16214-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10064-3">
    <skos:related rdf:resource="descriptor/10031-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10064-3">
    <skos:related rdf:resource="descriptor/10624-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10064-3">
    <skos:related rdf:resource="descriptor/10434-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10064-3">
    <skos:related rdf:resource="descriptor/10253-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13618-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70880">
    <skos:prefLabel xml:lang="de">W.19.02  Datenbanken</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24781-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18851-3">
    <skos:prefLabel xml:lang="en">Chamber of agriculture</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18851-3">
    <skos:prefLabel xml:lang="de">Landwirtschaftskammer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13980-3">
    <skos:altLabel xml:lang="de">Pilotinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13980-3">
    <skos:altLabel xml:lang="de">Piloten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13980-3">
    <skos:altLabel xml:lang="de">Flugsicherungsberufe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13980-3">
    <skos:altLabel xml:lang="de">Flugingenieurinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13980-3">
    <skos:altLabel xml:lang="de">Flugingenieure</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13072-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18647-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15876-2">
    <skos:related rdf:resource="descriptor/15669-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18741-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19495-1">
    <skos:narrower rdf:resource="descriptor/18150-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19361-1">
    <skos:related rdf:resource="descriptor/16215-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19361-1">
    <skos:related rdf:resource="descriptor/20842-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19361-1">
    <skos:related rdf:resource="descriptor/10525-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19361-1">
    <skos:related rdf:resource="descriptor/10492-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19361-1">
    <skos:related rdf:resource="descriptor/10513-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70307">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">G.06</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19379-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12408-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id304">
    <rdfs:seeAlso rdf:resource="descriptor/19476-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70892">
    <skos:broader rdf:resource="thsys/w"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10809-0">
    <skos:broader rdf:resource="thsys/71108"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10809-0">
    <skos:broader rdf:resource="thsys/71089"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10809-0">
    <skos:broader rdf:resource="descriptor/10807-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10809-0">
    <skos:broader rdf:resource="descriptor/11421-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13158-0">
    <skos:prefLabel xml:lang="en">Dairy industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13158-0">
    <skos:prefLabel xml:lang="de">Milchverarbeitung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19681-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11293-4">
    <skos:broader rdf:resource="thsys/73355"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11293-4">
    <skos:broader rdf:resource="thsys/71058"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11293-4">
    <skos:broader rdf:resource="thsys/70878"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11293-4">
    <skos:broader rdf:resource="descriptor/16554-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18223-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12381-2">
    <skos:related rdf:resource="descriptor/12898-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12381-2">
    <skos:related rdf:resource="descriptor/12337-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12381-2">
    <skos:related rdf:resource="descriptor/12326-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12381-2">
    <skos:related rdf:resource="descriptor/12217-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13814-6">
    <skos:altLabel xml:lang="de">Luftverkehrsversicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19152-3">
    <skos:related rdf:resource="descriptor/10463-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19152-3">
    <skos:related rdf:resource="descriptor/19116-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19152-3">
    <skos:related rdf:resource="descriptor/19096-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11857-5">
    <skos:related rdf:resource="descriptor/11879-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11857-5">
    <skos:related rdf:resource="descriptor/11875-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11857-5">
    <skos:related rdf:resource="descriptor/11962-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11857-5">
    <skos:related rdf:resource="descriptor/16552-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11857-5">
    <skos:related rdf:resource="descriptor/11900-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11857-5">
    <skos:related rdf:resource="descriptor/11912-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11857-5">
    <skos:related rdf:resource="descriptor/11951-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11857-5">
    <skos:related rdf:resource="descriptor/11895-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11857-5">
    <skos:related rdf:resource="descriptor/11869-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/b">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18011-5">
    <skos:altLabel xml:lang="de">Liberalisierung regulierter Märkte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18011-5">
    <skos:altLabel xml:lang="de">Liberalisierung des Wettbewerbs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13636-4">
    <skos:altLabel xml:lang="de">Jugendherberge</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13636-4">
    <skos:altLabel xml:lang="de">Campingurlaub</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13636-4">
    <skos:altLabel xml:lang="de">Campingplatz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14749-0">
    <skos:altLabel xml:lang="de">Tintenstrahldrucker</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14749-0">
    <skos:altLabel xml:lang="de">Matrixdrucker</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14749-0">
    <skos:altLabel xml:lang="de">Laserdrucker</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14749-0">
    <skos:altLabel xml:lang="de">Farbdrucker</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18390-6">
    <skos:narrower rdf:resource="descriptor/18393-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12700-5">
    <skos:narrower rdf:resource="descriptor/10211-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17703-2">
    <skos:prefLabel xml:lang="en">Gambia</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17703-2">
    <skos:prefLabel xml:lang="de">Gambia</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10414-2">
    <skos:broader rdf:resource="thsys/70406"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10414-2">
    <skos:broader rdf:resource="descriptor/10993-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14915-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12643-5">
    <skos:related rdf:resource="descriptor/18620-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12643-5">
    <skos:related rdf:resource="descriptor/19116-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12643-5">
    <skos:related rdf:resource="descriptor/12716-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10034-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17954-5">
    <skos:altLabel xml:lang="de">Atlantik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15961-4">
    <skos:broader rdf:resource="thsys/73376"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15961-4">
    <skos:broader rdf:resource="thsys/70823"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15961-4">
    <skos:broader rdf:resource="thsys/70160"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15961-4">
    <skos:broader rdf:resource="descriptor/16007-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12891-0">
    <skos:prefLabel xml:lang="en">Internal auditing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12891-0">
    <skos:prefLabel xml:lang="de">Interne Revision</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11779-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16059-1">
    <skos:related rdf:resource="descriptor/10621-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16587-4">
    <skos:scopeNote xml:lang="de">Kennzeichnung "primärer sozialer Verbundenheit" im Gegensatz zu "Gesellschaft" als "Zweckverband". Untersuchungen zur Struktur sozialer Einheiten, die auf Grund eines gemeinsamen Fühlens und Erlebens entstanden sind (Familie, Dorfgemeinschaft, Nachbarschaft, etc.) unter Ausschluss von Fremden.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18649-5">
    <skos:prefLabel xml:lang="en">Telecommunication company</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18649-5">
    <skos:prefLabel xml:lang="de">Telekommunikationsgesellschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13124-3">
    <skos:prefLabel xml:lang="en">Newspaper printing company</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13124-3">
    <skos:prefLabel xml:lang="de">Zeitungsdruckerei</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19045-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18591-3">
    <skos:related rdf:resource="descriptor/16187-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18591-3">
    <skos:related rdf:resource="descriptor/13545-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18591-3">
    <skos:related rdf:resource="descriptor/13256-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18591-3">
    <skos:related rdf:resource="descriptor/13252-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14141-5">
    <skos:broader rdf:resource="thsys/70403"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14141-5">
    <skos:broader rdf:resource="descriptor/14145-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18246-6">
    <skos:prefLabel xml:lang="en">Business aircraft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18246-6">
    <skos:prefLabel xml:lang="de">Geschäftsflugzeug</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70929">
    <skos:prefLabel xml:lang="en">W.12.04  Transport research</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70929">
    <skos:prefLabel xml:lang="de">W.12.04  Verkehrswissenschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12716-4">
    <skos:altLabel xml:lang="en">Make-to-order</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12716-4">
    <skos:altLabel xml:lang="en">Make to order</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12716-4">
    <skos:altLabel xml:lang="de">Kundenfertigung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12716-4">
    <skos:altLabel xml:lang="de">Kundenauftragsfertigung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11069-2">
    <skos:altLabel xml:lang="de">Wirtschaftsverfassungsrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11069-2">
    <skos:altLabel xml:lang="de">Wirtschaftsordnungsrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11069-2">
    <skos:altLabel xml:lang="de">Marktverfassung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19845-0">
    <zbwext:useInsteadNote rdf:nodeID="id34"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12208-2">
    <skos:altLabel xml:lang="en">Capital expenditure account</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12208-2">
    <skos:altLabel xml:lang="de">Rentabilitätsvergleichsrechnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12208-2">
    <skos:altLabel xml:lang="de">Pay-off-Periode</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12208-2">
    <skos:altLabel xml:lang="de">Kostenvergleichsrechnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12208-2">
    <skos:altLabel xml:lang="de">Investitionsbewertung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12208-2">
    <skos:altLabel xml:lang="de">Gewinnvergleichsrechnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12208-2">
    <skos:altLabel xml:lang="de">Bewertung von Investitionen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12208-2">
    <skos:altLabel xml:lang="de">Amortisationsrechnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17568-3">
    <skos:prefLabel xml:lang="en">Japan</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17568-3">
    <skos:prefLabel xml:lang="de">Japan</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10789-1">
    <skos:prefLabel xml:lang="en">Clearing system</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10789-1">
    <skos:prefLabel xml:lang="de">Verrechnungsverkehr</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19549-4">
    <skos:prefLabel xml:lang="en">Automotive market</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19549-4">
    <skos:prefLabel xml:lang="de">Kraftfahrzeugmarkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70292">
    <skos:broader rdf:resource="thsys/70041"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19687-6">
    <skos:broader rdf:resource="thsys/73381"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19687-6">
    <skos:broader rdf:resource="thsys/73361"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19687-6">
    <skos:broader rdf:resource="descriptor/16017-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11652-2">
    <skos:related rdf:resource="descriptor/12372-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11652-2">
    <skos:related rdf:resource="descriptor/11657-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/b">
    <skos:prefLabel xml:lang="en">B  Business economics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/b">
    <skos:prefLabel xml:lang="de">B  Betriebswirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18220-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19563-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11545-3">
    <skos:prefLabel xml:lang="en">Public Revenues</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11545-3">
    <skos:prefLabel xml:lang="de">Öffentliche Einnahmen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14849-3">
    <skos:broader rdf:resource="thsys/70201"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16314-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10751-5">
    <skos:altLabel xml:lang="en">Two-tier exchange rate</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10751-5">
    <skos:altLabel xml:lang="de">Gespaltener Wechselkurs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10751-5">
    <skos:altLabel xml:lang="de">Gespaltener Devisenmarkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10751-5">
    <skos:altLabel xml:lang="de">Gespaltener Devisenkurs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10751-5">
    <skos:altLabel xml:lang="de">Devisenmarktspaltung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70333">
    <skos:narrower rdf:resource="descriptor/14051-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70333">
    <skos:narrower rdf:resource="descriptor/18307-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70333">
    <skos:narrower rdf:resource="descriptor/14804-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70333">
    <skos:narrower rdf:resource="descriptor/18306-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70333">
    <skos:narrower rdf:resource="descriptor/14549-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70333">
    <skos:narrower rdf:resource="descriptor/18273-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70333">
    <skos:narrower rdf:resource="descriptor/18304-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70333">
    <skos:narrower rdf:resource="descriptor/14821-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70333">
    <skos:narrower rdf:resource="descriptor/14778-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70333">
    <skos:narrower rdf:resource="descriptor/18305-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70333">
    <skos:narrower rdf:resource="descriptor/14027-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11965-2">
    <skos:prefLabel xml:lang="en">Community statistics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11965-2">
    <skos:prefLabel xml:lang="de">Gemeindestatistik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13066-5">
    <skos:narrower rdf:resource="descriptor/18416-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19435-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#closeMatch">
    <rdfs:label xml:lang="en">close match</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12438-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14323-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19822-5">
    <skos:related rdf:resource="descriptor/19607-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19822-5">
    <skos:related rdf:resource="descriptor/19705-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19822-5">
    <skos:related rdf:resource="descriptor/16186-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19445-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16361-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11833-5">
    <skos:prefLabel xml:lang="en">Transport policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11833-5">
    <skos:prefLabel xml:lang="de">Verkehrspolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10124-4">
    <skos:altLabel xml:lang="de">Regionaler Kapitalverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10124-4">
    <skos:altLabel xml:lang="de">Regionale Kapitalmobilität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10124-4">
    <skos:altLabel xml:lang="de">Interregionale Kapitalanlage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19667-5">
    <skos:prefLabel xml:lang="en">Market exit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19667-5">
    <skos:prefLabel xml:lang="de">Marktaustritt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13069-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12711-0">
    <skos:broader rdf:resource="thsys/73345"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12711-0">
    <skos:broader rdf:resource="thsys/70310"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12711-0">
    <skos:broader rdf:resource="descriptor/10470-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12711-0">
    <skos:broader rdf:resource="descriptor/12725-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14098-1">
    <skos:altLabel xml:lang="en">Durum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18089-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18895-4">
    <skos:related rdf:resource="descriptor/19465-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18895-4">
    <skos:related rdf:resource="descriptor/16556-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18895-4">
    <skos:related rdf:resource="descriptor/18894-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19392-4">
    <skos:broader rdf:resource="thsys/70845"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19392-4">
    <skos:broader rdf:resource="descriptor/11377-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19392-4">
    <skos:broader rdf:resource="descriptor/12064-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17783-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18428-0">
    <skos:altLabel xml:lang="de">Nachrichtentechnische Industrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18428-0">
    <skos:altLabel xml:lang="de">Fernmeldeindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15889-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10695-3">
    <skos:prefLabel xml:lang="en">Free port</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10695-3">
    <skos:prefLabel xml:lang="de">Freihafen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70241">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11088-5">
    <skos:prefLabel xml:lang="en">Marxism</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11088-5">
    <skos:prefLabel xml:lang="de">Marxismus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17919-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19422-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18746-0">
    <skos:altLabel xml:lang="de">Technischer Gebäudeschutz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18746-0">
    <skos:altLabel xml:lang="de">Integrales Gebäudemanagement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18746-0">
    <skos:altLabel xml:lang="de">Haustechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18746-0">
    <skos:altLabel xml:lang="de">Gebäudetechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18746-0">
    <skos:altLabel xml:lang="de">Gebäudeautomatisierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18167-2">
    <skos:altLabel xml:lang="de">Zigarrenindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12735-0">
    <skos:altLabel xml:lang="en">Flexible manufacturing system</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12735-0">
    <skos:altLabel xml:lang="de">Flexibles Produktionssystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12735-0">
    <skos:altLabel xml:lang="de">Flexible Automatisierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11629-4">
    <skos:broader rdf:resource="thsys/71072"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11629-4">
    <skos:broader rdf:resource="thsys/70943"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11629-4">
    <skos:broader rdf:resource="thsys/70860"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11629-4">
    <skos:broader rdf:resource="descriptor/12931-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11629-4">
    <skos:broader rdf:resource="descriptor/13264-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11629-4">
    <skos:broader rdf:resource="descriptor/11628-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12720-6">
    <skos:prefLabel xml:lang="en">Cellular manufacturing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12720-6">
    <skos:prefLabel xml:lang="de">Fertigungsinsel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11261-3">
    <skos:broader rdf:resource="thsys/70052"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11261-3">
    <skos:broader rdf:resource="descriptor/11262-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11261-3">
    <skos:broader rdf:resource="descriptor/15904-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16417-1">
    <skos:prefLabel xml:lang="en">Neutrality</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16417-1">
    <skos:prefLabel xml:lang="de">Neutralität</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19625-0">
    <skos:related rdf:resource="descriptor/19630-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19625-0">
    <skos:related rdf:resource="descriptor/19629-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19625-0">
    <skos:related rdf:resource="descriptor/15635-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10842-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18029-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11875-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10807-4">
    <skos:narrower rdf:resource="descriptor/10817-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10807-4">
    <skos:narrower rdf:resource="descriptor/10809-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10807-4">
    <skos:narrower rdf:resource="descriptor/10801-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10807-4">
    <skos:narrower rdf:resource="descriptor/10779-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#definition">
    <rdfs:label xml:lang="en">definition</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12228-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16737-4">
    <skos:related rdf:resource="descriptor/16743-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16737-4">
    <skos:related rdf:resource="descriptor/16740-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16737-4">
    <skos:related rdf:resource="descriptor/10252-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16737-4">
    <skos:related rdf:resource="descriptor/16738-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17017-5">
    <skos:related rdf:resource="descriptor/13393-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17017-5">
    <skos:related rdf:resource="descriptor/17039-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17017-5">
    <skos:related rdf:resource="descriptor/15640-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16072-2">
    <skos:altLabel xml:lang="de">Sparkassengesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19470-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10287-4">
    <skos:broader rdf:resource="thsys/70530"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10287-4">
    <skos:broader rdf:resource="descriptor/10008-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16233-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10897-5">
    <skos:scopeNote xml:lang="de">Vereinigung von selbständigen Unternehmen zu einer rechtlichen und wirtschaftlichen Einheit.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13427-6">
    <skos:related rdf:resource="descriptor/19397-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13427-6">
    <skos:related rdf:resource="descriptor/12997-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13427-6">
    <skos:related rdf:resource="descriptor/18516-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13427-6">
    <skos:related rdf:resource="descriptor/12024-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70394">
    <skos:prefLabel xml:lang="en">B.05.02  Logistics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70394">
    <skos:prefLabel xml:lang="de">B.05.02  Logistik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19466-1">
    <skos:altLabel xml:lang="de">Tourismussoziologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19466-1">
    <skos:altLabel xml:lang="de">Fremdenverkehrsforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18029-0">
    <skos:related rdf:resource="descriptor/12158-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18029-0">
    <skos:related rdf:resource="descriptor/18184-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18029-0">
    <skos:related rdf:resource="descriptor/12083-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18029-0">
    <skos:related rdf:resource="descriptor/12212-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18029-0">
    <skos:related rdf:resource="descriptor/18106-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18029-0">
    <skos:related rdf:resource="descriptor/10192-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71006">
    <skos:broader rdf:resource="thsys/71009"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24666-5">
    <skos:prefLabel xml:lang="en">Fair value</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24666-5">
    <skos:prefLabel xml:lang="de">Fair-Value-Bewertung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18131-2">
    <skos:related rdf:resource="descriptor/16927-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13425-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13339-3">
    <skos:altLabel xml:lang="de">Rauchfangkehrer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13339-3">
    <skos:altLabel xml:lang="de">Kaminkehrer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13984-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11701-1">
    <skos:prefLabel xml:lang="en">Regional tax preference</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11701-1">
    <skos:prefLabel xml:lang="de">Regionale Steuerpräferenz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17623-0">
    <skos:prefLabel xml:lang="en">Israel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17623-0">
    <skos:prefLabel xml:lang="de">Israel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18867-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13249-4">
    <skos:altLabel xml:lang="de">ÖPNV</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13249-4">
    <skos:altLabel xml:lang="de">Öffentlicher Nahverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18755-6">
    <skos:broader rdf:resource="thsys/70884"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18755-6">
    <skos:broader rdf:resource="descriptor/11847-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16952-0">
    <skos:prefLabel xml:lang="en">Styria</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16952-0">
    <skos:prefLabel xml:lang="de">Steiermark</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18681-2">
    <skos:related rdf:resource="descriptor/12230-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18681-2">
    <skos:related rdf:resource="descriptor/12299-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18681-2">
    <skos:related rdf:resource="descriptor/18678-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18681-2">
    <skos:related rdf:resource="descriptor/12281-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18681-2">
    <skos:related rdf:resource="descriptor/13704-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18162-5">
    <skos:altLabel xml:lang="en">Minor metals</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18162-5">
    <skos:altLabel xml:lang="de">Strategisches Metall</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18162-5">
    <skos:altLabel xml:lang="de">Niobium</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18162-5">
    <skos:altLabel xml:lang="de">Niob</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18162-5">
    <skos:altLabel xml:lang="de">Indium</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18162-5">
    <skos:altLabel xml:lang="de">Germanium</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18162-5">
    <skos:altLabel xml:lang="de">Gallium</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18162-5">
    <skos:altLabel xml:lang="de">Beryllium</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70921">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18729-0">
    <skos:scopeNote xml:lang="de">Geschichtliche Darstellungen des Börsenwesens und der Finanzmärkte allgemein und in einzelnen Regionen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18001-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13572-4">
    <skos:narrower rdf:resource="descriptor/18581-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13572-4">
    <skos:narrower rdf:resource="descriptor/18594-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70076">
    <skos:narrower rdf:resource="descriptor/17642-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70076">
    <skos:narrower rdf:resource="thsys/70085"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70076">
    <skos:narrower rdf:resource="thsys/70354"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19701-3">
    <skos:related rdf:resource="descriptor/16357-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17784-4">
    <skos:broader rdf:resource="thsys/70086"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17784-4">
    <skos:broader rdf:resource="descriptor/17791-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19575-3">
    <skos:altLabel xml:lang="en">Augmented Dickey-Fuller-test</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19575-3">
    <skos:altLabel xml:lang="de">Phillips-Test</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19575-3">
    <skos:altLabel xml:lang="de">Phillips-Perron-Test</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19575-3">
    <skos:altLabel xml:lang="de">Einheitswurzeltest</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19575-3">
    <skos:altLabel xml:lang="de">Dickey-Fuller-Test</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19575-3">
    <skos:altLabel xml:lang="de">DF Test</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19575-3">
    <skos:altLabel xml:lang="de">ADF-Test</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17937-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16058-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12051-2">
    <skos:prefLabel xml:lang="en">Engineering economics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12051-2">
    <skos:prefLabel xml:lang="de">Industriebetriebslehre</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10141-4">
    <skos:prefLabel xml:lang="en">Economics of information</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10141-4">
    <skos:prefLabel xml:lang="de">Informationsökonomik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15727-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18326-1">
    <skos:altLabel xml:lang="de">Bewässerungslandbau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18326-1">
    <skos:altLabel xml:lang="de">Beregnungslandwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11262-1">
    <skos:narrower rdf:resource="descriptor/11284-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11262-1">
    <skos:narrower rdf:resource="descriptor/11282-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11262-1">
    <skos:narrower rdf:resource="descriptor/18083-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11262-1">
    <skos:narrower rdf:resource="descriptor/11258-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11262-1">
    <skos:narrower rdf:resource="descriptor/11277-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11262-1">
    <skos:narrower rdf:resource="descriptor/11275-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11262-1">
    <skos:narrower rdf:resource="descriptor/11274-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11262-1">
    <skos:narrower rdf:resource="descriptor/11269-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11262-1">
    <skos:narrower rdf:resource="descriptor/11289-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11262-1">
    <skos:narrower rdf:resource="descriptor/11268-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11262-1">
    <skos:narrower rdf:resource="descriptor/11286-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11262-1">
    <skos:narrower rdf:resource="descriptor/11285-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11262-1">
    <skos:narrower rdf:resource="descriptor/11261-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13224-6">
    <skos:related rdf:resource="descriptor/13190-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13224-6">
    <skos:related rdf:resource="descriptor/19184-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13224-6">
    <skos:related rdf:resource="descriptor/13034-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13224-6">
    <skos:related rdf:resource="descriptor/14266-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18878-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16010-3">
    <skos:related rdf:resource="descriptor/12847-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16010-3">
    <skos:related rdf:resource="descriptor/18449-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16010-3">
    <skos:related rdf:resource="descriptor/18211-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16010-3">
    <skos:related rdf:resource="descriptor/18768-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16010-3">
    <skos:related rdf:resource="descriptor/13662-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11406-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11297-3">
    <skos:altLabel xml:lang="de">Berufszählung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70066">
    <skos:narrower rdf:resource="descriptor/14837-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70066">
    <skos:narrower rdf:resource="descriptor/14840-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70066">
    <skos:narrower rdf:resource="descriptor/14846-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70066">
    <skos:narrower rdf:resource="descriptor/14897-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70066">
    <skos:narrower rdf:resource="descriptor/14889-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70066">
    <skos:narrower rdf:resource="descriptor/14888-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70066">
    <skos:narrower rdf:resource="descriptor/14844-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70066">
    <skos:narrower rdf:resource="descriptor/14914-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70066">
    <skos:narrower rdf:resource="descriptor/14895-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70066">
    <skos:narrower rdf:resource="descriptor/14885-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70066">
    <skos:narrower rdf:resource="thsys/73304"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73345">
    <rdfs:label xml:lang="de">Technik und Ingenieurwissenschaften</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14065-2">
    <skos:altLabel xml:lang="de">Kakaoprodukt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15981-5">
    <skos:related rdf:resource="descriptor/15972-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15981-5">
    <skos:related rdf:resource="descriptor/19849-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13784-3">
    <skos:narrower rdf:resource="descriptor/13790-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16470-4">
    <zbwext:useInsteadNote rdf:nodeID="id178"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15448-2">
    <skos:broader rdf:resource="thsys/73104"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15448-2">
    <skos:broader rdf:resource="descriptor/15458-6"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id9">
    <rdfs:seeAlso rdf:resource="descriptor/19100-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15408-0">
    <skos:prefLabel xml:lang="en">Logit model</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15408-0">
    <skos:prefLabel xml:lang="de">Logit-Modell</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12783-3">
    <skos:altLabel xml:lang="en">Trade show</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12783-3">
    <skos:altLabel xml:lang="de">Verkaufsausstellung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12783-3">
    <skos:altLabel xml:lang="de">Universalmesse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12783-3">
    <skos:altLabel xml:lang="de">Messewesen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12783-3">
    <skos:altLabel xml:lang="de">Gewerbliche Ausstellung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12783-3">
    <skos:altLabel xml:lang="de">Fachmesse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18572-0">
    <skos:prefLabel xml:lang="en">Railroad company</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18572-0">
    <skos:prefLabel xml:lang="de">Eisenbahngesellschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71080">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13849-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70169">
    <rdfs:label xml:lang="de">Gartenbauprodukte</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17553-2">
    <skos:scopeNote xml:lang="de">Einschließlich Türkei</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13407-5">
    <skos:related rdf:resource="descriptor/10882-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13407-5">
    <skos:related rdf:resource="descriptor/16493-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18318-0">
    <skos:broader rdf:resource="thsys/71028"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18318-0">
    <skos:broader rdf:resource="descriptor/10032-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18318-0">
    <skos:broader rdf:resource="descriptor/13446-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19689-2">
    <skos:narrower rdf:resource="descriptor/13422-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19689-2">
    <skos:narrower rdf:resource="descriptor/16333-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19689-2">
    <skos:narrower rdf:resource="descriptor/16329-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19689-2">
    <skos:narrower rdf:resource="descriptor/16340-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16026-2">
    <skos:narrower rdf:resource="descriptor/16036-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16026-2">
    <skos:narrower rdf:resource="descriptor/16035-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16026-2">
    <skos:narrower rdf:resource="descriptor/16030-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16026-2">
    <skos:narrower rdf:resource="descriptor/16029-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16026-2">
    <skos:narrower rdf:resource="descriptor/18433-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18105-3">
    <skos:broader rdf:resource="thsys/70891"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18105-3">
    <skos:broader rdf:resource="thsys/70832"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18105-3">
    <skos:broader rdf:resource="thsys/70458"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18105-3">
    <skos:broader rdf:resource="thsys/70245"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18105-3">
    <skos:broader rdf:resource="descriptor/13294-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18105-3">
    <skos:broader rdf:resource="descriptor/12591-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19506-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16332-2">
    <skos:related rdf:resource="descriptor/16295-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16332-2">
    <skos:related rdf:resource="descriptor/16013-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16332-2">
    <skos:related rdf:resource="descriptor/19855-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16332-2">
    <skos:related rdf:resource="descriptor/16286-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16332-2">
    <skos:related rdf:resource="descriptor/16395-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16332-2">
    <skos:related rdf:resource="descriptor/16521-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16332-2">
    <skos:related rdf:resource="descriptor/16510-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16332-2">
    <skos:related rdf:resource="descriptor/16509-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16332-2">
    <skos:related rdf:resource="descriptor/19847-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18361-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70113">
    <skos:prefLabel xml:lang="en">B.04  Human resource management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70113">
    <skos:prefLabel xml:lang="de">B.04  Personalwirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13405-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10614-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18577-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17485-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19815-2">
    <skos:altLabel xml:lang="de">Geldwäschegesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19309-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11483-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13953-6">
    <skos:broader rdf:resource="thsys/71056"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13953-6">
    <skos:broader rdf:resource="thsys/70970"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15872-3">
    <skos:altLabel xml:lang="de">Physische Geographie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15872-3">
    <skos:altLabel xml:lang="de">Mathematische Geographie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15872-3">
    <skos:altLabel xml:lang="de">Landschaftskunde</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15872-3">
    <skos:altLabel xml:lang="de">Erdkunde</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12029-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18948-2">
    <skos:related rdf:resource="descriptor/13398-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18948-2">
    <skos:related rdf:resource="descriptor/16639-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18948-2">
    <skos:related rdf:resource="descriptor/11660-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13693-6">
    <skos:broader rdf:resource="thsys/73390"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13693-6">
    <skos:broader rdf:resource="thsys/70905"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13693-6">
    <skos:broader rdf:resource="descriptor/16081-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13666-2">
    <skos:prefLabel xml:lang="en">Teleconferencing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13666-2">
    <skos:prefLabel xml:lang="de">Telekonferenz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13684-0">
    <skos:broader rdf:resource="thsys/70931"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13684-0">
    <skos:broader rdf:resource="thsys/70922"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13684-0">
    <skos:broader rdf:resource="descriptor/13674-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15740-6">
    <skos:altLabel xml:lang="de">TA-Lärm</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15740-6">
    <skos:altLabel xml:lang="de">Lärmbekämpfung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15740-6">
    <skos:altLabel xml:lang="de">Fluglärmgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21002-0">
    <skos:broader rdf:resource="thsys/70892"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21002-0">
    <skos:broader rdf:resource="thsys/70839"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21002-0">
    <skos:broader rdf:resource="thsys/70573"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18416-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14234-5">
    <skos:altLabel xml:lang="de">Travertin</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19058-2">
    <skos:prefLabel xml:lang="en">Construction machinery leasing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19058-2">
    <skos:prefLabel xml:lang="de">Baumaschinen-Leasing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16639-4">
    <skos:narrower rdf:resource="descriptor/16645-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16639-4">
    <skos:narrower rdf:resource="descriptor/19506-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16639-4">
    <skos:narrower rdf:resource="descriptor/10488-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19188-3">
    <skos:narrower rdf:resource="descriptor/19303-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19188-3">
    <skos:narrower rdf:resource="descriptor/19297-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19188-3">
    <skos:narrower rdf:resource="descriptor/19189-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14197-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13238-2">
    <skos:scopeNote xml:lang="de">Sowohl für den Rohtabak- als auch für den Tabakwarenhandel</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18458-5">
    <skos:altLabel xml:lang="en">Industrial self-generation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18458-5">
    <skos:altLabel xml:lang="en">Industrial self generation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18458-5">
    <skos:altLabel xml:lang="en">Industrial generation decision</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18458-5">
    <skos:altLabel xml:lang="en">Industrial cogeneration</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16727-0">
    <skos:related rdf:resource="descriptor/16732-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16727-0">
    <skos:related rdf:resource="descriptor/10032-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16727-0">
    <skos:related rdf:resource="descriptor/10065-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16727-0">
    <skos:related rdf:resource="descriptor/15045-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16727-0">
    <skos:related rdf:resource="descriptor/19744-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16727-0">
    <skos:related rdf:resource="descriptor/16712-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16727-0">
    <skos:related rdf:resource="descriptor/16745-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17668-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12142-6">
    <skos:altLabel xml:lang="de">Verantwortungsverteilung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12142-6">
    <skos:altLabel xml:lang="de">Verantwortungsstruktur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12142-6">
    <skos:altLabel xml:lang="de">Kompetenzverteilung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12142-6">
    <skos:altLabel xml:lang="de">Kompetenzübertragung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12142-6">
    <skos:altLabel xml:lang="de">Kompetenzdelegation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11790-4">
    <skos:narrower rdf:resource="descriptor/19246-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11790-4">
    <skos:narrower rdf:resource="descriptor/18478-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11790-4">
    <skos:narrower rdf:resource="descriptor/19245-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11790-4">
    <skos:narrower rdf:resource="descriptor/18060-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11790-4">
    <skos:narrower rdf:resource="descriptor/19248-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15786-3">
    <skos:related rdf:resource="descriptor/15666-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15786-3">
    <skos:related rdf:resource="descriptor/15801-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15786-3">
    <skos:related rdf:resource="descriptor/12619-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15786-3">
    <skos:related rdf:resource="descriptor/15656-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15786-3">
    <skos:related rdf:resource="descriptor/19601-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15786-3">
    <skos:related rdf:resource="descriptor/12617-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11972-5">
    <skos:prefLabel xml:lang="en">Mega-city</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11972-5">
    <skos:prefLabel xml:lang="de">Stadtlandschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13848-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13299-3">
    <skos:prefLabel xml:lang="en">Transportation insurance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13299-3">
    <skos:prefLabel xml:lang="de">Transportversicherung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13125-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14851-2">
    <skos:related rdf:resource="descriptor/14146-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11059-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19074-4">
    <skos:related rdf:resource="descriptor/12705-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19074-4">
    <skos:related rdf:resource="descriptor/12700-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19074-4">
    <skos:related rdf:resource="descriptor/12765-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19074-4">
    <skos:related rdf:resource="descriptor/11187-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16059-1">
    <skos:prefLabel xml:lang="en">International competition rules</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16059-1">
    <skos:prefLabel xml:lang="de">Internationales Wettbewerbsrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20289-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12905-1">
    <skos:altLabel xml:lang="de">Bauindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12905-1">
    <skos:altLabel xml:lang="de">Bauhauptgewerbe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12905-1">
    <skos:altLabel xml:lang="de">Bauhandwerk</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12905-1">
    <skos:altLabel xml:lang="de">Baugewerbe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19657-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13658-1">
    <skos:scopeNote xml:lang="de">Für die Telekommunikations- und Computerdienste BENUTZE die entsprechenden Deskriptoren.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13658-1">
    <skos:scopeNote xml:lang="de">Meist die klassischen Medien: Rundfunk, Fernsehen und Presse; z.T. einschließlich der neuen Telekommunikationsmedien.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14055-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10012-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11811-1">
    <skos:altLabel xml:lang="de">Energieeinsparungsprogramm</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11811-1">
    <skos:altLabel xml:lang="de">Energieeinsparung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15964-5">
    <skos:narrower rdf:resource="descriptor/16264-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15964-5">
    <skos:narrower rdf:resource="descriptor/19843-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18200-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18504-3">
    <skos:broader rdf:resource="thsys/70970"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18504-3">
    <skos:broader rdf:resource="descriptor/12041-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73389">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70178">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.02.03</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70953">
    <skos:prefLabel xml:lang="en">W.10.03  Wholesale trade</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70953">
    <skos:prefLabel xml:lang="de">W.10.03  Großhandel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70883">
    <skos:broader rdf:resource="thsys/w"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15929-0">
    <skos:related rdf:resource="descriptor/19642-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17201-5">
    <skos:prefLabel xml:lang="en">Belgium</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17201-5">
    <skos:prefLabel xml:lang="de">Belgien</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18657-6">
    <skos:scopeNote xml:lang="de">Offene Systeme sind prinzipiell mit anderen technischen Einrichtungen koppelbar. Gegensatz: proprietäre Systeme.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13349-0">
    <skos:narrower rdf:resource="descriptor/14017-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13349-0">
    <skos:narrower rdf:resource="descriptor/14015-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13349-0">
    <skos:narrower rdf:resource="descriptor/14011-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13349-0">
    <skos:narrower rdf:resource="descriptor/11372-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18854-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19605-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13383-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19007-5">
    <skos:broader rdf:resource="thsys/70244"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19007-5">
    <skos:broader rdf:resource="descriptor/12567-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19007-5">
    <skos:broader rdf:resource="descriptor/12766-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10460-2">
    <skos:altLabel xml:lang="en">Total factor productivity</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10460-2">
    <skos:altLabel xml:lang="en">TFP (Total factor productivity)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10460-2">
    <skos:altLabel xml:lang="en">Multifactor productivity</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10460-2">
    <skos:altLabel xml:lang="de">Faktorproduktivität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13242-4">
    <skos:related rdf:resource="descriptor/13529-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13242-4">
    <skos:related rdf:resource="descriptor/13528-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11820-0">
    <skos:related rdf:resource="descriptor/19829-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11820-0">
    <skos:related rdf:resource="descriptor/11821-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11820-0">
    <skos:related rdf:resource="descriptor/12913-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11820-0">
    <skos:related rdf:resource="descriptor/11822-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11994-2">
    <skos:broader rdf:resource="thsys/71064"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11994-2">
    <skos:broader rdf:resource="thsys/70888"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11994-2">
    <skos:broader rdf:resource="descriptor/13849-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11994-2">
    <skos:broader rdf:resource="descriptor/13845-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18182-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18733-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18320-6">
    <skos:altLabel xml:lang="en">Agroindustrial sector</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18320-6">
    <skos:altLabel xml:lang="en">Agro-food industry</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18320-6">
    <skos:altLabel xml:lang="en">Agrobusiness</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18320-6">
    <skos:altLabel xml:lang="en">Agri-food industry</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18320-6">
    <skos:altLabel xml:lang="en">Agribusiness</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18320-6">
    <skos:altLabel xml:lang="de">Agrarindustrielle Verflechtung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18320-6">
    <skos:altLabel xml:lang="de">Agrarindustrielle Kooperation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18320-6">
    <skos:altLabel xml:lang="de">Agrarindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11156-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19785-6">
    <skos:altLabel xml:lang="en">Pharmacology</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19785-6">
    <skos:altLabel xml:lang="de">Pharmakologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19785-6">
    <skos:altLabel xml:lang="de">Pharma-Forschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19785-6">
    <skos:altLabel xml:lang="de">Arzneimittelforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15717-1">
    <skos:prefLabel xml:lang="en">Social class</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15717-1">
    <skos:prefLabel xml:lang="de">Soziale Schicht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13395-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19490-4">
    <skos:scopeNote xml:lang="de">Substitution zwischen Rohstoffen oder zwischen Bezugsquellen von Rohstoffen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13810-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70582">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14995-6">
    <skos:broader rdf:resource="thsys/73303"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14995-6">
    <skos:broader rdf:resource="thsys/73298"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14995-6">
    <skos:broader rdf:resource="descriptor/14933-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14995-6">
    <skos:broader rdf:resource="descriptor/14996-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14995-6">
    <skos:broader rdf:resource="descriptor/14955-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18926-5">
    <skos:related rdf:resource="descriptor/13357-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18926-5">
    <skos:related rdf:resource="descriptor/18927-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18926-5">
    <skos:related rdf:resource="descriptor/14040-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18926-5">
    <skos:related rdf:resource="descriptor/15655-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24705-0">
    <skos:scopeNote xml:lang="de">Kommerzialisierung bzw. Ökonomisierung beschreibt den Prozess einer zunehmenden Regelung bislang privater und öffentlicher Tätigkeits- und Handlungsfelder durch ökonomische Vertragsbeziehungen und Marktmechanismen. Während die Wirtschaftswissenschaften den Zugewinn an Rationalität und Effizienz durch diese Entwicklung thematisieren, untersuchen die Sozialwissenschaften vor allem die damit verbundenen Strukturveränderungen im gesellschaftlichen Machtgefüge sowie den Verlust öffentlicher und privater Freiheitsgrade. Diese unterschiedlichen Ansätze führen zu divergenten Ergebnissen hinsichtlich der gesellschaftlichen Auswirkungen von Kommerzialisierung / Ökonomisierung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13577-1">
    <skos:prefLabel xml:lang="en">Transport technology</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13577-1">
    <skos:prefLabel xml:lang="de">Verkehrstechnik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19719-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18954-0">
    <skos:broader rdf:resource="thsys/70819"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18954-0">
    <skos:broader rdf:resource="descriptor/18101-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10089-1">
    <skos:altLabel xml:lang="de">Rangsummen-Pluralitätswahl</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10089-1">
    <skos:altLabel xml:lang="de">Punktwahlverfahren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10089-1">
    <skos:altLabel xml:lang="de">Pluralitätswahl</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10089-1">
    <skos:altLabel xml:lang="de">Pluralitätsverfahren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10089-1">
    <skos:altLabel xml:lang="de">Mehrheitsregel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10089-1">
    <skos:altLabel xml:lang="de">Einstimmigkeitsregel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id336">
    <rdf:value xml:lang="de">Für statistische Trendanalysen</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12458-0">
    <skos:broader rdf:resource="thsys/71043"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12458-0">
    <skos:broader rdf:resource="descriptor/12041-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18259-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16303-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14973-2">
    <skos:broader rdf:resource="thsys/73300"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14973-2">
    <skos:broader rdf:resource="descriptor/14972-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19292-1">
    <skos:related rdf:resource="descriptor/12955-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19292-1">
    <skos:related rdf:resource="descriptor/13156-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19292-1">
    <skos:related rdf:resource="descriptor/14989-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19292-1">
    <skos:related rdf:resource="descriptor/12939-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19292-1">
    <skos:related rdf:resource="descriptor/13431-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13903-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19025-3">
    <skos:prefLabel xml:lang="en">Process</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19025-3">
    <skos:prefLabel xml:lang="de">Ablauf</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70161">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10984-3">
    <skos:narrower rdf:resource="descriptor/13424-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10984-3">
    <skos:narrower rdf:resource="descriptor/13405-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10984-3">
    <skos:narrower rdf:resource="descriptor/10966-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10984-3">
    <skos:narrower rdf:resource="descriptor/13402-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10984-3">
    <skos:narrower rdf:resource="descriptor/13401-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10984-3">
    <skos:narrower rdf:resource="descriptor/13400-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10984-3">
    <skos:narrower rdf:resource="descriptor/11350-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10984-3">
    <skos:narrower rdf:resource="descriptor/13399-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10984-3">
    <skos:narrower rdf:resource="descriptor/13397-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10984-3">
    <skos:narrower rdf:resource="descriptor/13395-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10984-3">
    <skos:narrower rdf:resource="descriptor/13394-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10984-3">
    <skos:narrower rdf:resource="descriptor/16630-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10984-3">
    <skos:narrower rdf:resource="descriptor/10558-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10984-3">
    <skos:narrower rdf:resource="descriptor/16382-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10984-3">
    <skos:narrower rdf:resource="descriptor/18850-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19456-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11599-1">
    <skos:narrower rdf:resource="descriptor/11668-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11599-1">
    <skos:narrower rdf:resource="descriptor/11603-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11599-1">
    <skos:narrower rdf:resource="descriptor/11602-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11599-1">
    <skos:narrower rdf:resource="descriptor/19375-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70036">
    <skos:prefLabel xml:lang="de">B.02.02  Kapitalverwendung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10261-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19638-5">
    <skos:prefLabel xml:lang="en">Aging population</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19638-5">
    <skos:prefLabel xml:lang="de">Alternde Bevölkerung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id265">
    <rdf:value xml:lang="de">Für betriebswirtschaftliche Produktionsfunktionen</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12741-5">
    <skos:prefLabel xml:lang="en">Corporate production function</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12741-5">
    <skos:prefLabel xml:lang="de">Betriebswirtschaftliche Produktionsfunktion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10684-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16697-4">
    <skos:scopeNote xml:lang="de">Vorwiegend sozialwissenschaftliche Altersforschung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19355-3">
    <skos:altLabel xml:lang="en">Composition of foreign trade</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19355-3">
    <skos:altLabel xml:lang="de">Außenwirtschaftsstruktur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14926-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70550">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">B.07.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10706-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15995-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15835-2">
    <skos:broader rdf:resource="thsys/73386"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15835-2">
    <skos:broader rdf:resource="thsys/73358"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15835-2">
    <skos:broader rdf:resource="descriptor/15714-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70455">
    <skos:prefLabel xml:lang="en">V.05.04  Interest rate</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70455">
    <skos:prefLabel xml:lang="de">V.05.04  Zins</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70147">
    <rdfs:label xml:lang="en">East Asia</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70147">
    <rdfs:label xml:lang="de">Ostasien</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16535-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id209">
    <rdfs:seeAlso rdf:resource="descriptor/14728-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19149-6">
    <skos:scopeNote xml:lang="de">Verfahren zur Prüfung eines Tests auf seine Verwendbarkeit oder Vortest von Marketingmaßnahmen zur Gewinnung der besten Alternative.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11143-2">
    <skos:broader rdf:resource="thsys/71101"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11839-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12904-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19152-3">
    <skos:altLabel xml:lang="de">Schlankes Management</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10192-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14991-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10804-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13367-5">
    <skos:broader rdf:resource="thsys/73296"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21283-1">
    <skos:broader rdf:resource="thsys/70922"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21283-1">
    <skos:broader rdf:resource="thsys/70459"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21283-1">
    <skos:broader rdf:resource="thsys/70175"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21283-1">
    <skos:broader rdf:resource="descriptor/18666-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21283-1">
    <skos:broader rdf:resource="descriptor/19544-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12768-6">
    <skos:related rdf:resource="descriptor/19654-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11638-3">
    <skos:prefLabel xml:lang="en">Tax compliance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11638-3">
    <skos:prefLabel xml:lang="de">Steuermoral</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70007">
    <rdfs:label xml:lang="en">Historical perspective</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70007">
    <rdfs:label xml:lang="de">Geschichte</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16977-5">
    <skos:prefLabel xml:lang="en">Uri</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16977-5">
    <skos:prefLabel xml:lang="de">Uri</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10025-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19602-5">
    <skos:prefLabel xml:lang="en">Monetary statistics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19602-5">
    <skos:prefLabel xml:lang="de">Monetäre Statistik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14686-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14422-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16969-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13360-5">
    <skos:broader rdf:resource="thsys/70824"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13360-5">
    <skos:broader rdf:resource="descriptor/13347-4"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id22">
    <rdfs:seeAlso rdf:resource="descriptor/11239-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14885-6">
    <skos:prefLabel xml:lang="en">Yarn</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14885-6">
    <skos:prefLabel xml:lang="de">Garn</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19766-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13795-5">
    <skos:altLabel xml:lang="de">Stimmrechtsvertretung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13795-5">
    <skos:altLabel xml:lang="de">Bankenstimmrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73342">
    <skos:narrower rdf:resource="descriptor/15606-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73342">
    <skos:narrower rdf:resource="descriptor/15605-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73342">
    <skos:narrower rdf:resource="descriptor/15600-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73342">
    <skos:narrower rdf:resource="descriptor/16774-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73342">
    <skos:narrower rdf:resource="descriptor/19800-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73342">
    <skos:narrower rdf:resource="descriptor/19660-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73342">
    <skos:narrower rdf:resource="thsys/73339"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73342">
    <skos:narrower rdf:resource="thsys/73341"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73342">
    <skos:narrower rdf:resource="thsys/73338"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73342">
    <skos:narrower rdf:resource="thsys/73340"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11912-2">
    <skos:altLabel xml:lang="de">Regionaltheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11912-2">
    <skos:altLabel xml:lang="de">Regionale Wirtschaftstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16059-1">
    <skos:broader rdf:resource="thsys/73333"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16059-1">
    <skos:broader rdf:resource="thsys/71071"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16059-1">
    <skos:broader rdf:resource="descriptor/10964-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16059-1">
    <skos:broader rdf:resource="descriptor/16032-0"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id223">
    <rdfs:seeAlso rdf:resource="descriptor/10300-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13615-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="../stw">
    <dcterms:issued>2009-02-16</dcterms:issued>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18225-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19278-2">
    <skos:altLabel xml:lang="en">Split</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16713-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11276-4">
    <skos:prefLabel xml:lang="en">Agricultural workers</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11276-4">
    <skos:prefLabel xml:lang="de">Landwirtschaftsberufe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10294-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19610-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16958-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70989">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19264-6">
    <skos:altLabel xml:lang="en">Floating rate note</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19264-6">
    <skos:altLabel xml:lang="en">Floating rate CD</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19264-6">
    <skos:altLabel xml:lang="de">Zinsvariable Anleihe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19264-6">
    <skos:altLabel xml:lang="de">Variabel verzinsliche Anleihe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19264-6">
    <skos:altLabel xml:lang="de">Floater</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10636-5">
    <skos:related rdf:resource="descriptor/10527-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10636-5">
    <skos:related rdf:resource="descriptor/10631-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16635-5">
    <skos:related rdf:resource="descriptor/15658-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16635-5">
    <skos:related rdf:resource="descriptor/16670-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16635-5">
    <skos:related rdf:resource="descriptor/13371-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16635-5">
    <skos:related rdf:resource="descriptor/18892-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16635-5">
    <skos:related rdf:resource="descriptor/18891-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16635-5">
    <skos:related rdf:resource="descriptor/19578-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16635-5">
    <skos:related rdf:resource="descriptor/18444-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16635-5">
    <skos:related rdf:resource="descriptor/11157-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16635-5">
    <skos:related rdf:resource="descriptor/14926-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12047-0">
    <skos:scopeNote xml:lang="de">Nicht-erwerbswirtschaftlich ausgerichtete Organisationen und Institutionen, z.B. Interessenverbände und karitative Einrichtungen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11903-3">
    <skos:broader rdf:resource="thsys/71070"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11903-3">
    <skos:broader rdf:resource="thsys/71068"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11539-5">
    <skos:broader rdf:resource="thsys/70869"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11539-5">
    <skos:broader rdf:resource="thsys/70144"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11539-5">
    <skos:broader rdf:resource="descriptor/16335-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11539-5">
    <skos:broader rdf:resource="descriptor/11540-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11635-2">
    <skos:broader rdf:resource="thsys/71077"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11073-4">
    <skos:prefLabel xml:lang="en">Marxist approach to labour</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11073-4">
    <skos:prefLabel xml:lang="de">Marxistische Arbeitstheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18317-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12243-0">
    <skos:altLabel xml:lang="de">Finanzierungs-Leasing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14437-5">
    <skos:prefLabel xml:lang="en">Motorcycle</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14437-5">
    <skos:prefLabel xml:lang="de">Motorrad</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16342-6">
    <skos:related rdf:resource="descriptor/16340-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16342-6">
    <skos:related rdf:resource="descriptor/16246-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16342-6">
    <skos:related rdf:resource="descriptor/13984-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11146-3">
    <skos:prefLabel xml:lang="en">Socialist economic calculation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11146-3">
    <skos:prefLabel xml:lang="de">Wirtschaftliche Rechnungsführung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13117-0">
    <skos:related rdf:resource="descriptor/12930-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13117-0">
    <skos:related rdf:resource="descriptor/13226-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13117-0">
    <skos:related rdf:resource="descriptor/14805-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13117-0">
    <skos:related rdf:resource="descriptor/14803-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13117-0">
    <skos:related rdf:resource="descriptor/14385-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13117-0">
    <skos:related rdf:resource="descriptor/19389-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13117-0">
    <skos:related rdf:resource="descriptor/14802-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13117-0">
    <skos:related rdf:resource="descriptor/12912-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13117-0">
    <skos:related rdf:resource="descriptor/13191-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10684-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11683-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16552-2">
    <skos:scopeNote xml:lang="de">Nur für regionalpolitische Maßnahmen unterhalb der gesamtstaatlichen Ebene.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15751-1">
    <skos:narrower rdf:resource="descriptor/18235-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17989-0">
    <skos:prefLabel xml:lang="en">Caribbean countries</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17989-0">
    <skos:prefLabel xml:lang="de">Karibische Staaten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71072">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id75">
    <rdfs:seeAlso rdf:resource="descriptor/14483-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13816-2">
    <skos:prefLabel xml:lang="en">Property insurance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13816-2">
    <skos:prefLabel xml:lang="de">Sachversicherung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13080-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14698-5">
    <skos:related rdf:resource="descriptor/18257-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18960-5">
    <skos:altLabel xml:lang="de">Rückstandsmanagement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18960-5">
    <skos:altLabel xml:lang="de">Entsorgungsmanagement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18960-5">
    <skos:altLabel xml:lang="de">Entsorgungslogistik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18960-5">
    <skos:altLabel xml:lang="de">Abfallmanagement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14573-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13612-4">
    <skos:prefLabel xml:lang="en">Transport service</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13612-4">
    <skos:prefLabel xml:lang="de">Verkehrsleistung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19554-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13684-0">
    <skos:related rdf:resource="descriptor/13578-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13684-0">
    <skos:related rdf:resource="descriptor/15900-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13684-0">
    <skos:related rdf:resource="descriptor/14423-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13684-0">
    <skos:related rdf:resource="descriptor/18419-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13684-0">
    <skos:related rdf:resource="descriptor/18418-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71103">
    <skos:broader rdf:resource="thsys/70187"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11147-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10508-0">
    <skos:related rdf:resource="descriptor/10500-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10508-0">
    <skos:related rdf:resource="descriptor/10492-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10169-3">
    <skos:broader rdf:resource="thsys/70493"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10169-3">
    <skos:broader rdf:resource="thsys/70051"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10169-3">
    <skos:broader rdf:resource="descriptor/10177-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10169-3">
    <skos:broader rdf:resource="descriptor/11183-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10169-3">
    <skos:broader rdf:resource="descriptor/10000-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70873">
    <rdfs:label xml:lang="en">Trade fair industry</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70873">
    <rdfs:label xml:lang="de">Messewirtschaft</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18011-5">
    <skos:narrower rdf:resource="descriptor/19321-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18011-5">
    <skos:narrower rdf:resource="descriptor/19317-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18011-5">
    <skos:narrower rdf:resource="descriptor/19493-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15450-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19098-4">
    <skos:prefLabel xml:lang="en">Cost structure</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19098-4">
    <skos:prefLabel xml:lang="de">Kostenstruktur</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12337-5">
    <skos:broader rdf:resource="thsys/73391"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12337-5">
    <skos:broader rdf:resource="thsys/71043"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12337-5">
    <skos:broader rdf:resource="descriptor/12326-3"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id313">
    <rdf:value xml:lang="de">Für theoretische Literatur zur Statistik</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10647-0">
    <skos:altLabel xml:lang="de">Zollschutz, effektiver</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10647-0">
    <skos:altLabel xml:lang="de">Theorie der effektiven Protektion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10647-0">
    <skos:altLabel xml:lang="de">Effektivzoll</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10647-0">
    <skos:altLabel xml:lang="de">Effektiver Zollschutz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13511-3">
    <skos:related rdf:resource="descriptor/15706-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13511-3">
    <skos:related rdf:resource="descriptor/10465-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13511-3">
    <skos:related rdf:resource="descriptor/13462-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19781-0">
    <skos:broader rdf:resource="thsys/70581"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18588-6">
    <skos:related rdf:resource="descriptor/16169-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14980-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id87">
    <rdf:value xml:lang="de">Für Entwicklungszusammenarbeit im Sinne von Entwicklungshilfe</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16186-1">
    <skos:narrower rdf:resource="descriptor/16188-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16186-1">
    <skos:narrower rdf:resource="descriptor/10621-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16186-1">
    <skos:narrower rdf:resource="descriptor/16032-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16186-1">
    <skos:narrower rdf:resource="descriptor/19812-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16186-1">
    <skos:narrower rdf:resource="descriptor/16182-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71065">
    <skos:narrower rdf:resource="descriptor/19243-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71065">
    <skos:narrower rdf:resource="descriptor/18993-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71065">
    <skos:narrower rdf:resource="descriptor/19465-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71065">
    <skos:narrower rdf:resource="descriptor/18982-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71065">
    <skos:narrower rdf:resource="descriptor/18865-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71065">
    <skos:narrower rdf:resource="descriptor/18631-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71065">
    <skos:narrower rdf:resource="descriptor/10503-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71065">
    <skos:narrower rdf:resource="descriptor/19273-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71065">
    <skos:narrower rdf:resource="descriptor/18548-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71065">
    <skos:narrower rdf:resource="descriptor/19462-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71065">
    <skos:narrower rdf:resource="descriptor/19460-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71065">
    <skos:narrower rdf:resource="descriptor/19463-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71065">
    <skos:narrower rdf:resource="descriptor/19464-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71065">
    <skos:narrower rdf:resource="descriptor/18459-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71065">
    <skos:narrower rdf:resource="descriptor/19368-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71065">
    <skos:narrower rdf:resource="descriptor/18323-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71065">
    <skos:narrower rdf:resource="descriptor/10575-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71065">
    <skos:narrower rdf:resource="descriptor/18358-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71065">
    <skos:narrower rdf:resource="descriptor/19233-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71065">
    <skos:narrower rdf:resource="descriptor/19362-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71065">
    <skos:narrower rdf:resource="descriptor/19461-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71065">
    <skos:narrower rdf:resource="descriptor/10504-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71065">
    <skos:narrower rdf:resource="descriptor/13454-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71065">
    <skos:narrower rdf:resource="descriptor/19459-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71065">
    <skos:narrower rdf:resource="descriptor/11890-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71065">
    <skos:narrower rdf:resource="descriptor/19458-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71065">
    <skos:narrower rdf:resource="descriptor/11862-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71065">
    <skos:narrower rdf:resource="descriptor/13445-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11473-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13372-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10829-1">
    <skos:prefLabel xml:lang="en">International tax law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10829-1">
    <skos:prefLabel xml:lang="de">Internationales Steuerrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12768-6">
    <skos:altLabel xml:lang="en">Database-marketing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12768-6">
    <skos:altLabel xml:lang="de">VIS (Vertriebs-Informationssystem)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12768-6">
    <skos:altLabel xml:lang="de">Vertriebs-Informationssystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12768-6">
    <skos:altLabel xml:lang="de">MIAS (Marketing-Informations- und Analysesystem)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12768-6">
    <skos:altLabel xml:lang="de">Kundeninformationssystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12768-6">
    <skos:altLabel xml:lang="de">Kunden-Informationssystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12768-6">
    <skos:altLabel xml:lang="de">Kundendatenbank</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12768-6">
    <skos:altLabel xml:lang="de">Absatz-Informationssystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18279-5">
    <skos:broader rdf:resource="thsys/73343"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18279-5">
    <skos:broader rdf:resource="thsys/70201"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18279-5">
    <skos:broader rdf:resource="descriptor/12649-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18939-3">
    <skos:prefLabel xml:lang="en">Broadcasting finance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18939-3">
    <skos:prefLabel xml:lang="de">Rundfunkfinanzierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12137-6">
    <skos:broader rdf:resource="thsys/70562"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12137-6">
    <skos:broader rdf:resource="descriptor/12111-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14527-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19569-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12649-0">
    <skos:narrower rdf:resource="descriptor/18279-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12649-0">
    <skos:narrower rdf:resource="descriptor/18243-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18366-3">
    <skos:altLabel xml:lang="de">Rüstungstechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18366-3">
    <skos:altLabel xml:lang="de">Militärtechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10057-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18103-0">
    <skos:prefLabel xml:lang="en">Electronic product</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18103-0">
    <skos:prefLabel xml:lang="de">Elektronisches Produkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70869">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11999-6">
    <skos:altLabel xml:lang="en">Urban renewal</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11999-6">
    <skos:altLabel xml:lang="de">Stadtteilsanierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11999-6">
    <skos:altLabel xml:lang="de">Stadterneuerung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11999-6">
    <skos:altLabel xml:lang="de">Altstadtsanierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10170-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10605-2">
    <skos:related rdf:resource="descriptor/10617-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10605-2">
    <skos:related rdf:resource="descriptor/10784-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10605-2">
    <skos:related rdf:resource="descriptor/18614-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10605-2">
    <skos:related rdf:resource="descriptor/18001-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10605-2">
    <skos:related rdf:resource="descriptor/18794-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10605-2">
    <skos:related rdf:resource="descriptor/10604-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10605-2">
    <skos:related rdf:resource="descriptor/18544-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10605-2">
    <skos:related rdf:resource="descriptor/10629-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10605-2">
    <skos:related rdf:resource="descriptor/10682-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10605-2">
    <skos:related rdf:resource="descriptor/19355-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10605-2">
    <skos:related rdf:resource="descriptor/10686-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10605-2">
    <skos:related rdf:resource="descriptor/18534-1"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id346">
    <rdfs:seeAlso rdf:resource="descriptor/12711-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12533-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19534-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19387-4">
    <skos:altLabel xml:lang="de">Mineralölvorrat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19387-4">
    <skos:altLabel xml:lang="de">Mineralölvorkommen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19387-4">
    <skos:altLabel xml:lang="de">Mineralöllagerstätte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19101-6">
    <skos:altLabel xml:lang="de">Prozeßkosten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19622-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10248-0">
    <skos:scopeNote xml:lang="de">Nur für die Analyse des Wirtschaftskreislaufs.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12509-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14810-2">
    <skos:prefLabel xml:lang="en">Pulp</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14810-2">
    <skos:prefLabel xml:lang="de">Zellstoff</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11897-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73377">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71087">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11679-3">
    <skos:narrower rdf:resource="descriptor/12868-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11679-3">
    <skos:narrower rdf:resource="descriptor/11666-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11679-3">
    <skos:narrower rdf:resource="descriptor/11657-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11679-3">
    <skos:narrower rdf:resource="descriptor/19418-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13250-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11910-6">
    <skos:related rdf:resource="descriptor/16552-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11910-6">
    <skos:related rdf:resource="descriptor/19246-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11910-6">
    <skos:related rdf:resource="descriptor/11909-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11910-6">
    <skos:related rdf:resource="descriptor/19224-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11910-6">
    <skos:related rdf:resource="descriptor/19440-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11910-6">
    <skos:related rdf:resource="descriptor/19438-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11910-6">
    <skos:related rdf:resource="descriptor/11939-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70036">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">B.02.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16533-6">
    <skos:prefLabel xml:lang="en">Poverty alleviation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16533-6">
    <skos:prefLabel xml:lang="de">Armutspolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19763-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16273-6">
    <skos:narrower rdf:resource="descriptor/11847-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16273-6">
    <skos:narrower rdf:resource="descriptor/16487-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16273-6">
    <skos:narrower rdf:resource="descriptor/10987-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16273-6">
    <skos:narrower rdf:resource="descriptor/15768-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16273-6">
    <skos:narrower rdf:resource="descriptor/16483-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16273-6">
    <skos:narrower rdf:resource="descriptor/16545-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16273-6">
    <skos:narrower rdf:resource="descriptor/16463-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16273-6">
    <skos:narrower rdf:resource="descriptor/15970-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16273-6">
    <skos:narrower rdf:resource="descriptor/16432-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16273-6">
    <skos:narrower rdf:resource="descriptor/19631-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16273-6">
    <skos:narrower rdf:resource="descriptor/16435-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16273-6">
    <skos:narrower rdf:resource="descriptor/16509-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16273-6">
    <skos:narrower rdf:resource="descriptor/16479-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16273-6">
    <skos:narrower rdf:resource="descriptor/16502-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16273-6">
    <skos:narrower rdf:resource="descriptor/10437-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16273-6">
    <skos:narrower rdf:resource="descriptor/10560-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16273-6">
    <skos:narrower rdf:resource="descriptor/12018-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16273-6">
    <skos:narrower rdf:resource="descriptor/11397-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16273-6">
    <skos:narrower rdf:resource="descriptor/16489-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18619-0">
    <skos:related rdf:resource="descriptor/14933-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18619-0">
    <skos:related rdf:resource="descriptor/19064-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18619-0">
    <skos:related rdf:resource="descriptor/14337-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18805-3">
    <skos:altLabel xml:lang="de">Lizenz des Staates</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18805-3">
    <skos:altLabel xml:lang="de">Handwerkskonzession</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18805-3">
    <skos:altLabel xml:lang="de">Gewerbekonzession</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16325-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12778-3">
    <skos:prefLabel xml:lang="en">Distribution channel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12778-3">
    <skos:prefLabel xml:lang="de">Vertriebsweg</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11810-3">
    <skos:narrower rdf:resource="descriptor/18364-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11810-3">
    <skos:narrower rdf:resource="descriptor/18374-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11810-3">
    <skos:narrower rdf:resource="descriptor/18367-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11810-3">
    <skos:narrower rdf:resource="descriptor/18362-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11810-3">
    <skos:narrower rdf:resource="descriptor/18370-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11810-3">
    <skos:narrower rdf:resource="descriptor/18373-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11810-3">
    <skos:narrower rdf:resource="descriptor/18371-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11810-3">
    <skos:narrower rdf:resource="descriptor/11811-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11810-3">
    <skos:narrower rdf:resource="descriptor/18812-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11810-3">
    <skos:narrower rdf:resource="descriptor/18360-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18754-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11199-3">
    <skos:prefLabel xml:lang="en">Work sharing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11199-3">
    <skos:prefLabel xml:lang="de">Job Sharing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18983-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14329-1">
    <skos:altLabel xml:lang="de">Industriearmatur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14329-1">
    <skos:altLabel xml:lang="de">Gebäudearmatur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19349-5">
    <skos:related rdf:resource="descriptor/13149-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19349-5">
    <skos:related rdf:resource="descriptor/14130-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19349-5">
    <skos:related rdf:resource="descriptor/19347-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19349-5">
    <skos:related rdf:resource="descriptor/12952-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70991">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.06.01.07</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73337">
    <skos:narrower rdf:resource="descriptor/16122-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73337">
    <skos:narrower rdf:resource="descriptor/16119-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73337">
    <skos:narrower rdf:resource="descriptor/10847-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73337">
    <skos:narrower rdf:resource="descriptor/12855-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73337">
    <skos:narrower rdf:resource="descriptor/11710-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73337">
    <skos:narrower rdf:resource="descriptor/12437-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73337">
    <skos:narrower rdf:resource="descriptor/11709-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73337">
    <skos:narrower rdf:resource="descriptor/16087-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73337">
    <skos:narrower rdf:resource="descriptor/12558-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73337">
    <skos:narrower rdf:resource="descriptor/11640-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73337">
    <skos:narrower rdf:resource="descriptor/12849-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73337">
    <skos:narrower rdf:resource="descriptor/12880-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73337">
    <skos:narrower rdf:resource="descriptor/16112-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73337">
    <skos:narrower rdf:resource="descriptor/16111-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73337">
    <skos:narrower rdf:resource="descriptor/11708-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73337">
    <skos:narrower rdf:resource="descriptor/19410-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73337">
    <skos:narrower rdf:resource="descriptor/12874-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73337">
    <skos:narrower rdf:resource="descriptor/12863-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73337">
    <skos:narrower rdf:resource="descriptor/12867-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73337">
    <skos:narrower rdf:resource="descriptor/16104-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73337">
    <skos:narrower rdf:resource="descriptor/10829-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73337">
    <skos:narrower rdf:resource="descriptor/16100-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73337">
    <skos:narrower rdf:resource="descriptor/24703-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73337">
    <skos:narrower rdf:resource="descriptor/18831-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73337">
    <skos:narrower rdf:resource="descriptor/11535-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73337">
    <skos:narrower rdf:resource="descriptor/11525-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73337">
    <skos:narrower rdf:resource="descriptor/11532-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73337">
    <skos:narrower rdf:resource="descriptor/12419-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73337">
    <skos:narrower rdf:resource="descriptor/16092-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73337">
    <skos:narrower rdf:resource="descriptor/12416-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73337">
    <skos:narrower rdf:resource="descriptor/10828-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73337">
    <skos:narrower rdf:resource="descriptor/18302-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73337">
    <skos:narrower rdf:resource="descriptor/12850-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73337">
    <skos:narrower rdf:resource="descriptor/11698-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73337">
    <skos:narrower rdf:resource="descriptor/19315-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73337">
    <skos:narrower rdf:resource="descriptor/10827-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12192-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18489-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15928-2">
    <skos:related rdf:resource="descriptor/16513-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15928-2">
    <skos:related rdf:resource="descriptor/15659-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15928-2">
    <skos:related rdf:resource="descriptor/19642-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15928-2">
    <skos:related rdf:resource="descriptor/16556-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15928-2">
    <skos:related rdf:resource="descriptor/15918-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15928-2">
    <skos:related rdf:resource="descriptor/15932-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15928-2">
    <skos:related rdf:resource="descriptor/15931-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15928-2">
    <skos:related rdf:resource="descriptor/15627-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72181">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.16.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="http://zbw.eu/namespaces/skos-extensions/useInsteadNote">
    <rdfs:label xml:lang="en">useInsteadNote</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12120-2">
    <skos:broader rdf:resource="thsys/70562"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12120-2">
    <skos:broader rdf:resource="thsys/70485"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12120-2">
    <skos:broader rdf:resource="descriptor/19562-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13284-2">
    <skos:altLabel xml:lang="en">Building society</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13284-2">
    <skos:altLabel xml:lang="en">Building and loan association</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16493-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18674-6">
    <skos:altLabel xml:lang="en">Behavioral finance</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18674-6">
    <skos:altLabel xml:lang="de">Verhaltenswissenschaftliche Kapitalmarktforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18674-6">
    <skos:altLabel xml:lang="de">Börsenpsychologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18674-6">
    <skos:altLabel xml:lang="de">Anlegerverhalten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18674-6">
    <skos:altLabel xml:lang="de">Anlagestrategie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18674-6">
    <skos:altLabel xml:lang="de">Anlageentscheidung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11394-5">
    <skos:related rdf:resource="descriptor/15666-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11394-5">
    <skos:related rdf:resource="descriptor/18860-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11394-5">
    <skos:related rdf:resource="descriptor/11397-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11394-5">
    <skos:related rdf:resource="descriptor/11407-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11394-5">
    <skos:related rdf:resource="descriptor/11354-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11394-5">
    <skos:related rdf:resource="descriptor/11400-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11394-5">
    <skos:related rdf:resource="descriptor/11353-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11394-5">
    <skos:related rdf:resource="descriptor/11293-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11394-5">
    <skos:related rdf:resource="descriptor/19505-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11753-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10866-2">
    <skos:broader rdf:resource="thsys/70951"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10866-2">
    <skos:broader rdf:resource="thsys/70891"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10866-2">
    <skos:broader rdf:resource="descriptor/13808-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10866-2">
    <skos:broader rdf:resource="descriptor/13293-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18783-1">
    <skos:prefLabel xml:lang="en">Materials research</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18783-1">
    <skos:prefLabel xml:lang="de">Werkstoffforschung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20104-4">
    <skos:prefLabel xml:lang="en">Taylor rule</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20104-4">
    <skos:prefLabel xml:lang="de">Taylor-Regel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19631-5">
    <skos:altLabel xml:lang="de">Meeresgeopolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17948-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16011-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11159-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14631-2">
    <skos:broader rdf:resource="thsys/70014"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14631-2">
    <skos:broader rdf:resource="descriptor/14603-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14631-2">
    <skos:broader rdf:resource="descriptor/14599-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15821-6">
    <skos:related rdf:resource="descriptor/16730-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15821-6">
    <skos:related rdf:resource="descriptor/15682-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15821-6">
    <skos:related rdf:resource="descriptor/18859-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15821-6">
    <skos:related rdf:resource="descriptor/12806-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15821-6">
    <skos:related rdf:resource="descriptor/16287-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15821-6">
    <skos:related rdf:resource="descriptor/12574-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15821-6">
    <skos:related rdf:resource="descriptor/13662-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15821-6">
    <skos:related rdf:resource="descriptor/13383-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15821-6">
    <skos:related rdf:resource="descriptor/16405-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15821-6">
    <skos:related rdf:resource="descriptor/16633-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15821-6">
    <skos:related rdf:resource="descriptor/12075-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15821-6">
    <skos:related rdf:resource="descriptor/19745-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19557-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11451-5">
    <skos:broader rdf:resource="thsys/70306"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11391-4">
    <skos:broader rdf:resource="thsys/70843"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11391-4">
    <skos:broader rdf:resource="descriptor/11360-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18495-6">
    <skos:broader rdf:resource="thsys/70970"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18495-6">
    <skos:broader rdf:resource="descriptor/10470-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12816-0">
    <skos:broader rdf:resource="thsys/70947"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12816-0">
    <skos:broader rdf:resource="thsys/70384"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12816-0">
    <skos:broader rdf:resource="descriptor/12818-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12816-0">
    <skos:broader rdf:resource="descriptor/19139-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12816-0">
    <skos:broader rdf:resource="descriptor/18532-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13680-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14617-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18027-4">
    <skos:altLabel xml:lang="de">Rundfunksendung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18027-4">
    <skos:altLabel xml:lang="de">Rundfunk- und Fernsehprogramm</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18027-4">
    <skos:altLabel xml:lang="de">Radioprogramm</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18027-4">
    <skos:altLabel xml:lang="de">Hörfunksendung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18027-4">
    <skos:altLabel xml:lang="de">Hörfunkprogramm</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18709-6">
    <skos:related rdf:resource="descriptor/18716-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10789-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16980-2">
    <skos:broader rdf:resource="thsys/70003"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16980-2">
    <skos:broader rdf:resource="descriptor/16972-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18814-2">
    <skos:prefLabel xml:lang="en">Armaments research</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18814-2">
    <skos:prefLabel xml:lang="de">Rüstungsforschung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14993-3">
    <skos:prefLabel xml:lang="en">Mixed feed</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14993-3">
    <skos:prefLabel xml:lang="de">Mischfutter</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16314-4">
    <skos:related rdf:resource="descriptor/16312-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16314-4">
    <skos:related rdf:resource="descriptor/16421-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16314-4">
    <skos:related rdf:resource="descriptor/16324-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16314-4">
    <skos:related rdf:resource="descriptor/16315-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16314-4">
    <skos:related rdf:resource="descriptor/19681-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16314-4">
    <skos:related rdf:resource="descriptor/16297-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14022-6">
    <skos:narrower rdf:resource="descriptor/14897-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19827-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17668-6">
    <skos:prefLabel xml:lang="en">Mauritius</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17668-6">
    <skos:prefLabel xml:lang="de">Mauritius</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13692-1">
    <skos:related rdf:resource="descriptor/13694-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13692-1">
    <skos:related rdf:resource="descriptor/13746-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13692-1">
    <skos:related rdf:resource="descriptor/13688-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13692-1">
    <skos:related rdf:resource="descriptor/13687-1"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id220">
    <rdfs:seeAlso rdf:resource="descriptor/15918-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18661-1">
    <skos:related rdf:resource="descriptor/18766-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18661-1">
    <skos:related rdf:resource="descriptor/19808-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18661-1">
    <skos:related rdf:resource="descriptor/13683-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18661-1">
    <skos:related rdf:resource="descriptor/16757-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18661-1">
    <skos:related rdf:resource="descriptor/18139-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70999">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.06.01.03</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15708-2">
    <skos:altLabel xml:lang="en">Demographic development</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15708-2">
    <skos:altLabel xml:lang="de">Natürliche Bevölkerungsentwicklung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15708-2">
    <skos:altLabel xml:lang="de">Demographische Entwicklung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15708-2">
    <skos:altLabel xml:lang="de">Bevölkerungsdynamik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15708-2">
    <skos:altLabel xml:lang="de">Bevölkerungsbilanz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18812-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12972-0">
    <skos:broader rdf:resource="thsys/71025"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12972-0">
    <skos:broader rdf:resource="descriptor/12955-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24707-3">
    <skos:altLabel xml:lang="en">Informational cascades</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24707-3">
    <skos:altLabel xml:lang="en">Information cascades</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24707-3">
    <skos:altLabel xml:lang="en">Herding behaviour</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24707-3">
    <skos:altLabel xml:lang="en">Herding behavior</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24707-3">
    <skos:altLabel xml:lang="en">Herding</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24707-3">
    <skos:altLabel xml:lang="en">Herd behavior</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24707-3">
    <skos:altLabel xml:lang="en">Economic theory of herding</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10764-3">
    <skos:altLabel xml:lang="de">Weltwährung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11415-2">
    <skos:related rdf:resource="descriptor/13807-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11415-2">
    <skos:related rdf:resource="descriptor/11417-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11415-2">
    <skos:related rdf:resource="descriptor/10764-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11415-2">
    <skos:related rdf:resource="descriptor/11042-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11415-2">
    <skos:related rdf:resource="descriptor/11427-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11415-2">
    <skos:related rdf:resource="descriptor/18680-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11415-2">
    <skos:related rdf:resource="descriptor/11426-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11415-2">
    <skos:related rdf:resource="descriptor/10236-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11415-2">
    <skos:related rdf:resource="descriptor/11451-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11415-2">
    <skos:related rdf:resource="descriptor/10382-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11415-2">
    <skos:related rdf:resource="descriptor/11448-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11415-2">
    <skos:related rdf:resource="descriptor/11416-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11415-2">
    <skos:related rdf:resource="descriptor/11419-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70091">
    <rdfs:label xml:lang="de">Entwicklungsökonomik</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14039-3">
    <skos:prefLabel xml:lang="en">Power plant</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14039-3">
    <skos:prefLabel xml:lang="de">Kraftwerk</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10838-0">
    <skos:narrower rdf:resource="descriptor/20002-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10361-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70234">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18517-1">
    <skos:related rdf:resource="descriptor/13159-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18517-1">
    <skos:related rdf:resource="descriptor/19293-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18517-1">
    <skos:related rdf:resource="descriptor/19187-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18517-1">
    <skos:related rdf:resource="descriptor/19188-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18517-1">
    <skos:related rdf:resource="descriptor/18198-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18517-1">
    <skos:related rdf:resource="descriptor/14088-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11718-5">
    <skos:narrower rdf:resource="descriptor/19377-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15397-0">
    <skos:altLabel xml:lang="de">Numerische Taxonomie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15397-0">
    <skos:altLabel xml:lang="de">Multivariate Klassifikationsanalyse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15397-0">
    <skos:altLabel xml:lang="de">Klumpenanalyse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15397-0">
    <skos:altLabel xml:lang="de">Clusterverfahren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15397-0">
    <skos:altLabel xml:lang="de">Clusterbildung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13793-2">
    <skos:narrower rdf:resource="descriptor/10789-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13793-2">
    <skos:narrower rdf:resource="descriptor/13804-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13793-2">
    <skos:narrower rdf:resource="descriptor/13801-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13793-2">
    <skos:narrower rdf:resource="descriptor/13799-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13793-2">
    <skos:narrower rdf:resource="descriptor/18022-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17576-4">
    <skos:altLabel xml:lang="de">Brunei Darussalam</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19381-2">
    <skos:broader rdf:resource="thsys/73364"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19381-2">
    <skos:broader rdf:resource="thsys/71081"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19381-2">
    <skos:broader rdf:resource="descriptor/11051-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19381-2">
    <skos:broader rdf:resource="descriptor/25742-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18911-4">
    <skos:prefLabel xml:lang="en">Parental leave</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18911-4">
    <skos:prefLabel xml:lang="de">Erziehungsurlaub</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19130-6">
    <skos:altLabel xml:lang="de">NAFTA-Länder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14548-3">
    <skos:broader rdf:resource="thsys/70346"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14548-3">
    <skos:broader rdf:resource="descriptor/14269-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18899-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13611-6">
    <skos:scopeNote xml:lang="de">Kooperation von Verkehrsunternehmen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16318-3">
    <skos:related rdf:resource="descriptor/15725-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16318-3">
    <skos:related rdf:resource="descriptor/16159-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16318-3">
    <skos:related rdf:resource="descriptor/16545-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16318-3">
    <skos:related rdf:resource="descriptor/16544-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16318-3">
    <skos:related rdf:resource="descriptor/16542-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16318-3">
    <skos:related rdf:resource="descriptor/11059-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18190-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18868-0">
    <skos:prefLabel xml:lang="en">Didactics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18868-0">
    <skos:prefLabel xml:lang="de">Didaktik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12326-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10170-4">
    <skos:prefLabel xml:lang="en">Durable good</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10170-4">
    <skos:prefLabel xml:lang="de">Dauerhaftes Konsumgut</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11163-3">
    <skos:altLabel xml:lang="en">Conflict theory</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11163-3">
    <skos:altLabel xml:lang="de">Konflikttheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17622-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70026">
    <skos:broader rdf:resource="thsys/g"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18604-6">
    <skos:related rdf:resource="descriptor/18607-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18604-6">
    <skos:related rdf:resource="descriptor/13814-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18604-6">
    <skos:related rdf:resource="descriptor/18498-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18604-6">
    <skos:related rdf:resource="descriptor/13516-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12610-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18368-6">
    <skos:altLabel xml:lang="en">Nuclear safety</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18368-6">
    <skos:altLabel xml:lang="de">Reaktorsicherheitstechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18368-6">
    <skos:altLabel xml:lang="de">Kernreaktorsicherheit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18368-6">
    <skos:altLabel xml:lang="de">Atomreaktorsicherheit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13523-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14931-4">
    <skos:narrower rdf:resource="descriptor/14934-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14931-4">
    <skos:narrower rdf:resource="descriptor/14948-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14931-4">
    <skos:narrower rdf:resource="descriptor/14933-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14931-4">
    <skos:narrower rdf:resource="descriptor/14996-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14931-4">
    <skos:narrower rdf:resource="descriptor/14958-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14931-4">
    <skos:narrower rdf:resource="descriptor/18213-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14931-4">
    <skos:narrower rdf:resource="descriptor/14936-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14931-4">
    <skos:narrower rdf:resource="descriptor/14955-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14931-4">
    <skos:narrower rdf:resource="descriptor/14124-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14931-4">
    <skos:narrower rdf:resource="descriptor/14930-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14931-4">
    <skos:narrower rdf:resource="descriptor/14928-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14931-4">
    <skos:narrower rdf:resource="descriptor/14976-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14931-4">
    <skos:narrower rdf:resource="descriptor/14926-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14931-4">
    <skos:narrower rdf:resource="descriptor/14989-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14931-4">
    <skos:narrower rdf:resource="descriptor/14964-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14931-4">
    <skos:narrower rdf:resource="descriptor/14122-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14931-4">
    <skos:narrower rdf:resource="descriptor/14969-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14931-4">
    <skos:narrower rdf:resource="descriptor/14927-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14931-4">
    <skos:narrower rdf:resource="descriptor/18214-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14931-4">
    <skos:narrower rdf:resource="descriptor/14940-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14931-4">
    <skos:narrower rdf:resource="descriptor/14924-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19689-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16013-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18262-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11897-0">
    <skos:broader rdf:resource="thsys/71067"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12700-5">
    <skos:related rdf:resource="descriptor/10239-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12700-5">
    <skos:related rdf:resource="descriptor/12708-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12700-5">
    <skos:related rdf:resource="descriptor/12754-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12700-5">
    <skos:related rdf:resource="descriptor/19074-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12700-5">
    <skos:related rdf:resource="descriptor/11270-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11818-1">
    <skos:broader rdf:resource="thsys/70965"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11818-1">
    <skos:broader rdf:resource="thsys/70852"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11818-1">
    <skos:broader rdf:resource="descriptor/10966-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12643-5">
    <skos:broader rdf:resource="thsys/70394"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12643-5">
    <skos:broader rdf:resource="thsys/70029"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12643-5">
    <skos:broader rdf:resource="descriptor/12644-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12643-5">
    <skos:broader rdf:resource="descriptor/12760-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18127-0">
    <skos:scopeNote xml:lang="de">Das Halten fremder Währungen durch Privatpersonen und der Einsatz fremder Währungen als Zahlungsmittel vorwiegend in Staaten mit hoher Inflation; z.B. US-Dollar in Lateinamerika und D-Mark in Osteuropa.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19225-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id122">
    <rdf:value xml:lang="de">Für Gewaltausübung im Sinne von Macht</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15410-6">
    <skos:prefLabel xml:lang="en">Pattern recognition</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15410-6">
    <skos:prefLabel xml:lang="de">Mustererkennung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11239-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12246-1">
    <skos:prefLabel xml:lang="en">Suretyship and guarantee</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12246-1">
    <skos:prefLabel xml:lang="de">Bürgschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16004-5">
    <skos:related rdf:resource="descriptor/16485-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16004-5">
    <skos:related rdf:resource="descriptor/14997-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16004-5">
    <skos:related rdf:resource="descriptor/13160-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16004-5">
    <skos:related rdf:resource="descriptor/14931-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16004-5">
    <skos:related rdf:resource="descriptor/19395-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16004-5">
    <skos:related rdf:resource="descriptor/14611-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16004-5">
    <skos:related rdf:resource="descriptor/18444-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16004-5">
    <skos:related rdf:resource="descriptor/18259-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12111-3">
    <skos:broader rdf:resource="thsys/70562"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12111-3">
    <skos:broader rdf:resource="thsys/70321"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16406-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13330-0">
    <skos:related rdf:resource="descriptor/13321-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13330-0">
    <skos:related rdf:resource="descriptor/13340-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13330-0">
    <skos:related rdf:resource="descriptor/13339-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13330-0">
    <skos:related rdf:resource="descriptor/18746-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13330-0">
    <skos:related rdf:resource="descriptor/18496-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11566-2">
    <skos:related rdf:resource="descriptor/11682-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11566-2">
    <skos:related rdf:resource="descriptor/16156-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11566-2">
    <skos:related rdf:resource="descriptor/18912-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11566-2">
    <skos:related rdf:resource="descriptor/19722-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11566-2">
    <skos:related rdf:resource="descriptor/16524-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18372-1">
    <skos:altLabel xml:lang="de">Mineralölkonzession</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18372-1">
    <skos:altLabel xml:lang="de">Konzessionierung von Erdölförderrechten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18372-1">
    <skos:altLabel xml:lang="de">Erdölförderrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18372-1">
    <skos:altLabel xml:lang="de">Erdgaskonzession</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14182-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11895-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19631-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id146">
    <rdf:value xml:lang="de">Für Ingenieurfirmen</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70835">
    <rdfs:label xml:lang="de">Sozialwesen</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14300-4">
    <skos:prefLabel xml:lang="en">Magnesium</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14300-4">
    <skos:prefLabel xml:lang="de">Magnesium</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73345">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14949-6">
    <skos:prefLabel xml:lang="en">Fruit juice</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14949-6">
    <skos:prefLabel xml:lang="de">Fruchtsaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15476-4">
    <skos:related rdf:resource="descriptor/15036-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70265">
    <rdfs:label xml:lang="de">Druckereiprodukte</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73394">
    <skos:broader rdf:resource="thsys/70096"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19545-5">
    <skos:related rdf:resource="descriptor/12946-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19545-5">
    <skos:related rdf:resource="descriptor/19677-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19545-5">
    <skos:related rdf:resource="descriptor/19183-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19545-5">
    <skos:related rdf:resource="descriptor/14124-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10615-6">
    <skos:altLabel xml:lang="en">International factor movements</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10615-6">
    <skos:altLabel xml:lang="de">Internationale Faktorbewegung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10615-6">
    <skos:altLabel xml:lang="de">Internationale Faktorallokation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14365-4">
    <skos:prefLabel xml:lang="en">Excavator</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14365-4">
    <skos:prefLabel xml:lang="de">Bagger</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20817-2">
    <skos:scopeNote xml:lang="de">In der Dritte-Welt-Bewegung übliche Bezeichnung. Ziel des fairen Handels ist es, den Erzeugern in den Entwicklungsländern einen Preis zu zahlen, der ihren Ressourceneinsatz angemessen berücksichtigt.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11406-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15080-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13268-0">
    <skos:altLabel xml:lang="en">Wireless communication system</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13268-0">
    <skos:altLabel xml:lang="de">Seefunkdienst</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13268-0">
    <skos:altLabel xml:lang="de">Funkdienst</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13268-0">
    <skos:altLabel xml:lang="de">Drahtloses Netz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13268-0">
    <skos:altLabel xml:lang="de">Drahtlose Telekommunikation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13268-0">
    <skos:altLabel xml:lang="de">Drahtlose Nachrichtentechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18499-5">
    <skos:related rdf:resource="descriptor/18797-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18499-5">
    <skos:related rdf:resource="descriptor/12783-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18499-5">
    <skos:related rdf:resource="descriptor/13186-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19148-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14911-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19616-1">
    <skos:prefLabel xml:lang="en">Territorial sea</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19616-1">
    <skos:prefLabel xml:lang="de">Küstengewässer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17792-5">
    <skos:broader rdf:resource="thsys/70086"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17792-5">
    <skos:broader rdf:resource="descriptor/17791-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13890-4">
    <skos:broader rdf:resource="thsys/71056"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13890-4">
    <skos:broader rdf:resource="thsys/70996"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id318">
    <rdfs:seeAlso rdf:resource="descriptor/11692-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14840-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10016-0">
    <skos:prefLabel xml:lang="en">Product</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10016-0">
    <skos:prefLabel xml:lang="de">Produkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70999">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id251">
    <rdfs:seeAlso rdf:resource="descriptor/14433-6"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id18">
    <rdf:value xml:lang="de">Für Regelungen zum Gefahren- und Unfallschutz am Arbeitsplatz</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15919-3">
    <skos:altLabel xml:lang="en">Displaced persons</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15919-3">
    <skos:altLabel xml:lang="de">Vertriebener</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15919-3">
    <skos:altLabel xml:lang="de">Vertriebene</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15919-3">
    <skos:altLabel xml:lang="de">Politische Flüchtlinge</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15919-3">
    <skos:altLabel xml:lang="de">Flüchtling</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15919-3">
    <skos:altLabel xml:lang="de">Exilierte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15919-3">
    <skos:altLabel xml:lang="de">Asylanten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18224-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70080">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">G.04.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10745-0">
    <skos:prefLabel xml:lang="en">Currency agreement</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10745-0">
    <skos:prefLabel xml:lang="de">Internationale Währungspolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id349">
    <rdfs:seeAlso rdf:resource="descriptor/15860-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18211-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18726-6">
    <skos:narrower rdf:resource="descriptor/13795-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20110-2">
    <skos:prefLabel xml:lang="en">TRIPS agreement</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20110-2">
    <skos:prefLabel xml:lang="de">TRIPS</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13238-2">
    <skos:broader rdf:resource="thsys/70960"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13238-2">
    <skos:broader rdf:resource="descriptor/13210-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13793-2">
    <skos:related rdf:resource="descriptor/18271-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19085-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11976-4">
    <skos:related rdf:resource="descriptor/10512-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11976-4">
    <skos:related rdf:resource="descriptor/11991-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11976-4">
    <skos:related rdf:resource="descriptor/11990-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11976-4">
    <skos:related rdf:resource="descriptor/11993-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11976-4">
    <skos:related rdf:resource="descriptor/11996-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11976-4">
    <skos:related rdf:resource="descriptor/11902-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18416-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71053">
    <rdfs:label xml:lang="en">Caring professions</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71053">
    <rdfs:label xml:lang="de">Sozialberufe</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15685-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id164">
    <rdfs:seeAlso rdf:resource="descriptor/12225-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10616-4">
    <skos:prefLabel xml:lang="en">International trade policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10616-4">
    <skos:prefLabel xml:lang="de">Internationale Handelspolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71072">
    <skos:narrower rdf:resource="descriptor/12931-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71072">
    <skos:narrower rdf:resource="descriptor/11633-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71072">
    <skos:narrower rdf:resource="descriptor/11142-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71072">
    <skos:narrower rdf:resource="descriptor/18822-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71072">
    <skos:narrower rdf:resource="descriptor/11628-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71072">
    <skos:narrower rdf:resource="descriptor/13249-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71072">
    <skos:narrower rdf:resource="descriptor/10267-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71072">
    <skos:narrower rdf:resource="descriptor/11629-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71072">
    <skos:narrower rdf:resource="descriptor/11612-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71072">
    <skos:narrower rdf:resource="descriptor/11556-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71072">
    <skos:narrower rdf:resource="descriptor/18705-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71072">
    <skos:narrower rdf:resource="descriptor/11627-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71072">
    <skos:narrower rdf:resource="descriptor/11552-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71072">
    <skos:narrower rdf:resource="descriptor/11626-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15988-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19439-4">
    <skos:scopeNote xml:lang="de">Arbeitslosigkeit in einer Region im Verhältnis zu anderen Regionen oder zu einer übergeordneten räumlichen Einheit.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id297">
    <rdf:value xml:lang="de">Für Sozialversicherungen</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14005-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16588-2">
    <skos:prefLabel xml:lang="en">Society</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16588-2">
    <skos:prefLabel xml:lang="de">Gesellschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18191-5">
    <skos:prefLabel xml:lang="en">International coffee agreement</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18191-5">
    <skos:prefLabel xml:lang="de">Kaffeeabkommen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10462-5">
    <skos:related rdf:resource="descriptor/13079-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10462-5">
    <skos:related rdf:resource="descriptor/14465-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10462-5">
    <skos:related rdf:resource="descriptor/19796-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10462-5">
    <skos:related rdf:resource="descriptor/15415-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10962-6">
    <skos:narrower rdf:resource="descriptor/10960-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16730-4">
    <skos:broader rdf:resource="thsys/73393"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16730-4">
    <skos:broader rdf:resource="thsys/70049"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16730-4">
    <skos:broader rdf:resource="descriptor/16726-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13076-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13510-5">
    <skos:editorialNote xml:lang="de">Möglichst spezifischer indexieren</skos:editorialNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10058-5">
    <skos:related rdf:resource="descriptor/10112-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10058-5">
    <skos:related rdf:resource="descriptor/19670-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10058-5">
    <skos:related rdf:resource="descriptor/10111-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10058-5">
    <skos:related rdf:resource="descriptor/15773-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10058-5">
    <skos:related rdf:resource="descriptor/15763-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10058-5">
    <skos:related rdf:resource="descriptor/12394-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10058-5">
    <skos:related rdf:resource="descriptor/19237-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10058-5">
    <skos:related rdf:resource="descriptor/10220-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10058-5">
    <skos:related rdf:resource="descriptor/16470-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10058-5">
    <skos:related rdf:resource="descriptor/19831-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10058-5">
    <skos:related rdf:resource="descriptor/10092-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10058-5">
    <skos:related rdf:resource="descriptor/19254-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16401-2">
    <skos:broader rdf:resource="thsys/73388"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16401-2">
    <skos:broader rdf:resource="thsys/70850"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16401-2">
    <skos:broader rdf:resource="thsys/70835"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16401-2">
    <skos:broader rdf:resource="thsys/70504"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16401-2">
    <skos:broader rdf:resource="descriptor/16544-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16401-2">
    <skos:broader rdf:resource="descriptor/15640-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10909-3">
    <skos:broader rdf:resource="thsys/70101"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10909-3">
    <skos:broader rdf:resource="descriptor/19565-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10909-3">
    <skos:broader rdf:resource="descriptor/10895-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19158-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73340">
    <rdfs:label xml:lang="de">Programmierung und Software</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19408-1">
    <skos:related rdf:resource="descriptor/19400-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10780-5">
    <skos:prefLabel xml:lang="en">Foreign exchange regime</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10780-5">
    <skos:prefLabel xml:lang="de">Devisenrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14617-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id333">
    <rdfs:seeAlso rdf:resource="descriptor/14744-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18453-1">
    <skos:related rdf:resource="descriptor/18165-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18453-1">
    <skos:related rdf:resource="descriptor/15019-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19799-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19515-0">
    <skos:altLabel xml:lang="en">Pay as you go</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19515-0">
    <skos:altLabel xml:lang="de">PAYG</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19388-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18383-3">
    <skos:altLabel xml:lang="de">Grubenbetriebslehre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18383-3">
    <skos:altLabel xml:lang="de">Bergbaubetriebswirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11444-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13072-3">
    <skos:prefLabel xml:lang="en">Computer industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13072-3">
    <skos:prefLabel xml:lang="de">Computerindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70227">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19123-3">
    <skos:broader rdf:resource="thsys/70041"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#memberList">
    <rdfs:comment xml:lang="en">For any resource, every item in the list given as the value of the
      skos:memberList property is also a value of the skos:member property.</rdfs:comment>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14173-6">
    <skos:related rdf:resource="descriptor/14218-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14173-6">
    <skos:related rdf:resource="descriptor/18250-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14173-6">
    <skos:related rdf:resource="descriptor/14192-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11661-1">
    <skos:prefLabel xml:lang="en">Indirect tax</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11661-1">
    <skos:prefLabel xml:lang="de">Indirekte Steuer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14151-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13697-5">
    <skos:prefLabel xml:lang="en">Universal banking</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13697-5">
    <skos:prefLabel xml:lang="de">Universalbank</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18924-2">
    <skos:broader rdf:resource="thsys/73386"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18924-2">
    <skos:broader rdf:resource="descriptor/15811-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19014-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19517-3">
    <skos:broader rdf:resource="thsys/70839"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19517-3">
    <skos:broader rdf:resource="thsys/70573"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19517-3">
    <skos:broader rdf:resource="descriptor/18913-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11191-5">
    <skos:broader rdf:resource="thsys/70244"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11191-5">
    <skos:broader rdf:resource="thsys/70051"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11191-5">
    <skos:broader rdf:resource="descriptor/11185-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11191-5">
    <skos:broader rdf:resource="descriptor/11184-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11134-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19808-6">
    <skos:broader rdf:resource="thsys/73339"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16874-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73385">
    <skos:narrower rdf:resource="descriptor/12108-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73385">
    <skos:narrower rdf:resource="descriptor/18116-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73385">
    <skos:narrower rdf:resource="descriptor/12620-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73385">
    <skos:narrower rdf:resource="descriptor/12585-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73385">
    <skos:narrower rdf:resource="descriptor/12574-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73385">
    <skos:narrower rdf:resource="descriptor/12571-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73385">
    <skos:narrower rdf:resource="descriptor/19009-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73385">
    <skos:narrower rdf:resource="descriptor/15839-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73385">
    <skos:narrower rdf:resource="descriptor/19002-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73385">
    <skos:narrower rdf:resource="descriptor/11052-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73385">
    <skos:narrower rdf:resource="descriptor/12580-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73385">
    <skos:narrower rdf:resource="descriptor/12103-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73385">
    <skos:narrower rdf:resource="descriptor/12588-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73385">
    <skos:narrower rdf:resource="descriptor/12171-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73385">
    <skos:narrower rdf:resource="descriptor/12119-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73385">
    <skos:narrower rdf:resource="descriptor/15825-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73385">
    <skos:narrower rdf:resource="descriptor/15823-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73385">
    <skos:narrower rdf:resource="descriptor/12570-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73385">
    <skos:narrower rdf:resource="descriptor/12577-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73385">
    <skos:narrower rdf:resource="descriptor/15842-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73385">
    <skos:narrower rdf:resource="descriptor/11295-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73385">
    <skos:narrower rdf:resource="descriptor/12569-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73385">
    <skos:narrower rdf:resource="descriptor/15837-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73385">
    <skos:narrower rdf:resource="descriptor/25881-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73385">
    <skos:narrower rdf:resource="descriptor/12617-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73385">
    <skos:narrower rdf:resource="descriptor/12607-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73385">
    <skos:narrower rdf:resource="descriptor/11181-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73385">
    <skos:narrower rdf:resource="descriptor/11303-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11163-3">
    <skos:narrower rdf:resource="descriptor/15719-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11163-3">
    <skos:narrower rdf:resource="descriptor/16288-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19093-0">
    <skos:scopeNote xml:lang="de">Bestand an Geld- und Vermögenswerten eines Betriebes, die bei Bedarf in Geld verwandelt werden können.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12618-4">
    <skos:altLabel xml:lang="de">Berufsbedingte Erkrankung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19107-1">
    <skos:prefLabel xml:lang="en">Benchmarking</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19107-1">
    <skos:prefLabel xml:lang="de">Benchmarking</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18418-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11800-6">
    <skos:broader rdf:resource="thsys/71020"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11800-6">
    <skos:broader rdf:resource="descriptor/10894-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11800-6">
    <skos:broader rdf:resource="descriptor/11801-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18179-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19245-3">
    <skos:scopeNote xml:lang="de">Paradoxe Wirkung hoher Rohstoffexporte auf die einheimische Wirtschaft: Diverse Effekte, u.a. Aufwertungstendenzen der einheimischen Währung, können zur Schwächung der Volkswirtschaft führen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70114">
    <skos:prefLabel xml:lang="en">V.05.07  Payments</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70114">
    <skos:prefLabel xml:lang="de">V.05.07  Zahlungsverkehr</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70648">
    <skos:broader rdf:resource="thsys/70538"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10269-6">
    <skos:related rdf:resource="descriptor/10268-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10269-6">
    <skos:related rdf:resource="descriptor/10265-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10269-6">
    <skos:related rdf:resource="descriptor/11032-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18932-3">
    <skos:related rdf:resource="descriptor/12837-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18932-3">
    <skos:related rdf:resource="descriptor/12829-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18932-3">
    <skos:related rdf:resource="descriptor/18027-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18932-3">
    <skos:related rdf:resource="descriptor/18930-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18932-3">
    <skos:related rdf:resource="descriptor/16680-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18932-3">
    <skos:related rdf:resource="descriptor/13658-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18932-3">
    <skos:related rdf:resource="descriptor/16759-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18932-3">
    <skos:related rdf:resource="descriptor/18916-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18932-3">
    <skos:related rdf:resource="descriptor/16783-4"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id266">
    <rdf:value xml:lang="de">Für Produktivgenossenschaft</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10704-0">
    <skos:broader rdf:resource="thsys/71098"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10704-0">
    <skos:broader rdf:resource="descriptor/10687-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14905-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13613-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18126-2">
    <skos:broader rdf:resource="thsys/71091"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18126-2">
    <skos:broader rdf:resource="thsys/70958"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18126-2">
    <skos:broader rdf:resource="descriptor/10605-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15729-1">
    <skos:broader rdf:resource="thsys/70238"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15729-1">
    <skos:broader rdf:resource="thsys/70161"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15729-1">
    <skos:broader rdf:resource="descriptor/14188-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15729-1">
    <skos:broader rdf:resource="descriptor/15009-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12889-1">
    <skos:related rdf:resource="descriptor/12881-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12889-1">
    <skos:related rdf:resource="descriptor/11709-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12889-1">
    <skos:related rdf:resource="descriptor/11640-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12889-1">
    <skos:related rdf:resource="descriptor/12876-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12889-1">
    <skos:related rdf:resource="descriptor/13871-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18372-1">
    <skos:related rdf:resource="descriptor/18373-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18372-1">
    <skos:related rdf:resource="descriptor/12984-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18372-1">
    <skos:related rdf:resource="descriptor/12983-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12285-5">
    <skos:scopeNote xml:lang="de">Erträge aus der Anlage von Kapital.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16978-3">
    <skos:broader rdf:resource="thsys/70003"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16978-3">
    <skos:broader rdf:resource="descriptor/16972-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11692-4">
    <skos:related rdf:resource="descriptor/11695-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11692-4">
    <skos:related rdf:resource="descriptor/11735-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11692-4">
    <skos:related rdf:resource="descriptor/11713-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14957-0">
    <skos:altLabel xml:lang="de">Margarine</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14957-0">
    <skos:altLabel xml:lang="de">Butter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11375-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19354-5">
    <skos:narrower rdf:resource="descriptor/20810-2"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id330">
    <rdf:value xml:lang="de">Für die Exploration von Erdöl- und Ergaslagerstätten</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id292">
    <rdf:value xml:lang="de">Für den "Secondary Labour Market Sector"</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11969-1">
    <skos:related rdf:resource="descriptor/11992-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11969-1">
    <skos:related rdf:resource="descriptor/11999-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11969-1">
    <skos:related rdf:resource="descriptor/11983-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11969-1">
    <skos:related rdf:resource="descriptor/11994-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11969-1">
    <skos:related rdf:resource="descriptor/19474-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11969-1">
    <skos:related rdf:resource="descriptor/16663-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11969-1">
    <skos:related rdf:resource="descriptor/16671-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11969-1">
    <skos:related rdf:resource="descriptor/10504-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11969-1">
    <skos:related rdf:resource="descriptor/19469-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11070-3">
    <skos:altLabel xml:lang="de">Proletarisierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11070-3">
    <skos:altLabel xml:lang="de">Proletariat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16186-1">
    <skos:scopeNote xml:lang="de">Im wesentlichen die Rechtsnormen des internationalen Privatrechts, des supranationalen Rechts (Recht, das von internationalen Organisationen für mehrere Staaten einheitlich erlassen wird, z.B. Gemeinschaftsrecht der EU) und des Völkerrechts.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13508-6">
    <skos:broader rdf:resource="thsys/70943"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13508-6">
    <skos:broader rdf:resource="descriptor/13510-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19194-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18960-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11921-1">
    <skos:altLabel xml:lang="de">Land-Use Model</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11921-1">
    <skos:altLabel xml:lang="de">Flächennutzungstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11921-1">
    <skos:altLabel xml:lang="de">Bodenökonomie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10255-3">
    <skos:broader rdf:resource="thsys/71060"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10255-3">
    <skos:broader rdf:resource="thsys/70355"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70963">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13047-2">
    <skos:broader rdf:resource="thsys/70999"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13047-2">
    <skos:broader rdf:resource="descriptor/12993-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13821-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16111-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12354-5">
    <skos:prefLabel xml:lang="en">Fixed assets</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12354-5">
    <skos:prefLabel xml:lang="de">Betriebliches Anlagevermögen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16333-0">
    <skos:related rdf:resource="descriptor/16330-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16333-0">
    <skos:related rdf:resource="descriptor/19698-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16333-0">
    <skos:related rdf:resource="descriptor/13989-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16333-0">
    <skos:related rdf:resource="descriptor/16298-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16333-0">
    <skos:related rdf:resource="descriptor/16658-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16333-0">
    <skos:related rdf:resource="descriptor/19748-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73344">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19263-1">
    <skos:prefLabel xml:lang="en">Target zone</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19263-1">
    <skos:prefLabel xml:lang="de">Target Zone</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18986-1">
    <skos:related rdf:resource="descriptor/20409-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12957-3">
    <skos:altLabel xml:lang="de">Wiese</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12957-3">
    <skos:altLabel xml:lang="de">Weide</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12957-3">
    <skos:altLabel xml:lang="de">Grünlandwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11963-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12613-0">
    <skos:broader rdf:resource="thsys/73345"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12613-0">
    <skos:broader rdf:resource="thsys/71050"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12613-0">
    <skos:broader rdf:resource="thsys/70201"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12613-0">
    <skos:broader rdf:resource="descriptor/10470-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11479-4">
    <skos:broader rdf:resource="thsys/70305"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11479-4">
    <skos:broader rdf:resource="descriptor/10250-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11479-4">
    <skos:broader rdf:resource="descriptor/10382-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12056-6">
    <skos:altLabel xml:lang="de">Vertragskonzern</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12056-6">
    <skos:altLabel xml:lang="de">Unterordnungskonzern</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12056-6">
    <skos:altLabel xml:lang="de">Teilkonzern</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12056-6">
    <skos:altLabel xml:lang="de">Muttergesellschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12056-6">
    <skos:altLabel xml:lang="de">Gleichordnungskonzern</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12056-6">
    <skos:altLabel xml:lang="de">Faktischer Konzern</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18127-0">
    <skos:prefLabel xml:lang="en">Currency substitution</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18127-0">
    <skos:prefLabel xml:lang="de">Währungssubstitution</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10304-2">
    <skos:related rdf:resource="descriptor/10122-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10304-2">
    <skos:related rdf:resource="descriptor/10440-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10304-2">
    <skos:related rdf:resource="descriptor/11180-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18617-4">
    <skos:altLabel xml:lang="en">Nodal centres for goods</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11627-1">
    <skos:prefLabel xml:lang="en">Municipal enterprise</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11627-1">
    <skos:prefLabel xml:lang="de">Kommunales Unternehmen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70869">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18142-4">
    <skos:scopeNote xml:lang="de">Frauenpolitik bezeichnet alle Aktivitäten, die auf eine Besserstellung/Angleichung der Lebensbedingungen/Lebensverhältnisse von Frauen hinzielen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18812-6">
    <skos:broader rdf:resource="thsys/71012"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18812-6">
    <skos:broader rdf:resource="thsys/70539"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18812-6">
    <skos:broader rdf:resource="descriptor/10978-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18812-6">
    <skos:broader rdf:resource="descriptor/11810-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71064">
    <rdfs:label xml:lang="en">Urban economics</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71064">
    <rdfs:label xml:lang="de">Stadtökonomik</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19227-5">
    <skos:scopeNote xml:lang="de">Trennung "guter" und "schlechter" Risiken vor allem auf Versicherungs- und Kreditmärkten; z.B. Angebot von Lady- oder Wenigfahrer-Tarifen bei der Kfz-Versicherung</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10769-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15745-3">
    <skos:scopeNote xml:lang="de">Für Ressourcen im Sinne von Rohstoffen BENUTZE Rohstoff oder Rohstoffressourcen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15745-3">
    <skos:scopeNote xml:lang="de">Nur für Literatur über natürliche Ressourcen allgemein unter Einschluss von Luft, Wasser, Landschaft usw.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18904-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13400-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18440-3">
    <skos:prefLabel xml:lang="en">Bakery industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18440-3">
    <skos:prefLabel xml:lang="de">Backgewerbe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18068-4">
    <skos:narrower rdf:resource="descriptor/15963-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19613-0">
    <skos:broader rdf:resource="thsys/73354"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19613-0">
    <skos:broader rdf:resource="descriptor/14046-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14042-0">
    <skos:broader rdf:resource="thsys/70201"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13815-4">
    <skos:related rdf:resource="descriptor/15889-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13815-4">
    <skos:related rdf:resource="descriptor/18069-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19595-4">
    <skos:prefLabel xml:lang="en">Shapley value</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19595-4">
    <skos:prefLabel xml:lang="de">Shapley-Wert</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18175-3">
    <skos:broader rdf:resource="thsys/73341"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18175-3">
    <skos:broader rdf:resource="thsys/71037"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18175-3">
    <skos:broader rdf:resource="descriptor/15599-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71034">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18120-0">
    <skos:prefLabel xml:lang="en">Small and medium-sized enterprises</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18120-0">
    <skos:prefLabel xml:lang="de">Klein- und Mittelunternehmen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18081-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13225-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13617-1">
    <skos:related rdf:resource="descriptor/13570-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13617-1">
    <skos:related rdf:resource="descriptor/18613-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13617-1">
    <skos:related rdf:resource="descriptor/15992-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12969-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14436-0">
    <skos:broader rdf:resource="thsys/70258"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14436-0">
    <skos:broader rdf:resource="descriptor/14438-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18082-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13355-5">
    <skos:related rdf:resource="descriptor/13392-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13355-5">
    <skos:related rdf:resource="descriptor/14004-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13355-5">
    <skos:related rdf:resource="descriptor/15655-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13355-5">
    <skos:related rdf:resource="descriptor/16680-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13355-5">
    <skos:related rdf:resource="descriptor/18920-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13355-5">
    <skos:related rdf:resource="descriptor/18923-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13355-5">
    <skos:related rdf:resource="descriptor/18070-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13355-5">
    <skos:related rdf:resource="descriptor/13871-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73301">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">P.20.04</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12941-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18691-6">
    <skos:broader rdf:resource="thsys/71104"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18691-6">
    <skos:broader rdf:resource="thsys/70893"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18691-6">
    <skos:broader rdf:resource="thsys/70187"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18691-6">
    <skos:broader rdf:resource="descriptor/13279-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16295-3">
    <skos:broader rdf:resource="thsys/73381"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16295-3">
    <skos:broader rdf:resource="thsys/73361"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13716-6">
    <skos:altLabel xml:lang="en">Stockbroker</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13716-6">
    <skos:altLabel xml:lang="en">Marketmaker</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13716-6">
    <skos:altLabel xml:lang="de">Wertpapiermaklerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13716-6">
    <skos:altLabel xml:lang="de">Wertpapiermakler</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13716-6">
    <skos:altLabel xml:lang="de">Wertpapierhändlerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13716-6">
    <skos:altLabel xml:lang="de">Wertpapierhändler</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13716-6">
    <skos:altLabel xml:lang="de">Effektenmaklerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13716-6">
    <skos:altLabel xml:lang="de">Effektenmakler</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13716-6">
    <skos:altLabel xml:lang="de">Devisenhändlerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13716-6">
    <skos:altLabel xml:lang="de">Devisenhändler</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13716-6">
    <skos:altLabel xml:lang="de">Börsenmaklerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13620-5">
    <skos:broader rdf:resource="thsys/70940"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13620-5">
    <skos:broader rdf:resource="descriptor/13621-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19771-3">
    <skos:altLabel xml:lang="de">Phobie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19771-3">
    <skos:altLabel xml:lang="de">Furcht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19771-3">
    <skos:altLabel xml:lang="de">Aversion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18592-1">
    <skos:altLabel xml:lang="de">Seeschifffahrtsregister</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18592-1">
    <skos:altLabel xml:lang="de">Seeschiffahrtsregister</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13861-4">
    <skos:broader rdf:resource="thsys/73355"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13861-4">
    <skos:broader rdf:resource="thsys/70888"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13861-4">
    <skos:broader rdf:resource="descriptor/13860-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13861-4">
    <skos:broader rdf:resource="descriptor/16564-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16070-6">
    <skos:scopeNote xml:lang="de">Regelt national und international vor allem die Befrachtung und Versicherung von Seetransporten.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15993-5">
    <skos:prefLabel xml:lang="en">Energy law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15993-5">
    <skos:prefLabel xml:lang="de">Energierecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11349-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18511-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10241-0">
    <skos:related rdf:resource="descriptor/11787-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10241-0">
    <skos:related rdf:resource="descriptor/19231-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10241-0">
    <skos:related rdf:resource="descriptor/10237-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10241-0">
    <skos:related rdf:resource="descriptor/10377-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10241-0">
    <skos:related rdf:resource="descriptor/10229-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11806-1">
    <skos:altLabel xml:lang="de">EU-Agrarmarktpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11806-1">
    <skos:altLabel xml:lang="de">EG-Agrarmarktpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11806-1">
    <skos:altLabel xml:lang="de">EG-Agrarmarktordnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11248-2">
    <skos:narrower rdf:resource="descriptor/18166-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11248-2">
    <skos:narrower rdf:resource="descriptor/11212-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11248-2">
    <skos:narrower rdf:resource="descriptor/12628-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11248-2">
    <skos:narrower rdf:resource="descriptor/19504-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11248-2">
    <skos:narrower rdf:resource="descriptor/12624-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11248-2">
    <skos:narrower rdf:resource="descriptor/11253-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70815">
    <skos:narrower rdf:resource="descriptor/15774-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70815">
    <skos:narrower rdf:resource="descriptor/15074-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70815">
    <skos:narrower rdf:resource="descriptor/12928-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70815">
    <skos:narrower rdf:resource="descriptor/15773-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70815">
    <skos:narrower rdf:resource="descriptor/15771-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70815">
    <skos:narrower rdf:resource="descriptor/15770-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70815">
    <skos:narrower rdf:resource="descriptor/18071-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70815">
    <skos:narrower rdf:resource="descriptor/15769-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70815">
    <skos:narrower rdf:resource="descriptor/15768-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70815">
    <skos:narrower rdf:resource="descriptor/15767-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70815">
    <skos:narrower rdf:resource="descriptor/18165-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70815">
    <skos:narrower rdf:resource="descriptor/15765-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70815">
    <skos:narrower rdf:resource="descriptor/18966-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70815">
    <skos:narrower rdf:resource="descriptor/20125-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70815">
    <skos:narrower rdf:resource="descriptor/15763-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70815">
    <skos:narrower rdf:resource="descriptor/18967-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70815">
    <skos:narrower rdf:resource="descriptor/15746-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19782-5">
    <skos:broader rdf:resource="thsys/70151"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19782-5">
    <skos:broader rdf:resource="descriptor/15664-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19782-5">
    <skos:broader rdf:resource="descriptor/10436-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12311-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14366-2">
    <skos:broader rdf:resource="thsys/70318"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14366-2">
    <skos:broader rdf:resource="descriptor/14381-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19614-5">
    <skos:scopeNote xml:lang="de">Wirtschaftliche Nutzung des Meeres</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11212-2">
    <skos:broader rdf:resource="thsys/71059"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11212-2">
    <skos:broader rdf:resource="thsys/71050"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11212-2">
    <skos:broader rdf:resource="descriptor/19491-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11212-2">
    <skos:broader rdf:resource="descriptor/11248-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11212-2">
    <skos:broader rdf:resource="descriptor/11187-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13782-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15675-5">
    <skos:altLabel xml:lang="de">Leichtathletik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15675-5">
    <skos:altLabel xml:lang="de">Kampfsport</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15675-5">
    <skos:altLabel xml:lang="de">Freizeitsport</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15675-5">
    <skos:altLabel xml:lang="de">Breitensport</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15675-5">
    <skos:altLabel xml:lang="de">Amateursport</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13398-1">
    <skos:broader rdf:resource="thsys/70814"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13398-1">
    <skos:broader rdf:resource="thsys/70493"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13024-0">
    <skos:broader rdf:resource="thsys/71008"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13024-0">
    <skos:broader rdf:resource="descriptor/13014-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19135-3">
    <skos:altLabel xml:lang="de">Schaufenstergestaltung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15664-3">
    <skos:narrower rdf:resource="descriptor/15669-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15664-3">
    <skos:narrower rdf:resource="descriptor/19782-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15664-3">
    <skos:narrower rdf:resource="descriptor/15876-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15664-3">
    <skos:narrower rdf:resource="descriptor/15638-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15664-3">
    <skos:narrower rdf:resource="descriptor/15633-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19509-2">
    <zbwext:useInsteadNote rdf:nodeID="id151"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14829-2">
    <skos:related rdf:resource="descriptor/14383-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14829-2">
    <skos:related rdf:resource="descriptor/14831-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14829-2">
    <skos:related rdf:resource="descriptor/13135-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14829-2">
    <skos:related rdf:resource="descriptor/14127-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19044-6">
    <skos:broader rdf:resource="thsys/70582"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18800-6">
    <skos:altLabel xml:lang="en">Urban infrastructure</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18800-6">
    <skos:altLabel xml:lang="de">Städtische Infrastruktur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11243-5">
    <skos:altLabel xml:lang="en">Job creation program</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11243-5">
    <skos:altLabel xml:lang="de">Arbeitsbeschaffungsmaßnahmen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11243-5">
    <skos:altLabel xml:lang="de">Arbeitsbeschaffungsmaßnahme</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11243-5">
    <skos:altLabel xml:lang="de">ABM (Arbeitsbeschaffungsmaßnahme)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14160-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10087-5">
    <skos:broader rdf:resource="thsys/70234"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10087-5">
    <skos:broader rdf:resource="thsys/70188"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10087-5">
    <skos:broader rdf:resource="descriptor/10249-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15919-3">
    <skos:broader rdf:resource="thsys/73356"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15919-3">
    <skos:broader rdf:resource="thsys/70572"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15919-3">
    <skos:broader rdf:resource="descriptor/15909-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71044">
    <skos:prefLabel xml:lang="de">B.02.02.01  Investitionsplanung und -rechnung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15108-5">
    <zbwext:useInsteadNote rdf:nodeID="id344"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10196-0">
    <skos:prefLabel xml:lang="en">Cobweb</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10196-0">
    <skos:prefLabel xml:lang="de">Cobweb</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70310">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">B.06.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13401-3">
    <skos:related rdf:resource="descriptor/16485-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13401-3">
    <skos:related rdf:resource="descriptor/18976-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13401-3">
    <skos:related rdf:resource="descriptor/10298-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13401-3">
    <skos:related rdf:resource="descriptor/18908-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12302-3">
    <skos:related rdf:resource="descriptor/24859-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12302-3">
    <skos:related rdf:resource="descriptor/10019-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12302-3">
    <skos:related rdf:resource="descriptor/12253-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12302-3">
    <skos:related rdf:resource="descriptor/19159-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12302-3">
    <skos:related rdf:resource="descriptor/16058-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12302-3">
    <skos:related rdf:resource="descriptor/13799-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12302-3">
    <skos:related rdf:resource="descriptor/12228-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16080-3">
    <skos:related rdf:resource="descriptor/16081-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16080-3">
    <skos:related rdf:resource="descriptor/13742-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16080-3">
    <skos:related rdf:resource="descriptor/18682-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16080-3">
    <skos:related rdf:resource="descriptor/12299-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16080-3">
    <skos:related rdf:resource="descriptor/19652-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16080-3">
    <skos:related rdf:resource="descriptor/13731-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16080-3">
    <skos:related rdf:resource="descriptor/13693-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16080-3">
    <skos:related rdf:resource="descriptor/16043-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20744-3">
    <skos:prefLabel xml:lang="en">Clean development mechanism</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20744-3">
    <skos:prefLabel xml:lang="de">Clean Development Mechanism</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12366-5">
    <skos:altLabel xml:lang="de">Finanzanlagevermögen des Betriebs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14899-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73358">
    <skos:prefLabel xml:lang="de">N.06.03.02  Soziale Gruppen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15981-5">
    <skos:prefLabel xml:lang="en">Law of procedure</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15981-5">
    <skos:prefLabel xml:lang="de">Verfahrensrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11673-1">
    <skos:broader rdf:resource="thsys/71074"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11673-1">
    <skos:broader rdf:resource="thsys/70992"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11673-1">
    <skos:broader rdf:resource="descriptor/11675-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11673-1">
    <skos:broader rdf:resource="descriptor/18373-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11673-1">
    <skos:broader rdf:resource="descriptor/18376-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15416-1">
    <skos:narrower rdf:resource="descriptor/19809-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15416-1">
    <skos:narrower rdf:resource="descriptor/18003-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71013">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10115-5">
    <skos:broader rdf:resource="thsys/70486"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10115-5">
    <skos:broader rdf:resource="descriptor/10024-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10115-5">
    <skos:broader rdf:resource="descriptor/10120-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11905-6">
    <skos:related rdf:resource="descriptor/18490-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11905-6">
    <skos:related rdf:resource="descriptor/18748-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11905-6">
    <skos:related rdf:resource="descriptor/18122-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11905-6">
    <skos:related rdf:resource="descriptor/11981-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11905-6">
    <skos:related rdf:resource="descriptor/11904-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11998-1">
    <skos:related rdf:resource="descriptor/11999-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11998-1">
    <skos:related rdf:resource="descriptor/11844-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11998-1">
    <skos:related rdf:resource="descriptor/19450-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11998-1">
    <skos:related rdf:resource="descriptor/16549-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11998-1">
    <skos:related rdf:resource="descriptor/11852-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11998-1">
    <skos:related rdf:resource="descriptor/16266-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12525-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14423-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18281-4">
    <skos:prefLabel xml:lang="en">Cogeneration power plant</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18281-4">
    <skos:prefLabel xml:lang="de">Blockheizkraftwerk</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20762-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12651-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18938-5">
    <skos:broader rdf:resource="thsys/70824"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18938-5">
    <skos:broader rdf:resource="descriptor/18005-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70028">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10045-0">
    <skos:prefLabel xml:lang="en">Lag model</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10045-0">
    <skos:prefLabel xml:lang="de">Lag-Modell</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id86">
    <rdfs:seeAlso rdf:resource="descriptor/10589-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11295-0">
    <skos:prefLabel xml:lang="en">Occupational choice</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11295-0">
    <skos:prefLabel xml:lang="de">Berufswahl</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18970-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11653-0">
    <skos:related rdf:resource="descriptor/11701-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11653-0">
    <skos:related rdf:resource="descriptor/11668-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11653-0">
    <skos:related rdf:resource="descriptor/19420-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19941-4">
    <skos:broader rdf:resource="thsys/70144"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19941-4">
    <skos:broader rdf:resource="descriptor/11539-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11163-3">
    <skos:editorialNote xml:lang="de">Möglichst spezifischer indexieren.</skos:editorialNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11873-0">
    <skos:altLabel xml:lang="en">Spatial interaction theory</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11873-0">
    <skos:altLabel xml:lang="de">Regionale Interaktionstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11873-0">
    <skos:altLabel xml:lang="de">Regionale Interaktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11873-0">
    <skos:altLabel xml:lang="de">Räumliche Interaktionstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11873-0">
    <skos:altLabel xml:lang="de">Potenzialmodell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11873-0">
    <skos:altLabel xml:lang="de">Potentialmodell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18549-2">
    <skos:related rdf:resource="descriptor/12675-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19720-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19473-4">
    <skos:broader rdf:resource="thsys/73351"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19473-4">
    <skos:broader rdf:resource="thsys/71064"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19473-4">
    <skos:broader rdf:resource="thsys/70239"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19473-4">
    <skos:broader rdf:resource="descriptor/19449-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19473-4">
    <skos:broader rdf:resource="descriptor/11998-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18353-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15970-3">
    <skos:related rdf:resource="descriptor/16504-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15970-3">
    <skos:related rdf:resource="descriptor/15976-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15970-3">
    <skos:related rdf:resource="descriptor/19607-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15970-3">
    <skos:related rdf:resource="descriptor/15966-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15970-3">
    <skos:related rdf:resource="descriptor/16509-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15970-3">
    <skos:related rdf:resource="descriptor/13410-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12946-1">
    <skos:altLabel xml:lang="de">Milcherzeugungswirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14343-0">
    <skos:related rdf:resource="descriptor/18398-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14343-0">
    <skos:related rdf:resource="descriptor/18415-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14343-0">
    <skos:related rdf:resource="descriptor/12930-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14343-0">
    <skos:related rdf:resource="descriptor/14051-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18546-1">
    <skos:prefLabel xml:lang="en">Retail location</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18546-1">
    <skos:prefLabel xml:lang="de">Einzelhandelsstandort</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11573-5">
    <skos:prefLabel xml:lang="en">Reparations</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11573-5">
    <skos:prefLabel xml:lang="de">Wiedergutmachung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19438-6">
    <skos:prefLabel xml:lang="en">Spatial distribution</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19438-6">
    <skos:prefLabel xml:lang="de">Räumliche Verteilung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12891-0">
    <skos:broader rdf:resource="thsys/71041"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12891-0">
    <skos:broader rdf:resource="descriptor/12882-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18739-4">
    <skos:prefLabel xml:lang="en">Private pension</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18739-4">
    <skos:prefLabel xml:lang="de">Private Rentenversicherung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71116">
    <rdfs:label xml:lang="en">Time in economics</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71116">
    <rdfs:label xml:lang="de">Zeitökonomik</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17941-0">
    <skos:broader rdf:resource="thsys/70128"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17941-0">
    <skos:broader rdf:resource="descriptor/17948-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19428-2">
    <skos:scopeNote xml:lang="de">Für Netzwerke im Sinne von Unternehmenszusammenarbeit.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18024-3">
    <skos:prefLabel xml:lang="en">Renewable energy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18024-3">
    <skos:prefLabel xml:lang="de">Regenerative Energie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13386-1">
    <skos:prefLabel xml:lang="en">News agency</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13386-1">
    <skos:prefLabel xml:lang="de">Presseagentur</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12407-3">
    <skos:related rdf:resource="descriptor/12424-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12570-6">
    <skos:altLabel xml:lang="de">Rekrutierung von Führungsnachwuchs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12570-6">
    <skos:altLabel xml:lang="de">Rekrutierung von Führungskräften</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12570-6">
    <skos:altLabel xml:lang="de">Nachwuchskräfterekrutierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12570-6">
    <skos:altLabel xml:lang="de">Führungskräfterekrutierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12923-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19748-5">
    <skos:related rdf:resource="descriptor/16333-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19748-5">
    <skos:related rdf:resource="descriptor/19682-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19748-5">
    <skos:related rdf:resource="descriptor/16298-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19748-5">
    <skos:related rdf:resource="descriptor/19455-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19748-5">
    <skos:related rdf:resource="descriptor/16250-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19748-5">
    <skos:related rdf:resource="descriptor/16658-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17696-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12631-5">
    <skos:prefLabel xml:lang="en">Job evaluation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12631-5">
    <skos:prefLabel xml:lang="de">Arbeitsbewertung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11648-0">
    <skos:prefLabel xml:lang="en">Direct tax</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11648-0">
    <skos:prefLabel xml:lang="de">Direkte Steuer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13624-4">
    <skos:altLabel xml:lang="en">Lighter aboard ship</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13624-4">
    <skos:altLabel xml:lang="en">LASH (Lighter aboard Ship)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13624-4">
    <skos:altLabel xml:lang="en">Barge-carrier system</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10752-3">
    <skos:prefLabel xml:lang="en">Schilling</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10752-3">
    <skos:prefLabel xml:lang="de">Österreichischer Schilling</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14440-2">
    <skos:broader rdf:resource="thsys/70258"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14440-2">
    <skos:broader rdf:resource="descriptor/14433-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70990">
    <rdfs:label xml:lang="de">Papier- und Verlagsgewerbe</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18567-0">
    <skos:prefLabel xml:lang="en">International traffic legislation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18567-0">
    <skos:prefLabel xml:lang="de">Internationales Verkehrsrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12256-5">
    <skos:altLabel xml:lang="en">Equity-linked issue</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12256-5">
    <skos:altLabel xml:lang="de">Optionsschuldverschreibung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12256-5">
    <skos:altLabel xml:lang="de">Optionsschein</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12256-5">
    <skos:altLabel xml:lang="de">Optionsobligation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12256-5">
    <skos:altLabel xml:lang="de">Going-Public-Optionsanleihe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13639-5">
    <skos:related rdf:resource="descriptor/11833-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13639-5">
    <skos:related rdf:resource="descriptor/12927-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13639-5">
    <skos:related rdf:resource="descriptor/18639-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13639-5">
    <skos:related rdf:resource="descriptor/13644-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13639-5">
    <skos:related rdf:resource="descriptor/18637-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13639-5">
    <skos:related rdf:resource="descriptor/16552-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13639-5">
    <skos:related rdf:resource="descriptor/16551-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13639-5">
    <skos:related rdf:resource="descriptor/16549-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13639-5">
    <skos:related rdf:resource="descriptor/11850-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18719-3">
    <skos:related rdf:resource="descriptor/13753-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18719-3">
    <skos:related rdf:resource="descriptor/18722-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18719-3">
    <skos:related rdf:resource="descriptor/18721-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18719-3">
    <skos:related rdf:resource="descriptor/18141-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17941-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19190-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19404-2">
    <skos:broader rdf:resource="thsys/71075"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19404-2">
    <skos:broader rdf:resource="descriptor/11704-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19404-2">
    <skos:broader rdf:resource="descriptor/18160-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18921-1">
    <skos:altLabel xml:lang="de">Kunstökonomie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18921-1">
    <skos:altLabel xml:lang="de">Kulturökonomie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19355-3">
    <skos:broader rdf:resource="thsys/71091"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19355-3">
    <skos:broader rdf:resource="descriptor/11824-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12644-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19847-3">
    <skos:related rdf:resource="descriptor/19824-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19847-3">
    <skos:related rdf:resource="descriptor/16332-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19847-3">
    <skos:related rdf:resource="descriptor/19855-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19847-3">
    <skos:related rdf:resource="descriptor/15961-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19847-3">
    <skos:related rdf:resource="descriptor/15604-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19847-3">
    <skos:related rdf:resource="descriptor/19818-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19847-3">
    <skos:related rdf:resource="descriptor/19845-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19779-1">
    <skos:related rdf:resource="descriptor/11382-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19779-1">
    <skos:related rdf:resource="descriptor/15676-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19779-1">
    <skos:related rdf:resource="descriptor/19776-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19779-1">
    <skos:related rdf:resource="descriptor/19151-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19779-1">
    <skos:related rdf:resource="descriptor/15798-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19779-1">
    <skos:related rdf:resource="descriptor/18868-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19351-4">
    <skos:broader rdf:resource="thsys/71093"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19351-4">
    <skos:broader rdf:resource="descriptor/10626-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15929-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70900">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11039-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14473-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10031-4">
    <skos:related rdf:resource="descriptor/10342-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10031-4">
    <skos:related rdf:resource="descriptor/15075-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10031-4">
    <skos:related rdf:resource="descriptor/10064-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10031-4">
    <skos:related rdf:resource="descriptor/10025-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10031-4">
    <skos:related rdf:resource="descriptor/15072-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10031-4">
    <skos:related rdf:resource="descriptor/10253-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10031-4">
    <skos:related rdf:resource="descriptor/15912-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19828-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13510-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10058-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10359-5">
    <skos:related rdf:resource="descriptor/10350-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10359-5">
    <skos:related rdf:resource="descriptor/10347-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16440-6">
    <skos:scopeNote xml:lang="de">Jede Art von Verhandlungen, national und international.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11824-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13063-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18140-1">
    <skos:broader rdf:resource="thsys/71060"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18140-1">
    <skos:broader rdf:resource="thsys/70244"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18140-1">
    <skos:broader rdf:resource="thsys/70051"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12758-2">
    <skos:narrower rdf:resource="descriptor/18023-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12758-2">
    <skos:narrower rdf:resource="descriptor/12754-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12758-2">
    <skos:narrower rdf:resource="descriptor/12760-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12758-2">
    <skos:narrower rdf:resource="descriptor/12747-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70952">
    <skos:narrower rdf:resource="descriptor/13509-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70952">
    <skos:narrower rdf:resource="descriptor/11125-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70952">
    <skos:narrower rdf:resource="descriptor/10882-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70952">
    <skos:narrower rdf:resource="descriptor/20002-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70952">
    <skos:narrower rdf:resource="descriptor/10872-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70952">
    <skos:narrower rdf:resource="descriptor/18538-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70952">
    <skos:narrower rdf:resource="descriptor/10864-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70952">
    <skos:narrower rdf:resource="descriptor/18536-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70952">
    <skos:narrower rdf:resource="descriptor/10604-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70952">
    <skos:narrower rdf:resource="descriptor/18544-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70952">
    <skos:narrower rdf:resource="descriptor/10606-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70952">
    <skos:narrower rdf:resource="descriptor/10852-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70952">
    <skos:narrower rdf:resource="descriptor/10856-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70952">
    <skos:narrower rdf:resource="descriptor/10835-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70952">
    <skos:narrower rdf:resource="descriptor/15078-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70952">
    <skos:narrower rdf:resource="descriptor/10834-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70952">
    <skos:narrower rdf:resource="descriptor/10686-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70952">
    <skos:narrower rdf:resource="descriptor/18540-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70952">
    <skos:narrower rdf:resource="descriptor/10833-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70952">
    <skos:narrower rdf:resource="thsys/70951"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15001-2">
    <skos:prefLabel xml:lang="en">Nuclear weapon</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15001-2">
    <skos:prefLabel xml:lang="de">Atomwaffe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10459-1">
    <skos:related rdf:resource="descriptor/10466-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10459-1">
    <skos:related rdf:resource="descriptor/18117-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10459-1">
    <skos:related rdf:resource="descriptor/12808-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10459-1">
    <skos:related rdf:resource="descriptor/12807-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10459-1">
    <skos:related rdf:resource="descriptor/19244-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10459-1">
    <skos:related rdf:resource="descriptor/10928-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10459-1">
    <skos:related rdf:resource="descriptor/18070-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10459-1">
    <skos:related rdf:resource="descriptor/12702-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10459-1">
    <skos:related rdf:resource="descriptor/10945-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10459-1">
    <skos:related rdf:resource="descriptor/19251-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10459-1">
    <skos:related rdf:resource="descriptor/19548-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10459-1">
    <skos:related rdf:resource="descriptor/18082-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10459-1">
    <skos:related rdf:resource="descriptor/10947-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10459-1">
    <skos:related rdf:resource="descriptor/10464-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10459-1">
    <skos:related rdf:resource="descriptor/20468-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10459-1">
    <skos:related rdf:resource="descriptor/10475-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10459-1">
    <skos:related rdf:resource="descriptor/19146-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10459-1">
    <skos:related rdf:resource="descriptor/10443-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15067-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18106-1">
    <skos:broader rdf:resource="thsys/70485"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14789-2">
    <skos:related rdf:resource="descriptor/13106-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11307-5">
    <skos:related rdf:resource="descriptor/11347-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11307-5">
    <skos:related rdf:resource="descriptor/19844-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11307-5">
    <skos:related rdf:resource="descriptor/11185-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11307-5">
    <skos:related rdf:resource="descriptor/11339-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19559-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15730-2">
    <skos:broader rdf:resource="thsys/70817"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15730-2">
    <skos:broader rdf:resource="thsys/70239"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15730-2">
    <skos:broader rdf:resource="descriptor/18379-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15730-2">
    <skos:broader rdf:resource="descriptor/18957-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12135-3">
    <skos:altLabel xml:lang="de">Unternehmensabteilung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12135-3">
    <skos:altLabel xml:lang="de">Stabsabteilung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10042-6">
    <skos:prefLabel xml:lang="en">Small country model</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10042-6">
    <skos:prefLabel xml:lang="de">Kleines-Land-Modell</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10374-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id69">
    <rdf:value xml:lang="de">Für räumliche Dezentralisierung</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10772-4">
    <skos:broader rdf:resource="thsys/73387"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10772-4">
    <skos:broader rdf:resource="thsys/71095"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10772-4">
    <skos:broader rdf:resource="thsys/70305"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10772-4">
    <skos:broader rdf:resource="thsys/70109"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18591-3">
    <skos:narrower rdf:resource="descriptor/18592-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18591-3">
    <skos:narrower rdf:resource="descriptor/18602-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19741-5">
    <skos:prefLabel xml:lang="en">Field research</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19741-5">
    <skos:prefLabel xml:lang="de">Feldforschung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10103-5">
    <skos:altLabel xml:lang="en">Second-best theory</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10103-5">
    <skos:altLabel xml:lang="de">Theorie des Zweitbesten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13304-1">
    <skos:broader rdf:resource="thsys/70884"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13304-1">
    <skos:broader rdf:resource="descriptor/18747-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70897">
    <skos:narrower rdf:resource="descriptor/18679-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70897">
    <skos:narrower rdf:resource="descriptor/12212-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70897">
    <skos:narrower rdf:resource="descriptor/18683-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70897">
    <skos:narrower rdf:resource="descriptor/10210-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70897">
    <skos:narrower rdf:resource="descriptor/18733-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70897">
    <skos:narrower rdf:resource="descriptor/12210-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70897">
    <skos:narrower rdf:resource="descriptor/12204-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70897">
    <skos:narrower rdf:resource="descriptor/19271-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70897">
    <skos:narrower rdf:resource="descriptor/19196-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70897">
    <skos:narrower rdf:resource="descriptor/19537-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70897">
    <skos:narrower rdf:resource="descriptor/19269-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70897">
    <skos:narrower rdf:resource="descriptor/18138-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19479-6">
    <skos:prefLabel xml:lang="en">Green area</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19479-6">
    <skos:prefLabel xml:lang="de">Grünfläche</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19108-6">
    <skos:narrower rdf:resource="descriptor/12489-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19108-6">
    <skos:narrower rdf:resource="descriptor/19097-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19108-6">
    <skos:narrower rdf:resource="descriptor/12470-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19108-6">
    <skos:narrower rdf:resource="descriptor/12216-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19108-6">
    <skos:narrower rdf:resource="descriptor/12469-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19538-2">
    <skos:broader rdf:resource="thsys/70893"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19538-2">
    <skos:broader rdf:resource="thsys/70187"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19538-2">
    <skos:broader rdf:resource="descriptor/13715-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14923-3">
    <skos:prefLabel xml:lang="en">Underwear</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14923-3">
    <skos:prefLabel xml:lang="de">Unterwäsche</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10410-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12038-1">
    <skos:altLabel xml:lang="de">Unternehmensgröße</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12038-1">
    <skos:altLabel xml:lang="de">Firmengröße</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10470-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17583-0">
    <skos:prefLabel xml:lang="en">Laos</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17583-0">
    <skos:prefLabel xml:lang="de">Laos</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19459-5">
    <skos:prefLabel xml:lang="en">Village economy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19459-5">
    <skos:prefLabel xml:lang="de">Dorfwirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20881-0">
    <skos:broader rdf:resource="thsys/70578"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20881-0">
    <skos:broader rdf:resource="thsys/70481"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20881-0">
    <skos:broader rdf:resource="thsys/70270"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20881-0">
    <skos:broader rdf:resource="descriptor/10011-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70819">
    <skos:prefLabel xml:lang="en">W.31  Sports</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70819">
    <skos:prefLabel xml:lang="de">W.31  Sport</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14177-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10854-2">
    <skos:altLabel xml:lang="de">Remboursgeschäft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10854-2">
    <skos:altLabel xml:lang="de">Importfinanzierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10854-2">
    <skos:altLabel xml:lang="de">Exportfinanzierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10854-2">
    <skos:altLabel xml:lang="de">Einfuhrfinanzierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10854-2">
    <skos:altLabel xml:lang="de">Ausfuhrfinanzierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10854-2">
    <skos:altLabel xml:lang="de">Außenwirtschaftsfinanzierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10828-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18511-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15734-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10062-0">
    <skos:related rdf:resource="descriptor/10987-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10062-0">
    <skos:related rdf:resource="descriptor/10025-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10062-0">
    <skos:related rdf:resource="descriptor/19207-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10062-0">
    <skos:related rdf:resource="descriptor/16745-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14462-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14124-5">
    <skos:prefLabel xml:lang="en">Milk</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14124-5">
    <skos:prefLabel xml:lang="de">Milch</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10253-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18527-5">
    <skos:scopeNote xml:lang="de">Die Betriebsformen des Handels unterscheiden sich durch wesensbestimmende Merkmale oder eigene Marketingkonzeptionen; sie sind unterschiedlich für Groß- und Einzelhandel.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19443-6">
    <skos:broader rdf:resource="thsys/71070"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19443-6">
    <skos:broader rdf:resource="descriptor/11919-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70986">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18143-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10436-6">
    <skos:narrower rdf:resource="descriptor/16214-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10436-6">
    <skos:narrower rdf:resource="descriptor/15686-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10436-6">
    <skos:narrower rdf:resource="descriptor/15765-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10436-6">
    <skos:narrower rdf:resource="descriptor/18651-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10436-6">
    <skos:narrower rdf:resource="descriptor/11986-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10436-6">
    <skos:narrower rdf:resource="descriptor/16554-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10436-6">
    <skos:narrower rdf:resource="descriptor/18814-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10436-6">
    <skos:narrower rdf:resource="descriptor/11951-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10436-6">
    <skos:narrower rdf:resource="descriptor/12106-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10436-6">
    <skos:narrower rdf:resource="descriptor/10458-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10436-6">
    <skos:narrower rdf:resource="descriptor/19782-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10436-6">
    <skos:narrower rdf:resource="descriptor/18418-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10436-6">
    <skos:narrower rdf:resource="descriptor/18442-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10436-6">
    <skos:narrower rdf:resource="descriptor/16215-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10436-6">
    <skos:narrower rdf:resource="descriptor/18423-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10436-6">
    <skos:narrower rdf:resource="descriptor/19744-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10436-6">
    <skos:narrower rdf:resource="descriptor/18782-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10436-6">
    <skos:narrower rdf:resource="descriptor/10450-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10436-6">
    <skos:narrower rdf:resource="descriptor/16204-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10436-6">
    <skos:narrower rdf:resource="descriptor/18348-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10436-6">
    <skos:narrower rdf:resource="descriptor/10443-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10436-6">
    <skos:narrower rdf:resource="descriptor/18384-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10436-6">
    <skos:narrower rdf:resource="descriptor/18491-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10436-6">
    <skos:narrower rdf:resource="descriptor/16697-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18116-5">
    <skos:related rdf:resource="descriptor/16203-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18116-5">
    <skos:related rdf:resource="descriptor/18130-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18116-5">
    <skos:related rdf:resource="descriptor/12090-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18116-5">
    <skos:related rdf:resource="descriptor/12108-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18116-5">
    <skos:related rdf:resource="descriptor/12085-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18116-5">
    <skos:related rdf:resource="descriptor/16193-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18116-5">
    <skos:related rdf:resource="descriptor/16191-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18116-5">
    <skos:related rdf:resource="descriptor/25822-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10076-3">
    <skos:related rdf:resource="descriptor/10113-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10076-3">
    <skos:related rdf:resource="descriptor/10087-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10076-3">
    <skos:related rdf:resource="descriptor/10252-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10076-3">
    <skos:related rdf:resource="descriptor/10201-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10076-3">
    <skos:related rdf:resource="descriptor/10072-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10076-3">
    <skos:related rdf:resource="descriptor/10138-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10076-3">
    <skos:related rdf:resource="descriptor/10040-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12713-3">
    <skos:related rdf:resource="descriptor/10908-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12912-4">
    <skos:editorialNote xml:lang="de">Möglichst spezifischer indexieren.</skos:editorialNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18174-5">
    <skos:prefLabel xml:lang="en">Energy forecast</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18174-5">
    <skos:prefLabel xml:lang="de">Energieprognose</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13962-5">
    <skos:narrower rdf:resource="descriptor/13960-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13962-5">
    <skos:narrower rdf:resource="descriptor/13959-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13962-5">
    <skos:narrower rdf:resource="descriptor/13957-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10375-0">
    <skos:related rdf:resource="descriptor/10376-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10375-0">
    <skos:related rdf:resource="descriptor/10373-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10375-0">
    <skos:related rdf:resource="descriptor/19238-0"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id50">
    <rdf:value xml:lang="de">Für Forschung in der Industrie</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10034-5">
    <skos:altLabel xml:lang="de">Autoregressive Erwartungsbildung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10034-5">
    <skos:altLabel xml:lang="de">Adaptive Erwartungsbildung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70373">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">P.10.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13437-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13811-5">
    <skos:prefLabel xml:lang="en">Private insurance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13811-5">
    <skos:prefLabel xml:lang="de">Privatversicherung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15931-6">
    <skos:related rdf:resource="descriptor/16513-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15931-6">
    <skos:related rdf:resource="descriptor/15928-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15931-6">
    <skos:related rdf:resource="descriptor/16512-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15931-6">
    <skos:related rdf:resource="descriptor/19642-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15931-6">
    <skos:related rdf:resource="descriptor/16556-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15931-6">
    <skos:related rdf:resource="descriptor/15918-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18822-3">
    <skos:scopeNote xml:lang="de">Neuere Formen der Kooperation zwischen staatlichem Sektor und der Privatwirtschaft.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70923">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10984-3">
    <skos:broader rdf:resource="thsys/73378"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10984-3">
    <skos:broader rdf:resource="thsys/70853"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10984-3">
    <skos:broader rdf:resource="thsys/70274"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10984-3">
    <skos:broader rdf:resource="descriptor/12047-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12419-3">
    <skos:broader rdf:resource="thsys/73337"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12419-3">
    <skos:broader rdf:resource="thsys/70184"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12419-3">
    <skos:broader rdf:resource="thsys/70004"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12419-3">
    <skos:broader rdf:resource="descriptor/19673-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10010-5">
    <skos:related rdf:resource="descriptor/11680-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10010-5">
    <skos:related rdf:resource="descriptor/10298-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10010-5">
    <skos:related rdf:resource="descriptor/10301-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71059">
    <skos:narrower rdf:resource="descriptor/11260-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71059">
    <skos:narrower rdf:resource="descriptor/18166-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71059">
    <skos:narrower rdf:resource="descriptor/11215-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71059">
    <skos:narrower rdf:resource="descriptor/11281-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71059">
    <skos:narrower rdf:resource="descriptor/11212-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71059">
    <skos:narrower rdf:resource="descriptor/12628-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71059">
    <skos:narrower rdf:resource="descriptor/19504-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71059">
    <skos:narrower rdf:resource="descriptor/11199-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71059">
    <skos:narrower rdf:resource="descriptor/11194-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71059">
    <skos:narrower rdf:resource="descriptor/11192-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71059">
    <skos:narrower rdf:resource="descriptor/18911-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71059">
    <skos:narrower rdf:resource="descriptor/11248-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71059">
    <skos:narrower rdf:resource="descriptor/11187-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10865-4">
    <skos:prefLabel xml:lang="en">Export credit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10865-4">
    <skos:prefLabel xml:lang="de">Exportkredit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10000-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15336-6">
    <skos:prefLabel xml:lang="en">Multiple regression</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15336-6">
    <skos:prefLabel xml:lang="de">Multiple Regression</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18211-4">
    <skos:narrower rdf:resource="descriptor/13670-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18211-4">
    <skos:narrower rdf:resource="descriptor/13669-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18211-4">
    <skos:narrower rdf:resource="descriptor/18206-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17721-0">
    <skos:altLabel xml:lang="de">Republik Senegal</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10440-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13164-5">
    <skos:broader rdf:resource="thsys/73294"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13164-5">
    <skos:broader rdf:resource="descriptor/13160-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19428-2">
    <skos:broader rdf:resource="thsys/70101"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19428-2">
    <skos:broader rdf:resource="descriptor/12036-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18695-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17461-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15766-2">
    <skos:altLabel xml:lang="de">Umweltstrafrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15766-2">
    <skos:altLabel xml:lang="de">Umweltdelikt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17928-6">
    <skos:broader rdf:resource="thsys/71030"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17928-6">
    <skos:broader rdf:resource="descriptor/17931-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17928-6">
    <skos:broader rdf:resource="descriptor/19170-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13799-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18731-6">
    <skos:broader rdf:resource="thsys/70893"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18731-6">
    <skos:broader rdf:resource="descriptor/15598-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18731-6">
    <skos:broader rdf:resource="descriptor/19544-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18731-6">
    <skos:broader rdf:resource="descriptor/13279-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10450-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15705-1">
    <skos:altLabel xml:lang="en">Quantitative economic history</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15705-1">
    <skos:altLabel xml:lang="en">New economic history</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15705-1">
    <skos:altLabel xml:lang="de">Quantitative Wirtschaftsgeschichte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15705-1">
    <skos:altLabel xml:lang="de">Neue Wirtschaftsgeschichte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15705-1">
    <skos:altLabel xml:lang="de">Kliometrische Schule</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12189-1">
    <skos:related rdf:resource="descriptor/11628-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12189-1">
    <skos:related rdf:resource="descriptor/16007-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18400-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11337-3">
    <skos:prefLabel xml:lang="en">Wage bargaining model</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11337-3">
    <skos:prefLabel xml:lang="de">Lohnverhandlungstheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10217-4">
    <skos:prefLabel xml:lang="en">Price theory</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10217-4">
    <skos:prefLabel xml:lang="de">Preistheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12014-1">
    <skos:broader rdf:resource="thsys/71066"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12014-1">
    <skos:broader rdf:resource="thsys/70967"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12014-1">
    <skos:broader rdf:resource="thsys/70888"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12014-1">
    <skos:broader rdf:resource="descriptor/12015-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18028-2">
    <skos:scopeNote xml:lang="de">Publikumszeitschrift zu einem speziellen Sachgebiet, z.B. Ökologie, Sport, Reisen, etc.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13507-1">
    <skos:broader rdf:resource="thsys/70931"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13507-1">
    <skos:broader rdf:resource="descriptor/13510-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71064">
    <skos:broader rdf:resource="thsys/70058"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19546-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13525-6">
    <skos:related rdf:resource="descriptor/16166-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13525-6">
    <skos:related rdf:resource="descriptor/18575-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70148">
    <skos:prefLabel xml:lang="en">G.02  Asia</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70148">
    <skos:prefLabel xml:lang="de">G.02  Asien</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18489-1">
    <skos:altLabel xml:lang="de">Industrierevier</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18489-1">
    <skos:altLabel xml:lang="de">Industrieregion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18489-1">
    <skos:altLabel xml:lang="de">Industrieraum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70199">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13033-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16641-3">
    <skos:altLabel xml:lang="de">Soziale Sanktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19286-3">
    <skos:related rdf:resource="descriptor/10511-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19286-3">
    <skos:related rdf:resource="descriptor/19310-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19286-3">
    <skos:related rdf:resource="descriptor/13428-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19286-3">
    <skos:related rdf:resource="descriptor/10515-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14461-1">
    <skos:altLabel xml:lang="de">Kabellegeschiff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14461-1">
    <skos:altLabel xml:lang="de">Forschungsschiff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14461-1">
    <skos:altLabel xml:lang="de">Eisbrecher</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14202-4">
    <skos:related rdf:resource="descriptor/14200-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14202-4">
    <skos:related rdf:resource="descriptor/13032-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12280-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10297-1">
    <skos:prefLabel xml:lang="en">Inventory investment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10297-1">
    <skos:prefLabel xml:lang="de">Lagerinvestition</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70934">
    <skos:narrower rdf:resource="descriptor/18598-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70934">
    <skos:narrower rdf:resource="descriptor/16187-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70934">
    <skos:narrower rdf:resource="descriptor/16070-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70934">
    <skos:narrower rdf:resource="descriptor/18596-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70934">
    <skos:narrower rdf:resource="descriptor/18597-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70934">
    <skos:narrower rdf:resource="descriptor/13627-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70934">
    <skos:narrower rdf:resource="descriptor/16169-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70934">
    <skos:narrower rdf:resource="descriptor/13624-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70934">
    <skos:narrower rdf:resource="descriptor/13543-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70934">
    <skos:narrower rdf:resource="descriptor/13553-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70934">
    <skos:narrower rdf:resource="descriptor/12872-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70934">
    <skos:narrower rdf:resource="descriptor/18595-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70934">
    <skos:narrower rdf:resource="descriptor/18586-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70934">
    <skos:narrower rdf:resource="descriptor/18241-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11498-0">
    <skos:related rdf:resource="descriptor/11473-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11498-0">
    <skos:related rdf:resource="descriptor/11495-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13385-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70004">
    <skos:narrower rdf:resource="descriptor/19673-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70004">
    <skos:narrower rdf:resource="descriptor/12438-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70004">
    <skos:narrower rdf:resource="descriptor/12436-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70004">
    <skos:narrower rdf:resource="descriptor/19102-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70004">
    <skos:narrower rdf:resource="descriptor/12423-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70004">
    <skos:narrower rdf:resource="descriptor/12419-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70004">
    <skos:narrower rdf:resource="descriptor/12416-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13871-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70964">
    <skos:prefLabel xml:lang="de">W.08.01  Handwerkszweige</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16355-4">
    <skos:related rdf:resource="descriptor/19699-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16355-4">
    <skos:related rdf:resource="descriptor/16354-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14738-5">
    <skos:altLabel xml:lang="de">Toilettenseife</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14738-5">
    <skos:altLabel xml:lang="de">Syndet</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14738-5">
    <skos:altLabel xml:lang="de">Haushaltsseife</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14738-5">
    <skos:altLabel xml:lang="de">Feinseife</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13209-2">
    <skos:narrower rdf:resource="descriptor/19061-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13209-2">
    <skos:narrower rdf:resource="descriptor/19062-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13209-2">
    <skos:narrower rdf:resource="descriptor/19060-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13209-2">
    <skos:narrower rdf:resource="descriptor/13496-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18226-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/b">
    <rdfs:label xml:lang="en">Business economics</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/b">
    <rdfs:label xml:lang="de">Betriebswirtschaft</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10042-6">
    <skos:altLabel xml:lang="en">Small economy</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10042-6">
    <skos:altLabel xml:lang="de">Kleines Land Modell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10042-6">
    <skos:altLabel xml:lang="de">Kleines Land</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15779-0">
    <skos:related rdf:resource="descriptor/15897-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15779-0">
    <skos:related rdf:resource="descriptor/19670-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15779-0">
    <skos:related rdf:resource="descriptor/15755-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15779-0">
    <skos:related rdf:resource="descriptor/15743-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15779-0">
    <skos:related rdf:resource="descriptor/12016-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15779-0">
    <skos:related rdf:resource="descriptor/18337-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11696-3">
    <skos:prefLabel xml:lang="en">Effects of taxation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11696-3">
    <skos:prefLabel xml:lang="de">Steuerwirkung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14279-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14359-6">
    <skos:prefLabel xml:lang="en">Ship propulsion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14359-6">
    <skos:prefLabel xml:lang="de">Schiffsantrieb</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15774-3">
    <skos:related rdf:resource="descriptor/12928-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15774-3">
    <skos:related rdf:resource="descriptor/15765-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15774-3">
    <skos:related rdf:resource="descriptor/18966-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15774-3">
    <skos:related rdf:resource="descriptor/15727-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14022-6">
    <skos:broader rdf:resource="thsys/70201"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14022-6">
    <skos:broader rdf:resource="descriptor/18277-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14413-5">
    <skos:narrower rdf:resource="descriptor/14422-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14413-5">
    <skos:narrower rdf:resource="descriptor/14450-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14197-6">
    <skos:prefLabel xml:lang="en">Petroleum</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14197-6">
    <skos:prefLabel xml:lang="de">Erdöl</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14093-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73356">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70961">
    <skos:narrower rdf:resource="descriptor/18515-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70961">
    <skos:narrower rdf:resource="descriptor/13239-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70961">
    <skos:narrower rdf:resource="descriptor/19059-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70961">
    <skos:narrower rdf:resource="descriptor/13221-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70961">
    <skos:narrower rdf:resource="descriptor/19549-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70961">
    <skos:narrower rdf:resource="descriptor/13052-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70961">
    <skos:narrower rdf:resource="descriptor/13217-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70961">
    <skos:narrower rdf:resource="descriptor/18514-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12330-5">
    <skos:broader rdf:resource="thsys/71043"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12330-5">
    <skos:broader rdf:resource="thsys/71036"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12330-5">
    <skos:broader rdf:resource="descriptor/12325-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12330-5">
    <skos:broader rdf:resource="descriptor/12165-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17608-3">
    <skos:related rdf:resource="descriptor/18054-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17608-3">
    <skos:related rdf:resource="descriptor/17978-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15713-2">
    <skos:narrower rdf:resource="descriptor/19756-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15713-2">
    <skos:narrower rdf:resource="descriptor/15932-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15713-2">
    <skos:narrower rdf:resource="descriptor/11189-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14030-0">
    <skos:related rdf:resource="descriptor/14073-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14030-0">
    <skos:related rdf:resource="descriptor/14041-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14030-0">
    <skos:related rdf:resource="descriptor/14104-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14030-0">
    <skos:related rdf:resource="descriptor/12940-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14030-0">
    <skos:related rdf:resource="descriptor/14029-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14030-0">
    <skos:related rdf:resource="descriptor/14596-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15518-0">
    <skos:related rdf:resource="descriptor/13588-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15518-0">
    <skos:related rdf:resource="descriptor/13513-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15518-0">
    <skos:related rdf:resource="descriptor/15520-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15518-0">
    <skos:related rdf:resource="descriptor/18620-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15518-0">
    <skos:related rdf:resource="descriptor/13610-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15518-0">
    <skos:related rdf:resource="descriptor/13600-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15518-0">
    <skos:related rdf:resource="descriptor/11864-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12365-0">
    <skos:altLabel xml:lang="de">Gründungsbilanz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11335-0">
    <skos:altLabel xml:lang="en">Wage Stickiness</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11335-0">
    <skos:altLabel xml:lang="en">Wage inflexibility</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11335-0">
    <skos:altLabel xml:lang="en">Sticky wages</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11335-0">
    <skos:altLabel xml:lang="en">Inflexible wages</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17565-2">
    <skos:related rdf:resource="descriptor/17595-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17565-2">
    <skos:related rdf:resource="descriptor/17567-5"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id55">
    <rdfs:seeAlso rdf:resource="descriptor/19041-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10908-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71088">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14270-1">
    <skos:broader rdf:resource="thsys/70346"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14270-1">
    <skos:broader rdf:resource="descriptor/19051-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71044">
    <skos:broader rdf:resource="thsys/70036"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16272-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16914-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11664-2">
    <skos:altLabel xml:lang="de">Gesellschaftsteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13131-6">
    <skos:broader rdf:resource="thsys/70987"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13131-6">
    <skos:broader rdf:resource="descriptor/13145-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16496-0">
    <skos:scopeNote xml:lang="de">Auf Europa, die Europäische Union oder die Europäische Integration bezogene Politik einzelner Staaten innerhalb oder außerhalb der Europäischen Union.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17578-0">
    <skos:broader rdf:resource="thsys/70327"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17578-0">
    <skos:broader rdf:resource="descriptor/17589-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17578-0">
    <skos:broader rdf:resource="descriptor/17979-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14392-1">
    <skos:related rdf:resource="descriptor/18435-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14392-1">
    <skos:related rdf:resource="descriptor/18095-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14392-1">
    <skos:related rdf:resource="descriptor/14846-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14392-1">
    <skos:related rdf:resource="descriptor/18094-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14392-1">
    <skos:related rdf:resource="descriptor/14383-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14005-6">
    <skos:altLabel xml:lang="de">Pharmazeutinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14005-6">
    <skos:altLabel xml:lang="de">Pharmazeuten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14005-6">
    <skos:altLabel xml:lang="de">Apothekerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14698-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16215-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13826-6">
    <skos:related rdf:resource="descriptor/13823-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13826-6">
    <skos:related rdf:resource="descriptor/21072-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13826-6">
    <skos:related rdf:resource="descriptor/13827-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13826-6">
    <skos:related rdf:resource="descriptor/13818-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13826-6">
    <skos:related rdf:resource="descriptor/19274-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13826-6">
    <skos:related rdf:resource="descriptor/10207-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13826-6">
    <skos:related rdf:resource="descriptor/19227-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19542-4">
    <skos:broader rdf:resource="thsys/72394"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19542-4">
    <skos:broader rdf:resource="thsys/71035"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19542-4">
    <skos:broader rdf:resource="thsys/70292"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19542-4">
    <skos:broader rdf:resource="descriptor/15405-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16402-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19202-0">
    <skos:prefLabel xml:lang="en">Time consistency</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19202-0">
    <skos:prefLabel xml:lang="de">Zeitkonsistenz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14933-0">
    <skos:prefLabel xml:lang="en">Frozen food</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14933-0">
    <skos:prefLabel xml:lang="de">Tiefkühlkost</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11886-5">
    <skos:related rdf:resource="descriptor/13424-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11886-5">
    <skos:related rdf:resource="descriptor/16552-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11886-5">
    <skos:related rdf:resource="descriptor/10558-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19336-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13753-0">
    <skos:altLabel xml:lang="de">Zinsmarge</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13753-0">
    <skos:altLabel xml:lang="de">Teilzinsspannenrechnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13753-0">
    <skos:altLabel xml:lang="de">Marktzinsmethode</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71008">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19140-3">
    <skos:prefLabel xml:lang="en">Distribution (Marketing)</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19140-3">
    <skos:prefLabel xml:lang="de">Vertrieb</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18962-1">
    <skos:related rdf:resource="descriptor/18559-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18962-1">
    <skos:related rdf:resource="descriptor/19561-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70868">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.22.01.03</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10988-2">
    <skos:related rdf:resource="descriptor/10987-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10988-2">
    <skos:related rdf:resource="descriptor/18793-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11650-6">
    <skos:related rdf:resource="descriptor/11666-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11650-6">
    <skos:related rdf:resource="descriptor/11665-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11650-6">
    <skos:related rdf:resource="descriptor/19409-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11650-6">
    <skos:related rdf:resource="descriptor/16092-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11650-6">
    <skos:related rdf:resource="descriptor/19413-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12607-2">
    <skos:broader rdf:resource="thsys/73385"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12607-2">
    <skos:broader rdf:resource="thsys/71050"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12607-2">
    <skos:broader rdf:resource="descriptor/11305-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15657-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70879">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12298-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14631-2">
    <skos:prefLabel xml:lang="en">Aerosols</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14631-2">
    <skos:prefLabel xml:lang="de">Aerosol</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19241-4">
    <skos:prefLabel xml:lang="en">Seasonal fluctuation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19241-4">
    <skos:prefLabel xml:lang="de">Saisonschwankung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13054-5">
    <skos:prefLabel xml:lang="en">Agricultural machinery industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13054-5">
    <skos:prefLabel xml:lang="de">Landmaschinenbau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18760-6">
    <skos:broader rdf:resource="thsys/73392"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18760-6">
    <skos:broader rdf:resource="thsys/70883"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18760-6">
    <skos:broader rdf:resource="descriptor/15982-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18760-6">
    <skos:broader rdf:resource="descriptor/11840-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18892-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16396-4">
    <skos:prefLabel xml:lang="en">Political resistance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16396-4">
    <skos:prefLabel xml:lang="de">Politischer Widerstand</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19000-5">
    <skos:altLabel xml:lang="de">WEU-Länder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11339-6">
    <skos:altLabel xml:lang="de">Tarifkonflikt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11339-6">
    <skos:altLabel xml:lang="de">Arbeitskampfrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11339-6">
    <skos:altLabel xml:lang="de">Arbeitskampf</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19097-6">
    <skos:broader rdf:resource="thsys/70578"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19097-6">
    <skos:broader rdf:resource="descriptor/19096-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19097-6">
    <skos:broader rdf:resource="descriptor/19108-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12079-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17793-3">
    <skos:prefLabel xml:lang="en">Panama</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17793-3">
    <skos:prefLabel xml:lang="de">Panama</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70647">
    <rdfs:label xml:lang="de">Keramische Produkte</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19311-2">
    <skos:broader rdf:resource="thsys/71100"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19311-2">
    <skos:broader rdf:resource="descriptor/10629-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19499-0">
    <skos:altLabel xml:lang="de">Frauenlohn</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18860-2">
    <skos:prefLabel xml:lang="en">Education behaviour</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18860-2">
    <skos:prefLabel xml:lang="de">Bildungsverhalten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13853-3">
    <skos:altLabel xml:lang="de">Preis für Bauleistungen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#Concept">
    <rdfs:isDefinedBy rdf:resource="http://www.w3.org/2004/02/skos/core"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12024-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19493-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17571-0">
    <skos:broader rdf:resource="thsys/70147"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17571-0">
    <skos:broader rdf:resource="descriptor/17553-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10113-2">
    <skos:narrower rdf:resource="descriptor/10112-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10113-2">
    <skos:narrower rdf:resource="descriptor/10109-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10113-2">
    <skos:narrower rdf:resource="descriptor/10106-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10113-2">
    <skos:narrower rdf:resource="descriptor/11527-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10113-2">
    <skos:narrower rdf:resource="descriptor/10103-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10113-2">
    <skos:narrower rdf:resource="descriptor/10093-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10113-2">
    <skos:narrower rdf:resource="descriptor/10102-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10113-2">
    <skos:narrower rdf:resource="descriptor/10099-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10113-2">
    <skos:narrower rdf:resource="descriptor/10097-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10113-2">
    <skos:narrower rdf:resource="descriptor/10092-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10113-2">
    <skos:narrower rdf:resource="descriptor/10040-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10113-2">
    <skos:narrower rdf:resource="descriptor/10088-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15659-3">
    <skos:related rdf:resource="descriptor/19860-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15659-3">
    <skos:related rdf:resource="descriptor/14169-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15659-3">
    <skos:related rdf:resource="descriptor/15714-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15659-3">
    <skos:related rdf:resource="descriptor/15928-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15659-3">
    <skos:related rdf:resource="descriptor/15416-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15659-3">
    <skos:related rdf:resource="descriptor/15964-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15659-3">
    <skos:related rdf:resource="descriptor/15927-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15659-3">
    <skos:related rdf:resource="descriptor/11224-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15659-3">
    <skos:related rdf:resource="descriptor/15920-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15659-3">
    <skos:related rdf:resource="descriptor/15909-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15659-3">
    <skos:related rdf:resource="descriptor/15627-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/a">
    <skos:prefLabel xml:lang="en">A  General descriptors</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/a">
    <skos:prefLabel xml:lang="de">A  Allgemeinwörter</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19855-4">
    <skos:prefLabel xml:lang="en">Espionage</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19855-4">
    <skos:prefLabel xml:lang="de">Spionage</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71042">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19313-5">
    <skos:broader rdf:resource="thsys/71099"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19313-5">
    <skos:broader rdf:resource="descriptor/10676-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18765-3">
    <skos:prefLabel xml:lang="en">CASE (Computer-Aided Software Engineering)</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18765-3">
    <skos:prefLabel xml:lang="de">CASE (Computer-Aided Software Engineering)</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16264-0">
    <skos:related rdf:resource="descriptor/16389-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16264-0">
    <skos:related rdf:resource="descriptor/19688-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16264-0">
    <skos:related rdf:resource="descriptor/16309-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16264-0">
    <skos:related rdf:resource="descriptor/16422-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16264-0">
    <skos:related rdf:resource="descriptor/16421-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16264-0">
    <skos:related rdf:resource="descriptor/16311-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16264-0">
    <skos:related rdf:resource="descriptor/16472-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16264-0">
    <skos:related rdf:resource="descriptor/16300-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16264-0">
    <skos:related rdf:resource="descriptor/16416-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16264-0">
    <skos:related rdf:resource="descriptor/18062-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16264-0">
    <skos:related rdf:resource="descriptor/16299-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16264-0">
    <skos:related rdf:resource="descriptor/16512-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16264-0">
    <skos:related rdf:resource="descriptor/16426-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16264-0">
    <skos:related rdf:resource="descriptor/15797-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16264-0">
    <skos:related rdf:resource="descriptor/15918-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16264-0">
    <skos:related rdf:resource="descriptor/16246-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16297-6">
    <skos:related rdf:resource="descriptor/16312-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16297-6">
    <skos:related rdf:resource="descriptor/12005-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16297-6">
    <skos:related rdf:resource="descriptor/16422-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16297-6">
    <skos:related rdf:resource="descriptor/16308-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16297-6">
    <skos:related rdf:resource="descriptor/16421-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16297-6">
    <skos:related rdf:resource="descriptor/11849-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16297-6">
    <skos:related rdf:resource="descriptor/16322-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16297-6">
    <skos:related rdf:resource="descriptor/16314-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20881-0">
    <skos:related rdf:resource="descriptor/19218-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20881-0">
    <skos:related rdf:resource="descriptor/16739-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20881-0">
    <skos:related rdf:resource="descriptor/10198-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15763-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14107-5">
    <skos:broader rdf:resource="thsys/70169"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14107-5">
    <skos:broader rdf:resource="descriptor/14110-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10764-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19752-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12320-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14515-4">
    <skos:related rdf:resource="descriptor/13655-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16556-1">
    <skos:altLabel xml:lang="en">Social anthropology</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16556-1">
    <skos:altLabel xml:lang="en">Cultural anthropology</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16556-1">
    <skos:altLabel xml:lang="de">Völkerkunde</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16556-1">
    <skos:altLabel xml:lang="de">Sozialanthropologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16556-1">
    <skos:altLabel xml:lang="de">Kulturanthropologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16556-1">
    <skos:altLabel xml:lang="de">Ethnographie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16594-0">
    <skos:narrower rdf:resource="descriptor/10466-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16594-0">
    <skos:narrower rdf:resource="descriptor/10459-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13295-4">
    <skos:prefLabel xml:lang="en">Legal expenses insurance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13295-4">
    <skos:prefLabel xml:lang="de">Rechtsschutzversicherung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16599-4">
    <skos:related rdf:resource="descriptor/15654-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16599-4">
    <skos:related rdf:resource="descriptor/16588-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10583-0">
    <skos:altLabel xml:lang="de">Internationale Nahrungsmittelhilfe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16315-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11454-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19484-6">
    <skos:broader rdf:resource="thsys/73351"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19484-6">
    <skos:broader rdf:resource="thsys/71066"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19484-6">
    <skos:broader rdf:resource="thsys/70238"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19484-6">
    <skos:broader rdf:resource="descriptor/12016-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10432-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16709-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19597-0">
    <skos:broader rdf:resource="thsys/73104"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19597-0">
    <skos:broader rdf:resource="descriptor/15460-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12424-3">
    <skos:related rdf:resource="descriptor/12504-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12424-3">
    <skos:related rdf:resource="descriptor/12407-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15330-4">
    <skos:broader rdf:resource="thsys/72394"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15330-4">
    <skos:broader rdf:resource="descriptor/15405-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12649-0">
    <skos:broader rdf:resource="thsys/73343"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12649-0">
    <skos:broader rdf:resource="thsys/70301"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12649-0">
    <skos:broader rdf:resource="thsys/70201"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12649-0">
    <skos:broader rdf:resource="thsys/70030"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16188-4">
    <skos:altLabel xml:lang="de">Völkervertragsrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16188-4">
    <skos:altLabel xml:lang="de">Neutralitätsrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16188-4">
    <skos:altLabel xml:lang="de">Internationales öffentliches Recht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19849-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18548-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19789-5">
    <skos:altLabel xml:lang="de">Zuckerkrankheit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14839-6">
    <skos:related rdf:resource="descriptor/14895-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21306-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71058">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12275-1">
    <skos:broader rdf:resource="thsys/71042"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12275-1">
    <skos:broader rdf:resource="thsys/70036"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12275-1">
    <skos:broader rdf:resource="descriptor/12308-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16373-2">
    <skos:related rdf:resource="descriptor/16374-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16373-2">
    <skos:related rdf:resource="descriptor/16402-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13333-1">
    <skos:prefLabel xml:lang="en">Local waste management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13333-1">
    <skos:prefLabel xml:lang="de">Kommunale Abfallwirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18621-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19618-4">
    <skos:related rdf:resource="descriptor/12987-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19520-0">
    <skos:broader rdf:resource="thsys/73387"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19520-0">
    <skos:broader rdf:resource="thsys/70109"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19520-0">
    <skos:broader rdf:resource="descriptor/12062-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19520-0">
    <skos:broader rdf:resource="descriptor/18534-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14313-2">
    <skos:related rdf:resource="descriptor/14266-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14065-2">
    <skos:related rdf:resource="descriptor/13163-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14065-2">
    <skos:related rdf:resource="descriptor/14996-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14065-2">
    <skos:related rdf:resource="descriptor/14074-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14065-2">
    <skos:related rdf:resource="descriptor/19296-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14065-2">
    <skos:related rdf:resource="descriptor/18193-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14065-2">
    <skos:related rdf:resource="descriptor/18194-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14065-2">
    <skos:related rdf:resource="descriptor/14971-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10505-6">
    <zbwext:useInsteadNote rdf:nodeID="id189"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18424-1">
    <skos:narrower rdf:resource="descriptor/18427-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18424-1">
    <skos:narrower rdf:resource="descriptor/13074-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18424-1">
    <skos:narrower rdf:resource="descriptor/13072-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20553-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18948-2">
    <skos:broader rdf:resource="thsys/71115"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18948-2">
    <skos:broader rdf:resource="thsys/70814"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18948-2">
    <skos:broader rdf:resource="descriptor/10006-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16275-2">
    <skos:related rdf:resource="descriptor/16207-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16275-2">
    <skos:related rdf:resource="descriptor/16430-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16275-2">
    <skos:related rdf:resource="descriptor/16633-2"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id252">
    <rdfs:seeAlso rdf:resource="descriptor/14626-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11562-3">
    <skos:altLabel xml:lang="en">Farm program payments</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11562-3">
    <skos:altLabel xml:lang="de">Landwirtschaftssubvention</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11562-3">
    <skos:altLabel xml:lang="de">Agrarfinanzhilfe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13079-3">
    <skos:broader rdf:resource="thsys/70996"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13079-3">
    <skos:broader rdf:resource="descriptor/13076-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13228-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13102-6">
    <skos:altLabel xml:lang="de">Vervielfältigungsindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13102-6">
    <skos:altLabel xml:lang="de">Graphisches Gewerbe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13102-6">
    <skos:altLabel xml:lang="de">Grafisches Gewerbe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13102-6">
    <skos:altLabel xml:lang="de">Druckgewerbe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13102-6">
    <skos:altLabel xml:lang="de">Druckerei</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10574-1">
    <skos:narrower rdf:resource="descriptor/19288-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15479-5">
    <skos:altLabel xml:lang="en">Risk-neutral</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13689-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14750-1">
    <skos:altLabel xml:lang="de">Textverarbeitungsgerät</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14750-1">
    <skos:altLabel xml:lang="de">Textautomat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14750-1">
    <skos:altLabel xml:lang="de">Taschenrechner</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14750-1">
    <skos:altLabel xml:lang="de">Schreibmaschine</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14750-1">
    <skos:altLabel xml:lang="de">Registrierkasse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14750-1">
    <skos:altLabel xml:lang="de">Diktiergerät</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14750-1">
    <skos:altLabel xml:lang="de">Buchungsmaschine</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14750-1">
    <skos:altLabel xml:lang="de">Abrechnungsmaschine</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13583-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10976-2">
    <skos:prefLabel xml:lang="en">Ordnungspolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10976-2">
    <skos:prefLabel xml:lang="de">Ordnungspolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18295-0">
    <skos:broader rdf:resource="thsys/70987"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18295-0">
    <skos:broader rdf:resource="thsys/70828"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18295-0">
    <skos:broader rdf:resource="thsys/70581"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18295-0">
    <skos:broader rdf:resource="descriptor/15654-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19194-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13217-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18785-4">
    <skos:related rdf:resource="descriptor/12262-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18785-4">
    <skos:related rdf:resource="descriptor/19584-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18785-4">
    <skos:related rdf:resource="descriptor/18784-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14731-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18976-4">
    <skos:prefLabel xml:lang="en">Consumer policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18976-4">
    <skos:prefLabel xml:lang="de">Verbraucherpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20301-2">
    <skos:altLabel xml:lang="en">Real options theory</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13575-5">
    <skos:broader rdf:resource="thsys/70998"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13575-5">
    <skos:broader rdf:resource="thsys/70932"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13575-5">
    <skos:broader rdf:resource="descriptor/10470-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16563-4">
    <skos:prefLabel xml:lang="en">Social science</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16563-4">
    <skos:prefLabel xml:lang="de">Sozialwissenschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13450-4">
    <skos:related rdf:resource="descriptor/18443-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13450-4">
    <skos:related rdf:resource="descriptor/10505-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13450-4">
    <skos:related rdf:resource="descriptor/12945-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13450-4">
    <skos:related rdf:resource="descriptor/13054-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13450-4">
    <skos:related rdf:resource="descriptor/14380-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13450-4">
    <skos:related rdf:resource="descriptor/10497-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13450-4">
    <skos:related rdf:resource="descriptor/18344-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13450-4">
    <skos:related rdf:resource="descriptor/14171-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13450-4">
    <skos:related rdf:resource="descriptor/13446-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13450-4">
    <skos:related rdf:resource="descriptor/18317-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14093-4">
    <skos:altLabel xml:lang="de">Körnermais</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19201-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13562-0">
    <skos:altLabel xml:lang="de">Straßenplanung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18150-5">
    <skos:related rdf:resource="descriptor/10255-3"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id32">
    <rdf:value xml:lang="de">Für staatliche Einrichtungen zur Außenhandelsförderung</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19016-4">
    <skos:altLabel xml:lang="de">Umwelttoxikologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19016-4">
    <skos:altLabel xml:lang="de">Toxizität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19016-4">
    <skos:altLabel xml:lang="de">Giftigkeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12010-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18790-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18748-3">
    <skos:related rdf:resource="descriptor/18122-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18748-3">
    <skos:related rdf:resource="descriptor/11981-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18748-3">
    <skos:related rdf:resource="descriptor/11905-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18936-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11711-5">
    <skos:scopeNote xml:lang="de">Theoretische Analysen zur optimalen Aufgabenverteilung in der Finanzpolitik.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19818-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70538">
    <rdfs:label xml:lang="de">Energie und Wasser</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19709-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13465-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73360">
    <skos:prefLabel xml:lang="en">N.05.01.02  Administrative law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73360">
    <skos:prefLabel xml:lang="de">N.05.01.02  Verwaltungsrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15995-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12427-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70832">
    <skos:prefLabel xml:lang="en">W.26.03  Social insurance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70832">
    <skos:prefLabel xml:lang="de">W.26.03  Sozialversicherung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70940">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.12.03</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16002-2">
    <skos:related rdf:resource="descriptor/16017-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16002-2">
    <skos:related rdf:resource="descriptor/11665-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16002-2">
    <skos:related rdf:resource="descriptor/19706-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16002-2">
    <skos:related rdf:resource="descriptor/13393-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16136-2">
    <skos:narrower rdf:resource="descriptor/20455-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16136-2">
    <skos:narrower rdf:resource="descriptor/18169-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16654-1">
    <skos:related rdf:resource="descriptor/15722-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16654-1">
    <skos:related rdf:resource="descriptor/15718-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16654-1">
    <skos:related rdf:resource="descriptor/15712-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16654-1">
    <skos:related rdf:resource="descriptor/19646-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16654-1">
    <skos:related rdf:resource="descriptor/16656-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16654-1">
    <skos:related rdf:resource="descriptor/16663-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16654-1">
    <skos:related rdf:resource="descriptor/16661-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16654-1">
    <skos:related rdf:resource="descriptor/15713-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16247-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11069-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18658-4">
    <skos:altLabel xml:lang="de">Netzwerkserver</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18658-4">
    <skos:altLabel xml:lang="de">Netzwerk-Server</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12565-6">
    <skos:related rdf:resource="descriptor/12562-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12565-6">
    <skos:related rdf:resource="descriptor/12572-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12565-6">
    <skos:related rdf:resource="descriptor/20381-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70888">
    <rdfs:label xml:lang="en">Housing industry</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70888">
    <rdfs:label xml:lang="de">Wohnungswirtschaft</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10265-0">
    <skos:related rdf:resource="descriptor/10269-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10265-0">
    <skos:related rdf:resource="descriptor/10266-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18667-3">
    <skos:broader rdf:resource="thsys/70899"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12950-3">
    <skos:altLabel xml:lang="de">Rinderzucht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12950-3">
    <skos:altLabel xml:lang="de">Rindermast</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#altLabel">
    <rdfs:isDefinedBy rdf:resource="http://www.w3.org/2004/02/skos/core"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18809-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10778-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18709-6">
    <skos:altLabel xml:lang="de">Bankmanagerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10427-0">
    <skos:broader rdf:resource="thsys/70406"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10427-0">
    <skos:broader rdf:resource="descriptor/10434-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19036-5">
    <skos:broader rdf:resource="thsys/70582"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10597-3">
    <skos:prefLabel xml:lang="en">South south relations</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10597-3">
    <skos:prefLabel xml:lang="de">Süd-Süd-Beziehungen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19759-0">
    <skos:prefLabel xml:lang="en">Parents</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19759-0">
    <skos:prefLabel xml:lang="de">Eltern</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10701-6">
    <skos:altLabel xml:lang="de">Importliberalisierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10701-6">
    <skos:altLabel xml:lang="de">Einfuhrliberalisierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10701-6">
    <skos:altLabel xml:lang="de">Einfuhrförderung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18569-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18728-2">
    <skos:broader rdf:resource="thsys/70893"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18728-2">
    <skos:broader rdf:resource="thsys/70867"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18728-2">
    <skos:broader rdf:resource="descriptor/18810-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73378">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18357-4">
    <skos:broader rdf:resource="thsys/71016"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18357-4">
    <skos:broader rdf:resource="descriptor/18345-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18357-4">
    <skos:broader rdf:resource="descriptor/12975-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19163-5">
    <skos:narrower rdf:resource="descriptor/18029-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19163-5">
    <skos:narrower rdf:resource="descriptor/12158-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19163-5">
    <skos:narrower rdf:resource="descriptor/18184-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19163-5">
    <skos:narrower rdf:resource="descriptor/12083-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19163-5">
    <skos:narrower rdf:resource="descriptor/10524-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10141-4">
    <skos:altLabel xml:lang="en">Informations overload</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10141-4">
    <skos:altLabel xml:lang="en">Information economics</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10141-4">
    <skos:altLabel xml:lang="en">Economic theory of information</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10141-4">
    <skos:altLabel xml:lang="de">Ökonomische Theorie der Information</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10141-4">
    <skos:altLabel xml:lang="de">Ökonomische Informationstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10141-4">
    <skos:altLabel xml:lang="de">Informationsökonomie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10847-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70926">
    <skos:narrower rdf:resource="descriptor/18630-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70926">
    <skos:narrower rdf:resource="descriptor/18629-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70926">
    <skos:narrower rdf:resource="descriptor/11833-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70926">
    <skos:narrower rdf:resource="descriptor/11832-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70926">
    <skos:narrower rdf:resource="descriptor/11828-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70926">
    <skos:narrower rdf:resource="descriptor/11827-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70926">
    <skos:narrower rdf:resource="descriptor/18628-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70926">
    <skos:narrower rdf:resource="descriptor/18579-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70926">
    <skos:narrower rdf:resource="descriptor/18583-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70926">
    <skos:narrower rdf:resource="descriptor/13562-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70926">
    <skos:narrower rdf:resource="descriptor/13545-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70926">
    <skos:narrower rdf:resource="descriptor/18610-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70926">
    <skos:narrower rdf:resource="descriptor/13519-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70926">
    <skos:narrower rdf:resource="descriptor/11667-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70926">
    <skos:narrower rdf:resource="descriptor/18601-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70926">
    <skos:narrower rdf:resource="descriptor/18616-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70926">
    <skos:narrower rdf:resource="descriptor/11826-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70926">
    <skos:narrower rdf:resource="descriptor/18627-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70926">
    <skos:narrower rdf:resource="descriptor/18575-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11322-2">
    <skos:broader rdf:resource="thsys/70278"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11322-2">
    <skos:broader rdf:resource="thsys/70245"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11322-2">
    <skos:broader rdf:resource="descriptor/10186-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11322-2">
    <skos:broader rdf:resource="descriptor/10006-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11322-2">
    <skos:broader rdf:resource="descriptor/19566-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16436-4">
    <skos:related rdf:resource="descriptor/16278-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16436-4">
    <skos:related rdf:resource="descriptor/16435-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16436-4">
    <skos:related rdf:resource="descriptor/16441-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16436-4">
    <skos:related rdf:resource="descriptor/20826-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10598-1">
    <skos:prefLabel xml:lang="en">Technical assistance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10598-1">
    <skos:prefLabel xml:lang="de">Technische Hilfe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18101-4">
    <skos:prefLabel xml:lang="en">Services marketing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18101-4">
    <skos:prefLabel xml:lang="de">Dienstleistungsmarketing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71044">
    <rdfs:label xml:lang="de">Investitionsplanung und -rechnung</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11823-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19036-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19182-1">
    <skos:prefLabel xml:lang="en">Oil market</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19182-1">
    <skos:prefLabel xml:lang="de">Mineralölmarkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18224-2">
    <skos:scopeNote xml:lang="de">Selbstständiger Staat seit dem 1.1.1993.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16438-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12628-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19013-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18293-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12656-3">
    <skos:altLabel xml:lang="de">Materialbestellung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12656-3">
    <skos:altLabel xml:lang="de">Einkauf des Betriebs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12656-3">
    <skos:altLabel xml:lang="de">Betrieblicher Einkauf</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12656-3">
    <skos:altLabel xml:lang="de">Bestellzeitpunkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12656-3">
    <skos:altLabel xml:lang="de">Bestellrhythmus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12656-3">
    <skos:altLabel xml:lang="de">Bestellpunkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12656-3">
    <skos:altLabel xml:lang="de">Bestellpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12656-3">
    <skos:altLabel xml:lang="de">Bestellmenge</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10164-6">
    <skos:prefLabel xml:lang="en">Theory of preferences</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10164-6">
    <skos:prefLabel xml:lang="de">Präferenztheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11765-3">
    <skos:narrower rdf:resource="descriptor/10099-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14923-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11129-3">
    <skos:altLabel xml:lang="de">Produktionsfonds</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11129-3">
    <skos:altLabel xml:lang="de">Grundfonds</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19751-2">
    <skos:altLabel xml:lang="de">Selbstorganisierendes System</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19751-2">
    <skos:altLabel xml:lang="de">Autopoiesis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17255-3">
    <skos:broader rdf:resource="thsys/70557"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17255-3">
    <skos:broader rdf:resource="descriptor/17199-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17255-3">
    <skos:broader rdf:resource="descriptor/17991-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17255-3">
    <skos:broader rdf:resource="descriptor/19178-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17255-3">
    <skos:broader rdf:resource="descriptor/19177-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17255-3">
    <skos:broader rdf:resource="descriptor/17983-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19381-2">
    <skos:scopeNote xml:lang="de">Alle Arten der Aufbringung und Verwendung von finanziellen Ressourcen zur Kriegführung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11314-1">
    <zbwext:useInsteadNote rdf:nodeID="id20"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70994">
    <rdfs:label xml:lang="en">Chemical industry</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70994">
    <rdfs:label xml:lang="de">Chemieindustrie</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18331-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13279-2">
    <skos:related rdf:resource="descriptor/18733-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13279-2">
    <skos:related rdf:resource="descriptor/19270-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13279-2">
    <skos:related rdf:resource="descriptor/18676-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13279-2">
    <skos:related rdf:resource="descriptor/13716-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13279-2">
    <skos:related rdf:resource="descriptor/13715-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13279-2">
    <skos:related rdf:resource="descriptor/18729-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18673-1">
    <skos:altLabel xml:lang="en">Tontine</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18673-1">
    <skos:altLabel xml:lang="en">Rotating savings and credit association</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18673-1">
    <skos:altLabel xml:lang="en">Informal lending</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18673-1">
    <skos:altLabel xml:lang="en">Informal financial system</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18673-1">
    <skos:altLabel xml:lang="en">Informal financial market</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18673-1">
    <skos:altLabel xml:lang="de">Rotierender Spar- und Kreditfonds</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18673-1">
    <skos:altLabel xml:lang="de">Informeller Kapitalmarkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18673-1">
    <skos:altLabel xml:lang="de">Informeller Finanzmarkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18673-1">
    <skos:altLabel xml:lang="de">Dualer Finanzmarkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10481-1">
    <skos:prefLabel xml:lang="en">Patent law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10481-1">
    <skos:prefLabel xml:lang="de">Patentrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19705-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18971-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12759-0">
    <skos:prefLabel xml:lang="en">Product line</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12759-0">
    <skos:prefLabel xml:lang="de">Produktionsprogramm</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16297-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17687-2">
    <skos:broader rdf:resource="thsys/70429"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17687-2">
    <skos:broader rdf:resource="descriptor/17652-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13888-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20497-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18726-6">
    <skos:related rdf:resource="descriptor/12185-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18726-6">
    <skos:related rdf:resource="descriptor/13714-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18726-6">
    <skos:related rdf:resource="descriptor/19692-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10020-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13686-3">
    <skos:prefLabel xml:lang="en">Banking administration</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13686-3">
    <skos:prefLabel xml:lang="de">Bankbetriebslehre</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18892-3">
    <skos:broader rdf:resource="thsys/73347"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18892-3">
    <skos:broader rdf:resource="thsys/70839"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18892-3">
    <skos:broader rdf:resource="thsys/70573"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11988-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24786-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14957-0">
    <skos:prefLabel xml:lang="en">Yellow fat</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14957-0">
    <skos:prefLabel xml:lang="de">Streichfett</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15298-2">
    <skos:related rdf:resource="descriptor/15063-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15298-2">
    <skos:related rdf:resource="descriptor/15272-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15298-2">
    <skos:related rdf:resource="descriptor/15062-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16825-0">
    <skos:altLabel xml:lang="de">Mitteleuropäische Staaten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16825-0">
    <skos:altLabel xml:lang="de">Mitteleuropäische Länder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16512-0">
    <skos:altLabel xml:lang="de">Nationalitätenpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13278-4">
    <skos:narrower rdf:resource="descriptor/13667-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13278-4">
    <skos:narrower rdf:resource="descriptor/13665-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13278-4">
    <skos:narrower rdf:resource="descriptor/18655-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13278-4">
    <skos:narrower rdf:resource="descriptor/13655-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18339-6">
    <skos:prefLabel xml:lang="en">Forestry research</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18339-6">
    <skos:prefLabel xml:lang="de">Forstwissenschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18350-4">
    <skos:broader rdf:resource="thsys/71016"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18350-4">
    <skos:broader rdf:resource="thsys/70227"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18350-4">
    <skos:broader rdf:resource="descriptor/10032-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12785-6">
    <skos:prefLabel xml:lang="en">Terms of business</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12785-6">
    <skos:prefLabel xml:lang="de">Konditionen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12490-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11189-6">
    <skos:scopeNote xml:lang="de">Fragen der Gleichbehandlung und Chancengleichheit von Frauen und anderen traditionell benachteiligten Gruppen auf dem Arbeitsmarkt.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19001-3">
    <skos:related rdf:resource="descriptor/12564-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19001-3">
    <skos:related rdf:resource="descriptor/12585-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19001-3">
    <skos:related rdf:resource="descriptor/19002-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24856-0">
    <skos:broader rdf:resource="thsys/70485"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12217-1">
    <skos:prefLabel xml:lang="en">Accounting policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12217-1">
    <skos:prefLabel xml:lang="de">Bilanzpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13354-0">
    <skos:broader rdf:resource="thsys/70829"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13354-0">
    <skos:broader rdf:resource="descriptor/12907-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19387-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12514-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10241-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18701-1">
    <skos:related rdf:resource="descriptor/10135-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18389-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16688-5">
    <skos:broader rdf:resource="thsys/73355"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16688-5">
    <skos:broader rdf:resource="descriptor/16564-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19239-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17039-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18835-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12958-1">
    <skos:altLabel xml:lang="de">Weinwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12958-1">
    <skos:altLabel xml:lang="de">Weinkellerei</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12958-1">
    <skos:altLabel xml:lang="de">Weinanbau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10626-1">
    <skos:scopeNote xml:lang="de">Nur für internationale Integrationstendenzen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19815-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20826-1">
    <skos:broader rdf:resource="thsys/73366"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20826-1">
    <skos:broader rdf:resource="descriptor/16435-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20826-1">
    <skos:broader rdf:resource="descriptor/20855-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13282-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19475-0">
    <skos:altLabel xml:lang="en">Information services industry</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19475-0">
    <skos:altLabel xml:lang="de">Informationszentrum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19475-0">
    <skos:altLabel xml:lang="de">Informationssektor</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19475-0">
    <skos:altLabel xml:lang="de">Informationsindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19475-0">
    <skos:altLabel xml:lang="de">Informationsgewerbe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19475-0">
    <skos:altLabel xml:lang="de">Informationsanbieter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19475-0">
    <skos:altLabel xml:lang="de">Informations- und Dokumentationszentrum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19475-0">
    <skos:altLabel xml:lang="de">Dokumentationszentrum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19475-0">
    <skos:altLabel xml:lang="de">Dokumentationsstelle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15873-1">
    <skos:related rdf:resource="descriptor/15754-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15873-1">
    <skos:related rdf:resource="descriptor/14180-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15873-1">
    <skos:related rdf:resource="descriptor/12015-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15873-1">
    <skos:related rdf:resource="descriptor/12980-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18177-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10556-3">
    <skos:related rdf:resource="descriptor/10555-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10556-3">
    <skos:related rdf:resource="descriptor/18695-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15061-5">
    <skos:related rdf:resource="descriptor/15076-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15061-5">
    <skos:related rdf:resource="descriptor/15023-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15061-5">
    <skos:related rdf:resource="descriptor/15941-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15061-5">
    <skos:related rdf:resource="descriptor/15916-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15061-5">
    <skos:related rdf:resource="descriptor/15914-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15061-5">
    <skos:related rdf:resource="descriptor/15912-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15061-5">
    <skos:related rdf:resource="descriptor/15708-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15061-5">
    <skos:related rdf:resource="descriptor/15909-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16479-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13519-1">
    <skos:broader rdf:resource="thsys/70932"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13519-1">
    <skos:broader rdf:resource="thsys/70926"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13519-1">
    <skos:broader rdf:resource="descriptor/11833-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11418-3">
    <skos:broader rdf:resource="thsys/70055"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11418-3">
    <skos:broader rdf:resource="descriptor/11464-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11418-3">
    <skos:broader rdf:resource="descriptor/10000-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18142-4">
    <skos:related rdf:resource="descriptor/16429-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18142-4">
    <skos:related rdf:resource="descriptor/19756-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18142-4">
    <skos:related rdf:resource="descriptor/24663-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18142-4">
    <skos:related rdf:resource="descriptor/16678-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18142-4">
    <skos:related rdf:resource="descriptor/11273-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18142-4">
    <skos:related rdf:resource="descriptor/18863-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18142-4">
    <skos:related rdf:resource="descriptor/16628-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18142-4">
    <skos:related rdf:resource="descriptor/15920-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18142-4">
    <skos:related rdf:resource="descriptor/16477-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18142-4">
    <skos:related rdf:resource="descriptor/21306-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10107-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18662-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15868-1">
    <skos:broader rdf:resource="thsys/73353"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13662-3">
    <skos:related rdf:resource="descriptor/16010-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13662-3">
    <skos:related rdf:resource="descriptor/18211-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13662-3">
    <skos:related rdf:resource="descriptor/16303-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13662-3">
    <skos:related rdf:resource="descriptor/16379-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13662-3">
    <skos:related rdf:resource="descriptor/13661-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13662-3">
    <skos:related rdf:resource="descriptor/15821-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13662-3">
    <skos:related rdf:resource="descriptor/13660-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13662-3">
    <skos:related rdf:resource="descriptor/16515-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13662-3">
    <skos:related rdf:resource="descriptor/10142-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13662-3">
    <skos:related rdf:resource="descriptor/15961-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13662-3">
    <skos:related rdf:resource="descriptor/16604-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13662-3">
    <skos:related rdf:resource="descriptor/16426-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16143-5">
    <skos:related rdf:resource="descriptor/13418-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16143-5">
    <skos:related rdf:resource="descriptor/11322-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16143-5">
    <skos:related rdf:resource="descriptor/16531-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16143-5">
    <skos:related rdf:resource="descriptor/16131-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10764-3">
    <skos:related rdf:resource="descriptor/18127-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10764-3">
    <skos:related rdf:resource="descriptor/10770-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10764-3">
    <skos:related rdf:resource="descriptor/11417-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10764-3">
    <skos:related rdf:resource="descriptor/10769-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10764-3">
    <skos:related rdf:resource="descriptor/10721-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10764-3">
    <skos:related rdf:resource="descriptor/10676-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10764-3">
    <skos:related rdf:resource="descriptor/10746-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10764-3">
    <skos:related rdf:resource="descriptor/11481-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10764-3">
    <skos:related rdf:resource="descriptor/11416-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10764-3">
    <skos:related rdf:resource="descriptor/11415-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10764-3">
    <skos:related rdf:resource="descriptor/10776-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10764-3">
    <skos:related rdf:resource="descriptor/19260-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17666-3">
    <skos:prefLabel xml:lang="en">Malawi</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17666-3">
    <skos:prefLabel xml:lang="de">Malawi</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10285-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18423-3">
    <skos:related rdf:resource="descriptor/18585-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18423-3">
    <skos:related rdf:resource="descriptor/13558-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18423-3">
    <skos:related rdf:resource="descriptor/18421-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18423-3">
    <skos:related rdf:resource="descriptor/13051-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18423-3">
    <skos:related rdf:resource="descriptor/14433-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11064-5">
    <skos:related rdf:resource="descriptor/16566-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11064-5">
    <skos:related rdf:resource="descriptor/19138-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11064-5">
    <skos:related rdf:resource="descriptor/19118-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11064-5">
    <skos:related rdf:resource="descriptor/11042-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70187">
    <rdfs:label xml:lang="de">Finanzmärkte</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19234-1">
    <skos:altLabel xml:lang="de">Intraregionale Vermögensverteilung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19234-1">
    <skos:altLabel xml:lang="de">Interregionale Vermögensverteilung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11565-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13714-3">
    <skos:related rdf:resource="descriptor/18726-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13714-3">
    <skos:related rdf:resource="descriptor/20261-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13714-3">
    <skos:related rdf:resource="descriptor/19102-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13714-3">
    <skos:related rdf:resource="descriptor/19835-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13714-3">
    <skos:related rdf:resource="descriptor/12185-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13714-3">
    <skos:related rdf:resource="descriptor/19522-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13714-3">
    <skos:related rdf:resource="descriptor/19154-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13714-3">
    <skos:related rdf:resource="descriptor/18674-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13714-3">
    <skos:related rdf:resource="descriptor/12174-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13714-3">
    <skos:related rdf:resource="descriptor/12232-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11134-3">
    <skos:scopeNote xml:lang="de">Produktionsvereinigung in Planwirtschaften.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70310">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13117-0">
    <skos:narrower rdf:resource="descriptor/13125-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13117-0">
    <skos:narrower rdf:resource="descriptor/13102-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13117-0">
    <skos:narrower rdf:resource="descriptor/13100-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10019-1">
    <skos:related rdf:resource="descriptor/12302-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10019-1">
    <skos:related rdf:resource="descriptor/19563-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10019-1">
    <skos:related rdf:resource="descriptor/10022-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10019-1">
    <skos:related rdf:resource="descriptor/18678-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10019-1">
    <skos:related rdf:resource="descriptor/18178-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10019-1">
    <skos:related rdf:resource="descriptor/12251-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10019-1">
    <skos:related rdf:resource="descriptor/19088-0"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id135">
    <rdfs:seeAlso rdf:resource="descriptor/13110-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10621-4">
    <skos:prefLabel xml:lang="en">International economic legislation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10621-4">
    <skos:prefLabel xml:lang="de">Internationales Wirtschaftsrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11181-1">
    <skos:prefLabel xml:lang="en">Work ethic</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11181-1">
    <skos:prefLabel xml:lang="de">Arbeitsethik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18559-6">
    <skos:scopeNote xml:lang="de">Auch für Produkte, für welche die Umweltverträglichkeit lediglich behauptet wird.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16170-2">
    <skos:altLabel xml:lang="de">Startrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16170-2">
    <skos:altLabel xml:lang="de">Luftverkehrsrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16170-2">
    <skos:altLabel xml:lang="de">Luftverkehrsgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16170-2">
    <skos:altLabel xml:lang="de">Luftfrachtrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16170-2">
    <skos:altLabel xml:lang="de">Landerecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16170-2">
    <skos:altLabel xml:lang="de">Chartervertragsrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70854">
    <skos:narrower rdf:resource="descriptor/18503-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70854">
    <skos:narrower rdf:resource="descriptor/18843-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70854">
    <skos:narrower rdf:resource="descriptor/18842-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70854">
    <skos:narrower rdf:resource="descriptor/13287-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70854">
    <skos:narrower rdf:resource="descriptor/18018-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70854">
    <skos:narrower rdf:resource="descriptor/18510-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70854">
    <skos:narrower rdf:resource="descriptor/18841-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70854">
    <skos:narrower rdf:resource="descriptor/18840-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70854">
    <skos:narrower rdf:resource="descriptor/16031-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70854">
    <skos:narrower rdf:resource="descriptor/18839-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70854">
    <skos:narrower rdf:resource="descriptor/18838-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70854">
    <skos:narrower rdf:resource="descriptor/12180-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70854">
    <skos:narrower rdf:resource="descriptor/13456-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70854">
    <skos:narrower rdf:resource="descriptor/13429-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19646-6">
    <skos:related rdf:resource="descriptor/16654-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71046">
    <skos:broader rdf:resource="thsys/70183"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12964-6">
    <skos:broader rdf:resource="thsys/71018"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12964-6">
    <skos:broader rdf:resource="descriptor/12922-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12964-6">
    <skos:broader rdf:resource="descriptor/19383-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70154">
    <skos:broader rdf:resource="thsys/n"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11198-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11870-6">
    <skos:broader rdf:resource="thsys/73360"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11870-6">
    <skos:broader rdf:resource="thsys/71068"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11870-6">
    <skos:broader rdf:resource="descriptor/16019-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14854-3">
    <skos:narrower rdf:resource="descriptor/18291-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14854-3">
    <skos:narrower rdf:resource="descriptor/14883-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71043">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">B.03.01.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10773-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17630-3">
    <skos:prefLabel xml:lang="en">Lebanon</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17630-3">
    <skos:prefLabel xml:lang="de">Libanon</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19650-1">
    <skos:altLabel xml:lang="en">Theory of migration</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19650-1">
    <skos:altLabel xml:lang="en">Economics of migration</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19650-1">
    <skos:altLabel xml:lang="de">Migrationsökonomik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71108">
    <skos:narrower rdf:resource="descriptor/11618-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71108">
    <skos:narrower rdf:resource="descriptor/10809-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71108">
    <skos:narrower rdf:resource="descriptor/13726-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71108">
    <skos:narrower rdf:resource="descriptor/18088-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71108">
    <skos:narrower rdf:resource="descriptor/11421-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71108">
    <skos:narrower rdf:resource="descriptor/19277-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71108">
    <skos:narrower rdf:resource="descriptor/10801-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71108">
    <skos:narrower rdf:resource="descriptor/19276-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71108">
    <skos:narrower rdf:resource="descriptor/19275-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18493-3">
    <skos:related rdf:resource="descriptor/13860-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18493-3">
    <skos:related rdf:resource="descriptor/18508-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18493-3">
    <skos:related rdf:resource="descriptor/18495-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18493-3">
    <skos:related rdf:resource="descriptor/18491-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17618-0">
    <skos:broader rdf:resource="thsys/70253"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17618-0">
    <skos:broader rdf:resource="descriptor/17608-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17618-0">
    <skos:broader rdf:resource="descriptor/17978-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19536-6">
    <skos:altLabel xml:lang="de">Diskontierungsfaktor</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19536-6">
    <skos:altLabel xml:lang="de">Diskontierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19536-6">
    <skos:altLabel xml:lang="de">Abzinsungsfaktor</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12391-6">
    <skos:related rdf:resource="descriptor/12435-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12391-6">
    <skos:related rdf:resource="descriptor/12425-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15932-4">
    <skos:related rdf:resource="descriptor/19860-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15932-4">
    <skos:related rdf:resource="descriptor/16513-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15932-4">
    <skos:related rdf:resource="descriptor/15928-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15932-4">
    <skos:related rdf:resource="descriptor/16404-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15932-4">
    <skos:related rdf:resource="descriptor/16416-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15932-4">
    <skos:related rdf:resource="descriptor/19639-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15932-4">
    <skos:related rdf:resource="descriptor/19642-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15932-4">
    <skos:related rdf:resource="descriptor/15907-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15932-4">
    <skos:related rdf:resource="descriptor/19771-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70292">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">B.07.05</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16549-5">
    <skos:prefLabel xml:lang="en">Local politics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16549-5">
    <skos:prefLabel xml:lang="de">Kommunalpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10581-4">
    <skos:prefLabel xml:lang="en">MFA (Multifibre Arrangement)</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10581-4">
    <skos:prefLabel xml:lang="de">Welttextilabkommen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17706-3">
    <skos:broader rdf:resource="thsys/70560"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17706-3">
    <skos:broader rdf:resource="descriptor/17724-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18405-5">
    <skos:broader rdf:resource="thsys/71001"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18405-5">
    <skos:broader rdf:resource="thsys/70979"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18405-5">
    <skos:broader rdf:resource="descriptor/11820-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13541-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11540-6">
    <skos:altLabel xml:lang="de">Staatsfinanzen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11540-6">
    <skos:altLabel xml:lang="de">Öffentliche Finanzen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11540-6">
    <skos:altLabel xml:lang="de">Finanzwirtschaft des Staates</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19641-2">
    <skos:broader rdf:resource="thsys/70142"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19641-2">
    <skos:broader rdf:resource="descriptor/15916-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71006">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14200-1">
    <skos:altLabel xml:lang="en">Black coal</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14200-1">
    <skos:altLabel xml:lang="de">Steinkohle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10287-4">
    <skos:related rdf:resource="descriptor/10115-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16463-1">
    <skos:narrower rdf:resource="descriptor/19720-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16463-1">
    <skos:narrower rdf:resource="descriptor/16497-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18510-1">
    <skos:broader rdf:resource="thsys/70965"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18510-1">
    <skos:broader rdf:resource="thsys/70854"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18510-1">
    <skos:broader rdf:resource="descriptor/18511-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18510-1">
    <skos:broader rdf:resource="descriptor/12180-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13429-2">
    <skos:altLabel xml:lang="en">Rural cooperative</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13429-2">
    <skos:altLabel xml:lang="de">Landwirtschaftskooperative</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13429-2">
    <skos:altLabel xml:lang="de">Landwirtschaftsgenossenschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13429-2">
    <skos:altLabel xml:lang="de">Landwirtschaftliche Genossenschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13429-2">
    <skos:altLabel xml:lang="de">Landwirtschaftliche Erzeugergemeinschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13429-2">
    <skos:altLabel xml:lang="de">Ländliche Genossenschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13429-2">
    <skos:altLabel xml:lang="de">Dorfgemeinschaftskooperative</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16426-0">
    <skos:prefLabel xml:lang="en">Freedom</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16426-0">
    <skos:prefLabel xml:lang="de">Freiheit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10446-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11983-0">
    <skos:prefLabel xml:lang="en">Urban decline</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11983-0">
    <skos:prefLabel xml:lang="de">Städtischer Niedergang</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13320-3">
    <skos:related rdf:resource="descriptor/13362-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13320-3">
    <skos:related rdf:resource="descriptor/15675-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13320-3">
    <skos:related rdf:resource="descriptor/18892-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14739-3">
    <skos:prefLabel xml:lang="en">Dishwashing product</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14739-3">
    <skos:prefLabel xml:lang="de">Spülmittel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71060">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.13.01.03</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12898-0">
    <skos:related rdf:resource="descriptor/12381-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12898-0">
    <skos:related rdf:resource="descriptor/12216-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12898-0">
    <skos:related rdf:resource="descriptor/12896-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18837-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70989">
    <rdfs:label xml:lang="de">Kunststoff- und Gummiindustrie</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11366-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70924">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.12.07</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70814">
    <skos:broader rdf:resource="thsys/w"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19217-1">
    <skos:broader rdf:resource="thsys/70180"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19217-1">
    <skos:broader rdf:resource="descriptor/10030-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11928-1">
    <skos:altLabel xml:lang="de">Regionalmodell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11928-1">
    <skos:altLabel xml:lang="de">Raumwirtschaftslehre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11928-1">
    <skos:altLabel xml:lang="de">Räumliche Wirtschaftstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14464-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11388-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13058-4">
    <skos:broader rdf:resource="thsys/71002"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13058-4">
    <skos:broader rdf:resource="descriptor/12993-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15719-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13904-5">
    <skos:altLabel xml:lang="de">Schusterinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13904-5">
    <skos:altLabel xml:lang="de">Schuster</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13904-5">
    <skos:altLabel xml:lang="de">Schuhmacherinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13904-5">
    <skos:altLabel xml:lang="de">Schuhmacher</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13904-5">
    <skos:altLabel xml:lang="de">Sattlerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13904-5">
    <skos:altLabel xml:lang="de">Sattler</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13904-5">
    <skos:altLabel xml:lang="de">Pelznäherinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13904-5">
    <skos:altLabel xml:lang="de">Pelznäher</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13904-5">
    <skos:altLabel xml:lang="de">Lederverarbeitende Berufe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13904-5">
    <skos:altLabel xml:lang="de">Kürschnerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13904-5">
    <skos:altLabel xml:lang="de">Kürschner</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13904-5">
    <skos:altLabel xml:lang="de">Gerberinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13904-5">
    <skos:altLabel xml:lang="de">Gerber</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13904-5">
    <skos:altLabel xml:lang="de">Fellverarbeiterinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13904-5">
    <skos:altLabel xml:lang="de">Fellverarbeiter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13085-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19178-6">
    <skos:related rdf:resource="descriptor/19177-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19562-5">
    <skos:related rdf:resource="descriptor/12111-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19562-5">
    <skos:related rdf:resource="descriptor/12617-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12078-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id228">
    <rdf:value xml:lang="de">Für staatliche Souveränität</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14946-5">
    <skos:prefLabel xml:lang="en">Jam</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14946-5">
    <skos:prefLabel xml:lang="de">Konfitüre</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16712-6">
    <skos:related rdf:resource="descriptor/16727-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16712-6">
    <skos:related rdf:resource="descriptor/11161-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10895-2">
    <skos:prefLabel xml:lang="en">Competition</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10895-2">
    <skos:prefLabel xml:lang="de">Wettbewerb</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19462-2">
    <skos:related rdf:resource="descriptor/19472-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19462-2">
    <skos:related rdf:resource="descriptor/11207-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19462-2">
    <skos:related rdf:resource="descriptor/18982-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19462-2">
    <skos:related rdf:resource="descriptor/10503-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19462-2">
    <skos:related rdf:resource="descriptor/19460-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19462-2">
    <skos:related rdf:resource="descriptor/19461-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17679-1">
    <skos:prefLabel xml:lang="en">Uganda</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17679-1">
    <skos:prefLabel xml:lang="de">Uganda</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12921-3">
    <skos:related rdf:resource="descriptor/13222-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12921-3">
    <skos:related rdf:resource="descriptor/14043-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12921-3">
    <skos:related rdf:resource="descriptor/13113-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11579-0">
    <skos:altLabel xml:lang="en">Government procurement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11579-0">
    <skos:altLabel xml:lang="de">Vergabeverfahren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11579-0">
    <skos:altLabel xml:lang="de">Staatsauftrag</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11579-0">
    <skos:altLabel xml:lang="de">Regierungskauf</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11579-0">
    <skos:altLabel xml:lang="de">Öffentliches Vergabewesen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11579-0">
    <skos:altLabel xml:lang="de">Öffentliches Beschaffungswesen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11579-0">
    <skos:altLabel xml:lang="de">Öffentliches Auftragswesen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11579-0">
    <skos:altLabel xml:lang="de">Öffentlicher Auftrag</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11579-0">
    <skos:altLabel xml:lang="de">Öffentliche Beschaffungspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11579-0">
    <skos:altLabel xml:lang="de">Behördeneinkauf</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18774-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14169-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14794-2">
    <skos:related rdf:resource="descriptor/13120-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14169-4">
    <skos:related rdf:resource="descriptor/13374-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14169-4">
    <skos:related rdf:resource="descriptor/19627-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14169-4">
    <skos:related rdf:resource="descriptor/18901-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14169-4">
    <skos:related rdf:resource="descriptor/14129-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14169-4">
    <skos:related rdf:resource="descriptor/12955-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14169-4">
    <skos:related rdf:resource="descriptor/14731-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14169-4">
    <skos:related rdf:resource="descriptor/15659-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14169-4">
    <skos:related rdf:resource="descriptor/14989-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10120-5">
    <skos:broader rdf:resource="thsys/70486"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10120-5">
    <skos:broader rdf:resource="thsys/70037"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10120-5">
    <skos:broader rdf:resource="descriptor/10326-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10099-5">
    <skos:broader rdf:resource="thsys/71114"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10099-5">
    <skos:broader rdf:resource="thsys/70057"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10099-5">
    <skos:broader rdf:resource="descriptor/10113-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10099-5">
    <skos:broader rdf:resource="descriptor/11765-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18994-2">
    <skos:related rdf:resource="descriptor/21072-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18994-2">
    <skos:related rdf:resource="descriptor/13418-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18994-2">
    <skos:related rdf:resource="descriptor/13810-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18905-6">
    <skos:scopeNote xml:lang="de">Verpflichtung übergeordneter Gemeinschaften, im Bedarfsfall untergeordneten Gemeinschaften beizustehen, jedoch bei Beachtung der Eigenständigkeit der kleineren Gemeinschaften. Ursprünglich ein Gebot der Katholischen Soziallehre. Findet vor allem in der Sozialpolitik, im Staatsrecht und der Finanzwissenschaft Anwendung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18544-5">
    <skos:prefLabel xml:lang="en">International chamber of commerce</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18544-5">
    <skos:prefLabel xml:lang="de">Auslandshandelskammer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12897-2">
    <skos:broader rdf:resource="thsys/71041"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19232-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19660-5">
    <skos:altLabel xml:lang="de">Automatisierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18023-5">
    <skos:prefLabel xml:lang="en">Production planning and control system</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18023-5">
    <skos:prefLabel xml:lang="de">PPS-System</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19326-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10572-5">
    <skos:altLabel xml:lang="en">International joint venture</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10572-5">
    <skos:altLabel xml:lang="en">Cooperative joint venture</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10572-5">
    <skos:altLabel xml:lang="de">Internationales Gemeinschaftsunternehmen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10572-5">
    <skos:altLabel xml:lang="de">Gemeinschaftsunternehmen im Ausland</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10835-6">
    <skos:prefLabel xml:lang="en">Foreign trade statistics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10835-6">
    <skos:prefLabel xml:lang="de">Außenhandelsstatistik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10770-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19568-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15941-3">
    <skos:scopeNote xml:lang="de">Nur für die menschliche Fruchtbarkeit.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13391-1">
    <skos:altLabel xml:lang="de">Nachlaßverwaltung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13391-1">
    <skos:altLabel xml:lang="de">Nachlassverwaltung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12112-1">
    <skos:prefLabel xml:lang="en">Organization theory</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12112-1">
    <skos:prefLabel xml:lang="de">Organisationstheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19958-1">
    <skos:prefLabel xml:lang="en">GATS</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19958-1">
    <skos:prefLabel xml:lang="de">GATS</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19330-5">
    <skos:related rdf:resource="descriptor/10697-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18679-3">
    <skos:broader rdf:resource="thsys/70897"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18679-3">
    <skos:broader rdf:resource="thsys/70187"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18679-3">
    <skos:broader rdf:resource="thsys/70049"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14909-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70855">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18808-4">
    <skos:altLabel xml:lang="de">Provinzverwaltung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18808-4">
    <skos:altLabel xml:lang="de">Landesverwaltung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18808-4">
    <skos:altLabel xml:lang="de">Kreisverwaltung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18808-4">
    <skos:altLabel xml:lang="de">Kantonsverwaltung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18808-4">
    <skos:altLabel xml:lang="de">Kantonale Verwaltung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12955-0">
    <skos:altLabel xml:lang="de">Viehwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12955-0">
    <skos:altLabel xml:lang="de">Viehhaltung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12955-0">
    <skos:altLabel xml:lang="de">Tierproduktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12955-0">
    <skos:altLabel xml:lang="de">Tierische Veredlung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12955-0">
    <skos:altLabel xml:lang="de">Nutztierhaltung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12955-0">
    <skos:altLabel xml:lang="de">Landwirtschaftliche Tierhaltung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13232-0">
    <skos:altLabel xml:lang="de">Schuhhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13232-0">
    <skos:altLabel xml:lang="de">Schuhgeschäft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13232-0">
    <skos:altLabel xml:lang="de">Schuhfachhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11751-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70953">
    <skos:narrower rdf:resource="descriptor/18018-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70953">
    <skos:narrower rdf:resource="descriptor/13457-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70953">
    <skos:narrower rdf:resource="descriptor/15080-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70953">
    <skos:narrower rdf:resource="descriptor/13209-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70953">
    <skos:narrower rdf:resource="descriptor/12668-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70953">
    <skos:narrower rdf:resource="descriptor/13456-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70953">
    <skos:narrower rdf:resource="descriptor/13496-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18359-0">
    <skos:scopeNote xml:lang="de">Beschäftigt sich mit der Energieerzeugung in Kraftwerken sowie mit der Übertragung, Verteilung und Anwendung elektrischer Energie.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70477">
    <rdfs:label xml:lang="de">Elektrohaushaltsgeräte</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19433-2">
    <skos:altLabel xml:lang="en">Sliding-scale regulation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19433-2">
    <skos:altLabel xml:lang="en">Revenue caps</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19433-2">
    <skos:altLabel xml:lang="en">Price-cap regulation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19433-2">
    <skos:altLabel xml:lang="en">Price cap regulation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18289-2">
    <skos:prefLabel xml:lang="en">Genetic engineering</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18289-2">
    <skos:prefLabel xml:lang="de">Gentechnik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73375">
    <skos:broader rdf:resource="thsys/73382"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15001-2">
    <skos:broader rdf:resource="thsys/70337"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15001-2">
    <skos:broader rdf:resource="descriptor/15005-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11238-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11866-4">
    <skos:related rdf:resource="descriptor/19628-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11866-4">
    <skos:related rdf:resource="descriptor/15746-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11944-3">
    <skos:prefLabel xml:lang="en">EU cohesion fund</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11944-3">
    <skos:prefLabel xml:lang="de">EU-Regionalfonds</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16964-0">
    <skos:prefLabel xml:lang="en">Jura (Canton)</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16964-0">
    <skos:prefLabel xml:lang="de">Jura (Kanton)</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10899-1">
    <skos:broader rdf:resource="thsys/70212"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10899-1">
    <skos:broader rdf:resource="descriptor/10913-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12928-3">
    <skos:broader rdf:resource="thsys/70815"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12928-3">
    <skos:broader rdf:resource="thsys/70239"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16927-6">
    <skos:scopeNote xml:lang="de">Deutsche Demokratische Republik 1949-1990</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18487-5">
    <skos:prefLabel xml:lang="en">Industrial wages</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18487-5">
    <skos:prefLabel xml:lang="de">Industrielohn</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11554-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10686-4">
    <skos:related rdf:resource="descriptor/10882-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10686-4">
    <skos:related rdf:resource="descriptor/10616-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10686-4">
    <skos:related rdf:resource="descriptor/11823-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10686-4">
    <skos:related rdf:resource="descriptor/19324-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10686-4">
    <skos:related rdf:resource="descriptor/18544-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10686-4">
    <skos:related rdf:resource="descriptor/10629-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10686-4">
    <skos:related rdf:resource="descriptor/10605-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10686-4">
    <skos:related rdf:resource="descriptor/10852-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10686-4">
    <skos:related rdf:resource="descriptor/10833-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12232-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12171-6">
    <skos:prefLabel xml:lang="en">Intraorganizational communication</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12171-6">
    <skos:prefLabel xml:lang="de">Innerbetriebliche Kommunikation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15543-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13352-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id139">
    <rdf:value xml:lang="de">Für Impfstoffe</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14733-1">
    <skos:broader rdf:resource="thsys/70536"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14733-1">
    <skos:broader rdf:resource="descriptor/14734-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15825-5">
    <skos:prefLabel xml:lang="en">Leadership theory</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15825-5">
    <skos:prefLabel xml:lang="de">Führungstheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13462-4">
    <skos:broader rdf:resource="thsys/73387"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13462-4">
    <skos:broader rdf:resource="thsys/70963"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13462-4">
    <skos:broader rdf:resource="thsys/70109"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13462-4">
    <skos:broader rdf:resource="descriptor/15706-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13238-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13508-6">
    <skos:narrower rdf:resource="descriptor/18089-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18028-2">
    <skos:broader rdf:resource="thsys/70265"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18028-2">
    <skos:broader rdf:resource="descriptor/13669-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11913-0">
    <skos:narrower rdf:resource="descriptor/13862-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11913-0">
    <skos:narrower rdf:resource="descriptor/12005-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11913-0">
    <skos:narrower rdf:resource="descriptor/18816-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11913-0">
    <skos:narrower rdf:resource="descriptor/18799-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11913-0">
    <skos:narrower rdf:resource="descriptor/18490-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11913-0">
    <skos:narrower rdf:resource="descriptor/11905-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11288-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12697-3">
    <skos:altLabel xml:lang="en">Structural adjustment policy</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12697-3">
    <skos:altLabel xml:lang="en">Structural adjustment</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12697-3">
    <skos:altLabel xml:lang="en">Adjustment policy</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12697-3">
    <skos:altLabel xml:lang="de">Strukturanpassungspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12697-3">
    <skos:altLabel xml:lang="de">Anpassungspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12160-4">
    <skos:broader rdf:resource="thsys/70562"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10522-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19841-1">
    <skos:altLabel xml:lang="de">Staatsanwaltschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19841-1">
    <skos:altLabel xml:lang="de">Rechtsprechende Gewalt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19841-1">
    <skos:altLabel xml:lang="de">Rechtspflege</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19841-1">
    <skos:altLabel xml:lang="de">Judikative</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12029-2">
    <skos:prefLabel xml:lang="en">Real estate</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12029-2">
    <skos:prefLabel xml:lang="de">Immobilien</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14258-5">
    <skos:altLabel xml:lang="de">Asbestzement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15734-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10327-4">
    <skos:prefLabel xml:lang="en">Production function</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10327-4">
    <skos:prefLabel xml:lang="de">Produktionsfunktion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24733-2">
    <skos:altLabel xml:lang="en">Delisting (Exchange)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24733-2">
    <skos:altLabel xml:lang="de">Börsenrückzug</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11574-3">
    <skos:altLabel xml:lang="en">Housing assistance</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11574-3">
    <skos:altLabel xml:lang="de">Wohnbeihilfe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11574-3">
    <skos:altLabel xml:lang="de">Mietzuschuss</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11574-3">
    <skos:altLabel xml:lang="de">Mietzuschuß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11574-3">
    <skos:altLabel xml:lang="de">Mietsubvention</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19284-0">
    <skos:altLabel xml:lang="de">Voucherprivatisierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19284-0">
    <skos:altLabel xml:lang="de">Kupon-Privatisierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11378-3">
    <skos:related rdf:resource="descriptor/11379-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11378-3">
    <skos:related rdf:resource="descriptor/18879-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11378-3">
    <skos:related rdf:resource="descriptor/18878-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11378-3">
    <skos:related rdf:resource="descriptor/11373-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11378-3">
    <skos:related rdf:resource="descriptor/11356-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11378-3">
    <skos:related rdf:resource="descriptor/18856-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17914-3">
    <skos:altLabel xml:lang="de">Republik Bolivien</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18279-5">
    <skos:altLabel xml:lang="de">Kompositwerkstoff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20842-3">
    <skos:altLabel xml:lang="de">Verantwortliches Regieren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20842-3">
    <skos:altLabel xml:lang="de">Verantwortliche Regierungsführung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20842-3">
    <skos:altLabel xml:lang="de">Gute Regierungsführung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11659-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16949-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18289-2">
    <skos:broader rdf:resource="thsys/73348"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18289-2">
    <skos:broader rdf:resource="thsys/73344"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18289-2">
    <skos:broader rdf:resource="thsys/70993"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18289-2">
    <skos:broader rdf:resource="descriptor/15635-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10135-6">
    <skos:broader rdf:resource="thsys/70486"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10135-6">
    <skos:broader rdf:resource="thsys/70455"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10135-6">
    <skos:broader rdf:resource="thsys/70037"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10135-6">
    <skos:broader rdf:resource="descriptor/12249-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10135-6">
    <skos:broader rdf:resource="descriptor/12285-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10135-6">
    <skos:broader rdf:resource="descriptor/10186-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18130-4">
    <skos:broader rdf:resource="thsys/73394"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18130-4">
    <skos:broader rdf:resource="thsys/70175"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18130-4">
    <skos:broader rdf:resource="descriptor/16709-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#mappingRelation">
    <rdfs:subPropertyOf rdf:resource="http://www.w3.org/2004/02/skos/core#semanticRelation"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18810-3">
    <skos:altLabel xml:lang="en">Supervision</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18810-3">
    <skos:altLabel xml:lang="en">Regulatory agency</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18810-3">
    <skos:altLabel xml:lang="de">Wirtschaftsverwaltung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18810-3">
    <skos:altLabel xml:lang="de">Wirtschaftsbehörde</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18810-3">
    <skos:altLabel xml:lang="de">Wirtschaftsaufsicht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18810-3">
    <skos:altLabel xml:lang="de">Regulierungsbehörde</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19287-1">
    <skos:altLabel xml:lang="de">Schwundgeld</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19287-1">
    <skos:altLabel xml:lang="de">Regionalwährung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19287-1">
    <skos:altLabel xml:lang="de">Regionalgeld</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19287-1">
    <skos:altLabel xml:lang="de">Regiogeld</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19287-1">
    <skos:altLabel xml:lang="de">Komplementärgeld</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19287-1">
    <skos:altLabel xml:lang="de">Freiwirtschaftslehre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19287-1">
    <skos:altLabel xml:lang="de">Freigeld</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11233-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13146-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73298">
    <rdfs:label xml:lang="en">Luxury foodstuffs and tobacco</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73298">
    <rdfs:label xml:lang="de">Genussmittel</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70039">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71105">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.05.06.02.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12908-2">
    <skos:prefLabel xml:lang="en">Large manufacturing firms</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12908-2">
    <skos:prefLabel xml:lang="de">Großindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13457-4">
    <skos:related rdf:resource="descriptor/12781-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18677-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19127-2">
    <skos:narrower rdf:resource="descriptor/12788-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14368-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11995-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13245-5">
    <skos:prefLabel xml:lang="en">Pipeline transport</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13245-5">
    <skos:prefLabel xml:lang="de">Leitungstransport</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14317-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19857-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19789-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15225-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17750-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16233-4">
    <skos:related rdf:resource="descriptor/16017-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16233-4">
    <skos:related rdf:resource="descriptor/13420-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16233-4">
    <skos:related rdf:resource="descriptor/16329-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16233-4">
    <skos:related rdf:resource="descriptor/15976-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16233-4">
    <skos:related rdf:resource="descriptor/16292-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16233-4">
    <skos:related rdf:resource="descriptor/16405-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15668-2">
    <skos:altLabel xml:lang="en">Horse racing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15668-2">
    <skos:altLabel xml:lang="de">Pferderennen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13960-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73388">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17638-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id322">
    <rdf:value xml:lang="de">für den industriellen Strukturwandel</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18544-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13827-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71018">
    <skos:broader rdf:resource="thsys/w"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10781-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70128">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11378-3">
    <skos:altLabel xml:lang="de">Studentinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11378-3">
    <skos:altLabel xml:lang="de">Studenten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71003">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.06.01.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12138-4">
    <skos:broader rdf:resource="thsys/70562"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12138-4">
    <skos:broader rdf:resource="descriptor/12111-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70888">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10961-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13721-6">
    <skos:prefLabel xml:lang="en">Fixed-income securities</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13721-6">
    <skos:prefLabel xml:lang="de">Festverzinsliches Wertpapier</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19723-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15021-3">
    <skos:prefLabel xml:lang="en">Delphi technique</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15021-3">
    <skos:prefLabel xml:lang="de">Delphi-Technik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14225-6">
    <skos:altLabel xml:lang="de">Synthetischer Diamant</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14225-6">
    <skos:altLabel xml:lang="de">Industriediamant</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70931">
    <rdfs:label xml:lang="en">Space travel</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70931">
    <rdfs:label xml:lang="de">Raumfahrt</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10808-2">
    <skos:prefLabel xml:lang="en">Financial centre</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10808-2">
    <skos:prefLabel xml:lang="de">Finanzplatz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17791-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70461">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18391-4">
    <skos:broader rdf:resource="thsys/71007"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18391-4">
    <skos:broader rdf:resource="descriptor/12988-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13839-4">
    <skos:related rdf:resource="descriptor/13304-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13839-4">
    <skos:related rdf:resource="descriptor/19532-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13839-4">
    <skos:related rdf:resource="descriptor/11840-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13839-4">
    <skos:related rdf:resource="descriptor/18755-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13839-4">
    <skos:related rdf:resource="descriptor/13837-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12966-2">
    <skos:related rdf:resource="descriptor/19614-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17684-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15392-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14972-4">
    <skos:prefLabel xml:lang="en">Alcoholic beverage</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14972-4">
    <skos:prefLabel xml:lang="de">Alkoholisches Getränk</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10901-5">
    <skos:prefLabel xml:lang="en">Concentration index</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10901-5">
    <skos:prefLabel xml:lang="de">Konzentrationsindex</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19838-4">
    <skos:scopeNote xml:lang="de">Gesetzgebung auf dem Gebiet der Stromwirtschaft.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17642-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71075">
    <skos:prefLabel xml:lang="en">V.09.07.02  Tax policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71075">
    <skos:prefLabel xml:lang="de">V.09.07.02  Steuerpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16061-0">
    <skos:broader rdf:resource="thsys/73390"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16061-0">
    <skos:broader rdf:resource="thsys/70901"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16061-0">
    <skos:broader rdf:resource="descriptor/13693-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15908-1">
    <skos:prefLabel xml:lang="en">Disabled people</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15908-1">
    <skos:prefLabel xml:lang="de">Behinderte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70866">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11409-4">
    <skos:altLabel xml:lang="en">Public educational expenditure</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11409-4">
    <skos:altLabel xml:lang="de">Erziehungsausgaben der öffentlichen Hand</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11409-4">
    <skos:altLabel xml:lang="de">Bildungsbudget</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14070-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18718-5">
    <skos:altLabel xml:lang="de">Bankenmarketing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11337-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15709-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10144-5">
    <skos:narrower rdf:resource="descriptor/10148-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10144-5">
    <skos:narrower rdf:resource="descriptor/11111-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10144-5">
    <skos:narrower rdf:resource="descriptor/19215-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10144-5">
    <skos:narrower rdf:resource="descriptor/15861-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10144-5">
    <skos:narrower rdf:resource="descriptor/10190-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10144-5">
    <skos:narrower rdf:resource="descriptor/15860-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10144-5">
    <skos:narrower rdf:resource="descriptor/10217-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10144-5">
    <skos:narrower rdf:resource="descriptor/10161-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10144-5">
    <skos:narrower rdf:resource="descriptor/10141-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10144-5">
    <skos:narrower rdf:resource="descriptor/10177-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10144-5">
    <skos:narrower rdf:resource="descriptor/10038-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10144-5">
    <skos:narrower rdf:resource="descriptor/11065-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10144-5">
    <skos:narrower rdf:resource="descriptor/10138-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19063-2">
    <skos:broader rdf:resource="thsys/70962"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19063-2">
    <skos:broader rdf:resource="descriptor/13210-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11353-5">
    <skos:editorialNote xml:lang="de">Möglichst spezifischer indexieren.</skos:editorialNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19538-2">
    <skos:prefLabel xml:lang="en">Bid-ask spread</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19538-2">
    <skos:prefLabel xml:lang="de">Bid-Ask Spread</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16487-1">
    <skos:related rdf:resource="descriptor/15685-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16487-1">
    <skos:related rdf:resource="descriptor/19749-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16487-1">
    <skos:related rdf:resource="descriptor/16515-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16487-1">
    <skos:related rdf:resource="descriptor/19251-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16487-1">
    <skos:related rdf:resource="descriptor/11402-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16487-1">
    <skos:related rdf:resource="descriptor/10437-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10981-2">
    <skos:prefLabel xml:lang="en">Structural policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10981-2">
    <skos:prefLabel xml:lang="de">Strukturpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73297">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11771-1">
    <skos:broader rdf:resource="thsys/73360"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11771-1">
    <skos:broader rdf:resource="thsys/71113"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15754-2">
    <skos:related rdf:resource="descriptor/15753-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15754-2">
    <skos:related rdf:resource="descriptor/14047-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15754-2">
    <skos:related rdf:resource="descriptor/15873-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15754-2">
    <skos:related rdf:resource="descriptor/18387-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15754-2">
    <skos:related rdf:resource="descriptor/15751-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15754-2">
    <skos:related rdf:resource="descriptor/12020-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15754-2">
    <skos:related rdf:resource="descriptor/12015-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73354">
    <skos:prefLabel xml:lang="de">N.10.01  Geowissenschaften und Umwelt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72183">
    <skos:broader rdf:resource="thsys/72180"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16544-1">
    <skos:narrower rdf:resource="descriptor/18905-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16544-1">
    <skos:narrower rdf:resource="descriptor/16401-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11369-4">
    <skos:broader rdf:resource="thsys/70847"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11369-4">
    <skos:broader rdf:resource="thsys/70176"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11369-4">
    <skos:broader rdf:resource="thsys/70143"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11369-4">
    <skos:broader rdf:resource="descriptor/11367-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16587-4">
    <skos:narrower rdf:resource="descriptor/13399-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16587-4">
    <skos:narrower rdf:resource="descriptor/18969-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10278-5">
    <skos:altLabel xml:lang="de">Makroökonomische Konsumfunktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10278-5">
    <skos:altLabel xml:lang="de">Konsumfunktion der Gesamtwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10404-5">
    <skos:related rdf:resource="descriptor/10387-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15716-3">
    <skos:altLabel xml:lang="en">Social decline</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15716-3">
    <skos:altLabel xml:lang="en">Social advancement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15716-3">
    <skos:altLabel xml:lang="de">Vertikale Mobilität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15716-3">
    <skos:altLabel xml:lang="de">Statuswechsel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15716-3">
    <skos:altLabel xml:lang="de">Sozialer Aufstieg</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15716-3">
    <skos:altLabel xml:lang="de">Sozialer Abstieg</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11301-3">
    <skos:prefLabel xml:lang="en">Skilled workers</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11301-3">
    <skos:prefLabel xml:lang="de">Facharbeiter</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10138-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14065-2">
    <skos:prefLabel xml:lang="en">Cocoa</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14065-2">
    <skos:prefLabel xml:lang="de">Kakao</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13411-0">
    <skos:broader rdf:resource="thsys/70891"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13411-0">
    <skos:broader rdf:resource="thsys/70832"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13411-0">
    <skos:broader rdf:resource="thsys/70458"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13411-0">
    <skos:broader rdf:resource="descriptor/13297-0"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id325">
    <rdf:value xml:lang="de">Für den Anbau von Tabak</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13128-2">
    <skos:broader rdf:resource="thsys/70987"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13128-2">
    <skos:broader rdf:resource="descriptor/18094-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15535-0">
    <skos:altLabel xml:lang="en">Zero-one-programming</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15535-0">
    <skos:altLabel xml:lang="en">Set-covering problem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15535-0">
    <skos:altLabel xml:lang="en">Cutting planes</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15535-0">
    <skos:altLabel xml:lang="en">Boolean programming</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15535-0">
    <skos:altLabel xml:lang="de">Schnittebenenverfahren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15535-0">
    <skos:altLabel xml:lang="de">Rucksackproblem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15535-0">
    <skos:altLabel xml:lang="de">Partition Problem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15535-0">
    <skos:altLabel xml:lang="de">Null-Eins-Optimierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15535-0">
    <skos:altLabel xml:lang="de">Mengenzerlegungsproblem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15535-0">
    <skos:altLabel xml:lang="de">Knapsack Problem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15535-0">
    <skos:altLabel xml:lang="de">Gemischt-ganzzahlige Optimierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15535-0">
    <skos:altLabel xml:lang="de">Ganzzahlige Programmierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15535-0">
    <skos:altLabel xml:lang="de">Diskrete Optimierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15535-0">
    <skos:altLabel xml:lang="de">Binäre Optimierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15054-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13993-1">
    <skos:altLabel xml:lang="de">Verwaltungspersonal</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13993-1">
    <skos:altLabel xml:lang="de">Verwaltungsfachleute</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19300-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#inScheme">
    <skos:definition xml:lang="en">A concept scheme in which the concept is included.</skos:definition>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10253-0">
    <skos:altLabel xml:lang="de">Makroökonometrisches Modell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13469-4">
    <skos:scopeNote xml:lang="de">Für internationale staatliche Vereinbarungen zur Zusammenarbeit im Außenhandel BENUTZE die entsprechenden Deskriptoren.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13469-4">
    <skos:scopeNote xml:lang="de">Sowohl für horizontale als auch für vertikale Kooperationsformen von Handelsunternehmen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18805-3">
    <skos:related rdf:resource="descriptor/18065-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18805-3">
    <skos:related rdf:resource="descriptor/16052-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18805-3">
    <skos:related rdf:resource="descriptor/19821-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10823-6">
    <skos:altLabel xml:lang="en">FDI (Foreign Direct Investment)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10823-6">
    <skos:altLabel xml:lang="en">Direct investment</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10823-6">
    <skos:altLabel xml:lang="de">Auslandsinvestition</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13757-6">
    <skos:broader rdf:resource="thsys/71103"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13757-6">
    <skos:broader rdf:resource="thsys/71088"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15897-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19699-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10988-2">
    <skos:altLabel xml:lang="en">Council of economic advisers</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10988-2">
    <skos:altLabel xml:lang="de">Wissenschaftliche wirtschaftspolitische Beratung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10988-2">
    <skos:altLabel xml:lang="de">Wirtschaftsrat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12931-0">
    <skos:narrower rdf:resource="descriptor/12979-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12931-0">
    <skos:narrower rdf:resource="descriptor/11629-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12931-0">
    <skos:narrower rdf:resource="descriptor/18345-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12931-0">
    <skos:narrower rdf:resource="descriptor/12975-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11845-5">
    <skos:related rdf:resource="descriptor/18502-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11845-5">
    <skos:related rdf:resource="descriptor/12932-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19248-4">
    <skos:broader rdf:resource="thsys/71110"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19248-4">
    <skos:broader rdf:resource="descriptor/11790-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19248-4">
    <skos:broader rdf:resource="descriptor/10012-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11449-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15983-1">
    <skos:related rdf:resource="descriptor/16357-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15983-1">
    <skos:related rdf:resource="descriptor/16354-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10562-1">
    <skos:scopeNote xml:lang="de">Empirische Literatur über räumliche Entwicklungs- bzw. Wachstumsschwerpunkte.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16136-2">
    <skos:prefLabel xml:lang="en">Industrial constitution law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16136-2">
    <skos:prefLabel xml:lang="de">Betriebsverfassung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11801-4">
    <skos:altLabel xml:lang="de">Landwirtschaftspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20948-1">
    <skos:prefLabel xml:lang="en">Intrapreneuring</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20948-1">
    <skos:prefLabel xml:lang="de">Intrapreneuring</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id383">
    <rdfs:seeAlso rdf:resource="descriptor/10148-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10246-4">
    <skos:prefLabel xml:lang="en">ISLM model</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10246-4">
    <skos:prefLabel xml:lang="de">IS-LM-Diagramm</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18955-5">
    <skos:related rdf:resource="descriptor/18954-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18955-5">
    <skos:related rdf:resource="descriptor/15675-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18955-5">
    <skos:related rdf:resource="descriptor/18921-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18955-5">
    <skos:related rdf:resource="descriptor/18917-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10185-5">
    <skos:broader rdf:resource="thsys/70481"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10185-5">
    <skos:broader rdf:resource="thsys/70233"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10185-5">
    <skos:broader rdf:resource="descriptor/10013-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13197-4">
    <skos:related rdf:resource="descriptor/18450-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13197-4">
    <skos:related rdf:resource="descriptor/13226-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13197-4">
    <skos:related rdf:resource="descriptor/15657-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13197-4">
    <skos:related rdf:resource="descriptor/16767-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13197-4">
    <skos:related rdf:resource="descriptor/13365-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13197-4">
    <skos:related rdf:resource="descriptor/13650-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11235-4">
    <skos:narrower rdf:resource="descriptor/11241-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11235-4">
    <skos:narrower rdf:resource="descriptor/11240-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11235-4">
    <skos:narrower rdf:resource="descriptor/19497-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11235-4">
    <skos:narrower rdf:resource="descriptor/11252-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11235-4">
    <skos:narrower rdf:resource="descriptor/11236-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11235-4">
    <skos:narrower rdf:resource="descriptor/11243-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10904-6">
    <skos:related rdf:resource="descriptor/10913-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10904-6">
    <skos:related rdf:resource="descriptor/10942-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10904-6">
    <skos:related rdf:resource="descriptor/10962-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10904-6">
    <skos:related rdf:resource="descriptor/10911-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10904-6">
    <skos:related rdf:resource="descriptor/10949-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10904-6">
    <skos:related rdf:resource="descriptor/10960-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10904-6">
    <skos:related rdf:resource="descriptor/10205-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10904-6">
    <skos:related rdf:resource="descriptor/10905-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10904-6">
    <skos:related rdf:resource="descriptor/10902-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10904-6">
    <skos:related rdf:resource="descriptor/10012-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13136-3">
    <skos:altLabel xml:lang="de">Ledergewerbe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18209-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18003-4">
    <skos:broader rdf:resource="thsys/73338"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18003-4">
    <skos:broader rdf:resource="thsys/71037"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18003-4">
    <skos:broader rdf:resource="descriptor/15416-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70041">
    <skos:prefLabel xml:lang="en">B.07  Marketing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70041">
    <skos:prefLabel xml:lang="de">B.07  Marketing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13782-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18832-0">
    <skos:altLabel xml:lang="en">Utility maximization</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18832-0">
    <skos:altLabel xml:lang="de">Nutzenmaximierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18832-0">
    <skos:altLabel xml:lang="de">Eigennutzaxiom</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18832-0">
    <skos:altLabel xml:lang="de">Eigennutz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18832-0">
    <skos:altLabel xml:lang="de">Egoismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71038">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">B.09.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16111-4">
    <skos:prefLabel xml:lang="en">Tax secrecy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16111-4">
    <skos:prefLabel xml:lang="de">Steuergeheimnis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12246-1">
    <zbwext:useInsteadNote rdf:nodeID="id59"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15971-1">
    <zbwext:useInsteadNote rdf:nodeID="id277"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15928-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20104-4">
    <skos:scopeNote xml:lang="de">Geldpolitische Regel zur Inflationsbekämpfung über eine an der Inflationsrate ausgerichtete Zinsanhebung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13160-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14976-3">
    <skos:prefLabel xml:lang="en">Beverage</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14976-3">
    <skos:prefLabel xml:lang="de">Getränk</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19316-6">
    <skos:related rdf:resource="descriptor/19564-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19316-6">
    <skos:related rdf:resource="descriptor/16485-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19316-6">
    <skos:related rdf:resource="descriptor/16078-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19316-6">
    <skos:related rdf:resource="descriptor/12800-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19316-6">
    <skos:related rdf:resource="descriptor/20536-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19316-6">
    <skos:related rdf:resource="descriptor/10880-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11680-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15806-2">
    <skos:prefLabel xml:lang="en">Social norm</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15806-2">
    <skos:prefLabel xml:lang="de">Soziale Norm</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71116">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/v">
    <skos:prefLabel xml:lang="en">V  Economics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/v">
    <skos:prefLabel xml:lang="de">V  Volkswirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18653-0">
    <skos:broader rdf:resource="thsys/73368"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18653-0">
    <skos:broader rdf:resource="thsys/70922"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18653-0">
    <skos:broader rdf:resource="descriptor/10981-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18653-0">
    <skos:broader rdf:resource="descriptor/10437-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19122-5">
    <skos:prefLabel xml:lang="en">Niche market</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19122-5">
    <skos:prefLabel xml:lang="de">Marktnische</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16032-0">
    <skos:related rdf:resource="descriptor/16026-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18278-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70041">
    <skos:narrower rdf:resource="descriptor/10895-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70041">
    <skos:narrower rdf:resource="descriptor/21041-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70041">
    <skos:narrower rdf:resource="descriptor/12775-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70041">
    <skos:narrower rdf:resource="descriptor/10202-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70041">
    <skos:narrower rdf:resource="descriptor/19123-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70041">
    <skos:narrower rdf:resource="descriptor/19122-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70041">
    <skos:narrower rdf:resource="descriptor/12774-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70041">
    <skos:narrower rdf:resource="descriptor/10928-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70041">
    <skos:narrower rdf:resource="descriptor/19667-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70041">
    <skos:narrower rdf:resource="descriptor/10902-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70041">
    <skos:narrower rdf:resource="descriptor/10012-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70041">
    <skos:narrower rdf:resource="descriptor/19121-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70041">
    <skos:narrower rdf:resource="descriptor/12770-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70041">
    <skos:narrower rdf:resource="descriptor/12768-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70041">
    <skos:narrower rdf:resource="descriptor/12766-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70041">
    <skos:narrower rdf:resource="descriptor/20536-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70041">
    <skos:narrower rdf:resource="descriptor/18395-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70041">
    <skos:narrower rdf:resource="descriptor/19119-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70041">
    <skos:narrower rdf:resource="descriptor/19118-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70041">
    <skos:narrower rdf:resource="descriptor/19835-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70041">
    <skos:narrower rdf:resource="descriptor/18100-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70041">
    <skos:narrower rdf:resource="descriptor/10876-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70041">
    <skos:narrower rdf:resource="descriptor/12529-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70041">
    <skos:narrower rdf:resource="thsys/70550"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70041">
    <skos:narrower rdf:resource="thsys/70040"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70041">
    <skos:narrower rdf:resource="thsys/70292"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70041">
    <skos:narrower rdf:resource="thsys/70384"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70041">
    <skos:narrower rdf:resource="thsys/70459"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11805-3">
    <skos:altLabel xml:lang="de">Landwirtschaftlicher Überschuss</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11805-3">
    <skos:altLabel xml:lang="de">Landwirtschaftlicher Überschuß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11805-3">
    <skos:altLabel xml:lang="de">Landwirtschaftliche Überproduktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11805-3">
    <skos:altLabel xml:lang="de">Agrarwirtschaftlicher Überschuß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11805-3">
    <skos:altLabel xml:lang="de">Agrarüberschuß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11805-3">
    <skos:altLabel xml:lang="de">Agrarüberproduktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11398-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20982-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12438-6">
    <skos:broader rdf:resource="thsys/70004"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14779-5">
    <skos:broader rdf:resource="thsys/73306"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14779-5">
    <skos:broader rdf:resource="descriptor/14776-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10789-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10222-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#narrowerTransitive">
    <rdfs:isDefinedBy rdf:resource="http://www.w3.org/2004/02/skos/core"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11341-5">
    <skos:related rdf:resource="descriptor/12501-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11341-5">
    <skos:related rdf:resource="descriptor/19011-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11341-5">
    <skos:related rdf:resource="descriptor/19005-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11341-5">
    <skos:related rdf:resource="descriptor/12592-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11341-5">
    <skos:related rdf:resource="descriptor/11769-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10051-5">
    <skos:narrower rdf:resource="descriptor/10043-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17979-3">
    <skos:broader rdf:resource="thsys/70307"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19831-4">
    <skos:related rdf:resource="descriptor/10058-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14285-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19197-2">
    <skos:prefLabel xml:lang="en">Income effect</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19197-2">
    <skos:prefLabel xml:lang="de">Einkommenseffekt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15905-0">
    <skos:narrower rdf:resource="descriptor/19638-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15104-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18816-5">
    <skos:related rdf:resource="descriptor/18182-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18816-5">
    <skos:related rdf:resource="descriptor/18967-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18816-5">
    <skos:related rdf:resource="descriptor/16437-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14793-4">
    <skos:altLabel xml:lang="de">Presskorkwaren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14793-4">
    <skos:altLabel xml:lang="de">Preßkorkwaren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14793-4">
    <skos:altLabel xml:lang="de">Kork</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14393-6">
    <skos:broader rdf:resource="thsys/70318"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14393-6">
    <skos:broader rdf:resource="descriptor/14381-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70355">
    <skos:prefLabel xml:lang="en">V.05.03  Inflation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70355">
    <skos:prefLabel xml:lang="de">V.05.03  Inflation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13593-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18898-5">
    <skos:prefLabel xml:lang="en">AIDS</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18898-5">
    <skos:prefLabel xml:lang="de">AIDS</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19150-0">
    <skos:altLabel xml:lang="en">Ordinal data</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19150-0">
    <skos:altLabel xml:lang="en">Nominal data</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19150-0">
    <skos:altLabel xml:lang="en">Loglinear model</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19150-0">
    <skos:altLabel xml:lang="en">Discrete data</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19150-0">
    <skos:altLabel xml:lang="en">Contingency table</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19150-0">
    <skos:altLabel xml:lang="en">Categorical data</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19150-0">
    <skos:altLabel xml:lang="de">Scoring-Verfahren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19150-0">
    <skos:altLabel xml:lang="de">Qualitative Methode</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19150-0">
    <skos:altLabel xml:lang="de">Qualitative Datenanalyse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19150-0">
    <skos:altLabel xml:lang="de">Qualitative Daten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19150-0">
    <skos:altLabel xml:lang="de">Qualitative Data</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19150-0">
    <skos:altLabel xml:lang="de">Ordinale Daten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19150-0">
    <skos:altLabel xml:lang="de">Nominale Merkmale</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19150-0">
    <skos:altLabel xml:lang="de">Nominale Daten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19150-0">
    <skos:altLabel xml:lang="de">Loglineares Modell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19150-0">
    <skos:altLabel xml:lang="de">Kontingenztafel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19150-0">
    <skos:altLabel xml:lang="de">Kontingenzkoeffizient</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19150-0">
    <skos:altLabel xml:lang="de">Kategoriale Regression</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19150-0">
    <skos:altLabel xml:lang="de">Kategoriale Daten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19150-0">
    <skos:altLabel xml:lang="de">Diskrete Daten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70147">
    <skos:narrower rdf:resource="descriptor/17567-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70147">
    <skos:narrower rdf:resource="descriptor/17570-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70147">
    <skos:narrower rdf:resource="descriptor/17571-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70147">
    <skos:narrower rdf:resource="descriptor/17569-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70147">
    <skos:narrower rdf:resource="descriptor/19166-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70147">
    <skos:narrower rdf:resource="descriptor/17568-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70147">
    <skos:narrower rdf:resource="descriptor/19165-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70147">
    <skos:narrower rdf:resource="descriptor/17565-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10958-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13577-1">
    <skos:narrower rdf:resource="descriptor/18585-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13577-1">
    <skos:narrower rdf:resource="descriptor/13574-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19526-2">
    <zbwext:useInsteadNote rdf:nodeID="id139"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15718-6">
    <skos:broader rdf:resource="thsys/73395"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10615-6">
    <skos:broader rdf:resource="thsys/71100"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10615-6">
    <skos:broader rdf:resource="descriptor/10314-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19543-2">
    <skos:altLabel xml:lang="de">Duale Berufsbildung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id4">
    <rdf:value xml:lang="de">Für Wahlen</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18604-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11921-1">
    <skos:broader rdf:resource="thsys/71070"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11921-1">
    <skos:broader rdf:resource="thsys/71066"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11921-1">
    <skos:broader rdf:resource="descriptor/11932-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70096">
    <skos:broader rdf:resource="thsys/n"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14531-6">
    <skos:related rdf:resource="descriptor/14465-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14531-6">
    <skos:related rdf:resource="descriptor/14042-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14531-6">
    <skos:related rdf:resource="descriptor/13078-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14531-6">
    <skos:related rdf:resource="descriptor/14329-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10144-5">
    <skos:broader rdf:resource="thsys/70180"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10144-5">
    <skos:broader rdf:resource="descriptor/10062-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10184-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20314-0">
    <skos:narrower rdf:resource="descriptor/19128-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19629-6">
    <skos:related rdf:resource="descriptor/14073-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19629-6">
    <skos:related rdf:resource="descriptor/19625-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12022-2">
    <skos:broader rdf:resource="thsys/71066"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12022-2">
    <skos:broader rdf:resource="descriptor/11754-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12022-2">
    <skos:broader rdf:resource="descriptor/11921-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14914-4">
    <skos:prefLabel xml:lang="en">Knitwear</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14914-4">
    <skos:prefLabel xml:lang="de">Maschenwaren</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id225">
    <rdf:value xml:lang="de">Für Produkte</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11646-4">
    <skos:altLabel xml:lang="en">Alcohol taxation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11646-4">
    <skos:altLabel xml:lang="de">Branntweinsteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11646-4">
    <skos:altLabel xml:lang="de">Alkoholbesteuerung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11231-5">
    <skos:broader rdf:resource="thsys/70051"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11231-5">
    <skos:broader rdf:resource="descriptor/11211-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10671-3">
    <skos:prefLabel xml:lang="en">Monetary model of exchange rates</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10671-3">
    <skos:prefLabel xml:lang="de">Monetäre Wechselkurstheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#notation">
    <rdf:type rdf:resource="http://www.w3.org/1999/02/22-rdf-syntax-ns#Property"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#notation">
    <rdf:type rdf:resource="http://www.w3.org/2002/07/owl#DatatypeProperty"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19124-1">
    <skos:altLabel xml:lang="de">Gewährleistung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19039-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12196-4">
    <skos:scopeNote xml:lang="de">Nur für Änderungen der Rechtsform von Unternehmen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16246-2">
    <skos:prefLabel xml:lang="en">Democracy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16246-2">
    <skos:prefLabel xml:lang="de">Demokratie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15669-0">
    <skos:altLabel xml:lang="de">Theoretische Physik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15669-0">
    <skos:altLabel xml:lang="de">Relativitätstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15669-0">
    <skos:altLabel xml:lang="de">Quantentheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15669-0">
    <skos:altLabel xml:lang="de">Angewandte Physik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10031-4">
    <skos:narrower rdf:resource="descriptor/18187-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10031-4">
    <skos:narrower rdf:resource="descriptor/11707-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10031-4">
    <skos:narrower rdf:resource="descriptor/10374-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10031-4">
    <skos:narrower rdf:resource="descriptor/18174-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10031-4">
    <skos:narrower rdf:resource="descriptor/18168-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70828">
    <skos:prefLabel xml:lang="en">W.28  Cultural industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70828">
    <skos:prefLabel xml:lang="de">W.28  Kultursektor</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10761-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10626-1">
    <skos:related rdf:resource="descriptor/10619-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10626-1">
    <skos:related rdf:resource="descriptor/16278-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10626-1">
    <skos:related rdf:resource="descriptor/18121-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10626-1">
    <skos:related rdf:resource="descriptor/16438-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10626-1">
    <skos:related rdf:resource="descriptor/19071-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12879-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11501-2">
    <skos:scopeNote xml:lang="de">Zusammenhang zwischen langfristiger Zins- und Preisniveauentwicklung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10706-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10122-1">
    <skos:related rdf:resource="descriptor/10120-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10122-1">
    <skos:related rdf:resource="descriptor/10304-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16332-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12326-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16353-1">
    <skos:related rdf:resource="descriptor/16354-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16353-1">
    <skos:related rdf:resource="descriptor/15861-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16353-1">
    <skos:related rdf:resource="descriptor/10975-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16353-1">
    <skos:related rdf:resource="descriptor/16287-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16353-1">
    <skos:related rdf:resource="descriptor/11167-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16353-1">
    <skos:related rdf:resource="descriptor/16405-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16353-1">
    <skos:related rdf:resource="descriptor/16633-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10274-6">
    <skos:prefLabel xml:lang="en">National wealth</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10274-6">
    <skos:prefLabel xml:lang="de">Vermögensrechnung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19725-3">
    <skos:altLabel xml:lang="de">Wehrwissenschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19725-3">
    <skos:altLabel xml:lang="de">Wehrforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19725-3">
    <skos:altLabel xml:lang="de">Theorie der Kriegsführung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19725-3">
    <skos:altLabel xml:lang="de">Theorie der Kriegführung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19725-3">
    <skos:altLabel xml:lang="de">Strategische Kriegsführung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19725-3">
    <skos:altLabel xml:lang="de">Strategische Kriegführung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19725-3">
    <skos:altLabel xml:lang="de">Militärische Forschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17161-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18426-4">
    <skos:narrower rdf:resource="descriptor/18661-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18426-4">
    <skos:narrower rdf:resource="descriptor/13682-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18426-4">
    <skos:narrower rdf:resource="descriptor/10455-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18426-4">
    <skos:narrower rdf:resource="descriptor/10462-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18426-4">
    <skos:narrower rdf:resource="descriptor/16757-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18426-4">
    <skos:narrower rdf:resource="descriptor/18359-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10685-6">
    <skos:altLabel xml:lang="en">Import quota</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10685-6">
    <skos:altLabel xml:lang="en">Export quota</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10685-6">
    <skos:altLabel xml:lang="de">Kontingentierung des Außenhandels</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10685-6">
    <skos:altLabel xml:lang="de">Importkontingent</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10685-6">
    <skos:altLabel xml:lang="de">Exportkontingent</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10685-6">
    <skos:altLabel xml:lang="de">Einfuhrkontingent</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10685-6">
    <skos:altLabel xml:lang="de">Ausfuhrkontingent</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10685-6">
    <skos:altLabel xml:lang="de">Außenwirtschaftskontingent</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19552-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70835">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14175-2">
    <skos:narrower rdf:resource="descriptor/18234-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14175-2">
    <skos:narrower rdf:resource="descriptor/14182-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19419-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18597-5">
    <skos:prefLabel xml:lang="en">Sea freight shipping</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18597-5">
    <skos:prefLabel xml:lang="de">Seefrachtverkehr</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14318-6">
    <skos:broader rdf:resource="thsys/70346"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14318-6">
    <skos:broader rdf:resource="descriptor/14269-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14446-4">
    <skos:related rdf:resource="descriptor/14586-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73378">
    <rdfs:label xml:lang="de">Politische Organisationen und Bewegungen</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71017">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18144-0">
    <skos:broader rdf:resource="thsys/70814"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18144-0">
    <skos:broader rdf:resource="thsys/70052"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18144-0">
    <skos:broader rdf:resource="descriptor/11180-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10021-0">
    <skos:narrower rdf:resource="descriptor/13743-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10021-0">
    <skos:narrower rdf:resource="descriptor/10781-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10021-0">
    <skos:narrower rdf:resource="descriptor/19196-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10021-0">
    <skos:narrower rdf:resource="descriptor/12023-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13218-1">
    <skos:prefLabel xml:lang="en">Food retail trade</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13218-1">
    <skos:prefLabel xml:lang="de">Lebensmitteleinzelhandel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16176-4">
    <skos:scopeNote xml:lang="de">Regelungen des öffentlichen und des Privatrechts.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10534-6">
    <skos:prefLabel xml:lang="en">Self-reliance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10534-6">
    <skos:prefLabel xml:lang="de">Autozentrierte Entwicklung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19752-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70406">
    <skos:prefLabel xml:lang="en">V.04.02  Economic growth</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70406">
    <skos:prefLabel xml:lang="de">V.04.02  Wirtschaftswachstum</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11653-0">
    <skos:broader rdf:resource="thsys/71074"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11653-0">
    <skos:broader rdf:resource="thsys/71064"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11653-0">
    <skos:broader rdf:resource="descriptor/11547-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11653-0">
    <skos:broader rdf:resource="descriptor/11544-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16579-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10334-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14740-4">
    <skos:broader rdf:resource="thsys/70074"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14740-4">
    <skos:broader rdf:resource="descriptor/14602-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10282-0">
    <skos:related rdf:resource="descriptor/12057-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10282-0">
    <skos:related rdf:resource="descriptor/11740-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10282-0">
    <skos:related rdf:resource="descriptor/10006-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17992-4">
    <skos:related rdf:resource="descriptor/17642-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70394">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13369-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13076-2">
    <skos:narrower rdf:resource="descriptor/13084-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13076-2">
    <skos:narrower rdf:resource="descriptor/18428-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13076-2">
    <skos:narrower rdf:resource="descriptor/13079-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13076-2">
    <skos:narrower rdf:resource="descriptor/18424-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13076-2">
    <skos:narrower rdf:resource="descriptor/13075-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18108-4">
    <skos:altLabel xml:lang="en">European economic interest group</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18108-4">
    <skos:altLabel xml:lang="de">Europäische wirtschaftliche Interessenvereinigung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#Collection">
    <rdf:type rdf:resource="http://www.w3.org/2002/07/owl#Class"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18968-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20503-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13904-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10623-0">
    <skos:related rdf:resource="descriptor/10617-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14766-0">
    <skos:prefLabel xml:lang="en">Fine ceramics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14766-0">
    <skos:prefLabel xml:lang="de">Feinkeramik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10804-3">
    <skos:broader rdf:resource="thsys/71105"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10804-3">
    <skos:broader rdf:resource="thsys/71089"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10804-3">
    <skos:broader rdf:resource="thsys/71045"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10804-3">
    <skos:broader rdf:resource="descriptor/12234-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70647">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15633-0">
    <skos:prefLabel xml:lang="en">Biology</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15633-0">
    <skos:prefLabel xml:lang="de">Biologie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19045-4">
    <skos:broader rdf:resource="thsys/71024"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19045-4">
    <skos:broader rdf:resource="descriptor/18313-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13988-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19687-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18983-0">
    <skos:broader rdf:resource="thsys/71028"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18983-0">
    <skos:broader rdf:resource="descriptor/18982-2"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id305">
    <rdf:value xml:lang="de">Für städtische Wirtschaftsentwicklung</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18295-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11544-5">
    <skos:related rdf:resource="descriptor/11543-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10348-3">
    <skos:prefLabel xml:lang="en">Crisis theory</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10348-3">
    <skos:prefLabel xml:lang="de">Krisentheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14047-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13436-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19057-4">
    <skos:broader rdf:resource="thsys/70883"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19057-4">
    <skos:broader rdf:resource="descriptor/12254-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15149-5">
    <skos:altLabel xml:lang="en">Variance</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15149-5">
    <skos:altLabel xml:lang="en">Standard deviation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15149-5">
    <skos:altLabel xml:lang="de">Zentrales Moment</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15149-5">
    <skos:altLabel xml:lang="de">Verteilungsparameter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15149-5">
    <skos:altLabel xml:lang="de">Verteilungsmaßzahl</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15149-5">
    <skos:altLabel xml:lang="de">Verhältniszahl</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15149-5">
    <skos:altLabel xml:lang="de">Variationskoeffizient</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15149-5">
    <skos:altLabel xml:lang="de">Varianz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15149-5">
    <skos:altLabel xml:lang="de">Streuungsparameter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15149-5">
    <skos:altLabel xml:lang="de">Streuungsmaß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15149-5">
    <skos:altLabel xml:lang="de">Stichprobenparameter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15149-5">
    <skos:altLabel xml:lang="de">Stichprobenmaßzahl</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15149-5">
    <skos:altLabel xml:lang="de">Standardfehler</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15149-5">
    <skos:altLabel xml:lang="de">Standardabweichung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15149-5">
    <skos:altLabel xml:lang="de">Mittelwert</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15149-5">
    <skos:altLabel xml:lang="de">Meßziffer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15149-5">
    <skos:altLabel xml:lang="de">Messziffer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15149-5">
    <skos:altLabel xml:lang="de">Median</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15149-5">
    <skos:altLabel xml:lang="de">Geometrisches Mittel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15149-5">
    <skos:altLabel xml:lang="de">Erwartungswert</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15149-5">
    <skos:altLabel xml:lang="de">Empirisches Moment</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15149-5">
    <skos:altLabel xml:lang="de">Arithmetisches Mittel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14622-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20810-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18518-6">
    <zbwext:useInsteadNote rdf:nodeID="id161"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13330-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17588-4">
    <skos:prefLabel xml:lang="en">Singapore</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17588-4">
    <skos:prefLabel xml:lang="de">Singapur</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14095-0">
    <skos:broader rdf:resource="thsys/70070"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14095-0">
    <skos:broader rdf:resource="descriptor/14088-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15712-4">
    <skos:scopeNote xml:lang="de">Aktuelle oder potenzielle Vorgänge (Erwartungen, Wahrnehmungen, Kommunikation etc.) zwischen zwei oder mehreren Individuen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11306-0">
    <skos:broader rdf:resource="thsys/71051"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11306-0">
    <skos:broader rdf:resource="thsys/70244"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11306-0">
    <skos:broader rdf:resource="thsys/70235"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11306-0">
    <skos:broader rdf:resource="descriptor/12201-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11306-0">
    <skos:broader rdf:resource="descriptor/12585-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11306-0">
    <skos:broader rdf:resource="descriptor/11310-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11306-0">
    <skos:broader rdf:resource="descriptor/11302-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10685-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12031-1">
    <skos:prefLabel xml:lang="en">Real estate market</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12031-1">
    <skos:prefLabel xml:lang="de">Immobilienmarkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17642-3">
    <skos:broader rdf:resource="thsys/70076"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13274-5">
    <skos:related rdf:resource="descriptor/13278-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13274-5">
    <skos:related rdf:resource="descriptor/18644-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13274-5">
    <skos:related rdf:resource="descriptor/18646-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18442-6">
    <skos:related rdf:resource="descriptor/18301-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18442-6">
    <skos:related rdf:resource="descriptor/13160-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18442-6">
    <skos:related rdf:resource="descriptor/14931-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18442-6">
    <skos:related rdf:resource="descriptor/18443-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18442-6">
    <skos:related rdf:resource="descriptor/14611-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18442-6">
    <skos:related rdf:resource="descriptor/13446-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18705-0">
    <skos:prefLabel xml:lang="en">Public bank</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18705-0">
    <skos:prefLabel xml:lang="de">Öffentliche Bank</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19383-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13357-1">
    <skos:altLabel xml:lang="de">Kunstmuseum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19516-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#historyNote">
    <rdfs:subPropertyOf rdf:resource="http://www.w3.org/2004/02/skos/core#note"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71051">
    <skos:broader rdf:resource="thsys/70113"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14527-4">
    <skos:related rdf:resource="descriptor/14042-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17676-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17680-2">
    <skos:broader rdf:resource="thsys/70429"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17680-2">
    <skos:broader rdf:resource="descriptor/17687-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12981-6">
    <skos:related rdf:resource="descriptor/14194-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10383-1">
    <skos:broader rdf:resource="thsys/70038"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10383-1">
    <skos:broader rdf:resource="thsys/70020"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10383-1">
    <skos:broader rdf:resource="descriptor/10387-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10383-1">
    <skos:broader rdf:resource="descriptor/10247-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13981-1">
    <skos:broader rdf:resource="thsys/71055"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13981-1">
    <skos:broader rdf:resource="thsys/70923"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13981-1">
    <skos:broader rdf:resource="descriptor/13979-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13077-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10652-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14074-1">
    <skos:altLabel xml:lang="de">Pflanzliches Produkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14074-1">
    <skos:altLabel xml:lang="de">Hackfrucht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14074-1">
    <skos:altLabel xml:lang="de">Feldfrucht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14074-1">
    <skos:altLabel xml:lang="de">Ackerbauprodukt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13824-3">
    <skos:broader rdf:resource="thsys/70892"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13824-3">
    <skos:broader rdf:resource="descriptor/10213-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11928-1">
    <skos:prefLabel xml:lang="en">Spatial economic theory</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11928-1">
    <skos:prefLabel xml:lang="de">Raumwirtschaftstheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10173-5">
    <skos:prefLabel xml:lang="en">Family economics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10173-5">
    <skos:prefLabel xml:lang="de">Familienökonomik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11638-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18018-5">
    <skos:related rdf:resource="descriptor/10298-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11432-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12829-5">
    <skos:broader rdf:resource="thsys/70384"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15493-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14106-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12142-6">
    <skos:related rdf:resource="descriptor/19733-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12142-6">
    <skos:related rdf:resource="descriptor/12111-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12142-6">
    <skos:related rdf:resource="descriptor/12085-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12142-6">
    <skos:related rdf:resource="descriptor/19672-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12142-6">
    <skos:related rdf:resource="descriptor/11158-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12142-6">
    <skos:related rdf:resource="descriptor/12136-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11355-1">
    <skos:scopeNote xml:lang="de">Personen, die einen Bildungs- oder Ausbildungsweg mit einem Abschluss beendet haben.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13423-0">
    <skos:broader rdf:resource="thsys/73360"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13423-0">
    <skos:broader rdf:resource="thsys/70862"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13423-0">
    <skos:broader rdf:resource="descriptor/13410-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16489-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13658-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13964-1">
    <skos:prefLabel xml:lang="en">Economic-technological engineer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13964-1">
    <skos:prefLabel xml:lang="de">Wirtschaftsingenieure</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18377-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13742-5">
    <skos:related rdf:resource="descriptor/13743-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13742-5">
    <skos:related rdf:resource="descriptor/16080-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13742-5">
    <skos:related rdf:resource="descriptor/13769-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13742-5">
    <skos:related rdf:resource="descriptor/18682-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13742-5">
    <skos:related rdf:resource="descriptor/12299-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13742-5">
    <skos:related rdf:resource="descriptor/19554-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13742-5">
    <skos:related rdf:resource="descriptor/19652-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13742-5">
    <skos:related rdf:resource="descriptor/11664-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13742-5">
    <skos:related rdf:resource="descriptor/13731-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13742-5">
    <skos:related rdf:resource="descriptor/19669-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13742-5">
    <skos:related rdf:resource="descriptor/13716-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13742-5">
    <skos:related rdf:resource="descriptor/13715-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13742-5">
    <skos:related rdf:resource="descriptor/18728-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13742-5">
    <skos:related rdf:resource="descriptor/19538-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11595-2">
    <skos:prefLabel xml:lang="en">Public Budgeting</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11595-2">
    <skos:prefLabel xml:lang="de">Öffentliche Finanzplanung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19386-6">
    <skos:altLabel xml:lang="de">Erdgaslagerstätte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18193-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11327-6">
    <skos:broader rdf:resource="thsys/70278"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70952">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.10.04</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12787-2">
    <skos:narrower rdf:resource="descriptor/19127-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12787-2">
    <skos:narrower rdf:resource="descriptor/10880-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10298-6">
    <skos:prefLabel xml:lang="en">Consumers</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10298-6">
    <skos:prefLabel xml:lang="de">Verbraucher</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16324-1">
    <skos:scopeNote xml:lang="de">Subnationale Regierungen in föderalen Systemen</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19485-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11268-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16708-4">
    <skos:related rdf:resource="descriptor/16732-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16708-4">
    <skos:related rdf:resource="descriptor/15685-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16708-4">
    <skos:related rdf:resource="descriptor/19253-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16708-4">
    <skos:related rdf:resource="descriptor/16716-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16708-4">
    <skos:related rdf:resource="descriptor/19601-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14513-1">
    <skos:related rdf:resource="descriptor/18448-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14513-1">
    <skos:related rdf:resource="descriptor/14752-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70829">
    <skos:broader rdf:resource="thsys/w"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12729-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70493">
    <skos:narrower rdf:resource="descriptor/20176-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70493">
    <skos:narrower rdf:resource="descriptor/10298-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70493">
    <skos:narrower rdf:resource="descriptor/19200-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70493">
    <skos:narrower rdf:resource="descriptor/19221-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70493">
    <skos:narrower rdf:resource="descriptor/13398-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70493">
    <skos:narrower rdf:resource="descriptor/10183-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70493">
    <skos:narrower rdf:resource="descriptor/10162-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70493">
    <skos:narrower rdf:resource="descriptor/10182-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70493">
    <skos:narrower rdf:resource="descriptor/10302-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70493">
    <skos:narrower rdf:resource="descriptor/10181-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70493">
    <skos:narrower rdf:resource="descriptor/10172-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70493">
    <skos:narrower rdf:resource="descriptor/10301-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70493">
    <skos:narrower rdf:resource="descriptor/10300-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70493">
    <skos:narrower rdf:resource="descriptor/10180-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70493">
    <skos:narrower rdf:resource="descriptor/10178-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70493">
    <skos:narrower rdf:resource="descriptor/10176-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70493">
    <skos:narrower rdf:resource="descriptor/10177-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70493">
    <skos:narrower rdf:resource="descriptor/10173-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70493">
    <skos:narrower rdf:resource="descriptor/10171-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70493">
    <skos:narrower rdf:resource="descriptor/19197-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70493">
    <skos:narrower rdf:resource="descriptor/10170-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70493">
    <skos:narrower rdf:resource="descriptor/19220-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70493">
    <skos:narrower rdf:resource="descriptor/10169-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19837-6">
    <skos:broader rdf:resource="thsys/70307"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id124">
    <rdfs:seeAlso rdf:resource="descriptor/10979-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13733-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71103">
    <rdfs:label xml:lang="en">Foreign exchange market</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71103">
    <rdfs:label xml:lang="de">Devisenmarkt</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10248-0">
    <zbwext:useInsteadNote rdf:nodeID="id177"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19023-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11915-3">
    <skos:broader rdf:resource="thsys/71067"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16712-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id258">
    <rdf:value xml:lang="de">Für die Analyse gesellschaftlicher Präferenzen</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14435-2">
    <skos:altLabel xml:lang="de">Kraftfahrzeugzubehör</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14435-2">
    <skos:altLabel xml:lang="de">Kfz-Zubehör</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14435-2">
    <skos:altLabel xml:lang="de">Kfz-Teil</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14435-2">
    <skos:altLabel xml:lang="de">Karosserie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14435-2">
    <skos:altLabel xml:lang="de">Autozubehör</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73332">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.05.08.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id13">
    <rdfs:seeAlso rdf:resource="descriptor/13474-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10001-6">
    <skos:altLabel xml:lang="en">Financial arbitrage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10001-6">
    <skos:altLabel xml:lang="de">Börsenarbitrage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19659-4">
    <skos:altLabel xml:lang="en">Multiculturalism</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19659-4">
    <skos:altLabel xml:lang="en">Intercultural</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19659-4">
    <skos:altLabel xml:lang="de">Multikulturalität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19659-4">
    <skos:altLabel xml:lang="de">Multikulturalismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19659-4">
    <skos:altLabel xml:lang="de">Multikultural</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19659-4">
    <skos:altLabel xml:lang="de">Interkulturell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18393-0">
    <skos:altLabel xml:lang="de">Mineralölfördertechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18393-0">
    <skos:altLabel xml:lang="de">Fördertechnik zur Erdölgewinnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18393-0">
    <skos:altLabel xml:lang="de">Fördertechnik zur Erdgasgewinnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18393-0">
    <skos:altLabel xml:lang="de">Erdölgewinnungsanlage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18393-0">
    <skos:altLabel xml:lang="de">Erdölbohrtechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18393-0">
    <skos:altLabel xml:lang="de">Erdgasfördertechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18393-0">
    <skos:altLabel xml:lang="de">Bohrtechnik zur Erdölgewinnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18393-0">
    <skos:altLabel xml:lang="de">Bohranlage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19148-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14199-2">
    <skos:broader rdf:resource="thsys/70418"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14199-2">
    <skos:broader rdf:resource="descriptor/14236-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14346-1">
    <skos:scopeNote xml:lang="de">Produktion und Aufstellung von Waren- und Glücksspielautomaten sowie der einschlägige Warenabsatz</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16722-3">
    <skos:scopeNote xml:lang="de">Philosophische Methode zur rationalen Begründung von Glaubenswahrheiten.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16269-4">
    <skos:scopeNote xml:lang="de">Informationen der Regierung bzw. staatlicher Stellen, die für die Öffentlichkeit bestimmt sind</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73380">
    <skos:broader rdf:resource="thsys/73382"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14047-4">
    <skos:broader rdf:resource="thsys/70811"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14047-4">
    <skos:broader rdf:resource="thsys/70201"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16365-1">
    <skos:related rdf:resource="descriptor/19699-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16365-1">
    <skos:related rdf:resource="descriptor/18097-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16365-1">
    <skos:related rdf:resource="descriptor/16366-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16365-1">
    <skos:related rdf:resource="descriptor/16008-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24622-4">
    <skos:prefLabel xml:lang="en">Incentive contract</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24622-4">
    <skos:prefLabel xml:lang="de">Anreizvertrag</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19189-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18076-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25822-5">
    <skos:broader rdf:resource="thsys/70485"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10413-4">
    <skos:prefLabel xml:lang="en">Stagnation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10413-4">
    <skos:prefLabel xml:lang="de">Stagnation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19656-3">
    <skos:broader rdf:resource="thsys/70843"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19656-3">
    <skos:broader rdf:resource="thsys/70244"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19656-3">
    <skos:broader rdf:resource="descriptor/11360-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19472-6">
    <zbwext:useInsteadNote rdf:nodeID="id309"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73331">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12277-4">
    <skos:broader rdf:resource="thsys/73392"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12277-4">
    <skos:broader rdf:resource="thsys/71045"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12277-4">
    <skos:broader rdf:resource="thsys/71043"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10687-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24857-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13045-6">
    <skos:related rdf:resource="descriptor/13060-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13045-6">
    <skos:related rdf:resource="descriptor/18409-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13045-6">
    <skos:related rdf:resource="descriptor/12905-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13045-6">
    <skos:related rdf:resource="descriptor/18495-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13045-6">
    <skos:related rdf:resource="descriptor/14366-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70538">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">P.04</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14730-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71097">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73333">
    <rdfs:label xml:lang="en">International law</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73333">
    <rdfs:label xml:lang="de">Internationales Recht</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14736-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10475-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11216-1">
    <skos:related rdf:resource="descriptor/10261-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17602-1">
    <skos:altLabel xml:lang="en">Republic of India</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17602-1">
    <skos:altLabel xml:lang="en">Indian Union</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17602-1">
    <skos:altLabel xml:lang="de">Republik Indien</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17669-4">
    <skos:altLabel xml:lang="de">Volksrepublik Mosambik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17669-4">
    <skos:altLabel xml:lang="de">Portugiesisch-Ostafrika</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17669-4">
    <skos:altLabel xml:lang="de">Mocambique</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10593-4">
    <skos:broader rdf:resource="thsys/73332"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10593-4">
    <skos:broader rdf:resource="thsys/71084"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10593-4">
    <skos:broader rdf:resource="thsys/71010"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10593-4">
    <skos:broader rdf:resource="thsys/70811"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10593-4">
    <skos:broader rdf:resource="descriptor/15752-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10593-4">
    <skos:broader rdf:resource="descriptor/10698-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14502-6">
    <skos:altLabel xml:lang="de">Elektrogroßgerät</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14502-6">
    <skos:altLabel xml:lang="de">Einbauküchengerät</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15979-6">
    <skos:broader rdf:resource="thsys/73373"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15979-6">
    <skos:broader rdf:resource="thsys/73334"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15979-6">
    <skos:broader rdf:resource="thsys/70862"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15979-6">
    <skos:broader rdf:resource="descriptor/19841-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13237-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18523-6">
    <skos:related rdf:resource="descriptor/19396-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18523-6">
    <skos:related rdf:resource="descriptor/13072-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18523-6">
    <skos:related rdf:resource="descriptor/14751-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12837-6">
    <skos:related rdf:resource="descriptor/15859-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12837-6">
    <skos:related rdf:resource="descriptor/12846-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12837-6">
    <skos:related rdf:resource="descriptor/12829-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12837-6">
    <skos:related rdf:resource="descriptor/15858-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12837-6">
    <skos:related rdf:resource="descriptor/12844-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12837-6">
    <skos:related rdf:resource="descriptor/15793-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12837-6">
    <skos:related rdf:resource="descriptor/15801-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12837-6">
    <skos:related rdf:resource="descriptor/18932-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12837-6">
    <skos:related rdf:resource="descriptor/18930-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12837-6">
    <skos:related rdf:resource="descriptor/12774-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12837-6">
    <skos:related rdf:resource="descriptor/16265-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12837-6">
    <skos:related rdf:resource="descriptor/10300-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12837-6">
    <skos:related rdf:resource="descriptor/19599-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10644-6">
    <skos:broader rdf:resource="thsys/71100"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10644-6">
    <skos:broader rdf:resource="thsys/71096"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10644-6">
    <skos:broader rdf:resource="descriptor/10649-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10415-0">
    <skos:prefLabel xml:lang="en">Export-led growth</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10415-0">
    <skos:prefLabel xml:lang="de">Exportinduziertes Wachstum</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12311-2">
    <skos:related rdf:resource="descriptor/12318-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12311-2">
    <skos:related rdf:resource="descriptor/10566-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12311-2">
    <skos:related rdf:resource="descriptor/12309-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12311-2">
    <skos:related rdf:resource="descriptor/12306-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70415">
    <skos:broader rdf:resource="thsys/70257"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15125-5">
    <skos:altLabel xml:lang="de">Panel-Daten-Analyse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id181">
    <rdf:value xml:lang="de">für Bemühungen zum Schutz und zur Pflege des historischen Erbes</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12218-6">
    <skos:related rdf:resource="descriptor/12230-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12218-6">
    <skos:related rdf:resource="descriptor/12226-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12218-6">
    <skos:related rdf:resource="descriptor/19418-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12218-6">
    <skos:related rdf:resource="descriptor/12216-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12218-6">
    <skos:related rdf:resource="descriptor/12228-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18492-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11616-6">
    <skos:related rdf:resource="descriptor/11044-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11341-5">
    <skos:prefLabel xml:lang="en">Payment scheme</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11341-5">
    <skos:prefLabel xml:lang="de">Vergütungssystem</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16122-6">
    <skos:related rdf:resource="descriptor/10713-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16122-6">
    <skos:related rdf:resource="descriptor/10710-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16122-6">
    <skos:related rdf:resource="descriptor/11640-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14004-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19184-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19830-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11571-2">
    <skos:related rdf:resource="descriptor/11777-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11571-2">
    <skos:related rdf:resource="descriptor/13790-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11571-2">
    <skos:related rdf:resource="descriptor/10020-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11571-2">
    <skos:related rdf:resource="descriptor/11321-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12203-5">
    <skos:altLabel xml:lang="de">Simultane Investitionsplanung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12203-5">
    <skos:altLabel xml:lang="de">Simultane Investitions- und Produktionsplanung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12203-5">
    <skos:altLabel xml:lang="de">Simultane Investitions- und Finanzplanung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12203-5">
    <skos:altLabel xml:lang="de">Kapitalwertmethode</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12203-5">
    <skos:altLabel xml:lang="de">Interne Zinsfußmethode</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12203-5">
    <skos:altLabel xml:lang="de">Integrationsmodell der Investitionsrechnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12203-5">
    <skos:altLabel xml:lang="de">Annuitätenmethode der Investitionsrechnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18800-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14204-0">
    <skos:prefLabel xml:lang="en">Non-ferrous metal ore</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14204-0">
    <skos:prefLabel xml:lang="de">NE-Metallerz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10204-6">
    <skos:prefLabel xml:lang="en">Mark-up pricing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10204-6">
    <skos:prefLabel xml:lang="de">Mark-up Pricing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14980-5">
    <skos:prefLabel xml:lang="en">Fruit wine</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14980-5">
    <skos:prefLabel xml:lang="de">Obstwein</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70058">
    <skos:prefLabel xml:lang="de">V.11  Regionalwirtschaft und Infrastruktur</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13365-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18569-3">
    <skos:prefLabel xml:lang="en">Railway workers</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18569-3">
    <skos:prefLabel xml:lang="de">Eisenbahnberufe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12946-1">
    <skos:broader rdf:resource="thsys/71025"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12946-1">
    <skos:broader rdf:resource="descriptor/12955-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11392-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18698-6">
    <skos:altLabel xml:lang="de">Kreditregulierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18698-6">
    <skos:altLabel xml:lang="de">Kreditpolitik des Staates</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18698-6">
    <skos:altLabel xml:lang="de">Kreditkontrolle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18698-6">
    <skos:altLabel xml:lang="de">Kreditaufsicht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21014-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15470-2">
    <skos:narrower rdf:resource="descriptor/15476-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15470-2">
    <skos:narrower rdf:resource="descriptor/15475-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15470-2">
    <skos:narrower rdf:resource="descriptor/19147-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15470-2">
    <skos:narrower rdf:resource="descriptor/15468-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15470-2">
    <skos:narrower rdf:resource="descriptor/15467-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14090-3">
    <skos:prefLabel xml:lang="en">Oats</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14090-3">
    <skos:prefLabel xml:lang="de">Hafer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73299">
    <rdfs:label xml:lang="en">Feed</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73299">
    <rdfs:label xml:lang="de">Futtermittel</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19854-6">
    <skos:broader rdf:resource="thsys/73360"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19854-6">
    <skos:broader rdf:resource="descriptor/16019-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13618-6">
    <skos:broader rdf:resource="thsys/70940"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13618-6">
    <skos:broader rdf:resource="thsys/70925"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13618-6">
    <skos:broader rdf:resource="descriptor/13621-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13618-6">
    <skos:broader rdf:resource="descriptor/13499-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13670-4">
    <skos:related rdf:resource="descriptor/12847-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13670-4">
    <skos:related rdf:resource="descriptor/13369-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13670-4">
    <skos:related rdf:resource="descriptor/13124-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13229-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13621-3">
    <skos:narrower rdf:resource="descriptor/13244-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13621-3">
    <skos:narrower rdf:resource="descriptor/13518-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13621-3">
    <skos:narrower rdf:resource="descriptor/13245-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13621-3">
    <skos:narrower rdf:resource="descriptor/18618-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13621-3">
    <skos:narrower rdf:resource="descriptor/13622-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13621-3">
    <skos:narrower rdf:resource="descriptor/13620-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13621-3">
    <skos:narrower rdf:resource="descriptor/13618-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13621-3">
    <skos:narrower rdf:resource="descriptor/18614-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13621-3">
    <skos:narrower rdf:resource="descriptor/13617-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13621-3">
    <skos:narrower rdf:resource="descriptor/13553-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13621-3">
    <skos:narrower rdf:resource="descriptor/18573-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17988-2">
    <skos:altLabel xml:lang="en">OIC Countries</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17988-2">
    <skos:altLabel xml:lang="de">Islamische Länder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12094-5">
    <skos:broader rdf:resource="thsys/70562"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12094-5">
    <skos:broader rdf:resource="descriptor/12105-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19576-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id356">
    <rdf:value xml:lang="de">für Säbel, Degen und andere blanke Waffen</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11996-5">
    <skos:related rdf:resource="descriptor/11976-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11996-5">
    <skos:related rdf:resource="descriptor/11991-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11996-5">
    <skos:related rdf:resource="descriptor/11971-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19037-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/n">
    <skos:narrower rdf:resource="thsys/70007"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/n">
    <skos:narrower rdf:resource="thsys/70154"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/n">
    <skos:narrower rdf:resource="thsys/70060"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/n">
    <skos:narrower rdf:resource="thsys/70269"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/n">
    <skos:narrower rdf:resource="thsys/70192"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/n">
    <skos:narrower rdf:resource="thsys/70012"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/n">
    <skos:narrower rdf:resource="thsys/70142"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/n">
    <skos:narrower rdf:resource="thsys/70095"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/n">
    <skos:narrower rdf:resource="thsys/70096"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/n">
    <skos:narrower rdf:resource="thsys/70151"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/n">
    <skos:narrower rdf:resource="thsys/70017"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14012-2">
    <skos:broader rdf:resource="thsys/73388"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14012-2">
    <skos:broader rdf:resource="thsys/71052"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14012-2">
    <skos:broader rdf:resource="thsys/70850"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14012-2">
    <skos:broader rdf:resource="descriptor/18535-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71073">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18136-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16564-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11433-0">
    <skos:broader rdf:resource="thsys/70259"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11433-0">
    <skos:broader rdf:resource="thsys/70187"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11433-0">
    <skos:broader rdf:resource="descriptor/13723-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10833-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19424-3">
    <skos:altLabel xml:lang="de">Versicherungssteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19424-3">
    <skos:altLabel xml:lang="de">Versicherungsbesteuerung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13321-1">
    <skos:prefLabel xml:lang="en">Dry cleaning</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13321-1">
    <skos:prefLabel xml:lang="de">Textilreinigung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16212-5">
    <skos:related rdf:resource="descriptor/16019-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16212-5">
    <skos:related rdf:resource="descriptor/16209-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16212-5">
    <skos:related rdf:resource="descriptor/13413-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16212-5">
    <skos:related rdf:resource="descriptor/16205-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16212-5">
    <skos:related rdf:resource="descriptor/11526-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16212-5">
    <skos:related rdf:resource="descriptor/11038-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11862-5">
    <skos:related rdf:resource="descriptor/10503-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11862-5">
    <skos:related rdf:resource="descriptor/19457-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11862-5">
    <skos:related rdf:resource="descriptor/19459-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11862-5">
    <skos:related rdf:resource="descriptor/19458-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13246-3">
    <skos:narrower rdf:resource="descriptor/18608-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13246-3">
    <skos:narrower rdf:resource="descriptor/18611-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13246-3">
    <skos:narrower rdf:resource="descriptor/13518-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13246-3">
    <skos:narrower rdf:resource="descriptor/18606-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13511-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19819-1">
    <skos:altLabel xml:lang="en">Legal custom</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15966-1">
    <skos:prefLabel xml:lang="en">Law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15966-1">
    <skos:prefLabel xml:lang="de">Recht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#related">
    <rdf:type rdf:resource="http://www.w3.org/1999/02/22-rdf-syntax-ns#Property"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#related">
    <rdf:type rdf:resource="http://www.w3.org/2002/07/owl#SymmetricProperty"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#related">
    <rdf:type rdf:resource="http://www.w3.org/2002/07/owl#ObjectProperty"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18142-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18934-6">
    <skos:broader rdf:resource="thsys/70824"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18934-6">
    <skos:broader rdf:resource="descriptor/18027-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15888-2">
    <skos:altLabel xml:lang="de">Wetterkunde</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15888-2">
    <skos:altLabel xml:lang="de">Wetterforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15888-2">
    <skos:altLabel xml:lang="de">Wetterdienst</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18181-1">
    <skos:altLabel xml:lang="de">Regionalismus im Handel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18181-1">
    <skos:altLabel xml:lang="de">Regionale Wirtschaftsintegration</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18181-1">
    <skos:altLabel xml:lang="de">Handelsintegration</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16364-3">
    <skos:broader rdf:resource="thsys/73378"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16364-3">
    <skos:broader rdf:resource="descriptor/13397-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13469-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19512-6">
    <skos:prefLabel xml:lang="en">Public sector retirement system</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19512-6">
    <skos:prefLabel xml:lang="de">Ruhegehalt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11917-6">
    <skos:prefLabel xml:lang="en">Backward region</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11917-6">
    <skos:prefLabel xml:lang="de">Strukturschwacher Raum</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15397-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14004-1">
    <skos:related rdf:resource="descriptor/13347-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14004-1">
    <skos:related rdf:resource="descriptor/15657-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14004-1">
    <skos:related rdf:resource="descriptor/13355-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14004-1">
    <skos:related rdf:resource="descriptor/13658-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13376-4">
    <skos:prefLabel xml:lang="en">Architecture firm</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13376-4">
    <skos:prefLabel xml:lang="de">Architekturbüro</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19431-6">
    <skos:broader rdf:resource="thsys/70242"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19431-6">
    <skos:broader rdf:resource="descriptor/10092-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15859-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10503-3">
    <skos:prefLabel xml:lang="en">Rural area</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10503-3">
    <skos:prefLabel xml:lang="de">Ländlicher Raum</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11705-0">
    <skos:prefLabel xml:lang="en">Tax progressivity</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11705-0">
    <skos:prefLabel xml:lang="de">Steuerprogression</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17948-0">
    <skos:related rdf:resource="descriptor/17935-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11266-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11174-5">
    <skos:broader rdf:resource="thsys/70141"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11174-5">
    <skos:broader rdf:resource="descriptor/16745-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15624-1">
    <skos:related rdf:resource="descriptor/15621-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15624-1">
    <skos:related rdf:resource="descriptor/18766-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15624-1">
    <skos:related rdf:resource="descriptor/15415-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15624-1">
    <skos:related rdf:resource="descriptor/16757-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12256-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14438-3">
    <skos:related rdf:resource="descriptor/14328-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14438-3">
    <skos:related rdf:resource="descriptor/13060-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14438-3">
    <skos:related rdf:resource="descriptor/14366-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12191-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13470-5">
    <skos:related rdf:resource="descriptor/13210-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13470-5">
    <skos:related rdf:resource="descriptor/13478-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15149-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19610-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71002">
    <skos:narrower rdf:resource="descriptor/13067-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71002">
    <skos:narrower rdf:resource="descriptor/18415-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71002">
    <skos:narrower rdf:resource="descriptor/18095-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71002">
    <skos:narrower rdf:resource="descriptor/18414-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71002">
    <skos:narrower rdf:resource="descriptor/18413-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71002">
    <skos:narrower rdf:resource="descriptor/19186-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71002">
    <skos:narrower rdf:resource="descriptor/18412-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71002">
    <skos:narrower rdf:resource="descriptor/13058-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71002">
    <skos:narrower rdf:resource="descriptor/13054-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71002">
    <skos:narrower rdf:resource="descriptor/18411-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71002">
    <skos:narrower rdf:resource="descriptor/18410-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71002">
    <skos:narrower rdf:resource="descriptor/18409-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71002">
    <skos:narrower rdf:resource="descriptor/18408-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71002">
    <skos:narrower rdf:resource="descriptor/18407-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71002">
    <skos:narrower rdf:resource="descriptor/13045-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71002">
    <skos:narrower rdf:resource="descriptor/12914-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13462-4">
    <skos:prefLabel xml:lang="en">History of trade</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13462-4">
    <skos:prefLabel xml:lang="de">Handelsgeschichte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14327-5">
    <skos:related rdf:resource="descriptor/14435-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14327-5">
    <skos:related rdf:resource="descriptor/15728-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14042-0">
    <skos:altLabel xml:lang="de">Watte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14042-0">
    <skos:altLabel xml:lang="de">Verbandszeug</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14042-0">
    <skos:altLabel xml:lang="de">Verbandmittel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14042-0">
    <skos:altLabel xml:lang="de">Orthopädisches Produkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14042-0">
    <skos:altLabel xml:lang="de">Medizinprodukt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14042-0">
    <skos:altLabel xml:lang="de">Medicalprodukt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10966-5">
    <skos:broader rdf:resource="thsys/73378"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10966-5">
    <skos:broader rdf:resource="thsys/70852"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10966-5">
    <skos:broader rdf:resource="thsys/70038"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10966-5">
    <skos:broader rdf:resource="descriptor/10984-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10523-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19040-0">
    <skos:altLabel xml:lang="de">Zusammenarbeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19341-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15774-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70233">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15251-0">
    <skos:scopeNote xml:lang="de">Fehler einer Schätzung, der nicht auf die Zufallsauswahl der Stichprobe zurückzuführen ist.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13861-4">
    <skos:prefLabel xml:lang="en">Sociology of housing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13861-4">
    <skos:prefLabel xml:lang="de">Soziologie des Wohnens</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13137-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10137-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18640-2">
    <skos:scopeNote xml:lang="de">Analyse umwelt- und ressourcenbezogener Wirkungen des Tourismus.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19637-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19296-0">
    <skos:prefLabel xml:lang="en">Cocoa market</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19296-0">
    <skos:prefLabel xml:lang="de">Kakaomarkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10495-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11294-2">
    <skos:related rdf:resource="descriptor/11299-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11294-2">
    <skos:related rdf:resource="descriptor/19005-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11294-2">
    <skos:related rdf:resource="descriptor/11297-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11294-2">
    <skos:related rdf:resource="descriptor/11184-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10361-4">
    <skos:related rdf:resource="descriptor/10347-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10361-4">
    <skos:related rdf:resource="descriptor/10338-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14734-6">
    <skos:prefLabel xml:lang="en">Cosmetics and toiletries</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14734-6">
    <skos:prefLabel xml:lang="de">Körperpflegemittel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16672-6">
    <skos:broader rdf:resource="thsys/73356"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16672-6">
    <skos:broader rdf:resource="thsys/70958"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18551-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18491-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12705-2">
    <skos:prefLabel xml:lang="en">Production plant</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12705-2">
    <skos:prefLabel xml:lang="de">Produktionsanlage</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15125-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12760-1">
    <skos:related rdf:resource="descriptor/12747-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16708-4">
    <skos:altLabel xml:lang="en">Theory of cognition</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16708-4">
    <skos:altLabel xml:lang="de">Wissenslehre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16708-4">
    <skos:altLabel xml:lang="de">Erkenntnislehre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16708-4">
    <skos:altLabel xml:lang="de">Erkenntniskritik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16708-4">
    <skos:altLabel xml:lang="de">Epistemologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19625-0">
    <skos:prefLabel xml:lang="en">Biodiversity</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19625-0">
    <skos:prefLabel xml:lang="de">Biodiversität</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11197-0">
    <skos:narrower rdf:resource="descriptor/18112-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11598-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19457-2">
    <skos:related rdf:resource="descriptor/11988-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19457-2">
    <skos:related rdf:resource="descriptor/11862-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10807-4">
    <skos:broader rdf:resource="thsys/71089"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10807-4">
    <skos:broader rdf:resource="descriptor/13723-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13523-3">
    <skos:related rdf:resource="descriptor/13255-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13523-3">
    <skos:related rdf:resource="descriptor/13527-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71034">
    <skos:broader rdf:resource="thsys/70028"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14465-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10487-3">
    <skos:altLabel xml:lang="en">Intermediate technology</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10487-3">
    <skos:altLabel xml:lang="de">Sanfte Technologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10487-3">
    <skos:altLabel xml:lang="de">Mittlere Technologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10487-3">
    <skos:altLabel xml:lang="de">Angepaßte Technologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10487-3">
    <skos:altLabel xml:lang="de">Angepaßte Technik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10487-3">
    <skos:altLabel xml:lang="de">Angepasste Technik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10487-3">
    <skos:altLabel xml:lang="de">Alternative Technologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10487-3">
    <skos:altLabel xml:lang="de">Alternative Technik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15467-5">
    <skos:broader rdf:resource="thsys/73103"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15467-5">
    <skos:broader rdf:resource="descriptor/15060-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15467-5">
    <skos:broader rdf:resource="descriptor/15470-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15467-5">
    <skos:broader rdf:resource="descriptor/15466-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71071">
    <rdfs:label xml:lang="en">Competition policy</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71071">
    <rdfs:label xml:lang="de">Wettbewerbspolitik</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12602-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18249-0">
    <skos:broader rdf:resource="thsys/70135"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18249-0">
    <skos:broader rdf:resource="descriptor/14490-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18249-0">
    <skos:broader rdf:resource="descriptor/14750-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11995-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13287-3">
    <skos:related rdf:resource="descriptor/13702-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13287-3">
    <skos:related rdf:resource="descriptor/18673-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13287-3">
    <skos:related rdf:resource="descriptor/13429-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12041-5">
    <skos:prefLabel xml:lang="en">Business economics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12041-5">
    <skos:prefLabel xml:lang="de">Betriebswirtschaftslehre</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73354">
    <skos:narrower rdf:resource="descriptor/15902-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73354">
    <skos:narrower rdf:resource="descriptor/15900-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73354">
    <skos:narrower rdf:resource="descriptor/13594-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73354">
    <skos:narrower rdf:resource="descriptor/11987-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73354">
    <skos:narrower rdf:resource="descriptor/19611-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73354">
    <skos:narrower rdf:resource="descriptor/11857-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73354">
    <skos:narrower rdf:resource="descriptor/19614-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73354">
    <skos:narrower rdf:resource="descriptor/15891-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73354">
    <skos:narrower rdf:resource="descriptor/19613-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73354">
    <skos:narrower rdf:resource="descriptor/14814-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73354">
    <skos:narrower rdf:resource="descriptor/19612-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73354">
    <skos:narrower rdf:resource="descriptor/16215-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73354">
    <skos:narrower rdf:resource="descriptor/11957-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73354">
    <skos:narrower rdf:resource="descriptor/11906-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73354">
    <skos:narrower rdf:resource="descriptor/15876-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73354">
    <skos:narrower rdf:resource="descriptor/15874-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73354">
    <skos:narrower rdf:resource="descriptor/15873-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73354">
    <skos:narrower rdf:resource="descriptor/15872-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73354">
    <skos:narrower rdf:resource="descriptor/13592-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73354">
    <skos:narrower rdf:resource="descriptor/13443-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73354">
    <skos:narrower rdf:resource="thsys/73353"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73354">
    <skos:narrower rdf:resource="thsys/73351"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73354">
    <skos:narrower rdf:resource="thsys/73352"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70816">
    <skos:narrower rdf:resource="descriptor/18965-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70816">
    <skos:narrower rdf:resource="descriptor/13190-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70816">
    <skos:narrower rdf:resource="descriptor/18964-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70816">
    <skos:narrower rdf:resource="descriptor/13324-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70816">
    <skos:narrower rdf:resource="descriptor/18963-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70816">
    <skos:narrower rdf:resource="descriptor/18962-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70816">
    <skos:narrower rdf:resource="descriptor/13191-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11241-2">
    <skos:altLabel xml:lang="de">Lohnkostenzuschuss</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11241-2">
    <skos:altLabel xml:lang="de">Lohnkostenzuschuß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11241-2">
    <skos:altLabel xml:lang="de">Kombilohn</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16028-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12647-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16510-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id154">
    <rdf:value xml:lang="de">Für Auslandshilfe im Sinne von Entwicklungshilfe</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17806-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12889-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16805-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11360-1">
    <skos:broader rdf:resource="thsys/70843"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11360-1">
    <skos:broader rdf:resource="thsys/70176"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11360-1">
    <skos:broader rdf:resource="descriptor/11367-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18467-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17638-1">
    <skos:prefLabel xml:lang="en">Turkey</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17638-1">
    <skos:prefLabel xml:lang="de">Türkei</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15823-2">
    <skos:broader rdf:resource="thsys/73385"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15823-2">
    <skos:broader rdf:resource="descriptor/15787-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73103">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14285-2">
    <skos:related rdf:resource="descriptor/19051-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14285-2">
    <skos:related rdf:resource="descriptor/19390-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14285-2">
    <skos:related rdf:resource="descriptor/13029-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14285-2">
    <skos:related rdf:resource="descriptor/13236-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14285-2">
    <skos:related rdf:resource="descriptor/14274-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14285-2">
    <skos:related rdf:resource="descriptor/14195-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14788-4">
    <skos:altLabel xml:lang="de">Tischlerwaren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14788-4">
    <skos:altLabel xml:lang="de">Sarg</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14788-4">
    <skos:altLabel xml:lang="de">Holzwolle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14788-4">
    <skos:altLabel xml:lang="de">Holzwerkstoff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14788-4">
    <skos:altLabel xml:lang="de">Holzwaren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14788-4">
    <skos:altLabel xml:lang="de">Holzmehl</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24708-1">
    <skos:narrower rdf:resource="descriptor/18649-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24708-1">
    <skos:narrower rdf:resource="descriptor/18428-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24708-1">
    <skos:narrower rdf:resource="descriptor/18762-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18951-6">
    <skos:prefLabel xml:lang="en">Book inventories</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18951-6">
    <skos:prefLabel xml:lang="de">Buchbestand</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10173-5">
    <skos:scopeNote xml:lang="de">Ökonomische Theorie über das rationale Verhalten von Familien, z.B. über Heirat, Scheidung, Kinderzahl, familiäre Arbeitsteilung usw.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10750-0">
    <skos:altLabel xml:lang="de">Währungskonvertibilität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10750-0">
    <skos:altLabel xml:lang="de">Konvertierbarkeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10750-0">
    <skos:altLabel xml:lang="de">Interne Konvertibilität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18830-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18418-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18296-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14762-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19020-6">
    <skos:prefLabel xml:lang="en">Organic fertilizer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19020-6">
    <skos:prefLabel xml:lang="de">Organischer Dünger</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19596-2">
    <skos:prefLabel xml:lang="en">Evolutionary game theory</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19596-2">
    <skos:prefLabel xml:lang="de">Evolutionäre Spieltheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18431-4">
    <skos:broader rdf:resource="thsys/70994"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18431-4">
    <skos:broader rdf:resource="descriptor/12996-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15994-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12598-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id116">
    <rdf:value xml:lang="de">für die individuelle Liquidität</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19089-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11336-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14516-2">
    <skos:broader rdf:resource="thsys/70373"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14516-2">
    <skos:broader rdf:resource="descriptor/14522-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71058">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18923-4">
    <skos:broader rdf:resource="thsys/70828"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18923-4">
    <skos:broader rdf:resource="descriptor/18834-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19244-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18163-3">
    <skos:prefLabel xml:lang="en">Electricity price</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18163-3">
    <skos:prefLabel xml:lang="de">Stromtarif</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16732-0">
    <skos:related rdf:resource="descriptor/15686-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16732-0">
    <skos:related rdf:resource="descriptor/16727-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16732-0">
    <skos:related rdf:resource="descriptor/15685-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16732-0">
    <skos:related rdf:resource="descriptor/20022-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16732-0">
    <skos:related rdf:resource="descriptor/19989-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16732-0">
    <skos:related rdf:resource="descriptor/16708-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16732-0">
    <skos:related rdf:resource="descriptor/16745-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12251-1">
    <skos:broader rdf:resource="thsys/71045"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12251-1">
    <skos:broader rdf:resource="thsys/70899"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12251-1">
    <skos:broader rdf:resource="thsys/70259"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70007">
    <skos:narrower rdf:resource="descriptor/15700-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70007">
    <skos:narrower rdf:resource="descriptor/15699-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70007">
    <skos:narrower rdf:resource="descriptor/15695-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70007">
    <skos:narrower rdf:resource="descriptor/15723-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70007">
    <skos:narrower rdf:resource="descriptor/15968-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70007">
    <skos:narrower rdf:resource="descriptor/15696-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70007">
    <skos:narrower rdf:resource="descriptor/15694-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70007">
    <skos:narrower rdf:resource="descriptor/19635-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70007">
    <skos:narrower rdf:resource="descriptor/15690-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70007">
    <skos:narrower rdf:resource="descriptor/19634-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70007">
    <skos:narrower rdf:resource="descriptor/15689-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70007">
    <skos:narrower rdf:resource="thsys/73387"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70875">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19471-1">
    <skos:scopeNote xml:lang="de">Theoretische und empirische Untersuchungen zur Problematik städtischer Arbeitslosigkeit.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13315-3">
    <skos:prefLabel xml:lang="en">Undertaker</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13315-3">
    <skos:prefLabel xml:lang="de">Bestattungsgewerbe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13301-0">
    <skos:broader rdf:resource="thsys/70958"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13454-3">
    <skos:related rdf:resource="descriptor/11971-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13454-3">
    <skos:related rdf:resource="descriptor/18982-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13454-3">
    <skos:related rdf:resource="descriptor/10503-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13454-3">
    <skos:related rdf:resource="descriptor/19233-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13454-3">
    <skos:related rdf:resource="descriptor/13445-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12501-4">
    <skos:broader rdf:resource="thsys/70245"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12501-4">
    <skos:broader rdf:resource="descriptor/10011-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10297-1">
    <skos:broader rdf:resource="thsys/70530"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10297-1">
    <skos:broader rdf:resource="descriptor/10008-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17982-0">
    <skos:prefLabel xml:lang="en">EFTA countries</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17982-0">
    <skos:prefLabel xml:lang="de">EFTA-Staaten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17783-6">
    <skos:broader rdf:resource="thsys/70086"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17783-6">
    <skos:broader rdf:resource="descriptor/17791-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18586-3">
    <skos:related rdf:resource="descriptor/18595-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18586-3">
    <skos:related rdf:resource="descriptor/18242-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18336-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19363-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14995-6">
    <skos:prefLabel xml:lang="en">Ice cream</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14995-6">
    <skos:prefLabel xml:lang="de">Speiseeis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15075-1">
    <skos:narrower rdf:resource="descriptor/11721-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15075-1">
    <skos:narrower rdf:resource="descriptor/11485-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15075-1">
    <skos:narrower rdf:resource="descriptor/10373-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18482-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19446-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11324-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20136-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19692-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12201-2">
    <skos:prefLabel xml:lang="en">Board of directors</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12201-2">
    <skos:prefLabel xml:lang="de">Vorstand</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18966-0">
    <skos:prefLabel xml:lang="en">Environment personnel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18966-0">
    <skos:prefLabel xml:lang="de">Umweltberufe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12195-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11284-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id165">
    <rdf:value xml:lang="de">Für Börsenregelungen allein</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19375-4">
    <skos:prefLabel xml:lang="en">Flypaper effect</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19375-4">
    <skos:prefLabel xml:lang="de">Flypaper-Effekt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11609-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12907-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13384-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70550">
    <skos:broader rdf:resource="thsys/70041"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11345-4">
    <skos:related rdf:resource="descriptor/11350-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11345-4">
    <skos:related rdf:resource="descriptor/11336-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11345-4">
    <skos:related rdf:resource="descriptor/11322-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11345-4">
    <skos:related rdf:resource="descriptor/10380-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15891-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14724-2">
    <skos:broader rdf:resource="thsys/70201"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15520-6">
    <skos:broader rdf:resource="thsys/71035"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15520-6">
    <skos:broader rdf:resource="descriptor/12761-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18788-5">
    <skos:broader rdf:resource="thsys/71055"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18788-5">
    <skos:broader rdf:resource="thsys/70876"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18788-5">
    <skos:broader rdf:resource="descriptor/13355-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11722-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11814-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11682-0">
    <skos:broader rdf:resource="thsys/71113"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11682-0">
    <skos:broader rdf:resource="thsys/71074"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11682-0">
    <skos:broader rdf:resource="thsys/70184"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11682-0">
    <skos:broader rdf:resource="descriptor/11778-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11682-0">
    <skos:broader rdf:resource="descriptor/11676-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11682-0">
    <skos:broader rdf:resource="descriptor/19407-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13004-6">
    <skos:scopeNote xml:lang="de">Für die synthetische Gummiherstellung und die Erzeugung von Gummiwaren.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13333-1">
    <skos:broader rdf:resource="thsys/70818"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13333-1">
    <skos:broader rdf:resource="descriptor/11627-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13333-1">
    <skos:broader rdf:resource="descriptor/13325-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18842-4">
    <skos:prefLabel xml:lang="en">Production cooperative</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18842-4">
    <skos:prefLabel xml:lang="de">Produktionsgenossenschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10678-3">
    <skos:narrower rdf:resource="descriptor/10671-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10678-3">
    <skos:narrower rdf:resource="descriptor/10659-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11240-4">
    <skos:broader rdf:resource="thsys/71080"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11240-4">
    <skos:broader rdf:resource="thsys/71061"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11240-4">
    <skos:broader rdf:resource="descriptor/11570-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11240-4">
    <skos:broader rdf:resource="descriptor/11235-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13755-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71050">
    <skos:broader rdf:resource="thsys/70113"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11618-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19720-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id50">
    <rdfs:seeAlso rdf:resource="descriptor/18461-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14176-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13558-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18897-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18136-6">
    <skos:broader rdf:resource="thsys/70361"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18136-6">
    <skos:broader rdf:resource="descriptor/18012-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19834-5">
    <skos:broader rdf:resource="thsys/70485"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19834-5">
    <skos:broader rdf:resource="descriptor/12083-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10000-1">
    <skos:altLabel xml:lang="de">Angebotsfunktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12234-1">
    <skos:related rdf:resource="descriptor/13738-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12234-1">
    <skos:related rdf:resource="descriptor/12274-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13784-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16720-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15466-0">
    <skos:altLabel xml:lang="en">Decision making</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15466-0">
    <skos:altLabel xml:lang="de">Entscheidungsstrategie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15466-0">
    <skos:altLabel xml:lang="de">Entscheidungsprozess</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15466-0">
    <skos:altLabel xml:lang="de">Entscheidungsprozeß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13368-3">
    <skos:prefLabel xml:lang="en">Magazine publishing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13368-3">
    <skos:prefLabel xml:lang="de">Zeitschriftenverlag</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14296-4">
    <skos:broader rdf:resource="thsys/70134"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14296-4">
    <skos:broader rdf:resource="descriptor/14302-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16287-2">
    <skos:altLabel xml:lang="de">Willensbildungsprozess in der Politik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16287-2">
    <skos:altLabel xml:lang="de">Willensbildungsprozeß in der Politik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16287-2">
    <skos:altLabel xml:lang="de">Politischer Willensbildungsprozess</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16287-2">
    <skos:altLabel xml:lang="de">Politischer Willensbildungsprozeß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15727-5">
    <skos:broader rdf:resource="thsys/70818"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15727-5">
    <skos:broader rdf:resource="descriptor/10470-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18173-0">
    <skos:related rdf:resource="descriptor/19495-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18173-0">
    <skos:related rdf:resource="descriptor/11335-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18173-0">
    <skos:related rdf:resource="descriptor/12594-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13292-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12903-5">
    <skos:altLabel xml:lang="en">CPA (Certified public accountant)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12903-5">
    <skos:altLabel xml:lang="en">Chartered accountant</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12903-5">
    <skos:altLabel xml:lang="en">Certified public accountant</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12903-5">
    <skos:altLabel xml:lang="en">Auditor (Chartered accountant)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12903-5">
    <skos:altLabel xml:lang="de">Wirtschaftsprüfungspraxis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12903-5">
    <skos:altLabel xml:lang="de">Wirtschaftsprüfungsgesellschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12903-5">
    <skos:altLabel xml:lang="de">Wirtschaftsprüferinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12903-5">
    <skos:altLabel xml:lang="de">Wirtschaftsprüfer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12903-5">
    <skos:altLabel xml:lang="de">Pflichtprüfung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12903-5">
    <skos:altLabel xml:lang="de">Externe Revision</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19314-3">
    <skos:altLabel xml:lang="en">Countervailing duty</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19314-3">
    <skos:altLabel xml:lang="en">Antidumping rule</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19314-3">
    <skos:altLabel xml:lang="en">Anti-dumping rule</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19314-3">
    <skos:altLabel xml:lang="en">Anti-dumping duty</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19314-3">
    <skos:altLabel xml:lang="de">Dumpingabwehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19314-3">
    <skos:altLabel xml:lang="de">Antidumpingzoll</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19314-3">
    <skos:altLabel xml:lang="de">Antidumpingpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13565-1">
    <skos:prefLabel xml:lang="en">Transportation law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13565-1">
    <skos:prefLabel xml:lang="de">Frachtrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17622-2">
    <skos:broader rdf:resource="thsys/70253"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17622-2">
    <skos:broader rdf:resource="descriptor/17608-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14385-5">
    <skos:related rdf:resource="descriptor/14805-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14385-5">
    <skos:related rdf:resource="descriptor/13117-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14385-5">
    <skos:related rdf:resource="descriptor/14802-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14385-5">
    <skos:related rdf:resource="descriptor/14377-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14385-5">
    <skos:related rdf:resource="descriptor/14371-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70817">
    <rdfs:label xml:lang="en">Waste water management</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70817">
    <rdfs:label xml:lang="de">Abwasserwirtschaft</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12755-1">
    <skos:broader rdf:resource="thsys/70029"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14607-6">
    <skos:altLabel xml:lang="de">Synthetischer Klebstoff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14607-6">
    <skos:altLabel xml:lang="de">Selbstklebeband</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14607-6">
    <skos:altLabel xml:lang="de">Leim</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14607-6">
    <skos:altLabel xml:lang="de">Klebeband</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14607-6">
    <skos:altLabel xml:lang="de">Dichtstoff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12736-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13217-3">
    <skos:broader rdf:resource="thsys/70961"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13217-3">
    <skos:broader rdf:resource="descriptor/13210-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11314-1">
    <skos:prefLabel xml:lang="en">Labour relations</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11314-1">
    <skos:prefLabel xml:lang="de">Arbeitsbeziehungen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18781-5">
    <skos:prefLabel xml:lang="en">University research institute</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18781-5">
    <skos:prefLabel xml:lang="de">Hochschulinstitut</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18844-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13595-6">
    <skos:related rdf:resource="descriptor/13594-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13595-6">
    <skos:related rdf:resource="descriptor/13593-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13595-6">
    <skos:related rdf:resource="descriptor/11863-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10338-6">
    <skos:prefLabel xml:lang="en">Business cycle</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10338-6">
    <skos:prefLabel xml:lang="de">Konjunktur</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19747-0">
    <skos:prefLabel xml:lang="en">Aesthetics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19747-0">
    <skos:prefLabel xml:lang="de">Ästhetik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15372-2">
    <skos:related rdf:resource="descriptor/15272-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15372-2">
    <skos:related rdf:resource="descriptor/15340-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15372-2">
    <skos:related rdf:resource="descriptor/15376-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15372-2">
    <skos:related rdf:resource="descriptor/15054-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15372-2">
    <skos:related rdf:resource="descriptor/15026-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16979-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15398-5">
    <skos:related rdf:resource="descriptor/15410-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17982-0">
    <skos:broader rdf:resource="thsys/70307"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17982-0">
    <skos:broader rdf:resource="descriptor/17999-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10525-0">
    <skos:narrower rdf:resource="descriptor/10542-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10525-0">
    <skos:narrower rdf:resource="descriptor/20289-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10525-0">
    <skos:narrower rdf:resource="descriptor/10537-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10525-0">
    <skos:narrower rdf:resource="descriptor/10521-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10525-0">
    <skos:narrower rdf:resource="descriptor/10515-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14227-2">
    <skos:related rdf:resource="descriptor/13095-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14227-2">
    <skos:related rdf:resource="descriptor/13231-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14227-2">
    <skos:related rdf:resource="descriptor/14593-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19489-3">
    <skos:altLabel xml:lang="de">Ölpreisschock</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19489-3">
    <skos:altLabel xml:lang="de">Erdölpreisschock</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19544-0">
    <skos:narrower rdf:resource="descriptor/18552-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19544-0">
    <skos:narrower rdf:resource="descriptor/21283-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19544-0">
    <skos:narrower rdf:resource="descriptor/18731-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18727-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11170-6">
    <skos:scopeNote xml:lang="de">Die politikwissenschaftliche Policy-Forschung befaßt sich vor allem mit Fragen der politischen Steuerung - wie wird ein konkretes Problem durch die politischen Akteure verarbeitet und in Politik umgesetzt. Erklärende Variable in den Analysen sind vor allem politische und politisch-institutionelle Größen. Neuere Ansätze sind oft netzwerk- oder wissensorientiert bzw. basieren auf Verhandlungsmodellen. Ziel ist die wissenschaftliche Fundierung von Politikberatung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18690-1">
    <skos:prefLabel xml:lang="en">Commodity futures</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18690-1">
    <skos:prefLabel xml:lang="de">Rohstoff-Futures</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10185-5">
    <skos:narrower rdf:resource="descriptor/11266-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16055-2">
    <skos:prefLabel xml:lang="en">Crafts law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16055-2">
    <skos:prefLabel xml:lang="de">Handwerksrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19698-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13826-6">
    <skos:altLabel xml:lang="de">Versicherungsrisiko</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14899-2">
    <skos:prefLabel xml:lang="en">Clothing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14899-2">
    <skos:prefLabel xml:lang="de">Bekleidung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10202-3">
    <skos:related rdf:resource="descriptor/15859-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10202-3">
    <skos:related rdf:resource="descriptor/12810-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10202-3">
    <skos:related rdf:resource="descriptor/10215-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10202-3">
    <skos:related rdf:resource="descriptor/10902-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10202-3">
    <skos:related rdf:resource="descriptor/10012-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10202-3">
    <skos:related rdf:resource="descriptor/19121-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10202-3">
    <skos:related rdf:resource="descriptor/12770-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10202-3">
    <skos:related rdf:resource="descriptor/19754-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10202-3">
    <skos:related rdf:resource="descriptor/11211-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14366-2">
    <skos:altLabel xml:lang="de">Zementmaschine</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14366-2">
    <skos:altLabel xml:lang="de">Straßenbaumaschine</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14366-2">
    <skos:altLabel xml:lang="de">Ramme</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14366-2">
    <skos:altLabel xml:lang="de">Planiermaschine</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14366-2">
    <skos:altLabel xml:lang="de">Erdbewegungsmaschine</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14366-2">
    <skos:altLabel xml:lang="de">Betonmischer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14366-2">
    <skos:altLabel xml:lang="de">Baustoffmaschine</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19088-0">
    <skos:broader rdf:resource="thsys/73392"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19088-0">
    <skos:broader rdf:resource="thsys/70036"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19088-0">
    <skos:broader rdf:resource="descriptor/15909-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12848-1">
    <skos:broader rdf:resource="thsys/70046"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12848-1">
    <skos:broader rdf:resource="descriptor/11637-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12848-1">
    <skos:broader rdf:resource="descriptor/12041-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19674-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10565-2">
    <skos:altLabel xml:lang="en">Trade diversification</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10565-2">
    <skos:altLabel xml:lang="de">Exportorientierte Industrialisierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10565-2">
    <skos:altLabel xml:lang="de">Ausfuhrdiversifizierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12204-3">
    <skos:prefLabel xml:lang="en">Efficient-market hypothesis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12204-3">
    <skos:prefLabel xml:lang="de">Effizienzmarktthese</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10680-2">
    <skos:prefLabel xml:lang="en">Balance of payments imbalances</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10680-2">
    <skos:prefLabel xml:lang="de">Zahlungsbilanzungleichgewicht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13371-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18302-1">
    <skos:prefLabel xml:lang="en">Tax balance sheet law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18302-1">
    <skos:prefLabel xml:lang="de">Bilanzsteuerrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13621-3">
    <skos:prefLabel xml:lang="en">Freight transport</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13621-3">
    <skos:prefLabel xml:lang="de">Güterverkehr</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18805-3">
    <skos:broader rdf:resource="thsys/73360"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18805-3">
    <skos:broader rdf:resource="descriptor/16019-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19145-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71084">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.08.03</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70085">
    <skos:prefLabel xml:lang="en">G.03.02  North Africa</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70085">
    <skos:prefLabel xml:lang="de">G.03.02  Nordafrika</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13605-1">
    <skos:prefLabel xml:lang="en">Fare</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13605-1">
    <skos:prefLabel xml:lang="de">Fahrpreis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19639-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16529-4">
    <skos:broader rdf:resource="thsys/73370"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16529-4">
    <skos:broader rdf:resource="thsys/70833"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16529-4">
    <skos:broader rdf:resource="thsys/70324"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16529-4">
    <skos:broader rdf:resource="descriptor/16545-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17982-0">
    <skos:narrower rdf:resource="descriptor/16972-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17982-0">
    <skos:narrower rdf:resource="descriptor/16985-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17982-0">
    <skos:narrower rdf:resource="descriptor/16824-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17982-0">
    <skos:narrower rdf:resource="descriptor/16984-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12986-3">
    <skos:prefLabel xml:lang="en">Potash industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12986-3">
    <skos:prefLabel xml:lang="de">Kalibergbau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12070-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16542-5">
    <skos:altLabel xml:lang="en">Social protection</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16542-5">
    <skos:altLabel xml:lang="de">System der sozialen Sicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16542-5">
    <skos:altLabel xml:lang="de">Soziales Sicherungssystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16542-5">
    <skos:altLabel xml:lang="de">Soziale Sicherheit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16542-5">
    <skos:altLabel xml:lang="de">Beveridgeplan</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16542-5">
    <skos:altLabel xml:lang="de">Beveridge-Plan</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13965-6">
    <skos:broader rdf:resource="thsys/71054"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13965-6">
    <skos:broader rdf:resource="thsys/70905"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13965-6">
    <skos:broader rdf:resource="descriptor/18977-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10811-6">
    <skos:narrower rdf:resource="descriptor/12292-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10811-6">
    <skos:narrower rdf:resource="descriptor/10815-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10811-6">
    <skos:narrower rdf:resource="descriptor/10573-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10811-6">
    <skos:narrower rdf:resource="descriptor/10813-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10811-6">
    <skos:narrower rdf:resource="descriptor/10785-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10811-6">
    <skos:narrower rdf:resource="descriptor/10823-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11944-3">
    <skos:altLabel xml:lang="en">European regional development fund</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11944-3">
    <skos:altLabel xml:lang="en">ERDF (European Regional Development Fund)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11944-3">
    <skos:altLabel xml:lang="en">EFRE</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11944-3">
    <skos:altLabel xml:lang="de">Regionalfonds der EG</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11944-3">
    <skos:altLabel xml:lang="de">Kohäsionsfonds der EU</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11944-3">
    <skos:altLabel xml:lang="de">Europäischer Regionalfonds</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11944-3">
    <skos:altLabel xml:lang="de">Europäischer Fonds für die regionale Entwicklung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11944-3">
    <skos:altLabel xml:lang="de">EU-Kohäsionsfonds</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11944-3">
    <skos:altLabel xml:lang="de">EG-Regionalfonds</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73379">
    <skos:broader rdf:resource="thsys/73382"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12331-3">
    <skos:altLabel xml:lang="de">EU-Bilanzrichtlinie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12331-3">
    <skos:altLabel xml:lang="de">EG-Bilanzrichtlinie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12331-3">
    <skos:altLabel xml:lang="de">Buchführungsrichtlinie der EU</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12331-3">
    <skos:altLabel xml:lang="de">Bilanzrichtliniengesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12331-3">
    <skos:altLabel xml:lang="de">Bilanzierungsrichtlinie der EU</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13456-6">
    <skos:narrower rdf:resource="descriptor/18018-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18084-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11340-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19108-6">
    <skos:related rdf:resource="descriptor/19069-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19108-6">
    <skos:related rdf:resource="descriptor/12053-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16067-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11318-0">
    <skos:related rdf:resource="descriptor/18829-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11318-0">
    <skos:related rdf:resource="descriptor/11559-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11318-0">
    <skos:related rdf:resource="descriptor/11320-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70993">
    <rdfs:label xml:lang="en">Pharmaceutical industry</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70993">
    <rdfs:label xml:lang="de">Pharmazeutische Industrie</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13557-0">
    <skos:broader rdf:resource="thsys/70939"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13557-0">
    <skos:broader rdf:resource="thsys/70938"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13557-0">
    <skos:broader rdf:resource="descriptor/18578-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14985-2">
    <skos:prefLabel xml:lang="en">Wine</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14985-2">
    <skos:prefLabel xml:lang="de">Wein</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16441-4">
    <skos:prefLabel xml:lang="en">International relations</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16441-4">
    <skos:prefLabel xml:lang="de">Internationale Beziehungen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19777-5">
    <skos:broader rdf:resource="thsys/70580"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19777-5">
    <skos:broader rdf:resource="descriptor/15644-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12432-4">
    <skos:prefLabel xml:lang="en">Risk accounting</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12432-4">
    <skos:prefLabel xml:lang="de">Bilanzielle Risikobewertung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14656-0">
    <skos:related rdf:resource="descriptor/14633-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14656-0">
    <skos:related rdf:resource="descriptor/14044-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14656-0">
    <skos:related rdf:resource="descriptor/12995-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10991-6">
    <skos:narrower rdf:resource="descriptor/11238-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19702-1">
    <skos:broader rdf:resource="thsys/73376"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19702-1">
    <skos:broader rdf:resource="descriptor/16354-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70575">
    <skos:prefLabel xml:lang="de">W.00  Wirtschaftssektoren und spezielle Wirtschaftslehren</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16698-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19001-3">
    <skos:prefLabel xml:lang="en">Personnel controlling</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19001-3">
    <skos:prefLabel xml:lang="de">Personalcontrolling</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19338-3">
    <skos:altLabel xml:lang="de">Lebensmittelmarkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17039-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10209-3">
    <skos:narrower rdf:resource="descriptor/10197-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15954-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24847-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16037-4">
    <skos:prefLabel xml:lang="en">Copyright law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16037-4">
    <skos:prefLabel xml:lang="de">Urheberrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17758-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14305-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13638-0">
    <skos:altLabel xml:lang="de">Tourismusökonomie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13638-0">
    <skos:altLabel xml:lang="de">Ökonomische Theorie des Tourismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13638-0">
    <skos:altLabel xml:lang="de">Fremdenverkehrsökonomie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70179">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.07</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10074-0">
    <skos:related rdf:resource="descriptor/10080-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10074-0">
    <skos:related rdf:resource="descriptor/10082-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12175-5">
    <skos:broader rdf:resource="thsys/70876"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12175-5">
    <skos:broader rdf:resource="thsys/70087"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15610-5">
    <skos:altLabel xml:lang="de">Software-Kompatibilität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15610-5">
    <skos:altLabel xml:lang="de">Portabilität (Software)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11888-1">
    <skos:scopeNote xml:lang="de">Wanderung zwischen Regionen desselben Landes.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16207-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16004-5">
    <skos:altLabel xml:lang="de">Nahrungsmittelrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16004-5">
    <skos:altLabel xml:lang="de">Lebensmittelkennzeichnungsverordnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16004-5">
    <skos:altLabel xml:lang="de">Lebensmittel- und Bedarfsgegenständegesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16004-5">
    <skos:altLabel xml:lang="de">Bedarfsgegenständeverordnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16269-4">
    <skos:altLabel xml:lang="en">Government publicity</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16269-4">
    <skos:altLabel xml:lang="de">Staatliche Propaganda</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16269-4">
    <skos:altLabel xml:lang="de">Staatliche Öffentlichkeitsarbeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16269-4">
    <skos:altLabel xml:lang="de">Staatliche Informationspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16269-4">
    <skos:altLabel xml:lang="de">Regierungspropaganda</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16269-4">
    <skos:altLabel xml:lang="de">Regierungsinformation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16269-4">
    <skos:altLabel xml:lang="de">Propaganda der Regierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16269-4">
    <skos:altLabel xml:lang="de">Öffentlichkeitsarbeit des Staates</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16269-4">
    <skos:altLabel xml:lang="de">Öffentlichkeitsarbeit der Regierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11182-6">
    <skos:broader rdf:resource="thsys/71060"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15881-2">
    <skos:altLabel xml:lang="de">Klimatologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15881-2">
    <skos:altLabel xml:lang="de">Klimakunde</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15881-2">
    <skos:altLabel xml:lang="de">Klimabeobachtung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11806-1">
    <skos:prefLabel xml:lang="en">EU agricultural market regulations</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11806-1">
    <skos:prefLabel xml:lang="de">EU-Agrarmarktordnung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13472-1">
    <skos:broader rdf:resource="thsys/70957"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13472-1">
    <skos:broader rdf:resource="descriptor/12922-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13472-1">
    <skos:broader rdf:resource="descriptor/13210-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11061-4">
    <skos:prefLabel xml:lang="en">Centrally planned economy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11061-4">
    <skos:prefLabel xml:lang="de">Planwirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10735-3">
    <skos:broader rdf:resource="thsys/71094"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10735-3">
    <skos:broader rdf:resource="descriptor/19335-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10344-4">
    <skos:scopeNote xml:lang="de">Sunspot-Variablen ergänzen das Konzept der rationalen Erwartungen. Sie sind beliebige exogene Faktoren ("sunspots"), die keinen Einfluss auf die ökonomischen Fundamentalvariablen ausüben, von den Wirtschaftssubjekten aber subjektiv als relevant angesehen und bei der Erwartungsbildung berücksichtigt werden (selbsterfüllende Erwartungsänderungen). Sunspot-Variablen werden u.a. in Konjunktur- und Finanzmarktmodellen eingesetzt. Ursprüngliche Sonnenfleckenhypothese von W.S. Jevons zum Zusammenhang zwischen Sonnenfleckenaktivität und wirtschaftlicher Produktionshöhe.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13722-4">
    <skos:altLabel xml:lang="de">Finanzterminkontrakt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13722-4">
    <skos:altLabel xml:lang="de">Finanztermingeschäft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19154-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13024-0">
    <skos:related rdf:resource="descriptor/14252-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11957-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21282-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73367">
    <skos:prefLabel xml:lang="de">N.04.03.07  Landes- und Kommunalpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19200-4">
    <skos:prefLabel xml:lang="en">Substitution effect</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19200-4">
    <skos:prefLabel xml:lang="de">Substitutionseffekt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10446-3">
    <skos:prefLabel xml:lang="en">Endogeneous technical change</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10446-3">
    <skos:prefLabel xml:lang="de">Endogener technischer Fortschritt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70259">
    <skos:broader rdf:resource="thsys/70055"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18584-0">
    <skos:prefLabel xml:lang="en">Taxi industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18584-0">
    <skos:prefLabel xml:lang="de">Taxigewerbe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12009-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73373">
    <rdfs:label xml:lang="de">Innen- und Rechtspolitik</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18888-1">
    <skos:narrower rdf:resource="descriptor/19603-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19051-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18950-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10541-2">
    <skos:related rdf:resource="descriptor/10517-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12155-4">
    <skos:related rdf:resource="descriptor/12587-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14025-0">
    <skos:altLabel xml:lang="de">Zahntechnisches Produkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14025-0">
    <skos:altLabel xml:lang="de">Zahnprothese</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14025-0">
    <skos:altLabel xml:lang="de">Zahnmedizinisches Produkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14025-0">
    <skos:altLabel xml:lang="de">Zahnersatz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14025-0">
    <skos:altLabel xml:lang="de">Dentalporzellan</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14025-0">
    <skos:altLabel xml:lang="de">Dentalmedizinisches Gerät</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14516-2">
    <skos:narrower rdf:resource="descriptor/14518-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10007-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18421-0">
    <skos:altLabel xml:lang="de">Kfz-Technik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18421-0">
    <skos:altLabel xml:lang="de">Autotechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18421-0">
    <skos:altLabel xml:lang="de">Automobiltechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13743-3">
    <skos:altLabel xml:lang="en">Bucket shop</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13743-3">
    <skos:altLabel xml:lang="de">Effektenspekulation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13743-3">
    <skos:altLabel xml:lang="de">Börsenspekulation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11024-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id211">
    <rdfs:seeAlso rdf:resource="descriptor/15821-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12318-2">
    <skos:related rdf:resource="descriptor/11678-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12318-2">
    <skos:related rdf:resource="descriptor/12488-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12318-2">
    <skos:related rdf:resource="descriptor/12311-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10486-5">
    <skos:broader rdf:resource="thsys/70847"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10486-5">
    <skos:broader rdf:resource="thsys/70176"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10486-5">
    <skos:broader rdf:resource="thsys/70143"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10486-5">
    <skos:broader rdf:resource="thsys/70091"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10486-5">
    <skos:broader rdf:resource="descriptor/11354-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19308-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11580-1">
    <skos:related rdf:resource="descriptor/12540-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11580-1">
    <skos:related rdf:resource="descriptor/11578-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11580-1">
    <skos:related rdf:resource="descriptor/11579-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11580-1">
    <skos:related rdf:resource="descriptor/16176-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10037-6">
    <skos:prefLabel xml:lang="en">Economic research</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10037-6">
    <skos:prefLabel xml:lang="de">Wirtschaftsforschung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11890-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13149-1">
    <skos:prefLabel xml:lang="en">Wool industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13149-1">
    <skos:prefLabel xml:lang="de">Wollindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10087-5">
    <skos:prefLabel xml:lang="en">Disequilibrium model</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10087-5">
    <skos:prefLabel xml:lang="de">Ungleichgewichtstheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11938-5">
    <skos:broader rdf:resource="thsys/71069"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11938-5">
    <skos:broader rdf:resource="thsys/71020"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11938-5">
    <skos:broader rdf:resource="descriptor/16552-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11938-5">
    <skos:broader rdf:resource="descriptor/16314-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11938-5">
    <skos:broader rdf:resource="descriptor/11804-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17745-0">
    <skos:prefLabel xml:lang="en">Egypt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17745-0">
    <skos:prefLabel xml:lang="de">Ägypten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19205-1">
    <skos:prefLabel xml:lang="en">Feminist economics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19205-1">
    <skos:prefLabel xml:lang="de">Feministische Ökonomik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16955-1">
    <skos:related rdf:resource="descriptor/16948-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19359-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25743-1">
    <skos:prefLabel xml:lang="en">Schumpeterian approach</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25743-1">
    <skos:prefLabel xml:lang="de">Schumpeterismus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13587-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19084-1">
    <skos:prefLabel xml:lang="en">Central-Eastern Europe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19084-1">
    <skos:prefLabel xml:lang="de">Ostmitteleuropa</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13722-4">
    <skos:narrower rdf:resource="descriptor/18689-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70930">
    <skos:prefLabel xml:lang="en">W.12.03.01  Combined transport</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70930">
    <skos:prefLabel xml:lang="de">W.12.03.01  Kombinierter Verkehr</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19714-1">
    <skos:scopeNote xml:lang="de">Voraussetzungen und Rechtsfolgen von Änderungen der staatlichen Hoheit über ein bestimmtes Gebiet.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13553-1">
    <skos:broader rdf:resource="thsys/70940"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13553-1">
    <skos:broader rdf:resource="thsys/70934"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13553-1">
    <skos:broader rdf:resource="descriptor/13256-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13553-1">
    <skos:broader rdf:resource="descriptor/13621-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13844-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18598-3">
    <skos:related rdf:resource="descriptor/14462-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10658-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12821-0">
    <skos:broader rdf:resource="thsys/70947"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12821-0">
    <skos:broader rdf:resource="thsys/70384"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12821-0">
    <skos:broader rdf:resource="descriptor/12818-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11703-4">
    <skos:prefLabel xml:lang="en">Tax harmonization</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11703-4">
    <skos:prefLabel xml:lang="de">Steuerharmonisierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16783-4">
    <skos:broader rdf:resource="thsys/70822"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13862-2">
    <skos:broader rdf:resource="thsys/71067"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13862-2">
    <skos:broader rdf:resource="thsys/71064"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13862-2">
    <skos:broader rdf:resource="thsys/70888"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13862-2">
    <skos:broader rdf:resource="descriptor/11913-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18330-3">
    <skos:related rdf:resource="descriptor/10697-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18330-3">
    <skos:related rdf:resource="descriptor/11562-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18330-3">
    <skos:related rdf:resource="descriptor/18008-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16275-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13611-6">
    <skos:broader rdf:resource="thsys/70927"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13611-6">
    <skos:broader rdf:resource="descriptor/12036-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16789-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15467-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18792-0">
    <skos:prefLabel xml:lang="en">Temporary employment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18792-0">
    <skos:prefLabel xml:lang="de">Befristeter Arbeitsvertrag</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14936-1">
    <skos:related rdf:resource="descriptor/13159-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14936-1">
    <skos:related rdf:resource="descriptor/14060-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17015-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18101-4">
    <skos:narrower rdf:resource="descriptor/18744-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18101-4">
    <skos:narrower rdf:resource="descriptor/18639-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18101-4">
    <skos:narrower rdf:resource="descriptor/18954-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18101-4">
    <skos:narrower rdf:resource="descriptor/18798-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18101-4">
    <skos:narrower rdf:resource="descriptor/18532-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18101-4">
    <skos:narrower rdf:resource="descriptor/18718-5"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id29">
    <rdfs:seeAlso rdf:resource="descriptor/11569-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14710-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#mappingRelation">
    <skos:definition xml:lang="en">Definition</skos:definition>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10173-5">
    <skos:broader rdf:resource="thsys/70814"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10173-5">
    <skos:broader rdf:resource="thsys/70493"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10173-5">
    <skos:broader rdf:resource="descriptor/10032-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10042-6">
    <skos:related rdf:resource="descriptor/10051-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10042-6">
    <skos:related rdf:resource="descriptor/19612-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10042-6">
    <skos:related rdf:resource="descriptor/19228-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19125-6">
    <skos:prefLabel xml:lang="en">Market leader</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19125-6">
    <skos:prefLabel xml:lang="de">Marktführer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13666-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13363-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15775-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18821-5">
    <skos:altLabel xml:lang="de">Qualität von Dienstleistungen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19810-5">
    <skos:related rdf:resource="descriptor/15410-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19810-5">
    <skos:related rdf:resource="descriptor/15605-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19810-5">
    <skos:related rdf:resource="descriptor/15604-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19810-5">
    <skos:related rdf:resource="descriptor/16190-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19810-5">
    <skos:related rdf:resource="descriptor/18271-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10064-3">
    <skos:altLabel xml:lang="de">Globalmodell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10064-3">
    <skos:altLabel xml:lang="de">Global 2000</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id66">
    <rdfs:seeAlso rdf:resource="descriptor/15604-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10808-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71076">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10784-4">
    <skos:altLabel xml:lang="de">Außenhandelsbilanz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14044-3">
    <skos:broader rdf:resource="thsys/70201"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71116">
    <skos:narrower rdf:resource="descriptor/10156-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71116">
    <skos:narrower rdf:resource="descriptor/10148-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71116">
    <skos:narrower rdf:resource="descriptor/11260-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71116">
    <skos:narrower rdf:resource="descriptor/10184-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71116">
    <skos:narrower rdf:resource="descriptor/10077-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71116">
    <skos:narrower rdf:resource="descriptor/19209-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71116">
    <skos:narrower rdf:resource="descriptor/11194-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71116">
    <skos:narrower rdf:resource="descriptor/11187-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19232-5">
    <skos:prefLabel xml:lang="en">Capital gains</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19232-5">
    <skos:prefLabel xml:lang="de">Vermögenszuwachs</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19251-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10836-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18177-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13466-3">
    <skos:narrower rdf:resource="descriptor/10856-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11057-2">
    <skos:altLabel xml:lang="en">Economic theory of constitutions</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11057-2">
    <skos:altLabel xml:lang="en">Contractarian paradigm</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11057-2">
    <skos:altLabel xml:lang="en">Constitutional political economy</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11057-2">
    <skos:altLabel xml:lang="en">Constitutional choice</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11057-2">
    <skos:altLabel xml:lang="de">Vertragstheorie (Verfassungsökonomik)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11057-2">
    <skos:altLabel xml:lang="de">Verfassungsökonomie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11057-2">
    <skos:altLabel xml:lang="de">Ökonomische Theorie der Verfassung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11057-2">
    <skos:altLabel xml:lang="de">Konstitutionelle Ökonomie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11275-6">
    <skos:prefLabel xml:lang="en">Young workers</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11275-6">
    <skos:prefLabel xml:lang="de">Jugendliche Arbeitskräfte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18155-2">
    <skos:related rdf:resource="descriptor/10060-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18155-2">
    <skos:related rdf:resource="descriptor/11754-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18155-2">
    <skos:related rdf:resource="descriptor/11065-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18665-0">
    <skos:broader rdf:resource="thsys/73341"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18665-0">
    <skos:broader rdf:resource="thsys/70922"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18665-0">
    <skos:broader rdf:resource="descriptor/18666-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18665-0">
    <skos:broader rdf:resource="descriptor/14751-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19166-6">
    <skos:prefLabel xml:lang="en">Korea</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19166-6">
    <skos:prefLabel xml:lang="de">Korea</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18914-5">
    <skos:prefLabel xml:lang="en">Pension law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18914-5">
    <skos:prefLabel xml:lang="de">Rentenrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70013">
    <skos:prefLabel xml:lang="de">N.06.04  Spezielle Soziologien und sozialwissenschaftliche Teilbereiche</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70506">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">P.03</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id95">
    <rdf:value xml:lang="de">Für die Förderung der beruflichen Bildung durch die EU</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11367-1">
    <skos:narrower rdf:resource="descriptor/18865-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11367-1">
    <skos:narrower rdf:resource="descriptor/13342-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11367-1">
    <skos:narrower rdf:resource="descriptor/11369-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11367-1">
    <skos:narrower rdf:resource="descriptor/18857-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11367-1">
    <skos:narrower rdf:resource="descriptor/11360-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13867-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13141-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10054-6">
    <skos:prefLabel xml:lang="en">Post Keynesian economics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10054-6">
    <skos:prefLabel xml:lang="de">Postkeynesianismus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15038-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16277-5">
    <skos:prefLabel xml:lang="en">Political decision</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16277-5">
    <skos:prefLabel xml:lang="de">Politische Entscheidung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id257">
    <rdfs:seeAlso rdf:resource="descriptor/18649-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19480-0">
    <skos:altLabel xml:lang="de">Umweltschutzabkommen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19480-0">
    <skos:altLabel xml:lang="de">Internationales Umweltabkommen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12136-1">
    <skos:related rdf:resource="descriptor/12111-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12136-1">
    <skos:related rdf:resource="descriptor/12142-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12136-1">
    <skos:related rdf:resource="descriptor/11158-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14082-2">
    <skos:related rdf:resource="descriptor/14997-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14082-2">
    <skos:related rdf:resource="descriptor/13164-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14082-2">
    <skos:related rdf:resource="descriptor/13238-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14082-2">
    <skos:related rdf:resource="descriptor/18196-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11352-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18423-3">
    <skos:prefLabel xml:lang="en">Motor vehicle research</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18423-3">
    <skos:prefLabel xml:lang="de">Kraftfahrtforschung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10960-3">
    <skos:scopeNote xml:lang="de">Nur für die wettbewerbsrechtliche Missbrauchsaufsicht über die Preisbildung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14674-5">
    <skos:broader rdf:resource="thsys/70556"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14674-5">
    <skos:broader rdf:resource="descriptor/14656-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10706-3">
    <skos:related rdf:resource="descriptor/10699-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10706-3">
    <skos:related rdf:resource="descriptor/10692-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19100-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71012">
    <skos:prefLabel xml:lang="en">W.04.01.04  Energy policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71012">
    <skos:prefLabel xml:lang="de">W.04.01.04  Energiepolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14216-0">
    <skos:altLabel xml:lang="de">Treibstoff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70128">
    <skos:narrower rdf:resource="descriptor/17951-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70128">
    <skos:narrower rdf:resource="descriptor/17950-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70128">
    <skos:narrower rdf:resource="descriptor/17949-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70128">
    <skos:narrower rdf:resource="descriptor/17948-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70128">
    <skos:narrower rdf:resource="descriptor/17947-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70128">
    <skos:narrower rdf:resource="descriptor/17946-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70128">
    <skos:narrower rdf:resource="descriptor/17943-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70128">
    <skos:narrower rdf:resource="descriptor/19169-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70128">
    <skos:narrower rdf:resource="descriptor/20987-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70128">
    <skos:narrower rdf:resource="descriptor/17942-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70128">
    <skos:narrower rdf:resource="descriptor/17941-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70128">
    <skos:narrower rdf:resource="descriptor/17940-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70128">
    <skos:narrower rdf:resource="descriptor/24788-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70128">
    <skos:narrower rdf:resource="descriptor/19168-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70128">
    <skos:narrower rdf:resource="descriptor/17938-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70128">
    <skos:narrower rdf:resource="descriptor/17937-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70128">
    <skos:narrower rdf:resource="descriptor/17935-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19181-3">
    <skos:related rdf:resource="descriptor/12920-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19181-3">
    <skos:related rdf:resource="descriptor/19489-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19181-3">
    <skos:related rdf:resource="descriptor/19182-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19181-3">
    <skos:related rdf:resource="descriptor/13221-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19181-3">
    <skos:related rdf:resource="descriptor/18373-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19181-3">
    <skos:related rdf:resource="descriptor/19388-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19181-3">
    <skos:related rdf:resource="descriptor/14197-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19181-3">
    <skos:related rdf:resource="descriptor/18812-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11230-0">
    <skos:broader rdf:resource="thsys/71062"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11230-0">
    <skos:broader rdf:resource="descriptor/11211-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12262-3">
    <skos:altLabel xml:lang="de">Wagniskapital</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12262-3">
    <skos:altLabel xml:lang="de">Wagnisfinanzierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12262-3">
    <skos:altLabel xml:lang="de">Risikofinanzierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12262-3">
    <skos:altLabel xml:lang="de">Innovationsfinanzierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16995-3">
    <skos:broader rdf:resource="thsys/70505"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16995-3">
    <skos:broader rdf:resource="descriptor/16990-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16995-3">
    <skos:broader rdf:resource="descriptor/17991-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16995-3">
    <skos:broader rdf:resource="descriptor/17983-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70265">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12121-0">
    <skos:broader rdf:resource="thsys/70875"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12121-0">
    <skos:broader rdf:resource="descriptor/12881-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18982-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18967-5">
    <skos:broader rdf:resource="thsys/71068"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18967-5">
    <skos:broader rdf:resource="thsys/71066"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18967-5">
    <skos:broader rdf:resource="thsys/70815"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18967-5">
    <skos:broader rdf:resource="descriptor/19447-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18967-5">
    <skos:broader rdf:resource="descriptor/15746-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18967-5">
    <skos:broader rdf:resource="descriptor/11866-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13316-1">
    <skos:prefLabel xml:lang="en">Barber</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13316-1">
    <skos:prefLabel xml:lang="de">Friseurhandwerk</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18029-0">
    <skos:prefLabel xml:lang="en">Strategic management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18029-0">
    <skos:prefLabel xml:lang="de">Strategisches Management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10093-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13269-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16143-5">
    <skos:prefLabel xml:lang="en">Sickness beneficiaries</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16143-5">
    <skos:prefLabel xml:lang="de">Lohnfortzahlung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10734-5">
    <skos:related rdf:resource="descriptor/10771-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10734-5">
    <skos:related rdf:resource="descriptor/16048-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10734-5">
    <skos:related rdf:resource="descriptor/19336-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17696-1">
    <skos:broader rdf:resource="thsys/70560"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17696-1">
    <skos:broader rdf:resource="descriptor/17724-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14294-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11447-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70964">
    <skos:broader rdf:resource="thsys/70965"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18750-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13782-0">
    <skos:altLabel xml:lang="de">Recht der Forderungsabtretung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13782-0">
    <skos:altLabel xml:lang="de">Forderungsabtretung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13782-0">
    <skos:altLabel xml:lang="de">Abtretung einer Forderung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10770-1">
    <skos:related rdf:resource="descriptor/10791-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10770-1">
    <skos:related rdf:resource="descriptor/10722-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10770-1">
    <skos:related rdf:resource="descriptor/18127-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10770-1">
    <skos:related rdf:resource="descriptor/10764-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10770-1">
    <skos:related rdf:resource="descriptor/10754-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10770-1">
    <skos:related rdf:resource="descriptor/10746-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10770-1">
    <skos:related rdf:resource="descriptor/10745-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10770-1">
    <skos:related rdf:resource="descriptor/10778-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10770-1">
    <skos:related rdf:resource="descriptor/10776-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19539-0">
    <skos:broader rdf:resource="thsys/73347"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19539-0">
    <skos:broader rdf:resource="descriptor/18891-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10631-1">
    <skos:prefLabel xml:lang="en">Gains from trade</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10631-1">
    <skos:prefLabel xml:lang="de">Außenhandelsgewinn</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12717-2">
    <skos:scopeNote xml:lang="de">Produktionstyp, bei dem die Produktionsfaktoren ortsungebunden, die Produkte jedoch zumindest zeitweilig ortsgebunden sind.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18612-0">
    <skos:related rdf:resource="descriptor/18564-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18612-0">
    <skos:related rdf:resource="descriptor/12783-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18612-0">
    <skos:related rdf:resource="descriptor/19151-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18612-0">
    <skos:related rdf:resource="descriptor/13466-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18612-0">
    <skos:related rdf:resource="descriptor/18246-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18612-0">
    <skos:related rdf:resource="descriptor/12812-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19386-6">
    <skos:prefLabel xml:lang="en">Natural gas resources</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19386-6">
    <skos:prefLabel xml:lang="de">Erdgasvorkommen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10443-2">
    <skos:altLabel xml:lang="de">FuE-Management</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10443-2">
    <skos:altLabel xml:lang="de">Forschungsmanagement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10443-2">
    <skos:altLabel xml:lang="de">Forschungsbereich des Betriebs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10443-2">
    <skos:altLabel xml:lang="de">Forschungsbereich des Betriebes</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10443-2">
    <skos:altLabel xml:lang="de">Forschungs- und Entwicklungsmanagement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10443-2">
    <skos:altLabel xml:lang="de">Betrieblicher Forschungsbereich</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16182-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13515-2">
    <skos:related rdf:resource="descriptor/13580-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13515-2">
    <skos:related rdf:resource="descriptor/13587-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16483-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19739-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11506-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12742-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10960-3">
    <skos:prefLabel xml:lang="en">Price control</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10960-3">
    <skos:prefLabel xml:lang="de">Preisaufsicht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19484-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20948-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15026-0">
    <skos:altLabel xml:lang="en">Differential equation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15026-0">
    <skos:altLabel xml:lang="en">Difference equation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15026-0">
    <skos:altLabel xml:lang="de">Integralrechnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15026-0">
    <skos:altLabel xml:lang="de">Infinitesimalrechnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15026-0">
    <skos:altLabel xml:lang="de">Differenzialrechnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15026-0">
    <skos:altLabel xml:lang="de">Differenzialgleichung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15026-0">
    <skos:altLabel xml:lang="de">Differenzengleichung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15026-0">
    <skos:altLabel xml:lang="de">Differentialrechnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15026-0">
    <skos:altLabel xml:lang="de">Differentialgleichung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14335-6">
    <skos:prefLabel xml:lang="en">Gaming and vending machines</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14335-6">
    <skos:prefLabel xml:lang="de">Automat</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id239">
    <rdfs:seeAlso rdf:resource="descriptor/14035-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13763-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18980-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70014">
    <skos:narrower rdf:resource="descriptor/14635-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70014">
    <skos:narrower rdf:resource="descriptor/14632-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70014">
    <skos:narrower rdf:resource="descriptor/14218-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70014">
    <skos:narrower rdf:resource="descriptor/14614-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70014">
    <skos:narrower rdf:resource="descriptor/18268-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70014">
    <skos:narrower rdf:resource="descriptor/14607-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70014">
    <skos:narrower rdf:resource="descriptor/14619-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70014">
    <skos:narrower rdf:resource="descriptor/14744-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70014">
    <skos:narrower rdf:resource="descriptor/14743-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70014">
    <skos:narrower rdf:resource="descriptor/14601-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70014">
    <skos:narrower rdf:resource="descriptor/14600-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70014">
    <skos:narrower rdf:resource="descriptor/14599-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70014">
    <skos:narrower rdf:resource="descriptor/14631-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70014">
    <skos:narrower rdf:resource="thsys/70556"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70014">
    <skos:narrower rdf:resource="thsys/70074"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70014">
    <skos:narrower rdf:resource="thsys/70463"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70014">
    <skos:narrower rdf:resource="thsys/70461"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70014">
    <skos:narrower rdf:resource="thsys/70536"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70014">
    <skos:narrower rdf:resource="thsys/70199"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70014">
    <skos:narrower rdf:resource="thsys/70476"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14001-0">
    <skos:related rdf:resource="descriptor/16760-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14001-0">
    <skos:related rdf:resource="descriptor/19475-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13720-1">
    <skos:prefLabel xml:lang="en">Foreign exchange futures</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13720-1">
    <skos:prefLabel xml:lang="de">Devisentermingeschäft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10018-3">
    <skos:related rdf:resource="descriptor/16201-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10018-3">
    <skos:related rdf:resource="descriptor/19289-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10018-3">
    <skos:related rdf:resource="descriptor/10845-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10018-3">
    <skos:related rdf:resource="descriptor/10501-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10018-3">
    <skos:related rdf:resource="descriptor/10522-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19611-4">
    <skos:altLabel xml:lang="en">Human geography</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19611-4">
    <skos:altLabel xml:lang="en">Cultural geography</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19611-4">
    <skos:altLabel xml:lang="de">Kulturgeographie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19611-4">
    <skos:altLabel xml:lang="de">Humangeographie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19611-4">
    <skos:altLabel xml:lang="de">Anthropogeographie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18259-4">
    <skos:altLabel xml:lang="de">Arzneimittelgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12087-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11372-1">
    <skos:altLabel xml:lang="de">Universitätsprofessorinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11372-1">
    <skos:altLabel xml:lang="de">Universitätsprofessoren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11372-1">
    <skos:altLabel xml:lang="de">Professorinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11372-1">
    <skos:altLabel xml:lang="de">Professoren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11372-1">
    <skos:altLabel xml:lang="de">Hochschullehrerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11372-1">
    <skos:altLabel xml:lang="de">Hochschuldozentinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11372-1">
    <skos:altLabel xml:lang="de">Hochschuldozenten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11372-1">
    <skos:altLabel xml:lang="de">Fachhochschullehrerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11372-1">
    <skos:altLabel xml:lang="de">Fachhochschullehrer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15298-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21282-3">
    <skos:scopeNote xml:lang="de">Wettbewerbsvorteile erlangen besondere Bedeutung im Zusammenhang mit einer vom Unternehmen gewählten Wettbewerbsstrategie innerhalb seiner Branche. Durch Wettbewerbsvorteile erlangt das Unternehmen eine bessere Positionierung im Kreis seiner Wettbewerber.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16740-1">
    <skos:prefLabel xml:lang="en">Austrian Economics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16740-1">
    <skos:prefLabel xml:lang="de">Österreichische Schule</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10261-1">
    <skos:related rdf:resource="descriptor/11216-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10261-1">
    <skos:related rdf:resource="descriptor/11240-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10261-1">
    <skos:related rdf:resource="descriptor/10339-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10261-1">
    <skos:related rdf:resource="descriptor/11220-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10261-1">
    <skos:related rdf:resource="descriptor/11182-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17722-5">
    <skos:altLabel xml:lang="de">Republik Sierra Leone</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13378-0">
    <skos:altLabel xml:lang="de">Chemotechnisches Labor</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13378-0">
    <skos:altLabel xml:lang="de">Chemische Untersuchung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13378-0">
    <skos:altLabel xml:lang="de">Chemische Beratung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13378-0">
    <skos:altLabel xml:lang="de">Chemietechnisches Labor</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13378-0">
    <skos:altLabel xml:lang="de">Chemielabor</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24624-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70816">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19151-5">
    <skos:narrower rdf:resource="descriptor/19327-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19151-5">
    <skos:narrower rdf:resource="descriptor/13666-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19151-5">
    <skos:narrower rdf:resource="descriptor/16370-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19151-5">
    <skos:narrower rdf:resource="descriptor/19711-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17612-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19704-4">
    <skos:narrower rdf:resource="descriptor/19511-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19704-4">
    <skos:narrower rdf:resource="descriptor/15963-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19057-4">
    <skos:prefLabel xml:lang="en">Machinery leasing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19057-4">
    <skos:prefLabel xml:lang="de">Maschinen-Leasing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13028-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17749-6">
    <skos:prefLabel xml:lang="en">Morocco</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17749-6">
    <skos:prefLabel xml:lang="de">Marokko</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15372-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70575">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.00</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15797-5">
    <skos:broader rdf:resource="thsys/73386"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15797-5">
    <skos:broader rdf:resource="thsys/73381"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15797-5">
    <skos:broader rdf:resource="descriptor/18924-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15797-5">
    <skos:broader rdf:resource="descriptor/16556-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18541-4">
    <skos:prefLabel xml:lang="en">International market entry mode</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18541-4">
    <skos:prefLabel xml:lang="de">Internationale Markteintrittsstrategie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19406-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16032-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12432-4">
    <skos:altLabel xml:lang="de">Vorsichtsprinzip</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12432-4">
    <skos:altLabel xml:lang="de">Risikobilanzierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12432-4">
    <skos:altLabel xml:lang="de">Bilanzielle Risikoberücksichtigung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13389-2">
    <skos:narrower rdf:resource="descriptor/18791-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14200-1">
    <skos:broader rdf:resource="thsys/70418"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14200-1">
    <skos:broader rdf:resource="descriptor/18235-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14200-1">
    <skos:broader rdf:resource="descriptor/14192-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19124-1">
    <skos:prefLabel xml:lang="en">Warranty</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19124-1">
    <skos:prefLabel xml:lang="de">Garantie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18719-3">
    <skos:prefLabel xml:lang="en">Banks balance sheet</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18719-3">
    <skos:prefLabel xml:lang="de">Bankbilanz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19665-2">
    <skos:related rdf:resource="descriptor/10774-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19665-2">
    <skos:related rdf:resource="descriptor/19337-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19665-2">
    <skos:related rdf:resource="descriptor/10758-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19665-2">
    <skos:related rdf:resource="descriptor/13763-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19665-2">
    <skos:related rdf:resource="descriptor/13746-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17732-2">
    <skos:altLabel xml:lang="de">Vereinigte Republik Kamerun</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17732-2">
    <skos:altLabel xml:lang="de">Republik Kamerun</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14673-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10159-6">
    <skos:broader rdf:resource="thsys/71118"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10159-6">
    <skos:broader rdf:resource="descriptor/10161-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12215-5">
    <skos:altLabel xml:lang="de">Finanzpolitik des Betriebs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12215-5">
    <skos:altLabel xml:lang="de">Finanzplanung des Betriebs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12215-5">
    <skos:altLabel xml:lang="de">Finanzmanagement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12215-5">
    <skos:altLabel xml:lang="de">Finanzkontrolle des Betriebs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12215-5">
    <skos:altLabel xml:lang="de">Betriebsfinanzwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12215-5">
    <skos:altLabel xml:lang="de">Betriebliche Finanzpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13763-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11552-6">
    <skos:broader rdf:resource="thsys/71072"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11552-6">
    <skos:broader rdf:resource="thsys/71063"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11552-6">
    <skos:broader rdf:resource="thsys/70860"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11552-6">
    <skos:broader rdf:resource="thsys/70530"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11552-6">
    <skos:broader rdf:resource="descriptor/11556-5"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id48">
    <rdfs:seeAlso rdf:resource="descriptor/11406-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18729-0">
    <skos:related rdf:resource="descriptor/18730-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18729-0">
    <skos:related rdf:resource="descriptor/13279-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19605-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11481-3">
    <skos:broader rdf:resource="thsys/70305"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11481-3">
    <skos:broader rdf:resource="descriptor/11069-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19857-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13465-5">
    <skos:prefLabel xml:lang="en">Commercial statistics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13465-5">
    <skos:prefLabel xml:lang="de">Handelsstatistik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10573-3">
    <skos:prefLabel xml:lang="en">Financial aid</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10573-3">
    <skos:prefLabel xml:lang="de">Kapitalhilfe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19481-5">
    <skos:broader rdf:resource="thsys/73352"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19481-5">
    <skos:broader rdf:resource="thsys/70239"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19481-5">
    <skos:broader rdf:resource="descriptor/15770-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10931-3">
    <skos:related rdf:resource="descriptor/12529-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10931-3">
    <skos:related rdf:resource="descriptor/10889-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15741-4">
    <skos:prefLabel xml:lang="en">Plant raw material</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15741-4">
    <skos:prefLabel xml:lang="de">Nachwachsender Rohstoff</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19476-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19142-6">
    <skos:narrower rdf:resource="descriptor/12385-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19142-6">
    <skos:narrower rdf:resource="descriptor/12361-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19142-6">
    <skos:narrower rdf:resource="descriptor/12408-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19142-6">
    <skos:narrower rdf:resource="descriptor/12406-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17771-6">
    <skos:altLabel xml:lang="en">Bahama Islands</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17771-6">
    <skos:altLabel xml:lang="de">Bund der Bahamas</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14084-5">
    <skos:prefLabel xml:lang="en">Sugar plant</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14084-5">
    <skos:prefLabel xml:lang="de">Zuckerpflanze</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11870-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11523-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12053-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id316">
    <rdfs:seeAlso rdf:resource="descriptor/16090-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11152-1">
    <skos:broader rdf:resource="thsys/70274"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11152-1">
    <skos:broader rdf:resource="descriptor/10975-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71113">
    <skos:prefLabel xml:lang="en">V.03.04.02  Distributive politics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71113">
    <skos:prefLabel xml:lang="de">V.03.04.02  Verteilungspolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18252-4">
    <skos:altLabel xml:lang="de">Messgerät (Medizin)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18252-4">
    <skos:altLabel xml:lang="de">Meßgerät (Medizin)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18252-4">
    <skos:altLabel xml:lang="de">Medizintechnisches Produkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18252-4">
    <skos:altLabel xml:lang="de">Medizintechnisches Instrument</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18252-4">
    <skos:altLabel xml:lang="de">Medizintechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18252-4">
    <skos:altLabel xml:lang="de">Medizinisches Messgerät</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18252-4">
    <skos:altLabel xml:lang="de">Medizinisches Meßgerät</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15711-6">
    <zbwext:useInsteadNote rdf:nodeID="id296"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18589-4">
    <skos:narrower rdf:resource="descriptor/18587-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14762-1">
    <skos:altLabel xml:lang="de">Optisches Lesegerät</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14762-1">
    <skos:altLabel xml:lang="de">Chipkartenlesegerät</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14762-1">
    <skos:altLabel xml:lang="de">Belegleser</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19743-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10905-4">
    <skos:narrower rdf:resource="descriptor/10227-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10905-4">
    <skos:narrower rdf:resource="descriptor/10224-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10905-4">
    <skos:narrower rdf:resource="descriptor/10223-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10905-4">
    <skos:narrower rdf:resource="descriptor/19429-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10905-4">
    <skos:narrower rdf:resource="descriptor/10948-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10905-4">
    <skos:narrower rdf:resource="descriptor/10209-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10905-4">
    <skos:narrower rdf:resource="descriptor/10206-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10905-4">
    <skos:narrower rdf:resource="descriptor/10205-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11651-4">
    <skos:broader rdf:resource="thsys/71074"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11651-4">
    <skos:broader rdf:resource="thsys/70184"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11651-4">
    <skos:broader rdf:resource="descriptor/11681-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11651-4">
    <skos:broader rdf:resource="descriptor/11676-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11651-4">
    <skos:broader rdf:resource="descriptor/19407-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15466-0">
    <skos:prefLabel xml:lang="en">Decision</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15466-0">
    <skos:prefLabel xml:lang="de">Entscheidung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16019-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70438">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19489-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18246-6">
    <skos:broader rdf:resource="thsys/70256"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18246-6">
    <skos:broader rdf:resource="descriptor/14418-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10199-1">
    <skos:altLabel xml:lang="de">Hedonistischer Preis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10996-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16324-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10343-6">
    <skos:scopeNote xml:lang="de">Schwerwiegende, strukturbedingte Einbrüche in der gesamtwirtschaftlichen Tätigkeit</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19087-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18254-0">
    <skos:related rdf:resource="descriptor/19623-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14654-4">
    <skos:altLabel xml:lang="de">Stickstoffoxid</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21082-4">
    <skos:broader rdf:resource="thsys/71114"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12703-6">
    <skos:related rdf:resource="descriptor/13170-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12703-6">
    <skos:related rdf:resource="descriptor/14020-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14046-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12979-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11447-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18958-6">
    <skos:prefLabel xml:lang="en">Waste management policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18958-6">
    <skos:prefLabel xml:lang="de">Abfallpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10137-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10085-2">
    <skos:broader rdf:resource="thsys/70188"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10085-2">
    <skos:broader rdf:resource="descriptor/10076-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11902-5">
    <skos:altLabel xml:lang="de">Stadt-Umland-Wanderung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10622-2">
    <skos:prefLabel xml:lang="en">Low wages</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10622-2">
    <skos:prefLabel xml:lang="de">Niedriglohn</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18212-2">
    <skos:prefLabel xml:lang="en">Business journal</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18212-2">
    <skos:prefLabel xml:lang="de">Wirtschaftszeitschrift</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24662-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70957">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11073-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19586-5">
    <skos:related rdf:resource="descriptor/19587-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19586-5">
    <skos:related rdf:resource="descriptor/10008-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13706-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18365-5">
    <skos:related rdf:resource="descriptor/16048-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19005-2">
    <skos:prefLabel xml:lang="en">Job tenure</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19005-2">
    <skos:prefLabel xml:lang="de">Betriebszugehörigkeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20468-1">
    <skos:broader rdf:resource="thsys/70485"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20468-1">
    <skos:broader rdf:resource="descriptor/12083-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70944">
    <skos:prefLabel xml:lang="en">W.11  Hospitality industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70944">
    <skos:prefLabel xml:lang="de">W.11  Gastgewerbe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18896-2">
    <skos:broader rdf:resource="thsys/73384"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18896-2">
    <skos:broader rdf:resource="thsys/73346"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18896-2">
    <skos:broader rdf:resource="descriptor/16676-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19002-1">
    <skos:prefLabel xml:lang="en">Monitoring</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19002-1">
    <skos:prefLabel xml:lang="de">Leistungskontrolle</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10523-4">
    <skos:narrower rdf:resource="descriptor/19368-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10523-4">
    <skos:narrower rdf:resource="descriptor/10561-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10523-4">
    <skos:narrower rdf:resource="descriptor/10559-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15786-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13762-6">
    <skos:altLabel xml:lang="en">Repurchase agreement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13762-6">
    <skos:altLabel xml:lang="de">Wertpapierpensionsgeschäft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13762-6">
    <skos:altLabel xml:lang="de">Wechselpensionsgeschäft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13762-6">
    <skos:altLabel xml:lang="de">Repo-Geschäft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13762-6">
    <skos:altLabel xml:lang="de">Effektenpensionsgeschäft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13762-6">
    <skos:altLabel xml:lang="de">Devisenpensionsgeschäft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19834-5">
    <skos:altLabel xml:lang="en">Scorecard</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19834-5">
    <skos:altLabel xml:lang="en">BSC (Balanced Scorecard)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15392-3">
    <skos:prefLabel xml:lang="en">Seasonality (Statistics)</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15392-3">
    <skos:prefLabel xml:lang="de">Saisonbereinigung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70429">
    <skos:broader rdf:resource="thsys/70354"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13493-0">
    <skos:related rdf:resource="descriptor/13490-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73394">
    <skos:narrower rdf:resource="descriptor/19734-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73394">
    <skos:narrower rdf:resource="descriptor/18130-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73394">
    <skos:narrower rdf:resource="descriptor/11111-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73394">
    <skos:narrower rdf:resource="descriptor/19733-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73394">
    <skos:narrower rdf:resource="descriptor/16229-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73394">
    <skos:narrower rdf:resource="descriptor/10166-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73394">
    <skos:narrower rdf:resource="descriptor/18116-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73394">
    <skos:narrower rdf:resource="descriptor/19732-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73394">
    <skos:narrower rdf:resource="descriptor/16724-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73394">
    <skos:narrower rdf:resource="descriptor/16723-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73394">
    <skos:narrower rdf:resource="descriptor/16544-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73394">
    <skos:narrower rdf:resource="descriptor/16722-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73394">
    <skos:narrower rdf:resource="descriptor/19731-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73394">
    <skos:narrower rdf:resource="descriptor/15969-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73394">
    <skos:narrower rdf:resource="descriptor/16721-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73394">
    <skos:narrower rdf:resource="descriptor/19750-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73394">
    <skos:narrower rdf:resource="descriptor/16430-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73394">
    <skos:narrower rdf:resource="descriptor/16719-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73394">
    <skos:narrower rdf:resource="descriptor/15965-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73394">
    <skos:narrower rdf:resource="descriptor/16717-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73394">
    <skos:narrower rdf:resource="descriptor/11088-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73394">
    <skos:narrower rdf:resource="descriptor/19730-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73394">
    <skos:narrower rdf:resource="descriptor/16405-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73394">
    <skos:narrower rdf:resource="descriptor/16713-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73394">
    <skos:narrower rdf:resource="descriptor/19768-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73394">
    <skos:narrower rdf:resource="descriptor/16426-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73394">
    <skos:narrower rdf:resource="descriptor/16709-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73394">
    <skos:narrower rdf:resource="descriptor/16708-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73394">
    <skos:narrower rdf:resource="descriptor/16704-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73394">
    <skos:narrower rdf:resource="descriptor/19747-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73394">
    <skos:narrower rdf:resource="descriptor/11181-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16853-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13191-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10291-6">
    <skos:prefLabel xml:lang="en">Investment function</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10291-6">
    <skos:prefLabel xml:lang="de">Investitionsfunktion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18111-1">
    <skos:prefLabel xml:lang="en">Screen work</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18111-1">
    <skos:prefLabel xml:lang="de">Bildschirmarbeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12730-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70477">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">P.10.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10016-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11954-0">
    <skos:prefLabel xml:lang="en">Economic base model</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11954-0">
    <skos:prefLabel xml:lang="de">Exportbasistheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18065-3">
    <skos:broader rdf:resource="thsys/73391"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18065-3">
    <skos:broader rdf:resource="thsys/71109"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18065-3">
    <skos:broader rdf:resource="thsys/70876"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18065-3">
    <skos:broader rdf:resource="descriptor/16053-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19268-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14642-4">
    <skos:altLabel xml:lang="de">Salzsäure</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14642-4">
    <skos:altLabel xml:lang="de">Metallsalz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14642-4">
    <skos:altLabel xml:lang="de">Metalloxid</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14642-4">
    <skos:altLabel xml:lang="de">Kohlensäure</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18019-3">
    <skos:prefLabel xml:lang="en">Pacific rim</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18019-3">
    <skos:prefLabel xml:lang="de">Pazifischer Raum</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73383">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19044-6">
    <skos:scopeNote xml:lang="de">Nicht im juristischen Sinne.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14960-4">
    <skos:prefLabel xml:lang="en">Olive oil</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14960-4">
    <skos:prefLabel xml:lang="de">Olivenöl</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16830-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13518-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19842-6">
    <skos:broader rdf:resource="thsys/70160"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19842-6">
    <skos:broader rdf:resource="descriptor/16041-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18104-5">
    <skos:related rdf:resource="descriptor/12665-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18104-5">
    <skos:related rdf:resource="descriptor/12656-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17793-3">
    <skos:broader rdf:resource="thsys/70086"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17793-3">
    <skos:broader rdf:resource="descriptor/17791-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19724-5">
    <skos:altLabel xml:lang="de">Nachschuborganisation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19724-5">
    <skos:altLabel xml:lang="de">Militärischer Transport</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19724-5">
    <skos:altLabel xml:lang="de">Militärischer Nachschub</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19724-5">
    <skos:altLabel xml:lang="de">Militärische Materialwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19724-5">
    <skos:altLabel xml:lang="de">Militärische Lagerhaltung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14431-3">
    <skos:broader rdf:resource="thsys/70257"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14431-3">
    <skos:broader rdf:resource="descriptor/13513-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10247-2">
    <skos:broader rdf:resource="thsys/70141"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10247-2">
    <skos:broader rdf:resource="descriptor/16745-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13272-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16277-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14834-2">
    <skos:prefLabel xml:lang="en">Sports footwear</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14834-2">
    <skos:prefLabel xml:lang="de">Sportschuhe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11659-2">
    <skos:altLabel xml:lang="en">Land tax</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11659-2">
    <skos:altLabel xml:lang="de">Zweitwohnungsteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11659-2">
    <skos:altLabel xml:lang="de">Zweitwohnungssteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11659-2">
    <skos:altLabel xml:lang="de">Immobiliensteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11659-2">
    <skos:altLabel xml:lang="de">Gebäudesteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11659-2">
    <skos:altLabel xml:lang="de">Bodensteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11659-2">
    <skos:altLabel xml:lang="de">Baulandsteuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19274-3">
    <skos:scopeNote xml:lang="de">Äquivalent für das Eingehen besonderer Risiken.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10719-1">
    <skos:prefLabel xml:lang="en">Foreign exchange control</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10719-1">
    <skos:prefLabel xml:lang="de">Devisenkontrolle</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10114-0">
    <skos:altLabel xml:lang="de">X-Ineffizienz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12602-5">
    <skos:related rdf:resource="descriptor/15684-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12602-5">
    <skos:related rdf:resource="descriptor/16692-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12602-5">
    <skos:related rdf:resource="descriptor/12603-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12602-5">
    <skos:related rdf:resource="descriptor/11185-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12602-5">
    <skos:related rdf:resource="descriptor/12621-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12602-5">
    <skos:related rdf:resource="descriptor/11181-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12602-5">
    <skos:related rdf:resource="descriptor/11305-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12602-5">
    <skos:related rdf:resource="descriptor/11180-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10725-6">
    <skos:related rdf:resource="descriptor/19321-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10725-6">
    <skos:related rdf:resource="descriptor/10814-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10725-6">
    <skos:related rdf:resource="descriptor/10811-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18252-4">
    <skos:broader rdf:resource="thsys/73344"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18252-4">
    <skos:broader rdf:resource="thsys/70201"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18252-4">
    <skos:broader rdf:resource="descriptor/14042-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13565-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10962-6">
    <skos:altLabel xml:lang="de">Wettbewerbsbehörde</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10962-6">
    <skos:altLabel xml:lang="de">Missbrauchsaufsicht des Kartellrechts</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10962-6">
    <skos:altLabel xml:lang="de">Mißbrauchsaufsicht des Kartellrechts</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10962-6">
    <skos:altLabel xml:lang="de">Kartellrechtliche Missbrauchsaufsicht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10962-6">
    <skos:altLabel xml:lang="de">Kartellrechtliche Mißbrauchsaufsicht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10962-6">
    <skos:altLabel xml:lang="de">Kartellbehörde</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10962-6">
    <skos:altLabel xml:lang="de">Kartellaufsicht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10962-6">
    <skos:altLabel xml:lang="de">Kartellamt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70957">
    <skos:prefLabel xml:lang="de">W.10.01.04  Rohstoff- und Altmaterialhandel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19021-4">
    <skos:broader rdf:resource="thsys/73302"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19021-4">
    <skos:broader rdf:resource="descriptor/14122-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19198-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19323-2">
    <skos:prefLabel xml:lang="en">Freedom of establishment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19323-2">
    <skos:prefLabel xml:lang="de">Niederlassungsrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15470-2">
    <skos:prefLabel xml:lang="en">Decision theory</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15470-2">
    <skos:prefLabel xml:lang="de">Entscheidungstheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id231">
    <rdf:value xml:lang="de">Für nichteheliche Lebensgemeinschaften</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16722-3">
    <skos:related rdf:resource="descriptor/19268-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11611-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19793-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10224-0">
    <skos:narrower rdf:resource="descriptor/19226-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18868-0">
    <skos:broader rdf:resource="thsys/70143"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18868-0">
    <skos:broader rdf:resource="descriptor/15666-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18096-6">
    <skos:broader rdf:resource="thsys/71028"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18096-6">
    <skos:broader rdf:resource="descriptor/12922-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19219-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10785-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18731-6">
    <skos:prefLabel xml:lang="en">Electronic trading</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18731-6">
    <skos:prefLabel xml:lang="de">Börsen-Informationssystem</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10095-6">
    <skos:related rdf:resource="descriptor/10112-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10095-6">
    <skos:related rdf:resource="descriptor/10106-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18438-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11427-2">
    <skos:prefLabel xml:lang="en">Value of money</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11427-2">
    <skos:prefLabel xml:lang="de">Geldwert</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10928-6">
    <skos:altLabel xml:lang="en">Entry deterrence</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10928-6">
    <skos:altLabel xml:lang="en">Barrier to entry</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10928-6">
    <skos:altLabel xml:lang="de">Marktzutrittsschranke</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10928-6">
    <skos:altLabel xml:lang="de">Marktzutritt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10928-6">
    <skos:altLabel xml:lang="de">Marktzugang</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10928-6">
    <skos:altLabel xml:lang="de">Markteintrittsstrategie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10928-6">
    <skos:altLabel xml:lang="de">Markteintrittsschranke</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19725-3">
    <skos:narrower rdf:resource="descriptor/18817-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19725-3">
    <skos:narrower rdf:resource="descriptor/18814-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19725-3">
    <skos:narrower rdf:resource="descriptor/16684-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19725-3">
    <skos:narrower rdf:resource="descriptor/18815-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10865-4">
    <skos:related rdf:resource="descriptor/19399-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10865-4">
    <skos:related rdf:resource="descriptor/10866-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10865-4">
    <skos:related rdf:resource="descriptor/10836-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18514-0">
    <skos:prefLabel xml:lang="en">Used car trade</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18514-0">
    <skos:prefLabel xml:lang="de">Gebrauchtfahrzeughandel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16140-4">
    <skos:related rdf:resource="descriptor/11201-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16140-4">
    <skos:related rdf:resource="descriptor/16481-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16140-4">
    <skos:related rdf:resource="descriptor/11275-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16140-4">
    <skos:related rdf:resource="descriptor/19509-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16140-4">
    <skos:related rdf:resource="descriptor/16134-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10513-0">
    <zbwext:useInsteadNote rdf:nodeID="id85"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13674-3">
    <skos:related rdf:resource="descriptor/18647-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13674-3">
    <skos:related rdf:resource="descriptor/19658-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13674-3">
    <skos:related rdf:resource="descriptor/19653-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17608-3">
    <skos:narrower rdf:resource="descriptor/17641-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17608-3">
    <skos:narrower rdf:resource="descriptor/17639-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17608-3">
    <skos:narrower rdf:resource="descriptor/17638-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17608-3">
    <skos:narrower rdf:resource="descriptor/17636-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17608-3">
    <skos:narrower rdf:resource="descriptor/17634-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17608-3">
    <skos:narrower rdf:resource="descriptor/18285-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17608-3">
    <skos:narrower rdf:resource="descriptor/17632-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17608-3">
    <skos:narrower rdf:resource="descriptor/17630-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17608-3">
    <skos:narrower rdf:resource="descriptor/17628-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17608-3">
    <skos:narrower rdf:resource="descriptor/17627-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17608-3">
    <skos:narrower rdf:resource="descriptor/17626-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17608-3">
    <skos:narrower rdf:resource="descriptor/17624-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17608-3">
    <skos:narrower rdf:resource="descriptor/17623-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17608-3">
    <skos:narrower rdf:resource="descriptor/17622-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17608-3">
    <skos:narrower rdf:resource="descriptor/17621-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17608-3">
    <skos:narrower rdf:resource="descriptor/17618-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17608-3">
    <skos:narrower rdf:resource="descriptor/17596-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12134-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18955-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18350-4">
    <skos:prefLabel xml:lang="en">Energy economics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18350-4">
    <skos:prefLabel xml:lang="de">Energieökonomik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10641-5">
    <skos:prefLabel xml:lang="en">Comparative advantage</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10641-5">
    <skos:prefLabel xml:lang="de">Komparativer Kostenvorteil</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14607-6">
    <skos:broader rdf:resource="thsys/70014"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14607-6">
    <skos:broader rdf:resource="descriptor/14599-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19854-6">
    <skos:related rdf:resource="descriptor/19687-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19854-6">
    <skos:related rdf:resource="descriptor/18637-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19854-6">
    <skos:related rdf:resource="descriptor/13641-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19854-6">
    <skos:related rdf:resource="descriptor/15994-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19854-6">
    <skos:related rdf:resource="descriptor/15989-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19381-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16311-3">
    <skos:altLabel xml:lang="de">Volksinitiative</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16311-3">
    <skos:altLabel xml:lang="de">Volksentscheid</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16311-3">
    <skos:altLabel xml:lang="de">Volksbegehren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16311-3">
    <skos:altLabel xml:lang="de">Volksabstimmung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16311-3">
    <skos:altLabel xml:lang="de">Plebiszit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18564-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18644-1">
    <skos:broader rdf:resource="thsys/70923"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19354-5">
    <skos:broader rdf:resource="thsys/71092"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19354-5">
    <skos:broader rdf:resource="descriptor/19336-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13475-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#hiddenLabel">
    <rdfs:label xml:lang="en">hidden label</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14854-3">
    <skos:altLabel xml:lang="en">Synthetic fibre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14854-3">
    <skos:altLabel xml:lang="de">Synthetisches Filament</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14854-3">
    <skos:altLabel xml:lang="de">Synthesefaser</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14854-3">
    <skos:altLabel xml:lang="de">Polypropylenfaser</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14854-3">
    <skos:altLabel xml:lang="de">Polyolefinfaser</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14854-3">
    <skos:altLabel xml:lang="de">Polyethylenfaser</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14854-3">
    <skos:altLabel xml:lang="de">Polyesterfaser</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14854-3">
    <skos:altLabel xml:lang="de">Polyacrylfaser</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14854-3">
    <skos:altLabel xml:lang="de">Kunstfaser</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14854-3">
    <skos:altLabel xml:lang="de">Bikomponentenfaser</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73335">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70868">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11034-0">
    <skos:broader rdf:resource="thsys/71101"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10469-5">
    <skos:broader rdf:resource="thsys/73382"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10469-5">
    <skos:broader rdf:resource="thsys/71109"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10469-5">
    <skos:broader rdf:resource="descriptor/16292-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10404-5">
    <skos:prefLabel xml:lang="en">Tax-based incomes policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10404-5">
    <skos:prefLabel xml:lang="de">Tax-Based Incomes Policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14511-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17596-5">
    <skos:altLabel xml:lang="de">Republik Afghanistan</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17596-5">
    <skos:altLabel xml:lang="de">Islamischer Staat Afghanistan</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17596-5">
    <skos:altLabel xml:lang="de">Demokratische Republik Afghanistan</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17799-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11556-5">
    <skos:broader rdf:resource="thsys/71081"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11556-5">
    <skos:broader rdf:resource="thsys/71072"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11556-5">
    <skos:broader rdf:resource="thsys/70860"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11556-5">
    <skos:broader rdf:resource="descriptor/11722-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11556-5">
    <skos:broader rdf:resource="descriptor/11554-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11556-5">
    <skos:broader rdf:resource="descriptor/10008-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14068-3">
    <skos:broader rdf:resource="thsys/70498"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14068-3">
    <skos:broader rdf:resource="thsys/70341"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14068-3">
    <skos:broader rdf:resource="descriptor/14073-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18241-2">
    <skos:related rdf:resource="descriptor/18595-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18241-2">
    <skos:related rdf:resource="descriptor/18242-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18241-2">
    <skos:related rdf:resource="descriptor/14318-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18666-5">
    <skos:altLabel xml:lang="en">3G mobile telecommunications</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18666-5">
    <skos:altLabel xml:lang="en">Telepoint-system</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18666-5">
    <skos:altLabel xml:lang="en">Mobile communication</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18666-5">
    <skos:altLabel xml:lang="de">Zellularer Mobilfunk</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18666-5">
    <skos:altLabel xml:lang="de">UMTS</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18666-5">
    <skos:altLabel xml:lang="de">Mobilfunk</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18666-5">
    <skos:altLabel xml:lang="de">Mobile Datenkommunikation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18666-5">
    <skos:altLabel xml:lang="de">Digitaler Mobilfunk</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18014-6">
    <skos:broader rdf:resource="thsys/71023"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18014-6">
    <skos:broader rdf:resource="thsys/71020"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18014-6">
    <skos:broader rdf:resource="descriptor/11804-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18507-4">
    <skos:related rdf:resource="descriptor/11578-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18507-4">
    <skos:related rdf:resource="descriptor/12905-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18507-4">
    <skos:related rdf:resource="descriptor/16176-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18507-4">
    <skos:related rdf:resource="descriptor/18491-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11870-6">
    <skos:altLabel xml:lang="de">Raumplanungsrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11870-6">
    <skos:altLabel xml:lang="de">Raumordnungsgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11870-6">
    <skos:altLabel xml:lang="de">Landesplanungsrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11870-6">
    <skos:altLabel xml:lang="de">Bundesraumordnungsgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15062-3">
    <skos:altLabel xml:lang="en">Robust test</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15062-3">
    <skos:altLabel xml:lang="en">Robust statistics</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15062-3">
    <skos:altLabel xml:lang="en">Robust estimation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15062-3">
    <skos:altLabel xml:lang="de">Robustheit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15062-3">
    <skos:altLabel xml:lang="de">Robuster Test</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15062-3">
    <skos:altLabel xml:lang="de">Robuste Schätzung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11355-1">
    <skos:prefLabel xml:lang="en">Graduates</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11355-1">
    <skos:prefLabel xml:lang="de">Absolventen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10219-0">
    <skos:related rdf:resource="descriptor/19224-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10219-0">
    <skos:related rdf:resource="descriptor/10215-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16842-0">
    <skos:prefLabel xml:lang="en">Bavaria</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16842-0">
    <skos:prefLabel xml:lang="de">Bayern</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19704-4">
    <skos:prefLabel xml:lang="en">Criminal policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19704-4">
    <skos:prefLabel xml:lang="de">Kriminalpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24855-2">
    <skos:altLabel xml:lang="en">Exchange rate overshooting</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24855-2">
    <skos:altLabel xml:lang="de">Wechselkurs-Overshooting</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16745-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15739-5">
    <skos:related rdf:resource="descriptor/15740-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11459-3">
    <skos:broader rdf:resource="thsys/70305"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11459-3">
    <skos:broader rdf:resource="thsys/70259"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11459-3">
    <skos:broader rdf:resource="descriptor/10082-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11459-3">
    <skos:broader rdf:resource="descriptor/18696-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11346-2">
    <skos:prefLabel xml:lang="en">Wage bargaining</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11346-2">
    <skos:prefLabel xml:lang="de">Lohnverhandlungen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19608-0">
    <skos:broader rdf:resource="thsys/70580"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19608-0">
    <skos:broader rdf:resource="descriptor/15644-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16541-0">
    <skos:broader rdf:resource="thsys/70839"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16541-0">
    <skos:broader rdf:resource="thsys/70573"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16541-0">
    <skos:broader rdf:resource="descriptor/16479-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16541-0">
    <skos:broader rdf:resource="descriptor/16529-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13988-1">
    <skos:altLabel xml:lang="de">Verbandssekretärinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13988-1">
    <skos:altLabel xml:lang="de">Verbandssekretäre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13988-1">
    <skos:altLabel xml:lang="de">Verbandsleiterinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13988-1">
    <skos:altLabel xml:lang="de">Verbandsleiter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13988-1">
    <skos:altLabel xml:lang="de">Verbandsfunktionärinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13988-1">
    <skos:altLabel xml:lang="de">Verbandsfunktionäre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13988-1">
    <skos:altLabel xml:lang="de">Sozialistische Funktionärinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13988-1">
    <skos:altLabel xml:lang="de">Sozialistische Funktionäre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13988-1">
    <skos:altLabel xml:lang="de">Parteikader</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13988-1">
    <skos:altLabel xml:lang="de">Parteifunktionärinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13988-1">
    <skos:altLabel xml:lang="de">Parteifunktionäre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13988-1">
    <skos:altLabel xml:lang="de">Kader</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13988-1">
    <skos:altLabel xml:lang="de">Funktionärinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18616-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19365-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13576-3">
    <skos:broader rdf:resource="thsys/70936"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13576-3">
    <skos:broader rdf:resource="descriptor/10470-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11479-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19275-1">
    <skos:related rdf:resource="descriptor/12247-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19275-1">
    <skos:related rdf:resource="descriptor/19276-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12291-3">
    <skos:altLabel xml:lang="de">Pensionsverpflichtungen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12291-3">
    <skos:altLabel xml:lang="de">Pensionsverbindlichkeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10182-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12940-6">
    <skos:broader rdf:resource="thsys/71026"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12940-6">
    <skos:broader rdf:resource="descriptor/12949-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12609-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18367-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70935">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12172-4">
    <skos:broader rdf:resource="thsys/73339"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12172-4">
    <skos:broader rdf:resource="thsys/71039"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12172-4">
    <skos:broader rdf:resource="descriptor/15609-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12172-4">
    <skos:broader rdf:resource="descriptor/12041-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19625-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18834-3">
    <skos:altLabel xml:lang="en">Non-profit management</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70504">
    <skos:narrower rdf:resource="descriptor/19830-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70504">
    <skos:narrower rdf:resource="descriptor/18905-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70504">
    <skos:narrower rdf:resource="descriptor/19474-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70504">
    <skos:narrower rdf:resource="descriptor/15814-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70504">
    <skos:narrower rdf:resource="descriptor/16597-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70504">
    <skos:narrower rdf:resource="descriptor/18887-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70504">
    <skos:narrower rdf:resource="descriptor/19507-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70504">
    <skos:narrower rdf:resource="descriptor/15723-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70504">
    <skos:narrower rdf:resource="descriptor/16544-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70504">
    <skos:narrower rdf:resource="descriptor/15073-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70504">
    <skos:narrower rdf:resource="descriptor/15715-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70504">
    <skos:narrower rdf:resource="descriptor/16639-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70504">
    <skos:narrower rdf:resource="descriptor/16638-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70504">
    <skos:narrower rdf:resource="descriptor/15069-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70504">
    <skos:narrower rdf:resource="descriptor/15088-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70504">
    <skos:narrower rdf:resource="descriptor/19362-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70504">
    <skos:narrower rdf:resource="descriptor/19840-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70504">
    <skos:narrower rdf:resource="descriptor/10498-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70504">
    <skos:narrower rdf:resource="descriptor/19506-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70504">
    <skos:narrower rdf:resource="descriptor/19521-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70504">
    <skos:narrower rdf:resource="descriptor/16401-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70504">
    <skos:narrower rdf:resource="descriptor/11037-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70504">
    <skos:narrower rdf:resource="descriptor/10488-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70504">
    <skos:narrower rdf:resource="thsys/70324"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70504">
    <skos:narrower rdf:resource="thsys/70458"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10984-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18197-0">
    <skos:prefLabel xml:lang="en">Rice production</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18197-0">
    <skos:prefLabel xml:lang="de">Reisanbau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11212-2">
    <skos:narrower rdf:resource="descriptor/11199-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71008">
    <skos:narrower rdf:resource="descriptor/13024-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71008">
    <skos:narrower rdf:resource="descriptor/13023-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71008">
    <skos:narrower rdf:resource="descriptor/13020-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71008">
    <skos:narrower rdf:resource="descriptor/13014-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71008">
    <skos:narrower rdf:resource="descriptor/13019-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71008">
    <skos:narrower rdf:resource="descriptor/13011-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71008">
    <skos:narrower rdf:resource="descriptor/13018-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71008">
    <skos:narrower rdf:resource="descriptor/13010-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16354-6">
    <skos:narrower rdf:resource="descriptor/19702-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16354-6">
    <skos:narrower rdf:resource="descriptor/19698-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16354-6">
    <skos:narrower rdf:resource="descriptor/19697-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16354-6">
    <skos:narrower rdf:resource="descriptor/16349-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16354-6">
    <skos:narrower rdf:resource="descriptor/19110-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16606-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18362-4">
    <skos:scopeNote xml:lang="de">Politik zur Förderung regenerativer Energien.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14418-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18906-4">
    <skos:related rdf:resource="descriptor/19830-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18906-4">
    <skos:related rdf:resource="descriptor/13405-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18906-4">
    <skos:related rdf:resource="descriptor/16543-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18906-4">
    <skos:related rdf:resource="descriptor/14014-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18906-4">
    <skos:related rdf:resource="descriptor/18803-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10184-0">
    <zbwext:useInsteadNote rdf:nodeID="id385"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10207-0">
    <skos:prefLabel xml:lang="en">Moral hazard</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10207-0">
    <skos:prefLabel xml:lang="de">Moral Hazard</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19533-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19333-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16330-6">
    <skos:prefLabel xml:lang="en">Head of government</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16330-6">
    <skos:prefLabel xml:lang="de">Regierungschef</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71062">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10715-2">
    <skos:broader rdf:resource="thsys/71096"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11652-2">
    <skos:broader rdf:resource="thsys/71074"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11652-2">
    <skos:broader rdf:resource="thsys/70397"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11652-2">
    <skos:broader rdf:resource="descriptor/11547-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13209-2">
    <skos:broader rdf:resource="thsys/70953"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13209-2">
    <skos:broader rdf:resource="descriptor/13210-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18666-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11629-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16438-0">
    <skos:prefLabel xml:lang="en">International trade union</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16438-0">
    <skos:prefLabel xml:lang="de">Internationale Gewerkschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18320-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15807-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10738-4">
    <skos:broader rdf:resource="thsys/71095"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10738-4">
    <skos:broader rdf:resource="descriptor/10764-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14938-4">
    <skos:broader rdf:resource="thsys/70498"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14938-4">
    <skos:broader rdf:resource="descriptor/14936-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11519-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13724-0">
    <skos:broader rdf:resource="thsys/71107"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13724-0">
    <skos:broader rdf:resource="thsys/70893"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13724-0">
    <skos:broader rdf:resource="descriptor/13742-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17568-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11324-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73375">
    <skos:prefLabel xml:lang="de">N.04.02.07  Politische Prozesse und politischer Wandel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70028">
    <rdfs:label xml:lang="de">Unternehmensführung und Organisation</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13454-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16595-5">
    <skos:scopeNote xml:lang="de">Für Systeme der sozialen Sicherung BENUTZE Soziale Sicherung oder Sozialversicherung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16595-5">
    <skos:scopeNote xml:lang="de">Nur im Sinne von Beziehungsnetzwerken.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71005">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13622-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11544-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id95">
    <rdfs:seeAlso rdf:resource="descriptor/18882-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10398-2">
    <skos:altLabel xml:lang="en">Formula flexibility</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10398-2">
    <skos:altLabel xml:lang="de">Formularflexibilität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10398-2">
    <skos:altLabel xml:lang="de">Formelflexibilität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15163-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11738-6">
    <skos:prefLabel xml:lang="en">Gini coefficient</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11738-6">
    <skos:prefLabel xml:lang="de">Gini-Koeffizient</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18013-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15869-6">
    <skos:prefLabel xml:lang="en">Mountain area</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15869-6">
    <skos:prefLabel xml:lang="de">Gebirge</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16510-4">
    <skos:scopeNote xml:lang="de">Sicherheitsmaßnahmen gegen innere Subversion, Terrorismus, Aufstände, etc.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18338-1">
    <skos:narrower rdf:resource="descriptor/19419-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16439-5">
    <skos:prefLabel xml:lang="en">International organization</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16439-5">
    <skos:prefLabel xml:lang="de">Internationale Organisation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10465-6">
    <skos:broader rdf:resource="thsys/73387"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10465-6">
    <skos:broader rdf:resource="thsys/73345"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10465-6">
    <skos:broader rdf:resource="thsys/71109"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10465-6">
    <skos:broader rdf:resource="thsys/70109"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10465-6">
    <skos:broader rdf:resource="descriptor/15690-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70306">
    <skos:narrower rdf:resource="descriptor/10263-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70306">
    <skos:narrower rdf:resource="descriptor/19259-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70306">
    <skos:narrower rdf:resource="descriptor/11474-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70306">
    <skos:narrower rdf:resource="descriptor/11473-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70306">
    <skos:narrower rdf:resource="descriptor/11469-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70306">
    <skos:narrower rdf:resource="descriptor/10250-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70306">
    <skos:narrower rdf:resource="descriptor/11464-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70306">
    <skos:narrower rdf:resource="descriptor/11457-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70306">
    <skos:narrower rdf:resource="descriptor/19258-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70306">
    <skos:narrower rdf:resource="descriptor/11454-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70306">
    <skos:narrower rdf:resource="descriptor/11452-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70306">
    <skos:narrower rdf:resource="descriptor/10236-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70306">
    <skos:narrower rdf:resource="descriptor/11451-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70306">
    <skos:narrower rdf:resource="descriptor/11449-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70306">
    <skos:narrower rdf:resource="descriptor/11447-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70306">
    <skos:narrower rdf:resource="descriptor/11444-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70306">
    <skos:narrower rdf:resource="descriptor/19257-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70306">
    <skos:narrower rdf:resource="descriptor/19256-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70306">
    <skos:narrower rdf:resource="descriptor/11443-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14441-0">
    <skos:related rdf:resource="descriptor/13563-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14441-0">
    <skos:related rdf:resource="descriptor/13065-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11902-5">
    <skos:prefLabel xml:lang="en">Exodus from the cities</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11902-5">
    <skos:prefLabel xml:lang="de">Stadtflucht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70465">
    <skos:prefLabel xml:lang="de">P.02  Forstprodukte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19075-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13259-1">
    <skos:broader rdf:resource="thsys/70943"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13259-1">
    <skos:broader rdf:resource="descriptor/13510-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16603-4">
    <skos:prefLabel xml:lang="en">Industrial society</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16603-4">
    <skos:prefLabel xml:lang="de">Industriegesellschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14397-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19465-3">
    <skos:related rdf:resource="descriptor/18895-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19465-3">
    <skos:related rdf:resource="descriptor/19463-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19465-3">
    <skos:related rdf:resource="descriptor/18894-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18814-2">
    <skos:related rdf:resource="descriptor/18366-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18814-2">
    <skos:related rdf:resource="descriptor/18447-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18814-2">
    <skos:related rdf:resource="descriptor/12925-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18814-2">
    <skos:related rdf:resource="descriptor/15005-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18814-2">
    <skos:related rdf:resource="descriptor/16271-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18814-2">
    <skos:related rdf:resource="descriptor/18418-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18814-2">
    <skos:related rdf:resource="descriptor/15628-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11849-4">
    <skos:prefLabel xml:lang="en">Regional decentralization</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11849-4">
    <skos:prefLabel xml:lang="de">Regionale Dezentralisierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71013">
    <rdfs:label xml:lang="de">Kernenergiewirtschaft</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13445-4">
    <skos:broader rdf:resource="thsys/73355"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13445-4">
    <skos:broader rdf:resource="thsys/71065"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13445-4">
    <skos:broader rdf:resource="thsys/71028"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13445-4">
    <skos:broader rdf:resource="descriptor/16564-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13445-4">
    <skos:broader rdf:resource="descriptor/13446-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11834-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16670-3">
    <skos:altLabel xml:lang="de">Gesundheitsschaden</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14329-1">
    <skos:broader rdf:resource="thsys/70549"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14329-1">
    <skos:broader rdf:resource="descriptor/14336-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12754-3">
    <skos:prefLabel xml:lang="en">Capacity planning</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12754-3">
    <skos:prefLabel xml:lang="de">Kapazitätsplanung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14514-6">
    <skos:narrower rdf:resource="descriptor/14614-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14514-6">
    <skos:narrower rdf:resource="descriptor/18075-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70829">
    <skos:narrower rdf:resource="descriptor/18918-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70829">
    <skos:narrower rdf:resource="descriptor/18915-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70829">
    <skos:narrower rdf:resource="descriptor/13354-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70829">
    <skos:narrower rdf:resource="descriptor/12907-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70829">
    <skos:narrower rdf:resource="descriptor/18916-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70829">
    <skos:narrower rdf:resource="descriptor/18917-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70829">
    <skos:narrower rdf:resource="descriptor/16679-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70829">
    <skos:narrower rdf:resource="descriptor/11194-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70829">
    <skos:narrower rdf:resource="descriptor/13320-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70829">
    <skos:narrower rdf:resource="descriptor/18556-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20522-0">
    <skos:narrower rdf:resource="descriptor/10893-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20522-0">
    <skos:narrower rdf:resource="descriptor/19433-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12265-4">
    <skos:prefLabel xml:lang="en">Equity offering</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12265-4">
    <skos:prefLabel xml:lang="de">Wertpapieremission</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13697-5">
    <skos:broader rdf:resource="thsys/70903"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13697-5">
    <skos:broader rdf:resource="descriptor/13283-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19621-1">
    <skos:scopeNote xml:lang="de">Nur für natürliches Eis.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12531-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70201">
    <skos:prefLabel xml:lang="de">P.00  Produkte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18029-0">
    <skos:altLabel xml:lang="de">Strategische Unternehmensführung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73336">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.05.06</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11506-6">
    <skos:narrower rdf:resource="descriptor/11523-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11506-6">
    <skos:narrower rdf:resource="descriptor/10412-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11506-6">
    <skos:narrower rdf:resource="descriptor/10667-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11506-6">
    <skos:narrower rdf:resource="descriptor/11503-5"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id236">
    <rdf:value xml:lang="de">für Länder oder Regionen mit niedrigem Pro-Kopf-Einkommen</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14001-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71019">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19677-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12164-3">
    <skos:related rdf:resource="descriptor/12171-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12164-3">
    <skos:related rdf:resource="descriptor/18767-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12164-3">
    <skos:related rdf:resource="descriptor/12039-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12164-3">
    <skos:related rdf:resource="descriptor/12053-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12164-3">
    <skos:related rdf:resource="descriptor/20404-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12164-3">
    <skos:related rdf:resource="descriptor/13341-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18736-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15683-6">
    <skos:broader rdf:resource="thsys/70819"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15683-6">
    <skos:broader rdf:resource="thsys/70581"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15683-6">
    <skos:broader rdf:resource="descriptor/15675-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16535-2">
    <skos:broader rdf:resource="thsys/70834"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16535-2">
    <skos:broader rdf:resource="descriptor/16480-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19373-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11184-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17652-0">
    <skos:broader rdf:resource="thsys/70354"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17652-0">
    <skos:broader rdf:resource="descriptor/17642-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14020-3">
    <skos:altLabel xml:lang="de">Tür</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14020-3">
    <skos:altLabel xml:lang="de">Fertigbauteil</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14020-3">
    <skos:altLabel xml:lang="de">Fenster</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14020-3">
    <skos:altLabel xml:lang="de">Bauelement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11570-4">
    <skos:prefLabel xml:lang="en">Social transfer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11570-4">
    <skos:prefLabel xml:lang="de">Sozialtransfer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15940-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10076-3">
    <skos:prefLabel xml:lang="en">Equilibrium model</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10076-3">
    <skos:prefLabel xml:lang="de">Gleichgewichtstheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16712-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19717-2">
    <skos:prefLabel xml:lang="en">Colonial policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19717-2">
    <skos:prefLabel xml:lang="de">Kolonialpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11068-4">
    <skos:narrower rdf:resource="descriptor/10986-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11068-4">
    <skos:narrower rdf:resource="descriptor/11064-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11068-4">
    <skos:narrower rdf:resource="descriptor/10511-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11068-4">
    <skos:narrower rdf:resource="descriptor/12064-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11068-4">
    <skos:narrower rdf:resource="descriptor/11061-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11068-4">
    <skos:narrower rdf:resource="descriptor/11055-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11068-4">
    <skos:narrower rdf:resource="descriptor/19288-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11068-4">
    <skos:narrower rdf:resource="descriptor/11046-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11068-4">
    <skos:narrower rdf:resource="descriptor/11043-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11068-4">
    <skos:narrower rdf:resource="descriptor/11825-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11068-4">
    <skos:narrower rdf:resource="descriptor/11042-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11068-4">
    <skos:narrower rdf:resource="descriptor/19287-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11068-4">
    <skos:narrower rdf:resource="descriptor/19286-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11787-0">
    <skos:altLabel xml:lang="de">Gesamtwirtschaftliche Produktpalette</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11787-0">
    <skos:altLabel xml:lang="de">Angebotsstruktur der Volkswirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11787-0">
    <skos:altLabel xml:lang="de">Angebotsstruktur der Gesamtwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19242-2">
    <skos:altLabel xml:lang="en">Growth convergence</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19242-2">
    <skos:altLabel xml:lang="en">Catching-up</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19242-2">
    <skos:altLabel xml:lang="en">Catching up</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19242-2">
    <skos:altLabel xml:lang="de">Wirtschaftlicher Aufholprozess</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19242-2">
    <skos:altLabel xml:lang="de">Wirtschaftlicher Aufholprozeß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19242-2">
    <skos:altLabel xml:lang="de">Wirtschaftliche Konvergenz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19242-2">
    <skos:altLabel xml:lang="de">Wachstumskonvergenz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19242-2">
    <skos:altLabel xml:lang="de">Ökonomische Konvergenz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19242-2">
    <skos:altLabel xml:lang="de">Konvergenz der Entwicklung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19242-2">
    <skos:altLabel xml:lang="de">Aufholprozess</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19242-2">
    <skos:altLabel xml:lang="de">Aufholprozeß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15616-0">
    <skos:prefLabel xml:lang="en">Computer operations</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15616-0">
    <skos:prefLabel xml:lang="de">Rechnerbetrieb</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id292">
    <rdfs:seeAlso rdf:resource="descriptor/11230-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10602-1">
    <skos:scopeNote xml:lang="de">Allgemein für Geschäftstätigkeit aller Art im Ausland und für Auslandsaufträge von Firmen, die nicht zum produzierenden Gewerbe zählen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13513-6">
    <skos:narrower rdf:resource="descriptor/14463-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13513-6">
    <skos:narrower rdf:resource="descriptor/14441-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13513-6">
    <skos:narrower rdf:resource="descriptor/14428-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13513-6">
    <skos:narrower rdf:resource="descriptor/14323-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13513-6">
    <skos:narrower rdf:resource="descriptor/14423-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13513-6">
    <skos:narrower rdf:resource="descriptor/14413-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13513-6">
    <skos:narrower rdf:resource="descriptor/14453-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13513-6">
    <skos:narrower rdf:resource="descriptor/14421-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13513-6">
    <skos:narrower rdf:resource="descriptor/14431-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10412-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20442-5">
    <skos:prefLabel xml:lang="en">Efficient consumer response</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20442-5">
    <skos:prefLabel xml:lang="de">Efficient Consumer Response</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19486-2">
    <skos:prefLabel xml:lang="en">Ozone</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19486-2">
    <skos:prefLabel xml:lang="de">Ozon</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id242">
    <rdfs:seeAlso rdf:resource="descriptor/11052-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12719-5">
    <skos:prefLabel xml:lang="en">Flow-line production</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12719-5">
    <skos:prefLabel xml:lang="de">Fließfertigung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19816-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16477-4">
    <skos:narrower rdf:resource="descriptor/16146-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16477-4">
    <skos:narrower rdf:resource="descriptor/11564-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16477-4">
    <skos:narrower rdf:resource="descriptor/11660-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16477-4">
    <skos:narrower rdf:resource="descriptor/18911-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18813-4">
    <skos:broader rdf:resource="thsys/71082"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18813-4">
    <skos:broader rdf:resource="thsys/71063"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18813-4">
    <skos:broader rdf:resource="thsys/70530"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18813-4">
    <skos:broader rdf:resource="descriptor/18822-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18813-4">
    <skos:broader rdf:resource="descriptor/11552-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10909-3">
    <skos:prefLabel xml:lang="en">Price competition</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10909-3">
    <skos:prefLabel xml:lang="de">Preiswettbewerb</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13327-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18399-2">
    <skos:broader rdf:resource="thsys/71003"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18399-2">
    <skos:broader rdf:resource="descriptor/13034-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16269-4">
    <skos:prefLabel xml:lang="en">Public information</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16269-4">
    <skos:prefLabel xml:lang="de">Staatliche Information</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14596-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19722-2">
    <skos:broader rdf:resource="thsys/73364"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11573-5">
    <skos:altLabel xml:lang="de">Reparationen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18489-1">
    <skos:related rdf:resource="descriptor/18480-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18489-1">
    <skos:related rdf:resource="descriptor/11906-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18489-1">
    <skos:related rdf:resource="descriptor/11981-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71097">
    <skos:broader rdf:resource="thsys/71098"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12427-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19686-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19401-1">
    <skos:prefLabel xml:lang="en">Tax refund</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19401-1">
    <skos:prefLabel xml:lang="de">Rückvergütung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19843-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10057-0">
    <skos:prefLabel xml:lang="en">Risk</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10057-0">
    <skos:prefLabel xml:lang="de">Risiko</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13219-6">
    <skos:scopeNote xml:lang="de">Für den Groß- und Einzelhandel BENUTZE die entsprechenden Deskriptoren.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16629-0">
    <skos:related rdf:resource="descriptor/16210-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16629-0">
    <skos:related rdf:resource="descriptor/11060-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16629-0">
    <skos:related rdf:resource="descriptor/11313-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16629-0">
    <skos:related rdf:resource="descriptor/16387-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71098">
    <rdfs:label xml:lang="en">Foreign economic policy</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71098">
    <rdfs:label xml:lang="de">Außenwirtschaftspolitik</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19399-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12019-5">
    <skos:prefLabel xml:lang="en">Land price</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12019-5">
    <skos:prefLabel xml:lang="de">Bodenpreis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12808-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10353-3">
    <skos:related rdf:resource="descriptor/10255-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10353-3">
    <skos:related rdf:resource="descriptor/11167-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10353-3">
    <skos:related rdf:resource="descriptor/10387-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70346">
    <skos:broader rdf:resource="thsys/70358"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10898-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18659-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12837-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11209-5">
    <skos:prefLabel xml:lang="en">Moonlighting</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11209-5">
    <skos:prefLabel xml:lang="de">Schwarzarbeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12717-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10314-6">
    <skos:broader rdf:resource="thsys/70233"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18126-2">
    <skos:related rdf:resource="descriptor/10787-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18126-2">
    <skos:related rdf:resource="descriptor/18614-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18126-2">
    <skos:related rdf:resource="descriptor/19958-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18126-2">
    <skos:related rdf:resource="descriptor/18013-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18126-2">
    <skos:related rdf:resource="descriptor/13301-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18126-2">
    <skos:related rdf:resource="descriptor/10833-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15790-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11661-1">
    <skos:narrower rdf:resource="descriptor/11681-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11661-1">
    <skos:narrower rdf:resource="descriptor/11680-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71111">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70455">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14923-3">
    <skos:altLabel xml:lang="de">Unterbekleidung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14923-3">
    <skos:altLabel xml:lang="de">Nachtwäsche</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14923-3">
    <skos:altLabel xml:lang="de">Miederwaren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19566-4">
    <skos:narrower rdf:resource="descriptor/12502-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19566-4">
    <skos:narrower rdf:resource="descriptor/19012-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19566-4">
    <skos:narrower rdf:resource="descriptor/11322-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10216-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14054-0">
    <skos:broader rdf:resource="thsys/70071"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14054-0">
    <skos:broader rdf:resource="descriptor/10016-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16150-1">
    <skos:broader rdf:resource="thsys/73336"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16150-1">
    <skos:broader rdf:resource="thsys/71051"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16150-1">
    <skos:broader rdf:resource="thsys/70278"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16150-1">
    <skos:broader rdf:resource="descriptor/15982-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16150-1">
    <skos:broader rdf:resource="descriptor/16484-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16150-1">
    <skos:broader rdf:resource="descriptor/11185-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14408-5">
    <skos:related rdf:resource="descriptor/14341-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14408-5">
    <skos:related rdf:resource="descriptor/18346-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15941-3">
    <skos:narrower rdf:resource="descriptor/15943-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70274">
    <skos:narrower rdf:resource="descriptor/11057-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70274">
    <skos:narrower rdf:resource="descriptor/19214-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70274">
    <skos:narrower rdf:resource="descriptor/10107-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70274">
    <skos:narrower rdf:resource="descriptor/10980-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70274">
    <skos:narrower rdf:resource="descriptor/20077-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70274">
    <skos:narrower rdf:resource="descriptor/10106-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70274">
    <skos:narrower rdf:resource="descriptor/18155-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70274">
    <skos:narrower rdf:resource="descriptor/19213-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70274">
    <skos:narrower rdf:resource="descriptor/10975-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70274">
    <skos:narrower rdf:resource="descriptor/10353-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70274">
    <skos:narrower rdf:resource="descriptor/11167-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70274">
    <skos:narrower rdf:resource="descriptor/11160-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70274">
    <skos:narrower rdf:resource="descriptor/11165-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70274">
    <skos:narrower rdf:resource="descriptor/11054-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70274">
    <skos:narrower rdf:resource="descriptor/11164-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70274">
    <skos:narrower rdf:resource="descriptor/19212-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70274">
    <skos:narrower rdf:resource="descriptor/11156-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70274">
    <skos:narrower rdf:resource="descriptor/11163-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70274">
    <skos:narrower rdf:resource="descriptor/11177-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70274">
    <skos:narrower rdf:resource="descriptor/10984-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70274">
    <skos:narrower rdf:resource="descriptor/10968-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70274">
    <skos:narrower rdf:resource="descriptor/19211-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70274">
    <skos:narrower rdf:resource="descriptor/11168-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70274">
    <skos:narrower rdf:resource="descriptor/11038-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70274">
    <skos:narrower rdf:resource="descriptor/11152-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70274">
    <skos:narrower rdf:resource="descriptor/10089-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70274">
    <skos:narrower rdf:resource="descriptor/10088-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15745-3">
    <skos:narrower rdf:resource="descriptor/14188-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15745-3">
    <skos:narrower rdf:resource="descriptor/15897-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15745-3">
    <skos:narrower rdf:resource="descriptor/15754-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15745-3">
    <skos:narrower rdf:resource="descriptor/18343-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15745-3">
    <skos:narrower rdf:resource="descriptor/15751-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15745-3">
    <skos:narrower rdf:resource="descriptor/18286-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15745-3">
    <skos:narrower rdf:resource="descriptor/12015-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11037-1">
    <skos:broader rdf:resource="thsys/71113"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11037-1">
    <skos:broader rdf:resource="thsys/70855"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11037-1">
    <skos:broader rdf:resource="thsys/70835"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11037-1">
    <skos:broader rdf:resource="thsys/70504"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11037-1">
    <skos:broader rdf:resource="descriptor/10110-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10464-1">
    <skos:prefLabel xml:lang="en">High technology</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10464-1">
    <skos:prefLabel xml:lang="de">Hochtechnologiesektor</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11083-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16416-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15837-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71009">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19025-3">
    <skos:broader rdf:resource="thsys/70582"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16486-3">
    <skos:related rdf:resource="descriptor/19687-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16486-3">
    <skos:related rdf:resource="descriptor/16615-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16486-3">
    <skos:related rdf:resource="descriptor/16512-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16486-3">
    <skos:related rdf:resource="descriptor/15954-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16486-3">
    <skos:related rdf:resource="descriptor/15994-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16486-3">
    <skos:related rdf:resource="descriptor/19649-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16486-3">
    <skos:related rdf:resource="descriptor/15911-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16486-3">
    <skos:related rdf:resource="descriptor/15989-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16486-3">
    <skos:related rdf:resource="descriptor/15907-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16486-3">
    <skos:related rdf:resource="descriptor/19843-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10276-2">
    <skos:broader rdf:resource="thsys/71112"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10276-2">
    <skos:broader rdf:resource="thsys/71111"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10276-2">
    <skos:broader rdf:resource="descriptor/10010-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10276-2">
    <skos:broader rdf:resource="descriptor/10237-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12234-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14404-6">
    <skos:broader rdf:resource="thsys/70201"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17773-2">
    <skos:prefLabel xml:lang="en">Belize</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17773-2">
    <skos:prefLabel xml:lang="de">Belize</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16489-4">
    <skos:broader rdf:resource="thsys/73366"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16489-4">
    <skos:broader rdf:resource="thsys/70866"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16489-4">
    <skos:broader rdf:resource="descriptor/16273-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11627-1">
    <skos:broader rdf:resource="thsys/71072"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11627-1">
    <skos:broader rdf:resource="thsys/70860"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11627-1">
    <skos:broader rdf:resource="descriptor/11628-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10252-2">
    <skos:related rdf:resource="descriptor/16743-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10252-2">
    <skos:related rdf:resource="descriptor/15860-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10252-2">
    <skos:related rdf:resource="descriptor/10975-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10252-2">
    <skos:related rdf:resource="descriptor/16740-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10252-2">
    <skos:related rdf:resource="descriptor/19229-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10252-2">
    <skos:related rdf:resource="descriptor/10249-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10252-2">
    <skos:related rdf:resource="descriptor/16738-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10252-2">
    <skos:related rdf:resource="descriptor/10041-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10252-2">
    <skos:related rdf:resource="descriptor/16739-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10252-2">
    <skos:related rdf:resource="descriptor/10076-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10252-2">
    <skos:related rdf:resource="descriptor/16737-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10942-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11342-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18256-3">
    <skos:prefLabel xml:lang="en">Sulphur production</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18256-3">
    <skos:prefLabel xml:lang="de">Schwefelgewinnung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10013-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id339">
    <rdfs:seeAlso rdf:resource="descriptor/18153-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12971-2">
    <skos:related rdf:resource="descriptor/14916-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12971-2">
    <skos:related rdf:resource="descriptor/14127-6"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id267">
    <rdfs:seeAlso rdf:resource="descriptor/12039-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16608-1">
    <skos:broader rdf:resource="thsys/73395"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16608-1">
    <skos:broader rdf:resource="descriptor/15717-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12806-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id389">
    <rdfs:seeAlso rdf:resource="descriptor/15025-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15895-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19235-6">
    <skos:altLabel xml:lang="de">Indigenisierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16070-6">
    <skos:broader rdf:resource="thsys/73391"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16070-6">
    <skos:broader rdf:resource="thsys/73389"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16070-6">
    <skos:broader rdf:resource="thsys/73333"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16070-6">
    <skos:broader rdf:resource="thsys/70934"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16070-6">
    <skos:broader rdf:resource="descriptor/16187-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16070-6">
    <skos:broader rdf:resource="descriptor/13464-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12846-5">
    <skos:prefLabel xml:lang="en">Advertising</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12846-5">
    <skos:prefLabel xml:lang="de">Werbung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13757-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15918-5">
    <skos:altLabel xml:lang="en">Tribe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15918-5">
    <skos:altLabel xml:lang="de">Volksstamm</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15918-5">
    <skos:altLabel xml:lang="de">Volksgruppe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15918-5">
    <skos:altLabel xml:lang="de">Ethnische Minderheit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15918-5">
    <skos:altLabel xml:lang="de">Ethnie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14883-3">
    <skos:altLabel xml:lang="de">Zellwolle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14883-3">
    <skos:altLabel xml:lang="de">Zellulosefaser</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14883-3">
    <skos:altLabel xml:lang="de">Zellstofffaser</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14883-3">
    <skos:altLabel xml:lang="de">Zellstoffaser</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14883-3">
    <skos:altLabel xml:lang="de">Viskosefaser</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14883-3">
    <skos:altLabel xml:lang="de">Proteinfaser</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14883-3">
    <skos:altLabel xml:lang="de">Papierfaser</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14883-3">
    <skos:altLabel xml:lang="de">Eiweißfaser</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14883-3">
    <skos:altLabel xml:lang="de">Caseinfaser</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14883-3">
    <skos:altLabel xml:lang="de">Alginatfaser</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19587-3">
    <skos:prefLabel xml:lang="en">Short-term economic statistics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19587-3">
    <skos:prefLabel xml:lang="de">Konjunkturstatistik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17177-4">
    <skos:broader rdf:resource="thsys/70130"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17177-4">
    <skos:broader rdf:resource="descriptor/17179-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13818-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70814">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.34</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10807-4">
    <skos:related rdf:resource="descriptor/19321-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10807-4">
    <skos:related rdf:resource="descriptor/18694-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10807-4">
    <skos:related rdf:resource="descriptor/10811-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10807-4">
    <skos:related rdf:resource="descriptor/10805-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10807-4">
    <skos:related rdf:resource="descriptor/10806-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10807-4">
    <skos:related rdf:resource="descriptor/10804-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10807-4">
    <skos:related rdf:resource="descriptor/10808-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10807-4">
    <skos:related rdf:resource="descriptor/19664-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12088-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19624-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12867-4">
    <skos:related rdf:resource="descriptor/12192-5"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id298">
    <rdf:value xml:lang="de">Für soziale Integrationsprozesse und -politik</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12253-4">
    <skos:related rdf:resource="descriptor/12302-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12253-4">
    <skos:related rdf:resource="descriptor/19755-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12253-4">
    <skos:related rdf:resource="descriptor/10816-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12253-4">
    <skos:related rdf:resource="descriptor/12252-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12253-4">
    <skos:related rdf:resource="descriptor/13777-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12253-4">
    <skos:related rdf:resource="descriptor/12251-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12253-4">
    <skos:related rdf:resource="descriptor/19088-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12253-4">
    <skos:related rdf:resource="descriptor/20733-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12253-4">
    <skos:related rdf:resource="descriptor/13377-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18857-5">
    <skos:narrower rdf:resource="descriptor/11377-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18857-5">
    <skos:narrower rdf:resource="descriptor/11373-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70924">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11143-2">
    <skos:altLabel xml:lang="de">NÖSPL</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11143-2">
    <skos:altLabel xml:lang="de">Neues Ökonomisches System der Planung und Leitung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18967-5">
    <skos:related rdf:resource="descriptor/18816-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18967-5">
    <skos:related rdf:resource="descriptor/18327-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19155-4">
    <skos:related rdf:resource="descriptor/19832-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19155-4">
    <skos:related rdf:resource="descriptor/24704-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19155-4">
    <skos:related rdf:resource="descriptor/19833-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19155-4">
    <skos:related rdf:resource="descriptor/18767-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18228-1">
    <skos:altLabel xml:lang="x-other">Makedonija</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18228-1">
    <skos:altLabel xml:lang="de">Republik Mazedonien</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18228-1">
    <skos:altLabel xml:lang="de">Makedonien</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18228-1">
    <skos:altLabel xml:lang="de">Ehemalige Jugoslawische Republik Mazedonien</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12521-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18828-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19431-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10770-1">
    <skos:prefLabel xml:lang="en">Foreign exchange reserves</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10770-1">
    <skos:prefLabel xml:lang="de">Währungsreserven</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12057-4">
    <skos:prefLabel xml:lang="en">Life cycle</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12057-4">
    <skos:prefLabel xml:lang="de">Lebenszyklus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18792-0">
    <skos:broader rdf:resource="thsys/73336"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18792-0">
    <skos:broader rdf:resource="thsys/70235"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18792-0">
    <skos:broader rdf:resource="thsys/70052"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18792-0">
    <skos:broader rdf:resource="descriptor/16131-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18792-0">
    <skos:broader rdf:resource="descriptor/19902-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17709-4">
    <skos:prefLabel xml:lang="en">Liberia</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17709-4">
    <skos:prefLabel xml:lang="de">Liberia</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19212-4">
    <skos:related rdf:resource="descriptor/11724-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19212-4">
    <skos:related rdf:resource="descriptor/11721-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17652-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12263-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13665-4">
    <skos:related rdf:resource="descriptor/12825-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13665-4">
    <skos:related rdf:resource="descriptor/16012-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13665-4">
    <skos:related rdf:resource="descriptor/18027-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13665-4">
    <skos:related rdf:resource="descriptor/16517-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13665-4">
    <skos:related rdf:resource="descriptor/13360-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13665-4">
    <skos:related rdf:resource="descriptor/13655-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13665-4">
    <skos:related rdf:resource="descriptor/14516-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14220-2">
    <skos:prefLabel xml:lang="en">Plutonium</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14220-2">
    <skos:prefLabel xml:lang="de">Plutonium</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17609-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19295-2">
    <skos:related rdf:resource="descriptor/19294-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19295-2">
    <skos:related rdf:resource="descriptor/18519-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19295-2">
    <skos:related rdf:resource="descriptor/18192-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19295-2">
    <skos:related rdf:resource="descriptor/18191-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19295-2">
    <skos:related rdf:resource="descriptor/14064-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16247-0">
    <skos:altLabel xml:lang="x-other">Coup d'Etat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16247-0">
    <skos:altLabel xml:lang="en">Military rule</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16247-0">
    <skos:altLabel xml:lang="de">Totalitarismustheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16247-0">
    <skos:altLabel xml:lang="de">Totalitarismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16247-0">
    <skos:altLabel xml:lang="de">Totalitäres System</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16247-0">
    <skos:altLabel xml:lang="de">Totalitäre Herrschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16247-0">
    <skos:altLabel xml:lang="de">Staatsstreich</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16247-0">
    <skos:altLabel xml:lang="de">Putsch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16247-0">
    <skos:altLabel xml:lang="de">Polizeistaat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16247-0">
    <skos:altLabel xml:lang="de">Militärregime</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16247-0">
    <skos:altLabel xml:lang="de">Militärregierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16247-0">
    <skos:altLabel xml:lang="de">Militärputsch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16247-0">
    <skos:altLabel xml:lang="de">Militärherrschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16247-0">
    <skos:altLabel xml:lang="de">Militärdiktatur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19795-3">
    <skos:related rdf:resource="descriptor/10470-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19795-3">
    <skos:related rdf:resource="descriptor/15664-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19795-3">
    <skos:related rdf:resource="descriptor/13960-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18300-5">
    <skos:prefLabel xml:lang="en">Nonfood product</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18300-5">
    <skos:prefLabel xml:lang="de">Nonfood-Artikel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71065">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.11.05</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20733-1">
    <skos:altLabel xml:lang="en">New Basel Accord</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20733-1">
    <skos:altLabel xml:lang="en">Basel II Capital Accord</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20733-1">
    <skos:altLabel xml:lang="en">Basel convention</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20733-1">
    <skos:altLabel xml:lang="de">Kapitalstandard für Banken</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20733-1">
    <skos:altLabel xml:lang="de">Eigenkapitalvereinbarung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20733-1">
    <skos:altLabel xml:lang="de">Basel II-Abkommen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15149-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12867-4">
    <skos:broader rdf:resource="thsys/73337"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12867-4">
    <skos:broader rdf:resource="thsys/71052"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12867-4">
    <skos:broader rdf:resource="thsys/71040"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12867-4">
    <skos:broader rdf:resource="descriptor/19043-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18580-1">
    <skos:broader rdf:resource="thsys/70938"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18580-1">
    <skos:broader rdf:resource="descriptor/13570-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70994">
    <skos:narrower rdf:resource="descriptor/18261-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70994">
    <skos:narrower rdf:resource="descriptor/18431-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70994">
    <skos:narrower rdf:resource="descriptor/12998-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70994">
    <skos:narrower rdf:resource="descriptor/13000-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70994">
    <skos:narrower rdf:resource="descriptor/18262-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70994">
    <skos:narrower rdf:resource="descriptor/19397-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70994">
    <skos:narrower rdf:resource="descriptor/12997-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70994">
    <skos:narrower rdf:resource="descriptor/18429-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70994">
    <skos:narrower rdf:resource="descriptor/12995-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70994">
    <skos:narrower rdf:resource="descriptor/13959-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70994">
    <skos:narrower rdf:resource="descriptor/15992-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70994">
    <skos:narrower rdf:resource="descriptor/12996-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70994">
    <skos:narrower rdf:resource="descriptor/18430-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70994">
    <skos:narrower rdf:resource="descriptor/12994-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70994">
    <skos:narrower rdf:resource="descriptor/13888-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70994">
    <skos:narrower rdf:resource="thsys/70993"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70994">
    <skos:narrower rdf:resource="thsys/70992"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12721-4">
    <skos:altLabel xml:lang="de">Industrieproduktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12721-4">
    <skos:altLabel xml:lang="de">Industriegüterproduktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13454-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73385">
    <skos:broader rdf:resource="thsys/70017"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10602-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19324-0">
    <skos:prefLabel xml:lang="en">Trade dispute</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19324-0">
    <skos:prefLabel xml:lang="de">Handelskonflikt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70187">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16978-3">
    <skos:prefLabel xml:lang="en">Valais</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16978-3">
    <skos:prefLabel xml:lang="de">Wallis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13723-2">
    <skos:broader rdf:resource="thsys/70905"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13723-2">
    <skos:broader rdf:resource="thsys/70187"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13723-2">
    <skos:broader rdf:resource="descriptor/10012-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13307-2">
    <skos:narrower rdf:resource="descriptor/18564-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13307-2">
    <skos:narrower rdf:resource="descriptor/13308-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13307-2">
    <skos:narrower rdf:resource="descriptor/13306-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73355">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.06.04.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19697-3">
    <skos:related rdf:resource="descriptor/16549-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19697-3">
    <skos:related rdf:resource="descriptor/19691-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19697-3">
    <skos:related rdf:resource="descriptor/11852-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70891">
    <skos:prefLabel xml:lang="de">W.16.01  Versicherungszweige</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14465-0">
    <skos:prefLabel xml:lang="en">Measuring instrument</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14465-0">
    <skos:prefLabel xml:lang="de">Messgerät</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12214-0">
    <skos:related rdf:resource="descriptor/13769-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12214-0">
    <skos:related rdf:resource="descriptor/12230-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12214-0">
    <skos:related rdf:resource="descriptor/13391-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12214-0">
    <skos:related rdf:resource="descriptor/12881-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12214-0">
    <skos:related rdf:resource="descriptor/11709-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12214-0">
    <skos:related rdf:resource="descriptor/19556-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12214-0">
    <skos:related rdf:resource="descriptor/12212-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12214-0">
    <skos:related rdf:resource="descriptor/12280-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12214-0">
    <skos:related rdf:resource="descriptor/18674-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15883-5">
    <skos:broader rdf:resource="thsys/73353"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15883-5">
    <skos:broader rdf:resource="thsys/70058"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15883-5">
    <skos:broader rdf:resource="descriptor/11857-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13027-1">
    <skos:prefLabel xml:lang="en">High-grade steelmaking</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13027-1">
    <skos:prefLabel xml:lang="de">Edelstahlindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17754-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13390-3">
    <skos:broader rdf:resource="thsys/70881"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13390-3">
    <skos:broader rdf:resource="descriptor/18762-2"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id62">
    <rdf:value xml:lang="de">Für die englische Bedeutung von Catering</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10031-4">
    <skos:prefLabel xml:lang="en">Economic forecast</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10031-4">
    <skos:prefLabel xml:lang="de">Wirtschaftsprognose</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17724-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16786-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14279-4">
    <skos:prefLabel xml:lang="en">High-grade steel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14279-4">
    <skos:prefLabel xml:lang="de">Edelstahl</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18527-5">
    <skos:prefLabel xml:lang="en">Retail format</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18527-5">
    <skos:prefLabel xml:lang="de">Betriebsform des Handels</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13819-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11827-0">
    <skos:related rdf:resource="descriptor/15777-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18301-3">
    <skos:broader rdf:resource="thsys/73347"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18301-3">
    <skos:broader rdf:resource="thsys/70814"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18301-3">
    <skos:broader rdf:resource="descriptor/15685-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17465-6">
    <skos:scopeNote xml:lang="de">Erklärte sich 1991 unabhängig von der Sowjetunion und trat der GUS bei</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15907-3">
    <skos:narrower rdf:resource="descriptor/11268-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18759-5">
    <skos:prefLabel xml:lang="en">Leasing industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18759-5">
    <skos:prefLabel xml:lang="de">Leasinggesellschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12013-3">
    <skos:prefLabel xml:lang="en">Transport infrastructure</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12013-3">
    <skos:prefLabel xml:lang="de">Verkehrsinfrastruktur</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19309-3">
    <skos:related rdf:resource="descriptor/19071-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19309-3">
    <skos:related rdf:resource="descriptor/10612-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16053-6">
    <skos:related rdf:resource="descriptor/16081-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16053-6">
    <skos:related rdf:resource="descriptor/16037-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16053-6">
    <skos:related rdf:resource="descriptor/20110-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16053-6">
    <skos:related rdf:resource="descriptor/20002-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16053-6">
    <skos:related rdf:resource="descriptor/19217-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16053-6">
    <skos:related rdf:resource="descriptor/13464-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16053-6">
    <skos:related rdf:resource="descriptor/18433-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11281-4">
    <skos:related rdf:resource="descriptor/11248-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11281-4">
    <skos:related rdf:resource="descriptor/11220-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17750-0">
    <skos:related rdf:resource="descriptor/18054-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17750-0">
    <skos:related rdf:resource="descriptor/17978-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18733-2">
    <skos:broader rdf:resource="thsys/70897"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18733-2">
    <skos:broader rdf:resource="thsys/70893"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18733-2">
    <skos:broader rdf:resource="thsys/70187"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18733-2">
    <skos:broader rdf:resource="descriptor/12210-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15604-0">
    <skos:broader rdf:resource="thsys/73360"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15604-0">
    <skos:broader rdf:resource="thsys/71037"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15604-0">
    <skos:broader rdf:resource="thsys/70383"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15604-0">
    <skos:broader rdf:resource="descriptor/16019-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18498-0">
    <skos:prefLabel xml:lang="en">Airport constructing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18498-0">
    <skos:prefLabel xml:lang="de">Flughafenbau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id354">
    <rdfs:seeAlso rdf:resource="descriptor/10559-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16782-6">
    <skos:related rdf:resource="descriptor/18767-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16782-6">
    <skos:related rdf:resource="descriptor/20341-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14355-0">
    <skos:broader rdf:resource="thsys/70424"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14355-0">
    <skos:broader rdf:resource="descriptor/14381-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16708-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19293-6">
    <skos:related rdf:resource="descriptor/19187-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19293-6">
    <skos:related rdf:resource="descriptor/19188-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19293-6">
    <skos:related rdf:resource="descriptor/18517-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19293-6">
    <skos:related rdf:resource="descriptor/18198-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19293-6">
    <skos:related rdf:resource="descriptor/14088-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19293-6">
    <skos:related rdf:resource="descriptor/11802-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15077-4">
    <skos:broader rdf:resource="thsys/72184"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15077-4">
    <skos:broader rdf:resource="thsys/70947"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15077-4">
    <skos:broader rdf:resource="descriptor/15093-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15077-4">
    <skos:broader rdf:resource="descriptor/13465-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12381-2">
    <skos:narrower rdf:resource="descriptor/12452-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12381-2">
    <skos:narrower rdf:resource="descriptor/12380-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12381-2">
    <skos:narrower rdf:resource="descriptor/12371-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12381-2">
    <skos:narrower rdf:resource="descriptor/12370-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12381-2">
    <skos:narrower rdf:resource="descriptor/12357-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13193-5">
    <skos:narrower rdf:resource="descriptor/13192-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17967-3">
    <skos:prefLabel xml:lang="en">Baltic Sea</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17967-3">
    <skos:prefLabel xml:lang="de">Ostsee</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12754-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10698-4">
    <skos:broader rdf:resource="thsys/73366"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10698-4">
    <skos:broader rdf:resource="thsys/73332"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10698-4">
    <skos:broader rdf:resource="thsys/71097"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10698-4">
    <skos:broader rdf:resource="descriptor/16184-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10698-4">
    <skos:broader rdf:resource="descriptor/10616-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19725-3">
    <skos:broader rdf:resource="thsys/73363"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19725-3">
    <skos:broader rdf:resource="thsys/70865"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19725-3">
    <skos:broader rdf:resource="descriptor/15685-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19374-6">
    <skos:broader rdf:resource="thsys/71083"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19374-6">
    <skos:broader rdf:resource="descriptor/11715-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70845">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#closeMatch">
    <rdf:type rdf:resource="http://www.w3.org/1999/02/22-rdf-syntax-ns#Property"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#closeMatch">
    <rdf:type rdf:resource="http://www.w3.org/2002/07/owl#SymmetricProperty"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#closeMatch">
    <rdf:type rdf:resource="http://www.w3.org/2002/07/owl#ObjectProperty"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20536-3">
    <skos:related rdf:resource="descriptor/11710-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20536-3">
    <skos:related rdf:resource="descriptor/12806-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20536-3">
    <skos:related rdf:resource="descriptor/10816-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20536-3">
    <skos:related rdf:resource="descriptor/19316-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20536-3">
    <skos:related rdf:resource="descriptor/12075-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11277-2">
    <skos:broader rdf:resource="thsys/70052"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11277-2">
    <skos:broader rdf:resource="descriptor/15927-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11277-2">
    <skos:broader rdf:resource="descriptor/11262-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19272-0">
    <skos:prefLabel xml:lang="en">Regional financial market</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19272-0">
    <skos:prefLabel xml:lang="de">Regionaler Finanzmarkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10576-4">
    <skos:related rdf:resource="descriptor/20125-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10576-4">
    <skos:related rdf:resource="descriptor/10598-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10576-4">
    <skos:related rdf:resource="descriptor/10505-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12697-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19403-4">
    <skos:prefLabel xml:lang="en">Marginal tax rate</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19403-4">
    <skos:prefLabel xml:lang="de">Grenzsteuersatz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19372-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18787-0">
    <skos:scopeNote xml:lang="de">Einschließlich Ingenieurbüros für die bautechnische Gesamtplanung</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11897-0">
    <skos:prefLabel xml:lang="en">Geographic concentration</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11897-0">
    <skos:prefLabel xml:lang="de">Regionale Konzentration</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11986-1">
    <skos:prefLabel xml:lang="en">Urban research</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11986-1">
    <skos:prefLabel xml:lang="de">Stadtforschung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19211-6">
    <skos:prefLabel xml:lang="en">Institutional change</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19211-6">
    <skos:prefLabel xml:lang="de">Institutioneller Wandel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14983-6">
    <skos:related rdf:resource="descriptor/13234-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14983-6">
    <skos:related rdf:resource="descriptor/13152-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19853-1">
    <skos:broader rdf:resource="thsys/73333"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19555-2">
    <skos:altLabel xml:lang="en">Tie-in sale</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19555-2">
    <skos:altLabel xml:lang="en">Product bundling</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19555-2">
    <skos:altLabel xml:lang="en">Commodity bundling</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19555-2">
    <skos:altLabel xml:lang="de">Preisbündelung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15666-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13377-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70814">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11876-1">
    <skos:prefLabel xml:lang="en">Regional transport</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11876-1">
    <skos:prefLabel xml:lang="de">Regionalverkehr</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13881-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19073-6">
    <skos:altLabel xml:lang="de">Theoretisches Modell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15619-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18402-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17199-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11974-1">
    <skos:altLabel xml:lang="de">Innerstädtischer Verkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11238-5">
    <skos:related rdf:resource="descriptor/11235-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17931-3">
    <skos:altLabel xml:lang="x-other">El Cono Sur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17931-3">
    <skos:altLabel xml:lang="de">Südamerikanische Staaten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17931-3">
    <skos:altLabel xml:lang="de">Südamerikanische Länder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73360">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12058-2">
    <skos:related rdf:resource="descriptor/19095-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12058-2">
    <skos:related rdf:resource="descriptor/12575-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12058-2">
    <skos:related rdf:resource="descriptor/19667-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12058-2">
    <skos:related rdf:resource="descriptor/12057-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12058-2">
    <skos:related rdf:resource="descriptor/19159-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12058-2">
    <skos:related rdf:resource="descriptor/18107-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12058-2">
    <skos:related rdf:resource="descriptor/18032-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72186">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18304-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11981-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18906-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18088-5">
    <skos:prefLabel xml:lang="en">Money market fund</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18088-5">
    <skos:prefLabel xml:lang="de">Geldmarktfonds</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12563-3">
    <skos:related rdf:resource="descriptor/12748-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12181-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20722-6">
    <skos:prefLabel xml:lang="en">Agenda 21</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20722-6">
    <skos:prefLabel xml:lang="de">Agenda 21</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13268-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13333-1">
    <skos:narrower rdf:resource="descriptor/13340-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13333-1">
    <skos:narrower rdf:resource="descriptor/13334-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10217-4">
    <skos:related rdf:resource="descriptor/11111-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10217-4">
    <skos:related rdf:resource="descriptor/10978-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10217-4">
    <skos:related rdf:resource="descriptor/11754-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10217-4">
    <skos:related rdf:resource="descriptor/19222-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10217-4">
    <skos:related rdf:resource="descriptor/10213-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10217-4">
    <skos:related rdf:resource="descriptor/10668-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10217-4">
    <skos:related rdf:resource="descriptor/10891-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10217-4">
    <skos:related rdf:resource="descriptor/10070-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12724-5">
    <skos:related rdf:resource="descriptor/12727-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12724-5">
    <skos:related rdf:resource="descriptor/12719-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70171">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11971-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12062-4">
    <skos:related rdf:resource="descriptor/12796-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12062-4">
    <skos:related rdf:resource="descriptor/10876-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12062-4">
    <skos:related rdf:resource="descriptor/20449-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12062-4">
    <skos:related rdf:resource="descriptor/19071-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12062-4">
    <skos:related rdf:resource="descriptor/12446-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12062-4">
    <skos:related rdf:resource="descriptor/18437-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19639-3">
    <skos:prefLabel xml:lang="en">Migrants</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19639-3">
    <skos:prefLabel xml:lang="de">Migranten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17994-0">
    <skos:altLabel xml:lang="de">RGW-Länder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17994-0">
    <skos:altLabel xml:lang="de">COMECON-Staaten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17994-0">
    <skos:altLabel xml:lang="de">COMECON-Länder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13841-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13025-5">
    <skos:broader rdf:resource="thsys/71000"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13025-5">
    <skos:broader rdf:resource="descriptor/13037-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70828">
    <rdfs:label xml:lang="en">Cultural industry</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70828">
    <rdfs:label xml:lang="de">Kultursektor</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71066">
    <skos:narrower rdf:resource="descriptor/18749-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71066">
    <skos:narrower rdf:resource="descriptor/19456-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71066">
    <skos:narrower rdf:resource="descriptor/19453-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71066">
    <skos:narrower rdf:resource="descriptor/18967-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71066">
    <skos:narrower rdf:resource="descriptor/19398-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71066">
    <skos:narrower rdf:resource="descriptor/18329-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71066">
    <skos:narrower rdf:resource="descriptor/12032-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71066">
    <skos:narrower rdf:resource="descriptor/18745-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71066">
    <skos:narrower rdf:resource="descriptor/12031-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71066">
    <skos:narrower rdf:resource="descriptor/12030-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71066">
    <skos:narrower rdf:resource="descriptor/13704-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71066">
    <skos:narrower rdf:resource="descriptor/12029-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71066">
    <skos:narrower rdf:resource="descriptor/12028-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71066">
    <skos:narrower rdf:resource="descriptor/11659-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71066">
    <skos:narrower rdf:resource="descriptor/11658-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71066">
    <skos:narrower rdf:resource="descriptor/12026-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71066">
    <skos:narrower rdf:resource="descriptor/19455-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71066">
    <skos:narrower rdf:resource="descriptor/18748-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71066">
    <skos:narrower rdf:resource="descriptor/18122-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71066">
    <skos:narrower rdf:resource="descriptor/18496-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71066">
    <skos:narrower rdf:resource="descriptor/12024-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71066">
    <skos:narrower rdf:resource="descriptor/12023-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71066">
    <skos:narrower rdf:resource="descriptor/12021-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71066">
    <skos:narrower rdf:resource="descriptor/18327-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71066">
    <skos:narrower rdf:resource="descriptor/12022-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71066">
    <skos:narrower rdf:resource="descriptor/18333-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71066">
    <skos:narrower rdf:resource="descriptor/12020-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71066">
    <skos:narrower rdf:resource="descriptor/12019-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71066">
    <skos:narrower rdf:resource="descriptor/12018-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71066">
    <skos:narrower rdf:resource="descriptor/11921-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71066">
    <skos:narrower rdf:resource="descriptor/18328-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71066">
    <skos:narrower rdf:resource="descriptor/11859-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71066">
    <skos:narrower rdf:resource="descriptor/19454-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71066">
    <skos:narrower rdf:resource="descriptor/19484-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71066">
    <skos:narrower rdf:resource="descriptor/12016-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71066">
    <skos:narrower rdf:resource="descriptor/12015-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71066">
    <skos:narrower rdf:resource="descriptor/12014-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71066">
    <skos:narrower rdf:resource="descriptor/19339-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12882-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15515-6">
    <skos:related rdf:resource="descriptor/12208-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17911-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10942-5">
    <skos:narrower rdf:resource="descriptor/10940-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10942-5">
    <skos:narrower rdf:resource="descriptor/10931-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10942-5">
    <skos:narrower rdf:resource="descriptor/10930-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10942-5">
    <skos:narrower rdf:resource="descriptor/10926-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10942-5">
    <skos:narrower rdf:resource="descriptor/10923-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12062-4">
    <skos:broader rdf:resource="thsys/71090"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12062-4">
    <skos:broader rdf:resource="thsys/70140"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12062-4">
    <skos:broader rdf:resource="descriptor/12072-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13534-5">
    <skos:altLabel xml:lang="de">Hafenumschlagstechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13534-5">
    <skos:altLabel xml:lang="de">Hafenumschlagsanlagen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13534-5">
    <skos:altLabel xml:lang="de">Hafentechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13534-5">
    <skos:altLabel xml:lang="de">Hafenfazilitäten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13534-5">
    <skos:altLabel xml:lang="de">Hafenausrüstung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71055">
    <skos:narrower rdf:resource="descriptor/13971-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71055">
    <skos:narrower rdf:resource="descriptor/13979-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71055">
    <skos:narrower rdf:resource="descriptor/18966-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71055">
    <skos:narrower rdf:resource="descriptor/18947-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71055">
    <skos:narrower rdf:resource="descriptor/13363-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71055">
    <skos:narrower rdf:resource="descriptor/13998-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71055">
    <skos:narrower rdf:resource="descriptor/13983-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71055">
    <skos:narrower rdf:resource="descriptor/13995-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71055">
    <skos:narrower rdf:resource="descriptor/14004-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71055">
    <skos:narrower rdf:resource="descriptor/13981-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71055">
    <skos:narrower rdf:resource="descriptor/13967-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71055">
    <skos:narrower rdf:resource="descriptor/13980-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71055">
    <skos:narrower rdf:resource="descriptor/11374-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71055">
    <skos:narrower rdf:resource="descriptor/13355-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71055">
    <skos:narrower rdf:resource="descriptor/13382-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71055">
    <skos:narrower rdf:resource="descriptor/14001-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71055">
    <skos:narrower rdf:resource="descriptor/12025-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71055">
    <skos:narrower rdf:resource="descriptor/13974-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71055">
    <skos:narrower rdf:resource="descriptor/18599-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71055">
    <skos:narrower rdf:resource="descriptor/13975-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71055">
    <skos:narrower rdf:resource="descriptor/13966-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71055">
    <skos:narrower rdf:resource="descriptor/13871-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71055">
    <skos:narrower rdf:resource="descriptor/18569-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71055">
    <skos:narrower rdf:resource="descriptor/18535-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71055">
    <skos:narrower rdf:resource="descriptor/18788-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71055">
    <skos:narrower rdf:resource="descriptor/16301-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71055">
    <skos:narrower rdf:resource="descriptor/13716-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71055">
    <skos:narrower rdf:resource="descriptor/12879-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71055">
    <skos:narrower rdf:resource="descriptor/11358-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71055">
    <skos:narrower rdf:resource="thsys/71053"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71055">
    <skos:narrower rdf:resource="thsys/71054"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19469-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12714-1">
    <skos:prefLabel xml:lang="en">Set-up time</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12714-1">
    <skos:prefLabel xml:lang="de">Rüstzeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18371-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13375-6">
    <skos:related rdf:resource="descriptor/13371-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13375-6">
    <skos:related rdf:resource="descriptor/13871-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13375-6">
    <skos:related rdf:resource="descriptor/14025-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13375-6">
    <skos:related rdf:resource="descriptor/13370-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71052">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.13.06.03</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70467">
    <skos:narrower rdf:resource="descriptor/14130-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70467">
    <skos:narrower rdf:resource="descriptor/14633-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70467">
    <skos:narrower rdf:resource="descriptor/14129-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70467">
    <skos:narrower rdf:resource="descriptor/14962-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70467">
    <skos:narrower rdf:resource="descriptor/14125-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70467">
    <skos:narrower rdf:resource="descriptor/19020-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70467">
    <skos:narrower rdf:resource="descriptor/14124-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70467">
    <skos:narrower rdf:resource="descriptor/14123-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70467">
    <skos:narrower rdf:resource="descriptor/14127-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70467">
    <skos:narrower rdf:resource="descriptor/14122-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70467">
    <skos:narrower rdf:resource="descriptor/14121-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70845">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19562-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12916-3">
    <skos:broader rdf:resource="thsys/71004"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12916-3">
    <skos:broader rdf:resource="descriptor/12993-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71053">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14205-5">
    <skos:prefLabel xml:lang="en">Salt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14205-5">
    <skos:prefLabel xml:lang="de">Salz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10186-3">
    <skos:prefLabel xml:lang="en">Factor price</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10186-3">
    <skos:prefLabel xml:lang="de">Faktorpreis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18027-4">
    <skos:related rdf:resource="descriptor/13360-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18027-4">
    <skos:related rdf:resource="descriptor/13665-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18027-4">
    <skos:related rdf:resource="descriptor/18932-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10570-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11199-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10500-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11677-0">
    <skos:related rdf:resource="descriptor/14997-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11677-0">
    <skos:related rdf:resource="descriptor/13164-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11677-0">
    <skos:related rdf:resource="descriptor/18900-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19003-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14969-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12670-2">
    <skos:narrower rdf:resource="descriptor/18092-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12670-2">
    <skos:narrower rdf:resource="descriptor/18620-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12670-2">
    <skos:narrower rdf:resource="descriptor/21020-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12670-2">
    <skos:narrower rdf:resource="descriptor/19724-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12670-2">
    <skos:narrower rdf:resource="descriptor/12644-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12670-2">
    <skos:narrower rdf:resource="descriptor/19114-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12670-2">
    <skos:narrower rdf:resource="descriptor/12689-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12670-2">
    <skos:narrower rdf:resource="descriptor/12677-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16757-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10136-4">
    <skos:broader rdf:resource="thsys/70455"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10136-4">
    <skos:broader rdf:resource="descriptor/10137-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16769-5">
    <skos:related rdf:resource="descriptor/18951-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18919-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15050-3">
    <skos:prefLabel xml:lang="en">Matrix calculus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15050-3">
    <skos:prefLabel xml:lang="de">Matrizenrechnung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13533-0">
    <skos:narrower rdf:resource="descriptor/13534-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13533-0">
    <skos:narrower rdf:resource="descriptor/10695-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13533-0">
    <skos:narrower rdf:resource="descriptor/18241-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13533-0">
    <skos:narrower rdf:resource="descriptor/13528-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19279-0">
    <skos:altLabel xml:lang="en">Popular capitalism</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19279-0">
    <skos:altLabel xml:lang="de">Volksaktie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19279-0">
    <skos:altLabel xml:lang="de">Kleinaktionärinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19279-0">
    <skos:altLabel xml:lang="de">Kleinaktionär</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19279-0">
    <skos:altLabel xml:lang="de">Aktienkleinbesitz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71067">
    <skos:broader rdf:resource="thsys/70058"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14109-1">
    <skos:broader rdf:resource="thsys/70169"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14109-1">
    <skos:broader rdf:resource="descriptor/14110-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13481-0">
    <skos:broader rdf:resource="thsys/70958"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13481-0">
    <skos:broader rdf:resource="thsys/70947"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70824">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.29.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17999-4">
    <skos:narrower rdf:resource="descriptor/19000-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17999-4">
    <skos:narrower rdf:resource="descriptor/17993-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17999-4">
    <skos:narrower rdf:resource="descriptor/17991-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17999-4">
    <skos:narrower rdf:resource="descriptor/19178-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17999-4">
    <skos:narrower rdf:resource="descriptor/19177-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17999-4">
    <skos:narrower rdf:resource="descriptor/17983-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17999-4">
    <skos:narrower rdf:resource="descriptor/17982-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17999-4">
    <skos:narrower rdf:resource="descriptor/17980-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18944-3">
    <skos:prefLabel xml:lang="en">Information policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18944-3">
    <skos:prefLabel xml:lang="de">Informationspolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12160-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16173-3">
    <skos:altLabel xml:lang="de">Weltraumfreiheit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19565-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19352-2">
    <skos:broader rdf:resource="thsys/73362"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19352-2">
    <skos:broader rdf:resource="thsys/71092"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19352-2">
    <skos:broader rdf:resource="descriptor/16495-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10506-4">
    <skos:prefLabel xml:lang="en">Least developed countries</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10506-4">
    <skos:prefLabel xml:lang="de">Vierte Welt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14900-1">
    <skos:related rdf:resource="descriptor/14840-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70005">
    <rdfs:label xml:lang="de">Betriebswirtschaftliches Rechnungs- und Prüfungswesen</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18876-1">
    <skos:broader rdf:resource="thsys/73360"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18876-1">
    <skos:broader rdf:resource="thsys/70844"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18876-1">
    <skos:broader rdf:resource="descriptor/16019-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11177-6">
    <skos:altLabel xml:lang="en">Theory of association</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11177-6">
    <skos:altLabel xml:lang="en">Shared facility</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11177-6">
    <skos:altLabel xml:lang="en">Club goods</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11177-6">
    <skos:altLabel xml:lang="de">Ökonomische Theorie des Klubs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11177-6">
    <skos:altLabel xml:lang="de">Ökonomische Theorie der Klubs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11177-6">
    <skos:altLabel xml:lang="de">Klubgüter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11177-6">
    <skos:altLabel xml:lang="de">Klub</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11704-2">
    <skos:broader rdf:resource="thsys/73372"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11704-2">
    <skos:broader rdf:resource="thsys/71075"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11704-2">
    <skos:broader rdf:resource="descriptor/11728-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15483-0">
    <skos:altLabel xml:lang="en">OR (Operations research)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15483-0">
    <skos:altLabel xml:lang="en">Operational research</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15483-0">
    <skos:altLabel xml:lang="en">Management science</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15483-0">
    <skos:altLabel xml:lang="de">Unternehmensforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15483-0">
    <skos:altLabel xml:lang="de">Operations-Research-Theorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18067-6">
    <skos:scopeNote xml:lang="de">Keine einheitliche Definition. Vor allem das Einstehenmüssen für eine aus einem Schuldverhältnis herrührende Schuld. I.e.S. die Haftung des Schuldners mit seinem Vermögen gegenüber dem Gläubiger.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18737-1">
    <skos:related rdf:resource="descriptor/16034-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14514-6">
    <skos:altLabel xml:lang="en">Videodisc</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14514-6">
    <skos:altLabel xml:lang="en">Optical disc</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14514-6">
    <skos:altLabel xml:lang="de">Videoplatte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14514-6">
    <skos:altLabel xml:lang="de">Opto-elektronischer Speicher</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14514-6">
    <skos:altLabel xml:lang="de">Magneto-optischer Speicher</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14514-6">
    <skos:altLabel xml:lang="de">Laser-optischer Speicher</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14514-6">
    <skos:altLabel xml:lang="de">Bildplatte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13362-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13969-5">
    <skos:broader rdf:resource="thsys/71054"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13969-5">
    <skos:broader rdf:resource="thsys/70892"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13969-5">
    <skos:broader rdf:resource="descriptor/18977-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14181-0">
    <skos:related rdf:resource="descriptor/13003-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14181-0">
    <skos:related rdf:resource="descriptor/12976-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14181-0">
    <skos:related rdf:resource="descriptor/14035-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14181-0">
    <skos:related rdf:resource="descriptor/14182-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://zbw.eu/namespaces/skos-extensions/indexedItem">
    <rdfs:comment xml:lang="en">A substitute for the vanished skos:isSubjectOf. 
    May be used with collections of resources also.</rdfs:comment>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16539-1">
    <skos:narrower rdf:resource="descriptor/19515-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16539-1">
    <skos:narrower rdf:resource="descriptor/19514-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16539-1">
    <skos:narrower rdf:resource="descriptor/19513-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19822-5">
    <skos:prefLabel xml:lang="en">National law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19822-5">
    <skos:prefLabel xml:lang="de">Nationales Recht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70465">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">P.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19262-3">
    <skos:altLabel xml:lang="en">Monetary targets</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19262-3">
    <skos:altLabel xml:lang="de">Zwischenziele der Geldpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19262-3">
    <skos:altLabel xml:lang="de">Geldpolitische Zwischenziele</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70650">
    <skos:narrower rdf:resource="descriptor/14222-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70650">
    <skos:narrower rdf:resource="descriptor/15013-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70650">
    <skos:narrower rdf:resource="descriptor/14220-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70650">
    <skos:narrower rdf:resource="descriptor/14182-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70650">
    <skos:narrower rdf:resource="descriptor/14181-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13411-0">
    <skos:prefLabel xml:lang="en">Accident insurance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13411-0">
    <skos:prefLabel xml:lang="de">Unfallversicherung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10186-3">
    <skos:broader rdf:resource="thsys/70481"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10186-3">
    <skos:broader rdf:resource="thsys/70233"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10186-3">
    <skos:broader rdf:resource="descriptor/10213-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13148-3">
    <skos:prefLabel xml:lang="en">Weaving industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13148-3">
    <skos:prefLabel xml:lang="de">Weberei</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10062-0">
    <skos:altLabel xml:lang="de">Volkswirtschaftstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11398-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12656-3">
    <skos:prefLabel xml:lang="en">Order</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12656-3">
    <skos:prefLabel xml:lang="de">Bestellung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17665-5">
    <skos:prefLabel xml:lang="en">Madagascar</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17665-5">
    <skos:prefLabel xml:lang="de">Madagaskar</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15682-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11578-2">
    <skos:altLabel xml:lang="de">Öffentlicher Tiefbau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11578-2">
    <skos:altLabel xml:lang="de">Öffentlicher Hochbau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14229-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19680-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13080-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71116">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.02.07</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70870">
    <skos:broader rdf:resource="thsys/70871"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13392-6">
    <skos:prefLabel xml:lang="en">Reproduction rights organization</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13392-6">
    <skos:prefLabel xml:lang="de">Verwertungsgesellschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14037-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11784-6">
    <skos:altLabel xml:lang="de">Wirtschaftszweig</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11784-6">
    <skos:altLabel xml:lang="de">Wirtschaftssektor</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11784-6">
    <skos:altLabel xml:lang="de">Wirtschaftsbranche</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11784-6">
    <skos:altLabel xml:lang="de">Wirtschaftsbereich</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11784-6">
    <skos:altLabel xml:lang="de">Sektor</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73351">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17749-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17502-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12725-3">
    <skos:narrower rdf:resource="descriptor/12729-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12725-3">
    <skos:narrower rdf:resource="descriptor/12727-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12725-3">
    <skos:narrower rdf:resource="descriptor/10324-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12725-3">
    <skos:narrower rdf:resource="descriptor/10323-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12725-3">
    <skos:narrower rdf:resource="descriptor/12724-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12725-3">
    <skos:narrower rdf:resource="descriptor/12719-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12725-3">
    <skos:narrower rdf:resource="descriptor/12711-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12725-3">
    <skos:narrower rdf:resource="descriptor/12720-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12725-3">
    <skos:narrower rdf:resource="descriptor/12718-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12725-3">
    <skos:narrower rdf:resource="descriptor/10310-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12725-3">
    <skos:narrower rdf:resource="descriptor/12717-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12725-3">
    <skos:narrower rdf:resource="descriptor/12730-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12725-3">
    <skos:narrower rdf:resource="descriptor/12716-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10951-4">
    <skos:prefLabel xml:lang="en">Predation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10951-4">
    <skos:prefLabel xml:lang="de">Verdrängungswettbewerb</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="../stw">
    <cc:attributionURL>http://zbw.eu</cc:attributionURL>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20982-1">
    <skos:scopeNote xml:lang="de">Ökonomische Theorien in der Tradition von Smith, Ricardo, Marx, Sraffa und Leontief.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10338-6">
    <skos:narrower rdf:resource="descriptor/10340-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10338-6">
    <skos:narrower rdf:resource="descriptor/19240-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10338-6">
    <skos:narrower rdf:resource="descriptor/10337-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18835-1">
    <skos:altLabel xml:lang="en">Public sector marketing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18835-1">
    <skos:altLabel xml:lang="en">Non-profit marketing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18835-1">
    <skos:altLabel xml:lang="de">Verwaltungsmarketing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18835-1">
    <skos:altLabel xml:lang="de">Öffentliches Marketing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18835-1">
    <skos:altLabel xml:lang="de">Marketing für öffentliche Institutionen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19704-4">
    <skos:broader rdf:resource="thsys/73373"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19704-4">
    <skos:broader rdf:resource="thsys/73334"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19704-4">
    <skos:broader rdf:resource="descriptor/15970-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19021-4">
    <skos:prefLabel xml:lang="en">Mutton</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19021-4">
    <skos:prefLabel xml:lang="de">Schaffleisch</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12278-2">
    <skos:broader rdf:resource="thsys/71044"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12278-2">
    <skos:broader rdf:resource="thsys/70530"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12278-2">
    <skos:broader rdf:resource="descriptor/10007-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11681-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14643-2">
    <skos:broader rdf:resource="thsys/70463"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14643-2">
    <skos:broader rdf:resource="descriptor/14205-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14643-2">
    <skos:broader rdf:resource="descriptor/14642-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19246-1">
    <skos:broader rdf:resource="thsys/71110"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19246-1">
    <skos:broader rdf:resource="thsys/70058"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19246-1">
    <skos:broader rdf:resource="descriptor/11790-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11448-1">
    <skos:related rdf:resource="descriptor/19602-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11448-1">
    <skos:related rdf:resource="descriptor/11460-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11448-1">
    <skos:related rdf:resource="descriptor/11451-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11448-1">
    <skos:related rdf:resource="descriptor/11479-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11448-1">
    <skos:related rdf:resource="descriptor/11418-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11448-1">
    <skos:related rdf:resource="descriptor/11415-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11448-1">
    <skos:related rdf:resource="descriptor/13784-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70403">
    <skos:broader rdf:resource="thsys/70071"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17911-2">
    <skos:prefLabel xml:lang="en">Argentina</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17911-2">
    <skos:prefLabel xml:lang="de">Argentinien</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17161-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73343">
    <skos:broader rdf:resource="thsys/73345"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24858-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19215-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13124-3">
    <skos:broader rdf:resource="thsys/70990"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13124-3">
    <skos:broader rdf:resource="descriptor/13102-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15939-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15493-4">
    <skos:related rdf:resource="descriptor/10220-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13215-0">
    <skos:broader rdf:resource="thsys/70957"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13215-0">
    <skos:broader rdf:resource="descriptor/13472-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13215-0">
    <skos:broader rdf:resource="descriptor/12912-4"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id43">
    <rdfs:seeAlso rdf:resource="descriptor/13214-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14895-3">
    <skos:prefLabel xml:lang="en">Household textiles</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14895-3">
    <skos:prefLabel xml:lang="de">Heimtextilien</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18850-5">
    <skos:prefLabel xml:lang="en">Occupational organization</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18850-5">
    <skos:prefLabel xml:lang="de">Berufsverband</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14623-1">
    <skos:related rdf:resource="descriptor/14737-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14623-1">
    <skos:related rdf:resource="descriptor/18268-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14623-1">
    <skos:related rdf:resource="descriptor/14611-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14623-1">
    <skos:related rdf:resource="descriptor/14683-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15739-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13610-1">
    <skos:related rdf:resource="descriptor/13612-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13610-1">
    <skos:related rdf:resource="descriptor/15518-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13610-1">
    <skos:related rdf:resource="descriptor/13546-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13610-1">
    <skos:related rdf:resource="descriptor/12708-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13610-1">
    <skos:related rdf:resource="descriptor/13600-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19791-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18345-4">
    <skos:altLabel xml:lang="de">Stadtwerke</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18345-4">
    <skos:altLabel xml:lang="de">Städtische Versorgungswirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18345-4">
    <skos:altLabel xml:lang="de">Örtliche Versorgungswirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18345-4">
    <skos:altLabel xml:lang="de">Kommunaler Querverbund</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11666-5">
    <skos:broader rdf:resource="thsys/71074"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11666-5">
    <skos:broader rdf:resource="thsys/70397"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11666-5">
    <skos:broader rdf:resource="descriptor/11679-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11666-5">
    <skos:broader rdf:resource="descriptor/11652-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71075">
    <skos:broader rdf:resource="thsys/71077"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13533-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21042-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12203-5">
    <skos:prefLabel xml:lang="en">Dynamic investment calculation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12203-5">
    <skos:prefLabel xml:lang="de">Dynamische Investitionsrechnung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18569-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11735-5">
    <skos:narrower rdf:resource="descriptor/11872-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11735-5">
    <skos:narrower rdf:resource="descriptor/11739-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11735-5">
    <skos:narrower rdf:resource="descriptor/11736-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70815">
    <skos:broader rdf:resource="thsys/w"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18254-0">
    <skos:broader rdf:resource="thsys/70463"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18254-0">
    <skos:broader rdf:resource="descriptor/14603-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18254-0">
    <skos:broader rdf:resource="descriptor/14642-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21315-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18373-6">
    <skos:related rdf:resource="descriptor/17985-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18373-6">
    <skos:related rdf:resource="descriptor/12920-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18373-6">
    <skos:related rdf:resource="descriptor/14218-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18373-6">
    <skos:related rdf:resource="descriptor/19182-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18373-6">
    <skos:related rdf:resource="descriptor/19181-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18373-6">
    <skos:related rdf:resource="descriptor/18372-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18373-6">
    <skos:related rdf:resource="descriptor/14197-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18327-6">
    <skos:broader rdf:resource="thsys/71066"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18327-6">
    <skos:broader rdf:resource="thsys/70239"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18327-6">
    <skos:broader rdf:resource="descriptor/18071-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70556">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10647-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10229-4">
    <skos:prefLabel xml:lang="en">Aggregation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10229-4">
    <skos:prefLabel xml:lang="de">Aggregation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70957">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10305-0">
    <skos:altLabel xml:lang="en">Constant-elasticity-of-substitution production function</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10305-0">
    <skos:altLabel xml:lang="de">Produktionsfunktion (CES)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10305-0">
    <skos:altLabel xml:lang="de">Arrow-Chenery-Minhas-Solow-Produktionsfunktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10305-0">
    <skos:altLabel xml:lang="de">ACMS-Produktionsfunktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12196-4">
    <skos:altLabel xml:lang="en">Transformation of a company</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12196-4">
    <skos:altLabel xml:lang="en">Conversion of a company</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12196-4">
    <skos:altLabel xml:lang="en">Company conversion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12196-4">
    <skos:altLabel xml:lang="de">Unternehmensformänderung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12196-4">
    <skos:altLabel xml:lang="de">Umgründung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12196-4">
    <skos:altLabel xml:lang="de">Rechtsformwechsel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12196-4">
    <skos:altLabel xml:lang="de">Rechtsformänderung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12196-4">
    <skos:altLabel xml:lang="de">Änderung der Rechtsform</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15063-1">
    <skos:altLabel xml:lang="en">Sequential method</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15063-1">
    <skos:altLabel xml:lang="en">Sequential approach</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15063-1">
    <skos:altLabel xml:lang="de">Sequenzialanalyse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11731-6">
    <skos:related rdf:resource="descriptor/11596-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11731-6">
    <skos:related rdf:resource="descriptor/11613-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11731-6">
    <skos:related rdf:resource="descriptor/11586-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11731-6">
    <skos:related rdf:resource="descriptor/11607-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73103">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20455-3">
    <skos:prefLabel xml:lang="en">Enterprise bargaining</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20455-3">
    <skos:prefLabel xml:lang="de">Firmentarifvertrag</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18950-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13398-1">
    <skos:editorialNote xml:lang="de">Möglichst spezifischer indexieren.</skos:editorialNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11361-6">
    <skos:narrower rdf:resource="descriptor/11390-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11361-6">
    <skos:narrower rdf:resource="descriptor/18143-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11361-6">
    <skos:narrower rdf:resource="descriptor/11365-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11361-6">
    <skos:narrower rdf:resource="descriptor/11228-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73393">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25881-3">
    <skos:related rdf:resource="descriptor/24847-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25881-3">
    <skos:related rdf:resource="descriptor/15839-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25881-3">
    <skos:related rdf:resource="descriptor/11052-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11212-2">
    <skos:prefLabel xml:lang="en">Part-time employment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11212-2">
    <skos:prefLabel xml:lang="de">Teilzeitarbeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18442-6">
    <skos:broader rdf:resource="thsys/70986"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18442-6">
    <skos:broader rdf:resource="thsys/70878"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18442-6">
    <skos:broader rdf:resource="descriptor/10436-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18959-4">
    <skos:related rdf:resource="descriptor/13325-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18959-4">
    <skos:related rdf:resource="descriptor/18958-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18959-4">
    <skos:related rdf:resource="descriptor/24621-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18959-4">
    <skos:related rdf:resource="descriptor/15009-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14772-5">
    <skos:prefLabel xml:lang="en">Technical ceramics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14772-5">
    <skos:prefLabel xml:lang="de">Technische Keramik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15907-3">
    <skos:prefLabel xml:lang="en">Foreigners</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15907-3">
    <skos:prefLabel xml:lang="de">Ausländer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71105">
    <rdfs:label xml:lang="en">Bond market</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71105">
    <rdfs:label xml:lang="de">Rentenmarkt</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18910-6">
    <skos:altLabel xml:lang="en">Supplement security income</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18910-6">
    <skos:altLabel xml:lang="en">Income maintenance</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18910-6">
    <skos:altLabel xml:lang="de">Soziale Mindestvorsorge</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18910-6">
    <skos:altLabel xml:lang="de">Soziale Grundsicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18910-6">
    <skos:altLabel xml:lang="de">Basiseinkommen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15971-1">
    <skos:scopeNote xml:lang="de">Nur für Entscheidungen von Rechtsfragen durch die zuständigen Stellen/Gerichte.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12451-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12849-6">
    <skos:scopeNote xml:lang="de">U.a. für nationale steuerliche Doppelbelastungen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12065-5">
    <skos:narrower rdf:resource="descriptor/18987-6"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id384">
    <rdfs:seeAlso rdf:resource="descriptor/19534-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10102-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70424">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19140-3">
    <skos:altLabel xml:lang="de">Vertriebsmanagement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70530">
    <skos:narrower rdf:resource="descriptor/19523-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70530">
    <skos:narrower rdf:resource="descriptor/18813-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70530">
    <skos:narrower rdf:resource="descriptor/10297-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70530">
    <skos:narrower rdf:resource="descriptor/19586-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70530">
    <skos:narrower rdf:resource="descriptor/10293-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70530">
    <skos:narrower rdf:resource="descriptor/10438-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70530">
    <skos:narrower rdf:resource="descriptor/19826-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70530">
    <skos:narrower rdf:resource="descriptor/12278-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70530">
    <skos:narrower rdf:resource="descriptor/10291-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70530">
    <skos:narrower rdf:resource="descriptor/10008-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70530">
    <skos:narrower rdf:resource="descriptor/11552-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70530">
    <skos:narrower rdf:resource="descriptor/10294-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70530">
    <skos:narrower rdf:resource="descriptor/10287-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19739-6">
    <skos:prefLabel xml:lang="en">Mission</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19739-6">
    <skos:prefLabel xml:lang="de">Christliche Mission</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13019-0">
    <skos:broader rdf:resource="thsys/71008"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13019-0">
    <skos:broader rdf:resource="descriptor/12922-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13019-0">
    <skos:broader rdf:resource="descriptor/12913-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19270-4">
    <skos:broader rdf:resource="thsys/70187"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19270-4">
    <skos:broader rdf:resource="descriptor/10894-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19302-3">
    <skos:related rdf:resource="descriptor/19305-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19302-3">
    <skos:related rdf:resource="descriptor/19304-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19302-3">
    <skos:related rdf:resource="descriptor/19303-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19302-3">
    <skos:related rdf:resource="descriptor/14098-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71009">
    <rdfs:label xml:lang="en">Mining</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71009">
    <rdfs:label xml:lang="de">Bergbau</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13028-6">
    <skos:altLabel xml:lang="de">Metallgießerei</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13028-6">
    <skos:altLabel xml:lang="de">Gießerei</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18797-4">
    <skos:prefLabel xml:lang="en">Trade fair industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18797-4">
    <skos:prefLabel xml:lang="de">Messewirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18956-3">
    <skos:related rdf:resource="descriptor/16483-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18956-3">
    <skos:related rdf:resource="descriptor/18954-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18956-3">
    <skos:related rdf:resource="descriptor/13362-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18956-3">
    <skos:related rdf:resource="descriptor/15675-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18956-3">
    <skos:related rdf:resource="descriptor/12047-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19220-5">
    <skos:broader rdf:resource="thsys/70493"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19220-5">
    <skos:broader rdf:resource="thsys/70120"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18067-6">
    <skos:broader rdf:resource="thsys/73392"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18067-6">
    <skos:broader rdf:resource="descriptor/16036-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16968-6">
    <skos:prefLabel xml:lang="en">Nidwalden</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16968-6">
    <skos:prefLabel xml:lang="de">Nidwalden</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18117-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18955-5">
    <skos:scopeNote xml:lang="de">Volkswirtschaftliche Analyse des ökonomischen Verhaltens im Sportsektor, individuelle Nutzenabwägungen, usw.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11200-2">
    <skos:prefLabel xml:lang="en">Youth unemployment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11200-2">
    <skos:prefLabel xml:lang="de">Jugendarbeitslosigkeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15149-5">
    <skos:prefLabel xml:lang="en">Statistical figure</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15149-5">
    <skos:prefLabel xml:lang="de">Maßzahl</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11646-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17181-6">
    <skos:broader rdf:resource="thsys/70073"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17181-6">
    <skos:broader rdf:resource="descriptor/19085-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17181-6">
    <skos:broader rdf:resource="descriptor/19084-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17181-6">
    <skos:broader rdf:resource="descriptor/17983-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16131-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16487-1">
    <skos:altLabel xml:lang="de">Wissenschaftsförderung des Staates</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/g">
    <rdfs:label xml:lang="en">Geographic names</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/g">
    <rdfs:label xml:lang="de">Geographische Begriffe</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13782-0">
    <skos:prefLabel xml:lang="en">Cession</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13782-0">
    <skos:prefLabel xml:lang="de">Zession</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16079-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13650-3">
    <skos:related rdf:resource="descriptor/13365-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13650-3">
    <skos:related rdf:resource="descriptor/13197-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13650-3">
    <skos:related rdf:resource="descriptor/13100-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18840-1">
    <skos:related rdf:resource="descriptor/12180-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14223-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17374-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19698-1">
    <skos:related rdf:resource="descriptor/16333-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11807-6">
    <skos:related rdf:resource="descriptor/19631-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11807-6">
    <skos:related rdf:resource="descriptor/19626-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11807-6">
    <skos:related rdf:resource="descriptor/18312-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11807-6">
    <skos:related rdf:resource="descriptor/12964-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11807-6">
    <skos:related rdf:resource="descriptor/11801-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18987-6">
    <skos:related rdf:resource="descriptor/19367-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18987-6">
    <skos:related rdf:resource="descriptor/14150-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18987-6">
    <skos:related rdf:resource="descriptor/12961-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18987-6">
    <skos:related rdf:resource="descriptor/18988-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18987-6">
    <skos:related rdf:resource="descriptor/13874-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18987-6">
    <skos:related rdf:resource="descriptor/13434-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18987-6">
    <skos:related rdf:resource="descriptor/18337-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14702-5">
    <skos:prefLabel xml:lang="en">Plastics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14702-5">
    <skos:prefLabel xml:lang="de">Kunststoff</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18475-5">
    <skos:prefLabel xml:lang="en">Self-insurance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18475-5">
    <skos:prefLabel xml:lang="de">Selbstversicherung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11975-6">
    <skos:altLabel xml:lang="en">Urban centre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11975-6">
    <skos:altLabel xml:lang="en">Inner city</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11975-6">
    <skos:altLabel xml:lang="en">Downtown</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11975-6">
    <skos:altLabel xml:lang="en">City center</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11975-6">
    <skos:altLabel xml:lang="en">Central business district</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11975-6">
    <skos:altLabel xml:lang="de">Stadtkern</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11975-6">
    <skos:altLabel xml:lang="de">Innenstadt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11975-6">
    <skos:altLabel xml:lang="de">Altstadt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12907-4">
    <skos:prefLabel xml:lang="en">Leisure services</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12907-4">
    <skos:prefLabel xml:lang="de">Freizeitwirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18608-5">
    <skos:broader rdf:resource="thsys/70932"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18608-5">
    <skos:broader rdf:resource="thsys/70058"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18608-5">
    <skos:broader rdf:resource="descriptor/11876-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18608-5">
    <skos:broader rdf:resource="descriptor/13246-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18566-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12164-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13490-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19794-5">
    <skos:related rdf:resource="descriptor/14473-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16678-1">
    <skos:broader rdf:resource="thsys/73358"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16678-1">
    <skos:broader rdf:resource="thsys/70878"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16678-1">
    <skos:broader rdf:resource="descriptor/16554-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15025-2">
    <skos:related rdf:resource="descriptor/10471-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15025-2">
    <skos:related rdf:resource="descriptor/16696-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15025-2">
    <skos:related rdf:resource="descriptor/19150-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70120">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.06.03</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25742-3">
    <skos:scopeNote xml:lang="de">Möglichst spezifischer indexieren.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18168-0">
    <skos:related rdf:resource="descriptor/11270-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18168-0">
    <skos:related rdf:resource="descriptor/11264-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18168-0">
    <skos:related rdf:resource="descriptor/11183-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18168-0">
    <skos:related rdf:resource="descriptor/11182-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15717-1">
    <skos:narrower rdf:resource="descriptor/16619-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15717-1">
    <skos:narrower rdf:resource="descriptor/16611-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15717-1">
    <skos:narrower rdf:resource="descriptor/16658-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15717-1">
    <skos:narrower rdf:resource="descriptor/16608-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15717-1">
    <skos:narrower rdf:resource="descriptor/11070-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15717-1">
    <skos:narrower rdf:resource="descriptor/19748-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11370-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12508-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13358-6">
    <skos:altLabel xml:lang="de">Stadtbücherei</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13358-6">
    <skos:altLabel xml:lang="de">Stadtbibliothek</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13358-6">
    <skos:altLabel xml:lang="de">Öffentliches Bibliothekswesen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13358-6">
    <skos:altLabel xml:lang="de">Öffentliche Bücherei</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13358-6">
    <skos:altLabel xml:lang="de">Kommunale Bibliothek</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10327-4">
    <skos:related rdf:resource="descriptor/10332-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10327-4">
    <skos:related rdf:resource="descriptor/10187-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10327-4">
    <skos:related rdf:resource="descriptor/10313-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10327-4">
    <skos:related rdf:resource="descriptor/12741-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10327-4">
    <skos:related rdf:resource="descriptor/10229-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12021-4">
    <skos:broader rdf:resource="thsys/73351"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12021-4">
    <skos:broader rdf:resource="thsys/71066"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12021-4">
    <skos:broader rdf:resource="thsys/70239"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12021-4">
    <skos:broader rdf:resource="descriptor/15770-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12744-6">
    <skos:broader rdf:resource="thsys/70270"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12744-6">
    <skos:broader rdf:resource="descriptor/12041-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19123-3">
    <skos:altLabel xml:lang="de">Gesättigter Markt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11777-3">
    <skos:altLabel xml:lang="de">Vermögensbildungsrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11777-3">
    <skos:altLabel xml:lang="de">Vermögensbildungsgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11777-3">
    <skos:altLabel xml:lang="de">Vermögensbildungsförderung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11777-3">
    <skos:altLabel xml:lang="de">Vermögensbeteiligungsgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13348-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19349-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18596-0">
    <skos:broader rdf:resource="thsys/70934"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18596-0">
    <skos:broader rdf:resource="thsys/70933"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18596-0">
    <skos:broader rdf:resource="descriptor/13537-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12214-0">
    <skos:broader rdf:resource="thsys/70875"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16038-2">
    <skos:prefLabel xml:lang="en">Organization law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16038-2">
    <skos:prefLabel xml:lang="de">Verbandsrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19497-4">
    <skos:altLabel xml:lang="de">Qualifizierungsgesellschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19497-4">
    <skos:altLabel xml:lang="de">Arbeitsförderungsgesellschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19497-4">
    <skos:altLabel xml:lang="de">ABS-Gesellschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16156-3">
    <skos:broader rdf:resource="thsys/73335"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16156-3">
    <skos:broader rdf:resource="descriptor/16159-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12138-4">
    <skos:prefLabel xml:lang="en">Centralized management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12138-4">
    <skos:prefLabel xml:lang="de">Zentrale Organisation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70859">
    <rdfs:label xml:lang="en">Public sector management</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70859">
    <rdfs:label xml:lang="de">Öffentliche Betriebswirtschaftslehre</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13389-2">
    <skos:related rdf:resource="descriptor/12881-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13389-2">
    <skos:related rdf:resource="descriptor/15972-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13389-2">
    <skos:related rdf:resource="descriptor/13995-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13389-2">
    <skos:related rdf:resource="descriptor/15966-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13389-2">
    <skos:related rdf:resource="descriptor/13410-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13389-2">
    <skos:related rdf:resource="descriptor/13871-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13590-2">
    <skos:prefLabel xml:lang="en">Road costs</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13590-2">
    <skos:prefLabel xml:lang="de">Wegekosten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13757-6">
    <skos:altLabel xml:lang="de">Devisengeschäft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13757-6">
    <skos:altLabel xml:lang="de">Devisenbörse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10144-5">
    <skos:prefLabel xml:lang="en">Microeconomics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10144-5">
    <skos:prefLabel xml:lang="de">Mikroökonomik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18150-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18956-3">
    <skos:scopeNote xml:lang="de">Professionelle und Amateursportvereine</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24859-1">
    <skos:altLabel xml:lang="en">Payment practices</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24859-1">
    <skos:altLabel xml:lang="en">Payment behavior</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24859-1">
    <skos:altLabel xml:lang="de">Zahlungsbereitschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15930-1">
    <skos:broader rdf:resource="thsys/70572"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15930-1">
    <skos:broader rdf:resource="descriptor/15909-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18959-4">
    <skos:broader rdf:resource="thsys/70818"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14073-3">
    <skos:prefLabel xml:lang="en">Plant</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14073-3">
    <skos:prefLabel xml:lang="de">Pflanze</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10237-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14537-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16629-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13724-0">
    <skos:prefLabel xml:lang="en">Over-the-counter market</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13724-0">
    <skos:prefLabel xml:lang="de">OTC-Handel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71014">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.04.01.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19545-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19418-5">
    <skos:altLabel xml:lang="de">Cashflow-Steuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19418-5">
    <skos:altLabel xml:lang="de">Cash-Flow Steuer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18683-5">
    <skos:broader rdf:resource="thsys/70897"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18683-5">
    <skos:broader rdf:resource="thsys/70187"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18683-5">
    <skos:broader rdf:resource="descriptor/12212-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18683-5">
    <skos:broader rdf:resource="descriptor/10803-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15883-5">
    <skos:altLabel xml:lang="en">Maritime regions</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15883-5">
    <skos:altLabel xml:lang="de">Wattenmeer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15883-5">
    <skos:altLabel xml:lang="de">Meeresküste</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15883-5">
    <skos:altLabel xml:lang="de">Küste</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14466-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73364">
    <rdfs:label xml:lang="de">Internationale Konflikte</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19627-3">
    <skos:broader rdf:resource="thsys/73351"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19627-3">
    <skos:broader rdf:resource="descriptor/15746-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15336-6">
    <skos:altLabel xml:lang="en">Multiple regression model</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15336-6">
    <skos:altLabel xml:lang="de">Multivariate Regression</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15336-6">
    <skos:altLabel xml:lang="de">Mehrfachregression</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15336-6">
    <skos:altLabel xml:lang="de">Mehrfaches Regressionsmodell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15336-6">
    <skos:altLabel xml:lang="de">Mehrfache Regression</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19319-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16761-0">
    <skos:scopeNote xml:lang="de">Gegenstand ist die individuelle und soziale menschliche Kommunikation.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19011-0">
    <skos:related rdf:resource="descriptor/11341-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19011-0">
    <skos:related rdf:resource="descriptor/19440-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19011-0">
    <skos:related rdf:resource="descriptor/19756-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19011-0">
    <skos:related rdf:resource="descriptor/19499-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19011-0">
    <skos:related rdf:resource="descriptor/15713-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19011-0">
    <skos:related rdf:resource="descriptor/11211-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19011-0">
    <skos:related rdf:resource="descriptor/11189-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19011-0">
    <skos:related rdf:resource="descriptor/12631-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11048-3">
    <skos:prefLabel xml:lang="en">Socialist market economy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11048-3">
    <skos:prefLabel xml:lang="de">Marktsozialismus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18207-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19142-6">
    <skos:prefLabel xml:lang="en">Balancing of accounts</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19142-6">
    <skos:prefLabel xml:lang="de">Bilanzierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17748-1">
    <skos:prefLabel xml:lang="en">Libya</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17748-1">
    <skos:prefLabel xml:lang="de">Libyen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15655-4">
    <skos:related rdf:resource="descriptor/13355-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15655-4">
    <skos:related rdf:resource="descriptor/18927-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15655-4">
    <skos:related rdf:resource="descriptor/18926-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15655-4">
    <skos:related rdf:resource="descriptor/14040-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15655-4">
    <skos:related rdf:resource="descriptor/18070-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15655-4">
    <skos:related rdf:resource="descriptor/19747-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18907-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70934">
    <skos:broader rdf:resource="thsys/70936"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70037">
    <skos:narrower rdf:resource="descriptor/10135-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70037">
    <skos:narrower rdf:resource="descriptor/13807-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70037">
    <skos:narrower rdf:resource="descriptor/12302-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70037">
    <skos:narrower rdf:resource="descriptor/24859-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70037">
    <skos:narrower rdf:resource="descriptor/18539-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70037">
    <skos:narrower rdf:resource="descriptor/12205-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70037">
    <skos:narrower rdf:resource="descriptor/19557-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70037">
    <skos:narrower rdf:resource="descriptor/12249-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70037">
    <skos:narrower rdf:resource="descriptor/12226-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70037">
    <skos:narrower rdf:resource="descriptor/10120-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70037">
    <skos:narrower rdf:resource="descriptor/12206-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70037">
    <skos:narrower rdf:resource="descriptor/25742-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70037">
    <skos:narrower rdf:resource="descriptor/18084-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70037">
    <skos:narrower rdf:resource="descriptor/12218-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70037">
    <skos:narrower rdf:resource="descriptor/12228-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70037">
    <skos:narrower rdf:resource="descriptor/12053-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70037">
    <skos:narrower rdf:resource="descriptor/12215-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70037">
    <skos:narrower rdf:resource="descriptor/12309-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70037">
    <skos:narrower rdf:resource="thsys/70036"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70037">
    <skos:narrower rdf:resource="thsys/70183"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14924-1">
    <skos:prefLabel xml:lang="en">Baby food</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14924-1">
    <skos:prefLabel xml:lang="de">Babynahrung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16063-3">
    <skos:broader rdf:resource="thsys/73391"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16063-3">
    <skos:broader rdf:resource="descriptor/16050-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19521-5">
    <skos:broader rdf:resource="thsys/70853"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19521-5">
    <skos:broader rdf:resource="thsys/70504"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19521-5">
    <skos:broader rdf:resource="thsys/70052"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19521-5">
    <skos:broader rdf:resource="descriptor/11180-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13655-0">
    <skos:broader rdf:resource="thsys/70921"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13655-0">
    <skos:broader rdf:resource="thsys/70824"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13655-0">
    <skos:broader rdf:resource="descriptor/13278-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13655-0">
    <skos:broader rdf:resource="descriptor/13658-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18833-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13207-6">
    <skos:related rdf:resource="descriptor/19421-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13207-6">
    <skos:related rdf:resource="descriptor/13157-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13207-6">
    <skos:related rdf:resource="descriptor/14976-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16340-3">
    <skos:prefLabel xml:lang="en">Parliament</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16340-3">
    <skos:prefLabel xml:lang="de">Parlament</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11284-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18607-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18393-0">
    <skos:related rdf:resource="descriptor/14317-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18393-0">
    <skos:related rdf:resource="descriptor/12984-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18393-0">
    <skos:related rdf:resource="descriptor/19047-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18393-0">
    <skos:related rdf:resource="descriptor/12983-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19007-5">
    <skos:prefLabel xml:lang="en">Personnel marketing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19007-5">
    <skos:prefLabel xml:lang="de">Personalmarketing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13436-5">
    <skos:broader rdf:resource="thsys/71057"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13436-5">
    <skos:broader rdf:resource="thsys/71028"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13436-5">
    <skos:broader rdf:resource="descriptor/11276-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19133-0">
    <skos:prefLabel xml:lang="en">Customer relationship management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19133-0">
    <skos:prefLabel xml:lang="de">Customer Relationship Management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18492-5">
    <skos:altLabel xml:lang="de">Baustil</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18492-5">
    <skos:altLabel xml:lang="de">Baukunst</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13823-5">
    <skos:narrower rdf:resource="descriptor/19515-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13823-5">
    <skos:narrower rdf:resource="descriptor/19513-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11576-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18312-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19479-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12357-6">
    <skos:related rdf:resource="descriptor/12458-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12357-6">
    <skos:related rdf:resource="descriptor/12363-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12357-6">
    <skos:related rdf:resource="descriptor/12326-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12357-6">
    <skos:related rdf:resource="descriptor/19142-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12357-6">
    <skos:related rdf:resource="descriptor/12216-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19125-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10945-6">
    <skos:broader rdf:resource="thsys/70879"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10945-6">
    <skos:broader rdf:resource="thsys/70242"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10945-6">
    <skos:broader rdf:resource="descriptor/19565-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10945-6">
    <skos:broader rdf:resource="descriptor/10895-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10169-3">
    <skos:prefLabel xml:lang="en">Labour supply</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10169-3">
    <skos:prefLabel xml:lang="de">Arbeitsangebot</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="http://zbw.eu/namespaces/skos-extensions/Thsys">
    <rdfs:isDefinedBy rdf:resource="../namespaces/skos-extensions/skos-extensions.rdf"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11004-2">
    <skos:related rdf:resource="descriptor/11033-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11004-2">
    <skos:related rdf:resource="descriptor/10987-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11004-2">
    <skos:related rdf:resource="descriptor/11030-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11004-2">
    <skos:related rdf:resource="descriptor/11051-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11004-2">
    <skos:related rdf:resource="descriptor/10383-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11004-2">
    <skos:related rdf:resource="descriptor/10040-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19793-0">
    <skos:related rdf:resource="descriptor/15895-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12404-2">
    <skos:altLabel xml:lang="de">Sachumlaufvermögen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10274-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14353-4">
    <skos:altLabel xml:lang="de">Turboluftstrahl-Triebwerk</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14353-4">
    <skos:altLabel xml:lang="de">Triebwerk</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14353-4">
    <skos:altLabel xml:lang="de">Raketenantrieb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14353-4">
    <skos:altLabel xml:lang="de">Flugzeugmotor</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17576-4">
    <skos:prefLabel xml:lang="en">Brunei</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17576-4">
    <skos:prefLabel xml:lang="de">Brunei</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70242">
    <skos:broader rdf:resource="thsys/70101"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10987-4">
    <skos:altLabel xml:lang="en">Macroeconomic policy</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10987-4">
    <skos:altLabel xml:lang="de">Volkswirtschaftspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10987-4">
    <skos:altLabel xml:lang="de">Allgemeine Wirtschaftspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12613-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10614-1">
    <skos:broader rdf:resource="thsys/70179"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10614-1">
    <skos:broader rdf:resource="descriptor/11186-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18447-3">
    <skos:broader rdf:resource="thsys/70982"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18447-3">
    <skos:broader rdf:resource="thsys/70979"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18447-3">
    <skos:broader rdf:resource="descriptor/16460-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18447-3">
    <skos:broader rdf:resource="descriptor/11820-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12418-5">
    <skos:broader rdf:resource="thsys/71043"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12418-5">
    <skos:broader rdf:resource="descriptor/12408-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11032-4">
    <skos:prefLabel xml:lang="en">Household account</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11032-4">
    <skos:prefLabel xml:lang="de">Haushaltsbudget-Statistik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19087-2">
    <skos:scopeNote xml:lang="de">Vgl. auch freie Schlagwörter "Serbien" und "Montenegro".</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19087-2">
    <skos:scopeNote xml:lang="de">Serbien und Montenegro gründeten 1992 die Bundesrepublik Jugoslawien als Nachfolgestaat der Sozialistischen Föderativen Republik Jugoslawien</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11562-3">
    <skos:broader rdf:resource="thsys/71080"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11562-3">
    <skos:broader rdf:resource="thsys/71020"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11562-3">
    <skos:broader rdf:resource="descriptor/11572-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11562-3">
    <skos:broader rdf:resource="descriptor/11801-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19769-4">
    <skos:prefLabel xml:lang="en">Social stability</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19769-4">
    <skos:prefLabel xml:lang="de">Soziale Stabilität</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19015-6">
    <skos:broader rdf:resource="thsys/70403"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19015-6">
    <skos:broader rdf:resource="descriptor/14169-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71096">
    <skos:prefLabel xml:lang="en">V.07.02.02  Tariffs</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71096">
    <skos:prefLabel xml:lang="de">V.07.02.02  Zoll</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10573-3">
    <skos:narrower rdf:resource="descriptor/10610-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14143-1">
    <skos:prefLabel xml:lang="en">Sheep</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14143-1">
    <skos:prefLabel xml:lang="de">Schaf</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70028">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">B.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10829-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15935-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11209-5">
    <skos:altLabel xml:lang="en">Illegal employment</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11209-5">
    <skos:altLabel xml:lang="de">Schwarzarbeitsgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11209-5">
    <skos:altLabel xml:lang="de">Illegale Beschäftigung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17968-1">
    <skos:prefLabel xml:lang="en">Pacific Ocean</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17968-1">
    <skos:prefLabel xml:lang="de">Pazifischer Ozean</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19858-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13303-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12543-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18785-4">
    <skos:prefLabel xml:lang="en">Research expenditure</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18785-4">
    <skos:prefLabel xml:lang="de">Forschungskosten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19233-3">
    <skos:broader rdf:resource="thsys/71114"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19233-3">
    <skos:broader rdf:resource="thsys/71065"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19233-3">
    <skos:broader rdf:resource="descriptor/11872-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13583-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19807-1">
    <skos:broader rdf:resource="thsys/73339"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19807-1">
    <skos:broader rdf:resource="descriptor/15611-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19770-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19820-2">
    <skos:altLabel xml:lang="de">Angelsächsisches Rechtssystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13591-0">
    <skos:altLabel xml:lang="en">Haulage time</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13591-0">
    <skos:altLabel xml:lang="de">Reisezeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13591-0">
    <skos:altLabel xml:lang="de">Reisegeschwindigkeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13591-0">
    <skos:altLabel xml:lang="de">Reisedauer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13591-0">
    <skos:altLabel xml:lang="de">Fahrzeitvergleich</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13591-0">
    <skos:altLabel xml:lang="de">Fahrzeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14064-4">
    <skos:altLabel xml:lang="de">Pulverkaffee</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14064-4">
    <skos:altLabel xml:lang="de">Kaffee-Ersatz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18458-5">
    <skos:related rdf:resource="descriptor/18349-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16382-1">
    <skos:prefLabel xml:lang="en">Citizen's action committee</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16382-1">
    <skos:prefLabel xml:lang="de">Bürgerinitiative</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11555-0">
    <skos:broader rdf:resource="thsys/70839"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11555-0">
    <skos:broader rdf:resource="thsys/70573"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11555-0">
    <skos:broader rdf:resource="descriptor/10011-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16535-2">
    <skos:prefLabel xml:lang="en">Youth services</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16535-2">
    <skos:prefLabel xml:lang="de">Jugendhilfe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18137-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12855-4">
    <skos:broader rdf:resource="thsys/73337"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12855-4">
    <skos:broader rdf:resource="thsys/70397"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12855-4">
    <skos:broader rdf:resource="thsys/70036"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12855-4">
    <skos:broader rdf:resource="descriptor/16104-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12855-4">
    <skos:broader rdf:resource="descriptor/12276-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14102-1">
    <skos:prefLabel xml:lang="en">Grape</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14102-1">
    <skos:prefLabel xml:lang="de">Weintraube</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18301-3">
    <skos:prefLabel xml:lang="en">Oecotrophology</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18301-3">
    <skos:prefLabel xml:lang="de">Ökotrophologie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71085">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13953-6">
    <skos:prefLabel xml:lang="en">Construction workers</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13953-6">
    <skos:prefLabel xml:lang="de">Bauberufe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11262-1">
    <skos:prefLabel xml:lang="en">Employees</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11262-1">
    <skos:prefLabel xml:lang="de">Arbeitskräfte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12845-0">
    <skos:prefLabel xml:lang="en">Advertising industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12845-0">
    <skos:prefLabel xml:lang="de">Werbewirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71058">
    <skos:prefLabel xml:lang="de">V.13.05  Berufe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11853-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10115-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12787-2">
    <skos:altLabel xml:lang="de">Lieferungsbedingungen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12787-2">
    <skos:altLabel xml:lang="de">Lieferklauseln</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12787-2">
    <skos:altLabel xml:lang="de">Lieferbedingungen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12787-2">
    <skos:altLabel xml:lang="de">Handelsklauseln</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19818-3">
    <skos:altLabel xml:lang="de">Standesrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19818-3">
    <skos:altLabel xml:lang="de">Berufsordnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18703-4">
    <skos:prefLabel xml:lang="en">EU bank law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18703-4">
    <skos:prefLabel xml:lang="de">EU-Bankrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12039-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13469-4">
    <skos:prefLabel xml:lang="en">Trading cooperation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13469-4">
    <skos:prefLabel xml:lang="de">Handelskooperation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13221-5">
    <skos:broader rdf:resource="thsys/70961"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13221-5">
    <skos:broader rdf:resource="thsys/70957"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13221-5">
    <skos:broader rdf:resource="descriptor/13210-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12029-2">
    <skos:related rdf:resource="descriptor/21061-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12029-2">
    <skos:related rdf:resource="descriptor/18747-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12029-2">
    <skos:related rdf:resource="descriptor/12031-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12029-2">
    <skos:related rdf:resource="descriptor/18750-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12029-2">
    <skos:related rdf:resource="descriptor/12028-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12029-2">
    <skos:related rdf:resource="descriptor/11659-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12029-2">
    <skos:related rdf:resource="descriptor/12026-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12029-2">
    <skos:related rdf:resource="descriptor/12015-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18459-3">
    <skos:related rdf:resource="descriptor/19464-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15258-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18543-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10891-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18644-1">
    <skos:altLabel xml:lang="de">Postverwaltung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18644-1">
    <skos:altLabel xml:lang="de">Post- und Fernmeldeverwaltung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10092-5">
    <skos:altLabel xml:lang="en">External economies</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10092-5">
    <skos:altLabel xml:lang="en">Diseconomies</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10092-5">
    <skos:altLabel xml:lang="de">Externalität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12384-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11834-3">
    <skos:broader rdf:resource="thsys/70927"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11834-3">
    <skos:broader rdf:resource="descriptor/10213-5"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id59">
    <rdf:value xml:lang="de">Für Bankbürgschaften</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13588-3">
    <skos:altLabel xml:lang="en">Economics of transportation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13588-3">
    <skos:altLabel xml:lang="de">Verkehrswirtschaftstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13588-3">
    <skos:altLabel xml:lang="de">Verkehrswirtschaftslehre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13588-3">
    <skos:altLabel xml:lang="de">Verkehrsökonomie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13588-3">
    <skos:altLabel xml:lang="de">Transportökonomik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20442-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18192-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11001-1">
    <skos:altLabel xml:lang="de">Fünfjahresplan</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14899-2">
    <skos:narrower rdf:resource="descriptor/14923-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14899-2">
    <skos:narrower rdf:resource="descriptor/14921-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14899-2">
    <skos:narrower rdf:resource="descriptor/14915-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14899-2">
    <skos:narrower rdf:resource="descriptor/14911-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14899-2">
    <skos:narrower rdf:resource="descriptor/14909-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14899-2">
    <skos:narrower rdf:resource="descriptor/14906-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14899-2">
    <skos:narrower rdf:resource="descriptor/14902-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14899-2">
    <skos:narrower rdf:resource="descriptor/14900-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19674-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70020">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.04.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14665-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70843">
    <skos:narrower rdf:resource="descriptor/15684-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70843">
    <skos:narrower rdf:resource="descriptor/11391-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70843">
    <skos:narrower rdf:resource="descriptor/11390-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70843">
    <skos:narrower rdf:resource="descriptor/11267-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70843">
    <skos:narrower rdf:resource="descriptor/18143-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70843">
    <skos:narrower rdf:resource="descriptor/19543-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70843">
    <skos:narrower rdf:resource="descriptor/18885-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70843">
    <skos:narrower rdf:resource="descriptor/11366-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70843">
    <skos:narrower rdf:resource="descriptor/11365-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70843">
    <skos:narrower rdf:resource="descriptor/11393-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70843">
    <skos:narrower rdf:resource="descriptor/16134-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70843">
    <skos:narrower rdf:resource="descriptor/11392-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70843">
    <skos:narrower rdf:resource="descriptor/11360-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70843">
    <skos:narrower rdf:resource="descriptor/11362-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70843">
    <skos:narrower rdf:resource="descriptor/11228-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70843">
    <skos:narrower rdf:resource="descriptor/11361-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70843">
    <skos:narrower rdf:resource="descriptor/11288-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70843">
    <skos:narrower rdf:resource="descriptor/19656-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70843">
    <skos:narrower rdf:resource="descriptor/11384-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70843">
    <skos:narrower rdf:resource="descriptor/11406-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70843">
    <skos:narrower rdf:resource="descriptor/11358-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12031-1">
    <skos:altLabel xml:lang="de">Grundstücksmarkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10093-3">
    <skos:broader rdf:resource="thsys/70057"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10093-3">
    <skos:broader rdf:resource="descriptor/10113-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19393-2">
    <skos:scopeNote xml:lang="de">Nur für den Markt pflanzlicher bzw. tierischer Öle und Fette.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70989">
    <skos:narrower rdf:resource="descriptor/13007-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70989">
    <skos:narrower rdf:resource="descriptor/13006-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70989">
    <skos:narrower rdf:resource="descriptor/18411-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70989">
    <skos:narrower rdf:resource="descriptor/12919-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70989">
    <skos:narrower rdf:resource="descriptor/13000-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70989">
    <skos:narrower rdf:resource="descriptor/19366-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70989">
    <skos:narrower rdf:resource="descriptor/13004-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19607-2">
    <skos:prefLabel xml:lang="en">Legal order</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19607-2">
    <skos:prefLabel xml:lang="de">Rechtsordnung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12295-2">
    <skos:related rdf:resource="descriptor/12073-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12295-2">
    <skos:related rdf:resource="descriptor/11146-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12295-2">
    <skos:related rdf:resource="descriptor/18117-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12295-2">
    <skos:related rdf:resource="descriptor/12272-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12295-2">
    <skos:related rdf:resource="descriptor/10460-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12295-2">
    <skos:related rdf:resource="descriptor/19265-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18352-0">
    <skos:prefLabel xml:lang="en">Mining law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18352-0">
    <skos:prefLabel xml:lang="de">Bergrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14976-3">
    <skos:broader rdf:resource="thsys/73300"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14976-3">
    <skos:broader rdf:resource="descriptor/14931-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15686-0">
    <skos:broader rdf:resource="thsys/73393"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15686-0">
    <skos:broader rdf:resource="thsys/70878"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15686-0">
    <skos:broader rdf:resource="descriptor/10436-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11844-0">
    <skos:broader rdf:resource="thsys/73367"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11844-0">
    <skos:broader rdf:resource="thsys/71064"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11844-0">
    <skos:broader rdf:resource="descriptor/16549-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12513-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70176">
    <rdfs:label xml:lang="en">Education</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70176">
    <rdfs:label xml:lang="de">Bildung</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11261-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14446-4">
    <skos:prefLabel xml:lang="en">Boat</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14446-4">
    <skos:prefLabel xml:lang="de">Boot</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18184-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19067-1">
    <skos:prefLabel xml:lang="en">Typing services</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19067-1">
    <skos:prefLabel xml:lang="de">Schreibbüro</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16186-1">
    <skos:broader rdf:resource="thsys/73333"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16186-1">
    <skos:broader rdf:resource="descriptor/15966-1"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id56">
    <rdfs:seeAlso rdf:resource="descriptor/11354-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12278-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14755-5">
    <skos:broader rdf:resource="thsys/73305"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14755-5">
    <skos:broader rdf:resource="descriptor/14751-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19257-3">
    <skos:altLabel xml:lang="en">Cash-in-advance Modell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19150-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12129-5">
    <skos:prefLabel xml:lang="en">Organisational means</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12129-5">
    <skos:prefLabel xml:lang="de">Organisationsmittel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16182-2">
    <skos:broader rdf:resource="thsys/73331"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16182-2">
    <skos:broader rdf:resource="descriptor/16186-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11314-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18810-3">
    <skos:broader rdf:resource="thsys/73380"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18810-3">
    <skos:broader rdf:resource="thsys/70867"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18810-3">
    <skos:broader rdf:resource="descriptor/13413-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10605-2">
    <skos:prefLabel xml:lang="en">External sector</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10605-2">
    <skos:prefLabel xml:lang="de">Außenwirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19754-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19297-5">
    <skos:related rdf:resource="descriptor/19300-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19297-5">
    <skos:related rdf:resource="descriptor/19298-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19297-5">
    <skos:related rdf:resource="descriptor/18197-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19297-5">
    <skos:related rdf:resource="descriptor/14095-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73334">
    <rdfs:label xml:lang="en">Criminal law</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73334">
    <rdfs:label xml:lang="de">Strafrecht</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16912-5">
    <skos:broader rdf:resource="thsys/70361"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16912-5">
    <skos:broader rdf:resource="descriptor/18012-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70994">
    <skos:broader rdf:resource="thsys/71004"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13825-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19276-6">
    <skos:altLabel xml:lang="de">Einlagenzertifikat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16043-2">
    <skos:broader rdf:resource="thsys/73391"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16043-2">
    <skos:broader rdf:resource="thsys/71106"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16043-2">
    <skos:broader rdf:resource="thsys/70087"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16043-2">
    <skos:broader rdf:resource="descriptor/16050-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20409-3">
    <skos:broader rdf:resource="thsys/71110"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20409-3">
    <skos:broader rdf:resource="thsys/71029"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19005-2">
    <skos:related rdf:resource="descriptor/11341-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19005-2">
    <skos:related rdf:resource="descriptor/19491-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19005-2">
    <skos:related rdf:resource="descriptor/11299-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19005-2">
    <skos:related rdf:resource="descriptor/11294-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19005-2">
    <skos:related rdf:resource="descriptor/18792-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19005-2">
    <skos:related rdf:resource="descriptor/11246-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19005-2">
    <skos:related rdf:resource="descriptor/11184-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12540-1">
    <skos:prefLabel xml:lang="en">Original costs</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12540-1">
    <skos:prefLabel xml:lang="de">Selbstkosten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19009-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14755-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16641-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18110-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15251-0">
    <skos:broader rdf:resource="thsys/72186"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15251-0">
    <skos:broader rdf:resource="descriptor/15117-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15251-0">
    <skos:broader rdf:resource="descriptor/15272-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73378">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12026-1">
    <skos:prefLabel xml:lang="en">Real property</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12026-1">
    <skos:prefLabel xml:lang="de">Grundbesitz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13307-2">
    <skos:prefLabel xml:lang="en">Hospitality industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13307-2">
    <skos:prefLabel xml:lang="de">Gastgewerbe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17636-5">
    <skos:altLabel xml:lang="de">Arabische Republik Syrien</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13560-4">
    <skos:altLabel xml:lang="en">Street</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13560-4">
    <skos:altLabel xml:lang="de">Straßennetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13560-4">
    <skos:altLabel xml:lang="de">Straßeninfrastruktur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13560-4">
    <skos:altLabel xml:lang="de">Landstraße</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13560-4">
    <skos:altLabel xml:lang="de">Fernstraße</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19080-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16357-0">
    <skos:prefLabel xml:lang="en">Electoral system</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16357-0">
    <skos:prefLabel xml:lang="de">Wahlsystem</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18906-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19521-5">
    <skos:prefLabel xml:lang="en">Voluntary work</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19521-5">
    <skos:prefLabel xml:lang="de">Ehrenamtliche Arbeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13636-4">
    <skos:broader rdf:resource="thsys/70944"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13636-4">
    <skos:broader rdf:resource="thsys/70924"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13636-4">
    <skos:broader rdf:resource="descriptor/13644-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20381-6">
    <skos:altLabel xml:lang="en">Staff reductions</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20381-6">
    <skos:altLabel xml:lang="en">Staff cuts</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20381-6">
    <skos:altLabel xml:lang="de">Stellenabbau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20381-6">
    <skos:altLabel xml:lang="de">Personalreduzierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20381-6">
    <skos:altLabel xml:lang="de">Personaleinschränkung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16470-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18810-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18139-0">
    <skos:broader rdf:resource="thsys/73340"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18139-0">
    <skos:broader rdf:resource="thsys/71037"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18139-0">
    <skos:broader rdf:resource="descriptor/15621-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10966-5">
    <skos:narrower rdf:resource="descriptor/18851-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10966-5">
    <skos:narrower rdf:resource="descriptor/11818-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10966-5">
    <skos:narrower rdf:resource="descriptor/11822-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10966-5">
    <skos:narrower rdf:resource="descriptor/18840-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10966-5">
    <skos:narrower rdf:resource="descriptor/11304-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="../namespaces/skos-extensions/Thsys">
    <rdf:type rdf:resource="http://www.w3.org/2002/07/owl#Class"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70995">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19224-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10192-1">
    <skos:altLabel xml:lang="de">Zielsystem des Betriebs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10192-1">
    <skos:altLabel xml:lang="de">Zielkonflikt des Betriebs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10192-1">
    <skos:altLabel xml:lang="de">Zielfunktion des Betriebs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10192-1">
    <skos:altLabel xml:lang="de">Unternehmensziel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10192-1">
    <skos:altLabel xml:lang="de">Betriebsziel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18435-3">
    <skos:broader rdf:resource="thsys/70987"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18435-3">
    <skos:broader rdf:resource="descriptor/15677-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17570-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10962-6">
    <skos:related rdf:resource="descriptor/10904-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10962-6">
    <skos:related rdf:resource="descriptor/10923-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10962-6">
    <skos:related rdf:resource="descriptor/24662-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10513-0">
    <skos:narrower rdf:resource="descriptor/11895-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10513-0">
    <skos:narrower rdf:resource="descriptor/10500-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10513-0">
    <skos:narrower rdf:resource="descriptor/19467-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14473-1">
    <skos:prefLabel xml:lang="en">Electrical engineering product</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14473-1">
    <skos:prefLabel xml:lang="de">Elektrotechnisches Produkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11414-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10786-0">
    <skos:prefLabel xml:lang="en">Capital account</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10786-0">
    <skos:prefLabel xml:lang="de">Kapitalbilanz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11900-2">
    <skos:broader rdf:resource="thsys/73367"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11900-2">
    <skos:broader rdf:resource="thsys/71068"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11900-2">
    <skos:broader rdf:resource="descriptor/11004-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14824-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13499-2">
    <skos:altLabel xml:lang="de">Überregionaler Verkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13499-2">
    <skos:altLabel xml:lang="de">Überlandverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13499-2">
    <skos:altLabel xml:lang="de">Langstreckenverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15700-4">
    <skos:related rdf:resource="descriptor/16214-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15700-4">
    <skos:related rdf:resource="descriptor/16273-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18744-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10223-2">
    <skos:altLabel xml:lang="en">Market imperfection</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10223-2">
    <skos:altLabel xml:lang="en">Imperfect market</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10223-2">
    <skos:altLabel xml:lang="de">Marktunvollkommenheit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13072-3">
    <skos:narrower rdf:resource="descriptor/13073-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13072-3">
    <skos:narrower rdf:resource="descriptor/13071-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19107-1">
    <skos:broader rdf:resource="thsys/71042"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19107-1">
    <skos:broader rdf:resource="thsys/70485"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19107-1">
    <skos:broader rdf:resource="descriptor/19069-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16362-0">
    <skos:prefLabel xml:lang="en">Conservative party</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16362-0">
    <skos:prefLabel xml:lang="de">Konservative Partei</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18065-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19318-2">
    <skos:broader rdf:resource="thsys/71098"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19318-2">
    <skos:broader rdf:resource="descriptor/11572-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19318-2">
    <skos:broader rdf:resource="descriptor/10684-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15655-4">
    <skos:broader rdf:resource="thsys/70828"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15655-4">
    <skos:broader rdf:resource="thsys/70581"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15655-4">
    <skos:broader rdf:resource="descriptor/15654-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15664-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19463-0">
    <skos:related rdf:resource="descriptor/19465-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19463-0">
    <skos:related rdf:resource="descriptor/18865-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19463-0">
    <skos:related rdf:resource="descriptor/18631-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19463-0">
    <skos:related rdf:resource="descriptor/10503-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19463-0">
    <skos:related rdf:resource="descriptor/19368-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19463-0">
    <skos:related rdf:resource="descriptor/10575-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19463-0">
    <skos:related rdf:resource="descriptor/18358-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19463-0">
    <skos:related rdf:resource="descriptor/18800-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11635-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13569-0">
    <skos:broader rdf:resource="thsys/73389"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13569-0">
    <skos:broader rdf:resource="thsys/70943"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13569-0">
    <skos:broader rdf:resource="descriptor/16007-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19051-2">
    <skos:narrower rdf:resource="descriptor/14270-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19051-2">
    <skos:narrower rdf:resource="descriptor/14281-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19051-2">
    <skos:narrower rdf:resource="descriptor/14262-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18290-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18681-2">
    <skos:altLabel xml:lang="de">Wertpapiervermögen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18681-2">
    <skos:altLabel xml:lang="de">Wertpapiersparen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18681-2">
    <skos:altLabel xml:lang="de">Wertpapier-Portfolio</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11864-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13979-2">
    <skos:narrower rdf:resource="descriptor/13983-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13979-2">
    <skos:narrower rdf:resource="descriptor/13981-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13979-2">
    <skos:narrower rdf:resource="descriptor/13980-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13979-2">
    <skos:narrower rdf:resource="descriptor/18599-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13979-2">
    <skos:narrower rdf:resource="descriptor/13966-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13979-2">
    <skos:narrower rdf:resource="descriptor/18569-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14759-4">
    <skos:prefLabel xml:lang="en">Magnetic storage</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14759-4">
    <skos:prefLabel xml:lang="de">Magnetischer Speicher</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18427-2">
    <skos:related rdf:resource="descriptor/18251-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19666-0">
    <skos:prefLabel xml:lang="en">Macroeconomic effect</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19666-0">
    <skos:prefLabel xml:lang="de">Makroökonomischer Einfluss</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10177-4">
    <skos:altLabel xml:lang="de">Ökonomische Theorie des privaten Haushalts</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10177-4">
    <skos:altLabel xml:lang="de">Haushaltsökonomie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16815-3">
    <skos:narrower rdf:resource="descriptor/17199-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16815-3">
    <skos:narrower rdf:resource="descriptor/17179-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16815-3">
    <skos:narrower rdf:resource="descriptor/17016-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16815-3">
    <skos:narrower rdf:resource="descriptor/19084-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16815-3">
    <skos:narrower rdf:resource="descriptor/19077-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16815-3">
    <skos:narrower rdf:resource="descriptor/19083-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16815-3">
    <skos:narrower rdf:resource="descriptor/16990-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16815-3">
    <skos:narrower rdf:resource="descriptor/16825-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19468-4">
    <skos:broader rdf:resource="thsys/71064"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19468-4">
    <skos:broader rdf:resource="descriptor/10342-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18852-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17584-5">
    <skos:broader rdf:resource="thsys/70327"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17584-5">
    <skos:broader rdf:resource="descriptor/17589-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17584-5">
    <skos:broader rdf:resource="descriptor/17979-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10471-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11616-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70178">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16245-4">
    <skos:altLabel xml:lang="de">Volksdemokratie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16245-4">
    <skos:altLabel xml:lang="de">Autoritarismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16245-4">
    <skos:altLabel xml:lang="de">Autoritärer Staat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16245-4">
    <skos:altLabel xml:lang="de">Autoritäre Herrschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16245-4">
    <skos:altLabel xml:lang="de">Autokratie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16245-4">
    <skos:altLabel xml:lang="de">Absolutistischer Staat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16245-4">
    <skos:altLabel xml:lang="de">Absolutismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72182">
    <rdfs:label xml:lang="de">Allgemeine statistische Methoden</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10080-5">
    <skos:related rdf:resource="descriptor/10081-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10080-5">
    <skos:related rdf:resource="descriptor/10074-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10080-5">
    <skos:related rdf:resource="descriptor/10073-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10080-5">
    <skos:related rdf:resource="descriptor/10070-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12825-6">
    <skos:broader rdf:resource="thsys/70824"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12825-6">
    <skos:broader rdf:resource="thsys/70384"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12825-6">
    <skos:broader rdf:resource="descriptor/12829-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11398-4">
    <skos:altLabel xml:lang="en">School reform</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11398-4">
    <skos:altLabel xml:lang="de">Schulreform</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17771-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70256">
    <skos:broader rdf:resource="thsys/70257"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18127-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18296-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20826-1">
    <skos:related rdf:resource="descriptor/16436-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20826-1">
    <skos:related rdf:resource="descriptor/16329-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20826-1">
    <skos:related rdf:resource="descriptor/13395-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20826-1">
    <skos:related rdf:resource="descriptor/19708-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20826-1">
    <skos:related rdf:resource="descriptor/19711-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20826-1">
    <skos:related rdf:resource="descriptor/12009-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20826-1">
    <skos:related rdf:resource="descriptor/11161-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20826-1">
    <skos:related rdf:resource="descriptor/20842-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20826-1">
    <skos:related rdf:resource="descriptor/19071-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19434-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12291-3">
    <skos:prefLabel xml:lang="en">Pension accrual</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12291-3">
    <skos:prefLabel xml:lang="de">Pensionsrückstellung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11485-2">
    <skos:broader rdf:resource="thsys/70305"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11485-2">
    <skos:broader rdf:resource="descriptor/15075-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16435-6">
    <skos:altLabel xml:lang="de">Theorie der internationalen Politik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11163-3">
    <skos:prefLabel xml:lang="en">Conflict</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11163-3">
    <skos:prefLabel xml:lang="de">Konflikt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13063-4">
    <skos:prefLabel xml:lang="en">Railway vehicles industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13063-4">
    <skos:prefLabel xml:lang="de">Schienenfahrzeugbau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70845">
    <skos:prefLabel xml:lang="de">W.24.01  Schulen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73351">
    <skos:narrower rdf:resource="descriptor/19630-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73351">
    <skos:narrower rdf:resource="descriptor/14188-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73351">
    <skos:narrower rdf:resource="descriptor/15779-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73351">
    <skos:narrower rdf:resource="descriptor/15897-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73351">
    <skos:narrower rdf:resource="descriptor/18623-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73351">
    <skos:narrower rdf:resource="descriptor/15770-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73351">
    <skos:narrower rdf:resource="descriptor/15765-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73351">
    <skos:narrower rdf:resource="descriptor/15763-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73351">
    <skos:narrower rdf:resource="descriptor/15759-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73351">
    <skos:narrower rdf:resource="descriptor/18340-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73351">
    <skos:narrower rdf:resource="descriptor/18640-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73351">
    <skos:narrower rdf:resource="descriptor/19627-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73351">
    <skos:narrower rdf:resource="descriptor/14169-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73351">
    <skos:narrower rdf:resource="descriptor/19442-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73351">
    <skos:narrower rdf:resource="descriptor/19473-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73351">
    <skos:narrower rdf:resource="descriptor/15755-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73351">
    <skos:narrower rdf:resource="descriptor/15754-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73351">
    <skos:narrower rdf:resource="descriptor/18024-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73351">
    <skos:narrower rdf:resource="descriptor/19629-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73351">
    <skos:narrower rdf:resource="descriptor/14073-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73351">
    <skos:narrower rdf:resource="descriptor/19449-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73351">
    <skos:narrower rdf:resource="descriptor/19628-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73351">
    <skos:narrower rdf:resource="descriptor/15746-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73351">
    <skos:narrower rdf:resource="descriptor/15745-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73351">
    <skos:narrower rdf:resource="descriptor/15889-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73351">
    <skos:narrower rdf:resource="descriptor/15662-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73351">
    <skos:narrower rdf:resource="descriptor/15741-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73351">
    <skos:narrower rdf:resource="descriptor/15743-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73351">
    <skos:narrower rdf:resource="descriptor/15742-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73351">
    <skos:narrower rdf:resource="descriptor/11866-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73351">
    <skos:narrower rdf:resource="descriptor/19632-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73351">
    <skos:narrower rdf:resource="descriptor/15735-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73351">
    <skos:narrower rdf:resource="descriptor/15734-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73351">
    <skos:narrower rdf:resource="descriptor/15733-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73351">
    <skos:narrower rdf:resource="descriptor/19626-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73351">
    <skos:narrower rdf:resource="descriptor/18343-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73351">
    <skos:narrower rdf:resource="descriptor/15751-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73351">
    <skos:narrower rdf:resource="descriptor/18286-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73351">
    <skos:narrower rdf:resource="descriptor/19615-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73351">
    <skos:narrower rdf:resource="descriptor/19485-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73351">
    <skos:narrower rdf:resource="descriptor/12021-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73351">
    <skos:narrower rdf:resource="descriptor/19484-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73351">
    <skos:narrower rdf:resource="descriptor/12016-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73351">
    <skos:narrower rdf:resource="descriptor/12015-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73351">
    <skos:narrower rdf:resource="descriptor/19625-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73351">
    <skos:narrower rdf:resource="descriptor/18493-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73351">
    <skos:narrower rdf:resource="descriptor/18336-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73351">
    <skos:narrower rdf:resource="descriptor/18337-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10773-2">
    <skos:narrower rdf:resource="descriptor/24855-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10773-2">
    <skos:narrower rdf:resource="descriptor/10659-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13236-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10537-0">
    <skos:scopeNote xml:lang="de">Theorie und Empirie der Beschreibung von Entwicklungsstadien.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12089-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13402-1">
    <skos:prefLabel xml:lang="en">Association</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13402-1">
    <skos:prefLabel xml:lang="de">Verein</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17608-3">
    <skos:prefLabel xml:lang="en">Middle East</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17608-3">
    <skos:prefLabel xml:lang="de">Mittlerer Osten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13904-5">
    <skos:related rdf:resource="descriptor/13935-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13904-5">
    <skos:related rdf:resource="descriptor/13142-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13904-5">
    <skos:related rdf:resource="descriptor/13136-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13820-4">
    <skos:related rdf:resource="descriptor/13827-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13820-4">
    <skos:related rdf:resource="descriptor/13825-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13820-4">
    <skos:related rdf:resource="descriptor/10207-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18161-0">
    <skos:broader rdf:resource="thsys/73380"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18161-0">
    <skos:broader rdf:resource="thsys/72183"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18161-0">
    <skos:broader rdf:resource="thsys/70871"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18161-0">
    <skos:broader rdf:resource="descriptor/13413-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19447-5">
    <skos:altLabel xml:lang="de">Regionale Sanierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13192-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16032-0">
    <skos:scopeNote xml:lang="de">Regelt jene Konfliktfälle, in denen das Privatrecht verschiedener Staaten betroffen ist (z.B. binationale Familien, Auslandsarbeitsverträge) und keine zwischenstaatlichen Abkommen existieren. Das IPR bestimmt, welches nationale Recht im Einzelfall anzuwenden ist.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13865-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16343-4">
    <skos:altLabel xml:lang="de">Untersuchungsausschuss</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16343-4">
    <skos:altLabel xml:lang="de">Untersuchungsausschuß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16343-4">
    <skos:altLabel xml:lang="de">Parlamentskommission</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16343-4">
    <skos:altLabel xml:lang="de">Parlamentsausschuß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16343-4">
    <skos:altLabel xml:lang="de">Parlamentarischer Ausschuss</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16343-4">
    <skos:altLabel xml:lang="de">Parlamentarischer Ausschuß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16343-4">
    <skos:altLabel xml:lang="de">Landtagsausschuß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16343-4">
    <skos:altLabel xml:lang="de">Bundestagsausschuss</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16343-4">
    <skos:altLabel xml:lang="de">Bundestagsausschuß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16343-4">
    <skos:altLabel xml:lang="de">Ausschuss des Parlaments</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16343-4">
    <skos:altLabel xml:lang="de">Ausschuß des Parlaments</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16087-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18557-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12585-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19100-1">
    <skos:altLabel xml:lang="de">Vorgangskalkulation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19100-1">
    <skos:altLabel xml:lang="de">Prozeßkostenrechnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19692-6">
    <skos:narrower rdf:resource="descriptor/19857-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13738-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70128">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10851-1">
    <skos:altLabel xml:lang="en">International marketing research</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10851-1">
    <skos:altLabel xml:lang="de">Internationale Marktforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10851-1">
    <skos:altLabel xml:lang="de">Information über Auslandsmärkte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10851-1">
    <skos:altLabel xml:lang="de">Exportmarktforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10851-1">
    <skos:altLabel xml:lang="de">Exportinformation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10851-1">
    <skos:altLabel xml:lang="de">Auslandsmarktinformation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10851-1">
    <skos:altLabel xml:lang="de">Außenwirtschaftsinformation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10851-1">
    <skos:altLabel xml:lang="de">Außenhandelsinformation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12487-0">
    <skos:related rdf:resource="descriptor/12209-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11893-1">
    <skos:prefLabel xml:lang="en">Commuting</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11893-1">
    <skos:prefLabel xml:lang="de">Pendelverkehr</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18064-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15073-5">
    <skos:prefLabel xml:lang="en">Social indicator</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15073-5">
    <skos:prefLabel xml:lang="de">Sozialer Indikator</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70824">
    <skos:narrower rdf:resource="descriptor/12825-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70824">
    <skos:narrower rdf:resource="descriptor/16012-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70824">
    <skos:narrower rdf:resource="descriptor/18027-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70824">
    <skos:narrower rdf:resource="descriptor/16517-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70824">
    <skos:narrower rdf:resource="descriptor/18939-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70824">
    <skos:narrower rdf:resource="descriptor/13360-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70824">
    <skos:narrower rdf:resource="descriptor/13665-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70824">
    <skos:narrower rdf:resource="descriptor/18929-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70824">
    <skos:narrower rdf:resource="descriptor/18005-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70824">
    <skos:narrower rdf:resource="descriptor/18938-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70824">
    <skos:narrower rdf:resource="descriptor/18937-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70824">
    <skos:narrower rdf:resource="descriptor/18936-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70824">
    <skos:narrower rdf:resource="descriptor/18934-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70824">
    <skos:narrower rdf:resource="descriptor/13655-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70824">
    <skos:narrower rdf:resource="descriptor/18935-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14651-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19309-3">
    <skos:broader rdf:resource="thsys/71093"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19309-3">
    <skos:broader rdf:resource="thsys/70179"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19309-3">
    <skos:broader rdf:resource="descriptor/10626-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19528-5">
    <skos:broader rdf:resource="thsys/73347"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19528-5">
    <skos:broader rdf:resource="descriptor/18892-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11047-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13790-1">
    <skos:prefLabel xml:lang="en">Saving deposit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13790-1">
    <skos:prefLabel xml:lang="de">Spargeschäft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19308-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19442-1">
    <skos:narrower rdf:resource="descriptor/15033-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12515-0">
    <skos:broader rdf:resource="thsys/70578"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12515-0">
    <skos:broader rdf:resource="descriptor/12524-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14895-3">
    <skos:related rdf:resource="descriptor/14808-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14895-3">
    <skos:related rdf:resource="descriptor/14839-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14895-3">
    <skos:related rdf:resource="descriptor/13131-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13431-1">
    <skos:related rdf:resource="descriptor/18995-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13431-1">
    <skos:related rdf:resource="descriptor/13188-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13431-1">
    <skos:related rdf:resource="descriptor/19292-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13431-1">
    <skos:related rdf:resource="descriptor/19382-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13431-1">
    <skos:related rdf:resource="descriptor/14054-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13431-1">
    <skos:related rdf:resource="descriptor/19674-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13814-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10006-3">
    <skos:broader rdf:resource="thsys/71115"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10006-3">
    <skos:broader rdf:resource="thsys/70245"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19848-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12624-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19527-0">
    <skos:broader rdf:resource="thsys/73347"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19527-0">
    <skos:broader rdf:resource="descriptor/18891-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11715-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16606-5">
    <skos:broader rdf:resource="thsys/73396"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16606-5">
    <skos:broader rdf:resource="descriptor/16602-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16606-5">
    <skos:broader rdf:resource="descriptor/16588-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18648-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12933-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19002-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12774-4">
    <skos:related rdf:resource="descriptor/12837-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12774-4">
    <skos:related rdf:resource="descriptor/13383-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12774-4">
    <skos:related rdf:resource="descriptor/18930-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12774-4">
    <skos:related rdf:resource="descriptor/10012-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12774-4">
    <skos:related rdf:resource="descriptor/12766-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12774-4">
    <skos:related rdf:resource="descriptor/10300-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12774-4">
    <skos:related rdf:resource="descriptor/19542-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12774-4">
    <skos:related rdf:resource="descriptor/19745-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19217-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70870">
    <skos:narrower rdf:resource="descriptor/13424-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70870">
    <skos:narrower rdf:resource="descriptor/13421-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70870">
    <skos:narrower rdf:resource="descriptor/11970-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70870">
    <skos:narrower rdf:resource="descriptor/13420-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70870">
    <skos:narrower rdf:resource="descriptor/18808-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70870">
    <skos:narrower rdf:resource="descriptor/13414-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70870">
    <skos:narrower rdf:resource="descriptor/16550-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70870">
    <skos:narrower rdf:resource="descriptor/11852-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70870">
    <skos:narrower rdf:resource="descriptor/16322-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70870">
    <skos:narrower rdf:resource="descriptor/13409-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70870">
    <skos:narrower rdf:resource="descriptor/16314-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70870">
    <skos:narrower rdf:resource="descriptor/11862-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14422-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20404-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70879">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.20</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19242-2">
    <skos:broader rdf:resource="thsys/70406"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19242-2">
    <skos:broader rdf:resource="thsys/70091"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18762-2">
    <skos:scopeNote xml:lang="de">Sammelbegriff für Unternehmen der Elektronik- und Softwareindustrie sowie der verschiedenen Dienstleistungen auf dem Gebiet der Datenverarbeitung</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19475-0">
    <skos:prefLabel xml:lang="en">Information industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19475-0">
    <skos:prefLabel xml:lang="de">Informationswirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12181-3">
    <skos:broader rdf:resource="thsys/70087"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12181-3">
    <skos:broader rdf:resource="descriptor/12192-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19538-2">
    <skos:scopeNote xml:lang="de">Spanne zwischen Geld- und Briefkurs.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16990-6">
    <skos:broader rdf:resource="thsys/70505"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16990-6">
    <skos:broader rdf:resource="descriptor/16815-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19850-0">
    <skos:broader rdf:resource="thsys/73391"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19850-0">
    <skos:broader rdf:resource="descriptor/13464-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15225-1">
    <skos:altLabel xml:lang="en">Wiener process</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15225-1">
    <skos:altLabel xml:lang="en">Stationary stochastic process</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15225-1">
    <skos:altLabel xml:lang="en">Moving average</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15225-1">
    <skos:altLabel xml:lang="en">Gaussian process</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15225-1">
    <skos:altLabel xml:lang="en">Brownian motion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15225-1">
    <skos:altLabel xml:lang="en">Brownian bridge</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15225-1">
    <skos:altLabel xml:lang="de">Wiener-Prozess</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15225-1">
    <skos:altLabel xml:lang="de">Wiener-Prozeß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15225-1">
    <skos:altLabel xml:lang="de">Stochastischer Prozeß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15225-1">
    <skos:altLabel xml:lang="de">Stationärer Prozess</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15225-1">
    <skos:altLabel xml:lang="de">Stationärer Prozeß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15225-1">
    <skos:altLabel xml:lang="de">Poisson-Prozess</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15225-1">
    <skos:altLabel xml:lang="de">Poisson-Prozeß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15225-1">
    <skos:altLabel xml:lang="de">Gleitender Durchschnitt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15225-1">
    <skos:altLabel xml:lang="de">Gaußscher Prozess</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15225-1">
    <skos:altLabel xml:lang="de">Gaußscher Prozeß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15225-1">
    <skos:altLabel xml:lang="de">Brownsche Bewegung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15225-1">
    <skos:altLabel xml:lang="de">Box-Jenkins-Methode</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15225-1">
    <skos:altLabel xml:lang="de">Bernoulli-Prozess</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15225-1">
    <skos:altLabel xml:lang="de">Bernoulli-Prozeß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10015-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16509-3">
    <skos:prefLabel xml:lang="en">Domestic policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16509-3">
    <skos:prefLabel xml:lang="de">Innenpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18951-6">
    <skos:related rdf:resource="descriptor/16767-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18951-6">
    <skos:related rdf:resource="descriptor/16769-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18951-6">
    <skos:related rdf:resource="descriptor/18950-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12019-5">
    <skos:related rdf:resource="descriptor/12023-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12019-5">
    <skos:related rdf:resource="descriptor/11921-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12019-5">
    <skos:related rdf:resource="descriptor/19454-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18949-0">
    <skos:prefLabel xml:lang="en">Lending</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18949-0">
    <skos:prefLabel xml:lang="de">Ausleihe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id125">
    <rdfs:seeAlso rdf:resource="descriptor/15964-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18008-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15656-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13652-6">
    <skos:related rdf:resource="descriptor/13273-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13652-6">
    <skos:related rdf:resource="descriptor/21315-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13652-6">
    <skos:related rdf:resource="descriptor/18659-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19524-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16549-5">
    <skos:broader rdf:resource="thsys/73367"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16549-5">
    <skos:broader rdf:resource="thsys/71064"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16549-5">
    <skos:broader rdf:resource="descriptor/16509-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14252-3">
    <skos:prefLabel xml:lang="en">Brick</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14252-3">
    <skos:prefLabel xml:lang="de">Grobkeramik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11313-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10980-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18065-3">
    <skos:prefLabel xml:lang="en">Licensing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18065-3">
    <skos:prefLabel xml:lang="de">Lizenzvergabe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17636-5">
    <skos:prefLabel xml:lang="en">Syria</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17636-5">
    <skos:prefLabel xml:lang="de">Syrien</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19798-4">
    <skos:related rdf:resource="descriptor/19797-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19798-4">
    <skos:related rdf:resource="descriptor/10455-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19763-2">
    <skos:altLabel xml:lang="en">Criminals</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19763-2">
    <skos:altLabel xml:lang="de">Verbrecherinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19763-2">
    <skos:altLabel xml:lang="de">Verbrecher</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19763-2">
    <skos:altLabel xml:lang="de">Täterinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19763-2">
    <skos:altLabel xml:lang="de">Täter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19763-2">
    <skos:altLabel xml:lang="de">Straftäterinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19763-2">
    <skos:altLabel xml:lang="de">Strafgefangene</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19763-2">
    <skos:altLabel xml:lang="de">Straffällige</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19763-2">
    <skos:altLabel xml:lang="de">Kriminelle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12905-1">
    <skos:narrower rdf:resource="descriptor/12932-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12905-1">
    <skos:narrower rdf:resource="descriptor/13177-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12905-1">
    <skos:narrower rdf:resource="descriptor/19056-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12905-1">
    <skos:narrower rdf:resource="descriptor/13176-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12905-1">
    <skos:narrower rdf:resource="descriptor/11578-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12905-1">
    <skos:narrower rdf:resource="descriptor/13109-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12905-1">
    <skos:narrower rdf:resource="descriptor/13173-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12905-1">
    <skos:narrower rdf:resource="descriptor/12915-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12905-1">
    <skos:narrower rdf:resource="descriptor/13844-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12905-1">
    <skos:narrower rdf:resource="descriptor/19054-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12905-1">
    <skos:narrower rdf:resource="descriptor/11901-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12905-1">
    <skos:narrower rdf:resource="descriptor/18787-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12905-1">
    <skos:narrower rdf:resource="descriptor/18497-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12905-1">
    <skos:narrower rdf:resource="descriptor/13180-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12905-1">
    <skos:narrower rdf:resource="descriptor/13846-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14778-0">
    <skos:altLabel xml:lang="de">Verpackungsglas</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14778-0">
    <skos:altLabel xml:lang="de">Konservenglas</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14778-0">
    <skos:altLabel xml:lang="de">Glasflasche</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14778-0">
    <skos:altLabel xml:lang="de">Glasbehälter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13684-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17569-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70933">
    <skos:prefLabel xml:lang="en">W.12.01.03.03  Port management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70933">
    <skos:prefLabel xml:lang="de">W.12.01.03.03  Hafenwirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20817-2">
    <skos:prefLabel xml:lang="en">Fair trade</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20817-2">
    <skos:prefLabel xml:lang="de">Fairer Handel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11651-4">
    <skos:prefLabel xml:lang="en">Inheritance tax</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11651-4">
    <skos:prefLabel xml:lang="de">Erbschaftsteuer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14218-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16841-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13074-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70926">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.12.05.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11968-3">
    <skos:prefLabel xml:lang="en">Medium-size city</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11968-3">
    <skos:prefLabel xml:lang="de">Mittelstadt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13412-5">
    <skos:related rdf:resource="descriptor/18366-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13412-5">
    <skos:related rdf:resource="descriptor/16024-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13412-5">
    <skos:related rdf:resource="descriptor/16522-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13412-5">
    <skos:related rdf:resource="descriptor/13998-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13412-5">
    <skos:related rdf:resource="descriptor/16460-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13412-5">
    <skos:related rdf:resource="descriptor/15005-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13412-5">
    <skos:related rdf:resource="descriptor/16521-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13412-5">
    <skos:related rdf:resource="descriptor/19725-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13412-5">
    <skos:related rdf:resource="descriptor/16684-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13412-5">
    <skos:related rdf:resource="descriptor/19720-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13412-5">
    <skos:related rdf:resource="descriptor/16470-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13412-5">
    <skos:related rdf:resource="descriptor/16437-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10522-6">
    <skos:related rdf:resource="descriptor/10511-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10522-6">
    <skos:related rdf:resource="descriptor/10018-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10522-6">
    <skos:related rdf:resource="descriptor/10501-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10522-6">
    <skos:related rdf:resource="descriptor/10525-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10522-6">
    <skos:related rdf:resource="descriptor/11230-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11375-2">
    <skos:altLabel xml:lang="de">Schulkinder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11375-2">
    <skos:altLabel xml:lang="de">Schülerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19554-4">
    <skos:prefLabel xml:lang="en">Noise trading</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19554-4">
    <skos:prefLabel xml:lang="de">Noise Trading</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10006-3">
    <skos:narrower rdf:resource="descriptor/11322-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10006-3">
    <skos:narrower rdf:resource="descriptor/11740-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10006-3">
    <skos:narrower rdf:resource="descriptor/12285-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10006-3">
    <skos:narrower rdf:resource="descriptor/19966-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10006-3">
    <skos:narrower rdf:resource="descriptor/18948-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10006-3">
    <skos:narrower rdf:resource="descriptor/19693-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10006-3">
    <skos:narrower rdf:resource="descriptor/19676-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19813-6">
    <skos:prefLabel xml:lang="en">Penalty</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19813-6">
    <skos:prefLabel xml:lang="de">Strafe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24781-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12844-2">
    <skos:broader rdf:resource="thsys/70874"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12844-2">
    <skos:broader rdf:resource="thsys/70384"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12844-2">
    <skos:broader rdf:resource="descriptor/12770-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12165-1">
    <skos:prefLabel xml:lang="en">Business intelligence system</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12165-1">
    <skos:prefLabel xml:lang="de">Betriebliches Informationssystem</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12473-4">
    <skos:related rdf:resource="descriptor/19100-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12473-4">
    <skos:related rdf:resource="descriptor/12883-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12473-4">
    <skos:related rdf:resource="descriptor/12330-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13480-2">
    <skos:broader rdf:resource="thsys/70947"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13480-2">
    <skos:broader rdf:resource="descriptor/13473-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13480-2">
    <skos:broader rdf:resource="descriptor/13200-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16633-2">
    <skos:related rdf:resource="descriptor/16353-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16633-2">
    <skos:related rdf:resource="descriptor/19733-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16633-2">
    <skos:related rdf:resource="descriptor/16544-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16633-2">
    <skos:related rdf:resource="descriptor/16642-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16633-2">
    <skos:related rdf:resource="descriptor/16287-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16633-2">
    <skos:related rdf:resource="descriptor/16207-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16633-2">
    <skos:related rdf:resource="descriptor/16430-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16633-2">
    <skos:related rdf:resource="descriptor/16275-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16633-2">
    <skos:related rdf:resource="descriptor/15821-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16633-2">
    <skos:related rdf:resource="descriptor/16405-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11931-5">
    <skos:broader rdf:resource="thsys/71109"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11931-5">
    <skos:broader rdf:resource="thsys/71070"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11931-5">
    <skos:broader rdf:resource="descriptor/10092-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11375-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16012-6">
    <skos:scopeNote xml:lang="de">Rechtliche Regelungen zum Betreiben und inhaltlichen Gestalten von Rundfunk- und Fernsehprogrammen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18658-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19722-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18277-2">
    <skos:related rdf:resource="descriptor/13829-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18277-2">
    <skos:related rdf:resource="descriptor/13903-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18277-2">
    <skos:related rdf:resource="descriptor/14040-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18277-2">
    <skos:related rdf:resource="descriptor/14032-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14296-4">
    <skos:related rdf:resource="descriptor/14274-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14296-4">
    <skos:related rdf:resource="descriptor/13095-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14296-4">
    <skos:related rdf:resource="descriptor/13231-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14296-4">
    <skos:related rdf:resource="descriptor/14593-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14296-4">
    <skos:related rdf:resource="descriptor/13026-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16533-6">
    <skos:related rdf:resource="descriptor/20553-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16533-6">
    <skos:related rdf:resource="descriptor/10560-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16533-6">
    <skos:related rdf:resource="descriptor/10488-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10568-3">
    <skos:prefLabel xml:lang="en">Donor countries</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10568-3">
    <skos:prefLabel xml:lang="de">Geberländer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18546-1">
    <skos:altLabel xml:lang="de">Standort des Einzelhandels</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13639-5">
    <skos:broader rdf:resource="thsys/73367"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13639-5">
    <skos:broader rdf:resource="thsys/71110"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13639-5">
    <skos:broader rdf:resource="thsys/70924"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13639-5">
    <skos:broader rdf:resource="descriptor/10981-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18958-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70463">
    <skos:prefLabel xml:lang="de">P.12.01  Anorganische Chemikalien</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13238-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10567-5">
    <skos:related rdf:resource="descriptor/19291-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10567-5">
    <skos:related rdf:resource="descriptor/16552-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10567-5">
    <skos:related rdf:resource="descriptor/19308-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10567-5">
    <skos:related rdf:resource="descriptor/10824-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10567-5">
    <skos:related rdf:resource="descriptor/10711-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10567-5">
    <skos:related rdf:resource="descriptor/10823-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10567-5">
    <skos:related rdf:resource="descriptor/18011-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10343-6">
    <skos:prefLabel xml:lang="en">Economic crisis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10343-6">
    <skos:prefLabel xml:lang="de">Wirtschaftskrise</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17658-2">
    <skos:altLabel xml:lang="de">Republik Dschibuti</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17658-2">
    <skos:altLabel xml:lang="de">Französisches Afar- und Issa-Territorium</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17658-2">
    <skos:altLabel xml:lang="de">Französische Somaliküste</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19188-3">
    <skos:related rdf:resource="descriptor/19293-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19188-3">
    <skos:related rdf:resource="descriptor/19187-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19188-3">
    <skos:related rdf:resource="descriptor/18517-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19188-3">
    <skos:related rdf:resource="descriptor/18198-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19188-3">
    <skos:related rdf:resource="descriptor/14088-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11686-6">
    <skos:prefLabel xml:lang="en">Ability to pay taxes</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11686-6">
    <skos:prefLabel xml:lang="de">Leistungsfähigkeitsprinzip</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13419-5">
    <skos:related rdf:resource="descriptor/16159-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13419-5">
    <skos:related rdf:resource="descriptor/16545-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13419-5">
    <skos:related rdf:resource="descriptor/16543-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19674-1">
    <skos:related rdf:resource="descriptor/18382-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19674-1">
    <skos:related rdf:resource="descriptor/18991-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19674-1">
    <skos:related rdf:resource="descriptor/11802-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19674-1">
    <skos:related rdf:resource="descriptor/11800-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19674-1">
    <skos:related rdf:resource="descriptor/13431-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19737-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12901-2">
    <skos:broader rdf:resource="thsys/70875"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12901-2">
    <skos:broader rdf:resource="descriptor/13391-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19639-3">
    <skos:altLabel xml:lang="de">Immigranten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19639-3">
    <skos:altLabel xml:lang="de">Emigranten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19639-3">
    <skos:altLabel xml:lang="de">Einwanderer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19639-3">
    <skos:altLabel xml:lang="de">Auswanderer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71117">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.02.06</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12969-3">
    <skos:altLabel xml:lang="de">Teichwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12969-3">
    <skos:altLabel xml:lang="de">Fischzucht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12969-3">
    <skos:altLabel xml:lang="de">Fischhege</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12969-3">
    <skos:altLabel xml:lang="de">Fischbrutanlage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12969-3">
    <skos:altLabel xml:lang="de">Fischbesatz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18608-5">
    <skos:altLabel xml:lang="de">Zubringerluftverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18608-5">
    <skos:altLabel xml:lang="de">Regionalflugverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18608-5">
    <skos:altLabel xml:lang="de">Regionaler Luftverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18608-5">
    <skos:altLabel xml:lang="de">Kurzstreckenluftverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18608-5">
    <skos:altLabel xml:lang="de">Ergänzungsluftverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18608-5">
    <skos:altLabel xml:lang="de">Commuter-Luftverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15025-2">
    <skos:prefLabel xml:lang="en">Scenario analysis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15025-2">
    <skos:prefLabel xml:lang="de">Szenario-Technik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13127-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10170-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71031">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">G.03.01.04</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14953-1">
    <skos:altLabel xml:lang="en">Frozen yogurt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14953-1">
    <skos:altLabel xml:lang="de">Joghurtprodukt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14953-1">
    <skos:altLabel xml:lang="de">Joghurteis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13767-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16542-5">
    <skos:narrower rdf:resource="descriptor/13418-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16542-5">
    <skos:narrower rdf:resource="descriptor/18910-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16542-5">
    <skos:narrower rdf:resource="descriptor/16527-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19666-0">
    <skos:scopeNote xml:lang="de">Gesamtwirtschaftliche Effekte von Ereignissen oder Maßnahmen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19275-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14334-1">
    <skos:related rdf:resource="descriptor/14345-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70355">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10605-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71018">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10247-2">
    <skos:related rdf:resource="descriptor/10087-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10247-2">
    <skos:related rdf:resource="descriptor/10054-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10247-2">
    <skos:related rdf:resource="descriptor/10249-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10247-2">
    <skos:related rdf:resource="descriptor/10381-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18155-2">
    <skos:broader rdf:resource="thsys/70274"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18155-2">
    <skos:broader rdf:resource="descriptor/10975-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70463">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70458">
    <skos:narrower rdf:resource="descriptor/13411-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70458">
    <skos:narrower rdf:resource="descriptor/19515-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70458">
    <skos:narrower rdf:resource="descriptor/18913-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70458">
    <skos:narrower rdf:resource="descriptor/16547-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70458">
    <skos:narrower rdf:resource="descriptor/13418-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70458">
    <skos:narrower rdf:resource="descriptor/19512-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70458">
    <skos:narrower rdf:resource="descriptor/16539-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70458">
    <skos:narrower rdf:resource="descriptor/18739-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70458">
    <skos:narrower rdf:resource="descriptor/18180-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70458">
    <skos:narrower rdf:resource="descriptor/18105-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70458">
    <skos:narrower rdf:resource="descriptor/16548-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70458">
    <skos:narrower rdf:resource="descriptor/19514-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70458">
    <skos:narrower rdf:resource="descriptor/19551-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70458">
    <skos:narrower rdf:resource="descriptor/13292-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70458">
    <skos:narrower rdf:resource="descriptor/19513-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70458">
    <skos:narrower rdf:resource="descriptor/13416-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70458">
    <skos:narrower rdf:resource="descriptor/13415-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70458">
    <skos:narrower rdf:resource="descriptor/16532-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70458">
    <skos:narrower rdf:resource="descriptor/16531-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70458">
    <skos:narrower rdf:resource="descriptor/12591-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70458">
    <skos:narrower rdf:resource="descriptor/13406-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70458">
    <skos:narrower rdf:resource="descriptor/16527-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70458">
    <skos:narrower rdf:resource="descriptor/16526-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12625-0">
    <skos:altLabel xml:lang="en">Job rotation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12625-0">
    <skos:altLabel xml:lang="en">Job enrichment</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12625-0">
    <skos:altLabel xml:lang="de">Aufgabenvergrößerung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12625-0">
    <skos:altLabel xml:lang="de">Aufgabentausch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12625-0">
    <skos:altLabel xml:lang="de">Aufgabenbereicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12625-0">
    <skos:altLabel xml:lang="de">Arbeitsbereicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19282-4">
    <skos:narrower rdf:resource="descriptor/13785-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19282-4">
    <skos:narrower rdf:resource="descriptor/13784-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73306">
    <skos:prefLabel xml:lang="de">P.06.02  Glas und Glaswaren</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15858-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15478-0">
    <skos:broader rdf:resource="thsys/73103"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15478-0">
    <skos:broader rdf:resource="descriptor/15480-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16029-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10836-4">
    <skos:prefLabel xml:lang="en">Export</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10836-4">
    <skos:prefLabel xml:lang="de">Export</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70458">
    <rdfs:label xml:lang="en">Social insurance</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70458">
    <rdfs:label xml:lang="de">Sozialversicherung</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10824-4">
    <skos:prefLabel xml:lang="en">Investment guarantee</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10824-4">
    <skos:prefLabel xml:lang="de">Investitionsgarantie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12385-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18498-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10901-5">
    <skos:altLabel xml:lang="de">Konzentrationsgrad</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10901-5">
    <skos:altLabel xml:lang="de">Herfindahl-Index</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10901-5">
    <skos:altLabel xml:lang="de">Hall-Tidemann-Index</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10901-5">
    <skos:altLabel xml:lang="de">Disparitätsindex der Wirtschaftskonzentration</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16515-1">
    <skos:altLabel xml:lang="de">Kulturförderung des Staates</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16515-1">
    <skos:altLabel xml:lang="de">Denkmalschutz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16515-1">
    <skos:altLabel xml:lang="de">Denkmalpflege</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14955-4">
    <skos:broader rdf:resource="thsys/73303"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14955-4">
    <skos:broader rdf:resource="descriptor/14129-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14955-4">
    <skos:broader rdf:resource="descriptor/14931-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19716-4">
    <skos:broader rdf:resource="thsys/73366"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10049-6">
    <skos:broader rdf:resource="thsys/70575"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10049-6">
    <skos:broader rdf:resource="thsys/70049"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10049-6">
    <skos:broader rdf:resource="descriptor/10065-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70581">
    <rdfs:label xml:lang="de">Kultur und Kunst</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18887-3">
    <skos:prefLabel xml:lang="en">Social statistics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18887-3">
    <skos:prefLabel xml:lang="de">Sozialstatistik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16489-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10893-6">
    <skos:related rdf:resource="descriptor/12272-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18881-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14889-5">
    <skos:altLabel xml:lang="de">Technische Faser</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14889-5">
    <skos:altLabel xml:lang="de">Reifencord</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14889-5">
    <skos:altLabel xml:lang="de">Mineralfaser</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14889-5">
    <skos:altLabel xml:lang="de">Metallfaser</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14889-5">
    <skos:altLabel xml:lang="de">Kohlenstofffaser</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14889-5">
    <skos:altLabel xml:lang="de">Kohlenstoffaser</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14889-5">
    <skos:altLabel xml:lang="de">Kohlefaser</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14889-5">
    <skos:altLabel xml:lang="de">Industrietextilien</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14889-5">
    <skos:altLabel xml:lang="de">Industriefilamentgarn</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14889-5">
    <skos:altLabel xml:lang="de">Graphitfaser</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14889-5">
    <skos:altLabel xml:lang="de">Grafitfaser</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14889-5">
    <skos:altLabel xml:lang="de">Faserverbundstoff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14889-5">
    <skos:altLabel xml:lang="de">Carbonfaser</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14889-5">
    <skos:altLabel xml:lang="de">Aramidfaser</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14889-5">
    <skos:altLabel xml:lang="de">Anorganische Faser</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11365-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16633-2">
    <skos:broader rdf:resource="thsys/73376"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16633-2">
    <skos:broader rdf:resource="thsys/70120"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15704-3">
    <skos:prefLabel xml:lang="en">Mercantilism</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15704-3">
    <skos:prefLabel xml:lang="de">Merkantilismus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18681-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16874-1">
    <skos:prefLabel xml:lang="en">Hesse</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16874-1">
    <skos:prefLabel xml:lang="de">Hessen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70242">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.10.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19040-0">
    <skos:editorialNote xml:lang="de">Möglichst spezifischer indexieren.</skos:editorialNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11938-5">
    <skos:prefLabel xml:lang="en">Gemeinschaftsaufgabe Agrarstruktur</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11938-5">
    <skos:prefLabel xml:lang="de">Gemeinschaftsaufgabe Agrarstruktur</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id358">
    <rdf:value xml:lang="de">Für Währungsdualismus aus schwacher Inlandswährung und starker ausländischer "Zweitwährung"</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71025">
    <rdfs:label xml:lang="en">Animal husbandry</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71025">
    <rdfs:label xml:lang="de">Tierhaltung</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19582-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13615-5">
    <skos:prefLabel xml:lang="en">Container transport</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13615-5">
    <skos:prefLabel xml:lang="de">Containerverkehr</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70307">
    <skos:prefLabel xml:lang="de">G.06  Politische und ökonomische Staatengruppen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70091">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19539-0">
    <skos:altLabel xml:lang="de">Blutkonserve</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15396-2">
    <skos:prefLabel xml:lang="en">Time series analysis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15396-2">
    <skos:prefLabel xml:lang="de">Zeitreihenanalyse</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14171-3">
    <skos:broader rdf:resource="thsys/70651"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14171-3">
    <skos:broader rdf:resource="descriptor/18024-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13337-0">
    <skos:prefLabel xml:lang="en">Waste incineration</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13337-0">
    <skos:prefLabel xml:lang="de">Müllverbrennung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19578-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10291-6">
    <skos:broader rdf:resource="thsys/70530"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10291-6">
    <skos:broader rdf:resource="descriptor/10294-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17771-6">
    <skos:prefLabel xml:lang="en">Bahamas</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17771-6">
    <skos:prefLabel xml:lang="de">Bahamas</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14298-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18727-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11541-4">
    <skos:broader rdf:resource="thsys/73380"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11541-4">
    <skos:broader rdf:resource="thsys/70869"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11541-4">
    <skos:broader rdf:resource="thsys/70144"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11541-4">
    <skos:broader rdf:resource="descriptor/13413-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17709-4">
    <skos:broader rdf:resource="thsys/70560"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17709-4">
    <skos:broader rdf:resource="descriptor/17724-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10776-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11293-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18338-1">
    <skos:altLabel xml:lang="en">Forestry policy</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18338-1">
    <skos:altLabel xml:lang="de">Jagdpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18338-1">
    <skos:altLabel xml:lang="de">Forstwirtschaftspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10842-2">
    <skos:altLabel xml:lang="en">Competitiveness</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10842-2">
    <skos:altLabel xml:lang="de">Wettbewerbsfähigkeit am Weltmarkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10842-2">
    <skos:altLabel xml:lang="de">Internationale Wettbewerbsfähigkeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10842-2">
    <skos:altLabel xml:lang="de">Auslandskonkurrenz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10842-2">
    <skos:altLabel xml:lang="de">Außenwirtschaftswettbewerb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14381-6">
    <skos:broader rdf:resource="thsys/70318"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14381-6">
    <skos:broader rdf:resource="descriptor/14336-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14444-1">
    <skos:broader rdf:resource="thsys/70258"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14444-1">
    <skos:broader rdf:resource="descriptor/14441-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70556">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">P.12.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19632-3">
    <skos:related rdf:resource="descriptor/13179-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19632-3">
    <skos:related rdf:resource="descriptor/19624-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19632-3">
    <skos:related rdf:resource="descriptor/15883-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19632-3">
    <skos:related rdf:resource="descriptor/11938-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19000-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11857-5">
    <skos:editorialNote xml:lang="de">Möglichst spezifischer indexieren.</skos:editorialNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19614-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15678-6">
    <skos:related rdf:resource="descriptor/19731-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15678-6">
    <skos:related rdf:resource="descriptor/15671-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15678-6">
    <skos:related rdf:resource="descriptor/16719-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15678-6">
    <skos:related rdf:resource="descriptor/13393-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15678-6">
    <skos:related rdf:resource="descriptor/14012-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15678-6">
    <skos:related rdf:resource="descriptor/15640-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18126-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14524-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70650">
    <skos:prefLabel xml:lang="en">P.04.02  Nuclear energy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70650">
    <skos:prefLabel xml:lang="de">P.04.02  Kernenergie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13294-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id272">
    <rdfs:seeAlso rdf:resource="descriptor/10082-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12983-2">
    <skos:prefLabel xml:lang="en">Natural gas production</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12983-2">
    <skos:prefLabel xml:lang="de">Erdgasförderung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12207-4">
    <skos:related rdf:resource="descriptor/12208-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11979-5">
    <skos:broader rdf:resource="thsys/71064"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11979-5">
    <skos:broader rdf:resource="descriptor/11857-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70039">
    <skos:prefLabel xml:lang="de">V.06  Wirtschaftsordnung und Wirtschaftspolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18712-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11990-3">
    <skos:narrower rdf:resource="descriptor/19363-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11990-3">
    <skos:narrower rdf:resource="descriptor/19472-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11990-3">
    <skos:narrower rdf:resource="descriptor/19452-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11990-3">
    <skos:narrower rdf:resource="descriptor/19471-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11990-3">
    <skos:narrower rdf:resource="descriptor/11988-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11457-0">
    <skos:prefLabel xml:lang="en">Liquidity preference</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11457-0">
    <skos:prefLabel xml:lang="de">Liquiditätspräferenz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15475-6">
    <skos:broader rdf:resource="thsys/73103"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15475-6">
    <skos:broader rdf:resource="descriptor/15470-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15475-6">
    <skos:broader rdf:resource="descriptor/15466-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18183-4">
    <skos:prefLabel xml:lang="en">Green national accounting</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18183-4">
    <skos:prefLabel xml:lang="de">Ökosozialprodukt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id30">
    <rdf:value xml:lang="de">Für auktionstheoretische Ansätze</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19709-1">
    <skos:altLabel xml:lang="de">Weltreich</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19709-1">
    <skos:altLabel xml:lang="de">Weltmacht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19709-1">
    <skos:altLabel xml:lang="de">Supermacht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19709-1">
    <skos:altLabel xml:lang="de">Kolonialreich</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19709-1">
    <skos:altLabel xml:lang="de">Imperium</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19709-1">
    <skos:altLabel xml:lang="de">Hegemonialmacht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12078-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18320-6">
    <skos:related rdf:resource="descriptor/13160-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18320-6">
    <skos:related rdf:resource="descriptor/12945-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18320-6">
    <skos:related rdf:resource="descriptor/13054-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18320-6">
    <skos:related rdf:resource="descriptor/13188-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18320-6">
    <skos:related rdf:resource="descriptor/14596-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18320-6">
    <skos:related rdf:resource="descriptor/14054-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18320-6">
    <skos:related rdf:resource="descriptor/13429-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18672-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15723-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70429">
    <skos:narrower rdf:resource="descriptor/17689-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70429">
    <skos:narrower rdf:resource="descriptor/17687-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70429">
    <skos:narrower rdf:resource="descriptor/17688-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70429">
    <skos:narrower rdf:resource="descriptor/17686-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70429">
    <skos:narrower rdf:resource="descriptor/17684-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70429">
    <skos:narrower rdf:resource="descriptor/17682-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70429">
    <skos:narrower rdf:resource="descriptor/17680-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16789-6">
    <skos:broader rdf:resource="thsys/70822"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16789-6">
    <skos:broader rdf:resource="descriptor/16780-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14173-6">
    <skos:narrower rdf:resource="descriptor/14216-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14173-6">
    <skos:narrower rdf:resource="descriptor/14181-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14173-6">
    <skos:narrower rdf:resource="descriptor/14215-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19099-2">
    <skos:prefLabel xml:lang="en">Cost recording</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19099-2">
    <skos:prefLabel xml:lang="de">Kostenerfassung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70871">
    <skos:broader rdf:resource="thsys/70872"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10478-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19397-1">
    <skos:related rdf:resource="descriptor/12997-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19397-1">
    <skos:related rdf:resource="descriptor/18516-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19397-1">
    <skos:related rdf:resource="descriptor/13427-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11518-6">
    <skos:broader rdf:resource="thsys/70355"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18087-0">
    <skos:prefLabel xml:lang="en">Interest rate risk</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18087-0">
    <skos:prefLabel xml:lang="de">Zinsrisiko</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11657-6">
    <skos:narrower rdf:resource="descriptor/11670-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13299-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18386-4">
    <skos:related rdf:resource="descriptor/12980-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13394-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19838-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12677-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18502-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://purl.org/dc/terms/issued">
    <rdf:type rdf:resource="http://www.w3.org/2002/07/owl#DatatypeProperty"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12182-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18212-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16333-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11819-6">
    <skos:prefLabel xml:lang="en">Promotion of handicrafts</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11819-6">
    <skos:prefLabel xml:lang="de">Handwerkspolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18414-4">
    <skos:altLabel xml:lang="de">MHI-Industrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16374-0">
    <skos:broader rdf:resource="thsys/73378"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16374-0">
    <skos:broader rdf:resource="descriptor/13397-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16664-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18976-4">
    <skos:related rdf:resource="descriptor/13401-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18976-4">
    <skos:related rdf:resource="descriptor/10298-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12215-5">
    <skos:broader rdf:resource="thsys/70037"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18516-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16959-0">
    <skos:altLabel xml:lang="de">Bern (Region)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14029-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10801-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11042-1">
    <skos:altLabel xml:lang="de">Geldökonomie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13305-6">
    <skos:broader rdf:resource="thsys/70888"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13305-6">
    <skos:broader rdf:resource="descriptor/18747-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10211-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11464-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10798-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14141-5">
    <skos:prefLabel xml:lang="en">Reindeer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14141-5">
    <skos:prefLabel xml:lang="de">Rentier</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12024-5">
    <skos:related rdf:resource="descriptor/18985-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12024-5">
    <skos:related rdf:resource="descriptor/13427-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12024-5">
    <skos:related rdf:resource="descriptor/12021-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12024-5">
    <skos:related rdf:resource="descriptor/18326-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12024-5">
    <skos:related rdf:resource="descriptor/10490-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12024-5">
    <skos:related rdf:resource="descriptor/19339-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18704-2">
    <skos:related rdf:resource="descriptor/16046-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18704-2">
    <skos:related rdf:resource="descriptor/10774-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18704-2">
    <skos:related rdf:resource="descriptor/11481-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24847-1">
    <skos:related rdf:resource="descriptor/12108-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24847-1">
    <skos:related rdf:resource="descriptor/12105-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24847-1">
    <skos:related rdf:resource="descriptor/12110-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24847-1">
    <skos:related rdf:resource="descriptor/25881-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24847-1">
    <skos:related rdf:resource="descriptor/12617-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14211-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13962-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11832-0">
    <skos:altLabel xml:lang="de">Verkehrskonzept</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11832-0">
    <skos:altLabel xml:lang="de">Verkehrsentwicklungsplanung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11832-0">
    <skos:altLabel xml:lang="de">Gesamtverkehrsplan</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11832-0">
    <skos:altLabel xml:lang="de">Generalverkehrsplan</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18870-6">
    <skos:broader rdf:resource="thsys/73360"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18870-6">
    <skos:broader rdf:resource="thsys/70845"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18870-6">
    <skos:broader rdf:resource="thsys/70176"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18870-6">
    <skos:broader rdf:resource="descriptor/16019-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14712-2">
    <skos:related rdf:resource="descriptor/14692-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18945-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19192-5">
    <skos:altLabel xml:lang="de">Kohlemarkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18382-5">
    <skos:broader rdf:resource="thsys/71010"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18382-5">
    <skos:broader rdf:resource="thsys/70811"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18382-5">
    <skos:broader rdf:resource="descriptor/10213-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18134-3">
    <skos:broader rdf:resource="thsys/70361"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18134-3">
    <skos:broader rdf:resource="descriptor/18012-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11928-1">
    <skos:scopeNote xml:lang="de">Theorie über die räumliche Dimension des Wirtschaftens, über die Entstehung und den Aufbau der räumlichen Struktur wirtschaftlicher Aktivitäten.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19001-3">
    <skos:broader rdf:resource="thsys/71041"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19001-3">
    <skos:broader rdf:resource="thsys/70244"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19001-3">
    <skos:broader rdf:resource="descriptor/12567-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19001-3">
    <skos:broader rdf:resource="descriptor/12884-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73355">
    <skos:narrower rdf:resource="descriptor/16687-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73355">
    <skos:narrower rdf:resource="descriptor/16566-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73355">
    <skos:narrower rdf:resource="descriptor/13571-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73355">
    <skos:narrower rdf:resource="descriptor/16695-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73355">
    <skos:narrower rdf:resource="descriptor/16696-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73355">
    <skos:narrower rdf:resource="descriptor/11992-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73355">
    <skos:narrower rdf:resource="descriptor/13861-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73355">
    <skos:narrower rdf:resource="descriptor/16688-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73355">
    <skos:narrower rdf:resource="descriptor/12110-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73355">
    <skos:narrower rdf:resource="descriptor/16692-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73355">
    <skos:narrower rdf:resource="descriptor/11299-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73355">
    <skos:narrower rdf:resource="descriptor/11294-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73355">
    <skos:narrower rdf:resource="descriptor/11293-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73355">
    <skos:narrower rdf:resource="descriptor/11389-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73355">
    <skos:narrower rdf:resource="descriptor/19767-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73355">
    <skos:narrower rdf:resource="descriptor/13445-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12125-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18341-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10436-6">
    <skos:prefLabel xml:lang="en">Research</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10436-6">
    <skos:prefLabel xml:lang="de">Forschung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18637-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19704-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17011-3">
    <skos:broader rdf:resource="thsys/70131"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17011-3">
    <skos:broader rdf:resource="descriptor/17016-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11598-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19467-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14082-2">
    <skos:prefLabel xml:lang="en">Tobacco</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14082-2">
    <skos:prefLabel xml:lang="de">Tabak</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10427-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10099-5">
    <skos:related rdf:resource="descriptor/10110-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10099-5">
    <skos:related rdf:resource="descriptor/10102-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10099-5">
    <skos:related rdf:resource="descriptor/11770-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15416-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10263-4">
    <skos:related rdf:resource="descriptor/11449-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16013-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18842-4">
    <zbwext:useInsteadNote rdf:nodeID="id266"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18991-1">
    <skos:broader rdf:resource="thsys/70986"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18991-1">
    <skos:broader rdf:resource="descriptor/10213-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10880-1">
    <skos:broader rdf:resource="thsys/73333"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10880-1">
    <skos:broader rdf:resource="thsys/70951"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10880-1">
    <skos:broader rdf:resource="descriptor/19329-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10880-1">
    <skos:broader rdf:resource="descriptor/12787-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13179-6">
    <skos:related rdf:resource="descriptor/12979-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13179-6">
    <skos:related rdf:resource="descriptor/14053-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13179-6">
    <skos:related rdf:resource="descriptor/14188-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13179-6">
    <skos:related rdf:resource="descriptor/18630-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13179-6">
    <skos:related rdf:resource="descriptor/19624-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13179-6">
    <skos:related rdf:resource="descriptor/18380-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13179-6">
    <skos:related rdf:resource="descriptor/11578-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13179-6">
    <skos:related rdf:resource="descriptor/19632-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13179-6">
    <skos:related rdf:resource="descriptor/15731-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13179-6">
    <skos:related rdf:resource="descriptor/13529-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13179-6">
    <skos:related rdf:resource="descriptor/10490-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18957-1">
    <skos:broader rdf:resource="thsys/71081"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18957-1">
    <skos:broader rdf:resource="thsys/70818"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18957-1">
    <skos:broader rdf:resource="thsys/70239"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18957-1">
    <skos:broader rdf:resource="thsys/70209"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18957-1">
    <skos:broader rdf:resource="descriptor/15760-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18957-1">
    <skos:broader rdf:resource="descriptor/18958-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70873">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11007-3">
    <skos:broader rdf:resource="thsys/70038"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11007-3">
    <skos:broader rdf:resource="descriptor/11004-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19682-2">
    <skos:related rdf:resource="descriptor/16298-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19682-2">
    <skos:related rdf:resource="descriptor/16250-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19682-2">
    <skos:related rdf:resource="descriptor/16245-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19682-2">
    <skos:related rdf:resource="descriptor/19748-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11391-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10913-5">
    <skos:altLabel xml:lang="de">Konzentration der Wirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10913-5">
    <skos:altLabel xml:lang="de">Kapitalverflechtung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10913-5">
    <skos:altLabel xml:lang="de">Kapitalkonzentration</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10072-4">
    <skos:related rdf:resource="descriptor/10102-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10072-4">
    <skos:related rdf:resource="descriptor/10076-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18160-2">
    <skos:related rdf:resource="descriptor/11915-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18160-2">
    <skos:related rdf:resource="descriptor/19291-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18160-2">
    <skos:related rdf:resource="descriptor/11914-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18160-2">
    <skos:related rdf:resource="descriptor/11913-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18160-2">
    <skos:related rdf:resource="descriptor/10842-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18160-2">
    <skos:related rdf:resource="descriptor/19299-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18160-2">
    <skos:related rdf:resource="descriptor/10314-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18160-2">
    <skos:related rdf:resource="descriptor/11904-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17772-4">
    <skos:prefLabel xml:lang="en">Barbados</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17772-4">
    <skos:prefLabel xml:lang="de">Barbados</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70005">
    <skos:narrower rdf:resource="descriptor/19090-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70005">
    <skos:narrower rdf:resource="descriptor/12394-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70005">
    <skos:narrower rdf:resource="descriptor/12393-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70005">
    <skos:narrower rdf:resource="descriptor/11589-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70005">
    <skos:narrower rdf:resource="descriptor/12311-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70005">
    <skos:narrower rdf:resource="descriptor/12307-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70005">
    <skos:narrower rdf:resource="descriptor/12306-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70005">
    <skos:narrower rdf:resource="thsys/70578"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70005">
    <skos:narrower rdf:resource="thsys/70186"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70005">
    <skos:narrower rdf:resource="thsys/70004"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70005">
    <skos:narrower rdf:resource="thsys/71041"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11742-1">
    <skos:broader rdf:resource="thsys/71114"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11742-1">
    <skos:broader rdf:resource="thsys/70269"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11742-1">
    <skos:broader rdf:resource="descriptor/15045-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11742-1">
    <skos:broader rdf:resource="descriptor/11753-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18353-5">
    <skos:related rdf:resource="descriptor/13577-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18353-5">
    <skos:related rdf:resource="descriptor/15651-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18353-5">
    <skos:related rdf:resource="descriptor/12975-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18353-5">
    <skos:related rdf:resource="descriptor/14176-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18353-5">
    <skos:related rdf:resource="descriptor/18348-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11695-5">
    <skos:broader rdf:resource="thsys/71076"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11695-5">
    <skos:broader rdf:resource="descriptor/11696-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18622-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19245-3">
    <skos:broader rdf:resource="thsys/71110"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19245-3">
    <skos:broader rdf:resource="thsys/70179"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19245-3">
    <skos:broader rdf:resource="descriptor/11790-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70147">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19375-4">
    <skos:altLabel xml:lang="de">Flypaper Effekt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19300-0">
    <skos:related rdf:resource="descriptor/19298-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19300-0">
    <skos:related rdf:resource="descriptor/19297-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19300-0">
    <skos:related rdf:resource="descriptor/18197-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19300-0">
    <skos:related rdf:resource="descriptor/14095-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19183-6">
    <skos:related rdf:resource="descriptor/12946-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19183-6">
    <skos:related rdf:resource="descriptor/13158-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19183-6">
    <skos:related rdf:resource="descriptor/19677-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19183-6">
    <skos:related rdf:resource="descriptor/19545-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19183-6">
    <skos:related rdf:resource="descriptor/14124-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16207-5">
    <skos:prefLabel xml:lang="en">Political culture</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16207-5">
    <skos:prefLabel xml:lang="de">Politische Kultur</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19007-5">
    <skos:narrower rdf:resource="descriptor/19008-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17702-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19204-3">
    <skos:scopeNote xml:lang="de">Evolutionsökonomische Ansätze mit Schwerpunkt auf wirtschaftlicher Dynamik, selbstinduziertem Wandel (z.B. Institutionen, Verhalten, Einstellungen) und ungleichgewichtigen Prozessen zur Erklärung ökonomischer Entwicklungen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18905-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71037">
    <skos:prefLabel xml:lang="de">B.09.02  IS-Entwicklung und -Betrieb</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18559-6">
    <skos:altLabel xml:lang="de">Umweltfreundliches Produkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18559-6">
    <skos:altLabel xml:lang="de">Ökologisches Produkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70052">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10121-3">
    <zbwext:useInsteadNote rdf:nodeID="id164"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19050-4">
    <skos:broader rdf:resource="thsys/70348"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19050-4">
    <skos:broader rdf:resource="descriptor/14021-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10337-1">
    <skos:altLabel xml:lang="de">Sektoraler Konjunkturzyklus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10337-1">
    <skos:altLabel xml:lang="de">Branchenzyklus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13769-6">
    <skos:narrower rdf:resource="descriptor/18727-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13769-6">
    <skos:narrower rdf:resource="descriptor/13785-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13769-6">
    <skos:narrower rdf:resource="descriptor/13794-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19289-4">
    <skos:prefLabel xml:lang="en">Black market</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19289-4">
    <skos:prefLabel xml:lang="de">Schwarzmarkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12980-1">
    <skos:narrower rdf:resource="descriptor/12987-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12980-1">
    <skos:narrower rdf:resource="descriptor/12990-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12980-1">
    <skos:narrower rdf:resource="descriptor/18389-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12980-1">
    <skos:narrower rdf:resource="descriptor/12986-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12980-1">
    <skos:narrower rdf:resource="descriptor/13018-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12980-1">
    <skos:narrower rdf:resource="descriptor/18387-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12980-1">
    <skos:narrower rdf:resource="descriptor/12985-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12980-1">
    <skos:narrower rdf:resource="descriptor/12984-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12980-1">
    <skos:narrower rdf:resource="descriptor/12983-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70383">
    <rdfs:label xml:lang="de">Betriebliche Information und Kommunikation</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15381-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18798-2">
    <skos:prefLabel xml:lang="en">Trade fair marketing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18798-2">
    <skos:prefLabel xml:lang="de">Messemarketing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71112">
    <rdfs:label xml:lang="de">Konsum und Sparen</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13372-5">
    <skos:altLabel xml:lang="de">Vorsorgeklinik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13372-5">
    <skos:altLabel xml:lang="de">Stationäre Krankenversorgung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13372-5">
    <skos:altLabel xml:lang="de">Sanatorium</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13372-5">
    <skos:altLabel xml:lang="de">Reha-Zentrum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13372-5">
    <skos:altLabel xml:lang="de">Rehabilitationszentrum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13372-5">
    <skos:altLabel xml:lang="de">Rehabilitationseinrichtung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13372-5">
    <skos:altLabel xml:lang="de">Klinik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14463-4">
    <skos:narrower rdf:resource="descriptor/14455-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14463-4">
    <skos:narrower rdf:resource="descriptor/14446-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17712-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13496-1">
    <skos:prefLabel xml:lang="en">Cash and carry (Wholesale)</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13496-1">
    <skos:prefLabel xml:lang="de">Cash-und-Carry-Großhandel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11540-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20455-3">
    <skos:related rdf:resource="descriptor/16150-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20455-3">
    <skos:related rdf:resource="descriptor/11346-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20455-3">
    <skos:related rdf:resource="descriptor/18169-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17753-1">
    <skos:related rdf:resource="descriptor/17749-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19361-1">
    <skos:broader rdf:resource="thsys/70878"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19361-1">
    <skos:broader rdf:resource="thsys/70091"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19361-1">
    <skos:broader rdf:resource="thsys/70013"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19361-1">
    <skos:broader rdf:resource="descriptor/16554-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70183">
    <rdfs:label xml:lang="de">Kapitalbeschaffung</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18411-3">
    <skos:prefLabel xml:lang="en">Plastics machinery industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18411-3">
    <skos:prefLabel xml:lang="de">Kunststoffmaschinenbau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19041-5">
    <skos:prefLabel xml:lang="en">Criticism</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19041-5">
    <skos:prefLabel xml:lang="de">Kritik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19135-3">
    <skos:broader rdf:resource="thsys/70384"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10715-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18865-6">
    <skos:altLabel xml:lang="de">Dorfkulturarbeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18198-5">
    <skos:related rdf:resource="descriptor/19293-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18198-5">
    <skos:related rdf:resource="descriptor/19187-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18198-5">
    <skos:related rdf:resource="descriptor/19188-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18198-5">
    <skos:related rdf:resource="descriptor/18517-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18198-5">
    <skos:related rdf:resource="descriptor/14088-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15684-4">
    <skos:altLabel xml:lang="de">Wirtschaftsunterricht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15684-4">
    <skos:altLabel xml:lang="de">Wirtschaftskundeunterricht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15684-4">
    <skos:altLabel xml:lang="de">Wirtschaftserziehung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15684-4">
    <skos:altLabel xml:lang="de">Betriebspädagogik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15684-4">
    <skos:altLabel xml:lang="de">Berufspädagogik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15684-4">
    <skos:altLabel xml:lang="de">Berufserziehung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15684-4">
    <skos:altLabel xml:lang="de">Arbeitspädagogik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10902-3">
    <skos:altLabel xml:lang="de">Markterweiterung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10902-3">
    <skos:altLabel xml:lang="de">Marktdurchdringung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16014-2">
    <skos:related rdf:resource="descriptor/16019-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16014-2">
    <skos:related rdf:resource="descriptor/19714-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10209-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17608-3">
    <skos:altLabel xml:lang="de">Westasien</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17608-3">
    <skos:altLabel xml:lang="de">Vorderer Orient</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17608-3">
    <skos:altLabel xml:lang="de">Vorderasien</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17608-3">
    <skos:altLabel xml:lang="de">Nahost</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17608-3">
    <skos:altLabel xml:lang="de">Naher Osten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13378-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14180-2">
    <skos:related rdf:resource="descriptor/19619-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14180-2">
    <skos:related rdf:resource="descriptor/15873-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19065-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17994-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19776-0">
    <skos:related rdf:resource="descriptor/19779-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19776-0">
    <skos:related rdf:resource="descriptor/15676-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19776-0">
    <skos:related rdf:resource="descriptor/19608-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13998-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18382-5">
    <skos:prefLabel xml:lang="en">Commodity price</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18382-5">
    <skos:prefLabel xml:lang="de">Rohstoffpreis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70476">
    <skos:broader rdf:resource="thsys/70014"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18671-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11727-4">
    <skos:broader rdf:resource="thsys/71082"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11727-4">
    <skos:broader rdf:resource="descriptor/10381-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15974-2">
    <skos:related rdf:resource="descriptor/16016-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15974-2">
    <skos:related rdf:resource="descriptor/16258-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15974-2">
    <skos:related rdf:resource="descriptor/16315-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15974-2">
    <skos:related rdf:resource="descriptor/16246-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17933-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16958-2">
    <skos:scopeNote xml:lang="de">Schrifttum über das Gesamtgebiet der Kantone Basel-Stadt und Basel-Landschaft sowie über die Stadt Basel.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12214-0">
    <skos:altLabel xml:lang="de">Wertpapierberatung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12214-0">
    <skos:altLabel xml:lang="de">Vermögensberatung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10470-6">
    <skos:prefLabel xml:lang="en">Technology</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10470-6">
    <skos:prefLabel xml:lang="de">Technologie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21082-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17937-5">
    <skos:broader rdf:resource="thsys/70128"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17937-5">
    <skos:broader rdf:resource="descriptor/17948-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11129-3">
    <skos:prefLabel xml:lang="en">State fund (Central planning)</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11129-3">
    <skos:prefLabel xml:lang="de">Fonds (Planwirtschaft)</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15988-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11822-3">
    <skos:narrower rdf:resource="descriptor/18544-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11998-1">
    <skos:broader rdf:resource="thsys/71064"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11998-1">
    <skos:broader rdf:resource="descriptor/11900-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16638-6">
    <skos:related rdf:resource="descriptor/15770-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16638-6">
    <skos:related rdf:resource="descriptor/15073-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16638-6">
    <skos:related rdf:resource="descriptor/15715-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16638-6">
    <skos:related rdf:resource="descriptor/16639-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16638-6">
    <skos:related rdf:resource="descriptor/10498-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16638-6">
    <skos:related rdf:resource="descriptor/11305-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15743-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10092-5">
    <skos:broader rdf:resource="thsys/70209"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10092-5">
    <skos:broader rdf:resource="thsys/70057"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10092-5">
    <skos:broader rdf:resource="descriptor/10113-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15945-2">
    <skos:broader rdf:resource="thsys/73358"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15945-2">
    <skos:broader rdf:resource="thsys/70142"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11073-4">
    <skos:altLabel xml:lang="en">Labor theory</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11073-4">
    <skos:altLabel xml:lang="en">Exploitation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11073-4">
    <skos:altLabel xml:lang="de">Theorie der unproduktiven Arbeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11073-4">
    <skos:altLabel xml:lang="de">Theorie der produktiven Arbeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11073-4">
    <skos:altLabel xml:lang="de">Gesellschaftlich notwendige Arbeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11073-4">
    <skos:altLabel xml:lang="de">Entfremdete Arbeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11073-4">
    <skos:altLabel xml:lang="de">Ausbeutung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17793-3">
    <skos:altLabel xml:lang="de">Republik Panama</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11036-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13155-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14980-5">
    <skos:related rdf:resource="descriptor/14948-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14980-5">
    <skos:related rdf:resource="descriptor/14110-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18083-1">
    <skos:broader rdf:resource="thsys/71058"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18083-1">
    <skos:broader rdf:resource="descriptor/11294-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18083-1">
    <skos:broader rdf:resource="descriptor/11262-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19261-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14654-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12985-5">
    <skos:related rdf:resource="descriptor/19184-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12985-5">
    <skos:related rdf:resource="descriptor/13032-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12985-5">
    <skos:related rdf:resource="descriptor/14198-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70987">
    <skos:broader rdf:resource="thsys/71004"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16214-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10774-0">
    <skos:related rdf:resource="descriptor/16046-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10774-0">
    <skos:related rdf:resource="descriptor/18704-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10774-0">
    <skos:related rdf:resource="descriptor/10721-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10774-0">
    <skos:related rdf:resource="descriptor/19665-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10774-0">
    <skos:related rdf:resource="descriptor/10382-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10774-0">
    <skos:related rdf:resource="descriptor/13283-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12174-0">
    <skos:narrower rdf:resource="descriptor/18109-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19382-0">
    <skos:prefLabel xml:lang="en">Fisheries market</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19382-0">
    <skos:prefLabel xml:lang="de">Fischmarkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18701-1">
    <skos:broader rdf:resource="thsys/71103"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18701-1">
    <skos:broader rdf:resource="thsys/71089"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18701-1">
    <skos:broader rdf:resource="thsys/71088"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18701-1">
    <skos:broader rdf:resource="thsys/70455"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18701-1">
    <skos:broader rdf:resource="descriptor/13766-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24703-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16081-1">
    <skos:scopeNote xml:lang="de">Regelungen des öffentlichen und des Privatrechts. Keine einheitliche Abgrenzung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70995">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15648-1">
    <skos:broader rdf:resource="thsys/73388"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15648-1">
    <skos:broader rdf:resource="descriptor/15671-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18907-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14189-5">
    <skos:prefLabel xml:lang="en">Water power</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14189-5">
    <skos:prefLabel xml:lang="de">Wasserkraft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15676-3">
    <skos:broader rdf:resource="thsys/70580"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10814-0">
    <skos:related rdf:resource="descriptor/10830-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10814-0">
    <skos:related rdf:resource="descriptor/10725-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11479-4">
    <skos:altLabel xml:lang="en">Money stock targeting</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11479-4">
    <skos:altLabel xml:lang="de">Potenzialorientierte Geldpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11479-4">
    <skos:altLabel xml:lang="de">Potentialorientierte Geldpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11479-4">
    <skos:altLabel xml:lang="de">Geldmengenziel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11479-4">
    <skos:altLabel xml:lang="de">Geldmengenregulierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11479-4">
    <skos:altLabel xml:lang="de">Geldmengenregel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11479-4">
    <skos:altLabel xml:lang="de">Geldmengenpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11479-4">
    <skos:altLabel xml:lang="de">Friedman-Plan</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18682-0">
    <skos:prefLabel xml:lang="en">Stock exchange</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18682-0">
    <skos:prefLabel xml:lang="de">Wertpapierbörse</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16783-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11243-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18031-6">
    <skos:prefLabel xml:lang="en">Technology centre</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18031-6">
    <skos:prefLabel xml:lang="de">Technologiezentrum</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15992-0">
    <skos:broader rdf:resource="thsys/73360"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15992-0">
    <skos:broader rdf:resource="thsys/70994"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15992-0">
    <skos:broader rdf:resource="descriptor/16019-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19795-3">
    <skos:altLabel xml:lang="de">Technische Wissenschaften</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13485-6">
    <skos:prefLabel xml:lang="en">Customer tracking</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13485-6">
    <skos:prefLabel xml:lang="de">Kundenanalyse</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13427-6">
    <skos:prefLabel xml:lang="en">Fertilizer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13427-6">
    <skos:prefLabel xml:lang="de">Düngemittel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15988-5">
    <skos:prefLabel xml:lang="en">Waste law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15988-5">
    <skos:prefLabel xml:lang="de">Abfallrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18519-4">
    <skos:altLabel xml:lang="de">Kaffeegroßhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18519-4">
    <skos:altLabel xml:lang="de">Kaffeefachhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18519-4">
    <skos:altLabel xml:lang="de">Kaffee-Einzelhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18648-0">
    <skos:prefLabel xml:lang="en">Phone-banking</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18648-0">
    <skos:prefLabel xml:lang="de">Telefonbanking</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16269-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13562-0">
    <skos:broader rdf:resource="thsys/70970"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13562-0">
    <skos:broader rdf:resource="thsys/70938"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13562-0">
    <skos:broader rdf:resource="thsys/70926"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13562-0">
    <skos:broader rdf:resource="descriptor/18630-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13562-0">
    <skos:broader rdf:resource="descriptor/18579-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13562-0">
    <skos:broader rdf:resource="descriptor/18507-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12721-4">
    <skos:related rdf:resource="descriptor/10007-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12721-4">
    <skos:related rdf:resource="descriptor/18457-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10320-4">
    <skos:prefLabel xml:lang="en">Joint production</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10320-4">
    <skos:prefLabel xml:lang="de">Kuppelproduktion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#related">
    <rdfs:label xml:lang="en">related to</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12585-0">
    <skos:broader rdf:resource="thsys/73385"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12585-0">
    <skos:broader rdf:resource="thsys/70244"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12585-0">
    <skos:broader rdf:resource="descriptor/12567-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12585-0">
    <skos:broader rdf:resource="descriptor/12085-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18911-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11973-3">
    <skos:prefLabel xml:lang="en">City district</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11973-3">
    <skos:prefLabel xml:lang="de">Stadtteil</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17496-2">
    <skos:scopeNote xml:lang="de">Erklärte sich 1991 unabhängig von der Sowjetunion und trat der GUS bei</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12966-2">
    <skos:broader rdf:resource="thsys/71018"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12966-2">
    <skos:broader rdf:resource="descriptor/12964-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12697-3">
    <skos:prefLabel xml:lang="en">Economic adjustment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12697-3">
    <skos:prefLabel xml:lang="de">Wirtschaftliche Anpassung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18921-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16683-1">
    <skos:prefLabel xml:lang="en">Medical sociology</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16683-1">
    <skos:prefLabel xml:lang="de">Gesundheitssoziologie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13081-2">
    <skos:broader rdf:resource="thsys/70995"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13081-2">
    <skos:broader rdf:resource="descriptor/12993-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12617-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10040-3">
    <skos:related rdf:resource="descriptor/10326-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10040-3">
    <skos:related rdf:resource="descriptor/10201-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10040-3">
    <skos:related rdf:resource="descriptor/24705-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10040-3">
    <skos:related rdf:resource="descriptor/10076-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10040-3">
    <skos:related rdf:resource="descriptor/11004-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19228-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13307-2">
    <skos:related rdf:resource="descriptor/12927-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13307-2">
    <skos:related rdf:resource="descriptor/13975-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13065-0">
    <skos:related rdf:resource="descriptor/13563-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13065-0">
    <skos:related rdf:resource="descriptor/14441-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14630-4">
    <skos:broader rdf:resource="thsys/70074"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14630-4">
    <skos:broader rdf:resource="descriptor/14602-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10274-6">
    <skos:broader rdf:resource="thsys/71115"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10274-6">
    <skos:broader rdf:resource="thsys/71111"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10274-6">
    <skos:broader rdf:resource="descriptor/10272-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10274-6">
    <skos:broader rdf:resource="descriptor/10024-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12510-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14195-3">
    <skos:altLabel xml:lang="de">Magnetit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14195-3">
    <skos:altLabel xml:lang="de">Eisenkies</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18689-0">
    <skos:related rdf:resource="descriptor/18684-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18689-0">
    <skos:related rdf:resource="descriptor/13712-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13264-1">
    <skos:altLabel xml:lang="de">Verkehrswirtschaftssektor</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13264-1">
    <skos:altLabel xml:lang="de">Verkehrssektor</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13264-1">
    <skos:altLabel xml:lang="de">Transportwirtschaftssektor</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13264-1">
    <skos:altLabel xml:lang="de">Transportwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13264-1">
    <skos:altLabel xml:lang="de">Transportgewerbe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13264-1">
    <skos:altLabel xml:lang="de">Gewerblicher Verkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15811-2">
    <skos:broader rdf:resource="thsys/73386"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15811-2">
    <skos:broader rdf:resource="descriptor/16563-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15811-2">
    <skos:broader rdf:resource="descriptor/15790-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15647-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19168-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18328-4">
    <skos:prefLabel xml:lang="en">Intensity of land use</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18328-4">
    <skos:prefLabel xml:lang="de">Bodennutzungsintensität</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17496-2">
    <skos:broader rdf:resource="thsys/71032"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17496-2">
    <skos:broader rdf:resource="descriptor/17595-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17496-2">
    <skos:broader rdf:resource="descriptor/19078-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24705-0">
    <skos:prefLabel xml:lang="en">Commercialization</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24705-0">
    <skos:prefLabel xml:lang="de">Kommerzialisierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10243-3">
    <skos:prefLabel xml:lang="en">Input-output model</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10243-3">
    <skos:prefLabel xml:lang="de">Input-Output</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19279-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11192-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19246-1">
    <skos:related rdf:resource="descriptor/16552-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19246-1">
    <skos:related rdf:resource="descriptor/11910-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19246-1">
    <skos:related rdf:resource="descriptor/11895-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12225-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15036-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17811-6">
    <skos:broader rdf:resource="thsys/70080"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17811-6">
    <skos:broader rdf:resource="descriptor/17806-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17811-6">
    <skos:broader rdf:resource="descriptor/17991-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17811-6">
    <skos:broader rdf:resource="descriptor/19130-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17811-6">
    <skos:broader rdf:resource="descriptor/19178-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12108-6">
    <skos:scopeNote xml:lang="de">System von Verhaltensnormen in einer Unternehmung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16102-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15416-1">
    <skos:related rdf:resource="descriptor/15429-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15416-1">
    <skos:related rdf:resource="descriptor/18647-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15416-1">
    <skos:related rdf:resource="descriptor/15659-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15416-1">
    <skos:related rdf:resource="descriptor/10454-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15416-1">
    <skos:related rdf:resource="descriptor/15415-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15416-1">
    <skos:related rdf:resource="descriptor/12612-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15416-1">
    <skos:related rdf:resource="descriptor/19801-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24855-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15912-3">
    <skos:related rdf:resource="descriptor/10031-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15912-3">
    <skos:related rdf:resource="descriptor/15072-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15912-3">
    <skos:related rdf:resource="descriptor/15061-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15912-3">
    <skos:related rdf:resource="descriptor/15708-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15912-3">
    <skos:related rdf:resource="descriptor/15909-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13735-2">
    <skos:broader rdf:resource="thsys/71104"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13735-2">
    <skos:broader rdf:resource="descriptor/10221-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13735-2">
    <skos:broader rdf:resource="descriptor/18668-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13045-6">
    <skos:altLabel xml:lang="de">Baustoffmaschinenindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16309-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13033-6">
    <skos:altLabel xml:lang="de">Kupferverarbeitung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13033-6">
    <skos:altLabel xml:lang="de">Kupferschmelze</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13033-6">
    <skos:altLabel xml:lang="de">Kupferraffinerie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13033-6">
    <skos:altLabel xml:lang="de">Kupfererzeugung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15685-2">
    <skos:broader rdf:resource="thsys/73393"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16497-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13630-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13394-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19721-4">
    <skos:altLabel xml:lang="de">Kriegsgefallene</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19721-4">
    <skos:altLabel xml:lang="de">Kriegsbeschädigte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19721-4">
    <skos:altLabel xml:lang="de">Kriegerwitwen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16158-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16808-0">
    <skos:broader rdf:resource="thsys/70198"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14193-0">
    <skos:prefLabel xml:lang="en">Oil Shale</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14193-0">
    <skos:prefLabel xml:lang="de">Ölschiefer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71069">
    <skos:broader rdf:resource="thsys/70058"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18707-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15962-2">
    <skos:altLabel xml:lang="en">Judicial administration</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15962-2">
    <skos:altLabel xml:lang="de">Justizbehörde</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15410-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11400-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13046-4">
    <skos:broader rdf:resource="thsys/70997"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13046-4">
    <skos:broader rdf:resource="descriptor/13069-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15328-5">
    <skos:related rdf:resource="descriptor/19573-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15328-5">
    <skos:related rdf:resource="descriptor/15340-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15328-5">
    <skos:related rdf:resource="descriptor/10045-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15328-5">
    <skos:related rdf:resource="descriptor/15327-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11042-1">
    <skos:related rdf:resource="descriptor/11064-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11042-1">
    <skos:related rdf:resource="descriptor/11481-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11042-1">
    <skos:related rdf:resource="descriptor/11416-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11042-1">
    <skos:related rdf:resource="descriptor/11415-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12436-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11374-4">
    <skos:related rdf:resource="descriptor/11382-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11374-4">
    <skos:related rdf:resource="descriptor/15666-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11374-4">
    <skos:related rdf:resource="descriptor/11559-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11374-4">
    <skos:related rdf:resource="descriptor/18857-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15662-0">
    <skos:broader rdf:resource="thsys/73351"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18191-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13383-0">
    <skos:broader rdf:resource="thsys/73386"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13383-0">
    <skos:broader rdf:resource="thsys/70876"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13383-0">
    <skos:broader rdf:resource="thsys/70292"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13383-0">
    <skos:broader rdf:resource="thsys/70011"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13383-0">
    <skos:broader rdf:resource="descriptor/16554-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10401-4">
    <skos:related rdf:resource="descriptor/11752-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10401-4">
    <skos:related rdf:resource="descriptor/16484-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10401-4">
    <skos:related rdf:resource="descriptor/20077-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10401-4">
    <skos:related rdf:resource="descriptor/11350-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10401-4">
    <skos:related rdf:resource="descriptor/16384-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10401-4">
    <skos:related rdf:resource="descriptor/10380-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10401-4">
    <skos:related rdf:resource="descriptor/11239-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10401-4">
    <skos:related rdf:resource="descriptor/11314-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14302-0">
    <skos:prefLabel xml:lang="en">Non-ferrous metal</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14302-0">
    <skos:prefLabel xml:lang="de">NE-Metall</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72394">
    <skos:broader rdf:resource="thsys/72180"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14789-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id71">
    <rdfs:seeAlso rdf:resource="descriptor/16754-4"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id17">
    <rdfs:seeAlso rdf:resource="descriptor/16566-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18868-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14730-0">
    <skos:broader rdf:resource="thsys/70199"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14730-0">
    <skos:broader rdf:resource="descriptor/14728-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16742-4">
    <skos:altLabel xml:lang="de">Romantische Staats- und Wirtschaftslehre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16742-4">
    <skos:altLabel xml:lang="de">Romantische Schule</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73360">
    <rdfs:label xml:lang="en">Administrative law</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73360">
    <rdfs:label xml:lang="de">Verwaltungsrecht</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13985-0">
    <skos:broader rdf:resource="thsys/71054"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13985-0">
    <skos:broader rdf:resource="descriptor/18535-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20977-1">
    <skos:related rdf:resource="descriptor/12153-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20977-1">
    <skos:related rdf:resource="descriptor/12137-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71084">
    <skos:narrower rdf:resource="descriptor/18869-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71084">
    <skos:narrower rdf:resource="descriptor/10581-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71084">
    <skos:narrower rdf:resource="descriptor/10022-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71084">
    <skos:narrower rdf:resource="descriptor/10472-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71084">
    <skos:narrower rdf:resource="descriptor/10598-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71084">
    <skos:narrower rdf:resource="descriptor/10597-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71084">
    <skos:narrower rdf:resource="descriptor/18157-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71084">
    <skos:narrower rdf:resource="descriptor/18061-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71084">
    <skos:narrower rdf:resource="descriptor/10593-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71084">
    <skos:narrower rdf:resource="descriptor/10590-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71084">
    <skos:narrower rdf:resource="descriptor/10589-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71084">
    <skos:narrower rdf:resource="descriptor/10587-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71084">
    <skos:narrower rdf:resource="descriptor/13395-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71084">
    <skos:narrower rdf:resource="descriptor/10586-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71084">
    <skos:narrower rdf:resource="descriptor/10583-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71084">
    <skos:narrower rdf:resource="descriptor/10576-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71084">
    <skos:narrower rdf:resource="descriptor/10573-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71084">
    <skos:narrower rdf:resource="descriptor/10572-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71084">
    <skos:narrower rdf:resource="descriptor/10568-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71084">
    <skos:narrower rdf:resource="descriptor/10566-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71084">
    <skos:narrower rdf:resource="descriptor/10561-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71084">
    <skos:narrower rdf:resource="descriptor/10559-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71084">
    <skos:narrower rdf:resource="descriptor/10558-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71084">
    <skos:narrower rdf:resource="descriptor/10557-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71084">
    <skos:narrower rdf:resource="descriptor/10556-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71084">
    <skos:narrower rdf:resource="descriptor/10555-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71084">
    <skos:narrower rdf:resource="descriptor/10553-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71084">
    <skos:narrower rdf:resource="descriptor/10552-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71084">
    <skos:narrower rdf:resource="descriptor/10550-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71084">
    <skos:narrower rdf:resource="descriptor/16066-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71084">
    <skos:narrower rdf:resource="descriptor/10543-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71084">
    <skos:narrower rdf:resource="descriptor/19358-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71084">
    <skos:narrower rdf:resource="descriptor/18695-5"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id127">
    <rdf:value xml:lang="de">Für Waren/Produkte im Sinne der Betriebswirtschaftslehre und der Wirtschaftspraxis</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11590-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19563-3">
    <skos:prefLabel xml:lang="en">Borrowing constraints</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19563-3">
    <skos:prefLabel xml:lang="de">Verschuldungsrestriktion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11275-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11336-5">
    <skos:broader rdf:resource="thsys/71062"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11336-5">
    <skos:broader rdf:resource="thsys/70278"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11336-5">
    <skos:broader rdf:resource="descriptor/11219-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10826-0">
    <skos:altLabel xml:lang="en">Letter box company</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19232-5">
    <skos:altLabel xml:lang="de">Zugewinn</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19232-5">
    <skos:altLabel xml:lang="de">Kapitalzugewinn</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14327-5">
    <skos:altLabel xml:lang="de">Auspuffkatalysator</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14327-5">
    <skos:altLabel xml:lang="de">Abgaskatalysator</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20289-1">
    <skos:prefLabel xml:lang="en">Regulation approach</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20289-1">
    <skos:prefLabel xml:lang="de">Regulationsschule</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11443-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19765-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#topConceptOf">
    <skos:definition xml:lang="en">Relates a concept to the concept scheme that it is a top level
      concept of.</skos:definition>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71011">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.04.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11587-1">
    <skos:altLabel xml:lang="en">Rainy day fund</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17461-0">
    <skos:altLabel xml:lang="x-other">Sovjetskij Sojuz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17461-0">
    <skos:altLabel xml:lang="x-other">Sojuz Sovetskich Socialisticeskich Respublik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17461-0">
    <skos:altLabel xml:lang="en">Soviet Union</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17461-0">
    <skos:altLabel xml:lang="de">Union der Sozialistischen Sowjetrepubliken</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17461-0">
    <skos:altLabel xml:lang="de">UdSSR</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17461-0">
    <skos:altLabel xml:lang="de">SSSR</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11361-6">
    <skos:related rdf:resource="descriptor/11379-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11361-6">
    <skos:related rdf:resource="descriptor/19112-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11361-6">
    <skos:related rdf:resource="descriptor/11290-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11361-6">
    <skos:related rdf:resource="descriptor/11384-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11361-6">
    <skos:related rdf:resource="descriptor/11406-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10114-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14285-2">
    <skos:narrower rdf:resource="descriptor/14279-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18384-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11220-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18176-1">
    <skos:prefLabel xml:lang="en">Intellectual property rights</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18176-1">
    <skos:prefLabel xml:lang="de">Immaterialgüterrechte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11881-1">
    <skos:scopeNote xml:lang="de">Verkehr im Ballungsraum.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19540-1">
    <skos:broader rdf:resource="thsys/70183"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19540-1">
    <skos:broader rdf:resource="thsys/70180"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12284-0">
    <skos:related rdf:resource="descriptor/12259-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12284-0">
    <skos:related rdf:resource="descriptor/12273-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12284-0">
    <skos:related rdf:resource="descriptor/12239-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11611-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19308-5">
    <skos:scopeNote xml:lang="de">Auslagerung von Fertigungsprozessen in Billiglohnländer.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10276-2">
    <skos:related rdf:resource="descriptor/15108-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10276-2">
    <skos:related rdf:resource="descriptor/10285-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10276-2">
    <skos:related rdf:resource="descriptor/10181-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10276-2">
    <skos:related rdf:resource="descriptor/10376-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10276-2">
    <skos:related rdf:resource="descriptor/10278-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13825-1">
    <skos:related rdf:resource="descriptor/16081-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13825-1">
    <skos:related rdf:resource="descriptor/13820-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13825-1">
    <skos:related rdf:resource="descriptor/13818-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13825-1">
    <skos:related rdf:resource="descriptor/13828-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13825-1">
    <skos:related rdf:resource="descriptor/16485-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13825-1">
    <skos:related rdf:resource="descriptor/16160-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11506-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18700-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19772-1">
    <skos:altLabel xml:lang="de">Sozialneid</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14127-6">
    <skos:related rdf:resource="descriptor/14916-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14127-6">
    <skos:related rdf:resource="descriptor/12971-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14127-6">
    <skos:related rdf:resource="descriptor/13135-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14127-6">
    <skos:related rdf:resource="descriptor/14829-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19846-5">
    <skos:prefLabel xml:lang="en">Consumer boycott</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19846-5">
    <skos:prefLabel xml:lang="de">Verbraucherboykott</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18269-1">
    <skos:prefLabel xml:lang="en">Laptop</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18269-1">
    <skos:prefLabel xml:lang="de">Laptop</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19106-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19433-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15176-2">
    <skos:altLabel xml:lang="en">Random number generation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15176-2">
    <skos:altLabel xml:lang="en">Random number</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15176-2">
    <skos:altLabel xml:lang="en">Pseudorandom number generation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15176-2">
    <skos:altLabel xml:lang="de">Zufallsziffer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15176-2">
    <skos:altLabel xml:lang="de">Zufallszahlengenerator</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15176-2">
    <skos:altLabel xml:lang="de">Zufallszahlenerzeugung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15176-2">
    <skos:altLabel xml:lang="de">Zufallszahl</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15176-2">
    <skos:altLabel xml:lang="de">Zufallsgröße</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15176-2">
    <skos:altLabel xml:lang="de">Stochastische Variable</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15176-2">
    <skos:altLabel xml:lang="de">Stetige Zufallsvariable</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15176-2">
    <skos:altLabel xml:lang="de">Pseudozufallszahlenerzeugung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16214-1">
    <skos:narrower rdf:resource="descriptor/15025-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16722-3">
    <skos:prefLabel xml:lang="en">Scholasticism</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16722-3">
    <skos:prefLabel xml:lang="de">Scholastik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19141-1">
    <skos:altLabel xml:lang="de">Kundinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19141-1">
    <skos:altLabel xml:lang="de">Käuferinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19141-1">
    <skos:altLabel xml:lang="de">Käufer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13829-0">
    <skos:related rdf:resource="descriptor/13860-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13829-0">
    <skos:related rdf:resource="descriptor/13305-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13829-0">
    <skos:related rdf:resource="descriptor/11848-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13829-0">
    <skos:related rdf:resource="descriptor/11847-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13829-0">
    <skos:related rdf:resource="descriptor/13845-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13829-0">
    <skos:related rdf:resource="descriptor/18277-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13829-0">
    <skos:related rdf:resource="descriptor/13830-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13829-0">
    <skos:related rdf:resource="descriptor/19532-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13829-0">
    <skos:related rdf:resource="descriptor/13312-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16384-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18539-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10099-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73359">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11264-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18092-0">
    <skos:broader rdf:resource="thsys/70956"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18092-0">
    <skos:broader rdf:resource="thsys/70394"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18092-0">
    <skos:broader rdf:resource="descriptor/12670-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18092-0">
    <skos:broader rdf:resource="descriptor/12165-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10713-6">
    <skos:broader rdf:resource="thsys/71096"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10713-6">
    <skos:broader rdf:resource="descriptor/10686-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19121-0">
    <skos:broader rdf:resource="thsys/70041"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19121-0">
    <skos:broader rdf:resource="descriptor/12041-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11241-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15624-1">
    <skos:prefLabel xml:lang="en">Systems analysis (Data processing)</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15624-1">
    <skos:prefLabel xml:lang="de">Systemanalyse (Datenverarbeitung)</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18775-0">
    <skos:broader rdf:resource="thsys/71109"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18775-0">
    <skos:broader rdf:resource="thsys/70879"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19254-2">
    <skos:broader rdf:resource="thsys/71110"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19254-2">
    <skos:broader rdf:resource="descriptor/10011-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11793-5">
    <skos:prefLabel xml:lang="en">Growth industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11793-5">
    <skos:prefLabel xml:lang="de">Wachstumsbranche</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18647-2">
    <skos:scopeNote xml:lang="de">Multimedia bedeutet, computerunterstützt integriert verschiedene visuelle und akustische Medien (z.B. Standbilder, Bewegtbilder, Sprache, Text, Ton) zur Verdeutlichung von Informationen einzusetzen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70838">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12284-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25501-4">
    <skos:prefLabel xml:lang="en">Manufacture (Historical)</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25501-4">
    <skos:prefLabel xml:lang="de">Manufaktur</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10843-0">
    <skos:scopeNote xml:lang="de">Außenhandel innerhalb eines Wirtschaftszweigs.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17999-4">
    <skos:altLabel xml:lang="de">Westliche Welt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17999-4">
    <skos:altLabel xml:lang="de">Westliche Länder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17999-4">
    <skos:altLabel xml:lang="de">Kapitalistische Staaten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17999-4">
    <skos:altLabel xml:lang="de">Kapitalistische Länder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16249-3">
    <skos:altLabel xml:lang="de">Nationalsozialismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16249-3">
    <skos:altLabel xml:lang="de">Faschismustheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16249-3">
    <skos:altLabel xml:lang="de">Falangismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13536-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13281-1">
    <skos:altLabel xml:lang="en">Pawnbrokery</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13281-1">
    <skos:altLabel xml:lang="de">Pfandhaus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19033-4">
    <skos:altLabel xml:lang="de">Meßverfahren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19033-4">
    <skos:altLabel xml:lang="de">Messverfahren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11260-5">
    <skos:prefLabel xml:lang="en">Time use study</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11260-5">
    <skos:prefLabel xml:lang="de">Zeitbudgetforschung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18702-6">
    <skos:prefLabel xml:lang="en">Investor protection</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18702-6">
    <skos:prefLabel xml:lang="de">Anlegerschutz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12531-2">
    <skos:prefLabel xml:lang="en">Cost estimate</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12531-2">
    <skos:prefLabel xml:lang="de">Kalkulation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18738-6">
    <skos:broader rdf:resource="thsys/70891"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18738-6">
    <skos:broader rdf:resource="thsys/70839"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18738-6">
    <skos:broader rdf:resource="thsys/70573"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18738-6">
    <skos:broader rdf:resource="descriptor/13811-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18738-6">
    <skos:broader rdf:resource="descriptor/13292-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18154-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16061-0">
    <skos:related rdf:resource="descriptor/12281-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18918-4">
    <skos:prefLabel xml:lang="en">Showmen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18918-4">
    <skos:prefLabel xml:lang="de">Schausteller</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12744-6">
    <skos:narrower rdf:resource="descriptor/12741-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10735-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13959-1">
    <skos:altLabel xml:lang="de">Chemikerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13744-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10690-6">
    <skos:broader rdf:resource="thsys/71098"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10690-6">
    <skos:broader rdf:resource="descriptor/10686-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11222-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70481">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.02.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14230-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70853">
    <skos:broader rdf:resource="thsys/70856"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19285-5">
    <skos:related rdf:resource="descriptor/11068-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19285-5">
    <skos:related rdf:resource="descriptor/10343-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19285-5">
    <skos:related rdf:resource="descriptor/11039-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19285-5">
    <skos:related rdf:resource="descriptor/12697-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19285-5">
    <skos:related rdf:resource="descriptor/11063-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19285-5">
    <skos:related rdf:resource="descriptor/16295-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19285-5">
    <skos:related rdf:resource="descriptor/19769-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19285-5">
    <skos:related rdf:resource="descriptor/19683-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19285-5">
    <skos:related rdf:resource="descriptor/19668-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73355">
    <rdfs:label xml:lang="de">Wirtschafts- und Industriesoziologie, Siedlungssoziologie</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10488-1">
    <skos:altLabel xml:lang="de">Verelendung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10488-1">
    <skos:altLabel xml:lang="de">Verarmung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10488-1">
    <skos:altLabel xml:lang="de">Theorie der Armut</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10488-1">
    <skos:altLabel xml:lang="de">Pauperismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10488-1">
    <skos:altLabel xml:lang="de">Massenarmut</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10421-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17624-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18367-1">
    <skos:prefLabel xml:lang="en">Nuclear energy policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18367-1">
    <skos:prefLabel xml:lang="de">Kernenergiepolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13460-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16471-2">
    <skos:prefLabel xml:lang="en">Intervention</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16471-2">
    <skos:prefLabel xml:lang="de">Intervention</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17950-6">
    <skos:broader rdf:resource="thsys/70128"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17950-6">
    <skos:broader rdf:resource="descriptor/17948-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16190-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10022-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10904-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16280-2">
    <skos:narrower rdf:resource="descriptor/16396-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#hasTopConcept">
    <skos:definition xml:lang="en">A top level concept in the concept scheme.</skos:definition>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11111-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19357-6">
    <skos:prefLabel xml:lang="en">Regional foreign trade</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19357-6">
    <skos:prefLabel xml:lang="de">Regionaler Außenhandel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11903-3">
    <skos:related rdf:resource="descriptor/11987-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11903-3">
    <skos:related rdf:resource="descriptor/11880-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11903-3">
    <skos:related rdf:resource="descriptor/11928-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11903-3">
    <skos:related rdf:resource="descriptor/10562-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12308-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24857-5">
    <skos:related rdf:resource="descriptor/19524-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24857-5">
    <skos:related rdf:resource="descriptor/10201-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24857-5">
    <skos:related rdf:resource="descriptor/12103-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24857-5">
    <skos:related rdf:resource="descriptor/19199-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11890-0">
    <skos:related rdf:resource="descriptor/11999-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11890-0">
    <skos:related rdf:resource="descriptor/11901-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11443-4">
    <skos:prefLabel xml:lang="en">Banking theory</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11443-4">
    <skos:prefLabel xml:lang="de">Banking-Theorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19524-6">
    <skos:altLabel xml:lang="en">External shock</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19524-6">
    <skos:altLabel xml:lang="en">Exogenous shock</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19524-6">
    <skos:altLabel xml:lang="de">Externer Schock</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19524-6">
    <skos:altLabel xml:lang="de">Exogener Schock</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16012-6">
    <skos:broader rdf:resource="thsys/73360"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16012-6">
    <skos:broader rdf:resource="thsys/70824"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16012-6">
    <skos:broader rdf:resource="descriptor/13660-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70856">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70242">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19028-4">
    <skos:altLabel xml:lang="de">Global</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14738-5">
    <skos:prefLabel xml:lang="en">Soap</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14738-5">
    <skos:prefLabel xml:lang="de">Seife</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19208-2">
    <skos:altLabel xml:lang="en">Efficient allocation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19208-2">
    <skos:altLabel xml:lang="de">Allokative Effizienz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70824">
    <skos:prefLabel xml:lang="de">W.29.01  Rundfunk- und Fernsehanstalten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10789-1">
    <skos:broader rdf:resource="thsys/70114"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10789-1">
    <skos:broader rdf:resource="descriptor/13793-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19453-3">
    <skos:broader rdf:resource="thsys/71066"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19453-3">
    <skos:broader rdf:resource="thsys/71064"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19453-3">
    <skos:broader rdf:resource="descriptor/11859-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15373-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15888-2">
    <skos:prefLabel xml:lang="en">Meteorology</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15888-2">
    <skos:prefLabel xml:lang="de">Meteorologie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18952-4">
    <skos:broader rdf:resource="thsys/70822"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18952-4">
    <skos:broader rdf:resource="descriptor/19660-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18202-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19561-0">
    <skos:broader rdf:resource="thsys/71033"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19561-0">
    <skos:broader rdf:resource="thsys/70082"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19561-0">
    <skos:broader rdf:resource="descriptor/18325-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19561-0">
    <skos:broader rdf:resource="descriptor/18960-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19371-5">
    <skos:prefLabel xml:lang="en">Inflation tax</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19371-5">
    <skos:prefLabel xml:lang="de">Inflationssteuer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14456-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14705-6">
    <skos:broader rdf:resource="thsys/70476"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14705-6">
    <skos:broader rdf:resource="descriptor/14710-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15365-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12138-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18921-1">
    <skos:broader rdf:resource="thsys/70828"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18921-1">
    <skos:broader rdf:resource="descriptor/10032-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70829">
    <skos:prefLabel xml:lang="en">W.27  Leisure services</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70829">
    <skos:prefLabel xml:lang="de">W.27  Freizeitwirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12677-2">
    <skos:broader rdf:resource="thsys/70300"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12677-2">
    <skos:broader rdf:resource="descriptor/12670-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18815-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14101-3">
    <skos:prefLabel xml:lang="en">Nursery product</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14101-3">
    <skos:prefLabel xml:lang="de">Baumschulprodukt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11572-0">
    <skos:altLabel xml:lang="de">Wirtschaftssubvention</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11572-0">
    <skos:altLabel xml:lang="de">Subventionspolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11572-0">
    <skos:altLabel xml:lang="de">Öffentliche Finanzhilfe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11572-0">
    <skos:altLabel xml:lang="de">Finanzierungshilfe des Staates</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11572-0">
    <skos:altLabel xml:lang="de">Beihilfe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16704-5">
    <skos:related rdf:resource="descriptor/16721-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16704-5">
    <skos:related rdf:resource="descriptor/15965-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18094-3">
    <skos:broader rdf:resource="thsys/70987"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18094-3">
    <skos:broader rdf:resource="descriptor/12993-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11519-4">
    <skos:narrower rdf:resource="descriptor/11325-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15995-1">
    <skos:broader rdf:resource="thsys/73367"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15995-1">
    <skos:broader rdf:resource="thsys/73360"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15995-1">
    <skos:broader rdf:resource="descriptor/16019-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11503-5">
    <skos:altLabel xml:lang="de">Galoppierende Inflation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17931-3">
    <skos:related rdf:resource="descriptor/19179-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17931-3">
    <skos:related rdf:resource="descriptor/17990-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17931-3">
    <skos:related rdf:resource="descriptor/17989-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12624-2">
    <skos:broader rdf:resource="thsys/71050"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12624-2">
    <skos:broader rdf:resource="descriptor/11248-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18166-4">
    <skos:prefLabel xml:lang="en">Weekend working</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18166-4">
    <skos:prefLabel xml:lang="de">Wochenendarbeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10304-2">
    <skos:prefLabel xml:lang="en">Labour intensity</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10304-2">
    <skos:prefLabel xml:lang="de">Arbeitsintensität</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19534-3">
    <skos:broader rdf:resource="thsys/70582"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70854">
    <rdfs:label xml:lang="de">Genossenschaftswesen</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14607-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11927-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18207-2">
    <skos:altLabel xml:lang="de">Werkszeitschrift</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18207-2">
    <skos:altLabel xml:lang="de">Mitarbeiterzeitschrift</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18207-2">
    <skos:altLabel xml:lang="de">Betriebszeitung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15869-6">
    <skos:broader rdf:resource="thsys/73353"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18023-5">
    <skos:related rdf:resource="descriptor/18102-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11721-2">
    <skos:altLabel xml:lang="de">Staatswachstum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11721-2">
    <skos:altLabel xml:lang="de">Staatsausgabenquote</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11721-2">
    <skos:altLabel xml:lang="de">Staatsanteil</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11272-5">
    <skos:narrower rdf:resource="descriptor/19491-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11272-5">
    <skos:narrower rdf:resource="descriptor/19496-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11272-5">
    <skos:narrower rdf:resource="descriptor/11273-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11272-5">
    <skos:narrower rdf:resource="descriptor/19902-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19838-4">
    <skos:altLabel xml:lang="de">Stromeinspeisungsgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19838-4">
    <skos:altLabel xml:lang="de">Recht der Elektrizitätswirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11957-1">
    <skos:scopeNote xml:lang="de">Wissenschaft und Technik der Herstellung von Land- und Seekarten</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16068-0">
    <skos:prefLabel xml:lang="en">Check law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16068-0">
    <skos:prefLabel xml:lang="de">Scheckrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10586-1">
    <skos:prefLabel xml:lang="en">New international economic order</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10586-1">
    <skos:prefLabel xml:lang="de">Neue Weltwirtschaftsordnung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70993">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19137-6">
    <skos:prefLabel xml:lang="en">Corporate banking</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19137-6">
    <skos:prefLabel xml:lang="de">Firmenkundengeschäft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16502-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id362">
    <rdfs:seeAlso rdf:resource="descriptor/13067-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19091-4">
    <skos:broader rdf:resource="thsys/71043"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19091-4">
    <skos:broader rdf:resource="descriptor/12408-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16842-0">
    <skos:broader rdf:resource="thsys/70361"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16842-0">
    <skos:broader rdf:resource="descriptor/18012-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13852-5">
    <zbwext:useInsteadNote rdf:nodeID="id43"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10022-5">
    <skos:broader rdf:resource="thsys/71084"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10022-5">
    <skos:broader rdf:resource="thsys/71078"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10022-5">
    <skos:broader rdf:resource="thsys/71045"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17679-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15674-0">
    <skos:broader rdf:resource="thsys/70835"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15674-0">
    <skos:broader rdf:resource="thsys/70143"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15674-0">
    <skos:broader rdf:resource="descriptor/15666-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70135">
    <skos:broader rdf:resource="thsys/p"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19599-3">
    <skos:broader rdf:resource="thsys/70017"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20404-6">
    <skos:prefLabel xml:lang="en">Corporate documentation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20404-6">
    <skos:prefLabel xml:lang="de">Betriebliche Dokumentation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16971-3">
    <skos:prefLabel xml:lang="en">Schaffhausen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16971-3">
    <skos:prefLabel xml:lang="de">Schaffhausen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24781-5">
    <skos:broader rdf:resource="thsys/70292"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11240-4">
    <skos:related rdf:resource="descriptor/12502-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11240-4">
    <skos:related rdf:resource="descriptor/16146-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11240-4">
    <skos:related rdf:resource="descriptor/10261-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14914-4">
    <skos:related rdf:resource="descriptor/14923-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19063-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19069-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19106-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id366">
    <rdfs:seeAlso rdf:resource="descriptor/12094-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11152-1">
    <skos:prefLabel xml:lang="en">Exit and voice</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11152-1">
    <skos:prefLabel xml:lang="de">Abwanderung und Widerspruch</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16642-1">
    <skos:prefLabel xml:lang="en">Social values</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16642-1">
    <skos:prefLabel xml:lang="de">Soziale Werte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18077-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70109">
    <skos:broader rdf:resource="thsys/v"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10757-0">
    <skos:broader rdf:resource="thsys/71095"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10757-0">
    <skos:broader rdf:resource="descriptor/10764-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10882-4">
    <skos:altLabel xml:lang="de">Außenwirtschaftsvertretung des Staates</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10882-4">
    <skos:altLabel xml:lang="de">Außenwirtschaftsbehörde</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10882-4">
    <skos:altLabel xml:lang="de">Außenwirtschaftsamt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10882-4">
    <skos:altLabel xml:lang="de">Außenhandelsvertretung des Staates</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10882-4">
    <skos:altLabel xml:lang="de">Außenhandelsorganisation des Staates</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19222-1">
    <skos:related rdf:resource="descriptor/10217-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19222-1">
    <skos:related rdf:resource="descriptor/10213-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19222-1">
    <skos:related rdf:resource="descriptor/10246-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19820-2">
    <skos:broader rdf:resource="thsys/70154"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19820-2">
    <skos:broader rdf:resource="descriptor/19607-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19820-2">
    <skos:broader rdf:resource="descriptor/15966-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19853-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10969-6">
    <skos:prefLabel xml:lang="en">Economic policy coordination</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10969-6">
    <skos:prefLabel xml:lang="de">Internationale wirtschaftspolitische Koordination</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12665-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16948-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12191-0">
    <skos:broader rdf:resource="thsys/70087"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12191-0">
    <skos:broader rdf:resource="descriptor/12192-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73305">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17014-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10180-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18921-1">
    <skos:prefLabel xml:lang="en">Cultural economics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18921-1">
    <skos:prefLabel xml:lang="de">Kulturökonomik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70372">
    <skos:broader rdf:resource="thsys/p"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19344-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12480-0">
    <skos:broader rdf:resource="thsys/70578"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11436-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19488-5">
    <skos:prefLabel xml:lang="en">Common resources</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19488-5">
    <skos:prefLabel xml:lang="de">Allmenderessource</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70889">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.17</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10733-0">
    <skos:broader rdf:resource="thsys/71094"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10733-0">
    <skos:broader rdf:resource="thsys/71092"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10733-0">
    <skos:broader rdf:resource="descriptor/10764-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10733-0">
    <skos:broader rdf:resource="descriptor/10734-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12952-6">
    <skos:prefLabel xml:lang="en">Sheep farming</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12952-6">
    <skos:prefLabel xml:lang="de">Schafhaltung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73304">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14623-1">
    <skos:altLabel xml:lang="de">Riechstoff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14623-1">
    <skos:altLabel xml:lang="de">Resinoid</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14623-1">
    <skos:altLabel xml:lang="de">Etherisches Öl</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14623-1">
    <skos:altLabel xml:lang="de">Duftstoff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14623-1">
    <skos:altLabel xml:lang="de">Ätherisches Öl</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70036">
    <skos:broader rdf:resource="thsys/70037"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19085-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18985-3">
    <skos:scopeNote xml:lang="de">Trockenlegung von Feuchtgebieten zur Gewinnung von Nutzflächen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11983-0">
    <skos:scopeNote xml:lang="de">Wirtschaftlicher und sozialer Abstieg von Städten/Gemeinden oder Stadtteilen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73396">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19934-1">
    <skos:broader rdf:resource="thsys/71043"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19934-1">
    <skos:broader rdf:resource="descriptor/12408-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11944-3">
    <skos:broader rdf:resource="thsys/73362"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11944-3">
    <skos:broader rdf:resource="thsys/71069"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11944-3">
    <skos:broader rdf:resource="descriptor/19445-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19796-1">
    <skos:broader rdf:resource="thsys/73345"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73295">
    <rdfs:label xml:lang="en">Beverage industry</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73295">
    <rdfs:label xml:lang="de">Getränkeindustrie</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12492-0">
    <skos:prefLabel xml:lang="en">Overhead costs</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12492-0">
    <skos:prefLabel xml:lang="de">Gemeinkosten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71075">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14728-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11322-2">
    <skos:altLabel xml:lang="en">Wage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11322-2">
    <skos:altLabel xml:lang="de">Arbeitslohn</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11322-2">
    <skos:altLabel xml:lang="de">Arbeitsentgelt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11322-2">
    <skos:altLabel xml:lang="de">Arbeitseinkommen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18070-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19058-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19718-0">
    <skos:scopeNote xml:lang="de">Bestrebungen, die politische und wirtschaftliche Zusammenarbeit zwischen den afrikanischen Staaten zu intensivieren</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13807-3">
    <skos:related rdf:resource="descriptor/19127-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13807-3">
    <skos:related rdf:resource="descriptor/11415-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16656-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12747-0">
    <skos:altLabel xml:lang="de">Prozessplanung der Produktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12747-0">
    <skos:altLabel xml:lang="de">Prozessoptimierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12747-0">
    <skos:altLabel xml:lang="de">Prozeßplanung der Produktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12747-0">
    <skos:altLabel xml:lang="de">Prozeßoptimierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12747-0">
    <skos:altLabel xml:lang="de">Produktionsvollzugsplanung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12747-0">
    <skos:altLabel xml:lang="de">Produktionsprozessplanung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12747-0">
    <skos:altLabel xml:lang="de">Produktionsprozeßplanung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12747-0">
    <skos:altLabel xml:lang="de">Produktionsablaufplanung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12747-0">
    <skos:altLabel xml:lang="de">Maschinenbelegung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12747-0">
    <skos:altLabel xml:lang="de">Geschäftsprozessoptimierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12747-0">
    <skos:altLabel xml:lang="de">Geschäftsprozeßoptimierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12747-0">
    <skos:altLabel xml:lang="de">Fertigungsablaufplanung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12747-0">
    <skos:altLabel xml:lang="de">Durchlaufplanung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12747-0">
    <skos:altLabel xml:lang="de">Auftragssteuerung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12747-0">
    <skos:altLabel xml:lang="de">Auftragsreihenfolge</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12747-0">
    <skos:altLabel xml:lang="de">Auftragsplanung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14664-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18261-3">
    <skos:prefLabel xml:lang="en">Detergents industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18261-3">
    <skos:prefLabel xml:lang="de">Waschmittelindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18292-6">
    <skos:altLabel xml:lang="en">Training material</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18292-6">
    <skos:altLabel xml:lang="de">Unterrichtsmaterial</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18292-6">
    <skos:altLabel xml:lang="de">Lehrmittel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18292-6">
    <skos:altLabel xml:lang="de">Lehrbuch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12919-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16309-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15932-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19452-5">
    <skos:related rdf:resource="descriptor/13862-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19452-5">
    <skos:related rdf:resource="descriptor/19456-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19452-5">
    <skos:related rdf:resource="descriptor/19453-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11836-6">
    <skos:altLabel xml:lang="de">Vergleichsmiete</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11836-6">
    <skos:altLabel xml:lang="de">Mietspiegel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11836-6">
    <skos:altLabel xml:lang="de">Mietenspiegel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15775-1">
    <skos:related rdf:resource="descriptor/18559-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15775-1">
    <skos:related rdf:resource="descriptor/18364-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15775-1">
    <skos:related rdf:resource="descriptor/19483-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15775-1">
    <skos:related rdf:resource="descriptor/18307-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15775-1">
    <skos:related rdf:resource="descriptor/19765-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15775-1">
    <skos:related rdf:resource="descriptor/18137-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15775-1">
    <skos:related rdf:resource="descriptor/19160-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19000-5">
    <skos:prefLabel xml:lang="en">WEU countries</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19000-5">
    <skos:prefLabel xml:lang="de">WEU-Staaten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10034-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70438">
    <skos:prefLabel xml:lang="en">G.01.01.02  Austria</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70438">
    <skos:prefLabel xml:lang="de">G.01.01.02  Österreich</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71001">
    <skos:broader rdf:resource="thsys/71003"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14341-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16417-1">
    <skos:related rdf:resource="descriptor/16003-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16417-1">
    <skos:related rdf:resource="descriptor/16497-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16417-1">
    <skos:related rdf:resource="descriptor/16489-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19637-0">
    <skos:prefLabel xml:lang="en">Pensioners</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19637-0">
    <skos:prefLabel xml:lang="de">Rentner</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18099-0">
    <skos:prefLabel xml:lang="en">Sponsorship</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18099-0">
    <skos:prefLabel xml:lang="de">Sponsoring</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10276-2">
    <skos:prefLabel xml:lang="en">Aggregate consumption</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10276-2">
    <skos:prefLabel xml:lang="de">Gesamtwirtschaftlicher Konsum</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18825-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17800-4">
    <skos:altLabel xml:lang="en">Saint Lucia</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11678-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19854-6">
    <skos:prefLabel xml:lang="en">Travel restrictions</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19854-6">
    <skos:prefLabel xml:lang="de">Reisebestimmungen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11353-5">
    <zbwext:useInsteadNote rdf:nodeID="id56"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19096-1">
    <skos:related rdf:resource="descriptor/10463-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19096-1">
    <skos:related rdf:resource="descriptor/19152-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19096-1">
    <skos:related rdf:resource="descriptor/12480-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19096-1">
    <skos:related rdf:resource="descriptor/10011-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17104-3">
    <skos:broader rdf:resource="thsys/70131"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17104-3">
    <skos:broader rdf:resource="descriptor/17016-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17104-3">
    <skos:broader rdf:resource="descriptor/17991-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17104-3">
    <skos:broader rdf:resource="descriptor/17983-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13780-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13998-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70176">
    <skos:broader rdf:resource="thsys/70056"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10587-6">
    <skos:prefLabel xml:lang="en">North South relations</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10587-6">
    <skos:prefLabel xml:lang="de">Nord-Süd-Beziehungen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18655-3">
    <skos:related rdf:resource="descriptor/18000-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18655-3">
    <skos:related rdf:resource="descriptor/18941-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18655-3">
    <skos:related rdf:resource="descriptor/24664-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18450-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16306-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18816-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12669-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16288-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11318-0">
    <skos:prefLabel xml:lang="en">Public sector pay</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11318-0">
    <skos:prefLabel xml:lang="de">Besoldung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10313-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12075-2">
    <skos:related rdf:resource="descriptor/12108-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12075-2">
    <skos:related rdf:resource="descriptor/12806-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12075-2">
    <skos:related rdf:resource="descriptor/19758-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12075-2">
    <skos:related rdf:resource="descriptor/12775-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12075-2">
    <skos:related rdf:resource="descriptor/15821-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12075-2">
    <skos:related rdf:resource="descriptor/20536-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12075-2">
    <skos:related rdf:resource="descriptor/19835-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14249-6">
    <skos:altLabel xml:lang="de">Transportbeton</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14249-6">
    <skos:altLabel xml:lang="de">Gasbeton</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14249-6">
    <skos:altLabel xml:lang="de">Fertigbeton</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14249-6">
    <skos:altLabel xml:lang="de">Betonröhre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14249-6">
    <skos:altLabel xml:lang="de">Betonprodukt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13527-2">
    <skos:broader rdf:resource="thsys/70941"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13527-2">
    <skos:broader rdf:resource="descriptor/13514-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10753-1">
    <skos:altLabel xml:lang="de">Britisches Pfund</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18965-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11264-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10704-0">
    <skos:altLabel xml:lang="en">Most-favoured-nation clause</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10704-0">
    <skos:altLabel xml:lang="de">Meistbegünstigungsklausel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19273-5">
    <skos:related rdf:resource="descriptor/19272-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19273-5">
    <skos:related rdf:resource="descriptor/13702-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19273-5">
    <skos:related rdf:resource="descriptor/19460-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19273-5">
    <skos:related rdf:resource="descriptor/18323-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19273-5">
    <skos:related rdf:resource="descriptor/18673-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19273-5">
    <skos:related rdf:resource="descriptor/13430-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19797-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19841-1">
    <skos:scopeNote xml:lang="de">Umfasst jene staatlichen Aktivitäten, die der Rechtspflege in Zivil- und Strafsachen dienen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id224">
    <rdf:value xml:lang="de">Für gesamtwirtschaftliche Nachfrage</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70506">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18409-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11909-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14044-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16035-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19178-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12529-3">
    <skos:prefLabel xml:lang="en">Gross margin</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12529-3">
    <skos:prefLabel xml:lang="de">Handelsspanne</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10951-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20744-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12868-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12047-0">
    <zbwext:useInsteadNote rdf:nodeID="id237"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15786-3">
    <skos:broader rdf:resource="thsys/70847"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15786-3">
    <skos:broader rdf:resource="thsys/70292"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15786-3">
    <skos:broader rdf:resource="thsys/70143"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15786-3">
    <skos:broader rdf:resource="thsys/70017"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15786-3">
    <skos:broader rdf:resource="descriptor/15790-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10156-5">
    <skos:broader rdf:resource="thsys/71118"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10156-5">
    <skos:broader rdf:resource="thsys/71116"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10156-5">
    <skos:broader rdf:resource="descriptor/16739-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14940-3">
    <skos:related rdf:resource="descriptor/18440-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19840-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11537-2">
    <skos:narrower rdf:resource="descriptor/11610-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11537-2">
    <skos:narrower rdf:resource="descriptor/11591-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11537-2">
    <skos:narrower rdf:resource="descriptor/11544-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11537-2">
    <skos:narrower rdf:resource="descriptor/11553-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19010-2">
    <skos:altLabel xml:lang="de">Wissenschaftliche Arbeitstechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19010-2">
    <skos:altLabel xml:lang="de">Technik der wissenschaftlichen Arbeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12224-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18392-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15916-2">
    <skos:narrower rdf:resource="descriptor/19645-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15916-2">
    <skos:narrower rdf:resource="descriptor/16749-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15916-2">
    <skos:narrower rdf:resource="descriptor/15912-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15916-2">
    <skos:narrower rdf:resource="descriptor/19641-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15920-4">
    <skos:broader rdf:resource="thsys/73358"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15920-4">
    <skos:broader rdf:resource="thsys/70142"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15920-4">
    <skos:broader rdf:resource="descriptor/18861-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10315-4">
    <skos:related rdf:resource="descriptor/10334-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10315-4">
    <skos:related rdf:resource="descriptor/19490-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10315-4">
    <skos:related rdf:resource="descriptor/18129-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10315-4">
    <skos:related rdf:resource="descriptor/10326-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10315-4">
    <skos:related rdf:resource="descriptor/10186-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10315-4">
    <skos:related rdf:resource="descriptor/10313-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11194-6">
    <skos:narrower rdf:resource="descriptor/11215-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18594-4">
    <skos:related rdf:resource="descriptor/16070-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18594-4">
    <skos:related rdf:resource="descriptor/13298-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18594-4">
    <skos:related rdf:resource="descriptor/18593-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18594-4">
    <skos:related rdf:resource="descriptor/14455-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18594-4">
    <skos:related rdf:resource="descriptor/15744-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12915-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18676-2">
    <skos:altLabel xml:lang="de">Börsenusancen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18676-2">
    <skos:altLabel xml:lang="de">Börsenregeln</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18676-2">
    <skos:altLabel xml:lang="de">Börsenorganisationsrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18676-2">
    <skos:altLabel xml:lang="de">Börsenordnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18676-2">
    <skos:altLabel xml:lang="de">Börsengesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18676-2">
    <skos:altLabel xml:lang="de">Börsenaufsichtsrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10557-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14665-6">
    <skos:prefLabel xml:lang="en">Alcohol</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14665-6">
    <skos:prefLabel xml:lang="de">Alkohol</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71092">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.07.04.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10500-2">
    <skos:prefLabel xml:lang="en">Industrialization</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10500-2">
    <skos:prefLabel xml:lang="de">Industrialisierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19161-2">
    <skos:broader rdf:resource="thsys/70140"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13289-6">
    <skos:prefLabel xml:lang="en">Landesbank</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13289-6">
    <skos:prefLabel xml:lang="de">Landesbank</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70993">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.06.01.06.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11498-0">
    <skos:altLabel xml:lang="de">Fisher-Relation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11498-0">
    <skos:altLabel xml:lang="de">Fisher-Hypothese</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18811-1">
    <skos:altLabel xml:lang="de">Lebensmittelpreispolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15879-3">
    <skos:prefLabel xml:lang="en">Island</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15879-3">
    <skos:prefLabel xml:lang="de">Insel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13260-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15742-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19145-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10667-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16485-5">
    <skos:related rdf:resource="descriptor/16203-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16485-5">
    <skos:related rdf:resource="descriptor/13825-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16485-5">
    <skos:related rdf:resource="descriptor/13818-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16485-5">
    <skos:related rdf:resource="descriptor/13401-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16485-5">
    <skos:related rdf:resource="descriptor/16036-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16485-5">
    <skos:related rdf:resource="descriptor/18908-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16485-5">
    <skos:related rdf:resource="descriptor/18637-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16485-5">
    <skos:related rdf:resource="descriptor/16034-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16485-5">
    <skos:related rdf:resource="descriptor/16004-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16485-5">
    <skos:related rdf:resource="descriptor/19316-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17675-2">
    <skos:broader rdf:resource="thsys/70445"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17675-2">
    <skos:broader rdf:resource="descriptor/17672-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18623-2">
    <skos:scopeNote xml:lang="de">Analyse umweltbezogener Wirkungen des Verkehrs.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15062-3">
    <skos:broader rdf:resource="thsys/72182"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15062-3">
    <skos:broader rdf:resource="descriptor/15064-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16455-0">
    <skos:scopeNote xml:lang="de">Auch für multinationale Polizeikräfte in internationalen Friedenseinsätzen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14554-1">
    <skos:broader rdf:resource="thsys/70477"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14554-1">
    <skos:broader rdf:resource="thsys/70346"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13861-4">
    <skos:altLabel xml:lang="de">Wohnzufriedenheit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13861-4">
    <skos:altLabel xml:lang="de">Wohnverhalten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13861-4">
    <skos:altLabel xml:lang="de">Wohnform</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18287-6">
    <skos:broader rdf:resource="thsys/70346"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18287-6">
    <skos:broader rdf:resource="descriptor/14269-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18287-6">
    <skos:broader rdf:resource="descriptor/12675-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16611-5">
    <skos:altLabel xml:lang="de">Mittelschicht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16611-5">
    <skos:altLabel xml:lang="de">Mittelklasse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18908-0">
    <skos:broader rdf:resource="thsys/70834"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18908-0">
    <skos:broader rdf:resource="descriptor/16543-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13072-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13450-4">
    <skos:prefLabel xml:lang="en">Agricultural engineering</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13450-4">
    <skos:prefLabel xml:lang="de">Agrartechnik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18947-4">
    <skos:broader rdf:resource="thsys/71055"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18947-4">
    <skos:broader rdf:resource="descriptor/13981-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12800-1">
    <skos:prefLabel xml:lang="en">Brand</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12800-1">
    <skos:prefLabel xml:lang="de">Markenartikel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14157-4">
    <skos:prefLabel xml:lang="en">Tropical timber</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14157-4">
    <skos:prefLabel xml:lang="de">Tropenholz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18871-4">
    <skos:narrower rdf:resource="descriptor/18872-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70979">
    <skos:broader rdf:resource="thsys/71005"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16638-6">
    <skos:prefLabel xml:lang="en">Quality of life</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16638-6">
    <skos:prefLabel xml:lang="de">Lebensqualität</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12837-6">
    <skos:broader rdf:resource="thsys/73384"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12837-6">
    <skos:broader rdf:resource="thsys/70384"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12837-6">
    <skos:broader rdf:resource="thsys/70292"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id222">
    <rdf:value xml:lang="de">Für Mitarbeiter- und Leistungsmotivation</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19203-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14723-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17627-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11444-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10718-3">
    <skos:prefLabel xml:lang="en">Currency appreciation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10718-3">
    <skos:prefLabel xml:lang="de">Aufwertung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11503-5">
    <skos:prefLabel xml:lang="en">Hyperinflation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11503-5">
    <skos:prefLabel xml:lang="de">Hyperinflation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21061-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13302-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19577-6">
    <skos:prefLabel xml:lang="en">State space model</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19577-6">
    <skos:prefLabel xml:lang="de">Zustandsraummodell</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="../stw">
    <owl:versionInfo>8.04</owl:versionInfo>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18297-3">
    <skos:prefLabel xml:lang="en">Aircraft equipment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18297-3">
    <skos:prefLabel xml:lang="de">Flugzeugausrüstung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16760-2">
    <skos:related rdf:resource="descriptor/16761-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16760-2">
    <skos:related rdf:resource="descriptor/19475-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16760-2">
    <skos:related rdf:resource="descriptor/16757-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16760-2">
    <skos:related rdf:resource="descriptor/14001-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16760-2">
    <skos:related rdf:resource="descriptor/16754-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11024-3">
    <skos:prefLabel xml:lang="en">Socialist pricing mechanism</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11024-3">
    <skos:prefLabel xml:lang="de">Sozialistische Preisbildung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20442-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18443-4">
    <skos:narrower rdf:resource="descriptor/18445-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16463-1">
    <skos:related rdf:resource="descriptor/18817-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16463-1">
    <skos:related rdf:resource="descriptor/16288-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16463-1">
    <skos:related rdf:resource="descriptor/16521-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16463-1">
    <skos:related rdf:resource="descriptor/16455-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16463-1">
    <skos:related rdf:resource="descriptor/16454-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16463-1">
    <skos:related rdf:resource="descriptor/16435-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16463-1">
    <skos:related rdf:resource="descriptor/16437-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16463-1">
    <skos:related rdf:resource="descriptor/16489-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18480-5">
    <skos:broader rdf:resource="thsys/71005"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18480-5">
    <skos:broader rdf:resource="thsys/70058"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18480-5">
    <skos:broader rdf:resource="descriptor/11910-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18480-5">
    <skos:broader rdf:resource="descriptor/11821-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15998-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11342-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13215-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18292-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11569-3">
    <skos:related rdf:resource="descriptor/16158-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11569-3">
    <skos:related rdf:resource="descriptor/19507-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11569-3">
    <skos:related rdf:resource="descriptor/19852-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11569-3">
    <skos:related rdf:resource="descriptor/11564-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10147-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12276-6">
    <skos:broader rdf:resource="thsys/70036"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71084">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15149-5">
    <skos:narrower rdf:resource="descriptor/15082-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16525-5">
    <skos:prefLabel xml:lang="en">Care for the elderly</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16525-5">
    <skos:prefLabel xml:lang="de">Altenhilfe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13515-2">
    <skos:scopeNote xml:lang="de">Differenzierung in Land-, Wasser- und Luftverkehr.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12953-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15958-0">
    <skos:altLabel xml:lang="de">Bevölkerungsaustausch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14839-6">
    <skos:broader rdf:resource="thsys/70031"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14839-6">
    <skos:broader rdf:resource="descriptor/14846-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18027-4">
    <skos:broader rdf:resource="thsys/70824"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16110-6">
    <skos:broader rdf:resource="thsys/73334"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16110-6">
    <skos:broader rdf:resource="thsys/71073"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16110-6">
    <skos:broader rdf:resource="thsys/70869"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16110-6">
    <skos:broader rdf:resource="descriptor/11536-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11611-2">
    <skos:altLabel xml:lang="en">Sovereign bonds</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11611-2">
    <skos:altLabel xml:lang="en">Government bond</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11611-2">
    <skos:altLabel xml:lang="de">Staatspapier</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11611-2">
    <skos:altLabel xml:lang="de">Staatsanleihe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11611-2">
    <skos:altLabel xml:lang="de">Länderanleihe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11611-2">
    <skos:altLabel xml:lang="de">Konsols</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11611-2">
    <skos:altLabel xml:lang="de">Konsolidierte Staatspapiere</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11611-2">
    <skos:altLabel xml:lang="de">Bundesanleihe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11611-2">
    <skos:altLabel xml:lang="de">Anleihekonversion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11611-2">
    <skos:altLabel xml:lang="de">Anleihe der öffentlichen Hand</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17932-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12865-1">
    <skos:related rdf:resource="descriptor/12910-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12865-1">
    <skos:related rdf:resource="descriptor/15938-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16502-3">
    <skos:broader rdf:resource="thsys/73366"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16502-3">
    <skos:broader rdf:resource="descriptor/16273-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18083-1">
    <skos:altLabel xml:lang="en">Self-employment</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18083-1">
    <skos:altLabel xml:lang="de">Selbstständigkeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18083-1">
    <skos:altLabel xml:lang="de">Selbständigkeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18083-1">
    <skos:altLabel xml:lang="de">Selbständige</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14130-3">
    <skos:altLabel xml:lang="de">Schafwolle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16712-6">
    <skos:prefLabel xml:lang="en">Historical school of economics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16712-6">
    <skos:prefLabel xml:lang="de">Historische Schule</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10426-2">
    <skos:prefLabel xml:lang="en">Regional growth theory</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10426-2">
    <skos:prefLabel xml:lang="de">Regionale Wachstumstheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19111-3">
    <skos:broader rdf:resource="thsys/71050"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10250-6">
    <skos:prefLabel xml:lang="en">Monetarism</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10250-6">
    <skos:prefLabel xml:lang="de">Monetarismus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10269-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15988-5">
    <skos:broader rdf:resource="thsys/73360"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15988-5">
    <skos:broader rdf:resource="thsys/70818"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15988-5">
    <skos:broader rdf:resource="descriptor/16019-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14110-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73368">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.04.03.06</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71113">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.03.04.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12262-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16868-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16292-2">
    <skos:altLabel xml:lang="de">Politisches Regime</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16292-2">
    <skos:altLabel xml:lang="de">Politische Ordnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16292-2">
    <skos:altLabel xml:lang="de">Politische Herrschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16292-2">
    <skos:altLabel xml:lang="de">Herrschaftsform</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12782-5">
    <skos:altLabel xml:lang="de">Franchisesystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id31">
    <rdfs:seeAlso rdf:resource="descriptor/10605-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19819-1">
    <skos:broader rdf:resource="thsys/70154"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19819-1">
    <skos:broader rdf:resource="descriptor/15966-1"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id279">
    <rdfs:seeAlso rdf:resource="descriptor/11182-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18943-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16032-0">
    <skos:narrower rdf:resource="descriptor/16059-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16032-0">
    <skos:narrower rdf:resource="descriptor/16185-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16032-0">
    <skos:narrower rdf:resource="descriptor/19329-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16032-0">
    <skos:narrower rdf:resource="descriptor/18790-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13063-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10951-4">
    <skos:broader rdf:resource="thsys/70242"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10951-4">
    <skos:broader rdf:resource="descriptor/19565-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10951-4">
    <skos:broader rdf:resource="descriptor/10895-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17179-0">
    <skos:broader rdf:resource="thsys/70130"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17179-0">
    <skos:broader rdf:resource="descriptor/16815-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19087-2">
    <skos:broader rdf:resource="thsys/70130"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19087-2">
    <skos:broader rdf:resource="descriptor/17179-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19823-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12041-5">
    <skos:narrower rdf:resource="descriptor/12172-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12041-5">
    <skos:narrower rdf:resource="descriptor/13819-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12041-5">
    <skos:narrower rdf:resource="descriptor/13608-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12041-5">
    <skos:narrower rdf:resource="descriptor/18823-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12041-5">
    <skos:narrower rdf:resource="descriptor/18815-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12041-5">
    <skos:narrower rdf:resource="descriptor/19121-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12041-5">
    <skos:narrower rdf:resource="descriptor/19191-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12041-5">
    <skos:narrower rdf:resource="descriptor/12742-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12041-5">
    <skos:narrower rdf:resource="descriptor/12051-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12041-5">
    <skos:narrower rdf:resource="descriptor/13459-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12041-5">
    <skos:narrower rdf:resource="descriptor/18988-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12041-5">
    <skos:narrower rdf:resource="descriptor/12206-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12041-5">
    <skos:narrower rdf:resource="descriptor/12458-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12041-5">
    <skos:narrower rdf:resource="descriptor/12848-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12041-5">
    <skos:narrower rdf:resource="descriptor/12744-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12041-5">
    <skos:narrower rdf:resource="descriptor/12209-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12041-5">
    <skos:narrower rdf:resource="descriptor/18383-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12041-5">
    <skos:narrower rdf:resource="descriptor/18504-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12041-5">
    <skos:narrower rdf:resource="descriptor/13686-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16110-6">
    <skos:related rdf:resource="descriptor/16112-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10194-4">
    <skos:related rdf:resource="descriptor/20176-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10194-4">
    <skos:related rdf:resource="descriptor/10222-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10194-4">
    <skos:related rdf:resource="descriptor/10207-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10194-4">
    <skos:related rdf:resource="descriptor/11153-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10194-4">
    <skos:related rdf:resource="descriptor/19227-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15640-3">
    <skos:prefLabel xml:lang="en">Christianity</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15640-3">
    <skos:prefLabel xml:lang="de">Christentum</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24665-0">
    <skos:broader rdf:resource="thsys/70903"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24665-0">
    <skos:broader rdf:resource="descriptor/13283-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16587-4">
    <skos:prefLabel xml:lang="en">Community</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16587-4">
    <skos:prefLabel xml:lang="de">Gemeinschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17636-5">
    <skos:broader rdf:resource="thsys/70253"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17636-5">
    <skos:broader rdf:resource="descriptor/17608-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17636-5">
    <skos:broader rdf:resource="descriptor/17978-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12042-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11354-3">
    <skos:prefLabel xml:lang="en">Educational level</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11354-3">
    <skos:prefLabel xml:lang="de">Bildungsniveau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19835-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13671-2">
    <skos:altLabel xml:lang="en">Office system</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13671-2">
    <skos:altLabel xml:lang="en">Office information system</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13671-2">
    <skos:altLabel xml:lang="en">Office communication system</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13671-2">
    <skos:altLabel xml:lang="de">Computergestütztes Büroinformationssystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13671-2">
    <skos:altLabel xml:lang="de">Bürotik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13671-2">
    <skos:altLabel xml:lang="de">Bürotechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13671-2">
    <skos:altLabel xml:lang="de">Büroautomatisierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13671-2">
    <skos:altLabel xml:lang="de">Büro der Zukunft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15135-2">
    <skos:broader rdf:resource="thsys/70182"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15135-2">
    <skos:broader rdf:resource="descriptor/15115-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15135-2">
    <skos:broader rdf:resource="descriptor/15061-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19486-2">
    <skos:broader rdf:resource="thsys/73352"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19486-2">
    <skos:broader rdf:resource="thsys/70463"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19486-2">
    <skos:broader rdf:resource="thsys/70238"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19486-2">
    <skos:broader rdf:resource="descriptor/18254-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19486-2">
    <skos:broader rdf:resource="descriptor/15743-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#Collection">
    <rdfs:label xml:lang="en">Collection</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16173-3">
    <skos:prefLabel xml:lang="en">Space law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16173-3">
    <skos:prefLabel xml:lang="de">Weltraumrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19081-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13409-1">
    <skos:broader rdf:resource="thsys/73367"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13409-1">
    <skos:broader rdf:resource="thsys/70870"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13409-1">
    <skos:broader rdf:resource="descriptor/11857-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10682-5">
    <skos:prefLabel xml:lang="en">Foreign economic policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10682-5">
    <skos:prefLabel xml:lang="de">Außenwirtschaftspolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10815-5">
    <skos:prefLabel xml:lang="en">Capital imports</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10815-5">
    <skos:prefLabel xml:lang="de">Kapitalimport</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11600-0">
    <skos:broader rdf:resource="thsys/71079"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11600-0">
    <skos:broader rdf:resource="descriptor/16314-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10678-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10864-6">
    <skos:prefLabel xml:lang="en">Export intermediaries</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10864-6">
    <skos:prefLabel xml:lang="de">Exporthandel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14182-5">
    <skos:related rdf:resource="descriptor/15013-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14182-5">
    <skos:related rdf:resource="descriptor/14035-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14182-5">
    <skos:related rdf:resource="descriptor/15628-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14182-5">
    <skos:related rdf:resource="descriptor/16001-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14182-5">
    <skos:related rdf:resource="descriptor/18367-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14182-5">
    <skos:related rdf:resource="descriptor/14181-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14182-5">
    <skos:related rdf:resource="descriptor/15001-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17722-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14752-4">
    <skos:broader rdf:resource="thsys/73305"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14752-4">
    <skos:broader rdf:resource="thsys/71038"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19205-1">
    <skos:broader rdf:resource="thsys/70141"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19205-1">
    <skos:broader rdf:resource="descriptor/10032-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12702-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10324-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24784-6">
    <skos:prefLabel xml:lang="en">Open source software</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24784-6">
    <skos:prefLabel xml:lang="de">Open Source</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18160-2">
    <skos:prefLabel xml:lang="en">Territorial competition</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18160-2">
    <skos:prefLabel xml:lang="de">Standortwettbewerb</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14193-0">
    <skos:altLabel xml:lang="de">Schieferöl</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14193-0">
    <skos:altLabel xml:lang="de">Ölsand</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14193-0">
    <skos:altLabel xml:lang="de">Ölkreide</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13109-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19200-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11043-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10293-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10697-6">
    <skos:related rdf:resource="descriptor/10710-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10697-6">
    <skos:related rdf:resource="descriptor/19330-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10697-6">
    <skos:related rdf:resource="descriptor/18330-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20468-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13665-4">
    <skos:prefLabel xml:lang="en">Radio</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13665-4">
    <skos:prefLabel xml:lang="de">Rundfunk</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13003-1">
    <skos:related rdf:resource="descriptor/14035-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13003-1">
    <skos:related rdf:resource="descriptor/15628-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13003-1">
    <skos:related rdf:resource="descriptor/16001-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13003-1">
    <skos:related rdf:resource="descriptor/18367-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13003-1">
    <skos:related rdf:resource="descriptor/14181-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12250-3">
    <skos:broader rdf:resource="thsys/71045"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12250-3">
    <skos:broader rdf:resource="thsys/70259"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12250-3">
    <skos:broader rdf:resource="descriptor/12251-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11638-3">
    <skos:related rdf:resource="descriptor/24859-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11638-3">
    <skos:related rdf:resource="descriptor/11709-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11638-3">
    <skos:related rdf:resource="descriptor/16112-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11638-3">
    <skos:related rdf:resource="descriptor/11691-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11638-3">
    <skos:related rdf:resource="descriptor/10830-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11638-3">
    <skos:related rdf:resource="descriptor/19662-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13037-5">
    <skos:prefLabel xml:lang="en">Nonferrous metals industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13037-5">
    <skos:prefLabel xml:lang="de">NE-Metallindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13573-2">
    <skos:broader rdf:resource="thsys/70929"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13573-2">
    <skos:broader rdf:resource="descriptor/15685-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11802-2">
    <skos:narrower rdf:resource="descriptor/18331-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71045">
    <skos:narrower rdf:resource="descriptor/13745-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71045">
    <skos:narrower rdf:resource="descriptor/12263-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71045">
    <skos:narrower rdf:resource="descriptor/10022-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71045">
    <skos:narrower rdf:resource="descriptor/11572-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71045">
    <skos:narrower rdf:resource="descriptor/18097-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71045">
    <skos:narrower rdf:resource="descriptor/10019-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71045">
    <skos:narrower rdf:resource="descriptor/12257-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71045">
    <skos:narrower rdf:resource="descriptor/12256-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71045">
    <skos:narrower rdf:resource="descriptor/12255-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71045">
    <skos:narrower rdf:resource="descriptor/12254-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71045">
    <skos:narrower rdf:resource="descriptor/12253-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71045">
    <skos:narrower rdf:resource="descriptor/13293-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71045">
    <skos:narrower rdf:resource="descriptor/18178-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71045">
    <skos:narrower rdf:resource="descriptor/12252-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71045">
    <skos:narrower rdf:resource="descriptor/12251-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71045">
    <skos:narrower rdf:resource="descriptor/12250-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71045">
    <skos:narrower rdf:resource="descriptor/10804-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71045">
    <skos:narrower rdf:resource="descriptor/12247-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71045">
    <skos:narrower rdf:resource="descriptor/13775-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71045">
    <skos:narrower rdf:resource="descriptor/12277-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71045">
    <skos:narrower rdf:resource="descriptor/12274-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71045">
    <skos:narrower rdf:resource="descriptor/10870-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71045">
    <skos:narrower rdf:resource="descriptor/12243-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71045">
    <skos:narrower rdf:resource="descriptor/12241-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71045">
    <skos:narrower rdf:resource="descriptor/12246-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71045">
    <skos:narrower rdf:resource="descriptor/12236-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71045">
    <skos:narrower rdf:resource="descriptor/18685-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71045">
    <skos:narrower rdf:resource="descriptor/12234-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71045">
    <skos:narrower rdf:resource="descriptor/10850-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16650-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18069-2">
    <skos:scopeNote xml:lang="de">U.a. Veränderung des Klimas durch den Menschen und seine Aktivitäten.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11731-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12677-2">
    <skos:prefLabel xml:lang="en">Inventory management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12677-2">
    <skos:prefLabel xml:lang="de">Betriebliche Lagerhaltung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11721-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14619-6">
    <skos:related rdf:resource="descriptor/14614-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14619-6">
    <skos:related rdf:resource="descriptor/14539-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14619-6">
    <skos:related rdf:resource="descriptor/13229-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14619-6">
    <skos:related rdf:resource="descriptor/14046-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13935-1">
    <skos:related rdf:resource="descriptor/13321-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13935-1">
    <skos:related rdf:resource="descriptor/13141-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13935-1">
    <skos:related rdf:resource="descriptor/18094-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13935-1">
    <skos:related rdf:resource="descriptor/13904-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13935-1">
    <skos:related rdf:resource="descriptor/18788-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71062">
    <skos:narrower rdf:resource="descriptor/19495-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71062">
    <skos:narrower rdf:resource="descriptor/18150-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71062">
    <skos:narrower rdf:resource="descriptor/11337-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71062">
    <skos:narrower rdf:resource="descriptor/11336-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71062">
    <skos:narrower rdf:resource="descriptor/11335-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71062">
    <skos:narrower rdf:resource="descriptor/18848-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71062">
    <skos:narrower rdf:resource="descriptor/11159-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71062">
    <skos:narrower rdf:resource="descriptor/11224-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71062">
    <skos:narrower rdf:resource="descriptor/18173-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71062">
    <skos:narrower rdf:resource="descriptor/11230-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71062">
    <skos:narrower rdf:resource="descriptor/10143-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71062">
    <skos:narrower rdf:resource="descriptor/11220-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71062">
    <skos:narrower rdf:resource="descriptor/11219-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71062">
    <skos:narrower rdf:resource="descriptor/11211-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71062">
    <skos:narrower rdf:resource="descriptor/24623-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10769-0">
    <skos:broader rdf:resource="thsys/71094"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10769-0">
    <skos:broader rdf:resource="descriptor/10746-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11478-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12585-0">
    <skos:related rdf:resource="descriptor/19001-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12585-0">
    <skos:related rdf:resource="descriptor/15839-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12585-0">
    <skos:related rdf:resource="descriptor/15825-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12585-0">
    <skos:related rdf:resource="descriptor/15823-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12585-0">
    <skos:related rdf:resource="descriptor/21306-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12585-0">
    <skos:related rdf:resource="descriptor/12577-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12585-0">
    <skos:related rdf:resource="descriptor/15837-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12585-0">
    <skos:related rdf:resource="descriptor/12617-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16066-4">
    <skos:altLabel xml:lang="de">Lomé-Abkommen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#mappingRelation">
    <rdfs:comment xml:lang="en">These concept mapping relations mirror semantic relations, and the
      data model defined below is similar (with the exception of skos:exactMatch) to the data model
      defined for semantic relations. A distinct vocabulary is provided for concept mapping
      relations, to provide a convenient way to differentiate links within a concept scheme from
      links between concept schemes. However, this pattern of usage is not a formal requirement of
      the SKOS data model, and relies on informal definitions of best practice.</rdfs:comment>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17784-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16226-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15888-2">
    <skos:narrower rdf:resource="descriptor/15881-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16427-5">
    <skos:broader rdf:resource="thsys/73377"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16427-5">
    <skos:broader rdf:resource="thsys/73364"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10962-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19692-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14442-5">
    <skos:broader rdf:resource="thsys/70258"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14442-5">
    <skos:broader rdf:resource="descriptor/14433-6"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id100">
    <rdfs:seeAlso rdf:resource="descriptor/18729-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10060-4">
    <skos:broader rdf:resource="thsys/70049"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10060-4">
    <skos:broader rdf:resource="descriptor/10201-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10060-4">
    <skos:broader rdf:resource="descriptor/10011-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71082">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10619-5">
    <skos:prefLabel xml:lang="en">International economic policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10619-5">
    <skos:prefLabel xml:lang="de">Weltwirtschaftspolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18234-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16036-6">
    <skos:broader rdf:resource="thsys/73392"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16036-6">
    <skos:broader rdf:resource="descriptor/16026-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12170-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14662-5">
    <skos:prefLabel xml:lang="en">Alkenes</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14662-5">
    <skos:prefLabel xml:lang="de">Alkene</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18652-2">
    <skos:prefLabel xml:lang="en">Telecommunication tariff</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18652-2">
    <skos:prefLabel xml:lang="de">Telekommunikationsgebühr</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18375-2">
    <skos:prefLabel xml:lang="en">Mining policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18375-2">
    <skos:prefLabel xml:lang="de">Bergbaupolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71032">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19585-0">
    <skos:prefLabel xml:lang="en">Forestry statistics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19585-0">
    <skos:prefLabel xml:lang="de">Forststatistik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16479-0">
    <skos:altLabel xml:lang="de">Öffentliche Gesundheitspflege</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19479-6">
    <skos:altLabel xml:lang="en">Urban green</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19479-6">
    <skos:altLabel xml:lang="de">Ziergarten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19479-6">
    <skos:altLabel xml:lang="de">Volkspark</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19479-6">
    <skos:altLabel xml:lang="de">Stadtwald</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19479-6">
    <skos:altLabel xml:lang="de">Stadtpark</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19479-6">
    <skos:altLabel xml:lang="de">Städtische Grünfläche</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19479-6">
    <skos:altLabel xml:lang="de">Stadtforst</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19479-6">
    <skos:altLabel xml:lang="de">Spielplatz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19479-6">
    <skos:altLabel xml:lang="de">Schrebergarten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19479-6">
    <skos:altLabel xml:lang="de">Park</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19479-6">
    <skos:altLabel xml:lang="de">Kleingarten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19479-6">
    <skos:altLabel xml:lang="de">Kinderspielplatz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19479-6">
    <skos:altLabel xml:lang="de">Hausgarten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19479-6">
    <skos:altLabel xml:lang="de">Grüngürtel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19479-6">
    <skos:altLabel xml:lang="de">Garten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19479-6">
    <skos:altLabel xml:lang="de">Garden</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13148-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18811-1">
    <skos:prefLabel xml:lang="en">Food price policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18811-1">
    <skos:prefLabel xml:lang="de">Nahrungsmittelpreispolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19364-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20977-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11157-5">
    <skos:broader rdf:resource="thsys/70839"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11157-5">
    <skos:broader rdf:resource="thsys/70573"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11157-5">
    <skos:broader rdf:resource="descriptor/10032-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10381-5">
    <skos:related rdf:resource="descriptor/18156-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10381-5">
    <skos:related rdf:resource="descriptor/10387-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10381-5">
    <skos:related rdf:resource="descriptor/10247-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10381-5">
    <skos:related rdf:resource="descriptor/11494-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10381-5">
    <skos:related rdf:resource="descriptor/10383-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10381-5">
    <skos:related rdf:resource="descriptor/10237-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18719-3">
    <skos:broader rdf:resource="thsys/70900"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18719-3">
    <skos:broader rdf:resource="descriptor/12357-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12783-3">
    <skos:related rdf:resource="descriptor/18564-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12783-3">
    <skos:related rdf:resource="descriptor/18797-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12783-3">
    <skos:related rdf:resource="descriptor/18799-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12783-3">
    <skos:related rdf:resource="descriptor/18795-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12783-3">
    <skos:related rdf:resource="descriptor/18499-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12783-3">
    <skos:related rdf:resource="descriptor/19151-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12783-3">
    <skos:related rdf:resource="descriptor/18612-0"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id357">
    <rdfs:seeAlso rdf:resource="descriptor/21101-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18822-3">
    <skos:broader rdf:resource="thsys/71072"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18822-3">
    <skos:broader rdf:resource="thsys/70860"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13256-0">
    <skos:prefLabel xml:lang="en">Shipping</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13256-0">
    <skos:prefLabel xml:lang="de">Schifffahrt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12983-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13398-1">
    <zbwext:useInsteadNote rdf:nodeID="id262"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19594-6">
    <skos:broader rdf:resource="thsys/73104"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19594-6">
    <skos:broader rdf:resource="descriptor/15458-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19767-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14488-2">
    <skos:prefLabel xml:lang="en">Torch</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14488-2">
    <skos:prefLabel xml:lang="de">Leuchte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71038">
    <skos:narrower rdf:resource="descriptor/15621-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71038">
    <skos:narrower rdf:resource="descriptor/14762-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71038">
    <skos:narrower rdf:resource="descriptor/18004-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71038">
    <skos:narrower rdf:resource="descriptor/14752-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71038">
    <skos:narrower rdf:resource="descriptor/14751-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71038">
    <skos:narrower rdf:resource="descriptor/18658-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71038">
    <skos:narrower rdf:resource="descriptor/14765-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18762-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13813-1">
    <skos:altLabel xml:lang="de">Motorfahrzeugversicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13813-1">
    <skos:altLabel xml:lang="de">Kraftverkehrsversicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13813-1">
    <skos:altLabel xml:lang="de">Kraftfahrzeugversicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13813-1">
    <skos:altLabel xml:lang="de">Kraftfahrzeughaftpflichtversicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13813-1">
    <skos:altLabel xml:lang="de">Kraftfahrzeug-Haftpflichtversicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13813-1">
    <skos:altLabel xml:lang="de">Kraftfahrtunfallversicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13813-1">
    <skos:altLabel xml:lang="de">Kfz-Versicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13813-1">
    <skos:altLabel xml:lang="de">Kfz-Haftpflichtversicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13813-1">
    <skos:altLabel xml:lang="de">Autoversicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12809-4">
    <skos:broader rdf:resource="thsys/70550"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12809-4">
    <skos:broader rdf:resource="descriptor/12713-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13279-2">
    <skos:prefLabel xml:lang="en">Exchange</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13279-2">
    <skos:prefLabel xml:lang="de">Börse</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70209">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70900">
    <skos:narrower rdf:resource="descriptor/13753-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70900">
    <skos:narrower rdf:resource="descriptor/18648-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70900">
    <skos:narrower rdf:resource="descriptor/18022-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70900">
    <skos:narrower rdf:resource="descriptor/18712-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70900">
    <skos:narrower rdf:resource="descriptor/18722-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70900">
    <skos:narrower rdf:resource="descriptor/18721-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70900">
    <skos:narrower rdf:resource="descriptor/18718-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70900">
    <skos:narrower rdf:resource="descriptor/18716-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70900">
    <skos:narrower rdf:resource="descriptor/13692-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70900">
    <skos:narrower rdf:resource="descriptor/13747-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70900">
    <skos:narrower rdf:resource="descriptor/13746-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70900">
    <skos:narrower rdf:resource="descriptor/18719-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70900">
    <skos:narrower rdf:resource="descriptor/13686-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70900">
    <skos:narrower rdf:resource="descriptor/12268-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70900">
    <skos:narrower rdf:resource="descriptor/13755-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70900">
    <skos:narrower rdf:resource="thsys/70897"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70900">
    <skos:narrower rdf:resource="thsys/70899"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19732-6">
    <skos:broader rdf:resource="thsys/73394"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19732-6">
    <skos:broader rdf:resource="descriptor/16745-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12806-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12670-2">
    <skos:related rdf:resource="descriptor/12648-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18261-3">
    <skos:broader rdf:resource="thsys/70994"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18261-3">
    <skos:broader rdf:resource="descriptor/12996-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18694-0">
    <skos:related rdf:resource="descriptor/18061-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18694-0">
    <skos:related rdf:resource="descriptor/19399-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18694-0">
    <skos:related rdf:resource="descriptor/10816-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18694-0">
    <skos:related rdf:resource="descriptor/10812-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18694-0">
    <skos:related rdf:resource="descriptor/10807-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18694-0">
    <skos:related rdf:resource="descriptor/10805-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18694-0">
    <skos:related rdf:resource="descriptor/10797-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18479-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70982">
    <skos:prefLabel xml:lang="en">W.06.01.14  Defence industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70982">
    <skos:prefLabel xml:lang="de">W.06.01.14  Rüstungsindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10683-3">
    <skos:altLabel xml:lang="en">Foreign trade regulation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10683-3">
    <skos:altLabel xml:lang="de">Handelsbeschränkung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10683-3">
    <skos:altLabel xml:lang="de">Außenwirtschaftskontrolle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10683-3">
    <skos:altLabel xml:lang="de">Außenhandelsrestriktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18617-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70082">
    <rdfs:label xml:lang="de">Umwelt- und Ressourcenökonomik</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14538-6">
    <skos:broader rdf:resource="thsys/70423"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14538-6">
    <skos:broader rdf:resource="descriptor/14539-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12783-3">
    <skos:narrower rdf:resource="descriptor/18794-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12236-4">
    <skos:altLabel xml:lang="en">Credit guarantee</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12236-4">
    <skos:altLabel xml:lang="de">Bankbürgschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12236-4">
    <skos:altLabel xml:lang="de">Avalkredit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14561-4">
    <skos:altLabel xml:lang="de">Trockenrasierer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14561-4">
    <skos:altLabel xml:lang="de">Rasiermesser</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14561-4">
    <skos:altLabel xml:lang="de">Rasierklinge</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14561-4">
    <skos:altLabel xml:lang="de">Elektrorasierer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16184-5">
    <skos:scopeNote xml:lang="de">Internationale Abkommen werden geschlossen zwischen Staaten oder staatlichen Organisationen; sie setzen internationales Recht und können die Form von Konventionen, Protokollen, Pakten, Verträgen, usw. annehmen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11219-2">
    <skos:narrower rdf:resource="descriptor/19495-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11219-2">
    <skos:narrower rdf:resource="descriptor/18125-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11219-2">
    <skos:narrower rdf:resource="descriptor/11337-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11219-2">
    <skos:narrower rdf:resource="descriptor/11336-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11219-2">
    <skos:narrower rdf:resource="descriptor/11159-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11219-2">
    <skos:narrower rdf:resource="descriptor/10143-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11219-2">
    <skos:narrower rdf:resource="descriptor/11220-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12996-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18958-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10349-1">
    <skos:broader rdf:resource="thsys/70233"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10349-1">
    <skos:broader rdf:resource="thsys/70020"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10349-1">
    <skos:broader rdf:resource="descriptor/10347-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11374-4">
    <skos:altLabel xml:lang="en">Training staff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11374-4">
    <skos:altLabel xml:lang="de">Schullehrerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11374-4">
    <skos:altLabel xml:lang="de">Schullehrer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11374-4">
    <skos:altLabel xml:lang="de">Pädagoginnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11374-4">
    <skos:altLabel xml:lang="de">Pädagogen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11374-4">
    <skos:altLabel xml:lang="de">Lehrpersonal</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11374-4">
    <skos:altLabel xml:lang="de">Lehrerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11374-4">
    <skos:altLabel xml:lang="de">Lehrer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11048-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15405-6">
    <skos:altLabel xml:lang="en">Path analysis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15405-6">
    <skos:altLabel xml:lang="en">Multidimensional scaling</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15405-6">
    <skos:altLabel xml:lang="en">Latent-structure analysis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15405-6">
    <skos:altLabel xml:lang="en">Latent-profile analysis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15405-6">
    <skos:altLabel xml:lang="en">Latent variable model</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15405-6">
    <skos:altLabel xml:lang="en">Correspondence analysis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15405-6">
    <skos:altLabel xml:lang="de">Pfadanalyse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15405-6">
    <skos:altLabel xml:lang="de">Multivariates Verfahren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15405-6">
    <skos:altLabel xml:lang="de">Multivariate Statistik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15405-6">
    <skos:altLabel xml:lang="de">Multidimensionale Skalierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15405-6">
    <skos:altLabel xml:lang="de">Mehrdimensionale Skalierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15405-6">
    <skos:altLabel xml:lang="de">Latentes-Struktur-Modell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15405-6">
    <skos:altLabel xml:lang="de">Korrespondenzanalyse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70869">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.22.01.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17927-1">
    <skos:altLabel xml:lang="de">Republik Paraguay</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70130">
    <skos:narrower rdf:resource="descriptor/17179-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70130">
    <skos:narrower rdf:resource="descriptor/18226-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70130">
    <skos:narrower rdf:resource="descriptor/19087-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70130">
    <skos:narrower rdf:resource="descriptor/17177-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70130">
    <skos:narrower rdf:resource="descriptor/17480-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70130">
    <skos:narrower rdf:resource="descriptor/18228-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70130">
    <skos:narrower rdf:resource="descriptor/18227-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70130">
    <skos:narrower rdf:resource="descriptor/17176-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70130">
    <skos:narrower rdf:resource="descriptor/17184-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70130">
    <skos:narrower rdf:resource="descriptor/17175-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70130">
    <skos:narrower rdf:resource="descriptor/19086-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70130">
    <skos:narrower rdf:resource="descriptor/17171-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19684-5">
    <skos:prefLabel xml:lang="en">National territory</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19684-5">
    <skos:prefLabel xml:lang="de">Staatsgebiet</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19120-2">
    <skos:altLabel xml:lang="de">Außendienstunterstützungssystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19120-2">
    <skos:altLabel xml:lang="de">AUDIUS</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14021-1">
    <skos:related rdf:resource="descriptor/14236-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14021-1">
    <skos:related rdf:resource="descriptor/14607-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14021-1">
    <skos:related rdf:resource="descriptor/14033-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14021-1">
    <skos:related rdf:resource="descriptor/14252-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14021-1">
    <skos:related rdf:resource="descriptor/12905-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14021-1">
    <skos:related rdf:resource="descriptor/18495-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14021-1">
    <skos:related rdf:resource="descriptor/12904-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14021-1">
    <skos:related rdf:resource="descriptor/13194-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14021-1">
    <skos:related rdf:resource="descriptor/14366-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14021-1">
    <skos:related rdf:resource="descriptor/18491-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14413-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14092-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14455-3">
    <skos:narrower rdf:resource="descriptor/14461-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14455-3">
    <skos:narrower rdf:resource="descriptor/14456-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14455-3">
    <skos:narrower rdf:resource="descriptor/14359-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14455-3">
    <skos:narrower rdf:resource="descriptor/14450-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14455-3">
    <skos:narrower rdf:resource="descriptor/14449-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14455-3">
    <skos:narrower rdf:resource="descriptor/18247-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16365-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11709-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11525-2">
    <skos:narrower rdf:resource="descriptor/16122-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11525-2">
    <skos:narrower rdf:resource="descriptor/11640-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11525-2">
    <skos:narrower rdf:resource="descriptor/16100-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13220-0">
    <skos:related rdf:resource="descriptor/18436-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13220-0">
    <skos:related rdf:resource="descriptor/13232-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13220-0">
    <skos:related rdf:resource="descriptor/14831-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13220-0">
    <skos:related rdf:resource="descriptor/13137-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19827-2">
    <skos:scopeNote xml:lang="de">Grad der Funktionsfähigkeit der Institution "Markt" in bezug auf Koordination, Knappheitsanzeige, Einkommensverteilung und Allokation.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13367-5">
    <skos:related rdf:resource="descriptor/16037-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13367-5">
    <skos:related rdf:resource="descriptor/18448-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13367-5">
    <skos:related rdf:resource="descriptor/18434-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13367-5">
    <skos:related rdf:resource="descriptor/13102-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13367-5">
    <skos:related rdf:resource="descriptor/14812-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70342">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70183">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11211-4">
    <skos:broader rdf:resource="thsys/71062"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11211-4">
    <skos:broader rdf:resource="descriptor/11183-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11248-2">
    <skos:prefLabel xml:lang="en">Flexible working time</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11248-2">
    <skos:prefLabel xml:lang="de">Arbeitszeitflexibilisierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18670-0">
    <skos:prefLabel xml:lang="en">Securitization</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18670-0">
    <skos:prefLabel xml:lang="de">Securitization</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10505-6">
    <skos:prefLabel xml:lang="en">Agricultural development</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10505-6">
    <skos:prefLabel xml:lang="de">Landwirtschaftliche Entwicklung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11607-0">
    <skos:prefLabel xml:lang="en">Debt management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11607-0">
    <skos:prefLabel xml:lang="de">Debt Management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10833-3">
    <skos:broader rdf:resource="thsys/71091"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10833-3">
    <skos:broader rdf:resource="thsys/70952"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10833-3">
    <skos:broader rdf:resource="descriptor/10605-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12075-2">
    <skos:prefLabel xml:lang="en">Corporate reputation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12075-2">
    <skos:prefLabel xml:lang="de">Firmenimage</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19194-1">
    <skos:related rdf:resource="descriptor/19670-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19194-1">
    <skos:related rdf:resource="descriptor/13441-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19194-1">
    <skos:related rdf:resource="descriptor/13434-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12514-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19159-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13154-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17679-1">
    <skos:broader rdf:resource="thsys/70445"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17679-1">
    <skos:broader rdf:resource="descriptor/17672-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11207-2">
    <skos:broader rdf:resource="thsys/70058"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11207-2">
    <skos:broader rdf:resource="thsys/70051"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11207-2">
    <skos:broader rdf:resource="descriptor/11183-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11959-4">
    <skos:altLabel xml:lang="de">Regionale Input-Output Analyse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18924-2">
    <skos:prefLabel xml:lang="en">Cultural psychology</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18924-2">
    <skos:prefLabel xml:lang="de">Kulturpsychologie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13264-1">
    <skos:broader rdf:resource="thsys/70943"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10824-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12210-1">
    <skos:narrower rdf:resource="descriptor/18733-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12210-1">
    <skos:narrower rdf:resource="descriptor/12204-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12210-1">
    <skos:narrower rdf:resource="descriptor/19271-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12210-1">
    <skos:narrower rdf:resource="descriptor/19196-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12210-1">
    <skos:narrower rdf:resource="descriptor/19269-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12904-3">
    <skos:broader rdf:resource="thsys/71004"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12904-3">
    <skos:broader rdf:resource="descriptor/12913-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10361-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19358-4">
    <skos:altLabel xml:lang="en">Foreign debt service</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19358-4">
    <skos:altLabel xml:lang="en">Debt service (Foreign debt)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19358-4">
    <skos:altLabel xml:lang="de">Schuldendienst von Auslandsschulden</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19044-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70947">
    <skos:broader rdf:resource="thsys/70963"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73341">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17503-3">
    <skos:prefLabel xml:lang="en">Uzbekistan</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17503-3">
    <skos:prefLabel xml:lang="de">Usbekistan</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#scopeNote">
    <rdfs:isDefinedBy rdf:resource="http://www.w3.org/2004/02/skos/core"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19372-3">
    <skos:prefLabel xml:lang="en">Public works</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19372-3">
    <skos:prefLabel xml:lang="de">Öffentliche Arbeiten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18900-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10340-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14462-6">
    <skos:broader rdf:resource="thsys/70415"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14462-6">
    <skos:broader rdf:resource="descriptor/14449-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11531-0">
    <skos:broader rdf:resource="thsys/70144"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11716-2">
    <skos:related rdf:resource="descriptor/10239-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11716-2">
    <skos:related rdf:resource="descriptor/12931-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11716-2">
    <skos:related rdf:resource="descriptor/12009-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10180-1">
    <skos:altLabel xml:lang="en">Consumer surplus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10180-1">
    <skos:altLabel xml:lang="de">Verbraucherrente</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13709-3">
    <skos:prefLabel xml:lang="en">Bond fund</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13709-3">
    <skos:prefLabel xml:lang="de">Rentenfonds</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19502-2">
    <skos:related rdf:resource="descriptor/18125-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19502-2">
    <skos:related rdf:resource="descriptor/18847-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19502-2">
    <skos:related rdf:resource="descriptor/11337-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19502-2">
    <skos:related rdf:resource="descriptor/18123-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19502-2">
    <skos:related rdf:resource="descriptor/11313-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19032-6">
    <skos:scopeNote xml:lang="de">Nicht zu verwechseln mit politischer Kontrolle oder wirtschaftspolitischer Aufsichtsfunktion.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11530-2">
    <skos:prefLabel xml:lang="en">Cost-effectiveness analysis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11530-2">
    <skos:prefLabel xml:lang="de">Kosten-Wirksamkeits-Analyse</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11962-1">
    <skos:broader rdf:resource="thsys/72181"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11962-1">
    <skos:broader rdf:resource="thsys/70058"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11962-1">
    <skos:broader rdf:resource="descriptor/19072-1"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id61">
    <rdfs:seeAlso rdf:resource="descriptor/18664-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70384">
    <rdfs:label xml:lang="de">Verkaufsförderung und Werbung</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id368">
    <rdfs:seeAlso rdf:resource="descriptor/18871-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14420-1">
    <skos:broader rdf:resource="thsys/70256"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14420-1">
    <skos:broader rdf:resource="descriptor/14421-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10452-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19664-4">
    <skos:scopeNote xml:lang="de">Für Krisen des Bankensektors, von Börsen oder Währungen BENUTZE die entsprechenden Deskriptoren.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15080-1">
    <skos:altLabel xml:lang="de">Index der Verkaufspreise des Großhandels</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15080-1">
    <skos:altLabel xml:lang="de">Index der Großhandelspreise</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12729-2">
    <skos:broader rdf:resource="thsys/70310"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12729-2">
    <skos:broader rdf:resource="descriptor/12725-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16171-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13881-5">
    <skos:related rdf:resource="descriptor/13960-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13881-5">
    <skos:related rdf:resource="descriptor/12980-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#hasTopConcept">
    <rdfs:range rdf:resource="http://www.w3.org/2004/02/skos/core#Concept"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16671-1">
    <skos:related rdf:resource="descriptor/16619-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16671-1">
    <skos:related rdf:resource="descriptor/11992-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16671-1">
    <skos:related rdf:resource="descriptor/11969-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16671-1">
    <skos:related rdf:resource="descriptor/16663-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16671-1">
    <skos:related rdf:resource="descriptor/10488-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13180-0">
    <skos:narrower rdf:resource="descriptor/13186-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13180-0">
    <skos:narrower rdf:resource="descriptor/13181-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10051-5">
    <skos:broader rdf:resource="thsys/71100"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10051-5">
    <skos:broader rdf:resource="thsys/70234"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10051-5">
    <skos:broader rdf:resource="descriptor/10065-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14793-4">
    <skos:prefLabel xml:lang="en">Cork</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14793-4">
    <skos:prefLabel xml:lang="de">Korkwaren</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12509-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24664-2">
    <skos:altLabel xml:lang="en">Information goods</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24664-2">
    <skos:altLabel xml:lang="en">Digital good</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24664-2">
    <skos:altLabel xml:lang="de">Digitales Produkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24664-2">
    <skos:altLabel xml:lang="de">Digitales Gut</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24664-2">
    <skos:altLabel xml:lang="de">Digitale Produkte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19353-0">
    <skos:related rdf:resource="descriptor/19336-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19353-0">
    <skos:related rdf:resource="descriptor/10733-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14741-2">
    <skos:related rdf:resource="descriptor/18261-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14741-2">
    <skos:related rdf:resource="descriptor/14740-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14741-2">
    <skos:related rdf:resource="descriptor/14738-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14741-2">
    <skos:related rdf:resource="descriptor/14026-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70113">
    <rdfs:label xml:lang="en">Human resource management</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70113">
    <rdfs:label xml:lang="de">Personalwirtschaft</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11083-1">
    <skos:altLabel xml:lang="de">Neoimperialismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11083-1">
    <skos:altLabel xml:lang="de">Imperialismustheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#editorialNote">
    <rdfs:isDefinedBy rdf:resource="http://www.w3.org/2004/02/skos/core"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15946-0">
    <skos:prefLabel xml:lang="en">Child mortality</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15946-0">
    <skos:prefLabel xml:lang="de">Kindersterblichkeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13081-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19124-1">
    <skos:scopeNote xml:lang="de">Nur im Sinne der Gewährleistung des Verkäufers, für die Mangelfreiheit eines Produktes oder einer Dienstleistung einzustehen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19231-0">
    <skos:altLabel xml:lang="de">Produktionsvolumen der Gesamtwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19231-0">
    <skos:altLabel xml:lang="de">Produktion der Gesamtwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19231-0">
    <skos:altLabel xml:lang="de">Output (Volkswirtschaft)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19231-0">
    <skos:altLabel xml:lang="de">Gütererzeugung der Gesamtwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19231-0">
    <skos:altLabel xml:lang="de">Gesamtwirtschaftliches Produktionsvolumen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18094-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17934-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15762-3">
    <skos:altLabel xml:lang="de">Umweltverwaltung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15762-3">
    <skos:altLabel xml:lang="de">Umweltamt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18248-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11728-2">
    <skos:narrower rdf:resource="descriptor/11704-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11728-2">
    <skos:narrower rdf:resource="descriptor/19378-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11728-2">
    <skos:narrower rdf:resource="descriptor/11731-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11728-2">
    <skos:narrower rdf:resource="descriptor/11729-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11728-2">
    <skos:narrower rdf:resource="descriptor/10381-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14318-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18579-0">
    <skos:broader rdf:resource="thsys/70938"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18579-0">
    <skos:broader rdf:resource="thsys/70926"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18579-0">
    <skos:broader rdf:resource="descriptor/11833-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14231-4">
    <skos:broader rdf:resource="thsys/70348"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14231-4">
    <skos:broader rdf:resource="descriptor/14237-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19545-5">
    <skos:broader rdf:resource="thsys/71020"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19545-5">
    <skos:broader rdf:resource="descriptor/11800-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10243-3">
    <skos:broader rdf:resource="thsys/71111"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10243-3">
    <skos:broader rdf:resource="thsys/70270"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10243-3">
    <skos:broader rdf:resource="descriptor/10049-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13629-1">
    <skos:related rdf:resource="descriptor/11893-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12166-6">
    <skos:prefLabel xml:lang="en">Early warning system</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12166-6">
    <skos:prefLabel xml:lang="de">Frühwarnsystem</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18213-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11311-0">
    <skos:altLabel xml:lang="en">Shop committee</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11311-0">
    <skos:altLabel xml:lang="de">Vertrauensleute</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11311-0">
    <skos:altLabel xml:lang="de">Personalvertretung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11311-0">
    <skos:altLabel xml:lang="de">Personalrätinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11311-0">
    <skos:altLabel xml:lang="de">Personalrat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11311-0">
    <skos:altLabel xml:lang="de">Betriebsrätinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11800-6">
    <skos:related rdf:resource="descriptor/11805-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11800-6">
    <skos:related rdf:resource="descriptor/19674-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12012-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19470-3">
    <skos:prefLabel xml:lang="en">Local public services</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19470-3">
    <skos:prefLabel xml:lang="de">Kommunale Dienstleistung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19052-0">
    <skos:prefLabel xml:lang="en">Automotive bodywork</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19052-0">
    <skos:prefLabel xml:lang="de">Karosseriebau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18534-1">
    <skos:narrower rdf:resource="descriptor/18537-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18534-1">
    <skos:narrower rdf:resource="descriptor/19520-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18592-1">
    <skos:related rdf:resource="descriptor/18593-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18592-1">
    <skos:related rdf:resource="descriptor/13546-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18592-1">
    <skos:related rdf:resource="descriptor/14455-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18592-1">
    <skos:related rdf:resource="descriptor/12872-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16672-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70011">
    <skos:narrower rdf:resource="descriptor/11260-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70011">
    <skos:narrower rdf:resource="descriptor/15429-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70011">
    <skos:narrower rdf:resource="descriptor/16724-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70011">
    <skos:narrower rdf:resource="descriptor/15814-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70011">
    <skos:narrower rdf:resource="descriptor/16554-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70011">
    <skos:narrower rdf:resource="descriptor/19150-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70011">
    <skos:narrower rdf:resource="descriptor/12112-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70011">
    <skos:narrower rdf:resource="descriptor/12106-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70011">
    <skos:narrower rdf:resource="descriptor/19219-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70011">
    <skos:narrower rdf:resource="descriptor/13383-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70011">
    <skos:narrower rdf:resource="descriptor/16579-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70011">
    <skos:narrower rdf:resource="descriptor/19741-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70011">
    <skos:narrower rdf:resource="descriptor/19745-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10980-4">
    <skos:prefLabel xml:lang="en">Government failure</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10980-4">
    <skos:prefLabel xml:lang="de">Staatsversagen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11565-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18296-5">
    <skos:related rdf:resource="descriptor/18585-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18296-5">
    <skos:related rdf:resource="descriptor/13576-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18296-5">
    <skos:related rdf:resource="descriptor/18607-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18296-5">
    <skos:related rdf:resource="descriptor/13575-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18296-5">
    <skos:related rdf:resource="descriptor/13268-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18599-1">
    <skos:altLabel xml:lang="de">Hafenarbeiterinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16370-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13234-3">
    <skos:altLabel xml:lang="de">Spirituosenhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17433-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18296-5">
    <skos:broader rdf:resource="thsys/70135"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18296-5">
    <skos:broader rdf:resource="descriptor/14490-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14139-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19074-4">
    <skos:broader rdf:resource="thsys/70029"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19074-4">
    <skos:broader rdf:resource="descriptor/19784-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10085-2">
    <skos:altLabel xml:lang="de">Kurzfristiges Gleichgewicht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14839-6">
    <skos:prefLabel xml:lang="en">Off-the-peg textiles</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14839-6">
    <skos:prefLabel xml:lang="de">Konfektionierte Textilien</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15076-6">
    <skos:prefLabel xml:lang="en">Economic statistics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15076-6">
    <skos:prefLabel xml:lang="de">Wirtschaftsstatistik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10426-2">
    <skos:narrower rdf:resource="descriptor/10542-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10426-2">
    <skos:narrower rdf:resource="descriptor/11954-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70965">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14035-4">
    <skos:broader rdf:resource="thsys/71015"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14035-4">
    <skos:broader rdf:resource="thsys/71013"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14035-4">
    <skos:broader rdf:resource="descriptor/14039-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24704-2">
    <skos:broader rdf:resource="thsys/70562"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24704-2">
    <skos:broader rdf:resource="descriptor/12134-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14064-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10845-3">
    <skos:related rdf:resource="descriptor/10018-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10845-3">
    <skos:related rdf:resource="descriptor/19814-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10845-3">
    <skos:related rdf:resource="descriptor/18614-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18433-0">
    <skos:narrower rdf:resource="descriptor/18761-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10377-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12569-5">
    <skos:broader rdf:resource="thsys/73385"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12569-5">
    <skos:broader rdf:resource="thsys/70244"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12569-5">
    <skos:broader rdf:resource="descriptor/12574-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12569-5">
    <skos:broader rdf:resource="descriptor/12570-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16709-2">
    <skos:prefLabel xml:lang="en">Ethics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16709-2">
    <skos:prefLabel xml:lang="de">Ethik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13023-2">
    <skos:related rdf:resource="descriptor/14258-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13720-1">
    <skos:related rdf:resource="descriptor/10821-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13720-1">
    <skos:related rdf:resource="descriptor/18539-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70361">
    <rdfs:label xml:lang="en">Germany</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70361">
    <rdfs:label xml:lang="de">Deutschland</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10713-6">
    <skos:related rdf:resource="descriptor/10649-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10713-6">
    <skos:related rdf:resource="descriptor/16122-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10713-6">
    <skos:related rdf:resource="descriptor/10710-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10713-6">
    <skos:related rdf:resource="descriptor/11704-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10713-6">
    <skos:related rdf:resource="descriptor/19320-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17742-6">
    <skos:altLabel xml:lang="de">Zaire</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17742-6">
    <skos:altLabel xml:lang="de">Kongo-Zaire</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17742-6">
    <skos:altLabel xml:lang="de">Kongo-Leopoldville</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17742-6">
    <skos:altLabel xml:lang="de">Kongo-Kinshasa</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17742-6">
    <skos:altLabel xml:lang="de">Demokratische Republik Kongo</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17742-6">
    <skos:altLabel xml:lang="de">Belgisch-Kongo</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14627-0">
    <skos:related rdf:resource="descriptor/14652-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14627-0">
    <skos:related rdf:resource="descriptor/18256-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19406-5">
    <skos:scopeNote xml:lang="de">Prinzip, nach dem sich Leistung und Gegenleistung entsprechen sollen, vor allem bei Steuern und anderen öffentlichen Abgaben.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14236-1">
    <skos:narrower rdf:resource="descriptor/14245-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14236-1">
    <skos:narrower rdf:resource="descriptor/14244-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14236-1">
    <skos:narrower rdf:resource="descriptor/18162-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14236-1">
    <skos:narrower rdf:resource="descriptor/14627-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14236-1">
    <skos:narrower rdf:resource="descriptor/14240-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14236-1">
    <skos:narrower rdf:resource="descriptor/14205-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14236-1">
    <skos:narrower rdf:resource="descriptor/14237-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14236-1">
    <skos:narrower rdf:resource="descriptor/14266-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14236-1">
    <skos:narrower rdf:resource="descriptor/14313-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14236-1">
    <skos:narrower rdf:resource="descriptor/14230-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14236-1">
    <skos:narrower rdf:resource="descriptor/14229-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14236-1">
    <skos:narrower rdf:resource="descriptor/14199-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14236-1">
    <skos:narrower rdf:resource="descriptor/14198-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14236-1">
    <skos:narrower rdf:resource="descriptor/14223-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14634-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10434-3">
    <skos:prefLabel xml:lang="en">Growth theory</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10434-3">
    <skos:prefLabel xml:lang="de">Wachstumstheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19852-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16769-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19283-2">
    <skos:related rdf:resource="descriptor/19404-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19283-2">
    <skos:related rdf:resource="descriptor/10842-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19283-2">
    <skos:related rdf:resource="descriptor/12009-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11893-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18267-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18390-6">
    <skos:related rdf:resource="descriptor/18383-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18390-6">
    <skos:related rdf:resource="descriptor/18407-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18390-6">
    <skos:related rdf:resource="descriptor/14368-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18390-6">
    <skos:related rdf:resource="descriptor/18384-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18390-6">
    <skos:related rdf:resource="descriptor/12980-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10926-3">
    <skos:prefLabel xml:lang="en">Limit pricing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10926-3">
    <skos:prefLabel xml:lang="de">Limitpreis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10304-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11568-5">
    <skos:altLabel xml:lang="de">Transferausgaben der Öffentlichen Hand</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11568-5">
    <skos:altLabel xml:lang="de">Öffentliche Transferzahlung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15858-4">
    <skos:altLabel xml:lang="de">Reklamepsychologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15858-4">
    <skos:altLabel xml:lang="de">Psychologie der Werbung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18942-0">
    <skos:prefLabel xml:lang="en">Technical information</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18942-0">
    <skos:prefLabel xml:lang="de">Technische Information</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14751-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10986-6">
    <skos:prefLabel xml:lang="en">Economic liberalism</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10986-6">
    <skos:prefLabel xml:lang="de">Wirtschaftsliberalismus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14983-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11270-2">
    <skos:broader rdf:resource="thsys/70051"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19199-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10505-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18095-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17496-2">
    <skos:altLabel xml:lang="de">Republik Kasachstan</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19121-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14732-3">
    <skos:altLabel xml:lang="en">Shampoo</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14732-3">
    <skos:altLabel xml:lang="de">Haarwaschmittel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14732-3">
    <skos:altLabel xml:lang="de">Haartönung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14732-3">
    <skos:altLabel xml:lang="de">Haarspray</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14732-3">
    <skos:altLabel xml:lang="de">Haarshampoo</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14732-3">
    <skos:altLabel xml:lang="de">Haarfestiger</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10109-0">
    <skos:prefLabel xml:lang="en">Impossibility theorem</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10109-0">
    <skos:prefLabel xml:lang="de">Unmöglichkeitstheorem</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13389-2">
    <skos:prefLabel xml:lang="en">Legal services</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13389-2">
    <skos:prefLabel xml:lang="de">Rechtsberatung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71092">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16794-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id294">
    <rdfs:seeAlso rdf:resource="descriptor/10692-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17785-2">
    <skos:broader rdf:resource="thsys/70086"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17785-2">
    <skos:broader rdf:resource="descriptor/17791-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19830-6">
    <skos:prefLabel xml:lang="en">Charity</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19830-6">
    <skos:prefLabel xml:lang="de">Wohltätigkeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id301">
    <rdf:value xml:lang="de">Für Länder mit staatlichem Außenhandelsmonopol</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10242-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18000-3">
    <skos:broader rdf:resource="thsys/70922"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18000-3">
    <skos:broader rdf:resource="descriptor/18663-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18000-3">
    <skos:broader rdf:resource="descriptor/12010-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14217-5">
    <skos:prefLabel xml:lang="en">Lubricant</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14217-5">
    <skos:prefLabel xml:lang="de">Schmierstoff</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18189-6">
    <skos:related rdf:resource="descriptor/18200-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18189-6">
    <skos:related rdf:resource="descriptor/19365-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18188-1">
    <skos:prefLabel xml:lang="en">Interest rate parity</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18188-1">
    <skos:prefLabel xml:lang="de">Zinsparität</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id176">
    <rdfs:seeAlso rdf:resource="descriptor/13630-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18088-5">
    <skos:altLabel xml:lang="en">Money market mutual fund</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18088-5">
    <skos:altLabel xml:lang="en">Money market funds</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18088-5">
    <skos:altLabel xml:lang="en">MMMF</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18885-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10097-2">
    <skos:prefLabel xml:lang="en">Compensation problem</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10097-2">
    <skos:prefLabel xml:lang="de">Kompensationslösung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70996">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.06.01.04</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10899-1">
    <skos:prefLabel xml:lang="en">Horizontal merger</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10899-1">
    <skos:prefLabel xml:lang="de">Horizontale Konzentration</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12974-3">
    <skos:prefLabel xml:lang="en">Electric utility industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12974-3">
    <skos:prefLabel xml:lang="de">Elektrizitätswirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16514-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17485-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15787-1">
    <skos:related rdf:resource="descriptor/15859-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15787-1">
    <skos:related rdf:resource="descriptor/15682-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15787-1">
    <skos:related rdf:resource="descriptor/18070-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15787-1">
    <skos:related rdf:resource="descriptor/19601-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15787-1">
    <skos:related rdf:resource="descriptor/19600-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15787-1">
    <skos:related rdf:resource="descriptor/16621-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15787-1">
    <skos:related rdf:resource="descriptor/19599-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19025-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18275-6">
    <skos:prefLabel xml:lang="en">Office furniture</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18275-6">
    <skos:prefLabel xml:lang="de">Büromöbel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15698-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20381-6">
    <skos:prefLabel xml:lang="en">Downsizing (Staff)</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20381-6">
    <skos:prefLabel xml:lang="de">Personalabbau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16628-2">
    <skos:broader rdf:resource="thsys/73378"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16628-2">
    <skos:broader rdf:resource="descriptor/15711-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10898-3">
    <skos:prefLabel xml:lang="en">Countervailing power</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10898-3">
    <skos:prefLabel xml:lang="de">Gegengewichtige Marktmacht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10945-6">
    <skos:prefLabel xml:lang="en">Technological competition</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10945-6">
    <skos:prefLabel xml:lang="de">Innovationswettbewerb</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12881-3">
    <skos:narrower rdf:resource="descriptor/20125-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12881-3">
    <skos:narrower rdf:resource="descriptor/13384-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12881-3">
    <skos:narrower rdf:resource="descriptor/12121-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12881-3">
    <skos:narrower rdf:resource="descriptor/10576-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12881-3">
    <skos:narrower rdf:resource="descriptor/18791-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12881-3">
    <skos:narrower rdf:resource="descriptor/24624-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14824-5">
    <skos:related rdf:resource="descriptor/13007-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14824-5">
    <skos:related rdf:resource="descriptor/14435-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19449-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17964-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71029">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11994-2">
    <skos:prefLabel xml:lang="en">Urban housing market</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11994-2">
    <skos:prefLabel xml:lang="de">Städtische Wohnungsversorgung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14174-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24859-1">
    <skos:related rdf:resource="descriptor/12302-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24859-1">
    <skos:related rdf:resource="descriptor/19210-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24859-1">
    <skos:related rdf:resource="descriptor/11638-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24859-1">
    <skos:related rdf:resource="descriptor/19662-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24859-1">
    <skos:related rdf:resource="descriptor/13799-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24859-1">
    <skos:related rdf:resource="descriptor/12228-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14124-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19072-1">
    <skos:broader rdf:resource="thsys/72181"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18129-3">
    <skos:narrower rdf:resource="descriptor/19487-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13407-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13957-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14279-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17598-1">
    <skos:broader rdf:resource="thsys/70561"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17598-1">
    <skos:broader rdf:resource="descriptor/19167-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72182">
    <skos:broader rdf:resource="thsys/72180"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18261-3">
    <skos:altLabel xml:lang="de">Reinigungsmittelindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12580-3">
    <skos:altLabel xml:lang="en">MBO (Management-by-Objectives)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12580-3">
    <skos:altLabel xml:lang="en">Management-by-objectives</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12580-3">
    <skos:altLabel xml:lang="en">Management-by-delegation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12580-3">
    <skos:altLabel xml:lang="en">Linking pins</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12580-3">
    <skos:altLabel xml:lang="de">Zielvereinbarungen (Führungsinstrument)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12580-3">
    <skos:altLabel xml:lang="de">Überlappende Gruppen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12580-3">
    <skos:altLabel xml:lang="de">Partnerschaftliche Führung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12580-3">
    <skos:altLabel xml:lang="de">Partizipativer Führungsstil</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12580-3">
    <skos:altLabel xml:lang="de">Mitarbeiterorientierte Führung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12580-3">
    <skos:altLabel xml:lang="de">Mitarbeiterbeteiligung am Entscheidungsprozess</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12580-3">
    <skos:altLabel xml:lang="de">Mitarbeiterbeteiligung am Entscheidungsprozeß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12580-3">
    <skos:altLabel xml:lang="de">Harzburger Modell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12580-3">
    <skos:altLabel xml:lang="de">Führen mit Zielvereinbarungen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12580-3">
    <skos:altLabel xml:lang="de">Demokratischer Führungsstil</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12580-3">
    <skos:altLabel xml:lang="de">Delegationsprinzip</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18619-0">
    <skos:prefLabel xml:lang="en">Refrigeration industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18619-0">
    <skos:prefLabel xml:lang="de">Tiefkühlwirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11424-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70274">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12396-3">
    <skos:altLabel xml:lang="de">Steuerrechtliche Bilanz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12396-3">
    <skos:altLabel xml:lang="de">Ertragsteuerbilanz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15777-4">
    <skos:broader rdf:resource="thsys/70943"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15777-4">
    <skos:broader rdf:resource="descriptor/18623-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15777-4">
    <skos:broader rdf:resource="descriptor/15739-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10081-3">
    <skos:altLabel xml:lang="de">Partielle Gleichgewichtstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10081-3">
    <skos:altLabel xml:lang="de">Partialgleichgewicht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19092-2">
    <skos:prefLabel xml:lang="en">Discretionary valuation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19092-2">
    <skos:prefLabel xml:lang="de">Bewertungswahlrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18957-1">
    <skos:prefLabel xml:lang="en">Waste charge</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18957-1">
    <skos:prefLabel xml:lang="de">Abfallabgabe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11276-4">
    <skos:related rdf:resource="descriptor/12945-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11276-4">
    <skos:related rdf:resource="descriptor/13874-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11276-4">
    <skos:related rdf:resource="descriptor/13873-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11276-4">
    <skos:related rdf:resource="descriptor/18983-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19836-1">
    <skos:scopeNote xml:lang="de">für Prognosen BENUTZE: Prognose und spezifischere Deskriptoren</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19836-1">
    <skos:scopeNote xml:lang="de">Für kurzfristig erwartete Entwicklungstendenzen in diversen Lebensbereichen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20987-5">
    <skos:broader rdf:resource="thsys/70128"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20987-5">
    <skos:broader rdf:resource="descriptor/17948-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17770-1">
    <skos:altLabel xml:lang="de">Barbuda</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19390-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19234-1">
    <skos:broader rdf:resource="thsys/71114"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19234-1">
    <skos:broader rdf:resource="thsys/70058"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19234-1">
    <skos:broader rdf:resource="descriptor/11751-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19093-0">
    <skos:altLabel xml:lang="de">Flüssige Mittel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14170-5">
    <skos:prefLabel xml:lang="en">Wild animal</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14170-5">
    <skos:prefLabel xml:lang="de">Wild</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10251-4">
    <skos:prefLabel xml:lang="en">Multiplier</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10251-4">
    <skos:prefLabel xml:lang="de">Multiplikator</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13260-2">
    <skos:prefLabel xml:lang="en">Forwarding company</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13260-2">
    <skos:prefLabel xml:lang="de">Spedition</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12941-4">
    <skos:altLabel xml:lang="de">Legehennenhaltung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12941-4">
    <skos:altLabel xml:lang="de">Geflügelzucht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12941-4">
    <skos:altLabel xml:lang="de">Geflügelwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12941-4">
    <skos:altLabel xml:lang="de">Geflügelmast</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10278-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10122-1">
    <skos:prefLabel xml:lang="en">Capital intensity</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10122-1">
    <skos:prefLabel xml:lang="de">Kapitalintensität</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18192-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18516-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70811">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18584-0">
    <skos:altLabel xml:lang="en">Cab business</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18584-0">
    <skos:altLabel xml:lang="de">Taxiverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73295">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12032-6">
    <skos:related rdf:resource="descriptor/18014-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12032-6">
    <skos:related rdf:resource="descriptor/13434-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12032-6">
    <skos:related rdf:resource="descriptor/18328-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12032-6">
    <skos:related rdf:resource="descriptor/13452-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12032-6">
    <skos:related rdf:resource="descriptor/11804-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12032-6">
    <skos:related rdf:resource="descriptor/11803-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12032-6">
    <skos:related rdf:resource="descriptor/13443-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12032-6">
    <skos:related rdf:resource="descriptor/19339-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19196-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15017-1">
    <skos:related rdf:resource="descriptor/14632-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70947">
    <skos:narrower rdf:resource="descriptor/19564-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70947">
    <skos:narrower rdf:resource="descriptor/13493-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70947">
    <skos:narrower rdf:resource="descriptor/13492-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70947">
    <skos:narrower rdf:resource="descriptor/12821-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70947">
    <skos:narrower rdf:resource="descriptor/18552-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70947">
    <skos:narrower rdf:resource="descriptor/12816-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70947">
    <skos:narrower rdf:resource="descriptor/13490-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70947">
    <skos:narrower rdf:resource="descriptor/13488-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70947">
    <skos:narrower rdf:resource="descriptor/18550-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70947">
    <skos:narrower rdf:resource="descriptor/18548-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70947">
    <skos:narrower rdf:resource="descriptor/13486-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70947">
    <skos:narrower rdf:resource="descriptor/13485-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70947">
    <skos:narrower rdf:resource="descriptor/12798-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70947">
    <skos:narrower rdf:resource="descriptor/13483-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70947">
    <skos:narrower rdf:resource="descriptor/18549-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70947">
    <skos:narrower rdf:resource="descriptor/18556-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70947">
    <skos:narrower rdf:resource="descriptor/18546-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70947">
    <skos:narrower rdf:resource="descriptor/18555-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70947">
    <skos:narrower rdf:resource="descriptor/15077-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70947">
    <skos:narrower rdf:resource="descriptor/18545-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70947">
    <skos:narrower rdf:resource="descriptor/13200-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70947">
    <skos:narrower rdf:resource="descriptor/13481-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70947">
    <skos:narrower rdf:resource="descriptor/13480-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70947">
    <skos:narrower rdf:resource="descriptor/18553-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70947">
    <skos:narrower rdf:resource="descriptor/18558-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70947">
    <skos:narrower rdf:resource="descriptor/13478-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11567-0">
    <skos:altLabel xml:lang="en">Food stamps</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11567-0">
    <skos:altLabel xml:lang="de">Nahrungsmittelhilfe (Sozialhilfe)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11567-0">
    <skos:altLabel xml:lang="de">Nahrungsmittelgutschein</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11567-0">
    <skos:altLabel xml:lang="de">Lebensmittelsubvention</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11369-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14032-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19009-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15972-6">
    <zbwext:useInsteadNote rdf:nodeID="id278"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14188-0">
    <skos:broader rdf:resource="thsys/73351"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14188-0">
    <skos:broader rdf:resource="thsys/70648"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14188-0">
    <skos:broader rdf:resource="descriptor/15745-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71076">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.09.07.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13033-6">
    <skos:related rdf:resource="descriptor/19185-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13033-6">
    <skos:related rdf:resource="descriptor/18392-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13033-6">
    <skos:related rdf:resource="descriptor/14299-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14202-4">
    <skos:broader rdf:resource="thsys/70418"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14202-4">
    <skos:broader rdf:resource="descriptor/14176-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70505">
    <skos:broader rdf:resource="thsys/70002"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11814-2">
    <skos:broader rdf:resource="thsys/70867"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11814-2">
    <skos:broader rdf:resource="thsys/70862"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11814-2">
    <skos:broader rdf:resource="thsys/70235"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11814-2">
    <skos:broader rdf:resource="descriptor/18810-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11717-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10359-5">
    <skos:prefLabel xml:lang="en">Growth cycle</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10359-5">
    <skos:prefLabel xml:lang="de">Wachstumszyklus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10192-1">
    <skos:related rdf:resource="descriptor/10993-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10192-1">
    <skos:related rdf:resource="descriptor/12071-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10192-1">
    <skos:related rdf:resource="descriptor/12089-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10192-1">
    <skos:related rdf:resource="descriptor/10190-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10192-1">
    <skos:related rdf:resource="descriptor/18029-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10192-1">
    <skos:related rdf:resource="descriptor/12083-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10192-1">
    <skos:related rdf:resource="descriptor/12275-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10192-1">
    <skos:related rdf:resource="descriptor/19154-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10192-1">
    <skos:related rdf:resource="descriptor/19834-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24666-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18138-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16783-4">
    <skos:related rdf:resource="descriptor/18932-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16783-4">
    <skos:related rdf:resource="descriptor/16759-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16783-4">
    <skos:related rdf:resource="descriptor/18949-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16956-6">
    <skos:broader rdf:resource="thsys/70003"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16956-6">
    <skos:broader rdf:resource="descriptor/16972-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10956-1">
    <skos:broader rdf:resource="thsys/70242"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10956-1">
    <skos:broader rdf:resource="descriptor/10062-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16001-4">
    <skos:broader rdf:resource="thsys/73390"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16001-4">
    <skos:broader rdf:resource="thsys/71013"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16001-4">
    <skos:broader rdf:resource="thsys/70227"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16001-4">
    <skos:broader rdf:resource="descriptor/15993-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11591-3">
    <skos:related rdf:resource="descriptor/11590-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16767-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13804-2">
    <skos:prefLabel xml:lang="en">Cheque</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13804-2">
    <skos:prefLabel xml:lang="de">Scheck</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13397-3">
    <skos:broader rdf:resource="thsys/73378"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13397-3">
    <skos:broader rdf:resource="descriptor/10984-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12829-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13397-3">
    <skos:narrower rdf:resource="descriptor/16374-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13397-3">
    <skos:narrower rdf:resource="descriptor/16373-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13397-3">
    <skos:narrower rdf:resource="descriptor/16364-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13397-3">
    <skos:narrower rdf:resource="descriptor/16372-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13397-3">
    <skos:narrower rdf:resource="descriptor/16363-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13397-3">
    <skos:narrower rdf:resource="descriptor/16362-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13397-3">
    <skos:narrower rdf:resource="descriptor/16361-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12186-0">
    <skos:prefLabel xml:lang="en">Corporation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12186-0">
    <skos:prefLabel xml:lang="de">Kapitalgesellschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15694-1">
    <skos:altLabel xml:lang="de">Mentalitätengeschichte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15694-1">
    <skos:altLabel xml:lang="de">Kunstgeschichte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10850-3">
    <skos:altLabel xml:lang="en">Documentary letter of credit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10850-3">
    <skos:altLabel xml:lang="en">Commercial letter of credit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10850-3">
    <skos:altLabel xml:lang="de">Dokumentenakkreditiv</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10850-3">
    <skos:altLabel xml:lang="de">Bankakkreditiv</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14740-4">
    <skos:altLabel xml:lang="en">Surface active agents</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14740-4">
    <skos:altLabel xml:lang="de">Grenzflächenaktive Stoffe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70256">
    <rdfs:label xml:lang="de">Luftfahrzeuge</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16548-0">
    <skos:broader rdf:resource="thsys/70832"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16548-0">
    <skos:broader rdf:resource="thsys/70458"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16548-0">
    <skos:broader rdf:resource="descriptor/18910-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16548-0">
    <skos:broader rdf:resource="descriptor/16527-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14537-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19652-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10142-2">
    <skos:broader rdf:resource="thsys/71117"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10142-2">
    <skos:broader rdf:resource="thsys/70823"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20110-2">
    <skos:related rdf:resource="descriptor/16037-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20110-2">
    <skos:related rdf:resource="descriptor/18176-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20110-2">
    <skos:related rdf:resource="descriptor/10698-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20110-2">
    <skos:related rdf:resource="descriptor/16053-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16287-2">
    <skos:scopeNote xml:lang="de">Der politische Willensbildungsprozess, der schließlich in politischen Entscheidungen mündet, wird in modernen Demokratien maßgeblich bestimmt von Parteien und organisierten Gruppeninteressen; möglich sind auch Medieneinflüsse.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15102-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70361">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16193-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16308-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13537-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13106-5">
    <skos:narrower rdf:resource="descriptor/13120-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10911-2">
    <skos:altLabel xml:lang="en">Trust</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10911-2">
    <skos:altLabel xml:lang="de">Unternehmenszusammenschluß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11812-6">
    <skos:prefLabel xml:lang="en">Energy supply</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11812-6">
    <skos:prefLabel xml:lang="de">Energieversorgung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12001-3">
    <skos:broader rdf:resource="thsys/71063"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18750-2">
    <skos:broader rdf:resource="thsys/70970"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18750-2">
    <skos:broader rdf:resource="thsys/70889"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18750-2">
    <skos:broader rdf:resource="descriptor/25742-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18174-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11264-4">
    <skos:narrower rdf:resource="descriptor/11278-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11264-4">
    <skos:narrower rdf:resource="descriptor/19494-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11600-0">
    <skos:altLabel xml:lang="en">Tax assignment</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11600-0">
    <skos:altLabel xml:lang="de">Föderale Finanzwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11600-0">
    <skos:altLabel xml:lang="de">Fiskalföderalismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14640-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13027-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id22">
    <rdf:value xml:lang="de">Für staatliche Maßnahmen zur Steigerung der gesamtwirtschaftlichen Beschäftigung</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13419-5">
    <skos:altLabel xml:lang="de">Wohlfahrtsamt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13419-5">
    <skos:altLabel xml:lang="de">Sozialministerium</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13419-5">
    <skos:altLabel xml:lang="de">Sozialbehörde</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13419-5">
    <skos:altLabel xml:lang="de">Sozialamt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13419-5">
    <skos:altLabel xml:lang="de">Öffentliche Sozialverwaltung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16338-4">
    <skos:related rdf:resource="descriptor/15966-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16338-4">
    <skos:related rdf:resource="descriptor/16340-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16338-4">
    <skos:related rdf:resource="descriptor/16315-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17496-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13425-3">
    <skos:prefLabel xml:lang="en">Rural society</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13425-3">
    <skos:prefLabel xml:lang="de">Agrargesellschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19581-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18202-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18549-2">
    <skos:prefLabel xml:lang="en">Factory outlet</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18549-2">
    <skos:prefLabel xml:lang="de">Fabrikverkauf</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12788-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14548-3">
    <skos:altLabel xml:lang="de">Stricknadel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14548-3">
    <skos:altLabel xml:lang="de">Niet</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14548-3">
    <skos:altLabel xml:lang="de">Nähnadel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14548-3">
    <skos:altLabel xml:lang="de">Nagel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14548-3">
    <skos:altLabel xml:lang="de">Nadel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14548-3">
    <skos:altLabel xml:lang="de">Metallschraube</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14548-3">
    <skos:altLabel xml:lang="de">Metallschloss</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14548-3">
    <skos:altLabel xml:lang="de">Metallschloß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14548-3">
    <skos:altLabel xml:lang="de">Metallkurzwaren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14548-3">
    <skos:altLabel xml:lang="de">Metallkette</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14548-3">
    <skos:altLabel xml:lang="de">Metallgeschirr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14548-3">
    <skos:altLabel xml:lang="de">Metallfolie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14548-3">
    <skos:altLabel xml:lang="de">Eisenwaren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14548-3">
    <skos:altLabel xml:lang="de">Eisen-, Blech- und Metallwaren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14548-3">
    <skos:altLabel xml:lang="de">Degen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14548-3">
    <skos:altLabel xml:lang="de">Blechwaren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14548-3">
    <skos:altLabel xml:lang="de">Blanke Waffen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14548-3">
    <skos:altLabel xml:lang="de">Beschläge</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14548-3">
    <skos:altLabel xml:lang="de">Anstecknadel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70851">
    <skos:broader rdf:resource="thsys/70853"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13217-3">
    <skos:altLabel xml:lang="de">Kraftfahrzeugteilehandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13217-3">
    <skos:altLabel xml:lang="de">Kfz-Handel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13217-3">
    <skos:altLabel xml:lang="de">Automobilhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13217-3">
    <skos:altLabel xml:lang="de">Autohaus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13217-3">
    <skos:altLabel xml:lang="de">Autohandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71100">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12166-6">
    <skos:altLabel xml:lang="en">Corporate monitoring</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12166-6">
    <skos:altLabel xml:lang="de">Strategische Frühaufklärung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12166-6">
    <skos:altLabel xml:lang="de">Früherkennungssystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12166-6">
    <skos:altLabel xml:lang="de">Betriebliches Frühwarnsystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12166-6">
    <skos:altLabel xml:lang="de">Betriebliches Früherkennungssystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18094-3">
    <skos:narrower rdf:resource="descriptor/13145-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18094-3">
    <skos:narrower rdf:resource="descriptor/13128-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18671-5">
    <skos:scopeNote xml:lang="de">Eine allgemein verbindliche Definition fehlt</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19095-3">
    <skos:related rdf:resource="descriptor/11683-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19095-3">
    <skos:related rdf:resource="descriptor/19232-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19095-3">
    <skos:related rdf:resource="descriptor/12058-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19095-3">
    <skos:related rdf:resource="descriptor/18032-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17780-5">
    <skos:altLabel xml:lang="de">Republik El Salvador</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21072-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14285-2">
    <skos:prefLabel xml:lang="en">Steel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14285-2">
    <skos:prefLabel xml:lang="de">Stahl</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14270-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13641-4">
    <skos:broader rdf:resource="thsys/70924"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13641-4">
    <skos:broader rdf:resource="descriptor/13644-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13641-4">
    <skos:broader rdf:resource="descriptor/18609-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18352-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/v">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13831-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12939-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14996-4">
    <skos:broader rdf:resource="thsys/73298"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14996-4">
    <skos:broader rdf:resource="descriptor/14931-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14996-4">
    <skos:broader rdf:resource="descriptor/18222-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11397-6">
    <skos:prefLabel xml:lang="en">Educational policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11397-6">
    <skos:prefLabel xml:lang="de">Bildungspolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16957-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14976-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16300-1">
    <skos:broader rdf:resource="thsys/73381"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16300-1">
    <skos:broader rdf:resource="descriptor/13420-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13790-1">
    <skos:broader rdf:resource="thsys/70899"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13790-1">
    <skos:broader rdf:resource="descriptor/13784-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16792-3">
    <skos:prefLabel xml:lang="en">Library catalogue</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16792-3">
    <skos:prefLabel xml:lang="de">Bibliothekskatalog</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19752-0">
    <skos:scopeNote xml:lang="de">Das Konzept bezeichnet das Ausmaß, in dem in soziale Beziehungen und Netzwerke investiert wird, das Vertrauen in die soziale Interaktion und in die Einhaltung sozialer Regeln. Das Social Capital ist ein Produktionsfaktor im Rahmen der institutionellen Gegebenheiten einer Volkswirtschaft.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12071-3">
    <skos:altLabel xml:lang="en">Theory of the growth of the firm</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12071-3">
    <skos:altLabel xml:lang="de">Wachstum des Betriebs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12071-3">
    <skos:altLabel xml:lang="de">Unternehmenserweiterung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12071-3">
    <skos:altLabel xml:lang="de">Unternehmensausweitung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12071-3">
    <skos:altLabel xml:lang="de">Theorie des Unternehmenswachstums</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12071-3">
    <skos:altLabel xml:lang="de">Betriebswachstum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15721-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13726-3">
    <skos:altLabel xml:lang="en">MTN (Medium-Term Note)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13726-3">
    <skos:altLabel xml:lang="en">Medium-term note</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13726-3">
    <skos:altLabel xml:lang="de">Privatdiskonten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13726-3">
    <skos:altLabel xml:lang="de">Geldmarkttitel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20409-3">
    <skos:related rdf:resource="descriptor/18986-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20409-3">
    <skos:related rdf:resource="descriptor/16476-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20409-3">
    <skos:related rdf:resource="descriptor/16668-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12847-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19175-5">
    <skos:altLabel xml:lang="de">PTA-Staaten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19175-5">
    <skos:altLabel xml:lang="de">PTA-Länder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19175-5">
    <skos:altLabel xml:lang="de">COMESA-Länder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16432-5">
    <skos:broader rdf:resource="thsys/73377"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16432-5">
    <skos:broader rdf:resource="descriptor/16273-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14596-6">
    <skos:broader rdf:resource="thsys/70461"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14596-6">
    <skos:broader rdf:resource="descriptor/14599-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10715-2">
    <skos:altLabel xml:lang="de">Zolltarifschema</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10715-2">
    <skos:altLabel xml:lang="de">Torquay Protokoll</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16048-6">
    <skos:broader rdf:resource="thsys/73331"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16048-6">
    <skos:broader rdf:resource="descriptor/10621-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16048-6">
    <skos:broader rdf:resource="descriptor/16182-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11954-0">
    <skos:related rdf:resource="descriptor/11929-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11954-0">
    <skos:related rdf:resource="descriptor/11959-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11954-0">
    <skos:related rdf:resource="descriptor/10836-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71053">
    <skos:narrower rdf:resource="descriptor/13375-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71053">
    <skos:narrower rdf:resource="descriptor/14014-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71053">
    <skos:narrower rdf:resource="descriptor/13348-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71053">
    <skos:narrower rdf:resource="descriptor/14007-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71053">
    <skos:narrower rdf:resource="descriptor/18890-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71053">
    <skos:narrower rdf:resource="descriptor/13370-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71053">
    <skos:narrower rdf:resource="descriptor/14005-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19513-4">
    <skos:related rdf:resource="descriptor/19515-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70306">
    <skos:prefLabel xml:lang="en">V.05.01  Monetary theory</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70306">
    <skos:prefLabel xml:lang="de">V.05.01  Geldtheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13349-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19120-2">
    <skos:broader rdf:resource="thsys/71036"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19120-2">
    <skos:broader rdf:resource="thsys/70459"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19120-2">
    <skos:broader rdf:resource="descriptor/12768-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73333">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12896-4">
    <skos:related rdf:resource="descriptor/12898-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14051-6">
    <skos:broader rdf:resource="thsys/70550"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14051-6">
    <skos:broader rdf:resource="thsys/70333"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10458-3">
    <skos:prefLabel xml:lang="en">Public Research</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10458-3">
    <skos:prefLabel xml:lang="de">Öffentliche Forschung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19776-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10822-1">
    <skos:narrower rdf:resource="descriptor/19308-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10822-1">
    <skos:narrower rdf:resource="descriptor/18497-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19778-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13993-1">
    <skos:prefLabel xml:lang="en">Administrative personnel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13993-1">
    <skos:prefLabel xml:lang="de">Verwaltungsberufe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73384">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.07.03</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15925-1">
    <skos:related rdf:resource="descriptor/14581-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15925-1">
    <skos:related rdf:resource="descriptor/11375-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15925-1">
    <skos:related rdf:resource="descriptor/19760-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15925-1">
    <skos:related rdf:resource="descriptor/15946-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15925-1">
    <skos:related rdf:resource="descriptor/18864-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15925-1">
    <skos:related rdf:resource="descriptor/11201-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15925-1">
    <skos:related rdf:resource="descriptor/15924-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15925-1">
    <skos:related rdf:resource="descriptor/16699-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15925-1">
    <skos:related rdf:resource="descriptor/11564-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15925-1">
    <skos:related rdf:resource="descriptor/19759-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15925-1">
    <skos:related rdf:resource="descriptor/14019-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18961-3">
    <skos:related rdf:resource="descriptor/19817-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18961-3">
    <skos:related rdf:resource="descriptor/19727-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73353">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.10.01.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13578-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11304-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16397-2">
    <skos:broader rdf:resource="thsys/73383"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16397-2">
    <skos:broader rdf:resource="descriptor/16405-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13959-1">
    <skos:related rdf:resource="descriptor/13378-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13959-1">
    <skos:related rdf:resource="descriptor/12996-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13959-1">
    <skos:related rdf:resource="descriptor/18430-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13959-1">
    <skos:related rdf:resource="descriptor/13888-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13959-1">
    <skos:related rdf:resource="descriptor/15638-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15880-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13573-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18122-3">
    <skos:related rdf:resource="descriptor/18748-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18122-3">
    <skos:related rdf:resource="descriptor/18756-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18122-3">
    <skos:related rdf:resource="descriptor/12915-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18122-3">
    <skos:related rdf:resource="descriptor/11940-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18122-3">
    <skos:related rdf:resource="descriptor/11905-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13653-4">
    <skos:prefLabel xml:lang="en">Fax</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13653-4">
    <skos:prefLabel xml:lang="de">Telefax</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19478-1">
    <skos:related rdf:resource="descriptor/19477-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19478-1">
    <skos:related rdf:resource="descriptor/11971-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19478-1">
    <skos:related rdf:resource="descriptor/11965-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16909-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13085-1">
    <skos:altLabel xml:lang="de">Metallwarenindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13085-1">
    <skos:altLabel xml:lang="de">Eisenwarenindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13085-1">
    <skos:altLabel xml:lang="de">Eisen-, Blech- und Metallwarenindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13085-1">
    <skos:altLabel xml:lang="de">Blechwarenindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18673-1">
    <skos:prefLabel xml:lang="en">Informal credit market</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18673-1">
    <skos:prefLabel xml:lang="de">Informeller Finanzsektor</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15683-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18194-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12508-4">
    <skos:related rdf:resource="descriptor/12531-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18062-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10687-2">
    <skos:narrower rdf:resource="descriptor/10714-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10687-2">
    <skos:narrower rdf:resource="descriptor/10704-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18073-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19523-1">
    <skos:prefLabel xml:lang="en">Tobin's Q</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19523-1">
    <skos:prefLabel xml:lang="de">Tobin's Q</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10686-4">
    <skos:broader rdf:resource="thsys/71098"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10686-4">
    <skos:broader rdf:resource="thsys/71091"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10686-4">
    <skos:broader rdf:resource="thsys/70952"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10686-4">
    <skos:broader rdf:resource="descriptor/10682-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13269-5">
    <skos:prefLabel xml:lang="en">Local-Area Networks</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13269-5">
    <skos:prefLabel xml:lang="de">LAN</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19195-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12033-4">
    <skos:prefLabel xml:lang="en">General terms and conditions</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12033-4">
    <skos:prefLabel xml:lang="de">Allgemeine Geschäftsbedingungen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10801-2">
    <skos:related rdf:resource="descriptor/19277-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10801-2">
    <skos:related rdf:resource="descriptor/18686-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14393-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13202-2">
    <skos:related rdf:resource="descriptor/14473-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13202-2">
    <skos:related rdf:resource="descriptor/13076-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13202-2">
    <skos:related rdf:resource="descriptor/13077-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70947">
    <rdfs:label xml:lang="en">Retail trade</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70947">
    <rdfs:label xml:lang="de">Einzelhandel</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15722-1">
    <skos:altLabel xml:lang="de">Sozialverhalten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15722-1">
    <skos:altLabel xml:lang="de">Soziales Handeln</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10382-3">
    <skos:prefLabel xml:lang="en">Monetary policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10382-3">
    <skos:prefLabel xml:lang="de">Geldpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18879-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21408-0">
    <skos:prefLabel xml:lang="en">Couponing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21408-0">
    <skos:prefLabel xml:lang="de">Couponing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14428-6">
    <skos:prefLabel xml:lang="en">Rolling stock</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14428-6">
    <skos:prefLabel xml:lang="de">Schienenfahrzeug</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11454-6">
    <skos:broader rdf:resource="thsys/70306"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11454-6">
    <skos:broader rdf:resource="descriptor/11449-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18903-3">
    <skos:broader rdf:resource="thsys/70839"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18903-3">
    <skos:broader rdf:resource="thsys/70573"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18903-3">
    <skos:broader rdf:resource="descriptor/19517-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70648">
    <skos:narrower rdf:resource="descriptor/14189-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70648">
    <skos:narrower rdf:resource="descriptor/14188-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70648">
    <skos:narrower rdf:resource="descriptor/18232-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70648">
    <skos:narrower rdf:resource="descriptor/18233-1"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id304">
    <rdf:value xml:lang="de">Für die Beschreibung einzelner Städte</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18407-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17641-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15951-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13706-2">
    <skos:altLabel xml:lang="de">Private Banking (Privatbank)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13706-2">
    <skos:altLabel xml:lang="de">Privatbankier</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13706-2">
    <skos:altLabel xml:lang="de">Privatbankhaus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71066">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12884-4">
    <skos:altLabel xml:lang="de">Operatives Controlling</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12182-1">
    <skos:altLabel xml:lang="de">GmbH-Gesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12182-1">
    <skos:altLabel xml:lang="de">Gesellschaft mit beschränkter Haftung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12182-1">
    <skos:altLabel xml:lang="de">Einmann-GmbH</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13185-4">
    <skos:broader rdf:resource="thsys/70968"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13185-4">
    <skos:broader rdf:resource="thsys/70964"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13185-4">
    <skos:broader rdf:resource="descriptor/13181-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19319-0">
    <skos:related rdf:resource="descriptor/19310-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19319-0">
    <skos:related rdf:resource="descriptor/19312-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19319-0">
    <skos:related rdf:resource="descriptor/10684-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14972-4">
    <skos:narrower rdf:resource="descriptor/14985-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14972-4">
    <skos:narrower rdf:resource="descriptor/14983-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14972-4">
    <skos:narrower rdf:resource="descriptor/14973-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19608-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15458-6">
    <skos:prefLabel xml:lang="en">Game theory</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15458-6">
    <skos:prefLabel xml:lang="de">Spieltheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18197-0">
    <skos:related rdf:resource="descriptor/19300-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18197-0">
    <skos:related rdf:resource="descriptor/19298-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18197-0">
    <skos:related rdf:resource="descriptor/19297-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18197-0">
    <skos:related rdf:resource="descriptor/14095-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18555-0">
    <skos:related rdf:resource="descriptor/12529-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18555-0">
    <skos:related rdf:resource="descriptor/13200-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17199-1">
    <skos:related rdf:resource="descriptor/19000-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17199-1">
    <skos:related rdf:resource="descriptor/17980-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15057-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19323-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12956-5">
    <zbwext:useInsteadNote rdf:nodeID="id332"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16034-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10201-5">
    <skos:narrower rdf:resource="descriptor/10060-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10201-5">
    <skos:narrower rdf:resource="descriptor/10055-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10201-5">
    <skos:narrower rdf:resource="descriptor/11054-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10201-5">
    <skos:narrower rdf:resource="descriptor/10196-0"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id70">
    <rdf:value xml:lang="de">für organisatorische Dezentralisierung</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15373-0">
    <skos:scopeNote xml:lang="de">Empirische Untersuchung ökonomischer Zusammenhänge unter Anwendung statistischer Methoden.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18218-4">
    <skos:prefLabel xml:lang="en">Poultry meat</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18218-4">
    <skos:prefLabel xml:lang="de">Geflügelfleisch</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10414-2">
    <skos:narrower rdf:resource="descriptor/10541-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10414-2">
    <skos:narrower rdf:resource="descriptor/10413-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10414-2">
    <skos:narrower rdf:resource="descriptor/10412-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10414-2">
    <skos:narrower rdf:resource="descriptor/19363-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10414-2">
    <skos:narrower rdf:resource="descriptor/11943-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10414-2">
    <skos:narrower rdf:resource="descriptor/19243-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10414-2">
    <skos:narrower rdf:resource="descriptor/10517-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10183-2">
    <skos:related rdf:resource="descriptor/10182-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16483-2">
    <skos:prefLabel xml:lang="en">Sports policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16483-2">
    <skos:prefLabel xml:lang="de">Sportpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18814-2">
    <skos:broader rdf:resource="thsys/73363"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18814-2">
    <skos:broader rdf:resource="thsys/70982"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18814-2">
    <skos:broader rdf:resource="thsys/70878"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18814-2">
    <skos:broader rdf:resource="thsys/70865"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18814-2">
    <skos:broader rdf:resource="descriptor/19725-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18814-2">
    <skos:broader rdf:resource="descriptor/10436-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18898-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18897-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10099-5">
    <skos:prefLabel xml:lang="en">Optimal distribution</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10099-5">
    <skos:prefLabel xml:lang="de">Optimale Verteilung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18178-4">
    <skos:related rdf:resource="descriptor/10022-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18178-4">
    <skos:related rdf:resource="descriptor/18061-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18178-4">
    <skos:related rdf:resource="descriptor/10019-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18178-4">
    <skos:related rdf:resource="descriptor/18678-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18178-4">
    <skos:related rdf:resource="descriptor/12251-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16058-3">
    <skos:broader rdf:resource="thsys/73390"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16058-3">
    <skos:broader rdf:resource="thsys/71034"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16058-3">
    <skos:broader rdf:resource="descriptor/15981-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16515-1">
    <skos:prefLabel xml:lang="en">Cultural policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16515-1">
    <skos:prefLabel xml:lang="de">Kulturpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12210-1">
    <skos:related rdf:resource="descriptor/19274-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12210-1">
    <skos:related rdf:resource="descriptor/10210-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12210-1">
    <skos:related rdf:resource="descriptor/13723-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12210-1">
    <skos:related rdf:resource="descriptor/18674-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12210-1">
    <skos:related rdf:resource="descriptor/18138-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18984-5">
    <skos:broader rdf:resource="thsys/73353"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12225-2">
    <skos:scopeNote xml:lang="de">Nur für Kapitalbedarf von Unternehmen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70424">
    <skos:prefLabel xml:lang="de">P.08.01  Kraftmaschinen und Motoren</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11932-3">
    <skos:broader rdf:resource="thsys/71070"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11932-3">
    <skos:broader rdf:resource="thsys/71067"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11932-3">
    <skos:broader rdf:resource="descriptor/11928-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13272-2">
    <skos:related rdf:resource="descriptor/13290-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10184-0">
    <skos:prefLabel xml:lang="en">Time allocation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10184-0">
    <skos:prefLabel xml:lang="de">Zeitallokation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19833-0">
    <skos:prefLabel xml:lang="en">Enterprise resource planning</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19833-0">
    <skos:prefLabel xml:lang="de">Enterprise-Resource-Planning</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19637-0">
    <skos:related rdf:resource="descriptor/13416-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19637-0">
    <skos:related rdf:resource="descriptor/19638-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19637-0">
    <skos:related rdf:resource="descriptor/15904-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10671-3">
    <skos:scopeNote xml:lang="de">Monetäre und Finanzmarktansätze zur Erklärung des Wechselkurses.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19798-4">
    <skos:altLabel xml:lang="de">Mikrotechnologie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19798-4">
    <skos:altLabel xml:lang="de">Mikrotechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11545-3">
    <skos:related rdf:resource="descriptor/11635-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11545-3">
    <skos:related rdf:resource="descriptor/11596-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11545-3">
    <skos:related rdf:resource="descriptor/11613-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15350-5">
    <skos:prefLabel xml:lang="en">Analysis of variance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15350-5">
    <skos:prefLabel xml:lang="de">Varianzanalyse</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10673-6">
    <skos:altLabel xml:lang="en">Theory of optimum currency areas</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10673-6">
    <skos:altLabel xml:lang="en">OCA (Optimum currency area)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10673-6">
    <skos:altLabel xml:lang="de">Theorie der optimalen Währungsräume</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10673-6">
    <skos:altLabel xml:lang="de">Optimale Abgrenzung von Währungsgebieten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10221-6">
    <skos:altLabel xml:lang="en">Futures market</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10221-6">
    <skos:altLabel xml:lang="en">Forward market</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10221-6">
    <skos:altLabel xml:lang="en">Forward contract</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10221-6">
    <skos:altLabel xml:lang="de">Terminmarkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10221-6">
    <skos:altLabel xml:lang="de">Terminkontrakt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10221-6">
    <skos:altLabel xml:lang="de">Terminhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10221-6">
    <skos:altLabel xml:lang="de">Börsentermingeschäft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10298-6">
    <skos:broader rdf:resource="thsys/70814"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10298-6">
    <skos:broader rdf:resource="thsys/70493"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10298-6">
    <skos:broader rdf:resource="descriptor/15909-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16982-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18223-4">
    <skos:prefLabel xml:lang="en">Electronic warfare</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18223-4">
    <skos:prefLabel xml:lang="de">Militärelektronik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13142-1">
    <skos:altLabel xml:lang="de">Schuhmacherhandwerk</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15101-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10008-6">
    <skos:altLabel xml:lang="en">Corporate investment</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10008-6">
    <skos:altLabel xml:lang="en">Capital accumulation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10008-6">
    <skos:altLabel xml:lang="de">Realinvestition</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10008-6">
    <skos:altLabel xml:lang="de">Kapitalbildung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10008-6">
    <skos:altLabel xml:lang="de">Kapitalakkumulation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10008-6">
    <skos:altLabel xml:lang="de">Investitionsquote</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10008-6">
    <skos:altLabel xml:lang="de">Gesamtwirtschaftliche Investition</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10008-6">
    <skos:altLabel xml:lang="de">Betriebliche Investition</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11669-6">
    <skos:related rdf:resource="descriptor/11322-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11669-6">
    <skos:related rdf:resource="descriptor/11660-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70188">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.01.03</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10383-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16158-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14019-2">
    <skos:narrower rdf:resource="descriptor/18282-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14019-2">
    <skos:narrower rdf:resource="descriptor/14924-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14412-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73297">
    <skos:prefLabel xml:lang="de">W.06.01.04.02  Büromaschinen- und Computerindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11071-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14997-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12083-3">
    <skos:prefLabel xml:lang="en">Corporate strategy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12083-3">
    <skos:prefLabel xml:lang="de">Strategische Unternehmensplanung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19065-5">
    <skos:related rdf:resource="descriptor/11859-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19065-5">
    <skos:related rdf:resource="descriptor/11995-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19065-5">
    <skos:related rdf:resource="descriptor/12014-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16809-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18730-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10114-0">
    <skos:broader rdf:resource="thsys/70481"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10114-0">
    <skos:broader rdf:resource="thsys/70233"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10114-0">
    <skos:broader rdf:resource="descriptor/11039-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14345-3">
    <skos:related rdf:resource="descriptor/14334-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12418-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15779-0">
    <skos:altLabel xml:lang="de">Waldsterben</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15779-0">
    <skos:altLabel xml:lang="de">Forstschaden</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15676-3">
    <skos:narrower rdf:resource="descriptor/15619-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15676-3">
    <skos:narrower rdf:resource="descriptor/15642-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15676-3">
    <skos:narrower rdf:resource="descriptor/16777-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10842-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13345-1">
    <skos:altLabel xml:lang="en">Information resources</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13345-1">
    <skos:altLabel xml:lang="de">Informationsprodukt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13345-1">
    <skos:altLabel xml:lang="de">Fachinformationsdienstleistung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12485-4">
    <skos:related rdf:resource="descriptor/12488-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11289-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18838-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10187-1">
    <skos:altLabel xml:lang="de">Kostenkurve</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19277-4">
    <skos:broader rdf:resource="thsys/71108"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19277-4">
    <skos:broader rdf:resource="thsys/71089"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19277-4">
    <skos:broader rdf:resource="descriptor/18670-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19277-4">
    <skos:broader rdf:resource="descriptor/13726-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11274-1">
    <skos:prefLabel xml:lang="en">Highly skilled labour</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11274-1">
    <skos:prefLabel xml:lang="de">Hochqualifizierte Arbeitskräfte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13379-5">
    <skos:scopeNote xml:lang="de">Dienstleistungen, die vor allem im Zusammenhang mit dem Betreiben von EDV-Anlagen, dem Erfassen von Daten und der Beratung im EDV-Bereich erbracht werden</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71104">
    <rdfs:label xml:lang="de">Terminmarkt</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11736-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12656-3">
    <skos:broader rdf:resource="thsys/70368"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12656-3">
    <skos:broader rdf:resource="descriptor/12640-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19462-2">
    <skos:altLabel xml:lang="en">Rural labor market</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11299-6">
    <skos:altLabel xml:lang="de">Laufbahn</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11299-6">
    <skos:altLabel xml:lang="de">Karriere</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11299-6">
    <skos:altLabel xml:lang="de">Berufsweg</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11299-6">
    <skos:altLabel xml:lang="de">Berufsunterbrechung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11299-6">
    <skos:altLabel xml:lang="de">Berufskarriere</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11299-6">
    <skos:altLabel xml:lang="de">Berufsaufstieg</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11299-6">
    <skos:altLabel xml:lang="de">Berufsabstieg</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11299-6">
    <skos:altLabel xml:lang="de">Beförderung im Beruf</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15943-6">
    <skos:altLabel xml:lang="de">Natalität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15943-6">
    <skos:altLabel xml:lang="de">Geburtenziffer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15943-6">
    <skos:altLabel xml:lang="de">Geburtenrückgang</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15943-6">
    <skos:altLabel xml:lang="de">Geburten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19118-3">
    <skos:related rdf:resource="descriptor/12304-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19118-3">
    <skos:related rdf:resource="descriptor/11064-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19118-3">
    <skos:related rdf:resource="descriptor/19138-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19118-3">
    <skos:related rdf:resource="descriptor/10300-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19118-3">
    <skos:related rdf:resource="descriptor/18896-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19118-3">
    <skos:related rdf:resource="descriptor/12656-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11224-2">
    <skos:related rdf:resource="descriptor/19112-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11224-2">
    <skos:related rdf:resource="descriptor/15659-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11224-2">
    <skos:related rdf:resource="descriptor/10120-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11224-2">
    <skos:related rdf:resource="descriptor/12314-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11224-2">
    <skos:related rdf:resource="descriptor/11271-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11224-2">
    <skos:related rdf:resource="descriptor/10491-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11224-2">
    <skos:related rdf:resource="descriptor/11407-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11224-2">
    <skos:related rdf:resource="descriptor/11354-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11224-2">
    <skos:related rdf:resource="descriptor/11222-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11224-2">
    <skos:related rdf:resource="descriptor/19916-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24622-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19162-0">
    <skos:prefLabel xml:lang="en">Coordination</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19162-0">
    <skos:prefLabel xml:lang="de">Koordination</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18686-6">
    <skos:related rdf:resource="descriptor/10801-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13056-1">
    <skos:prefLabel xml:lang="en">Aircraft and aerospace industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13056-1">
    <skos:prefLabel xml:lang="de">Luft- und Raumfahrtindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18889-6">
    <skos:narrower rdf:resource="descriptor/18882-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16955-1">
    <skos:broader rdf:resource="thsys/70438"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16955-1">
    <skos:broader rdf:resource="descriptor/16950-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15036-4">
    <skos:broader rdf:resource="thsys/73103"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15036-4">
    <skos:broader rdf:resource="thsys/70269"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15036-4">
    <skos:broader rdf:resource="descriptor/15045-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14096-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19173-2">
    <skos:prefLabel xml:lang="en">CACM countries</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19173-2">
    <skos:prefLabel xml:lang="de">CACM-Staaten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id251">
    <rdf:value xml:lang="de">Für Kraftfahrzeuge im allgemeinen</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70870">
    <skos:prefLabel xml:lang="de">W.22.01.01  Gebietskörperschaften</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13289-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10994-0">
    <skos:prefLabel xml:lang="en">Economic reform</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10994-0">
    <skos:prefLabel xml:lang="de">Wirtschaftsreform</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18391-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11161-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10629-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19129-5">
    <skos:related rdf:resource="descriptor/19564-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19129-5">
    <skos:related rdf:resource="descriptor/18066-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19129-5">
    <skos:related rdf:resource="descriptor/12801-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19129-5">
    <skos:related rdf:resource="descriptor/16078-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19129-5">
    <skos:related rdf:resource="descriptor/12800-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16661-4">
    <skos:prefLabel xml:lang="en">Minority</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16661-4">
    <skos:prefLabel xml:lang="de">Minderheit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14927-2">
    <skos:related rdf:resource="descriptor/14933-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14927-2">
    <skos:related rdf:resource="descriptor/14930-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14927-2">
    <skos:related rdf:resource="descriptor/18214-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10854-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18972-5">
    <skos:prefLabel xml:lang="en">Family care</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18972-5">
    <skos:prefLabel xml:lang="de">Familienpflege</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71035">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10994-0">
    <skos:editorialNote xml:lang="de">Möglichst spezifischer indexieren</skos:editorialNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15971-1">
    <skos:narrower rdf:resource="descriptor/16087-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15971-1">
    <skos:narrower rdf:resource="descriptor/19844-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16554-5">
    <skos:related rdf:resource="descriptor/15814-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16554-5">
    <skos:related rdf:resource="descriptor/15073-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16554-5">
    <skos:related rdf:resource="descriptor/19744-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16554-5">
    <skos:related rdf:resource="descriptor/16697-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18231-5">
    <skos:prefLabel xml:lang="en">Wind energy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18231-5">
    <skos:prefLabel xml:lang="de">Windenergie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17705-5">
    <skos:broader rdf:resource="thsys/70560"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17705-5">
    <skos:broader rdf:resource="descriptor/17724-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14921-0">
    <skos:prefLabel xml:lang="en">Hosiery</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14921-0">
    <skos:prefLabel xml:lang="de">Strumpfwaren</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10979-3">
    <skos:prefLabel xml:lang="en">Privatization</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10979-3">
    <skos:prefLabel xml:lang="de">Privatisierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14997-2">
    <skos:prefLabel xml:lang="en">Tobacco product</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14997-2">
    <skos:prefLabel xml:lang="de">Tabakwaren</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12543-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71072">
    <rdfs:label xml:lang="de">Öffentliche Wirtschaft</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#member">
    <rdfs:isDefinedBy rdf:resource="http://www.w3.org/2004/02/skos/core"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id39">
    <rdf:value xml:lang="de">für Auslandsniederlassungen</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13290-0">
    <skos:related rdf:resource="descriptor/16072-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13290-0">
    <skos:related rdf:resource="descriptor/13272-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13290-0">
    <skos:related rdf:resource="descriptor/18705-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73373">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.04.03.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14732-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16951-2">
    <skos:broader rdf:resource="thsys/70438"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16951-2">
    <skos:broader rdf:resource="descriptor/16950-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12758-2">
    <skos:altLabel xml:lang="en">Manufacturing planning</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12758-2">
    <skos:altLabel xml:lang="de">Fertigungsplanung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15373-0">
    <skos:narrower rdf:resource="descriptor/15376-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15373-0">
    <skos:narrower rdf:resource="descriptor/10139-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15373-0">
    <skos:narrower rdf:resource="descriptor/18076-5"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id233">
    <rdfs:seeAlso rdf:resource="descriptor/10622-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73357">
    <skos:narrower rdf:resource="descriptor/19762-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73357">
    <skos:narrower rdf:resource="descriptor/25681-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73357">
    <skos:narrower rdf:resource="descriptor/19774-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73357">
    <skos:narrower rdf:resource="descriptor/18970-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73357">
    <skos:narrower rdf:resource="descriptor/19761-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73357">
    <skos:narrower rdf:resource="descriptor/18969-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73357">
    <skos:narrower rdf:resource="descriptor/19760-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73357">
    <skos:narrower rdf:resource="descriptor/15925-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73357">
    <skos:narrower rdf:resource="descriptor/16700-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73357">
    <skos:narrower rdf:resource="descriptor/15924-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73357">
    <skos:narrower rdf:resource="descriptor/16699-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73357">
    <skos:narrower rdf:resource="descriptor/16698-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73357">
    <skos:narrower rdf:resource="descriptor/15938-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73357">
    <skos:narrower rdf:resource="descriptor/18861-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73357">
    <skos:narrower rdf:resource="descriptor/19759-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73357">
    <skos:narrower rdf:resource="descriptor/15937-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73357">
    <skos:narrower rdf:resource="descriptor/16697-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73357">
    <skos:narrower rdf:resource="descriptor/15904-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73357">
    <skos:narrower rdf:resource="descriptor/18971-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73357">
    <skos:narrower rdf:resource="descriptor/18968-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10706-3">
    <skos:broader rdf:resource="thsys/71098"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10706-3">
    <skos:broader rdf:resource="descriptor/10683-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15962-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14098-1">
    <skos:prefLabel xml:lang="en">Wheat</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14098-1">
    <skos:prefLabel xml:lang="de">Weizen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18687-4">
    <skos:prefLabel xml:lang="en">Stock option</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18687-4">
    <skos:prefLabel xml:lang="de">Aktienoption</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20455-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14722-6">
    <skos:altLabel xml:lang="en">GMO (Genetically modified organism)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14722-6">
    <skos:altLabel xml:lang="en">Genetically modified organism</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14722-6">
    <skos:altLabel xml:lang="de">Vitamin</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14722-6">
    <skos:altLabel xml:lang="de">Plasmaprodukt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14722-6">
    <skos:altLabel xml:lang="de">Kohlenhydrat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14722-6">
    <skos:altLabel xml:lang="de">Gentechnisches Produkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14722-6">
    <skos:altLabel xml:lang="de">Genforschungsprodukt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14722-6">
    <skos:altLabel xml:lang="de">Enzymbildner</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14722-6">
    <skos:altLabel xml:lang="de">Enzym</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14722-6">
    <skos:altLabel xml:lang="de">Biopharmazeutikum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14722-6">
    <skos:altLabel xml:lang="de">Biochemikalie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19401-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18220-3">
    <skos:broader rdf:resource="thsys/73302"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18220-3">
    <skos:broader rdf:resource="descriptor/14122-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17639-6">
    <skos:altLabel xml:lang="x-other">Umm al Kaiwain</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17639-6">
    <skos:altLabel xml:lang="x-other">Ras al Chaima</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17639-6">
    <skos:altLabel xml:lang="en">UAE (United Arab Emirates)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17639-6">
    <skos:altLabel xml:lang="de">Schardscha</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17639-6">
    <skos:altLabel xml:lang="de">Fudschaira</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17639-6">
    <skos:altLabel xml:lang="de">Dubaij</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17639-6">
    <skos:altLabel xml:lang="de">Dubai</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17639-6">
    <skos:altLabel xml:lang="de">Adschman</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17639-6">
    <skos:altLabel xml:lang="de">Abu Dhabi</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11111-1">
    <skos:broader rdf:resource="thsys/73394"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11111-1">
    <skos:broader rdf:resource="thsys/70178"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11111-1">
    <skos:broader rdf:resource="descriptor/10144-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13633-3">
    <skos:altLabel xml:lang="en">P+R (Park and Ride)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13633-3">
    <skos:altLabel xml:lang="en">Park and ride</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13633-3">
    <skos:altLabel xml:lang="de">Parkraum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13633-3">
    <skos:altLabel xml:lang="de">Parkplatz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13633-3">
    <skos:altLabel xml:lang="de">Parkierungsanlage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13633-3">
    <skos:altLabel xml:lang="de">Parkgelegenheit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13633-3">
    <skos:altLabel xml:lang="de">Garage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13633-3">
    <skos:altLabel xml:lang="de">Anwohnerparken</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16909-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11246-6">
    <skos:scopeNote xml:lang="de">Sicherheit und Sicherung von Arbeitsplätzen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10045-0">
    <skos:broader rdf:resource="thsys/72393"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10045-0">
    <skos:broader rdf:resource="thsys/70049"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10045-0">
    <skos:broader rdf:resource="thsys/70038"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10045-0">
    <skos:broader rdf:resource="descriptor/15376-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10045-0">
    <skos:broader rdf:resource="descriptor/10035-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15914-6">
    <skos:narrower rdf:resource="descriptor/19478-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15914-6">
    <skos:narrower rdf:resource="descriptor/16661-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15914-6">
    <skos:narrower rdf:resource="descriptor/11271-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15914-6">
    <skos:narrower rdf:resource="descriptor/15905-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19347-2">
    <skos:broader rdf:resource="thsys/71025"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19347-2">
    <skos:broader rdf:resource="thsys/70960"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19347-2">
    <skos:broader rdf:resource="descriptor/13431-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12194-1">
    <skos:altLabel xml:lang="de">Unternehmensform</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13530-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11553-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11610-4">
    <skos:altLabel xml:lang="de">Kommunalkredit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11610-4">
    <skos:altLabel xml:lang="de">Gemeindeschulden</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11610-4">
    <skos:altLabel xml:lang="de">Gemeindekredit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16525-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10180-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17978-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71065">
    <rdfs:label xml:lang="en">Rural economy</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71065">
    <rdfs:label xml:lang="de">Ländliche Wirtschaft</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18977-2">
    <skos:broader rdf:resource="thsys/71054"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18977-2">
    <skos:broader rdf:resource="descriptor/18535-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14434-4">
    <skos:altLabel xml:lang="de">Wohnwagen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14434-4">
    <skos:altLabel xml:lang="de">Kfz-Anhänger</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14434-4">
    <skos:altLabel xml:lang="de">Campinganhänger</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12058-2">
    <skos:broader rdf:resource="thsys/71034"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12194-1">
    <skos:broader rdf:resource="thsys/73391"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12194-1">
    <skos:broader rdf:resource="thsys/71040"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12194-1">
    <skos:broader rdf:resource="thsys/70087"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11268-3">
    <skos:related rdf:resource="descriptor/11258-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11268-3">
    <skos:related rdf:resource="descriptor/19360-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11268-3">
    <skos:related rdf:resource="descriptor/10613-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70455">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11696-3">
    <skos:narrower rdf:resource="descriptor/11695-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11696-3">
    <skos:narrower rdf:resource="descriptor/19411-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11696-3">
    <skos:narrower rdf:resource="descriptor/11692-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11696-3">
    <skos:narrower rdf:resource="descriptor/11685-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11352-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19170-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12868-2">
    <skos:prefLabel xml:lang="en">Taxation of company reorganization</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12868-2">
    <skos:prefLabel xml:lang="de">Umwandlungssteuer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15860-3">
    <skos:scopeNote xml:lang="de">Modell des wirtschaftlich handelnden Menschen im Sinne des Rationalprinzips.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12419-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19374-6">
    <skos:related rdf:resource="descriptor/18156-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19374-6">
    <skos:related rdf:resource="descriptor/11613-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19374-6">
    <skos:related rdf:resource="descriptor/19375-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12997-5">
    <skos:broader rdf:resource="thsys/70994"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12997-5">
    <skos:broader rdf:resource="descriptor/12996-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11310-2">
    <skos:narrower rdf:resource="descriptor/18159-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11310-2">
    <skos:narrower rdf:resource="descriptor/11306-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19437-1">
    <skos:related rdf:resource="descriptor/11983-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19437-1">
    <skos:related rdf:resource="descriptor/19447-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19437-1">
    <skos:related rdf:resource="descriptor/18060-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18913-0">
    <skos:narrower rdf:resource="descriptor/16547-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18913-0">
    <skos:narrower rdf:resource="descriptor/16539-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18913-0">
    <skos:narrower rdf:resource="descriptor/19517-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70384">
    <skos:prefLabel xml:lang="de">B.07.03  Verkaufsförderung und Werbung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14851-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17568-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14926-4">
    <skos:related rdf:resource="descriptor/18892-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14926-4">
    <skos:related rdf:resource="descriptor/16635-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13645-3">
    <skos:broader rdf:resource="thsys/70924"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13645-3">
    <skos:broader rdf:resource="thsys/70013"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13645-3">
    <skos:broader rdf:resource="descriptor/18916-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13160-6">
    <skos:narrower rdf:resource="descriptor/13165-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13160-6">
    <skos:narrower rdf:resource="descriptor/13161-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13160-6">
    <skos:narrower rdf:resource="descriptor/13164-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13160-6">
    <skos:narrower rdf:resource="descriptor/13163-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13160-6">
    <skos:narrower rdf:resource="descriptor/13153-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13160-6">
    <skos:narrower rdf:resource="descriptor/13159-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13160-6">
    <skos:narrower rdf:resource="descriptor/13158-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13160-6">
    <skos:narrower rdf:resource="descriptor/18518-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13160-6">
    <skos:narrower rdf:resource="descriptor/13157-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13160-6">
    <skos:narrower rdf:resource="descriptor/13156-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13160-6">
    <skos:narrower rdf:resource="descriptor/13155-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13160-6">
    <skos:narrower rdf:resource="descriptor/13940-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13160-6">
    <skos:narrower rdf:resource="descriptor/13154-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13160-6">
    <skos:narrower rdf:resource="descriptor/18440-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10555-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14030-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20022-6">
    <skos:related rdf:resource="descriptor/16732-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20022-6">
    <skos:related rdf:resource="descriptor/16720-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20022-6">
    <skos:related rdf:resource="descriptor/19989-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17947-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71112">
    <skos:narrower rdf:resource="descriptor/15108-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71112">
    <skos:narrower rdf:resource="descriptor/18976-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71112">
    <skos:narrower rdf:resource="descriptor/13780-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71112">
    <skos:narrower rdf:resource="descriptor/19236-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71112">
    <skos:narrower rdf:resource="descriptor/10285-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71112">
    <skos:narrower rdf:resource="descriptor/10284-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71112">
    <skos:narrower rdf:resource="descriptor/11571-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71112">
    <skos:narrower rdf:resource="descriptor/10020-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71112">
    <skos:narrower rdf:resource="descriptor/10179-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71112">
    <skos:narrower rdf:resource="descriptor/10183-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71112">
    <skos:narrower rdf:resource="descriptor/10267-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71112">
    <skos:narrower rdf:resource="descriptor/19552-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71112">
    <skos:narrower rdf:resource="descriptor/10302-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71112">
    <skos:narrower rdf:resource="descriptor/10181-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71112">
    <skos:narrower rdf:resource="descriptor/10376-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71112">
    <skos:narrower rdf:resource="descriptor/10172-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71112">
    <skos:narrower rdf:resource="descriptor/10301-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71112">
    <skos:narrower rdf:resource="descriptor/10300-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71112">
    <skos:narrower rdf:resource="descriptor/10010-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71112">
    <skos:narrower rdf:resource="descriptor/11432-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71112">
    <skos:narrower rdf:resource="descriptor/10178-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71112">
    <skos:narrower rdf:resource="descriptor/10276-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71112">
    <skos:narrower rdf:resource="descriptor/10278-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71112">
    <skos:narrower rdf:resource="descriptor/10282-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71112">
    <skos:narrower rdf:resource="descriptor/10170-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13561-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14698-5">
    <zbwext:useInsteadNote rdf:nodeID="id81"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12278-2">
    <skos:prefLabel xml:lang="en">Capital good</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12278-2">
    <skos:prefLabel xml:lang="de">Investitionsgut</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11988-4">
    <skos:related rdf:resource="descriptor/19363-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11988-4">
    <skos:related rdf:resource="descriptor/11986-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11988-4">
    <skos:related rdf:resource="descriptor/11968-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11988-4">
    <skos:related rdf:resource="descriptor/19612-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11988-4">
    <skos:related rdf:resource="descriptor/19457-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11988-4">
    <skos:related rdf:resource="descriptor/11966-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11988-4">
    <skos:related rdf:resource="descriptor/11979-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11988-4">
    <skos:related rdf:resource="descriptor/11919-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18895-4">
    <skos:prefLabel xml:lang="en">Traditional medicine</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18895-4">
    <skos:prefLabel xml:lang="de">Traditionelle Medizin</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17979-3">
    <skos:related rdf:resource="descriptor/17589-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17979-3">
    <skos:related rdf:resource="descriptor/19172-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18401-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10984-3">
    <skos:related rdf:resource="descriptor/16038-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10984-3">
    <skos:related rdf:resource="descriptor/15714-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10984-3">
    <skos:related rdf:resource="descriptor/15711-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10984-3">
    <skos:related rdf:resource="descriptor/16650-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10984-3">
    <skos:related rdf:resource="descriptor/16287-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10984-3">
    <skos:related rdf:resource="descriptor/16379-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10984-3">
    <skos:related rdf:resource="descriptor/18847-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10984-3">
    <skos:related rdf:resource="descriptor/16384-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10984-3">
    <skos:related rdf:resource="descriptor/10968-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10984-3">
    <skos:related rdf:resource="descriptor/11161-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10984-3">
    <skos:related rdf:resource="descriptor/11168-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10984-3">
    <skos:related rdf:resource="descriptor/13988-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12946-1">
    <skos:related rdf:resource="descriptor/12952-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12946-1">
    <skos:related rdf:resource="descriptor/19343-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12946-1">
    <skos:related rdf:resource="descriptor/12950-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12946-1">
    <skos:related rdf:resource="descriptor/13158-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12946-1">
    <skos:related rdf:resource="descriptor/19545-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12946-1">
    <skos:related rdf:resource="descriptor/19183-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12946-1">
    <skos:related rdf:resource="descriptor/14124-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16815-3">
    <skos:broader rdf:resource="thsys/70002"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10525-0">
    <skos:prefLabel xml:lang="en">Development theory</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10525-0">
    <skos:prefLabel xml:lang="de">Entwicklungstheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20855-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12259-6">
    <skos:narrower rdf:resource="descriptor/12265-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73104">
    <skos:broader rdf:resource="thsys/v"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13975-3">
    <skos:prefLabel xml:lang="en">Catering professions</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13975-3">
    <skos:prefLabel xml:lang="de">Gastgewerbeberufe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18627-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11599-1">
    <skos:scopeNote xml:lang="de">Finanzausgleich zwischen öffentlichen Körperschaften. Verteilung öffentlicher Einnahmen und Ausgaben auf die öffentlichen Körperschaften.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13690-5">
    <skos:related rdf:resource="descriptor/13697-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13690-5">
    <skos:related rdf:resource="descriptor/13291-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13690-5">
    <skos:related rdf:resource="descriptor/10905-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13690-5">
    <skos:related rdf:resource="descriptor/13693-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13690-5">
    <skos:related rdf:resource="descriptor/18707-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13690-5">
    <skos:related rdf:resource="descriptor/13283-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18222-6">
    <skos:prefLabel xml:lang="en">Luxury foodstuffs and tobacco</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18222-6">
    <skos:prefLabel xml:lang="de">Genussmittel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12174-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13558-5">
    <skos:related rdf:resource="descriptor/13564-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13558-5">
    <skos:related rdf:resource="descriptor/18585-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13558-5">
    <skos:related rdf:resource="descriptor/13239-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13558-5">
    <skos:related rdf:resource="descriptor/13244-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13558-5">
    <skos:related rdf:resource="descriptor/13051-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13558-5">
    <skos:related rdf:resource="descriptor/14433-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13558-5">
    <skos:related rdf:resource="descriptor/13813-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13558-5">
    <skos:related rdf:resource="descriptor/18423-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13558-5">
    <skos:related rdf:resource="descriptor/13342-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13558-5">
    <skos:related rdf:resource="descriptor/13248-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19491-2">
    <skos:prefLabel xml:lang="en">Precarious employment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19491-2">
    <skos:prefLabel xml:lang="de">Ungeschützte Beschäftigung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18168-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13245-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10412-6">
    <skos:prefLabel xml:lang="en">Stagflation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10412-6">
    <skos:prefLabel xml:lang="de">Stagflation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13295-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15738-0">
    <skos:narrower rdf:resource="descriptor/19480-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15738-0">
    <skos:narrower rdf:resource="descriptor/20967-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15738-0">
    <skos:narrower rdf:resource="descriptor/20888-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15738-0">
    <skos:narrower rdf:resource="descriptor/20762-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15738-0">
    <skos:narrower rdf:resource="descriptor/20744-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15738-0">
    <skos:narrower rdf:resource="descriptor/20722-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14627-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12186-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19283-2">
    <skos:altLabel xml:lang="de">Wettbewerb der Institutionen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19283-2">
    <skos:altLabel xml:lang="de">Tiebout-Modell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10251-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18956-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16038-2">
    <skos:scopeNote xml:lang="de">Regelungen des öffentlichen und des Privatrechts.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18440-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10392-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10828-3">
    <skos:broader rdf:resource="thsys/73337"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10828-3">
    <skos:broader rdf:resource="thsys/73332"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10828-3">
    <skos:broader rdf:resource="thsys/71075"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10828-3">
    <skos:broader rdf:resource="thsys/71073"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10828-3">
    <skos:broader rdf:resource="thsys/71040"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10828-3">
    <skos:broader rdf:resource="descriptor/10829-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14144-6">
    <skos:broader rdf:resource="thsys/70403"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14144-6">
    <skos:broader rdf:resource="descriptor/14145-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10387-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10575-6">
    <skos:altLabel xml:lang="en">Community development</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13368-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11042-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19204-3">
    <skos:altLabel xml:lang="de">Evolutorische Ökonomik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19204-3">
    <skos:altLabel xml:lang="de">Evolutionsökonomie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10813-2">
    <skos:related rdf:resource="descriptor/19360-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10813-2">
    <skos:related rdf:resource="descriptor/10836-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10629-2">
    <skos:prefLabel xml:lang="en">International economics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10629-2">
    <skos:prefLabel xml:lang="de">Außenwirtschaftstheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19325-5">
    <skos:prefLabel xml:lang="en">Social clauses</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19325-5">
    <skos:prefLabel xml:lang="de">Sozialklausel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id83">
    <rdf:value xml:lang="de">für die Branche</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12823-3">
    <skos:prefLabel xml:lang="en">Outdoor advertising</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12823-3">
    <skos:prefLabel xml:lang="de">Außenwerbung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18576-6">
    <skos:prefLabel xml:lang="en">Urban rail transit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18576-6">
    <skos:prefLabel xml:lang="de">Schienenpersonennahverkehr</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70865">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10263-4">
    <skos:altLabel xml:lang="de">Pigou-Effekt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10263-4">
    <skos:altLabel xml:lang="de">Keynes-Effekt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10263-4">
    <skos:altLabel xml:lang="de">Geldeffekt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70928">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71079">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19273-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11821-5">
    <skos:scopeNote xml:lang="de">Analyse des industriellen Sektors nach verschiedenen Kriterien, z.B. Zahl, Größe und Rechtsform der Unternehmen; Beschäftigung; Marktanteil einzelner Industriezweige.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18311-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15929-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15806-2">
    <skos:broader rdf:resource="thsys/73358"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10147-6">
    <skos:altLabel xml:lang="en">Theory of search</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10147-6">
    <skos:altLabel xml:lang="en">Stopping-rule-problem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10147-6">
    <skos:altLabel xml:lang="en">Stopping-rule problem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10147-6">
    <skos:altLabel xml:lang="en">Sequential search</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10147-6">
    <skos:altLabel xml:lang="en">Optimal search</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10147-6">
    <skos:altLabel xml:lang="en">Economics of search</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10147-6">
    <skos:altLabel xml:lang="de">Suchmodell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10147-6">
    <skos:altLabel xml:lang="de">Stopproblem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10147-6">
    <skos:altLabel xml:lang="de">Stopp-Problem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10147-6">
    <skos:altLabel xml:lang="de">Optimales Stoppen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16497-5">
    <skos:prefLabel xml:lang="en">Peace-keeping</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16497-5">
    <skos:prefLabel xml:lang="de">Friedenssicherung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18223-4">
    <skos:altLabel xml:lang="de">Rüstungselektronik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18223-4">
    <skos:altLabel xml:lang="de">Elektronisches Kriegsgerät</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15976-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18359-0">
    <skos:related rdf:resource="descriptor/18361-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18359-0">
    <skos:related rdf:resource="descriptor/18410-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18359-0">
    <skos:related rdf:resource="descriptor/14039-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18359-0">
    <skos:related rdf:resource="descriptor/14483-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18359-0">
    <skos:related rdf:resource="descriptor/14479-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18359-0">
    <skos:related rdf:resource="descriptor/14431-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18359-0">
    <skos:related rdf:resource="descriptor/14174-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14808-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18831-2">
    <skos:related rdf:resource="descriptor/16038-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18831-2">
    <skos:related rdf:resource="descriptor/18099-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18831-2">
    <skos:related rdf:resource="descriptor/18097-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18831-2">
    <skos:related rdf:resource="descriptor/12047-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19563-3">
    <skos:altLabel xml:lang="en">Liquidity constraints</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19563-3">
    <skos:altLabel xml:lang="en">Liquidity constraint</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19563-3">
    <skos:altLabel xml:lang="en">Credit constraints</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19563-3">
    <skos:altLabel xml:lang="de">Liquiditätsrestriktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11969-1">
    <skos:broader rdf:resource="thsys/71064"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11969-1">
    <skos:broader rdf:resource="thsys/70888"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11969-1">
    <skos:broader rdf:resource="descriptor/11973-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18276-4">
    <skos:altLabel xml:lang="de">Einbauküchenmöbel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10207-0">
    <skos:broader rdf:resource="thsys/70892"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10207-0">
    <skos:broader rdf:resource="thsys/70180"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18293-4">
    <skos:broader rdf:resource="thsys/70647"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18293-4">
    <skos:broader rdf:resource="descriptor/14766-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70002">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">G.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70961">
    <skos:prefLabel xml:lang="en">W.10.01.01  Motor vehicle trade</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70961">
    <skos:prefLabel xml:lang="de">W.10.01.01  Kraftfahrzeughandel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17770-1">
    <skos:broader rdf:resource="thsys/70086"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17770-1">
    <skos:broader rdf:resource="descriptor/17791-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15376-1">
    <skos:related rdf:resource="descriptor/15340-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15376-1">
    <skos:related rdf:resource="descriptor/15372-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15376-1">
    <skos:related rdf:resource="descriptor/15381-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19043-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18582-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15767-0">
    <skos:prefLabel xml:lang="en">Environmental economics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15767-0">
    <skos:prefLabel xml:lang="de">Umweltökonomik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18098-2">
    <skos:broader rdf:resource="thsys/73384"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18098-2">
    <skos:broader rdf:resource="thsys/70384"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18098-2">
    <skos:broader rdf:resource="descriptor/12846-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13002-3">
    <skos:narrower rdf:resource="descriptor/18260-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18389-5">
    <skos:broader rdf:resource="thsys/71009"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18389-5">
    <skos:broader rdf:resource="descriptor/12917-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18389-5">
    <skos:broader rdf:resource="descriptor/12980-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73337">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10248-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11216-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16980-2">
    <skos:prefLabel xml:lang="en">Zurich (Canton)</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16980-2">
    <skos:prefLabel xml:lang="de">Zürich (Kanton)</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14231-4">
    <skos:altLabel xml:lang="de">Syenit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19592-3">
    <skos:broader rdf:resource="thsys/73104"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19592-3">
    <skos:broader rdf:resource="descriptor/15060-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19592-3">
    <skos:broader rdf:resource="descriptor/15458-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11426-4">
    <skos:prefLabel xml:lang="en">Monetary circulation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11426-4">
    <skos:prefLabel xml:lang="de">Geldumlauf</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14092-6">
    <skos:related rdf:resource="descriptor/19193-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16170-2">
    <skos:prefLabel xml:lang="en">Air law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16170-2">
    <skos:prefLabel xml:lang="de">Luftrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18383-3">
    <skos:related rdf:resource="descriptor/12051-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18383-3">
    <skos:related rdf:resource="descriptor/18390-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18383-3">
    <skos:related rdf:resource="descriptor/12980-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="">
    <rdf:type rdf:resource="http://www.w3.org/2002/07/owl#Ontology"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19506-1">
    <skos:prefLabel xml:lang="en">Subsistence minimum</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19506-1">
    <skos:prefLabel xml:lang="de">Existenzminimum</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12615-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71062">
    <skos:broader rdf:resource="thsys/70051"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11239-3">
    <skos:narrower rdf:resource="descriptor/11246-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20022-6">
    <skos:prefLabel xml:lang="en">Positive economics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20022-6">
    <skos:prefLabel xml:lang="de">Positive Ökonomik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19148-1">
    <skos:prefLabel xml:lang="en">Multimoment analysis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19148-1">
    <skos:prefLabel xml:lang="de">Multimomentverfahren</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19184-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16674-2">
    <skos:related rdf:resource="descriptor/16696-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16674-2">
    <skos:related rdf:resource="descriptor/19765-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16674-2">
    <skos:related rdf:resource="descriptor/15721-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16674-2">
    <skos:related rdf:resource="descriptor/15718-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16674-2">
    <skos:related rdf:resource="descriptor/19722-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16674-2">
    <skos:related rdf:resource="descriptor/15713-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10809-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12594-6">
    <skos:narrower rdf:resource="descriptor/20368-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12594-6">
    <skos:narrower rdf:resource="descriptor/12590-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19279-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70816">
    <skos:prefLabel xml:lang="en">W.32.02  Recycling</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70816">
    <skos:prefLabel xml:lang="de">W.32.02  Recycling</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70992">
    <skos:broader rdf:resource="thsys/70994"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10711-3">
    <skos:scopeNote xml:lang="de">Zollfreigebiete, in denen importiert und produziert werden kann, ohne dass Zölle anfallen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18364-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20897-6">
    <skos:related rdf:resource="descriptor/12013-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20897-6">
    <skos:related rdf:resource="descriptor/18617-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20897-6">
    <skos:related rdf:resource="descriptor/13516-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15676-3">
    <skos:prefLabel xml:lang="en">Language</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15676-3">
    <skos:prefLabel xml:lang="de">Sprache</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19752-0">
    <skos:broader rdf:resource="thsys/70321"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19752-0">
    <skos:broader rdf:resource="descriptor/12009-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18806-1">
    <skos:broader rdf:resource="thsys/71064"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18806-1">
    <skos:broader rdf:resource="descriptor/11556-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70867">
    <skos:broader rdf:resource="thsys/70871"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18107-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17567-5">
    <skos:related rdf:resource="descriptor/17565-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11422-5">
    <skos:prefLabel xml:lang="en">Demand for money</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11422-5">
    <skos:prefLabel xml:lang="de">Geldnachfrage</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18122-3">
    <skos:altLabel xml:lang="de">Industriebauten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18122-3">
    <skos:altLabel xml:lang="de">Gewerbliche Immobilien</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18122-3">
    <skos:altLabel xml:lang="de">Gewerbe-Immobilien</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18122-3">
    <skos:altLabel xml:lang="de">Gewerbebauten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18122-3">
    <skos:altLabel xml:lang="de">Betriebsgrundstück</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70868">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14765-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19102-4">
    <skos:related rdf:resource="descriptor/19835-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19102-4">
    <skos:related rdf:resource="descriptor/19154-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19102-4">
    <skos:related rdf:resource="descriptor/13714-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18378-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10856-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10097-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12083-3">
    <skos:altLabel xml:lang="en">Business strategy</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12083-3">
    <skos:altLabel xml:lang="de">Unternehmensstrategie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12083-3">
    <skos:altLabel xml:lang="de">Langfristige Unternehmensplanung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15599-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11853-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19270-4">
    <skos:narrower rdf:resource="descriptor/19417-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11391-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18743-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70939">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18262-1">
    <skos:related rdf:resource="descriptor/14734-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10323-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16684-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18249-0">
    <skos:altLabel xml:lang="de">Fernkopiergerät</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18249-0">
    <skos:altLabel xml:lang="de">Faxgerät</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18517-1">
    <skos:altLabel xml:lang="de">Getreidegroßhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16042-4">
    <skos:related rdf:resource="descriptor/16041-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11272-5">
    <skos:prefLabel xml:lang="en">Labour force participation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11272-5">
    <skos:prefLabel xml:lang="de">Erwerbstätigkeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17988-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12760-1">
    <skos:prefLabel xml:lang="en">Production control</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12760-1">
    <skos:prefLabel xml:lang="de">Fertigungssteuerung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70832">
    <rdfs:label xml:lang="en">Social insurance</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70832">
    <rdfs:label xml:lang="de">Sozialversicherung</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12416-2">
    <skos:prefLabel xml:lang="en">Assessed value</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12416-2">
    <skos:prefLabel xml:lang="de">Einheitswert</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17749-6">
    <skos:related rdf:resource="descriptor/17753-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13502-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14796-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70994">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.06.01.06</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70905">
    <skos:broader rdf:resource="thsys/w"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11729-0">
    <skos:scopeNote xml:lang="de">Reform des öffentlichen Finanz- und Haushaltswesens.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10884-0">
    <skos:related rdf:resource="descriptor/13743-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10884-0">
    <skos:related rdf:resource="descriptor/18382-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10884-0">
    <skos:related rdf:resource="descriptor/18995-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10884-0">
    <skos:related rdf:resource="descriptor/13472-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10884-0">
    <skos:related rdf:resource="descriptor/18690-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10884-0">
    <skos:related rdf:resource="descriptor/13188-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10884-0">
    <skos:related rdf:resource="descriptor/10803-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15646-5">
    <skos:prefLabel xml:lang="en">Hinduism</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15646-5">
    <skos:prefLabel xml:lang="de">Hinduismus</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18515-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13188-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19205-1">
    <skos:altLabel xml:lang="de">Feministischer Ansatz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19205-1">
    <skos:altLabel xml:lang="de">Feministische Wirtschaftstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11911-4">
    <skos:related rdf:resource="descriptor/19234-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11911-4">
    <skos:related rdf:resource="descriptor/11895-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11911-4">
    <skos:related rdf:resource="descriptor/11872-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11911-4">
    <skos:related rdf:resource="descriptor/19242-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14011-4">
    <skos:altLabel xml:lang="de">Physikerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14011-4">
    <skos:altLabel xml:lang="de">Physiker</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14011-4">
    <skos:altLabel xml:lang="de">Naturwissenschaftlerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14011-4">
    <skos:altLabel xml:lang="de">Geologinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14011-4">
    <skos:altLabel xml:lang="de">Geologen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14011-4">
    <skos:altLabel xml:lang="de">Geographinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14011-4">
    <skos:altLabel xml:lang="de">Geographen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14011-4">
    <skos:altLabel xml:lang="de">Biowissenschaftlerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14011-4">
    <skos:altLabel xml:lang="de">Biowissenschaftler</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14011-4">
    <skos:altLabel xml:lang="de">Biologinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14011-4">
    <skos:altLabel xml:lang="de">Biologen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16484-0">
    <skos:scopeNote xml:lang="de">Auch für allgemeine Tarifverhandlungen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17571-0">
    <skos:prefLabel xml:lang="en">East Asia</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17571-0">
    <skos:prefLabel xml:lang="de">Ostasien</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10035-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70578">
    <skos:prefLabel xml:lang="de">B.03.02  Kosten- und Leistungsrechnung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17177-4">
    <skos:altLabel xml:lang="de">Sozialistische Republik Rumänien</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11729-0">
    <skos:altLabel xml:lang="de">Haushaltsreform</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14065-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11628-6">
    <skos:altLabel xml:lang="en">State owned enterprise</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11628-6">
    <skos:altLabel xml:lang="en">State enterprise</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11628-6">
    <skos:altLabel xml:lang="en">SOE (State owned enterprise)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11628-6">
    <skos:altLabel xml:lang="de">Staatsunternehmen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11628-6">
    <skos:altLabel xml:lang="de">Staatsbetrieb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11628-6">
    <skos:altLabel xml:lang="de">Regiebetrieb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11628-6">
    <skos:altLabel xml:lang="de">Öffentlicher Betrieb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11628-6">
    <skos:altLabel xml:lang="de">Öffentliche Wirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10271-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15376-1">
    <skos:altLabel xml:lang="en">Single-equation model</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15376-1">
    <skos:altLabel xml:lang="de">Ökonometrisches Einzelgleichungsmodell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10087-5">
    <skos:related rdf:resource="descriptor/10082-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10087-5">
    <skos:related rdf:resource="descriptor/10247-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10087-5">
    <skos:related rdf:resource="descriptor/10076-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13051-4">
    <skos:related rdf:resource="descriptor/13558-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13051-4">
    <skos:related rdf:resource="descriptor/18422-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13051-4">
    <skos:related rdf:resource="descriptor/18421-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13051-4">
    <skos:related rdf:resource="descriptor/19549-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13051-4">
    <skos:related rdf:resource="descriptor/13052-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13051-4">
    <skos:related rdf:resource="descriptor/13217-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13051-4">
    <skos:related rdf:resource="descriptor/14433-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13051-4">
    <skos:related rdf:resource="descriptor/18423-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18867-2">
    <skos:related rdf:resource="descriptor/15429-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18867-2">
    <skos:related rdf:resource="descriptor/12110-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18867-2">
    <skos:related rdf:resource="descriptor/12619-2"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id346">
    <rdf:value xml:lang="de">Für Produktionstechniken</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15720-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70245">
    <skos:broader rdf:resource="thsys/70113"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16325-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id203">
    <rdf:value xml:lang="de">Für die Segmentierung des Arbeitsmarktes</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18803-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12495-1">
    <skos:related rdf:resource="descriptor/12432-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18588-6">
    <skos:broader rdf:resource="thsys/70936"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18588-6">
    <skos:broader rdf:resource="descriptor/13256-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18588-6">
    <skos:broader rdf:resource="descriptor/13504-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13004-6">
    <skos:altLabel xml:lang="de">Kautschukindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13841-3">
    <skos:prefLabel xml:lang="en">Housing costs</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13841-3">
    <skos:prefLabel xml:lang="de">Wohnkosten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19363-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12105-5">
    <skos:altLabel xml:lang="de">Organisationswandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12105-5">
    <skos:altLabel xml:lang="de">Organisationsänderung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12105-5">
    <skos:altLabel xml:lang="de">Geplanter organisatorischer Wandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19689-2">
    <skos:broader rdf:resource="thsys/73381"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19091-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19413-1">
    <skos:broader rdf:resource="thsys/71075"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19413-1">
    <skos:broader rdf:resource="descriptor/11704-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16266-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11937-0">
    <skos:narrower rdf:resource="descriptor/19445-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16777-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71098">
    <skos:broader rdf:resource="thsys/70179"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18651-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14991-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71072">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.09.09</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14930-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18660-3">
    <skos:altLabel xml:lang="en">EDIFACT</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18660-3">
    <skos:altLabel xml:lang="en">EDI (Electronic Data Interchange)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18660-3">
    <skos:altLabel xml:lang="de">Elektronischer Dokumentenaustausch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18660-3">
    <skos:altLabel xml:lang="de">Elektronische Datenübermittlung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11650-6">
    <skos:narrower rdf:resource="descriptor/12868-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11650-6">
    <skos:narrower rdf:resource="descriptor/11669-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11650-6">
    <skos:narrower rdf:resource="descriptor/11663-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11650-6">
    <skos:narrower rdf:resource="descriptor/19415-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11951-6">
    <skos:prefLabel xml:lang="en">Regional science</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11951-6">
    <skos:prefLabel xml:lang="de">Regionalforschung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18111-1">
    <skos:narrower rdf:resource="descriptor/18112-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18155-2">
    <skos:scopeNote xml:lang="de">"Socially wasteful competition" bzw. das Streben von Verteilungskoalitionen nach Renten.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12619-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11180-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11533-3">
    <skos:related rdf:resource="descriptor/11540-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18660-3">
    <skos:narrower rdf:resource="descriptor/13652-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70992">
    <skos:narrower rdf:resource="descriptor/13001-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70992">
    <skos:narrower rdf:resource="descriptor/12920-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70992">
    <skos:narrower rdf:resource="descriptor/19426-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70992">
    <skos:narrower rdf:resource="descriptor/11673-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70992">
    <skos:narrower rdf:resource="descriptor/19182-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70992">
    <skos:narrower rdf:resource="descriptor/19387-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70992">
    <skos:narrower rdf:resource="descriptor/19427-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70992">
    <skos:narrower rdf:resource="descriptor/19181-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70992">
    <skos:narrower rdf:resource="descriptor/18373-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70992">
    <skos:narrower rdf:resource="descriptor/19388-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70992">
    <skos:narrower rdf:resource="descriptor/18393-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12274-3">
    <skos:broader rdf:resource="thsys/71045"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12274-3">
    <skos:broader rdf:resource="thsys/71043"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12274-3">
    <skos:broader rdf:resource="descriptor/10120-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18415-2">
    <skos:broader rdf:resource="thsys/71002"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18415-2">
    <skos:broader rdf:resource="thsys/70985"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18415-2">
    <skos:broader rdf:resource="descriptor/13058-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18567-0">
    <skos:narrower rdf:resource="descriptor/18605-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12228-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19640-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19763-2">
    <skos:broader rdf:resource="thsys/73356"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19763-2">
    <skos:broader rdf:resource="thsys/73334"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19763-2">
    <skos:broader rdf:resource="descriptor/15909-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18935-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12990-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20497-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13621-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12009-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17732-2">
    <skos:broader rdf:resource="thsys/71031"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17732-2">
    <skos:broader rdf:resource="descriptor/17743-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18797-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15033-3">
    <skos:related rdf:resource="descriptor/19561-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15033-3">
    <skos:related rdf:resource="descriptor/10901-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15033-3">
    <skos:related rdf:resource="descriptor/11753-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12368-1">
    <skos:altLabel xml:lang="de">Außenstände</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19581-1">
    <skos:scopeNote xml:lang="de">Für Anwendungen statistischer Verfahren BENUTZE Statistische Methode oder spezifischere Deskriptoren.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19581-1">
    <skos:scopeNote xml:lang="de">Für Literatur über die Statistik als Wissenschaft.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16960-1">
    <skos:altLabel xml:lang="x-other">Fribourg</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16720-0">
    <skos:altLabel xml:lang="de">Neupositivismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16720-0">
    <skos:altLabel xml:lang="de">Neopositivismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11503-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19764-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14438-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10904-6">
    <skos:narrower rdf:resource="descriptor/10907-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10904-6">
    <skos:narrower rdf:resource="descriptor/10898-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13194-3">
    <skos:prefLabel xml:lang="en">Construction material trade</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13194-3">
    <skos:prefLabel xml:lang="de">Baustoffhandel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14483-5">
    <skos:broader rdf:resource="thsys/70135"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14483-5">
    <skos:broader rdf:resource="descriptor/14473-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19189-1">
    <skos:related rdf:resource="descriptor/19190-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19189-1">
    <skos:related rdf:resource="descriptor/18981-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19189-1">
    <skos:related rdf:resource="descriptor/14093-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12786-4">
    <skos:related rdf:resource="descriptor/21041-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12786-4">
    <skos:related rdf:resource="descriptor/19134-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12786-4">
    <skos:related rdf:resource="descriptor/24785-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12786-4">
    <skos:related rdf:resource="descriptor/12785-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12786-4">
    <skos:related rdf:resource="descriptor/19128-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12786-4">
    <skos:related rdf:resource="descriptor/19133-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12786-4">
    <skos:related rdf:resource="descriptor/19541-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18598-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14033-1">
    <skos:broader rdf:resource="thsys/70201"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10801-2">
    <skos:broader rdf:resource="thsys/71108"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10801-2">
    <skos:broader rdf:resource="thsys/71089"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10801-2">
    <skos:broader rdf:resource="thsys/70187"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10801-2">
    <skos:broader rdf:resource="descriptor/10807-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11275-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19395-5">
    <skos:altLabel xml:lang="de">Nahrungsmittelüberwachung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19395-5">
    <skos:altLabel xml:lang="de">Nahrungsmittelkontrolle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19395-5">
    <skos:altLabel xml:lang="de">Lebensmittelkontrolle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16782-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10725-6">
    <skos:broader rdf:resource="thsys/71098"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10725-6">
    <skos:broader rdf:resource="thsys/71089"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10725-6">
    <skos:broader rdf:resource="descriptor/10682-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14812-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70923">
    <skos:narrower rdf:resource="descriptor/18644-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70923">
    <skos:narrower rdf:resource="descriptor/13647-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70923">
    <skos:narrower rdf:resource="descriptor/18642-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70923">
    <skos:narrower rdf:resource="descriptor/18646-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70923">
    <skos:narrower rdf:resource="descriptor/13274-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70923">
    <skos:narrower rdf:resource="descriptor/13981-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70923">
    <skos:narrower rdf:resource="descriptor/13273-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70923">
    <skos:narrower rdf:resource="descriptor/13272-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70923">
    <skos:narrower rdf:resource="descriptor/13270-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70923">
    <skos:narrower rdf:resource="descriptor/18645-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70923">
    <skos:narrower rdf:resource="thsys/70922"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70923">
    <skos:narrower rdf:resource="thsys/70921"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14017-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19360-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12372-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19325-5">
    <skos:related rdf:resource="descriptor/11305-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19325-5">
    <skos:related rdf:resource="descriptor/19314-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16545-6">
    <skos:broader rdf:resource="thsys/73370"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16545-6">
    <skos:broader rdf:resource="thsys/70833"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16545-6">
    <skos:broader rdf:resource="thsys/70324"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16545-6">
    <skos:broader rdf:resource="descriptor/16273-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11735-5">
    <skos:altLabel xml:lang="en">Inequality of income</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11735-5">
    <skos:altLabel xml:lang="en">Income dispersion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11735-5">
    <skos:altLabel xml:lang="de">Personelle Einkommensverteilung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11735-5">
    <skos:altLabel xml:lang="de">Einkommensschichtung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11735-5">
    <skos:altLabel xml:lang="de">Einkommensdisparität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19407-3">
    <skos:prefLabel xml:lang="en">Personal tax</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19407-3">
    <skos:prefLabel xml:lang="de">Personalsteuer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14004-1">
    <skos:prefLabel xml:lang="en">Writing professionals</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14004-1">
    <skos:prefLabel xml:lang="de">Publizistische Berufe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16554-5">
    <skos:broader rdf:resource="thsys/70878"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16554-5">
    <skos:broader rdf:resource="thsys/70011"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16554-5">
    <skos:broader rdf:resource="descriptor/16563-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16554-5">
    <skos:broader rdf:resource="descriptor/10436-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70161">
    <skos:broader rdf:resource="thsys/p"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15604-0">
    <skos:related rdf:resource="descriptor/19810-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15604-0">
    <skos:related rdf:resource="descriptor/18643-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15604-0">
    <skos:related rdf:resource="descriptor/13660-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15604-0">
    <skos:related rdf:resource="descriptor/15961-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15604-0">
    <skos:related rdf:resource="descriptor/19847-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15604-0">
    <skos:related rdf:resource="descriptor/15605-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15604-0">
    <skos:related rdf:resource="descriptor/18433-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15604-0">
    <skos:related rdf:resource="descriptor/19845-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10904-6">
    <skos:altLabel xml:lang="en">Price-cost margin</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10904-6">
    <skos:altLabel xml:lang="en">Dominating firm</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10904-6">
    <skos:altLabel xml:lang="de">Marktbeherrschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10904-6">
    <skos:altLabel xml:lang="de">Marktbeherrschendes Unternehmen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10904-6">
    <skos:altLabel xml:lang="de">Lerner-Index</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10904-6">
    <skos:altLabel xml:lang="de">Anbietermacht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14034-6">
    <skos:altLabel xml:lang="de">Kaliumchloriddünger</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12210-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12040-0">
    <skos:scopeNote xml:lang="de">Nicht nur für Bewertung durch Zahlen und Daten sondern auch z.B. für Bewertung anhand von Kriterien.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19708-3">
    <skos:narrower rdf:resource="descriptor/19712-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19708-3">
    <skos:narrower rdf:resource="descriptor/10452-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19708-3">
    <skos:narrower rdf:resource="descriptor/10557-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19708-3">
    <skos:narrower rdf:resource="descriptor/16437-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13194-3">
    <skos:related rdf:resource="descriptor/13472-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13194-3">
    <skos:related rdf:resource="descriptor/12904-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13194-3">
    <skos:related rdf:resource="descriptor/14021-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18126-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16955-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18436-1">
    <skos:broader rdf:resource="thsys/70959"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18436-1">
    <skos:broader rdf:resource="descriptor/18522-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18436-1">
    <skos:broader rdf:resource="descriptor/13200-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18504-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21014-0">
    <skos:broader rdf:resource="thsys/71075"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21014-0">
    <skos:broader rdf:resource="descriptor/11704-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20744-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18657-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13583-6">
    <skos:related rdf:resource="descriptor/18625-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13583-6">
    <skos:related rdf:resource="descriptor/18187-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13583-6">
    <skos:related rdf:resource="descriptor/13612-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#narrowMatch">
    <rdfs:isDefinedBy rdf:resource="http://www.w3.org/2004/02/skos/core"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11736-3">
    <skos:related rdf:resource="descriptor/11326-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11736-3">
    <skos:related rdf:resource="descriptor/11774-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17433-5">
    <skos:broader rdf:resource="thsys/70557"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17433-5">
    <skos:broader rdf:resource="descriptor/17199-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17433-5">
    <skos:broader rdf:resource="descriptor/17991-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17433-5">
    <skos:broader rdf:resource="descriptor/17983-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17433-5">
    <skos:broader rdf:resource="descriptor/17980-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18635-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12079-1">
    <skos:scopeNote xml:lang="de">Dient der Unternehmensführung zur Vorbereitung von Entscheidungen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13078-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15411-4">
    <skos:broader rdf:resource="thsys/72393"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15411-4">
    <skos:broader rdf:resource="descriptor/19150-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18707-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15743-0">
    <skos:prefLabel xml:lang="en">Air pollution</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15743-0">
    <skos:prefLabel xml:lang="de">Luftverunreinigung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70040">
    <skos:broader rdf:resource="thsys/70041"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12196-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19803-2">
    <skos:related rdf:resource="descriptor/18765-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13824-3">
    <skos:prefLabel xml:lang="en">Insurance premium</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13824-3">
    <skos:prefLabel xml:lang="de">Versicherungsprämie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18775-0">
    <skos:narrower rdf:resource="descriptor/19712-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19769-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25681-4">
    <skos:altLabel xml:lang="de">Marital dissolution</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25681-4">
    <skos:altLabel xml:lang="de">Ehescheidung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id35">
    <rdf:value xml:lang="de">Für touristische Auslandsreisen</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13321-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13658-1">
    <skos:narrower rdf:resource="descriptor/13665-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13658-1">
    <skos:narrower rdf:resource="descriptor/18211-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13658-1">
    <skos:narrower rdf:resource="descriptor/19657-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13658-1">
    <skos:narrower rdf:resource="descriptor/13655-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13151-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18986-1">
    <skos:broader rdf:resource="thsys/71024"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18986-1">
    <skos:broader rdf:resource="descriptor/13452-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15773-5">
    <skos:related rdf:resource="descriptor/10111-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15773-5">
    <skos:related rdf:resource="descriptor/15771-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15773-5">
    <skos:related rdf:resource="descriptor/15768-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15773-5">
    <skos:related rdf:resource="descriptor/15767-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15773-5">
    <skos:related rdf:resource="descriptor/10058-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16605-0">
    <skos:prefLabel xml:lang="en">Consumer society</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16605-0">
    <skos:prefLabel xml:lang="de">Konsumgesellschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16170-2">
    <skos:broader rdf:resource="thsys/73389"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16170-2">
    <skos:broader rdf:resource="thsys/70932"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16170-2">
    <skos:broader rdf:resource="descriptor/13569-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11088-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11518-6">
    <skos:altLabel xml:lang="de">Lohn-Preis-Spirale</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16493-6">
    <skos:broader rdf:resource="thsys/73366"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19759-0">
    <skos:related rdf:resource="descriptor/18864-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19759-0">
    <skos:related rdf:resource="descriptor/15925-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19759-0">
    <skos:related rdf:resource="descriptor/16699-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19759-0">
    <skos:related rdf:resource="descriptor/18911-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19759-0">
    <skos:related rdf:resource="descriptor/18861-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16668-4">
    <skos:narrower rdf:resource="descriptor/18900-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16668-4">
    <skos:narrower rdf:resource="descriptor/16666-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12361-1">
    <skos:altLabel xml:lang="de">Passivierungswahlrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12361-1">
    <skos:altLabel xml:lang="de">Aktivierungswahlrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10808-2">
    <skos:related rdf:resource="descriptor/10817-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10808-2">
    <skos:related rdf:resource="descriptor/10807-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18929-6">
    <skos:broader rdf:resource="thsys/70824"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18929-6">
    <skos:broader rdf:resource="descriptor/13347-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19468-4">
    <skos:prefLabel xml:lang="en">Local economic development</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19468-4">
    <skos:prefLabel xml:lang="de">Kommunale Wirtschaftsentwicklung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11049-1">
    <skos:altLabel xml:lang="de">Konvergenztheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19472-6">
    <skos:prefLabel xml:lang="en">Urban labour market</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19472-6">
    <skos:prefLabel xml:lang="de">Städtischer Arbeitsmarkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19676-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18388-0">
    <skos:narrower rdf:resource="descriptor/19387-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18388-0">
    <skos:narrower rdf:resource="descriptor/19386-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13151-0">
    <skos:altLabel xml:lang="de">Mälzerei</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13151-0">
    <skos:altLabel xml:lang="de">Brauereiwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13151-0">
    <skos:altLabel xml:lang="de">Bierbrauerei</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70536">
    <skos:broader rdf:resource="thsys/70014"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10677-5">
    <skos:broader rdf:resource="thsys/71099"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10677-5">
    <skos:broader rdf:resource="descriptor/10676-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13615-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14812-5">
    <skos:prefLabel xml:lang="en">Printed matter</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14812-5">
    <skos:prefLabel xml:lang="de">Druckereiprodukt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18336-5">
    <skos:altLabel xml:lang="de">Wiederaufforstung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12711-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15437-0">
    <skos:narrower rdf:resource="descriptor/15460-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15220-4">
    <skos:related rdf:resource="descriptor/15475-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16204-4">
    <skos:prefLabel xml:lang="en">Peace research</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16204-4">
    <skos:prefLabel xml:lang="de">Friedensforschung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15905-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16638-6">
    <skos:altLabel xml:lang="en">Subjective well-being</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12366-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18605-4">
    <skos:altLabel xml:lang="en">Open skies</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18605-4">
    <skos:altLabel xml:lang="de">Warschauer Abkommen (Luftrecht)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18605-4">
    <skos:altLabel xml:lang="de">Luftverkehrsabkommen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18605-4">
    <skos:altLabel xml:lang="de">Luftfahrtabkommen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18605-4">
    <skos:altLabel xml:lang="de">Freier Luftraum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71064">
    <skos:narrower rdf:resource="descriptor/13862-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71064">
    <skos:narrower rdf:resource="descriptor/11881-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71064">
    <skos:narrower rdf:resource="descriptor/10512-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71064">
    <skos:narrower rdf:resource="descriptor/11976-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71064">
    <skos:narrower rdf:resource="descriptor/11975-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71064">
    <skos:narrower rdf:resource="descriptor/19363-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71064">
    <skos:narrower rdf:resource="descriptor/18633-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71064">
    <skos:narrower rdf:resource="descriptor/11974-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71064">
    <skos:narrower rdf:resource="descriptor/11973-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71064">
    <skos:narrower rdf:resource="descriptor/11992-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71064">
    <skos:narrower rdf:resource="descriptor/11999-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71064">
    <skos:narrower rdf:resource="descriptor/11991-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71064">
    <skos:narrower rdf:resource="descriptor/11990-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71064">
    <skos:narrower rdf:resource="descriptor/19473-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71064">
    <skos:narrower rdf:resource="descriptor/18837-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71064">
    <skos:narrower rdf:resource="descriptor/11972-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71064">
    <skos:narrower rdf:resource="descriptor/11993-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71064">
    <skos:narrower rdf:resource="descriptor/18753-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71064">
    <skos:narrower rdf:resource="descriptor/11983-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71064">
    <skos:narrower rdf:resource="descriptor/19456-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71064">
    <skos:narrower rdf:resource="descriptor/19472-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71064">
    <skos:narrower rdf:resource="descriptor/11994-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71064">
    <skos:narrower rdf:resource="descriptor/19452-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71064">
    <skos:narrower rdf:resource="descriptor/19453-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71064">
    <skos:narrower rdf:resource="descriptor/11996-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71064">
    <skos:narrower rdf:resource="descriptor/19478-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71064">
    <skos:narrower rdf:resource="descriptor/19477-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71064">
    <skos:narrower rdf:resource="descriptor/19474-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71064">
    <skos:narrower rdf:resource="descriptor/19471-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71064">
    <skos:narrower rdf:resource="descriptor/11892-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71064">
    <skos:narrower rdf:resource="descriptor/11988-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71064">
    <skos:narrower rdf:resource="descriptor/15695-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71064">
    <skos:narrower rdf:resource="descriptor/11987-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71064">
    <skos:narrower rdf:resource="descriptor/11986-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71064">
    <skos:narrower rdf:resource="descriptor/11902-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71064">
    <skos:narrower rdf:resource="descriptor/18638-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71064">
    <skos:narrower rdf:resource="descriptor/10510-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71064">
    <skos:narrower rdf:resource="descriptor/11844-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71064">
    <skos:narrower rdf:resource="descriptor/11971-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71064">
    <skos:narrower rdf:resource="descriptor/19476-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71064">
    <skos:narrower rdf:resource="descriptor/11970-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71064">
    <skos:narrower rdf:resource="descriptor/11969-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71064">
    <skos:narrower rdf:resource="descriptor/11968-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71064">
    <skos:narrower rdf:resource="descriptor/16205-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71064">
    <skos:narrower rdf:resource="descriptor/11610-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71064">
    <skos:narrower rdf:resource="descriptor/16549-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71064">
    <skos:narrower rdf:resource="descriptor/11998-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71064">
    <skos:narrower rdf:resource="descriptor/11591-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71064">
    <skos:narrower rdf:resource="descriptor/11603-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71064">
    <skos:narrower rdf:resource="descriptor/19446-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71064">
    <skos:narrower rdf:resource="descriptor/19468-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71064">
    <skos:narrower rdf:resource="descriptor/18806-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71064">
    <skos:narrower rdf:resource="descriptor/18800-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71064">
    <skos:narrower rdf:resource="descriptor/19378-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71064">
    <skos:narrower rdf:resource="descriptor/11590-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71064">
    <skos:narrower rdf:resource="descriptor/11544-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71064">
    <skos:narrower rdf:resource="descriptor/19470-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71064">
    <skos:narrower rdf:resource="descriptor/11553-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71064">
    <skos:narrower rdf:resource="descriptor/19457-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71064">
    <skos:narrower rdf:resource="descriptor/19479-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71064">
    <skos:narrower rdf:resource="descriptor/11966-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71064">
    <skos:narrower rdf:resource="descriptor/19469-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71064">
    <skos:narrower rdf:resource="descriptor/11981-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71064">
    <skos:narrower rdf:resource="descriptor/11653-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71064">
    <skos:narrower rdf:resource="descriptor/11965-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71064">
    <skos:narrower rdf:resource="descriptor/11537-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71064">
    <skos:narrower rdf:resource="descriptor/19467-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71064">
    <skos:narrower rdf:resource="descriptor/11852-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71064">
    <skos:narrower rdf:resource="descriptor/11995-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71064">
    <skos:narrower rdf:resource="descriptor/11979-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16732-0">
    <skos:narrower rdf:resource="descriptor/16729-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16732-0">
    <skos:narrower rdf:resource="descriptor/16726-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16330-6">
    <skos:related rdf:resource="descriptor/16333-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16330-6">
    <skos:related rdf:resource="descriptor/13989-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16330-6">
    <skos:related rdf:resource="descriptor/16658-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19766-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13130-1">
    <skos:altLabel xml:lang="de">Spinnerei</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13620-5">
    <skos:prefLabel xml:lang="en">Cargo handling</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13620-5">
    <skos:prefLabel xml:lang="de">Güterumschlag</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18188-1">
    <skos:broader rdf:resource="thsys/71095"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18188-1">
    <skos:broader rdf:resource="thsys/70455"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19512-6">
    <skos:broader rdf:resource="thsys/70832"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19512-6">
    <skos:broader rdf:resource="thsys/70458"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19512-6">
    <skos:broader rdf:resource="descriptor/11318-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19512-6">
    <skos:broader rdf:resource="descriptor/16527-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15971-1">
    <skos:prefLabel xml:lang="en">Legal decisions</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15971-1">
    <skos:prefLabel xml:lang="de">Rechtsprechung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12271-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10779-4">
    <skos:prefLabel xml:lang="en">Foreign exchange market</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10779-4">
    <skos:prefLabel xml:lang="de">Devisenmarkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16140-4">
    <skos:broader rdf:resource="thsys/73336"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16140-4">
    <skos:broader rdf:resource="descriptor/16480-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16140-4">
    <skos:broader rdf:resource="descriptor/16124-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19130-6">
    <skos:broader rdf:resource="thsys/70307"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13560-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16304-0">
    <skos:related rdf:resource="descriptor/11217-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16304-0">
    <skos:related rdf:resource="descriptor/19855-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16304-0">
    <skos:related rdf:resource="descriptor/16396-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16304-0">
    <skos:related rdf:resource="descriptor/16303-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16304-0">
    <skos:related rdf:resource="descriptor/16395-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16304-0">
    <skos:related rdf:resource="descriptor/16510-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13145-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id118">
    <rdfs:seeAlso rdf:resource="descriptor/10523-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18274-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10637-3">
    <skos:altLabel xml:lang="en">Factor endowment theory</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10637-3">
    <skos:altLabel xml:lang="de">Heckscher-Ohlin-Theorem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10637-3">
    <skos:altLabel xml:lang="de">Faktorausstattungstheorem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19785-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15265-3">
    <skos:prefLabel xml:lang="en">Least squares estimation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15265-3">
    <skos:prefLabel xml:lang="de">Methode der kleinsten Quadrate</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10617-2">
    <skos:narrower rdf:resource="descriptor/10626-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10617-2">
    <skos:narrower rdf:resource="descriptor/10603-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10617-2">
    <skos:narrower rdf:resource="descriptor/18121-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10617-2">
    <skos:narrower rdf:resource="descriptor/10452-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10617-2">
    <skos:narrower rdf:resource="descriptor/19071-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70235">
    <skos:broader rdf:resource="thsys/70052"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18894-6">
    <skos:prefLabel xml:lang="en">Primary health care</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18894-6">
    <skos:prefLabel xml:lang="de">Basisgesundheitsversorgung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14438-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17781-3">
    <skos:prefLabel xml:lang="en">Grenada</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17781-3">
    <skos:prefLabel xml:lang="de">Grenada</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18716-2">
    <skos:altLabel xml:lang="en">Banking strategy</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18716-2">
    <skos:altLabel xml:lang="de">Bankstrategie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18716-2">
    <skos:altLabel xml:lang="de">Bankführung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15966-1">
    <skos:broader rdf:resource="thsys/70154"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14081-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10949-5">
    <skos:prefLabel xml:lang="en">Market definition</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10949-5">
    <skos:prefLabel xml:lang="de">Relevanter Markt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70990">
    <skos:broader rdf:resource="thsys/71004"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13110-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15476-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19138-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11033-2">
    <skos:narrower rdf:resource="descriptor/11030-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11033-2">
    <skos:narrower rdf:resource="descriptor/11024-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11033-2">
    <skos:narrower rdf:resource="descriptor/10996-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16335-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id74">
    <rdf:value xml:lang="de">Für nationale steuerliche Doppelbelastung</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13962-5">
    <skos:broader rdf:resource="thsys/71056"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16008-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10263-4">
    <skos:broader rdf:resource="thsys/70306"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19572-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19458-0">
    <skos:broader rdf:resource="thsys/71065"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19458-0">
    <skos:broader rdf:resource="descriptor/10575-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19458-0">
    <skos:broader rdf:resource="descriptor/19467-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13151-0">
    <skos:related rdf:resource="descriptor/14973-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16078-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18332-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70932">
    <skos:broader rdf:resource="thsys/70942"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19483-1">
    <skos:broader rdf:resource="thsys/70239"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19483-1">
    <skos:broader rdf:resource="descriptor/15768-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18503-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19382-0">
    <skos:broader rdf:resource="thsys/71018"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19382-0">
    <skos:broader rdf:resource="thsys/70960"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19382-0">
    <skos:broader rdf:resource="descriptor/10012-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71046">
    <skos:narrower rdf:resource="descriptor/12265-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71046">
    <skos:narrower rdf:resource="descriptor/12261-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71046">
    <skos:narrower rdf:resource="descriptor/12259-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71046">
    <skos:narrower rdf:resource="descriptor/19081-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71046">
    <skos:narrower rdf:resource="descriptor/12296-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71046">
    <skos:narrower rdf:resource="descriptor/12284-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71046">
    <skos:narrower rdf:resource="descriptor/12282-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71046">
    <skos:narrower rdf:resource="descriptor/13727-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71046">
    <skos:narrower rdf:resource="descriptor/19080-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71046">
    <skos:narrower rdf:resource="descriptor/12273-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71046">
    <skos:narrower rdf:resource="descriptor/12239-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71046">
    <skos:narrower rdf:resource="descriptor/11769-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71046">
    <skos:narrower rdf:resource="descriptor/19535-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71046">
    <skos:narrower rdf:resource="descriptor/12232-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11057-2">
    <skos:prefLabel xml:lang="en">Constitutional economics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11057-2">
    <skos:prefLabel xml:lang="de">Verfassungsökonomik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12408-1">
    <skos:altLabel xml:lang="de">Niederstwertprinzip</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12408-1">
    <skos:altLabel xml:lang="de">Höchstwertprinzip</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12408-1">
    <skos:altLabel xml:lang="de">Bewertungsmethode der Bilanz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12408-1">
    <skos:altLabel xml:lang="de">Bewertungsgrundsatz der Bilanz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12408-1">
    <skos:altLabel xml:lang="de">Bewertung in der Bilanz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16496-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21315-0">
    <skos:broader rdf:resource="thsys/70922"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21315-0">
    <skos:broader rdf:resource="thsys/70874"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21315-0">
    <skos:broader rdf:resource="thsys/70384"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21315-0">
    <skos:broader rdf:resource="descriptor/12829-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12297-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70824">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14035-4">
    <skos:altLabel xml:lang="de">Nukleare Anlage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14035-4">
    <skos:altLabel xml:lang="de">Leichtwasserreaktor</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14035-4">
    <skos:altLabel xml:lang="de">Kerntechnische Anlage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14035-4">
    <skos:altLabel xml:lang="de">Kernreaktor</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14035-4">
    <skos:altLabel xml:lang="de">Atomreaktor</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14035-4">
    <skos:altLabel xml:lang="de">Atomkraftwerk</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19509-2">
    <skos:prefLabel xml:lang="en">International labour standards</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19509-2">
    <skos:prefLabel xml:lang="de">Internationale Sozialstandards</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17666-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70046">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11844-0">
    <skos:related rdf:resource="descriptor/18753-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11844-0">
    <skos:related rdf:resource="descriptor/11994-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11844-0">
    <skos:related rdf:resource="descriptor/19453-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11844-0">
    <skos:related rdf:resource="descriptor/11998-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11844-0">
    <skos:related rdf:resource="descriptor/12018-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11318-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14531-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11165-6">
    <skos:related rdf:resource="descriptor/10089-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18645-6">
    <skos:prefLabel xml:lang="en">Courier service</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18645-6">
    <skos:prefLabel xml:lang="de">Kurierdienst</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73378">
    <skos:narrower rdf:resource="descriptor/13424-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73378">
    <skos:narrower rdf:resource="descriptor/13405-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73378">
    <skos:narrower rdf:resource="descriptor/10966-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73378">
    <skos:narrower rdf:resource="descriptor/13401-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73378">
    <skos:narrower rdf:resource="descriptor/16389-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73378">
    <skos:narrower rdf:resource="descriptor/13400-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73378">
    <skos:narrower rdf:resource="descriptor/16374-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73378">
    <skos:narrower rdf:resource="descriptor/15711-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73378">
    <skos:narrower rdf:resource="descriptor/16373-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73378">
    <skos:narrower rdf:resource="descriptor/13397-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73378">
    <skos:narrower rdf:resource="descriptor/16280-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73378">
    <skos:narrower rdf:resource="descriptor/13989-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73378">
    <skos:narrower rdf:resource="descriptor/16370-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73378">
    <skos:narrower rdf:resource="descriptor/16369-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73378">
    <skos:narrower rdf:resource="descriptor/19696-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73378">
    <skos:narrower rdf:resource="descriptor/19695-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73378">
    <skos:narrower rdf:resource="descriptor/16366-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73378">
    <skos:narrower rdf:resource="descriptor/16008-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73378">
    <skos:narrower rdf:resource="descriptor/16365-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73378">
    <skos:narrower rdf:resource="descriptor/12091-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73378">
    <skos:narrower rdf:resource="descriptor/16364-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73378">
    <skos:narrower rdf:resource="descriptor/13395-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73378">
    <skos:narrower rdf:resource="descriptor/16372-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73378">
    <skos:narrower rdf:resource="descriptor/16363-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73378">
    <skos:narrower rdf:resource="descriptor/16362-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73378">
    <skos:narrower rdf:resource="descriptor/16361-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73378">
    <skos:narrower rdf:resource="descriptor/13394-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73378">
    <skos:narrower rdf:resource="descriptor/16630-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73378">
    <skos:narrower rdf:resource="descriptor/10984-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73378">
    <skos:narrower rdf:resource="descriptor/10968-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73378">
    <skos:narrower rdf:resource="descriptor/18123-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73378">
    <skos:narrower rdf:resource="descriptor/16629-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73378">
    <skos:narrower rdf:resource="descriptor/11313-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73378">
    <skos:narrower rdf:resource="descriptor/13988-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73378">
    <skos:narrower rdf:resource="descriptor/16391-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73378">
    <skos:narrower rdf:resource="descriptor/16628-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73378">
    <skos:narrower rdf:resource="descriptor/10558-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73378">
    <skos:narrower rdf:resource="descriptor/16382-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73378">
    <skos:narrower rdf:resource="descriptor/18850-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73378">
    <skos:narrower rdf:resource="descriptor/11304-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73378">
    <skos:narrower rdf:resource="descriptor/16387-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12183-6">
    <skos:altLabel xml:lang="de">GmbH &amp;amp; Co. KG</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13745-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70051">
    <skos:prefLabel xml:lang="en">V.13.01  Labour market</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70051">
    <skos:prefLabel xml:lang="de">V.13.01  Arbeitsmarkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15708-2">
    <skos:narrower rdf:resource="descriptor/11894-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15708-2">
    <skos:narrower rdf:resource="descriptor/10489-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14653-6">
    <skos:broader rdf:resource="thsys/70463"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14653-6">
    <skos:broader rdf:resource="descriptor/14205-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11272-5">
    <zbwext:useInsteadNote rdf:nodeID="id92"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70186">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19750-4">
    <skos:altLabel xml:lang="de">Dekonstruktivismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14449-5">
    <skos:broader rdf:resource="thsys/70415"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14449-5">
    <skos:broader rdf:resource="descriptor/14455-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18589-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19318-2">
    <skos:prefLabel xml:lang="en">Export subsidies</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19318-2">
    <skos:prefLabel xml:lang="de">Exportsubvention</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11004-2">
    <skos:narrower rdf:resource="descriptor/11900-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11004-2">
    <skos:narrower rdf:resource="descriptor/11007-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11004-2">
    <skos:narrower rdf:resource="descriptor/10523-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11355-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11404-0">
    <skos:broader rdf:resource="thsys/73369"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11404-0">
    <skos:broader rdf:resource="thsys/70845"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11404-0">
    <skos:broader rdf:resource="thsys/70176"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11404-0">
    <skos:broader rdf:resource="descriptor/11397-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14930-6">
    <skos:broader rdf:resource="thsys/70498"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14930-6">
    <skos:broader rdf:resource="descriptor/14931-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15719-4">
    <skos:related rdf:resource="descriptor/16597-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15719-4">
    <skos:related rdf:resource="descriptor/15721-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15719-4">
    <skos:related rdf:resource="descriptor/15718-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15719-4">
    <skos:related rdf:resource="descriptor/15717-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15719-4">
    <skos:related rdf:resource="descriptor/16288-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15719-4">
    <skos:related rdf:resource="descriptor/19715-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15719-4">
    <skos:related rdf:resource="descriptor/16621-2"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id119">
    <rdfs:seeAlso rdf:resource="descriptor/12744-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13292-3">
    <skos:editorialNote xml:lang="de">Möglichst spezifischer indexieren.</skos:editorialNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11416-0">
    <skos:altLabel xml:lang="de">Währungsgeschichte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11416-0">
    <skos:altLabel xml:lang="de">Münzgeschichte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11416-0">
    <skos:altLabel xml:lang="de">Geschichte des Geldwesens</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11416-0">
    <skos:altLabel xml:lang="de">Geld- und Währungsgeschichte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18754-1">
    <skos:altLabel xml:lang="de">Soziale Wohnpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18665-0">
    <skos:related rdf:resource="descriptor/21283-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10486-5">
    <skos:scopeNote xml:lang="de">Analphabetismus sowie Maßnahmen zu seiner Verringerung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11547-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16274-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13242-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19157-0">
    <skos:broader rdf:resource="thsys/70087"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19157-0">
    <skos:broader rdf:resource="descriptor/12192-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18956-3">
    <skos:broader rdf:resource="thsys/70819"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18090-4">
    <skos:related rdf:resource="descriptor/13177-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73367">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11981-4">
    <skos:altLabel xml:lang="de">Städtisches Gewerbegebiet</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11981-4">
    <skos:altLabel xml:lang="de">Kommunales Industriegebiet</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11981-4">
    <skos:altLabel xml:lang="de">Kommunales Gewerbegebiet</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11981-4">
    <skos:altLabel xml:lang="de">Industriegelände</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11981-4">
    <skos:altLabel xml:lang="de">Gewerbepark</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16195-0">
    <skos:narrower rdf:resource="descriptor/16201-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16195-0">
    <skos:narrower rdf:resource="descriptor/19860-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16195-0">
    <skos:narrower rdf:resource="descriptor/15766-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16195-0">
    <skos:narrower rdf:resource="descriptor/16112-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16195-0">
    <skos:narrower rdf:resource="descriptor/19855-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16195-0">
    <skos:narrower rdf:resource="descriptor/19814-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16195-0">
    <skos:narrower rdf:resource="descriptor/13486-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16195-0">
    <skos:narrower rdf:resource="descriptor/16193-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16195-0">
    <skos:narrower rdf:resource="descriptor/16192-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16195-0">
    <skos:narrower rdf:resource="descriptor/11419-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16195-0">
    <skos:narrower rdf:resource="descriptor/18736-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15919-3">
    <skos:prefLabel xml:lang="en">Refugees</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15919-3">
    <skos:prefLabel xml:lang="de">Flüchtlinge</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15798-3">
    <skos:broader rdf:resource="thsys/73386"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15798-3">
    <skos:broader rdf:resource="thsys/73376"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15798-3">
    <skos:broader rdf:resource="thsys/73359"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15798-3">
    <skos:broader rdf:resource="thsys/71039"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15798-3">
    <skos:broader rdf:resource="thsys/70383"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11129-3">
    <skos:scopeNote xml:lang="de">für Staatsfonds BENUTZE: Staatsfonds.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11129-3">
    <skos:scopeNote xml:lang="de">Für Wertpapierfonds BENUTZE: Investmentfonds oder Unterbegriffe</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11129-3">
    <skos:scopeNote xml:lang="de">Nur für die Fonds im Sinne der Planwirtschaft, Äquivalente des Anlagevermögens. Auch für die entsprechenden Fonds in den ehemaligen sozialistischen Staaten.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70560">
    <rdfs:label xml:lang="en">West Africa</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70560">
    <rdfs:label xml:lang="de">Westafrika</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18305-2">
    <skos:broader rdf:resource="thsys/70333"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18305-2">
    <skos:broader rdf:resource="descriptor/18304-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16480-1">
    <skos:altLabel xml:lang="de">Jugendrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13200-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14434-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13090-1">
    <skos:altLabel xml:lang="en">Mechanical tool industry</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13090-1">
    <skos:altLabel xml:lang="en">Die and mold industry</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13090-1">
    <skos:altLabel xml:lang="en">Die &amp;amp; mold industry</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13090-1">
    <skos:altLabel xml:lang="de">Werkzeugbau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13090-1">
    <skos:altLabel xml:lang="de">Formenbau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19280-1">
    <skos:scopeNote xml:lang="de">Modell zur Beurteilung, ob Optionskontrakte angemessen bewertet sind.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18547-6">
    <skos:related rdf:resource="descriptor/13200-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10459-1">
    <skos:broader rdf:resource="thsys/71109"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10459-1">
    <skos:broader rdf:resource="thsys/70879"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10459-1">
    <skos:broader rdf:resource="thsys/70550"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10459-1">
    <skos:broader rdf:resource="thsys/70485"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10459-1">
    <skos:broader rdf:resource="descriptor/16594-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12787-2">
    <skos:broader rdf:resource="thsys/73391"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12787-2">
    <skos:broader rdf:resource="thsys/70956"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10301-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16299-2">
    <skos:altLabel xml:lang="en">Nation-buildung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16299-2">
    <skos:altLabel xml:lang="de">Nationenbildung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73354">
    <skos:broader rdf:resource="thsys/70151"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12587-3">
    <skos:altLabel xml:lang="de">KVP (Kontinuierlicher Verbesserungsprozess)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12587-3">
    <skos:altLabel xml:lang="de">KVP (Kontinuierlicher Verbesserungsprozeß)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12587-3">
    <skos:altLabel xml:lang="de">Kontinuierlicher Verbesserungsprozess</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12587-3">
    <skos:altLabel xml:lang="de">Kontinuierlicher Verbesserungsprozeß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12587-3">
    <skos:altLabel xml:lang="de">Ideenmanagement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12587-3">
    <skos:altLabel xml:lang="de">BVW (Betriebliches Vorschlagswesen)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12587-3">
    <skos:altLabel xml:lang="de">Betriebliches Vorschlagswesen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12587-3">
    <skos:altLabel xml:lang="de">Betrieblicher Verbesserungsprozess</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12587-3">
    <skos:altLabel xml:lang="de">Betrieblicher Verbesserungsprozeß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12587-3">
    <skos:altLabel xml:lang="de">Arbeitnehmererfindung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71087">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19588-1">
    <skos:broader rdf:resource="thsys/72181"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19588-1">
    <skos:broader rdf:resource="thsys/71010"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19588-1">
    <skos:broader rdf:resource="descriptor/15076-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21020-5">
    <skos:broader rdf:resource="thsys/71036"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21020-5">
    <skos:broader rdf:resource="thsys/70394"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21020-5">
    <skos:broader rdf:resource="descriptor/12670-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14769-1">
    <skos:prefLabel xml:lang="en">Ceramics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14769-1">
    <skos:prefLabel xml:lang="de">Keramisches Produkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71009">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.05.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17987-4">
    <skos:related rdf:resource="descriptor/17687-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10784-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24664-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12602-5">
    <skos:altLabel xml:lang="en">Industrial engineering</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10443-2">
    <skos:narrower rdf:resource="descriptor/18461-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18440-3">
    <skos:altLabel xml:lang="de">Konditorei</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18440-3">
    <skos:altLabel xml:lang="de">Großbäckerei</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18440-3">
    <skos:altLabel xml:lang="de">Brotindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18440-3">
    <skos:altLabel xml:lang="de">Bäckerhandwerk</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18440-3">
    <skos:altLabel xml:lang="de">Bäckerei</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14379-0">
    <skos:broader rdf:resource="thsys/70318"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14379-0">
    <skos:broader rdf:resource="descriptor/14381-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19391-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15077-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15638-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19423-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14772-5">
    <skos:broader rdf:resource="thsys/73344"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14772-5">
    <skos:broader rdf:resource="thsys/70647"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14772-5">
    <skos:broader rdf:resource="descriptor/14766-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18788-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14602-2">
    <skos:broader rdf:resource="thsys/70074"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14602-2">
    <skos:broader rdf:resource="descriptor/14599-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10434-3">
    <skos:narrower rdf:resource="descriptor/10359-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10434-3">
    <skos:narrower rdf:resource="descriptor/10433-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10434-3">
    <skos:narrower rdf:resource="descriptor/10432-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10434-3">
    <skos:narrower rdf:resource="descriptor/10428-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10434-3">
    <skos:narrower rdf:resource="descriptor/10427-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10434-3">
    <skos:narrower rdf:resource="descriptor/10426-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10434-3">
    <skos:narrower rdf:resource="descriptor/10424-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10434-3">
    <skos:narrower rdf:resource="descriptor/19244-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10434-3">
    <skos:narrower rdf:resource="descriptor/20982-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10434-3">
    <skos:narrower rdf:resource="descriptor/10421-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10434-3">
    <skos:narrower rdf:resource="descriptor/10418-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10434-3">
    <skos:narrower rdf:resource="descriptor/10415-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11916-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18536-4">
    <skos:scopeNote xml:lang="de">Außenhandel zwischen den EU-Mitgliedsstaaten.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19811-3">
    <skos:altLabel xml:lang="de">Medizinische Informatik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19811-3">
    <skos:altLabel xml:lang="de">Biometrik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13345-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18859-1">
    <skos:broader rdf:resource="thsys/73359"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18859-1">
    <skos:broader rdf:resource="thsys/70017"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10616-4">
    <skos:altLabel xml:lang="de">Welthandelssystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10616-4">
    <skos:altLabel xml:lang="de">Welthandelsordnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19599-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15699-5">
    <skos:related rdf:resource="descriptor/19634-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15699-5">
    <skos:related rdf:resource="descriptor/15689-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19482-3">
    <skos:prefLabel xml:lang="en">Ozone layer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19482-3">
    <skos:prefLabel xml:lang="de">Ozonschicht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15737-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11201-0">
    <skos:broader rdf:resource="thsys/70052"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11201-0">
    <skos:broader rdf:resource="descriptor/11180-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13177-3">
    <skos:related rdf:resource="descriptor/18090-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13177-3">
    <skos:related rdf:resource="descriptor/19055-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10866-2">
    <skos:prefLabel xml:lang="en">Export credit insurance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10866-2">
    <skos:prefLabel xml:lang="de">Exportkreditversicherung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18115-0">
    <skos:broader rdf:resource="thsys/70485"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18725-1">
    <skos:broader rdf:resource="thsys/73392"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18725-1">
    <skos:broader rdf:resource="thsys/70899"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18725-1">
    <skos:broader rdf:resource="descriptor/16039-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73380">
    <skos:narrower rdf:resource="descriptor/16336-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73380">
    <skos:narrower rdf:resource="descriptor/16019-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73380">
    <skos:narrower rdf:resource="descriptor/16335-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73380">
    <skos:narrower rdf:resource="descriptor/15762-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73380">
    <skos:narrower rdf:resource="descriptor/18161-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73380">
    <skos:narrower rdf:resource="descriptor/13419-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73380">
    <skos:narrower rdf:resource="descriptor/18808-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73380">
    <skos:narrower rdf:resource="descriptor/16306-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73380">
    <skos:narrower rdf:resource="descriptor/19694-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73380">
    <skos:narrower rdf:resource="descriptor/16330-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73380">
    <skos:narrower rdf:resource="descriptor/16329-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73380">
    <skos:narrower rdf:resource="descriptor/11541-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73380">
    <skos:narrower rdf:resource="descriptor/11020-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73380">
    <skos:narrower rdf:resource="descriptor/11722-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73380">
    <skos:narrower rdf:resource="descriptor/11559-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73380">
    <skos:narrower rdf:resource="descriptor/13413-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73380">
    <skos:narrower rdf:resource="descriptor/16327-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73380">
    <skos:narrower rdf:resource="descriptor/16324-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73380">
    <skos:narrower rdf:resource="descriptor/16550-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73380">
    <skos:narrower rdf:resource="descriptor/19691-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73380">
    <skos:narrower rdf:resource="descriptor/15962-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73380">
    <skos:narrower rdf:resource="descriptor/16322-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73380">
    <skos:narrower rdf:resource="descriptor/11536-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73380">
    <skos:narrower rdf:resource="descriptor/11154-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73380">
    <skos:narrower rdf:resource="descriptor/13407-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73380">
    <skos:narrower rdf:resource="descriptor/18810-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73380">
    <skos:narrower rdf:resource="descriptor/11237-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14703-3">
    <skos:altLabel xml:lang="de">Polyethylenterephthalatharz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14703-3">
    <skos:altLabel xml:lang="de">Polyesterharz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14703-3">
    <skos:altLabel xml:lang="de">Phenolharz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14703-3">
    <skos:altLabel xml:lang="de">PET-Harz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14703-3">
    <skos:altLabel xml:lang="de">Melaminharz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14703-3">
    <skos:altLabel xml:lang="de">Epoxidharz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14703-3">
    <skos:altLabel xml:lang="de">Duroplaste</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14703-3">
    <skos:altLabel xml:lang="de">Duromere</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11348-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70327">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71083">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.09.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19208-2">
    <skos:related rdf:resource="descriptor/10114-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19208-2">
    <skos:related rdf:resource="descriptor/10328-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19208-2">
    <skos:related rdf:resource="descriptor/10102-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19208-2">
    <skos:related rdf:resource="descriptor/19827-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18865-6">
    <skos:broader rdf:resource="thsys/71065"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18865-6">
    <skos:broader rdf:resource="thsys/70847"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18865-6">
    <skos:broader rdf:resource="thsys/70176"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18865-6">
    <skos:broader rdf:resource="descriptor/11367-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13753-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18803-0">
    <skos:broader rdf:resource="thsys/71063"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18803-0">
    <skos:broader rdf:resource="thsys/70872"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18803-0">
    <skos:broader rdf:resource="descriptor/11716-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10823-6">
    <skos:narrower rdf:resource="descriptor/12292-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14949-6">
    <skos:broader rdf:resource="thsys/73300"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14949-6">
    <skos:broader rdf:resource="thsys/70342"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14949-6">
    <skos:broader rdf:resource="descriptor/14948-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14949-6">
    <skos:broader rdf:resource="descriptor/14971-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15897-1">
    <skos:narrower rdf:resource="descriptor/18340-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14984-4">
    <skos:prefLabel xml:lang="en">Mineral water</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14984-4">
    <skos:prefLabel xml:lang="de">Mineralwasser</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13959-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19650-1">
    <skos:related rdf:resource="descriptor/15954-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19650-1">
    <skos:related rdf:resource="descriptor/16749-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19650-1">
    <skos:related rdf:resource="descriptor/19641-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14049-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19304-6">
    <skos:related rdf:resource="descriptor/19305-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19304-6">
    <skos:related rdf:resource="descriptor/19303-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19304-6">
    <skos:related rdf:resource="descriptor/19302-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19304-6">
    <skos:related rdf:resource="descriptor/14098-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11223-4">
    <skos:broader rdf:resource="thsys/70049"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19511-1">
    <skos:altLabel xml:lang="en">Probation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19511-1">
    <skos:altLabel xml:lang="de">Straftäterfürsorge</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19511-1">
    <skos:altLabel xml:lang="de">Strafgefangenenfürsorge</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19511-1">
    <skos:altLabel xml:lang="de">Strafentlassenenfürsorge</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19511-1">
    <skos:altLabel xml:lang="de">Soziale Gerichtshilfe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19511-1">
    <skos:altLabel xml:lang="de">Gerichtshilfe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19511-1">
    <skos:altLabel xml:lang="de">Gefangenenfürsorge</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19511-1">
    <skos:altLabel xml:lang="de">Entlassungsfürsorge</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19511-1">
    <skos:altLabel xml:lang="de">Bewährungshilfe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16304-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17717-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13767-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18546-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17779-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11464-3">
    <skos:related rdf:resource="descriptor/11485-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11464-3">
    <skos:related rdf:resource="descriptor/10229-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21041-4">
    <skos:related rdf:resource="descriptor/19555-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21041-4">
    <skos:related rdf:resource="descriptor/12786-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21041-4">
    <skos:related rdf:resource="descriptor/19548-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21041-4">
    <skos:related rdf:resource="descriptor/12730-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14653-6">
    <skos:altLabel xml:lang="en">Sodium hydroxide</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14653-6">
    <skos:altLabel xml:lang="en">Sodium carbonate</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14653-6">
    <skos:altLabel xml:lang="de">Natriumoxid</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14653-6">
    <skos:altLabel xml:lang="de">Natriumcarbonat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19338-3">
    <skos:prefLabel xml:lang="en">Food market</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19338-3">
    <skos:prefLabel xml:lang="de">Nahrungsmittelmarkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11578-2">
    <skos:broader rdf:resource="thsys/70969"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11578-2">
    <skos:broader rdf:resource="descriptor/12905-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18660-3">
    <skos:prefLabel xml:lang="en">Electronic data interchange</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18660-3">
    <skos:prefLabel xml:lang="de">Elektronischer Datenaustausch</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12261-5">
    <skos:altLabel xml:lang="en">Preferred share</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12261-5">
    <skos:altLabel xml:lang="de">Stimmrechtlose Vorzugsaktie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10525-0">
    <skos:broader rdf:resource="thsys/73383"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10525-0">
    <skos:broader rdf:resource="thsys/71086"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10525-0">
    <skos:broader rdf:resource="descriptor/10062-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19814-4">
    <skos:altLabel xml:lang="de">Triaden (Organisierte Kriminalität)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19814-4">
    <skos:altLabel xml:lang="de">Organisiertes Verbrechen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19814-4">
    <skos:altLabel xml:lang="de">OK (Organisierte Kriminalität)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19814-4">
    <skos:altLabel xml:lang="de">Mafia</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15097-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12779-1">
    <skos:prefLabel xml:lang="en">Big event</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12779-1">
    <skos:prefLabel xml:lang="de">Großveranstaltung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11921-1">
    <skos:narrower rdf:resource="descriptor/12022-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13343-5">
    <skos:prefLabel xml:lang="en">Research centre</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13343-5">
    <skos:prefLabel xml:lang="de">Forschungsinstitut</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18871-4">
    <skos:scopeNote xml:lang="de">Für Wirtschaftswissenschaftliche Fakultäten und Hochschulen BENUTZE Wirtschaftswissenschaftliche Hochschule.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70091">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18413-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10501-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10114-0">
    <skos:prefLabel xml:lang="en">X-efficiency</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10114-0">
    <skos:prefLabel xml:lang="de">X-Effizienz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14739-3">
    <skos:altLabel xml:lang="de">Geschirrspülmittel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16068-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10778-6">
    <skos:broader rdf:resource="thsys/71087"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10778-6">
    <skos:broader rdf:resource="descriptor/10791-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18777-3">
    <skos:prefLabel xml:lang="en">Biotechnological research</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18777-3">
    <skos:prefLabel xml:lang="de">Biotechnische Forschung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19527-0">
    <skos:prefLabel xml:lang="en">Organ transplantation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19527-0">
    <skos:prefLabel xml:lang="de">Organtransplantation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10301-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14328-3">
    <skos:altLabel xml:lang="de">Trecker</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14328-3">
    <skos:altLabel xml:lang="de">Ackerschlepper</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10672-1">
    <skos:scopeNote xml:lang="de">Erklärt Zahlungsbilanzungleichgewichte durch Geldmarktungleichgewichte bei festen Wechselkursen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19661-3">
    <skos:altLabel xml:lang="de">Rechnerunterstützung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19661-3">
    <skos:altLabel xml:lang="de">Elektronisches Verfahren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19661-3">
    <skos:altLabel xml:lang="de">Elektronikanwendung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19661-3">
    <skos:altLabel xml:lang="de">Computerunterstütztes Verfahren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19661-3">
    <skos:altLabel xml:lang="de">Computergestützte Methode</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19661-3">
    <skos:altLabel xml:lang="de">Computeranwendung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70871">
    <skos:prefLabel xml:lang="en">W.22.01  Public Administration</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70871">
    <skos:prefLabel xml:lang="de">W.22.01  Öffentliche Verwaltung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16946-2">
    <skos:broader rdf:resource="thsys/70438"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16946-2">
    <skos:broader rdf:resource="descriptor/16950-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70869">
    <skos:narrower rdf:resource="descriptor/16110-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70869">
    <skos:narrower rdf:resource="descriptor/11541-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70869">
    <skos:narrower rdf:resource="descriptor/11540-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70869">
    <skos:narrower rdf:resource="descriptor/11595-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70869">
    <skos:narrower rdf:resource="descriptor/11539-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70869">
    <skos:narrower rdf:resource="descriptor/11536-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70869">
    <skos:narrower rdf:resource="descriptor/11534-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13329-6">
    <skos:narrower rdf:resource="descriptor/13334-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14700-2">
    <skos:altLabel xml:lang="de">Kohlefaserverstärkter Kunststoff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14700-2">
    <skos:altLabel xml:lang="de">Glasfaserverstärkter Kunststoff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14700-2">
    <skos:altLabel xml:lang="de">GFK-Kunststoff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14084-5">
    <skos:broader rdf:resource="thsys/70341"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14084-5">
    <skos:broader rdf:resource="descriptor/14073-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11088-5">
    <skos:altLabel xml:lang="de">Neomarxismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11088-5">
    <skos:altLabel xml:lang="de">Marxismus-Leninismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11088-5">
    <skos:altLabel xml:lang="de">Leninismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10266-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19553-6">
    <skos:scopeNote xml:lang="de">Für Haushalte mit niedrigem Einkommen, gleichgültig aus welcher Quelle und Ursache.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18659-2">
    <skos:related rdf:resource="descriptor/24782-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18659-2">
    <skos:related rdf:resource="descriptor/19145-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18659-2">
    <skos:related rdf:resource="descriptor/19657-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18659-2">
    <skos:related rdf:resource="descriptor/19651-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18659-2">
    <skos:related rdf:resource="descriptor/13652-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18659-2">
    <skos:related rdf:resource="descriptor/19658-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18659-2">
    <skos:related rdf:resource="descriptor/20341-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14453-0">
    <skos:altLabel xml:lang="en">Hovercraft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14453-0">
    <skos:altLabel xml:lang="en">Aerotrain</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13190-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10025-6">
    <skos:narrower rdf:resource="descriptor/10018-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10025-6">
    <skos:narrower rdf:resource="descriptor/19460-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10025-6">
    <skos:narrower rdf:resource="descriptor/10241-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10025-6">
    <skos:narrower rdf:resource="descriptor/10237-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12589-6">
    <zbwext:useInsteadNote rdf:nodeID="id250"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15677-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12520-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18799-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13173-4">
    <skos:prefLabel xml:lang="en">Building construction</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13173-4">
    <skos:prefLabel xml:lang="de">Hochbau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13007-0">
    <skos:prefLabel xml:lang="en">Tyre industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13007-0">
    <skos:prefLabel xml:lang="de">Reifenindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10214-3">
    <skos:altLabel xml:lang="en">Kinked demand curve</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10214-3">
    <skos:altLabel xml:lang="de">Preis-Absatzfunktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10214-3">
    <skos:altLabel xml:lang="de">Geknickte Preis-Absatz-Funktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70169">
    <skos:narrower rdf:resource="descriptor/19024-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70169">
    <skos:narrower rdf:resource="descriptor/14119-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70169">
    <skos:narrower rdf:resource="descriptor/14118-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70169">
    <skos:narrower rdf:resource="descriptor/14102-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70169">
    <skos:narrower rdf:resource="descriptor/14117-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70169">
    <skos:narrower rdf:resource="descriptor/14116-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70169">
    <skos:narrower rdf:resource="descriptor/14081-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70169">
    <skos:narrower rdf:resource="descriptor/14076-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70169">
    <skos:narrower rdf:resource="descriptor/14110-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70169">
    <skos:narrower rdf:resource="descriptor/14109-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70169">
    <skos:narrower rdf:resource="descriptor/14066-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70169">
    <skos:narrower rdf:resource="descriptor/14107-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70169">
    <skos:narrower rdf:resource="descriptor/14106-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70169">
    <skos:narrower rdf:resource="descriptor/19018-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70169">
    <skos:narrower rdf:resource="descriptor/14105-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70169">
    <skos:narrower rdf:resource="descriptor/14104-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70169">
    <skos:narrower rdf:resource="descriptor/14056-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70169">
    <skos:narrower rdf:resource="descriptor/14101-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70169">
    <skos:narrower rdf:resource="descriptor/14100-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70169">
    <skos:narrower rdf:resource="descriptor/19014-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11272-5">
    <skos:scopeNote xml:lang="de">Wirtschaftliche Tätigkeit in selbständigem oder nicht-selbständigem Arbeitsverhältnis; auch für theoretische Analysen zur Entscheidung über Erwerbstätigkeit bzw. Nichterwerbstätigkeit.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16489-4">
    <skos:scopeNote xml:lang="de">Politik eines Staates/einer Staatengruppe in Bezug auf Drittstaaten.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18794-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16277-5">
    <skos:altLabel xml:lang="de">Politischer Entscheidungsprozess</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16277-5">
    <skos:altLabel xml:lang="de">Politischer Entscheidungsprozeß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15682-1">
    <skos:narrower rdf:resource="descriptor/15671-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15682-1">
    <skos:narrower rdf:resource="descriptor/19735-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11302-1">
    <skos:narrower rdf:resource="descriptor/18978-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11302-1">
    <skos:narrower rdf:resource="descriptor/12081-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11302-1">
    <skos:narrower rdf:resource="descriptor/16252-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11302-1">
    <skos:narrower rdf:resource="descriptor/18488-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11302-1">
    <skos:narrower rdf:resource="descriptor/18709-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11302-1">
    <skos:narrower rdf:resource="descriptor/11306-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10176-6">
    <skos:scopeNote xml:lang="de">Ökonomische Bewertung der Arbeiten im Haushalt, wie z.B. Kinderbetreuung, Krankenpflege, Verpflegung, Putzen, Einkaufen, usw.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19172-4">
    <skos:prefLabel xml:lang="en">Asia-Pacific Region</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19172-4">
    <skos:prefLabel xml:lang="de">Asiatisch-pazifischer Raum</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12798-4">
    <skos:related rdf:resource="descriptor/18532-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13199-0">
    <skos:altLabel xml:lang="de">Parfümerie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13199-0">
    <skos:altLabel xml:lang="de">Kosmetikfachhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13199-0">
    <skos:altLabel xml:lang="de">Körperpflegemittelfachhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13199-0">
    <skos:altLabel xml:lang="de">Drogeriemarkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10184-0">
    <skos:altLabel xml:lang="en">Theory of the allocation of time</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10184-0">
    <skos:altLabel xml:lang="de">Zeitallokationstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10184-0">
    <skos:altLabel xml:lang="de">Ökonomische Theorie der Zeitverwendung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10184-0">
    <skos:altLabel xml:lang="de">Freizeitnachfragetheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10184-0">
    <skos:altLabel xml:lang="de">Arbeitszeittheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18569-3">
    <skos:broader rdf:resource="thsys/71055"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18569-3">
    <skos:broader rdf:resource="thsys/70941"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18569-3">
    <skos:broader rdf:resource="descriptor/13979-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12314-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12901-2">
    <skos:prefLabel xml:lang="en">Fiduciary</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12901-2">
    <skos:prefLabel xml:lang="de">Treuhänder</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14914-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10641-5">
    <skos:altLabel xml:lang="en">Revealed comparative advantage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10641-5">
    <skos:altLabel xml:lang="en">RCA (Revealed comparative advantage)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10641-5">
    <skos:altLabel xml:lang="de">Theorie der komparativen Kosten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10641-5">
    <skos:altLabel xml:lang="de">Ricardo-Theorem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10641-5">
    <skos:altLabel xml:lang="de">Ricardo Theorem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15690-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13032-1">
    <skos:related rdf:resource="descriptor/18402-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13032-1">
    <skos:related rdf:resource="descriptor/14202-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13032-1">
    <skos:related rdf:resource="descriptor/15651-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13032-1">
    <skos:related rdf:resource="descriptor/12985-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18373-6">
    <skos:narrower rdf:resource="descriptor/11673-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11705-0">
    <skos:broader rdf:resource="thsys/71075"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11705-0">
    <skos:broader rdf:resource="descriptor/19408-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16308-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19662-1">
    <skos:related rdf:resource="descriptor/24859-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19662-1">
    <skos:related rdf:resource="descriptor/15761-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19662-1">
    <skos:related rdf:resource="descriptor/11638-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19662-1">
    <skos:related rdf:resource="descriptor/19560-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19662-1">
    <skos:related rdf:resource="descriptor/19728-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19662-1">
    <skos:related rdf:resource="descriptor/19715-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19662-1">
    <skos:related rdf:resource="descriptor/16195-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19662-1">
    <skos:related rdf:resource="descriptor/11551-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19121-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12302-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12121-0">
    <skos:prefLabel xml:lang="en">Organizational consultancy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12121-0">
    <skos:prefLabel xml:lang="de">Organisationsberatung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13453-5">
    <skos:altLabel xml:lang="de">Landwirtschaftlicher Nebenerwerb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13453-5">
    <skos:altLabel xml:lang="de">Agrarnebenerwerb</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18820-0">
    <skos:altLabel xml:lang="en">Fire brigade</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18820-0">
    <skos:altLabel xml:lang="de">Feuerwehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18820-0">
    <skos:altLabel xml:lang="de">Feuerschutz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16623-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14184-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15761-5">
    <skos:broader rdf:resource="thsys/70239"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15761-5">
    <skos:broader rdf:resource="descriptor/15768-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11558-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11784-6">
    <skos:related rdf:resource="descriptor/10966-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11784-6">
    <skos:related rdf:resource="descriptor/18456-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11784-6">
    <skos:related rdf:resource="descriptor/18010-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10049-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16305-5">
    <skos:narrower rdf:resource="descriptor/16312-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16305-5">
    <skos:narrower rdf:resource="descriptor/16298-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16305-5">
    <skos:narrower rdf:resource="descriptor/19681-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16305-5">
    <skos:narrower rdf:resource="descriptor/16314-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14035-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11556-5">
    <skos:narrower rdf:resource="descriptor/11829-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11556-5">
    <skos:narrower rdf:resource="descriptor/18806-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11556-5">
    <skos:narrower rdf:resource="descriptor/11552-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19768-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12570-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19001-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14098-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19265-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16745-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16594-0">
    <skos:related rdf:resource="descriptor/15720-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16058-3">
    <skos:related rdf:resource="descriptor/12302-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16058-3">
    <skos:related rdf:resource="descriptor/12303-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16058-3">
    <skos:related rdf:resource="descriptor/12575-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16058-3">
    <skos:related rdf:resource="descriptor/15977-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16058-3">
    <skos:related rdf:resource="descriptor/18791-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16058-3">
    <skos:related rdf:resource="descriptor/19159-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16058-3">
    <skos:related rdf:resource="descriptor/12277-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12357-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24847-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12268-5">
    <skos:prefLabel xml:lang="en">Banking interests</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12268-5">
    <skos:prefLabel xml:lang="de">Bankbeteiligung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19304-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18004-2">
    <skos:related rdf:resource="descriptor/14522-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18004-2">
    <skos:related rdf:resource="descriptor/18020-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18004-2">
    <skos:related rdf:resource="descriptor/13073-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18942-0">
    <skos:altLabel xml:lang="de">Wissenschaftlich-technische Information</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18942-0">
    <skos:altLabel xml:lang="de">Technische Dokumentation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18942-0">
    <skos:altLabel xml:lang="de">Naturwissenschaftlich-technische Information</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18156-0">
    <skos:broader rdf:resource="thsys/71083"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18156-0">
    <skos:broader rdf:resource="descriptor/11715-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13078-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13379-5">
    <skos:narrower rdf:resource="descriptor/20068-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13379-5">
    <skos:narrower rdf:resource="descriptor/13387-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70938">
    <skos:broader rdf:resource="thsys/70942"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14336-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19624-2">
    <skos:prefLabel xml:lang="en">Flood</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19624-2">
    <skos:prefLabel xml:lang="de">Überschwemmung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13490-6">
    <skos:related rdf:resource="descriptor/13493-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13490-6">
    <skos:related rdf:resource="descriptor/18551-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19388-2">
    <skos:prefLabel xml:lang="en">Petroleum consumption</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19388-2">
    <skos:prefLabel xml:lang="de">Erdölkonsum</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10418-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14123-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18650-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70036">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11273-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15810-4">
    <skos:related rdf:resource="descriptor/15811-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15810-4">
    <skos:related rdf:resource="descriptor/15722-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15810-4">
    <skos:related rdf:resource="descriptor/15807-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15810-4">
    <skos:related rdf:resource="descriptor/15806-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15810-4">
    <skos:related rdf:resource="descriptor/16615-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12574-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14163-2">
    <skos:prefLabel xml:lang="en">Salt water fish</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14163-2">
    <skos:prefLabel xml:lang="de">Seefisch</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18649-5">
    <skos:altLabel xml:lang="de">Telekommunikationsunternehmen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14456-1">
    <skos:altLabel xml:lang="de">Schiffszubehör</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73295">
    <skos:broader rdf:resource="thsys/70986"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16122-6">
    <skos:narrower rdf:resource="descriptor/10847-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14975-5">
    <skos:prefLabel xml:lang="en">Soft drink</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14975-5">
    <skos:prefLabel xml:lang="de">Erfrischungsgetränk</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73345">
    <skos:prefLabel xml:lang="de">N.10.06  Technik und Ingenieurwissenschaften</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17928-6">
    <skos:prefLabel xml:lang="en">Peru</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17928-6">
    <skos:prefLabel xml:lang="de">Peru</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13536-1">
    <skos:broader rdf:resource="thsys/70933"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13536-1">
    <skos:broader rdf:resource="descriptor/18010-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70999">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70852">
    <skos:narrower rdf:resource="descriptor/10966-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70852">
    <skos:narrower rdf:resource="descriptor/10029-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70852">
    <skos:narrower rdf:resource="descriptor/11350-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70852">
    <skos:narrower rdf:resource="descriptor/18851-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70852">
    <skos:narrower rdf:resource="descriptor/11818-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70852">
    <skos:narrower rdf:resource="descriptor/11822-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70852">
    <skos:narrower rdf:resource="descriptor/18840-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70852">
    <skos:narrower rdf:resource="descriptor/18850-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70852">
    <skos:narrower rdf:resource="descriptor/18544-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70852">
    <skos:narrower rdf:resource="descriptor/11304-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19916-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18790-4">
    <skos:prefLabel xml:lang="en">International commercial arbitration</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18790-4">
    <skos:prefLabel xml:lang="de">Internationale Schiedsgerichtsbarkeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19839-2">
    <skos:broader rdf:resource="thsys/73392"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19839-2">
    <skos:broader rdf:resource="descriptor/16030-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19247-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11357-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10068-2">
    <skos:broader rdf:resource="thsys/71100"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10068-2">
    <skos:broader rdf:resource="thsys/70234"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10068-2">
    <skos:broader rdf:resource="descriptor/10048-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11341-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71050">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14938-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18252-4">
    <skos:prefLabel xml:lang="en">Medical technology</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18252-4">
    <skos:prefLabel xml:lang="de">Medizintechnisches Gerät</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17723-3">
    <skos:prefLabel xml:lang="en">Togo</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17723-3">
    <skos:prefLabel xml:lang="de">Togo</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70318">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13437-3">
    <skos:altLabel xml:lang="en">Sharecropping</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13437-3">
    <skos:altLabel xml:lang="de">Agrarpacht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70897">
    <skos:prefLabel xml:lang="en">W.14.03.02  Financial economics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70897">
    <skos:prefLabel xml:lang="de">W.14.03.02  Kapitalmarkttheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70939">
    <skos:narrower rdf:resource="descriptor/18584-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70939">
    <skos:narrower rdf:resource="descriptor/18578-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70939">
    <skos:narrower rdf:resource="descriptor/18576-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70939">
    <skos:narrower rdf:resource="descriptor/13632-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70939">
    <skos:narrower rdf:resource="descriptor/11893-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70939">
    <skos:narrower rdf:resource="descriptor/18589-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70939">
    <skos:narrower rdf:resource="descriptor/18611-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70939">
    <skos:narrower rdf:resource="descriptor/13250-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70939">
    <skos:narrower rdf:resource="descriptor/13249-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70939">
    <skos:narrower rdf:resource="descriptor/18610-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70939">
    <skos:narrower rdf:resource="descriptor/13630-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70939">
    <skos:narrower rdf:resource="descriptor/18587-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70939">
    <skos:narrower rdf:resource="descriptor/18609-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70939">
    <skos:narrower rdf:resource="descriptor/18612-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70939">
    <skos:narrower rdf:resource="descriptor/13557-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70939">
    <skos:narrower rdf:resource="descriptor/18577-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70939">
    <skos:narrower rdf:resource="descriptor/13605-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70939">
    <skos:narrower rdf:resource="descriptor/18574-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70939">
    <skos:narrower rdf:resource="descriptor/13629-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11185-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19602-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://creativecommons.org/ns#attributionName">
    <rdf:type rdf:resource="http://www.w3.org/2002/07/owl#DatatypeProperty"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12036-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10785-2">
    <skos:prefLabel xml:lang="en">International payments</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10785-2">
    <skos:prefLabel xml:lang="de">Internationaler Zahlungsverkehr</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12236-4">
    <skos:prefLabel xml:lang="en">Bank guarantee</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12236-4">
    <skos:prefLabel xml:lang="de">Bankgarantie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10030-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13970-6">
    <skos:altLabel xml:lang="de">Warenkaufleute</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13970-6">
    <skos:altLabel xml:lang="de">Verkaufsleiterinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13970-6">
    <skos:altLabel xml:lang="de">Verkaufsleiter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13970-6">
    <skos:altLabel xml:lang="de">Tankwartinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13970-6">
    <skos:altLabel xml:lang="de">Tankwarte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13970-6">
    <skos:altLabel xml:lang="de">Händlerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13970-6">
    <skos:altLabel xml:lang="de">Händler</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13970-6">
    <skos:altLabel xml:lang="de">Großhandelskaufleute</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13970-6">
    <skos:altLabel xml:lang="de">Einzelhandelskaufleute</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13970-6">
    <skos:altLabel xml:lang="de">Einzelhandelsfilialleiterinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13970-6">
    <skos:altLabel xml:lang="de">Einzelhandelsfilialleiter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19152-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18812-6">
    <skos:prefLabel xml:lang="en">Energy pricing policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18812-6">
    <skos:prefLabel xml:lang="de">Energiepreispolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12208-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16422-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11301-3">
    <skos:broader rdf:resource="thsys/71058"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11301-3">
    <skos:broader rdf:resource="descriptor/11286-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11153-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11375-2">
    <skos:broader rdf:resource="thsys/70845"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11375-2">
    <skos:broader rdf:resource="descriptor/15909-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19258-1">
    <skos:broader rdf:resource="thsys/70306"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19258-1">
    <skos:broader rdf:resource="descriptor/10137-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71061">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18402-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19643-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11448-1">
    <skos:broader rdf:resource="thsys/70055"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11448-1">
    <skos:broader rdf:resource="descriptor/11464-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17576-4">
    <skos:broader rdf:resource="thsys/70327"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17576-4">
    <skos:broader rdf:resource="descriptor/17589-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17576-4">
    <skos:broader rdf:resource="descriptor/17979-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16275-2">
    <skos:prefLabel xml:lang="en">Political education</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16275-2">
    <skos:prefLabel xml:lang="de">Politische Bildung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11454-6">
    <skos:related rdf:resource="descriptor/11457-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19228-3">
    <skos:broader rdf:resource="thsys/73366"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19228-3">
    <skos:broader rdf:resource="descriptor/13420-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12649-0">
    <skos:prefLabel xml:lang="en">Materials</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12649-0">
    <skos:prefLabel xml:lang="de">Werkstoff</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14573-4">
    <skos:broader rdf:resource="thsys/70201"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18680-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10151-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10627-6">
    <skos:altLabel xml:lang="de">Wirtschaftsgemeinschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12155-4">
    <skos:altLabel xml:lang="en">Quality circle concept</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12155-4">
    <skos:altLabel xml:lang="de">Qualitätskreis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10876-6">
    <skos:narrower rdf:resource="descriptor/18543-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10876-6">
    <skos:narrower rdf:resource="descriptor/18542-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10876-6">
    <skos:narrower rdf:resource="descriptor/18541-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15905-0">
    <skos:broader rdf:resource="thsys/70142"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15905-0">
    <skos:broader rdf:resource="descriptor/15914-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11152-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id355">
    <rdfs:seeAlso rdf:resource="descriptor/15005-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71115">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.03.04</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70963">
    <skos:prefLabel xml:lang="en">W.10  Trade</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70963">
    <skos:prefLabel xml:lang="de">W.10  Handel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11239-3">
    <skos:related rdf:resource="descriptor/10406-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11239-3">
    <skos:related rdf:resource="descriptor/10401-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11239-3">
    <skos:related rdf:resource="descriptor/16011-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11239-3">
    <skos:related rdf:resource="descriptor/10383-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11239-3">
    <skos:related rdf:resource="descriptor/19111-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11239-3">
    <skos:related rdf:resource="descriptor/11235-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11239-3">
    <skos:related rdf:resource="descriptor/19493-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12307-0">
    <skos:altLabel xml:lang="en">Managerial accounting</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12307-0">
    <skos:altLabel xml:lang="en">Management accounting</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12307-0">
    <skos:altLabel xml:lang="de">Theorie des Rechnungswesens</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12307-0">
    <skos:altLabel xml:lang="de">Rechnungswesen des Betriebs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12307-0">
    <skos:altLabel xml:lang="de">Betriebswirtschaftliches Rechnungswesen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12307-0">
    <skos:altLabel xml:lang="de">Axiomatik des Rechnungswesens</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13135-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12172-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72184">
    <skos:narrower rdf:resource="descriptor/15093-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72184">
    <skos:narrower rdf:resource="descriptor/15088-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72184">
    <skos:narrower rdf:resource="descriptor/10901-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72184">
    <skos:narrower rdf:resource="descriptor/15086-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72184">
    <skos:narrower rdf:resource="descriptor/15097-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72184">
    <skos:narrower rdf:resource="descriptor/15082-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72184">
    <skos:narrower rdf:resource="descriptor/15080-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72184">
    <skos:narrower rdf:resource="descriptor/15077-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72184">
    <skos:narrower rdf:resource="descriptor/15078-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72184">
    <skos:narrower rdf:resource="descriptor/13712-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10055-4">
    <skos:prefLabel xml:lang="en">Price elasticity</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10055-4">
    <skos:prefLabel xml:lang="de">Preiselastizität</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18365-5">
    <skos:altLabel xml:lang="de">EU-Sektorenrichtlinie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18365-5">
    <skos:altLabel xml:lang="de">EU-Lieferkoordinierungsrichtlinie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18365-5">
    <skos:altLabel xml:lang="de">EU-Dienstleistungsrichtlinie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19193-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15494-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19781-0">
    <skos:altLabel xml:lang="en">Historic site</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19781-0">
    <skos:altLabel xml:lang="en">Historic monument</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19781-0">
    <skos:altLabel xml:lang="en">Cultural heritage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19781-0">
    <skos:altLabel xml:lang="en">Cultural good</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19781-0">
    <skos:altLabel xml:lang="de">Kulturelles Erbe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19781-0">
    <skos:altLabel xml:lang="de">Historisches Monument</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19781-0">
    <skos:altLabel xml:lang="de">Denkmal</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73298">
    <skos:prefLabel xml:lang="en">P.20.07  Luxury foodstuffs and tobacco</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73298">
    <skos:prefLabel xml:lang="de">P.20.07  Genussmittel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#related">
    <skos:definition xml:lang="en">A concept with which there is an associative semantic
      relationship.</skos:definition>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18337-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18748-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16402-0">
    <skos:broader rdf:resource="thsys/73383"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16402-0">
    <skos:broader rdf:resource="descriptor/16405-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70970">
    <skos:prefLabel xml:lang="en">W.07  Construction industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70970">
    <skos:prefLabel xml:lang="de">W.07  Bauwirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18142-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20442-5">
    <skos:related rdf:resource="descriptor/12665-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20442-5">
    <skos:related rdf:resource="descriptor/19133-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10521-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18206-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16340-3">
    <skos:broader rdf:resource="thsys/73379"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16340-3">
    <skos:broader rdf:resource="descriptor/19689-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12184-4">
    <skos:related rdf:resource="descriptor/12198-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12184-4">
    <skos:related rdf:resource="descriptor/12182-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18160-2">
    <skos:altLabel xml:lang="en">Locational competition</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18160-2">
    <skos:altLabel xml:lang="de">Standortkonkurrenz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18578-2">
    <skos:narrower rdf:resource="descriptor/18584-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18578-2">
    <skos:narrower rdf:resource="descriptor/13250-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18578-2">
    <skos:narrower rdf:resource="descriptor/13630-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18578-2">
    <skos:narrower rdf:resource="descriptor/13557-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18578-2">
    <skos:narrower rdf:resource="descriptor/18577-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73297">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11424-1">
    <skos:prefLabel xml:lang="en">Money creation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11424-1">
    <skos:prefLabel xml:lang="de">Geldschöpfung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16362-0">
    <skos:related rdf:resource="descriptor/16410-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18861-0">
    <skos:narrower rdf:resource="descriptor/15927-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18861-0">
    <skos:narrower rdf:resource="descriptor/15920-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18861-0">
    <skos:narrower rdf:resource="descriptor/15904-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70554">
    <skos:broader rdf:resource="thsys/70002"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12727-6">
    <skos:altLabel xml:lang="en">Wholesale manufacturing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12727-6">
    <skos:altLabel xml:lang="en">Serial production</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12727-6">
    <skos:altLabel xml:lang="de">Sortenproduktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12727-6">
    <skos:altLabel xml:lang="de">Sortenfertigung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12727-6">
    <skos:altLabel xml:lang="de">Serienproduktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12727-6">
    <skos:altLabel xml:lang="de">Großserienproduktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11057-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17672-1">
    <skos:narrower rdf:resource="descriptor/17679-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17672-1">
    <skos:narrower rdf:resource="descriptor/17678-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17672-1">
    <skos:narrower rdf:resource="descriptor/17677-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17672-1">
    <skos:narrower rdf:resource="descriptor/17676-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17672-1">
    <skos:narrower rdf:resource="descriptor/17675-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17672-1">
    <skos:narrower rdf:resource="descriptor/17674-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17672-1">
    <skos:narrower rdf:resource="descriptor/17738-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17672-1">
    <skos:narrower rdf:resource="descriptor/17669-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17672-1">
    <skos:narrower rdf:resource="descriptor/17668-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17672-1">
    <skos:narrower rdf:resource="descriptor/17666-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17672-1">
    <skos:narrower rdf:resource="descriptor/17665-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17672-1">
    <skos:narrower rdf:resource="descriptor/17662-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17672-1">
    <skos:narrower rdf:resource="descriptor/17660-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17672-1">
    <skos:narrower rdf:resource="descriptor/18284-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17672-1">
    <skos:narrower rdf:resource="descriptor/17658-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17672-1">
    <skos:narrower rdf:resource="descriptor/17731-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17672-1">
    <skos:narrower rdf:resource="descriptor/17654-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13039-1">
    <skos:related rdf:resource="descriptor/18406-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13039-1">
    <skos:related rdf:resource="descriptor/19051-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13039-1">
    <skos:related rdf:resource="descriptor/13029-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13039-1">
    <skos:related rdf:resource="descriptor/14272-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15699-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11616-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18245-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18867-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19386-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16768-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14173-6">
    <skos:prefLabel xml:lang="en">Fuel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14173-6">
    <skos:prefLabel xml:lang="de">Brennstoff</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16477-4">
    <skos:prefLabel xml:lang="en">Family policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16477-4">
    <skos:prefLabel xml:lang="de">Familienpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17738-4">
    <skos:prefLabel xml:lang="en">Rwanda</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17738-4">
    <skos:prefLabel xml:lang="de">Ruanda</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18791-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10458-3">
    <skos:broader rdf:resource="thsys/71109"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10458-3">
    <skos:broader rdf:resource="thsys/70878"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10458-3">
    <skos:broader rdf:resource="thsys/70872"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10458-3">
    <skos:broader rdf:resource="descriptor/13303-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10458-3">
    <skos:broader rdf:resource="descriptor/10436-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10606-0">
    <skos:broader rdf:resource="thsys/73390"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10606-0">
    <skos:broader rdf:resource="thsys/71098"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10606-0">
    <skos:broader rdf:resource="thsys/70952"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10606-0">
    <skos:broader rdf:resource="descriptor/16081-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12913-2">
    <skos:scopeNote xml:lang="de">Vgl. auch Verarbeitendes Gewerbe.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10239-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14296-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14662-5">
    <skos:related rdf:resource="descriptor/14603-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19751-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24785-4">
    <skos:prefLabel xml:lang="en">Customer satisfaction</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24785-4">
    <skos:prefLabel xml:lang="de">Kundenzufriedenheit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10560-5">
    <skos:related rdf:resource="descriptor/10987-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10560-5">
    <skos:related rdf:resource="descriptor/10586-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10560-5">
    <skos:related rdf:resource="descriptor/20842-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10560-5">
    <skos:related rdf:resource="descriptor/10525-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10560-5">
    <skos:related rdf:resource="descriptor/10558-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10560-5">
    <skos:related rdf:resource="descriptor/10492-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10560-5">
    <skos:related rdf:resource="descriptor/10557-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10560-5">
    <skos:related rdf:resource="descriptor/10555-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10560-5">
    <skos:related rdf:resource="descriptor/10513-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10560-5">
    <skos:related rdf:resource="descriptor/16533-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18681-2">
    <skos:broader rdf:resource="thsys/71107"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18681-2">
    <skos:broader rdf:resource="descriptor/12280-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19763-2">
    <skos:prefLabel xml:lang="en">Offenders</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19763-2">
    <skos:prefLabel xml:lang="de">Straftäter</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16283-3">
    <skos:broader rdf:resource="thsys/73375"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16283-3">
    <skos:broader rdf:resource="thsys/73373"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16283-3">
    <skos:broader rdf:resource="descriptor/16290-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11123-1">
    <skos:altLabel xml:lang="de">Sozialistische Länder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11123-1">
    <skos:altLabel xml:lang="de">Ostblockstaaten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11123-1">
    <skos:altLabel xml:lang="de">Kommunistische Staaten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11123-1">
    <skos:altLabel xml:lang="de">Kommunistische Länder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10404-5">
    <skos:broader rdf:resource="thsys/71113"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10404-5">
    <skos:broader rdf:resource="descriptor/11704-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10404-5">
    <skos:broader rdf:resource="descriptor/10380-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19492-0">
    <skos:related rdf:resource="descriptor/11258-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19492-0">
    <skos:related rdf:resource="descriptor/19439-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19492-0">
    <skos:related rdf:resource="descriptor/10613-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19492-0">
    <skos:related rdf:resource="descriptor/11888-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10380-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13727-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18008-1">
    <skos:prefLabel xml:lang="en">International agricultural trade</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18008-1">
    <skos:prefLabel xml:lang="de">Agraraußenhandel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17039-2">
    <skos:prefLabel xml:lang="en">Italy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17039-2">
    <skos:prefLabel xml:lang="de">Italien</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72185">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10080-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19546-3">
    <skos:prefLabel xml:lang="en">EU milk regulations</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19546-3">
    <skos:prefLabel xml:lang="de">EU-Milchmarktordnung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19047-0">
    <skos:broader rdf:resource="thsys/71006"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19047-0">
    <skos:broader rdf:resource="thsys/70876"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19047-0">
    <skos:broader rdf:resource="descriptor/18397-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19047-0">
    <skos:broader rdf:resource="descriptor/12920-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13202-2">
    <skos:narrower rdf:resource="descriptor/13228-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13202-2">
    <skos:narrower rdf:resource="descriptor/18523-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19836-1">
    <skos:prefLabel xml:lang="en">Trend</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19836-1">
    <skos:prefLabel xml:lang="de">Trend</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11779-6">
    <skos:prefLabel xml:lang="en">Distributive politics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11779-6">
    <skos:prefLabel xml:lang="de">Verteilungspolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14805-2">
    <skos:prefLabel xml:lang="en">Cardboard</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14805-2">
    <skos:prefLabel xml:lang="de">Pappe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12741-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12872-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19621-1">
    <skos:prefLabel xml:lang="en">Ice</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19621-1">
    <skos:prefLabel xml:lang="de">Eis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14824-5">
    <skos:prefLabel xml:lang="en">Tyre</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14824-5">
    <skos:prefLabel xml:lang="de">Reifen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11602-3">
    <skos:prefLabel xml:lang="en">Horizontal fiscal transfer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11602-3">
    <skos:prefLabel xml:lang="de">Horizontaler Finanzausgleich</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70969">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19122-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12953-4">
    <skos:broader rdf:resource="thsys/71025"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12953-4">
    <skos:broader rdf:resource="descriptor/12955-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17570-2">
    <skos:prefLabel xml:lang="en">South Korea</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17570-2">
    <skos:prefLabel xml:lang="de">Südkorea</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15814-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15328-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id156">
    <rdf:value xml:lang="de">Für die Normen des Völkerrechts</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71092">
    <skos:narrower rdf:resource="descriptor/19354-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71092">
    <skos:narrower rdf:resource="descriptor/20810-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71092">
    <skos:narrower rdf:resource="descriptor/10734-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71092">
    <skos:narrower rdf:resource="descriptor/19336-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71092">
    <skos:narrower rdf:resource="descriptor/16495-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71092">
    <skos:narrower rdf:resource="descriptor/19353-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71092">
    <skos:narrower rdf:resource="descriptor/19352-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71092">
    <skos:narrower rdf:resource="descriptor/10608-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71092">
    <skos:narrower rdf:resource="descriptor/19663-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71092">
    <skos:narrower rdf:resource="descriptor/10733-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71092">
    <skos:narrower rdf:resource="descriptor/19350-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13161-4">
    <skos:related rdf:resource="descriptor/14948-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13161-4">
    <skos:related rdf:resource="descriptor/19342-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13161-4">
    <skos:related rdf:resource="descriptor/12947-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13161-4">
    <skos:related rdf:resource="descriptor/14110-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13161-4">
    <skos:related rdf:resource="descriptor/18445-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13161-4">
    <skos:related rdf:resource="descriptor/19341-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13161-4">
    <skos:related rdf:resource="descriptor/12942-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13161-4">
    <skos:related rdf:resource="descriptor/14105-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13161-4">
    <skos:related rdf:resource="descriptor/13225-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12272-0">
    <skos:related rdf:resource="descriptor/12230-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12272-0">
    <skos:related rdf:resource="descriptor/12295-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12272-0">
    <skos:related rdf:resource="descriptor/10893-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12272-0">
    <skos:related rdf:resource="descriptor/19265-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12272-0">
    <skos:related rdf:resource="descriptor/19537-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12027-6">
    <skos:altLabel xml:lang="de">Wohnungsverwaltung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12027-6">
    <skos:altLabel xml:lang="de">Immobilienmanagement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12027-6">
    <skos:altLabel xml:lang="de">Immobilienbewirtschaftung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12027-6">
    <skos:altLabel xml:lang="de">Grundstücksverwaltung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71017">
    <skos:prefLabel xml:lang="de">W.04  Energie- und Wasserwirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14433-6">
    <skos:related rdf:resource="descriptor/13558-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14433-6">
    <skos:related rdf:resource="descriptor/18421-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14433-6">
    <skos:related rdf:resource="descriptor/11667-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14433-6">
    <skos:related rdf:resource="descriptor/19549-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14433-6">
    <skos:related rdf:resource="descriptor/13051-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14433-6">
    <skos:related rdf:resource="descriptor/13052-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14433-6">
    <skos:related rdf:resource="descriptor/13217-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14433-6">
    <skos:related rdf:resource="descriptor/13813-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14433-6">
    <skos:related rdf:resource="descriptor/18423-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14433-6">
    <skos:related rdf:resource="descriptor/13342-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14433-6">
    <skos:related rdf:resource="descriptor/13248-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18427-2">
    <skos:prefLabel xml:lang="en">Optoelectronics industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18427-2">
    <skos:prefLabel xml:lang="de">Optoelektronische Industrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17201-5">
    <skos:broader rdf:resource="thsys/70557"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17201-5">
    <skos:broader rdf:resource="descriptor/17199-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17201-5">
    <skos:broader rdf:resource="descriptor/17991-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17201-5">
    <skos:broader rdf:resource="descriptor/17983-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17201-5">
    <skos:broader rdf:resource="descriptor/17980-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18351-2">
    <skos:broader rdf:resource="thsys/72181"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18351-2">
    <skos:broader rdf:resource="thsys/71016"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18351-2">
    <skos:broader rdf:resource="thsys/70227"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18351-2">
    <skos:broader rdf:resource="descriptor/15076-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10057-0">
    <skos:altLabel xml:lang="en">Uncertainty</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10057-0">
    <skos:altLabel xml:lang="de">Wagnis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10057-0">
    <skos:altLabel xml:lang="de">Unsicherheit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10057-0">
    <skos:altLabel xml:lang="de">Ungewissheit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10057-0">
    <skos:altLabel xml:lang="de">Ungewißheit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10141-4">
    <skos:related rdf:resource="descriptor/11223-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10141-4">
    <skos:related rdf:resource="descriptor/16759-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10141-4">
    <skos:related rdf:resource="descriptor/10142-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10141-4">
    <skos:related rdf:resource="descriptor/18943-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10141-4">
    <skos:related rdf:resource="descriptor/10140-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11079-6">
    <skos:related rdf:resource="descriptor/11071-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14024-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18270-2">
    <skos:scopeNote xml:lang="de">Mehrere Prozessoren werden innerhalb eines Computers parallel betrieben.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10901-5">
    <skos:scopeNote xml:lang="de">Nur für das Maß der Wirtschaftskonzentration.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12306-2">
    <skos:related rdf:resource="descriptor/10011-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12306-2">
    <skos:related rdf:resource="descriptor/12311-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12306-2">
    <skos:related rdf:resource="descriptor/12309-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18679-3">
    <skos:related rdf:resource="descriptor/10135-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18679-3">
    <skos:related rdf:resource="descriptor/10773-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18679-3">
    <skos:related rdf:resource="descriptor/19241-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18679-3">
    <skos:related rdf:resource="descriptor/10213-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18679-3">
    <skos:related rdf:resource="descriptor/24855-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18679-3">
    <skos:related rdf:resource="descriptor/19669-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18679-3">
    <skos:related rdf:resource="descriptor/13715-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12371-5">
    <skos:related rdf:resource="descriptor/12403-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12371-5">
    <skos:related rdf:resource="descriptor/12275-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13666-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13399-6">
    <skos:prefLabel xml:lang="en">Religious community</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13399-6">
    <skos:prefLabel xml:lang="de">Religionsgemeinschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13670-4">
    <skos:broader rdf:resource="thsys/70265"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13670-4">
    <skos:broader rdf:resource="descriptor/18211-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19312-0">
    <skos:broader rdf:resource="thsys/71100"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19312-0">
    <skos:broader rdf:resource="descriptor/10629-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11823-1">
    <skos:related rdf:resource="descriptor/11824-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11823-1">
    <skos:related rdf:resource="descriptor/11822-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11823-1">
    <skos:related rdf:resource="descriptor/13460-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11823-1">
    <skos:related rdf:resource="descriptor/13210-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11823-1">
    <skos:related rdf:resource="descriptor/10686-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19326-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73344">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14777-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11207-2">
    <skos:scopeNote xml:lang="de">Der Arbeitsmarkt einer Region analysiert im Verhältnis zu einer übergeordneten räumlichen Einheit.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12053-5">
    <skos:altLabel xml:lang="en">Operating ratios</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12053-5">
    <skos:altLabel xml:lang="en">KPI (Key performance indicator)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12053-5">
    <skos:altLabel xml:lang="en">Key performance indicator</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12053-5">
    <skos:altLabel xml:lang="de">Kennzahl zum Betriebsvergleich</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12053-5">
    <skos:altLabel xml:lang="de">Finanzkennzahl</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12053-5">
    <skos:altLabel xml:lang="de">Bilanzkennzahl</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12214-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70927">
    <skos:narrower rdf:resource="descriptor/13613-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70927">
    <skos:narrower rdf:resource="descriptor/11834-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70927">
    <skos:narrower rdf:resource="descriptor/13612-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70927">
    <skos:narrower rdf:resource="descriptor/13611-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70927">
    <skos:narrower rdf:resource="descriptor/13608-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70927">
    <skos:narrower rdf:resource="descriptor/15519-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70927">
    <skos:narrower rdf:resource="descriptor/18620-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70927">
    <skos:narrower rdf:resource="descriptor/11934-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70927">
    <skos:narrower rdf:resource="descriptor/15518-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70927">
    <skos:narrower rdf:resource="descriptor/13600-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18126-2">
    <skos:altLabel xml:lang="en">Service trade</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18126-2">
    <skos:altLabel xml:lang="en">Invisible trade</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18126-2">
    <skos:altLabel xml:lang="de">Internationaler Dienstleistungshandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18126-2">
    <skos:altLabel xml:lang="de">Dienstleistungsaußenhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10576-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14586-2">
    <skos:prefLabel xml:lang="en">Sporting goods</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14586-2">
    <skos:prefLabel xml:lang="de">Sportartikel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19647-4">
    <skos:narrower rdf:resource="descriptor/11902-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19647-4">
    <skos:narrower rdf:resource="descriptor/10504-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18237-0">
    <skos:altLabel xml:lang="de">Kadmium</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14040-4">
    <skos:prefLabel xml:lang="en">Work of art</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14040-4">
    <skos:prefLabel xml:lang="de">Kunstgegenstand</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19056-6">
    <skos:altLabel xml:lang="de">Verputzarbeiten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19056-6">
    <skos:altLabel xml:lang="de">Stukkateurgewerbe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19056-6">
    <skos:altLabel xml:lang="de">Putzerei</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19056-6">
    <skos:altLabel xml:lang="de">Gipserei</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14455-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10107-4">
    <skos:broader rdf:resource="thsys/70274"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10107-4">
    <skos:broader rdf:resource="descriptor/10975-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24662-6">
    <skos:broader rdf:resource="thsys/71071"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24662-6">
    <skos:broader rdf:resource="thsys/70867"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24662-6">
    <skos:broader rdf:resource="descriptor/10964-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18331-1">
    <skos:broader rdf:resource="thsys/73362"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18331-1">
    <skos:broader rdf:resource="thsys/71020"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18331-1">
    <skos:broader rdf:resource="descriptor/11806-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18331-1">
    <skos:broader rdf:resource="descriptor/11802-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19569-5">
    <skos:altLabel xml:lang="en">Bootstrapping</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19569-5">
    <skos:altLabel xml:lang="en">Bootstrap approach</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19569-5">
    <skos:altLabel xml:lang="de">Bootstrap-Statistik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19074-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16396-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14510-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15976-5">
    <skos:prefLabel xml:lang="en">Jurisprudence</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15976-5">
    <skos:prefLabel xml:lang="de">Rechtswissenschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16645-2">
    <skos:prefLabel xml:lang="en">Wealth</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16645-2">
    <skos:prefLabel xml:lang="de">Reichtum</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19151-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13069-6">
    <skos:related rdf:resource="descriptor/14444-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16258-2">
    <skos:altLabel xml:lang="en">Political legitimacy</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16258-2">
    <skos:altLabel xml:lang="de">Politische Legitimität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16258-2">
    <skos:altLabel xml:lang="de">Legitimitätsprinzip</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16258-2">
    <skos:altLabel xml:lang="de">Legitimitätskrise</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14164-0">
    <skos:altLabel xml:lang="de">Salm</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14164-0">
    <skos:altLabel xml:lang="de">Lachs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14164-0">
    <skos:altLabel xml:lang="de">Karpfen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14164-0">
    <skos:altLabel xml:lang="de">Hecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14164-0">
    <skos:altLabel xml:lang="de">Forelle</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14164-0">
    <skos:altLabel xml:lang="de">Barsch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14164-0">
    <skos:altLabel xml:lang="de">Aal</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19597-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11848-6">
    <skos:scopeNote xml:lang="de">Verschiedene rechtliche Regelungen, die das Wohnen betreffen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12013-3">
    <skos:altLabel xml:lang="de">Transportinfrastruktur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13572-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11414-4">
    <skos:prefLabel xml:lang="en">Financial intermediation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11414-4">
    <skos:prefLabel xml:lang="de">Finanzintermediär</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18754-1">
    <skos:scopeNote xml:lang="de">Zur sozialen Wohnungspolitik gehört auch die Förderung des Wohnungsbaus für Unterschichten in Entwicklungsländern.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19439-4">
    <zbwext:useInsteadNote rdf:nodeID="id279"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id16">
    <rdfs:seeAlso rdf:resource="descriptor/16556-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11857-5">
    <skos:altLabel xml:lang="de">Strukturraum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11857-5">
    <skos:altLabel xml:lang="de">Sozialraum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11857-5">
    <skos:altLabel xml:lang="de">Regionsabgrenzung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11857-5">
    <skos:altLabel xml:lang="de">Raum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11857-5">
    <skos:altLabel xml:lang="de">Planungsregion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11857-5">
    <skos:altLabel xml:lang="de">Planungsraum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11857-5">
    <skos:altLabel xml:lang="de">Gebiet</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12510-3">
    <skos:altLabel xml:lang="en">Selling costs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12510-3">
    <skos:altLabel xml:lang="de">Warenverteilungskosten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12510-3">
    <skos:altLabel xml:lang="de">Marketingkosten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12510-3">
    <skos:altLabel xml:lang="de">Distributionskosten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12510-3">
    <skos:altLabel xml:lang="de">Absatzkosten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11917-6">
    <skos:broader rdf:resource="thsys/70058"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11917-6">
    <skos:broader rdf:resource="descriptor/11857-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19792-2">
    <skos:altLabel xml:lang="en">Foodborne illness</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19792-2">
    <skos:altLabel xml:lang="en">Food safety</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19792-2">
    <skos:altLabel xml:lang="de">Nahrungsmittelvergiftung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19792-2">
    <skos:altLabel xml:lang="de">Nahrungsmittelsicherheit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19792-2">
    <skos:altLabel xml:lang="de">Lebensmittelvergiftung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19792-2">
    <skos:altLabel xml:lang="de">Lebensmittelbelastung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70868">
    <skos:narrower rdf:resource="descriptor/13419-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70868">
    <skos:narrower rdf:resource="descriptor/11237-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70868">
    <skos:narrower rdf:resource="descriptor/11236-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70868">
    <skos:narrower rdf:resource="descriptor/13406-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11187-3">
    <skos:prefLabel xml:lang="en">Working time</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11187-3">
    <skos:prefLabel xml:lang="de">Arbeitszeit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13621-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19551-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17104-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13957-5">
    <skos:broader rdf:resource="thsys/71056"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13957-5">
    <skos:broader rdf:resource="descriptor/13962-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15903-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17641-5">
    <skos:broader rdf:resource="thsys/70253"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17641-5">
    <skos:broader rdf:resource="descriptor/17608-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17641-5">
    <skos:broader rdf:resource="descriptor/17983-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11900-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11895-4">
    <skos:related rdf:resource="descriptor/11917-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11895-4">
    <skos:related rdf:resource="descriptor/16552-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11895-4">
    <skos:related rdf:resource="descriptor/11943-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11895-4">
    <skos:related rdf:resource="descriptor/19246-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11895-4">
    <skos:related rdf:resource="descriptor/11909-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11895-4">
    <skos:related rdf:resource="descriptor/11911-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11895-4">
    <skos:related rdf:resource="descriptor/11894-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11895-4">
    <skos:related rdf:resource="descriptor/11857-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19542-4">
    <skos:scopeNote xml:lang="de">Verfahren zur Ermittlung von (zumeist) Konsumentenpräferenzen in der Marktforschung aufgrund von Einzelurteilen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11665-0">
    <skos:related rdf:resource="descriptor/16002-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11665-0">
    <skos:related rdf:resource="descriptor/13393-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11665-0">
    <skos:related rdf:resource="descriptor/11650-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14438-3">
    <skos:prefLabel xml:lang="en">Commercial vehicle</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14438-3">
    <skos:prefLabel xml:lang="de">Nutzfahrzeug</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13949-4">
    <skos:related rdf:resource="descriptor/13160-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13949-4">
    <skos:related rdf:resource="descriptor/13974-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13949-4">
    <skos:related rdf:resource="descriptor/13975-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15678-6">
    <skos:prefLabel xml:lang="en">Theology</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15678-6">
    <skos:prefLabel xml:lang="de">Theologie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19796-1">
    <skos:related rdf:resource="descriptor/10462-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19796-1">
    <skos:related rdf:resource="descriptor/14465-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19796-1">
    <skos:related rdf:resource="descriptor/15149-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13793-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10224-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13860-6">
    <skos:altLabel xml:lang="de">Wohnwissenschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13860-6">
    <skos:altLabel xml:lang="de">Wohnforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10789-1">
    <skos:related rdf:resource="descriptor/10785-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18732-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18828-5">
    <skos:related rdf:resource="descriptor/10956-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18828-5">
    <skos:related rdf:resource="descriptor/10894-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18828-5">
    <skos:related rdf:resource="descriptor/24662-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16562-6">
    <skos:altLabel xml:lang="de">Sozialökonomie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16547-2">
    <skos:related rdf:resource="descriptor/13824-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16547-2">
    <skos:related rdf:resource="descriptor/21002-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16547-2">
    <skos:related rdf:resource="descriptor/12502-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16547-2">
    <skos:related rdf:resource="descriptor/19514-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15923-5">
    <skos:broader rdf:resource="thsys/70142"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15923-5">
    <skos:broader rdf:resource="descriptor/15918-5"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id246">
    <rdf:value xml:lang="de">Für die politische Einflussnahme von Verbänden u.ä.</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11916-1">
    <skos:broader rdf:resource="thsys/70058"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11916-1">
    <skos:broader rdf:resource="descriptor/11857-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14118-0">
    <skos:related rdf:resource="descriptor/12960-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15684-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18186-5">
    <skos:altLabel xml:lang="de">UmweltHG</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18186-5">
    <skos:altLabel xml:lang="de">Umwelthaftungsrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18186-5">
    <skos:altLabel xml:lang="de">Umwelthaftungsgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id247">
    <rdfs:seeAlso rdf:resource="descriptor/16266-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13777-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15458-6">
    <skos:altLabel xml:lang="en">Theory of games</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15458-6">
    <skos:altLabel xml:lang="de">Theorie der strategischen Spiele</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15458-6">
    <skos:altLabel xml:lang="de">Strategisches Spiel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14121-4">
    <skos:prefLabel xml:lang="en">Egg</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14121-4">
    <skos:prefLabel xml:lang="de">Ei</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70227">
    <rdfs:label xml:lang="en">Energy economics</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70227">
    <rdfs:label xml:lang="de">Energieökonomik</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18950-1">
    <skos:related rdf:resource="descriptor/16767-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18950-1">
    <skos:related rdf:resource="descriptor/18951-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18175-3">
    <skos:prefLabel xml:lang="en">Integrated information systems</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18175-3">
    <skos:prefLabel xml:lang="de">Integriertes Informationssystem</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10505-6">
    <skos:narrower rdf:resource="descriptor/10497-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18308-3">
    <skos:prefLabel xml:lang="en">Sectoral employment distribution</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18308-3">
    <skos:prefLabel xml:lang="de">Sektorale Beschäftigungsstruktur</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15900-3">
    <skos:related rdf:resource="descriptor/13578-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15900-3">
    <skos:related rdf:resource="descriptor/16173-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15900-3">
    <skos:related rdf:resource="descriptor/15899-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15900-3">
    <skos:related rdf:resource="descriptor/13684-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15900-3">
    <skos:related rdf:resource="descriptor/13507-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15900-3">
    <skos:related rdf:resource="descriptor/18418-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11810-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70825">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11648-0">
    <skos:related rdf:resource="descriptor/19407-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11648-0">
    <skos:related rdf:resource="descriptor/11661-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20503-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18869-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18828-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15979-6">
    <skos:altLabel xml:lang="en">Prison</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15979-6">
    <skos:altLabel xml:lang="en">Penitentiary</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15979-6">
    <skos:altLabel xml:lang="en">Jail</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15979-6">
    <skos:altLabel xml:lang="de">Zuchthaus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15979-6">
    <skos:altLabel xml:lang="de">Gefängnis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19581-1">
    <skos:altLabel xml:lang="en">Mathematical statistics</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19581-1">
    <skos:altLabel xml:lang="de">Theoretische Statistik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19581-1">
    <skos:altLabel xml:lang="de">Statistikwissenschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19581-1">
    <skos:altLabel xml:lang="de">Statistiktheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19581-1">
    <skos:altLabel xml:lang="de">Methodenlehre der Statistik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19581-1">
    <skos:altLabel xml:lang="de">Mathematische Statistik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18895-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13478-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13135-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12615-3">
    <skos:broader rdf:resource="thsys/70838"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12615-3">
    <skos:broader rdf:resource="thsys/70245"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12615-3">
    <skos:broader rdf:resource="descriptor/16479-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12615-3">
    <skos:broader rdf:resource="descriptor/12592-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18455-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19235-6">
    <skos:prefLabel xml:lang="en">Nationalization</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19235-6">
    <skos:prefLabel xml:lang="de">Nationalisierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19693-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18651-4">
    <skos:related rdf:resource="descriptor/18653-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18651-4">
    <skos:related rdf:resource="descriptor/18649-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18651-4">
    <skos:related rdf:resource="descriptor/13278-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18651-4">
    <skos:related rdf:resource="descriptor/13682-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10110-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18623-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19046-2">
    <skos:broader rdf:resource="thsys/71028"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19046-2">
    <skos:broader rdf:resource="thsys/70876"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19046-2">
    <skos:broader rdf:resource="descriptor/18397-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19046-2">
    <skos:broader rdf:resource="descriptor/12945-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19581-1">
    <skos:prefLabel xml:lang="en">Statistics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19581-1">
    <skos:prefLabel xml:lang="de">Statistische Methodenlehre</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17622-2">
    <skos:prefLabel xml:lang="en">Iran</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17622-2">
    <skos:prefLabel xml:lang="de">Iran</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13595-6">
    <skos:broader rdf:resource="thsys/70925"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13595-6">
    <skos:broader rdf:resource="descriptor/11897-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10768-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10838-0">
    <skos:altLabel xml:lang="de">Einfuhr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18072-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18936-2">
    <skos:prefLabel xml:lang="en">Television advertising</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18936-2">
    <skos:prefLabel xml:lang="de">Fernsehwerbung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71043">
    <skos:narrower rdf:resource="descriptor/12299-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71043">
    <skos:narrower rdf:resource="descriptor/12441-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71043">
    <skos:narrower rdf:resource="descriptor/18700-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71043">
    <skos:narrower rdf:resource="descriptor/12404-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71043">
    <skos:narrower rdf:resource="descriptor/12401-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71043">
    <skos:narrower rdf:resource="descriptor/12399-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71043">
    <skos:narrower rdf:resource="descriptor/12437-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71043">
    <skos:narrower rdf:resource="descriptor/12435-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71043">
    <skos:narrower rdf:resource="descriptor/12558-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71043">
    <skos:narrower rdf:resource="descriptor/12396-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71043">
    <skos:narrower rdf:resource="descriptor/12392-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71043">
    <skos:narrower rdf:resource="descriptor/12433-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71043">
    <skos:narrower rdf:resource="descriptor/12297-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71043">
    <skos:narrower rdf:resource="descriptor/12296-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71043">
    <skos:narrower rdf:resource="descriptor/12387-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71043">
    <skos:narrower rdf:resource="descriptor/12291-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71043">
    <skos:narrower rdf:resource="descriptor/12385-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71043">
    <skos:narrower rdf:resource="descriptor/12384-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71043">
    <skos:narrower rdf:resource="descriptor/12427-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71043">
    <skos:narrower rdf:resource="descriptor/19093-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71043">
    <skos:narrower rdf:resource="descriptor/12341-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71043">
    <skos:narrower rdf:resource="descriptor/12340-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71043">
    <skos:narrower rdf:resource="descriptor/12226-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71043">
    <skos:narrower rdf:resource="descriptor/12282-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71043">
    <skos:narrower rdf:resource="descriptor/12425-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71043">
    <skos:narrower rdf:resource="descriptor/12376-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71043">
    <skos:narrower rdf:resource="descriptor/12314-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71043">
    <skos:narrower rdf:resource="descriptor/12424-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71043">
    <skos:narrower rdf:resource="descriptor/12337-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71043">
    <skos:narrower rdf:resource="descriptor/12277-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71043">
    <skos:narrower rdf:resource="descriptor/12423-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71043">
    <skos:narrower rdf:resource="descriptor/12274-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71043">
    <skos:narrower rdf:resource="descriptor/12368-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71043">
    <skos:narrower rdf:resource="descriptor/12420-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71043">
    <skos:narrower rdf:resource="descriptor/24666-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71043">
    <skos:narrower rdf:resource="descriptor/12331-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71043">
    <skos:narrower rdf:resource="descriptor/12365-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71043">
    <skos:narrower rdf:resource="descriptor/12418-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71043">
    <skos:narrower rdf:resource="descriptor/12273-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71043">
    <skos:narrower rdf:resource="descriptor/12330-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71043">
    <skos:narrower rdf:resource="descriptor/19934-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71043">
    <skos:narrower rdf:resource="descriptor/12325-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71043">
    <skos:narrower rdf:resource="descriptor/12458-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71043">
    <skos:narrower rdf:resource="descriptor/12363-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71043">
    <skos:narrower rdf:resource="descriptor/18302-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71043">
    <skos:narrower rdf:resource="descriptor/12326-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71043">
    <skos:narrower rdf:resource="descriptor/12217-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71043">
    <skos:narrower rdf:resource="descriptor/12361-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71043">
    <skos:narrower rdf:resource="descriptor/19142-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71043">
    <skos:narrower rdf:resource="descriptor/12432-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71043">
    <skos:narrower rdf:resource="descriptor/12408-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71043">
    <skos:narrower rdf:resource="descriptor/12270-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71043">
    <skos:narrower rdf:resource="descriptor/12216-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71043">
    <skos:narrower rdf:resource="descriptor/12357-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71043">
    <skos:narrower rdf:resource="descriptor/19092-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71043">
    <skos:narrower rdf:resource="descriptor/12356-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71043">
    <skos:narrower rdf:resource="descriptor/19158-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71043">
    <skos:narrower rdf:resource="descriptor/19094-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71043">
    <skos:narrower rdf:resource="descriptor/12366-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71043">
    <skos:narrower rdf:resource="descriptor/12354-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71043">
    <skos:narrower rdf:resource="descriptor/19091-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71043">
    <skos:narrower rdf:resource="descriptor/12407-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71043">
    <skos:narrower rdf:resource="descriptor/12406-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71043">
    <skos:narrower rdf:resource="descriptor/12543-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12125-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19431-6">
    <skos:scopeNote xml:lang="de">Netzwerkeffekte treten auf bei Produkten, bei denen der Nutzen für die Verbraucher/innen mit der Zahl anderer Nutzer/innen steigt, z.B. bei Computernetzen oder anderen technischen Systemen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18202-5">
    <skos:prefLabel xml:lang="en">Hardwood</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18202-5">
    <skos:prefLabel xml:lang="de">Hartholz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11843-2">
    <skos:broader rdf:resource="thsys/70967"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11843-2">
    <skos:broader rdf:resource="thsys/70888"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11843-2">
    <skos:broader rdf:resource="descriptor/11845-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11843-2">
    <skos:broader rdf:resource="descriptor/12932-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11843-2">
    <skos:broader rdf:resource="descriptor/18754-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13387-6">
    <skos:related rdf:resource="descriptor/20068-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13387-6">
    <skos:related rdf:resource="descriptor/14755-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18167-2">
    <skos:related rdf:resource="descriptor/14998-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15807-0">
    <skos:broader rdf:resource="thsys/73358"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19026-1">
    <skos:broader rdf:resource="thsys/70582"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18562-3">
    <skos:related rdf:resource="descriptor/18413-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18562-3">
    <skos:related rdf:resource="descriptor/14384-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18562-3">
    <skos:related rdf:resource="descriptor/13186-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18562-3">
    <skos:related rdf:resource="descriptor/14337-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18562-3">
    <skos:related rdf:resource="descriptor/13306-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19430-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11543-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19140-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20881-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12841-1">
    <skos:broader rdf:resource="thsys/70874"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12841-1">
    <skos:broader rdf:resource="thsys/70384"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12841-1">
    <skos:broader rdf:resource="thsys/70201"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12841-1">
    <skos:broader rdf:resource="descriptor/12846-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/a">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">A</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17680-2">
    <skos:altLabel xml:lang="de">Republik Botsuana</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17680-2">
    <skos:altLabel xml:lang="de">Betschuanaland</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19283-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19805-5">
    <skos:related rdf:resource="descriptor/18766-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19805-5">
    <skos:related rdf:resource="descriptor/19801-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19327-1">
    <skos:related rdf:resource="descriptor/10966-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19327-1">
    <skos:related rdf:resource="descriptor/10969-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19327-1">
    <skos:related rdf:resource="descriptor/10618-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19252-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14119-5">
    <skos:altLabel xml:lang="de">Zitrone</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14119-5">
    <skos:altLabel xml:lang="de">Tangerine</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14119-5">
    <skos:altLabel xml:lang="de">Pampelmuse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14119-5">
    <skos:altLabel xml:lang="de">Orange</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14119-5">
    <skos:altLabel xml:lang="de">Mandarine</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14119-5">
    <skos:altLabel xml:lang="de">Limone</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14119-5">
    <skos:altLabel xml:lang="de">Limette</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14119-5">
    <skos:altLabel xml:lang="de">Grapefruit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14119-5">
    <skos:altLabel xml:lang="de">Clementine</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14119-5">
    <skos:altLabel xml:lang="de">Apfelsine</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11231-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13207-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10725-6">
    <skos:narrower rdf:resource="descriptor/19333-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10725-6">
    <skos:narrower rdf:resource="descriptor/10719-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18754-1">
    <skos:broader rdf:resource="thsys/70888"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18754-1">
    <skos:broader rdf:resource="thsys/70833"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18754-1">
    <skos:broader rdf:resource="thsys/70324"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18754-1">
    <skos:broader rdf:resource="descriptor/11847-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18754-1">
    <skos:broader rdf:resource="descriptor/16545-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15429-6">
    <skos:altLabel xml:lang="en">GST (General Systems Theory)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15429-6">
    <skos:altLabel xml:lang="en">General systems theory</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15429-6">
    <skos:altLabel xml:lang="de">Allgemeine Systemtheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16549-5">
    <skos:narrower rdf:resource="descriptor/18633-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16549-5">
    <skos:narrower rdf:resource="descriptor/11844-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16549-5">
    <skos:narrower rdf:resource="descriptor/19446-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16549-5">
    <skos:narrower rdf:resource="descriptor/19378-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14741-2">
    <skos:altLabel xml:lang="de">Weichspüler</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14741-2">
    <skos:altLabel xml:lang="de">Wasch- und Reinigungsmittel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14741-2">
    <skos:altLabel xml:lang="de">Reinigungsmittel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14741-2">
    <skos:altLabel xml:lang="de">Detergenzien</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13655-0">
    <skos:prefLabel xml:lang="en">Television</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13655-0">
    <skos:prefLabel xml:lang="de">Fernsehen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19398-6">
    <skos:related rdf:resource="descriptor/13362-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19398-6">
    <skos:related rdf:resource="descriptor/11578-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19398-6">
    <skos:related rdf:resource="descriptor/18803-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70881">
    <skos:narrower rdf:resource="descriptor/18761-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70881">
    <skos:narrower rdf:resource="descriptor/13390-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70881">
    <skos:narrower rdf:resource="descriptor/18766-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11607-0">
    <skos:related rdf:resource="descriptor/10022-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11607-0">
    <skos:related rdf:resource="descriptor/11613-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11607-0">
    <skos:related rdf:resource="descriptor/11731-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17642-3">
    <skos:prefLabel xml:lang="en">Africa</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17642-3">
    <skos:prefLabel xml:lang="de">Afrika</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10160-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11995-0">
    <skos:altLabel xml:lang="de">Kommunale Bauleitplanung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11995-0">
    <skos:altLabel xml:lang="de">Flächennutzungsplan</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11995-0">
    <skos:altLabel xml:lang="de">Bebauungsplan</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15633-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12177-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14292-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19017-2">
    <skos:altLabel xml:lang="de">Cannabis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70355">
    <skos:narrower rdf:resource="descriptor/11523-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70355">
    <skos:narrower rdf:resource="descriptor/10412-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70355">
    <skos:narrower rdf:resource="descriptor/10978-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70355">
    <skos:narrower rdf:resource="descriptor/19225-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70355">
    <skos:narrower rdf:resource="descriptor/15104-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70355">
    <skos:narrower rdf:resource="descriptor/10397-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70355">
    <skos:narrower rdf:resource="descriptor/11519-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70355">
    <skos:narrower rdf:resource="descriptor/11518-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70355">
    <skos:narrower rdf:resource="descriptor/10213-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70355">
    <skos:narrower rdf:resource="descriptor/10255-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70355">
    <skos:narrower rdf:resource="descriptor/11516-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70355">
    <skos:narrower rdf:resource="descriptor/18149-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70355">
    <skos:narrower rdf:resource="descriptor/11510-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70355">
    <skos:narrower rdf:resource="descriptor/15069-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70355">
    <skos:narrower rdf:resource="descriptor/10667-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70355">
    <skos:narrower rdf:resource="descriptor/10242-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70355">
    <skos:narrower rdf:resource="descriptor/11507-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70355">
    <skos:narrower rdf:resource="descriptor/11495-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70355">
    <skos:narrower rdf:resource="descriptor/11494-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70355">
    <skos:narrower rdf:resource="descriptor/11506-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70355">
    <skos:narrower rdf:resource="descriptor/11504-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70355">
    <skos:narrower rdf:resource="descriptor/11503-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70355">
    <skos:narrower rdf:resource="descriptor/11501-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70355">
    <skos:narrower rdf:resource="descriptor/11498-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70355">
    <skos:narrower rdf:resource="descriptor/11496-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19345-6">
    <skos:broader rdf:resource="thsys/71025"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19345-6">
    <skos:broader rdf:resource="thsys/70960"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19345-6">
    <skos:broader rdf:resource="descriptor/19347-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15746-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10415-0">
    <skos:related rdf:resource="descriptor/10565-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10415-0">
    <skos:related rdf:resource="descriptor/10836-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16759-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10962-6">
    <skos:broader rdf:resource="thsys/71071"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10962-6">
    <skos:broader rdf:resource="thsys/70867"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10962-6">
    <skos:broader rdf:resource="descriptor/10963-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10962-6">
    <skos:broader rdf:resource="descriptor/18810-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11065-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18632-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16960-1">
    <skos:prefLabel xml:lang="en">Fribourg</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16960-1">
    <skos:prefLabel xml:lang="de">Freiburg (Kanton)</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16002-2">
    <skos:broader rdf:resource="thsys/70850"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16002-2">
    <skos:broader rdf:resource="thsys/70160"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16002-2">
    <skos:broader rdf:resource="descriptor/16007-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14160-1">
    <skos:broader rdf:resource="thsys/70506"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14160-1">
    <skos:broader rdf:resource="thsys/70403"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14160-1">
    <skos:broader rdf:resource="descriptor/14169-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14160-1">
    <skos:broader rdf:resource="descriptor/14161-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18195-4">
    <skos:narrower rdf:resource="descriptor/14117-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18195-4">
    <skos:narrower rdf:resource="descriptor/14157-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12337-5">
    <skos:related rdf:resource="descriptor/12340-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12337-5">
    <skos:related rdf:resource="descriptor/12381-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12337-5">
    <skos:related rdf:resource="descriptor/12458-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12337-5">
    <skos:related rdf:resource="descriptor/12363-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16119-2">
    <skos:altLabel xml:lang="en">Income-related expenses</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18433-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19579-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11576-6">
    <skos:altLabel xml:lang="en">Submission</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11576-6">
    <skos:altLabel xml:lang="de">Verdingung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11576-6">
    <skos:altLabel xml:lang="de">Öffentliche Ausschreibung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13357-1">
    <skos:related rdf:resource="descriptor/18926-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13357-1">
    <skos:related rdf:resource="descriptor/14040-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13357-1">
    <skos:related rdf:resource="descriptor/19781-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10826-0">
    <skos:prefLabel xml:lang="en">Foreign-based company</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10826-0">
    <skos:prefLabel xml:lang="de">Auslandssitz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14202-4">
    <skos:prefLabel xml:lang="en">Coke</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14202-4">
    <skos:prefLabel xml:lang="de">Koks</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16196-5">
    <skos:prefLabel xml:lang="en">Criminology</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16196-5">
    <skos:prefLabel xml:lang="de">Kriminologie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13454-3">
    <skos:broader rdf:resource="thsys/71065"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13454-3">
    <skos:broader rdf:resource="thsys/70142"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13454-3">
    <skos:broader rdf:resource="descriptor/15909-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14237-6">
    <skos:narrower rdf:resource="descriptor/19048-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14237-6">
    <skos:narrower rdf:resource="descriptor/14234-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14237-6">
    <skos:narrower rdf:resource="descriptor/14232-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14237-6">
    <skos:narrower rdf:resource="descriptor/14231-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14237-6">
    <skos:narrower rdf:resource="descriptor/14227-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19828-0">
    <skos:related rdf:resource="descriptor/12212-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19339-1">
    <skos:related rdf:resource="descriptor/18329-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19339-1">
    <skos:related rdf:resource="descriptor/12032-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19339-1">
    <skos:related rdf:resource="descriptor/12024-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19339-1">
    <skos:related rdf:resource="descriptor/14596-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10653-5">
    <skos:altLabel xml:lang="en">Traded good</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10653-5">
    <skos:altLabel xml:lang="en">Tradables</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10653-5">
    <skos:altLabel xml:lang="de">Außenhandelsgut</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14068-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10455-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18743-6">
    <skos:prefLabel xml:lang="en">Insurance management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18743-6">
    <skos:prefLabel xml:lang="de">Versicherungsmanagement</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19692-6">
    <skos:broader rdf:resource="thsys/73379"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70922">
    <skos:narrower rdf:resource="descriptor/18654-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70922">
    <skos:narrower rdf:resource="descriptor/13666-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70922">
    <skos:narrower rdf:resource="descriptor/18643-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70922">
    <skos:narrower rdf:resource="descriptor/18653-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70922">
    <skos:narrower rdf:resource="descriptor/18000-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70922">
    <skos:narrower rdf:resource="descriptor/18663-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70922">
    <skos:narrower rdf:resource="descriptor/18649-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70922">
    <skos:narrower rdf:resource="descriptor/18652-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70922">
    <skos:narrower rdf:resource="descriptor/18651-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70922">
    <skos:narrower rdf:resource="descriptor/13278-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70922">
    <skos:narrower rdf:resource="descriptor/13678-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70922">
    <skos:narrower rdf:resource="descriptor/18650-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70922">
    <skos:narrower rdf:resource="descriptor/18648-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70922">
    <skos:narrower rdf:resource="descriptor/13667-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70922">
    <skos:narrower rdf:resource="descriptor/13653-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70922">
    <skos:narrower rdf:resource="descriptor/18112-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70922">
    <skos:narrower rdf:resource="descriptor/13684-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70922">
    <skos:narrower rdf:resource="descriptor/21315-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70922">
    <skos:narrower rdf:resource="descriptor/18771-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70922">
    <skos:narrower rdf:resource="descriptor/13682-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70922">
    <skos:narrower rdf:resource="descriptor/18647-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70922">
    <skos:narrower rdf:resource="descriptor/18081-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70922">
    <skos:narrower rdf:resource="descriptor/18666-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70922">
    <skos:narrower rdf:resource="descriptor/18665-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70922">
    <skos:narrower rdf:resource="descriptor/21283-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70922">
    <skos:narrower rdf:resource="descriptor/18655-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70922">
    <skos:narrower rdf:resource="descriptor/13679-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70922">
    <skos:narrower rdf:resource="descriptor/13681-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70922">
    <skos:narrower rdf:resource="descriptor/13680-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70922">
    <skos:narrower rdf:resource="descriptor/20921-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70922">
    <skos:narrower rdf:resource="descriptor/18659-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70922">
    <skos:narrower rdf:resource="descriptor/19657-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70922">
    <skos:narrower rdf:resource="descriptor/19651-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70922">
    <skos:narrower rdf:resource="descriptor/13268-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70922">
    <skos:narrower rdf:resource="descriptor/18660-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70922">
    <skos:narrower rdf:resource="descriptor/13652-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70922">
    <skos:narrower rdf:resource="descriptor/18712-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70922">
    <skos:narrower rdf:resource="descriptor/19544-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70922">
    <skos:narrower rdf:resource="descriptor/19653-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70922">
    <skos:narrower rdf:resource="descriptor/19541-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70922">
    <skos:narrower rdf:resource="descriptor/13674-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70922">
    <skos:narrower rdf:resource="descriptor/13649-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18544-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70860">
    <rdfs:label xml:lang="de">Öffentliche Wirtschaft</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18856-0">
    <skos:scopeNote xml:lang="de">Personen, die einen Bildungs- oder Ausbildungsweg nicht zu Ende geführt haben.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14794-2">
    <skos:broader rdf:resource="thsys/70416"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14794-2">
    <skos:broader rdf:resource="descriptor/14788-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10976-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10454-4">
    <skos:prefLabel xml:lang="en">Mechanization</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10454-4">
    <skos:prefLabel xml:lang="de">Mechanisierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70833">
    <rdfs:label xml:lang="en">Social policy</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70833">
    <rdfs:label xml:lang="de">Sozialpolitik</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70992">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11268-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19654-0">
    <skos:altLabel xml:lang="en">Web-warehousing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19654-0">
    <skos:altLabel xml:lang="en">Data warehousing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70055">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16023-1">
    <skos:altLabel xml:lang="de">Wasserhaushaltsgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16023-1">
    <skos:altLabel xml:lang="de">Wassergesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16023-1">
    <skos:altLabel xml:lang="de">Waschmittelgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16023-1">
    <skos:altLabel xml:lang="de">Gewässerschutzrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16023-1">
    <skos:altLabel xml:lang="de">Abwasserabgabengesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16023-1">
    <skos:altLabel xml:lang="de">AbwAG (Abwasserabgabengesetz)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10805-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13653-4">
    <skos:related rdf:resource="descriptor/18249-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13653-4">
    <skos:related rdf:resource="descriptor/13671-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14654-4">
    <skos:prefLabel xml:lang="en">Nitrogen oxide</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14654-4">
    <skos:prefLabel xml:lang="de">Stickoxid</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16024-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10699-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19583-4">
    <skos:prefLabel xml:lang="en">Euler equation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19583-4">
    <skos:prefLabel xml:lang="de">Euler-Gleichung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18320-6">
    <skos:narrower rdf:resource="descriptor/11147-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19639-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11694-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16651-0">
    <skos:related rdf:resource="descriptor/11060-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16651-0">
    <skos:related rdf:resource="descriptor/16544-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16651-0">
    <skos:related rdf:resource="descriptor/19752-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16651-0">
    <skos:related rdf:resource="descriptor/15965-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16651-0">
    <skos:related rdf:resource="descriptor/16401-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16651-0">
    <skos:related rdf:resource="descriptor/16387-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14941-1">
    <skos:prefLabel xml:lang="en">Bread</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14941-1">
    <skos:prefLabel xml:lang="de">Brot</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10490-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15911-5">
    <skos:narrower rdf:resource="descriptor/15939-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16036-6">
    <skos:related rdf:resource="descriptor/16485-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16036-6">
    <skos:related rdf:resource="descriptor/12277-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16036-6">
    <skos:related rdf:resource="descriptor/19088-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16036-6">
    <skos:related rdf:resource="descriptor/16176-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18286-1">
    <skos:altLabel xml:lang="de">Regenerierbarer Rohstoff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18286-1">
    <skos:altLabel xml:lang="de">Regenerierbare Ressourcen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18286-1">
    <skos:altLabel xml:lang="de">Erneuerbarer Rohstoff</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19491-2">
    <skos:scopeNote xml:lang="de">Sammelbegriff für Beschäftigungsverhältnisse, die im Vergleich zu Normalarbeitsverhältnissen rechtlich wenig abgesichert sind.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19627-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70235">
    <rdfs:label xml:lang="en">Labour relations</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70235">
    <rdfs:label xml:lang="de">Arbeitsbeziehungen</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19505-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18759-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19238-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11741-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14244-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20888-0">
    <skos:altLabel xml:lang="en">Green certificate</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18731-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71116">
    <skos:broader rdf:resource="thsys/70180"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70060">
    <skos:broader rdf:resource="thsys/n"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19296-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16743-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11568-5">
    <skos:prefLabel xml:lang="en">Public Transfer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11568-5">
    <skos:prefLabel xml:lang="de">Öffentlicher Transfer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16092-3">
    <skos:broader rdf:resource="thsys/73337"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16092-3">
    <skos:broader rdf:resource="thsys/71073"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16092-3">
    <skos:broader rdf:resource="thsys/70397"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16092-3">
    <skos:broader rdf:resource="descriptor/11640-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16948-5">
    <skos:broader rdf:resource="thsys/70438"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16948-5">
    <skos:broader rdf:resource="descriptor/16950-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18373-6">
    <skos:altLabel xml:lang="de">Mineralölpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12340-2">
    <skos:broader rdf:resource="thsys/71043"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12340-2">
    <skos:broader rdf:resource="descriptor/12325-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10315-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10625-3">
    <skos:related rdf:resource="descriptor/19326-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10625-3">
    <skos:related rdf:resource="descriptor/10619-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10625-3">
    <skos:related rdf:resource="descriptor/10624-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10625-3">
    <skos:related rdf:resource="descriptor/10618-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10625-3">
    <skos:related rdf:resource="descriptor/10617-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10625-3">
    <skos:related rdf:resource="descriptor/19299-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15621-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19628-1">
    <skos:altLabel xml:lang="de">Naturschutzpark</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19628-1">
    <skos:altLabel xml:lang="de">Nationalpark</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19628-1">
    <skos:altLabel xml:lang="de">Landschaftsschutzgebiet</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18959-4">
    <skos:altLabel xml:lang="de">Müllvermeidung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11397-6">
    <skos:related rdf:resource="descriptor/11409-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11397-6">
    <skos:related rdf:resource="descriptor/16515-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11397-6">
    <skos:related rdf:resource="descriptor/11367-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11397-6">
    <skos:related rdf:resource="descriptor/11222-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11397-6">
    <skos:related rdf:resource="descriptor/11394-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11397-6">
    <skos:related rdf:resource="descriptor/18884-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11397-6">
    <skos:related rdf:resource="descriptor/11400-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11397-6">
    <skos:related rdf:resource="descriptor/11353-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70358">
    <skos:narrower rdf:resource="descriptor/14285-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70358">
    <skos:narrower rdf:resource="descriptor/14274-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70358">
    <skos:narrower rdf:resource="descriptor/14273-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70358">
    <skos:narrower rdf:resource="descriptor/14266-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70358">
    <skos:narrower rdf:resource="descriptor/14267-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70358">
    <skos:narrower rdf:resource="descriptor/14313-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70358">
    <skos:narrower rdf:resource="descriptor/14281-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70358">
    <skos:narrower rdf:resource="descriptor/14279-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70358">
    <skos:narrower rdf:resource="thsys/70346"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70358">
    <skos:narrower rdf:resource="thsys/70134"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18920-3">
    <skos:narrower rdf:resource="descriptor/13364-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18920-3">
    <skos:narrower rdf:resource="descriptor/13357-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18920-3">
    <skos:narrower rdf:resource="descriptor/13347-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18920-3">
    <skos:narrower rdf:resource="descriptor/18927-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18920-3">
    <skos:narrower rdf:resource="descriptor/18926-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18920-3">
    <skos:narrower rdf:resource="descriptor/13352-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19721-4">
    <skos:related rdf:resource="descriptor/13998-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19721-4">
    <skos:related rdf:resource="descriptor/16003-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19721-4">
    <skos:related rdf:resource="descriptor/11565-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19721-4">
    <skos:related rdf:resource="descriptor/15919-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16422-1">
    <skos:altLabel xml:lang="en">Secession</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16422-1">
    <skos:altLabel xml:lang="de">Zerfall eines Staates</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16422-1">
    <skos:altLabel xml:lang="de">Sezession</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16422-1">
    <skos:altLabel xml:lang="de">Separatistische Bewegung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16422-1">
    <skos:altLabel xml:lang="de">Auflösung eines Staates</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15469-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14135-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11111-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17948-0">
    <skos:broader rdf:resource="thsys/70128"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17948-0">
    <skos:broader rdf:resource="descriptor/18019-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15315-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16332-2">
    <skos:prefLabel xml:lang="en">Intelligence services</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16332-2">
    <skos:prefLabel xml:lang="de">Staatsschutz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70476">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18272-5">
    <skos:related rdf:resource="descriptor/18398-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18272-5">
    <skos:related rdf:resource="descriptor/18965-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18272-5">
    <skos:related rdf:resource="descriptor/12930-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18272-5">
    <skos:related rdf:resource="descriptor/14051-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18787-0">
    <skos:broader rdf:resource="thsys/70970"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18787-0">
    <skos:broader rdf:resource="thsys/70876"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18787-0">
    <skos:broader rdf:resource="descriptor/13381-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18787-0">
    <skos:broader rdf:resource="descriptor/12905-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16510-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15011-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12243-0">
    <skos:broader rdf:resource="thsys/71045"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12243-0">
    <skos:broader rdf:resource="thsys/70905"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12243-0">
    <skos:broader rdf:resource="thsys/70883"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12243-0">
    <skos:broader rdf:resource="descriptor/12254-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12243-0">
    <skos:broader rdf:resource="descriptor/18085-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20241-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19052-0">
    <skos:altLabel xml:lang="de">Herstellung von Fahrzeugaufbauten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19052-0">
    <skos:altLabel xml:lang="de">Herstellung von Anhängern</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18927-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12823-3">
    <skos:related rdf:resource="descriptor/19135-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15786-3">
    <skos:prefLabel xml:lang="en">Learning psychology</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15786-3">
    <skos:prefLabel xml:lang="de">Lernpsychologie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18120-0">
    <skos:related rdf:resource="descriptor/10973-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18120-0">
    <skos:related rdf:resource="descriptor/16611-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18120-0">
    <skos:related rdf:resource="descriptor/12910-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18120-0">
    <skos:related rdf:resource="descriptor/12038-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11840-1">
    <skos:narrower rdf:resource="descriptor/18760-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10827-5">
    <skos:altLabel xml:lang="de">Gesetz über die Besteuerung bei Auslandsbeziehungen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10827-5">
    <skos:altLabel xml:lang="de">Außensteuergesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10827-5">
    <skos:altLabel xml:lang="de">AStG</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73349">
    <skos:broader rdf:resource="thsys/70151"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18450-0">
    <skos:related rdf:resource="descriptor/13238-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18450-0">
    <skos:related rdf:resource="descriptor/13226-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18450-0">
    <skos:related rdf:resource="descriptor/18449-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18450-0">
    <skos:related rdf:resource="descriptor/18211-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18450-0">
    <skos:related rdf:resource="descriptor/13102-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18450-0">
    <skos:related rdf:resource="descriptor/13197-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11389-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70259">
    <rdfs:label xml:lang="en">Credit</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70259">
    <rdfs:label xml:lang="de">Kredit</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13205-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14161-6">
    <skos:related rdf:resource="descriptor/14593-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14161-6">
    <skos:related rdf:resource="descriptor/14931-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14161-6">
    <skos:related rdf:resource="descriptor/19614-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14161-6">
    <skos:related rdf:resource="descriptor/15887-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14161-6">
    <skos:related rdf:resource="descriptor/19383-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14161-6">
    <skos:related rdf:resource="descriptor/18343-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14161-6">
    <skos:related rdf:resource="descriptor/12964-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14161-6">
    <skos:related rdf:resource="descriptor/12969-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14161-6">
    <skos:related rdf:resource="descriptor/14054-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12788-0">
    <skos:broader rdf:resource="thsys/70040"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12788-0">
    <skos:broader rdf:resource="descriptor/19127-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12788-0">
    <skos:broader rdf:resource="descriptor/12794-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13443-1">
    <skos:broader rdf:resource="thsys/73354"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13443-1">
    <skos:broader rdf:resource="thsys/71028"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13443-1">
    <skos:broader rdf:resource="descriptor/15902-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13443-1">
    <skos:broader rdf:resource="descriptor/13446-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15708-2">
    <skos:prefLabel xml:lang="en">Population dynamics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15708-2">
    <skos:prefLabel xml:lang="de">Bevölkerungsentwicklung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13372-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16439-5">
    <skos:narrower rdf:resource="descriptor/10618-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16439-5">
    <skos:narrower rdf:resource="descriptor/16438-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11722-0">
    <skos:broader rdf:resource="thsys/73380"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11722-0">
    <skos:broader rdf:resource="thsys/70872"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11722-0">
    <skos:broader rdf:resource="thsys/70144"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17748-1">
    <skos:broader rdf:resource="thsys/70085"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17748-1">
    <skos:broader rdf:resource="descriptor/17750-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17748-1">
    <skos:broader rdf:resource="descriptor/17978-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12976-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11049-1">
    <skos:prefLabel xml:lang="en">Convergence of economic systems</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11049-1">
    <skos:prefLabel xml:lang="de">Systemkonvergenz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18032-4">
    <skos:prefLabel xml:lang="en">Change of ownership</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18032-4">
    <skos:prefLabel xml:lang="de">Betriebsübergang</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18012-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15142-5">
    <skos:prefLabel xml:lang="en">Descriptive statistics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15142-5">
    <skos:prefLabel xml:lang="de">Deskriptive Statistik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14151-2">
    <skos:narrower rdf:resource="descriptor/18257-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14151-2">
    <skos:narrower rdf:resource="descriptor/18189-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13050-6">
    <skos:altLabel xml:lang="de">Kesselbau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13050-6">
    <skos:altLabel xml:lang="de">Containerbau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13050-6">
    <skos:altLabel xml:lang="de">Boilerbau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19114-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13793-2">
    <skos:prefLabel xml:lang="en">Noncash payments</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13793-2">
    <skos:prefLabel xml:lang="de">Bargeldloser Zahlungsverkehr</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13026-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15705-1">
    <skos:prefLabel xml:lang="en">Cliometrics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15705-1">
    <skos:prefLabel xml:lang="de">Cliometrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10750-0">
    <skos:broader rdf:resource="thsys/71095"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10750-0">
    <skos:broader rdf:resource="thsys/71094"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10750-0">
    <skos:broader rdf:resource="descriptor/10721-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18983-0">
    <skos:related rdf:resource="descriptor/11276-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18983-0">
    <skos:related rdf:resource="descriptor/18980-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18983-0">
    <skos:related rdf:resource="descriptor/11802-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20817-2">
    <skos:altLabel xml:lang="en">Fair-trade movement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13381-4">
    <skos:prefLabel xml:lang="en">Engineering firm</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13381-4">
    <skos:prefLabel xml:lang="de">Ingenieurbüro</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12419-3">
    <skos:prefLabel xml:lang="en">Capitalized earning power</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12419-3">
    <skos:prefLabel xml:lang="de">Ertragswert</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18870-6">
    <skos:altLabel xml:lang="de">Schulgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19760-1">
    <skos:prefLabel xml:lang="en">Age</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19760-1">
    <skos:prefLabel xml:lang="de">Lebensalter</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19525-4">
    <skos:prefLabel xml:lang="en">Abortion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19525-4">
    <skos:prefLabel xml:lang="de">Schwangerschaftsabbruch</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10649-3">
    <skos:prefLabel xml:lang="en">Theory of tariffs</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10649-3">
    <skos:prefLabel xml:lang="de">Zolltheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13746-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17781-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17753-1">
    <skos:broader rdf:resource="thsys/70085"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17753-1">
    <skos:broader rdf:resource="descriptor/17750-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17753-1">
    <skos:broader rdf:resource="descriptor/17978-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70418">
    <rdfs:label xml:lang="de">Bergbauprodukte</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12057-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11609-3">
    <skos:altLabel xml:lang="de">Kommunalanleihe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10791-0">
    <skos:broader rdf:resource="thsys/71087"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10048-1">
    <skos:prefLabel xml:lang="en">Multi-country model</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10048-1">
    <skos:prefLabel xml:lang="de">Mehr-Länder-Modell</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14909-4">
    <skos:altLabel xml:lang="de">Kinderwäsche</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14909-4">
    <skos:altLabel xml:lang="de">Kinderoberbekleidung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14909-4">
    <skos:altLabel xml:lang="de">Babybekleidung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14883-3">
    <skos:prefLabel xml:lang="en">Cellulosic fibre</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14883-3">
    <skos:prefLabel xml:lang="de">Cellulosefaser</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16214-1">
    <skos:scopeNote xml:lang="de">Nur für längerfristig angelegte, ausführlichere Darstellungen von Zukunftsperspektiven in Wirtschaft, Politik, usw.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11262-1">
    <skos:related rdf:resource="descriptor/12562-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11262-1">
    <skos:related rdf:resource="descriptor/11185-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11262-1">
    <skos:related rdf:resource="descriptor/11180-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13485-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24787-0">
    <skos:narrower rdf:resource="descriptor/13260-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24787-0">
    <skos:narrower rdf:resource="descriptor/19064-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13684-0">
    <skos:altLabel xml:lang="de">Satellitenübertragung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13684-0">
    <skos:altLabel xml:lang="de">Satellitentechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15738-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20948-1">
    <skos:scopeNote xml:lang="de">Kunstwort, gebildet aus INTRAorganizational EntrePRENEURING. Gemeint ist damit, im Rahmen eines Unternehmens innovativ denkende und handelnde Mitarbeiter/innen wie eigenständige Unternehmer agieren zu lassen (vgl. GABLER).</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11141-6">
    <skos:prefLabel xml:lang="en">Socialist reproduction</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11141-6">
    <skos:prefLabel xml:lang="de">Sozialistische Reproduktion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10205-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19662-1">
    <skos:prefLabel xml:lang="en">Legal compliance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19662-1">
    <skos:prefLabel xml:lang="de">Normbefolgung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70245">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11067-6">
    <skos:broader rdf:resource="thsys/70235"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11067-6">
    <skos:broader rdf:resource="thsys/70039"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11067-6">
    <skos:broader rdf:resource="descriptor/16246-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11067-6">
    <skos:broader rdf:resource="descriptor/11314-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13205-3">
    <skos:altLabel xml:lang="de">Fischgroßhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13205-3">
    <skos:altLabel xml:lang="de">Fischeinzelhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71109">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.04.04</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12574-5">
    <skos:altLabel xml:lang="en">Performance appraisal</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12574-5">
    <skos:altLabel xml:lang="de">Psychische Leistungsbewertung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12574-5">
    <skos:altLabel xml:lang="de">Physische Leistungsbewertung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12574-5">
    <skos:altLabel xml:lang="de">Leistungsbewertung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12574-5">
    <skos:altLabel xml:lang="de">Leistungsbeurteilung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12574-5">
    <skos:altLabel xml:lang="de">Arbeitszeugnis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16668-4">
    <skos:altLabel xml:lang="en">Drug addiction</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16668-4">
    <skos:altLabel xml:lang="en">Drug abuse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16668-4">
    <skos:altLabel xml:lang="de">Rauschmittelkonsum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16668-4">
    <skos:altLabel xml:lang="de">Rauschgiftmissbrauch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16668-4">
    <skos:altLabel xml:lang="de">Rauschgiftmißbrauch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16668-4">
    <skos:altLabel xml:lang="de">Rauschgiftkonsum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16668-4">
    <skos:altLabel xml:lang="de">Drogensucht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16668-4">
    <skos:altLabel xml:lang="de">Drogenmissbrauch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16668-4">
    <skos:altLabel xml:lang="de">Drogenmißbrauch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16668-4">
    <skos:altLabel xml:lang="de">Drogenabhängigkeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11880-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70825">
    <skos:broader rdf:resource="thsys/w"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11286-1">
    <skos:broader rdf:resource="thsys/71058"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11286-1">
    <skos:broader rdf:resource="descriptor/11294-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11286-1">
    <skos:broader rdf:resource="descriptor/11262-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18809-2">
    <skos:altLabel xml:lang="de">Recht der Bundesländer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16849-0">
    <skos:altLabel xml:lang="de">Bremen (Region)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16849-0">
    <skos:altLabel xml:lang="de">Bremen (IHK-Bezirk)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16328-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11536-4">
    <skos:prefLabel xml:lang="en">Financial administration</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11536-4">
    <skos:prefLabel xml:lang="de">Finanzverwaltung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11519-4">
    <skos:broader rdf:resource="thsys/70355"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11519-4">
    <skos:broader rdf:resource="descriptor/11427-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18812-6">
    <skos:related rdf:resource="descriptor/18163-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18812-6">
    <skos:related rdf:resource="descriptor/19181-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13397-3">
    <skos:prefLabel xml:lang="en">Political party</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13397-3">
    <skos:prefLabel xml:lang="de">Politische Partei</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12677-2">
    <skos:related rdf:resource="descriptor/12404-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12677-2">
    <skos:related rdf:resource="descriptor/10349-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12677-2">
    <skos:related rdf:resource="descriptor/15571-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12677-2">
    <skos:related rdf:resource="descriptor/12675-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12677-2">
    <skos:related rdf:resource="descriptor/19089-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12677-2">
    <skos:related rdf:resource="descriptor/12380-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12677-2">
    <skos:related rdf:resource="descriptor/12656-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12677-2">
    <skos:related rdf:resource="descriptor/19091-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13687-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11695-5">
    <skos:altLabel xml:lang="de">Steuerabwälzung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10656-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19488-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19112-1">
    <skos:prefLabel xml:lang="en">Occupational qualification</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19112-1">
    <skos:prefLabel xml:lang="de">Qualifikation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18325-3">
    <skos:related rdf:resource="descriptor/15767-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18325-3">
    <skos:related rdf:resource="descriptor/18381-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18325-3">
    <skos:related rdf:resource="descriptor/18137-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18325-3">
    <skos:related rdf:resource="descriptor/10411-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18325-3">
    <skos:related rdf:resource="descriptor/10513-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18325-3">
    <skos:related rdf:resource="descriptor/20722-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10636-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12028-4">
    <skos:altLabel xml:lang="de">Liegenschaftsrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12028-4">
    <skos:altLabel xml:lang="de">Kataster</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12028-4">
    <skos:altLabel xml:lang="de">Immobilienrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12028-4">
    <skos:altLabel xml:lang="de">Hypothekenrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12028-4">
    <skos:altLabel xml:lang="de">Grundbuchrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12028-4">
    <skos:altLabel xml:lang="de">Grundbesitzrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12028-4">
    <skos:altLabel xml:lang="de">ErbbauVO</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12028-4">
    <skos:altLabel xml:lang="de">Erbbaurecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15493-4">
    <skos:prefLabel xml:lang="en">Duality approach (mathematical programming)</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15493-4">
    <skos:prefLabel xml:lang="de">Duales Optimierungsproblem</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16438-0">
    <skos:broader rdf:resource="thsys/73366"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16438-0">
    <skos:broader rdf:resource="thsys/70851"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16438-0">
    <skos:broader rdf:resource="descriptor/16439-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16438-0">
    <skos:broader rdf:resource="descriptor/11313-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11642-5">
    <skos:related rdf:resource="descriptor/11637-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11642-5">
    <skos:related rdf:resource="descriptor/11640-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11642-5">
    <skos:related rdf:resource="descriptor/11704-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11642-5">
    <skos:related rdf:resource="descriptor/19405-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11642-5">
    <skos:related rdf:resource="descriptor/11547-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12809-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13523-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12180-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70648">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70504">
    <skos:prefLabel xml:lang="de">V.14.01  Soziales</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11307-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19822-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12681-4">
    <skos:related rdf:resource="descriptor/12644-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12681-4">
    <skos:related rdf:resource="descriptor/19064-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12681-4">
    <skos:related rdf:resource="descriptor/12675-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12681-4">
    <skos:related rdf:resource="descriptor/12736-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12681-4">
    <skos:related rdf:resource="descriptor/12693-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12681-4">
    <skos:related rdf:resource="descriptor/19115-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12681-4">
    <skos:related rdf:resource="descriptor/12730-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11327-6">
    <skos:altLabel xml:lang="de">Lohndifferenzial</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11327-6">
    <skos:altLabel xml:lang="de">Lohndifferential</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19207-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11579-0">
    <skos:prefLabel xml:lang="en">Public Procurement</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11579-0">
    <skos:prefLabel xml:lang="de">Öffentliche Beschaffung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10263-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core">
    <rdf:type rdf:resource="http://www.w3.org/2002/07/owl#Ontology"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18344-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13709-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70465">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16355-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10072-4">
    <skos:altLabel xml:lang="en">Theory of the core</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10072-4">
    <skos:altLabel xml:lang="de">Syndikatentheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10072-4">
    <skos:altLabel xml:lang="de">Nucleolus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10072-4">
    <skos:altLabel xml:lang="de">Limittheorem eines Kerns</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10072-4">
    <skos:altLabel xml:lang="de">Kern-Lösung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10072-4">
    <skos:altLabel xml:lang="de">Kernel-Theorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10072-4">
    <skos:altLabel xml:lang="de">Kern eines Spiels</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10072-4">
    <skos:altLabel xml:lang="de">Kern einer Ökonomie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10072-4">
    <skos:altLabel xml:lang="de">Existenztheorem eines Kerns</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13346-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10618-0">
    <skos:related rdf:resource="descriptor/10966-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10618-0">
    <skos:related rdf:resource="descriptor/19327-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10618-0">
    <skos:related rdf:resource="descriptor/10619-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10618-0">
    <skos:related rdf:resource="descriptor/10625-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10618-0">
    <skos:related rdf:resource="descriptor/10617-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10618-0">
    <skos:related rdf:resource="descriptor/16438-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10618-0">
    <skos:related rdf:resource="descriptor/10558-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13412-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14300-4">
    <skos:broader rdf:resource="thsys/70134"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14300-4">
    <skos:broader rdf:resource="descriptor/14302-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18086-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20722-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12212-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14109-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10012-1">
    <skos:related rdf:resource="descriptor/11055-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10012-1">
    <skos:related rdf:resource="descriptor/10905-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10012-1">
    <skos:related rdf:resource="descriptor/10202-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10012-1">
    <skos:related rdf:resource="descriptor/19123-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10012-1">
    <skos:related rdf:resource="descriptor/10904-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10012-1">
    <skos:related rdf:resource="descriptor/12774-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10012-1">
    <skos:related rdf:resource="descriptor/10928-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10012-1">
    <skos:related rdf:resource="descriptor/19667-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10012-1">
    <skos:related rdf:resource="descriptor/10902-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19267-0">
    <skos:related rdf:resource="descriptor/10136-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19267-0">
    <skos:related rdf:resource="descriptor/13753-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19267-0">
    <skos:related rdf:resource="descriptor/19266-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19267-0">
    <skos:related rdf:resource="descriptor/10135-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19267-0">
    <skos:related rdf:resource="descriptor/18678-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19719-5">
    <skos:scopeNote xml:lang="de">Von den USA ausgehende Bestrebungen, die politische und wirtschaftliche Zuammenarbeit zwischen den amerikanischen Staaten zu intensivieren</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70017">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">N.07</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10589-2">
    <skos:broader rdf:resource="thsys/71084"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10589-2">
    <skos:broader rdf:resource="descriptor/10555-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19085-6">
    <skos:narrower rdf:resource="descriptor/17181-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19085-6">
    <skos:narrower rdf:resource="descriptor/18224-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19085-6">
    <skos:narrower rdf:resource="descriptor/18225-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19085-6">
    <skos:narrower rdf:resource="descriptor/16826-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11446-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11532-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13358-6">
    <skos:prefLabel xml:lang="en">Public library</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13358-6">
    <skos:prefLabel xml:lang="de">Öffentliche Bibliothek</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13445-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15723-6">
    <skos:broader rdf:resource="thsys/73387"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15723-6">
    <skos:broader rdf:resource="thsys/70835"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15723-6">
    <skos:broader rdf:resource="thsys/70504"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15723-6">
    <skos:broader rdf:resource="thsys/70109"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15723-6">
    <skos:broader rdf:resource="thsys/70007"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15723-6">
    <skos:broader rdf:resource="descriptor/16563-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15723-6">
    <skos:broader rdf:resource="descriptor/15690-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13735-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13639-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12406-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12754-3">
    <skos:broader rdf:resource="thsys/70029"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12754-3">
    <skos:broader rdf:resource="descriptor/12758-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15711-6">
    <skos:scopeNote xml:lang="de">vgl. auch Bürgerinitiative</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15711-6">
    <skos:scopeNote xml:lang="de">Soziale und politische Initiativen zur Beeinflussung der politischen Willensbildung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11184-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10075-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13399-6">
    <skos:narrower rdf:resource="descriptor/19737-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13399-6">
    <skos:narrower rdf:resource="descriptor/13393-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18215-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11275-6">
    <skos:related rdf:resource="descriptor/11201-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11275-6">
    <skos:related rdf:resource="descriptor/16140-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11275-6">
    <skos:related rdf:resource="descriptor/11200-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17990-1">
    <skos:altLabel xml:lang="x-other">America Latina</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17990-1">
    <skos:altLabel xml:lang="de">Lateinamerikanische Staaten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17990-1">
    <skos:altLabel xml:lang="de">Lateinamerikanische Länder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17990-1">
    <skos:altLabel xml:lang="de">Iberoamerika</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10446-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10382-3">
    <skos:related rdf:resource="descriptor/10774-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10382-3">
    <skos:related rdf:resource="descriptor/18698-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10382-3">
    <skos:related rdf:resource="descriptor/10387-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10382-3">
    <skos:related rdf:resource="descriptor/11481-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10382-3">
    <skos:related rdf:resource="descriptor/10236-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10382-3">
    <skos:related rdf:resource="descriptor/11418-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10382-3">
    <skos:related rdf:resource="descriptor/11415-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10382-3">
    <skos:related rdf:resource="descriptor/10378-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10382-3">
    <skos:related rdf:resource="descriptor/19260-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10382-3">
    <skos:related rdf:resource="descriptor/18138-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11788-5">
    <skos:related rdf:resource="descriptor/11793-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11788-5">
    <skos:related rdf:resource="descriptor/19123-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11788-5">
    <skos:related rdf:resource="descriptor/18060-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12540-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19131-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71066">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.11.04</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10121-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11507-4">
    <skos:related rdf:resource="descriptor/10397-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11507-4">
    <skos:related rdf:resource="descriptor/15093-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11507-4">
    <skos:related rdf:resource="descriptor/10255-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11507-4">
    <skos:related rdf:resource="descriptor/11506-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11507-4">
    <skos:related rdf:resource="descriptor/11501-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18771-1">
    <skos:broader rdf:resource="thsys/70922"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18771-1">
    <skos:broader rdf:resource="descriptor/18655-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11416-0">
    <skos:narrower rdf:resource="descriptor/19255-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11416-0">
    <skos:narrower rdf:resource="descriptor/19261-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11416-0">
    <skos:narrower rdf:resource="descriptor/18729-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14713-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71024">
    <skos:broader rdf:resource="thsys/71028"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12607-2">
    <skos:scopeNote xml:lang="de">Anpassung der Arbeit an den Menschen zum Abbau von Belastungen und zur Steigerung von Leistung und Arbeitszufriedenheit.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10462-5">
    <skos:scopeNote xml:lang="de">Nur für Forschung und Entwicklung der Messtechnik selbst.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10183-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13731-3">
    <skos:broader rdf:resource="thsys/73390"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13731-3">
    <skos:broader rdf:resource="thsys/70905"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13731-3">
    <skos:broader rdf:resource="thsys/70187"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13731-3">
    <skos:broader rdf:resource="descriptor/16081-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18578-2">
    <skos:broader rdf:resource="thsys/70939"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18578-2">
    <skos:broader rdf:resource="thsys/70938"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18578-2">
    <skos:broader rdf:resource="descriptor/13563-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18578-2">
    <skos:broader rdf:resource="descriptor/13632-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10631-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16274-4">
    <skos:altLabel xml:lang="de">Wissenschaftliche Politikberatung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16274-4">
    <skos:altLabel xml:lang="de">Politische Beratung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16704-5">
    <skos:altLabel xml:lang="de">Aufklärungsphilosophie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19774-4">
    <skos:broader rdf:resource="thsys/73357"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19774-4">
    <skos:broader rdf:resource="descriptor/15925-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71083">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70952">
    <skos:prefLabel xml:lang="en">W.10.04  Foreign trade industries</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70952">
    <skos:prefLabel xml:lang="de">W.10.04  Außenhandelswirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13244-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18658-4">
    <skos:prefLabel xml:lang="en">Client-server system</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18658-4">
    <skos:prefLabel xml:lang="de">Client-Server-System</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16623-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21110-4">
    <skos:related rdf:resource="descriptor/19858-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21110-4">
    <skos:related rdf:resource="descriptor/15005-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21110-4">
    <skos:related rdf:resource="descriptor/14548-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14700-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18402-4">
    <skos:narrower rdf:resource="descriptor/18406-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19632-3">
    <skos:scopeNote xml:lang="de">Sowohl für Deichbaumaßnahmen etc. zum Schutz vor Zerstörungen der Küstenregion durch das Meer als auch im Sinne von Umweltschutzmaßnahmen im Küstenbereich.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12277-4">
    <skos:related rdf:resource="descriptor/16036-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12277-4">
    <skos:related rdf:resource="descriptor/12296-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12277-4">
    <skos:related rdf:resource="descriptor/16058-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12277-4">
    <skos:related rdf:resource="descriptor/19088-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12277-4">
    <skos:related rdf:resource="descriptor/12408-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12277-4">
    <skos:related rdf:resource="descriptor/13687-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12770-5">
    <skos:narrower rdf:resource="descriptor/12844-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12770-5">
    <skos:narrower rdf:resource="descriptor/12818-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12770-5">
    <skos:narrower rdf:resource="descriptor/12808-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12770-5">
    <skos:narrower rdf:resource="descriptor/12785-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12770-5">
    <skos:narrower rdf:resource="descriptor/12794-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11682-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18405-5">
    <skos:related rdf:resource="descriptor/19390-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18405-5">
    <skos:related rdf:resource="descriptor/13029-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10891-3">
    <skos:narrower rdf:resource="descriptor/10950-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10891-3">
    <skos:narrower rdf:resource="descriptor/10905-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10895-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70814">
    <rdfs:label xml:lang="de">Private Haushalte</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12313-5">
    <skos:broader rdf:resource="thsys/73333"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12313-5">
    <skos:broader rdf:resource="thsys/70186"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12313-5">
    <skos:broader rdf:resource="descriptor/19329-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20888-0">
    <skos:broader rdf:resource="thsys/71014"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20888-0">
    <skos:broader rdf:resource="thsys/71012"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20888-0">
    <skos:broader rdf:resource="thsys/70539"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20888-0">
    <skos:broader rdf:resource="descriptor/15738-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20888-0">
    <skos:broader rdf:resource="descriptor/18362-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10975-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16794-6">
    <skos:prefLabel xml:lang="en">Interlending</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16794-6">
    <skos:prefLabel xml:lang="de">Leihverkehr</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70953">
    <rdfs:label xml:lang="en">Wholesale trade</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70953">
    <rdfs:label xml:lang="de">Großhandel</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14844-6">
    <skos:related rdf:resource="descriptor/14840-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70212">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14216-0">
    <skos:related rdf:resource="descriptor/13239-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70095">
    <skos:narrower rdf:resource="descriptor/16214-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70095">
    <skos:narrower rdf:resource="descriptor/15700-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70095">
    <skos:narrower rdf:resource="descriptor/16212-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70095">
    <skos:narrower rdf:resource="descriptor/16209-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70095">
    <skos:narrower rdf:resource="descriptor/16274-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70095">
    <skos:narrower rdf:resource="descriptor/16273-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70095">
    <skos:narrower rdf:resource="descriptor/16205-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70095">
    <skos:narrower rdf:resource="thsys/73366"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70095">
    <skos:narrower rdf:resource="thsys/73374"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70095">
    <skos:narrower rdf:resource="thsys/73382"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70095">
    <skos:narrower rdf:resource="thsys/73383"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19010-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18838-2">
    <skos:broader rdf:resource="thsys/73387"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18838-2">
    <skos:broader rdf:resource="thsys/70854"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18838-2">
    <skos:broader rdf:resource="thsys/70109"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18838-2">
    <skos:broader rdf:resource="descriptor/15706-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11564-6">
    <skos:related rdf:resource="descriptor/11569-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11564-6">
    <skos:related rdf:resource="descriptor/15925-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11564-6">
    <skos:related rdf:resource="descriptor/11660-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14766-0">
    <skos:altLabel xml:lang="de">Zierkeramik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14766-0">
    <skos:altLabel xml:lang="de">Töpferwaren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14766-0">
    <skos:altLabel xml:lang="de">Tonwaren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14766-0">
    <skos:altLabel xml:lang="de">Steingutgeschirr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14766-0">
    <skos:altLabel xml:lang="de">Porzellanprodukt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14766-0">
    <skos:altLabel xml:lang="de">Porzellangeschirr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14766-0">
    <skos:altLabel xml:lang="de">Keramikgeschirr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14766-0">
    <skos:altLabel xml:lang="de">Haushaltsporzellan</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14766-0">
    <skos:altLabel xml:lang="de">Haushaltskeramik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13426-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19420-4">
    <skos:related rdf:resource="descriptor/11653-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25742-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13010-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19190-2">
    <skos:prefLabel xml:lang="en">Maize policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19190-2">
    <skos:prefLabel xml:lang="de">Maispolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18579-0">
    <skos:related rdf:resource="descriptor/18585-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18579-0">
    <skos:related rdf:resource="descriptor/18580-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18579-0">
    <skos:related rdf:resource="descriptor/16171-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18579-0">
    <skos:related rdf:resource="descriptor/13563-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18579-0">
    <skos:related rdf:resource="descriptor/13560-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10213-5">
    <skos:related rdf:resource="descriptor/18679-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10213-5">
    <skos:related rdf:resource="descriptor/10978-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10213-5">
    <skos:related rdf:resource="descriptor/11024-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10213-5">
    <skos:related rdf:resource="descriptor/10909-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10213-5">
    <skos:related rdf:resource="descriptor/10217-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10213-5">
    <skos:related rdf:resource="descriptor/15104-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10213-5">
    <skos:related rdf:resource="descriptor/19222-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10213-5">
    <skos:related rdf:resource="descriptor/11519-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10213-5">
    <skos:related rdf:resource="descriptor/10668-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10213-5">
    <skos:related rdf:resource="descriptor/11506-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10213-5">
    <skos:related rdf:resource="descriptor/11543-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10213-5">
    <skos:related rdf:resource="descriptor/12794-5"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id319">
    <rdfs:seeAlso rdf:resource="descriptor/13558-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12920-5">
    <skos:scopeNote xml:lang="de">Nur für den gesamten Bereich der Gewinnung und Verarbeitung von Erdöl</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19703-6">
    <skos:related rdf:resource="descriptor/16016-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19703-6">
    <skos:related rdf:resource="descriptor/11127-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19703-6">
    <skos:related rdf:resource="descriptor/18068-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19703-6">
    <skos:related rdf:resource="descriptor/16283-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19859-3">
    <skos:broader rdf:resource="thsys/73392"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19859-3">
    <skos:broader rdf:resource="descriptor/15982-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71026">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.01.01.01</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11876-1">
    <skos:related rdf:resource="descriptor/13613-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11876-1">
    <skos:related rdf:resource="descriptor/11881-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11876-1">
    <skos:related rdf:resource="descriptor/11993-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11876-1">
    <skos:related rdf:resource="descriptor/18576-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11876-1">
    <skos:related rdf:resource="descriptor/18632-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11876-1">
    <skos:related rdf:resource="descriptor/11893-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11876-1">
    <skos:related rdf:resource="descriptor/13249-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11876-1">
    <skos:related rdf:resource="descriptor/18631-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11876-1">
    <skos:related rdf:resource="descriptor/11863-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11876-1">
    <skos:related rdf:resource="descriptor/11864-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16142-0">
    <skos:broader rdf:resource="thsys/73336"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16142-0">
    <skos:broader rdf:resource="descriptor/16124-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14182-5">
    <skos:prefLabel xml:lang="en">Nuclear energy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14182-5">
    <skos:prefLabel xml:lang="de">Kernenergie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12170-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12796-1">
    <skos:prefLabel xml:lang="en">Transfer pricing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12796-1">
    <skos:prefLabel xml:lang="de">Transferpreis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19241-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#member">
    <rdfs:label xml:lang="en">member</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16853-2">
    <skos:altLabel xml:lang="de">Westliche Bundesländer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16853-2">
    <skos:altLabel xml:lang="de">Früheres Bundesgebiet</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16853-2">
    <skos:altLabel xml:lang="de">Deutschland (Alte Bundesländer)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16472-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id105">
    <rdf:value xml:lang="de">Für Freizeitverhalten</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11064-5">
    <skos:prefLabel xml:lang="en">Barter economy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11064-5">
    <skos:prefLabel xml:lang="de">Tauschwirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20077-2">
    <skos:scopeNote xml:lang="de">Gesellschafts- und Sozialverträge in der Philosophie und der Neuen politischen Ökonomie.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20301-2">
    <skos:broader rdf:resource="thsys/71044"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20301-2">
    <skos:broader rdf:resource="descriptor/12209-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12406-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10836-4">
    <skos:related rdf:resource="descriptor/10813-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10836-4">
    <skos:related rdf:resource="descriptor/19318-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10836-4">
    <skos:related rdf:resource="descriptor/10866-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10836-4">
    <skos:related rdf:resource="descriptor/10865-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10836-4">
    <skos:related rdf:resource="descriptor/10415-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10836-4">
    <skos:related rdf:resource="descriptor/18538-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10836-4">
    <skos:related rdf:resource="descriptor/10864-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10836-4">
    <skos:related rdf:resource="descriptor/10693-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10836-4">
    <skos:related rdf:resource="descriptor/10566-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10836-4">
    <skos:related rdf:resource="descriptor/10565-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10836-4">
    <skos:related rdf:resource="descriptor/10692-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10836-4">
    <skos:related rdf:resource="descriptor/11954-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10836-4">
    <skos:related rdf:resource="descriptor/10602-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10305-0">
    <skos:prefLabel xml:lang="en">CES production function</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10305-0">
    <skos:prefLabel xml:lang="de">CES-Produktionsfunktion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19099-2">
    <skos:broader rdf:resource="thsys/70578"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19099-2">
    <skos:broader rdf:resource="descriptor/12497-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15952-5">
    <skos:altLabel xml:lang="de">Zuwanderung aus dem Ausland</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19835-3">
    <skos:related rdf:resource="descriptor/19102-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19835-3">
    <skos:related rdf:resource="descriptor/16067-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19835-3">
    <skos:related rdf:resource="descriptor/12287-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19835-3">
    <skos:related rdf:resource="descriptor/12075-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19835-3">
    <skos:related rdf:resource="descriptor/13714-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10443-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11350-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18731-6">
    <skos:altLabel xml:lang="en">Electronic exchange</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18731-6">
    <skos:altLabel xml:lang="en">Electronic broking system</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18731-6">
    <skos:altLabel xml:lang="de">Integriertes Börsenhandels- und Informationssystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18731-6">
    <skos:altLabel xml:lang="de">IBIS (Integriertes Börsenhandels- und Informationssystem)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18731-6">
    <skos:altLabel xml:lang="de">Elektronisches Effektenhandelssystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18731-6">
    <skos:altLabel xml:lang="de">Elektronisches Börsen-Informationssystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18731-6">
    <skos:altLabel xml:lang="de">Computergestützter Börsenhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18731-6">
    <skos:altLabel xml:lang="de">Computerbörse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10823-6">
    <skos:scopeNote xml:lang="de">Für Direktinvestitionen sowie zusammenfassende Literatur über alle Auslandsinvestitionen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13957-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11198-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18578-2">
    <skos:prefLabel xml:lang="en">Passenger road traffic</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18578-2">
    <skos:prefLabel xml:lang="de">Straßenpersonenverkehr</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16328-0">
    <skos:prefLabel xml:lang="en">Police</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16328-0">
    <skos:prefLabel xml:lang="de">Polizei</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19544-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/72185">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.16.04</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11563-1">
    <skos:prefLabel xml:lang="en">Contractual savings for housing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11563-1">
    <skos:prefLabel xml:lang="de">Bausparen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15077-4">
    <skos:prefLabel xml:lang="en">Retail price index</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15077-4">
    <skos:prefLabel xml:lang="de">Einzelhandelspreisindex</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18088-5">
    <skos:broader rdf:resource="thsys/71108"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18088-5">
    <skos:broader rdf:resource="thsys/70901"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18088-5">
    <skos:broader rdf:resource="descriptor/12281-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12474-2">
    <skos:prefLabel xml:lang="en">Costing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12474-2">
    <skos:prefLabel xml:lang="de">Kostenbewertung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11459-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14840-0">
    <skos:related rdf:resource="descriptor/14846-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14840-0">
    <skos:related rdf:resource="descriptor/14844-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14840-0">
    <skos:related rdf:resource="descriptor/14548-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14840-0">
    <skos:related rdf:resource="descriptor/14900-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19289-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14266-6">
    <skos:narrower rdf:resource="descriptor/14274-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14266-6">
    <skos:narrower rdf:resource="descriptor/14273-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14266-6">
    <skos:narrower rdf:resource="descriptor/14302-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18968-3">
    <skos:altLabel xml:lang="en">Single parents</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18968-3">
    <skos:altLabel xml:lang="en">Single mothers</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18968-3">
    <skos:altLabel xml:lang="en">One-parent family</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18968-3">
    <skos:altLabel xml:lang="en">Lone-parent family</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18968-3">
    <skos:altLabel xml:lang="en">Lone parents</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18968-3">
    <skos:altLabel xml:lang="en">Lone mothers</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18968-3">
    <skos:altLabel xml:lang="de">Unvollständige Familie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18968-3">
    <skos:altLabel xml:lang="de">Einelternfamilie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18968-3">
    <skos:altLabel xml:lang="de">Alleinerziehende Väter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18968-3">
    <skos:altLabel xml:lang="de">Alleinerziehende Mütter</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id183">
    <rdfs:seeAlso rdf:resource="descriptor/10298-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70026">
    <skos:narrower rdf:resource="descriptor/17758-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70026">
    <skos:narrower rdf:resource="thsys/71030"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70026">
    <skos:narrower rdf:resource="thsys/70080"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70026">
    <skos:narrower rdf:resource="thsys/70086"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19788-0">
    <skos:broader rdf:resource="thsys/73347"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19375-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15920-4">
    <skos:narrower rdf:resource="descriptor/11284-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15920-4">
    <skos:narrower rdf:resource="descriptor/18970-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12947-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18064-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11616-6">
    <skos:narrower rdf:resource="descriptor/11124-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11616-6">
    <skos:narrower rdf:resource="descriptor/11612-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11616-6">
    <skos:narrower rdf:resource="descriptor/19398-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18586-3">
    <skos:prefLabel xml:lang="en">Container shipping company</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18586-3">
    <skos:prefLabel xml:lang="de">Container-Reederei</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12883-6">
    <skos:prefLabel xml:lang="en">Computer-assisted audit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12883-6">
    <skos:prefLabel xml:lang="de">Computergestützte Revision</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13464-0">
    <skos:altLabel xml:lang="en">Business law</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13464-0">
    <skos:altLabel xml:lang="de">HGB (Handelsgesetzbuch)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13464-0">
    <skos:altLabel xml:lang="de">Handelsgesetzbuch</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13832-4">
    <skos:altLabel xml:lang="de">Freizeitwohnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13832-4">
    <skos:altLabel xml:lang="de">Ferienwohnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id315">
    <rdfs:seeAlso rdf:resource="descriptor/11153-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71033">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10721-0">
    <skos:narrower rdf:resource="descriptor/10722-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10721-0">
    <skos:narrower rdf:resource="descriptor/10750-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10721-0">
    <skos:narrower rdf:resource="descriptor/10745-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10721-0">
    <skos:narrower rdf:resource="descriptor/10719-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13222-3">
    <skos:related rdf:resource="descriptor/12921-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13222-3">
    <skos:related rdf:resource="descriptor/14043-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70939">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15604-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18369-4">
    <skos:prefLabel xml:lang="en">International leasing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18369-4">
    <skos:prefLabel xml:lang="de">Internationales Leasing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14345-3">
    <skos:broader rdf:resource="thsys/70549"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14345-3">
    <skos:broader rdf:resource="descriptor/14336-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16955-1">
    <skos:prefLabel xml:lang="en">Vienna</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16955-1">
    <skos:prefLabel xml:lang="de">Wien</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16548-0">
    <skos:prefLabel xml:lang="en">Minimum pension</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16548-0">
    <skos:prefLabel xml:lang="de">Mindestrente</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13678-2">
    <skos:broader rdf:resource="thsys/70922"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13678-2">
    <skos:broader rdf:resource="descriptor/18000-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14073-3">
    <skos:related rdf:resource="descriptor/19629-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14073-3">
    <skos:related rdf:resource="descriptor/14074-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14073-3">
    <skos:related rdf:resource="descriptor/12949-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14073-3">
    <skos:related rdf:resource="descriptor/14931-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14073-3">
    <skos:related rdf:resource="descriptor/14030-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13881-5">
    <skos:broader rdf:resource="thsys/71057"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13881-5">
    <skos:broader rdf:resource="thsys/71009"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13881-5">
    <skos:broader rdf:resource="descriptor/18400-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18286-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14662-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18558-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12392-4">
    <skos:prefLabel xml:lang="en">Pending transactions</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12392-4">
    <skos:prefLabel xml:lang="de">Schwebende Geschäfte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13993-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18064-5">
    <skos:broader rdf:resource="thsys/73364"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18064-5">
    <skos:broader rdf:resource="thsys/73332"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18064-5">
    <skos:broader rdf:resource="thsys/71097"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18064-5">
    <skos:broader rdf:resource="descriptor/16188-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11829-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18000-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11194-6">
    <skos:broader rdf:resource="thsys/71116"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11194-6">
    <skos:broader rdf:resource="thsys/71059"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11194-6">
    <skos:broader rdf:resource="thsys/71050"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11194-6">
    <skos:broader rdf:resource="thsys/70924"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11194-6">
    <skos:broader rdf:resource="thsys/70829"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11194-6">
    <skos:broader rdf:resource="descriptor/19784-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12540-1">
    <skos:altLabel xml:lang="de">Selbstkostenrechnung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12540-1">
    <skos:altLabel xml:lang="de">Selbstkostenpreis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18102-2">
    <skos:related rdf:resource="descriptor/18023-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18486-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#editorialNote">
    <rdf:type rdf:resource="http://www.w3.org/1999/02/22-rdf-syntax-ns#Property"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#editorialNote">
    <rdf:type rdf:resource="http://www.w3.org/2002/07/owl#AnnotationProperty"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20967-4">
    <skos:scopeNote xml:lang="de">Instrument der internationalen Klimapolitik: Gemeinschaftliche Umsetzung von Vorgaben internationaler Klimakonferenzen durch Industrie- und Entwicklungsländer. Industrieländer können sich Maßnahmen im Ausland auf ihre nationale Emissionsreduktion anrechnen lassen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15381-1">
    <skos:prefLabel xml:lang="en">Model specification</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15381-1">
    <skos:prefLabel xml:lang="de">Modell-Spezifikation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10711-3">
    <skos:broader rdf:resource="thsys/71096"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18067-6">
    <skos:prefLabel xml:lang="en">Liability</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18067-6">
    <skos:prefLabel xml:lang="de">Haftung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13354-0">
    <skos:prefLabel xml:lang="en">Gambling</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13354-0">
    <skos:prefLabel xml:lang="de">Glücksspiel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11573-5">
    <skos:broader rdf:resource="thsys/73360"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11573-5">
    <skos:broader rdf:resource="thsys/71097"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11573-5">
    <skos:broader rdf:resource="thsys/71080"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11573-5">
    <skos:broader rdf:resource="descriptor/11568-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11573-5">
    <skos:broader rdf:resource="descriptor/19328-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70318">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/21072-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10978-5">
    <skos:prefLabel xml:lang="en">Price policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10978-5">
    <skos:prefLabel xml:lang="de">Staatliche Preispolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18623-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13690-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11890-0">
    <skos:altLabel xml:lang="de">Dorfsanierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15883-5">
    <skos:related rdf:resource="descriptor/15887-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15883-5">
    <skos:related rdf:resource="descriptor/19632-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15883-5">
    <skos:related rdf:resource="descriptor/13543-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15883-5">
    <skos:related rdf:resource="descriptor/19616-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15883-5">
    <skos:related rdf:resource="descriptor/12967-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15883-5">
    <skos:related rdf:resource="descriptor/11938-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15883-5">
    <skos:related rdf:resource="descriptor/18984-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11371-3">
    <skos:prefLabel xml:lang="en">Distance learning</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11371-3">
    <skos:prefLabel xml:lang="de">Fernunterricht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18481-3">
    <skos:narrower rdf:resource="descriptor/10570-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18481-3">
    <skos:narrower rdf:resource="descriptor/10565-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18481-3">
    <skos:narrower rdf:resource="descriptor/10564-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14735-4">
    <skos:altLabel xml:lang="de">Wimperntusche</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14735-4">
    <skos:altLabel xml:lang="de">Schönheitspflegemittel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14735-4">
    <skos:altLabel xml:lang="de">Naturkosmetika</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14735-4">
    <skos:altLabel xml:lang="de">Nagellack</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14735-4">
    <skos:altLabel xml:lang="de">Lippenstift</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14735-4">
    <skos:altLabel xml:lang="de">Lidschatten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14798-1">
    <skos:prefLabel xml:lang="en">Household paper product</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14798-1">
    <skos:prefLabel xml:lang="de">Haushaltspapierprodukt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15935-5">
    <skos:prefLabel xml:lang="en">Gypsies</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15935-5">
    <skos:prefLabel xml:lang="de">Roma (Personen)</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10506-4">
    <skos:altLabel xml:lang="en">LLDCs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10506-4">
    <skos:altLabel xml:lang="de">Am wenigsten entwickelte Länder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10494-6">
    <skos:broader rdf:resource="thsys/70091"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10494-6">
    <skos:broader rdf:resource="descriptor/15073-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71033">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12841-1">
    <skos:prefLabel xml:lang="en">Means of advertising</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12841-1">
    <skos:prefLabel xml:lang="de">Werbemittel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14538-6">
    <skos:altLabel xml:lang="de">Photoapparat</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19018-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14456-1">
    <skos:broader rdf:resource="thsys/70415"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14456-1">
    <skos:broader rdf:resource="descriptor/14455-3"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id155">
    <rdfs:seeAlso rdf:resource="descriptor/10698-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71059">
    <skos:broader rdf:resource="thsys/70052"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13165-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15806-2">
    <skos:altLabel xml:lang="de">Umgangsformen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15806-2">
    <skos:altLabel xml:lang="de">Konformität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15806-2">
    <skos:altLabel xml:lang="de">Konformismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15806-2">
    <skos:altLabel xml:lang="de">Knigge</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15806-2">
    <skos:altLabel xml:lang="de">Gesellschaftliche Norm</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15806-2">
    <skos:altLabel xml:lang="de">Gesellschaftliche Etiquette</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15806-2">
    <skos:altLabel xml:lang="de">Gesellschaftliche Etikette</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15806-2">
    <skos:altLabel xml:lang="de">Etikette (Umgangsformen)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10692-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18434-5">
    <skos:broader rdf:resource="thsys/70990"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18434-5">
    <skos:broader rdf:resource="descriptor/15677-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17954-5">
    <skos:broader rdf:resource="thsys/70198"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17954-5">
    <skos:broader rdf:resource="descriptor/15887-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18325-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17465-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13320-3">
    <skos:altLabel xml:lang="de">Türkisches Bad</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13320-3">
    <skos:altLabel xml:lang="de">Sonnen-Studio</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13320-3">
    <skos:altLabel xml:lang="de">Solarium</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13320-3">
    <skos:altLabel xml:lang="de">Sauna</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13320-3">
    <skos:altLabel xml:lang="de">Massagesalon</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13320-3">
    <skos:altLabel xml:lang="de">Fitneßzentrum</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13320-3">
    <skos:altLabel xml:lang="de">Fitness-Center</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13320-3">
    <skos:altLabel xml:lang="de">Fitneß-Center</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14004-1">
    <skos:broader rdf:resource="thsys/73296"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14004-1">
    <skos:broader rdf:resource="thsys/71055"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14004-1">
    <skos:broader rdf:resource="thsys/70825"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14004-1">
    <skos:broader rdf:resource="descriptor/18535-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19653-2">
    <skos:broader rdf:resource="thsys/73341"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19653-2">
    <skos:broader rdf:resource="thsys/71036"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19653-2">
    <skos:broader rdf:resource="thsys/70922"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19653-2">
    <skos:broader rdf:resource="descriptor/18175-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70880">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18384-1">
    <skos:prefLabel xml:lang="en">Mining research</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18384-1">
    <skos:prefLabel xml:lang="de">Bergbauforschung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13096-3">
    <skos:prefLabel xml:lang="en">Toy industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13096-3">
    <skos:prefLabel xml:lang="de">Spielwarenindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14483-5">
    <skos:altLabel xml:lang="de">Telefonkabel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14483-5">
    <skos:altLabel xml:lang="de">Isolierter Draht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14483-5">
    <skos:altLabel xml:lang="de">Elektroleitung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14483-5">
    <skos:altLabel xml:lang="de">Elektrokabel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14483-5">
    <skos:altLabel xml:lang="de">Elektrodraht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18071-1">
    <skos:broader rdf:resource="thsys/70815"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18071-1">
    <skos:broader rdf:resource="thsys/70239"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10572-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12567-2">
    <skos:narrower rdf:resource="descriptor/11341-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12567-2">
    <skos:narrower rdf:resource="descriptor/12565-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12567-2">
    <skos:narrower rdf:resource="descriptor/19007-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12567-2">
    <skos:narrower rdf:resource="descriptor/12564-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12567-2">
    <skos:narrower rdf:resource="descriptor/12585-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12567-2">
    <skos:narrower rdf:resource="descriptor/12589-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12567-2">
    <skos:narrower rdf:resource="descriptor/19001-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12567-2">
    <skos:narrower rdf:resource="descriptor/12573-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12567-2">
    <skos:narrower rdf:resource="descriptor/21306-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12567-2">
    <skos:narrower rdf:resource="descriptor/12592-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11506-6">
    <skos:related rdf:resource="descriptor/20104-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11506-6">
    <skos:related rdf:resource="descriptor/10213-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11506-6">
    <skos:related rdf:resource="descriptor/10242-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11506-6">
    <skos:related rdf:resource="descriptor/11507-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11506-6">
    <skos:related rdf:resource="descriptor/12425-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11506-6">
    <skos:related rdf:resource="descriptor/11494-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11506-6">
    <skos:related rdf:resource="descriptor/11427-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11506-6">
    <skos:related rdf:resource="descriptor/11496-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10124-4">
    <skos:broader rdf:resource="thsys/70486"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10124-4">
    <skos:broader rdf:resource="thsys/70187"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10124-4">
    <skos:broader rdf:resource="thsys/70058"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10124-4">
    <skos:broader rdf:resource="descriptor/10314-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19587-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14025-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16979-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12829-5">
    <skos:related rdf:resource="descriptor/15859-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12829-5">
    <skos:related rdf:resource="descriptor/12846-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12829-5">
    <skos:related rdf:resource="descriptor/12837-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12829-5">
    <skos:related rdf:resource="descriptor/12844-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12829-5">
    <skos:related rdf:resource="descriptor/18932-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12829-5">
    <skos:related rdf:resource="descriptor/18928-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12829-5">
    <skos:related rdf:resource="descriptor/18930-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12829-5">
    <skos:related rdf:resource="descriptor/13658-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12619-2">
    <skos:altLabel xml:lang="en">Learning curve</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12619-2">
    <skos:altLabel xml:lang="de">Sozialer Lernprozess</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12619-2">
    <skos:altLabel xml:lang="de">Sozialer Lernprozeß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12619-2">
    <skos:altLabel xml:lang="de">Lernprozeß</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12619-2">
    <skos:altLabel xml:lang="de">Lernkurve</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12619-2">
    <skos:altLabel xml:lang="de">Lerngesetz der Produktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12619-2">
    <skos:altLabel xml:lang="de">Erfahrungskurventheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19805-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19696-5">
    <skos:narrower rdf:resource="descriptor/16369-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11237-0">
    <skos:prefLabel xml:lang="en">Labour administration</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11237-0">
    <skos:prefLabel xml:lang="de">Arbeitsverwaltung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18271-0">
    <skos:altLabel xml:lang="de">Zugriffskarte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18271-0">
    <skos:altLabel xml:lang="de">Zugangskarte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18271-0">
    <skos:altLabel xml:lang="de">Zahlungskarte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18271-0">
    <skos:altLabel xml:lang="de">SmartCard</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18271-0">
    <skos:altLabel xml:lang="de">Geldkarte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18271-0">
    <skos:altLabel xml:lang="de">Elektronische Geldbörse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16185-3">
    <skos:related rdf:resource="descriptor/10621-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10478-4">
    <skos:altLabel xml:lang="de">Patentwesen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10478-4">
    <skos:altLabel xml:lang="de">Patentsystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10478-4">
    <skos:altLabel xml:lang="de">Patentpool</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10478-4">
    <skos:altLabel xml:lang="de">Patentdauer</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10478-4">
    <skos:altLabel xml:lang="de">Lizenz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70039">
    <skos:narrower rdf:resource="descriptor/11069-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70039">
    <skos:narrower rdf:resource="descriptor/16081-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70039">
    <skos:narrower rdf:resource="descriptor/11068-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70039">
    <skos:narrower rdf:resource="descriptor/11067-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70039">
    <skos:narrower rdf:resource="descriptor/19285-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70039">
    <skos:narrower rdf:resource="descriptor/19284-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70039">
    <skos:narrower rdf:resource="descriptor/11105-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70039">
    <skos:narrower rdf:resource="descriptor/11063-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70039">
    <skos:narrower rdf:resource="descriptor/11127-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70039">
    <skos:narrower rdf:resource="descriptor/11049-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70039">
    <skos:narrower rdf:resource="descriptor/19686-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70039">
    <skos:narrower rdf:resource="descriptor/12064-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70039">
    <skos:narrower rdf:resource="descriptor/11066-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70039">
    <skos:narrower rdf:resource="descriptor/10976-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70039">
    <skos:narrower rdf:resource="descriptor/19668-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70039">
    <skos:narrower rdf:resource="descriptor/24705-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70039">
    <skos:narrower rdf:resource="descriptor/19283-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70039">
    <skos:narrower rdf:resource="descriptor/19211-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70039">
    <skos:narrower rdf:resource="descriptor/12009-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70039">
    <skos:narrower rdf:resource="descriptor/11044-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70039">
    <skos:narrower rdf:resource="descriptor/11040-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70039">
    <skos:narrower rdf:resource="descriptor/10005-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70039">
    <skos:narrower rdf:resource="thsys/70171"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70039">
    <skos:narrower rdf:resource="thsys/70038"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14272-4">
    <skos:altLabel xml:lang="de">Oberflächenveredeltes Metallprodukt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14272-4">
    <skos:altLabel xml:lang="de">Metallbeschichtetes Produkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73382">
    <skos:narrower rdf:resource="descriptor/16312-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73382">
    <skos:narrower rdf:resource="descriptor/10469-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73382">
    <skos:narrower rdf:resource="descriptor/11063-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73382">
    <skos:narrower rdf:resource="descriptor/11127-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73382">
    <skos:narrower rdf:resource="descriptor/16305-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73382">
    <skos:narrower rdf:resource="descriptor/11060-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73382">
    <skos:narrower rdf:resource="descriptor/16292-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73382">
    <skos:narrower rdf:resource="descriptor/19683-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73382">
    <skos:narrower rdf:resource="descriptor/16379-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73382">
    <skos:narrower rdf:resource="descriptor/19682-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73382">
    <skos:narrower rdf:resource="descriptor/16298-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73382">
    <skos:narrower rdf:resource="descriptor/11164-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73382">
    <skos:narrower rdf:resource="descriptor/16384-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73382">
    <skos:narrower rdf:resource="descriptor/11047-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73382">
    <skos:narrower rdf:resource="descriptor/11046-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73382">
    <skos:narrower rdf:resource="descriptor/19681-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73382">
    <skos:narrower rdf:resource="descriptor/16314-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73382">
    <skos:narrower rdf:resource="descriptor/16658-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73382">
    <skos:narrower rdf:resource="descriptor/16247-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73382">
    <skos:narrower rdf:resource="descriptor/16246-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73382">
    <skos:narrower rdf:resource="descriptor/16245-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73382">
    <skos:narrower rdf:resource="thsys/73381"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73382">
    <skos:narrower rdf:resource="thsys/73380"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73382">
    <skos:narrower rdf:resource="thsys/73376"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73382">
    <skos:narrower rdf:resource="thsys/73375"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73382">
    <skos:narrower rdf:resource="thsys/73377"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73382">
    <skos:narrower rdf:resource="thsys/73379"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73382">
    <skos:narrower rdf:resource="thsys/73378"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18142-4">
    <skos:broader rdf:resource="thsys/73370"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18142-4">
    <skos:broader rdf:resource="thsys/70235"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18142-4">
    <skos:broader rdf:resource="descriptor/16545-6"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id90">
    <rdf:value xml:lang="de">Für die Entwicklungspolitik der Geberländer</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11812-6">
    <skos:altLabel xml:lang="de">Energieversorgungssicherheit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11812-6">
    <skos:altLabel xml:lang="de">Energiesicherung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11812-6">
    <skos:altLabel xml:lang="de">Energieproblem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11812-6">
    <skos:altLabel xml:lang="de">Energielücke</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11812-6">
    <skos:altLabel xml:lang="de">Energiekrise</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16695-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18247-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10573-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13587-5">
    <skos:scopeNote xml:lang="de">Aufteilung des Verkehrs auf die verschiedenen Verkehrsträger und Verkehrsmittel.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14170-5">
    <skos:altLabel xml:lang="de">Haarwild</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14170-5">
    <skos:altLabel xml:lang="de">Federwild</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15983-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16439-5">
    <skos:broader rdf:resource="thsys/73366"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16439-5">
    <skos:broader rdf:resource="descriptor/12091-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13303-3">
    <skos:broader rdf:resource="thsys/70872"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13303-3">
    <skos:broader rdf:resource="descriptor/11722-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13303-3">
    <skos:broader rdf:resource="descriptor/13301-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18659-2">
    <skos:broader rdf:resource="thsys/73341"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18659-2">
    <skos:broader rdf:resource="thsys/70922"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18659-2">
    <skos:broader rdf:resource="descriptor/18655-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18659-2">
    <skos:broader rdf:resource="descriptor/15599-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18937-0">
    <skos:prefLabel xml:lang="en">Local radio station</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18937-0">
    <skos:prefLabel xml:lang="de">Lokaler Rundfunk</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10462-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18127-0">
    <skos:broader rdf:resource="thsys/71095"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18127-0">
    <skos:broader rdf:resource="thsys/71094"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16361-2">
    <skos:related rdf:resource="descriptor/11047-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11723-5">
    <skos:broader rdf:resource="thsys/71083"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11723-5">
    <skos:broader rdf:resource="thsys/71063"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id59">
    <rdfs:seeAlso rdf:resource="descriptor/12236-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71115">
    <rdfs:label xml:lang="de">Einkommen und Vermögen</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10687-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14046-6">
    <skos:altLabel xml:lang="de">Photographie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14046-6">
    <skos:altLabel xml:lang="de">Fotographie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14046-6">
    <skos:altLabel xml:lang="de">Foto</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14953-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11729-0">
    <skos:narrower rdf:resource="descriptor/11706-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18434-5">
    <skos:related rdf:resource="descriptor/13367-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18434-5">
    <skos:related rdf:resource="descriptor/14371-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18434-5">
    <skos:related rdf:resource="descriptor/13102-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73332">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12919-4">
    <skos:scopeNote xml:lang="de">Für die Erzeugung bzw. Verarbeitung von Kunststoffen allein BENUTZE die entsprechenden Deskriptoren.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12919-4">
    <skos:scopeNote xml:lang="de">Nur für Gesamtbetrachtungen des kunststofferzeugenden und -verarbeitenden Gewerbes.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13817-0">
    <skos:broader rdf:resource="thsys/70891"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13817-0">
    <skos:broader rdf:resource="descriptor/13808-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13817-0">
    <skos:broader rdf:resource="descriptor/13297-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18894-6">
    <skos:broader rdf:resource="thsys/73347"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18894-6">
    <skos:broader rdf:resource="thsys/71085"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18894-6">
    <skos:broader rdf:resource="thsys/70839"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18894-6">
    <skos:broader rdf:resource="thsys/70573"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18894-6">
    <skos:broader rdf:resource="descriptor/18891-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19330-5">
    <skos:broader rdf:resource="thsys/71096"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19330-5">
    <skos:broader rdf:resource="thsys/71020"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19330-5">
    <skos:broader rdf:resource="descriptor/10710-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19330-5">
    <skos:broader rdf:resource="descriptor/18330-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16808-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13810-0">
    <skos:broader rdf:resource="thsys/70891"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13810-0">
    <skos:broader rdf:resource="descriptor/13297-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10758-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16552-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16954-3">
    <skos:broader rdf:resource="thsys/70438"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16954-3">
    <skos:broader rdf:resource="descriptor/16950-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18315-6">
    <skos:altLabel xml:lang="en">Rain-fed farming</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18315-6">
    <skos:altLabel xml:lang="en">Rainfed agriculture</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18315-6">
    <skos:altLabel xml:lang="de">Subtropische Landwirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18315-6">
    <skos:altLabel xml:lang="de">Regenfeldbau</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12031-1">
    <skos:narrower rdf:resource="descriptor/13845-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12031-1">
    <skos:narrower rdf:resource="descriptor/19456-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12031-1">
    <skos:narrower rdf:resource="descriptor/19454-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18454-6">
    <skos:prefLabel xml:lang="en">Office equipment industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18454-6">
    <skos:prefLabel xml:lang="de">Bürowirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10851-1">
    <skos:prefLabel xml:lang="en">Export market research</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10851-1">
    <skos:prefLabel xml:lang="de">Auslandsmarktforschung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16588-2">
    <skos:scopeNote xml:lang="de">Für Gesellschaftsformen von Unternehmen BENUTZE die entsprechenden Deskriptoren.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16588-2">
    <skos:scopeNote xml:lang="de">Nur im soziologischen Sinne.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10572-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18244-3">
    <skos:related rdf:resource="descriptor/13130-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18244-3">
    <skos:related rdf:resource="descriptor/14885-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18817-3">
    <skos:related rdf:resource="descriptor/11560-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18817-3">
    <skos:related rdf:resource="descriptor/16463-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18817-3">
    <skos:related rdf:resource="descriptor/18182-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18817-3">
    <skos:related rdf:resource="descriptor/10975-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18817-3">
    <skos:related rdf:resource="descriptor/18815-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18817-3">
    <skos:related rdf:resource="descriptor/11051-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14003-3">
    <skos:prefLabel xml:lang="en">Translation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14003-3">
    <skos:prefLabel xml:lang="de">Übersetzung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19734-2">
    <skos:broader rdf:resource="thsys/73394"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19734-2">
    <skos:broader rdf:resource="descriptor/16719-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16048-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10808-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19067-1">
    <skos:broader rdf:resource="thsys/70876"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12281-6">
    <skos:prefLabel xml:lang="en">Investment fund</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12281-6">
    <skos:prefLabel xml:lang="de">Investmentfonds</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13815-4">
    <skos:prefLabel xml:lang="en">Disaster insurance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13815-4">
    <skos:prefLabel xml:lang="de">Elementarschadenversicherung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19383-5">
    <skos:prefLabel xml:lang="en">Fishing industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19383-5">
    <skos:prefLabel xml:lang="de">Fischwirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71107">
    <skos:prefLabel xml:lang="de">V.05.06.02  Wertpapiermärkte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12090-6">
    <skos:scopeNote xml:lang="de">Gesamtheit der konstitutiven und langfristig angelegten Regelungen für ein Unternehmen. Interne formale Machtverteilung zwischen Interessengruppen, externe Regelungen zum Schutz verfassungsrelevanter Interessen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17778-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19102-4">
    <skos:altLabel xml:lang="de">Wertorientierung (Shareholder Value)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19102-4">
    <skos:altLabel xml:lang="de">Wertorientiertes Management</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19102-4">
    <skos:altLabel xml:lang="de">Wertorientierte Unternehmensführung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18913-0">
    <skos:prefLabel xml:lang="en">Social security finance</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18913-0">
    <skos:prefLabel xml:lang="de">Sozialversicherungsfinanzierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18914-5">
    <skos:altLabel xml:lang="de">Rentenversicherungsrecht</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19791-4">
    <skos:broader rdf:resource="thsys/73346"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19791-4">
    <skos:broader rdf:resource="descriptor/16670-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10337-1">
    <skos:narrower rdf:resource="descriptor/18457-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10337-1">
    <skos:narrower rdf:resource="descriptor/18506-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18870-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19814-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14816-4">
    <skos:broader rdf:resource="thsys/70265"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13903-0">
    <skos:related rdf:resource="descriptor/18277-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13903-0">
    <skos:related rdf:resource="descriptor/13109-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13903-0">
    <skos:related rdf:resource="descriptor/13181-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24622-4">
    <skos:related rdf:resource="descriptor/19215-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24622-4">
    <skos:related rdf:resource="descriptor/12594-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24622-4">
    <skos:related rdf:resource="descriptor/15839-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24622-4">
    <skos:related rdf:resource="descriptor/11052-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24622-4">
    <skos:related rdf:resource="descriptor/11153-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18059-5">
    <skos:related rdf:resource="descriptor/18212-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18059-5">
    <skos:related rdf:resource="descriptor/18881-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19775-2">
    <skos:broader rdf:resource="thsys/70580"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19079-1">
    <skos:prefLabel xml:lang="en">Shareholder loan</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19079-1">
    <skos:prefLabel xml:lang="de">Gesellschafterdarlehen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10499-3">
    <skos:narrower rdf:resource="descriptor/17993-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10499-3">
    <skos:narrower rdf:resource="descriptor/19178-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10499-3">
    <skos:narrower rdf:resource="descriptor/19177-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13965-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19545-5">
    <zbwext:useInsteadNote rdf:nodeID="id217"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17198-3">
    <skos:prefLabel xml:lang="en">Luxembourg</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17198-3">
    <skos:prefLabel xml:lang="de">Luxemburg</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18581-6">
    <skos:prefLabel xml:lang="en">Road accident</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18581-6">
    <skos:prefLabel xml:lang="de">Straßenverkehrsunfall</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19574-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19679-5">
    <skos:related rdf:resource="descriptor/13824-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19679-5">
    <skos:related rdf:resource="descriptor/18744-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19679-5">
    <skos:related rdf:resource="descriptor/13828-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11382-5">
    <skos:related rdf:resource="descriptor/19779-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11382-5">
    <skos:related rdf:resource="descriptor/15656-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11382-5">
    <skos:related rdf:resource="descriptor/11401-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11382-5">
    <skos:related rdf:resource="descriptor/18292-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11382-5">
    <skos:related rdf:resource="descriptor/11374-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11382-5">
    <skos:related rdf:resource="descriptor/18868-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11382-5">
    <skos:related rdf:resource="descriptor/18857-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10748-1">
    <skos:broader rdf:resource="thsys/71095"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10748-1">
    <skos:broader rdf:resource="descriptor/10764-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13969-5">
    <skos:related rdf:resource="descriptor/13828-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12631-5">
    <skos:altLabel xml:lang="en">MTM (Methods-Time Measurement)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12631-5">
    <skos:altLabel xml:lang="en">Methods-time measurement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12631-5">
    <skos:altLabel xml:lang="de">Work-Factor-Verfahren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12631-5">
    <skos:altLabel xml:lang="de">Vorgabezeitermittlung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12631-5">
    <skos:altLabel xml:lang="de">Verteilzeitstudie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12631-5">
    <skos:altLabel xml:lang="de">Verfahren vorbestimmter Zeiten</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12631-5">
    <skos:altLabel xml:lang="de">Stufenwertzahlverfahren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12631-5">
    <skos:altLabel xml:lang="de">REFA-System</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12631-5">
    <skos:altLabel xml:lang="de">Lohngruppenverfahren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12631-5">
    <skos:altLabel xml:lang="de">Bewegungsablaufstudie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12631-5">
    <skos:altLabel xml:lang="de">Arbeitszeitstudie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12631-5">
    <skos:altLabel xml:lang="de">Arbeitsstudie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12631-5">
    <skos:altLabel xml:lang="de">Arbeitsplatzstudie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12631-5">
    <skos:altLabel xml:lang="de">Arbeitsanalyse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10894-4">
    <skos:prefLabel xml:lang="en">Regulation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10894-4">
    <skos:prefLabel xml:lang="de">Regulierung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19326-3">
    <skos:prefLabel xml:lang="en">WTO rules</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19326-3">
    <skos:prefLabel xml:lang="de">WTO-Regeln</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20401-5">
    <skos:broader rdf:resource="thsys/70582"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19784-1">
    <skos:altLabel xml:lang="en">Weekend effect</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19784-1">
    <skos:altLabel xml:lang="en">Timing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19784-1">
    <skos:altLabel xml:lang="en">Day-of-the-week effect</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19784-1">
    <skos:altLabel xml:lang="en">Day of the week effect</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19784-1">
    <skos:altLabel xml:lang="en">Data-driven inference</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19784-1">
    <skos:altLabel xml:lang="en">Calendar effects</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19784-1">
    <skos:altLabel xml:lang="de">Zeitpunkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19784-1">
    <skos:altLabel xml:lang="de">Zeitliche Dimension</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19784-1">
    <skos:altLabel xml:lang="de">Zeitfaktor</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19784-1">
    <skos:altLabel xml:lang="de">Kalendereffekt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17735-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id13">
    <rdf:value xml:lang="de">für die Angebotspalette im Handel</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70186">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20733-1">
    <skos:scopeNote xml:lang="de">Internationales Abkommen über Vorschriften zur Bonitätsprüfung anlässlich einer Kreditvergabe (Rating) durch Banken sowie über die sich daraus ergebende Eigenkapitalunterlegung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18716-2">
    <skos:related rdf:resource="descriptor/18709-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14984-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70963">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20306-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15429-6">
    <skos:scopeNote xml:lang="de">Sowohl in der kybernetischen wie auch in der sozialwissenschaftlichen Interpretation.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16742-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12729-2">
    <skos:altLabel xml:lang="de">Werkstattproduktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13302-5">
    <skos:prefLabel xml:lang="en">Non-profit housing organization</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13302-5">
    <skos:prefLabel xml:lang="de">Gemeinnützige Wohnungswirtschaft</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17198-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18781-5">
    <skos:altLabel xml:lang="de">Universitätsinstitut</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/25621-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18349-3">
    <skos:scopeNote xml:lang="de">Nur für die betriebliche Energiewirtschaft.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70071">
    <skos:narrower rdf:resource="descriptor/18195-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70071">
    <skos:narrower rdf:resource="descriptor/14054-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70071">
    <skos:narrower rdf:resource="thsys/70403"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70071">
    <skos:narrower rdf:resource="thsys/70467"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70071">
    <skos:narrower rdf:resource="thsys/70341"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16092-3">
    <skos:altLabel xml:lang="de">EStG (Einkommensteuergesetz)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16092-3">
    <skos:altLabel xml:lang="de">Einkommensteuerrichtlinie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16092-3">
    <skos:altLabel xml:lang="de">Einkommensteuergesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id170">
    <rdfs:seeAlso rdf:resource="descriptor/10276-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10993-2">
    <skos:broader rdf:resource="thsys/73377"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10993-2">
    <skos:broader rdf:resource="thsys/70038"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10993-2">
    <skos:broader rdf:resource="descriptor/10987-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10993-2">
    <skos:broader rdf:resource="descriptor/16432-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18830-4">
    <skos:broader rdf:resource="thsys/73377"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18830-4">
    <skos:broader rdf:resource="thsys/70856"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18830-4">
    <skos:broader rdf:resource="descriptor/16709-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19078-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10527-3">
    <skos:broader rdf:resource="thsys/71086"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10527-3">
    <skos:broader rdf:resource="thsys/70406"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10527-3">
    <skos:broader rdf:resource="thsys/70179"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10527-3">
    <skos:broader rdf:resource="descriptor/10415-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10583-0">
    <zbwext:useInsteadNote rdf:nodeID="id227"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10595-0">
    <skos:altLabel xml:lang="de">Selbsthilfe in Entwicklungsländern</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10595-0">
    <skos:altLabel xml:lang="de">Hilfe zur Selbsthilfe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71109">
    <rdfs:label xml:lang="de">Forschung und Technik</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11543-0">
    <skos:broader rdf:resource="thsys/71081"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11543-0">
    <skos:broader rdf:resource="descriptor/11545-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16265-5">
    <skos:altLabel xml:lang="de">Meinungsbeeinflussung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14674-5">
    <skos:altLabel xml:lang="de">Propylenoxid</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14674-5">
    <skos:altLabel xml:lang="de">Ethylenoxid</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14674-5">
    <skos:altLabel xml:lang="de">Epoxyd</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14674-5">
    <skos:altLabel xml:lang="de">Epichlorhydrin</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14674-5">
    <skos:altLabel xml:lang="de">Äthylenoxid</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73389">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19070-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13160-6">
    <skos:altLabel xml:lang="en">Food processing industry</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13160-6">
    <skos:altLabel xml:lang="de">Nahrungsmittelindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13160-6">
    <skos:altLabel xml:lang="de">Nahrungsmittelhandwerk</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13160-6">
    <skos:altLabel xml:lang="de">Lebensmittelindustrie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13160-6">
    <skos:altLabel xml:lang="de">Genussmittelgewerbe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13160-6">
    <skos:altLabel xml:lang="de">Genußmittelgewerbe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13160-6">
    <skos:altLabel xml:lang="de">Ernährungswirtschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13160-6">
    <skos:altLabel xml:lang="de">Ernährungshandwerk</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19722-2">
    <skos:related rdf:resource="descriptor/11573-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19722-2">
    <skos:related rdf:resource="descriptor/16674-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19722-2">
    <skos:related rdf:resource="descriptor/16472-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19722-2">
    <skos:related rdf:resource="descriptor/11566-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19722-2">
    <skos:related rdf:resource="descriptor/18912-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19722-2">
    <skos:related rdf:resource="descriptor/16470-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19722-2">
    <skos:related rdf:resource="descriptor/16524-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19722-2">
    <skos:related rdf:resource="descriptor/16427-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19722-2">
    <skos:related rdf:resource="descriptor/15919-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16730-4">
    <skos:altLabel xml:lang="de">Werturteilsstreit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16730-4">
    <skos:altLabel xml:lang="de">Wertfreiheit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12781-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14504-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13629-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11986-1">
    <skos:broader rdf:resource="thsys/71064"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11986-1">
    <skos:broader rdf:resource="thsys/70878"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11986-1">
    <skos:broader rdf:resource="descriptor/10436-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17181-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18348-5">
    <skos:broader rdf:resource="thsys/71016"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18348-5">
    <skos:broader rdf:resource="thsys/70878"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18348-5">
    <skos:broader rdf:resource="thsys/70227"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18348-5">
    <skos:broader rdf:resource="descriptor/10436-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70055">
    <skos:broader rdf:resource="thsys/v"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12889-1">
    <skos:broader rdf:resource="thsys/70875"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19419-3">
    <skos:prefLabel xml:lang="en">Forest tax</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19419-3">
    <skos:prefLabel xml:lang="de">Forststeuer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16215-6">
    <skos:altLabel xml:lang="en">Regional geography</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16215-6">
    <skos:altLabel xml:lang="en">Country report</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16215-6">
    <skos:altLabel xml:lang="de">Wissenschaftliche Länderkunde</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16215-6">
    <skos:altLabel xml:lang="de">Vergleichende Länderforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16215-6">
    <skos:altLabel xml:lang="de">Politische Länderkunde</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16215-6">
    <skos:altLabel xml:lang="de">Länderstudien</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16215-6">
    <skos:altLabel xml:lang="de">Länderkunde</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16215-6">
    <skos:altLabel xml:lang="de">Einzelländerforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16215-6">
    <skos:altLabel xml:lang="de">Auslandswissenschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16215-6">
    <skos:altLabel xml:lang="de">Auslandsstudien</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16215-6">
    <skos:altLabel xml:lang="de">Auslandsforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11361-6">
    <zbwext:useInsteadNote rdf:nodeID="id46"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11361-6">
    <zbwext:useInsteadNote rdf:nodeID="id45"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14921-0">
    <skos:broader rdf:resource="thsys/70031"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14921-0">
    <skos:broader rdf:resource="descriptor/14914-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14921-0">
    <skos:broader rdf:resource="descriptor/14899-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70999">
    <skos:prefLabel xml:lang="en">W.06.01.03  Vehicle construction</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70999">
    <skos:prefLabel xml:lang="de">W.06.01.03  Fahrzeugbau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11804-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15644-2">
    <skos:altLabel xml:lang="de">Kulturwissenschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19101-6">
    <skos:related rdf:resource="descriptor/19100-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16052-1">
    <skos:related rdf:resource="descriptor/19824-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16052-1">
    <skos:related rdf:resource="descriptor/11209-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16052-1">
    <skos:related rdf:resource="descriptor/18805-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16052-1">
    <skos:related rdf:resource="descriptor/11822-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16052-1">
    <skos:related rdf:resource="descriptor/11817-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16052-1">
    <skos:related rdf:resource="descriptor/11814-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16052-1">
    <skos:related rdf:resource="descriptor/11185-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11044-4">
    <skos:related rdf:resource="descriptor/10985-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11044-4">
    <skos:related rdf:resource="descriptor/11124-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11044-4">
    <skos:related rdf:resource="descriptor/11142-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11044-4">
    <skos:related rdf:resource="descriptor/11616-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11044-4">
    <skos:related rdf:resource="descriptor/16409-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11044-4">
    <skos:related rdf:resource="descriptor/11825-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11044-4">
    <skos:related rdf:resource="descriptor/11036-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11044-4">
    <skos:related rdf:resource="descriptor/19488-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18837-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13736-0">
    <skos:prefLabel xml:lang="en">Mortgage bond</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13736-0">
    <skos:prefLabel xml:lang="de">Pfandbrief</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11684-3">
    <skos:related rdf:resource="descriptor/19567-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16967-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15097-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13032-1">
    <skos:broader rdf:resource="thsys/71003"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13032-1">
    <skos:broader rdf:resource="descriptor/13034-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12585-0">
    <skos:narrower rdf:resource="descriptor/12580-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12585-0">
    <skos:narrower rdf:resource="descriptor/11306-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18764-5">
    <skos:prefLabel xml:lang="en">Computer scientists</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18764-5">
    <skos:prefLabel xml:lang="de">Informatiker</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11038-6">
    <skos:related rdf:resource="descriptor/16212-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11038-6">
    <skos:related rdf:resource="descriptor/12112-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11038-6">
    <skos:related rdf:resource="descriptor/11154-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15554-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14679-2">
    <skos:related rdf:resource="descriptor/14049-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70014">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">P.12</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11974-1">
    <skos:broader rdf:resource="thsys/71064"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11974-1">
    <skos:broader rdf:resource="thsys/70925"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11974-1">
    <skos:broader rdf:resource="descriptor/13510-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70257">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15977-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11174-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10124-4">
    <skos:prefLabel xml:lang="en">Inter-regional capital mobility</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10124-4">
    <skos:prefLabel xml:lang="de">Interregionale Kapitalmobilität</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71067">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13426-1">
    <skos:related rdf:resource="descriptor/15769-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13426-1">
    <skos:related rdf:resource="descriptor/18851-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13426-1">
    <skos:related rdf:resource="descriptor/12945-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13426-1">
    <skos:related rdf:resource="descriptor/18311-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13426-1">
    <skos:related rdf:resource="descriptor/18312-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13426-1">
    <skos:related rdf:resource="descriptor/12020-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13426-1">
    <skos:related rdf:resource="descriptor/13451-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13426-1">
    <skos:related rdf:resource="descriptor/11801-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12921-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19269-3">
    <skos:broader rdf:resource="thsys/70897"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19269-3">
    <skos:broader rdf:resource="thsys/70187"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19269-3">
    <skos:broader rdf:resource="descriptor/12210-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19502-2">
    <skos:prefLabel xml:lang="en">Unionization</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19502-2">
    <skos:prefLabel xml:lang="de">Gewerkschaftlicher Organisationsgrad</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10791-0">
    <skos:related rdf:resource="descriptor/19590-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10791-0">
    <skos:related rdf:resource="descriptor/10677-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10791-0">
    <skos:related rdf:resource="descriptor/10770-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10791-0">
    <skos:related rdf:resource="descriptor/10272-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10791-0">
    <skos:related rdf:resource="descriptor/10676-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10791-0">
    <skos:related rdf:resource="descriptor/10785-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10297-1">
    <skos:scopeNote xml:lang="de">Makroökonomische Betrachtung der Lagerhaltung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71090">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.07.06</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16406-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70278">
    <skos:narrower rdf:resource="descriptor/11341-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70278">
    <skos:narrower rdf:resource="descriptor/19501-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70278">
    <skos:narrower rdf:resource="descriptor/16150-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70278">
    <skos:narrower rdf:resource="descriptor/16484-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70278">
    <skos:narrower rdf:resource="descriptor/11331-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70278">
    <skos:narrower rdf:resource="descriptor/18309-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70278">
    <skos:narrower rdf:resource="descriptor/19440-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70278">
    <skos:narrower rdf:resource="descriptor/11330-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70278">
    <skos:narrower rdf:resource="descriptor/12820-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70278">
    <skos:narrower rdf:resource="descriptor/10622-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70278">
    <skos:narrower rdf:resource="descriptor/11328-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70278">
    <skos:narrower rdf:resource="descriptor/11337-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70278">
    <skos:narrower rdf:resource="descriptor/11346-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70278">
    <skos:narrower rdf:resource="descriptor/11336-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70278">
    <skos:narrower rdf:resource="descriptor/11327-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70278">
    <skos:narrower rdf:resource="descriptor/15089-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70278">
    <skos:narrower rdf:resource="descriptor/11335-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70278">
    <skos:narrower rdf:resource="descriptor/11326-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70278">
    <skos:narrower rdf:resource="descriptor/11345-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70278">
    <skos:narrower rdf:resource="descriptor/11325-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70278">
    <skos:narrower rdf:resource="descriptor/10392-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70278">
    <skos:narrower rdf:resource="descriptor/11510-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70278">
    <skos:narrower rdf:resource="descriptor/16143-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70278">
    <skos:narrower rdf:resource="descriptor/11324-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70278">
    <skos:narrower rdf:resource="descriptor/19011-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70278">
    <skos:narrower rdf:resource="descriptor/11342-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70278">
    <skos:narrower rdf:resource="descriptor/11322-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70278">
    <skos:narrower rdf:resource="descriptor/11741-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70278">
    <skos:narrower rdf:resource="descriptor/12594-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70278">
    <skos:narrower rdf:resource="descriptor/11321-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70278">
    <skos:narrower rdf:resource="descriptor/19500-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70278">
    <skos:narrower rdf:resource="descriptor/11320-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70278">
    <skos:narrower rdf:resource="descriptor/19499-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70278">
    <skos:narrower rdf:resource="descriptor/18173-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70278">
    <skos:narrower rdf:resource="descriptor/11318-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70278">
    <skos:narrower rdf:resource="descriptor/20368-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19474-2">
    <skos:broader rdf:resource="thsys/71064"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19474-2">
    <skos:broader rdf:resource="thsys/70504"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19474-2">
    <skos:broader rdf:resource="descriptor/10488-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core">
    <ns0:title xmlns:ns0="http://purl.org/dc/elements/1.1/" xml:lang="en">SKOS Core Vocabulary</ns0:title>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14736-2">
    <skos:broader rdf:resource="thsys/70536"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14736-2">
    <skos:broader rdf:resource="descriptor/14734-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70113">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">B.04</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70011">
    <skos:broader rdf:resource="thsys/70012"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71040">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12055-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17986-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15011-6">
    <skos:altLabel xml:lang="de">Recycling-Produkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12435-5">
    <skos:prefLabel xml:lang="en">Maintenance of assets</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12435-5">
    <skos:prefLabel xml:lang="de">Substanzerhaltung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18217-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70175">
    <skos:narrower rdf:resource="descriptor/10032-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70175">
    <skos:narrower rdf:resource="descriptor/18871-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70175">
    <skos:narrower rdf:resource="descriptor/18880-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70175">
    <skos:narrower rdf:resource="descriptor/18780-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70175">
    <skos:narrower rdf:resource="descriptor/10037-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70175">
    <skos:narrower rdf:resource="descriptor/18130-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70175">
    <skos:narrower rdf:resource="descriptor/19563-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70175">
    <skos:narrower rdf:resource="descriptor/19073-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70175">
    <skos:narrower rdf:resource="descriptor/10013-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70175">
    <skos:narrower rdf:resource="descriptor/21283-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70175">
    <skos:narrower rdf:resource="descriptor/10012-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70175">
    <skos:narrower rdf:resource="descriptor/10011-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70175">
    <skos:narrower rdf:resource="descriptor/10030-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70175">
    <skos:narrower rdf:resource="descriptor/19544-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70175">
    <skos:narrower rdf:resource="descriptor/10000-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10346-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19363-4">
    <skos:prefLabel xml:lang="en">Urban growth</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19363-4">
    <skos:prefLabel xml:lang="de">Stadtwachstum</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14042-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15429-6">
    <skos:related rdf:resource="descriptor/16563-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15429-6">
    <skos:related rdf:resource="descriptor/19751-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15429-6">
    <skos:related rdf:resource="descriptor/15054-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15429-6">
    <skos:related rdf:resource="descriptor/15416-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15429-6">
    <skos:related rdf:resource="descriptor/18867-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15429-6">
    <skos:related rdf:resource="descriptor/15415-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15429-6">
    <skos:related rdf:resource="descriptor/15555-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15429-6">
    <skos:related rdf:resource="descriptor/16579-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71067">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10286-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14516-2">
    <skos:related rdf:resource="descriptor/13665-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11365-5">
    <skos:scopeNote xml:lang="de">Nur für formale Ausbildungsgänge.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13363-6">
    <skos:related rdf:resource="descriptor/18954-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13363-6">
    <skos:related rdf:resource="descriptor/15675-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16697-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core#semanticRelation">
    <rdfs:isDefinedBy rdf:resource="http://www.w3.org/2004/02/skos/core"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19307-0">
    <skos:broader rdf:resource="thsys/71029"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19307-0">
    <skos:broader rdf:resource="thsys/71020"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19307-0">
    <skos:broader rdf:resource="descriptor/11801-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11669-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13612-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12930-2">
    <skos:prefLabel xml:lang="en">Packaging industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12930-2">
    <skos:prefLabel xml:lang="de">Verpackungsindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11222-6">
    <skos:broader rdf:resource="thsys/70847"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11222-6">
    <skos:broader rdf:resource="thsys/70176"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11222-6">
    <skos:broader rdf:resource="descriptor/11552-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11222-6">
    <skos:broader rdf:resource="descriptor/11407-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14788-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19529-3">
    <skos:altLabel xml:lang="de">Strahlenphysik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19529-3">
    <skos:altLabel xml:lang="de">Strahlenkunde</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19529-3">
    <skos:altLabel xml:lang="de">Röntgentherapie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19529-3">
    <skos:altLabel xml:lang="de">Röntgen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19529-3">
    <skos:altLabel xml:lang="de">Radiologisches Verfahren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19529-3">
    <skos:altLabel xml:lang="de">Nuklearmedizin</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19529-3">
    <skos:altLabel xml:lang="de">Computertomographie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11556-5">
    <skos:prefLabel xml:lang="en">Public Investment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11556-5">
    <skos:prefLabel xml:lang="de">Öffentliche Investition</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13399-6">
    <skos:related rdf:resource="descriptor/15682-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13399-6">
    <skos:related rdf:resource="descriptor/15671-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13399-6">
    <skos:related rdf:resource="descriptor/19706-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13399-6">
    <skos:related rdf:resource="descriptor/14012-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15398-5">
    <skos:prefLabel xml:lang="en">Discriminant analysis</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15398-5">
    <skos:prefLabel xml:lang="de">Diskriminanzanalyse</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11934-6">
    <skos:broader rdf:resource="thsys/71070"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11934-6">
    <skos:broader rdf:resource="thsys/71067"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11934-6">
    <skos:broader rdf:resource="thsys/70927"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11934-6">
    <skos:broader rdf:resource="thsys/70386"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11934-6">
    <skos:broader rdf:resource="descriptor/10011-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18789-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16318-3">
    <skos:prefLabel xml:lang="en">Welfare state</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16318-3">
    <skos:prefLabel xml:lang="de">Sozialstaat</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11527-5">
    <skos:broader rdf:resource="thsys/70057"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11527-5">
    <skos:broader rdf:resource="descriptor/10113-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18103-0">
    <skos:broader rdf:resource="thsys/70254"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18103-0">
    <skos:broader rdf:resource="descriptor/14473-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10250-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18291-1">
    <skos:broader rdf:resource="thsys/70476"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18291-1">
    <skos:broader rdf:resource="descriptor/14854-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14814-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12423-5">
    <skos:related rdf:resource="descriptor/19673-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12423-5">
    <skos:related rdf:resource="descriptor/12438-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12423-5">
    <skos:related rdf:resource="descriptor/12408-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10344-4">
    <skos:altLabel xml:lang="en">Self-fulfilling expectations</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10344-4">
    <skos:altLabel xml:lang="en">Animal spirits hypothesis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10344-4">
    <skos:altLabel xml:lang="de">Sonnenfleckentheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10344-4">
    <skos:altLabel xml:lang="de">Sonnenfleckenhypothese</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18993-4">
    <skos:prefLabel xml:lang="en">Rural small business</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18993-4">
    <skos:prefLabel xml:lang="de">Ländliches Kleingewerbe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18573-5">
    <skos:broader rdf:resource="thsys/70941"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18573-5">
    <skos:broader rdf:resource="thsys/70940"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18573-5">
    <skos:broader rdf:resource="descriptor/13255-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18573-5">
    <skos:broader rdf:resource="descriptor/13621-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19156-2">
    <skos:related rdf:resource="descriptor/19154-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19156-2">
    <skos:related rdf:resource="descriptor/12175-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19624-2">
    <skos:broader rdf:resource="thsys/73352"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19624-2">
    <skos:broader rdf:resource="descriptor/15889-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18552-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71078">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">V.09.06</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19775-2">
    <skos:altLabel xml:lang="en">Bilingualism</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19775-2">
    <skos:altLabel xml:lang="de">Zweisprachigkeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19775-2">
    <skos:altLabel xml:lang="de">Bilinguismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19775-2">
    <skos:altLabel xml:lang="de">Bilingualismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14413-5">
    <skos:prefLabel xml:lang="en">Military vehicle</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14413-5">
    <skos:prefLabel xml:lang="de">Militärfahrzeug</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10508-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73384">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14042-0">
    <skos:narrower rdf:resource="descriptor/18252-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14042-0">
    <skos:narrower rdf:resource="descriptor/14025-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15473-3">
    <skos:prefLabel xml:lang="en">Information acquisition</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15473-3">
    <skos:prefLabel xml:lang="de">Informationsbeschaffung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12325-5">
    <skos:broader rdf:resource="thsys/71043"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12325-5">
    <skos:broader rdf:resource="descriptor/12307-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16566-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13811-5">
    <skos:narrower rdf:resource="descriptor/18739-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13811-5">
    <skos:narrower rdf:resource="descriptor/18738-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13051-4">
    <skos:prefLabel xml:lang="en">Automobile industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13051-4">
    <skos:prefLabel xml:lang="de">Kraftfahrzeugindustrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11839-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12699-6">
    <skos:narrower rdf:resource="descriptor/13844-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10658-2">
    <skos:related rdf:resource="descriptor/11206-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10658-2">
    <skos:related rdf:resource="descriptor/11873-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11970-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13415-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17940-2">
    <skos:prefLabel xml:lang="en">Micronesia</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17940-2">
    <skos:prefLabel xml:lang="de">Mikronesien</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20401-5">
    <skos:prefLabel xml:lang="en">Integration</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20401-5">
    <skos:prefLabel xml:lang="de">Integration</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12952-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19500-6">
    <skos:broader rdf:resource="thsys/70278"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19500-6">
    <skos:broader rdf:resource="descriptor/11341-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12580-3">
    <skos:prefLabel xml:lang="en">Participative management</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12580-3">
    <skos:prefLabel xml:lang="de">Kooperative Führung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19834-5">
    <skos:prefLabel xml:lang="en">Balanced scorecard</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19834-5">
    <skos:prefLabel xml:lang="de">Balanced Scorecard</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13762-6">
    <skos:scopeNote xml:lang="de">Veräußerung von Vermögenswerten mit Rückkaufverpflichtung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18323-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19436-3">
    <skos:broader rdf:resource="thsys/71081"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19436-3">
    <skos:broader rdf:resource="descriptor/11715-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19237-2">
    <skos:altLabel xml:lang="en">SAM (Social Accounting Matrix)</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14740-4">
    <skos:related rdf:resource="descriptor/18261-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14740-4">
    <skos:related rdf:resource="descriptor/14741-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14740-4">
    <skos:related rdf:resource="descriptor/14630-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70874">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19294-4">
    <skos:related rdf:resource="descriptor/19295-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19294-4">
    <skos:related rdf:resource="descriptor/18518-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19294-4">
    <skos:related rdf:resource="descriptor/18519-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19294-4">
    <skos:related rdf:resource="descriptor/18192-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19294-4">
    <skos:related rdf:resource="descriptor/18191-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19294-4">
    <skos:related rdf:resource="descriptor/14064-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16521-6">
    <skos:altLabel xml:lang="de">Äußere Sicherheit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13965-6">
    <skos:altLabel xml:lang="de">Sparkassenangestellte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13965-6">
    <skos:altLabel xml:lang="de">Bankpersonal</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13965-6">
    <skos:altLabel xml:lang="de">Bankkaufleute</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13965-6">
    <skos:altLabel xml:lang="de">Bankangestellte</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10525-0">
    <skos:related rdf:resource="descriptor/15720-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10525-0">
    <skos:related rdf:resource="descriptor/19204-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10525-0">
    <skos:related rdf:resource="descriptor/10560-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10525-0">
    <skos:related rdf:resource="descriptor/10492-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10525-0">
    <skos:related rdf:resource="descriptor/19242-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10525-0">
    <skos:related rdf:resource="descriptor/10555-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10525-0">
    <skos:related rdf:resource="descriptor/19361-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10525-0">
    <skos:related rdf:resource="descriptor/10513-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10525-0">
    <skos:related rdf:resource="descriptor/10522-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10206-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16330-6">
    <skos:altLabel xml:lang="en">Prime minister</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16330-6">
    <skos:altLabel xml:lang="de">Premierminister</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16330-6">
    <skos:altLabel xml:lang="de">Ministerpräsident</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16330-6">
    <skos:altLabel xml:lang="de">Kanzler</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16330-6">
    <skos:altLabel xml:lang="de">Bundeskanzler</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11297-3">
    <skos:broader rdf:resource="thsys/72181"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11297-3">
    <skos:broader rdf:resource="thsys/71058"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11297-3">
    <skos:broader rdf:resource="descriptor/15076-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11297-3">
    <skos:broader rdf:resource="descriptor/15061-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14449-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16961-6">
    <skos:scopeNote xml:lang="de">Für die Region Genf BENUTZE: Genf (Region).</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16961-6">
    <skos:scopeNote xml:lang="de">Hier Literatur zu Stadt und Kanton Genf.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19559-1">
    <skos:related rdf:resource="descriptor/11404-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19559-1">
    <skos:related rdf:resource="descriptor/11377-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19559-1">
    <skos:related rdf:resource="descriptor/18874-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19758-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11536-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17923-2">
    <skos:broader rdf:resource="thsys/71030"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17923-2">
    <skos:broader rdf:resource="descriptor/17931-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17923-2">
    <skos:broader rdf:resource="descriptor/19170-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10160-0">
    <skos:broader rdf:resource="thsys/71118"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10160-0">
    <skos:broader rdf:resource="descriptor/10161-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16159-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16953-5">
    <skos:prefLabel xml:lang="en">Tyrol</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16953-5">
    <skos:prefLabel xml:lang="de">Tirol</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70852">
    <skos:broader rdf:resource="thsys/70853"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19294-4">
    <skos:broader rdf:resource="thsys/73294"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19294-4">
    <skos:broader rdf:resource="thsys/71029"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19294-4">
    <skos:broader rdf:resource="thsys/70960"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19294-4">
    <skos:broader rdf:resource="descriptor/19338-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19294-4">
    <skos:broader rdf:resource="descriptor/13431-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70582">
    <rdfs:label xml:lang="de">Allgemeinwörter</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16595-5">
    <skos:broader rdf:resource="thsys/70321"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11777-3">
    <skos:broader rdf:resource="thsys/71113"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11777-3">
    <skos:broader rdf:resource="thsys/70184"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11777-3">
    <skos:broader rdf:resource="descriptor/11778-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13246-3">
    <skos:altLabel xml:lang="de">Zivilluftfahrt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13246-3">
    <skos:altLabel xml:lang="de">Luftschifffahrt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13246-3">
    <skos:altLabel xml:lang="de">Luftschiffahrt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13246-3">
    <skos:altLabel xml:lang="de">Luftfahrt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13246-3">
    <skos:altLabel xml:lang="de">Flugverkehr</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19390-1">
    <skos:prefLabel xml:lang="en">Steel market</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19390-1">
    <skos:prefLabel xml:lang="de">Stahlmarkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18394-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10139-5">
    <skos:altLabel xml:lang="de">Empirische Mikroökonomik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18561-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15635-3">
    <skos:altLabel xml:lang="de">Biotechnik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15635-3">
    <skos:altLabel xml:lang="de">Biochemie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18639-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id326">
    <rdf:value xml:lang="de">Für die Anbieter von Telekommunikationsdiensten</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14561-4">
    <skos:related rdf:resource="descriptor/14545-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14561-4">
    <skos:related rdf:resource="descriptor/14504-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16427-5">
    <skos:prefLabel xml:lang="en">Peace</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16427-5">
    <skos:prefLabel xml:lang="de">Frieden</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18188-1">
    <skos:altLabel xml:lang="en">Uncovered interest parity</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18188-1">
    <skos:altLabel xml:lang="en">Interest rate linkage</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18188-1">
    <skos:altLabel xml:lang="en">Interest parity</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18188-1">
    <skos:altLabel xml:lang="en">Covered interest parity</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18188-1">
    <skos:altLabel xml:lang="de">Ungedeckte Zinsparität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18188-1">
    <skos:altLabel xml:lang="de">Internationaler Zinszusammenhang</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18188-1">
    <skos:altLabel xml:lang="de">Gedeckte Zinsparität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14638-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15655-4">
    <skos:narrower rdf:resource="descriptor/15661-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15655-4">
    <skos:narrower rdf:resource="descriptor/15657-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15655-4">
    <skos:narrower rdf:resource="descriptor/18492-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19159-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13060-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19350-6">
    <skos:scopeNote xml:lang="de">Vertraglicher Zusammenschluss mehrerer Staaten zur Einleitung eines wirtschaftlichen Integrationsprozesses.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19176-3">
    <skos:altLabel xml:lang="de">ECOWAS-Länder</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13387-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71103">
    <skos:prefLabel xml:lang="en">V.05.06.04  Foreign exchange market</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71103">
    <skos:prefLabel xml:lang="de">V.05.06.04  Devisenmarkt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12185-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18149-4">
    <skos:scopeNote xml:lang="de">Budgetärer Inflationsansatz (Public Finance Approach to Inflation): langfristig bestimmt sich die Inflation durch den Finanzierungsbedarf des Staates, der bei Geldschöpfung den Münzgewinn einbehält. Gesucht wird die einnahmenmaximierende Inflationsrate.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11668-1">
    <skos:broader rdf:resource="thsys/71079"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11668-1">
    <skos:broader rdf:resource="descriptor/11547-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11668-1">
    <skos:broader rdf:resource="descriptor/19370-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11668-1">
    <skos:broader rdf:resource="descriptor/11599-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70901">
    <skos:broader rdf:resource="thsys/70903"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19902-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16053-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11905-6">
    <skos:prefLabel xml:lang="en">Firm location</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11905-6">
    <skos:prefLabel xml:lang="de">Betrieblicher Standort</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12702-1">
    <skos:broader rdf:resource="thsys/73345"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12702-1">
    <skos:broader rdf:resource="thsys/70030"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70872">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73364">
    <skos:narrower rdf:resource="descriptor/18064-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73364">
    <skos:narrower rdf:resource="descriptor/16288-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73364">
    <skos:narrower rdf:resource="descriptor/16472-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73364">
    <skos:narrower rdf:resource="descriptor/11051-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73364">
    <skos:narrower rdf:resource="descriptor/18912-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73364">
    <skos:narrower rdf:resource="descriptor/16003-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73364">
    <skos:narrower rdf:resource="descriptor/19721-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73364">
    <skos:narrower rdf:resource="descriptor/19722-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73364">
    <skos:narrower rdf:resource="descriptor/19381-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73364">
    <skos:narrower rdf:resource="descriptor/16470-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73364">
    <skos:narrower rdf:resource="descriptor/16524-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73364">
    <skos:narrower rdf:resource="descriptor/16471-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73364">
    <skos:narrower rdf:resource="descriptor/16204-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73364">
    <skos:narrower rdf:resource="descriptor/16427-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core">
    <ns0:creator xmlns:ns0="http://purl.org/dc/elements/1.1/">Sean Bechhofer</ns0:creator>
  </rdf:Description>
  <rdf:Description rdf:about="http://www.w3.org/2004/02/skos/core">
    <ns0:creator xmlns:ns0="http://purl.org/dc/elements/1.1/">Alistair Miles</ns0:creator>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11007-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13269-5">
    <skos:related rdf:resource="descriptor/18664-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13269-5">
    <skos:related rdf:resource="descriptor/19145-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19168-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18885-0">
    <skos:broader rdf:resource="thsys/70843"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18885-0">
    <skos:broader rdf:resource="thsys/70244"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18885-0">
    <skos:broader rdf:resource="descriptor/11391-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18885-0">
    <skos:broader rdf:resource="descriptor/11366-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19179-4">
    <skos:broader rdf:resource="thsys/70307"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19179-4">
    <skos:broader rdf:resource="descriptor/10492-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18633-6">
    <skos:prefLabel xml:lang="en">Urban transport policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18633-6">
    <skos:prefLabel xml:lang="de">Stadtverkehrspolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10139-5">
    <skos:prefLabel xml:lang="en">Microeconometrics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10139-5">
    <skos:prefLabel xml:lang="de">Mikroökonometrie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19813-6">
    <skos:related rdf:resource="descriptor/11217-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19813-6">
    <skos:related rdf:resource="descriptor/15979-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19813-6">
    <skos:related rdf:resource="descriptor/19816-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19813-6">
    <skos:related rdf:resource="descriptor/19763-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19813-6">
    <skos:related rdf:resource="descriptor/16198-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19813-6">
    <skos:related rdf:resource="descriptor/16504-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19813-6">
    <skos:related rdf:resource="descriptor/16195-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70881">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71026">
    <skos:broader rdf:resource="thsys/71029"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18619-0">
    <skos:altLabel xml:lang="de">Kühlhaus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13028-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18381-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14729-6">
    <skos:related rdf:resource="descriptor/19013-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13516-0">
    <skos:related rdf:resource="descriptor/18607-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13516-0">
    <skos:related rdf:resource="descriptor/13246-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13516-0">
    <skos:related rdf:resource="descriptor/20897-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13516-0">
    <skos:related rdf:resource="descriptor/18604-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13516-0">
    <skos:related rdf:resource="descriptor/18498-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71014">
    <skos:narrower rdf:resource="descriptor/14364-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71014">
    <skos:narrower rdf:resource="descriptor/14053-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71014">
    <skos:narrower rdf:resource="descriptor/14189-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71014">
    <skos:narrower rdf:resource="descriptor/18364-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71014">
    <skos:narrower rdf:resource="descriptor/18363-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71014">
    <skos:narrower rdf:resource="descriptor/14186-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71014">
    <skos:narrower rdf:resource="descriptor/14407-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71014">
    <skos:narrower rdf:resource="descriptor/18274-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71014">
    <skos:narrower rdf:resource="descriptor/20888-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71014">
    <skos:narrower rdf:resource="descriptor/18362-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70942">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70860">
    <skos:notation rdf:datatype="http://www.w3.org/2001/XMLSchema#string">W.22.02</skos:notation>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14102-1">
    <skos:altLabel xml:lang="de">Weinbeere</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14102-1">
    <skos:altLabel xml:lang="de">Tafeltraube</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14102-1">
    <skos:altLabel xml:lang="de">Rosine</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18100-6">
    <skos:broader rdf:resource="thsys/70041"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18100-6">
    <skos:broader rdf:resource="descriptor/12766-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18100-6">
    <skos:broader rdf:resource="descriptor/12665-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15995-1">
    <skos:scopeNote xml:lang="de">Gesamtheit der Rechtsvorschriften zu Organisation, Rechtsstellung, Bildung und Auflösung von Gemeinden sowie anderen Gebietskörperschaften.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10546-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12777-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19337-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12587-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12473-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70854">
    <skos:broader rdf:resource="thsys/70855"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19094-5">
    <skos:broader rdf:resource="thsys/71043"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19094-5">
    <skos:broader rdf:resource="descriptor/12354-5"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id148">
    <rdfs:seeAlso rdf:resource="descriptor/12105-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12963-1">
    <skos:prefLabel xml:lang="en">Inland fishery</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12963-1">
    <skos:prefLabel xml:lang="de">Binnenfischerei</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10304-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18567-0">
    <skos:broader rdf:resource="thsys/73389"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18567-0">
    <skos:broader rdf:resource="thsys/73332"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18567-0">
    <skos:broader rdf:resource="thsys/70943"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18567-0">
    <skos:broader rdf:resource="descriptor/16188-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18567-0">
    <skos:broader rdf:resource="descriptor/13569-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14371-2">
    <skos:related rdf:resource="descriptor/14385-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14371-2">
    <skos:related rdf:resource="descriptor/18434-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14371-2">
    <skos:related rdf:resource="descriptor/13102-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14371-2">
    <skos:related rdf:resource="descriptor/14812-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12631-5">
    <zbwext:useInsteadNote rdf:nodeID="id19"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13721-6">
    <skos:related rdf:resource="descriptor/19264-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13721-6">
    <skos:related rdf:resource="descriptor/13738-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13721-6">
    <skos:related rdf:resource="descriptor/13709-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13828-2">
    <skos:narrower rdf:resource="descriptor/13808-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13828-2">
    <skos:narrower rdf:resource="descriptor/13418-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13828-2">
    <skos:narrower rdf:resource="descriptor/13297-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13828-2">
    <skos:narrower rdf:resource="descriptor/13811-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13828-2">
    <skos:narrower rdf:resource="descriptor/13294-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13828-2">
    <skos:narrower rdf:resource="descriptor/13292-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13828-2">
    <skos:narrower rdf:resource="descriptor/16532-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12780-2">
    <skos:prefLabel xml:lang="en">Direct marketing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12780-2">
    <skos:prefLabel xml:lang="de">Direktmarketing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11494-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15715-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11356-6">
    <skos:narrower rdf:resource="descriptor/13375-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11356-6">
    <skos:narrower rdf:resource="descriptor/13349-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11356-6">
    <skos:narrower rdf:resource="descriptor/13960-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11356-6">
    <skos:narrower rdf:resource="descriptor/13370-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11356-6">
    <skos:narrower rdf:resource="descriptor/14005-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18534-1">
    <skos:related rdf:resource="descriptor/10605-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18534-1">
    <skos:related rdf:resource="descriptor/10833-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10653-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13624-4">
    <skos:prefLabel xml:lang="en">Lightering</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13624-4">
    <skos:prefLabel xml:lang="de">Leichterverkehr</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18662-6">
    <skos:altLabel xml:lang="de">Netzwerkverwaltung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18662-6">
    <skos:altLabel xml:lang="de">Netzwerkadministration</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18662-6">
    <skos:altLabel xml:lang="de">Netzmanagement</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19489-3">
    <skos:broader rdf:resource="thsys/70811"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19489-3">
    <skos:broader rdf:resource="descriptor/19524-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18972-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10852-6">
    <skos:broader rdf:resource="thsys/71091"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10852-6">
    <skos:broader rdf:resource="thsys/70952"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12994-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12228-3">
    <skos:altLabel xml:lang="de">Zahlungsfähigkeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12228-3">
    <skos:altLabel xml:lang="de">Unternehmensliquidität</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12228-3">
    <skos:altLabel xml:lang="de">Solvenz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12228-3">
    <skos:altLabel xml:lang="de">Liquidität des Betriebs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10593-4">
    <skos:related rdf:resource="descriptor/10566-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10593-4">
    <skos:related rdf:resource="descriptor/10545-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12234-1">
    <skos:prefLabel xml:lang="en">Bond</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12234-1">
    <skos:prefLabel xml:lang="de">Anleihe</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16808-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19054-3">
    <skos:related rdf:resource="descriptor/18985-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18509-0">
    <skos:altLabel xml:lang="de">Meisterinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18509-0">
    <skos:altLabel xml:lang="de">Meister</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18509-0">
    <skos:altLabel xml:lang="de">Handwerksmeisterinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18509-0">
    <skos:altLabel xml:lang="de">Handwerksmeister</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18509-0">
    <skos:altLabel xml:lang="de">Handwerkerinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18509-0">
    <skos:altLabel xml:lang="de">Handwerker</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18509-0">
    <skos:altLabel xml:lang="de">Gesellinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18509-0">
    <skos:altLabel xml:lang="de">Gesellen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16076-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19651-6">
    <skos:narrower rdf:resource="descriptor/20921-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13452-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16541-0">
    <skos:prefLabel xml:lang="en">Rehabilitation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16541-0">
    <skos:prefLabel xml:lang="de">Rehabilitation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11347-0">
    <skos:broader rdf:resource="thsys/73336"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11347-0">
    <skos:broader rdf:resource="thsys/71051"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11347-0">
    <skos:broader rdf:resource="thsys/70235"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11347-0">
    <skos:broader rdf:resource="descriptor/16484-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11347-0">
    <skos:broader rdf:resource="descriptor/11185-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12850-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19859-3">
    <skos:related rdf:resource="descriptor/19491-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19859-3">
    <skos:related rdf:resource="descriptor/16131-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10401-4">
    <skos:altLabel xml:lang="en">Social compact</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10401-4">
    <skos:altLabel xml:lang="en">Social accord</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10401-4">
    <skos:altLabel xml:lang="de">Konzertierte Aktion</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10401-4">
    <skos:altLabel xml:lang="de">Bündnis für Arbeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18645-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11417-5">
    <skos:broader rdf:resource="thsys/73390"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11417-5">
    <skos:broader rdf:resource="thsys/70055"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11417-5">
    <skos:broader rdf:resource="descriptor/16081-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11417-5">
    <skos:broader rdf:resource="descriptor/16007-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16975-2">
    <skos:prefLabel xml:lang="en">Thurgau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16975-2">
    <skos:prefLabel xml:lang="de">Thurgau</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19749-3">
    <skos:broader rdf:resource="thsys/73393"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11248-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15981-5">
    <skos:scopeNote xml:lang="de">Gesamtheit der Regelungen, die rechtlich relevante Verfahren (z.B. Verwaltungsverfahren, Gerichtsverfahren) betreffen.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19134-5">
    <skos:related rdf:resource="descriptor/16034-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19134-5">
    <skos:related rdf:resource="descriptor/12786-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19134-5">
    <skos:related rdf:resource="descriptor/19124-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19134-5">
    <skos:related rdf:resource="descriptor/19133-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19002-1">
    <skos:broader rdf:resource="thsys/73385"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19002-1">
    <skos:broader rdf:resource="thsys/70244"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73299">
    <skos:prefLabel xml:lang="en">P.20.06  Feed</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73299">
    <skos:prefLabel xml:lang="de">P.20.06  Futtermittel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13205-3">
    <skos:broader rdf:resource="thsys/70960"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13205-3">
    <skos:broader rdf:resource="descriptor/13219-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13205-3">
    <skos:broader rdf:resource="descriptor/19383-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12207-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id219">
    <rdf:value xml:lang="de">Für die Milchverarbeitende Industrie</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18572-0">
    <skos:scopeNote xml:lang="de">Private und öffentliche Betreiber von Eisenbahnlinien.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16841-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73298">
    <skos:narrower rdf:resource="descriptor/14999-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73298">
    <skos:narrower rdf:resource="descriptor/14998-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73298">
    <skos:narrower rdf:resource="descriptor/14083-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73298">
    <skos:narrower rdf:resource="descriptor/14997-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73298">
    <skos:narrower rdf:resource="descriptor/14996-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73298">
    <skos:narrower rdf:resource="descriptor/14995-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73298">
    <skos:narrower rdf:resource="descriptor/18213-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73298">
    <skos:narrower rdf:resource="descriptor/14065-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73298">
    <skos:narrower rdf:resource="descriptor/14064-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73298">
    <skos:narrower rdf:resource="descriptor/14060-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73298">
    <skos:narrower rdf:resource="descriptor/18222-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73298">
    <skos:narrower rdf:resource="descriptor/14972-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73368">
    <skos:narrower rdf:resource="descriptor/11833-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73368">
    <skos:narrower rdf:resource="descriptor/18653-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73368">
    <skos:narrower rdf:resource="descriptor/10981-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73368">
    <skos:narrower rdf:resource="descriptor/16552-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73368">
    <skos:narrower rdf:resource="descriptor/12004-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73368">
    <skos:narrower rdf:resource="descriptor/18481-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73368">
    <skos:narrower rdf:resource="descriptor/10437-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73368">
    <skos:narrower rdf:resource="descriptor/10524-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73368">
    <skos:narrower rdf:resource="descriptor/10560-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73368">
    <skos:narrower rdf:resource="descriptor/12018-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10434-3">
    <skos:altLabel xml:lang="en">Warranted rate of growth</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10434-3">
    <skos:altLabel xml:lang="de">Wirtschaftswachstumstheorie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10434-3">
    <skos:altLabel xml:lang="de">Wachstumsmodell</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10434-3">
    <skos:altLabel xml:lang="de">Traverse-Problem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10434-3">
    <skos:altLabel xml:lang="de">Befriedigende Wachstumsrate</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15916-2">
    <skos:altLabel xml:lang="de">Bevölkerungswissenschaft</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15916-2">
    <skos:altLabel xml:lang="de">Bevölkerungslehre</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15916-2">
    <skos:altLabel xml:lang="de">Bevölkerungsforschung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19328-6">
    <skos:broader rdf:resource="thsys/71097"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71062">
    <skos:prefLabel xml:lang="en">V.13.01.01  Labour market theory</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71062">
    <skos:prefLabel xml:lang="de">V.13.01.01  Arbeitsmarkttheorie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11705-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11335-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17176-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13837-1">
    <skos:altLabel xml:lang="de">Wohnungsmiete</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13837-1">
    <skos:altLabel xml:lang="de">Mietzins</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13837-1">
    <skos:altLabel xml:lang="de">Mietpreis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13837-1">
    <skos:altLabel xml:lang="de">Miethöhe</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13837-1">
    <skos:altLabel xml:lang="de">Mietbelastung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12401-1">
    <skos:altLabel xml:lang="de">Verschuldung des Betriebs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12401-1">
    <skos:altLabel xml:lang="de">Schulden des Betriebs</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18025-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71001">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14932-2">
    <skos:related rdf:resource="descriptor/18137-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73296">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11360-1">
    <skos:related rdf:resource="descriptor/15684-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11360-1">
    <skos:related rdf:resource="descriptor/16134-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11360-1">
    <skos:related rdf:resource="descriptor/11392-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13014-3">
    <skos:narrower rdf:resource="descriptor/13024-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13014-3">
    <skos:narrower rdf:resource="descriptor/13010-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15995-1">
    <skos:prefLabel xml:lang="en">Local constitution</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15995-1">
    <skos:prefLabel xml:lang="de">Kommunalrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12700-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19602-5">
    <skos:related rdf:resource="descriptor/10271-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19602-5">
    <skos:related rdf:resource="descriptor/11460-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19602-5">
    <skos:related rdf:resource="descriptor/11448-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70575">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73375">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19425-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19478-1">
    <skos:broader rdf:resource="thsys/71064"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19478-1">
    <skos:broader rdf:resource="thsys/70142"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19478-1">
    <skos:broader rdf:resource="descriptor/15914-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16226-1">
    <skos:altLabel xml:lang="de">Soziale Partizipation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16226-1">
    <skos:altLabel xml:lang="de">Politische Partizipation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16226-1">
    <skos:altLabel xml:lang="de">Politische Beteiligung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19956-5">
    <skos:prefLabel xml:lang="en">European Works Council</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19956-5">
    <skos:prefLabel xml:lang="de">EU-Betriebsrat</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20485-1">
    <skos:altLabel xml:lang="de">Scheinselbständigkeit</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20485-1">
    <skos:altLabel xml:lang="de">Arbeitnehmerähnliche Selbstständige</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20485-1">
    <skos:altLabel xml:lang="de">Arbeitnehmerähnliche Selbständige</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11845-5">
    <skos:narrower rdf:resource="descriptor/11843-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11845-5">
    <skos:narrower rdf:resource="descriptor/11563-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12594-6">
    <skos:broader rdf:resource="thsys/70278"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12594-6">
    <skos:broader rdf:resource="thsys/70245"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12594-6">
    <skos:broader rdf:resource="descriptor/11341-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12594-6">
    <skos:broader rdf:resource="descriptor/11322-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12085-6">
    <skos:broader rdf:resource="thsys/70485"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20536-3">
    <skos:altLabel xml:lang="en">Country-of-origin effect</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20536-3">
    <skos:altLabel xml:lang="en">Country image</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71073">
    <skos:broader rdf:resource="thsys/70144"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15444-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18214-5">
    <skos:scopeNote xml:lang="de">Fast Food sind nicht nur die in der Schnellgastronomie verkauften Produkte, sondern z.B. auch jene, die in den "Heißen Theken" der Schlachtereien oder als vorgefertigte Gerichte in Bäckereien angeboten werden.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11800-6">
    <skos:narrower rdf:resource="descriptor/19545-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11800-6">
    <skos:narrower rdf:resource="descriptor/10697-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11800-6">
    <skos:narrower rdf:resource="descriptor/11806-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11800-6">
    <skos:narrower rdf:resource="descriptor/18330-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11800-6">
    <skos:narrower rdf:resource="descriptor/11802-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19213-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19844-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16343-4">
    <skos:broader rdf:resource="thsys/73379"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16343-4">
    <skos:broader rdf:resource="descriptor/16340-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17626-1">
    <skos:prefLabel xml:lang="en">Jordan</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17626-1">
    <skos:prefLabel xml:lang="de">Jordanien</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16173-3">
    <skos:scopeNote xml:lang="de">Grundlage ist der "Vertrag über die Grundsätze zur Regelung der Tätigkeit von Staaten bei der Erforschung und Nutzung des Weltraums einschließlich des Mondes und anderer Himmelskörper" vom 27.1.1967.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70184">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id325">
    <rdfs:seeAlso rdf:resource="descriptor/18196-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18331-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19043-1">
    <skos:altLabel xml:lang="de">Gesellschafterinnen</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16597-1">
    <skos:altLabel xml:lang="en">Social stratification</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16597-1">
    <skos:altLabel xml:lang="de">Soziographie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16597-1">
    <skos:altLabel xml:lang="de">Soziale Struktur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16597-1">
    <skos:altLabel xml:lang="de">Gesellschaftsstruktur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16597-1">
    <skos:altLabel xml:lang="de">Gesellschaftliche Struktur</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18507-4">
    <skos:broader rdf:resource="thsys/70970"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18507-4">
    <skos:broader rdf:resource="descriptor/11817-3"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id379">
    <rdfs:seeAlso rdf:resource="descriptor/13830-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19735-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12285-5">
    <skos:narrower rdf:resource="descriptor/10135-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12285-5">
    <skos:narrower rdf:resource="descriptor/12272-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12285-5">
    <skos:narrower rdf:resource="descriptor/16033-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12285-5">
    <skos:narrower rdf:resource="descriptor/13837-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12285-5">
    <skos:narrower rdf:resource="descriptor/12271-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20987-5">
    <skos:prefLabel xml:lang="en">Niue</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20987-5">
    <skos:prefLabel xml:lang="de">Niue</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12182-1">
    <skos:related rdf:resource="descriptor/12184-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12182-1">
    <skos:related rdf:resource="descriptor/12183-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12182-1">
    <skos:related rdf:resource="descriptor/19729-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18239-3">
    <skos:related rdf:resource="descriptor/14642-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18568-5">
    <skos:broader rdf:resource="thsys/70941"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18568-5">
    <skos:broader rdf:resource="descriptor/15598-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18568-5">
    <skos:broader rdf:resource="descriptor/13574-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15375-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73303">
    <rdfs:label xml:lang="de">Milch und Milchprodukte</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19260-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18219-2">
    <skos:prefLabel xml:lang="en">Pork</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18219-2">
    <skos:prefLabel xml:lang="de">Schweinefleisch</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13837-1">
    <skos:related rdf:resource="descriptor/13841-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13837-1">
    <skos:related rdf:resource="descriptor/11574-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13837-1">
    <skos:related rdf:resource="descriptor/13304-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13837-1">
    <skos:related rdf:resource="descriptor/13839-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13837-1">
    <skos:related rdf:resource="descriptor/11840-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13837-1">
    <skos:related rdf:resource="descriptor/18755-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13837-1">
    <skos:related rdf:resource="descriptor/18745-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19768-6">
    <skos:narrower rdf:resource="descriptor/16623-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19217-1">
    <skos:prefLabel xml:lang="en">Intangibles</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19217-1">
    <skos:prefLabel xml:lang="de">Intangibles Gut</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18869-5">
    <skos:prefLabel xml:lang="en">Knowledge transfer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18869-5">
    <skos:prefLabel xml:lang="de">Wissenstransfer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19713-3">
    <skos:prefLabel xml:lang="en">Middle power</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19713-3">
    <skos:prefLabel xml:lang="de">Mittelmacht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10123-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19557-5">
    <skos:related rdf:resource="descriptor/12205-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19557-5">
    <skos:related rdf:resource="descriptor/12154-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13953-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13081-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14104-4">
    <skos:prefLabel xml:lang="en">Horticultural product</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14104-4">
    <skos:prefLabel xml:lang="de">Gartenbauprodukt</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16483-2">
    <skos:altLabel xml:lang="de">Sportförderung des Staates</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19480-0">
    <skos:related rdf:resource="descriptor/15772-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18086-2">
    <skos:related rdf:resource="descriptor/13769-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18086-2">
    <skos:related rdf:resource="descriptor/12265-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18086-2">
    <skos:related rdf:resource="descriptor/12304-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14723-4">
    <skos:broader rdf:resource="thsys/70201"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11276-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13237-4">
    <skos:related rdf:resource="descriptor/13163-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13237-4">
    <skos:related rdf:resource="descriptor/14996-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13237-4">
    <skos:related rdf:resource="descriptor/18213-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13237-4">
    <skos:related rdf:resource="descriptor/14942-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12016-4">
    <skos:altLabel xml:lang="en">Soil pollution</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12016-4">
    <skos:altLabel xml:lang="en">Land degradation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12016-4">
    <skos:altLabel xml:lang="en">Desertification</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12016-4">
    <skos:altLabel xml:lang="de">Bodenverunreinigung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12016-4">
    <skos:altLabel xml:lang="de">Bodenversauerung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12016-4">
    <skos:altLabel xml:lang="de">Bodenversalzung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12016-4">
    <skos:altLabel xml:lang="de">Bodenkontamination</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10466-4">
    <skos:narrower rdf:resource="descriptor/10463-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10466-4">
    <skos:narrower rdf:resource="descriptor/10454-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10466-4">
    <skos:narrower rdf:resource="descriptor/10475-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10466-4">
    <skos:narrower rdf:resource="descriptor/10446-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13137-1">
    <skos:related rdf:resource="descriptor/13142-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13137-1">
    <skos:related rdf:resource="descriptor/13220-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13137-1">
    <skos:related rdf:resource="descriptor/14831-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13137-1">
    <skos:related rdf:resource="descriptor/13128-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18455-4">
    <skos:altLabel xml:lang="de">Warenverzeichnis</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18455-4">
    <skos:altLabel xml:lang="de">Warensystematik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18455-4">
    <skos:altLabel xml:lang="de">Güterklassifikation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19393-2">
    <skos:broader rdf:resource="thsys/70957"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19393-2">
    <skos:broader rdf:resource="descriptor/18995-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12842-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19788-0">
    <skos:related rdf:resource="descriptor/15672-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19788-0">
    <skos:related rdf:resource="descriptor/19525-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19788-0">
    <skos:related rdf:resource="descriptor/16146-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19788-0">
    <skos:related rdf:resource="descriptor/18970-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19788-0">
    <skos:related rdf:resource="descriptor/15941-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19788-0">
    <skos:related rdf:resource="descriptor/14724-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70998">
    <skos:narrower rdf:resource="descriptor/13578-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70998">
    <skos:narrower rdf:resource="descriptor/18419-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70998">
    <skos:narrower rdf:resource="descriptor/13575-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70998">
    <skos:narrower rdf:resource="descriptor/13057-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70998">
    <skos:narrower rdf:resource="descriptor/18418-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70998">
    <skos:narrower rdf:resource="descriptor/13056-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71086">
    <skos:narrower rdf:resource="descriptor/10542-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71086">
    <skos:narrower rdf:resource="descriptor/10541-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71086">
    <skos:narrower rdf:resource="descriptor/10533-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71086">
    <skos:narrower rdf:resource="descriptor/19364-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71086">
    <skos:narrower rdf:resource="descriptor/10527-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71086">
    <skos:narrower rdf:resource="descriptor/10517-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71086">
    <skos:narrower rdf:resource="descriptor/10525-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71086">
    <skos:narrower rdf:resource="descriptor/10537-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71086">
    <skos:narrower rdf:resource="descriptor/10521-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71086">
    <skos:narrower rdf:resource="descriptor/10515-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12214-0">
    <skos:prefLabel xml:lang="en">Investment advisory service</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12214-0">
    <skos:prefLabel xml:lang="de">Anlageberatung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10771-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70868">
    <rdfs:label xml:lang="de">Arbeits- und Sozialverwaltung</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19177-1">
    <skos:related rdf:resource="descriptor/19178-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19697-3">
    <skos:prefLabel xml:lang="en">Local election</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19697-3">
    <skos:prefLabel xml:lang="de">Kommunalwahl</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19127-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19210-1">
    <skos:related rdf:resource="descriptor/24859-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19210-1">
    <skos:related rdf:resource="descriptor/11723-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19210-1">
    <skos:related rdf:resource="descriptor/11718-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19210-1">
    <skos:related rdf:resource="descriptor/10162-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10791-0">
    <skos:narrower rdf:resource="descriptor/10680-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10791-0">
    <skos:narrower rdf:resource="descriptor/10410-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10791-0">
    <skos:narrower rdf:resource="descriptor/10787-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10791-0">
    <skos:narrower rdf:resource="descriptor/10786-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10791-0">
    <skos:narrower rdf:resource="descriptor/10778-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24708-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15105-4">
    <skos:related rdf:resource="descriptor/19587-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15105-4">
    <skos:related rdf:resource="descriptor/15103-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15105-4">
    <skos:related rdf:resource="descriptor/19231-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19030-3">
    <skos:prefLabel xml:lang="en">Definition</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19030-3">
    <skos:prefLabel xml:lang="de">Definition</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19497-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13433-4">
    <skos:prefLabel xml:lang="en">Agricultural statistics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13433-4">
    <skos:prefLabel xml:lang="de">Agrarstatistik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11243-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14465-0">
    <skos:altLabel xml:lang="de">Strahlungsmessgerät</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14465-0">
    <skos:altLabel xml:lang="de">Strahlungsmeßgerät</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14465-0">
    <skos:altLabel xml:lang="de">Steuerungstechnisches Gerät</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14465-0">
    <skos:altLabel xml:lang="de">Sensorsystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14465-0">
    <skos:altLabel xml:lang="de">Sensor</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14465-0">
    <skos:altLabel xml:lang="de">Regeltechnisches Gerät</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14465-0">
    <skos:altLabel xml:lang="de">Prüftechnisches Gerät</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14465-0">
    <skos:altLabel xml:lang="de">Oszillator</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14465-0">
    <skos:altLabel xml:lang="de">Meßtechnisches Gerät</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14465-0">
    <skos:altLabel xml:lang="de">Meßgerät</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14465-0">
    <skos:altLabel xml:lang="de">Elektronisches Messgerät</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14465-0">
    <skos:altLabel xml:lang="de">Elektronisches Meßgerät</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14465-0">
    <skos:altLabel xml:lang="de">Elektrisches Prüfgerät</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14465-0">
    <skos:altLabel xml:lang="de">Elektrisches Messgerät</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14465-0">
    <skos:altLabel xml:lang="de">Elektrisches Meßgerät</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11543-0">
    <skos:prefLabel xml:lang="en">User charge</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11543-0">
    <skos:prefLabel xml:lang="de">Gebühr</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11574-3">
    <skos:related rdf:resource="descriptor/13849-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11574-3">
    <skos:related rdf:resource="descriptor/13837-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18061-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13340-4">
    <skos:broader rdf:resource="thsys/70818"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13340-4">
    <skos:broader rdf:resource="descriptor/13333-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13679-0">
    <skos:related rdf:resource="descriptor/14777-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19821-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17959-2">
    <skos:broader rdf:resource="thsys/70198"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17959-2">
    <skos:broader rdf:resource="descriptor/15887-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16759-1">
    <skos:prefLabel xml:lang="en">Information seeking behaviour</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16759-1">
    <skos:prefLabel xml:lang="de">Informationsverhalten</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19073-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id63">
    <rdf:value xml:lang="de">Für die christliche Theologie</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16272-1">
    <skos:prefLabel xml:lang="en">Public Interest</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16272-1">
    <skos:prefLabel xml:lang="de">Öffentliches Interesse</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14837-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14053-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10985-1">
    <skos:broader rdf:resource="thsys/71101"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10985-1">
    <skos:broader rdf:resource="descriptor/11771-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19326-3">
    <skos:broader rdf:resource="thsys/71097"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19326-3">
    <skos:broader rdf:resource="descriptor/10616-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70997">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70942">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16061-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11582-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18773-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24784-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19704-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13053-0">
    <skos:prefLabel xml:lang="en">Auto parts industry</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13053-0">
    <skos:prefLabel xml:lang="de">Kraftfahrzeugzulieferer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16046-3">
    <skos:prefLabel xml:lang="en">Central bank law</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16046-3">
    <skos:prefLabel xml:lang="de">Zentralbankrecht</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15778-2">
    <skos:altLabel xml:lang="de">Präventive Umweltpolitik</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18195-4">
    <skos:broader rdf:resource="thsys/70071"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id381">
    <rdf:value xml:lang="de">Für den internationalen Zahlungsverkehr</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17979-3">
    <skos:narrower rdf:resource="descriptor/17592-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17979-3">
    <skos:narrower rdf:resource="descriptor/17590-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17979-3">
    <skos:narrower rdf:resource="descriptor/17588-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17979-3">
    <skos:narrower rdf:resource="descriptor/17586-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17979-3">
    <skos:narrower rdf:resource="descriptor/17584-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17979-3">
    <skos:narrower rdf:resource="descriptor/17578-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17979-3">
    <skos:narrower rdf:resource="descriptor/17576-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12247-6">
    <skos:prefLabel xml:lang="en">Corporate bond</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12247-6">
    <skos:prefLabel xml:lang="de">Industrieobligation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15444-3">
    <skos:altLabel xml:lang="en">Strategic coalition</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15444-3">
    <skos:altLabel xml:lang="de">Strategische Koalition</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15444-3">
    <skos:altLabel xml:lang="de">Koalitionsbildung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12504-5">
    <skos:scopeNote xml:lang="de">Begriff der Kostenrechnung für jene Kosten, die durch die Herstellung eines Produkts entstanden sind.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14710-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18163-3">
    <skos:broader rdf:resource="thsys/71015"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18163-3">
    <skos:broader rdf:resource="thsys/70227"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18163-3">
    <skos:broader rdf:resource="descriptor/10889-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19402-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14309-0">
    <skos:broader rdf:resource="thsys/70134"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14309-0">
    <skos:broader rdf:resource="descriptor/14274-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14309-0">
    <skos:broader rdf:resource="descriptor/14302-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16663-0">
    <skos:broader rdf:resource="thsys/73358"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16663-0">
    <skos:broader rdf:resource="descriptor/15714-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71099">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18143-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16343-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10845-3">
    <skos:prefLabel xml:lang="en">Smuggling</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10845-3">
    <skos:prefLabel xml:lang="de">Schmuggel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18176-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18940-4">
    <skos:prefLabel xml:lang="en">Electronic archives</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18940-4">
    <skos:prefLabel xml:lang="de">Elektronisches Archiv</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15479-5">
    <skos:broader rdf:resource="thsys/73103"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15479-5">
    <skos:broader rdf:resource="descriptor/15480-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19303-1">
    <skos:broader rdf:resource="thsys/71029"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19303-1">
    <skos:broader rdf:resource="thsys/70960"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19303-1">
    <skos:broader rdf:resource="descriptor/19188-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71111">
    <skos:broader rdf:resource="thsys/70234"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15498-1">
    <skos:related rdf:resource="descriptor/19150-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15498-1">
    <skos:related rdf:resource="descriptor/15483-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15498-1">
    <skos:related rdf:resource="descriptor/15045-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15498-1">
    <skos:related rdf:resource="descriptor/15611-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15498-1">
    <skos:related rdf:resource="descriptor/15470-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11821-5">
    <skos:broader rdf:resource="thsys/71110"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11821-5">
    <skos:broader rdf:resource="thsys/71005"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11821-5">
    <skos:broader rdf:resource="descriptor/11782-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19143-4">
    <skos:prefLabel xml:lang="en">Deferred taxes</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19143-4">
    <skos:prefLabel xml:lang="de">Latente Steuern</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14919-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18502-0">
    <skos:related rdf:resource="descriptor/11845-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18502-0">
    <skos:related rdf:resource="descriptor/13848-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18502-0">
    <skos:related rdf:resource="descriptor/18503-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18502-0">
    <skos:related rdf:resource="descriptor/12932-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18502-0">
    <skos:related rdf:resource="descriptor/13284-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15698-0">
    <skos:related rdf:resource="descriptor/11217-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15698-0">
    <skos:related rdf:resource="descriptor/15718-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15698-0">
    <skos:related rdf:resource="descriptor/10574-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70038">
    <skos:narrower rdf:resource="descriptor/10966-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70038">
    <skos:narrower rdf:resource="descriptor/10567-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70038">
    <skos:narrower rdf:resource="descriptor/10994-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70038">
    <skos:narrower rdf:resource="descriptor/10993-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70038">
    <skos:narrower rdf:resource="descriptor/10991-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70038">
    <skos:narrower rdf:resource="descriptor/10988-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70038">
    <skos:narrower rdf:resource="descriptor/10987-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70038">
    <skos:narrower rdf:resource="descriptor/19327-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70038">
    <skos:narrower rdf:resource="descriptor/19829-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70038">
    <skos:narrower rdf:resource="descriptor/10029-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70038">
    <skos:narrower rdf:resource="descriptor/19284-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70038">
    <skos:narrower rdf:resource="descriptor/11572-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70038">
    <skos:narrower rdf:resource="descriptor/19291-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70038">
    <skos:narrower rdf:resource="descriptor/10978-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70038">
    <skos:narrower rdf:resource="descriptor/10401-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70038">
    <skos:narrower rdf:resource="descriptor/10398-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70038">
    <skos:narrower rdf:resource="descriptor/10979-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70038">
    <skos:narrower rdf:resource="descriptor/10395-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70038">
    <skos:narrower rdf:resource="descriptor/10976-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70038">
    <skos:narrower rdf:resource="descriptor/19399-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70038">
    <skos:narrower rdf:resource="descriptor/10973-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70038">
    <skos:narrower rdf:resource="descriptor/10045-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70038">
    <skos:narrower rdf:resource="descriptor/10970-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70038">
    <skos:narrower rdf:resource="descriptor/10969-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70038">
    <skos:narrower rdf:resource="descriptor/10968-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70038">
    <skos:narrower rdf:resource="descriptor/11007-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70038">
    <skos:narrower rdf:resource="descriptor/10383-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70038">
    <skos:narrower rdf:resource="descriptor/19198-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70038">
    <skos:narrower rdf:resource="descriptor/11817-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70038">
    <skos:narrower rdf:resource="descriptor/11004-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70038">
    <skos:narrower rdf:resource="descriptor/11551-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70038">
    <skos:narrower rdf:resource="descriptor/10378-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70038">
    <skos:narrower rdf:resource="descriptor/11239-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70038">
    <skos:narrower rdf:resource="descriptor/18138-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70038">
    <skos:narrower rdf:resource="descriptor/10377-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18576-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14318-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10996-3">
    <skos:related rdf:resource="descriptor/10406-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10996-3">
    <skos:related rdf:resource="descriptor/19505-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10996-3">
    <skos:related rdf:resource="descriptor/11235-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19651-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73384">
    <skos:narrower rdf:resource="descriptor/15859-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73384">
    <skos:narrower rdf:resource="descriptor/12846-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73384">
    <skos:narrower rdf:resource="descriptor/12837-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73384">
    <skos:narrower rdf:resource="descriptor/15858-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73384">
    <skos:narrower rdf:resource="descriptor/12818-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73384">
    <skos:narrower rdf:resource="descriptor/19846-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73384">
    <skos:narrower rdf:resource="descriptor/12806-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73384">
    <skos:narrower rdf:resource="descriptor/18098-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73384">
    <skos:narrower rdf:resource="descriptor/12774-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73384">
    <skos:narrower rdf:resource="descriptor/12766-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73384">
    <skos:narrower rdf:resource="descriptor/12669-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73384">
    <skos:narrower rdf:resource="descriptor/12665-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73384">
    <skos:narrower rdf:resource="descriptor/20536-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73384">
    <skos:narrower rdf:resource="descriptor/13485-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73384">
    <skos:narrower rdf:resource="descriptor/10300-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73384">
    <skos:narrower rdf:resource="descriptor/18896-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73384">
    <skos:narrower rdf:resource="descriptor/12075-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73384">
    <skos:narrower rdf:resource="descriptor/19133-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73384">
    <skos:narrower rdf:resource="descriptor/10851-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11177-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15965-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/24782-3">
    <skos:broader rdf:resource="thsys/71110"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18952-4">
    <skos:altLabel xml:lang="de">EDV-gestützte Bibliotheksdienstleistung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18952-4">
    <skos:altLabel xml:lang="de">Bibliotheksinformationssystem</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18952-4">
    <skos:altLabel xml:lang="de">Bibliotheksautomation</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13682-4">
    <skos:prefLabel xml:lang="en">Communication engineering</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13682-4">
    <skos:prefLabel xml:lang="de">Nachrichtentechnik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16533-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13775-4">
    <skos:prefLabel xml:lang="en">Mortgage</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13775-4">
    <skos:prefLabel xml:lang="de">Hypothek</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18449-6">
    <skos:prefLabel xml:lang="en">Press publishing</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18449-6">
    <skos:prefLabel xml:lang="de">Presseverlag</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18763-0">
    <skos:related rdf:resource="descriptor/18941-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70052">
    <rdfs:label xml:lang="en">Labour</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70052">
    <rdfs:label xml:lang="de">Arbeit</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11330-3">
    <skos:broader rdf:resource="thsys/70278"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11330-3">
    <skos:broader rdf:resource="descriptor/11322-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19359-2">
    <skos:prefLabel xml:lang="en">Loan agreement</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19359-2">
    <skos:prefLabel xml:lang="de">Kreditabkommen</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17480-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11199-3">
    <skos:altLabel xml:lang="en">Job pairing</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19365-0">
    <skos:scopeNote xml:lang="de">Nur für die natürliche Kautschukgewinnung.</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14237-6">
    <skos:broader rdf:resource="thsys/70348"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14237-6">
    <skos:broader rdf:resource="descriptor/14236-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71002">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13669-3">
    <skos:editorialNote xml:lang="de">Möglichst spezifischer indexieren.</skos:editorialNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15966-1">
    <skos:altLabel xml:lang="de">Gesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16286-4">
    <skos:related rdf:resource="descriptor/11127-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16286-4">
    <skos:related rdf:resource="descriptor/16332-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16286-4">
    <skos:related rdf:resource="descriptor/16295-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16286-4">
    <skos:related rdf:resource="descriptor/16404-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16286-4">
    <skos:related rdf:resource="descriptor/16396-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16286-4">
    <skos:related rdf:resource="descriptor/16290-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16286-4">
    <skos:related rdf:resource="descriptor/16288-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16286-4">
    <skos:related rdf:resource="descriptor/19683-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16286-4">
    <skos:related rdf:resource="descriptor/16395-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16286-4">
    <skos:related rdf:resource="descriptor/11087-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16286-4">
    <skos:related rdf:resource="descriptor/16510-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16286-4">
    <skos:related rdf:resource="descriptor/16247-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16286-4">
    <skos:related rdf:resource="descriptor/16397-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10412-6">
    <skos:broader rdf:resource="thsys/70406"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10412-6">
    <skos:broader rdf:resource="thsys/70355"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10412-6">
    <skos:broader rdf:resource="descriptor/10414-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10412-6">
    <skos:broader rdf:resource="descriptor/11506-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19298-3">
    <skos:related rdf:resource="descriptor/19300-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19298-3">
    <skos:related rdf:resource="descriptor/19297-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19298-3">
    <skos:related rdf:resource="descriptor/18197-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19298-3">
    <skos:related rdf:resource="descriptor/14095-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11507-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19034-2">
    <skos:altLabel xml:lang="de">Zertifizierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19034-2">
    <skos:altLabel xml:lang="de">Typung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19034-2">
    <skos:altLabel xml:lang="de">Standardisierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19034-2">
    <skos:altLabel xml:lang="de">Standard</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19034-2">
    <skos:altLabel xml:lang="de">Normierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19034-2">
    <skos:altLabel xml:lang="de">Internationaler Standard</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19034-2">
    <skos:altLabel xml:lang="de">Internationale Standardisierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19034-2">
    <skos:altLabel xml:lang="de">Internationale Normierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19034-2">
    <skos:altLabel xml:lang="de">Internationale Norm</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70429">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15225-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13801-1">
    <skos:prefLabel xml:lang="en">Credit card</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13801-1">
    <skos:prefLabel xml:lang="de">Kreditkarte</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11994-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11282-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19413-1">
    <skos:prefLabel xml:lang="en">Income tax policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19413-1">
    <skos:prefLabel xml:lang="de">Einkommensteuerpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10719-1">
    <skos:broader rdf:resource="thsys/71103"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10719-1">
    <skos:broader rdf:resource="thsys/71095"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10719-1">
    <skos:broader rdf:resource="thsys/71088"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10719-1">
    <skos:broader rdf:resource="descriptor/10721-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10719-1">
    <skos:broader rdf:resource="descriptor/10725-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10719-1">
    <skos:broader rdf:resource="descriptor/10780-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14531-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13341-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11663-4">
    <skos:prefLabel xml:lang="en">Capital income tax</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11663-4">
    <skos:prefLabel xml:lang="de">Kapitalertragsteuer</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19605-6">
    <skos:altLabel xml:lang="de">Verweildaueranalyse</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11177-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71076">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18093-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19358-4">
    <skos:narrower rdf:resource="descriptor/18157-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19358-4">
    <skos:narrower rdf:resource="descriptor/18061-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19358-4">
    <skos:narrower rdf:resource="descriptor/19359-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19358-4">
    <skos:narrower rdf:resource="descriptor/18695-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13662-3">
    <skos:prefLabel xml:lang="en">Freedom of opinion</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13662-3">
    <skos:prefLabel xml:lang="de">Meinungsfreiheit</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14083-0">
    <skos:broader rdf:resource="thsys/73298"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14083-0">
    <skos:broader rdf:resource="thsys/70341"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14083-0">
    <skos:broader rdf:resource="descriptor/18222-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19102-4">
    <skos:prefLabel xml:lang="en">Shareholder value</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19102-4">
    <skos:prefLabel xml:lang="de">Shareholder Value</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12872-4">
    <skos:altLabel xml:lang="de">Umflaggung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12872-4">
    <skos:altLabel xml:lang="de">Panholib</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12872-4">
    <skos:altLabel xml:lang="de">Billigflagge</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12872-4">
    <skos:altLabel xml:lang="de">Ausflaggung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18923-4">
    <skos:related rdf:resource="descriptor/13355-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18923-4">
    <skos:related rdf:resource="descriptor/16515-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18923-4">
    <skos:related rdf:resource="descriptor/18921-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13571-6">
    <skos:broader rdf:resource="thsys/73355"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13571-6">
    <skos:broader rdf:resource="thsys/70929"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13571-6">
    <skos:broader rdf:resource="descriptor/13573-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13571-6">
    <skos:broader rdf:resource="descriptor/16564-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70897">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Thsys"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19757-4">
    <skos:prefLabel xml:lang="en">Gender studies</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19757-4">
    <skos:prefLabel xml:lang="de">Geschlechterforschung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16797-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13248-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73390">
    <rdfs:label xml:lang="en">Economic law</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/73390">
    <rdfs:label xml:lang="de">Wirtschaftsrecht</rdfs:label>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id336">
    <rdfs:seeAlso rdf:resource="descriptor/15396-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11544-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17950-6">
    <skos:altLabel xml:lang="en">Ellice Islands</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17950-6">
    <skos:altLabel xml:lang="de">Elliceinseln</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12514-2">
    <skos:related rdf:resource="descriptor/12524-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12514-2">
    <skos:related rdf:resource="descriptor/12475-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12514-2">
    <skos:related rdf:resource="descriptor/12491-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12514-2">
    <skos:related rdf:resource="descriptor/12470-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17988-2">
    <skos:related rdf:resource="descriptor/18177-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17988-2">
    <skos:related rdf:resource="descriptor/15647-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17988-2">
    <skos:related rdf:resource="descriptor/17978-5"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id171">
    <rdfs:seeAlso rdf:resource="descriptor/11753-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11735-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18342-3">
    <skos:related rdf:resource="descriptor/12964-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19421-2">
    <skos:broader rdf:resource="thsys/71074"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19421-2">
    <skos:broader rdf:resource="descriptor/19422-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12978-2">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18649-5">
    <skos:broader rdf:resource="thsys/70922"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18649-5">
    <skos:broader rdf:resource="descriptor/18644-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18649-5">
    <skos:broader rdf:resource="descriptor/24708-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18601-5">
    <skos:prefLabel xml:lang="en">Ports policy</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18601-5">
    <skos:prefLabel xml:lang="de">Hafenpolitik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14715-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19018-0">
    <skos:broader rdf:resource="thsys/70169"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19018-0">
    <skos:broader rdf:resource="descriptor/14104-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70337">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19768-6">
    <zbwext:useInsteadNote rdf:nodeID="id112"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13190-4">
    <skos:related rdf:resource="descriptor/14273-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13190-4">
    <skos:related rdf:resource="descriptor/13224-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18360-1">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16480-1">
    <skos:related rdf:resource="descriptor/15924-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16480-1">
    <skos:related rdf:resource="descriptor/16477-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19407-3">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19517-3">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10450-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19636-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14125-3">
    <skos:prefLabel xml:lang="en">Silk</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14125-3">
    <skos:prefLabel xml:lang="de">Seide</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70902">
    <skos:broader rdf:resource="thsys/70903"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10027-2">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14161-6">
    <skos:narrower rdf:resource="descriptor/14160-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14161-6">
    <skos:narrower rdf:resource="descriptor/14969-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12053-5">
    <skos:prefLabel xml:lang="en">Ratio (firm)</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12053-5">
    <skos:prefLabel xml:lang="de">Betriebliche Kennzahl</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18590-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19473-4">
    <skos:prefLabel xml:lang="en">Urban environment</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19473-4">
    <skos:prefLabel xml:lang="de">Stadtökologie</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17943-3">
    <skos:altLabel xml:lang="de">Unabhängiger Staat Papua-Neuguinea</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17943-3">
    <skos:altLabel xml:lang="de">Neuguinea</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10460-2">
    <skos:broader rdf:resource="thsys/70233"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19024-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16953-5">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14295-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17485-0">
    <skos:scopeNote xml:lang="de">Erklärte sich 1991 unabhängig von der Sowjetunion und trat der GUS bei</skos:scopeNote>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18153-6">
    <skos:altLabel xml:lang="en">Producer services</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18153-6">
    <skos:altLabel xml:lang="de">Produzentendienstleistung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18153-6">
    <skos:altLabel xml:lang="de">Produktionsorientierte Dienstleistung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18153-6">
    <skos:altLabel xml:lang="de">Produktbegleitende Dienstleistung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18153-6">
    <skos:altLabel xml:lang="de">Gewerbliche Dienstleistung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18153-6">
    <skos:altLabel xml:lang="de">Fertigungsorientierte Dienstleistung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18419-1">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18642-5">
    <skos:altLabel xml:lang="en">Telecommunication monopoly</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18642-5">
    <skos:altLabel xml:lang="de">Telekommunikationsmonopol</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18642-5">
    <skos:altLabel xml:lang="de">Fernmeldemonopol</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18926-5">
    <skos:altLabel xml:lang="de">Kunstmarkt</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18926-5">
    <skos:altLabel xml:lang="de">Kunstgalerie</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18926-5">
    <skos:altLabel xml:lang="de">Antiquitätenhandel</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18834-3">
    <skos:broader rdf:resource="thsys/70859"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18834-3">
    <skos:broader rdf:resource="thsys/70856"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18834-3">
    <skos:broader rdf:resource="descriptor/12085-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13534-5">
    <skos:related rdf:resource="descriptor/13576-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13534-5">
    <skos:related rdf:resource="descriptor/13537-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13534-5">
    <skos:related rdf:resource="descriptor/13535-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70423">
    <skos:broader rdf:resource="thsys/p"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18661-1">
    <skos:prefLabel xml:lang="en">Computer architecture</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18661-1">
    <skos:prefLabel xml:lang="de">Rechnerarchitektur</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12181-3">
    <skos:prefLabel xml:lang="en">Partnership at will</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12181-3">
    <skos:prefLabel xml:lang="de">Gesellschaft des bürgerlichen Rechts</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19123-3">
    <skos:related rdf:resource="descriptor/11788-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19123-3">
    <skos:related rdf:resource="descriptor/10013-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19123-3">
    <skos:related rdf:resource="descriptor/10012-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14313-2">
    <skos:prefLabel xml:lang="en">Semi-metal</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14313-2">
    <skos:prefLabel xml:lang="de">Halbmetall</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17480-3">
    <skos:broader rdf:resource="thsys/70130"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17480-3">
    <skos:broader rdf:resource="descriptor/17179-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17480-3">
    <skos:broader rdf:resource="descriptor/19078-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13643-0">
    <skos:altLabel xml:lang="de">Pauschalreise</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14656-0">
    <skos:broader rdf:resource="thsys/70556"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14656-0">
    <skos:broader rdf:resource="descriptor/14599-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14854-3">
    <skos:broader rdf:resource="thsys/70476"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14854-3">
    <skos:broader rdf:resource="descriptor/14849-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14854-3">
    <skos:broader rdf:resource="descriptor/14599-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12090-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15472-5">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19268-5">
    <skos:altLabel xml:lang="de">Zinsgesetz</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13202-2">
    <skos:prefLabel xml:lang="en">Electrical goods trade</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13202-2">
    <skos:prefLabel xml:lang="de">Elektrohandel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18295-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13987-3">
    <skos:related rdf:resource="descriptor/18766-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13987-3">
    <skos:related rdf:resource="descriptor/18762-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13987-3">
    <skos:related rdf:resource="descriptor/15606-3"/>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id176">
    <rdf:value xml:lang="de">Für Pkw-Verkehr</rdf:value>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12105-5">
    <skos:related rdf:resource="descriptor/10463-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12105-5">
    <skos:related rdf:resource="descriptor/24847-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12105-5">
    <skos:related rdf:resource="descriptor/12121-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12105-5">
    <skos:related rdf:resource="descriptor/12091-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16150-1">
    <skos:related rdf:resource="descriptor/11331-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16150-1">
    <skos:related rdf:resource="descriptor/11346-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16150-1">
    <skos:related rdf:resource="descriptor/20455-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16150-1">
    <skos:related rdf:resource="descriptor/18169-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16150-1">
    <skos:related rdf:resource="descriptor/16131-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16150-1">
    <skos:related rdf:resource="descriptor/11314-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14121-4">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/11101-4">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19604-1">
    <skos:prefLabel xml:lang="en">Services statistics</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19604-1">
    <skos:prefLabel xml:lang="de">Dienstleistungsstatistik</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/20503-4">
    <skos:related rdf:resource="descriptor/18121-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10213-5">
    <skos:narrower rdf:resource="descriptor/10773-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10213-5">
    <skos:narrower rdf:resource="descriptor/13824-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10213-5">
    <skos:narrower rdf:resource="descriptor/11834-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10213-5">
    <skos:narrower rdf:resource="descriptor/10220-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10213-5">
    <skos:narrower rdf:resource="descriptor/18382-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10213-5">
    <skos:narrower rdf:resource="descriptor/19225-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10213-5">
    <skos:narrower rdf:resource="descriptor/18991-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10213-5">
    <skos:narrower rdf:resource="descriptor/13837-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10213-5">
    <skos:narrower rdf:resource="descriptor/10926-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10213-5">
    <skos:narrower rdf:resource="descriptor/18745-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10213-5">
    <skos:narrower rdf:resource="descriptor/10186-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10213-5">
    <skos:narrower rdf:resource="descriptor/13715-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10213-5">
    <skos:narrower rdf:resource="descriptor/13853-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10213-5">
    <skos:narrower rdf:resource="descriptor/13747-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10213-5">
    <skos:narrower rdf:resource="descriptor/10834-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10213-5">
    <skos:narrower rdf:resource="descriptor/19674-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10213-5">
    <skos:narrower rdf:resource="descriptor/10889-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15752-6">
    <skos:narrower rdf:resource="descriptor/18379-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15752-6">
    <skos:narrower rdf:resource="descriptor/11675-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15752-6">
    <skos:narrower rdf:resource="descriptor/10593-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15752-6">
    <skos:narrower rdf:resource="descriptor/19295-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15752-6">
    <skos:narrower rdf:resource="descriptor/10545-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15752-6">
    <skos:narrower rdf:resource="descriptor/18375-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15752-6">
    <skos:narrower rdf:resource="descriptor/18958-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19091-4">
    <skos:prefLabel xml:lang="en">Inventory valuation</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19091-4">
    <skos:prefLabel xml:lang="de">Bestandsbewertung</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19475-0">
    <skos:related rdf:resource="descriptor/18920-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19475-0">
    <skos:related rdf:resource="descriptor/16760-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19475-0">
    <skos:related rdf:resource="descriptor/16788-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19475-0">
    <skos:related rdf:resource="descriptor/15961-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19475-0">
    <skos:related rdf:resource="descriptor/18944-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19475-0">
    <skos:related rdf:resource="descriptor/18943-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19475-0">
    <skos:related rdf:resource="descriptor/13345-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19475-0">
    <skos:related rdf:resource="descriptor/14001-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19475-0">
    <skos:related rdf:resource="descriptor/16754-4"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19475-0">
    <skos:related rdf:resource="descriptor/18762-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19081-0">
    <skos:prefLabel xml:lang="en">Non-cash contribution</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19081-0">
    <skos:prefLabel xml:lang="de">Sacheinlage</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19317-4">
    <skos:altLabel xml:lang="de">Liberalisierung des Außenhandels</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19317-4">
    <skos:altLabel xml:lang="de">Außenwirtschaftsliberalisierung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19317-4">
    <skos:altLabel xml:lang="de">Außenhandelsliberalismus</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19525-4">
    <skos:broader rdf:resource="thsys/73346"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19525-4">
    <skos:broader rdf:resource="descriptor/18444-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71004">
    <skos:narrower rdf:resource="descriptor/13097-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71004">
    <skos:narrower rdf:resource="descriptor/13096-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71004">
    <skos:narrower rdf:resource="descriptor/13095-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71004">
    <skos:narrower rdf:resource="descriptor/18448-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71004">
    <skos:narrower rdf:resource="descriptor/12921-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71004">
    <skos:narrower rdf:resource="descriptor/18396-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71004">
    <skos:narrower rdf:resource="descriptor/12918-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71004">
    <skos:narrower rdf:resource="descriptor/12916-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71004">
    <skos:narrower rdf:resource="descriptor/18454-6"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71004">
    <skos:narrower rdf:resource="descriptor/12904-3"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71004">
    <skos:narrower rdf:resource="descriptor/14346-1"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71004">
    <skos:narrower rdf:resource="thsys/70994"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71004">
    <skos:narrower rdf:resource="thsys/70987"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71004">
    <skos:narrower rdf:resource="thsys/70995"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71004">
    <skos:narrower rdf:resource="thsys/70986"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71004">
    <skos:narrower rdf:resource="thsys/70990"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71004">
    <skos:narrower rdf:resource="thsys/70991"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71004">
    <skos:narrower rdf:resource="thsys/70988"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71004">
    <skos:narrower rdf:resource="thsys/70982"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71004">
    <skos:narrower rdf:resource="thsys/70989"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71004">
    <skos:narrower rdf:resource="thsys/71003"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71004">
    <skos:narrower rdf:resource="thsys/70985"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71004">
    <skos:narrower rdf:resource="thsys/70996"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71004">
    <skos:narrower rdf:resource="thsys/71002"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/71004">
    <skos:narrower rdf:resource="thsys/70999"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10978-5">
    <zbwext:useInsteadNote rdf:nodeID="id303"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10978-5">
    <zbwext:useInsteadNote rdf:nodeID="id302"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/18686-6">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19304-6">
    <skos:broader rdf:resource="thsys/71029"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19304-6">
    <skos:broader rdf:resource="thsys/71020"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19304-6">
    <skos:broader rdf:resource="descriptor/19187-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="thsys/70467">
    <skos:broader rdf:resource="thsys/70071"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10687-2">
    <skos:prefLabel xml:lang="en">Trade preferences</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10687-2">
    <skos:prefLabel xml:lang="de">Außenhandelspräferenz</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:nodeID="id165">
    <rdfs:seeAlso rdf:resource="descriptor/18676-2"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13761-1">
    <skos:altLabel xml:lang="de">Mengengeschäft von Banken</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/13761-1">
    <skos:altLabel xml:lang="de">Massengeschäft von Banken</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12341-0">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15176-2">
    <skos:broader rdf:resource="thsys/72185"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/15176-2">
    <skos:broader rdf:resource="descriptor/15060-0"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16016-5">
    <skos:broader rdf:resource="thsys/73381"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/16016-5">
    <skos:broader rdf:resource="thsys/73361"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10710-5">
    <skos:narrower rdf:resource="descriptor/19330-5"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10470-6">
    <skos:broader rdf:resource="thsys/73345"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/10470-6">
    <skos:broader rdf:resource="thsys/71109"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/14453-0">
    <rdf:type rdf:resource="../namespaces/skos-extensions/Descriptor"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19715-6">
    <skos:altLabel xml:lang="de">Mediationsverfahren</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19715-6">
    <skos:altLabel xml:lang="de">Internationale Vermittlung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19715-6">
    <skos:altLabel xml:lang="de">Internationale Konfliktregelung</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12015-6">
    <skos:altLabel xml:lang="de">Grund und Boden</skos:altLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/17668-6">
    <skos:inScheme rdf:resource="../stw"/>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12155-4">
    <skos:prefLabel xml:lang="en">Quality circle</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/12155-4">
    <skos:prefLabel xml:lang="de">Qualitätszirkel</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19017-2">
    <skos:prefLabel xml:lang="en">Hemp</skos:prefLabel>
  </rdf:Description>
  <rdf:Description rdf:about="descriptor/19017-2">
    <skos:prefLabel xml:lang="de">Hanf</skos:prefLabel>
  </rdf:Description>
</rdf:RDF>
